
declare module "babylonjs/abstractScene" {
import { Scene } from "babylonjs/scene";
import { Nullable } from "babylonjs/types";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { TransformNode } from "babylonjs/Meshes/transformNode";
import { Geometry } from "babylonjs/Meshes/geometry";
import { Skeleton } from "babylonjs/Bones/skeleton";
import { MorphTargetManager } from "babylonjs/Morph/morphTargetManager";
import { AssetContainer } from "babylonjs/assetContainer";
import { IParticleSystem } from "babylonjs/Particles/IParticleSystem";
import { AnimationGroup } from "babylonjs/Animations/animationGroup";
import { BaseTexture } from "babylonjs/Materials/Textures/baseTexture";
import { Material } from "babylonjs/Materials/material";
import { MultiMaterial } from "babylonjs/Materials/multiMaterial";
import { AbstractActionManager } from "babylonjs/Actions/abstractActionManager";
import { Camera } from "babylonjs/Cameras/camera";
import { Light } from "babylonjs/Lights/light";
import { Node } from "babylonjs/node";
import { Animation } from "babylonjs/Animations/animation";
/**
 * Defines how the parser contract is defined.
 * These parsers are used to parse a list of specific assets (like particle systems, etc..)
 */
export type BabylonFileParser = (parsedData: any, scene: Scene, container: AssetContainer, rootUrl: string) => void;
/**
 * Defines how the individual parser contract is defined.
 * These parser can parse an individual asset
 */
export type IndividualBabylonFileParser = (parsedData: any, scene: Scene, rootUrl: string) => any;
/**
 * Base class of the scene acting as a container for the different elements composing a scene.
 * This class is dynamically extended by the different components of the scene increasing
 * flexibility and reducing coupling
 */
export abstract class AbstractScene {
    /**
     * Stores the list of available parsers in the application.
     */
    private static _BabylonFileParsers;
    /**
     * Stores the list of available individual parsers in the application.
     */
    private static _IndividualBabylonFileParsers;
    /**
     * Adds a parser in the list of available ones
     * @param name Defines the name of the parser
     * @param parser Defines the parser to add
     */
    static AddParser(name: string, parser: BabylonFileParser): void;
    /**
     * Gets a general parser from the list of available ones
     * @param name Defines the name of the parser
     * @returns the requested parser or null
     */
    static GetParser(name: string): Nullable<BabylonFileParser>;
    /**
     * Adds n individual parser in the list of available ones
     * @param name Defines the name of the parser
     * @param parser Defines the parser to add
     */
    static AddIndividualParser(name: string, parser: IndividualBabylonFileParser): void;
    /**
     * Gets an individual parser from the list of available ones
     * @param name Defines the name of the parser
     * @returns the requested parser or null
     */
    static GetIndividualParser(name: string): Nullable<IndividualBabylonFileParser>;
    /**
     * Parser json data and populate both a scene and its associated container object
     * @param jsonData Defines the data to parse
     * @param scene Defines the scene to parse the data for
     * @param container Defines the container attached to the parsing sequence
     * @param rootUrl Defines the root url of the data
     */
    static Parse(jsonData: any, scene: Scene, container: AssetContainer, rootUrl: string): void;
    /**
     * Gets the list of root nodes (ie. nodes with no parent)
     */
    rootNodes: Node[];
    /** All of the cameras added to this scene
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras
     */
    cameras: Camera[];
    /**
     * All of the lights added to this scene
     * @see https://doc.babylonjs.com/features/featuresDeepDive/lights/lights_introduction
     */
    lights: Light[];
    /**
     * All of the (abstract) meshes added to this scene
     */
    meshes: AbstractMesh[];
    /**
     * The list of skeletons added to the scene
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/bonesSkeletons
     */
    skeletons: Skeleton[];
    /**
     * All of the particle systems added to this scene
     * @see https://doc.babylonjs.com/features/featuresDeepDive/particles/particle_system/particle_system_intro
     */
    particleSystems: IParticleSystem[];
    /**
     * Gets a list of Animations associated with the scene
     */
    animations: Animation[];
    /**
     * All of the animation groups added to this scene
     * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/groupAnimations
     */
    animationGroups: AnimationGroup[];
    /**
     * All of the multi-materials added to this scene
     * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/multiMaterials
     */
    multiMaterials: MultiMaterial[];
    /**
     * All of the materials added to this scene
     * In the context of a Scene, it is not supposed to be modified manually.
     * Any addition or removal should be done using the addMaterial and removeMaterial Scene methods.
     * Note also that the order of the Material within the array is not significant and might change.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/materials_introduction
     */
    materials: Material[];
    /**
     * The list of morph target managers added to the scene
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/dynamicMeshMorph
     */
    morphTargetManagers: MorphTargetManager[];
    /**
     * The list of geometries used in the scene.
     */
    geometries: Geometry[];
    /**
     * All of the transform nodes added to this scene
     * In the context of a Scene, it is not supposed to be modified manually.
     * Any addition or removal should be done using the addTransformNode and removeTransformNode Scene methods.
     * Note also that the order of the TransformNode within the array is not significant and might change.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/transforms/parent_pivot/transform_node
     */
    transformNodes: TransformNode[];
    /**
     * ActionManagers available on the scene.
     * @deprecated
     */
    actionManagers: AbstractActionManager[];
    /**
     * Textures to keep.
     */
    textures: BaseTexture[];
    /** @internal */
    protected _environmentTexture: Nullable<BaseTexture>;
    /**
     * Texture used in all pbr material as the reflection texture.
     * As in the majority of the scene they are the same (exception for multi room and so on),
     * this is easier to reference from here than from all the materials.
     */
    get environmentTexture(): Nullable<BaseTexture>;
    set environmentTexture(value: Nullable<BaseTexture>);
    /**
     * The list of postprocesses added to the scene
     */
    postProcesses: import("babylonjs/PostProcesses/postProcess").PostProcess[];
    /**
     * @returns all meshes, lights, cameras, transformNodes and bones
     */
    getNodes(): Array<Node>;
}
export {};

}
declare module "babylonjs/Actions/abstractActionManager" {
import { IDisposable } from "babylonjs/scene";
import { IActionEvent } from "babylonjs/Actions/actionEvent";
import { IAction } from "babylonjs/Actions/action";
import { Nullable } from "babylonjs/types";
/**
 * Abstract class used to decouple action Manager from scene and meshes.
 * Do not instantiate.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions
 */
export abstract class AbstractActionManager implements IDisposable {
    /** Gets the list of active triggers */
    static Triggers: {
        [key: string]: number;
    };
    /** Gets the cursor to use when hovering items */
    hoverCursor: string;
    /** Gets the list of actions */
    actions: IAction[];
    /**
     * Gets or sets a boolean indicating that the manager is recursive meaning that it can trigger action from children
     */
    isRecursive: boolean;
    /**
     * Releases all associated resources
     */
    abstract dispose(): void;
    /**
     * Does this action manager has pointer triggers
     */
    abstract get hasPointerTriggers(): boolean;
    /**
     * Does this action manager has pick triggers
     */
    abstract get hasPickTriggers(): boolean;
    /**
     * Process a specific trigger
     * @param trigger defines the trigger to process
     * @param evt defines the event details to be processed
     */
    abstract processTrigger(trigger: number, evt?: IActionEvent): void;
    /**
     * Does this action manager handles actions of any of the given triggers
     * @param triggers defines the triggers to be tested
     * @returns a boolean indicating whether one (or more) of the triggers is handled
     */
    abstract hasSpecificTriggers(triggers: number[]): boolean;
    /**
     * Does this action manager handles actions of any of the given triggers. This function takes two arguments for
     * speed.
     * @param triggerA defines the trigger to be tested
     * @param triggerB defines the trigger to be tested
     * @returns a boolean indicating whether one (or more) of the triggers is handled
     */
    abstract hasSpecificTriggers2(triggerA: number, triggerB: number): boolean;
    /**
     * Does this action manager handles actions of a given trigger
     * @param trigger defines the trigger to be tested
     * @param parameterPredicate defines an optional predicate to filter triggers by parameter
     * @returns whether the trigger is handled
     */
    abstract hasSpecificTrigger(trigger: number, parameterPredicate?: (parameter: any) => boolean): boolean;
    /**
     * Serialize this manager to a JSON object
     * @param name defines the property name to store this manager
     * @returns a JSON representation of this manager
     */
    abstract serialize(name: string): any;
    /**
     * Registers an action to this action manager
     * @param action defines the action to be registered
     * @returns the action amended (prepared) after registration
     */
    abstract registerAction(action: IAction): Nullable<IAction>;
    /**
     * Unregisters an action to this action manager
     * @param action defines the action to be unregistered
     * @returns a boolean indicating whether the action has been unregistered
     */
    abstract unregisterAction(action: IAction): Boolean;
    /**
     * Does exist one action manager with at least one trigger
     **/
    static get HasTriggers(): boolean;
    /**
     * Does exist one action manager with at least one pick trigger
     **/
    static get HasPickTriggers(): boolean;
    /**
     * Does exist one action manager that handles actions of a given trigger
     * @param trigger defines the trigger to be tested
     * @returns a boolean indicating whether the trigger is handled by at least one action manager
     **/
    static HasSpecificTrigger(trigger: number): boolean;
}

}
declare module "babylonjs/Actions/action" {
import { Observable } from "babylonjs/Misc/observable";
import { Condition } from "babylonjs/Actions/condition";
import { AbstractActionManager } from "babylonjs/Actions/abstractActionManager";
import { Nullable } from "babylonjs/types";
import { Material } from "babylonjs/Materials/material";
import { Scene } from "babylonjs/scene";
import { ActionManager } from "babylonjs/Actions/actionManager";
import { ActionEvent } from "babylonjs/Actions/actionEvent";
import { Node } from "babylonjs/node";
/**
 * Interface used to define Action
 */
export interface IAction {
    /**
     * Trigger for the action
     */
    trigger: number;
    /** Options of the trigger */
    triggerOptions: any;
    /**
     * Gets the trigger parameters
     * @returns the trigger parameters
     */
    getTriggerParameter(): any;
    /**
     * Internal only - executes current action event
     * @internal
     */
    _executeCurrent(evt?: ActionEvent): void;
    /**
     * Serialize placeholder for child classes
     * @param parent of child
     * @returns the serialized object
     */
    serialize(parent: any): any;
    /**
     * Internal only
     * @internal
     */
    _prepare(): void;
    /**
     * Internal only - manager for action
     * @internal
     */
    _actionManager: Nullable<AbstractActionManager>;
    /**
     * Adds action to chain of actions, may be a DoNothingAction
     * @param action defines the next action to execute
     * @returns The action passed in
     * @see https://www.babylonjs-playground.com/#1T30HR#0
     */
    then(action: IAction): IAction;
}
/**
 * The action to be carried out following a trigger
 * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#available-actions
 */
export class Action implements IAction {
    /** the trigger, with or without parameters, for the action */
    triggerOptions: any;
    /**
     * Trigger for the action
     */
    trigger: number;
    /**
     * Internal only - manager for action
     * @internal
     */
    _actionManager: ActionManager;
    private _nextActiveAction;
    private _child;
    private _condition?;
    private _triggerParameter;
    /**
     * An event triggered prior to action being executed.
     */
    onBeforeExecuteObservable: Observable<Action>;
    /**
     * Creates a new Action
     * @param triggerOptions the trigger, with or without parameters, for the action
     * @param condition an optional determinant of action
     */
    constructor(
    /** the trigger, with or without parameters, for the action */
    triggerOptions: any, condition?: Condition);
    /**
     * Internal only
     * @internal
     */
    _prepare(): void;
    /**
     * Gets the trigger parameter
     * @returns the trigger parameter
     */
    getTriggerParameter(): any;
    /**
     * Sets the trigger parameter
     * @param value defines the new trigger parameter
     */
    setTriggerParameter(value: any): void;
    /**
     * Internal only - Returns if the current condition allows to run the action
     * @internal
     */
    _evaluateConditionForCurrentFrame(): boolean;
    /**
     * Internal only - executes current action event
     * @internal
     */
    _executeCurrent(evt?: ActionEvent): void;
    /**
     * Execute placeholder for child classes
     * @param evt optional action event
     */
    execute(evt?: ActionEvent): void;
    /**
     * Skips to next active action
     */
    skipToNextActiveAction(): void;
    /**
     * Adds action to chain of actions, may be a DoNothingAction
     * @param action defines the next action to execute
     * @returns The action passed in
     * @see https://www.babylonjs-playground.com/#1T30HR#0
     */
    then(action: Action): Action;
    /**
     * Internal only
     * @internal
     */
    _getProperty(propertyPath: string): string;
    /**
     * @internal
     */
    _getEffectiveTarget(target: any, propertyPath: string): any;
    /**
     * Serialize placeholder for child classes
     * @param parent of child
     * @returns the serialized object
     */
    serialize(parent: any): any;
    /**
     * Internal only called by serialize
     * @internal
     */
    protected _serialize(serializedAction: any, parent?: any): any;
    /**
     * Internal only
     * @internal
     */
    static _SerializeValueAsString: (value: any) => string;
    /**
     * Internal only
     * @internal
     */
    static _GetTargetProperty: (target: Scene | Node | Material) => {
        name: string;
        targetType: string;
        value: string;
    };
}
export {};

}
declare module "babylonjs/Actions/actionEvent" {
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { Nullable } from "babylonjs/types";
import { Sprite } from "babylonjs/Sprites/sprite";
import { Scene } from "babylonjs/scene";
import { Vector2 } from "babylonjs/Maths/math.vector";
/**
 * Interface used to define ActionEvent
 */
export interface IActionEvent {
    /** The mesh or sprite that triggered the action */
    source: any;
    /** The X mouse cursor position at the time of the event */
    pointerX: number;
    /** The Y mouse cursor position at the time of the event */
    pointerY: number;
    /** The mesh that is currently pointed at (can be null) */
    meshUnderPointer: Nullable<AbstractMesh>;
    /** the original (browser) event that triggered the ActionEvent */
    sourceEvent?: any;
    /** additional data for the event */
    additionalData?: any;
}
/**
 * ActionEvent is the event being sent when an action is triggered.
 */
export class ActionEvent implements IActionEvent {
    /** The mesh or sprite that triggered the action */
    source: any;
    /** The X mouse cursor position at the time of the event */
    pointerX: number;
    /** The Y mouse cursor position at the time of the event */
    pointerY: number;
    /** The mesh that is currently pointed at (can be null) */
    meshUnderPointer: Nullable<AbstractMesh>;
    /** the original (browser) event that triggered the ActionEvent */
    sourceEvent?: any;
    /** additional data for the event */
    additionalData?: any;
    /**
     * Creates a new ActionEvent
     * @param source The mesh or sprite that triggered the action
     * @param pointerX The X mouse cursor position at the time of the event
     * @param pointerY The Y mouse cursor position at the time of the event
     * @param meshUnderPointer The mesh that is currently pointed at (can be null)
     * @param sourceEvent the original (browser) event that triggered the ActionEvent
     * @param additionalData additional data for the event
     */
    constructor(
    /** The mesh or sprite that triggered the action */
    source: any, 
    /** The X mouse cursor position at the time of the event */
    pointerX: number, 
    /** The Y mouse cursor position at the time of the event */
    pointerY: number, 
    /** The mesh that is currently pointed at (can be null) */
    meshUnderPointer: Nullable<AbstractMesh>, 
    /** the original (browser) event that triggered the ActionEvent */
    sourceEvent?: any, 
    /** additional data for the event */
    additionalData?: any);
    /**
     * Helper function to auto-create an ActionEvent from a source mesh.
     * @param source The source mesh that triggered the event
     * @param evt The original (browser) event
     * @param additionalData additional data for the event
     * @returns the new ActionEvent
     */
    static CreateNew(source: AbstractMesh, evt?: any, additionalData?: any): ActionEvent;
    /**
     * Helper function to auto-create an ActionEvent from a source sprite
     * @param source The source sprite that triggered the event
     * @param scene Scene associated with the sprite
     * @param evt The original (browser) event
     * @param additionalData additional data for the event
     * @returns the new ActionEvent
     */
    static CreateNewFromSprite(source: Sprite, scene: Scene, evt?: any, additionalData?: any): ActionEvent;
    /**
     * Helper function to auto-create an ActionEvent from a scene. If triggered by a mesh use ActionEvent.CreateNew
     * @param scene the scene where the event occurred
     * @param evt The original (browser) event
     * @returns the new ActionEvent
     */
    static CreateNewFromScene(scene: Scene, evt: any): ActionEvent;
    /**
     * Helper function to auto-create an ActionEvent from a primitive
     * @param prim defines the target primitive
     * @param pointerPos defines the pointer position
     * @param evt The original (browser) event
     * @param additionalData additional data for the event
     * @returns the new ActionEvent
     */
    static CreateNewFromPrimitive(prim: any, pointerPos: Vector2, evt?: Event, additionalData?: any): ActionEvent;
}

}
declare module "babylonjs/Actions/actionManager" {
import { Nullable } from "babylonjs/types";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { Scene } from "babylonjs/scene";
import { IAction } from "babylonjs/Actions/action";
import { IActionEvent } from "babylonjs/Actions/actionEvent";
import { AbstractActionManager } from "babylonjs/Actions/abstractActionManager";
/**
 * Action Manager manages all events to be triggered on a given mesh or the global scene.
 * A single scene can have many Action Managers to handle predefined actions on specific meshes.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions
 */
export class ActionManager extends AbstractActionManager {
    /**
     * Nothing
     * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
     */
    static readonly NothingTrigger: number;
    /**
     * On pick
     * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
     */
    static readonly OnPickTrigger: number;
    /**
     * On left pick
     * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
     */
    static readonly OnLeftPickTrigger: number;
    /**
     * On right pick
     * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
     */
    static readonly OnRightPickTrigger: number;
    /**
     * On center pick
     * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
     */
    static readonly OnCenterPickTrigger: number;
    /**
     * On pick down
     * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
     */
    static readonly OnPickDownTrigger: number;
    /**
     * On double pick
     * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
     */
    static readonly OnDoublePickTrigger: number;
    /**
     * On pick up
     * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
     */
    static readonly OnPickUpTrigger: number;
    /**
     * On pick out.
     * This trigger will only be raised if you also declared a OnPickDown
     * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
     */
    static readonly OnPickOutTrigger: number;
    /**
     * On long press
     * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
     */
    static readonly OnLongPressTrigger: number;
    /**
     * On pointer over
     * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
     */
    static readonly OnPointerOverTrigger: number;
    /**
     * On pointer out
     * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
     */
    static readonly OnPointerOutTrigger: number;
    /**
     * On every frame
     * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
     */
    static readonly OnEveryFrameTrigger: number;
    /**
     * On intersection enter
     * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
     */
    static readonly OnIntersectionEnterTrigger: number;
    /**
     * On intersection exit
     * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
     */
    static readonly OnIntersectionExitTrigger: number;
    /**
     * On key down
     * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
     */
    static readonly OnKeyDownTrigger: number;
    /**
     * On key up
     * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
     */
    static readonly OnKeyUpTrigger: number;
    private _scene;
    /**
     * Creates a new action manager
     * @param scene defines the hosting scene
     */
    constructor(scene?: Nullable<Scene>);
    /**
     * Releases all associated resources
     */
    dispose(): void;
    /**
     * Gets hosting scene
     * @returns the hosting scene
     */
    getScene(): Scene;
    /**
     * Does this action manager handles actions of any of the given triggers
     * @param triggers defines the triggers to be tested
     * @returns a boolean indicating whether one (or more) of the triggers is handled
     */
    hasSpecificTriggers(triggers: number[]): boolean;
    /**
     * Does this action manager handles actions of any of the given triggers. This function takes two arguments for
     * speed.
     * @param triggerA defines the trigger to be tested
     * @param triggerB defines the trigger to be tested
     * @returns a boolean indicating whether one (or more) of the triggers is handled
     */
    hasSpecificTriggers2(triggerA: number, triggerB: number): boolean;
    /**
     * Does this action manager handles actions of a given trigger
     * @param trigger defines the trigger to be tested
     * @param parameterPredicate defines an optional predicate to filter triggers by parameter
     * @returns whether the trigger is handled
     */
    hasSpecificTrigger(trigger: number, parameterPredicate?: (parameter: any) => boolean): boolean;
    /**
     * Does this action manager has pointer triggers
     */
    get hasPointerTriggers(): boolean;
    /**
     * Does this action manager has pick triggers
     */
    get hasPickTriggers(): boolean;
    /**
     * Registers an action to this action manager
     * @param action defines the action to be registered
     * @returns the action amended (prepared) after registration
     */
    registerAction(action: IAction): Nullable<IAction>;
    /**
     * Unregisters an action to this action manager
     * @param action defines the action to be unregistered
     * @returns a boolean indicating whether the action has been unregistered
     */
    unregisterAction(action: IAction): Boolean;
    /**
     * Process a specific trigger
     * @param trigger defines the trigger to process
     * @param evt defines the event details to be processed
     */
    processTrigger(trigger: number, evt?: IActionEvent): void;
    /**
     * @internal
     */
    _getEffectiveTarget(target: any, propertyPath: string): any;
    /**
     * @internal
     */
    _getProperty(propertyPath: string): string;
    /**
     * Serialize this manager to a JSON object
     * @param name defines the property name to store this manager
     * @returns a JSON representation of this manager
     */
    serialize(name: string): any;
    /**
     * Creates a new ActionManager from a JSON data
     * @param parsedActions defines the JSON data to read from
     * @param object defines the hosting mesh
     * @param scene defines the hosting scene
     */
    static Parse(parsedActions: any, object: Nullable<AbstractMesh>, scene: Scene): void;
    /**
     * Get a trigger name by index
     * @param trigger defines the trigger index
     * @returns a trigger name
     */
    static GetTriggerName(trigger: number): string;
}

}
declare module "babylonjs/Actions/condition" {
import { ActionManager } from "babylonjs/Actions/actionManager";
/**
 * A Condition applied to an Action
 */
export class Condition {
    /**
     * Internal only - manager for action
     * @internal
     */
    _actionManager: ActionManager;
    /**
     * @internal
     */
    _evaluationId: number;
    /**
     * @internal
     */
    _currentResult: boolean;
    /**
     * Creates a new Condition
     * @param actionManager the manager of the action the condition is applied to
     */
    constructor(actionManager: ActionManager);
    /**
     * Check if the current condition is valid
     * @returns a boolean
     */
    isValid(): boolean;
    /**
     * @internal
     */
    _getProperty(propertyPath: string): string;
    /**
     * @internal
     */
    _getEffectiveTarget(target: any, propertyPath: string): any;
    /**
     * Serialize placeholder for child classes
     * @returns the serialized object
     */
    serialize(): any;
    /**
     * @internal
     */
    protected _serialize(serializedCondition: any): any;
}
/**
 * Defines specific conditional operators as extensions of Condition
 */
export class ValueCondition extends Condition {
    /** path to specify the property of the target the conditional operator uses  */
    propertyPath: string;
    /** the value compared by the conditional operator against the current value of the property */
    value: any;
    /** the conditional operator, default ValueCondition.IsEqual */
    operator: number;
    private static _IsEqual;
    private static _IsDifferent;
    private static _IsGreater;
    private static _IsLesser;
    /**
     * returns the number for IsEqual
     */
    static get IsEqual(): number;
    /**
     * Returns the number for IsDifferent
     */
    static get IsDifferent(): number;
    /**
     * Returns the number for IsGreater
     */
    static get IsGreater(): number;
    /**
     * Returns the number for IsLesser
     */
    static get IsLesser(): number;
    /**
     * Internal only The action manager for the condition
     * @internal
     */
    _actionManager: ActionManager;
    private _target;
    private _effectiveTarget;
    private _property;
    /**
     * Creates a new ValueCondition
     * @param actionManager manager for the action the condition applies to
     * @param target for the action
     * @param propertyPath path to specify the property of the target the conditional operator uses
     * @param value the value compared by the conditional operator against the current value of the property
     * @param operator the conditional operator, default ValueCondition.IsEqual
     */
    constructor(actionManager: ActionManager, target: any, 
    /** path to specify the property of the target the conditional operator uses  */
    propertyPath: string, 
    /** the value compared by the conditional operator against the current value of the property */
    value: any, 
    /** the conditional operator, default ValueCondition.IsEqual */
    operator?: number);
    /**
     * Compares the given value with the property value for the specified conditional operator
     * @returns the result of the comparison
     */
    isValid(): boolean;
    /**
     * Serialize the ValueCondition into a JSON compatible object
     * @returns serialization object
     */
    serialize(): any;
    /**
     * Gets the name of the conditional operator for the ValueCondition
     * @param operator the conditional operator
     * @returns the name
     */
    static GetOperatorName(operator: number): string;
}
/**
 * Defines a predicate condition as an extension of Condition
 */
export class PredicateCondition extends Condition {
    /** defines the predicate function used to validate the condition */
    predicate: () => boolean;
    /**
     * Internal only - manager for action
     * @internal
     */
    _actionManager: ActionManager;
    /**
     * Creates a new PredicateCondition
     * @param actionManager manager for the action the condition applies to
     * @param predicate defines the predicate function used to validate the condition
     */
    constructor(actionManager: ActionManager, 
    /** defines the predicate function used to validate the condition */
    predicate: () => boolean);
    /**
     * @returns the validity of the predicate condition
     */
    isValid(): boolean;
}
/**
 * Defines a state condition as an extension of Condition
 */
export class StateCondition extends Condition {
    /** Value to compare with target state  */
    value: string;
    /**
     * Internal only - manager for action
     * @internal
     */
    _actionManager: ActionManager;
    private _target;
    /**
     * Creates a new StateCondition
     * @param actionManager manager for the action the condition applies to
     * @param target of the condition
     * @param value to compare with target state
     */
    constructor(actionManager: ActionManager, target: any, 
    /** Value to compare with target state  */
    value: string);
    /**
     * Gets a boolean indicating if the current condition is met
     * @returns the validity of the state
     */
    isValid(): boolean;
    /**
     * Serialize the StateCondition into a JSON compatible object
     * @returns serialization object
     */
    serialize(): any;
}
export {};

}
declare module "babylonjs/Actions/directActions" {
import { Action } from "babylonjs/Actions/action";
import { Condition } from "babylonjs/Actions/condition";
import { ActionEvent } from "babylonjs/Actions/actionEvent";
/**
 * This defines an action responsible to toggle a boolean once triggered.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions
 */
export class SwitchBooleanAction extends Action {
    /**
     * The path to the boolean property in the target object
     */
    propertyPath: string;
    private _target;
    private _effectiveTarget;
    private _property;
    /**
     * Instantiate the action
     * @param triggerOptions defines the trigger options
     * @param target defines the object containing the boolean
     * @param propertyPath defines the path to the boolean property in the target object
     * @param condition defines the trigger related conditions
     */
    constructor(triggerOptions: any, target: any, propertyPath: string, condition?: Condition);
    /** @internal */
    _prepare(): void;
    /**
     * Execute the action toggle the boolean value.
     */
    execute(): void;
    /**
     * Serializes the actions and its related information.
     * @param parent defines the object to serialize in
     * @returns the serialized object
     */
    serialize(parent: any): any;
}
/**
 * This defines an action responsible to set a the state field of the target
 *  to a desired value once triggered.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions
 */
export class SetStateAction extends Action {
    /**
     * The value to store in the state field.
     */
    value: string;
    private _target;
    /**
     * Instantiate the action
     * @param triggerOptions defines the trigger options
     * @param target defines the object containing the state property
     * @param value defines the value to store in the state field
     * @param condition defines the trigger related conditions
     */
    constructor(triggerOptions: any, target: any, value: string, condition?: Condition);
    /**
     * Execute the action and store the value on the target state property.
     */
    execute(): void;
    /**
     * Serializes the actions and its related information.
     * @param parent defines the object to serialize in
     * @returns the serialized object
     */
    serialize(parent: any): any;
}
/**
 * This defines an action responsible to set a property of the target
 *  to a desired value once triggered.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions
 */
export class SetValueAction extends Action {
    /**
     * The path of the property to set in the target.
     */
    propertyPath: string;
    /**
     * The value to set in the property
     */
    value: any;
    private _target;
    private _effectiveTarget;
    private _property;
    /**
     * Instantiate the action
     * @param triggerOptions defines the trigger options
     * @param target defines the object containing the property
     * @param propertyPath defines the path of the property to set in the target
     * @param value defines the value to set in the property
     * @param condition defines the trigger related conditions
     */
    constructor(triggerOptions: any, target: any, propertyPath: string, value: any, condition?: Condition);
    /** @internal */
    _prepare(): void;
    /**
     * Execute the action and set the targeted property to the desired value.
     */
    execute(): void;
    /**
     * Serializes the actions and its related information.
     * @param parent defines the object to serialize in
     * @returns the serialized object
     */
    serialize(parent: any): any;
}
/**
 * This defines an action responsible to increment the target value
 *  to a desired value once triggered.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions
 */
export class IncrementValueAction extends Action {
    /**
     * The path of the property to increment in the target.
     */
    propertyPath: string;
    /**
     * The value we should increment the property by.
     */
    value: any;
    private _target;
    private _effectiveTarget;
    private _property;
    /**
     * Instantiate the action
     * @param triggerOptions defines the trigger options
     * @param target defines the object containing the property
     * @param propertyPath defines the path of the property to increment in the target
     * @param value defines the value value we should increment the property by
     * @param condition defines the trigger related conditions
     */
    constructor(triggerOptions: any, target: any, propertyPath: string, value: any, condition?: Condition);
    /** @internal */
    _prepare(): void;
    /**
     * Execute the action and increment the target of the value amount.
     */
    execute(): void;
    /**
     * Serializes the actions and its related information.
     * @param parent defines the object to serialize in
     * @returns the serialized object
     */
    serialize(parent: any): any;
}
/**
 * This defines an action responsible to start an animation once triggered.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions
 */
export class PlayAnimationAction extends Action {
    /**
     * Where the animation should start (animation frame)
     */
    from: number;
    /**
     * Where the animation should stop (animation frame)
     */
    to: number;
    /**
     * Define if the animation should loop or stop after the first play.
     */
    loop?: boolean;
    private _target;
    /**
     * Instantiate the action
     * @param triggerOptions defines the trigger options
     * @param target defines the target animation or animation name
     * @param from defines from where the animation should start (animation frame)
     * @param to defines where the animation should stop (animation frame)
     * @param loop defines if the animation should loop or stop after the first play
     * @param condition defines the trigger related conditions
     */
    constructor(triggerOptions: any, target: any, from: number, to: number, loop?: boolean, condition?: Condition);
    /** @internal */
    _prepare(): void;
    /**
     * Execute the action and play the animation.
     */
    execute(): void;
    /**
     * Serializes the actions and its related information.
     * @param parent defines the object to serialize in
     * @returns the serialized object
     */
    serialize(parent: any): any;
}
/**
 * This defines an action responsible to stop an animation once triggered.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions
 */
export class StopAnimationAction extends Action {
    private _target;
    /**
     * Instantiate the action
     * @param triggerOptions defines the trigger options
     * @param target defines the target animation or animation name
     * @param condition defines the trigger related conditions
     */
    constructor(triggerOptions: any, target: any, condition?: Condition);
    /** @internal */
    _prepare(): void;
    /**
     * Execute the action and stop the animation.
     */
    execute(): void;
    /**
     * Serializes the actions and its related information.
     * @param parent defines the object to serialize in
     * @returns the serialized object
     */
    serialize(parent: any): any;
}
/**
 * This defines an action responsible that does nothing once triggered.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions
 */
export class DoNothingAction extends Action {
    /**
     * Instantiate the action
     * @param triggerOptions defines the trigger options
     * @param condition defines the trigger related conditions
     */
    constructor(triggerOptions?: any, condition?: Condition);
    /**
     * Execute the action and do nothing.
     */
    execute(): void;
    /**
     * Serializes the actions and its related information.
     * @param parent defines the object to serialize in
     * @returns the serialized object
     */
    serialize(parent: any): any;
}
/**
 * This defines an action responsible to trigger several actions once triggered.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions
 */
export class CombineAction extends Action {
    /**
     * The list of aggregated animations to run.
     */
    children: Action[];
    /**
     * defines if the children actions conditions should be check before execution
     */
    enableChildrenConditions: boolean;
    /**
     * Instantiate the action
     * @param triggerOptions defines the trigger options
     * @param children defines the list of aggregated animations to run
     * @param condition defines the trigger related conditions
     * @param enableChildrenConditions defines if the children actions conditions should be check before execution
     */
    constructor(triggerOptions: any, children: Action[], condition?: Condition, enableChildrenConditions?: boolean);
    /** @internal */
    _prepare(): void;
    /**
     * Execute the action and executes all the aggregated actions.
     * @param evt
     */
    execute(evt: ActionEvent): void;
    /**
     * Serializes the actions and its related information.
     * @param parent defines the object to serialize in
     * @returns the serialized object
     */
    serialize(parent: any): any;
}
/**
 * This defines an action responsible to run code (external event) once triggered.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions
 */
export class ExecuteCodeAction extends Action {
    /**
     * The callback function to run.
     */
    func: (evt: ActionEvent) => void;
    /**
     * Instantiate the action
     * @param triggerOptions defines the trigger options
     * @param func defines the callback function to run
     * @param condition defines the trigger related conditions
     */
    constructor(triggerOptions: any, func: (evt: ActionEvent) => void, condition?: Condition);
    /**
     * Execute the action and run the attached code.
     * @param evt
     */
    execute(evt: ActionEvent): void;
}
/**
 * This defines an action responsible to set the parent property of the target once triggered.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions
 */
export class SetParentAction extends Action {
    private _parent;
    private _target;
    /**
     * Instantiate the action
     * @param triggerOptions defines the trigger options
     * @param target defines the target containing the parent property
     * @param parent defines from where the animation should start (animation frame)
     * @param condition defines the trigger related conditions
     */
    constructor(triggerOptions: any, target: any, parent: any, condition?: Condition);
    /** @internal */
    _prepare(): void;
    /**
     * Execute the action and set the parent property.
     */
    execute(): void;
    /**
     * Serializes the actions and its related information.
     * @param parent defines the object to serialize in
     * @returns the serialized object
     */
    serialize(parent: any): any;
}
export {};

}
declare module "babylonjs/Actions/directAudioActions" {
import { Action } from "babylonjs/Actions/action";
import { Condition } from "babylonjs/Actions/condition";
import { Sound } from "babylonjs/Audio/sound";
/**
 * This defines an action helpful to play a defined sound on a triggered action.
 */
export class PlaySoundAction extends Action {
    private _sound;
    /**
     * Instantiate the action
     * @param triggerOptions defines the trigger options
     * @param sound defines the sound to play
     * @param condition defines the trigger related conditions
     */
    constructor(triggerOptions: any, sound: Sound, condition?: Condition);
    /** @internal */
    _prepare(): void;
    /**
     * Execute the action and play the sound.
     */
    execute(): void;
    /**
     * Serializes the actions and its related information.
     * @param parent defines the object to serialize in
     * @returns the serialized object
     */
    serialize(parent: any): any;
}
/**
 * This defines an action helpful to stop a defined sound on a triggered action.
 */
export class StopSoundAction extends Action {
    private _sound;
    /**
     * Instantiate the action
     * @param triggerOptions defines the trigger options
     * @param sound defines the sound to stop
     * @param condition defines the trigger related conditions
     */
    constructor(triggerOptions: any, sound: Sound, condition?: Condition);
    /** @internal */
    _prepare(): void;
    /**
     * Execute the action and stop the sound.
     */
    execute(): void;
    /**
     * Serializes the actions and its related information.
     * @param parent defines the object to serialize in
     * @returns the serialized object
     */
    serialize(parent: any): any;
}

}
declare module "babylonjs/Actions/index" {
export * from "babylonjs/Actions/abstractActionManager";
export * from "babylonjs/Actions/action";
export * from "babylonjs/Actions/actionEvent";
export * from "babylonjs/Actions/actionManager";
export * from "babylonjs/Actions/condition";
export * from "babylonjs/Actions/directActions";
export * from "babylonjs/Actions/directAudioActions";
export * from "babylonjs/Actions/interpolateValueAction";

}
declare module "babylonjs/Actions/interpolateValueAction" {
import { Action } from "babylonjs/Actions/action";
import { Condition } from "babylonjs/Actions/condition";
import { Observable } from "babylonjs/Misc/observable";
/**
 * This defines an action responsible to change the value of a property
 * by interpolating between its current value and the newly set one once triggered.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions
 */
export class InterpolateValueAction extends Action {
    /**
     * Defines the path of the property where the value should be interpolated
     */
    propertyPath: string;
    /**
     * Defines the target value at the end of the interpolation.
     */
    value: any;
    /**
     * Defines the time it will take for the property to interpolate to the value.
     */
    duration: number;
    /**
     * Defines if the other scene animations should be stopped when the action has been triggered
     */
    stopOtherAnimations?: boolean;
    /**
     * Defines a callback raised once the interpolation animation has been done.
     */
    onInterpolationDone?: () => void;
    /**
     * Observable triggered once the interpolation animation has been done.
     */
    onInterpolationDoneObservable: Observable<InterpolateValueAction>;
    private _target;
    private _effectiveTarget;
    private _property;
    /**
     * Instantiate the action
     * @param triggerOptions defines the trigger options
     * @param target defines the object containing the value to interpolate
     * @param propertyPath defines the path to the property in the target object
     * @param value defines the target value at the end of the interpolation
     * @param duration defines the time it will take for the property to interpolate to the value.
     * @param condition defines the trigger related conditions
     * @param stopOtherAnimations defines if the other scene animations should be stopped when the action has been triggered
     * @param onInterpolationDone defines a callback raised once the interpolation animation has been done
     */
    constructor(triggerOptions: any, target: any, propertyPath: string, value: any, duration?: number, condition?: Condition, stopOtherAnimations?: boolean, onInterpolationDone?: () => void);
    /** @internal */
    _prepare(): void;
    /**
     * Execute the action starts the value interpolation.
     */
    execute(): void;
    /**
     * Serializes the actions and its related information.
     * @param parent defines the object to serialize in
     * @returns the serialized object
     */
    serialize(parent: any): any;
}

}
declare module "babylonjs/Animations/animatable" {
import { Animation } from "babylonjs/Animations/animation";
import { RuntimeAnimation } from "babylonjs/Animations/runtimeAnimation";
import { Nullable } from "babylonjs/types";
import { Observable } from "babylonjs/Misc/observable";
import { Scene } from "babylonjs/scene";
import { Matrix, Quaternion, Vector3 } from "babylonjs/Maths/math.vector";
import { Node } from "babylonjs/node";
/**
 * Class used to store an actual running animation
 */
export class Animatable {
    /** defines the target object */
    target: any;
    /** defines the starting frame number (default is 0) */
    fromFrame: number;
    /** defines the ending frame number (default is 100) */
    toFrame: number;
    /** defines if the animation must loop (default is false)  */
    loopAnimation: boolean;
    /** defines a callback to call when animation ends if it is not looping */
    onAnimationEnd?: Nullable<() => void> | undefined;
    /** defines a callback to call when animation loops */
    onAnimationLoop?: Nullable<() => void> | undefined;
    /** defines whether the animation should be evaluated additively */
    isAdditive: boolean;
    private _localDelayOffset;
    private _pausedDelay;
    private _manualJumpDelay;
    private _runtimeAnimations;
    private _paused;
    private _scene;
    private _speedRatio;
    private _weight;
    private _syncRoot;
    private _frameToSyncFromJump;
    private _goToFrame;
    /**
     * Gets or sets a boolean indicating if the animatable must be disposed and removed at the end of the animation.
     * This will only apply for non looping animation (default is true)
     */
    disposeOnEnd: boolean;
    /**
     * Gets a boolean indicating if the animation has started
     */
    animationStarted: boolean;
    /**
     * Observer raised when the animation ends
     */
    onAnimationEndObservable: Observable<Animatable>;
    /**
     * Observer raised when the animation loops
     */
    onAnimationLoopObservable: Observable<Animatable>;
    /**
     * Gets the root Animatable used to synchronize and normalize animations
     */
    get syncRoot(): Nullable<Animatable>;
    /**
     * Gets the current frame of the first RuntimeAnimation
     * Used to synchronize Animatables
     */
    get masterFrame(): number;
    /**
     * Gets or sets the animatable weight (-1.0 by default meaning not weighted)
     */
    get weight(): number;
    set weight(value: number);
    /**
     * Gets or sets the speed ratio to apply to the animatable (1.0 by default)
     */
    get speedRatio(): number;
    set speedRatio(value: number);
    /**
     * Creates a new Animatable
     * @param scene defines the hosting scene
     * @param target defines the target object
     * @param fromFrame defines the starting frame number (default is 0)
     * @param toFrame defines the ending frame number (default is 100)
     * @param loopAnimation defines if the animation must loop (default is false)
     * @param speedRatio defines the factor to apply to animation speed (default is 1)
     * @param onAnimationEnd defines a callback to call when animation ends if it is not looping
     * @param animations defines a group of animation to add to the new Animatable
     * @param onAnimationLoop defines a callback to call when animation loops
     * @param isAdditive defines whether the animation should be evaluated additively
     */
    constructor(scene: Scene, 
    /** defines the target object */
    target: any, 
    /** defines the starting frame number (default is 0) */
    fromFrame?: number, 
    /** defines the ending frame number (default is 100) */
    toFrame?: number, 
    /** defines if the animation must loop (default is false)  */
    loopAnimation?: boolean, speedRatio?: number, 
    /** defines a callback to call when animation ends if it is not looping */
    onAnimationEnd?: Nullable<() => void> | undefined, animations?: Animation[], 
    /** defines a callback to call when animation loops */
    onAnimationLoop?: Nullable<() => void> | undefined, 
    /** defines whether the animation should be evaluated additively */
    isAdditive?: boolean);
    /**
     * Synchronize and normalize current Animatable with a source Animatable
     * This is useful when using animation weights and when animations are not of the same length
     * @param root defines the root Animatable to synchronize with (null to stop synchronizing)
     * @returns the current Animatable
     */
    syncWith(root: Nullable<Animatable>): Animatable;
    /**
     * Gets the list of runtime animations
     * @returns an array of RuntimeAnimation
     */
    getAnimations(): RuntimeAnimation[];
    /**
     * Adds more animations to the current animatable
     * @param target defines the target of the animations
     * @param animations defines the new animations to add
     */
    appendAnimations(target: any, animations: Animation[]): void;
    /**
     * Gets the source animation for a specific property
     * @param property defines the property to look for
     * @returns null or the source animation for the given property
     */
    getAnimationByTargetProperty(property: string): Nullable<Animation>;
    /**
     * Gets the runtime animation for a specific property
     * @param property defines the property to look for
     * @returns null or the runtime animation for the given property
     */
    getRuntimeAnimationByTargetProperty(property: string): Nullable<RuntimeAnimation>;
    /**
     * Resets the animatable to its original state
     */
    reset(): void;
    /**
     * Allows the animatable to blend with current running animations
     * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/advanced_animations#animation-blending
     * @param blendingSpeed defines the blending speed to use
     */
    enableBlending(blendingSpeed: number): void;
    /**
     * Disable animation blending
     * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/advanced_animations#animation-blending
     */
    disableBlending(): void;
    /**
     * Jump directly to a given frame
     * @param frame defines the frame to jump to
     */
    goToFrame(frame: number): void;
    /**
     * Pause the animation
     */
    pause(): void;
    /**
     * Restart the animation
     */
    restart(): void;
    private _raiseOnAnimationEnd;
    /**
     * Stop and delete the current animation
     * @param animationName defines a string used to only stop some of the runtime animations instead of all
     * @param targetMask - a function that determines if the animation should be stopped based on its target (all animations will be stopped if both this and animationName are empty)
     */
    stop(animationName?: string, targetMask?: (target: any) => boolean): void;
    /**
     * Wait asynchronously for the animation to end
     * @returns a promise which will be fulfilled when the animation ends
     */
    waitAsync(): Promise<Animatable>;
    /**
     * @internal
     */
    _animate(delay: number): boolean;
}
module "babylonjs/scene" {
    interface Scene {
        /** @internal */
        _registerTargetForLateAnimationBinding(runtimeAnimation: RuntimeAnimation, originalValue: any): void;
        /** @internal */
        _processLateAnimationBindingsForMatrices(holder: {
            totalWeight: number;
            totalAdditiveWeight: number;
            animations: RuntimeAnimation[];
            additiveAnimations: RuntimeAnimation[];
            originalValue: Matrix;
        }): any;
        /** @internal */
        _processLateAnimationBindingsForQuaternions(holder: {
            totalWeight: number;
            totalAdditiveWeight: number;
            animations: RuntimeAnimation[];
            additiveAnimations: RuntimeAnimation[];
            originalValue: Quaternion;
        }, refQuaternion: Quaternion): Quaternion;
        /** @internal */
        _processLateAnimationBindings(): void;
        /**
         * Will start the animation sequence of a given target
         * @param target defines the target
         * @param from defines from which frame should animation start
         * @param to defines until which frame should animation run.
         * @param weight defines the weight to apply to the animation (1.0 by default)
         * @param loop defines if the animation loops
         * @param speedRatio defines the speed in which to run the animation (1.0 by default)
         * @param onAnimationEnd defines the function to be executed when the animation ends
         * @param animatable defines an animatable object. If not provided a new one will be created from the given params
         * @param targetMask defines if the target should be animated if animations are present (this is called recursively on descendant animatables regardless of return value)
         * @param onAnimationLoop defines the callback to call when an animation loops
         * @param isAdditive defines whether the animation should be evaluated additively (false by default)
         * @returns the animatable object created for this animation
         */
        beginWeightedAnimation(target: any, from: number, to: number, weight: number, loop?: boolean, speedRatio?: number, onAnimationEnd?: () => void, animatable?: Animatable, targetMask?: (target: any) => boolean, onAnimationLoop?: () => void, isAdditive?: boolean): Animatable;
        /**
         * Will start the animation sequence of a given target
         * @param target defines the target
         * @param from defines from which frame should animation start
         * @param to defines until which frame should animation run.
         * @param loop defines if the animation loops
         * @param speedRatio defines the speed in which to run the animation (1.0 by default)
         * @param onAnimationEnd defines the function to be executed when the animation ends
         * @param animatable defines an animatable object. If not provided a new one will be created from the given params
         * @param stopCurrent defines if the current animations must be stopped first (true by default)
         * @param targetMask defines if the target should be animate if animations are present (this is called recursively on descendant animatables regardless of return value)
         * @param onAnimationLoop defines the callback to call when an animation loops
         * @param isAdditive defines whether the animation should be evaluated additively (false by default)
         * @returns the animatable object created for this animation
         */
        beginAnimation(target: any, from: number, to: number, loop?: boolean, speedRatio?: number, onAnimationEnd?: () => void, animatable?: Animatable, stopCurrent?: boolean, targetMask?: (target: any) => boolean, onAnimationLoop?: () => void, isAdditive?: boolean): Animatable;
        /**
         * Will start the animation sequence of a given target and its hierarchy
         * @param target defines the target
         * @param directDescendantsOnly if true only direct descendants will be used, if false direct and also indirect (children of children, an so on in a recursive manner) descendants will be used.
         * @param from defines from which frame should animation start
         * @param to defines until which frame should animation run.
         * @param loop defines if the animation loops
         * @param speedRatio defines the speed in which to run the animation (1.0 by default)
         * @param onAnimationEnd defines the function to be executed when the animation ends
         * @param animatable defines an animatable object. If not provided a new one will be created from the given params
         * @param stopCurrent defines if the current animations must be stopped first (true by default)
         * @param targetMask defines if the target should be animated if animations are present (this is called recursively on descendant animatables regardless of return value)
         * @param onAnimationLoop defines the callback to call when an animation loops
         * @param isAdditive defines whether the animation should be evaluated additively (false by default)
         * @returns the list of created animatables
         */
        beginHierarchyAnimation(target: any, directDescendantsOnly: boolean, from: number, to: number, loop?: boolean, speedRatio?: number, onAnimationEnd?: () => void, animatable?: Animatable, stopCurrent?: boolean, targetMask?: (target: any) => boolean, onAnimationLoop?: () => void, isAdditive?: boolean): Animatable[];
        /**
         * Begin a new animation on a given node
         * @param target defines the target where the animation will take place
         * @param animations defines the list of animations to start
         * @param from defines the initial value
         * @param to defines the final value
         * @param loop defines if you want animation to loop (off by default)
         * @param speedRatio defines the speed ratio to apply to all animations
         * @param onAnimationEnd defines the callback to call when an animation ends (will be called once per node)
         * @param onAnimationLoop defines the callback to call when an animation loops
         * @param isAdditive defines whether the animation should be evaluated additively (false by default)
         * @returns the list of created animatables
         */
        beginDirectAnimation(target: any, animations: Animation[], from: number, to: number, loop?: boolean, speedRatio?: number, onAnimationEnd?: () => void, onAnimationLoop?: () => void, isAdditive?: boolean): Animatable;
        /**
         * Begin a new animation on a given node and its hierarchy
         * @param target defines the root node where the animation will take place
         * @param directDescendantsOnly if true only direct descendants will be used, if false direct and also indirect (children of children, an so on in a recursive manner) descendants will be used.
         * @param animations defines the list of animations to start
         * @param from defines the initial value
         * @param to defines the final value
         * @param loop defines if you want animation to loop (off by default)
         * @param speedRatio defines the speed ratio to apply to all animations
         * @param onAnimationEnd defines the callback to call when an animation ends (will be called once per node)
         * @param onAnimationLoop defines the callback to call when an animation loops
         * @param isAdditive defines whether the animation should be evaluated additively (false by default)
         * @returns the list of animatables created for all nodes
         */
        beginDirectHierarchyAnimation(target: Node, directDescendantsOnly: boolean, animations: Animation[], from: number, to: number, loop?: boolean, speedRatio?: number, onAnimationEnd?: () => void, onAnimationLoop?: () => void, isAdditive?: boolean): Animatable[];
        /**
         * Gets the animatable associated with a specific target
         * @param target defines the target of the animatable
         * @returns the required animatable if found
         */
        getAnimatableByTarget(target: any): Nullable<Animatable>;
        /**
         * Gets all animatables associated with a given target
         * @param target defines the target to look animatables for
         * @returns an array of Animatables
         */
        getAllAnimatablesByTarget(target: any): Array<Animatable>;
        /**
         * Stops and removes all animations that have been applied to the scene
         */
        stopAllAnimations(): void;
        /**
         * Gets the current delta time used by animation engine
         */
        deltaTime: number;
    }
}
module "babylonjs/Bones/bone" {
    interface Bone {
        /**
         * Copy an animation range from another bone
         * @param source defines the source bone
         * @param rangeName defines the range name to copy
         * @param frameOffset defines the frame offset
         * @param rescaleAsRequired defines if rescaling must be applied if required
         * @param skelDimensionsRatio defines the scaling ratio
         * @returns true if operation was successful
         */
        copyAnimationRange(source: Bone, rangeName: string, frameOffset: number, rescaleAsRequired: boolean, skelDimensionsRatio: Nullable<Vector3>): boolean;
    }
}

}
declare module "babylonjs/Animations/animatable.interface" {
import { Nullable } from "babylonjs/types";
import { Animation } from "babylonjs/Animations/animation";
/**
 * Interface containing an array of animations
 */
export interface IAnimatable {
    /**
     * Array of animations
     */
    animations: Nullable<Array<Animation>>;
}
export {};

}
declare module "babylonjs/Animations/animation" {
import { IEasingFunction, EasingFunction } from "babylonjs/Animations/easing";
import { Vector3, Quaternion, Vector2, Matrix } from "babylonjs/Maths/math.vector";
import { Color3, Color4 } from "babylonjs/Maths/math.color";
import { Nullable } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
import { IAnimationKey } from "babylonjs/Animations/animationKey";
import { AnimationRange } from "babylonjs/Animations/animationRange";
import { AnimationEvent } from "babylonjs/Animations/animationEvent";
import { Node } from "babylonjs/node";
import { IAnimatable } from "babylonjs/Animations/animatable.interface";
import { Size } from "babylonjs/Maths/math.size";
import { Animatable } from "babylonjs/Animations/animatable";
import { RuntimeAnimation } from "babylonjs/Animations/runtimeAnimation";
/**
 * @internal
 */
export class _IAnimationState {
    key: number;
    repeatCount: number;
    workValue?: any;
    loopMode?: number;
    offsetValue?: any;
    highLimitValue?: any;
}
/**
 * Class used to store any kind of animation
 */
export class Animation {
    /**Name of the animation */
    name: string;
    /**Property to animate */
    targetProperty: string;
    /**The frames per second of the animation */
    framePerSecond: number;
    /**The data type of the animation */
    dataType: number;
    /**The loop mode of the animation */
    loopMode?: number | undefined;
    /**Specifies if blending should be enabled */
    enableBlending?: boolean | undefined;
    private static _UniqueIdGenerator;
    /**
     * Use matrix interpolation instead of using direct key value when animating matrices
     */
    static AllowMatricesInterpolation: boolean;
    /**
     * When matrix interpolation is enabled, this boolean forces the system to use Matrix.DecomposeLerp instead of Matrix.Lerp. Interpolation is more precise but slower
     */
    static AllowMatrixDecomposeForInterpolation: boolean;
    /**
     * Gets or sets the unique id of the animation (the uniqueness is solely among other animations)
     */
    uniqueId: number;
    /** Define the Url to load snippets */
    static SnippetUrl: string;
    /** Snippet ID if the animation was created from the snippet server */
    snippetId: string;
    /**
     * Stores the key frames of the animation
     */
    private _keys;
    /**
     * Stores the easing function of the animation
     */
    private _easingFunction;
    /**
     * @internal Internal use only
     */
    _runtimeAnimations: import("babylonjs/Animations/runtimeAnimation").RuntimeAnimation[];
    /**
     * The set of event that will be linked to this animation
     */
    private _events;
    /**
     * Stores an array of target property paths
     */
    targetPropertyPath: string[];
    /**
     * Stores the blending speed of the animation
     */
    blendingSpeed: number;
    /**
     * Stores the animation ranges for the animation
     */
    private _ranges;
    /**
     * @internal Internal use
     */
    static _PrepareAnimation(name: string, targetProperty: string, framePerSecond: number, totalFrame: number, from: any, to: any, loopMode?: number, easingFunction?: EasingFunction): Nullable<Animation>;
    /**
     * Sets up an animation
     * @param property The property to animate
     * @param animationType The animation type to apply
     * @param framePerSecond The frames per second of the animation
     * @param easingFunction The easing function used in the animation
     * @returns The created animation
     */
    static CreateAnimation(property: string, animationType: number, framePerSecond: number, easingFunction: EasingFunction): Animation;
    /**
     * Create and start an animation on a node
     * @param name defines the name of the global animation that will be run on all nodes
     * @param target defines the target where the animation will take place
     * @param targetProperty defines property to animate
     * @param framePerSecond defines the number of frame per second yo use
     * @param totalFrame defines the number of frames in total
     * @param from defines the initial value
     * @param to defines the final value
     * @param loopMode defines which loop mode you want to use (off by default)
     * @param easingFunction defines the easing function to use (linear by default)
     * @param onAnimationEnd defines the callback to call when animation end
     * @param scene defines the hosting scene
     * @returns the animatable created for this animation
     */
    static CreateAndStartAnimation(name: string, target: any, targetProperty: string, framePerSecond: number, totalFrame: number, from: any, to: any, loopMode?: number, easingFunction?: EasingFunction, onAnimationEnd?: () => void, scene?: Scene): Nullable<Animatable>;
    /**
     * Create and start an animation on a node and its descendants
     * @param name defines the name of the global animation that will be run on all nodes
     * @param node defines the root node where the animation will take place
     * @param directDescendantsOnly if true only direct descendants will be used, if false direct and also indirect (children of children, an so on in a recursive manner) descendants will be used
     * @param targetProperty defines property to animate
     * @param framePerSecond defines the number of frame per second to use
     * @param totalFrame defines the number of frames in total
     * @param from defines the initial value
     * @param to defines the final value
     * @param loopMode defines which loop mode you want to use (off by default)
     * @param easingFunction defines the easing function to use (linear by default)
     * @param onAnimationEnd defines the callback to call when an animation ends (will be called once per node)
     * @returns the list of animatables created for all nodes
     * @example https://www.babylonjs-playground.com/#MH0VLI
     */
    static CreateAndStartHierarchyAnimation(name: string, node: Node, directDescendantsOnly: boolean, targetProperty: string, framePerSecond: number, totalFrame: number, from: any, to: any, loopMode?: number, easingFunction?: EasingFunction, onAnimationEnd?: () => void): Nullable<Animatable[]>;
    /**
     * Creates a new animation, merges it with the existing animations and starts it
     * @param name Name of the animation
     * @param node Node which contains the scene that begins the animations
     * @param targetProperty Specifies which property to animate
     * @param framePerSecond The frames per second of the animation
     * @param totalFrame The total number of frames
     * @param from The frame at the beginning of the animation
     * @param to The frame at the end of the animation
     * @param loopMode Specifies the loop mode of the animation
     * @param easingFunction (Optional) The easing function of the animation, which allow custom mathematical formulas for animations
     * @param onAnimationEnd Callback to run once the animation is complete
     * @returns Nullable animation
     */
    static CreateMergeAndStartAnimation(name: string, node: Node, targetProperty: string, framePerSecond: number, totalFrame: number, from: any, to: any, loopMode?: number, easingFunction?: EasingFunction, onAnimationEnd?: () => void): Nullable<Animatable>;
    /**
     * Convert the keyframes for all animations belonging to the group to be relative to a given reference frame.
     * @param sourceAnimation defines the Animation containing keyframes to convert
     * @param referenceFrame defines the frame that keyframes in the range will be relative to
     * @param range defines the name of the AnimationRange belonging to the Animation to convert
     * @param cloneOriginal defines whether or not to clone the animation and convert the clone or convert the original animation (default is false)
     * @param clonedName defines the name of the resulting cloned Animation if cloneOriginal is true
     * @returns a new Animation if cloneOriginal is true or the original Animation if cloneOriginal is false
     */
    static MakeAnimationAdditive(sourceAnimation: Animation, referenceFrame?: number, range?: string, cloneOriginal?: boolean, clonedName?: string): Animation;
    /**
     * Transition property of an host to the target Value
     * @param property The property to transition
     * @param targetValue The target Value of the property
     * @param host The object where the property to animate belongs
     * @param scene Scene used to run the animation
     * @param frameRate Framerate (in frame/s) to use
     * @param transition The transition type we want to use
     * @param duration The duration of the animation, in milliseconds
     * @param onAnimationEnd Callback trigger at the end of the animation
     * @returns Nullable animation
     */
    static TransitionTo(property: string, targetValue: any, host: any, scene: Scene, frameRate: number, transition: Animation, duration: number, onAnimationEnd?: Nullable<() => void>): Nullable<Animatable>;
    /**
     * Return the array of runtime animations currently using this animation
     */
    get runtimeAnimations(): RuntimeAnimation[];
    /**
     * Specifies if any of the runtime animations are currently running
     */
    get hasRunningRuntimeAnimations(): boolean;
    /**
     * Initializes the animation
     * @param name Name of the animation
     * @param targetProperty Property to animate
     * @param framePerSecond The frames per second of the animation
     * @param dataType The data type of the animation
     * @param loopMode The loop mode of the animation
     * @param enableBlending Specifies if blending should be enabled
     */
    constructor(
    /**Name of the animation */
    name: string, 
    /**Property to animate */
    targetProperty: string, 
    /**The frames per second of the animation */
    framePerSecond: number, 
    /**The data type of the animation */
    dataType: number, 
    /**The loop mode of the animation */
    loopMode?: number | undefined, 
    /**Specifies if blending should be enabled */
    enableBlending?: boolean | undefined);
    /**
     * Converts the animation to a string
     * @param fullDetails support for multiple levels of logging within scene loading
     * @returns String form of the animation
     */
    toString(fullDetails?: boolean): string;
    /**
     * Add an event to this animation
     * @param event Event to add
     */
    addEvent(event: AnimationEvent): void;
    /**
     * Remove all events found at the given frame
     * @param frame The frame to remove events from
     */
    removeEvents(frame: number): void;
    /**
     * Retrieves all the events from the animation
     * @returns Events from the animation
     */
    getEvents(): AnimationEvent[];
    /**
     * Creates an animation range
     * @param name Name of the animation range
     * @param from Starting frame of the animation range
     * @param to Ending frame of the animation
     */
    createRange(name: string, from: number, to: number): void;
    /**
     * Deletes an animation range by name
     * @param name Name of the animation range to delete
     * @param deleteFrames Specifies if the key frames for the range should also be deleted (true) or not (false)
     */
    deleteRange(name: string, deleteFrames?: boolean): void;
    /**
     * Gets the animation range by name, or null if not defined
     * @param name Name of the animation range
     * @returns Nullable animation range
     */
    getRange(name: string): Nullable<AnimationRange>;
    /**
     * Gets the key frames from the animation
     * @returns The key frames of the animation
     */
    getKeys(): Array<IAnimationKey>;
    /**
     * Gets the highest frame rate of the animation
     * @returns Highest frame rate of the animation
     */
    getHighestFrame(): number;
    /**
     * Gets the easing function of the animation
     * @returns Easing function of the animation
     */
    getEasingFunction(): Nullable<IEasingFunction>;
    /**
     * Sets the easing function of the animation
     * @param easingFunction A custom mathematical formula for animation
     */
    setEasingFunction(easingFunction: Nullable<IEasingFunction>): void;
    /**
     * Interpolates a scalar linearly
     * @param startValue Start value of the animation curve
     * @param endValue End value of the animation curve
     * @param gradient Scalar amount to interpolate
     * @returns Interpolated scalar value
     */
    floatInterpolateFunction(startValue: number, endValue: number, gradient: number): number;
    /**
     * Interpolates a scalar cubically
     * @param startValue Start value of the animation curve
     * @param outTangent End tangent of the animation
     * @param endValue End value of the animation curve
     * @param inTangent Start tangent of the animation curve
     * @param gradient Scalar amount to interpolate
     * @returns Interpolated scalar value
     */
    floatInterpolateFunctionWithTangents(startValue: number, outTangent: number, endValue: number, inTangent: number, gradient: number): number;
    /**
     * Interpolates a quaternion using a spherical linear interpolation
     * @param startValue Start value of the animation curve
     * @param endValue End value of the animation curve
     * @param gradient Scalar amount to interpolate
     * @returns Interpolated quaternion value
     */
    quaternionInterpolateFunction(startValue: Quaternion, endValue: Quaternion, gradient: number): Quaternion;
    /**
     * Interpolates a quaternion cubically
     * @param startValue Start value of the animation curve
     * @param outTangent End tangent of the animation curve
     * @param endValue End value of the animation curve
     * @param inTangent Start tangent of the animation curve
     * @param gradient Scalar amount to interpolate
     * @returns Interpolated quaternion value
     */
    quaternionInterpolateFunctionWithTangents(startValue: Quaternion, outTangent: Quaternion, endValue: Quaternion, inTangent: Quaternion, gradient: number): Quaternion;
    /**
     * Interpolates a Vector3 linearly
     * @param startValue Start value of the animation curve
     * @param endValue End value of the animation curve
     * @param gradient Scalar amount to interpolate (value between 0 and 1)
     * @returns Interpolated scalar value
     */
    vector3InterpolateFunction(startValue: Vector3, endValue: Vector3, gradient: number): Vector3;
    /**
     * Interpolates a Vector3 cubically
     * @param startValue Start value of the animation curve
     * @param outTangent End tangent of the animation
     * @param endValue End value of the animation curve
     * @param inTangent Start tangent of the animation curve
     * @param gradient Scalar amount to interpolate (value between 0 and 1)
     * @returns InterpolatedVector3 value
     */
    vector3InterpolateFunctionWithTangents(startValue: Vector3, outTangent: Vector3, endValue: Vector3, inTangent: Vector3, gradient: number): Vector3;
    /**
     * Interpolates a Vector2 linearly
     * @param startValue Start value of the animation curve
     * @param endValue End value of the animation curve
     * @param gradient Scalar amount to interpolate (value between 0 and 1)
     * @returns Interpolated Vector2 value
     */
    vector2InterpolateFunction(startValue: Vector2, endValue: Vector2, gradient: number): Vector2;
    /**
     * Interpolates a Vector2 cubically
     * @param startValue Start value of the animation curve
     * @param outTangent End tangent of the animation
     * @param endValue End value of the animation curve
     * @param inTangent Start tangent of the animation curve
     * @param gradient Scalar amount to interpolate (value between 0 and 1)
     * @returns Interpolated Vector2 value
     */
    vector2InterpolateFunctionWithTangents(startValue: Vector2, outTangent: Vector2, endValue: Vector2, inTangent: Vector2, gradient: number): Vector2;
    /**
     * Interpolates a size linearly
     * @param startValue Start value of the animation curve
     * @param endValue End value of the animation curve
     * @param gradient Scalar amount to interpolate
     * @returns Interpolated Size value
     */
    sizeInterpolateFunction(startValue: Size, endValue: Size, gradient: number): Size;
    /**
     * Interpolates a Color3 linearly
     * @param startValue Start value of the animation curve
     * @param endValue End value of the animation curve
     * @param gradient Scalar amount to interpolate
     * @returns Interpolated Color3 value
     */
    color3InterpolateFunction(startValue: Color3, endValue: Color3, gradient: number): Color3;
    /**
     * Interpolates a Color3 cubically
     * @param startValue Start value of the animation curve
     * @param outTangent End tangent of the animation
     * @param endValue End value of the animation curve
     * @param inTangent Start tangent of the animation curve
     * @param gradient Scalar amount to interpolate
     * @returns interpolated value
     */
    color3InterpolateFunctionWithTangents(startValue: Color3, outTangent: Color3, endValue: Color3, inTangent: Color3, gradient: number): Color3;
    /**
     * Interpolates a Color4 linearly
     * @param startValue Start value of the animation curve
     * @param endValue End value of the animation curve
     * @param gradient Scalar amount to interpolate
     * @returns Interpolated Color3 value
     */
    color4InterpolateFunction(startValue: Color4, endValue: Color4, gradient: number): Color4;
    /**
     * Interpolates a Color4 cubically
     * @param startValue Start value of the animation curve
     * @param outTangent End tangent of the animation
     * @param endValue End value of the animation curve
     * @param inTangent Start tangent of the animation curve
     * @param gradient Scalar amount to interpolate
     * @returns interpolated value
     */
    color4InterpolateFunctionWithTangents(startValue: Color4, outTangent: Color4, endValue: Color4, inTangent: Color4, gradient: number): Color4;
    /**
     * @internal Internal use only
     */
    _getKeyValue(value: any): any;
    /**
     * Evaluate the animation value at a given frame
     * @param currentFrame defines the frame where we want to evaluate the animation
     * @returns the animation value
     */
    evaluate(currentFrame: number): any;
    /**
     * @internal Internal use only
     */
    _interpolate(currentFrame: number, state: _IAnimationState): any;
    /**
     * Defines the function to use to interpolate matrices
     * @param startValue defines the start matrix
     * @param endValue defines the end matrix
     * @param gradient defines the gradient between both matrices
     * @param result defines an optional target matrix where to store the interpolation
     * @returns the interpolated matrix
     */
    matrixInterpolateFunction(startValue: Matrix, endValue: Matrix, gradient: number, result?: Matrix): Matrix;
    /**
     * Makes a copy of the animation
     * @returns Cloned animation
     */
    clone(): Animation;
    /**
     * Sets the key frames of the animation
     * @param values The animation key frames to set
     */
    setKeys(values: Array<IAnimationKey>): void;
    /**
     * Serializes the animation to an object
     * @returns Serialized object
     */
    serialize(): any;
    /**
     * Float animation type
     */
    static readonly ANIMATIONTYPE_FLOAT: number;
    /**
     * Vector3 animation type
     */
    static readonly ANIMATIONTYPE_VECTOR3: number;
    /**
     * Quaternion animation type
     */
    static readonly ANIMATIONTYPE_QUATERNION: number;
    /**
     * Matrix animation type
     */
    static readonly ANIMATIONTYPE_MATRIX: number;
    /**
     * Color3 animation type
     */
    static readonly ANIMATIONTYPE_COLOR3: number;
    /**
     * Color3 animation type
     */
    static readonly ANIMATIONTYPE_COLOR4: number;
    /**
     * Vector2 animation type
     */
    static readonly ANIMATIONTYPE_VECTOR2: number;
    /**
     * Size animation type
     */
    static readonly ANIMATIONTYPE_SIZE: number;
    /**
     * Relative Loop Mode
     */
    static readonly ANIMATIONLOOPMODE_RELATIVE: number;
    /**
     * Cycle Loop Mode
     */
    static readonly ANIMATIONLOOPMODE_CYCLE: number;
    /**
     * Constant Loop Mode
     */
    static readonly ANIMATIONLOOPMODE_CONSTANT: number;
    /**
     * @internal
     */
    static _UniversalLerp(left: any, right: any, amount: number): any;
    /**
     * Parses an animation object and creates an animation
     * @param parsedAnimation Parsed animation object
     * @returns Animation object
     */
    static Parse(parsedAnimation: any): Animation;
    /**
     * Appends the serialized animations from the source animations
     * @param source Source containing the animations
     * @param destination Target to store the animations
     */
    static AppendSerializedAnimations(source: IAnimatable, destination: any): void;
    /**
     * Creates a new animation or an array of animations from a snippet saved in a remote file
     * @param name defines the name of the animation to create (can be null or empty to use the one from the json data)
     * @param url defines the url to load from
     * @returns a promise that will resolve to the new animation or an array of animations
     */
    static ParseFromFileAsync(name: Nullable<string>, url: string): Promise<Animation | Array<Animation>>;
    /**
     * Creates an animation or an array of animations from a snippet saved by the Inspector
     * @param snippetId defines the snippet to load
     * @returns a promise that will resolve to the new animation or a new array of animations
     */
    static ParseFromSnippetAsync(snippetId: string): Promise<Animation | Array<Animation>>;
    /**
     * Creates an animation or an array of animations from a snippet saved by the Inspector
     * @deprecated Please use ParseFromSnippetAsync instead
     * @param snippetId defines the snippet to load
     * @returns a promise that will resolve to the new animation or a new array of animations
     */
    static CreateFromSnippetAsync: typeof Animation.ParseFromSnippetAsync;
}
export {};

}
declare module "babylonjs/Animations/animationEvent" {
/**
 * Composed of a frame, and an action function
 */
export class AnimationEvent {
    /** The frame for which the event is triggered **/
    frame: number;
    /** The event to perform when triggered **/
    action: (currentFrame: number) => void;
    /** Specifies if the event should be triggered only once**/
    onlyOnce?: boolean | undefined;
    /**
     * Specifies if the animation event is done
     */
    isDone: boolean;
    /**
     * Initializes the animation event
     * @param frame The frame for which the event is triggered
     * @param action The event to perform when triggered
     * @param onlyOnce Specifies if the event should be triggered only once
     */
    constructor(
    /** The frame for which the event is triggered **/
    frame: number, 
    /** The event to perform when triggered **/
    action: (currentFrame: number) => void, 
    /** Specifies if the event should be triggered only once**/
    onlyOnce?: boolean | undefined);
    /** @internal */
    _clone(): AnimationEvent;
}

}
declare module "babylonjs/Animations/animationGroup" {
import { Animatable } from "babylonjs/Animations/animatable";
import { Animation } from "babylonjs/Animations/animation";
import { Scene, IDisposable } from "babylonjs/scene";
import { Observable } from "babylonjs/Misc/observable";
import { Nullable } from "babylonjs/types";
import { AbstractScene } from "babylonjs/abstractScene";
/**
 * This class defines the direct association between an animation and a target
 */
export class TargetedAnimation {
    /**
     * Animation to perform
     */
    animation: Animation;
    /**
     * Target to animate
     */
    target: any;
    /**
     * Returns the string "TargetedAnimation"
     * @returns "TargetedAnimation"
     */
    getClassName(): string;
    /**
     * Serialize the object
     * @returns the JSON object representing the current entity
     */
    serialize(): any;
}
/**
 * Use this class to create coordinated animations on multiple targets
 */
export class AnimationGroup implements IDisposable {
    /** The name of the animation group */
    name: string;
    private _scene;
    private _targetedAnimations;
    private _animatables;
    private _from;
    private _to;
    private _isStarted;
    private _isPaused;
    private _speedRatio;
    private _loopAnimation;
    private _isAdditive;
    /** @internal */
    _parentContainer: Nullable<AbstractScene>;
    /**
     * Gets or sets the unique id of the node
     */
    uniqueId: number;
    /**
     * This observable will notify when one animation have ended
     */
    onAnimationEndObservable: Observable<TargetedAnimation>;
    /**
     * Observer raised when one animation loops
     */
    onAnimationLoopObservable: Observable<TargetedAnimation>;
    /**
     * Observer raised when all animations have looped
     */
    onAnimationGroupLoopObservable: Observable<AnimationGroup>;
    /**
     * This observable will notify when all animations have ended.
     */
    onAnimationGroupEndObservable: Observable<AnimationGroup>;
    /**
     * This observable will notify when all animations have paused.
     */
    onAnimationGroupPauseObservable: Observable<AnimationGroup>;
    /**
     * This observable will notify when all animations are playing.
     */
    onAnimationGroupPlayObservable: Observable<AnimationGroup>;
    /**
     * Gets or sets an object used to store user defined information for the node
     */
    metadata: any;
    /**
     * Gets the first frame
     */
    get from(): number;
    /**
     * Gets the last frame
     */
    get to(): number;
    /**
     * Define if the animations are started
     */
    get isStarted(): boolean;
    /**
     * Gets a value indicating that the current group is playing
     */
    get isPlaying(): boolean;
    /**
     * Gets or sets the speed ratio to use for all animations
     */
    get speedRatio(): number;
    /**
     * Gets or sets the speed ratio to use for all animations
     */
    set speedRatio(value: number);
    /**
     * Gets or sets if all animations should loop or not
     */
    get loopAnimation(): boolean;
    set loopAnimation(value: boolean);
    /**
     * Gets or sets if all animations should be evaluated additively
     */
    get isAdditive(): boolean;
    set isAdditive(value: boolean);
    /**
     * Gets the targeted animations for this animation group
     */
    get targetedAnimations(): Array<TargetedAnimation>;
    /**
     * returning the list of animatables controlled by this animation group.
     */
    get animatables(): Array<Animatable>;
    /**
     * Gets the list of target animations
     */
    get children(): TargetedAnimation[];
    /**
     * Instantiates a new Animation Group.
     * This helps managing several animations at once.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/groupAnimations
     * @param name Defines the name of the group
     * @param scene Defines the scene the group belongs to
     */
    constructor(
    /** The name of the animation group */
    name: string, scene?: Nullable<Scene>);
    /**
     * Add an animation (with its target) in the group
     * @param animation defines the animation we want to add
     * @param target defines the target of the animation
     * @returns the TargetedAnimation object
     */
    addTargetedAnimation(animation: Animation, target: any): TargetedAnimation;
    /**
     * This function will normalize every animation in the group to make sure they all go from beginFrame to endFrame
     * It can add constant keys at begin or end
     * @param beginFrame defines the new begin frame for all animations or the smallest begin frame of all animations if null (defaults to null)
     * @param endFrame defines the new end frame for all animations or the largest end frame of all animations if null (defaults to null)
     * @returns the animation group
     */
    normalize(beginFrame?: Nullable<number>, endFrame?: Nullable<number>): AnimationGroup;
    private _animationLoopCount;
    private _animationLoopFlags;
    private _processLoop;
    /**
     * Start all animations on given targets
     * @param loop defines if animations must loop
     * @param speedRatio defines the ratio to apply to animation speed (1 by default)
     * @param from defines the from key (optional)
     * @param to defines the to key (optional)
     * @param isAdditive defines the additive state for the resulting animatables (optional)
     * @returns the current animation group
     */
    start(loop?: boolean, speedRatio?: number, from?: number, to?: number, isAdditive?: boolean): AnimationGroup;
    /**
     * Pause all animations
     * @returns the animation group
     */
    pause(): AnimationGroup;
    /**
     * Play all animations to initial state
     * This function will start() the animations if they were not started or will restart() them if they were paused
     * @param loop defines if animations must loop
     * @returns the animation group
     */
    play(loop?: boolean): AnimationGroup;
    /**
     * Reset all animations to initial state
     * @returns the animation group
     */
    reset(): AnimationGroup;
    /**
     * Restart animations from key 0
     * @returns the animation group
     */
    restart(): AnimationGroup;
    /**
     * Stop all animations
     * @returns the animation group
     */
    stop(): AnimationGroup;
    /**
     * Set animation weight for all animatables
     * @param weight defines the weight to use
     * @returns the animationGroup
     * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/advanced_animations#animation-weights
     */
    setWeightForAllAnimatables(weight: number): AnimationGroup;
    /**
     * Synchronize and normalize all animatables with a source animatable
     * @param root defines the root animatable to synchronize with (null to stop synchronizing)
     * @returns the animationGroup
     * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/advanced_animations#animation-weights
     */
    syncAllAnimationsWith(root: Nullable<Animatable>): AnimationGroup;
    /**
     * Goes to a specific frame in this animation group
     * @param frame the frame number to go to
     * @returns the animationGroup
     */
    goToFrame(frame: number): AnimationGroup;
    /**
     * Dispose all associated resources
     */
    dispose(): void;
    private _checkAnimationGroupEnded;
    /**
     * Clone the current animation group and returns a copy
     * @param newName defines the name of the new group
     * @param targetConverter defines an optional function used to convert current animation targets to new ones
     * @param cloneAnimations defines if the animations should be cloned or referenced
     * @returns the new animation group
     */
    clone(newName: string, targetConverter?: (oldTarget: any) => any, cloneAnimations?: boolean): AnimationGroup;
    /**
     * Serializes the animationGroup to an object
     * @returns Serialized object
     */
    serialize(): any;
    /**
     * Returns a new AnimationGroup object parsed from the source provided.
     * @param parsedAnimationGroup defines the source
     * @param scene defines the scene that will receive the animationGroup
     * @returns a new AnimationGroup
     */
    static Parse(parsedAnimationGroup: any, scene: Scene): AnimationGroup;
    /**
     * Convert the keyframes for all animations belonging to the group to be relative to a given reference frame.
     * @param sourceAnimationGroup defines the AnimationGroup containing animations to convert
     * @param referenceFrame defines the frame that keyframes in the range will be relative to
     * @param range defines the name of the AnimationRange belonging to the animations in the group to convert
     * @param cloneOriginal defines whether or not to clone the group and convert the clone or convert the original group (default is false)
     * @param clonedName defines the name of the resulting cloned AnimationGroup if cloneOriginal is true
     * @returns a new AnimationGroup if cloneOriginal is true or the original AnimationGroup if cloneOriginal is false
     */
    static MakeAnimationAdditive(sourceAnimationGroup: AnimationGroup, referenceFrame?: number, range?: string, cloneOriginal?: boolean, clonedName?: string): AnimationGroup;
    /**
     * Returns the string "AnimationGroup"
     * @returns "AnimationGroup"
     */
    getClassName(): string;
    /**
     * Creates a detailed string about the object
     * @param fullDetails defines if the output string will support multiple levels of logging within scene loading
     * @returns a string representing the object
     */
    toString(fullDetails?: boolean): string;
}

}
declare module "babylonjs/Animations/animationKey" {
/**
 * Defines an interface which represents an animation key frame
 */
export interface IAnimationKey {
    /**
     * Frame of the key frame
     */
    frame: number;
    /**
     * Value at the specifies key frame
     */
    value: any;
    /**
     * The input tangent for the cubic hermite spline
     */
    inTangent?: any;
    /**
     * The output tangent for the cubic hermite spline
     */
    outTangent?: any;
    /**
     * The animation interpolation type
     */
    interpolation?: AnimationKeyInterpolation;
    /**
     * Property defined by UI tools to link (or not ) the tangents
     */
    lockedTangent?: boolean;
}
/**
 * Enum for the animation key frame interpolation type
 */
export enum AnimationKeyInterpolation {
    /**
     * Use tangents to interpolate between start and end values.
     */
    NONE = 0,
    /**
     * Do not interpolate between keys and use the start key value only. Tangents are ignored
     */
    STEP = 1
}

}
declare module "babylonjs/Animations/animationPropertiesOverride" {
/**
 * Class used to override all child animations of a given target
 */
export class AnimationPropertiesOverride {
    /**
     * Gets or sets a value indicating if animation blending must be used
     */
    enableBlending: boolean;
    /**
     * Gets or sets the blending speed to use when enableBlending is true
     */
    blendingSpeed: number;
    /**
     * Gets or sets the default loop mode to use
     */
    loopMode: number;
}

}
declare module "babylonjs/Animations/animationRange" {
/**
 * Represents the range of an animation
 */
export class AnimationRange {
    /**The name of the animation range**/
    name: string;
    /**The starting frame of the animation */
    from: number;
    /**The ending frame of the animation*/
    to: number;
    /**
     * Initializes the range of an animation
     * @param name The name of the animation range
     * @param from The starting frame of the animation
     * @param to The ending frame of the animation
     */
    constructor(
    /**The name of the animation range**/
    name: string, 
    /**The starting frame of the animation */
    from: number, 
    /**The ending frame of the animation*/
    to: number);
    /**
     * Makes a copy of the animation range
     * @returns A copy of the animation range
     */
    clone(): AnimationRange;
}

}
declare module "babylonjs/Animations/easing" {
/**
 * This represents the main contract an easing function should follow.
 * Easing functions are used throughout the animation system.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/advanced_animations#easing-functions
 */
export interface IEasingFunction {
    /**
     * Given an input gradient between 0 and 1, this returns the corresponding value
     * of the easing function.
     * The link below provides some of the most common examples of easing functions.
     * @see https://easings.net/
     * @param gradient Defines the value between 0 and 1 we want the easing value for
     * @returns the corresponding value on the curve defined by the easing function
     */
    ease(gradient: number): number;
}
/**
 * Base class used for every default easing function.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/advanced_animations#easing-functions
 */
export class EasingFunction implements IEasingFunction {
    /**
     * Interpolation follows the mathematical formula associated with the easing function.
     */
    static readonly EASINGMODE_EASEIN: number;
    /**
     * Interpolation follows 100% interpolation minus the output of the formula associated with the easing function.
     */
    static readonly EASINGMODE_EASEOUT: number;
    /**
     * Interpolation uses EaseIn for the first half of the animation and EaseOut for the second half.
     */
    static readonly EASINGMODE_EASEINOUT: number;
    private _easingMode;
    /**
     * Sets the easing mode of the current function.
     * @param easingMode Defines the willing mode (EASINGMODE_EASEIN, EASINGMODE_EASEOUT or EASINGMODE_EASEINOUT)
     */
    setEasingMode(easingMode: number): void;
    /**
     * Gets the current easing mode.
     * @returns the easing mode
     */
    getEasingMode(): number;
    /**
     * @internal
     */
    easeInCore(gradient: number): number;
    /**
     * Given an input gradient between 0 and 1, this returns the corresponding value
     * of the easing function.
     * @param gradient Defines the value between 0 and 1 we want the easing value for
     * @returns the corresponding value on the curve defined by the easing function
     */
    ease(gradient: number): number;
}
/**
 * Easing function with a circle shape (see link below).
 * @see https://easings.net/#easeInCirc
 * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/advanced_animations#easing-functions
 */
export class CircleEase extends EasingFunction implements IEasingFunction {
    /**
     * @internal
     */
    easeInCore(gradient: number): number;
}
/**
 * Easing function with a ease back shape (see link below).
 * @see https://easings.net/#easeInBack
 * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/advanced_animations#easing-functions
 */
export class BackEase extends EasingFunction implements IEasingFunction {
    /** Defines the amplitude of the function */
    amplitude: number;
    /**
     * Instantiates a back ease easing
     * @see https://easings.net/#easeInBack
     * @param amplitude Defines the amplitude of the function
     */
    constructor(
    /** Defines the amplitude of the function */
    amplitude?: number);
    /**
     * @internal
     */
    easeInCore(gradient: number): number;
}
/**
 * Easing function with a bouncing shape (see link below).
 * @see https://easings.net/#easeInBounce
 * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/advanced_animations#easing-functions
 */
export class BounceEase extends EasingFunction implements IEasingFunction {
    /** Defines the number of bounces */
    bounces: number;
    /** Defines the amplitude of the bounce */
    bounciness: number;
    /**
     * Instantiates a bounce easing
     * @see https://easings.net/#easeInBounce
     * @param bounces Defines the number of bounces
     * @param bounciness Defines the amplitude of the bounce
     */
    constructor(
    /** Defines the number of bounces */
    bounces?: number, 
    /** Defines the amplitude of the bounce */
    bounciness?: number);
    /**
     * @internal
     */
    easeInCore(gradient: number): number;
}
/**
 * Easing function with a power of 3 shape (see link below).
 * @see https://easings.net/#easeInCubic
 * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/advanced_animations#easing-functions
 */
export class CubicEase extends EasingFunction implements IEasingFunction {
    /**
     * @internal
     */
    easeInCore(gradient: number): number;
}
/**
 * Easing function with an elastic shape (see link below).
 * @see https://easings.net/#easeInElastic
 * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/advanced_animations#easing-functions
 */
export class ElasticEase extends EasingFunction implements IEasingFunction {
    /** Defines the number of oscillations*/
    oscillations: number;
    /** Defines the amplitude of the oscillations*/
    springiness: number;
    /**
     * Instantiates an elastic easing function
     * @see https://easings.net/#easeInElastic
     * @param oscillations Defines the number of oscillations
     * @param springiness Defines the amplitude of the oscillations
     */
    constructor(
    /** Defines the number of oscillations*/
    oscillations?: number, 
    /** Defines the amplitude of the oscillations*/
    springiness?: number);
    /**
     * @internal
     */
    easeInCore(gradient: number): number;
}
/**
 * Easing function with an exponential shape (see link below).
 * @see https://easings.net/#easeInExpo
 * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/advanced_animations#easing-functions
 */
export class ExponentialEase extends EasingFunction implements IEasingFunction {
    /** Defines the exponent of the function */
    exponent: number;
    /**
     * Instantiates an exponential easing function
     * @see https://easings.net/#easeInExpo
     * @param exponent Defines the exponent of the function
     */
    constructor(
    /** Defines the exponent of the function */
    exponent?: number);
    /**
     * @internal
     */
    easeInCore(gradient: number): number;
}
/**
 * Easing function with a power shape (see link below).
 * @see https://easings.net/#easeInQuad
 * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/advanced_animations#easing-functions
 */
export class PowerEase extends EasingFunction implements IEasingFunction {
    /** Defines the power of the function */
    power: number;
    /**
     * Instantiates an power base easing function
     * @see https://easings.net/#easeInQuad
     * @param power Defines the power of the function
     */
    constructor(
    /** Defines the power of the function */
    power?: number);
    /**
     * @internal
     */
    easeInCore(gradient: number): number;
}
/**
 * Easing function with a power of 2 shape (see link below).
 * @see https://easings.net/#easeInQuad
 * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/advanced_animations#easing-functions
 */
export class QuadraticEase extends EasingFunction implements IEasingFunction {
    /**
     * @internal
     */
    easeInCore(gradient: number): number;
}
/**
 * Easing function with a power of 4 shape (see link below).
 * @see https://easings.net/#easeInQuart
 * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/advanced_animations#easing-functions
 */
export class QuarticEase extends EasingFunction implements IEasingFunction {
    /**
     * @internal
     */
    easeInCore(gradient: number): number;
}
/**
 * Easing function with a power of 5 shape (see link below).
 * @see https://easings.net/#easeInQuint
 * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/advanced_animations#easing-functions
 */
export class QuinticEase extends EasingFunction implements IEasingFunction {
    /**
     * @internal
     */
    easeInCore(gradient: number): number;
}
/**
 * Easing function with a sin shape (see link below).
 * @see https://easings.net/#easeInSine
 * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/advanced_animations#easing-functions
 */
export class SineEase extends EasingFunction implements IEasingFunction {
    /**
     * @internal
     */
    easeInCore(gradient: number): number;
}
/**
 * Easing function with a bezier shape (see link below).
 * @see http://cubic-bezier.com/#.17,.67,.83,.67
 * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/advanced_animations#easing-functions
 */
export class BezierCurveEase extends EasingFunction implements IEasingFunction {
    /** Defines the x component of the start tangent in the bezier curve */
    x1: number;
    /** Defines the y component of the start tangent in the bezier curve */
    y1: number;
    /** Defines the x component of the end tangent in the bezier curve */
    x2: number;
    /** Defines the y component of the end tangent in the bezier curve */
    y2: number;
    /**
     * Instantiates a bezier function
     * @see http://cubic-bezier.com/#.17,.67,.83,.67
     * @param x1 Defines the x component of the start tangent in the bezier curve
     * @param y1 Defines the y component of the start tangent in the bezier curve
     * @param x2 Defines the x component of the end tangent in the bezier curve
     * @param y2 Defines the y component of the end tangent in the bezier curve
     */
    constructor(
    /** Defines the x component of the start tangent in the bezier curve */
    x1?: number, 
    /** Defines the y component of the start tangent in the bezier curve */
    y1?: number, 
    /** Defines the x component of the end tangent in the bezier curve */
    x2?: number, 
    /** Defines the y component of the end tangent in the bezier curve */
    y2?: number);
    /**
     * @internal
     */
    easeInCore(gradient: number): number;
}

}
declare module "babylonjs/Animations/index" {
export * from "babylonjs/Animations/animatable";
export * from "babylonjs/Animations/animation";
export * from "babylonjs/Animations/animationPropertiesOverride";
export * from "babylonjs/Animations/easing";
export * from "babylonjs/Animations/runtimeAnimation";
export * from "babylonjs/Animations/animationEvent";
export * from "babylonjs/Animations/animationGroup";
export * from "babylonjs/Animations/animationKey";
export * from "babylonjs/Animations/animationRange";
export * from "babylonjs/Animations/animatable.interface";
export * from "babylonjs/Animations/pathCursor";

}
declare module "babylonjs/Animations/pathCursor" {
import { Vector3 } from "babylonjs/Maths/math.vector";
import { Path2 } from "babylonjs/Maths/math.path";
/**
 * A cursor which tracks a point on a path
 */
export class PathCursor {
    private _path;
    /**
     * Stores path cursor callbacks for when an onchange event is triggered
     */
    private _onchange;
    /**
     * The value of the path cursor
     */
    value: number;
    /**
     * The animation array of the path cursor
     */
    animations: Animation[];
    /**
     * Initializes the path cursor
     * @param _path The path to track
     */
    constructor(_path: Path2);
    /**
     * Gets the cursor point on the path
     * @returns A point on the path cursor at the cursor location
     */
    getPoint(): Vector3;
    /**
     * Moves the cursor ahead by the step amount
     * @param step The amount to move the cursor forward
     * @returns This path cursor
     */
    moveAhead(step?: number): PathCursor;
    /**
     * Moves the cursor behind by the step amount
     * @param step The amount to move the cursor back
     * @returns This path cursor
     */
    moveBack(step?: number): PathCursor;
    /**
     * Moves the cursor by the step amount
     * If the step amount is greater than one, an exception is thrown
     * @param step The amount to move the cursor
     * @returns This path cursor
     */
    move(step: number): PathCursor;
    /**
     * Ensures that the value is limited between zero and one
     * @returns This path cursor
     */
    private _ensureLimits;
    /**
     * Runs onchange callbacks on change (used by the animation engine)
     * @returns This path cursor
     */
    private _raiseOnChange;
    /**
     * Executes a function on change
     * @param f A path cursor onchange callback
     * @returns This path cursor
     */
    onchange(f: (cursor: PathCursor) => void): PathCursor;
}

}
declare module "babylonjs/Animations/runtimeAnimation" {
import { _IAnimationState } from "babylonjs/Animations/animation";
import { Animation } from "babylonjs/Animations/animation";
import { Animatable } from "babylonjs/Animations/animatable";
import { Scene } from "babylonjs/scene";
/**
 * Defines a runtime animation
 */
export class RuntimeAnimation {
    private _events;
    /**
     * The current frame of the runtime animation
     */
    private _currentFrame;
    /**
     * The animation used by the runtime animation
     */
    private _animation;
    /**
     * The target of the runtime animation
     */
    private _target;
    /**
     * The initiating animatable
     */
    private _host;
    /**
     * The original value of the runtime animation
     */
    private _originalValue;
    /**
     * The original blend value of the runtime animation
     */
    private _originalBlendValue;
    /**
     * The offsets cache of the runtime animation
     */
    private _offsetsCache;
    /**
     * The high limits cache of the runtime animation
     */
    private _highLimitsCache;
    /**
     * Specifies if the runtime animation has been stopped
     */
    private _stopped;
    /**
     * The blending factor of the runtime animation
     */
    private _blendingFactor;
    /**
     * The BabylonJS scene
     */
    private _scene;
    /**
     * The current value of the runtime animation
     */
    private _currentValue;
    /** @internal */
    _animationState: _IAnimationState;
    /**
     * The active target of the runtime animation
     */
    private _activeTargets;
    private _currentActiveTarget;
    private _directTarget;
    /**
     * The target path of the runtime animation
     */
    private _targetPath;
    /**
     * The weight of the runtime animation
     */
    private _weight;
    /**
     * The ratio offset of the runtime animation
     */
    private _ratioOffset;
    /**
     * The previous delay of the runtime animation
     */
    private _previousDelay;
    /**
     * The previous ratio of the runtime animation
     */
    private _previousRatio;
    private _enableBlending;
    private _keys;
    private _minFrame;
    private _maxFrame;
    private _minValue;
    private _maxValue;
    private _targetIsArray;
    /**
     * Gets the current frame of the runtime animation
     */
    get currentFrame(): number;
    /**
     * Gets the weight of the runtime animation
     */
    get weight(): number;
    /**
     * Gets the current value of the runtime animation
     */
    get currentValue(): any;
    /**
     * Gets or sets the target path of the runtime animation
     */
    get targetPath(): string;
    /**
     * Gets the actual target of the runtime animation
     */
    get target(): any;
    /**
     * Gets the additive state of the runtime animation
     */
    get isAdditive(): boolean;
    /** @internal */
    _onLoop: () => void;
    /**
     * Create a new RuntimeAnimation object
     * @param target defines the target of the animation
     * @param animation defines the source animation object
     * @param scene defines the hosting scene
     * @param host defines the initiating Animatable
     */
    constructor(target: any, animation: Animation, scene: Scene, host: Animatable);
    private _preparePath;
    /**
     * Gets the animation from the runtime animation
     */
    get animation(): Animation;
    /**
     * Resets the runtime animation to the beginning
     * @param restoreOriginal defines whether to restore the target property to the original value
     */
    reset(restoreOriginal?: boolean): void;
    /**
     * Specifies if the runtime animation is stopped
     * @returns Boolean specifying if the runtime animation is stopped
     */
    isStopped(): boolean;
    /**
     * Disposes of the runtime animation
     */
    dispose(): void;
    /**
     * Apply the interpolated value to the target
     * @param currentValue defines the value computed by the animation
     * @param weight defines the weight to apply to this value (Defaults to 1.0)
     */
    setValue(currentValue: any, weight: number): void;
    private _getOriginalValues;
    private _setValue;
    /**
     * Gets the loop pmode of the runtime animation
     * @returns Loop Mode
     */
    private _getCorrectLoopMode;
    /**
     * Move the current animation to a given frame
     * @param frame defines the frame to move to
     */
    goToFrame(frame: number): void;
    /**
     * @internal Internal use only
     */
    _prepareForSpeedRatioChange(newSpeedRatio: number): void;
    /**
     * Execute the current animation
     * @param delay defines the delay to add to the current frame
     * @param from defines the lower bound of the animation range
     * @param to defines the upper bound of the animation range
     * @param loop defines if the current animation must loop
     * @param speedRatio defines the current speed ratio
     * @param weight defines the weight of the animation (default is -1 so no weight)
     * @returns a boolean indicating if the animation is running
     */
    animate(delay: number, from: number, to: number, loop: boolean, speedRatio: number, weight?: number): boolean;
}
export {};

}
declare module "babylonjs/assetContainer" {
import { AbstractScene } from "babylonjs/abstractScene";
import { Scene } from "babylonjs/scene";
import { Mesh } from "babylonjs/Meshes/mesh";
import { TransformNode } from "babylonjs/Meshes/transformNode";
import { Skeleton } from "babylonjs/Bones/skeleton";
import { AnimationGroup } from "babylonjs/Animations/animationGroup";
import { Animatable } from "babylonjs/Animations/animatable";
import { Nullable } from "babylonjs/types";
import { Node } from "babylonjs/node";
/**
 * Set of assets to keep when moving a scene into an asset container.
 */
export class KeepAssets extends AbstractScene {
}
/**
 * Class used to store the output of the AssetContainer.instantiateAllMeshesToScene function
 */
export class InstantiatedEntries {
    /**
     * List of new root nodes (eg. nodes with no parent)
     */
    rootNodes: TransformNode[];
    /**
     * List of new skeletons
     */
    skeletons: Skeleton[];
    /**
     * List of new animation groups
     */
    animationGroups: AnimationGroup[];
    /**
     * Disposes the instantiated entries from the scene
     */
    dispose(): void;
}
/**
 * Container with a set of assets that can be added or removed from a scene.
 */
export class AssetContainer extends AbstractScene {
    private _wasAddedToScene;
    private _onContextRestoredObserver;
    /**
     * The scene the AssetContainer belongs to.
     */
    scene: Scene;
    /**
     * Instantiates an AssetContainer.
     * @param scene The scene the AssetContainer belongs to.
     */
    constructor(scene?: Nullable<Scene>);
    /**
     * Given a list of nodes, return a topological sorting of them.
     * @param nodes
     */
    private _topologicalSort;
    private _addNodeAndDescendantsToList;
    /**
     * Check if a specific node is contained in this asset container.
     * @param node
     */
    private _isNodeInContainer;
    /**
     * For every node in the scene, check if its parent node is also in the scene.
     */
    private _isValidHierarchy;
    /**
     * Instantiate or clone all meshes and add the new ones to the scene.
     * Skeletons and animation groups will all be cloned
     * @param nameFunction defines an optional function used to get new names for clones
     * @param cloneMaterials defines an optional boolean that defines if materials must be cloned as well (false by default)
     * @param options defines an optional list of options to control how to instantiate / clone models
     * @param options.doNotInstantiate defines if the model must be instantiated or just cloned
     * @param options.predicate defines a predicate used to filter whih mesh to instantiate/clone
     * @returns a list of rootNodes, skeletons and animation groups that were duplicated
     */
    instantiateModelsToScene(nameFunction?: (sourceName: string) => string, cloneMaterials?: boolean, options?: {
        doNotInstantiate?: boolean | ((node: TransformNode) => boolean);
        predicate?: (entity: any) => boolean;
    }): InstantiatedEntries;
    /**
     * Adds all the assets from the container to the scene.
     */
    addAllToScene(): void;
    /**
     * Adds assets from the container to the scene.
     * @param predicate defines a predicate used to select which entity will be added (can be null)
     */
    addToScene(predicate?: Nullable<(entity: any) => boolean>): void;
    /**
     * Removes all the assets in the container from the scene
     */
    removeAllFromScene(): void;
    /**
     * Removes assets in the container from the scene
     * @param predicate defines a predicate used to select which entity will be added (can be null)
     */
    removeFromScene(predicate?: Nullable<(entity: any) => boolean>): void;
    /**
     * Disposes all the assets in the container
     */
    dispose(): void;
    private _moveAssets;
    /**
     * Removes all the assets contained in the scene and adds them to the container.
     * @param keepAssets Set of assets to keep in the scene. (default: empty)
     */
    moveAllFromScene(keepAssets?: KeepAssets): void;
    /**
     * Adds all meshes in the asset container to a root mesh that can be used to position all the contained meshes. The root mesh is then added to the front of the meshes in the assetContainer.
     * @returns the root mesh
     */
    createRootMesh(): Mesh;
    /**
     * Merge animations (direct and animation groups) from this asset container into a scene
     * @param scene is the instance of BABYLON.Scene to append to (default: last created scene)
     * @param animatables set of animatables to retarget to a node from the scene
     * @param targetConverter defines a function used to convert animation targets from the asset container to the scene (default: search node by name)
     * @returns an array of the new AnimationGroup added to the scene (empty array if none)
     */
    mergeAnimationsTo(scene: Nullable<Scene> | undefined, animatables: Animatable[], targetConverter?: Nullable<(target: any) => Nullable<Node>>): AnimationGroup[];
}

}
declare module "babylonjs/Audio/analyser" {
import { Nullable } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
/**
 * Class used to work with sound analyzer using fast fourier transform (FFT)
 * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic
 */
export class Analyser {
    /**
     * Gets or sets the smoothing
     * @ignorenaming
     */
    SMOOTHING: number;
    /**
     * Gets or sets the FFT table size
     * @ignorenaming
     */
    FFT_SIZE: number;
    /**
     * Gets or sets the bar graph amplitude
     * @ignorenaming
     */
    BARGRAPHAMPLITUDE: number;
    /**
     * Gets or sets the position of the debug canvas
     * @ignorenaming
     */
    DEBUGCANVASPOS: {
        x: number;
        y: number;
    };
    /**
     * Gets or sets the debug canvas size
     * @ignorenaming
     */
    DEBUGCANVASSIZE: {
        width: number;
        height: number;
    };
    private _byteFreqs;
    private _byteTime;
    private _floatFreqs;
    private _webAudioAnalyser;
    private _debugCanvas;
    private _debugCanvasContext;
    private _scene;
    private _registerFunc;
    private _audioEngine;
    /**
     * Creates a new analyser
     * @param scene defines hosting scene
     */
    constructor(scene?: Nullable<Scene>);
    /**
     * Get the number of data values you will have to play with for the visualization
     * @see https://developer.mozilla.org/en-US/docs/Web/API/AnalyserNode/frequencyBinCount
     * @returns a number
     */
    getFrequencyBinCount(): number;
    /**
     * Gets the current frequency data as a byte array
     * @see https://developer.mozilla.org/en-US/docs/Web/API/AnalyserNode/getByteFrequencyData
     * @returns a Uint8Array
     */
    getByteFrequencyData(): Uint8Array;
    /**
     * Gets the current waveform as a byte array
     * @see https://developer.mozilla.org/en-US/docs/Web/API/AnalyserNode/getByteTimeDomainData
     * @returns a Uint8Array
     */
    getByteTimeDomainData(): Uint8Array;
    /**
     * Gets the current frequency data as a float array
     * @see https://developer.mozilla.org/en-US/docs/Web/API/AnalyserNode/getByteFrequencyData
     * @returns a Float32Array
     */
    getFloatFrequencyData(): Float32Array;
    /**
     * Renders the debug canvas
     */
    drawDebugCanvas(): void;
    /**
     * Stops rendering the debug canvas and removes it
     */
    stopDebugCanvas(): void;
    /**
     * Connects two audio nodes
     * @param inputAudioNode defines first node to connect
     * @param outputAudioNode defines second node to connect
     */
    connectAudioNodes(inputAudioNode: AudioNode, outputAudioNode: AudioNode): void;
    /**
     * Releases all associated resources
     */
    dispose(): void;
}

}
declare module "babylonjs/Audio/audioEngine" {
import { Analyser } from "babylonjs/Audio/analyser";
import { Nullable } from "babylonjs/types";
import { Observable } from "babylonjs/Misc/observable";
import { IAudioEngine } from "babylonjs/Audio/Interfaces/IAudioEngine";
/**
 * This represents the default audio engine used in babylon.
 * It is responsible to play, synchronize and analyse sounds throughout the  application.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic
 */
export class AudioEngine implements IAudioEngine {
    private _audioContext;
    private _audioContextInitialized;
    private _muteButton;
    private _hostElement;
    private _audioDestination;
    /**
     * Gets whether the current host supports Web Audio and thus could create AudioContexts.
     */
    canUseWebAudio: boolean;
    /**
     * The master gain node defines the global audio volume of your audio engine.
     */
    masterGain: GainNode;
    /**
     * Defines if Babylon should emit a warning if WebAudio is not supported.
     * @ignoreNaming
     */
    WarnedWebAudioUnsupported: boolean;
    /**
     * Gets whether or not mp3 are supported by your browser.
     */
    isMP3supported: boolean;
    /**
     * Gets whether or not ogg are supported by your browser.
     */
    isOGGsupported: boolean;
    /**
     * Gets whether audio has been unlocked on the device.
     * Some Browsers have strong restrictions about Audio and won t autoplay unless
     * a user interaction has happened.
     */
    unlocked: boolean;
    /**
     * Defines if the audio engine relies on a custom unlocked button.
     * In this case, the embedded button will not be displayed.
     */
    useCustomUnlockedButton: boolean;
    /**
     * Event raised when audio has been unlocked on the browser.
     */
    onAudioUnlockedObservable: Observable<IAudioEngine>;
    /**
     * Event raised when audio has been locked on the browser.
     */
    onAudioLockedObservable: Observable<IAudioEngine>;
    /**
     * Gets the current AudioContext if available.
     */
    get audioContext(): Nullable<AudioContext>;
    private _connectedAnalyser;
    /**
     * Instantiates a new audio engine.
     *
     * There should be only one per page as some browsers restrict the number
     * of audio contexts you can create.
     * @param hostElement defines the host element where to display the mute icon if necessary
     * @param audioContext defines the audio context to be used by the audio engine
     * @param audioDestination defines the audio destination node to be used by audio engine
     */
    constructor(hostElement?: Nullable<HTMLElement>, audioContext?: Nullable<AudioContext>, audioDestination?: Nullable<AudioDestinationNode | MediaStreamAudioDestinationNode>);
    /**
     * Flags the audio engine in Locked state.
     * This happens due to new browser policies preventing audio to autoplay.
     */
    lock(): void;
    /**
     * Unlocks the audio engine once a user action has been done on the dom.
     * This is helpful to resume play once browser policies have been satisfied.
     */
    unlock(): void;
    private _resumeAudioContext;
    private _initializeAudioContext;
    private _tryToRun;
    private _triggerRunningState;
    private _triggerSuspendedState;
    private _displayMuteButton;
    private _moveButtonToTopLeft;
    private _onResize;
    private _hideMuteButton;
    /**
     * Destroy and release the resources associated with the audio context.
     */
    dispose(): void;
    /**
     * Gets the global volume sets on the master gain.
     * @returns the global volume if set or -1 otherwise
     */
    getGlobalVolume(): number;
    /**
     * Sets the global volume of your experience (sets on the master gain).
     * @param newVolume Defines the new global volume of the application
     */
    setGlobalVolume(newVolume: number): void;
    /**
     * Connect the audio engine to an audio analyser allowing some amazing
     * synchronization between the sounds/music and your visualization (VuMeter for instance).
     * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic#using-the-analyser
     * @param analyser The analyser to connect to the engine
     */
    connectToAnalyser(analyser: Analyser): void;
}

}
declare module "babylonjs/Audio/audioSceneComponent" {
import { Sound } from "babylonjs/Audio/sound";
import { SoundTrack } from "babylonjs/Audio/soundTrack";
import { Nullable } from "babylonjs/types";
import { Vector3 } from "babylonjs/Maths/math.vector";
import { ISceneSerializableComponent } from "babylonjs/sceneComponent";
import { Scene } from "babylonjs/scene";
import { AbstractScene } from "babylonjs/abstractScene";
import "babylonjs/Audio/audioEngine";
module "babylonjs/abstractScene" {
    interface AbstractScene {
        /**
         * The list of sounds used in the scene.
         */
        sounds: Nullable<Array<Sound>>;
    }
}
module "babylonjs/scene" {
    interface Scene {
        /**
         * @internal
         * Backing field
         */
        _mainSoundTrack: SoundTrack;
        /**
         * The main sound track played by the scene.
         * It contains your primary collection of sounds.
         */
        mainSoundTrack: SoundTrack;
        /**
         * The list of sound tracks added to the scene
         * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic
         */
        soundTracks: Nullable<Array<SoundTrack>>;
        /**
         * Gets a sound using a given name
         * @param name defines the name to search for
         * @returns the found sound or null if not found at all.
         */
        getSoundByName(name: string): Nullable<Sound>;
        /**
         * Gets or sets if audio support is enabled
         * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic
         */
        audioEnabled: boolean;
        /**
         * Gets or sets if audio will be output to headphones
         * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic
         */
        headphone: boolean;
        /**
         * Gets or sets custom audio listener position provider
         * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic
         */
        audioListenerPositionProvider: Nullable<() => Vector3>;
        /**
         * Gets or sets custom audio listener rotation provider
         * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic
         */
        audioListenerRotationProvider: Nullable<() => Vector3>;
        /**
         * Gets or sets a refresh rate when using 3D audio positioning
         */
        audioPositioningRefreshRate: number;
    }
}
/**
 * Defines the sound scene component responsible to manage any sounds
 * in a given scene.
 */
export class AudioSceneComponent implements ISceneSerializableComponent {
    private static _CameraDirection;
    /**
     * The component name helpful to identify the component in the list of scene components.
     */
    readonly name: string;
    /**
     * The scene the component belongs to.
     */
    scene: Scene;
    private _audioEnabled;
    /**
     * Gets whether audio is enabled or not.
     * Please use related enable/disable method to switch state.
     */
    get audioEnabled(): boolean;
    private _headphone;
    /**
     * Gets whether audio is outputting to headphone or not.
     * Please use the according Switch methods to change output.
     */
    get headphone(): boolean;
    /**
     * Gets or sets a refresh rate when using 3D audio positioning
     */
    audioPositioningRefreshRate: number;
    /**
     * Gets or Sets a custom listener position for all sounds in the scene
     * By default, this is the position of the first active camera
     */
    audioListenerPositionProvider: Nullable<() => Vector3>;
    /**
     * Gets or Sets a custom listener rotation for all sounds in the scene
     * By default, this is the rotation of the first active camera
     */
    audioListenerRotationProvider: Nullable<() => Vector3>;
    /**
     * Creates a new instance of the component for the given scene
     * @param scene Defines the scene to register the component in
     */
    constructor(scene?: Nullable<Scene>);
    /**
     * Registers the component in a given scene
     */
    register(): void;
    /**
     * Rebuilds the elements related to this component in case of
     * context lost for instance.
     */
    rebuild(): void;
    /**
     * Serializes the component data to the specified json object
     * @param serializationObject The object to serialize to
     */
    serialize(serializationObject: any): void;
    /**
     * Adds all the elements from the container to the scene
     * @param container the container holding the elements
     */
    addFromContainer(container: AbstractScene): void;
    /**
     * Removes all the elements in the container from the scene
     * @param container contains the elements to remove
     * @param dispose if the removed element should be disposed (default: false)
     */
    removeFromContainer(container: AbstractScene, dispose?: boolean): void;
    /**
     * Disposes the component and the associated resources.
     */
    dispose(): void;
    /**
     * Disables audio in the associated scene.
     */
    disableAudio(): void;
    /**
     * Enables audio in the associated scene.
     */
    enableAudio(): void;
    /**
     * Switch audio to headphone output.
     */
    switchAudioModeForHeadphones(): void;
    /**
     * Switch audio to normal speakers.
     */
    switchAudioModeForNormalSpeakers(): void;
    private _cachedCameraDirection;
    private _cachedCameraPosition;
    private _lastCheck;
    private _invertMatrixTemp;
    private _cameraDirectionTemp;
    private _afterRender;
}

}
declare module "babylonjs/Audio/index" {
export * from "babylonjs/Audio/Interfaces/IAudioEngine";
export * from "babylonjs/Audio/Interfaces/ISoundOptions";
export * from "babylonjs/Audio/analyser";
export * from "babylonjs/Audio/audioEngine";
export * from "babylonjs/Audio/audioSceneComponent";
export * from "babylonjs/Audio/sound";
export * from "babylonjs/Audio/soundTrack";
export * from "babylonjs/Audio/weightedsound";

}
declare module "babylonjs/Audio/Interfaces/IAudioEngine" {
import { Observable } from "babylonjs/Misc/observable";
import { IDisposable } from "babylonjs/scene";
import { Nullable } from "babylonjs/types";
import { Analyser } from "babylonjs/Audio/analyser";
/**
 * This represents an audio engine and it is responsible
 * to play, synchronize and analyse sounds throughout the application.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic
 */
export interface IAudioEngine extends IDisposable {
    /**
     * Gets whether the current host supports Web Audio and thus could create AudioContexts.
     */
    readonly canUseWebAudio: boolean;
    /**
     * Gets the current AudioContext if available.
     */
    readonly audioContext: Nullable<AudioContext>;
    /**
     * The master gain node defines the global audio volume of your audio engine.
     */
    readonly masterGain: GainNode;
    /**
     * Gets whether or not mp3 are supported by your browser.
     */
    readonly isMP3supported: boolean;
    /**
     * Gets whether or not ogg are supported by your browser.
     */
    readonly isOGGsupported: boolean;
    /**
     * Defines if Babylon should emit a warning if WebAudio is not supported.
     * @ignoreNaming
     */
    WarnedWebAudioUnsupported: boolean;
    /**
     * Defines if the audio engine relies on a custom unlocked button.
     * In this case, the embedded button will not be displayed.
     */
    useCustomUnlockedButton: boolean;
    /**
     * Gets whether or not the audio engine is unlocked (require first a user gesture on some browser).
     */
    readonly unlocked: boolean;
    /**
     * Event raised when audio has been unlocked on the browser.
     */
    onAudioUnlockedObservable: Observable<IAudioEngine>;
    /**
     * Event raised when audio has been locked on the browser.
     */
    onAudioLockedObservable: Observable<IAudioEngine>;
    /**
     * Flags the audio engine in Locked state.
     * This happens due to new browser policies preventing audio to autoplay.
     */
    lock(): void;
    /**
     * Unlocks the audio engine once a user action has been done on the dom.
     * This is helpful to resume play once browser policies have been satisfied.
     */
    unlock(): void;
    /**
     * Gets the global volume sets on the master gain.
     * @returns the global volume if set or -1 otherwise
     */
    getGlobalVolume(): number;
    /**
     * Sets the global volume of your experience (sets on the master gain).
     * @param newVolume Defines the new global volume of the application
     */
    setGlobalVolume(newVolume: number): void;
    /**
     * Connect the audio engine to an audio analyser allowing some amazing
     * synchronization between the sounds/music and your visualization (VuMeter for instance).
     * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic#using-the-analyser
     * @param analyser The analyser to connect to the engine
     */
    connectToAnalyser(analyser: Analyser): void;
}

}
declare module "babylonjs/Audio/Interfaces/IAudioEngineOptions" {
/**
 * Interface used to define options for the Audio Engine
 * @since 5.0.0
 */
export interface IAudioEngineOptions {
    /**
     * Specifies an existing Audio Context for the audio engine
     */
    audioContext?: AudioContext;
    /**
     * Specifies a destination node for the audio engine
     */
    audioDestination?: AudioDestinationNode | MediaStreamAudioDestinationNode;
}

}
declare module "babylonjs/Audio/Interfaces/ISoundOptions" {
/**
 * Interface used to define options for Sound class
 */
export interface ISoundOptions {
    /**
     * Does the sound autoplay once loaded.
     */
    autoplay?: boolean;
    /**
     * Does the sound loop after it finishes playing once.
     */
    loop?: boolean;
    /**
     * Sound's volume
     */
    volume?: number;
    /**
     * Is it a spatial sound?
     */
    spatialSound?: boolean;
    /**
     * Maximum distance to hear that sound
     */
    maxDistance?: number;
    /**
     * Uses user defined attenuation function
     */
    useCustomAttenuation?: boolean;
    /**
     * Define the roll off factor of spatial sounds.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic#creating-a-spatial-3d-sound
     */
    rolloffFactor?: number;
    /**
     * Define the reference distance the sound should be heard perfectly.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic#creating-a-spatial-3d-sound
     */
    refDistance?: number;
    /**
     * Define the distance attenuation model the sound will follow.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic#creating-a-spatial-3d-sound
     */
    distanceModel?: string;
    /**
     * Defines the playback speed (1 by default)
     */
    playbackRate?: number;
    /**
     * Defines if the sound is from a streaming source
     */
    streaming?: boolean;
    /**
     * Defines an optional length (in seconds) inside the sound file
     */
    length?: number;
    /**
     * Defines an optional offset (in seconds) inside the sound file
     */
    offset?: number;
    /**
     * If true, URLs will not be required to state the audio file codec to use.
     */
    skipCodecCheck?: boolean;
}

}
declare module "babylonjs/Audio/sound" {
import { Observable } from "babylonjs/Misc/observable";
import { Vector3 } from "babylonjs/Maths/math.vector";
import { Nullable } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
import { TransformNode } from "babylonjs/Meshes/transformNode";
import { ISoundOptions } from "babylonjs/Audio/Interfaces/ISoundOptions";
/**
 * Defines a sound that can be played in the application.
 * The sound can either be an ambient track or a simple sound played in reaction to a user action.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic
 */
export class Sound {
    /**
     * The name of the sound in the scene.
     */
    name: string;
    /**
     * Does the sound autoplay once loaded.
     */
    autoplay: boolean;
    private _loop;
    /**
     * Does the sound loop after it finishes playing once.
     */
    get loop(): boolean;
    set loop(value: boolean);
    /**
     * Does the sound use a custom attenuation curve to simulate the falloff
     * happening when the source gets further away from the camera.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic#creating-your-own-custom-attenuation-function
     */
    useCustomAttenuation: boolean;
    /**
     * The sound track id this sound belongs to.
     */
    soundTrackId: number;
    /**
     * Is this sound currently played.
     */
    isPlaying: boolean;
    /**
     * Is this sound currently paused.
     */
    isPaused: boolean;
    /**
     * Define the reference distance the sound should be heard perfectly.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic#creating-a-spatial-3d-sound
     */
    refDistance: number;
    /**
     * Define the roll off factor of spatial sounds.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic#creating-a-spatial-3d-sound
     */
    rolloffFactor: number;
    /**
     * Define the max distance the sound should be heard (intensity just became 0 at this point).
     * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic#creating-a-spatial-3d-sound
     */
    maxDistance: number;
    /**
     * Define the distance attenuation model the sound will follow.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic#creating-a-spatial-3d-sound
     */
    distanceModel: string;
    /**
     * @internal
     * Back Compat
     **/
    onended: () => any;
    /**
     * Gets or sets an object used to store user defined information for the sound.
     */
    metadata: any;
    /**
     * Observable event when the current playing sound finishes.
     */
    onEndedObservable: Observable<Sound>;
    /**
     * Gets the current time for the sound.
     */
    get currentTime(): number;
    /**
     * Does this sound enables spatial sound.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic#creating-a-spatial-3d-sound
     */
    get spatialSound(): boolean;
    /**
     * Does this sound enables spatial sound.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic#creating-a-spatial-3d-sound
     */
    set spatialSound(newValue: boolean);
    private _spatialSound;
    private _panningModel;
    private _playbackRate;
    private _streaming;
    private _startTime;
    private _currentTime;
    private _position;
    private _localDirection;
    private _volume;
    private _isReadyToPlay;
    private _isDirectional;
    private _readyToPlayCallback;
    private _audioBuffer;
    private _soundSource;
    private _streamingSource;
    private _soundPanner;
    private _soundGain;
    private _inputAudioNode;
    private _outputAudioNode;
    private _coneInnerAngle;
    private _coneOuterAngle;
    private _coneOuterGain;
    private _scene;
    private _connectedTransformNode;
    private _customAttenuationFunction;
    private _registerFunc;
    private _isOutputConnected;
    private _htmlAudioElement;
    private _urlType;
    private _length?;
    private _offset?;
    /**
     * @internal
     */
    static _SceneComponentInitialization: (scene: Scene) => void;
    /**
     * Create a sound and attach it to a scene
     * @param name Name of your sound
     * @param urlOrArrayBuffer Url to the sound to load async or ArrayBuffer, it also works with MediaStreams and AudioBuffers
     * @param scene defines the scene the sound belongs to
     * @param readyToPlayCallback Provide a callback function if you'd like to load your code once the sound is ready to be played
     * @param options Objects to provide with the current available options: autoplay, loop, volume, spatialSound, maxDistance, rolloffFactor, refDistance, distanceModel, panningModel, streaming
     */
    constructor(name: string, urlOrArrayBuffer: any, scene?: Nullable<Scene>, readyToPlayCallback?: Nullable<() => void>, options?: ISoundOptions);
    /**
     * Release the sound and its associated resources
     */
    dispose(): void;
    /**
     * Gets if the sounds is ready to be played or not.
     * @returns true if ready, otherwise false
     */
    isReady(): boolean;
    /**
     * Get the current class name.
     * @returns current class name
     */
    getClassName(): string;
    private _audioBufferLoaded;
    private _soundLoaded;
    /**
     * Sets the data of the sound from an audiobuffer
     * @param audioBuffer The audioBuffer containing the data
     */
    setAudioBuffer(audioBuffer: AudioBuffer): void;
    /**
     * Updates the current sounds options such as maxdistance, loop...
     * @param options A JSON object containing values named as the object properties
     */
    updateOptions(options: ISoundOptions): void;
    private _createSpatialParameters;
    private _updateSpatialParameters;
    /**
     * Switch the panning model to HRTF:
     * Renders a stereo output of higher quality than equalpower — it uses a convolution with measured impulse responses from human subjects.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic#creating-a-spatial-3d-sound
     */
    switchPanningModelToHRTF(): void;
    /**
     * Switch the panning model to Equal Power:
     * Represents the equal-power panning algorithm, generally regarded as simple and efficient. equalpower is the default value.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic#creating-a-spatial-3d-sound
     */
    switchPanningModelToEqualPower(): void;
    private _switchPanningModel;
    /**
     * Connect this sound to a sound track audio node like gain...
     * @param soundTrackAudioNode the sound track audio node to connect to
     */
    connectToSoundTrackAudioNode(soundTrackAudioNode: AudioNode): void;
    /**
     * Transform this sound into a directional source
     * @param coneInnerAngle Size of the inner cone in degree
     * @param coneOuterAngle Size of the outer cone in degree
     * @param coneOuterGain Volume of the sound outside the outer cone (between 0.0 and 1.0)
     */
    setDirectionalCone(coneInnerAngle: number, coneOuterAngle: number, coneOuterGain: number): void;
    /**
     * Gets or sets the inner angle for the directional cone.
     */
    get directionalConeInnerAngle(): number;
    /**
     * Gets or sets the inner angle for the directional cone.
     */
    set directionalConeInnerAngle(value: number);
    /**
     * Gets or sets the outer angle for the directional cone.
     */
    get directionalConeOuterAngle(): number;
    /**
     * Gets or sets the outer angle for the directional cone.
     */
    set directionalConeOuterAngle(value: number);
    /**
     * Sets the position of the emitter if spatial sound is enabled
     * @param newPosition Defines the new position
     */
    setPosition(newPosition: Vector3): void;
    /**
     * Sets the local direction of the emitter if spatial sound is enabled
     * @param newLocalDirection Defines the new local direction
     */
    setLocalDirectionToMesh(newLocalDirection: Vector3): void;
    private _updateDirection;
    /** @internal */
    updateDistanceFromListener(): void;
    /**
     * Sets a new custom attenuation function for the sound.
     * @param callback Defines the function used for the attenuation
     * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic#creating-your-own-custom-attenuation-function
     */
    setAttenuationFunction(callback: (currentVolume: number, currentDistance: number, maxDistance: number, refDistance: number, rolloffFactor: number) => number): void;
    /**
     * Play the sound
     * @param time (optional) Start the sound after X seconds. Start immediately (0) by default.
     * @param offset (optional) Start the sound at a specific time in seconds
     * @param length (optional) Sound duration (in seconds)
     */
    play(time?: number, offset?: number, length?: number): void;
    private _onended;
    /**
     * Stop the sound
     * @param time (optional) Stop the sound after X seconds. Stop immediately (0) by default.
     */
    stop(time?: number): void;
    /**
     * Put the sound in pause
     */
    pause(): void;
    /**
     * Sets a dedicated volume for this sounds
     * @param newVolume Define the new volume of the sound
     * @param time Define time for gradual change to new volume
     */
    setVolume(newVolume: number, time?: number): void;
    /**
     * Set the sound play back rate
     * @param newPlaybackRate Define the playback rate the sound should be played at
     */
    setPlaybackRate(newPlaybackRate: number): void;
    /**
     * Gets the sound play back rate.
     * @returns the  play back rate of the sound
     */
    getPlaybackRate(): number;
    /**
     * Gets the volume of the sound.
     * @returns the volume of the sound
     */
    getVolume(): number;
    /**
     * Attach the sound to a dedicated mesh
     * @param transformNode The transform node to connect the sound with
     * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic#attaching-a-sound-to-a-mesh
     */
    attachToMesh(transformNode: TransformNode): void;
    /**
     * Detach the sound from the previously attached mesh
     * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic#attaching-a-sound-to-a-mesh
     */
    detachFromMesh(): void;
    private _onRegisterAfterWorldMatrixUpdate;
    /**
     * Clone the current sound in the scene.
     * @returns the new sound clone
     */
    clone(): Nullable<Sound>;
    /**
     * Gets the current underlying audio buffer containing the data
     * @returns the audio buffer
     */
    getAudioBuffer(): Nullable<AudioBuffer>;
    /**
     * Gets the WebAudio AudioBufferSourceNode, lets you keep track of and stop instances of this Sound.
     * @returns the source node
     */
    getSoundSource(): Nullable<AudioBufferSourceNode>;
    /**
     * Gets the WebAudio GainNode, gives you precise control over the gain of instances of this Sound.
     * @returns the gain node
     */
    getSoundGain(): Nullable<GainNode>;
    /**
     * Serializes the Sound in a JSON representation
     * @returns the JSON representation of the sound
     */
    serialize(): any;
    /**
     * Parse a JSON representation of a sound to instantiate in a given scene
     * @param parsedSound Define the JSON representation of the sound (usually coming from the serialize method)
     * @param scene Define the scene the new parsed sound should be created in
     * @param rootUrl Define the rooturl of the load in case we need to fetch relative dependencies
     * @param sourceSound Define a sound place holder if do not need to instantiate a new one
     * @returns the newly parsed sound
     */
    static Parse(parsedSound: any, scene: Scene, rootUrl: string, sourceSound?: Sound): Sound;
    private _setOffset;
}

}
declare module "babylonjs/Audio/soundTrack" {
import { Sound } from "babylonjs/Audio/sound";
import { Analyser } from "babylonjs/Audio/analyser";
import { Nullable } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
/**
 * Options allowed during the creation of a sound track.
 */
export interface ISoundTrackOptions {
    /**
     * The volume the sound track should take during creation
     */
    volume?: number;
    /**
     * Define if the sound track is the main sound track of the scene
     */
    mainTrack?: boolean;
}
/**
 * It could be useful to isolate your music & sounds on several tracks to better manage volume on a grouped instance of sounds.
 * It will be also used in a future release to apply effects on a specific track.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic#using-sound-tracks
 */
export class SoundTrack {
    /**
     * The unique identifier of the sound track in the scene.
     */
    id: number;
    /**
     * The list of sounds included in the sound track.
     */
    soundCollection: Array<Sound>;
    private _outputAudioNode;
    private _scene;
    private _connectedAnalyser;
    private _options;
    private _isInitialized;
    /**
     * Creates a new sound track.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic#using-sound-tracks
     * @param scene Define the scene the sound track belongs to
     * @param options
     */
    constructor(scene?: Nullable<Scene>, options?: ISoundTrackOptions);
    private _initializeSoundTrackAudioGraph;
    /**
     * Release the sound track and its associated resources
     */
    dispose(): void;
    /**
     * Adds a sound to this sound track
     * @param sound define the sound to add
     * @ignoreNaming
     */
    addSound(sound: Sound): void;
    /**
     * Removes a sound to this sound track
     * @param sound define the sound to remove
     * @ignoreNaming
     */
    removeSound(sound: Sound): void;
    /**
     * Set a global volume for the full sound track.
     * @param newVolume Define the new volume of the sound track
     */
    setVolume(newVolume: number): void;
    /**
     * Switch the panning model to HRTF:
     * Renders a stereo output of higher quality than equalpower — it uses a convolution with measured impulse responses from human subjects.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic#creating-a-spatial-3d-sound
     */
    switchPanningModelToHRTF(): void;
    /**
     * Switch the panning model to Equal Power:
     * Represents the equal-power panning algorithm, generally regarded as simple and efficient. equalpower is the default value.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic#creating-a-spatial-3d-sound
     */
    switchPanningModelToEqualPower(): void;
    /**
     * Connect the sound track to an audio analyser allowing some amazing
     * synchronization between the sounds/music and your visualization (VuMeter for instance).
     * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic#using-the-analyser
     * @param analyser The analyser to connect to the engine
     */
    connectToAnalyser(analyser: Analyser): void;
}

}
declare module "babylonjs/Audio/weightedsound" {
import { Sound } from "babylonjs/Audio/sound";
/**
 * Wraps one or more Sound objects and selects one with random weight for playback.
 */
export class WeightedSound {
    /** When true a Sound will be selected and played when the current playing Sound completes. */
    loop: boolean;
    private _coneInnerAngle;
    private _coneOuterAngle;
    private _volume;
    /** A Sound is currently playing. */
    isPlaying: boolean;
    /** A Sound is currently paused. */
    isPaused: boolean;
    private _sounds;
    private _weights;
    private _currentIndex?;
    /**
     * Creates a new WeightedSound from the list of sounds given.
     * @param loop When true a Sound will be selected and played when the current playing Sound completes.
     * @param sounds Array of Sounds that will be selected from.
     * @param weights Array of number values for selection weights; length must equal sounds, values will be normalized to 1
     */
    constructor(loop: boolean, sounds: Sound[], weights: number[]);
    /**
     * The size of cone in degrees for a directional sound in which there will be no attenuation.
     */
    get directionalConeInnerAngle(): number;
    /**
     * The size of cone in degrees for a directional sound in which there will be no attenuation.
     */
    set directionalConeInnerAngle(value: number);
    /**
     * Size of cone in degrees for a directional sound outside of which there will be no sound.
     * Listener angles between innerAngle and outerAngle will falloff linearly.
     */
    get directionalConeOuterAngle(): number;
    /**
     * Size of cone in degrees for a directional sound outside of which there will be no sound.
     * Listener angles between innerAngle and outerAngle will falloff linearly.
     */
    set directionalConeOuterAngle(value: number);
    /**
     * Playback volume.
     */
    get volume(): number;
    /**
     * Playback volume.
     */
    set volume(value: number);
    private _onended;
    /**
     * Suspend playback
     */
    pause(): void;
    /**
     * Stop playback
     */
    stop(): void;
    /**
     * Start playback.
     * @param startOffset Position the clip head at a specific time in seconds.
     */
    play(startOffset?: number): void;
}

}
declare module "babylonjs/BakedVertexAnimation/bakedVertexAnimationManager" {
import { Nullable } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
import { BaseTexture } from "babylonjs/Materials/Textures/baseTexture";
import { Vector4 } from "babylonjs/Maths/math.vector";
import { Effect } from "babylonjs/Materials/effect";
/**
 * Interface for baked vertex animation texture, see BakedVertexAnimationManager
 * @since 5.0
 */
export interface IBakedVertexAnimationManager {
    /**
     * The vertex animation texture
     */
    texture: Nullable<BaseTexture>;
    /**
     * Gets or sets a boolean indicating if the edgesRenderer is active
     */
    isEnabled: boolean;
    /**
     * The animation parameters for the mesh. See setAnimationParameters()
     */
    animationParameters: Vector4;
    /**
     * The time counter, to pick the correct animation frame.
     */
    time: number;
    /**
     * Binds to the effect.
     * @param effect The effect to bind to.
     * @param useInstances True when it's an instance.
     */
    bind(effect: Effect, useInstances: boolean): void;
    /**
     * Sets animation parameters.
     * @param startFrame The first frame of the animation.
     * @param endFrame The last frame of the animation.
     * @param offset The offset when starting the animation.
     * @param speedFramesPerSecond The frame rate.
     */
    setAnimationParameters(startFrame: number, endFrame: number, offset: number, speedFramesPerSecond: number): void;
    /**
     * Disposes the resources of the manager.
     * @param forceDisposeTextures - Forces the disposal of all textures.
     */
    dispose(forceDisposeTextures?: boolean): void;
    /**
     * Get the current class name useful for serialization or dynamic coding.
     * @returns "BakedVertexAnimationManager"
     */
    getClassName(): string;
}
/**
 * This class is used to animate meshes using a baked vertex animation texture
 * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/baked_texture_animations
 * @since 5.0
 */
export class BakedVertexAnimationManager implements IBakedVertexAnimationManager {
    private _scene;
    private _texture;
    /**
     * The vertex animation texture
     */
    texture: Nullable<BaseTexture>;
    private _isEnabled;
    /**
     * Enable or disable the vertex animation manager
     */
    isEnabled: boolean;
    /**
     * The animation parameters for the mesh. See setAnimationParameters()
     */
    animationParameters: Vector4;
    /**
     * The time counter, to pick the correct animation frame.
     */
    time: number;
    /**
     * Creates a new BakedVertexAnimationManager
     * @param scene defines the current scene
     */
    constructor(scene?: Nullable<Scene>);
    /** @internal */
    _markSubMeshesAsAttributesDirty(): void;
    /**
     * Binds to the effect.
     * @param effect The effect to bind to.
     * @param useInstances True when it's an instance.
     */
    bind(effect: Effect, useInstances?: boolean): void;
    /**
     * Clone the current manager
     * @returns a new BakedVertexAnimationManager
     */
    clone(): BakedVertexAnimationManager;
    /**
     * Sets animation parameters.
     * @param startFrame The first frame of the animation.
     * @param endFrame The last frame of the animation.
     * @param offset The offset when starting the animation.
     * @param speedFramesPerSecond The frame rate.
     */
    setAnimationParameters(startFrame: number, endFrame: number, offset?: number, speedFramesPerSecond?: number): void;
    /**
     * Disposes the resources of the manager.
     * @param forceDisposeTextures - Forces the disposal of all textures.
     */
    dispose(forceDisposeTextures?: boolean): void;
    /**
     * Get the current class name useful for serialization or dynamic coding.
     * @returns "BakedVertexAnimationManager"
     */
    getClassName(): string;
    /**
     * Makes a duplicate of the current instance into another one.
     * @param vatMap define the instance where to copy the info
     */
    copyTo(vatMap: BakedVertexAnimationManager): void;
    /**
     * Serializes this vertex animation instance
     * @returns - An object with the serialized instance.
     */
    serialize(): any;
    /**
     * Parses a vertex animation setting from a serialized object.
     * @param source - Serialized object.
     * @param scene Defines the scene we are parsing for
     * @param rootUrl Defines the rootUrl to load from
     */
    parse(source: any, scene: Scene, rootUrl: string): void;
}

}
declare module "babylonjs/BakedVertexAnimation/index" {
export * from "babylonjs/BakedVertexAnimation/bakedVertexAnimationManager";
export * from "babylonjs/BakedVertexAnimation/vertexAnimationBaker";

}
declare module "babylonjs/BakedVertexAnimation/vertexAnimationBaker" {
import { AnimationRange } from "babylonjs/Animations/animationRange";
import { RawTexture } from "babylonjs/Materials/Textures/rawTexture";
import { Mesh } from "babylonjs/Meshes/mesh";
import { Scene } from "babylonjs/scene";
/**
 * Class to bake vertex animation textures.
 * @since 5.0
 */
export class VertexAnimationBaker {
    private _scene;
    private _mesh;
    /**
     * Create a new VertexAnimationBaker object which can help baking animations into a texture.
     * @param scene Defines the scene the VAT belongs to
     * @param mesh Defines the mesh the VAT belongs to
     */
    constructor(scene: Scene, mesh: Mesh);
    /**
     * Bakes the animation into the texture. This should be called once, when the
     * scene starts, so the VAT is generated and associated to the mesh.
     * @param ranges Defines the ranges in the animation that will be baked.
     * @returns The array of matrix transforms for each vertex (columns) and frame (rows), as a Float32Array.
     */
    bakeVertexData(ranges: AnimationRange[]): Promise<Float32Array>;
    /**
     * Runs an animation frame and stores its vertex data
     *
     * @param vertexData The array to save data to.
     * @param frameIndex Current frame in the skeleton animation to render.
     * @param textureIndex Current index of the texture data.
     */
    private _executeAnimationFrame;
    /**
     * Builds a vertex animation texture given the vertexData in an array.
     * @param vertexData The vertex animation data. You can generate it with bakeVertexData().
     * @returns The vertex animation texture to be used with BakedVertexAnimationManager.
     */
    textureFromBakedVertexData(vertexData: Float32Array): RawTexture;
    /**
     * Serializes our vertexData to an object, with a nice string for the vertexData.
     * @param vertexData The vertex array data.
     * @returns This object serialized to a JS dict.
     */
    serializeBakedVertexDataToObject(vertexData: Float32Array): Record<string, any>;
    /**
     * Loads previously baked data.
     * @param data The object as serialized by serializeBakedVertexDataToObject()
     * @returns The array of matrix transforms for each vertex (columns) and frame (rows), as a Float32Array.
     */
    loadBakedVertexDataFromObject(data: Record<string, any>): Float32Array;
    /**
     * Serializes our vertexData to a JSON string, with a nice string for the vertexData.
     * Should be called right after bakeVertexData().
     * @param vertexData The vertex array data.
     * @returns This object serialized to a safe string.
     */
    serializeBakedVertexDataToJSON(vertexData: Float32Array): string;
    /**
     * Loads previously baked data in string format.
     * @param json The json string as serialized by serializeBakedVertexDataToJSON().
     * @returns The array of matrix transforms for each vertex (columns) and frame (rows), as a Float32Array.
     */
    loadBakedVertexDataFromJSON(json: string): Float32Array;
}

}
declare module "babylonjs/Behaviors/behavior" {
import { Nullable } from "babylonjs/types";
/**
 * Interface used to define a behavior
 */
export interface Behavior<T> {
    /** gets or sets behavior's name */
    name: string;
    /**
     * Function called when the behavior needs to be initialized (after attaching it to a target)
     */
    init(): void;
    /**
     * Called when the behavior is attached to a target
     * @param target defines the target where the behavior is attached to
     */
    attach(target: T): void;
    /**
     * Called when the behavior is detached from its target
     */
    detach(): void;
}
/**
 * Interface implemented by classes supporting behaviors
 */
export interface IBehaviorAware<T> {
    /**
     * Attach a behavior
     * @param behavior defines the behavior to attach
     * @returns the current host
     */
    addBehavior(behavior: Behavior<T>): T;
    /**
     * Remove a behavior from the current object
     * @param behavior defines the behavior to detach
     * @returns the current host
     */
    removeBehavior(behavior: Behavior<T>): T;
    /**
     * Gets a behavior using its name to search
     * @param name defines the name to search
     * @returns the behavior or null if not found
     */
    getBehaviorByName(name: string): Nullable<Behavior<T>>;
}

}
declare module "babylonjs/Behaviors/Cameras/autoRotationBehavior" {
import { Behavior } from "babylonjs/Behaviors/behavior";
import { ArcRotateCamera } from "babylonjs/Cameras/arcRotateCamera";
import { Nullable } from "babylonjs/types";
/**
 * The autoRotation behavior (AutoRotationBehavior) is designed to create a smooth rotation of an ArcRotateCamera when there is no user interaction.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/behaviors/cameraBehaviors#autorotation-behavior
 */
export class AutoRotationBehavior implements Behavior<ArcRotateCamera> {
    /**
     * Gets the name of the behavior.
     */
    get name(): string;
    private _zoomStopsAnimation;
    private _idleRotationSpeed;
    private _idleRotationWaitTime;
    private _idleRotationSpinupTime;
    targetAlpha: Nullable<number>;
    /**
     * Sets the flag that indicates if user zooming should stop animation.
     */
    set zoomStopsAnimation(flag: boolean);
    /**
     * Gets the flag that indicates if user zooming should stop animation.
     */
    get zoomStopsAnimation(): boolean;
    /**
     * Sets the default speed at which the camera rotates around the model.
     */
    set idleRotationSpeed(speed: number);
    /**
     * Gets the default speed at which the camera rotates around the model.
     */
    get idleRotationSpeed(): number;
    /**
     * Sets the time (in milliseconds) to wait after user interaction before the camera starts rotating.
     */
    set idleRotationWaitTime(time: number);
    /**
     * Gets the time (milliseconds) to wait after user interaction before the camera starts rotating.
     */
    get idleRotationWaitTime(): number;
    /**
     * Sets the time (milliseconds) to take to spin up to the full idle rotation speed.
     */
    set idleRotationSpinupTime(time: number);
    /**
     * Gets the time (milliseconds) to take to spin up to the full idle rotation speed.
     */
    get idleRotationSpinupTime(): number;
    /**
     * Gets a value indicating if the camera is currently rotating because of this behavior
     */
    get rotationInProgress(): boolean;
    private _onPrePointerObservableObserver;
    private _onAfterCheckInputsObserver;
    private _attachedCamera;
    private _isPointerDown;
    private _lastFrameTime;
    private _lastInteractionTime;
    private _cameraRotationSpeed;
    /**
     * Initializes the behavior.
     */
    init(): void;
    /**
     * Attaches the behavior to its arc rotate camera.
     * @param camera Defines the camera to attach the behavior to
     */
    attach(camera: ArcRotateCamera): void;
    /**
     * Detaches the behavior from its current arc rotate camera.
     */
    detach(): void;
    /**
     * Force-reset the last interaction time
     * @param customTime an optional time that will be used instead of the current last interaction time. For example `Date.now()`
     */
    resetLastInteractionTime(customTime?: number): void;
    /**
     * Returns true if camera alpha reaches the target alpha
     * @returns true if camera alpha reaches the target alpha
     */
    private _reachTargetAlpha;
    /**
     * Returns true if user is scrolling.
     * @returns true if user is scrolling.
     */
    private _userIsZooming;
    private _lastFrameRadius;
    private _shouldAnimationStopForInteraction;
    /**
     *  Applies any current user interaction to the camera. Takes into account maximum alpha rotation.
     */
    private _applyUserInteraction;
    private _userIsMoving;
}

}
declare module "babylonjs/Behaviors/Cameras/bouncingBehavior" {
import { Behavior } from "babylonjs/Behaviors/behavior";
import { ArcRotateCamera } from "babylonjs/Cameras/arcRotateCamera";
import { BackEase } from "babylonjs/Animations/easing";
/**
 * Add a bouncing effect to an ArcRotateCamera when reaching a specified minimum and maximum radius
 * @see https://doc.babylonjs.com/features/featuresDeepDive/behaviors/cameraBehaviors#bouncing-behavior
 */
export class BouncingBehavior implements Behavior<ArcRotateCamera> {
    /**
     * Gets the name of the behavior.
     */
    get name(): string;
    /**
     * The easing function used by animations
     */
    static EasingFunction: BackEase;
    /**
     * The easing mode used by animations
     */
    static EasingMode: number;
    /**
     * The duration of the animation, in milliseconds
     */
    transitionDuration: number;
    /**
     * Length of the distance animated by the transition when lower radius is reached
     */
    lowerRadiusTransitionRange: number;
    /**
     * Length of the distance animated by the transition when upper radius is reached
     */
    upperRadiusTransitionRange: number;
    private _autoTransitionRange;
    /**
     * Gets a value indicating if the lowerRadiusTransitionRange and upperRadiusTransitionRange are defined automatically
     */
    get autoTransitionRange(): boolean;
    /**
     * Sets a value indicating if the lowerRadiusTransitionRange and upperRadiusTransitionRange are defined automatically
     * Transition ranges will be set to 5% of the bounding box diagonal in world space
     */
    set autoTransitionRange(value: boolean);
    private _attachedCamera;
    private _onAfterCheckInputsObserver;
    private _onMeshTargetChangedObserver;
    /**
     * Initializes the behavior.
     */
    init(): void;
    /**
     * Attaches the behavior to its arc rotate camera.
     * @param camera Defines the camera to attach the behavior to
     */
    attach(camera: ArcRotateCamera): void;
    /**
     * Detaches the behavior from its current arc rotate camera.
     */
    detach(): void;
    private _radiusIsAnimating;
    private _radiusBounceTransition;
    private _animatables;
    private _cachedWheelPrecision;
    /**
     * Checks if the camera radius is at the specified limit. Takes into account animation locks.
     * @param radiusLimit The limit to check against.
     * @returns Bool to indicate if at limit.
     */
    private _isRadiusAtLimit;
    /**
     * Applies an animation to the radius of the camera, extending by the radiusDelta.
     * @param radiusDelta The delta by which to animate to. Can be negative.
     */
    private _applyBoundRadiusAnimation;
    /**
     * Removes all animation locks. Allows new animations to be added to any of the camera properties.
     */
    protected _clearAnimationLocks(): void;
    /**
     * Stops and removes all animations that have been applied to the camera
     */
    stopAllAnimations(): void;
}

}
declare module "babylonjs/Behaviors/Cameras/framingBehavior" {
import { Behavior } from "babylonjs/Behaviors/behavior";
import { ArcRotateCamera } from "babylonjs/Cameras/arcRotateCamera";
import { ExponentialEase } from "babylonjs/Animations/easing";
import { Observable } from "babylonjs/Misc/observable";
import { Nullable } from "babylonjs/types";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { Vector3 } from "babylonjs/Maths/math.vector";
/**
 * The framing behavior (FramingBehavior) is designed to automatically position an ArcRotateCamera when its target is set to a mesh. It is also useful if you want to prevent the camera to go under a virtual horizontal plane.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/behaviors/cameraBehaviors#framing-behavior
 */
export class FramingBehavior implements Behavior<ArcRotateCamera> {
    /**
     * Gets the name of the behavior.
     */
    get name(): string;
    /**
     * An event triggered when the animation to zoom on target mesh has ended
     */
    onTargetFramingAnimationEndObservable: Observable<void>;
    private _mode;
    private _radiusScale;
    private _positionScale;
    private _defaultElevation;
    private _elevationReturnTime;
    private _elevationReturnWaitTime;
    private _zoomStopsAnimation;
    private _framingTime;
    /**
     * The easing function used by animations
     */
    static EasingFunction: ExponentialEase;
    /**
     * The easing mode used by animations
     */
    static EasingMode: number;
    /**
     * Sets the current mode used by the behavior
     */
    set mode(mode: number);
    /**
     * Gets current mode used by the behavior.
     */
    get mode(): number;
    /**
     * Sets the scale applied to the radius (1 by default)
     */
    set radiusScale(radius: number);
    /**
     * Gets the scale applied to the radius
     */
    get radiusScale(): number;
    /**
     * Sets the scale to apply on Y axis to position camera focus. 0.5 by default which means the center of the bounding box.
     */
    set positionScale(scale: number);
    /**
     * Gets the scale to apply on Y axis to position camera focus. 0.5 by default which means the center of the bounding box.
     */
    get positionScale(): number;
    /**
     * Sets the angle above/below the horizontal plane to return to when the return to default elevation idle
     * behaviour is triggered, in radians.
     */
    set defaultElevation(elevation: number);
    /**
     * Gets the angle above/below the horizontal plane to return to when the return to default elevation idle
     * behaviour is triggered, in radians.
     */
    get defaultElevation(): number;
    /**
     * Sets the time (in milliseconds) taken to return to the default beta position.
     * Negative value indicates camera should not return to default.
     */
    set elevationReturnTime(speed: number);
    /**
     * Gets the time (in milliseconds) taken to return to the default beta position.
     * Negative value indicates camera should not return to default.
     */
    get elevationReturnTime(): number;
    /**
     * Sets the delay (in milliseconds) taken before the camera returns to the default beta position.
     */
    set elevationReturnWaitTime(time: number);
    /**
     * Gets the delay (in milliseconds) taken before the camera returns to the default beta position.
     */
    get elevationReturnWaitTime(): number;
    /**
     * Sets the flag that indicates if user zooming should stop animation.
     */
    set zoomStopsAnimation(flag: boolean);
    /**
     * Gets the flag that indicates if user zooming should stop animation.
     */
    get zoomStopsAnimation(): boolean;
    /**
     * Sets the transition time when framing the mesh, in milliseconds
     */
    set framingTime(time: number);
    /**
     * Gets the transition time when framing the mesh, in milliseconds
     */
    get framingTime(): number;
    /**
     * Define if the behavior should automatically change the configured
     * camera limits and sensibilities.
     */
    autoCorrectCameraLimitsAndSensibility: boolean;
    private _onPrePointerObservableObserver;
    private _onAfterCheckInputsObserver;
    private _onMeshTargetChangedObserver;
    private _attachedCamera;
    private _isPointerDown;
    private _lastInteractionTime;
    /**
     * Initializes the behavior.
     */
    init(): void;
    /**
     * Attaches the behavior to its arc rotate camera.
     * @param camera Defines the camera to attach the behavior to
     */
    attach(camera: ArcRotateCamera): void;
    /**
     * Detaches the behavior from its current arc rotate camera.
     */
    detach(): void;
    private _animatables;
    private _betaIsAnimating;
    private _betaTransition;
    private _radiusTransition;
    private _vectorTransition;
    /**
     * Targets the given mesh and updates zoom level accordingly.
     * @param mesh  The mesh to target.
     * @param focusOnOriginXZ Determines if the camera should focus on 0 in the X and Z axis instead of the mesh
     * @param onAnimationEnd Callback triggered at the end of the framing animation
     */
    zoomOnMesh(mesh: AbstractMesh, focusOnOriginXZ?: boolean, onAnimationEnd?: Nullable<() => void>): void;
    /**
     * Targets the given mesh with its children and updates zoom level accordingly.
     * @param mesh  The mesh to target.
     * @param focusOnOriginXZ Determines if the camera should focus on 0 in the X and Z axis instead of the mesh
     * @param onAnimationEnd Callback triggered at the end of the framing animation
     */
    zoomOnMeshHierarchy(mesh: AbstractMesh, focusOnOriginXZ?: boolean, onAnimationEnd?: Nullable<() => void>): void;
    /**
     * Targets the given meshes with their children and updates zoom level accordingly.
     * @param meshes  The mesh to target.
     * @param focusOnOriginXZ Determines if the camera should focus on 0 in the X and Z axis instead of the mesh
     * @param onAnimationEnd Callback triggered at the end of the framing animation
     */
    zoomOnMeshesHierarchy(meshes: AbstractMesh[], focusOnOriginXZ?: boolean, onAnimationEnd?: Nullable<() => void>): void;
    /**
     * Targets the bounding box info defined by its extends and updates zoom level accordingly.
     * @param minimumWorld Determines the smaller position of the bounding box extend
     * @param maximumWorld Determines the bigger position of the bounding box extend
     * @param focusOnOriginXZ Determines if the camera should focus on 0 in the X and Z axis instead of the mesh
     * @param onAnimationEnd Callback triggered at the end of the framing animation
     */
    zoomOnBoundingInfo(minimumWorld: Vector3, maximumWorld: Vector3, focusOnOriginXZ?: boolean, onAnimationEnd?: Nullable<() => void>): void;
    /**
     * Calculates the lowest radius for the camera based on the bounding box of the mesh.
     * @param minimumWorld
     * @param maximumWorld
     * @returns The minimum distance from the primary mesh's center point at which the camera must be kept in order
     *		 to fully enclose the mesh in the viewing frustum.
     */
    protected _calculateLowerRadiusFromModelBoundingSphere(minimumWorld: Vector3, maximumWorld: Vector3): number;
    /**
     * Keeps the camera above the ground plane. If the user pulls the camera below the ground plane, the camera
     * is automatically returned to its default position (expected to be above ground plane).
     */
    private _maintainCameraAboveGround;
    /**
     * Returns the frustum slope based on the canvas ratio and camera FOV
     * @returns The frustum slope represented as a Vector2 with X and Y slopes
     */
    private _getFrustumSlope;
    /**
     * Removes all animation locks. Allows new animations to be added to any of the arcCamera properties.
     */
    private _clearAnimationLocks;
    /**
     *  Applies any current user interaction to the camera. Takes into account maximum alpha rotation.
     */
    private _applyUserInteraction;
    /**
     * Stops and removes all animations that have been applied to the camera
     */
    stopAllAnimations(): void;
    /**
     * Gets a value indicating if the user is moving the camera
     */
    get isUserIsMoving(): boolean;
    /**
     * The camera can move all the way towards the mesh.
     */
    static IgnoreBoundsSizeMode: number;
    /**
     * The camera is not allowed to zoom closer to the mesh than the point at which the adjusted bounding sphere touches the frustum sides
     */
    static FitFrustumSidesMode: number;
}

}
declare module "babylonjs/Behaviors/Cameras/index" {
export * from "babylonjs/Behaviors/Cameras/autoRotationBehavior";
export * from "babylonjs/Behaviors/Cameras/bouncingBehavior";
export * from "babylonjs/Behaviors/Cameras/framingBehavior";

}
declare module "babylonjs/Behaviors/index" {
export * from "babylonjs/Behaviors/behavior";
export * from "babylonjs/Behaviors/Cameras/index";
export * from "babylonjs/Behaviors/Meshes/index";

}
declare module "babylonjs/Behaviors/Meshes/attachToBoxBehavior" {
import { Mesh } from "babylonjs/Meshes/mesh";
import { TransformNode } from "babylonjs/Meshes/transformNode";
import { Behavior } from "babylonjs/Behaviors/behavior";
/**
 * A behavior that when attached to a mesh will will place a specified node on the meshes face pointing towards the camera
 */
export class AttachToBoxBehavior implements Behavior<Mesh> {
    private _ui;
    /**
     *  The name of the behavior
     */
    name: string;
    /**
     * The distance away from the face of the mesh that the UI should be attached to (default: 0.15)
     */
    distanceAwayFromFace: number;
    /**
     * The distance from the bottom of the face that the UI should be attached to (default: 0.15)
     */
    distanceAwayFromBottomOfFace: number;
    private _faceVectors;
    private _target;
    private _scene;
    private _onRenderObserver;
    private _tmpMatrix;
    private _tmpVector;
    /**
     * Creates the AttachToBoxBehavior, used to attach UI to the closest face of the box to a camera
     * @param _ui The transform node that should be attached to the mesh
     */
    constructor(_ui: TransformNode);
    /**
     *  Initializes the behavior
     */
    init(): void;
    private _closestFace;
    private _zeroVector;
    private _lookAtTmpMatrix;
    private _lookAtToRef;
    /**
     * Attaches the AttachToBoxBehavior to the passed in mesh
     * @param target The mesh that the specified node will be attached to
     */
    attach(target: Mesh): void;
    /**
     *  Detaches the behavior from the mesh
     */
    detach(): void;
}

}
declare module "babylonjs/Behaviors/Meshes/baseSixDofDragBehavior" {
import { Behavior } from "babylonjs/Behaviors/behavior";
import { Mesh } from "babylonjs/Meshes/mesh";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { Scene } from "babylonjs/scene";
import { Nullable } from "babylonjs/types";
import { Vector3, Quaternion } from "babylonjs/Maths/math.vector";
import { Observable } from "babylonjs/Misc/observable";
import { TransformNode } from "babylonjs/Meshes/transformNode";
import { PickingInfo } from "babylonjs/Collisions/pickingInfo";
/**
 * Data store to track virtual pointers movement
 */
type VirtualMeshInfo = {
    dragging: boolean;
    moving: boolean;
    dragMesh: AbstractMesh;
    originMesh: AbstractMesh;
    pivotMesh: AbstractMesh;
    startingPivotPosition: Vector3;
    startingPivotOrientation: Quaternion;
    startingPosition: Vector3;
    startingOrientation: Quaternion;
    lastOriginPosition: Vector3;
    lastDragPosition: Vector3;
};
/**
 * Base behavior for six degrees of freedom interactions in XR experiences.
 * Creates virtual meshes that are dragged around
 * And observables for position/rotation changes
 */
export class BaseSixDofDragBehavior implements Behavior<Mesh> {
    protected static _virtualScene: Scene;
    private _pointerObserver;
    private _attachedToElement;
    protected _virtualMeshesInfo: {
        [id: number]: VirtualMeshInfo;
    };
    private _tmpVector;
    private _tmpQuaternion;
    protected _dragType: {
        NONE: number;
        DRAG: number;
        DRAG_WITH_CONTROLLER: number;
        NEAR_DRAG: number;
    };
    protected _scene: Scene;
    protected _moving: boolean;
    protected _ownerNode: TransformNode;
    protected _dragging: number;
    /**
     * The list of child meshes that can receive drag events
     * If `null`, all child meshes will receive drag event
     */
    draggableMeshes: Nullable<AbstractMesh[]>;
    /**
     * How much faster the object should move when the controller is moving towards it. This is useful to bring objects that are far away from the user to them faster. Set this to 0 to avoid any speed increase. (Default: 3)
     */
    zDragFactor: number;
    /**
     * The id of the pointer that is currently interacting with the behavior (-1 when no pointer is active)
     */
    get currentDraggingPointerId(): number;
    set currentDraggingPointerId(value: number);
    /**
     * In case of multipointer interaction, all pointer ids currently active are stored here
     */
    currentDraggingPointerIds: number[];
    /**
     * Get or set the currentDraggingPointerId
     * @deprecated Please use currentDraggingPointerId instead
     */
    get currentDraggingPointerID(): number;
    set currentDraggingPointerID(currentDraggingPointerID: number);
    /**
    /**
     * If camera controls should be detached during the drag
     */
    detachCameraControls: boolean;
    /**
     * Fires each time a drag starts
     */
    onDragStartObservable: Observable<{
        position: Vector3;
    }>;
    /**
     * Fires each time a drag happens
     */
    onDragObservable: Observable<{
        delta: Vector3;
        position: Vector3;
        pickInfo: PickingInfo;
    }>;
    /**
     *  Fires each time a drag ends (eg. mouse release after drag)
     */
    onDragEndObservable: Observable<{}>;
    /**
     * Should the behavior allow simultaneous pointers to interact with the owner node.
     */
    allowMultiPointer: boolean;
    /**
     *  The name of the behavior
     */
    get name(): string;
    /**
     *  Returns true if the attached mesh is currently moving with this behavior
     */
    get isMoving(): boolean;
    /**
     *  Initializes the behavior
     */
    init(): void;
    /**
     * In the case of multiple active cameras, the cameraToUseForPointers should be used if set instead of active camera
     */
    private get _pointerCamera();
    private _createVirtualMeshInfo;
    protected _resetVirtualMeshesPosition(): void;
    private _pointerUpdate2D;
    private _pointerUpdateXR;
    /**
     * Attaches the scale behavior the passed in mesh
     * @param ownerNode The mesh that will be scaled around once attached
     */
    attach(ownerNode: TransformNode): void;
    private _applyZOffset;
    protected _targetDragStart(worldPosition: Vector3, worldRotation: Quaternion, pointerId: number): void;
    protected _targetDrag(worldDeltaPosition: Vector3, worldDeltaRotation: Quaternion, pointerId: number): void;
    protected _targetDragEnd(pointerId: number): void;
    protected _reattachCameraControls(): void;
    /**
     * Detaches the behavior from the mesh
     */
    detach(): void;
}
export {};

}
declare module "babylonjs/Behaviors/Meshes/fadeInOutBehavior" {
import { Behavior } from "babylonjs/Behaviors/behavior";
import { Mesh } from "babylonjs/Meshes/mesh";
/**
 * A behavior that when attached to a mesh will allow the mesh to fade in and out
 */
export class FadeInOutBehavior implements Behavior<Mesh> {
    /**
     * Time in milliseconds to delay before fading in (Default: 0)
     */
    fadeInDelay: number;
    /**
     * Time in milliseconds to delay before fading out (Default: 0)
     */
    fadeOutDelay: number;
    /**
     * Time in milliseconds for the mesh to fade in (Default: 300)
     */
    fadeInTime: number;
    /**
     * Time in milliseconds for the mesh to fade out (Default: 300)
     */
    fadeOutTime: number;
    /**
     * Time in milliseconds to delay before fading in (Default: 0)
     * Will set both fade in and out delay to the same value
     */
    get delay(): number;
    set delay(value: number);
    private _millisecondsPerFrame;
    private _hovered;
    private _hoverValue;
    private _ownerNode;
    private _onBeforeRenderObserver;
    private _delay;
    private _time;
    /**
     * Instantiates the FadeInOutBehavior
     */
    constructor();
    /**
     *  The name of the behavior
     */
    get name(): string;
    /**
     *  Initializes the behavior
     */
    init(): void;
    /**
     * Attaches the fade behavior on the passed in mesh
     * @param ownerNode The mesh that will be faded in/out once attached
     */
    attach(ownerNode: Mesh): void;
    /**
     *  Detaches the behavior from the mesh
     */
    detach(): void;
    /**
     * Triggers the mesh to begin fading in (or out)
     * @param fadeIn if the object should fade in or out (true to fade in)
     */
    fadeIn(fadeIn?: boolean): void;
    /**
     * Triggers the mesh to begin fading out
     */
    fadeOut(): void;
    private _update;
    private _setAllVisibility;
    private _attachObserver;
    private _detachObserver;
}

}
declare module "babylonjs/Behaviors/Meshes/followBehavior" {
import { Behavior } from "babylonjs/Behaviors/behavior";
import { Nullable } from "babylonjs/types";
import { Camera } from "babylonjs/Cameras/camera";
import { TransformNode } from "babylonjs/Meshes/transformNode";
/**
 * A behavior that when attached to a mesh will follow a camera
 * @since 5.0.0
 */
export class FollowBehavior implements Behavior<TransformNode> {
    private _scene;
    private _tmpQuaternion;
    private _tmpVectors;
    private _tmpMatrix;
    private _tmpInvertView;
    private _tmpForward;
    private _tmpNodeForward;
    private _tmpPosition;
    private _followedCamera;
    private _onBeforeRender;
    private _workingPosition;
    private _workingQuaternion;
    private _lastTick;
    private _recenterNextUpdate;
    /**
     * Attached node of this behavior
     */
    attachedNode: Nullable<TransformNode>;
    /**
     * Set to false if the node should strictly follow the camera without any interpolation time
     */
    interpolatePose: boolean;
    /**
     * Rate of interpolation of position and rotation of the attached node.
     * Higher values will give a slower interpolation.
     */
    lerpTime: number;
    /**
     * If the behavior should ignore the pitch and roll of the camera.
     */
    ignoreCameraPitchAndRoll: boolean;
    /**
     * Pitch offset from camera (relative to Max Distance)
     * Is only effective if `ignoreCameraPitchAndRoll` is set to `true`.
     */
    pitchOffset: number;
    /**
     * The vertical angle from the camera forward axis to the owner will not exceed this value
     */
    maxViewVerticalDegrees: number;
    /**
     * The horizontal angle from the camera forward axis to the owner will not exceed this value
     */
    maxViewHorizontalDegrees: number;
    /**
     * The attached node will not reorient until the angle between its forward vector and the vector to the camera is greater than this value
     */
    orientToCameraDeadzoneDegrees: number;
    /**
     * Option to ignore distance clamping
     */
    ignoreDistanceClamp: boolean;
    /**
     * Option to ignore angle clamping
     */
    ignoreAngleClamp: boolean;
    /**
     * Max vertical distance between the attachedNode and camera
     */
    verticalMaxDistance: number;
    /**
     *  Default distance from eye to attached node, i.e. the sphere radius
     */
    defaultDistance: number;
    /**
     *  Max distance from eye to attached node, i.e. the sphere radius
     */
    maximumDistance: number;
    /**
     *  Min distance from eye to attached node, i.e. the sphere radius
     */
    minimumDistance: number;
    /**
     * Ignore vertical movement and lock the Y position of the object.
     */
    useFixedVerticalOffset: boolean;
    /**
     * Fixed vertical position offset distance.
     */
    fixedVerticalOffset: number;
    /**
     * Enables/disables the behavior
     * @internal
     */
    _enabled: boolean;
    /**
     * The camera that should be followed by this behavior
     */
    get followedCamera(): Nullable<Camera>;
    set followedCamera(camera: Nullable<Camera>);
    /**
     *  The name of the behavior
     */
    get name(): string;
    /**
     *  Initializes the behavior
     */
    init(): void;
    /**
     * Attaches the follow behavior
     * @param ownerNode The mesh that will be following once attached
     * @param followedCamera The camera that should be followed by the node
     */
    attach(ownerNode: TransformNode, followedCamera?: Camera): void;
    /**
     *  Detaches the behavior from the mesh
     */
    detach(): void;
    /**
     * Recenters the attached node in front of the camera on the next update
     */
    recenter(): void;
    private _angleBetweenVectorAndPlane;
    private _length2D;
    private _distanceClamp;
    private _applyVerticalClamp;
    private _toOrientationQuatToRef;
    private _applyPitchOffset;
    private _angularClamp;
    private _orientationClamp;
    private _passedOrientationDeadzone;
    private _updateLeashing;
    private _updateTransformToGoal;
    private _addObservables;
    private _removeObservables;
}

}
declare module "babylonjs/Behaviors/Meshes/handConstraintBehavior" {
import { TransformNode } from "babylonjs/Meshes/transformNode";
import { WebXRFeaturesManager } from "babylonjs/XR/webXRFeaturesManager";
import { WebXRExperienceHelper } from "babylonjs/XR/webXRExperienceHelper";
import { Behavior } from "babylonjs/Behaviors/behavior";
/**
 * Zones around the hand
 */
export enum HandConstraintZone {
    /**
     * Above finger tips
     */
    ABOVE_FINGER_TIPS = 0,
    /**
     * Next to the thumb
     */
    RADIAL_SIDE = 1,
    /**
     * Next to the pinky finger
     */
    ULNAR_SIDE = 2,
    /**
     * Below the wrist
     */
    BELOW_WRIST = 3
}
/**
 * Orientations for the hand zones and for the attached node
 */
export enum HandConstraintOrientation {
    /**
     * Orientation is towards the camera
     */
    LOOK_AT_CAMERA = 0,
    /**
     * Orientation is determined by the rotation of the palm
     */
    HAND_ROTATION = 1
}
/**
 * Orientations for the hand zones and for the attached node
 */
export enum HandConstraintVisibility {
    /**
     * Constraint is always visible
     */
    ALWAYS_VISIBLE = 0,
    /**
     * Constraint is only visible when the palm is up
     */
    PALM_UP = 1,
    /**
     * Constraint is only visible when the user is looking at the constraint.
     * Uses XR Eye Tracking if enabled/available, otherwise uses camera direction
     */
    GAZE_FOCUS = 2,
    /**
     * Constraint is only visible when the palm is up and the user is looking at it
     */
    PALM_AND_GAZE = 3
}
/**
 * Hand constraint behavior that makes the attached `TransformNode` follow hands in XR experiences.
 * @since 5.0.0
 */
export class HandConstraintBehavior implements Behavior<TransformNode> {
    private _scene;
    private _node;
    private _eyeTracking;
    private _handTracking;
    private _sceneRenderObserver;
    private _zoneAxis;
    /**
     * Sets the HandConstraintVisibility level for the hand constraint
     */
    handConstraintVisibility: HandConstraintVisibility;
    /**
     * A number from 0.0 to 1.0, marking how restricted the direction the palm faces is for the attached node to be enabled.
     * A 1 means the palm must be directly facing the user before the node is enabled, a 0 means it is always enabled.
     * Used with HandConstraintVisibility.PALM_UP
     */
    palmUpStrictness: number;
    /**
     * The radius in meters around the center of the hand that the user must gaze inside for the attached node to be enabled and appear.
     * Used with HandConstraintVisibility.GAZE_FOCUS
     */
    gazeProximityRadius: number;
    /**
     * Offset distance from the hand in meters
     */
    targetOffset: number;
    /**
     * Where to place the node regarding the center of the hand.
     */
    targetZone: HandConstraintZone;
    /**
     * Orientation mode of the 4 zones around the hand
     */
    zoneOrientationMode: HandConstraintOrientation;
    /**
     * Orientation mode of the node attached to this behavior
     */
    nodeOrientationMode: HandConstraintOrientation;
    /**
     * Set the hand this behavior should follow. If set to "none", it will follow any visible hand (prioritising the left one).
     */
    handedness: XRHandedness;
    /**
     * Rate of interpolation of position and rotation of the attached node.
     * Higher values will give a slower interpolation.
     */
    lerpTime: number;
    /**
     * Builds a hand constraint behavior
     */
    constructor();
    /** gets or sets behavior's name */
    get name(): string;
    /** Enable the behavior */
    enable(): void;
    /** Disable the behavior */
    disable(): void;
    private _getHandPose;
    /**
     * Initializes the hand constraint behavior
     */
    init(): void;
    /**
     * Attaches the hand constraint to a `TransformNode`
     * @param node defines the node to attach the behavior to
     */
    attach(node: TransformNode): void;
    private _setVisibility;
    /**
     * Detaches the behavior from the `TransformNode`
     */
    detach(): void;
    /**
     * Links the behavior to the XR experience in which to retrieve hand transform information.
     * @param xr xr experience
     */
    linkToXRExperience(xr: WebXRExperienceHelper | WebXRFeaturesManager): void;
}

}
declare module "babylonjs/Behaviors/Meshes/index" {
export * from "babylonjs/Behaviors/Meshes/attachToBoxBehavior";
export * from "babylonjs/Behaviors/Meshes/fadeInOutBehavior";
export * from "babylonjs/Behaviors/Meshes/multiPointerScaleBehavior";
export * from "babylonjs/Behaviors/Meshes/pointerDragBehavior";
export * from "babylonjs/Behaviors/Meshes/sixDofDragBehavior";
export * from "babylonjs/Behaviors/Meshes/surfaceMagnetismBehavior";
export * from "babylonjs/Behaviors/Meshes/baseSixDofDragBehavior";
export * from "babylonjs/Behaviors/Meshes/followBehavior";
export * from "babylonjs/Behaviors/Meshes/handConstraintBehavior";

}
declare module "babylonjs/Behaviors/Meshes/multiPointerScaleBehavior" {
import { Mesh } from "babylonjs/Meshes/mesh";
import { Behavior } from "babylonjs/Behaviors/behavior";
/**
 * A behavior that when attached to a mesh will allow the mesh to be scaled
 */
export class MultiPointerScaleBehavior implements Behavior<Mesh> {
    private _dragBehaviorA;
    private _dragBehaviorB;
    private _startDistance;
    private _initialScale;
    private _targetScale;
    private _ownerNode;
    private _sceneRenderObserver;
    /**
     * Instantiate a new behavior that when attached to a mesh will allow the mesh to be scaled
     */
    constructor();
    /**
     *  The name of the behavior
     */
    get name(): string;
    /**
     *  Initializes the behavior
     */
    init(): void;
    private _getCurrentDistance;
    /**
     * Attaches the scale behavior the passed in mesh
     * @param ownerNode The mesh that will be scaled around once attached
     */
    attach(ownerNode: Mesh): void;
    /**
     *  Detaches the behavior from the mesh
     */
    detach(): void;
}

}
declare module "babylonjs/Behaviors/Meshes/pointerDragBehavior" {
import { Behavior } from "babylonjs/Behaviors/behavior";
import { Mesh } from "babylonjs/Meshes/mesh";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { Nullable } from "babylonjs/types";
import { Observable } from "babylonjs/Misc/observable";
import { Vector3 } from "babylonjs/Maths/math.vector";
import { PointerInfo } from "babylonjs/Events/pointerEvents";
import { Ray } from "babylonjs/Culling/ray";
/**
 * A behavior that when attached to a mesh will allow the mesh to be dragged around the screen based on pointer events
 */
export class PointerDragBehavior implements Behavior<AbstractMesh> {
    private static _AnyMouseId;
    /**
     * Abstract mesh the behavior is set on
     */
    attachedNode: AbstractMesh;
    protected _dragPlane: Mesh;
    private _scene;
    private _pointerObserver;
    private _beforeRenderObserver;
    private static _PlaneScene;
    private _useAlternatePickedPointAboveMaxDragAngleDragSpeed;
    private _activeDragButton;
    private _activePointerInfo;
    /**
     * The maximum tolerated angle between the drag plane and dragging pointer rays to trigger pointer events. Set to 0 to allow any angle (default: 0)
     */
    maxDragAngle: number;
    /**
     * Butttons that can be used to initiate a drag
     */
    dragButtons: number[];
    /**
     * @internal
     */
    _useAlternatePickedPointAboveMaxDragAngle: boolean;
    /**
     * Get or set the currentDraggingPointerId
     * @deprecated Please use currentDraggingPointerId instead
     */
    get currentDraggingPointerID(): number;
    set currentDraggingPointerID(currentDraggingPointerID: number);
    /**
     * The id of the pointer that is currently interacting with the behavior (-1 when no pointer is active)
     */
    currentDraggingPointerId: number;
    /**
     * The last position where the pointer hit the drag plane in world space
     */
    lastDragPosition: Vector3;
    /**
     * If the behavior is currently in a dragging state
     */
    dragging: boolean;
    /**
     * The distance towards the target drag position to move each frame. This can be useful to avoid jitter. Set this to 1 for no delay. (Default: 0.2)
     */
    dragDeltaRatio: number;
    /**
     * If the drag plane orientation should be updated during the dragging (Default: true)
     */
    updateDragPlane: boolean;
    private _debugMode;
    private _moving;
    /**
     *  Fires each time the attached mesh is dragged with the pointer
     *  * delta between last drag position and current drag position in world space
     *  * dragDistance along the drag axis
     *  * dragPlaneNormal normal of the current drag plane used during the drag
     *  * dragPlanePoint in world space where the drag intersects the drag plane
     *
     *  (if validatedDrag is used, the position of the attached mesh might not equal dragPlanePoint)
     */
    onDragObservable: Observable<{
        delta: Vector3;
        dragPlanePoint: Vector3;
        dragPlaneNormal: Vector3;
        dragDistance: number;
        pointerId: number;
        pointerInfo: Nullable<PointerInfo>;
    }>;
    /**
     *  Fires each time a drag begins (eg. mouse down on mesh)
     *  * dragPlanePoint in world space where the drag intersects the drag plane
     *
     *  (if validatedDrag is used, the position of the attached mesh might not equal dragPlanePoint)
     */
    onDragStartObservable: Observable<{
        dragPlanePoint: Vector3;
        pointerId: number;
        pointerInfo: Nullable<PointerInfo>;
    }>;
    /**
     *  Fires each time a drag ends (eg. mouse release after drag)
     *  * dragPlanePoint in world space where the drag intersects the drag plane
     *
     *  (if validatedDrag is used, the position of the attached mesh might not equal dragPlanePoint)
     */
    onDragEndObservable: Observable<{
        dragPlanePoint: Vector3;
        pointerId: number;
        pointerInfo: Nullable<PointerInfo>;
    }>;
    /**
     *  Fires each time behavior enabled state changes
     */
    onEnabledObservable: Observable<boolean>;
    /**
     *  If the attached mesh should be moved when dragged
     */
    moveAttached: boolean;
    /**
     *  If the drag behavior will react to drag events (Default: true)
     */
    set enabled(value: boolean);
    get enabled(): boolean;
    private _enabled;
    /**
     * If pointer events should start and release the drag (Default: true)
     */
    startAndReleaseDragOnPointerEvents: boolean;
    /**
     * If camera controls should be detached during the drag
     */
    detachCameraControls: boolean;
    /**
     * If set, the drag plane/axis will be rotated based on the attached mesh's world rotation (Default: true)
     */
    useObjectOrientationForDragging: boolean;
    private _options;
    /**
     * Gets the options used by the behavior
     */
    get options(): {
        dragAxis?: Vector3;
        dragPlaneNormal?: Vector3;
    };
    /**
     * Sets the options used by the behavior
     */
    set options(options: {
        dragAxis?: Vector3;
        dragPlaneNormal?: Vector3;
    });
    /**
     * Creates a pointer drag behavior that can be attached to a mesh
     * @param options The drag axis or normal of the plane that will be dragged across. If no options are specified the drag plane will always face the ray's origin (eg. camera)
     * @param options.dragAxis
     * @param options.dragPlaneNormal
     */
    constructor(options?: {
        dragAxis?: Vector3;
        dragPlaneNormal?: Vector3;
    });
    /**
     * Predicate to determine if it is valid to move the object to a new position when it is moved
     * @param targetPosition
     */
    validateDrag: (targetPosition: Vector3) => boolean;
    /**
     *  The name of the behavior
     */
    get name(): string;
    /**
     *  Initializes the behavior
     */
    init(): void;
    private _tmpVector;
    private _alternatePickedPoint;
    private _worldDragAxis;
    private _targetPosition;
    private _attachedToElement;
    /**
     * Attaches the drag behavior the passed in mesh
     * @param ownerNode The mesh that will be dragged around once attached
     * @param predicate Predicate to use for pick filtering
     */
    attach(ownerNode: AbstractMesh, predicate?: (m: AbstractMesh) => boolean): void;
    /**
     * Force release the drag action by code.
     */
    releaseDrag(): void;
    private _startDragRay;
    private _lastPointerRay;
    /**
     * Simulates the start of a pointer drag event on the behavior
     * @param pointerId pointerID of the pointer that should be simulated (Default: Any mouse pointer ID)
     * @param fromRay initial ray of the pointer to be simulated (Default: Ray from camera to attached mesh)
     * @param startPickedPoint picked point of the pointer to be simulated (Default: attached mesh position)
     */
    startDrag(pointerId?: number, fromRay?: Ray, startPickedPoint?: Vector3): void;
    protected _startDrag(pointerId: number, fromRay?: Ray, startPickedPoint?: Vector3): void;
    private _dragDelta;
    protected _moveDrag(ray: Ray): void;
    private _pickWithRayOnDragPlane;
    private _pointA;
    private _pointC;
    private _localAxis;
    private _lookAt;
    private _updateDragPlanePosition;
    /**
     *  Detaches the behavior from the mesh
     */
    detach(): void;
}

}
declare module "babylonjs/Behaviors/Meshes/sixDofDragBehavior" {
import { Mesh } from "babylonjs/Meshes/mesh";
import { Vector3, Quaternion } from "babylonjs/Maths/math.vector";
import { Observable } from "babylonjs/Misc/observable";
import { BaseSixDofDragBehavior } from "babylonjs/Behaviors/Meshes/baseSixDofDragBehavior";
/**
 * A behavior that when attached to a mesh will allow the mesh to be dragged around based on directions and origin of the pointer's ray
 */
export class SixDofDragBehavior extends BaseSixDofDragBehavior {
    private _sceneRenderObserver;
    private _virtualTransformNode;
    protected _targetPosition: Vector3;
    protected _targetOrientation: Quaternion;
    protected _targetScaling: Vector3;
    protected _startingPosition: Vector3;
    protected _startingOrientation: Quaternion;
    protected _startingScaling: Vector3;
    /**
     * Fires when position is updated
     */
    onPositionChangedObservable: Observable<{
        position: Vector3;
    }>;
    /**
     * The distance towards the target drag position to move each frame. This can be useful to avoid jitter. Set this to 1 for no delay. (Default: 0.2)
     */
    dragDeltaRatio: number;
    /**
     * If the object should rotate to face the drag origin
     */
    rotateDraggedObject: boolean;
    /**
     * If `rotateDraggedObject` is set to `true`, this parameter determines if we are only rotating around the y axis (yaw)
     */
    rotateAroundYOnly: boolean;
    /**
     * Should the behavior rotate 1:1 with the motion controller, when one is used.
     */
    rotateWithMotionController: boolean;
    /**
     *  The name of the behavior
     */
    get name(): string;
    /**
     * Use this flag to update the target but not move the owner node towards the target
     */
    disableMovement: boolean;
    /**
     * Should the object rotate towards the camera when we start dragging it
     */
    faceCameraOnDragStart: boolean;
    /**
     * Attaches the six DoF drag behavior
     * @param ownerNode The mesh that will be dragged around once attached
     */
    attach(ownerNode: Mesh): void;
    private _getPositionOffsetAround;
    private _onePointerPositionUpdated;
    private _twoPointersPositionUpdated;
    protected _targetDragStart(): void;
    protected _targetDrag(worldDeltaPosition: Vector3, worldDeltaRotation: Quaternion): void;
    protected _targetDragEnd(): void;
    /**
     *  Detaches the behavior from the mesh
     */
    detach(): void;
}

}
declare module "babylonjs/Behaviors/Meshes/surfaceMagnetismBehavior" {
import { PickingInfo } from "babylonjs/Collisions/pickingInfo";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { Mesh } from "babylonjs/Meshes/mesh";
import { Scene } from "babylonjs/scene";
import { Behavior } from "babylonjs/Behaviors/behavior";
/**
 * A behavior that allows a transform node to stick to a surface position/orientation
 * @since 5.0.0
 */
export class SurfaceMagnetismBehavior implements Behavior<Mesh> {
    private _scene;
    private _attachedMesh;
    private _attachPointLocalOffset;
    private _pointerObserver;
    private _workingPosition;
    private _workingQuaternion;
    private _lastTick;
    private _onBeforeRender;
    private _hit;
    /**
     * Distance offset from the hit point to place the target at, along the hit normal.
     */
    hitNormalOffset: number;
    /**
     * Name of the behavior
     */
    get name(): string;
    /**
     * Spatial mapping meshes to collide with
     */
    meshes: AbstractMesh[];
    /**
     * Function called when the behavior needs to be initialized (after attaching it to a target)
     */
    init(): void;
    /**
     * Set to false if the node should strictly follow the camera without any interpolation time
     */
    interpolatePose: boolean;
    /**
     * Rate of interpolation of position and rotation of the attached node.
     * Higher values will give a slower interpolation.
     */
    lerpTime: number;
    /**
     * If true, pitch and roll are omitted.
     */
    keepOrientationVertical: boolean;
    /**
     * Is this behavior reacting to pointer events
     */
    enabled: boolean;
    /**
     * Maximum distance for the node to stick to the surface
     */
    maxStickingDistance: number;
    /**
     * Attaches the behavior to a transform node
     * @param target defines the target where the behavior is attached to
     * @param scene the scene
     */
    attach(target: Mesh, scene?: Scene): void;
    /**
     * Detaches the behavior
     */
    detach(): void;
    private _getTargetPose;
    /**
     * Updates the attach point with the current geometry extents of the attached mesh
     */
    updateAttachPoint(): void;
    /**
     * Finds the intersection point of the given ray onto the meshes and updates the target.
     * Transformation will be interpolated according to `interpolatePose` and `lerpTime` properties.
     * If no mesh of `meshes` are hit, this does nothing.
     * @param pickInfo The input pickingInfo that will be used to intersect the meshes
     * @returns a boolean indicating if we found a hit to stick to
     */
    findAndUpdateTarget(pickInfo: PickingInfo): boolean;
    private _getAttachPointOffsetToRef;
    private _updateTransformToGoal;
    private _addObservables;
    private _removeObservables;
}

}
declare module "babylonjs/Bones/bone" {
import { Skeleton } from "babylonjs/Bones/skeleton";
import { Vector3, Quaternion, Matrix } from "babylonjs/Maths/math.vector";
import { Nullable } from "babylonjs/types";
import { TransformNode } from "babylonjs/Meshes/transformNode";
import { Node } from "babylonjs/node";
import { Space } from "babylonjs/Maths/math.axis";
import { AnimationPropertiesOverride } from "babylonjs/Animations/animationPropertiesOverride";
/**
 * Class used to store bone information
 * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/bonesSkeletons
 */
export class Bone extends Node {
    /**
     * defines the bone name
     */
    name: string;
    private static _TmpVecs;
    private static _TmpQuat;
    private static _TmpMats;
    /**
     * Gets the list of child bones
     */
    children: Bone[];
    /** Gets the animations associated with this bone */
    animations: import("babylonjs/Animations/animation").Animation[];
    /**
     * Gets or sets bone length
     */
    length: number;
    /**
     * @internal Internal only
     * Set this value to map this bone to a different index in the transform matrices
     * Set this value to -1 to exclude the bone from the transform matrices
     */
    _index: Nullable<number>;
    private _skeleton;
    private _localMatrix;
    private _restPose;
    private _baseMatrix;
    private _absoluteTransform;
    private _invertedAbsoluteTransform;
    private _scalingDeterminant;
    private _worldTransform;
    private _localScaling;
    private _localRotation;
    private _localPosition;
    private _needToDecompose;
    private _needToCompose;
    /** @internal */
    _linkedTransformNode: Nullable<TransformNode>;
    /** @internal */
    _waitingTransformNodeId: Nullable<string>;
    /** @internal */
    get _matrix(): Matrix;
    /** @internal */
    set _matrix(value: Matrix);
    /**
     * Create a new bone
     * @param name defines the bone name
     * @param skeleton defines the parent skeleton
     * @param parentBone defines the parent (can be null if the bone is the root)
     * @param localMatrix defines the local matrix
     * @param restPose defines the rest pose matrix
     * @param baseMatrix defines the base matrix
     * @param index defines index of the bone in the hierarchy
     */
    constructor(
    /**
     * defines the bone name
     */
    name: string, skeleton: Skeleton, parentBone?: Nullable<Bone>, localMatrix?: Nullable<Matrix>, restPose?: Nullable<Matrix>, baseMatrix?: Nullable<Matrix>, index?: Nullable<number>);
    /**
     * Gets the current object class name.
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the parent skeleton
     * @returns a skeleton
     */
    getSkeleton(): Skeleton;
    get parent(): Bone;
    /**
     * Gets parent bone
     * @returns a bone or null if the bone is the root of the bone hierarchy
     */
    getParent(): Nullable<Bone>;
    /**
     * Returns an array containing the root bones
     * @returns an array containing the root bones
     */
    getChildren(): Array<Bone>;
    /**
     * Gets the node index in matrix array generated for rendering
     * @returns the node index
     */
    getIndex(): number;
    set parent(newParent: Nullable<Bone>);
    /**
     * Sets the parent bone
     * @param parent defines the parent (can be null if the bone is the root)
     * @param updateDifferenceMatrix defines if the difference matrix must be updated
     */
    setParent(parent: Nullable<Bone>, updateDifferenceMatrix?: boolean): void;
    /**
     * Gets the local matrix
     * @returns a matrix
     */
    getLocalMatrix(): Matrix;
    /**
     * Gets the base matrix (initial matrix which remains unchanged)
     * @returns the base matrix (as known as bind pose matrix)
     */
    getBaseMatrix(): Matrix;
    /**
     * Gets the rest pose matrix
     * @returns a matrix
     */
    getRestPose(): Matrix;
    /**
     * Sets the rest pose matrix
     * @param matrix the local-space rest pose to set for this bone
     */
    setRestPose(matrix: Matrix): void;
    /**
     * Gets the bind pose matrix
     * @returns the bind pose matrix
     * @deprecated Please use getBaseMatrix instead
     */
    getBindPose(): Matrix;
    /**
     * Sets the bind pose matrix
     * @param matrix the local-space bind pose to set for this bone
     * @deprecated Please use updateMatrix instead
     */
    setBindPose(matrix: Matrix): void;
    /**
     * Gets a matrix used to store world matrix (ie. the matrix sent to shaders)
     */
    getWorldMatrix(): Matrix;
    /**
     * Sets the local matrix to rest pose matrix
     */
    returnToRest(): void;
    /**
     * Gets the inverse of the absolute transform matrix.
     * This matrix will be multiplied by local matrix to get the difference matrix (ie. the difference between original state and current state)
     * @returns a matrix
     */
    getInvertedAbsoluteTransform(): Matrix;
    /**
     * Gets the absolute transform matrix (ie base matrix * parent world matrix)
     * @returns a matrix
     */
    getAbsoluteTransform(): Matrix;
    /**
     * Links with the given transform node.
     * The local matrix of this bone is copied from the transform node every frame.
     * @param transformNode defines the transform node to link to
     */
    linkTransformNode(transformNode: Nullable<TransformNode>): void;
    /**
     * Gets the node used to drive the bone's transformation
     * @returns a transform node or null
     */
    getTransformNode(): Nullable<TransformNode>;
    /** Gets or sets current position (in local space) */
    get position(): Vector3;
    set position(newPosition: Vector3);
    /** Gets or sets current rotation (in local space) */
    get rotation(): Vector3;
    set rotation(newRotation: Vector3);
    /** Gets or sets current rotation quaternion (in local space) */
    get rotationQuaternion(): Quaternion;
    set rotationQuaternion(newRotation: Quaternion);
    /** Gets or sets current scaling (in local space) */
    get scaling(): Vector3;
    set scaling(newScaling: Vector3);
    /**
     * Gets the animation properties override
     */
    get animationPropertiesOverride(): Nullable<AnimationPropertiesOverride>;
    private _decompose;
    private _compose;
    /**
     * Update the base and local matrices
     * @param matrix defines the new base or local matrix
     * @param updateDifferenceMatrix defines if the difference matrix must be updated
     * @param updateLocalMatrix defines if the local matrix should be updated
     */
    updateMatrix(matrix: Matrix, updateDifferenceMatrix?: boolean, updateLocalMatrix?: boolean): void;
    /**
     * @internal
     */
    _updateDifferenceMatrix(rootMatrix?: Matrix, updateChildren?: boolean): void;
    /**
     * Flag the bone as dirty (Forcing it to update everything)
     * @returns this bone
     */
    markAsDirty(): Bone;
    /** @internal */
    _markAsDirtyAndCompose(): void;
    private _markAsDirtyAndDecompose;
    /**
     * Translate the bone in local or world space
     * @param vec The amount to translate the bone
     * @param space The space that the translation is in
     * @param tNode The TransformNode that this bone is attached to. This is only used in world space
     */
    translate(vec: Vector3, space?: Space, tNode?: TransformNode): void;
    /**
     * Set the position of the bone in local or world space
     * @param position The position to set the bone
     * @param space The space that the position is in
     * @param tNode The TransformNode that this bone is attached to.  This is only used in world space
     */
    setPosition(position: Vector3, space?: Space, tNode?: TransformNode): void;
    /**
     * Set the absolute position of the bone (world space)
     * @param position The position to set the bone
     * @param tNode The TransformNode that this bone is attached to
     */
    setAbsolutePosition(position: Vector3, tNode?: TransformNode): void;
    /**
     * Scale the bone on the x, y and z axes (in local space)
     * @param x The amount to scale the bone on the x axis
     * @param y The amount to scale the bone on the y axis
     * @param z The amount to scale the bone on the z axis
     * @param scaleChildren sets this to true if children of the bone should be scaled as well (false by default)
     */
    scale(x: number, y: number, z: number, scaleChildren?: boolean): void;
    /**
     * Set the bone scaling in local space
     * @param scale defines the scaling vector
     */
    setScale(scale: Vector3): void;
    /**
     * Gets the current scaling in local space
     * @returns the current scaling vector
     */
    getScale(): Vector3;
    /**
     * Gets the current scaling in local space and stores it in a target vector
     * @param result defines the target vector
     */
    getScaleToRef(result: Vector3): void;
    /**
     * Set the yaw, pitch, and roll of the bone in local or world space
     * @param yaw The rotation of the bone on the y axis
     * @param pitch The rotation of the bone on the x axis
     * @param roll The rotation of the bone on the z axis
     * @param space The space that the axes of rotation are in
     * @param tNode The TransformNode that this bone is attached to.  This is only used in world space
     */
    setYawPitchRoll(yaw: number, pitch: number, roll: number, space?: Space, tNode?: TransformNode): void;
    /**
     * Add a rotation to the bone on an axis in local or world space
     * @param axis The axis to rotate the bone on
     * @param amount The amount to rotate the bone
     * @param space The space that the axis is in
     * @param tNode The TransformNode that this bone is attached to. This is only used in world space
     */
    rotate(axis: Vector3, amount: number, space?: Space, tNode?: TransformNode): void;
    /**
     * Set the rotation of the bone to a particular axis angle in local or world space
     * @param axis The axis to rotate the bone on
     * @param angle The angle that the bone should be rotated to
     * @param space The space that the axis is in
     * @param tNode The TransformNode that this bone is attached to.  This is only used in world space
     */
    setAxisAngle(axis: Vector3, angle: number, space?: Space, tNode?: TransformNode): void;
    /**
     * Set the euler rotation of the bone in local or world space
     * @param rotation The euler rotation that the bone should be set to
     * @param space The space that the rotation is in
     * @param tNode The TransformNode that this bone is attached to. This is only used in world space
     */
    setRotation(rotation: Vector3, space?: Space, tNode?: TransformNode): void;
    /**
     * Set the quaternion rotation of the bone in local or world space
     * @param quat The quaternion rotation that the bone should be set to
     * @param space The space that the rotation is in
     * @param tNode The TransformNode that this bone is attached to. This is only used in world space
     */
    setRotationQuaternion(quat: Quaternion, space?: Space, tNode?: TransformNode): void;
    /**
     * Set the rotation matrix of the bone in local or world space
     * @param rotMat The rotation matrix that the bone should be set to
     * @param space The space that the rotation is in
     * @param tNode The TransformNode that this bone is attached to. This is only used in world space
     */
    setRotationMatrix(rotMat: Matrix, space?: Space, tNode?: TransformNode): void;
    private _rotateWithMatrix;
    private _getNegativeRotationToRef;
    /**
     * Get the position of the bone in local or world space
     * @param space The space that the returned position is in
     * @param tNode The TransformNode that this bone is attached to. This is only used in world space
     * @returns The position of the bone
     */
    getPosition(space?: Space, tNode?: Nullable<TransformNode>): Vector3;
    /**
     * Copy the position of the bone to a vector3 in local or world space
     * @param space The space that the returned position is in
     * @param tNode The TransformNode that this bone is attached to. This is only used in world space
     * @param result The vector3 to copy the position to
     */
    getPositionToRef(space: Space | undefined, tNode: Nullable<TransformNode>, result: Vector3): void;
    /**
     * Get the absolute position of the bone (world space)
     * @param tNode The TransformNode that this bone is attached to
     * @returns The absolute position of the bone
     */
    getAbsolutePosition(tNode?: Nullable<TransformNode>): Vector3;
    /**
     * Copy the absolute position of the bone (world space) to the result param
     * @param tNode The TransformNode that this bone is attached to
     * @param result The vector3 to copy the absolute position to
     */
    getAbsolutePositionToRef(tNode: TransformNode, result: Vector3): void;
    /**
     * Compute the absolute transforms of this bone and its children
     */
    computeAbsoluteTransforms(): void;
    /**
     * Get the world direction from an axis that is in the local space of the bone
     * @param localAxis The local direction that is used to compute the world direction
     * @param tNode The TransformNode that this bone is attached to
     * @returns The world direction
     */
    getDirection(localAxis: Vector3, tNode?: Nullable<TransformNode>): Vector3;
    /**
     * Copy the world direction to a vector3 from an axis that is in the local space of the bone
     * @param localAxis The local direction that is used to compute the world direction
     * @param tNode The TransformNode that this bone is attached to
     * @param result The vector3 that the world direction will be copied to
     */
    getDirectionToRef(localAxis: Vector3, tNode: Nullable<TransformNode> | undefined, result: Vector3): void;
    /**
     * Get the euler rotation of the bone in local or world space
     * @param space The space that the rotation should be in
     * @param tNode The TransformNode that this bone is attached to.  This is only used in world space
     * @returns The euler rotation
     */
    getRotation(space?: Space, tNode?: Nullable<TransformNode>): Vector3;
    /**
     * Copy the euler rotation of the bone to a vector3.  The rotation can be in either local or world space
     * @param space The space that the rotation should be in
     * @param tNode The TransformNode that this bone is attached to.  This is only used in world space
     * @param result The vector3 that the rotation should be copied to
     */
    getRotationToRef(space: Space | undefined, tNode: Nullable<TransformNode> | undefined, result: Vector3): void;
    /**
     * Get the quaternion rotation of the bone in either local or world space
     * @param space The space that the rotation should be in
     * @param tNode The TransformNode that this bone is attached to.  This is only used in world space
     * @returns The quaternion rotation
     */
    getRotationQuaternion(space?: Space, tNode?: Nullable<TransformNode>): Quaternion;
    /**
     * Copy the quaternion rotation of the bone to a quaternion.  The rotation can be in either local or world space
     * @param space The space that the rotation should be in
     * @param tNode The TransformNode that this bone is attached to.  This is only used in world space
     * @param result The quaternion that the rotation should be copied to
     */
    getRotationQuaternionToRef(space: Space | undefined, tNode: Nullable<TransformNode> | undefined, result: Quaternion): void;
    /**
     * Get the rotation matrix of the bone in local or world space
     * @param space The space that the rotation should be in
     * @param tNode The TransformNode that this bone is attached to.  This is only used in world space
     * @returns The rotation matrix
     */
    getRotationMatrix(space: Space | undefined, tNode: TransformNode): Matrix;
    /**
     * Copy the rotation matrix of the bone to a matrix.  The rotation can be in either local or world space
     * @param space The space that the rotation should be in
     * @param tNode The TransformNode that this bone is attached to.  This is only used in world space
     * @param result The quaternion that the rotation should be copied to
     */
    getRotationMatrixToRef(space: Space | undefined, tNode: TransformNode, result: Matrix): void;
    /**
     * Get the world position of a point that is in the local space of the bone
     * @param position The local position
     * @param tNode The TransformNode that this bone is attached to
     * @returns The world position
     */
    getAbsolutePositionFromLocal(position: Vector3, tNode?: Nullable<TransformNode>): Vector3;
    /**
     * Get the world position of a point that is in the local space of the bone and copy it to the result param
     * @param position The local position
     * @param tNode The TransformNode that this bone is attached to
     * @param result The vector3 that the world position should be copied to
     */
    getAbsolutePositionFromLocalToRef(position: Vector3, tNode: Nullable<TransformNode> | undefined, result: Vector3): void;
    /**
     * Get the local position of a point that is in world space
     * @param position The world position
     * @param tNode The TransformNode that this bone is attached to
     * @returns The local position
     */
    getLocalPositionFromAbsolute(position: Vector3, tNode?: Nullable<TransformNode>): Vector3;
    /**
     * Get the local position of a point that is in world space and copy it to the result param
     * @param position The world position
     * @param tNode The TransformNode that this bone is attached to
     * @param result The vector3 that the local position should be copied to
     */
    getLocalPositionFromAbsoluteToRef(position: Vector3, tNode: Nullable<TransformNode> | undefined, result: Vector3): void;
    /**
     * Set the current local matrix as the restPose for this bone.
     */
    setCurrentPoseAsRest(): void;
}
export {};

}
declare module "babylonjs/Bones/boneIKController" {
import { Bone } from "babylonjs/Bones/bone";
import { Vector3 } from "babylonjs/Maths/math.vector";
import { TransformNode } from "babylonjs/Meshes/transformNode";
import { Nullable } from "babylonjs/types";
/**
 * Class used to apply inverse kinematics to bones
 * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/bonesSkeletons#boneikcontroller
 */
export class BoneIKController {
    private static _TmpVecs;
    private static _TmpQuat;
    private static _TmpMats;
    /**
     * Gets or sets the target TransformNode
     * Name kept as mesh for back compatibility
     */
    targetMesh: TransformNode;
    /** Gets or sets the mesh used as pole */
    poleTargetMesh: TransformNode;
    /**
     * Gets or sets the bone used as pole
     */
    poleTargetBone: Nullable<Bone>;
    /**
     * Gets or sets the target position
     */
    targetPosition: Vector3;
    /**
     * Gets or sets the pole target position
     */
    poleTargetPosition: Vector3;
    /**
     * Gets or sets the pole target local offset
     */
    poleTargetLocalOffset: Vector3;
    /**
     * Gets or sets the pole angle
     */
    poleAngle: number;
    /**
     * Gets or sets the TransformNode associated with the controller
     * Name kept as mesh for back compatibility
     */
    mesh: TransformNode;
    /**
     * The amount to slerp (spherical linear interpolation) to the target.  Set this to a value between 0 and 1 (a value of 1 disables slerp)
     */
    slerpAmount: number;
    private _bone1Quat;
    private _bone1Mat;
    private _bone2Ang;
    private _bone1;
    private _bone2;
    private _bone1Length;
    private _bone2Length;
    private _maxAngle;
    private _maxReach;
    private _rightHandedSystem;
    private _bendAxis;
    private _slerping;
    private _adjustRoll;
    private _notEnoughInformation;
    /**
     * Gets or sets maximum allowed angle
     */
    get maxAngle(): number;
    set maxAngle(value: number);
    /**
     * Creates a new BoneIKController
     * @param mesh defines the TransformNode to control
     * @param bone defines the bone to control. The bone needs to have a parent bone. It also needs to have a length greater than 0 or a children we can use to infer its length.
     * @param options defines options to set up the controller
     * @param options.targetMesh
     * @param options.poleTargetMesh
     * @param options.poleTargetBone
     * @param options.poleTargetLocalOffset
     * @param options.poleAngle
     * @param options.bendAxis
     * @param options.maxAngle
     * @param options.slerpAmount
     */
    constructor(mesh: TransformNode, bone: Bone, options?: {
        targetMesh?: TransformNode;
        poleTargetMesh?: TransformNode;
        poleTargetBone?: Bone;
        poleTargetLocalOffset?: Vector3;
        poleAngle?: number;
        bendAxis?: Vector3;
        maxAngle?: number;
        slerpAmount?: number;
    });
    private _setMaxAngle;
    /**
     * Force the controller to update the bones
     */
    update(): void;
    private _updateLinkedTransformRotation;
}

}
declare module "babylonjs/Bones/boneLookController" {
import { Vector3 } from "babylonjs/Maths/math.vector";
import { TransformNode } from "babylonjs/Meshes/transformNode";
import { Bone } from "babylonjs/Bones/bone";
import { Space } from "babylonjs/Maths/math.axis";
/**
 * Class used to make a bone look toward a point in space
 * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/bonesSkeletons#bonelookcontroller
 */
export class BoneLookController {
    private static _TmpVecs;
    private static _TmpQuat;
    private static _TmpMats;
    /**
     * The target Vector3 that the bone will look at
     */
    target: Vector3;
    /**
     * The TransformNode that the bone is attached to
     * Name kept as mesh for back compatibility
     */
    mesh: TransformNode;
    /**
     * The bone that will be looking to the target
     */
    bone: Bone;
    /**
     * The up axis of the coordinate system that is used when the bone is rotated
     */
    upAxis: Vector3;
    /**
     * The space that the up axis is in - Space.BONE, Space.LOCAL (default), or Space.WORLD
     */
    upAxisSpace: Space;
    /**
     * Used to make an adjustment to the yaw of the bone
     */
    adjustYaw: number;
    /**
     * Used to make an adjustment to the pitch of the bone
     */
    adjustPitch: number;
    /**
     * Used to make an adjustment to the roll of the bone
     */
    adjustRoll: number;
    /**
     * The amount to slerp (spherical linear interpolation) to the target.  Set this to a value between 0 and 1 (a value of 1 disables slerp)
     */
    slerpAmount: number;
    private _minYaw;
    private _maxYaw;
    private _minPitch;
    private _maxPitch;
    private _minYawSin;
    private _minYawCos;
    private _maxYawSin;
    private _maxYawCos;
    private _midYawConstraint;
    private _minPitchTan;
    private _maxPitchTan;
    private _boneQuat;
    private _slerping;
    private _transformYawPitch;
    private _transformYawPitchInv;
    private _firstFrameSkipped;
    private _yawRange;
    private _fowardAxis;
    /**
     * Gets or sets the minimum yaw angle that the bone can look to
     */
    get minYaw(): number;
    set minYaw(value: number);
    /**
     * Gets or sets the maximum yaw angle that the bone can look to
     */
    get maxYaw(): number;
    set maxYaw(value: number);
    /**
     * Gets or sets the minimum pitch angle that the bone can look to
     */
    get minPitch(): number;
    set minPitch(value: number);
    /**
     * Gets or sets the maximum pitch angle that the bone can look to
     */
    get maxPitch(): number;
    set maxPitch(value: number);
    /**
     * Create a BoneLookController
     * @param mesh the TransformNode that the bone belongs to
     * @param bone the bone that will be looking to the target
     * @param target the target Vector3 to look at
     * @param options optional settings:
     * * maxYaw: the maximum angle the bone will yaw to
     * * minYaw: the minimum angle the bone will yaw to
     * * maxPitch: the maximum angle the bone will pitch to
     * * minPitch: the minimum angle the bone will yaw to
     * * slerpAmount: set the between 0 and 1 to make the bone slerp to the target.
     * * upAxis: the up axis of the coordinate system
     * * upAxisSpace: the space that the up axis is in - Space.BONE, Space.LOCAL (default), or Space.WORLD.
     * * yawAxis: set yawAxis if the bone does not yaw on the y axis
     * * pitchAxis: set pitchAxis if the bone does not pitch on the x axis
     * * adjustYaw: used to make an adjustment to the yaw of the bone
     * * adjustPitch: used to make an adjustment to the pitch of the bone
     * * adjustRoll: used to make an adjustment to the roll of the bone
     * @param options.maxYaw
     * @param options.minYaw
     * @param options.maxPitch
     * @param options.minPitch
     * @param options.slerpAmount
     * @param options.upAxis
     * @param options.upAxisSpace
     * @param options.yawAxis
     * @param options.pitchAxis
     * @param options.adjustYaw
     * @param options.adjustPitch
     * @param options.adjustRoll
     **/
    constructor(mesh: TransformNode, bone: Bone, target: Vector3, options?: {
        maxYaw?: number;
        minYaw?: number;
        maxPitch?: number;
        minPitch?: number;
        slerpAmount?: number;
        upAxis?: Vector3;
        upAxisSpace?: Space;
        yawAxis?: Vector3;
        pitchAxis?: Vector3;
        adjustYaw?: number;
        adjustPitch?: number;
        adjustRoll?: number;
    });
    /**
     * Update the bone to look at the target.  This should be called before the scene is rendered (use scene.registerBeforeRender())
     */
    update(): void;
    private _getAngleDiff;
    private _getAngleBetween;
    private _isAngleBetween;
    private _updateLinkedTransformRotation;
}

}
declare module "babylonjs/Bones/index" {
export * from "babylonjs/Bones/bone";
export * from "babylonjs/Bones/boneIKController";
export * from "babylonjs/Bones/boneLookController";
export * from "babylonjs/Bones/skeleton";

}
declare module "babylonjs/Bones/skeleton" {
import { Bone } from "babylonjs/Bones/bone";
import { Observable } from "babylonjs/Misc/observable";
import { Vector3, Matrix } from "babylonjs/Maths/math.vector";
import { Scene } from "babylonjs/scene";
import { Nullable } from "babylonjs/types";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { RawTexture } from "babylonjs/Materials/Textures/rawTexture";
import { Animatable } from "babylonjs/Animations/animatable";
import { AnimationPropertiesOverride } from "babylonjs/Animations/animationPropertiesOverride";
import { Animation } from "babylonjs/Animations/animation";
import { AnimationRange } from "babylonjs/Animations/animationRange";
import { IInspectable } from "babylonjs/Misc/iInspectable";
import { IAnimatable } from "babylonjs/Animations/animatable.interface";
import { AbstractScene } from "babylonjs/abstractScene";
/**
 * Class used to handle skinning animations
 * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/bonesSkeletons
 */
export class Skeleton implements IAnimatable {
    /** defines the skeleton name */
    name: string;
    /** defines the skeleton Id */
    id: string;
    /**
     * Defines the list of child bones
     */
    bones: Bone[];
    /**
     * Defines an estimate of the dimension of the skeleton at rest
     */
    dimensionsAtRest: Vector3;
    /**
     * Defines a boolean indicating if the root matrix is provided by meshes or by the current skeleton (this is the default value)
     */
    needInitialSkinMatrix: boolean;
    /**
     * Gets the list of animations attached to this skeleton
     */
    animations: Array<Animation>;
    private _scene;
    private _isDirty;
    private _transformMatrices;
    private _transformMatrixTexture;
    private _meshesWithPoseMatrix;
    private _animatables;
    private _identity;
    private _synchronizedWithMesh;
    private _ranges;
    private _absoluteTransformIsDirty;
    private _canUseTextureForBones;
    private _uniqueId;
    /** @internal */
    _numBonesWithLinkedTransformNode: number;
    /** @internal */
    _hasWaitingData: Nullable<boolean>;
    /** @internal */
    _parentContainer: Nullable<AbstractScene>;
    /**
     * Specifies if the skeleton should be serialized
     */
    doNotSerialize: boolean;
    private _useTextureToStoreBoneMatrices;
    /**
     * Gets or sets a boolean indicating that bone matrices should be stored as a texture instead of using shader uniforms (default is true).
     * Please note that this option is not available if the hardware does not support it
     */
    get useTextureToStoreBoneMatrices(): boolean;
    set useTextureToStoreBoneMatrices(value: boolean);
    private _animationPropertiesOverride;
    /**
     * Gets or sets the animation properties override
     */
    get animationPropertiesOverride(): Nullable<AnimationPropertiesOverride>;
    set animationPropertiesOverride(value: Nullable<AnimationPropertiesOverride>);
    /**
     * List of inspectable custom properties (used by the Inspector)
     * @see https://doc.babylonjs.com/toolsAndResources/inspector#extensibility
     */
    inspectableCustomProperties: IInspectable[];
    /**
     * An observable triggered before computing the skeleton's matrices
     */
    onBeforeComputeObservable: Observable<Skeleton>;
    /**
     * Gets a boolean indicating that the skeleton effectively stores matrices into a texture
     */
    get isUsingTextureForMatrices(): boolean;
    /**
     * Gets the unique ID of this skeleton
     */
    get uniqueId(): number;
    /**
     * Creates a new skeleton
     * @param name defines the skeleton name
     * @param id defines the skeleton Id
     * @param scene defines the hosting scene
     */
    constructor(
    /** defines the skeleton name */
    name: string, 
    /** defines the skeleton Id */
    id: string, scene: Scene);
    /**
     * Gets the current object class name.
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Returns an array containing the root bones
     * @returns an array containing the root bones
     */
    getChildren(): Array<Bone>;
    /**
     * Gets the list of transform matrices to send to shaders (one matrix per bone)
     * @param mesh defines the mesh to use to get the root matrix (if needInitialSkinMatrix === true)
     * @returns a Float32Array containing matrices data
     */
    getTransformMatrices(mesh: AbstractMesh): Float32Array;
    /**
     * Gets the list of transform matrices to send to shaders inside a texture (one matrix per bone)
     * @param mesh defines the mesh to use to get the root matrix (if needInitialSkinMatrix === true)
     * @returns a raw texture containing the data
     */
    getTransformMatrixTexture(mesh: AbstractMesh): Nullable<RawTexture>;
    /**
     * Gets the current hosting scene
     * @returns a scene object
     */
    getScene(): Scene;
    /**
     * Gets a string representing the current skeleton data
     * @param fullDetails defines a boolean indicating if we want a verbose version
     * @returns a string representing the current skeleton data
     */
    toString(fullDetails?: boolean): string;
    /**
     * Get bone's index searching by name
     * @param name defines bone's name to search for
     * @returns the indice of the bone. Returns -1 if not found
     */
    getBoneIndexByName(name: string): number;
    /**
     * Create a new animation range
     * @param name defines the name of the range
     * @param from defines the start key
     * @param to defines the end key
     */
    createAnimationRange(name: string, from: number, to: number): void;
    /**
     * Delete a specific animation range
     * @param name defines the name of the range
     * @param deleteFrames defines if frames must be removed as well
     */
    deleteAnimationRange(name: string, deleteFrames?: boolean): void;
    /**
     * Gets a specific animation range
     * @param name defines the name of the range to look for
     * @returns the requested animation range or null if not found
     */
    getAnimationRange(name: string): Nullable<AnimationRange>;
    /**
     * Gets the list of all animation ranges defined on this skeleton
     * @returns an array
     */
    getAnimationRanges(): Nullable<AnimationRange>[];
    /**
     * Copy animation range from a source skeleton.
     * This is not for a complete retargeting, only between very similar skeleton's with only possible bone length differences
     * @param source defines the source skeleton
     * @param name defines the name of the range to copy
     * @param rescaleAsRequired defines if rescaling must be applied if required
     * @returns true if operation was successful
     */
    copyAnimationRange(source: Skeleton, name: string, rescaleAsRequired?: boolean): boolean;
    /**
     * Forces the skeleton to go to rest pose
     */
    returnToRest(): void;
    private _getHighestAnimationFrame;
    /**
     * Begin a specific animation range
     * @param name defines the name of the range to start
     * @param loop defines if looping must be turned on (false by default)
     * @param speedRatio defines the speed ratio to apply (1 by default)
     * @param onAnimationEnd defines a callback which will be called when animation will end
     * @returns a new animatable
     */
    beginAnimation(name: string, loop?: boolean, speedRatio?: number, onAnimationEnd?: () => void): Nullable<Animatable>;
    /**
     * Convert the keyframes for a range of animation on a skeleton to be relative to a given reference frame.
     * @param skeleton defines the Skeleton containing the animation range to convert
     * @param referenceFrame defines the frame that keyframes in the range will be relative to
     * @param range defines the name of the AnimationRange belonging to the Skeleton to convert
     * @returns the original skeleton
     */
    static MakeAnimationAdditive(skeleton: Skeleton, referenceFrame: number | undefined, range: string): Nullable<Skeleton>;
    /** @internal */
    _markAsDirty(): void;
    /**
     * @internal
     */
    _registerMeshWithPoseMatrix(mesh: AbstractMesh): void;
    /**
     * @internal
     */
    _unregisterMeshWithPoseMatrix(mesh: AbstractMesh): void;
    private _computeTransformMatrices;
    /**
     * Build all resources required to render a skeleton
     */
    prepare(): void;
    /**
     * Gets the list of animatables currently running for this skeleton
     * @returns an array of animatables
     */
    getAnimatables(): IAnimatable[];
    /**
     * Clone the current skeleton
     * @param name defines the name of the new skeleton
     * @param id defines the id of the new skeleton
     * @returns the new skeleton
     */
    clone(name: string, id?: string): Skeleton;
    /**
     * Enable animation blending for this skeleton
     * @param blendingSpeed defines the blending speed to apply
     * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/advanced_animations#animation-blending
     */
    enableBlending(blendingSpeed?: number): void;
    /**
     * Releases all resources associated with the current skeleton
     */
    dispose(): void;
    /**
     * Serialize the skeleton in a JSON object
     * @returns a JSON object
     */
    serialize(): any;
    /**
     * Creates a new skeleton from serialized data
     * @param parsedSkeleton defines the serialized data
     * @param scene defines the hosting scene
     * @returns a new skeleton
     */
    static Parse(parsedSkeleton: any, scene: Scene): Skeleton;
    /**
     * Compute all node absolute transforms
     * @param forceUpdate defines if computation must be done even if cache is up to date
     */
    computeAbsoluteTransforms(forceUpdate?: boolean): void;
    /**
     * Gets the root pose matrix
     * @returns a matrix
     */
    getPoseMatrix(): Nullable<Matrix>;
    /**
     * Sorts bones per internal index
     */
    sortBones(): void;
    private _sortBones;
    /**
     * Set the current local matrix as the restPose for all bones in the skeleton.
     */
    setCurrentPoseAsRest(): void;
}

}
declare module "babylonjs/Buffers/buffer" {
import { Nullable, DataArray, FloatArray } from "babylonjs/types";
import { DataBuffer } from "babylonjs/Buffers/dataBuffer";
/**
 * Class used to store data that will be store in GPU memory
 */
export class Buffer {
    private _engine;
    private _buffer;
    /** @internal */
    _data: Nullable<DataArray>;
    private _updatable;
    private _instanced;
    private _divisor;
    private _isAlreadyOwned;
    /**
     * Gets the byte stride.
     */
    readonly byteStride: number;
    /**
     * Constructor
     * @param engine the engine
     * @param data the data to use for this buffer
     * @param updatable whether the data is updatable
     * @param stride the stride (optional)
     * @param postponeInternalCreation whether to postpone creating the internal WebGL buffer (optional)
     * @param instanced whether the buffer is instanced (optional)
     * @param useBytes set to true if the stride in in bytes (optional)
     * @param divisor sets an optional divisor for instances (1 by default)
     */
    constructor(engine: any, data: DataArray | DataBuffer, updatable: boolean, stride?: number, postponeInternalCreation?: boolean, instanced?: boolean, useBytes?: boolean, divisor?: number);
    /**
     * Create a new VertexBuffer based on the current buffer
     * @param kind defines the vertex buffer kind (position, normal, etc.)
     * @param offset defines offset in the buffer (0 by default)
     * @param size defines the size in floats of attributes (position is 3 for instance)
     * @param stride defines the stride size in floats in the buffer (the offset to apply to reach next value when data is interleaved)
     * @param instanced defines if the vertex buffer contains indexed data
     * @param useBytes defines if the offset and stride are in bytes     *
     * @param divisor sets an optional divisor for instances (1 by default)
     * @returns the new vertex buffer
     */
    createVertexBuffer(kind: string, offset: number, size: number, stride?: number, instanced?: boolean, useBytes?: boolean, divisor?: number): VertexBuffer;
    /**
     * Gets a boolean indicating if the Buffer is updatable?
     * @returns true if the buffer is updatable
     */
    isUpdatable(): boolean;
    /**
     * Gets current buffer's data
     * @returns a DataArray or null
     */
    getData(): Nullable<DataArray>;
    /**
     * Gets underlying native buffer
     * @returns underlying native buffer
     */
    getBuffer(): Nullable<DataBuffer>;
    /**
     * Gets the stride in float32 units (i.e. byte stride / 4).
     * May not be an integer if the byte stride is not divisible by 4.
     * @returns the stride in float32 units
     * @deprecated Please use byteStride instead.
     */
    getStrideSize(): number;
    /**
     * Store data into the buffer. Creates the buffer if not used already.
     * If the buffer was already used, it will be updated only if it is updatable, otherwise it will do nothing.
     * @param data defines the data to store
     */
    create(data?: Nullable<DataArray>): void;
    /** @internal */
    _rebuild(): void;
    /**
     * Update current buffer data
     * @param data defines the data to store
     */
    update(data: DataArray): void;
    /**
     * Updates the data directly.
     * @param data the new data
     * @param offset the new offset
     * @param vertexCount the vertex count (optional)
     * @param useBytes set to true if the offset is in bytes
     */
    updateDirectly(data: DataArray, offset: number, vertexCount?: number, useBytes?: boolean): void;
    /** @internal */
    _increaseReferences(): void;
    /**
     * Release all resources
     */
    dispose(): void;
}
/**
 * Specialized buffer used to store vertex data
 */
export class VertexBuffer {
    private static _Counter;
    /** @internal */
    _buffer: Buffer;
    /** @internal */
    _validOffsetRange: boolean;
    private _kind;
    private _size;
    private _ownsBuffer;
    private _instanced;
    private _instanceDivisor;
    /**
     * The byte type.
     */
    static readonly BYTE: number;
    /**
     * The unsigned byte type.
     */
    static readonly UNSIGNED_BYTE: number;
    /**
     * The short type.
     */
    static readonly SHORT: number;
    /**
     * The unsigned short type.
     */
    static readonly UNSIGNED_SHORT: number;
    /**
     * The integer type.
     */
    static readonly INT: number;
    /**
     * The unsigned integer type.
     */
    static readonly UNSIGNED_INT: number;
    /**
     * The float type.
     */
    static readonly FLOAT: number;
    /**
     * Gets or sets the instance divisor when in instanced mode
     */
    get instanceDivisor(): number;
    set instanceDivisor(value: number);
    /**
     * Gets the byte stride.
     */
    readonly byteStride: number;
    /**
     * Gets the byte offset.
     */
    readonly byteOffset: number;
    /**
     * Gets whether integer data values should be normalized into a certain range when being casted to a float.
     */
    readonly normalized: boolean;
    /**
     * Gets the data type of each component in the array.
     */
    readonly type: number;
    /**
     * Gets the unique id of this vertex buffer
     */
    readonly uniqueId: number;
    /**
     * Gets a hash code representing the format (type, normalized, size, instanced, stride) of this buffer
     * All buffers with the same format will have the same hash code
     */
    readonly hashCode: number;
    /**
     * Constructor
     * @param engine the engine
     * @param data the data to use for this vertex buffer
     * @param kind the vertex buffer kind
     * @param updatable whether the data is updatable
     * @param postponeInternalCreation whether to postpone creating the internal WebGL buffer (optional)
     * @param stride the stride (optional)
     * @param instanced whether the buffer is instanced (optional)
     * @param offset the offset of the data (optional)
     * @param size the number of components (optional)
     * @param type the type of the component (optional)
     * @param normalized whether the data contains normalized data (optional)
     * @param useBytes set to true if stride and offset are in bytes (optional)
     * @param divisor defines the instance divisor to use (1 by default)
     * @param takeBufferOwnership defines if the buffer should be released when the vertex buffer is disposed
     */
    constructor(engine: any, data: DataArray | Buffer | DataBuffer, kind: string, updatable: boolean, postponeInternalCreation?: boolean, stride?: number, instanced?: boolean, offset?: number, size?: number, type?: number, normalized?: boolean, useBytes?: boolean, divisor?: number, takeBufferOwnership?: boolean);
    private _computeHashCode;
    /** @internal */
    _rebuild(): void;
    /**
     * Returns the kind of the VertexBuffer (string)
     * @returns a string
     */
    getKind(): string;
    /**
     * Gets a boolean indicating if the VertexBuffer is updatable?
     * @returns true if the buffer is updatable
     */
    isUpdatable(): boolean;
    /**
     * Gets current buffer's data
     * @returns a DataArray or null
     */
    getData(): Nullable<DataArray>;
    /**
     * Gets current buffer's data as a float array. Float data is constructed if the vertex buffer data cannot be returned directly.
     * @param totalVertices number of vertices in the buffer to take into account
     * @param forceCopy defines a boolean indicating that the returned array must be cloned upon returning it
     * @returns a float array containing vertex data
     */
    getFloatData(totalVertices: number, forceCopy?: boolean): Nullable<FloatArray>;
    /**
     * Gets underlying native buffer
     * @returns underlying native buffer
     */
    getBuffer(): Nullable<DataBuffer>;
    /**
     * Gets the stride in float32 units (i.e. byte stride / 4).
     * May not be an integer if the byte stride is not divisible by 4.
     * @returns the stride in float32 units
     * @deprecated Please use byteStride instead.
     */
    getStrideSize(): number;
    /**
     * Returns the offset as a multiple of the type byte length.
     * @returns the offset in bytes
     * @deprecated Please use byteOffset instead.
     */
    getOffset(): number;
    /**
     * Returns the number of components or the byte size per vertex attribute
     * @param sizeInBytes If true, returns the size in bytes or else the size in number of components of the vertex attribute (default: false)
     * @returns the number of components
     */
    getSize(sizeInBytes?: boolean): number;
    /**
     * Gets a boolean indicating is the internal buffer of the VertexBuffer is instanced
     * @returns true if this buffer is instanced
     */
    getIsInstanced(): boolean;
    /**
     * Returns the instancing divisor, zero for non-instanced (integer).
     * @returns a number
     */
    getInstanceDivisor(): number;
    /**
     * Store data into the buffer. If the buffer was already used it will be either recreated or updated depending on isUpdatable property
     * @param data defines the data to store
     */
    create(data?: DataArray): void;
    /**
     * Updates the underlying buffer according to the passed numeric array or Float32Array.
     * This function will create a new buffer if the current one is not updatable
     * @param data defines the data to store
     */
    update(data: DataArray): void;
    /**
     * Updates directly the underlying WebGLBuffer according to the passed numeric array or Float32Array.
     * Returns the directly updated WebGLBuffer.
     * @param data the new data
     * @param offset the new offset
     * @param useBytes set to true if the offset is in bytes
     */
    updateDirectly(data: DataArray, offset: number, useBytes?: boolean): void;
    /**
     * Disposes the VertexBuffer and the underlying WebGLBuffer.
     */
    dispose(): void;
    /**
     * Enumerates each value of this vertex buffer as numbers.
     * @param count the number of values to enumerate
     * @param callback the callback function called for each value
     */
    forEach(count: number, callback: (value: number, index: number) => void): void;
    /**
     * Positions
     */
    static readonly PositionKind: string;
    /**
     * Normals
     */
    static readonly NormalKind: string;
    /**
     * Tangents
     */
    static readonly TangentKind: string;
    /**
     * Texture coordinates
     */
    static readonly UVKind: string;
    /**
     * Texture coordinates 2
     */
    static readonly UV2Kind: string;
    /**
     * Texture coordinates 3
     */
    static readonly UV3Kind: string;
    /**
     * Texture coordinates 4
     */
    static readonly UV4Kind: string;
    /**
     * Texture coordinates 5
     */
    static readonly UV5Kind: string;
    /**
     * Texture coordinates 6
     */
    static readonly UV6Kind: string;
    /**
     * Colors
     */
    static readonly ColorKind: string;
    /**
     * Instance Colors
     */
    static readonly ColorInstanceKind: string;
    /**
     * Matrix indices (for bones)
     */
    static readonly MatricesIndicesKind: string;
    /**
     * Matrix weights (for bones)
     */
    static readonly MatricesWeightsKind: string;
    /**
     * Additional matrix indices (for bones)
     */
    static readonly MatricesIndicesExtraKind: string;
    /**
     * Additional matrix weights (for bones)
     */
    static readonly MatricesWeightsExtraKind: string;
    /**
     * Deduces the stride given a kind.
     * @param kind The kind string to deduce
     * @returns The deduced stride
     */
    static DeduceStride(kind: string): number;
    /**
     * Gets the byte length of the given type.
     * @param type the type
     * @returns the number of bytes
     */
    static GetTypeByteLength(type: number): number;
    /**
     * Enumerates each value of the given parameters as numbers.
     * @param data the data to enumerate
     * @param byteOffset the byte offset of the data
     * @param byteStride the byte stride of the data
     * @param componentCount the number of components per element
     * @param componentType the type of the component
     * @param count the number of values to enumerate
     * @param normalized whether the data is normalized
     * @param callback the callback function called for each value
     */
    static ForEach(data: DataArray, byteOffset: number, byteStride: number, componentCount: number, componentType: number, count: number, normalized: boolean, callback: (value: number, index: number) => void): void;
    private static _GetFloatValue;
}

}
declare module "babylonjs/Buffers/dataBuffer" {
/**
 * Class used to store gfx data (like WebGLBuffer)
 */
export class DataBuffer {
    private static _Counter;
    /**
     * Gets or sets the number of objects referencing this buffer
     */
    references: number;
    /** Gets or sets the size of the underlying buffer */
    capacity: number;
    /**
     * Gets or sets a boolean indicating if the buffer contains 32bits indices
     */
    is32Bits: boolean;
    /**
     * Gets the underlying buffer
     */
    get underlyingResource(): any;
    /**
     * Gets the unique id of this buffer
     */
    readonly uniqueId: number;
    /**
     * Constructs the buffer
     */
    constructor();
}

}
declare module "babylonjs/Buffers/index" {
export * from "babylonjs/Buffers/buffer";
export * from "babylonjs/Buffers/dataBuffer";
export * from "babylonjs/Buffers/storageBuffer";

}
declare module "babylonjs/Buffers/storageBuffer" {
import { ThinEngine } from "babylonjs/Engines/thinEngine";
import { DataBuffer } from "babylonjs/Buffers/dataBuffer";
import { DataArray } from "babylonjs/types";
/**
 * This class is a small wrapper around a native buffer that can be read and/or written
 */
export class StorageBuffer {
    private _engine;
    private _buffer;
    private _bufferSize;
    private _creationFlags;
    /**
     * Creates a new storage buffer instance
     * @param engine The engine the buffer will be created inside
     * @param size The size of the buffer in bytes
     * @param creationFlags flags to use when creating the buffer (see Constants.BUFFER_CREATIONFLAG_XXX). The BUFFER_CREATIONFLAG_STORAGE flag will be automatically added.
     */
    constructor(engine: ThinEngine, size: number, creationFlags?: number);
    private _create;
    /** @internal */
    _rebuild(): void;
    /**
     * Gets underlying native buffer
     * @returns underlying native buffer
     */
    getBuffer(): DataBuffer;
    /**
     * Updates the storage buffer
     * @param data the data used to update the storage buffer
     * @param byteOffset the byte offset of the data (optional)
     * @param byteLength the byte length of the data (optional)
     */
    update(data: DataArray, byteOffset?: number, byteLength?: number): void;
    /**
     * Reads data from the storage buffer
     * @param offset The offset in the storage buffer to start reading from (default: 0)
     * @param size  The number of bytes to read from the storage buffer (default: capacity of the buffer)
     * @param buffer The buffer to write the data we have read from the storage buffer to (optional)
     * @returns If not undefined, returns the (promise) buffer (as provided by the 4th parameter) filled with the data, else it returns a (promise) Uint8Array with the data read from the storage buffer
     */
    read(offset?: number, size?: number, buffer?: ArrayBufferView): Promise<ArrayBufferView>;
    /**
     * Disposes the storage buffer
     */
    dispose(): void;
}

}
declare module "babylonjs/Cameras/arcRotateCamera" {
import { Observable } from "babylonjs/Misc/observable";
import { Nullable } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
import { Matrix, Vector3, Vector2 } from "babylonjs/Maths/math.vector";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { AutoRotationBehavior } from "babylonjs/Behaviors/Cameras/autoRotationBehavior";
import { BouncingBehavior } from "babylonjs/Behaviors/Cameras/bouncingBehavior";
import { FramingBehavior } from "babylonjs/Behaviors/Cameras/framingBehavior";
import { Camera } from "babylonjs/Cameras/camera";
import { TargetCamera } from "babylonjs/Cameras/targetCamera";
import { ArcRotateCameraInputsManager } from "babylonjs/Cameras/arcRotateCameraInputsManager";
import { Collider } from "babylonjs/Collisions/collider";
/**
 * This represents an orbital type of camera.
 *
 * This camera always points towards a given target position and can be rotated around that target with the target as the centre of rotation. It can be controlled with cursors and mouse, or with touch events.
 * Think of this camera as one orbiting its target position, or more imaginatively as a spy satellite orbiting the earth. Its position relative to the target (earth) can be set by three parameters, alpha (radians) the longitudinal rotation, beta (radians) the latitudinal rotation and radius the distance from the target position.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_introduction#arc-rotate-camera
 */
export class ArcRotateCamera extends TargetCamera {
    /**
     * Defines the rotation angle of the camera along the longitudinal axis.
     */
    alpha: number;
    /**
     * Defines the rotation angle of the camera along the latitudinal axis.
     */
    beta: number;
    /**
     * Defines the radius of the camera from it s target point.
     */
    radius: number;
    /**
     * Defines an override value to use as the parameter to setTarget.
     * This allows the parameter to be specified when animating the target (e.g. using FramingBehavior).
     */
    overrideCloneAlphaBetaRadius: Nullable<boolean>;
    protected _target: Vector3;
    protected _targetHost: Nullable<AbstractMesh>;
    /**
     * Defines the target point of the camera.
     * The camera looks towards it from the radius distance.
     */
    get target(): Vector3;
    set target(value: Vector3);
    /**
     * Defines the target mesh of the camera.
     * The camera looks towards it from the radius distance.
     * Please note that setting a target host will disable panning.
     */
    get targetHost(): Nullable<AbstractMesh>;
    set targetHost(value: Nullable<AbstractMesh>);
    /**
     * Return the current target position of the camera. This value is expressed in local space.
     * @returns the target position
     */
    getTarget(): Vector3;
    /**
     * Define the current local position of the camera in the scene
     */
    get position(): Vector3;
    set position(newPosition: Vector3);
    protected _upToYMatrix: Matrix;
    protected _yToUpMatrix: Matrix;
    /**
     * The vector the camera should consider as up. (default is Vector3(0, 1, 0) as returned by Vector3.Up())
     * Setting this will copy the given vector to the camera's upVector, and set rotation matrices to and from Y up.
     * DO NOT set the up vector using copyFrom or copyFromFloats, as this bypasses setting the above matrices.
     */
    set upVector(vec: Vector3);
    get upVector(): Vector3;
    /**
     * Sets the Y-up to camera up-vector rotation matrix, and the up-vector to Y-up rotation matrix.
     */
    setMatUp(): void;
    /**
     * Current inertia value on the longitudinal axis.
     * The bigger this number the longer it will take for the camera to stop.
     */
    inertialAlphaOffset: number;
    /**
     * Current inertia value on the latitudinal axis.
     * The bigger this number the longer it will take for the camera to stop.
     */
    inertialBetaOffset: number;
    /**
     * Current inertia value on the radius axis.
     * The bigger this number the longer it will take for the camera to stop.
     */
    inertialRadiusOffset: number;
    /**
     * Minimum allowed angle on the longitudinal axis.
     * This can help limiting how the Camera is able to move in the scene.
     */
    lowerAlphaLimit: Nullable<number>;
    /**
     * Maximum allowed angle on the longitudinal axis.
     * This can help limiting how the Camera is able to move in the scene.
     */
    upperAlphaLimit: Nullable<number>;
    /**
     * Minimum allowed angle on the latitudinal axis.
     * This can help limiting how the Camera is able to move in the scene.
     */
    lowerBetaLimit: Nullable<number>;
    /**
     * Maximum allowed angle on the latitudinal axis.
     * This can help limiting how the Camera is able to move in the scene.
     */
    upperBetaLimit: Nullable<number>;
    /**
     * Minimum allowed distance of the camera to the target (The camera can not get closer).
     * This can help limiting how the Camera is able to move in the scene.
     */
    lowerRadiusLimit: Nullable<number>;
    /**
     * Maximum allowed distance of the camera to the target (The camera can not get further).
     * This can help limiting how the Camera is able to move in the scene.
     */
    upperRadiusLimit: Nullable<number>;
    /**
     * Defines the current inertia value used during panning of the camera along the X axis.
     */
    inertialPanningX: number;
    /**
     * Defines the current inertia value used during panning of the camera along the Y axis.
     */
    inertialPanningY: number;
    /**
     * Defines the distance used to consider the camera in pan mode vs pinch/zoom.
     * Basically if your fingers moves away from more than this distance you will be considered
     * in pinch mode.
     */
    pinchToPanMaxDistance: number;
    /**
     * Defines the maximum distance the camera can pan.
     * This could help keeping the camera always in your scene.
     */
    panningDistanceLimit: Nullable<number>;
    /**
     * Defines the target of the camera before panning.
     */
    panningOriginTarget: Vector3;
    /**
     * Defines the value of the inertia used during panning.
     * 0 would mean stop inertia and one would mean no deceleration at all.
     */
    panningInertia: number;
    /**
     * Gets or Set the pointer angular sensibility  along the X axis or how fast is the camera rotating.
     */
    get angularSensibilityX(): number;
    set angularSensibilityX(value: number);
    /**
     * Gets or Set the pointer angular sensibility along the Y axis or how fast is the camera rotating.
     */
    get angularSensibilityY(): number;
    set angularSensibilityY(value: number);
    /**
     * Gets or Set the pointer pinch precision or how fast is the camera zooming.
     */
    get pinchPrecision(): number;
    set pinchPrecision(value: number);
    /**
     * Gets or Set the pointer pinch delta percentage or how fast is the camera zooming.
     * It will be used instead of pinchDeltaPrecision if different from 0.
     * It defines the percentage of current camera.radius to use as delta when pinch zoom is used.
     */
    get pinchDeltaPercentage(): number;
    set pinchDeltaPercentage(value: number);
    /**
     * Gets or Set the pointer use natural pinch zoom to override the pinch precision
     * and pinch delta percentage.
     * When useNaturalPinchZoom is true, multi touch zoom will zoom in such
     * that any object in the plane at the camera's target point will scale
     * perfectly with finger motion.
     */
    get useNaturalPinchZoom(): boolean;
    set useNaturalPinchZoom(value: boolean);
    /**
     * Gets or Set the pointer panning sensibility or how fast is the camera moving.
     */
    get panningSensibility(): number;
    set panningSensibility(value: number);
    /**
     * Gets or Set the list of keyboard keys used to control beta angle in a positive direction.
     */
    get keysUp(): number[];
    set keysUp(value: number[]);
    /**
     * Gets or Set the list of keyboard keys used to control beta angle in a negative direction.
     */
    get keysDown(): number[];
    set keysDown(value: number[]);
    /**
     * Gets or Set the list of keyboard keys used to control alpha angle in a negative direction.
     */
    get keysLeft(): number[];
    set keysLeft(value: number[]);
    /**
     * Gets or Set the list of keyboard keys used to control alpha angle in a positive direction.
     */
    get keysRight(): number[];
    set keysRight(value: number[]);
    /**
     * Gets or Set the mouse wheel precision or how fast is the camera zooming.
     */
    get wheelPrecision(): number;
    set wheelPrecision(value: number);
    /**
     * Gets or Set the boolean value that controls whether or not the mouse wheel
     * zooms to the location of the mouse pointer or not.  The default is false.
     */
    get zoomToMouseLocation(): boolean;
    set zoomToMouseLocation(value: boolean);
    /**
     * Gets or Set the mouse wheel delta percentage or how fast is the camera zooming.
     * It will be used instead of pinchDeltaPrecision if different from 0.
     * It defines the percentage of current camera.radius to use as delta when pinch zoom is used.
     */
    get wheelDeltaPercentage(): number;
    set wheelDeltaPercentage(value: number);
    /**
     * Defines how much the radius should be scaled while zooming on a particular mesh (through the zoomOn function)
     */
    zoomOnFactor: number;
    /**
     * Defines a screen offset for the camera position.
     */
    targetScreenOffset: Vector2;
    /**
     * Allows the camera to be completely reversed.
     * If false the camera can not arrive upside down.
     */
    allowUpsideDown: boolean;
    /**
     * Define if double tap/click is used to restore the previously saved state of the camera.
     */
    useInputToRestoreState: boolean;
    /** @internal */
    _viewMatrix: Matrix;
    /** @internal */
    _useCtrlForPanning: boolean;
    /** @internal */
    _panningMouseButton: number;
    /**
     * Defines the input associated to the camera.
     */
    inputs: ArcRotateCameraInputsManager;
    /** @internal */
    _reset: () => void;
    /**
     * Defines the allowed panning axis.
     */
    panningAxis: Vector3;
    protected _transformedDirection: Vector3;
    /**
     * Defines if camera will eliminate transform on y axis.
     */
    mapPanning: boolean;
    private _bouncingBehavior;
    /**
     * Gets the bouncing behavior of the camera if it has been enabled.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/behaviors/cameraBehaviors#bouncing-behavior
     */
    get bouncingBehavior(): Nullable<BouncingBehavior>;
    /**
     * Defines if the bouncing behavior of the camera is enabled on the camera.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/behaviors/cameraBehaviors#bouncing-behavior
     */
    get useBouncingBehavior(): boolean;
    set useBouncingBehavior(value: boolean);
    private _framingBehavior;
    /**
     * Gets the framing behavior of the camera if it has been enabled.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/behaviors/cameraBehaviors#framing-behavior
     */
    get framingBehavior(): Nullable<FramingBehavior>;
    /**
     * Defines if the framing behavior of the camera is enabled on the camera.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/behaviors/cameraBehaviors#framing-behavior
     */
    get useFramingBehavior(): boolean;
    set useFramingBehavior(value: boolean);
    private _autoRotationBehavior;
    /**
     * Gets the auto rotation behavior of the camera if it has been enabled.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/behaviors/cameraBehaviors#autorotation-behavior
     */
    get autoRotationBehavior(): Nullable<AutoRotationBehavior>;
    /**
     * Defines if the auto rotation behavior of the camera is enabled on the camera.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/behaviors/cameraBehaviors#autorotation-behavior
     */
    get useAutoRotationBehavior(): boolean;
    set useAutoRotationBehavior(value: boolean);
    /**
     * Observable triggered when the mesh target has been changed on the camera.
     */
    onMeshTargetChangedObservable: Observable<Nullable<AbstractMesh>>;
    /**
     * Event raised when the camera is colliding with a mesh.
     */
    onCollide: (collidedMesh: AbstractMesh) => void;
    /**
     * Defines whether the camera should check collision with the objects oh the scene.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_collisions#how-can-i-do-this-
     */
    checkCollisions: boolean;
    /**
     * Defines the collision radius of the camera.
     * This simulates a sphere around the camera.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_collisions#arcrotatecamera
     */
    collisionRadius: Vector3;
    protected _collider: Collider;
    protected _previousPosition: Vector3;
    protected _collisionVelocity: Vector3;
    protected _newPosition: Vector3;
    protected _previousAlpha: number;
    protected _previousBeta: number;
    protected _previousRadius: number;
    protected _collisionTriggered: boolean;
    protected _targetBoundingCenter: Nullable<Vector3>;
    private _computationVector;
    /**
     * Instantiates a new ArcRotateCamera in a given scene
     * @param name Defines the name of the camera
     * @param alpha Defines the camera rotation along the longitudinal axis
     * @param beta Defines the camera rotation along the latitudinal axis
     * @param radius Defines the camera distance from its target
     * @param target Defines the camera target
     * @param scene Defines the scene the camera belongs to
     * @param setActiveOnSceneIfNoneActive Defines whether the camera should be marked as active if not other active cameras have been defined
     */
    constructor(name: string, alpha: number, beta: number, radius: number, target: Vector3, scene?: Scene, setActiveOnSceneIfNoneActive?: boolean);
    /** @internal */
    _initCache(): void;
    /**
     * @internal
     */
    _updateCache(ignoreParentClass?: boolean): void;
    protected _getTargetPosition(): Vector3;
    private _storedAlpha;
    private _storedBeta;
    private _storedRadius;
    private _storedTarget;
    private _storedTargetScreenOffset;
    /**
     * Stores the current state of the camera (alpha, beta, radius and target)
     * @returns the camera itself
     */
    storeState(): Camera;
    /**
     * @internal
     * Restored camera state. You must call storeState() first
     */
    _restoreStateValues(): boolean;
    /** @internal */
    _isSynchronizedViewMatrix(): boolean;
    /**
     * Attach the input controls to a specific dom element to get the input from.
     * @param noPreventDefault Defines whether event caught by the controls should call preventdefault() (https://developer.mozilla.org/en-US/docs/Web/API/Event/preventDefault)
     */
    attachControl(noPreventDefault?: boolean): void;
    /**
     * Attach the input controls to a specific dom element to get the input from.
     * @param ignored defines an ignored parameter kept for backward compatibility.
     * @param noPreventDefault Defines whether event caught by the controls should call preventdefault() (https://developer.mozilla.org/en-US/docs/Web/API/Event/preventDefault)
     */
    attachControl(ignored: any, noPreventDefault?: boolean): void;
    /**
     * Attached controls to the current camera.
     * @param noPreventDefault Defines whether event caught by the controls should call preventdefault() (https://developer.mozilla.org/en-US/docs/Web/API/Event/preventDefault)
     * @param useCtrlForPanning  Defines whether ctrl is used for panning within the controls
     */
    attachControl(noPreventDefault: boolean, useCtrlForPanning: boolean): void;
    /**
     * Attached controls to the current camera.
     * @param ignored defines an ignored parameter kept for backward compatibility.
     * @param noPreventDefault Defines whether event caught by the controls should call preventdefault() (https://developer.mozilla.org/en-US/docs/Web/API/Event/preventDefault)
     * @param useCtrlForPanning  Defines whether ctrl is used for panning within the controls
     */
    attachControl(ignored: any, noPreventDefault: boolean, useCtrlForPanning: boolean): void;
    /**
     * Attached controls to the current camera.
     * @param noPreventDefault Defines whether event caught by the controls should call preventdefault() (https://developer.mozilla.org/en-US/docs/Web/API/Event/preventDefault)
     * @param useCtrlForPanning  Defines whether ctrl is used for panning within the controls
     * @param panningMouseButton Defines whether panning is allowed through mouse click button
     */
    attachControl(noPreventDefault: boolean, useCtrlForPanning: boolean, panningMouseButton: number): void;
    /**
     * Detach the current controls from the specified dom element.
     */
    detachControl(): void;
    /** @internal */
    _checkInputs(): void;
    protected _checkLimits(): void;
    /**
     * Rebuilds angles (alpha, beta) and radius from the give position and target
     */
    rebuildAnglesAndRadius(): void;
    /**
     * Use a position to define the current camera related information like alpha, beta and radius
     * @param position Defines the position to set the camera at
     */
    setPosition(position: Vector3): void;
    /**
     * Defines the target the camera should look at.
     * This will automatically adapt alpha beta and radius to fit within the new target.
     * Please note that setting a target as a mesh will disable panning.
     * @param target Defines the new target as a Vector or a mesh
     * @param toBoundingCenter In case of a mesh target, defines whether to target the mesh position or its bounding information center
     * @param allowSamePosition If false, prevents reapplying the new computed position if it is identical to the current one (optim)
     * @param cloneAlphaBetaRadius If true, replicate the current setup (alpha, beta, radius) on the new target
     */
    setTarget(target: AbstractMesh | Vector3, toBoundingCenter?: boolean, allowSamePosition?: boolean, cloneAlphaBetaRadius?: boolean): void;
    /** @internal */
    _getViewMatrix(): Matrix;
    protected _onCollisionPositionChange: (collisionId: number, newPosition: Vector3, collidedMesh?: Nullable<AbstractMesh>) => void;
    /**
     * Zooms on a mesh to be at the min distance where we could see it fully in the current viewport.
     * @param meshes Defines the mesh to zoom on
     * @param doNotUpdateMaxZ Defines whether or not maxZ should be updated whilst zooming on the mesh (this can happen if the mesh is big and the maxradius pretty small for instance)
     */
    zoomOn(meshes?: AbstractMesh[], doNotUpdateMaxZ?: boolean): void;
    /**
     * Focus on a mesh or a bounding box. This adapts the target and maxRadius if necessary but does not update the current radius.
     * The target will be changed but the radius
     * @param meshesOrMinMaxVectorAndDistance Defines the mesh or bounding info to focus on
     * @param doNotUpdateMaxZ Defines whether or not maxZ should be updated whilst zooming on the mesh (this can happen if the mesh is big and the maxradius pretty small for instance)
     */
    focusOn(meshesOrMinMaxVectorAndDistance: AbstractMesh[] | {
        min: Vector3;
        max: Vector3;
        distance: number;
    }, doNotUpdateMaxZ?: boolean): void;
    /**
     * @override
     * Override Camera.createRigCamera
     */
    createRigCamera(name: string, cameraIndex: number): Camera;
    /**
     * @internal
     * @override
     * Override Camera._updateRigCameras
     */
    _updateRigCameras(): void;
    /**
     * Destroy the camera and release the current resources hold by it.
     */
    dispose(): void;
    /**
     * Gets the current object class name.
     * @returns the class name
     */
    getClassName(): string;
}
export {};

}
declare module "babylonjs/Cameras/arcRotateCameraInputsManager" {
import { ArcRotateCamera } from "babylonjs/Cameras/arcRotateCamera";
import { CameraInputsManager } from "babylonjs/Cameras/cameraInputsManager";
/**
 * Default Inputs manager for the ArcRotateCamera.
 * It groups all the default supported inputs for ease of use.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/customizingCameraInputs
 */
export class ArcRotateCameraInputsManager extends CameraInputsManager<ArcRotateCamera> {
    /**
     * Instantiates a new ArcRotateCameraInputsManager.
     * @param camera Defines the camera the inputs belong to
     */
    constructor(camera: ArcRotateCamera);
    /**
     * Add mouse wheel input support to the input manager.
     * @returns the current input manager
     */
    addMouseWheel(): ArcRotateCameraInputsManager;
    /**
     * Add pointers input support to the input manager.
     * @returns the current input manager
     */
    addPointers(): ArcRotateCameraInputsManager;
    /**
     * Add keyboard input support to the input manager.
     * @returns the current input manager
     */
    addKeyboard(): ArcRotateCameraInputsManager;
}

}
declare module "babylonjs/Cameras/camera" {
import { SmartArray } from "babylonjs/Misc/smartArray";
import { Observable } from "babylonjs/Misc/observable";
import { Nullable } from "babylonjs/types";
import { CameraInputsManager } from "babylonjs/Cameras/cameraInputsManager";
import { Scene } from "babylonjs/scene";
import { Matrix, Vector3, Quaternion } from "babylonjs/Maths/math.vector";
import { Node } from "babylonjs/node";
import { Mesh } from "babylonjs/Meshes/mesh";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { ICullable } from "babylonjs/Culling/boundingInfo";
import { Viewport } from "babylonjs/Maths/math.viewport";
import { PostProcess } from "babylonjs/PostProcesses/postProcess";
import { RenderTargetTexture } from "babylonjs/Materials/Textures/renderTargetTexture";
import { FreeCamera } from "babylonjs/Cameras/freeCamera";
import { Ray } from "babylonjs/Culling/ray";
/**
 * This is the base class of all the camera used in the application.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras
 */
export class Camera extends Node {
    /**
     * @internal
     */
    static _CreateDefaultParsedCamera: (name: string, scene: Scene) => Camera;
    /**
     * This is the default projection mode used by the cameras.
     * It helps recreating a feeling of perspective and better appreciate depth.
     * This is the best way to simulate real life cameras.
     */
    static readonly PERSPECTIVE_CAMERA: number;
    /**
     * This helps creating camera with an orthographic mode.
     * Orthographic is commonly used in engineering as a means to produce object specifications that communicate dimensions unambiguously, each line of 1 unit length (cm, meter..whatever) will appear to have the same length everywhere on the drawing. This allows the drafter to dimension only a subset of lines and let the reader know that other lines of that length on the drawing are also that length in reality. Every parallel line in the drawing is also parallel in the object.
     */
    static readonly ORTHOGRAPHIC_CAMERA: number;
    /**
     * This is the default FOV mode for perspective cameras.
     * This setting aligns the upper and lower bounds of the viewport to the upper and lower bounds of the camera frustum.
     */
    static readonly FOVMODE_VERTICAL_FIXED: number;
    /**
     * This setting aligns the left and right bounds of the viewport to the left and right bounds of the camera frustum.
     */
    static readonly FOVMODE_HORIZONTAL_FIXED: number;
    /**
     * This specifies there is no need for a camera rig.
     * Basically only one eye is rendered corresponding to the camera.
     */
    static readonly RIG_MODE_NONE: number;
    /**
     * Simulates a camera Rig with one blue eye and one red eye.
     * This can be use with 3d blue and red glasses.
     */
    static readonly RIG_MODE_STEREOSCOPIC_ANAGLYPH: number;
    /**
     * Defines that both eyes of the camera will be rendered side by side with a parallel target.
     */
    static readonly RIG_MODE_STEREOSCOPIC_SIDEBYSIDE_PARALLEL: number;
    /**
     * Defines that both eyes of the camera will be rendered side by side with a none parallel target.
     */
    static readonly RIG_MODE_STEREOSCOPIC_SIDEBYSIDE_CROSSEYED: number;
    /**
     * Defines that both eyes of the camera will be rendered over under each other.
     */
    static readonly RIG_MODE_STEREOSCOPIC_OVERUNDER: number;
    /**
     * Defines that both eyes of the camera will be rendered on successive lines interlaced for passive 3d monitors.
     */
    static readonly RIG_MODE_STEREOSCOPIC_INTERLACED: number;
    /**
     * Defines that both eyes of the camera should be renderered in a VR mode (carbox).
     */
    static readonly RIG_MODE_VR: number;
    /**
     * Defines that both eyes of the camera should be renderered in a VR mode (webVR).
     */
    static readonly RIG_MODE_WEBVR: number;
    /**
     * Custom rig mode allowing rig cameras to be populated manually with any number of cameras
     */
    static readonly RIG_MODE_CUSTOM: number;
    /**
     * Defines if by default attaching controls should prevent the default javascript event to continue.
     */
    static ForceAttachControlToAlwaysPreventDefault: boolean;
    /**
     * Define the input manager associated with the camera.
     */
    inputs: CameraInputsManager<Camera>;
    /** @internal */
    _position: Vector3;
    /**
     * Define the current local position of the camera in the scene
     */
    get position(): Vector3;
    set position(newPosition: Vector3);
    protected _upVector: Vector3;
    /**
     * The vector the camera should consider as up.
     * (default is Vector3(0, 1, 0) aka Vector3.Up())
     */
    set upVector(vec: Vector3);
    get upVector(): Vector3;
    /**
     * The screen area in scene units squared
     */
    get screenArea(): number;
    /**
     * Define the current limit on the left side for an orthographic camera
     * In scene unit
     */
    private _orthoLeft;
    set orthoLeft(value: Nullable<number>);
    get orthoLeft(): Nullable<number>;
    /**
     * Define the current limit on the right side for an orthographic camera
     * In scene unit
     */
    private _orthoRight;
    set orthoRight(value: Nullable<number>);
    get orthoRight(): Nullable<number>;
    /**
     * Define the current limit on the bottom side for an orthographic camera
     * In scene unit
     */
    private _orthoBottom;
    set orthoBottom(value: Nullable<number>);
    get orthoBottom(): Nullable<number>;
    /**
     * Define the current limit on the top side for an orthographic camera
     * In scene unit
     */
    private _orthoTop;
    set orthoTop(value: Nullable<number>);
    get orthoTop(): Nullable<number>;
    /**
     * Field Of View is set in Radians. (default is 0.8)
     */
    fov: number;
    /**
     * Projection plane tilt around the X axis (horizontal), set in Radians. (default is 0)
     * Can be used to make vertical lines in world space actually vertical on the screen.
     * See https://forum.babylonjs.com/t/add-vertical-shift-to-3ds-max-exporter-babylon-cameras/17480
     */
    projectionPlaneTilt: number;
    /**
     * Define the minimum distance the camera can see from.
     * This is important to note that the depth buffer are not infinite and the closer it starts
     * the more your scene might encounter depth fighting issue.
     */
    minZ: number;
    /**
     * Define the maximum distance the camera can see to.
     * This is important to note that the depth buffer are not infinite and the further it end
     * the more your scene might encounter depth fighting issue.
     */
    maxZ: number;
    /**
     * Define the default inertia of the camera.
     * This helps giving a smooth feeling to the camera movement.
     */
    inertia: number;
    /**
     * Define the mode of the camera (Camera.PERSPECTIVE_CAMERA or Camera.ORTHOGRAPHIC_CAMERA)
     */
    private _mode;
    set mode(mode: number);
    get mode(): number;
    /**
     * Define whether the camera is intermediate.
     * This is useful to not present the output directly to the screen in case of rig without post process for instance
     */
    isIntermediate: boolean;
    /**
     * Define the viewport of the camera.
     * This correspond to the portion of the screen the camera will render to in normalized 0 to 1 unit.
     */
    viewport: Viewport;
    /**
     * Restricts the camera to viewing objects with the same layerMask.
     * A camera with a layerMask of 1 will render mesh.layerMask & camera.layerMask!== 0
     */
    layerMask: number;
    /**
     * fovMode sets the camera frustum bounds to the viewport bounds. (default is FOVMODE_VERTICAL_FIXED)
     */
    fovMode: number;
    /**
     * Rig mode of the camera.
     * This is useful to create the camera with two "eyes" instead of one to create VR or stereoscopic scenes.
     * This is normally controlled byt the camera themselves as internal use.
     */
    cameraRigMode: number;
    /**
     * Defines the distance between both "eyes" in case of a RIG
     */
    interaxialDistance: number;
    /**
     * Defines if stereoscopic rendering is done side by side or over under.
     */
    isStereoscopicSideBySide: boolean;
    /**
     * Defines the list of custom render target which are rendered to and then used as the input to this camera's render. Eg. display another camera view on a TV in the main scene
     * This is pretty helpful if you wish to make a camera render to a texture you could reuse somewhere
     * else in the scene. (Eg. security camera)
     *
     * To change the final output target of the camera, camera.outputRenderTarget should be used instead (eg. webXR renders to a render target corresponding to an HMD)
     */
    customRenderTargets: import("babylonjs/Materials/Textures/renderTargetTexture").RenderTargetTexture[];
    /**
     * When set, the camera will render to this render target instead of the default canvas
     *
     * If the desire is to use the output of a camera as a texture in the scene consider using camera.customRenderTargets instead
     */
    outputRenderTarget: Nullable<RenderTargetTexture>;
    /**
     * Observable triggered when the camera view matrix has changed.
     */
    onViewMatrixChangedObservable: Observable<Camera>;
    /**
     * Observable triggered when the camera Projection matrix has changed.
     */
    onProjectionMatrixChangedObservable: Observable<Camera>;
    /**
     * Observable triggered when the inputs have been processed.
     */
    onAfterCheckInputsObservable: Observable<Camera>;
    /**
     * Observable triggered when reset has been called and applied to the camera.
     */
    onRestoreStateObservable: Observable<Camera>;
    /**
     * Is this camera a part of a rig system?
     */
    isRigCamera: boolean;
    /**
     * If isRigCamera set to true this will be set with the parent camera.
     * The parent camera is not (!) necessarily the .parent of this camera (like in the case of XR)
     */
    rigParent?: Camera;
    /**
     * Render pass id used by the camera to render into the main framebuffer
     */
    renderPassId: number;
    /** @internal */
    _cameraRigParams: any;
    /** @internal */
    _rigCameras: Camera[];
    /** @internal */
    _rigPostProcess: Nullable<PostProcess>;
    protected _webvrViewMatrix: Matrix;
    /** @internal */
    _skipRendering: boolean;
    /** @internal */
    _projectionMatrix: Matrix;
    /** @internal */
    _postProcesses: Nullable<import("babylonjs/PostProcesses/postProcess").PostProcess>[];
    /** @internal */
    _activeMeshes: SmartArray<AbstractMesh>;
    protected _globalPosition: Vector3;
    /** @internal */
    _computedViewMatrix: Matrix;
    private _doNotComputeProjectionMatrix;
    private _transformMatrix;
    private _frustumPlanes;
    private _refreshFrustumPlanes;
    private _storedFov;
    private _stateStored;
    private _absoluteRotation;
    /**
     * Instantiates a new camera object.
     * This should not be used directly but through the inherited cameras: ArcRotate, Free...
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras
     * @param name Defines the name of the camera in the scene
     * @param position Defines the position of the camera
     * @param scene Defines the scene the camera belongs too
     * @param setActiveOnSceneIfNoneActive Defines if the camera should be set as active after creation if no other camera have been defined in the scene
     */
    constructor(name: string, position: Vector3, scene?: Scene, setActiveOnSceneIfNoneActive?: boolean);
    /**
     * Store current camera state (fov, position, etc..)
     * @returns the camera
     */
    storeState(): Camera;
    /**
     * Restores the camera state values if it has been stored. You must call storeState() first
     */
    protected _restoreStateValues(): boolean;
    /**
     * Restored camera state. You must call storeState() first.
     * @returns true if restored and false otherwise
     */
    restoreState(): boolean;
    /**
     * Gets the class name of the camera.
     * @returns the class name
     */
    getClassName(): string;
    /** @internal */
    readonly _isCamera: boolean;
    /**
     * Gets a string representation of the camera useful for debug purpose.
     * @param fullDetails Defines that a more verbose level of logging is required
     * @returns the string representation
     */
    toString(fullDetails?: boolean): string;
    /**
     * Automatically tilts the projection plane, using `projectionPlaneTilt`, to correct the perspective effect on vertical lines.
     */
    applyVerticalCorrection(): void;
    /**
     * Gets the current world space position of the camera.
     */
    get globalPosition(): Vector3;
    /**
     * Gets the list of active meshes this frame (meshes no culled or excluded by lod s in the frame)
     * @returns the active meshe list
     */
    getActiveMeshes(): SmartArray<AbstractMesh>;
    /**
     * Check whether a mesh is part of the current active mesh list of the camera
     * @param mesh Defines the mesh to check
     * @returns true if active, false otherwise
     */
    isActiveMesh(mesh: Mesh): boolean;
    /**
     * Is this camera ready to be used/rendered
     * @param completeCheck defines if a complete check (including post processes) has to be done (false by default)
     * @returns true if the camera is ready
     */
    isReady(completeCheck?: boolean): boolean;
    /** @internal */
    _initCache(): void;
    /**
     * @internal
     */
    _updateCache(ignoreParentClass?: boolean): void;
    /** @internal */
    _isSynchronized(): boolean;
    /** @internal */
    _isSynchronizedViewMatrix(): boolean;
    /** @internal */
    _isSynchronizedProjectionMatrix(): boolean;
    /**
     * Attach the input controls to a specific dom element to get the input from.
     * @param noPreventDefault Defines whether event caught by the controls should call preventdefault() (https://developer.mozilla.org/en-US/docs/Web/API/Event/preventDefault)
     */
    attachControl(noPreventDefault?: boolean): void;
    /**
     * Attach the input controls to a specific dom element to get the input from.
     * @param ignored defines an ignored parameter kept for backward compatibility.
     * @param noPreventDefault Defines whether event caught by the controls should call preventdefault() (https://developer.mozilla.org/en-US/docs/Web/API/Event/preventDefault)
     * BACK COMPAT SIGNATURE ONLY.
     */
    attachControl(ignored: any, noPreventDefault?: boolean): void;
    /**
     * Detach the current controls from the specified dom element.
     */
    detachControl(): void;
    /**
     * Detach the current controls from the specified dom element.
     * @param ignored defines an ignored parameter kept for backward compatibility.
     */
    detachControl(ignored?: any): void;
    /**
     * Update the camera state according to the different inputs gathered during the frame.
     */
    update(): void;
    /** @internal */
    _checkInputs(): void;
    /** @internal */
    get rigCameras(): Camera[];
    /**
     * Gets the post process used by the rig cameras
     */
    get rigPostProcess(): Nullable<PostProcess>;
    /**
     * Internal, gets the first post process.
     * @returns the first post process to be run on this camera.
     */
    _getFirstPostProcess(): Nullable<PostProcess>;
    private _cascadePostProcessesToRigCams;
    /**
     * Attach a post process to the camera.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/postProcesses/usePostProcesses#attach-postprocess
     * @param postProcess The post process to attach to the camera
     * @param insertAt The position of the post process in case several of them are in use in the scene
     * @returns the position the post process has been inserted at
     */
    attachPostProcess(postProcess: PostProcess, insertAt?: Nullable<number>): number;
    /**
     * Detach a post process to the camera.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/postProcesses/usePostProcesses#attach-postprocess
     * @param postProcess The post process to detach from the camera
     */
    detachPostProcess(postProcess: PostProcess): void;
    /**
     * Gets the current world matrix of the camera
     */
    getWorldMatrix(): Matrix;
    /** @internal */
    _getViewMatrix(): Matrix;
    /**
     * Gets the current view matrix of the camera.
     * @param force forces the camera to recompute the matrix without looking at the cached state
     * @returns the view matrix
     */
    getViewMatrix(force?: boolean): Matrix;
    /**
     * Freeze the projection matrix.
     * It will prevent the cache check of the camera projection compute and can speed up perf
     * if no parameter of the camera are meant to change
     * @param projection Defines manually a projection if necessary
     */
    freezeProjectionMatrix(projection?: Matrix): void;
    /**
     * Unfreeze the projection matrix if it has previously been freezed by freezeProjectionMatrix.
     */
    unfreezeProjectionMatrix(): void;
    /**
     * Gets the current projection matrix of the camera.
     * @param force forces the camera to recompute the matrix without looking at the cached state
     * @returns the projection matrix
     */
    getProjectionMatrix(force?: boolean): Matrix;
    /**
     * Gets the transformation matrix (ie. the multiplication of view by projection matrices)
     * @returns a Matrix
     */
    getTransformationMatrix(): Matrix;
    private _updateFrustumPlanes;
    /**
     * Checks if a cullable object (mesh...) is in the camera frustum
     * This checks the bounding box center. See isCompletelyInFrustum for a full bounding check
     * @param target The object to check
     * @param checkRigCameras If the rig cameras should be checked (eg. with webVR camera both eyes should be checked) (Default: false)
     * @returns true if the object is in frustum otherwise false
     */
    isInFrustum(target: ICullable, checkRigCameras?: boolean): boolean;
    /**
     * Checks if a cullable object (mesh...) is in the camera frustum
     * Unlike isInFrustum this checks the full bounding box
     * @param target The object to check
     * @returns true if the object is in frustum otherwise false
     */
    isCompletelyInFrustum(target: ICullable): boolean;
    /**
     * Gets a ray in the forward direction from the camera.
     * @param length Defines the length of the ray to create
     * @param transform Defines the transform to apply to the ray, by default the world matrix is used to create a workd space ray
     * @param origin Defines the start point of the ray which defaults to the camera position
     * @returns the forward ray
     */
    getForwardRay(length?: number, transform?: Matrix, origin?: Vector3): Ray;
    /**
     * Gets a ray in the forward direction from the camera.
     * @param refRay the ray to (re)use when setting the values
     * @param length Defines the length of the ray to create
     * @param transform Defines the transform to apply to the ray, by default the world matrx is used to create a workd space ray
     * @param origin Defines the start point of the ray which defaults to the camera position
     * @returns the forward ray
     */
    getForwardRayToRef(refRay: Ray, length?: number, transform?: Matrix, origin?: Vector3): Ray;
    /**
     * Releases resources associated with this node.
     * @param doNotRecurse Set to true to not recurse into each children (recurse into each children by default)
     * @param disposeMaterialAndTextures Set to true to also dispose referenced materials and textures (false by default)
     */
    dispose(doNotRecurse?: boolean, disposeMaterialAndTextures?: boolean): void;
    /** @internal */
    _isLeftCamera: boolean;
    /**
     * Gets the left camera of a rig setup in case of Rigged Camera
     */
    get isLeftCamera(): boolean;
    /** @internal */
    _isRightCamera: boolean;
    /**
     * Gets the right camera of a rig setup in case of Rigged Camera
     */
    get isRightCamera(): boolean;
    /**
     * Gets the left camera of a rig setup in case of Rigged Camera
     */
    get leftCamera(): Nullable<FreeCamera>;
    /**
     * Gets the right camera of a rig setup in case of Rigged Camera
     */
    get rightCamera(): Nullable<FreeCamera>;
    /**
     * Gets the left camera target of a rig setup in case of Rigged Camera
     * @returns the target position
     */
    getLeftTarget(): Nullable<Vector3>;
    /**
     * Gets the right camera target of a rig setup in case of Rigged Camera
     * @returns the target position
     */
    getRightTarget(): Nullable<Vector3>;
    /**
     * @internal
     */
    setCameraRigMode(mode: number, rigParams: any): void;
    protected _setRigMode(rigParams: any): void;
    /** @internal */
    _getVRProjectionMatrix(): Matrix;
    protected _updateCameraRotationMatrix(): void;
    protected _updateWebVRCameraRotationMatrix(): void;
    /**
     * This function MUST be overwritten by the different WebVR cameras available.
     * The context in which it is running is the RIG camera. So 'this' is the TargetCamera, left or right.
     * @internal
     */
    _getWebVRProjectionMatrix(): Matrix;
    /**
     * This function MUST be overwritten by the different WebVR cameras available.
     * The context in which it is running is the RIG camera. So 'this' is the TargetCamera, left or right.
     * @internal
     */
    _getWebVRViewMatrix(): Matrix;
    /**
     * @internal
     */
    setCameraRigParameter(name: string, value: any): void;
    /**
     * needs to be overridden by children so sub has required properties to be copied
     * @internal
     */
    createRigCamera(name: string, cameraIndex: number): Nullable<Camera>;
    /**
     * May need to be overridden by children
     * @internal
     */
    _updateRigCameras(): void;
    /** @internal */
    _setupInputs(): void;
    /**
     * Serialiaze the camera setup to a json representation
     * @returns the JSON representation
     */
    serialize(): any;
    /**
     * Clones the current camera.
     * @param name The cloned camera name
     * @param newParent The cloned camera's new parent (none by default)
     * @returns the cloned camera
     */
    clone(name: string, newParent?: Nullable<Node>): Camera;
    /**
     * Gets the direction of the camera relative to a given local axis.
     * @param localAxis Defines the reference axis to provide a relative direction.
     * @returns the direction
     */
    getDirection(localAxis: Vector3): Vector3;
    /**
     * Returns the current camera absolute rotation
     */
    get absoluteRotation(): Quaternion;
    /**
     * Gets the direction of the camera relative to a given local axis into a passed vector.
     * @param localAxis Defines the reference axis to provide a relative direction.
     * @param result Defines the vector to store the result in
     */
    getDirectionToRef(localAxis: Vector3, result: Vector3): void;
    /**
     * Gets a camera constructor for a given camera type
     * @param type The type of the camera to construct (should be equal to one of the camera class name)
     * @param name The name of the camera the result will be able to instantiate
     * @param scene The scene the result will construct the camera in
     * @param interaxial_distance In case of stereoscopic setup, the distance between both eyes
     * @param isStereoscopicSideBySide In case of stereoscopic setup, should the sereo be side b side
     * @returns a factory method to construct the camera
     */
    static GetConstructorFromName(type: string, name: string, scene: Scene, interaxial_distance?: number, isStereoscopicSideBySide?: boolean): () => Camera;
    /**
     * Compute the world  matrix of the camera.
     * @returns the camera world matrix
     */
    computeWorldMatrix(): Matrix;
    /**
     * Parse a JSON and creates the camera from the parsed information
     * @param parsedCamera The JSON to parse
     * @param scene The scene to instantiate the camera in
     * @returns the newly constructed camera
     */
    static Parse(parsedCamera: any, scene: Scene): Camera;
}
export {};

}
declare module "babylonjs/Cameras/cameraInputsManager" {
import { Nullable } from "babylonjs/types";
import { Camera } from "babylonjs/Cameras/camera";
/**
 * @ignore
 * This is a list of all the different input types that are available in the application.
 * Fo instance: ArcRotateCameraGamepadInput...
 */
export var CameraInputTypes: {};
/**
 * This is the contract to implement in order to create a new input class.
 * Inputs are dealing with listening to user actions and moving the camera accordingly.
 */
export interface ICameraInput<TCamera extends Camera> {
    /**
     * Defines the camera the input is attached to.
     */
    camera: Nullable<TCamera>;
    /**
     * Gets the class name of the current input.
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Get the friendly name associated with the input class.
     * @returns the input friendly name
     */
    getSimpleName(): string;
    /**
     * Attach the input controls to a specific dom element to get the input from.
     * @param noPreventDefault Defines whether event caught by the controls should call preventdefault() (https://developer.mozilla.org/en-US/docs/Web/API/Event/preventDefault)
     */
    attachControl(noPreventDefault?: boolean): void;
    /**
     * Detach the current controls from the specified dom element.
     */
    detachControl(): void;
    /**
     * Update the current camera state depending on the inputs that have been used this frame.
     * This is a dynamically created lambda to avoid the performance penalty of looping for inputs in the render loop.
     */
    checkInputs?: () => void;
}
/**
 * Represents a map of input types to input instance or input index to input instance.
 */
export interface CameraInputsMap<TCamera extends Camera> {
    /**
     * Accessor to the input by input type.
     */
    [name: string]: ICameraInput<TCamera>;
    /**
     * Accessor to the input by input index.
     */
    [idx: number]: ICameraInput<TCamera>;
}
/**
 * This represents the input manager used within a camera.
 * It helps dealing with all the different kind of input attached to a camera.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/customizingCameraInputs
 */
export class CameraInputsManager<TCamera extends Camera> {
    /**
     * Defines the list of inputs attached to the camera.
     */
    attached: CameraInputsMap<TCamera>;
    /**
     * Defines the dom element the camera is collecting inputs from.
     * This is null if the controls have not been attached.
     */
    attachedToElement: boolean;
    /**
     * Defines whether event caught by the controls should call preventdefault() (https://developer.mozilla.org/en-US/docs/Web/API/Event/preventDefault)
     */
    noPreventDefault: boolean;
    /**
     * Defined the camera the input manager belongs to.
     */
    camera: TCamera;
    /**
     * Update the current camera state depending on the inputs that have been used this frame.
     * This is a dynamically created lambda to avoid the performance penalty of looping for inputs in the render loop.
     */
    checkInputs: () => void;
    /**
     * Instantiate a new Camera Input Manager.
     * @param camera Defines the camera the input manager belongs to
     */
    constructor(camera: TCamera);
    /**
     * Add an input method to a camera
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/customizingCameraInputs
     * @param input Camera input method
     */
    add(input: ICameraInput<TCamera>): void;
    /**
     * Remove a specific input method from a camera
     * example: camera.inputs.remove(camera.inputs.attached.mouse);
     * @param inputToRemove camera input method
     */
    remove(inputToRemove: ICameraInput<TCamera>): void;
    /**
     * Remove a specific input type from a camera
     * example: camera.inputs.remove("ArcRotateCameraGamepadInput");
     * @param inputType the type of the input to remove
     */
    removeByType(inputType: string): void;
    private _addCheckInputs;
    /**
     * Attach the input controls to the currently attached dom element to listen the events from.
     * @param input Defines the input to attach
     */
    attachInput(input: ICameraInput<TCamera>): void;
    /**
     * Attach the current manager inputs controls to a specific dom element to listen the events from.
     * @param noPreventDefault Defines whether event caught by the controls should call preventdefault() (https://developer.mozilla.org/en-US/docs/Web/API/Event/preventDefault)
     */
    attachElement(noPreventDefault?: boolean): void;
    /**
     * Detach the current manager inputs controls from a specific dom element.
     * @param disconnect Defines whether the input should be removed from the current list of attached inputs
     */
    detachElement(disconnect?: boolean): void;
    /**
     * Rebuild the dynamic inputCheck function from the current list of
     * defined inputs in the manager.
     */
    rebuildInputCheck(): void;
    /**
     * Remove all attached input methods from a camera
     */
    clear(): void;
    /**
     * Serialize the current input manager attached to a camera.
     * This ensures than once parsed,
     * the input associated to the camera will be identical to the current ones
     * @param serializedCamera Defines the camera serialization JSON the input serialization should write to
     */
    serialize(serializedCamera: any): void;
    /**
     * Parses an input manager serialized JSON to restore the previous list of inputs
     * and states associated to a camera.
     * @param parsedCamera Defines the JSON to parse
     */
    parse(parsedCamera: any): void;
}

}
declare module "babylonjs/Cameras/deviceOrientationCamera" {
import { FreeCamera } from "babylonjs/Cameras/freeCamera";
import { Scene } from "babylonjs/scene";
import { Vector3 } from "babylonjs/Maths/math.vector";
import "babylonjs/Cameras/Inputs/freeCameraDeviceOrientationInput";
import { Axis } from "babylonjs/Maths/math.axis";
/**
 * This is a camera specifically designed to react to device orientation events such as a modern mobile device
 * being tilted forward or back and left or right.
 */
export class DeviceOrientationCamera extends FreeCamera {
    private _initialQuaternion;
    private _quaternionCache;
    private _tmpDragQuaternion;
    private _disablePointerInputWhenUsingDeviceOrientation;
    /**
     * Creates a new device orientation camera
     * @param name The name of the camera
     * @param position The start position camera
     * @param scene The scene the camera belongs to
     */
    constructor(name: string, position: Vector3, scene?: Scene);
    /**
     * Gets or sets a boolean indicating that pointer input must be disabled on first orientation sensor update (Default: true)
     */
    get disablePointerInputWhenUsingDeviceOrientation(): boolean;
    set disablePointerInputWhenUsingDeviceOrientation(value: boolean);
    private _dragFactor;
    /**
     * Enabled turning on the y axis when the orientation sensor is active
     * @param dragFactor the factor that controls the turn speed (default: 1/300)
     */
    enableHorizontalDragging(dragFactor?: number): void;
    /**
     * Gets the current instance class name ("DeviceOrientationCamera").
     * This helps avoiding instanceof at run time.
     * @returns the class name
     */
    getClassName(): string;
    /**
     * @internal
     * Checks and applies the current values of the inputs to the camera. (Internal use only)
     */
    _checkInputs(): void;
    /**
     * Reset the camera to its default orientation on the specified axis only.
     * @param axis The axis to reset
     */
    resetToCurrentRotation(axis?: Axis): void;
}

}
declare module "babylonjs/Cameras/flyCamera" {
import { Scene } from "babylonjs/scene";
import { Quaternion } from "babylonjs/Maths/math.vector";
import { Vector3 } from "babylonjs/Maths/math.vector";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { TargetCamera } from "babylonjs/Cameras/targetCamera";
import { FlyCameraInputsManager } from "babylonjs/Cameras/flyCameraInputsManager";
/**
 * This is a flying camera, designed for 3D movement and rotation in all directions,
 * such as in a 3D Space Shooter or a Flight Simulator.
 */
export class FlyCamera extends TargetCamera {
    /**
     * Define the collision ellipsoid of the camera.
     * This is helpful for simulating a camera body, like a player's body.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_collisions#arcrotatecamera
     */
    ellipsoid: Vector3;
    /**
     * Define an offset for the position of the ellipsoid around the camera.
     * This can be helpful if the camera is attached away from the player's body center,
     * such as at its head.
     */
    ellipsoidOffset: Vector3;
    /**
     * Enable or disable collisions of the camera with the rest of the scene objects.
     */
    checkCollisions: boolean;
    /**
     * Enable or disable gravity on the camera.
     */
    applyGravity: boolean;
    /**
     * Define the current direction the camera is moving to.
     */
    cameraDirection: Vector3;
    /**
     * Define the current local rotation of the camera as a quaternion to prevent Gimbal lock.
     * This overrides and empties cameraRotation.
     */
    rotationQuaternion: Quaternion;
    /**
     * Track Roll to maintain the wanted Rolling when looking around.
     */
    _trackRoll: number;
    /**
     * Slowly correct the Roll to its original value after a Pitch+Yaw rotation.
     */
    rollCorrect: number;
    /**
     * Mimic a banked turn, Rolling the camera when Yawing.
     * It's recommended to use rollCorrect = 10 for faster banking correction.
     */
    bankedTurn: boolean;
    /**
     * Limit in radians for how much Roll banking will add. (Default: 90°)
     */
    bankedTurnLimit: number;
    /**
     * Value of 0 disables the banked Roll.
     * Value of 1 is equal to the Yaw angle in radians.
     */
    bankedTurnMultiplier: number;
    /**
     * The inputs manager loads all the input sources, such as keyboard and mouse.
     */
    inputs: FlyCameraInputsManager;
    /**
     * Gets the input sensibility for mouse input.
     * Higher values reduce sensitivity.
     */
    get angularSensibility(): number;
    /**
     * Sets the input sensibility for a mouse input.
     * Higher values reduce sensitivity.
     */
    set angularSensibility(value: number);
    /**
     * Get the keys for camera movement forward.
     */
    get keysForward(): number[];
    /**
     * Set the keys for camera movement forward.
     */
    set keysForward(value: number[]);
    /**
     * Get the keys for camera movement backward.
     */
    get keysBackward(): number[];
    set keysBackward(value: number[]);
    /**
     * Get the keys for camera movement up.
     */
    get keysUp(): number[];
    /**
     * Set the keys for camera movement up.
     */
    set keysUp(value: number[]);
    /**
     * Get the keys for camera movement down.
     */
    get keysDown(): number[];
    /**
     * Set the keys for camera movement down.
     */
    set keysDown(value: number[]);
    /**
     * Get the keys for camera movement left.
     */
    get keysLeft(): number[];
    /**
     * Set the keys for camera movement left.
     */
    set keysLeft(value: number[]);
    /**
     * Set the keys for camera movement right.
     */
    get keysRight(): number[];
    /**
     * Set the keys for camera movement right.
     */
    set keysRight(value: number[]);
    /**
     * Event raised when the camera collides with a mesh in the scene.
     */
    onCollide: (collidedMesh: AbstractMesh) => void;
    private _collider;
    private _needMoveForGravity;
    private _oldPosition;
    private _diffPosition;
    private _newPosition;
    /** @internal */
    _localDirection: Vector3;
    /** @internal */
    _transformedDirection: Vector3;
    /**
     * Instantiates a FlyCamera.
     * This is a flying camera, designed for 3D movement and rotation in all directions,
     * such as in a 3D Space Shooter or a Flight Simulator.
     * @param name Define the name of the camera in the scene.
     * @param position Define the starting position of the camera in the scene.
     * @param scene Define the scene the camera belongs to.
     * @param setActiveOnSceneIfNoneActive Defines whether the camera should be marked as active, if no other camera has been defined as active.
     */
    constructor(name: string, position: Vector3, scene?: Scene, setActiveOnSceneIfNoneActive?: boolean);
    /**
     * Attach the input controls to a specific dom element to get the input from.
     * @param noPreventDefault Defines whether event caught by the controls should call preventdefault() (https://developer.mozilla.org/en-US/docs/Web/API/Event/preventDefault)
     */
    attachControl(noPreventDefault?: boolean): void;
    /**
     * Detach a control from the HTML DOM element.
     * The camera will stop reacting to that input.
     */
    detachControl(): void;
    private _collisionMask;
    /**
     * Get the mask that the camera ignores in collision events.
     */
    get collisionMask(): number;
    /**
     * Set the mask that the camera ignores in collision events.
     */
    set collisionMask(mask: number);
    /**
     * @internal
     */
    _collideWithWorld(displacement: Vector3): void;
    /**
     * @internal
     */
    private _onCollisionPositionChange;
    /** @internal */
    _checkInputs(): void;
    /** @internal */
    _decideIfNeedsToMove(): boolean;
    /** @internal */
    _updatePosition(): void;
    /**
     * Restore the Roll to its target value at the rate specified.
     * @param rate - Higher means slower restoring.
     * @internal
     */
    restoreRoll(rate: number): void;
    /**
     * Destroy the camera and release the current resources held by it.
     */
    dispose(): void;
    /**
     * Get the current object class name.
     * @returns the class name.
     */
    getClassName(): string;
}

}
declare module "babylonjs/Cameras/flyCameraInputsManager" {
import { FlyCamera } from "babylonjs/Cameras/flyCamera";
import { CameraInputsManager } from "babylonjs/Cameras/cameraInputsManager";
/**
 * Default Inputs manager for the FlyCamera.
 * It groups all the default supported inputs for ease of use.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/customizingCameraInputs
 */
export class FlyCameraInputsManager extends CameraInputsManager<FlyCamera> {
    /**
     * Instantiates a new FlyCameraInputsManager.
     * @param camera Defines the camera the inputs belong to.
     */
    constructor(camera: FlyCamera);
    /**
     * Add keyboard input support to the input manager.
     * @returns the new FlyCameraKeyboardMoveInput().
     */
    addKeyboard(): FlyCameraInputsManager;
    /**
     * Add mouse input support to the input manager.
     * @returns the new FlyCameraMouseInput().
     */
    addMouse(): FlyCameraInputsManager;
}

}
declare module "babylonjs/Cameras/followCamera" {
import { Nullable } from "babylonjs/types";
import { TargetCamera } from "babylonjs/Cameras/targetCamera";
import { Scene } from "babylonjs/scene";
import { Vector3 } from "babylonjs/Maths/math.vector";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { FollowCameraInputsManager } from "babylonjs/Cameras/followCameraInputsManager";
/**
 * A follow camera takes a mesh as a target and follows it as it moves. Both a free camera version followCamera and
 * an arc rotate version arcFollowCamera are available.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_introduction#followcamera
 */
export class FollowCamera extends TargetCamera {
    /**
     * Distance the follow camera should follow an object at
     */
    radius: number;
    /**
     * Minimum allowed distance of the camera to the axis of rotation
     * (The camera can not get closer).
     * This can help limiting how the Camera is able to move in the scene.
     */
    lowerRadiusLimit: Nullable<number>;
    /**
     * Maximum allowed distance of the camera to the axis of rotation
     * (The camera can not get further).
     * This can help limiting how the Camera is able to move in the scene.
     */
    upperRadiusLimit: Nullable<number>;
    /**
     * Define a rotation offset between the camera and the object it follows
     */
    rotationOffset: number;
    /**
     * Minimum allowed angle to camera position relative to target object.
     * This can help limiting how the Camera is able to move in the scene.
     */
    lowerRotationOffsetLimit: Nullable<number>;
    /**
     * Maximum allowed angle to camera position relative to target object.
     * This can help limiting how the Camera is able to move in the scene.
     */
    upperRotationOffsetLimit: Nullable<number>;
    /**
     * Define a height offset between the camera and the object it follows.
     * It can help following an object from the top (like a car chasing a plane)
     */
    heightOffset: number;
    /**
     * Minimum allowed height of camera position relative to target object.
     * This can help limiting how the Camera is able to move in the scene.
     */
    lowerHeightOffsetLimit: Nullable<number>;
    /**
     * Maximum allowed height of camera position relative to target object.
     * This can help limiting how the Camera is able to move in the scene.
     */
    upperHeightOffsetLimit: Nullable<number>;
    /**
     * Define how fast the camera can accelerate to follow it s target.
     */
    cameraAcceleration: number;
    /**
     * Define the speed limit of the camera following an object.
     */
    maxCameraSpeed: number;
    /**
     * Define the target of the camera.
     */
    lockedTarget: Nullable<AbstractMesh>;
    /**
     * Defines the input associated with the camera.
     */
    inputs: FollowCameraInputsManager;
    /**
     * Instantiates the follow camera.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_introduction#followcamera
     * @param name Define the name of the camera in the scene
     * @param position Define the position of the camera
     * @param scene Define the scene the camera belong to
     * @param lockedTarget Define the target of the camera
     */
    constructor(name: string, position: Vector3, scene?: Scene, lockedTarget?: Nullable<AbstractMesh>);
    private _follow;
    /**
     * Attach the input controls to a specific dom element to get the input from.
     * @param noPreventDefault Defines whether event caught by the controls should call preventdefault() (https://developer.mozilla.org/en-US/docs/Web/API/Event/preventDefault)
     */
    attachControl(noPreventDefault?: boolean): void;
    /**
     * Detach the current controls from the specified dom element.
     */
    detachControl(): void;
    /** @internal */
    _checkInputs(): void;
    private _checkLimits;
    /**
     * Gets the camera class name.
     * @returns the class name
     */
    getClassName(): string;
}
/**
 * Arc Rotate version of the follow camera.
 * It still follows a Defined mesh but in an Arc Rotate Camera fashion.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_introduction#followcamera
 */
export class ArcFollowCamera extends TargetCamera {
    /** The longitudinal angle of the camera */
    alpha: number;
    /** The latitudinal angle of the camera */
    beta: number;
    /** The radius of the camera from its target */
    radius: number;
    private _cartesianCoordinates;
    /** Define the camera target (the mesh it should follow) */
    private _meshTarget;
    /**
     * Instantiates a new ArcFollowCamera
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_introduction#followcamera
     * @param name Define the name of the camera
     * @param alpha Define the rotation angle of the camera around the longitudinal axis
     * @param beta Define the rotation angle of the camera around the elevation axis
     * @param radius Define the radius of the camera from its target point
     * @param target Define the target of the camera
     * @param scene Define the scene the camera belongs to
     */
    constructor(name: string, 
    /** The longitudinal angle of the camera */
    alpha: number, 
    /** The latitudinal angle of the camera */
    beta: number, 
    /** The radius of the camera from its target */
    radius: number, 
    /** Define the camera target (the mesh it should follow) */
    target: Nullable<AbstractMesh>, scene: Scene);
    /**
     * Sets the mesh to follow with this camera.
     * @param target the target to follow
     */
    setMeshTarget(target: Nullable<AbstractMesh>): void;
    private _follow;
    /** @internal */
    _checkInputs(): void;
    /**
     * Returns the class name of the object.
     * It is mostly used internally for serialization purposes.
     */
    getClassName(): string;
}

}
declare module "babylonjs/Cameras/followCameraInputsManager" {
import { CameraInputsManager } from "babylonjs/Cameras/cameraInputsManager";
import { FollowCamera } from "babylonjs/Cameras/followCamera";
/**
 * Default Inputs manager for the FollowCamera.
 * It groups all the default supported inputs for ease of use.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/customizingCameraInputs
 */
export class FollowCameraInputsManager extends CameraInputsManager<FollowCamera> {
    /**
     * Instantiates a new FollowCameraInputsManager.
     * @param camera Defines the camera the inputs belong to
     */
    constructor(camera: FollowCamera);
    /**
     * Add keyboard input support to the input manager.
     * @returns the current input manager
     */
    addKeyboard(): FollowCameraInputsManager;
    /**
     * Add mouse wheel input support to the input manager.
     * @returns the current input manager
     */
    addMouseWheel(): FollowCameraInputsManager;
    /**
     * Add pointers input support to the input manager.
     * @returns the current input manager
     */
    addPointers(): FollowCameraInputsManager;
    /**
     * Add orientation input support to the input manager.
     * @returns the current input manager
     */
    addVRDeviceOrientation(): FollowCameraInputsManager;
}

}
declare module "babylonjs/Cameras/freeCamera" {
import { Vector3 } from "babylonjs/Maths/math.vector";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { Scene } from "babylonjs/scene";
import { TargetCamera } from "babylonjs/Cameras/targetCamera";
import { FreeCameraInputsManager } from "babylonjs/Cameras/freeCameraInputsManager";
/**
 * This represents a free type of camera. It can be useful in First Person Shooter game for instance.
 * Please consider using the new UniversalCamera instead as it adds more functionality like the gamepad.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_introduction#universal-camera
 */
export class FreeCamera extends TargetCamera {
    /**
     * Define the collision ellipsoid of the camera.
     * This is helpful to simulate a camera body like the player body around the camera
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_collisions#arcrotatecamera
     */
    ellipsoid: Vector3;
    /**
     * Define an offset for the position of the ellipsoid around the camera.
     * This can be helpful to determine the center of the body near the gravity center of the body
     * instead of its head.
     */
    ellipsoidOffset: Vector3;
    /**
     * Enable or disable collisions of the camera with the rest of the scene objects.
     */
    checkCollisions: boolean;
    /**
     * Enable or disable gravity on the camera.
     */
    applyGravity: boolean;
    /**
     * Define the input manager associated to the camera.
     */
    inputs: FreeCameraInputsManager;
    /**
     * Gets the input sensibility for a mouse input. (default is 2000.0)
     * Higher values reduce sensitivity.
     */
    get angularSensibility(): number;
    /**
     * Sets the input sensibility for a mouse input. (default is 2000.0)
     * Higher values reduce sensitivity.
     */
    set angularSensibility(value: number);
    /**
     * Gets or Set the list of keyboard keys used to control the forward move of the camera.
     */
    get keysUp(): number[];
    set keysUp(value: number[]);
    /**
     * Gets or Set the list of keyboard keys used to control the upward move of the camera.
     */
    get keysUpward(): number[];
    set keysUpward(value: number[]);
    /**
     * Gets or Set the list of keyboard keys used to control the backward move of the camera.
     */
    get keysDown(): number[];
    set keysDown(value: number[]);
    /**
     * Gets or Set the list of keyboard keys used to control the downward move of the camera.
     */
    get keysDownward(): number[];
    set keysDownward(value: number[]);
    /**
     * Gets or Set the list of keyboard keys used to control the left strafe move of the camera.
     */
    get keysLeft(): number[];
    set keysLeft(value: number[]);
    /**
     * Gets or Set the list of keyboard keys used to control the right strafe move of the camera.
     */
    get keysRight(): number[];
    set keysRight(value: number[]);
    /**
     * Gets or Set the list of keyboard keys used to control the left rotation move of the camera.
     */
    get keysRotateLeft(): number[];
    set keysRotateLeft(value: number[]);
    /**
     * Gets or Set the list of keyboard keys used to control the right rotation move of the camera.
     */
    get keysRotateRight(): number[];
    set keysRotateRight(value: number[]);
    /**
     * Event raised when the camera collide with a mesh in the scene.
     */
    onCollide: (collidedMesh: AbstractMesh) => void;
    private _collider;
    private _needMoveForGravity;
    private _oldPosition;
    private _diffPosition;
    private _newPosition;
    /** @internal */
    _localDirection: Vector3;
    /** @internal */
    _transformedDirection: Vector3;
    /**
     * Instantiates a Free Camera.
     * This represents a free type of camera. It can be useful in First Person Shooter game for instance.
     * Please consider using the new UniversalCamera instead as it adds more functionality like touch to this camera.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_introduction#universal-camera
     * @param name Define the name of the camera in the scene
     * @param position Define the start position of the camera in the scene
     * @param scene Define the scene the camera belongs to
     * @param setActiveOnSceneIfNoneActive Defines whether the camera should be marked as active if not other active cameras have been defined
     */
    constructor(name: string, position: Vector3, scene?: Scene, setActiveOnSceneIfNoneActive?: boolean);
    /**
     * Attach the input controls to a specific dom element to get the input from.
     * @param noPreventDefault Defines whether event caught by the controls should call preventdefault() (https://developer.mozilla.org/en-US/docs/Web/API/Event/preventDefault)
     */
    attachControl(noPreventDefault?: boolean): void;
    /**
     * Attach the input controls to a specific dom element to get the input from.
     * @param ignored defines an ignored parameter kept for backward compatibility.
     * @param noPreventDefault Defines whether event caught by the controls should call preventdefault() (https://developer.mozilla.org/en-US/docs/Web/API/Event/preventDefault)
     * BACK COMPAT SIGNATURE ONLY.
     */
    attachControl(ignored: any, noPreventDefault?: boolean): void;
    /**
     * Detach the current controls from the specified dom element.
     */
    detachControl(): void;
    private _collisionMask;
    /**
     * Define a collision mask to limit the list of object the camera can collide with
     */
    get collisionMask(): number;
    set collisionMask(mask: number);
    /**
     * @internal
     */
    _collideWithWorld(displacement: Vector3): void;
    private _onCollisionPositionChange;
    /** @internal */
    _checkInputs(): void;
    /** @internal */
    _decideIfNeedsToMove(): boolean;
    /** @internal */
    _updatePosition(): void;
    /**
     * Destroy the camera and release the current resources hold by it.
     */
    dispose(): void;
    /**
     * Gets the current object class name.
     * @returns the class name
     */
    getClassName(): string;
}

}
declare module "babylonjs/Cameras/freeCameraInputsManager" {
import { FreeCamera } from "babylonjs/Cameras/freeCamera";
import { CameraInputsManager } from "babylonjs/Cameras/cameraInputsManager";
import { FreeCameraMouseInput } from "babylonjs/Cameras/Inputs/freeCameraMouseInput";
import { FreeCameraMouseWheelInput } from "babylonjs/Cameras/Inputs/freeCameraMouseWheelInput";
import { Nullable } from "babylonjs/types";
/**
 * Default Inputs manager for the FreeCamera.
 * It groups all the default supported inputs for ease of use.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/customizingCameraInputs
 */
export class FreeCameraInputsManager extends CameraInputsManager<FreeCamera> {
    /**
     * @internal
     */
    _mouseInput: Nullable<FreeCameraMouseInput>;
    /**
     * @internal
     */
    _mouseWheelInput: Nullable<FreeCameraMouseWheelInput>;
    /**
     * Instantiates a new FreeCameraInputsManager.
     * @param camera Defines the camera the inputs belong to
     */
    constructor(camera: FreeCamera);
    /**
     * Add keyboard input support to the input manager.
     * @returns the current input manager
     */
    addKeyboard(): FreeCameraInputsManager;
    /**
     * Add mouse input support to the input manager.
     * @param touchEnabled if the FreeCameraMouseInput should support touch (default: true)
     * @returns the current input manager
     */
    addMouse(touchEnabled?: boolean): FreeCameraInputsManager;
    /**
     * Removes the mouse input support from the manager
     * @returns the current input manager
     */
    removeMouse(): FreeCameraInputsManager;
    /**
     * Add mouse wheel input support to the input manager.
     * @returns the current input manager
     */
    addMouseWheel(): FreeCameraInputsManager;
    /**
     * Removes the mouse wheel input support from the manager
     * @returns the current input manager
     */
    removeMouseWheel(): FreeCameraInputsManager;
    /**
     * Add touch input support to the input manager.
     * @returns the current input manager
     */
    addTouch(): FreeCameraInputsManager;
    /**
     * Remove all attached input methods from a camera
     */
    clear(): void;
}

}
declare module "babylonjs/Cameras/gamepadCamera" {
import { UniversalCamera } from "babylonjs/Cameras/universalCamera";
import { Scene } from "babylonjs/scene";
import { Vector3 } from "babylonjs/Maths/math.vector";
/**
 * This represents a FPS type of camera. This is only here for back compat purpose.
 * Please use the UniversalCamera instead as both are identical.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_introduction#universal-camera
 */
export class GamepadCamera extends UniversalCamera {
    /**
     * Instantiates a new Gamepad Camera
     * This represents a FPS type of camera. This is only here for back compat purpose.
     * Please use the UniversalCamera instead as both are identical.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_introduction#universal-camera
     * @param name Define the name of the camera in the scene
     * @param position Define the start position of the camera in the scene
     * @param scene Define the scene the camera belongs to
     */
    constructor(name: string, position: Vector3, scene?: Scene);
    /**
     * Gets the current object class name.
     * @returns the class name
     */
    getClassName(): string;
}

}
declare module "babylonjs/Cameras/index" {
export * from "babylonjs/Cameras/Inputs/index";
export * from "babylonjs/Cameras/cameraInputsManager";
export * from "babylonjs/Cameras/camera";
export * from "babylonjs/Cameras/targetCamera";
export * from "babylonjs/Cameras/freeCamera";
export * from "babylonjs/Cameras/freeCameraInputsManager";
export * from "babylonjs/Cameras/touchCamera";
export * from "babylonjs/Cameras/arcRotateCamera";
export * from "babylonjs/Cameras/arcRotateCameraInputsManager";
export * from "babylonjs/Cameras/deviceOrientationCamera";
export * from "babylonjs/Cameras/flyCamera";
export * from "babylonjs/Cameras/flyCameraInputsManager";
export * from "babylonjs/Cameras/followCamera";
export * from "babylonjs/Cameras/followCameraInputsManager";
export * from "babylonjs/Cameras/gamepadCamera";
export * from "babylonjs/Cameras/Stereoscopic/index";
export * from "babylonjs/Cameras/universalCamera";
export * from "babylonjs/Cameras/virtualJoysticksCamera";
export * from "babylonjs/Cameras/VR/index";
export * from "babylonjs/Cameras/RigModes/index";

}
declare module "babylonjs/Cameras/Inputs/arcRotateCameraGamepadInput" {
import { Nullable } from "babylonjs/types";
import { ArcRotateCamera } from "babylonjs/Cameras/arcRotateCamera";
import { ICameraInput } from "babylonjs/Cameras/cameraInputsManager";
import { Gamepad } from "babylonjs/Gamepads/gamepad";
/**
 * Manage the gamepad inputs to control an arc rotate camera.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/customizingCameraInputs
 */
export class ArcRotateCameraGamepadInput implements ICameraInput<ArcRotateCamera> {
    /**
     * Defines the camera the input is attached to.
     */
    camera: ArcRotateCamera;
    /**
     * Defines the gamepad the input is gathering event from.
     */
    gamepad: Nullable<Gamepad>;
    /**
     * Defines the gamepad rotation sensibility.
     * This is the threshold from when rotation starts to be accounted for to prevent jittering.
     */
    gamepadRotationSensibility: number;
    /**
     * Defines the gamepad move sensibility.
     * This is the threshold from when moving starts to be accounted for for to prevent jittering.
     */
    gamepadMoveSensibility: number;
    private _yAxisScale;
    /**
     * Gets or sets a boolean indicating that Yaxis (for right stick) should be inverted
     */
    get invertYAxis(): boolean;
    set invertYAxis(value: boolean);
    private _onGamepadConnectedObserver;
    private _onGamepadDisconnectedObserver;
    /**
     * Attach the input controls to a specific dom element to get the input from.
     */
    attachControl(): void;
    /**
     * Detach the current controls from the specified dom element.
     */
    detachControl(): void;
    /**
     * Update the current camera state depending on the inputs that have been used this frame.
     * This is a dynamically created lambda to avoid the performance penalty of looping for inputs in the render loop.
     */
    checkInputs(): void;
    /**
     * Gets the class name of the current intput.
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Get the friendly name associated with the input class.
     * @returns the input friendly name
     */
    getSimpleName(): string;
}

}
declare module "babylonjs/Cameras/Inputs/arcRotateCameraKeyboardMoveInput" {
import { ArcRotateCamera } from "babylonjs/Cameras/arcRotateCamera";
import { ICameraInput } from "babylonjs/Cameras/cameraInputsManager";
/**
 * Manage the keyboard inputs to control the movement of an arc rotate camera.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/customizingCameraInputs
 */
export class ArcRotateCameraKeyboardMoveInput implements ICameraInput<ArcRotateCamera> {
    /**
     * Defines the camera the input is attached to.
     */
    camera: ArcRotateCamera;
    /**
     * Defines the list of key codes associated with the up action (increase alpha)
     */
    keysUp: number[];
    /**
     * Defines the list of key codes associated with the down action (decrease alpha)
     */
    keysDown: number[];
    /**
     * Defines the list of key codes associated with the left action (increase beta)
     */
    keysLeft: number[];
    /**
     * Defines the list of key codes associated with the right action (decrease beta)
     */
    keysRight: number[];
    /**
     * Defines the list of key codes associated with the reset action.
     * Those keys reset the camera to its last stored state (with the method camera.storeState())
     */
    keysReset: number[];
    /**
     * Defines the panning sensibility of the inputs.
     * (How fast is the camera panning)
     */
    panningSensibility: number;
    /**
     * Defines the zooming sensibility of the inputs.
     * (How fast is the camera zooming)
     */
    zoomingSensibility: number;
    /**
     * Defines whether maintaining the alt key down switch the movement mode from
     * orientation to zoom.
     */
    useAltToZoom: boolean;
    /**
     * Rotation speed of the camera
     */
    angularSpeed: number;
    private _keys;
    private _ctrlPressed;
    private _altPressed;
    private _onCanvasBlurObserver;
    private _onKeyboardObserver;
    private _engine;
    private _scene;
    /**
     * Attach the input controls to a specific dom element to get the input from.
     * @param noPreventDefault Defines whether event caught by the controls should call preventdefault() (https://developer.mozilla.org/en-US/docs/Web/API/Event/preventDefault)
     */
    attachControl(noPreventDefault?: boolean): void;
    /**
     * Detach the current controls from the specified dom element.
     */
    detachControl(): void;
    /**
     * Update the current camera state depending on the inputs that have been used this frame.
     * This is a dynamically created lambda to avoid the performance penalty of looping for inputs in the render loop.
     */
    checkInputs(): void;
    /**
     * Gets the class name of the current input.
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Get the friendly name associated with the input class.
     * @returns the input friendly name
     */
    getSimpleName(): string;
}

}
declare module "babylonjs/Cameras/Inputs/arcRotateCameraMouseWheelInput" {
import { Nullable } from "babylonjs/types";
import { ArcRotateCamera } from "babylonjs/Cameras/arcRotateCamera";
import { ICameraInput } from "babylonjs/Cameras/cameraInputsManager";
import { IWheelEvent } from "babylonjs/Events/deviceInputEvents";
/**
 * Manage the mouse wheel inputs to control an arc rotate camera.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/customizingCameraInputs
 */
export class ArcRotateCameraMouseWheelInput implements ICameraInput<ArcRotateCamera> {
    /**
     * Defines the camera the input is attached to.
     */
    camera: ArcRotateCamera;
    /**
     * Gets or Set the mouse wheel precision or how fast is the camera zooming.
     */
    wheelPrecision: number;
    /**
     * Gets or Set the boolean value that controls whether or not the mouse wheel
     * zooms to the location of the mouse pointer or not.  The default is false.
     */
    zoomToMouseLocation: boolean;
    /**
     * wheelDeltaPercentage will be used instead of wheelPrecision if different from 0.
     * It defines the percentage of current camera.radius to use as delta when wheel is used.
     */
    wheelDeltaPercentage: number;
    /**
     * If set, this function will be used to set the radius delta that will be added to the current camera radius
     */
    customComputeDeltaFromMouseWheel: Nullable<(wheelDelta: number, input: ArcRotateCameraMouseWheelInput, event: IWheelEvent) => number>;
    private _wheel;
    private _observer;
    private _hitPlane;
    protected _computeDeltaFromMouseWheelLegacyEvent(mouseWheelDelta: number, radius: number): number;
    /**
     * Attach the input controls to a specific dom element to get the input from.
     * @param noPreventDefault Defines whether event caught by the controls should call preventdefault() (https://developer.mozilla.org/en-US/docs/Web/API/Event/preventDefault)
     */
    attachControl(noPreventDefault?: boolean): void;
    /**
     * Detach the current controls from the specified dom element.
     */
    detachControl(): void;
    /**
     * Update the current camera state depending on the inputs that have been used this frame.
     * This is a dynamically created lambda to avoid the performance penalty of looping for inputs in the render loop.
     */
    checkInputs(): void;
    /**
     * Gets the class name of the current input.
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Get the friendly name associated with the input class.
     * @returns the input friendly name
     */
    getSimpleName(): string;
    private _updateHitPlane;
    private _getPosition;
    private _inertialPanning;
    private _zoomToMouse;
    private _zeroIfClose;
}

}
declare module "babylonjs/Cameras/Inputs/arcRotateCameraPointersInput" {
import { Nullable } from "babylonjs/types";
import { ArcRotateCamera } from "babylonjs/Cameras/arcRotateCamera";
import { BaseCameraPointersInput } from "babylonjs/Cameras/Inputs/BaseCameraPointersInput";
import { PointerTouch } from "babylonjs/Events/pointerEvents";
import { IPointerEvent } from "babylonjs/Events/deviceInputEvents";
/**
 * Manage the pointers inputs to control an arc rotate camera.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/customizingCameraInputs
 */
export class ArcRotateCameraPointersInput extends BaseCameraPointersInput {
    /**
     * Defines the camera the input is attached to.
     */
    camera: ArcRotateCamera;
    /**
     * The minimum radius used for pinch, to avoid radius lock at 0
     */
    static MinimumRadiusForPinch: number;
    /**
     * Gets the class name of the current input.
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Defines the buttons associated with the input to handle camera move.
     */
    buttons: number[];
    /**
     * Defines the pointer angular sensibility  along the X axis or how fast is
     * the camera rotating.
     */
    angularSensibilityX: number;
    /**
     * Defines the pointer angular sensibility along the Y axis or how fast is
     * the camera rotating.
     */
    angularSensibilityY: number;
    /**
     * Defines the pointer pinch precision or how fast is the camera zooming.
     */
    pinchPrecision: number;
    /**
     * pinchDeltaPercentage will be used instead of pinchPrecision if different
     * from 0.
     * It defines the percentage of current camera.radius to use as delta when
     * pinch zoom is used.
     */
    pinchDeltaPercentage: number;
    /**
     * When useNaturalPinchZoom is true, multi touch zoom will zoom in such
     * that any object in the plane at the camera's target point will scale
     * perfectly with finger motion.
     * Overrides pinchDeltaPercentage and pinchPrecision.
     */
    useNaturalPinchZoom: boolean;
    /**
     * Defines whether zoom (2 fingers pinch) is enabled through multitouch
     */
    pinchZoom: boolean;
    /**
     * Defines the pointer panning sensibility or how fast is the camera moving.
     */
    panningSensibility: number;
    /**
     * Defines whether panning (2 fingers swipe) is enabled through multitouch.
     */
    multiTouchPanning: boolean;
    /**
     * Defines whether panning is enabled for both pan (2 fingers swipe) and
     * zoom (pinch) through multitouch.
     */
    multiTouchPanAndZoom: boolean;
    /**
     * Revers pinch action direction.
     */
    pinchInwards: boolean;
    private _isPanClick;
    private _twoFingerActivityCount;
    private _isPinching;
    /**
     * Move camera from multi touch panning positions.
     * @param previousMultiTouchPanPosition
     * @param multiTouchPanPosition
     */
    private _computeMultiTouchPanning;
    /**
     * Move camera from pinch zoom distances.
     * @param previousPinchSquaredDistance
     * @param pinchSquaredDistance
     */
    private _computePinchZoom;
    /**
     * Called on pointer POINTERMOVE event if only a single touch is active.
     * @param point
     * @param offsetX
     * @param offsetY
     */
    onTouch(point: Nullable<PointerTouch>, offsetX: number, offsetY: number): void;
    /**
     * Called on pointer POINTERDOUBLETAP event.
     */
    onDoubleTap(): void;
    /**
     * Called on pointer POINTERMOVE event if multiple touches are active.
     * @param pointA
     * @param pointB
     * @param previousPinchSquaredDistance
     * @param pinchSquaredDistance
     * @param previousMultiTouchPanPosition
     * @param multiTouchPanPosition
     */
    onMultiTouch(pointA: Nullable<PointerTouch>, pointB: Nullable<PointerTouch>, previousPinchSquaredDistance: number, pinchSquaredDistance: number, previousMultiTouchPanPosition: Nullable<PointerTouch>, multiTouchPanPosition: Nullable<PointerTouch>): void;
    /**
     * Called each time a new POINTERDOWN event occurs. Ie, for each button
     * press.
     * @param evt
     */
    onButtonDown(evt: IPointerEvent): void;
    /**
     * Called each time a new POINTERUP event occurs. Ie, for each button
     * release.
     */
    onButtonUp(): void;
    /**
     * Called when window becomes inactive.
     */
    onLostFocus(): void;
}

}
declare module "babylonjs/Cameras/Inputs/arcRotateCameraVRDeviceOrientationInput" {
import { ArcRotateCamera } from "babylonjs/Cameras/arcRotateCamera";
import { ICameraInput } from "babylonjs/Cameras/cameraInputsManager";
module "babylonjs/Cameras/arcRotateCameraInputsManager" {
    interface ArcRotateCameraInputsManager {
        /**
         * Add orientation input support to the input manager.
         * @returns the current input manager
         */
        addVRDeviceOrientation(): ArcRotateCameraInputsManager;
    }
}
/**
 * Manage the device orientation inputs (gyroscope) to control an arc rotate camera.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/customizingCameraInputs
 */
export class ArcRotateCameraVRDeviceOrientationInput implements ICameraInput<ArcRotateCamera> {
    /**
     * Defines the camera the input is attached to.
     */
    camera: ArcRotateCamera;
    /**
     * Defines a correction factor applied on the alpha value retrieved from the orientation events.
     */
    alphaCorrection: number;
    /**
     * Defines a correction factor applied on the gamma value retrieved from the orientation events.
     */
    gammaCorrection: number;
    private _alpha;
    private _gamma;
    private _dirty;
    private _deviceOrientationHandler;
    /**
     * Instantiate a new ArcRotateCameraVRDeviceOrientationInput.
     */
    constructor();
    /**
     * Attach the input controls to a specific dom element to get the input from.
     * @param noPreventDefault Defines whether event caught by the controls should call preventdefault() (https://developer.mozilla.org/en-US/docs/Web/API/Event/preventDefault)
     */
    attachControl(noPreventDefault?: boolean): void;
    /**
     * @internal
     */
    _onOrientationEvent(evt: DeviceOrientationEvent): void;
    /**
     * Update the current camera state depending on the inputs that have been used this frame.
     * This is a dynamically created lambda to avoid the performance penalty of looping for inputs in the render loop.
     */
    checkInputs(): void;
    /**
     * Detach the current controls from the specified dom element.
     */
    detachControl(): void;
    /**
     * Gets the class name of the current input.
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Get the friendly name associated with the input class.
     * @returns the input friendly name
     */
    getSimpleName(): string;
}

}
declare module "babylonjs/Cameras/Inputs/BaseCameraMouseWheelInput" {
import { Observable } from "babylonjs/Misc/observable";
import { Camera } from "babylonjs/Cameras/camera";
import { ICameraInput } from "babylonjs/Cameras/cameraInputsManager";
/**
 * Base class for mouse wheel input..
 * See FollowCameraMouseWheelInput in src/Cameras/Inputs/freeCameraMouseWheelInput.ts
 * for example usage.
 */
export abstract class BaseCameraMouseWheelInput implements ICameraInput<Camera> {
    /**
     * Defines the camera the input is attached to.
     */
    abstract camera: Camera;
    /**
     * How fast is the camera moves in relation to X axis mouseWheel events.
     * Use negative value to reverse direction.
     */
    wheelPrecisionX: number;
    /**
     * How fast is the camera moves in relation to Y axis mouseWheel events.
     * Use negative value to reverse direction.
     */
    wheelPrecisionY: number;
    /**
     * How fast is the camera moves in relation to Z axis mouseWheel events.
     * Use negative value to reverse direction.
     */
    wheelPrecisionZ: number;
    /**
     * Observable for when a mouse wheel move event occurs.
     */
    onChangedObservable: Observable<{
        wheelDeltaX: number;
        wheelDeltaY: number;
        wheelDeltaZ: number;
    }>;
    private _wheel;
    private _observer;
    /**
     * Attach the input controls to a specific dom element to get the input from.
     * @param noPreventDefault Defines whether event caught by the controls
     *   should call preventdefault().
     *   (https://developer.mozilla.org/en-US/docs/Web/API/Event/preventDefault)
     */
    attachControl(noPreventDefault?: boolean): void;
    /**
     * Detach the current controls from the specified dom element.
     */
    detachControl(): void;
    /**
     * Called for each rendered frame.
     */
    checkInputs(): void;
    /**
     * Gets the class name of the current input.
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Get the friendly name associated with the input class.
     * @returns the input friendly name
     */
    getSimpleName(): string;
    /**
     * Incremental value of multiple mouse wheel movements of the X axis.
     * Should be zero-ed when read.
     */
    protected _wheelDeltaX: number;
    /**
     * Incremental value of multiple mouse wheel movements of the Y axis.
     * Should be zero-ed when read.
     */
    protected _wheelDeltaY: number;
    /**
     * Incremental value of multiple mouse wheel movements of the Z axis.
     * Should be zero-ed when read.
     */
    protected _wheelDeltaZ: number;
    /**
     * Firefox uses a different scheme to report scroll distances to other
     * browsers. Rather than use complicated methods to calculate the exact
     * multiple we need to apply, let's just cheat and use a constant.
     * https://developer.mozilla.org/en-US/docs/Web/API/WheelEvent/deltaMode
     * https://stackoverflow.com/questions/20110224/what-is-the-height-of-a-line-in-a-wheel-event-deltamode-dom-delta-line
     */
    private readonly _ffMultiplier;
    /**
     * Different event attributes for wheel data fall into a few set ranges.
     * Some relevant but dated date here:
     * https://stackoverflow.com/questions/5527601/normalizing-mousewheel-speed-across-browsers
     */
    private readonly _normalize;
}

}
declare module "babylonjs/Cameras/Inputs/BaseCameraPointersInput" {
import { Nullable } from "babylonjs/types";
import { Camera } from "babylonjs/Cameras/camera";
import { ICameraInput } from "babylonjs/Cameras/cameraInputsManager";
import { PointerTouch } from "babylonjs/Events/pointerEvents";
import { IPointerEvent } from "babylonjs/Events/deviceInputEvents";
/**
 * Base class for Camera Pointer Inputs.
 * See FollowCameraPointersInput in src/Cameras/Inputs/followCameraPointersInput.ts
 * for example usage.
 */
export abstract class BaseCameraPointersInput implements ICameraInput<Camera> {
    /**
     * Defines the camera the input is attached to.
     */
    abstract camera: Camera;
    /**
     * Whether keyboard modifier keys are pressed at time of last mouse event.
     */
    protected _altKey: boolean;
    protected _ctrlKey: boolean;
    protected _metaKey: boolean;
    protected _shiftKey: boolean;
    /**
     * Which mouse buttons were pressed at time of last mouse event.
     * https://developer.mozilla.org/en-US/docs/Web/API/MouseEvent/buttons
     */
    protected _buttonsPressed: number;
    private _currentActiveButton;
    private _contextMenuBind;
    /**
     * Defines the buttons associated with the input to handle camera move.
     */
    buttons: number[];
    /**
     * Attach the input controls to a specific dom element to get the input from.
     * @param noPreventDefault Defines whether event caught by the controls should call preventdefault() (https://developer.mozilla.org/en-US/docs/Web/API/Event/preventDefault)
     */
    attachControl(noPreventDefault?: boolean): void;
    /**
     * Detach the current controls from the specified dom element.
     */
    detachControl(): void;
    /**
     * Gets the class name of the current input.
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Get the friendly name associated with the input class.
     * @returns the input friendly name
     */
    getSimpleName(): string;
    /**
     * Called on pointer POINTERDOUBLETAP event.
     * Override this method to provide functionality on POINTERDOUBLETAP event.
     * @param type
     */
    onDoubleTap(type: string): void;
    /**
     * Called on pointer POINTERMOVE event if only a single touch is active.
     * Override this method to provide functionality.
     * @param point
     * @param offsetX
     * @param offsetY
     */
    onTouch(point: Nullable<PointerTouch>, offsetX: number, offsetY: number): void;
    /**
     * Called on pointer POINTERMOVE event if multiple touches are active.
     * Override this method to provide functionality.
     * @param _pointA
     * @param _pointB
     * @param previousPinchSquaredDistance
     * @param pinchSquaredDistance
     * @param previousMultiTouchPanPosition
     * @param multiTouchPanPosition
     */
    onMultiTouch(_pointA: Nullable<PointerTouch>, _pointB: Nullable<PointerTouch>, previousPinchSquaredDistance: number, pinchSquaredDistance: number, previousMultiTouchPanPosition: Nullable<PointerTouch>, multiTouchPanPosition: Nullable<PointerTouch>): void;
    /**
     * Called on JS contextmenu event.
     * Override this method to provide functionality.
     * @param evt
     */
    onContextMenu(evt: PointerEvent): void;
    /**
     * Called each time a new POINTERDOWN event occurs. Ie, for each button
     * press.
     * Override this method to provide functionality.
     * @param evt
     */
    onButtonDown(evt: IPointerEvent): void;
    /**
     * Called each time a new POINTERUP event occurs. Ie, for each button
     * release.
     * Override this method to provide functionality.
     * @param evt
     */
    onButtonUp(evt: IPointerEvent): void;
    /**
     * Called when window becomes inactive.
     * Override this method to provide functionality.
     */
    onLostFocus(): void;
    private _pointerInput;
    private _observer;
    private _onLostFocus;
    private _pointA;
    private _pointB;
}

}
declare module "babylonjs/Cameras/Inputs/flyCameraKeyboardInput" {
import { ICameraInput } from "babylonjs/Cameras/cameraInputsManager";
import { FlyCamera } from "babylonjs/Cameras/flyCamera";
/**
 * Listen to keyboard events to control the camera.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/customizingCameraInputs
 */
export class FlyCameraKeyboardInput implements ICameraInput<FlyCamera> {
    /**
     * Defines the camera the input is attached to.
     */
    camera: FlyCamera;
    /**
     * The list of keyboard keys used to control the forward move of the camera.
     */
    keysForward: number[];
    /**
     * The list of keyboard keys used to control the backward move of the camera.
     */
    keysBackward: number[];
    /**
     * The list of keyboard keys used to control the forward move of the camera.
     */
    keysUp: number[];
    /**
     * The list of keyboard keys used to control the backward move of the camera.
     */
    keysDown: number[];
    /**
     * The list of keyboard keys used to control the right strafe move of the camera.
     */
    keysRight: number[];
    /**
     * The list of keyboard keys used to control the left strafe move of the camera.
     */
    keysLeft: number[];
    private _keys;
    private _onCanvasBlurObserver;
    private _onKeyboardObserver;
    private _engine;
    private _scene;
    /**
     * Attach the input controls to a specific dom element to get the input from.
     * @param noPreventDefault Defines whether event caught by the controls should call preventdefault() (https://developer.mozilla.org/en-US/docs/Web/API/Event/preventDefault)
     */
    attachControl(noPreventDefault?: boolean): void;
    /**
     * Detach the current controls from the specified dom element.
     */
    detachControl(): void;
    /**
     * Gets the class name of the current input.
     * @returns the class name
     */
    getClassName(): string;
    /**
     * @internal
     */
    _onLostFocus(): void;
    /**
     * Get the friendly name associated with the input class.
     * @returns the input friendly name
     */
    getSimpleName(): string;
    /**
     * Update the current camera state depending on the inputs that have been used this frame.
     * This is a dynamically created lambda to avoid the performance penalty of looping for inputs in the render loop.
     */
    checkInputs(): void;
}

}
declare module "babylonjs/Cameras/Inputs/flyCameraMouseInput" {
import { ICameraInput } from "babylonjs/Cameras/cameraInputsManager";
import { FlyCamera } from "babylonjs/Cameras/flyCamera";
/**
 * Listen to mouse events to control the camera.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/customizingCameraInputs
 */
export class FlyCameraMouseInput implements ICameraInput<FlyCamera> {
    /**
     * Defines the camera the input is attached to.
     */
    camera: FlyCamera;
    /**
     * Defines if touch is enabled. (Default is true.)
     */
    touchEnabled: boolean;
    /**
     * Defines the buttons associated with the input to handle camera rotation.
     */
    buttons: number[];
    /**
     * Assign buttons for Yaw control.
     */
    buttonsYaw: number[];
    /**
     * Assign buttons for Pitch control.
     */
    buttonsPitch: number[];
    /**
     * Assign buttons for Roll control.
     */
    buttonsRoll: number[];
    /**
     * Detect if any button is being pressed while mouse is moved.
     * -1 = Mouse locked.
     * 0 = Left button.
     * 1 = Middle Button.
     * 2 = Right Button.
     */
    activeButton: number;
    /**
     * Defines the pointer's angular sensibility, to control the camera rotation speed.
     * Higher values reduce its sensitivity.
     */
    angularSensibility: number;
    private _observer;
    private _rollObserver;
    private _previousPosition;
    private _noPreventDefault;
    private _element;
    /**
     * Listen to mouse events to control the camera.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/customizingCameraInputs
     */
    constructor();
    /**
     * Attach the mouse control to the HTML DOM element.
     * @param noPreventDefault Defines whether events caught by the controls should call preventdefault().
     */
    attachControl(noPreventDefault?: boolean): void;
    /**
     * Detach the current controls from the specified dom element.
     */
    detachControl(): void;
    /**
     * Gets the class name of the current input.
     * @returns the class name.
     */
    getClassName(): string;
    /**
     * Get the friendly name associated with the input class.
     * @returns the input's friendly name.
     */
    getSimpleName(): string;
    private _pointerInput;
    private _onMouseMove;
    /**
     * Rotate camera by mouse offset.
     * @param offsetX
     * @param offsetY
     */
    private _rotateCamera;
}

}
declare module "babylonjs/Cameras/Inputs/followCameraKeyboardMoveInput" {
import { ICameraInput } from "babylonjs/Cameras/cameraInputsManager";
import { FollowCamera } from "babylonjs/Cameras/followCamera";
/**
 * Manage the keyboard inputs to control the movement of a follow camera.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/customizingCameraInputs
 */
export class FollowCameraKeyboardMoveInput implements ICameraInput<FollowCamera> {
    /**
     * Defines the camera the input is attached to.
     */
    camera: FollowCamera;
    /**
     * Defines the list of key codes associated with the up action (increase heightOffset)
     */
    keysHeightOffsetIncr: number[];
    /**
     * Defines the list of key codes associated with the down action (decrease heightOffset)
     */
    keysHeightOffsetDecr: number[];
    /**
     * Defines whether the Alt modifier key is required to move up/down (alter heightOffset)
     */
    keysHeightOffsetModifierAlt: boolean;
    /**
     * Defines whether the Ctrl modifier key is required to move up/down (alter heightOffset)
     */
    keysHeightOffsetModifierCtrl: boolean;
    /**
     * Defines whether the Shift modifier key is required to move up/down (alter heightOffset)
     */
    keysHeightOffsetModifierShift: boolean;
    /**
     * Defines the list of key codes associated with the left action (increase rotationOffset)
     */
    keysRotationOffsetIncr: number[];
    /**
     * Defines the list of key codes associated with the right action (decrease rotationOffset)
     */
    keysRotationOffsetDecr: number[];
    /**
     * Defines whether the Alt modifier key is required to move left/right (alter rotationOffset)
     */
    keysRotationOffsetModifierAlt: boolean;
    /**
     * Defines whether the Ctrl modifier key is required to move left/right (alter rotationOffset)
     */
    keysRotationOffsetModifierCtrl: boolean;
    /**
     * Defines whether the Shift modifier key is required to move left/right (alter rotationOffset)
     */
    keysRotationOffsetModifierShift: boolean;
    /**
     * Defines the list of key codes associated with the zoom-in action (decrease radius)
     */
    keysRadiusIncr: number[];
    /**
     * Defines the list of key codes associated with the zoom-out action (increase radius)
     */
    keysRadiusDecr: number[];
    /**
     * Defines whether the Alt modifier key is required to zoom in/out (alter radius value)
     */
    keysRadiusModifierAlt: boolean;
    /**
     * Defines whether the Ctrl modifier key is required to zoom in/out (alter radius value)
     */
    keysRadiusModifierCtrl: boolean;
    /**
     * Defines whether the Shift modifier key is required to zoom in/out (alter radius value)
     */
    keysRadiusModifierShift: boolean;
    /**
     * Defines the rate of change of heightOffset.
     */
    heightSensibility: number;
    /**
     * Defines the rate of change of rotationOffset.
     */
    rotationSensibility: number;
    /**
     * Defines the rate of change of radius.
     */
    radiusSensibility: number;
    private _keys;
    private _ctrlPressed;
    private _altPressed;
    private _shiftPressed;
    private _onCanvasBlurObserver;
    private _onKeyboardObserver;
    private _engine;
    private _scene;
    /**
     * Attach the input controls to a specific dom element to get the input from.
     * @param noPreventDefault Defines whether event caught by the controls should call preventdefault() (https://developer.mozilla.org/en-US/docs/Web/API/Event/preventDefault)
     */
    attachControl(noPreventDefault?: boolean): void;
    /**
     * Detach the current controls from the specified dom element.
     */
    detachControl(): void;
    /**
     * Update the current camera state depending on the inputs that have been used this frame.
     * This is a dynamically created lambda to avoid the performance penalty of looping for inputs in the render loop.
     */
    checkInputs(): void;
    /**
     * Gets the class name of the current input.
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Get the friendly name associated with the input class.
     * @returns the input friendly name
     */
    getSimpleName(): string;
    /**
     * Check if the pressed modifier keys (Alt/Ctrl/Shift) match those configured to
     * allow modification of the heightOffset value.
     */
    private _modifierHeightOffset;
    /**
     * Check if the pressed modifier keys (Alt/Ctrl/Shift) match those configured to
     * allow modification of the rotationOffset value.
     */
    private _modifierRotationOffset;
    /**
     * Check if the pressed modifier keys (Alt/Ctrl/Shift) match those configured to
     * allow modification of the radius value.
     */
    private _modifierRadius;
}

}
declare module "babylonjs/Cameras/Inputs/followCameraMouseWheelInput" {
import { FollowCamera } from "babylonjs/Cameras/followCamera";
import { ICameraInput } from "babylonjs/Cameras/cameraInputsManager";
/**
 * Manage the mouse wheel inputs to control a follow camera.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/customizingCameraInputs
 */
export class FollowCameraMouseWheelInput implements ICameraInput<FollowCamera> {
    /**
     * Defines the camera the input is attached to.
     */
    camera: FollowCamera;
    /**
     * Moue wheel controls zoom. (Mouse wheel modifies camera.radius value.)
     */
    axisControlRadius: boolean;
    /**
     * Moue wheel controls height. (Mouse wheel modifies camera.heightOffset value.)
     */
    axisControlHeight: boolean;
    /**
     * Moue wheel controls angle. (Mouse wheel modifies camera.rotationOffset value.)
     */
    axisControlRotation: boolean;
    /**
     * Gets or Set the mouse wheel precision or how fast is the camera moves in
     * relation to mouseWheel events.
     */
    wheelPrecision: number;
    /**
     * wheelDeltaPercentage will be used instead of wheelPrecision if different from 0.
     * It defines the percentage of current camera.radius to use as delta when wheel is used.
     */
    wheelDeltaPercentage: number;
    private _wheel;
    private _observer;
    /**
     * Attach the input controls to a specific dom element to get the input from.
     * @param noPreventDefault Defines whether event caught by the controls should call preventdefault() (https://developer.mozilla.org/en-US/docs/Web/API/Event/preventDefault)
     */
    attachControl(noPreventDefault?: boolean): void;
    /**
     * Detach the current controls from the specified dom element.
     */
    detachControl(): void;
    /**
     * Gets the class name of the current input.
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Get the friendly name associated with the input class.
     * @returns the input friendly name
     */
    getSimpleName(): string;
}

}
declare module "babylonjs/Cameras/Inputs/followCameraPointersInput" {
import { Nullable } from "babylonjs/types";
import { FollowCamera } from "babylonjs/Cameras/followCamera";
import { BaseCameraPointersInput } from "babylonjs/Cameras/Inputs/BaseCameraPointersInput";
import { PointerTouch } from "babylonjs/Events/pointerEvents";
/**
 * Manage the pointers inputs to control an follow camera.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/customizingCameraInputs
 */
export class FollowCameraPointersInput extends BaseCameraPointersInput {
    /**
     * Defines the camera the input is attached to.
     */
    camera: FollowCamera;
    /**
     * Gets the class name of the current input.
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Defines the pointer angular sensibility along the X axis or how fast is
     * the camera rotating.
     * A negative number will reverse the axis direction.
     */
    angularSensibilityX: number;
    /**
     * Defines the pointer angular sensibility along the Y axis or how fast is
     * the camera rotating.
     * A negative number will reverse the axis direction.
     */
    angularSensibilityY: number;
    /**
     * Defines the pointer pinch precision or how fast is the camera zooming.
     * A negative number will reverse the axis direction.
     */
    pinchPrecision: number;
    /**
     * pinchDeltaPercentage will be used instead of pinchPrecision if different
     * from 0.
     * It defines the percentage of current camera.radius to use as delta when
     * pinch zoom is used.
     */
    pinchDeltaPercentage: number;
    /**
     * Pointer X axis controls zoom. (X axis modifies camera.radius value.)
     */
    axisXControlRadius: boolean;
    /**
     * Pointer X axis controls height. (X axis modifies camera.heightOffset value.)
     */
    axisXControlHeight: boolean;
    /**
     * Pointer X axis controls angle. (X axis modifies camera.rotationOffset value.)
     */
    axisXControlRotation: boolean;
    /**
     * Pointer Y axis controls zoom. (Y axis modifies camera.radius value.)
     */
    axisYControlRadius: boolean;
    /**
     * Pointer Y axis controls height. (Y axis modifies camera.heightOffset value.)
     */
    axisYControlHeight: boolean;
    /**
     * Pointer Y axis controls angle. (Y axis modifies camera.rotationOffset value.)
     */
    axisYControlRotation: boolean;
    /**
     * Pinch controls zoom. (Pinch modifies camera.radius value.)
     */
    axisPinchControlRadius: boolean;
    /**
     * Pinch controls height. (Pinch modifies camera.heightOffset value.)
     */
    axisPinchControlHeight: boolean;
    /**
     * Pinch controls angle. (Pinch modifies camera.rotationOffset value.)
     */
    axisPinchControlRotation: boolean;
    /**
     * Log error messages if basic misconfiguration has occurred.
     */
    warningEnable: boolean;
    onTouch(pointA: Nullable<PointerTouch>, offsetX: number, offsetY: number): void;
    onMultiTouch(pointA: Nullable<PointerTouch>, pointB: Nullable<PointerTouch>, previousPinchSquaredDistance: number, pinchSquaredDistance: number, previousMultiTouchPanPosition: Nullable<PointerTouch>, multiTouchPanPosition: Nullable<PointerTouch>): void;
    private _warningCounter;
    private _warning;
}

}
declare module "babylonjs/Cameras/Inputs/freeCameraDeviceOrientationInput" {
import { Nullable } from "babylonjs/types";
import { ICameraInput } from "babylonjs/Cameras/cameraInputsManager";
import { FreeCamera } from "babylonjs/Cameras/freeCamera";
import { Observable } from "babylonjs/Misc/observable";
module "babylonjs/Cameras/freeCameraInputsManager" {
    interface FreeCameraInputsManager {
        /**
         * @internal
         */
        _deviceOrientationInput: Nullable<FreeCameraDeviceOrientationInput>;
        /**
         * Add orientation input support to the input manager.
         * @param smoothFactor deviceOrientation smoothing. 0: no smoothing, 1: new data ignored, 0.9 recommended for smoothing
         * @returns the current input manager
         */
        addDeviceOrientation(smoothFactor?: number): FreeCameraInputsManager;
    }
}
/**
 * Takes information about the orientation of the device as reported by the deviceorientation event to orient the camera.
 * Screen rotation is taken into account.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/customizingCameraInputs
 */
export class FreeCameraDeviceOrientationInput implements ICameraInput<FreeCamera> {
    private _camera;
    private _screenOrientationAngle;
    private _constantTranform;
    private _screenQuaternion;
    private _alpha;
    private _beta;
    private _gamma;
    /** alpha+beta+gamma smoothing. 0: no smoothing, 1: new data ignored, 0.9 recommended for smoothing */
    smoothFactor: number;
    /**
     * Can be used to detect if a device orientation sensor is available on a device
     * @param timeout amount of time in milliseconds to wait for a response from the sensor (default: infinite)
     * @returns a promise that will resolve on orientation change
     */
    static WaitForOrientationChangeAsync(timeout?: number): Promise<void>;
    /**
     * @internal
     */
    _onDeviceOrientationChangedObservable: Observable<void>;
    /**
     * Instantiates a new input
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/customizingCameraInputs
     */
    constructor();
    /**
     * Define the camera controlled by the input.
     */
    get camera(): FreeCamera;
    set camera(camera: FreeCamera);
    /**
     * Attach the input controls to a specific dom element to get the input from.
     */
    attachControl(): void;
    private _orientationChanged;
    private _deviceOrientation;
    /**
     * Detach the current controls from the specified dom element.
     */
    detachControl(): void;
    /**
     * Update the current camera state depending on the inputs that have been used this frame.
     * This is a dynamically created lambda to avoid the performance penalty of looping for inputs in the render loop.
     */
    checkInputs(): void;
    /**
     * Gets the class name of the current input.
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Get the friendly name associated with the input class.
     * @returns the input friendly name
     */
    getSimpleName(): string;
}

}
declare module "babylonjs/Cameras/Inputs/freeCameraGamepadInput" {
import { Nullable } from "babylonjs/types";
import { ICameraInput } from "babylonjs/Cameras/cameraInputsManager";
import { FreeCamera } from "babylonjs/Cameras/freeCamera";
import { Gamepad } from "babylonjs/Gamepads/gamepad";
/**
 * Manage the gamepad inputs to control a free camera.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/customizingCameraInputs
 */
export class FreeCameraGamepadInput implements ICameraInput<FreeCamera> {
    /**
     * Define the camera the input is attached to.
     */
    camera: FreeCamera;
    /**
     * Define the Gamepad controlling the input
     */
    gamepad: Nullable<Gamepad>;
    /**
     * Defines the gamepad rotation sensibility.
     * This is the threshold from when rotation starts to be accounted for to prevent jittering.
     */
    gamepadAngularSensibility: number;
    /**
     * Defines the gamepad move sensibility.
     * This is the threshold from when moving starts to be accounted for for to prevent jittering.
     */
    gamepadMoveSensibility: number;
    /**
     * Defines the minimum value at which any analog stick input is ignored.
     * Note: This value should only be a value between 0 and 1.
     */
    deadzoneDelta: number;
    private _yAxisScale;
    /**
     * Gets or sets a boolean indicating that Yaxis (for right stick) should be inverted
     */
    get invertYAxis(): boolean;
    set invertYAxis(value: boolean);
    private _onGamepadConnectedObserver;
    private _onGamepadDisconnectedObserver;
    private _cameraTransform;
    private _deltaTransform;
    private _vector3;
    private _vector2;
    /**
     * Attach the input controls to a specific dom element to get the input from.
     */
    attachControl(): void;
    /**
     * Detach the current controls from the specified dom element.
     */
    detachControl(): void;
    /**
     * Update the current camera state depending on the inputs that have been used this frame.
     * This is a dynamically created lambda to avoid the performance penalty of looping for inputs in the render loop.
     */
    checkInputs(): void;
    /**
     * Gets the class name of the current input.
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Get the friendly name associated with the input class.
     * @returns the input friendly name
     */
    getSimpleName(): string;
}

}
declare module "babylonjs/Cameras/Inputs/freeCameraKeyboardMoveInput" {
import { ICameraInput } from "babylonjs/Cameras/cameraInputsManager";
import { FreeCamera } from "babylonjs/Cameras/freeCamera";
/**
 * Manage the keyboard inputs to control the movement of a free camera.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/customizingCameraInputs
 */
export class FreeCameraKeyboardMoveInput implements ICameraInput<FreeCamera> {
    /**
     * Defines the camera the input is attached to.
     */
    camera: FreeCamera;
    /**
     * Gets or Set the list of keyboard keys used to control the forward move of the camera.
     */
    keysUp: number[];
    /**
     * Gets or Set the list of keyboard keys used to control the upward move of the camera.
     */
    keysUpward: number[];
    /**
     * Gets or Set the list of keyboard keys used to control the backward move of the camera.
     */
    keysDown: number[];
    /**
     * Gets or Set the list of keyboard keys used to control the downward move of the camera.
     */
    keysDownward: number[];
    /**
     * Gets or Set the list of keyboard keys used to control the left strafe move of the camera.
     */
    keysLeft: number[];
    /**
     * Gets or Set the list of keyboard keys used to control the right strafe move of the camera.
     */
    keysRight: number[];
    /**
     * Defines the pointer angular sensibility  along the X and Y axis or how fast is the camera rotating.
     */
    rotationSpeed: number;
    /**
     * Gets or Set the list of keyboard keys used to control the left rotation move of the camera.
     */
    keysRotateLeft: number[];
    /**
     * Gets or Set the list of keyboard keys used to control the right rotation move of the camera.
     */
    keysRotateRight: number[];
    private _keys;
    private _onCanvasBlurObserver;
    private _onKeyboardObserver;
    private _engine;
    private _scene;
    /**
     * Attach the input controls to a specific dom element to get the input from.
     * @param noPreventDefault Defines whether event caught by the controls should call preventdefault() (https://developer.mozilla.org/en-US/docs/Web/API/Event/preventDefault)
     */
    attachControl(noPreventDefault?: boolean): void;
    /**
     * Detach the current controls from the specified dom element.
     */
    detachControl(): void;
    /**
     * Update the current camera state depending on the inputs that have been used this frame.
     * This is a dynamically created lambda to avoid the performance penalty of looping for inputs in the render loop.
     */
    checkInputs(): void;
    /**
     * Gets the class name of the current input.
     * @returns the class name
     */
    getClassName(): string;
    /** @internal */
    _onLostFocus(): void;
    /**
     * Get the friendly name associated with the input class.
     * @returns the input friendly name
     */
    getSimpleName(): string;
    private _getLocalRotation;
}

}
declare module "babylonjs/Cameras/Inputs/freeCameraMouseInput" {
import { Observable } from "babylonjs/Misc/observable";
import { ICameraInput } from "babylonjs/Cameras/cameraInputsManager";
import { FreeCamera } from "babylonjs/Cameras/freeCamera";
/**
 * Manage the mouse inputs to control the movement of a free camera.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/customizingCameraInputs
 */
export class FreeCameraMouseInput implements ICameraInput<FreeCamera> {
    /**
     * Define if touch is enabled in the mouse input
     */
    touchEnabled: boolean;
    /**
     * Defines the camera the input is attached to.
     */
    camera: FreeCamera;
    /**
     * Defines the buttons associated with the input to handle camera move.
     */
    buttons: number[];
    /**
     * Defines the pointer angular sensibility  along the X and Y axis or how fast is the camera rotating.
     */
    angularSensibility: number;
    private _pointerInput;
    private _onMouseMove;
    private _observer;
    private _previousPosition;
    /**
     * Observable for when a pointer move event occurs containing the move offset
     */
    onPointerMovedObservable: Observable<{
        offsetX: number;
        offsetY: number;
    }>;
    /**
     * @internal
     * If the camera should be rotated automatically based on pointer movement
     */
    _allowCameraRotation: boolean;
    private _currentActiveButton;
    private _activePointerId;
    private _contextMenuBind;
    /**
     * Manage the mouse inputs to control the movement of a free camera.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/customizingCameraInputs
     * @param touchEnabled Defines if touch is enabled or not
     */
    constructor(
    /**
     * Define if touch is enabled in the mouse input
     */
    touchEnabled?: boolean);
    /**
     * Attach the input controls to a specific dom element to get the input from.
     * @param noPreventDefault Defines whether event caught by the controls should call preventdefault() (https://developer.mozilla.org/en-US/docs/Web/API/Event/preventDefault)
     */
    attachControl(noPreventDefault?: boolean): void;
    /**
     * Called on JS contextmenu event.
     * Override this method to provide functionality.
     * @param evt
     */
    onContextMenu(evt: PointerEvent): void;
    /**
     * Detach the current controls from the specified dom element.
     */
    detachControl(): void;
    /**
     * Gets the class name of the current input.
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Get the friendly name associated with the input class.
     * @returns the input friendly name
     */
    getSimpleName(): string;
}

}
declare module "babylonjs/Cameras/Inputs/freeCameraMouseWheelInput" {
import { Nullable } from "babylonjs/types";
import { FreeCamera } from "babylonjs/Cameras/freeCamera";
import { BaseCameraMouseWheelInput } from "babylonjs/Cameras/Inputs/BaseCameraMouseWheelInput";
import { Coordinate } from "babylonjs/Maths/math.axis";
/**
 * Manage the mouse wheel inputs to control a free camera.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/customizingCameraInputs
 */
export class FreeCameraMouseWheelInput extends BaseCameraMouseWheelInput {
    /**
     * Defines the camera the input is attached to.
     */
    camera: FreeCamera;
    /**
     * Gets the class name of the current input.
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Set which movement axis (relative to camera's orientation) the mouse
     * wheel's X axis controls.
     * @param axis The axis to be moved. Set null to clear.
     */
    set wheelXMoveRelative(axis: Nullable<Coordinate>);
    /**
     * Get the configured movement axis (relative to camera's orientation) the
     * mouse wheel's X axis controls.
     * @returns The configured axis or null if none.
     */
    get wheelXMoveRelative(): Nullable<Coordinate>;
    /**
     * Set which movement axis (relative to camera's orientation) the mouse
     * wheel's Y axis controls.
     * @param axis The axis to be moved. Set null to clear.
     */
    set wheelYMoveRelative(axis: Nullable<Coordinate>);
    /**
     * Get the configured movement axis (relative to camera's orientation) the
     * mouse wheel's Y axis controls.
     * @returns The configured axis or null if none.
     */
    get wheelYMoveRelative(): Nullable<Coordinate>;
    /**
     * Set which movement axis (relative to camera's orientation) the mouse
     * wheel's Z axis controls.
     * @param axis The axis to be moved. Set null to clear.
     */
    set wheelZMoveRelative(axis: Nullable<Coordinate>);
    /**
     * Get the configured movement axis (relative to camera's orientation) the
     * mouse wheel's Z axis controls.
     * @returns The configured axis or null if none.
     */
    get wheelZMoveRelative(): Nullable<Coordinate>;
    /**
     * Set which rotation axis (relative to camera's orientation) the mouse
     * wheel's X axis controls.
     * @param axis The axis to be moved. Set null to clear.
     */
    set wheelXRotateRelative(axis: Nullable<Coordinate>);
    /**
     * Get the configured rotation axis (relative to camera's orientation) the
     * mouse wheel's X axis controls.
     * @returns The configured axis or null if none.
     */
    get wheelXRotateRelative(): Nullable<Coordinate>;
    /**
     * Set which rotation axis (relative to camera's orientation) the mouse
     * wheel's Y axis controls.
     * @param axis The axis to be moved. Set null to clear.
     */
    set wheelYRotateRelative(axis: Nullable<Coordinate>);
    /**
     * Get the configured rotation axis (relative to camera's orientation) the
     * mouse wheel's Y axis controls.
     * @returns The configured axis or null if none.
     */
    get wheelYRotateRelative(): Nullable<Coordinate>;
    /**
     * Set which rotation axis (relative to camera's orientation) the mouse
     * wheel's Z axis controls.
     * @param axis The axis to be moved. Set null to clear.
     */
    set wheelZRotateRelative(axis: Nullable<Coordinate>);
    /**
     * Get the configured rotation axis (relative to camera's orientation) the
     * mouse wheel's Z axis controls.
     * @returns The configured axis or null if none.
     */
    get wheelZRotateRelative(): Nullable<Coordinate>;
    /**
     * Set which movement axis (relative to the scene) the mouse wheel's X axis
     * controls.
     * @param axis The axis to be moved. Set null to clear.
     */
    set wheelXMoveScene(axis: Nullable<Coordinate>);
    /**
     * Get the configured movement axis (relative to the scene) the mouse wheel's
     * X axis controls.
     * @returns The configured axis or null if none.
     */
    get wheelXMoveScene(): Nullable<Coordinate>;
    /**
     * Set which movement axis (relative to the scene) the mouse wheel's Y axis
     * controls.
     * @param axis The axis to be moved. Set null to clear.
     */
    set wheelYMoveScene(axis: Nullable<Coordinate>);
    /**
     * Get the configured movement axis (relative to the scene) the mouse wheel's
     * Y axis controls.
     * @returns The configured axis or null if none.
     */
    get wheelYMoveScene(): Nullable<Coordinate>;
    /**
     * Set which movement axis (relative to the scene) the mouse wheel's Z axis
     * controls.
     * @param axis The axis to be moved. Set null to clear.
     */
    set wheelZMoveScene(axis: Nullable<Coordinate>);
    /**
     * Get the configured movement axis (relative to the scene) the mouse wheel's
     * Z axis controls.
     * @returns The configured axis or null if none.
     */
    get wheelZMoveScene(): Nullable<Coordinate>;
    /**
     * Called for each rendered frame.
     */
    checkInputs(): void;
    private _moveRelative;
    private _rotateRelative;
    private _moveScene;
    /**
     * These are set to the desired default behaviour.
     */
    private _wheelXAction;
    private _wheelXActionCoordinate;
    private _wheelYAction;
    private _wheelYActionCoordinate;
    private _wheelZAction;
    private _wheelZActionCoordinate;
    /**
     * Update the camera according to any configured properties for the 3
     * mouse-wheel axis.
     */
    private _updateCamera;
    /**
     * Update one property of the camera.
     * @param value
     * @param cameraProperty
     * @param coordinate
     */
    private _updateCameraProperty;
}

}
declare module "babylonjs/Cameras/Inputs/freeCameraTouchInput" {
import { ICameraInput } from "babylonjs/Cameras/cameraInputsManager";
import { FreeCamera } from "babylonjs/Cameras/freeCamera";
/**
 * Manage the touch inputs to control the movement of a free camera.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/customizingCameraInputs
 */
export class FreeCameraTouchInput implements ICameraInput<FreeCamera> {
    /**
     * Define if mouse events can be treated as touch events
     */
    allowMouse: boolean;
    /**
     * Defines the camera the input is attached to.
     */
    camera: FreeCamera;
    /**
     * Defines the touch sensibility for rotation.
     * The lower the faster.
     */
    touchAngularSensibility: number;
    /**
     * Defines the touch sensibility for move.
     * The lower the faster.
     */
    touchMoveSensibility: number;
    /**
     * Swap touch actions so that one touch is used for rotation and multiple for movement
     */
    singleFingerRotate: boolean;
    private _offsetX;
    private _offsetY;
    private _pointerPressed;
    private _pointerInput?;
    private _observer;
    private _onLostFocus;
    private _isSafari;
    /**
     * Manage the touch inputs to control the movement of a free camera.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/customizingCameraInputs
     * @param allowMouse Defines if mouse events can be treated as touch events
     */
    constructor(
    /**
     * Define if mouse events can be treated as touch events
     */
    allowMouse?: boolean);
    /**
     * Attach the input controls to a specific dom element to get the input from.
     * @param noPreventDefault Defines whether event caught by the controls should call preventdefault() (https://developer.mozilla.org/en-US/docs/Web/API/Event/preventDefault)
     */
    attachControl(noPreventDefault?: boolean): void;
    /**
     * Detach the current controls from the specified dom element.
     */
    detachControl(): void;
    /**
     * Update the current camera state depending on the inputs that have been used this frame.
     * This is a dynamically created lambda to avoid the performance penalty of looping for inputs in the render loop.
     */
    checkInputs(): void;
    /**
     * Gets the class name of the current input.
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Get the friendly name associated with the input class.
     * @returns the input friendly name
     */
    getSimpleName(): string;
}

}
declare module "babylonjs/Cameras/Inputs/freeCameraVirtualJoystickInput" {
import { VirtualJoystick } from "babylonjs/Misc/virtualJoystick";
import { ICameraInput } from "babylonjs/Cameras/cameraInputsManager";
import { FreeCamera } from "babylonjs/Cameras/freeCamera";
module "babylonjs/Cameras/freeCameraInputsManager" {
    interface FreeCameraInputsManager {
        /**
         * Add virtual joystick input support to the input manager.
         * @returns the current input manager
         */
        addVirtualJoystick(): FreeCameraInputsManager;
    }
}
/**
 * Manage the Virtual Joystick inputs to control the movement of a free camera.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/customizingCameraInputs
 */
export class FreeCameraVirtualJoystickInput implements ICameraInput<FreeCamera> {
    /**
     * Defines the camera the input is attached to.
     */
    camera: FreeCamera;
    private _leftjoystick;
    private _rightjoystick;
    /**
     * Gets the left stick of the virtual joystick.
     * @returns The virtual Joystick
     */
    getLeftJoystick(): VirtualJoystick;
    /**
     * Gets the right stick of the virtual joystick.
     * @returns The virtual Joystick
     */
    getRightJoystick(): VirtualJoystick;
    /**
     * Update the current camera state depending on the inputs that have been used this frame.
     * This is a dynamically created lambda to avoid the performance penalty of looping for inputs in the render loop.
     */
    checkInputs(): void;
    /**
     * Attach the input controls to a specific dom element to get the input from.
     */
    attachControl(): void;
    /**
     * Detach the current controls from the specified dom element.
     */
    detachControl(): void;
    /**
     * Gets the class name of the current input.
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Get the friendly name associated with the input class.
     * @returns the input friendly name
     */
    getSimpleName(): string;
}

}
declare module "babylonjs/Cameras/Inputs/index" {
export * from "babylonjs/Cameras/Inputs/BaseCameraMouseWheelInput";
export * from "babylonjs/Cameras/Inputs/BaseCameraPointersInput";
export * from "babylonjs/Cameras/Inputs/arcRotateCameraGamepadInput";
export * from "babylonjs/Cameras/Inputs/arcRotateCameraKeyboardMoveInput";
export * from "babylonjs/Cameras/Inputs/arcRotateCameraMouseWheelInput";
export * from "babylonjs/Cameras/Inputs/arcRotateCameraPointersInput";
export * from "babylonjs/Cameras/Inputs/arcRotateCameraVRDeviceOrientationInput";
export * from "babylonjs/Cameras/Inputs/flyCameraKeyboardInput";
export * from "babylonjs/Cameras/Inputs/flyCameraMouseInput";
export * from "babylonjs/Cameras/Inputs/followCameraKeyboardMoveInput";
export * from "babylonjs/Cameras/Inputs/followCameraMouseWheelInput";
export * from "babylonjs/Cameras/Inputs/followCameraPointersInput";
export * from "babylonjs/Cameras/Inputs/freeCameraDeviceOrientationInput";
export * from "babylonjs/Cameras/Inputs/freeCameraGamepadInput";
export * from "babylonjs/Cameras/Inputs/freeCameraKeyboardMoveInput";
export * from "babylonjs/Cameras/Inputs/freeCameraMouseInput";
export * from "babylonjs/Cameras/Inputs/freeCameraMouseWheelInput";
export * from "babylonjs/Cameras/Inputs/freeCameraTouchInput";
export * from "babylonjs/Cameras/Inputs/freeCameraVirtualJoystickInput";

}
declare module "babylonjs/Cameras/RigModes/index" {
export * from "babylonjs/Cameras/RigModes/stereoscopicAnaglyphRigMode";
export * from "babylonjs/Cameras/RigModes/stereoscopicRigMode";
export * from "babylonjs/Cameras/RigModes/vrRigMode";
export * from "babylonjs/Cameras/RigModes/webVRRigMode";

}
declare module "babylonjs/Cameras/RigModes/stereoscopicAnaglyphRigMode" {
import { Camera } from "babylonjs/Cameras/camera";
/**
 * @internal
 */
export function setStereoscopicAnaglyphRigMode(camera: Camera): void;

}
declare module "babylonjs/Cameras/RigModes/stereoscopicRigMode" {
import { Camera } from "babylonjs/Cameras/camera";
/**
 * @internal
 */
export function setStereoscopicRigMode(camera: Camera): void;

}
declare module "babylonjs/Cameras/RigModes/vrRigMode" {
import { Camera } from "babylonjs/Cameras/camera";
/**
 * @internal
 */
export function setVRRigMode(camera: Camera, rigParams: any): void;

}
declare module "babylonjs/Cameras/RigModes/webVRRigMode" {
import { Camera } from "babylonjs/Cameras/camera";
/**
 * @internal
 */
export function setWebVRRigMode(camera: Camera, rigParams: any): void;

}
declare module "babylonjs/Cameras/Stereoscopic/anaglyphArcRotateCamera" {
import { ArcRotateCamera } from "babylonjs/Cameras/arcRotateCamera";
import { Scene } from "babylonjs/scene";
import { Vector3 } from "babylonjs/Maths/math.vector";
/**
 * Camera used to simulate anaglyphic rendering (based on ArcRotateCamera)
 * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_introduction#anaglyph-cameras
 */
export class AnaglyphArcRotateCamera extends ArcRotateCamera {
    /**
     * Creates a new AnaglyphArcRotateCamera
     * @param name defines camera name
     * @param alpha defines alpha angle (in radians)
     * @param beta defines beta angle (in radians)
     * @param radius defines radius
     * @param target defines camera target
     * @param interaxialDistance defines distance between each color axis
     * @param scene defines the hosting scene
     */
    constructor(name: string, alpha: number, beta: number, radius: number, target: Vector3, interaxialDistance: number, scene?: Scene);
    /**
     * Gets camera class name
     * @returns AnaglyphArcRotateCamera
     */
    getClassName(): string;
    protected _setRigMode: any;
}

}
declare module "babylonjs/Cameras/Stereoscopic/anaglyphFreeCamera" {
import { FreeCamera } from "babylonjs/Cameras/freeCamera";
import { Scene } from "babylonjs/scene";
import { Vector3 } from "babylonjs/Maths/math.vector";
/**
 * Camera used to simulate anaglyphic rendering (based on FreeCamera)
 * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_introduction#anaglyph-cameras
 */
export class AnaglyphFreeCamera extends FreeCamera {
    /**
     * Creates a new AnaglyphFreeCamera
     * @param name defines camera name
     * @param position defines initial position
     * @param interaxialDistance defines distance between each color axis
     * @param scene defines the hosting scene
     */
    constructor(name: string, position: Vector3, interaxialDistance: number, scene?: Scene);
    /**
     * Gets camera class name
     * @returns AnaglyphFreeCamera
     */
    getClassName(): string;
    protected _setRigMode: any;
}

}
declare module "babylonjs/Cameras/Stereoscopic/anaglyphGamepadCamera" {
import { GamepadCamera } from "babylonjs/Cameras/gamepadCamera";
import { Scene } from "babylonjs/scene";
import { Vector3 } from "babylonjs/Maths/math.vector";
/**
 * Camera used to simulate anaglyphic rendering (based on GamepadCamera)
 * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_introduction#anaglyph-cameras
 */
export class AnaglyphGamepadCamera extends GamepadCamera {
    /**
     * Creates a new AnaglyphGamepadCamera
     * @param name defines camera name
     * @param position defines initial position
     * @param interaxialDistance defines distance between each color axis
     * @param scene defines the hosting scene
     */
    constructor(name: string, position: Vector3, interaxialDistance: number, scene?: Scene);
    /**
     * Gets camera class name
     * @returns AnaglyphGamepadCamera
     */
    getClassName(): string;
    protected _setRigMode: any;
}

}
declare module "babylonjs/Cameras/Stereoscopic/anaglyphUniversalCamera" {
import { UniversalCamera } from "babylonjs/Cameras/universalCamera";
import { Scene } from "babylonjs/scene";
import { Vector3 } from "babylonjs/Maths/math.vector";
/**
 * Camera used to simulate anaglyphic rendering (based on UniversalCamera)
 * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_introduction#anaglyph-cameras
 */
export class AnaglyphUniversalCamera extends UniversalCamera {
    /**
     * Creates a new AnaglyphUniversalCamera
     * @param name defines camera name
     * @param position defines initial position
     * @param interaxialDistance defines distance between each color axis
     * @param scene defines the hosting scene
     */
    constructor(name: string, position: Vector3, interaxialDistance: number, scene?: Scene);
    /**
     * Gets camera class name
     * @returns AnaglyphUniversalCamera
     */
    getClassName(): string;
    protected _setRigMode: any;
}

}
declare module "babylonjs/Cameras/Stereoscopic/index" {
export * from "babylonjs/Cameras/Stereoscopic/anaglyphArcRotateCamera";
export * from "babylonjs/Cameras/Stereoscopic/anaglyphFreeCamera";
export * from "babylonjs/Cameras/Stereoscopic/anaglyphGamepadCamera";
export * from "babylonjs/Cameras/Stereoscopic/anaglyphUniversalCamera";
export * from "babylonjs/Cameras/Stereoscopic/stereoscopicArcRotateCamera";
export * from "babylonjs/Cameras/Stereoscopic/stereoscopicFreeCamera";
export * from "babylonjs/Cameras/Stereoscopic/stereoscopicGamepadCamera";
export * from "babylonjs/Cameras/Stereoscopic/stereoscopicUniversalCamera";
export * from "babylonjs/Cameras/Stereoscopic/stereoscopicScreenUniversalCamera";

}
declare module "babylonjs/Cameras/Stereoscopic/stereoscopicArcRotateCamera" {
import { ArcRotateCamera } from "babylonjs/Cameras/arcRotateCamera";
import { Scene } from "babylonjs/scene";
import { Vector3 } from "babylonjs/Maths/math.vector";
/**
 * Camera used to simulate stereoscopic rendering (based on ArcRotateCamera)
 * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras
 */
export class StereoscopicArcRotateCamera extends ArcRotateCamera {
    /**
     * Creates a new StereoscopicArcRotateCamera
     * @param name defines camera name
     * @param alpha defines alpha angle (in radians)
     * @param beta defines beta angle (in radians)
     * @param radius defines radius
     * @param target defines camera target
     * @param interaxialDistance defines distance between each color axis
     * @param isStereoscopicSideBySide defines is stereoscopic is done side by side or over under
     * @param scene defines the hosting scene
     */
    constructor(name: string, alpha: number, beta: number, radius: number, target: Vector3, interaxialDistance: number, isStereoscopicSideBySide: boolean, scene?: Scene);
    /**
     * Gets camera class name
     * @returns StereoscopicArcRotateCamera
     */
    getClassName(): string;
    protected _setRigMode: any;
}

}
declare module "babylonjs/Cameras/Stereoscopic/stereoscopicFreeCamera" {
import { FreeCamera } from "babylonjs/Cameras/freeCamera";
import { Scene } from "babylonjs/scene";
import { Vector3 } from "babylonjs/Maths/math.vector";
/**
 * Camera used to simulate stereoscopic rendering (based on FreeCamera)
 * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras
 */
export class StereoscopicFreeCamera extends FreeCamera {
    /**
     * Creates a new StereoscopicFreeCamera
     * @param name defines camera name
     * @param position defines initial position
     * @param interaxialDistance defines distance between each color axis
     * @param isStereoscopicSideBySide defines is stereoscopic is done side by side or over under
     * @param scene defines the hosting scene
     */
    constructor(name: string, position: Vector3, interaxialDistance: number, isStereoscopicSideBySide: boolean, scene?: Scene);
    /**
     * Gets camera class name
     * @returns StereoscopicFreeCamera
     */
    getClassName(): string;
    protected _setRigMode: any;
}

}
declare module "babylonjs/Cameras/Stereoscopic/stereoscopicGamepadCamera" {
import { GamepadCamera } from "babylonjs/Cameras/gamepadCamera";
import { Scene } from "babylonjs/scene";
import { Vector3 } from "babylonjs/Maths/math.vector";
/**
 * Camera used to simulate stereoscopic rendering (based on GamepadCamera)
 * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras
 */
export class StereoscopicGamepadCamera extends GamepadCamera {
    /**
     * Creates a new StereoscopicGamepadCamera
     * @param name defines camera name
     * @param position defines initial position
     * @param interaxialDistance defines distance between each color axis
     * @param isStereoscopicSideBySide defines is stereoscopic is done side by side or over under
     * @param scene defines the hosting scene
     */
    constructor(name: string, position: Vector3, interaxialDistance: number, isStereoscopicSideBySide: boolean, scene?: Scene);
    /**
     * Gets camera class name
     * @returns StereoscopicGamepadCamera
     */
    getClassName(): string;
    protected _setRigMode: any;
}

}
declare module "babylonjs/Cameras/Stereoscopic/stereoscopicScreenUniversalCamera" {
import { Camera } from "babylonjs/Cameras/camera";
import { UniversalCamera } from "babylonjs/Cameras/universalCamera";
import { Scene } from "babylonjs/scene";
import { Vector3 } from "babylonjs/Maths/math.vector";
import { Nullable } from "babylonjs/types";
/**
 * Camera used to simulate stereoscopic rendering on real screens (based on UniversalCamera)
 * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras
 */
export class StereoscopicScreenUniversalCamera extends UniversalCamera {
    private _distanceToProjectionPlane;
    private _distanceBetweenEyes;
    set distanceBetweenEyes(newValue: number);
    /**
     * distance between the eyes
     */
    get distanceBetweenEyes(): number;
    set distanceToProjectionPlane(newValue: number);
    /**
     * Distance to projection plane (should be the same units the like distance between the eyes)
     */
    get distanceToProjectionPlane(): number;
    /**
     * Creates a new StereoscopicScreenUniversalCamera
     * @param name defines camera name
     * @param position defines initial position
     * @param scene defines the hosting scene
     * @param distanceToProjectionPlane defines distance between each color axis. The rig cameras will receive this as their negative z position!
     * @param distanceBetweenEyes defines is stereoscopic is done side by side or over under
     */
    constructor(name: string, position: Vector3, scene?: Scene, distanceToProjectionPlane?: number, distanceBetweenEyes?: number);
    /**
     * Gets camera class name
     * @returns StereoscopicScreenUniversalCamera
     */
    getClassName(): string;
    /**
     * @internal
     */
    createRigCamera(name: string): Nullable<Camera>;
    /**
     * @internal
     */
    _updateRigCameras(): void;
    private _updateCamera;
    protected _setRigMode(): void;
}

}
declare module "babylonjs/Cameras/Stereoscopic/stereoscopicUniversalCamera" {
import { UniversalCamera } from "babylonjs/Cameras/universalCamera";
import { Scene } from "babylonjs/scene";
import { Vector3 } from "babylonjs/Maths/math.vector";
/**
 * Camera used to simulate stereoscopic rendering (based on UniversalCamera)
 * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras
 */
export class StereoscopicUniversalCamera extends UniversalCamera {
    /**
     * Creates a new StereoscopicUniversalCamera
     * @param name defines camera name
     * @param position defines initial position
     * @param interaxialDistance defines distance between each color axis
     * @param isStereoscopicSideBySide defines is stereoscopic is done side by side or over under
     * @param scene defines the hosting scene
     */
    constructor(name: string, position: Vector3, interaxialDistance: number, isStereoscopicSideBySide: boolean, scene?: Scene);
    /**
     * Gets camera class name
     * @returns StereoscopicUniversalCamera
     */
    getClassName(): string;
    protected _setRigMode: any;
}

}
declare module "babylonjs/Cameras/targetCamera" {
import { Nullable } from "babylonjs/types";
import { Camera } from "babylonjs/Cameras/camera";
import { Scene } from "babylonjs/scene";
import { Quaternion, Matrix, Vector3, Vector2 } from "babylonjs/Maths/math.vector";
/**
 * A target camera takes a mesh or position as a target and continues to look at it while it moves.
 * This is the base of the follow, arc rotate cameras and Free camera
 * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras
 */
export class TargetCamera extends Camera {
    private static _RigCamTransformMatrix;
    private static _TargetTransformMatrix;
    private static _TargetFocalPoint;
    private _tmpUpVector;
    private _tmpTargetVector;
    /**
     * Define the current direction the camera is moving to
     */
    cameraDirection: Vector3;
    /**
     * Define the current rotation the camera is rotating to
     */
    cameraRotation: Vector2;
    /** Gets or sets a boolean indicating that the scaling of the parent hierarchy will not be taken in account by the camera */
    ignoreParentScaling: boolean;
    /**
     * When set, the up vector of the camera will be updated by the rotation of the camera
     */
    updateUpVectorFromRotation: boolean;
    private _tmpQuaternion;
    /**
     * Define the current rotation of the camera
     */
    rotation: Vector3;
    /**
     * Define the current rotation of the camera as a quaternion to prevent Gimbal lock
     */
    rotationQuaternion: Quaternion;
    /**
     * Define the current speed of the camera
     */
    speed: number;
    /**
     * Add constraint to the camera to prevent it to move freely in all directions and
     * around all axis.
     */
    noRotationConstraint: boolean;
    /**
     * Reverses mouselook direction to 'natural' panning as opposed to traditional direct
     * panning
     */
    invertRotation: boolean;
    /**
     * Speed multiplier for inverse camera panning
     */
    inverseRotationSpeed: number;
    /**
     * Define the current target of the camera as an object or a position.
     * Please note that locking a target will disable panning.
     */
    lockedTarget: any;
    /** @internal */
    _currentTarget: Vector3;
    /** @internal */
    _initialFocalDistance: number;
    /** @internal */
    _viewMatrix: Matrix;
    /** @internal */
    _camMatrix: Matrix;
    /** @internal */
    _cameraTransformMatrix: Matrix;
    /** @internal */
    _cameraRotationMatrix: Matrix;
    /** @internal */
    _referencePoint: Vector3;
    /** @internal */
    _transformedReferencePoint: Vector3;
    /** @internal */
    _reset: () => void;
    private _defaultUp;
    /**
     * Instantiates a target camera that takes a mesh or position as a target and continues to look at it while it moves.
     * This is the base of the follow, arc rotate cameras and Free camera
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras
     * @param name Defines the name of the camera in the scene
     * @param position Defines the start position of the camera in the scene
     * @param scene Defines the scene the camera belongs to
     * @param setActiveOnSceneIfNoneActive Defines whether the camera should be marked as active if not other active cameras have been defined
     */
    constructor(name: string, position: Vector3, scene?: Scene, setActiveOnSceneIfNoneActive?: boolean);
    /**
     * Gets the position in front of the camera at a given distance.
     * @param distance The distance from the camera we want the position to be
     * @returns the position
     */
    getFrontPosition(distance: number): Vector3;
    /** @internal */
    _getLockedTargetPosition(): Nullable<Vector3>;
    private _storedPosition;
    private _storedRotation;
    private _storedRotationQuaternion;
    /**
     * Store current camera state of the camera (fov, position, rotation, etc..)
     * @returns the camera
     */
    storeState(): Camera;
    /**
     * Restored camera state. You must call storeState() first
     * @returns whether it was successful or not
     * @internal
     */
    _restoreStateValues(): boolean;
    /** @internal */
    _initCache(): void;
    /**
     * @internal
     */
    _updateCache(ignoreParentClass?: boolean): void;
    /** @internal */
    _isSynchronizedViewMatrix(): boolean;
    /** @internal */
    _computeLocalCameraSpeed(): number;
    /**
     * Defines the target the camera should look at.
     * @param target Defines the new target as a Vector
     */
    setTarget(target: Vector3): void;
    /**
     * Defines the target point of the camera.
     * The camera looks towards it form the radius distance.
     */
    get target(): Vector3;
    set target(value: Vector3);
    /**
     * Return the current target position of the camera. This value is expressed in local space.
     * @returns the target position
     */
    getTarget(): Vector3;
    /** @internal */
    _decideIfNeedsToMove(): boolean;
    /** @internal */
    _updatePosition(): void;
    /** @internal */
    _checkInputs(): void;
    protected _updateCameraRotationMatrix(): void;
    /**
     * Update the up vector to apply the rotation of the camera (So if you changed the camera rotation.z this will let you update the up vector as well)
     * @returns the current camera
     */
    private _rotateUpVectorWithCameraRotationMatrix;
    private _cachedRotationZ;
    private _cachedQuaternionRotationZ;
    /** @internal */
    _getViewMatrix(): Matrix;
    protected _computeViewMatrix(position: Vector3, target: Vector3, up: Vector3): void;
    /**
     * @internal
     */
    createRigCamera(name: string, cameraIndex: number): Nullable<Camera>;
    /**
     * @internal
     */
    _updateRigCameras(): void;
    private _getRigCamPositionAndTarget;
    /**
     * Gets the current object class name.
     * @returns the class name
     */
    getClassName(): string;
}

}
declare module "babylonjs/Cameras/touchCamera" {
import { FreeCamera } from "babylonjs/Cameras/freeCamera";
import { Scene } from "babylonjs/scene";
import { Vector3 } from "babylonjs/Maths/math.vector";
/**
 * This represents a FPS type of camera controlled by touch.
 * This is like a universal camera minus the Gamepad controls.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_introduction#universal-camera
 */
export class TouchCamera extends FreeCamera {
    /**
     * Defines the touch sensibility for rotation.
     * The higher the faster.
     */
    get touchAngularSensibility(): number;
    set touchAngularSensibility(value: number);
    /**
     * Defines the touch sensibility for move.
     * The higher the faster.
     */
    get touchMoveSensibility(): number;
    set touchMoveSensibility(value: number);
    /**
     * Instantiates a new touch camera.
     * This represents a FPS type of camera controlled by touch.
     * This is like a universal camera minus the Gamepad controls.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_introduction#universal-camera
     * @param name Define the name of the camera in the scene
     * @param position Define the start position of the camera in the scene
     * @param scene Define the scene the camera belongs to
     */
    constructor(name: string, position: Vector3, scene?: Scene);
    /**
     * Gets the current object class name.
     * @returns the class name
     */
    getClassName(): string;
    /** @internal */
    _setupInputs(): void;
}

}
declare module "babylonjs/Cameras/universalCamera" {
import { TouchCamera } from "babylonjs/Cameras/touchCamera";
import { Scene } from "babylonjs/scene";
import { Vector3 } from "babylonjs/Maths/math.vector";
import "babylonjs/Gamepads/gamepadSceneComponent";
/**
 * The Universal Camera is the one to choose for first person shooter type games, and works with all the keyboard, mouse, touch and gamepads. This replaces the earlier Free Camera,
 * which still works and will still be found in many Playgrounds.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_introduction#universal-camera
 */
export class UniversalCamera extends TouchCamera {
    /**
     * Defines the gamepad rotation sensibility.
     * This is the threshold from when rotation starts to be accounted for to prevent jittering.
     */
    get gamepadAngularSensibility(): number;
    set gamepadAngularSensibility(value: number);
    /**
     * Defines the gamepad move sensibility.
     * This is the threshold from when moving starts to be accounted for to prevent jittering.
     */
    get gamepadMoveSensibility(): number;
    set gamepadMoveSensibility(value: number);
    /**
     * The Universal Camera is the one to choose for first person shooter type games, and works with all the keyboard, mouse, touch and gamepads. This replaces the earlier Free Camera,
     * which still works and will still be found in many Playgrounds.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_introduction#universal-camera
     * @param name Define the name of the camera in the scene
     * @param position Define the start position of the camera in the scene
     * @param scene Define the scene the camera belongs to
     */
    constructor(name: string, position: Vector3, scene?: Scene);
    /**
     * Gets the current object class name.
     * @returns the class name
     */
    getClassName(): string;
}

}
declare module "babylonjs/Cameras/virtualJoysticksCamera" {
import { FreeCamera } from "babylonjs/Cameras/freeCamera";
import { Scene } from "babylonjs/scene";
import { Vector3 } from "babylonjs/Maths/math.vector";
import "babylonjs/Cameras/Inputs/freeCameraVirtualJoystickInput";
/**
 * This represents a free type of camera. It can be useful in First Person Shooter game for instance.
 * It is identical to the Free Camera and simply adds by default a virtual joystick.
 * Virtual Joysticks are on-screen 2D graphics that are used to control the camera or other scene items.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_introduction#virtual-joysticks-camera
 */
export class VirtualJoysticksCamera extends FreeCamera {
    /**
     * Instantiates a VirtualJoysticksCamera. It can be useful in First Person Shooter game for instance.
     * It is identical to the Free Camera and simply adds by default a virtual joystick.
     * Virtual Joysticks are on-screen 2D graphics that are used to control the camera or other scene items.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_introduction#virtual-joysticks-camera
     * @param name Define the name of the camera in the scene
     * @param position Define the start position of the camera in the scene
     * @param scene Define the scene the camera belongs to
     */
    constructor(name: string, position: Vector3, scene?: Scene);
    /**
     * Gets the current object class name.
     * @returns the class name
     */
    getClassName(): string;
}

}
declare module "babylonjs/Cameras/VR/index" {
export * from "babylonjs/Cameras/VR/vrCameraMetrics";
export * from "babylonjs/Cameras/VR/vrDeviceOrientationArcRotateCamera";
export * from "babylonjs/Cameras/VR/vrDeviceOrientationFreeCamera";
export * from "babylonjs/Cameras/VR/vrDeviceOrientationGamepadCamera";
export * from "babylonjs/Cameras/VR/vrExperienceHelper";
export * from "babylonjs/Cameras/VR/webVRCamera";

}
declare module "babylonjs/Cameras/VR/vrCameraMetrics" {
import { Matrix } from "babylonjs/Maths/math.vector";
/**
 * This represents all the required metrics to create a VR camera.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_introduction#device-orientation-camera
 */
export class VRCameraMetrics {
    /**
     * Define the horizontal resolution off the screen.
     */
    hResolution: number;
    /**
     * Define the vertical resolution off the screen.
     */
    vResolution: number;
    /**
     * Define the horizontal screen size.
     */
    hScreenSize: number;
    /**
     * Define the vertical screen size.
     */
    vScreenSize: number;
    /**
     * Define the vertical screen center position.
     */
    vScreenCenter: number;
    /**
     * Define the distance of the eyes to the screen.
     */
    eyeToScreenDistance: number;
    /**
     * Define the distance between both lenses
     */
    lensSeparationDistance: number;
    /**
     * Define the distance between both viewer's eyes.
     */
    interpupillaryDistance: number;
    /**
     * Define the distortion factor of the VR postprocess.
     * Please, touch with care.
     */
    distortionK: number[];
    /**
     * Define the chromatic aberration correction factors for the VR post process.
     */
    chromaAbCorrection: number[];
    /**
     * Define the scale factor of the post process.
     * The smaller the better but the slower.
     */
    postProcessScaleFactor: number;
    /**
     * Define an offset for the lens center.
     */
    lensCenterOffset: number;
    /**
     * Define if the current vr camera should compensate the distortion of the lens or not.
     */
    compensateDistortion: boolean;
    /**
     * Defines if multiview should be enabled when rendering (Default: false)
     */
    multiviewEnabled: boolean;
    /**
     * Gets the rendering aspect ratio based on the provided resolutions.
     */
    get aspectRatio(): number;
    /**
     * Gets the aspect ratio based on the FOV, scale factors, and real screen sizes.
     */
    get aspectRatioFov(): number;
    /**
     * @internal
     */
    get leftHMatrix(): Matrix;
    /**
     * @internal
     */
    get rightHMatrix(): Matrix;
    /**
     * @internal
     */
    get leftPreViewMatrix(): Matrix;
    /**
     * @internal
     */
    get rightPreViewMatrix(): Matrix;
    /**
     * Get the default VRMetrics based on the most generic setup.
     * @returns the default vr metrics
     */
    static GetDefault(): VRCameraMetrics;
}

}
declare module "babylonjs/Cameras/VR/vrDeviceOrientationArcRotateCamera" {
import { ArcRotateCamera } from "babylonjs/Cameras/arcRotateCamera";
import { VRCameraMetrics } from "babylonjs/Cameras/VR/vrCameraMetrics";
import { Scene } from "babylonjs/scene";
import { Vector3 } from "babylonjs/Maths/math.vector";
import "babylonjs/Cameras/Inputs/arcRotateCameraVRDeviceOrientationInput";
/**
 * Camera used to simulate VR rendering (based on ArcRotateCamera)
 * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_introduction#vr-device-orientation-cameras
 */
export class VRDeviceOrientationArcRotateCamera extends ArcRotateCamera {
    /**
     * Creates a new VRDeviceOrientationArcRotateCamera
     * @param name defines camera name
     * @param alpha defines the camera rotation along the longitudinal axis
     * @param beta defines the camera rotation along the latitudinal axis
     * @param radius defines the camera distance from its target
     * @param target defines the camera target
     * @param scene defines the scene the camera belongs to
     * @param compensateDistortion defines if the camera needs to compensate the lens distortion
     * @param vrCameraMetrics defines the vr metrics associated to the camera
     */
    constructor(name: string, alpha: number, beta: number, radius: number, target: Vector3, scene?: Scene, compensateDistortion?: boolean, vrCameraMetrics?: VRCameraMetrics);
    /**
     * Gets camera class name
     * @returns VRDeviceOrientationArcRotateCamera
     */
    getClassName(): string;
    protected _setRigMode: any;
}

}
declare module "babylonjs/Cameras/VR/vrDeviceOrientationFreeCamera" {
import { DeviceOrientationCamera } from "babylonjs/Cameras/deviceOrientationCamera";
import { VRCameraMetrics } from "babylonjs/Cameras/VR/vrCameraMetrics";
import { Scene } from "babylonjs/scene";
import { Vector3 } from "babylonjs/Maths/math.vector";
/**
 * Camera used to simulate VR rendering (based on FreeCamera)
 * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_introduction#vr-device-orientation-cameras
 */
export class VRDeviceOrientationFreeCamera extends DeviceOrientationCamera {
    /**
     * Creates a new VRDeviceOrientationFreeCamera
     * @param name defines camera name
     * @param position defines the start position of the camera
     * @param scene defines the scene the camera belongs to
     * @param compensateDistortion defines if the camera needs to compensate the lens distortion
     * @param vrCameraMetrics defines the vr metrics associated to the camera
     */
    constructor(name: string, position: Vector3, scene?: Scene, compensateDistortion?: boolean, vrCameraMetrics?: VRCameraMetrics);
    /**
     * Gets camera class name
     * @returns VRDeviceOrientationFreeCamera
     */
    getClassName(): string;
    protected _setRigMode: any;
}

}
declare module "babylonjs/Cameras/VR/vrDeviceOrientationGamepadCamera" {
import { VRDeviceOrientationFreeCamera } from "babylonjs/Cameras/VR/vrDeviceOrientationFreeCamera";
import { VRCameraMetrics } from "babylonjs/Cameras/VR/vrCameraMetrics";
import { Scene } from "babylonjs/scene";
import { Vector3 } from "babylonjs/Maths/math.vector";
import "babylonjs/Gamepads/gamepadSceneComponent";
/**
 * Camera used to simulate VR rendering (based on VRDeviceOrientationFreeCamera)
 * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_introduction#vr-device-orientation-cameras
 */
export class VRDeviceOrientationGamepadCamera extends VRDeviceOrientationFreeCamera {
    /**
     * Creates a new VRDeviceOrientationGamepadCamera
     * @param name defines camera name
     * @param position defines the start position of the camera
     * @param scene defines the scene the camera belongs to
     * @param compensateDistortion defines if the camera needs to compensate the lens distortion
     * @param vrCameraMetrics defines the vr metrics associated to the camera
     */
    constructor(name: string, position: Vector3, scene?: Scene, compensateDistortion?: boolean, vrCameraMetrics?: VRCameraMetrics);
    /**
     * Gets camera class name
     * @returns VRDeviceOrientationGamepadCamera
     */
    getClassName(): string;
    protected _setRigMode: any;
}

}
declare module "babylonjs/Cameras/VR/vrExperienceHelper" {
import { Observable } from "babylonjs/Misc/observable";
import { Nullable } from "babylonjs/types";
import { Camera } from "babylonjs/Cameras/camera";
import { DeviceOrientationCamera } from "babylonjs/Cameras/deviceOrientationCamera";
import { VRDeviceOrientationFreeCamera } from "babylonjs/Cameras/VR/vrDeviceOrientationFreeCamera";
import { WebVROptions } from "babylonjs/Cameras/VR/webVRCamera";
import { WebVRFreeCamera } from "babylonjs/Cameras/VR/webVRCamera";
import { Scene } from "babylonjs/scene";
import { Vector3 } from "babylonjs/Maths/math.vector";
import { Color3 } from "babylonjs/Maths/math.color";
import { WebVRController } from "babylonjs/Gamepads/Controllers/webVRController";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { Mesh } from "babylonjs/Meshes/mesh";
import { PickingInfo } from "babylonjs/Collisions/pickingInfo";
import { EasingFunction } from "babylonjs/Animations/easing";
import { VRCameraMetrics } from "babylonjs/Cameras/VR/vrCameraMetrics";
import "babylonjs/Gamepads/gamepadSceneComponent";
import "babylonjs/Animations/animatable";
import { WebXRDefaultExperience } from "babylonjs/XR/webXRDefaultExperience";
/**
 * Options to modify the vr teleportation behavior.
 */
export interface VRTeleportationOptions {
    /**
     * The name of the mesh which should be used as the teleportation floor. (default: null)
     */
    floorMeshName?: string;
    /**
     * A list of meshes to be used as the teleportation floor. (default: empty)
     */
    floorMeshes?: Mesh[];
    /**
     * The teleportation mode. (default: TELEPORTATIONMODE_CONSTANTTIME)
     */
    teleportationMode?: number;
    /**
     * The duration of the animation in ms, apply when animationMode is TELEPORTATIONMODE_CONSTANTTIME. (default 122ms)
     */
    teleportationTime?: number;
    /**
     * The speed of the animation in distance/sec, apply when animationMode is TELEPORTATIONMODE_CONSTANTSPEED. (default 20 units / sec)
     */
    teleportationSpeed?: number;
    /**
     * The easing function used in the animation or null for Linear. (default CircleEase)
     */
    easingFunction?: EasingFunction;
}
/**
 * Options to modify the vr experience helper's behavior.
 */
export interface VRExperienceHelperOptions extends WebVROptions {
    /**
     * Create a DeviceOrientationCamera to be used as your out of vr camera. (default: true)
     */
    createDeviceOrientationCamera?: boolean;
    /**
     * Create a VRDeviceOrientationFreeCamera to be used for VR when no external HMD is found. (default: true)
     */
    createFallbackVRDeviceOrientationFreeCamera?: boolean;
    /**
     * Uses the main button on the controller to toggle the laser casted. (default: true)
     */
    laserToggle?: boolean;
    /**
     * A list of meshes to be used as the teleportation floor. If specified, teleportation will be enabled (default: undefined)
     */
    floorMeshes?: Mesh[];
    /**
     * Distortion metrics for the fallback vrDeviceOrientationCamera (default: VRCameraMetrics.Default)
     */
    vrDeviceOrientationCameraMetrics?: VRCameraMetrics;
    /**
     * Defines if WebXR should be used instead of WebVR (if available)
     */
    useXR?: boolean;
}
/**
 * Event containing information after VR has been entered
 */
export class OnAfterEnteringVRObservableEvent {
    /**
     * If entering vr was successful
     */
    success: boolean;
}
/**
 * Helps to quickly add VR support to an existing scene.
 * See https://doc.babylonjs.com/features/featuresDeepDive/cameras/webVRHelper
 * @deprecated
 */
export class VRExperienceHelper {
    /** Options to modify the vr experience helper's behavior. */
    webVROptions: VRExperienceHelperOptions;
    private _scene;
    private _position;
    private _btnVR;
    private _btnVRDisplayed;
    private _webVRsupported;
    private _webVRready;
    private _webVRrequesting;
    private _webVRpresenting;
    private _hasEnteredVR;
    private _fullscreenVRpresenting;
    private _inputElement;
    private _webVRCamera;
    private _vrDeviceOrientationCamera;
    private _deviceOrientationCamera;
    private _existingCamera;
    private _onKeyDown;
    private _onVrDisplayPresentChangeBind;
    private _onVRDisplayChangedBind;
    private _onVRRequestPresentStart;
    private _onVRRequestPresentComplete;
    /**
     * Gets or sets a boolean indicating that gaze can be enabled even if pointer lock is not engage (useful on iOS where fullscreen mode and pointer lock are not supported)
     */
    enableGazeEvenWhenNoPointerLock: boolean;
    /**
     * Gets or sets a boolean indicating that the VREXperienceHelper will exit VR if double tap is detected
     */
    exitVROnDoubleTap: boolean;
    /**
     * Observable raised right before entering VR.
     */
    onEnteringVRObservable: Observable<VRExperienceHelper>;
    /**
     * Observable raised when entering VR has completed.
     */
    onAfterEnteringVRObservable: Observable<OnAfterEnteringVRObservableEvent>;
    /**
     * Observable raised when exiting VR.
     */
    onExitingVRObservable: Observable<VRExperienceHelper>;
    /**
     * Observable raised when controller mesh is loaded.
     */
    onControllerMeshLoadedObservable: Observable<WebVRController>;
    /** Return this.onEnteringVRObservable
     * Note: This one is for backward compatibility. Please use onEnteringVRObservable directly
     */
    get onEnteringVR(): Observable<VRExperienceHelper>;
    /** Return this.onExitingVRObservable
     * Note: This one is for backward compatibility. Please use onExitingVRObservable directly
     */
    get onExitingVR(): Observable<VRExperienceHelper>;
    /** Return this.onControllerMeshLoadedObservable
     * Note: This one is for backward compatibility. Please use onControllerMeshLoadedObservable directly
     */
    get onControllerMeshLoaded(): Observable<WebVRController>;
    private _rayLength;
    private _useCustomVRButton;
    private _teleportationRequested;
    private _teleportActive;
    private _floorMeshName;
    private _floorMeshesCollection;
    private _teleportationMode;
    private _teleportationTime;
    private _teleportationSpeed;
    private _teleportationEasing;
    private _rotationAllowed;
    private _teleportBackwardsVector;
    private _teleportationTarget;
    private _isDefaultTeleportationTarget;
    private _postProcessMove;
    private _teleportationFillColor;
    private _teleportationBorderColor;
    private _rotationAngle;
    private _haloCenter;
    private _cameraGazer;
    private _padSensibilityUp;
    private _padSensibilityDown;
    private _leftController;
    private _rightController;
    private _gazeColor;
    private _laserColor;
    private _pickedLaserColor;
    private _pickedGazeColor;
    /**
     * Observable raised when a new mesh is selected based on meshSelectionPredicate
     */
    onNewMeshSelected: Observable<AbstractMesh>;
    /**
     * Observable raised when a new mesh is selected based on meshSelectionPredicate.
     * This observable will provide the mesh and the controller used to select the mesh
     */
    onMeshSelectedWithController: Observable<{
        mesh: AbstractMesh;
        controller: WebVRController;
    }>;
    /**
     * Observable raised when a new mesh is picked based on meshSelectionPredicate
     */
    onNewMeshPicked: Observable<PickingInfo>;
    private _circleEase;
    /**
     * Observable raised before camera teleportation
     */
    onBeforeCameraTeleport: Observable<Vector3>;
    /**
     *  Observable raised after camera teleportation
     */
    onAfterCameraTeleport: Observable<Vector3>;
    /**
     * Observable raised when current selected mesh gets unselected
     */
    onSelectedMeshUnselected: Observable<AbstractMesh>;
    private _raySelectionPredicate;
    /**
     * To be optionally changed by user to define custom ray selection
     */
    raySelectionPredicate: (mesh: AbstractMesh) => boolean;
    /**
     * To be optionally changed by user to define custom selection logic (after ray selection)
     */
    meshSelectionPredicate: (mesh: AbstractMesh) => boolean;
    /**
     * Set teleportation enabled. If set to false camera teleportation will be disabled but camera rotation will be kept.
     */
    teleportationEnabled: boolean;
    private _defaultHeight;
    private _teleportationInitialized;
    private _interactionsEnabled;
    private _interactionsRequested;
    private _displayGaze;
    private _displayLaserPointer;
    /**
     * The mesh used to display where the user is going to teleport.
     */
    get teleportationTarget(): Mesh;
    /**
     * Sets the mesh to be used to display where the user is going to teleport.
     */
    set teleportationTarget(value: Mesh);
    /**
     * The mesh used to display where the user is selecting, this mesh will be cloned and set as the gazeTracker for the left and right controller
     * when set bakeCurrentTransformIntoVertices will be called on the mesh.
     * See https://doc.babylonjs.com/features/featuresDeepDive/mesh/transforms/center_origin/bakingTransforms
     */
    get gazeTrackerMesh(): Mesh;
    set gazeTrackerMesh(value: Mesh);
    /**
     * If the gaze trackers scale should be updated to be constant size when pointing at near/far meshes
     */
    updateGazeTrackerScale: boolean;
    /**
     * If the gaze trackers color should be updated when selecting meshes
     */
    updateGazeTrackerColor: boolean;
    /**
     * If the controller laser color should be updated when selecting meshes
     */
    updateControllerLaserColor: boolean;
    /**
     * The gaze tracking mesh corresponding to the left controller
     */
    get leftControllerGazeTrackerMesh(): Nullable<Mesh>;
    /**
     * The gaze tracking mesh corresponding to the right controller
     */
    get rightControllerGazeTrackerMesh(): Nullable<Mesh>;
    /**
     * If the ray of the gaze should be displayed.
     */
    get displayGaze(): boolean;
    /**
     * Sets if the ray of the gaze should be displayed.
     */
    set displayGaze(value: boolean);
    /**
     * If the ray of the LaserPointer should be displayed.
     */
    get displayLaserPointer(): boolean;
    /**
     * Sets if the ray of the LaserPointer should be displayed.
     */
    set displayLaserPointer(value: boolean);
    /**
     * The deviceOrientationCamera used as the camera when not in VR.
     */
    get deviceOrientationCamera(): Nullable<DeviceOrientationCamera>;
    /**
     * Based on the current WebVR support, returns the current VR camera used.
     */
    get currentVRCamera(): Nullable<Camera>;
    /**
     * The webVRCamera which is used when in VR.
     */
    get webVRCamera(): WebVRFreeCamera;
    /**
     * The deviceOrientationCamera that is used as a fallback when vr device is not connected.
     */
    get vrDeviceOrientationCamera(): Nullable<VRDeviceOrientationFreeCamera>;
    /**
     * The html button that is used to trigger entering into VR.
     */
    get vrButton(): Nullable<HTMLButtonElement>;
    private get _teleportationRequestInitiated();
    /**
     * Defines whether or not Pointer lock should be requested when switching to
     * full screen.
     */
    requestPointerLockOnFullScreen: boolean;
    /**
     * If asking to force XR, this will be populated with the default xr experience
     */
    xr: WebXRDefaultExperience;
    /**
     * Was the XR test done already. If this is true AND this.xr exists, xr is initialized.
     * If this is true and no this.xr, xr exists but is not supported, using WebVR.
     */
    xrTestDone: boolean;
    /**
     * Instantiates a VRExperienceHelper.
     * Helps to quickly add VR support to an existing scene.
     * @param scene The scene the VRExperienceHelper belongs to.
     * @param webVROptions Options to modify the vr experience helper's behavior.
     */
    constructor(scene: Scene, 
    /** Options to modify the vr experience helper's behavior. */
    webVROptions?: VRExperienceHelperOptions);
    private _completeVRInit;
    private _onDefaultMeshLoaded;
    private _onResize;
    private _onFullscreenChange;
    /**
     * Gets a value indicating if we are currently in VR mode.
     */
    get isInVRMode(): boolean;
    private _onVrDisplayPresentChange;
    private _onVRDisplayChanged;
    private _moveButtonToBottomRight;
    private _displayVRButton;
    private _updateButtonVisibility;
    private _cachedAngularSensibility;
    /**
     * Attempt to enter VR. If a headset is connected and ready, will request present on that.
     * Otherwise, will use the fullscreen API.
     */
    enterVR(): void;
    /**
     * Attempt to exit VR, or fullscreen.
     */
    exitVR(): void;
    /**
     * The position of the vr experience helper.
     */
    get position(): Vector3;
    /**
     * Sets the position of the vr experience helper.
     */
    set position(value: Vector3);
    /**
     * Enables controllers and user interactions such as selecting and object or clicking on an object.
     */
    enableInteractions(): void;
    private get _noControllerIsActive();
    private _beforeRender;
    private _isTeleportationFloor;
    /**
     * Adds a floor mesh to be used for teleportation.
     * @param floorMesh the mesh to be used for teleportation.
     */
    addFloorMesh(floorMesh: Mesh): void;
    /**
     * Removes a floor mesh from being used for teleportation.
     * @param floorMesh the mesh to be removed.
     */
    removeFloorMesh(floorMesh: Mesh): void;
    /**
     * Enables interactions and teleportation using the VR controllers and gaze.
     * @param vrTeleportationOptions options to modify teleportation behavior.
     */
    enableTeleportation(vrTeleportationOptions?: VRTeleportationOptions): void;
    private _onNewGamepadConnected;
    private _tryEnableInteractionOnController;
    private _onNewGamepadDisconnected;
    private _enableInteractionOnController;
    private _checkTeleportWithRay;
    private _checkRotate;
    private _checkTeleportBackwards;
    private _enableTeleportationOnController;
    private _createTeleportationCircles;
    private _displayTeleportationTarget;
    private _hideTeleportationTarget;
    private _rotateCamera;
    private _moveTeleportationSelectorTo;
    private _workingVector;
    private _workingQuaternion;
    private _workingMatrix;
    /**
     * Time Constant Teleportation Mode
     */
    static readonly TELEPORTATIONMODE_CONSTANTTIME: number;
    /**
     * Speed Constant Teleportation Mode
     */
    static readonly TELEPORTATIONMODE_CONSTANTSPEED: number;
    /**
     * Teleports the users feet to the desired location
     * @param location The location where the user's feet should be placed
     */
    teleportCamera(location: Vector3): void;
    private _convertNormalToDirectionOfRay;
    private _castRayAndSelectObject;
    private _notifySelectedMeshUnselected;
    /**
     * Permanently set new colors for the laser pointer
     * @param color the new laser color
     * @param pickedColor the new laser color when picked mesh detected
     */
    setLaserColor(color: Color3, pickedColor?: Color3): void;
    /**
     * Set lighting enabled / disabled on the laser pointer of both controllers
     * @param enabled should the lighting be enabled on the laser pointer
     */
    setLaserLightingState(enabled?: boolean): void;
    /**
     * Permanently set new colors for the gaze pointer
     * @param color the new gaze color
     * @param pickedColor the new gaze color when picked mesh detected
     */
    setGazeColor(color: Color3, pickedColor?: Color3): void;
    /**
     * Sets the color of the laser ray from the vr controllers.
     * @param color new color for the ray.
     */
    changeLaserColor(color: Color3): void;
    /**
     * Sets the color of the ray from the vr headsets gaze.
     * @param color new color for the ray.
     */
    changeGazeColor(color: Color3): void;
    /**
     * Exits VR and disposes of the vr experience helper
     */
    dispose(): void;
    /**
     * Gets the name of the VRExperienceHelper class
     * @returns "VRExperienceHelper"
     */
    getClassName(): string;
}

}
declare module "babylonjs/Cameras/VR/webVRCamera" {
import { Nullable } from "babylonjs/types";
import { Observable } from "babylonjs/Misc/observable";
import { FreeCamera } from "babylonjs/Cameras/freeCamera";
import { Scene } from "babylonjs/scene";
import { Quaternion, Matrix, Vector3 } from "babylonjs/Maths/math.vector";
import { WebVRController } from "babylonjs/Gamepads/Controllers/webVRController";
import { Node } from "babylonjs/node";
import { Ray } from "babylonjs/Culling/ray";
import "babylonjs/Engines/Extensions/engine.webVR";
/**
 * This is a copy of VRPose. See https://developer.mozilla.org/en-US/docs/Web/API/VRPose
 * IMPORTANT!! The data is right-hand data.
 * @export
 * @interface DevicePose
 */
export interface DevicePose {
    /**
     * The position of the device, values in array are [x,y,z].
     */
    readonly position: Nullable<Float32Array>;
    /**
     * The linearVelocity of the device, values in array are [x,y,z].
     */
    readonly linearVelocity: Nullable<Float32Array>;
    /**
     * The linearAcceleration of the device, values in array are [x,y,z].
     */
    readonly linearAcceleration: Nullable<Float32Array>;
    /**
     * The orientation of the device in a quaternion array, values in array are [x,y,z,w].
     */
    readonly orientation: Nullable<Float32Array>;
    /**
     * The angularVelocity of the device, values in array are [x,y,z].
     */
    readonly angularVelocity: Nullable<Float32Array>;
    /**
     * The angularAcceleration of the device, values in array are [x,y,z].
     */
    readonly angularAcceleration: Nullable<Float32Array>;
}
/**
 * Interface representing a pose controlled object in Babylon.
 * A pose controlled object has both regular pose values as well as pose values
 * from an external device such as a VR head mounted display
 */
export interface PoseControlled {
    /**
     * The position of the object in babylon space.
     */
    position: Vector3;
    /**
     * The rotation quaternion of the object in babylon space.
     */
    rotationQuaternion: Quaternion;
    /**
     * The position of the device in babylon space.
     */
    devicePosition?: Vector3;
    /**
     * The rotation quaternion of the device in babylon space.
     */
    deviceRotationQuaternion: Quaternion;
    /**
     * The raw pose coming from the device.
     */
    rawPose: Nullable<DevicePose>;
    /**
     * The scale of the device to be used when translating from device space to babylon space.
     */
    deviceScaleFactor: number;
    /**
     * Updates the poseControlled values based on the input device pose.
     * @param poseData the pose data to update the object with
     */
    updateFromDevice(poseData: DevicePose): void;
}
/**
 * Set of options to customize the webVRCamera
 */
export interface WebVROptions {
    /**
     * Sets if the webVR camera should be tracked to the vrDevice. (default: true)
     */
    trackPosition?: boolean;
    /**
     * Sets the scale of the vrDevice in babylon space. (default: 1)
     */
    positionScale?: number;
    /**
     * If there are more than one VRDisplays, this will choose the display matching this name. (default: pick first vrDisplay)
     */
    displayName?: string;
    /**
     * Should the native controller meshes be initialized. (default: true)
     */
    controllerMeshes?: boolean;
    /**
     * Creating a default HemiLight only on controllers. (default: true)
     */
    defaultLightingOnControllers?: boolean;
    /**
     * If you don't want to use the default VR button of the helper. (default: false)
     */
    useCustomVRButton?: boolean;
    /**
     * If you'd like to provide your own button to the VRHelper. (default: standard babylon vr button)
     */
    customVRButton?: HTMLButtonElement;
    /**
     * To change the length of the ray for gaze/controllers. Will be scaled by positionScale. (default: 100)
     */
    rayLength?: number;
    /**
     * To change the default offset from the ground to account for user's height in meters. Will be scaled by positionScale. (default: 1.7)
     */
    defaultHeight?: number;
    /**
     * If multiview should be used if available (default: false)
     */
    useMultiview?: boolean;
}
/**
 * This represents a WebVR camera.
 * The WebVR camera is Babylon's simple interface to interaction with Windows Mixed Reality, HTC Vive and Oculus Rift.
 * @deprecated Use WebXR instead - https://doc.babylonjs.com/features/featuresDeepDive/webXR
 * @example https://doc.babylonjs.com/features/featuresDeepDive/cameras/webVRCamera
 */
export class WebVRFreeCamera extends FreeCamera implements PoseControlled {
    private _webVROptions;
    /**
     * @internal
     * The vrDisplay tied to the camera. See https://developer.mozilla.org/en-US/docs/Web/API/VRDisplay
     */
    _vrDevice: any;
    /**
     * The rawPose of the vrDevice.
     */
    rawPose: Nullable<DevicePose>;
    private _onVREnabled;
    private _specsVersion;
    private _attached;
    private _frameData;
    protected _descendants: Array<Node>;
    private _deviceRoomPosition;
    /** @internal */
    _deviceRoomRotationQuaternion: Quaternion;
    private _standingMatrix;
    /**
     * Represents device position in babylon space.
     */
    devicePosition: Vector3;
    /**
     * Represents device rotation in babylon space.
     */
    deviceRotationQuaternion: Quaternion;
    /**
     * The scale of the device to be used when translating from device space to babylon space.
     */
    deviceScaleFactor: number;
    private _deviceToWorld;
    private _worldToDevice;
    /**
     * References to the webVR controllers for the vrDevice.
     */
    controllers: Array<WebVRController>;
    /**
     * Emits an event when a controller is attached.
     */
    onControllersAttachedObservable: Observable<WebVRController[]>;
    /**
     * Emits an event when a controller's mesh has been loaded;
     */
    onControllerMeshLoadedObservable: Observable<WebVRController>;
    /**
     * Emits an event when the HMD's pose has been updated.
     */
    onPoseUpdatedFromDeviceObservable: Observable<any>;
    private _poseSet;
    /**
     * If the rig cameras be used as parent instead of this camera.
     */
    rigParenting: boolean;
    private _lightOnControllers;
    private _defaultHeight?;
    /**
     * Instantiates a WebVRFreeCamera.
     * @param name The name of the WebVRFreeCamera
     * @param position The starting anchor position for the camera
     * @param scene The scene the camera belongs to
     * @param _webVROptions a set of customizable options for the webVRCamera
     */
    constructor(name: string, position: Vector3, scene?: Scene, _webVROptions?: WebVROptions);
    protected _setRigMode: any;
    /**
     * Gets the device distance from the ground in meters.
     * @returns the distance in meters from the vrDevice to ground in device space. If standing matrix is not supported for the vrDevice 0 is returned.
     */
    deviceDistanceToRoomGround(): number;
    /**
     * Enables the standing matrix when supported. This can be used to position the user's view the correct height from the ground.
     * @param callback will be called when the standing matrix is set. Callback parameter is if the standing matrix is supported.
     */
    useStandingMatrix(callback?: (bool: boolean) => void): void;
    /**
     * Enables the standing matrix when supported. This can be used to position the user's view the correct height from the ground.
     * @returns A promise with a boolean set to if the standing matrix is supported.
     */
    useStandingMatrixAsync(): Promise<boolean>;
    /**
     * Disposes the camera
     */
    dispose(): void;
    /**
     * Gets a vrController by name.
     * @param name The name of the controller to retrieve
     * @returns the controller matching the name specified or null if not found
     */
    getControllerByName(name: string): Nullable<WebVRController>;
    private _leftController;
    /**
     * The controller corresponding to the users left hand.
     */
    get leftController(): Nullable<WebVRController>;
    private _rightController;
    /**
     * The controller corresponding to the users right hand.
     */
    get rightController(): Nullable<WebVRController>;
    /**
     * Casts a ray forward from the vrCamera's gaze.
     * @param length Length of the ray (default: 100)
     * @returns the ray corresponding to the gaze
     */
    getForwardRay(length?: number): Ray;
    /**
     * @internal
     * Updates the camera based on device's frame data
     */
    _checkInputs(): void;
    /**
     * Updates the poseControlled values based on the input device pose.
     * @param poseData Pose coming from the device
     */
    updateFromDevice(poseData: DevicePose): void;
    private _detachIfAttached;
    /**
     * WebVR's attach control will start broadcasting frames to the device.
     * Note that in certain browsers (chrome for example) this function must be called
     * within a user-interaction callback. Example:
     * <pre> scene.onPointerDown = function() { camera.attachControl(canvas); }</pre>
     *
     * @param noPreventDefault prevent the default html element operation when attaching the vrDevice
     */
    attachControl(noPreventDefault?: boolean): void;
    /**
     * Detach the current controls from the specified dom element.
     */
    detachControl(): void;
    /**
     * @returns the name of this class
     */
    getClassName(): string;
    /**
     * Calls resetPose on the vrDisplay
     * See: https://developer.mozilla.org/en-US/docs/Web/API/VRDisplay/resetPose
     */
    resetToCurrentRotation(): void;
    /**
     * @internal
     * Updates the rig cameras (left and right eye)
     */
    _updateRigCameras(): void;
    private _workingVector;
    private _oneVector;
    private _workingMatrix;
    private _updateCacheCalled;
    private _correctPositionIfNotTrackPosition;
    /**
     * @internal
     * Updates the cached values of the camera
     * @param ignoreParentClass ignores updating the parent class's cache (default: false)
     */
    _updateCache(ignoreParentClass?: boolean): void;
    /**
     * @internal
     * Get current device position in babylon world
     */
    _computeDevicePosition(): void;
    /**
     * Updates the current device position and rotation in the babylon world
     */
    update(): void;
    /**
     * @internal
     * Gets the view matrix of this camera (Always set to identity as left and right eye cameras contain the actual view matrix)
     * @returns an identity matrix
     */
    _getViewMatrix(): Matrix;
    private _tmpMatrix;
    /**
     * This function is called by the two RIG cameras.
     * 'this' is the left or right camera (and NOT (!!!) the WebVRFreeCamera instance)
     * @internal
     */
    _getWebVRViewMatrix(): Matrix;
    /** @internal */
    _getWebVRProjectionMatrix(): Matrix;
    private _onGamepadConnectedObserver;
    private _onGamepadDisconnectedObserver;
    private _updateCacheWhenTrackingDisabledObserver;
    /**
     * Initializes the controllers and their meshes
     */
    initControllers(): void;
}

}
declare module "babylonjs/Collisions/collider" {
import { Nullable, IndicesArray } from "babylonjs/types";
import { Vector3 } from "babylonjs/Maths/math.vector";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { Plane } from "babylonjs/Maths/math.plane";
/** @internal */
export class Collider {
    /** Define if a collision was found */
    collisionFound: boolean;
    /**
     * Define last intersection point in local space
     */
    intersectionPoint: Vector3;
    /**
     * Define last collided mesh
     */
    collidedMesh: Nullable<AbstractMesh>;
    /**
     * If true, it check for double sided faces and only returns 1 collision instead of 2
     */
    static DoubleSidedCheck: boolean;
    private _collisionPoint;
    private _planeIntersectionPoint;
    private _tempVector;
    private _tempVector2;
    private _tempVector3;
    private _tempVector4;
    private _edge;
    private _baseToVertex;
    private _destinationPoint;
    private _slidePlaneNormal;
    private _displacementVector;
    /** @internal */
    _radius: Vector3;
    /** @internal */
    _retry: number;
    private _velocity;
    private _basePoint;
    private _epsilon;
    /** @internal */
    _velocityWorldLength: number;
    /** @internal */
    _basePointWorld: Vector3;
    private _velocityWorld;
    private _normalizedVelocity;
    /** @internal */
    _initialVelocity: Vector3;
    /** @internal */
    _initialPosition: Vector3;
    private _nearestDistance;
    private _collisionMask;
    private _velocitySquaredLength;
    private _nearestDistanceSquared;
    get collisionMask(): number;
    set collisionMask(mask: number);
    /**
     * Gets the plane normal used to compute the sliding response (in local space)
     */
    get slidePlaneNormal(): Vector3;
    /**
     * @internal
     */
    _initialize(source: Vector3, dir: Vector3, e: number): void;
    /**
     * @internal
     */
    _checkPointInTriangle(point: Vector3, pa: Vector3, pb: Vector3, pc: Vector3, n: Vector3): boolean;
    /**
     * @internal
     */
    _canDoCollision(sphereCenter: Vector3, sphereRadius: number, vecMin: Vector3, vecMax: Vector3): boolean;
    /**
     * @internal
     */
    _testTriangle(faceIndex: number, trianglePlaneArray: Array<Plane>, p1: Vector3, p2: Vector3, p3: Vector3, hasMaterial: boolean, hostMesh: AbstractMesh): void;
    /**
     * @internal
     */
    _collide(trianglePlaneArray: Array<Plane>, pts: Vector3[], indices: IndicesArray, indexStart: number, indexEnd: number, decal: number, hasMaterial: boolean, hostMesh: AbstractMesh, invertTriangles?: boolean, triangleStrip?: boolean): void;
    /**
     * @internal
     */
    _getResponse(pos: Vector3, vel: Vector3): void;
}

}
declare module "babylonjs/Collisions/collisionCoordinator" {
import { Nullable } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
import { Vector3 } from "babylonjs/Maths/math.vector";
import { Collider } from "babylonjs/Collisions/collider";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
/** @internal */
export interface ICollisionCoordinator {
    createCollider(): Collider;
    getNewPosition(position: Vector3, displacement: Vector3, collider: Collider, maximumRetry: number, excludedMesh: Nullable<AbstractMesh>, onNewPosition: (collisionIndex: number, newPosition: Vector3, collidedMesh: Nullable<AbstractMesh>) => void, collisionIndex: number): void;
    init(scene: Scene): void;
}
/** @internal */
export class DefaultCollisionCoordinator implements ICollisionCoordinator {
    private _scene;
    private _scaledPosition;
    private _scaledVelocity;
    private _finalPosition;
    getNewPosition(position: Vector3, displacement: Vector3, collider: Collider, maximumRetry: number, excludedMesh: AbstractMesh, onNewPosition: (collisionIndex: number, newPosition: Vector3, collidedMesh: Nullable<AbstractMesh>) => void, collisionIndex: number): void;
    createCollider(): Collider;
    init(scene: Scene): void;
    private _collideWithWorld;
}

}
declare module "babylonjs/Collisions/index" {
export * from "babylonjs/Collisions/collider";
export * from "babylonjs/Collisions/collisionCoordinator";
export * from "babylonjs/Collisions/pickingInfo";
export * from "babylonjs/Collisions/intersectionInfo";
export * from "babylonjs/Collisions/meshCollisionData";

}
declare module "babylonjs/Collisions/intersectionInfo" {
import { Nullable } from "babylonjs/types";
/**
 * @internal
 */
export class IntersectionInfo {
    bu: Nullable<number>;
    bv: Nullable<number>;
    distance: number;
    faceId: number;
    subMeshId: number;
    constructor(bu: Nullable<number>, bv: Nullable<number>, distance: number);
}

}
declare module "babylonjs/Collisions/meshCollisionData" {
import { Collider } from "babylonjs/Collisions/collider";
import { Vector3 } from "babylonjs/Maths/math.vector";
import { Nullable } from "babylonjs/types";
import { Observer } from "babylonjs/Misc/observable";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
/**
 * @internal
 */
export class _MeshCollisionData {
    _checkCollisions: boolean;
    _collisionMask: number;
    _collisionGroup: number;
    _surroundingMeshes: Nullable<AbstractMesh[]>;
    _collider: Nullable<Collider>;
    _oldPositionForCollisions: Vector3;
    _diffPositionForCollisions: Vector3;
    _onCollideObserver: Nullable<Observer<AbstractMesh>>;
    _onCollisionPositionChangeObserver: Nullable<Observer<Vector3>>;
    _collisionResponse: boolean;
}
export {};

}
declare module "babylonjs/Collisions/pickingInfo" {
import { Nullable } from "babylonjs/types";
import { Vector3, Vector2 } from "babylonjs/Maths/math.vector";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { TransformNode } from "babylonjs/Meshes/transformNode";
import { Sprite } from "babylonjs/Sprites/sprite";
import { Ray } from "babylonjs/Culling/ray";
/**
 * Information about the result of picking within a scene
 * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/interactions/picking_collisions
 */
export class PickingInfo {
    /**
     * If the pick collided with an object
     */
    hit: boolean;
    /**
     * Distance away where the pick collided
     */
    distance: number;
    /**
     * The location of pick collision
     */
    pickedPoint: Nullable<Vector3>;
    /**
     * The mesh corresponding the pick collision
     */
    pickedMesh: Nullable<AbstractMesh>;
    /** (See getTextureCoordinates) The barycentric U coordinate that is used when calculating the texture coordinates of the collision.*/
    bu: number;
    /** (See getTextureCoordinates) The barycentric V coordinate that is used when calculating the texture coordinates of the collision.*/
    bv: number;
    /** The index of the face on the mesh that was picked, or the index of the Line if the picked Mesh is a LinesMesh */
    faceId: number;
    /** The index of the face on the subMesh that was picked, or the index of the Line if the picked Mesh is a LinesMesh */
    subMeshFaceId: number;
    /** Id of the submesh that was picked */
    subMeshId: number;
    /** If a sprite was picked, this will be the sprite the pick collided with */
    pickedSprite: Nullable<Sprite>;
    /** If we are picking a mesh with thin instance, this will give you the picked thin instance */
    thinInstanceIndex: number;
    /**
     * The ray that was used to perform the picking.
     */
    ray: Nullable<Ray>;
    /**
     * If a mesh was used to do the picking (eg. 6dof controller) as a "near interaction", this will be populated.
     */
    originMesh: Nullable<AbstractMesh>;
    /**
     * The aim-space transform of the input used for picking, if it is an XR input source.
     */
    aimTransform: Nullable<TransformNode>;
    /**
     * The grip-space transform of the input used for picking, if it is an XR input source.
     * Some XR sources, such as input coming from head mounted displays, do not have this.
     */
    gripTransform: Nullable<TransformNode>;
    /**
     * Gets the normal corresponding to the face the pick collided with
     * @param useWorldCoordinates If the resulting normal should be relative to the world (default: false)
     * @param useVerticesNormals If the vertices normals should be used to calculate the normal instead of the normal map (default: true)
     * @returns The normal corresponding to the face the pick collided with
     * @remarks Note that the returned normal will always point towards the picking ray.
     */
    getNormal(useWorldCoordinates?: boolean, useVerticesNormals?: boolean): Nullable<Vector3>;
    /**
     * Gets the texture coordinates of where the pick occurred
     * @returns The vector containing the coordinates of the texture
     */
    getTextureCoordinates(): Nullable<Vector2>;
}
export {};

}
declare module "babylonjs/Compat/compatibilityOptions" {
/**
 * Options used to control default behaviors regarding compatibility support
 */
export class CompatibilityOptions {
    /**
     * Defines if the system should use OpenGL convention for UVs when creating geometry or loading .babylon files (false by default)
     */
    static UseOpenGLOrientationForUV: boolean;
}

}
declare module "babylonjs/Compat/index" {
export * from "babylonjs/Compat/compatibilityOptions";

}
declare module "babylonjs/Compute/computeEffect" {
import { Nullable } from "babylonjs/types";
import { Observable } from "babylonjs/Misc/observable";
import { IComputePipelineContext } from "babylonjs/Compute/IComputePipelineContext";
import { Engine } from "babylonjs/Engines/engine";
/**
 * Options to be used when creating a compute effect.
 */
export interface IComputeEffectCreationOptions {
    /**
     * Define statements that will be set in the shader.
     */
    defines: any;
    /**
     * The name of the entry point in the shader source (default: "main")
     */
    entryPoint?: string;
    /**
     * Callback that will be called when the shader is compiled.
     */
    onCompiled: Nullable<(effect: ComputeEffect) => void>;
    /**
     * Callback that will be called if an error occurs during shader compilation.
     */
    onError: Nullable<(effect: ComputeEffect, errors: string) => void>;
    /**
     * If provided, will be called with the shader code so that this code can be updated before it is compiled by the GPU
     */
    processFinalCode?: Nullable<(code: string) => string>;
}
/**
 * Effect wrapping a compute shader and let execute (dispatch) the shader
 */
export class ComputeEffect {
    private static _UniqueIdSeed;
    /**
     * Enable logging of the shader code when a compilation error occurs
     */
    static LogShaderCodeOnCompilationError: boolean;
    /**
     * Name of the effect.
     */
    name: any;
    /**
     * String container all the define statements that should be set on the shader.
     */
    defines: string;
    /**
     * Callback that will be called when the shader is compiled.
     */
    onCompiled: Nullable<(effect: ComputeEffect) => void>;
    /**
     * Callback that will be called if an error occurs during shader compilation.
     */
    onError: Nullable<(effect: ComputeEffect, errors: string) => void>;
    /**
     * Unique ID of the effect.
     */
    uniqueId: number;
    /**
     * Observable that will be called when the shader is compiled.
     * It is recommended to use executeWhenCompile() or to make sure that scene.isReady() is called to get this observable raised.
     */
    onCompileObservable: Observable<ComputeEffect>;
    /**
     * Observable that will be called if an error occurs during shader compilation.
     */
    onErrorObservable: Observable<ComputeEffect>;
    /**
     * Observable that will be called when effect is bound.
     */
    onBindObservable: Observable<ComputeEffect>;
    /**
     * @internal
     * Specifies if the effect was previously ready
     */
    _wasPreviouslyReady: boolean;
    private _engine;
    private _isReady;
    private _compilationError;
    /** @internal */
    _key: string;
    private _computeSourceCodeOverride;
    /** @internal */
    _pipelineContext: Nullable<IComputePipelineContext>;
    /** @internal */
    _computeSourceCode: string;
    private _rawComputeSourceCode;
    private _entryPoint;
    private _shaderLanguage;
    private _shaderStore;
    private _shaderRepository;
    private _includeShaderStore;
    /**
     * Creates a compute effect that can be used to execute a compute shader
     * @param baseName Name of the effect
     * @param options Set of all options to create the effect
     * @param engine The engine the effect is created for
     * @param key Effect Key identifying uniquely compiled shader variants
     */
    constructor(baseName: any, options: IComputeEffectCreationOptions, engine: Engine, key?: string);
    private _useFinalCode;
    /**
     * Unique key for this effect
     */
    get key(): string;
    /**
     * If the effect has been compiled and prepared.
     * @returns if the effect is compiled and prepared.
     */
    isReady(): boolean;
    private _isReadyInternal;
    /**
     * The engine the effect was initialized with.
     * @returns the engine.
     */
    getEngine(): Engine;
    /**
     * The pipeline context for this effect
     * @returns the associated pipeline context
     */
    getPipelineContext(): Nullable<IComputePipelineContext>;
    /**
     * The error from the last compilation.
     * @returns the error string.
     */
    getCompilationError(): string;
    /**
     * Adds a callback to the onCompiled observable and call the callback immediately if already ready.
     * @param func The callback to be used.
     */
    executeWhenCompiled(func: (effect: ComputeEffect) => void): void;
    private _checkIsReady;
    private _loadShader;
    /**
     * Gets the compute shader source code of this effect
     */
    get computeSourceCode(): string;
    /**
     * Gets the compute shader source code before it has been processed by the preprocessor
     */
    get rawComputeSourceCode(): string;
    /**
     * Prepares the effect
     * @internal
     */
    _prepareEffect(): void;
    private _getShaderCodeAndErrorLine;
    private _processCompilationErrors;
    /**
     * Release all associated resources.
     **/
    dispose(): void;
    /**
     * This function will add a new compute shader to the shader store
     * @param name the name of the shader
     * @param computeShader compute shader content
     */
    static RegisterShader(name: string, computeShader: string): void;
}
export {};

}
declare module "babylonjs/Compute/computeShader" {
import { UniformBuffer } from "babylonjs/Materials/uniformBuffer";
import { ThinEngine } from "babylonjs/Engines/thinEngine";
import { Scene } from "babylonjs/scene";
import { Nullable } from "babylonjs/types";
import { ComputeEffect } from "babylonjs/Compute/computeEffect";
import { ComputeBindingMapping } from "babylonjs/Engines/Extensions/engine.computeShader";
import { BaseTexture } from "babylonjs/Materials/Textures/baseTexture";
import { StorageBuffer } from "babylonjs/Buffers/storageBuffer";
import { TextureSampler } from "babylonjs/Materials/Textures/textureSampler";
/**
 * Defines the options associated with the creation of a compute shader.
 */
export interface IComputeShaderOptions {
    /**
     * list of bindings mapping (key is property name, value is binding location)
     * Must be provided because browsers don't support reflection for wgsl shaders yet (so there's no way to query the binding/group from a variable name)
     * TODO: remove this when browsers support reflection for wgsl shaders
     */
    bindingsMapping: ComputeBindingMapping;
    /**
     * The list of defines used in the shader
     */
    defines?: string[];
    /**
     * The name of the entry point in the shader source (default: "main")
     */
    entryPoint?: string;
    /**
     * If provided, will be called with the shader code so that this code can be updated before it is compiled by the GPU
     */
    processFinalCode?: Nullable<(code: string) => string>;
}
/**
 * The ComputeShader object lets you execute a compute shader on your GPU (if supported by the engine)
 */
export class ComputeShader {
    private _engine;
    private _shaderPath;
    private _options;
    private _effect;
    private _cachedDefines;
    private _bindings;
    private _samplers;
    private _context;
    private _contextIsDirty;
    /**
     * Gets the unique id of the compute shader
     */
    readonly uniqueId: number;
    /**
     * The name of the shader
     */
    name: string;
    /**
     * The options used to create the shader
     */
    get options(): IComputeShaderOptions;
    /**
     * The shaderPath used to create the shader
     */
    get shaderPath(): any;
    /**
     * Callback triggered when the shader is compiled
     */
    onCompiled: Nullable<(effect: ComputeEffect) => void>;
    /**
     * Callback triggered when an error occurs
     */
    onError: Nullable<(effect: ComputeEffect, errors: string) => void>;
    /**
     * Instantiates a new compute shader.
     * @param name Defines the name of the compute shader in the scene
     * @param engine Defines the engine the compute shader belongs to
     * @param shaderPath Defines  the route to the shader code in one of three ways:
     *  * object: { compute: "custom" }, used with ShaderStore.ShadersStoreWGSL["customComputeShader"]
     *  * object: { computeElement: "HTMLElementId" }, used with shader code in script tags
     *  * object: { computeSource: "compute shader code string" using with string containing the shader code
     *  * string: try first to find the code in ShaderStore.ShadersStoreWGSL[shaderPath + "ComputeShader"]. If not, assumes it is a file with name shaderPath.compute.fx in index.html folder.
     * @param options Define the options used to create the shader
     */
    constructor(name: string, engine: ThinEngine, shaderPath: any, options?: Partial<IComputeShaderOptions>);
    /**
     * Gets the current class name of the material e.g. "ComputeShader"
     * Mainly use in serialization.
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Binds a texture to the shader
     * @param name Binding name of the texture
     * @param texture Texture to bind
     * @param bindSampler Bind the sampler corresponding to the texture (default: true). The sampler will be bound just before the binding index of the texture
     */
    setTexture(name: string, texture: BaseTexture, bindSampler?: boolean): void;
    /**
     * Binds a storage texture to the shader
     * @param name Binding name of the texture
     * @param texture Texture to bind
     */
    setStorageTexture(name: string, texture: BaseTexture): void;
    /**
     * Binds a uniform buffer to the shader
     * @param name Binding name of the buffer
     * @param buffer Buffer to bind
     */
    setUniformBuffer(name: string, buffer: UniformBuffer): void;
    /**
     * Binds a storage buffer to the shader
     * @param name Binding name of the buffer
     * @param buffer Buffer to bind
     */
    setStorageBuffer(name: string, buffer: StorageBuffer): void;
    /**
     * Binds a texture sampler to the shader
     * @param name Binding name of the sampler
     * @param sampler Sampler to bind
     */
    setTextureSampler(name: string, sampler: TextureSampler): void;
    /**
     * Specifies that the compute shader is ready to be executed (the compute effect and all the resources are ready)
     * @returns true if the compute shader is ready to be executed
     */
    isReady(): boolean;
    /**
     * Dispatches (executes) the compute shader
     * @param x Number of workgroups to execute on the X dimension
     * @param y Number of workgroups to execute on the Y dimension (default: 1)
     * @param z Number of workgroups to execute on the Z dimension (default: 1)
     * @returns True if the dispatch could be done, else false (meaning either the compute effect or at least one of the bound resources was not ready)
     */
    dispatch(x: number, y?: number, z?: number): boolean;
    /**
     * Waits for the compute shader to be ready and executes it
     * @param x Number of workgroups to execute on the X dimension
     * @param y Number of workgroups to execute on the Y dimension (default: 1)
     * @param z Number of workgroups to execute on the Z dimension (default: 1)
     * @param delay Delay between the retries while the shader is not ready (in milliseconds - 10 by default)
     * @returns A promise that is resolved once the shader has been sent to the GPU. Note that it does not mean that the shader execution itself is finished!
     */
    dispatchWhenReady(x: number, y?: number, z?: number, delay?: number): Promise<void>;
    /**
     * Serializes this compute shader in a JSON representation
     * @returns the serialized compute shader object
     */
    serialize(): any;
    /**
     * Creates a compute shader from parsed compute shader data
     * @param source defines the JSON representation of the compute shader
     * @param scene defines the hosting scene
     * @param rootUrl defines the root URL to use to load textures and relative dependencies
     * @returns a new compute shader
     */
    static Parse(source: any, scene: Scene, rootUrl: string): ComputeShader;
}

}
declare module "babylonjs/Compute/IComputeContext" {
/** @internal */
export interface IComputeContext {
    clear(): void;
}

}
declare module "babylonjs/Compute/IComputePipelineContext" {
/**
 * Class used to store and describe the pipeline context associated with a compute effect
 */
export interface IComputePipelineContext {
    /**
     * Gets a boolean indicating that this pipeline context is supporting asynchronous creating
     */
    isAsync: boolean;
    /**
     * Gets a boolean indicating that the context is ready to be used (like shader / pipeline are compiled and ready for instance)
     */
    isReady: boolean;
    /** @internal */
    _name?: string;
    /** @internal */
    _getComputeShaderCode(): string | null;
    /** Releases the resources associated with the pipeline. */
    dispose(): void;
}

}
declare module "babylonjs/Compute/index" {
export * from "babylonjs/Compute/computeEffect";
export * from "babylonjs/Compute/computeShader";

}
declare module "babylonjs/Culling/boundingBox" {
import { DeepImmutable, Nullable } from "babylonjs/types";
import { Matrix, Vector3 } from "babylonjs/Maths/math.vector";
import { BoundingSphere } from "babylonjs/Culling/boundingSphere";
import { ICullable } from "babylonjs/Culling/boundingInfo";
import { Plane } from "babylonjs/Maths/math.plane";
import { DrawWrapper } from "babylonjs/Materials/drawWrapper";
/**
 * Class used to store bounding box information
 */
export class BoundingBox implements ICullable {
    /**
     * Gets the 8 vectors representing the bounding box in local space
     */
    readonly vectors: Vector3[];
    /**
     * Gets the center of the bounding box in local space
     */
    readonly center: Vector3;
    /**
     * Gets the center of the bounding box in world space
     */
    readonly centerWorld: Vector3;
    /**
     * Gets the extend size in local space
     */
    readonly extendSize: Vector3;
    /**
     * Gets the extend size in world space
     */
    readonly extendSizeWorld: Vector3;
    /**
     * Gets the OBB (object bounding box) directions
     */
    readonly directions: Vector3[];
    /**
     * Gets the 8 vectors representing the bounding box in world space
     */
    readonly vectorsWorld: Vector3[];
    /**
     * Gets the minimum vector in world space
     */
    readonly minimumWorld: Vector3;
    /**
     * Gets the maximum vector in world space
     */
    readonly maximumWorld: Vector3;
    /**
     * Gets the minimum vector in local space
     */
    readonly minimum: Vector3;
    /**
     * Gets the maximum vector in local space
     */
    readonly maximum: Vector3;
    private _worldMatrix;
    private static readonly _TmpVector3;
    /**
     * @internal
     */
    _tag: number;
    /** @internal */
    _drawWrapperFront: Nullable<DrawWrapper>;
    /** @internal */
    _drawWrapperBack: Nullable<DrawWrapper>;
    /**
     * Creates a new bounding box
     * @param min defines the minimum vector (in local space)
     * @param max defines the maximum vector (in local space)
     * @param worldMatrix defines the new world matrix
     */
    constructor(min: DeepImmutable<Vector3>, max: DeepImmutable<Vector3>, worldMatrix?: DeepImmutable<Matrix>);
    /**
     * Recreates the entire bounding box from scratch as if we call the constructor in place
     * @param min defines the new minimum vector (in local space)
     * @param max defines the new maximum vector (in local space)
     * @param worldMatrix defines the new world matrix
     */
    reConstruct(min: DeepImmutable<Vector3>, max: DeepImmutable<Vector3>, worldMatrix?: DeepImmutable<Matrix>): void;
    /**
     * Scale the current bounding box by applying a scale factor
     * @param factor defines the scale factor to apply
     * @returns the current bounding box
     */
    scale(factor: number): BoundingBox;
    /**
     * Gets the world matrix of the bounding box
     * @returns a matrix
     */
    getWorldMatrix(): DeepImmutable<Matrix>;
    /**
     * @internal
     */
    _update(world: DeepImmutable<Matrix>): void;
    /**
     * Tests if the bounding box is intersecting the frustum planes
     * @param frustumPlanes defines the frustum planes to test
     * @returns true if there is an intersection
     */
    isInFrustum(frustumPlanes: Array<DeepImmutable<Plane>>): boolean;
    /**
     * Tests if the bounding box is entirely inside the frustum planes
     * @param frustumPlanes defines the frustum planes to test
     * @returns true if there is an inclusion
     */
    isCompletelyInFrustum(frustumPlanes: Array<DeepImmutable<Plane>>): boolean;
    /**
     * Tests if a point is inside the bounding box
     * @param point defines the point to test
     * @returns true if the point is inside the bounding box
     */
    intersectsPoint(point: DeepImmutable<Vector3>): boolean;
    /**
     * Tests if the bounding box intersects with a bounding sphere
     * @param sphere defines the sphere to test
     * @returns true if there is an intersection
     */
    intersectsSphere(sphere: DeepImmutable<BoundingSphere>): boolean;
    /**
     * Tests if the bounding box intersects with a box defined by a min and max vectors
     * @param min defines the min vector to use
     * @param max defines the max vector to use
     * @returns true if there is an intersection
     */
    intersectsMinMax(min: DeepImmutable<Vector3>, max: DeepImmutable<Vector3>): boolean;
    /**
     * Disposes the resources of the class
     */
    dispose(): void;
    /**
     * Tests if two bounding boxes are intersections
     * @param box0 defines the first box to test
     * @param box1 defines the second box to test
     * @returns true if there is an intersection
     */
    static Intersects(box0: DeepImmutable<BoundingBox>, box1: DeepImmutable<BoundingBox>): boolean;
    /**
     * Tests if a bounding box defines by a min/max vectors intersects a sphere
     * @param minPoint defines the minimum vector of the bounding box
     * @param maxPoint defines the maximum vector of the bounding box
     * @param sphereCenter defines the sphere center
     * @param sphereRadius defines the sphere radius
     * @returns true if there is an intersection
     */
    static IntersectsSphere(minPoint: DeepImmutable<Vector3>, maxPoint: DeepImmutable<Vector3>, sphereCenter: DeepImmutable<Vector3>, sphereRadius: number): boolean;
    /**
     * Tests if a bounding box defined with 8 vectors is entirely inside frustum planes
     * @param boundingVectors defines an array of 8 vectors representing a bounding box
     * @param frustumPlanes defines the frustum planes to test
     * @returns true if there is an inclusion
     */
    static IsCompletelyInFrustum(boundingVectors: Array<DeepImmutable<Vector3>>, frustumPlanes: Array<DeepImmutable<Plane>>): boolean;
    /**
     * Tests if a bounding box defined with 8 vectors intersects frustum planes
     * @param boundingVectors defines an array of 8 vectors representing a bounding box
     * @param frustumPlanes defines the frustum planes to test
     * @returns true if there is an intersection
     */
    static IsInFrustum(boundingVectors: Array<DeepImmutable<Vector3>>, frustumPlanes: Array<DeepImmutable<Plane>>): boolean;
}
export {};

}
declare module "babylonjs/Culling/boundingInfo" {
import { DeepImmutable } from "babylonjs/types";
import { Matrix } from "babylonjs/Maths/math.vector";
import { Vector3 } from "babylonjs/Maths/math.vector";
import { BoundingBox } from "babylonjs/Culling/boundingBox";
import { BoundingSphere } from "babylonjs/Culling/boundingSphere";
import { Plane } from "babylonjs/Maths/math.plane";
import { Collider } from "babylonjs/Collisions/collider";
/**
 * Interface for cullable objects
 * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/materials_introduction#back-face-culling
 */
export interface ICullable {
    /**
     * Checks if the object or part of the object is in the frustum
     * @param frustumPlanes Camera near/planes
     * @returns true if the object is in frustum otherwise false
     */
    isInFrustum(frustumPlanes: Plane[]): boolean;
    /**
     * Checks if a cullable object (mesh...) is in the camera frustum
     * Unlike isInFrustum this checks the full bounding box
     * @param frustumPlanes Camera near/planes
     * @returns true if the object is in frustum otherwise false
     */
    isCompletelyInFrustum(frustumPlanes: Plane[]): boolean;
}
/**
 * Info for a bounding data of a mesh
 */
export class BoundingInfo implements ICullable {
    /**
     * Bounding box for the mesh
     */
    readonly boundingBox: BoundingBox;
    /**
     * Bounding sphere for the mesh
     */
    readonly boundingSphere: BoundingSphere;
    private _isLocked;
    private static readonly _TmpVector3;
    /**
     * Constructs bounding info
     * @param minimum min vector of the bounding box/sphere
     * @param maximum max vector of the bounding box/sphere
     * @param worldMatrix defines the new world matrix
     */
    constructor(minimum: DeepImmutable<Vector3>, maximum: DeepImmutable<Vector3>, worldMatrix?: DeepImmutable<Matrix>);
    /**
     * Recreates the entire bounding info from scratch as if we call the constructor in place
     * @param min defines the new minimum vector (in local space)
     * @param max defines the new maximum vector (in local space)
     * @param worldMatrix defines the new world matrix
     */
    reConstruct(min: DeepImmutable<Vector3>, max: DeepImmutable<Vector3>, worldMatrix?: DeepImmutable<Matrix>): void;
    /**
     * min vector of the bounding box/sphere
     */
    get minimum(): Vector3;
    /**
     * max vector of the bounding box/sphere
     */
    get maximum(): Vector3;
    /**
     * If the info is locked and won't be updated to avoid perf overhead
     */
    get isLocked(): boolean;
    set isLocked(value: boolean);
    /**
     * Updates the bounding sphere and box
     * @param world world matrix to be used to update
     */
    update(world: DeepImmutable<Matrix>): void;
    /**
     * Recreate the bounding info to be centered around a specific point given a specific extend.
     * @param center New center of the bounding info
     * @param extend New extend of the bounding info
     * @returns the current bounding info
     */
    centerOn(center: DeepImmutable<Vector3>, extend: DeepImmutable<Vector3>): BoundingInfo;
    /**
     * Grows the bounding info to include the given point.
     * @param point The point that will be included in the current bounding info
     * @returns the current bounding info
     */
    encapsulate(point: Vector3): BoundingInfo;
    /**
     * Grows the bounding info to encapsulate the given bounding info.
     * @param toEncapsulate The bounding info that will be encapsulated in the current bounding info
     * @returns the current bounding info
     */
    encapsulateBoundingInfo(toEncapsulate: BoundingInfo): BoundingInfo;
    /**
     * Scale the current bounding info by applying a scale factor
     * @param factor defines the scale factor to apply
     * @returns the current bounding info
     */
    scale(factor: number): BoundingInfo;
    /**
     * Returns `true` if the bounding info is within the frustum defined by the passed array of planes.
     * @param frustumPlanes defines the frustum to test
     * @param strategy defines the strategy to use for the culling (default is BABYLON.AbstractMesh.CULLINGSTRATEGY_STANDARD)
     * The different strategies available are:
     * * BABYLON.AbstractMesh.CULLINGSTRATEGY_STANDARD most accurate but slower @see https://doc.babylonjs.com/typedoc/classes/BABYLON.AbstractMesh#CULLINGSTRATEGY_STANDARD
     * * BABYLON.AbstractMesh.CULLINGSTRATEGY_BOUNDINGSPHERE_ONLY faster but less accurate @see https://doc.babylonjs.com/typedoc/classes/BABYLON.AbstractMesh#CULLINGSTRATEGY_BOUNDINGSPHERE_ONLY
     * * BABYLON.AbstractMesh.CULLINGSTRATEGY_OPTIMISTIC_INCLUSION can be faster if always visible @see https://doc.babylonjs.com/typedoc/classes/BABYLON.AbstractMesh#CULLINGSTRATEGY_OPTIMISTIC_INCLUSION
     * * BABYLON.AbstractMesh.CULLINGSTRATEGY_OPTIMISTIC_INCLUSION_THEN_BSPHERE_ONLY can be faster if always visible @see https://doc.babylonjs.com/typedoc/classes/BABYLON.AbstractMesh#CULLINGSTRATEGY_OPTIMISTIC_INCLUSION_THEN_BSPHERE_ONLY
     * @returns true if the bounding info is in the frustum planes
     */
    isInFrustum(frustumPlanes: Array<DeepImmutable<Plane>>, strategy?: number): boolean;
    /**
     * Gets the world distance between the min and max points of the bounding box
     */
    get diagonalLength(): number;
    /**
     * Checks if a cullable object (mesh...) is in the camera frustum
     * Unlike isInFrustum this checks the full bounding box
     * @param frustumPlanes Camera near/planes
     * @returns true if the object is in frustum otherwise false
     */
    isCompletelyInFrustum(frustumPlanes: Array<DeepImmutable<Plane>>): boolean;
    /**
     * @internal
     */
    _checkCollision(collider: Collider): boolean;
    /**
     * Checks if a point is inside the bounding box and bounding sphere or the mesh
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/interactions/mesh_intersect
     * @param point the point to check intersection with
     * @returns if the point intersects
     */
    intersectsPoint(point: DeepImmutable<Vector3>): boolean;
    /**
     * Checks if another bounding info intersects the bounding box and bounding sphere or the mesh
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/interactions/mesh_intersect
     * @param boundingInfo the bounding info to check intersection with
     * @param precise if the intersection should be done using OBB
     * @returns if the bounding info intersects
     */
    intersects(boundingInfo: DeepImmutable<BoundingInfo>, precise: boolean): boolean;
}
export {};

}
declare module "babylonjs/Culling/boundingSphere" {
import { DeepImmutable } from "babylonjs/types";
import { Matrix, Vector3 } from "babylonjs/Maths/math.vector";
import { Plane } from "babylonjs/Maths/math.plane";
/**
 * Class used to store bounding sphere information
 */
export class BoundingSphere {
    /**
     * Gets the center of the bounding sphere in local space
     */
    readonly center: Vector3;
    /**
     * Radius of the bounding sphere in local space
     */
    radius: number;
    /**
     * Gets the center of the bounding sphere in world space
     */
    readonly centerWorld: Vector3;
    /**
     * Radius of the bounding sphere in world space
     */
    radiusWorld: number;
    /**
     * Gets the minimum vector in local space
     */
    readonly minimum: Vector3;
    /**
     * Gets the maximum vector in local space
     */
    readonly maximum: Vector3;
    private _worldMatrix;
    private static readonly _TmpVector3;
    /**
     * Creates a new bounding sphere
     * @param min defines the minimum vector (in local space)
     * @param max defines the maximum vector (in local space)
     * @param worldMatrix defines the new world matrix
     */
    constructor(min: DeepImmutable<Vector3>, max: DeepImmutable<Vector3>, worldMatrix?: DeepImmutable<Matrix>);
    /**
     * Recreates the entire bounding sphere from scratch as if we call the constructor in place
     * @param min defines the new minimum vector (in local space)
     * @param max defines the new maximum vector (in local space)
     * @param worldMatrix defines the new world matrix
     */
    reConstruct(min: DeepImmutable<Vector3>, max: DeepImmutable<Vector3>, worldMatrix?: DeepImmutable<Matrix>): void;
    /**
     * Scale the current bounding sphere by applying a scale factor
     * @param factor defines the scale factor to apply
     * @returns the current bounding box
     */
    scale(factor: number): BoundingSphere;
    /**
     * Gets the world matrix of the bounding box
     * @returns a matrix
     */
    getWorldMatrix(): DeepImmutable<Matrix>;
    /**
     * @internal
     */
    _update(worldMatrix: DeepImmutable<Matrix>): void;
    /**
     * Tests if the bounding sphere is intersecting the frustum planes
     * @param frustumPlanes defines the frustum planes to test
     * @returns true if there is an intersection
     */
    isInFrustum(frustumPlanes: Array<DeepImmutable<Plane>>): boolean;
    /**
     * Tests if the bounding sphere center is in between the frustum planes.
     * Used for optimistic fast inclusion.
     * @param frustumPlanes defines the frustum planes to test
     * @returns true if the sphere center is in between the frustum planes
     */
    isCenterInFrustum(frustumPlanes: Array<DeepImmutable<Plane>>): boolean;
    /**
     * Tests if a point is inside the bounding sphere
     * @param point defines the point to test
     * @returns true if the point is inside the bounding sphere
     */
    intersectsPoint(point: DeepImmutable<Vector3>): boolean;
    /**
     * Checks if two sphere intersect
     * @param sphere0 sphere 0
     * @param sphere1 sphere 1
     * @returns true if the spheres intersect
     */
    static Intersects(sphere0: DeepImmutable<BoundingSphere>, sphere1: DeepImmutable<BoundingSphere>): boolean;
    /**
     * Creates a sphere from a center and a radius
     * @param center The center
     * @param radius radius
     * @param matrix Optional worldMatrix
     * @returns The sphere
     */
    static CreateFromCenterAndRadius(center: DeepImmutable<Vector3>, radius: number, matrix?: DeepImmutable<Matrix>): BoundingSphere;
}

}
declare module "babylonjs/Culling/index" {
export * from "babylonjs/Culling/boundingBox";
export * from "babylonjs/Culling/boundingInfo";
export * from "babylonjs/Culling/boundingSphere";
export * from "babylonjs/Culling/Octrees/index";
export * from "babylonjs/Culling/ray";

}
declare module "babylonjs/Culling/Octrees/index" {
export * from "babylonjs/Culling/Octrees/octree";
export * from "babylonjs/Culling/Octrees/octreeBlock";
export * from "babylonjs/Culling/Octrees/octreeSceneComponent";

}
declare module "babylonjs/Culling/Octrees/octree" {
import { SmartArray } from "babylonjs/Misc/smartArray";
import { Vector3 } from "babylonjs/Maths/math.vector";
import { SubMesh } from "babylonjs/Meshes/subMesh";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { Ray } from "babylonjs/Culling/ray";
import { OctreeBlock } from "babylonjs/Culling/Octrees/octreeBlock";
import { Plane } from "babylonjs/Maths/math.plane";
/**
 * Octrees are a really powerful data structure that can quickly select entities based on space coordinates.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/optimizeOctrees
 */
export class Octree<T> {
    /** Defines the maximum depth (sub-levels) for your octree. Default value is 2, which means 8 8 8 = 512 blocks :) (This parameter takes precedence over capacity.) */
    maxDepth: number;
    /**
     * Blocks within the octree containing objects
     */
    blocks: Array<OctreeBlock<T>>;
    /**
     * Content stored in the octree
     */
    dynamicContent: T[];
    private _maxBlockCapacity;
    private _selectionContent;
    private _creationFunc;
    /**
     * Creates a octree
     * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/optimizeOctrees
     * @param creationFunc function to be used to instantiate the octree
     * @param maxBlockCapacity defines the maximum number of meshes you want on your octree's leaves (default: 64)
     * @param maxDepth defines the maximum depth (sub-levels) for your octree. Default value is 2, which means 8 8 8 = 512 blocks :) (This parameter takes precedence over capacity.)
     */
    constructor(creationFunc: (entry: T, block: OctreeBlock<T>) => void, maxBlockCapacity?: number, 
    /** Defines the maximum depth (sub-levels) for your octree. Default value is 2, which means 8 8 8 = 512 blocks :) (This parameter takes precedence over capacity.) */
    maxDepth?: number);
    /**
     * Updates the octree by adding blocks for the passed in meshes within the min and max world parameters
     * @param worldMin worldMin for the octree blocks var blockSize = new Vector3((worldMax.x - worldMin.x) / 2, (worldMax.y - worldMin.y) / 2, (worldMax.z - worldMin.z) / 2);
     * @param worldMax worldMax for the octree blocks var blockSize = new Vector3((worldMax.x - worldMin.x) / 2, (worldMax.y - worldMin.y) / 2, (worldMax.z - worldMin.z) / 2);
     * @param entries meshes to be added to the octree blocks
     */
    update(worldMin: Vector3, worldMax: Vector3, entries: T[]): void;
    /**
     * Adds a mesh to the octree
     * @param entry Mesh to add to the octree
     */
    addMesh(entry: T): void;
    /**
     * Remove an element from the octree
     * @param entry defines the element to remove
     */
    removeMesh(entry: T): void;
    /**
     * Selects an array of meshes within the frustum
     * @param frustumPlanes The frustum planes to use which will select all meshes within it
     * @param allowDuplicate If duplicate objects are allowed in the resulting object array
     * @returns array of meshes within the frustum
     */
    select(frustumPlanes: Plane[], allowDuplicate?: boolean): SmartArray<T>;
    /**
     * Test if the octree intersect with the given bounding sphere and if yes, then add its content to the selection array
     * @param sphereCenter defines the bounding sphere center
     * @param sphereRadius defines the bounding sphere radius
     * @param allowDuplicate defines if the selection array can contains duplicated entries
     * @returns an array of objects that intersect the sphere
     */
    intersects(sphereCenter: Vector3, sphereRadius: number, allowDuplicate?: boolean): SmartArray<T>;
    /**
     * Test if the octree intersect with the given ray and if yes, then add its content to resulting array
     * @param ray defines the ray to test with
     * @returns array of intersected objects
     */
    intersectsRay(ray: Ray): SmartArray<T>;
    /**
     * Adds a mesh into the octree block if it intersects the block
     * @param entry
     * @param block
     */
    static CreationFuncForMeshes: (entry: AbstractMesh, block: OctreeBlock<AbstractMesh>) => void;
    /**
     * Adds a submesh into the octree block if it intersects the block
     * @param entry
     * @param block
     */
    static CreationFuncForSubMeshes: (entry: SubMesh, block: OctreeBlock<SubMesh>) => void;
}

}
declare module "babylonjs/Culling/Octrees/octreeBlock" {
import { SmartArrayNoDuplicate } from "babylonjs/Misc/smartArray";
import { Vector3 } from "babylonjs/Maths/math.vector";
import { Ray } from "babylonjs/Culling/ray";
import { Plane } from "babylonjs/Maths/math.plane";
/**
 * Contains an array of blocks representing the octree
 */
export interface IOctreeContainer<T> {
    /**
     * Blocks within the octree
     */
    blocks: Array<OctreeBlock<T>>;
}
/**
 * Class used to store a cell in an octree
 * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/optimizeOctrees
 */
export class OctreeBlock<T> {
    /**
     * Gets the content of the current block
     */
    entries: T[];
    /**
     * Gets the list of block children
     */
    blocks: Array<OctreeBlock<T>>;
    private _depth;
    private _maxDepth;
    private _capacity;
    private _minPoint;
    private _maxPoint;
    private _boundingVectors;
    private _creationFunc;
    /**
     * Creates a new block
     * @param minPoint defines the minimum vector (in world space) of the block's bounding box
     * @param maxPoint defines the maximum vector (in world space) of the block's bounding box
     * @param capacity defines the maximum capacity of this block (if capacity is reached the block will be split into sub blocks)
     * @param depth defines the current depth of this block in the octree
     * @param maxDepth defines the maximal depth allowed (beyond this value, the capacity is ignored)
     * @param creationFunc defines a callback to call when an element is added to the block
     */
    constructor(minPoint: Vector3, maxPoint: Vector3, capacity: number, depth: number, maxDepth: number, creationFunc: (entry: T, block: OctreeBlock<T>) => void);
    /**
     * Gets the maximum capacity of this block (if capacity is reached the block will be split into sub blocks)
     */
    get capacity(): number;
    /**
     * Gets the minimum vector (in world space) of the block's bounding box
     */
    get minPoint(): Vector3;
    /**
     * Gets the maximum vector (in world space) of the block's bounding box
     */
    get maxPoint(): Vector3;
    /**
     * Add a new element to this block
     * @param entry defines the element to add
     */
    addEntry(entry: T): void;
    /**
     * Remove an element from this block
     * @param entry defines the element to remove
     */
    removeEntry(entry: T): void;
    /**
     * Add an array of elements to this block
     * @param entries defines the array of elements to add
     */
    addEntries(entries: T[]): void;
    /**
     * Test if the current block intersects the frustum planes and if yes, then add its content to the selection array
     * @param frustumPlanes defines the frustum planes to test
     * @param selection defines the array to store current content if selection is positive
     * @param allowDuplicate defines if the selection array can contains duplicated entries
     */
    select(frustumPlanes: Plane[], selection: SmartArrayNoDuplicate<T>, allowDuplicate?: boolean): void;
    /**
     * Test if the current block intersect with the given bounding sphere and if yes, then add its content to the selection array
     * @param sphereCenter defines the bounding sphere center
     * @param sphereRadius defines the bounding sphere radius
     * @param selection defines the array to store current content if selection is positive
     * @param allowDuplicate defines if the selection array can contains duplicated entries
     */
    intersects(sphereCenter: Vector3, sphereRadius: number, selection: SmartArrayNoDuplicate<T>, allowDuplicate?: boolean): void;
    /**
     * Test if the current block intersect with the given ray and if yes, then add its content to the selection array
     * @param ray defines the ray to test with
     * @param selection defines the array to store current content if selection is positive
     */
    intersectsRay(ray: Ray, selection: SmartArrayNoDuplicate<T>): void;
    /**
     * Subdivide the content into child blocks (this block will then be empty)
     */
    createInnerBlocks(): void;
    /**
     * @internal
     */
    static _CreateBlocks<T>(worldMin: Vector3, worldMax: Vector3, entries: T[], maxBlockCapacity: number, currentDepth: number, maxDepth: number, target: IOctreeContainer<T>, creationFunc: (entry: T, block: OctreeBlock<T>) => void): void;
}

}
declare module "babylonjs/Culling/Octrees/octreeSceneComponent" {
import { ISmartArrayLike } from "babylonjs/Misc/smartArray";
import { Scene } from "babylonjs/scene";
import { SubMesh } from "babylonjs/Meshes/subMesh";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { Ray } from "babylonjs/Culling/ray";
import { Octree } from "babylonjs/Culling/Octrees/octree";
import { Collider } from "babylonjs/Collisions/collider";
module "babylonjs/scene" {
    interface Scene {
        /**
         * @internal
         * Backing Filed
         */
        _selectionOctree: Octree<AbstractMesh>;
        /**
         * Gets the octree used to boost mesh selection (picking)
         * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/optimizeOctrees
         */
        selectionOctree: Octree<AbstractMesh>;
        /**
         * Creates or updates the octree used to boost selection (picking)
         * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/optimizeOctrees
         * @param maxCapacity defines the maximum capacity per leaf
         * @param maxDepth defines the maximum depth of the octree
         * @returns an octree of AbstractMesh
         */
        createOrUpdateSelectionOctree(maxCapacity?: number, maxDepth?: number): Octree<AbstractMesh>;
    }
}
module "babylonjs/Meshes/abstractMesh" {
    interface AbstractMesh {
        /**
         * @internal
         * Backing Field
         */
        _submeshesOctree: Octree<SubMesh>;
        /**
         * This function will create an octree to help to select the right submeshes for rendering, picking and collision computations.
         * Please note that you must have a decent number of submeshes to get performance improvements when using an octree
         * @param maxCapacity defines the maximum size of each block (64 by default)
         * @param maxDepth defines the maximum depth to use (no more than 2 levels by default)
         * @returns the new octree
         * @see https://www.babylonjs-playground.com/#NA4OQ#12
         * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/optimizeOctrees
         */
        createOrUpdateSubmeshesOctree(maxCapacity?: number, maxDepth?: number): Octree<SubMesh>;
    }
}
/**
 * Defines the octree scene component responsible to manage any octrees
 * in a given scene.
 */
export class OctreeSceneComponent {
    /**
     * The component name help to identify the component in the list of scene components.
     */
    readonly name: string;
    /**
     * The scene the component belongs to.
     */
    scene: Scene;
    /**
     * Indicates if the meshes have been checked to make sure they are isEnabled()
     */
    readonly checksIsEnabled: boolean;
    /**
     * Creates a new instance of the component for the given scene
     * @param scene Defines the scene to register the component in
     */
    constructor(scene?: Scene);
    /**
     * Registers the component in a given scene
     */
    register(): void;
    /**
     * Return the list of active meshes
     * @returns the list of active meshes
     */
    getActiveMeshCandidates(): ISmartArrayLike<AbstractMesh>;
    /**
     * Return the list of active sub meshes
     * @param mesh The mesh to get the candidates sub meshes from
     * @returns the list of active sub meshes
     */
    getActiveSubMeshCandidates(mesh: AbstractMesh): ISmartArrayLike<SubMesh>;
    private _tempRay;
    /**
     * Return the list of sub meshes intersecting with a given local ray
     * @param mesh defines the mesh to find the submesh for
     * @param localRay defines the ray in local space
     * @returns the list of intersecting sub meshes
     */
    getIntersectingSubMeshCandidates(mesh: AbstractMesh, localRay: Ray): ISmartArrayLike<SubMesh>;
    /**
     * Return the list of sub meshes colliding with a collider
     * @param mesh defines the mesh to find the submesh for
     * @param collider defines the collider to evaluate the collision against
     * @returns the list of colliding sub meshes
     */
    getCollidingSubMeshCandidates(mesh: AbstractMesh, collider: Collider): ISmartArrayLike<SubMesh>;
    /**
     * Rebuilds the elements related to this component in case of
     * context lost for instance.
     */
    rebuild(): void;
    /**
     * Disposes the component and the associated resources.
     */
    dispose(): void;
}
export {};

}
declare module "babylonjs/Culling/ray" {
import { DeepImmutable, Nullable, float } from "babylonjs/types";
import { Matrix, Vector3 } from "babylonjs/Maths/math.vector";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { PickingInfo } from "babylonjs/Collisions/pickingInfo";
import { IntersectionInfo } from "babylonjs/Collisions/intersectionInfo";
import { BoundingBox } from "babylonjs/Culling/boundingBox";
import { BoundingSphere } from "babylonjs/Culling/boundingSphere";
import { Plane } from "babylonjs/Maths/math.plane";
/**
 * Class representing a ray with position and direction
 */
export class Ray {
    /** origin point */
    origin: Vector3;
    /** direction */
    direction: Vector3;
    /** length of the ray */
    length: number;
    private static readonly _TmpVector3;
    private static _RayDistant;
    private _tmpRay;
    /**
     * Creates a new ray
     * @param origin origin point
     * @param direction direction
     * @param length length of the ray
     */
    constructor(
    /** origin point */
    origin: Vector3, 
    /** direction */
    direction: Vector3, 
    /** length of the ray */
    length?: number);
    /**
     * Clone the current ray
     * @returns a new ray
     */
    clone(): Ray;
    /**
     * Checks if the ray intersects a box
     * This does not account for the ray length by design to improve perfs.
     * @param minimum bound of the box
     * @param maximum bound of the box
     * @param intersectionTreshold extra extend to be added to the box in all direction
     * @returns if the box was hit
     */
    intersectsBoxMinMax(minimum: DeepImmutable<Vector3>, maximum: DeepImmutable<Vector3>, intersectionTreshold?: number): boolean;
    /**
     * Checks if the ray intersects a box
     * This does not account for the ray lenght by design to improve perfs.
     * @param box the bounding box to check
     * @param intersectionTreshold extra extend to be added to the BoundingBox in all direction
     * @returns if the box was hit
     */
    intersectsBox(box: DeepImmutable<BoundingBox>, intersectionTreshold?: number): boolean;
    /**
     * If the ray hits a sphere
     * @param sphere the bounding sphere to check
     * @param intersectionTreshold extra extend to be added to the BoundingSphere in all direction
     * @returns true if it hits the sphere
     */
    intersectsSphere(sphere: DeepImmutable<BoundingSphere>, intersectionTreshold?: number): boolean;
    /**
     * If the ray hits a triange
     * @param vertex0 triangle vertex
     * @param vertex1 triangle vertex
     * @param vertex2 triangle vertex
     * @returns intersection information if hit
     */
    intersectsTriangle(vertex0: DeepImmutable<Vector3>, vertex1: DeepImmutable<Vector3>, vertex2: DeepImmutable<Vector3>): Nullable<IntersectionInfo>;
    /**
     * Checks if ray intersects a plane
     * @param plane the plane to check
     * @returns the distance away it was hit
     */
    intersectsPlane(plane: DeepImmutable<Plane>): Nullable<number>;
    /**
     * Calculate the intercept of a ray on a given axis
     * @param axis to check 'x' | 'y' | 'z'
     * @param offset from axis interception (i.e. an offset of 1y is intercepted above ground)
     * @returns a vector containing the coordinates where 'axis' is equal to zero (else offset), or null if there is no intercept.
     */
    intersectsAxis(axis: string, offset?: number): Nullable<Vector3>;
    /**
     * Checks if ray intersects a mesh
     * @param mesh the mesh to check
     * @param fastCheck defines if the first intersection will be used (and not the closest)
     * @returns picking info of the intersection
     */
    intersectsMesh(mesh: DeepImmutable<AbstractMesh>, fastCheck?: boolean): PickingInfo;
    /**
     * Checks if ray intersects a mesh
     * @param meshes the meshes to check
     * @param fastCheck defines if the first intersection will be used (and not the closest)
     * @param results array to store result in
     * @returns Array of picking infos
     */
    intersectsMeshes(meshes: Array<DeepImmutable<AbstractMesh>>, fastCheck?: boolean, results?: Array<PickingInfo>): Array<PickingInfo>;
    private _comparePickingInfo;
    private static _Smallnum;
    private static _Rayl;
    /**
     * Intersection test between the ray and a given segment within a given tolerance (threshold)
     * @param sega the first point of the segment to test the intersection against
     * @param segb the second point of the segment to test the intersection against
     * @param threshold the tolerance margin, if the ray doesn't intersect the segment but is close to the given threshold, the intersection is successful
     * @returns the distance from the ray origin to the intersection point if there's intersection, or -1 if there's no intersection
     */
    intersectionSegment(sega: DeepImmutable<Vector3>, segb: DeepImmutable<Vector3>, threshold: number): number;
    /**
     * Update the ray from viewport position
     * @param x position
     * @param y y position
     * @param viewportWidth viewport width
     * @param viewportHeight viewport height
     * @param world world matrix
     * @param view view matrix
     * @param projection projection matrix
     * @param enableDistantPicking defines if picking should handle large values for mesh position/scaling (false by default)
     * @returns this ray updated
     */
    update(x: number, y: number, viewportWidth: number, viewportHeight: number, world: DeepImmutable<Matrix>, view: DeepImmutable<Matrix>, projection: DeepImmutable<Matrix>, enableDistantPicking?: boolean): Ray;
    /**
     * Creates a ray with origin and direction of 0,0,0
     * @returns the new ray
     */
    static Zero(): Ray;
    /**
     * Creates a new ray from screen space and viewport
     * @param x position
     * @param y y position
     * @param viewportWidth viewport width
     * @param viewportHeight viewport height
     * @param world world matrix
     * @param view view matrix
     * @param projection projection matrix
     * @returns new ray
     */
    static CreateNew(x: number, y: number, viewportWidth: number, viewportHeight: number, world: DeepImmutable<Matrix>, view: DeepImmutable<Matrix>, projection: DeepImmutable<Matrix>): Ray;
    /**
     * Function will create a new transformed ray starting from origin and ending at the end point. Ray's length will be set, and ray will be
     * transformed to the given world matrix.
     * @param origin The origin point
     * @param end The end point
     * @param world a matrix to transform the ray to. Default is the identity matrix.
     * @returns the new ray
     */
    static CreateNewFromTo(origin: Vector3, end: Vector3, world?: DeepImmutable<Matrix>): Ray;
    /**
     * Transforms a ray by a matrix
     * @param ray ray to transform
     * @param matrix matrix to apply
     * @returns the resulting new ray
     */
    static Transform(ray: DeepImmutable<Ray>, matrix: DeepImmutable<Matrix>): Ray;
    /**
     * Transforms a ray by a matrix
     * @param ray ray to transform
     * @param matrix matrix to apply
     * @param result ray to store result in
     */
    static TransformToRef(ray: DeepImmutable<Ray>, matrix: DeepImmutable<Matrix>, result: Ray): void;
    /**
     * Unproject a ray from screen space to object space
     * @param sourceX defines the screen space x coordinate to use
     * @param sourceY defines the screen space y coordinate to use
     * @param viewportWidth defines the current width of the viewport
     * @param viewportHeight defines the current height of the viewport
     * @param world defines the world matrix to use (can be set to Identity to go to world space)
     * @param view defines the view matrix to use
     * @param projection defines the projection matrix to use
     */
    unprojectRayToRef(sourceX: float, sourceY: float, viewportWidth: number, viewportHeight: number, world: DeepImmutable<Matrix>, view: DeepImmutable<Matrix>, projection: DeepImmutable<Matrix>): void;
}
/**
 * Type used to define predicate used to select faces when a mesh intersection is detected
 */
export type TrianglePickingPredicate = (p0: Vector3, p1: Vector3, p2: Vector3, ray: Ray, i0: number, i1: number, i2: number) => boolean;
module "babylonjs/scene" {
    interface Scene {
        /** @internal */
        _tempPickingRay: Nullable<Ray>;
        /** @internal */
        _cachedRayForTransform: Ray;
        /** @internal */
        _pickWithRayInverseMatrix: Matrix;
        /** @internal */
        _internalPick(rayFunction: (world: Matrix, enableDistantPicking: boolean) => Ray, predicate?: (mesh: AbstractMesh) => boolean, fastCheck?: boolean, onlyBoundingInfo?: boolean, trianglePredicate?: TrianglePickingPredicate): PickingInfo;
        /** @internal */
        _internalMultiPick(rayFunction: (world: Matrix, enableDistantPicking: boolean) => Ray, predicate?: (mesh: AbstractMesh) => boolean, trianglePredicate?: TrianglePickingPredicate): Nullable<PickingInfo[]>;
        /** @internal */
        _internalPickForMesh(pickingInfo: Nullable<PickingInfo>, rayFunction: (world: Matrix, enableDistantPicking: boolean) => Ray, mesh: AbstractMesh, world: Matrix, fastCheck?: boolean, onlyBoundingInfo?: boolean, trianglePredicate?: TrianglePickingPredicate, skipBoundingInfo?: boolean): Nullable<PickingInfo>;
    }
}

}
declare module "babylonjs/Debug/axesViewer" {
import { Vector3 } from "babylonjs/Maths/math.vector";
import { Nullable } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
import { TransformNode } from "babylonjs/Meshes/transformNode";
/**
 * The Axes viewer will show 3 axes in a specific point in space
 * @see https://doc.babylonjs.com/toolsAndResources/utilities/World_Axes
 */
export class AxesViewer {
    private _xAxis;
    private _yAxis;
    private _zAxis;
    private _scaleLinesFactor;
    private _instanced;
    /**
     * Gets the hosting scene
     */
    scene: Nullable<Scene>;
    /**
     * Gets or sets a number used to scale line length
     */
    scaleLines: number;
    /** Gets the node hierarchy used to render x-axis */
    get xAxis(): TransformNode;
    /** Gets the node hierarchy used to render y-axis */
    get yAxis(): TransformNode;
    /** Gets the node hierarchy used to render z-axis */
    get zAxis(): TransformNode;
    /**
     * Creates a new AxesViewer
     * @param scene defines the hosting scene
     * @param scaleLines defines a number used to scale line length (1 by default)
     * @param renderingGroupId defines a number used to set the renderingGroupId of the meshes (2 by default)
     * @param xAxis defines the node hierarchy used to render the x-axis
     * @param yAxis defines the node hierarchy used to render the y-axis
     * @param zAxis defines the node hierarchy used to render the z-axis
     * @param lineThickness The line thickness to use when creating the arrow. defaults to 1.
     */
    constructor(scene?: Scene, scaleLines?: number, renderingGroupId?: Nullable<number>, xAxis?: TransformNode, yAxis?: TransformNode, zAxis?: TransformNode, lineThickness?: number);
    /**
     * Force the viewer to update
     * @param position defines the position of the viewer
     * @param xaxis defines the x axis of the viewer
     * @param yaxis defines the y axis of the viewer
     * @param zaxis defines the z axis of the viewer
     */
    update(position: Vector3, xaxis: Vector3, yaxis: Vector3, zaxis: Vector3): void;
    /**
     * Creates an instance of this axes viewer.
     * @returns a new axes viewer with instanced meshes
     */
    createInstance(): AxesViewer;
    /** Releases resources */
    dispose(): void;
    private static _SetRenderingGroupId;
}

}
declare module "babylonjs/Debug/boneAxesViewer" {
import { Nullable } from "babylonjs/types";
import { AxesViewer } from "babylonjs/Debug/axesViewer";
import { Vector3 } from "babylonjs/Maths/math.vector";
import { Mesh } from "babylonjs/Meshes/mesh";
import { Bone } from "babylonjs/Bones/bone";
import { Scene } from "babylonjs/scene";
/**
 * The BoneAxesViewer will attach 3 axes to a specific bone of a specific mesh
 * @see demo here: https://www.babylonjs-playground.com/#0DE8F4#8
 */
export class BoneAxesViewer extends AxesViewer {
    /**
     * Gets or sets the target mesh where to display the axes viewer
     */
    mesh: Nullable<Mesh>;
    /**
     * Gets or sets the target bone where to display the axes viewer
     */
    bone: Nullable<Bone>;
    /** Gets current position */
    pos: Vector3;
    /** Gets direction of X axis */
    xaxis: Vector3;
    /** Gets direction of Y axis */
    yaxis: Vector3;
    /** Gets direction of Z axis */
    zaxis: Vector3;
    /**
     * Creates a new BoneAxesViewer
     * @param scene defines the hosting scene
     * @param bone defines the target bone
     * @param mesh defines the target mesh
     * @param scaleLines defines a scaling factor for line length (1 by default)
     */
    constructor(scene: Scene, bone: Bone, mesh: Mesh, scaleLines?: number);
    /**
     * Force the viewer to update
     */
    update(): void;
    /** Releases resources */
    dispose(): void;
}

}
declare module "babylonjs/Debug/debugLayer" {
import { Scene } from "babylonjs/scene";
import { IInspectable } from "babylonjs/Misc/iInspectable";
import { Camera } from "babylonjs/Cameras/camera";
/**
 * Interface used to define scene explorer extensibility option
 */
export interface IExplorerExtensibilityOption {
    /**
     * Define the option label
     */
    label: string;
    /**
     * Defines the action to execute on click
     */
    action: (entity: any) => void;
}
/**
 * Defines a group of actions associated with a predicate to use when extending the Inspector scene explorer
 */
export interface IExplorerExtensibilityGroup {
    /**
     * Defines a predicate to test if a given type mut be extended
     */
    predicate: (entity: any) => boolean;
    /**
     * Gets the list of options added to a type
     */
    entries: IExplorerExtensibilityOption[];
}
/**
 * Defines a new node that will be displayed as top level node in the explorer
 */
export interface IExplorerAdditionalChild {
    /**
     * Gets the name of the additional node
     */
    name: string;
    /**
     * Function used to return the class name of the child node
     */
    getClassName(): string;
    /**
     * List of inspectable custom properties (used by the Inspector)
     * @see https://doc.babylonjs.com/toolsAndResources/inspector#extensibility
     */
    inspectableCustomProperties: IInspectable[];
}
/**
 * Defines a new node that will be displayed as top level node in the explorer
 */
export interface IExplorerAdditionalNode {
    /**
     * Gets the name of the additional node
     */
    name: string;
    /**
     * Function used to return the list of child entries
     */
    getContent(): IExplorerAdditionalChild[];
}
/**
 * Interface used to define the options to use to create the Inspector
 */
export interface IInspectorOptions {
    /**
     * Display in overlay mode (default: false)
     */
    overlay?: boolean;
    /**
     * HTML element to use as root (the parent of the rendering canvas will be used as default value)
     */
    globalRoot?: HTMLElement;
    /**
     * Display the Scene explorer
     */
    showExplorer?: boolean;
    /**
     * Display the property inspector
     */
    showInspector?: boolean;
    /**
     * Display in embed mode (both panes on the right)
     */
    embedMode?: boolean;
    /**
     * let the Inspector handles resize of the canvas when panes are resized (default to true)
     */
    handleResize?: boolean;
    /**
     * Allow the panes to popup (default: true)
     */
    enablePopup?: boolean;
    /**
     * Allow the panes to be closed by users (default: true)
     */
    enableClose?: boolean;
    /**
     * Optional list of extensibility entries
     */
    explorerExtensibility?: IExplorerExtensibilityGroup[];
    /**
     * Optional list of additional top level nodes
     */
    additionalNodes?: IExplorerAdditionalNode[];
    /**
     * Optional URL to get the inspector script from (by default it uses the babylonjs CDN).
     */
    inspectorURL?: string;
    /**
     * Optional initial tab (default to DebugLayerTab.Properties)
     */
    initialTab?: DebugLayerTab;
    /**
     * Optional camera to use to render the gizmos from the inspector (default to the scene.activeCamera or the latest from scene.activeCameras)
     */
    gizmoCamera?: Camera;
}
module "babylonjs/scene" {
    interface Scene {
        /**
         * @internal
         * Backing field
         */
        _debugLayer: DebugLayer;
        /**
         * Gets the debug layer (aka Inspector) associated with the scene
         * @see https://doc.babylonjs.com/toolsAndResources/inspector
         */
        debugLayer: DebugLayer;
    }
}
/**
 * Enum of inspector action tab
 */
export enum DebugLayerTab {
    /**
     * Properties tag (default)
     */
    Properties = 0,
    /**
     * Debug tab
     */
    Debug = 1,
    /**
     * Statistics tab
     */
    Statistics = 2,
    /**
     * Tools tab
     */
    Tools = 3,
    /**
     * Settings tab
     */
    Settings = 4
}
/**
 * The debug layer (aka Inspector) is the go to tool in order to better understand
 * what is happening in your scene
 * @see https://doc.babylonjs.com/toolsAndResources/inspector
 */
export class DebugLayer {
    /**
     * Define the url to get the inspector script from.
     * By default it uses the babylonjs CDN.
     * @ignoreNaming
     */
    static InspectorURL: string;
    private _scene;
    private BJSINSPECTOR;
    private _onPropertyChangedObservable?;
    /**
     * Observable triggered when a property is changed through the inspector.
     */
    get onPropertyChangedObservable(): any;
    private _onSelectionChangedObservable?;
    /**
     * Observable triggered when the selection is changed through the inspector.
     */
    get onSelectionChangedObservable(): any;
    /**
     * Instantiates a new debug layer.
     * The debug layer (aka Inspector) is the go to tool in order to better understand
     * what is happening in your scene
     * @see https://doc.babylonjs.com/toolsAndResources/inspector
     * @param scene Defines the scene to inspect
     */
    constructor(scene?: Scene);
    /**
     * Creates the inspector window.
     * @param config
     */
    private _createInspector;
    /**
     * Select a specific entity in the scene explorer and highlight a specific block in that entity property grid
     * @param entity defines the entity to select
     * @param lineContainerTitles defines the specific blocks to highlight (could be a string or an array of strings)
     */
    select(entity: any, lineContainerTitles?: string | string[]): void;
    /** Get the inspector from bundle or global */
    private _getGlobalInspector;
    /**
     * Get if the inspector is visible or not.
     * @returns true if visible otherwise, false
     */
    isVisible(): boolean;
    /**
     * Hide the inspector and close its window.
     */
    hide(): void;
    /**
     * Update the scene in the inspector
     */
    setAsActiveScene(): void;
    /**
     * Launch the debugLayer.
     * @param config Define the configuration of the inspector
     * @returns a promise fulfilled when the debug layer is visible
     */
    show(config?: IInspectorOptions): Promise<DebugLayer>;
}

}
declare module "babylonjs/Debug/directionalLightFrustumViewer" {
import { Camera } from "babylonjs/Cameras/camera";
import { DirectionalLight } from "babylonjs/Lights/directionalLight";
import { Matrix } from "babylonjs/Maths/math.vector";
/**
 * Class used to render a debug view of the frustum for a directional light
 * @see https://playground.babylonjs.com/#7EFGSG#4
 * @since 5.0.0
 */
export class DirectionalLightFrustumViewer {
    private _scene;
    private _light;
    private _camera;
    private _inverseViewMatrix;
    private _visible;
    private _rootNode;
    private _lightHelperFrustumMeshes;
    private _nearLinesPoints;
    private _farLinesPoints;
    private _trLinesPoints;
    private _brLinesPoints;
    private _tlLinesPoints;
    private _blLinesPoints;
    private _nearPlaneVertices;
    private _farPlaneVertices;
    private _rightPlaneVertices;
    private _leftPlaneVertices;
    private _topPlaneVertices;
    private _bottomPlaneVertices;
    private _oldPosition;
    private _oldDirection;
    private _oldAutoCalc;
    private _oldMinZ;
    private _oldMaxZ;
    private _transparency;
    /**
     * Gets or sets the transparency of the frustum planes
     */
    get transparency(): number;
    set transparency(alpha: number);
    private _showLines;
    /**
     * true to display the edges of the frustum
     */
    get showLines(): boolean;
    set showLines(show: boolean);
    private _showPlanes;
    /**
     * true to display the planes of the frustum
     */
    get showPlanes(): boolean;
    set showPlanes(show: boolean);
    /**
     * Creates a new frustum viewer
     * @param light directional light to display the frustum for
     * @param camera camera used to retrieve the minZ / maxZ values if the shadowMinZ/shadowMaxZ values of the light are not setup
     */
    constructor(light: DirectionalLight, camera: Camera);
    /**
     * Shows the frustum
     */
    show(): void;
    /**
     * Hides the frustum
     */
    hide(): void;
    /**
     * Updates the frustum.
     * Call this method to update the frustum view if the light has changed position/direction
     */
    update(): void;
    /**
     * Dispose of the class / remove the frustum view
     */
    dispose(): void;
    protected _createGeometry(): void;
    protected _getInvertViewMatrix(): Matrix;
}

}
declare module "babylonjs/Debug/index" {
export * from "babylonjs/Debug/axesViewer";
export * from "babylonjs/Debug/boneAxesViewer";
export * from "babylonjs/Debug/debugLayer";
export * from "babylonjs/Debug/physicsViewer";
export * from "babylonjs/Debug/rayHelper";
export * from "babylonjs/Debug/skeletonViewer";
export * from "babylonjs/Debug/ISkeletonViewer";
export * from "babylonjs/Debug/directionalLightFrustumViewer";

}
declare module "babylonjs/Debug/ISkeletonViewer" {
import { Skeleton } from "babylonjs/Bones/skeleton";
import { Color3 } from "babylonjs/Maths/math.color";
/**
 * Defines the options associated with the creation of a SkeletonViewer.
 */
export interface ISkeletonViewerOptions {
    /** Should the system pause animations before building the Viewer? */
    pauseAnimations: boolean;
    /** Should the system return the skeleton to rest before building? */
    returnToRest: boolean;
    /** public Display Mode of the Viewer */
    displayMode: number;
    /** Flag to toggle if the Viewer should use the CPU for animations or not? */
    displayOptions: ISkeletonViewerDisplayOptions;
    /** Flag to toggle if the Viewer should use the CPU for animations or not? */
    computeBonesUsingShaders: boolean;
    /** Flag ignore non weighted bones */
    useAllBones: boolean;
}
/**
 * Defines how to display the various bone meshes for the viewer.
 */
export interface ISkeletonViewerDisplayOptions {
    /** How far down to start tapering the bone spurs */
    midStep?: number;
    /** How big is the midStep? */
    midStepFactor?: number;
    /** Base for the Sphere Size */
    sphereBaseSize?: number;
    /** The ratio of the sphere to the longest bone in units */
    sphereScaleUnit?: number;
    /** Ratio for the Sphere Size */
    sphereFactor?: number;
    /** Whether a spur should attach its far end to the child bone position */
    spurFollowsChild?: boolean;
    /** Whether to show local axes or not  */
    showLocalAxes?: boolean;
    /** Length of each local axis */
    localAxesSize?: number;
}
/**
 * Defines the constructor options for the BoneWeight Shader.
 */
export interface IBoneWeightShaderOptions {
    /** Skeleton to Map */
    skeleton: Skeleton;
    /** Colors for Uninfluenced bones */
    colorBase?: Color3;
    /** Colors for 0.0-0.25 Weight bones */
    colorZero?: Color3;
    /** Color for 0.25-0.5 Weight Influence */
    colorQuarter?: Color3;
    /** Color for 0.5-0.75 Weight Influence */
    colorHalf?: Color3;
    /** Color for 0.75-1 Weight Influence */
    colorFull?: Color3;
    /** Color for Zero Weight Influence */
    targetBoneIndex?: number;
}
/**
 * Simple structure of the gradient steps for the Color Map.
 */
export interface ISkeletonMapShaderColorMapKnot {
    /** Color of the Knot */
    color: Color3;
    /** Location of the Knot */
    location: number;
}
/**
 * Defines the constructor options for the SkeletonMap Shader.
 */
export interface ISkeletonMapShaderOptions {
    /** Skeleton to Map */
    skeleton: Skeleton;
    /** Array of ColorMapKnots that make the gradient must be ordered with knot[i].location < knot[i+1].location*/
    colorMap?: ISkeletonMapShaderColorMapKnot[];
}

}
declare module "babylonjs/Debug/physicsViewer" {
import { Nullable } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { Mesh } from "babylonjs/Meshes/mesh";
import { IPhysicsEnginePlugin as IPhysicsEnginePluginV1 } from "babylonjs/Physics/v1/IPhysicsEnginePlugin";
import { IPhysicsEnginePluginV2 } from "babylonjs/Physics/v2/IPhysicsEnginePlugin";
import { PhysicsImpostor } from "babylonjs/Physics/v1/physicsImpostor";
import { PhysicsBody } from "babylonjs/Physics/v2/physicsBody";
/**
 * Used to show the physics impostor around the specific mesh
 */
export class PhysicsViewer {
    /** @internal */
    protected _impostors: Array<Nullable<PhysicsImpostor>>;
    /** @internal */
    protected _meshes: Array<Nullable<AbstractMesh>>;
    /** @internal */
    protected _bodies: Array<Nullable<PhysicsBody>>;
    /** @internal */
    protected _bodyMeshes: Array<Nullable<AbstractMesh>>;
    /** @internal */
    protected _scene: Nullable<Scene>;
    /** @internal */
    protected _numMeshes: number;
    /** @internal */
    protected _numBodies: number;
    /** @internal */
    protected _physicsEnginePlugin: IPhysicsEnginePluginV1 | IPhysicsEnginePluginV2 | null;
    private _renderFunction;
    private _utilityLayer;
    private _debugBoxMesh;
    private _debugSphereMesh;
    private _debugCapsuleMesh;
    private _debugCylinderMesh;
    private _debugMaterial;
    private _debugMeshMeshes;
    /**
     * Creates a new PhysicsViewer
     * @param scene defines the hosting scene
     */
    constructor(scene?: Scene);
    /**
     * Updates the debug meshes of the physics engine.
     *
     * This code is useful for synchronizing the debug meshes of the physics engine with the physics impostor and mesh.
     * It checks if the impostor is disposed and if the plugin version is 1, then it syncs the mesh with the impostor.
     * This ensures that the debug meshes are up to date with the physics engine.
     */
    protected _updateDebugMeshes(): void;
    /**
     * Updates the debug meshes of the physics engine.
     *
     * This method is useful for synchronizing the debug meshes with the physics impostors.
     * It iterates through the impostors and meshes, and if the plugin version is 1, it syncs the mesh with the impostor.
     * This ensures that the debug meshes accurately reflect the physics impostors, which is important for debugging the physics engine.
     */
    protected _updateDebugMeshesV1(): void;
    /**
     * Updates the debug meshes of the physics engine for V2 plugin.
     *
     * This method is useful for synchronizing the debug meshes of the physics engine with the current state of the bodies.
     * It iterates through the bodies array and updates the debug meshes with the current transform of each body.
     * This ensures that the debug meshes accurately reflect the current state of the physics engine.
     */
    protected _updateDebugMeshesV2(): void;
    /**
     * Renders a specified physic impostor
     * @param impostor defines the impostor to render
     * @param targetMesh defines the mesh represented by the impostor
     * @returns the new debug mesh used to render the impostor
     */
    showImpostor(impostor: PhysicsImpostor, targetMesh?: Mesh): Nullable<AbstractMesh>;
    /**
     * Shows a debug mesh for a given physics body.
     * @param body The physics body to show.
     * @returns The debug mesh, or null if the body is already shown.
     *
     * This function is useful for visualizing the physics body in the scene.
     * It creates a debug mesh for the given body and adds it to the scene.
     * It also registers a before render function to update the debug mesh position and rotation.
     */
    showBody(body: PhysicsBody): Nullable<AbstractMesh>;
    /**
     * Hides an impostor from the scene.
     * @param impostor - The impostor to hide.
     *
     * This method is useful for hiding an impostor from the scene. It removes the
     * impostor from the utility layer scene, disposes the mesh, and removes the
     * impostor from the list of impostors. If the impostor is the last one in the
     * list, it also unregisters the render function.
     */
    hideImpostor(impostor: Nullable<PhysicsImpostor>): void;
    /**
     * Hides a body from the physics engine.
     * @param body - The body to hide.
     *
     * This function is useful for hiding a body from the physics engine.
     * It removes the body from the utility layer scene and disposes the mesh associated with it.
     * It also unregisters the render function if the number of bodies is 0.
     * This is useful for hiding a body from the physics engine without deleting it.
     */
    hideBody(body: Nullable<PhysicsBody>): void;
    private _getDebugMaterial;
    private _getDebugBoxMesh;
    private _getDebugSphereMesh;
    private _getDebugCapsuleMesh;
    private _getDebugCylinderMesh;
    private _getDebugMeshMesh;
    private _getDebugMesh;
    /**
     * Creates a debug mesh for a given physics body
     * @param body The physics body to create the debug mesh for
     * @returns The created debug mesh or null if the utility layer is not available
     *
     * This code is useful for creating a debug mesh for a given physics body.
     * It creates a Mesh object with a VertexData object containing the positions and indices
     * of the geometry of the body. The mesh is then assigned a debug material from the utility layer scene.
     * This allows for visualizing the physics body in the scene.
     */
    private _getDebugBodyMesh;
    /**
     * Clean up physics debug display
     */
    dispose(): void;
}

}
declare module "babylonjs/Debug/rayHelper" {
import { Nullable } from "babylonjs/types";
import { Ray } from "babylonjs/Culling/ray";
import { Vector3 } from "babylonjs/Maths/math.vector";
import { Color3 } from "babylonjs/Maths/math.color";
import { Scene } from "babylonjs/scene";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
/**
 * As raycast might be hard to debug, the RayHelper can help rendering the different rays
 * in order to better appreciate the issue one might have.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/interactions/picking_collisions#debugging
 */
export class RayHelper {
    /**
     * Defines the ray we are currently tryin to visualize.
     */
    ray: Nullable<Ray>;
    private _renderPoints;
    private _renderLine;
    private _renderFunction;
    private _scene;
    private _onAfterRenderObserver;
    private _onAfterStepObserver;
    private _attachedToMesh;
    private _meshSpaceDirection;
    private _meshSpaceOrigin;
    /**
     * Helper function to create a colored helper in a scene in one line.
     * @param ray Defines the ray we are currently tryin to visualize
     * @param scene Defines the scene the ray is used in
     * @param color Defines the color we want to see the ray in
     * @returns The newly created ray helper.
     */
    static CreateAndShow(ray: Ray, scene: Scene, color: Color3): RayHelper;
    /**
     * Instantiate a new ray helper.
     * As raycast might be hard to debug, the RayHelper can help rendering the different rays
     * in order to better appreciate the issue one might have.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/interactions/picking_collisions#debugging
     * @param ray Defines the ray we are currently tryin to visualize
     */
    constructor(ray: Ray);
    /**
     * Shows the ray we are willing to debug.
     * @param scene Defines the scene the ray needs to be rendered in
     * @param color Defines the color the ray needs to be rendered in
     */
    show(scene: Scene, color?: Color3): void;
    /**
     * Hides the ray we are debugging.
     */
    hide(): void;
    private _render;
    /**
     * Attach a ray helper to a mesh so that we can easily see its orientation for instance or information like its normals.
     * @param mesh Defines the mesh we want the helper attached to
     * @param meshSpaceDirection Defines the direction of the Ray in mesh space (local space of the mesh node)
     * @param meshSpaceOrigin Defines the origin of the Ray in mesh space (local space of the mesh node)
     * @param length Defines the length of the ray
     */
    attachToMesh(mesh: AbstractMesh, meshSpaceDirection?: Vector3, meshSpaceOrigin?: Vector3, length?: number): void;
    /**
     * Detach the ray helper from the mesh it has previously been attached to.
     */
    detachFromMesh(): void;
    private _updateToMesh;
    /**
     * Dispose the helper and release its associated resources.
     */
    dispose(): void;
}

}
declare module "babylonjs/Debug/skeletonViewer" {
import { Color3 } from "babylonjs/Maths/math.color";
import { Scene } from "babylonjs/scene";
import { Nullable } from "babylonjs/types";
import { Skeleton } from "babylonjs/Bones/skeleton";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { LinesMesh } from "babylonjs/Meshes/linesMesh";
import { UtilityLayerRenderer } from "babylonjs/Rendering/utilityLayerRenderer";
import { ShaderMaterial } from "babylonjs/Materials/shaderMaterial";
import { ISkeletonViewerOptions, IBoneWeightShaderOptions, ISkeletonMapShaderOptions } from "babylonjs/Debug/ISkeletonViewer";
/**
 * Class used to render a debug view of a given skeleton
 * @see http://www.babylonjs-playground.com/#1BZJVJ#8
 */
export class SkeletonViewer {
    /** defines the skeleton to render */
    skeleton: Skeleton;
    /** defines the mesh attached to the skeleton */
    mesh: AbstractMesh;
    /** defines a boolean indicating if bones matrices must be forced to update before rendering (true by default)  */
    autoUpdateBonesMatrices: boolean;
    /** defines the rendering group id to use with the viewer */
    renderingGroupId: number;
    /** is the options for the viewer */
    options: Partial<ISkeletonViewerOptions>;
    /** public Display constants BABYLON.SkeletonViewer.DISPLAY_LINES */
    static readonly DISPLAY_LINES: number;
    /** public Display constants BABYLON.SkeletonViewer.DISPLAY_SPHERES */
    static readonly DISPLAY_SPHERES: number;
    /** public Display constants BABYLON.SkeletonViewer.DISPLAY_SPHERE_AND_SPURS */
    static readonly DISPLAY_SPHERE_AND_SPURS: number;
    /** public static method to create a BoneWeight Shader
     * @param options The constructor options
     * @param scene The scene that the shader is scoped to
     * @returns The created ShaderMaterial
     * @see http://www.babylonjs-playground.com/#1BZJVJ#395
     */
    static CreateBoneWeightShader(options: IBoneWeightShaderOptions, scene: Scene): ShaderMaterial;
    /** public static method to create a BoneWeight Shader
     * @param options The constructor options
     * @param scene The scene that the shader is scoped to
     * @returns The created ShaderMaterial
     */
    static CreateSkeletonMapShader(options: ISkeletonMapShaderOptions, scene: Scene): ShaderMaterial;
    /** private static method to create a BoneWeight Shader
     * @param size The size of the buffer to create (usually the bone count)
     * @param colorMap The gradient data to generate
     * @param scene The scene that the shader is scoped to
     * @returns an Array of floats from the color gradient values
     */
    private static _CreateBoneMapColorBuffer;
    /** If SkeletonViewer scene scope. */
    private _scene;
    /** Gets or sets the color used to render the skeleton */
    color: Color3;
    /** Array of the points of the skeleton fo the line view. */
    private _debugLines;
    /** The SkeletonViewers Mesh. */
    private _debugMesh;
    /** The local axes Meshes. */
    private _localAxes;
    /** If SkeletonViewer is enabled. */
    private _isEnabled;
    /** If SkeletonViewer is ready. */
    private _ready;
    /** SkeletonViewer render observable. */
    private _obs;
    /** The Utility Layer to render the gizmos in. */
    private _utilityLayer;
    private _boneIndices;
    /** Gets the Scene. */
    get scene(): Scene;
    /** Gets the utilityLayer. */
    get utilityLayer(): Nullable<UtilityLayerRenderer>;
    /** Checks Ready Status. */
    get isReady(): Boolean;
    /** Sets Ready Status. */
    set ready(value: boolean);
    /** Gets the debugMesh */
    get debugMesh(): Nullable<AbstractMesh> | Nullable<LinesMesh>;
    /** Sets the debugMesh */
    set debugMesh(value: Nullable<AbstractMesh> | Nullable<LinesMesh>);
    /** Gets the displayMode */
    get displayMode(): number;
    /** Sets the displayMode */
    set displayMode(value: number);
    /**
     * Creates a new SkeletonViewer
     * @param skeleton defines the skeleton to render
     * @param mesh defines the mesh attached to the skeleton
     * @param scene defines the hosting scene
     * @param autoUpdateBonesMatrices defines a boolean indicating if bones matrices must be forced to update before rendering (true by default)
     * @param renderingGroupId defines the rendering group id to use with the viewer
     * @param options All of the extra constructor options for the SkeletonViewer
     */
    constructor(
    /** defines the skeleton to render */
    skeleton: Skeleton, 
    /** defines the mesh attached to the skeleton */
    mesh: AbstractMesh, 
    /** The Scene scope*/
    scene: Scene, 
    /** defines a boolean indicating if bones matrices must be forced to update before rendering (true by default)  */
    autoUpdateBonesMatrices?: boolean, 
    /** defines the rendering group id to use with the viewer */
    renderingGroupId?: number, 
    /** is the options for the viewer */
    options?: Partial<ISkeletonViewerOptions>);
    /** The Dynamic bindings for the update functions */
    private _bindObs;
    /** Update the viewer to sync with current skeleton state, only used to manually update. */
    update(): void;
    /** Gets or sets a boolean indicating if the viewer is enabled */
    set isEnabled(value: boolean);
    get isEnabled(): boolean;
    private _getBonePosition;
    private _getLinesForBonesWithLength;
    private _getLinesForBonesNoLength;
    /**
     * function to revert the mesh and scene back to the initial state.
     * @param animationState
     */
    private _revert;
    /**
     * function to get the absolute bind pose of a bone by accumulating transformations up the bone hierarchy.
     * @param bone
     * @param matrix
     */
    private _getAbsoluteBindPoseToRef;
    /**
     * function to build and bind sphere joint points and spur bone representations.
     * @param spheresOnly
     */
    private _buildSpheresAndSpurs;
    private _buildLocalAxes;
    /** Update the viewer to sync with current skeleton state, only used for the line display. */
    private _displayLinesUpdate;
    /** Changes the displayMode of the skeleton viewer
     * @param mode The displayMode numerical value
     */
    changeDisplayMode(mode: number): void;
    /** Sets a display option of the skeleton viewer
     *
     * | Option           | Type    | Default | Description |
     * | ---------------- | ------- | ------- | ----------- |
     * | midStep          | float   | 0.235   | A percentage between a bone and its child that determines the widest part of a spur. Only used when `displayMode` is set to `DISPLAY_SPHERE_AND_SPURS`. |
     * | midStepFactor    | float   | 0.15    | Mid step width expressed as a factor of the length. A value of 0.5 makes the spur width half of the spur length. Only used when `displayMode` is set to `DISPLAY_SPHERE_AND_SPURS`. |
     * | sphereBaseSize   | float   | 2       | Sphere base size. Only used when `displayMode` is set to `DISPLAY_SPHERE_AND_SPURS`. |
     * | sphereScaleUnit  | float   | 0.865   | Sphere scale factor used to scale spheres in relation to the longest bone. Only used when `displayMode` is set to `DISPLAY_SPHERE_AND_SPURS`. |
     * | spurFollowsChild | boolean | false   | Whether a spur should attach its far end to the child bone. |
     * | showLocalAxes    | boolean | false   | Displays local axes on all bones. |
     * | localAxesSize    | float   | 0.075   | Determines the length of each local axis. |
     *
     * @param option String of the option name
     * @param value The numerical option value
     */
    changeDisplayOptions(option: string, value: number): void;
    /** Release associated resources */
    dispose(): void;
}

}
declare module "babylonjs/DeviceInput/eventFactory" {
import { IUIEvent } from "babylonjs/Events/deviceInputEvents";
import { Nullable } from "babylonjs/types";
import { DeviceType } from "babylonjs/DeviceInput/InputDevices/deviceEnums";
import { IDeviceInputSystem } from "babylonjs/DeviceInput/inputInterfaces";
/**
 * Class to wrap DeviceInputSystem data into an event object
 */
export class DeviceEventFactory {
    /**
     * Create device input events based on provided type and slot
     *
     * @param deviceType Type of device
     * @param deviceSlot "Slot" or index that device is referenced in
     * @param inputIndex Id of input to be checked
     * @param currentState Current value for given input
     * @param deviceInputSystem Reference to DeviceInputSystem
     * @param elementToAttachTo HTMLElement to reference as target for inputs
     * @returns IUIEvent object
     */
    static CreateDeviceEvent(deviceType: DeviceType, deviceSlot: number, inputIndex: number, currentState: Nullable<number>, deviceInputSystem: IDeviceInputSystem, elementToAttachTo?: any): IUIEvent;
    /**
     * Creates pointer event
     *
     * @param deviceType Type of device
     * @param deviceSlot "Slot" or index that device is referenced in
     * @param inputIndex Id of input to be checked
     * @param currentState Current value for given input
     * @param deviceInputSystem Reference to DeviceInputSystem
     * @param elementToAttachTo HTMLElement to reference as target for inputs
     * @returns IUIEvent object (Pointer)
     */
    private static _CreatePointerEvent;
    /**
     * Create Mouse Wheel Event
     * @param deviceType Type of device
     * @param deviceSlot "Slot" or index that device is referenced in
     * @param inputIndex Id of input to be checked
     * @param currentState Current value for given input
     * @param deviceInputSystem Reference to DeviceInputSystem
     * @param elementToAttachTo HTMLElement to reference as target for inputs
     * @returns IUIEvent object (Wheel)
     */
    private static _CreateWheelEvent;
    /**
     * Create Mouse Event
     * @param deviceType Type of device
     * @param deviceSlot "Slot" or index that device is referenced in
     * @param inputIndex Id of input to be checked
     * @param currentState Current value for given input
     * @param deviceInputSystem Reference to DeviceInputSystem
     * @param elementToAttachTo HTMLElement to reference as target for inputs
     * @returns IUIEvent object (Mouse)
     */
    private static _CreateMouseEvent;
    /**
     * Create Keyboard Event
     * @param inputIndex Id of input to be checked
     * @param currentState Current value for given input
     * @param deviceInputSystem Reference to DeviceInputSystem
     * @param elementToAttachTo HTMLElement to reference as target for inputs
     * @returns IEvent object (Keyboard)
     */
    private static _CreateKeyboardEvent;
    /**
     * Add parameters for non-character keys (Ctrl, Alt, Meta, Shift)
     * @param evt Event object to add parameters to
     * @param deviceInputSystem DeviceInputSystem to pull values from
     */
    private static _CheckNonCharacterKeys;
    /**
     * Create base event object
     * @param elementToAttachTo Value to use as event target
     * @returns
     */
    private static _CreateEvent;
}

}
declare module "babylonjs/DeviceInput/index" {
export * from "babylonjs/DeviceInput/InputDevices/deviceEnums";
export * from "babylonjs/DeviceInput/InputDevices/deviceTypes";
export * from "babylonjs/DeviceInput/InputDevices/deviceSource";
export * from "babylonjs/DeviceInput/InputDevices/deviceSourceManager";

}
declare module "babylonjs/DeviceInput/InputDevices/deviceEnums" {
/**
 * Enum for Device Types
 */
export enum DeviceType {
    /** Generic */
    Generic = 0,
    /** Keyboard */
    Keyboard = 1,
    /** Mouse */
    Mouse = 2,
    /** Touch Pointers */
    Touch = 3,
    /** PS4 Dual Shock */
    DualShock = 4,
    /** Xbox */
    Xbox = 5,
    /** Switch Controller */
    Switch = 6,
    /** PS5 DualSense */
    DualSense = 7
}
/**
 * Enum for All Pointers (Touch/Mouse)
 */
export enum PointerInput {
    /** Horizontal Axis (Not used in events/observables; only in polling) */
    Horizontal = 0,
    /** Vertical Axis (Not used in events/observables; only in polling) */
    Vertical = 1,
    /** Left Click or Touch */
    LeftClick = 2,
    /** Middle Click */
    MiddleClick = 3,
    /** Right Click */
    RightClick = 4,
    /** Browser Back */
    BrowserBack = 5,
    /** Browser Forward */
    BrowserForward = 6,
    /** Mouse Wheel X */
    MouseWheelX = 7,
    /** Mouse Wheel Y */
    MouseWheelY = 8,
    /** Mouse Wheel Z */
    MouseWheelZ = 9,
    /** Used in events/observables to identify if x/y changes occurred */
    Move = 12
}
/** @internal */
export enum NativePointerInput {
    /** Horizontal Axis */
    Horizontal = 0,
    /** Vertical Axis */
    Vertical = 1,
    /** Left Click or Touch */
    LeftClick = 2,
    /** Middle Click */
    MiddleClick = 3,
    /** Right Click */
    RightClick = 4,
    /** Browser Back */
    BrowserBack = 5,
    /** Browser Forward */
    BrowserForward = 6,
    /** Mouse Wheel X */
    MouseWheelX = 7,
    /** Mouse Wheel Y */
    MouseWheelY = 8,
    /** Mouse Wheel Z */
    MouseWheelZ = 9,
    /** Delta X */
    DeltaHorizontal = 10,
    /** Delta Y */
    DeltaVertical = 11
}
/**
 * Enum for Dual Shock Gamepad
 */
export enum DualShockInput {
    /** Cross */
    Cross = 0,
    /** Circle */
    Circle = 1,
    /** Square */
    Square = 2,
    /** Triangle */
    Triangle = 3,
    /** L1 */
    L1 = 4,
    /** R1 */
    R1 = 5,
    /** L2 */
    L2 = 6,
    /** R2 */
    R2 = 7,
    /** Share */
    Share = 8,
    /** Options */
    Options = 9,
    /** L3 */
    L3 = 10,
    /** R3 */
    R3 = 11,
    /** DPadUp */
    DPadUp = 12,
    /** DPadDown */
    DPadDown = 13,
    /** DPadLeft */
    DPadLeft = 14,
    /** DRight */
    DPadRight = 15,
    /** Home */
    Home = 16,
    /** TouchPad */
    TouchPad = 17,
    /** LStickXAxis */
    LStickXAxis = 18,
    /** LStickYAxis */
    LStickYAxis = 19,
    /** RStickXAxis */
    RStickXAxis = 20,
    /** RStickYAxis */
    RStickYAxis = 21
}
/**
 * Enum for Dual Sense Gamepad
 */
export enum DualSenseInput {
    /** Cross */
    Cross = 0,
    /** Circle */
    Circle = 1,
    /** Square */
    Square = 2,
    /** Triangle */
    Triangle = 3,
    /** L1 */
    L1 = 4,
    /** R1 */
    R1 = 5,
    /** L2 */
    L2 = 6,
    /** R2 */
    R2 = 7,
    /** Create */
    Create = 8,
    /** Options */
    Options = 9,
    /** L3 */
    L3 = 10,
    /** R3 */
    R3 = 11,
    /** DPadUp */
    DPadUp = 12,
    /** DPadDown */
    DPadDown = 13,
    /** DPadLeft */
    DPadLeft = 14,
    /** DRight */
    DPadRight = 15,
    /** Home */
    Home = 16,
    /** TouchPad */
    TouchPad = 17,
    /** LStickXAxis */
    LStickXAxis = 18,
    /** LStickYAxis */
    LStickYAxis = 19,
    /** RStickXAxis */
    RStickXAxis = 20,
    /** RStickYAxis */
    RStickYAxis = 21
}
/**
 * Enum for Xbox Gamepad
 */
export enum XboxInput {
    /** A */
    A = 0,
    /** B */
    B = 1,
    /** X */
    X = 2,
    /** Y */
    Y = 3,
    /** LB */
    LB = 4,
    /** RB */
    RB = 5,
    /** LT */
    LT = 6,
    /** RT */
    RT = 7,
    /** Back */
    Back = 8,
    /** Start */
    Start = 9,
    /** LS */
    LS = 10,
    /** RS */
    RS = 11,
    /** DPadUp */
    DPadUp = 12,
    /** DPadDown */
    DPadDown = 13,
    /** DPadLeft */
    DPadLeft = 14,
    /** DRight */
    DPadRight = 15,
    /** Home */
    Home = 16,
    /** LStickXAxis */
    LStickXAxis = 17,
    /** LStickYAxis */
    LStickYAxis = 18,
    /** RStickXAxis */
    RStickXAxis = 19,
    /** RStickYAxis */
    RStickYAxis = 20
}
/**
 * Enum for Switch (Pro/JoyCon L+R) Gamepad
 */
export enum SwitchInput {
    /** B */
    B = 0,
    /** A */
    A = 1,
    /** Y */
    Y = 2,
    /** X */
    X = 3,
    /** L */
    L = 4,
    /** R */
    R = 5,
    /** ZL */
    ZL = 6,
    /** ZR */
    ZR = 7,
    /** Minus */
    Minus = 8,
    /** Plus */
    Plus = 9,
    /** LS */
    LS = 10,
    /** RS */
    RS = 11,
    /** DPadUp */
    DPadUp = 12,
    /** DPadDown */
    DPadDown = 13,
    /** DPadLeft */
    DPadLeft = 14,
    /** DRight */
    DPadRight = 15,
    /** Home */
    Home = 16,
    /** Capture */
    Capture = 17,
    /** LStickXAxis */
    LStickXAxis = 18,
    /** LStickYAxis */
    LStickYAxis = 19,
    /** RStickXAxis */
    RStickXAxis = 20,
    /** RStickYAxis */
    RStickYAxis = 21
}

}
declare module "babylonjs/DeviceInput/InputDevices/deviceSource" {
import { DeviceType } from "babylonjs/DeviceInput/InputDevices/deviceEnums";
import { Observable } from "babylonjs/Misc/observable";
import { DeviceInput } from "babylonjs/DeviceInput/InputDevices/deviceTypes";
import { IDeviceInputSystem } from "babylonjs/DeviceInput/inputInterfaces";
import { IKeyboardEvent, IPointerEvent, IWheelEvent } from "babylonjs/Events/deviceInputEvents";
/**
 * Subset of DeviceInput that only handles pointers and keyboard
 */
export type DeviceSourceEvent<T extends DeviceType> = T extends DeviceType.Keyboard ? IKeyboardEvent : T extends DeviceType.Mouse ? IWheelEvent | IPointerEvent : T extends DeviceType.Touch ? IPointerEvent : never;
/**
 * Class that handles all input for a specific device
 */
export class DeviceSource<T extends DeviceType> {
    /** Type of device */
    readonly deviceType: T;
    /** "Slot" or index that device is referenced in */
    readonly deviceSlot: number;
    /**
     * Observable to handle device input changes per device
     */
    readonly onInputChangedObservable: Observable<DeviceSourceEvent<T>>;
    private readonly _deviceInputSystem;
    /**
     * Default Constructor
     * @param deviceInputSystem - Reference to DeviceInputSystem
     * @param deviceType - Type of device
     * @param deviceSlot - "Slot" or index that device is referenced in
     */
    constructor(deviceInputSystem: IDeviceInputSystem, 
    /** Type of device */
    deviceType: T, 
    /** "Slot" or index that device is referenced in */
    deviceSlot?: number);
    /**
     * Get input for specific input
     * @param inputIndex - index of specific input on device
     * @returns Input value from DeviceInputSystem
     */
    getInput(inputIndex: DeviceInput<T>): number;
}

}
declare module "babylonjs/DeviceInput/InputDevices/deviceSourceManager" {
import { Engine } from "babylonjs/Engines/engine";
import { DeviceType } from "babylonjs/DeviceInput/InputDevices/deviceEnums";
import { Nullable } from "babylonjs/types";
import { Observable } from "babylonjs/Misc/observable";
import { DeviceSource } from "babylonjs/DeviceInput/InputDevices/deviceSource";
import { IObservableManager, DeviceSourceType } from "babylonjs/DeviceInput/internalDeviceSourceManager";
import { IDisposable } from "babylonjs/scene";
import { IUIEvent } from "babylonjs/Events/deviceInputEvents";
/**
 * Class to keep track of devices
 */
export class DeviceSourceManager implements IDisposable, IObservableManager {
    /**
     * Observable to be triggered when after a device is connected, any new observers added will be triggered against already connected devices
     */
    readonly onDeviceConnectedObservable: Observable<DeviceSourceType>;
    /**
     * Observable to be triggered when after a device is disconnected
     */
    readonly onDeviceDisconnectedObservable: Observable<DeviceSourceType>;
    private _engine;
    private _onDisposeObserver;
    private readonly _devices;
    private readonly _firstDevice;
    /**
     * Gets a DeviceSource, given a type and slot
     * @param deviceType - Type of Device
     * @param deviceSlot - Slot or ID of device
     * @returns DeviceSource
     */
    getDeviceSource<T extends DeviceType>(deviceType: T, deviceSlot?: number): Nullable<DeviceSource<T>>;
    /**
     * Gets an array of DeviceSource objects for a given device type
     * @param deviceType - Type of Device
     * @returns All available DeviceSources of a given type
     */
    getDeviceSources<T extends DeviceType>(deviceType: T): ReadonlyArray<DeviceSource<T>>;
    /**
     * Default constructor
     * @param engine - Used to get canvas (if applicable)
     */
    constructor(engine: Engine);
    /**
     * Dispose of DeviceSourceManager
     */
    dispose(): void;
    /**
     * @param deviceSource - Source to add
     * @internal
     */
    _addDevice(deviceSource: DeviceSourceType): void;
    /**
     * @param deviceType - DeviceType
     * @param deviceSlot - DeviceSlot
     * @internal
     */
    _removeDevice(deviceType: DeviceType, deviceSlot: number): void;
    /**
     * @param deviceType - DeviceType
     * @param deviceSlot - DeviceSlot
     * @param eventData - Event
     * @internal
     */
    _onInputChanged<T extends DeviceType>(deviceType: T, deviceSlot: number, eventData: IUIEvent): void;
    private _updateFirstDevices;
}

}
declare module "babylonjs/DeviceInput/InputDevices/deviceTypes" {
import { DeviceType, PointerInput, DualShockInput, XboxInput, SwitchInput, DualSenseInput } from "babylonjs/DeviceInput/InputDevices/deviceEnums";
/**
 * Type to handle enforcement of inputs
 */
export type DeviceInput<T extends DeviceType> = T extends DeviceType.Keyboard | DeviceType.Generic ? number : T extends DeviceType.Mouse | DeviceType.Touch ? Exclude<PointerInput, PointerInput.Move | PointerInput.MouseWheelX | PointerInput.MouseWheelY | PointerInput.MouseWheelZ> : T extends DeviceType.DualShock ? DualShockInput : T extends DeviceType.Xbox ? XboxInput : T extends DeviceType.Switch ? SwitchInput : T extends DeviceType.DualSense ? DualSenseInput : never;

}
declare module "babylonjs/DeviceInput/inputInterfaces" {
import { IDisposable } from "babylonjs/scene";
import { DeviceType } from "babylonjs/DeviceInput/InputDevices/deviceEnums";
/**
 * Interface for DeviceInputSystem implementations (JS and Native)
 */
export interface IDeviceInputSystem extends IDisposable {
    /**
     * Checks for current device input value, given an id and input index. Throws exception if requested device not initialized.
     * @param deviceType Enum specifying device type
     * @param deviceSlot "Slot" or index that device is referenced in
     * @param inputIndex Id of input to be checked
     * @returns Current value of input
     */
    pollInput(deviceType: DeviceType, deviceSlot: number, inputIndex: number): number;
    /**
     * Check for a specific device in the DeviceInputSystem
     * @param deviceType Type of device to check for
     * @returns bool with status of device's existence
     */
    isDeviceAvailable(deviceType: DeviceType): boolean;
}

}
declare module "babylonjs/DeviceInput/internalDeviceSourceManager" {
import { IDisposable } from "babylonjs/scene";
import { DeviceType } from "babylonjs/DeviceInput/InputDevices/deviceEnums";
import { Observable } from "babylonjs/Misc/observable";
import { IDeviceInputSystem } from "babylonjs/DeviceInput/inputInterfaces";
import { DeviceSource } from "babylonjs/DeviceInput/InputDevices/deviceSource";
import { Engine } from "babylonjs/Engines/engine";
import { IUIEvent } from "babylonjs/Events/deviceInputEvents";
type Distribute<T> = T extends DeviceType ? DeviceSource<T> : never;
export type DeviceSourceType = Distribute<DeviceType>;
module "babylonjs/Engines/engine" {
    interface Engine {
        /** @internal */
        _deviceSourceManager?: InternalDeviceSourceManager;
    }
}
/** @internal */
export interface IObservableManager {
    onDeviceConnectedObservable: Observable<DeviceSourceType>;
    onDeviceDisconnectedObservable: Observable<DeviceSourceType>;
    _onInputChanged(deviceType: DeviceType, deviceSlot: number, eventData: IUIEvent): void;
    _addDevice(deviceSource: DeviceSource<DeviceType>): void;
    _removeDevice(deviceType: DeviceType, deviceSlot: number): void;
}
/** @internal */
export class InternalDeviceSourceManager implements IDisposable {
    readonly _deviceInputSystem: IDeviceInputSystem;
    private readonly _devices;
    private readonly _registeredManagers;
    _refCount: number;
    constructor(engine: Engine);
    readonly registerManager: (manager: IObservableManager) => void;
    readonly unregisterManager: (manager: IObservableManager) => void;
    dispose(): void;
}
export {};

}
declare module "babylonjs/DeviceInput/nativeDeviceInputSystem" {
import { IUIEvent } from "babylonjs/Events/deviceInputEvents";
import { DeviceType } from "babylonjs/DeviceInput/InputDevices/deviceEnums";
import { IDeviceInputSystem } from "babylonjs/DeviceInput/inputInterfaces";
/** @internal */
export class NativeDeviceInputSystem implements IDeviceInputSystem {
    private readonly _nativeInput;
    constructor(onDeviceConnected: (deviceType: DeviceType, deviceSlot: number) => void, onDeviceDisconnected: (deviceType: DeviceType, deviceSlot: number) => void, onInputChanged: (deviceType: DeviceType, deviceSlot: number, eventData: IUIEvent) => void);
    /**
     * Checks for current device input value, given an id and input index. Throws exception if requested device not initialized.
     * @param deviceType Enum specifying device type
     * @param deviceSlot "Slot" or index that device is referenced in
     * @param inputIndex Id of input to be checked
     * @returns Current value of input
     */
    pollInput(deviceType: DeviceType, deviceSlot: number, inputIndex: number): number;
    /**
     * Check for a specific device in the DeviceInputSystem
     * @param deviceType Type of device to check for
     * @returns bool with status of device's existence
     */
    isDeviceAvailable(deviceType: DeviceType): boolean;
    /**
     * Dispose of all the observables
     */
    dispose(): void;
    /**
     * For versions of BabylonNative that don't have the NativeInput plugin initialized, create a dummy version
     * @returns Object with dummy functions
     */
    private _createDummyNativeInput;
}

}
declare module "babylonjs/DeviceInput/webDeviceInputSystem" {
import { Engine } from "babylonjs/Engines/engine";
import { IUIEvent } from "babylonjs/Events/deviceInputEvents";
import { DeviceType } from "babylonjs/DeviceInput/InputDevices/deviceEnums";
import { IDeviceInputSystem } from "babylonjs/DeviceInput/inputInterfaces";
/** @internal */
export class WebDeviceInputSystem implements IDeviceInputSystem {
    private _inputs;
    private _gamepads;
    private _keyboardActive;
    private _pointerActive;
    private _elementToAttachTo;
    private _metaKeys;
    private readonly _engine;
    private readonly _usingSafari;
    private readonly _usingMacOS;
    private _onDeviceConnected;
    private _onDeviceDisconnected;
    private _onInputChanged;
    private _keyboardDownEvent;
    private _keyboardUpEvent;
    private _keyboardBlurEvent;
    private _pointerMoveEvent;
    private _pointerDownEvent;
    private _pointerUpEvent;
    private _pointerCancelEvent;
    private _pointerWheelEvent;
    private _pointerBlurEvent;
    private _wheelEventName;
    private _eventsAttached;
    private _mouseId;
    private readonly _isUsingFirefox;
    private _activeTouchIds;
    private _maxTouchPoints;
    private _pointerInputClearObserver;
    private _gamepadConnectedEvent;
    private _gamepadDisconnectedEvent;
    private _eventPrefix;
    constructor(engine: Engine, onDeviceConnected: (deviceType: DeviceType, deviceSlot: number) => void, onDeviceDisconnected: (deviceType: DeviceType, deviceSlot: number) => void, onInputChanged: (deviceType: DeviceType, deviceSlot: number, eventData: IUIEvent) => void);
    /**
     * Checks for current device input value, given an id and input index. Throws exception if requested device not initialized.
     * @param deviceType Enum specifying device type
     * @param deviceSlot "Slot" or index that device is referenced in
     * @param inputIndex Id of input to be checked
     * @returns Current value of input
     */
    pollInput(deviceType: DeviceType, deviceSlot: number, inputIndex: number): number;
    /**
     * Check for a specific device in the DeviceInputSystem
     * @param deviceType Type of device to check for
     * @returns bool with status of device's existence
     */
    isDeviceAvailable(deviceType: DeviceType): boolean;
    /**
     * Dispose of all the eventlisteners
     */
    dispose(): void;
    /**
     * Enable listening for user input events
     */
    private _enableEvents;
    /**
     * Disable listening for user input events
     */
    private _disableEvents;
    /**
     * Checks for existing connections to devices and register them, if necessary
     * Currently handles gamepads and mouse
     */
    private _checkForConnectedDevices;
    /**
     * Add a gamepad to the DeviceInputSystem
     * @param gamepad A single DOM Gamepad object
     */
    private _addGamePad;
    /**
     * Add pointer device to DeviceInputSystem
     * @param deviceType Type of Pointer to add
     * @param deviceSlot Pointer ID (0 for mouse, pointerId for Touch)
     * @param currentX Current X at point of adding
     * @param currentY Current Y at point of adding
     */
    private _addPointerDevice;
    /**
     * Add device and inputs to device array
     * @param deviceType Enum specifying device type
     * @param deviceSlot "Slot" or index that device is referenced in
     * @param numberOfInputs Number of input entries to create for given device
     */
    private _registerDevice;
    /**
     * Given a specific device name, remove that device from the device map
     * @param deviceType Enum specifying device type
     * @param deviceSlot "Slot" or index that device is referenced in
     */
    private _unregisterDevice;
    /**
     * Handle all actions that come from keyboard interaction
     */
    private _handleKeyActions;
    /**
     * Handle all actions that come from pointer interaction
     */
    private _handlePointerActions;
    /**
     * Handle all actions that come from gamepad interaction
     */
    private _handleGamepadActions;
    /**
     * Update all non-event based devices with each frame
     * @param deviceType Enum specifying device type
     * @param deviceSlot "Slot" or index that device is referenced in
     * @param inputIndex Id of input to be checked
     */
    private _updateDevice;
    /**
     * Gets DeviceType from the device name
     * @param deviceName Name of Device from DeviceInputSystem
     * @returns DeviceType enum value
     */
    private _getGamepadDeviceType;
    /**
     * Get DeviceType from a given pointer/mouse/touch event.
     * @param evt PointerEvent to evaluate
     * @returns DeviceType interpreted from event
     */
    private _getPointerType;
}

}
declare module "babylonjs/Engines/constants" {
/** Defines the cross module used constants to avoid circular dependencies */
export class Constants {
    /** Defines that alpha blending is disabled */
    static readonly ALPHA_DISABLE: number;
    /** Defines that alpha blending is SRC ALPHA * SRC + DEST */
    static readonly ALPHA_ADD: number;
    /** Defines that alpha blending is SRC ALPHA * SRC + (1 - SRC ALPHA) * DEST */
    static readonly ALPHA_COMBINE: number;
    /** Defines that alpha blending is DEST - SRC * DEST */
    static readonly ALPHA_SUBTRACT: number;
    /** Defines that alpha blending is SRC * DEST */
    static readonly ALPHA_MULTIPLY: number;
    /** Defines that alpha blending is SRC ALPHA * SRC + (1 - SRC) * DEST */
    static readonly ALPHA_MAXIMIZED: number;
    /** Defines that alpha blending is SRC + DEST */
    static readonly ALPHA_ONEONE: number;
    /** Defines that alpha blending is SRC + (1 - SRC ALPHA) * DEST */
    static readonly ALPHA_PREMULTIPLIED: number;
    /**
     * Defines that alpha blending is SRC + (1 - SRC ALPHA) * DEST
     * Alpha will be set to (1 - SRC ALPHA) * DEST ALPHA
     */
    static readonly ALPHA_PREMULTIPLIED_PORTERDUFF: number;
    /** Defines that alpha blending is CST * SRC + (1 - CST) * DEST */
    static readonly ALPHA_INTERPOLATE: number;
    /**
     * Defines that alpha blending is SRC + (1 - SRC) * DEST
     * Alpha will be set to SRC ALPHA + (1 - SRC ALPHA) * DEST ALPHA
     */
    static readonly ALPHA_SCREENMODE: number;
    /**
     * Defines that alpha blending is SRC + DST
     * Alpha will be set to SRC ALPHA + DST ALPHA
     */
    static readonly ALPHA_ONEONE_ONEONE: number;
    /**
     * Defines that alpha blending is SRC * DST ALPHA + DST
     * Alpha will be set to 0
     */
    static readonly ALPHA_ALPHATOCOLOR: number;
    /**
     * Defines that alpha blending is SRC * (1 - DST) + DST * (1 - SRC)
     */
    static readonly ALPHA_REVERSEONEMINUS: number;
    /**
     * Defines that alpha blending is SRC + DST * (1 - SRC ALPHA)
     * Alpha will be set to SRC ALPHA + DST ALPHA * (1 - SRC ALPHA)
     */
    static readonly ALPHA_SRC_DSTONEMINUSSRCALPHA: number;
    /**
     * Defines that alpha blending is SRC + DST
     * Alpha will be set to SRC ALPHA
     */
    static readonly ALPHA_ONEONE_ONEZERO: number;
    /**
     * Defines that alpha blending is SRC * (1 - DST) + DST * (1 - SRC)
     * Alpha will be set to DST ALPHA
     */
    static readonly ALPHA_EXCLUSION: number;
    /**
     * Defines that alpha blending is SRC * SRC ALPHA + DST * (1 - SRC ALPHA)
     * Alpha will be set to SRC ALPHA + (1 - SRC ALPHA) * DST ALPHA
     */
    static readonly ALPHA_LAYER_ACCUMULATE: number;
    /** Defines that alpha blending equation a SUM */
    static readonly ALPHA_EQUATION_ADD: number;
    /** Defines that alpha blending equation a SUBSTRACTION */
    static readonly ALPHA_EQUATION_SUBSTRACT: number;
    /** Defines that alpha blending equation a REVERSE SUBSTRACTION */
    static readonly ALPHA_EQUATION_REVERSE_SUBTRACT: number;
    /** Defines that alpha blending equation a MAX operation */
    static readonly ALPHA_EQUATION_MAX: number;
    /** Defines that alpha blending equation a MIN operation */
    static readonly ALPHA_EQUATION_MIN: number;
    /**
     * Defines that alpha blending equation a DARKEN operation:
     * It takes the min of the src and sums the alpha channels.
     */
    static readonly ALPHA_EQUATION_DARKEN: number;
    /** Defines that the resource is not delayed*/
    static readonly DELAYLOADSTATE_NONE: number;
    /** Defines that the resource was successfully delay loaded */
    static readonly DELAYLOADSTATE_LOADED: number;
    /** Defines that the resource is currently delay loading */
    static readonly DELAYLOADSTATE_LOADING: number;
    /** Defines that the resource is delayed and has not started loading */
    static readonly DELAYLOADSTATE_NOTLOADED: number;
    /** Passed to depthFunction or stencilFunction to specify depth or stencil tests will never pass. i.e. Nothing will be drawn */
    static readonly NEVER: number;
    /** Passed to depthFunction or stencilFunction to specify depth or stencil tests will always pass. i.e. Pixels will be drawn in the order they are drawn */
    static readonly ALWAYS: number;
    /** Passed to depthFunction or stencilFunction to specify depth or stencil tests will pass if the new depth value is less than the stored value */
    static readonly LESS: number;
    /** Passed to depthFunction or stencilFunction to specify depth or stencil tests will pass if the new depth value is equals to the stored value */
    static readonly EQUAL: number;
    /** Passed to depthFunction or stencilFunction to specify depth or stencil tests will pass if the new depth value is less than or equal to the stored value */
    static readonly LEQUAL: number;
    /** Passed to depthFunction or stencilFunction to specify depth or stencil tests will pass if the new depth value is greater than the stored value */
    static readonly GREATER: number;
    /** Passed to depthFunction or stencilFunction to specify depth or stencil tests will pass if the new depth value is greater than or equal to the stored value */
    static readonly GEQUAL: number;
    /** Passed to depthFunction or stencilFunction to specify depth or stencil tests will pass if the new depth value is not equal to the stored value */
    static readonly NOTEQUAL: number;
    /** Passed to stencilOperation to specify that stencil value must be kept */
    static readonly KEEP: number;
    /** Passed to stencilOperation to specify that stencil value must be zero */
    static readonly ZERO: number;
    /** Passed to stencilOperation to specify that stencil value must be replaced */
    static readonly REPLACE: number;
    /** Passed to stencilOperation to specify that stencil value must be incremented */
    static readonly INCR: number;
    /** Passed to stencilOperation to specify that stencil value must be decremented */
    static readonly DECR: number;
    /** Passed to stencilOperation to specify that stencil value must be inverted */
    static readonly INVERT: number;
    /** Passed to stencilOperation to specify that stencil value must be incremented with wrapping */
    static readonly INCR_WRAP: number;
    /** Passed to stencilOperation to specify that stencil value must be decremented with wrapping */
    static readonly DECR_WRAP: number;
    /** Texture is not repeating outside of 0..1 UVs */
    static readonly TEXTURE_CLAMP_ADDRESSMODE: number;
    /** Texture is repeating outside of 0..1 UVs */
    static readonly TEXTURE_WRAP_ADDRESSMODE: number;
    /** Texture is repeating and mirrored */
    static readonly TEXTURE_MIRROR_ADDRESSMODE: number;
    /** Flag to create a storage texture */
    static readonly TEXTURE_CREATIONFLAG_STORAGE: number;
    /** ALPHA */
    static readonly TEXTUREFORMAT_ALPHA: number;
    /** LUMINANCE */
    static readonly TEXTUREFORMAT_LUMINANCE: number;
    /** LUMINANCE_ALPHA */
    static readonly TEXTUREFORMAT_LUMINANCE_ALPHA: number;
    /** RGB */
    static readonly TEXTUREFORMAT_RGB: number;
    /** RGBA */
    static readonly TEXTUREFORMAT_RGBA: number;
    /** RED */
    static readonly TEXTUREFORMAT_RED: number;
    /** RED (2nd reference) */
    static readonly TEXTUREFORMAT_R: number;
    /** RG */
    static readonly TEXTUREFORMAT_RG: number;
    /** RED_INTEGER */
    static readonly TEXTUREFORMAT_RED_INTEGER: number;
    /** RED_INTEGER (2nd reference) */
    static readonly TEXTUREFORMAT_R_INTEGER: number;
    /** RG_INTEGER */
    static readonly TEXTUREFORMAT_RG_INTEGER: number;
    /** RGB_INTEGER */
    static readonly TEXTUREFORMAT_RGB_INTEGER: number;
    /** RGBA_INTEGER */
    static readonly TEXTUREFORMAT_RGBA_INTEGER: number;
    /** BGRA */
    static readonly TEXTUREFORMAT_BGRA: number;
    /** Depth 24 bits + Stencil 8 bits */
    static readonly TEXTUREFORMAT_DEPTH24_STENCIL8: number;
    /** Depth 32 bits float */
    static readonly TEXTUREFORMAT_DEPTH32_FLOAT: number;
    /** Depth 16 bits */
    static readonly TEXTUREFORMAT_DEPTH16: number;
    /** Depth 24 bits */
    static readonly TEXTUREFORMAT_DEPTH24: number;
    /** Depth 24 bits unorm + Stencil 8 bits */
    static readonly TEXTUREFORMAT_DEPTH24UNORM_STENCIL8: number;
    /** Depth 32 bits float + Stencil 8 bits */
    static readonly TEXTUREFORMAT_DEPTH32FLOAT_STENCIL8: number;
    /** Compressed BC7 */
    static readonly TEXTUREFORMAT_COMPRESSED_RGBA_BPTC_UNORM: number;
    /** Compressed BC7 (SRGB) */
    static readonly TEXTUREFORMAT_COMPRESSED_SRGB_ALPHA_BPTC_UNORM: number;
    /** Compressed BC6 unsigned float */
    static readonly TEXTUREFORMAT_COMPRESSED_RGB_BPTC_UNSIGNED_FLOAT: number;
    /** Compressed BC6 signed float */
    static readonly TEXTUREFORMAT_COMPRESSED_RGB_BPTC_SIGNED_FLOAT: number;
    /** Compressed BC3 */
    static readonly TEXTUREFORMAT_COMPRESSED_RGBA_S3TC_DXT5: number;
    /** Compressed BC3 (SRGB) */
    static readonly TEXTUREFORMAT_COMPRESSED_SRGB_ALPHA_S3TC_DXT5_EXT: number;
    /** Compressed BC2 */
    static readonly TEXTUREFORMAT_COMPRESSED_RGBA_S3TC_DXT3: number;
    /** Compressed BC2 (SRGB) */
    static readonly TEXTUREFORMAT_COMPRESSED_SRGB_ALPHA_S3TC_DXT3_EXT: number;
    /** Compressed BC1 (RGBA) */
    static readonly TEXTUREFORMAT_COMPRESSED_RGBA_S3TC_DXT1: number;
    /** Compressed BC1 (RGB) */
    static readonly TEXTUREFORMAT_COMPRESSED_RGB_S3TC_DXT1: number;
    /** Compressed BC1 (SRGB+A) */
    static readonly TEXTUREFORMAT_COMPRESSED_SRGB_ALPHA_S3TC_DXT1_EXT: number;
    /** Compressed BC1 (SRGB) */
    static readonly TEXTUREFORMAT_COMPRESSED_SRGB_S3TC_DXT1_EXT: number;
    /** Compressed ASTC 4x4 */
    static readonly TEXTUREFORMAT_COMPRESSED_RGBA_ASTC_4x4: number;
    /** Compressed ASTC 4x4 (SRGB) */
    static readonly TEXTUREFORMAT_COMPRESSED_SRGB8_ALPHA8_ASTC_4x4_KHR: number;
    /** Compressed ETC1 (RGB) */
    static readonly TEXTUREFORMAT_COMPRESSED_RGB_ETC1_WEBGL: number;
    /** Compressed ETC2 (RGB) */
    static readonly TEXTUREFORMAT_COMPRESSED_RGB8_ETC2: number;
    /** Compressed ETC2 (SRGB) */
    static readonly TEXTUREFORMAT_COMPRESSED_SRGB8_ETC2: number;
    /** Compressed ETC2 (RGB+A1) */
    static readonly TEXTUREFORMAT_COMPRESSED_RGB8_PUNCHTHROUGH_ALPHA1_ETC2: number;
    /** Compressed ETC2 (SRGB+A1)*/
    static readonly TEXTUREFORMAT_COMPRESSED_SRGB8_PUNCHTHROUGH_ALPHA1_ETC2: number;
    /** Compressed ETC2 (RGB+A) */
    static readonly TEXTUREFORMAT_COMPRESSED_RGBA8_ETC2_EAC: number;
    /** Compressed ETC2 (SRGB+1) */
    static readonly TEXTUREFORMAT_COMPRESSED_SRGB8_ALPHA8_ETC2_EAC: number;
    /** UNSIGNED_BYTE */
    static readonly TEXTURETYPE_UNSIGNED_BYTE: number;
    /** UNSIGNED_BYTE (2nd reference) */
    static readonly TEXTURETYPE_UNSIGNED_INT: number;
    /** FLOAT */
    static readonly TEXTURETYPE_FLOAT: number;
    /** HALF_FLOAT */
    static readonly TEXTURETYPE_HALF_FLOAT: number;
    /** BYTE */
    static readonly TEXTURETYPE_BYTE: number;
    /** SHORT */
    static readonly TEXTURETYPE_SHORT: number;
    /** UNSIGNED_SHORT */
    static readonly TEXTURETYPE_UNSIGNED_SHORT: number;
    /** INT */
    static readonly TEXTURETYPE_INT: number;
    /** UNSIGNED_INT */
    static readonly TEXTURETYPE_UNSIGNED_INTEGER: number;
    /** UNSIGNED_SHORT_4_4_4_4 */
    static readonly TEXTURETYPE_UNSIGNED_SHORT_4_4_4_4: number;
    /** UNSIGNED_SHORT_5_5_5_1 */
    static readonly TEXTURETYPE_UNSIGNED_SHORT_5_5_5_1: number;
    /** UNSIGNED_SHORT_5_6_5 */
    static readonly TEXTURETYPE_UNSIGNED_SHORT_5_6_5: number;
    /** UNSIGNED_INT_2_10_10_10_REV */
    static readonly TEXTURETYPE_UNSIGNED_INT_2_10_10_10_REV: number;
    /** UNSIGNED_INT_24_8 */
    static readonly TEXTURETYPE_UNSIGNED_INT_24_8: number;
    /** UNSIGNED_INT_10F_11F_11F_REV */
    static readonly TEXTURETYPE_UNSIGNED_INT_10F_11F_11F_REV: number;
    /** UNSIGNED_INT_5_9_9_9_REV */
    static readonly TEXTURETYPE_UNSIGNED_INT_5_9_9_9_REV: number;
    /** FLOAT_32_UNSIGNED_INT_24_8_REV */
    static readonly TEXTURETYPE_FLOAT_32_UNSIGNED_INT_24_8_REV: number;
    /** UNDEFINED */
    static readonly TEXTURETYPE_UNDEFINED: number;
    /** nearest is mag = nearest and min = nearest and no mip */
    static readonly TEXTURE_NEAREST_SAMPLINGMODE: number;
    /** mag = nearest and min = nearest and mip = none */
    static readonly TEXTURE_NEAREST_NEAREST: number;
    /** Bilinear is mag = linear and min = linear and no mip */
    static readonly TEXTURE_BILINEAR_SAMPLINGMODE: number;
    /** mag = linear and min = linear and mip = none */
    static readonly TEXTURE_LINEAR_LINEAR: number;
    /** Trilinear is mag = linear and min = linear and mip = linear */
    static readonly TEXTURE_TRILINEAR_SAMPLINGMODE: number;
    /** Trilinear is mag = linear and min = linear and mip = linear */
    static readonly TEXTURE_LINEAR_LINEAR_MIPLINEAR: number;
    /** mag = nearest and min = nearest and mip = nearest */
    static readonly TEXTURE_NEAREST_NEAREST_MIPNEAREST: number;
    /** mag = nearest and min = linear and mip = nearest */
    static readonly TEXTURE_NEAREST_LINEAR_MIPNEAREST: number;
    /** mag = nearest and min = linear and mip = linear */
    static readonly TEXTURE_NEAREST_LINEAR_MIPLINEAR: number;
    /** mag = nearest and min = linear and mip = none */
    static readonly TEXTURE_NEAREST_LINEAR: number;
    /** nearest is mag = nearest and min = nearest and mip = linear */
    static readonly TEXTURE_NEAREST_NEAREST_MIPLINEAR: number;
    /** mag = linear and min = nearest and mip = nearest */
    static readonly TEXTURE_LINEAR_NEAREST_MIPNEAREST: number;
    /** mag = linear and min = nearest and mip = linear */
    static readonly TEXTURE_LINEAR_NEAREST_MIPLINEAR: number;
    /** Bilinear is mag = linear and min = linear and mip = nearest */
    static readonly TEXTURE_LINEAR_LINEAR_MIPNEAREST: number;
    /** mag = linear and min = nearest and mip = none */
    static readonly TEXTURE_LINEAR_NEAREST: number;
    /** Explicit coordinates mode */
    static readonly TEXTURE_EXPLICIT_MODE: number;
    /** Spherical coordinates mode */
    static readonly TEXTURE_SPHERICAL_MODE: number;
    /** Planar coordinates mode */
    static readonly TEXTURE_PLANAR_MODE: number;
    /** Cubic coordinates mode */
    static readonly TEXTURE_CUBIC_MODE: number;
    /** Projection coordinates mode */
    static readonly TEXTURE_PROJECTION_MODE: number;
    /** Skybox coordinates mode */
    static readonly TEXTURE_SKYBOX_MODE: number;
    /** Inverse Cubic coordinates mode */
    static readonly TEXTURE_INVCUBIC_MODE: number;
    /** Equirectangular coordinates mode */
    static readonly TEXTURE_EQUIRECTANGULAR_MODE: number;
    /** Equirectangular Fixed coordinates mode */
    static readonly TEXTURE_FIXED_EQUIRECTANGULAR_MODE: number;
    /** Equirectangular Fixed Mirrored coordinates mode */
    static readonly TEXTURE_FIXED_EQUIRECTANGULAR_MIRRORED_MODE: number;
    /** Offline (baking) quality for texture filtering */
    static readonly TEXTURE_FILTERING_QUALITY_OFFLINE: number;
    /** High quality for texture filtering */
    static readonly TEXTURE_FILTERING_QUALITY_HIGH: number;
    /** Medium quality for texture filtering */
    static readonly TEXTURE_FILTERING_QUALITY_MEDIUM: number;
    /** Low quality for texture filtering */
    static readonly TEXTURE_FILTERING_QUALITY_LOW: number;
    /** Defines that texture rescaling will use a floor to find the closer power of 2 size */
    static readonly SCALEMODE_FLOOR: number;
    /** Defines that texture rescaling will look for the nearest power of 2 size */
    static readonly SCALEMODE_NEAREST: number;
    /** Defines that texture rescaling will use a ceil to find the closer power of 2 size */
    static readonly SCALEMODE_CEILING: number;
    /**
     * The dirty texture flag value
     */
    static readonly MATERIAL_TextureDirtyFlag: number;
    /**
     * The dirty light flag value
     */
    static readonly MATERIAL_LightDirtyFlag: number;
    /**
     * The dirty fresnel flag value
     */
    static readonly MATERIAL_FresnelDirtyFlag: number;
    /**
     * The dirty attribute flag value
     */
    static readonly MATERIAL_AttributesDirtyFlag: number;
    /**
     * The dirty misc flag value
     */
    static readonly MATERIAL_MiscDirtyFlag: number;
    /**
     * The dirty prepass flag value
     */
    static readonly MATERIAL_PrePassDirtyFlag: number;
    /**
     * The all dirty flag value
     */
    static readonly MATERIAL_AllDirtyFlag: number;
    /**
     * Returns the triangle fill mode
     */
    static readonly MATERIAL_TriangleFillMode: number;
    /**
     * Returns the wireframe mode
     */
    static readonly MATERIAL_WireFrameFillMode: number;
    /**
     * Returns the point fill mode
     */
    static readonly MATERIAL_PointFillMode: number;
    /**
     * Returns the point list draw mode
     */
    static readonly MATERIAL_PointListDrawMode: number;
    /**
     * Returns the line list draw mode
     */
    static readonly MATERIAL_LineListDrawMode: number;
    /**
     * Returns the line loop draw mode
     */
    static readonly MATERIAL_LineLoopDrawMode: number;
    /**
     * Returns the line strip draw mode
     */
    static readonly MATERIAL_LineStripDrawMode: number;
    /**
     * Returns the triangle strip draw mode
     */
    static readonly MATERIAL_TriangleStripDrawMode: number;
    /**
     * Returns the triangle fan draw mode
     */
    static readonly MATERIAL_TriangleFanDrawMode: number;
    /**
     * Stores the clock-wise side orientation
     */
    static readonly MATERIAL_ClockWiseSideOrientation: number;
    /**
     * Stores the counter clock-wise side orientation
     */
    static readonly MATERIAL_CounterClockWiseSideOrientation: number;
    /**
     * Nothing
     * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
     */
    static readonly ACTION_NothingTrigger: number;
    /**
     * On pick
     * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
     */
    static readonly ACTION_OnPickTrigger: number;
    /**
     * On left pick
     * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
     */
    static readonly ACTION_OnLeftPickTrigger: number;
    /**
     * On right pick
     * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
     */
    static readonly ACTION_OnRightPickTrigger: number;
    /**
     * On center pick
     * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
     */
    static readonly ACTION_OnCenterPickTrigger: number;
    /**
     * On pick down
     * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
     */
    static readonly ACTION_OnPickDownTrigger: number;
    /**
     * On double pick
     * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
     */
    static readonly ACTION_OnDoublePickTrigger: number;
    /**
     * On pick up
     * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
     */
    static readonly ACTION_OnPickUpTrigger: number;
    /**
     * On pick out.
     * This trigger will only be raised if you also declared a OnPickDown
     * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
     */
    static readonly ACTION_OnPickOutTrigger: number;
    /**
     * On long press
     * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
     */
    static readonly ACTION_OnLongPressTrigger: number;
    /**
     * On pointer over
     * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
     */
    static readonly ACTION_OnPointerOverTrigger: number;
    /**
     * On pointer out
     * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
     */
    static readonly ACTION_OnPointerOutTrigger: number;
    /**
     * On every frame
     * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
     */
    static readonly ACTION_OnEveryFrameTrigger: number;
    /**
     * On intersection enter
     * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
     */
    static readonly ACTION_OnIntersectionEnterTrigger: number;
    /**
     * On intersection exit
     * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
     */
    static readonly ACTION_OnIntersectionExitTrigger: number;
    /**
     * On key down
     * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
     */
    static readonly ACTION_OnKeyDownTrigger: number;
    /**
     * On key up
     * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
     */
    static readonly ACTION_OnKeyUpTrigger: number;
    /**
     * Billboard mode will only apply to Y axis
     */
    static readonly PARTICLES_BILLBOARDMODE_Y: number;
    /**
     * Billboard mode will apply to all axes
     */
    static readonly PARTICLES_BILLBOARDMODE_ALL: number;
    /**
     * Special billboard mode where the particle will be biilboard to the camera but rotated to align with direction
     */
    static readonly PARTICLES_BILLBOARDMODE_STRETCHED: number;
    /**
     * Special billboard mode where the particle will be billboard to the camera but only around the axis of the direction of particle emission
     */
    static readonly PARTICLES_BILLBOARDMODE_STRETCHED_LOCAL: number;
    /** Default culling strategy : this is an exclusion test and it's the more accurate.
     *  Test order :
     *  Is the bounding sphere outside the frustum ?
     *  If not, are the bounding box vertices outside the frustum ?
     *  It not, then the cullable object is in the frustum.
     */
    static readonly MESHES_CULLINGSTRATEGY_STANDARD: number;
    /** Culling strategy : Bounding Sphere Only.
     *  This is an exclusion test. It's faster than the standard strategy because the bounding box is not tested.
     *  It's also less accurate than the standard because some not visible objects can still be selected.
     *  Test : is the bounding sphere outside the frustum ?
     *  If not, then the cullable object is in the frustum.
     */
    static readonly MESHES_CULLINGSTRATEGY_BOUNDINGSPHERE_ONLY: number;
    /** Culling strategy : Optimistic Inclusion.
     *  This in an inclusion test first, then the standard exclusion test.
     *  This can be faster when a cullable object is expected to be almost always in the camera frustum.
     *  This could also be a little slower than the standard test when the tested object center is not the frustum but one of its bounding box vertex is still inside.
     *  Anyway, it's as accurate as the standard strategy.
     *  Test :
     *  Is the cullable object bounding sphere center in the frustum ?
     *  If not, apply the default culling strategy.
     */
    static readonly MESHES_CULLINGSTRATEGY_OPTIMISTIC_INCLUSION: number;
    /** Culling strategy : Optimistic Inclusion then Bounding Sphere Only.
     *  This in an inclusion test first, then the bounding sphere only exclusion test.
     *  This can be the fastest test when a cullable object is expected to be almost always in the camera frustum.
     *  This could also be a little slower than the BoundingSphereOnly strategy when the tested object center is not in the frustum but its bounding sphere still intersects it.
     *  It's less accurate than the standard strategy and as accurate as the BoundingSphereOnly strategy.
     *  Test :
     *  Is the cullable object bounding sphere center in the frustum ?
     *  If not, apply the Bounding Sphere Only strategy. No Bounding Box is tested here.
     */
    static readonly MESHES_CULLINGSTRATEGY_OPTIMISTIC_INCLUSION_THEN_BSPHERE_ONLY: number;
    /**
     * No logging while loading
     */
    static readonly SCENELOADER_NO_LOGGING: number;
    /**
     * Minimal logging while loading
     */
    static readonly SCENELOADER_MINIMAL_LOGGING: number;
    /**
     * Summary logging while loading
     */
    static readonly SCENELOADER_SUMMARY_LOGGING: number;
    /**
     * Detailed logging while loading
     */
    static readonly SCENELOADER_DETAILED_LOGGING: number;
    /**
     * Constant used to retrieve the irradiance texture index in the textures array in the prepass
     * using getIndex(Constants.PREPASS_IRRADIANCE_TEXTURE_TYPE)
     */
    static readonly PREPASS_IRRADIANCE_TEXTURE_TYPE: number;
    /**
     * Constant used to retrieve the position texture index in the textures array in the prepass
     * using getIndex(Constants.PREPASS_POSITION_TEXTURE_INDEX)
     */
    static readonly PREPASS_POSITION_TEXTURE_TYPE: number;
    /**
     * Constant used to retrieve the velocity texture index in the textures array in the prepass
     * using getIndex(Constants.PREPASS_VELOCITY_TEXTURE_INDEX)
     */
    static readonly PREPASS_VELOCITY_TEXTURE_TYPE: number;
    /**
     * Constant used to retrieve the reflectivity texture index in the textures array in the prepass
     * using the getIndex(Constants.PREPASS_REFLECTIVITY_TEXTURE_TYPE)
     */
    static readonly PREPASS_REFLECTIVITY_TEXTURE_TYPE: number;
    /**
     * Constant used to retrieve the lit color texture index in the textures array in the prepass
     * using the getIndex(Constants.PREPASS_COLOR_TEXTURE_TYPE)
     */
    static readonly PREPASS_COLOR_TEXTURE_TYPE: number;
    /**
     * Constant used to retrieve depth index in the textures array in the prepass
     * using the getIndex(Constants.PREPASS_DEPTH_TEXTURE_TYPE)
     */
    static readonly PREPASS_DEPTH_TEXTURE_TYPE: number;
    /**
     * Constant used to retrieve normal index in the textures array in the prepass
     * using the getIndex(Constants.PREPASS_NORMAL_TEXTURE_TYPE)
     */
    static readonly PREPASS_NORMAL_TEXTURE_TYPE: number;
    /**
     * Constant used to retrieve albedo index in the textures array in the prepass
     * using the getIndex(Constants.PREPASS_ALBEDO_SQRT_TEXTURE_TYPE)
     */
    static readonly PREPASS_ALBEDO_SQRT_TEXTURE_TYPE: number;
    /** Flag to create a readable buffer (the buffer can be the source of a copy) */
    static readonly BUFFER_CREATIONFLAG_READ: number;
    /** Flag to create a writable buffer (the buffer can be the destination of a copy) */
    static readonly BUFFER_CREATIONFLAG_WRITE: number;
    /** Flag to create a readable and writable buffer */
    static readonly BUFFER_CREATIONFLAG_READWRITE: number;
    /** Flag to create a buffer suitable to be used as a uniform buffer */
    static readonly BUFFER_CREATIONFLAG_UNIFORM: number;
    /** Flag to create a buffer suitable to be used as a vertex buffer */
    static readonly BUFFER_CREATIONFLAG_VERTEX: number;
    /** Flag to create a buffer suitable to be used as an index buffer */
    static readonly BUFFER_CREATIONFLAG_INDEX: number;
    /** Flag to create a buffer suitable to be used as a storage buffer */
    static readonly BUFFER_CREATIONFLAG_STORAGE: number;
    /**
     * Prefixes used by the engine for sub mesh draw wrappers
     */
    /** @internal */
    static readonly RENDERPASS_MAIN: number;
    /**
     * Constant used as key code for Alt key
     */
    static readonly INPUT_ALT_KEY: number;
    /**
     * Constant used as key code for Ctrl key
     */
    static readonly INPUT_CTRL_KEY: number;
    /**
     * Constant used as key code for Meta key (Left Win, Left Cmd)
     */
    static readonly INPUT_META_KEY1: number;
    /**
     * Constant used as key code for Meta key (Right Win)
     */
    static readonly INPUT_META_KEY2: number;
    /**
     * Constant used as key code for Meta key (Right Win, Right Cmd)
     */
    static readonly INPUT_META_KEY3: number;
    /**
     * Constant used as key code for Shift key
     */
    static readonly INPUT_SHIFT_KEY: number;
    /** Standard snapshot rendering. In this mode, some form of dynamic behavior is possible (for eg, uniform buffers are still updated) */
    static readonly SNAPSHOTRENDERING_STANDARD: number;
    /** Fast snapshot rendering. In this mode, everything is static and only some limited form of dynamic behaviour is possible */
    static readonly SNAPSHOTRENDERING_FAST: number;
    /**
     * This is the default projection mode used by the cameras.
     * It helps recreating a feeling of perspective and better appreciate depth.
     * This is the best way to simulate real life cameras.
     */
    static readonly PERSPECTIVE_CAMERA: number;
    /**
     * This helps creating camera with an orthographic mode.
     * Orthographic is commonly used in engineering as a means to produce object specifications that communicate dimensions unambiguously, each line of 1 unit length (cm, meter..whatever) will appear to have the same length everywhere on the drawing. This allows the drafter to dimension only a subset of lines and let the reader know that other lines of that length on the drawing are also that length in reality. Every parallel line in the drawing is also parallel in the object.
     */
    static readonly ORTHOGRAPHIC_CAMERA: number;
    /**
     * This is the default FOV mode for perspective cameras.
     * This setting aligns the upper and lower bounds of the viewport to the upper and lower bounds of the camera frustum.
     */
    static readonly FOVMODE_VERTICAL_FIXED: number;
    /**
     * This setting aligns the left and right bounds of the viewport to the left and right bounds of the camera frustum.
     */
    static readonly FOVMODE_HORIZONTAL_FIXED: number;
    /**
     * This specifies there is no need for a camera rig.
     * Basically only one eye is rendered corresponding to the camera.
     */
    static readonly RIG_MODE_NONE: number;
    /**
     * Simulates a camera Rig with one blue eye and one red eye.
     * This can be use with 3d blue and red glasses.
     */
    static readonly RIG_MODE_STEREOSCOPIC_ANAGLYPH: number;
    /**
     * Defines that both eyes of the camera will be rendered side by side with a parallel target.
     */
    static readonly RIG_MODE_STEREOSCOPIC_SIDEBYSIDE_PARALLEL: number;
    /**
     * Defines that both eyes of the camera will be rendered side by side with a none parallel target.
     */
    static readonly RIG_MODE_STEREOSCOPIC_SIDEBYSIDE_CROSSEYED: number;
    /**
     * Defines that both eyes of the camera will be rendered over under each other.
     */
    static readonly RIG_MODE_STEREOSCOPIC_OVERUNDER: number;
    /**
     * Defines that both eyes of the camera will be rendered on successive lines interlaced for passive 3d monitors.
     */
    static readonly RIG_MODE_STEREOSCOPIC_INTERLACED: number;
    /**
     * Defines that both eyes of the camera should be renderered in a VR mode (carbox).
     */
    static readonly RIG_MODE_VR: number;
    /**
     * Defines that both eyes of the camera should be renderered in a VR mode (webVR).
     */
    static readonly RIG_MODE_WEBVR: number;
    /**
     * Custom rig mode allowing rig cameras to be populated manually with any number of cameras
     */
    static readonly RIG_MODE_CUSTOM: number;
    /**
     * Maximum number of uv sets supported
     */
    static readonly MAX_SUPPORTED_UV_SETS: number;
    /**
     * GL constants
     */
    /** Alpha blend equation: ADD */
    static readonly GL_ALPHA_EQUATION_ADD: number;
    /** Alpha equation: MIN */
    static readonly GL_ALPHA_EQUATION_MIN: number;
    /** Alpha equation: MAX */
    static readonly GL_ALPHA_EQUATION_MAX: number;
    /** Alpha equation: SUBTRACT */
    static readonly GL_ALPHA_EQUATION_SUBTRACT: number;
    /** Alpha equation: REVERSE_SUBTRACT */
    static readonly GL_ALPHA_EQUATION_REVERSE_SUBTRACT: number;
    /** Alpha blend function: SRC */
    static readonly GL_ALPHA_FUNCTION_SRC: number;
    /** Alpha blend function: ONE_MINUS_SRC */
    static readonly GL_ALPHA_FUNCTION_ONE_MINUS_SRC_COLOR: number;
    /** Alpha blend function: SRC_ALPHA */
    static readonly GL_ALPHA_FUNCTION_SRC_ALPHA: number;
    /** Alpha blend function: ONE_MINUS_SRC_ALPHA */
    static readonly GL_ALPHA_FUNCTION_ONE_MINUS_SRC_ALPHA: number;
    /** Alpha blend function: DST_ALPHA */
    static readonly GL_ALPHA_FUNCTION_DST_ALPHA: number;
    /** Alpha blend function: ONE_MINUS_DST_ALPHA */
    static readonly GL_ALPHA_FUNCTION_ONE_MINUS_DST_ALPHA: number;
    /** Alpha blend function: ONE_MINUS_DST */
    static readonly GL_ALPHA_FUNCTION_DST_COLOR: number;
    /** Alpha blend function: ONE_MINUS_DST */
    static readonly GL_ALPHA_FUNCTION_ONE_MINUS_DST_COLOR: number;
    /** Alpha blend function: SRC_ALPHA_SATURATED */
    static readonly GL_ALPHA_FUNCTION_SRC_ALPHA_SATURATED: number;
    /** Alpha blend function: CONSTANT */
    static readonly GL_ALPHA_FUNCTION_CONSTANT_COLOR: number;
    /** Alpha blend function: ONE_MINUS_CONSTANT */
    static readonly GL_ALPHA_FUNCTION_ONE_MINUS_CONSTANT_COLOR: number;
    /** Alpha blend function: CONSTANT_ALPHA */
    static readonly GL_ALPHA_FUNCTION_CONSTANT_ALPHA: number;
    /** Alpha blend function: ONE_MINUS_CONSTANT_ALPHA */
    static readonly GL_ALPHA_FUNCTION_ONE_MINUS_CONSTANT_ALPHA: number;
    /** URL to the snippet server. Points to the public snippet server by default */
    static SnippetUrl: string;
}

}
declare module "babylonjs/Engines/engine" {
import { Observable } from "babylonjs/Misc/observable";
import { Nullable } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
import { InternalTexture } from "babylonjs/Materials/Textures/internalTexture";
import { IOfflineProvider } from "babylonjs/Offline/IOfflineProvider";
import { ILoadingScreen } from "babylonjs/Loading/loadingScreen";
import { WebGLPipelineContext } from "babylonjs/Engines/WebGL/webGLPipelineContext";
import { IPipelineContext } from "babylonjs/Engines/IPipelineContext";
import { ICustomAnimationFrameRequester } from "babylonjs/Misc/customAnimationFrameRequester";
import { EngineOptions } from "babylonjs/Engines/thinEngine";
import { ThinEngine } from "babylonjs/Engines/thinEngine";
import { IViewportLike, IColor4Like } from "babylonjs/Maths/math.like";
import { RenderTargetTexture } from "babylonjs/Materials/Textures/renderTargetTexture";
import { PerformanceMonitor } from "babylonjs/Misc/performanceMonitor";
import { DataBuffer } from "babylonjs/Buffers/dataBuffer";
import { PerfCounter } from "babylonjs/Misc/perfCounter";
import { RenderTargetWrapper } from "babylonjs/Engines/renderTargetWrapper";
import "babylonjs/Engines/Extensions/engine.alpha";
import "babylonjs/Engines/Extensions/engine.readTexture";
import "babylonjs/Engines/Extensions/engine.dynamicBuffer";
import { IAudioEngine } from "babylonjs/Audio/Interfaces/IAudioEngine";
import { Material } from "babylonjs/Materials/material";
import { PostProcess } from "babylonjs/PostProcesses/postProcess";
/**
 * Defines the interface used by display changed events
 */
export interface IDisplayChangedEventArgs {
    /** Gets the vrDisplay object (if any) */
    vrDisplay: Nullable<any>;
    /** Gets a boolean indicating if webVR is supported */
    vrSupported: boolean;
}
/**
 * Defines the interface used by objects containing a viewport (like a camera)
 */
interface IViewportOwnerLike {
    /**
     * Gets or sets the viewport
     */
    viewport: IViewportLike;
}
/**
 * The engine class is responsible for interfacing with all lower-level APIs such as WebGL and Audio
 */
export class Engine extends ThinEngine {
    /** Defines that alpha blending is disabled */
    static readonly ALPHA_DISABLE: number;
    /** Defines that alpha blending to SRC ALPHA * SRC + DEST */
    static readonly ALPHA_ADD: number;
    /** Defines that alpha blending to SRC ALPHA * SRC + (1 - SRC ALPHA) * DEST */
    static readonly ALPHA_COMBINE: number;
    /** Defines that alpha blending to DEST - SRC * DEST */
    static readonly ALPHA_SUBTRACT: number;
    /** Defines that alpha blending to SRC * DEST */
    static readonly ALPHA_MULTIPLY: number;
    /** Defines that alpha blending to SRC ALPHA * SRC + (1 - SRC) * DEST */
    static readonly ALPHA_MAXIMIZED: number;
    /** Defines that alpha blending to SRC + DEST */
    static readonly ALPHA_ONEONE: number;
    /** Defines that alpha blending to SRC + (1 - SRC ALPHA) * DEST */
    static readonly ALPHA_PREMULTIPLIED: number;
    /**
     * Defines that alpha blending to SRC + (1 - SRC ALPHA) * DEST
     * Alpha will be set to (1 - SRC ALPHA) * DEST ALPHA
     */
    static readonly ALPHA_PREMULTIPLIED_PORTERDUFF: number;
    /** Defines that alpha blending to CST * SRC + (1 - CST) * DEST */
    static readonly ALPHA_INTERPOLATE: number;
    /**
     * Defines that alpha blending to SRC + (1 - SRC) * DEST
     * Alpha will be set to SRC ALPHA + (1 - SRC ALPHA) * DEST ALPHA
     */
    static readonly ALPHA_SCREENMODE: number;
    /** Defines that the resource is not delayed*/
    static readonly DELAYLOADSTATE_NONE: number;
    /** Defines that the resource was successfully delay loaded */
    static readonly DELAYLOADSTATE_LOADED: number;
    /** Defines that the resource is currently delay loading */
    static readonly DELAYLOADSTATE_LOADING: number;
    /** Defines that the resource is delayed and has not started loading */
    static readonly DELAYLOADSTATE_NOTLOADED: number;
    /** Passed to depthFunction or stencilFunction to specify depth or stencil tests will never pass. i.e. Nothing will be drawn */
    static readonly NEVER: number;
    /** Passed to depthFunction or stencilFunction to specify depth or stencil tests will always pass. i.e. Pixels will be drawn in the order they are drawn */
    static readonly ALWAYS: number;
    /** Passed to depthFunction or stencilFunction to specify depth or stencil tests will pass if the new depth value is less than the stored value */
    static readonly LESS: number;
    /** Passed to depthFunction or stencilFunction to specify depth or stencil tests will pass if the new depth value is equals to the stored value */
    static readonly EQUAL: number;
    /** Passed to depthFunction or stencilFunction to specify depth or stencil tests will pass if the new depth value is less than or equal to the stored value */
    static readonly LEQUAL: number;
    /** Passed to depthFunction or stencilFunction to specify depth or stencil tests will pass if the new depth value is greater than the stored value */
    static readonly GREATER: number;
    /** Passed to depthFunction or stencilFunction to specify depth or stencil tests will pass if the new depth value is greater than or equal to the stored value */
    static readonly GEQUAL: number;
    /** Passed to depthFunction or stencilFunction to specify depth or stencil tests will pass if the new depth value is not equal to the stored value */
    static readonly NOTEQUAL: number;
    /** Passed to stencilOperation to specify that stencil value must be kept */
    static readonly KEEP: number;
    /** Passed to stencilOperation to specify that stencil value must be replaced */
    static readonly REPLACE: number;
    /** Passed to stencilOperation to specify that stencil value must be incremented */
    static readonly INCR: number;
    /** Passed to stencilOperation to specify that stencil value must be decremented */
    static readonly DECR: number;
    /** Passed to stencilOperation to specify that stencil value must be inverted */
    static readonly INVERT: number;
    /** Passed to stencilOperation to specify that stencil value must be incremented with wrapping */
    static readonly INCR_WRAP: number;
    /** Passed to stencilOperation to specify that stencil value must be decremented with wrapping */
    static readonly DECR_WRAP: number;
    /** Texture is not repeating outside of 0..1 UVs */
    static readonly TEXTURE_CLAMP_ADDRESSMODE: number;
    /** Texture is repeating outside of 0..1 UVs */
    static readonly TEXTURE_WRAP_ADDRESSMODE: number;
    /** Texture is repeating and mirrored */
    static readonly TEXTURE_MIRROR_ADDRESSMODE: number;
    /** ALPHA */
    static readonly TEXTUREFORMAT_ALPHA: number;
    /** LUMINANCE */
    static readonly TEXTUREFORMAT_LUMINANCE: number;
    /** LUMINANCE_ALPHA */
    static readonly TEXTUREFORMAT_LUMINANCE_ALPHA: number;
    /** RGB */
    static readonly TEXTUREFORMAT_RGB: number;
    /** RGBA */
    static readonly TEXTUREFORMAT_RGBA: number;
    /** RED */
    static readonly TEXTUREFORMAT_RED: number;
    /** RED (2nd reference) */
    static readonly TEXTUREFORMAT_R: number;
    /** RG */
    static readonly TEXTUREFORMAT_RG: number;
    /** RED_INTEGER */
    static readonly TEXTUREFORMAT_RED_INTEGER: number;
    /** RED_INTEGER (2nd reference) */
    static readonly TEXTUREFORMAT_R_INTEGER: number;
    /** RG_INTEGER */
    static readonly TEXTUREFORMAT_RG_INTEGER: number;
    /** RGB_INTEGER */
    static readonly TEXTUREFORMAT_RGB_INTEGER: number;
    /** RGBA_INTEGER */
    static readonly TEXTUREFORMAT_RGBA_INTEGER: number;
    /** UNSIGNED_BYTE */
    static readonly TEXTURETYPE_UNSIGNED_BYTE: number;
    /** UNSIGNED_BYTE (2nd reference) */
    static readonly TEXTURETYPE_UNSIGNED_INT: number;
    /** FLOAT */
    static readonly TEXTURETYPE_FLOAT: number;
    /** HALF_FLOAT */
    static readonly TEXTURETYPE_HALF_FLOAT: number;
    /** BYTE */
    static readonly TEXTURETYPE_BYTE: number;
    /** SHORT */
    static readonly TEXTURETYPE_SHORT: number;
    /** UNSIGNED_SHORT */
    static readonly TEXTURETYPE_UNSIGNED_SHORT: number;
    /** INT */
    static readonly TEXTURETYPE_INT: number;
    /** UNSIGNED_INT */
    static readonly TEXTURETYPE_UNSIGNED_INTEGER: number;
    /** UNSIGNED_SHORT_4_4_4_4 */
    static readonly TEXTURETYPE_UNSIGNED_SHORT_4_4_4_4: number;
    /** UNSIGNED_SHORT_5_5_5_1 */
    static readonly TEXTURETYPE_UNSIGNED_SHORT_5_5_5_1: number;
    /** UNSIGNED_SHORT_5_6_5 */
    static readonly TEXTURETYPE_UNSIGNED_SHORT_5_6_5: number;
    /** UNSIGNED_INT_2_10_10_10_REV */
    static readonly TEXTURETYPE_UNSIGNED_INT_2_10_10_10_REV: number;
    /** UNSIGNED_INT_24_8 */
    static readonly TEXTURETYPE_UNSIGNED_INT_24_8: number;
    /** UNSIGNED_INT_10F_11F_11F_REV */
    static readonly TEXTURETYPE_UNSIGNED_INT_10F_11F_11F_REV: number;
    /** UNSIGNED_INT_5_9_9_9_REV */
    static readonly TEXTURETYPE_UNSIGNED_INT_5_9_9_9_REV: number;
    /** FLOAT_32_UNSIGNED_INT_24_8_REV */
    static readonly TEXTURETYPE_FLOAT_32_UNSIGNED_INT_24_8_REV: number;
    /** nearest is mag = nearest and min = nearest and mip = none */
    static readonly TEXTURE_NEAREST_SAMPLINGMODE: number;
    /** Bilinear is mag = linear and min = linear and mip = nearest */
    static readonly TEXTURE_BILINEAR_SAMPLINGMODE: number;
    /** Trilinear is mag = linear and min = linear and mip = linear */
    static readonly TEXTURE_TRILINEAR_SAMPLINGMODE: number;
    /** nearest is mag = nearest and min = nearest and mip = linear */
    static readonly TEXTURE_NEAREST_NEAREST_MIPLINEAR: number;
    /** Bilinear is mag = linear and min = linear and mip = nearest */
    static readonly TEXTURE_LINEAR_LINEAR_MIPNEAREST: number;
    /** Trilinear is mag = linear and min = linear and mip = linear */
    static readonly TEXTURE_LINEAR_LINEAR_MIPLINEAR: number;
    /** mag = nearest and min = nearest and mip = nearest */
    static readonly TEXTURE_NEAREST_NEAREST_MIPNEAREST: number;
    /** mag = nearest and min = linear and mip = nearest */
    static readonly TEXTURE_NEAREST_LINEAR_MIPNEAREST: number;
    /** mag = nearest and min = linear and mip = linear */
    static readonly TEXTURE_NEAREST_LINEAR_MIPLINEAR: number;
    /** mag = nearest and min = linear and mip = none */
    static readonly TEXTURE_NEAREST_LINEAR: number;
    /** mag = nearest and min = nearest and mip = none */
    static readonly TEXTURE_NEAREST_NEAREST: number;
    /** mag = linear and min = nearest and mip = nearest */
    static readonly TEXTURE_LINEAR_NEAREST_MIPNEAREST: number;
    /** mag = linear and min = nearest and mip = linear */
    static readonly TEXTURE_LINEAR_NEAREST_MIPLINEAR: number;
    /** mag = linear and min = linear and mip = none */
    static readonly TEXTURE_LINEAR_LINEAR: number;
    /** mag = linear and min = nearest and mip = none */
    static readonly TEXTURE_LINEAR_NEAREST: number;
    /** Explicit coordinates mode */
    static readonly TEXTURE_EXPLICIT_MODE: number;
    /** Spherical coordinates mode */
    static readonly TEXTURE_SPHERICAL_MODE: number;
    /** Planar coordinates mode */
    static readonly TEXTURE_PLANAR_MODE: number;
    /** Cubic coordinates mode */
    static readonly TEXTURE_CUBIC_MODE: number;
    /** Projection coordinates mode */
    static readonly TEXTURE_PROJECTION_MODE: number;
    /** Skybox coordinates mode */
    static readonly TEXTURE_SKYBOX_MODE: number;
    /** Inverse Cubic coordinates mode */
    static readonly TEXTURE_INVCUBIC_MODE: number;
    /** Equirectangular coordinates mode */
    static readonly TEXTURE_EQUIRECTANGULAR_MODE: number;
    /** Equirectangular Fixed coordinates mode */
    static readonly TEXTURE_FIXED_EQUIRECTANGULAR_MODE: number;
    /** Equirectangular Fixed Mirrored coordinates mode */
    static readonly TEXTURE_FIXED_EQUIRECTANGULAR_MIRRORED_MODE: number;
    /** Defines that texture rescaling will use a floor to find the closer power of 2 size */
    static readonly SCALEMODE_FLOOR: number;
    /** Defines that texture rescaling will look for the nearest power of 2 size */
    static readonly SCALEMODE_NEAREST: number;
    /** Defines that texture rescaling will use a ceil to find the closer power of 2 size */
    static readonly SCALEMODE_CEILING: number;
    /**
     * Returns the current npm package of the sdk
     */
    static get NpmPackage(): string;
    /**
     * Returns the current version of the framework
     */
    static get Version(): string;
    /** Gets the list of created engines */
    static get Instances(): Engine[];
    /**
     * Gets the latest created engine
     */
    static get LastCreatedEngine(): Nullable<Engine>;
    /**
     * Gets the latest created scene
     */
    static get LastCreatedScene(): Nullable<Scene>;
    /** @internal */
    /**
     * Engine abstraction for loading and creating an image bitmap from a given source string.
     * @param imageSource source to load the image from.
     * @param options An object that sets options for the image's extraction.
     * @returns ImageBitmap.
     */
    _createImageBitmapFromSource(imageSource: string, options?: ImageBitmapOptions): Promise<ImageBitmap>;
    /**
     * Engine abstraction for createImageBitmap
     * @param image source for image
     * @param options An object that sets options for the image's extraction.
     * @returns ImageBitmap
     */
    createImageBitmap(image: ImageBitmapSource, options?: ImageBitmapOptions): Promise<ImageBitmap>;
    /**
     * Resize an image and returns the image data as an uint8array
     * @param image image to resize
     * @param bufferWidth destination buffer width
     * @param bufferHeight destination buffer height
     * @returns an uint8array containing RGBA values of bufferWidth * bufferHeight size
     */
    resizeImageBitmap(image: HTMLImageElement | ImageBitmap, bufferWidth: number, bufferHeight: number): Uint8Array;
    /**
     * Will flag all materials in all scenes in all engines as dirty to trigger new shader compilation
     * @param flag defines which part of the materials must be marked as dirty
     * @param predicate defines a predicate used to filter which materials should be affected
     */
    static MarkAllMaterialsAsDirty(flag: number, predicate?: (mat: Material) => boolean): void;
    /**
     * Method called to create the default loading screen.
     * This can be overridden in your own app.
     * @param canvas The rendering canvas element
     * @returns The loading screen
     */
    static DefaultLoadingScreenFactory(canvas: HTMLCanvasElement): ILoadingScreen;
    /**
     * Method called to create the default rescale post process on each engine.
     */
    static _RescalePostProcessFactory: Nullable<(engine: Engine) => PostProcess>;
    /**
     * Gets or sets a boolean to enable/disable IndexedDB support and avoid XHR on .manifest
     **/
    enableOfflineSupport: boolean;
    /**
     * Gets or sets a boolean to enable/disable checking manifest if IndexedDB support is enabled (js will always consider the database is up to date)
     **/
    disableManifestCheck: boolean;
    /**
     * Gets or sets a boolean to enable/disable the context menu (right-click) from appearing on the main canvas
     */
    disableContextMenu: boolean;
    /**
     * Gets the list of created scenes
     */
    scenes: Scene[];
    /** @internal */
    _virtualScenes: Scene[];
    /**
     * Event raised when a new scene is created
     */
    onNewSceneAddedObservable: Observable<Scene>;
    /**
     * Gets the list of created postprocesses
     */
    postProcesses: import("babylonjs/PostProcesses/postProcess").PostProcess[];
    /**
     * Gets a boolean indicating if the pointer is currently locked
     */
    isPointerLock: boolean;
    /**
     * Observable event triggered each time the rendering canvas is resized
     */
    onResizeObservable: Observable<Engine>;
    /**
     * Observable event triggered each time the canvas loses focus
     */
    onCanvasBlurObservable: Observable<Engine>;
    /**
     * Observable event triggered each time the canvas gains focus
     */
    onCanvasFocusObservable: Observable<Engine>;
    /**
     * Observable event triggered each time the canvas receives pointerout event
     */
    onCanvasPointerOutObservable: Observable<PointerEvent>;
    /**
     * Observable raised when the engine begins a new frame
     */
    onBeginFrameObservable: Observable<Engine>;
    /**
     * If set, will be used to request the next animation frame for the render loop
     */
    customAnimationFrameRequester: Nullable<ICustomAnimationFrameRequester>;
    /**
     * Observable raised when the engine ends the current frame
     */
    onEndFrameObservable: Observable<Engine>;
    /**
     * Observable raised when the engine is about to compile a shader
     */
    onBeforeShaderCompilationObservable: Observable<Engine>;
    /**
     * Observable raised when the engine has just compiled a shader
     */
    onAfterShaderCompilationObservable: Observable<Engine>;
    /**
     * Gets the audio engine
     * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic
     * @ignorenaming
     */
    static audioEngine: Nullable<IAudioEngine>;
    /**
     * Default AudioEngine factory responsible of creating the Audio Engine.
     * By default, this will create a BabylonJS Audio Engine if the workload has been embedded.
     */
    static AudioEngineFactory: (hostElement: Nullable<HTMLElement>, audioContext: Nullable<AudioContext>, audioDestination: Nullable<AudioDestinationNode | MediaStreamAudioDestinationNode>) => IAudioEngine;
    /**
     * Default offline support factory responsible of creating a tool used to store data locally.
     * By default, this will create a Database object if the workload has been embedded.
     */
    static OfflineProviderFactory: (urlToScene: string, callbackManifestChecked: (checked: boolean) => any, disableManifestCheck: boolean) => IOfflineProvider;
    private _loadingScreen;
    private _pointerLockRequested;
    private _rescalePostProcess;
    protected _deterministicLockstep: boolean;
    protected _lockstepMaxSteps: number;
    protected _timeStep: number;
    protected get _supportsHardwareTextureRescaling(): boolean;
    private _fps;
    private _deltaTime;
    /** @internal */
    _drawCalls: PerfCounter;
    /** Gets or sets the tab index to set to the rendering canvas. 1 is the minimum value to set to be able to capture keyboard events */
    canvasTabIndex: number;
    /**
     * Turn this value on if you want to pause FPS computation when in background
     */
    disablePerformanceMonitorInBackground: boolean;
    private _performanceMonitor;
    /**
     * Gets the performance monitor attached to this engine
     * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/optimize_your_scene#engineinstrumentation
     */
    get performanceMonitor(): PerformanceMonitor;
    private _onFocus;
    private _onBlur;
    private _onCanvasPointerOut;
    private _onCanvasBlur;
    private _onCanvasFocus;
    private _onCanvasContextMenu;
    private _onFullscreenChange;
    private _onPointerLockChange;
    protected _compatibilityMode: boolean;
    /**
     * (WebGPU only) True (default) to be in compatibility mode, meaning rendering all existing scenes without artifacts (same rendering than WebGL).
     * Setting the property to false will improve performances but may not work in some scenes if some precautions are not taken.
     * See https://doc.babylonjs.com/setup/support/webGPU/webGPUOptimization/webGPUNonCompatibilityMode for more details
     */
    get compatibilityMode(): boolean;
    set compatibilityMode(mode: boolean);
    /**
     * Gets the HTML element used to attach event listeners
     * @returns a HTML element
     */
    getInputElement(): Nullable<HTMLElement>;
    /**
     * Creates a new engine
     * @param canvasOrContext defines the canvas or WebGL context to use for rendering. If you provide a WebGL context, Babylon.js will not hook events on the canvas (like pointers, keyboards, etc...) so no event observables will be available. This is mostly used when Babylon.js is used as a plugin on a system which already used the WebGL context
     * @param antialias defines enable antialiasing (default: false)
     * @param options defines further options to be sent to the getContext() function
     * @param adaptToDeviceRatio defines whether to adapt to the device's viewport characteristics (default: false)
     */
    constructor(canvasOrContext: Nullable<HTMLCanvasElement | OffscreenCanvas | WebGLRenderingContext | WebGL2RenderingContext>, antialias?: boolean, options?: EngineOptions, adaptToDeviceRatio?: boolean);
    protected _initGLContext(): void;
    /**
     * Shared initialization across engines types.
     * @param canvas The canvas associated with this instance of the engine.
     * @param doNotHandleTouchAction Defines that engine should ignore modifying touch action attribute and style
     * @param audioEngine Defines if an audio engine should be created by default
     */
    protected _sharedInit(canvas: HTMLCanvasElement, doNotHandleTouchAction: boolean, audioEngine: boolean): void;
    /**
     * Gets current aspect ratio
     * @param viewportOwner defines the camera to use to get the aspect ratio
     * @param useScreen defines if screen size must be used (or the current render target if any)
     * @returns a number defining the aspect ratio
     */
    getAspectRatio(viewportOwner: IViewportOwnerLike, useScreen?: boolean): number;
    /**
     * Gets current screen aspect ratio
     * @returns a number defining the aspect ratio
     */
    getScreenAspectRatio(): number;
    /**
     * Gets the client rect of the HTML canvas attached with the current webGL context
     * @returns a client rectangle
     */
    getRenderingCanvasClientRect(): Nullable<ClientRect>;
    /**
     * Gets the client rect of the HTML element used for events
     * @returns a client rectangle
     */
    getInputElementClientRect(): Nullable<ClientRect>;
    /**
     * Gets a boolean indicating that the engine is running in deterministic lock step mode
     * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/advanced_animations#deterministic-lockstep
     * @returns true if engine is in deterministic lock step mode
     */
    isDeterministicLockStep(): boolean;
    /**
     * Gets the max steps when engine is running in deterministic lock step
     * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/advanced_animations#deterministic-lockstep
     * @returns the max steps
     */
    getLockstepMaxSteps(): number;
    /**
     * Returns the time in ms between steps when using deterministic lock step.
     * @returns time step in (ms)
     */
    getTimeStep(): number;
    /**
     * Force the mipmap generation for the given render target texture
     * @param texture defines the render target texture to use
     * @param unbind defines whether or not to unbind the texture after generation. Defaults to true.
     */
    generateMipMapsForCubemap(texture: InternalTexture, unbind?: boolean): void;
    /** States */
    /**
     * Gets a boolean indicating if depth writing is enabled
     * @returns the current depth writing state
     */
    getDepthWrite(): boolean;
    /**
     * Enable or disable depth writing
     * @param enable defines the state to set
     */
    setDepthWrite(enable: boolean): void;
    /**
     * Gets a boolean indicating if stencil buffer is enabled
     * @returns the current stencil buffer state
     */
    getStencilBuffer(): boolean;
    /**
     * Enable or disable the stencil buffer
     * @param enable defines if the stencil buffer must be enabled or disabled
     */
    setStencilBuffer(enable: boolean): void;
    /**
     * Gets the current stencil mask
     * @returns a number defining the new stencil mask to use
     */
    getStencilMask(): number;
    /**
     * Sets the current stencil mask
     * @param mask defines the new stencil mask to use
     */
    setStencilMask(mask: number): void;
    /**
     * Gets the current stencil function
     * @returns a number defining the stencil function to use
     */
    getStencilFunction(): number;
    /**
     * Gets the current stencil reference value
     * @returns a number defining the stencil reference value to use
     */
    getStencilFunctionReference(): number;
    /**
     * Gets the current stencil mask
     * @returns a number defining the stencil mask to use
     */
    getStencilFunctionMask(): number;
    /**
     * Sets the current stencil function
     * @param stencilFunc defines the new stencil function to use
     */
    setStencilFunction(stencilFunc: number): void;
    /**
     * Sets the current stencil reference
     * @param reference defines the new stencil reference to use
     */
    setStencilFunctionReference(reference: number): void;
    /**
     * Sets the current stencil mask
     * @param mask defines the new stencil mask to use
     */
    setStencilFunctionMask(mask: number): void;
    /**
     * Gets the current stencil operation when stencil fails
     * @returns a number defining stencil operation to use when stencil fails
     */
    getStencilOperationFail(): number;
    /**
     * Gets the current stencil operation when depth fails
     * @returns a number defining stencil operation to use when depth fails
     */
    getStencilOperationDepthFail(): number;
    /**
     * Gets the current stencil operation when stencil passes
     * @returns a number defining stencil operation to use when stencil passes
     */
    getStencilOperationPass(): number;
    /**
     * Sets the stencil operation to use when stencil fails
     * @param operation defines the stencil operation to use when stencil fails
     */
    setStencilOperationFail(operation: number): void;
    /**
     * Sets the stencil operation to use when depth fails
     * @param operation defines the stencil operation to use when depth fails
     */
    setStencilOperationDepthFail(operation: number): void;
    /**
     * Sets the stencil operation to use when stencil passes
     * @param operation defines the stencil operation to use when stencil passes
     */
    setStencilOperationPass(operation: number): void;
    /**
     * Sets a boolean indicating if the dithering state is enabled or disabled
     * @param value defines the dithering state
     */
    setDitheringState(value: boolean): void;
    /**
     * Sets a boolean indicating if the rasterizer state is enabled or disabled
     * @param value defines the rasterizer state
     */
    setRasterizerState(value: boolean): void;
    /**
     * Gets the current depth function
     * @returns a number defining the depth function
     */
    getDepthFunction(): Nullable<number>;
    /**
     * Sets the current depth function
     * @param depthFunc defines the function to use
     */
    setDepthFunction(depthFunc: number): void;
    /**
     * Sets the current depth function to GREATER
     */
    setDepthFunctionToGreater(): void;
    /**
     * Sets the current depth function to GEQUAL
     */
    setDepthFunctionToGreaterOrEqual(): void;
    /**
     * Sets the current depth function to LESS
     */
    setDepthFunctionToLess(): void;
    /**
     * Sets the current depth function to LEQUAL
     */
    setDepthFunctionToLessOrEqual(): void;
    private _cachedStencilBuffer;
    private _cachedStencilFunction;
    private _cachedStencilMask;
    private _cachedStencilOperationPass;
    private _cachedStencilOperationFail;
    private _cachedStencilOperationDepthFail;
    private _cachedStencilReference;
    /**
     * Caches the the state of the stencil buffer
     */
    cacheStencilState(): void;
    /**
     * Restores the state of the stencil buffer
     */
    restoreStencilState(): void;
    /**
     * Directly set the WebGL Viewport
     * @param x defines the x coordinate of the viewport (in screen space)
     * @param y defines the y coordinate of the viewport (in screen space)
     * @param width defines the width of the viewport (in screen space)
     * @param height defines the height of the viewport (in screen space)
     * @returns the current viewport Object (if any) that is being replaced by this call. You can restore this viewport later on to go back to the original state
     */
    setDirectViewport(x: number, y: number, width: number, height: number): Nullable<IViewportLike>;
    /**
     * Executes a scissor clear (ie. a clear on a specific portion of the screen)
     * @param x defines the x-coordinate of the bottom left corner of the clear rectangle
     * @param y defines the y-coordinate of the corner of the clear rectangle
     * @param width defines the width of the clear rectangle
     * @param height defines the height of the clear rectangle
     * @param clearColor defines the clear color
     */
    scissorClear(x: number, y: number, width: number, height: number, clearColor: IColor4Like): void;
    /**
     * Enable scissor test on a specific rectangle (ie. render will only be executed on a specific portion of the screen)
     * @param x defines the x-coordinate of the bottom left corner of the clear rectangle
     * @param y defines the y-coordinate of the corner of the clear rectangle
     * @param width defines the width of the clear rectangle
     * @param height defines the height of the clear rectangle
     */
    enableScissor(x: number, y: number, width: number, height: number): void;
    /**
     * Disable previously set scissor test rectangle
     */
    disableScissor(): void;
    /**
     * @internal
     */
    _reportDrawCall(numDrawCalls?: number): void;
    /**
     * Initializes a webVR display and starts listening to display change events
     * The onVRDisplayChangedObservable will be notified upon these changes
     * @returns The onVRDisplayChangedObservable
     */
    initWebVR(): Observable<IDisplayChangedEventArgs>;
    /** @internal */
    _prepareVRComponent(): void;
    /**
     * @internal
     */
    _connectVREvents(canvas?: HTMLCanvasElement, document?: any): void;
    /** @internal */
    _submitVRFrame(): void;
    /**
     * Call this function to leave webVR mode
     * Will do nothing if webVR is not supported or if there is no webVR device
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/webVRCamera
     */
    disableVR(): void;
    /**
     * Gets a boolean indicating that the system is in VR mode and is presenting
     * @returns true if VR mode is engaged
     */
    isVRPresenting(): boolean;
    /** @internal */
    _requestVRFrame(): void;
    /**
     * @internal
     */
    _loadFileAsync(url: string, offlineProvider?: IOfflineProvider, useArrayBuffer?: boolean): Promise<string | ArrayBuffer>;
    /**
     * Gets the source code of the vertex shader associated with a specific webGL program
     * @param program defines the program to use
     * @returns a string containing the source code of the vertex shader associated with the program
     */
    getVertexShaderSource(program: WebGLProgram): Nullable<string>;
    /**
     * Gets the source code of the fragment shader associated with a specific webGL program
     * @param program defines the program to use
     * @returns a string containing the source code of the fragment shader associated with the program
     */
    getFragmentShaderSource(program: WebGLProgram): Nullable<string>;
    /**
     * Sets a depth stencil texture from a render target to the according uniform.
     * @param channel The texture channel
     * @param uniform The uniform to set
     * @param texture The render target texture containing the depth stencil texture to apply
     * @param name The texture name
     */
    setDepthStencilTexture(channel: number, uniform: Nullable<WebGLUniformLocation>, texture: Nullable<RenderTargetTexture>, name?: string): void;
    /**
     * Sets a texture to the webGL context from a postprocess
     * @param channel defines the channel to use
     * @param postProcess defines the source postprocess
     * @param name name of the channel
     */
    setTextureFromPostProcess(channel: number, postProcess: Nullable<PostProcess>, name: string): void;
    /**
     * Binds the output of the passed in post process to the texture channel specified
     * @param channel The channel the texture should be bound to
     * @param postProcess The post process which's output should be bound
     * @param name name of the channel
     */
    setTextureFromPostProcessOutput(channel: number, postProcess: Nullable<PostProcess>, name: string): void;
    protected _rebuildBuffers(): void;
    /** @internal */
    _renderFrame(): void;
    _renderLoop(): void;
    /** @internal */
    _renderViews(): boolean;
    /**
     * Toggle full screen mode
     * @param requestPointerLock defines if a pointer lock should be requested from the user
     */
    switchFullscreen(requestPointerLock: boolean): void;
    /**
     * Enters full screen mode
     * @param requestPointerLock defines if a pointer lock should be requested from the user
     */
    enterFullscreen(requestPointerLock: boolean): void;
    /**
     * Exits full screen mode
     */
    exitFullscreen(): void;
    /**
     * Enters Pointerlock mode
     */
    enterPointerlock(): void;
    /**
     * Exits Pointerlock mode
     */
    exitPointerlock(): void;
    /**
     * Begin a new frame
     */
    beginFrame(): void;
    /**
     * End the current frame
     */
    endFrame(): void;
    /**
     * Resize the view according to the canvas' size
     * @param forceSetSize true to force setting the sizes of the underlying canvas
     */
    resize(forceSetSize?: boolean): void;
    /**
     * Force a specific size of the canvas
     * @param width defines the new canvas' width
     * @param height defines the new canvas' height
     * @param forceSetSize true to force setting the sizes of the underlying canvas
     * @returns true if the size was changed
     */
    setSize(width: number, height: number, forceSetSize?: boolean): boolean;
    _deletePipelineContext(pipelineContext: IPipelineContext): void;
    createShaderProgram(pipelineContext: IPipelineContext, vertexCode: string, fragmentCode: string, defines: Nullable<string>, context?: WebGLRenderingContext, transformFeedbackVaryings?: Nullable<string[]>): WebGLProgram;
    protected _createShaderProgram(pipelineContext: WebGLPipelineContext, vertexShader: WebGLShader, fragmentShader: WebGLShader, context: WebGLRenderingContext, transformFeedbackVaryings?: Nullable<string[]>): WebGLProgram;
    /**
     * @internal
     */
    _releaseTexture(texture: InternalTexture): void;
    /**
     * @internal
     */
    _releaseRenderTargetWrapper(rtWrapper: RenderTargetWrapper): void;
    protected static _RenderPassIdCounter: number;
    /**
     * Gets or sets the current render pass id
     */
    currentRenderPassId: number;
    private _renderPassNames;
    /**
     * Gets the names of the render passes that are currently created
     * @returns list of the render pass names
     */
    getRenderPassNames(): string[];
    /**
     * Gets the name of the current render pass
     * @returns name of the current render pass
     */
    getCurrentRenderPassName(): string;
    /**
     * Creates a render pass id
     * @param name Name of the render pass (for debug purpose only)
     * @returns the id of the new render pass
     */
    createRenderPassId(name?: string): number;
    /**
     * Releases a render pass id
     * @param id id of the render pass to release
     */
    releaseRenderPassId(id: number): void;
    /**
     * @internal
     * Rescales a texture
     * @param source input texture
     * @param destination destination texture
     * @param scene scene to use to render the resize
     * @param internalFormat format to use when resizing
     * @param onComplete callback to be called when resize has completed
     */
    _rescaleTexture(source: InternalTexture, destination: InternalTexture, scene: Nullable<any>, internalFormat: number, onComplete: () => void): void;
    /**
     * Gets the current framerate
     * @returns a number representing the framerate
     */
    getFps(): number;
    /**
     * Gets the time spent between current and previous frame
     * @returns a number representing the delta time in ms
     */
    getDeltaTime(): number;
    private _measureFps;
    /**
     * Wraps an external web gl texture in a Babylon texture.
     * @param texture defines the external texture
     * @returns the babylon internal texture
     */
    wrapWebGLTexture(texture: WebGLTexture): InternalTexture;
    /**
     * @internal
     */
    _uploadImageToTexture(texture: InternalTexture, image: HTMLImageElement | ImageBitmap, faceIndex?: number, lod?: number): void;
    /**
     * Updates a depth texture Comparison Mode and Function.
     * If the comparison Function is equal to 0, the mode will be set to none.
     * Otherwise, this only works in webgl 2 and requires a shadow sampler in the shader.
     * @param texture The texture to set the comparison function for
     * @param comparisonFunction The comparison function to set, 0 if no comparison required
     */
    updateTextureComparisonFunction(texture: InternalTexture, comparisonFunction: number): void;
    /**
     * Creates a webGL buffer to use with instantiation
     * @param capacity defines the size of the buffer
     * @returns the webGL buffer
     */
    createInstancesBuffer(capacity: number): DataBuffer;
    /**
     * Delete a webGL buffer used with instantiation
     * @param buffer defines the webGL buffer to delete
     */
    deleteInstancesBuffer(buffer: WebGLBuffer): void;
    private _clientWaitAsync;
    /**
     * @internal
     */
    _readPixelsAsync(x: number, y: number, w: number, h: number, format: number, type: number, outputBuffer: ArrayBufferView): Promise<ArrayBufferView> | null;
    dispose(): void;
    private _disableTouchAction;
    /**
     * Display the loading screen
     * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/customLoadingScreen
     */
    displayLoadingUI(): void;
    /**
     * Hide the loading screen
     * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/customLoadingScreen
     */
    hideLoadingUI(): void;
    /**
     * Gets the current loading screen object
     * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/customLoadingScreen
     */
    get loadingScreen(): ILoadingScreen;
    /**
     * Sets the current loading screen object
     * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/customLoadingScreen
     */
    set loadingScreen(loadingScreen: ILoadingScreen);
    /**
     * Sets the current loading screen text
     * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/customLoadingScreen
     */
    set loadingUIText(text: string);
    /**
     * Sets the current loading screen background color
     * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/customLoadingScreen
     */
    set loadingUIBackgroundColor(color: string);
    /**
     * creates and returns a new video element
     * @param constraints video constraints
     * @returns video element
     */
    createVideoElement(constraints: MediaTrackConstraints): any;
    /** Pointerlock and fullscreen */
    /**
     * Ask the browser to promote the current element to pointerlock mode
     * @param element defines the DOM element to promote
     */
    static _RequestPointerlock(element: HTMLElement): void;
    /**
     * Asks the browser to exit pointerlock mode
     */
    static _ExitPointerlock(): void;
    /**
     * Ask the browser to promote the current element to fullscreen rendering mode
     * @param element defines the DOM element to promote
     */
    static _RequestFullscreen(element: HTMLElement): void;
    /**
     * Asks the browser to exit fullscreen mode
     */
    static _ExitFullscreen(): void;
    /**
     * Get Font size information
     * @param font font name
     * @returns an object containing ascent, height and descent
     */
    getFontOffset(font: string): {
        ascent: number;
        height: number;
        descent: number;
    };
}
export {};

}
declare module "babylonjs/Engines/engineCapabilities" {
/**
 * Interface used to describe the capabilities of the engine relatively to the current browser
 */
export interface EngineCapabilities {
    /** Maximum textures units per fragment shader */
    maxTexturesImageUnits: number;
    /** Maximum texture units per vertex shader */
    maxVertexTextureImageUnits: number;
    /** Maximum textures units in the entire pipeline */
    maxCombinedTexturesImageUnits: number;
    /** Maximum texture size */
    maxTextureSize: number;
    /** Maximum texture samples */
    maxSamples?: number;
    /** Maximum cube texture size */
    maxCubemapTextureSize: number;
    /** Maximum render texture size */
    maxRenderTextureSize: number;
    /** Maximum number of vertex attributes */
    maxVertexAttribs: number;
    /** Maximum number of varyings */
    maxVaryingVectors: number;
    /** Maximum number of uniforms per vertex shader */
    maxVertexUniformVectors: number;
    /** Maximum number of uniforms per fragment shader */
    maxFragmentUniformVectors: number;
    /** Defines if standard derivatives (dx/dy) are supported */
    standardDerivatives: boolean;
    /** Defines if s3tc texture compression is supported */
    s3tc?: WEBGL_compressed_texture_s3tc;
    /** Defines if s3tc sRGB texture compression is supported */
    s3tc_srgb?: WEBGL_compressed_texture_s3tc_srgb;
    /** Defines if pvrtc texture compression is supported */
    pvrtc: any;
    /** Defines if etc1 texture compression is supported */
    etc1: any;
    /** Defines if etc2 texture compression is supported */
    etc2: any;
    /** Defines if astc texture compression is supported */
    astc: any;
    /** Defines if bptc texture compression is supported */
    bptc: any;
    /** Defines if float textures are supported */
    textureFloat: boolean;
    /** Defines if vertex array objects are supported */
    vertexArrayObject: boolean;
    /** Gets the webgl extension for anisotropic filtering (null if not supported) */
    textureAnisotropicFilterExtension?: EXT_texture_filter_anisotropic;
    /** Gets the maximum level of anisotropy supported */
    maxAnisotropy: number;
    /** Defines if instancing is supported */
    instancedArrays: boolean;
    /** Defines if 32 bits indices are supported */
    uintIndices: boolean;
    /** Defines if high precision shaders are supported */
    highPrecisionShaderSupported: boolean;
    /** Defines if depth reading in the fragment shader is supported */
    fragmentDepthSupported: boolean;
    /** Defines if float texture linear filtering is supported*/
    textureFloatLinearFiltering: boolean;
    /** Defines if rendering to float textures is supported */
    textureFloatRender: boolean;
    /** Defines if half float textures are supported*/
    textureHalfFloat: boolean;
    /** Defines if half float texture linear filtering is supported*/
    textureHalfFloatLinearFiltering: boolean;
    /** Defines if rendering to half float textures is supported */
    textureHalfFloatRender: boolean;
    /** Defines if textureLOD shader command is supported */
    textureLOD: boolean;
    /** Defines if draw buffers extension is supported */
    drawBuffersExtension: boolean;
    /** Defines if depth textures are supported */
    depthTextureExtension: boolean;
    /** Defines if float color buffer are supported */
    colorBufferFloat: boolean;
    /** Gets disjoint timer query extension (null if not supported) */
    timerQuery?: EXT_disjoint_timer_query;
    /** Defines if timestamp can be used with timer query */
    canUseTimestampForTimerQuery: boolean;
    /** Defines if occlusion queries are supported by the engine */
    supportOcclusionQuery: boolean;
    /** Defines if multiview is supported (https://www.khronos.org/registry/webgl/extensions/WEBGL_multiview/) */
    multiview?: any;
    /** Defines if oculus multiview is supported (https://developer.oculus.com/documentation/oculus-browser/latest/concepts/browser-multiview/) */
    oculusMultiview?: any;
    /** Function used to let the system compiles shaders in background */
    parallelShaderCompile?: {
        COMPLETION_STATUS_KHR: number;
    };
    /** Max number of texture samples for MSAA */
    maxMSAASamples: number;
    /** Defines if the blend min max extension is supported */
    blendMinMax: boolean;
    /** In some iOS + WebGL1, gl_InstanceID (and gl_InstanceIDEXT) is undefined even if instancedArrays is true. So don't use gl_InstanceID in those cases */
    canUseGLInstanceID: boolean;
    /** Defines if gl_vertexID is available */
    canUseGLVertexID: boolean;
    /** Defines if compute shaders are supported by the engine */
    supportComputeShaders: boolean;
    /** Defines if sRGB texture formats are supported */
    supportSRGBBuffers: boolean;
    /** Defines if transform feedbacks are supported */
    supportTransformFeedbacks: boolean;
    /** Defines if texture max level are supported */
    textureMaxLevel: boolean;
    /** Defines the maximum layer count for a 2D Texture array. */
    texture2DArrayMaxLayerCount: number;
}

}
declare module "babylonjs/Engines/engineFactory" {
import { Engine } from "babylonjs/Engines/engine";
/**
 * Helper class to create the best engine depending on the current hardware
 */
export class EngineFactory {
    /**
     * Creates an engine based on the capabilities of the underlying hardware
     * @param canvas Defines the canvas to use to display the result
     * @param options Defines the options passed to the engine to create the context dependencies
     * @returns a promise that resolves with the created engine
     */
    static CreateAsync(canvas: HTMLCanvasElement, options: any): Promise<Engine>;
}

}
declare module "babylonjs/Engines/engineFeatures" {
/** @internal */
export interface EngineFeatures {
    /** Force using Bitmap when Bitmap or HTMLImageElement can be used */
    forceBitmapOverHTMLImageElement: boolean;
    /** Indicates that the engine support rendering to as well as copying to lod float textures */
    supportRenderAndCopyToLodForFloatTextures: boolean;
    /** Indicates that the engine support handling depth/stencil textures */
    supportDepthStencilTexture: boolean;
    /** Indicates that the engine support shadow samplers */
    supportShadowSamplers: boolean;
    /** Indicates to check the matrix bytes per bytes to know if it has changed or not. If false, only the updateFlag of the matrix is checked */
    uniformBufferHardCheckMatrix: boolean;
    /** Indicates that prefiltered mipmaps can be generated in some processes (for eg when loading an HDR cube texture) */
    allowTexturePrefiltering: boolean;
    /** Indicates to track the usage of ubos and to create new ones as necessary during a frame duration */
    trackUbosInFrame: boolean;
    /** Indicates that the current content of a ubo should be compared to the content of the corresponding GPU buffer and the GPU buffer updated only if different. Requires trackUbosInFrame to be true */
    checkUbosContentBeforeUpload: boolean;
    /** Indicates that the Cascaded Shadow Map technic is supported */
    supportCSM: boolean;
    /** Indicates that the textures transcoded by the basis transcoder must have power of 2 width and height */
    basisNeedsPOT: boolean;
    /** Indicates that the engine supports 3D textures */
    support3DTextures: boolean;
    /** Indicates that constants need a type suffix in shaders (used by realtime filtering...) */
    needTypeSuffixInShaderConstants: boolean;
    /** Indicates that MSAA is supported */
    supportMSAA: boolean;
    /** Indicates that SSAO2 is supported */
    supportSSAO2: boolean;
    /** Indicates that some additional texture formats are supported (like TEXTUREFORMAT_R for eg) */
    supportExtendedTextureFormats: boolean;
    /** Indicates that the switch/case construct is supported in shaders */
    supportSwitchCaseInShader: boolean;
    /** Indicates that synchronous texture reading is supported */
    supportSyncTextureRead: boolean;
    /** Indicates that y should be inverted when dealing with bitmaps (notably in environment tools) */
    needsInvertingBitmap: boolean;
    /** Indicates that the engine should cache the bound UBO */
    useUBOBindingCache: boolean;
    /** Indicates that the inliner should be run over every shader code */
    needShaderCodeInlining: boolean;
    /** Indicates that even if we don't have to update the properties of a uniform buffer (because of some optimzations in the material) we still need to bind the uniform buffer themselves */
    needToAlwaysBindUniformBuffers: boolean;
    /**  Indicates that the engine supports render passes */
    supportRenderPasses: boolean;
    /**  Indicates that the engine supports sprite instancing */
    supportSpriteInstancing: boolean;
    /** @internal */
    _collectUbosUpdatedInFrame: boolean;
}

}
declare module "babylonjs/Engines/engineStore" {
import { Nullable } from "babylonjs/types";
import { Engine } from "babylonjs/Engines/engine";
import { Scene } from "babylonjs/scene";
/**
 * The engine store class is responsible to hold all the instances of Engine and Scene created
 * during the life time of the application.
 */
export class EngineStore {
    /** Gets the list of created engines */
    static Instances: import("babylonjs/Engines/engine").Engine[];
    /** @internal */
    static _LastCreatedScene: Nullable<Scene>;
    /**
     * Gets the latest created engine
     */
    static get LastCreatedEngine(): Nullable<Engine>;
    /**
     * Gets the latest created scene
     */
    static get LastCreatedScene(): Nullable<Scene>;
    /**
     * Gets or sets a global variable indicating if fallback texture must be used when a texture cannot be loaded
     * @ignorenaming
     */
    static UseFallbackTexture: boolean;
    /**
     * Texture content used if a texture cannot loaded
     * @ignorenaming
     */
    static FallbackTexture: string;
}
export {};

}
declare module "babylonjs/Engines/Extensions/engine.alpha" {
module "babylonjs/Engines/thinEngine" {
    interface ThinEngine {
        /**
         * Sets alpha constants used by some alpha blending modes
         * @param r defines the red component
         * @param g defines the green component
         * @param b defines the blue component
         * @param a defines the alpha component
         */
        setAlphaConstants(r: number, g: number, b: number, a: number): void;
        /**
         * Sets the current alpha mode
         * @param mode defines the mode to use (one of the Engine.ALPHA_XXX)
         * @param noDepthWriteChange defines if depth writing state should remains unchanged (false by default)
         * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/advanced/transparent_rendering
         */
        setAlphaMode(mode: number, noDepthWriteChange?: boolean): void;
        /**
         * Gets the current alpha mode
         * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/advanced/transparent_rendering
         * @returns the current alpha mode
         */
        getAlphaMode(): number;
        /**
         * Sets the current alpha equation
         * @param equation defines the equation to use (one of the Engine.ALPHA_EQUATION_XXX)
         */
        setAlphaEquation(equation: number): void;
        /**
         * Gets the current alpha equation.
         * @returns the current alpha equation
         */
        getAlphaEquation(): number;
    }
}
export {};

}
declare module "babylonjs/Engines/Extensions/engine.computeShader" {
import { ComputeEffect, IComputeEffectCreationOptions } from "babylonjs/Compute/computeEffect";
import { IComputeContext } from "babylonjs/Compute/IComputeContext";
import { IComputePipelineContext } from "babylonjs/Compute/IComputePipelineContext";
import { Nullable } from "babylonjs/types";
/**
 * Type used to locate a resource in a compute shader.
 * TODO: remove this when browsers support reflection for wgsl shaders
 */
export type ComputeBindingLocation = {
    group: number;
    binding: number;
};
/**
 * Type used to lookup a resource and retrieve its binding location
 * TODO: remove this when browsers support reflection for wgsl shaders
 */
export type ComputeBindingMapping = {
    [key: string]: ComputeBindingLocation;
};
/** @internal */
export enum ComputeBindingType {
    Texture = 0,
    StorageTexture = 1,
    UniformBuffer = 2,
    StorageBuffer = 3,
    TextureWithoutSampler = 4,
    Sampler = 5
}
/** @internal */
export type ComputeBindingList = {
    [key: string]: {
        type: ComputeBindingType;
        object: any;
        indexInGroupEntries?: number;
    };
};
module "babylonjs/Engines/thinEngine" {
    interface ThinEngine {
        /**
         * Creates a new compute effect
         * @param baseName Name of the effect
         * @param options Options used to create the effect
         * @returns The new compute effect
         */
        createComputeEffect(baseName: any, options: IComputeEffectCreationOptions): ComputeEffect;
        /**
         * Creates a new compute pipeline context
         * @returns the new pipeline
         */
        createComputePipelineContext(): IComputePipelineContext;
        /**
         * Creates a new compute context
         * @returns the new context
         */
        createComputeContext(): IComputeContext | undefined;
        /**
         * Dispatches a compute shader
         * @param effect The compute effect
         * @param context The compute context
         * @param bindings The list of resources to bind to the shader
         * @param x The number of workgroups to execute on the X dimension
         * @param y The number of workgroups to execute on the Y dimension
         * @param z The number of workgroups to execute on the Z dimension
         * @param bindingsMapping list of bindings mapping (key is property name, value is binding location)
         */
        computeDispatch(effect: ComputeEffect, context: IComputeContext, bindings: ComputeBindingList, x: number, y?: number, z?: number, bindingsMapping?: ComputeBindingMapping): void;
        /**
         * Gets a boolean indicating if all created compute effects are ready
         * @returns true if all effects are ready
         */
        areAllComputeEffectsReady(): boolean;
        /**
         * Forces the engine to release all cached compute effects. This means that next effect compilation will have to be done completely even if a similar effect was already compiled
         */
        releaseComputeEffects(): void;
        /** @internal */
        _prepareComputePipelineContext(pipelineContext: IComputePipelineContext, computeSourceCode: string, rawComputeSourceCode: string, defines: Nullable<string>, entryPoint: string): void;
        /** @internal */
        _rebuildComputeEffects(): void;
        /** @internal */
        _executeWhenComputeStateIsCompiled(pipelineContext: IComputePipelineContext, action: () => void): void;
        /** @internal */
        _releaseComputeEffect(effect: ComputeEffect): void;
        /** @internal */
        _deleteComputePipelineContext(pipelineContext: IComputePipelineContext): void;
    }
}

}
declare module "babylonjs/Engines/Extensions/engine.cubeTexture" {
import { InternalTexture } from "babylonjs/Materials/Textures/internalTexture";
import { Nullable } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
import { DepthTextureCreationOptions } from "babylonjs/Materials/Textures/textureCreationOptions";
import { RenderTargetWrapper } from "babylonjs/Engines/renderTargetWrapper";
module "babylonjs/Engines/thinEngine" {
    interface ThinEngine {
        /**
         * Creates a depth stencil cube texture.
         * This is only available in WebGL 2.
         * @param size The size of face edge in the cube texture.
         * @param options The options defining the cube texture.
         * @param rtWrapper The render target wrapper for which the depth/stencil texture must be created
         * @returns The cube texture
         */
        _createDepthStencilCubeTexture(size: number, options: DepthTextureCreationOptions, rtWrapper: RenderTargetWrapper): InternalTexture;
        /**
         * Creates a cube texture
         * @param rootUrl defines the url where the files to load is located
         * @param scene defines the current scene
         * @param files defines the list of files to load (1 per face)
         * @param noMipmap defines a boolean indicating that no mipmaps shall be generated (false by default)
         * @param onLoad defines an optional callback raised when the texture is loaded
         * @param onError defines an optional callback raised if there is an issue to load the texture
         * @param format defines the format of the data
         * @param forcedExtension defines the extension to use to pick the right loader
         * @param createPolynomials if a polynomial sphere should be created for the cube texture
         * @param lodScale defines the scale applied to environment texture. This manages the range of LOD level used for IBL according to the roughness
         * @param lodOffset defines the offset applied to environment texture. This manages first LOD level used for IBL according to the roughness
         * @param fallback defines texture to use while falling back when (compressed) texture file not found.
         * @param loaderOptions options to be passed to the loader
         * @param useSRGBBuffer defines if the texture must be loaded in a sRGB GPU buffer (if supported by the GPU).
         * @returns the cube texture as an InternalTexture
         */
        createCubeTexture(rootUrl: string, scene: Nullable<Scene>, files: Nullable<string[]>, noMipmap: boolean | undefined, onLoad: Nullable<(data?: any) => void>, onError: Nullable<(message?: string, exception?: any) => void>, format: number | undefined, forcedExtension: any, createPolynomials: boolean, lodScale: number, lodOffset: number, fallback: Nullable<InternalTexture>, loaderOptions: any, useSRGBBuffer: boolean): InternalTexture;
        /**
         * Creates a cube texture
         * @param rootUrl defines the url where the files to load is located
         * @param scene defines the current scene
         * @param files defines the list of files to load (1 per face)
         * @param noMipmap defines a boolean indicating that no mipmaps shall be generated (false by default)
         * @param onLoad defines an optional callback raised when the texture is loaded
         * @param onError defines an optional callback raised if there is an issue to load the texture
         * @param format defines the format of the data
         * @param forcedExtension defines the extension to use to pick the right loader
         * @returns the cube texture as an InternalTexture
         */
        createCubeTexture(rootUrl: string, scene: Nullable<Scene>, files: Nullable<string[]>, noMipmap: boolean, onLoad: Nullable<(data?: any) => void>, onError: Nullable<(message?: string, exception?: any) => void>, format: number | undefined, forcedExtension: any): InternalTexture;
        /**
         * Creates a cube texture
         * @param rootUrl defines the url where the files to load is located
         * @param scene defines the current scene
         * @param files defines the list of files to load (1 per face)
         * @param noMipmap defines a boolean indicating that no mipmaps shall be generated (false by default)
         * @param onLoad defines an optional callback raised when the texture is loaded
         * @param onError defines an optional callback raised if there is an issue to load the texture
         * @param format defines the format of the data
         * @param forcedExtension defines the extension to use to pick the right loader
         * @param createPolynomials if a polynomial sphere should be created for the cube texture
         * @param lodScale defines the scale applied to environment texture. This manages the range of LOD level used for IBL according to the roughness
         * @param lodOffset defines the offset applied to environment texture. This manages first LOD level used for IBL according to the roughness
         * @returns the cube texture as an InternalTexture
         */
        createCubeTexture(rootUrl: string, scene: Nullable<Scene>, files: Nullable<string[]>, noMipmap: boolean, onLoad: Nullable<(data?: any) => void>, onError: Nullable<(message?: string, exception?: any) => void>, format: number | undefined, forcedExtension: any, createPolynomials: boolean, lodScale: number, lodOffset: number): InternalTexture;
        /** @internal */
        createCubeTextureBase(rootUrl: string, scene: Nullable<Scene>, files: Nullable<string[]>, noMipmap: boolean, onLoad: Nullable<(data?: any) => void>, onError: Nullable<(message?: string, exception?: any) => void>, format: number | undefined, forcedExtension: any, createPolynomials: boolean, lodScale: number, lodOffset: number, fallback: Nullable<InternalTexture>, beforeLoadCubeDataCallback: Nullable<(texture: InternalTexture, data: ArrayBufferView | ArrayBufferView[]) => void>, imageHandler: Nullable<(texture: InternalTexture, imgs: HTMLImageElement[] | ImageBitmap[]) => void>, useSRGBBuffer: boolean): InternalTexture;
        /** @internal */
        _partialLoadFile(url: string, index: number, loadedFiles: ArrayBuffer[], onfinish: (files: ArrayBuffer[]) => void, onErrorCallBack: Nullable<(message?: string, exception?: any) => void>): void;
        /** @internal */
        _cascadeLoadFiles(scene: Nullable<Scene>, onfinish: (images: ArrayBuffer[]) => void, files: string[], onError: Nullable<(message?: string, exception?: any) => void>): void;
        /** @internal */
        _cascadeLoadImgs(scene: Nullable<Scene>, texture: InternalTexture, onfinish: Nullable<(texture: InternalTexture, images: HTMLImageElement[] | ImageBitmap[]) => void>, files: string[], onError: Nullable<(message?: string, exception?: any) => void>, mimeType?: string): void;
        /** @internal */
        _partialLoadImg(url: string, index: number, loadedImages: HTMLImageElement[] | ImageBitmap[], scene: Nullable<Scene>, texture: InternalTexture, onfinish: Nullable<(texture: InternalTexture, images: HTMLImageElement[] | ImageBitmap[]) => void>, onErrorCallBack: Nullable<(message?: string, exception?: any) => void>, mimeType?: string): void;
        /**
         * @internal
         */
        _setCubeMapTextureParams(texture: InternalTexture, loadMipmap: boolean, maxLevel?: number): void;
    }
}

}
declare module "babylonjs/Engines/Extensions/engine.debugging" {
module "babylonjs/Engines/thinEngine" {
    interface ThinEngine {
        /** @internal */
        _debugPushGroup(groupName: string, targetObject?: number): void;
        /** @internal */
        _debugPopGroup(targetObject?: number): void;
        /** @internal */
        _debugInsertMarker(text: string, targetObject?: number): void;
        /** @internal */
        _debugFlushPendingCommands(): void;
    }
}
export {};

}
declare module "babylonjs/Engines/Extensions/engine.dynamicBuffer" {
import { DataBuffer } from "babylonjs/Buffers/dataBuffer";
import { IndicesArray, DataArray } from "babylonjs/types";
module "babylonjs/Engines/thinEngine" {
    interface ThinEngine {
        /**
         * Update a dynamic index buffer
         * @param indexBuffer defines the target index buffer
         * @param indices defines the data to update
         * @param offset defines the offset in the target index buffer where update should start
         */
        updateDynamicIndexBuffer(indexBuffer: DataBuffer, indices: IndicesArray, offset?: number): void;
        /**
         * Updates a dynamic vertex buffer.
         * @param vertexBuffer the vertex buffer to update
         * @param data the data used to update the vertex buffer
         * @param byteOffset the byte offset of the data
         * @param byteLength the byte length of the data
         */
        updateDynamicVertexBuffer(vertexBuffer: DataBuffer, data: DataArray, byteOffset?: number, byteLength?: number): void;
    }
}

}
declare module "babylonjs/Engines/Extensions/engine.dynamicTexture" {
import { InternalTexture } from "babylonjs/Materials/Textures/internalTexture";
import { Nullable } from "babylonjs/types";
import { ICanvas } from "babylonjs/Engines/ICanvas";
module "babylonjs/Engines/thinEngine" {
    interface ThinEngine {
        /**
         * Creates a dynamic texture
         * @param width defines the width of the texture
         * @param height defines the height of the texture
         * @param generateMipMaps defines if the engine should generate the mip levels
         * @param samplingMode defines the required sampling mode (Texture.NEAREST_SAMPLINGMODE by default)
         * @returns the dynamic texture inside an InternalTexture
         */
        createDynamicTexture(width: number, height: number, generateMipMaps: boolean, samplingMode: number): InternalTexture;
        /**
         * Update the content of a dynamic texture
         * @param texture defines the texture to update
         * @param source defines the source containing the data
         * @param invertY defines if data must be stored with Y axis inverted
         * @param premulAlpha defines if alpha is stored as premultiplied
         * @param format defines the format of the data
         * @param forceBindTexture if the texture should be forced to be bound eg. after a graphics context loss (Default: false)
         * @param allowGPUOptimization true to allow some specific GPU optimizations (subject to engine feature "allowGPUOptimizationsForGUI" being true)
         */
        updateDynamicTexture(texture: Nullable<InternalTexture>, source: ImageBitmap | ImageData | HTMLImageElement | HTMLCanvasElement | HTMLVideoElement | OffscreenCanvas | ICanvas, invertY?: boolean, premulAlpha?: boolean, format?: number, forceBindTexture?: boolean, allowGPUOptimization?: boolean): void;
    }
}

}
declare module "babylonjs/Engines/Extensions/engine.externalTexture" {
import { ExternalTexture } from "babylonjs/Materials/Textures/externalTexture";
import { Nullable } from "babylonjs/types";
module "babylonjs/Engines/thinEngine" {
    interface ThinEngine {
        /**
         * Creates an external texture
         * @param video video element
         * @returns the external texture, or null if external textures are not supported by the engine
         */
        createExternalTexture(video: HTMLVideoElement): Nullable<ExternalTexture>;
        /**
         * Sets an internal texture to the according uniform.
         * @param name The name of the uniform in the effect
         * @param texture The texture to apply
         */
        setExternalTexture(name: string, texture: Nullable<ExternalTexture>): void;
    }
}

}
declare module "babylonjs/Engines/Extensions/engine.multiRender" {
import { IMultiRenderTargetOptions } from "babylonjs/Materials/Textures/multiRenderTarget";
import { Nullable } from "babylonjs/types";
import { RenderTargetWrapper } from "babylonjs/Engines/renderTargetWrapper";
import { TextureSize } from "babylonjs/Materials/Textures/textureCreationOptions";
module "babylonjs/Engines/thinEngine" {
    interface ThinEngine {
        /**
         * Unbind a list of render target textures from the webGL context
         * This is used only when drawBuffer extension or webGL2 are active
         * @param rtWrapper defines the render target wrapper to unbind
         * @param disableGenerateMipMaps defines a boolean indicating that mipmaps must not be generated
         * @param onBeforeUnbind defines a function which will be called before the effective unbind
         */
        unBindMultiColorAttachmentFramebuffer(rtWrapper: RenderTargetWrapper, disableGenerateMipMaps: boolean, onBeforeUnbind?: () => void): void;
        /**
         * Create a multi render target texture
         * @see https://doc.babylonjs.com/setup/support/webGL2#multiple-render-target
         * @param size defines the size of the texture
         * @param options defines the creation options
         * @param initializeBuffers if set to true, the engine will make an initializing call of drawBuffers
         * @returns a new render target wrapper ready to render textures
         */
        createMultipleRenderTarget(size: TextureSize, options: IMultiRenderTargetOptions, initializeBuffers?: boolean): RenderTargetWrapper;
        /**
         * Update the sample count for a given multiple render target texture
         * @see https://doc.babylonjs.com/setup/support/webGL2#multisample-render-targets
         * @param rtWrapper defines the render target wrapper to update
         * @param samples defines the sample count to set
         * @param initializeBuffers if set to true, the engine will make an initializing call of drawBuffers
         * @returns the effective sample count (could be 0 if multisample render targets are not supported)
         */
        updateMultipleRenderTargetTextureSampleCount(rtWrapper: Nullable<RenderTargetWrapper>, samples: number, initializeBuffers?: boolean): number;
        /**
         * Select a subsets of attachments to draw to.
         * @param attachments gl attachments
         */
        bindAttachments(attachments: number[]): void;
        /**
         * Creates a layout object to draw/clear on specific textures in a MRT
         * @param textureStatus textureStatus[i] indicates if the i-th is active
         * @returns A layout to be fed to the engine, calling `bindAttachments`.
         */
        buildTextureLayout(textureStatus: boolean[]): number[];
        /**
         * Restores the webgl state to only draw on the main color attachment
         * when the frame buffer associated is the canvas frame buffer
         */
        restoreSingleAttachment(): void;
        /**
         * Restores the webgl state to only draw on the main color attachment
         * when the frame buffer associated is not the canvas frame buffer
         */
        restoreSingleAttachmentForRenderTarget(): void;
    }
}

}
declare module "babylonjs/Engines/Extensions/engine.multiview" {
import { Camera } from "babylonjs/Cameras/camera";
import { Nullable } from "babylonjs/types";
import { RenderTargetTexture } from "babylonjs/Materials/Textures/renderTargetTexture";
import { Matrix } from "babylonjs/Maths/math.vector";
import { UniformBuffer } from "babylonjs/Materials/uniformBuffer";
import { RenderTargetWrapper } from "babylonjs/Engines/renderTargetWrapper";
module "babylonjs/Engines/engine" {
    interface Engine {
        /**
         * Creates a new multiview render target
         * @param width defines the width of the texture
         * @param height defines the height of the texture
         * @returns the created multiview render target wrapper
         */
        createMultiviewRenderTargetTexture(width: number, height: number): RenderTargetWrapper;
        /**
         * Binds a multiview render target wrapper to be drawn to
         * @param multiviewTexture render target wrapper to bind
         */
        bindMultiviewFramebuffer(multiviewTexture: RenderTargetWrapper): void;
    }
}
module "babylonjs/Cameras/camera" {
    interface Camera {
        /**
         * @internal
         * For cameras that cannot use multiview images to display directly. (e.g. webVR camera will render to multiview texture, then copy to each eye texture and go from there)
         */
        _useMultiviewToSingleView: boolean;
        /**
         * @internal
         * For cameras that cannot use multiview images to display directly. (e.g. webVR camera will render to multiview texture, then copy to each eye texture and go from there)
         */
        _multiviewTexture: Nullable<RenderTargetTexture>;
        /**
         * @internal
         * For WebXR cameras that are rendering to multiview texture arrays.
         */
        _renderingMultiview: boolean;
        /**
         * @internal
         * ensures the multiview texture of the camera exists and has the specified width/height
         * @param width height to set on the multiview texture
         * @param height width to set on the multiview texture
         */
        _resizeOrCreateMultiviewTexture(width: number, height: number): void;
    }
}
module "babylonjs/scene" {
    interface Scene {
        /** @internal */
        _transformMatrixR: Matrix;
        /** @internal */
        _multiviewSceneUbo: Nullable<UniformBuffer>;
        /** @internal */
        _createMultiviewUbo(): void;
        /** @internal */
        _updateMultiviewUbo(viewR?: Matrix, projectionR?: Matrix): void;
        /** @internal */
        _renderMultiviewToSingleView(camera: Camera): void;
    }
}

}
declare module "babylonjs/Engines/Extensions/engine.query" {
import { Nullable, int } from "babylonjs/types";
import { _TimeToken } from "babylonjs/Instrumentation/timeToken";
import { PerfCounter } from "babylonjs/Misc/perfCounter";
import { Observer } from "babylonjs/Misc/observable";
/** @internal */
export type OcclusionQuery = WebGLQuery | number;
/** @internal */
export class _OcclusionDataStorage {
    /** @internal */
    occlusionInternalRetryCounter: number;
    /** @internal */
    isOcclusionQueryInProgress: boolean;
    /** @internal */
    isOccluded: boolean;
    /** @internal */
    occlusionRetryCount: number;
    /** @internal */
    occlusionType: number;
    /** @internal */
    occlusionQueryAlgorithmType: number;
    /** @internal */
    forceRenderingWhenOccluded: boolean;
}
module "babylonjs/Engines/engine" {
    interface Engine {
        /**
         * Create a new webGL query (you must be sure that queries are supported by checking getCaps() function)
         * @returns the new query
         */
        createQuery(): OcclusionQuery;
        /**
         * Delete and release a webGL query
         * @param query defines the query to delete
         * @returns the current engine
         */
        deleteQuery(query: OcclusionQuery): Engine;
        /**
         * Check if a given query has resolved and got its value
         * @param query defines the query to check
         * @returns true if the query got its value
         */
        isQueryResultAvailable(query: OcclusionQuery): boolean;
        /**
         * Gets the value of a given query
         * @param query defines the query to check
         * @returns the value of the query
         */
        getQueryResult(query: OcclusionQuery): number;
        /**
         * Initiates an occlusion query
         * @param algorithmType defines the algorithm to use
         * @param query defines the query to use
         * @returns the current engine
         * @see https://doc.babylonjs.com/features/featuresDeepDive/occlusionQueries
         */
        beginOcclusionQuery(algorithmType: number, query: OcclusionQuery): boolean;
        /**
         * Ends an occlusion query
         * @see https://doc.babylonjs.com/features/featuresDeepDive/occlusionQueries
         * @param algorithmType defines the algorithm to use
         * @returns the current engine
         */
        endOcclusionQuery(algorithmType: number): Engine;
        /**
         * Starts a time query (used to measure time spent by the GPU on a specific frame)
         * Please note that only one query can be issued at a time
         * @returns a time token used to track the time span
         */
        startTimeQuery(): Nullable<_TimeToken>;
        /**
         * Ends a time query
         * @param token defines the token used to measure the time span
         * @returns the time spent (in ns)
         */
        endTimeQuery(token: _TimeToken): int;
        /**
         * Get the performance counter associated with the frame time computation
         * @returns the perf counter
         */
        getGPUFrameTimeCounter(): PerfCounter;
        /**
         * Enable or disable the GPU frame time capture
         * @param value True to enable, false to disable
         */
        captureGPUFrameTime(value: boolean): void;
        /** @internal */
        _currentNonTimestampToken: Nullable<_TimeToken>;
        /** @internal */
        _captureGPUFrameTime: boolean;
        /** @internal */
        _gpuFrameTimeToken: Nullable<_TimeToken>;
        /** @internal */
        _gpuFrameTime: PerfCounter;
        /** @internal */
        _onBeginFrameObserver: Nullable<Observer<Engine>>;
        /** @internal */
        _onEndFrameObserver: Nullable<Observer<Engine>>;
        /** @internal */
        _createTimeQuery(): WebGLQuery;
        /** @internal */
        _deleteTimeQuery(query: WebGLQuery): void;
        /** @internal */
        _getGlAlgorithmType(algorithmType: number): number;
        /** @internal */
        _getTimeQueryResult(query: WebGLQuery): any;
        /** @internal */
        _getTimeQueryAvailability(query: WebGLQuery): any;
    }
}
module "babylonjs/Meshes/abstractMesh" {
    interface AbstractMesh {
        /**
         * Backing filed
         * @internal
         */
        __occlusionDataStorage: _OcclusionDataStorage;
        /**
         * Access property
         * @internal
         */
        _occlusionDataStorage: _OcclusionDataStorage;
        /**
         * This number indicates the number of allowed retries before stop the occlusion query, this is useful if the occlusion query is taking long time before to the query result is retrieved, the query result indicates if the object is visible within the scene or not and based on that Babylon.Js engine decides to show or hide the object.
         * The default value is -1 which means don't break the query and wait till the result
         * @see https://doc.babylonjs.com/features/featuresDeepDive/occlusionQueries
         */
        occlusionRetryCount: number;
        /**
         * This property is responsible for starting the occlusion query within the Mesh or not, this property is also used to determine what should happen when the occlusionRetryCount is reached. It has supports 3 values:
         * * OCCLUSION_TYPE_NONE (Default Value): this option means no occlusion query within the Mesh.
         * * OCCLUSION_TYPE_OPTIMISTIC: this option is means use occlusion query and if occlusionRetryCount is reached and the query is broken show the mesh.
         * * OCCLUSION_TYPE_STRICT: this option is means use occlusion query and if occlusionRetryCount is reached and the query is broken restore the last state of the mesh occlusion if the mesh was visible then show the mesh if was hidden then hide don't show.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/occlusionQueries
         */
        occlusionType: number;
        /**
         * This property determines the type of occlusion query algorithm to run in WebGl, you can use:
         * * AbstractMesh.OCCLUSION_ALGORITHM_TYPE_ACCURATE which is mapped to GL_ANY_SAMPLES_PASSED.
         * * AbstractMesh.OCCLUSION_ALGORITHM_TYPE_CONSERVATIVE (Default Value) which is mapped to GL_ANY_SAMPLES_PASSED_CONSERVATIVE which is a false positive algorithm that is faster than GL_ANY_SAMPLES_PASSED but less accurate.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/occlusionQueries
         */
        occlusionQueryAlgorithmType: number;
        /**
         * Gets or sets whether the mesh is occluded or not, it is used also to set the initial state of the mesh to be occluded or not
         * @see https://doc.babylonjs.com/features/featuresDeepDive/occlusionQueries
         */
        isOccluded: boolean;
        /**
         * Flag to check the progress status of the query
         * @see https://doc.babylonjs.com/features/featuresDeepDive/occlusionQueries
         */
        isOcclusionQueryInProgress: boolean;
        /**
         * Flag to force rendering the mesh even if occluded
         * @see https://doc.babylonjs.com/features/featuresDeepDive/occlusionQueries
         */
        forceRenderingWhenOccluded: boolean;
    }
}

}
declare module "babylonjs/Engines/Extensions/engine.rawTexture" {
import { Nullable } from "babylonjs/types";
import { InternalTexture } from "babylonjs/Materials/Textures/internalTexture";
import { Scene } from "babylonjs/scene";
module "babylonjs/Engines/thinEngine" {
    interface ThinEngine {
        /**
         * Creates a raw texture
         * @param data defines the data to store in the texture
         * @param width defines the width of the texture
         * @param height defines the height of the texture
         * @param format defines the format of the data
         * @param generateMipMaps defines if the engine should generate the mip levels
         * @param invertY defines if data must be stored with Y axis inverted
         * @param samplingMode defines the required sampling mode (Texture.NEAREST_SAMPLINGMODE by default)
         * @param compression defines the compression used (null by default)
         * @param type defines the type fo the data (Engine.TEXTURETYPE_UNSIGNED_INT by default)
         * @param creationFlags specific flags to use when creating the texture (Constants.TEXTURE_CREATIONFLAG_STORAGE for storage textures, for eg)
         * @param useSRGBBuffer defines if the texture must be loaded in a sRGB GPU buffer (if supported by the GPU).
         * @returns the raw texture inside an InternalTexture
         */
        createRawTexture(data: Nullable<ArrayBufferView>, width: number, height: number, format: number, generateMipMaps: boolean, invertY: boolean, samplingMode: number, compression: Nullable<string>, type: number, creationFlags?: number, useSRGBBuffer?: boolean): InternalTexture;
        /**
         * Update a raw texture
         * @param texture defines the texture to update
         * @param data defines the data to store in the texture
         * @param format defines the format of the data
         * @param invertY defines if data must be stored with Y axis inverted
         */
        updateRawTexture(texture: Nullable<InternalTexture>, data: Nullable<ArrayBufferView>, format: number, invertY: boolean): void;
        /**
         * Update a raw texture
         * @param texture defines the texture to update
         * @param data defines the data to store in the texture
         * @param format defines the format of the data
         * @param invertY defines if data must be stored with Y axis inverted
         * @param compression defines the compression used (null by default)
         * @param type defines the type fo the data (Engine.TEXTURETYPE_UNSIGNED_INT by default)
         * @param useSRGBBuffer defines if the texture must be loaded in a sRGB GPU buffer (if supported by the GPU).
         */
        updateRawTexture(texture: Nullable<InternalTexture>, data: Nullable<ArrayBufferView>, format: number, invertY: boolean, compression: Nullable<string>, type: number, useSRGBBuffer: boolean): void;
        /**
         * Creates a new raw cube texture
         * @param data defines the array of data to use to create each face
         * @param size defines the size of the textures
         * @param format defines the format of the data
         * @param type defines the type of the data (like Engine.TEXTURETYPE_UNSIGNED_INT)
         * @param generateMipMaps  defines if the engine should generate the mip levels
         * @param invertY defines if data must be stored with Y axis inverted
         * @param samplingMode defines the required sampling mode (like Texture.NEAREST_SAMPLINGMODE)
         * @param compression defines the compression used (null by default)
         * @returns the cube texture as an InternalTexture
         */
        createRawCubeTexture(data: Nullable<ArrayBufferView[]>, size: number, format: number, type: number, generateMipMaps: boolean, invertY: boolean, samplingMode: number, compression: Nullable<string>): InternalTexture;
        /**
         * Update a raw cube texture
         * @param texture defines the texture to update
         * @param data defines the data to store
         * @param format defines the data format
         * @param type defines the type fo the data (Engine.TEXTURETYPE_UNSIGNED_INT by default)
         * @param invertY defines if data must be stored with Y axis inverted
         */
        updateRawCubeTexture(texture: InternalTexture, data: ArrayBufferView[], format: number, type: number, invertY: boolean): void;
        /**
         * Update a raw cube texture
         * @param texture defines the texture to update
         * @param data defines the data to store
         * @param format defines the data format
         * @param type defines the type fo the data (Engine.TEXTURETYPE_UNSIGNED_INT by default)
         * @param invertY defines if data must be stored with Y axis inverted
         * @param compression defines the compression used (null by default)
         */
        updateRawCubeTexture(texture: InternalTexture, data: ArrayBufferView[], format: number, type: number, invertY: boolean, compression: Nullable<string>): void;
        /**
         * Update a raw cube texture
         * @param texture defines the texture to update
         * @param data defines the data to store
         * @param format defines the data format
         * @param type defines the type fo the data (Engine.TEXTURETYPE_UNSIGNED_INT by default)
         * @param invertY defines if data must be stored with Y axis inverted
         * @param compression defines the compression used (null by default)
         * @param level defines which level of the texture to update
         */
        updateRawCubeTexture(texture: InternalTexture, data: ArrayBufferView[], format: number, type: number, invertY: boolean, compression: Nullable<string>, level: number): void;
        /**
         * Creates a new raw cube texture from a specified url
         * @param url defines the url where the data is located
         * @param scene defines the current scene
         * @param size defines the size of the textures
         * @param format defines the format of the data
         * @param type defines the type fo the data (like Engine.TEXTURETYPE_UNSIGNED_INT)
         * @param noMipmap defines if the engine should avoid generating the mip levels
         * @param callback defines a callback used to extract texture data from loaded data
         * @param mipmapGenerator defines to provide an optional tool to generate mip levels
         * @param onLoad defines a callback called when texture is loaded
         * @param onError defines a callback called if there is an error
         * @returns the cube texture as an InternalTexture
         */
        createRawCubeTextureFromUrl(url: string, scene: Nullable<Scene>, size: number, format: number, type: number, noMipmap: boolean, callback: (ArrayBuffer: ArrayBuffer) => Nullable<ArrayBufferView[]>, mipmapGenerator: Nullable<(faces: ArrayBufferView[]) => ArrayBufferView[][]>, onLoad: Nullable<() => void>, onError: Nullable<(message?: string, exception?: any) => void>): InternalTexture;
        /**
         * Creates a new raw cube texture from a specified url
         * @param url defines the url where the data is located
         * @param scene defines the current scene
         * @param size defines the size of the textures
         * @param format defines the format of the data
         * @param type defines the type fo the data (like Engine.TEXTURETYPE_UNSIGNED_INT)
         * @param noMipmap defines if the engine should avoid generating the mip levels
         * @param callback defines a callback used to extract texture data from loaded data
         * @param mipmapGenerator defines to provide an optional tool to generate mip levels
         * @param onLoad defines a callback called when texture is loaded
         * @param onError defines a callback called if there is an error
         * @param samplingMode defines the required sampling mode (like Texture.NEAREST_SAMPLINGMODE)
         * @param invertY defines if data must be stored with Y axis inverted
         * @returns the cube texture as an InternalTexture
         */
        createRawCubeTextureFromUrl(url: string, scene: Nullable<Scene>, size: number, format: number, type: number, noMipmap: boolean, callback: (ArrayBuffer: ArrayBuffer) => Nullable<ArrayBufferView[]>, mipmapGenerator: Nullable<(faces: ArrayBufferView[]) => ArrayBufferView[][]>, onLoad: Nullable<() => void>, onError: Nullable<(message?: string, exception?: any) => void>, samplingMode: number, invertY: boolean): InternalTexture;
        /**
         * Creates a new raw 3D texture
         * @param data defines the data used to create the texture
         * @param width defines the width of the texture
         * @param height defines the height of the texture
         * @param depth defines the depth of the texture
         * @param format defines the format of the texture
         * @param generateMipMaps defines if the engine must generate mip levels
         * @param invertY defines if data must be stored with Y axis inverted
         * @param samplingMode defines the required sampling mode (like Texture.NEAREST_SAMPLINGMODE)
         * @param compression defines the compressed used (can be null)
         * @param textureType defines the compressed used (can be null)
         * @param creationFlags specific flags to use when creating the texture (Constants.TEXTURE_CREATIONFLAG_STORAGE for storage textures, for eg)
         * @returns a new raw 3D texture (stored in an InternalTexture)
         */
        createRawTexture3D(data: Nullable<ArrayBufferView>, width: number, height: number, depth: number, format: number, generateMipMaps: boolean, invertY: boolean, samplingMode: number, compression: Nullable<string>, textureType: number, creationFlags?: number): InternalTexture;
        /**
         * Update a raw 3D texture
         * @param texture defines the texture to update
         * @param data defines the data to store
         * @param format defines the data format
         * @param invertY defines if data must be stored with Y axis inverted
         */
        updateRawTexture3D(texture: InternalTexture, data: Nullable<ArrayBufferView>, format: number, invertY: boolean): void;
        /**
         * Update a raw 3D texture
         * @param texture defines the texture to update
         * @param data defines the data to store
         * @param format defines the data format
         * @param invertY defines if data must be stored with Y axis inverted
         * @param compression defines the used compression (can be null)
         * @param textureType defines the texture Type (Engine.TEXTURETYPE_UNSIGNED_INT, Engine.TEXTURETYPE_FLOAT...)
         */
        updateRawTexture3D(texture: InternalTexture, data: Nullable<ArrayBufferView>, format: number, invertY: boolean, compression: Nullable<string>, textureType: number): void;
        /**
         * Creates a new raw 2D array texture
         * @param data defines the data used to create the texture
         * @param width defines the width of the texture
         * @param height defines the height of the texture
         * @param depth defines the number of layers of the texture
         * @param format defines the format of the texture
         * @param generateMipMaps defines if the engine must generate mip levels
         * @param invertY defines if data must be stored with Y axis inverted
         * @param samplingMode defines the required sampling mode (like Texture.NEAREST_SAMPLINGMODE)
         * @param compression defines the compressed used (can be null)
         * @param textureType defines the compressed used (can be null)
         * @param creationFlags specific flags to use when creating the texture (Constants.TEXTURE_CREATIONFLAG_STORAGE for storage textures, for eg)
         * @returns a new raw 2D array texture (stored in an InternalTexture)
         */
        createRawTexture2DArray(data: Nullable<ArrayBufferView>, width: number, height: number, depth: number, format: number, generateMipMaps: boolean, invertY: boolean, samplingMode: number, compression: Nullable<string>, textureType: number, creationFlags?: number): InternalTexture;
        /**
         * Update a raw 2D array texture
         * @param texture defines the texture to update
         * @param data defines the data to store
         * @param format defines the data format
         * @param invertY defines if data must be stored with Y axis inverted
         */
        updateRawTexture2DArray(texture: InternalTexture, data: Nullable<ArrayBufferView>, format: number, invertY: boolean): void;
        /**
         * Update a raw 2D array texture
         * @param texture defines the texture to update
         * @param data defines the data to store
         * @param format defines the data format
         * @param invertY defines if data must be stored with Y axis inverted
         * @param compression defines the used compression (can be null)
         * @param textureType defines the texture Type (Engine.TEXTURETYPE_UNSIGNED_INT, Engine.TEXTURETYPE_FLOAT...)
         */
        updateRawTexture2DArray(texture: InternalTexture, data: Nullable<ArrayBufferView>, format: number, invertY: boolean, compression: Nullable<string>, textureType: number): void;
    }
}

}
declare module "babylonjs/Engines/Extensions/engine.readTexture" {
import { InternalTexture } from "babylonjs/Materials/Textures/internalTexture";
import { Nullable } from "babylonjs/types";
module "babylonjs/Engines/thinEngine" {
    interface ThinEngine {
        /** @internal */
        _readTexturePixels(texture: InternalTexture, width: number, height: number, faceIndex?: number, level?: number, buffer?: Nullable<ArrayBufferView>, flushRenderer?: boolean, noDataConversion?: boolean, x?: number, y?: number): Promise<ArrayBufferView>;
        /** @internal */
        _readTexturePixelsSync(texture: InternalTexture, width: number, height: number, faceIndex?: number, level?: number, buffer?: Nullable<ArrayBufferView>, flushRenderer?: boolean, noDataConversion?: boolean, x?: number, y?: number): ArrayBufferView;
    }
}
/**
 * Allocate a typed array depending on a texture type. Optionally can copy existing data in the buffer.
 * @param type type of the texture
 * @param sizeOrDstBuffer size of the array OR an existing buffer that will be used as the destination of the copy (if copyBuffer is provided)
 * @param sizeInBytes true if the size of the array is given in bytes, false if it is the number of elements of the array
 * @param copyBuffer if provided, buffer to copy into the destination buffer (either a newly allocated buffer if sizeOrDstBuffer is a number or use sizeOrDstBuffer as the destination buffer otherwise)
 * @returns the allocated buffer or sizeOrDstBuffer if the latter is an ArrayBuffer
 */
export function allocateAndCopyTypedBuffer(type: number, sizeOrDstBuffer: number | ArrayBuffer, sizeInBytes?: boolean, copyBuffer?: ArrayBuffer): ArrayBufferView;

}
declare module "babylonjs/Engines/Extensions/engine.renderTarget" {
import { InternalTexture } from "babylonjs/Materials/Textures/internalTexture";
import { RenderTargetCreationOptions, DepthTextureCreationOptions, TextureSize } from "babylonjs/Materials/Textures/textureCreationOptions";
import { Nullable } from "babylonjs/types";
import { RenderTargetWrapper } from "babylonjs/Engines/renderTargetWrapper";
/**
 * Type used to define a texture size (either with a number or with a rect width and height)
 * @deprecated please use TextureSize instead
 */
export type RenderTargetTextureSize = TextureSize;
module "babylonjs/Engines/thinEngine" {
    interface ThinEngine {
        /**
         * Creates a new render target texture
         * @param size defines the size of the texture
         * @param options defines the options used to create the texture
         * @returns a new render target wrapper ready to render texture
         */
        createRenderTargetTexture(size: TextureSize, options: boolean | RenderTargetCreationOptions): RenderTargetWrapper;
        /**
         * Creates a depth stencil texture.
         * This is only available in WebGL 2 or with the depth texture extension available.
         * @param size The size of face edge in the texture.
         * @param options The options defining the texture.
         * @param rtWrapper The render target wrapper for which the depth/stencil texture must be created
         * @returns The texture
         */
        createDepthStencilTexture(size: TextureSize, options: DepthTextureCreationOptions, rtWrapper: RenderTargetWrapper): InternalTexture;
        /**
         * Updates the sample count of a render target texture
         * @see https://doc.babylonjs.com/setup/support/webGL2#multisample-render-targets
         * @param rtWrapper defines the render target wrapper to update
         * @param samples defines the sample count to set
         * @returns the effective sample count (could be 0 if multisample render targets are not supported)
         */
        updateRenderTargetTextureSampleCount(rtWrapper: Nullable<RenderTargetWrapper>, samples: number): number;
        /** @internal */
        _createDepthStencilTexture(size: TextureSize, options: DepthTextureCreationOptions, rtWrapper: RenderTargetWrapper): InternalTexture;
        /** @internal */
        _createHardwareRenderTargetWrapper(isMulti: boolean, isCube: boolean, size: TextureSize): RenderTargetWrapper;
    }
}

}
declare module "babylonjs/Engines/Extensions/engine.renderTargetCube" {
import { RenderTargetWrapper } from "babylonjs/Engines/renderTargetWrapper";
import { RenderTargetCreationOptions } from "babylonjs/Materials/Textures/textureCreationOptions";
module "babylonjs/Engines/thinEngine" {
    interface ThinEngine {
        /**
         * Creates a new render target cube wrapper
         * @param size defines the size of the texture
         * @param options defines the options used to create the texture
         * @returns a new render target cube wrapper
         */
        createRenderTargetCubeTexture(size: number, options?: RenderTargetCreationOptions): RenderTargetWrapper;
    }
}

}
declare module "babylonjs/Engines/Extensions/engine.storageBuffer" {
import { DataBuffer } from "babylonjs/Buffers/dataBuffer";
import { DataArray, Nullable } from "babylonjs/types";
import { StorageBuffer } from "babylonjs/Buffers/storageBuffer";
module "babylonjs/Engines/thinEngine" {
    interface ThinEngine {
        /**
         * Creates a storage buffer
         * @param data the data for the storage buffer or the size of the buffer
         * @param creationFlags flags to use when creating the buffer (see Constants.BUFFER_CREATIONFLAG_XXX). The BUFFER_CREATIONFLAG_STORAGE flag will be automatically added
         * @returns the new buffer
         */
        createStorageBuffer(data: DataArray | number, creationFlags: number): DataBuffer;
        /**
         * Updates a storage buffer
         * @param buffer the storage buffer to update
         * @param data the data used to update the storage buffer
         * @param byteOffset the byte offset of the data
         * @param byteLength the byte length of the data
         */
        updateStorageBuffer(buffer: DataBuffer, data: DataArray, byteOffset?: number, byteLength?: number): void;
        /**
         * Read data from a storage buffer
         * @param storageBuffer The storage buffer to read from
         * @param offset The offset in the storage buffer to start reading from (default: 0)
         * @param size  The number of bytes to read from the storage buffer (default: capacity of the buffer)
         * @param buffer The buffer to write the data we have read from the storage buffer to (optional)
         * @returns If not undefined, returns the (promise) buffer (as provided by the 4th parameter) filled with the data, else it returns a (promise) Uint8Array with the data read from the storage buffer
         */
        readFromStorageBuffer(storageBuffer: DataBuffer, offset?: number, size?: number, buffer?: ArrayBufferView): Promise<ArrayBufferView>;
        /**
         * Sets a storage buffer in the shader
         * @param name Defines the name of the storage buffer as defined in the shader
         * @param buffer Defines the value to give to the uniform
         */
        setStorageBuffer(name: string, buffer: Nullable<StorageBuffer>): void;
    }
}
export {};

}
declare module "babylonjs/Engines/Extensions/engine.textureSampler" {
import { Nullable } from "babylonjs/types";
import { TextureSampler } from "babylonjs/Materials/Textures/textureSampler";
module "babylonjs/Engines/thinEngine" {
    interface ThinEngine {
        /**
         * Sets a texture sampler to the according uniform.
         * @param name The name of the uniform in the effect
         * @param sampler The sampler to apply
         */
        setTextureSampler(name: string, sampler: Nullable<TextureSampler>): void;
    }
}
export {};

}
declare module "babylonjs/Engines/Extensions/engine.textureSelector" {
import { Nullable } from "babylonjs/types";
module "babylonjs/Engines/engine" {
    interface Engine {
        /** @internal */
        _excludedCompressedTextures: string[];
        /** @internal */
        _textureFormatInUse: string;
        /**
         * Gets the list of texture formats supported
         */
        readonly texturesSupported: Array<string>;
        /**
         * Gets the texture format in use
         */
        readonly textureFormatInUse: Nullable<string>;
        /**
         * Set the compressed texture extensions or file names to skip.
         *
         * @param skippedFiles defines the list of those texture files you want to skip
         * Example: [".dds", ".env", "myfile.png"]
         */
        setCompressedTextureExclusions(skippedFiles: Array<string>): void;
        /**
         * Set the compressed texture format to use, based on the formats you have, and the formats
         * supported by the hardware / browser.
         *
         * Khronos Texture Container (.ktx) files are used to support this.  This format has the
         * advantage of being specifically designed for OpenGL.  Header elements directly correspond
         * to API arguments needed to compressed textures.  This puts the burden on the container
         * generator to house the arcane code for determining these for current & future formats.
         *
         * for description see https://www.khronos.org/opengles/sdk/tools/KTX/
         * for file layout see https://www.khronos.org/opengles/sdk/tools/KTX/file_format_spec/
         *
         * Note: The result of this call is not taken into account when a texture is base64.
         *
         * @param formatsAvailable defines the list of those format families you have created
         * on your server.  Syntax: '-' + format family + '.ktx'.  (Case and order do not matter.)
         *
         * Current families are astc, dxt, pvrtc, etc2, & etc1.
         * @returns The extension selected.
         */
        setTextureFormatToUse(formatsAvailable: Array<string>): Nullable<string>;
    }
}

}
declare module "babylonjs/Engines/Extensions/engine.transformFeedback" {
import { Nullable } from "babylonjs/types";
import { DataBuffer } from "babylonjs/Buffers/dataBuffer";
/** @internal */
export var _forceTransformFeedbackToBundle: boolean;
module "babylonjs/Engines/engine" {
    interface Engine {
        /**
         * Creates a webGL transform feedback object
         * Please makes sure to check webGLVersion property to check if you are running webGL 2+
         * @returns the webGL transform feedback object
         */
        createTransformFeedback(): WebGLTransformFeedback;
        /**
         * Delete a webGL transform feedback object
         * @param value defines the webGL transform feedback object to delete
         */
        deleteTransformFeedback(value: WebGLTransformFeedback): void;
        /**
         * Bind a webGL transform feedback object to the webgl context
         * @param value defines the webGL transform feedback object to bind
         */
        bindTransformFeedback(value: Nullable<WebGLTransformFeedback>): void;
        /**
         * Begins a transform feedback operation
         * @param usePoints defines if points or triangles must be used
         */
        beginTransformFeedback(usePoints: boolean): void;
        /**
         * Ends a transform feedback operation
         */
        endTransformFeedback(): void;
        /**
         * Specify the varyings to use with transform feedback
         * @param program defines the associated webGL program
         * @param value defines the list of strings representing the varying names
         */
        setTranformFeedbackVaryings(program: WebGLProgram, value: string[]): void;
        /**
         * Bind a webGL buffer for a transform feedback operation
         * @param value defines the webGL buffer to bind
         */
        bindTransformFeedbackBuffer(value: Nullable<DataBuffer>): void;
    }
}

}
declare module "babylonjs/Engines/Extensions/engine.uniformBuffer" {
import { FloatArray, Nullable } from "babylonjs/types";
import { DataBuffer } from "babylonjs/Buffers/dataBuffer";
import { IPipelineContext } from "babylonjs/Engines/IPipelineContext";
module "babylonjs/Engines/thinEngine" {
    interface ThinEngine {
        /**
         * Create an uniform buffer
         * @see https://doc.babylonjs.com/setup/support/webGL2#uniform-buffer-objets
         * @param elements defines the content of the uniform buffer
         * @returns the webGL uniform buffer
         */
        createUniformBuffer(elements: FloatArray): DataBuffer;
        /**
         * Create a dynamic uniform buffer
         * @see https://doc.babylonjs.com/setup/support/webGL2#uniform-buffer-objets
         * @param elements defines the content of the uniform buffer
         * @returns the webGL uniform buffer
         */
        createDynamicUniformBuffer(elements: FloatArray): DataBuffer;
        /**
         * Update an existing uniform buffer
         * @see https://doc.babylonjs.com/setup/support/webGL2#uniform-buffer-objets
         * @param uniformBuffer defines the target uniform buffer
         * @param elements defines the content to update
         * @param offset defines the offset in the uniform buffer where update should start
         * @param count defines the size of the data to update
         */
        updateUniformBuffer(uniformBuffer: DataBuffer, elements: FloatArray, offset?: number, count?: number): void;
        /**
         * Bind an uniform buffer to the current webGL context
         * @param buffer defines the buffer to bind
         */
        bindUniformBuffer(buffer: Nullable<DataBuffer>): void;
        /**
         * Bind a buffer to the current webGL context at a given location
         * @param buffer defines the buffer to bind
         * @param location defines the index where to bind the buffer
         * @param name Name of the uniform variable to bind
         */
        bindUniformBufferBase(buffer: DataBuffer, location: number, name: string): void;
        /**
         * Bind a specific block at a given index in a specific shader program
         * @param pipelineContext defines the pipeline context to use
         * @param blockName defines the block name
         * @param index defines the index where to bind the block
         */
        bindUniformBlock(pipelineContext: IPipelineContext, blockName: string, index: number): void;
    }
}

}
declare module "babylonjs/Engines/Extensions/engine.videoTexture" {
import { InternalTexture } from "babylonjs/Materials/Textures/internalTexture";
import { Nullable } from "babylonjs/types";
import { ExternalTexture } from "babylonjs/Materials/Textures/externalTexture";
module "babylonjs/Engines/thinEngine" {
    interface ThinEngine {
        /**
         * Update a video texture
         * @param texture defines the texture to update
         * @param video defines the video element to use
         * @param invertY defines if data must be stored with Y axis inverted
         */
        updateVideoTexture(texture: Nullable<InternalTexture>, video: HTMLVideoElement | Nullable<ExternalTexture>, invertY: boolean): void;
    }
}

}
declare module "babylonjs/Engines/Extensions/engine.views" {
import { Camera } from "babylonjs/Cameras/camera";
import { Nullable } from "babylonjs/types";
import { Observable } from "babylonjs/Misc/observable";
/**
 * Class used to define an additional view for the engine
 * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/multiCanvas
 */
export class EngineView {
    /**
     * A randomly generated unique id
     */
    readonly id: string;
    /** Defines the canvas where to render the view */
    target: HTMLCanvasElement;
    /** Defines an optional camera used to render the view (will use active camera else) */
    camera?: Camera;
    /** Indicates if the destination view canvas should be cleared before copying the parent canvas. Can help if the scene clear color has alpha < 1 */
    clearBeforeCopy?: boolean;
    /** Indicates if the view is enabled (true by default) */
    enabled: boolean;
    /** Defines a custom function to handle canvas size changes. (the canvas to render into is provided to the callback) */
    customResize?: (canvas: HTMLCanvasElement) => void;
}
module "babylonjs/Engines/engine" {
    interface Engine {
        /** @internal */
        _inputElement: Nullable<HTMLElement>;
        /**
         * Gets or sets the  HTML element to use for attaching events
         */
        inputElement: Nullable<HTMLElement>;
        /**
         * Observable to handle when a change to inputElement occurs
         * @internal
         */
        _onEngineViewChanged?: () => void;
        /**
         * Will be triggered before the view renders
         */
        readonly onBeforeViewRenderObservable: Observable<EngineView>;
        /**
         * Will be triggered after the view rendered
         */
        readonly onAfterViewRenderObservable: Observable<EngineView>;
        /**
         * Gets the current engine view
         * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/multiCanvas
         */
        activeView: Nullable<EngineView>;
        /** Gets or sets the list of views */
        views: EngineView[];
        /**
         * Register a new child canvas
         * @param canvas defines the canvas to register
         * @param camera defines an optional camera to use with this canvas (it will overwrite the scene.camera for this view)
         * @param clearBeforeCopy Indicates if the destination view canvas should be cleared before copying the parent canvas. Can help if the scene clear color has alpha < 1
         * @returns the associated view
         */
        registerView(canvas: HTMLCanvasElement, camera?: Camera, clearBeforeCopy?: boolean): EngineView;
        /**
         * Remove a registered child canvas
         * @param canvas defines the canvas to remove
         * @returns the current engine
         */
        unRegisterView(canvas: HTMLCanvasElement): Engine;
        /**
         * @internal
         */
        _renderViewStep(view: EngineView): boolean;
    }
}

}
declare module "babylonjs/Engines/Extensions/engine.webVR" {
import { Nullable } from "babylonjs/types";
import { Size } from "babylonjs/Maths/math.size";
import { Observable } from "babylonjs/Misc/observable";
import { WebVROptions } from "babylonjs/Cameras/VR/webVRCamera";
/**
 * Interface used to define additional presentation attributes
 */
export interface IVRPresentationAttributes {
    /**
     * Defines a boolean indicating that we want to get 72hz mode on Oculus Browser (default is off eg. 60hz)
     */
    highRefreshRate: boolean;
    /**
     * Enables foveation in VR to improve perf. 0 none, 1 low, 2 medium, 3 high (Default is 1)
     */
    foveationLevel: number;
}
module "babylonjs/Engines/engine" {
    interface Engine {
        /** @internal */
        _vrDisplay: any;
        /** @internal */
        _vrSupported: boolean;
        /** @internal */
        _oldSize: Size;
        /** @internal */
        _oldHardwareScaleFactor: number;
        /** @internal */
        _vrExclusivePointerMode: boolean;
        /** @internal */
        _webVRInitPromise: Promise<IDisplayChangedEventArgs>;
        /** @internal */
        _onVRDisplayPointerRestricted: () => void;
        /** @internal */
        _onVRDisplayPointerUnrestricted: () => void;
        /** @internal */
        _onVrDisplayConnect: Nullable<(display: any) => void>;
        /** @internal */
        _onVrDisplayDisconnect: Nullable<() => void>;
        /** @internal */
        _onVrDisplayPresentChange: Nullable<() => void>;
        /**
         * Observable signaled when VR display mode changes
         */
        onVRDisplayChangedObservable: Observable<IDisplayChangedEventArgs>;
        /**
         * Observable signaled when VR request present is complete
         */
        onVRRequestPresentComplete: Observable<boolean>;
        /**
         * Observable signaled when VR request present starts
         */
        onVRRequestPresentStart: Observable<Engine>;
        /**
         * Gets a boolean indicating that the engine is currently in VR exclusive mode for the pointers
         * @see https://docs.microsoft.com/en-us/microsoft-edge/webvr/essentials#mouse-input
         */
        isInVRExclusivePointerMode: boolean;
        /**
         * Gets a boolean indicating if a webVR device was detected
         * @returns true if a webVR device was detected
         */
        isVRDevicePresent(): boolean;
        /**
         * Gets the current webVR device
         * @returns the current webVR device (or null)
         */
        getVRDevice(): any;
        /**
         * Initializes a webVR display and starts listening to display change events
         * The onVRDisplayChangedObservable will be notified upon these changes
         * @returns A promise containing a VRDisplay and if vr is supported
         */
        initWebVRAsync(): Promise<IDisplayChangedEventArgs>;
        /** @internal */
        _getVRDisplaysAsync(): Promise<IDisplayChangedEventArgs>;
        /**
         * Gets or sets the presentation attributes used to configure VR rendering
         */
        vrPresentationAttributes?: IVRPresentationAttributes;
        /**
         * Call this function to switch to webVR mode
         * Will do nothing if webVR is not supported or if there is no webVR device
         * @param options the webvr options provided to the camera. mainly used for multiview
         * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/webVRCamera
         */
        enableVR(options: WebVROptions): void;
        /** @internal */
        _onVRFullScreenTriggered(): void;
    }
}

}
declare module "babylonjs/Engines/Extensions/index" {
export * from "babylonjs/Engines/Extensions/engine.alpha";
export * from "babylonjs/Engines/Extensions/engine.debugging";
export * from "babylonjs/Engines/Extensions/engine.query";
export * from "babylonjs/Engines/Extensions/engine.transformFeedback";
export * from "babylonjs/Engines/Extensions/engine.multiview";
export * from "babylonjs/Engines/Extensions/engine.rawTexture";
export * from "babylonjs/Engines/Extensions/engine.dynamicTexture";
export * from "babylonjs/Engines/Extensions/engine.externalTexture";
export * from "babylonjs/Engines/Extensions/engine.videoTexture";
export * from "babylonjs/Engines/Extensions/engine.multiRender";
export * from "babylonjs/Engines/Extensions/engine.cubeTexture";
export * from "babylonjs/Engines/Extensions/engine.renderTarget";
export * from "babylonjs/Engines/Extensions/engine.renderTargetCube";
export * from "babylonjs/Engines/Extensions/engine.textureSampler";
export * from "babylonjs/Engines/Extensions/engine.webVR";
export * from "babylonjs/Engines/Extensions/engine.uniformBuffer";
export * from "babylonjs/Engines/Extensions/engine.dynamicBuffer";
export * from "babylonjs/Engines/Extensions/engine.views";
export * from "babylonjs/Engines/Extensions/engine.readTexture";
export * from "babylonjs/Engines/Extensions/engine.computeShader";
export * from "babylonjs/Engines/Extensions/engine.storageBuffer";
import "babylonjs/Engines/Extensions/engine.textureSelector";
export * from "babylonjs/Engines/Extensions/engine.textureSelector";

}
declare module "babylonjs/Engines/ICanvas" {
/**
 * Class used to abstract a canvas
 */
export interface ICanvas {
    /**
     * Canvas width.
     */
    width: number;
    /**
     * Canvas height.
     */
    height: number;
    /**
     * returns a drawing context on the canvas.
     * @param contextType context identifier.
     * @param contextAttributes context attributes.
     * @returns ICanvasRenderingContext object.
     */
    getContext(contextType: string, contextAttributes?: any): ICanvasRenderingContext;
    /**
     * returns a data URI containing a representation of the image in the format specified by the type parameter.
     * @param mime the image format.
     * @returns string containing the requested data URI.
     */
    toDataURL(mime: string): string;
}
/**
 * Class used to abstract am image to use with the canvas and its context
 */
export interface IImage {
    /**
     * onload callback.
     */
    onload: ((this: GlobalEventHandlers, ev: Event) => any) | null;
    /**
     * Error callback.
     */
    onerror: ((this: GlobalEventHandlers, ev: Event) => any) | null;
    /**
     * Image source.
     */
    src: string;
    /**
     * Image width.
     */
    readonly width: number;
    /**
     * Image height.
     */
    readonly height: number;
    /**
     * The original height of the image resource before sizing.
     */
    readonly naturalHeight: number;
    /**
     * The original width of the image resource before sizing.
     */
    readonly naturalWidth: number;
    /**
     * provides support for CORS, defining how the element handles crossorigin requests,
     * thereby enabling the configuration of the CORS requests for the element's fetched data.
     */
    crossOrigin: string | null;
    /**
     * provides support for referrer policy on xhr load request,
     * it is used to control the request header.
     */
    referrerPolicy: string;
}
/**
 * Class used to abstract a canvas gradient
 */
export interface ICanvasGradient {
    /**
     * adds a new color stop, defined by an offset and a color, to a given canvas gradient.
     * @param offset A number between 0 and 1, inclusive, representing the position of the color stop. 0 represents the start of the gradient and 1 represents the end.
     * @param color value representing the color of the stop.
     */
    addColorStop(offset: number, color: string): void;
}
/**
 * Class used to abstract a text measurement
 */
export interface ITextMetrics {
    /**
     * Text width.
     */
    readonly width: number;
    /**
     * distance (in pixels) parallel to the baseline from the alignment point given by the CanvasRenderingContext2D.textAlign
     * property to the left side of the bounding rectangle of the given text
     */
    readonly actualBoundingBoxLeft: number;
    /**
     * distance (in pixels) parallel to the baseline from the alignment point given by the CanvasRenderingContext2D.textAlign
     * property to the right side of the bounding rectangle of the given text
     */
    readonly actualBoundingBoxRight: number;
}
/**
 * Class used to abstract canvas rendering
 */
export interface ICanvasRenderingContext {
    /**
     * Defines the type of corners where two lines meet. Possible values: round, bevel, miter (default).
     */
    lineJoin: string;
    /**
     * Miter limit ratio. Default 10.
     */
    miterLimit: number;
    /**
     * Font setting. Default value 10px sans-serif.
     */
    font: string;
    /**
     * Color or style to use for the lines around shapes. Default #000 (black).
     */
    strokeStyle: string | ICanvasGradient;
    /**
     * Color or style to use inside shapes. Default #000 (black).
     */
    fillStyle: string | ICanvasGradient;
    /**
     * Alpha value that is applied to shapes and images before they are composited onto the canvas. Default 1.0 (opaque).
     */
    globalAlpha: number;
    /**
     * Color of the shadow. Default: fully-transparent black.
     */
    shadowColor: string;
    /**
     * Specifies the blurring effect. Default: 0.
     */
    shadowBlur: number;
    /**
     * Horizontal distance the shadow will be offset. Default: 0.
     */
    shadowOffsetX: number;
    /**
     * Vertical distance the shadow will be offset. Default: 0.
     */
    shadowOffsetY: number;
    /**
     * Width of lines. Default 1.0.
     */
    lineWidth: number;
    /**
     * canvas is a read-only reference to ICanvas.
     */
    readonly canvas: ICanvas;
    /**
     * Sets all pixels in the rectangle defined by starting point (x, y) and size (width, height) to transparent black, erasing any previously drawn content.
     * @param x The x-axis coordinate of the rectangle's starting point.
     * @param y The y-axis coordinate of the rectangle's starting point.
     * @param width The rectangle's width. Positive values are to the right, and negative to the left.
     * @param height The rectangle's height. Positive values are down, and negative are up.
     */
    clearRect(x: number, y: number, width: number, height: number): void;
    /**
     * Saves the current drawing style state using a stack so you can revert any change you make to it using restore().
     */
    save(): void;
    /**
     * Restores the drawing style state to the last element on the 'state stack' saved by save().
     */
    restore(): void;
    /**
     * Draws a filled rectangle at (x, y) position whose size is determined by width and height.
     * @param x The x-axis coordinate of the rectangle's starting point.
     * @param y The y-axis coordinate of the rectangle's starting point.
     * @param width The rectangle's width. Positive values are to the right, and negative to the left.
     * @param height The rectangle's height. Positive values are down, and negative are up.
     */
    fillRect(x: number, y: number, width: number, height: number): void;
    /**
     * Adds a scaling transformation to the canvas units by x horizontally and by y vertically.
     * @param x Scaling factor in the horizontal direction. A negative value flips pixels across the vertical axis. A value of 1 results in no horizontal scaling.
     * @param y Scaling factor in the vertical direction. A negative value flips pixels across the horizontal axis. A value of 1 results in no vertical scaling.
     */
    scale(x: number, y: number): void;
    /**
     * Adds a rotation to the transformation matrix. The angle argument represents a clockwise rotation angle and is expressed in radians.
     * @param angle The rotation angle, clockwise in radians. You can use degree * Math.PI / 180 to calculate a radian from a degree.
     */
    rotate(angle: number): void;
    /**
     * Adds a translation transformation by moving the canvas and its origin x horizontally and y vertically on the grid.
     * @param x Distance to move in the horizontal direction. Positive values are to the right, and negative to the left.
     * @param y Distance to move in the vertical direction. Positive values are down, and negative are up.
     */
    translate(x: number, y: number): void;
    /**
     * Paints a rectangle which has a starting point at (x, y) and has a w width and an h height onto the canvas, using the current stroke style.
     * @param x The x-axis coordinate of the rectangle's starting point.
     * @param y The y-axis coordinate of the rectangle's starting point.
     * @param width The rectangle's width. Positive values are to the right, and negative to the left.
     * @param height The rectangle's height. Positive values are down, and negative are up.
     */
    strokeRect(x: number, y: number, width: number, height: number): void;
    /**
     * Creates a path for a rectangle at position (x, y) with a size that is determined by width and height.
     * @param x The x-axis coordinate of the rectangle's starting point.
     * @param y The y-axis coordinate of the rectangle's starting point.
     * @param width The rectangle's width. Positive values are to the right, and negative to the left.
     * @param height The rectangle's height. Positive values are down, and negative are up.
     */
    rect(x: number, y: number, width: number, height: number): void;
    /**
     * Creates a clipping path from the current sub-paths. Everything drawn after clip() is called appears inside the clipping path only.
     */
    clip(): void;
    /**
     * Paints data from the given ImageData object onto the bitmap. If a dirty rectangle is provided, only the pixels from that rectangle are painted.
     * @param imageData An ImageData object containing the array of pixel values.
     * @param dx Horizontal position (x coordinate) at which to place the image data in the destination canvas.
     * @param dy Vertical position (y coordinate) at which to place the image data in the destination canvas.
     */
    putImageData(imageData: ImageData, dx: number, dy: number): void;
    /**
     * Adds a circular arc to the current path.
     * @param x The horizontal coordinate of the arc's center.
     * @param y The vertical coordinate of the arc's center.
     * @param radius The arc's radius. Must be positive.
     * @param startAngle The angle at which the arc starts in radians, measured from the positive x-axis.
     * @param endAngle The angle at which the arc ends in radians, measured from the positive x-axis.
     * @param anticlockwise An optional Boolean. If true, draws the arc counter-clockwise between the start and end angles. The default is false (clockwise).
     */
    arc(x: number, y: number, radius: number, startAngle: number, endAngle: number, anticlockwise?: boolean): void;
    /**
     * Starts a new path by emptying the list of sub-paths. Call this method when you want to create a new path.
     */
    beginPath(): void;
    /**
     * Causes the point of the pen to move back to the start of the current sub-path. It tries to draw a straight line from the current point to the start.
     * If the shape has already been closed or has only one point, this function does nothing.
     */
    closePath(): void;
    /**
     * Moves the starting point of a new sub-path to the (x, y) coordinates.
     * @param x The x-axis (horizontal) coordinate of the point.
     * @param y The y-axis (vertical) coordinate of the point.
     */
    moveTo(x: number, y: number): void;
    /**
     * Connects the last point in the current sub-path to the specified (x, y) coordinates with a straight line.
     * @param x The x-axis coordinate of the line's end point.
     * @param y The y-axis coordinate of the line's end point.
     */
    lineTo(x: number, y: number): void;
    /**
     * Adds a quadratic Bézier curve to the current path.
     * @param cpx The x-axis coordinate of the control point.
     * @param cpy The y-axis coordinate of the control point.
     * @param x The x-axis coordinate of the end point.
     * @param y The y-axis coordinate of the end point.
     */
    quadraticCurveTo(cpx: number, cpy: number, x: number, y: number): void;
    /**
     * Returns a TextMetrics object.
     * @param text The text String to measure.
     * @returns ITextMetrics A ITextMetrics object.
     */
    measureText(text: string): ITextMetrics;
    /**
     * Strokes the current sub-paths with the current stroke style.
     */
    stroke(): void;
    /**
     * Fills the current sub-paths with the current fill style.
     */
    fill(): void;
    /**
     * Draws the specified image. This method is available in multiple formats, providing a great deal of flexibility in its use.
     * @param image An element to draw into the context.
     * @param sx The x-axis coordinate of the top left corner of the sub-rectangle of the source image to draw into the destination context.
     * @param sy The y-axis coordinate of the top left corner of the sub-rectangle of the source image to draw into the destination context.
     * @param sWidth The width of the sub-rectangle of the source image to draw into the destination context. If not specified, the entire rectangle from the coordinates specified by sx and sy to the bottom-right corner of the image is used.
     * @param sHeight The height of the sub-rectangle of the source image to draw into the destination context.
     * @param dx The x-axis coordinate in the destination canvas at which to place the top-left corner of the source image.
     * @param dy The y-axis coordinate in the destination canvas at which to place the top-left corner of the source image.
     * @param dWidth The width to draw the image in the destination canvas. This allows scaling of the drawn image. If not specified, the image is not scaled in width when drawn.
     * @param dHeight The height to draw the image in the destination canvas. This allows scaling of the drawn image. If not specified, the image is not scaled in height when drawn.
     */
    drawImage(image: any, sx: number, sy: number, sWidth: number, sHeight: number, dx: number, dy: number, dWidth: number, dHeight: number): void;
    /**
     * Draws the specified image. This method is available in multiple formats, providing a great deal of flexibility in its use.
     * @param image An element to draw into the context.
     * @param dx The x-axis coordinate in the destination canvas at which to place the top-left corner of the source image.
     * @param dy The y-axis coordinate in the destination canvas at which to place the top-left corner of the source image.
     * @param dWidth The width to draw the image in the destination canvas. This allows scaling of the drawn image. If not specified, the image is not scaled in width when drawn.
     * @param dHeight The height to draw the image in the destination canvas. This allows scaling of the drawn image. If not specified, the image is not scaled in height when drawn.
     */
    drawImage(image: any, dx: number, dy: number, dWidth: number, dHeight: number): void;
    /**
     * Draws the specified image. This method is available in multiple formats, providing a great deal of flexibility in its use.
     * @param image An element to draw into the context.
     * @param dx The x-axis coordinate in the destination canvas at which to place the top-left corner of the source image.
     * @param dy The y-axis coordinate in the destination canvas at which to place the top-left corner of the source image.
     */
    drawImage(image: any, dx: number, dy: number): void;
    /**
     * Returns an ImageData object representing the underlying pixel data for the area of the canvas denoted by the rectangle which starts at (sx, sy) and has an sw width and sh height.
     * @param sx The x-axis coordinate of the top-left corner of the rectangle from which the ImageData will be extracted.
     * @param sy The y-axis coordinate of the top-left corner of the rectangle from which the ImageData will be extracted.
     * @param sw The width of the rectangle from which the ImageData will be extracted. Positive values are to the right, and negative to the left.
     * @param sh The height of the rectangle from which the ImageData will be extracted. Positive values are down, and negative are up.
     * @returns ImageData An ImageData object containing the image data for the rectangle of the canvas specified.
     */
    getImageData(sx: number, sy: number, sw: number, sh: number): ImageData;
    /**
     * Sets the current line dash pattern.
     * @param segments An Array of numbers that specify distances to alternately draw a line and a gap (in coordinate space units).
     */
    setLineDash(segments: Array<number>): void;
    /**
     * Draws (fills) a given text at the given (x, y) position.
     * @param text A String specifying the text string to render into the context. The text is rendered using the settings specified by font, textAlign, textBaseline, and direction.
     * @param x The x-axis coordinate of the point at which to begin drawing the text, in pixels.
     * @param y The y-axis coordinate of the baseline on which to begin drawing the text, in pixels.
     * @param maxWidth The maximum number of pixels wide the text may be once rendered. If not specified, there is no limit to the width of the text.
     */
    fillText(text: string, x: number, y: number, maxWidth?: number): void;
    /**
     * Draws (strokes) a given text at the given (x, y) position.
     * @param text A String specifying the text string to render into the context. The text is rendered using the settings specified by font, textAlign, textBaseline, and direction.
     * @param x The x-axis coordinate of the point at which to begin drawing the text, in pixels.
     * @param y The y-axis coordinate of the baseline on which to begin drawing the text, in pixels.
     * @param maxWidth The maximum number of pixels wide the text may be once rendered. If not specified, there is no limit to the width of the text.
     */
    strokeText(text: string, x: number, y: number, maxWidth?: number): void;
    /**
     * Creates a linear gradient along the line given by the coordinates represented by the parameters.
     * @param x0 The x-axis coordinate of the start point.
     * @param y0 The y-axis coordinate of the start point.
     * @param x1 The x-axis coordinate of the end point.
     * @param y1 The y-axis coordinate of the end point.
     * @returns ICanvasGradient A linear ICanvasGradient initialized with the specified line.
     */
    createLinearGradient(x0: number, y0: number, x1: number, y1: number): ICanvasGradient;
    /**
     * Creates a linear gradient along the line given by the coordinates represented by the parameters.
     * @param x0 The x-axis coordinate of the start circle.
     * @param y0 The y-axis coordinate of the start circle.
     * @param r0 The radius of the start circle. Must be non-negative and finite.
     * @param x1 The x-axis coordinate of the end point.
     * @param y1 The y-axis coordinate of the end point.
     * @param r1 The radius of the end circle. Must be non-negative and finite.
     * @returns ICanvasGradient A linear ICanvasGradient initialized with the two specified circles.
     */
    createRadialGradient(x0: number, y0: number, r0: number, x1: number, y1: number, r1: number): ICanvasGradient;
    /**
     * Resets the current transform to matrix composed with a, b, c, d, e, f.
     * @param a Horizontal scaling. A value of 1 results in no scaling.
     * @param b Vertical skewing.
     * @param c Horizontal skewing.
     * @param d Vertical scaling. A value of 1 results in no scaling.
     * @param e Horizontal translation (moving).
     * @param f Vertical translation (moving).
     */
    setTransform(a: number, b: number, c: number, d: number, e: number, f: number): void;
}

}
declare module "babylonjs/Engines/IDrawContext" {
/** @internal */
export interface IDrawContext {
    uniqueId: number;
    useInstancing: boolean;
    reset(): void;
    dispose(): void;
}

}
declare module "babylonjs/Engines/IMaterialContext" {
/** @internal */
export interface IMaterialContext {
    uniqueId: number;
    reset(): void;
}

}
declare module "babylonjs/Engines/index" {
export * from "babylonjs/Engines/constants";
export * from "babylonjs/Engines/engineCapabilities";
export * from "babylonjs/Engines/instancingAttributeInfo";
export * from "babylonjs/Engines/thinEngine";
export * from "babylonjs/Engines/engine";
export * from "babylonjs/Engines/engineStore";
export * from "babylonjs/Engines/nullEngine";
export * from "babylonjs/Engines/Extensions/index";
export * from "babylonjs/Engines/Native/index";
export * from "babylonjs/Engines/WebGPU/Extensions/index";
export * from "babylonjs/Engines/IPipelineContext";
export * from "babylonjs/Engines/ICanvas";
export * from "babylonjs/Engines/WebGL/webGLPipelineContext";
export * from "babylonjs/Engines/WebGL/webGLHardwareTexture";
export * from "babylonjs/Engines/WebGPU/webgpuConstants";
export * from "babylonjs/Engines/webgpuEngine";
export * from "babylonjs/Engines/WebGPU/webgpuCacheRenderPipeline";
export * from "babylonjs/Engines/WebGPU/webgpuCacheRenderPipelineTree";
export * from "babylonjs/Engines/WebGPU/webgpuCacheBindGroups";
export * from "babylonjs/Engines/WebGPU/webgpuCacheSampler";
export * from "babylonjs/Engines/WebGPU/webgpuDrawContext";
export * from "babylonjs/Engines/WebGPU/webgpuTintWASM";
export * from "babylonjs/Engines/WebGL/webGL2ShaderProcessors";
export * from "babylonjs/Engines/nativeEngine";
export * from "babylonjs/Engines/Processors/shaderCodeInliner";
export * from "babylonjs/Engines/performanceConfigurator";
export * from "babylonjs/Engines/engineFeatures";
export * from "babylonjs/Engines/engineFactory";
export * from "babylonjs/Engines/IMaterialContext";
export * from "babylonjs/Engines/IDrawContext";
export * from "babylonjs/Engines/shaderStore";
export * from "babylonjs/Engines/renderTargetWrapper";
export * from "babylonjs/Engines/Processors/iShaderProcessor";

}
declare module "babylonjs/Engines/instancingAttributeInfo" {
/**
 * Interface for attribute information associated with buffer instantiation
 */
export interface InstancingAttributeInfo {
    /**
     * Name of the GLSL attribute
     * if attribute index is not specified, this is used to retrieve the index from the effect
     */
    attributeName: string;
    /**
     * Index/offset of the attribute in the vertex shader
     * if not specified, this will be computes from the name.
     */
    index?: number;
    /**
     * size of the attribute, 1, 2, 3 or 4
     */
    attributeSize: number;
    /**
     * Offset of the data in the Vertex Buffer acting as the instancing buffer
     */
    offset: number;
    /**
     * Modifies the rate at which generic vertex attributes advance when rendering multiple instances
     * default to 1
     */
    divisor?: number;
    /**
     * type of the attribute, gl.BYTE, gl.UNSIGNED_BYTE, gl.SHORT, gl.UNSIGNED_SHORT, gl.FIXED, gl.FLOAT.
     * default is FLOAT
     */
    attributeType?: number;
    /**
     * normalization of fixed-point data. behavior unclear, use FALSE, default is FALSE
     */
    normalized?: boolean;
}

}
declare module "babylonjs/Engines/IPipelineContext" {
import { Nullable } from "babylonjs/types";
import { Effect } from "babylonjs/Materials/effect";
import { IMatrixLike, IVector2Like, IVector3Like, IVector4Like, IColor3Like, IColor4Like, IQuaternionLike } from "babylonjs/Maths/math.like";
/**
 * Class used to store and describe the pipeline context associated with an effect
 */
export interface IPipelineContext {
    /**
     * Gets a boolean indicating that this pipeline context is supporting asynchronous creating
     */
    isAsync: boolean;
    /**
     * Gets a boolean indicating that the context is ready to be used (like shaders / pipelines are compiled and ready for instance)
     */
    isReady: boolean;
    /** @internal */
    _name?: string;
    /** @internal */
    _getVertexShaderCode(): string | null;
    /** @internal */
    _getFragmentShaderCode(): string | null;
    /** @internal */
    _handlesSpectorRebuildCallback(onCompiled: (compiledObject: any) => void): void;
    /** @internal */
    _fillEffectInformation(effect: Effect, uniformBuffersNames: {
        [key: string]: number;
    }, uniformsNames: string[], uniforms: {
        [key: string]: Nullable<WebGLUniformLocation>;
    }, samplerList: string[], samplers: {
        [key: string]: number;
    }, attributesNames: string[], attributes: number[]): void;
    /** Releases the resources associated with the pipeline. */
    dispose(): void;
    /**
     * Sets an integer value on a uniform variable.
     * @param uniformName Name of the variable.
     * @param value Value to be set.
     */
    setInt(uniformName: string, value: number): void;
    /**
     * Sets an int2 value on a uniform variable.
     * @param uniformName Name of the variable.
     * @param x First int in int2.
     * @param y Second int in int2.
     */
    setInt2(uniformName: string, x: number, y: number): void;
    /**
     * Sets an int3 value on a uniform variable.
     * @param uniformName Name of the variable.
     * @param x First int in int3.
     * @param y Second int in int3.
     * @param z Third int in int3.
     */
    setInt3(uniformName: string, x: number, y: number, z: number): void;
    /**
     * Sets an int4 value on a uniform variable.
     * @param uniformName Name of the variable.
     * @param x First int in int4.
     * @param y Second int in int4.
     * @param z Third int in int4.
     * @param w Fourth int in int4.
     */
    setInt4(uniformName: string, x: number, y: number, z: number, w: number): void;
    /**
     * Sets an int array on a uniform variable.
     * @param uniformName Name of the variable.
     * @param array array to be set.
     */
    setIntArray(uniformName: string, array: Int32Array): void;
    /**
     * Sets an int array 2 on a uniform variable. (Array is specified as single array eg. [1,2,3,4] will result in [[1,2],[3,4]] in the shader)
     * @param uniformName Name of the variable.
     * @param array array to be set.
     */
    setIntArray2(uniformName: string, array: Int32Array): void;
    /**
     * Sets an int array 3 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6] will result in [[1,2,3],[4,5,6]] in the shader)
     * @param uniformName Name of the variable.
     * @param array array to be set.
     */
    setIntArray3(uniformName: string, array: Int32Array): void;
    /**
     * Sets an int array 4 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6,7,8] will result in [[1,2,3,4],[5,6,7,8]] in the shader)
     * @param uniformName Name of the variable.
     * @param array array to be set.
     */
    setIntArray4(uniformName: string, array: Int32Array): void;
    /**
     * Sets an unsigned integer value on a uniform variable.
     * @param uniformName Name of the variable.
     * @param value Value to be set.
     */
    setUInt(uniformName: string, value: number): void;
    /**
     * Sets an unsigned int2 value on a uniform variable.
     * @param uniformName Name of the variable.
     * @param x First unsigned int in uint2.
     * @param y Second unsigned int in uint2.
     */
    setUInt2(uniformName: string, x: number, y: number): void;
    /**
     * Sets an unsigned int3 value on a uniform variable.
     * @param uniformName Name of the variable.
     * @param x First unsigned int in uint3.
     * @param y Second unsigned int in uint3.
     * @param z Third unsigned int in uint3.
     */
    setUInt3(uniformName: string, x: number, y: number, z: number): void;
    /**
     * Sets an unsigned int4 value on a uniform variable.
     * @param uniformName Name of the variable.
     * @param x First unsigned int in uint4.
     * @param y Second unsigned int in uint4.
     * @param z Third unsigned int in uint4.
     * @param w Fourth unsigned int in uint4.
     */
    setUInt4(uniformName: string, x: number, y: number, z: number, w: number): void;
    /**
     * Sets an unsigned int array on a uniform variable.
     * @param uniformName Name of the variable.
     * @param array array to be set.
     */
    setUIntArray(uniformName: string, array: Uint32Array): void;
    /**
     * Sets an unsigned int array 2 on a uniform variable. (Array is specified as single array eg. [1,2,3,4] will result in [[1,2],[3,4]] in the shader)
     * @param uniformName Name of the variable.
     * @param array array to be set.
     */
    setUIntArray2(uniformName: string, array: Uint32Array): void;
    /**
     * Sets an unsigned int array 3 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6] will result in [[1,2,3],[4,5,6]] in the shader)
     * @param uniformName Name of the variable.
     * @param array array to be set.
     */
    setUIntArray3(uniformName: string, array: Uint32Array): void;
    /**
     * Sets an unsigned int array 4 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6,7,8] will result in [[1,2,3,4],[5,6,7,8]] in the shader)
     * @param uniformName Name of the variable.
     * @param array array to be set.
     */
    setUIntArray4(uniformName: string, array: Uint32Array): void;
    /**
     * Sets an array on a uniform variable.
     * @param uniformName Name of the variable.
     * @param array array to be set.
     */
    setArray(uniformName: string, array: number[] | Float32Array): void;
    /**
     * Sets an array 2 on a uniform variable. (Array is specified as single array eg. [1,2,3,4] will result in [[1,2],[3,4]] in the shader)
     * @param uniformName Name of the variable.
     * @param array array to be set.
     */
    setArray2(uniformName: string, array: number[] | Float32Array): void;
    /**
     * Sets an array 3 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6] will result in [[1,2,3],[4,5,6]] in the shader)
     * @param uniformName Name of the variable.
     * @param array array to be set.
     */
    setArray3(uniformName: string, array: number[] | Float32Array): void;
    /**
     * Sets an array 4 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6,7,8] will result in [[1,2,3,4],[5,6,7,8]] in the shader)
     * @param uniformName Name of the variable.
     * @param array array to be set.
     */
    setArray4(uniformName: string, array: number[] | Float32Array): void;
    /**
     * Sets matrices on a uniform variable.
     * @param uniformName Name of the variable.
     * @param matrices matrices to be set.
     */
    setMatrices(uniformName: string, matrices: Float32Array): void;
    /**
     * Sets matrix on a uniform variable.
     * @param uniformName Name of the variable.
     * @param matrix matrix to be set.
     */
    setMatrix(uniformName: string, matrix: IMatrixLike): void;
    /**
     * Sets a 3x3 matrix on a uniform variable. (Specified as [1,2,3,4,5,6,7,8,9] will result in [1,2,3][4,5,6][7,8,9] matrix)
     * @param uniformName Name of the variable.
     * @param matrix matrix to be set.
     */
    setMatrix3x3(uniformName: string, matrix: Float32Array): void;
    /**
     * Sets a 2x2 matrix on a uniform variable. (Specified as [1,2,3,4] will result in [1,2][3,4] matrix)
     * @param uniformName Name of the variable.
     * @param matrix matrix to be set.
     */
    setMatrix2x2(uniformName: string, matrix: Float32Array): void;
    /**
     * Sets a float on a uniform variable.
     * @param uniformName Name of the variable.
     * @param value value to be set.
     */
    setFloat(uniformName: string, value: number): void;
    /**
     * Sets a Vector2 on a uniform variable.
     * @param uniformName Name of the variable.
     * @param vector2 vector2 to be set.
     */
    setVector2(uniformName: string, vector2: IVector2Like): void;
    /**
     * Sets a float2 on a uniform variable.
     * @param uniformName Name of the variable.
     * @param x First float in float2.
     * @param y Second float in float2.
     */
    setFloat2(uniformName: string, x: number, y: number): void;
    /**
     * Sets a Vector3 on a uniform variable.
     * @param uniformName Name of the variable.
     * @param vector3 Value to be set.
     */
    setVector3(uniformName: string, vector3: IVector3Like): void;
    /**
     * Sets a float3 on a uniform variable.
     * @param uniformName Name of the variable.
     * @param x First float in float3.
     * @param y Second float in float3.
     * @param z Third float in float3.
     */
    setFloat3(uniformName: string, x: number, y: number, z: number): void;
    /**
     * Sets a Vector4 on a uniform variable.
     * @param uniformName Name of the variable.
     * @param vector4 Value to be set.
     */
    setVector4(uniformName: string, vector4: IVector4Like): void;
    /**
     * Sets a Quaternion on a uniform variable.
     * @param uniformName Name of the variable.
     * @param quaternion Value to be set.
     */
    setQuaternion(uniformName: string, quaternion: IQuaternionLike): void;
    /**
     * Sets a float4 on a uniform variable.
     * @param uniformName Name of the variable.
     * @param x First float in float4.
     * @param y Second float in float4.
     * @param z Third float in float4.
     * @param w Fourth float in float4.
     */
    setFloat4(uniformName: string, x: number, y: number, z: number, w: number): void;
    /**
     * Sets a Color3 on a uniform variable.
     * @param uniformName Name of the variable.
     * @param color3 Value to be set.
     */
    setColor3(uniformName: string, color3: IColor3Like): void;
    /**
     * Sets a Color4 on a uniform variable.
     * @param uniformName Name of the variable.
     * @param color3 Value to be set.
     * @param alpha Alpha value to be set.
     */
    setColor4(uniformName: string, color3: IColor3Like, alpha: number): void;
    /**
     * Sets a Color4 on a uniform variable
     * @param uniformName defines the name of the variable
     * @param color4 defines the value to be set
     */
    setDirectColor4(uniformName: string, color4: IColor4Like): void;
}

}
declare module "babylonjs/Engines/Native/index" {
export * from "babylonjs/Engines/Native/nativeDataStream";
export * from "babylonjs/Engines/Native/validatedNativeDataStream";

}
declare module "babylonjs/Engines/Native/nativeDataStream" {
/** @internal */
export type NativeData = Uint32Array;
/** @internal */
export class NativeDataStream {
    private readonly _uint32s;
    private readonly _int32s;
    private readonly _float32s;
    private readonly _length;
    private _position;
    private readonly _nativeDataStream;
    private static readonly DEFAULT_BUFFER_SIZE;
    constructor();
    writeUint32(value: number): void;
    writeInt32(value: number): void;
    writeFloat32(value: number): void;
    writeUint32Array(values: Uint32Array): void;
    writeInt32Array(values: Int32Array): void;
    writeFloat32Array(values: Float32Array): void;
    writeNativeData(handle: NativeData): void;
    writeBoolean(value: boolean): void;
    private _flushIfNecessary;
    private _flush;
}

}
declare module "babylonjs/Engines/Native/nativeHardwareTexture" {
import { HardwareTextureWrapper } from "babylonjs/Materials/Textures/hardwareTextureWrapper";
import { Nullable } from "babylonjs/types";
import { INativeEngine, NativeTexture } from "babylonjs/Engines/Native/nativeInterfaces";
/** @internal */
export class NativeHardwareTexture implements HardwareTextureWrapper {
    private readonly _engine;
    private _nativeTexture;
    get underlyingResource(): Nullable<NativeTexture>;
    constructor(existingTexture: NativeTexture, engine: INativeEngine);
    setUsage(): void;
    set(hardwareTexture: NativeTexture): void;
    reset(): void;
    release(): void;
}

}
declare module "babylonjs/Engines/Native/nativeInterfaces" {
import { DeviceType } from "babylonjs/DeviceInput/InputDevices/deviceEnums";
import { IDeviceInputSystem } from "babylonjs/DeviceInput/inputInterfaces";
import { InternalTexture } from "babylonjs/Materials/Textures/internalTexture";
import { Nullable } from "babylonjs/types";
import { ICanvas, IImage } from "babylonjs/Engines/ICanvas";
import { NativeData, NativeDataStream } from "babylonjs/Engines/Native/nativeDataStream";
export type NativeTexture = NativeData;
export type NativeFramebuffer = NativeData;
export type NativeVertexArrayObject = NativeData;
export type NativeProgram = NativeData;
export type NativeUniform = NativeData;
/** @internal */
export interface INativeEngine {
    dispose(): void;
    requestAnimationFrame(callback: () => void): void;
    createVertexArray(): NativeData;
    createIndexBuffer(bytes: ArrayBuffer, byteOffset: number, byteLength: number, is32Bits: boolean, dynamic: boolean): NativeData;
    recordIndexBuffer(vertexArray: NativeData, indexBuffer: NativeData): void;
    updateDynamicIndexBuffer(buffer: NativeData, bytes: ArrayBuffer, byteOffset: number, byteLength: number, startIndex: number): void;
    createVertexBuffer(bytes: ArrayBuffer, byteOffset: number, byteLength: number, dynamic: boolean): NativeData;
    recordVertexBuffer(vertexArray: NativeData, vertexBuffer: NativeData, location: number, byteOffset: number, byteStride: number, numElements: number, type: number, normalized: boolean, instanceDivisor: number): void;
    updateDynamicVertexBuffer(vertexBuffer: NativeData, bytes: ArrayBuffer, byteOffset: number, byteLength: number): void;
    createProgram(vertexShader: string, fragmentShader: string): NativeProgram;
    getUniforms(shaderProgram: NativeProgram, uniformsNames: string[]): WebGLUniformLocation[];
    getAttributes(shaderProgram: NativeProgram, attributeNames: string[]): number[];
    createTexture(): NativeTexture;
    initializeTexture(texture: NativeTexture, width: number, height: number, hasMips: boolean, format: number, renderTarget: boolean, srgb: boolean): void;
    loadTexture(texture: NativeTexture, data: ArrayBufferView, generateMips: boolean, invertY: boolean, srgb: boolean, onSuccess: () => void, onError: () => void): void;
    loadRawTexture(texture: NativeTexture, data: ArrayBufferView, width: number, height: number, format: number, generateMips: boolean, invertY: boolean): void;
    loadRawTexture2DArray(texture: NativeTexture, data: Nullable<ArrayBufferView>, width: number, height: number, depth: number, format: number, generateMipMaps: boolean, invertY: boolean): void;
    loadCubeTexture(texture: NativeTexture, data: Array<ArrayBufferView>, generateMips: boolean, invertY: boolean, srgb: boolean, onSuccess: () => void, onError: () => void): void;
    loadCubeTextureWithMips(texture: NativeTexture, data: Array<Array<ArrayBufferView>>, invertY: boolean, srgb: boolean, onSuccess: () => void, onError: () => void): void;
    getTextureWidth(texture: NativeTexture): number;
    getTextureHeight(texture: NativeTexture): number;
    copyTexture(desination: NativeTexture, source: NativeTexture): void;
    deleteTexture(texture: NativeTexture): void;
    readTexture(texture: NativeTexture, mipLevel: number, x: number, y: number, width: number, height: number, buffer: Nullable<ArrayBuffer>, bufferOffset: number, bufferLength: number): Promise<ArrayBuffer>;
    createImageBitmap(data: ArrayBufferView | IImage): ImageBitmap;
    resizeImageBitmap(image: ImageBitmap, bufferWidth: number, bufferHeight: number): Uint8Array;
    createFrameBuffer(texture: Nullable<NativeTexture>, width: number, height: number, generateStencilBuffer: boolean, generateDepthBuffer: boolean): NativeFramebuffer;
    getRenderWidth(): number;
    getRenderHeight(): number;
    getHardwareScalingLevel(): number;
    setHardwareScalingLevel(level: number): void;
    setViewPort(x: number, y: number, width: number, height: number): void;
    setCommandDataStream(dataStream: NativeDataStream): void;
    submitCommands(): void;
}
/** @internal */
interface INativeEngineConstructor {
    prototype: INativeEngine;
    new (): INativeEngine;
    readonly PROTOCOL_VERSION: number;
    readonly CAPS_LIMITS_MAX_TEXTURE_SIZE: number;
    readonly CAPS_LIMITS_MAX_TEXTURE_LAYERS: number;
    readonly TEXTURE_NEAREST_NEAREST: number;
    readonly TEXTURE_LINEAR_LINEAR: number;
    readonly TEXTURE_LINEAR_LINEAR_MIPLINEAR: number;
    readonly TEXTURE_NEAREST_NEAREST_MIPNEAREST: number;
    readonly TEXTURE_NEAREST_LINEAR_MIPNEAREST: number;
    readonly TEXTURE_NEAREST_LINEAR_MIPLINEAR: number;
    readonly TEXTURE_NEAREST_LINEAR: number;
    readonly TEXTURE_NEAREST_NEAREST_MIPLINEAR: number;
    readonly TEXTURE_LINEAR_NEAREST_MIPNEAREST: number;
    readonly TEXTURE_LINEAR_NEAREST_MIPLINEAR: number;
    readonly TEXTURE_LINEAR_LINEAR_MIPNEAREST: number;
    readonly TEXTURE_LINEAR_NEAREST: number;
    readonly DEPTH_TEST_LESS: number;
    readonly DEPTH_TEST_LEQUAL: number;
    readonly DEPTH_TEST_EQUAL: number;
    readonly DEPTH_TEST_GEQUAL: number;
    readonly DEPTH_TEST_GREATER: number;
    readonly DEPTH_TEST_NOTEQUAL: number;
    readonly DEPTH_TEST_NEVER: number;
    readonly DEPTH_TEST_ALWAYS: number;
    readonly ADDRESS_MODE_WRAP: number;
    readonly ADDRESS_MODE_MIRROR: number;
    readonly ADDRESS_MODE_CLAMP: number;
    readonly ADDRESS_MODE_BORDER: number;
    readonly ADDRESS_MODE_MIRROR_ONCE: number;
    readonly TEXTURE_FORMAT_RGB8: number;
    readonly TEXTURE_FORMAT_RGBA8: number;
    readonly TEXTURE_FORMAT_RGBA16F: number;
    readonly TEXTURE_FORMAT_RGBA32F: number;
    readonly ATTRIB_TYPE_INT8: number;
    readonly ATTRIB_TYPE_UINT8: number;
    readonly ATTRIB_TYPE_INT16: number;
    readonly ATTRIB_TYPE_UINT16: number;
    readonly ATTRIB_TYPE_FLOAT: number;
    readonly ALPHA_DISABLE: number;
    readonly ALPHA_ADD: number;
    readonly ALPHA_COMBINE: number;
    readonly ALPHA_SUBTRACT: number;
    readonly ALPHA_MULTIPLY: number;
    readonly ALPHA_MAXIMIZED: number;
    readonly ALPHA_ONEONE: number;
    readonly ALPHA_PREMULTIPLIED: number;
    readonly ALPHA_PREMULTIPLIED_PORTERDUFF: number;
    readonly ALPHA_INTERPOLATE: number;
    readonly ALPHA_SCREENMODE: number;
    readonly STENCIL_TEST_LESS: number;
    readonly STENCIL_TEST_LEQUAL: number;
    readonly STENCIL_TEST_EQUAL: number;
    readonly STENCIL_TEST_GEQUAL: number;
    readonly STENCIL_TEST_GREATER: number;
    readonly STENCIL_TEST_NOTEQUAL: number;
    readonly STENCIL_TEST_NEVER: number;
    readonly STENCIL_TEST_ALWAYS: number;
    readonly STENCIL_OP_FAIL_S_ZERO: number;
    readonly STENCIL_OP_FAIL_S_KEEP: number;
    readonly STENCIL_OP_FAIL_S_REPLACE: number;
    readonly STENCIL_OP_FAIL_S_INCR: number;
    readonly STENCIL_OP_FAIL_S_INCRSAT: number;
    readonly STENCIL_OP_FAIL_S_DECR: number;
    readonly STENCIL_OP_FAIL_S_DECRSAT: number;
    readonly STENCIL_OP_FAIL_S_INVERT: number;
    readonly STENCIL_OP_FAIL_Z_ZERO: number;
    readonly STENCIL_OP_FAIL_Z_KEEP: number;
    readonly STENCIL_OP_FAIL_Z_REPLACE: number;
    readonly STENCIL_OP_FAIL_Z_INCR: number;
    readonly STENCIL_OP_FAIL_Z_INCRSAT: number;
    readonly STENCIL_OP_FAIL_Z_DECR: number;
    readonly STENCIL_OP_FAIL_Z_DECRSAT: number;
    readonly STENCIL_OP_FAIL_Z_INVERT: number;
    readonly STENCIL_OP_PASS_Z_ZERO: number;
    readonly STENCIL_OP_PASS_Z_KEEP: number;
    readonly STENCIL_OP_PASS_Z_REPLACE: number;
    readonly STENCIL_OP_PASS_Z_INCR: number;
    readonly STENCIL_OP_PASS_Z_INCRSAT: number;
    readonly STENCIL_OP_PASS_Z_DECR: number;
    readonly STENCIL_OP_PASS_Z_DECRSAT: number;
    readonly STENCIL_OP_PASS_Z_INVERT: number;
    readonly COMMAND_DELETEVERTEXARRAY: NativeData;
    readonly COMMAND_DELETEINDEXBUFFER: NativeData;
    readonly COMMAND_DELETEVERTEXBUFFER: NativeData;
    readonly COMMAND_SETPROGRAM: NativeData;
    readonly COMMAND_SETMATRIX: NativeData;
    readonly COMMAND_SETMATRIX3X3: NativeData;
    readonly COMMAND_SETMATRIX2X2: NativeData;
    readonly COMMAND_SETMATRICES: NativeData;
    readonly COMMAND_SETINT: NativeData;
    readonly COMMAND_SETINTARRAY: NativeData;
    readonly COMMAND_SETINTARRAY2: NativeData;
    readonly COMMAND_SETINTARRAY3: NativeData;
    readonly COMMAND_SETINTARRAY4: NativeData;
    readonly COMMAND_SETFLOATARRAY: NativeData;
    readonly COMMAND_SETFLOATARRAY2: NativeData;
    readonly COMMAND_SETFLOATARRAY3: NativeData;
    readonly COMMAND_SETFLOATARRAY4: NativeData;
    readonly COMMAND_SETTEXTURESAMPLING: NativeData;
    readonly COMMAND_SETTEXTUREWRAPMODE: NativeData;
    readonly COMMAND_SETTEXTUREANISOTROPICLEVEL: NativeData;
    readonly COMMAND_SETTEXTURE: NativeData;
    readonly COMMAND_BINDVERTEXARRAY: NativeData;
    readonly COMMAND_SETSTATE: NativeData;
    readonly COMMAND_DELETEPROGRAM: NativeData;
    readonly COMMAND_SETZOFFSET: NativeData;
    readonly COMMAND_SETZOFFSETUNITS: NativeData;
    readonly COMMAND_SETDEPTHTEST: NativeData;
    readonly COMMAND_SETDEPTHWRITE: NativeData;
    readonly COMMAND_SETCOLORWRITE: NativeData;
    readonly COMMAND_SETBLENDMODE: NativeData;
    readonly COMMAND_SETFLOAT: NativeData;
    readonly COMMAND_SETFLOAT2: NativeData;
    readonly COMMAND_SETFLOAT3: NativeData;
    readonly COMMAND_SETFLOAT4: NativeData;
    readonly COMMAND_BINDFRAMEBUFFER: NativeData;
    readonly COMMAND_UNBINDFRAMEBUFFER: NativeData;
    readonly COMMAND_DELETEFRAMEBUFFER: NativeData;
    readonly COMMAND_DRAWINDEXED: NativeData;
    readonly COMMAND_DRAW: NativeData;
    readonly COMMAND_CLEAR: NativeData;
    readonly COMMAND_SETSTENCIL: NativeData;
    readonly COMMAND_SETVIEWPORT: NativeData;
}
/** @internal */
export interface INativeCamera {
    createVideo(constraints: MediaTrackConstraints): any;
    updateVideoTexture(texture: Nullable<InternalTexture>, video: HTMLVideoElement, invertY: boolean): void;
}
/** @internal */
interface INativeCameraConstructor {
    prototype: INativeCamera;
    new (): INativeCamera;
}
/** @internal */
interface INativeCanvasConstructor {
    prototype: ICanvas;
    new (): ICanvas;
    loadTTFAsync(fontName: string, buffer: ArrayBuffer): void;
}
/** @internal */
interface INativeImageConstructor {
    prototype: IImage;
    new (): IImage;
}
/** @internal */
interface IDeviceInputSystemConstructor {
    prototype: IDeviceInputSystem;
    new (onDeviceConnected: (deviceType: DeviceType, deviceSlot: number) => void, onDeviceDisconnected: (deviceType: DeviceType, deviceSlot: number) => void, onInputChanged: (deviceType: DeviceType, deviceSlot: number, inputIndex: number, currentState: number) => void): IDeviceInputSystem;
}
/** @internal */
export interface INativeDataStream {
    writeBuffer(buffer: ArrayBuffer, length: number): void;
}
/** @internal */
interface INativeDataStreamConstructor {
    prototype: INativeDataStream;
    new (requestFlushCallback: () => void): INativeDataStream;
    readonly VALIDATION_ENABLED: boolean;
    readonly VALIDATION_UINT_32: number;
    readonly VALIDATION_INT_32: number;
    readonly VALIDATION_FLOAT_32: number;
    readonly VALIDATION_UINT_32_ARRAY: number;
    readonly VALIDATION_INT_32_ARRAY: number;
    readonly VALIDATION_FLOAT_32_ARRAY: number;
    readonly VALIDATION_NATIVE_DATA: number;
    readonly VALIDATION_BOOLEAN: number;
}
/** @internal */
export interface INative {
    Engine: INativeEngineConstructor;
    Camera: INativeCameraConstructor;
    Canvas: INativeCanvasConstructor;
    Image: INativeImageConstructor;
    XMLHttpRequest: any;
    DeviceInputSystem: IDeviceInputSystemConstructor;
    NativeDataStream: INativeDataStreamConstructor;
}
export {};

}
declare module "babylonjs/Engines/Native/nativePipelineContext" {
import { Nullable } from "babylonjs/types";
import { Effect } from "babylonjs/Materials/effect";
import { IMatrixLike, IVector2Like, IVector3Like, IVector4Like, IColor3Like, IColor4Like, IQuaternionLike } from "babylonjs/Maths/math.like";
import { IPipelineContext } from "babylonjs/Engines/IPipelineContext";
import { NativeEngine } from "babylonjs/Engines/nativeEngine";
export class NativePipelineContext implements IPipelineContext {
    isAsync: boolean;
    isReady: boolean;
    _getVertexShaderCode(): string | null;
    _getFragmentShaderCode(): string | null;
    _handlesSpectorRebuildCallback(onCompiled: (compiledObject: any) => void): void;
    nativeProgram: any;
    private _engine;
    private _valueCache;
    private _uniforms;
    constructor(engine: NativeEngine);
    _fillEffectInformation(effect: Effect, uniformBuffersNames: {
        [key: string]: number;
    }, uniformsNames: string[], uniforms: {
        [key: string]: Nullable<WebGLUniformLocation>;
    }, samplerList: string[], samplers: {
        [key: string]: number;
    }, attributesNames: string[], attributes: number[]): void;
    /**
     * Release all associated resources.
     **/
    dispose(): void;
    /**
     * @internal
     */
    _cacheMatrix(uniformName: string, matrix: IMatrixLike): boolean;
    /**
     * @internal
     */
    _cacheFloat2(uniformName: string, x: number, y: number): boolean;
    /**
     * @internal
     */
    _cacheFloat3(uniformName: string, x: number, y: number, z: number): boolean;
    /**
     * @internal
     */
    _cacheFloat4(uniformName: string, x: number, y: number, z: number, w: number): boolean;
    /**
     * Sets an integer value on a uniform variable.
     * @param uniformName Name of the variable.
     * @param value Value to be set.
     */
    setInt(uniformName: string, value: number): void;
    /**
     * Sets a int2 on a uniform variable.
     * @param uniformName Name of the variable.
     * @param x First int in int2.
     * @param y Second int in int2.
     */
    setInt2(uniformName: string, x: number, y: number): void;
    /**
     * Sets a int3 on a uniform variable.
     * @param uniformName Name of the variable.
     * @param x First int in int3.
     * @param y Second int in int3.
     * @param z Third int in int3.
     */
    setInt3(uniformName: string, x: number, y: number, z: number): void;
    /**
     * Sets a int4 on a uniform variable.
     * @param uniformName Name of the variable.
     * @param x First int in int4.
     * @param y Second int in int4.
     * @param z Third int in int4.
     * @param w Fourth int in int4.
     */
    setInt4(uniformName: string, x: number, y: number, z: number, w: number): void;
    /**
     * Sets an int array on a uniform variable.
     * @param uniformName Name of the variable.
     * @param array array to be set.
     */
    setIntArray(uniformName: string, array: Int32Array): void;
    /**
     * Sets an int array 2 on a uniform variable. (Array is specified as single array eg. [1,2,3,4] will result in [[1,2],[3,4]] in the shader)
     * @param uniformName Name of the variable.
     * @param array array to be set.
     */
    setIntArray2(uniformName: string, array: Int32Array): void;
    /**
     * Sets an int array 3 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6] will result in [[1,2,3],[4,5,6]] in the shader)
     * @param uniformName Name of the variable.
     * @param array array to be set.
     */
    setIntArray3(uniformName: string, array: Int32Array): void;
    /**
     * Sets an int array 4 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6,7,8] will result in [[1,2,3,4],[5,6,7,8]] in the shader)
     * @param uniformName Name of the variable.
     * @param array array to be set.
     */
    setIntArray4(uniformName: string, array: Int32Array): void;
    /**
     * Sets an unsigned integer value on a uniform variable.
     * @param uniformName Name of the variable.
     * @param value Value to be set.
     */
    setUInt(uniformName: string, value: number): void;
    /**
     * Sets a unsigned int2 on a uniform variable.
     * @param uniformName Name of the variable.
     * @param x First unsigned int in uint2.
     * @param y Second unsigned int in uint2.
     */
    setUInt2(uniformName: string, x: number, y: number): void;
    /**
     * Sets a unsigned int3 on a uniform variable.
     * @param uniformName Name of the variable.
     * @param x First unsigned int in uint3.
     * @param y Second unsigned int in uint3.
     * @param z Third unsigned int in uint3.
     */
    setUInt3(uniformName: string, x: number, y: number, z: number): void;
    /**
     * Sets a unsigned int4 on a uniform variable.
     * @param uniformName Name of the variable.
     * @param x First unsigned int in uint4.
     * @param y Second unsigned int in uint4.
     * @param z Third unsigned int in uint4.
     * @param w Fourth unsigned int in uint4.
     */
    setUInt4(uniformName: string, x: number, y: number, z: number, w: number): void;
    /**
     * Sets an unsigned int array on a uniform variable.
     * @param uniformName Name of the variable.
     * @param array array to be set.
     */
    setUIntArray(uniformName: string, array: Uint32Array): void;
    /**
     * Sets an unsigned int array 2 on a uniform variable. (Array is specified as single array eg. [1,2,3,4] will result in [[1,2],[3,4]] in the shader)
     * @param uniformName Name of the variable.
     * @param array array to be set.
     */
    setUIntArray2(uniformName: string, array: Uint32Array): void;
    /**
     * Sets an unsigned int array 3 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6] will result in [[1,2,3],[4,5,6]] in the shader)
     * @param uniformName Name of the variable.
     * @param array array to be set.
     */
    setUIntArray3(uniformName: string, array: Uint32Array): void;
    /**
     * Sets an unsigned int array 4 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6,7,8] will result in [[1,2,3,4],[5,6,7,8]] in the shader)
     * @param uniformName Name of the variable.
     * @param array array to be set.
     */
    setUIntArray4(uniformName: string, array: Uint32Array): void;
    /**
     * Sets an float array on a uniform variable.
     * @param uniformName Name of the variable.
     * @param array array to be set.
     */
    setFloatArray(uniformName: string, array: Float32Array): void;
    /**
     * Sets an float array 2 on a uniform variable. (Array is specified as single array eg. [1,2,3,4] will result in [[1,2],[3,4]] in the shader)
     * @param uniformName Name of the variable.
     * @param array array to be set.
     */
    setFloatArray2(uniformName: string, array: Float32Array): void;
    /**
     * Sets an float array 3 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6] will result in [[1,2,3],[4,5,6]] in the shader)
     * @param uniformName Name of the variable.
     * @param array array to be set.
     */
    setFloatArray3(uniformName: string, array: Float32Array): void;
    /**
     * Sets an float array 4 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6,7,8] will result in [[1,2,3,4],[5,6,7,8]] in the shader)
     * @param uniformName Name of the variable.
     * @param array array to be set.
     */
    setFloatArray4(uniformName: string, array: Float32Array): void;
    /**
     * Sets an array on a uniform variable.
     * @param uniformName Name of the variable.
     * @param array array to be set.
     */
    setArray(uniformName: string, array: number[]): void;
    /**
     * Sets an array 2 on a uniform variable. (Array is specified as single array eg. [1,2,3,4] will result in [[1,2],[3,4]] in the shader)
     * @param uniformName Name of the variable.
     * @param array array to be set.
     */
    setArray2(uniformName: string, array: number[]): void;
    /**
     * Sets an array 3 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6] will result in [[1,2,3],[4,5,6]] in the shader)
     * @param uniformName Name of the variable.
     * @param array array to be set.
     * @returns this effect.
     */
    setArray3(uniformName: string, array: number[]): void;
    /**
     * Sets an array 4 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6,7,8] will result in [[1,2,3,4],[5,6,7,8]] in the shader)
     * @param uniformName Name of the variable.
     * @param array array to be set.
     */
    setArray4(uniformName: string, array: number[]): void;
    /**
     * Sets matrices on a uniform variable.
     * @param uniformName Name of the variable.
     * @param matrices matrices to be set.
     */
    setMatrices(uniformName: string, matrices: Float32Array): void;
    /**
     * Sets matrix on a uniform variable.
     * @param uniformName Name of the variable.
     * @param matrix matrix to be set.
     */
    setMatrix(uniformName: string, matrix: IMatrixLike): void;
    /**
     * Sets a 3x3 matrix on a uniform variable. (Specified as [1,2,3,4,5,6,7,8,9] will result in [1,2,3][4,5,6][7,8,9] matrix)
     * @param uniformName Name of the variable.
     * @param matrix matrix to be set.
     */
    setMatrix3x3(uniformName: string, matrix: Float32Array): void;
    /**
     * Sets a 2x2 matrix on a uniform variable. (Specified as [1,2,3,4] will result in [1,2][3,4] matrix)
     * @param uniformName Name of the variable.
     * @param matrix matrix to be set.
     */
    setMatrix2x2(uniformName: string, matrix: Float32Array): void;
    /**
     * Sets a float on a uniform variable.
     * @param uniformName Name of the variable.
     * @param value value to be set.
     * @returns this effect.
     */
    setFloat(uniformName: string, value: number): void;
    /**
     * Sets a boolean on a uniform variable.
     * @param uniformName Name of the variable.
     * @param bool value to be set.
     */
    setBool(uniformName: string, bool: boolean): void;
    /**
     * Sets a Vector2 on a uniform variable.
     * @param uniformName Name of the variable.
     * @param vector2 vector2 to be set.
     */
    setVector2(uniformName: string, vector2: IVector2Like): void;
    /**
     * Sets a float2 on a uniform variable.
     * @param uniformName Name of the variable.
     * @param x First float in float2.
     * @param y Second float in float2.
     */
    setFloat2(uniformName: string, x: number, y: number): void;
    /**
     * Sets a Vector3 on a uniform variable.
     * @param uniformName Name of the variable.
     * @param vector3 Value to be set.
     */
    setVector3(uniformName: string, vector3: IVector3Like): void;
    /**
     * Sets a float3 on a uniform variable.
     * @param uniformName Name of the variable.
     * @param x First float in float3.
     * @param y Second float in float3.
     * @param z Third float in float3.
     */
    setFloat3(uniformName: string, x: number, y: number, z: number): void;
    /**
     * Sets a Vector4 on a uniform variable.
     * @param uniformName Name of the variable.
     * @param vector4 Value to be set.
     */
    setVector4(uniformName: string, vector4: IVector4Like): void;
    /**
     * Sets a Quaternion on a uniform variable.
     * @param uniformName Name of the variable.
     * @param quaternion Value to be set.
     */
    setQuaternion(uniformName: string, quaternion: IQuaternionLike): void;
    /**
     * Sets a float4 on a uniform variable.
     * @param uniformName Name of the variable.
     * @param x First float in float4.
     * @param y Second float in float4.
     * @param z Third float in float4.
     * @param w Fourth float in float4.
     * @returns this effect.
     */
    setFloat4(uniformName: string, x: number, y: number, z: number, w: number): void;
    /**
     * Sets a Color3 on a uniform variable.
     * @param uniformName Name of the variable.
     * @param color3 Value to be set.
     */
    setColor3(uniformName: string, color3: IColor3Like): void;
    /**
     * Sets a Color4 on a uniform variable.
     * @param uniformName Name of the variable.
     * @param color3 Value to be set.
     * @param alpha Alpha value to be set.
     */
    setColor4(uniformName: string, color3: IColor3Like, alpha: number): void;
    /**
     * Sets a Color4 on a uniform variable
     * @param uniformName defines the name of the variable
     * @param color4 defines the value to be set
     */
    setDirectColor4(uniformName: string, color4: IColor4Like): void;
}

}
declare module "babylonjs/Engines/Native/nativeRenderTargetWrapper" {
import { Nullable } from "babylonjs/types";
import { TextureSize } from "babylonjs/Materials/Textures/textureCreationOptions";
import { RenderTargetWrapper } from "babylonjs/Engines/renderTargetWrapper";
import { NativeEngine } from "babylonjs/Engines/nativeEngine";
import { NativeFramebuffer } from "babylonjs/Engines/Native/nativeInterfaces";
export class NativeRenderTargetWrapper extends RenderTargetWrapper {
    readonly _engine: NativeEngine;
    private __framebuffer;
    private __framebufferDepthStencil;
    get _framebuffer(): Nullable<NativeFramebuffer>;
    set _framebuffer(framebuffer: Nullable<NativeFramebuffer>);
    get _framebufferDepthStencil(): Nullable<NativeFramebuffer>;
    set _framebufferDepthStencil(framebufferDepthStencil: Nullable<NativeFramebuffer>);
    constructor(isMulti: boolean, isCube: boolean, size: TextureSize, engine: NativeEngine);
    dispose(disposeOnlyFramebuffers?: boolean): void;
}

}
declare module "babylonjs/Engines/Native/validatedNativeDataStream" {
import { NativeData } from "babylonjs/Engines/Native/nativeDataStream";
import { NativeDataStream } from "babylonjs/Engines/Native/nativeDataStream";
export class ValidatedNativeDataStream extends NativeDataStream {
    constructor();
    writeUint32(value: number): void;
    writeInt32(value: number): void;
    writeFloat32(value: number): void;
    writeUint32Array(values: Uint32Array): void;
    writeInt32Array(values: Int32Array): void;
    writeFloat32Array(values: Float32Array): void;
    writeNativeData(handle: NativeData): void;
    writeBoolean(value: boolean): void;
}

}
declare module "babylonjs/Engines/nativeEngine" {
import { Nullable, IndicesArray, DataArray } from "babylonjs/types";
import { Engine } from "babylonjs/Engines/engine";
import { VertexBuffer } from "babylonjs/Buffers/buffer";
import { InternalTexture, InternalTextureSource } from "babylonjs/Materials/Textures/internalTexture";
import { BaseTexture } from "babylonjs/Materials/Textures/baseTexture";
import { Effect } from "babylonjs/Materials/effect";
import { DataBuffer } from "babylonjs/Buffers/dataBuffer";
import { Scene } from "babylonjs/scene";
import { RenderTargetCreationOptions, TextureSize, DepthTextureCreationOptions, InternalTextureCreationOptions } from "babylonjs/Materials/Textures/textureCreationOptions";
import { IPipelineContext } from "babylonjs/Engines/IPipelineContext";
import { IColor3Like, IColor4Like, IViewportLike } from "babylonjs/Maths/math.like";
import { ISceneLike } from "babylonjs/Engines/thinEngine";
import { IMaterialContext } from "babylonjs/Engines/IMaterialContext";
import { IDrawContext } from "babylonjs/Engines/IDrawContext";
import { ICanvas, IImage } from "babylonjs/Engines/ICanvas";
import { IStencilState } from "babylonjs/States/IStencilState";
import { RenderTargetWrapper } from "babylonjs/Engines/renderTargetWrapper";
import { NativeData } from "babylonjs/Engines/Native/nativeDataStream";
import { NativeDataStream } from "babylonjs/Engines/Native/nativeDataStream";
import { INative, NativeFramebuffer } from "babylonjs/Engines/Native/nativeInterfaces";
import { HardwareTextureWrapper } from "babylonjs/Materials/Textures/hardwareTextureWrapper";
/**
 * Returns _native only after it has been defined by BabylonNative.
 * @internal
 */
export function AcquireNativeObjectAsync(): Promise<INative>;
/**
 * Registers a constructor on the _native object. See NativeXRFrame for an example.
 * @internal
 */
export function RegisterNativeTypeAsync<Type>(typeName: string, constructor: Type): Promise<void>;
/**
 * Container for accessors for natively-stored mesh data buffers.
 */
class NativeDataBuffer extends DataBuffer {
    /**
     * Accessor value used to identify/retrieve a natively-stored index buffer.
     */
    nativeIndexBuffer?: NativeData;
    /**
     * Accessor value used to identify/retrieve a natively-stored vertex buffer.
     */
    nativeVertexBuffer?: NativeData;
}
/**
 * Options to create the Native engine
 */
export interface NativeEngineOptions {
    /**
     * defines whether to adapt to the device's viewport characteristics (default: false)
     */
    adaptToDeviceRatio?: boolean;
}
/** @internal */
export class NativeEngine extends Engine {
    private static readonly PROTOCOL_VERSION;
    private readonly _engine;
    private readonly _camera;
    private readonly _commandBufferEncoder;
    private _boundBuffersVertexArray;
    private _currentDepthTest;
    private _stencilTest;
    private _stencilMask;
    private _stencilFunc;
    private _stencilFuncRef;
    private _stencilFuncMask;
    private _stencilOpStencilFail;
    private _stencilOpDepthFail;
    private _stencilOpStencilDepthPass;
    private _zOffset;
    private _zOffsetUnits;
    private _depthWrite;
    getHardwareScalingLevel(): number;
    setHardwareScalingLevel(level: number): void;
    constructor(options?: NativeEngineOptions);
    dispose(): void;
    /** @internal */
    static _createNativeDataStream(): NativeDataStream;
    /**
     * Can be used to override the current requestAnimationFrame requester.
     * @internal
     */
    protected _queueNewFrame(bindedRenderFunction: any, requester?: any): number;
    /**
     * Override default engine behavior.
     * @param framebuffer
     */
    _bindUnboundFramebuffer(framebuffer: Nullable<WebGLFramebuffer>): void;
    /**
     * Gets host document
     * @returns the host document object
     */
    getHostDocument(): Nullable<Document>;
    clear(color: Nullable<IColor4Like>, backBuffer: boolean, depth: boolean, stencil?: boolean): void;
    createIndexBuffer(indices: IndicesArray, updateable?: boolean): NativeDataBuffer;
    createVertexBuffer(vertices: DataArray, updateable?: boolean): NativeDataBuffer;
    protected _recordVertexArrayObject(vertexArray: any, vertexBuffers: {
        [key: string]: VertexBuffer;
    }, indexBuffer: Nullable<NativeDataBuffer>, effect: Effect, overrideVertexBuffers?: {
        [kind: string]: Nullable<VertexBuffer>;
    }): void;
    bindBuffers(vertexBuffers: {
        [key: string]: VertexBuffer;
    }, indexBuffer: Nullable<NativeDataBuffer>, effect: Effect): void;
    recordVertexArrayObject(vertexBuffers: {
        [key: string]: VertexBuffer;
    }, indexBuffer: Nullable<NativeDataBuffer>, effect: Effect, overrideVertexBuffers?: {
        [kind: string]: Nullable<VertexBuffer>;
    }): WebGLVertexArrayObject;
    private _deleteVertexArray;
    bindVertexArrayObject(vertexArray: WebGLVertexArrayObject): void;
    releaseVertexArrayObject(vertexArray: WebGLVertexArrayObject): void;
    getAttributes(pipelineContext: IPipelineContext, attributesNames: string[]): number[];
    /**
     * Draw a list of indexed primitives
     * @param fillMode defines the primitive to use
     * @param indexStart defines the starting index
     * @param indexCount defines the number of index to draw
     * @param instancesCount defines the number of instances to draw (if instantiation is enabled)
     */
    drawElementsType(fillMode: number, indexStart: number, indexCount: number, instancesCount?: number): void;
    /**
     * Draw a list of unindexed primitives
     * @param fillMode defines the primitive to use
     * @param verticesStart defines the index of first vertex to draw
     * @param verticesCount defines the count of vertices to draw
     * @param instancesCount defines the number of instances to draw (if instantiation is enabled)
     */
    drawArraysType(fillMode: number, verticesStart: number, verticesCount: number, instancesCount?: number): void;
    createPipelineContext(): IPipelineContext;
    createMaterialContext(): IMaterialContext | undefined;
    createDrawContext(): IDrawContext | undefined;
    _preparePipelineContext(pipelineContext: IPipelineContext, vertexSourceCode: string, fragmentSourceCode: string, createAsRaw: boolean, _rawVertexSourceCode: string, _rawFragmentSourceCode: string, _rebuildRebind: any, defines: Nullable<string>): void;
    /**
     * @internal
     */
    _isRenderingStateCompiled(pipelineContext: IPipelineContext): boolean;
    /**
     * @internal
     */
    _executeWhenRenderingStateIsCompiled(pipelineContext: IPipelineContext, action: () => void): void;
    createRawShaderProgram(): WebGLProgram;
    createShaderProgram(_pipelineContext: IPipelineContext, vertexCode: string, fragmentCode: string, defines: Nullable<string>): WebGLProgram;
    /**
     * Inline functions in shader code that are marked to be inlined
     * @param code code to inline
     * @returns inlined code
     */
    inlineShaderCode(code: string): string;
    protected _setProgram(program: WebGLProgram): void;
    _deletePipelineContext(pipelineContext: IPipelineContext): void;
    getUniforms(pipelineContext: IPipelineContext, uniformsNames: string[]): WebGLUniformLocation[];
    bindUniformBlock(pipelineContext: IPipelineContext, blockName: string, index: number): void;
    bindSamplers(effect: Effect): void;
    getRenderWidth(useScreen?: boolean): number;
    getRenderHeight(useScreen?: boolean): number;
    setViewport(viewport: IViewportLike, requiredWidth?: number, requiredHeight?: number): void;
    setState(culling: boolean, zOffset?: number, force?: boolean, reverseSide?: boolean, cullBackFaces?: boolean, stencil?: IStencilState, zOffsetUnits?: number): void;
    /**
     * Gets the client rect of native canvas.  Needed for InputManager.
     * @returns a client rectangle
     */
    getInputElementClientRect(): Nullable<DOMRect>;
    /**
     * Set the z offset Factor to apply to current rendering
     * @param value defines the offset to apply
     */
    setZOffset(value: number): void;
    /**
     * Gets the current value of the zOffset Factor
     * @returns the current zOffset Factor state
     */
    getZOffset(): number;
    /**
     * Set the z offset Units to apply to current rendering
     * @param value defines the offset to apply
     */
    setZOffsetUnits(value: number): void;
    /**
     * Gets the current value of the zOffset Units
     * @returns the current zOffset Units state
     */
    getZOffsetUnits(): number;
    /**
     * Enable or disable depth buffering
     * @param enable defines the state to set
     */
    setDepthBuffer(enable: boolean): void;
    /**
     * Gets a boolean indicating if depth writing is enabled
     * @returns the current depth writing state
     */
    getDepthWrite(): boolean;
    getDepthFunction(): Nullable<number>;
    setDepthFunction(depthFunc: number): void;
    /**
     * Enable or disable depth writing
     * @param enable defines the state to set
     */
    setDepthWrite(enable: boolean): void;
    /**
     * Enable or disable color writing
     * @param enable defines the state to set
     */
    setColorWrite(enable: boolean): void;
    /**
     * Gets a boolean indicating if color writing is enabled
     * @returns the current color writing state
     */
    getColorWrite(): boolean;
    private applyStencil;
    private _setStencil;
    /**
     * Enable or disable the stencil buffer
     * @param enable defines if the stencil buffer must be enabled or disabled
     */
    setStencilBuffer(enable: boolean): void;
    /**
     * Gets a boolean indicating if stencil buffer is enabled
     * @returns the current stencil buffer state
     */
    getStencilBuffer(): boolean;
    /**
     * Gets the current stencil operation when stencil passes
     * @returns a number defining stencil operation to use when stencil passes
     */
    getStencilOperationPass(): number;
    /**
     * Sets the stencil operation to use when stencil passes
     * @param operation defines the stencil operation to use when stencil passes
     */
    setStencilOperationPass(operation: number): void;
    /**
     * Sets the current stencil mask
     * @param mask defines the new stencil mask to use
     */
    setStencilMask(mask: number): void;
    /**
     * Sets the current stencil function
     * @param stencilFunc defines the new stencil function to use
     */
    setStencilFunction(stencilFunc: number): void;
    /**
     * Sets the current stencil reference
     * @param reference defines the new stencil reference to use
     */
    setStencilFunctionReference(reference: number): void;
    /**
     * Sets the current stencil mask
     * @param mask defines the new stencil mask to use
     */
    setStencilFunctionMask(mask: number): void;
    /**
     * Sets the stencil operation to use when stencil fails
     * @param operation defines the stencil operation to use when stencil fails
     */
    setStencilOperationFail(operation: number): void;
    /**
     * Sets the stencil operation to use when depth fails
     * @param operation defines the stencil operation to use when depth fails
     */
    setStencilOperationDepthFail(operation: number): void;
    /**
     * Gets the current stencil mask
     * @returns a number defining the new stencil mask to use
     */
    getStencilMask(): number;
    /**
     * Gets the current stencil function
     * @returns a number defining the stencil function to use
     */
    getStencilFunction(): number;
    /**
     * Gets the current stencil reference value
     * @returns a number defining the stencil reference value to use
     */
    getStencilFunctionReference(): number;
    /**
     * Gets the current stencil mask
     * @returns a number defining the stencil mask to use
     */
    getStencilFunctionMask(): number;
    /**
     * Gets the current stencil operation when stencil fails
     * @returns a number defining stencil operation to use when stencil fails
     */
    getStencilOperationFail(): number;
    /**
     * Gets the current stencil operation when depth fails
     * @returns a number defining stencil operation to use when depth fails
     */
    getStencilOperationDepthFail(): number;
    /**
     * Sets alpha constants used by some alpha blending modes
     * @param r defines the red component
     * @param g defines the green component
     * @param b defines the blue component
     * @param a defines the alpha component
     */
    setAlphaConstants(r: number, g: number, b: number, a: number): void;
    /**
     * Sets the current alpha mode
     * @param mode defines the mode to use (one of the BABYLON.Constants.ALPHA_XXX)
     * @param noDepthWriteChange defines if depth writing state should remains unchanged (false by default)
     * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/advanced/transparent_rendering
     */
    setAlphaMode(mode: number, noDepthWriteChange?: boolean): void;
    /**
     * Gets the current alpha mode
     * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/advanced/transparent_rendering
     * @returns the current alpha mode
     */
    getAlphaMode(): number;
    setInt(uniform: WebGLUniformLocation, int: number): boolean;
    setIntArray(uniform: WebGLUniformLocation, array: Int32Array): boolean;
    setIntArray2(uniform: WebGLUniformLocation, array: Int32Array): boolean;
    setIntArray3(uniform: WebGLUniformLocation, array: Int32Array): boolean;
    setIntArray4(uniform: WebGLUniformLocation, array: Int32Array): boolean;
    setFloatArray(uniform: WebGLUniformLocation, array: Float32Array): boolean;
    setFloatArray2(uniform: WebGLUniformLocation, array: Float32Array): boolean;
    setFloatArray3(uniform: WebGLUniformLocation, array: Float32Array): boolean;
    setFloatArray4(uniform: WebGLUniformLocation, array: Float32Array): boolean;
    setArray(uniform: WebGLUniformLocation, array: number[]): boolean;
    setArray2(uniform: WebGLUniformLocation, array: number[]): boolean;
    setArray3(uniform: WebGLUniformLocation, array: number[]): boolean;
    setArray4(uniform: WebGLUniformLocation, array: number[]): boolean;
    setMatrices(uniform: WebGLUniformLocation, matrices: Float32Array): boolean;
    setMatrix3x3(uniform: WebGLUniformLocation, matrix: Float32Array): boolean;
    setMatrix2x2(uniform: WebGLUniformLocation, matrix: Float32Array): boolean;
    setFloat(uniform: WebGLUniformLocation, value: number): boolean;
    setFloat2(uniform: WebGLUniformLocation, x: number, y: number): boolean;
    setFloat3(uniform: WebGLUniformLocation, x: number, y: number, z: number): boolean;
    setFloat4(uniform: WebGLUniformLocation, x: number, y: number, z: number, w: number): boolean;
    setColor3(uniform: WebGLUniformLocation, color3: IColor3Like): boolean;
    setColor4(uniform: WebGLUniformLocation, color3: IColor3Like, alpha: number): boolean;
    wipeCaches(bruteForce?: boolean): void;
    protected _createTexture(): WebGLTexture;
    protected _deleteTexture(texture: Nullable<WebGLTexture>): void;
    /**
     * Update the content of a dynamic texture
     * @param texture defines the texture to update
     * @param canvas defines the canvas containing the source
     * @param invertY defines if data must be stored with Y axis inverted
     * @param premulAlpha defines if alpha is stored as premultiplied
     * @param format defines the format of the data
     */
    updateDynamicTexture(texture: Nullable<InternalTexture>, canvas: any, invertY: boolean, premulAlpha?: boolean, format?: number): void;
    createDynamicTexture(width: number, height: number, generateMipMaps: boolean, samplingMode: number): InternalTexture;
    createVideoElement(constraints: MediaTrackConstraints): any;
    updateVideoTexture(texture: Nullable<InternalTexture>, video: HTMLVideoElement, invertY: boolean): void;
    createRawTexture(data: Nullable<ArrayBufferView>, width: number, height: number, format: number, generateMipMaps: boolean, invertY: boolean, samplingMode: number, compression?: Nullable<string>, type?: number, creationFlags?: number, useSRGBBuffer?: boolean): InternalTexture;
    createRawTexture2DArray(data: Nullable<ArrayBufferView>, width: number, height: number, depth: number, format: number, generateMipMaps: boolean, invertY: boolean, samplingMode: number, compression?: Nullable<string>, textureType?: number): InternalTexture;
    updateRawTexture(texture: Nullable<InternalTexture>, bufferView: Nullable<ArrayBufferView>, format: number, invertY: boolean, compression?: Nullable<string>, type?: number, useSRGBBuffer?: boolean): void;
    /**
     * Usually called from Texture.ts.
     * Passed information to create a NativeTexture
     * @param url defines a value which contains one of the following:
     * * A conventional http URL, e.g. 'http://...' or 'file://...'
     * * A base64 string of in-line texture data, e.g. 'data:image/jpg;base64,/...'
     * * An indicator that data being passed using the buffer parameter, e.g. 'data:mytexture.jpg'
     * @param noMipmap defines a boolean indicating that no mipmaps shall be generated.  Ignored for compressed textures.  They must be in the file
     * @param invertY when true, image is flipped when loaded.  You probably want true. Certain compressed textures may invert this if their default is inverted (eg. ktx)
     * @param scene needed for loading to the correct scene
     * @param samplingMode mode with should be used sample / access the texture (Default: Texture.TRILINEAR_SAMPLINGMODE)
     * @param onLoad optional callback to be called upon successful completion
     * @param onError optional callback to be called upon failure
     * @param buffer a source of a file previously fetched as either a base64 string, an ArrayBuffer (compressed or image format), HTMLImageElement (image format), or a Blob
     * @param fallback an internal argument in case the function must be called again, due to etc1 not having alpha capabilities
     * @param format internal format.  Default: RGB when extension is '.jpg' else RGBA.  Ignored for compressed textures
     * @param forcedExtension defines the extension to use to pick the right loader
     * @param mimeType defines an optional mime type
     * @param loaderOptions options to be passed to the loader
     * @param creationFlags specific flags to use when creating the texture (Constants.TEXTURE_CREATIONFLAG_STORAGE for storage textures, for eg)
     * @param useSRGBBuffer defines if the texture must be loaded in a sRGB GPU buffer (if supported by the GPU).
     * @returns a InternalTexture for assignment back into BABYLON.Texture
     */
    createTexture(url: Nullable<string>, noMipmap: boolean, invertY: boolean, scene: Nullable<ISceneLike>, samplingMode?: number, onLoad?: Nullable<(texture: InternalTexture) => void>, onError?: Nullable<(message: string, exception: any) => void>, buffer?: Nullable<string | ArrayBuffer | ArrayBufferView | HTMLImageElement | Blob | ImageBitmap>, fallback?: Nullable<InternalTexture>, format?: Nullable<number>, forcedExtension?: Nullable<string>, mimeType?: string, loaderOptions?: any, creationFlags?: number, useSRGBBuffer?: boolean): InternalTexture;
    /**
     * Wraps an external native texture in a Babylon texture.
     * @param texture defines the external texture
     * @returns the babylon internal texture
     */
    wrapNativeTexture(texture: any): InternalTexture;
    /**
     * Wraps an external web gl texture in a Babylon texture.
     * @returns the babylon internal texture
     */
    wrapWebGLTexture(): InternalTexture;
    _createDepthStencilTexture(size: TextureSize, options: DepthTextureCreationOptions, rtWrapper: RenderTargetWrapper): InternalTexture;
    /**
     * @internal
     */
    _releaseFramebufferObjects(framebuffer: Nullable<NativeFramebuffer>): void;
    /**
     * @internal Engine abstraction for loading and creating an image bitmap from a given source string.
     * @param imageSource source to load the image from.
     * @param options An object that sets options for the image's extraction.
     * @returns ImageBitmap
     */
    _createImageBitmapFromSource(imageSource: string, options?: ImageBitmapOptions): Promise<ImageBitmap>;
    /**
     * Engine abstraction for createImageBitmap
     * @param image source for image
     * @param options An object that sets options for the image's extraction.
     * @returns ImageBitmap
     */
    createImageBitmap(image: ImageBitmapSource, options?: ImageBitmapOptions): Promise<ImageBitmap>;
    /**
     * Resize an image and returns the image data as an uint8array
     * @param image image to resize
     * @param bufferWidth destination buffer width
     * @param bufferHeight destination buffer height
     * @returns an uint8array containing RGBA values of bufferWidth * bufferHeight size
     */
    resizeImageBitmap(image: ImageBitmap, bufferWidth: number, bufferHeight: number): Uint8Array;
    /**
     * Creates a cube texture
     * @param rootUrl defines the url where the files to load is located
     * @param scene defines the current scene
     * @param files defines the list of files to load (1 per face)
     * @param noMipmap defines a boolean indicating that no mipmaps shall be generated (false by default)
     * @param onLoad defines an optional callback raised when the texture is loaded
     * @param onError defines an optional callback raised if there is an issue to load the texture
     * @param format defines the format of the data
     * @param forcedExtension defines the extension to use to pick the right loader
     * @param createPolynomials if a polynomial sphere should be created for the cube texture
     * @param lodScale defines the scale applied to environment texture. This manages the range of LOD level used for IBL according to the roughness
     * @param lodOffset defines the offset applied to environment texture. This manages first LOD level used for IBL according to the roughness
     * @param fallback defines texture to use while falling back when (compressed) texture file not found.
     * @param loaderOptions options to be passed to the loader
     * @param useSRGBBuffer defines if the texture must be loaded in a sRGB GPU buffer (if supported by the GPU).
     * @returns the cube texture as an InternalTexture
     */
    createCubeTexture(rootUrl: string, scene: Nullable<Scene>, files: Nullable<string[]>, noMipmap?: boolean, onLoad?: Nullable<(data?: any) => void>, onError?: Nullable<(message?: string, exception?: any) => void>, format?: number, forcedExtension?: any, createPolynomials?: boolean, lodScale?: number, lodOffset?: number, fallback?: Nullable<InternalTexture>, loaderOptions?: any, useSRGBBuffer?: boolean): InternalTexture;
    /** @internal */
    _createHardwareTexture(): HardwareTextureWrapper;
    /** @internal */
    _createHardwareRenderTargetWrapper(isMulti: boolean, isCube: boolean, size: TextureSize): RenderTargetWrapper;
    /** @internal */
    _createInternalTexture(size: TextureSize, options: boolean | InternalTextureCreationOptions, _delayGPUTextureCreation?: boolean, source?: InternalTextureSource): InternalTexture;
    createRenderTargetTexture(size: number | {
        width: number;
        height: number;
    }, options: boolean | RenderTargetCreationOptions): RenderTargetWrapper;
    updateTextureSamplingMode(samplingMode: number, texture: InternalTexture): void;
    bindFramebuffer(texture: RenderTargetWrapper, faceIndex?: number, requiredWidth?: number, requiredHeight?: number, forceFullscreenViewport?: boolean): void;
    unBindFramebuffer(texture: RenderTargetWrapper, disableGenerateMipMaps?: boolean, onBeforeUnbind?: () => void): void;
    createDynamicVertexBuffer(data: DataArray): DataBuffer;
    updateDynamicIndexBuffer(indexBuffer: DataBuffer, indices: IndicesArray, offset?: number): void;
    updateDynamicVertexBuffer(vertexBuffer: DataBuffer, verticies: DataArray, byteOffset?: number, byteLength?: number): void;
    protected _setTexture(channel: number, texture: Nullable<BaseTexture>, isPartOfTextureArray?: boolean, depthStencilTexture?: boolean): boolean;
    private _setTextureSampling;
    private _setTextureWrapMode;
    private _setTextureCore;
    private _updateAnisotropicLevel;
    private _getAddressMode;
    /**
     * @internal
     */
    _bindTexture(channel: number, texture: InternalTexture): void;
    protected _deleteBuffer(buffer: NativeDataBuffer): void;
    /**
     * Create a canvas
     * @param width width
     * @param height height
     * @returns ICanvas interface
     */
    createCanvas(width: number, height: number): ICanvas;
    /**
     * Create an image to use with canvas
     * @returns IImage interface
     */
    createCanvasImage(): IImage;
    /**
     * Update a portion of an internal texture
     * @param texture defines the texture to update
     * @param imageData defines the data to store into the texture
     * @param xOffset defines the x coordinates of the update rectangle
     * @param yOffset defines the y coordinates of the update rectangle
     * @param width defines the width of the update rectangle
     * @param height defines the height of the update rectangle
     * @param faceIndex defines the face index if texture is a cube (0 by default)
     * @param lod defines the lod level to update (0 by default)
     * @param generateMipMaps defines whether to generate mipmaps or not
     */
    updateTextureData(texture: InternalTexture, imageData: ArrayBufferView, xOffset: number, yOffset: number, width: number, height: number, faceIndex?: number, lod?: number, generateMipMaps?: boolean): void;
    /**
     * @internal
     */
    _uploadCompressedDataToTextureDirectly(texture: InternalTexture, internalFormat: number, width: number, height: number, data: ArrayBufferView, faceIndex?: number, lod?: number): void;
    /**
     * @internal
     */
    _uploadDataToTextureDirectly(texture: InternalTexture, imageData: ArrayBufferView, faceIndex?: number, lod?: number): void;
    /**
     * @internal
     */
    _uploadArrayBufferViewToTexture(texture: InternalTexture, imageData: ArrayBufferView, faceIndex?: number, lod?: number): void;
    /**
     * @internal
     */
    _uploadImageToTexture(texture: InternalTexture, image: HTMLImageElement, faceIndex?: number, lod?: number): void;
    private _getNativeSamplingMode;
    private _getStencilFunc;
    private _getStencilOpFail;
    private _getStencilDepthFail;
    private _getStencilDepthPass;
    private _getNativeTextureFormat;
    private _getNativeAlphaMode;
    private _getNativeAttribType;
    getFontOffset(font: string): {
        ascent: number;
        height: number;
        descent: number;
    };
    _readTexturePixels(texture: InternalTexture, width: number, height: number, faceIndex?: number, level?: number, buffer?: Nullable<ArrayBufferView>, _flushRenderer?: boolean, _noDataConversion?: boolean, x?: number, y?: number): Promise<ArrayBufferView>;
}
export {};

}
declare module "babylonjs/Engines/nullEngine" {
import { Nullable, FloatArray, IndicesArray } from "babylonjs/types";
import { Engine } from "babylonjs/Engines/engine";
import { RenderTargetCreationOptions } from "babylonjs/Materials/Textures/textureCreationOptions";
import { VertexBuffer } from "babylonjs/Buffers/buffer";
import { InternalTexture } from "babylonjs/Materials/Textures/internalTexture";
import { Effect } from "babylonjs/Materials/effect";
import { IPipelineContext } from "babylonjs/Engines/IPipelineContext";
import { DataBuffer } from "babylonjs/Buffers/dataBuffer";
import { IColor4Like, IViewportLike } from "babylonjs/Maths/math.like";
import { ISceneLike } from "babylonjs/Engines/thinEngine";
import { DrawWrapper } from "babylonjs/Materials/drawWrapper";
import { RenderTargetWrapper } from "babylonjs/Engines/renderTargetWrapper";
import { IStencilState } from "babylonjs/States/IStencilState";
/**
 * Options to create the null engine
 */
export class NullEngineOptions {
    /**
     * Render width (Default: 512)
     */
    renderWidth: number;
    /**
     * Render height (Default: 256)
     */
    renderHeight: number;
    /**
     * Texture size (Default: 512)
     */
    textureSize: number;
    /**
     * If delta time between frames should be constant
     * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/advanced_animations#deterministic-lockstep
     */
    deterministicLockstep: boolean;
    /**
     * Maximum about of steps between frames (Default: 4)
     * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/advanced_animations#deterministic-lockstep
     */
    lockstepMaxSteps: number;
    /**
     * Make the matrix computations to be performed in 64 bits instead of 32 bits. False by default
     */
    useHighPrecisionMatrix?: boolean;
}
/**
 * The null engine class provides support for headless version of babylon.js.
 * This can be used in server side scenario or for testing purposes
 */
export class NullEngine extends Engine {
    private _options;
    /**
     * Gets a boolean indicating that the engine is running in deterministic lock step mode
     * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/advanced_animations#deterministic-lockstep
     * @returns true if engine is in deterministic lock step mode
     */
    isDeterministicLockStep(): boolean;
    /**
     * Gets the max steps when engine is running in deterministic lock step
     * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/advanced_animations#deterministic-lockstep
     * @returns the max steps
     */
    getLockstepMaxSteps(): number;
    /**
     * Gets the current hardware scaling level.
     * By default the hardware scaling level is computed from the window device ratio.
     * if level = 1 then the engine will render at the exact resolution of the canvas. If level = 0.5 then the engine will render at twice the size of the canvas.
     * @returns a number indicating the current hardware scaling level
     */
    getHardwareScalingLevel(): number;
    constructor(options?: NullEngineOptions);
    /**
     * Creates a vertex buffer
     * @param vertices the data for the vertex buffer
     * @returns the new WebGL static buffer
     */
    createVertexBuffer(vertices: FloatArray): DataBuffer;
    /**
     * Creates a new index buffer
     * @param indices defines the content of the index buffer
     * @returns a new webGL buffer
     */
    createIndexBuffer(indices: IndicesArray): DataBuffer;
    /**
     * Clear the current render buffer or the current render target (if any is set up)
     * @param color defines the color to use
     * @param backBuffer defines if the back buffer must be cleared
     * @param depth defines if the depth buffer must be cleared
     * @param stencil defines if the stencil buffer must be cleared
     */
    clear(color: IColor4Like, backBuffer: boolean, depth: boolean, stencil?: boolean): void;
    /**
     * Gets the current render width
     * @param useScreen defines if screen size must be used (or the current render target if any)
     * @returns a number defining the current render width
     */
    getRenderWidth(useScreen?: boolean): number;
    /**
     * Gets the current render height
     * @param useScreen defines if screen size must be used (or the current render target if any)
     * @returns a number defining the current render height
     */
    getRenderHeight(useScreen?: boolean): number;
    /**
     * Set the WebGL's viewport
     * @param viewport defines the viewport element to be used
     * @param requiredWidth defines the width required for rendering. If not provided the rendering canvas' width is used
     * @param requiredHeight defines the height required for rendering. If not provided the rendering canvas' height is used
     */
    setViewport(viewport: IViewportLike, requiredWidth?: number, requiredHeight?: number): void;
    createShaderProgram(pipelineContext: IPipelineContext, vertexCode: string, fragmentCode: string, defines: string, context?: WebGLRenderingContext): WebGLProgram;
    /**
     * Gets the list of webGL uniform locations associated with a specific program based on a list of uniform names
     * @param pipelineContext defines the pipeline context to use
     * @param uniformsNames defines the list of uniform names
     * @returns an array of webGL uniform locations
     */
    getUniforms(pipelineContext: IPipelineContext, uniformsNames: string[]): Nullable<WebGLUniformLocation>[];
    /**
     * Gets the lsit of active attributes for a given webGL program
     * @param pipelineContext defines the pipeline context to use
     * @param attributesNames defines the list of attribute names to get
     * @returns an array of indices indicating the offset of each attribute
     */
    getAttributes(pipelineContext: IPipelineContext, attributesNames: string[]): number[];
    /**
     * Binds an effect to the webGL context
     * @param effect defines the effect to bind
     */
    bindSamplers(effect: Effect): void;
    /**
     * Activates an effect, making it the current one (ie. the one used for rendering)
     * @param effect defines the effect to activate
     */
    enableEffect(effect: Nullable<Effect | DrawWrapper>): void;
    /**
     * Set various states to the webGL context
     * @param culling defines culling state: true to enable culling, false to disable it
     * @param zOffset defines the value to apply to zOffset (0 by default)
     * @param force defines if states must be applied even if cache is up to date
     * @param reverseSide defines if culling must be reversed (CCW if false, CW if true)
     * @param cullBackFaces true to cull back faces, false to cull front faces (if culling is enabled)
     * @param stencil stencil states to set
     * @param zOffsetUnits defines the value to apply to zOffsetUnits (0 by default)
     */
    setState(culling: boolean, zOffset?: number, force?: boolean, reverseSide?: boolean, cullBackFaces?: boolean, stencil?: IStencilState, zOffsetUnits?: number): void;
    /**
     * Set the value of an uniform to an array of int32
     * @param uniform defines the webGL uniform location where to store the value
     * @param array defines the array of int32 to store
     * @returns true if value was set
     */
    setIntArray(uniform: WebGLUniformLocation, array: Int32Array): boolean;
    /**
     * Set the value of an uniform to an array of int32 (stored as vec2)
     * @param uniform defines the webGL uniform location where to store the value
     * @param array defines the array of int32 to store
     * @returns true if value was set
     */
    setIntArray2(uniform: WebGLUniformLocation, array: Int32Array): boolean;
    /**
     * Set the value of an uniform to an array of int32 (stored as vec3)
     * @param uniform defines the webGL uniform location where to store the value
     * @param array defines the array of int32 to store
     * @returns true if value was set
     */
    setIntArray3(uniform: WebGLUniformLocation, array: Int32Array): boolean;
    /**
     * Set the value of an uniform to an array of int32 (stored as vec4)
     * @param uniform defines the webGL uniform location where to store the value
     * @param array defines the array of int32 to store
     * @returns true if value was set
     */
    setIntArray4(uniform: WebGLUniformLocation, array: Int32Array): boolean;
    /**
     * Set the value of an uniform to an array of float32
     * @param uniform defines the webGL uniform location where to store the value
     * @param array defines the array of float32 to store
     * @returns true if value was set
     */
    setFloatArray(uniform: WebGLUniformLocation, array: Float32Array): boolean;
    /**
     * Set the value of an uniform to an array of float32 (stored as vec2)
     * @param uniform defines the webGL uniform location where to store the value
     * @param array defines the array of float32 to store
     * @returns true if value was set
     */
    setFloatArray2(uniform: WebGLUniformLocation, array: Float32Array): boolean;
    /**
     * Set the value of an uniform to an array of float32 (stored as vec3)
     * @param uniform defines the webGL uniform location where to store the value
     * @param array defines the array of float32 to store
     * @returns true if value was set
     */
    setFloatArray3(uniform: WebGLUniformLocation, array: Float32Array): boolean;
    /**
     * Set the value of an uniform to an array of float32 (stored as vec4)
     * @param uniform defines the webGL uniform location where to store the value
     * @param array defines the array of float32 to store
     * @returns true if value was set
     */
    setFloatArray4(uniform: WebGLUniformLocation, array: Float32Array): boolean;
    /**
     * Set the value of an uniform to an array of number
     * @param uniform defines the webGL uniform location where to store the value
     * @param array defines the array of number to store
     * @returns true if value was set
     */
    setArray(uniform: WebGLUniformLocation, array: number[]): boolean;
    /**
     * Set the value of an uniform to an array of number (stored as vec2)
     * @param uniform defines the webGL uniform location where to store the value
     * @param array defines the array of number to store
     * @returns true if value was set
     */
    setArray2(uniform: WebGLUniformLocation, array: number[]): boolean;
    /**
     * Set the value of an uniform to an array of number (stored as vec3)
     * @param uniform defines the webGL uniform location where to store the value
     * @param array defines the array of number to store
     * @returns true if value was set
     */
    setArray3(uniform: WebGLUniformLocation, array: number[]): boolean;
    /**
     * Set the value of an uniform to an array of number (stored as vec4)
     * @param uniform defines the webGL uniform location where to store the value
     * @param array defines the array of number to store
     * @returns true if value was set
     */
    setArray4(uniform: WebGLUniformLocation, array: number[]): boolean;
    /**
     * Set the value of an uniform to an array of float32 (stored as matrices)
     * @param uniform defines the webGL uniform location where to store the value
     * @param matrices defines the array of float32 to store
     * @returns true if value was set
     */
    setMatrices(uniform: WebGLUniformLocation, matrices: Float32Array): boolean;
    /**
     * Set the value of an uniform to a matrix (3x3)
     * @param uniform defines the webGL uniform location where to store the value
     * @param matrix defines the Float32Array representing the 3x3 matrix to store
     * @returns true if value was set
     */
    setMatrix3x3(uniform: WebGLUniformLocation, matrix: Float32Array): boolean;
    /**
     * Set the value of an uniform to a matrix (2x2)
     * @param uniform defines the webGL uniform location where to store the value
     * @param matrix defines the Float32Array representing the 2x2 matrix to store
     * @returns true if value was set
     */
    setMatrix2x2(uniform: WebGLUniformLocation, matrix: Float32Array): boolean;
    /**
     * Set the value of an uniform to a number (float)
     * @param uniform defines the webGL uniform location where to store the value
     * @param value defines the float number to store
     * @returns true if value was set
     */
    setFloat(uniform: WebGLUniformLocation, value: number): boolean;
    /**
     * Set the value of an uniform to a vec2
     * @param uniform defines the webGL uniform location where to store the value
     * @param x defines the 1st component of the value
     * @param y defines the 2nd component of the value
     * @returns true if value was set
     */
    setFloat2(uniform: WebGLUniformLocation, x: number, y: number): boolean;
    /**
     * Set the value of an uniform to a vec3
     * @param uniform defines the webGL uniform location where to store the value
     * @param x defines the 1st component of the value
     * @param y defines the 2nd component of the value
     * @param z defines the 3rd component of the value
     * @returns true if value was set
     */
    setFloat3(uniform: WebGLUniformLocation, x: number, y: number, z: number): boolean;
    /**
     * Set the value of an uniform to a boolean
     * @param uniform defines the webGL uniform location where to store the value
     * @param bool defines the boolean to store
     * @returns true if value was set
     */
    setBool(uniform: WebGLUniformLocation, bool: number): boolean;
    /**
     * Set the value of an uniform to a vec4
     * @param uniform defines the webGL uniform location where to store the value
     * @param x defines the 1st component of the value
     * @param y defines the 2nd component of the value
     * @param z defines the 3rd component of the value
     * @param w defines the 4th component of the value
     * @returns true if value was set
     */
    setFloat4(uniform: WebGLUniformLocation, x: number, y: number, z: number, w: number): boolean;
    /**
     * Sets the current alpha mode
     * @param mode defines the mode to use (one of the Engine.ALPHA_XXX)
     * @param noDepthWriteChange defines if depth writing state should remains unchanged (false by default)
     * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/advanced/transparent_rendering
     */
    setAlphaMode(mode: number, noDepthWriteChange?: boolean): void;
    /**
     * Bind webGl buffers directly to the webGL context
     * @param vertexBuffers defines the vertex buffer to bind
     * @param indexBuffer defines the index buffer to bind
     * @param effect defines the effect associated with the vertex buffer
     */
    bindBuffers(vertexBuffers: {
        [key: string]: VertexBuffer;
    }, indexBuffer: DataBuffer, effect: Effect): void;
    /**
     * Force the entire cache to be cleared
     * You should not have to use this function unless your engine needs to share the webGL context with another engine
     * @param bruteForce defines a boolean to force clearing ALL caches (including stencil, detoh and alpha states)
     */
    wipeCaches(bruteForce?: boolean): void;
    /**
     * Send a draw order
     * @param useTriangles defines if triangles must be used to draw (else wireframe will be used)
     * @param indexStart defines the starting index
     * @param indexCount defines the number of index to draw
     * @param instancesCount defines the number of instances to draw (if instantiation is enabled)
     */
    draw(useTriangles: boolean, indexStart: number, indexCount: number, instancesCount?: number): void;
    /**
     * Draw a list of indexed primitives
     * @param fillMode defines the primitive to use
     * @param indexStart defines the starting index
     * @param indexCount defines the number of index to draw
     * @param instancesCount defines the number of instances to draw (if instantiation is enabled)
     */
    drawElementsType(fillMode: number, indexStart: number, indexCount: number, instancesCount?: number): void;
    /**
     * Draw a list of unindexed primitives
     * @param fillMode defines the primitive to use
     * @param verticesStart defines the index of first vertex to draw
     * @param verticesCount defines the count of vertices to draw
     * @param instancesCount defines the number of instances to draw (if instantiation is enabled)
     */
    drawArraysType(fillMode: number, verticesStart: number, verticesCount: number, instancesCount?: number): void;
    /** @internal */
    protected _createTexture(): WebGLTexture;
    /**
     * @internal
     */
    _releaseTexture(texture: InternalTexture): void;
    /**
     * Usually called from Texture.ts.
     * Passed information to create a WebGLTexture
     * @param urlArg defines a value which contains one of the following:
     * * A conventional http URL, e.g. 'http://...' or 'file://...'
     * * A base64 string of in-line texture data, e.g. 'data:image/jpg;base64,/...'
     * * An indicator that data being passed using the buffer parameter, e.g. 'data:mytexture.jpg'
     * @param noMipmap defines a boolean indicating that no mipmaps shall be generated.  Ignored for compressed textures.  They must be in the file
     * @param invertY when true, image is flipped when loaded.  You probably want true. Certain compressed textures may invert this if their default is inverted (eg. ktx)
     * @param scene needed for loading to the correct scene
     * @param samplingMode mode with should be used sample / access the texture (Default: Texture.TRILINEAR_SAMPLINGMODE)
     * @param onLoad optional callback to be called upon successful completion
     * @param onError optional callback to be called upon failure
     * @param buffer a source of a file previously fetched as either a base64 string, an ArrayBuffer (compressed or image format), HTMLImageElement (image format), or a Blob
     * @param fallback an internal argument in case the function must be called again, due to etc1 not having alpha capabilities
     * @param format internal format.  Default: RGB when extension is '.jpg' else RGBA.  Ignored for compressed textures
     * @param forcedExtension defines the extension to use to pick the right loader
     * @param mimeType defines an optional mime type
     * @returns a InternalTexture for assignment back into BABYLON.Texture
     */
    createTexture(urlArg: Nullable<string>, noMipmap: boolean, invertY: boolean, scene: Nullable<ISceneLike>, samplingMode?: number, onLoad?: Nullable<(texture: InternalTexture) => void>, onError?: Nullable<(message: string, exception: any) => void>, buffer?: Nullable<string | ArrayBuffer | ArrayBufferView | HTMLImageElement | Blob | ImageBitmap>, fallback?: Nullable<InternalTexture>, format?: Nullable<number>, forcedExtension?: Nullable<string>, mimeType?: string): InternalTexture;
    /**
     * @internal
     */
    _createHardwareRenderTargetWrapper(isMulti: boolean, isCube: boolean, size: number | {
        width: number;
        height: number;
        layers?: number;
    }): RenderTargetWrapper;
    /**
     * Creates a new render target wrapper
     * @param size defines the size of the texture
     * @param options defines the options used to create the texture
     * @returns a new render target wrapper
     */
    createRenderTargetTexture(size: any, options: boolean | RenderTargetCreationOptions): RenderTargetWrapper;
    /**
     * Creates a new render target wrapper
     * @param size defines the size of the texture
     * @param options defines the options used to create the texture
     * @returns a new render target wrapper
     */
    createRenderTargetCubeTexture(size: number, options?: RenderTargetCreationOptions): RenderTargetWrapper;
    /**
     * Update the sampling mode of a given texture
     * @param samplingMode defines the required sampling mode
     * @param texture defines the texture to update
     */
    updateTextureSamplingMode(samplingMode: number, texture: InternalTexture): void;
    /**
     * Creates a raw texture
     * @param data defines the data to store in the texture
     * @param width defines the width of the texture
     * @param height defines the height of the texture
     * @param format defines the format of the data
     * @param generateMipMaps defines if the engine should generate the mip levels
     * @param invertY defines if data must be stored with Y axis inverted
     * @param samplingMode defines the required sampling mode (Texture.NEAREST_SAMPLINGMODE by default)
     * @param compression defines the compression used (null by default)
     * @param type defines the type fo the data (Engine.TEXTURETYPE_UNSIGNED_INT by default)
     * @param creationFlags specific flags to use when creating the texture (Constants.TEXTURE_CREATIONFLAG_STORAGE for storage textures, for eg)
     * @param useSRGBBuffer defines if the texture must be loaded in a sRGB GPU buffer (if supported by the GPU).
     * @returns the raw texture inside an InternalTexture
     */
    createRawTexture(data: Nullable<ArrayBufferView>, width: number, height: number, format: number, generateMipMaps: boolean, invertY: boolean, samplingMode: number, compression?: Nullable<string>, type?: number, creationFlags?: number, useSRGBBuffer?: boolean): InternalTexture;
    /**
     * Update a raw texture
     * @param texture defines the texture to update
     * @param data defines the data to store in the texture
     * @param format defines the format of the data
     * @param invertY defines if data must be stored with Y axis inverted
     * @param compression defines the compression used (null by default)
     * @param type defines the type fo the data (Engine.TEXTURETYPE_UNSIGNED_INT by default)
     * @param useSRGBBuffer defines if the texture must be loaded in a sRGB GPU buffer (if supported by the GPU).
     */
    updateRawTexture(texture: Nullable<InternalTexture>, data: Nullable<ArrayBufferView>, format: number, invertY: boolean, compression?: Nullable<string>, type?: number, useSRGBBuffer?: boolean): void;
    /**
     * Binds the frame buffer to the specified texture.
     * @param rtWrapper The render target wrapper to render to
     * @param faceIndex The face of the texture to render to in case of cube texture
     * @param requiredWidth The width of the target to render to
     * @param requiredHeight The height of the target to render to
     * @param forceFullscreenViewport Forces the viewport to be the entire texture/screen if true
     */
    bindFramebuffer(rtWrapper: RenderTargetWrapper, faceIndex?: number, requiredWidth?: number, requiredHeight?: number, forceFullscreenViewport?: boolean): void;
    /**
     * Unbind the current render target texture from the webGL context
     * @param rtWrapper defines the render target wrapper to unbind
     * @param disableGenerateMipMaps defines a boolean indicating that mipmaps must not be generated
     * @param onBeforeUnbind defines a function which will be called before the effective unbind
     */
    unBindFramebuffer(rtWrapper: RenderTargetWrapper, disableGenerateMipMaps?: boolean, onBeforeUnbind?: () => void): void;
    /**
     * Creates a dynamic vertex buffer
     * @param vertices the data for the dynamic vertex buffer
     * @returns the new WebGL dynamic buffer
     */
    createDynamicVertexBuffer(vertices: FloatArray): DataBuffer;
    /**
     * Update the content of a dynamic texture
     * @param texture defines the texture to update
     * @param canvas defines the canvas containing the source
     * @param invertY defines if data must be stored with Y axis inverted
     * @param premulAlpha defines if alpha is stored as premultiplied
     * @param format defines the format of the data
     */
    updateDynamicTexture(texture: Nullable<InternalTexture>, canvas: HTMLCanvasElement, invertY: boolean, premulAlpha?: boolean, format?: number): void;
    /**
     * Gets a boolean indicating if all created effects are ready
     * @returns true if all effects are ready
     */
    areAllEffectsReady(): boolean;
    /**
     * @internal
     * Get the current error code of the webGL context
     * @returns the error code
     * @see https://developer.mozilla.org/en-US/docs/Web/API/WebGLRenderingContext/getError
     */
    getError(): number;
    /** @internal */
    _getUnpackAlignement(): number;
    /**
     * @internal
     */
    _unpackFlipY(value: boolean): void;
    /**
     * Update a dynamic index buffer
     * @param indexBuffer defines the target index buffer
     * @param indices defines the data to update
     * @param offset defines the offset in the target index buffer where update should start
     */
    updateDynamicIndexBuffer(indexBuffer: WebGLBuffer, indices: IndicesArray, offset?: number): void;
    /**
     * Updates a dynamic vertex buffer.
     * @param vertexBuffer the vertex buffer to update
     * @param vertices the data used to update the vertex buffer
     * @param byteOffset the byte offset of the data (optional)
     * @param byteLength the byte length of the data (optional)
     */
    updateDynamicVertexBuffer(vertexBuffer: WebGLBuffer, vertices: FloatArray, byteOffset?: number, byteLength?: number): void;
    /**
     * @internal
     */
    _bindTextureDirectly(target: number, texture: InternalTexture): boolean;
    /**
     * @internal
     */
    _bindTexture(channel: number, texture: InternalTexture): void;
    protected _deleteBuffer(buffer: WebGLBuffer): void;
    /**
     * Force the engine to release all cached effects. This means that next effect compilation will have to be done completely even if a similar effect was already compiled
     */
    releaseEffects(): void;
    displayLoadingUI(): void;
    hideLoadingUI(): void;
    set loadingUIText(_: string);
    /**
     * @internal
     */
    _uploadCompressedDataToTextureDirectly(texture: InternalTexture, internalFormat: number, width: number, height: number, data: ArrayBufferView, faceIndex?: number, lod?: number): void;
    /**
     * @internal
     */
    _uploadDataToTextureDirectly(texture: InternalTexture, imageData: ArrayBufferView, faceIndex?: number, lod?: number): void;
    /**
     * @internal
     */
    _uploadArrayBufferViewToTexture(texture: InternalTexture, imageData: ArrayBufferView, faceIndex?: number, lod?: number): void;
    /**
     * @internal
     */
    _uploadImageToTexture(texture: InternalTexture, image: HTMLImageElement, faceIndex?: number, lod?: number): void;
}

}
declare module "babylonjs/Engines/performanceConfigurator" {
/** @internal */
export class PerformanceConfigurator {
    /** @internal */
    static MatrixUse64Bits: boolean;
    /** @internal */
    static MatrixTrackPrecisionChange: boolean;
    /** @internal */
    static MatrixCurrentType: any;
    /** @internal */
    static MatrixTrackedMatrices: Array<any> | null;
    /**
     * @internal
     */
    static SetMatrixPrecision(use64bits: boolean): void;
}

}
declare module "babylonjs/Engines/Processors/Expressions/index" {
export * from "babylonjs/Engines/Processors/Expressions/shaderDefineExpression";
export * from "babylonjs/Engines/Processors/Expressions/Operators/index";

}
declare module "babylonjs/Engines/Processors/Expressions/Operators/index" {
export * from "babylonjs/Engines/Processors/Expressions/Operators/shaderDefineAndOperator";
export * from "babylonjs/Engines/Processors/Expressions/Operators/shaderDefineArithmeticOperator";
export * from "babylonjs/Engines/Processors/Expressions/Operators/shaderDefineIsDefinedOperator";
export * from "babylonjs/Engines/Processors/Expressions/Operators/shaderDefineOrOperator";

}
declare module "babylonjs/Engines/Processors/Expressions/Operators/shaderDefineAndOperator" {
import { ShaderDefineExpression } from "babylonjs/Engines/Processors/Expressions/shaderDefineExpression";
/** @internal */
export class ShaderDefineAndOperator extends ShaderDefineExpression {
    leftOperand: ShaderDefineExpression;
    rightOperand: ShaderDefineExpression;
    isTrue(preprocessors: {
        [key: string]: string;
    }): boolean;
}

}
declare module "babylonjs/Engines/Processors/Expressions/Operators/shaderDefineArithmeticOperator" {
import { ShaderDefineExpression } from "babylonjs/Engines/Processors/Expressions/shaderDefineExpression";
/** @internal */
export class ShaderDefineArithmeticOperator extends ShaderDefineExpression {
    define: string;
    operand: string;
    testValue: string;
    constructor(define: string, operand: string, testValue: string);
    isTrue(preprocessors: {
        [key: string]: string;
    }): boolean;
}

}
declare module "babylonjs/Engines/Processors/Expressions/Operators/shaderDefineIsDefinedOperator" {
import { ShaderDefineExpression } from "babylonjs/Engines/Processors/Expressions/shaderDefineExpression";
/** @internal */
export class ShaderDefineIsDefinedOperator extends ShaderDefineExpression {
    define: string;
    not: boolean;
    constructor(define: string, not?: boolean);
    isTrue(preprocessors: {
        [key: string]: string;
    }): boolean;
}

}
declare module "babylonjs/Engines/Processors/Expressions/Operators/shaderDefineOrOperator" {
import { ShaderDefineExpression } from "babylonjs/Engines/Processors/Expressions/shaderDefineExpression";
/** @internal */
export class ShaderDefineOrOperator extends ShaderDefineExpression {
    leftOperand: ShaderDefineExpression;
    rightOperand: ShaderDefineExpression;
    isTrue(preprocessors: {
        [key: string]: string;
    }): boolean;
}

}
declare module "babylonjs/Engines/Processors/Expressions/shaderDefineExpression" {
/** @internal */
export class ShaderDefineExpression {
    isTrue(preprocessors: {
        [key: string]: string;
    }): boolean;
    private static _OperatorPriority;
    private static _Stack;
    static postfixToInfix(postfix: string[]): string;
    static infixToPostfix(infix: string): string[];
}

}
declare module "babylonjs/Engines/Processors/index" {
export * from "babylonjs/Engines/Processors/iShaderProcessor";
export * from "babylonjs/Engines/Processors/Expressions/index";
export * from "babylonjs/Engines/Processors/shaderCodeConditionNode";
export * from "babylonjs/Engines/Processors/shaderCodeCursor";
export * from "babylonjs/Engines/Processors/shaderCodeNode";
export * from "babylonjs/Engines/Processors/shaderCodeTestNode";
export * from "babylonjs/Engines/Processors/shaderProcessingOptions";
export * from "babylonjs/Engines/Processors/shaderProcessor";
export * from "babylonjs/Engines/Processors/shaderCodeInliner";

}
declare module "babylonjs/Engines/Processors/iShaderProcessor" {
import { ShaderLanguage } from "babylonjs/Materials/shaderLanguage";
import { Nullable } from "babylonjs/types";
import { ShaderProcessingContext } from "babylonjs/Engines/Processors/shaderProcessingOptions";
import { ThinEngine } from "babylonjs/Engines/thinEngine";
/** @internal */
export interface IShaderProcessor {
    shaderLanguage: ShaderLanguage;
    uniformRegexp?: RegExp;
    uniformBufferRegexp?: RegExp;
    textureRegexp?: RegExp;
    noPrecision?: boolean;
    parseGLES3?: boolean;
    attributeKeywordName?: string;
    varyingVertexKeywordName?: string;
    varyingFragmentKeywordName?: string;
    preProcessShaderCode?: (code: string, isFragment: boolean) => string;
    attributeProcessor?: (attribute: string, preProcessors: {
        [key: string]: string;
    }, processingContext: Nullable<ShaderProcessingContext>) => string;
    varyingProcessor?: (varying: string, isFragment: boolean, preProcessors: {
        [key: string]: string;
    }, processingContext: Nullable<ShaderProcessingContext>) => string;
    uniformProcessor?: (uniform: string, isFragment: boolean, preProcessors: {
        [key: string]: string;
    }, processingContext: Nullable<ShaderProcessingContext>) => string;
    uniformBufferProcessor?: (uniformBuffer: string, isFragment: boolean, processingContext: Nullable<ShaderProcessingContext>) => string;
    textureProcessor?: (texture: string, isFragment: boolean, preProcessors: {
        [key: string]: string;
    }, processingContext: Nullable<ShaderProcessingContext>) => string;
    endOfUniformBufferProcessor?: (closingBracketLine: string, isFragment: boolean, processingContext: Nullable<ShaderProcessingContext>) => string;
    lineProcessor?: (line: string, isFragment: boolean, processingContext: Nullable<ShaderProcessingContext>) => string;
    preProcessor?: (code: string, defines: string[], isFragment: boolean, processingContext: Nullable<ShaderProcessingContext>) => string;
    postProcessor?: (code: string, defines: string[], isFragment: boolean, processingContext: Nullable<ShaderProcessingContext>, engine: ThinEngine) => string;
    initializeShaders?: (processingContext: Nullable<ShaderProcessingContext>) => void;
    finalizeShaders?: (vertexCode: string, fragmentCode: string, processingContext: Nullable<ShaderProcessingContext>) => {
        vertexCode: string;
        fragmentCode: string;
    };
}
export {};

}
declare module "babylonjs/Engines/Processors/shaderCodeConditionNode" {
import { ShaderCodeNode } from "babylonjs/Engines/Processors/shaderCodeNode";
import { ProcessingOptions } from "babylonjs/Engines/Processors/shaderProcessingOptions";
/** @internal */
export class ShaderCodeConditionNode extends ShaderCodeNode {
    process(preprocessors: {
        [key: string]: string;
    }, options: ProcessingOptions): string;
}

}
declare module "babylonjs/Engines/Processors/shaderCodeCursor" {
/** @internal */
export class ShaderCodeCursor {
    private _lines;
    lineIndex: number;
    get currentLine(): string;
    get canRead(): boolean;
    set lines(value: string[]);
}

}
declare module "babylonjs/Engines/Processors/shaderCodeInliner" {
/**
 * Class used to inline functions in shader code
 */
export class ShaderCodeInliner {
    private static readonly _RegexpFindFunctionNameAndType;
    private _sourceCode;
    private _functionDescr;
    private _numMaxIterations;
    /** Gets or sets the token used to mark the functions to inline */
    inlineToken: string;
    /** Gets or sets the debug mode */
    debug: boolean;
    /** Gets the code after the inlining process */
    get code(): string;
    /**
     * Initializes the inliner
     * @param sourceCode shader code source to inline
     * @param numMaxIterations maximum number of iterations (used to detect recursive calls)
     */
    constructor(sourceCode: string, numMaxIterations?: number);
    /**
     * Start the processing of the shader code
     */
    processCode(): void;
    private _collectFunctions;
    private _processInlining;
    private _replaceFunctionCallsByCode;
    private _replaceNames;
}

}
declare module "babylonjs/Engines/Processors/shaderCodeNode" {
import { ProcessingOptions } from "babylonjs/Engines/Processors/shaderProcessingOptions";
/** @internal */
export class ShaderCodeNode {
    line: string;
    children: ShaderCodeNode[];
    additionalDefineKey?: string;
    additionalDefineValue?: string;
    isValid(preprocessors: {
        [key: string]: string;
    }): boolean;
    process(preprocessors: {
        [key: string]: string;
    }, options: ProcessingOptions): string;
}

}
declare module "babylonjs/Engines/Processors/shaderCodeTestNode" {
import { ShaderCodeNode } from "babylonjs/Engines/Processors/shaderCodeNode";
import { ShaderDefineExpression } from "babylonjs/Engines/Processors/Expressions/shaderDefineExpression";
/** @internal */
export class ShaderCodeTestNode extends ShaderCodeNode {
    testExpression: ShaderDefineExpression;
    isValid(preprocessors: {
        [key: string]: string;
    }): boolean;
}

}
declare module "babylonjs/Engines/Processors/shaderProcessingOptions" {
import { IShaderProcessor } from "babylonjs/Engines/Processors/iShaderProcessor";
import { Nullable } from "babylonjs/types";
/**
 * Function for custom code generation
 */
export type ShaderCustomProcessingFunction = (shaderType: string, code: string) => string;
/** @internal */
export interface ShaderProcessingContext {
}
/** @internal */
export interface ProcessingOptions {
    defines: string[];
    indexParameters: any;
    isFragment: boolean;
    shouldUseHighPrecisionShader: boolean;
    supportsUniformBuffers: boolean;
    shadersRepository: string;
    includesShadersStore: {
        [key: string]: string;
    };
    processor: Nullable<IShaderProcessor>;
    version: string;
    platformName: string;
    lookForClosingBracketForUniformBuffer?: boolean;
    processingContext: Nullable<ShaderProcessingContext>;
    isNDCHalfZRange: boolean;
    useReverseDepthBuffer: boolean;
    processCodeAfterIncludes?: ShaderCustomProcessingFunction;
}

}
declare module "babylonjs/Engines/Processors/shaderProcessor" {
import { ProcessingOptions } from "babylonjs/Engines/Processors/shaderProcessingOptions";
import { WebRequest } from "babylonjs/Misc/webRequest";
import { LoadFileError } from "babylonjs/Misc/fileTools";
import { IOfflineProvider } from "babylonjs/Offline/IOfflineProvider";
import { IFileRequest } from "babylonjs/Misc/fileRequest";
import { ThinEngine } from "babylonjs/Engines/thinEngine";
/** @internal */
export class ShaderProcessor {
    static Initialize(options: ProcessingOptions): void;
    static Process(sourceCode: string, options: ProcessingOptions, callback: (migratedCode: string, codeBeforeMigration: string) => void, engine: ThinEngine): void;
    static PreProcess(sourceCode: string, options: ProcessingOptions, callback: (migratedCode: string, codeBeforeMigration: string) => void, engine: ThinEngine): void;
    static Finalize(vertexCode: string, fragmentCode: string, options: ProcessingOptions): {
        vertexCode: string;
        fragmentCode: string;
    };
    private static _ProcessPrecision;
    private static _ExtractOperation;
    private static _BuildSubExpression;
    private static _BuildExpression;
    private static _MoveCursorWithinIf;
    private static _MoveCursor;
    private static _EvaluatePreProcessors;
    private static _PreparePreProcessors;
    private static _ProcessShaderConversion;
    private static _ApplyPreProcessing;
    private static _ProcessIncludes;
    /**
     * Loads a file from a url
     * @param url url to load
     * @param onSuccess callback called when the file successfully loads
     * @param onProgress callback called while file is loading (if the server supports this mode)
     * @param offlineProvider defines the offline provider for caching
     * @param useArrayBuffer defines a boolean indicating that date must be returned as ArrayBuffer
     * @param onError callback called when the file fails to load
     * @returns a file request object
     * @internal
     */
    static _FileToolsLoadFile(url: string, onSuccess: (data: string | ArrayBuffer, responseURL?: string) => void, onProgress?: (ev: ProgressEvent) => void, offlineProvider?: IOfflineProvider, useArrayBuffer?: boolean, onError?: (request?: WebRequest, exception?: LoadFileError) => void): IFileRequest;
}
export {};

}
declare module "babylonjs/Engines/renderTargetWrapper" {
import { InternalTexture } from "babylonjs/Materials/Textures/internalTexture";
import { TextureSize } from "babylonjs/Materials/Textures/textureCreationOptions";
import { Nullable } from "babylonjs/types";
import { ThinEngine } from "babylonjs/Engines/thinEngine";
/**
 * An interface enforcing the renderTarget accessor to used by render target textures.
 */
export interface IRenderTargetTexture {
    /**
     * Entry point to access the wrapper on a texture.
     */
    renderTarget: Nullable<RenderTargetWrapper>;
}
/**
 * Wrapper around a render target (either single or multi textures)
 */
export class RenderTargetWrapper {
    protected _engine: ThinEngine;
    private _size;
    private _isCube;
    private _isMulti;
    private _textures;
    /** @internal */
    _samples: number;
    /** @internal */
    _attachments: Nullable<number[]>;
    /** @internal */
    _generateStencilBuffer: boolean;
    /** @internal */
    _generateDepthBuffer: boolean;
    /** @internal */
    _depthStencilTexture: Nullable<InternalTexture>;
    /** @internal */
    _depthStencilTextureWithStencil: boolean;
    /**
     * Gets the depth/stencil texture (if created by a createDepthStencilTexture() call)
     */
    get depthStencilTexture(): Nullable<InternalTexture>;
    /**
     * Indicates if the depth/stencil texture has a stencil aspect
     */
    get depthStencilTextureWithStencil(): boolean;
    /**
     * Defines if the render target wrapper is for a cube texture or if false a 2d texture
     */
    get isCube(): boolean;
    /**
     * Defines if the render target wrapper is for a single or multi target render wrapper
     */
    get isMulti(): boolean;
    /**
     * Defines if the render target wrapper is for a single or an array of textures
     */
    get is2DArray(): boolean;
    /**
     * Gets the size of the render target wrapper (used for cubes, as width=height in this case)
     */
    get size(): number;
    /**
     * Gets the width of the render target wrapper
     */
    get width(): number;
    /**
     * Gets the height of the render target wrapper
     */
    get height(): number;
    /**
     * Gets the number of layers of the render target wrapper (only used if is2DArray is true)
     */
    get layers(): number;
    /**
     * Gets the render texture. If this is a multi render target, gets the first texture
     */
    get texture(): Nullable<InternalTexture>;
    /**
     * Gets the list of render textures. If we are not in a multi render target, the list will be null (use the texture getter instead)
     */
    get textures(): Nullable<InternalTexture[]>;
    /**
     * Gets the sample count of the render target
     */
    get samples(): number;
    /**
     * Sets the sample count of the render target
     * @param value sample count
     * @param initializeBuffers If set to true, the engine will make an initializing call to drawBuffers (only used when isMulti=true).
     * @param force true to force calling the update sample count engine function even if the current sample count is equal to value
     * @returns the sample count that has been set
     */
    setSamples(value: number, initializeBuffers?: boolean, force?: boolean): number;
    /**
     * Initializes the render target wrapper
     * @param isMulti true if the wrapper is a multi render target
     * @param isCube true if the wrapper should render to a cube texture
     * @param size size of the render target (width/height/layers)
     * @param engine engine used to create the render target
     */
    constructor(isMulti: boolean, isCube: boolean, size: TextureSize, engine: ThinEngine);
    /**
     * Sets the render target texture(s)
     * @param textures texture(s) to set
     */
    setTextures(textures: Nullable<InternalTexture> | Nullable<InternalTexture[]>): void;
    /**
     * Set a texture in the textures array
     * @param texture the texture to set
     * @param index the index in the textures array to set
     * @param disposePrevious If this function should dispose the previous texture
     */
    setTexture(texture: InternalTexture, index?: number, disposePrevious?: boolean): void;
    /**
     * Creates the depth/stencil texture
     * @param comparisonFunction Comparison function to use for the texture
     * @param bilinearFiltering true if bilinear filtering should be used when sampling the texture
     * @param generateStencil true if the stencil aspect should also be created
     * @param samples sample count to use when creating the texture
     * @param format format of the depth texture
     * @returns the depth/stencil created texture
     */
    createDepthStencilTexture(comparisonFunction?: number, bilinearFiltering?: boolean, generateStencil?: boolean, samples?: number, format?: number): InternalTexture;
    /**
     * Shares the depth buffer of this render target with another render target.
     * @internal
     * @param renderTarget Destination renderTarget
     */
    _shareDepth(renderTarget: RenderTargetWrapper): void;
    /**
     * @internal
     */
    _swapAndDie(target: InternalTexture): void;
    protected _cloneRenderTargetWrapper(): Nullable<RenderTargetWrapper>;
    protected _swapRenderTargetWrapper(target: RenderTargetWrapper): void;
    /** @internal */
    _rebuild(): void;
    /**
     * Releases the internal render textures
     */
    releaseTextures(): void;
    /**
     * Disposes the whole render target wrapper
     * @param disposeOnlyFramebuffers true if only the frame buffers should be released (used for the WebGL engine). If false, all the textures will also be released
     */
    dispose(disposeOnlyFramebuffers?: boolean): void;
}

}
declare module "babylonjs/Engines/shaderStore" {
import { ShaderLanguage } from "babylonjs/Materials/shaderLanguage";
/**
 * Defines the shader related stores and directory
 */
export class ShaderStore {
    /**
     * Gets or sets the relative url used to load shaders if using the engine in non-minified mode
     */
    static ShadersRepository: string;
    /**
     * Store of each shader (The can be looked up using effect.key)
     */
    static ShadersStore: {
        [key: string]: string;
    };
    /**
     * Store of each included file for a shader (The can be looked up using effect.key)
     */
    static IncludesShadersStore: {
        [key: string]: string;
    };
    /**
     * Gets or sets the relative url used to load shaders (WGSL) if using the engine in non-minified mode
     */
    static ShadersRepositoryWGSL: string;
    /**
     * Store of each shader  (WGSL)
     */
    static ShadersStoreWGSL: {
        [key: string]: string;
    };
    /**
     * Store of each included file for a shader (WGSL)
     */
    static IncludesShadersStoreWGSL: {
        [key: string]: string;
    };
    /**
     * Gets the shaders repository path for a given shader language
     * @param shaderLanguage the shader language
     * @returns the path to the shaders repository
     */
    static GetShadersRepository(shaderLanguage?: ShaderLanguage): string;
    /**
     * Gets the shaders store of a given shader language
     * @param shaderLanguage the shader language
     * @returns the shaders store
     */
    static GetShadersStore(shaderLanguage?: ShaderLanguage): {
        [key: string]: string;
    };
    /**
     * Gets the include shaders store of a given shader language
     * @param shaderLanguage the shader language
     * @returns the include shaders store
     */
    static GetIncludesShadersStore(shaderLanguage?: ShaderLanguage): {
        [key: string]: string;
    };
}

}
declare module "babylonjs/Engines/thinEngine" {
import { IInternalTextureLoader } from "babylonjs/Materials/Textures/internalTextureLoader";
import { IEffectCreationOptions } from "babylonjs/Materials/effect";
import { Effect } from "babylonjs/Materials/effect";
import { IShaderProcessor } from "babylonjs/Engines/Processors/iShaderProcessor";
import { ShaderProcessingContext } from "babylonjs/Engines/Processors/shaderProcessingOptions";
import { UniformBuffer } from "babylonjs/Materials/uniformBuffer";
import { Nullable, DataArray, IndicesArray } from "babylonjs/types";
import { EngineCapabilities } from "babylonjs/Engines/engineCapabilities";
import { Observable } from "babylonjs/Misc/observable";
import { DepthCullingState } from "babylonjs/States/depthCullingState";
import { StencilState } from "babylonjs/States/stencilState";
import { AlphaState } from "babylonjs/States/alphaCullingState";
import { InternalTexture, InternalTextureSource } from "babylonjs/Materials/Textures/internalTexture";
import { IViewportLike, IColor4Like } from "babylonjs/Maths/math.like";
import { DataBuffer } from "babylonjs/Buffers/dataBuffer";
import { IFileRequest } from "babylonjs/Misc/fileRequest";
import { IPipelineContext } from "babylonjs/Engines/IPipelineContext";
import { WebGLPipelineContext } from "babylonjs/Engines/WebGL/webGLPipelineContext";
import { VertexBuffer } from "babylonjs/Buffers/buffer";
import { InstancingAttributeInfo } from "babylonjs/Engines/instancingAttributeInfo";
import { ThinTexture } from "babylonjs/Materials/Textures/thinTexture";
import { IOfflineProvider } from "babylonjs/Offline/IOfflineProvider";
import { IEffectFallbacks } from "babylonjs/Materials/iEffectFallbacks";
import { IWebRequest } from "babylonjs/Misc/interfaces/iWebRequest";
import { EngineFeatures } from "babylonjs/Engines/engineFeatures";
import { HardwareTextureWrapper } from "babylonjs/Materials/Textures/hardwareTextureWrapper";
import { DrawWrapper } from "babylonjs/Materials/drawWrapper";
import { IMaterialContext } from "babylonjs/Engines/IMaterialContext";
import { IDrawContext } from "babylonjs/Engines/IDrawContext";
import { ICanvas, ICanvasRenderingContext, IImage } from "babylonjs/Engines/ICanvas";
import { StencilStateComposer } from "babylonjs/States/stencilStateComposer";
import { StorageBuffer } from "babylonjs/Buffers/storageBuffer";
import { IAudioEngineOptions } from "babylonjs/Audio/Interfaces/IAudioEngineOptions";
import { IStencilState } from "babylonjs/States/IStencilState";
import { InternalTextureCreationOptions, TextureSize } from "babylonjs/Materials/Textures/textureCreationOptions";
import { ShaderLanguage } from "babylonjs/Materials/shaderLanguage";
import { RenderTargetWrapper } from "babylonjs/Engines/renderTargetWrapper";
import { WebRequest } from "babylonjs/Misc/webRequest";
import { LoadFileError } from "babylonjs/Misc/fileTools";
import { Texture } from "babylonjs/Materials/Textures/texture";
/**
 * Defines the interface used by objects working like Scene
 * @internal
 */
export interface ISceneLike {
    addPendingData(data: any): void;
    removePendingData(data: any): void;
    offlineProvider: IOfflineProvider;
}
/**
 * Information about the current host
 */
export interface HostInformation {
    /**
     * Defines if the current host is a mobile
     */
    isMobile: boolean;
}
/** Interface defining initialization parameters for Engine class */
export interface EngineOptions extends WebGLContextAttributes {
    /**
     * Defines if the engine should no exceed a specified device ratio
     * @see https://developer.mozilla.org/en-US/docs/Web/API/Window/devicePixelRatio
     */
    limitDeviceRatio?: number;
    /**
     * Defines if webvr should be enabled automatically
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/webVRCamera
     */
    autoEnableWebVR?: boolean;
    /**
     * Defines if webgl2 should be turned off even if supported
     * @see https://doc.babylonjs.com/setup/support/webGL2
     */
    disableWebGL2Support?: boolean;
    /**
     * Defines if webaudio should be initialized as well
     * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic
     */
    audioEngine?: boolean;
    /**
     * Specifies options for the audio engine
     */
    audioEngineOptions?: IAudioEngineOptions;
    /**
     * Defines if animations should run using a deterministic lock step
     * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/advanced_animations#deterministic-lockstep
     */
    deterministicLockstep?: boolean;
    /** Defines the maximum steps to use with deterministic lock step mode */
    lockstepMaxSteps?: number;
    /** Defines the seconds between each deterministic lock step */
    timeStep?: number;
    /**
     * Defines that engine should ignore context lost events
     * If this event happens when this parameter is true, you will have to reload the page to restore rendering
     */
    doNotHandleContextLost?: boolean;
    /**
     * Defines that engine should ignore modifying touch action attribute and style
     * If not handle, you might need to set it up on your side for expected touch devices behavior.
     */
    doNotHandleTouchAction?: boolean;
    /**
     * Defines that engine should compile shaders with high precision floats (if supported). True by default
     */
    useHighPrecisionFloats?: boolean;
    /**
     * Make the canvas XR Compatible for XR sessions
     */
    xrCompatible?: boolean;
    /**
     * Make the matrix computations to be performed in 64 bits instead of 32 bits. False by default
     */
    useHighPrecisionMatrix?: boolean;
    /**
     * Will prevent the system from falling back to software implementation if a hardware device cannot be created
     */
    failIfMajorPerformanceCaveat?: boolean;
    /**
     * Defines whether to adapt to the device's viewport characteristics (default: false)
     */
    adaptToDeviceRatio?: boolean;
    /**
     * If sRGB Buffer support is not set during construction, use this value to force a specific state
     * This is added due to an issue when processing textures in chrome/edge/firefox
     * This will not influence NativeEngine and WebGPUEngine which set the behavior to true during construction.
     */
    forceSRGBBufferSupportState?: boolean;
    /**
     * True if the more expensive but exact conversions should be used for transforming colors to and from linear space within shaders.
     * Otherwise, the default is to use a cheaper approximation.
     */
    useExactSrgbConversions?: boolean;
}
/**
 * The base engine class (root of all engines)
 */
export class ThinEngine {
    /** Use this array to turn off some WebGL2 features on known buggy browsers version */
    static ExceptionList: ({
        key: string;
        capture: string;
        captureConstraint: number;
        targets: string[];
    } | {
        key: string;
        capture: null;
        captureConstraint: null;
        targets: string[];
    })[];
    /** @internal */
    static _TextureLoaders: IInternalTextureLoader[];
    /**
     * Returns the current npm package of the sdk
     */
    static get NpmPackage(): string;
    /**
     * Returns the current version of the framework
     */
    static get Version(): string;
    /**
     * Returns a string describing the current engine
     */
    get description(): string;
    /** @internal */
    protected _name: string;
    /**
     * Gets or sets the name of the engine
     */
    get name(): string;
    set name(value: string);
    /**
     * Returns the version of the engine
     */
    get version(): number;
    /**
     * Gets or sets the epsilon value used by collision engine
     */
    static CollisionsEpsilon: number;
    /**
     * Gets or sets the relative url used to load shaders if using the engine in non-minified mode
     */
    static get ShadersRepository(): string;
    static set ShadersRepository(value: string);
    protected _shaderProcessor: Nullable<IShaderProcessor>;
    /**
     * @internal
     */
    _getShaderProcessor(shaderLanguage: ShaderLanguage): Nullable<IShaderProcessor>;
    /**
     * Gets or sets a boolean that indicates if textures must be forced to power of 2 size even if not required
     */
    forcePOTTextures: boolean;
    /**
     * Gets a boolean indicating if the engine is currently rendering in fullscreen mode
     */
    isFullscreen: boolean;
    /**
     * Gets or sets a boolean indicating if back faces must be culled. If false, front faces are culled instead (true by default)
     * If non null, this takes precedence over the value from the material
     */
    cullBackFaces: Nullable<boolean>;
    /**
     * Gets or sets a boolean indicating if the engine must keep rendering even if the window is not in foreground
     */
    renderEvenInBackground: boolean;
    /**
     * Gets or sets a boolean indicating that cache can be kept between frames
     */
    preventCacheWipeBetweenFrames: boolean;
    /** Gets or sets a boolean indicating if the engine should validate programs after compilation */
    validateShaderPrograms: boolean;
    private _useReverseDepthBuffer;
    /**
     * Gets or sets a boolean indicating if depth buffer should be reverse, going from far to near.
     * This can provide greater z depth for distant objects.
     */
    get useReverseDepthBuffer(): boolean;
    set useReverseDepthBuffer(useReverse: boolean);
    /**
     * Indicates if the z range in NDC space is 0..1 (value: true) or -1..1 (value: false)
     */
    readonly isNDCHalfZRange: boolean;
    /**
     * Indicates that the origin of the texture/framebuffer space is the bottom left corner. If false, the origin is top left
     */
    readonly hasOriginBottomLeft: boolean;
    /**
     * Gets or sets a boolean indicating that uniform buffers must be disabled even if they are supported
     */
    disableUniformBuffers: boolean;
    /**
     * An event triggered when the engine is disposed.
     */
    readonly onDisposeObservable: Observable<ThinEngine>;
    private _frameId;
    /**
     * Gets the current frame id
     */
    get frameId(): number;
    /**
     * The time (in milliseconds elapsed since the current page has been loaded) when the engine was initialized
     */
    readonly startTime: number;
    /** @internal */
    _uniformBuffers: UniformBuffer[];
    /** @internal */
    _storageBuffers: StorageBuffer[];
    /**
     * Gets a boolean indicating that the engine supports uniform buffers
     * @see https://doc.babylonjs.com/setup/support/webGL2#uniform-buffer-objets
     */
    get supportsUniformBuffers(): boolean;
    /** @internal */
    _gl: WebGL2RenderingContext;
    /** @internal */
    _webGLVersion: number;
    protected _renderingCanvas: Nullable<HTMLCanvasElement>;
    protected _windowIsBackground: boolean;
    protected _creationOptions: EngineOptions;
    protected _audioContext: Nullable<AudioContext>;
    protected _audioDestination: Nullable<AudioDestinationNode | MediaStreamAudioDestinationNode>;
    /**
     * Gets the options used for engine creation
     * @returns EngineOptions object
     */
    getCreationOptions(): EngineOptions;
    protected _highPrecisionShadersAllowed: boolean;
    /** @internal */
    get _shouldUseHighPrecisionShader(): boolean;
    /**
     * Gets a boolean indicating that only power of 2 textures are supported
     * Please note that you can still use non power of 2 textures but in this case the engine will forcefully convert them
     */
    get needPOTTextures(): boolean;
    /** @internal */
    _badOS: boolean;
    /** @internal */
    _badDesktopOS: boolean;
    /** @internal */
    _hardwareScalingLevel: number;
    /** @internal */
    _caps: EngineCapabilities;
    /** @internal */
    _features: EngineFeatures;
    protected _isStencilEnable: boolean;
    private _glVersion;
    private _glRenderer;
    private _glVendor;
    /** @internal */
    _videoTextureSupported: boolean;
    protected _renderingQueueLaunched: boolean;
    protected _activeRenderLoops: (() => void)[];
    /**
     * Gets the list of current active render loop functions
     * @returns an array with the current render loop functions
     */
    get activeRenderLoops(): Array<() => void>;
    /**
     * Observable signaled when a context lost event is raised
     */
    onContextLostObservable: Observable<ThinEngine>;
    /**
     * Observable signaled when a context restored event is raised
     */
    onContextRestoredObservable: Observable<ThinEngine>;
    private _onContextLost;
    private _onContextRestored;
    protected _contextWasLost: boolean;
    /** @internal */
    _doNotHandleContextLost: boolean;
    /**
     * Gets or sets a boolean indicating if resources should be retained to be able to handle context lost events
     * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/optimize_your_scene#handling-webgl-context-lost
     */
    get doNotHandleContextLost(): boolean;
    set doNotHandleContextLost(value: boolean);
    /**
     * Gets or sets a boolean indicating that vertex array object must be disabled even if they are supported
     */
    disableVertexArrayObjects: boolean;
    /** @internal */
    protected _colorWrite: boolean;
    /** @internal */
    protected _colorWriteChanged: boolean;
    /** @internal */
    protected _depthCullingState: DepthCullingState;
    /** @internal */
    protected _stencilStateComposer: StencilStateComposer;
    /** @internal */
    protected _stencilState: StencilState;
    /** @internal */
    _alphaState: AlphaState;
    /** @internal */
    _alphaMode: number;
    /** @internal */
    _alphaEquation: number;
    /** @internal */
    _internalTexturesCache: InternalTexture[];
    /** @internal */
    _renderTargetWrapperCache: RenderTargetWrapper[];
    /** @internal */
    protected _activeChannel: number;
    private _currentTextureChannel;
    /** @internal */
    protected _boundTexturesCache: {
        [key: string]: Nullable<InternalTexture>;
    };
    protected _currentEffect: Nullable<Effect>;
    /** @internal */
    _currentDrawContext: IDrawContext;
    /** @internal */
    _currentMaterialContext: IMaterialContext;
    /** @internal */
    protected _currentProgram: Nullable<WebGLProgram>;
    protected _compiledEffects: {
        [key: string]: Effect;
    };
    private _vertexAttribArraysEnabled;
    /** @internal */
    protected _cachedViewport: Nullable<IViewportLike>;
    private _cachedVertexArrayObject;
    /** @internal */
    protected _cachedVertexBuffers: any;
    /** @internal */
    protected _cachedIndexBuffer: Nullable<DataBuffer>;
    /** @internal */
    protected _cachedEffectForVertexBuffers: Nullable<Effect>;
    /** @internal */
    _currentRenderTarget: Nullable<RenderTargetWrapper>;
    private _uintIndicesCurrentlySet;
    protected _currentBoundBuffer: Nullable<DataBuffer>[];
    /** @internal */
    _currentFramebuffer: Nullable<WebGLFramebuffer>;
    /** @internal */
    _dummyFramebuffer: Nullable<WebGLFramebuffer>;
    private _currentBufferPointers;
    private _currentInstanceLocations;
    private _currentInstanceBuffers;
    private _textureUnits;
    /** @internal */
    _workingCanvas: Nullable<ICanvas>;
    /** @internal */
    _workingContext: Nullable<ICanvasRenderingContext>;
    /** @internal */
    _boundRenderFunction: any;
    private _vaoRecordInProgress;
    private _mustWipeVertexAttributes;
    private _emptyTexture;
    private _emptyCubeTexture;
    private _emptyTexture3D;
    private _emptyTexture2DArray;
    /** @internal */
    _frameHandler: number;
    private _nextFreeTextureSlots;
    private _maxSimultaneousTextures;
    private _maxMSAASamplesOverride;
    private _activeRequests;
    /**
     * If set to true zooming in and out in the browser will rescale the hardware-scaling correctly.
     */
    adaptToDeviceRatio: boolean;
    /** @internal */
    private _lastDevicePixelRatio;
    /** @internal */
    _transformTextureUrl: Nullable<(url: string) => string>;
    /**
     * Gets information about the current host
     */
    hostInformation: HostInformation;
    protected get _supportsHardwareTextureRescaling(): boolean;
    private _framebufferDimensionsObject;
    /**
     * sets the object from which width and height will be taken from when getting render width and height
     * Will fallback to the gl object
     * @param dimensions the framebuffer width and height that will be used.
     */
    set framebufferDimensionsObject(dimensions: Nullable<{
        framebufferWidth: number;
        framebufferHeight: number;
    }>);
    /**
     * Gets the current viewport
     */
    get currentViewport(): Nullable<IViewportLike>;
    /**
     * Gets the default empty texture
     */
    get emptyTexture(): InternalTexture;
    /**
     * Gets the default empty 3D texture
     */
    get emptyTexture3D(): InternalTexture;
    /**
     * Gets the default empty 2D array texture
     */
    get emptyTexture2DArray(): InternalTexture;
    /**
     * Gets the default empty cube texture
     */
    get emptyCubeTexture(): InternalTexture;
    /**
     * Defines whether the engine has been created with the premultipliedAlpha option on or not.
     */
    premultipliedAlpha: boolean;
    /**
     * Observable event triggered before each texture is initialized
     */
    onBeforeTextureInitObservable: Observable<Texture>;
    /** @internal */
    protected _isWebGPU: boolean;
    /**
     * Gets a boolean indicating if the engine runs in WebGPU or not.
     */
    get isWebGPU(): boolean;
    /** @internal */
    protected _shaderPlatformName: string;
    /**
     * Gets the shader platform name used by the effects.
     */
    get shaderPlatformName(): string;
    /**
     * Enables or disables the snapshot rendering mode
     * Note that the WebGL engine does not support snapshot rendering so setting the value won't have any effect for this engine
     */
    get snapshotRendering(): boolean;
    set snapshotRendering(activate: boolean);
    protected _snapshotRenderingMode: number;
    /**
     * Gets or sets the snapshot rendering mode
     */
    get snapshotRenderingMode(): number;
    set snapshotRenderingMode(mode: number);
    protected _useExactSrgbConversions: boolean;
    /**
     * Gets a boolean indicating if the exact sRGB conversions or faster approximations are used for converting to and from linear space.
     */
    get useExactSrgbConversions(): boolean;
    /**
     * Creates a new snapshot at the next frame using the current snapshotRenderingMode
     */
    snapshotRenderingReset(): void;
    private _checkForMobile;
    private static _CreateCanvas;
    /**
     * Create a canvas. This method is overridden by other engines
     * @param width width
     * @param height height
     * @returns ICanvas interface
     */
    createCanvas(width: number, height: number): ICanvas;
    /**
     * Create an image to use with canvas
     * @returns IImage interface
     */
    createCanvasImage(): IImage;
    /**
     * Creates a new engine
     * @param canvasOrContext defines the canvas or WebGL context to use for rendering. If you provide a WebGL context, Babylon.js will not hook events on the canvas (like pointers, keyboards, etc...) so no event observables will be available. This is mostly used when Babylon.js is used as a plugin on a system which already used the WebGL context
     * @param antialias defines enable antialiasing (default: false)
     * @param options defines further options to be sent to the getContext() function
     * @param adaptToDeviceRatio defines whether to adapt to the device's viewport characteristics (default: false)
     */
    constructor(canvasOrContext: Nullable<HTMLCanvasElement | OffscreenCanvas | WebGLRenderingContext | WebGL2RenderingContext>, antialias?: boolean, options?: EngineOptions, adaptToDeviceRatio?: boolean);
    protected _restoreEngineAfterContextLost(initEngine: () => void): void;
    /**
     * Shared initialization across engines types.
     * @param canvas The canvas associated with this instance of the engine.
     * @param doNotHandleTouchAction Defines that engine should ignore modifying touch action attribute and style
     * @param audioEngine Defines if an audio engine should be created by default
     */
    protected _sharedInit(canvas: HTMLCanvasElement, doNotHandleTouchAction: boolean, audioEngine: boolean): void;
    /**
     * @internal
     */
    _getShaderProcessingContext(shaderLanguage: ShaderLanguage): Nullable<ShaderProcessingContext>;
    private _rebuildInternalTextures;
    private _rebuildRenderTargetWrappers;
    private _rebuildEffects;
    /**
     * Gets a boolean indicating if all created effects are ready
     * @returns true if all effects are ready
     */
    areAllEffectsReady(): boolean;
    protected _rebuildBuffers(): void;
    protected _initGLContext(): void;
    protected _initFeatures(): void;
    /**
     * Gets version of the current webGL context
     * Keep it for back compat - use version instead
     */
    get webGLVersion(): number;
    /**
     * Gets a string identifying the name of the class
     * @returns "Engine" string
     */
    getClassName(): string;
    /**
     * Returns true if the stencil buffer has been enabled through the creation option of the context.
     */
    get isStencilEnable(): boolean;
    /** @internal */
    _prepareWorkingCanvas(): void;
    /**
     * Reset the texture cache to empty state
     */
    resetTextureCache(): void;
    /**
     * Gets an object containing information about the current engine context
     * @returns an object containing the vendor, the renderer and the version of the current engine context
     */
    getInfo(): {
        vendor: string;
        renderer: string;
        version: string;
    };
    /**
     * Gets an object containing information about the current webGL context
     * @returns an object containing the vendor, the renderer and the version of the current webGL context
     */
    getGlInfo(): {
        vendor: string;
        renderer: string;
        version: string;
    };
    /**
     * Defines the hardware scaling level.
     * By default the hardware scaling level is computed from the window device ratio.
     * if level = 1 then the engine will render at the exact resolution of the canvas. If level = 0.5 then the engine will render at twice the size of the canvas.
     * @param level defines the level to use
     */
    setHardwareScalingLevel(level: number): void;
    /**
     * Gets the current hardware scaling level.
     * By default the hardware scaling level is computed from the window device ratio.
     * if level = 1 then the engine will render at the exact resolution of the canvas. If level = 0.5 then the engine will render at twice the size of the canvas.
     * @returns a number indicating the current hardware scaling level
     */
    getHardwareScalingLevel(): number;
    /**
     * Gets the list of loaded textures
     * @returns an array containing all loaded textures
     */
    getLoadedTexturesCache(): InternalTexture[];
    /**
     * Gets the object containing all engine capabilities
     * @returns the EngineCapabilities object
     */
    getCaps(): EngineCapabilities;
    /**
     * stop executing a render loop function and remove it from the execution array
     * @param renderFunction defines the function to be removed. If not provided all functions will be removed.
     */
    stopRenderLoop(renderFunction?: () => void): void;
    /** @internal */
    _renderLoop(): void;
    /**
     * Gets the HTML canvas attached with the current webGL context
     * @returns a HTML canvas
     */
    getRenderingCanvas(): Nullable<HTMLCanvasElement>;
    /**
     * Gets the audio context specified in engine initialization options
     * @returns an Audio Context
     */
    getAudioContext(): Nullable<AudioContext>;
    /**
     * Gets the audio destination specified in engine initialization options
     * @returns an audio destination node
     */
    getAudioDestination(): Nullable<AudioDestinationNode | MediaStreamAudioDestinationNode>;
    /**
     * Gets host window
     * @returns the host window object
     */
    getHostWindow(): Nullable<Window>;
    /**
     * Gets the current render width
     * @param useScreen defines if screen size must be used (or the current render target if any)
     * @returns a number defining the current render width
     */
    getRenderWidth(useScreen?: boolean): number;
    /**
     * Gets the current render height
     * @param useScreen defines if screen size must be used (or the current render target if any)
     * @returns a number defining the current render height
     */
    getRenderHeight(useScreen?: boolean): number;
    /**
     * Can be used to override the current requestAnimationFrame requester.
     * @internal
     */
    protected _queueNewFrame(bindedRenderFunction: any, requester?: any): number;
    /**
     * Register and execute a render loop. The engine can have more than one render function
     * @param renderFunction defines the function to continuously execute
     */
    runRenderLoop(renderFunction: () => void): void;
    /**
     * Clear the current render buffer or the current render target (if any is set up)
     * @param color defines the color to use
     * @param backBuffer defines if the back buffer must be cleared
     * @param depth defines if the depth buffer must be cleared
     * @param stencil defines if the stencil buffer must be cleared
     */
    clear(color: Nullable<IColor4Like>, backBuffer: boolean, depth: boolean, stencil?: boolean): void;
    protected _viewportCached: {
        x: number;
        y: number;
        z: number;
        w: number;
    };
    /**
     * @internal
     */
    _viewport(x: number, y: number, width: number, height: number): void;
    /**
     * Set the WebGL's viewport
     * @param viewport defines the viewport element to be used
     * @param requiredWidth defines the width required for rendering. If not provided the rendering canvas' width is used
     * @param requiredHeight defines the height required for rendering. If not provided the rendering canvas' height is used
     */
    setViewport(viewport: IViewportLike, requiredWidth?: number, requiredHeight?: number): void;
    /**
     * Begin a new frame
     */
    beginFrame(): void;
    /**
     * Enf the current frame
     */
    endFrame(): void;
    /**
     * Resize the view according to the canvas' size
     * @param forceSetSize true to force setting the sizes of the underlying canvas
     */
    resize(forceSetSize?: boolean): void;
    /**
     * Force a specific size of the canvas
     * @param width defines the new canvas' width
     * @param height defines the new canvas' height
     * @param forceSetSize true to force setting the sizes of the underlying canvas
     * @returns true if the size was changed
     */
    setSize(width: number, height: number, forceSetSize?: boolean): boolean;
    /**
     * Binds the frame buffer to the specified texture.
     * @param texture The render target wrapper to render to
     * @param faceIndex The face of the texture to render to in case of cube texture
     * @param requiredWidth The width of the target to render to
     * @param requiredHeight The height of the target to render to
     * @param forceFullscreenViewport Forces the viewport to be the entire texture/screen if true
     * @param lodLevel defines the lod level to bind to the frame buffer
     * @param layer defines the 2d array index to bind to frame buffer to
     */
    bindFramebuffer(texture: RenderTargetWrapper, faceIndex?: number, requiredWidth?: number, requiredHeight?: number, forceFullscreenViewport?: boolean, lodLevel?: number, layer?: number): void;
    /**
     * Set various states to the webGL context
     * @param culling defines culling state: true to enable culling, false to disable it
     * @param zOffset defines the value to apply to zOffset (0 by default)
     * @param force defines if states must be applied even if cache is up to date
     * @param reverseSide defines if culling must be reversed (CCW if false, CW if true)
     * @param cullBackFaces true to cull back faces, false to cull front faces (if culling is enabled)
     * @param stencil stencil states to set
     * @param zOffsetUnits defines the value to apply to zOffsetUnits (0 by default)
     */
    setState(culling: boolean, zOffset?: number, force?: boolean, reverseSide?: boolean, cullBackFaces?: boolean, stencil?: IStencilState, zOffsetUnits?: number): void;
    /**
     * Gets a boolean indicating if depth testing is enabled
     * @returns the current state
     */
    getDepthBuffer(): boolean;
    /**
     * Enable or disable depth buffering
     * @param enable defines the state to set
     */
    setDepthBuffer(enable: boolean): void;
    /**
     * Set the z offset Factor to apply to current rendering
     * @param value defines the offset to apply
     */
    setZOffset(value: number): void;
    /**
     * Gets the current value of the zOffset Factor
     * @returns the current zOffset Factor state
     */
    getZOffset(): number;
    /**
     * Set the z offset Units to apply to current rendering
     * @param value defines the offset to apply
     */
    setZOffsetUnits(value: number): void;
    /**
     * Gets the current value of the zOffset Units
     * @returns the current zOffset Units state
     */
    getZOffsetUnits(): number;
    /**
     * @internal
     */
    _bindUnboundFramebuffer(framebuffer: Nullable<WebGLFramebuffer>): void;
    /** @internal */
    _currentFrameBufferIsDefaultFrameBuffer(): boolean;
    /**
     * Generates the mipmaps for a texture
     * @param texture texture to generate the mipmaps for
     */
    generateMipmaps(texture: InternalTexture): void;
    /**
     * Unbind the current render target texture from the webGL context
     * @param texture defines the render target wrapper to unbind
     * @param disableGenerateMipMaps defines a boolean indicating that mipmaps must not be generated
     * @param onBeforeUnbind defines a function which will be called before the effective unbind
     */
    unBindFramebuffer(texture: RenderTargetWrapper, disableGenerateMipMaps?: boolean, onBeforeUnbind?: () => void): void;
    /**
     * Force a webGL flush (ie. a flush of all waiting webGL commands)
     */
    flushFramebuffer(): void;
    /**
     * Unbind the current render target and bind the default framebuffer
     */
    restoreDefaultFramebuffer(): void;
    /** @internal */
    protected _resetVertexBufferBinding(): void;
    /**
     * Creates a vertex buffer
     * @param data the data for the vertex buffer
     * @returns the new WebGL static buffer
     */
    createVertexBuffer(data: DataArray): DataBuffer;
    private _createVertexBuffer;
    /**
     * Creates a dynamic vertex buffer
     * @param data the data for the dynamic vertex buffer
     * @returns the new WebGL dynamic buffer
     */
    createDynamicVertexBuffer(data: DataArray): DataBuffer;
    protected _resetIndexBufferBinding(): void;
    /**
     * Creates a new index buffer
     * @param indices defines the content of the index buffer
     * @param updatable defines if the index buffer must be updatable
     * @returns a new webGL buffer
     */
    createIndexBuffer(indices: IndicesArray, updatable?: boolean): DataBuffer;
    protected _normalizeIndexData(indices: IndicesArray): Uint16Array | Uint32Array;
    /**
     * Bind a webGL buffer to the webGL context
     * @param buffer defines the buffer to bind
     */
    bindArrayBuffer(buffer: Nullable<DataBuffer>): void;
    protected bindIndexBuffer(buffer: Nullable<DataBuffer>): void;
    private _bindBuffer;
    /**
     * update the bound buffer with the given data
     * @param data defines the data to update
     */
    updateArrayBuffer(data: Float32Array): void;
    private _vertexAttribPointer;
    /**
     * @internal
     */
    _bindIndexBufferWithCache(indexBuffer: Nullable<DataBuffer>): void;
    private _bindVertexBuffersAttributes;
    /**
     * Records a vertex array object
     * @see https://doc.babylonjs.com/setup/support/webGL2#vertex-array-objects
     * @param vertexBuffers defines the list of vertex buffers to store
     * @param indexBuffer defines the index buffer to store
     * @param effect defines the effect to store
     * @param overrideVertexBuffers defines optional list of avertex buffers that overrides the entries in vertexBuffers
     * @returns the new vertex array object
     */
    recordVertexArrayObject(vertexBuffers: {
        [key: string]: VertexBuffer;
    }, indexBuffer: Nullable<DataBuffer>, effect: Effect, overrideVertexBuffers?: {
        [kind: string]: Nullable<VertexBuffer>;
    }): WebGLVertexArrayObject;
    /**
     * Bind a specific vertex array object
     * @see https://doc.babylonjs.com/setup/support/webGL2#vertex-array-objects
     * @param vertexArrayObject defines the vertex array object to bind
     * @param indexBuffer defines the index buffer to bind
     */
    bindVertexArrayObject(vertexArrayObject: WebGLVertexArrayObject, indexBuffer: Nullable<DataBuffer>): void;
    /**
     * Bind webGl buffers directly to the webGL context
     * @param vertexBuffer defines the vertex buffer to bind
     * @param indexBuffer defines the index buffer to bind
     * @param vertexDeclaration defines the vertex declaration to use with the vertex buffer
     * @param vertexStrideSize defines the vertex stride of the vertex buffer
     * @param effect defines the effect associated with the vertex buffer
     */
    bindBuffersDirectly(vertexBuffer: DataBuffer, indexBuffer: DataBuffer, vertexDeclaration: number[], vertexStrideSize: number, effect: Effect): void;
    private _unbindVertexArrayObject;
    /**
     * Bind a list of vertex buffers to the webGL context
     * @param vertexBuffers defines the list of vertex buffers to bind
     * @param indexBuffer defines the index buffer to bind
     * @param effect defines the effect associated with the vertex buffers
     * @param overrideVertexBuffers defines optional list of avertex buffers that overrides the entries in vertexBuffers
     */
    bindBuffers(vertexBuffers: {
        [key: string]: Nullable<VertexBuffer>;
    }, indexBuffer: Nullable<DataBuffer>, effect: Effect, overrideVertexBuffers?: {
        [kind: string]: Nullable<VertexBuffer>;
    }): void;
    /**
     * Unbind all instance attributes
     */
    unbindInstanceAttributes(): void;
    /**
     * Release and free the memory of a vertex array object
     * @param vao defines the vertex array object to delete
     */
    releaseVertexArrayObject(vao: WebGLVertexArrayObject): void;
    /**
     * @internal
     */
    _releaseBuffer(buffer: DataBuffer): boolean;
    protected _deleteBuffer(buffer: DataBuffer): void;
    /**
     * Update the content of a webGL buffer used with instantiation and bind it to the webGL context
     * @param instancesBuffer defines the webGL buffer to update and bind
     * @param data defines the data to store in the buffer
     * @param offsetLocations defines the offsets or attributes information used to determine where data must be stored in the buffer
     */
    updateAndBindInstancesBuffer(instancesBuffer: DataBuffer, data: Float32Array, offsetLocations: number[] | InstancingAttributeInfo[]): void;
    /**
     * Bind the content of a webGL buffer used with instantiation
     * @param instancesBuffer defines the webGL buffer to bind
     * @param attributesInfo defines the offsets or attributes information used to determine where data must be stored in the buffer
     * @param computeStride defines Whether to compute the strides from the info or use the default 0
     */
    bindInstancesBuffer(instancesBuffer: DataBuffer, attributesInfo: InstancingAttributeInfo[], computeStride?: boolean): void;
    /**
     * Disable the instance attribute corresponding to the name in parameter
     * @param name defines the name of the attribute to disable
     */
    disableInstanceAttributeByName(name: string): void;
    /**
     * Disable the instance attribute corresponding to the location in parameter
     * @param attributeLocation defines the attribute location of the attribute to disable
     */
    disableInstanceAttribute(attributeLocation: number): void;
    /**
     * Disable the attribute corresponding to the location in parameter
     * @param attributeLocation defines the attribute location of the attribute to disable
     */
    disableAttributeByIndex(attributeLocation: number): void;
    /**
     * Send a draw order
     * @param useTriangles defines if triangles must be used to draw (else wireframe will be used)
     * @param indexStart defines the starting index
     * @param indexCount defines the number of index to draw
     * @param instancesCount defines the number of instances to draw (if instantiation is enabled)
     */
    draw(useTriangles: boolean, indexStart: number, indexCount: number, instancesCount?: number): void;
    /**
     * Draw a list of points
     * @param verticesStart defines the index of first vertex to draw
     * @param verticesCount defines the count of vertices to draw
     * @param instancesCount defines the number of instances to draw (if instantiation is enabled)
     */
    drawPointClouds(verticesStart: number, verticesCount: number, instancesCount?: number): void;
    /**
     * Draw a list of unindexed primitives
     * @param useTriangles defines if triangles must be used to draw (else wireframe will be used)
     * @param verticesStart defines the index of first vertex to draw
     * @param verticesCount defines the count of vertices to draw
     * @param instancesCount defines the number of instances to draw (if instantiation is enabled)
     */
    drawUnIndexed(useTriangles: boolean, verticesStart: number, verticesCount: number, instancesCount?: number): void;
    /**
     * Draw a list of indexed primitives
     * @param fillMode defines the primitive to use
     * @param indexStart defines the starting index
     * @param indexCount defines the number of index to draw
     * @param instancesCount defines the number of instances to draw (if instantiation is enabled)
     */
    drawElementsType(fillMode: number, indexStart: number, indexCount: number, instancesCount?: number): void;
    /**
     * Draw a list of unindexed primitives
     * @param fillMode defines the primitive to use
     * @param verticesStart defines the index of first vertex to draw
     * @param verticesCount defines the count of vertices to draw
     * @param instancesCount defines the number of instances to draw (if instantiation is enabled)
     */
    drawArraysType(fillMode: number, verticesStart: number, verticesCount: number, instancesCount?: number): void;
    private _drawMode;
    /** @internal */
    protected _reportDrawCall(): void;
    /**
     * @internal
     */
    _releaseEffect(effect: Effect): void;
    /**
     * @internal
     */
    _deletePipelineContext(pipelineContext: IPipelineContext): void;
    /** @internal */
    _getGlobalDefines(defines?: {
        [key: string]: string;
    }): string | undefined;
    /**
     * Create a new effect (used to store vertex/fragment shaders)
     * @param baseName defines the base name of the effect (The name of file without .fragment.fx or .vertex.fx)
     * @param attributesNamesOrOptions defines either a list of attribute names or an IEffectCreationOptions object
     * @param uniformsNamesOrEngine defines either a list of uniform names or the engine to use
     * @param samplers defines an array of string used to represent textures
     * @param defines defines the string containing the defines to use to compile the shaders
     * @param fallbacks defines the list of potential fallbacks to use if shader compilation fails
     * @param onCompiled defines a function to call when the effect creation is successful
     * @param onError defines a function to call when the effect creation has failed
     * @param indexParameters defines an object containing the index values to use to compile shaders (like the maximum number of simultaneous lights)
     * @param shaderLanguage the language the shader is written in (default: GLSL)
     * @returns the new Effect
     */
    createEffect(baseName: any, attributesNamesOrOptions: string[] | IEffectCreationOptions, uniformsNamesOrEngine: string[] | ThinEngine, samplers?: string[], defines?: string, fallbacks?: IEffectFallbacks, onCompiled?: Nullable<(effect: Effect) => void>, onError?: Nullable<(effect: Effect, errors: string) => void>, indexParameters?: any, shaderLanguage?: ShaderLanguage): Effect;
    protected static _ConcatenateShader(source: string, defines: Nullable<string>, shaderVersion?: string): string;
    private _compileShader;
    private _compileRawShader;
    /**
     * @internal
     */
    _getShaderSource(shader: WebGLShader): Nullable<string>;
    /**
     * Directly creates a webGL program
     * @param pipelineContext  defines the pipeline context to attach to
     * @param vertexCode defines the vertex shader code to use
     * @param fragmentCode defines the fragment shader code to use
     * @param context defines the webGL context to use (if not set, the current one will be used)
     * @param transformFeedbackVaryings defines the list of transform feedback varyings to use
     * @returns the new webGL program
     */
    createRawShaderProgram(pipelineContext: IPipelineContext, vertexCode: string, fragmentCode: string, context?: WebGLRenderingContext, transformFeedbackVaryings?: Nullable<string[]>): WebGLProgram;
    /**
     * Creates a webGL program
     * @param pipelineContext  defines the pipeline context to attach to
     * @param vertexCode  defines the vertex shader code to use
     * @param fragmentCode defines the fragment shader code to use
     * @param defines defines the string containing the defines to use to compile the shaders
     * @param context defines the webGL context to use (if not set, the current one will be used)
     * @param transformFeedbackVaryings defines the list of transform feedback varyings to use
     * @returns the new webGL program
     */
    createShaderProgram(pipelineContext: IPipelineContext, vertexCode: string, fragmentCode: string, defines: Nullable<string>, context?: WebGLRenderingContext, transformFeedbackVaryings?: Nullable<string[]>): WebGLProgram;
    /**
     * Inline functions in shader code that are marked to be inlined
     * @param code code to inline
     * @returns inlined code
     */
    inlineShaderCode(code: string): string;
    /**
     * Creates a new pipeline context
     * @param shaderProcessingContext defines the shader processing context used during the processing if available
     * @returns the new pipeline
     */
    createPipelineContext(shaderProcessingContext: Nullable<ShaderProcessingContext>): IPipelineContext;
    /**
     * Creates a new material context
     * @returns the new context
     */
    createMaterialContext(): IMaterialContext | undefined;
    /**
     * Creates a new draw context
     * @returns the new context
     */
    createDrawContext(): IDrawContext | undefined;
    protected _createShaderProgram(pipelineContext: WebGLPipelineContext, vertexShader: WebGLShader, fragmentShader: WebGLShader, context: WebGLRenderingContext, transformFeedbackVaryings?: Nullable<string[]>): WebGLProgram;
    protected _finalizePipelineContext(pipelineContext: WebGLPipelineContext): void;
    /**
     * @internal
     */
    _preparePipelineContext(pipelineContext: IPipelineContext, vertexSourceCode: string, fragmentSourceCode: string, createAsRaw: boolean, rawVertexSourceCode: string, rawFragmentSourceCode: string, rebuildRebind: any, defines: Nullable<string>, transformFeedbackVaryings: Nullable<string[]>, key: string): void;
    /**
     * @internal
     */
    _isRenderingStateCompiled(pipelineContext: IPipelineContext): boolean;
    /**
     * @internal
     */
    _executeWhenRenderingStateIsCompiled(pipelineContext: IPipelineContext, action: () => void): void;
    /**
     * Gets the list of webGL uniform locations associated with a specific program based on a list of uniform names
     * @param pipelineContext defines the pipeline context to use
     * @param uniformsNames defines the list of uniform names
     * @returns an array of webGL uniform locations
     */
    getUniforms(pipelineContext: IPipelineContext, uniformsNames: string[]): Nullable<WebGLUniformLocation>[];
    /**
     * Gets the list of active attributes for a given webGL program
     * @param pipelineContext defines the pipeline context to use
     * @param attributesNames defines the list of attribute names to get
     * @returns an array of indices indicating the offset of each attribute
     */
    getAttributes(pipelineContext: IPipelineContext, attributesNames: string[]): number[];
    /**
     * Activates an effect, making it the current one (ie. the one used for rendering)
     * @param effect defines the effect to activate
     */
    enableEffect(effect: Nullable<Effect | DrawWrapper>): void;
    /**
     * Set the value of an uniform to a number (int)
     * @param uniform defines the webGL uniform location where to store the value
     * @param value defines the int number to store
     * @returns true if the value was set
     */
    setInt(uniform: Nullable<WebGLUniformLocation>, value: number): boolean;
    /**
     * Set the value of an uniform to a int2
     * @param uniform defines the webGL uniform location where to store the value
     * @param x defines the 1st component of the value
     * @param y defines the 2nd component of the value
     * @returns true if the value was set
     */
    setInt2(uniform: Nullable<WebGLUniformLocation>, x: number, y: number): boolean;
    /**
     * Set the value of an uniform to a int3
     * @param uniform defines the webGL uniform location where to store the value
     * @param x defines the 1st component of the value
     * @param y defines the 2nd component of the value
     * @param z defines the 3rd component of the value
     * @returns true if the value was set
     */
    setInt3(uniform: Nullable<WebGLUniformLocation>, x: number, y: number, z: number): boolean;
    /**
     * Set the value of an uniform to a int4
     * @param uniform defines the webGL uniform location where to store the value
     * @param x defines the 1st component of the value
     * @param y defines the 2nd component of the value
     * @param z defines the 3rd component of the value
     * @param w defines the 4th component of the value
     * @returns true if the value was set
     */
    setInt4(uniform: Nullable<WebGLUniformLocation>, x: number, y: number, z: number, w: number): boolean;
    /**
     * Set the value of an uniform to an array of int32
     * @param uniform defines the webGL uniform location where to store the value
     * @param array defines the array of int32 to store
     * @returns true if the value was set
     */
    setIntArray(uniform: Nullable<WebGLUniformLocation>, array: Int32Array): boolean;
    /**
     * Set the value of an uniform to an array of int32 (stored as vec2)
     * @param uniform defines the webGL uniform location where to store the value
     * @param array defines the array of int32 to store
     * @returns true if the value was set
     */
    setIntArray2(uniform: Nullable<WebGLUniformLocation>, array: Int32Array): boolean;
    /**
     * Set the value of an uniform to an array of int32 (stored as vec3)
     * @param uniform defines the webGL uniform location where to store the value
     * @param array defines the array of int32 to store
     * @returns true if the value was set
     */
    setIntArray3(uniform: Nullable<WebGLUniformLocation>, array: Int32Array): boolean;
    /**
     * Set the value of an uniform to an array of int32 (stored as vec4)
     * @param uniform defines the webGL uniform location where to store the value
     * @param array defines the array of int32 to store
     * @returns true if the value was set
     */
    setIntArray4(uniform: Nullable<WebGLUniformLocation>, array: Int32Array): boolean;
    /**
     * Set the value of an uniform to a number (unsigned int)
     * @param uniform defines the webGL uniform location where to store the value
     * @param value defines the unsigned int number to store
     * @returns true if the value was set
     */
    setUInt(uniform: Nullable<WebGLUniformLocation>, value: number): boolean;
    /**
     * Set the value of an uniform to a unsigned int2
     * @param uniform defines the webGL uniform location where to store the value
     * @param x defines the 1st component of the value
     * @param y defines the 2nd component of the value
     * @returns true if the value was set
     */
    setUInt2(uniform: Nullable<WebGLUniformLocation>, x: number, y: number): boolean;
    /**
     * Set the value of an uniform to a unsigned int3
     * @param uniform defines the webGL uniform location where to store the value
     * @param x defines the 1st component of the value
     * @param y defines the 2nd component of the value
     * @param z defines the 3rd component of the value
     * @returns true if the value was set
     */
    setUInt3(uniform: Nullable<WebGLUniformLocation>, x: number, y: number, z: number): boolean;
    /**
     * Set the value of an uniform to a unsigned int4
     * @param uniform defines the webGL uniform location where to store the value
     * @param x defines the 1st component of the value
     * @param y defines the 2nd component of the value
     * @param z defines the 3rd component of the value
     * @param w defines the 4th component of the value
     * @returns true if the value was set
     */
    setUInt4(uniform: Nullable<WebGLUniformLocation>, x: number, y: number, z: number, w: number): boolean;
    /**
     * Set the value of an uniform to an array of unsigned int32
     * @param uniform defines the webGL uniform location where to store the value
     * @param array defines the array of unsigned int32 to store
     * @returns true if the value was set
     */
    setUIntArray(uniform: Nullable<WebGLUniformLocation>, array: Uint32Array): boolean;
    /**
     * Set the value of an uniform to an array of unsigned int32 (stored as vec2)
     * @param uniform defines the webGL uniform location where to store the value
     * @param array defines the array of unsigned int32 to store
     * @returns true if the value was set
     */
    setUIntArray2(uniform: Nullable<WebGLUniformLocation>, array: Uint32Array): boolean;
    /**
     * Set the value of an uniform to an array of unsigned int32 (stored as vec3)
     * @param uniform defines the webGL uniform location where to store the value
     * @param array defines the array of unsigned int32 to store
     * @returns true if the value was set
     */
    setUIntArray3(uniform: Nullable<WebGLUniformLocation>, array: Uint32Array): boolean;
    /**
     * Set the value of an uniform to an array of unsigned int32 (stored as vec4)
     * @param uniform defines the webGL uniform location where to store the value
     * @param array defines the array of unsigned int32 to store
     * @returns true if the value was set
     */
    setUIntArray4(uniform: Nullable<WebGLUniformLocation>, array: Uint32Array): boolean;
    /**
     * Set the value of an uniform to an array of number
     * @param uniform defines the webGL uniform location where to store the value
     * @param array defines the array of number to store
     * @returns true if the value was set
     */
    setArray(uniform: Nullable<WebGLUniformLocation>, array: number[] | Float32Array): boolean;
    /**
     * Set the value of an uniform to an array of number (stored as vec2)
     * @param uniform defines the webGL uniform location where to store the value
     * @param array defines the array of number to store
     * @returns true if the value was set
     */
    setArray2(uniform: Nullable<WebGLUniformLocation>, array: number[] | Float32Array): boolean;
    /**
     * Set the value of an uniform to an array of number (stored as vec3)
     * @param uniform defines the webGL uniform location where to store the value
     * @param array defines the array of number to store
     * @returns true if the value was set
     */
    setArray3(uniform: Nullable<WebGLUniformLocation>, array: number[] | Float32Array): boolean;
    /**
     * Set the value of an uniform to an array of number (stored as vec4)
     * @param uniform defines the webGL uniform location where to store the value
     * @param array defines the array of number to store
     * @returns true if the value was set
     */
    setArray4(uniform: Nullable<WebGLUniformLocation>, array: number[] | Float32Array): boolean;
    /**
     * Set the value of an uniform to an array of float32 (stored as matrices)
     * @param uniform defines the webGL uniform location where to store the value
     * @param matrices defines the array of float32 to store
     * @returns true if the value was set
     */
    setMatrices(uniform: Nullable<WebGLUniformLocation>, matrices: Float32Array): boolean;
    /**
     * Set the value of an uniform to a matrix (3x3)
     * @param uniform defines the webGL uniform location where to store the value
     * @param matrix defines the Float32Array representing the 3x3 matrix to store
     * @returns true if the value was set
     */
    setMatrix3x3(uniform: Nullable<WebGLUniformLocation>, matrix: Float32Array): boolean;
    /**
     * Set the value of an uniform to a matrix (2x2)
     * @param uniform defines the webGL uniform location where to store the value
     * @param matrix defines the Float32Array representing the 2x2 matrix to store
     * @returns true if the value was set
     */
    setMatrix2x2(uniform: Nullable<WebGLUniformLocation>, matrix: Float32Array): boolean;
    /**
     * Set the value of an uniform to a number (float)
     * @param uniform defines the webGL uniform location where to store the value
     * @param value defines the float number to store
     * @returns true if the value was transferred
     */
    setFloat(uniform: Nullable<WebGLUniformLocation>, value: number): boolean;
    /**
     * Set the value of an uniform to a vec2
     * @param uniform defines the webGL uniform location where to store the value
     * @param x defines the 1st component of the value
     * @param y defines the 2nd component of the value
     * @returns true if the value was set
     */
    setFloat2(uniform: Nullable<WebGLUniformLocation>, x: number, y: number): boolean;
    /**
     * Set the value of an uniform to a vec3
     * @param uniform defines the webGL uniform location where to store the value
     * @param x defines the 1st component of the value
     * @param y defines the 2nd component of the value
     * @param z defines the 3rd component of the value
     * @returns true if the value was set
     */
    setFloat3(uniform: Nullable<WebGLUniformLocation>, x: number, y: number, z: number): boolean;
    /**
     * Set the value of an uniform to a vec4
     * @param uniform defines the webGL uniform location where to store the value
     * @param x defines the 1st component of the value
     * @param y defines the 2nd component of the value
     * @param z defines the 3rd component of the value
     * @param w defines the 4th component of the value
     * @returns true if the value was set
     */
    setFloat4(uniform: Nullable<WebGLUniformLocation>, x: number, y: number, z: number, w: number): boolean;
    /**
     * Apply all cached states (depth, culling, stencil and alpha)
     */
    applyStates(): void;
    /**
     * Enable or disable color writing
     * @param enable defines the state to set
     */
    setColorWrite(enable: boolean): void;
    /**
     * Gets a boolean indicating if color writing is enabled
     * @returns the current color writing state
     */
    getColorWrite(): boolean;
    /**
     * Gets the depth culling state manager
     */
    get depthCullingState(): DepthCullingState;
    /**
     * Gets the alpha state manager
     */
    get alphaState(): AlphaState;
    /**
     * Gets the stencil state manager
     */
    get stencilState(): StencilState;
    /**
     * Gets the stencil state composer
     */
    get stencilStateComposer(): StencilStateComposer;
    /**
     * Clears the list of texture accessible through engine.
     * This can help preventing texture load conflict due to name collision.
     */
    clearInternalTexturesCache(): void;
    /**
     * Force the entire cache to be cleared
     * You should not have to use this function unless your engine needs to share the webGL context with another engine
     * @param bruteForce defines a boolean to force clearing ALL caches (including stencil, detoh and alpha states)
     */
    wipeCaches(bruteForce?: boolean): void;
    /**
     * @internal
     */
    _getSamplingParameters(samplingMode: number, generateMipMaps: boolean): {
        min: number;
        mag: number;
    };
    /** @internal */
    protected _createTexture(): WebGLTexture;
    /** @internal */
    _createHardwareTexture(): HardwareTextureWrapper;
    /**
     * Creates an internal texture without binding it to a framebuffer
     * @internal
     * @param size defines the size of the texture
     * @param options defines the options used to create the texture
     * @param delayGPUTextureCreation true to delay the texture creation the first time it is really needed. false to create it right away
     * @param source source type of the texture
     * @returns a new internal texture
     */
    _createInternalTexture(size: TextureSize, options: boolean | InternalTextureCreationOptions, delayGPUTextureCreation?: boolean, source?: InternalTextureSource): InternalTexture;
    /**
     * @internal
     */
    _getUseSRGBBuffer(useSRGBBuffer: boolean, noMipmap: boolean): boolean;
    protected _createTextureBase(url: Nullable<string>, noMipmap: boolean, invertY: boolean, scene: Nullable<ISceneLike>, samplingMode: number | undefined, onLoad: Nullable<(texture: InternalTexture) => void> | undefined, onError: Nullable<(message: string, exception: any) => void> | undefined, prepareTexture: (texture: InternalTexture, extension: string, scene: Nullable<ISceneLike>, img: HTMLImageElement | ImageBitmap | {
        width: number;
        height: number;
    }, invertY: boolean, noMipmap: boolean, isCompressed: boolean, processFunction: (width: number, height: number, img: HTMLImageElement | ImageBitmap | {
        width: number;
        height: number;
    }, extension: string, texture: InternalTexture, continuationCallback: () => void) => boolean, samplingMode: number) => void, prepareTextureProcessFunction: (width: number, height: number, img: HTMLImageElement | ImageBitmap | {
        width: number;
        height: number;
    }, extension: string, texture: InternalTexture, continuationCallback: () => void) => boolean, buffer?: Nullable<string | ArrayBuffer | ArrayBufferView | HTMLImageElement | Blob | ImageBitmap>, fallback?: Nullable<InternalTexture>, format?: Nullable<number>, forcedExtension?: Nullable<string>, mimeType?: string, loaderOptions?: any, useSRGBBuffer?: boolean): InternalTexture;
    /**
     * Usually called from Texture.ts.
     * Passed information to create a WebGLTexture
     * @param url defines a value which contains one of the following:
     * * A conventional http URL, e.g. 'http://...' or 'file://...'
     * * A base64 string of in-line texture data, e.g. 'data:image/jpg;base64,/...'
     * * An indicator that data being passed using the buffer parameter, e.g. 'data:mytexture.jpg'
     * @param noMipmap defines a boolean indicating that no mipmaps shall be generated.  Ignored for compressed textures.  They must be in the file
     * @param invertY when true, image is flipped when loaded.  You probably want true. Certain compressed textures may invert this if their default is inverted (eg. ktx)
     * @param scene needed for loading to the correct scene
     * @param samplingMode mode with should be used sample / access the texture (Default: Texture.TRILINEAR_SAMPLINGMODE)
     * @param onLoad optional callback to be called upon successful completion
     * @param onError optional callback to be called upon failure
     * @param buffer a source of a file previously fetched as either a base64 string, an ArrayBuffer (compressed or image format), HTMLImageElement (image format), or a Blob
     * @param fallback an internal argument in case the function must be called again, due to etc1 not having alpha capabilities
     * @param format internal format.  Default: RGB when extension is '.jpg' else RGBA.  Ignored for compressed textures
     * @param forcedExtension defines the extension to use to pick the right loader
     * @param mimeType defines an optional mime type
     * @param loaderOptions options to be passed to the loader
     * @param creationFlags specific flags to use when creating the texture (Constants.TEXTURE_CREATIONFLAG_STORAGE for storage textures, for eg)
     * @param useSRGBBuffer defines if the texture must be loaded in a sRGB GPU buffer (if supported by the GPU).
     * @returns a InternalTexture for assignment back into BABYLON.Texture
     */
    createTexture(url: Nullable<string>, noMipmap: boolean, invertY: boolean, scene: Nullable<ISceneLike>, samplingMode?: number, onLoad?: Nullable<(texture: InternalTexture) => void>, onError?: Nullable<(message: string, exception: any) => void>, buffer?: Nullable<string | ArrayBuffer | ArrayBufferView | HTMLImageElement | Blob | ImageBitmap>, fallback?: Nullable<InternalTexture>, format?: Nullable<number>, forcedExtension?: Nullable<string>, mimeType?: string, loaderOptions?: any, creationFlags?: number, useSRGBBuffer?: boolean): InternalTexture;
    /**
     * Loads an image as an HTMLImageElement.
     * @param input url string, ArrayBuffer, or Blob to load
     * @param onLoad callback called when the image successfully loads
     * @param onError callback called when the image fails to load
     * @param offlineProvider offline provider for caching
     * @param mimeType optional mime type
     * @param imageBitmapOptions optional the options to use when creating an ImageBitmap
     * @returns the HTMLImageElement of the loaded image
     * @internal
     */
    static _FileToolsLoadImage(input: string | ArrayBuffer | ArrayBufferView | Blob, onLoad: (img: HTMLImageElement | ImageBitmap) => void, onError: (message?: string, exception?: any) => void, offlineProvider: Nullable<IOfflineProvider>, mimeType?: string, imageBitmapOptions?: ImageBitmapOptions): Nullable<HTMLImageElement>;
    /**
     * @internal
     */
    _rescaleTexture(source: InternalTexture, destination: InternalTexture, scene: Nullable<any>, internalFormat: number, onComplete: () => void): void;
    private _unpackFlipYCached;
    /**
     * In case you are sharing the context with other applications, it might
     * be interested to not cache the unpack flip y state to ensure a consistent
     * value would be set.
     */
    enableUnpackFlipYCached: boolean;
    /**
     * @internal
     */
    _unpackFlipY(value: boolean): void;
    /** @internal */
    _getUnpackAlignement(): number;
    private _getTextureTarget;
    /**
     * Update the sampling mode of a given texture
     * @param samplingMode defines the required sampling mode
     * @param texture defines the texture to update
     * @param generateMipMaps defines whether to generate mipmaps for the texture
     */
    updateTextureSamplingMode(samplingMode: number, texture: InternalTexture, generateMipMaps?: boolean): void;
    /**
     * Update the dimensions of a texture
     * @param texture texture to update
     * @param width new width of the texture
     * @param height new height of the texture
     * @param depth new depth of the texture
     */
    updateTextureDimensions(texture: InternalTexture, width: number, height: number, depth?: number): void;
    /**
     * Update the sampling mode of a given texture
     * @param texture defines the texture to update
     * @param wrapU defines the texture wrap mode of the u coordinates
     * @param wrapV defines the texture wrap mode of the v coordinates
     * @param wrapR defines the texture wrap mode of the r coordinates
     */
    updateTextureWrappingMode(texture: InternalTexture, wrapU: Nullable<number>, wrapV?: Nullable<number>, wrapR?: Nullable<number>): void;
    /**
     * @internal
     */
    _setupDepthStencilTexture(internalTexture: InternalTexture, size: number | {
        width: number;
        height: number;
        layers?: number;
    }, generateStencil: boolean, bilinearFiltering: boolean, comparisonFunction: number, samples?: number): void;
    /**
     * @internal
     */
    _uploadCompressedDataToTextureDirectly(texture: InternalTexture, internalFormat: number, width: number, height: number, data: ArrayBufferView, faceIndex?: number, lod?: number): void;
    /**
     * @internal
     */
    _uploadDataToTextureDirectly(texture: InternalTexture, imageData: ArrayBufferView, faceIndex?: number, lod?: number, babylonInternalFormat?: number, useTextureWidthAndHeight?: boolean): void;
    /**
     * Update a portion of an internal texture
     * @param texture defines the texture to update
     * @param imageData defines the data to store into the texture
     * @param xOffset defines the x coordinates of the update rectangle
     * @param yOffset defines the y coordinates of the update rectangle
     * @param width defines the width of the update rectangle
     * @param height defines the height of the update rectangle
     * @param faceIndex defines the face index if texture is a cube (0 by default)
     * @param lod defines the lod level to update (0 by default)
     * @param generateMipMaps defines whether to generate mipmaps or not
     */
    updateTextureData(texture: InternalTexture, imageData: ArrayBufferView, xOffset: number, yOffset: number, width: number, height: number, faceIndex?: number, lod?: number, generateMipMaps?: boolean): void;
    /**
     * @internal
     */
    _uploadArrayBufferViewToTexture(texture: InternalTexture, imageData: ArrayBufferView, faceIndex?: number, lod?: number): void;
    protected _prepareWebGLTextureContinuation(texture: InternalTexture, scene: Nullable<ISceneLike>, noMipmap: boolean, isCompressed: boolean, samplingMode: number): void;
    private _prepareWebGLTexture;
    /**
     * @internal
     */
    _setupFramebufferDepthAttachments(generateStencilBuffer: boolean, generateDepthBuffer: boolean, width: number, height: number, samples?: number): Nullable<WebGLRenderbuffer>;
    /**
     * @internal
     */
    _createRenderBuffer(width: number, height: number, samples: number, internalFormat: number, msInternalFormat: number, attachment: number, unbindBuffer?: boolean): Nullable<WebGLRenderbuffer>;
    _updateRenderBuffer(renderBuffer: Nullable<WebGLRenderbuffer>, width: number, height: number, samples: number, internalFormat: number, msInternalFormat: number, attachment: number, unbindBuffer?: boolean): Nullable<WebGLRenderbuffer>;
    /**
     * @internal
     */
    _releaseTexture(texture: InternalTexture): void;
    /**
     * @internal
     */
    _releaseRenderTargetWrapper(rtWrapper: RenderTargetWrapper): void;
    protected _deleteTexture(texture: Nullable<WebGLTexture>): void;
    protected _setProgram(program: WebGLProgram): void;
    protected _boundUniforms: {
        [key: number]: WebGLUniformLocation;
    };
    /**
     * Binds an effect to the webGL context
     * @param effect defines the effect to bind
     */
    bindSamplers(effect: Effect): void;
    private _activateCurrentTexture;
    /**
     * @internal
     */
    _bindTextureDirectly(target: number, texture: Nullable<InternalTexture>, forTextureDataUpdate?: boolean, force?: boolean): boolean;
    /**
     * @internal
     */
    _bindTexture(channel: number, texture: Nullable<InternalTexture>, name: string): void;
    /**
     * Unbind all textures from the webGL context
     */
    unbindAllTextures(): void;
    /**
     * Sets a texture to the according uniform.
     * @param channel The texture channel
     * @param uniform The uniform to set
     * @param texture The texture to apply
     * @param name The name of the uniform in the effect
     */
    setTexture(channel: number, uniform: Nullable<WebGLUniformLocation>, texture: Nullable<ThinTexture>, name: string): void;
    private _bindSamplerUniformToChannel;
    private _getTextureWrapMode;
    protected _setTexture(channel: number, texture: Nullable<ThinTexture>, isPartOfTextureArray?: boolean, depthStencilTexture?: boolean, name?: string): boolean;
    /**
     * Sets an array of texture to the webGL context
     * @param channel defines the channel where the texture array must be set
     * @param uniform defines the associated uniform location
     * @param textures defines the array of textures to bind
     * @param name name of the channel
     */
    setTextureArray(channel: number, uniform: Nullable<WebGLUniformLocation>, textures: ThinTexture[], name: string): void;
    /**
     * @internal
     */
    _setAnisotropicLevel(target: number, internalTexture: InternalTexture, anisotropicFilteringLevel: number): void;
    private _setTextureParameterFloat;
    private _setTextureParameterInteger;
    /**
     * Unbind all vertex attributes from the webGL context
     */
    unbindAllAttributes(): void;
    /**
     * Force the engine to release all cached effects. This means that next effect compilation will have to be done completely even if a similar effect was already compiled
     */
    releaseEffects(): void;
    /**
     * Dispose and release all associated resources
     */
    dispose(): void;
    /**
     * Attach a new callback raised when context lost event is fired
     * @param callback defines the callback to call
     */
    attachContextLostEvent(callback: (event: WebGLContextEvent) => void): void;
    /**
     * Attach a new callback raised when context restored event is fired
     * @param callback defines the callback to call
     */
    attachContextRestoredEvent(callback: (event: WebGLContextEvent) => void): void;
    /**
     * Get the current error code of the webGL context
     * @returns the error code
     * @see https://developer.mozilla.org/en-US/docs/Web/API/WebGLRenderingContext/getError
     */
    getError(): number;
    private _canRenderToFloatFramebuffer;
    private _canRenderToHalfFloatFramebuffer;
    private _canRenderToFramebuffer;
    /**
     * @internal
     */
    _getWebGLTextureType(type: number): number;
    /**
     * @internal
     */
    _getInternalFormat(format: number, useSRGBBuffer?: boolean): number;
    /**
     * @internal
     */
    _getRGBABufferInternalSizedFormat(type: number, format?: number, useSRGBBuffer?: boolean): number;
    /**
     * @internal
     */
    _getRGBAMultiSampleBufferFormat(type: number): number;
    /**
     * @internal
     */
    _loadFile(url: string, onSuccess: (data: string | ArrayBuffer, responseURL?: string) => void, onProgress?: (data: any) => void, offlineProvider?: IOfflineProvider, useArrayBuffer?: boolean, onError?: (request?: IWebRequest, exception?: any) => void): IFileRequest;
    /**
     * Loads a file from a url
     * @param url url to load
     * @param onSuccess callback called when the file successfully loads
     * @param onProgress callback called while file is loading (if the server supports this mode)
     * @param offlineProvider defines the offline provider for caching
     * @param useArrayBuffer defines a boolean indicating that date must be returned as ArrayBuffer
     * @param onError callback called when the file fails to load
     * @returns a file request object
     * @internal
     */
    static _FileToolsLoadFile(url: string, onSuccess: (data: string | ArrayBuffer, responseURL?: string) => void, onProgress?: (ev: ProgressEvent) => void, offlineProvider?: IOfflineProvider, useArrayBuffer?: boolean, onError?: (request?: WebRequest, exception?: LoadFileError) => void): IFileRequest;
    /**
     * Reads pixels from the current frame buffer. Please note that this function can be slow
     * @param x defines the x coordinate of the rectangle where pixels must be read
     * @param y defines the y coordinate of the rectangle where pixels must be read
     * @param width defines the width of the rectangle where pixels must be read
     * @param height defines the height of the rectangle where pixels must be read
     * @param hasAlpha defines whether the output should have alpha or not (defaults to true)
     * @param flushRenderer true to flush the renderer from the pending commands before reading the pixels
     * @returns a ArrayBufferView promise (Uint8Array) containing RGBA colors
     */
    readPixels(x: number, y: number, width: number, height: number, hasAlpha?: boolean, flushRenderer?: boolean): Promise<ArrayBufferView>;
    private static _IsSupported;
    private static _HasMajorPerformanceCaveat;
    /**
     * Gets a Promise<boolean> indicating if the engine can be instantiated (ie. if a webGL context can be found)
     */
    static get IsSupportedAsync(): Promise<boolean>;
    /**
     * Gets a boolean indicating if the engine can be instantiated (ie. if a webGL context can be found)
     */
    static get IsSupported(): boolean;
    /**
     * Gets a boolean indicating if the engine can be instantiated (ie. if a webGL context can be found)
     * @returns true if the engine can be created
     * @ignorenaming
     */
    static isSupported(): boolean;
    /**
     * Gets a boolean indicating if the engine can be instantiated on a performant device (ie. if a webGL context can be found and it does not use a slow implementation)
     */
    static get HasMajorPerformanceCaveat(): boolean;
    /**
     * Find the next highest power of two.
     * @param x Number to start search from.
     * @returns Next highest power of two.
     */
    static CeilingPOT(x: number): number;
    /**
     * Find the next lowest power of two.
     * @param x Number to start search from.
     * @returns Next lowest power of two.
     */
    static FloorPOT(x: number): number;
    /**
     * Find the nearest power of two.
     * @param x Number to start search from.
     * @returns Next nearest power of two.
     */
    static NearestPOT(x: number): number;
    /**
     * Get the closest exponent of two
     * @param value defines the value to approximate
     * @param max defines the maximum value to return
     * @param mode defines how to define the closest value
     * @returns closest exponent of two of the given value
     */
    static GetExponentOfTwo(value: number, max: number, mode?: number): number;
    /**
     * Queue a new function into the requested animation frame pool (ie. this function will be executed by the browser (or the javascript engine) for the next frame)
     * @param func - the function to be called
     * @param requester - the object that will request the next frame. Falls back to window.
     * @returns frame number
     */
    static QueueNewFrame(func: () => void, requester?: any): number;
    /**
     * Gets host document
     * @returns the host document object
     */
    getHostDocument(): Nullable<Document>;
}

}
declare module "babylonjs/Engines/WebGL/webGL2ShaderProcessors" {
import { ShaderLanguage } from "babylonjs/Materials/shaderLanguage";
import { IShaderProcessor } from "babylonjs/Engines/Processors/iShaderProcessor";
/** @internal */
export class WebGL2ShaderProcessor implements IShaderProcessor {
    shaderLanguage: ShaderLanguage;
    attributeProcessor(attribute: string): string;
    varyingProcessor(varying: string, isFragment: boolean): string;
    postProcessor(code: string, defines: string[], isFragment: boolean): string;
}

}
declare module "babylonjs/Engines/WebGL/webGLHardwareTexture" {
import { HardwareTextureWrapper } from "babylonjs/Materials/Textures/hardwareTextureWrapper";
import { Nullable } from "babylonjs/types";
/** @internal */
export class WebGLHardwareTexture implements HardwareTextureWrapper {
    private _webGLTexture;
    private _context;
    _MSAARenderBuffer: Nullable<WebGLRenderbuffer>;
    get underlyingResource(): Nullable<WebGLTexture>;
    constructor(existingTexture: Nullable<WebGLTexture> | undefined, context: WebGLRenderingContext);
    setUsage(): void;
    set(hardwareTexture: WebGLTexture): void;
    reset(): void;
    release(): void;
}

}
declare module "babylonjs/Engines/WebGL/webGLPipelineContext" {
import { IPipelineContext } from "babylonjs/Engines/IPipelineContext";
import { Nullable } from "babylonjs/types";
import { Effect } from "babylonjs/Materials/effect";
import { IMatrixLike, IVector2Like, IVector3Like, IVector4Like, IColor3Like, IColor4Like, IQuaternionLike } from "babylonjs/Maths/math.like";
import { ThinEngine } from "babylonjs/Engines/thinEngine";
/** @internal */
export class WebGLPipelineContext implements IPipelineContext {
    private _valueCache;
    private _uniforms;
    engine: ThinEngine;
    program: Nullable<WebGLProgram>;
    context?: WebGLRenderingContext;
    vertexShader?: WebGLShader;
    fragmentShader?: WebGLShader;
    isParallelCompiled: boolean;
    onCompiled?: () => void;
    transformFeedback?: WebGLTransformFeedback | null;
    vertexCompilationError: Nullable<string>;
    fragmentCompilationError: Nullable<string>;
    programLinkError: Nullable<string>;
    programValidationError: Nullable<string>;
    constructor();
    get isAsync(): boolean;
    get isReady(): boolean;
    _handlesSpectorRebuildCallback(onCompiled: (program: WebGLProgram) => void): void;
    _fillEffectInformation(effect: Effect, uniformBuffersNames: {
        [key: string]: number;
    }, uniformsNames: string[], uniforms: {
        [key: string]: Nullable<WebGLUniformLocation>;
    }, samplerList: string[], samplers: {
        [key: string]: number;
    }, attributesNames: string[], attributes: number[]): void;
    /**
     * Release all associated resources.
     **/
    dispose(): void;
    /**
     * @internal
     */
    _cacheMatrix(uniformName: string, matrix: IMatrixLike): boolean;
    /**
     * @internal
     */
    _cacheFloatN(_uniformName: string, _x: number, _y?: number, _z?: number, _w?: number): boolean;
    /**
     * @internal
     */
    _cacheFloat2(uniformName: string, x: number, y: number): boolean;
    /**
     * @internal
     */
    _cacheFloat3(uniformName: string, x: number, y: number, z: number): boolean;
    /**
     * @internal
     */
    _cacheFloat4(uniformName: string, x: number, y: number, z: number, w: number): boolean;
    /**
     * Sets an integer value on a uniform variable.
     * @param uniformName Name of the variable.
     * @param value Value to be set.
     */
    setInt: (uniformName: string, value: number) => void;
    /**
     * Sets a int2 on a uniform variable.
     * @param uniformName Name of the variable.
     * @param x First int in int2.
     * @param y Second int in int2.
     */
    setInt2: (uniformName: string, x: number, y: number) => void;
    /**
     * Sets a int3 on a uniform variable.
     * @param uniformName Name of the variable.
     * @param x First int in int3.
     * @param y Second int in int3.
     * @param z Third int in int3.
     */
    setInt3: (uniformName: string, x: number, y: number, z: number) => void;
    /**
     * Sets a int4 on a uniform variable.
     * @param uniformName Name of the variable.
     * @param x First int in int4.
     * @param y Second int in int4.
     * @param z Third int in int4.
     * @param w Fourth int in int4.
     */
    setInt4: (uniformName: string, x: number, y: number, z: number, w: number) => void;
    /**
     * Sets an int array on a uniform variable.
     * @param uniformName Name of the variable.
     * @param array array to be set.
     */
    setIntArray: (uniformName: string, array: Int32Array) => void;
    /**
     * Sets an int array 2 on a uniform variable. (Array is specified as single array eg. [1,2,3,4] will result in [[1,2],[3,4]] in the shader)
     * @param uniformName Name of the variable.
     * @param array array to be set.
     */
    setIntArray2: (uniformName: string, array: Int32Array) => void;
    /**
     * Sets an int array 3 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6] will result in [[1,2,3],[4,5,6]] in the shader)
     * @param uniformName Name of the variable.
     * @param array array to be set.
     */
    setIntArray3: (uniformName: string, array: Int32Array) => void;
    /**
     * Sets an int array 4 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6,7,8] will result in [[1,2,3,4],[5,6,7,8]] in the shader)
     * @param uniformName Name of the variable.
     * @param array array to be set.
     */
    setIntArray4: (uniformName: string, array: Int32Array) => void;
    /**
     * Sets an unsigned integer value on a uniform variable.
     * @param uniformName Name of the variable.
     * @param value Value to be set.
     */
    setUInt: (uniformName: string, value: number) => void;
    /**
     * Sets a unsigned int2 on a uniform variable.
     * @param uniformName Name of the variable.
     * @param x First unsigned int in uint2.
     * @param y Second unsigned int in uint2.
     */
    setUInt2: (uniformName: string, x: number, y: number) => void;
    /**
     * Sets a unsigned int3 on a uniform variable.
     * @param uniformName Name of the variable.
     * @param x First unsigned int in uint3.
     * @param y Second unsigned int in uint3.
     * @param z Third unsigned int in uint3.
     */
    setUInt3: (uniformName: string, x: number, y: number, z: number) => void;
    /**
     * Sets a unsigned int4 on a uniform variable.
     * @param uniformName Name of the variable.
     * @param x First unsigned int in uint4.
     * @param y Second unsigned int in uint4.
     * @param z Third unsigned int in uint4.
     * @param w Fourth unsigned int in uint4.
     */
    setUInt4: (uniformName: string, x: number, y: number, z: number, w: number) => void;
    /**
     * Sets an unsigned int array on a uniform variable.
     * @param uniformName Name of the variable.
     * @param array array to be set.
     */
    setUIntArray: (uniformName: string, array: Uint32Array) => void;
    /**
     * Sets an unsigned int array 2 on a uniform variable. (Array is specified as single array eg. [1,2,3,4] will result in [[1,2],[3,4]] in the shader)
     * @param uniformName Name of the variable.
     * @param array array to be set.
     */
    setUIntArray2: (uniformName: string, array: Uint32Array) => void;
    /**
     * Sets an unsigned int array 3 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6] will result in [[1,2,3],[4,5,6]] in the shader)
     * @param uniformName Name of the variable.
     * @param array array to be set.
     */
    setUIntArray3: (uniformName: string, array: Uint32Array) => void;
    /**
     * Sets an unsigned int array 4 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6,7,8] will result in [[1,2,3,4],[5,6,7,8]] in the shader)
     * @param uniformName Name of the variable.
     * @param array array to be set.
     */
    setUIntArray4: (uniformName: string, array: Uint32Array) => void;
    /**
     * Sets an array on a uniform variable.
     * @param uniformName Name of the variable.
     * @param array array to be set.
     */
    setArray: (uniformName: string, array: number[]) => void;
    /**
     * Sets an array 2 on a uniform variable. (Array is specified as single array eg. [1,2,3,4] will result in [[1,2],[3,4]] in the shader)
     * @param uniformName Name of the variable.
     * @param array array to be set.
     */
    setArray2: (uniformName: string, array: number[]) => void;
    /**
     * Sets an array 3 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6] will result in [[1,2,3],[4,5,6]] in the shader)
     * @param uniformName Name of the variable.
     * @param array array to be set.
     * @returns this effect.
     */
    setArray3: (uniformName: string, array: number[]) => void;
    /**
     * Sets an array 4 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6,7,8] will result in [[1,2,3,4],[5,6,7,8]] in the shader)
     * @param uniformName Name of the variable.
     * @param array array to be set.
     */
    setArray4: (uniformName: string, array: number[]) => void;
    /**
     * Sets matrices on a uniform variable.
     * @param uniformName Name of the variable.
     * @param matrices matrices to be set.
     */
    setMatrices: (uniformName: string, matrices: Float32Array) => void;
    /**
     * Sets matrix on a uniform variable.
     * @param uniformName Name of the variable.
     * @param matrix matrix to be set.
     */
    setMatrix(uniformName: string, matrix: IMatrixLike): void;
    /**
     * Sets a 3x3 matrix on a uniform variable. (Specified as [1,2,3,4,5,6,7,8,9] will result in [1,2,3][4,5,6][7,8,9] matrix)
     * @param uniformName Name of the variable.
     * @param matrix matrix to be set.
     */
    setMatrix3x3: (uniformName: string, matrix: Float32Array) => void;
    /**
     * Sets a 2x2 matrix on a uniform variable. (Specified as [1,2,3,4] will result in [1,2][3,4] matrix)
     * @param uniformName Name of the variable.
     * @param matrix matrix to be set.
     */
    setMatrix2x2: (uniformName: string, matrix: Float32Array) => void;
    /**
     * Sets a float on a uniform variable.
     * @param uniformName Name of the variable.
     * @param value value to be set.
     * @returns this effect.
     */
    setFloat: (uniformName: string, value: number) => void;
    /**
     * Sets a Vector2 on a uniform variable.
     * @param uniformName Name of the variable.
     * @param vector2 vector2 to be set.
     */
    setVector2(uniformName: string, vector2: IVector2Like): void;
    /**
     * Sets a float2 on a uniform variable.
     * @param uniformName Name of the variable.
     * @param x First float in float2.
     * @param y Second float in float2.
     */
    setFloat2: (uniformName: string, x: number, y: number) => void;
    /**
     * Sets a Vector3 on a uniform variable.
     * @param uniformName Name of the variable.
     * @param vector3 Value to be set.
     */
    setVector3(uniformName: string, vector3: IVector3Like): void;
    /**
     * Sets a float3 on a uniform variable.
     * @param uniformName Name of the variable.
     * @param x First float in float3.
     * @param y Second float in float3.
     * @param z Third float in float3.
     */
    setFloat3: (uniformName: string, x: number, y: number, z: number) => void;
    /**
     * Sets a Vector4 on a uniform variable.
     * @param uniformName Name of the variable.
     * @param vector4 Value to be set.
     */
    setVector4(uniformName: string, vector4: IVector4Like): void;
    /**
     * Sets a Quaternion on a uniform variable.
     * @param uniformName Name of the variable.
     * @param quaternion Value to be set.
     */
    setQuaternion(uniformName: string, quaternion: IQuaternionLike): void;
    /**
     * Sets a float4 on a uniform variable.
     * @param uniformName Name of the variable.
     * @param x First float in float4.
     * @param y Second float in float4.
     * @param z Third float in float4.
     * @param w Fourth float in float4.
     * @returns this effect.
     */
    setFloat4: (uniformName: string, x: number, y: number, z: number, w: number) => void;
    /**
     * Sets a Color3 on a uniform variable.
     * @param uniformName Name of the variable.
     * @param color3 Value to be set.
     */
    setColor3(uniformName: string, color3: IColor3Like): void;
    /**
     * Sets a Color4 on a uniform variable.
     * @param uniformName Name of the variable.
     * @param color3 Value to be set.
     * @param alpha Alpha value to be set.
     */
    setColor4(uniformName: string, color3: IColor3Like, alpha: number): void;
    /**
     * Sets a Color4 on a uniform variable
     * @param uniformName defines the name of the variable
     * @param color4 defines the value to be set
     */
    setDirectColor4(uniformName: string, color4: IColor4Like): void;
    _getVertexShaderCode(): string | null;
    _getFragmentShaderCode(): string | null;
}

}
declare module "babylonjs/Engines/WebGL/webGLRenderTargetWrapper" {
import { InternalTexture } from "babylonjs/Materials/Textures/internalTexture";
import { TextureSize } from "babylonjs/Materials/Textures/textureCreationOptions";
import { Nullable } from "babylonjs/types";
import { RenderTargetWrapper } from "babylonjs/Engines/renderTargetWrapper";
import { ThinEngine } from "babylonjs/Engines/thinEngine";
/** @internal */
export class WebGLRenderTargetWrapper extends RenderTargetWrapper {
    private _context;
    _framebuffer: Nullable<WebGLFramebuffer>;
    _depthStencilBuffer: Nullable<WebGLRenderbuffer>;
    _MSAAFramebuffer: Nullable<WebGLFramebuffer>;
    _colorTextureArray: Nullable<WebGLTexture>;
    _depthStencilTextureArray: Nullable<WebGLTexture>;
    constructor(isMulti: boolean, isCube: boolean, size: TextureSize, engine: ThinEngine, context: WebGLRenderingContext);
    protected _cloneRenderTargetWrapper(): Nullable<RenderTargetWrapper>;
    protected _swapRenderTargetWrapper(target: WebGLRenderTargetWrapper): void;
    /**
     * Shares the depth buffer of this render target with another render target.
     * @internal
     * @param renderTarget Destination renderTarget
     */
    _shareDepth(renderTarget: WebGLRenderTargetWrapper): void;
    /**
     * Binds a texture to this render target on a specific attachment
     * @param texture The texture to bind to the framebuffer
     * @param attachmentIndex Index of the attachment
     * @param faceIndex The face of the texture to render to in case of cube texture
     * @param lodLevel defines the lod level to bind to the frame buffer
     */
    private _bindTextureRenderTarget;
    /**
     * Set a texture in the textures array
     * @param texture the texture to set
     * @param index the index in the textures array to set
     * @param disposePrevious If this function should dispose the previous texture
     */
    setTexture(texture: InternalTexture, index?: number, disposePrevious?: boolean): void;
    dispose(disposeOnlyFramebuffers?: boolean): void;
}

}
declare module "babylonjs/Engines/WebGL/webGLShaderProcessors" {
import { ShaderLanguage } from "babylonjs/Materials/shaderLanguage";
import { Nullable } from "babylonjs/types";
import { IShaderProcessor } from "babylonjs/Engines/Processors/iShaderProcessor";
import { ShaderProcessingContext } from "babylonjs/Engines/Processors/shaderProcessingOptions";
import { ThinEngine } from "babylonjs/Engines/thinEngine";
/** @internal */
export class WebGLShaderProcessor implements IShaderProcessor {
    shaderLanguage: ShaderLanguage;
    postProcessor(code: string, defines: string[], isFragment: boolean, processingContext: Nullable<ShaderProcessingContext>, engine: ThinEngine): string;
}
export {};

}
declare module "babylonjs/Engines/WebGPU/Extensions/engine.alpha" {
export {};

}
declare module "babylonjs/Engines/WebGPU/Extensions/engine.computeShader" {
import { Nullable } from "babylonjs/types";
module "babylonjs/Engines/webgpuEngine" {
    interface WebGPUEngine {
        /** @internal */
        _createComputePipelineStageDescriptor(computeShader: string, defines: Nullable<string>, entryPoint: string): GPUProgrammableStage;
    }
}

}
declare module "babylonjs/Engines/WebGPU/Extensions/engine.cubeTexture" {
export {};

}
declare module "babylonjs/Engines/WebGPU/Extensions/engine.debugging" {
export {};

}
declare module "babylonjs/Engines/WebGPU/Extensions/engine.dynamicBuffer" {
export {};

}
declare module "babylonjs/Engines/WebGPU/Extensions/engine.dynamicTexture" {
export {};

}
declare module "babylonjs/Engines/WebGPU/Extensions/engine.externalTexture" {
import { ExternalTexture } from "babylonjs/Materials/Textures/externalTexture";
import { Nullable } from "babylonjs/types";
module "babylonjs/Materials/effect" {
    interface Effect {
        /**
         * Sets an external texture on the engine to be used in the shader.
         * @param name Name of the external texture variable.
         * @param texture Texture to set.
         */
        setExternalTexture(name: string, texture: Nullable<ExternalTexture>): void;
    }
}

}
declare module "babylonjs/Engines/WebGPU/Extensions/engine.multiRender" {
export {};

}
declare module "babylonjs/Engines/WebGPU/Extensions/engine.query" {
export {};

}
declare module "babylonjs/Engines/WebGPU/Extensions/engine.rawTexture" {
export {};

}
declare module "babylonjs/Engines/WebGPU/Extensions/engine.readTexture" {
export {};

}
declare module "babylonjs/Engines/WebGPU/Extensions/engine.renderTarget" {
export {};

}
declare module "babylonjs/Engines/WebGPU/Extensions/engine.renderTargetCube" {
export {};

}
declare module "babylonjs/Engines/WebGPU/Extensions/engine.storageBuffer" {
import { Nullable } from "babylonjs/types";
import { StorageBuffer } from "babylonjs/Buffers/storageBuffer";
module "babylonjs/Materials/effect" {
    interface Effect {
        /**
         * Sets a storage buffer on the engine to be used in the shader.
         * @param name Name of the storage buffer variable.
         * @param buffer Storage buffer to set.
         */
        setStorageBuffer(name: string, buffer: Nullable<StorageBuffer>): void;
    }
}
export {};

}
declare module "babylonjs/Engines/WebGPU/Extensions/engine.textureSampler" {
import { Nullable } from "babylonjs/types";
import { TextureSampler } from "babylonjs/Materials/Textures/textureSampler";
module "babylonjs/Materials/effect" {
    interface Effect {
        /**
         * Sets a sampler on the engine to be used in the shader.
         * @param name Name of the sampler variable.
         * @param sampler Sampler to set.
         */
        setTextureSampler(name: string, sampler: Nullable<TextureSampler>): void;
    }
}
export {};

}
declare module "babylonjs/Engines/WebGPU/Extensions/engine.uniformBuffer" {
export {};

}
declare module "babylonjs/Engines/WebGPU/Extensions/engine.videoTexture" {
export {};

}
declare module "babylonjs/Engines/WebGPU/Extensions/index" {
import "babylonjs/Engines/WebGPU/Extensions/engine.alpha";
import "babylonjs/Engines/WebGPU/Extensions/engine.computeShader";
import "babylonjs/Engines/WebGPU/Extensions/engine.cubeTexture";
import "babylonjs/Engines/WebGPU/Extensions/engine.debugging";
import "babylonjs/Engines/WebGPU/Extensions/engine.dynamicBuffer";
import "babylonjs/Engines/WebGPU/Extensions/engine.dynamicTexture";
import "babylonjs/Engines/WebGPU/Extensions/engine.externalTexture";
import "babylonjs/Engines/WebGPU/Extensions/engine.multiRender";
import "babylonjs/Engines/WebGPU/Extensions/engine.query";
import "babylonjs/Engines/WebGPU/Extensions/engine.rawTexture";
import "babylonjs/Engines/WebGPU/Extensions/engine.readTexture";
import "babylonjs/Engines/WebGPU/Extensions/engine.renderTarget";
import "babylonjs/Engines/WebGPU/Extensions/engine.renderTargetCube";
import "babylonjs/Engines/WebGPU/Extensions/engine.textureSampler";
import "babylonjs/Engines/WebGPU/Extensions/engine.storageBuffer";
import "babylonjs/Engines/WebGPU/Extensions/engine.uniformBuffer";
import "babylonjs/Engines/WebGPU/Extensions/engine.videoTexture";

}
declare module "babylonjs/Engines/WebGPU/webgpuBufferManager" {
import { DataBuffer } from "babylonjs/Buffers/dataBuffer";
import { WebGPUDataBuffer } from "babylonjs/Meshes/WebGPU/webgpuDataBuffer";
import { Nullable } from "babylonjs/types";
/** @internal */
export class WebGPUBufferManager {
    private _device;
    private _deferredReleaseBuffers;
    private static _IsGPUBuffer;
    constructor(device: GPUDevice);
    createRawBuffer(viewOrSize: ArrayBufferView | number, flags: GPUBufferUsageFlags, mappedAtCreation?: boolean): GPUBuffer;
    createBuffer(viewOrSize: ArrayBufferView | number, flags: GPUBufferUsageFlags): WebGPUDataBuffer;
    setRawData(buffer: GPUBuffer, dstByteOffset: number, src: ArrayBufferView, srcByteOffset: number, byteLength: number): void;
    setSubData(dataBuffer: WebGPUDataBuffer, dstByteOffset: number, src: ArrayBufferView, srcByteOffset?: number, byteLength?: number): void;
    private _getHalfFloatAsFloatRGBAArrayBuffer;
    readDataFromBuffer(gpuBuffer: GPUBuffer, size: number, width: number, height: number, bytesPerRow: number, bytesPerRowAligned: number, type?: number, offset?: number, buffer?: Nullable<ArrayBufferView>, destroyBuffer?: boolean, noDataConversion?: boolean): Promise<ArrayBufferView>;
    releaseBuffer(buffer: DataBuffer | GPUBuffer): boolean;
    destroyDeferredBuffers(): void;
}

}
declare module "babylonjs/Engines/WebGPU/webgpuBundleList" {
import { Nullable } from "babylonjs/types";
/** @internal */
interface IWebGPURenderItem {
    run(renderPass: GPURenderPassEncoder): void;
    clone(): IWebGPURenderItem;
}
/** @internal */
export class WebGPURenderItemViewport implements IWebGPURenderItem {
    x: number;
    y: number;
    w: number;
    h: number;
    constructor(x: number, y: number, w: number, h: number);
    run(renderPass: GPURenderPassEncoder): void;
    clone(): WebGPURenderItemViewport;
}
/** @internal */
export class WebGPURenderItemScissor implements IWebGPURenderItem {
    x: number;
    y: number;
    w: number;
    h: number;
    constructor(x: number, y: number, w: number, h: number);
    run(renderPass: GPURenderPassEncoder): void;
    clone(): WebGPURenderItemScissor;
}
/** @internal */
export class WebGPURenderItemStencilRef implements IWebGPURenderItem {
    ref: number;
    constructor(ref: number);
    run(renderPass: GPURenderPassEncoder): void;
    clone(): WebGPURenderItemStencilRef;
}
/** @internal */
export class WebGPURenderItemBlendColor implements IWebGPURenderItem {
    color: Nullable<number>[];
    constructor(color: Nullable<number>[]);
    run(renderPass: GPURenderPassEncoder): void;
    clone(): WebGPURenderItemBlendColor;
}
/** @internal */
export class WebGPURenderItemBeginOcclusionQuery implements IWebGPURenderItem {
    query: number;
    constructor(query: number);
    run(renderPass: GPURenderPassEncoder): void;
    clone(): WebGPURenderItemBeginOcclusionQuery;
}
/** @internal */
export class WebGPURenderItemEndOcclusionQuery implements IWebGPURenderItem {
    constructor();
    run(renderPass: GPURenderPassEncoder): void;
    clone(): WebGPURenderItemEndOcclusionQuery;
}
/** @internal */
export class WebGPUBundleList {
    private _device;
    private _bundleEncoder;
    private _list;
    private _listLength;
    private _currentItemIsBundle;
    private _currentBundleList;
    numDrawCalls: number;
    constructor(device: GPUDevice);
    addBundle(bundle?: GPURenderBundle): void;
    private _finishBundle;
    addItem(item: IWebGPURenderItem): void;
    getBundleEncoder(colorFormats: (GPUTextureFormat | null)[], depthStencilFormat: GPUTextureFormat | undefined, sampleCount: number): GPURenderBundleEncoder;
    close(): void;
    run(renderPass: GPURenderPassEncoder): void;
    reset(): void;
    clone(): WebGPUBundleList;
}
export {};

}
declare module "babylonjs/Engines/WebGPU/webgpuCacheBindGroups" {
import { WebGPUCacheSampler } from "babylonjs/Engines/WebGPU/webgpuCacheSampler";
import { WebGPUMaterialContext } from "babylonjs/Engines/WebGPU/webgpuMaterialContext";
import { WebGPUPipelineContext } from "babylonjs/Engines/WebGPU/webgpuPipelineContext";
import { WebGPUEngine } from "babylonjs/Engines/webgpuEngine";
import { WebGPUDrawContext } from "babylonjs/Engines/WebGPU/webgpuDrawContext";
/** @internal */
export class WebGPUCacheBindGroups {
    static NumBindGroupsCreatedTotal: number;
    static NumBindGroupsCreatedLastFrame: number;
    static NumBindGroupsLookupLastFrame: number;
    static NumBindGroupsNoLookupLastFrame: number;
    private static _Cache;
    private static _NumBindGroupsCreatedCurrentFrame;
    private static _NumBindGroupsLookupCurrentFrame;
    private static _NumBindGroupsNoLookupCurrentFrame;
    private _device;
    private _cacheSampler;
    private _engine;
    disabled: boolean;
    static get Statistics(): {
        totalCreated: number;
        lastFrameCreated: number;
        lookupLastFrame: number;
        noLookupLastFrame: number;
    };
    constructor(device: GPUDevice, cacheSampler: WebGPUCacheSampler, engine: WebGPUEngine);
    endFrame(): void;
    /**
     * Cache is currently based on the uniform/storage buffers, samplers and textures used by the binding groups.
     * Note that all uniform buffers have an offset of 0 in Babylon and we don't have a use case where we would have the same buffer used with different capacity values:
     * that means we don't need to factor in the offset/size of the buffer in the cache, only the id
     * @param webgpuPipelineContext
     * @param drawContext
     * @param materialContext
     */
    getBindGroups(webgpuPipelineContext: WebGPUPipelineContext, drawContext: WebGPUDrawContext, materialContext: WebGPUMaterialContext): GPUBindGroup[];
}

}
declare module "babylonjs/Engines/WebGPU/webgpuCacheRenderPipeline" {
import { Effect } from "babylonjs/Materials/effect";
import { InternalTexture } from "babylonjs/Materials/Textures/internalTexture";
import { VertexBuffer } from "babylonjs/Buffers/buffer";
import { DataBuffer } from "babylonjs/Buffers/dataBuffer";
import { Nullable } from "babylonjs/types";
/** @internal */
export abstract class WebGPUCacheRenderPipeline {
    static NumCacheHitWithoutHash: number;
    static NumCacheHitWithHash: number;
    static NumCacheMiss: number;
    static NumPipelineCreationLastFrame: number;
    disabled: boolean;
    private static _NumPipelineCreationCurrentFrame;
    protected _states: number[];
    protected _statesLength: number;
    protected _stateDirtyLowestIndex: number;
    lastStateDirtyLowestIndex: number;
    private _device;
    private _isDirty;
    private _emptyVertexBuffer;
    private _parameter;
    private _kMaxVertexBufferStride;
    private _shaderId;
    private _alphaToCoverageEnabled;
    private _frontFace;
    private _cullEnabled;
    private _cullFace;
    private _clampDepth;
    private _rasterizationState;
    private _depthBias;
    private _depthBiasClamp;
    private _depthBiasSlopeScale;
    private _colorFormat;
    private _webgpuColorFormat;
    private _mrtAttachments1;
    private _mrtAttachments2;
    private _mrtFormats;
    private _mrtEnabledMask;
    private _alphaBlendEnabled;
    private _alphaBlendFuncParams;
    private _alphaBlendEqParams;
    private _writeMask;
    private _colorStates;
    private _depthStencilFormat;
    private _webgpuDepthStencilFormat;
    private _depthTestEnabled;
    private _depthWriteEnabled;
    private _depthCompare;
    private _stencilEnabled;
    private _stencilFrontCompare;
    private _stencilFrontDepthFailOp;
    private _stencilFrontPassOp;
    private _stencilFrontFailOp;
    private _stencilReadMask;
    private _stencilWriteMask;
    private _depthStencilState;
    private _vertexBuffers;
    private _overrideVertexBuffers;
    private _indexBuffer;
    private _textureState;
    private _useTextureStage;
    constructor(device: GPUDevice, emptyVertexBuffer: VertexBuffer, useTextureStage: boolean);
    reset(): void;
    protected abstract _getRenderPipeline(param: {
        token: any;
        pipeline: Nullable<GPURenderPipeline>;
    }): void;
    protected abstract _setRenderPipeline(param: {
        token: any;
        pipeline: Nullable<GPURenderPipeline>;
    }): void;
    readonly vertexBuffers: VertexBuffer[];
    get colorFormats(): (GPUTextureFormat | null)[];
    readonly mrtAttachments: number[];
    readonly mrtTextureArray: InternalTexture[];
    readonly mrtTextureCount: number;
    getRenderPipeline(fillMode: number, effect: Effect, sampleCount: number, textureState?: number): GPURenderPipeline;
    endFrame(): void;
    setAlphaToCoverage(enabled: boolean): void;
    setFrontFace(frontFace: number): void;
    setCullEnabled(enabled: boolean): void;
    setCullFace(cullFace: number): void;
    setClampDepth(clampDepth: boolean): void;
    resetDepthCullingState(): void;
    setDepthCullingState(cullEnabled: boolean, frontFace: number, cullFace: number, zOffset: number, zOffsetUnits: number, depthTestEnabled: boolean, depthWriteEnabled: boolean, depthCompare: Nullable<number>): void;
    setDepthBias(depthBias: number): void;
    setDepthBiasSlopeScale(depthBiasSlopeScale: number): void;
    setColorFormat(format: GPUTextureFormat | null): void;
    setMRTAttachments(attachments: number[]): void;
    setMRT(textureArray: InternalTexture[], textureCount?: number): void;
    setAlphaBlendEnabled(enabled: boolean): void;
    setAlphaBlendFactors(factors: Array<Nullable<number>>, operations: Array<Nullable<number>>): void;
    setWriteMask(mask: number): void;
    setDepthStencilFormat(format: GPUTextureFormat | undefined): void;
    setDepthTestEnabled(enabled: boolean): void;
    setDepthWriteEnabled(enabled: boolean): void;
    setDepthCompare(func: Nullable<number>): void;
    setStencilEnabled(enabled: boolean): void;
    setStencilCompare(func: Nullable<number>): void;
    setStencilDepthFailOp(op: Nullable<number>): void;
    setStencilPassOp(op: Nullable<number>): void;
    setStencilFailOp(op: Nullable<number>): void;
    setStencilReadMask(mask: number): void;
    setStencilWriteMask(mask: number): void;
    resetStencilState(): void;
    setStencilState(stencilEnabled: boolean, compare: Nullable<number>, depthFailOp: Nullable<number>, passOp: Nullable<number>, failOp: Nullable<number>, readMask: number, writeMask: number): void;
    setBuffers(vertexBuffers: Nullable<{
        [key: string]: Nullable<VertexBuffer>;
    }>, indexBuffer: Nullable<DataBuffer>, overrideVertexBuffers: Nullable<{
        [key: string]: Nullable<VertexBuffer>;
    }>): void;
    private static _GetTopology;
    private static _GetAphaBlendOperation;
    private static _GetAphaBlendFactor;
    private static _GetCompareFunction;
    private static _GetStencilOpFunction;
    private static _GetVertexInputDescriptorFormat;
    private _getAphaBlendState;
    private _getColorBlendState;
    private _setShaderStage;
    private _setRasterizationState;
    private _setColorStates;
    private _setDepthStencilState;
    private _setVertexState;
    private _setTextureState;
    private _createPipelineLayout;
    private _createPipelineLayoutWithTextureStage;
    private _getVertexInputDescriptor;
    private _createRenderPipeline;
}

}
declare module "babylonjs/Engines/WebGPU/webgpuCacheRenderPipelineString" {
import { Nullable } from "babylonjs/types";
import { WebGPUCacheRenderPipeline } from "babylonjs/Engines/WebGPU/webgpuCacheRenderPipeline";
/**
 * Class not used, WebGPUCacheRenderPipelineTree is faster
 * @internal
 */
export class WebGPUCacheRenderPipelineString extends WebGPUCacheRenderPipeline {
    private static _Cache;
    protected _getRenderPipeline(param: {
        token: any;
        pipeline: Nullable<GPURenderPipeline>;
    }): void;
    protected _setRenderPipeline(param: {
        token: any;
        pipeline: Nullable<GPURenderPipeline>;
    }): void;
}

}
declare module "babylonjs/Engines/WebGPU/webgpuCacheRenderPipelineTree" {
import { VertexBuffer } from "babylonjs/Buffers/buffer";
import { Nullable } from "babylonjs/types";
import { WebGPUCacheRenderPipeline } from "babylonjs/Engines/WebGPU/webgpuCacheRenderPipeline";
/** @internal */
class NodeState {
    values: {
        [id: number]: NodeState;
    };
    pipeline: GPURenderPipeline;
    constructor();
    count(): [number, number];
}
/** @internal */
export class WebGPUCacheRenderPipelineTree extends WebGPUCacheRenderPipeline {
    private static _Cache;
    private _nodeStack;
    static GetNodeCounts(): {
        nodeCount: number;
        pipelineCount: number;
    };
    static _GetPipelines(node: NodeState, pipelines: Array<Array<number>>, curPath: Array<number>, curPathLen: number): void;
    static GetPipelines(): Array<Array<number>>;
    constructor(device: GPUDevice, emptyVertexBuffer: VertexBuffer, useTextureStage: boolean);
    protected _getRenderPipeline(param: {
        token: any;
        pipeline: Nullable<GPURenderPipeline>;
    }): void;
    protected _setRenderPipeline(param: {
        token: NodeState;
        pipeline: Nullable<GPURenderPipeline>;
    }): void;
}
export {};

}
declare module "babylonjs/Engines/WebGPU/webgpuCacheSampler" {
import { TextureSampler } from "babylonjs/Materials/Textures/textureSampler";
import { Nullable } from "babylonjs/types";
/** @internal */
export class WebGPUCacheSampler {
    private _samplers;
    private _device;
    disabled: boolean;
    constructor(device: GPUDevice);
    static GetSamplerHashCode(sampler: TextureSampler): number;
    private static _GetSamplerFilterDescriptor;
    private static _GetWrappingMode;
    private static _GetSamplerWrappingDescriptor;
    private static _GetSamplerDescriptor;
    static GetCompareFunction(compareFunction: Nullable<number>): GPUCompareFunction;
    getSampler(sampler: TextureSampler, bypassCache?: boolean, hash?: number): GPUSampler;
}

}
declare module "babylonjs/Engines/WebGPU/webgpuClearQuad" {
import { InternalTexture } from "babylonjs/Materials/Textures/internalTexture";
import { IColor4Like } from "babylonjs/Maths/math.like";
import { VertexBuffer } from "babylonjs/Buffers/buffer";
import { Nullable } from "babylonjs/types";
import { WebGPUEngine } from "babylonjs/Engines/webgpuEngine";
import "babylonjs/Shaders/clearQuad.vertex";
import "babylonjs/Shaders/clearQuad.fragment";
/** @internal */
export class WebGPUClearQuad {
    private _device;
    private _engine;
    private _cacheRenderPipeline;
    private _effect;
    private _bindGroups;
    private _depthTextureFormat;
    private _bundleCache;
    private _keyTemp;
    setDepthStencilFormat(format: GPUTextureFormat | undefined): void;
    setColorFormat(format: GPUTextureFormat | null): void;
    setMRTAttachments(attachments: number[], textureArray: InternalTexture[], textureCount: number): void;
    constructor(device: GPUDevice, engine: WebGPUEngine, emptyVertexBuffer: VertexBuffer);
    clear(renderPass: Nullable<GPURenderPassEncoder>, clearColor?: Nullable<IColor4Like>, clearDepth?: boolean, clearStencil?: boolean, sampleCount?: number): Nullable<GPURenderBundle>;
}

}
declare module "babylonjs/Engines/WebGPU/webgpuComputeContext" {
import { IComputeContext } from "babylonjs/Compute/IComputeContext";
import { ComputeBindingList, ComputeBindingMapping } from "babylonjs/Engines/Extensions/engine.computeShader";
import { WebGPUCacheSampler } from "babylonjs/Engines/WebGPU/webgpuCacheSampler";
/** @internal */
export class WebGPUComputeContext implements IComputeContext {
    private static _Counter;
    readonly uniqueId: number;
    private _device;
    private _cacheSampler;
    private _bindGroups;
    private _bindGroupEntries;
    getBindGroups(bindings: ComputeBindingList, computePipeline: GPUComputePipeline, bindingsMapping?: ComputeBindingMapping): GPUBindGroup[];
    constructor(device: GPUDevice, cacheSampler: WebGPUCacheSampler);
    clear(): void;
}

}
declare module "babylonjs/Engines/WebGPU/webgpuComputePipelineContext" {
import { IComputePipelineContext } from "babylonjs/Compute/IComputePipelineContext";
import { Nullable } from "babylonjs/types";
import { WebGPUEngine } from "babylonjs/Engines/webgpuEngine";
/** @internal */
export class WebGPUComputePipelineContext implements IComputePipelineContext {
    engine: WebGPUEngine;
    sources: {
        compute: string;
        rawCompute: string;
    };
    stage: Nullable<GPUProgrammableStage>;
    computePipeline: GPUComputePipeline;
    get isAsync(): boolean;
    get isReady(): boolean;
    /** @internal */
    _name: string;
    constructor(engine: WebGPUEngine);
    _getComputeShaderCode(): string | null;
    dispose(): void;
}

}
declare module "babylonjs/Engines/WebGPU/webgpuConstants" {
/** @internal */
export enum PredefinedColorSpace {
    SRGB = "srgb"
}
/** @internal */
export enum PowerPreference {
    LowPower = "low-power",
    HighPerformance = "high-performance"
}
/** @internal */
export enum FeatureName {
    DepthClipControl = "depth-clip-control",
    Depth24UnormStencil8 = "depth24unorm-stencil8",
    Depth32FloatStencil8 = "depth32float-stencil8",
    TextureCompressionBC = "texture-compression-bc",
    TextureCompressionETC2 = "texture-compression-etc2",
    TextureCompressionASTC = "texture-compression-astc",
    TimestampQuery = "timestamp-query",
    IndirectFirstInstance = "indirect-first-instance",
    ShaderF16 = "shader-f16",
    BGRA8UnormStorage = "bgra8unorm-storage"
}
/** @internal */
export enum BufferUsage {
    MapRead = 1,
    MapWrite = 2,
    CopySrc = 4,
    CopyDst = 8,
    Index = 16,
    Vertex = 32,
    Uniform = 64,
    Storage = 128,
    Indirect = 256,
    QueryResolve = 512
}
/** @internal */
export enum MapMode {
    Read = 1,
    Write = 2
}
/** @internal */
export enum TextureDimension {
    E1d = "1d",
    E2d = "2d",
    E3d = "3d"
}
/** @internal */
export enum TextureUsage {
    CopySrc = 1,
    CopyDst = 2,
    TextureBinding = 4,
    StorageBinding = 8,
    RenderAttachment = 16
}
/** @internal */
export enum TextureViewDimension {
    E1d = "1d",
    E2d = "2d",
    E2dArray = "2d-array",
    Cube = "cube",
    CubeArray = "cube-array",
    E3d = "3d"
}
/** @internal */
export enum TextureAspect {
    All = "all",
    StencilOnly = "stencil-only",
    DepthOnly = "depth-only"
}
/**
 * Comments taken from https://github.com/gfx-rs/wgpu/blob/master/wgpu-types/src/lib.rs
 * @internal
 */
export enum TextureFormat {
    R8Unorm = "r8unorm",
    R8Snorm = "r8snorm",
    R8Uint = "r8uint",
    R8Sint = "r8sint",
    R16Uint = "r16uint",
    R16Sint = "r16sint",
    R16Float = "r16float",
    RG8Unorm = "rg8unorm",
    RG8Snorm = "rg8snorm",
    RG8Uint = "rg8uint",
    RG8Sint = "rg8sint",
    R32Uint = "r32uint",
    R32Sint = "r32sint",
    R32Float = "r32float",
    RG16Uint = "rg16uint",
    RG16Sint = "rg16sint",
    RG16Float = "rg16float",
    RGBA8Unorm = "rgba8unorm",
    RGBA8UnormSRGB = "rgba8unorm-srgb",
    RGBA8Snorm = "rgba8snorm",
    RGBA8Uint = "rgba8uint",
    RGBA8Sint = "rgba8sint",
    BGRA8Unorm = "bgra8unorm",
    BGRA8UnormSRGB = "bgra8unorm-srgb",
    RGB9E5UFloat = "rgb9e5ufloat",
    RGB10A2Unorm = "rgb10a2unorm",
    RG11B10UFloat = "rg11b10ufloat",
    RG32Uint = "rg32uint",
    RG32Sint = "rg32sint",
    RG32Float = "rg32float",
    RGBA16Uint = "rgba16uint",
    RGBA16Sint = "rgba16sint",
    RGBA16Float = "rgba16float",
    RGBA32Uint = "rgba32uint",
    RGBA32Sint = "rgba32sint",
    RGBA32Float = "rgba32float",
    Stencil8 = "stencil8",
    Depth16Unorm = "depth16unorm",
    Depth24Plus = "depth24plus",
    Depth24PlusStencil8 = "depth24plus-stencil8",
    Depth32Float = "depth32float",
    BC1RGBAUnorm = "bc1-rgba-unorm",
    BC1RGBAUnormSRGB = "bc1-rgba-unorm-srgb",
    BC2RGBAUnorm = "bc2-rgba-unorm",
    BC2RGBAUnormSRGB = "bc2-rgba-unorm-srgb",
    BC3RGBAUnorm = "bc3-rgba-unorm",
    BC3RGBAUnormSRGB = "bc3-rgba-unorm-srgb",
    BC4RUnorm = "bc4-r-unorm",
    BC4RSnorm = "bc4-r-snorm",
    BC5RGUnorm = "bc5-rg-unorm",
    BC5RGSnorm = "bc5-rg-snorm",
    BC6HRGBUFloat = "bc6h-rgb-ufloat",
    BC6HRGBFloat = "bc6h-rgb-float",
    BC7RGBAUnorm = "bc7-rgba-unorm",
    BC7RGBAUnormSRGB = "bc7-rgba-unorm-srgb",
    ETC2RGB8Unorm = "etc2-rgb8unorm",
    ETC2RGB8UnormSRGB = "etc2-rgb8unorm-srgb",
    ETC2RGB8A1Unorm = "etc2-rgb8a1unorm",
    ETC2RGB8A1UnormSRGB = "etc2-rgb8a1unorm-srgb",
    ETC2RGBA8Unorm = "etc2-rgba8unorm",
    ETC2RGBA8UnormSRGB = "etc2-rgba8unorm-srgb",
    EACR11Unorm = "eac-r11unorm",
    EACR11Snorm = "eac-r11snorm",
    EACRG11Unorm = "eac-rg11unorm",
    EACRG11Snorm = "eac-rg11snorm",
    ASTC4x4Unorm = "astc-4x4-unorm",
    ASTC4x4UnormSRGB = "astc-4x4-unorm-srgb",
    ASTC5x4Unorm = "astc-5x4-unorm",
    ASTC5x4UnormSRGB = "astc-5x4-unorm-srgb",
    ASTC5x5Unorm = "astc-5x5-unorm",
    ASTC5x5UnormSRGB = "astc-5x5-unorm-srgb",
    ASTC6x5Unorm = "astc-6x5-unorm",
    ASTC6x5UnormSRGB = "astc-6x5-unorm-srgb",
    ASTC6x6Unorm = "astc-6x6-unorm",
    ASTC6x6UnormSRGB = "astc-6x6-unorm-srgb",
    ASTC8x5Unorm = "astc-8x5-unorm",
    ASTC8x5UnormSRGB = "astc-8x5-unorm-srgb",
    ASTC8x6Unorm = "astc-8x6-unorm",
    ASTC8x6UnormSRGB = "astc-8x6-unorm-srgb",
    ASTC8x8Unorm = "astc-8x8-unorm",
    ASTC8x8UnormSRGB = "astc-8x8-unorm-srgb",
    ASTC10x5Unorm = "astc-10x5-unorm",
    ASTC10x5UnormSRGB = "astc-10x5-unorm-srgb",
    ASTC10x6Unorm = "astc-10x6-unorm",
    ASTC10x6UnormSRGB = "astc-10x6-unorm-srgb",
    ASTC10x8Unorm = "astc-10x8-unorm",
    ASTC10x8UnormSRGB = "astc-10x8-unorm-srgb",
    ASTC10x10Unorm = "astc-10x10-unorm",
    ASTC10x10UnormSRGB = "astc-10x10-unorm-srgb",
    ASTC12x10Unorm = "astc-12x10-unorm",
    ASTC12x10UnormSRGB = "astc-12x10-unorm-srgb",
    ASTC12x12Unorm = "astc-12x12-unorm",
    ASTC12x12UnormSRGB = "astc-12x12-unorm-srgb",
    Depth24UnormStencil8 = "depth24unorm-stencil8",
    Depth32FloatStencil8 = "depth32float-stencil8"
}
/** @internal */
export enum AddressMode {
    ClampToEdge = "clamp-to-edge",
    Repeat = "repeat",
    MirrorRepeat = "mirror-repeat"
}
/** @internal */
export enum FilterMode {
    Nearest = "nearest",
    Linear = "linear"
}
/** @internal */
export enum CompareFunction {
    Never = "never",
    Less = "less",
    Equal = "equal",
    LessEqual = "less-equal",
    Greater = "greater",
    NotEqual = "not-equal",
    GreaterEqual = "greater-equal",
    Always = "always"
}
/** @internal */
export enum ShaderStage {
    Vertex = 1,
    Fragment = 2,
    Compute = 4
}
/** @internal */
export enum BufferBindingType {
    Uniform = "uniform",
    Storage = "storage",
    ReadOnlyStorage = "read-only-storage"
}
/** @internal */
export enum SamplerBindingType {
    Filtering = "filtering",
    NonFiltering = "non-filtering",
    Comparison = "comparison"
}
/** @internal */
export enum TextureSampleType {
    Float = "float",
    UnfilterableFloat = "unfilterable-float",
    Depth = "depth",
    Sint = "sint",
    Uint = "uint"
}
/** @internal */
export enum StorageTextureAccess {
    WriteOnly = "write-only"
}
/** @internal */
export enum CompilationMessageType {
    Error = "error",
    Warning = "warning",
    Info = "info"
}
/** @internal */
export enum AutoLayoutMode {
    Auto = "auto"
}
/** @internal */
export enum PrimitiveTopology {
    PointList = "point-list",
    LineList = "line-list",
    LineStrip = "line-strip",
    TriangleList = "triangle-list",
    TriangleStrip = "triangle-strip"
}
/** @internal */
export enum FrontFace {
    CCW = "ccw",
    CW = "cw"
}
/** @internal */
export enum CullMode {
    None = "none",
    Front = "front",
    Back = "back"
}
/** @internal */
export enum ColorWriteFlags {
    Red = 1,
    Green = 2,
    Blue = 4,
    Alpha = 8,
    All = 15
}
/** @internal */
export enum BlendFactor {
    Zero = "zero",
    One = "one",
    Src = "src",
    OneMinusSrc = "one-minus-src",
    SrcAlpha = "src-alpha",
    OneMinusSrcAlpha = "one-minus-src-alpha",
    Dst = "dst",
    OneMinusDst = "one-minus-dst",
    DstAlpha = "dst-alpha",
    OneMinusDstAlpha = "one-minus-dst-alpha",
    SrcAlphaSaturated = "src-alpha-saturated",
    Constant = "constant",
    OneMinusConstant = "one-minus-constant"
}
/** @internal */
export enum BlendOperation {
    Add = "add",
    Subtract = "subtract",
    ReverseSubtract = "reverse-subtract",
    Min = "min",
    Max = "max"
}
/** @internal */
export enum StencilOperation {
    Keep = "keep",
    Zero = "zero",
    Replace = "replace",
    Invert = "invert",
    IncrementClamp = "increment-clamp",
    DecrementClamp = "decrement-clamp",
    IncrementWrap = "increment-wrap",
    DecrementWrap = "decrement-wrap"
}
/** @internal */
export enum IndexFormat {
    Uint16 = "uint16",
    Uint32 = "uint32"
}
/** @internal */
export enum VertexFormat {
    Uint8x2 = "uint8x2",
    Uint8x4 = "uint8x4",
    Sint8x2 = "sint8x2",
    Sint8x4 = "sint8x4",
    Unorm8x2 = "unorm8x2",
    Unorm8x4 = "unorm8x4",
    Snorm8x2 = "snorm8x2",
    Snorm8x4 = "snorm8x4",
    Uint16x2 = "uint16x2",
    Uint16x4 = "uint16x4",
    Sint16x2 = "sint16x2",
    Sint16x4 = "sint16x4",
    Unorm16x2 = "unorm16x2",
    Unorm16x4 = "unorm16x4",
    Snorm16x2 = "snorm16x2",
    Snorm16x4 = "snorm16x4",
    Float16x2 = "float16x2",
    Float16x4 = "float16x4",
    Float32 = "float32",
    Float32x2 = "float32x2",
    Float32x3 = "float32x3",
    Float32x4 = "float32x4",
    Uint32 = "uint32",
    Uint32x2 = "uint32x2",
    Uint32x3 = "uint32x3",
    Uint32x4 = "uint32x4",
    Sint32 = "sint32",
    Sint32x2 = "sint32x2",
    Sint32x3 = "sint32x3",
    Sint32x4 = "sint32x4"
}
/** @internal */
export enum InputStepMode {
    Vertex = "vertex",
    Instance = "instance"
}
/** @internal */
export enum ComputePassTimestampLocation {
    Beginning = "beginning",
    End = "end"
}
/** @internal */
export enum RenderPassTimestampLocation {
    Beginning = "beginning",
    End = "end"
}
/** @internal */
export enum LoadOp {
    Load = "load",
    Clear = "clear"
}
/** @internal */
export enum StoreOp {
    Store = "store",
    Discard = "discard"
}
/** @internal */
export enum QueryType {
    Occlusion = "occlusion",
    Timestamp = "timestamp"
}
/** @internal */
export enum CanvasAlphaMode {
    Opaque = "opaque",
    Premultiplied = "premultiplied"
}
/** @internal */
export enum DeviceLostReason {
    Destroyed = "destroyed"
}
/** @internal */
export enum ErrorFilter {
    OutOfMemory = "out-of-memory",
    Validation = "validation"
}

}
declare module "babylonjs/Engines/WebGPU/webgpuDepthCullingState" {
import { Nullable } from "babylonjs/types";
import { WebGPUCacheRenderPipeline } from "babylonjs/Engines/WebGPU/webgpuCacheRenderPipeline";
import { DepthCullingState } from "babylonjs/States/depthCullingState";
/**
 * @internal
 **/
export class WebGPUDepthCullingState extends DepthCullingState {
    private _cache;
    /**
     * Initializes the state.
     * @param cache
     */
    constructor(cache: WebGPUCacheRenderPipeline);
    get zOffset(): number;
    set zOffset(value: number);
    get zOffsetUnits(): number;
    set zOffsetUnits(value: number);
    get cullFace(): Nullable<number>;
    set cullFace(value: Nullable<number>);
    get cull(): Nullable<boolean>;
    set cull(value: Nullable<boolean>);
    get depthFunc(): Nullable<number>;
    set depthFunc(value: Nullable<number>);
    get depthMask(): boolean;
    set depthMask(value: boolean);
    get depthTest(): boolean;
    set depthTest(value: boolean);
    get frontFace(): Nullable<number>;
    set frontFace(value: Nullable<number>);
    reset(): void;
    apply(): void;
}

}
declare module "babylonjs/Engines/WebGPU/webgpuDrawContext" {
import { WebGPUDataBuffer } from "babylonjs/Meshes/WebGPU/webgpuDataBuffer";
import { Nullable } from "babylonjs/types";
import { IDrawContext } from "babylonjs/Engines/IDrawContext";
import { WebGPUBufferManager } from "babylonjs/Engines/WebGPU/webgpuBufferManager";
/** @internal */
export class WebGPUDrawContext implements IDrawContext {
    private static _Counter;
    fastBundle?: GPURenderBundle;
    bindGroups?: GPUBindGroup[];
    uniqueId: number;
    buffers: {
        [name: string]: Nullable<WebGPUDataBuffer>;
    };
    indirectDrawBuffer?: GPUBuffer;
    private _materialContextUpdateId;
    private _bufferManager;
    private _useInstancing;
    private _indirectDrawData?;
    private _currentInstanceCount;
    private _isDirty;
    isDirty(materialContextUpdateId: number): boolean;
    resetIsDirty(materialContextUpdateId: number): void;
    get useInstancing(): boolean;
    set useInstancing(use: boolean);
    constructor(bufferManager: WebGPUBufferManager);
    reset(): void;
    setBuffer(name: string, buffer: Nullable<WebGPUDataBuffer>): void;
    setIndirectData(indexOrVertexCount: number, instanceCount: number, firstIndexOrVertex: number): void;
    dispose(): void;
}

}
declare module "babylonjs/Engines/WebGPU/webgpuExternalTexture" {
import { ExternalTexture } from "babylonjs/Materials/Textures/externalTexture";
/**
 * Nothing specific to WebGPU in this class, but the spec is not final yet so let's remove it later on
 * if it is not needed
 * @internal
 **/
export class WebGPUExternalTexture extends ExternalTexture {
    constructor(video: HTMLVideoElement);
}

}
declare module "babylonjs/Engines/WebGPU/webgpuHardwareTexture" {
import { HardwareTextureWrapper } from "babylonjs/Materials/Textures/hardwareTextureWrapper";
import { Nullable } from "babylonjs/types";
import { WebGPUBundleList } from "babylonjs/Engines/WebGPU/webgpuBundleList";
/** @internal */
export class WebGPUHardwareTexture implements HardwareTextureWrapper {
    /**
     * List of bundles collected in the snapshot rendering mode when the texture is a render target texture
     * The index in this array is the current layer we are rendering into
     * @internal
     */
    _bundleLists: WebGPUBundleList[];
    /**
     * Current layer we are rendering into when in snapshot rendering mode (if the texture is a render target texture)
     * @internal
     */
    _currentLayer: number;
    /**
     * Cache of RenderPassDescriptor and BindGroup used when generating mipmaps (see WebGPUTextureHelper.generateMipmaps)
     * @internal
     */
    _mipmapGenRenderPassDescr: GPURenderPassDescriptor[][];
    /** @internal */
    _mipmapGenBindGroup: GPUBindGroup[][];
    /**
     * Cache for the invertYPreMultiplyAlpha function (see WebGPUTextureHelper)
     * @internal
     */
    _copyInvertYTempTexture?: GPUTexture;
    /** @internal */
    _copyInvertYRenderPassDescr: GPURenderPassDescriptor;
    /** @internal */
    _copyInvertYBindGroup: GPUBindGroup;
    /** @internal */
    _copyInvertYBindGroupWithOfst: GPUBindGroup;
    private _webgpuTexture;
    private _webgpuMSAATexture;
    get underlyingResource(): Nullable<GPUTexture>;
    get msaaTexture(): Nullable<GPUTexture>;
    set msaaTexture(texture: Nullable<GPUTexture>);
    view: Nullable<GPUTextureView>;
    viewForWriting: Nullable<GPUTextureView>;
    format: GPUTextureFormat;
    textureUsages: number;
    textureAdditionalUsages: number;
    constructor(existingTexture?: Nullable<GPUTexture>);
    set(hardwareTexture: GPUTexture): void;
    setUsage(textureSource: number, generateMipMaps: boolean, isCube: boolean, width: number, height: number): void;
    createView(descriptor?: GPUTextureViewDescriptor, createViewForWriting?: boolean): void;
    reset(): void;
    release(): void;
}
export {};

}
declare module "babylonjs/Engines/WebGPU/webgpuMaterialContext" {
import { ExternalTexture } from "babylonjs/Materials/Textures/externalTexture";
import { InternalTexture } from "babylonjs/Materials/Textures/internalTexture";
import { TextureSampler } from "babylonjs/Materials/Textures/textureSampler";
import { Nullable } from "babylonjs/types";
import { IMaterialContext } from "babylonjs/Engines/IMaterialContext";
/** @internal */
interface IWebGPUMaterialContextSamplerCache {
    sampler: Nullable<TextureSampler>;
    hashCode: number;
}
/** @internal */
interface IWebGPUMaterialContextTextureCache {
    texture: Nullable<InternalTexture | ExternalTexture>;
    isFloatTexture: boolean;
    isExternalTexture: boolean;
}
/** @internal */
export class WebGPUMaterialContext implements IMaterialContext {
    private static _Counter;
    uniqueId: number;
    updateId: number;
    isDirty: boolean;
    samplers: {
        [name: string]: Nullable<IWebGPUMaterialContextSamplerCache>;
    };
    textures: {
        [name: string]: Nullable<IWebGPUMaterialContextTextureCache>;
    };
    get forceBindGroupCreation(): boolean;
    get hasFloatTextures(): boolean;
    protected _numFloatTextures: number;
    protected _numExternalTextures: number;
    constructor();
    reset(): void;
    setSampler(name: string, sampler: Nullable<TextureSampler>): void;
    setTexture(name: string, texture: Nullable<InternalTexture | ExternalTexture>): void;
}
export {};

}
declare module "babylonjs/Engines/WebGPU/webgpuOcclusionQuery" {
import { WebGPUEngine } from "babylonjs/Engines/webgpuEngine";
import { WebGPUBufferManager } from "babylonjs/Engines/WebGPU/webgpuBufferManager";
/** @internal */
export class WebGPUOcclusionQuery {
    private _engine;
    private _device;
    private _bufferManager;
    private _currentTotalIndices;
    private _countIncrement;
    private _querySet;
    private _availableIndices;
    private _lastBuffer;
    private _frameLastBuffer;
    get querySet(): GPUQuerySet;
    get hasQueries(): boolean;
    get canBeginQuery(): boolean;
    constructor(engine: WebGPUEngine, device: GPUDevice, bufferManager: WebGPUBufferManager, startCount?: number, incrementCount?: number);
    createQuery(): number;
    deleteQuery(index: number): void;
    isQueryResultAvailable(index: number): boolean;
    getQueryResult(index: number): number;
    private _retrieveQueryBuffer;
    private _allocateNewIndices;
    private _delayQuerySetDispose;
    dispose(): void;
}

}
declare module "babylonjs/Engines/WebGPU/webgpuPipelineContext" {
import { IPipelineContext } from "babylonjs/Engines/IPipelineContext";
import { Nullable } from "babylonjs/types";
import { WebGPUEngine } from "babylonjs/Engines/webgpuEngine";
import { Effect } from "babylonjs/Materials/effect";
import { WebGPUShaderProcessingContext } from "babylonjs/Engines/WebGPU/webgpuShaderProcessingContext";
import { UniformBuffer } from "babylonjs/Materials/uniformBuffer";
import { IMatrixLike, IVector2Like, IVector3Like, IVector4Like, IColor3Like, IColor4Like, IQuaternionLike } from "babylonjs/Maths/math.like";
/** @internal */
export interface IWebGPURenderPipelineStageDescriptor {
    vertexStage: GPUProgrammableStage;
    fragmentStage?: GPUProgrammableStage;
}
/** @internal */
export class WebGPUPipelineContext implements IPipelineContext {
    engine: WebGPUEngine;
    shaderProcessingContext: WebGPUShaderProcessingContext;
    protected _leftOverUniformsByName: {
        [name: string]: string;
    };
    sources: {
        vertex: string;
        fragment: string;
        rawVertex: string;
        rawFragment: string;
    };
    stages: Nullable<IWebGPURenderPipelineStageDescriptor>;
    bindGroupLayouts: GPUBindGroupLayout[];
    /**
     * Stores the left-over uniform buffer
     */
    uniformBuffer: Nullable<UniformBuffer>;
    onCompiled?: () => void;
    get isAsync(): boolean;
    get isReady(): boolean;
    /** @internal */
    _name: string;
    constructor(shaderProcessingContext: WebGPUShaderProcessingContext, engine: WebGPUEngine);
    _handlesSpectorRebuildCallback(): void;
    _fillEffectInformation(effect: Effect, uniformBuffersNames: {
        [key: string]: number;
    }, uniformsNames: string[], uniforms: {
        [key: string]: Nullable<WebGLUniformLocation>;
    }, samplerList: string[], samplers: {
        [key: string]: number;
    }, attributesNames: string[], attributes: number[]): void;
    /** @internal */
    /**
     * Build the uniform buffer used in the material.
     */
    buildUniformLayout(): void;
    /**
     * Release all associated resources.
     **/
    dispose(): void;
    /**
     * Sets an integer value on a uniform variable.
     * @param uniformName Name of the variable.
     * @param value Value to be set.
     */
    setInt(uniformName: string, value: number): void;
    /**
     * Sets an int2 value on a uniform variable.
     * @param uniformName Name of the variable.
     * @param x First int in int2.
     * @param y Second int in int2.
     */
    setInt2(uniformName: string, x: number, y: number): void;
    /**
     * Sets an int3 value on a uniform variable.
     * @param uniformName Name of the variable.
     * @param x First int in int3.
     * @param y Second int in int3.
     * @param z Third int in int3.
     */
    setInt3(uniformName: string, x: number, y: number, z: number): void;
    /**
     * Sets an int4 value on a uniform variable.
     * @param uniformName Name of the variable.
     * @param x First int in int4.
     * @param y Second int in int4.
     * @param z Third int in int4.
     * @param w Fourth int in int4.
     */
    setInt4(uniformName: string, x: number, y: number, z: number, w: number): void;
    /**
     * Sets an int array on a uniform variable.
     * @param uniformName Name of the variable.
     * @param array array to be set.
     */
    setIntArray(uniformName: string, array: Int32Array): void;
    /**
     * Sets an int array 2 on a uniform variable. (Array is specified as single array eg. [1,2,3,4] will result in [[1,2],[3,4]] in the shader)
     * @param uniformName Name of the variable.
     * @param array array to be set.
     */
    setIntArray2(uniformName: string, array: Int32Array): void;
    /**
     * Sets an int array 3 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6] will result in [[1,2,3],[4,5,6]] in the shader)
     * @param uniformName Name of the variable.
     * @param array array to be set.
     */
    setIntArray3(uniformName: string, array: Int32Array): void;
    /**
     * Sets an int array 4 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6,7,8] will result in [[1,2,3,4],[5,6,7,8]] in the shader)
     * @param uniformName Name of the variable.
     * @param array array to be set.
     */
    setIntArray4(uniformName: string, array: Int32Array): void;
    /**
     * Sets an unsigned integer value on a uniform variable.
     * @param uniformName Name of the variable.
     * @param value Value to be set.
     */
    setUInt(uniformName: string, value: number): void;
    /**
     * Sets an unsigned int2 value on a uniform variable.
     * @param uniformName Name of the variable.
     * @param x First unsigned int in uint2.
     * @param y Second unsigned int in uint2.
     */
    setUInt2(uniformName: string, x: number, y: number): void;
    /**
     * Sets an unsigned int3 value on a uniform variable.
     * @param uniformName Name of the variable.
     * @param x First unsigned int in uint3.
     * @param y Second unsigned int in uint3.
     * @param z Third unsigned int in uint3.
     */
    setUInt3(uniformName: string, x: number, y: number, z: number): void;
    /**
     * Sets an unsigned int4 value on a uniform variable.
     * @param uniformName Name of the variable.
     * @param x First unsigned int in uint4.
     * @param y Second unsigned int in uint4.
     * @param z Third unsigned int in uint4.
     * @param w Fourth unsigned int in uint4.
     */
    setUInt4(uniformName: string, x: number, y: number, z: number, w: number): void;
    /**
     * Sets an unsigned int array on a uniform variable.
     * @param uniformName Name of the variable.
     * @param array array to be set.
     */
    setUIntArray(uniformName: string, array: Uint32Array): void;
    /**
     * Sets an unsigned int array 2 on a uniform variable. (Array is specified as single array eg. [1,2,3,4] will result in [[1,2],[3,4]] in the shader)
     * @param uniformName Name of the variable.
     * @param array array to be set.
     */
    setUIntArray2(uniformName: string, array: Uint32Array): void;
    /**
     * Sets an unsigned int array 3 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6] will result in [[1,2,3],[4,5,6]] in the shader)
     * @param uniformName Name of the variable.
     * @param array array to be set.
     */
    setUIntArray3(uniformName: string, array: Uint32Array): void;
    /**
     * Sets an unsigned int array 4 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6,7,8] will result in [[1,2,3,4],[5,6,7,8]] in the shader)
     * @param uniformName Name of the variable.
     * @param array array to be set.
     */
    setUIntArray4(uniformName: string, array: Uint32Array): void;
    /**
     * Sets an array on a uniform variable.
     * @param uniformName Name of the variable.
     * @param array array to be set.
     */
    setArray(uniformName: string, array: number[]): void;
    /**
     * Sets an array 2 on a uniform variable. (Array is specified as single array eg. [1,2,3,4] will result in [[1,2],[3,4]] in the shader)
     * @param uniformName Name of the variable.
     * @param array array to be set.
     */
    setArray2(uniformName: string, array: number[]): void;
    /**
     * Sets an array 3 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6] will result in [[1,2,3],[4,5,6]] in the shader)
     * @param uniformName Name of the variable.
     * @param array array to be set.
     * @returns this effect.
     */
    setArray3(uniformName: string, array: number[]): void;
    /**
     * Sets an array 4 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6,7,8] will result in [[1,2,3,4],[5,6,7,8]] in the shader)
     * @param uniformName Name of the variable.
     * @param array array to be set.
     */
    setArray4(uniformName: string, array: number[]): void;
    /**
     * Sets matrices on a uniform variable.
     * @param uniformName Name of the variable.
     * @param matrices matrices to be set.
     */
    setMatrices(uniformName: string, matrices: Float32Array): void;
    /**
     * Sets matrix on a uniform variable.
     * @param uniformName Name of the variable.
     * @param matrix matrix to be set.
     */
    setMatrix(uniformName: string, matrix: IMatrixLike): void;
    /**
     * Sets a 3x3 matrix on a uniform variable. (Specified as [1,2,3,4,5,6,7,8,9] will result in [1,2,3][4,5,6][7,8,9] matrix)
     * @param uniformName Name of the variable.
     * @param matrix matrix to be set.
     */
    setMatrix3x3(uniformName: string, matrix: Float32Array): void;
    /**
     * Sets a 2x2 matrix on a uniform variable. (Specified as [1,2,3,4] will result in [1,2][3,4] matrix)
     * @param uniformName Name of the variable.
     * @param matrix matrix to be set.
     */
    setMatrix2x2(uniformName: string, matrix: Float32Array): void;
    /**
     * Sets a float on a uniform variable.
     * @param uniformName Name of the variable.
     * @param value value to be set.
     * @returns this effect.
     */
    setFloat(uniformName: string, value: number): void;
    /**
     * Sets a Vector2 on a uniform variable.
     * @param uniformName Name of the variable.
     * @param vector2 vector2 to be set.
     */
    setVector2(uniformName: string, vector2: IVector2Like): void;
    /**
     * Sets a float2 on a uniform variable.
     * @param uniformName Name of the variable.
     * @param x First float in float2.
     * @param y Second float in float2.
     */
    setFloat2(uniformName: string, x: number, y: number): void;
    /**
     * Sets a Vector3 on a uniform variable.
     * @param uniformName Name of the variable.
     * @param vector3 Value to be set.
     */
    setVector3(uniformName: string, vector3: IVector3Like): void;
    /**
     * Sets a float3 on a uniform variable.
     * @param uniformName Name of the variable.
     * @param x First float in float3.
     * @param y Second float in float3.
     * @param z Third float in float3.
     */
    setFloat3(uniformName: string, x: number, y: number, z: number): void;
    /**
     * Sets a Vector4 on a uniform variable.
     * @param uniformName Name of the variable.
     * @param vector4 Value to be set.
     */
    setVector4(uniformName: string, vector4: IVector4Like): void;
    /**
     * Sets a Quaternion on a uniform variable.
     * @param uniformName Name of the variable.
     * @param quaternion Value to be set.
     */
    setQuaternion(uniformName: string, quaternion: IQuaternionLike): void;
    /**
     * Sets a float4 on a uniform variable.
     * @param uniformName Name of the variable.
     * @param x First float in float4.
     * @param y Second float in float4.
     * @param z Third float in float4.
     * @param w Fourth float in float4.
     * @returns this effect.
     */
    setFloat4(uniformName: string, x: number, y: number, z: number, w: number): void;
    /**
     * Sets a Color3 on a uniform variable.
     * @param uniformName Name of the variable.
     * @param color3 Value to be set.
     */
    setColor3(uniformName: string, color3: IColor3Like): void;
    /**
     * Sets a Color4 on a uniform variable.
     * @param uniformName Name of the variable.
     * @param color3 Value to be set.
     * @param alpha Alpha value to be set.
     */
    setColor4(uniformName: string, color3: IColor3Like, alpha: number): void;
    /**
     * Sets a Color4 on a uniform variable
     * @param uniformName defines the name of the variable
     * @param color4 defines the value to be set
     */
    setDirectColor4(uniformName: string, color4: IColor4Like): void;
    _getVertexShaderCode(): string | null;
    _getFragmentShaderCode(): string | null;
}

}
declare module "babylonjs/Engines/WebGPU/webgpuQuerySet" {
import { WebGPUBufferManager } from "babylonjs/Engines/WebGPU/webgpuBufferManager";
import { QueryType } from "babylonjs/Engines/WebGPU/webgpuConstants";
/** @internal */
export class WebGPUQuerySet {
    private _device;
    private _bufferManager;
    private _count;
    private _canUseMultipleBuffers;
    private _querySet;
    private _queryBuffer;
    private _dstBuffers;
    get querySet(): GPUQuerySet;
    constructor(count: number, type: QueryType, device: GPUDevice, bufferManager: WebGPUBufferManager, canUseMultipleBuffers?: boolean);
    private _getBuffer;
    readValues(firstQuery?: number, queryCount?: number): Promise<BigUint64Array | null>;
    readValue(firstQuery?: number): Promise<number | null>;
    readTwoValuesAndSubtract(firstQuery?: number): Promise<number | null>;
    dispose(): void;
}

}
declare module "babylonjs/Engines/WebGPU/webgpuRenderPassWrapper" {
import { Nullable } from "babylonjs/types";
import { WebGPUHardwareTexture } from "babylonjs/Engines/WebGPU/webgpuHardwareTexture";
/** @internal */
export class WebGPURenderPassWrapper {
    renderPassDescriptor: Nullable<GPURenderPassDescriptor>;
    renderPass: Nullable<GPURenderPassEncoder>;
    colorAttachmentViewDescriptor: Nullable<GPUTextureViewDescriptor>;
    depthAttachmentViewDescriptor: Nullable<GPUTextureViewDescriptor>;
    colorAttachmentGPUTextures: (WebGPUHardwareTexture | null)[];
    depthTextureFormat: GPUTextureFormat | undefined;
    constructor();
    reset(fullReset?: boolean): void;
}

}
declare module "babylonjs/Engines/WebGPU/webgpuRenderTargetWrapper" {
import { RenderTargetWrapper } from "babylonjs/Engines/renderTargetWrapper";
/** @internal */
export class WebGPURenderTargetWrapper extends RenderTargetWrapper {
    /** @internal */
    _defaultAttachments: number[];
}

}
declare module "babylonjs/Engines/WebGPU/webgpuShaderProcessingContext" {
import { ShaderLanguage } from "babylonjs/Materials/shaderLanguage";
import { ShaderProcessingContext } from "babylonjs/Engines/Processors/shaderProcessingOptions";
/** @internal */
export interface WebGPUBindingInfo {
    groupIndex: number;
    bindingIndex: number;
}
/** @internal */
export interface WebGPUTextureDescription {
    autoBindSampler?: boolean;
    isTextureArray: boolean;
    isStorageTexture: boolean;
    textures: Array<WebGPUBindingInfo>;
    sampleType?: GPUTextureSampleType;
}
/** @internal */
export interface WebGPUSamplerDescription {
    binding: WebGPUBindingInfo;
    type: GPUSamplerBindingType;
}
/** @internal */
export interface WebGPUBufferDescription {
    binding: WebGPUBindingInfo;
}
/** @internal */
export interface WebGPUBindGroupLayoutEntryInfo {
    name: string;
    index: number;
    nameInArrayOfTexture?: string;
}
/**
 * @internal
 */
export class WebGPUShaderProcessingContext implements ShaderProcessingContext {
    /** @internal */
    static _SimplifiedKnownBindings: boolean;
    protected static _SimplifiedKnownUBOs: {
        [key: string]: WebGPUBufferDescription;
    };
    protected static _KnownUBOs: {
        [key: string]: WebGPUBufferDescription;
    };
    static get KnownUBOs(): {
        [key: string]: WebGPUBufferDescription;
    };
    shaderLanguage: ShaderLanguage;
    uboNextBindingIndex: number;
    freeGroupIndex: number;
    freeBindingIndex: number;
    availableVaryings: {
        [key: string]: number;
    };
    availableAttributes: {
        [key: string]: number;
    };
    availableBuffers: {
        [key: string]: WebGPUBufferDescription;
    };
    availableTextures: {
        [key: string]: WebGPUTextureDescription;
    };
    availableSamplers: {
        [key: string]: WebGPUSamplerDescription;
    };
    leftOverUniforms: {
        name: string;
        type: string;
        length: number;
    }[];
    orderedAttributes: string[];
    bindGroupLayoutEntries: GPUBindGroupLayoutEntry[][];
    bindGroupLayoutEntryInfo: WebGPUBindGroupLayoutEntryInfo[][];
    bindGroupEntries: GPUBindGroupEntry[][];
    bufferNames: string[];
    textureNames: string[];
    samplerNames: string[];
    attributeNamesFromEffect: string[];
    attributeLocationsFromEffect: number[];
    private _attributeNextLocation;
    private _varyingNextLocation;
    constructor(shaderLanguage: ShaderLanguage);
    private _findStartingGroupBinding;
    getAttributeNextLocation(dataType: string, arrayLength?: number): number;
    getVaryingNextLocation(dataType: string, arrayLength?: number): number;
    getNextFreeUBOBinding(): {
        groupIndex: number;
        bindingIndex: number;
    };
    private _getNextFreeBinding;
}

}
declare module "babylonjs/Engines/WebGPU/webgpuShaderProcessor" {
import { ShaderLanguage } from "babylonjs/Materials/shaderLanguage";
import { Nullable } from "babylonjs/types";
import { IShaderProcessor } from "babylonjs/Engines/Processors/iShaderProcessor";
import { WebGPUSamplerDescription, WebGPUShaderProcessingContext, WebGPUTextureDescription, WebGPUBufferDescription } from "babylonjs/Engines/WebGPU/webgpuShaderProcessingContext";
/** @internal */
export abstract class WebGPUShaderProcessor implements IShaderProcessor {
    static readonly AutoSamplerSuffix: string;
    static readonly LeftOvertUBOName: string;
    static readonly InternalsUBOName: string;
    static UniformSizes: {
        [type: string]: number;
    };
    protected static _SamplerFunctionByWebGLSamplerType: {
        [key: string]: string;
    };
    protected static _TextureTypeByWebGLSamplerType: {
        [key: string]: string;
    };
    protected static _GpuTextureViewDimensionByWebGPUTextureType: {
        [key: string]: GPUTextureViewDimension;
    };
    protected static _SamplerTypeByWebGLSamplerType: {
        [key: string]: string;
    };
    protected static _IsComparisonSamplerByWebGPUSamplerType: {
        [key: string]: boolean;
    };
    shaderLanguage: ShaderLanguage;
    protected _webgpuProcessingContext: WebGPUShaderProcessingContext;
    protected abstract _getArraySize(name: string, type: string, preProcessors: {
        [key: string]: string;
    }): [string, string, number];
    protected abstract _generateLeftOverUBOCode(name: string, uniformBufferDescription: WebGPUBufferDescription): string;
    protected _addUniformToLeftOverUBO(name: string, uniformType: string, preProcessors: {
        [key: string]: string;
    }): void;
    protected _buildLeftOverUBO(): string;
    protected _collectBindingNames(): void;
    protected _preCreateBindGroupEntries(): void;
    protected _addTextureBindingDescription(name: string, textureInfo: WebGPUTextureDescription, textureIndex: number, dimension: Nullable<GPUTextureViewDimension>, format: Nullable<GPUTextureFormat>, isVertex: boolean): void;
    protected _addSamplerBindingDescription(name: string, samplerInfo: WebGPUSamplerDescription, isVertex: boolean): void;
    protected _addBufferBindingDescription(name: string, uniformBufferInfo: WebGPUBufferDescription, bufferType: GPUBufferBindingType, isVertex: boolean): void;
    protected _injectStartingAndEndingCode(code: string, mainFuncDecl: string, startingCode?: string, endingCode?: string): string;
}

}
declare module "babylonjs/Engines/WebGPU/webgpuShaderProcessorsGLSL" {
import { Nullable } from "babylonjs/types";
import { ShaderProcessingContext } from "babylonjs/Engines/Processors/shaderProcessingOptions";
import { WebGPUBufferDescription } from "babylonjs/Engines/WebGPU/webgpuShaderProcessingContext";
import { ThinEngine } from "babylonjs/Engines/thinEngine";
import { WebGPUShaderProcessor } from "babylonjs/Engines/WebGPU/webgpuShaderProcessor";
import { ShaderLanguage } from "babylonjs/Materials/shaderLanguage";
/** @internal */
export class WebGPUShaderProcessorGLSL extends WebGPUShaderProcessor {
    protected _missingVaryings: Array<string>;
    protected _textureArrayProcessing: Array<string>;
    protected _preProcessors: {
        [key: string]: string;
    };
    protected _vertexIsGLES3: boolean;
    protected _fragmentIsGLES3: boolean;
    shaderLanguage: ShaderLanguage;
    parseGLES3: boolean;
    attributeKeywordName: string | undefined;
    varyingVertexKeywordName: string | undefined;
    varyingFragmentKeywordName: string | undefined;
    protected _getArraySize(name: string, type: string, preProcessors: {
        [key: string]: string;
    }): [string, string, number];
    initializeShaders(processingContext: Nullable<ShaderProcessingContext>): void;
    preProcessShaderCode(code: string, isFragment: boolean): string;
    varyingProcessor(varying: string, isFragment: boolean, preProcessors: {
        [key: string]: string;
    }): string;
    attributeProcessor(attribute: string, preProcessors: {
        [key: string]: string;
    }): string;
    uniformProcessor(uniform: string, isFragment: boolean, preProcessors: {
        [key: string]: string;
    }): string;
    uniformBufferProcessor(uniformBuffer: string, isFragment: boolean): string;
    postProcessor(code: string, defines: string[], isFragment: boolean, processingContext: Nullable<ShaderProcessingContext>, engine: ThinEngine): string;
    private _applyTextureArrayProcessing;
    protected _generateLeftOverUBOCode(name: string, uniformBufferDescription: WebGPUBufferDescription): string;
    finalizeShaders(vertexCode: string, fragmentCode: string): {
        vertexCode: string;
        fragmentCode: string;
    };
}

}
declare module "babylonjs/Engines/WebGPU/webgpuShaderProcessorsWGSL" {
import { Nullable } from "babylonjs/types";
import { ShaderProcessingContext } from "babylonjs/Engines/Processors/shaderProcessingOptions";
import { WebGPUBufferDescription } from "babylonjs/Engines/WebGPU/webgpuShaderProcessingContext";
import { WebGPUShaderProcessor } from "babylonjs/Engines/WebGPU/webgpuShaderProcessor";
import "babylonjs/ShadersWGSL/ShadersInclude/bonesDeclaration";
import "babylonjs/ShadersWGSL/ShadersInclude/bonesVertex";
import "babylonjs/ShadersWGSL/ShadersInclude/bakedVertexAnimationDeclaration";
import "babylonjs/ShadersWGSL/ShadersInclude/bakedVertexAnimation";
import "babylonjs/ShadersWGSL/ShadersInclude/clipPlaneFragment";
import "babylonjs/ShadersWGSL/ShadersInclude/clipPlaneFragmentDeclaration";
import "babylonjs/ShadersWGSL/ShadersInclude/clipPlaneVertex";
import "babylonjs/ShadersWGSL/ShadersInclude/clipPlaneVertexDeclaration";
import "babylonjs/ShadersWGSL/ShadersInclude/instancesDeclaration";
import "babylonjs/ShadersWGSL/ShadersInclude/instancesVertex";
import "babylonjs/ShadersWGSL/ShadersInclude/meshUboDeclaration";
import "babylonjs/ShadersWGSL/ShadersInclude/morphTargetsVertex";
import "babylonjs/ShadersWGSL/ShadersInclude/morphTargetsVertexDeclaration";
import "babylonjs/ShadersWGSL/ShadersInclude/morphTargetsVertexGlobal";
import "babylonjs/ShadersWGSL/ShadersInclude/morphTargetsVertexGlobalDeclaration";
import "babylonjs/ShadersWGSL/ShadersInclude/sceneUboDeclaration";
import { ShaderLanguage } from "babylonjs/Materials/shaderLanguage";
/** @internal */
export class WebGPUShaderProcessorWGSL extends WebGPUShaderProcessor {
    protected _attributesWGSL: string[];
    protected _attributesDeclWGSL: string[];
    protected _attributeNamesWGSL: string[];
    protected _varyingsWGSL: string[];
    protected _varyingsDeclWGSL: string[];
    protected _varyingNamesWGSL: string[];
    protected _stridedUniformArrays: string[];
    shaderLanguage: ShaderLanguage;
    uniformRegexp: RegExp;
    textureRegexp: RegExp;
    noPrecision: boolean;
    protected _getArraySize(name: string, uniformType: string, preProcessors: {
        [key: string]: string;
    }): [string, string, number];
    initializeShaders(processingContext: Nullable<ShaderProcessingContext>): void;
    preProcessShaderCode(code: string): string;
    varyingProcessor(varying: string, isFragment: boolean, preProcessors: {
        [key: string]: string;
    }): string;
    attributeProcessor(attribute: string, preProcessors: {
        [key: string]: string;
    }): string;
    uniformProcessor(uniform: string, isFragment: boolean, preProcessors: {
        [key: string]: string;
    }): string;
    textureProcessor(texture: string, isFragment: boolean, preProcessors: {
        [key: string]: string;
    }): string;
    postProcessor(code: string): string;
    finalizeShaders(vertexCode: string, fragmentCode: string): {
        vertexCode: string;
        fragmentCode: string;
    };
    protected _generateLeftOverUBOCode(name: string, uniformBufferDescription: WebGPUBufferDescription): string;
    private _processSamplers;
    private _processCustomBuffers;
    private _processStridedUniformArrays;
}

}
declare module "babylonjs/Engines/WebGPU/webgpuSnapshotRendering" {
import { Nullable } from "babylonjs/types";
import { WebGPUEngine } from "babylonjs/Engines/webgpuEngine";
import { WebGPUBundleList } from "babylonjs/Engines/WebGPU/webgpuBundleList";
import { WebGPUHardwareTexture } from "babylonjs/Engines/WebGPU/webgpuHardwareTexture";
/** @internal */
export class WebGPUSnapshotRendering {
    private _engine;
    private _record;
    private _play;
    private _mainPassBundleList;
    private _modeSaved;
    private _bundleList;
    private _bundleListRenderTarget;
    private _enabled;
    private _mode;
    constructor(engine: WebGPUEngine, renderingMode: number, bundleList: WebGPUBundleList, bundleListRenderTarget: WebGPUBundleList);
    get enabled(): boolean;
    get play(): boolean;
    get record(): boolean;
    set enabled(activate: boolean);
    get mode(): number;
    set mode(mode: number);
    endMainRenderPass(): void;
    endRenderTargetPass(currentRenderPass: GPURenderPassEncoder, gpuWrapper: WebGPUHardwareTexture): boolean;
    endFrame(mainRenderPass: Nullable<GPURenderPassEncoder>): void;
    reset(): void;
}

}
declare module "babylonjs/Engines/WebGPU/webgpuStencilStateComposer" {
import { WebGPUCacheRenderPipeline } from "babylonjs/Engines/WebGPU/webgpuCacheRenderPipeline";
import { StencilStateComposer } from "babylonjs/States/stencilStateComposer";
/**
 * @internal
 **/
export class WebGPUStencilStateComposer extends StencilStateComposer {
    private _cache;
    constructor(cache: WebGPUCacheRenderPipeline);
    get func(): number;
    set func(value: number);
    get funcMask(): number;
    set funcMask(value: number);
    get opStencilFail(): number;
    set opStencilFail(value: number);
    get opDepthFail(): number;
    set opDepthFail(value: number);
    get opStencilDepthPass(): number;
    set opStencilDepthPass(value: number);
    get mask(): number;
    set mask(value: number);
    get enabled(): boolean;
    set enabled(value: boolean);
    reset(): void;
    apply(): void;
}

}
declare module "babylonjs/Engines/WebGPU/webgpuTextureHelper" {
import { WebGPUBufferManager } from "babylonjs/Engines/WebGPU/webgpuBufferManager";
import { Nullable } from "babylonjs/types";
import { InternalTexture } from "babylonjs/Materials/Textures/internalTexture";
import { WebGPUHardwareTexture } from "babylonjs/Engines/WebGPU/webgpuHardwareTexture";
import { WebGPUTintWASM } from "babylonjs/Engines/WebGPU/webgpuTintWASM";
import { ExternalTexture } from "babylonjs/Materials/Textures/externalTexture";
/**
 * Map a (renderable) texture format (GPUTextureFormat) to an index for fast lookup (in caches for eg)
 */
export const renderableTextureFormatToIndex: {
    [name: string]: number;
};
/** @internal */
export class WebGPUTextureHelper {
    private _device;
    private _glslang;
    private _tintWASM;
    private _bufferManager;
    private _mipmapSampler;
    private _videoSampler;
    private _ubCopyWithOfst;
    private _pipelines;
    private _compiledShaders;
    private _videoPipelines;
    private _videoCompiledShaders;
    private _deferredReleaseTextures;
    private _commandEncoderForCreation;
    static ComputeNumMipmapLevels(width: number, height: number): number;
    constructor(device: GPUDevice, glslang: any, tintWASM: Nullable<WebGPUTintWASM>, bufferManager: WebGPUBufferManager);
    private _getPipeline;
    private _getVideoPipeline;
    private static _GetTextureTypeFromFormat;
    private static _GetBlockInformationFromFormat;
    private static _IsHardwareTexture;
    private static _IsInternalTexture;
    static IsImageBitmap(imageBitmap: ImageBitmap | {
        width: number;
        height: number;
    }): imageBitmap is ImageBitmap;
    static IsImageBitmapArray(imageBitmap: ImageBitmap[] | {
        width: number;
        height: number;
    }): imageBitmap is ImageBitmap[];
    setCommandEncoder(encoder: GPUCommandEncoder): void;
    static IsCompressedFormat(format: GPUTextureFormat): boolean;
    static GetWebGPUTextureFormat(type: number, format: number, useSRGBBuffer?: boolean): GPUTextureFormat;
    static GetNumChannelsFromWebGPUTextureFormat(format: GPUTextureFormat): number;
    static HasStencilAspect(format: GPUTextureFormat): boolean;
    static HasDepthAndStencilAspects(format: GPUTextureFormat): boolean;
    copyVideoToTexture(video: ExternalTexture, texture: InternalTexture, format: GPUTextureFormat, invertY?: boolean, commandEncoder?: GPUCommandEncoder): void;
    invertYPreMultiplyAlpha(gpuOrHdwTexture: GPUTexture | WebGPUHardwareTexture, width: number, height: number, format: GPUTextureFormat, invertY?: boolean, premultiplyAlpha?: boolean, faceIndex?: number, mipLevel?: number, layers?: number, ofstX?: number, ofstY?: number, rectWidth?: number, rectHeight?: number, commandEncoder?: GPUCommandEncoder, allowGPUOptimization?: boolean): void;
    copyWithInvertY(srcTextureView: GPUTextureView, format: GPUTextureFormat, renderPassDescriptor: GPURenderPassDescriptor, commandEncoder?: GPUCommandEncoder): void;
    createTexture(imageBitmap: ImageBitmap | {
        width: number;
        height: number;
        layers: number;
    }, hasMipmaps?: boolean, generateMipmaps?: boolean, invertY?: boolean, premultiplyAlpha?: boolean, is3D?: boolean, format?: GPUTextureFormat, sampleCount?: number, commandEncoder?: GPUCommandEncoder, usage?: number, additionalUsages?: number): GPUTexture;
    createCubeTexture(imageBitmaps: ImageBitmap[] | {
        width: number;
        height: number;
    }, hasMipmaps?: boolean, generateMipmaps?: boolean, invertY?: boolean, premultiplyAlpha?: boolean, format?: GPUTextureFormat, sampleCount?: number, commandEncoder?: GPUCommandEncoder, usage?: number, additionalUsages?: number): GPUTexture;
    generateCubeMipmaps(gpuTexture: GPUTexture | WebGPUHardwareTexture, format: GPUTextureFormat, mipLevelCount: number, commandEncoder?: GPUCommandEncoder): void;
    generateMipmaps(gpuOrHdwTexture: GPUTexture | WebGPUHardwareTexture, format: GPUTextureFormat, mipLevelCount: number, faceIndex?: number, commandEncoder?: GPUCommandEncoder): void;
    createGPUTextureForInternalTexture(texture: InternalTexture, width?: number, height?: number, depth?: number, creationFlags?: number): WebGPUHardwareTexture;
    createMSAATexture(texture: InternalTexture, samples: number): void;
    updateCubeTextures(imageBitmaps: ImageBitmap[] | Uint8Array[], gpuTexture: GPUTexture, width: number, height: number, format: GPUTextureFormat, invertY?: boolean, premultiplyAlpha?: boolean, offsetX?: number, offsetY?: number): void;
    updateTexture(imageBitmap: ImageBitmap | Uint8Array | HTMLCanvasElement | OffscreenCanvas, texture: GPUTexture | InternalTexture, width: number, height: number, layers: number, format: GPUTextureFormat, faceIndex?: number, mipLevel?: number, invertY?: boolean, premultiplyAlpha?: boolean, offsetX?: number, offsetY?: number, allowGPUOptimization?: boolean): void;
    readPixels(texture: GPUTexture, x: number, y: number, width: number, height: number, format: GPUTextureFormat, faceIndex?: number, mipLevel?: number, buffer?: Nullable<ArrayBufferView>, noDataConversion?: boolean): Promise<ArrayBufferView>;
    releaseTexture(texture: InternalTexture | GPUTexture): void;
    destroyDeferredTextures(): void;
}

}
declare module "babylonjs/Engines/WebGPU/webgpuTimestampQuery" {
import { WebGPUBufferManager } from "babylonjs/Engines/WebGPU/webgpuBufferManager";
import { PerfCounter } from "babylonjs/Misc/perfCounter";
/** @internal */
export class WebGPUTimestampQuery {
    private _device;
    private _bufferManager;
    private _enabled;
    private _gpuFrameTimeCounter;
    private _measureDuration;
    private _measureDurationState;
    get gpuFrameTimeCounter(): PerfCounter;
    constructor(device: GPUDevice, bufferManager: WebGPUBufferManager);
    get enable(): boolean;
    set enable(value: boolean);
    startFrame(commandEncoder: GPUCommandEncoder): void;
    endFrame(commandEncoder: GPUCommandEncoder): void;
}
/** @internal */
export class WebGPUDurationMeasure {
    private _querySet;
    constructor(device: GPUDevice, bufferManager: WebGPUBufferManager);
    start(encoder: GPUCommandEncoder): void;
    stop(encoder: GPUCommandEncoder): Promise<number | null>;
    dispose(): void;
}

}
declare module "babylonjs/Engines/WebGPU/webgpuTintWASM" {
/**
 * Options to load the associated Twgsl library
 */
export interface TwgslOptions {
    /**
     * Defines an existing instance of Twgsl (useful in modules who do not access the global instance).
     */
    twgsl?: any;
    /**
     * Defines the URL of the twgsl JS File.
     */
    jsPath?: string;
    /**
     * Defines the URL of the twgsl WASM File.
     */
    wasmPath?: string;
}
/** @internal */
export class WebGPUTintWASM {
    private static readonly _TWgslDefaultOptions;
    static ShowWGSLShaderCode: boolean;
    private _twgsl;
    initTwgsl(twgslOptions?: TwgslOptions): Promise<void>;
    convertSpirV2WGSL(code: Uint32Array): string;
}

}
declare module "babylonjs/Engines/webgpuEngine" {
import { Nullable, DataArray, IndicesArray, Immutable } from "babylonjs/types";
import { Engine } from "babylonjs/Engines/engine";
import { InternalTexture, InternalTextureSource } from "babylonjs/Materials/Textures/internalTexture";
import { IEffectCreationOptions } from "babylonjs/Materials/effect";
import { Effect } from "babylonjs/Materials/effect";
import { EffectFallbacks } from "babylonjs/Materials/effectFallbacks";
import { VertexBuffer } from "babylonjs/Buffers/buffer";
import { IPipelineContext } from "babylonjs/Engines/IPipelineContext";
import { DataBuffer } from "babylonjs/Buffers/dataBuffer";
import { BaseTexture } from "babylonjs/Materials/Textures/baseTexture";
import { IShaderProcessor } from "babylonjs/Engines/Processors/iShaderProcessor";
import { ShaderProcessingContext } from "babylonjs/Engines/Processors/shaderProcessingOptions";
import { WebGPUTextureHelper } from "babylonjs/Engines/WebGPU/webgpuTextureHelper";
import { ISceneLike } from "babylonjs/Engines/thinEngine";
import { WebGPUBufferManager } from "babylonjs/Engines/WebGPU/webgpuBufferManager";
import { HardwareTextureWrapper } from "babylonjs/Materials/Textures/hardwareTextureWrapper";
import { IColor4Like } from "babylonjs/Maths/math.like";
import { WebGPURenderPassWrapper } from "babylonjs/Engines/WebGPU/webgpuRenderPassWrapper";
import { WebGPUCacheSampler } from "babylonjs/Engines/WebGPU/webgpuCacheSampler";
import { WebGPUCacheRenderPipeline } from "babylonjs/Engines/WebGPU/webgpuCacheRenderPipeline";
import { DrawWrapper } from "babylonjs/Materials/drawWrapper";
import { WebGPUMaterialContext } from "babylonjs/Engines/WebGPU/webgpuMaterialContext";
import { WebGPUDrawContext } from "babylonjs/Engines/WebGPU/webgpuDrawContext";
import { IStencilState } from "babylonjs/States/IStencilState";
import { WebGPUBundleList } from "babylonjs/Engines/WebGPU/webgpuBundleList";
import { WebGPUTimestampQuery } from "babylonjs/Engines/WebGPU/webgpuTimestampQuery";
import { ComputeEffect } from "babylonjs/Compute/computeEffect";
import { WebGPUOcclusionQuery } from "babylonjs/Engines/WebGPU/webgpuOcclusionQuery";
import { Observable } from "babylonjs/Misc/observable";
import { TwgslOptions } from "babylonjs/Engines/WebGPU/webgpuTintWASM";
import { ExternalTexture } from "babylonjs/Materials/Textures/externalTexture";
import { ShaderLanguage } from "babylonjs/Materials/shaderLanguage";
import { InternalTextureCreationOptions, TextureSize } from "babylonjs/Materials/Textures/textureCreationOptions";
import { WebGPUDataBuffer } from "babylonjs/Meshes/WebGPU/webgpuDataBuffer";
import { RenderTargetWrapper } from "babylonjs/Engines/renderTargetWrapper";
/**
 * Options to load the associated Glslang library
 */
export interface GlslangOptions {
    /**
     * Defines an existing instance of Glslang (useful in modules who do not access the global instance).
     */
    glslang?: any;
    /**
     * Defines the URL of the glslang JS File.
     */
    jsPath?: string;
    /**
     * Defines the URL of the glslang WASM File.
     */
    wasmPath?: string;
}
/**
 * Options to create the WebGPU engine
 */
export interface WebGPUEngineOptions extends GPURequestAdapterOptions {
    /**
     * If delta time between frames should be constant
     * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/advanced_animations#deterministic-lockstep
     */
    deterministicLockstep?: boolean;
    /**
     * Maximum about of steps between frames (Default: 4)
     * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/advanced_animations#deterministic-lockstep
     */
    lockstepMaxSteps?: number;
    /**
     * Defines the seconds between each deterministic lock step
     */
    timeStep?: number;
    /**
     * Defines that engine should ignore context lost events
     * If this event happens when this parameter is true, you will have to reload the page to restore rendering
     */
    doNotHandleContextLost?: boolean;
    /**
     * Defines that engine should ignore modifying touch action attribute and style
     * If not handle, you might need to set it up on your side for expected touch devices behavior.
     */
    doNotHandleTouchAction?: boolean;
    /**
     * Defines if webaudio should be initialized as well
     * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic
     */
    audioEngine?: boolean;
    /**
     * Defines the category of adapter to use.
     * Is it the discrete or integrated device.
     */
    powerPreference?: GPUPowerPreference;
    /**
     * Defines the device descriptor used to create a device.
     */
    deviceDescriptor?: GPUDeviceDescriptor;
    /**
     * Defines the requested Swap Chain Format.
     */
    swapChainFormat?: GPUTextureFormat;
    /**
     * Defines whether MSAA is enabled on the canvas.
     */
    antialiasing?: boolean;
    /**
     * Defines whether the stencil buffer should be enabled.
     */
    stencil?: boolean;
    /**
     * Defines whether we should generate debug markers in the gpu command lists (can be seen with PIX for eg)
     */
    enableGPUDebugMarkers?: boolean;
    /**
     * Options to load the associated Glslang library
     */
    glslangOptions?: GlslangOptions;
    /**
     * Options to load the associated Twgsl library
     */
    twgslOptions?: TwgslOptions;
    /**
     * Defines if the engine should no exceed a specified device ratio
     * @see https://developer.mozilla.org/en-US/docs/Web/API/Window/devicePixelRatio
     */
    limitDeviceRatio?: number;
    /**
     * Defines whether to adapt to the device's viewport characteristics (default: false)
     */
    adaptToDeviceRatio?: boolean;
    /**
     * Defines whether the canvas should be created in "premultiplied" mode (if false, the canvas is created in the "opaque" mode) (true by default)
     */
    premultipliedAlpha?: boolean;
    /**
     * Make the matrix computations to be performed in 64 bits instead of 32 bits. False by default
     */
    useHighPrecisionMatrix?: boolean;
}
/**
 * The web GPU engine class provides support for WebGPU version of babylon.js.
 * @since 5.0.0
 */
export class WebGPUEngine extends Engine {
    private static readonly _GLSLslangDefaultOptions;
    /** true to enable using TintWASM to convert Spir-V to WGSL */
    static UseTWGSL: boolean;
    private readonly _uploadEncoderDescriptor;
    private readonly _renderEncoderDescriptor;
    private readonly _renderTargetEncoderDescriptor;
    /** @internal */
    readonly _clearDepthValue: number;
    /** @internal */
    readonly _clearReverseDepthValue: number;
    /** @internal */
    readonly _clearStencilValue: number;
    private readonly _defaultSampleCount;
    private _canvas;
    /** @internal */
    _options: WebGPUEngineOptions;
    private _glslang;
    private _tintWASM;
    private _adapter;
    private _adapterSupportedExtensions;
    /** @internal */
    _device: GPUDevice;
    private _deviceEnabledExtensions;
    private _context;
    private _mainPassSampleCount;
    /** @internal */
    _textureHelper: WebGPUTextureHelper;
    /** @internal */
    _bufferManager: WebGPUBufferManager;
    private _clearQuad;
    /** @internal */
    _cacheSampler: WebGPUCacheSampler;
    /** @internal */
    _cacheRenderPipeline: WebGPUCacheRenderPipeline;
    private _cacheBindGroups;
    private _emptyVertexBuffer;
    /** @internal */
    _mrtAttachments: number[];
    /** @internal */
    _timestampQuery: WebGPUTimestampQuery;
    /** @internal */
    _occlusionQuery: WebGPUOcclusionQuery;
    /** @internal */
    _compiledComputeEffects: {
        [key: string]: ComputeEffect;
    };
    /** @internal */
    _counters: {
        numEnableEffects: number;
        numEnableDrawWrapper: number;
        numBundleCreationNonCompatMode: number;
        numBundleReuseNonCompatMode: number;
    };
    /**
     * Counters from last frame
     */
    readonly countersLastFrame: {
        numEnableEffects: number;
        numEnableDrawWrapper: number;
        numBundleCreationNonCompatMode: number;
        numBundleReuseNonCompatMode: number;
    };
    /**
     * Max number of uncaptured error messages to log
     */
    numMaxUncapturedErrors: number;
    private _mainTexture;
    private _depthTexture;
    private _mainTextureExtends;
    private _depthTextureFormat;
    private _colorFormat;
    /** @internal */
    _ubInvertY: WebGPUDataBuffer;
    /** @internal */
    _ubDontInvertY: WebGPUDataBuffer;
    /** @internal */
    _uploadEncoder: GPUCommandEncoder;
    /** @internal */
    _renderEncoder: GPUCommandEncoder;
    /** @internal */
    _renderTargetEncoder: GPUCommandEncoder;
    private _commandBuffers;
    /** @internal */
    _currentRenderPass: Nullable<GPURenderPassEncoder>;
    /** @internal */
    _mainRenderPassWrapper: WebGPURenderPassWrapper;
    /** @internal */
    _rttRenderPassWrapper: WebGPURenderPassWrapper;
    /** @internal */
    _pendingDebugCommands: Array<[string, Nullable<string>]>;
    /** @internal */
    _bundleList: WebGPUBundleList;
    /** @internal */
    _bundleListRenderTarget: WebGPUBundleList;
    /** @internal */
    _onAfterUnbindFrameBufferObservable: Observable<WebGPUEngine>;
    private _defaultDrawContext;
    private _defaultMaterialContext;
    /** @internal */
    _currentDrawContext: WebGPUDrawContext;
    /** @internal */
    _currentMaterialContext: WebGPUMaterialContext;
    private _currentOverrideVertexBuffers;
    private _currentIndexBuffer;
    private _colorWriteLocal;
    private _forceEnableEffect;
    /** @internal */
    dbgShowShaderCode: boolean;
    /** @internal */
    dbgSanityChecks: boolean;
    /** @internal */
    dbgVerboseLogsForFirstFrames: boolean;
    /** @internal */
    dbgVerboseLogsNumFrames: number;
    /** @internal */
    dbgLogIfNotDrawWrapper: boolean;
    /** @internal */
    dbgShowEmptyEnableEffectCalls: boolean;
    private _snapshotRendering;
    /**
     * Gets or sets the snapshot rendering mode
     */
    get snapshotRenderingMode(): number;
    set snapshotRenderingMode(mode: number);
    /**
     * Creates a new snapshot at the next frame using the current snapshotRenderingMode
     */
    snapshotRenderingReset(): void;
    /**
     * Enables or disables the snapshot rendering mode
     * Note that the WebGL engine does not support snapshot rendering so setting the value won't have any effect for this engine
     */
    get snapshotRendering(): boolean;
    set snapshotRendering(activate: boolean);
    /**
     * Sets this to true to disable the cache for the samplers. You should do it only for testing purpose!
     */
    get disableCacheSamplers(): boolean;
    set disableCacheSamplers(disable: boolean);
    /**
     * Sets this to true to disable the cache for the render pipelines. You should do it only for testing purpose!
     */
    get disableCacheRenderPipelines(): boolean;
    set disableCacheRenderPipelines(disable: boolean);
    /**
     * Sets this to true to disable the cache for the bind groups. You should do it only for testing purpose!
     */
    get disableCacheBindGroups(): boolean;
    set disableCacheBindGroups(disable: boolean);
    /**
     * Gets a Promise<boolean> indicating if the engine can be instantiated (ie. if a WebGPU context can be found)
     */
    static get IsSupportedAsync(): Promise<boolean>;
    /**
     * Not supported by WebGPU, you should call IsSupportedAsync instead!
     */
    static get IsSupported(): boolean;
    /**
     * Gets a boolean indicating that the engine supports uniform buffers
     */
    get supportsUniformBuffers(): boolean;
    /** Gets the supported extensions by the WebGPU adapter */
    get supportedExtensions(): Immutable<GPUFeatureName[]>;
    /** Gets the currently enabled extensions on the WebGPU device */
    get enabledExtensions(): Immutable<GPUFeatureName[]>;
    /**
     * Returns a string describing the current engine
     */
    get description(): string;
    /**
     * Returns the version of the engine
     */
    get version(): number;
    /**
     * Gets an object containing information about the current engine context
     * @returns an object containing the vendor, the renderer and the version of the current engine context
     */
    getInfo(): {
        vendor: string;
        renderer: string;
        version: string;
    };
    /**
     * (WebGPU only) True (default) to be in compatibility mode, meaning rendering all existing scenes without artifacts (same rendering than WebGL).
     * Setting the property to false will improve performances but may not work in some scenes if some precautions are not taken.
     * See https://doc.babylonjs.com/setup/support/webGPU/webGPUOptimization/webGPUNonCompatibilityMode for more details
     */
    get compatibilityMode(): boolean;
    set compatibilityMode(mode: boolean);
    /** @internal */
    get currentSampleCount(): number;
    /**
     * Create a new instance of the gpu engine asynchronously
     * @param canvas Defines the canvas to use to display the result
     * @param options Defines the options passed to the engine to create the GPU context dependencies
     * @returns a promise that resolves with the created engine
     */
    static CreateAsync(canvas: HTMLCanvasElement, options?: WebGPUEngineOptions): Promise<WebGPUEngine>;
    /**
     * Create a new instance of the gpu engine.
     * @param canvas Defines the canvas to use to display the result
     * @param options Defines the options passed to the engine to create the GPU context dependencies
     */
    constructor(canvas: HTMLCanvasElement, options?: WebGPUEngineOptions);
    /**
     * Initializes the WebGPU context and dependencies.
     * @param glslangOptions Defines the GLSLang compiler options if necessary
     * @param twgslOptions Defines the Twgsl compiler options if necessary
     * @returns a promise notifying the readiness of the engine.
     */
    initAsync(glslangOptions?: GlslangOptions, twgslOptions?: TwgslOptions): Promise<void>;
    private _initGlslang;
    private _initializeLimits;
    private _initializeContextAndSwapChain;
    private _initializeMainAttachments;
    private _configureContext;
    /**
     * Force a specific size of the canvas
     * @param width defines the new canvas' width
     * @param height defines the new canvas' height
     * @param forceSetSize true to force setting the sizes of the underlying canvas
     * @returns true if the size was changed
     */
    setSize(width: number, height: number, forceSetSize?: boolean): boolean;
    private _shaderProcessorWGSL;
    /**
     * @internal
     */
    _getShaderProcessor(shaderLanguage: ShaderLanguage): Nullable<IShaderProcessor>;
    /**
     * @internal
     */
    _getShaderProcessingContext(shaderLanguage: ShaderLanguage): Nullable<ShaderProcessingContext>;
    /** @internal */
    applyStates(): void;
    /**
     * Force the entire cache to be cleared
     * You should not have to use this function unless your engine needs to share the WebGPU context with another engine
     * @param bruteForce defines a boolean to force clearing ALL caches (including stencil, detoh and alpha states)
     */
    wipeCaches(bruteForce?: boolean): void;
    /**
     * Enable or disable color writing
     * @param enable defines the state to set
     */
    setColorWrite(enable: boolean): void;
    /**
     * Gets a boolean indicating if color writing is enabled
     * @returns the current color writing state
     */
    getColorWrite(): boolean;
    private _viewportsCurrent;
    private _resetCurrentViewport;
    private _mustUpdateViewport;
    private _applyViewport;
    /**
     * @internal
     */
    _viewport(x: number, y: number, width: number, height: number): void;
    private _scissorsCurrent;
    protected _scissorCached: {
        x: number;
        y: number;
        z: number;
        w: number;
    };
    private _resetCurrentScissor;
    private _mustUpdateScissor;
    private _applyScissor;
    private _scissorIsActive;
    enableScissor(x: number, y: number, width: number, height: number): void;
    disableScissor(): void;
    private _stencilRefsCurrent;
    private _resetCurrentStencilRef;
    private _mustUpdateStencilRef;
    /**
     * @internal
     */
    _applyStencilRef(renderPass: GPURenderPassEncoder): void;
    private _blendColorsCurrent;
    private _resetCurrentColorBlend;
    private _mustUpdateBlendColor;
    private _applyBlendColor;
    /**
     * Clear the current render buffer or the current render target (if any is set up)
     * @param color defines the color to use
     * @param backBuffer defines if the back buffer must be cleared
     * @param depth defines if the depth buffer must be cleared
     * @param stencil defines if the stencil buffer must be cleared
     */
    clear(color: Nullable<IColor4Like>, backBuffer: boolean, depth: boolean, stencil?: boolean): void;
    private _clearFullQuad;
    /**
     * Creates a vertex buffer
     * @param data the data for the vertex buffer
     * @returns the new buffer
     */
    createVertexBuffer(data: DataArray): DataBuffer;
    /**
     * Creates a vertex buffer
     * @param data the data for the dynamic vertex buffer
     * @returns the new buffer
     */
    createDynamicVertexBuffer(data: DataArray): DataBuffer;
    /**
     * Creates a new index buffer
     * @param indices defines the content of the index buffer
     * @returns a new buffer
     */
    createIndexBuffer(indices: IndicesArray): DataBuffer;
    /**
     * @internal
     */
    _createBuffer(data: DataArray | number, creationFlags: number): DataBuffer;
    /**
     * @internal
     */
    bindBuffersDirectly(): void;
    /**
     * @internal
     */
    updateAndBindInstancesBuffer(): void;
    /**
     * Bind a list of vertex buffers with the engine
     * @param vertexBuffers defines the list of vertex buffers to bind
     * @param indexBuffer defines the index buffer to bind
     * @param effect defines the effect associated with the vertex buffers
     * @param overrideVertexBuffers defines optional list of avertex buffers that overrides the entries in vertexBuffers
     */
    bindBuffers(vertexBuffers: {
        [key: string]: Nullable<VertexBuffer>;
    }, indexBuffer: Nullable<DataBuffer>, effect: Effect, overrideVertexBuffers?: {
        [kind: string]: Nullable<VertexBuffer>;
    }): void;
    /**
     * @internal
     */
    _releaseBuffer(buffer: DataBuffer): boolean;
    /**
     * Create a new effect (used to store vertex/fragment shaders)
     * @param baseName defines the base name of the effect (The name of file without .fragment.fx or .vertex.fx)
     * @param attributesNamesOrOptions defines either a list of attribute names or an IEffectCreationOptions object
     * @param uniformsNamesOrEngine defines either a list of uniform names or the engine to use
     * @param samplers defines an array of string used to represent textures
     * @param defines defines the string containing the defines to use to compile the shaders
     * @param fallbacks defines the list of potential fallbacks to use if shader compilation fails
     * @param onCompiled defines a function to call when the effect creation is successful
     * @param onError defines a function to call when the effect creation has failed
     * @param indexParameters defines an object containing the index values to use to compile shaders (like the maximum number of simultaneous lights)
     * @param shaderLanguage the language the shader is written in (default: GLSL)
     * @returns the new Effect
     */
    createEffect(baseName: any, attributesNamesOrOptions: string[] | IEffectCreationOptions, uniformsNamesOrEngine: string[] | Engine, samplers?: string[], defines?: string, fallbacks?: EffectFallbacks, onCompiled?: Nullable<(effect: Effect) => void>, onError?: Nullable<(effect: Effect, errors: string) => void>, indexParameters?: any, shaderLanguage?: ShaderLanguage): Effect;
    private _compileRawShaderToSpirV;
    private _compileShaderToSpirV;
    private _getWGSLShader;
    private _createPipelineStageDescriptor;
    private _compileRawPipelineStageDescriptor;
    private _compilePipelineStageDescriptor;
    /**
     * @internal
     */
    createRawShaderProgram(): WebGLProgram;
    /**
     * @internal
     */
    createShaderProgram(): WebGLProgram;
    /**
     * Inline functions in shader code that are marked to be inlined
     * @param code code to inline
     * @returns inlined code
     */
    inlineShaderCode(code: string): string;
    /**
     * Creates a new pipeline context
     * @param shaderProcessingContext defines the shader processing context used during the processing if available
     * @returns the new pipeline
     */
    createPipelineContext(shaderProcessingContext: Nullable<ShaderProcessingContext>): IPipelineContext;
    /**
     * Creates a new material context
     * @returns the new context
     */
    createMaterialContext(): WebGPUMaterialContext | undefined;
    /**
     * Creates a new draw context
     * @returns the new context
     */
    createDrawContext(): WebGPUDrawContext | undefined;
    /**
     * @internal
     */
    _preparePipelineContext(pipelineContext: IPipelineContext, vertexSourceCode: string, fragmentSourceCode: string, createAsRaw: boolean, rawVertexSourceCode: string, rawFragmentSourceCode: string, rebuildRebind: any, defines: Nullable<string>): void;
    /**
     * Gets the list of active attributes for a given WebGPU program
     * @param pipelineContext defines the pipeline context to use
     * @param attributesNames defines the list of attribute names to get
     * @returns an array of indices indicating the offset of each attribute
     */
    getAttributes(pipelineContext: IPipelineContext, attributesNames: string[]): number[];
    /**
     * Activates an effect, making it the current one (ie. the one used for rendering)
     * @param effect defines the effect to activate
     */
    enableEffect(effect: Nullable<Effect | DrawWrapper>): void;
    /**
     * @internal
     */
    _releaseEffect(effect: Effect): void;
    /**
     * Force the engine to release all cached effects. This means that next effect compilation will have to be done completely even if a similar effect was already compiled
     */
    releaseEffects(): void;
    _deletePipelineContext(pipelineContext: IPipelineContext): void;
    /**
     * Gets a boolean indicating that only power of 2 textures are supported
     * Please note that you can still use non power of 2 textures but in this case the engine will forcefully convert them
     */
    get needPOTTextures(): boolean;
    /** @internal */
    _createHardwareTexture(): HardwareTextureWrapper;
    /**
     * @internal
     */
    _releaseTexture(texture: InternalTexture): void;
    /**
     * @internal
     */
    _getRGBABufferInternalSizedFormat(): number;
    updateTextureComparisonFunction(texture: InternalTexture, comparisonFunction: number): void;
    /**
     * Creates an internal texture without binding it to a framebuffer
     * @internal
     * @param size defines the size of the texture
     * @param options defines the options used to create the texture
     * @param delayGPUTextureCreation true to delay the texture creation the first time it is really needed. false to create it right away
     * @param source source type of the texture
     * @returns a new internal texture
     */
    _createInternalTexture(size: TextureSize, options: boolean | InternalTextureCreationOptions, delayGPUTextureCreation?: boolean, source?: InternalTextureSource): InternalTexture;
    /**
     * Usually called from Texture.ts.
     * Passed information to create a hardware texture
     * @param url defines a value which contains one of the following:
     * * A conventional http URL, e.g. 'http://...' or 'file://...'
     * * A base64 string of in-line texture data, e.g. 'data:image/jpg;base64,/...'
     * * An indicator that data being passed using the buffer parameter, e.g. 'data:mytexture.jpg'
     * @param noMipmap defines a boolean indicating that no mipmaps shall be generated.  Ignored for compressed textures.  They must be in the file
     * @param invertY when true, image is flipped when loaded.  You probably want true. Certain compressed textures may invert this if their default is inverted (eg. ktx)
     * @param scene needed for loading to the correct scene
     * @param samplingMode mode with should be used sample / access the texture (Default: Texture.TRILINEAR_SAMPLINGMODE)
     * @param onLoad optional callback to be called upon successful completion
     * @param onError optional callback to be called upon failure
     * @param buffer a source of a file previously fetched as either a base64 string, an ArrayBuffer (compressed or image format), HTMLImageElement (image format), or a Blob
     * @param fallback an internal argument in case the function must be called again, due to etc1 not having alpha capabilities
     * @param format internal format.  Default: RGB when extension is '.jpg' else RGBA.  Ignored for compressed textures
     * @param forcedExtension defines the extension to use to pick the right loader
     * @param mimeType defines an optional mime type
     * @param loaderOptions options to be passed to the loader
     * @param creationFlags specific flags to use when creating the texture (Constants.TEXTURE_CREATIONFLAG_STORAGE for storage textures, for eg)
     * @param useSRGBBuffer defines if the texture must be loaded in a sRGB GPU buffer (if supported by the GPU).
     * @returns a InternalTexture for assignment back into BABYLON.Texture
     */
    createTexture(url: Nullable<string>, noMipmap: boolean, invertY: boolean, scene: Nullable<ISceneLike>, samplingMode?: number, onLoad?: Nullable<(texture: InternalTexture) => void>, onError?: Nullable<(message: string, exception: any) => void>, buffer?: Nullable<string | ArrayBuffer | ArrayBufferView | HTMLImageElement | Blob | ImageBitmap>, fallback?: Nullable<InternalTexture>, format?: Nullable<number>, forcedExtension?: Nullable<string>, mimeType?: string, loaderOptions?: any, creationFlags?: number, useSRGBBuffer?: boolean): InternalTexture;
    /**
     * Wraps an external web gpu texture in a Babylon texture.
     * @param texture defines the external texture
     * @returns the babylon internal texture
     */
    wrapWebGPUTexture(texture: GPUTexture): InternalTexture;
    /**
     * Wraps an external web gl texture in a Babylon texture.
     * @returns the babylon internal texture
     */
    wrapWebGLTexture(): InternalTexture;
    generateMipMapsForCubemap(texture: InternalTexture): void;
    /**
     * Update the sampling mode of a given texture
     * @param samplingMode defines the required sampling mode
     * @param texture defines the texture to update
     * @param generateMipMaps defines whether to generate mipmaps for the texture
     */
    updateTextureSamplingMode(samplingMode: number, texture: InternalTexture, generateMipMaps?: boolean): void;
    /**
     * Update the sampling mode of a given texture
     * @param texture defines the texture to update
     * @param wrapU defines the texture wrap mode of the u coordinates
     * @param wrapV defines the texture wrap mode of the v coordinates
     * @param wrapR defines the texture wrap mode of the r coordinates
     */
    updateTextureWrappingMode(texture: InternalTexture, wrapU: Nullable<number>, wrapV?: Nullable<number>, wrapR?: Nullable<number>): void;
    /**
     * Update the dimensions of a texture
     * @param texture texture to update
     * @param width new width of the texture
     * @param height new height of the texture
     * @param depth new depth of the texture
     */
    updateTextureDimensions(texture: InternalTexture, width: number, height: number, depth?: number): void;
    /**
     * @internal
     */
    _setInternalTexture(name: string, texture: Nullable<InternalTexture | ExternalTexture>, baseName?: string): void;
    /**
     * Sets a texture to the according uniform.
     * @param channel The texture channel
     * @param unused unused parameter
     * @param texture The texture to apply
     * @param name The name of the uniform in the effect
     */
    setTexture(channel: number, unused: Nullable<WebGLUniformLocation>, texture: Nullable<BaseTexture>, name: string): void;
    /**
     * Sets an array of texture to the WebGPU context
     * @param channel defines the channel where the texture array must be set
     * @param unused unused parameter
     * @param textures defines the array of textures to bind
     * @param name name of the channel
     */
    setTextureArray(channel: number, unused: Nullable<WebGLUniformLocation>, textures: BaseTexture[], name: string): void;
    protected _setTexture(channel: number, texture: Nullable<BaseTexture>, isPartOfTextureArray?: boolean, depthStencilTexture?: boolean, name?: string, baseName?: string): boolean;
    /**
     * @internal
     */
    _setAnisotropicLevel(target: number, internalTexture: InternalTexture, anisotropicFilteringLevel: number): void;
    /**
     * @internal
     */
    _bindTexture(channel: number, texture: InternalTexture, name: string): void;
    /**
     * Generates the mipmaps for a texture
     * @param texture texture to generate the mipmaps for
     */
    generateMipmaps(texture: InternalTexture): void;
    /**
     * @internal
     */
    _generateMipmaps(texture: InternalTexture, commandEncoder?: GPUCommandEncoder): void;
    /**
     * Update a portion of an internal texture
     * @param texture defines the texture to update
     * @param imageData defines the data to store into the texture
     * @param xOffset defines the x coordinates of the update rectangle
     * @param yOffset defines the y coordinates of the update rectangle
     * @param width defines the width of the update rectangle
     * @param height defines the height of the update rectangle
     * @param faceIndex defines the face index if texture is a cube (0 by default)
     * @param lod defines the lod level to update (0 by default)
     * @param generateMipMaps defines whether to generate mipmaps or not
     */
    updateTextureData(texture: InternalTexture, imageData: ArrayBufferView, xOffset: number, yOffset: number, width: number, height: number, faceIndex?: number, lod?: number, generateMipMaps?: boolean): void;
    /**
     * @internal
     */
    _uploadCompressedDataToTextureDirectly(texture: InternalTexture, internalFormat: number, width: number, height: number, imageData: ArrayBufferView, faceIndex?: number, lod?: number): void;
    /**
     * @internal
     */
    _uploadDataToTextureDirectly(texture: InternalTexture, imageData: ArrayBufferView, faceIndex?: number, lod?: number, babylonInternalFormat?: number, useTextureWidthAndHeight?: boolean): void;
    /**
     * @internal
     */
    _uploadArrayBufferViewToTexture(texture: InternalTexture, imageData: ArrayBufferView, faceIndex?: number, lod?: number): void;
    /**
     * @internal
     */
    _uploadImageToTexture(texture: InternalTexture, image: HTMLImageElement | ImageBitmap, faceIndex?: number, lod?: number): void;
    /**
     * Reads pixels from the current frame buffer. Please note that this function can be slow
     * @param x defines the x coordinate of the rectangle where pixels must be read
     * @param y defines the y coordinate of the rectangle where pixels must be read
     * @param width defines the width of the rectangle where pixels must be read
     * @param height defines the height of the rectangle where pixels must be read
     * @param hasAlpha defines whether the output should have alpha or not (defaults to true)
     * @param flushRenderer true to flush the renderer from the pending commands before reading the pixels
     * @returns a ArrayBufferView promise (Uint8Array) containing RGBA colors
     */
    readPixels(x: number, y: number, width: number, height: number, hasAlpha?: boolean, flushRenderer?: boolean): Promise<ArrayBufferView>;
    /**
     * Begin a new frame
     */
    beginFrame(): void;
    /**
     * End the current frame
     */
    endFrame(): void;
    /**
     * Force a WebGPU flush (ie. a flush of all waiting commands)
     * @param reopenPass true to reopen at the end of the function the pass that was active when entering the function
     */
    flushFramebuffer(reopenPass?: boolean): void;
    /** @internal */
    _currentFrameBufferIsDefaultFrameBuffer(): boolean;
    private _startRenderTargetRenderPass;
    /** @internal */
    _endRenderTargetRenderPass(): void;
    private _getCurrentRenderPass;
    /** @internal */
    _getCurrentRenderPassIndex(): number;
    private _startMainRenderPass;
    private _endMainRenderPass;
    /**
     * Binds the frame buffer to the specified texture.
     * @param texture The render target wrapper to render to
     * @param faceIndex The face of the texture to render to in case of cube texture
     * @param requiredWidth The width of the target to render to
     * @param requiredHeight The height of the target to render to
     * @param forceFullscreenViewport Forces the viewport to be the entire texture/screen if true
     * @param lodLevel defines the lod level to bind to the frame buffer
     * @param layer defines the 2d array index to bind to frame buffer to
     */
    bindFramebuffer(texture: RenderTargetWrapper, faceIndex?: number, requiredWidth?: number, requiredHeight?: number, forceFullscreenViewport?: boolean, lodLevel?: number, layer?: number): void;
    /**
     * Unbind the current render target texture from the WebGPU context
     * @param texture defines the render target wrapper to unbind
     * @param disableGenerateMipMaps defines a boolean indicating that mipmaps must not be generated
     * @param onBeforeUnbind defines a function which will be called before the effective unbind
     */
    unBindFramebuffer(texture: RenderTargetWrapper, disableGenerateMipMaps?: boolean, onBeforeUnbind?: () => void): void;
    /**
     * Unbind the current render target and bind the default framebuffer
     */
    restoreDefaultFramebuffer(): void;
    /**
     * @internal
     */
    _setColorFormat(wrapper: WebGPURenderPassWrapper): void;
    /**
     * @internal
     */
    _setDepthTextureFormat(wrapper: WebGPURenderPassWrapper): void;
    setDitheringState(): void;
    setRasterizerState(): void;
    /**
     * Set various states to the webGL context
     * @param culling defines culling state: true to enable culling, false to disable it
     * @param zOffset defines the value to apply to zOffset (0 by default)
     * @param force defines if states must be applied even if cache is up to date
     * @param reverseSide defines if culling must be reversed (CCW if false, CW if true)
     * @param cullBackFaces true to cull back faces, false to cull front faces (if culling is enabled)
     * @param stencil stencil states to set
     * @param zOffsetUnits defines the value to apply to zOffsetUnits (0 by default)
     */
    setState(culling: boolean, zOffset?: number, force?: boolean, reverseSide?: boolean, cullBackFaces?: boolean, stencil?: IStencilState, zOffsetUnits?: number): void;
    private _applyRenderPassChanges;
    private _draw;
    /**
     * Draw a list of indexed primitives
     * @param fillMode defines the primitive to use
     * @param indexStart defines the starting index
     * @param indexCount defines the number of index to draw
     * @param instancesCount defines the number of instances to draw (if instantiation is enabled)
     */
    drawElementsType(fillMode: number, indexStart: number, indexCount: number, instancesCount?: number): void;
    /**
     * Draw a list of unindexed primitives
     * @param fillMode defines the primitive to use
     * @param verticesStart defines the index of first vertex to draw
     * @param verticesCount defines the count of vertices to draw
     * @param instancesCount defines the number of instances to draw (if instantiation is enabled)
     */
    drawArraysType(fillMode: number, verticesStart: number, verticesCount: number, instancesCount?: number): void;
    /**
     * Dispose and release all associated resources
     */
    dispose(): void;
    /**
     * Gets the current render width
     * @param useScreen defines if screen size must be used (or the current render target if any)
     * @returns a number defining the current render width
     */
    getRenderWidth(useScreen?: boolean): number;
    /**
     * Gets the current render height
     * @param useScreen defines if screen size must be used (or the current render target if any)
     * @returns a number defining the current render height
     */
    getRenderHeight(useScreen?: boolean): number;
    /**
     * Gets the HTML canvas attached with the current WebGPU context
     * @returns a HTML canvas
     */
    getRenderingCanvas(): Nullable<HTMLCanvasElement>;
    /**
     * Get the current error code of the WebGPU context
     * @returns the error code
     */
    getError(): number;
    /**
     * @internal
     */
    bindSamplers(): void;
    /**
     * @internal
     */
    _bindTextureDirectly(): boolean;
    /**
     * Gets a boolean indicating if all created effects are ready
     * @returns always true - No parallel shader compilation
     */
    areAllEffectsReady(): boolean;
    /**
     * @internal
     */
    _executeWhenRenderingStateIsCompiled(pipelineContext: IPipelineContext, action: () => void): void;
    /**
     * @internal
     */
    _isRenderingStateCompiled(): boolean;
    /** @internal */
    _getUnpackAlignement(): number;
    /**
     * @internal
     */
    _unpackFlipY(): void;
    /**
     * @internal
     */
    _bindUnboundFramebuffer(): void;
    /**
     * @internal
     */
    _getSamplingParameters(): {
        min: number;
        mag: number;
    };
    /**
     * @internal
     */
    getUniforms(): Nullable<WebGLUniformLocation>[];
    /**
     * @internal
     */
    setIntArray(): boolean;
    /**
     * @internal
     */
    setIntArray2(): boolean;
    /**
     * @internal
     */
    setIntArray3(): boolean;
    /**
     * @internal
     */
    setIntArray4(): boolean;
    /**
     * @internal
     */
    setArray(): boolean;
    /**
     * @internal
     */
    setArray2(): boolean;
    /**
     * @internal
     */
    setArray3(): boolean;
    /**
     * @internal
     */
    setArray4(): boolean;
    /**
     * @internal
     */
    setMatrices(): boolean;
    /**
     * @internal
     */
    setMatrix3x3(): boolean;
    /**
     * @internal
     */
    setMatrix2x2(): boolean;
    /**
     * @internal
     */
    setFloat(): boolean;
    /**
     * @internal
     */
    setFloat2(): boolean;
    /**
     * @internal
     */
    setFloat3(): boolean;
    /**
     * @internal
     */
    setFloat4(): boolean;
}
export {};

}
declare module "babylonjs/Events/clipboardEvents" {
/**
 * Gather the list of clipboard event types as constants.
 */
export class ClipboardEventTypes {
    /**
     * The clipboard event is fired when a copy command is active (pressed).
     */
    static readonly COPY: number;
    /**
     *  The clipboard event is fired when a cut command is active (pressed).
     */
    static readonly CUT: number;
    /**
     * The clipboard event is fired when a paste command is active (pressed).
     */
    static readonly PASTE: number;
}
/**
 * This class is used to store clipboard related info for the onClipboardObservable event.
 */
export class ClipboardInfo {
    /**
     * Defines the type of event (BABYLON.ClipboardEventTypes)
     */
    type: number;
    /**
     * Defines the related dom event
     */
    event: ClipboardEvent;
    /**
     *Creates an instance of ClipboardInfo.
     * @param type Defines the type of event (BABYLON.ClipboardEventTypes)
     * @param event Defines the related dom event
     */
    constructor(
    /**
     * Defines the type of event (BABYLON.ClipboardEventTypes)
     */
    type: number, 
    /**
     * Defines the related dom event
     */
    event: ClipboardEvent);
    /**
     *  Get the clipboard event's type from the keycode.
     * @param keyCode Defines the keyCode for the current keyboard event.
     * @returns {number}
     */
    static GetTypeFromCharacter(keyCode: number): number;
}

}
declare module "babylonjs/Events/deviceInputEvents" {
import { PointerInput } from "babylonjs/DeviceInput/InputDevices/deviceEnums";
/**
 * Event Types
 */
export enum DeviceInputEventType {
    /** PointerMove */
    PointerMove = 0,
    /** PointerDown */
    PointerDown = 1,
    /** PointerUp */
    PointerUp = 2
}
/**
 * Native friendly interface for Event Object
 */
export interface IUIEvent {
    /**
     * Input array index
     */
    inputIndex: number;
    /**
     * Current target for an event
     */
    currentTarget?: any;
    /**
     * Alias for target
     * @deprecated Use target instead
     */
    srcElement?: any;
    /**
     * Type of event
     */
    type: string;
    /**
     * Reference to object where object was dispatched
     */
    target: any;
    /**
     * Tells user agent what to do when not explicitly handled
     */
    preventDefault: () => void;
}
/**
 * Native friendly interface for KeyboardEvent Object
 */
export interface IKeyboardEvent extends IUIEvent {
    /**
     * Status of Alt key being pressed
     */
    altKey: boolean;
    /**
     * Unicode value of character pressed
     * @deprecated Required for event, use keyCode instead.
     */
    charCode?: number;
    /**
     * Code for key based on layout
     */
    code: string;
    /**
     * Status of Ctrl key being pressed
     */
    ctrlKey: boolean;
    /**
     * String representation of key
     */
    key: string;
    /**
     * ASCII value of key
     * @deprecated Used with DeviceSourceManager
     */
    keyCode: number;
    /**
     * Status of Meta key (eg. Windows key) being pressed
     */
    metaKey: boolean;
    /**
     * Status of Shift key being pressed
     */
    shiftKey: boolean;
}
/**
 * Native friendly interface for MouseEvent Object
 */
export interface IMouseEvent extends IUIEvent {
    /**
     * Subset of possible PointerInput values for events, excluding ones that CANNOT be in events organically
     */
    inputIndex: Exclude<PointerInput, PointerInput.Horizontal | PointerInput.Vertical>;
    /**
     * Status of Alt key being pressed
     */
    altKey: boolean;
    /**
     * Value of single mouse button pressed
     */
    button: number;
    /**
     * Value of all mouse buttons pressed
     */
    buttons: number;
    /**
     * Current X coordinate
     */
    clientX: number;
    /**
     * Current Y coordinate
     */
    clientY: number;
    /**
     * Status of Ctrl key being pressed
     */
    ctrlKey: boolean;
    /**
     * Provides current click count
     */
    detail?: number;
    /**
     * Status of Meta key (eg. Windows key) being pressed
     */
    metaKey: boolean;
    /**
     * Delta of movement on X axis
     */
    movementX: number;
    /**
     * Delta of movement on Y axis
     */
    movementY: number;
    /**
     * Delta of movement on X axis
     * @deprecated Use 'movementX' instead
     */
    mozMovementX?: number;
    /**
     * Delta of movement on Y axis
     * @deprecated Use 'movementY' instead
     */
    mozMovementY?: number;
    /**
     * Delta of movement on X axis
     * @deprecated Use 'movementX' instead
     */
    msMovementX?: number;
    /**
     * Delta of movement on Y axis
     * @deprecated Use 'movementY' instead
     */
    msMovementY?: number;
    /**
     * Current coordinate of X within container
     */
    offsetX: number;
    /**
     * Current coordinate of Y within container
     */
    offsetY: number;
    /**
     * Horizontal coordinate of event
     */
    pageX: number;
    /**
     * Vertical coordinate of event
     */
    pageY: number;
    /**
     * Status of Shift key being pressed
     */
    shiftKey: boolean;
    /**
     * Delta of movement on X axis
     * @deprecated Use 'movementX' instead
     */
    webkitMovementX?: number;
    /**
     * Delta of movement on Y axis
     * @deprecated Use 'movementY' instead
     */
    webkitMovementY?: number;
    /**
     * Alias of clientX
     */
    x: number;
    /**
     * Alias of clientY
     */
    y: number;
}
/**
 * Native friendly interface for PointerEvent Object
 */
export interface IPointerEvent extends IMouseEvent {
    /**
     * Subset of possible PointerInput values for events, excluding ones that CANNOT be in events organically and mouse wheel values
     */
    inputIndex: Exclude<PointerInput, PointerInput.Horizontal | PointerInput.Vertical | PointerInput.MouseWheelX | PointerInput.MouseWheelY | PointerInput.MouseWheelZ>;
    /**
     * Pointer Event ID
     */
    pointerId: number;
    /**
     * Type of pointer
     */
    pointerType: string;
}
/**
 * Native friendly interface for WheelEvent Object
 */
export interface IWheelEvent extends IMouseEvent {
    /**
     * Subset of possible PointerInput values for events that can only be used with mouse wheel
     */
    inputIndex: PointerInput.MouseWheelX | PointerInput.MouseWheelY | PointerInput.MouseWheelZ;
    /**
     * Units for delta value
     */
    deltaMode: number;
    /**
     * Horizontal scroll delta
     */
    deltaX: number;
    /**
     * Vertical scroll delta
     */
    deltaY: number;
    /**
     * Z-Axis scroll delta
     */
    deltaZ: number;
    /**
     * WheelDelta (From MouseWheel Event)
     * @deprecated
     */
    wheelDelta?: number;
}
/**
 * Constants used for Events
 */
export class EventConstants {
    /**
     * Pixel delta for Wheel Events (Default)
     */
    static DOM_DELTA_PIXEL: number;
    /**
     * Line delta for Wheel Events
     */
    static DOM_DELTA_LINE: number;
    /**
     * Page delta for Wheel Events
     */
    static DOM_DELTA_PAGE: number;
}

}
declare module "babylonjs/Events/index" {
export * from "babylonjs/Events/keyboardEvents";
export * from "babylonjs/Events/pointerEvents";
export * from "babylonjs/Events/clipboardEvents";
export * from "babylonjs/Events/deviceInputEvents";

}
declare module "babylonjs/Events/keyboardEvents" {
import { IKeyboardEvent } from "babylonjs/Events/deviceInputEvents";
/**
 * Gather the list of keyboard event types as constants.
 */
export class KeyboardEventTypes {
    /**
     * The keydown event is fired when a key becomes active (pressed).
     */
    static readonly KEYDOWN: number;
    /**
     * The keyup event is fired when a key has been released.
     */
    static readonly KEYUP: number;
}
/**
 * This class is used to store keyboard related info for the onKeyboardObservable event.
 */
export class KeyboardInfo {
    /**
     * Defines the type of event (KeyboardEventTypes)
     */
    type: number;
    /**
     * Defines the related dom event
     */
    event: IKeyboardEvent;
    /**
     * Instantiates a new keyboard info.
     * This class is used to store keyboard related info for the onKeyboardObservable event.
     * @param type Defines the type of event (KeyboardEventTypes)
     * @param event Defines the related dom event
     */
    constructor(
    /**
     * Defines the type of event (KeyboardEventTypes)
     */
    type: number, 
    /**
     * Defines the related dom event
     */
    event: IKeyboardEvent);
}
/**
 * This class is used to store keyboard related info for the onPreKeyboardObservable event.
 * Set the skipOnKeyboardObservable property to true if you want the engine to stop any process after this event is triggered, even not calling onKeyboardObservable
 */
export class KeyboardInfoPre extends KeyboardInfo {
    /**
     * Defines the type of event (KeyboardEventTypes)
     */
    type: number;
    /**
     * Defines the related dom event
     */
    event: IKeyboardEvent;
    /**
     * Defines whether the engine should skip the next onKeyboardObservable associated to this pre.
     */
    skipOnKeyboardObservable: boolean;
    /**
     * Defines whether the engine should skip the next onKeyboardObservable associated to this pre.
     * @deprecated use skipOnKeyboardObservable property instead
     */
    get skipOnPointerObservable(): boolean;
    set skipOnPointerObservable(value: boolean);
    /**
     * Instantiates a new keyboard pre info.
     * This class is used to store keyboard related info for the onPreKeyboardObservable event.
     * @param type Defines the type of event (KeyboardEventTypes)
     * @param event Defines the related dom event
     */
    constructor(
    /**
     * Defines the type of event (KeyboardEventTypes)
     */
    type: number, 
    /**
     * Defines the related dom event
     */
    event: IKeyboardEvent);
}

}
declare module "babylonjs/Events/pointerEvents" {
import { Nullable } from "babylonjs/types";
import { Vector2 } from "babylonjs/Maths/math.vector";
import { PickingInfo } from "babylonjs/Collisions/pickingInfo";
import { IMouseEvent } from "babylonjs/Events/deviceInputEvents";
import { InputManager } from "babylonjs/Inputs/scene.inputManager";
import { Ray } from "babylonjs/Culling/ray";
/**
 * Gather the list of pointer event types as constants.
 */
export class PointerEventTypes {
    /**
     * The pointerdown event is fired when a pointer becomes active. For mouse, it is fired when the device transitions from no buttons depressed to at least one button depressed. For touch, it is fired when physical contact is made with the digitizer. For pen, it is fired when the stylus makes physical contact with the digitizer.
     */
    static readonly POINTERDOWN: number;
    /**
     * The pointerup event is fired when a pointer is no longer active.
     */
    static readonly POINTERUP: number;
    /**
     * The pointermove event is fired when a pointer changes coordinates.
     */
    static readonly POINTERMOVE: number;
    /**
     * The pointerwheel event is fired when a mouse wheel has been rotated.
     */
    static readonly POINTERWHEEL: number;
    /**
     * The pointerpick event is fired when a mesh or sprite has been picked by the pointer.
     */
    static readonly POINTERPICK: number;
    /**
     * The pointertap event is fired when a the object has been touched and released without drag.
     */
    static readonly POINTERTAP: number;
    /**
     * The pointerdoubletap event is fired when a the object has been touched and released twice without drag.
     */
    static readonly POINTERDOUBLETAP: number;
}
/**
 * Base class of pointer info types.
 */
export class PointerInfoBase {
    /**
     * Defines the type of event (PointerEventTypes)
     */
    type: number;
    /**
     * Defines the related dom event
     */
    event: IMouseEvent;
    /**
     * Instantiates the base class of pointers info.
     * @param type Defines the type of event (PointerEventTypes)
     * @param event Defines the related dom event
     */
    constructor(
    /**
     * Defines the type of event (PointerEventTypes)
     */
    type: number, 
    /**
     * Defines the related dom event
     */
    event: IMouseEvent);
}
/**
 * This class is used to store pointer related info for the onPrePointerObservable event.
 * Set the skipOnPointerObservable property to true if you want the engine to stop any process after this event is triggered, even not calling onPointerObservable
 */
export class PointerInfoPre extends PointerInfoBase {
    /**
     * Ray from a pointer if available (eg. 6dof controller)
     */
    ray: Nullable<Ray>;
    /**
     * Defines picking info coming from a near interaction (proximity instead of ray-based picking)
     */
    nearInteractionPickingInfo: Nullable<PickingInfo>;
    /**
     * The original picking info that was used to trigger the pointer event
     */
    originalPickingInfo: Nullable<PickingInfo>;
    /**
     * Defines the local position of the pointer on the canvas.
     */
    localPosition: Vector2;
    /**
     * Defines whether the engine should skip the next OnPointerObservable associated to this pre.
     */
    skipOnPointerObservable: boolean;
    /**
     * Instantiates a PointerInfoPre to store pointer related info to the onPrePointerObservable event.
     * @param type Defines the type of event (PointerEventTypes)
     * @param event Defines the related dom event
     * @param localX Defines the local x coordinates of the pointer when the event occured
     * @param localY Defines the local y coordinates of the pointer when the event occured
     */
    constructor(type: number, event: IMouseEvent, localX: number, localY: number);
}
/**
 * This type contains all the data related to a pointer event in Babylon.js.
 * The event member is an instance of PointerEvent for all types except PointerWheel and is of type MouseWheelEvent when type equals PointerWheel. The different event types can be found in the PointerEventTypes class.
 */
export class PointerInfo extends PointerInfoBase {
    private _pickInfo;
    private _inputManager;
    /**
     * Defines the picking info associated with this PointerInfo object (if applicable)
     */
    get pickInfo(): Nullable<PickingInfo>;
    /**
     * Instantiates a PointerInfo to store pointer related info to the onPointerObservable event.
     * @param type Defines the type of event (PointerEventTypes)
     * @param event Defines the related dom event
     * @param pickInfo Defines the picking info associated to the info (if any)
     * @param inputManager Defines the InputManager to use if there is no pickInfo
     */
    constructor(type: number, event: IMouseEvent, pickInfo: Nullable<PickingInfo>, inputManager?: Nullable<InputManager>);
    /**
     * Generates the picking info if needed
     */
    /** @internal */
    _generatePickInfo(): void;
}
/**
 * Data relating to a touch event on the screen.
 */
export interface PointerTouch {
    /**
     * X coordinate of touch.
     */
    x: number;
    /**
     * Y coordinate of touch.
     */
    y: number;
    /**
     * Id of touch. Unique for each finger.
     */
    pointerId: number;
    /**
     * Event type passed from DOM.
     */
    type: any;
}
export {};

}
declare module "babylonjs/Gamepads/Controllers/daydreamController" {
import { Scene } from "babylonjs/scene";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { WebVRController } from "babylonjs/Gamepads/Controllers/webVRController";
import { ExtendedGamepadButton } from "babylonjs/Gamepads/Controllers/poseEnabledController";
/**
 * Google Daydream controller
 */
export class DaydreamController extends WebVRController {
    /**
     * Base Url for the controller model.
     */
    static MODEL_BASE_URL: string;
    /**
     * File name for the controller model.
     */
    static MODEL_FILENAME: string;
    /**
     * Gamepad Id prefix used to identify Daydream Controller.
     */
    static readonly GAMEPAD_ID_PREFIX: string;
    /**
     * Creates a new DaydreamController from a gamepad
     * @param vrGamepad the gamepad that the controller should be created from
     */
    constructor(vrGamepad: any);
    /**
     * Implements abstract method on WebVRController class, loading controller meshes and calling this.attachToMesh if successful.
     * @param scene scene in which to add meshes
     * @param meshLoaded optional callback function that will be called if the mesh loads successfully.
     */
    initControllerMesh(scene: Scene, meshLoaded?: (mesh: AbstractMesh) => void): void;
    /**
     * Called once for each button that changed state since the last frame
     * @param buttonIdx Which button index changed
     * @param state New state of the button
     */
    protected _handleButtonChange(buttonIdx: number, state: ExtendedGamepadButton): void;
}

}
declare module "babylonjs/Gamepads/Controllers/gearVRController" {
import { Scene } from "babylonjs/scene";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { WebVRController } from "babylonjs/Gamepads/Controllers/webVRController";
import { ExtendedGamepadButton } from "babylonjs/Gamepads/Controllers/poseEnabledController";
/**
 * Gear VR Controller
 */
export class GearVRController extends WebVRController {
    /**
     * Base Url for the controller model.
     */
    static MODEL_BASE_URL: string;
    /**
     * File name for the controller model.
     */
    static MODEL_FILENAME: string;
    /**
     * Gamepad Id prefix used to identify this controller.
     */
    static readonly GAMEPAD_ID_PREFIX: string;
    private readonly _buttonIndexToObservableNameMap;
    /**
     * Creates a new GearVRController from a gamepad
     * @param vrGamepad the gamepad that the controller should be created from
     */
    constructor(vrGamepad: any);
    /**
     * Implements abstract method on WebVRController class, loading controller meshes and calling this.attachToMesh if successful.
     * @param scene scene in which to add meshes
     * @param meshLoaded optional callback function that will be called if the mesh loads successfully.
     */
    initControllerMesh(scene: Scene, meshLoaded?: (mesh: AbstractMesh) => void): void;
    /**
     * Called once for each button that changed state since the last frame
     * @param buttonIdx Which button index changed
     * @param state New state of the button
     */
    protected _handleButtonChange(buttonIdx: number, state: ExtendedGamepadButton): void;
}

}
declare module "babylonjs/Gamepads/Controllers/genericController" {
import { Scene } from "babylonjs/scene";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { WebVRController } from "babylonjs/Gamepads/Controllers/webVRController";
import { ExtendedGamepadButton } from "babylonjs/Gamepads/Controllers/poseEnabledController";
/**
 * Generic Controller
 */
export class GenericController extends WebVRController {
    /**
     * Base Url for the controller model.
     */
    static readonly MODEL_BASE_URL: string;
    /**
     * File name for the controller model.
     */
    static readonly MODEL_FILENAME: string;
    /**
     * Creates a new GenericController from a gamepad
     * @param vrGamepad the gamepad that the controller should be created from
     */
    constructor(vrGamepad: any);
    /**
     * Implements abstract method on WebVRController class, loading controller meshes and calling this.attachToMesh if successful.
     * @param scene scene in which to add meshes
     * @param meshLoaded optional callback function that will be called if the mesh loads successfully.
     */
    initControllerMesh(scene: Scene, meshLoaded?: (mesh: AbstractMesh) => void): void;
    /**
     * Called once for each button that changed state since the last frame
     * @param buttonIdx Which button index changed
     * @param state New state of the button
     */
    protected _handleButtonChange(buttonIdx: number, state: ExtendedGamepadButton): void;
}

}
declare module "babylonjs/Gamepads/Controllers/index" {
export * from "babylonjs/Gamepads/Controllers/daydreamController";
export * from "babylonjs/Gamepads/Controllers/gearVRController";
export * from "babylonjs/Gamepads/Controllers/genericController";
export * from "babylonjs/Gamepads/Controllers/oculusTouchController";
export * from "babylonjs/Gamepads/Controllers/poseEnabledController";
export * from "babylonjs/Gamepads/Controllers/viveController";
export * from "babylonjs/Gamepads/Controllers/webVRController";
export * from "babylonjs/Gamepads/Controllers/windowsMotionController";

}
declare module "babylonjs/Gamepads/Controllers/oculusTouchController" {
import { Observable } from "babylonjs/Misc/observable";
import { Scene } from "babylonjs/scene";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { WebVRController } from "babylonjs/Gamepads/Controllers/webVRController";
import { ExtendedGamepadButton } from "babylonjs/Gamepads/Controllers/poseEnabledController";
/**
 * Oculus Touch Controller
 */
export class OculusTouchController extends WebVRController {
    /**
     * Base Url for the controller model.
     */
    static MODEL_BASE_URL: string;
    /**
     * File name for the left controller model.
     */
    static MODEL_LEFT_FILENAME: string;
    /**
     * File name for the right controller model.
     */
    static MODEL_RIGHT_FILENAME: string;
    /**
     * Base Url for the Quest controller model.
     */
    static QUEST_MODEL_BASE_URL: string;
    /**
     * @internal
     * If the controllers are running on a device that needs the updated Quest controller models
     */
    static _IsQuest: boolean;
    /**
     * Fired when the secondary trigger on this controller is modified
     */
    onSecondaryTriggerStateChangedObservable: Observable<ExtendedGamepadButton>;
    /**
     * Fired when the thumb rest on this controller is modified
     */
    onThumbRestChangedObservable: Observable<ExtendedGamepadButton>;
    /**
     * Creates a new OculusTouchController from a gamepad
     * @param vrGamepad the gamepad that the controller should be created from
     */
    constructor(vrGamepad: any);
    /**
     * Implements abstract method on WebVRController class, loading controller meshes and calling this.attachToMesh if successful.
     * @param scene scene in which to add meshes
     * @param meshLoaded optional callback function that will be called if the mesh loads successfully.
     */
    initControllerMesh(scene: Scene, meshLoaded?: (mesh: AbstractMesh) => void): void;
    /**
     * Fired when the A button on this controller is modified
     */
    get onAButtonStateChangedObservable(): Observable<ExtendedGamepadButton>;
    /**
     * Fired when the B button on this controller is modified
     */
    get onBButtonStateChangedObservable(): Observable<ExtendedGamepadButton>;
    /**
     * Fired when the X button on this controller is modified
     */
    get onXButtonStateChangedObservable(): Observable<ExtendedGamepadButton>;
    /**
     * Fired when the Y button on this controller is modified
     */
    get onYButtonStateChangedObservable(): Observable<ExtendedGamepadButton>;
    /**
     * Called once for each button that changed state since the last frame
     * 0) thumb stick (touch, press, value = pressed (0,1)). value is in this.leftStick
     * 1) index trigger (touch (?), press (only when value > 0.1), value 0 to 1)
     * 2) secondary trigger (same)
     * 3) A (right) X (left), touch, pressed = value
     * 4) B / Y
     * 5) thumb rest
     * @param buttonIdx Which button index changed
     * @param state New state of the button
     */
    protected _handleButtonChange(buttonIdx: number, state: ExtendedGamepadButton): void;
}

}
declare module "babylonjs/Gamepads/Controllers/poseEnabledController" {
import { Observable } from "babylonjs/Misc/observable";
import { Nullable } from "babylonjs/types";
import { Quaternion, Matrix, Vector3 } from "babylonjs/Maths/math.vector";
import { TransformNode } from "babylonjs/Meshes/transformNode";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { Ray } from "babylonjs/Culling/ray";
import { Gamepad } from "babylonjs/Gamepads/gamepad";
import { PoseControlled, DevicePose } from "babylonjs/Cameras/VR/webVRCamera";
import { TargetCamera } from "babylonjs/Cameras/targetCamera";
/**
 * Defines the types of pose enabled controllers that are supported
 */
export enum PoseEnabledControllerType {
    /**
     * HTC Vive
     */
    VIVE = 0,
    /**
     * Oculus Rift
     */
    OCULUS = 1,
    /**
     * Windows mixed reality
     */
    WINDOWS = 2,
    /**
     * Samsung gear VR
     */
    GEAR_VR = 3,
    /**
     * Google Daydream
     */
    DAYDREAM = 4,
    /**
     * Generic
     */
    GENERIC = 5
}
/**
 * Defines the MutableGamepadButton interface for the state of a gamepad button
 */
export interface MutableGamepadButton {
    /**
     * Value of the button/trigger
     */
    value: number;
    /**
     * If the button/trigger is currently touched
     */
    touched: boolean;
    /**
     * If the button/trigger is currently pressed
     */
    pressed: boolean;
}
/**
 * Defines the ExtendedGamepadButton interface for a gamepad button which includes state provided by a pose controller
 * @internal
 */
export interface ExtendedGamepadButton extends GamepadButton {
    /**
     * If the button/trigger is currently pressed
     */
    readonly pressed: boolean;
    /**
     * If the button/trigger is currently touched
     */
    readonly touched: boolean;
    /**
     * Value of the button/trigger
     */
    readonly value: number;
}
/** @internal */
export interface _GamePadFactory {
    /**
     * Returns whether or not the current gamepad can be created for this type of controller.
     * @param gamepadInfo Defines the gamepad info as received from the controller APIs.
     * @returns true if it can be created, otherwise false
     */
    canCreate(gamepadInfo: any): boolean;
    /**
     * Creates a new instance of the Gamepad.
     * @param gamepadInfo Defines the gamepad info as received from the controller APIs.
     * @returns the new gamepad instance
     */
    create(gamepadInfo: any): Gamepad;
}
/**
 * Defines the PoseEnabledControllerHelper object that is used initialize a gamepad as the controller type it is specified as (eg. windows mixed reality controller)
 */
export class PoseEnabledControllerHelper {
    /** @internal */
    static _ControllerFactories: _GamePadFactory[];
    /** @internal */
    static _DefaultControllerFactory: Nullable<(gamepadInfo: any) => Gamepad>;
    /**
     * Initializes a gamepad as the controller type it is specified as (eg. windows mixed reality controller)
     * @param vrGamepad the gamepad to initialized
     * @returns a vr controller of the type the gamepad identified as
     */
    static InitiateController(vrGamepad: any): Gamepad;
}
/**
 * Defines the PoseEnabledController object that contains state of a vr capable controller
 */
export class PoseEnabledController extends Gamepad implements PoseControlled {
    /**
     * If the controller is used in a webXR session
     */
    isXR: boolean;
    private _deviceRoomPosition;
    private _deviceRoomRotationQuaternion;
    /**
     * The device position in babylon space
     */
    devicePosition: Vector3;
    /**
     * The device rotation in babylon space
     */
    deviceRotationQuaternion: Quaternion;
    /**
     * The scale factor of the device in babylon space
     */
    deviceScaleFactor: number;
    /**
     * (Likely devicePosition should be used instead) The device position in its room space
     */
    position: Vector3;
    /**
     * (Likely deviceRotationQuaternion should be used instead) The device rotation in its room space
     */
    rotationQuaternion: Quaternion;
    /**
     * The type of controller (Eg. Windows mixed reality)
     */
    controllerType: PoseEnabledControllerType;
    protected _calculatedPosition: Vector3;
    private _calculatedRotation;
    /**
     * The raw pose from the device
     */
    rawPose: DevicePose;
    private _trackPosition;
    private _maxRotationDistFromHeadset;
    private _draggedRoomRotation;
    /**
     * @internal
     */
    _disableTrackPosition(fixedPosition: Vector3): void;
    /**
     * Internal, the mesh attached to the controller
     * @internal
     */
    _mesh: Nullable<AbstractMesh>;
    private _poseControlledCamera;
    private _leftHandSystemQuaternion;
    /**
     * Internal, matrix used to convert room space to babylon space
     * @internal
     */
    _deviceToWorld: Matrix;
    /**
     * Node to be used when casting a ray from the controller
     * @internal
     */
    _pointingPoseNode: Nullable<TransformNode>;
    /**
     * Name of the child mesh that can be used to cast a ray from the controller
     */
    static readonly POINTING_POSE: string;
    /**
     * Creates a new PoseEnabledController from a gamepad
     * @param browserGamepad the gamepad that the PoseEnabledController should be created from
     */
    constructor(browserGamepad: any);
    private _workingMatrix;
    /**
     * Updates the state of the pose enabled controller and mesh based on the current position and rotation of the controller
     */
    update(): void;
    /**
     * Updates only the pose device and mesh without doing any button event checking
     */
    protected _updatePoseAndMesh(): void;
    /**
     * Updates the state of the pose enbaled controller based on the raw pose data from the device
     * @param poseData raw pose fromthe device
     */
    updateFromDevice(poseData: DevicePose): void;
    /**
     * @internal
     */
    _meshAttachedObservable: Observable<AbstractMesh>;
    /**
     * Attaches a mesh to the controller
     * @param mesh the mesh to be attached
     */
    attachToMesh(mesh: AbstractMesh): void;
    /**
     * Attaches the controllers mesh to a camera
     * @param camera the camera the mesh should be attached to
     */
    attachToPoseControlledCamera(camera: TargetCamera): void;
    /**
     * Disposes of the controller
     */
    dispose(): void;
    /**
     * The mesh that is attached to the controller
     */
    get mesh(): Nullable<AbstractMesh>;
    /**
     * Gets the ray of the controller in the direction the controller is pointing
     * @param length the length the resulting ray should be
     * @returns a ray in the direction the controller is pointing
     */
    getForwardRay(length?: number): Ray;
}

}
declare module "babylonjs/Gamepads/Controllers/viveController" {
import { Scene } from "babylonjs/scene";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { WebVRController } from "babylonjs/Gamepads/Controllers/webVRController";
import { ExtendedGamepadButton } from "babylonjs/Gamepads/Controllers/poseEnabledController";
import { Observable } from "babylonjs/Misc/observable";
/**
 * Vive Controller
 */
export class ViveController extends WebVRController {
    /**
     * Base Url for the controller model.
     */
    static MODEL_BASE_URL: string;
    /**
     * File name for the controller model.
     */
    static MODEL_FILENAME: string;
    /**
     * Creates a new ViveController from a gamepad
     * @param vrGamepad the gamepad that the controller should be created from
     */
    constructor(vrGamepad: any);
    /**
     * Implements abstract method on WebVRController class, loading controller meshes and calling this.attachToMesh if successful.
     * @param scene scene in which to add meshes
     * @param meshLoaded optional callback function that will be called if the mesh loads successfully.
     */
    initControllerMesh(scene: Scene, meshLoaded?: (mesh: AbstractMesh) => void): void;
    /**
     * Fired when the left button on this controller is modified
     */
    get onLeftButtonStateChangedObservable(): Observable<ExtendedGamepadButton>;
    /**
     * Fired when the right button on this controller is modified
     */
    get onRightButtonStateChangedObservable(): Observable<ExtendedGamepadButton>;
    /**
     * Fired when the menu button on this controller is modified
     */
    get onMenuButtonStateChangedObservable(): Observable<ExtendedGamepadButton>;
    /**
     * Called once for each button that changed state since the last frame
     * Vive mapping:
     * 0: touchpad
     * 1: trigger
     * 2: left AND right buttons
     * 3: menu button
     * @param buttonIdx Which button index changed
     * @param state New state of the button
     */
    protected _handleButtonChange(buttonIdx: number, state: ExtendedGamepadButton): void;
}

}
declare module "babylonjs/Gamepads/Controllers/webVRController" {
import { Observable } from "babylonjs/Misc/observable";
import { Scene } from "babylonjs/scene";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { ExtendedGamepadButton, MutableGamepadButton } from "babylonjs/Gamepads/Controllers/poseEnabledController";
import { PoseEnabledController } from "babylonjs/Gamepads/Controllers/poseEnabledController";
import { StickValues, GamepadButtonChanges } from "babylonjs/Gamepads/gamepad";
import { Nullable } from "babylonjs/types";
/**
 * Defines the WebVRController object that represents controllers tracked in 3D space
 * @deprecated Use WebXR instead
 */
export abstract class WebVRController extends PoseEnabledController {
    /**
     * Internal, the default controller model for the controller
     */
    protected _defaultModel: Nullable<AbstractMesh>;
    /**
     * Fired when the trigger state has changed
     */
    onTriggerStateChangedObservable: Observable<ExtendedGamepadButton>;
    /**
     * Fired when the main button state has changed
     */
    onMainButtonStateChangedObservable: Observable<ExtendedGamepadButton>;
    /**
     * Fired when the secondary button state has changed
     */
    onSecondaryButtonStateChangedObservable: Observable<ExtendedGamepadButton>;
    /**
     * Fired when the pad state has changed
     */
    onPadStateChangedObservable: Observable<ExtendedGamepadButton>;
    /**
     * Fired when controllers stick values have changed
     */
    onPadValuesChangedObservable: Observable<StickValues>;
    /**
     * Array of button available on the controller
     */
    protected _buttons: Array<MutableGamepadButton>;
    private _onButtonStateChange;
    /**
     * Fired when a controller button's state has changed
     * @param callback the callback containing the button that was modified
     */
    onButtonStateChange(callback: (controlledIndex: number, buttonIndex: number, state: ExtendedGamepadButton) => void): void;
    /**
     * X and Y axis corresponding to the controllers joystick
     */
    pad: StickValues;
    /**
     * 'left' or 'right', see https://w3c.github.io/gamepad/extensions.html#gamepadhand-enum
     */
    hand: string;
    /**
     * The default controller model for the controller
     */
    get defaultModel(): Nullable<AbstractMesh>;
    /**
     * Creates a new WebVRController from a gamepad
     * @param vrGamepad the gamepad that the WebVRController should be created from
     */
    constructor(vrGamepad: any);
    /**
     * Updates the state of the controller and mesh based on the current position and rotation of the controller
     */
    update(): void;
    /**
     * Function to be called when a button is modified
     */
    protected abstract _handleButtonChange(buttonIdx: number, value: ExtendedGamepadButton, changes: GamepadButtonChanges): void;
    /**
     * Loads a mesh and attaches it to the controller
     * @param scene the scene the mesh should be added to
     * @param meshLoaded callback for when the mesh has been loaded
     */
    abstract initControllerMesh(scene: Scene, meshLoaded?: (mesh: AbstractMesh) => void): void;
    private _setButtonValue;
    private _changes;
    private _checkChanges;
    /**
     * Disposes of th webVRController
     */
    dispose(): void;
}

}
declare module "babylonjs/Gamepads/Controllers/windowsMotionController" {
import { Observable } from "babylonjs/Misc/observable";
import { Scene } from "babylonjs/scene";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { Ray } from "babylonjs/Culling/ray";
import { WebVRController } from "babylonjs/Gamepads/Controllers/webVRController";
import { ExtendedGamepadButton } from "babylonjs/Gamepads/Controllers/poseEnabledController";
import { StickValues } from "babylonjs/Gamepads/gamepad";
/**
 * Defines the WindowsMotionController object that the state of the windows motion controller
 */
export class WindowsMotionController extends WebVRController {
    /**
     * The base url used to load the left and right controller models
     */
    static MODEL_BASE_URL: string;
    /**
     * The name of the left controller model file
     */
    static MODEL_LEFT_FILENAME: string;
    /**
     * The name of the right controller model file
     */
    static MODEL_RIGHT_FILENAME: string;
    /**
     * The controller name prefix for this controller type
     */
    static readonly GAMEPAD_ID_PREFIX: string;
    /**
     * The controller id pattern for this controller type
     */
    private static readonly GAMEPAD_ID_PATTERN;
    private _loadedMeshInfo;
    protected readonly _mapping: {
        buttons: string[];
        buttonMeshNames: {
            trigger: string;
            menu: string;
            grip: string;
            thumbstick: string;
            trackpad: string;
        };
        buttonObservableNames: {
            trigger: string;
            menu: string;
            grip: string;
            thumbstick: string;
            trackpad: string;
        };
        axisMeshNames: string[];
        pointingPoseMeshName: string;
    };
    /**
     * Fired when the trackpad on this controller is clicked
     */
    onTrackpadChangedObservable: Observable<ExtendedGamepadButton>;
    /**
     * Fired when the trackpad on this controller is modified
     */
    onTrackpadValuesChangedObservable: Observable<StickValues>;
    /**
     * The current x and y values of this controller's trackpad
     */
    trackpad: StickValues;
    /**
     * Creates a new WindowsMotionController from a gamepad
     * @param vrGamepad the gamepad that the controller should be created from
     */
    constructor(vrGamepad: any);
    /**
     * Fired when the trigger on this controller is modified
     */
    get onTriggerButtonStateChangedObservable(): Observable<ExtendedGamepadButton>;
    /**
     * Fired when the menu button on this controller is modified
     */
    get onMenuButtonStateChangedObservable(): Observable<ExtendedGamepadButton>;
    /**
     * Fired when the grip button on this controller is modified
     */
    get onGripButtonStateChangedObservable(): Observable<ExtendedGamepadButton>;
    /**
     * Fired when the thumbstick button on this controller is modified
     */
    get onThumbstickButtonStateChangedObservable(): Observable<ExtendedGamepadButton>;
    /**
     * Fired when the touchpad button on this controller is modified
     */
    get onTouchpadButtonStateChangedObservable(): Observable<ExtendedGamepadButton>;
    /**
     * Fired when the touchpad values on this controller are modified
     */
    get onTouchpadValuesChangedObservable(): Observable<StickValues>;
    protected _updateTrackpad(): void;
    /**
     * Called once per frame by the engine.
     */
    update(): void;
    /**
     * Called once for each button that changed state since the last frame
     * @param buttonIdx Which button index changed
     * @param state New state of the button
     */
    protected _handleButtonChange(buttonIdx: number, state: ExtendedGamepadButton): void;
    /**
     * Moves the buttons on the controller mesh based on their current state
     * @param buttonName the name of the button to move
     * @param buttonValue the value of the button which determines the buttons new position
     */
    protected _lerpButtonTransform(buttonName: string, buttonValue: number): void;
    /**
     * Moves the axis on the controller mesh based on its current state
     * @param axis the index of the axis
     * @param axisValue the value of the axis which determines the meshes new position
     * @internal
     */
    protected _lerpAxisTransform(axis: number, axisValue: number): void;
    /**
     * Implements abstract method on WebVRController class, loading controller meshes and calling this.attachToMesh if successful.
     * @param scene scene in which to add meshes
     * @param meshLoaded optional callback function that will be called if the mesh loads successfully.
     * @param forceDefault
     */
    initControllerMesh(scene: Scene, meshLoaded?: (mesh: AbstractMesh) => void, forceDefault?: boolean): void;
    /**
     * Takes a list of meshes (as loaded from the glTF file) and finds the root node, as well as nodes that
     * can be transformed by button presses and axes values, based on this._mapping.
     *
     * @param scene scene in which the meshes exist
     * @param meshes list of meshes that make up the controller model to process
     * @returns structured view of the given meshes, with mapping of buttons and axes to meshes that can be transformed.
     */
    private _processModel;
    private _createMeshInfo;
    /**
     * Gets the ray of the controller in the direction the controller is pointing
     * @param length the length the resulting ray should be
     * @returns a ray in the direction the controller is pointing
     */
    getForwardRay(length?: number): Ray;
    /**
     * Disposes of the controller
     */
    dispose(): void;
}
/**
 * This class represents a new windows motion controller in XR.
 */
export class XRWindowsMotionController extends WindowsMotionController {
    /**
     * Changing the original WIndowsMotionController mapping to fir the new mapping
     */
    protected readonly _mapping: {
        buttons: string[];
        buttonMeshNames: {
            trigger: string;
            menu: string;
            grip: string;
            thumbstick: string;
            trackpad: string;
        };
        buttonObservableNames: {
            trigger: string;
            menu: string;
            grip: string;
            thumbstick: string;
            trackpad: string;
        };
        axisMeshNames: string[];
        pointingPoseMeshName: string;
    };
    /**
     * Construct a new XR-Based windows motion controller
     *
     * @param gamepadInfo the gamepad object from the browser
     */
    constructor(gamepadInfo: any);
    /**
     * holds the thumbstick values (X,Y)
     */
    thumbstickValues: StickValues;
    /**
     * Fired when the thumbstick on this controller is clicked
     */
    onThumbstickStateChangedObservable: Observable<ExtendedGamepadButton>;
    /**
     * Fired when the thumbstick on this controller is modified
     */
    onThumbstickValuesChangedObservable: Observable<StickValues>;
    /**
     * Fired when the touchpad button on this controller is modified
     */
    onTrackpadChangedObservable: Observable<ExtendedGamepadButton>;
    /**
     * Fired when the touchpad values on this controller are modified
     */
    onTrackpadValuesChangedObservable: Observable<StickValues>;
    /**
     * Fired when the thumbstick button on this controller is modified
     * here to prevent breaking changes
     */
    get onThumbstickButtonStateChangedObservable(): Observable<ExtendedGamepadButton>;
    /**
     * updating the thumbstick(!) and not the trackpad.
     * This is named this way due to the difference between WebVR and XR and to avoid
     * changing the parent class.
     */
    protected _updateTrackpad(): void;
    /**
     * Disposes the class with joy
     */
    dispose(): void;
}

}
declare module "babylonjs/Gamepads/dualShockGamepad" {
import { Observable } from "babylonjs/Misc/observable";
import { Gamepad } from "babylonjs/Gamepads/gamepad";
/**
 * Defines supported buttons for DualShock compatible gamepads
 */
export enum DualShockButton {
    /** Cross */
    Cross = 0,
    /** Circle */
    Circle = 1,
    /** Square */
    Square = 2,
    /** Triangle */
    Triangle = 3,
    /** L1 */
    L1 = 4,
    /** R1 */
    R1 = 5,
    /** Share */
    Share = 8,
    /** Options */
    Options = 9,
    /** Left stick */
    LeftStick = 10,
    /** Right stick */
    RightStick = 11
}
/** Defines values for DualShock DPad  */
export enum DualShockDpad {
    /** Up */
    Up = 12,
    /** Down */
    Down = 13,
    /** Left */
    Left = 14,
    /** Right */
    Right = 15
}
/**
 * Defines a DualShock gamepad
 */
export class DualShockPad extends Gamepad {
    private _leftTrigger;
    private _rightTrigger;
    private _onlefttriggerchanged;
    private _onrighttriggerchanged;
    private _onbuttondown;
    private _onbuttonup;
    private _ondpaddown;
    private _ondpadup;
    /** Observable raised when a button is pressed */
    onButtonDownObservable: Observable<DualShockButton>;
    /** Observable raised when a button is released */
    onButtonUpObservable: Observable<DualShockButton>;
    /** Observable raised when a pad is pressed */
    onPadDownObservable: Observable<DualShockDpad>;
    /** Observable raised when a pad is released */
    onPadUpObservable: Observable<DualShockDpad>;
    private _buttonCross;
    private _buttonCircle;
    private _buttonSquare;
    private _buttonTriangle;
    private _buttonShare;
    private _buttonOptions;
    private _buttonL1;
    private _buttonR1;
    private _buttonLeftStick;
    private _buttonRightStick;
    private _dPadUp;
    private _dPadDown;
    private _dPadLeft;
    private _dPadRight;
    /**
     * Creates a new DualShock gamepad object
     * @param id defines the id of this gamepad
     * @param index defines its index
     * @param gamepad defines the internal HTML gamepad object
     */
    constructor(id: string, index: number, gamepad: any);
    /**
     * Defines the callback to call when left trigger is pressed
     * @param callback defines the callback to use
     */
    onlefttriggerchanged(callback: (value: number) => void): void;
    /**
     * Defines the callback to call when right trigger is pressed
     * @param callback defines the callback to use
     */
    onrighttriggerchanged(callback: (value: number) => void): void;
    /**
     * Gets the left trigger value
     */
    get leftTrigger(): number;
    /**
     * Sets the left trigger value
     */
    set leftTrigger(newValue: number);
    /**
     * Gets the right trigger value
     */
    get rightTrigger(): number;
    /**
     * Sets the right trigger value
     */
    set rightTrigger(newValue: number);
    /**
     * Defines the callback to call when a button is pressed
     * @param callback defines the callback to use
     */
    onbuttondown(callback: (buttonPressed: DualShockButton) => void): void;
    /**
     * Defines the callback to call when a button is released
     * @param callback defines the callback to use
     */
    onbuttonup(callback: (buttonReleased: DualShockButton) => void): void;
    /**
     * Defines the callback to call when a pad is pressed
     * @param callback defines the callback to use
     */
    ondpaddown(callback: (dPadPressed: DualShockDpad) => void): void;
    /**
     * Defines the callback to call when a pad is released
     * @param callback defines the callback to use
     */
    ondpadup(callback: (dPadReleased: DualShockDpad) => void): void;
    private _setButtonValue;
    private _setDPadValue;
    /**
     * Gets the value of the `Cross` button
     */
    get buttonCross(): number;
    /**
     * Sets the value of the `Cross` button
     */
    set buttonCross(value: number);
    /**
     * Gets the value of the `Circle` button
     */
    get buttonCircle(): number;
    /**
     * Sets the value of the `Circle` button
     */
    set buttonCircle(value: number);
    /**
     * Gets the value of the `Square` button
     */
    get buttonSquare(): number;
    /**
     * Sets the value of the `Square` button
     */
    set buttonSquare(value: number);
    /**
     * Gets the value of the `Triangle` button
     */
    get buttonTriangle(): number;
    /**
     * Sets the value of the `Triangle` button
     */
    set buttonTriangle(value: number);
    /**
     * Gets the value of the `Options` button
     */
    get buttonOptions(): number;
    /**
     * Sets the value of the `Options` button
     */
    set buttonOptions(value: number);
    /**
     * Gets the value of the `Share` button
     */
    get buttonShare(): number;
    /**
     * Sets the value of the `Share` button
     */
    set buttonShare(value: number);
    /**
     * Gets the value of the `L1` button
     */
    get buttonL1(): number;
    /**
     * Sets the value of the `L1` button
     */
    set buttonL1(value: number);
    /**
     * Gets the value of the `R1` button
     */
    get buttonR1(): number;
    /**
     * Sets the value of the `R1` button
     */
    set buttonR1(value: number);
    /**
     * Gets the value of the Left joystick
     */
    get buttonLeftStick(): number;
    /**
     * Sets the value of the Left joystick
     */
    set buttonLeftStick(value: number);
    /**
     * Gets the value of the Right joystick
     */
    get buttonRightStick(): number;
    /**
     * Sets the value of the Right joystick
     */
    set buttonRightStick(value: number);
    /**
     * Gets the value of D-pad up
     */
    get dPadUp(): number;
    /**
     * Sets the value of D-pad up
     */
    set dPadUp(value: number);
    /**
     * Gets the value of D-pad down
     */
    get dPadDown(): number;
    /**
     * Sets the value of D-pad down
     */
    set dPadDown(value: number);
    /**
     * Gets the value of D-pad left
     */
    get dPadLeft(): number;
    /**
     * Sets the value of D-pad left
     */
    set dPadLeft(value: number);
    /**
     * Gets the value of D-pad right
     */
    get dPadRight(): number;
    /**
     * Sets the value of D-pad right
     */
    set dPadRight(value: number);
    /**
     * Force the gamepad to synchronize with device values
     */
    update(): void;
    /**
     * Disposes the gamepad
     */
    dispose(): void;
}

}
declare module "babylonjs/Gamepads/gamepad" {
import { Observable } from "babylonjs/Misc/observable";
/**
 * Represents a gamepad control stick position
 */
export class StickValues {
    /**
     * The x component of the control stick
     */
    x: number;
    /**
     * The y component of the control stick
     */
    y: number;
    /**
     * Initializes the gamepad x and y control stick values
     * @param x The x component of the gamepad control stick value
     * @param y The y component of the gamepad control stick value
     */
    constructor(
    /**
     * The x component of the control stick
     */
    x: number, 
    /**
     * The y component of the control stick
     */
    y: number);
}
/**
 * An interface which manages callbacks for gamepad button changes
 */
export interface GamepadButtonChanges {
    /**
     * Called when a gamepad has been changed
     */
    changed: boolean;
    /**
     * Called when a gamepad press event has been triggered
     */
    pressChanged: boolean;
    /**
     * Called when a touch event has been triggered
     */
    touchChanged: boolean;
    /**
     * Called when a value has changed
     */
    valueChanged: boolean;
}
/**
 * Represents a gamepad
 */
export class Gamepad {
    /**
     * The id of the gamepad
     */
    id: string;
    /**
     * The index of the gamepad
     */
    index: number;
    /**
     * The browser gamepad
     */
    browserGamepad: any;
    /**
     * Specifies what type of gamepad this represents
     */
    type: number;
    private _leftStick;
    private _rightStick;
    /** @internal */
    _isConnected: boolean;
    private _leftStickAxisX;
    private _leftStickAxisY;
    private _rightStickAxisX;
    private _rightStickAxisY;
    /**
     * Triggered when the left control stick has been changed
     */
    private _onleftstickchanged;
    /**
     * Triggered when the right control stick has been changed
     */
    private _onrightstickchanged;
    /**
     * Represents a gamepad controller
     */
    static GAMEPAD: number;
    /**
     * Represents a generic controller
     */
    static GENERIC: number;
    /**
     * Represents an XBox controller
     */
    static XBOX: number;
    /**
     * Represents a pose-enabled controller
     */
    static POSE_ENABLED: number;
    /**
     * Represents an Dual Shock controller
     */
    static DUALSHOCK: number;
    /**
     * Specifies whether the left control stick should be Y-inverted
     */
    protected _invertLeftStickY: boolean;
    /**
     * Specifies if the gamepad has been connected
     */
    get isConnected(): boolean;
    /**
     * Initializes the gamepad
     * @param id The id of the gamepad
     * @param index The index of the gamepad
     * @param browserGamepad The browser gamepad
     * @param leftStickX The x component of the left joystick
     * @param leftStickY The y component of the left joystick
     * @param rightStickX The x component of the right joystick
     * @param rightStickY The y component of the right joystick
     */
    constructor(
    /**
     * The id of the gamepad
     */
    id: string, 
    /**
     * The index of the gamepad
     */
    index: number, 
    /**
     * The browser gamepad
     */
    browserGamepad: any, leftStickX?: number, leftStickY?: number, rightStickX?: number, rightStickY?: number);
    /**
     * Callback triggered when the left joystick has changed
     * @param callback
     */
    onleftstickchanged(callback: (values: StickValues) => void): void;
    /**
     * Callback triggered when the right joystick has changed
     * @param callback
     */
    onrightstickchanged(callback: (values: StickValues) => void): void;
    /**
     * Gets the left joystick
     */
    get leftStick(): StickValues;
    /**
     * Sets the left joystick values
     */
    set leftStick(newValues: StickValues);
    /**
     * Gets the right joystick
     */
    get rightStick(): StickValues;
    /**
     * Sets the right joystick value
     */
    set rightStick(newValues: StickValues);
    /**
     * Updates the gamepad joystick positions
     */
    update(): void;
    /**
     * Disposes the gamepad
     */
    dispose(): void;
}
/**
 * Represents a generic gamepad
 */
export class GenericPad extends Gamepad {
    private _buttons;
    private _onbuttondown;
    private _onbuttonup;
    /**
     * Observable triggered when a button has been pressed
     */
    onButtonDownObservable: Observable<number>;
    /**
     * Observable triggered when a button has been released
     */
    onButtonUpObservable: Observable<number>;
    /**
     * Callback triggered when a button has been pressed
     * @param callback Called when a button has been pressed
     */
    onbuttondown(callback: (buttonPressed: number) => void): void;
    /**
     * Callback triggered when a button has been released
     * @param callback Called when a button has been released
     */
    onbuttonup(callback: (buttonReleased: number) => void): void;
    /**
     * Initializes the generic gamepad
     * @param id The id of the generic gamepad
     * @param index The index of the generic gamepad
     * @param browserGamepad The browser gamepad
     */
    constructor(id: string, index: number, browserGamepad: any);
    private _setButtonValue;
    /**
     * Updates the generic gamepad
     */
    update(): void;
    /**
     * Disposes the generic gamepad
     */
    dispose(): void;
}

}
declare module "babylonjs/Gamepads/gamepadManager" {
import { Observable } from "babylonjs/Misc/observable";
import { Nullable } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
import { Gamepad } from "babylonjs/Gamepads/gamepad";
/**
 * Manager for handling gamepads
 */
export class GamepadManager {
    private _scene?;
    private _babylonGamepads;
    private _oneGamepadConnected;
    /** @internal */
    _isMonitoring: boolean;
    private _gamepadEventSupported;
    private _gamepadSupport?;
    /**
     * observable to be triggered when the gamepad controller has been connected
     */
    onGamepadConnectedObservable: Observable<Gamepad>;
    /**
     * observable to be triggered when the gamepad controller has been disconnected
     */
    onGamepadDisconnectedObservable: Observable<Gamepad>;
    private _onGamepadConnectedEvent;
    private _onGamepadDisconnectedEvent;
    /**
     * Initializes the gamepad manager
     * @param _scene BabylonJS scene
     */
    constructor(_scene?: Scene | undefined);
    /**
     * The gamepads in the game pad manager
     */
    get gamepads(): Gamepad[];
    /**
     * Get the gamepad controllers based on type
     * @param type The type of gamepad controller
     * @returns Nullable gamepad
     */
    getGamepadByType(type?: number): Nullable<Gamepad>;
    /**
     * Disposes the gamepad manager
     */
    dispose(): void;
    private _addNewGamepad;
    private _startMonitoringGamepads;
    private _stopMonitoringGamepads;
    private _loggedErrors;
    /** @internal */
    _checkGamepadsStatus(): void;
    private _updateGamepadObjects;
}

}
declare module "babylonjs/Gamepads/gamepadSceneComponent" {
import { Nullable } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
import { ISceneComponent } from "babylonjs/sceneComponent";
import { GamepadManager } from "babylonjs/Gamepads/gamepadManager";
module "babylonjs/scene" {
    interface Scene {
        /** @internal */
        _gamepadManager: Nullable<GamepadManager>;
        /**
         * Gets the gamepad manager associated with the scene
         * @see https://doc.babylonjs.com/features/featuresDeepDive/input/gamepads
         */
        gamepadManager: GamepadManager;
    }
}
module "babylonjs/Cameras/freeCameraInputsManager" {
    /**
     * Interface representing a free camera inputs manager
     */
    interface FreeCameraInputsManager {
        /**
         * Adds gamepad input support to the FreeCameraInputsManager.
         * @returns the FreeCameraInputsManager
         */
        addGamepad(): FreeCameraInputsManager;
    }
}
module "babylonjs/Cameras/arcRotateCameraInputsManager" {
    /**
     * Interface representing an arc rotate camera inputs manager
     */
    interface ArcRotateCameraInputsManager {
        /**
         * Adds gamepad input support to the ArcRotateCamera InputManager.
         * @returns the camera inputs manager
         */
        addGamepad(): ArcRotateCameraInputsManager;
    }
}
/**
 * Defines the gamepad scene component responsible to manage gamepads in a given scene
 */
export class GamepadSystemSceneComponent implements ISceneComponent {
    /**
     * The component name helpfull to identify the component in the list of scene components.
     */
    readonly name: string;
    /**
     * The scene the component belongs to.
     */
    scene: Scene;
    /**
     * Creates a new instance of the component for the given scene
     * @param scene Defines the scene to register the component in
     */
    constructor(scene: Scene);
    /**
     * Registers the component in a given scene
     */
    register(): void;
    /**
     * Rebuilds the elements related to this component in case of
     * context lost for instance.
     */
    rebuild(): void;
    /**
     * Disposes the component and the associated resources
     */
    dispose(): void;
    private _beforeCameraUpdate;
}

}
declare module "babylonjs/Gamepads/index" {
export * from "babylonjs/Gamepads/Controllers/index";
export * from "babylonjs/Gamepads/gamepad";
export * from "babylonjs/Gamepads/gamepadManager";
export * from "babylonjs/Gamepads/gamepadSceneComponent";
export * from "babylonjs/Gamepads/xboxGamepad";
export * from "babylonjs/Gamepads/dualShockGamepad";

}
declare module "babylonjs/Gamepads/xboxGamepad" {
import { Observable } from "babylonjs/Misc/observable";
import { Gamepad } from "babylonjs/Gamepads/gamepad";
/**
 * Defines supported buttons for XBox360 compatible gamepads
 */
export enum Xbox360Button {
    /** A */
    A = 0,
    /** B */
    B = 1,
    /** X */
    X = 2,
    /** Y */
    Y = 3,
    /** Left button */
    LB = 4,
    /** Right button */
    RB = 5,
    /** Back */
    Back = 8,
    /** Start */
    Start = 9,
    /** Left stick */
    LeftStick = 10,
    /** Right stick */
    RightStick = 11
}
/** Defines values for XBox360 DPad  */
export enum Xbox360Dpad {
    /** Up */
    Up = 12,
    /** Down */
    Down = 13,
    /** Left */
    Left = 14,
    /** Right */
    Right = 15
}
/**
 * Defines a XBox360 gamepad
 */
export class Xbox360Pad extends Gamepad {
    private _leftTrigger;
    private _rightTrigger;
    private _onlefttriggerchanged;
    private _onrighttriggerchanged;
    private _onbuttondown;
    private _onbuttonup;
    private _ondpaddown;
    private _ondpadup;
    /** Observable raised when a button is pressed */
    onButtonDownObservable: Observable<Xbox360Button>;
    /** Observable raised when a button is released */
    onButtonUpObservable: Observable<Xbox360Button>;
    /** Observable raised when a pad is pressed */
    onPadDownObservable: Observable<Xbox360Dpad>;
    /** Observable raised when a pad is released */
    onPadUpObservable: Observable<Xbox360Dpad>;
    private _buttonA;
    private _buttonB;
    private _buttonX;
    private _buttonY;
    private _buttonBack;
    private _buttonStart;
    private _buttonLB;
    private _buttonRB;
    private _buttonLeftStick;
    private _buttonRightStick;
    private _dPadUp;
    private _dPadDown;
    private _dPadLeft;
    private _dPadRight;
    private _isXboxOnePad;
    /**
     * Creates a new XBox360 gamepad object
     * @param id defines the id of this gamepad
     * @param index defines its index
     * @param gamepad defines the internal HTML gamepad object
     * @param xboxOne defines if it is a XBox One gamepad
     */
    constructor(id: string, index: number, gamepad: any, xboxOne?: boolean);
    /**
     * Defines the callback to call when left trigger is pressed
     * @param callback defines the callback to use
     */
    onlefttriggerchanged(callback: (value: number) => void): void;
    /**
     * Defines the callback to call when right trigger is pressed
     * @param callback defines the callback to use
     */
    onrighttriggerchanged(callback: (value: number) => void): void;
    /**
     * Gets the left trigger value
     */
    get leftTrigger(): number;
    /**
     * Sets the left trigger value
     */
    set leftTrigger(newValue: number);
    /**
     * Gets the right trigger value
     */
    get rightTrigger(): number;
    /**
     * Sets the right trigger value
     */
    set rightTrigger(newValue: number);
    /**
     * Defines the callback to call when a button is pressed
     * @param callback defines the callback to use
     */
    onbuttondown(callback: (buttonPressed: Xbox360Button) => void): void;
    /**
     * Defines the callback to call when a button is released
     * @param callback defines the callback to use
     */
    onbuttonup(callback: (buttonReleased: Xbox360Button) => void): void;
    /**
     * Defines the callback to call when a pad is pressed
     * @param callback defines the callback to use
     */
    ondpaddown(callback: (dPadPressed: Xbox360Dpad) => void): void;
    /**
     * Defines the callback to call when a pad is released
     * @param callback defines the callback to use
     */
    ondpadup(callback: (dPadReleased: Xbox360Dpad) => void): void;
    private _setButtonValue;
    private _setDPadValue;
    /**
     * Gets the value of the `A` button
     */
    get buttonA(): number;
    /**
     * Sets the value of the `A` button
     */
    set buttonA(value: number);
    /**
     * Gets the value of the `B` button
     */
    get buttonB(): number;
    /**
     * Sets the value of the `B` button
     */
    set buttonB(value: number);
    /**
     * Gets the value of the `X` button
     */
    get buttonX(): number;
    /**
     * Sets the value of the `X` button
     */
    set buttonX(value: number);
    /**
     * Gets the value of the `Y` button
     */
    get buttonY(): number;
    /**
     * Sets the value of the `Y` button
     */
    set buttonY(value: number);
    /**
     * Gets the value of the `Start` button
     */
    get buttonStart(): number;
    /**
     * Sets the value of the `Start` button
     */
    set buttonStart(value: number);
    /**
     * Gets the value of the `Back` button
     */
    get buttonBack(): number;
    /**
     * Sets the value of the `Back` button
     */
    set buttonBack(value: number);
    /**
     * Gets the value of the `Left` button
     */
    get buttonLB(): number;
    /**
     * Sets the value of the `Left` button
     */
    set buttonLB(value: number);
    /**
     * Gets the value of the `Right` button
     */
    get buttonRB(): number;
    /**
     * Sets the value of the `Right` button
     */
    set buttonRB(value: number);
    /**
     * Gets the value of the Left joystick
     */
    get buttonLeftStick(): number;
    /**
     * Sets the value of the Left joystick
     */
    set buttonLeftStick(value: number);
    /**
     * Gets the value of the Right joystick
     */
    get buttonRightStick(): number;
    /**
     * Sets the value of the Right joystick
     */
    set buttonRightStick(value: number);
    /**
     * Gets the value of D-pad up
     */
    get dPadUp(): number;
    /**
     * Sets the value of D-pad up
     */
    set dPadUp(value: number);
    /**
     * Gets the value of D-pad down
     */
    get dPadDown(): number;
    /**
     * Sets the value of D-pad down
     */
    set dPadDown(value: number);
    /**
     * Gets the value of D-pad left
     */
    get dPadLeft(): number;
    /**
     * Sets the value of D-pad left
     */
    set dPadLeft(value: number);
    /**
     * Gets the value of D-pad right
     */
    get dPadRight(): number;
    /**
     * Sets the value of D-pad right
     */
    set dPadRight(value: number);
    /**
     * Force the gamepad to synchronize with device values
     */
    update(): void;
    /**
     * Disposes the gamepad
     */
    dispose(): void;
}

}
declare module "babylonjs/Gizmos/axisDragGizmo" {
import { Observer } from "babylonjs/Misc/observable";
import { Observable } from "babylonjs/Misc/observable";
import { Nullable } from "babylonjs/types";
import { PointerInfo } from "babylonjs/Events/pointerEvents";
import { Vector3 } from "babylonjs/Maths/math.vector";
import { TransformNode } from "babylonjs/Meshes/transformNode";
import { Node } from "babylonjs/node";
import { Mesh } from "babylonjs/Meshes/mesh";
import { PointerDragBehavior } from "babylonjs/Behaviors/Meshes/pointerDragBehavior";
import { IGizmo } from "babylonjs/Gizmos/gizmo";
import { Gizmo } from "babylonjs/Gizmos/gizmo";
import { UtilityLayerRenderer } from "babylonjs/Rendering/utilityLayerRenderer";
import { StandardMaterial } from "babylonjs/Materials/standardMaterial";
import { Scene } from "babylonjs/scene";
import { PositionGizmo } from "babylonjs/Gizmos/positionGizmo";
import { Color3 } from "babylonjs/Maths/math.color";
/**
 * Interface for axis drag gizmo
 */
export interface IAxisDragGizmo extends IGizmo {
    /** Drag behavior responsible for the gizmos dragging interactions */
    dragBehavior: PointerDragBehavior;
    /** Drag distance in babylon units that the gizmo will snap to when dragged */
    snapDistance: number;
    /**
     * Event that fires each time the gizmo snaps to a new location.
     * * snapDistance is the the change in distance
     */
    onSnapObservable: Observable<{
        snapDistance: number;
    }>;
    /** If the gizmo is enabled */
    isEnabled: boolean;
    /** Default material used to render when gizmo is not disabled or hovered */
    coloredMaterial: StandardMaterial;
    /** Material used to render when gizmo is hovered with mouse*/
    hoverMaterial: StandardMaterial;
    /** Material used to render when gizmo is disabled. typically grey.*/
    disableMaterial: StandardMaterial;
}
/**
 * Single axis drag gizmo
 */
export class AxisDragGizmo extends Gizmo implements IAxisDragGizmo {
    /**
     * Drag behavior responsible for the gizmos dragging interactions
     */
    dragBehavior: PointerDragBehavior;
    protected _pointerObserver: Nullable<Observer<PointerInfo>>;
    /**
     * Drag distance in babylon units that the gizmo will snap to when dragged (Default: 0)
     */
    snapDistance: number;
    /**
     * Event that fires each time the gizmo snaps to a new location.
     * * snapDistance is the the change in distance
     */
    onSnapObservable: Observable<{
        snapDistance: number;
    }>;
    protected _isEnabled: boolean;
    protected _parent: Nullable<PositionGizmo>;
    protected _gizmoMesh: Mesh;
    protected _coloredMaterial: StandardMaterial;
    protected _hoverMaterial: StandardMaterial;
    protected _disableMaterial: StandardMaterial;
    protected _dragging: boolean;
    /** Default material used to render when gizmo is not disabled or hovered */
    get coloredMaterial(): StandardMaterial;
    /** Material used to render when gizmo is hovered with mouse*/
    get hoverMaterial(): StandardMaterial;
    /** Material used to render when gizmo is disabled. typically grey.*/
    get disableMaterial(): StandardMaterial;
    /**
     * @internal
     */
    static _CreateArrow(scene: Scene, material: StandardMaterial, thickness?: number, isCollider?: boolean): TransformNode;
    /**
     * @internal
     */
    static _CreateArrowInstance(scene: Scene, arrow: TransformNode): TransformNode;
    /**
     * Creates an AxisDragGizmo
     * @param dragAxis The axis which the gizmo will be able to drag on
     * @param color The color of the gizmo
     * @param gizmoLayer The utility layer the gizmo will be added to
     * @param parent
     * @param thickness display gizmo axis thickness
     */
    constructor(dragAxis: Vector3, color?: Color3, gizmoLayer?: UtilityLayerRenderer, parent?: Nullable<PositionGizmo>, thickness?: number);
    protected _attachedNodeChanged(value: Nullable<Node>): void;
    /**
     * If the gizmo is enabled
     */
    set isEnabled(value: boolean);
    get isEnabled(): boolean;
    /**
     * Disposes of the gizmo
     */
    dispose(): void;
}

}
declare module "babylonjs/Gizmos/axisScaleGizmo" {
import { Observer } from "babylonjs/Misc/observable";
import { Observable } from "babylonjs/Misc/observable";
import { Nullable } from "babylonjs/types";
import { PointerInfo } from "babylonjs/Events/pointerEvents";
import { Vector3 } from "babylonjs/Maths/math.vector";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { Node } from "babylonjs/node";
import { Mesh } from "babylonjs/Meshes/mesh";
import { StandardMaterial } from "babylonjs/Materials/standardMaterial";
import { PointerDragBehavior } from "babylonjs/Behaviors/Meshes/pointerDragBehavior";
import { IGizmo } from "babylonjs/Gizmos/gizmo";
import { Gizmo } from "babylonjs/Gizmos/gizmo";
import { UtilityLayerRenderer } from "babylonjs/Rendering/utilityLayerRenderer";
import { ScaleGizmo } from "babylonjs/Gizmos/scaleGizmo";
import { Color3 } from "babylonjs/Maths/math.color";
/**
 * Interface for axis scale gizmo
 */
export interface IAxisScaleGizmo extends IGizmo {
    /** Drag behavior responsible for the gizmos dragging interactions */
    dragBehavior: PointerDragBehavior;
    /** Drag distance in babylon units that the gizmo will snap to when dragged */
    snapDistance: number;
    /**
     * Event that fires each time the gizmo snaps to a new location.
     * * snapDistance is the the change in distance
     */
    onSnapObservable: Observable<{
        snapDistance: number;
    }>;
    /** If the scaling operation should be done on all axis */
    uniformScaling: boolean;
    /** Custom sensitivity value for the drag strength */
    sensitivity: number;
    /** The magnitude of the drag strength (scaling factor) */
    dragScale: number;
    /** If the gizmo is enabled */
    isEnabled: boolean;
    /** Default material used to render when gizmo is not disabled or hovered */
    coloredMaterial: StandardMaterial;
    /** Material used to render when gizmo is hovered with mouse*/
    hoverMaterial: StandardMaterial;
    /** Material used to render when gizmo is disabled. typically grey.*/
    disableMaterial: StandardMaterial;
}
/**
 * Single axis scale gizmo
 */
export class AxisScaleGizmo extends Gizmo implements IAxisScaleGizmo {
    /**
     * Drag behavior responsible for the gizmos dragging interactions
     */
    dragBehavior: PointerDragBehavior;
    protected _pointerObserver: Nullable<Observer<PointerInfo>>;
    /**
     * Scale distance in babylon units that the gizmo will snap to when dragged (Default: 0)
     */
    snapDistance: number;
    /**
     * Event that fires each time the gizmo snaps to a new location.
     * * snapDistance is the the change in distance
     */
    onSnapObservable: Observable<{
        snapDistance: number;
    }>;
    /**
     * If the scaling operation should be done on all axis (default: false)
     */
    uniformScaling: boolean;
    /**
     * Custom sensitivity value for the drag strength
     */
    sensitivity: number;
    /**
     * The magnitude of the drag strength (scaling factor)
     */
    dragScale: number;
    protected _isEnabled: boolean;
    protected _parent: Nullable<ScaleGizmo>;
    protected _gizmoMesh: Mesh;
    protected _coloredMaterial: StandardMaterial;
    protected _hoverMaterial: StandardMaterial;
    protected _disableMaterial: StandardMaterial;
    protected _dragging: boolean;
    private _tmpVector;
    /** Default material used to render when gizmo is not disabled or hovered */
    get coloredMaterial(): StandardMaterial;
    /** Material used to render when gizmo is hovered with mouse*/
    get hoverMaterial(): StandardMaterial;
    /** Material used to render when gizmo is disabled. typically grey.*/
    get disableMaterial(): StandardMaterial;
    /**
     * Creates an AxisScaleGizmo
     * @param dragAxis The axis which the gizmo will be able to scale on
     * @param color The color of the gizmo
     * @param gizmoLayer The utility layer the gizmo will be added to
     * @param parent
     * @param thickness display gizmo axis thickness
     */
    constructor(dragAxis: Vector3, color?: Color3, gizmoLayer?: UtilityLayerRenderer, parent?: Nullable<ScaleGizmo>, thickness?: number);
    /**
     * Create Geometry for Gizmo
     * @param parentMesh
     * @param thickness
     * @param isCollider
     */
    protected _createGizmoMesh(parentMesh: AbstractMesh, thickness: number, isCollider?: boolean): {
        arrowMesh: Mesh;
        arrowTail: Mesh;
    };
    protected _attachedNodeChanged(value: Nullable<Node>): void;
    /**
     * If the gizmo is enabled
     */
    set isEnabled(value: boolean);
    get isEnabled(): boolean;
    /**
     * Disposes of the gizmo
     */
    dispose(): void;
    /**
     * Disposes and replaces the current meshes in the gizmo with the specified mesh
     * @param mesh The mesh to replace the default mesh of the gizmo
     * @param useGizmoMaterial If the gizmo's default material should be used (default: false)
     */
    setCustomMesh(mesh: Mesh, useGizmoMaterial?: boolean): void;
}

}
declare module "babylonjs/Gizmos/boundingBoxGizmo" {
import { Observer } from "babylonjs/Misc/observable";
import { Observable } from "babylonjs/Misc/observable";
import { Nullable } from "babylonjs/types";
import { PointerInfo } from "babylonjs/Events/pointerEvents";
import { Scene } from "babylonjs/scene";
import { Vector3 } from "babylonjs/Maths/math.vector";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { Mesh } from "babylonjs/Meshes/mesh";
import { PointerDragBehavior } from "babylonjs/Behaviors/Meshes/pointerDragBehavior";
import { IGizmo } from "babylonjs/Gizmos/gizmo";
import { Gizmo } from "babylonjs/Gizmos/gizmo";
import { UtilityLayerRenderer } from "babylonjs/Rendering/utilityLayerRenderer";
import { StandardMaterial } from "babylonjs/Materials/standardMaterial";
import { Color3 } from "babylonjs/Maths/math.color";
/**
 * Interface for bounding box gizmo
 */
export interface IBoundingBoxGizmo extends IGizmo {
    /**
     * If child meshes should be ignored when calculating the bounding box. This should be set to true to avoid perf hits with heavily nested meshes.
     */
    ignoreChildren: boolean;
    /**
     * Returns true if a descendant should be included when computing the bounding box. When null, all descendants are included. If ignoreChildren is set this will be ignored.
     */
    includeChildPredicate: Nullable<(abstractMesh: AbstractMesh) => boolean>;
    /** The size of the rotation spheres attached to the bounding box */
    rotationSphereSize: number;
    /** The size of the scale boxes attached to the bounding box */
    scaleBoxSize: number;
    /**
     * If set, the rotation spheres and scale boxes will increase in size based on the distance away from the camera to have a consistent screen size
     * Note : fixedDragMeshScreenSize takes precedence over fixedDragMeshBoundsSize if both are true
     */
    fixedDragMeshScreenSize: boolean;
    /**
     * If set, the rotation spheres and scale boxes will increase in size based on the size of the bounding box
     * Note : fixedDragMeshScreenSize takes precedence over fixedDragMeshBoundsSize if both are true
     */
    fixedDragMeshBoundsSize: boolean;
    /**
     * The distance away from the object which the draggable meshes should appear world sized when fixedDragMeshScreenSize is set to true
     */
    fixedDragMeshScreenSizeDistanceFactor: number;
    /** Fired when a rotation sphere or scale box is dragged */
    onDragStartObservable: Observable<{}>;
    /** Fired when a scale box is dragged */
    onScaleBoxDragObservable: Observable<{}>;
    /** Fired when a scale box drag is ended */
    onScaleBoxDragEndObservable: Observable<{}>;
    /** Fired when a rotation sphere is dragged */
    onRotationSphereDragObservable: Observable<{}>;
    /** Fired when a rotation sphere drag is ended */
    onRotationSphereDragEndObservable: Observable<{}>;
    /** Relative bounding box pivot used when scaling the attached node. */
    scalePivot: Nullable<Vector3>;
    /** Scale factor vector used for masking some axis */
    axisFactor: Vector3;
    /** Scale factor scalar affecting all axes' drag speed */
    scaleDragSpeed: number;
    /**
     * Sets the color of the bounding box gizmo
     * @param color the color to set
     */
    setColor(color: Color3): void;
    /** Returns an array containing all boxes used for scaling (in increasing x, y and z orders) */
    getScaleBoxes(): AbstractMesh[];
    /** Updates the bounding box information for the Gizmo */
    updateBoundingBox(): void;
    /**
     * Enables rotation on the specified axis and disables rotation on the others
     * @param axis The list of axis that should be enabled (eg. "xy" or "xyz")
     */
    setEnabledRotationAxis(axis: string): void;
    /**
     * Enables/disables scaling
     * @param enable if scaling should be enabled
     * @param homogeneousScaling defines if scaling should only be homogeneous
     */
    setEnabledScaling(enable: boolean, homogeneousScaling?: boolean): void;
    /** Enables a pointer drag behavior on the bounding box of the gizmo */
    enableDragBehavior(): void;
    /** Default material used to render when gizmo is not disabled or hovered */
    coloredMaterial: StandardMaterial;
    /** Material used to render when gizmo is hovered with mouse*/
    hoverMaterial: StandardMaterial;
}
/**
 * Bounding box gizmo
 */
export class BoundingBoxGizmo extends Gizmo implements IBoundingBoxGizmo {
    protected _lineBoundingBox: AbstractMesh;
    protected _rotateSpheresParent: AbstractMesh;
    protected _scaleBoxesParent: AbstractMesh;
    protected _boundingDimensions: Vector3;
    protected _renderObserver: Nullable<Observer<Scene>>;
    protected _pointerObserver: Nullable<Observer<PointerInfo>>;
    protected _scaleDragSpeed: number;
    private _tmpQuaternion;
    private _tmpVector;
    private _tmpRotationMatrix;
    /**
     * If child meshes should be ignored when calculating the bounding box. This should be set to true to avoid perf hits with heavily nested meshes (Default: false)
     */
    ignoreChildren: boolean;
    /**
     * Returns true if a descendant should be included when computing the bounding box. When null, all descendants are included. If ignoreChildren is set this will be ignored. (Default: null)
     */
    includeChildPredicate: Nullable<(abstractMesh: AbstractMesh) => boolean>;
    /**
     * The size of the rotation spheres attached to the bounding box (Default: 0.1)
     */
    rotationSphereSize: number;
    /**
     * The size of the scale boxes attached to the bounding box (Default: 0.1)
     */
    scaleBoxSize: number;
    /**
     * If set, the rotation spheres and scale boxes will increase in size based on the distance away from the camera to have a consistent screen size (Default: false)
     * Note : fixedDragMeshScreenSize takes precedence over fixedDragMeshBoundsSize if both are true
     */
    fixedDragMeshScreenSize: boolean;
    /**
     * If set, the rotation spheres and scale boxes will increase in size based on the size of the bounding box
     * Note : fixedDragMeshScreenSize takes precedence over fixedDragMeshBoundsSize if both are true
     */
    fixedDragMeshBoundsSize: boolean;
    /**
     * The distance away from the object which the draggable meshes should appear world sized when fixedDragMeshScreenSize is set to true (default: 10)
     */
    fixedDragMeshScreenSizeDistanceFactor: number;
    /**
     * Fired when a rotation sphere or scale box is dragged
     */
    onDragStartObservable: Observable<{}>;
    /**
     * Fired when a scale box is dragged
     */
    onScaleBoxDragObservable: Observable<{}>;
    /**
     * Fired when a scale box drag is ended
     */
    onScaleBoxDragEndObservable: Observable<{}>;
    /**
     * Fired when a rotation sphere is dragged
     */
    onRotationSphereDragObservable: Observable<{}>;
    /**
     * Fired when a rotation sphere drag is ended
     */
    onRotationSphereDragEndObservable: Observable<{}>;
    /**
     * Relative bounding box pivot used when scaling the attached node. When null object with scale from the opposite corner. 0.5,0.5,0.5 for center and 0.5,0,0.5 for bottom (Default: null)
     */
    scalePivot: Nullable<Vector3>;
    /**
     * Scale factor used for masking some axis
     */
    protected _axisFactor: Vector3;
    /**
     * Sets the axis factor
     * @param factor the Vector3 value
     */
    set axisFactor(factor: Vector3);
    /**
     * Gets the axis factor
     * @returns the Vector3 factor value
     */
    get axisFactor(): Vector3;
    /**
     * Sets scale drag speed value
     * @param value the new speed value
     */
    set scaleDragSpeed(value: number);
    /**
     * Gets scale drag speed
     * @returns the scale speed number
     */
    get scaleDragSpeed(): number;
    /**
     * Mesh used as a pivot to rotate the attached node
     */
    protected _anchorMesh: AbstractMesh;
    protected _existingMeshScale: Vector3;
    protected _dragMesh: Nullable<Mesh>;
    protected _pointerDragBehavior: PointerDragBehavior;
    protected _coloredMaterial: StandardMaterial;
    protected _hoverColoredMaterial: StandardMaterial;
    /** Default material used to render when gizmo is not disabled or hovered */
    get coloredMaterial(): StandardMaterial;
    /** Material used to render when gizmo is hovered with mouse*/
    get hoverMaterial(): StandardMaterial;
    /**
     * Get the pointerDragBehavior
     */
    get pointerDragBehavior(): PointerDragBehavior;
    /**
     * Sets the color of the bounding box gizmo
     * @param color the color to set
     */
    setColor(color: Color3): void;
    /**
     * Creates an BoundingBoxGizmo
     * @param color The color of the gizmo
     * @param gizmoLayer The utility layer the gizmo will be added to
     */
    constructor(color?: Color3, gizmoLayer?: UtilityLayerRenderer);
    protected _attachedNodeChanged(value: Nullable<AbstractMesh>): void;
    protected _selectNode(selectedMesh: Nullable<Mesh>): void;
    protected _unhoverMeshOnTouchUp(pointerInfo: Nullable<PointerInfo>, selectedMesh: AbstractMesh): void;
    /**
     * returns an array containing all boxes used for scaling (in increasing x, y and z orders)
     */
    getScaleBoxes(): AbstractMesh[];
    /**
     * Updates the bounding box information for the Gizmo
     */
    updateBoundingBox(): void;
    protected _updateRotationSpheres(): void;
    protected _updateScaleBoxes(): void;
    /**
     * Enables rotation on the specified axis and disables rotation on the others
     * @param axis The list of axis that should be enabled (eg. "xy" or "xyz")
     */
    setEnabledRotationAxis(axis: string): void;
    /**
     * Enables/disables scaling
     * @param enable if scaling should be enabled
     * @param homogeneousScaling defines if scaling should only be homogeneous
     */
    setEnabledScaling(enable: boolean, homogeneousScaling?: boolean): void;
    protected _updateDummy(): void;
    /**
     * Enables a pointer drag behavior on the bounding box of the gizmo
     */
    enableDragBehavior(): void;
    /**
     * Disposes of the gizmo
     */
    dispose(): void;
    /**
     * Makes a mesh not pickable and wraps the mesh inside of a bounding box mesh that is pickable. (This is useful to avoid picking within complex geometry)
     * @param mesh the mesh to wrap in the bounding box mesh and make not pickable
     * @returns the bounding box mesh with the passed in mesh as a child
     */
    static MakeNotPickableAndWrapInBoundingBox(mesh: Mesh): Mesh;
    /**
     * CustomMeshes are not supported by this gizmo
     */
    setCustomMesh(): void;
}

}
declare module "babylonjs/Gizmos/cameraGizmo" {
import { Nullable } from "babylonjs/types";
import { Mesh } from "babylonjs/Meshes/mesh";
import { IGizmo } from "babylonjs/Gizmos/gizmo";
import { Gizmo } from "babylonjs/Gizmos/gizmo";
import { UtilityLayerRenderer } from "babylonjs/Rendering/utilityLayerRenderer";
import { StandardMaterial } from "babylonjs/Materials/standardMaterial";
import { Camera } from "babylonjs/Cameras/camera";
import { PointerInfo } from "babylonjs/Events/pointerEvents";
import { Observer } from "babylonjs/Misc/observable";
import { Observable } from "babylonjs/Misc/observable";
/**
 * Interface for camera gizmo
 */
export interface ICameraGizmo extends IGizmo {
    /** Event that fires each time the gizmo is clicked */
    onClickedObservable: Observable<Camera>;
    /** A boolean indicating if frustum lines must be rendered */
    displayFrustum: boolean;
    /** The camera that the gizmo is attached to */
    camera: Nullable<Camera>;
    /** The material used to render the camera gizmo */
    readonly material: StandardMaterial;
}
/**
 * Gizmo that enables viewing a camera
 */
export class CameraGizmo extends Gizmo implements ICameraGizmo {
    protected _cameraMesh: Mesh;
    protected _cameraLinesMesh: Mesh;
    protected _material: StandardMaterial;
    protected _pointerObserver: Nullable<Observer<PointerInfo>>;
    /**
     * Event that fires each time the gizmo is clicked
     */
    onClickedObservable: Observable<Camera>;
    /**
     * Creates a CameraGizmo
     * @param gizmoLayer The utility layer the gizmo will be added to
     */
    constructor(gizmoLayer?: UtilityLayerRenderer);
    protected _camera: Nullable<Camera>;
    /** Gets or sets a boolean indicating if frustum lines must be rendered (true by default)) */
    get displayFrustum(): boolean;
    set displayFrustum(value: boolean);
    /**
     * The camera that the gizmo is attached to
     */
    set camera(camera: Nullable<Camera>);
    get camera(): Nullable<Camera>;
    /**
     * Gets the material used to render the camera gizmo
     */
    get material(): StandardMaterial;
    /**
     * @internal
     * Updates the gizmo to match the attached mesh's position/rotation
     */
    protected _update(): void;
    private static _Scale;
    private _invProjection;
    /**
     * Disposes of the camera gizmo
     */
    dispose(): void;
    private static _CreateCameraMesh;
    private static _CreateCameraFrustum;
}

}
declare module "babylonjs/Gizmos/gizmo" {
import { Observer } from "babylonjs/Misc/observable";
import { Nullable } from "babylonjs/types";
import { Scene, IDisposable } from "babylonjs/scene";
import { Quaternion } from "babylonjs/Maths/math.vector";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { Mesh } from "babylonjs/Meshes/mesh";
import { Node } from "babylonjs/node";
import { UtilityLayerRenderer } from "babylonjs/Rendering/utilityLayerRenderer";
import { StandardMaterial } from "babylonjs/Materials/standardMaterial";
import { PointerInfo } from "babylonjs/Events/pointerEvents";
import { PointerDragBehavior } from "babylonjs/Behaviors/Meshes/pointerDragBehavior";
/**
 * Cache built by each axis. Used for managing state between all elements of gizmo for enhanced UI
 */
export interface GizmoAxisCache {
    /** Mesh used to render the Gizmo */
    gizmoMeshes: Mesh[];
    /** Mesh used to detect user interaction with Gizmo */
    colliderMeshes: Mesh[];
    /** Material used to indicate color of gizmo mesh */
    material: StandardMaterial;
    /** Material used to indicate hover state of the Gizmo */
    hoverMaterial: StandardMaterial;
    /** Material used to indicate disabled state of the Gizmo */
    disableMaterial: StandardMaterial;
    /** Used to indicate Active state of the Gizmo */
    active: boolean;
    /** DragBehavior */
    dragBehavior: PointerDragBehavior;
}
/**
 * Interface for basic gizmo
 */
export interface IGizmo extends IDisposable {
    /** True when the mouse pointer is hovered a gizmo mesh */
    readonly isHovered: boolean;
    /** The root mesh of the gizmo */
    _rootMesh: Mesh;
    /** Ratio for the scale of the gizmo */
    scaleRatio: number;
    /**
     * Mesh that the gizmo will be attached to. (eg. on a drag gizmo the mesh that will be dragged)
     * * When set, interactions will be enabled
     */
    attachedMesh: Nullable<AbstractMesh>;
    /**
     * Node that the gizmo will be attached to. (eg. on a drag gizmo the mesh, bone or NodeTransform that will be dragged)
     * * When set, interactions will be enabled
     */
    attachedNode: Nullable<Node>;
    /**
     * If set the gizmo's rotation will be updated to match the attached mesh each frame (Default: true)
     */
    updateGizmoRotationToMatchAttachedMesh: boolean;
    /** The utility layer the gizmo will be added to */
    gizmoLayer: UtilityLayerRenderer;
    /**
     * If set the gizmo's position will be updated to match the attached mesh each frame (Default: true)
     */
    updateGizmoPositionToMatchAttachedMesh: boolean;
    /**
     * When set, the gizmo will always appear the same size no matter where the camera is (default: true)
     */
    updateScale: boolean;
    /**
     * posture that the gizmo will be display
     * When set null, default value will be used (Quaternion(0, 0, 0, 1))
     */
    customRotationQuaternion: Nullable<Quaternion>;
    /** Disposes and replaces the current meshes in the gizmo with the specified mesh */
    setCustomMesh(mesh: Mesh): void;
}
/**
 * Renders gizmos on top of an existing scene which provide controls for position, rotation, etc.
 */
export class Gizmo implements IGizmo {
    /** The utility layer the gizmo will be added to */
    gizmoLayer: UtilityLayerRenderer;
    /**
     * The root mesh of the gizmo
     */
    _rootMesh: Mesh;
    protected _attachedMesh: Nullable<AbstractMesh>;
    protected _attachedNode: Nullable<Node>;
    protected _customRotationQuaternion: Nullable<Quaternion>;
    /**
     * Ratio for the scale of the gizmo (Default: 1)
     */
    protected _scaleRatio: number;
    /**
     * boolean updated by pointermove when a gizmo mesh is hovered
     */
    protected _isHovered: boolean;
    /**
     * When enabled, any gizmo operation will perserve scaling sign. Default is off.
     * Only valid for TransformNode derived classes (Mesh, AbstractMesh, ...)
     */
    static PreserveScaling: boolean;
    /**
     * Ratio for the scale of the gizmo (Default: 1)
     */
    set scaleRatio(value: number);
    get scaleRatio(): number;
    /**
     * True when the mouse pointer is hovered a gizmo mesh
     */
    get isHovered(): boolean;
    /**
     * If a custom mesh has been set (Default: false)
     */
    protected _customMeshSet: boolean;
    /**
     * Mesh that the gizmo will be attached to. (eg. on a drag gizmo the mesh that will be dragged)
     * * When set, interactions will be enabled
     */
    get attachedMesh(): Nullable<AbstractMesh>;
    set attachedMesh(value: Nullable<AbstractMesh>);
    /**
     * Node that the gizmo will be attached to. (eg. on a drag gizmo the mesh, bone or NodeTransform that will be dragged)
     * * When set, interactions will be enabled
     */
    get attachedNode(): Nullable<Node>;
    set attachedNode(value: Nullable<Node>);
    /**
     * Disposes and replaces the current meshes in the gizmo with the specified mesh
     * @param mesh The mesh to replace the default mesh of the gizmo
     */
    setCustomMesh(mesh: Mesh): void;
    protected _updateGizmoRotationToMatchAttachedMesh: boolean;
    protected _updateGizmoPositionToMatchAttachedMesh: boolean;
    protected _updateScale: boolean;
    /**
     * If set the gizmo's rotation will be updated to match the attached mesh each frame (Default: true)
     * NOTE: This is only possible for meshes with uniform scaling, as otherwise it's not possible to decompose the rotation
     */
    set updateGizmoRotationToMatchAttachedMesh(value: boolean);
    get updateGizmoRotationToMatchAttachedMesh(): boolean;
    /**
     * If set the gizmo's position will be updated to match the attached mesh each frame (Default: true)
     */
    set updateGizmoPositionToMatchAttachedMesh(value: boolean);
    get updateGizmoPositionToMatchAttachedMesh(): boolean;
    /**
     * When set, the gizmo will always appear the same size no matter where the camera is (default: true)
     */
    set updateScale(value: boolean);
    get updateScale(): boolean;
    protected _interactionsEnabled: boolean;
    protected _attachedNodeChanged(value: Nullable<Node>): void;
    protected _beforeRenderObserver: Nullable<Observer<Scene>>;
    private _rightHandtoLeftHandMatrix;
    /**
     * Creates a gizmo
     * @param gizmoLayer The utility layer the gizmo will be added to
     */
    constructor(
    /** The utility layer the gizmo will be added to */
    gizmoLayer?: UtilityLayerRenderer);
    /**
     * posture that the gizmo will be display
     * When set null, default value will be used (Quaternion(0, 0, 0, 1))
     */
    get customRotationQuaternion(): Nullable<Quaternion>;
    set customRotationQuaternion(customRotationQuaternion: Nullable<Quaternion>);
    /**
     * Updates the gizmo to match the attached mesh's position/rotation
     */
    protected _update(): void;
    /**
     * Handle position/translation when using an attached node using pivot
     */
    protected _handlePivot(): void;
    /**
     * computes the rotation/scaling/position of the transform once the Node world matrix has changed.
     */
    protected _matrixChanged(): void;
    /**
     * refresh gizmo mesh material
     * @param gizmoMeshes
     * @param material material to apply
     */
    protected _setGizmoMeshMaterial(gizmoMeshes: Mesh[], material: StandardMaterial): void;
    /**
     * Subscribes to pointer up, down, and hover events. Used for responsive gizmos.
     * @param gizmoLayer The utility layer the gizmo will be added to
     * @param gizmoAxisCache Gizmo axis definition used for reactive gizmo UI
     * @returns {Observer<PointerInfo>} pointerObserver
     */
    static GizmoAxisPointerObserver(gizmoLayer: UtilityLayerRenderer, gizmoAxisCache: Map<Mesh, GizmoAxisCache>): Observer<PointerInfo>;
    /**
     * Disposes of the gizmo
     */
    dispose(): void;
}

}
declare module "babylonjs/Gizmos/gizmoManager" {
import { Observer } from "babylonjs/Misc/observable";
import { Observable } from "babylonjs/Misc/observable";
import { Nullable } from "babylonjs/types";
import { PointerInfo } from "babylonjs/Events/pointerEvents";
import { Scene, IDisposable } from "babylonjs/scene";
import { Node } from "babylonjs/node";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { Mesh } from "babylonjs/Meshes/mesh";
import { UtilityLayerRenderer } from "babylonjs/Rendering/utilityLayerRenderer";
import { Color3 } from "babylonjs/Maths/math.color";
import { SixDofDragBehavior } from "babylonjs/Behaviors/Meshes/sixDofDragBehavior";
import { GizmoAxisCache } from "babylonjs/Gizmos/gizmo";
import { IRotationGizmo } from "babylonjs/Gizmos/rotationGizmo";
import { IPositionGizmo } from "babylonjs/Gizmos/positionGizmo";
import { IScaleGizmo } from "babylonjs/Gizmos/scaleGizmo";
import { IBoundingBoxGizmo } from "babylonjs/Gizmos/boundingBoxGizmo";
/**
 * Helps setup gizmo's in the scene to rotate/scale/position nodes
 */
export class GizmoManager implements IDisposable {
    private _scene;
    /**
     * Gizmo's created by the gizmo manager, gizmo will be null until gizmo has been enabled for the first time
     */
    gizmos: {
        positionGizmo: Nullable<IPositionGizmo>;
        rotationGizmo: Nullable<IRotationGizmo>;
        scaleGizmo: Nullable<IScaleGizmo>;
        boundingBoxGizmo: Nullable<IBoundingBoxGizmo>;
    };
    /** When true, the gizmo will be detached from the current object when a pointer down occurs with an empty picked mesh */
    clearGizmoOnEmptyPointerEvent: boolean;
    /** When true (default), picking to attach a new mesh is enabled. This works in sync with inspector autopicking. */
    enableAutoPicking: boolean;
    /** Fires an event when the manager is attached to a mesh */
    onAttachedToMeshObservable: Observable<Nullable<AbstractMesh>>;
    /** Fires an event when the manager is attached to a node */
    onAttachedToNodeObservable: Observable<Nullable<Node>>;
    protected _gizmosEnabled: {
        positionGizmo: boolean;
        rotationGizmo: boolean;
        scaleGizmo: boolean;
        boundingBoxGizmo: boolean;
    };
    protected _pointerObservers: Observer<PointerInfo>[];
    protected _attachedMesh: Nullable<AbstractMesh>;
    protected _attachedNode: Nullable<Node>;
    protected _boundingBoxColor: Color3;
    protected _defaultUtilityLayer: UtilityLayerRenderer;
    protected _defaultKeepDepthUtilityLayer: UtilityLayerRenderer;
    protected _thickness: number;
    protected _scaleRatio: number;
    /** Node Caching for quick lookup */
    private _gizmoAxisCache;
    /**
     * When bounding box gizmo is enabled, this can be used to track drag/end events
     */
    boundingBoxDragBehavior: SixDofDragBehavior;
    /**
     * Array of meshes which will have the gizmo attached when a pointer selected them. If null, all meshes are attachable. (Default: null)
     */
    attachableMeshes: Nullable<Array<AbstractMesh>>;
    /**
     * Array of nodes which will have the gizmo attached when a pointer selected them. If null, all nodes are attachable. (Default: null)
     */
    attachableNodes: Nullable<Array<Node>>;
    /**
     * If pointer events should perform attaching/detaching a gizmo, if false this can be done manually via attachToMesh/attachToNode. (Default: true)
     */
    usePointerToAttachGizmos: boolean;
    /**
     * Utility layer that the bounding box gizmo belongs to
     */
    get keepDepthUtilityLayer(): UtilityLayerRenderer;
    /**
     * Utility layer that all gizmos besides bounding box belong to
     */
    get utilityLayer(): UtilityLayerRenderer;
    /**
     * True when the mouse pointer is hovering a gizmo mesh
     */
    get isHovered(): boolean;
    /**
     * Ratio for the scale of the gizmo (Default: 1)
     */
    set scaleRatio(value: number);
    get scaleRatio(): number;
    /**
     * Instantiates a gizmo manager
     * @param _scene the scene to overlay the gizmos on top of
     * @param thickness display gizmo axis thickness
     * @param utilityLayer the layer where gizmos are rendered
     * @param keepDepthUtilityLayer the layer where occluded gizmos are rendered
     */
    constructor(_scene: Scene, thickness?: number, utilityLayer?: UtilityLayerRenderer, keepDepthUtilityLayer?: UtilityLayerRenderer);
    /**
     * Subscribes to pointer down events, for attaching and detaching mesh
     * @param scene The scene layer the observer will be added to
     */
    private _attachToMeshPointerObserver;
    /**
     * Attaches a set of gizmos to the specified mesh
     * @param mesh The mesh the gizmo's should be attached to
     */
    attachToMesh(mesh: Nullable<AbstractMesh>): void;
    /**
     * Attaches a set of gizmos to the specified node
     * @param node The node the gizmo's should be attached to
     */
    attachToNode(node: Nullable<Node>): void;
    /**
     * If the position gizmo is enabled
     */
    set positionGizmoEnabled(value: boolean);
    get positionGizmoEnabled(): boolean;
    /**
     * If the rotation gizmo is enabled
     */
    set rotationGizmoEnabled(value: boolean);
    get rotationGizmoEnabled(): boolean;
    /**
     * If the scale gizmo is enabled
     */
    set scaleGizmoEnabled(value: boolean);
    get scaleGizmoEnabled(): boolean;
    /**
     * If the boundingBox gizmo is enabled
     */
    set boundingBoxGizmoEnabled(value: boolean);
    get boundingBoxGizmoEnabled(): boolean;
    /**
     * Builds Gizmo Axis Cache to enable features such as hover state preservation and graying out other axis during manipulation
     * @param gizmoAxisCache Gizmo axis definition used for reactive gizmo UI
     */
    addToAxisCache(gizmoAxisCache: Map<Mesh, GizmoAxisCache>): void;
    /**
     * Disposes of the gizmo manager
     */
    dispose(): void;
}

}
declare module "babylonjs/Gizmos/index" {
export * from "babylonjs/Gizmos/axisDragGizmo";
export * from "babylonjs/Gizmos/axisScaleGizmo";
export * from "babylonjs/Gizmos/boundingBoxGizmo";
export * from "babylonjs/Gizmos/gizmo";
export * from "babylonjs/Gizmos/gizmoManager";
export * from "babylonjs/Gizmos/planeRotationGizmo";
export * from "babylonjs/Gizmos/positionGizmo";
export * from "babylonjs/Gizmos/rotationGizmo";
export * from "babylonjs/Gizmos/scaleGizmo";
export * from "babylonjs/Gizmos/lightGizmo";
export * from "babylonjs/Gizmos/cameraGizmo";
export * from "babylonjs/Gizmos/planeDragGizmo";

}
declare module "babylonjs/Gizmos/lightGizmo" {
import { Nullable } from "babylonjs/types";
import { Vector3 } from "babylonjs/Maths/math.vector";
import { Mesh } from "babylonjs/Meshes/mesh";
import { IGizmo } from "babylonjs/Gizmos/gizmo";
import { Gizmo } from "babylonjs/Gizmos/gizmo";
import { UtilityLayerRenderer } from "babylonjs/Rendering/utilityLayerRenderer";
import { Node } from "babylonjs/node";
import { StandardMaterial } from "babylonjs/Materials/standardMaterial";
import { Light } from "babylonjs/Lights/light";
import { TransformNode } from "babylonjs/Meshes/transformNode";
import { PointerInfo } from "babylonjs/Events/pointerEvents";
import { Observer } from "babylonjs/Misc/observable";
import { Observable } from "babylonjs/Misc/observable";
/**
 * Interface for light gizmo
 */
export interface ILightGizmo extends IGizmo {
    /** Event that fires each time the gizmo is clicked */
    onClickedObservable: Observable<Light>;
    /** The light that the gizmo is attached to */
    light: Nullable<Light>;
    /** The material used to render the light gizmo */
    readonly material: StandardMaterial;
}
/**
 * Gizmo that enables viewing a light
 */
export class LightGizmo extends Gizmo implements ILightGizmo {
    protected _lightMesh: Mesh;
    protected _material: StandardMaterial;
    protected _cachedPosition: Vector3;
    protected _cachedForward: Vector3;
    protected _attachedMeshParent: TransformNode;
    protected _pointerObserver: Nullable<Observer<PointerInfo>>;
    /**
     * Event that fires each time the gizmo is clicked
     */
    onClickedObservable: Observable<Light>;
    /**
     * Creates a LightGizmo
     * @param gizmoLayer The utility layer the gizmo will be added to
     */
    constructor(gizmoLayer?: UtilityLayerRenderer);
    protected _light: Nullable<Light>;
    /**
     * Override attachedNode because lightgizmo only support attached mesh
     * It will return the attached mesh (if any) and setting an attached node will log
     * a warning
     */
    get attachedNode(): Nullable<Node>;
    set attachedNode(value: Nullable<Node>);
    /**
     * The light that the gizmo is attached to
     */
    set light(light: Nullable<Light>);
    get light(): Nullable<Light>;
    /**
     * Gets the material used to render the light gizmo
     */
    get material(): StandardMaterial;
    /**
     * @internal
     * Updates the gizmo to match the attached mesh's position/rotation
     */
    protected _update(): void;
    private static _Scale;
    /**
     * Creates the lines for a light mesh
     * @param levels
     * @param scene
     */
    private static _CreateLightLines;
    /**
     * Disposes of the light gizmo
     */
    dispose(): void;
    private static _CreateHemisphericLightMesh;
    private static _CreatePointLightMesh;
    private static _CreateSpotLightMesh;
    private static _CreateDirectionalLightMesh;
}

}
declare module "babylonjs/Gizmos/planeDragGizmo" {
import { Observer } from "babylonjs/Misc/observable";
import { Observable } from "babylonjs/Misc/observable";
import { Nullable } from "babylonjs/types";
import { PointerInfo } from "babylonjs/Events/pointerEvents";
import { Vector3 } from "babylonjs/Maths/math.vector";
import { Color3 } from "babylonjs/Maths/math.color";
import { TransformNode } from "babylonjs/Meshes/transformNode";
import { Node } from "babylonjs/node";
import { PointerDragBehavior } from "babylonjs/Behaviors/Meshes/pointerDragBehavior";
import { IGizmo } from "babylonjs/Gizmos/gizmo";
import { Gizmo } from "babylonjs/Gizmos/gizmo";
import { UtilityLayerRenderer } from "babylonjs/Rendering/utilityLayerRenderer";
import { StandardMaterial } from "babylonjs/Materials/standardMaterial";
import { Scene } from "babylonjs/scene";
import { PositionGizmo } from "babylonjs/Gizmos/positionGizmo";
/**
 * Interface for plane drag gizmo
 */
export interface IPlaneDragGizmo extends IGizmo {
    /** Drag behavior responsible for the gizmos dragging interactions */
    dragBehavior: PointerDragBehavior;
    /** Drag distance in babylon units that the gizmo will snap to when dragged */
    snapDistance: number;
    /**
     * Event that fires each time the gizmo snaps to a new location.
     * * snapDistance is the the change in distance
     */
    onSnapObservable: Observable<{
        snapDistance: number;
    }>;
    /** If the gizmo is enabled */
    isEnabled: boolean;
    /** Default material used to render when gizmo is not disabled or hovered */
    coloredMaterial: StandardMaterial;
    /** Material used to render when gizmo is hovered with mouse*/
    hoverMaterial: StandardMaterial;
    /** Material used to render when gizmo is disabled. typically grey.*/
    disableMaterial: StandardMaterial;
}
/**
 * Single plane drag gizmo
 */
export class PlaneDragGizmo extends Gizmo implements IPlaneDragGizmo {
    /**
     * Drag behavior responsible for the gizmos dragging interactions
     */
    dragBehavior: PointerDragBehavior;
    protected _pointerObserver: Nullable<Observer<PointerInfo>>;
    /**
     * Drag distance in babylon units that the gizmo will snap to when dragged (Default: 0)
     */
    snapDistance: number;
    /**
     * Event that fires each time the gizmo snaps to a new location.
     * * snapDistance is the the change in distance
     */
    onSnapObservable: Observable<{
        snapDistance: number;
    }>;
    protected _gizmoMesh: TransformNode;
    protected _coloredMaterial: StandardMaterial;
    protected _hoverMaterial: StandardMaterial;
    protected _disableMaterial: StandardMaterial;
    protected _isEnabled: boolean;
    protected _parent: Nullable<PositionGizmo>;
    protected _dragging: boolean;
    /** Default material used to render when gizmo is not disabled or hovered */
    get coloredMaterial(): StandardMaterial;
    /** Material used to render when gizmo is hovered with mouse*/
    get hoverMaterial(): StandardMaterial;
    /** Material used to render when gizmo is disabled. typically grey.*/
    get disableMaterial(): StandardMaterial;
    /**
     * @internal
     */
    static _CreatePlane(scene: Scene, material: StandardMaterial): TransformNode;
    /**
     * Creates a PlaneDragGizmo
     * @param dragPlaneNormal The axis normal to which the gizmo will be able to drag on
     * @param color The color of the gizmo
     * @param gizmoLayer The utility layer the gizmo will be added to
     * @param parent
     */
    constructor(dragPlaneNormal: Vector3, color?: Color3, gizmoLayer?: UtilityLayerRenderer, parent?: Nullable<PositionGizmo>);
    protected _attachedNodeChanged(value: Nullable<Node>): void;
    /**
     * If the gizmo is enabled
     */
    set isEnabled(value: boolean);
    get isEnabled(): boolean;
    /**
     * Disposes of the gizmo
     */
    dispose(): void;
}

}
declare module "babylonjs/Gizmos/planeRotationGizmo" {
import { Observer } from "babylonjs/Misc/observable";
import { Observable } from "babylonjs/Misc/observable";
import { Nullable } from "babylonjs/types";
import { PointerInfo } from "babylonjs/Events/pointerEvents";
import { Vector3 } from "babylonjs/Maths/math.vector";
import { Color3 } from "babylonjs/Maths/math.color";
import "babylonjs/Meshes/Builders/linesBuilder";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { Mesh } from "babylonjs/Meshes/mesh";
import { Node } from "babylonjs/node";
import { PointerDragBehavior } from "babylonjs/Behaviors/Meshes/pointerDragBehavior";
import { IGizmo } from "babylonjs/Gizmos/gizmo";
import { Gizmo } from "babylonjs/Gizmos/gizmo";
import { UtilityLayerRenderer } from "babylonjs/Rendering/utilityLayerRenderer";
import { StandardMaterial } from "babylonjs/Materials/standardMaterial";
import { RotationGizmo } from "babylonjs/Gizmos/rotationGizmo";
import { ShaderMaterial } from "babylonjs/Materials/shaderMaterial";
/**
 * Interface for plane rotation gizmo
 */
export interface IPlaneRotationGizmo extends IGizmo {
    /** Drag behavior responsible for the gizmos dragging interactions */
    dragBehavior: PointerDragBehavior;
    /** Drag distance in babylon units that the gizmo will snap to when dragged */
    snapDistance: number;
    /**
     * Event that fires each time the gizmo snaps to a new location.
     * * snapDistance is the the change in distance
     */
    onSnapObservable: Observable<{
        snapDistance: number;
    }>;
    /** Accumulated relative angle value for rotation on the axis. */
    angle: number;
    /** If the gizmo is enabled */
    isEnabled: boolean;
    /** Default material used to render when gizmo is not disabled or hovered */
    coloredMaterial: StandardMaterial;
    /** Material used to render when gizmo is hovered with mouse*/
    hoverMaterial: StandardMaterial;
    /** Material used to render when gizmo is disabled. typically grey.*/
    disableMaterial: StandardMaterial;
}
/**
 * Single plane rotation gizmo
 */
export class PlaneRotationGizmo extends Gizmo implements IPlaneRotationGizmo {
    /**
     * Drag behavior responsible for the gizmos dragging interactions
     */
    dragBehavior: PointerDragBehavior;
    protected _pointerObserver: Nullable<Observer<PointerInfo>>;
    /**
     * Rotation distance in radians that the gizmo will snap to (Default: 0)
     */
    snapDistance: number;
    /**
     * Event that fires each time the gizmo snaps to a new location.
     * * snapDistance is the the change in distance
     */
    onSnapObservable: Observable<{
        snapDistance: number;
    }>;
    /**
     * The maximum angle between the camera and the rotation allowed for interaction
     * If a rotation plane appears 'flat', a lower value allows interaction.
     */
    static MaxDragAngle: number;
    /**
     * Accumulated relative angle value for rotation on the axis. Reset to 0 when a dragStart occurs
     */
    angle: number;
    /** Default material used to render when gizmo is not disabled or hovered */
    get coloredMaterial(): StandardMaterial;
    /** Material used to render when gizmo is hovered with mouse*/
    get hoverMaterial(): StandardMaterial;
    /** Material used to render when gizmo is disabled. typically grey.*/
    get disableMaterial(): StandardMaterial;
    protected _isEnabled: boolean;
    protected _parent: Nullable<RotationGizmo>;
    protected _coloredMaterial: StandardMaterial;
    protected _hoverMaterial: StandardMaterial;
    protected _disableMaterial: StandardMaterial;
    protected _gizmoMesh: Mesh;
    protected _rotationDisplayPlane: Mesh;
    protected _dragging: boolean;
    protected _angles: Vector3;
    protected static _RotationGizmoVertexShader: string;
    protected static _RotationGizmoFragmentShader: string;
    protected _rotationShaderMaterial: ShaderMaterial;
    /**
     * Creates a PlaneRotationGizmo
     * @param planeNormal The normal of the plane which the gizmo will be able to rotate on
     * @param color The color of the gizmo
     * @param gizmoLayer The utility layer the gizmo will be added to
     * @param tessellation Amount of tessellation to be used when creating rotation circles
     * @param parent
     * @param useEulerRotation Use and update Euler angle instead of quaternion
     * @param thickness display gizmo axis thickness
     */
    constructor(planeNormal: Vector3, color?: Color3, gizmoLayer?: UtilityLayerRenderer, tessellation?: number, parent?: Nullable<RotationGizmo>, useEulerRotation?: boolean, thickness?: number);
    /**
     * Create Geometry for Gizmo
     * @param parentMesh
     * @param thickness
     * @param tessellation
     */
    protected _createGizmoMesh(parentMesh: AbstractMesh, thickness: number, tessellation: number): {
        rotationMesh: Mesh;
        collider: Mesh;
    };
    protected _attachedNodeChanged(value: Nullable<Node>): void;
    /**
     * If the gizmo is enabled
     */
    set isEnabled(value: boolean);
    get isEnabled(): boolean;
    /**
     * Disposes of the gizmo
     */
    dispose(): void;
}

}
declare module "babylonjs/Gizmos/positionGizmo" {
import { Observer } from "babylonjs/Misc/observable";
import { Observable } from "babylonjs/Misc/observable";
import { Nullable } from "babylonjs/types";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { Node } from "babylonjs/node";
import { Mesh } from "babylonjs/Meshes/mesh";
import { GizmoAxisCache, IGizmo } from "babylonjs/Gizmos/gizmo";
import { Gizmo } from "babylonjs/Gizmos/gizmo";
import { IAxisDragGizmo } from "babylonjs/Gizmos/axisDragGizmo";
import { IPlaneDragGizmo } from "babylonjs/Gizmos/planeDragGizmo";
import { UtilityLayerRenderer } from "babylonjs/Rendering/utilityLayerRenderer";
import { PointerInfo } from "babylonjs/Events/pointerEvents";
import { GizmoManager } from "babylonjs/Gizmos/gizmoManager";
/**
 * Interface for position gizmo
 */
export interface IPositionGizmo extends IGizmo {
    /** Internal gizmo used for interactions on the x axis */
    xGizmo: IAxisDragGizmo;
    /** Internal gizmo used for interactions on the y axis */
    yGizmo: IAxisDragGizmo;
    /** Internal gizmo used for interactions on the z axis */
    zGizmo: IAxisDragGizmo;
    /** Internal gizmo used for interactions on the yz plane */
    xPlaneGizmo: IPlaneDragGizmo;
    /** Internal gizmo used for interactions on the xz plane */
    yPlaneGizmo: IPlaneDragGizmo;
    /** Internal gizmo used for interactions on the xy plane */
    zPlaneGizmo: IPlaneDragGizmo;
    /** Fires an event when any of it's sub gizmos are dragged */
    onDragStartObservable: Observable<unknown>;
    /** Fires an event when any of it's sub gizmos are released from dragging */
    onDragEndObservable: Observable<unknown>;
    /**
     * If the planar drag gizmo is enabled
     * setting this will enable/disable XY, XZ and YZ planes regardless of individual gizmo settings.
     */
    planarGizmoEnabled: boolean;
    /** Drag distance in babylon units that the gizmo will snap to when dragged */
    snapDistance: number;
    /**
     * Builds Gizmo Axis Cache to enable features such as hover state preservation and graying out other axis during manipulation
     * @param mesh Axis gizmo mesh
     * @param cache Gizmo axis definition used for reactive gizmo UI
     */
    addToAxisCache(mesh: Mesh, cache: GizmoAxisCache): void;
}
/**
 * Gizmo that enables dragging a mesh along 3 axis
 */
export class PositionGizmo extends Gizmo implements IPositionGizmo {
    /**
     * Internal gizmo used for interactions on the x axis
     */
    xGizmo: IAxisDragGizmo;
    /**
     * Internal gizmo used for interactions on the y axis
     */
    yGizmo: IAxisDragGizmo;
    /**
     * Internal gizmo used for interactions on the z axis
     */
    zGizmo: IAxisDragGizmo;
    /**
     * Internal gizmo used for interactions on the yz plane
     */
    xPlaneGizmo: IPlaneDragGizmo;
    /**
     * Internal gizmo used for interactions on the xz plane
     */
    yPlaneGizmo: IPlaneDragGizmo;
    /**
     * Internal gizmo used for interactions on the xy plane
     */
    zPlaneGizmo: IPlaneDragGizmo;
    /**
     * protected variables
     */
    protected _meshAttached: Nullable<AbstractMesh>;
    protected _nodeAttached: Nullable<Node>;
    protected _snapDistance: number;
    protected _observables: Observer<PointerInfo>[];
    /** Node Caching for quick lookup */
    protected _gizmoAxisCache: Map<Mesh, GizmoAxisCache>;
    /** Fires an event when any of it's sub gizmos are dragged */
    onDragStartObservable: Observable<unknown>;
    /** Fires an event when any of it's sub gizmos are released from dragging */
    onDragEndObservable: Observable<unknown>;
    /**
     * If set to true, planar drag is enabled
     */
    protected _planarGizmoEnabled: boolean;
    get attachedMesh(): Nullable<AbstractMesh>;
    set attachedMesh(mesh: Nullable<AbstractMesh>);
    get attachedNode(): Nullable<Node>;
    set attachedNode(node: Nullable<Node>);
    /**
     * True when the mouse pointer is hovering a gizmo mesh
     */
    get isHovered(): boolean;
    /**
     * Creates a PositionGizmo
     * @param gizmoLayer The utility layer the gizmo will be added to
      @param thickness display gizmo axis thickness
     * @param gizmoManager
     */
    constructor(gizmoLayer?: UtilityLayerRenderer, thickness?: number, gizmoManager?: GizmoManager);
    /**
     * If the planar drag gizmo is enabled
     * setting this will enable/disable XY, XZ and YZ planes regardless of individual gizmo settings.
     */
    set planarGizmoEnabled(value: boolean);
    get planarGizmoEnabled(): boolean;
    /**
     * If set the gizmo's rotation will be updated to match the attached mesh each frame (Default: true)
     * NOTE: This is only possible for meshes with uniform scaling, as otherwise it's not possible to decompose the rotation
     */
    set updateGizmoRotationToMatchAttachedMesh(value: boolean);
    get updateGizmoRotationToMatchAttachedMesh(): boolean;
    set updateGizmoPositionToMatchAttachedMesh(value: boolean);
    get updateGizmoPositionToMatchAttachedMesh(): boolean;
    set updateScale(value: boolean);
    get updateScale(): boolean;
    /**
     * Drag distance in babylon units that the gizmo will snap to when dragged (Default: 0)
     */
    set snapDistance(value: number);
    get snapDistance(): number;
    /**
     * Ratio for the scale of the gizmo (Default: 1)
     */
    set scaleRatio(value: number);
    get scaleRatio(): number;
    /**
     * Builds Gizmo Axis Cache to enable features such as hover state preservation and graying out other axis during manipulation
     * @param mesh Axis gizmo mesh
     * @param cache Gizmo axis definition used for reactive gizmo UI
     */
    addToAxisCache(mesh: Mesh, cache: GizmoAxisCache): void;
    /**
     * Disposes of the gizmo
     */
    dispose(): void;
    /**
     * CustomMeshes are not supported by this gizmo
     */
    setCustomMesh(): void;
}

}
declare module "babylonjs/Gizmos/rotationGizmo" {
import { Observer } from "babylonjs/Misc/observable";
import { Observable } from "babylonjs/Misc/observable";
import { Nullable } from "babylonjs/types";
import { Color3 } from "babylonjs/Maths/math.color";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { Mesh } from "babylonjs/Meshes/mesh";
import { GizmoAxisCache, IGizmo } from "babylonjs/Gizmos/gizmo";
import { Gizmo } from "babylonjs/Gizmos/gizmo";
import { IPlaneRotationGizmo } from "babylonjs/Gizmos/planeRotationGizmo";
import { UtilityLayerRenderer } from "babylonjs/Rendering/utilityLayerRenderer";
import { Node } from "babylonjs/node";
import { PointerInfo } from "babylonjs/Events/pointerEvents";
import { GizmoManager } from "babylonjs/Gizmos/gizmoManager";
/**
 * Interface for rotation gizmo
 */
export interface IRotationGizmo extends IGizmo {
    /** Internal gizmo used for interactions on the x axis */
    xGizmo: IPlaneRotationGizmo;
    /** Internal gizmo used for interactions on the y axis */
    yGizmo: IPlaneRotationGizmo;
    /** Internal gizmo used for interactions on the z axis */
    zGizmo: IPlaneRotationGizmo;
    /** Fires an event when any of it's sub gizmos are dragged */
    onDragStartObservable: Observable<unknown>;
    /** Fires an event when any of it's sub gizmos are released from dragging */
    onDragEndObservable: Observable<unknown>;
    /** Drag distance in babylon units that the gizmo will snap to when dragged */
    snapDistance: number;
    /**
     * Builds Gizmo Axis Cache to enable features such as hover state preservation and graying out other axis during manipulation
     * @param mesh Axis gizmo mesh
     * @param cache Gizmo axis definition used for reactive gizmo UI
     */
    addToAxisCache(mesh: Mesh, cache: GizmoAxisCache): void;
}
/**
 * Options for each individual plane rotation gizmo contained within RotationGizmo
 * @since 5.0.0
 */
export interface PlaneRotationGizmoOptions {
    /**
     * Color to use for the plane rotation gizmo
     */
    color?: Color3;
}
/**
 * Additional options for each rotation gizmo
 */
export interface RotationGizmoOptions {
    /**
     * When set, the gizmo will always appear the same size no matter where the camera is (default: true)
     */
    updateScale?: boolean;
    /**
     * Specific options for xGizmo
     */
    xOptions?: PlaneRotationGizmoOptions;
    /**
     * Specific options for yGizmo
     */
    yOptions?: PlaneRotationGizmoOptions;
    /**
     * Specific options for zGizmo
     */
    zOptions?: PlaneRotationGizmoOptions;
}
/**
 * Gizmo that enables rotating a mesh along 3 axis
 */
export class RotationGizmo extends Gizmo implements IRotationGizmo {
    /**
     * Internal gizmo used for interactions on the x axis
     */
    xGizmo: IPlaneRotationGizmo;
    /**
     * Internal gizmo used for interactions on the y axis
     */
    yGizmo: IPlaneRotationGizmo;
    /**
     * Internal gizmo used for interactions on the z axis
     */
    zGizmo: IPlaneRotationGizmo;
    /** Fires an event when any of it's sub gizmos are dragged */
    onDragStartObservable: Observable<unknown>;
    /** Fires an event when any of it's sub gizmos are released from dragging */
    onDragEndObservable: Observable<unknown>;
    protected _meshAttached: Nullable<AbstractMesh>;
    protected _nodeAttached: Nullable<Node>;
    protected _observables: Observer<PointerInfo>[];
    /** Node Caching for quick lookup */
    protected _gizmoAxisCache: Map<Mesh, GizmoAxisCache>;
    get attachedMesh(): Nullable<AbstractMesh>;
    set attachedMesh(mesh: Nullable<AbstractMesh>);
    get attachedNode(): Nullable<Node>;
    set attachedNode(node: Nullable<Node>);
    protected _checkBillboardTransform(): void;
    /**
     * True when the mouse pointer is hovering a gizmo mesh
     */
    get isHovered(): boolean;
    /**
     * Creates a RotationGizmo
     * @param gizmoLayer The utility layer the gizmo will be added to
     * @param tessellation Amount of tessellation to be used when creating rotation circles
     * @param useEulerRotation Use and update Euler angle instead of quaternion
     * @param thickness display gizmo axis thickness
     * @param gizmoManager Gizmo manager
     * @param options More options
     */
    constructor(gizmoLayer?: UtilityLayerRenderer, tessellation?: number, useEulerRotation?: boolean, thickness?: number, gizmoManager?: GizmoManager, options?: RotationGizmoOptions);
    /**
     * If set the gizmo's rotation will be updated to match the attached mesh each frame (Default: true)
     * NOTE: This is only possible for meshes with uniform scaling, as otherwise it's not possible to decompose the rotation
     */
    set updateGizmoRotationToMatchAttachedMesh(value: boolean);
    get updateGizmoRotationToMatchAttachedMesh(): boolean;
    set updateGizmoPositionToMatchAttachedMesh(value: boolean);
    get updateGizmoPositionToMatchAttachedMesh(): boolean;
    set updateScale(value: boolean);
    get updateScale(): boolean;
    /**
     * Drag distance in babylon units that the gizmo will snap to when dragged (Default: 0)
     */
    set snapDistance(value: number);
    get snapDistance(): number;
    /**
     * Ratio for the scale of the gizmo (Default: 1)
     */
    set scaleRatio(value: number);
    get scaleRatio(): number;
    /**
     * Builds Gizmo Axis Cache to enable features such as hover state preservation and graying out other axis during manipulation
     * @param mesh Axis gizmo mesh
     * @param cache Gizmo axis definition used for reactive gizmo UI
     */
    addToAxisCache(mesh: Mesh, cache: GizmoAxisCache): void;
    /**
     * Disposes of the gizmo
     */
    dispose(): void;
    /**
     * CustomMeshes are not supported by this gizmo
     */
    setCustomMesh(): void;
}

}
declare module "babylonjs/Gizmos/scaleGizmo" {
import { Observer } from "babylonjs/Misc/observable";
import { Observable } from "babylonjs/Misc/observable";
import { Nullable } from "babylonjs/types";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { GizmoAxisCache, IGizmo } from "babylonjs/Gizmos/gizmo";
import { Gizmo } from "babylonjs/Gizmos/gizmo";
import { IAxisScaleGizmo } from "babylonjs/Gizmos/axisScaleGizmo";
import { AxisScaleGizmo } from "babylonjs/Gizmos/axisScaleGizmo";
import { UtilityLayerRenderer } from "babylonjs/Rendering/utilityLayerRenderer";
import { Mesh } from "babylonjs/Meshes/mesh";
import { Node } from "babylonjs/node";
import { PointerInfo } from "babylonjs/Events/pointerEvents";
import { StandardMaterial } from "babylonjs/Materials/standardMaterial";
import { GizmoManager } from "babylonjs/Gizmos/gizmoManager";
/**
 * Interface for scale gizmo
 */
export interface IScaleGizmo extends IGizmo {
    /** Internal gizmo used for interactions on the x axis */
    xGizmo: IAxisScaleGizmo;
    /** Internal gizmo used for interactions on the y axis */
    yGizmo: IAxisScaleGizmo;
    /** Internal gizmo used for interactions on the z axis */
    zGizmo: IAxisScaleGizmo;
    /** Internal gizmo used to scale all axis equally*/
    uniformScaleGizmo: IAxisScaleGizmo;
    /** Fires an event when any of it's sub gizmos are dragged */
    onDragStartObservable: Observable<unknown>;
    /** Fires an event when any of it's sub gizmos are released from dragging */
    onDragEndObservable: Observable<unknown>;
    /** Drag distance in babylon units that the gizmo will snap to when dragged */
    snapDistance: number;
    /** Sensitivity factor for dragging */
    sensitivity: number;
    /**
     * Builds Gizmo Axis Cache to enable features such as hover state preservation and graying out other axis during manipulation
     * @param mesh Axis gizmo mesh
     * @param cache Gizmo axis definition used for reactive gizmo UI
     */
    addToAxisCache(mesh: Mesh, cache: GizmoAxisCache): void;
    /** Default material used to render when gizmo is not disabled or hovered */
    coloredMaterial: StandardMaterial;
    /** Material used to render when gizmo is hovered with mouse*/
    hoverMaterial: StandardMaterial;
    /** Material used to render when gizmo is disabled. typically grey.*/
    disableMaterial: StandardMaterial;
}
/**
 * Gizmo that enables scaling a mesh along 3 axis
 */
export class ScaleGizmo extends Gizmo implements IScaleGizmo {
    /**
     * Internal gizmo used for interactions on the x axis
     */
    xGizmo: IAxisScaleGizmo;
    /**
     * Internal gizmo used for interactions on the y axis
     */
    yGizmo: IAxisScaleGizmo;
    /**
     * Internal gizmo used for interactions on the z axis
     */
    zGizmo: IAxisScaleGizmo;
    /**
     * Internal gizmo used to scale all axis equally
     */
    uniformScaleGizmo: IAxisScaleGizmo;
    protected _meshAttached: Nullable<AbstractMesh>;
    protected _nodeAttached: Nullable<Node>;
    protected _snapDistance: number;
    protected _uniformScalingMesh: Mesh;
    protected _octahedron: Mesh;
    protected _sensitivity: number;
    protected _coloredMaterial: StandardMaterial;
    protected _hoverMaterial: StandardMaterial;
    protected _disableMaterial: StandardMaterial;
    protected _observables: Observer<PointerInfo>[];
    /** Node Caching for quick lookup */
    protected _gizmoAxisCache: Map<Mesh, GizmoAxisCache>;
    /** Default material used to render when gizmo is not disabled or hovered */
    get coloredMaterial(): StandardMaterial;
    /** Material used to render when gizmo is hovered with mouse*/
    get hoverMaterial(): StandardMaterial;
    /** Material used to render when gizmo is disabled. typically grey.*/
    get disableMaterial(): StandardMaterial;
    /** Fires an event when any of it's sub gizmos are dragged */
    onDragStartObservable: Observable<unknown>;
    /** Fires an event when any of it's sub gizmos are released from dragging */
    onDragEndObservable: Observable<unknown>;
    get attachedMesh(): Nullable<AbstractMesh>;
    set attachedMesh(mesh: Nullable<AbstractMesh>);
    get attachedNode(): Nullable<Node>;
    set attachedNode(node: Nullable<Node>);
    set updateScale(value: boolean);
    get updateScale(): boolean;
    /**
     * True when the mouse pointer is hovering a gizmo mesh
     */
    get isHovered(): boolean;
    /**
     * Creates a ScaleGizmo
     * @param gizmoLayer The utility layer the gizmo will be added to
     * @param thickness display gizmo axis thickness
     * @param gizmoManager
     */
    constructor(gizmoLayer?: UtilityLayerRenderer, thickness?: number, gizmoManager?: GizmoManager);
    /** Create Geometry for Gizmo */
    protected _createUniformScaleMesh(): AxisScaleGizmo;
    set updateGizmoRotationToMatchAttachedMesh(value: boolean);
    get updateGizmoRotationToMatchAttachedMesh(): boolean;
    /**
     * Drag distance in babylon units that the gizmo will snap to when dragged (Default: 0)
     */
    set snapDistance(value: number);
    get snapDistance(): number;
    /**
     * Ratio for the scale of the gizmo (Default: 1)
     */
    set scaleRatio(value: number);
    get scaleRatio(): number;
    /**
     * Sensitivity factor for dragging (Default: 1)
     */
    set sensitivity(value: number);
    get sensitivity(): number;
    /**
     * Builds Gizmo Axis Cache to enable features such as hover state preservation and graying out other axis during manipulation
     * @param mesh Axis gizmo mesh
     * @param cache Gizmo axis definition used for reactive gizmo UI
     */
    addToAxisCache(mesh: Mesh, cache: GizmoAxisCache): void;
    /**
     * Disposes of the gizmo
     */
    dispose(): void;
}

}
declare module "babylonjs/Helpers/environmentHelper" {
import { Observable } from "babylonjs/Misc/observable";
import { Nullable } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
import { Vector3 } from "babylonjs/Maths/math.vector";
import { Color3 } from "babylonjs/Maths/math.color";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { Mesh } from "babylonjs/Meshes/mesh";
import { BaseTexture } from "babylonjs/Materials/Textures/baseTexture";
import { MirrorTexture } from "babylonjs/Materials/Textures/mirrorTexture";
import { BackgroundMaterial } from "babylonjs/Materials/Background/backgroundMaterial";
/**
 * Represents the different options available during the creation of
 * a Environment helper.
 *
 * This can control the default ground, skybox and image processing setup of your scene.
 */
export interface IEnvironmentHelperOptions {
    /**
     * Specifies whether or not to create a ground.
     * True by default.
     */
    createGround: boolean;
    /**
     * Specifies the ground size.
     * 15 by default.
     */
    groundSize: number;
    /**
     * The texture used on the ground for the main color.
     * Comes from the BabylonJS CDN by default.
     *
     * Remarks: Can be either a texture or a url.
     */
    groundTexture: string | BaseTexture;
    /**
     * The color mixed in the ground texture by default.
     * BabylonJS clearColor by default.
     */
    groundColor: Color3;
    /**
     * Specifies the ground opacity.
     * 1 by default.
     */
    groundOpacity: number;
    /**
     * Enables the ground to receive shadows.
     * True by default.
     */
    enableGroundShadow: boolean;
    /**
     * Helps preventing the shadow to be fully black on the ground.
     * 0.5 by default.
     */
    groundShadowLevel: number;
    /**
     * Creates a mirror texture attach to the ground.
     * false by default.
     */
    enableGroundMirror: boolean;
    /**
     * Specifies the ground mirror size ratio.
     * 0.3 by default as the default kernel is 64.
     */
    groundMirrorSizeRatio: number;
    /**
     * Specifies the ground mirror blur kernel size.
     * 64 by default.
     */
    groundMirrorBlurKernel: number;
    /**
     * Specifies the ground mirror visibility amount.
     * 1 by default
     */
    groundMirrorAmount: number;
    /**
     * Specifies the ground mirror reflectance weight.
     * This uses the standard weight of the background material to setup the fresnel effect
     * of the mirror.
     * 1 by default.
     */
    groundMirrorFresnelWeight: number;
    /**
     * Specifies the ground mirror Falloff distance.
     * This can helps reducing the size of the reflection.
     * 0 by Default.
     */
    groundMirrorFallOffDistance: number;
    /**
     * Specifies the ground mirror texture type.
     * Unsigned Int by Default.
     */
    groundMirrorTextureType: number;
    /**
     * Specifies a bias applied to the ground vertical position to prevent z-fighting with
     * the shown objects.
     */
    groundYBias: number;
    /**
     * Specifies whether or not to create a skybox.
     * True by default.
     */
    createSkybox: boolean;
    /**
     * Specifies the skybox size.
     * 20 by default.
     */
    skyboxSize: number;
    /**
     * The texture used on the skybox for the main color.
     * Comes from the BabylonJS CDN by default.
     *
     * Remarks: Can be either a texture or a url.
     */
    skyboxTexture: string | BaseTexture;
    /**
     * The color mixed in the skybox texture by default.
     * BabylonJS clearColor by default.
     */
    skyboxColor: Color3;
    /**
     * The background rotation around the Y axis of the scene.
     * This helps aligning the key lights of your scene with the background.
     * 0 by default.
     */
    backgroundYRotation: number;
    /**
     * Compute automatically the size of the elements to best fit with the scene.
     */
    sizeAuto: boolean;
    /**
     * Default position of the rootMesh if autoSize is not true.
     */
    rootPosition: Vector3;
    /**
     * Sets up the image processing in the scene.
     * true by default.
     */
    setupImageProcessing: boolean;
    /**
     * The texture used as your environment texture in the scene.
     * Comes from the BabylonJS CDN by default and in use if setupImageProcessing is true.
     *
     * Remarks: Can be either a texture or a url.
     */
    environmentTexture: string | BaseTexture;
    /**
     * The value of the exposure to apply to the scene.
     * 0.6 by default if setupImageProcessing is true.
     */
    cameraExposure: number;
    /**
     * The value of the contrast to apply to the scene.
     * 1.6 by default if setupImageProcessing is true.
     */
    cameraContrast: number;
    /**
     * Specifies whether or not tonemapping should be enabled in the scene.
     * true by default if setupImageProcessing is true.
     */
    toneMappingEnabled: boolean;
}
/**
 * The Environment helper class can be used to add a fully featured none expensive background to your scene.
 * It includes by default a skybox and a ground relying on the BackgroundMaterial.
 * It also helps with the default setup of your imageProcessing configuration.
 */
export class EnvironmentHelper {
    /**
     * Default ground texture URL.
     */
    private static _GroundTextureCDNUrl;
    /**
     * Default skybox texture URL.
     */
    private static _SkyboxTextureCDNUrl;
    /**
     * Default environment texture URL.
     */
    private static _EnvironmentTextureCDNUrl;
    /**
     * Creates the default options for the helper.
     */
    private static _GetDefaultOptions;
    private _rootMesh;
    /**
     * Gets the root mesh created by the helper.
     */
    get rootMesh(): Mesh;
    private _skybox;
    /**
     * Gets the skybox created by the helper.
     */
    get skybox(): Nullable<Mesh>;
    private _skyboxTexture;
    /**
     * Gets the skybox texture created by the helper.
     */
    get skyboxTexture(): Nullable<BaseTexture>;
    private _skyboxMaterial;
    /**
     * Gets the skybox material created by the helper.
     */
    get skyboxMaterial(): Nullable<BackgroundMaterial>;
    private _ground;
    /**
     * Gets the ground mesh created by the helper.
     */
    get ground(): Nullable<Mesh>;
    private _groundTexture;
    /**
     * Gets the ground texture created by the helper.
     */
    get groundTexture(): Nullable<BaseTexture>;
    private _groundMirror;
    /**
     * Gets the ground mirror created by the helper.
     */
    get groundMirror(): Nullable<MirrorTexture>;
    /**
     * Gets the ground mirror render list to helps pushing the meshes
     * you wish in the ground reflection.
     */
    get groundMirrorRenderList(): Nullable<AbstractMesh[]>;
    private _groundMaterial;
    /**
     * Gets the ground material created by the helper.
     */
    get groundMaterial(): Nullable<BackgroundMaterial>;
    /**
     * Stores the creation options.
     */
    private readonly _scene;
    private _options;
    /**
     * This observable will be notified with any error during the creation of the environment,
     * mainly texture creation errors.
     */
    onErrorObservable: Observable<{
        message?: string;
        exception?: any;
    }>;
    /**
     * constructor
     * @param options Defines the options we want to customize the helper
     * @param scene The scene to add the material to
     */
    constructor(options: Partial<IEnvironmentHelperOptions>, scene: Scene);
    /**
     * Updates the background according to the new options
     * @param options
     */
    updateOptions(options: Partial<IEnvironmentHelperOptions>): void;
    /**
     * Sets the primary color of all the available elements.
     * @param color the main color to affect to the ground and the background
     */
    setMainColor(color: Color3): void;
    /**
     * Setup the image processing according to the specified options.
     */
    private _setupImageProcessing;
    /**
     * Setup the environment texture according to the specified options.
     */
    private _setupEnvironmentTexture;
    /**
     * Setup the background according to the specified options.
     */
    private _setupBackground;
    /**
     * Get the scene sizes according to the setup.
     */
    private _getSceneSize;
    /**
     * Setup the ground according to the specified options.
     * @param sceneSize
     */
    private _setupGround;
    /**
     * Setup the ground material according to the specified options.
     */
    private _setupGroundMaterial;
    /**
     * Setup the ground diffuse texture according to the specified options.
     */
    private _setupGroundDiffuseTexture;
    /**
     * Setup the ground mirror texture according to the specified options.
     * @param sceneSize
     */
    private _setupGroundMirrorTexture;
    /**
     * Setup the ground to receive the mirror texture.
     */
    private _setupMirrorInGroundMaterial;
    /**
     * Setup the skybox according to the specified options.
     * @param sceneSize
     */
    private _setupSkybox;
    /**
     * Setup the skybox material according to the specified options.
     */
    private _setupSkyboxMaterial;
    /**
     * Setup the skybox reflection texture according to the specified options.
     */
    private _setupSkyboxReflectionTexture;
    private _errorHandler;
    /**
     * Dispose all the elements created by the Helper.
     */
    dispose(): void;
}

}
declare module "babylonjs/Helpers/index" {
export * from "babylonjs/Helpers/environmentHelper";
export * from "babylonjs/Helpers/photoDome";
export * from "babylonjs/Helpers/sceneHelpers";
export * from "babylonjs/Helpers/videoDome";

}
declare module "babylonjs/Helpers/photoDome" {
import { Scene } from "babylonjs/scene";
import { Texture } from "babylonjs/Materials/Textures/texture";
import { TextureDome } from "babylonjs/Helpers/textureDome";
/**
 * Display a 360 degree photo on an approximately spherical surface, useful for VR applications or skyboxes.
 * As a subclass of TransformNode, this allow parenting to the camera with different locations in the scene.
 * This class achieves its effect with a Texture and a correctly configured BackgroundMaterial on an inverted sphere.
 * Potential additions to this helper include zoom and and non-infinite distance rendering effects.
 */
export class PhotoDome extends TextureDome<Texture> {
    /**
     * Define the image as a Monoscopic panoramic 360 image.
     */
    static readonly MODE_MONOSCOPIC: number;
    /**
     * Define the image as a Stereoscopic TopBottom/OverUnder panoramic 360 image.
     */
    static readonly MODE_TOPBOTTOM: number;
    /**
     * Define the image as a Stereoscopic Side by Side panoramic 360 image.
     */
    static readonly MODE_SIDEBYSIDE: number;
    /**
     * Gets or sets the texture being displayed on the sphere
     */
    get photoTexture(): Texture;
    /**
     * sets the texture being displayed on the sphere
     */
    set photoTexture(value: Texture);
    /**
     * Gets the current video mode for the video. It can be:
     * * TextureDome.MODE_MONOSCOPIC : Define the texture source as a Monoscopic panoramic 360.
     * * TextureDome.MODE_TOPBOTTOM  : Define the texture source as a Stereoscopic TopBottom/OverUnder panoramic 360.
     * * TextureDome.MODE_SIDEBYSIDE : Define the texture source as a Stereoscopic Side by Side panoramic 360.
     */
    get imageMode(): number;
    /**
     * Sets the current video mode for the video. It can be:
     * * TextureDome.MODE_MONOSCOPIC : Define the texture source as a Monoscopic panoramic 360.
     * * TextureDome.MODE_TOPBOTTOM  : Define the texture source as a Stereoscopic TopBottom/OverUnder panoramic 360.
     * * TextureDome.MODE_SIDEBYSIDE : Define the texture source as a Stereoscopic Side by Side panoramic 360.
     */
    set imageMode(value: number);
    protected _initTexture(urlsOrElement: string, scene: Scene, options: any): Texture;
}

}
declare module "babylonjs/Helpers/sceneHelpers" {
import { Nullable } from "babylonjs/types";
import { Mesh } from "babylonjs/Meshes/mesh";
import { BaseTexture } from "babylonjs/Materials/Textures/baseTexture";
import { IEnvironmentHelperOptions } from "babylonjs/Helpers/environmentHelper";
import { EnvironmentHelper } from "babylonjs/Helpers/environmentHelper";
import { VRExperienceHelperOptions } from "babylonjs/Cameras/VR/vrExperienceHelper";
import { VRExperienceHelper } from "babylonjs/Cameras/VR/vrExperienceHelper";
import "babylonjs/Materials/Textures/Loaders/ddsTextureLoader";
import "babylonjs/Materials/Textures/Loaders/envTextureLoader";
import "babylonjs/Materials/Textures/Loaders/ktxTextureLoader";
import { WebXRDefaultExperienceOptions } from "babylonjs/XR/webXRDefaultExperience";
import { WebXRDefaultExperience } from "babylonjs/XR/webXRDefaultExperience";
/** @internal */
export var _forceSceneHelpersToBundle: boolean;
module "babylonjs/scene" {
    interface Scene {
        /**
         * Creates a default light for the scene.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/fastBuildWorld#create-default-light
         * @param replace has the default false, when true replaces the existing lights in the scene with a hemispheric light
         */
        createDefaultLight(replace?: boolean): void;
        /**
         * Creates a default camera for the scene.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/fastBuildWorld#create-default-camera
         * @param createArcRotateCamera has the default false which creates a free camera, when true creates an arc rotate camera
         * @param replace has default false, when true replaces the active camera in the scene
         * @param attachCameraControls has default false, when true attaches camera controls to the canvas.
         */
        createDefaultCamera(createArcRotateCamera?: boolean, replace?: boolean, attachCameraControls?: boolean): void;
        /**
         * Creates a default camera and a default light.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/fastBuildWorld#create-default-camera-or-light
         * @param createArcRotateCamera has the default false which creates a free camera, when true creates an arc rotate camera
         * @param replace has the default false, when true replaces the active camera/light in the scene
         * @param attachCameraControls has the default false, when true attaches camera controls to the canvas.
         */
        createDefaultCameraOrLight(createArcRotateCamera?: boolean, replace?: boolean, attachCameraControls?: boolean): void;
        /**
         * Creates a new sky box
         * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/fastBuildWorld#create-default-skybox
         * @param environmentTexture defines the texture to use as environment texture
         * @param pbr has default false which requires the StandardMaterial to be used, when true PBRMaterial must be used
         * @param scale defines the overall scale of the skybox
         * @param blur is only available when pbr is true, default is 0, no blur, maximum value is 1
         * @param setGlobalEnvTexture has default true indicating that scene.environmentTexture must match the current skybox texture
         * @returns a new mesh holding the sky box
         */
        createDefaultSkybox(environmentTexture?: BaseTexture, pbr?: boolean, scale?: number, blur?: number, setGlobalEnvTexture?: boolean): Nullable<Mesh>;
        /**
         * Creates a new environment
         * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/fastBuildWorld#create-default-environment
         * @param options defines the options you can use to configure the environment
         * @returns the new EnvironmentHelper
         */
        createDefaultEnvironment(options?: Partial<IEnvironmentHelperOptions>): Nullable<EnvironmentHelper>;
        /**
         * Creates a new VREXperienceHelper
         * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/webVRHelper
         * @param webVROptions defines the options used to create the new VREXperienceHelper
         * @deprecated Please use createDefaultXRExperienceAsync instead
         * @returns a new VREXperienceHelper
         */
        createDefaultVRExperience(webVROptions?: VRExperienceHelperOptions): VRExperienceHelper;
        /**
         * Creates a new WebXRDefaultExperience
         * @see https://doc.babylonjs.com/features/featuresDeepDive/webXR/introToWebXR
         * @param options experience options
         * @returns a promise for a new WebXRDefaultExperience
         */
        createDefaultXRExperienceAsync(options?: WebXRDefaultExperienceOptions): Promise<WebXRDefaultExperience>;
    }
}

}
declare module "babylonjs/Helpers/textureDome" {
import { Scene } from "babylonjs/scene";
import { TransformNode } from "babylonjs/Meshes/transformNode";
import { Mesh } from "babylonjs/Meshes/mesh";
import { Texture } from "babylonjs/Materials/Textures/texture";
import { BackgroundMaterial } from "babylonjs/Materials/Background/backgroundMaterial";
import { Nullable } from "babylonjs/types";
import { Observable } from "babylonjs/Misc/observable";
/**
 * Display a 360/180 degree texture on an approximately spherical surface, useful for VR applications or skyboxes.
 * As a subclass of TransformNode, this allow parenting to the camera or multiple textures with different locations in the scene.
 * This class achieves its effect with a Texture and a correctly configured BackgroundMaterial on an inverted sphere.
 * Potential additions to this helper include zoom and and non-infinite distance rendering effects.
 */
export abstract class TextureDome<T extends Texture> extends TransformNode {
    protected onError: Nullable<(message?: string, exception?: any) => void>;
    /**
     * Define the source as a Monoscopic panoramic 360/180.
     */
    static readonly MODE_MONOSCOPIC: number;
    /**
     * Define the source as a Stereoscopic TopBottom/OverUnder panoramic 360/180.
     */
    static readonly MODE_TOPBOTTOM: number;
    /**
     * Define the source as a Stereoscopic Side by Side panoramic 360/180.
     */
    static readonly MODE_SIDEBYSIDE: number;
    private _halfDome;
    private _crossEye;
    protected _useDirectMapping: boolean;
    /**
     * The texture being displayed on the sphere
     */
    protected _texture: T;
    /**
     * Gets the texture being displayed on the sphere
     */
    get texture(): T;
    /**
     * Sets the texture being displayed on the sphere
     */
    set texture(newTexture: T);
    /**
     * The skybox material
     */
    protected _material: BackgroundMaterial;
    /**
     * The surface used for the dome
     */
    protected _mesh: Mesh;
    /**
     * Gets the mesh used for the dome.
     */
    get mesh(): Mesh;
    /**
     * A mesh that will be used to mask the back of the dome in case it is a 180 degree movie.
     */
    private _halfDomeMask;
    /**
     * The current fov(field of view) multiplier, 0.0 - 2.0. Defaults to 1.0. Lower values "zoom in" and higher values "zoom out".
     * Also see the options.resolution property.
     */
    get fovMultiplier(): number;
    set fovMultiplier(value: number);
    protected _textureMode: number;
    /**
     * Gets or set the current texture mode for the texture. It can be:
     * * TextureDome.MODE_MONOSCOPIC : Define the texture source as a Monoscopic panoramic 360.
     * * TextureDome.MODE_TOPBOTTOM  : Define the texture source as a Stereoscopic TopBottom/OverUnder panoramic 360.
     * * TextureDome.MODE_SIDEBYSIDE : Define the texture source as a Stereoscopic Side by Side panoramic 360.
     */
    get textureMode(): number;
    /**
     * Sets the current texture mode for the texture. It can be:
     * * TextureDome.MODE_MONOSCOPIC : Define the texture source as a Monoscopic panoramic 360.
     * * TextureDome.MODE_TOPBOTTOM  : Define the texture source as a Stereoscopic TopBottom/OverUnder panoramic 360.
     * * TextureDome.MODE_SIDEBYSIDE : Define the texture source as a Stereoscopic Side by Side panoramic 360.
     */
    set textureMode(value: number);
    /**
     * Is it a 180 degrees dome (half dome) or 360 texture (full dome)
     */
    get halfDome(): boolean;
    /**
     * Set the halfDome mode. If set, only the front (180 degrees) will be displayed and the back will be blacked out.
     */
    set halfDome(enabled: boolean);
    /**
     * Set the cross-eye mode. If set, images that can be seen when crossing eyes will render correctly
     */
    set crossEye(enabled: boolean);
    /**
     * Is it a cross-eye texture?
     */
    get crossEye(): boolean;
    /**
     * The background material of this dome.
     */
    get material(): BackgroundMaterial;
    /**
     * Oberserver used in Stereoscopic VR Mode.
     */
    private _onBeforeCameraRenderObserver;
    /**
     * Observable raised when an error occurred while loading the texture
     */
    onLoadErrorObservable: Observable<string>;
    /**
     * Observable raised when the texture finished loading
     */
    onLoadObservable: Observable<void>;
    /**
     * Create an instance of this class and pass through the parameters to the relevant classes- Texture, StandardMaterial, and Mesh.
     * @param name Element's name, child elements will append suffixes for their own names.
     * @param textureUrlOrElement defines the url(s) or the (video) HTML element to use
     * @param options An object containing optional or exposed sub element properties
     * @param options.resolution
     * @param options.clickToPlay
     * @param options.autoPlay
     * @param options.loop
     * @param options.size
     * @param options.poster
     * @param options.faceForward
     * @param options.useDirectMapping
     * @param options.halfDomeMode
     * @param options.crossEyeMode
     * @param options.generateMipMaps
     * @param options.mesh
     * @param scene
     * @param onError
     */
    constructor(name: string, textureUrlOrElement: string | string[] | HTMLVideoElement, options: {
        resolution?: number;
        clickToPlay?: boolean;
        autoPlay?: boolean;
        loop?: boolean;
        size?: number;
        poster?: string;
        faceForward?: boolean;
        useDirectMapping?: boolean;
        halfDomeMode?: boolean;
        crossEyeMode?: boolean;
        generateMipMaps?: boolean;
        mesh?: Mesh;
    }, scene: Scene, onError?: Nullable<(message?: string, exception?: any) => void>);
    protected abstract _initTexture(urlsOrElement: string | string[] | HTMLElement, scene: Scene, options: any): T;
    protected _changeTextureMode(value: number): void;
    /**
     * Releases resources associated with this node.
     * @param doNotRecurse Set to true to not recurse into each children (recurse into each children by default)
     * @param disposeMaterialAndTextures Set to true to also dispose referenced materials and textures (false by default)
     */
    dispose(doNotRecurse?: boolean, disposeMaterialAndTextures?: boolean): void;
}

}
declare module "babylonjs/Helpers/videoDome" {
import { Scene } from "babylonjs/scene";
import { VideoTexture } from "babylonjs/Materials/Textures/videoTexture";
import { TextureDome } from "babylonjs/Helpers/textureDome";
/**
 * Display a 360/180 degree video on an approximately spherical surface, useful for VR applications or skyboxes.
 * As a subclass of TransformNode, this allow parenting to the camera or multiple videos with different locations in the scene.
 * This class achieves its effect with a VideoTexture and a correctly configured BackgroundMaterial on an inverted sphere.
 * Potential additions to this helper include zoom and and non-infinite distance rendering effects.
 */
export class VideoDome extends TextureDome<VideoTexture> {
    /**
     * Define the video source as a Monoscopic panoramic 360 video.
     */
    static readonly MODE_MONOSCOPIC: number;
    /**
     * Define the video source as a Stereoscopic TopBottom/OverUnder panoramic 360 video.
     */
    static readonly MODE_TOPBOTTOM: number;
    /**
     * Define the video source as a Stereoscopic Side by Side panoramic 360 video.
     */
    static readonly MODE_SIDEBYSIDE: number;
    /**
     * Get the video texture associated with this video dome
     */
    get videoTexture(): VideoTexture;
    /**
     * Get the video mode of this dome
     */
    get videoMode(): number;
    /**
     * Set the video mode of this dome.
     * @see textureMode
     */
    set videoMode(value: number);
    private _pointerObserver;
    private _textureObserver;
    protected _initTexture(urlsOrElement: string | string[] | HTMLVideoElement, scene: Scene, options: any): VideoTexture;
    /**
     * Releases resources associated with this node.
     * @param doNotRecurse Set to true to not recurse into each children (recurse into each children by default)
     * @param disposeMaterialAndTextures Set to true to also dispose referenced materials and textures (false by default)
     */
    dispose(doNotRecurse?: boolean, disposeMaterialAndTextures?: boolean): void;
}

}
declare module "babylonjs/IAccessibilityTag" {
/**
 * Define an interface for a node to indicate it's info for accessibility.
 * By default, Node type doesn't imply accessibility info unless this tag is assigned. Whereas GUI controls already indicate accessibility info, but one can override the info using this tag.
 */
export interface IAccessibilityTag {
    /**
     * A string as alt text of the node, describing what the node is/does, for accessibility purpose.
     */
    description?: string;
    /**
     * Customize the event of the accessible object.
     * This will be applied on the generated HTML twin node.
     */
    eventHandler?: {
        [key in keyof HTMLElementEventMap]: (e?: Event) => void;
    };
    /**
     * ARIA roles and attributes to customize accessibility support.
     * If you use BabylonJS's accessibility html twin renderer, and want to override the default behavior (not suggested), this can be your way.
     * Learn more about ARIA: https://developer.mozilla.org/en-US/docs/Web/Accessibility/ARIA
     */
    role?: AcceptedRole;
    aria?: {
        [key in AcceptedARIA]: any;
    };
}
type AcceptedRole = "toolbar" | "tooltip" | "feed" | "math" | "presentation" | "none" | "note" | "application" | "article" | "cell" | "columnheader" | "definition" | "directory" | "document" | "figure" | "group" | "heading" | "img" | "list" | "listitem" | "meter" | "row" | "rowgroup" | "rowheader" | "separator" | "table" | "term" | "scrollbar" | "searchbox" | "separator" | "slider" | "spinbutton" | "switch" | "tab" | "tabpanel" | "treeitem" | "combobox" | "menu" | "menubar" | "tablist" | "tree" | "treegrid" | "banner" | "complementary" | "contentinfo" | "form" | "main" | "navigation" | "region" | "search" | "alert" | "log" | "marquee" | "status" | "timer" | "alertdialog" | "dialog";
type AcceptedARIA = "aria-autocomplete" | "aria-checked" | "aria-disabled" | "aria-errormessage" | "aria-expanded" | "aria-haspopup" | "aria-hidden" | "aria-invalid" | "aria-label" | "aria-level" | "aria-modal" | "aria-multiline" | "aria-multiselectable" | "aria-orientation" | "aria-placeholder" | "aria-pressed" | "aria-readonly" | "aria-required" | "aria-selected" | "aria-sort" | "aria-valuemax" | "aria-valuemin" | "aria-valuenow" | "aria-valuetext" | "aria-busy" | "aria-live" | "aria-relevant" | "aria-atomic" | "aria-dropeffect" | "aria-grabbed" | "aria-activedescendant" | "aria-colcount" | "aria-colindex" | "aria-colspan" | "aria-controls" | "aria-describedby" | "aria-description" | "aria-details" | "aria-errormessage" | "aria-flowto" | "aria-labelledby" | "aria-owns" | "aria-posinset" | "aria-rowcount" | "aria-rowindex" | "aria-rowspan" | "aria-setsize";
export {};

}
declare module "babylonjs/index" {
export * from "babylonjs/abstractScene";
export * from "babylonjs/Actions/index";
export * from "babylonjs/Animations/index";
export * from "babylonjs/assetContainer";
export * from "babylonjs/Audio/index";
export * from "babylonjs/BakedVertexAnimation/index";
export * from "babylonjs/Behaviors/index";
export * from "babylonjs/Bones/index";
export * from "babylonjs/Buffers/index";
export * from "babylonjs/Cameras/index";
export * from "babylonjs/Collisions/index";
export * from "babylonjs/Compute/index";
export * from "babylonjs/Culling/index";
export * from "babylonjs/Debug/index";
export * from "babylonjs/DeviceInput/index";
export * from "babylonjs/Engines/index";
export * from "babylonjs/Events/index";
export * from "babylonjs/Gamepads/index";
export * from "babylonjs/Gizmos/index";
export * from "babylonjs/Helpers/index";
export * from "babylonjs/Instrumentation/index";
export * from "babylonjs/Layers/index";
export * from "babylonjs/LensFlares/index";
export * from "babylonjs/Lights/index";
export * from "babylonjs/Loading/index";
export * from "babylonjs/Materials/index";
export * from "babylonjs/Maths/index";
export * from "babylonjs/Meshes/index";
export * from "babylonjs/Morph/index";
export * from "babylonjs/Navigation/index";
export * from "babylonjs/node";
export * from "babylonjs/Offline/index";
export * from "babylonjs/Particles/index";
export * from "babylonjs/Physics/index";
export * from "babylonjs/PostProcesses/index";
export * from "babylonjs/Probes/index";
export * from "babylonjs/Rendering/index";
export * from "babylonjs/scene";
export * from "babylonjs/sceneComponent";
export * from "babylonjs/Sprites/index";
export * from "babylonjs/States/index";
export * from "babylonjs/Misc/index";
export * from "babylonjs/XR/index";
export * from "babylonjs/types";
export * from "babylonjs/Compat/index";

}
declare module "babylonjs/Inputs/scene.inputManager" {
import { EventState, Observer } from "babylonjs/Misc/observable";
import { PointerInfo } from "babylonjs/Events/pointerEvents";
import { Nullable } from "babylonjs/types";
import { PickingInfo } from "babylonjs/Collisions/pickingInfo";
import { Vector2 } from "babylonjs/Maths/math.vector";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { IMouseEvent } from "babylonjs/Events/deviceInputEvents";
import { Scene } from "babylonjs/scene";
/**
 * Class used to manage all inputs for the scene.
 */
export class InputManager {
    /** The distance in pixel that you have to move to prevent some events */
    static DragMovementThreshold: number;
    /** Time in milliseconds to wait to raise long press events if button is still pressed */
    static LongPressDelay: number;
    /** Time in milliseconds with two consecutive clicks will be considered as a double click */
    static DoubleClickDelay: number;
    /** If you need to check double click without raising a single click at first click, enable this flag */
    static ExclusiveDoubleClickMode: boolean;
    /** This is a defensive check to not allow control attachment prior to an already active one. If already attached, previous control is unattached before attaching the new one. */
    private _alreadyAttached;
    private _alreadyAttachedTo;
    private _onPointerMove;
    private _onPointerDown;
    private _onPointerUp;
    private _initClickEvent;
    private _initActionManager;
    private _delayedSimpleClick;
    private _delayedSimpleClickTimeout;
    private _previousDelayedSimpleClickTimeout;
    private _meshPickProceed;
    private _previousButtonPressed;
    private _currentPickResult;
    private _previousPickResult;
    private _totalPointersPressed;
    private _doubleClickOccured;
    private _isSwiping;
    private _swipeButtonPressed;
    private _skipPointerTap;
    private _isMultiTouchGesture;
    private _pointerOverMesh;
    private _pickedDownMesh;
    private _pickedUpMesh;
    private _pointerX;
    private _pointerY;
    private _unTranslatedPointerX;
    private _unTranslatedPointerY;
    private _startingPointerPosition;
    private _previousStartingPointerPosition;
    private _startingPointerTime;
    private _previousStartingPointerTime;
    private _pointerCaptures;
    private _meshUnderPointerId;
    private _movePointerInfo;
    private _cameraObserverCount;
    private _onKeyDown;
    private _onKeyUp;
    private _scene;
    private _deviceSourceManager;
    /**
     * Creates a new InputManager
     * @param scene - defines the hosting scene
     */
    constructor(scene?: Scene);
    /**
     * Gets the mesh that is currently under the pointer
     * @returns Mesh that the pointer is pointer is hovering over
     */
    get meshUnderPointer(): Nullable<AbstractMesh>;
    /**
     * When using more than one pointer (for example in XR) you can get the mesh under the specific pointer
     * @param pointerId - the pointer id to use
     * @returns The mesh under this pointer id or null if not found
     */
    getMeshUnderPointerByPointerId(pointerId: number): Nullable<AbstractMesh>;
    /**
     * Gets the pointer coordinates in 2D without any translation (ie. straight out of the pointer event)
     * @returns Vector with X/Y values directly from pointer event
     */
    get unTranslatedPointer(): Vector2;
    /**
     * Gets or sets the current on-screen X position of the pointer
     * @returns Translated X with respect to screen
     */
    get pointerX(): number;
    set pointerX(value: number);
    /**
     * Gets or sets the current on-screen Y position of the pointer
     * @returns Translated Y with respect to screen
     */
    get pointerY(): number;
    set pointerY(value: number);
    private _updatePointerPosition;
    private _processPointerMove;
    /** @internal */
    _setRayOnPointerInfo(pickInfo: Nullable<PickingInfo>, event: IMouseEvent): void;
    /** @internal */
    _addCameraPointerObserver(observer: (p: PointerInfo, s: EventState) => void, mask?: number): Nullable<Observer<PointerInfo>>;
    /** @internal */
    _removeCameraPointerObserver(observer: Observer<PointerInfo>): boolean;
    private _checkForPicking;
    private _checkPrePointerObservable;
    /** @internal */
    _pickMove(pointerId: number): PickingInfo;
    private _setCursorAndPointerOverMesh;
    /**
     * Use this method to simulate a pointer move on a mesh
     * The pickResult parameter can be obtained from a scene.pick or scene.pickWithRay
     * @param pickResult - pickingInfo of the object wished to simulate pointer event on
     * @param pointerEventInit - pointer event state to be used when simulating the pointer event (eg. pointer id for multitouch)
     */
    simulatePointerMove(pickResult: PickingInfo, pointerEventInit?: PointerEventInit): void;
    /**
     * Use this method to simulate a pointer down on a mesh
     * The pickResult parameter can be obtained from a scene.pick or scene.pickWithRay
     * @param pickResult - pickingInfo of the object wished to simulate pointer event on
     * @param pointerEventInit - pointer event state to be used when simulating the pointer event (eg. pointer id for multitouch)
     */
    simulatePointerDown(pickResult: PickingInfo, pointerEventInit?: PointerEventInit): void;
    private _processPointerDown;
    /**
     * @internal
     * @internals Boolean if delta for pointer exceeds drag movement threshold
     */
    _isPointerSwiping(): boolean;
    /**
     * Use this method to simulate a pointer up on a mesh
     * The pickResult parameter can be obtained from a scene.pick or scene.pickWithRay
     * @param pickResult - pickingInfo of the object wished to simulate pointer event on
     * @param pointerEventInit - pointer event state to be used when simulating the pointer event (eg. pointer id for multitouch)
     * @param doubleTap - indicates that the pointer up event should be considered as part of a double click (false by default)
     */
    simulatePointerUp(pickResult: PickingInfo, pointerEventInit?: PointerEventInit, doubleTap?: boolean): void;
    private _processPointerUp;
    /**
     * Gets a boolean indicating if the current pointer event is captured (meaning that the scene has already handled the pointer down)
     * @param pointerId - defines the pointer id to use in a multi-touch scenario (0 by default)
     * @returns true if the pointer was captured
     */
    isPointerCaptured(pointerId?: number): boolean;
    /**
     * Attach events to the canvas (To handle actionManagers triggers and raise onPointerMove, onPointerDown and onPointerUp
     * @param attachUp - defines if you want to attach events to pointerup
     * @param attachDown - defines if you want to attach events to pointerdown
     * @param attachMove - defines if you want to attach events to pointermove
     * @param elementToAttachTo - defines the target DOM element to attach to (will use the canvas by default)
     */
    attachControl(attachUp?: boolean, attachDown?: boolean, attachMove?: boolean, elementToAttachTo?: Nullable<HTMLElement>): void;
    /**
     * Detaches all event handlers
     */
    detachControl(): void;
    /**
     * Force the value of meshUnderPointer
     * @param mesh - defines the mesh to use
     * @param pointerId - optional pointer id when using more than one pointer. Defaults to 0
     * @param pickResult - optional pickingInfo data used to find mesh
     */
    setPointerOverMesh(mesh: Nullable<AbstractMesh>, pointerId?: number, pickResult?: Nullable<PickingInfo>): void;
    /**
     * Gets the mesh under the pointer
     * @returns a Mesh or null if no mesh is under the pointer
     */
    getPointerOverMesh(): Nullable<AbstractMesh>;
    /**
     * @param mesh - Mesh to invalidate
     * @internal
     */
    _invalidateMesh(mesh: AbstractMesh): void;
}
export {};

}
declare module "babylonjs/Instrumentation/engineInstrumentation" {
import { PerfCounter } from "babylonjs/Misc/perfCounter";
import { IDisposable } from "babylonjs/scene";
import { Engine } from "babylonjs/Engines/engine";
/**
 * This class can be used to get instrumentation data from a Babylon engine
 * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/optimize_your_scene#engineinstrumentation
 */
export class EngineInstrumentation implements IDisposable {
    /**
     * Define the instrumented engine.
     */
    engine: Engine;
    private _captureGPUFrameTime;
    private _captureShaderCompilationTime;
    private _shaderCompilationTime;
    private _onBeginFrameObserver;
    private _onEndFrameObserver;
    private _onBeforeShaderCompilationObserver;
    private _onAfterShaderCompilationObserver;
    /**
     * Gets the perf counter used for GPU frame time
     */
    get gpuFrameTimeCounter(): PerfCounter;
    /**
     * Gets the GPU frame time capture status
     */
    get captureGPUFrameTime(): boolean;
    /**
     * Enable or disable the GPU frame time capture
     */
    set captureGPUFrameTime(value: boolean);
    /**
     * Gets the perf counter used for shader compilation time
     */
    get shaderCompilationTimeCounter(): PerfCounter;
    /**
     * Gets the shader compilation time capture status
     */
    get captureShaderCompilationTime(): boolean;
    /**
     * Enable or disable the shader compilation time capture
     */
    set captureShaderCompilationTime(value: boolean);
    /**
     * Instantiates a new engine instrumentation.
     * This class can be used to get instrumentation data from a Babylon engine
     * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/optimize_your_scene#engineinstrumentation
     * @param engine Defines the engine to instrument
     */
    constructor(
    /**
     * Define the instrumented engine.
     */
    engine: Engine);
    /**
     * Dispose and release associated resources.
     */
    dispose(): void;
}

}
declare module "babylonjs/Instrumentation/index" {
export * from "babylonjs/Instrumentation/engineInstrumentation";
export * from "babylonjs/Instrumentation/sceneInstrumentation";
export * from "babylonjs/Instrumentation/timeToken";

}
declare module "babylonjs/Instrumentation/sceneInstrumentation" {
import { Scene, IDisposable } from "babylonjs/scene";
import { PerfCounter } from "babylonjs/Misc/perfCounter";
/**
 * This class can be used to get instrumentation data from a Babylon engine
 * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/optimize_your_scene#sceneinstrumentation
 */
export class SceneInstrumentation implements IDisposable {
    /**
     * Defines the scene to instrument
     */
    scene: Scene;
    private _captureActiveMeshesEvaluationTime;
    private _activeMeshesEvaluationTime;
    private _captureRenderTargetsRenderTime;
    private _renderTargetsRenderTime;
    private _captureFrameTime;
    private _frameTime;
    private _captureRenderTime;
    private _renderTime;
    private _captureInterFrameTime;
    private _interFrameTime;
    private _captureParticlesRenderTime;
    private _particlesRenderTime;
    private _captureSpritesRenderTime;
    private _spritesRenderTime;
    private _capturePhysicsTime;
    private _physicsTime;
    private _captureAnimationsTime;
    private _animationsTime;
    private _captureCameraRenderTime;
    private _cameraRenderTime;
    private _onBeforeActiveMeshesEvaluationObserver;
    private _onAfterActiveMeshesEvaluationObserver;
    private _onBeforeRenderTargetsRenderObserver;
    private _onAfterRenderTargetsRenderObserver;
    private _onAfterRenderObserver;
    private _onBeforeDrawPhaseObserver;
    private _onAfterDrawPhaseObserver;
    private _onBeforeAnimationsObserver;
    private _onBeforeParticlesRenderingObserver;
    private _onAfterParticlesRenderingObserver;
    private _onBeforeSpritesRenderingObserver;
    private _onAfterSpritesRenderingObserver;
    private _onBeforePhysicsObserver;
    private _onAfterPhysicsObserver;
    private _onAfterAnimationsObserver;
    private _onBeforeCameraRenderObserver;
    private _onAfterCameraRenderObserver;
    /**
     * Gets the perf counter used for active meshes evaluation time
     */
    get activeMeshesEvaluationTimeCounter(): PerfCounter;
    /**
     * Gets the active meshes evaluation time capture status
     */
    get captureActiveMeshesEvaluationTime(): boolean;
    /**
     * Enable or disable the active meshes evaluation time capture
     */
    set captureActiveMeshesEvaluationTime(value: boolean);
    /**
     * Gets the perf counter used for render targets render time
     */
    get renderTargetsRenderTimeCounter(): PerfCounter;
    /**
     * Gets the render targets render time capture status
     */
    get captureRenderTargetsRenderTime(): boolean;
    /**
     * Enable or disable the render targets render time capture
     */
    set captureRenderTargetsRenderTime(value: boolean);
    /**
     * Gets the perf counter used for particles render time
     */
    get particlesRenderTimeCounter(): PerfCounter;
    /**
     * Gets the particles render time capture status
     */
    get captureParticlesRenderTime(): boolean;
    /**
     * Enable or disable the particles render time capture
     */
    set captureParticlesRenderTime(value: boolean);
    /**
     * Gets the perf counter used for sprites render time
     */
    get spritesRenderTimeCounter(): PerfCounter;
    /**
     * Gets the sprites render time capture status
     */
    get captureSpritesRenderTime(): boolean;
    /**
     * Enable or disable the sprites render time capture
     */
    set captureSpritesRenderTime(value: boolean);
    /**
     * Gets the perf counter used for physics time
     */
    get physicsTimeCounter(): PerfCounter;
    /**
     * Gets the physics time capture status
     */
    get capturePhysicsTime(): boolean;
    /**
     * Enable or disable the physics time capture
     */
    set capturePhysicsTime(value: boolean);
    /**
     * Gets the perf counter used for animations time
     */
    get animationsTimeCounter(): PerfCounter;
    /**
     * Gets the animations time capture status
     */
    get captureAnimationsTime(): boolean;
    /**
     * Enable or disable the animations time capture
     */
    set captureAnimationsTime(value: boolean);
    /**
     * Gets the perf counter used for frame time capture
     */
    get frameTimeCounter(): PerfCounter;
    /**
     * Gets the frame time capture status
     */
    get captureFrameTime(): boolean;
    /**
     * Enable or disable the frame time capture
     */
    set captureFrameTime(value: boolean);
    /**
     * Gets the perf counter used for inter-frames time capture
     */
    get interFrameTimeCounter(): PerfCounter;
    /**
     * Gets the inter-frames time capture status
     */
    get captureInterFrameTime(): boolean;
    /**
     * Enable or disable the inter-frames time capture
     */
    set captureInterFrameTime(value: boolean);
    /**
     * Gets the perf counter used for render time capture
     */
    get renderTimeCounter(): PerfCounter;
    /**
     * Gets the render time capture status
     */
    get captureRenderTime(): boolean;
    /**
     * Enable or disable the render time capture
     */
    set captureRenderTime(value: boolean);
    /**
     * Gets the perf counter used for camera render time capture
     */
    get cameraRenderTimeCounter(): PerfCounter;
    /**
     * Gets the camera render time capture status
     */
    get captureCameraRenderTime(): boolean;
    /**
     * Enable or disable the camera render time capture
     */
    set captureCameraRenderTime(value: boolean);
    /**
     * Gets the perf counter used for draw calls
     */
    get drawCallsCounter(): PerfCounter;
    /**
     * Instantiates a new scene instrumentation.
     * This class can be used to get instrumentation data from a Babylon engine
     * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/optimize_your_scene#sceneinstrumentation
     * @param scene Defines the scene to instrument
     */
    constructor(
    /**
     * Defines the scene to instrument
     */
    scene: Scene);
    /**
     * Dispose and release associated resources.
     */
    dispose(): void;
}

}
declare module "babylonjs/Instrumentation/timeToken" {
import { Nullable } from "babylonjs/types";
/**
 * @internal
 **/
export class _TimeToken {
    _startTimeQuery: Nullable<WebGLQuery>;
    _endTimeQuery: Nullable<WebGLQuery>;
    _timeElapsedQuery: Nullable<WebGLQuery>;
    _timeElapsedQueryEnded: boolean;
}

}
declare module "babylonjs/Layers/effectLayer" {
import { Observable } from "babylonjs/Misc/observable";
import { Nullable } from "babylonjs/types";
import { Camera } from "babylonjs/Cameras/camera";
import { Scene } from "babylonjs/scene";
import { ISize } from "babylonjs/Maths/math.size";
import { Color4 } from "babylonjs/Maths/math.color";
import { Engine } from "babylonjs/Engines/engine";
import { SubMesh } from "babylonjs/Meshes/subMesh";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { Mesh } from "babylonjs/Meshes/mesh";
import { PostProcess } from "babylonjs/PostProcesses/postProcess";
import { BaseTexture } from "babylonjs/Materials/Textures/baseTexture";
import { RenderTargetTexture } from "babylonjs/Materials/Textures/renderTargetTexture";
import { Effect } from "babylonjs/Materials/effect";
import { Material } from "babylonjs/Materials/material";
import "babylonjs/Shaders/glowMapGeneration.fragment";
import "babylonjs/Shaders/glowMapGeneration.vertex";
/**
 * Effect layer options. This helps customizing the behaviour
 * of the effect layer.
 */
export interface IEffectLayerOptions {
    /**
     * Multiplication factor apply to the canvas size to compute the render target size
     * used to generated the objects (the smaller the faster). Default: 0.5
     */
    mainTextureRatio: number;
    /**
     * Enforces a fixed size texture to ensure effect stability across devices. Default: undefined
     */
    mainTextureFixedSize?: number;
    /**
     * Alpha blending mode used to apply the blur. Default depends of the implementation. Default: ALPHA_COMBINE
     */
    alphaBlendingMode: number;
    /**
     * The camera attached to the layer. Default: null
     */
    camera: Nullable<Camera>;
    /**
     * The rendering group to draw the layer in. Default: -1
     */
    renderingGroupId: number;
    /**
     * The type of the main texture. Default: TEXTURETYPE_UNSIGNED_INT
     */
    mainTextureType: number;
}
/**
 * The effect layer Helps adding post process effect blended with the main pass.
 *
 * This can be for instance use to generate glow or highlight effects on the scene.
 *
 * The effect layer class can not be used directly and is intented to inherited from to be
 * customized per effects.
 */
export abstract class EffectLayer {
    private _vertexBuffers;
    private _indexBuffer;
    private _effectLayerOptions;
    private _mergeDrawWrapper;
    protected _scene: Scene;
    protected _engine: Engine;
    protected _maxSize: number;
    protected _mainTextureDesiredSize: ISize;
    protected _mainTexture: RenderTargetTexture;
    protected _shouldRender: boolean;
    protected _postProcesses: PostProcess[];
    protected _textures: BaseTexture[];
    protected _emissiveTextureAndColor: {
        texture: Nullable<BaseTexture>;
        color: Color4;
    };
    /**
     * The name of the layer
     */
    name: string;
    /**
     * The clear color of the texture used to generate the glow map.
     */
    neutralColor: Color4;
    /**
     * Specifies whether the highlight layer is enabled or not.
     */
    isEnabled: boolean;
    /**
     * Gets the camera attached to the layer.
     */
    get camera(): Nullable<Camera>;
    /**
     * Gets the rendering group id the layer should render in.
     */
    get renderingGroupId(): number;
    set renderingGroupId(renderingGroupId: number);
    /**
     * Specifies if the bounding boxes should be rendered normally or if they should undergo the effect of the layer
     */
    disableBoundingBoxesFromEffectLayer: boolean;
    /**
     * An event triggered when the effect layer has been disposed.
     */
    onDisposeObservable: Observable<EffectLayer>;
    /**
     * An event triggered when the effect layer is about rendering the main texture with the glowy parts.
     */
    onBeforeRenderMainTextureObservable: Observable<EffectLayer>;
    /**
     * An event triggered when the generated texture is being merged in the scene.
     */
    onBeforeComposeObservable: Observable<EffectLayer>;
    /**
     * An event triggered when the mesh is rendered into the effect render target.
     */
    onBeforeRenderMeshToEffect: Observable<AbstractMesh>;
    /**
     * An event triggered after the mesh has been rendered into the effect render target.
     */
    onAfterRenderMeshToEffect: Observable<AbstractMesh>;
    /**
     * An event triggered when the generated texture has been merged in the scene.
     */
    onAfterComposeObservable: Observable<EffectLayer>;
    /**
     * An event triggered when the effect layer changes its size.
     */
    onSizeChangedObservable: Observable<EffectLayer>;
    /**
     * Gets the main texture where the effect is rendered
     */
    get mainTexture(): RenderTargetTexture;
    /**
     * @internal
     */
    static _SceneComponentInitialization: (scene: Scene) => void;
    private _materialForRendering;
    /**
     * Sets a specific material to be used to render a mesh/a list of meshes in the layer
     * @param mesh mesh or array of meshes
     * @param material material to use by the layer when rendering the mesh(es). If undefined is passed, the specific material created by the layer will be used.
     */
    setMaterialForRendering(mesh: AbstractMesh | AbstractMesh[], material?: Material): void;
    /**
     * Instantiates a new effect Layer and references it in the scene.
     * @param name The name of the layer
     * @param scene The scene to use the layer in
     */
    constructor(
    /** The Friendly of the effect in the scene */
    name: string, scene?: Scene);
    /**
     * Get the effect name of the layer.
     * @returns The effect name
     */
    abstract getEffectName(): string;
    /**
     * Checks for the readiness of the element composing the layer.
     * @param subMesh the mesh to check for
     * @param useInstances specify whether or not to use instances to render the mesh
     * @returns true if ready otherwise, false
     */
    abstract isReady(subMesh: SubMesh, useInstances: boolean): boolean;
    /**
     * Returns whether or not the layer needs stencil enabled during the mesh rendering.
     * @returns true if the effect requires stencil during the main canvas render pass.
     */
    abstract needStencil(): boolean;
    /**
     * Create the merge effect. This is the shader use to blit the information back
     * to the main canvas at the end of the scene rendering.
     * @returns The effect containing the shader used to merge the effect on the  main canvas
     */
    protected abstract _createMergeEffect(): Effect;
    /**
     * Creates the render target textures and post processes used in the effect layer.
     */
    protected abstract _createTextureAndPostProcesses(): void;
    /**
     * Implementation specific of rendering the generating effect on the main canvas.
     * @param effect The effect used to render through
     * @param renderNum Index of the _internalRender call (0 for the first time _internalRender is called, 1 for the second time, etc. _internalRender is called the number of times returned by _numInternalDraws())
     */
    protected abstract _internalRender(effect: Effect, renderIndex: number): void;
    /**
     * Sets the required values for both the emissive texture and and the main color.
     */
    protected abstract _setEmissiveTextureAndColor(mesh: Mesh, subMesh: SubMesh, material: Material): void;
    /**
     * Free any resources and references associated to a mesh.
     * Internal use
     * @param mesh The mesh to free.
     */
    abstract _disposeMesh(mesh: Mesh): void;
    /**
     * Serializes this layer (Glow or Highlight for example)
     * @returns a serialized layer object
     */
    abstract serialize?(): any;
    /**
     * Number of times _internalRender will be called. Some effect layers need to render the mesh several times, so they should override this method with the number of times the mesh should be rendered
     * @returns Number of times a mesh must be rendered in the layer
     */
    protected _numInternalDraws(): number;
    /**
     * Initializes the effect layer with the required options.
     * @param options Sets of none mandatory options to use with the layer (see IEffectLayerOptions for more information)
     */
    protected _init(options: Partial<IEffectLayerOptions>): void;
    /**
     * Generates the index buffer of the full screen quad blending to the main canvas.
     */
    private _generateIndexBuffer;
    /**
     * Generates the vertex buffer of the full screen quad blending to the main canvas.
     */
    private _generateVertexBuffer;
    /**
     * Sets the main texture desired size which is the closest power of two
     * of the engine canvas size.
     */
    private _setMainTextureSize;
    /**
     * Creates the main texture for the effect layer.
     */
    protected _createMainTexture(): void;
    /**
     * Adds specific effects defines.
     * @param defines The defines to add specifics to.
     */
    protected _addCustomEffectDefines(defines: string[]): void;
    /**
     * Checks for the readiness of the element composing the layer.
     * @param subMesh the mesh to check for
     * @param useInstances specify whether or not to use instances to render the mesh
     * @param emissiveTexture the associated emissive texture used to generate the glow
     * @returns true if ready otherwise, false
     */
    protected _isReady(subMesh: SubMesh, useInstances: boolean, emissiveTexture: Nullable<BaseTexture>): boolean;
    /**
     * Renders the glowing part of the scene by blending the blurred glowing meshes on top of the rendered scene.
     */
    render(): void;
    /**
     * Determine if a given mesh will be used in the current effect.
     * @param mesh mesh to test
     * @returns true if the mesh will be used
     */
    hasMesh(mesh: AbstractMesh): boolean;
    /**
     * Returns true if the layer contains information to display, otherwise false.
     * @returns true if the glow layer should be rendered
     */
    shouldRender(): boolean;
    /**
     * Returns true if the mesh should render, otherwise false.
     * @param mesh The mesh to render
     * @returns true if it should render otherwise false
     */
    protected _shouldRenderMesh(mesh: AbstractMesh): boolean;
    /**
     * Returns true if the mesh can be rendered, otherwise false.
     * @param mesh The mesh to render
     * @param material The material used on the mesh
     * @returns true if it can be rendered otherwise false
     */
    protected _canRenderMesh(mesh: AbstractMesh, material: Material): boolean;
    /**
     * Returns true if the mesh should render, otherwise false.
     * @returns true if it should render otherwise false
     */
    protected _shouldRenderEmissiveTextureForMesh(): boolean;
    /**
     * Renders the submesh passed in parameter to the generation map.
     * @param subMesh
     * @param enableAlphaMode
     */
    protected _renderSubMesh(subMesh: SubMesh, enableAlphaMode?: boolean): void;
    /**
     * Defines whether the current material of the mesh should be use to render the effect.
     * @param mesh defines the current mesh to render
     */
    protected _useMeshMaterial(mesh: AbstractMesh): boolean;
    /**
     * Rebuild the required buffers.
     * @internal Internal use only.
     */
    _rebuild(): void;
    /**
     * Dispose only the render target textures and post process.
     */
    private _disposeTextureAndPostProcesses;
    /**
     * Dispose the highlight layer and free resources.
     */
    dispose(): void;
    /**
     * Gets the class name of the effect layer
     * @returns the string with the class name of the effect layer
     */
    getClassName(): string;
    /**
     * Creates an effect layer from parsed effect layer data
     * @param parsedEffectLayer defines effect layer data
     * @param scene defines the current scene
     * @param rootUrl defines the root URL containing the effect layer information
     * @returns a parsed effect Layer
     */
    static Parse(parsedEffectLayer: any, scene: Scene, rootUrl: string): EffectLayer;
}

}
declare module "babylonjs/Layers/effectLayerSceneComponent" {
import { Scene } from "babylonjs/scene";
import { ISceneSerializableComponent } from "babylonjs/sceneComponent";
import { EffectLayer } from "babylonjs/Layers/effectLayer";
import { AbstractScene } from "babylonjs/abstractScene";
module "babylonjs/abstractScene" {
    interface AbstractScene {
        /**
         * The list of effect layers (highlights/glow) added to the scene
         * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/highlightLayer
         * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/glowLayer
         */
        effectLayers: Array<EffectLayer>;
        /**
         * Removes the given effect layer from this scene.
         * @param toRemove defines the effect layer to remove
         * @returns the index of the removed effect layer
         */
        removeEffectLayer(toRemove: EffectLayer): number;
        /**
         * Adds the given effect layer to this scene
         * @param newEffectLayer defines the effect layer to add
         */
        addEffectLayer(newEffectLayer: EffectLayer): void;
    }
}
/**
 * Defines the layer scene component responsible to manage any effect layers
 * in a given scene.
 */
export class EffectLayerSceneComponent implements ISceneSerializableComponent {
    /**
     * The component name helpful to identify the component in the list of scene components.
     */
    readonly name: string;
    /**
     * The scene the component belongs to.
     */
    scene: Scene;
    private _engine;
    private _renderEffects;
    private _needStencil;
    private _previousStencilState;
    /**
     * Creates a new instance of the component for the given scene
     * @param scene Defines the scene to register the component in
     */
    constructor(scene?: Scene);
    /**
     * Registers the component in a given scene
     */
    register(): void;
    /**
     * Rebuilds the elements related to this component in case of
     * context lost for instance.
     */
    rebuild(): void;
    /**
     * Serializes the component data to the specified json object
     * @param serializationObject The object to serialize to
     */
    serialize(serializationObject: any): void;
    /**
     * Adds all the elements from the container to the scene
     * @param container the container holding the elements
     */
    addFromContainer(container: AbstractScene): void;
    /**
     * Removes all the elements in the container from the scene
     * @param container contains the elements to remove
     * @param dispose if the removed element should be disposed (default: false)
     */
    removeFromContainer(container: AbstractScene, dispose?: boolean): void;
    /**
     * Disposes the component and the associated resources.
     */
    dispose(): void;
    private _isReadyForMesh;
    private _renderMainTexture;
    private _setStencil;
    private _setStencilBack;
    private _draw;
    private _drawCamera;
    private _drawRenderingGroup;
}

}
declare module "babylonjs/Layers/glowLayer" {
import { Nullable } from "babylonjs/types";
import { Camera } from "babylonjs/Cameras/camera";
import { Scene } from "babylonjs/scene";
import { SubMesh } from "babylonjs/Meshes/subMesh";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { Mesh } from "babylonjs/Meshes/mesh";
import { Texture } from "babylonjs/Materials/Textures/texture";
import { Effect } from "babylonjs/Materials/effect";
import { Material } from "babylonjs/Materials/material";
import { EffectLayer } from "babylonjs/Layers/effectLayer";
import { Color4 } from "babylonjs/Maths/math.color";
import "babylonjs/Shaders/glowMapMerge.fragment";
import "babylonjs/Shaders/glowMapMerge.vertex";
import "babylonjs/Layers/effectLayerSceneComponent";
module "babylonjs/abstractScene" {
    interface AbstractScene {
        /**
         * Return the first glow layer of the scene with a given name.
         * @param name The name of the glow layer to look for.
         * @returns The glow layer if found otherwise null.
         */
        getGlowLayerByName(name: string): Nullable<GlowLayer>;
    }
}
/**
 * Glow layer options. This helps customizing the behaviour
 * of the glow layer.
 */
export interface IGlowLayerOptions {
    /**
     * Multiplication factor apply to the canvas size to compute the render target size
     * used to generated the glowing objects (the smaller the faster). Default: 0.5
     */
    mainTextureRatio: number;
    /**
     * Enforces a fixed size texture to ensure resize independent blur. Default: undefined
     */
    mainTextureFixedSize?: number;
    /**
     * How big is the kernel of the blur texture. Default: 32
     */
    blurKernelSize: number;
    /**
     * The camera attached to the layer. Default: null
     */
    camera: Nullable<Camera>;
    /**
     * Enable MSAA by choosing the number of samples. Default: 1
     */
    mainTextureSamples?: number;
    /**
     * The rendering group to draw the layer in. Default: -1
     */
    renderingGroupId: number;
    /**
     * Forces the merge step to be done in ldr (clamp values > 1). Default: false
     */
    ldrMerge?: boolean;
    /**
     * Defines the blend mode used by the merge. Default: ALPHA_ADD
     */
    alphaBlendingMode?: number;
    /**
     * The type of the main texture. Default: TEXTURETYPE_UNSIGNED_INT
     */
    mainTextureType: number;
}
/**
 * The glow layer Helps adding a glow effect around the emissive parts of a mesh.
 *
 * Once instantiated in a scene, by default, all the emissive meshes will glow.
 *
 * Documentation: https://doc.babylonjs.com/features/featuresDeepDive/mesh/glowLayer
 */
export class GlowLayer extends EffectLayer {
    /**
     * Effect Name of the layer.
     */
    static readonly EffectName: string;
    /**
     * The default blur kernel size used for the glow.
     */
    static DefaultBlurKernelSize: number;
    /**
     * The default texture size ratio used for the glow.
     */
    static DefaultTextureRatio: number;
    /**
     * Sets the kernel size of the blur.
     */
    set blurKernelSize(value: number);
    /**
     * Gets the kernel size of the blur.
     */
    get blurKernelSize(): number;
    /**
     * Sets the glow intensity.
     */
    set intensity(value: number);
    /**
     * Gets the glow intensity.
     */
    get intensity(): number;
    private _options;
    private _intensity;
    private _horizontalBlurPostprocess1;
    private _verticalBlurPostprocess1;
    private _horizontalBlurPostprocess2;
    private _verticalBlurPostprocess2;
    private _blurTexture1;
    private _blurTexture2;
    private _postProcesses1;
    private _postProcesses2;
    private _includedOnlyMeshes;
    private _excludedMeshes;
    private _meshesUsingTheirOwnMaterials;
    /**
     * Callback used to let the user override the color selection on a per mesh basis
     */
    customEmissiveColorSelector: (mesh: Mesh, subMesh: SubMesh, material: Material, result: Color4) => void;
    /**
     * Callback used to let the user override the texture selection on a per mesh basis
     */
    customEmissiveTextureSelector: (mesh: Mesh, subMesh: SubMesh, material: Material) => Texture;
    /**
     * Instantiates a new glow Layer and references it to the scene.
     * @param name The name of the layer
     * @param scene The scene to use the layer in
     * @param options Sets of none mandatory options to use with the layer (see IGlowLayerOptions for more information)
     */
    constructor(name: string, scene?: Scene, options?: Partial<IGlowLayerOptions>);
    /**
     * Get the effect name of the layer.
     * @returns The effect name
     */
    getEffectName(): string;
    /**
     * Create the merge effect. This is the shader use to blit the information back
     * to the main canvas at the end of the scene rendering.
     */
    protected _createMergeEffect(): Effect;
    /**
     * Creates the render target textures and post processes used in the glow layer.
     */
    protected _createTextureAndPostProcesses(): void;
    /**
     * @returns The blur kernel size used by the glow.
     * Note: The value passed in the options is divided by 2 for back compatibility.
     */
    private _getEffectiveBlurKernelSize;
    /**
     * Checks for the readiness of the element composing the layer.
     * @param subMesh the mesh to check for
     * @param useInstances specify whether or not to use instances to render the mesh
     * @returns true if ready otherwise, false
     */
    isReady(subMesh: SubMesh, useInstances: boolean): boolean;
    /**
     * Returns whether or not the layer needs stencil enabled during the mesh rendering.
     */
    needStencil(): boolean;
    /**
     * Returns true if the mesh can be rendered, otherwise false.
     * @param mesh The mesh to render
     * @param material The material used on the mesh
     * @returns true if it can be rendered otherwise false
     */
    protected _canRenderMesh(mesh: AbstractMesh, material: Material): boolean;
    /**
     * Implementation specific of rendering the generating effect on the main canvas.
     * @param effect The effect used to render through
     */
    protected _internalRender(effect: Effect): void;
    /**
     * Sets the required values for both the emissive texture and and the main color.
     * @param mesh
     * @param subMesh
     * @param material
     */
    protected _setEmissiveTextureAndColor(mesh: Mesh, subMesh: SubMesh, material: Material): void;
    /**
     * Returns true if the mesh should render, otherwise false.
     * @param mesh The mesh to render
     * @returns true if it should render otherwise false
     */
    protected _shouldRenderMesh(mesh: Mesh): boolean;
    /**
     * Adds specific effects defines.
     * @param defines The defines to add specifics to.
     */
    protected _addCustomEffectDefines(defines: string[]): void;
    /**
     * Add a mesh in the exclusion list to prevent it to impact or being impacted by the glow layer.
     * @param mesh The mesh to exclude from the glow layer
     */
    addExcludedMesh(mesh: Mesh): void;
    /**
     * Remove a mesh from the exclusion list to let it impact or being impacted by the glow layer.
     * @param mesh The mesh to remove
     */
    removeExcludedMesh(mesh: Mesh): void;
    /**
     * Add a mesh in the inclusion list to impact or being impacted by the glow layer.
     * @param mesh The mesh to include in the glow layer
     */
    addIncludedOnlyMesh(mesh: Mesh): void;
    /**
     * Remove a mesh from the Inclusion list to prevent it to impact or being impacted by the glow layer.
     * @param mesh The mesh to remove
     */
    removeIncludedOnlyMesh(mesh: Mesh): void;
    /**
     * Determine if a given mesh will be used in the glow layer
     * @param mesh The mesh to test
     * @returns true if the mesh will be highlighted by the current glow layer
     */
    hasMesh(mesh: AbstractMesh): boolean;
    /**
     * Defines whether the current material of the mesh should be use to render the effect.
     * @param mesh defines the current mesh to render
     */
    protected _useMeshMaterial(mesh: AbstractMesh): boolean;
    /**
     * Add a mesh to be rendered through its own material and not with emissive only.
     * @param mesh The mesh for which we need to use its material
     */
    referenceMeshToUseItsOwnMaterial(mesh: AbstractMesh): void;
    /**
     * Remove a mesh from being rendered through its own material and not with emissive only.
     * @param mesh The mesh for which we need to not use its material
     */
    unReferenceMeshFromUsingItsOwnMaterial(mesh: AbstractMesh): void;
    /**
     * Free any resources and references associated to a mesh.
     * Internal use
     * @param mesh The mesh to free.
     * @internal
     */
    _disposeMesh(mesh: Mesh): void;
    /**
     * Gets the class name of the effect layer
     * @returns the string with the class name of the effect layer
     */
    getClassName(): string;
    /**
     * Serializes this glow layer
     * @returns a serialized glow layer object
     */
    serialize(): any;
    /**
     * Creates a Glow Layer from parsed glow layer data
     * @param parsedGlowLayer defines glow layer data
     * @param scene defines the current scene
     * @param rootUrl defines the root URL containing the glow layer information
     * @returns a parsed Glow Layer
     */
    static Parse(parsedGlowLayer: any, scene: Scene, rootUrl: string): GlowLayer;
}

}
declare module "babylonjs/Layers/highlightLayer" {
import { Observable } from "babylonjs/Misc/observable";
import { Nullable } from "babylonjs/types";
import { Camera } from "babylonjs/Cameras/camera";
import { Scene } from "babylonjs/scene";
import { SubMesh } from "babylonjs/Meshes/subMesh";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { Mesh } from "babylonjs/Meshes/mesh";
import { Effect } from "babylonjs/Materials/effect";
import { Material } from "babylonjs/Materials/material";
import { EffectLayer } from "babylonjs/Layers/effectLayer";
import { Color4, Color3 } from "babylonjs/Maths/math.color";
import "babylonjs/Shaders/glowMapMerge.fragment";
import "babylonjs/Shaders/glowMapMerge.vertex";
import "babylonjs/Shaders/glowBlurPostProcess.fragment";
import "babylonjs/Layers/effectLayerSceneComponent";
module "babylonjs/abstractScene" {
    interface AbstractScene {
        /**
         * Return a the first highlight layer of the scene with a given name.
         * @param name The name of the highlight layer to look for.
         * @returns The highlight layer if found otherwise null.
         */
        getHighlightLayerByName(name: string): Nullable<HighlightLayer>;
    }
}
/**
 * Highlight layer options. This helps customizing the behaviour
 * of the highlight layer.
 */
export interface IHighlightLayerOptions {
    /**
     * Multiplication factor apply to the canvas size to compute the render target size
     * used to generated the glowing objects (the smaller the faster). Default: 0.5
     */
    mainTextureRatio: number;
    /**
     * Enforces a fixed size texture to ensure resize independent blur. Default: undefined
     */
    mainTextureFixedSize?: number;
    /**
     * Multiplication factor apply to the main texture size in the first step of the blur to reduce the size
     * of the picture to blur (the smaller the faster). Default: 0.5
     */
    blurTextureSizeRatio: number;
    /**
     * How big in texel of the blur texture is the vertical blur. Default: 1
     */
    blurVerticalSize: number;
    /**
     * How big in texel of the blur texture is the horizontal blur. Default: 1
     */
    blurHorizontalSize: number;
    /**
     * Alpha blending mode used to apply the blur.  Default: ALPHA_COMBINE
     */
    alphaBlendingMode: number;
    /**
     * The camera attached to the layer. Default: null
     */
    camera: Nullable<Camera>;
    /**
     * Should we display highlight as a solid stroke? Default: false
     */
    isStroke?: boolean;
    /**
     * The rendering group to draw the layer in. Default: -1
     */
    renderingGroupId: number;
    /**
     * The type of the main texture. Default: TEXTURETYPE_UNSIGNED_INT
     */
    mainTextureType: number;
}
/**
 * The highlight layer Helps adding a glow effect around a mesh.
 *
 * Once instantiated in a scene, simply use the addMesh or removeMesh method to add or remove
 * glowy meshes to your scene.
 *
 * !!! THIS REQUIRES AN ACTIVE STENCIL BUFFER ON THE CANVAS !!!
 */
export class HighlightLayer extends EffectLayer {
    name: string;
    /**
     * Effect Name of the highlight layer.
     */
    static readonly EffectName: string;
    /**
     * The neutral color used during the preparation of the glow effect.
     * This is black by default as the blend operation is a blend operation.
     */
    static NeutralColor: Color4;
    /**
     * Stencil value used for glowing meshes.
     */
    static GlowingMeshStencilReference: number;
    /**
     * Stencil value used for the other meshes in the scene.
     */
    static NormalMeshStencilReference: number;
    /**
     * Specifies whether or not the inner glow is ACTIVE in the layer.
     */
    innerGlow: boolean;
    /**
     * Specifies whether or not the outer glow is ACTIVE in the layer.
     */
    outerGlow: boolean;
    /**
     * Specifies the horizontal size of the blur.
     */
    set blurHorizontalSize(value: number);
    /**
     * Specifies the vertical size of the blur.
     */
    set blurVerticalSize(value: number);
    /**
     * Gets the horizontal size of the blur.
     */
    get blurHorizontalSize(): number;
    /**
     * Gets the vertical size of the blur.
     */
    get blurVerticalSize(): number;
    /**
     * An event triggered when the highlight layer is being blurred.
     */
    onBeforeBlurObservable: Observable<HighlightLayer>;
    /**
     * An event triggered when the highlight layer has been blurred.
     */
    onAfterBlurObservable: Observable<HighlightLayer>;
    private _instanceGlowingMeshStencilReference;
    private _options;
    private _downSamplePostprocess;
    private _horizontalBlurPostprocess;
    private _verticalBlurPostprocess;
    private _blurTexture;
    private _meshes;
    private _excludedMeshes;
    /**
     * Instantiates a new highlight Layer and references it to the scene..
     * @param name The name of the layer
     * @param scene The scene to use the layer in
     * @param options Sets of none mandatory options to use with the layer (see IHighlightLayerOptions for more information)
     */
    constructor(name: string, scene?: Scene, options?: Partial<IHighlightLayerOptions>);
    /**
     * Get the effect name of the layer.
     * @returns The effect name
     */
    getEffectName(): string;
    protected _numInternalDraws(): number;
    /**
     * Create the merge effect. This is the shader use to blit the information back
     * to the main canvas at the end of the scene rendering.
     */
    protected _createMergeEffect(): Effect;
    /**
     * Creates the render target textures and post processes used in the highlight layer.
     */
    protected _createTextureAndPostProcesses(): void;
    /**
     * Returns whether or not the layer needs stencil enabled during the mesh rendering.
     */
    needStencil(): boolean;
    /**
     * Checks for the readiness of the element composing the layer.
     * @param subMesh the mesh to check for
     * @param useInstances specify whether or not to use instances to render the mesh
     * @returns true if ready otherwise, false
     */
    isReady(subMesh: SubMesh, useInstances: boolean): boolean;
    /**
     * Implementation specific of rendering the generating effect on the main canvas.
     * @param effect The effect used to render through
     * @param renderIndex
     */
    protected _internalRender(effect: Effect, renderIndex: number): void;
    /**
     * Returns true if the layer contains information to display, otherwise false.
     */
    shouldRender(): boolean;
    /**
     * Returns true if the mesh should render, otherwise false.
     * @param mesh The mesh to render
     * @returns true if it should render otherwise false
     */
    protected _shouldRenderMesh(mesh: Mesh): boolean;
    /**
     * Returns true if the mesh can be rendered, otherwise false.
     * @param mesh The mesh to render
     * @param material The material used on the mesh
     * @returns true if it can be rendered otherwise false
     */
    protected _canRenderMesh(mesh: AbstractMesh, material: Material): boolean;
    /**
     * Adds specific effects defines.
     * @param defines The defines to add specifics to.
     */
    protected _addCustomEffectDefines(defines: string[]): void;
    /**
     * Sets the required values for both the emissive texture and and the main color.
     * @param mesh
     * @param subMesh
     * @param material
     */
    protected _setEmissiveTextureAndColor(mesh: Mesh, subMesh: SubMesh, material: Material): void;
    /**
     * Add a mesh in the exclusion list to prevent it to impact or being impacted by the highlight layer.
     * @param mesh The mesh to exclude from the highlight layer
     */
    addExcludedMesh(mesh: Mesh): void;
    /**
     * Remove a mesh from the exclusion list to let it impact or being impacted by the highlight layer.
     * @param mesh The mesh to highlight
     */
    removeExcludedMesh(mesh: Mesh): void;
    /**
     * Determine if a given mesh will be highlighted by the current HighlightLayer
     * @param mesh mesh to test
     * @returns true if the mesh will be highlighted by the current HighlightLayer
     */
    hasMesh(mesh: AbstractMesh): boolean;
    /**
     * Add a mesh in the highlight layer in order to make it glow with the chosen color.
     * @param mesh The mesh to highlight
     * @param color The color of the highlight
     * @param glowEmissiveOnly Extract the glow from the emissive texture
     */
    addMesh(mesh: Mesh, color: Color3, glowEmissiveOnly?: boolean): void;
    /**
     * Remove a mesh from the highlight layer in order to make it stop glowing.
     * @param mesh The mesh to highlight
     */
    removeMesh(mesh: Mesh): void;
    /**
     * Remove all the meshes currently referenced in the highlight layer
     */
    removeAllMeshes(): void;
    /**
     * Force the stencil to the normal expected value for none glowing parts
     * @param mesh
     */
    private _defaultStencilReference;
    /**
     * Free any resources and references associated to a mesh.
     * Internal use
     * @param mesh The mesh to free.
     * @internal
     */
    _disposeMesh(mesh: Mesh): void;
    /**
     * Dispose the highlight layer and free resources.
     */
    dispose(): void;
    /**
     * Gets the class name of the effect layer
     * @returns the string with the class name of the effect layer
     */
    getClassName(): string;
    /**
     * Serializes this Highlight layer
     * @returns a serialized Highlight layer object
     */
    serialize(): any;
    /**
     * Creates a Highlight layer from parsed Highlight layer data
     * @param parsedHightlightLayer defines the Highlight layer data
     * @param scene defines the current scene
     * @param rootUrl defines the root URL containing the Highlight layer information
     * @returns a parsed Highlight layer
     */
    static Parse(parsedHightlightLayer: any, scene: Scene, rootUrl: string): HighlightLayer;
}

}
declare module "babylonjs/Layers/index" {
export * from "babylonjs/Layers/effectLayer";
export * from "babylonjs/Layers/effectLayerSceneComponent";
export * from "babylonjs/Layers/glowLayer";
export * from "babylonjs/Layers/highlightLayer";
export * from "babylonjs/Layers/layer";
export * from "babylonjs/Layers/layerSceneComponent";

}
declare module "babylonjs/Layers/layer" {
import { Observable } from "babylonjs/Misc/observable";
import { Nullable } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
import { Vector2 } from "babylonjs/Maths/math.vector";
import { Color4 } from "babylonjs/Maths/math.color";
import { BaseTexture } from "babylonjs/Materials/Textures/baseTexture";
import { RenderTargetTexture } from "babylonjs/Materials/Textures/renderTargetTexture";
import "babylonjs/Shaders/layer.fragment";
import "babylonjs/Shaders/layer.vertex";
/**
 * This represents a full screen 2d layer.
 * This can be useful to display a picture in the  background of your scene for instance.
 * @see https://www.babylonjs-playground.com/#08A2BS#1
 */
export class Layer {
    /**
     * Define the name of the layer.
     */
    name: string;
    /**
     * Define the texture the layer should display.
     */
    texture: Nullable<BaseTexture>;
    /**
     * Is the layer in background or foreground.
     */
    isBackground: boolean;
    private _applyPostProcess;
    /**
     * Determines if the layer is drawn before (true) or after (false) post-processing.
     * If the layer is background, it is always before.
     */
    set applyPostProcess(value: boolean);
    get applyPostProcess(): boolean;
    /**
     * Define the color of the layer (instead of texture).
     */
    color: Color4;
    /**
     * Define the scale of the layer in order to zoom in out of the texture.
     */
    scale: Vector2;
    /**
     * Define an offset for the layer in order to shift the texture.
     */
    offset: Vector2;
    /**
     * Define the alpha blending mode used in the layer in case the texture or color has an alpha.
     */
    alphaBlendingMode: number;
    /**
     * Define if the layer should alpha test or alpha blend with the rest of the scene.
     * Alpha test will not mix with the background color in case of transparency.
     * It will either use the texture color or the background depending on the alpha value of the current pixel.
     */
    alphaTest: boolean;
    /**
     * Define a mask to restrict the layer to only some of the scene cameras.
     */
    layerMask: number;
    /**
     * Define the list of render target the layer is visible into.
     */
    renderTargetTextures: RenderTargetTexture[];
    /**
     * Define if the layer is only used in renderTarget or if it also
     * renders in the main frame buffer of the canvas.
     */
    renderOnlyInRenderTargetTextures: boolean;
    /**
     * Define if the layer is enabled (ie. should be displayed). Default: true
     */
    isEnabled: boolean;
    private _scene;
    private _vertexBuffers;
    private _indexBuffer;
    private _drawWrapper;
    private _previousDefines;
    /**
     * An event triggered when the layer is disposed.
     */
    onDisposeObservable: Observable<Layer>;
    private _onDisposeObserver;
    /**
     * Back compatibility with callback before the onDisposeObservable existed.
     * The set callback will be triggered when the layer has been disposed.
     */
    set onDispose(callback: () => void);
    /**
     * An event triggered before rendering the scene
     */
    onBeforeRenderObservable: Observable<Layer>;
    private _onBeforeRenderObserver;
    /**
     * Back compatibility with callback before the onBeforeRenderObservable existed.
     * The set callback will be triggered just before rendering the layer.
     */
    set onBeforeRender(callback: () => void);
    /**
     * An event triggered after rendering the scene
     */
    onAfterRenderObservable: Observable<Layer>;
    private _onAfterRenderObserver;
    /**
     * Back compatibility with callback before the onAfterRenderObservable existed.
     * The set callback will be triggered just after rendering the layer.
     */
    set onAfterRender(callback: () => void);
    /**
     * Instantiates a new layer.
     * This represents a full screen 2d layer.
     * This can be useful to display a picture in the  background of your scene for instance.
     * @see https://www.babylonjs-playground.com/#08A2BS#1
     * @param name Define the name of the layer in the scene
     * @param imgUrl Define the url of the texture to display in the layer
     * @param scene Define the scene the layer belongs to
     * @param isBackground Defines whether the layer is displayed in front or behind the scene
     * @param color Defines a color for the layer
     */
    constructor(
    /**
     * Define the name of the layer.
     */
    name: string, imgUrl: Nullable<string>, scene: Nullable<Scene>, isBackground?: boolean, color?: Color4);
    private _createIndexBuffer;
    /** @internal */
    _rebuild(): void;
    /**
     * Renders the layer in the scene.
     */
    render(): void;
    /**
     * Disposes and releases the associated resources.
     */
    dispose(): void;
}

}
declare module "babylonjs/Layers/layerSceneComponent" {
import { Scene } from "babylonjs/scene";
import { ISceneComponent } from "babylonjs/sceneComponent";
import { Layer } from "babylonjs/Layers/layer";
import { AbstractScene } from "babylonjs/abstractScene";
module "babylonjs/abstractScene" {
    interface AbstractScene {
        /**
         * The list of layers (background and foreground) of the scene
         */
        layers: Array<Layer>;
    }
}
/**
 * Defines the layer scene component responsible to manage any layers
 * in a given scene.
 */
export class LayerSceneComponent implements ISceneComponent {
    /**
     * The component name helpful to identify the component in the list of scene components.
     */
    readonly name: string;
    /**
     * The scene the component belongs to.
     */
    scene: Scene;
    private _engine;
    /**
     * Creates a new instance of the component for the given scene
     * @param scene Defines the scene to register the component in
     */
    constructor(scene?: Scene);
    /**
     * Registers the component in a given scene
     */
    register(): void;
    /**
     * Rebuilds the elements related to this component in case of
     * context lost for instance.
     */
    rebuild(): void;
    /**
     * Disposes the component and the associated resources.
     */
    dispose(): void;
    private _draw;
    private _drawCameraPredicate;
    private _drawCameraBackground;
    private _drawCameraForegroundWithPostProcessing;
    private _drawCameraForegroundWithoutPostProcessing;
    private _drawRenderTargetPredicate;
    private _drawRenderTargetBackground;
    private _drawRenderTargetForegroundWithPostProcessing;
    private _drawRenderTargetForegroundWithoutPostProcessing;
    /**
     * Adds all the elements from the container to the scene
     * @param container the container holding the elements
     */
    addFromContainer(container: AbstractScene): void;
    /**
     * Removes all the elements in the container from the scene
     * @param container contains the elements to remove
     * @param dispose if the removed element should be disposed (default: false)
     */
    removeFromContainer(container: AbstractScene, dispose?: boolean): void;
}

}
declare module "babylonjs/Legacy/legacy" {
import * as BABYLON from "babylonjs/index";
export * from "babylonjs/index";
export const Debug: {
    AxesViewer: typeof BABYLON.AxesViewer;
    BoneAxesViewer: typeof BABYLON.BoneAxesViewer;
    PhysicsViewer: typeof BABYLON.PhysicsViewer;
    SkeletonViewer: typeof BABYLON.SkeletonViewer;
};

}
declare module "babylonjs/LensFlares/index" {
export * from "babylonjs/LensFlares/lensFlare";
export * from "babylonjs/LensFlares/lensFlareSystem";
export * from "babylonjs/LensFlares/lensFlareSystemSceneComponent";

}
declare module "babylonjs/LensFlares/lensFlare" {
import { Nullable } from "babylonjs/types";
import { Color3 } from "babylonjs/Maths/math.color";
import { Texture } from "babylonjs/Materials/Textures/texture";
import { LensFlareSystem } from "babylonjs/LensFlares/lensFlareSystem";
import { DrawWrapper } from "babylonjs/Materials/drawWrapper";
/**
 * This represents one of the lens effect in a `lensFlareSystem`.
 * It controls one of the individual texture used in the effect.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/environment/lenseFlare
 */
export class LensFlare {
    /**
     * Define the size of the lens flare in the system (a floating value between 0 and 1)
     */
    size: number;
    /**
     * Define the position of the lens flare in the system. (a floating value between -1 and 1). A value of 0 is located on the emitter. A value greater than 0 is beyond the emitter and a value lesser than 0 is behind.
     */
    position: number;
    /**
     * Define the lens color.
     */
    color: Color3;
    /**
     * Define the lens texture.
     */
    texture: Nullable<Texture>;
    /**
     * Define the alpha mode to render this particular lens.
     */
    alphaMode: number;
    /** @internal */
    _drawWrapper: DrawWrapper;
    private _system;
    /**
     * Creates a new Lens Flare.
     * This represents one of the lens effect in a `lensFlareSystem`.
     * It controls one of the individual texture used in the effect.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/environment/lenseFlare
     * @param size Define the size of the lens flare (a floating value between 0 and 1)
     * @param position Define the position of the lens flare in the system. (a floating value between -1 and 1). A value of 0 is located on the emitter. A value greater than 0 is beyond the emitter and a value lesser than 0 is behind.
     * @param color Define the lens color
     * @param imgUrl Define the lens texture url
     * @param system Define the `lensFlareSystem` this flare is part of
     * @returns The newly created Lens Flare
     */
    static AddFlare(size: number, position: number, color: Color3, imgUrl: string, system: LensFlareSystem): LensFlare;
    /**
     * Instantiates a new Lens Flare.
     * This represents one of the lens effect in a `lensFlareSystem`.
     * It controls one of the individual texture used in the effect.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/environment/lenseFlare
     * @param size Define the size of the lens flare in the system (a floating value between 0 and 1)
     * @param position Define the position of the lens flare in the system. (a floating value between -1 and 1). A value of 0 is located on the emitter. A value greater than 0 is beyond the emitter and a value lesser than 0 is behind.
     * @param color Define the lens color
     * @param imgUrl Define the lens texture url
     * @param system Define the `lensFlareSystem` this flare is part of
     */
    constructor(
    /**
     * Define the size of the lens flare in the system (a floating value between 0 and 1)
     */
    size: number, 
    /**
     * Define the position of the lens flare in the system. (a floating value between -1 and 1). A value of 0 is located on the emitter. A value greater than 0 is beyond the emitter and a value lesser than 0 is behind.
     */
    position: number, color: Color3, imgUrl: string, system: LensFlareSystem);
    /**
     * Dispose and release the lens flare with its associated resources.
     */
    dispose(): void;
}

}
declare module "babylonjs/LensFlares/lensFlareSystem" {
import { Scene } from "babylonjs/scene";
import { Vector3 } from "babylonjs/Maths/math.vector";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { LensFlare } from "babylonjs/LensFlares/lensFlare";
import "babylonjs/Shaders/lensFlare.fragment";
import "babylonjs/Shaders/lensFlare.vertex";
import { Viewport } from "babylonjs/Maths/math.viewport";
/**
 * This represents a Lens Flare System or the shiny effect created by the light reflection on the  camera lenses.
 * It is usually composed of several `lensFlare`.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/environment/lenseFlare
 */
export class LensFlareSystem {
    /**
     * Define the name of the lens flare system
     */
    name: string;
    /**
     * List of lens flares used in this system.
     */
    lensFlares: LensFlare[];
    /**
     * Define a limit from the border the lens flare can be visible.
     */
    borderLimit: number;
    /**
     * Define a viewport border we do not want to see the lens flare in.
     */
    viewportBorder: number;
    /**
     * Define a predicate which could limit the list of meshes able to occlude the effect.
     */
    meshesSelectionPredicate: (mesh: AbstractMesh) => boolean;
    /**
     * Restricts the rendering of the effect to only the camera rendering this layer mask.
     */
    layerMask: number;
    /** Gets the scene */
    get scene(): Scene;
    /**
     * Define the id of the lens flare system in the scene.
     * (equal to name by default)
     */
    id: string;
    private _scene;
    private _emitter;
    private _vertexBuffers;
    private _indexBuffer;
    private _positionX;
    private _positionY;
    private _isEnabled;
    /**
     * @internal
     */
    static _SceneComponentInitialization: (scene: Scene) => void;
    /**
     * Instantiates a lens flare system.
     * This represents a Lens Flare System or the shiny effect created by the light reflection on the  camera lenses.
     * It is usually composed of several `lensFlare`.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/environment/lenseFlare
     * @param name Define the name of the lens flare system in the scene
     * @param emitter Define the source (the emitter) of the lens flares (it can be a camera, a light or a mesh).
     * @param scene Define the scene the lens flare system belongs to
     */
    constructor(
    /**
     * Define the name of the lens flare system
     */
    name: string, emitter: any, scene: Scene);
    private _createIndexBuffer;
    /**
     * Define if the lens flare system is enabled.
     */
    get isEnabled(): boolean;
    set isEnabled(value: boolean);
    /**
     * Get the scene the effects belongs to.
     * @returns the scene holding the lens flare system
     */
    getScene(): Scene;
    /**
     * Get the emitter of the lens flare system.
     * It defines the source of the lens flares (it can be a camera, a light or a mesh).
     * @returns the emitter of the lens flare system
     */
    getEmitter(): any;
    /**
     * Set the emitter of the lens flare system.
     * It defines the source of the lens flares (it can be a camera, a light or a mesh).
     * @param newEmitter Define the new emitter of the system
     */
    setEmitter(newEmitter: any): void;
    /**
     * Get the lens flare system emitter position.
     * The emitter defines the source of the lens flares (it can be a camera, a light or a mesh).
     * @returns the position
     */
    getEmitterPosition(): Vector3;
    /**
     * @internal
     */
    computeEffectivePosition(globalViewport: Viewport): boolean;
    /** @internal */
    _isVisible(): boolean;
    /**
     * @internal
     */
    render(): boolean;
    /**
     * Rebuilds the lens flare system
     */
    rebuild(): void;
    /**
     * Dispose and release the lens flare with its associated resources.
     */
    dispose(): void;
    /**
     * Parse a lens flare system from a JSON representation
     * @param parsedLensFlareSystem Define the JSON to parse
     * @param scene Define the scene the parsed system should be instantiated in
     * @param rootUrl Define the rootUrl of the load sequence to easily find a load relative dependencies such as textures
     * @returns the parsed system
     */
    static Parse(parsedLensFlareSystem: any, scene: Scene, rootUrl: string): LensFlareSystem;
    /**
     * Serialize the current Lens Flare System into a JSON representation.
     * @returns the serialized JSON
     */
    serialize(): any;
}

}
declare module "babylonjs/LensFlares/lensFlareSystemSceneComponent" {
import { Nullable } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
import { ISceneSerializableComponent } from "babylonjs/sceneComponent";
import { AbstractScene } from "babylonjs/abstractScene";
import { LensFlareSystem } from "babylonjs/LensFlares/lensFlareSystem";
module "babylonjs/abstractScene" {
    interface AbstractScene {
        /**
         * The list of lens flare system added to the scene
         * @see https://doc.babylonjs.com/features/featuresDeepDive/environment/lenseFlare
         */
        lensFlareSystems: Array<LensFlareSystem>;
        /**
         * Removes the given lens flare system from this scene.
         * @param toRemove The lens flare system to remove
         * @returns The index of the removed lens flare system
         */
        removeLensFlareSystem(toRemove: LensFlareSystem): number;
        /**
         * Adds the given lens flare system to this scene
         * @param newLensFlareSystem The lens flare system to add
         */
        addLensFlareSystem(newLensFlareSystem: LensFlareSystem): void;
        /**
         * Gets a lens flare system using its name
         * @param name defines the name to look for
         * @returns the lens flare system or null if not found
         */
        getLensFlareSystemByName(name: string): Nullable<LensFlareSystem>;
        /**
         * Gets a lens flare system using its Id
         * @param id defines the Id to look for
         * @returns the lens flare system or null if not found
         * @deprecated Please use getLensFlareSystemById instead
         */
        getLensFlareSystemByID(id: string): Nullable<LensFlareSystem>;
        /**
         * Gets a lens flare system using its Id
         * @param id defines the Id to look for
         * @returns the lens flare system or null if not found
         */
        getLensFlareSystemById(id: string): Nullable<LensFlareSystem>;
    }
}
/**
 * Defines the lens flare scene component responsible to manage any lens flares
 * in a given scene.
 */
export class LensFlareSystemSceneComponent implements ISceneSerializableComponent {
    /**
     * The component name helpful to identify the component in the list of scene components.
     */
    readonly name: string;
    /**
     * The scene the component belongs to.
     */
    scene: Scene;
    /**
     * Creates a new instance of the component for the given scene
     * @param scene Defines the scene to register the component in
     */
    constructor(scene: Scene);
    /**
     * Registers the component in a given scene
     */
    register(): void;
    /**
     * Rebuilds the elements related to this component in case of
     * context lost for instance.
     */
    rebuild(): void;
    /**
     * Adds all the elements from the container to the scene
     * @param container the container holding the elements
     */
    addFromContainer(container: AbstractScene): void;
    /**
     * Removes all the elements in the container from the scene
     * @param container contains the elements to remove
     * @param dispose if the removed element should be disposed (default: false)
     */
    removeFromContainer(container: AbstractScene, dispose?: boolean): void;
    /**
     * Serializes the component data to the specified json object
     * @param serializationObject The object to serialize to
     */
    serialize(serializationObject: any): void;
    /**
     * Disposes the component and the associated resources.
     */
    dispose(): void;
    private _draw;
}

}
declare module "babylonjs/Lights/directionalLight" {
import { Camera } from "babylonjs/Cameras/camera";
import { Scene } from "babylonjs/scene";
import { Matrix, Vector3 } from "babylonjs/Maths/math.vector";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { Light } from "babylonjs/Lights/light";
import { ShadowLight } from "babylonjs/Lights/shadowLight";
import { Effect } from "babylonjs/Materials/effect";
/**
 * A directional light is defined by a direction (what a surprise!).
 * The light is emitted from everywhere in the specified direction, and has an infinite range.
 * An example of a directional light is when a distance planet is lit by the apparently parallel lines of light from its sun. Light in a downward direction will light the top of an object.
 * Documentation: https://doc.babylonjs.com/features/featuresDeepDive/lights/lights_introduction
 */
export class DirectionalLight extends ShadowLight {
    private _shadowFrustumSize;
    /**
     * Fix frustum size for the shadow generation. This is disabled if the value is 0.
     */
    get shadowFrustumSize(): number;
    /**
     * Specifies a fix frustum size for the shadow generation.
     */
    set shadowFrustumSize(value: number);
    private _shadowOrthoScale;
    /**
     * Gets the shadow projection scale against the optimal computed one.
     * 0.1 by default which means that the projection window is increase by 10% from the optimal size.
     * This does not impact in fixed frustum size (shadowFrustumSize being set)
     */
    get shadowOrthoScale(): number;
    /**
     * Sets the shadow projection scale against the optimal computed one.
     * 0.1 by default which means that the projection window is increase by 10% from the optimal size.
     * This does not impact in fixed frustum size (shadowFrustumSize being set)
     */
    set shadowOrthoScale(value: number);
    /**
     * Automatically compute the projection matrix to best fit (including all the casters)
     * on each frame.
     */
    autoUpdateExtends: boolean;
    /**
     * Automatically compute the shadowMinZ and shadowMaxZ for the projection matrix to best fit (including all the casters)
     * on each frame. autoUpdateExtends must be set to true for this to work
     */
    autoCalcShadowZBounds: boolean;
    private _orthoLeft;
    private _orthoRight;
    private _orthoTop;
    private _orthoBottom;
    /**
     * Gets or sets the orthoLeft property used to build the light frustum
     */
    get orthoLeft(): number;
    set orthoLeft(left: number);
    /**
     * Gets or sets the orthoRight property used to build the light frustum
     */
    get orthoRight(): number;
    set orthoRight(right: number);
    /**
     * Gets or sets the orthoTop property used to build the light frustum
     */
    get orthoTop(): number;
    set orthoTop(top: number);
    /**
     * Gets or sets the orthoBottom property used to build the light frustum
     */
    get orthoBottom(): number;
    set orthoBottom(bottom: number);
    /**
     * Creates a DirectionalLight object in the scene, oriented towards the passed direction (Vector3).
     * The directional light is emitted from everywhere in the given direction.
     * It can cast shadows.
     * Documentation : https://doc.babylonjs.com/features/featuresDeepDive/lights/lights_introduction
     * @param name The friendly name of the light
     * @param direction The direction of the light
     * @param scene The scene the light belongs to
     */
    constructor(name: string, direction: Vector3, scene: Scene);
    /**
     * Returns the string "DirectionalLight".
     * @returns The class name
     */
    getClassName(): string;
    /**
     * Returns the integer 1.
     * @returns The light Type id as a constant defines in Light.LIGHTTYPEID_x
     */
    getTypeID(): number;
    /**
     * Sets the passed matrix "matrix" as projection matrix for the shadows cast by the light according to the passed view matrix.
     * Returns the DirectionalLight Shadow projection matrix.
     * @param matrix
     * @param viewMatrix
     * @param renderList
     */
    protected _setDefaultShadowProjectionMatrix(matrix: Matrix, viewMatrix: Matrix, renderList: Array<AbstractMesh>): void;
    /**
     * Sets the passed matrix "matrix" as fixed frustum projection matrix for the shadows cast by the light according to the passed view matrix.
     * Returns the DirectionalLight Shadow projection matrix.
     * @param matrix
     */
    protected _setDefaultFixedFrustumShadowProjectionMatrix(matrix: Matrix): void;
    /**
     * Sets the passed matrix "matrix" as auto extend projection matrix for the shadows cast by the light according to the passed view matrix.
     * Returns the DirectionalLight Shadow projection matrix.
     * @param matrix
     * @param viewMatrix
     * @param renderList
     */
    protected _setDefaultAutoExtendShadowProjectionMatrix(matrix: Matrix, viewMatrix: Matrix, renderList: Array<AbstractMesh>): void;
    protected _buildUniformLayout(): void;
    /**
     * Sets the passed Effect object with the DirectionalLight transformed position (or position if not parented) and the passed name.
     * @param effect The effect to update
     * @param lightIndex The index of the light in the effect to update
     * @returns The directional light
     */
    transferToEffect(effect: Effect, lightIndex: string): DirectionalLight;
    transferToNodeMaterialEffect(effect: Effect, lightDataUniformName: string): Light;
    /**
     * Gets the minZ used for shadow according to both the scene and the light.
     *
     * Values are fixed on directional lights as it relies on an ortho projection hence the need to convert being
     * -1 and 1 to 0 and 1 doing (depth + min) / (min + max) -> (depth + 1) / (1 + 1) -> (depth * 0.5) + 0.5.
     * (when not using reverse depth buffer / NDC half Z range)
     * @param activeCamera The camera we are returning the min for
     * @returns the depth min z
     */
    getDepthMinZ(activeCamera: Camera): number;
    /**
     * Gets the maxZ used for shadow according to both the scene and the light.
     *
     * Values are fixed on directional lights as it relies on an ortho projection hence the need to convert being
     * -1 and 1 to 0 and 1 doing (depth + min) / (min + max) -> (depth + 1) / (1 + 1) -> (depth * 0.5) + 0.5.
     * (when not using reverse depth buffer / NDC half Z range)
     * @param activeCamera The camera we are returning the max for
     * @returns the depth max z
     */
    getDepthMaxZ(activeCamera: Camera): number;
    /**
     * Prepares the list of defines specific to the light type.
     * @param defines the list of defines
     * @param lightIndex defines the index of the light for the effect
     */
    prepareLightSpecificDefines(defines: any, lightIndex: number): void;
}

}
declare module "babylonjs/Lights/hemisphericLight" {
import { Nullable } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
import { Matrix, Vector3 } from "babylonjs/Maths/math.vector";
import { Color3 } from "babylonjs/Maths/math.color";
import { Effect } from "babylonjs/Materials/effect";
import { Light } from "babylonjs/Lights/light";
import { IShadowGenerator } from "babylonjs/Lights/Shadows/shadowGenerator";
/**
 * The HemisphericLight simulates the ambient environment light,
 * so the passed direction is the light reflection direction, not the incoming direction.
 */
export class HemisphericLight extends Light {
    /**
     * The groundColor is the light in the opposite direction to the one specified during creation.
     * You can think of the diffuse and specular light as coming from the centre of the object in the given direction and the groundColor light in the opposite direction.
     */
    groundColor: Color3;
    /**
     * The light reflection direction, not the incoming direction.
     */
    direction: Vector3;
    /**
     * Creates a HemisphericLight object in the scene according to the passed direction (Vector3).
     * The HemisphericLight simulates the ambient environment light, so the passed direction is the light reflection direction, not the incoming direction.
     * The HemisphericLight can't cast shadows.
     * Documentation : https://doc.babylonjs.com/features/featuresDeepDive/lights/lights_introduction
     * @param name The friendly name of the light
     * @param direction The direction of the light reflection
     * @param scene The scene the light belongs to
     */
    constructor(name: string, direction: Vector3, scene: Scene);
    protected _buildUniformLayout(): void;
    /**
     * Returns the string "HemisphericLight".
     * @returns The class name
     */
    getClassName(): string;
    /**
     * Sets the HemisphericLight direction towards the passed target (Vector3).
     * Returns the updated direction.
     * @param target The target the direction should point to
     * @returns The computed direction
     */
    setDirectionToTarget(target: Vector3): Vector3;
    /**
     * Returns the shadow generator associated to the light.
     * @returns Always null for hemispheric lights because it does not support shadows.
     */
    getShadowGenerator(): Nullable<IShadowGenerator>;
    /**
     * Sets the passed Effect object with the HemisphericLight normalized direction and color and the passed name (string).
     * @param _effect The effect to update
     * @param lightIndex The index of the light in the effect to update
     * @returns The hemispheric light
     */
    transferToEffect(_effect: Effect, lightIndex: string): HemisphericLight;
    transferToNodeMaterialEffect(effect: Effect, lightDataUniformName: string): this;
    /**
     * Computes the world matrix of the node
     * @returns the world matrix
     */
    computeWorldMatrix(): Matrix;
    /**
     * Returns the integer 3.
     * @returns The light Type id as a constant defines in Light.LIGHTTYPEID_x
     */
    getTypeID(): number;
    /**
     * Prepares the list of defines specific to the light type.
     * @param defines the list of defines
     * @param lightIndex defines the index of the light for the effect
     */
    prepareLightSpecificDefines(defines: any, lightIndex: number): void;
}

}
declare module "babylonjs/Lights/index" {
export * from "babylonjs/Lights/light";
export * from "babylonjs/Lights/shadowLight";
export * from "babylonjs/Lights/Shadows/index";
export * from "babylonjs/Lights/directionalLight";
export * from "babylonjs/Lights/hemisphericLight";
export * from "babylonjs/Lights/pointLight";
export * from "babylonjs/Lights/spotLight";

}
declare module "babylonjs/Lights/light" {
import { Nullable } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
import { Vector3 } from "babylonjs/Maths/math.vector";
import { Color3 } from "babylonjs/Maths/math.color";
import { Node } from "babylonjs/node";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { Effect } from "babylonjs/Materials/effect";
import { UniformBuffer } from "babylonjs/Materials/uniformBuffer";
import { IShadowGenerator } from "babylonjs/Lights/Shadows/shadowGenerator";
import { ISortableLight } from "babylonjs/Lights/lightConstants";
import { Camera } from "babylonjs/Cameras/camera";
/**
 * Base class of all the lights in Babylon. It groups all the generic information about lights.
 * Lights are used, as you would expect, to affect how meshes are seen, in terms of both illumination and colour.
 * All meshes allow light to pass through them unless shadow generation is activated. The default number of lights allowed is four but this can be increased.
 */
export abstract class Light extends Node implements ISortableLight {
    /**
     * Falloff Default: light is falling off following the material specification:
     * standard material is using standard falloff whereas pbr material can request special falloff per materials.
     */
    static readonly FALLOFF_DEFAULT: number;
    /**
     * Falloff Physical: light is falling off following the inverse squared distance law.
     */
    static readonly FALLOFF_PHYSICAL: number;
    /**
     * Falloff gltf: light is falling off as described in the gltf moving to PBR document
     * to enhance interoperability with other engines.
     */
    static readonly FALLOFF_GLTF: number;
    /**
     * Falloff Standard: light is falling off like in the standard material
     * to enhance interoperability with other materials.
     */
    static readonly FALLOFF_STANDARD: number;
    /**
     * If every light affecting the material is in this lightmapMode,
     * material.lightmapTexture adds or multiplies
     * (depends on material.useLightmapAsShadowmap)
     * after every other light calculations.
     */
    static readonly LIGHTMAP_DEFAULT: number;
    /**
     * material.lightmapTexture as only diffuse lighting from this light
     * adds only specular lighting from this light
     * adds dynamic shadows
     */
    static readonly LIGHTMAP_SPECULAR: number;
    /**
     * material.lightmapTexture as only lighting
     * no light calculation from this light
     * only adds dynamic shadows from this light
     */
    static readonly LIGHTMAP_SHADOWSONLY: number;
    /**
     * Each light type uses the default quantity according to its type:
     *      point/spot lights use luminous intensity
     *      directional lights use illuminance
     */
    static readonly INTENSITYMODE_AUTOMATIC: number;
    /**
     * lumen (lm)
     */
    static readonly INTENSITYMODE_LUMINOUSPOWER: number;
    /**
     * candela (lm/sr)
     */
    static readonly INTENSITYMODE_LUMINOUSINTENSITY: number;
    /**
     * lux (lm/m^2)
     */
    static readonly INTENSITYMODE_ILLUMINANCE: number;
    /**
     * nit (cd/m^2)
     */
    static readonly INTENSITYMODE_LUMINANCE: number;
    /**
     * Light type const id of the point light.
     */
    static readonly LIGHTTYPEID_POINTLIGHT: number;
    /**
     * Light type const id of the directional light.
     */
    static readonly LIGHTTYPEID_DIRECTIONALLIGHT: number;
    /**
     * Light type const id of the spot light.
     */
    static readonly LIGHTTYPEID_SPOTLIGHT: number;
    /**
     * Light type const id of the hemispheric light.
     */
    static readonly LIGHTTYPEID_HEMISPHERICLIGHT: number;
    /**
     * Diffuse gives the basic color to an object.
     */
    diffuse: Color3;
    /**
     * Specular produces a highlight color on an object.
     * Note: This is not affecting PBR materials.
     */
    specular: Color3;
    /**
     * Defines the falloff type for this light. This lets overriding how punctual light are
     * falling off base on range or angle.
     * This can be set to any values in Light.FALLOFF_x.
     *
     * Note: This is only useful for PBR Materials at the moment. This could be extended if required to
     * other types of materials.
     */
    falloffType: number;
    /**
     * Strength of the light.
     * Note: By default it is define in the framework own unit.
     * Note: In PBR materials the intensityMode can be use to chose what unit the intensity is defined in.
     */
    intensity: number;
    private _range;
    protected _inverseSquaredRange: number;
    /**
     * Defines how far from the source the light is impacting in scene units.
     * Note: Unused in PBR material as the distance light falloff is defined following the inverse squared falloff.
     */
    get range(): number;
    /**
     * Defines how far from the source the light is impacting in scene units.
     * Note: Unused in PBR material as the distance light falloff is defined following the inverse squared falloff.
     */
    set range(value: number);
    /**
     * Cached photometric scale default to 1.0 as the automatic intensity mode defaults to 1.0 for every type
     * of light.
     */
    private _photometricScale;
    private _intensityMode;
    /**
     * Gets the photometric scale used to interpret the intensity.
     * This is only relevant with PBR Materials where the light intensity can be defined in a physical way.
     */
    get intensityMode(): number;
    /**
     * Sets the photometric scale used to interpret the intensity.
     * This is only relevant with PBR Materials where the light intensity can be defined in a physical way.
     */
    set intensityMode(value: number);
    private _radius;
    /**
     * Gets the light radius used by PBR Materials to simulate soft area lights.
     */
    get radius(): number;
    /**
     * sets the light radius used by PBR Materials to simulate soft area lights.
     */
    set radius(value: number);
    private _renderPriority;
    /**
     * Defines the rendering priority of the lights. It can help in case of fallback or number of lights
     * exceeding the number allowed of the materials.
     */
    renderPriority: number;
    private _shadowEnabled;
    /**
     * Gets whether or not the shadows are enabled for this light. This can help turning off/on shadow without detaching
     * the current shadow generator.
     */
    get shadowEnabled(): boolean;
    /**
     * Sets whether or not the shadows are enabled for this light. This can help turning off/on shadow without detaching
     * the current shadow generator.
     */
    set shadowEnabled(value: boolean);
    private _includedOnlyMeshes;
    /**
     * Gets the only meshes impacted by this light.
     */
    get includedOnlyMeshes(): AbstractMesh[];
    /**
     * Sets the only meshes impacted by this light.
     */
    set includedOnlyMeshes(value: AbstractMesh[]);
    private _excludedMeshes;
    /**
     * Gets the meshes not impacted by this light.
     */
    get excludedMeshes(): AbstractMesh[];
    /**
     * Sets the meshes not impacted by this light.
     */
    set excludedMeshes(value: AbstractMesh[]);
    private _excludeWithLayerMask;
    /**
     * Gets the layer id use to find what meshes are not impacted by the light.
     * Inactive if 0
     */
    get excludeWithLayerMask(): number;
    /**
     * Sets the layer id use to find what meshes are not impacted by the light.
     * Inactive if 0
     */
    set excludeWithLayerMask(value: number);
    private _includeOnlyWithLayerMask;
    /**
     * Gets the layer id use to find what meshes are impacted by the light.
     * Inactive if 0
     */
    get includeOnlyWithLayerMask(): number;
    /**
     * Sets the layer id use to find what meshes are impacted by the light.
     * Inactive if 0
     */
    set includeOnlyWithLayerMask(value: number);
    private _lightmapMode;
    /**
     * Gets the lightmap mode of this light (should be one of the constants defined by Light.LIGHTMAP_x)
     */
    get lightmapMode(): number;
    /**
     * Sets the lightmap mode of this light (should be one of the constants defined by Light.LIGHTMAP_x)
     */
    set lightmapMode(value: number);
    /**
     * Shadow generators associated to the light.
     * @internal Internal use only.
     */
    _shadowGenerators: Nullable<Map<Nullable<Camera>, IShadowGenerator>>;
    /**
     * @internal Internal use only.
     */
    _excludedMeshesIds: string[];
    /**
     * @internal Internal use only.
     */
    _includedOnlyMeshesIds: string[];
    /**
     * The current light uniform buffer.
     * @internal Internal use only.
     */
    _uniformBuffer: UniformBuffer;
    /** @internal */
    _renderId: number;
    private _lastUseSpecular;
    /**
     * Creates a Light object in the scene.
     * Documentation : https://doc.babylonjs.com/features/featuresDeepDive/lights/lights_introduction
     * @param name The friendly name of the light
     * @param scene The scene the light belongs too
     */
    constructor(name: string, scene: Scene);
    protected abstract _buildUniformLayout(): void;
    /**
     * Sets the passed Effect "effect" with the Light information.
     * @param effect The effect to update
     * @param lightIndex The index of the light in the effect to update
     * @returns The light
     */
    abstract transferToEffect(effect: Effect, lightIndex: string): Light;
    /**
     * Sets the passed Effect "effect" with the Light textures.
     * @param effect The effect to update
     * @param lightIndex The index of the light in the effect to update
     * @returns The light
     */
    transferTexturesToEffect(effect: Effect, lightIndex: string): Light;
    /**
     * Binds the lights information from the scene to the effect for the given mesh.
     * @param lightIndex Light index
     * @param scene The scene where the light belongs to
     * @param effect The effect we are binding the data to
     * @param useSpecular Defines if specular is supported
     * @param receiveShadows Defines if the effect (mesh) we bind the light for receives shadows
     */
    _bindLight(lightIndex: number, scene: Scene, effect: Effect, useSpecular: boolean, receiveShadows?: boolean): void;
    /**
     * Sets the passed Effect "effect" with the Light information.
     * @param effect The effect to update
     * @param lightDataUniformName The uniform used to store light data (position or direction)
     * @returns The light
     */
    abstract transferToNodeMaterialEffect(effect: Effect, lightDataUniformName: string): Light;
    /**
     * Returns the string "Light".
     * @returns the class name
     */
    getClassName(): string;
    /** @internal */
    readonly _isLight: boolean;
    /**
     * Converts the light information to a readable string for debug purpose.
     * @param fullDetails Supports for multiple levels of logging within scene loading
     * @returns the human readable light info
     */
    toString(fullDetails?: boolean): string;
    /** @internal */
    protected _syncParentEnabledState(): void;
    /**
     * Set the enabled state of this node.
     * @param value - the new enabled state
     */
    setEnabled(value: boolean): void;
    /**
     * Returns the Light associated shadow generator if any.
     * @param camera Camera for which the shadow generator should be retrieved (default: null). If null, retrieves the default shadow generator
     * @returns the associated shadow generator.
     */
    getShadowGenerator(camera?: Nullable<Camera>): Nullable<IShadowGenerator>;
    /**
     * Returns all the shadow generators associated to this light
     * @returns
     */
    getShadowGenerators(): Nullable<Map<Nullable<Camera>, IShadowGenerator>>;
    /**
     * Returns a Vector3, the absolute light position in the World.
     * @returns the world space position of the light
     */
    getAbsolutePosition(): Vector3;
    /**
     * Specifies if the light will affect the passed mesh.
     * @param mesh The mesh to test against the light
     * @returns true the mesh is affected otherwise, false.
     */
    canAffectMesh(mesh: AbstractMesh): boolean;
    /**
     * Releases resources associated with this node.
     * @param doNotRecurse Set to true to not recurse into each children (recurse into each children by default)
     * @param disposeMaterialAndTextures Set to true to also dispose referenced materials and textures (false by default)
     */
    dispose(doNotRecurse?: boolean, disposeMaterialAndTextures?: boolean): void;
    /**
     * Returns the light type ID (integer).
     * @returns The light Type id as a constant defines in Light.LIGHTTYPEID_x
     */
    getTypeID(): number;
    /**
     * Returns the intensity scaled by the Photometric Scale according to the light type and intensity mode.
     * @returns the scaled intensity in intensity mode unit
     */
    getScaledIntensity(): number;
    /**
     * Returns a new Light object, named "name", from the current one.
     * @param name The name of the cloned light
     * @param newParent The parent of this light, if it has one
     * @returns the new created light
     */
    clone(name: string, newParent?: Nullable<Node>): Nullable<Light>;
    /**
     * Serializes the current light into a Serialization object.
     * @returns the serialized object.
     */
    serialize(): any;
    /**
     * Creates a new typed light from the passed type (integer) : point light = 0, directional light = 1, spot light = 2, hemispheric light = 3.
     * This new light is named "name" and added to the passed scene.
     * @param type Type according to the types available in Light.LIGHTTYPEID_x
     * @param name The friendly name of the light
     * @param scene The scene the new light will belong to
     * @returns the constructor function
     */
    static GetConstructorFromName(type: number, name: string, scene: Scene): Nullable<() => Light>;
    /**
     * Parses the passed "parsedLight" and returns a new instanced Light from this parsing.
     * @param parsedLight The JSON representation of the light
     * @param scene The scene to create the parsed light in
     * @returns the created light after parsing
     */
    static Parse(parsedLight: any, scene: Scene): Nullable<Light>;
    private _hookArrayForExcluded;
    private _hookArrayForIncludedOnly;
    private _resyncMeshes;
    /**
     * Forces the meshes to update their light related information in their rendering used effects
     * @internal Internal Use Only
     */
    _markMeshesAsLightDirty(): void;
    /**
     * Recomputes the cached photometric scale if needed.
     */
    private _computePhotometricScale;
    /**
     * Returns the Photometric Scale according to the light type and intensity mode.
     */
    private _getPhotometricScale;
    /**
     * Reorder the light in the scene according to their defined priority.
     * @internal Internal Use Only
     */
    _reorderLightsInScene(): void;
    /**
     * Prepares the list of defines specific to the light type.
     * @param defines the list of defines
     * @param lightIndex defines the index of the light for the effect
     */
    abstract prepareLightSpecificDefines(defines: any, lightIndex: number): void;
}

}
declare module "babylonjs/Lights/lightConstants" {
/** Defines the cross module constantsused by lights to avoid circular dependencies */
export class LightConstants {
    /**
     * Falloff Default: light is falling off following the material specification:
     * standard material is using standard falloff whereas pbr material can request special falloff per materials.
     */
    static readonly FALLOFF_DEFAULT: number;
    /**
     * Falloff Physical: light is falling off following the inverse squared distance law.
     */
    static readonly FALLOFF_PHYSICAL: number;
    /**
     * Falloff gltf: light is falling off as described in the gltf moving to PBR document
     * to enhance interoperability with other engines.
     */
    static readonly FALLOFF_GLTF: number;
    /**
     * Falloff Standard: light is falling off like in the standard material
     * to enhance interoperability with other materials.
     */
    static readonly FALLOFF_STANDARD: number;
    /**
     * If every light affecting the material is in this lightmapMode,
     * material.lightmapTexture adds or multiplies
     * (depends on material.useLightmapAsShadowmap)
     * after every other light calculations.
     */
    static readonly LIGHTMAP_DEFAULT: number;
    /**
     * material.lightmapTexture as only diffuse lighting from this light
     * adds only specular lighting from this light
     * adds dynamic shadows
     */
    static readonly LIGHTMAP_SPECULAR: number;
    /**
     * material.lightmapTexture as only lighting
     * no light calculation from this light
     * only adds dynamic shadows from this light
     */
    static readonly LIGHTMAP_SHADOWSONLY: number;
    /**
     * Each light type uses the default quantity according to its type:
     *      point/spot lights use luminous intensity
     *      directional lights use illuminance
     */
    static readonly INTENSITYMODE_AUTOMATIC: number;
    /**
     * lumen (lm)
     */
    static readonly INTENSITYMODE_LUMINOUSPOWER: number;
    /**
     * candela (lm/sr)
     */
    static readonly INTENSITYMODE_LUMINOUSINTENSITY: number;
    /**
     * lux (lm/m^2)
     */
    static readonly INTENSITYMODE_ILLUMINANCE: number;
    /**
     * nit (cd/m^2)
     */
    static readonly INTENSITYMODE_LUMINANCE: number;
    /**
     * Light type const id of the point light.
     */
    static readonly LIGHTTYPEID_POINTLIGHT: number;
    /**
     * Light type const id of the directional light.
     */
    static readonly LIGHTTYPEID_DIRECTIONALLIGHT: number;
    /**
     * Light type const id of the spot light.
     */
    static readonly LIGHTTYPEID_SPOTLIGHT: number;
    /**
     * Light type const id of the hemispheric light.
     */
    static readonly LIGHTTYPEID_HEMISPHERICLIGHT: number;
    /**
     * Sort function to order lights for rendering.
     * @param a First Light object to compare to second.
     * @param b Second Light object to compare first.
     * @returns -1 to reduce's a's index relative to be, 0 for no change, 1 to increase a's index relative to b.
     */
    static CompareLightsPriority(a: ISortableLight, b: ISortableLight): number;
}
/**
 * Defines the common interface of sortable lights
 */
export interface ISortableLight {
    /**
     * Gets or sets whether or not the shadows are enabled for this light. This can help turning off/on shadow without detaching
     * the current shadow generator.
     */
    shadowEnabled: boolean;
    /**
     * Defines the rendering priority of the lights. It can help in case of fallback or number of lights
     * exceeding the number allowed of the materials.
     */
    renderPriority: number;
}

}
declare module "babylonjs/Lights/pointLight" {
import { Scene } from "babylonjs/scene";
import { Matrix, Vector3 } from "babylonjs/Maths/math.vector";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { ShadowLight } from "babylonjs/Lights/shadowLight";
import { Effect } from "babylonjs/Materials/effect";
/**
 * A point light is a light defined by an unique point in world space.
 * The light is emitted in every direction from this point.
 * A good example of a point light is a standard light bulb.
 * Documentation: https://doc.babylonjs.com/features/featuresDeepDive/lights/lights_introduction
 */
export class PointLight extends ShadowLight {
    private _shadowAngle;
    /**
     * Getter: In case of direction provided, the shadow will not use a cube texture but simulate a spot shadow as a fallback
     * This specifies what angle the shadow will use to be created.
     *
     * It default to 90 degrees to work nicely with the cube texture generation for point lights shadow maps.
     */
    get shadowAngle(): number;
    /**
     * Setter: In case of direction provided, the shadow will not use a cube texture but simulate a spot shadow as a fallback
     * This specifies what angle the shadow will use to be created.
     *
     * It default to 90 degrees to work nicely with the cube texture generation for point lights shadow maps.
     */
    set shadowAngle(value: number);
    /**
     * Gets the direction if it has been set.
     * In case of direction provided, the shadow will not use a cube texture but simulate a spot shadow as a fallback
     */
    get direction(): Vector3;
    /**
     * In case of direction provided, the shadow will not use a cube texture but simulate a spot shadow as a fallback
     */
    set direction(value: Vector3);
    /**
     * Creates a PointLight object from the passed name and position (Vector3) and adds it in the scene.
     * A PointLight emits the light in every direction.
     * It can cast shadows.
     * If the scene camera is already defined and you want to set your PointLight at the camera position, just set it :
     * ```javascript
     * var pointLight = new PointLight("pl", camera.position, scene);
     * ```
     * Documentation : https://doc.babylonjs.com/features/featuresDeepDive/lights/lights_introduction
     * @param name The light friendly name
     * @param position The position of the point light in the scene
     * @param scene The scene the lights belongs to
     */
    constructor(name: string, position: Vector3, scene: Scene);
    /**
     * Returns the string "PointLight"
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Returns the integer 0.
     * @returns The light Type id as a constant defines in Light.LIGHTTYPEID_x
     */
    getTypeID(): number;
    /**
     * Specifies whether or not the shadowmap should be a cube texture.
     * @returns true if the shadowmap needs to be a cube texture.
     */
    needCube(): boolean;
    /**
     * Returns a new Vector3 aligned with the PointLight cube system according to the passed cube face index (integer).
     * @param faceIndex The index of the face we are computed the direction to generate shadow
     * @returns The set direction in 2d mode otherwise the direction to the cubemap face if needCube() is true
     */
    getShadowDirection(faceIndex?: number): Vector3;
    /**
     * Sets the passed matrix "matrix" as a left-handed perspective projection matrix with the following settings :
     * - fov = PI / 2
     * - aspect ratio : 1.0
     * - z-near and far equal to the active camera minZ and maxZ.
     * Returns the PointLight.
     * @param matrix
     * @param viewMatrix
     * @param renderList
     */
    protected _setDefaultShadowProjectionMatrix(matrix: Matrix, viewMatrix: Matrix, renderList: Array<AbstractMesh>): void;
    protected _buildUniformLayout(): void;
    /**
     * Sets the passed Effect "effect" with the PointLight transformed position (or position, if none) and passed name (string).
     * @param effect The effect to update
     * @param lightIndex The index of the light in the effect to update
     * @returns The point light
     */
    transferToEffect(effect: Effect, lightIndex: string): PointLight;
    transferToNodeMaterialEffect(effect: Effect, lightDataUniformName: string): this;
    /**
     * Prepares the list of defines specific to the light type.
     * @param defines the list of defines
     * @param lightIndex defines the index of the light for the effect
     */
    prepareLightSpecificDefines(defines: any, lightIndex: number): void;
}

}
declare module "babylonjs/Lights/shadowLight" {
import { Camera } from "babylonjs/Cameras/camera";
import { Scene } from "babylonjs/scene";
import { Matrix, Vector3 } from "babylonjs/Maths/math.vector";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { Light } from "babylonjs/Lights/light";
/**
 * Interface describing all the common properties and methods a shadow light needs to implement.
 * This helps both the shadow generator and materials to generate the corresponding shadow maps
 * as well as binding the different shadow properties to the effects.
 */
export interface IShadowLight extends Light {
    /**
     * The light id in the scene (used in scene.getLightById for instance)
     */
    id: string;
    /**
     * The position the shadow will be casted from.
     */
    position: Vector3;
    /**
     * In 2d mode (needCube being false), the direction used to cast the shadow.
     */
    direction: Vector3;
    /**
     * The transformed position. Position of the light in world space taking parenting in account.
     */
    transformedPosition: Vector3;
    /**
     * The transformed direction. Direction of the light in world space taking parenting in account.
     */
    transformedDirection: Vector3;
    /**
     * The friendly name of the light in the scene.
     */
    name: string;
    /**
     * Defines the shadow projection clipping minimum z value.
     */
    shadowMinZ: number;
    /**
     * Defines the shadow projection clipping maximum z value.
     */
    shadowMaxZ: number;
    /**
     * Computes the transformed information (transformedPosition and transformedDirection in World space) of the current light
     * @returns true if the information has been computed, false if it does not need to (no parenting)
     */
    computeTransformedInformation(): boolean;
    /**
     * Gets the scene the light belongs to.
     * @returns The scene
     */
    getScene(): Scene;
    /**
     * Callback defining a custom Projection Matrix Builder.
     * This can be used to override the default projection matrix computation.
     */
    customProjectionMatrixBuilder: (viewMatrix: Matrix, renderList: Array<AbstractMesh>, result: Matrix) => void;
    /**
     * Sets the shadow projection matrix in parameter to the generated projection matrix.
     * @param matrix The matrix to updated with the projection information
     * @param viewMatrix The transform matrix of the light
     * @param renderList The list of mesh to render in the map
     * @returns The current light
     */
    setShadowProjectionMatrix(matrix: Matrix, viewMatrix: Matrix, renderList: Array<AbstractMesh>): IShadowLight;
    /**
     * Gets the current depth scale used in ESM.
     * @returns The scale
     */
    getDepthScale(): number;
    /**
     * Returns whether or not the shadow generation require a cube texture or a 2d texture.
     * @returns true if a cube texture needs to be use
     */
    needCube(): boolean;
    /**
     * Detects if the projection matrix requires to be recomputed this frame.
     * @returns true if it requires to be recomputed otherwise, false.
     */
    needProjectionMatrixCompute(): boolean;
    /**
     * Forces the shadow generator to recompute the projection matrix even if position and direction did not changed.
     */
    forceProjectionMatrixCompute(): void;
    /**
     * Get the direction to use to render the shadow map. In case of cube texture, the face index can be passed.
     * @param faceIndex The index of the face we are computed the direction to generate shadow
     * @returns The set direction in 2d mode otherwise the direction to the cubemap face if needCube() is true
     */
    getShadowDirection(faceIndex?: number): Vector3;
    /**
     * Gets the minZ used for shadow according to both the scene and the light.
     * @param activeCamera The camera we are returning the min for
     * @returns the depth min z
     */
    getDepthMinZ(activeCamera: Camera): number;
    /**
     * Gets the maxZ used for shadow according to both the scene and the light.
     * @param activeCamera The camera we are returning the max for
     * @returns the depth max z
     */
    getDepthMaxZ(activeCamera: Camera): number;
}
/**
 * Base implementation IShadowLight
 * It groups all the common behaviour in order to reduce duplication and better follow the DRY pattern.
 */
export abstract class ShadowLight extends Light implements IShadowLight {
    protected abstract _setDefaultShadowProjectionMatrix(matrix: Matrix, viewMatrix: Matrix, renderList: Array<AbstractMesh>): void;
    protected _position: Vector3;
    protected _setPosition(value: Vector3): void;
    /**
     * Sets the position the shadow will be casted from. Also use as the light position for both
     * point and spot lights.
     */
    get position(): Vector3;
    /**
     * Sets the position the shadow will be casted from. Also use as the light position for both
     * point and spot lights.
     */
    set position(value: Vector3);
    protected _direction: Vector3;
    protected _setDirection(value: Vector3): void;
    /**
     * In 2d mode (needCube being false), gets the direction used to cast the shadow.
     * Also use as the light direction on spot and directional lights.
     */
    get direction(): Vector3;
    /**
     * In 2d mode (needCube being false), sets the direction used to cast the shadow.
     * Also use as the light direction on spot and directional lights.
     */
    set direction(value: Vector3);
    protected _shadowMinZ: number;
    /**
     * Gets the shadow projection clipping minimum z value.
     */
    get shadowMinZ(): number;
    /**
     * Sets the shadow projection clipping minimum z value.
     */
    set shadowMinZ(value: number);
    protected _shadowMaxZ: number;
    /**
     * Sets the shadow projection clipping maximum z value.
     */
    get shadowMaxZ(): number;
    /**
     * Gets the shadow projection clipping maximum z value.
     */
    set shadowMaxZ(value: number);
    /**
     * Callback defining a custom Projection Matrix Builder.
     * This can be used to override the default projection matrix computation.
     */
    customProjectionMatrixBuilder: (viewMatrix: Matrix, renderList: Array<AbstractMesh>, result: Matrix) => void;
    /**
     * The transformed position. Position of the light in world space taking parenting in account.
     */
    transformedPosition: Vector3;
    /**
     * The transformed direction. Direction of the light in world space taking parenting in account.
     */
    transformedDirection: Vector3;
    private _needProjectionMatrixCompute;
    /**
     * Computes the transformed information (transformedPosition and transformedDirection in World space) of the current light
     * @returns true if the information has been computed, false if it does not need to (no parenting)
     */
    computeTransformedInformation(): boolean;
    /**
     * Return the depth scale used for the shadow map.
     * @returns the depth scale.
     */
    getDepthScale(): number;
    /**
     * Get the direction to use to render the shadow map. In case of cube texture, the face index can be passed.
     * @param faceIndex The index of the face we are computed the direction to generate shadow
     * @returns The set direction in 2d mode otherwise the direction to the cubemap face if needCube() is true
     */
    getShadowDirection(faceIndex?: number): Vector3;
    /**
     * Returns the ShadowLight absolute position in the World.
     * @returns the position vector in world space
     */
    getAbsolutePosition(): Vector3;
    /**
     * Sets the ShadowLight direction toward the passed target.
     * @param target The point to target in local space
     * @returns the updated ShadowLight direction
     */
    setDirectionToTarget(target: Vector3): Vector3;
    /**
     * Returns the light rotation in euler definition.
     * @returns the x y z rotation in local space.
     */
    getRotation(): Vector3;
    /**
     * Returns whether or not the shadow generation require a cube texture or a 2d texture.
     * @returns true if a cube texture needs to be use
     */
    needCube(): boolean;
    /**
     * Detects if the projection matrix requires to be recomputed this frame.
     * @returns true if it requires to be recomputed otherwise, false.
     */
    needProjectionMatrixCompute(): boolean;
    /**
     * Forces the shadow generator to recompute the projection matrix even if position and direction did not changed.
     */
    forceProjectionMatrixCompute(): void;
    /** @internal */
    _initCache(): void;
    /** @internal */
    _isSynchronized(): boolean;
    /**
     * Computes the world matrix of the node
     * @param force defines if the cache version should be invalidated forcing the world matrix to be created from scratch
     * @returns the world matrix
     */
    computeWorldMatrix(force?: boolean): Matrix;
    /**
     * Gets the minZ used for shadow according to both the scene and the light.
     * @param activeCamera The camera we are returning the min for
     * @returns the depth min z
     */
    getDepthMinZ(activeCamera: Camera): number;
    /**
     * Gets the maxZ used for shadow according to both the scene and the light.
     * @param activeCamera The camera we are returning the max for
     * @returns the depth max z
     */
    getDepthMaxZ(activeCamera: Camera): number;
    /**
     * Sets the shadow projection matrix in parameter to the generated projection matrix.
     * @param matrix The matrix to updated with the projection information
     * @param viewMatrix The transform matrix of the light
     * @param renderList The list of mesh to render in the map
     * @returns The current light
     */
    setShadowProjectionMatrix(matrix: Matrix, viewMatrix: Matrix, renderList: Array<AbstractMesh>): IShadowLight;
    /** @internal */
    protected _syncParentEnabledState(): void;
}

}
declare module "babylonjs/Lights/Shadows/cascadedShadowGenerator" {
import { Nullable } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
import { Matrix, Vector3 } from "babylonjs/Maths/math.vector";
import { SubMesh } from "babylonjs/Meshes/subMesh";
import { Effect } from "babylonjs/Materials/effect";
import "babylonjs/Shaders/shadowMap.fragment";
import "babylonjs/Shaders/shadowMap.vertex";
import "babylonjs/Shaders/depthBoxBlur.fragment";
import { ShadowGenerator } from "babylonjs/Lights/Shadows/shadowGenerator";
import { DirectionalLight } from "babylonjs/Lights/directionalLight";
import { BoundingInfo } from "babylonjs/Culling/boundingInfo";
import { DepthRenderer } from "babylonjs/Rendering/depthRenderer";
import { Camera } from "babylonjs/Cameras/camera";
/**
 * A CSM implementation allowing casting shadows on large scenes.
 * Documentation : https://doc.babylonjs.com/babylon101/cascadedShadows
 * Based on: https://github.com/TheRealMJP/Shadows and https://johanmedestrom.wordpress.com/2016/03/18/opengl-cascaded-shadow-maps/
 */
export class CascadedShadowGenerator extends ShadowGenerator {
    private static readonly _FrustumCornersNDCSpace;
    /**
     * Name of the CSM class
     */
    static CLASSNAME: string;
    /**
     * Defines the default number of cascades used by the CSM.
     */
    static readonly DEFAULT_CASCADES_COUNT: number;
    /**
     * Defines the minimum number of cascades used by the CSM.
     */
    static readonly MIN_CASCADES_COUNT: number;
    /**
     * Defines the maximum number of cascades used by the CSM.
     */
    static readonly MAX_CASCADES_COUNT: number;
    protected _validateFilter(filter: number): number;
    /**
     * Gets or sets the actual darkness of the soft shadows while using PCSS filtering (value between 0. and 1.)
     */
    penumbraDarkness: number;
    private _numCascades;
    /**
     * Gets or set the number of cascades used by the CSM.
     */
    get numCascades(): number;
    set numCascades(value: number);
    /**
     * Sets this to true if you want that the edges of the shadows don't "swimm" / "shimmer" when rotating the camera.
     * The trade off is that you lose some precision in the shadow rendering when enabling this setting.
     */
    stabilizeCascades: boolean;
    private _freezeShadowCastersBoundingInfo;
    private _freezeShadowCastersBoundingInfoObservable;
    /**
     * Enables or disables the shadow casters bounding info computation.
     * If your shadow casters don't move, you can disable this feature.
     * If it is enabled, the bounding box computation is done every frame.
     */
    get freezeShadowCastersBoundingInfo(): boolean;
    set freezeShadowCastersBoundingInfo(freeze: boolean);
    private _scbiMin;
    private _scbiMax;
    protected _computeShadowCastersBoundingInfo(): void;
    protected _shadowCastersBoundingInfo: BoundingInfo;
    /**
     * Gets or sets the shadow casters bounding info.
     * If you provide your own shadow casters bounding info, first enable freezeShadowCastersBoundingInfo
     * so that the system won't overwrite the bounds you provide
     */
    get shadowCastersBoundingInfo(): BoundingInfo;
    set shadowCastersBoundingInfo(boundingInfo: BoundingInfo);
    protected _breaksAreDirty: boolean;
    protected _minDistance: number;
    protected _maxDistance: number;
    /**
     * Sets the minimal and maximal distances to use when computing the cascade breaks.
     *
     * The values of min / max are typically the depth zmin and zmax values of your scene, for a given frame.
     * If you don't know these values, simply leave them to their defaults and don't call this function.
     * @param min minimal distance for the breaks (default to 0.)
     * @param max maximal distance for the breaks (default to 1.)
     */
    setMinMaxDistance(min: number, max: number): void;
    /** Gets the minimal distance used in the cascade break computation */
    get minDistance(): number;
    /** Gets the maximal distance used in the cascade break computation */
    get maxDistance(): number;
    /**
     * Gets the class name of that object
     * @returns "CascadedShadowGenerator"
     */
    getClassName(): string;
    private _cascadeMinExtents;
    private _cascadeMaxExtents;
    /**
     * Gets a cascade minimum extents
     * @param cascadeIndex index of the cascade
     * @returns the minimum cascade extents
     */
    getCascadeMinExtents(cascadeIndex: number): Nullable<Vector3>;
    /**
     * Gets a cascade maximum extents
     * @param cascadeIndex index of the cascade
     * @returns the maximum cascade extents
     */
    getCascadeMaxExtents(cascadeIndex: number): Nullable<Vector3>;
    private _cascades;
    private _currentLayer;
    private _viewSpaceFrustumsZ;
    private _viewMatrices;
    private _projectionMatrices;
    private _transformMatrices;
    private _transformMatricesAsArray;
    private _frustumLengths;
    private _lightSizeUVCorrection;
    private _depthCorrection;
    private _frustumCornersWorldSpace;
    private _frustumCenter;
    private _shadowCameraPos;
    private _shadowMaxZ;
    /**
     * Gets the shadow max z distance. It's the limit beyond which shadows are not displayed.
     * It defaults to camera.maxZ
     */
    get shadowMaxZ(): number;
    /**
     * Sets the shadow max z distance.
     */
    set shadowMaxZ(value: number);
    protected _debug: boolean;
    /**
     * Gets or sets the debug flag.
     * When enabled, the cascades are materialized by different colors on the screen.
     */
    get debug(): boolean;
    set debug(dbg: boolean);
    private _depthClamp;
    /**
     * Gets or sets the depth clamping value.
     *
     * When enabled, it improves the shadow quality because the near z plane of the light frustum don't need to be adjusted
     * to account for the shadow casters far away.
     *
     * Note that this property is incompatible with PCSS filtering, so it won't be used in that case.
     */
    get depthClamp(): boolean;
    set depthClamp(value: boolean);
    private _cascadeBlendPercentage;
    /**
     * Gets or sets the percentage of blending between two cascades (value between 0. and 1.).
     * It defaults to 0.1 (10% blending).
     */
    get cascadeBlendPercentage(): number;
    set cascadeBlendPercentage(value: number);
    private _lambda;
    /**
     * Gets or set the lambda parameter.
     * This parameter is used to split the camera frustum and create the cascades.
     * It's a value between 0. and 1.: If 0, the split is a uniform split of the frustum, if 1 it is a logarithmic split.
     * For all values in-between, it's a linear combination of the uniform and logarithm split algorithm.
     */
    get lambda(): number;
    set lambda(value: number);
    /**
     * Gets the view matrix corresponding to a given cascade
     * @param cascadeNum cascade to retrieve the view matrix from
     * @returns the cascade view matrix
     */
    getCascadeViewMatrix(cascadeNum: number): Nullable<Matrix>;
    /**
     * Gets the projection matrix corresponding to a given cascade
     * @param cascadeNum cascade to retrieve the projection matrix from
     * @returns the cascade projection matrix
     */
    getCascadeProjectionMatrix(cascadeNum: number): Nullable<Matrix>;
    /**
     * Gets the transformation matrix corresponding to a given cascade
     * @param cascadeNum cascade to retrieve the transformation matrix from
     * @returns the cascade transformation matrix
     */
    getCascadeTransformMatrix(cascadeNum: number): Nullable<Matrix>;
    private _depthRenderer;
    /**
     * Sets the depth renderer to use when autoCalcDepthBounds is enabled.
     *
     * Note that if no depth renderer is set, a new one will be automatically created internally when necessary.
     *
     * You should call this function if you already have a depth renderer enabled in your scene, to avoid
     * doing multiple depth rendering each frame. If you provide your own depth renderer, make sure it stores linear depth!
     * @param depthRenderer The depth renderer to use when autoCalcDepthBounds is enabled. If you pass null or don't call this function at all, a depth renderer will be automatically created
     */
    setDepthRenderer(depthRenderer: Nullable<DepthRenderer>): void;
    private _depthReducer;
    private _autoCalcDepthBounds;
    /**
     * Gets or sets the autoCalcDepthBounds property.
     *
     * When enabled, a depth rendering pass is first performed (with an internally created depth renderer or with the one
     * you provide by calling setDepthRenderer). Then, a min/max reducing is applied on the depth map to compute the
     * minimal and maximal depth of the map and those values are used as inputs for the setMinMaxDistance() function.
     * It can greatly enhance the shadow quality, at the expense of more GPU works.
     * When using this option, you should increase the value of the lambda parameter, and even set it to 1 for best results.
     */
    get autoCalcDepthBounds(): boolean;
    set autoCalcDepthBounds(value: boolean);
    /**
     * Defines the refresh rate of the min/max computation used when autoCalcDepthBounds is set to true
     * Use 0 to compute just once, 1 to compute on every frame, 2 to compute every two frames and so on...
     * Note that if you provided your own depth renderer through a call to setDepthRenderer, you are responsible
     * for setting the refresh rate on the renderer yourself!
     */
    get autoCalcDepthBoundsRefreshRate(): number;
    set autoCalcDepthBoundsRefreshRate(value: number);
    /**
     * Create the cascade breaks according to the lambda, shadowMaxZ and min/max distance properties, as well as the camera near and far planes.
     * This function is automatically called when updating lambda, shadowMaxZ and min/max distances, however you should call it yourself if
     * you change the camera near/far planes!
     */
    splitFrustum(): void;
    private _splitFrustum;
    private _computeMatrices;
    private _computeFrustumInWorldSpace;
    private _computeCascadeFrustum;
    protected _recreateSceneUBOs(): void;
    /**
     *  Support test.
     */
    static get IsSupported(): boolean;
    /**
     * @internal
     */
    static _SceneComponentInitialization: (scene: Scene) => void;
    /**
     * Creates a Cascaded Shadow Generator object.
     * A ShadowGenerator is the required tool to use the shadows.
     * Each directional light casting shadows needs to use its own ShadowGenerator.
     * Documentation : https://doc.babylonjs.com/babylon101/cascadedShadows
     * @param mapSize The size of the texture what stores the shadows. Example : 1024.
     * @param light The directional light object generating the shadows.
     * @param usefulFloatFirst By default the generator will try to use half float textures but if you need precision (for self shadowing for instance), you can use this option to enforce full float texture.
     * @param camera Camera associated with this shadow generator (default: null). If null, takes the scene active camera at the time we need to access it
     */
    constructor(mapSize: number, light: DirectionalLight, usefulFloatFirst?: boolean, camera?: Nullable<Camera>);
    protected _initializeGenerator(): void;
    protected _createTargetRenderTexture(): void;
    protected _initializeShadowMap(): void;
    protected _bindCustomEffectForRenderSubMeshForShadowMap(subMesh: SubMesh, effect: Effect): void;
    protected _isReadyCustomDefines(defines: any): void;
    /**
     * Prepare all the defines in a material relying on a shadow map at the specified light index.
     * @param defines Defines of the material we want to update
     * @param lightIndex Index of the light in the enabled light list of the material
     */
    prepareDefines(defines: any, lightIndex: number): void;
    /**
     * Binds the shadow related information inside of an effect (information like near, far, darkness...
     * defined in the generator but impacting the effect).
     * @param lightIndex Index of the light in the enabled light list of the material owning the effect
     * @param effect The effect we are binfing the information for
     */
    bindShadowLight(lightIndex: string, effect: Effect): void;
    /**
     * Gets the transformation matrix of the first cascade used to project the meshes into the map from the light point of view.
     * (eq to view projection * shadow projection matrices)
     * @returns The transform matrix used to create the shadow map
     */
    getTransformMatrix(): Matrix;
    /**
     * Disposes the ShadowGenerator.
     * Returns nothing.
     */
    dispose(): void;
    /**
     * Serializes the shadow generator setup to a json object.
     * @returns The serialized JSON object
     */
    serialize(): any;
    /**
     * Parses a serialized ShadowGenerator and returns a new ShadowGenerator.
     * @param parsedShadowGenerator The JSON object to parse
     * @param scene The scene to create the shadow map for
     * @returns The parsed shadow generator
     */
    static Parse(parsedShadowGenerator: any, scene: Scene): ShadowGenerator;
}

}
declare module "babylonjs/Lights/Shadows/index" {
export * from "babylonjs/Lights/Shadows/shadowGenerator";
export * from "babylonjs/Lights/Shadows/cascadedShadowGenerator";
export * from "babylonjs/Lights/Shadows/shadowGeneratorSceneComponent";

}
declare module "babylonjs/Lights/Shadows/shadowGenerator" {
import { SmartArray } from "babylonjs/Misc/smartArray";
import { Nullable } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
import { Matrix, Vector3 } from "babylonjs/Maths/math.vector";
import { SubMesh } from "babylonjs/Meshes/subMesh";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { Mesh } from "babylonjs/Meshes/mesh";
import { IShadowLight } from "babylonjs/Lights/shadowLight";
import { MaterialDefines } from "babylonjs/Materials/materialDefines";
import { Effect } from "babylonjs/Materials/effect";
import { RenderTargetTexture } from "babylonjs/Materials/Textures/renderTargetTexture";
import { PostProcess } from "babylonjs/PostProcesses/postProcess";
import { Observable } from "babylonjs/Misc/observable";
import { UniformBuffer } from "babylonjs/Materials/uniformBuffer";
import { Camera } from "babylonjs/Cameras/camera";
import "babylonjs/Shaders/shadowMap.fragment";
import "babylonjs/Shaders/shadowMap.vertex";
import "babylonjs/Shaders/depthBoxBlur.fragment";
import "babylonjs/Shaders/ShadersInclude/shadowMapFragmentSoftTransparentShadow";
/**
 * Defines the options associated with the creation of a custom shader for a shadow generator.
 */
export interface ICustomShaderOptions {
    /**
     * Gets or sets the custom shader name to use
     */
    shaderName: string;
    /**
     * The list of attribute names used in the shader
     */
    attributes?: string[];
    /**
     * The list of uniform names used in the shader
     */
    uniforms?: string[];
    /**
     * The list of sampler names used in the shader
     */
    samplers?: string[];
    /**
     * The list of defines used in the shader
     */
    defines?: string[];
}
/**
 * Interface to implement to create a shadow generator compatible with BJS.
 */
export interface IShadowGenerator {
    /** Gets or set the id of the shadow generator. It will be the one from the light if not defined */
    id: string;
    /**
     * Gets the main RTT containing the shadow map (usually storing depth from the light point of view).
     * @returns The render target texture if present otherwise, null
     */
    getShadowMap(): Nullable<RenderTargetTexture>;
    /**
     * Determine whether the shadow generator is ready or not (mainly all effects and related post processes needs to be ready).
     * @param subMesh The submesh we want to render in the shadow map
     * @param useInstances Defines whether will draw in the map using instances
     * @param isTransparent Indicates that isReady is called for a transparent subMesh
     * @returns true if ready otherwise, false
     */
    isReady(subMesh: SubMesh, useInstances: boolean, isTransparent: boolean): boolean;
    /**
     * Prepare all the defines in a material relying on a shadow map at the specified light index.
     * @param defines Defines of the material we want to update
     * @param lightIndex Index of the light in the enabled light list of the material
     */
    prepareDefines(defines: MaterialDefines, lightIndex: number): void;
    /**
     * Binds the shadow related information inside of an effect (information like near, far, darkness...
     * defined in the generator but impacting the effect).
     * It implies the uniforms available on the materials are the standard BJS ones.
     * @param lightIndex Index of the light in the enabled light list of the material owning the effect
     * @param effect The effect we are binding the information for
     */
    bindShadowLight(lightIndex: string, effect: Effect): void;
    /**
     * Gets the transformation matrix used to project the meshes into the map from the light point of view.
     * (eq to shadow projection matrix * light transform matrix)
     * @returns The transform matrix used to create the shadow map
     */
    getTransformMatrix(): Matrix;
    /**
     * Recreates the shadow map dependencies like RTT and post processes. This can be used during the switch between
     * Cube and 2D textures for instance.
     */
    recreateShadowMap(): void;
    /**
     * Forces all the attached effect to compile to enable rendering only once ready vs. lazily compiling effects.
     * @param onCompiled Callback triggered at the and of the effects compilation
     * @param options Sets of optional options forcing the compilation with different modes
     */
    forceCompilation(onCompiled?: (generator: IShadowGenerator) => void, options?: Partial<{
        useInstances: boolean;
    }>): void;
    /**
     * Forces all the attached effect to compile to enable rendering only once ready vs. lazily compiling effects.
     * @param options Sets of optional options forcing the compilation with different modes
     * @returns A promise that resolves when the compilation completes
     */
    forceCompilationAsync(options?: Partial<{
        useInstances: boolean;
    }>): Promise<void>;
    /**
     * Serializes the shadow generator setup to a json object.
     * @returns The serialized JSON object
     */
    serialize(): any;
    /**
     * Disposes the Shadow map and related Textures and effects.
     */
    dispose(): void;
}
/**
 * Default implementation IShadowGenerator.
 * This is the main object responsible of generating shadows in the framework.
 * Documentation: https://doc.babylonjs.com/features/featuresDeepDive/lights/shadows
 */
export class ShadowGenerator implements IShadowGenerator {
    /**
     * Name of the shadow generator class
     */
    static CLASSNAME: string;
    /**
     * Shadow generator mode None: no filtering applied.
     */
    static readonly FILTER_NONE: number;
    /**
     * Shadow generator mode ESM: Exponential Shadow Mapping.
     * (http://developer.download.nvidia.com/presentations/2008/GDC/GDC08_SoftShadowMapping.pdf)
     */
    static readonly FILTER_EXPONENTIALSHADOWMAP: number;
    /**
     * Shadow generator mode Poisson Sampling: Percentage Closer Filtering.
     * (Multiple Tap around evenly distributed around the pixel are used to evaluate the shadow strength)
     */
    static readonly FILTER_POISSONSAMPLING: number;
    /**
     * Shadow generator mode ESM: Blurred Exponential Shadow Mapping.
     * (http://developer.download.nvidia.com/presentations/2008/GDC/GDC08_SoftShadowMapping.pdf)
     */
    static readonly FILTER_BLUREXPONENTIALSHADOWMAP: number;
    /**
     * Shadow generator mode ESM: Exponential Shadow Mapping using the inverse of the exponential preventing
     * edge artifacts on steep falloff.
     * (http://developer.download.nvidia.com/presentations/2008/GDC/GDC08_SoftShadowMapping.pdf)
     */
    static readonly FILTER_CLOSEEXPONENTIALSHADOWMAP: number;
    /**
     * Shadow generator mode ESM: Blurred Exponential Shadow Mapping using the inverse of the exponential preventing
     * edge artifacts on steep falloff.
     * (http://developer.download.nvidia.com/presentations/2008/GDC/GDC08_SoftShadowMapping.pdf)
     */
    static readonly FILTER_BLURCLOSEEXPONENTIALSHADOWMAP: number;
    /**
     * Shadow generator mode PCF: Percentage Closer Filtering
     * benefits from Webgl 2 shadow samplers. Fallback to Poisson Sampling in Webgl 1
     * (https://developer.nvidia.com/gpugems/GPUGems/gpugems_ch11.html)
     */
    static readonly FILTER_PCF: number;
    /**
     * Shadow generator mode PCSS: Percentage Closering Soft Shadow.
     * benefits from Webgl 2 shadow samplers. Fallback to Poisson Sampling in Webgl 1
     * Contact Hardening
     */
    static readonly FILTER_PCSS: number;
    /**
     * Reserved for PCF and PCSS
     * Highest Quality.
     *
     * Execute PCF on a 5*5 kernel improving a lot the shadow aliasing artifacts.
     *
     * Execute PCSS with 32 taps blocker search and 64 taps PCF.
     */
    static readonly QUALITY_HIGH: number;
    /**
     * Reserved for PCF and PCSS
     * Good tradeoff for quality/perf cross devices
     *
     * Execute PCF on a 3*3 kernel.
     *
     * Execute PCSS with 16 taps blocker search and 32 taps PCF.
     */
    static readonly QUALITY_MEDIUM: number;
    /**
     * Reserved for PCF and PCSS
     * The lowest quality but the fastest.
     *
     * Execute PCF on a 1*1 kernel.
     *
     * Execute PCSS with 16 taps blocker search and 16 taps PCF.
     */
    static readonly QUALITY_LOW: number;
    /**
     * Defines the default alpha cutoff value used for transparent alpha tested materials.
     */
    static DEFAULT_ALPHA_CUTOFF: number;
    /** Gets or set the id of the shadow generator. It will be the one from the light if not defined */
    id: string;
    /** Gets or sets the custom shader name to use */
    customShaderOptions: ICustomShaderOptions;
    /** Gets or sets a custom function to allow/disallow rendering a sub mesh in the shadow map */
    customAllowRendering: (subMesh: SubMesh) => boolean;
    /**
     * Observable triggered before the shadow is rendered. Can be used to update internal effect state
     */
    onBeforeShadowMapRenderObservable: Observable<Effect>;
    /**
     * Observable triggered after the shadow is rendered. Can be used to restore internal effect state
     */
    onAfterShadowMapRenderObservable: Observable<Effect>;
    /**
     * Observable triggered before a mesh is rendered in the shadow map.
     * Can be used to update internal effect state (that you can get from the onBeforeShadowMapRenderObservable)
     */
    onBeforeShadowMapRenderMeshObservable: Observable<Mesh>;
    /**
     * Observable triggered after a mesh is rendered in the shadow map.
     * Can be used to update internal effect state (that you can get from the onAfterShadowMapRenderObservable)
     */
    onAfterShadowMapRenderMeshObservable: Observable<Mesh>;
    protected _bias: number;
    /**
     * Gets the bias: offset applied on the depth preventing acnea (in light direction).
     */
    get bias(): number;
    /**
     * Sets the bias: offset applied on the depth preventing acnea (in light direction).
     */
    set bias(bias: number);
    protected _normalBias: number;
    /**
     * Gets the normalBias: offset applied on the depth preventing acnea (along side the normal direction and proportional to the light/normal angle).
     */
    get normalBias(): number;
    /**
     * Sets the normalBias: offset applied on the depth preventing acnea (along side the normal direction and proportional to the light/normal angle).
     */
    set normalBias(normalBias: number);
    protected _blurBoxOffset: number;
    /**
     * Gets the blur box offset: offset applied during the blur pass.
     * Only useful if useKernelBlur = false
     */
    get blurBoxOffset(): number;
    /**
     * Sets the blur box offset: offset applied during the blur pass.
     * Only useful if useKernelBlur = false
     */
    set blurBoxOffset(value: number);
    protected _blurScale: number;
    /**
     * Gets the blur scale: scale of the blurred texture compared to the main shadow map.
     * 2 means half of the size.
     */
    get blurScale(): number;
    /**
     * Sets the blur scale: scale of the blurred texture compared to the main shadow map.
     * 2 means half of the size.
     */
    set blurScale(value: number);
    protected _blurKernel: number;
    /**
     * Gets the blur kernel: kernel size of the blur pass.
     * Only useful if useKernelBlur = true
     */
    get blurKernel(): number;
    /**
     * Sets the blur kernel: kernel size of the blur pass.
     * Only useful if useKernelBlur = true
     */
    set blurKernel(value: number);
    protected _useKernelBlur: boolean;
    /**
     * Gets whether the blur pass is a kernel blur (if true) or box blur.
     * Only useful in filtered mode (useBlurExponentialShadowMap...)
     */
    get useKernelBlur(): boolean;
    /**
     * Sets whether the blur pass is a kernel blur (if true) or box blur.
     * Only useful in filtered mode (useBlurExponentialShadowMap...)
     */
    set useKernelBlur(value: boolean);
    protected _depthScale: number;
    /**
     * Gets the depth scale used in ESM mode.
     */
    get depthScale(): number;
    /**
     * Sets the depth scale used in ESM mode.
     * This can override the scale stored on the light.
     */
    set depthScale(value: number);
    protected _validateFilter(filter: number): number;
    protected _filter: number;
    /**
     * Gets the current mode of the shadow generator (normal, PCF, ESM...).
     * The returned value is a number equal to one of the available mode defined in ShadowMap.FILTER_x like _FILTER_NONE
     */
    get filter(): number;
    /**
     * Sets the current mode of the shadow generator (normal, PCF, ESM...).
     * The returned value is a number equal to one of the available mode defined in ShadowMap.FILTER_x like _FILTER_NONE
     */
    set filter(value: number);
    /**
     * Gets if the current filter is set to Poisson Sampling.
     */
    get usePoissonSampling(): boolean;
    /**
     * Sets the current filter to Poisson Sampling.
     */
    set usePoissonSampling(value: boolean);
    /**
     * Gets if the current filter is set to ESM.
     */
    get useExponentialShadowMap(): boolean;
    /**
     * Sets the current filter is to ESM.
     */
    set useExponentialShadowMap(value: boolean);
    /**
     * Gets if the current filter is set to filtered ESM.
     */
    get useBlurExponentialShadowMap(): boolean;
    /**
     * Gets if the current filter is set to filtered  ESM.
     */
    set useBlurExponentialShadowMap(value: boolean);
    /**
     * Gets if the current filter is set to "close ESM" (using the inverse of the
     * exponential to prevent steep falloff artifacts).
     */
    get useCloseExponentialShadowMap(): boolean;
    /**
     * Sets the current filter to "close ESM" (using the inverse of the
     * exponential to prevent steep falloff artifacts).
     */
    set useCloseExponentialShadowMap(value: boolean);
    /**
     * Gets if the current filter is set to filtered "close ESM" (using the inverse of the
     * exponential to prevent steep falloff artifacts).
     */
    get useBlurCloseExponentialShadowMap(): boolean;
    /**
     * Sets the current filter to filtered "close ESM" (using the inverse of the
     * exponential to prevent steep falloff artifacts).
     */
    set useBlurCloseExponentialShadowMap(value: boolean);
    /**
     * Gets if the current filter is set to "PCF" (percentage closer filtering).
     */
    get usePercentageCloserFiltering(): boolean;
    /**
     * Sets the current filter to "PCF" (percentage closer filtering).
     */
    set usePercentageCloserFiltering(value: boolean);
    protected _filteringQuality: number;
    /**
     * Gets the PCF or PCSS Quality.
     * Only valid if usePercentageCloserFiltering or usePercentageCloserFiltering is true.
     */
    get filteringQuality(): number;
    /**
     * Sets the PCF or PCSS Quality.
     * Only valid if usePercentageCloserFiltering or usePercentageCloserFiltering is true.
     */
    set filteringQuality(filteringQuality: number);
    /**
     * Gets if the current filter is set to "PCSS" (contact hardening).
     */
    get useContactHardeningShadow(): boolean;
    /**
     * Sets the current filter to "PCSS" (contact hardening).
     */
    set useContactHardeningShadow(value: boolean);
    protected _contactHardeningLightSizeUVRatio: number;
    /**
     * Gets the Light Size (in shadow map uv unit) used in PCSS to determine the blocker search area and the penumbra size.
     * Using a ratio helps keeping shape stability independently of the map size.
     *
     * It does not account for the light projection as it was having too much
     * instability during the light setup or during light position changes.
     *
     * Only valid if useContactHardeningShadow is true.
     */
    get contactHardeningLightSizeUVRatio(): number;
    /**
     * Sets the Light Size (in shadow map uv unit) used in PCSS to determine the blocker search area and the penumbra size.
     * Using a ratio helps keeping shape stability independently of the map size.
     *
     * It does not account for the light projection as it was having too much
     * instability during the light setup or during light position changes.
     *
     * Only valid if useContactHardeningShadow is true.
     */
    set contactHardeningLightSizeUVRatio(contactHardeningLightSizeUVRatio: number);
    protected _darkness: number;
    /** Gets or sets the actual darkness of a shadow */
    get darkness(): number;
    set darkness(value: number);
    /**
     * Returns the darkness value (float). This can only decrease the actual darkness of a shadow.
     * 0 means strongest and 1 would means no shadow.
     * @returns the darkness.
     */
    getDarkness(): number;
    /**
     * Sets the darkness value (float). This can only decrease the actual darkness of a shadow.
     * @param darkness The darkness value 0 means strongest and 1 would means no shadow.
     * @returns the shadow generator allowing fluent coding.
     */
    setDarkness(darkness: number): ShadowGenerator;
    protected _transparencyShadow: boolean;
    /** Gets or sets the ability to have transparent shadow  */
    get transparencyShadow(): boolean;
    set transparencyShadow(value: boolean);
    /**
     * Sets the ability to have transparent shadow (boolean).
     * @param transparent True if transparent else False
     * @returns the shadow generator allowing fluent coding
     */
    setTransparencyShadow(transparent: boolean): ShadowGenerator;
    /**
     * Enables or disables shadows with varying strength based on the transparency
     * When it is enabled, the strength of the shadow is taken equal to mesh.visibility
     * If you enabled an alpha texture on your material, the alpha value red from the texture is also combined to compute the strength:
     *          mesh.visibility * alphaTexture.a
     * The texture used is the diffuse by default, but it can be set to the opacity by setting useOpacityTextureForTransparentShadow
     * Note that by definition transparencyShadow must be set to true for enableSoftTransparentShadow to work!
     */
    enableSoftTransparentShadow: boolean;
    /**
     * If this is true, use the opacity texture's alpha channel for transparent shadows instead of the diffuse one
     */
    useOpacityTextureForTransparentShadow: boolean;
    protected _shadowMap: Nullable<RenderTargetTexture>;
    protected _shadowMap2: Nullable<RenderTargetTexture>;
    /**
     * Gets the main RTT containing the shadow map (usually storing depth from the light point of view).
     * @returns The render target texture if present otherwise, null
     */
    getShadowMap(): Nullable<RenderTargetTexture>;
    /**
     * Gets the RTT used during rendering (can be a blurred version of the shadow map or the shadow map itself).
     * @returns The render target texture if the shadow map is present otherwise, null
     */
    getShadowMapForRendering(): Nullable<RenderTargetTexture>;
    /**
     * Gets the class name of that object
     * @returns "ShadowGenerator"
     */
    getClassName(): string;
    /**
     * Helper function to add a mesh and its descendants to the list of shadow casters.
     * @param mesh Mesh to add
     * @param includeDescendants boolean indicating if the descendants should be added. Default to true
     * @returns the Shadow Generator itself
     */
    addShadowCaster(mesh: AbstractMesh, includeDescendants?: boolean): ShadowGenerator;
    /**
     * Helper function to remove a mesh and its descendants from the list of shadow casters
     * @param mesh Mesh to remove
     * @param includeDescendants boolean indicating if the descendants should be removed. Default to true
     * @returns the Shadow Generator itself
     */
    removeShadowCaster(mesh: AbstractMesh, includeDescendants?: boolean): ShadowGenerator;
    /**
     * Controls the extent to which the shadows fade out at the edge of the frustum
     */
    frustumEdgeFalloff: number;
    protected _light: IShadowLight;
    /**
     * Returns the associated light object.
     * @returns the light generating the shadow
     */
    getLight(): IShadowLight;
    /**
     * If true the shadow map is generated by rendering the back face of the mesh instead of the front face.
     * This can help with self-shadowing as the geometry making up the back of objects is slightly offset.
     * It might on the other hand introduce peter panning.
     */
    forceBackFacesOnly: boolean;
    protected _camera: Nullable<Camera>;
    protected _getCamera(): Nullable<Camera>;
    protected _scene: Scene;
    protected _lightDirection: Vector3;
    protected _viewMatrix: Matrix;
    protected _projectionMatrix: Matrix;
    protected _transformMatrix: Matrix;
    protected _cachedPosition: Vector3;
    protected _cachedDirection: Vector3;
    protected _cachedDefines: string;
    protected _currentRenderId: number;
    protected _boxBlurPostprocess: Nullable<PostProcess>;
    protected _kernelBlurXPostprocess: Nullable<PostProcess>;
    protected _kernelBlurYPostprocess: Nullable<PostProcess>;
    protected _blurPostProcesses: PostProcess[];
    protected _mapSize: number;
    protected _currentFaceIndex: number;
    protected _currentFaceIndexCache: number;
    protected _textureType: number;
    protected _defaultTextureMatrix: Matrix;
    protected _storedUniqueId: Nullable<number>;
    protected _useUBO: boolean;
    protected _sceneUBOs: UniformBuffer[];
    protected _currentSceneUBO: UniformBuffer;
    /**
     * @internal
     */
    static _SceneComponentInitialization: (scene: Scene) => void;
    /**
     * Gets or sets the size of the texture what stores the shadows
     */
    get mapSize(): number;
    set mapSize(size: number);
    /**
     * Creates a ShadowGenerator object.
     * A ShadowGenerator is the required tool to use the shadows.
     * Each light casting shadows needs to use its own ShadowGenerator.
     * Documentation : https://doc.babylonjs.com/features/featuresDeepDive/lights/shadows
     * @param mapSize The size of the texture what stores the shadows. Example : 1024.
     * @param light The light object generating the shadows.
     * @param usefullFloatFirst By default the generator will try to use half float textures but if you need precision (for self shadowing for instance), you can use this option to enforce full float texture.
     * @param camera Camera associated with this shadow generator (default: null). If null, takes the scene active camera at the time we need to access it
     */
    constructor(mapSize: number, light: IShadowLight, usefullFloatFirst?: boolean, camera?: Nullable<Camera>);
    protected _initializeGenerator(): void;
    protected _createTargetRenderTexture(): void;
    protected _initializeShadowMap(): void;
    protected _initializeBlurRTTAndPostProcesses(): void;
    protected _renderForShadowMap(opaqueSubMeshes: SmartArray<SubMesh>, alphaTestSubMeshes: SmartArray<SubMesh>, transparentSubMeshes: SmartArray<SubMesh>, depthOnlySubMeshes: SmartArray<SubMesh>): void;
    protected _bindCustomEffectForRenderSubMeshForShadowMap(subMesh: SubMesh, effect: Effect, mesh: AbstractMesh): void;
    protected _renderSubMeshForShadowMap(subMesh: SubMesh, isTransparent?: boolean): void;
    protected _applyFilterValues(): void;
    /**
     * Forces all the attached effect to compile to enable rendering only once ready vs. lazily compiling effects.
     * @param onCompiled Callback triggered at the and of the effects compilation
     * @param options Sets of optional options forcing the compilation with different modes
     */
    forceCompilation(onCompiled?: (generator: IShadowGenerator) => void, options?: Partial<{
        useInstances: boolean;
    }>): void;
    /**
     * Forces all the attached effect to compile to enable rendering only once ready vs. lazily compiling effects.
     * @param options Sets of optional options forcing the compilation with different modes
     * @returns A promise that resolves when the compilation completes
     */
    forceCompilationAsync(options?: Partial<{
        useInstances: boolean;
    }>): Promise<void>;
    protected _isReadyCustomDefines(defines: any, subMesh: SubMesh, useInstances: boolean): void;
    private _prepareShadowDefines;
    /**
     * Determine whether the shadow generator is ready or not (mainly all effects and related post processes needs to be ready).
     * @param subMesh The submesh we want to render in the shadow map
     * @param useInstances Defines whether will draw in the map using instances
     * @param isTransparent Indicates that isReady is called for a transparent subMesh
     * @returns true if ready otherwise, false
     */
    isReady(subMesh: SubMesh, useInstances: boolean, isTransparent: boolean): boolean;
    /**
     * Prepare all the defines in a material relying on a shadow map at the specified light index.
     * @param defines Defines of the material we want to update
     * @param lightIndex Index of the light in the enabled light list of the material
     */
    prepareDefines(defines: any, lightIndex: number): void;
    /**
     * Binds the shadow related information inside of an effect (information like near, far, darkness...
     * defined in the generator but impacting the effect).
     * @param lightIndex Index of the light in the enabled light list of the material owning the effect
     * @param effect The effect we are binding the information for
     */
    bindShadowLight(lightIndex: string, effect: Effect): void;
    /**
     * Gets the transformation matrix used to project the meshes into the map from the light point of view.
     * (eq to shadow projection matrix * light transform matrix)
     * @returns The transform matrix used to create the shadow map
     */
    getTransformMatrix(): Matrix;
    /**
     * Recreates the shadow map dependencies like RTT and post processes. This can be used during the switch between
     * Cube and 2D textures for instance.
     */
    recreateShadowMap(): void;
    protected _disposeBlurPostProcesses(): void;
    protected _disposeRTTandPostProcesses(): void;
    protected _disposeSceneUBOs(): void;
    /**
     * Disposes the ShadowGenerator.
     * Returns nothing.
     */
    dispose(): void;
    /**
     * Serializes the shadow generator setup to a json object.
     * @returns The serialized JSON object
     */
    serialize(): any;
    /**
     * Parses a serialized ShadowGenerator and returns a new ShadowGenerator.
     * @param parsedShadowGenerator The JSON object to parse
     * @param scene The scene to create the shadow map for
     * @param constr A function that builds a shadow generator or undefined to create an instance of the default shadow generator
     * @returns The parsed shadow generator
     */
    static Parse(parsedShadowGenerator: any, scene: Scene, constr?: (mapSize: number, light: IShadowLight, camera: Nullable<Camera>) => ShadowGenerator): ShadowGenerator;
}

}
declare module "babylonjs/Lights/Shadows/shadowGeneratorSceneComponent" {
import { Scene } from "babylonjs/scene";
import { ISceneSerializableComponent } from "babylonjs/sceneComponent";
import { AbstractScene } from "babylonjs/abstractScene";
/**
 * Defines the shadow generator component responsible to manage any shadow generators
 * in a given scene.
 */
export class ShadowGeneratorSceneComponent implements ISceneSerializableComponent {
    /**
     * The component name helpful to identify the component in the list of scene components.
     */
    readonly name: string;
    /**
     * The scene the component belongs to.
     */
    scene: Scene;
    /**
     * Creates a new instance of the component for the given scene
     * @param scene Defines the scene to register the component in
     */
    constructor(scene: Scene);
    /**
     * Registers the component in a given scene
     */
    register(): void;
    /**
     * Rebuilds the elements related to this component in case of
     * context lost for instance.
     */
    rebuild(): void;
    /**
     * Serializes the component data to the specified json object
     * @param serializationObject The object to serialize to
     */
    serialize(serializationObject: any): void;
    /**
     * Adds all the elements from the container to the scene
     * @param container the container holding the elements
     */
    addFromContainer(container: AbstractScene): void;
    /**
     * Removes all the elements in the container from the scene
     * @param container contains the elements to remove
     * @param dispose if the removed element should be disposed (default: false)
     */
    removeFromContainer(container: AbstractScene, dispose?: boolean): void;
    /**
     * Rebuilds the elements related to this component in case of
     * context lost for instance.
     */
    dispose(): void;
    private _gatherRenderTargets;
}

}
declare module "babylonjs/Lights/spotLight" {
import { Nullable } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
import { Matrix, Vector3 } from "babylonjs/Maths/math.vector";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { Effect } from "babylonjs/Materials/effect";
import { BaseTexture } from "babylonjs/Materials/Textures/baseTexture";
import { Light } from "babylonjs/Lights/light";
import { ShadowLight } from "babylonjs/Lights/shadowLight";
import { Camera } from "babylonjs/Cameras/camera";
/**
 * A spot light is defined by a position, a direction, an angle, and an exponent.
 * These values define a cone of light starting from the position, emitting toward the direction.
 * The angle, in radians, defines the size (field of illumination) of the spotlight's conical beam,
 * and the exponent defines the speed of the decay of the light with distance (reach).
 * Documentation: https://doc.babylonjs.com/features/featuresDeepDive/lights/lights_introduction
 */
export class SpotLight extends ShadowLight {
    private _angle;
    private _innerAngle;
    private _cosHalfAngle;
    private _lightAngleScale;
    private _lightAngleOffset;
    /**
     * Gets the cone angle of the spot light in Radians.
     */
    get angle(): number;
    /**
     * Sets the cone angle of the spot light in Radians.
     */
    set angle(value: number);
    /**
     * Only used in gltf falloff mode, this defines the angle where
     * the directional falloff will start before cutting at angle which could be seen
     * as outer angle.
     */
    get innerAngle(): number;
    /**
     * Only used in gltf falloff mode, this defines the angle where
     * the directional falloff will start before cutting at angle which could be seen
     * as outer angle.
     */
    set innerAngle(value: number);
    private _shadowAngleScale;
    /**
     * Allows scaling the angle of the light for shadow generation only.
     */
    get shadowAngleScale(): number;
    /**
     * Allows scaling the angle of the light for shadow generation only.
     */
    set shadowAngleScale(value: number);
    /**
     * The light decay speed with the distance from the emission spot.
     */
    exponent: number;
    private _projectionTextureMatrix;
    /**
     * Allows reading the projection texture
     */
    get projectionTextureMatrix(): Matrix;
    protected _projectionTextureLightNear: number;
    /**
     * Gets the near clip of the Spotlight for texture projection.
     */
    get projectionTextureLightNear(): number;
    /**
     * Sets the near clip of the Spotlight for texture projection.
     */
    set projectionTextureLightNear(value: number);
    protected _projectionTextureLightFar: number;
    /**
     * Gets the far clip of the Spotlight for texture projection.
     */
    get projectionTextureLightFar(): number;
    /**
     * Sets the far clip of the Spotlight for texture projection.
     */
    set projectionTextureLightFar(value: number);
    protected _projectionTextureUpDirection: Vector3;
    /**
     * Gets the Up vector of the Spotlight for texture projection.
     */
    get projectionTextureUpDirection(): Vector3;
    /**
     * Sets the Up vector of the Spotlight for texture projection.
     */
    set projectionTextureUpDirection(value: Vector3);
    private _projectionTexture;
    /**
     * Gets the projection texture of the light.
     */
    get projectionTexture(): Nullable<BaseTexture>;
    /**
     * Sets the projection texture of the light.
     */
    set projectionTexture(value: Nullable<BaseTexture>);
    private static _IsProceduralTexture;
    private static _IsTexture;
    private _projectionTextureViewLightDirty;
    private _projectionTextureProjectionLightDirty;
    private _projectionTextureDirty;
    private _projectionTextureViewTargetVector;
    private _projectionTextureViewLightMatrix;
    private _projectionTextureProjectionLightMatrix;
    /**
     * Gets or sets the light projection matrix as used by the projection texture
     */
    get projectionTextureProjectionLightMatrix(): Matrix;
    set projectionTextureProjectionLightMatrix(projection: Matrix);
    private _projectionTextureScalingMatrix;
    /**
     * Creates a SpotLight object in the scene. A spot light is a simply light oriented cone.
     * It can cast shadows.
     * Documentation : https://doc.babylonjs.com/features/featuresDeepDive/lights/lights_introduction
     * @param name The light friendly name
     * @param position The position of the spot light in the scene
     * @param direction The direction of the light in the scene
     * @param angle The cone angle of the light in Radians
     * @param exponent The light decay speed with the distance from the emission spot
     * @param scene The scene the lights belongs to
     */
    constructor(name: string, position: Vector3, direction: Vector3, angle: number, exponent: number, scene: Scene);
    /**
     * Returns the string "SpotLight".
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Returns the integer 2.
     * @returns The light Type id as a constant defines in Light.LIGHTTYPEID_x
     */
    getTypeID(): number;
    /**
     * Overrides the direction setter to recompute the projection texture view light Matrix.
     * @param value
     */
    protected _setDirection(value: Vector3): void;
    /**
     * Overrides the position setter to recompute the projection texture view light Matrix.
     * @param value
     */
    protected _setPosition(value: Vector3): void;
    /**
     * Sets the passed matrix "matrix" as perspective projection matrix for the shadows and the passed view matrix with the fov equal to the SpotLight angle and and aspect ratio of 1.0.
     * Returns the SpotLight.
     * @param matrix
     * @param viewMatrix
     * @param renderList
     */
    protected _setDefaultShadowProjectionMatrix(matrix: Matrix, viewMatrix: Matrix, renderList: Array<AbstractMesh>): void;
    protected _computeProjectionTextureViewLightMatrix(): void;
    protected _computeProjectionTextureProjectionLightMatrix(): void;
    /**
     * Main function for light texture projection matrix computing.
     */
    protected _computeProjectionTextureMatrix(): void;
    protected _buildUniformLayout(): void;
    private _computeAngleValues;
    /**
     * Sets the passed Effect "effect" with the Light textures.
     * @param effect The effect to update
     * @param lightIndex The index of the light in the effect to update
     * @returns The light
     */
    transferTexturesToEffect(effect: Effect, lightIndex: string): Light;
    /**
     * Sets the passed Effect object with the SpotLight transformed position (or position if not parented) and normalized direction.
     * @param effect The effect to update
     * @param lightIndex The index of the light in the effect to update
     * @returns The spot light
     */
    transferToEffect(effect: Effect, lightIndex: string): SpotLight;
    transferToNodeMaterialEffect(effect: Effect, lightDataUniformName: string): this;
    /**
     * Disposes the light and the associated resources.
     */
    dispose(): void;
    /**
     * Gets the minZ used for shadow according to both the scene and the light.
     * @param activeCamera The camera we are returning the min for
     * @returns the depth min z
     */
    getDepthMinZ(activeCamera: Camera): number;
    /**
     * Gets the maxZ used for shadow according to both the scene and the light.
     * @param activeCamera The camera we are returning the max for
     * @returns the depth max z
     */
    getDepthMaxZ(activeCamera: Camera): number;
    /**
     * Prepares the list of defines specific to the light type.
     * @param defines the list of defines
     * @param lightIndex defines the index of the light for the effect
     */
    prepareLightSpecificDefines(defines: any, lightIndex: number): void;
}

}
declare module "babylonjs/Loading/index" {
export * from "babylonjs/Loading/loadingScreen";
export * from "babylonjs/Loading/Plugins/index";
export * from "babylonjs/Loading/sceneLoader";
export * from "babylonjs/Loading/sceneLoaderFlags";

}
declare module "babylonjs/Loading/loadingScreen" {
/**
 * Interface used to present a loading screen while loading a scene
 * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/customLoadingScreen
 */
export interface ILoadingScreen {
    /**
     * Function called to display the loading screen
     */
    displayLoadingUI: () => void;
    /**
     * Function called to hide the loading screen
     */
    hideLoadingUI: () => void;
    /**
     * Gets or sets the color to use for the background
     */
    loadingUIBackgroundColor: string;
    /**
     * Gets or sets the text to display while loading
     */
    loadingUIText: string;
}
/**
 * Class used for the default loading screen
 * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/customLoadingScreen
 */
export class DefaultLoadingScreen implements ILoadingScreen {
    private _renderingCanvas;
    private _loadingText;
    private _loadingDivBackgroundColor;
    private _loadingDiv;
    private _loadingTextDiv;
    private _style;
    /** Gets or sets the logo url to use for the default loading screen */
    static DefaultLogoUrl: string;
    /** Gets or sets the spinner url to use for the default loading screen */
    static DefaultSpinnerUrl: string;
    /**
     * Creates a new default loading screen
     * @param _renderingCanvas defines the canvas used to render the scene
     * @param _loadingText defines the default text to display
     * @param _loadingDivBackgroundColor defines the default background color
     */
    constructor(_renderingCanvas: HTMLCanvasElement, _loadingText?: string, _loadingDivBackgroundColor?: string);
    /**
     * Function called to display the loading screen
     */
    displayLoadingUI(): void;
    /**
     * Function called to hide the loading screen
     */
    hideLoadingUI(): void;
    /**
     * Gets or sets the text to display while loading
     */
    set loadingUIText(text: string);
    get loadingUIText(): string;
    /**
     * Gets or sets the color to use for the background
     */
    get loadingUIBackgroundColor(): string;
    set loadingUIBackgroundColor(color: string);
    private _resizeLoadingUI;
}

}
declare module "babylonjs/Loading/Plugins/babylonFileLoader" {
/** @internal */
export var _BabylonLoaderRegistered: boolean;
/**
 * Helps setting up some configuration for the babylon file loader.
 */
export class BabylonFileLoaderConfiguration {
    /**
     * The loader does not allow injecting custom physics engine into the plugins.
     * Unfortunately in ES6, we need to manually inject them into the plugin.
     * So you could set this variable to your engine import to make it work.
     */
    static LoaderInjectedPhysicsEngine: any;
}

}
declare module "babylonjs/Loading/Plugins/index" {
export * from "babylonjs/Loading/Plugins/babylonFileLoader";

}
declare module "babylonjs/Loading/sceneLoader" {
import { Observable } from "babylonjs/Misc/observable";
import { Nullable } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
import { Engine } from "babylonjs/Engines/engine";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { AnimationGroup } from "babylonjs/Animations/animationGroup";
import { AssetContainer } from "babylonjs/assetContainer";
import { IParticleSystem } from "babylonjs/Particles/IParticleSystem";
import { Skeleton } from "babylonjs/Bones/skeleton";
import { IFileRequest } from "babylonjs/Misc/fileRequest";
import { WebRequest } from "babylonjs/Misc/webRequest";
import { LoadFileError } from "babylonjs/Misc/fileTools";
import { TransformNode } from "babylonjs/Meshes/transformNode";
import { Geometry } from "babylonjs/Meshes/geometry";
import { Light } from "babylonjs/Lights/light";
/**
 * Type used for the success callback of ImportMesh
 */
export type SceneLoaderSuccessCallback = (meshes: AbstractMesh[], particleSystems: IParticleSystem[], skeletons: Skeleton[], animationGroups: AnimationGroup[], transformNodes: TransformNode[], geometries: Geometry[], lights: Light[]) => void;
/**
 * Interface used for the result of ImportMeshAsync
 */
export interface ISceneLoaderAsyncResult {
    /**
     * The array of loaded meshes
     */
    readonly meshes: AbstractMesh[];
    /**
     * The array of loaded particle systems
     */
    readonly particleSystems: IParticleSystem[];
    /**
     * The array of loaded skeletons
     */
    readonly skeletons: Skeleton[];
    /**
     * The array of loaded animation groups
     */
    readonly animationGroups: AnimationGroup[];
    /**
     * The array of loaded transform nodes
     */
    readonly transformNodes: TransformNode[];
    /**
     * The array of loaded geometries
     */
    readonly geometries: Geometry[];
    /**
     * The array of loaded lights
     */
    readonly lights: Light[];
}
/**
 * Interface used to represent data loading progression
 */
export interface ISceneLoaderProgressEvent {
    /**
     * Defines if data length to load can be evaluated
     */
    readonly lengthComputable: boolean;
    /**
     * Defines the loaded data length
     */
    readonly loaded: number;
    /**
     * Defines the data length to load
     */
    readonly total: number;
}
/**
 * Interface used by SceneLoader plugins to define supported file extensions
 */
export interface ISceneLoaderPluginExtensions {
    /**
     * Defines the list of supported extensions
     */
    [extension: string]: {
        isBinary: boolean;
    };
}
/**
 * Interface used by SceneLoader plugin factory
 */
export interface ISceneLoaderPluginFactory {
    /**
     * Defines the name of the factory
     */
    name: string;
    /**
     * Function called to create a new plugin
     * @returns the new plugin
     */
    createPlugin(): ISceneLoaderPlugin | ISceneLoaderPluginAsync;
    /**
     * The callback that returns true if the data can be directly loaded.
     * @param data string containing the file data
     * @returns if the data can be loaded directly
     */
    canDirectLoad?(data: string): boolean;
}
/**
 * Interface used to define the base of ISceneLoaderPlugin and ISceneLoaderPluginAsync
 */
export interface ISceneLoaderPluginBase {
    /**
     * The friendly name of this plugin.
     */
    name: string;
    /**
     * The file extensions supported by this plugin.
     */
    extensions: string | ISceneLoaderPluginExtensions;
    /**
     * The callback called when loading from a url.
     * @param scene scene loading this url
     * @param fileOrUrl file or url to load
     * @param onSuccess callback called when the file successfully loads
     * @param onProgress callback called while file is loading (if the server supports this mode)
     * @param useArrayBuffer defines a boolean indicating that date must be returned as ArrayBuffer
     * @param onError callback called when the file fails to load
     * @returns a file request object
     */
    loadFile?(scene: Scene, fileOrUrl: File | string, onSuccess: (data: any, responseURL?: string) => void, onProgress?: (ev: ISceneLoaderProgressEvent) => void, useArrayBuffer?: boolean, onError?: (request?: WebRequest, exception?: LoadFileError) => void): IFileRequest;
    /**
     * The callback that returns true if the data can be directly loaded.
     * @param data string containing the file data
     * @returns if the data can be loaded directly
     */
    canDirectLoad?(data: string): boolean;
    /**
     * The callback that returns the data to pass to the plugin if the data can be directly loaded.
     * @param scene scene loading this data
     * @param data string containing the data
     * @returns data to pass to the plugin
     */
    directLoad?(scene: Scene, data: string): any;
    /**
     * The callback that allows custom handling of the root url based on the response url.
     * @param rootUrl the original root url
     * @param responseURL the response url if available
     * @returns the new root url
     */
    rewriteRootURL?(rootUrl: string, responseURL?: string): string;
}
/**
 * Interface used to define a SceneLoader plugin
 */
export interface ISceneLoaderPlugin extends ISceneLoaderPluginBase {
    /**
     * Import meshes into a scene.
     * @param meshesNames An array of mesh names, a single mesh name, or empty string for all meshes that filter what meshes are imported
     * @param scene The scene to import into
     * @param data The data to import
     * @param rootUrl The root url for scene and resources
     * @param meshes The meshes array to import into
     * @param particleSystems The particle systems array to import into
     * @param skeletons The skeletons array to import into
     * @param onError The callback when import fails
     * @returns True if successful or false otherwise
     */
    importMesh(meshesNames: any, scene: Scene, data: any, rootUrl: string, meshes: AbstractMesh[], particleSystems: IParticleSystem[], skeletons: Skeleton[], onError?: (message: string, exception?: any) => void): boolean;
    /**
     * Load into a scene.
     * @param scene The scene to load into
     * @param data The data to import
     * @param rootUrl The root url for scene and resources
     * @param onError The callback when import fails
     * @returns True if successful or false otherwise
     */
    load(scene: Scene, data: any, rootUrl: string, onError?: (message: string, exception?: any) => void): boolean;
    /**
     * Load into an asset container.
     * @param scene The scene to load into
     * @param data The data to import
     * @param rootUrl The root url for scene and resources
     * @param onError The callback when import fails
     * @returns The loaded asset container
     */
    loadAssetContainer(scene: Scene, data: any, rootUrl: string, onError?: (message: string, exception?: any) => void): AssetContainer;
}
/**
 * Interface used to define an async SceneLoader plugin
 */
export interface ISceneLoaderPluginAsync extends ISceneLoaderPluginBase {
    /**
     * Import meshes into a scene.
     * @param meshesNames An array of mesh names, a single mesh name, or empty string for all meshes that filter what meshes are imported
     * @param scene The scene to import into
     * @param data The data to import
     * @param rootUrl The root url for scene and resources
     * @param onProgress The callback when the load progresses
     * @param fileName Defines the name of the file to load
     * @returns The loaded objects (e.g. meshes, particle systems, skeletons, animation groups, etc.)
     */
    importMeshAsync(meshesNames: any, scene: Scene, data: any, rootUrl: string, onProgress?: (event: ISceneLoaderProgressEvent) => void, fileName?: string): Promise<ISceneLoaderAsyncResult>;
    /**
     * Load into a scene.
     * @param scene The scene to load into
     * @param data The data to import
     * @param rootUrl The root url for scene and resources
     * @param onProgress The callback when the load progresses
     * @param fileName Defines the name of the file to load
     * @returns Nothing
     */
    loadAsync(scene: Scene, data: any, rootUrl: string, onProgress?: (event: ISceneLoaderProgressEvent) => void, fileName?: string): Promise<void>;
    /**
     * Load into an asset container.
     * @param scene The scene to load into
     * @param data The data to import
     * @param rootUrl The root url for scene and resources
     * @param onProgress The callback when the load progresses
     * @param fileName Defines the name of the file to load
     * @returns The loaded asset container
     */
    loadAssetContainerAsync(scene: Scene, data: any, rootUrl: string, onProgress?: (event: ISceneLoaderProgressEvent) => void, fileName?: string): Promise<AssetContainer>;
}
/**
 * Mode that determines how to handle old animation groups before loading new ones.
 */
export enum SceneLoaderAnimationGroupLoadingMode {
    /**
     * Reset all old animations to initial state then dispose them.
     */
    Clean = 0,
    /**
     * Stop all old animations.
     */
    Stop = 1,
    /**
     * Restart old animations from first frame.
     */
    Sync = 2,
    /**
     * Old animations remains untouched.
     */
    NoSync = 3
}
/**
 * Defines a plugin registered by the SceneLoader
 */
interface IRegisteredPlugin {
    /**
     * Defines the plugin to use
     */
    plugin: ISceneLoaderPlugin | ISceneLoaderPluginAsync | ISceneLoaderPluginFactory;
    /**
     * Defines if the plugin supports binary data
     */
    isBinary: boolean;
}
/**
 * Class used to load scene from various file formats using registered plugins
 * @see https://doc.babylonjs.com/features/featuresDeepDive/importers/loadingFileTypes
 */
export class SceneLoader {
    /**
     * No logging while loading
     */
    static readonly NO_LOGGING: number;
    /**
     * Minimal logging while loading
     */
    static readonly MINIMAL_LOGGING: number;
    /**
     * Summary logging while loading
     */
    static readonly SUMMARY_LOGGING: number;
    /**
     * Detailed logging while loading
     */
    static readonly DETAILED_LOGGING: number;
    /**
     * Gets or sets a boolean indicating if entire scene must be loaded even if scene contains incremental data
     */
    static get ForceFullSceneLoadingForIncremental(): boolean;
    static set ForceFullSceneLoadingForIncremental(value: boolean);
    /**
     * Gets or sets a boolean indicating if loading screen must be displayed while loading a scene
     */
    static get ShowLoadingScreen(): boolean;
    static set ShowLoadingScreen(value: boolean);
    /**
     * Defines the current logging level (while loading the scene)
     * @ignorenaming
     */
    static get loggingLevel(): number;
    static set loggingLevel(value: number);
    /**
     * Gets or set a boolean indicating if matrix weights must be cleaned upon loading
     */
    static get CleanBoneMatrixWeights(): boolean;
    static set CleanBoneMatrixWeights(value: boolean);
    /**
     * Event raised when a plugin is used to load a scene
     */
    static OnPluginActivatedObservable: Observable<ISceneLoaderPlugin | ISceneLoaderPluginAsync>;
    private static _RegisteredPlugins;
    private static _ShowingLoadingScreen;
    /**
     * Gets the default plugin (used to load Babylon files)
     * @returns the .babylon plugin
     */
    static GetDefaultPlugin(): IRegisteredPlugin;
    private static _GetPluginForExtension;
    private static _GetPluginForDirectLoad;
    private static _GetPluginForFilename;
    private static _GetDirectLoad;
    private static _FormatErrorMessage;
    private static _LoadData;
    private static _GetFileInfo;
    /**
     * Gets a plugin that can load the given extension
     * @param extension defines the extension to load
     * @returns a plugin or null if none works
     */
    static GetPluginForExtension(extension: string): ISceneLoaderPlugin | ISceneLoaderPluginAsync | ISceneLoaderPluginFactory;
    /**
     * Gets a boolean indicating that the given extension can be loaded
     * @param extension defines the extension to load
     * @returns true if the extension is supported
     */
    static IsPluginForExtensionAvailable(extension: string): boolean;
    /**
     * Adds a new plugin to the list of registered plugins
     * @param plugin defines the plugin to add
     */
    static RegisterPlugin(plugin: ISceneLoaderPlugin | ISceneLoaderPluginAsync): void;
    /**
     * Import meshes into a scene
     * @param meshNames an array of mesh names, a single mesh name, or empty string for all meshes that filter what meshes are imported
     * @param rootUrl a string that defines the root url for the scene and resources or the concatenation of rootURL and filename (e.g. http://example.com/test.glb)
     * @param sceneFilename a string that defines the name of the scene file or starts with "data:" following by the stringified version of the scene or a File object (default: empty string)
     * @param scene the instance of BABYLON.Scene to append to
     * @param onSuccess a callback with a list of imported meshes, particleSystems, skeletons, and animationGroups when import succeeds
     * @param onProgress a callback with a progress event for each file being loaded
     * @param onError a callback with the scene, a message, and possibly an exception when import fails
     * @param pluginExtension the extension used to determine the plugin
     * @returns The loaded plugin
     */
    static ImportMesh(meshNames: any, rootUrl: string, sceneFilename?: string | File, scene?: Nullable<Scene>, onSuccess?: Nullable<SceneLoaderSuccessCallback>, onProgress?: Nullable<(event: ISceneLoaderProgressEvent) => void>, onError?: Nullable<(scene: Scene, message: string, exception?: any) => void>, pluginExtension?: Nullable<string>): Nullable<ISceneLoaderPlugin | ISceneLoaderPluginAsync>;
    /**
     * Import meshes into a scene
     * @param meshNames an array of mesh names, a single mesh name, or empty string for all meshes that filter what meshes are imported
     * @param rootUrl a string that defines the root url for the scene and resources or the concatenation of rootURL and filename (e.g. http://example.com/test.glb)
     * @param sceneFilename a string that defines the name of the scene file or starts with "data:" following by the stringified version of the scene or a File object (default: empty string)
     * @param scene the instance of BABYLON.Scene to append to
     * @param onProgress a callback with a progress event for each file being loaded
     * @param pluginExtension the extension used to determine the plugin
     * @returns The loaded list of imported meshes, particle systems, skeletons, and animation groups
     */
    static ImportMeshAsync(meshNames: any, rootUrl: string, sceneFilename?: string | File, scene?: Nullable<Scene>, onProgress?: Nullable<(event: ISceneLoaderProgressEvent) => void>, pluginExtension?: Nullable<string>): Promise<ISceneLoaderAsyncResult>;
    /**
     * Load a scene
     * @param rootUrl a string that defines the root url for the scene and resources or the concatenation of rootURL and filename (e.g. http://example.com/test.glb)
     * @param sceneFilename a string that defines the name of the scene file or starts with "data:" following by the stringified version of the scene or a File object (default: empty string)
     * @param engine is the instance of BABYLON.Engine to use to create the scene
     * @param onSuccess a callback with the scene when import succeeds
     * @param onProgress a callback with a progress event for each file being loaded
     * @param onError a callback with the scene, a message, and possibly an exception when import fails
     * @param pluginExtension the extension used to determine the plugin
     * @returns The loaded plugin
     */
    static Load(rootUrl: string, sceneFilename?: string | File, engine?: Nullable<Engine>, onSuccess?: Nullable<(scene: Scene) => void>, onProgress?: Nullable<(event: ISceneLoaderProgressEvent) => void>, onError?: Nullable<(scene: Scene, message: string, exception?: any) => void>, pluginExtension?: Nullable<string>): Nullable<ISceneLoaderPlugin | ISceneLoaderPluginAsync>;
    /**
     * Load a scene
     * @param rootUrl a string that defines the root url for the scene and resources or the concatenation of rootURL and filename (e.g. http://example.com/test.glb)
     * @param sceneFilename a string that defines the name of the scene file or starts with "data:" following by the stringified version of the scene or a File object (default: empty string)
     * @param engine is the instance of BABYLON.Engine to use to create the scene
     * @param onProgress a callback with a progress event for each file being loaded
     * @param pluginExtension the extension used to determine the plugin
     * @returns The loaded scene
     */
    static LoadAsync(rootUrl: string, sceneFilename?: string | File, engine?: Nullable<Engine>, onProgress?: Nullable<(event: ISceneLoaderProgressEvent) => void>, pluginExtension?: Nullable<string>): Promise<Scene>;
    /**
     * Append a scene
     * @param rootUrl a string that defines the root url for the scene and resources or the concatenation of rootURL and filename (e.g. http://example.com/test.glb)
     * @param sceneFilename a string that defines the name of the scene file or starts with "data:" following by the stringified version of the scene or a File object (default: empty string)
     * @param scene is the instance of BABYLON.Scene to append to
     * @param onSuccess a callback with the scene when import succeeds
     * @param onProgress a callback with a progress event for each file being loaded
     * @param onError a callback with the scene, a message, and possibly an exception when import fails
     * @param pluginExtension the extension used to determine the plugin
     * @returns The loaded plugin
     */
    static Append(rootUrl: string, sceneFilename?: string | File, scene?: Nullable<Scene>, onSuccess?: Nullable<(scene: Scene) => void>, onProgress?: Nullable<(event: ISceneLoaderProgressEvent) => void>, onError?: Nullable<(scene: Scene, message: string, exception?: any) => void>, pluginExtension?: Nullable<string>): Nullable<ISceneLoaderPlugin | ISceneLoaderPluginAsync>;
    /**
     * Append a scene
     * @param rootUrl a string that defines the root url for the scene and resources or the concatenation of rootURL and filename (e.g. http://example.com/test.glb)
     * @param sceneFilename a string that defines the name of the scene file or starts with "data:" following by the stringified version of the scene or a File object (default: empty string)
     * @param scene is the instance of BABYLON.Scene to append to
     * @param onProgress a callback with a progress event for each file being loaded
     * @param pluginExtension the extension used to determine the plugin
     * @returns The given scene
     */
    static AppendAsync(rootUrl: string, sceneFilename?: string | File, scene?: Nullable<Scene>, onProgress?: Nullable<(event: ISceneLoaderProgressEvent) => void>, pluginExtension?: Nullable<string>): Promise<Scene>;
    /**
     * Load a scene into an asset container
     * @param rootUrl a string that defines the root url for the scene and resources or the concatenation of rootURL and filename (e.g. http://example.com/test.glb)
     * @param sceneFilename a string that defines the name of the scene file or starts with "data:" following by the stringified version of the scene or a File object (default: empty string)
     * @param scene is the instance of BABYLON.Scene to append to (default: last created scene)
     * @param onSuccess a callback with the scene when import succeeds
     * @param onProgress a callback with a progress event for each file being loaded
     * @param onError a callback with the scene, a message, and possibly an exception when import fails
     * @param pluginExtension the extension used to determine the plugin
     * @returns The loaded plugin
     */
    static LoadAssetContainer(rootUrl: string, sceneFilename?: string | File, scene?: Nullable<Scene>, onSuccess?: Nullable<(assets: AssetContainer) => void>, onProgress?: Nullable<(event: ISceneLoaderProgressEvent) => void>, onError?: Nullable<(scene: Scene, message: string, exception?: any) => void>, pluginExtension?: Nullable<string>): Nullable<ISceneLoaderPlugin | ISceneLoaderPluginAsync>;
    /**
     * Load a scene into an asset container
     * @param rootUrl a string that defines the root url for the scene and resources or the concatenation of rootURL and filename (e.g. http://example.com/test.glb)
     * @param sceneFilename a string that defines the name of the scene file or starts with "data:" following by the stringified version of the scene (default: empty string)
     * @param scene is the instance of Scene to append to
     * @param onProgress a callback with a progress event for each file being loaded
     * @param pluginExtension the extension used to determine the plugin
     * @returns The loaded asset container
     */
    static LoadAssetContainerAsync(rootUrl: string, sceneFilename?: string | File, scene?: Nullable<Scene>, onProgress?: Nullable<(event: ISceneLoaderProgressEvent) => void>, pluginExtension?: Nullable<string>): Promise<AssetContainer>;
    /**
     * Import animations from a file into a scene
     * @param rootUrl a string that defines the root url for the scene and resources or the concatenation of rootURL and filename (e.g. http://example.com/test.glb)
     * @param sceneFilename a string that defines the name of the scene file or starts with "data:" following by the stringified version of the scene or a File object (default: empty string)
     * @param scene is the instance of BABYLON.Scene to append to (default: last created scene)
     * @param overwriteAnimations when true, animations are cleaned before importing new ones. Animations are appended otherwise
     * @param animationGroupLoadingMode defines how to handle old animations groups before importing new ones
     * @param targetConverter defines a function used to convert animation targets from loaded scene to current scene (default: search node by name)
     * @param onSuccess a callback with the scene when import succeeds
     * @param onProgress a callback with a progress event for each file being loaded
     * @param onError a callback with the scene, a message, and possibly an exception when import fails
     * @param pluginExtension the extension used to determine the plugin
     */
    static ImportAnimations(rootUrl: string, sceneFilename?: string | File, scene?: Nullable<Scene>, overwriteAnimations?: boolean, animationGroupLoadingMode?: SceneLoaderAnimationGroupLoadingMode, targetConverter?: Nullable<(target: any) => any>, onSuccess?: Nullable<(scene: Scene) => void>, onProgress?: Nullable<(event: ISceneLoaderProgressEvent) => void>, onError?: Nullable<(scene: Scene, message: string, exception?: any) => void>, pluginExtension?: Nullable<string>): void;
    /**
     * Import animations from a file into a scene
     * @param rootUrl a string that defines the root url for the scene and resources or the concatenation of rootURL and filename (e.g. http://example.com/test.glb)
     * @param sceneFilename a string that defines the name of the scene file or starts with "data:" following by the stringified version of the scene or a File object (default: empty string)
     * @param scene is the instance of BABYLON.Scene to append to (default: last created scene)
     * @param overwriteAnimations when true, animations are cleaned before importing new ones. Animations are appended otherwise
     * @param animationGroupLoadingMode defines how to handle old animations groups before importing new ones
     * @param targetConverter defines a function used to convert animation targets from loaded scene to current scene (default: search node by name)
     * @param onSuccess a callback with the scene when import succeeds
     * @param onProgress a callback with a progress event for each file being loaded
     * @param onError a callback with the scene, a message, and possibly an exception when import fails
     * @param pluginExtension the extension used to determine the plugin
     * @returns the updated scene with imported animations
     */
    static ImportAnimationsAsync(rootUrl: string, sceneFilename?: string | File, scene?: Nullable<Scene>, overwriteAnimations?: boolean, animationGroupLoadingMode?: SceneLoaderAnimationGroupLoadingMode, targetConverter?: Nullable<(target: any) => any>, onSuccess?: Nullable<(scene: Scene) => void>, onProgress?: Nullable<(event: ISceneLoaderProgressEvent) => void>, onError?: Nullable<(scene: Scene, message: string, exception?: any) => void>, pluginExtension?: Nullable<string>): Promise<Scene>;
}
export {};

}
declare module "babylonjs/Loading/sceneLoaderFlags" {
/**
 * Class used to represent data loading progression
 */
export class SceneLoaderFlags {
    private static _ForceFullSceneLoadingForIncremental;
    private static _ShowLoadingScreen;
    private static _CleanBoneMatrixWeights;
    private static _LoggingLevel;
    /**
     * Gets or sets a boolean indicating if entire scene must be loaded even if scene contains incremental data
     */
    static get ForceFullSceneLoadingForIncremental(): boolean;
    static set ForceFullSceneLoadingForIncremental(value: boolean);
    /**
     * Gets or sets a boolean indicating if loading screen must be displayed while loading a scene
     */
    static get ShowLoadingScreen(): boolean;
    static set ShowLoadingScreen(value: boolean);
    /**
     * Defines the current logging level (while loading the scene)
     * @ignorenaming
     */
    static get loggingLevel(): number;
    static set loggingLevel(value: number);
    /**
     * Gets or set a boolean indicating if matrix weights must be cleaned upon loading
     */
    static get CleanBoneMatrixWeights(): boolean;
    static set CleanBoneMatrixWeights(value: boolean);
}

}
declare module "babylonjs/Materials/Background/backgroundMaterial" {
import { Nullable, int, float } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
import { Matrix } from "babylonjs/Maths/math.vector";
import { Vector3 } from "babylonjs/Maths/math.vector";
import { SubMesh } from "babylonjs/Meshes/subMesh";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { Mesh } from "babylonjs/Meshes/mesh";
import { PushMaterial } from "babylonjs/Materials/pushMaterial";
import { ColorCurves } from "babylonjs/Materials/colorCurves";
import { ImageProcessingConfiguration } from "babylonjs/Materials/imageProcessingConfiguration";
import { BaseTexture } from "babylonjs/Materials/Textures/baseTexture";
import { IShadowLight } from "babylonjs/Lights/shadowLight";
import { Color3 } from "babylonjs/Maths/math.color";
import "babylonjs/Shaders/background.fragment";
import "babylonjs/Shaders/background.vertex";
/**
 * Background material used to create an efficient environment around your scene.
 */
export class BackgroundMaterial extends PushMaterial {
    /**
     * Standard reflectance value at parallel view angle.
     */
    static StandardReflectance0: number;
    /**
     * Standard reflectance value at grazing angle.
     */
    static StandardReflectance90: number;
    protected _primaryColor: Color3;
    /**
     * Key light Color (multiply against the environment texture)
     */
    primaryColor: Color3;
    protected __perceptualColor: Nullable<Color3>;
    /**
     * Experimental Internal Use Only.
     *
     * Key light Color in "perceptual value" meaning the color you would like to see on screen.
     * This acts as a helper to set the primary color to a more "human friendly" value.
     * Conversion to linear space as well as exposure and tone mapping correction will be applied to keep the
     * output color as close as possible from the chosen value.
     * (This does not account for contrast color grading and color curves as they are considered post effect and not directly
     * part of lighting setup.)
     */
    get _perceptualColor(): Nullable<Color3>;
    set _perceptualColor(value: Nullable<Color3>);
    protected _primaryColorShadowLevel: float;
    /**
     * Defines the level of the shadows (dark area of the reflection map) in order to help scaling the colors.
     * The color opposite to the primary color is used at the level chosen to define what the black area would look.
     */
    get primaryColorShadowLevel(): float;
    set primaryColorShadowLevel(value: float);
    protected _primaryColorHighlightLevel: float;
    /**
     * Defines the level of the highlights (highlight area of the reflection map) in order to help scaling the colors.
     * The primary color is used at the level chosen to define what the white area would look.
     */
    get primaryColorHighlightLevel(): float;
    set primaryColorHighlightLevel(value: float);
    protected _reflectionTexture: Nullable<BaseTexture>;
    /**
     * Reflection Texture used in the material.
     * Should be author in a specific way for the best result (refer to the documentation).
     */
    reflectionTexture: Nullable<BaseTexture>;
    protected _reflectionBlur: float;
    /**
     * Reflection Texture level of blur.
     *
     * Can be use to reuse an existing HDR Texture and target a specific LOD to prevent authoring the
     * texture twice.
     */
    reflectionBlur: float;
    protected _diffuseTexture: Nullable<BaseTexture>;
    /**
     * Diffuse Texture used in the material.
     * Should be author in a specific way for the best result (refer to the documentation).
     */
    diffuseTexture: Nullable<BaseTexture>;
    protected _shadowLights: Nullable<IShadowLight[]>;
    /**
     * Specify the list of lights casting shadow on the material.
     * All scene shadow lights will be included if null.
     */
    shadowLights: Nullable<IShadowLight[]>;
    protected _shadowLevel: float;
    /**
     * Helps adjusting the shadow to a softer level if required.
     * 0 means black shadows and 1 means no shadows.
     */
    shadowLevel: float;
    protected _sceneCenter: Vector3;
    /**
     * In case of opacity Fresnel or reflection falloff, this is use as a scene center.
     * It is usually zero but might be interesting to modify according to your setup.
     */
    sceneCenter: Vector3;
    protected _opacityFresnel: boolean;
    /**
     * This helps specifying that the material is falling off to the sky box at grazing angle.
     * This helps ensuring a nice transition when the camera goes under the ground.
     */
    opacityFresnel: boolean;
    protected _reflectionFresnel: boolean;
    /**
     * This helps specifying that the material is falling off from diffuse to the reflection texture at grazing angle.
     * This helps adding a mirror texture on the ground.
     */
    reflectionFresnel: boolean;
    protected _reflectionFalloffDistance: number;
    /**
     * This helps specifying the falloff radius off the reflection texture from the sceneCenter.
     * This helps adding a nice falloff effect to the reflection if used as a mirror for instance.
     */
    reflectionFalloffDistance: number;
    protected _reflectionAmount: number;
    /**
     * This specifies the weight of the reflection against the background in case of reflection Fresnel.
     */
    reflectionAmount: number;
    protected _reflectionReflectance0: number;
    /**
     * This specifies the weight of the reflection at grazing angle.
     */
    reflectionReflectance0: number;
    protected _reflectionReflectance90: number;
    /**
     * This specifies the weight of the reflection at a perpendicular point of view.
     */
    reflectionReflectance90: number;
    /**
     * Sets the reflection reflectance fresnel values according to the default standard
     * empirically know to work well :-)
     */
    set reflectionStandardFresnelWeight(value: number);
    protected _useRGBColor: boolean;
    /**
     * Helps to directly use the maps channels instead of their level.
     */
    useRGBColor: boolean;
    protected _enableNoise: boolean;
    /**
     * This helps reducing the banding effect that could occur on the background.
     */
    enableNoise: boolean;
    /**
     * The current fov(field of view) multiplier, 0.0 - 2.0. Defaults to 1.0. Lower values "zoom in" and higher values "zoom out".
     * Best used when trying to implement visual zoom effects like fish-eye or binoculars while not adjusting camera fov.
     * Recommended to be keep at 1.0 except for special cases.
     */
    get fovMultiplier(): number;
    set fovMultiplier(value: number);
    private _fovMultiplier;
    /**
     * Enable the FOV adjustment feature controlled by fovMultiplier.
     */
    useEquirectangularFOV: boolean;
    private _maxSimultaneousLights;
    /**
     * Number of Simultaneous lights allowed on the material.
     */
    maxSimultaneousLights: int;
    private _shadowOnly;
    /**
     * Make the material only render shadows
     */
    shadowOnly: boolean;
    /**
     * Default configuration related to image processing available in the Background Material.
     */
    protected _imageProcessingConfiguration: ImageProcessingConfiguration;
    /**
     * Keep track of the image processing observer to allow dispose and replace.
     */
    private _imageProcessingObserver;
    /**
     * Attaches a new image processing configuration to the PBR Material.
     * @param configuration (if null the scene configuration will be use)
     */
    protected _attachImageProcessingConfiguration(configuration: Nullable<ImageProcessingConfiguration>): void;
    /**
     * Gets the image processing configuration used either in this material.
     */
    get imageProcessingConfiguration(): Nullable<ImageProcessingConfiguration>;
    /**
     * Sets the Default image processing configuration used either in the this material.
     *
     * If sets to null, the scene one is in use.
     */
    set imageProcessingConfiguration(value: Nullable<ImageProcessingConfiguration>);
    /**
     * Gets whether the color curves effect is enabled.
     */
    get cameraColorCurvesEnabled(): boolean;
    /**
     * Sets whether the color curves effect is enabled.
     */
    set cameraColorCurvesEnabled(value: boolean);
    /**
     * Gets whether the color grading effect is enabled.
     */
    get cameraColorGradingEnabled(): boolean;
    /**
     * Gets whether the color grading effect is enabled.
     */
    set cameraColorGradingEnabled(value: boolean);
    /**
     * Gets whether tonemapping is enabled or not.
     */
    get cameraToneMappingEnabled(): boolean;
    /**
     * Sets whether tonemapping is enabled or not
     */
    set cameraToneMappingEnabled(value: boolean);
    /**
     * The camera exposure used on this material.
     * This property is here and not in the camera to allow controlling exposure without full screen post process.
     * This corresponds to a photographic exposure.
     */
    get cameraExposure(): float;
    /**
     * The camera exposure used on this material.
     * This property is here and not in the camera to allow controlling exposure without full screen post process.
     * This corresponds to a photographic exposure.
     */
    set cameraExposure(value: float);
    /**
     * Gets The camera contrast used on this material.
     */
    get cameraContrast(): float;
    /**
     * Sets The camera contrast used on this material.
     */
    set cameraContrast(value: float);
    /**
     * Gets the Color Grading 2D Lookup Texture.
     */
    get cameraColorGradingTexture(): Nullable<BaseTexture>;
    /**
     * Sets the Color Grading 2D Lookup Texture.
     */
    set cameraColorGradingTexture(value: Nullable<BaseTexture>);
    /**
     * The color grading curves provide additional color adjustment that is applied after any color grading transform (3D LUT).
     * They allow basic adjustment of saturation and small exposure adjustments, along with color filter tinting to provide white balance adjustment or more stylistic effects.
     * These are similar to controls found in many professional imaging or colorist software. The global controls are applied to the entire image. For advanced tuning, extra controls are provided to adjust the shadow, midtone and highlight areas of the image;
     * corresponding to low luminance, medium luminance, and high luminance areas respectively.
     */
    get cameraColorCurves(): Nullable<ColorCurves>;
    /**
     * The color grading curves provide additional color adjustment that is applied after any color grading transform (3D LUT).
     * They allow basic adjustment of saturation and small exposure adjustments, along with color filter tinting to provide white balance adjustment or more stylistic effects.
     * These are similar to controls found in many professional imaging or colorist software. The global controls are applied to the entire image. For advanced tuning, extra controls are provided to adjust the shadow, midtone and highlight areas of the image;
     * corresponding to low luminance, medium luminance, and high luminance areas respectively.
     */
    set cameraColorCurves(value: Nullable<ColorCurves>);
    /**
     * Due to a bug in iOS10, video tags (which are using the background material) are in BGR and not RGB.
     * Setting this flag to true (not done automatically!) will convert it back to RGB.
     */
    switchToBGR: boolean;
    private _renderTargets;
    private _reflectionControls;
    private _white;
    private _primaryShadowColor;
    private _primaryHighlightColor;
    /**
     * Instantiates a Background Material in the given scene
     * @param name The friendly name of the material
     * @param scene The scene to add the material to
     */
    constructor(name: string, scene?: Scene);
    /**
     * Gets a boolean indicating that current material needs to register RTT
     */
    get hasRenderTargetTextures(): boolean;
    /**
     * The entire material has been created in order to prevent overdraw.
     * @returns false
     */
    needAlphaTesting(): boolean;
    /**
     * The entire material has been created in order to prevent overdraw.
     * @returns true if blending is enable
     */
    needAlphaBlending(): boolean;
    /**
     * Checks whether the material is ready to be rendered for a given mesh.
     * @param mesh The mesh to render
     * @param subMesh The submesh to check against
     * @param useInstances Specify wether or not the material is used with instances
     * @returns true if all the dependencies are ready (Textures, Effects...)
     */
    isReadyForSubMesh(mesh: AbstractMesh, subMesh: SubMesh, useInstances?: boolean): boolean;
    /**
     * Compute the primary color according to the chosen perceptual color.
     */
    private _computePrimaryColorFromPerceptualColor;
    /**
     * Compute the highlights and shadow colors according to their chosen levels.
     */
    private _computePrimaryColors;
    /**
     * Build the uniform buffer used in the material.
     */
    buildUniformLayout(): void;
    /**
     * Unbind the material.
     */
    unbind(): void;
    /**
     * Bind only the world matrix to the material.
     * @param world The world matrix to bind.
     */
    bindOnlyWorldMatrix(world: Matrix): void;
    /**
     * Bind the material for a dedicated submeh (every used meshes will be considered opaque).
     * @param world The world matrix to bind.
     * @param mesh
     * @param subMesh The submesh to bind for.
     */
    bindForSubMesh(world: Matrix, mesh: Mesh, subMesh: SubMesh): void;
    /**
     * Checks to see if a texture is used in the material.
     * @param texture - Base texture to use.
     * @returns - Boolean specifying if a texture is used in the material.
     */
    hasTexture(texture: BaseTexture): boolean;
    /**
     * Dispose the material.
     * @param forceDisposeEffect Force disposal of the associated effect.
     * @param forceDisposeTextures Force disposal of the associated textures.
     */
    dispose(forceDisposeEffect?: boolean, forceDisposeTextures?: boolean): void;
    /**
     * Clones the material.
     * @param name The cloned name.
     * @returns The cloned material.
     */
    clone(name: string): BackgroundMaterial;
    /**
     * Serializes the current material to its JSON representation.
     * @returns The JSON representation.
     */
    serialize(): any;
    /**
     * Gets the class name of the material
     * @returns "BackgroundMaterial"
     */
    getClassName(): string;
    /**
     * Parse a JSON input to create back a background material.
     * @param source The JSON data to parse
     * @param scene The scene to create the parsed material in
     * @param rootUrl The root url of the assets the material depends upon
     * @returns the instantiated BackgroundMaterial.
     */
    static Parse(source: any, scene: Scene, rootUrl: string): BackgroundMaterial;
}

}
declare module "babylonjs/Materials/Background/index" {
export * from "babylonjs/Materials/Background/backgroundMaterial";

}
declare module "babylonjs/Materials/clipPlaneMaterialHelper" {
import { Effect } from "babylonjs/Materials/effect";
import { IClipPlanesHolder } from "babylonjs/Misc/interfaces/iClipPlanesHolder";
/** @internal */
export function addClipPlaneUniforms(uniforms: string[]): void;
/** @internal */
export function prepareDefinesForClipPlanes(primaryHolder: IClipPlanesHolder, secondaryHolder: IClipPlanesHolder, defines: string[] | Record<string, any>): boolean;
/** @internal */
export function bindClipPlane(effect: Effect, primaryHolder: IClipPlanesHolder, secondaryHolder: IClipPlanesHolder): void;

}
declare module "babylonjs/Materials/colorCurves" {
import { Effect } from "babylonjs/Materials/effect";
/**
 * The color grading curves provide additional color adjustment that is applied after any color grading transform (3D LUT).
 * They allow basic adjustment of saturation and small exposure adjustments, along with color filter tinting to provide white balance adjustment or more stylistic effects.
 * These are similar to controls found in many professional imaging or colorist software. The global controls are applied to the entire image. For advanced tuning, extra controls are provided to adjust the shadow, midtone and highlight areas of the image;
 * corresponding to low luminance, medium luminance, and high luminance areas respectively.
 */
export class ColorCurves {
    private _dirty;
    private _tempColor;
    private _globalCurve;
    private _highlightsCurve;
    private _midtonesCurve;
    private _shadowsCurve;
    private _positiveCurve;
    private _negativeCurve;
    private _globalHue;
    private _globalDensity;
    private _globalSaturation;
    private _globalExposure;
    /**
     * Gets the global Hue value.
     * The hue value is a standard HSB hue in the range [0,360] where 0=red, 120=green and 240=blue. The default value is 30 degrees (orange).
     */
    get globalHue(): number;
    /**
     * Sets the global Hue value.
     * The hue value is a standard HSB hue in the range [0,360] where 0=red, 120=green and 240=blue. The default value is 30 degrees (orange).
     */
    set globalHue(value: number);
    /**
     * Gets the global Density value.
     * The density value is in range [-100,+100] where 0 means the color filter has no effect and +100 means the color filter has maximum effect.
     * Values less than zero provide a filter of opposite hue.
     */
    get globalDensity(): number;
    /**
     * Sets the global Density value.
     * The density value is in range [-100,+100] where 0 means the color filter has no effect and +100 means the color filter has maximum effect.
     * Values less than zero provide a filter of opposite hue.
     */
    set globalDensity(value: number);
    /**
     * Gets the global Saturation value.
     * This is an adjustment value in the range [-100,+100], where the default value of 0.0 makes no adjustment, positive values increase saturation and negative values decrease saturation.
     */
    get globalSaturation(): number;
    /**
     * Sets the global Saturation value.
     * This is an adjustment value in the range [-100,+100], where the default value of 0.0 makes no adjustment, positive values increase saturation and negative values decrease saturation.
     */
    set globalSaturation(value: number);
    /**
     * Gets the global Exposure value.
     * This is an adjustment value in the range [-100,+100], where the default value of 0.0 makes no adjustment, positive values increase exposure and negative values decrease exposure.
     */
    get globalExposure(): number;
    /**
     * Sets the global Exposure value.
     * This is an adjustment value in the range [-100,+100], where the default value of 0.0 makes no adjustment, positive values increase exposure and negative values decrease exposure.
     */
    set globalExposure(value: number);
    private _highlightsHue;
    private _highlightsDensity;
    private _highlightsSaturation;
    private _highlightsExposure;
    /**
     * Gets the highlights Hue value.
     * The hue value is a standard HSB hue in the range [0,360] where 0=red, 120=green and 240=blue. The default value is 30 degrees (orange).
     */
    get highlightsHue(): number;
    /**
     * Sets the highlights Hue value.
     * The hue value is a standard HSB hue in the range [0,360] where 0=red, 120=green and 240=blue. The default value is 30 degrees (orange).
     */
    set highlightsHue(value: number);
    /**
     * Gets the highlights Density value.
     * The density value is in range [-100,+100] where 0 means the color filter has no effect and +100 means the color filter has maximum effect.
     * Values less than zero provide a filter of opposite hue.
     */
    get highlightsDensity(): number;
    /**
     * Sets the highlights Density value.
     * The density value is in range [-100,+100] where 0 means the color filter has no effect and +100 means the color filter has maximum effect.
     * Values less than zero provide a filter of opposite hue.
     */
    set highlightsDensity(value: number);
    /**
     * Gets the highlights Saturation value.
     * This is an adjustment value in the range [-100,+100], where the default value of 0.0 makes no adjustment, positive values increase saturation and negative values decrease saturation.
     */
    get highlightsSaturation(): number;
    /**
     * Sets the highlights Saturation value.
     * This is an adjustment value in the range [-100,+100], where the default value of 0.0 makes no adjustment, positive values increase saturation and negative values decrease saturation.
     */
    set highlightsSaturation(value: number);
    /**
     * Gets the highlights Exposure value.
     * This is an adjustment value in the range [-100,+100], where the default value of 0.0 makes no adjustment, positive values increase exposure and negative values decrease exposure.
     */
    get highlightsExposure(): number;
    /**
     * Sets the highlights Exposure value.
     * This is an adjustment value in the range [-100,+100], where the default value of 0.0 makes no adjustment, positive values increase exposure and negative values decrease exposure.
     */
    set highlightsExposure(value: number);
    private _midtonesHue;
    private _midtonesDensity;
    private _midtonesSaturation;
    private _midtonesExposure;
    /**
     * Gets the midtones Hue value.
     * The hue value is a standard HSB hue in the range [0,360] where 0=red, 120=green and 240=blue. The default value is 30 degrees (orange).
     */
    get midtonesHue(): number;
    /**
     * Sets the midtones Hue value.
     * The hue value is a standard HSB hue in the range [0,360] where 0=red, 120=green and 240=blue. The default value is 30 degrees (orange).
     */
    set midtonesHue(value: number);
    /**
     * Gets the midtones Density value.
     * The density value is in range [-100,+100] where 0 means the color filter has no effect and +100 means the color filter has maximum effect.
     * Values less than zero provide a filter of opposite hue.
     */
    get midtonesDensity(): number;
    /**
     * Sets the midtones Density value.
     * The density value is in range [-100,+100] where 0 means the color filter has no effect and +100 means the color filter has maximum effect.
     * Values less than zero provide a filter of opposite hue.
     */
    set midtonesDensity(value: number);
    /**
     * Gets the midtones Saturation value.
     * This is an adjustment value in the range [-100,+100], where the default value of 0.0 makes no adjustment, positive values increase saturation and negative values decrease saturation.
     */
    get midtonesSaturation(): number;
    /**
     * Sets the midtones Saturation value.
     * This is an adjustment value in the range [-100,+100], where the default value of 0.0 makes no adjustment, positive values increase saturation and negative values decrease saturation.
     */
    set midtonesSaturation(value: number);
    /**
     * Gets the midtones Exposure value.
     * This is an adjustment value in the range [-100,+100], where the default value of 0.0 makes no adjustment, positive values increase exposure and negative values decrease exposure.
     */
    get midtonesExposure(): number;
    /**
     * Sets the midtones Exposure value.
     * This is an adjustment value in the range [-100,+100], where the default value of 0.0 makes no adjustment, positive values increase exposure and negative values decrease exposure.
     */
    set midtonesExposure(value: number);
    private _shadowsHue;
    private _shadowsDensity;
    private _shadowsSaturation;
    private _shadowsExposure;
    /**
     * Gets the shadows Hue value.
     * The hue value is a standard HSB hue in the range [0,360] where 0=red, 120=green and 240=blue. The default value is 30 degrees (orange).
     */
    get shadowsHue(): number;
    /**
     * Sets the shadows Hue value.
     * The hue value is a standard HSB hue in the range [0,360] where 0=red, 120=green and 240=blue. The default value is 30 degrees (orange).
     */
    set shadowsHue(value: number);
    /**
     * Gets the shadows Density value.
     * The density value is in range [-100,+100] where 0 means the color filter has no effect and +100 means the color filter has maximum effect.
     * Values less than zero provide a filter of opposite hue.
     */
    get shadowsDensity(): number;
    /**
     * Sets the shadows Density value.
     * The density value is in range [-100,+100] where 0 means the color filter has no effect and +100 means the color filter has maximum effect.
     * Values less than zero provide a filter of opposite hue.
     */
    set shadowsDensity(value: number);
    /**
     * Gets the shadows Saturation value.
     * This is an adjustment value in the range [-100,+100], where the default value of 0.0 makes no adjustment, positive values increase saturation and negative values decrease saturation.
     */
    get shadowsSaturation(): number;
    /**
     * Sets the shadows Saturation value.
     * This is an adjustment value in the range [-100,+100], where the default value of 0.0 makes no adjustment, positive values increase saturation and negative values decrease saturation.
     */
    set shadowsSaturation(value: number);
    /**
     * Gets the shadows Exposure value.
     * This is an adjustment value in the range [-100,+100], where the default value of 0.0 makes no adjustment, positive values increase exposure and negative values decrease exposure.
     */
    get shadowsExposure(): number;
    /**
     * Sets the shadows Exposure value.
     * This is an adjustment value in the range [-100,+100], where the default value of 0.0 makes no adjustment, positive values increase exposure and negative values decrease exposure.
     */
    set shadowsExposure(value: number);
    /**
     * Returns the class name
     * @returns The class name
     */
    getClassName(): string;
    /**
     * Binds the color curves to the shader.
     * @param colorCurves The color curve to bind
     * @param effect The effect to bind to
     * @param positiveUniform The positive uniform shader parameter
     * @param neutralUniform The neutral uniform shader parameter
     * @param negativeUniform The negative uniform shader parameter
     */
    static Bind(colorCurves: ColorCurves, effect: Effect, positiveUniform?: string, neutralUniform?: string, negativeUniform?: string): void;
    /**
     * Prepare the list of uniforms associated with the ColorCurves effects.
     * @param uniformsList The list of uniforms used in the effect
     */
    static PrepareUniforms(uniformsList: string[]): void;
    /**
     * Returns color grading data based on a hue, density, saturation and exposure value.
     * @param hue
     * @param density
     * @param saturation The saturation.
     * @param exposure The exposure.
     * @param result The result data container.
     */
    private _getColorGradingDataToRef;
    /**
     * Takes an input slider value and returns an adjusted value that provides extra control near the centre.
     * @param value The input slider value in range [-100,100].
     * @returns Adjusted value.
     */
    private static _ApplyColorGradingSliderNonlinear;
    /**
     * Returns an RGBA Color4 based on Hue, Saturation and Brightness (also referred to as value, HSV).
     * @param hue The hue (H) input.
     * @param saturation The saturation (S) input.
     * @param brightness The brightness (B) input.
     * @param result
     * @result An RGBA color represented as Vector4.
     */
    private static _FromHSBToRef;
    /**
     * Returns a value clamped between min and max
     * @param value The value to clamp
     * @param min The minimum of value
     * @param max The maximum of value
     * @returns The clamped value.
     */
    private static _Clamp;
    /**
     * Clones the current color curve instance.
     * @returns The cloned curves
     */
    clone(): ColorCurves;
    /**
     * Serializes the current color curve instance to a json representation.
     * @returns a JSON representation
     */
    serialize(): any;
    /**
     * Parses the color curve from a json representation.
     * @param source the JSON source to parse
     * @returns The parsed curves
     */
    static Parse(source: any): ColorCurves;
}

}
declare module "babylonjs/Materials/drawWrapper" {
import { IDrawContext } from "babylonjs/Engines/IDrawContext";
import { IMaterialContext } from "babylonjs/Engines/IMaterialContext";
import { Nullable } from "babylonjs/types";
import { ThinEngine } from "babylonjs/Engines/thinEngine";
import { Effect } from "babylonjs/Materials/effect";
import { MaterialDefines } from "babylonjs/Materials/materialDefines";
/** @internal */
export class DrawWrapper {
    effect: Nullable<Effect>;
    defines: Nullable<string | MaterialDefines>;
    materialContext?: IMaterialContext;
    drawContext?: IDrawContext;
    static IsWrapper(effect: Effect | DrawWrapper): effect is DrawWrapper;
    static GetEffect(effect: Effect | DrawWrapper): Nullable<Effect>;
    constructor(engine: ThinEngine, createMaterialContext?: boolean);
    setEffect(effect: Nullable<Effect>, defines?: Nullable<string | MaterialDefines>, resetContext?: boolean): void;
    dispose(): void;
}
export {};

}
declare module "babylonjs/Materials/effect" {
import { Observable } from "babylonjs/Misc/observable";
import { FloatArray, Nullable } from "babylonjs/types";
import { IDisposable } from "babylonjs/scene";
import { IPipelineContext } from "babylonjs/Engines/IPipelineContext";
import { DataBuffer } from "babylonjs/Buffers/dataBuffer";
import { ShaderCustomProcessingFunction } from "babylonjs/Engines/Processors/shaderProcessingOptions";
import { IMatrixLike, IVector2Like, IVector3Like, IVector4Like, IColor3Like, IColor4Like, IQuaternionLike } from "babylonjs/Maths/math.like";
import { ThinEngine } from "babylonjs/Engines/thinEngine";
import { IEffectFallbacks } from "babylonjs/Materials/iEffectFallbacks";
import { ShaderLanguage } from "babylonjs/Materials/shaderLanguage";
import { Engine } from "babylonjs/Engines/engine";
import { InternalTexture } from "babylonjs/Materials/Textures/internalTexture";
import { ThinTexture } from "babylonjs/Materials/Textures/thinTexture";
import { RenderTargetTexture } from "babylonjs/Materials/Textures/renderTargetTexture";
import { PostProcess } from "babylonjs/PostProcesses/postProcess";
/**
 * Options to be used when creating an effect.
 */
export interface IEffectCreationOptions {
    /**
     * Attributes that will be used in the shader.
     */
    attributes: string[];
    /**
     * Uniform variable names that will be set in the shader.
     */
    uniformsNames: string[];
    /**
     * Uniform buffer variable names that will be set in the shader.
     */
    uniformBuffersNames: string[];
    /**
     * Sampler texture variable names that will be set in the shader.
     */
    samplers: string[];
    /**
     * Define statements that will be set in the shader.
     */
    defines: any;
    /**
     * Possible fallbacks for this effect to improve performance when needed.
     */
    fallbacks: Nullable<IEffectFallbacks>;
    /**
     * Callback that will be called when the shader is compiled.
     */
    onCompiled: Nullable<(effect: Effect) => void>;
    /**
     * Callback that will be called if an error occurs during shader compilation.
     */
    onError: Nullable<(effect: Effect, errors: string) => void>;
    /**
     * Parameters to be used with Babylons include syntax to iterate over an array (eg. {lights: 10})
     */
    indexParameters?: any;
    /**
     * Max number of lights that can be used in the shader.
     */
    maxSimultaneousLights?: number;
    /**
     * See https://developer.mozilla.org/en-US/docs/Web/API/WebGL2RenderingContext/transformFeedbackVaryings
     */
    transformFeedbackVaryings?: Nullable<string[]>;
    /**
     * If provided, will be called two times with the vertex and fragment code so that this code can be updated before it is compiled by the GPU
     */
    processFinalCode?: Nullable<ShaderCustomProcessingFunction>;
    /**
     * If provided, will be called two times with the vertex and fragment code so that this code can be updated after the #include have been processed
     */
    processCodeAfterIncludes?: Nullable<ShaderCustomProcessingFunction>;
    /**
     * Is this effect rendering to several color attachments ?
     */
    multiTarget?: boolean;
    /**
     * The language the shader is written in (default: GLSL)
     */
    shaderLanguage?: ShaderLanguage;
}
/**
 * Effect containing vertex and fragment shader that can be executed on an object.
 */
export class Effect implements IDisposable {
    /**
     * Gets or sets the relative url used to load shaders if using the engine in non-minified mode
     */
    static get ShadersRepository(): string;
    static set ShadersRepository(repo: string);
    /**
     * Enable logging of the shader code when a compilation error occurs
     */
    static LogShaderCodeOnCompilationError: boolean;
    /**
     * Name of the effect.
     */
    name: any;
    /**
     * String container all the define statements that should be set on the shader.
     */
    defines: string;
    /**
     * Callback that will be called when the shader is compiled.
     */
    onCompiled: Nullable<(effect: Effect) => void>;
    /**
     * Callback that will be called if an error occurs during shader compilation.
     */
    onError: Nullable<(effect: Effect, errors: string) => void>;
    /**
     * Callback that will be called when effect is bound.
     */
    onBind: Nullable<(effect: Effect) => void>;
    /**
     * Unique ID of the effect.
     */
    uniqueId: number;
    /**
     * Observable that will be called when the shader is compiled.
     * It is recommended to use executeWhenCompile() or to make sure that scene.isReady() is called to get this observable raised.
     */
    onCompileObservable: Observable<Effect>;
    /**
     * Observable that will be called if an error occurs during shader compilation.
     */
    onErrorObservable: Observable<Effect>;
    /** @internal */
    _onBindObservable: Nullable<Observable<Effect>>;
    /**
     * @internal
     * Specifies if the effect was previously ready
     */
    _wasPreviouslyReady: boolean;
    /**
     * @internal
     * Forces the code from bindForSubMesh to be fully run the next time it is called
     * It is used in frozen mode to make sure the effect is properly rebound when a new effect is created
     */
    _forceRebindOnNextCall: boolean;
    /**
     * @internal
     * Specifies if the effect was previously using instances
     */
    _wasPreviouslyUsingInstances: Nullable<boolean>;
    private _isDisposed;
    /**
     * Observable that will be called when effect is bound.
     */
    get onBindObservable(): Observable<Effect>;
    /** @internal */
    _bonesComputationForcedToCPU: boolean;
    /** @internal */
    _uniformBuffersNames: {
        [key: string]: number;
    };
    /** @internal */
    _samplerList: string[];
    /** @internal */
    _multiTarget: boolean;
    private static _UniqueIdSeed;
    /** @internal */
    _engine: Engine;
    private _uniformBuffersNamesList;
    private _uniformsNames;
    private _samplers;
    private _isReady;
    private _compilationError;
    private _allFallbacksProcessed;
    private _attributesNames;
    private _attributes;
    private _attributeLocationByName;
    private _uniforms;
    /**
     * Key for the effect.
     * @internal
     */
    _key: string;
    private _indexParameters;
    private _fallbacks;
    private _vertexSourceCodeOverride;
    private _fragmentSourceCodeOverride;
    private _transformFeedbackVaryings;
    private _shaderLanguage;
    /**
     * Compiled shader to webGL program.
     * @internal
     */
    _pipelineContext: Nullable<IPipelineContext>;
    /** @internal */
    _vertexSourceCode: string;
    /** @internal */
    _fragmentSourceCode: string;
    /** @internal */
    private _vertexSourceCodeBeforeMigration;
    /** @internal */
    private _fragmentSourceCodeBeforeMigration;
    /** @internal */
    private _rawVertexSourceCode;
    /** @internal */
    private _rawFragmentSourceCode;
    private static _BaseCache;
    private _processingContext;
    /**
     * Instantiates an effect.
     * An effect can be used to create/manage/execute vertex and fragment shaders.
     * @param baseName Name of the effect.
     * @param attributesNamesOrOptions List of attribute names that will be passed to the shader or set of all options to create the effect.
     * @param uniformsNamesOrEngine List of uniform variable names that will be passed to the shader or the engine that will be used to render effect.
     * @param samplers List of sampler variables that will be passed to the shader.
     * @param engine Engine to be used to render the effect
     * @param defines Define statements to be added to the shader.
     * @param fallbacks Possible fallbacks for this effect to improve performance when needed.
     * @param onCompiled Callback that will be called when the shader is compiled.
     * @param onError Callback that will be called if an error occurs during shader compilation.
     * @param indexParameters Parameters to be used with Babylons include syntax to iterate over an array (eg. {lights: 10})
     * @param key Effect Key identifying uniquely compiled shader variants
     * @param shaderLanguage the language the shader is written in (default: GLSL)
     */
    constructor(baseName: any, attributesNamesOrOptions: string[] | IEffectCreationOptions, uniformsNamesOrEngine: string[] | ThinEngine, samplers?: Nullable<string[]>, engine?: ThinEngine, defines?: Nullable<string>, fallbacks?: Nullable<IEffectFallbacks>, onCompiled?: Nullable<(effect: Effect) => void>, onError?: Nullable<(effect: Effect, errors: string) => void>, indexParameters?: any, key?: string, shaderLanguage?: ShaderLanguage);
    private _useFinalCode;
    /**
     * Unique key for this effect
     */
    get key(): string;
    /**
     * If the effect has been compiled and prepared.
     * @returns if the effect is compiled and prepared.
     */
    isReady(): boolean;
    private _isReadyInternal;
    /**
     * The engine the effect was initialized with.
     * @returns the engine.
     */
    getEngine(): Engine;
    /**
     * The pipeline context for this effect
     * @returns the associated pipeline context
     */
    getPipelineContext(): Nullable<IPipelineContext>;
    /**
     * The set of names of attribute variables for the shader.
     * @returns An array of attribute names.
     */
    getAttributesNames(): string[];
    /**
     * Returns the attribute at the given index.
     * @param index The index of the attribute.
     * @returns The location of the attribute.
     */
    getAttributeLocation(index: number): number;
    /**
     * Returns the attribute based on the name of the variable.
     * @param name of the attribute to look up.
     * @returns the attribute location.
     */
    getAttributeLocationByName(name: string): number;
    /**
     * The number of attributes.
     * @returns the number of attributes.
     */
    getAttributesCount(): number;
    /**
     * Gets the index of a uniform variable.
     * @param uniformName of the uniform to look up.
     * @returns the index.
     */
    getUniformIndex(uniformName: string): number;
    /**
     * Returns the attribute based on the name of the variable.
     * @param uniformName of the uniform to look up.
     * @returns the location of the uniform.
     */
    getUniform(uniformName: string): Nullable<WebGLUniformLocation>;
    /**
     * Returns an array of sampler variable names
     * @returns The array of sampler variable names.
     */
    getSamplers(): string[];
    /**
     * Returns an array of uniform variable names
     * @returns The array of uniform variable names.
     */
    getUniformNames(): string[];
    /**
     * Returns an array of uniform buffer variable names
     * @returns The array of uniform buffer variable names.
     */
    getUniformBuffersNames(): string[];
    /**
     * Returns the index parameters used to create the effect
     * @returns The index parameters object
     */
    getIndexParameters(): any;
    /**
     * The error from the last compilation.
     * @returns the error string.
     */
    getCompilationError(): string;
    /**
     * Gets a boolean indicating that all fallbacks were used during compilation
     * @returns true if all fallbacks were used
     */
    allFallbacksProcessed(): boolean;
    /**
     * Adds a callback to the onCompiled observable and call the callback immediately if already ready.
     * @param func The callback to be used.
     */
    executeWhenCompiled(func: (effect: Effect) => void): void;
    private _checkIsReady;
    private _loadShader;
    /**
     * Gets the vertex shader source code of this effect
     * This is the final source code that will be compiled, after all the processing has been done (pre-processing applied, code injection/replacement, etc)
     */
    get vertexSourceCode(): string;
    /**
     * Gets the fragment shader source code of this effect
     * This is the final source code that will be compiled, after all the processing has been done (pre-processing applied, code injection/replacement, etc)
     */
    get fragmentSourceCode(): string;
    /**
     * Gets the vertex shader source code before migration.
     * This is the source code after the include directives have been replaced by their contents but before the code is migrated, i.e. before ShaderProcess._ProcessShaderConversion is executed.
     * This method is, among other things, responsible for parsing #if/#define directives as well as converting GLES2 syntax to GLES3 (in the case of WebGL).
     */
    get vertexSourceCodeBeforeMigration(): string;
    /**
     * Gets the fragment shader source code before migration.
     * This is the source code after the include directives have been replaced by their contents but before the code is migrated, i.e. before ShaderProcess._ProcessShaderConversion is executed.
     * This method is, among other things, responsible for parsing #if/#define directives as well as converting GLES2 syntax to GLES3 (in the case of WebGL).
     */
    get fragmentSourceCodeBeforeMigration(): string;
    /**
     * Gets the vertex shader source code before it has been modified by any processing
     */
    get rawVertexSourceCode(): string;
    /**
     * Gets the fragment shader source code before it has been modified by any processing
     */
    get rawFragmentSourceCode(): string;
    /**
     * Recompiles the webGL program
     * @param vertexSourceCode The source code for the vertex shader.
     * @param fragmentSourceCode The source code for the fragment shader.
     * @param onCompiled Callback called when completed.
     * @param onError Callback called on error.
     * @internal
     */
    _rebuildProgram(vertexSourceCode: string, fragmentSourceCode: string, onCompiled: (pipelineContext: IPipelineContext) => void, onError: (message: string) => void): void;
    /**
     * Prepares the effect
     * @internal
     */
    _prepareEffect(): void;
    private _getShaderCodeAndErrorLine;
    private _processCompilationErrors;
    /**
     * Checks if the effect is supported. (Must be called after compilation)
     */
    get isSupported(): boolean;
    /**
     * Binds a texture to the engine to be used as output of the shader.
     * @param channel Name of the output variable.
     * @param texture Texture to bind.
     * @internal
     */
    _bindTexture(channel: string, texture: Nullable<InternalTexture>): void;
    /**
     * Sets a texture on the engine to be used in the shader.
     * @param channel Name of the sampler variable.
     * @param texture Texture to set.
     */
    setTexture(channel: string, texture: Nullable<ThinTexture>): void;
    /**
     * Sets a depth stencil texture from a render target on the engine to be used in the shader.
     * @param channel Name of the sampler variable.
     * @param texture Texture to set.
     */
    setDepthStencilTexture(channel: string, texture: Nullable<RenderTargetTexture>): void;
    /**
     * Sets an array of textures on the engine to be used in the shader.
     * @param channel Name of the variable.
     * @param textures Textures to set.
     */
    setTextureArray(channel: string, textures: ThinTexture[]): void;
    /**
     * Sets a texture to be the input of the specified post process. (To use the output, pass in the next post process in the pipeline)
     * @param channel Name of the sampler variable.
     * @param postProcess Post process to get the input texture from.
     */
    setTextureFromPostProcess(channel: string, postProcess: Nullable<PostProcess>): void;
    /**
     * (Warning! setTextureFromPostProcessOutput may be desired instead)
     * Sets the input texture of the passed in post process to be input of this effect. (To use the output of the passed in post process use setTextureFromPostProcessOutput)
     * @param channel Name of the sampler variable.
     * @param postProcess Post process to get the output texture from.
     */
    setTextureFromPostProcessOutput(channel: string, postProcess: Nullable<PostProcess>): void;
    /**
     * Binds a buffer to a uniform.
     * @param buffer Buffer to bind.
     * @param name Name of the uniform variable to bind to.
     */
    bindUniformBuffer(buffer: DataBuffer, name: string): void;
    /**
     * Binds block to a uniform.
     * @param blockName Name of the block to bind.
     * @param index Index to bind.
     */
    bindUniformBlock(blockName: string, index: number): void;
    /**
     * Sets an integer value on a uniform variable.
     * @param uniformName Name of the variable.
     * @param value Value to be set.
     * @returns this effect.
     */
    setInt: (uniformName: string, value: number) => Effect;
    /**
     * Sets an int2 value on a uniform variable.
     * @param uniformName Name of the variable.
     * @param x First int in int2.
     * @param y Second int in int2.
     * @returns this effect.
     */
    setInt2: (uniformName: string, x: number, y: number) => Effect;
    /**
     * Sets an int3 value on a uniform variable.
     * @param uniformName Name of the variable.
     * @param x First int in int3.
     * @param y Second int in int3.
     * @param z Third int in int3.
     * @returns this effect.
     */
    setInt3: (uniformName: string, x: number, y: number, z: number) => Effect;
    /**
     * Sets an int4 value on a uniform variable.
     * @param uniformName Name of the variable.
     * @param x First int in int4.
     * @param y Second int in int4.
     * @param z Third int in int4.
     * @param w Fourth int in int4.
     * @returns this effect.
     */
    setInt4: (uniformName: string, x: number, y: number, z: number, w: number) => Effect;
    /**
     * Sets an int array on a uniform variable.
     * @param uniformName Name of the variable.
     * @param array array to be set.
     * @returns this effect.
     */
    setIntArray: (uniformName: string, array: Int32Array) => Effect;
    /**
     * Sets an int array 2 on a uniform variable. (Array is specified as single array eg. [1,2,3,4] will result in [[1,2],[3,4]] in the shader)
     * @param uniformName Name of the variable.
     * @param array array to be set.
     * @returns this effect.
     */
    setIntArray2: (uniformName: string, array: Int32Array) => Effect;
    /**
     * Sets an int array 3 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6] will result in [[1,2,3],[4,5,6]] in the shader)
     * @param uniformName Name of the variable.
     * @param array array to be set.
     * @returns this effect.
     */
    setIntArray3: (uniformName: string, array: Int32Array) => Effect;
    /**
     * Sets an int array 4 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6,7,8] will result in [[1,2,3,4],[5,6,7,8]] in the shader)
     * @param uniformName Name of the variable.
     * @param array array to be set.
     * @returns this effect.
     */
    setIntArray4: (uniformName: string, array: Int32Array) => Effect;
    /**
     * Sets an unsigned integer value on a uniform variable.
     * @param uniformName Name of the variable.
     * @param value Value to be set.
     * @returns this effect.
     */
    setUInt: (uniformName: string, value: number) => Effect;
    /**
     * Sets an unsigned int2 value on a uniform variable.
     * @param uniformName Name of the variable.
     * @param x First unsigned int in uint2.
     * @param y Second unsigned int in uint2.
     * @returns this effect.
     */
    setUInt2: (uniformName: string, x: number, y: number) => Effect;
    /**
     * Sets an unsigned int3 value on a uniform variable.
     * @param uniformName Name of the variable.
     * @param x First unsigned int in uint3.
     * @param y Second unsigned int in uint3.
     * @param z Third unsigned int in uint3.
     * @returns this effect.
     */
    setUInt3: (uniformName: string, x: number, y: number, z: number) => Effect;
    /**
     * Sets an unsigned int4 value on a uniform variable.
     * @param uniformName Name of the variable.
     * @param x First unsigned int in uint4.
     * @param y Second unsigned int in uint4.
     * @param z Third unsigned int in uint4.
     * @param w Fourth unsigned int in uint4.
     * @returns this effect.
     */
    setUInt4: (uniformName: string, x: number, y: number, z: number, w: number) => Effect;
    /**
     * Sets an unsigned int array on a uniform variable.
     * @param uniformName Name of the variable.
     * @param array array to be set.
     * @returns this effect.
     */
    setUIntArray: (uniformName: string, array: Uint32Array) => Effect;
    /**
     * Sets an unsigned int array 2 on a uniform variable. (Array is specified as single array eg. [1,2,3,4] will result in [[1,2],[3,4]] in the shader)
     * @param uniformName Name of the variable.
     * @param array array to be set.
     * @returns this effect.
     */
    setUIntArray2: (uniformName: string, array: Uint32Array) => Effect;
    /**
     * Sets an unsigned int array 3 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6] will result in [[1,2,3],[4,5,6]] in the shader)
     * @param uniformName Name of the variable.
     * @param array array to be set.
     * @returns this effect.
     */
    setUIntArray3: (uniformName: string, array: Uint32Array) => Effect;
    /**
     * Sets an unsigned int array 4 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6,7,8] will result in [[1,2,3,4],[5,6,7,8]] in the shader)
     * @param uniformName Name of the variable.
     * @param array array to be set.
     * @returns this effect.
     */
    setUIntArray4: (uniformName: string, array: Uint32Array) => Effect;
    /**
     * Sets an float array on a uniform variable.
     * @param uniformName Name of the variable.
     * @param array array to be set.
     * @returns this effect.
     */
    setFloatArray(uniformName: string, array: FloatArray): Effect;
    /**
     * Sets an float array 2 on a uniform variable. (Array is specified as single array eg. [1,2,3,4] will result in [[1,2],[3,4]] in the shader)
     * @param uniformName Name of the variable.
     * @param array array to be set.
     * @returns this effect.
     */
    setFloatArray2(uniformName: string, array: FloatArray): Effect;
    /**
     * Sets an float array 3 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6] will result in [[1,2,3],[4,5,6]] in the shader)
     * @param uniformName Name of the variable.
     * @param array array to be set.
     * @returns this effect.
     */
    setFloatArray3(uniformName: string, array: FloatArray): Effect;
    /**
     * Sets an float array 4 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6,7,8] will result in [[1,2,3,4],[5,6,7,8]] in the shader)
     * @param uniformName Name of the variable.
     * @param array array to be set.
     * @returns this effect.
     */
    setFloatArray4(uniformName: string, array: FloatArray): Effect;
    /**
     * Sets an array on a uniform variable.
     * @param uniformName Name of the variable.
     * @param array array to be set.
     * @returns this effect.
     */
    setArray: (uniformName: string, array: number[]) => Effect;
    /**
     * Sets an array 2 on a uniform variable. (Array is specified as single array eg. [1,2,3,4] will result in [[1,2],[3,4]] in the shader)
     * @param uniformName Name of the variable.
     * @param array array to be set.
     * @returns this effect.
     */
    setArray2: (uniformName: string, array: number[]) => Effect;
    /**
     * Sets an array 3 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6] will result in [[1,2,3],[4,5,6]] in the shader)
     * @param uniformName Name of the variable.
     * @param array array to be set.
     * @returns this effect.
     */
    setArray3: (uniformName: string, array: number[]) => Effect;
    /**
     * Sets an array 4 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6,7,8] will result in [[1,2,3,4],[5,6,7,8]] in the shader)
     * @param uniformName Name of the variable.
     * @param array array to be set.
     * @returns this effect.
     */
    setArray4: (uniformName: string, array: number[]) => Effect;
    /**
     * Sets matrices on a uniform variable.
     * @param uniformName Name of the variable.
     * @param matrices matrices to be set.
     * @returns this effect.
     */
    setMatrices: (uniformName: string, matrices: Float32Array | Array<number>) => Effect;
    /**
     * Sets matrix on a uniform variable.
     * @param uniformName Name of the variable.
     * @param matrix matrix to be set.
     * @returns this effect.
     */
    setMatrix: (uniformName: string, matrix: IMatrixLike) => Effect;
    /**
     * Sets a 3x3 matrix on a uniform variable. (Specified as [1,2,3,4,5,6,7,8,9] will result in [1,2,3][4,5,6][7,8,9] matrix)
     * @param uniformName Name of the variable.
     * @param matrix matrix to be set.
     * @returns this effect.
     */
    setMatrix3x3: (uniformName: string, matrix: Float32Array | Array<number>) => Effect;
    /**
     * Sets a 2x2 matrix on a uniform variable. (Specified as [1,2,3,4] will result in [1,2][3,4] matrix)
     * @param uniformName Name of the variable.
     * @param matrix matrix to be set.
     * @returns this effect.
     */
    setMatrix2x2: (uniformName: string, matrix: Float32Array | Array<number>) => Effect;
    /**
     * Sets a float on a uniform variable.
     * @param uniformName Name of the variable.
     * @param value value to be set.
     * @returns this effect.
     */
    setFloat: (uniformName: string, value: number) => Effect;
    /**
     * Sets a boolean on a uniform variable.
     * @param uniformName Name of the variable.
     * @param bool value to be set.
     * @returns this effect.
     */
    setBool(uniformName: string, bool: boolean): Effect;
    /**
     * Sets a Vector2 on a uniform variable.
     * @param uniformName Name of the variable.
     * @param vector2 vector2 to be set.
     * @returns this effect.
     */
    setVector2: (uniformName: string, vector2: IVector2Like) => Effect;
    /**
     * Sets a float2 on a uniform variable.
     * @param uniformName Name of the variable.
     * @param x First float in float2.
     * @param y Second float in float2.
     * @returns this effect.
     */
    setFloat2: (uniformName: string, x: number, y: number) => Effect;
    /**
     * Sets a Vector3 on a uniform variable.
     * @param uniformName Name of the variable.
     * @param vector3 Value to be set.
     * @returns this effect.
     */
    setVector3: (uniformName: string, vector3: IVector3Like) => Effect;
    /**
     * Sets a float3 on a uniform variable.
     * @param uniformName Name of the variable.
     * @param x First float in float3.
     * @param y Second float in float3.
     * @param z Third float in float3.
     * @returns this effect.
     */
    setFloat3: (uniformName: string, x: number, y: number, z: number) => Effect;
    /**
     * Sets a Vector4 on a uniform variable.
     * @param uniformName Name of the variable.
     * @param vector4 Value to be set.
     * @returns this effect.
     */
    setVector4: (uniformName: string, vector4: IVector4Like) => Effect;
    /**
     * Sets a Quaternion on a uniform variable.
     * @param uniformName Name of the variable.
     * @param quaternion Value to be set.
     * @returns this effect.
     */
    setQuaternion: (uniformName: string, quaternion: IQuaternionLike) => Effect;
    /**
     * Sets a float4 on a uniform variable.
     * @param uniformName Name of the variable.
     * @param x First float in float4.
     * @param y Second float in float4.
     * @param z Third float in float4.
     * @param w Fourth float in float4.
     * @returns this effect.
     */
    setFloat4: (uniformName: string, x: number, y: number, z: number, w: number) => Effect;
    /**
     * Sets a Color3 on a uniform variable.
     * @param uniformName Name of the variable.
     * @param color3 Value to be set.
     * @returns this effect.
     */
    setColor3: (uniformName: string, color3: IColor3Like) => Effect;
    /**
     * Sets a Color4 on a uniform variable.
     * @param uniformName Name of the variable.
     * @param color3 Value to be set.
     * @param alpha Alpha value to be set.
     * @returns this effect.
     */
    setColor4: (uniformName: string, color3: IColor3Like, alpha: number) => Effect;
    /**
     * Sets a Color4 on a uniform variable
     * @param uniformName defines the name of the variable
     * @param color4 defines the value to be set
     * @returns this effect.
     */
    setDirectColor4: (uniformName: string, color4: IColor4Like) => Effect;
    /**
     * Release all associated resources.
     **/
    dispose(): void;
    /**
     * This function will add a new shader to the shader store
     * @param name the name of the shader
     * @param pixelShader optional pixel shader content
     * @param vertexShader optional vertex shader content
     * @param shaderLanguage the language the shader is written in (default: GLSL)
     */
    static RegisterShader(name: string, pixelShader?: string, vertexShader?: string, shaderLanguage?: ShaderLanguage): void;
    /**
     * Store of each shader (The can be looked up using effect.key)
     */
    static ShadersStore: {
        [key: string]: string;
    };
    /**
     * Store of each included file for a shader (The can be looked up using effect.key)
     */
    static IncludesShadersStore: {
        [key: string]: string;
    };
    /**
     * Resets the cache of effects.
     */
    static ResetCache(): void;
}
export {};

}
declare module "babylonjs/Materials/effectFallbacks" {
import { IEffectFallbacks } from "babylonjs/Materials/iEffectFallbacks";
import { Effect } from "babylonjs/Materials/effect";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
/**
 * EffectFallbacks can be used to add fallbacks (properties to disable) to certain properties when desired to improve performance.
 * (Eg. Start at high quality with reflection and fog, if fps is low, remove reflection, if still low remove fog)
 */
export class EffectFallbacks implements IEffectFallbacks {
    private _defines;
    private _currentRank;
    private _maxRank;
    private _mesh;
    /**
     * Removes the fallback from the bound mesh.
     */
    unBindMesh(): void;
    /**
     * Adds a fallback on the specified property.
     * @param rank The rank of the fallback (Lower ranks will be fallbacked to first)
     * @param define The name of the define in the shader
     */
    addFallback(rank: number, define: string): void;
    /**
     * Sets the mesh to use CPU skinning when needing to fallback.
     * @param rank The rank of the fallback (Lower ranks will be fallbacked to first)
     * @param mesh The mesh to use the fallbacks.
     */
    addCPUSkinningFallback(rank: number, mesh: AbstractMesh): void;
    /**
     * Checks to see if more fallbacks are still available.
     */
    get hasMoreFallbacks(): boolean;
    /**
     * Removes the defines that should be removed when falling back.
     * @param currentDefines defines the current define statements for the shader.
     * @param effect defines the current effect we try to compile
     * @returns The resulting defines with defines of the current rank removed.
     */
    reduce(currentDefines: string, effect: Effect): string;
}
export {};

}
declare module "babylonjs/Materials/effectRenderer" {
import { Nullable } from "babylonjs/types";
import { ThinEngine } from "babylonjs/Engines/thinEngine";
import { Viewport } from "babylonjs/Maths/math.viewport";
import { Observable } from "babylonjs/Misc/observable";
import { Effect } from "babylonjs/Materials/effect";
import { DrawWrapper } from "babylonjs/Materials/drawWrapper";
import { IRenderTargetTexture, RenderTargetWrapper } from "babylonjs/Engines/renderTargetWrapper";
import { ShaderLanguage } from "babylonjs/Materials/shaderLanguage";
import "babylonjs/Shaders/postprocess.vertex";
/**
 * Effect Render Options
 */
export interface IEffectRendererOptions {
    /**
     * Defines the vertices positions.
     */
    positions?: number[];
    /**
     * Defines the indices.
     */
    indices?: number[];
}
/**
 * Helper class to render one or more effects.
 * You can access the previous rendering in your shader by declaring a sampler named textureSampler
 */
export class EffectRenderer {
    /**
     * The engine the effect renderer has been created for.
     */
    readonly engine: ThinEngine;
    private _vertexBuffers;
    private _indexBuffer;
    private _fullscreenViewport;
    private _onContextRestoredObserver;
    /**
     * Creates an effect renderer
     * @param engine the engine to use for rendering
     * @param options defines the options of the effect renderer
     */
    constructor(engine: ThinEngine, options?: IEffectRendererOptions);
    /**
     * Sets the current viewport in normalized coordinates 0-1
     * @param viewport Defines the viewport to set (defaults to 0 0 1 1)
     */
    setViewport(viewport?: Viewport): void;
    /**
     * Binds the embedded attributes buffer to the effect.
     * @param effect Defines the effect to bind the attributes for
     */
    bindBuffers(effect: Effect): void;
    /**
     * Sets the current effect wrapper to use during draw.
     * The effect needs to be ready before calling this api.
     * This also sets the default full screen position attribute.
     * @param effectWrapper Defines the effect to draw with
     */
    applyEffectWrapper(effectWrapper: EffectWrapper): void;
    /**
     * Restores engine states
     */
    restoreStates(): void;
    /**
     * Draws a full screen quad.
     */
    draw(): void;
    private _isRenderTargetTexture;
    /**
     * renders one or more effects to a specified texture
     * @param effectWrapper the effect to renderer
     * @param outputTexture texture to draw to, if null it will render to the screen.
     */
    render(effectWrapper: EffectWrapper, outputTexture?: Nullable<RenderTargetWrapper | IRenderTargetTexture>): void;
    /**
     * Disposes of the effect renderer
     */
    dispose(): void;
}
/**
 * Options to create an EffectWrapper
 */
interface EffectWrapperCreationOptions {
    /**
     * Engine to use to create the effect
     */
    engine: ThinEngine;
    /**
     * Fragment shader for the effect
     */
    fragmentShader: string;
    /**
     * Use the shader store instead of direct source code
     */
    useShaderStore?: boolean;
    /**
     * Vertex shader for the effect
     */
    vertexShader?: string;
    /**
     * Attributes to use in the shader
     */
    attributeNames?: Array<string>;
    /**
     * Uniforms to use in the shader
     */
    uniformNames?: Array<string>;
    /**
     * Texture sampler names to use in the shader
     */
    samplerNames?: Array<string>;
    /**
     * Defines to use in the shader
     */
    defines?: Array<string>;
    /**
     * Callback when effect is compiled
     */
    onCompiled?: Nullable<(effect: Effect) => void>;
    /**
     * The friendly name of the effect displayed in Spector.
     */
    name?: string;
    /**
     * The language the shader is written in (default: GLSL)
     */
    shaderLanguage?: ShaderLanguage;
}
/**
 * Wraps an effect to be used for rendering
 */
export class EffectWrapper {
    /**
     * Event that is fired right before the effect is drawn (should be used to update uniforms)
     */
    onApplyObservable: Observable<{}>;
    /**
     * The underlying effect
     */
    get effect(): Effect;
    set effect(effect: Effect);
    /** @internal */
    _drawWrapper: DrawWrapper;
    private _onContextRestoredObserver;
    /**
     * Creates an effect to be renderer
     * @param creationOptions options to create the effect
     */
    constructor(creationOptions: EffectWrapperCreationOptions);
    /**
     * Disposes of the effect wrapper
     */
    dispose(): void;
}
export {};

}
declare module "babylonjs/Materials/fresnelParameters" {
import { DeepImmutable } from "babylonjs/types";
import { Color3 } from "babylonjs/Maths/math.color";
/**
 * Options to be used when creating a FresnelParameters.
 */
export type IFresnelParametersCreationOptions = {
    /**
     * Define the color used on edges (grazing angle)
     */
    leftColor?: Color3;
    /**
     * Define the color used on center
     */
    rightColor?: Color3;
    /**
     * Define bias applied to computed fresnel term
     */
    bias?: number;
    /**
     * Defined the power exponent applied to fresnel term
     */
    power?: number;
    /**
     * Define if the fresnel effect is enable or not.
     */
    isEnabled?: boolean;
};
/**
 * Serialized format for FresnelParameters.
 */
export type IFresnelParametersSerialized = {
    /**
     * Define the color used on edges (grazing angle) [as an array]
     */
    leftColor: number[];
    /**
     * Define the color used on center [as an array]
     */
    rightColor: number[];
    /**
     * Define bias applied to computed fresnel term
     */
    bias: number;
    /**
     * Defined the power exponent applied to fresnel term
     */
    power?: number;
    /**
     * Define if the fresnel effect is enable or not.
     */
    isEnabled: boolean;
};
/**
 * This represents all the required information to add a fresnel effect on a material:
 * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/fresnelParameters
 */
export class FresnelParameters {
    private _isEnabled;
    /**
     * Define if the fresnel effect is enable or not.
     */
    get isEnabled(): boolean;
    set isEnabled(value: boolean);
    /**
     * Define the color used on edges (grazing angle)
     */
    leftColor: Color3;
    /**
     * Define the color used on center
     */
    rightColor: Color3;
    /**
     * Define bias applied to computed fresnel term
     */
    bias: number;
    /**
     * Defined the power exponent applied to fresnel term
     */
    power: number;
    /**
     * Creates a new FresnelParameters object.
     *
     * @param options provide your own settings to optionally to override defaults
     */
    constructor(options?: IFresnelParametersCreationOptions);
    /**
     * Clones the current fresnel and its values
     * @returns a clone fresnel configuration
     */
    clone(): FresnelParameters;
    /**
     * Determines equality between FresnelParameters objects
     * @param otherFresnelParameters defines the second operand
     * @returns true if the power, bias, leftColor, rightColor and isEnabled values are equal to the given ones
     */
    equals(otherFresnelParameters: DeepImmutable<FresnelParameters>): boolean;
    /**
     * Serializes the current fresnel parameters to a JSON representation.
     * @returns the JSON serialization
     */
    serialize(): IFresnelParametersSerialized;
    /**
     * Parse a JSON object and deserialize it to a new Fresnel parameter object.
     * @param parsedFresnelParameters Define the JSON representation
     * @returns the parsed parameters
     */
    static Parse(parsedFresnelParameters: IFresnelParametersSerialized): FresnelParameters;
}

}
declare module "babylonjs/Materials/iEffectFallbacks" {
import { Effect } from "babylonjs/Materials/effect";
/**
 * Interface used to define common properties for effect fallbacks
 */
export interface IEffectFallbacks {
    /**
     * Removes the defines that should be removed when falling back.
     * @param currentDefines defines the current define statements for the shader.
     * @param effect defines the current effect we try to compile
     * @returns The resulting defines with defines of the current rank removed.
     */
    reduce(currentDefines: string, effect: Effect): string;
    /**
     * Removes the fallback from the bound mesh.
     */
    unBindMesh(): void;
    /**
     * Checks to see if more fallbacks are still available.
     */
    hasMoreFallbacks: boolean;
}
export {};

}
declare module "babylonjs/Materials/imageProcessingConfiguration" {
import { Observable } from "babylonjs/Misc/observable";
import { Nullable } from "babylonjs/types";
import { Color4 } from "babylonjs/Maths/math.color";
import { MaterialDefines } from "babylonjs/Materials/materialDefines";
import { ColorCurves } from "babylonjs/Materials/colorCurves";
import { BaseTexture } from "babylonjs/Materials/Textures/baseTexture";
import { Effect } from "babylonjs/Materials/effect";
/**
 * Interface to follow in your material defines to integrate easily the
 * Image processing functions.
 * @internal
 */
export interface IImageProcessingConfigurationDefines {
    IMAGEPROCESSING: boolean;
    VIGNETTE: boolean;
    VIGNETTEBLENDMODEMULTIPLY: boolean;
    VIGNETTEBLENDMODEOPAQUE: boolean;
    TONEMAPPING: boolean;
    TONEMAPPING_ACES: boolean;
    CONTRAST: boolean;
    EXPOSURE: boolean;
    COLORCURVES: boolean;
    COLORGRADING: boolean;
    COLORGRADING3D: boolean;
    SAMPLER3DGREENDEPTH: boolean;
    SAMPLER3DBGRMAP: boolean;
    DITHER: boolean;
    IMAGEPROCESSINGPOSTPROCESS: boolean;
    SKIPFINALCOLORCLAMP: boolean;
}
/**
 * @internal
 */
export class ImageProcessingConfigurationDefines extends MaterialDefines implements IImageProcessingConfigurationDefines {
    IMAGEPROCESSING: boolean;
    VIGNETTE: boolean;
    VIGNETTEBLENDMODEMULTIPLY: boolean;
    VIGNETTEBLENDMODEOPAQUE: boolean;
    TONEMAPPING: boolean;
    TONEMAPPING_ACES: boolean;
    CONTRAST: boolean;
    COLORCURVES: boolean;
    COLORGRADING: boolean;
    COLORGRADING3D: boolean;
    SAMPLER3DGREENDEPTH: boolean;
    SAMPLER3DBGRMAP: boolean;
    DITHER: boolean;
    IMAGEPROCESSINGPOSTPROCESS: boolean;
    EXPOSURE: boolean;
    SKIPFINALCOLORCLAMP: boolean;
    constructor();
}
/**
 * This groups together the common properties used for image processing either in direct forward pass
 * or through post processing effect depending on the use of the image processing pipeline in your scene
 * or not.
 */
export class ImageProcessingConfiguration {
    /**
     * Default tone mapping applied in BabylonJS.
     */
    static readonly TONEMAPPING_STANDARD: number;
    /**
     * ACES Tone mapping (used by default in unreal and unity). This can help getting closer
     * to other engines rendering to increase portability.
     */
    static readonly TONEMAPPING_ACES: number;
    /**
     * Color curves setup used in the effect if colorCurvesEnabled is set to true
     */
    colorCurves: Nullable<ColorCurves>;
    private _colorCurvesEnabled;
    /**
     * Gets whether the color curves effect is enabled.
     */
    get colorCurvesEnabled(): boolean;
    /**
     * Sets whether the color curves effect is enabled.
     */
    set colorCurvesEnabled(value: boolean);
    private _colorGradingTexture;
    /**
     * Color grading LUT texture used in the effect if colorGradingEnabled is set to true
     */
    get colorGradingTexture(): Nullable<BaseTexture>;
    /**
     * Color grading LUT texture used in the effect if colorGradingEnabled is set to true
     */
    set colorGradingTexture(value: Nullable<BaseTexture>);
    private _colorGradingEnabled;
    /**
     * Gets whether the color grading effect is enabled.
     */
    get colorGradingEnabled(): boolean;
    /**
     * Sets whether the color grading effect is enabled.
     */
    set colorGradingEnabled(value: boolean);
    private _colorGradingWithGreenDepth;
    /**
     * Gets whether the color grading effect is using a green depth for the 3d Texture.
     */
    get colorGradingWithGreenDepth(): boolean;
    /**
     * Sets whether the color grading effect is using a green depth for the 3d Texture.
     */
    set colorGradingWithGreenDepth(value: boolean);
    private _colorGradingBGR;
    /**
     * Gets whether the color grading texture contains BGR values.
     */
    get colorGradingBGR(): boolean;
    /**
     * Sets whether the color grading texture contains BGR values.
     */
    set colorGradingBGR(value: boolean);
    /** @internal */
    _exposure: number;
    /**
     * Gets the Exposure used in the effect.
     */
    get exposure(): number;
    /**
     * Sets the Exposure used in the effect.
     */
    set exposure(value: number);
    private _toneMappingEnabled;
    /**
     * Gets whether the tone mapping effect is enabled.
     */
    get toneMappingEnabled(): boolean;
    /**
     * Sets whether the tone mapping effect is enabled.
     */
    set toneMappingEnabled(value: boolean);
    private _toneMappingType;
    /**
     * Gets the type of tone mapping effect.
     */
    get toneMappingType(): number;
    /**
     * Sets the type of tone mapping effect used in BabylonJS.
     */
    set toneMappingType(value: number);
    protected _contrast: number;
    /**
     * Gets the contrast used in the effect.
     */
    get contrast(): number;
    /**
     * Sets the contrast used in the effect.
     */
    set contrast(value: number);
    /**
     * Vignette stretch size.
     */
    vignetteStretch: number;
    /**
     * Vignette center X Offset.
     */
    vignetteCenterX: number;
    /**
     * Vignette center Y Offset.
     */
    vignetteCenterY: number;
    /**
     * Back Compat: Vignette center Y Offset.
     * @deprecated use vignetteCenterY instead
     */
    get vignetteCentreY(): number;
    set vignetteCentreY(value: number);
    /**
     * Back Compat: Vignette center X Offset.
     * @deprecated use vignetteCenterX instead
     */
    get vignetteCentreX(): number;
    set vignetteCentreX(value: number);
    /**
     * Vignette weight or intensity of the vignette effect.
     */
    vignetteWeight: number;
    /**
     * Color of the vignette applied on the screen through the chosen blend mode (vignetteBlendMode)
     * if vignetteEnabled is set to true.
     */
    vignetteColor: Color4;
    /**
     * Camera field of view used by the Vignette effect.
     */
    vignetteCameraFov: number;
    private _vignetteBlendMode;
    /**
     * Gets the vignette blend mode allowing different kind of effect.
     */
    get vignetteBlendMode(): number;
    /**
     * Sets the vignette blend mode allowing different kind of effect.
     */
    set vignetteBlendMode(value: number);
    private _vignetteEnabled;
    /**
     * Gets whether the vignette effect is enabled.
     */
    get vignetteEnabled(): boolean;
    /**
     * Sets whether the vignette effect is enabled.
     */
    set vignetteEnabled(value: boolean);
    private _ditheringEnabled;
    /**
     * Gets whether the dithering effect is enabled.
     * The dithering effect can be used to reduce banding.
     */
    get ditheringEnabled(): boolean;
    /**
     * Sets whether the dithering effect is enabled.
     * The dithering effect can be used to reduce banding.
     */
    set ditheringEnabled(value: boolean);
    private _ditheringIntensity;
    /**
     * Gets the dithering intensity. 0 is no dithering. Default is 1.0 / 255.0.
     */
    get ditheringIntensity(): number;
    /**
     * Sets the dithering intensity. 0 is no dithering. Default is 1.0 / 255.0.
     */
    set ditheringIntensity(value: number);
    /** @internal */
    _skipFinalColorClamp: boolean;
    /**
     * If apply by post process is set to true, setting this to true will skip the the final color clamp step in the fragment shader
     * Applies to PBR materials.
     */
    get skipFinalColorClamp(): boolean;
    /**
     * If apply by post process is set to true, setting this to true will skip the the final color clamp step in the fragment shader
     * Applies to PBR materials.
     */
    set skipFinalColorClamp(value: boolean);
    /** @internal */
    _applyByPostProcess: boolean;
    /**
     * Gets whether the image processing is applied through a post process or not.
     */
    get applyByPostProcess(): boolean;
    /**
     * Sets whether the image processing is applied through a post process or not.
     */
    set applyByPostProcess(value: boolean);
    private _isEnabled;
    /**
     * Gets whether the image processing is enabled or not.
     */
    get isEnabled(): boolean;
    /**
     * Sets whether the image processing is enabled or not.
     */
    set isEnabled(value: boolean);
    /**
     * An event triggered when the configuration changes and requires Shader to Update some parameters.
     */
    onUpdateParameters: Observable<ImageProcessingConfiguration>;
    /**
     * Method called each time the image processing information changes requires to recompile the effect.
     */
    protected _updateParameters(): void;
    /**
     * Gets the current class name.
     * @returns "ImageProcessingConfiguration"
     */
    getClassName(): string;
    /**
     * Prepare the list of uniforms associated with the Image Processing effects.
     * @param uniforms The list of uniforms used in the effect
     * @param defines the list of defines currently in use
     */
    static PrepareUniforms(uniforms: string[], defines: IImageProcessingConfigurationDefines): void;
    /**
     * Prepare the list of samplers associated with the Image Processing effects.
     * @param samplersList The list of uniforms used in the effect
     * @param defines the list of defines currently in use
     */
    static PrepareSamplers(samplersList: string[], defines: IImageProcessingConfigurationDefines): void;
    /**
     * Prepare the list of defines associated to the shader.
     * @param defines the list of defines to complete
     * @param forPostProcess Define if we are currently in post process mode or not
     */
    prepareDefines(defines: IImageProcessingConfigurationDefines, forPostProcess?: boolean): void;
    /**
     * Returns true if all the image processing information are ready.
     * @returns True if ready, otherwise, false
     */
    isReady(): boolean;
    /**
     * Binds the image processing to the shader.
     * @param effect The effect to bind to
     * @param overrideAspectRatio Override the aspect ratio of the effect
     */
    bind(effect: Effect, overrideAspectRatio?: number): void;
    /**
     * Clones the current image processing instance.
     * @returns The cloned image processing
     */
    clone(): ImageProcessingConfiguration;
    /**
     * Serializes the current image processing instance to a json representation.
     * @returns a JSON representation
     */
    serialize(): any;
    /**
     * Parses the image processing from a json representation.
     * @param source the JSON source to parse
     * @returns The parsed image processing
     */
    static Parse(source: any): ImageProcessingConfiguration;
    private static _VIGNETTEMODE_MULTIPLY;
    private static _VIGNETTEMODE_OPAQUE;
    /**
     * Used to apply the vignette as a mix with the pixel color.
     */
    static get VIGNETTEMODE_MULTIPLY(): number;
    /**
     * Used to apply the vignette as a replacement of the pixel color.
     */
    static get VIGNETTEMODE_OPAQUE(): number;
}
export {};

}
declare module "babylonjs/Materials/index" {
export * from "babylonjs/Materials/Background/index";
export * from "babylonjs/Materials/colorCurves";
export * from "babylonjs/Materials/iEffectFallbacks";
export * from "babylonjs/Materials/effectFallbacks";
export * from "babylonjs/Materials/effect";
export * from "babylonjs/Materials/fresnelParameters";
export * from "babylonjs/Materials/imageProcessingConfiguration";
export * from "babylonjs/Materials/material";
export * from "babylonjs/Materials/materialDefines";
export * from "babylonjs/Materials/clipPlaneMaterialHelper";
export * from "babylonjs/Materials/materialHelper";
export * from "babylonjs/Materials/multiMaterial";
export * from "babylonjs/Materials/Occlusion/index";
export * from "babylonjs/Materials/PBR/index";
export * from "babylonjs/Materials/pushMaterial";
export * from "babylonjs/Materials/shaderLanguage";
export * from "babylonjs/Materials/shaderMaterial";
export * from "babylonjs/Materials/standardMaterial";
export * from "babylonjs/Materials/Textures/index";
export * from "babylonjs/Materials/uniformBuffer";
export * from "babylonjs/Materials/materialFlags";
export * from "babylonjs/Materials/Node/index";
export * from "babylonjs/Materials/effectRenderer";
export * from "babylonjs/Materials/shadowDepthWrapper";
export * from "babylonjs/Materials/drawWrapper";
export * from "babylonjs/Materials/materialPluginBase";
export * from "babylonjs/Materials/materialPluginManager";
export * from "babylonjs/Materials/materialPluginEvent";
export * from "babylonjs/Materials/material.detailMapConfiguration";
export * from "babylonjs/Materials/materialPluginFactoryExport";

}
declare module "babylonjs/Materials/material" {
import { IAnimatable } from "babylonjs/Animations/animatable.interface";
import { SmartArray } from "babylonjs/Misc/smartArray";
import { Observable } from "babylonjs/Misc/observable";
import { Nullable } from "babylonjs/types";
import { Matrix } from "babylonjs/Maths/math.vector";
import { SubMesh } from "babylonjs/Meshes/subMesh";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { UniformBuffer } from "babylonjs/Materials/uniformBuffer";
import { Effect } from "babylonjs/Materials/effect";
import { BaseTexture } from "babylonjs/Materials/Textures/baseTexture";
import { RenderTargetTexture } from "babylonjs/Materials/Textures/renderTargetTexture";
import { MaterialDefines } from "babylonjs/Materials/materialDefines";
import { IInspectable } from "babylonjs/Misc/iInspectable";
import { Plane } from "babylonjs/Maths/math.plane";
import { ShadowDepthWrapper } from "babylonjs/Materials/shadowDepthWrapper";
import { IMaterialContext } from "babylonjs/Engines/IMaterialContext";
import { DrawWrapper } from "babylonjs/Materials/drawWrapper";
import { MaterialStencilState } from "babylonjs/Materials/materialStencilState";
import { Scene } from "babylonjs/scene";
import { AbstractScene } from "babylonjs/abstractScene";
import { MaterialPluginDisposed, MaterialPluginIsReadyForSubMesh, MaterialPluginGetDefineNames, MaterialPluginBindForSubMesh, MaterialPluginGetActiveTextures, MaterialPluginHasTexture, MaterialPluginGetAnimatables, MaterialPluginPrepareDefines, MaterialPluginPrepareEffect, MaterialPluginPrepareUniformBuffer, MaterialPluginCreated, MaterialPluginFillRenderTargetTextures, MaterialPluginHasRenderTargetTextures, MaterialPluginHardBindForSubMesh } from "babylonjs/Materials/materialPluginEvent";
import { ShaderCustomProcessingFunction } from "babylonjs/Engines/Processors/shaderProcessingOptions";
import { IClipPlanesHolder } from "babylonjs/Misc/interfaces/iClipPlanesHolder";
import { PrePassRenderer } from "babylonjs/Rendering/prePassRenderer";
import { Mesh } from "babylonjs/Meshes/mesh";
import { Animation } from "babylonjs/Animations/animation";
/**
 * Options for compiling materials.
 */
export interface IMaterialCompilationOptions {
    /**
     * Defines whether clip planes are enabled.
     */
    clipPlane: boolean;
    /**
     * Defines whether instances are enabled.
     */
    useInstances: boolean;
}
/**
 * Options passed when calling customShaderNameResolve
 */
export interface ICustomShaderNameResolveOptions {
    /**
     * If provided, will be called two times with the vertex and fragment code so that this code can be updated before it is compiled by the GPU
     */
    processFinalCode?: Nullable<ShaderCustomProcessingFunction>;
}
/**
 * Base class for the main features of a material in Babylon.js
 */
export class Material implements IAnimatable, IClipPlanesHolder {
    /**
     * Returns the triangle fill mode
     */
    static readonly TriangleFillMode: number;
    /**
     * Returns the wireframe mode
     */
    static readonly WireFrameFillMode: number;
    /**
     * Returns the point fill mode
     */
    static readonly PointFillMode: number;
    /**
     * Returns the point list draw mode
     */
    static readonly PointListDrawMode: number;
    /**
     * Returns the line list draw mode
     */
    static readonly LineListDrawMode: number;
    /**
     * Returns the line loop draw mode
     */
    static readonly LineLoopDrawMode: number;
    /**
     * Returns the line strip draw mode
     */
    static readonly LineStripDrawMode: number;
    /**
     * Returns the triangle strip draw mode
     */
    static readonly TriangleStripDrawMode: number;
    /**
     * Returns the triangle fan draw mode
     */
    static readonly TriangleFanDrawMode: number;
    /**
     * Stores the clock-wise side orientation
     */
    static readonly ClockWiseSideOrientation: number;
    /**
     * Stores the counter clock-wise side orientation
     */
    static readonly CounterClockWiseSideOrientation: number;
    /**
     * The dirty texture flag value
     */
    static readonly TextureDirtyFlag: number;
    /**
     * The dirty light flag value
     */
    static readonly LightDirtyFlag: number;
    /**
     * The dirty fresnel flag value
     */
    static readonly FresnelDirtyFlag: number;
    /**
     * The dirty attribute flag value
     */
    static readonly AttributesDirtyFlag: number;
    /**
     * The dirty misc flag value
     */
    static readonly MiscDirtyFlag: number;
    /**
     * The dirty prepass flag value
     */
    static readonly PrePassDirtyFlag: number;
    /**
     * The all dirty flag value
     */
    static readonly AllDirtyFlag: number;
    /**
     * MaterialTransparencyMode: No transparency mode, Alpha channel is not use.
     */
    static readonly MATERIAL_OPAQUE: number;
    /**
     * MaterialTransparencyMode: Alpha Test mode, pixel are discarded below a certain threshold defined by the alpha cutoff value.
     */
    static readonly MATERIAL_ALPHATEST: number;
    /**
     * MaterialTransparencyMode: Pixels are blended (according to the alpha mode) with the already drawn pixels in the current frame buffer.
     */
    static readonly MATERIAL_ALPHABLEND: number;
    /**
     * MaterialTransparencyMode: Pixels are blended (according to the alpha mode) with the already drawn pixels in the current frame buffer.
     * They are also discarded below the alpha cutoff threshold to improve performances.
     */
    static readonly MATERIAL_ALPHATESTANDBLEND: number;
    /**
     * The Whiteout method is used to blend normals.
     * Details of the algorithm can be found here: https://blog.selfshadow.com/publications/blending-in-detail/
     */
    static readonly MATERIAL_NORMALBLENDMETHOD_WHITEOUT: number;
    /**
     * The Reoriented Normal Mapping method is used to blend normals.
     * Details of the algorithm can be found here: https://blog.selfshadow.com/publications/blending-in-detail/
     */
    static readonly MATERIAL_NORMALBLENDMETHOD_RNM: number;
    /**
     * Event observable which raises global events common to all materials (like MaterialPluginEvent.Created)
     */
    static OnEventObservable: Observable<Material>;
    /**
     * Custom callback helping to override the default shader used in the material.
     */
    customShaderNameResolve: (shaderName: string, uniforms: string[], uniformBuffers: string[], samplers: string[], defines: MaterialDefines | string[], attributes?: string[], options?: ICustomShaderNameResolveOptions) => string;
    /**
     * Custom shadow depth material to use for shadow rendering instead of the in-built one
     */
    shadowDepthWrapper: Nullable<ShadowDepthWrapper>;
    /**
     * Gets or sets a boolean indicating that the material is allowed (if supported) to do shader hot swapping.
     * This means that the material can keep using a previous shader while a new one is being compiled.
     * This is mostly used when shader parallel compilation is supported (true by default)
     */
    allowShaderHotSwapping: boolean;
    /**
     * The ID of the material
     */
    id: string;
    /**
     * Gets or sets the unique id of the material
     */
    uniqueId: number;
    /** @internal */
    _loadedUniqueId: string;
    /**
     * The name of the material
     */
    name: string;
    /**
     * Gets or sets user defined metadata
     */
    metadata: any;
    /** @internal */
    _internalMetadata: any;
    /**
     * For internal use only. Please do not use.
     */
    reservedDataStore: any;
    /**
     * Specifies if the ready state should be checked on each call
     */
    checkReadyOnEveryCall: boolean;
    /**
     * Specifies if the ready state should be checked once
     */
    checkReadyOnlyOnce: boolean;
    /**
     * The state of the material
     */
    state: string;
    /**
     * If the material can be rendered to several textures with MRT extension
     */
    get canRenderToMRT(): boolean;
    /**
     * The alpha value of the material
     */
    protected _alpha: number;
    /**
     * List of inspectable custom properties (used by the Inspector)
     * @see https://doc.babylonjs.com/toolsAndResources/inspector#extensibility
     */
    inspectableCustomProperties: IInspectable[];
    /**
     * Sets the alpha value of the material
     */
    set alpha(value: number);
    /**
     * Gets the alpha value of the material
     */
    get alpha(): number;
    /**
     * Specifies if back face culling is enabled
     */
    protected _backFaceCulling: boolean;
    /**
     * Sets the culling state (true to enable culling, false to disable)
     */
    set backFaceCulling(value: boolean);
    /**
     * Gets the culling state
     */
    get backFaceCulling(): boolean;
    /**
     * Specifies if back or front faces should be culled (when culling is enabled)
     */
    protected _cullBackFaces: boolean;
    /**
     * Sets the type of faces that should be culled (true for back faces, false for front faces)
     */
    set cullBackFaces(value: boolean);
    /**
     * Gets the type of faces that should be culled
     */
    get cullBackFaces(): boolean;
    private _blockDirtyMechanism;
    /**
     * Block the dirty-mechanism for this specific material
     * When set to false after being true the material will be marked as dirty.
     */
    get blockDirtyMechanism(): boolean;
    set blockDirtyMechanism(value: boolean);
    /**
     * This allows you to modify the material without marking it as dirty after every change.
     * This function should be used if you need to make more than one dirty-enabling change to the material - adding a texture, setting a new fill mode and so on.
     * The callback will pass the material as an argument, so you can make your changes to it.
     * @param callback the callback to be executed that will update the material
     */
    atomicMaterialsUpdate(callback: (material: this) => void): void;
    /**
     * Stores the value for side orientation
     */
    sideOrientation: number;
    /**
     * Callback triggered when the material is compiled
     */
    onCompiled: Nullable<(effect: Effect) => void>;
    /**
     * Callback triggered when an error occurs
     */
    onError: Nullable<(effect: Effect, errors: string) => void>;
    /**
     * Callback triggered to get the render target textures
     */
    getRenderTargetTextures: Nullable<() => SmartArray<RenderTargetTexture>>;
    /**
     * Gets a boolean indicating that current material needs to register RTT
     */
    get hasRenderTargetTextures(): boolean;
    /**
     * Specifies if the material should be serialized
     */
    doNotSerialize: boolean;
    /**
     * @internal
     */
    _storeEffectOnSubMeshes: boolean;
    /**
     * Stores the animations for the material
     */
    animations: Nullable<Array<Animation>>;
    /**
     * An event triggered when the material is disposed
     */
    onDisposeObservable: Observable<Material>;
    /**
     * An observer which watches for dispose events
     */
    private _onDisposeObserver;
    private _onUnBindObservable;
    /**
     * Called during a dispose event
     */
    set onDispose(callback: () => void);
    private _onBindObservable;
    /**
     * An event triggered when the material is bound
     */
    get onBindObservable(): Observable<AbstractMesh>;
    /**
     * An observer which watches for bind events
     */
    private _onBindObserver;
    /**
     * Called during a bind event
     */
    set onBind(callback: (Mesh: AbstractMesh) => void);
    /**
     * An event triggered when the material is unbound
     */
    get onUnBindObservable(): Observable<Material>;
    protected _onEffectCreatedObservable: Nullable<Observable<{
        effect: Effect;
        subMesh: Nullable<SubMesh>;
    }>>;
    /**
     * An event triggered when the effect is (re)created
     */
    get onEffectCreatedObservable(): Observable<{
        effect: Effect;
        subMesh: Nullable<SubMesh>;
    }>;
    /**
     * Stores the value of the alpha mode
     */
    private _alphaMode;
    /**
     * Sets the value of the alpha mode.
     *
     * | Value | Type | Description |
     * | --- | --- | --- |
     * | 0 | ALPHA_DISABLE |   |
     * | 1 | ALPHA_ADD |   |
     * | 2 | ALPHA_COMBINE |   |
     * | 3 | ALPHA_SUBTRACT |   |
     * | 4 | ALPHA_MULTIPLY |   |
     * | 5 | ALPHA_MAXIMIZED |   |
     * | 6 | ALPHA_ONEONE |   |
     * | 7 | ALPHA_PREMULTIPLIED |   |
     * | 8 | ALPHA_PREMULTIPLIED_PORTERDUFF |   |
     * | 9 | ALPHA_INTERPOLATE |   |
     * | 10 | ALPHA_SCREENMODE |   |
     *
     */
    set alphaMode(value: number);
    /**
     * Gets the value of the alpha mode
     */
    get alphaMode(): number;
    /**
     * Stores the state of the need depth pre-pass value
     */
    private _needDepthPrePass;
    /**
     * Sets the need depth pre-pass value
     */
    set needDepthPrePass(value: boolean);
    /**
     * Gets the depth pre-pass value
     */
    get needDepthPrePass(): boolean;
    /**
     * Can this material render to prepass
     */
    get isPrePassCapable(): boolean;
    /**
     * Specifies if depth writing should be disabled
     */
    disableDepthWrite: boolean;
    /**
     * Specifies if color writing should be disabled
     */
    disableColorWrite: boolean;
    /**
     * Specifies if depth writing should be forced
     */
    forceDepthWrite: boolean;
    /**
     * Specifies the depth function that should be used. 0 means the default engine function
     */
    depthFunction: number;
    /**
     * Specifies if there should be a separate pass for culling
     */
    separateCullingPass: boolean;
    /**
     * Stores the state specifying if fog should be enabled
     */
    private _fogEnabled;
    /**
     * Sets the state for enabling fog
     */
    set fogEnabled(value: boolean);
    /**
     * Gets the value of the fog enabled state
     */
    get fogEnabled(): boolean;
    /**
     * Stores the size of points
     */
    pointSize: number;
    /**
     * Stores the z offset Factor value
     */
    zOffset: number;
    /**
     * Stores the z offset Units value
     */
    zOffsetUnits: number;
    get wireframe(): boolean;
    /**
     * Sets the state of wireframe mode
     */
    set wireframe(value: boolean);
    /**
     * Gets the value specifying if point clouds are enabled
     */
    get pointsCloud(): boolean;
    /**
     * Sets the state of point cloud mode
     */
    set pointsCloud(value: boolean);
    /**
     * Gets the material fill mode
     */
    get fillMode(): number;
    /**
     * Sets the material fill mode
     */
    set fillMode(value: number);
    /**
     * Gets or sets the active clipplane 1
     */
    clipPlane: Nullable<Plane>;
    /**
     * Gets or sets the active clipplane 2
     */
    clipPlane2: Nullable<Plane>;
    /**
     * Gets or sets the active clipplane 3
     */
    clipPlane3: Nullable<Plane>;
    /**
     * Gets or sets the active clipplane 4
     */
    clipPlane4: Nullable<Plane>;
    /**
     * Gets or sets the active clipplane 5
     */
    clipPlane5: Nullable<Plane>;
    /**
     * Gets or sets the active clipplane 6
     */
    clipPlane6: Nullable<Plane>;
    /**
     * Gives access to the stencil properties of the material
     */
    readonly stencil: MaterialStencilState;
    /**
     * @internal
     * Stores the effects for the material
     */
    protected _materialContext: IMaterialContext | undefined;
    protected _drawWrapper: DrawWrapper;
    /** @internal */
    _getDrawWrapper(): DrawWrapper;
    /**
     * @internal
     */
    _setDrawWrapper(drawWrapper: DrawWrapper): void;
    /**
     * Specifies if uniform buffers should be used
     */
    private _useUBO;
    /**
     * Stores a reference to the scene
     */
    private _scene;
    protected _needToBindSceneUbo: boolean;
    /**
     * Stores the fill mode state
     */
    private _fillMode;
    /**
     * Specifies if the depth write state should be cached
     */
    private _cachedDepthWriteState;
    /**
     * Specifies if the color write state should be cached
     */
    private _cachedColorWriteState;
    /**
     * Specifies if the depth function state should be cached
     */
    private _cachedDepthFunctionState;
    /**
     * Stores the uniform buffer
     * @internal
     */
    _uniformBuffer: UniformBuffer;
    /** @internal */
    _indexInSceneMaterialArray: number;
    /** @internal */
    meshMap: Nullable<{
        [id: string]: AbstractMesh | undefined;
    }>;
    /** @internal */
    _parentContainer: Nullable<AbstractScene>;
    /** @internal */
    _dirtyCallbacks: {
        [code: number]: () => void;
    };
    /** @internal */
    _uniformBufferLayoutBuilt: boolean;
    protected _eventInfo: MaterialPluginCreated & MaterialPluginDisposed & MaterialPluginHasTexture & MaterialPluginIsReadyForSubMesh & MaterialPluginGetDefineNames & MaterialPluginPrepareEffect & MaterialPluginPrepareDefines & MaterialPluginPrepareUniformBuffer & MaterialPluginBindForSubMesh & MaterialPluginGetAnimatables & MaterialPluginGetActiveTextures & MaterialPluginFillRenderTargetTextures & MaterialPluginHasRenderTargetTextures & MaterialPluginHardBindForSubMesh;
    /** @internal */
    _callbackPluginEventGeneric: (id: number, info: MaterialPluginGetActiveTextures | MaterialPluginGetAnimatables | MaterialPluginHasTexture | MaterialPluginDisposed | MaterialPluginGetDefineNames | MaterialPluginPrepareEffect | MaterialPluginPrepareUniformBuffer) => void;
    /** @internal */
    _callbackPluginEventIsReadyForSubMesh: (eventData: MaterialPluginIsReadyForSubMesh) => void;
    /** @internal */
    _callbackPluginEventPrepareDefines: (eventData: MaterialPluginPrepareDefines) => void;
    /** @internal */
    _callbackPluginEventPrepareDefinesBeforeAttributes: (eventData: MaterialPluginPrepareDefines) => void;
    /** @internal */
    _callbackPluginEventHardBindForSubMesh: (eventData: MaterialPluginHardBindForSubMesh) => void;
    /** @internal */
    _callbackPluginEventBindForSubMesh: (eventData: MaterialPluginBindForSubMesh) => void;
    /** @internal */
    _callbackPluginEventHasRenderTargetTextures: (eventData: MaterialPluginHasRenderTargetTextures) => void;
    /** @internal */
    _callbackPluginEventFillRenderTargetTextures: (eventData: MaterialPluginFillRenderTargetTextures) => void;
    /**
     * Creates a material instance
     * @param name defines the name of the material
     * @param scene defines the scene to reference
     * @param doNotAdd specifies if the material should be added to the scene
     */
    constructor(name: string, scene?: Nullable<Scene>, doNotAdd?: boolean);
    /**
     * Returns a string representation of the current material
     * @param fullDetails defines a boolean indicating which levels of logging is desired
     * @returns a string with material information
     */
    toString(fullDetails?: boolean): string;
    /**
     * Gets the class name of the material
     * @returns a string with the class name of the material
     */
    getClassName(): string;
    /** @internal */
    get _isMaterial(): boolean;
    /**
     * Specifies if updates for the material been locked
     */
    get isFrozen(): boolean;
    /**
     * Locks updates for the material
     */
    freeze(): void;
    /**
     * Unlocks updates for the material
     */
    unfreeze(): void;
    /**
     * Specifies if the material is ready to be used
     * @param mesh defines the mesh to check
     * @param useInstances specifies if instances should be used
     * @returns a boolean indicating if the material is ready to be used
     */
    isReady(mesh?: AbstractMesh, useInstances?: boolean): boolean;
    /**
     * Specifies that the submesh is ready to be used
     * @param mesh defines the mesh to check
     * @param subMesh defines which submesh to check
     * @param useInstances specifies that instances should be used
     * @returns a boolean indicating that the submesh is ready or not
     */
    isReadyForSubMesh(mesh: AbstractMesh, subMesh: SubMesh, useInstances?: boolean): boolean;
    /**
     * Returns the material effect
     * @returns the effect associated with the material
     */
    getEffect(): Nullable<Effect>;
    /**
     * Returns the current scene
     * @returns a Scene
     */
    getScene(): Scene;
    /**
     * Enforces alpha test in opaque or blend mode in order to improve the performances of some situations.
     */
    protected _forceAlphaTest: boolean;
    /**
     * The transparency mode of the material.
     */
    protected _transparencyMode: Nullable<number>;
    /**
     * Gets the current transparency mode.
     */
    get transparencyMode(): Nullable<number>;
    /**
     * Sets the transparency mode of the material.
     *
     * | Value | Type                                | Description |
     * | ----- | ----------------------------------- | ----------- |
     * | 0     | OPAQUE                              |             |
     * | 1     | ALPHATEST                           |             |
     * | 2     | ALPHABLEND                          |             |
     * | 3     | ALPHATESTANDBLEND                   |             |
     *
     */
    set transparencyMode(value: Nullable<number>);
    /**
     * Returns true if alpha blending should be disabled.
     */
    protected get _disableAlphaBlending(): boolean;
    /**
     * Specifies whether or not this material should be rendered in alpha blend mode.
     * @returns a boolean specifying if alpha blending is needed
     */
    needAlphaBlending(): boolean;
    /**
     * Specifies if the mesh will require alpha blending
     * @param mesh defines the mesh to check
     * @returns a boolean specifying if alpha blending is needed for the mesh
     */
    needAlphaBlendingForMesh(mesh: AbstractMesh): boolean;
    /**
     * Specifies whether or not this material should be rendered in alpha test mode.
     * @returns a boolean specifying if an alpha test is needed.
     */
    needAlphaTesting(): boolean;
    /**
     * Specifies if material alpha testing should be turned on for the mesh
     * @param mesh defines the mesh to check
     */
    protected _shouldTurnAlphaTestOn(mesh: AbstractMesh): boolean;
    /**
     * Gets the texture used for the alpha test
     * @returns the texture to use for alpha testing
     */
    getAlphaTestTexture(): Nullable<BaseTexture>;
    /**
     * Marks the material to indicate that it needs to be re-calculated
     * @param forceMaterialDirty - Forces the material to be marked as dirty for all components (same as this.markAsDirty(Material.AllDirtyFlag)). You should use this flag if the material is frozen and you want to force a recompilation.
     */
    markDirty(forceMaterialDirty?: boolean): void;
    /**
     * @internal
     */
    _preBind(effect?: Effect | DrawWrapper, overrideOrientation?: Nullable<number>): boolean;
    /**
     * Binds the material to the mesh
     * @param world defines the world transformation matrix
     * @param mesh defines the mesh to bind the material to
     */
    bind(world: Matrix, mesh?: Mesh): void;
    /**
     * Initializes the uniform buffer layout for the shader.
     */
    buildUniformLayout(): void;
    /**
     * Binds the submesh to the material
     * @param world defines the world transformation matrix
     * @param mesh defines the mesh containing the submesh
     * @param subMesh defines the submesh to bind the material to
     */
    bindForSubMesh(world: Matrix, mesh: Mesh, subMesh: SubMesh): void;
    /**
     * Binds the world matrix to the material
     * @param world defines the world transformation matrix
     */
    bindOnlyWorldMatrix(world: Matrix): void;
    /**
     * Binds the view matrix to the effect
     * @param effect defines the effect to bind the view matrix to
     */
    bindView(effect: Effect): void;
    /**
     * Binds the view projection and projection matrices to the effect
     * @param effect defines the effect to bind the view projection and projection matrices to
     */
    bindViewProjection(effect: Effect): void;
    /**
     * Binds the view matrix to the effect
     * @param effect defines the effect to bind the view matrix to
     * @param variableName name of the shader variable that will hold the eye position
     */
    bindEyePosition(effect: Effect, variableName?: string): void;
    /**
     * Processes to execute after binding the material to a mesh
     * @param mesh defines the rendered mesh
     * @param effect
     */
    protected _afterBind(mesh?: Mesh, effect?: Nullable<Effect>): void;
    /**
     * Unbinds the material from the mesh
     */
    unbind(): void;
    /**
     * Returns the animatable textures.
     * @returns - Array of animatable textures.
     */
    getAnimatables(): IAnimatable[];
    /**
     * Gets the active textures from the material
     * @returns an array of textures
     */
    getActiveTextures(): BaseTexture[];
    /**
     * Specifies if the material uses a texture
     * @param texture defines the texture to check against the material
     * @returns a boolean specifying if the material uses the texture
     */
    hasTexture(texture: BaseTexture): boolean;
    /**
     * Makes a duplicate of the material, and gives it a new name
     * @param name defines the new name for the duplicated material
     * @returns the cloned material
     */
    clone(name: string): Nullable<Material>;
    /**
     * Gets the meshes bound to the material
     * @returns an array of meshes bound to the material
     */
    getBindedMeshes(): AbstractMesh[];
    /**
     * Force shader compilation
     * @param mesh defines the mesh associated with this material
     * @param onCompiled defines a function to execute once the material is compiled
     * @param options defines the options to configure the compilation
     * @param onError defines a function to execute if the material fails compiling
     */
    forceCompilation(mesh: AbstractMesh, onCompiled?: (material: Material) => void, options?: Partial<IMaterialCompilationOptions>, onError?: (reason: string) => void): void;
    /**
     * Force shader compilation
     * @param mesh defines the mesh that will use this material
     * @param options defines additional options for compiling the shaders
     * @returns a promise that resolves when the compilation completes
     */
    forceCompilationAsync(mesh: AbstractMesh, options?: Partial<IMaterialCompilationOptions>): Promise<void>;
    private static readonly _AllDirtyCallBack;
    private static readonly _ImageProcessingDirtyCallBack;
    private static readonly _TextureDirtyCallBack;
    private static readonly _FresnelDirtyCallBack;
    private static readonly _MiscDirtyCallBack;
    private static readonly _PrePassDirtyCallBack;
    private static readonly _LightsDirtyCallBack;
    private static readonly _AttributeDirtyCallBack;
    private static _FresnelAndMiscDirtyCallBack;
    private static _TextureAndMiscDirtyCallBack;
    private static readonly _DirtyCallbackArray;
    private static readonly _RunDirtyCallBacks;
    /**
     * Marks a define in the material to indicate that it needs to be re-computed
     * @param flag defines a flag used to determine which parts of the material have to be marked as dirty
     */
    markAsDirty(flag: number): void;
    /**
     * Resets the draw wrappers cache for all submeshes that are using this material
     */
    resetDrawCache(): void;
    /**
     * Marks all submeshes of a material to indicate that their material defines need to be re-calculated
     * @param func defines a function which checks material defines against the submeshes
     */
    protected _markAllSubMeshesAsDirty(func: (defines: MaterialDefines) => void): void;
    /**
     * Indicates that the scene should check if the rendering now needs a prepass
     */
    protected _markScenePrePassDirty(): void;
    /**
     * Indicates that we need to re-calculated for all submeshes
     */
    protected _markAllSubMeshesAsAllDirty(): void;
    /**
     * Indicates that image processing needs to be re-calculated for all submeshes
     */
    protected _markAllSubMeshesAsImageProcessingDirty(): void;
    /**
     * Indicates that textures need to be re-calculated for all submeshes
     */
    protected _markAllSubMeshesAsTexturesDirty(): void;
    /**
     * Indicates that fresnel needs to be re-calculated for all submeshes
     */
    protected _markAllSubMeshesAsFresnelDirty(): void;
    /**
     * Indicates that fresnel and misc need to be re-calculated for all submeshes
     */
    protected _markAllSubMeshesAsFresnelAndMiscDirty(): void;
    /**
     * Indicates that lights need to be re-calculated for all submeshes
     */
    protected _markAllSubMeshesAsLightsDirty(): void;
    /**
     * Indicates that attributes need to be re-calculated for all submeshes
     */
    protected _markAllSubMeshesAsAttributesDirty(): void;
    /**
     * Indicates that misc needs to be re-calculated for all submeshes
     */
    protected _markAllSubMeshesAsMiscDirty(): void;
    /**
     * Indicates that prepass needs to be re-calculated for all submeshes
     */
    protected _markAllSubMeshesAsPrePassDirty(): void;
    /**
     * Indicates that textures and misc need to be re-calculated for all submeshes
     */
    protected _markAllSubMeshesAsTexturesAndMiscDirty(): void;
    /**
     * Sets the required values to the prepass renderer.
     * @param prePassRenderer defines the prepass renderer to setup.
     * @returns true if the pre pass is needed.
     */
    setPrePassRenderer(prePassRenderer: PrePassRenderer): boolean;
    /**
     * Disposes the material
     * @param forceDisposeEffect specifies if effects should be forcefully disposed
     * @param forceDisposeTextures specifies if textures should be forcefully disposed
     * @param notBoundToMesh specifies if the material that is being disposed is known to be not bound to any mesh
     */
    dispose(forceDisposeEffect?: boolean, forceDisposeTextures?: boolean, notBoundToMesh?: boolean): void;
    /**
     * @internal
     */
    private releaseVertexArrayObject;
    /**
     * Serializes this material
     * @returns the serialized material object
     */
    serialize(): any;
    /**
     * Creates a material from parsed material data
     * @param parsedMaterial defines parsed material data
     * @param scene defines the hosting scene
     * @param rootUrl defines the root URL to use to load textures
     * @returns a new material
     */
    static Parse(parsedMaterial: any, scene: Scene, rootUrl: string): Nullable<Material>;
}
export {};

}
declare module "babylonjs/Materials/material.detailMapConfiguration" {
import { Nullable } from "babylonjs/types";
import { BaseTexture } from "babylonjs/Materials/Textures/baseTexture";
import { UniformBuffer } from "babylonjs/Materials/uniformBuffer";
import { IAnimatable } from "babylonjs/Animations/animatable.interface";
import { MaterialDefines } from "babylonjs/Materials/materialDefines";
import { MaterialPluginBase } from "babylonjs/Materials/materialPluginBase";
import { Engine } from "babylonjs/Engines/engine";
import { Scene } from "babylonjs/scene";
import { StandardMaterial } from "babylonjs/Materials/standardMaterial";
import { PBRBaseMaterial } from "babylonjs/Materials/PBR/pbrBaseMaterial";
/**
 * @internal
 */
export class MaterialDetailMapDefines extends MaterialDefines {
    DETAIL: boolean;
    DETAILDIRECTUV: number;
    DETAIL_NORMALBLENDMETHOD: number;
}
/**
 * Plugin that implements the detail map component of a material
 *
 * Inspired from:
 *   Unity: https://docs.unity3d.com/Packages/com.unity.render-pipelines.high-definition@9.0/manual/Mask-Map-and-Detail-Map.html and https://docs.unity3d.com/Manual/StandardShaderMaterialParameterDetail.html
 *   Unreal: https://docs.unrealengine.com/en-US/Engine/Rendering/Materials/HowTo/DetailTexturing/index.html
 *   Cryengine: https://docs.cryengine.com/display/SDKDOC2/Detail+Maps
 */
export class DetailMapConfiguration extends MaterialPluginBase {
    private _texture;
    /**
     * The detail texture of the material.
     */
    texture: Nullable<BaseTexture>;
    /**
     * Defines how strongly the detail diffuse/albedo channel is blended with the regular diffuse/albedo texture
     * Bigger values mean stronger blending
     */
    diffuseBlendLevel: number;
    /**
     * Defines how strongly the detail roughness channel is blended with the regular roughness value
     * Bigger values mean stronger blending. Only used with PBR materials
     */
    roughnessBlendLevel: number;
    /**
     * Defines how strong the bump effect from the detail map is
     * Bigger values mean stronger effect
     */
    bumpLevel: number;
    private _normalBlendMethod;
    /**
     * The method used to blend the bump and detail normals together
     */
    normalBlendMethod: number;
    private _isEnabled;
    /**
     * Enable or disable the detail map on this material
     */
    isEnabled: boolean;
    /** @internal */
    private _internalMarkAllSubMeshesAsTexturesDirty;
    /** @internal */
    _markAllSubMeshesAsTexturesDirty(): void;
    constructor(material: PBRBaseMaterial | StandardMaterial, addToPluginList?: boolean);
    isReadyForSubMesh(defines: MaterialDetailMapDefines, scene: Scene, engine: Engine): boolean;
    prepareDefines(defines: MaterialDetailMapDefines, scene: Scene): void;
    bindForSubMesh(uniformBuffer: UniformBuffer, scene: Scene): void;
    hasTexture(texture: BaseTexture): boolean;
    getActiveTextures(activeTextures: BaseTexture[]): void;
    getAnimatables(animatables: IAnimatable[]): void;
    dispose(forceDisposeTextures?: boolean): void;
    getClassName(): string;
    getSamplers(samplers: string[]): void;
    getUniforms(): {
        ubo?: Array<{
            name: string;
            size: number;
            type: string;
        }>;
        vertex?: string;
        fragment?: string;
    };
}
export {};

}
declare module "babylonjs/Materials/materialDefines" {
/**
 * Manages the defines for the Material
 */
export class MaterialDefines {
    /** @internal */
    protected _keys: string[];
    private _isDirty;
    /** @internal */
    _renderId: number;
    /** @internal */
    _areLightsDirty: boolean;
    /** @internal */
    _areLightsDisposed: boolean;
    /** @internal */
    _areAttributesDirty: boolean;
    /** @internal */
    _areTexturesDirty: boolean;
    /** @internal */
    _areFresnelDirty: boolean;
    /** @internal */
    _areMiscDirty: boolean;
    /** @internal */
    _arePrePassDirty: boolean;
    /** @internal */
    _areImageProcessingDirty: boolean;
    /** @internal */
    _normals: boolean;
    /** @internal */
    _uvs: boolean;
    /** @internal */
    _needNormals: boolean;
    /** @internal */
    _needUVs: boolean;
    protected _externalProperties?: {
        [name: string]: {
            type: string;
            default: any;
        };
    };
    [id: string]: any;
    /**
     * Creates a new instance
     * @param externalProperties list of external properties to inject into the object
     */
    constructor(externalProperties?: {
        [name: string]: {
            type: string;
            default: any;
        };
    });
    /**
     * Specifies if the material needs to be re-calculated
     */
    get isDirty(): boolean;
    /**
     * Marks the material to indicate that it has been re-calculated
     */
    markAsProcessed(): void;
    /**
     * Marks the material to indicate that it needs to be re-calculated
     */
    markAsUnprocessed(): void;
    /**
     * Marks the material to indicate all of its defines need to be re-calculated
     */
    markAllAsDirty(): void;
    /**
     * Marks the material to indicate that image processing needs to be re-calculated
     */
    markAsImageProcessingDirty(): void;
    /**
     * Marks the material to indicate the lights need to be re-calculated
     * @param disposed Defines whether the light is dirty due to dispose or not
     */
    markAsLightDirty(disposed?: boolean): void;
    /**
     * Marks the attribute state as changed
     */
    markAsAttributesDirty(): void;
    /**
     * Marks the texture state as changed
     */
    markAsTexturesDirty(): void;
    /**
     * Marks the fresnel state as changed
     */
    markAsFresnelDirty(): void;
    /**
     * Marks the misc state as changed
     */
    markAsMiscDirty(): void;
    /**
     * Marks the prepass state as changed
     */
    markAsPrePassDirty(): void;
    /**
     * Rebuilds the material defines
     */
    rebuild(): void;
    /**
     * Specifies if two material defines are equal
     * @param other - A material define instance to compare to
     * @returns - Boolean indicating if the material defines are equal (true) or not (false)
     */
    isEqual(other: MaterialDefines): boolean;
    /**
     * Clones this instance's defines to another instance
     * @param other - material defines to clone values to
     */
    cloneTo(other: MaterialDefines): void;
    /**
     * Resets the material define values
     */
    reset(): void;
    private _setDefaultValue;
    /**
     * Converts the material define values to a string
     * @returns - String of material define information
     */
    toString(): string;
}

}
declare module "babylonjs/Materials/materialFlags" {
/**
 * This groups all the flags used to control the materials channel.
 */
export class MaterialFlags {
    private static _DiffuseTextureEnabled;
    /**
     * Are diffuse textures enabled in the application.
     */
    static get DiffuseTextureEnabled(): boolean;
    static set DiffuseTextureEnabled(value: boolean);
    private static _DetailTextureEnabled;
    /**
     * Are detail textures enabled in the application.
     */
    static get DetailTextureEnabled(): boolean;
    static set DetailTextureEnabled(value: boolean);
    private static _AmbientTextureEnabled;
    /**
     * Are ambient textures enabled in the application.
     */
    static get AmbientTextureEnabled(): boolean;
    static set AmbientTextureEnabled(value: boolean);
    private static _OpacityTextureEnabled;
    /**
     * Are opacity textures enabled in the application.
     */
    static get OpacityTextureEnabled(): boolean;
    static set OpacityTextureEnabled(value: boolean);
    private static _ReflectionTextureEnabled;
    /**
     * Are reflection textures enabled in the application.
     */
    static get ReflectionTextureEnabled(): boolean;
    static set ReflectionTextureEnabled(value: boolean);
    private static _EmissiveTextureEnabled;
    /**
     * Are emissive textures enabled in the application.
     */
    static get EmissiveTextureEnabled(): boolean;
    static set EmissiveTextureEnabled(value: boolean);
    private static _SpecularTextureEnabled;
    /**
     * Are specular textures enabled in the application.
     */
    static get SpecularTextureEnabled(): boolean;
    static set SpecularTextureEnabled(value: boolean);
    private static _BumpTextureEnabled;
    /**
     * Are bump textures enabled in the application.
     */
    static get BumpTextureEnabled(): boolean;
    static set BumpTextureEnabled(value: boolean);
    private static _LightmapTextureEnabled;
    /**
     * Are lightmap textures enabled in the application.
     */
    static get LightmapTextureEnabled(): boolean;
    static set LightmapTextureEnabled(value: boolean);
    private static _RefractionTextureEnabled;
    /**
     * Are refraction textures enabled in the application.
     */
    static get RefractionTextureEnabled(): boolean;
    static set RefractionTextureEnabled(value: boolean);
    private static _ColorGradingTextureEnabled;
    /**
     * Are color grading textures enabled in the application.
     */
    static get ColorGradingTextureEnabled(): boolean;
    static set ColorGradingTextureEnabled(value: boolean);
    private static _FresnelEnabled;
    /**
     * Are fresnels enabled in the application.
     */
    static get FresnelEnabled(): boolean;
    static set FresnelEnabled(value: boolean);
    private static _ClearCoatTextureEnabled;
    /**
     * Are clear coat textures enabled in the application.
     */
    static get ClearCoatTextureEnabled(): boolean;
    static set ClearCoatTextureEnabled(value: boolean);
    private static _ClearCoatBumpTextureEnabled;
    /**
     * Are clear coat bump textures enabled in the application.
     */
    static get ClearCoatBumpTextureEnabled(): boolean;
    static set ClearCoatBumpTextureEnabled(value: boolean);
    private static _ClearCoatTintTextureEnabled;
    /**
     * Are clear coat tint textures enabled in the application.
     */
    static get ClearCoatTintTextureEnabled(): boolean;
    static set ClearCoatTintTextureEnabled(value: boolean);
    private static _SheenTextureEnabled;
    /**
     * Are sheen textures enabled in the application.
     */
    static get SheenTextureEnabled(): boolean;
    static set SheenTextureEnabled(value: boolean);
    private static _AnisotropicTextureEnabled;
    /**
     * Are anisotropic textures enabled in the application.
     */
    static get AnisotropicTextureEnabled(): boolean;
    static set AnisotropicTextureEnabled(value: boolean);
    private static _ThicknessTextureEnabled;
    /**
     * Are thickness textures enabled in the application.
     */
    static get ThicknessTextureEnabled(): boolean;
    static set ThicknessTextureEnabled(value: boolean);
    private static _RefractionIntensityTextureEnabled;
    /**
     * Are refraction intensity textures enabled in the application.
     */
    static get RefractionIntensityTextureEnabled(): boolean;
    static set RefractionIntensityTextureEnabled(value: boolean);
    private static _TranslucencyIntensityTextureEnabled;
    /**
     * Are translucency intensity textures enabled in the application.
     */
    static get TranslucencyIntensityTextureEnabled(): boolean;
    static set TranslucencyIntensityTextureEnabled(value: boolean);
    private static _IridescenceTextureEnabled;
    /**
     * Are translucency intensity textures enabled in the application.
     */
    static get IridescenceTextureEnabled(): boolean;
    static set IridescenceTextureEnabled(value: boolean);
}

}
declare module "babylonjs/Materials/materialHelper" {
import { Nullable } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
import { Engine } from "babylonjs/Engines/engine";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { Light } from "babylonjs/Lights/light";
import { PrePassConfiguration } from "babylonjs/Materials/prePassConfiguration";
import { UniformBuffer } from "babylonjs/Materials/uniformBuffer";
import { Effect, IEffectCreationOptions } from "babylonjs/Materials/effect";
import { BaseTexture } from "babylonjs/Materials/Textures/baseTexture";
import { MaterialDefines } from "babylonjs/Materials/materialDefines";
import { EffectFallbacks } from "babylonjs/Materials/effectFallbacks";
import { Material } from "babylonjs/Materials/material";
/**
 * "Static Class" containing the most commonly used helper while dealing with material for rendering purpose.
 *
 * It contains the basic tools to help defining defines, binding uniform for the common part of the materials.
 *
 * This works by convention in BabylonJS but is meant to be use only with shader following the in place naming rules and conventions.
 */
export class MaterialHelper {
    /**
     * Binds the scene's uniform buffer to the effect.
     * @param effect defines the effect to bind to the scene uniform buffer
     * @param sceneUbo defines the uniform buffer storing scene data
     */
    static BindSceneUniformBuffer(effect: Effect, sceneUbo: UniformBuffer): void;
    /**
     * Helps preparing the defines values about the UVs in used in the effect.
     * UVs are shared as much as we can across channels in the shaders.
     * @param texture The texture we are preparing the UVs for
     * @param defines The defines to update
     * @param key The channel key "diffuse", "specular"... used in the shader
     */
    static PrepareDefinesForMergedUV(texture: BaseTexture, defines: any, key: string): void;
    /**
     * Binds a texture matrix value to its corresponding uniform
     * @param texture The texture to bind the matrix for
     * @param uniformBuffer The uniform buffer receiving the data
     * @param key The channel key "diffuse", "specular"... used in the shader
     */
    static BindTextureMatrix(texture: BaseTexture, uniformBuffer: UniformBuffer, key: string): void;
    /**
     * Gets the current status of the fog (should it be enabled?)
     * @param mesh defines the mesh to evaluate for fog support
     * @param scene defines the hosting scene
     * @returns true if fog must be enabled
     */
    static GetFogState(mesh: AbstractMesh, scene: Scene): boolean;
    /**
     * Helper used to prepare the list of defines associated with misc. values for shader compilation
     * @param mesh defines the current mesh
     * @param scene defines the current scene
     * @param useLogarithmicDepth defines if logarithmic depth has to be turned on
     * @param pointsCloud defines if point cloud rendering has to be turned on
     * @param fogEnabled defines if fog has to be turned on
     * @param alphaTest defines if alpha testing has to be turned on
     * @param defines defines the current list of defines
     */
    static PrepareDefinesForMisc(mesh: AbstractMesh, scene: Scene, useLogarithmicDepth: boolean, pointsCloud: boolean, fogEnabled: boolean, alphaTest: boolean, defines: any): void;
    /**
     * Helper used to prepare the defines relative to the active camera
     * @param scene defines the current scene
     * @param defines specifies the list of active defines
     * @returns true if the defines have been updated, else false
     */
    static PrepareDefinesForCamera(scene: Scene, defines: any): boolean;
    /**
     * Helper used to prepare the list of defines associated with frame values for shader compilation
     * @param scene defines the current scene
     * @param engine defines the current engine
     * @param material defines the material we are compiling the shader for
     * @param defines specifies the list of active defines
     * @param useInstances defines if instances have to be turned on
     * @param useClipPlane defines if clip plane have to be turned on
     * @param useThinInstances defines if thin instances have to be turned on
     */
    static PrepareDefinesForFrameBoundValues(scene: Scene, engine: Engine, material: Material, defines: any, useInstances: boolean, useClipPlane?: Nullable<boolean>, useThinInstances?: boolean): void;
    /**
     * Prepares the defines for bones
     * @param mesh The mesh containing the geometry data we will draw
     * @param defines The defines to update
     */
    static PrepareDefinesForBones(mesh: AbstractMesh, defines: any): void;
    /**
     * Prepares the defines for morph targets
     * @param mesh The mesh containing the geometry data we will draw
     * @param defines The defines to update
     */
    static PrepareDefinesForMorphTargets(mesh: AbstractMesh, defines: any): void;
    /**
     * Prepares the defines for baked vertex animation
     * @param mesh The mesh containing the geometry data we will draw
     * @param defines The defines to update
     */
    static PrepareDefinesForBakedVertexAnimation(mesh: AbstractMesh, defines: any): void;
    /**
     * Prepares the defines used in the shader depending on the attributes data available in the mesh
     * @param mesh The mesh containing the geometry data we will draw
     * @param defines The defines to update
     * @param useVertexColor Precise whether vertex colors should be used or not (override mesh info)
     * @param useBones Precise whether bones should be used or not (override mesh info)
     * @param useMorphTargets Precise whether morph targets should be used or not (override mesh info)
     * @param useVertexAlpha Precise whether vertex alpha should be used or not (override mesh info)
     * @param useBakedVertexAnimation Precise whether baked vertex animation should be used or not (override mesh info)
     * @returns false if defines are considered not dirty and have not been checked
     */
    static PrepareDefinesForAttributes(mesh: AbstractMesh, defines: any, useVertexColor: boolean, useBones: boolean, useMorphTargets?: boolean, useVertexAlpha?: boolean, useBakedVertexAnimation?: boolean): boolean;
    /**
     * Prepares the defines related to multiview
     * @param scene The scene we are intending to draw
     * @param defines The defines to update
     */
    static PrepareDefinesForMultiview(scene: Scene, defines: any): void;
    /**
     * Prepares the defines related to order independant transparency
     * @param scene The scene we are intending to draw
     * @param defines The defines to update
     * @param needAlphaBlending Determines if the material needs alpha blending
     */
    static PrepareDefinesForOIT(scene: Scene, defines: any, needAlphaBlending: boolean): void;
    /**
     * Prepares the defines related to the prepass
     * @param scene The scene we are intending to draw
     * @param defines The defines to update
     * @param canRenderToMRT Indicates if this material renders to several textures in the prepass
     */
    static PrepareDefinesForPrePass(scene: Scene, defines: any, canRenderToMRT: boolean): void;
    /**
     * Prepares the defines related to the light information passed in parameter
     * @param scene The scene we are intending to draw
     * @param mesh The mesh the effect is compiling for
     * @param light The light the effect is compiling for
     * @param lightIndex The index of the light
     * @param defines The defines to update
     * @param specularSupported Specifies whether specular is supported or not (override lights data)
     * @param state Defines the current state regarding what is needed (normals, etc...)
     * @param state.needNormals
     * @param state.needRebuild
     * @param state.shadowEnabled
     * @param state.specularEnabled
     * @param state.lightmapMode
     */
    static PrepareDefinesForLight(scene: Scene, mesh: AbstractMesh, light: Light, lightIndex: number, defines: any, specularSupported: boolean, state: {
        needNormals: boolean;
        needRebuild: boolean;
        shadowEnabled: boolean;
        specularEnabled: boolean;
        lightmapMode: boolean;
    }): void;
    /**
     * Prepares the defines related to the light information passed in parameter
     * @param scene The scene we are intending to draw
     * @param mesh The mesh the effect is compiling for
     * @param defines The defines to update
     * @param specularSupported Specifies whether specular is supported or not (override lights data)
     * @param maxSimultaneousLights Specifies how manuy lights can be added to the effect at max
     * @param disableLighting Specifies whether the lighting is disabled (override scene and light)
     * @returns true if normals will be required for the rest of the effect
     */
    static PrepareDefinesForLights(scene: Scene, mesh: AbstractMesh, defines: any, specularSupported: boolean, maxSimultaneousLights?: number, disableLighting?: boolean): boolean;
    /**
     * Prepares the uniforms and samplers list to be used in the effect (for a specific light)
     * @param lightIndex defines the light index
     * @param uniformsList The uniform list
     * @param samplersList The sampler list
     * @param projectedLightTexture defines if projected texture must be used
     * @param uniformBuffersList defines an optional list of uniform buffers
     * @param updateOnlyBuffersList True to only update the uniformBuffersList array
     */
    static PrepareUniformsAndSamplersForLight(lightIndex: number, uniformsList: string[], samplersList: string[], projectedLightTexture?: any, uniformBuffersList?: Nullable<string[]>, updateOnlyBuffersList?: boolean): void;
    /**
     * Prepares the uniforms and samplers list to be used in the effect
     * @param uniformsListOrOptions The uniform names to prepare or an EffectCreationOptions containing the list and extra information
     * @param samplersList The sampler list
     * @param defines The defines helping in the list generation
     * @param maxSimultaneousLights The maximum number of simultaneous light allowed in the effect
     */
    static PrepareUniformsAndSamplersList(uniformsListOrOptions: string[] | IEffectCreationOptions, samplersList?: string[], defines?: any, maxSimultaneousLights?: number): void;
    /**
     * This helps decreasing rank by rank the shadow quality (0 being the highest rank and quality)
     * @param defines The defines to update while falling back
     * @param fallbacks The authorized effect fallbacks
     * @param maxSimultaneousLights The maximum number of lights allowed
     * @param rank the current rank of the Effect
     * @returns The newly affected rank
     */
    static HandleFallbacksForShadows(defines: any, fallbacks: EffectFallbacks, maxSimultaneousLights?: number, rank?: number): number;
    private static _TmpMorphInfluencers;
    /**
     * Prepares the list of attributes required for morph targets according to the effect defines.
     * @param attribs The current list of supported attribs
     * @param mesh The mesh to prepare the morph targets attributes for
     * @param influencers The number of influencers
     */
    static PrepareAttributesForMorphTargetsInfluencers(attribs: string[], mesh: AbstractMesh, influencers: number): void;
    /**
     * Prepares the list of attributes required for morph targets according to the effect defines.
     * @param attribs The current list of supported attribs
     * @param mesh The mesh to prepare the morph targets attributes for
     * @param defines The current Defines of the effect
     */
    static PrepareAttributesForMorphTargets(attribs: string[], mesh: AbstractMesh, defines: any): void;
    /**
     * Prepares the list of attributes required for baked vertex animations according to the effect defines.
     * @param attribs The current list of supported attribs
     * @param mesh The mesh to prepare the morph targets attributes for
     * @param defines The current Defines of the effect
     */
    static PrepareAttributesForBakedVertexAnimation(attribs: string[], mesh: AbstractMesh, defines: any): void;
    /**
     * Prepares the list of attributes required for bones according to the effect defines.
     * @param attribs The current list of supported attribs
     * @param mesh The mesh to prepare the bones attributes for
     * @param defines The current Defines of the effect
     * @param fallbacks The current effect fallback strategy
     */
    static PrepareAttributesForBones(attribs: string[], mesh: AbstractMesh, defines: any, fallbacks: EffectFallbacks): void;
    /**
     * Check and prepare the list of attributes required for instances according to the effect defines.
     * @param attribs The current list of supported attribs
     * @param defines The current MaterialDefines of the effect
     */
    static PrepareAttributesForInstances(attribs: string[], defines: MaterialDefines): void;
    /**
     * Add the list of attributes required for instances to the attribs array.
     * @param attribs The current list of supported attribs
     * @param needsPreviousMatrices If the shader needs previous matrices
     */
    static PushAttributesForInstances(attribs: string[], needsPreviousMatrices?: boolean): void;
    /**
     * Binds the light information to the effect.
     * @param light The light containing the generator
     * @param effect The effect we are binding the data to
     * @param lightIndex The light index in the effect used to render
     */
    static BindLightProperties(light: Light, effect: Effect, lightIndex: number): void;
    /**
     * Binds the lights information from the scene to the effect for the given mesh.
     * @param light Light to bind
     * @param lightIndex Light index
     * @param scene The scene where the light belongs to
     * @param effect The effect we are binding the data to
     * @param useSpecular Defines if specular is supported
     * @param receiveShadows Defines if the effect (mesh) we bind the light for receives shadows
     */
    static BindLight(light: Light, lightIndex: number, scene: Scene, effect: Effect, useSpecular: boolean, receiveShadows?: boolean): void;
    /**
     * Binds the lights information from the scene to the effect for the given mesh.
     * @param scene The scene the lights belongs to
     * @param mesh The mesh we are binding the information to render
     * @param effect The effect we are binding the data to
     * @param defines The generated defines for the effect
     * @param maxSimultaneousLights The maximum number of light that can be bound to the effect
     */
    static BindLights(scene: Scene, mesh: AbstractMesh, effect: Effect, defines: any, maxSimultaneousLights?: number): void;
    private static _TempFogColor;
    /**
     * Binds the fog information from the scene to the effect for the given mesh.
     * @param scene The scene the lights belongs to
     * @param mesh The mesh we are binding the information to render
     * @param effect The effect we are binding the data to
     * @param linearSpace Defines if the fog effect is applied in linear space
     */
    static BindFogParameters(scene: Scene, mesh: AbstractMesh, effect: Effect, linearSpace?: boolean): void;
    /**
     * Binds the bones information from the mesh to the effect.
     * @param mesh The mesh we are binding the information to render
     * @param effect The effect we are binding the data to
     * @param prePassConfiguration Configuration for the prepass, in case prepass is activated
     */
    static BindBonesParameters(mesh?: AbstractMesh, effect?: Effect, prePassConfiguration?: PrePassConfiguration): void;
    private static _CopyBonesTransformationMatrices;
    /**
     * Binds the morph targets information from the mesh to the effect.
     * @param abstractMesh The mesh we are binding the information to render
     * @param effect The effect we are binding the data to
     */
    static BindMorphTargetParameters(abstractMesh: AbstractMesh, effect: Effect): void;
    /**
     * Binds the logarithmic depth information from the scene to the effect for the given defines.
     * @param defines The generated defines used in the effect
     * @param effect The effect we are binding the data to
     * @param scene The scene we are willing to render with logarithmic scale for
     */
    static BindLogDepth(defines: any, effect: Effect, scene: Scene): void;
}

}
declare module "babylonjs/Materials/materialPluginBase" {
import { Nullable } from "babylonjs/types";
import { MaterialPluginManager } from "babylonjs/Materials/materialPluginManager";
import { SmartArray } from "babylonjs/Misc/smartArray";
import { Engine } from "babylonjs/Engines/engine";
import { Scene } from "babylonjs/scene";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { SubMesh } from "babylonjs/Meshes/subMesh";
import { IAnimatable } from "babylonjs/Animations/animatable.interface";
import { UniformBuffer } from "babylonjs/Materials/uniformBuffer";
import { EffectFallbacks } from "babylonjs/Materials/effectFallbacks";
import { MaterialDefines } from "babylonjs/Materials/materialDefines";
import { Material } from "babylonjs/Materials/material";
import { BaseTexture } from "babylonjs/Materials/Textures/baseTexture";
import { RenderTargetTexture } from "babylonjs/Materials/Textures/renderTargetTexture";
/**
 * Base class for material plugins.
 * @since 5.0
 */
export class MaterialPluginBase {
    /**
     * Defines the name of the plugin
     */
    name: string;
    /**
     * Defines the priority of the plugin. Lower numbers run first.
     */
    priority: number;
    /**
     * Indicates that this plugin should be notified for the extra events (HasRenderTargetTextures / FillRenderTargetTextures / HardBindForSubMesh)
     */
    registerForExtraEvents: boolean;
    protected _material: Material;
    protected _pluginManager: MaterialPluginManager;
    protected _pluginDefineNames?: {
        [name: string]: any;
    };
    protected _enable(enable: boolean): void;
    /**
     * Helper function to mark defines as being dirty.
     */
    readonly markAllDefinesAsDirty: () => void;
    /**
     * Creates a new material plugin
     * @param material parent material of the plugin
     * @param name name of the plugin
     * @param priority priority of the plugin
     * @param defines list of defines used by the plugin. The value of the property is the default value for this property
     * @param addToPluginList true to add the plugin to the list of plugins managed by the material plugin manager of the material (default: true)
     * @param enable true to enable the plugin (it is handy if the plugin does not handle properties to switch its current activation)
     */
    constructor(material: Material, name: string, priority: number, defines?: {
        [key: string]: any;
    }, addToPluginList?: boolean, enable?: boolean);
    /**
     * Gets the current class name useful for serialization or dynamic coding.
     * @returns The class name.
     */
    getClassName(): string;
    /**
     * Specifies that the submesh is ready to be used.
     * @param defines the list of "defines" to update.
     * @param scene defines the scene the material belongs to.
     * @param engine the engine this scene belongs to.
     * @param subMesh the submesh to check for readiness
     * @returns - boolean indicating that the submesh is ready or not.
     */
    isReadyForSubMesh(defines: MaterialDefines, scene: Scene, engine: Engine, subMesh: SubMesh): boolean;
    /**
     * Binds the material data (this function is called even if mustRebind() returns false)
     * @param uniformBuffer defines the Uniform buffer to fill in.
     * @param scene defines the scene the material belongs to.
     * @param engine defines the engine the material belongs to.
     * @param subMesh the submesh to bind data for
     */
    hardBindForSubMesh(uniformBuffer: UniformBuffer, scene: Scene, engine: Engine, subMesh: SubMesh): void;
    /**
     * Binds the material data.
     * @param uniformBuffer defines the Uniform buffer to fill in.
     * @param scene defines the scene the material belongs to.
     * @param engine the engine this scene belongs to.
     * @param subMesh the submesh to bind data for
     */
    bindForSubMesh(uniformBuffer: UniformBuffer, scene: Scene, engine: Engine, subMesh: SubMesh): void;
    /**
     * Disposes the resources of the material.
     * @param forceDisposeTextures - Forces the disposal of all textures.
     */
    dispose(forceDisposeTextures?: boolean): void;
    /**
     * Returns a list of custom shader code fragments to customize the shader.
     * @param shaderType "vertex" or "fragment"
     * @returns null if no code to be added, or a list of pointName => code.
     * Note that `pointName` can also be a regular expression if it starts with a `!`.
     * In that case, the string found by the regular expression (if any) will be
     * replaced by the code provided.
     */
    getCustomCode(shaderType: string): Nullable<{
        [pointName: string]: string;
    }>;
    /**
     * Collects all defines.
     * @param defines The object to append to.
     */
    collectDefines(defines: {
        [name: string]: {
            type: string;
            default: any;
        };
    }): void;
    /**
     * Sets the defines for the next rendering. Called before MaterialHelper.PrepareDefinesForAttributes is called.
     * @param defines the list of "defines" to update.
     * @param scene defines the scene to the material belongs to.
     * @param mesh the mesh being rendered
     */
    prepareDefinesBeforeAttributes(defines: MaterialDefines, scene: Scene, mesh: AbstractMesh): void;
    /**
     * Sets the defines for the next rendering
     * @param defines the list of "defines" to update.
     * @param scene defines the scene to the material belongs to.
     * @param mesh the mesh being rendered
     */
    prepareDefines(defines: MaterialDefines, scene: Scene, mesh: AbstractMesh): void;
    /**
     * Checks to see if a texture is used in the material.
     * @param texture - Base texture to use.
     * @returns - Boolean specifying if a texture is used in the material.
     */
    hasTexture(texture: BaseTexture): boolean;
    /**
     * Gets a boolean indicating that current material needs to register RTT
     * @returns true if this uses a render target otherwise false.
     */
    hasRenderTargetTextures(): boolean;
    /**
     * Fills the list of render target textures.
     * @param renderTargets the list of render targets to update
     */
    fillRenderTargetTextures(renderTargets: SmartArray<RenderTargetTexture>): void;
    /**
     * Returns an array of the actively used textures.
     * @param activeTextures Array of BaseTextures
     */
    getActiveTextures(activeTextures: BaseTexture[]): void;
    /**
     * Returns the animatable textures.
     * @param animatables Array of animatable textures.
     */
    getAnimatables(animatables: IAnimatable[]): void;
    /**
     * Add fallbacks to the effect fallbacks list.
     * @param defines defines the Base texture to use.
     * @param fallbacks defines the current fallback list.
     * @param currentRank defines the current fallback rank.
     * @returns the new fallback rank.
     */
    addFallbacks(defines: MaterialDefines, fallbacks: EffectFallbacks, currentRank: number): number;
    /**
     * Gets the samplers used by the plugin.
     * @param samplers list that the sampler names should be added to.
     */
    getSamplers(samplers: string[]): void;
    /**
     * Gets the attributes used by the plugin.
     * @param attributes list that the attribute names should be added to.
     * @param scene the scene that the material belongs to.
     * @param mesh the mesh being rendered.
     */
    getAttributes(attributes: string[], scene: Scene, mesh: AbstractMesh): void;
    /**
     * Gets the uniform buffers names added by the plugin.
     * @param ubos list that the ubo names should be added to.
     */
    getUniformBuffersNames(ubos: string[]): void;
    /**
     * Gets the description of the uniforms to add to the ubo (if engine supports ubos) or to inject directly in the vertex/fragment shaders (if engine does not support ubos)
     * @returns the description of the uniforms
     */
    getUniforms(): {
        ubo?: Array<{
            name: string;
            size: number;
            type: string;
        }>;
        vertex?: string;
        fragment?: string;
    };
    /**
     * Makes a duplicate of the current configuration into another one.
     * @param plugin define the config where to copy the info
     */
    copyTo(plugin: MaterialPluginBase): void;
    /**
     * Serializes this clear coat configuration.
     * @returns - An object with the serialized config.
     */
    serialize(): any;
    /**
     * Parses a anisotropy Configuration from a serialized object.
     * @param source - Serialized object.
     * @param scene Defines the scene we are parsing for
     * @param rootUrl Defines the rootUrl to load from
     */
    parse(source: any, scene: Scene, rootUrl: string): void;
}
export {};

}
declare module "babylonjs/Materials/materialPluginEvent" {
import { ShaderCustomProcessingFunction } from "babylonjs/Engines/Processors/shaderProcessingOptions";
import { SmartArray } from "babylonjs/Misc/smartArray";
import { BaseTexture } from "babylonjs/Materials/Textures/baseTexture";
import { EffectFallbacks } from "babylonjs/Materials/effectFallbacks";
import { MaterialDefines } from "babylonjs/Materials/materialDefines";
import { UniformBuffer } from "babylonjs/Materials/uniformBuffer";
import { SubMesh } from "babylonjs/Meshes/subMesh";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { IAnimatable } from "babylonjs/Animations/animatable.interface";
import { RenderTargetTexture } from "babylonjs/Materials/Textures/renderTargetTexture";
/** @internal */
export type MaterialPluginCreated = {};
/** @internal */
export type MaterialPluginDisposed = {
    forceDisposeTextures?: boolean;
};
/** @internal */
export type MaterialPluginHasTexture = {
    hasTexture: boolean;
    texture: BaseTexture;
};
/** @internal */
export type MaterialPluginIsReadyForSubMesh = {
    isReadyForSubMesh: boolean;
    defines: MaterialDefines;
    subMesh: SubMesh;
};
/** @internal */
export type MaterialPluginGetDefineNames = {
    defineNames?: {
        [name: string]: {
            type: string;
            default: any;
        };
    };
};
/** @internal */
export type MaterialPluginPrepareEffect = {
    defines: MaterialDefines;
    fallbacks: EffectFallbacks;
    fallbackRank: number;
    customCode?: ShaderCustomProcessingFunction;
    attributes: string[];
    uniforms: string[];
    samplers: string[];
    uniformBuffersNames: string[];
    mesh: AbstractMesh;
};
/** @internal */
export type MaterialPluginPrepareDefines = {
    defines: MaterialDefines;
    mesh: AbstractMesh;
};
/** @internal */
export type MaterialPluginPrepareUniformBuffer = {
    ubo: UniformBuffer;
};
/** @internal */
export type MaterialPluginBindForSubMesh = {
    subMesh: SubMesh;
};
/** @internal */
export type MaterialPluginGetAnimatables = {
    animatables: IAnimatable[];
};
/** @internal */
export type MaterialPluginGetActiveTextures = {
    activeTextures: BaseTexture[];
};
/** @internal */
export type MaterialPluginFillRenderTargetTextures = {
    renderTargets: SmartArray<RenderTargetTexture>;
};
/** @internal */
export type MaterialPluginHasRenderTargetTextures = {
    hasRenderTargetTextures: boolean;
};
/** @internal */
export type MaterialPluginHardBindForSubMesh = {
    subMesh: SubMesh;
};
/**
 * @internal
 */
export enum MaterialPluginEvent {
    Created = 1,
    Disposed = 2,
    GetDefineNames = 4,
    PrepareUniformBuffer = 8,
    IsReadyForSubMesh = 16,
    PrepareDefines = 32,
    BindForSubMesh = 64,
    PrepareEffect = 128,
    GetAnimatables = 256,
    GetActiveTextures = 512,
    HasTexture = 1024,
    FillRenderTargetTextures = 2048,
    HasRenderTargetTextures = 4096,
    HardBindForSubMesh = 8192
}
export {};

}
declare module "babylonjs/Materials/materialPluginFactoryExport" {
import { Nullable } from "babylonjs/types";
import { MaterialPluginBase } from "babylonjs/Materials/materialPluginBase";
import { Material } from "babylonjs/Materials/material";
/**
 * Creates an instance of the anisotropic plugin
 * @param material parent material the plugin will be created for
 * @returns the plugin instance or null if the plugin is incompatible with material
 */
export function createPBRAnisotropicPlugin(material: Material): Nullable<MaterialPluginBase>;
/**
 * Creates an instance of the brdf plugin
 * @param material parent material the plugin will be created for
 * @returns the plugin instance or null if the plugin is incompatible with material
 */
export function createPBRBRDFPlugin(material: Material): Nullable<MaterialPluginBase>;
/**
 * Creates an instance of the clear coat plugin
 * @param material parent material the plugin will be created for
 * @returns the plugin instance or null if the plugin is incompatible with material
 */
export function createPBRClearCoatPlugin(material: Material): Nullable<MaterialPluginBase>;
/**
 * Creates an instance of the iridescence plugin
 * @param material parent material the plugin will be created for
 * @returns the plugin instance or null if the plugin is incompatible with material
 */
export function createPBRIridescencePlugin(material: Material): Nullable<MaterialPluginBase>;
/**
 * Creates an instance of the sheen plugin
 * @param material parent material the plugin will be created for
 * @returns the plugin instance or null if the plugin is incompatible with material
 */
export function createPBRSheenPlugin(material: Material): Nullable<MaterialPluginBase>;
/**
 * Creates an instance of the sub surface plugin
 * @param material parent material the plugin will be created for
 * @returns the plugin instance or null if the plugin is incompatible with material
 */
export function createPBRSubSurfacePlugin(material: Material): Nullable<MaterialPluginBase>;
/**
 * Creates an instance of the detail map plugin
 * @param material parent material the plugin will be created for
 * @returns the plugin instance or null if the plugin is incompatible with material
 */
export function createDetailMapPlugin(material: Material): Nullable<MaterialPluginBase>;
export {};

}
declare module "babylonjs/Materials/materialPluginManager" {
import { ShaderCustomProcessingFunction } from "babylonjs/Engines/Processors/shaderProcessingOptions";
import { Nullable } from "babylonjs/types";
import { Material } from "babylonjs/Materials/material";
import { MaterialPluginPrepareEffect, MaterialPluginBindForSubMesh, MaterialPluginDisposed, MaterialPluginGetActiveTextures, MaterialPluginGetAnimatables, MaterialPluginGetDefineNames, MaterialPluginHasTexture, MaterialPluginIsReadyForSubMesh, MaterialPluginPrepareDefines, MaterialPluginPrepareUniformBuffer, MaterialPluginHardBindForSubMesh, MaterialPluginHasRenderTargetTextures, MaterialPluginFillRenderTargetTextures } from "babylonjs/Materials/materialPluginEvent";
import { Scene } from "babylonjs/scene";
import { Engine } from "babylonjs/Engines/engine";
import { MaterialPluginBase } from "babylonjs/Materials/materialPluginBase";
module "babylonjs/Materials/material" {
    interface Material {
        /**
         * Plugin manager for this material
         */
        pluginManager?: MaterialPluginManager;
    }
}
/**
 * Class that manages the plugins of a material
 * @since 5.0
 */
export class MaterialPluginManager {
    /** Map a plugin class name to a #define name (used in the vertex/fragment shaders as a marker of the plugin usage) */
    private static _MaterialPluginClassToMainDefine;
    private static _MaterialPluginCounter;
    protected _material: Material;
    protected _scene: Scene;
    protected _engine: Engine;
    protected _plugins: MaterialPluginBase[];
    protected _activePlugins: MaterialPluginBase[];
    protected _activePluginsForExtraEvents: MaterialPluginBase[];
    protected _codeInjectionPoints: {
        [shaderType: string]: {
            [codeName: string]: boolean;
        };
    };
    protected _defineNamesFromPlugins?: {
        [name: string]: {
            type: string;
            default: any;
        };
    };
    protected _uboDeclaration: string;
    protected _vertexDeclaration: string;
    protected _fragmentDeclaration: string;
    protected _uniformList: string[];
    protected _samplerList: string[];
    protected _uboList: string[];
    /**
     * Creates a new instance of the plugin manager
     * @param material material that this manager will manage the plugins for
     */
    constructor(material: Material);
    /**
     * @internal
     */
    _addPlugin(plugin: MaterialPluginBase): void;
    /**
     * @internal
     */
    _activatePlugin(plugin: MaterialPluginBase): void;
    /**
     * Gets a plugin from the list of plugins managed by this manager
     * @param name name of the plugin
     * @returns the plugin if found, else null
     */
    getPlugin(name: string): Nullable<MaterialPluginBase>;
    protected _handlePluginEventIsReadyForSubMesh(eventData: MaterialPluginIsReadyForSubMesh): void;
    protected _handlePluginEventPrepareDefinesBeforeAttributes(eventData: MaterialPluginPrepareDefines): void;
    protected _handlePluginEventPrepareDefines(eventData: MaterialPluginPrepareDefines): void;
    protected _handlePluginEventHardBindForSubMesh(eventData: MaterialPluginHardBindForSubMesh): void;
    protected _handlePluginEventBindForSubMesh(eventData: MaterialPluginBindForSubMesh): void;
    protected _handlePluginEventHasRenderTargetTextures(eventData: MaterialPluginHasRenderTargetTextures): void;
    protected _handlePluginEventFillRenderTargetTextures(eventData: MaterialPluginFillRenderTargetTextures): void;
    protected _handlePluginEvent(id: number, info: MaterialPluginGetActiveTextures | MaterialPluginGetAnimatables | MaterialPluginHasTexture | MaterialPluginDisposed | MaterialPluginGetDefineNames | MaterialPluginPrepareEffect | MaterialPluginPrepareUniformBuffer): void;
    protected _collectPointNames(shaderType: string, customCode: Nullable<{
        [pointName: string]: string;
    }> | undefined): void;
    protected _injectCustomCode(existingCallback?: (shaderType: string, code: string) => string): ShaderCustomProcessingFunction;
}
/**
 * Type for plugin material factories.
 */
export type PluginMaterialFactory = (material: Material) => Nullable<MaterialPluginBase>;
/**
 * Registers a new material plugin through a factory, or updates it. This makes the plugin available to all materials instantiated after its registration.
 * @param pluginName The plugin name
 * @param factory The factory function which allows to create the plugin
 */
export function RegisterMaterialPlugin(pluginName: string, factory: PluginMaterialFactory): void;
/**
 * Removes a material plugin from the list of global plugins.
 * @param pluginName The plugin name
 * @returns true if the plugin has been removed, else false
 */
export function UnregisterMaterialPlugin(pluginName: string): boolean;
/**
 * Clear the list of global material plugins
 */
export function UnregisterAllMaterialPlugins(): void;
export {};

}
declare module "babylonjs/Materials/materialStencilState" {
import { IStencilState } from "babylonjs/States/IStencilState";
import { Scene } from "babylonjs/scene";
/**
 * Class that holds the different stencil states of a material
 * Usage example: https://playground.babylonjs.com/#CW5PRI#10
 */
export class MaterialStencilState implements IStencilState {
    /**
     * Creates a material stencil state instance
     */
    constructor();
    /**
     * Resets all the stencil states to default values
     */
    reset(): void;
    private _func;
    /**
     * Gets or sets the stencil function
     */
    get func(): number;
    set func(value: number);
    private _funcRef;
    /**
     * Gets or sets the stencil function reference
     */
    get funcRef(): number;
    set funcRef(value: number);
    private _funcMask;
    /**
     * Gets or sets the stencil function mask
     */
    get funcMask(): number;
    set funcMask(value: number);
    private _opStencilFail;
    /**
     * Gets or sets the operation when the stencil test fails
     */
    get opStencilFail(): number;
    set opStencilFail(value: number);
    private _opDepthFail;
    /**
     * Gets or sets the operation when the depth test fails
     */
    get opDepthFail(): number;
    set opDepthFail(value: number);
    private _opStencilDepthPass;
    /**
     * Gets or sets the operation when the stencil+depth test succeeds
     */
    get opStencilDepthPass(): number;
    set opStencilDepthPass(value: number);
    private _mask;
    /**
     * Gets or sets the stencil mask
     */
    get mask(): number;
    set mask(value: number);
    private _enabled;
    /**
     * Enables or disables the stencil test
     */
    get enabled(): boolean;
    set enabled(value: boolean);
    /**
     * Get the current class name, useful for serialization or dynamic coding.
     * @returns "MaterialStencilState"
     */
    getClassName(): string;
    /**
     * Makes a duplicate of the current configuration into another one.
     * @param stencilState defines stencil state where to copy the info
     */
    copyTo(stencilState: MaterialStencilState): void;
    /**
     * Serializes this stencil configuration.
     * @returns - An object with the serialized config.
     */
    serialize(): any;
    /**
     * Parses a stencil state configuration from a serialized object.
     * @param source - Serialized object.
     * @param scene Defines the scene we are parsing for
     * @param rootUrl Defines the rootUrl to load from
     */
    parse(source: any, scene: Scene, rootUrl: string): void;
}
export {};

}
declare module "babylonjs/Materials/multiMaterial" {
import { Nullable } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { SubMesh } from "babylonjs/Meshes/subMesh";
import { BaseTexture } from "babylonjs/Materials/Textures/baseTexture";
import { Material } from "babylonjs/Materials/material";
/**
 * A multi-material is used to apply different materials to different parts of the same object without the need of
 * separate meshes. This can be use to improve performances.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/multiMaterials
 */
export class MultiMaterial extends Material {
    private _subMaterials;
    /** @internal */
    _waitingSubMaterialsUniqueIds: string[];
    /**
     * Gets or Sets the list of Materials used within the multi material.
     * They need to be ordered according to the submeshes order in the associated mesh
     */
    get subMaterials(): Nullable<Material>[];
    set subMaterials(value: Nullable<Material>[]);
    /**
     * Function used to align with Node.getChildren()
     * @returns the list of Materials used within the multi material
     */
    getChildren(): Nullable<Material>[];
    /**
     * Instantiates a new Multi Material
     * A multi-material is used to apply different materials to different parts of the same object without the need of
     * separate meshes. This can be use to improve performances.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/multiMaterials
     * @param name Define the name in the scene
     * @param scene Define the scene the material belongs to
     */
    constructor(name: string, scene?: Scene);
    private _hookArray;
    /**
     * Get one of the submaterial by its index in the submaterials array
     * @param index The index to look the sub material at
     * @returns The Material if the index has been defined
     */
    getSubMaterial(index: number): Nullable<Material>;
    /**
     * Get the list of active textures for the whole sub materials list.
     * @returns All the textures that will be used during the rendering
     */
    getActiveTextures(): BaseTexture[];
    /**
     * Specifies if any sub-materials of this multi-material use a given texture.
     * @param texture Defines the texture to check against this multi-material's sub-materials.
     * @returns A boolean specifying if any sub-material of this multi-material uses the texture.
     */
    hasTexture(texture: BaseTexture): boolean;
    /**
     * Gets the current class name of the material e.g. "MultiMaterial"
     * Mainly use in serialization.
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Checks if the material is ready to render the requested sub mesh
     * @param mesh Define the mesh the submesh belongs to
     * @param subMesh Define the sub mesh to look readiness for
     * @param useInstances Define whether or not the material is used with instances
     * @returns true if ready, otherwise false
     */
    isReadyForSubMesh(mesh: AbstractMesh, subMesh: SubMesh, useInstances?: boolean): boolean;
    /**
     * Clones the current material and its related sub materials
     * @param name Define the name of the newly cloned material
     * @param cloneChildren Define if submaterial will be cloned or shared with the parent instance
     * @returns the cloned material
     */
    clone(name: string, cloneChildren?: boolean): MultiMaterial;
    /**
     * Serializes the materials into a JSON representation.
     * @returns the JSON representation
     */
    serialize(): any;
    /**
     * Dispose the material and release its associated resources
     * @param forceDisposeEffect Define if we want to force disposing the associated effect (if false the shader is not released and could be reuse later on)
     * @param forceDisposeTextures Define if we want to force disposing the associated textures (if false, they will not be disposed and can still be use elsewhere in the app)
     * @param forceDisposeChildren Define if we want to force disposing the associated submaterials (if false, they will not be disposed and can still be use elsewhere in the app)
     */
    dispose(forceDisposeEffect?: boolean, forceDisposeTextures?: boolean, forceDisposeChildren?: boolean): void;
    /**
     * Creates a MultiMaterial from parsed MultiMaterial data.
     * @param parsedMultiMaterial defines parsed MultiMaterial data.
     * @param scene defines the hosting scene
     * @returns a new MultiMaterial
     */
    static ParseMultiMaterial(parsedMultiMaterial: any, scene: Scene): MultiMaterial;
}

}
declare module "babylonjs/Materials/Node/Blocks/addBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
/**
 * Block used to add 2 vectors
 */
export class AddBlock extends NodeMaterialBlock {
    /**
     * Creates a new AddBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the left operand input component
     */
    get left(): NodeMaterialConnectionPoint;
    /**
     * Gets the right operand input component
     */
    get right(): NodeMaterialConnectionPoint;
    /**
     * Gets the output component
     */
    get output(): NodeMaterialConnectionPoint;
    protected _buildBlock(state: NodeMaterialBuildState): this;
}

}
declare module "babylonjs/Materials/Node/Blocks/arcTan2Block" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
/**
 * Block used to compute arc tangent of 2 values
 */
export class ArcTan2Block extends NodeMaterialBlock {
    /**
     * Creates a new ArcTan2Block
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the x operand input component
     */
    get x(): NodeMaterialConnectionPoint;
    /**
     * Gets the y operand input component
     */
    get y(): NodeMaterialConnectionPoint;
    /**
     * Gets the output component
     */
    get output(): NodeMaterialConnectionPoint;
    protected _buildBlock(state: NodeMaterialBuildState): this;
}

}
declare module "babylonjs/Materials/Node/Blocks/biPlanarBlock" {
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { TriPlanarBlock } from "babylonjs/Materials/Node/Blocks/triPlanarBlock";
/**
 * Block used to read a texture with triplanar mapping (see https://iquilezles.org/articles/biplanar/)
 */
export class BiPlanarBlock extends TriPlanarBlock {
    /**
     * Create a new BiPlanarBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    protected _generateTextureLookup(state: NodeMaterialBuildState): void;
}

}
declare module "babylonjs/Materials/Node/Blocks/clampBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
import { Scene } from "babylonjs/scene";
/**
 * Block used to clamp a float
 */
export class ClampBlock extends NodeMaterialBlock {
    /** Gets or sets the minimum range */
    minimum: number;
    /** Gets or sets the maximum range */
    maximum: number;
    /**
     * Creates a new ClampBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the value input component
     */
    get value(): NodeMaterialConnectionPoint;
    /**
     * Gets the output component
     */
    get output(): NodeMaterialConnectionPoint;
    protected _buildBlock(state: NodeMaterialBuildState): this;
    protected _dumpPropertiesCode(): string;
    serialize(): any;
    _deserialize(serializationObject: any, scene: Scene, rootUrl: string): void;
}

}
declare module "babylonjs/Materials/Node/Blocks/cloudBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
import { Scene } from "babylonjs/scene";
/**
 * block used to Generate Fractal Brownian Motion Clouds
 */
export class CloudBlock extends NodeMaterialBlock {
    /** Gets or sets the number of octaves */
    octaves: number;
    /**
     * Creates a new CloudBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the seed input component
     */
    get seed(): NodeMaterialConnectionPoint;
    /**
     * Gets the chaos input component
     */
    get chaos(): NodeMaterialConnectionPoint;
    /**
     * Gets the offset X input component
     */
    get offsetX(): NodeMaterialConnectionPoint;
    /**
     * Gets the offset Y input component
     */
    get offsetY(): NodeMaterialConnectionPoint;
    /**
     * Gets the offset Z input component
     */
    get offsetZ(): NodeMaterialConnectionPoint;
    /**
     * Gets the output component
     */
    get output(): NodeMaterialConnectionPoint;
    protected _buildBlock(state: NodeMaterialBuildState): this | undefined;
    protected _dumpPropertiesCode(): string;
    serialize(): any;
    _deserialize(serializationObject: any, scene: Scene, rootUrl: string): void;
}

}
declare module "babylonjs/Materials/Node/Blocks/colorMergerBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
import { Scene } from "babylonjs/scene";
/**
 * Block used to create a Color3/4 out of individual inputs (one for each component)
 */
export class ColorMergerBlock extends NodeMaterialBlock {
    /**
     * Gets or sets the swizzle for r (meaning which component to affect to the output.r)
     */
    rSwizzle: "r" | "g" | "b" | "a";
    /**
     * Gets or sets the swizzle for g (meaning which component to affect to the output.g)
     */
    gSwizzle: "r" | "g" | "b" | "a";
    /**
     * Gets or sets the swizzle for b (meaning which component to affect to the output.b)
     */
    bSwizzle: "r" | "g" | "b" | "a";
    /**
     * Gets or sets the swizzle for a (meaning which component to affect to the output.a)
     */
    aSwizzle: "r" | "g" | "b" | "a";
    /**
     * Create a new ColorMergerBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the rgb component (input)
     */
    get rgbIn(): NodeMaterialConnectionPoint;
    /**
     * Gets the r component (input)
     */
    get r(): NodeMaterialConnectionPoint;
    /**
     * Gets the g component (input)
     */
    get g(): NodeMaterialConnectionPoint;
    /**
     * Gets the b component (input)
     */
    get b(): NodeMaterialConnectionPoint;
    /**
     * Gets the a component (input)
     */
    get a(): NodeMaterialConnectionPoint;
    /**
     * Gets the rgba component (output)
     */
    get rgba(): NodeMaterialConnectionPoint;
    /**
     * Gets the rgb component (output)
     */
    get rgbOut(): NodeMaterialConnectionPoint;
    /**
     * Gets the rgb component (output)
     * @deprecated Please use rgbOut instead.
     */
    get rgb(): NodeMaterialConnectionPoint;
    protected _inputRename(name: string): string;
    private _buildSwizzle;
    protected _buildBlock(state: NodeMaterialBuildState): this;
    serialize(): any;
    _deserialize(serializationObject: any, scene: Scene, rootUrl: string): void;
    protected _dumpPropertiesCode(): string;
}

}
declare module "babylonjs/Materials/Node/Blocks/colorSplitterBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
/**
 * Block used to expand a Color3/4 into 4 outputs (one for each component)
 */
export class ColorSplitterBlock extends NodeMaterialBlock {
    /**
     * Create a new ColorSplitterBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the rgba component (input)
     */
    get rgba(): NodeMaterialConnectionPoint;
    /**
     * Gets the rgb component (input)
     */
    get rgbIn(): NodeMaterialConnectionPoint;
    /**
     * Gets the rgb component (output)
     */
    get rgbOut(): NodeMaterialConnectionPoint;
    /**
     * Gets the r component (output)
     */
    get r(): NodeMaterialConnectionPoint;
    /**
     * Gets the g component (output)
     */
    get g(): NodeMaterialConnectionPoint;
    /**
     * Gets the b component (output)
     */
    get b(): NodeMaterialConnectionPoint;
    /**
     * Gets the a component (output)
     */
    get a(): NodeMaterialConnectionPoint;
    protected _inputRename(name: string): string;
    protected _outputRename(name: string): string;
    protected _buildBlock(state: NodeMaterialBuildState): this | undefined;
}

}
declare module "babylonjs/Materials/Node/Blocks/conditionalBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
import { Scene } from "babylonjs/scene";
/**
 * Operations supported by the ConditionalBlock block
 */
export enum ConditionalBlockConditions {
    /** Equal */
    Equal = 0,
    /** NotEqual */
    NotEqual = 1,
    /** LessThan */
    LessThan = 2,
    /** GreaterThan */
    GreaterThan = 3,
    /** LessOrEqual */
    LessOrEqual = 4,
    /** GreaterOrEqual */
    GreaterOrEqual = 5,
    /** Logical Exclusive OR */
    Xor = 6,
    /** Logical Or */
    Or = 7,
    /** Logical And */
    And = 8
}
/**
 * Block used to apply conditional operation between floats
 * @since 5.0.0
 */
export class ConditionalBlock extends NodeMaterialBlock {
    /**
     * Gets or sets the condition applied by the block
     */
    condition: ConditionalBlockConditions;
    /**
     * Creates a new ConditionalBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the first operand component
     */
    get a(): NodeMaterialConnectionPoint;
    /**
     * Gets the second operand component
     */
    get b(): NodeMaterialConnectionPoint;
    /**
     * Gets the value to return if condition is true
     */
    get true(): NodeMaterialConnectionPoint;
    /**
     * Gets the value to return if condition is false
     */
    get false(): NodeMaterialConnectionPoint;
    /**
     * Gets the output component
     */
    get output(): NodeMaterialConnectionPoint;
    protected _buildBlock(state: NodeMaterialBuildState): this;
    serialize(): any;
    _deserialize(serializationObject: any, scene: Scene, rootUrl: string): void;
    protected _dumpPropertiesCode(): string;
}

}
declare module "babylonjs/Materials/Node/Blocks/crossBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
/**
 * Block used to apply a cross product between 2 vectors
 */
export class CrossBlock extends NodeMaterialBlock {
    /**
     * Creates a new CrossBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the left operand input component
     */
    get left(): NodeMaterialConnectionPoint;
    /**
     * Gets the right operand input component
     */
    get right(): NodeMaterialConnectionPoint;
    /**
     * Gets the output component
     */
    get output(): NodeMaterialConnectionPoint;
    protected _buildBlock(state: NodeMaterialBuildState): this;
}

}
declare module "babylonjs/Materials/Node/Blocks/customBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { Scene } from "babylonjs/scene";
/**
 * Custom block created from user-defined json
 */
export class CustomBlock extends NodeMaterialBlock {
    private _options;
    private _code;
    /**
     * Gets or sets the options for this custom block
     */
    get options(): any;
    set options(options: any);
    /**
     * Creates a new CustomBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    protected _buildBlock(state: NodeMaterialBuildState): this;
    protected _dumpPropertiesCode(): string;
    serialize(): any;
    _deserialize(serializationObject: any, scene: Scene, rootUrl: string): void;
    private _deserializeOptions;
    private _findInputByName;
}

}
declare module "babylonjs/Materials/Node/Blocks/desaturateBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
/**
 * Block used to desaturate a color
 */
export class DesaturateBlock extends NodeMaterialBlock {
    /**
     * Creates a new DesaturateBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the color operand input component
     */
    get color(): NodeMaterialConnectionPoint;
    /**
     * Gets the level operand input component
     */
    get level(): NodeMaterialConnectionPoint;
    /**
     * Gets the output component
     */
    get output(): NodeMaterialConnectionPoint;
    protected _buildBlock(state: NodeMaterialBuildState): this;
}

}
declare module "babylonjs/Materials/Node/Blocks/distanceBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
/**
 * Block used to get the distance between 2 values
 */
export class DistanceBlock extends NodeMaterialBlock {
    /**
     * Creates a new DistanceBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the left operand input component
     */
    get left(): NodeMaterialConnectionPoint;
    /**
     * Gets the right operand input component
     */
    get right(): NodeMaterialConnectionPoint;
    /**
     * Gets the output component
     */
    get output(): NodeMaterialConnectionPoint;
    protected _buildBlock(state: NodeMaterialBuildState): this;
}

}
declare module "babylonjs/Materials/Node/Blocks/divideBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
/**
 * Block used to divide 2 vectors
 */
export class DivideBlock extends NodeMaterialBlock {
    /**
     * Creates a new DivideBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the left operand input component
     */
    get left(): NodeMaterialConnectionPoint;
    /**
     * Gets the right operand input component
     */
    get right(): NodeMaterialConnectionPoint;
    /**
     * Gets the output component
     */
    get output(): NodeMaterialConnectionPoint;
    protected _buildBlock(state: NodeMaterialBuildState): this;
}

}
declare module "babylonjs/Materials/Node/Blocks/dotBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
/**
 * Block used to apply a dot product between 2 vectors
 */
export class DotBlock extends NodeMaterialBlock {
    /**
     * Creates a new DotBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the left operand input component
     */
    get left(): NodeMaterialConnectionPoint;
    /**
     * Gets the right operand input component
     */
    get right(): NodeMaterialConnectionPoint;
    /**
     * Gets the output component
     */
    get output(): NodeMaterialConnectionPoint;
    protected _buildBlock(state: NodeMaterialBuildState): this;
}

}
declare module "babylonjs/Materials/Node/Blocks/Dual/clipPlanesBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialBlockTargets } from "babylonjs/Materials/Node/Enums/nodeMaterialBlockTargets";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
import { Effect } from "babylonjs/Materials/effect";
import { NodeMaterial, NodeMaterialDefines } from "babylonjs/Materials/Node/nodeMaterial";
import { Mesh } from "babylonjs/Meshes/mesh";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
/**
 * Block used to implement clip planes
 */
export class ClipPlanesBlock extends NodeMaterialBlock {
    /**
     * Create a new ClipPlanesBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Initialize the block and prepare the context for build
     * @param state defines the state that will be used for the build
     */
    initialize(state: NodeMaterialBuildState): void;
    /**
     * Gets the worldPosition input component
     */
    get worldPosition(): NodeMaterialConnectionPoint;
    get target(): NodeMaterialBlockTargets;
    set target(value: NodeMaterialBlockTargets);
    prepareDefines(mesh: AbstractMesh, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines): void;
    bind(effect: Effect, nodeMaterial: NodeMaterial, mesh?: Mesh): void;
    protected _buildBlock(state: NodeMaterialBuildState): this | undefined;
}

}
declare module "babylonjs/Materials/Node/Blocks/Dual/currentScreenBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialBlockTargets } from "babylonjs/Materials/Node/Enums/nodeMaterialBlockTargets";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { NodeMaterialDefines } from "babylonjs/Materials/Node/nodeMaterial";
import { BaseTexture } from "babylonjs/Materials/Textures/baseTexture";
import { Nullable } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
import { NodeMaterial } from "babylonjs/Materials/Node/nodeMaterial";
/**
 * Base block used as input for post process
 */
export class CurrentScreenBlock extends NodeMaterialBlock {
    private _samplerName;
    private _linearDefineName;
    private _gammaDefineName;
    private _mainUVName;
    private _tempTextureRead;
    /**
     * Gets or sets the texture associated with the node
     */
    texture: Nullable<BaseTexture>;
    /**
     * Gets or sets a boolean indicating if content needs to be converted to gamma space
     */
    convertToGammaSpace: boolean;
    /**
     * Gets or sets a boolean indicating if content needs to be converted to linear space
     */
    convertToLinearSpace: boolean;
    /**
     * Create a new CurrentScreenBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the uv input component
     */
    get uv(): NodeMaterialConnectionPoint;
    /**
     * Gets the rgba output component
     */
    get rgba(): NodeMaterialConnectionPoint;
    /**
     * Gets the rgb output component
     */
    get rgb(): NodeMaterialConnectionPoint;
    /**
     * Gets the r output component
     */
    get r(): NodeMaterialConnectionPoint;
    /**
     * Gets the g output component
     */
    get g(): NodeMaterialConnectionPoint;
    /**
     * Gets the b output component
     */
    get b(): NodeMaterialConnectionPoint;
    /**
     * Gets the a output component
     */
    get a(): NodeMaterialConnectionPoint;
    /**
     * Initialize the block and prepare the context for build
     * @param state defines the state that will be used for the build
     */
    initialize(state: NodeMaterialBuildState): void;
    get target(): NodeMaterialBlockTargets;
    prepareDefines(mesh: AbstractMesh, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines): void;
    isReady(): boolean;
    private _injectVertexCode;
    private _writeTextureRead;
    private _writeOutput;
    protected _buildBlock(state: NodeMaterialBuildState): this | undefined;
    serialize(): any;
    _deserialize(serializationObject: any, scene: Scene, rootUrl: string): void;
}
export {};

}
declare module "babylonjs/Materials/Node/Blocks/Dual/fogBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { Mesh } from "babylonjs/Meshes/mesh";
import { Effect } from "babylonjs/Materials/effect";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { NodeMaterial, NodeMaterialDefines } from "babylonjs/Materials/Node/nodeMaterial";
import "babylonjs/Shaders/ShadersInclude/fogFragmentDeclaration";
/**
 * Block used to add support for scene fog
 */
export class FogBlock extends NodeMaterialBlock {
    private _fogDistanceName;
    private _fogParameters;
    /**
     * Create a new FogBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the world position input component
     */
    get worldPosition(): NodeMaterialConnectionPoint;
    /**
     * Gets the view input component
     */
    get view(): NodeMaterialConnectionPoint;
    /**
     * Gets the color input component
     */
    get input(): NodeMaterialConnectionPoint;
    /**
     * Gets the fog color input component
     */
    get fogColor(): NodeMaterialConnectionPoint;
    /**
     * Gets the output component
     */
    get output(): NodeMaterialConnectionPoint;
    autoConfigure(material: NodeMaterial): void;
    prepareDefines(mesh: AbstractMesh, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines): void;
    bind(effect: Effect, nodeMaterial: NodeMaterial, mesh?: Mesh): void;
    protected _buildBlock(state: NodeMaterialBuildState): this;
}

}
declare module "babylonjs/Materials/Node/Blocks/Dual/imageSourceBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
import { Nullable } from "babylonjs/types";
import { Texture } from "babylonjs/Materials/Textures/texture";
import { Effect } from "babylonjs/Materials/effect";
import { Scene } from "babylonjs/scene";
/**
 * Block used to provide an image for a TextureBlock
 */
export class ImageSourceBlock extends NodeMaterialBlock {
    private _samplerName;
    protected _texture: Nullable<Texture>;
    /**
     * Gets or sets the texture associated with the node
     */
    get texture(): Nullable<Texture>;
    set texture(texture: Nullable<Texture>);
    /**
     * Gets the sampler name associated with this image source
     */
    get samplerName(): string;
    /**
     * Creates a new ImageSourceBlock
     * @param name defines the block name
     */
    constructor(name: string);
    bind(effect: Effect): void;
    isReady(): boolean;
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the output component
     */
    get source(): NodeMaterialConnectionPoint;
    protected _buildBlock(state: NodeMaterialBuildState): this;
    protected _dumpPropertiesCode(): string;
    serialize(): any;
    _deserialize(serializationObject: any, scene: Scene, rootUrl: string): void;
}

}
declare module "babylonjs/Materials/Node/Blocks/Dual/index" {
export * from "babylonjs/Materials/Node/Blocks/Dual/fogBlock";
export * from "babylonjs/Materials/Node/Blocks/Dual/lightBlock";
export * from "babylonjs/Materials/Node/Blocks/Dual/textureBlock";
export * from "babylonjs/Materials/Node/Blocks/Dual/reflectionTextureBlock";
export * from "babylonjs/Materials/Node/Blocks/Dual/currentScreenBlock";
export * from "babylonjs/Materials/Node/Blocks/Dual/sceneDepthBlock";
export * from "babylonjs/Materials/Node/Blocks/Dual/imageSourceBlock";
export * from "babylonjs/Materials/Node/Blocks/Dual/clipPlanesBlock";

}
declare module "babylonjs/Materials/Node/Blocks/Dual/lightBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { NodeMaterial, NodeMaterialDefines } from "babylonjs/Materials/Node/nodeMaterial";
import { Effect } from "babylonjs/Materials/effect";
import { Mesh } from "babylonjs/Meshes/mesh";
import { Light } from "babylonjs/Lights/light";
import { Nullable } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
import "babylonjs/Shaders/ShadersInclude/lightFragmentDeclaration";
import "babylonjs/Shaders/ShadersInclude/lightVxFragmentDeclaration";
import "babylonjs/Shaders/ShadersInclude/lightUboDeclaration";
import "babylonjs/Shaders/ShadersInclude/lightVxUboDeclaration";
import "babylonjs/Shaders/ShadersInclude/lightFragment";
import "babylonjs/Shaders/ShadersInclude/helperFunctions";
import "babylonjs/Shaders/ShadersInclude/lightsFragmentFunctions";
import "babylonjs/Shaders/ShadersInclude/shadowsFragmentFunctions";
import "babylonjs/Shaders/ShadersInclude/shadowsVertex";
/**
 * Block used to add light in the fragment shader
 */
export class LightBlock extends NodeMaterialBlock {
    private _lightId;
    /**
     * Gets or sets the light associated with this block
     */
    light: Nullable<Light>;
    /** Indicates that no code should be generated in the vertex shader. Can be useful in some specific circumstances (like when doing ray marching for eg) */
    generateOnlyFragmentCode: boolean;
    private static _OnGenerateOnlyFragmentCodeChanged;
    private _setTarget;
    /**
     * Create a new LightBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the world position input component
     */
    get worldPosition(): NodeMaterialConnectionPoint;
    /**
     * Gets the world normal input component
     */
    get worldNormal(): NodeMaterialConnectionPoint;
    /**
     * Gets the camera (or eye) position component
     */
    get cameraPosition(): NodeMaterialConnectionPoint;
    /**
     * Gets the glossiness component
     */
    get glossiness(): NodeMaterialConnectionPoint;
    /**
     * Gets the glossiness power component
     */
    get glossPower(): NodeMaterialConnectionPoint;
    /**
     * Gets the diffuse color component
     */
    get diffuseColor(): NodeMaterialConnectionPoint;
    /**
     * Gets the specular color component
     */
    get specularColor(): NodeMaterialConnectionPoint;
    /**
     * Gets the view matrix component
     */
    get view(): NodeMaterialConnectionPoint;
    /**
     * Gets the diffuse output component
     */
    get diffuseOutput(): NodeMaterialConnectionPoint;
    /**
     * Gets the specular output component
     */
    get specularOutput(): NodeMaterialConnectionPoint;
    /**
     * Gets the shadow output component
     */
    get shadow(): NodeMaterialConnectionPoint;
    autoConfigure(material: NodeMaterial): void;
    prepareDefines(mesh: AbstractMesh, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines): void;
    updateUniformsAndSamples(state: NodeMaterialBuildState, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines, uniformBuffers: string[]): void;
    bind(effect: Effect, nodeMaterial: NodeMaterial, mesh?: Mesh): void;
    private _injectVertexCode;
    protected _buildBlock(state: NodeMaterialBuildState): this | undefined;
    serialize(): any;
    _deserialize(serializationObject: any, scene: Scene, rootUrl: string): void;
}

}
declare module "babylonjs/Materials/Node/Blocks/Dual/reflectionTextureBaseBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
import { BaseTexture } from "babylonjs/Materials/Textures/baseTexture";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { NodeMaterialDefines } from "babylonjs/Materials/Node/nodeMaterial";
import { NodeMaterial } from "babylonjs/Materials/Node/nodeMaterial";
import { Effect } from "babylonjs/Materials/effect";
import { Mesh } from "babylonjs/Meshes/mesh";
import { Nullable } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
import "babylonjs/Shaders/ShadersInclude/reflectionFunction";
/**
 * Base block used to read a reflection texture from a sampler
 */
export abstract class ReflectionTextureBaseBlock extends NodeMaterialBlock {
    /** @internal */
    _define3DName: string;
    /** @internal */
    _defineCubicName: string;
    /** @internal */
    _defineExplicitName: string;
    /** @internal */
    _defineProjectionName: string;
    /** @internal */
    _defineLocalCubicName: string;
    /** @internal */
    _defineSphericalName: string;
    /** @internal */
    _definePlanarName: string;
    /** @internal */
    _defineEquirectangularName: string;
    /** @internal */
    _defineMirroredEquirectangularFixedName: string;
    /** @internal */
    _defineEquirectangularFixedName: string;
    /** @internal */
    _defineSkyboxName: string;
    /** @internal */
    _defineOppositeZ: string;
    /** @internal */
    _cubeSamplerName: string;
    /** @internal */
    _2DSamplerName: string;
    /** @internal */
    _reflectionPositionName: string;
    /** @internal */
    _reflectionSizeName: string;
    protected _positionUVWName: string;
    protected _directionWName: string;
    protected _reflectionVectorName: string;
    /** @internal */
    _reflectionCoordsName: string;
    /** @internal */
    _reflectionMatrixName: string;
    protected _reflectionColorName: string;
    protected _worldPositionNameInFragmentOnlyMode: string;
    protected _texture: Nullable<BaseTexture>;
    /**
     * Gets or sets the texture associated with the node
     */
    get texture(): Nullable<BaseTexture>;
    set texture(texture: Nullable<BaseTexture>);
    /** Indicates that no code should be generated in the vertex shader. Can be useful in some specific circumstances (like when doing ray marching for eg) */
    generateOnlyFragmentCode: boolean;
    protected static _OnGenerateOnlyFragmentCodeChanged(block: NodeMaterialBlock, _propertyName: string): boolean;
    protected _onGenerateOnlyFragmentCodeChanged(): boolean;
    protected _setTarget(): void;
    /**
     * Create a new ReflectionTextureBaseBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the world position input component
     */
    abstract get position(): NodeMaterialConnectionPoint;
    /**
     * Gets the world position input component
     */
    abstract get worldPosition(): NodeMaterialConnectionPoint;
    /**
     * Gets the world normal input component
     */
    abstract get worldNormal(): NodeMaterialConnectionPoint;
    /**
     * Gets the world input component
     */
    abstract get world(): NodeMaterialConnectionPoint;
    /**
     * Gets the camera (or eye) position component
     */
    abstract get cameraPosition(): NodeMaterialConnectionPoint;
    /**
     * Gets the view input component
     */
    abstract get view(): NodeMaterialConnectionPoint;
    protected _getTexture(): Nullable<BaseTexture>;
    autoConfigure(material: NodeMaterial): void;
    prepareDefines(mesh: AbstractMesh, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines): void;
    isReady(): boolean;
    bind(effect: Effect, nodeMaterial: NodeMaterial, mesh?: Mesh): void;
    /**
     * Gets the code to inject in the vertex shader
     * @param state current state of the node material building
     * @returns the shader code
     */
    handleVertexSide(state: NodeMaterialBuildState): string;
    /**
     * Handles the inits for the fragment code path
     * @param state node material build state
     */
    handleFragmentSideInits(state: NodeMaterialBuildState): void;
    /**
     * Generates the reflection coords code for the fragment code path
     * @param worldNormalVarName name of the world normal variable
     * @param worldPos name of the world position variable. If not provided, will use the world position connected to this block
     * @param onlyReflectionVector if true, generates code only for the reflection vector computation, not for the reflection coordinates
     * @param doNotEmitInvertZ if true, does not emit the invertZ code
     * @returns the shader code
     */
    handleFragmentSideCodeReflectionCoords(worldNormalVarName: string, worldPos?: string, onlyReflectionVector?: boolean, doNotEmitInvertZ?: boolean): string;
    /**
     * Generates the reflection color code for the fragment code path
     * @param lodVarName name of the lod variable
     * @param swizzleLookupTexture swizzle to use for the final color variable
     * @returns the shader code
     */
    handleFragmentSideCodeReflectionColor(lodVarName?: string, swizzleLookupTexture?: string): string;
    /**
     * Generates the code corresponding to the connected output points
     * @param state node material build state
     * @param varName name of the variable to output
     * @returns the shader code
     */
    writeOutputs(state: NodeMaterialBuildState, varName: string): string;
    protected _buildBlock(state: NodeMaterialBuildState): this;
    protected _dumpPropertiesCode(): string;
    serialize(): any;
    _deserialize(serializationObject: any, scene: Scene, rootUrl: string): void;
}

}
declare module "babylonjs/Materials/Node/Blocks/Dual/reflectionTextureBlock" {
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
import { NodeMaterial } from "babylonjs/Materials/Node/nodeMaterial";
import { ReflectionTextureBaseBlock } from "babylonjs/Materials/Node/Blocks/Dual/reflectionTextureBaseBlock";
/**
 * Block used to read a reflection texture from a sampler
 */
export class ReflectionTextureBlock extends ReflectionTextureBaseBlock {
    protected _onGenerateOnlyFragmentCodeChanged(): boolean;
    protected _setTarget(): void;
    /**
     * Create a new ReflectionTextureBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the world position input component
     */
    get position(): NodeMaterialConnectionPoint;
    /**
     * Gets the world position input component
     */
    get worldPosition(): NodeMaterialConnectionPoint;
    /**
     * Gets the world normal input component
     */
    get worldNormal(): NodeMaterialConnectionPoint;
    /**
     * Gets the world input component
     */
    get world(): NodeMaterialConnectionPoint;
    /**
     * Gets the camera (or eye) position component
     */
    get cameraPosition(): NodeMaterialConnectionPoint;
    /**
     * Gets the view input component
     */
    get view(): NodeMaterialConnectionPoint;
    /**
     * Gets the rgb output component
     */
    get rgb(): NodeMaterialConnectionPoint;
    /**
     * Gets the rgba output component
     */
    get rgba(): NodeMaterialConnectionPoint;
    /**
     * Gets the r output component
     */
    get r(): NodeMaterialConnectionPoint;
    /**
     * Gets the g output component
     */
    get g(): NodeMaterialConnectionPoint;
    /**
     * Gets the b output component
     */
    get b(): NodeMaterialConnectionPoint;
    /**
     * Gets the a output component
     */
    get a(): NodeMaterialConnectionPoint;
    autoConfigure(material: NodeMaterial): void;
    protected _buildBlock(state: NodeMaterialBuildState): this;
}

}
declare module "babylonjs/Materials/Node/Blocks/Dual/sceneDepthBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialBlockTargets } from "babylonjs/Materials/Node/Enums/nodeMaterialBlockTargets";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
import { Scene } from "babylonjs/scene";
import { Effect } from "babylonjs/Materials/effect";
import { NodeMaterial } from "babylonjs/Materials/Node/nodeMaterial";
/**
 * Block used to retrieve the depth (zbuffer) of the scene
 * @since 5.0.0
 */
export class SceneDepthBlock extends NodeMaterialBlock {
    private _samplerName;
    private _mainUVName;
    private _tempTextureRead;
    /**
     * Defines if the depth renderer should be setup in non linear mode
     */
    useNonLinearDepth: boolean;
    /**
     * Defines if the depth renderer should be setup in full 32 bits float mode
     */
    force32itsFloat: boolean;
    /**
     * Create a new SceneDepthBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the uv input component
     */
    get uv(): NodeMaterialConnectionPoint;
    /**
     * Gets the depth output component
     */
    get depth(): NodeMaterialConnectionPoint;
    /**
     * Initialize the block and prepare the context for build
     * @param state defines the state that will be used for the build
     */
    initialize(state: NodeMaterialBuildState): void;
    get target(): NodeMaterialBlockTargets;
    private _getTexture;
    bind(effect: Effect, nodeMaterial: NodeMaterial): void;
    private _injectVertexCode;
    private _writeTextureRead;
    private _writeOutput;
    protected _buildBlock(state: NodeMaterialBuildState): this | undefined;
    serialize(): any;
    _deserialize(serializationObject: any, scene: Scene, rootUrl: string): void;
}
export {};

}
declare module "babylonjs/Materials/Node/Blocks/Dual/textureBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialBlockTargets } from "babylonjs/Materials/Node/Enums/nodeMaterialBlockTargets";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { NodeMaterialDefines } from "babylonjs/Materials/Node/nodeMaterial";
import { NodeMaterial } from "babylonjs/Materials/Node/nodeMaterial";
import { Effect } from "babylonjs/Materials/effect";
import { Nullable } from "babylonjs/types";
import { Texture } from "babylonjs/Materials/Textures/texture";
import { Scene } from "babylonjs/scene";
import "babylonjs/Shaders/ShadersInclude/helperFunctions";
/**
 * Block used to read a texture from a sampler
 */
export class TextureBlock extends NodeMaterialBlock {
    private _defineName;
    private _linearDefineName;
    private _gammaDefineName;
    private _tempTextureRead;
    private _samplerName;
    private _transformedUVName;
    private _textureTransformName;
    private _textureInfoName;
    private _mainUVName;
    private _mainUVDefineName;
    private _fragmentOnly;
    private _imageSource;
    protected _texture: Nullable<Texture>;
    /**
     * Gets or sets the texture associated with the node
     */
    get texture(): Nullable<Texture>;
    set texture(texture: Nullable<Texture>);
    /**
     * Gets the sampler name associated with this texture
     */
    get samplerName(): string;
    /**
     * Gets a boolean indicating that this block is linked to an ImageSourceBlock
     */
    get hasImageSource(): boolean;
    private _convertToGammaSpace;
    /**
     * Gets or sets a boolean indicating if content needs to be converted to gamma space
     */
    set convertToGammaSpace(value: boolean);
    get convertToGammaSpace(): boolean;
    private _convertToLinearSpace;
    /**
     * Gets or sets a boolean indicating if content needs to be converted to linear space
     */
    set convertToLinearSpace(value: boolean);
    get convertToLinearSpace(): boolean;
    /**
     * Gets or sets a boolean indicating if multiplication of texture with level should be disabled
     */
    disableLevelMultiplication: boolean;
    /**
     * Create a new TextureBlock
     * @param name defines the block name
     * @param fragmentOnly
     */
    constructor(name: string, fragmentOnly?: boolean);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the uv input component
     */
    get uv(): NodeMaterialConnectionPoint;
    /**
     * Gets the source input component
     */
    get source(): NodeMaterialConnectionPoint;
    /**
     * Gets the rgba output component
     */
    get rgba(): NodeMaterialConnectionPoint;
    /**
     * Gets the rgb output component
     */
    get rgb(): NodeMaterialConnectionPoint;
    /**
     * Gets the r output component
     */
    get r(): NodeMaterialConnectionPoint;
    /**
     * Gets the g output component
     */
    get g(): NodeMaterialConnectionPoint;
    /**
     * Gets the b output component
     */
    get b(): NodeMaterialConnectionPoint;
    /**
     * Gets the a output component
     */
    get a(): NodeMaterialConnectionPoint;
    /**
     * Gets the level output component
     */
    get level(): NodeMaterialConnectionPoint;
    get target(): NodeMaterialBlockTargets;
    set target(value: NodeMaterialBlockTargets);
    autoConfigure(material: NodeMaterial): void;
    initializeDefines(mesh: AbstractMesh, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines): void;
    prepareDefines(mesh: AbstractMesh, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines): void;
    isReady(): boolean;
    bind(effect: Effect): void;
    private get _isMixed();
    private _injectVertexCode;
    private _generateTextureLookup;
    private _writeTextureRead;
    private _generateConversionCode;
    private _writeOutput;
    protected _buildBlock(state: NodeMaterialBuildState): this | undefined;
    protected _dumpPropertiesCode(): string;
    serialize(): any;
    _deserialize(serializationObject: any, scene: Scene, rootUrl: string): void;
}

}
declare module "babylonjs/Materials/Node/Blocks/elbowBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
import { NodeMaterialBlockTargets } from "babylonjs/Materials/Node/Enums/nodeMaterialBlockTargets";
/**
 * Block used as a pass through
 */
export class ElbowBlock extends NodeMaterialBlock {
    /**
     * Creates a new ElbowBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the input component
     */
    get input(): NodeMaterialConnectionPoint;
    /**
     * Gets the output component
     */
    get output(): NodeMaterialConnectionPoint;
    /**
     * Gets or sets the target of the block
     */
    get target(): NodeMaterialBlockTargets;
    set target(value: NodeMaterialBlockTargets);
    protected _buildBlock(state: NodeMaterialBuildState): this;
}

}
declare module "babylonjs/Materials/Node/Blocks/Fragment/derivativeBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
/**
 * Block used to get the derivative value on x and y of a given input
 */
export class DerivativeBlock extends NodeMaterialBlock {
    /**
     * Create a new DerivativeBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the input component
     */
    get input(): NodeMaterialConnectionPoint;
    /**
     * Gets the derivative output on x
     */
    get dx(): NodeMaterialConnectionPoint;
    /**
     * Gets the derivative output on y
     */
    get dy(): NodeMaterialConnectionPoint;
    protected _buildBlock(state: NodeMaterialBuildState): this;
}

}
declare module "babylonjs/Materials/Node/Blocks/Fragment/discardBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
/**
 * Block used to discard a pixel if a value is smaller than a cutoff
 */
export class DiscardBlock extends NodeMaterialBlock {
    /**
     * Create a new DiscardBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the color input component
     */
    get value(): NodeMaterialConnectionPoint;
    /**
     * Gets the cutoff input component
     */
    get cutoff(): NodeMaterialConnectionPoint;
    protected _buildBlock(state: NodeMaterialBuildState): this | undefined;
}

}
declare module "babylonjs/Materials/Node/Blocks/Fragment/fragCoordBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
/**
 * Block used to make gl_FragCoord available
 */
export class FragCoordBlock extends NodeMaterialBlock {
    /**
     * Creates a new FragCoordBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the xy component
     */
    get xy(): NodeMaterialConnectionPoint;
    /**
     * Gets the xyz component
     */
    get xyz(): NodeMaterialConnectionPoint;
    /**
     * Gets the xyzw component
     */
    get xyzw(): NodeMaterialConnectionPoint;
    /**
     * Gets the x component
     */
    get x(): NodeMaterialConnectionPoint;
    /**
     * Gets the y component
     */
    get y(): NodeMaterialConnectionPoint;
    /**
     * Gets the z component
     */
    get z(): NodeMaterialConnectionPoint;
    /**
     * Gets the w component
     */
    get output(): NodeMaterialConnectionPoint;
    protected writeOutputs(state: NodeMaterialBuildState): string;
    protected _buildBlock(state: NodeMaterialBuildState): this;
}

}
declare module "babylonjs/Materials/Node/Blocks/Fragment/fragDepthBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
/**
 * Block used to write the fragment depth
 */
export class FragDepthBlock extends NodeMaterialBlock {
    /**
     * Create a new FragDepthBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the depth input component
     */
    get depth(): NodeMaterialConnectionPoint;
    /**
     * Gets the worldPos input component
     */
    get worldPos(): NodeMaterialConnectionPoint;
    /**
     * Gets the viewProjection input component
     */
    get viewProjection(): NodeMaterialConnectionPoint;
    protected _buildBlock(state: NodeMaterialBuildState): this;
}

}
declare module "babylonjs/Materials/Node/Blocks/Fragment/fragmentOutputBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
import { Scene } from "babylonjs/scene";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { NodeMaterialDefines } from "babylonjs/Materials/Node/nodeMaterial";
import { NodeMaterial } from "babylonjs/Materials/Node/nodeMaterial";
import { Effect } from "babylonjs/Materials/effect";
import { Mesh } from "babylonjs/Meshes/mesh";
/**
 * Block used to output the final color
 */
export class FragmentOutputBlock extends NodeMaterialBlock {
    private _linearDefineName;
    private _gammaDefineName;
    /**
     * Create a new FragmentOutputBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /** Gets or sets a boolean indicating if content needs to be converted to gamma space */
    convertToGammaSpace: boolean;
    /** Gets or sets a boolean indicating if content needs to be converted to linear space */
    convertToLinearSpace: boolean;
    /** Gets or sets a boolean indicating if logarithmic depth should be used */
    useLogarithmicDepth: boolean;
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Initialize the block and prepare the context for build
     * @param state defines the state that will be used for the build
     */
    initialize(state: NodeMaterialBuildState): void;
    /**
     * Gets the rgba input component
     */
    get rgba(): NodeMaterialConnectionPoint;
    /**
     * Gets the rgb input component
     */
    get rgb(): NodeMaterialConnectionPoint;
    /**
     * Gets the a input component
     */
    get a(): NodeMaterialConnectionPoint;
    prepareDefines(mesh: AbstractMesh, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines): void;
    bind(effect: Effect, nodeMaterial: NodeMaterial, mesh?: Mesh): void;
    protected _buildBlock(state: NodeMaterialBuildState): this;
    protected _dumpPropertiesCode(): string;
    serialize(): any;
    _deserialize(serializationObject: any, scene: Scene, rootUrl: string): void;
}
export {};

}
declare module "babylonjs/Materials/Node/Blocks/Fragment/frontFacingBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
/**
 * Block used to test if the fragment shader is front facing
 */
export class FrontFacingBlock extends NodeMaterialBlock {
    /**
     * Creates a new FrontFacingBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the output component
     */
    get output(): NodeMaterialConnectionPoint;
    protected _buildBlock(state: NodeMaterialBuildState): this;
}

}
declare module "babylonjs/Materials/Node/Blocks/Fragment/heightToNormalBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
import { Scene } from "babylonjs/scene";
/**
 * Block used to convert a height vector to a normal
 */
export class HeightToNormalBlock extends NodeMaterialBlock {
    /**
     * Creates a new HeightToNormalBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Defines if the output should be generated in world or tangent space.
     * Note that in tangent space the result is also scaled by 0.5 and offsetted by 0.5 so that it can directly be used as a PerturbNormal.normalMapColor input
     */
    generateInWorldSpace: boolean;
    /**
     * Defines that the worldNormal input will be normalized by the HeightToNormal block before being used
     */
    automaticNormalizationNormal: boolean;
    /**
     * Defines that the worldTangent input will be normalized by the HeightToNormal block before being used
     */
    automaticNormalizationTangent: boolean;
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the input component
     */
    get input(): NodeMaterialConnectionPoint;
    /**
     * Gets the position component
     */
    get worldPosition(): NodeMaterialConnectionPoint;
    /**
     * Gets the normal component
     */
    get worldNormal(): NodeMaterialConnectionPoint;
    /**
     * Gets the tangent component
     */
    get worldTangent(): NodeMaterialConnectionPoint;
    /**
     * Gets the output component
     */
    get output(): NodeMaterialConnectionPoint;
    /**
     * Gets the xyz component
     */
    get xyz(): NodeMaterialConnectionPoint;
    protected _buildBlock(state: NodeMaterialBuildState): this;
    protected _dumpPropertiesCode(): string;
    serialize(): any;
    _deserialize(serializationObject: any, scene: Scene, rootUrl: string): void;
}

}
declare module "babylonjs/Materials/Node/Blocks/Fragment/imageProcessingBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { NodeMaterial, NodeMaterialDefines } from "babylonjs/Materials/Node/nodeMaterial";
import { Effect } from "babylonjs/Materials/effect";
import { Mesh } from "babylonjs/Meshes/mesh";
import { Scene } from "babylonjs/scene";
import "babylonjs/Shaders/ShadersInclude/helperFunctions";
import "babylonjs/Shaders/ShadersInclude/imageProcessingDeclaration";
import "babylonjs/Shaders/ShadersInclude/imageProcessingFunctions";
/**
 * Block used to add image processing support to fragment shader
 */
export class ImageProcessingBlock extends NodeMaterialBlock {
    /**
     * Create a new ImageProcessingBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Defines if the input should be converted to linear space (default: true)
     */
    convertInputToLinearSpace: boolean;
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the color input component
     */
    get color(): NodeMaterialConnectionPoint;
    /**
     * Gets the output component
     */
    get output(): NodeMaterialConnectionPoint;
    /**
     * Gets the rgb component
     */
    get rgb(): NodeMaterialConnectionPoint;
    /**
     * Initialize the block and prepare the context for build
     * @param state defines the state that will be used for the build
     */
    initialize(state: NodeMaterialBuildState): void;
    isReady(mesh: AbstractMesh, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines): boolean;
    prepareDefines(mesh: AbstractMesh, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines): void;
    bind(effect: Effect, nodeMaterial: NodeMaterial, mesh?: Mesh): void;
    protected _buildBlock(state: NodeMaterialBuildState): this;
    protected _dumpPropertiesCode(): string;
    serialize(): any;
    _deserialize(serializationObject: any, scene: Scene, rootUrl: string): void;
}

}
declare module "babylonjs/Materials/Node/Blocks/Fragment/index" {
export * from "babylonjs/Materials/Node/Blocks/Fragment/fragmentOutputBlock";
export * from "babylonjs/Materials/Node/Blocks/Fragment/imageProcessingBlock";
export * from "babylonjs/Materials/Node/Blocks/Fragment/perturbNormalBlock";
export * from "babylonjs/Materials/Node/Blocks/Fragment/discardBlock";
export * from "babylonjs/Materials/Node/Blocks/Fragment/frontFacingBlock";
export * from "babylonjs/Materials/Node/Blocks/Fragment/derivativeBlock";
export * from "babylonjs/Materials/Node/Blocks/Fragment/fragCoordBlock";
export * from "babylonjs/Materials/Node/Blocks/Fragment/screenSizeBlock";
export * from "babylonjs/Materials/Node/Blocks/Fragment/screenSpaceBlock";
export * from "babylonjs/Materials/Node/Blocks/Fragment/twirlBlock";
export * from "babylonjs/Materials/Node/Blocks/Fragment/TBNBlock";
export * from "babylonjs/Materials/Node/Blocks/Fragment/heightToNormalBlock";
export * from "babylonjs/Materials/Node/Blocks/Fragment/fragDepthBlock";
export * from "babylonjs/Materials/Node/Blocks/Fragment/shadowMapBlock";

}
declare module "babylonjs/Materials/Node/Blocks/Fragment/perturbNormalBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
import { NodeMaterial, NodeMaterialDefines } from "babylonjs/Materials/Node/nodeMaterial";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { Mesh } from "babylonjs/Meshes/mesh";
import { Effect } from "babylonjs/Materials/effect";
import { Scene } from "babylonjs/scene";
import "babylonjs/Shaders/ShadersInclude/bumpFragmentMainFunctions";
import "babylonjs/Shaders/ShadersInclude/bumpFragmentFunctions";
import "babylonjs/Shaders/ShadersInclude/bumpFragment";
/**
 * Block used to perturb normals based on a normal map
 */
export class PerturbNormalBlock extends NodeMaterialBlock {
    private _tangentSpaceParameterName;
    private _tangentCorrectionFactorName;
    private _worldMatrixName;
    /** Gets or sets a boolean indicating that normal should be inverted on X axis */
    invertX: boolean;
    /** Gets or sets a boolean indicating that normal should be inverted on Y axis */
    invertY: boolean;
    /** Gets or sets a boolean indicating that parallax occlusion should be enabled */
    useParallaxOcclusion: boolean;
    /** Gets or sets a boolean indicating that sampling mode is in Object space */
    useObjectSpaceNormalMap: boolean;
    /**
     * Create a new PerturbNormalBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the world position input component
     */
    get worldPosition(): NodeMaterialConnectionPoint;
    /**
     * Gets the world normal input component
     */
    get worldNormal(): NodeMaterialConnectionPoint;
    /**
     * Gets the world tangent input component
     */
    get worldTangent(): NodeMaterialConnectionPoint;
    /**
     * Gets the uv input component
     */
    get uv(): NodeMaterialConnectionPoint;
    /**
     * Gets the normal map color input component
     */
    get normalMapColor(): NodeMaterialConnectionPoint;
    /**
     * Gets the strength input component
     */
    get strength(): NodeMaterialConnectionPoint;
    /**
     * Gets the view direction input component
     */
    get viewDirection(): NodeMaterialConnectionPoint;
    /**
     * Gets the parallax scale input component
     */
    get parallaxScale(): NodeMaterialConnectionPoint;
    /**
     * Gets the parallax height input component
     */
    get parallaxHeight(): NodeMaterialConnectionPoint;
    /**
     * Gets the TBN input component
     */
    get TBN(): NodeMaterialConnectionPoint;
    /**
     * Gets the World input component
     */
    get world(): NodeMaterialConnectionPoint;
    /**
     * Gets the output component
     */
    get output(): NodeMaterialConnectionPoint;
    /**
     * Gets the uv offset output component
     */
    get uvOffset(): NodeMaterialConnectionPoint;
    prepareDefines(mesh: AbstractMesh, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines): void;
    bind(effect: Effect, nodeMaterial: NodeMaterial, mesh?: Mesh): void;
    autoConfigure(material: NodeMaterial): void;
    protected _buildBlock(state: NodeMaterialBuildState): this;
    protected _dumpPropertiesCode(): string;
    serialize(): any;
    _deserialize(serializationObject: any, scene: Scene, rootUrl: string): void;
}

}
declare module "babylonjs/Materials/Node/Blocks/Fragment/screenSizeBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
import { Effect } from "babylonjs/Materials/effect";
/**
 * Block used to get the screen sizes
 */
export class ScreenSizeBlock extends NodeMaterialBlock {
    private _varName;
    private _scene;
    /**
     * Creates a new ScreenSizeBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the xy component
     */
    get xy(): NodeMaterialConnectionPoint;
    /**
     * Gets the x component
     */
    get x(): NodeMaterialConnectionPoint;
    /**
     * Gets the y component
     */
    get y(): NodeMaterialConnectionPoint;
    bind(effect: Effect): void;
    protected writeOutputs(state: NodeMaterialBuildState, varName: string): string;
    protected _buildBlock(state: NodeMaterialBuildState): this;
}

}
declare module "babylonjs/Materials/Node/Blocks/Fragment/screenSpaceBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
import { NodeMaterial } from "babylonjs/Materials/Node/nodeMaterial";
/**
 * Block used to transform a vector3 or a vector4 into screen space
 */
export class ScreenSpaceBlock extends NodeMaterialBlock {
    /**
     * Creates a new ScreenSpaceBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the vector input
     */
    get vector(): NodeMaterialConnectionPoint;
    /**
     * Gets the worldViewProjection transform input
     */
    get worldViewProjection(): NodeMaterialConnectionPoint;
    /**
     * Gets the output component
     */
    get output(): NodeMaterialConnectionPoint;
    /**
     * Gets the x output component
     */
    get x(): NodeMaterialConnectionPoint;
    /**
     * Gets the y output component
     */
    get y(): NodeMaterialConnectionPoint;
    autoConfigure(material: NodeMaterial): void;
    protected _buildBlock(state: NodeMaterialBuildState): this | undefined;
}

}
declare module "babylonjs/Materials/Node/Blocks/Fragment/shadowMapBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
/**
 * Block used to output the depth to a shadow map
 */
export class ShadowMapBlock extends NodeMaterialBlock {
    /**
     * Create a new ShadowMapBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Initialize the block and prepare the context for build
     * @param state defines the state that will be used for the build
     */
    initialize(state: NodeMaterialBuildState): void;
    /**
     * Gets the world position input component
     */
    get worldPosition(): NodeMaterialConnectionPoint;
    /**
     * Gets the view x projection input component
     */
    get viewProjection(): NodeMaterialConnectionPoint;
    /**
     * Gets the world normal input component
     */
    get worldNormal(): NodeMaterialConnectionPoint;
    /**
     * Gets the depth output component
     */
    get depth(): NodeMaterialConnectionPoint;
    protected _buildBlock(state: NodeMaterialBuildState): this;
}

}
declare module "babylonjs/Materials/Node/Blocks/Fragment/TBNBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialBlockTargets } from "babylonjs/Materials/Node/Enums/nodeMaterialBlockTargets";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
import { NodeMaterial, NodeMaterialDefines } from "babylonjs/Materials/Node/nodeMaterial";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
/**
 * Block used to implement TBN matrix
 */
export class TBNBlock extends NodeMaterialBlock {
    /**
     * Create a new TBNBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Initialize the block and prepare the context for build
     * @param state defines the state that will be used for the build
     */
    initialize(state: NodeMaterialBuildState): void;
    /**
     * Gets the normal input component
     */
    get normal(): NodeMaterialConnectionPoint;
    /**
     * Gets the tangent input component
     */
    get tangent(): NodeMaterialConnectionPoint;
    /**
     * Gets the world matrix input component
     */
    get world(): NodeMaterialConnectionPoint;
    /**
     * Gets the TBN output component
     */
    get TBN(): NodeMaterialConnectionPoint;
    /**
     * Gets the row0 of the output matrix
     */
    get row0(): NodeMaterialConnectionPoint;
    /**
     * Gets the row1 of the output matrix
     */
    get row1(): NodeMaterialConnectionPoint;
    /**
     * Gets the row2 of the output matrix
     */
    get row2(): NodeMaterialConnectionPoint;
    get target(): NodeMaterialBlockTargets;
    set target(value: NodeMaterialBlockTargets);
    autoConfigure(material: NodeMaterial): void;
    prepareDefines(mesh: AbstractMesh, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines): void;
    protected _buildBlock(state: NodeMaterialBuildState): this;
}

}
declare module "babylonjs/Materials/Node/Blocks/Fragment/twirlBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
/**
 * Block used to generate a twirl
 */
export class TwirlBlock extends NodeMaterialBlock {
    /**
     * Creates a new TwirlBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the input component
     */
    get input(): NodeMaterialConnectionPoint;
    /**
     * Gets the strength component
     */
    get strength(): NodeMaterialConnectionPoint;
    /**
     * Gets the center component
     */
    get center(): NodeMaterialConnectionPoint;
    /**
     * Gets the offset component
     */
    get offset(): NodeMaterialConnectionPoint;
    /**
     * Gets the output component
     */
    get output(): NodeMaterialConnectionPoint;
    /**
     * Gets the x output component
     */
    get x(): NodeMaterialConnectionPoint;
    /**
     * Gets the y output component
     */
    get y(): NodeMaterialConnectionPoint;
    autoConfigure(): void;
    protected _buildBlock(state: NodeMaterialBuildState): this;
}

}
declare module "babylonjs/Materials/Node/Blocks/fresnelBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
import { NodeMaterial } from "babylonjs/Materials/Node/nodeMaterial";
import "babylonjs/Shaders/ShadersInclude/fresnelFunction";
/**
 * Block used to compute fresnel value
 */
export class FresnelBlock extends NodeMaterialBlock {
    /**
     * Create a new FresnelBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the world normal input component
     */
    get worldNormal(): NodeMaterialConnectionPoint;
    /**
     * Gets the view direction input component
     */
    get viewDirection(): NodeMaterialConnectionPoint;
    /**
     * Gets the bias input component
     */
    get bias(): NodeMaterialConnectionPoint;
    /**
     * Gets the camera (or eye) position component
     */
    get power(): NodeMaterialConnectionPoint;
    /**
     * Gets the fresnel output component
     */
    get fresnel(): NodeMaterialConnectionPoint;
    autoConfigure(material: NodeMaterial): void;
    protected _buildBlock(state: NodeMaterialBuildState): this;
}

}
declare module "babylonjs/Materials/Node/Blocks/gradientBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
import { Color3 } from "babylonjs/Maths/math.color";
import { Scene } from "babylonjs/scene";
import { Observable } from "babylonjs/Misc/observable";
/**
 * Class used to store a color step for the GradientBlock
 */
export class GradientBlockColorStep {
    private _step;
    /**
     * Gets value indicating which step this color is associated with (between 0 and 1)
     */
    get step(): number;
    /**
     * Sets a value indicating which step this color is associated with (between 0 and 1)
     */
    set step(val: number);
    private _color;
    /**
     * Gets the color associated with this step
     */
    get color(): Color3;
    /**
     * Sets the color associated with this step
     */
    set color(val: Color3);
    /**
     * Creates a new GradientBlockColorStep
     * @param step defines a value indicating which step this color is associated with (between 0 and 1)
     * @param color defines the color associated with this step
     */
    constructor(step: number, color: Color3);
}
/**
 * Block used to return a color from a gradient based on an input value between 0 and 1
 */
export class GradientBlock extends NodeMaterialBlock {
    /**
     * Gets or sets the list of color steps
     */
    colorSteps: GradientBlockColorStep[];
    /** Gets an observable raised when the value is changed */
    onValueChangedObservable: Observable<GradientBlock>;
    /** calls observable when the value is changed*/
    colorStepsUpdated(): void;
    /**
     * Creates a new GradientBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the gradient input component
     */
    get gradient(): NodeMaterialConnectionPoint;
    /**
     * Gets the output component
     */
    get output(): NodeMaterialConnectionPoint;
    private _writeColorConstant;
    protected _buildBlock(state: NodeMaterialBuildState): this | undefined;
    serialize(): any;
    _deserialize(serializationObject: any, scene: Scene, rootUrl: string): void;
    protected _dumpPropertiesCode(): string;
}

}
declare module "babylonjs/Materials/Node/Blocks/index" {
export * from "babylonjs/Materials/Node/Blocks/Vertex/index";
export * from "babylonjs/Materials/Node/Blocks/Fragment/index";
export * from "babylonjs/Materials/Node/Blocks/Dual/index";
export * from "babylonjs/Materials/Node/Blocks/Input/index";
export * from "babylonjs/Materials/Node/Blocks/multiplyBlock";
export * from "babylonjs/Materials/Node/Blocks/addBlock";
export * from "babylonjs/Materials/Node/Blocks/scaleBlock";
export * from "babylonjs/Materials/Node/Blocks/clampBlock";
export * from "babylonjs/Materials/Node/Blocks/crossBlock";
export * from "babylonjs/Materials/Node/Blocks/customBlock";
export * from "babylonjs/Materials/Node/Blocks/dotBlock";
export * from "babylonjs/Materials/Node/Blocks/transformBlock";
export * from "babylonjs/Materials/Node/Blocks/remapBlock";
export * from "babylonjs/Materials/Node/Blocks/normalizeBlock";
export * from "babylonjs/Materials/Node/Blocks/trigonometryBlock";
export * from "babylonjs/Materials/Node/Blocks/colorMergerBlock";
export * from "babylonjs/Materials/Node/Blocks/vectorMergerBlock";
export * from "babylonjs/Materials/Node/Blocks/colorSplitterBlock";
export * from "babylonjs/Materials/Node/Blocks/vectorSplitterBlock";
export * from "babylonjs/Materials/Node/Blocks/lerpBlock";
export * from "babylonjs/Materials/Node/Blocks/divideBlock";
export * from "babylonjs/Materials/Node/Blocks/subtractBlock";
export * from "babylonjs/Materials/Node/Blocks/stepBlock";
export * from "babylonjs/Materials/Node/Blocks/oneMinusBlock";
export * from "babylonjs/Materials/Node/Blocks/viewDirectionBlock";
export * from "babylonjs/Materials/Node/Blocks/fresnelBlock";
export * from "babylonjs/Materials/Node/Blocks/maxBlock";
export * from "babylonjs/Materials/Node/Blocks/minBlock";
export * from "babylonjs/Materials/Node/Blocks/distanceBlock";
export * from "babylonjs/Materials/Node/Blocks/lengthBlock";
export * from "babylonjs/Materials/Node/Blocks/negateBlock";
export * from "babylonjs/Materials/Node/Blocks/powBlock";
export * from "babylonjs/Materials/Node/Blocks/randomNumberBlock";
export * from "babylonjs/Materials/Node/Blocks/arcTan2Block";
export * from "babylonjs/Materials/Node/Blocks/smoothStepBlock";
export * from "babylonjs/Materials/Node/Blocks/reciprocalBlock";
export * from "babylonjs/Materials/Node/Blocks/replaceColorBlock";
export * from "babylonjs/Materials/Node/Blocks/posterizeBlock";
export * from "babylonjs/Materials/Node/Blocks/waveBlock";
export * from "babylonjs/Materials/Node/Blocks/gradientBlock";
export * from "babylonjs/Materials/Node/Blocks/nLerpBlock";
export * from "babylonjs/Materials/Node/Blocks/worleyNoise3DBlock";
export * from "babylonjs/Materials/Node/Blocks/simplexPerlin3DBlock";
export * from "babylonjs/Materials/Node/Blocks/normalBlendBlock";
export * from "babylonjs/Materials/Node/Blocks/rotate2dBlock";
export * from "babylonjs/Materials/Node/Blocks/reflectBlock";
export * from "babylonjs/Materials/Node/Blocks/refractBlock";
export * from "babylonjs/Materials/Node/Blocks/desaturateBlock";
export * from "babylonjs/Materials/Node/Blocks/PBR/index";
export * from "babylonjs/Materials/Node/Blocks/Particle/index";
export * from "babylonjs/Materials/Node/Blocks/modBlock";
export * from "babylonjs/Materials/Node/Blocks/matrixBuilderBlock";
export * from "babylonjs/Materials/Node/Blocks/conditionalBlock";
export * from "babylonjs/Materials/Node/Blocks/cloudBlock";
export * from "babylonjs/Materials/Node/Blocks/voronoiNoiseBlock";
export * from "babylonjs/Materials/Node/Blocks/elbowBlock";
export * from "babylonjs/Materials/Node/Blocks/triPlanarBlock";
export * from "babylonjs/Materials/Node/Blocks/biPlanarBlock";
export * from "babylonjs/Materials/Node/Blocks/matrixDeterminantBlock";
export * from "babylonjs/Materials/Node/Blocks/matrixTransposeBlock";

}
declare module "babylonjs/Materials/Node/Blocks/Input/animatedInputBlockTypes" {
/**
 * Enum defining the type of animations supported by InputBlock
 */
export enum AnimatedInputBlockTypes {
    /** No animation */
    None = 0,
    /** Time based animation (is incremented by 0.6 each second). Will only work for floats */
    Time = 1,
    /** Time elapsed (in seconds) since the engine was initialized. Will only work for floats */
    RealTime = 2
}

}
declare module "babylonjs/Materials/Node/Blocks/Input/index" {
export * from "babylonjs/Materials/Node/Blocks/Input/inputBlock";
export * from "babylonjs/Materials/Node/Blocks/Input/animatedInputBlockTypes";

}
declare module "babylonjs/Materials/Node/Blocks/Input/inputBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBlockConnectionPointTypes } from "babylonjs/Materials/Node/Enums/nodeMaterialBlockConnectionPointTypes";
import { NodeMaterialSystemValues } from "babylonjs/Materials/Node/Enums/nodeMaterialSystemValues";
import { Nullable } from "babylonjs/types";
import { Effect } from "babylonjs/Materials/effect";
import { Matrix } from "babylonjs/Maths/math.vector";
import { Scene } from "babylonjs/scene";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialBlockTargets } from "babylonjs/Materials/Node/Enums/nodeMaterialBlockTargets";
import { AnimatedInputBlockTypes } from "babylonjs/Materials/Node/Blocks/Input/animatedInputBlockTypes";
import { Observable } from "babylonjs/Misc/observable";
import { NodeMaterial } from "babylonjs/Materials/Node/nodeMaterial";
/**
 * Block used to expose an input value
 */
export class InputBlock extends NodeMaterialBlock {
    private _mode;
    private _associatedVariableName;
    private _storedValue;
    private _valueCallback;
    private _type;
    private _animationType;
    /** Gets or set a value used to limit the range of float values */
    min: number;
    /** Gets or set a value used to limit the range of float values */
    max: number;
    /** Gets or set a value indicating that this input can only get 0 and 1 values */
    isBoolean: boolean;
    /** Gets or sets a value used by the Node Material editor to determine how to configure the current value if it is a matrix */
    matrixMode: number;
    /** @internal */
    _systemValue: Nullable<NodeMaterialSystemValues>;
    /** Gets or sets a boolean indicating that the value of this input will not change after a build */
    isConstant: boolean;
    /** Gets or sets the group to use to display this block in the Inspector */
    groupInInspector: string;
    /** Gets an observable raised when the value is changed */
    onValueChangedObservable: Observable<InputBlock>;
    /** Gets or sets a boolean indicating if content needs to be converted to gamma space (for color3/4 only) */
    convertToGammaSpace: boolean;
    /** Gets or sets a boolean indicating if content needs to be converted to linear space (for color3/4 only) */
    convertToLinearSpace: boolean;
    /**
     * Gets or sets the connection point type (default is float)
     */
    get type(): NodeMaterialBlockConnectionPointTypes;
    /**
     * Creates a new InputBlock
     * @param name defines the block name
     * @param target defines the target of that block (Vertex by default)
     * @param type defines the type of the input (can be set to NodeMaterialBlockConnectionPointTypes.AutoDetect)
     */
    constructor(name: string, target?: NodeMaterialBlockTargets, type?: NodeMaterialBlockConnectionPointTypes);
    /**
     * Validates if a name is a reserve word.
     * @param newName the new name to be given to the node.
     * @returns false if the name is a reserve word, else true.
     */
    validateBlockName(newName: string): boolean;
    /**
     * Gets the output component
     */
    get output(): NodeMaterialConnectionPoint;
    /**
     * Set the source of this connection point to a vertex attribute
     * @param attributeName defines the attribute name (position, uv, normal, etc...). If not specified it will take the connection point name
     * @returns the current connection point
     */
    setAsAttribute(attributeName?: string): InputBlock;
    /**
     * Set the source of this connection point to a system value
     * @param value define the system value to use (world, view, etc...) or null to switch to manual value
     * @returns the current connection point
     */
    setAsSystemValue(value: Nullable<NodeMaterialSystemValues>): InputBlock;
    /**
     * Gets or sets the value of that point.
     * Please note that this value will be ignored if valueCallback is defined
     */
    get value(): any;
    set value(value: any);
    /**
     * Gets or sets a callback used to get the value of that point.
     * Please note that setting this value will force the connection point to ignore the value property
     */
    get valueCallback(): () => any;
    set valueCallback(value: () => any);
    /**
     * Gets or sets the associated variable name in the shader
     */
    get associatedVariableName(): string;
    set associatedVariableName(value: string);
    /** Gets or sets the type of animation applied to the input */
    get animationType(): AnimatedInputBlockTypes;
    set animationType(value: AnimatedInputBlockTypes);
    /**
     * Gets a boolean indicating that this connection point not defined yet
     */
    get isUndefined(): boolean;
    /**
     * Gets or sets a boolean indicating that this connection point is coming from an uniform.
     * In this case the connection point name must be the name of the uniform to use.
     * Can only be set on inputs
     */
    get isUniform(): boolean;
    set isUniform(value: boolean);
    /**
     * Gets or sets a boolean indicating that this connection point is coming from an attribute.
     * In this case the connection point name must be the name of the attribute to use
     * Can only be set on inputs
     */
    get isAttribute(): boolean;
    set isAttribute(value: boolean);
    /**
     * Gets or sets a boolean indicating that this connection point is generating a varying variable.
     * Can only be set on exit points
     */
    get isVarying(): boolean;
    set isVarying(value: boolean);
    /**
     * Gets a boolean indicating that the current connection point is a system value
     */
    get isSystemValue(): boolean;
    /**
     * Gets or sets the current well known value or null if not defined as a system value
     */
    get systemValue(): Nullable<NodeMaterialSystemValues>;
    set systemValue(value: Nullable<NodeMaterialSystemValues>);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Animate the input if animationType !== None
     * @param scene defines the rendering scene
     */
    animate(scene: Scene): void;
    private _emitDefine;
    initialize(): void;
    /**
     * Set the input block to its default value (based on its type)
     */
    setDefaultValue(): void;
    private _emitConstant;
    /** @internal */
    get _noContextSwitch(): boolean;
    private _emit;
    /**
     * @internal
     */
    _transmitWorld(effect: Effect, world: Matrix, worldView: Matrix, worldViewProjection: Matrix): void;
    /**
     * @internal
     */
    _transmit(effect: Effect, scene: Scene, material: NodeMaterial): void;
    protected _buildBlock(state: NodeMaterialBuildState): void;
    protected _dumpPropertiesCode(): string;
    dispose(): void;
    serialize(): any;
    _deserialize(serializationObject: any, scene: Scene, rootUrl: string): void;
}

}
declare module "babylonjs/Materials/Node/Blocks/lengthBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
/**
 * Block used to get the length of a vector
 */
export class LengthBlock extends NodeMaterialBlock {
    /**
     * Creates a new LengthBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the value input component
     */
    get value(): NodeMaterialConnectionPoint;
    /**
     * Gets the output component
     */
    get output(): NodeMaterialConnectionPoint;
    protected _buildBlock(state: NodeMaterialBuildState): this;
}

}
declare module "babylonjs/Materials/Node/Blocks/lerpBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
/**
 * Block used to lerp between 2 values
 */
export class LerpBlock extends NodeMaterialBlock {
    /**
     * Creates a new LerpBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the left operand input component
     */
    get left(): NodeMaterialConnectionPoint;
    /**
     * Gets the right operand input component
     */
    get right(): NodeMaterialConnectionPoint;
    /**
     * Gets the gradient operand input component
     */
    get gradient(): NodeMaterialConnectionPoint;
    /**
     * Gets the output component
     */
    get output(): NodeMaterialConnectionPoint;
    protected _buildBlock(state: NodeMaterialBuildState): this;
}

}
declare module "babylonjs/Materials/Node/Blocks/matrixBuilderBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
/**
 * Block used to build a matrix from 4 Vector4
 */
export class MatrixBuilderBlock extends NodeMaterialBlock {
    /**
     * Creates a new MatrixBuilder
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the row0 vector
     */
    get row0(): NodeMaterialConnectionPoint;
    /**
     * Gets the row1 vector
     */
    get row1(): NodeMaterialConnectionPoint;
    /**
     * Gets the row2 vector
     */
    get row2(): NodeMaterialConnectionPoint;
    /**
     * Gets the row3 vector
     */
    get row3(): NodeMaterialConnectionPoint;
    /**
     * Gets the output component
     */
    get output(): NodeMaterialConnectionPoint;
    autoConfigure(): void;
    protected _buildBlock(state: NodeMaterialBuildState): this;
}

}
declare module "babylonjs/Materials/Node/Blocks/matrixDeterminantBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
/**
 * Block used to compute the determinant of a matrix
 */
export class MatrixDeterminantBlock extends NodeMaterialBlock {
    /**
     * Creates a new MatrixDeterminantBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the input matrix
     */
    get input(): NodeMaterialConnectionPoint;
    /**
     * Gets the output component
     */
    get output(): NodeMaterialConnectionPoint;
    protected _buildBlock(state: NodeMaterialBuildState): this;
}

}
declare module "babylonjs/Materials/Node/Blocks/matrixTransposeBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
/**
 * Block used to transpose a matrix
 */
export class MatrixTransposeBlock extends NodeMaterialBlock {
    /**
     * Creates a new MatrixTransposeBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the input matrix
     */
    get input(): NodeMaterialConnectionPoint;
    /**
     * Gets the output component
     */
    get output(): NodeMaterialConnectionPoint;
    protected _buildBlock(state: NodeMaterialBuildState): this;
}

}
declare module "babylonjs/Materials/Node/Blocks/maxBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
/**
 * Block used to get the max of 2 values
 */
export class MaxBlock extends NodeMaterialBlock {
    /**
     * Creates a new MaxBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the left operand input component
     */
    get left(): NodeMaterialConnectionPoint;
    /**
     * Gets the right operand input component
     */
    get right(): NodeMaterialConnectionPoint;
    /**
     * Gets the output component
     */
    get output(): NodeMaterialConnectionPoint;
    protected _buildBlock(state: NodeMaterialBuildState): this;
}

}
declare module "babylonjs/Materials/Node/Blocks/minBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
/**
 * Block used to get the min of 2 values
 */
export class MinBlock extends NodeMaterialBlock {
    /**
     * Creates a new MinBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the left operand input component
     */
    get left(): NodeMaterialConnectionPoint;
    /**
     * Gets the right operand input component
     */
    get right(): NodeMaterialConnectionPoint;
    /**
     * Gets the output component
     */
    get output(): NodeMaterialConnectionPoint;
    protected _buildBlock(state: NodeMaterialBuildState): this;
}

}
declare module "babylonjs/Materials/Node/Blocks/modBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
/**
 * Block used to compute value of one parameter modulo another
 */
export class ModBlock extends NodeMaterialBlock {
    /**
     * Creates a new ModBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the left operand input component
     */
    get left(): NodeMaterialConnectionPoint;
    /**
     * Gets the right operand input component
     */
    get right(): NodeMaterialConnectionPoint;
    /**
     * Gets the output component
     */
    get output(): NodeMaterialConnectionPoint;
    protected _buildBlock(state: NodeMaterialBuildState): this;
}

}
declare module "babylonjs/Materials/Node/Blocks/multiplyBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
/**
 * Block used to multiply 2 values
 */
export class MultiplyBlock extends NodeMaterialBlock {
    /**
     * Creates a new MultiplyBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the left operand input component
     */
    get left(): NodeMaterialConnectionPoint;
    /**
     * Gets the right operand input component
     */
    get right(): NodeMaterialConnectionPoint;
    /**
     * Gets the output component
     */
    get output(): NodeMaterialConnectionPoint;
    protected _buildBlock(state: NodeMaterialBuildState): this;
}

}
declare module "babylonjs/Materials/Node/Blocks/negateBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
/**
 * Block used to get negative version of a value (i.e. x * -1)
 */
export class NegateBlock extends NodeMaterialBlock {
    /**
     * Creates a new NegateBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the value input component
     */
    get value(): NodeMaterialConnectionPoint;
    /**
     * Gets the output component
     */
    get output(): NodeMaterialConnectionPoint;
    protected _buildBlock(state: NodeMaterialBuildState): this;
}

}
declare module "babylonjs/Materials/Node/Blocks/nLerpBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
/**
 * Block used to normalize lerp between 2 values
 */
export class NLerpBlock extends NodeMaterialBlock {
    /**
     * Creates a new NLerpBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the left operand input component
     */
    get left(): NodeMaterialConnectionPoint;
    /**
     * Gets the right operand input component
     */
    get right(): NodeMaterialConnectionPoint;
    /**
     * Gets the gradient operand input component
     */
    get gradient(): NodeMaterialConnectionPoint;
    /**
     * Gets the output component
     */
    get output(): NodeMaterialConnectionPoint;
    protected _buildBlock(state: NodeMaterialBuildState): this;
}

}
declare module "babylonjs/Materials/Node/Blocks/normalBlendBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
/**
 * Block used to blend normals
 */
export class NormalBlendBlock extends NodeMaterialBlock {
    /**
     * Creates a new NormalBlendBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the first input component
     */
    get normalMap0(): NodeMaterialConnectionPoint;
    /**
     * Gets the second input component
     */
    get normalMap1(): NodeMaterialConnectionPoint;
    /**
     * Gets the output component
     */
    get output(): NodeMaterialConnectionPoint;
    protected _buildBlock(state: NodeMaterialBuildState): this;
}

}
declare module "babylonjs/Materials/Node/Blocks/normalizeBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
/**
 * Block used to normalize a vector
 */
export class NormalizeBlock extends NodeMaterialBlock {
    /**
     * Creates a new NormalizeBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the input component
     */
    get input(): NodeMaterialConnectionPoint;
    /**
     * Gets the output component
     */
    get output(): NodeMaterialConnectionPoint;
    protected _buildBlock(state: NodeMaterialBuildState): this;
}

}
declare module "babylonjs/Materials/Node/Blocks/oneMinusBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
/**
 * Block used to get the opposite (1 - x) of a value
 */
export class OneMinusBlock extends NodeMaterialBlock {
    /**
     * Creates a new OneMinusBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the input component
     */
    get input(): NodeMaterialConnectionPoint;
    /**
     * Gets the output component
     */
    get output(): NodeMaterialConnectionPoint;
    protected _buildBlock(state: NodeMaterialBuildState): this;
}

}
declare module "babylonjs/Materials/Node/Blocks/Particle/index" {
export * from "babylonjs/Materials/Node/Blocks/Particle/particleTextureBlock";
export * from "babylonjs/Materials/Node/Blocks/Particle/particleRampGradientBlock";
export * from "babylonjs/Materials/Node/Blocks/Particle/particleBlendMultiplyBlock";

}
declare module "babylonjs/Materials/Node/Blocks/Particle/particleBlendMultiplyBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
/**
 * Block used for the particle blend multiply section
 */
export class ParticleBlendMultiplyBlock extends NodeMaterialBlock {
    /**
     * Create a new ParticleBlendMultiplyBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the color input component
     */
    get color(): NodeMaterialConnectionPoint;
    /**
     * Gets the alphaTexture input component
     */
    get alphaTexture(): NodeMaterialConnectionPoint;
    /**
     * Gets the alphaColor input component
     */
    get alphaColor(): NodeMaterialConnectionPoint;
    /**
     * Gets the blendColor output component
     */
    get blendColor(): NodeMaterialConnectionPoint;
    /**
     * Initialize the block and prepare the context for build
     * @param state defines the state that will be used for the build
     */
    initialize(state: NodeMaterialBuildState): void;
    protected _buildBlock(state: NodeMaterialBuildState): this | undefined;
}

}
declare module "babylonjs/Materials/Node/Blocks/Particle/particleRampGradientBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
/**
 * Block used for the particle ramp gradient section
 */
export class ParticleRampGradientBlock extends NodeMaterialBlock {
    /**
     * Create a new ParticleRampGradientBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the color input component
     */
    get color(): NodeMaterialConnectionPoint;
    /**
     * Gets the rampColor output component
     */
    get rampColor(): NodeMaterialConnectionPoint;
    /**
     * Initialize the block and prepare the context for build
     * @param state defines the state that will be used for the build
     */
    initialize(state: NodeMaterialBuildState): void;
    protected _buildBlock(state: NodeMaterialBuildState): this | undefined;
}

}
declare module "babylonjs/Materials/Node/Blocks/Particle/particleTextureBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { NodeMaterialDefines } from "babylonjs/Materials/Node/nodeMaterial";
import { BaseTexture } from "babylonjs/Materials/Textures/baseTexture";
import { Nullable } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
import { NodeMaterial } from "babylonjs/Materials/Node/nodeMaterial";
/**
 * Base block used for the particle texture
 */
export class ParticleTextureBlock extends NodeMaterialBlock {
    private _samplerName;
    private _linearDefineName;
    private _gammaDefineName;
    private _tempTextureRead;
    /**
     * Gets or sets the texture associated with the node
     */
    texture: Nullable<BaseTexture>;
    /**
     * Gets or sets a boolean indicating if content needs to be converted to gamma space
     */
    convertToGammaSpace: boolean;
    /**
     * Gets or sets a boolean indicating if content needs to be converted to linear space
     */
    convertToLinearSpace: boolean;
    /**
     * Create a new ParticleTextureBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the uv input component
     */
    get uv(): NodeMaterialConnectionPoint;
    /**
     * Gets the rgba output component
     */
    get rgba(): NodeMaterialConnectionPoint;
    /**
     * Gets the rgb output component
     */
    get rgb(): NodeMaterialConnectionPoint;
    /**
     * Gets the r output component
     */
    get r(): NodeMaterialConnectionPoint;
    /**
     * Gets the g output component
     */
    get g(): NodeMaterialConnectionPoint;
    /**
     * Gets the b output component
     */
    get b(): NodeMaterialConnectionPoint;
    /**
     * Gets the a output component
     */
    get a(): NodeMaterialConnectionPoint;
    /**
     * Initialize the block and prepare the context for build
     * @param state defines the state that will be used for the build
     */
    initialize(state: NodeMaterialBuildState): void;
    autoConfigure(material: NodeMaterial): void;
    prepareDefines(mesh: AbstractMesh, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines): void;
    isReady(): boolean;
    private _writeOutput;
    protected _buildBlock(state: NodeMaterialBuildState): this | undefined;
    serialize(): any;
    _deserialize(serializationObject: any, scene: Scene, rootUrl: string): void;
}
export {};

}
declare module "babylonjs/Materials/Node/Blocks/PBR/anisotropyBlock" {
import { NodeMaterial, NodeMaterialDefines } from "babylonjs/Materials/Node/nodeMaterial";
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { Mesh } from "babylonjs/Meshes/mesh";
import { Effect } from "babylonjs/Materials/effect";
/**
 * Block used to implement the anisotropy module of the PBR material
 */
export class AnisotropyBlock extends NodeMaterialBlock {
    private _tangentCorrectionFactorName;
    /**
     * The two properties below are set by the main PBR block prior to calling methods of this class.
     * This is to avoid having to add them as inputs here whereas they are already inputs of the main block, so already known.
     * It's less burden on the user side in the editor part.
     */
    /** @internal */
    worldPositionConnectionPoint: NodeMaterialConnectionPoint;
    /** @internal */
    worldNormalConnectionPoint: NodeMaterialConnectionPoint;
    /**
     * Create a new AnisotropyBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Initialize the block and prepare the context for build
     * @param state defines the state that will be used for the build
     */
    initialize(state: NodeMaterialBuildState): void;
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the intensity input component
     */
    get intensity(): NodeMaterialConnectionPoint;
    /**
     * Gets the direction input component
     */
    get direction(): NodeMaterialConnectionPoint;
    /**
     * Gets the uv input component
     */
    get uv(): NodeMaterialConnectionPoint;
    /**
     * Gets the worldTangent input component
     */
    get worldTangent(): NodeMaterialConnectionPoint;
    /**
     * Gets the TBN input component
     */
    get TBN(): NodeMaterialConnectionPoint;
    /**
     * Gets the anisotropy object output component
     */
    get anisotropy(): NodeMaterialConnectionPoint;
    private _generateTBNSpace;
    /**
     * Gets the main code of the block (fragment side)
     * @param state current state of the node material building
     * @param generateTBNSpace if true, the code needed to create the TBN coordinate space is generated
     * @returns the shader code
     */
    getCode(state: NodeMaterialBuildState, generateTBNSpace?: boolean): string;
    prepareDefines(mesh: AbstractMesh, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines): void;
    bind(effect: Effect, nodeMaterial: NodeMaterial, mesh?: Mesh): void;
    protected _buildBlock(state: NodeMaterialBuildState): this;
}

}
declare module "babylonjs/Materials/Node/Blocks/PBR/clearCoatBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
import { NodeMaterial, NodeMaterialDefines } from "babylonjs/Materials/Node/nodeMaterial";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { ReflectionBlock } from "babylonjs/Materials/Node/Blocks/PBR/reflectionBlock";
import { Scene } from "babylonjs/scene";
import { Nullable } from "babylonjs/types";
import { Mesh } from "babylonjs/Meshes/mesh";
import { Effect } from "babylonjs/Materials/effect";
/**
 * Block used to implement the clear coat module of the PBR material
 */
export class ClearCoatBlock extends NodeMaterialBlock {
    private _scene;
    private _tangentCorrectionFactorName;
    /**
     * Create a new ClearCoatBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Defines if the F0 value should be remapped to account for the interface change in the material.
     */
    remapF0OnInterfaceChange: boolean;
    /**
     * Initialize the block and prepare the context for build
     * @param state defines the state that will be used for the build
     */
    initialize(state: NodeMaterialBuildState): void;
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the intensity input component
     */
    get intensity(): NodeMaterialConnectionPoint;
    /**
     * Gets the roughness input component
     */
    get roughness(): NodeMaterialConnectionPoint;
    /**
     * Gets the ior input component
     */
    get indexOfRefraction(): NodeMaterialConnectionPoint;
    /**
     * Gets the bump texture input component
     */
    get normalMapColor(): NodeMaterialConnectionPoint;
    /**
     * Gets the uv input component
     */
    get uv(): NodeMaterialConnectionPoint;
    /**
     * Gets the tint color input component
     */
    get tintColor(): NodeMaterialConnectionPoint;
    /**
     * Gets the tint "at distance" input component
     */
    get tintAtDistance(): NodeMaterialConnectionPoint;
    /**
     * Gets the tint thickness input component
     */
    get tintThickness(): NodeMaterialConnectionPoint;
    /**
     * Gets the world tangent input component
     */
    get worldTangent(): NodeMaterialConnectionPoint;
    /**
     * Gets the world normal input component
     */
    get worldNormal(): NodeMaterialConnectionPoint;
    /**
     * Gets the TBN input component
     */
    get TBN(): NodeMaterialConnectionPoint;
    /**
     * Gets the clear coat object output component
     */
    get clearcoat(): NodeMaterialConnectionPoint;
    autoConfigure(): void;
    prepareDefines(mesh: AbstractMesh, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines): void;
    bind(effect: Effect, nodeMaterial: NodeMaterial, mesh?: Mesh): void;
    private _generateTBNSpace;
    /**
     * Gets the main code of the block (fragment side)
     * @param state current state of the node material building
     * @param ccBlock instance of a ClearCoatBlock or null if the code must be generated without an active clear coat module
     * @param reflectionBlock instance of a ReflectionBlock null if the code must be generated without an active reflection module
     * @param worldPosVarName name of the variable holding the world position
     * @param generateTBNSpace if true, the code needed to create the TBN coordinate space is generated
     * @param vTBNAvailable indicate that the vTBN variable is already existing because it has already been generated by another block (PerturbNormal or Anisotropy)
     * @param worldNormalVarName name of the variable holding the world normal
     * @returns the shader code
     */
    static GetCode(state: NodeMaterialBuildState, ccBlock: Nullable<ClearCoatBlock>, reflectionBlock: Nullable<ReflectionBlock>, worldPosVarName: string, generateTBNSpace: boolean, vTBNAvailable: boolean, worldNormalVarName: string): string;
    protected _buildBlock(state: NodeMaterialBuildState): this;
    protected _dumpPropertiesCode(): string;
    serialize(): any;
    _deserialize(serializationObject: any, scene: Scene, rootUrl: string): void;
}

}
declare module "babylonjs/Materials/Node/Blocks/PBR/index" {
export * from "babylonjs/Materials/Node/Blocks/PBR/pbrMetallicRoughnessBlock";
export * from "babylonjs/Materials/Node/Blocks/PBR/sheenBlock";
export * from "babylonjs/Materials/Node/Blocks/PBR/anisotropyBlock";
export * from "babylonjs/Materials/Node/Blocks/PBR/reflectionBlock";
export * from "babylonjs/Materials/Node/Blocks/PBR/clearCoatBlock";
export * from "babylonjs/Materials/Node/Blocks/PBR/refractionBlock";
export * from "babylonjs/Materials/Node/Blocks/PBR/subSurfaceBlock";

}
declare module "babylonjs/Materials/Node/Blocks/PBR/iridescenceBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
import { NodeMaterial, NodeMaterialDefines } from "babylonjs/Materials/Node/nodeMaterial";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { Scene } from "babylonjs/scene";
import { Nullable } from "babylonjs/types";
/**
 * Block used to implement the iridescence module of the PBR material
 */
export class IridescenceBlock extends NodeMaterialBlock {
    /**
     * Create a new IridescenceBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Initialize the block and prepare the context for build
     * @param state defines the state that will be used for the build
     */
    initialize(state: NodeMaterialBuildState): void;
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the intensity input component
     */
    get intensity(): NodeMaterialConnectionPoint;
    /**
     * Gets the indexOfRefraction input component
     */
    get indexOfRefraction(): NodeMaterialConnectionPoint;
    /**
     * Gets the thickness input component
     */
    get thickness(): NodeMaterialConnectionPoint;
    /**
     * Gets the iridescence object output component
     */
    get iridescence(): NodeMaterialConnectionPoint;
    autoConfigure(): void;
    prepareDefines(mesh: AbstractMesh, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines): void;
    /**
     * Gets the main code of the block (fragment side)
     * @param iridescenceBlock instance of a IridescenceBlock or null if the code must be generated without an active iridescence module
     * @returns the shader code
     */
    static GetCode(iridescenceBlock: Nullable<IridescenceBlock>): string;
    protected _buildBlock(state: NodeMaterialBuildState): this;
    serialize(): any;
    _deserialize(serializationObject: any, scene: Scene, rootUrl: string): void;
}

}
declare module "babylonjs/Materials/Node/Blocks/PBR/pbrMetallicRoughnessBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
import { NodeMaterial, NodeMaterialDefines } from "babylonjs/Materials/Node/nodeMaterial";
import { Light } from "babylonjs/Lights/light";
import { Nullable } from "babylonjs/types";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { Effect } from "babylonjs/Materials/effect";
import { Mesh } from "babylonjs/Meshes/mesh";
import { Scene } from "babylonjs/scene";
/**
 * Block used to implement the PBR metallic/roughness model
 */
export class PBRMetallicRoughnessBlock extends NodeMaterialBlock {
    /**
     * Gets or sets the light associated with this block
     */
    light: Nullable<Light>;
    private static _OnGenerateOnlyFragmentCodeChanged;
    private _setTarget;
    private _lightId;
    private _scene;
    private _environmentBRDFTexture;
    private _environmentBrdfSamplerName;
    private _vNormalWName;
    private _invertNormalName;
    private _metallicReflectanceColor;
    private _metallicF0Factor;
    private _vMetallicReflectanceFactorsName;
    /**
     * Create a new ReflectionBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Intensity of the direct lights e.g. the four lights available in your scene.
     * This impacts both the direct diffuse and specular highlights.
     */
    directIntensity: number;
    /**
     * Intensity of the environment e.g. how much the environment will light the object
     * either through harmonics for rough material or through the reflection for shiny ones.
     */
    environmentIntensity: number;
    /**
     * This is a special control allowing the reduction of the specular highlights coming from the
     * four lights of the scene. Those highlights may not be needed in full environment lighting.
     */
    specularIntensity: number;
    /**
     * Defines the  falloff type used in this material.
     * It by default is Physical.
     */
    lightFalloff: number;
    /**
     * Specifies that alpha test should be used
     */
    useAlphaTest: boolean;
    /**
     * Defines the alpha limits in alpha test mode.
     */
    alphaTestCutoff: number;
    /**
     * Specifies that alpha blending should be used
     */
    useAlphaBlending: boolean;
    /**
     * Specifies that the material will keeps the reflection highlights over a transparent surface (only the most luminous ones).
     * A car glass is a good example of that. When the street lights reflects on it you can not see what is behind.
     */
    useRadianceOverAlpha: boolean;
    /**
     * Specifies that the material will keeps the specular highlights over a transparent surface (only the most luminous ones).
     * A car glass is a good example of that. When sun reflects on it you can not see what is behind.
     */
    useSpecularOverAlpha: boolean;
    /**
     * Enables specular anti aliasing in the PBR shader.
     * It will both interacts on the Geometry for analytical and IBL lighting.
     * It also prefilter the roughness map based on the bump values.
     */
    enableSpecularAntiAliasing: boolean;
    /**
     * Enables realtime filtering on the texture.
     */
    realTimeFiltering: boolean;
    /**
     * Quality switch for realtime filtering
     */
    realTimeFilteringQuality: number;
    /**
     * Defines if the material uses energy conservation.
     */
    useEnergyConservation: boolean;
    /**
     * This parameters will enable/disable radiance occlusion by preventing the radiance to lit
     * too much the area relying on ambient texture to define their ambient occlusion.
     */
    useRadianceOcclusion: boolean;
    /**
     * This parameters will enable/disable Horizon occlusion to prevent normal maps to look shiny when the normal
     * makes the reflect vector face the model (under horizon).
     */
    useHorizonOcclusion: boolean;
    /**
     * If set to true, no lighting calculations will be applied.
     */
    unlit: boolean;
    /**
     * Force normal to face away from face.
     */
    forceNormalForward: boolean;
    /** Indicates that no code should be generated in the vertex shader. Can be useful in some specific circumstances (like when doing ray marching for eg) */
    generateOnlyFragmentCode: boolean;
    /**
     * Defines the material debug mode.
     * It helps seeing only some components of the material while troubleshooting.
     */
    debugMode: number;
    /**
     * Specify from where on screen the debug mode should start.
     * The value goes from -1 (full screen) to 1 (not visible)
     * It helps with side by side comparison against the final render
     * This defaults to 0
     */
    debugLimit: number;
    /**
     * As the default viewing range might not be enough (if the ambient is really small for instance)
     * You can use the factor to better multiply the final value.
     */
    debugFactor: number;
    /**
     * Initialize the block and prepare the context for build
     * @param state defines the state that will be used for the build
     */
    initialize(state: NodeMaterialBuildState): void;
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the world position input component
     */
    get worldPosition(): NodeMaterialConnectionPoint;
    /**
     * Gets the world normal input component
     */
    get worldNormal(): NodeMaterialConnectionPoint;
    /**
     * Gets the view matrix parameter
     */
    get view(): NodeMaterialConnectionPoint;
    /**
     * Gets the camera position input component
     */
    get cameraPosition(): NodeMaterialConnectionPoint;
    /**
     * Gets the perturbed normal input component
     */
    get perturbedNormal(): NodeMaterialConnectionPoint;
    /**
     * Gets the base color input component
     */
    get baseColor(): NodeMaterialConnectionPoint;
    /**
     * Gets the metallic input component
     */
    get metallic(): NodeMaterialConnectionPoint;
    /**
     * Gets the roughness input component
     */
    get roughness(): NodeMaterialConnectionPoint;
    /**
     * Gets the ambient occlusion input component
     */
    get ambientOcc(): NodeMaterialConnectionPoint;
    /**
     * Gets the opacity input component
     */
    get opacity(): NodeMaterialConnectionPoint;
    /**
     * Gets the index of refraction input component
     */
    get indexOfRefraction(): NodeMaterialConnectionPoint;
    /**
     * Gets the ambient color input component
     */
    get ambientColor(): NodeMaterialConnectionPoint;
    /**
     * Gets the reflection object parameters
     */
    get reflection(): NodeMaterialConnectionPoint;
    /**
     * Gets the clear coat object parameters
     */
    get clearcoat(): NodeMaterialConnectionPoint;
    /**
     * Gets the sheen object parameters
     */
    get sheen(): NodeMaterialConnectionPoint;
    /**
     * Gets the sub surface object parameters
     */
    get subsurface(): NodeMaterialConnectionPoint;
    /**
     * Gets the anisotropy object parameters
     */
    get anisotropy(): NodeMaterialConnectionPoint;
    /**
     * Gets the iridescence object parameters
     */
    get iridescence(): NodeMaterialConnectionPoint;
    /**
     * Gets the ambient output component
     */
    get ambientClr(): NodeMaterialConnectionPoint;
    /**
     * Gets the diffuse output component
     */
    get diffuseDir(): NodeMaterialConnectionPoint;
    /**
     * Gets the specular output component
     */
    get specularDir(): NodeMaterialConnectionPoint;
    /**
     * Gets the clear coat output component
     */
    get clearcoatDir(): NodeMaterialConnectionPoint;
    /**
     * Gets the sheen output component
     */
    get sheenDir(): NodeMaterialConnectionPoint;
    /**
     * Gets the indirect diffuse output component
     */
    get diffuseInd(): NodeMaterialConnectionPoint;
    /**
     * Gets the indirect specular output component
     */
    get specularInd(): NodeMaterialConnectionPoint;
    /**
     * Gets the indirect clear coat output component
     */
    get clearcoatInd(): NodeMaterialConnectionPoint;
    /**
     * Gets the indirect sheen output component
     */
    get sheenInd(): NodeMaterialConnectionPoint;
    /**
     * Gets the refraction output component
     */
    get refraction(): NodeMaterialConnectionPoint;
    /**
     * Gets the global lighting output component
     */
    get lighting(): NodeMaterialConnectionPoint;
    /**
     * Gets the shadow output component
     */
    get shadow(): NodeMaterialConnectionPoint;
    /**
     * Gets the alpha output component
     */
    get alpha(): NodeMaterialConnectionPoint;
    autoConfigure(material: NodeMaterial): void;
    prepareDefines(mesh: AbstractMesh, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines): void;
    updateUniformsAndSamples(state: NodeMaterialBuildState, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines, uniformBuffers: string[]): void;
    isReady(mesh: AbstractMesh, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines): boolean;
    bind(effect: Effect, nodeMaterial: NodeMaterial, mesh?: Mesh): void;
    private _injectVertexCode;
    private _getAlbedoOpacityCode;
    private _getAmbientOcclusionCode;
    private _getReflectivityCode;
    protected _buildBlock(state: NodeMaterialBuildState): this;
    protected _dumpPropertiesCode(): string;
    serialize(): any;
    _deserialize(serializationObject: any, scene: Scene, rootUrl: string): void;
}

}
declare module "babylonjs/Materials/Node/Blocks/PBR/reflectionBlock" {
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
import { NodeMaterial, NodeMaterialDefines } from "babylonjs/Materials/Node/nodeMaterial";
import { ReflectionTextureBaseBlock } from "babylonjs/Materials/Node/Blocks/Dual/reflectionTextureBaseBlock";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { Nullable } from "babylonjs/types";
import { BaseTexture } from "babylonjs/Materials/Textures/baseTexture";
import { Mesh } from "babylonjs/Meshes/mesh";
import { SubMesh } from "babylonjs/Meshes/subMesh";
import { Effect } from "babylonjs/Materials/effect";
import { Scene } from "babylonjs/scene";
/**
 * Block used to implement the reflection module of the PBR material
 */
export class ReflectionBlock extends ReflectionTextureBaseBlock {
    /** @internal */
    _defineLODReflectionAlpha: string;
    /** @internal */
    _defineLinearSpecularReflection: string;
    private _vEnvironmentIrradianceName;
    /** @internal */
    _vReflectionMicrosurfaceInfosName: string;
    /** @internal */
    _vReflectionInfosName: string;
    /** @internal */
    _vReflectionFilteringInfoName: string;
    private _scene;
    /**
     * The properties below are set by the main PBR block prior to calling methods of this class.
     * This is to avoid having to add them as inputs here whereas they are already inputs of the main block, so already known.
     * It's less burden on the user side in the editor part.
     */
    /** @internal */
    worldPositionConnectionPoint: NodeMaterialConnectionPoint;
    /** @internal */
    worldNormalConnectionPoint: NodeMaterialConnectionPoint;
    /** @internal */
    cameraPositionConnectionPoint: NodeMaterialConnectionPoint;
    /** @internal */
    viewConnectionPoint: NodeMaterialConnectionPoint;
    /**
     * Defines if the material uses spherical harmonics vs spherical polynomials for the
     * diffuse part of the IBL.
     */
    useSphericalHarmonics: boolean;
    /**
     * Force the shader to compute irradiance in the fragment shader in order to take bump in account.
     */
    forceIrradianceInFragment: boolean;
    protected _onGenerateOnlyFragmentCodeChanged(): boolean;
    protected _setTarget(): void;
    /**
     * Create a new ReflectionBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the position input component
     */
    get position(): NodeMaterialConnectionPoint;
    /**
     * Gets the world position input component
     */
    get worldPosition(): NodeMaterialConnectionPoint;
    /**
     * Gets the world normal input component
     */
    get worldNormal(): NodeMaterialConnectionPoint;
    /**
     * Gets the world input component
     */
    get world(): NodeMaterialConnectionPoint;
    /**
     * Gets the camera (or eye) position component
     */
    get cameraPosition(): NodeMaterialConnectionPoint;
    /**
     * Gets the view input component
     */
    get view(): NodeMaterialConnectionPoint;
    /**
     * Gets the color input component
     */
    get color(): NodeMaterialConnectionPoint;
    /**
     * Gets the reflection object output component
     */
    get reflection(): NodeMaterialConnectionPoint;
    /**
     * Returns true if the block has a texture (either its own texture or the environment texture from the scene, if set)
     */
    get hasTexture(): boolean;
    /**
     * Gets the reflection color (either the name of the variable if the color input is connected, else a default value)
     */
    get reflectionColor(): string;
    protected _getTexture(): Nullable<BaseTexture>;
    prepareDefines(mesh: AbstractMesh, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines): void;
    bind(effect: Effect, nodeMaterial: NodeMaterial, mesh?: Mesh, subMesh?: SubMesh): void;
    /**
     * Gets the code to inject in the vertex shader
     * @param state current state of the node material building
     * @returns the shader code
     */
    handleVertexSide(state: NodeMaterialBuildState): string;
    /**
     * Gets the main code of the block (fragment side)
     * @param state current state of the node material building
     * @param normalVarName name of the existing variable corresponding to the normal
     * @returns the shader code
     */
    getCode(state: NodeMaterialBuildState, normalVarName: string): string;
    protected _buildBlock(state: NodeMaterialBuildState): this;
    protected _dumpPropertiesCode(): string;
    serialize(): any;
    _deserialize(serializationObject: any, scene: Scene, rootUrl: string): void;
}

}
declare module "babylonjs/Materials/Node/Blocks/PBR/refractionBlock" {
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
import { NodeMaterial, NodeMaterialDefines } from "babylonjs/Materials/Node/nodeMaterial";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { Nullable } from "babylonjs/types";
import { BaseTexture } from "babylonjs/Materials/Textures/baseTexture";
import { Mesh } from "babylonjs/Meshes/mesh";
import { Effect } from "babylonjs/Materials/effect";
import { Scene } from "babylonjs/scene";
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
/**
 * Block used to implement the refraction part of the sub surface module of the PBR material
 */
export class RefractionBlock extends NodeMaterialBlock {
    /** @internal */
    _define3DName: string;
    /** @internal */
    _refractionMatrixName: string;
    /** @internal */
    _defineLODRefractionAlpha: string;
    /** @internal */
    _defineLinearSpecularRefraction: string;
    /** @internal */
    _defineOppositeZ: string;
    /** @internal */
    _cubeSamplerName: string;
    /** @internal */
    _2DSamplerName: string;
    /** @internal */
    _vRefractionMicrosurfaceInfosName: string;
    /** @internal */
    _vRefractionInfosName: string;
    /** @internal */
    _vRefractionFilteringInfoName: string;
    private _scene;
    /**
     * The properties below are set by the main PBR block prior to calling methods of this class.
     * This is to avoid having to add them as inputs here whereas they are already inputs of the main block, so already known.
     * It's less burden on the user side in the editor part.
     */
    /** @internal */
    viewConnectionPoint: NodeMaterialConnectionPoint;
    /** @internal */
    indexOfRefractionConnectionPoint: NodeMaterialConnectionPoint;
    /**
     * This parameters will make the material used its opacity to control how much it is refracting against not.
     * Materials half opaque for instance using refraction could benefit from this control.
     */
    linkRefractionWithTransparency: boolean;
    /**
     * Controls if refraction needs to be inverted on Y. This could be useful for procedural texture.
     */
    invertRefractionY: boolean;
    /**
     * Controls if refraction needs to be inverted on Y. This could be useful for procedural texture.
     */
    useThicknessAsDepth: boolean;
    /**
     * Gets or sets the texture associated with the node
     */
    texture: Nullable<BaseTexture>;
    /**
     * Create a new RefractionBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Initialize the block and prepare the context for build
     * @param state defines the state that will be used for the build
     */
    initialize(state: NodeMaterialBuildState): void;
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the intensity input component
     */
    get intensity(): NodeMaterialConnectionPoint;
    /**
     * Gets the tint at distance input component
     */
    get tintAtDistance(): NodeMaterialConnectionPoint;
    /**
     * Gets the volume index of refraction input component
     */
    get volumeIndexOfRefraction(): NodeMaterialConnectionPoint;
    /**
     * Gets the view input component
     */
    get view(): NodeMaterialConnectionPoint;
    /**
     * Gets the refraction object output component
     */
    get refraction(): NodeMaterialConnectionPoint;
    /**
     * Returns true if the block has a texture
     */
    get hasTexture(): boolean;
    protected _getTexture(): Nullable<BaseTexture>;
    autoConfigure(material: NodeMaterial): void;
    prepareDefines(mesh: AbstractMesh, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines): void;
    isReady(): boolean;
    bind(effect: Effect, nodeMaterial: NodeMaterial, mesh?: Mesh): void;
    /**
     * Gets the main code of the block (fragment side)
     * @param state current state of the node material building
     * @returns the shader code
     */
    getCode(state: NodeMaterialBuildState): string;
    protected _buildBlock(state: NodeMaterialBuildState): this;
    protected _dumpPropertiesCode(): string;
    serialize(): any;
    _deserialize(serializationObject: any, scene: Scene, rootUrl: string): void;
}

}
declare module "babylonjs/Materials/Node/Blocks/PBR/sheenBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
import { NodeMaterial, NodeMaterialDefines } from "babylonjs/Materials/Node/nodeMaterial";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { ReflectionBlock } from "babylonjs/Materials/Node/Blocks/PBR/reflectionBlock";
import { Scene } from "babylonjs/scene";
import { Nullable } from "babylonjs/types";
/**
 * Block used to implement the sheen module of the PBR material
 */
export class SheenBlock extends NodeMaterialBlock {
    /**
     * Create a new SheenBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * If true, the sheen effect is layered above the base BRDF with the albedo-scaling technique.
     * It allows the strength of the sheen effect to not depend on the base color of the material,
     * making it easier to setup and tweak the effect
     */
    albedoScaling: boolean;
    /**
     * Defines if the sheen is linked to the sheen color.
     */
    linkSheenWithAlbedo: boolean;
    /**
     * Initialize the block and prepare the context for build
     * @param state defines the state that will be used for the build
     */
    initialize(state: NodeMaterialBuildState): void;
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the intensity input component
     */
    get intensity(): NodeMaterialConnectionPoint;
    /**
     * Gets the color input component
     */
    get color(): NodeMaterialConnectionPoint;
    /**
     * Gets the roughness input component
     */
    get roughness(): NodeMaterialConnectionPoint;
    /**
     * Gets the sheen object output component
     */
    get sheen(): NodeMaterialConnectionPoint;
    prepareDefines(mesh: AbstractMesh, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines): void;
    /**
     * Gets the main code of the block (fragment side)
     * @param reflectionBlock instance of a ReflectionBlock null if the code must be generated without an active reflection module
     * @returns the shader code
     */
    getCode(reflectionBlock: Nullable<ReflectionBlock>): string;
    protected _buildBlock(state: NodeMaterialBuildState): this;
    protected _dumpPropertiesCode(): string;
    serialize(): any;
    _deserialize(serializationObject: any, scene: Scene, rootUrl: string): void;
}

}
declare module "babylonjs/Materials/Node/Blocks/PBR/subSurfaceBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
import { NodeMaterial, NodeMaterialDefines } from "babylonjs/Materials/Node/nodeMaterial";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { ReflectionBlock } from "babylonjs/Materials/Node/Blocks/PBR/reflectionBlock";
import { Nullable } from "babylonjs/types";
/**
 * Block used to implement the sub surface module of the PBR material
 */
export class SubSurfaceBlock extends NodeMaterialBlock {
    /**
     * Create a new SubSurfaceBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Initialize the block and prepare the context for build
     * @param state defines the state that will be used for the build
     */
    initialize(state: NodeMaterialBuildState): void;
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the thickness component
     */
    get thickness(): NodeMaterialConnectionPoint;
    /**
     * Gets the tint color input component
     */
    get tintColor(): NodeMaterialConnectionPoint;
    /**
     * Gets the translucency intensity input component
     */
    get translucencyIntensity(): NodeMaterialConnectionPoint;
    /**
     * Gets the translucency diffusion distance input component
     */
    get translucencyDiffusionDist(): NodeMaterialConnectionPoint;
    /**
     * Gets the refraction object parameters
     */
    get refraction(): NodeMaterialConnectionPoint;
    /**
     * Gets the sub surface object output component
     */
    get subsurface(): NodeMaterialConnectionPoint;
    autoConfigure(): void;
    prepareDefines(mesh: AbstractMesh, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines): void;
    /**
     * Gets the main code of the block (fragment side)
     * @param state current state of the node material building
     * @param ssBlock instance of a SubSurfaceBlock or null if the code must be generated without an active sub surface module
     * @param reflectionBlock instance of a ReflectionBlock null if the code must be generated without an active reflection module
     * @param worldPosVarName name of the variable holding the world position
     * @returns the shader code
     */
    static GetCode(state: NodeMaterialBuildState, ssBlock: Nullable<SubSurfaceBlock>, reflectionBlock: Nullable<ReflectionBlock>, worldPosVarName: string): string;
    protected _buildBlock(state: NodeMaterialBuildState): this;
}

}
declare module "babylonjs/Materials/Node/Blocks/posterizeBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
/**
 * Block used to posterize a value
 * @see https://en.wikipedia.org/wiki/Posterization
 */
export class PosterizeBlock extends NodeMaterialBlock {
    /**
     * Creates a new PosterizeBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the value input component
     */
    get value(): NodeMaterialConnectionPoint;
    /**
     * Gets the steps input component
     */
    get steps(): NodeMaterialConnectionPoint;
    /**
     * Gets the output component
     */
    get output(): NodeMaterialConnectionPoint;
    protected _buildBlock(state: NodeMaterialBuildState): this;
}

}
declare module "babylonjs/Materials/Node/Blocks/powBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
/**
 * Block used to get the value of the first parameter raised to the power of the second
 */
export class PowBlock extends NodeMaterialBlock {
    /**
     * Creates a new PowBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the value operand input component
     */
    get value(): NodeMaterialConnectionPoint;
    /**
     * Gets the power operand input component
     */
    get power(): NodeMaterialConnectionPoint;
    /**
     * Gets the output component
     */
    get output(): NodeMaterialConnectionPoint;
    protected _buildBlock(state: NodeMaterialBuildState): this;
}

}
declare module "babylonjs/Materials/Node/Blocks/randomNumberBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
import "babylonjs/Shaders/ShadersInclude/helperFunctions";
/**
 * Block used to get a random number
 */
export class RandomNumberBlock extends NodeMaterialBlock {
    /**
     * Creates a new RandomNumberBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the seed input component
     */
    get seed(): NodeMaterialConnectionPoint;
    /**
     * Gets the output component
     */
    get output(): NodeMaterialConnectionPoint;
    protected _buildBlock(state: NodeMaterialBuildState): this;
}

}
declare module "babylonjs/Materials/Node/Blocks/reciprocalBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
/**
 * Block used to get the reciprocal (1 / x) of a value
 */
export class ReciprocalBlock extends NodeMaterialBlock {
    /**
     * Creates a new ReciprocalBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the input component
     */
    get input(): NodeMaterialConnectionPoint;
    /**
     * Gets the output component
     */
    get output(): NodeMaterialConnectionPoint;
    protected _buildBlock(state: NodeMaterialBuildState): this;
}

}
declare module "babylonjs/Materials/Node/Blocks/reflectBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
/**
 * Block used to get the reflected vector from a direction and a normal
 */
export class ReflectBlock extends NodeMaterialBlock {
    /**
     * Creates a new ReflectBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the incident component
     */
    get incident(): NodeMaterialConnectionPoint;
    /**
     * Gets the normal component
     */
    get normal(): NodeMaterialConnectionPoint;
    /**
     * Gets the output component
     */
    get output(): NodeMaterialConnectionPoint;
    protected _buildBlock(state: NodeMaterialBuildState): this;
}

}
declare module "babylonjs/Materials/Node/Blocks/refractBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
/**
 * Block used to get the refracted vector from a direction and a normal
 */
export class RefractBlock extends NodeMaterialBlock {
    /**
     * Creates a new RefractBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the incident component
     */
    get incident(): NodeMaterialConnectionPoint;
    /**
     * Gets the normal component
     */
    get normal(): NodeMaterialConnectionPoint;
    /**
     * Gets the index of refraction component
     */
    get ior(): NodeMaterialConnectionPoint;
    /**
     * Gets the output component
     */
    get output(): NodeMaterialConnectionPoint;
    protected _buildBlock(state: NodeMaterialBuildState): this;
}

}
declare module "babylonjs/Materials/Node/Blocks/remapBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
import { Vector2 } from "babylonjs/Maths/math.vector";
import { Scene } from "babylonjs/scene";
/**
 * Block used to remap a float from a range to a new one
 */
export class RemapBlock extends NodeMaterialBlock {
    /**
     * Gets or sets the source range
     */
    sourceRange: Vector2;
    /**
     * Gets or sets the target range
     */
    targetRange: Vector2;
    /**
     * Creates a new RemapBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the input component
     */
    get input(): NodeMaterialConnectionPoint;
    /**
     * Gets the source min input component
     */
    get sourceMin(): NodeMaterialConnectionPoint;
    /**
     * Gets the source max input component
     */
    get sourceMax(): NodeMaterialConnectionPoint;
    /**
     * Gets the target min input component
     */
    get targetMin(): NodeMaterialConnectionPoint;
    /**
     * Gets the target max input component
     */
    get targetMax(): NodeMaterialConnectionPoint;
    /**
     * Gets the output component
     */
    get output(): NodeMaterialConnectionPoint;
    protected _buildBlock(state: NodeMaterialBuildState): this;
    protected _dumpPropertiesCode(): string;
    serialize(): any;
    _deserialize(serializationObject: any, scene: Scene, rootUrl: string): void;
}

}
declare module "babylonjs/Materials/Node/Blocks/replaceColorBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
/**
 * Block used to replace a color by another one
 */
export class ReplaceColorBlock extends NodeMaterialBlock {
    /**
     * Creates a new ReplaceColorBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the value input component
     */
    get value(): NodeMaterialConnectionPoint;
    /**
     * Gets the reference input component
     */
    get reference(): NodeMaterialConnectionPoint;
    /**
     * Gets the distance input component
     */
    get distance(): NodeMaterialConnectionPoint;
    /**
     * Gets the replacement input component
     */
    get replacement(): NodeMaterialConnectionPoint;
    /**
     * Gets the output component
     */
    get output(): NodeMaterialConnectionPoint;
    protected _buildBlock(state: NodeMaterialBuildState): this;
}

}
declare module "babylonjs/Materials/Node/Blocks/rotate2dBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
/**
 * Block used to rotate a 2d vector by a given angle
 */
export class Rotate2dBlock extends NodeMaterialBlock {
    /**
     * Creates a new Rotate2dBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the input vector
     */
    get input(): NodeMaterialConnectionPoint;
    /**
     * Gets the input angle
     */
    get angle(): NodeMaterialConnectionPoint;
    /**
     * Gets the output component
     */
    get output(): NodeMaterialConnectionPoint;
    autoConfigure(): void;
    protected _buildBlock(state: NodeMaterialBuildState): this;
}

}
declare module "babylonjs/Materials/Node/Blocks/scaleBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
/**
 * Block used to scale a vector by a float
 */
export class ScaleBlock extends NodeMaterialBlock {
    /**
     * Creates a new ScaleBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the input component
     */
    get input(): NodeMaterialConnectionPoint;
    /**
     * Gets the factor input component
     */
    get factor(): NodeMaterialConnectionPoint;
    /**
     * Gets the output component
     */
    get output(): NodeMaterialConnectionPoint;
    protected _buildBlock(state: NodeMaterialBuildState): this;
}

}
declare module "babylonjs/Materials/Node/Blocks/simplexPerlin3DBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
/**
 * block used to Generate a Simplex Perlin 3d Noise Pattern
 */
export class SimplexPerlin3DBlock extends NodeMaterialBlock {
    /**
     * Creates a new SimplexPerlin3DBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the seed operand input component
     */
    get seed(): NodeMaterialConnectionPoint;
    /**
     * Gets the output component
     */
    get output(): NodeMaterialConnectionPoint;
    protected _buildBlock(state: NodeMaterialBuildState): this | undefined;
}

}
declare module "babylonjs/Materials/Node/Blocks/smoothStepBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
/**
 * Block used to smooth step a value
 */
export class SmoothStepBlock extends NodeMaterialBlock {
    /**
     * Creates a new SmoothStepBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the value operand input component
     */
    get value(): NodeMaterialConnectionPoint;
    /**
     * Gets the first edge operand input component
     */
    get edge0(): NodeMaterialConnectionPoint;
    /**
     * Gets the second edge operand input component
     */
    get edge1(): NodeMaterialConnectionPoint;
    /**
     * Gets the output component
     */
    get output(): NodeMaterialConnectionPoint;
    protected _buildBlock(state: NodeMaterialBuildState): this;
}

}
declare module "babylonjs/Materials/Node/Blocks/stepBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
/**
 * Block used to step a value
 */
export class StepBlock extends NodeMaterialBlock {
    /**
     * Creates a new StepBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the value operand input component
     */
    get value(): NodeMaterialConnectionPoint;
    /**
     * Gets the edge operand input component
     */
    get edge(): NodeMaterialConnectionPoint;
    /**
     * Gets the output component
     */
    get output(): NodeMaterialConnectionPoint;
    protected _buildBlock(state: NodeMaterialBuildState): this;
}

}
declare module "babylonjs/Materials/Node/Blocks/subtractBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
/**
 * Block used to subtract 2 vectors
 */
export class SubtractBlock extends NodeMaterialBlock {
    /**
     * Creates a new SubtractBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the left operand input component
     */
    get left(): NodeMaterialConnectionPoint;
    /**
     * Gets the right operand input component
     */
    get right(): NodeMaterialConnectionPoint;
    /**
     * Gets the output component
     */
    get output(): NodeMaterialConnectionPoint;
    protected _buildBlock(state: NodeMaterialBuildState): this;
}

}
declare module "babylonjs/Materials/Node/Blocks/transformBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
import { Scene } from "babylonjs/scene";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { NodeMaterial, NodeMaterialDefines } from "babylonjs/Materials/Node/nodeMaterial";
/**
 * Block used to transform a vector (2, 3 or 4) with a matrix. It will generate a Vector4
 */
export class TransformBlock extends NodeMaterialBlock {
    /**
     * Defines the value to use to complement W value to transform it to a Vector4
     */
    complementW: number;
    /**
     * Defines the value to use to complement z value to transform it to a Vector4
     */
    complementZ: number;
    /**
     * Creates a new TransformBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the vector input
     */
    get vector(): NodeMaterialConnectionPoint;
    /**
     * Gets the output component
     */
    get output(): NodeMaterialConnectionPoint;
    /**
     * Gets the xyz output component
     */
    get xyz(): NodeMaterialConnectionPoint;
    /**
     * Gets the matrix transform input
     */
    get transform(): NodeMaterialConnectionPoint;
    protected _buildBlock(state: NodeMaterialBuildState): this;
    /**
     * Update defines for shader compilation
     * @param mesh defines the mesh to be rendered
     * @param nodeMaterial defines the node material requesting the update
     * @param defines defines the material defines to update
     */
    prepareDefines(mesh: AbstractMesh, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines): void;
    serialize(): any;
    _deserialize(serializationObject: any, scene: Scene, rootUrl: string): void;
    protected _dumpPropertiesCode(): string;
}

}
declare module "babylonjs/Materials/Node/Blocks/trigonometryBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
import { Scene } from "babylonjs/scene";
/**
 * Operations supported by the Trigonometry block
 */
export enum TrigonometryBlockOperations {
    /** Cos */
    Cos = 0,
    /** Sin */
    Sin = 1,
    /** Abs */
    Abs = 2,
    /** Exp */
    Exp = 3,
    /** Exp2 */
    Exp2 = 4,
    /** Round */
    Round = 5,
    /** Floor */
    Floor = 6,
    /** Ceiling */
    Ceiling = 7,
    /** Square root */
    Sqrt = 8,
    /** Log */
    Log = 9,
    /** Tangent */
    Tan = 10,
    /** Arc tangent */
    ArcTan = 11,
    /** Arc cosinus */
    ArcCos = 12,
    /** Arc sinus */
    ArcSin = 13,
    /** Fraction */
    Fract = 14,
    /** Sign */
    Sign = 15,
    /** To radians (from degrees) */
    Radians = 16,
    /** To degrees (from radians) */
    Degrees = 17
}
/**
 * Block used to apply trigonometry operation to floats
 */
export class TrigonometryBlock extends NodeMaterialBlock {
    /**
     * Gets or sets the operation applied by the block
     */
    operation: TrigonometryBlockOperations;
    /**
     * Creates a new TrigonometryBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the input component
     */
    get input(): NodeMaterialConnectionPoint;
    /**
     * Gets the output component
     */
    get output(): NodeMaterialConnectionPoint;
    protected _buildBlock(state: NodeMaterialBuildState): this;
    serialize(): any;
    _deserialize(serializationObject: any, scene: Scene, rootUrl: string): void;
    protected _dumpPropertiesCode(): string;
}

}
declare module "babylonjs/Materials/Node/Blocks/triPlanarBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { NodeMaterialDefines } from "babylonjs/Materials/Node/nodeMaterial";
import { NodeMaterial } from "babylonjs/Materials/Node/nodeMaterial";
import { Effect } from "babylonjs/Materials/effect";
import { Nullable } from "babylonjs/types";
import { Texture } from "babylonjs/Materials/Textures/texture";
import { Scene } from "babylonjs/scene";
import "babylonjs/Shaders/ShadersInclude/helperFunctions";
import { ImageSourceBlock } from "babylonjs/Materials/Node/Blocks/Dual/imageSourceBlock";
/**
 * Block used to read a texture with triplanar mapping (see "boxmap" in https://iquilezles.org/articles/biplanar/)
 */
export class TriPlanarBlock extends NodeMaterialBlock {
    private _linearDefineName;
    private _gammaDefineName;
    protected _tempTextureRead: string;
    private _samplerName;
    private _textureInfoName;
    private _imageSource;
    protected _texture: Nullable<Texture>;
    /**
     * Gets or sets the texture associated with the node
     */
    get texture(): Nullable<Texture>;
    set texture(texture: Nullable<Texture>);
    /**
     * Gets the textureY associated with the node
     */
    get textureY(): Nullable<Texture>;
    /**
     * Gets the textureZ associated with the node
     */
    get textureZ(): Nullable<Texture>;
    protected _getImageSourceBlock(connectionPoint: Nullable<NodeMaterialConnectionPoint>): Nullable<ImageSourceBlock>;
    /**
     * Gets the sampler name associated with this texture
     */
    get samplerName(): string;
    /**
     * Gets the samplerY name associated with this texture
     */
    get samplerYName(): Nullable<string>;
    /**
     * Gets the samplerZ name associated with this texture
     */
    get samplerZName(): Nullable<string>;
    /**
     * Gets a boolean indicating that this block is linked to an ImageSourceBlock
     */
    get hasImageSource(): boolean;
    private _convertToGammaSpace;
    /**
     * Gets or sets a boolean indicating if content needs to be converted to gamma space
     */
    set convertToGammaSpace(value: boolean);
    get convertToGammaSpace(): boolean;
    private _convertToLinearSpace;
    /**
     * Gets or sets a boolean indicating if content needs to be converted to linear space
     */
    set convertToLinearSpace(value: boolean);
    get convertToLinearSpace(): boolean;
    /**
     * Gets or sets a boolean indicating if multiplication of texture with level should be disabled
     */
    disableLevelMultiplication: boolean;
    /**
     * Create a new TriPlanarBlock
     * @param name defines the block name
     */
    constructor(name: string, hideSourceZ?: boolean);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the position input component
     */
    get position(): NodeMaterialConnectionPoint;
    /**
     * Gets the normal input component
     */
    get normal(): NodeMaterialConnectionPoint;
    /**
     * Gets the sharpness input component
     */
    get sharpness(): NodeMaterialConnectionPoint;
    /**
     * Gets the source input component
     */
    get source(): NodeMaterialConnectionPoint;
    /**
     * Gets the sourceY input component
     */
    get sourceY(): NodeMaterialConnectionPoint;
    /**
     * Gets the sourceZ input component
     */
    get sourceZ(): Nullable<NodeMaterialConnectionPoint>;
    /**
     * Gets the rgba output component
     */
    get rgba(): NodeMaterialConnectionPoint;
    /**
     * Gets the rgb output component
     */
    get rgb(): NodeMaterialConnectionPoint;
    /**
     * Gets the r output component
     */
    get r(): NodeMaterialConnectionPoint;
    /**
     * Gets the g output component
     */
    get g(): NodeMaterialConnectionPoint;
    /**
     * Gets the b output component
     */
    get b(): NodeMaterialConnectionPoint;
    /**
     * Gets the a output component
     */
    get a(): NodeMaterialConnectionPoint;
    /**
     * Gets the level output component
     */
    get level(): NodeMaterialConnectionPoint;
    prepareDefines(mesh: AbstractMesh, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines): void;
    isReady(): boolean;
    bind(effect: Effect): void;
    protected _generateTextureLookup(state: NodeMaterialBuildState): void;
    private _generateConversionCode;
    private _writeOutput;
    protected _buildBlock(state: NodeMaterialBuildState): this;
    protected _dumpPropertiesCode(): string;
    serialize(): any;
    _deserialize(serializationObject: any, scene: Scene, rootUrl: string): void;
}

}
declare module "babylonjs/Materials/Node/Blocks/vectorMergerBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
import { Scene } from "babylonjs/scene";
/**
 * Block used to create a Vector2/3/4 out of individual inputs (one for each component)
 */
export class VectorMergerBlock extends NodeMaterialBlock {
    /**
     * Gets or sets the swizzle for x (meaning which component to affect to the output.x)
     */
    xSwizzle: "x" | "y" | "z" | "w";
    /**
     * Gets or sets the swizzle for y (meaning which component to affect to the output.y)
     */
    ySwizzle: "x" | "y" | "z" | "w";
    /**
     * Gets or sets the swizzle for z (meaning which component to affect to the output.z)
     */
    zSwizzle: "x" | "y" | "z" | "w";
    /**
     * Gets or sets the swizzle for w (meaning which component to affect to the output.w)
     */
    wSwizzle: "x" | "y" | "z" | "w";
    /**
     * Create a new VectorMergerBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the xyzw component (input)
     */
    get xyzwIn(): NodeMaterialConnectionPoint;
    /**
     * Gets the xyz component (input)
     */
    get xyzIn(): NodeMaterialConnectionPoint;
    /**
     * Gets the xy component (input)
     */
    get xyIn(): NodeMaterialConnectionPoint;
    /**
     * Gets the zw component (input)
     */
    get zwIn(): NodeMaterialConnectionPoint;
    /**
     * Gets the x component (input)
     */
    get x(): NodeMaterialConnectionPoint;
    /**
     * Gets the y component (input)
     */
    get y(): NodeMaterialConnectionPoint;
    /**
     * Gets the z component (input)
     */
    get z(): NodeMaterialConnectionPoint;
    /**
     * Gets the w component (input)
     */
    get w(): NodeMaterialConnectionPoint;
    /**
     * Gets the xyzw component (output)
     */
    get xyzw(): NodeMaterialConnectionPoint;
    /**
     * Gets the xyz component (output)
     */
    get xyzOut(): NodeMaterialConnectionPoint;
    /**
     * Gets the xy component (output)
     */
    get xyOut(): NodeMaterialConnectionPoint;
    /**
     * Gets the zw component (output)
     */
    get zwOut(): NodeMaterialConnectionPoint;
    /**
     * Gets the xy component (output)
     * @deprecated Please use xyOut instead.
     */
    get xy(): NodeMaterialConnectionPoint;
    /**
     * Gets the xyz component (output)
     * @deprecated Please use xyzOut instead.
     */
    get xyz(): NodeMaterialConnectionPoint;
    protected _inputRename(name: string): string;
    private _buildSwizzle;
    protected _buildBlock(state: NodeMaterialBuildState): this;
    serialize(): any;
    _deserialize(serializationObject: any, scene: Scene, rootUrl: string): void;
    protected _dumpPropertiesCode(): string;
}

}
declare module "babylonjs/Materials/Node/Blocks/vectorSplitterBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
/**
 * Block used to expand a Vector3/4 into 4 outputs (one for each component)
 */
export class VectorSplitterBlock extends NodeMaterialBlock {
    /**
     * Create a new VectorSplitterBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the xyzw component (input)
     */
    get xyzw(): NodeMaterialConnectionPoint;
    /**
     * Gets the xyz component (input)
     */
    get xyzIn(): NodeMaterialConnectionPoint;
    /**
     * Gets the xy component (input)
     */
    get xyIn(): NodeMaterialConnectionPoint;
    /**
     * Gets the xyz component (output)
     */
    get xyzOut(): NodeMaterialConnectionPoint;
    /**
     * Gets the xy component (output)
     */
    get xyOut(): NodeMaterialConnectionPoint;
    /**
     * Gets the zw component (output)
     */
    get zw(): NodeMaterialConnectionPoint;
    /**
     * Gets the x component (output)
     */
    get x(): NodeMaterialConnectionPoint;
    /**
     * Gets the y component (output)
     */
    get y(): NodeMaterialConnectionPoint;
    /**
     * Gets the z component (output)
     */
    get z(): NodeMaterialConnectionPoint;
    /**
     * Gets the w component (output)
     */
    get w(): NodeMaterialConnectionPoint;
    protected _inputRename(name: string): string;
    protected _outputRename(name: string): string;
    protected _buildBlock(state: NodeMaterialBuildState): this;
}

}
declare module "babylonjs/Materials/Node/Blocks/Vertex/bonesBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { Mesh } from "babylonjs/Meshes/mesh";
import { Effect } from "babylonjs/Materials/effect";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
import { NodeMaterial, NodeMaterialDefines } from "babylonjs/Materials/Node/nodeMaterial";
import "babylonjs/Shaders/ShadersInclude/bonesDeclaration";
import "babylonjs/Shaders/ShadersInclude/bonesVertex";
import { EffectFallbacks } from "babylonjs/Materials/effectFallbacks";
/**
 * Block used to add support for vertex skinning (bones)
 */
export class BonesBlock extends NodeMaterialBlock {
    /**
     * Creates a new BonesBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Initialize the block and prepare the context for build
     * @param state defines the state that will be used for the build
     */
    initialize(state: NodeMaterialBuildState): void;
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the matrix indices input component
     */
    get matricesIndices(): NodeMaterialConnectionPoint;
    /**
     * Gets the matrix weights input component
     */
    get matricesWeights(): NodeMaterialConnectionPoint;
    /**
     * Gets the extra matrix indices input component
     */
    get matricesIndicesExtra(): NodeMaterialConnectionPoint;
    /**
     * Gets the extra matrix weights input component
     */
    get matricesWeightsExtra(): NodeMaterialConnectionPoint;
    /**
     * Gets the world input component
     */
    get world(): NodeMaterialConnectionPoint;
    /**
     * Gets the output component
     */
    get output(): NodeMaterialConnectionPoint;
    autoConfigure(material: NodeMaterial): void;
    provideFallbacks(mesh: AbstractMesh, fallbacks: EffectFallbacks): void;
    bind(effect: Effect, nodeMaterial: NodeMaterial, mesh?: Mesh): void;
    prepareDefines(mesh: AbstractMesh, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines): void;
    protected _buildBlock(state: NodeMaterialBuildState): this;
}

}
declare module "babylonjs/Materials/Node/Blocks/Vertex/index" {
export * from "babylonjs/Materials/Node/Blocks/Vertex/vertexOutputBlock";
export * from "babylonjs/Materials/Node/Blocks/Vertex/bonesBlock";
export * from "babylonjs/Materials/Node/Blocks/Vertex/instancesBlock";
export * from "babylonjs/Materials/Node/Blocks/Vertex/morphTargetsBlock";
export * from "babylonjs/Materials/Node/Blocks/Vertex/lightInformationBlock";

}
declare module "babylonjs/Materials/Node/Blocks/Vertex/instancesBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { NodeMaterial, NodeMaterialDefines } from "babylonjs/Materials/Node/nodeMaterial";
import { SubMesh } from "babylonjs/Meshes/subMesh";
/**
 * Block used to add support for instances
 * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/copies/instances
 */
export class InstancesBlock extends NodeMaterialBlock {
    /**
     * Creates a new InstancesBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the first world row input component
     */
    get world0(): NodeMaterialConnectionPoint;
    /**
     * Gets the second world row input component
     */
    get world1(): NodeMaterialConnectionPoint;
    /**
     * Gets the third world row input component
     */
    get world2(): NodeMaterialConnectionPoint;
    /**
     * Gets the forth world row input component
     */
    get world3(): NodeMaterialConnectionPoint;
    /**
     * Gets the world input component
     */
    get world(): NodeMaterialConnectionPoint;
    /**
     * Gets the output component
     */
    get output(): NodeMaterialConnectionPoint;
    /**
     * Gets the instanceID component
     */
    get instanceID(): NodeMaterialConnectionPoint;
    autoConfigure(material: NodeMaterial): void;
    prepareDefines(mesh: AbstractMesh, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines, useInstances?: boolean, subMesh?: SubMesh): void;
    protected _buildBlock(state: NodeMaterialBuildState): this;
}

}
declare module "babylonjs/Materials/Node/Blocks/Vertex/lightInformationBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
import { Nullable } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
import { Effect } from "babylonjs/Materials/effect";
import { NodeMaterial, NodeMaterialDefines } from "babylonjs/Materials/Node/nodeMaterial";
import { Mesh } from "babylonjs/Meshes/mesh";
import { Light } from "babylonjs/Lights/light";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
/**
 * Block used to get data information from a light
 */
export class LightInformationBlock extends NodeMaterialBlock {
    private _lightDataUniformName;
    private _lightColorUniformName;
    private _lightShadowUniformName;
    private _lightShadowExtraUniformName;
    private _lightTypeDefineName;
    private _forcePrepareDefines;
    /**
     * Gets or sets the light associated with this block
     */
    light: Nullable<Light>;
    /**
     * Creates a new LightInformationBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the world position input component
     */
    get worldPosition(): NodeMaterialConnectionPoint;
    /**
     * Gets the direction output component
     */
    get direction(): NodeMaterialConnectionPoint;
    /**
     * Gets the direction output component
     */
    get color(): NodeMaterialConnectionPoint;
    /**
     * Gets the direction output component
     */
    get intensity(): NodeMaterialConnectionPoint;
    /**
     * Gets the shadow bias output component
     */
    get shadowBias(): NodeMaterialConnectionPoint;
    /**
     * Gets the shadow normal bias output component
     */
    get shadowNormalBias(): NodeMaterialConnectionPoint;
    /**
     * Gets the shadow depth scale component
     */
    get shadowDepthScale(): NodeMaterialConnectionPoint;
    /**
     * Gets the shadow depth range component
     */
    get shadowDepthRange(): NodeMaterialConnectionPoint;
    bind(effect: Effect, nodeMaterial: NodeMaterial, mesh?: Mesh): void;
    prepareDefines(mesh: AbstractMesh, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines): void;
    protected _buildBlock(state: NodeMaterialBuildState): this;
    serialize(): any;
    _deserialize(serializationObject: any, scene: Scene, rootUrl: string): void;
}

}
declare module "babylonjs/Materials/Node/Blocks/Vertex/morphTargetsBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { NodeMaterial, NodeMaterialDefines } from "babylonjs/Materials/Node/nodeMaterial";
import { Effect } from "babylonjs/Materials/effect";
import { Mesh } from "babylonjs/Meshes/mesh";
import "babylonjs/Shaders/ShadersInclude/morphTargetsVertexDeclaration";
import "babylonjs/Shaders/ShadersInclude/morphTargetsVertexGlobalDeclaration";
/**
 * Block used to add morph targets support to vertex shader
 */
export class MorphTargetsBlock extends NodeMaterialBlock {
    private _repeatableContentAnchor;
    /**
     * Create a new MorphTargetsBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the position input component
     */
    get position(): NodeMaterialConnectionPoint;
    /**
     * Gets the normal input component
     */
    get normal(): NodeMaterialConnectionPoint;
    /**
     * Gets the tangent input component
     */
    get tangent(): NodeMaterialConnectionPoint;
    /**
     * Gets the tangent input component
     */
    get uv(): NodeMaterialConnectionPoint;
    /**
     * Gets the position output component
     */
    get positionOutput(): NodeMaterialConnectionPoint;
    /**
     * Gets the normal output component
     */
    get normalOutput(): NodeMaterialConnectionPoint;
    /**
     * Gets the tangent output component
     */
    get tangentOutput(): NodeMaterialConnectionPoint;
    /**
     * Gets the tangent output component
     */
    get uvOutput(): NodeMaterialConnectionPoint;
    initialize(state: NodeMaterialBuildState): void;
    autoConfigure(material: NodeMaterial): void;
    prepareDefines(mesh: AbstractMesh, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines): void;
    bind(effect: Effect, nodeMaterial: NodeMaterial, mesh?: Mesh): void;
    replaceRepeatableContent(vertexShaderState: NodeMaterialBuildState, fragmentShaderState: NodeMaterialBuildState, mesh: AbstractMesh, defines: NodeMaterialDefines): void;
    protected _buildBlock(state: NodeMaterialBuildState): this;
}

}
declare module "babylonjs/Materials/Node/Blocks/Vertex/vertexOutputBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
/**
 * Block used to output the vertex position
 */
export class VertexOutputBlock extends NodeMaterialBlock {
    /**
     * Creates a new VertexOutputBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the vector input component
     */
    get vector(): NodeMaterialConnectionPoint;
    private _isLogarithmicDepthEnabled;
    protected _buildBlock(state: NodeMaterialBuildState): this;
}

}
declare module "babylonjs/Materials/Node/Blocks/viewDirectionBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
import { NodeMaterial } from "babylonjs/Materials/Node/nodeMaterial";
/**
 * Block used to get the view direction
 */
export class ViewDirectionBlock extends NodeMaterialBlock {
    /**
     * Creates a new ViewDirectionBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the world position component
     */
    get worldPosition(): NodeMaterialConnectionPoint;
    /**
     * Gets the camera position component
     */
    get cameraPosition(): NodeMaterialConnectionPoint;
    /**
     * Gets the output component
     */
    get output(): NodeMaterialConnectionPoint;
    autoConfigure(material: NodeMaterial): void;
    protected _buildBlock(state: NodeMaterialBuildState): this;
}

}
declare module "babylonjs/Materials/Node/Blocks/voronoiNoiseBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
/**
 * block used to Generate a Voronoi Noise Pattern
 */
export class VoronoiNoiseBlock extends NodeMaterialBlock {
    /**
     * Creates a new VoronoiNoiseBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the seed input component
     */
    get seed(): NodeMaterialConnectionPoint;
    /**
     * Gets the offset input component
     */
    get offset(): NodeMaterialConnectionPoint;
    /**
     * Gets the density input component
     */
    get density(): NodeMaterialConnectionPoint;
    /**
     * Gets the output component
     */
    get output(): NodeMaterialConnectionPoint;
    /**
     * Gets the output component
     */
    get cells(): NodeMaterialConnectionPoint;
    protected _buildBlock(state: NodeMaterialBuildState): this | undefined;
}

}
declare module "babylonjs/Materials/Node/Blocks/waveBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
import { Scene } from "babylonjs/scene";
/**
 * Operations supported by the Wave block
 */
export enum WaveBlockKind {
    /** SawTooth */
    SawTooth = 0,
    /** Square */
    Square = 1,
    /** Triangle */
    Triangle = 2
}
/**
 * Block used to apply wave operation to floats
 */
export class WaveBlock extends NodeMaterialBlock {
    /**
     * Gets or sets the kibnd of wave to be applied by the block
     */
    kind: WaveBlockKind;
    /**
     * Creates a new WaveBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the input component
     */
    get input(): NodeMaterialConnectionPoint;
    /**
     * Gets the output component
     */
    get output(): NodeMaterialConnectionPoint;
    protected _buildBlock(state: NodeMaterialBuildState): this;
    serialize(): any;
    _deserialize(serializationObject: any, scene: Scene, rootUrl: string): void;
}

}
declare module "babylonjs/Materials/Node/Blocks/worleyNoise3DBlock" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
import { Scene } from "babylonjs/scene";
/**
 * block used to Generate a Worley Noise 3D Noise Pattern
 */
export class WorleyNoise3DBlock extends NodeMaterialBlock {
    /** Gets or sets a boolean indicating that normal should be inverted on X axis */
    manhattanDistance: boolean;
    /**
     * Creates a new WorleyNoise3DBlock
     * @param name defines the block name
     */
    constructor(name: string);
    /**
     * Gets the current class name
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets the seed input component
     */
    get seed(): NodeMaterialConnectionPoint;
    /**
     * Gets the jitter input component
     */
    get jitter(): NodeMaterialConnectionPoint;
    /**
     * Gets the output component
     */
    get output(): NodeMaterialConnectionPoint;
    /**
     * Gets the x component
     */
    get x(): NodeMaterialConnectionPoint;
    /**
     * Gets the y component
     */
    get y(): NodeMaterialConnectionPoint;
    protected _buildBlock(state: NodeMaterialBuildState): this | undefined;
    /**
     * Exposes the properties to the UI?
     */
    protected _dumpPropertiesCode(): string;
    /**
     * Exposes the properties to the Serialize?
     */
    serialize(): any;
    /**
     * Exposes the properties to the deserialize?
     * @param serializationObject
     * @param scene
     * @param rootUrl
     */
    _deserialize(serializationObject: any, scene: Scene, rootUrl: string): void;
}

}
declare module "babylonjs/Materials/Node/Enums/index" {
export * from "babylonjs/Materials/Node/Enums/nodeMaterialBlockTargets";
export * from "babylonjs/Materials/Node/Enums/nodeMaterialBlockConnectionPointTypes";
export * from "babylonjs/Materials/Node/Enums/nodeMaterialBlockConnectionPointMode";
export * from "babylonjs/Materials/Node/Enums/nodeMaterialSystemValues";
export * from "babylonjs/Materials/Node/Enums/nodeMaterialModes";

}
declare module "babylonjs/Materials/Node/Enums/nodeMaterialBlockConnectionPointMode" {
/**
 * Enum defining the mode of a NodeMaterialBlockConnectionPoint
 */
export enum NodeMaterialBlockConnectionPointMode {
    /** Value is an uniform */
    Uniform = 0,
    /** Value is a mesh attribute */
    Attribute = 1,
    /** Value is a varying between vertex and fragment shaders */
    Varying = 2,
    /** Mode is undefined */
    Undefined = 3
}

}
declare module "babylonjs/Materials/Node/Enums/nodeMaterialBlockConnectionPointTypes" {
/**
 * Defines the kind of connection point for node based material
 */
export enum NodeMaterialBlockConnectionPointTypes {
    /** Float */
    Float = 1,
    /** Int */
    Int = 2,
    /** Vector2 */
    Vector2 = 4,
    /** Vector3 */
    Vector3 = 8,
    /** Vector4 */
    Vector4 = 16,
    /** Color3 */
    Color3 = 32,
    /** Color4 */
    Color4 = 64,
    /** Matrix */
    Matrix = 128,
    /** Custom object */
    Object = 256,
    /** Detect type based on connection */
    AutoDetect = 1024,
    /** Output type that will be defined by input type */
    BasedOnInput = 2048,
    /** Bitmask of all types */
    All = 4095
}

}
declare module "babylonjs/Materials/Node/Enums/nodeMaterialBlockTargets" {
/**
 * Enum used to define the target of a block
 */
export enum NodeMaterialBlockTargets {
    /** Vertex shader */
    Vertex = 1,
    /** Fragment shader */
    Fragment = 2,
    /** Neutral */
    Neutral = 4,
    /** Vertex and Fragment */
    VertexAndFragment = 3
}

}
declare module "babylonjs/Materials/Node/Enums/nodeMaterialModes" {
/**
 * Enum used to define the material modes
 */
export enum NodeMaterialModes {
    /** Regular material */
    Material = 0,
    /** For post process */
    PostProcess = 1,
    /** For particle system */
    Particle = 2,
    /** For procedural texture */
    ProceduralTexture = 3
}

}
declare module "babylonjs/Materials/Node/Enums/nodeMaterialSystemValues" {
/**
 * Enum used to define system values e.g. values automatically provided by the system
 */
export enum NodeMaterialSystemValues {
    /** World */
    World = 1,
    /** View */
    View = 2,
    /** Projection */
    Projection = 3,
    /** ViewProjection */
    ViewProjection = 4,
    /** WorldView */
    WorldView = 5,
    /** WorldViewProjection */
    WorldViewProjection = 6,
    /** CameraPosition */
    CameraPosition = 7,
    /** Fog Color */
    FogColor = 8,
    /** Delta time */
    DeltaTime = 9,
    /** Camera parameters */
    CameraParameters = 10,
    /** Material alpha */
    MaterialAlpha = 11
}

}
declare module "babylonjs/Materials/Node/index" {
export * from "babylonjs/Materials/Node/Enums/index";
export * from "babylonjs/Materials/Node/nodeMaterialConnectionPointCustomObject";
export * from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
export * from "babylonjs/Materials/Node/nodeMaterialBlock";
export * from "babylonjs/Materials/Node/nodeMaterial";
export * from "babylonjs/Materials/Node/Blocks/index";
export * from "babylonjs/Materials/Node/Optimizers/index";
export * from "babylonjs/Materials/Node/nodeMaterialDecorator";

}
declare module "babylonjs/Materials/Node/nodeMaterial" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { PushMaterial } from "babylonjs/Materials/pushMaterial";
import { Scene } from "babylonjs/scene";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { Matrix } from "babylonjs/Maths/math.vector";
import { Mesh } from "babylonjs/Meshes/mesh";
import { Engine } from "babylonjs/Engines/engine";
import { Effect } from "babylonjs/Materials/effect";
import { BaseTexture } from "babylonjs/Materials/Textures/baseTexture";
import { Observable } from "babylonjs/Misc/observable";
import { SubMesh } from "babylonjs/Meshes/subMesh";
import { MaterialDefines } from "babylonjs/Materials/materialDefines";
import { NodeMaterialOptimizer } from "babylonjs/Materials/Node/Optimizers/nodeMaterialOptimizer";
import { ImageProcessingConfiguration, IImageProcessingConfigurationDefines } from "babylonjs/Materials/imageProcessingConfiguration";
import { Nullable } from "babylonjs/types";
import { InputBlock } from "babylonjs/Materials/Node/Blocks/Input/inputBlock";
import { TextureBlock } from "babylonjs/Materials/Node/Blocks/Dual/textureBlock";
import { ReflectionTextureBaseBlock } from "babylonjs/Materials/Node/Blocks/Dual/reflectionTextureBaseBlock";
import { RefractionBlock } from "babylonjs/Materials/Node/Blocks/PBR/refractionBlock";
import { CurrentScreenBlock } from "babylonjs/Materials/Node/Blocks/Dual/currentScreenBlock";
import { ParticleTextureBlock } from "babylonjs/Materials/Node/Blocks/Particle/particleTextureBlock";
import { PostProcessOptions } from "babylonjs/PostProcesses/postProcess";
import { PostProcess } from "babylonjs/PostProcesses/postProcess";
import { Camera } from "babylonjs/Cameras/camera";
import { NodeMaterialModes } from "babylonjs/Materials/Node/Enums/nodeMaterialModes";
import { IParticleSystem } from "babylonjs/Particles/IParticleSystem";
import { ProceduralTexture } from "babylonjs/Materials/Textures/Procedurals/proceduralTexture";
import { ImageSourceBlock } from "babylonjs/Materials/Node/Blocks/Dual/imageSourceBlock";
import { Material } from "babylonjs/Materials/material";
import { TriPlanarBlock } from "babylonjs/Materials/Node/Blocks/triPlanarBlock";
import { BiPlanarBlock } from "babylonjs/Materials/Node/Blocks/biPlanarBlock";
/**
 * Interface used to configure the node material editor
 */
export interface INodeMaterialEditorOptions {
    /** Define the URl to load node editor script */
    editorURL?: string;
}
/** @internal */
export class NodeMaterialDefines extends MaterialDefines implements IImageProcessingConfigurationDefines {
    NORMAL: boolean;
    TANGENT: boolean;
    UV1: boolean;
    UV2: boolean;
    UV3: boolean;
    UV4: boolean;
    UV5: boolean;
    UV6: boolean;
    /** BONES */
    NUM_BONE_INFLUENCERS: number;
    BonesPerMesh: number;
    BONETEXTURE: boolean;
    /** MORPH TARGETS */
    MORPHTARGETS: boolean;
    MORPHTARGETS_NORMAL: boolean;
    MORPHTARGETS_TANGENT: boolean;
    MORPHTARGETS_UV: boolean;
    NUM_MORPH_INFLUENCERS: number;
    MORPHTARGETS_TEXTURE: boolean;
    /** IMAGE PROCESSING */
    IMAGEPROCESSING: boolean;
    VIGNETTE: boolean;
    VIGNETTEBLENDMODEMULTIPLY: boolean;
    VIGNETTEBLENDMODEOPAQUE: boolean;
    TONEMAPPING: boolean;
    TONEMAPPING_ACES: boolean;
    CONTRAST: boolean;
    EXPOSURE: boolean;
    COLORCURVES: boolean;
    COLORGRADING: boolean;
    COLORGRADING3D: boolean;
    SAMPLER3DGREENDEPTH: boolean;
    SAMPLER3DBGRMAP: boolean;
    DITHER: boolean;
    IMAGEPROCESSINGPOSTPROCESS: boolean;
    SKIPFINALCOLORCLAMP: boolean;
    /** MISC. */
    BUMPDIRECTUV: number;
    CAMERA_ORTHOGRAPHIC: boolean;
    CAMERA_PERSPECTIVE: boolean;
    constructor();
    setValue(name: string, value: any, markAsUnprocessedIfDirty?: boolean): void;
}
/**
 * Class used to configure NodeMaterial
 */
export interface INodeMaterialOptions {
    /**
     * Defines if blocks should emit comments
     */
    emitComments: boolean;
}
/**
 * Blocks that manage a texture
 */
export type NodeMaterialTextureBlocks = TextureBlock | ReflectionTextureBaseBlock | RefractionBlock | CurrentScreenBlock | ParticleTextureBlock | ImageSourceBlock | TriPlanarBlock | BiPlanarBlock;
/**
 * Class used to create a node based material built by assembling shader blocks
 */
export class NodeMaterial extends PushMaterial {
    private static _BuildIdGenerator;
    private _options;
    private _vertexCompilationState;
    private _fragmentCompilationState;
    private _sharedData;
    private _buildId;
    private _buildWasSuccessful;
    private _cachedWorldViewMatrix;
    private _cachedWorldViewProjectionMatrix;
    private _optimizers;
    private _animationFrame;
    /** Define the Url to load node editor script */
    static EditorURL: string;
    /** Define the Url to load snippets */
    static SnippetUrl: string;
    /** Gets or sets a boolean indicating that node materials should not deserialize textures from json / snippet content */
    static IgnoreTexturesAtLoadTime: boolean;
    /**
     * Checks if a block is a texture block
     * @param block The block to check
     * @returns True if the block is a texture block
     */
    static _BlockIsTextureBlock(block: NodeMaterialBlock): block is NodeMaterialTextureBlocks;
    private BJSNODEMATERIALEDITOR;
    /** Get the inspector from bundle or global */
    private _getGlobalNodeMaterialEditor;
    /**
     * Snippet ID if the material was created from the snippet server
     */
    snippetId: string;
    /**
     * Gets or sets data used by visual editor
     * @see https://nme.babylonjs.com
     */
    editorData: any;
    /**
     * Gets or sets a boolean indicating that alpha value must be ignored (This will turn alpha blending off even if an alpha value is produced by the material)
     */
    ignoreAlpha: boolean;
    /**
     * Defines the maximum number of lights that can be used in the material
     */
    maxSimultaneousLights: number;
    /**
     * Observable raised when the material is built
     */
    onBuildObservable: Observable<NodeMaterial>;
    /**
     * Gets or sets the root nodes of the material vertex shader
     */
    _vertexOutputNodes: NodeMaterialBlock[];
    /**
     * Gets or sets the root nodes of the material fragment (pixel) shader
     */
    _fragmentOutputNodes: NodeMaterialBlock[];
    /** Gets or sets options to control the node material overall behavior */
    get options(): INodeMaterialOptions;
    set options(options: INodeMaterialOptions);
    /**
     * Default configuration related to image processing available in the standard Material.
     */
    protected _imageProcessingConfiguration: ImageProcessingConfiguration;
    /**
     * Gets the image processing configuration used either in this material.
     */
    get imageProcessingConfiguration(): ImageProcessingConfiguration;
    /**
     * Sets the Default image processing configuration used either in the this material.
     *
     * If sets to null, the scene one is in use.
     */
    set imageProcessingConfiguration(value: ImageProcessingConfiguration);
    /**
     * Gets an array of blocks that needs to be serialized even if they are not yet connected
     */
    attachedBlocks: NodeMaterialBlock[];
    /**
     * Specifies the mode of the node material
     * @internal
     */
    _mode: NodeMaterialModes;
    /**
     * Gets or sets the mode property
     */
    get mode(): NodeMaterialModes;
    set mode(value: NodeMaterialModes);
    /** Gets or sets the unique identifier used to identified the effect associated with the material */
    get buildId(): number;
    set buildId(value: number);
    /**
     * A free comment about the material
     */
    comment: string;
    /**
     * Create a new node based material
     * @param name defines the material name
     * @param scene defines the hosting scene
     * @param options defines creation option
     */
    constructor(name: string, scene?: Scene, options?: Partial<INodeMaterialOptions>);
    /**
     * Gets the current class name of the material e.g. "NodeMaterial"
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Keep track of the image processing observer to allow dispose and replace.
     */
    private _imageProcessingObserver;
    /**
     * Attaches a new image processing configuration to the Standard Material.
     * @param configuration
     */
    protected _attachImageProcessingConfiguration(configuration: Nullable<ImageProcessingConfiguration>): void;
    /**
     * Get a block by its name
     * @param name defines the name of the block to retrieve
     * @returns the required block or null if not found
     */
    getBlockByName(name: string): NodeMaterialBlock | null;
    /**
     * Get a block by its name
     * @param predicate defines the predicate used to find the good candidate
     * @returns the required block or null if not found
     */
    getBlockByPredicate(predicate: (block: NodeMaterialBlock) => boolean): NodeMaterialBlock | null;
    /**
     * Get an input block by its name
     * @param predicate defines the predicate used to find the good candidate
     * @returns the required input block or null if not found
     */
    getInputBlockByPredicate(predicate: (block: InputBlock) => boolean): Nullable<InputBlock>;
    /**
     * Gets the list of input blocks attached to this material
     * @returns an array of InputBlocks
     */
    getInputBlocks(): InputBlock[];
    /**
     * Adds a new optimizer to the list of optimizers
     * @param optimizer defines the optimizers to add
     * @returns the current material
     */
    registerOptimizer(optimizer: NodeMaterialOptimizer): this | undefined;
    /**
     * Remove an optimizer from the list of optimizers
     * @param optimizer defines the optimizers to remove
     * @returns the current material
     */
    unregisterOptimizer(optimizer: NodeMaterialOptimizer): this | undefined;
    /**
     * Add a new block to the list of output nodes
     * @param node defines the node to add
     * @returns the current material
     */
    addOutputNode(node: NodeMaterialBlock): this;
    /**
     * Remove a block from the list of root nodes
     * @param node defines the node to remove
     * @returns the current material
     */
    removeOutputNode(node: NodeMaterialBlock): this;
    private _addVertexOutputNode;
    private _removeVertexOutputNode;
    private _addFragmentOutputNode;
    private _removeFragmentOutputNode;
    /**
     * Gets or sets a boolean indicating that alpha blending must be enabled no matter what alpha value or alpha channel of the FragmentBlock are
     */
    forceAlphaBlending: boolean;
    /**
     * Specifies if the material will require alpha blending
     * @returns a boolean specifying if alpha blending is needed
     */
    needAlphaBlending(): boolean;
    /**
     * Specifies if this material should be rendered in alpha test mode
     * @returns a boolean specifying if an alpha test is needed.
     */
    needAlphaTesting(): boolean;
    private _initializeBlock;
    private _resetDualBlocks;
    /**
     * Remove a block from the current node material
     * @param block defines the block to remove
     */
    removeBlock(block: NodeMaterialBlock): void;
    /**
     * Build the material and generates the inner effect
     * @param verbose defines if the build should log activity
     * @param updateBuildId defines if the internal build Id should be updated (default is true)
     * @param autoConfigure defines if the autoConfigure method should be called when initializing blocks (default is true)
     */
    build(verbose?: boolean, updateBuildId?: boolean, autoConfigure?: boolean): void;
    /**
     * Runs an otpimization phase to try to improve the shader code
     */
    optimize(): void;
    private _prepareDefinesForAttributes;
    /**
     * Create a post process from the material
     * @param camera The camera to apply the render pass to.
     * @param options The required width/height ratio to downsize to before computing the render pass. (Use 1.0 for full size)
     * @param samplingMode The sampling mode to be used when computing the pass. (default: 0)
     * @param engine The engine which the post process will be applied. (default: current engine)
     * @param reusable If the post process can be reused on the same frame. (default: false)
     * @param textureType Type of textures used when performing the post process. (default: 0)
     * @param textureFormat Format of textures used when performing the post process. (default: TEXTUREFORMAT_RGBA)
     * @returns the post process created
     */
    createPostProcess(camera: Nullable<Camera>, options?: number | PostProcessOptions, samplingMode?: number, engine?: Engine, reusable?: boolean, textureType?: number, textureFormat?: number): Nullable<PostProcess>;
    /**
     * Create the post process effect from the material
     * @param postProcess The post process to create the effect for
     */
    createEffectForPostProcess(postProcess: PostProcess): void;
    private _createEffectForPostProcess;
    /**
     * Create a new procedural texture based on this node material
     * @param size defines the size of the texture
     * @param scene defines the hosting scene
     * @returns the new procedural texture attached to this node material
     */
    createProceduralTexture(size: number | {
        width: number;
        height: number;
        layers?: number;
    }, scene: Scene): Nullable<ProceduralTexture>;
    private _createEffectForParticles;
    private _checkInternals;
    /**
     * Create the effect to be used as the custom effect for a particle system
     * @param particleSystem Particle system to create the effect for
     * @param onCompiled defines a function to call when the effect creation is successful
     * @param onError defines a function to call when the effect creation has failed
     */
    createEffectForParticles(particleSystem: IParticleSystem, onCompiled?: (effect: Effect) => void, onError?: (effect: Effect, errors: string) => void): void;
    /**
     * Use this material as the shadow depth wrapper of a target material
     * @param targetMaterial defines the target material
     */
    createAsShadowDepthWrapper(targetMaterial: Material): void;
    private _processDefines;
    /**
     * Get if the submesh is ready to be used and all its information available.
     * Child classes can use it to update shaders
     * @param mesh defines the mesh to check
     * @param subMesh defines which submesh to check
     * @param useInstances specifies that instances should be used
     * @returns a boolean indicating that the submesh is ready or not
     */
    isReadyForSubMesh(mesh: AbstractMesh, subMesh: SubMesh, useInstances?: boolean): boolean;
    /**
     * Get a string representing the shaders built by the current node graph
     */
    get compiledShaders(): string;
    /**
     * Binds the world matrix to the material
     * @param world defines the world transformation matrix
     */
    bindOnlyWorldMatrix(world: Matrix): void;
    /**
     * Binds the submesh to this material by preparing the effect and shader to draw
     * @param world defines the world transformation matrix
     * @param mesh defines the mesh containing the submesh
     * @param subMesh defines the submesh to bind the material to
     */
    bindForSubMesh(world: Matrix, mesh: Mesh, subMesh: SubMesh): void;
    /**
     * Gets the active textures from the material
     * @returns an array of textures
     */
    getActiveTextures(): BaseTexture[];
    /**
     * Gets the list of texture blocks
     * Note that this method will only return blocks that are reachable from the final block(s) and only after the material has been built!
     * @returns an array of texture blocks
     */
    getTextureBlocks(): NodeMaterialTextureBlocks[];
    /**
     * Gets the list of all texture blocks
     * Note that this method will scan all attachedBlocks and return blocks that are texture blocks
     * @returns
     */
    getAllTextureBlocks(): NodeMaterialTextureBlocks[];
    /**
     * Specifies if the material uses a texture
     * @param texture defines the texture to check against the material
     * @returns a boolean specifying if the material uses the texture
     */
    hasTexture(texture: BaseTexture): boolean;
    /**
     * Disposes the material
     * @param forceDisposeEffect specifies if effects should be forcefully disposed
     * @param forceDisposeTextures specifies if textures should be forcefully disposed
     * @param notBoundToMesh specifies if the material that is being disposed is known to be not bound to any mesh
     */
    dispose(forceDisposeEffect?: boolean, forceDisposeTextures?: boolean, notBoundToMesh?: boolean): void;
    /** Creates the node editor window. */
    private _createNodeEditor;
    /**
     * Launch the node material editor
     * @param config Define the configuration of the editor
     * @returns a promise fulfilled when the node editor is visible
     */
    edit(config?: INodeMaterialEditorOptions): Promise<void>;
    /**
     * Clear the current material
     */
    clear(): void;
    /**
     * Clear the current material and set it to a default state
     */
    setToDefault(): void;
    /**
     * Clear the current material and set it to a default state for post process
     */
    setToDefaultPostProcess(): void;
    /**
     * Clear the current material and set it to a default state for procedural texture
     */
    setToDefaultProceduralTexture(): void;
    /**
     * Clear the current material and set it to a default state for particle
     */
    setToDefaultParticle(): void;
    /**
     * Loads the current Node Material from a url pointing to a file save by the Node Material Editor
     * @deprecated Please use NodeMaterial.ParseFromFileAsync instead
     * @param url defines the url to load from
     * @param rootUrl defines the root URL for nested url in the node material
     * @returns a promise that will fulfil when the material is fully loaded
     */
    loadAsync(url: string, rootUrl?: string): Promise<NodeMaterial>;
    private _gatherBlocks;
    /**
     * Generate a string containing the code declaration required to create an equivalent of this material
     * @returns a string
     */
    generateCode(): string;
    /**
     * Serializes this material in a JSON representation
     * @param selectedBlocks
     * @returns the serialized material object
     */
    serialize(selectedBlocks?: NodeMaterialBlock[]): any;
    private _restoreConnections;
    /**
     * Clear the current graph and load a new one from a serialization object
     * @param source defines the JSON representation of the material
     * @param rootUrl defines the root URL to use to load textures and relative dependencies
     * @param merge defines whether or not the source must be merged or replace the current content
     */
    parseSerializedObject(source: any, rootUrl?: string, merge?: boolean): void;
    /**
     * Clear the current graph and load a new one from a serialization object
     * @param source defines the JSON representation of the material
     * @param rootUrl defines the root URL to use to load textures and relative dependencies
     * @param merge defines whether or not the source must be merged or replace the current content
     * @deprecated Please use the parseSerializedObject method instead
     */
    loadFromSerialization(source: any, rootUrl?: string, merge?: boolean): void;
    /**
     * Makes a duplicate of the current material.
     * @param name defines the name to use for the new material
     * @param shareEffect defines if the clone material should share the same effect (default is false)
     */
    clone(name: string, shareEffect?: boolean): NodeMaterial;
    /**
     * Creates a node material from parsed material data
     * @param source defines the JSON representation of the material
     * @param scene defines the hosting scene
     * @param rootUrl defines the root URL to use to load textures and relative dependencies
     * @returns a new node material
     */
    static Parse(source: any, scene: Scene, rootUrl?: string): NodeMaterial;
    /**
     * Creates a node material from a snippet saved in a remote file
     * @param name defines the name of the material to create
     * @param url defines the url to load from
     * @param scene defines the hosting scene
     * @param rootUrl defines the root URL for nested url in the node material
     * @param skipBuild defines whether to build the node material
     * @param targetMaterial defines a material to use instead of creating a new one
     * @returns a promise that will resolve to the new node material
     */
    static ParseFromFileAsync(name: string, url: string, scene: Scene, rootUrl?: string, skipBuild?: boolean, targetMaterial?: NodeMaterial): Promise<NodeMaterial>;
    /**
     * Creates a node material from a snippet saved by the node material editor
     * @param snippetId defines the snippet to load
     * @param scene defines the hosting scene
     * @param rootUrl defines the root URL to use to load textures and relative dependencies
     * @param nodeMaterial defines a node material to update (instead of creating a new one)
     * @param skipBuild defines whether to build the node material
     * @returns a promise that will resolve to the new node material
     */
    static ParseFromSnippetAsync(snippetId: string, scene?: Scene, rootUrl?: string, nodeMaterial?: NodeMaterial, skipBuild?: boolean): Promise<NodeMaterial>;
    /**
     * Creates a new node material set to default basic configuration
     * @param name defines the name of the material
     * @param scene defines the hosting scene
     * @returns a new NodeMaterial
     */
    static CreateDefault(name: string, scene?: Scene): NodeMaterial;
}

}
declare module "babylonjs/Materials/Node/nodeMaterialBlock" {
import { NodeMaterialBlockConnectionPointTypes } from "babylonjs/Materials/Node/Enums/nodeMaterialBlockConnectionPointTypes";
import { NodeMaterialBuildState } from "babylonjs/Materials/Node/nodeMaterialBuildState";
import { Nullable } from "babylonjs/types";
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
import { NodeMaterialBlockTargets } from "babylonjs/Materials/Node/Enums/nodeMaterialBlockTargets";
import { Effect } from "babylonjs/Materials/effect";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { Mesh } from "babylonjs/Meshes/mesh";
import { SubMesh } from "babylonjs/Meshes/subMesh";
import { NodeMaterial, NodeMaterialDefines } from "babylonjs/Materials/Node/nodeMaterial";
import { Scene } from "babylonjs/scene";
import { EffectFallbacks } from "babylonjs/Materials/effectFallbacks";
/**
 * Defines a block that can be used inside a node based material
 */
export class NodeMaterialBlock {
    private _buildId;
    private _buildTarget;
    protected _target: NodeMaterialBlockTargets;
    private _isFinalMerger;
    private _isInput;
    private _name;
    protected _isUnique: boolean;
    /** Gets or sets a boolean indicating that only one input can be connected at a time */
    inputsAreExclusive: boolean;
    /** @internal */
    _codeVariableName: string;
    /** @internal */
    _inputs: NodeMaterialConnectionPoint[];
    /** @internal */
    _outputs: NodeMaterialConnectionPoint[];
    /** @internal */
    _preparationId: number;
    /** @internal */
    readonly _originalTargetIsNeutral: boolean;
    /**
     * Gets the name of the block
     */
    get name(): string;
    /**
     * Sets the name of the block. Will check if the name is valid.
     */
    set name(newName: string);
    /**
     * Gets or sets the unique id of the node
     */
    uniqueId: number;
    /**
     * Gets or sets the comments associated with this block
     */
    comments: string;
    /**
     * Gets a boolean indicating that this block can only be used once per NodeMaterial
     */
    get isUnique(): boolean;
    /**
     * Gets a boolean indicating that this block is an end block (e.g. it is generating a system value)
     */
    get isFinalMerger(): boolean;
    /**
     * Gets a boolean indicating that this block is an input (e.g. it sends data to the shader)
     */
    get isInput(): boolean;
    /**
     * Gets or sets the build Id
     */
    get buildId(): number;
    set buildId(value: number);
    /**
     * Gets or sets the target of the block
     */
    get target(): NodeMaterialBlockTargets;
    set target(value: NodeMaterialBlockTargets);
    /**
     * Gets the list of input points
     */
    get inputs(): NodeMaterialConnectionPoint[];
    /** Gets the list of output points */
    get outputs(): NodeMaterialConnectionPoint[];
    /**
     * Find an input by its name
     * @param name defines the name of the input to look for
     * @returns the input or null if not found
     */
    getInputByName(name: string): NodeMaterialConnectionPoint | null;
    /**
     * Find an output by its name
     * @param name defines the name of the output to look for
     * @returns the output or null if not found
     */
    getOutputByName(name: string): NodeMaterialConnectionPoint | null;
    /** Gets or sets a boolean indicating that this input can be edited in the Inspector (false by default) */
    visibleInInspector: boolean;
    /** Gets or sets a boolean indicating that this input can be edited from a collapsed frame */
    visibleOnFrame: boolean;
    /**
     * Creates a new NodeMaterialBlock
     * @param name defines the block name
     * @param target defines the target of that block (Vertex by default)
     * @param isFinalMerger defines a boolean indicating that this block is an end block (e.g. it is generating a system value). Default is false
     * @param isInput defines a boolean indicating that this block is an input (e.g. it sends data to the shader). Default is false
     */
    constructor(name: string, target?: NodeMaterialBlockTargets, isFinalMerger?: boolean, isInput?: boolean);
    /** @internal */
    _setInitialTarget(target: NodeMaterialBlockTargets): void;
    /**
     * Initialize the block and prepare the context for build
     * @param state defines the state that will be used for the build
     */
    initialize(state: NodeMaterialBuildState): void;
    /**
     * Bind data to effect. Will only be called for blocks with isBindable === true
     * @param effect defines the effect to bind data to
     * @param nodeMaterial defines the hosting NodeMaterial
     * @param mesh defines the mesh that will be rendered
     * @param subMesh defines the submesh that will be rendered
     */
    bind(effect: Effect, nodeMaterial: NodeMaterial, mesh?: Mesh, subMesh?: SubMesh): void;
    protected _declareOutput(output: NodeMaterialConnectionPoint, state: NodeMaterialBuildState): string;
    protected _writeVariable(currentPoint: NodeMaterialConnectionPoint): string;
    protected _writeFloat(value: number): string;
    /**
     * Gets the current class name e.g. "NodeMaterialBlock"
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Register a new input. Must be called inside a block constructor
     * @param name defines the connection point name
     * @param type defines the connection point type
     * @param isOptional defines a boolean indicating that this input can be omitted
     * @param target defines the target to use to limit the connection point (will be VertexAndFragment by default)
     * @param point an already created connection point. If not provided, create a new one
     * @returns the current block
     */
    registerInput(name: string, type: NodeMaterialBlockConnectionPointTypes, isOptional?: boolean, target?: NodeMaterialBlockTargets, point?: NodeMaterialConnectionPoint): this;
    /**
     * Register a new output. Must be called inside a block constructor
     * @param name defines the connection point name
     * @param type defines the connection point type
     * @param target defines the target to use to limit the connection point (will be VertexAndFragment by default)
     * @param point an already created connection point. If not provided, create a new one
     * @returns the current block
     */
    registerOutput(name: string, type: NodeMaterialBlockConnectionPointTypes, target?: NodeMaterialBlockTargets, point?: NodeMaterialConnectionPoint): this;
    /**
     * Will return the first available input e.g. the first one which is not an uniform or an attribute
     * @param forOutput defines an optional connection point to check compatibility with
     * @returns the first available input or null
     */
    getFirstAvailableInput(forOutput?: Nullable<NodeMaterialConnectionPoint>): NodeMaterialConnectionPoint | null;
    /**
     * Will return the first available output e.g. the first one which is not yet connected and not a varying
     * @param forBlock defines an optional block to check compatibility with
     * @returns the first available input or null
     */
    getFirstAvailableOutput(forBlock?: Nullable<NodeMaterialBlock>): NodeMaterialConnectionPoint | null;
    /**
     * Gets the sibling of the given output
     * @param current defines the current output
     * @returns the next output in the list or null
     */
    getSiblingOutput(current: NodeMaterialConnectionPoint): NodeMaterialConnectionPoint | null;
    /**
     * Checks if the current block is an ancestor of a given block
     * @param block defines the potential descendant block to check
     * @returns true if block is a descendant
     */
    isAnAncestorOf(block: NodeMaterialBlock): boolean;
    /**
     * Connect current block with another block
     * @param other defines the block to connect with
     * @param options define the various options to help pick the right connections
     * @param options.input
     * @param options.output
     * @param options.outputSwizzle
     * @returns the current block
     */
    connectTo(other: NodeMaterialBlock, options?: {
        input?: string;
        output?: string;
        outputSwizzle?: string;
    }): this | undefined;
    protected _buildBlock(state: NodeMaterialBuildState): void;
    /**
     * Add uniforms, samplers and uniform buffers at compilation time
     * @param state defines the state to update
     * @param nodeMaterial defines the node material requesting the update
     * @param defines defines the material defines to update
     * @param uniformBuffers defines the list of uniform buffer names
     */
    updateUniformsAndSamples(state: NodeMaterialBuildState, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines, uniformBuffers: string[]): void;
    /**
     * Add potential fallbacks if shader compilation fails
     * @param mesh defines the mesh to be rendered
     * @param fallbacks defines the current prioritized list of fallbacks
     */
    provideFallbacks(mesh: AbstractMesh, fallbacks: EffectFallbacks): void;
    /**
     * Initialize defines for shader compilation
     * @param mesh defines the mesh to be rendered
     * @param nodeMaterial defines the node material requesting the update
     * @param defines defines the material defines to update
     * @param useInstances specifies that instances should be used
     */
    initializeDefines(mesh: AbstractMesh, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines, useInstances?: boolean): void;
    /**
     * Update defines for shader compilation
     * @param mesh defines the mesh to be rendered
     * @param nodeMaterial defines the node material requesting the update
     * @param defines defines the material defines to update
     * @param useInstances specifies that instances should be used
     * @param subMesh defines which submesh to render
     */
    prepareDefines(mesh: AbstractMesh, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines, useInstances?: boolean, subMesh?: SubMesh): void;
    /**
     * Lets the block try to connect some inputs automatically
     * @param material defines the hosting NodeMaterial
     */
    autoConfigure(material: NodeMaterial): void;
    /**
     * Function called when a block is declared as repeatable content generator
     * @param vertexShaderState defines the current compilation state for the vertex shader
     * @param fragmentShaderState defines the current compilation state for the fragment shader
     * @param mesh defines the mesh to be rendered
     * @param defines defines the material defines to update
     */
    replaceRepeatableContent(vertexShaderState: NodeMaterialBuildState, fragmentShaderState: NodeMaterialBuildState, mesh: AbstractMesh, defines: NodeMaterialDefines): void;
    /** Gets a boolean indicating that the code of this block will be promoted to vertex shader even if connected to fragment output */
    get willBeGeneratedIntoVertexShaderFromFragmentShader(): boolean;
    /**
     * Checks if the block is ready
     * @param mesh defines the mesh to be rendered
     * @param nodeMaterial defines the node material requesting the update
     * @param defines defines the material defines to update
     * @param useInstances specifies that instances should be used
     * @returns true if the block is ready
     */
    isReady(mesh: AbstractMesh, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines, useInstances?: boolean): boolean;
    protected _linkConnectionTypes(inputIndex0: number, inputIndex1: number, looseCoupling?: boolean): void;
    private _processBuild;
    /**
     * Validates the new name for the block node.
     * @param newName the new name to be given to the node.
     * @returns false if the name is a reserve word, else true.
     */
    validateBlockName(newName: string): boolean;
    /**
     * Compile the current node and generate the shader code
     * @param state defines the current compilation state (uniforms, samplers, current string)
     * @param activeBlocks defines the list of active blocks (i.e. blocks to compile)
     * @returns true if already built
     */
    build(state: NodeMaterialBuildState, activeBlocks: NodeMaterialBlock[]): boolean;
    protected _inputRename(name: string): string;
    protected _outputRename(name: string): string;
    protected _dumpPropertiesCode(): string;
    /**
     * @internal
     */
    _dumpCode(uniqueNames: string[], alreadyDumped: NodeMaterialBlock[]): string;
    /**
     * @internal
     */
    _dumpCodeForOutputConnections(alreadyDumped: NodeMaterialBlock[]): string;
    /**
     * Clone the current block to a new identical block
     * @param scene defines the hosting scene
     * @param rootUrl defines the root URL to use to load textures and relative dependencies
     * @returns a copy of the current block
     */
    clone(scene: Scene, rootUrl?: string): NodeMaterialBlock | null;
    /**
     * Serializes this block in a JSON representation
     * @returns the serialized block object
     */
    serialize(): any;
    /**
     * @internal
     */
    _deserialize(serializationObject: any, scene: Scene, rootUrl: string): void;
    private _deserializePortDisplayNamesAndExposedOnFrame;
    /**
     * Release resources
     */
    dispose(): void;
}

}
declare module "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint" {
import { NodeMaterialBlockConnectionPointTypes } from "babylonjs/Materials/Node/Enums/nodeMaterialBlockConnectionPointTypes";
import { NodeMaterialBlockTargets } from "babylonjs/Materials/Node/Enums/nodeMaterialBlockTargets";
import { Nullable } from "babylonjs/types";
import { InputBlock } from "babylonjs/Materials/Node/Blocks/Input/inputBlock";
import { Observable } from "babylonjs/Misc/observable";
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
/**
 * Enum used to define the compatibility state between two connection points
 */
export enum NodeMaterialConnectionPointCompatibilityStates {
    /** Points are compatibles */
    Compatible = 0,
    /** Points are incompatible because of their types */
    TypeIncompatible = 1,
    /** Points are incompatible because of their targets (vertex vs fragment) */
    TargetIncompatible = 2,
    /** Points are incompatible because they are in the same hierarchy **/
    HierarchyIssue = 3
}
/**
 * Defines the direction of a connection point
 */
export enum NodeMaterialConnectionPointDirection {
    /** Input */
    Input = 0,
    /** Output */
    Output = 1
}
/**
 * Defines a connection point for a block
 */
export class NodeMaterialConnectionPoint {
    /**
     * Checks if two types are equivalent
     * @param type1 type 1 to check
     * @param type2 type 2 to check
     * @returns true if both types are equivalent, else false
     */
    static AreEquivalentTypes(type1: number, type2: number): boolean;
    /** @internal */
    _ownerBlock: NodeMaterialBlock;
    /** @internal */
    _connectedPoint: Nullable<NodeMaterialConnectionPoint>;
    private _endpoints;
    private _associatedVariableName;
    private _direction;
    /** @internal */
    _typeConnectionSource: Nullable<NodeMaterialConnectionPoint>;
    /** @internal */
    _defaultConnectionPointType: Nullable<NodeMaterialBlockConnectionPointTypes>;
    /** @internal */
    _linkedConnectionSource: Nullable<NodeMaterialConnectionPoint>;
    /** @internal */
    _acceptedConnectionPointType: Nullable<NodeMaterialConnectionPoint>;
    private _type;
    /** @internal */
    _enforceAssociatedVariableName: boolean;
    /** Gets the direction of the point */
    get direction(): NodeMaterialConnectionPointDirection;
    /** Indicates that this connection point needs dual validation before being connected to another point */
    needDualDirectionValidation: boolean;
    /**
     * Gets or sets the additional types supported by this connection point
     */
    acceptedConnectionPointTypes: NodeMaterialBlockConnectionPointTypes[];
    /**
     * Gets or sets the additional types excluded by this connection point
     */
    excludedConnectionPointTypes: NodeMaterialBlockConnectionPointTypes[];
    /**
     * Observable triggered when this point is connected
     */
    onConnectionObservable: Observable<NodeMaterialConnectionPoint>;
    /**
     * Gets or sets the associated variable name in the shader
     */
    get associatedVariableName(): string;
    set associatedVariableName(value: string);
    /** Get the inner type (ie AutoDetect for instance instead of the inferred one) */
    get innerType(): NodeMaterialBlockConnectionPointTypes;
    /**
     * Gets or sets the connection point type (default is float)
     */
    get type(): NodeMaterialBlockConnectionPointTypes;
    set type(value: NodeMaterialBlockConnectionPointTypes);
    /**
     * Gets or sets the connection point name
     */
    name: string;
    /**
     * Gets or sets the connection point name
     */
    displayName: string;
    /**
     * Gets or sets a boolean indicating that this connection point can be omitted
     */
    isOptional: boolean;
    /**
     * Gets or sets a boolean indicating that this connection point is exposed on a frame
     */
    isExposedOnFrame: boolean;
    /**
     * Gets or sets number indicating the position that the port is exposed to on a frame
     */
    exposedPortPosition: number;
    /**
     * Gets or sets a string indicating that this uniform must be defined under a #ifdef
     */
    define: string;
    /** @internal */
    _prioritizeVertex: boolean;
    private _target;
    /** Gets or sets the target of that connection point */
    get target(): NodeMaterialBlockTargets;
    set target(value: NodeMaterialBlockTargets);
    /**
     * Gets a boolean indicating that the current point is connected to another NodeMaterialBlock
     */
    get isConnected(): boolean;
    /**
     * Gets a boolean indicating that the current point is connected to an input block
     */
    get isConnectedToInputBlock(): boolean;
    /**
     * Gets a the connected input block (if any)
     */
    get connectInputBlock(): Nullable<InputBlock>;
    /** Get the other side of the connection (if any) */
    get connectedPoint(): Nullable<NodeMaterialConnectionPoint>;
    /** Get the block that owns this connection point */
    get ownerBlock(): NodeMaterialBlock;
    /** Get the block connected on the other side of this connection (if any) */
    get sourceBlock(): Nullable<NodeMaterialBlock>;
    /** Get the block connected on the endpoints of this connection (if any) */
    get connectedBlocks(): Array<NodeMaterialBlock>;
    /** Gets the list of connected endpoints */
    get endpoints(): NodeMaterialConnectionPoint[];
    /** Gets a boolean indicating if that output point is connected to at least one input */
    get hasEndpoints(): boolean;
    /** Gets a boolean indicating that this connection has a path to the vertex output*/
    get isDirectlyConnectedToVertexOutput(): boolean;
    /** Gets a boolean indicating that this connection will be used in the vertex shader */
    get isConnectedInVertexShader(): boolean;
    /** Gets a boolean indicating that this connection will be used in the fragment shader */
    get isConnectedInFragmentShader(): boolean;
    /**
     * Creates a block suitable to be used as an input for this input point.
     * If null is returned, a block based on the point type will be created.
     * @returns The returned string parameter is the name of the output point of NodeMaterialBlock (first parameter of the returned array) that can be connected to the input
     */
    createCustomInputBlock(): Nullable<[NodeMaterialBlock, string]>;
    /**
     * Creates a new connection point
     * @param name defines the connection point name
     * @param ownerBlock defines the block hosting this connection point
     * @param direction defines the direction of the connection point
     */
    constructor(name: string, ownerBlock: NodeMaterialBlock, direction: NodeMaterialConnectionPointDirection);
    /**
     * Gets the current class name e.g. "NodeMaterialConnectionPoint"
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Gets a boolean indicating if the current point can be connected to another point
     * @param connectionPoint defines the other connection point
     * @returns a boolean
     */
    canConnectTo(connectionPoint: NodeMaterialConnectionPoint): boolean;
    /**
     * Gets a number indicating if the current point can be connected to another point
     * @param connectionPoint defines the other connection point
     * @returns a number defining the compatibility state
     */
    checkCompatibilityState(connectionPoint: NodeMaterialConnectionPoint): NodeMaterialConnectionPointCompatibilityStates;
    /**
     * Connect this point to another connection point
     * @param connectionPoint defines the other connection point
     * @param ignoreConstraints defines if the system will ignore connection type constraints (default is false)
     * @returns the current connection point
     */
    connectTo(connectionPoint: NodeMaterialConnectionPoint, ignoreConstraints?: boolean): NodeMaterialConnectionPoint;
    /**
     * Disconnect this point from one of his endpoint
     * @param endpoint defines the other connection point
     * @returns the current connection point
     */
    disconnectFrom(endpoint: NodeMaterialConnectionPoint): NodeMaterialConnectionPoint;
    /**
     * Fill the list of excluded connection point types with all types other than those passed in the parameter
     * @param mask Types (ORed values of NodeMaterialBlockConnectionPointTypes) that are allowed, and thus will not be pushed to the excluded list
     */
    addExcludedConnectionPointFromAllowedTypes(mask: number): void;
    /**
     * Serializes this point in a JSON representation
     * @param isInput defines if the connection point is an input (default is true)
     * @returns the serialized point object
     */
    serialize(isInput?: boolean): any;
    /**
     * Release resources
     */
    dispose(): void;
}

}
declare module "babylonjs/Materials/Node/nodeMaterialBuildState" {
import { NodeMaterialBlockConnectionPointTypes } from "babylonjs/Materials/Node/Enums/nodeMaterialBlockConnectionPointTypes";
import { NodeMaterialBlockTargets } from "babylonjs/Materials/Node/Enums/nodeMaterialBlockTargets";
import { NodeMaterialBuildStateSharedData } from "babylonjs/Materials/Node/nodeMaterialBuildStateSharedData";
/**
 * Class used to store node based material build state
 */
export class NodeMaterialBuildState {
    /** Gets or sets a boolean indicating if the current state can emit uniform buffers */
    supportUniformBuffers: boolean;
    /**
     * Gets the list of emitted attributes
     */
    attributes: string[];
    /**
     * Gets the list of emitted uniforms
     */
    uniforms: string[];
    /**
     * Gets the list of emitted constants
     */
    constants: string[];
    /**
     * Gets the list of emitted samplers
     */
    samplers: string[];
    /**
     * Gets the list of emitted functions
     */
    functions: {
        [key: string]: string;
    };
    /**
     * Gets the list of emitted extensions
     */
    extensions: {
        [key: string]: string;
    };
    /**
     * Gets the target of the compilation state
     */
    target: NodeMaterialBlockTargets;
    /**
     * Gets the list of emitted counters
     */
    counters: {
        [key: string]: number;
    };
    /**
     * Shared data between multiple NodeMaterialBuildState instances
     */
    sharedData: NodeMaterialBuildStateSharedData;
    /** @internal */
    _vertexState: NodeMaterialBuildState;
    /** @internal */
    _attributeDeclaration: string;
    /** @internal */
    _uniformDeclaration: string;
    /** @internal */
    _constantDeclaration: string;
    /** @internal */
    _samplerDeclaration: string;
    /** @internal */
    _varyingTransfer: string;
    /** @internal */
    _injectAtEnd: string;
    private _repeatableContentAnchorIndex;
    /** @internal */
    _builtCompilationString: string;
    /**
     * Gets the emitted compilation strings
     */
    compilationString: string;
    /**
     * Finalize the compilation strings
     * @param state defines the current compilation state
     */
    finalize(state: NodeMaterialBuildState): void;
    /** @internal */
    get _repeatableContentAnchor(): string;
    /**
     * @internal
     */
    _getFreeVariableName(prefix: string): string;
    /**
     * @internal
     */
    _getFreeDefineName(prefix: string): string;
    /**
     * @internal
     */
    _excludeVariableName(name: string): void;
    /**
     * @internal
     */
    _emit2DSampler(name: string): void;
    /**
     * @internal
     */
    _getGLType(type: NodeMaterialBlockConnectionPointTypes): string;
    /**
     * @internal
     */
    _emitExtension(name: string, extension: string, define?: string): void;
    /**
     * @internal
     */
    _emitFunction(name: string, code: string, comments: string): void;
    /**
     * @internal
     */
    _emitCodeFromInclude(includeName: string, comments: string, options?: {
        replaceStrings?: {
            search: RegExp;
            replace: string;
        }[];
        repeatKey?: string;
        substitutionVars?: string;
    }): string;
    /**
     * @internal
     */
    _emitFunctionFromInclude(includeName: string, comments: string, options?: {
        repeatKey?: string;
        substitutionVars?: string;
        removeAttributes?: boolean;
        removeUniforms?: boolean;
        removeVaryings?: boolean;
        removeIfDef?: boolean;
        replaceStrings?: {
            search: RegExp;
            replace: string;
        }[];
    }, storeKey?: string): void;
    /**
     * @internal
     */
    _registerTempVariable(name: string): boolean;
    /**
     * @internal
     */
    _emitVaryingFromString(name: string, type: string, define?: string, notDefine?: boolean): boolean;
    /**
     * @internal
     */
    _emitUniformFromString(name: string, type: string, define?: string, notDefine?: boolean): void;
    /**
     * @internal
     */
    _emitFloat(value: number): string;
}

}
declare module "babylonjs/Materials/Node/nodeMaterialBuildStateSharedData" {
import { NodeMaterialConnectionPoint } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { InputBlock } from "babylonjs/Materials/Node/Blocks/Input/inputBlock";
import { Scene } from "babylonjs/scene";
import { Immutable } from "babylonjs/types";
import { NodeMaterialTextureBlocks } from "babylonjs/Materials/Node/nodeMaterial";
/**
 * Class used to store shared data between 2 NodeMaterialBuildState
 */
export class NodeMaterialBuildStateSharedData {
    /**
     * Gets the list of emitted varyings
     */
    temps: string[];
    /**
     * Gets the list of emitted varyings
     */
    varyings: string[];
    /**
     * Gets the varying declaration string
     */
    varyingDeclaration: string;
    /**
     * List of the fragment output nodes
     */
    fragmentOutputNodes: Immutable<Array<NodeMaterialBlock>>;
    /**
     * Input blocks
     */
    inputBlocks: InputBlock[];
    /**
     * Input blocks
     */
    textureBlocks: NodeMaterialTextureBlocks[];
    /**
     * Bindable blocks (Blocks that need to set data to the effect)
     */
    bindableBlocks: NodeMaterialBlock[];
    /**
     * Bindable blocks (Blocks that need to set data to the effect) that will always be called (by bindForSubMesh), contrary to bindableBlocks that won't be called if _mustRebind() returns false
     */
    forcedBindableBlocks: NodeMaterialBlock[];
    /**
     * List of blocks that can provide a compilation fallback
     */
    blocksWithFallbacks: NodeMaterialBlock[];
    /**
     * List of blocks that can provide a define update
     */
    blocksWithDefines: NodeMaterialBlock[];
    /**
     * List of blocks that can provide a repeatable content
     */
    repeatableContentBlocks: NodeMaterialBlock[];
    /**
     * List of blocks that can provide a dynamic list of uniforms
     */
    dynamicUniformBlocks: NodeMaterialBlock[];
    /**
     * List of blocks that can block the isReady function for the material
     */
    blockingBlocks: NodeMaterialBlock[];
    /**
     * Gets the list of animated inputs
     */
    animatedInputs: InputBlock[];
    /**
     * Build Id used to avoid multiple recompilations
     */
    buildId: number;
    /** List of emitted variables */
    variableNames: {
        [key: string]: number;
    };
    /** List of emitted defines */
    defineNames: {
        [key: string]: number;
    };
    /** Should emit comments? */
    emitComments: boolean;
    /** Emit build activity */
    verbose: boolean;
    /** Gets or sets the hosting scene */
    scene: Scene;
    /**
     * Gets the compilation hints emitted at compilation time
     */
    hints: {
        needWorldViewMatrix: boolean;
        needWorldViewProjectionMatrix: boolean;
        needAlphaBlending: boolean;
        needAlphaTesting: boolean;
    };
    /**
     * List of compilation checks
     */
    checks: {
        emitVertex: boolean;
        emitFragment: boolean;
        notConnectedNonOptionalInputs: NodeMaterialConnectionPoint[];
    };
    /**
     * Is vertex program allowed to be empty?
     */
    allowEmptyVertexProgram: boolean;
    /** Creates a new shared data */
    constructor();
    /**
     * Emits console errors and exceptions if there is a failing check
     */
    emitErrors(): void;
}

}
declare module "babylonjs/Materials/Node/nodeMaterialConnectionPointCustomObject" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
import { NodeMaterialConnectionPointDirection } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
import { NodeMaterialConnectionPoint, NodeMaterialConnectionPointCompatibilityStates } from "babylonjs/Materials/Node/nodeMaterialBlockConnectionPoint";
import { Nullable } from "babylonjs/types";
/**
 * Defines a connection point to be used for points with a custom object type
 */
export class NodeMaterialConnectionPointCustomObject<T extends NodeMaterialBlock> extends NodeMaterialConnectionPoint {
    private _blockType;
    private _blockName;
    /**
     * Creates a new connection point
     * @param name defines the connection point name
     * @param ownerBlock defines the block hosting this connection point
     * @param direction defines the direction of the connection point
     * @param _blockType
     * @param _blockName
     */
    constructor(name: string, ownerBlock: NodeMaterialBlock, direction: NodeMaterialConnectionPointDirection, _blockType: new (...args: any[]) => T, _blockName: string);
    /**
     * Gets a number indicating if the current point can be connected to another point
     * @param connectionPoint defines the other connection point
     * @returns a number defining the compatibility state
     */
    checkCompatibilityState(connectionPoint: NodeMaterialConnectionPoint): NodeMaterialConnectionPointCompatibilityStates;
    /**
     * Creates a block suitable to be used as an input for this input point.
     * If null is returned, a block based on the point type will be created.
     * @returns The returned string parameter is the name of the output point of NodeMaterialBlock (first parameter of the returned array) that can be connected to the input
     */
    createCustomInputBlock(): Nullable<[NodeMaterialBlock, string]>;
}

}
declare module "babylonjs/Materials/Node/nodeMaterialDecorator" {
import { Scene } from "babylonjs/scene";
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
/**
 * Enum defining the type of properties that can be edited in the property pages in the NME
 */
export enum PropertyTypeForEdition {
    /** property is a boolean */
    Boolean = 0,
    /** property is a float */
    Float = 1,
    /** property is a int */
    Int = 2,
    /** property is a Vector2 */
    Vector2 = 3,
    /** property is a list of values */
    List = 4
}
/**
 * Interface that defines an option in a variable of type list
 */
export interface IEditablePropertyListOption {
    /** label of the option */
    label: string;
    /** value of the option */
    value: number;
}
/**
 * Interface that defines the options available for an editable property
 */
export interface IEditablePropertyOption {
    /** min value */
    min?: number;
    /** max value */
    max?: number;
    /** notifiers: indicates which actions to take when the property is changed */
    notifiers?: {
        /** the material should be rebuilt */
        rebuild?: boolean;
        /** the preview should be updated */
        update?: boolean;
        /** the onPreviewCommandActivated observer of the preview manager should be triggered */
        activatePreviewCommand?: boolean;
        /** a callback to trigger */
        callback?: (scene: Scene, block: NodeMaterialBlock) => void;
        /** a callback to validate the property. Returns true if the property is ok, else false. If false, the rebuild/update/callback events won't be called */
        onValidation?: (block: NodeMaterialBlock, propertyName: string) => boolean;
    };
    /** list of the options for a variable of type list */
    options?: IEditablePropertyListOption[];
}
/**
 * Interface that describes an editable property
 */
export interface IPropertyDescriptionForEdition {
    /** name of the property */
    propertyName: string;
    /** display name of the property */
    displayName: string;
    /** type of the property */
    type: PropertyTypeForEdition;
    /** group of the property - all properties with the same group value will be displayed in a specific section */
    groupName: string;
    /** options for the property */
    options: IEditablePropertyOption;
}
/**
 * Decorator that flags a property in a node material block as being editable
 * @param displayName
 * @param propertyType
 * @param groupName
 * @param options
 */
export function editableInPropertyPage(displayName: string, propertyType?: PropertyTypeForEdition, groupName?: string, options?: IEditablePropertyOption): (target: any, propertyKey: string) => void;
export {};

}
declare module "babylonjs/Materials/Node/Optimizers/index" {
export * from "babylonjs/Materials/Node/Optimizers/nodeMaterialOptimizer";

}
declare module "babylonjs/Materials/Node/Optimizers/nodeMaterialOptimizer" {
import { NodeMaterialBlock } from "babylonjs/Materials/Node/nodeMaterialBlock";
/**
 * Root class for all node material optimizers
 */
export class NodeMaterialOptimizer {
    /**
     * Function used to optimize a NodeMaterial graph
     * @param _vertexOutputNodes defines the list of output nodes for the vertex shader
     * @param _fragmentOutputNodes defines the list of output nodes for the fragment shader
     */
    optimize(_vertexOutputNodes: NodeMaterialBlock[], _fragmentOutputNodes: NodeMaterialBlock[]): void;
}

}
declare module "babylonjs/Materials/Occlusion/index" {
export * from "babylonjs/Materials/Occlusion/occlusionMaterial";

}
declare module "babylonjs/Materials/Occlusion/occlusionMaterial" {
import { Scene } from "babylonjs/scene";
import { ShaderMaterial } from "babylonjs/Materials/shaderMaterial";
import "babylonjs/Shaders/color.fragment";
import "babylonjs/Shaders/color.vertex";
/**
 * A material to use for fast depth-only rendering.
 * @since 5.0.0
 */
export class OcclusionMaterial extends ShaderMaterial {
    constructor(name: string, scene: Scene);
}

}
declare module "babylonjs/Materials/PBR/index" {
export * from "babylonjs/Materials/PBR/pbrAnisotropicConfiguration";
export * from "babylonjs/Materials/PBR/pbrBaseMaterial";
export * from "babylonjs/Materials/PBR/pbrBaseSimpleMaterial";
export * from "babylonjs/Materials/PBR/pbrClearCoatConfiguration";
export * from "babylonjs/Materials/PBR/pbrIridescenceConfiguration";
export * from "babylonjs/Materials/PBR/pbrMaterial";
export * from "babylonjs/Materials/PBR/pbrMetallicRoughnessMaterial";
export * from "babylonjs/Materials/PBR/pbrSpecularGlossinessMaterial";
export * from "babylonjs/Materials/PBR/pbrSheenConfiguration";
export * from "babylonjs/Materials/PBR/pbrSubSurfaceConfiguration";

}
declare module "babylonjs/Materials/PBR/pbrAnisotropicConfiguration" {
import { UniformBuffer } from "babylonjs/Materials/uniformBuffer";
import { Vector2 } from "babylonjs/Maths/math.vector";
import { BaseTexture } from "babylonjs/Materials/Textures/baseTexture";
import { Nullable } from "babylonjs/types";
import { IAnimatable } from "babylonjs/Animations/animatable.interface";
import { EffectFallbacks } from "babylonjs/Materials/effectFallbacks";
import { MaterialPluginBase } from "babylonjs/Materials/materialPluginBase";
import { MaterialDefines } from "babylonjs/Materials/materialDefines";
import { Scene } from "babylonjs/scene";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { PBRBaseMaterial } from "babylonjs/Materials/PBR/pbrBaseMaterial";
/**
 * @internal
 */
export class MaterialAnisotropicDefines extends MaterialDefines {
    ANISOTROPIC: boolean;
    ANISOTROPIC_TEXTURE: boolean;
    ANISOTROPIC_TEXTUREDIRECTUV: number;
    MAINUV1: boolean;
}
/**
 * Plugin that implements the anisotropic component of the PBR material
 */
export class PBRAnisotropicConfiguration extends MaterialPluginBase {
    private _isEnabled;
    /**
     * Defines if the anisotropy is enabled in the material.
     */
    isEnabled: boolean;
    /**
     * Defines the anisotropy strength (between 0 and 1) it defaults to 1.
     */
    intensity: number;
    /**
     * Defines if the effect is along the tangents, bitangents or in between.
     * By default, the effect is "stretching" the highlights along the tangents.
     */
    direction: Vector2;
    private _texture;
    /**
     * Stores the anisotropy values in a texture.
     * rg is direction (like normal from -1 to 1)
     * b is a intensity
     */
    texture: Nullable<BaseTexture>;
    /** @internal */
    private _internalMarkAllSubMeshesAsTexturesDirty;
    /** @internal */
    _markAllSubMeshesAsTexturesDirty(): void;
    constructor(material: PBRBaseMaterial, addToPluginList?: boolean);
    isReadyForSubMesh(defines: MaterialAnisotropicDefines, scene: Scene): boolean;
    prepareDefinesBeforeAttributes(defines: MaterialAnisotropicDefines, scene: Scene, mesh: AbstractMesh): void;
    bindForSubMesh(uniformBuffer: UniformBuffer, scene: Scene): void;
    hasTexture(texture: BaseTexture): boolean;
    getActiveTextures(activeTextures: BaseTexture[]): void;
    getAnimatables(animatables: IAnimatable[]): void;
    dispose(forceDisposeTextures?: boolean): void;
    getClassName(): string;
    addFallbacks(defines: MaterialAnisotropicDefines, fallbacks: EffectFallbacks, currentRank: number): number;
    getSamplers(samplers: string[]): void;
    getUniforms(): {
        ubo?: Array<{
            name: string;
            size: number;
            type: string;
        }>;
        vertex?: string;
        fragment?: string;
    };
}
export {};

}
declare module "babylonjs/Materials/PBR/pbrBaseMaterial" {
import { Nullable } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
import { Matrix } from "babylonjs/Maths/math.vector";
import { SubMesh } from "babylonjs/Meshes/subMesh";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { Mesh } from "babylonjs/Meshes/mesh";
import { PBRBRDFConfiguration } from "babylonjs/Materials/PBR/pbrBRDFConfiguration";
import { PrePassConfiguration } from "babylonjs/Materials/prePassConfiguration";
import { Color3 } from "babylonjs/Maths/math.color";
import { IImageProcessingConfigurationDefines } from "babylonjs/Materials/imageProcessingConfiguration";
import { ImageProcessingConfiguration } from "babylonjs/Materials/imageProcessingConfiguration";
import { IMaterialCompilationOptions } from "babylonjs/Materials/material";
import { Material } from "babylonjs/Materials/material";
import { MaterialDefines } from "babylonjs/Materials/materialDefines";
import { PushMaterial } from "babylonjs/Materials/pushMaterial";
import { BaseTexture } from "babylonjs/Materials/Textures/baseTexture";
import { IAnimatable } from "babylonjs/Animations/animatable.interface";
import "babylonjs/Materials/Textures/baseTexture.polynomial";
import "babylonjs/Shaders/pbr.fragment";
import "babylonjs/Shaders/pbr.vertex";
import { PBRClearCoatConfiguration } from "babylonjs/Materials/PBR/pbrClearCoatConfiguration";
import { PBRIridescenceConfiguration } from "babylonjs/Materials/PBR/pbrIridescenceConfiguration";
import { PBRAnisotropicConfiguration } from "babylonjs/Materials/PBR/pbrAnisotropicConfiguration";
import { PBRSheenConfiguration } from "babylonjs/Materials/PBR/pbrSheenConfiguration";
import { PBRSubSurfaceConfiguration } from "babylonjs/Materials/PBR/pbrSubSurfaceConfiguration";
import { DetailMapConfiguration } from "babylonjs/Materials/material.detailMapConfiguration";
/**
 * Manages the defines for the PBR Material.
 * @internal
 */
export class PBRMaterialDefines extends MaterialDefines implements IImageProcessingConfigurationDefines {
    PBR: boolean;
    NUM_SAMPLES: string;
    REALTIME_FILTERING: boolean;
    MAINUV1: boolean;
    MAINUV2: boolean;
    MAINUV3: boolean;
    MAINUV4: boolean;
    MAINUV5: boolean;
    MAINUV6: boolean;
    UV1: boolean;
    UV2: boolean;
    UV3: boolean;
    UV4: boolean;
    UV5: boolean;
    UV6: boolean;
    ALBEDO: boolean;
    GAMMAALBEDO: boolean;
    ALBEDODIRECTUV: number;
    VERTEXCOLOR: boolean;
    BAKED_VERTEX_ANIMATION_TEXTURE: boolean;
    AMBIENT: boolean;
    AMBIENTDIRECTUV: number;
    AMBIENTINGRAYSCALE: boolean;
    OPACITY: boolean;
    VERTEXALPHA: boolean;
    OPACITYDIRECTUV: number;
    OPACITYRGB: boolean;
    ALPHATEST: boolean;
    DEPTHPREPASS: boolean;
    ALPHABLEND: boolean;
    ALPHAFROMALBEDO: boolean;
    ALPHATESTVALUE: string;
    SPECULAROVERALPHA: boolean;
    RADIANCEOVERALPHA: boolean;
    ALPHAFRESNEL: boolean;
    LINEARALPHAFRESNEL: boolean;
    PREMULTIPLYALPHA: boolean;
    EMISSIVE: boolean;
    EMISSIVEDIRECTUV: number;
    GAMMAEMISSIVE: boolean;
    REFLECTIVITY: boolean;
    REFLECTIVITY_GAMMA: boolean;
    REFLECTIVITYDIRECTUV: number;
    SPECULARTERM: boolean;
    MICROSURFACEFROMREFLECTIVITYMAP: boolean;
    MICROSURFACEAUTOMATIC: boolean;
    LODBASEDMICROSFURACE: boolean;
    MICROSURFACEMAP: boolean;
    MICROSURFACEMAPDIRECTUV: number;
    METALLICWORKFLOW: boolean;
    ROUGHNESSSTOREINMETALMAPALPHA: boolean;
    ROUGHNESSSTOREINMETALMAPGREEN: boolean;
    METALLNESSSTOREINMETALMAPBLUE: boolean;
    AOSTOREINMETALMAPRED: boolean;
    METALLIC_REFLECTANCE: boolean;
    METALLIC_REFLECTANCE_GAMMA: boolean;
    METALLIC_REFLECTANCEDIRECTUV: number;
    METALLIC_REFLECTANCE_USE_ALPHA_ONLY: boolean;
    REFLECTANCE: boolean;
    REFLECTANCE_GAMMA: boolean;
    REFLECTANCEDIRECTUV: number;
    ENVIRONMENTBRDF: boolean;
    ENVIRONMENTBRDF_RGBD: boolean;
    NORMAL: boolean;
    TANGENT: boolean;
    BUMP: boolean;
    BUMPDIRECTUV: number;
    OBJECTSPACE_NORMALMAP: boolean;
    PARALLAX: boolean;
    PARALLAXOCCLUSION: boolean;
    NORMALXYSCALE: boolean;
    LIGHTMAP: boolean;
    LIGHTMAPDIRECTUV: number;
    USELIGHTMAPASSHADOWMAP: boolean;
    GAMMALIGHTMAP: boolean;
    RGBDLIGHTMAP: boolean;
    REFLECTION: boolean;
    REFLECTIONMAP_3D: boolean;
    REFLECTIONMAP_SPHERICAL: boolean;
    REFLECTIONMAP_PLANAR: boolean;
    REFLECTIONMAP_CUBIC: boolean;
    USE_LOCAL_REFLECTIONMAP_CUBIC: boolean;
    REFLECTIONMAP_PROJECTION: boolean;
    REFLECTIONMAP_SKYBOX: boolean;
    REFLECTIONMAP_EXPLICIT: boolean;
    REFLECTIONMAP_EQUIRECTANGULAR: boolean;
    REFLECTIONMAP_EQUIRECTANGULAR_FIXED: boolean;
    REFLECTIONMAP_MIRROREDEQUIRECTANGULAR_FIXED: boolean;
    INVERTCUBICMAP: boolean;
    USESPHERICALFROMREFLECTIONMAP: boolean;
    USEIRRADIANCEMAP: boolean;
    USESPHERICALINVERTEX: boolean;
    REFLECTIONMAP_OPPOSITEZ: boolean;
    LODINREFLECTIONALPHA: boolean;
    GAMMAREFLECTION: boolean;
    RGBDREFLECTION: boolean;
    LINEARSPECULARREFLECTION: boolean;
    RADIANCEOCCLUSION: boolean;
    HORIZONOCCLUSION: boolean;
    INSTANCES: boolean;
    THIN_INSTANCES: boolean;
    INSTANCESCOLOR: boolean;
    PREPASS: boolean;
    PREPASS_IRRADIANCE: boolean;
    PREPASS_IRRADIANCE_INDEX: number;
    PREPASS_ALBEDO_SQRT: boolean;
    PREPASS_ALBEDO_SQRT_INDEX: number;
    PREPASS_DEPTH: boolean;
    PREPASS_DEPTH_INDEX: number;
    PREPASS_NORMAL: boolean;
    PREPASS_NORMAL_INDEX: number;
    PREPASS_POSITION: boolean;
    PREPASS_POSITION_INDEX: number;
    PREPASS_VELOCITY: boolean;
    PREPASS_VELOCITY_INDEX: number;
    PREPASS_REFLECTIVITY: boolean;
    PREPASS_REFLECTIVITY_INDEX: number;
    SCENE_MRT_COUNT: number;
    NUM_BONE_INFLUENCERS: number;
    BonesPerMesh: number;
    BONETEXTURE: boolean;
    BONES_VELOCITY_ENABLED: boolean;
    NONUNIFORMSCALING: boolean;
    MORPHTARGETS: boolean;
    MORPHTARGETS_NORMAL: boolean;
    MORPHTARGETS_TANGENT: boolean;
    MORPHTARGETS_UV: boolean;
    NUM_MORPH_INFLUENCERS: number;
    MORPHTARGETS_TEXTURE: boolean;
    IMAGEPROCESSING: boolean;
    VIGNETTE: boolean;
    VIGNETTEBLENDMODEMULTIPLY: boolean;
    VIGNETTEBLENDMODEOPAQUE: boolean;
    TONEMAPPING: boolean;
    TONEMAPPING_ACES: boolean;
    CONTRAST: boolean;
    COLORCURVES: boolean;
    COLORGRADING: boolean;
    COLORGRADING3D: boolean;
    SAMPLER3DGREENDEPTH: boolean;
    SAMPLER3DBGRMAP: boolean;
    DITHER: boolean;
    IMAGEPROCESSINGPOSTPROCESS: boolean;
    SKIPFINALCOLORCLAMP: boolean;
    EXPOSURE: boolean;
    MULTIVIEW: boolean;
    ORDER_INDEPENDENT_TRANSPARENCY: boolean;
    ORDER_INDEPENDENT_TRANSPARENCY_16BITS: boolean;
    USEPHYSICALLIGHTFALLOFF: boolean;
    USEGLTFLIGHTFALLOFF: boolean;
    TWOSIDEDLIGHTING: boolean;
    SHADOWFLOAT: boolean;
    CLIPPLANE: boolean;
    CLIPPLANE2: boolean;
    CLIPPLANE3: boolean;
    CLIPPLANE4: boolean;
    CLIPPLANE5: boolean;
    CLIPPLANE6: boolean;
    POINTSIZE: boolean;
    FOG: boolean;
    LOGARITHMICDEPTH: boolean;
    CAMERA_ORTHOGRAPHIC: boolean;
    CAMERA_PERSPECTIVE: boolean;
    FORCENORMALFORWARD: boolean;
    SPECULARAA: boolean;
    UNLIT: boolean;
    DEBUGMODE: number;
    /**
     * Initializes the PBR Material defines.
     * @param externalProperties The external properties
     */
    constructor(externalProperties?: {
        [name: string]: {
            type: string;
            default: any;
        };
    });
    /**
     * Resets the PBR Material defines.
     */
    reset(): void;
}
/**
 * The Physically based material base class of BJS.
 *
 * This offers the main features of a standard PBR material.
 * For more information, please refer to the documentation :
 * https://doc.babylonjs.com/features/featuresDeepDive/materials/using/introToPBR
 */
export abstract class PBRBaseMaterial extends PushMaterial {
    /**
     * PBRMaterialTransparencyMode: No transparency mode, Alpha channel is not use.
     */
    static readonly PBRMATERIAL_OPAQUE: number;
    /**
     * PBRMaterialTransparencyMode: Alpha Test mode, pixel are discarded below a certain threshold defined by the alpha cutoff value.
     */
    static readonly PBRMATERIAL_ALPHATEST: number;
    /**
     * PBRMaterialTransparencyMode: Pixels are blended (according to the alpha mode) with the already drawn pixels in the current frame buffer.
     */
    static readonly PBRMATERIAL_ALPHABLEND: number;
    /**
     * PBRMaterialTransparencyMode: Pixels are blended (according to the alpha mode) with the already drawn pixels in the current frame buffer.
     * They are also discarded below the alpha cutoff threshold to improve performances.
     */
    static readonly PBRMATERIAL_ALPHATESTANDBLEND: number;
    /**
     * Defines the default value of how much AO map is occluding the analytical lights
     * (point spot...).
     */
    static DEFAULT_AO_ON_ANALYTICAL_LIGHTS: number;
    /**
     * PBRMaterialLightFalloff Physical: light is falling off following the inverse squared distance law.
     */
    static readonly LIGHTFALLOFF_PHYSICAL: number;
    /**
     * PBRMaterialLightFalloff gltf: light is falling off as described in the gltf moving to PBR document
     * to enhance interoperability with other engines.
     */
    static readonly LIGHTFALLOFF_GLTF: number;
    /**
     * PBRMaterialLightFalloff Standard: light is falling off like in the standard material
     * to enhance interoperability with other materials.
     */
    static readonly LIGHTFALLOFF_STANDARD: number;
    /**
     * Intensity of the direct lights e.g. the four lights available in your scene.
     * This impacts both the direct diffuse and specular highlights.
     * @internal
     */
    _directIntensity: number;
    /**
     * Intensity of the emissive part of the material.
     * This helps controlling the emissive effect without modifying the emissive color.
     * @internal
     */
    _emissiveIntensity: number;
    /**
     * Intensity of the environment e.g. how much the environment will light the object
     * either through harmonics for rough material or through the reflection for shiny ones.
     * @internal
     */
    _environmentIntensity: number;
    /**
     * This is a special control allowing the reduction of the specular highlights coming from the
     * four lights of the scene. Those highlights may not be needed in full environment lighting.
     * @internal
     */
    _specularIntensity: number;
    /**
     * This stores the direct, emissive, environment, and specular light intensities into a Vector4.
     */
    private _lightingInfos;
    /**
     * Debug Control allowing disabling the bump map on this material.
     * @internal
     */
    _disableBumpMap: boolean;
    /**
     * AKA Diffuse Texture in standard nomenclature.
     * @internal
     */
    _albedoTexture: Nullable<BaseTexture>;
    /**
     * AKA Occlusion Texture in other nomenclature.
     * @internal
     */
    _ambientTexture: Nullable<BaseTexture>;
    /**
     * AKA Occlusion Texture Intensity in other nomenclature.
     * @internal
     */
    _ambientTextureStrength: number;
    /**
     * Defines how much the AO map is occluding the analytical lights (point spot...).
     * 1 means it completely occludes it
     * 0 mean it has no impact
     * @internal
     */
    _ambientTextureImpactOnAnalyticalLights: number;
    /**
     * Stores the alpha values in a texture.
     * @internal
     */
    _opacityTexture: Nullable<BaseTexture>;
    /**
     * Stores the reflection values in a texture.
     * @internal
     */
    _reflectionTexture: Nullable<BaseTexture>;
    /**
     * Stores the emissive values in a texture.
     * @internal
     */
    _emissiveTexture: Nullable<BaseTexture>;
    /**
     * AKA Specular texture in other nomenclature.
     * @internal
     */
    _reflectivityTexture: Nullable<BaseTexture>;
    /**
     * Used to switch from specular/glossiness to metallic/roughness workflow.
     * @internal
     */
    _metallicTexture: Nullable<BaseTexture>;
    /**
     * Specifies the metallic scalar of the metallic/roughness workflow.
     * Can also be used to scale the metalness values of the metallic texture.
     * @internal
     */
    _metallic: Nullable<number>;
    /**
     * Specifies the roughness scalar of the metallic/roughness workflow.
     * Can also be used to scale the roughness values of the metallic texture.
     * @internal
     */
    _roughness: Nullable<number>;
    /**
     * In metallic workflow, specifies an F0 factor to help configuring the material F0.
     * By default the indexOfrefraction is used to compute F0;
     *
     * This is used as a factor against the default reflectance at normal incidence to tweak it.
     *
     * F0 = defaultF0 * metallicF0Factor * metallicReflectanceColor;
     * F90 = metallicReflectanceColor;
     * @internal
     */
    _metallicF0Factor: number;
    /**
     * In metallic workflow, specifies an F90 color to help configuring the material F90.
     * By default the F90 is always 1;
     *
     * Please note that this factor is also used as a factor against the default reflectance at normal incidence.
     *
     * F0 = defaultF0 * metallicF0Factor * metallicReflectanceColor
     * F90 = metallicReflectanceColor;
     * @internal
     */
    _metallicReflectanceColor: Color3;
    /**
     * Specifies that only the A channel from _metallicReflectanceTexture should be used.
     * If false, both RGB and A channels will be used
     * @internal
     */
    _useOnlyMetallicFromMetallicReflectanceTexture: boolean;
    /**
     * Defines to store metallicReflectanceColor in RGB and metallicF0Factor in A
     * This is multiply against the scalar values defined in the material.
     * @internal
     */
    _metallicReflectanceTexture: Nullable<BaseTexture>;
    /**
     * Defines to store reflectanceColor in RGB
     * This is multiplied against the scalar values defined in the material.
     * If both _reflectanceTexture and _metallicReflectanceTexture textures are provided and _useOnlyMetallicFromMetallicReflectanceTexture
     * is false, _metallicReflectanceTexture takes precedence and _reflectanceTexture is not used
     * @internal
     */
    _reflectanceTexture: Nullable<BaseTexture>;
    /**
     * Used to enable roughness/glossiness fetch from a separate channel depending on the current mode.
     * Gray Scale represents roughness in metallic mode and glossiness in specular mode.
     * @internal
     */
    _microSurfaceTexture: Nullable<BaseTexture>;
    /**
     * Stores surface normal data used to displace a mesh in a texture.
     * @internal
     */
    _bumpTexture: Nullable<BaseTexture>;
    /**
     * Stores the pre-calculated light information of a mesh in a texture.
     * @internal
     */
    _lightmapTexture: Nullable<BaseTexture>;
    /**
     * The color of a material in ambient lighting.
     * @internal
     */
    _ambientColor: Color3;
    /**
     * AKA Diffuse Color in other nomenclature.
     * @internal
     */
    _albedoColor: Color3;
    /**
     * AKA Specular Color in other nomenclature.
     * @internal
     */
    _reflectivityColor: Color3;
    /**
     * The color applied when light is reflected from a material.
     * @internal
     */
    _reflectionColor: Color3;
    /**
     * The color applied when light is emitted from a material.
     * @internal
     */
    _emissiveColor: Color3;
    /**
     * AKA Glossiness in other nomenclature.
     * @internal
     */
    _microSurface: number;
    /**
     * Specifies that the material will use the light map as a show map.
     * @internal
     */
    _useLightmapAsShadowmap: boolean;
    /**
     * This parameters will enable/disable Horizon occlusion to prevent normal maps to look shiny when the normal
     * makes the reflect vector face the model (under horizon).
     * @internal
     */
    _useHorizonOcclusion: boolean;
    /**
     * This parameters will enable/disable radiance occlusion by preventing the radiance to lit
     * too much the area relying on ambient texture to define their ambient occlusion.
     * @internal
     */
    _useRadianceOcclusion: boolean;
    /**
     * Specifies that the alpha is coming form the albedo channel alpha channel for alpha blending.
     * @internal
     */
    _useAlphaFromAlbedoTexture: boolean;
    /**
     * Specifies that the material will keeps the specular highlights over a transparent surface (only the most luminous ones).
     * A car glass is a good example of that. When sun reflects on it you can not see what is behind.
     * @internal
     */
    _useSpecularOverAlpha: boolean;
    /**
     * Specifies if the reflectivity texture contains the glossiness information in its alpha channel.
     * @internal
     */
    _useMicroSurfaceFromReflectivityMapAlpha: boolean;
    /**
     * Specifies if the metallic texture contains the roughness information in its alpha channel.
     * @internal
     */
    _useRoughnessFromMetallicTextureAlpha: boolean;
    /**
     * Specifies if the metallic texture contains the roughness information in its green channel.
     * @internal
     */
    _useRoughnessFromMetallicTextureGreen: boolean;
    /**
     * Specifies if the metallic texture contains the metallness information in its blue channel.
     * @internal
     */
    _useMetallnessFromMetallicTextureBlue: boolean;
    /**
     * Specifies if the metallic texture contains the ambient occlusion information in its red channel.
     * @internal
     */
    _useAmbientOcclusionFromMetallicTextureRed: boolean;
    /**
     * Specifies if the ambient texture contains the ambient occlusion information in its red channel only.
     * @internal
     */
    _useAmbientInGrayScale: boolean;
    /**
     * In case the reflectivity map does not contain the microsurface information in its alpha channel,
     * The material will try to infer what glossiness each pixel should be.
     * @internal
     */
    _useAutoMicroSurfaceFromReflectivityMap: boolean;
    /**
     * Defines the  falloff type used in this material.
     * It by default is Physical.
     * @internal
     */
    _lightFalloff: number;
    /**
     * Specifies that the material will keeps the reflection highlights over a transparent surface (only the most luminous ones).
     * A car glass is a good example of that. When the street lights reflects on it you can not see what is behind.
     * @internal
     */
    _useRadianceOverAlpha: boolean;
    /**
     * Allows using an object space normal map (instead of tangent space).
     * @internal
     */
    _useObjectSpaceNormalMap: boolean;
    /**
     * Allows using the bump map in parallax mode.
     * @internal
     */
    _useParallax: boolean;
    /**
     * Allows using the bump map in parallax occlusion mode.
     * @internal
     */
    _useParallaxOcclusion: boolean;
    /**
     * Controls the scale bias of the parallax mode.
     * @internal
     */
    _parallaxScaleBias: number;
    /**
     * If sets to true, disables all the lights affecting the material.
     * @internal
     */
    _disableLighting: boolean;
    /**
     * Number of Simultaneous lights allowed on the material.
     * @internal
     */
    _maxSimultaneousLights: number;
    /**
     * If sets to true, x component of normal map value will be inverted (x = 1.0 - x).
     * @internal
     */
    _invertNormalMapX: boolean;
    /**
     * If sets to true, y component of normal map value will be inverted (y = 1.0 - y).
     * @internal
     */
    _invertNormalMapY: boolean;
    /**
     * If sets to true and backfaceCulling is false, normals will be flipped on the backside.
     * @internal
     */
    _twoSidedLighting: boolean;
    /**
     * Defines the alpha limits in alpha test mode.
     * @internal
     */
    _alphaCutOff: number;
    /**
     * Enforces alpha test in opaque or blend mode in order to improve the performances of some situations.
     * @internal
     */
    _forceAlphaTest: boolean;
    /**
     * A fresnel is applied to the alpha of the model to ensure grazing angles edges are not alpha tested.
     * And/Or occlude the blended part. (alpha is converted to gamma to compute the fresnel)
     * @internal
     */
    _useAlphaFresnel: boolean;
    /**
     * A fresnel is applied to the alpha of the model to ensure grazing angles edges are not alpha tested.
     * And/Or occlude the blended part. (alpha stays linear to compute the fresnel)
     * @internal
     */
    _useLinearAlphaFresnel: boolean;
    /**
     * Specifies the environment BRDF texture used to compute the scale and offset roughness values
     * from cos theta and roughness:
     * http://blog.selfshadow.com/publications/s2013-shading-course/karis/s2013_pbs_epic_notes_v2.pdf
     * @internal
     */
    _environmentBRDFTexture: Nullable<BaseTexture>;
    /**
     * Force the shader to compute irradiance in the fragment shader in order to take bump in account.
     * @internal
     */
    _forceIrradianceInFragment: boolean;
    private _realTimeFiltering;
    /**
     * Enables realtime filtering on the texture.
     */
    get realTimeFiltering(): boolean;
    set realTimeFiltering(b: boolean);
    private _realTimeFilteringQuality;
    /**
     * Quality switch for realtime filtering
     */
    get realTimeFilteringQuality(): number;
    set realTimeFilteringQuality(n: number);
    /**
     * Can this material render to several textures at once
     */
    get canRenderToMRT(): boolean;
    /**
     * Force normal to face away from face.
     * @internal
     */
    _forceNormalForward: boolean;
    /**
     * Enables specular anti aliasing in the PBR shader.
     * It will both interacts on the Geometry for analytical and IBL lighting.
     * It also prefilter the roughness map based on the bump values.
     * @internal
     */
    _enableSpecularAntiAliasing: boolean;
    /**
     * Default configuration related to image processing available in the PBR Material.
     */
    protected _imageProcessingConfiguration: ImageProcessingConfiguration;
    /**
     * Keep track of the image processing observer to allow dispose and replace.
     */
    private _imageProcessingObserver;
    /**
     * Attaches a new image processing configuration to the PBR Material.
     * @param configuration
     */
    protected _attachImageProcessingConfiguration(configuration: Nullable<ImageProcessingConfiguration>): void;
    /**
     * Stores the available render targets.
     */
    private _renderTargets;
    /**
     * Sets the global ambient color for the material used in lighting calculations.
     */
    private _globalAmbientColor;
    /**
     * Enables the use of logarithmic depth buffers, which is good for wide depth buffers.
     */
    private _useLogarithmicDepth;
    /**
     * If set to true, no lighting calculations will be applied.
     */
    private _unlit;
    private _debugMode;
    /**
     * @internal
     * This is reserved for the inspector.
     * Defines the material debug mode.
     * It helps seeing only some components of the material while troubleshooting.
     */
    debugMode: number;
    /**
     * @internal
     * This is reserved for the inspector.
     * Specify from where on screen the debug mode should start.
     * The value goes from -1 (full screen) to 1 (not visible)
     * It helps with side by side comparison against the final render
     * This defaults to -1
     */
    private _debugLimit;
    /**
     * @internal
     * This is reserved for the inspector.
     * As the default viewing range might not be enough (if the ambient is really small for instance)
     * You can use the factor to better multiply the final value.
     */
    private _debugFactor;
    /**
     * Defines the clear coat layer parameters for the material.
     */
    readonly clearCoat: PBRClearCoatConfiguration;
    /**
     * Defines the iridescence layer parameters for the material.
     */
    readonly iridescence: PBRIridescenceConfiguration;
    /**
     * Defines the anisotropic parameters for the material.
     */
    readonly anisotropy: PBRAnisotropicConfiguration;
    /**
     * Defines the BRDF parameters for the material.
     */
    readonly brdf: PBRBRDFConfiguration;
    /**
     * Defines the Sheen parameters for the material.
     */
    readonly sheen: PBRSheenConfiguration;
    /**
     * Defines the SubSurface parameters for the material.
     */
    readonly subSurface: PBRSubSurfaceConfiguration;
    /**
     * Defines additional PrePass parameters for the material.
     */
    readonly prePassConfiguration: PrePassConfiguration;
    /**
     * Defines the detail map parameters for the material.
     */
    readonly detailMap: DetailMapConfiguration;
    protected _cacheHasRenderTargetTextures: boolean;
    /**
     * Instantiates a new PBRMaterial instance.
     *
     * @param name The material name
     * @param scene The scene the material will be use in.
     */
    constructor(name: string, scene?: Scene);
    /**
     * Gets a boolean indicating that current material needs to register RTT
     */
    get hasRenderTargetTextures(): boolean;
    /**
     * Can this material render to prepass
     */
    get isPrePassCapable(): boolean;
    /**
     * Gets the name of the material class.
     */
    getClassName(): string;
    /**
     * Enabled the use of logarithmic depth buffers, which is good for wide depth buffers.
     */
    get useLogarithmicDepth(): boolean;
    /**
     * Enabled the use of logarithmic depth buffers, which is good for wide depth buffers.
     */
    set useLogarithmicDepth(value: boolean);
    /**
     * Returns true if alpha blending should be disabled.
     */
    protected get _disableAlphaBlending(): boolean;
    /**
     * Specifies whether or not this material should be rendered in alpha blend mode.
     */
    needAlphaBlending(): boolean;
    /**
     * Specifies whether or not this material should be rendered in alpha test mode.
     */
    needAlphaTesting(): boolean;
    /**
     * Specifies whether or not the alpha value of the albedo texture should be used for alpha blending.
     */
    protected _shouldUseAlphaFromAlbedoTexture(): boolean;
    /**
     * Specifies whether or not there is a usable alpha channel for transparency.
     */
    protected _hasAlphaChannel(): boolean;
    /**
     * Gets the texture used for the alpha test.
     */
    getAlphaTestTexture(): Nullable<BaseTexture>;
    /**
     * Specifies that the submesh is ready to be used.
     * @param mesh - BJS mesh.
     * @param subMesh - A submesh of the BJS mesh.  Used to check if it is ready.
     * @param useInstances - Specifies that instances should be used.
     * @returns - boolean indicating that the submesh is ready or not.
     */
    isReadyForSubMesh(mesh: AbstractMesh, subMesh: SubMesh, useInstances?: boolean): boolean;
    /**
     * Specifies if the material uses metallic roughness workflow.
     * @returns boolean specifying if the material uses metallic roughness workflow.
     */
    isMetallicWorkflow(): boolean;
    private _prepareEffect;
    private _prepareDefines;
    /**
     * Force shader compilation
     * @param mesh
     * @param onCompiled
     * @param options
     */
    forceCompilation(mesh: AbstractMesh, onCompiled?: (material: Material) => void, options?: Partial<IMaterialCompilationOptions>): void;
    /**
     * Initializes the uniform buffer layout for the shader.
     */
    buildUniformLayout(): void;
    /**
     * Binds the submesh data.
     * @param world - The world matrix.
     * @param mesh - The BJS mesh.
     * @param subMesh - A submesh of the BJS mesh.
     */
    bindForSubMesh(world: Matrix, mesh: Mesh, subMesh: SubMesh): void;
    /**
     * Returns the animatable textures.
     * If material have animatable metallic texture, then reflectivity texture will not be returned, even if it has animations.
     * @returns - Array of animatable textures.
     */
    getAnimatables(): IAnimatable[];
    /**
     * Returns the texture used for reflections.
     * @returns - Reflection texture if present.  Otherwise, returns the environment texture.
     */
    private _getReflectionTexture;
    /**
     * Returns an array of the actively used textures.
     * @returns - Array of BaseTextures
     */
    getActiveTextures(): BaseTexture[];
    /**
     * Checks to see if a texture is used in the material.
     * @param texture - Base texture to use.
     * @returns - Boolean specifying if a texture is used in the material.
     */
    hasTexture(texture: BaseTexture): boolean;
    /**
     * Sets the required values to the prepass renderer.
     * It can't be sets when subsurface scattering of this material is disabled.
     * When scene have ability to enable subsurface prepass effect, it will enable.
     */
    setPrePassRenderer(): boolean;
    /**
     * Disposes the resources of the material.
     * @param forceDisposeEffect - Forces the disposal of effects.
     * @param forceDisposeTextures - Forces the disposal of all textures.
     */
    dispose(forceDisposeEffect?: boolean, forceDisposeTextures?: boolean): void;
}

}
declare module "babylonjs/Materials/PBR/pbrBaseSimpleMaterial" {
import { Scene } from "babylonjs/scene";
import { Color3 } from "babylonjs/Maths/math.color";
import { PBRBaseMaterial } from "babylonjs/Materials/PBR/pbrBaseMaterial";
import { BaseTexture } from "babylonjs/Materials/Textures/baseTexture";
import { Nullable } from "babylonjs/types";
/**
 * The Physically based simple base material of BJS.
 *
 * This enables better naming and convention enforcements on top of the pbrMaterial.
 * It is used as the base class for both the specGloss and metalRough conventions.
 */
export abstract class PBRBaseSimpleMaterial extends PBRBaseMaterial {
    /**
     * Number of Simultaneous lights allowed on the material.
     */
    maxSimultaneousLights: number;
    /**
     * If sets to true, disables all the lights affecting the material.
     */
    disableLighting: boolean;
    /**
     * Environment Texture used in the material (this is use for both reflection and environment lighting).
     */
    environmentTexture: Nullable<BaseTexture>;
    /**
     * If sets to true, x component of normal map value will invert (x = 1.0 - x).
     */
    invertNormalMapX: boolean;
    /**
     * If sets to true, y component of normal map value will invert (y = 1.0 - y).
     */
    invertNormalMapY: boolean;
    /**
     * Normal map used in the model.
     */
    normalTexture: Nullable<BaseTexture>;
    /**
     * Emissivie color used to self-illuminate the model.
     */
    emissiveColor: Color3;
    /**
     * Emissivie texture used to self-illuminate the model.
     */
    emissiveTexture: Nullable<BaseTexture>;
    /**
     * Occlusion Channel Strength.
     */
    occlusionStrength: number;
    /**
     * Occlusion Texture of the material (adding extra occlusion effects).
     */
    occlusionTexture: Nullable<BaseTexture>;
    /**
     * Defines the alpha limits in alpha test mode.
     */
    alphaCutOff: number;
    /**
     * Gets the current double sided mode.
     */
    get doubleSided(): boolean;
    /**
     * If sets to true and backfaceCulling is false, normals will be flipped on the backside.
     */
    set doubleSided(value: boolean);
    /**
     * Stores the pre-calculated light information of a mesh in a texture.
     */
    lightmapTexture: Nullable<BaseTexture>;
    /**
     * If true, the light map contains occlusion information instead of lighting info.
     */
    useLightmapAsShadowmap: boolean;
    /**
     * Instantiates a new PBRMaterial instance.
     *
     * @param name The material name
     * @param scene The scene the material will be use in.
     */
    constructor(name: string, scene?: Scene);
    getClassName(): string;
}

}
declare module "babylonjs/Materials/PBR/pbrBRDFConfiguration" {
import { MaterialDefines } from "babylonjs/Materials/materialDefines";
import { MaterialPluginBase } from "babylonjs/Materials/materialPluginBase";
import { PBRBaseMaterial } from "babylonjs/Materials/PBR/pbrBaseMaterial";
/**
 * @internal
 */
export class MaterialBRDFDefines extends MaterialDefines {
    BRDF_V_HEIGHT_CORRELATED: boolean;
    MS_BRDF_ENERGY_CONSERVATION: boolean;
    SPHERICAL_HARMONICS: boolean;
    SPECULAR_GLOSSINESS_ENERGY_CONSERVATION: boolean;
}
/**
 * Plugin that implements the BRDF component of the PBR material
 */
export class PBRBRDFConfiguration extends MaterialPluginBase {
    /**
     * Default value used for the energy conservation.
     * This should only be changed to adapt to the type of texture in scene.environmentBRDFTexture.
     */
    static DEFAULT_USE_ENERGY_CONSERVATION: boolean;
    /**
     * Default value used for the Smith Visibility Height Correlated mode.
     * This should only be changed to adapt to the type of texture in scene.environmentBRDFTexture.
     */
    static DEFAULT_USE_SMITH_VISIBILITY_HEIGHT_CORRELATED: boolean;
    /**
     * Default value used for the IBL diffuse part.
     * This can help switching back to the polynomials mode globally which is a tiny bit
     * less GPU intensive at the drawback of a lower quality.
     */
    static DEFAULT_USE_SPHERICAL_HARMONICS: boolean;
    /**
     * Default value used for activating energy conservation for the specular workflow.
     * If activated, the albedo color is multiplied with (1. - maxChannel(specular color)).
     * If deactivated, a material is only physically plausible, when (albedo color + specular color) < 1.
     */
    static DEFAULT_USE_SPECULAR_GLOSSINESS_INPUT_ENERGY_CONSERVATION: boolean;
    private _useEnergyConservation;
    /**
     * Defines if the material uses energy conservation.
     */
    useEnergyConservation: boolean;
    private _useSmithVisibilityHeightCorrelated;
    /**
     * LEGACY Mode set to false
     * Defines if the material uses height smith correlated visibility term.
     * If you intent to not use our default BRDF, you need to load a separate BRDF Texture for the PBR
     * You can either load https://assets.babylonjs.com/environments/uncorrelatedBRDF.png
     * or https://assets.babylonjs.com/environments/uncorrelatedBRDF.dds to have more precision
     * Not relying on height correlated will also disable energy conservation.
     */
    useSmithVisibilityHeightCorrelated: boolean;
    private _useSphericalHarmonics;
    /**
     * LEGACY Mode set to false
     * Defines if the material uses spherical harmonics vs spherical polynomials for the
     * diffuse part of the IBL.
     * The harmonics despite a tiny bigger cost has been proven to provide closer results
     * to the ground truth.
     */
    useSphericalHarmonics: boolean;
    private _useSpecularGlossinessInputEnergyConservation;
    /**
     * Defines if the material uses energy conservation, when the specular workflow is active.
     * If activated, the albedo color is multiplied with (1. - maxChannel(specular color)).
     * If deactivated, a material is only physically plausible, when (albedo color + specular color) < 1.
     * In the deactivated case, the material author has to ensure energy conservation, for a physically plausible rendering.
     */
    useSpecularGlossinessInputEnergyConservation: boolean;
    /** @internal */
    private _internalMarkAllSubMeshesAsMiscDirty;
    /** @internal */
    _markAllSubMeshesAsMiscDirty(): void;
    constructor(material: PBRBaseMaterial, addToPluginList?: boolean);
    prepareDefines(defines: MaterialBRDFDefines): void;
    getClassName(): string;
}

}
declare module "babylonjs/Materials/PBR/pbrClearCoatConfiguration" {
import { Nullable } from "babylonjs/types";
import { Color3 } from "babylonjs/Maths/math.color";
import { BaseTexture } from "babylonjs/Materials/Textures/baseTexture";
import { UniformBuffer } from "babylonjs/Materials/uniformBuffer";
import { IAnimatable } from "babylonjs/Animations/animatable.interface";
import { EffectFallbacks } from "babylonjs/Materials/effectFallbacks";
import { SubMesh } from "babylonjs/Meshes/subMesh";
import { MaterialPluginBase } from "babylonjs/Materials/materialPluginBase";
import { MaterialDefines } from "babylonjs/Materials/materialDefines";
import { Engine } from "babylonjs/Engines/engine";
import { Scene } from "babylonjs/scene";
import { PBRBaseMaterial } from "babylonjs/Materials/PBR/pbrBaseMaterial";
/**
 * @internal
 */
export class MaterialClearCoatDefines extends MaterialDefines {
    CLEARCOAT: boolean;
    CLEARCOAT_DEFAULTIOR: boolean;
    CLEARCOAT_TEXTURE: boolean;
    CLEARCOAT_TEXTURE_ROUGHNESS: boolean;
    CLEARCOAT_TEXTUREDIRECTUV: number;
    CLEARCOAT_TEXTURE_ROUGHNESSDIRECTUV: number;
    CLEARCOAT_BUMP: boolean;
    CLEARCOAT_BUMPDIRECTUV: number;
    CLEARCOAT_USE_ROUGHNESS_FROM_MAINTEXTURE: boolean;
    CLEARCOAT_TEXTURE_ROUGHNESS_IDENTICAL: boolean;
    CLEARCOAT_REMAP_F0: boolean;
    CLEARCOAT_TINT: boolean;
    CLEARCOAT_TINT_TEXTURE: boolean;
    CLEARCOAT_TINT_TEXTUREDIRECTUV: number;
    CLEARCOAT_TINT_GAMMATEXTURE: boolean;
}
/**
 * Plugin that implements the clear coat component of the PBR material
 */
export class PBRClearCoatConfiguration extends MaterialPluginBase {
    protected _material: PBRBaseMaterial;
    /**
     * This defaults to 1.5 corresponding to a 0.04 f0 or a 4% reflectance at normal incidence
     * The default fits with a polyurethane material.
     * @internal
     */
    static readonly _DefaultIndexOfRefraction: number;
    private _isEnabled;
    /**
     * Defines if the clear coat is enabled in the material.
     */
    isEnabled: boolean;
    /**
     * Defines the clear coat layer strength (between 0 and 1) it defaults to 1.
     */
    intensity: number;
    /**
     * Defines the clear coat layer roughness.
     */
    roughness: number;
    private _indexOfRefraction;
    /**
     * Defines the index of refraction of the clear coat.
     * This defaults to 1.5 corresponding to a 0.04 f0 or a 4% reflectance at normal incidence
     * The default fits with a polyurethane material.
     * Changing the default value is more performance intensive.
     */
    indexOfRefraction: number;
    private _texture;
    /**
     * Stores the clear coat values in a texture (red channel is intensity and green channel is roughness)
     * If useRoughnessFromMainTexture is false, the green channel of texture is not used and the green channel of textureRoughness is used instead
     * if textureRoughness is not empty, else no texture roughness is used
     */
    texture: Nullable<BaseTexture>;
    private _useRoughnessFromMainTexture;
    /**
     * Indicates that the green channel of the texture property will be used for roughness (default: true)
     * If false, the green channel from textureRoughness is used for roughness
     */
    useRoughnessFromMainTexture: boolean;
    private _textureRoughness;
    /**
     * Stores the clear coat roughness in a texture (green channel)
     * Not used if useRoughnessFromMainTexture is true
     */
    textureRoughness: Nullable<BaseTexture>;
    private _remapF0OnInterfaceChange;
    /**
     * Defines if the F0 value should be remapped to account for the interface change in the material.
     */
    remapF0OnInterfaceChange: boolean;
    private _bumpTexture;
    /**
     * Define the clear coat specific bump texture.
     */
    bumpTexture: Nullable<BaseTexture>;
    private _isTintEnabled;
    /**
     * Defines if the clear coat tint is enabled in the material.
     */
    isTintEnabled: boolean;
    /**
     * Defines the clear coat tint of the material.
     * This is only use if tint is enabled
     */
    tintColor: Color3;
    /**
     * Defines the distance at which the tint color should be found in the
     * clear coat media.
     * This is only use if tint is enabled
     */
    tintColorAtDistance: number;
    /**
     * Defines the clear coat layer thickness.
     * This is only use if tint is enabled
     */
    tintThickness: number;
    private _tintTexture;
    /**
     * Stores the clear tint values in a texture.
     * rgb is tint
     * a is a thickness factor
     */
    tintTexture: Nullable<BaseTexture>;
    /** @internal */
    private _internalMarkAllSubMeshesAsTexturesDirty;
    /** @internal */
    _markAllSubMeshesAsTexturesDirty(): void;
    constructor(material: PBRBaseMaterial, addToPluginList?: boolean);
    isReadyForSubMesh(defines: MaterialClearCoatDefines, scene: Scene, engine: Engine): boolean;
    prepareDefinesBeforeAttributes(defines: MaterialClearCoatDefines, scene: Scene): void;
    bindForSubMesh(uniformBuffer: UniformBuffer, scene: Scene, engine: Engine, subMesh: SubMesh): void;
    hasTexture(texture: BaseTexture): boolean;
    getActiveTextures(activeTextures: BaseTexture[]): void;
    getAnimatables(animatables: IAnimatable[]): void;
    dispose(forceDisposeTextures?: boolean): void;
    getClassName(): string;
    addFallbacks(defines: MaterialClearCoatDefines, fallbacks: EffectFallbacks, currentRank: number): number;
    getSamplers(samplers: string[]): void;
    getUniforms(): {
        ubo?: Array<{
            name: string;
            size: number;
            type: string;
        }>;
        vertex?: string;
        fragment?: string;
    };
}
export {};

}
declare module "babylonjs/Materials/PBR/pbrIridescenceConfiguration" {
import { Nullable } from "babylonjs/types";
import { BaseTexture } from "babylonjs/Materials/Textures/baseTexture";
import { UniformBuffer } from "babylonjs/Materials/uniformBuffer";
import { IAnimatable } from "babylonjs/Animations/animatable.interface";
import { EffectFallbacks } from "babylonjs/Materials/effectFallbacks";
import { SubMesh } from "babylonjs/Meshes/subMesh";
import { MaterialPluginBase } from "babylonjs/Materials/materialPluginBase";
import { MaterialDefines } from "babylonjs/Materials/materialDefines";
import { Engine } from "babylonjs/Engines/engine";
import { Scene } from "babylonjs/scene";
import { PBRBaseMaterial } from "babylonjs/Materials/PBR/pbrBaseMaterial";
/**
 * @internal
 */
export class MaterialIridescenceDefines extends MaterialDefines {
    IRIDESCENCE: boolean;
    IRIDESCENCE_TEXTURE: boolean;
    IRIDESCENCE_TEXTUREDIRECTUV: number;
    IRIDESCENCE_THICKNESS_TEXTURE: boolean;
    IRIDESCENCE_THICKNESS_TEXTUREDIRECTUV: number;
    IRIDESCENCE_USE_THICKNESS_FROM_MAINTEXTURE: boolean;
}
/**
 * Plugin that implements the iridescence (thin film) component of the PBR material
 */
export class PBRIridescenceConfiguration extends MaterialPluginBase {
    protected _material: PBRBaseMaterial;
    /**
     * The default minimum thickness of the thin-film layer given in nanometers (nm).
     * Defaults to 100 nm.
     * @internal
     */
    static readonly _DefaultMinimumThickness: number;
    /**
     * The default maximum thickness of the thin-film layer given in nanometers (nm).
     * Defaults to 400 nm.
     * @internal
     */
    static readonly _DefaultMaximumThickness: number;
    /**
     * The default index of refraction of the thin-film layer.
     * Defaults to 1.3
     * @internal
     */
    static readonly _DefaultIndexOfRefraction: number;
    private _isEnabled;
    /**
     * Defines if the iridescence is enabled in the material.
     */
    isEnabled: boolean;
    /**
     * Defines the iridescence layer strength (between 0 and 1) it defaults to 1.
     */
    intensity: number;
    /**
     * Defines the minimum thickness of the thin-film layer given in nanometers (nm).
     */
    minimumThickness: number;
    /**
     * Defines the maximum thickness of the thin-film layer given in nanometers (nm). This will be the thickness used if not thickness texture has been set.
     */
    maximumThickness: number;
    /**
     * Defines the maximum thickness of the thin-film layer given in nanometers (nm).
     */
    indexOfRefraction: number;
    private _texture;
    /**
     * Stores the iridescence intensity in a texture (red channel)
     */
    texture: Nullable<BaseTexture>;
    private _thicknessTexture;
    /**
     * Stores the iridescence thickness in a texture (green channel)
     */
    thicknessTexture: Nullable<BaseTexture>;
    /** @internal */
    private _internalMarkAllSubMeshesAsTexturesDirty;
    /** @internal */
    _markAllSubMeshesAsTexturesDirty(): void;
    constructor(material: PBRBaseMaterial, addToPluginList?: boolean);
    isReadyForSubMesh(defines: MaterialIridescenceDefines, scene: Scene): boolean;
    prepareDefinesBeforeAttributes(defines: MaterialIridescenceDefines, scene: Scene): void;
    bindForSubMesh(uniformBuffer: UniformBuffer, scene: Scene, engine: Engine, subMesh: SubMesh): void;
    hasTexture(texture: BaseTexture): boolean;
    getActiveTextures(activeTextures: BaseTexture[]): void;
    getAnimatables(animatables: IAnimatable[]): void;
    dispose(forceDisposeTextures?: boolean): void;
    getClassName(): string;
    addFallbacks(defines: MaterialIridescenceDefines, fallbacks: EffectFallbacks, currentRank: number): number;
    getSamplers(samplers: string[]): void;
    getUniforms(): {
        ubo?: Array<{
            name: string;
            size: number;
            type: string;
        }>;
        vertex?: string;
        fragment?: string;
    };
}
export {};

}
declare module "babylonjs/Materials/PBR/pbrMaterial" {
import { Nullable } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
import { Color3 } from "babylonjs/Maths/math.color";
import { ImageProcessingConfiguration } from "babylonjs/Materials/imageProcessingConfiguration";
import { ColorCurves } from "babylonjs/Materials/colorCurves";
import { BaseTexture } from "babylonjs/Materials/Textures/baseTexture";
import { PBRBaseMaterial } from "babylonjs/Materials/PBR/pbrBaseMaterial";
/**
 * The Physically based material of BJS.
 *
 * This offers the main features of a standard PBR material.
 * For more information, please refer to the documentation :
 * https://doc.babylonjs.com/features/featuresDeepDive/materials/using/introToPBR
 */
export class PBRMaterial extends PBRBaseMaterial {
    /**
     * PBRMaterialTransparencyMode: No transparency mode, Alpha channel is not use.
     */
    static readonly PBRMATERIAL_OPAQUE: number;
    /**
     * PBRMaterialTransparencyMode: Alpha Test mode, pixel are discarded below a certain threshold defined by the alpha cutoff value.
     */
    static readonly PBRMATERIAL_ALPHATEST: number;
    /**
     * PBRMaterialTransparencyMode: Pixels are blended (according to the alpha mode) with the already drawn pixels in the current frame buffer.
     */
    static readonly PBRMATERIAL_ALPHABLEND: number;
    /**
     * PBRMaterialTransparencyMode: Pixels are blended (according to the alpha mode) with the already drawn pixels in the current frame buffer.
     * They are also discarded below the alpha cutoff threshold to improve performances.
     */
    static readonly PBRMATERIAL_ALPHATESTANDBLEND: number;
    /**
     * Defines the default value of how much AO map is occluding the analytical lights
     * (point spot...).
     */
    static DEFAULT_AO_ON_ANALYTICAL_LIGHTS: number;
    /**
     * Intensity of the direct lights e.g. the four lights available in your scene.
     * This impacts both the direct diffuse and specular highlights.
     */
    directIntensity: number;
    /**
     * Intensity of the emissive part of the material.
     * This helps controlling the emissive effect without modifying the emissive color.
     */
    emissiveIntensity: number;
    /**
     * Intensity of the environment e.g. how much the environment will light the object
     * either through harmonics for rough material or through the reflection for shiny ones.
     */
    environmentIntensity: number;
    /**
     * This is a special control allowing the reduction of the specular highlights coming from the
     * four lights of the scene. Those highlights may not be needed in full environment lighting.
     */
    specularIntensity: number;
    /**
     * Debug Control allowing disabling the bump map on this material.
     */
    disableBumpMap: boolean;
    /**
     * AKA Diffuse Texture in standard nomenclature.
     */
    albedoTexture: Nullable<BaseTexture>;
    /**
     * AKA Occlusion Texture in other nomenclature.
     */
    ambientTexture: Nullable<BaseTexture>;
    /**
     * AKA Occlusion Texture Intensity in other nomenclature.
     */
    ambientTextureStrength: number;
    /**
     * Defines how much the AO map is occluding the analytical lights (point spot...).
     * 1 means it completely occludes it
     * 0 mean it has no impact
     */
    ambientTextureImpactOnAnalyticalLights: number;
    /**
     * Stores the alpha values in a texture. Use luminance if texture.getAlphaFromRGB is true.
     */
    opacityTexture: Nullable<BaseTexture>;
    /**
     * Stores the reflection values in a texture.
     */
    reflectionTexture: Nullable<BaseTexture>;
    /**
     * Stores the emissive values in a texture.
     */
    emissiveTexture: Nullable<BaseTexture>;
    /**
     * AKA Specular texture in other nomenclature.
     */
    reflectivityTexture: Nullable<BaseTexture>;
    /**
     * Used to switch from specular/glossiness to metallic/roughness workflow.
     */
    metallicTexture: Nullable<BaseTexture>;
    /**
     * Specifies the metallic scalar of the metallic/roughness workflow.
     * Can also be used to scale the metalness values of the metallic texture.
     */
    metallic: Nullable<number>;
    /**
     * Specifies the roughness scalar of the metallic/roughness workflow.
     * Can also be used to scale the roughness values of the metallic texture.
     */
    roughness: Nullable<number>;
    /**
     * In metallic workflow, specifies an F0 factor to help configuring the material F0.
     * By default the indexOfrefraction is used to compute F0;
     *
     * This is used as a factor against the default reflectance at normal incidence to tweak it.
     *
     * F0 = defaultF0 * metallicF0Factor * metallicReflectanceColor;
     * F90 = metallicReflectanceColor;
     */
    metallicF0Factor: number;
    /**
     * In metallic workflow, specifies an F90 color to help configuring the material F90.
     * By default the F90 is always 1;
     *
     * Please note that this factor is also used as a factor against the default reflectance at normal incidence.
     *
     * F0 = defaultF0 * metallicF0Factor * metallicReflectanceColor
     * F90 = metallicReflectanceColor;
     */
    metallicReflectanceColor: Color3;
    /**
     * Specifies that only the A channel from metallicReflectanceTexture should be used.
     * If false, both RGB and A channels will be used
     */
    useOnlyMetallicFromMetallicReflectanceTexture: boolean;
    /**
     * Defines to store metallicReflectanceColor in RGB and metallicF0Factor in A
     * This is multiplied against the scalar values defined in the material.
     * If useOnlyMetallicFromMetallicReflectanceTexture is true, don't use the RGB channels, only A
     */
    metallicReflectanceTexture: Nullable<BaseTexture>;
    /**
     * Defines to store reflectanceColor in RGB
     * This is multiplied against the scalar values defined in the material.
     * If both reflectanceTexture and metallicReflectanceTexture textures are provided and useOnlyMetallicFromMetallicReflectanceTexture
     * is false, metallicReflectanceTexture takes priority and reflectanceTexture is not used
     */
    reflectanceTexture: Nullable<BaseTexture>;
    /**
     * Used to enable roughness/glossiness fetch from a separate channel depending on the current mode.
     * Gray Scale represents roughness in metallic mode and glossiness in specular mode.
     */
    microSurfaceTexture: Nullable<BaseTexture>;
    /**
     * Stores surface normal data used to displace a mesh in a texture.
     */
    bumpTexture: Nullable<BaseTexture>;
    /**
     * Stores the pre-calculated light information of a mesh in a texture.
     */
    lightmapTexture: Nullable<BaseTexture>;
    /**
     * Stores the refracted light information in a texture.
     */
    get refractionTexture(): Nullable<BaseTexture>;
    set refractionTexture(value: Nullable<BaseTexture>);
    /**
     * The color of a material in ambient lighting.
     */
    ambientColor: Color3;
    /**
     * AKA Diffuse Color in other nomenclature.
     */
    albedoColor: Color3;
    /**
     * AKA Specular Color in other nomenclature.
     */
    reflectivityColor: Color3;
    /**
     * The color reflected from the material.
     */
    reflectionColor: Color3;
    /**
     * The color emitted from the material.
     */
    emissiveColor: Color3;
    /**
     * AKA Glossiness in other nomenclature.
     */
    microSurface: number;
    /**
     * Index of refraction of the material base layer.
     * https://en.wikipedia.org/wiki/List_of_refractive_indices
     *
     * This does not only impact refraction but also the Base F0 of Dielectric Materials.
     *
     * From dielectric fresnel rules: F0 = square((iorT - iorI) / (iorT + iorI))
     */
    get indexOfRefraction(): number;
    set indexOfRefraction(value: number);
    /**
     * Controls if refraction needs to be inverted on Y. This could be useful for procedural texture.
     */
    get invertRefractionY(): boolean;
    set invertRefractionY(value: boolean);
    /**
     * This parameters will make the material used its opacity to control how much it is refracting against not.
     * Materials half opaque for instance using refraction could benefit from this control.
     */
    get linkRefractionWithTransparency(): boolean;
    set linkRefractionWithTransparency(value: boolean);
    /**
     * If true, the light map contains occlusion information instead of lighting info.
     */
    useLightmapAsShadowmap: boolean;
    /**
     * Specifies that the alpha is coming form the albedo channel alpha channel for alpha blending.
     */
    useAlphaFromAlbedoTexture: boolean;
    /**
     * Enforces alpha test in opaque or blend mode in order to improve the performances of some situations.
     */
    forceAlphaTest: boolean;
    /**
     * Defines the alpha limits in alpha test mode.
     */
    alphaCutOff: number;
    /**
     * Specifies that the material will keep the specular highlights over a transparent surface (only the most luminous ones).
     * A car glass is a good example of that. When sun reflects on it you can not see what is behind.
     */
    useSpecularOverAlpha: boolean;
    /**
     * Specifies if the reflectivity texture contains the glossiness information in its alpha channel.
     */
    useMicroSurfaceFromReflectivityMapAlpha: boolean;
    /**
     * Specifies if the metallic texture contains the roughness information in its alpha channel.
     */
    useRoughnessFromMetallicTextureAlpha: boolean;
    /**
     * Specifies if the metallic texture contains the roughness information in its green channel.
     */
    useRoughnessFromMetallicTextureGreen: boolean;
    /**
     * Specifies if the metallic texture contains the metallness information in its blue channel.
     */
    useMetallnessFromMetallicTextureBlue: boolean;
    /**
     * Specifies if the metallic texture contains the ambient occlusion information in its red channel.
     */
    useAmbientOcclusionFromMetallicTextureRed: boolean;
    /**
     * Specifies if the ambient texture contains the ambient occlusion information in its red channel only.
     */
    useAmbientInGrayScale: boolean;
    /**
     * In case the reflectivity map does not contain the microsurface information in its alpha channel,
     * The material will try to infer what glossiness each pixel should be.
     */
    useAutoMicroSurfaceFromReflectivityMap: boolean;
    /**
     * BJS is using an hardcoded light falloff based on a manually sets up range.
     * In PBR, one way to represents the falloff is to use the inverse squared root algorithm.
     * This parameter can help you switch back to the BJS mode in order to create scenes using both materials.
     */
    get usePhysicalLightFalloff(): boolean;
    /**
     * BJS is using an hardcoded light falloff based on a manually sets up range.
     * In PBR, one way to represents the falloff is to use the inverse squared root algorithm.
     * This parameter can help you switch back to the BJS mode in order to create scenes using both materials.
     */
    set usePhysicalLightFalloff(value: boolean);
    /**
     * In order to support the falloff compatibility with gltf, a special mode has been added
     * to reproduce the gltf light falloff.
     */
    get useGLTFLightFalloff(): boolean;
    /**
     * In order to support the falloff compatibility with gltf, a special mode has been added
     * to reproduce the gltf light falloff.
     */
    set useGLTFLightFalloff(value: boolean);
    /**
     * Specifies that the material will keeps the reflection highlights over a transparent surface (only the most luminous ones).
     * A car glass is a good example of that. When the street lights reflects on it you can not see what is behind.
     */
    useRadianceOverAlpha: boolean;
    /**
     * Allows using an object space normal map (instead of tangent space).
     */
    useObjectSpaceNormalMap: boolean;
    /**
     * Allows using the bump map in parallax mode.
     */
    useParallax: boolean;
    /**
     * Allows using the bump map in parallax occlusion mode.
     */
    useParallaxOcclusion: boolean;
    /**
     * Controls the scale bias of the parallax mode.
     */
    parallaxScaleBias: number;
    /**
     * If sets to true, disables all the lights affecting the material.
     */
    disableLighting: boolean;
    /**
     * Force the shader to compute irradiance in the fragment shader in order to take bump in account.
     */
    forceIrradianceInFragment: boolean;
    /**
     * Number of Simultaneous lights allowed on the material.
     */
    maxSimultaneousLights: number;
    /**
     * If sets to true, x component of normal map value will invert (x = 1.0 - x).
     */
    invertNormalMapX: boolean;
    /**
     * If sets to true, y component of normal map value will invert (y = 1.0 - y).
     */
    invertNormalMapY: boolean;
    /**
     * If sets to true and backfaceCulling is false, normals will be flipped on the backside.
     */
    twoSidedLighting: boolean;
    /**
     * A fresnel is applied to the alpha of the model to ensure grazing angles edges are not alpha tested.
     * And/Or occlude the blended part. (alpha is converted to gamma to compute the fresnel)
     */
    useAlphaFresnel: boolean;
    /**
     * A fresnel is applied to the alpha of the model to ensure grazing angles edges are not alpha tested.
     * And/Or occlude the blended part. (alpha stays linear to compute the fresnel)
     */
    useLinearAlphaFresnel: boolean;
    /**
     * Let user defines the brdf lookup texture used for IBL.
     * A default 8bit version is embedded but you could point at :
     * * Default texture: https://assets.babylonjs.com/environments/correlatedMSBRDF_RGBD.png
     * * Default 16bit pixel depth texture: https://assets.babylonjs.com/environments/correlatedMSBRDF.dds
     * * LEGACY Default None correlated https://assets.babylonjs.com/environments/uncorrelatedBRDF_RGBD.png
     * * LEGACY Default None correlated 16bit pixel depth https://assets.babylonjs.com/environments/uncorrelatedBRDF.dds
     */
    environmentBRDFTexture: Nullable<BaseTexture>;
    /**
     * Force normal to face away from face.
     */
    forceNormalForward: boolean;
    /**
     * Enables specular anti aliasing in the PBR shader.
     * It will both interacts on the Geometry for analytical and IBL lighting.
     * It also prefilter the roughness map based on the bump values.
     */
    enableSpecularAntiAliasing: boolean;
    /**
     * This parameters will enable/disable Horizon occlusion to prevent normal maps to look shiny when the normal
     * makes the reflect vector face the model (under horizon).
     */
    useHorizonOcclusion: boolean;
    /**
     * This parameters will enable/disable radiance occlusion by preventing the radiance to lit
     * too much the area relying on ambient texture to define their ambient occlusion.
     */
    useRadianceOcclusion: boolean;
    /**
     * If set to true, no lighting calculations will be applied.
     */
    unlit: boolean;
    /**
     * Gets the image processing configuration used either in this material.
     */
    get imageProcessingConfiguration(): ImageProcessingConfiguration;
    /**
     * Sets the Default image processing configuration used either in the this material.
     *
     * If sets to null, the scene one is in use.
     */
    set imageProcessingConfiguration(value: ImageProcessingConfiguration);
    /**
     * Gets whether the color curves effect is enabled.
     */
    get cameraColorCurvesEnabled(): boolean;
    /**
     * Sets whether the color curves effect is enabled.
     */
    set cameraColorCurvesEnabled(value: boolean);
    /**
     * Gets whether the color grading effect is enabled.
     */
    get cameraColorGradingEnabled(): boolean;
    /**
     * Gets whether the color grading effect is enabled.
     */
    set cameraColorGradingEnabled(value: boolean);
    /**
     * Gets whether tonemapping is enabled or not.
     */
    get cameraToneMappingEnabled(): boolean;
    /**
     * Sets whether tonemapping is enabled or not
     */
    set cameraToneMappingEnabled(value: boolean);
    /**
     * The camera exposure used on this material.
     * This property is here and not in the camera to allow controlling exposure without full screen post process.
     * This corresponds to a photographic exposure.
     */
    get cameraExposure(): number;
    /**
     * The camera exposure used on this material.
     * This property is here and not in the camera to allow controlling exposure without full screen post process.
     * This corresponds to a photographic exposure.
     */
    set cameraExposure(value: number);
    /**
     * Gets The camera contrast used on this material.
     */
    get cameraContrast(): number;
    /**
     * Sets The camera contrast used on this material.
     */
    set cameraContrast(value: number);
    /**
     * Gets the Color Grading 2D Lookup Texture.
     */
    get cameraColorGradingTexture(): Nullable<BaseTexture>;
    /**
     * Sets the Color Grading 2D Lookup Texture.
     */
    set cameraColorGradingTexture(value: Nullable<BaseTexture>);
    /**
     * The color grading curves provide additional color adjustment that is applied after any color grading transform (3D LUT).
     * They allow basic adjustment of saturation and small exposure adjustments, along with color filter tinting to provide white balance adjustment or more stylistic effects.
     * These are similar to controls found in many professional imaging or colorist software. The global controls are applied to the entire image. For advanced tuning, extra controls are provided to adjust the shadow, midtone and highlight areas of the image;
     * corresponding to low luminance, medium luminance, and high luminance areas respectively.
     */
    get cameraColorCurves(): Nullable<ColorCurves>;
    /**
     * The color grading curves provide additional color adjustment that is applied after any color grading transform (3D LUT).
     * They allow basic adjustment of saturation and small exposure adjustments, along with color filter tinting to provide white balance adjustment or more stylistic effects.
     * These are similar to controls found in many professional imaging or colorist software. The global controls are applied to the entire image. For advanced tuning, extra controls are provided to adjust the shadow, midtone and highlight areas of the image;
     * corresponding to low luminance, medium luminance, and high luminance areas respectively.
     */
    set cameraColorCurves(value: Nullable<ColorCurves>);
    /**
     * Instantiates a new PBRMaterial instance.
     *
     * @param name The material name
     * @param scene The scene the material will be use in.
     */
    constructor(name: string, scene?: Scene);
    /**
     * Returns the name of this material class.
     */
    getClassName(): string;
    /**
     * Makes a duplicate of the current material.
     * @param name - name to use for the new material.
     */
    clone(name: string): PBRMaterial;
    /**
     * Serializes this PBR Material.
     * @returns - An object with the serialized material.
     */
    serialize(): any;
    /**
     * Parses a PBR Material from a serialized object.
     * @param source - Serialized object.
     * @param scene - BJS scene instance.
     * @param rootUrl - url for the scene object
     * @returns - PBRMaterial
     */
    static Parse(source: any, scene: Scene, rootUrl: string): PBRMaterial;
}

}
declare module "babylonjs/Materials/PBR/pbrMetallicRoughnessMaterial" {
import { Scene } from "babylonjs/scene";
import { Color3 } from "babylonjs/Maths/math.color";
import { BaseTexture } from "babylonjs/Materials/Textures/baseTexture";
import { PBRBaseSimpleMaterial } from "babylonjs/Materials/PBR/pbrBaseSimpleMaterial";
import { Nullable } from "babylonjs/types";
/**
 * The PBR material of BJS following the metal roughness convention.
 *
 * This fits to the PBR convention in the GLTF definition:
 * https://github.com/KhronosGroup/glTF/blob/main/extensions/2.0/Archived/KHR_materials_pbrSpecularGlossiness/README.md
 */
export class PBRMetallicRoughnessMaterial extends PBRBaseSimpleMaterial {
    /**
     * The base color has two different interpretations depending on the value of metalness.
     * When the material is a metal, the base color is the specific measured reflectance value
     * at normal incidence (F0). For a non-metal the base color represents the reflected diffuse color
     * of the material.
     */
    baseColor: Color3;
    /**
     * Base texture of the metallic workflow. It contains both the baseColor information in RGB as
     * well as opacity information in the alpha channel.
     */
    baseTexture: Nullable<BaseTexture>;
    /**
     * Specifies the metallic scalar value of the material.
     * Can also be used to scale the metalness values of the metallic texture.
     */
    metallic: number;
    /**
     * Specifies the roughness scalar value of the material.
     * Can also be used to scale the roughness values of the metallic texture.
     */
    roughness: number;
    /**
     * Texture containing both the metallic value in the B channel and the
     * roughness value in the G channel to keep better precision.
     */
    metallicRoughnessTexture: Nullable<BaseTexture>;
    /**
     * Instantiates a new PBRMetalRoughnessMaterial instance.
     *
     * @param name The material name
     * @param scene The scene the material will be use in.
     */
    constructor(name: string, scene?: Scene);
    /**
     * Return the current class name of the material.
     */
    getClassName(): string;
    /**
     * Makes a duplicate of the current material.
     * @param name - name to use for the new material.
     */
    clone(name: string): PBRMetallicRoughnessMaterial;
    /**
     * Serialize the material to a parsable JSON object.
     */
    serialize(): any;
    /**
     * Parses a JSON object corresponding to the serialize function.
     * @param source
     * @param scene
     * @param rootUrl
     */
    static Parse(source: any, scene: Scene, rootUrl: string): PBRMetallicRoughnessMaterial;
}

}
declare module "babylonjs/Materials/PBR/pbrSheenConfiguration" {
import { UniformBuffer } from "babylonjs/Materials/uniformBuffer";
import { Color3 } from "babylonjs/Maths/math.color";
import { BaseTexture } from "babylonjs/Materials/Textures/baseTexture";
import { Nullable } from "babylonjs/types";
import { IAnimatable } from "babylonjs/Animations/animatable.interface";
import { EffectFallbacks } from "babylonjs/Materials/effectFallbacks";
import { SubMesh } from "babylonjs/Meshes/subMesh";
import { MaterialPluginBase } from "babylonjs/Materials/materialPluginBase";
import { MaterialDefines } from "babylonjs/Materials/materialDefines";
import { Engine } from "babylonjs/Engines/engine";
import { Scene } from "babylonjs/scene";
import { PBRBaseMaterial } from "babylonjs/Materials/PBR/pbrBaseMaterial";
/**
 * @internal
 */
export class MaterialSheenDefines extends MaterialDefines {
    SHEEN: boolean;
    SHEEN_TEXTURE: boolean;
    SHEEN_GAMMATEXTURE: boolean;
    SHEEN_TEXTURE_ROUGHNESS: boolean;
    SHEEN_TEXTUREDIRECTUV: number;
    SHEEN_TEXTURE_ROUGHNESSDIRECTUV: number;
    SHEEN_LINKWITHALBEDO: boolean;
    SHEEN_ROUGHNESS: boolean;
    SHEEN_ALBEDOSCALING: boolean;
    SHEEN_USE_ROUGHNESS_FROM_MAINTEXTURE: boolean;
    SHEEN_TEXTURE_ROUGHNESS_IDENTICAL: boolean;
}
/**
 * Plugin that implements the sheen component of the PBR material.
 */
export class PBRSheenConfiguration extends MaterialPluginBase {
    private _isEnabled;
    /**
     * Defines if the material uses sheen.
     */
    isEnabled: boolean;
    private _linkSheenWithAlbedo;
    /**
     * Defines if the sheen is linked to the sheen color.
     */
    linkSheenWithAlbedo: boolean;
    /**
     * Defines the sheen intensity.
     */
    intensity: number;
    /**
     * Defines the sheen color.
     */
    color: Color3;
    private _texture;
    /**
     * Stores the sheen tint values in a texture.
     * rgb is tint
     * a is a intensity or roughness if the roughness property has been defined and useRoughnessFromTexture is true (in that case, textureRoughness won't be used)
     * If the roughness property has been defined and useRoughnessFromTexture is false then the alpha channel is not used to modulate roughness
     */
    texture: Nullable<BaseTexture>;
    private _useRoughnessFromMainTexture;
    /**
     * Indicates that the alpha channel of the texture property will be used for roughness.
     * Has no effect if the roughness (and texture!) property is not defined
     */
    useRoughnessFromMainTexture: boolean;
    private _roughness;
    /**
     * Defines the sheen roughness.
     * It is not taken into account if linkSheenWithAlbedo is true.
     * To stay backward compatible, material roughness is used instead if sheen roughness = null
     */
    roughness: Nullable<number>;
    private _textureRoughness;
    /**
     * Stores the sheen roughness in a texture.
     * alpha channel is the roughness. This texture won't be used if the texture property is not empty and useRoughnessFromTexture is true
     */
    textureRoughness: Nullable<BaseTexture>;
    private _albedoScaling;
    /**
     * If true, the sheen effect is layered above the base BRDF with the albedo-scaling technique.
     * It allows the strength of the sheen effect to not depend on the base color of the material,
     * making it easier to setup and tweak the effect
     */
    albedoScaling: boolean;
    /** @internal */
    private _internalMarkAllSubMeshesAsTexturesDirty;
    /** @internal */
    _markAllSubMeshesAsTexturesDirty(): void;
    constructor(material: PBRBaseMaterial, addToPluginList?: boolean);
    isReadyForSubMesh(defines: MaterialSheenDefines, scene: Scene): boolean;
    prepareDefinesBeforeAttributes(defines: MaterialSheenDefines, scene: Scene): void;
    bindForSubMesh(uniformBuffer: UniformBuffer, scene: Scene, engine: Engine, subMesh: SubMesh): void;
    hasTexture(texture: BaseTexture): boolean;
    getActiveTextures(activeTextures: BaseTexture[]): void;
    getAnimatables(animatables: IAnimatable[]): void;
    dispose(forceDisposeTextures?: boolean): void;
    getClassName(): string;
    addFallbacks(defines: MaterialSheenDefines, fallbacks: EffectFallbacks, currentRank: number): number;
    getSamplers(samplers: string[]): void;
    getUniforms(): {
        ubo?: Array<{
            name: string;
            size: number;
            type: string;
        }>;
        vertex?: string;
        fragment?: string;
    };
}
export {};

}
declare module "babylonjs/Materials/PBR/pbrSpecularGlossinessMaterial" {
import { Scene } from "babylonjs/scene";
import { Color3 } from "babylonjs/Maths/math.color";
import { BaseTexture } from "babylonjs/Materials/Textures/baseTexture";
import { PBRBaseSimpleMaterial } from "babylonjs/Materials/PBR/pbrBaseSimpleMaterial";
import { Nullable } from "babylonjs/types";
/**
 * The PBR material of BJS following the specular glossiness convention.
 *
 * This fits to the PBR convention in the GLTF definition:
 * https://github.com/KhronosGroup/glTF/tree/2.0/extensions/Khronos/KHR_materials_pbrSpecularGlossiness
 */
export class PBRSpecularGlossinessMaterial extends PBRBaseSimpleMaterial {
    /**
     * Specifies the diffuse color of the material.
     */
    diffuseColor: Color3;
    /**
     * Specifies the diffuse texture of the material. This can also contains the opacity value in its alpha
     * channel.
     */
    diffuseTexture: Nullable<BaseTexture>;
    /**
     * Specifies the specular color of the material. This indicates how reflective is the material (none to mirror).
     */
    specularColor: Color3;
    /**
     * Specifies the glossiness of the material. This indicates "how sharp is the reflection".
     */
    glossiness: number;
    /**
     * Specifies both the specular color RGB and the glossiness A of the material per pixels.
     */
    specularGlossinessTexture: Nullable<BaseTexture>;
    /**
     * Specifies if the reflectivity texture contains the glossiness information in its alpha channel.
     */
    get useMicroSurfaceFromReflectivityMapAlpha(): boolean;
    /**
     * Instantiates a new PBRSpecularGlossinessMaterial instance.
     *
     * @param name The material name
     * @param scene The scene the material will be use in.
     */
    constructor(name: string, scene?: Scene);
    /**
     * Return the current class name of the material.
     */
    getClassName(): string;
    /**
     * Makes a duplicate of the current material.
     * @param name - name to use for the new material.
     */
    clone(name: string): PBRSpecularGlossinessMaterial;
    /**
     * Serialize the material to a parsable JSON object.
     */
    serialize(): any;
    /**
     * Parses a JSON object corresponding to the serialize function.
     * @param source
     * @param scene
     * @param rootUrl
     */
    static Parse(source: any, scene: Scene, rootUrl: string): PBRSpecularGlossinessMaterial;
}

}
declare module "babylonjs/Materials/PBR/pbrSubSurfaceConfiguration" {
import { Nullable } from "babylonjs/types";
import { IAnimatable } from "babylonjs/Animations/animatable.interface";
import { Color3 } from "babylonjs/Maths/math.color";
import { SmartArray } from "babylonjs/Misc/smartArray";
import { BaseTexture } from "babylonjs/Materials/Textures/baseTexture";
import { RenderTargetTexture } from "babylonjs/Materials/Textures/renderTargetTexture";
import { UniformBuffer } from "babylonjs/Materials/uniformBuffer";
import { EffectFallbacks } from "babylonjs/Materials/effectFallbacks";
import { SubMesh } from "babylonjs/Meshes/subMesh";
import { MaterialPluginBase } from "babylonjs/Materials/materialPluginBase";
import { MaterialDefines } from "babylonjs/Materials/materialDefines";
import { Engine } from "babylonjs/Engines/engine";
import { Scene } from "babylonjs/scene";
import { PBRBaseMaterial } from "babylonjs/Materials/PBR/pbrBaseMaterial";
/**
 * @internal
 */
export class MaterialSubSurfaceDefines extends MaterialDefines {
    SUBSURFACE: boolean;
    SS_REFRACTION: boolean;
    SS_REFRACTION_USE_INTENSITY_FROM_TEXTURE: boolean;
    SS_TRANSLUCENCY: boolean;
    SS_TRANSLUCENCY_USE_INTENSITY_FROM_TEXTURE: boolean;
    SS_SCATTERING: boolean;
    SS_THICKNESSANDMASK_TEXTURE: boolean;
    SS_THICKNESSANDMASK_TEXTUREDIRECTUV: number;
    SS_HAS_THICKNESS: boolean;
    SS_REFRACTIONINTENSITY_TEXTURE: boolean;
    SS_REFRACTIONINTENSITY_TEXTUREDIRECTUV: number;
    SS_TRANSLUCENCYINTENSITY_TEXTURE: boolean;
    SS_TRANSLUCENCYINTENSITY_TEXTUREDIRECTUV: number;
    SS_REFRACTIONMAP_3D: boolean;
    SS_REFRACTIONMAP_OPPOSITEZ: boolean;
    SS_LODINREFRACTIONALPHA: boolean;
    SS_GAMMAREFRACTION: boolean;
    SS_RGBDREFRACTION: boolean;
    SS_LINEARSPECULARREFRACTION: boolean;
    SS_LINKREFRACTIONTOTRANSPARENCY: boolean;
    SS_ALBEDOFORREFRACTIONTINT: boolean;
    SS_ALBEDOFORTRANSLUCENCYTINT: boolean;
    SS_USE_LOCAL_REFRACTIONMAP_CUBIC: boolean;
    SS_USE_THICKNESS_AS_DEPTH: boolean;
    SS_MASK_FROM_THICKNESS_TEXTURE: boolean;
    SS_USE_GLTF_TEXTURES: boolean;
}
/**
 * Plugin that implements the sub surface component of the PBR material
 */
export class PBRSubSurfaceConfiguration extends MaterialPluginBase {
    protected _material: PBRBaseMaterial;
    private _isRefractionEnabled;
    /**
     * Defines if the refraction is enabled in the material.
     */
    isRefractionEnabled: boolean;
    private _isTranslucencyEnabled;
    /**
     * Defines if the translucency is enabled in the material.
     */
    isTranslucencyEnabled: boolean;
    private _isScatteringEnabled;
    /**
     * Defines if the sub surface scattering is enabled in the material.
     */
    isScatteringEnabled: boolean;
    private _scatteringDiffusionProfileIndex;
    /**
     * Diffusion profile for subsurface scattering.
     * Useful for better scattering in the skins or foliages.
     */
    get scatteringDiffusionProfile(): Nullable<Color3>;
    set scatteringDiffusionProfile(c: Nullable<Color3>);
    /**
     * Defines the refraction intensity of the material.
     * The refraction when enabled replaces the Diffuse part of the material.
     * The intensity helps transitioning between diffuse and refraction.
     */
    refractionIntensity: number;
    /**
     * Defines the translucency intensity of the material.
     * When translucency has been enabled, this defines how much of the "translucency"
     * is added to the diffuse part of the material.
     */
    translucencyIntensity: number;
    /**
     * When enabled, transparent surfaces will be tinted with the albedo colour (independent of thickness)
     */
    useAlbedoToTintRefraction: boolean;
    /**
     * When enabled, translucent surfaces will be tinted with the albedo colour (independent of thickness)
     */
    useAlbedoToTintTranslucency: boolean;
    private _thicknessTexture;
    /**
     * Stores the average thickness of a mesh in a texture (The texture is holding the values linearly).
     * The red (or green if useGltfStyleTextures=true) channel of the texture should contain the thickness remapped between 0 and 1.
     * 0 would mean minimumThickness
     * 1 would mean maximumThickness
     * The other channels might be use as a mask to vary the different effects intensity.
     */
    thicknessTexture: Nullable<BaseTexture>;
    private _refractionTexture;
    /**
     * Defines the texture to use for refraction.
     */
    refractionTexture: Nullable<BaseTexture>;
    /** @internal */
    _indexOfRefraction: number;
    /**
     * Index of refraction of the material base layer.
     * https://en.wikipedia.org/wiki/List_of_refractive_indices
     *
     * This does not only impact refraction but also the Base F0 of Dielectric Materials.
     *
     * From dielectric fresnel rules: F0 = square((iorT - iorI) / (iorT + iorI))
     */
    indexOfRefraction: number;
    private _volumeIndexOfRefraction;
    /**
     * Index of refraction of the material's volume.
     * https://en.wikipedia.org/wiki/List_of_refractive_indices
     *
     * This ONLY impacts refraction. If not provided or given a non-valid value,
     * the volume will use the same IOR as the surface.
     */
    get volumeIndexOfRefraction(): number;
    set volumeIndexOfRefraction(value: number);
    private _invertRefractionY;
    /**
     * Controls if refraction needs to be inverted on Y. This could be useful for procedural texture.
     */
    invertRefractionY: boolean;
    /** @internal */
    _linkRefractionWithTransparency: boolean;
    /**
     * This parameters will make the material used its opacity to control how much it is refracting against not.
     * Materials half opaque for instance using refraction could benefit from this control.
     */
    linkRefractionWithTransparency: boolean;
    /**
     * Defines the minimum thickness stored in the thickness map.
     * If no thickness map is defined, this value will be used to simulate thickness.
     */
    minimumThickness: number;
    /**
     * Defines the maximum thickness stored in the thickness map.
     */
    maximumThickness: number;
    /**
     * Defines that the thickness should be used as a measure of the depth volume.
     */
    useThicknessAsDepth: boolean;
    /**
     * Defines the volume tint of the material.
     * This is used for both translucency and scattering.
     */
    tintColor: Color3;
    /**
     * Defines the distance at which the tint color should be found in the media.
     * This is used for refraction only.
     */
    tintColorAtDistance: number;
    /**
     * Defines how far each channel transmit through the media.
     * It is defined as a color to simplify it selection.
     */
    diffusionDistance: Color3;
    private _useMaskFromThicknessTexture;
    /**
     * Stores the intensity of the different subsurface effects in the thickness texture.
     * Note that if refractionIntensityTexture and/or translucencyIntensityTexture is provided it takes precedence over thicknessTexture + useMaskFromThicknessTexture
     * * the green (red if useGltfStyleTextures = true) channel is the refraction intensity.
     * * the blue channel is the translucency intensity.
     */
    useMaskFromThicknessTexture: boolean;
    private _refractionIntensityTexture;
    /**
     * Stores the intensity of the refraction. If provided, it takes precedence over thicknessTexture + useMaskFromThicknessTexture
     * * the green (red if useGltfStyleTextures = true) channel is the refraction intensity.
     */
    refractionIntensityTexture: Nullable<BaseTexture>;
    private _translucencyIntensityTexture;
    /**
     * Stores the intensity of the translucency. If provided, it takes precedence over thicknessTexture + useMaskFromThicknessTexture
     * * the blue channel is the translucency intensity.
     */
    translucencyIntensityTexture: Nullable<BaseTexture>;
    private _scene;
    private _useGltfStyleTextures;
    /**
     * Use channels layout used by glTF:
     * * thicknessTexture: the green (instead of red) channel is the thickness
     * * thicknessTexture/refractionIntensityTexture: the red (instead of green) channel is the refraction intensity
     * * thicknessTexture/translucencyIntensityTexture: no change, use the blue channel for the translucency intensity
     */
    useGltfStyleTextures: boolean;
    /** @internal */
    private _internalMarkAllSubMeshesAsTexturesDirty;
    private _internalMarkScenePrePassDirty;
    /** @internal */
    _markAllSubMeshesAsTexturesDirty(): void;
    /** @internal */
    _markScenePrePassDirty(): void;
    constructor(material: PBRBaseMaterial, addToPluginList?: boolean);
    isReadyForSubMesh(defines: MaterialSubSurfaceDefines, scene: Scene): boolean;
    prepareDefinesBeforeAttributes(defines: MaterialSubSurfaceDefines, scene: Scene): void;
    /**
     * Binds the material data (this function is called even if mustRebind() returns false)
     * @param uniformBuffer defines the Uniform buffer to fill in.
     * @param scene defines the scene the material belongs to.
     * @param engine defines the engine the material belongs to.
     * @param subMesh the submesh to bind data for
     */
    hardBindForSubMesh(uniformBuffer: UniformBuffer, scene: Scene, engine: Engine, subMesh: SubMesh): void;
    bindForSubMesh(uniformBuffer: UniformBuffer, scene: Scene, engine: Engine, subMesh: SubMesh): void;
    /**
     * Returns the texture used for refraction or null if none is used.
     * @param scene defines the scene the material belongs to.
     * @returns - Refraction texture if present.  If no refraction texture and refraction
     * is linked with transparency, returns environment texture.  Otherwise, returns null.
     */
    private _getRefractionTexture;
    /**
     * Returns true if alpha blending should be disabled.
     */
    get disableAlphaBlending(): boolean;
    /**
     * Fills the list of render target textures.
     * @param renderTargets the list of render targets to update
     */
    fillRenderTargetTextures(renderTargets: SmartArray<RenderTargetTexture>): void;
    hasTexture(texture: BaseTexture): boolean;
    hasRenderTargetTextures(): boolean;
    getActiveTextures(activeTextures: BaseTexture[]): void;
    getAnimatables(animatables: IAnimatable[]): void;
    dispose(forceDisposeTextures?: boolean): void;
    getClassName(): string;
    addFallbacks(defines: MaterialSubSurfaceDefines, fallbacks: EffectFallbacks, currentRank: number): number;
    getSamplers(samplers: string[]): void;
    getUniforms(): {
        ubo?: Array<{
            name: string;
            size: number;
            type: string;
        }>;
        vertex?: string;
        fragment?: string;
    };
}
export {};

}
declare module "babylonjs/Materials/prePassConfiguration" {
import { Matrix } from "babylonjs/Maths/math.vector";
import { Mesh } from "babylonjs/Meshes/mesh";
import { Scene } from "babylonjs/scene";
import { Effect } from "babylonjs/Materials/effect";
/**
 * Configuration needed for prepass-capable materials
 */
export class PrePassConfiguration {
    /**
     * Previous world matrices of meshes carrying this material
     * Used for computing velocity
     */
    previousWorldMatrices: {
        [index: number]: Matrix;
    };
    /**
     * Previous view project matrix
     * Used for computing velocity
     */
    previousViewProjection: Matrix;
    /**
     * Current view projection matrix
     * Used for computing velocity
     */
    currentViewProjection: Matrix;
    /**
     * Previous bones of meshes carrying this material
     * Used for computing velocity
     */
    previousBones: {
        [index: number]: Float32Array;
    };
    private _lastUpdateFrameId;
    /**
     * Add the required uniforms to the current list.
     * @param uniforms defines the current uniform list.
     */
    static AddUniforms(uniforms: string[]): void;
    /**
     * Add the required samplers to the current list.
     * @param samplers defines the current sampler list.
     */
    static AddSamplers(samplers: string[]): void;
    /**
     * Binds the material data.
     * @param effect defines the effect to update
     * @param scene defines the scene the material belongs to.
     * @param mesh The mesh
     * @param world World matrix of this mesh
     * @param isFrozen Is the material frozen
     */
    bindForSubMesh(effect: Effect, scene: Scene, mesh: Mesh, world: Matrix, isFrozen: boolean): void;
}

}
declare module "babylonjs/Materials/pushMaterial" {
import { Nullable } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
import { Matrix } from "babylonjs/Maths/math.vector";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { Mesh } from "babylonjs/Meshes/mesh";
import { Material } from "babylonjs/Materials/material";
import { Effect } from "babylonjs/Materials/effect";
import { SubMesh } from "babylonjs/Meshes/subMesh";
/**
 * Base class of materials working in push mode in babylon JS
 * @internal
 */
export class PushMaterial extends Material {
    protected _activeEffect: Effect;
    protected _normalMatrix: Matrix;
    constructor(name: string, scene?: Scene, storeEffectOnSubMeshes?: boolean);
    getEffect(): Effect;
    isReady(mesh?: AbstractMesh, useInstances?: boolean): boolean;
    protected _isReadyForSubMesh(subMesh: SubMesh): boolean;
    /**
     * Binds the given world matrix to the active effect
     *
     * @param world the matrix to bind
     */
    bindOnlyWorldMatrix(world: Matrix): void;
    /**
     * Binds the given normal matrix to the active effect
     *
     * @param normalMatrix the matrix to bind
     */
    bindOnlyNormalMatrix(normalMatrix: Matrix): void;
    bind(world: Matrix, mesh?: Mesh): void;
    protected _afterBind(mesh?: Mesh, effect?: Nullable<Effect>): void;
    protected _mustRebind(scene: Scene, effect: Effect, visibility?: number): boolean;
}

}
declare module "babylonjs/Materials/shaderLanguage" {
/**
 * Language of the shader code
 */
export enum ShaderLanguage {
    /** language is GLSL (used by WebGL) */
    GLSL = 0,
    /** language is WGSL (used by WebGPU) */
    WGSL = 1
}

}
declare module "babylonjs/Materials/shaderMaterial" {
import { Nullable } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
import { Matrix, Vector3, Vector2, Vector4, Quaternion } from "babylonjs/Maths/math.vector";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { Mesh } from "babylonjs/Meshes/mesh";
import { SubMesh } from "babylonjs/Meshes/subMesh";
import { BaseTexture } from "babylonjs/Materials/Textures/baseTexture";
import { Effect } from "babylonjs/Materials/effect";
import { Color3, Color4 } from "babylonjs/Maths/math.color";
import { ShaderLanguage } from "babylonjs/Materials/shaderLanguage";
import { UniformBuffer } from "babylonjs/Materials/uniformBuffer";
import { TextureSampler } from "babylonjs/Materials/Textures/textureSampler";
import { StorageBuffer } from "babylonjs/Buffers/storageBuffer";
import { PushMaterial } from "babylonjs/Materials/pushMaterial";
import { ExternalTexture } from "babylonjs/Materials/Textures/externalTexture";
/**
 * Defines the options associated with the creation of a shader material.
 */
export interface IShaderMaterialOptions {
    /**
     * Does the material work in alpha blend mode
     */
    needAlphaBlending: boolean;
    /**
     * Does the material work in alpha test mode
     */
    needAlphaTesting: boolean;
    /**
     * The list of attribute names used in the shader
     */
    attributes: string[];
    /**
     * The list of uniform names used in the shader
     */
    uniforms: string[];
    /**
     * The list of UBO names used in the shader
     */
    uniformBuffers: string[];
    /**
     * The list of sampler (texture) names used in the shader
     */
    samplers: string[];
    /**
     * The list of external texture names used in the shader
     */
    externalTextures: string[];
    /**
     * The list of sampler object names used in the shader
     */
    samplerObjects: string[];
    /**
     * The list of storage buffer names used in the shader
     */
    storageBuffers: string[];
    /**
     * The list of defines used in the shader
     */
    defines: string[];
    /**
     * Defines if clip planes have to be turned on: true to turn them on, false to turn them off and null to turn them on/off depending on the scene configuration (scene.clipPlaneX)
     */
    useClipPlane: Nullable<boolean>;
    /**
     * The language the shader is written in (default: GLSL)
     */
    shaderLanguage?: ShaderLanguage;
}
/**
 * The ShaderMaterial object has the necessary methods to pass data from your scene to the Vertex and Fragment Shaders and returns a material that can be applied to any mesh.
 *
 * This returned material effects how the mesh will look based on the code in the shaders.
 *
 * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/shaders/shaderMaterial
 */
export class ShaderMaterial extends PushMaterial {
    private _shaderPath;
    private _options;
    private _textures;
    private _textureArrays;
    private _externalTextures;
    private _floats;
    private _ints;
    private _uints;
    private _floatsArrays;
    private _colors3;
    private _colors3Arrays;
    private _colors4;
    private _colors4Arrays;
    private _vectors2;
    private _vectors3;
    private _vectors4;
    private _quaternions;
    private _quaternionsArrays;
    private _matrices;
    private _matrixArrays;
    private _matrices3x3;
    private _matrices2x2;
    private _vectors2Arrays;
    private _vectors3Arrays;
    private _vectors4Arrays;
    private _uniformBuffers;
    private _textureSamplers;
    private _storageBuffers;
    private _cachedWorldViewMatrix;
    private _cachedWorldViewProjectionMatrix;
    private _multiview;
    /** Define the Url to load snippets */
    static SnippetUrl: string;
    /** Snippet ID if the material was created from the snippet server */
    snippetId: string;
    /**
     * Instantiate a new shader material.
     * The ShaderMaterial object has the necessary methods to pass data from your scene to the Vertex and Fragment Shaders and returns a material that can be applied to any mesh.
     * This returned material effects how the mesh will look based on the code in the shaders.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/shaders/shaderMaterial
     * @param name Define the name of the material in the scene
     * @param scene Define the scene the material belongs to
     * @param shaderPath Defines  the route to the shader code in one of three ways:
     *  * object: { vertex: "custom", fragment: "custom" }, used with Effect.ShadersStore["customVertexShader"] and Effect.ShadersStore["customFragmentShader"]
     *  * object: { vertexElement: "vertexShaderCode", fragmentElement: "fragmentShaderCode" }, used with shader code in script tags
     *  * object: { vertexSource: "vertex shader code string", fragmentSource: "fragment shader code string" } using with strings containing the shaders code
     *  * string: "./COMMON_NAME", used with external files COMMON_NAME.vertex.fx and COMMON_NAME.fragment.fx in index.html folder.
     * @param options Define the options used to create the shader
     * @param storeEffectOnSubMeshes true to store effect on submeshes, false to store the effect directly in the material class.
     */
    constructor(name: string, scene: Scene, shaderPath: any, options?: Partial<IShaderMaterialOptions>, storeEffectOnSubMeshes?: boolean);
    /**
     * Gets the shader path used to define the shader code
     * It can be modified to trigger a new compilation
     */
    get shaderPath(): any;
    /**
     * Sets the shader path used to define the shader code
     * It can be modified to trigger a new compilation
     */
    set shaderPath(shaderPath: any);
    /**
     * Gets the options used to compile the shader.
     * They can be modified to trigger a new compilation
     */
    get options(): IShaderMaterialOptions;
    /**
     * Gets the current class name of the material e.g. "ShaderMaterial"
     * Mainly use in serialization.
     * @returns the class name
     */
    getClassName(): string;
    /**
     * Specifies if the material will require alpha blending
     * @returns a boolean specifying if alpha blending is needed
     */
    needAlphaBlending(): boolean;
    /**
     * Specifies if this material should be rendered in alpha test mode
     * @returns a boolean specifying if an alpha test is needed.
     */
    needAlphaTesting(): boolean;
    private _checkUniform;
    /**
     * Set a texture in the shader.
     * @param name Define the name of the uniform samplers as defined in the shader
     * @param texture Define the texture to bind to this sampler
     * @returns the material itself allowing "fluent" like uniform updates
     */
    setTexture(name: string, texture: BaseTexture): ShaderMaterial;
    /**
     * Set a texture array in the shader.
     * @param name Define the name of the uniform sampler array as defined in the shader
     * @param textures Define the list of textures to bind to this sampler
     * @returns the material itself allowing "fluent" like uniform updates
     */
    setTextureArray(name: string, textures: BaseTexture[]): ShaderMaterial;
    /**
     * Set an internal texture in the shader.
     * @param name Define the name of the uniform samplers as defined in the shader
     * @param texture Define the texture to bind to this sampler
     * @returns the material itself allowing "fluent" like uniform updates
     */
    setExternalTexture(name: string, texture: ExternalTexture): ShaderMaterial;
    /**
     * Set a float in the shader.
     * @param name Define the name of the uniform as defined in the shader
     * @param value Define the value to give to the uniform
     * @returns the material itself allowing "fluent" like uniform updates
     */
    setFloat(name: string, value: number): ShaderMaterial;
    /**
     * Set a int in the shader.
     * @param name Define the name of the uniform as defined in the shader
     * @param value Define the value to give to the uniform
     * @returns the material itself allowing "fluent" like uniform updates
     */
    setInt(name: string, value: number): ShaderMaterial;
    /**
     * Set a unsigned int in the shader.
     * @param name Define the name of the uniform as defined in the shader
     * @param value Define the value to give to the uniform
     * @return the material itself allowing "fluent" like uniform updates
     */
    setUInt(name: string, value: number): ShaderMaterial;
    /**
     * Set an array of floats in the shader.
     * @param name Define the name of the uniform as defined in the shader
     * @param value Define the value to give to the uniform
     * @returns the material itself allowing "fluent" like uniform updates
     */
    setFloats(name: string, value: number[]): ShaderMaterial;
    /**
     * Set a vec3 in the shader from a Color3.
     * @param name Define the name of the uniform as defined in the shader
     * @param value Define the value to give to the uniform
     * @returns the material itself allowing "fluent" like uniform updates
     */
    setColor3(name: string, value: Color3): ShaderMaterial;
    /**
     * Set a vec3 array in the shader from a Color3 array.
     * @param name Define the name of the uniform as defined in the shader
     * @param value Define the value to give to the uniform
     * @returns the material itself allowing "fluent" like uniform updates
     */
    setColor3Array(name: string, value: Color3[]): ShaderMaterial;
    /**
     * Set a vec4 in the shader from a Color4.
     * @param name Define the name of the uniform as defined in the shader
     * @param value Define the value to give to the uniform
     * @returns the material itself allowing "fluent" like uniform updates
     */
    setColor4(name: string, value: Color4): ShaderMaterial;
    /**
     * Set a vec4 array in the shader from a Color4 array.
     * @param name Define the name of the uniform as defined in the shader
     * @param value Define the value to give to the uniform
     * @returns the material itself allowing "fluent" like uniform updates
     */
    setColor4Array(name: string, value: Color4[]): ShaderMaterial;
    /**
     * Set a vec2 in the shader from a Vector2.
     * @param name Define the name of the uniform as defined in the shader
     * @param value Define the value to give to the uniform
     * @returns the material itself allowing "fluent" like uniform updates
     */
    setVector2(name: string, value: Vector2): ShaderMaterial;
    /**
     * Set a vec3 in the shader from a Vector3.
     * @param name Define the name of the uniform as defined in the shader
     * @param value Define the value to give to the uniform
     * @returns the material itself allowing "fluent" like uniform updates
     */
    setVector3(name: string, value: Vector3): ShaderMaterial;
    /**
     * Set a vec4 in the shader from a Vector4.
     * @param name Define the name of the uniform as defined in the shader
     * @param value Define the value to give to the uniform
     * @returns the material itself allowing "fluent" like uniform updates
     */
    setVector4(name: string, value: Vector4): ShaderMaterial;
    /**
     * Set a vec4 in the shader from a Quaternion.
     * @param name Define the name of the uniform as defined in the shader
     * @param value Define the value to give to the uniform
     * @returns the material itself allowing "fluent" like uniform updates
     */
    setQuaternion(name: string, value: Quaternion): ShaderMaterial;
    /**
     * Set a vec4 array in the shader from a Quaternion array.
     * @param name Define the name of the uniform as defined in the shader
     * @param value Define the value to give to the uniform
     * @returns the material itself allowing "fluent" like uniform updates
     */
    setQuaternionArray(name: string, value: Quaternion[]): ShaderMaterial;
    /**
     * Set a mat4 in the shader from a Matrix.
     * @param name Define the name of the uniform as defined in the shader
     * @param value Define the value to give to the uniform
     * @returns the material itself allowing "fluent" like uniform updates
     */
    setMatrix(name: string, value: Matrix): ShaderMaterial;
    /**
     * Set a float32Array in the shader from a matrix array.
     * @param name Define the name of the uniform as defined in the shader
     * @param value Define the value to give to the uniform
     * @returns the material itself allowing "fluent" like uniform updates
     */
    setMatrices(name: string, value: Matrix[]): ShaderMaterial;
    /**
     * Set a mat3 in the shader from a Float32Array.
     * @param name Define the name of the uniform as defined in the shader
     * @param value Define the value to give to the uniform
     * @returns the material itself allowing "fluent" like uniform updates
     */
    setMatrix3x3(name: string, value: Float32Array | Array<number>): ShaderMaterial;
    /**
     * Set a mat2 in the shader from a Float32Array.
     * @param name Define the name of the uniform as defined in the shader
     * @param value Define the value to give to the uniform
     * @returns the material itself allowing "fluent" like uniform updates
     */
    setMatrix2x2(name: string, value: Float32Array | Array<number>): ShaderMaterial;
    /**
     * Set a vec2 array in the shader from a number array.
     * @param name Define the name of the uniform as defined in the shader
     * @param value Define the value to give to the uniform
     * @returns the material itself allowing "fluent" like uniform updates
     */
    setArray2(name: string, value: number[]): ShaderMaterial;
    /**
     * Set a vec3 array in the shader from a number array.
     * @param name Define the name of the uniform as defined in the shader
     * @param value Define the value to give to the uniform
     * @returns the material itself allowing "fluent" like uniform updates
     */
    setArray3(name: string, value: number[]): ShaderMaterial;
    /**
     * Set a vec4 array in the shader from a number array.
     * @param name Define the name of the uniform as defined in the shader
     * @param value Define the value to give to the uniform
     * @returns the material itself allowing "fluent" like uniform updates
     */
    setArray4(name: string, value: number[]): ShaderMaterial;
    /**
     * Set a uniform buffer in the shader
     * @param name Define the name of the uniform as defined in the shader
     * @param buffer Define the value to give to the uniform
     * @returns the material itself allowing "fluent" like uniform updates
     */
    setUniformBuffer(name: string, buffer: UniformBuffer): ShaderMaterial;
    /**
     * Set a texture sampler in the shader
     * @param name Define the name of the uniform as defined in the shader
     * @param sampler Define the value to give to the uniform
     * @returns the material itself allowing "fluent" like uniform updates
     */
    setTextureSampler(name: string, sampler: TextureSampler): ShaderMaterial;
    /**
     * Set a storage buffer in the shader
     * @param name Define the name of the storage buffer as defined in the shader
     * @param buffer Define the value to give to the uniform
     * @returns the material itself allowing "fluent" like uniform updates
     */
    setStorageBuffer(name: string, buffer: StorageBuffer): ShaderMaterial;
    /**
     * Specifies that the submesh is ready to be used
     * @param mesh defines the mesh to check
     * @param subMesh defines which submesh to check
     * @param useInstances specifies that instances should be used
     * @returns a boolean indicating that the submesh is ready or not
     */
    isReadyForSubMesh(mesh: AbstractMesh, subMesh: SubMesh, useInstances?: boolean): boolean;
    /**
     * Checks if the material is ready to render the requested mesh
     * @param mesh Define the mesh to render
     * @param useInstances Define whether or not the material is used with instances
     * @param subMesh defines which submesh to render
     * @returns true if ready, otherwise false
     */
    isReady(mesh?: AbstractMesh, useInstances?: boolean, subMesh?: SubMesh): boolean;
    /**
     * Binds the world matrix to the material
     * @param world defines the world transformation matrix
     * @param effectOverride - If provided, use this effect instead of internal effect
     */
    bindOnlyWorldMatrix(world: Matrix, effectOverride?: Nullable<Effect>): void;
    /**
     * Binds the submesh to this material by preparing the effect and shader to draw
     * @param world defines the world transformation matrix
     * @param mesh defines the mesh containing the submesh
     * @param subMesh defines the submesh to bind the material to
     */
    bindForSubMesh(world: Matrix, mesh: Mesh, subMesh: SubMesh): void;
    /**
     * Binds the material to the mesh
     * @param world defines the world transformation matrix
     * @param mesh defines the mesh to bind the material to
     * @param effectOverride - If provided, use this effect instead of internal effect
     * @param subMesh defines the submesh to bind the material to
     */
    bind(world: Matrix, mesh?: Mesh, effectOverride?: Nullable<Effect>, subMesh?: SubMesh): void;
    /**
     * Gets the active textures from the material
     * @returns an array of textures
     */
    getActiveTextures(): BaseTexture[];
    /**
     * Specifies if the material uses a texture
     * @param texture defines the texture to check against the material
     * @returns a boolean specifying if the material uses the texture
     */
    hasTexture(texture: BaseTexture): boolean;
    /**
     * Makes a duplicate of the material, and gives it a new name
     * @param name defines the new name for the duplicated material
     * @returns the cloned material
     */
    clone(name: string): ShaderMaterial;
    /**
     * Disposes the material
     * @param forceDisposeEffect specifies if effects should be forcefully disposed
     * @param forceDisposeTextures specifies if textures should be forcefully disposed
     * @param notBoundToMesh specifies if the material that is being disposed is known to be not bound to any mesh
     */
    dispose(forceDisposeEffect?: boolean, forceDisposeTextures?: boolean, notBoundToMesh?: boolean): void;
    /**
     * Serializes this material in a JSON representation
     * @returns the serialized material object
     */
    serialize(): any;
    /**
     * Creates a shader material from parsed shader material data
     * @param source defines the JSON representation of the material
     * @param scene defines the hosting scene
     * @param rootUrl defines the root URL to use to load textures and relative dependencies
     * @returns a new material
     */
    static Parse(source: any, scene: Scene, rootUrl: string): ShaderMaterial;
    /**
     * Creates a new ShaderMaterial from a snippet saved in a remote file
     * @param name defines the name of the ShaderMaterial to create (can be null or empty to use the one from the json data)
     * @param url defines the url to load from
     * @param scene defines the hosting scene
     * @param rootUrl defines the root URL to use to load textures and relative dependencies
     * @returns a promise that will resolve to the new ShaderMaterial
     */
    static ParseFromFileAsync(name: Nullable<string>, url: string, scene: Scene, rootUrl?: string): Promise<ShaderMaterial>;
    /**
     * Creates a ShaderMaterial from a snippet saved by the Inspector
     * @param snippetId defines the snippet to load
     * @param scene defines the hosting scene
     * @param rootUrl defines the root URL to use to load textures and relative dependencies
     * @returns a promise that will resolve to the new ShaderMaterial
     */
    static ParseFromSnippetAsync(snippetId: string, scene: Scene, rootUrl?: string): Promise<ShaderMaterial>;
    /**
     * Creates a ShaderMaterial from a snippet saved by the Inspector
     * @deprecated Please use ParseFromSnippetAsync instead
     * @param snippetId defines the snippet to load
     * @param scene defines the hosting scene
     * @param rootUrl defines the root URL to use to load textures and relative dependencies
     * @returns a promise that will resolve to the new ShaderMaterial
     */
    static CreateFromSnippetAsync: typeof ShaderMaterial.ParseFromSnippetAsync;
}
export {};

}
declare module "babylonjs/Materials/shadowDepthWrapper" {
import { Nullable } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
import { SubMesh } from "babylonjs/Meshes/subMesh";
import { Material } from "babylonjs/Materials/material";
import { ShadowGenerator } from "babylonjs/Lights/Shadows/shadowGenerator";
import { DrawWrapper } from "babylonjs/Materials/drawWrapper";
/**
 * Options to be used when creating a shadow depth material
 */
export interface IIOptionShadowDepthMaterial {
    /** Variables in the vertex shader code that need to have their names remapped.
     * The format is: ["var_name", "var_remapped_name", "var_name", "var_remapped_name", ...]
     * "var_name" should be either: worldPos or vNormalW
     * So, if the variable holding the world position in your vertex shader is not named worldPos, you must tell the system
     * the name to use instead by using: ["worldPos", "myWorldPosVar"] assuming the variable is named myWorldPosVar in your code.
     * If the normal must also be remapped: ["worldPos", "myWorldPosVar", "vNormalW", "myWorldNormal"]
     */
    remappedVariables?: string[];
    /** Set standalone to true if the base material wrapped by ShadowDepthMaterial is not used for a regular object but for depth shadow generation only */
    standalone?: boolean;
    /** Set doNotInjectCode if the specific shadow map generation code is already implemented by the material. That will prevent this code to be injected twice by ShadowDepthWrapper */
    doNotInjectCode?: boolean;
}
/**
 * Class that can be used to wrap a base material to generate accurate shadows when using custom vertex/fragment code in the base material
 */
export class ShadowDepthWrapper {
    private _scene;
    private _options?;
    private _baseMaterial;
    private _onEffectCreatedObserver;
    private _subMeshToEffect;
    private _subMeshToDepthWrapper;
    private _meshes;
    /** Gets the standalone status of the wrapper */
    get standalone(): boolean;
    /** Gets the base material the wrapper is built upon */
    get baseMaterial(): Material;
    /** Gets the doNotInjectCode status of the wrapper */
    get doNotInjectCode(): boolean;
    /**
     * Instantiate a new shadow depth wrapper.
     * It works by injecting some specific code in the vertex/fragment shaders of the base material and is used by a shadow generator to
     * generate the shadow depth map. For more information, please refer to the documentation:
     * https://doc.babylonjs.com/features/featuresDeepDive/lights/shadows
     * @param baseMaterial Material to wrap
     * @param scene Define the scene the material belongs to
     * @param options Options used to create the wrapper
     */
    constructor(baseMaterial: Material, scene?: Scene, options?: IIOptionShadowDepthMaterial);
    /**
     * Gets the effect to use to generate the depth map
     * @param subMesh subMesh to get the effect for
     * @param shadowGenerator shadow generator to get the effect for
     * @param passIdForDrawWrapper Id of the pass for which the effect from the draw wrapper must be retrieved from
     * @returns the effect to use to generate the depth map for the subMesh + shadow generator specified
     */
    getEffect(subMesh: Nullable<SubMesh>, shadowGenerator: ShadowGenerator, passIdForDrawWrapper: number): Nullable<DrawWrapper>;
    /**
     * Specifies that the submesh is ready to be used for depth rendering
     * @param subMesh submesh to check
     * @param defines the list of defines to take into account when checking the effect
     * @param shadowGenerator combined with subMesh, it defines the effect to check
     * @param useInstances specifies that instances should be used
     * @param passIdForDrawWrapper Id of the pass for which the draw wrapper should be created
     * @returns a boolean indicating that the submesh is ready or not
     */
    isReadyForSubMesh(subMesh: SubMesh, defines: string[], shadowGenerator: ShadowGenerator, useInstances: boolean, passIdForDrawWrapper: number): boolean;
    /**
     * Disposes the resources
     */
    dispose(): void;
    private _makeEffect;
}

}
declare module "babylonjs/Materials/standardMaterial" {
import { SmartArray } from "babylonjs/Misc/smartArray";
import { IAnimatable } from "babylonjs/Animations/animatable.interface";
import { Nullable } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
import { Matrix } from "babylonjs/Maths/math.vector";
import { Color3 } from "babylonjs/Maths/math.color";
import { SubMesh } from "babylonjs/Meshes/subMesh";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { Mesh } from "babylonjs/Meshes/mesh";
import { PrePassConfiguration } from "babylonjs/Materials/prePassConfiguration";
import { IImageProcessingConfigurationDefines } from "babylonjs/Materials/imageProcessingConfiguration";
import { ImageProcessingConfiguration } from "babylonjs/Materials/imageProcessingConfiguration";
import { ColorCurves } from "babylonjs/Materials/colorCurves";
import { FresnelParameters } from "babylonjs/Materials/fresnelParameters";
import { MaterialDefines } from "babylonjs/Materials/materialDefines";
import { PushMaterial } from "babylonjs/Materials/pushMaterial";
import { BaseTexture } from "babylonjs/Materials/Textures/baseTexture";
import { RenderTargetTexture } from "babylonjs/Materials/Textures/renderTargetTexture";
import "babylonjs/Shaders/default.fragment";
import "babylonjs/Shaders/default.vertex";
import { DetailMapConfiguration } from "babylonjs/Materials/material.detailMapConfiguration";
/** @internal */
export class StandardMaterialDefines extends MaterialDefines implements IImageProcessingConfigurationDefines {
    MAINUV1: boolean;
    MAINUV2: boolean;
    MAINUV3: boolean;
    MAINUV4: boolean;
    MAINUV5: boolean;
    MAINUV6: boolean;
    DIFFUSE: boolean;
    DIFFUSEDIRECTUV: number;
    BAKED_VERTEX_ANIMATION_TEXTURE: boolean;
    AMBIENT: boolean;
    AMBIENTDIRECTUV: number;
    OPACITY: boolean;
    OPACITYDIRECTUV: number;
    OPACITYRGB: boolean;
    REFLECTION: boolean;
    EMISSIVE: boolean;
    EMISSIVEDIRECTUV: number;
    SPECULAR: boolean;
    SPECULARDIRECTUV: number;
    BUMP: boolean;
    BUMPDIRECTUV: number;
    PARALLAX: boolean;
    PARALLAXOCCLUSION: boolean;
    SPECULAROVERALPHA: boolean;
    CLIPPLANE: boolean;
    CLIPPLANE2: boolean;
    CLIPPLANE3: boolean;
    CLIPPLANE4: boolean;
    CLIPPLANE5: boolean;
    CLIPPLANE6: boolean;
    ALPHATEST: boolean;
    DEPTHPREPASS: boolean;
    ALPHAFROMDIFFUSE: boolean;
    POINTSIZE: boolean;
    FOG: boolean;
    SPECULARTERM: boolean;
    DIFFUSEFRESNEL: boolean;
    OPACITYFRESNEL: boolean;
    REFLECTIONFRESNEL: boolean;
    REFRACTIONFRESNEL: boolean;
    EMISSIVEFRESNEL: boolean;
    FRESNEL: boolean;
    NORMAL: boolean;
    TANGENT: boolean;
    UV1: boolean;
    UV2: boolean;
    UV3: boolean;
    UV4: boolean;
    UV5: boolean;
    UV6: boolean;
    VERTEXCOLOR: boolean;
    VERTEXALPHA: boolean;
    NUM_BONE_INFLUENCERS: number;
    BonesPerMesh: number;
    BONETEXTURE: boolean;
    BONES_VELOCITY_ENABLED: boolean;
    INSTANCES: boolean;
    THIN_INSTANCES: boolean;
    INSTANCESCOLOR: boolean;
    GLOSSINESS: boolean;
    ROUGHNESS: boolean;
    EMISSIVEASILLUMINATION: boolean;
    LINKEMISSIVEWITHDIFFUSE: boolean;
    REFLECTIONFRESNELFROMSPECULAR: boolean;
    LIGHTMAP: boolean;
    LIGHTMAPDIRECTUV: number;
    OBJECTSPACE_NORMALMAP: boolean;
    USELIGHTMAPASSHADOWMAP: boolean;
    REFLECTIONMAP_3D: boolean;
    REFLECTIONMAP_SPHERICAL: boolean;
    REFLECTIONMAP_PLANAR: boolean;
    REFLECTIONMAP_CUBIC: boolean;
    USE_LOCAL_REFLECTIONMAP_CUBIC: boolean;
    USE_LOCAL_REFRACTIONMAP_CUBIC: boolean;
    REFLECTIONMAP_PROJECTION: boolean;
    REFLECTIONMAP_SKYBOX: boolean;
    REFLECTIONMAP_EXPLICIT: boolean;
    REFLECTIONMAP_EQUIRECTANGULAR: boolean;
    REFLECTIONMAP_EQUIRECTANGULAR_FIXED: boolean;
    REFLECTIONMAP_MIRROREDEQUIRECTANGULAR_FIXED: boolean;
    REFLECTIONMAP_OPPOSITEZ: boolean;
    INVERTCUBICMAP: boolean;
    LOGARITHMICDEPTH: boolean;
    REFRACTION: boolean;
    REFRACTIONMAP_3D: boolean;
    REFLECTIONOVERALPHA: boolean;
    TWOSIDEDLIGHTING: boolean;
    SHADOWFLOAT: boolean;
    MORPHTARGETS: boolean;
    MORPHTARGETS_NORMAL: boolean;
    MORPHTARGETS_TANGENT: boolean;
    MORPHTARGETS_UV: boolean;
    NUM_MORPH_INFLUENCERS: number;
    MORPHTARGETS_TEXTURE: boolean;
    NONUNIFORMSCALING: boolean;
    PREMULTIPLYALPHA: boolean;
    ALPHATEST_AFTERALLALPHACOMPUTATIONS: boolean;
    ALPHABLEND: boolean;
    PREPASS: boolean;
    PREPASS_IRRADIANCE: boolean;
    PREPASS_IRRADIANCE_INDEX: number;
    PREPASS_ALBEDO_SQRT: boolean;
    PREPASS_ALBEDO_SQRT_INDEX: number;
    PREPASS_DEPTH: boolean;
    PREPASS_DEPTH_INDEX: number;
    PREPASS_NORMAL: boolean;
    PREPASS_NORMAL_INDEX: number;
    PREPASS_POSITION: boolean;
    PREPASS_POSITION_INDEX: number;
    PREPASS_VELOCITY: boolean;
    PREPASS_VELOCITY_INDEX: number;
    PREPASS_REFLECTIVITY: boolean;
    PREPASS_REFLECTIVITY_INDEX: number;
    SCENE_MRT_COUNT: number;
    RGBDLIGHTMAP: boolean;
    RGBDREFLECTION: boolean;
    RGBDREFRACTION: boolean;
    IMAGEPROCESSING: boolean;
    VIGNETTE: boolean;
    VIGNETTEBLENDMODEMULTIPLY: boolean;
    VIGNETTEBLENDMODEOPAQUE: boolean;
    TONEMAPPING: boolean;
    TONEMAPPING_ACES: boolean;
    CONTRAST: boolean;
    COLORCURVES: boolean;
    COLORGRADING: boolean;
    COLORGRADING3D: boolean;
    SAMPLER3DGREENDEPTH: boolean;
    SAMPLER3DBGRMAP: boolean;
    DITHER: boolean;
    IMAGEPROCESSINGPOSTPROCESS: boolean;
    SKIPFINALCOLORCLAMP: boolean;
    MULTIVIEW: boolean;
    ORDER_INDEPENDENT_TRANSPARENCY: boolean;
    ORDER_INDEPENDENT_TRANSPARENCY_16BITS: boolean;
    CAMERA_ORTHOGRAPHIC: boolean;
    CAMERA_PERSPECTIVE: boolean;
    /**
     * If the reflection texture on this material is in linear color space
     * @internal
     */
    IS_REFLECTION_LINEAR: boolean;
    /**
     * If the refraction texture on this material is in linear color space
     * @internal
     */
    IS_REFRACTION_LINEAR: boolean;
    EXPOSURE: boolean;
    /**
     * Initializes the Standard Material defines.
     * @param externalProperties The external properties
     */
    constructor(externalProperties?: {
        [name: string]: {
            type: string;
            default: any;
        };
    });
    setReflectionMode(modeToEnable: string): void;
}
/**
 * This is the default material used in Babylon. It is the best trade off between quality
 * and performances.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/materials_introduction
 */
export class StandardMaterial extends PushMaterial {
    private _diffuseTexture;
    /**
     * The basic texture of the material as viewed under a light.
     */
    diffuseTexture: Nullable<BaseTexture>;
    private _ambientTexture;
    /**
     * AKA Occlusion Texture in other nomenclature, it helps adding baked shadows into your material.
     */
    ambientTexture: Nullable<BaseTexture>;
    private _opacityTexture;
    /**
     * Define the transparency of the material from a texture.
     * The final alpha value can be read either from the red channel (if texture.getAlphaFromRGB is false)
     * or from the luminance or the current texel (if texture.getAlphaFromRGB is true)
     */
    opacityTexture: Nullable<BaseTexture>;
    private _reflectionTexture;
    /**
     * Define the texture used to display the reflection.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/reflectionTexture#how-to-obtain-reflections-and-refractions
     */
    reflectionTexture: Nullable<BaseTexture>;
    private _emissiveTexture;
    /**
     * Define texture of the material as if self lit.
     * This will be mixed in the final result even in the absence of light.
     */
    emissiveTexture: Nullable<BaseTexture>;
    private _specularTexture;
    /**
     * Define how the color and intensity of the highlight given by the light in the material.
     */
    specularTexture: Nullable<BaseTexture>;
    private _bumpTexture;
    /**
     * Bump mapping is a technique to simulate bump and dents on a rendered surface.
     * These are made by creating a normal map from an image. The means to do this can be found on the web, a search for 'normal map generator' will bring up free and paid for methods of doing this.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/moreMaterials#bump-map
     */
    bumpTexture: Nullable<BaseTexture>;
    private _lightmapTexture;
    /**
     * Complex lighting can be computationally expensive to compute at runtime.
     * To save on computation, lightmaps may be used to store calculated lighting in a texture which will be applied to a given mesh.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/lights/lights_introduction#lightmaps
     */
    lightmapTexture: Nullable<BaseTexture>;
    private _refractionTexture;
    /**
     * Define the texture used to display the refraction.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/reflectionTexture#how-to-obtain-reflections-and-refractions
     */
    refractionTexture: Nullable<BaseTexture>;
    /**
     * The color of the material lit by the environmental background lighting.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/materials_introduction#ambient-color-example
     */
    ambientColor: Color3;
    /**
     * The basic color of the material as viewed under a light.
     */
    diffuseColor: Color3;
    /**
     * Define how the color and intensity of the highlight given by the light in the material.
     */
    specularColor: Color3;
    /**
     * Define the color of the material as if self lit.
     * This will be mixed in the final result even in the absence of light.
     */
    emissiveColor: Color3;
    /**
     * Defines how sharp are the highlights in the material.
     * The bigger the value the sharper giving a more glossy feeling to the result.
     * Reversely, the smaller the value the blurrier giving a more rough feeling to the result.
     */
    specularPower: number;
    private _useAlphaFromDiffuseTexture;
    /**
     * Does the transparency come from the diffuse texture alpha channel.
     */
    useAlphaFromDiffuseTexture: boolean;
    private _useEmissiveAsIllumination;
    /**
     * If true, the emissive value is added into the end result, otherwise it is multiplied in.
     */
    useEmissiveAsIllumination: boolean;
    private _linkEmissiveWithDiffuse;
    /**
     * If true, some kind of energy conservation will prevent the end result to be more than 1 by reducing
     * the emissive level when the final color is close to one.
     */
    linkEmissiveWithDiffuse: boolean;
    private _useSpecularOverAlpha;
    /**
     * Specifies that the material will keep the specular highlights over a transparent surface (only the most luminous ones).
     * A car glass is a good exemple of that. When sun reflects on it you can not see what is behind.
     */
    useSpecularOverAlpha: boolean;
    private _useReflectionOverAlpha;
    /**
     * Specifies that the material will keeps the reflection highlights over a transparent surface (only the most luminous ones).
     * A car glass is a good exemple of that. When the street lights reflects on it you can not see what is behind.
     */
    useReflectionOverAlpha: boolean;
    private _disableLighting;
    /**
     * Does lights from the scene impacts this material.
     * It can be a nice trick for performance to disable lighting on a fully emissive material.
     */
    disableLighting: boolean;
    private _useObjectSpaceNormalMap;
    /**
     * Allows using an object space normal map (instead of tangent space).
     */
    useObjectSpaceNormalMap: boolean;
    private _useParallax;
    /**
     * Is parallax enabled or not.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/parallaxMapping
     */
    useParallax: boolean;
    private _useParallaxOcclusion;
    /**
     * Is parallax occlusion enabled or not.
     * If true, the outcome is way more realistic than traditional Parallax but you can expect a performance hit that worthes consideration.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/parallaxMapping
     */
    useParallaxOcclusion: boolean;
    /**
     * Apply a scaling factor that determine which "depth" the height map should reprensent. A value between 0.05 and 0.1 is reasonnable in Parallax, you can reach 0.2 using Parallax Occlusion.
     */
    parallaxScaleBias: number;
    private _roughness;
    /**
     * Helps to define how blurry the reflections should appears in the material.
     */
    roughness: number;
    /**
     * In case of refraction, define the value of the index of refraction.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/reflectionTexture#how-to-obtain-reflections-and-refractions
     */
    indexOfRefraction: number;
    /**
     * Invert the refraction texture alongside the y axis.
     * It can be useful with procedural textures or probe for instance.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/reflectionTexture#how-to-obtain-reflections-and-refractions
     */
    invertRefractionY: boolean;
    /**
     * Defines the alpha limits in alpha test mode.
     */
    alphaCutOff: number;
    private _useLightmapAsShadowmap;
    /**
     * In case of light mapping, define whether the map contains light or shadow informations.
     */
    useLightmapAsShadowmap: boolean;
    private _diffuseFresnelParameters;
    /**
     * Define the diffuse fresnel parameters of the material.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/fresnelParameters
     */
    diffuseFresnelParameters: FresnelParameters;
    private _opacityFresnelParameters;
    /**
     * Define the opacity fresnel parameters of the material.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/fresnelParameters
     */
    opacityFresnelParameters: FresnelParameters;
    private _reflectionFresnelParameters;
    /**
     * Define the reflection fresnel parameters of the material.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/fresnelParameters
     */
    reflectionFresnelParameters: FresnelParameters;
    private _refractionFresnelParameters;
    /**
     * Define the refraction fresnel parameters of the material.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/fresnelParameters
     */
    refractionFresnelParameters: FresnelParameters;
    private _emissiveFresnelParameters;
    /**
     * Define the emissive fresnel parameters of the material.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/fresnelParameters
     */
    emissiveFresnelParameters: FresnelParameters;
    private _useReflectionFresnelFromSpecular;
    /**
     * If true automatically deducts the fresnels values from the material specularity.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/fresnelParameters
     */
    useReflectionFresnelFromSpecular: boolean;
    private _useGlossinessFromSpecularMapAlpha;
    /**
     * Defines if the glossiness/roughness of the material should be read from the specular map alpha channel
     */
    useGlossinessFromSpecularMapAlpha: boolean;
    private _maxSimultaneousLights;
    /**
     * Defines the maximum number of lights that can be used in the material
     */
    maxSimultaneousLights: number;
    private _invertNormalMapX;
    /**
     * If sets to true, x component of normal map value will invert (x = 1.0 - x).
     */
    invertNormalMapX: boolean;
    private _invertNormalMapY;
    /**
     * If sets to true, y component of normal map value will invert (y = 1.0 - y).
     */
    invertNormalMapY: boolean;
    private _twoSidedLighting;
    /**
     * If sets to true and backfaceCulling is false, normals will be flipped on the backside.
     */
    twoSidedLighting: boolean;
    /**
     * Default configuration related to image processing available in the standard Material.
     */
    protected _imageProcessingConfiguration: ImageProcessingConfiguration;
    /**
     * Gets the image processing configuration used either in this material.
     */
    get imageProcessingConfiguration(): ImageProcessingConfiguration;
    /**
     * Sets the Default image processing configuration used either in the this material.
     *
     * If sets to null, the scene one is in use.
     */
    set imageProcessingConfiguration(value: ImageProcessingConfiguration);
    /**
     * Keep track of the image processing observer to allow dispose and replace.
     */
    private _imageProcessingObserver;
    /**
     * Attaches a new image processing configuration to the Standard Material.
     * @param configuration
     */
    protected _attachImageProcessingConfiguration(configuration: Nullable<ImageProcessingConfiguration>): void;
    /**
     * Defines additional PrePass parameters for the material.
     */
    readonly prePassConfiguration: PrePassConfiguration;
    /**
     * Can this material render to prepass
     */
    get isPrePassCapable(): boolean;
    /**
     * Gets whether the color curves effect is enabled.
     */
    get cameraColorCurvesEnabled(): boolean;
    /**
     * Sets whether the color curves effect is enabled.
     */
    set cameraColorCurvesEnabled(value: boolean);
    /**
     * Gets whether the color grading effect is enabled.
     */
    get cameraColorGradingEnabled(): boolean;
    /**
     * Gets whether the color grading effect is enabled.
     */
    set cameraColorGradingEnabled(value: boolean);
    /**
     * Gets whether tonemapping is enabled or not.
     */
    get cameraToneMappingEnabled(): boolean;
    /**
     * Sets whether tonemapping is enabled or not
     */
    set cameraToneMappingEnabled(value: boolean);
    /**
     * The camera exposure used on this material.
     * This property is here and not in the camera to allow controlling exposure without full screen post process.
     * This corresponds to a photographic exposure.
     */
    get cameraExposure(): number;
    /**
     * The camera exposure used on this material.
     * This property is here and not in the camera to allow controlling exposure without full screen post process.
     * This corresponds to a photographic exposure.
     */
    set cameraExposure(value: number);
    /**
     * Gets The camera contrast used on this material.
     */
    get cameraContrast(): number;
    /**
     * Sets The camera contrast used on this material.
     */
    set cameraContrast(value: number);
    /**
     * Gets the Color Grading 2D Lookup Texture.
     */
    get cameraColorGradingTexture(): Nullable<BaseTexture>;
    /**
     * Sets the Color Grading 2D Lookup Texture.
     */
    set cameraColorGradingTexture(value: Nullable<BaseTexture>);
    /**
     * The color grading curves provide additional color adjustmnent that is applied after any color grading transform (3D LUT).
     * They allow basic adjustment of saturation and small exposure adjustments, along with color filter tinting to provide white balance adjustment or more stylistic effects.
     * These are similar to controls found in many professional imaging or colorist software. The global controls are applied to the entire image. For advanced tuning, extra controls are provided to adjust the shadow, midtone and highlight areas of the image;
     * corresponding to low luminance, medium luminance, and high luminance areas respectively.
     */
    get cameraColorCurves(): Nullable<ColorCurves>;
    /**
     * The color grading curves provide additional color adjustment that is applied after any color grading transform (3D LUT).
     * They allow basic adjustment of saturation and small exposure adjustments, along with color filter tinting to provide white balance adjustment or more stylistic effects.
     * These are similar to controls found in many professional imaging or colorist software. The global controls are applied to the entire image. For advanced tuning, extra controls are provided to adjust the shadow, midtone and highlight areas of the image;
     * corresponding to low luminance, medium luminance, and high luminance areas respectively.
     */
    set cameraColorCurves(value: Nullable<ColorCurves>);
    /**
     * Can this material render to several textures at once
     */
    get canRenderToMRT(): boolean;
    /**
     * Defines the detail map parameters for the material.
     */
    readonly detailMap: DetailMapConfiguration;
    protected _renderTargets: SmartArray<RenderTargetTexture>;
    protected _worldViewProjectionMatrix: Matrix;
    protected _globalAmbientColor: Color3;
    protected _useLogarithmicDepth: boolean;
    protected _cacheHasRenderTargetTextures: boolean;
    /**
     * Instantiates a new standard material.
     * This is the default material used in Babylon. It is the best trade off between quality
     * and performances.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/materials_introduction
     * @param name Define the name of the material in the scene
     * @param scene Define the scene the material belong to
     */
    constructor(name: string, scene?: Scene);
    /**
     * Gets a boolean indicating that current material needs to register RTT
     */
    get hasRenderTargetTextures(): boolean;
    /**
     * Gets the current class name of the material e.g. "StandardMaterial"
     * Mainly use in serialization.
     * @returns the class name
     */
    getClassName(): string;
    /**
     * In case the depth buffer does not allow enough depth precision for your scene (might be the case in large scenes)
     * You can try switching to logarithmic depth.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/advanced/logarithmicDepthBuffer
     */
    get useLogarithmicDepth(): boolean;
    set useLogarithmicDepth(value: boolean);
    /**
     * Specifies if the material will require alpha blending
     * @returns a boolean specifying if alpha blending is needed
     */
    needAlphaBlending(): boolean;
    /**
     * Specifies if this material should be rendered in alpha test mode
     * @returns a boolean specifying if an alpha test is needed.
     */
    needAlphaTesting(): boolean;
    /**
     * Specifies whether or not the alpha value of the diffuse texture should be used for alpha blending.
     */
    protected _shouldUseAlphaFromDiffuseTexture(): boolean;
    /**
     * Specifies whether or not there is a usable alpha channel for transparency.
     */
    protected _hasAlphaChannel(): boolean;
    /**
     * Get the texture used for alpha test purpose.
     * @returns the diffuse texture in case of the standard material.
     */
    getAlphaTestTexture(): Nullable<BaseTexture>;
    /**
     * Get if the submesh is ready to be used and all its information available.
     * Child classes can use it to update shaders
     * @param mesh defines the mesh to check
     * @param subMesh defines which submesh to check
     * @param useInstances specifies that instances should be used
     * @returns a boolean indicating that the submesh is ready or not
     */
    isReadyForSubMesh(mesh: AbstractMesh, subMesh: SubMesh, useInstances?: boolean): boolean;
    /**
     * Builds the material UBO layouts.
     * Used internally during the effect preparation.
     */
    buildUniformLayout(): void;
    /**
     * Binds the submesh to this material by preparing the effect and shader to draw
     * @param world defines the world transformation matrix
     * @param mesh defines the mesh containing the submesh
     * @param subMesh defines the submesh to bind the material to
     */
    bindForSubMesh(world: Matrix, mesh: Mesh, subMesh: SubMesh): void;
    /**
     * Get the list of animatables in the material.
     * @returns the list of animatables object used in the material
     */
    getAnimatables(): IAnimatable[];
    /**
     * Gets the active textures from the material
     * @returns an array of textures
     */
    getActiveTextures(): BaseTexture[];
    /**
     * Specifies if the material uses a texture
     * @param texture defines the texture to check against the material
     * @returns a boolean specifying if the material uses the texture
     */
    hasTexture(texture: BaseTexture): boolean;
    /**
     * Disposes the material
     * @param forceDisposeEffect specifies if effects should be forcefully disposed
     * @param forceDisposeTextures specifies if textures should be forcefully disposed
     */
    dispose(forceDisposeEffect?: boolean, forceDisposeTextures?: boolean): void;
    /**
     * Makes a duplicate of the material, and gives it a new name
     * @param name defines the new name for the duplicated material
     * @returns the cloned material
     */
    clone(name: string): StandardMaterial;
    /**
     * Creates a standard material from parsed material data
     * @param source defines the JSON representation of the material
     * @param scene defines the hosting scene
     * @param rootUrl defines the root URL to use to load textures and relative dependencies
     * @returns a new standard material
     */
    static Parse(source: any, scene: Scene, rootUrl: string): StandardMaterial;
    /**
     * Are diffuse textures enabled in the application.
     */
    static get DiffuseTextureEnabled(): boolean;
    static set DiffuseTextureEnabled(value: boolean);
    /**
     * Are detail textures enabled in the application.
     */
    static get DetailTextureEnabled(): boolean;
    static set DetailTextureEnabled(value: boolean);
    /**
     * Are ambient textures enabled in the application.
     */
    static get AmbientTextureEnabled(): boolean;
    static set AmbientTextureEnabled(value: boolean);
    /**
     * Are opacity textures enabled in the application.
     */
    static get OpacityTextureEnabled(): boolean;
    static set OpacityTextureEnabled(value: boolean);
    /**
     * Are reflection textures enabled in the application.
     */
    static get ReflectionTextureEnabled(): boolean;
    static set ReflectionTextureEnabled(value: boolean);
    /**
     * Are emissive textures enabled in the application.
     */
    static get EmissiveTextureEnabled(): boolean;
    static set EmissiveTextureEnabled(value: boolean);
    /**
     * Are specular textures enabled in the application.
     */
    static get SpecularTextureEnabled(): boolean;
    static set SpecularTextureEnabled(value: boolean);
    /**
     * Are bump textures enabled in the application.
     */
    static get BumpTextureEnabled(): boolean;
    static set BumpTextureEnabled(value: boolean);
    /**
     * Are lightmap textures enabled in the application.
     */
    static get LightmapTextureEnabled(): boolean;
    static set LightmapTextureEnabled(value: boolean);
    /**
     * Are refraction textures enabled in the application.
     */
    static get RefractionTextureEnabled(): boolean;
    static set RefractionTextureEnabled(value: boolean);
    /**
     * Are color grading textures enabled in the application.
     */
    static get ColorGradingTextureEnabled(): boolean;
    static set ColorGradingTextureEnabled(value: boolean);
    /**
     * Are fresnels enabled in the application.
     */
    static get FresnelEnabled(): boolean;
    static set FresnelEnabled(value: boolean);
}

}
declare module "babylonjs/Materials/Textures/baseTexture" {
import { Observable } from "babylonjs/Misc/observable";
import { Nullable } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
import { Matrix } from "babylonjs/Maths/math.vector";
import { InternalTexture } from "babylonjs/Materials/Textures/internalTexture";
import { IAnimatable } from "babylonjs/Animations/animatable.interface";
import "babylonjs/Misc/fileTools";
import { ThinEngine } from "babylonjs/Engines/thinEngine";
import { ThinTexture } from "babylonjs/Materials/Textures/thinTexture";
import { AbstractScene } from "babylonjs/abstractScene";
/**
 * Base class of all the textures in babylon.
 * It groups all the common properties the materials, post process, lights... might need
 * in order to make a correct use of the texture.
 */
export class BaseTexture extends ThinTexture implements IAnimatable {
    /**
     * Default anisotropic filtering level for the application.
     * It is set to 4 as a good tradeoff between perf and quality.
     */
    static DEFAULT_ANISOTROPIC_FILTERING_LEVEL: number;
    /**
     * Gets or sets the unique id of the texture
     */
    uniqueId: number;
    /**
     * Define the name of the texture.
     */
    name: string;
    /**
     * Gets or sets an object used to store user defined information.
     */
    metadata: any;
    /** @internal */
    _internalMetadata: any;
    /**
     * For internal use only. Please do not use.
     */
    reservedDataStore: any;
    private _hasAlpha;
    /**
     * Define if the texture is having a usable alpha value (can be use for transparency or glossiness for instance).
     */
    set hasAlpha(value: boolean);
    get hasAlpha(): boolean;
    private _getAlphaFromRGB;
    /**
     * Defines if the alpha value should be determined via the rgb values.
     * If true the luminance of the pixel might be used to find the corresponding alpha value.
     */
    set getAlphaFromRGB(value: boolean);
    get getAlphaFromRGB(): boolean;
    /**
     * Intensity or strength of the texture.
     * It is commonly used by materials to fine tune the intensity of the texture
     */
    level: number;
    protected _coordinatesIndex: number;
    /**
     * Gets or sets a boolean indicating that the texture should try to reduce shader code if there is no UV manipulation.
     * (ie. when texture.getTextureMatrix().isIdentityAs3x2() returns true)
     */
    optimizeUVAllocation: boolean;
    /**
     * Define the UV channel to use starting from 0 and defaulting to 0.
     * This is part of the texture as textures usually maps to one uv set.
     */
    set coordinatesIndex(value: number);
    get coordinatesIndex(): number;
    protected _coordinatesMode: number;
    /**
     * How a texture is mapped.
     *
     * | Value | Type                                | Description |
     * | ----- | ----------------------------------- | ----------- |
     * | 0     | EXPLICIT_MODE                       |             |
     * | 1     | SPHERICAL_MODE                      |             |
     * | 2     | PLANAR_MODE                         |             |
     * | 3     | CUBIC_MODE                          |             |
     * | 4     | PROJECTION_MODE                     |             |
     * | 5     | SKYBOX_MODE                         |             |
     * | 6     | INVCUBIC_MODE                       |             |
     * | 7     | EQUIRECTANGULAR_MODE                |             |
     * | 8     | FIXED_EQUIRECTANGULAR_MODE          |             |
     * | 9     | FIXED_EQUIRECTANGULAR_MIRRORED_MODE |             |
     */
    set coordinatesMode(value: number);
    get coordinatesMode(): number;
    /**
     * | Value | Type               | Description |
     * | ----- | ------------------ | ----------- |
     * | 0     | CLAMP_ADDRESSMODE  |             |
     * | 1     | WRAP_ADDRESSMODE   |             |
     * | 2     | MIRROR_ADDRESSMODE |             |
     */
    get wrapU(): number;
    set wrapU(value: number);
    /**
     * | Value | Type               | Description |
     * | ----- | ------------------ | ----------- |
     * | 0     | CLAMP_ADDRESSMODE  |             |
     * | 1     | WRAP_ADDRESSMODE   |             |
     * | 2     | MIRROR_ADDRESSMODE |             |
     */
    get wrapV(): number;
    set wrapV(value: number);
    /**
     * | Value | Type               | Description |
     * | ----- | ------------------ | ----------- |
     * | 0     | CLAMP_ADDRESSMODE  |             |
     * | 1     | WRAP_ADDRESSMODE   |             |
     * | 2     | MIRROR_ADDRESSMODE |             |
     */
    wrapR: number;
    /**
     * With compliant hardware and browser (supporting anisotropic filtering)
     * this defines the level of anisotropic filtering in the texture.
     * The higher the better but the slower. This defaults to 4 as it seems to be the best tradeoff.
     */
    anisotropicFilteringLevel: number;
    /** @internal */
    _isCube: boolean;
    /**
     * Define if the texture is a cube texture or if false a 2d texture.
     */
    get isCube(): boolean;
    protected set isCube(value: boolean);
    /**
     * Define if the texture is a 3d texture (webgl 2) or if false a 2d texture.
     */
    get is3D(): boolean;
    protected set is3D(value: boolean);
    /**
     * Define if the texture is a 2d array texture (webgl 2) or if false a 2d texture.
     */
    get is2DArray(): boolean;
    protected set is2DArray(value: boolean);
    private _gammaSpace;
    /**
     * Define if the texture contains data in gamma space (most of the png/jpg aside bump).
     * HDR texture are usually stored in linear space.
     * This only impacts the PBR and Background materials
     */
    get gammaSpace(): boolean;
    set gammaSpace(gamma: boolean);
    /**
     * Gets or sets whether or not the texture contains RGBD data.
     */
    get isRGBD(): boolean;
    set isRGBD(value: boolean);
    /**
     * Is Z inverted in the texture (useful in a cube texture).
     */
    invertZ: boolean;
    /**
     * Are mip maps generated for this texture or not.
     */
    get noMipmap(): boolean;
    /**
     * @internal
     */
    lodLevelInAlpha: boolean;
    /**
     * With prefiltered texture, defined the offset used during the prefiltering steps.
     */
    get lodGenerationOffset(): number;
    set lodGenerationOffset(value: number);
    /**
     * With prefiltered texture, defined the scale used during the prefiltering steps.
     */
    get lodGenerationScale(): number;
    set lodGenerationScale(value: number);
    /**
     * With prefiltered texture, defined if the specular generation is based on a linear ramp.
     * By default we are using a log2 of the linear roughness helping to keep a better resolution for
     * average roughness values.
     */
    get linearSpecularLOD(): boolean;
    set linearSpecularLOD(value: boolean);
    /**
     * In case a better definition than spherical harmonics is required for the diffuse part of the environment.
     * You can set the irradiance texture to rely on a texture instead of the spherical approach.
     * This texture need to have the same characteristics than its parent (Cube vs 2d, coordinates mode, Gamma/Linear, RGBD).
     */
    get irradianceTexture(): Nullable<BaseTexture>;
    set irradianceTexture(value: Nullable<BaseTexture>);
    /**
     * Define if the texture is a render target.
     */
    isRenderTarget: boolean;
    /**
     * Define the unique id of the texture in the scene.
     */
    get uid(): string;
    /** @internal */
    _prefiltered: boolean;
    /** @internal */
    _forceSerialize: boolean;
    /**
     * Return a string representation of the texture.
     * @returns the texture as a string
     */
    toString(): string;
    /**
     * Get the class name of the texture.
     * @returns "BaseTexture"
     */
    getClassName(): string;
    /**
     * Define the list of animation attached to the texture.
     */
    animations: import("babylonjs/Animations/animation").Animation[];
    /**
     * An event triggered when the texture is disposed.
     */
    onDisposeObservable: Observable<BaseTexture>;
    private _onDisposeObserver;
    /**
     * Callback triggered when the texture has been disposed.
     * Kept for back compatibility, you can use the onDisposeObservable instead.
     */
    set onDispose(callback: () => void);
    protected _scene: Nullable<Scene>;
    /** @internal */
    private _uid;
    /**
     * Define if the texture is preventing a material to render or not.
     * If not and the texture is not ready, the engine will use a default black texture instead.
     */
    get isBlocking(): boolean;
    /** @internal */
    _parentContainer: Nullable<AbstractScene>;
    protected _loadingError: boolean;
    protected _errorObject?: {
        message?: string;
        exception?: any;
    };
    /**
     * Was there any loading error?
     */
    get loadingError(): boolean;
    /**
     * If a loading error occurred this object will be populated with information about the error.
     */
    get errorObject(): {
        message?: string;
        exception?: any;
    } | undefined;
    /**
     * Instantiates a new BaseTexture.
     * Base class of all the textures in babylon.
     * It groups all the common properties the materials, post process, lights... might need
     * in order to make a correct use of the texture.
     * @param sceneOrEngine Define the scene or engine the texture belongs to
     * @param internalTexture Define the internal texture associated with the texture
     */
    constructor(sceneOrEngine?: Nullable<Scene | ThinEngine>, internalTexture?: Nullable<InternalTexture>);
    /**
     * Get the scene the texture belongs to.
     * @returns the scene or null if undefined
     */
    getScene(): Nullable<Scene>;
    /** @internal */
    protected _getEngine(): Nullable<ThinEngine>;
    /**
     * Checks if the texture has the same transform matrix than another texture
     * @param texture texture to check against
     * @returns true if the transforms are the same, else false
     */
    checkTransformsAreIdentical(texture: Nullable<BaseTexture>): boolean;
    /**
     * Get the texture transform matrix used to offset tile the texture for instance.
     * @returns the transformation matrix
     */
    getTextureMatrix(): Matrix;
    /**
     * Get the texture reflection matrix used to rotate/transform the reflection.
     * @returns the reflection matrix
     */
    getReflectionTextureMatrix(): Matrix;
    /**
     * Get if the texture is ready to be consumed (either it is ready or it is not blocking)
     * @returns true if ready, not blocking or if there was an error loading the texture
     */
    isReadyOrNotBlocking(): boolean;
    /**
     * Scales the texture if is `canRescale()`
     * @param ratio the resize factor we want to use to rescale
     */
    scale(ratio: number): void;
    /**
     * Get if the texture can rescale.
     */
    get canRescale(): boolean;
    /**
     * @internal
     */
    _getFromCache(url: Nullable<string>, noMipmap: boolean, sampling?: number, invertY?: boolean, useSRGBBuffer?: boolean, isCube?: boolean): Nullable<InternalTexture>;
    /** @internal */
    _rebuild(): void;
    /**
     * Clones the texture.
     * @returns the cloned texture
     */
    clone(): Nullable<BaseTexture>;
    /**
     * Get the texture underlying type (INT, FLOAT...)
     */
    get textureType(): number;
    /**
     * Get the texture underlying format (RGB, RGBA...)
     */
    get textureFormat(): number;
    /**
     * Indicates that textures need to be re-calculated for all materials
     */
    protected _markAllSubMeshesAsTexturesDirty(): void;
    /**
     * Reads the pixels stored in the webgl texture and returns them as an ArrayBuffer.
     * This will returns an RGBA array buffer containing either in values (0-255) or
     * float values (0-1) depending of the underlying buffer type.
     * @param faceIndex defines the face of the texture to read (in case of cube texture)
     * @param level defines the LOD level of the texture to read (in case of Mip Maps)
     * @param buffer defines a user defined buffer to fill with data (can be null)
     * @param flushRenderer true to flush the renderer from the pending commands before reading the pixels
     * @param noDataConversion false to convert the data to Uint8Array (if texture type is UNSIGNED_BYTE) or to Float32Array (if texture type is anything but UNSIGNED_BYTE). If true, the type of the generated buffer (if buffer==null) will depend on the type of the texture
     * @param x defines the region x coordinates to start reading from (default to 0)
     * @param y defines the region y coordinates to start reading from (default to 0)
     * @param width defines the region width to read from (default to the texture size at level)
     * @param height defines the region width to read from (default to the texture size at level)
     * @returns The Array buffer promise containing the pixels data.
     */
    readPixels(faceIndex?: number, level?: number, buffer?: Nullable<ArrayBufferView>, flushRenderer?: boolean, noDataConversion?: boolean, x?: number, y?: number, width?: number, height?: number): Nullable<Promise<ArrayBufferView>>;
    /**
     * @internal
     */
    _readPixelsSync(faceIndex?: number, level?: number, buffer?: Nullable<ArrayBufferView>, flushRenderer?: boolean, noDataConversion?: boolean): Nullable<ArrayBufferView>;
    /** @internal */
    get _lodTextureHigh(): Nullable<BaseTexture>;
    /** @internal */
    get _lodTextureMid(): Nullable<BaseTexture>;
    /** @internal */
    get _lodTextureLow(): Nullable<BaseTexture>;
    /**
     * Dispose the texture and release its associated resources.
     */
    dispose(): void;
    /**
     * Serialize the texture into a JSON representation that can be parsed later on.
     * @param allowEmptyName True to force serialization even if name is empty. Default: false
     * @returns the JSON representation of the texture
     */
    serialize(allowEmptyName?: boolean): any;
    /**
     * Helper function to be called back once a list of texture contains only ready textures.
     * @param textures Define the list of textures to wait for
     * @param callback Define the callback triggered once the entire list will be ready
     */
    static WhenAllReady(textures: BaseTexture[], callback: () => void): void;
    private static _IsScene;
}

}
declare module "babylonjs/Materials/Textures/baseTexture.polynomial" {
import { Nullable } from "babylonjs/types";
import { SphericalPolynomial } from "babylonjs/Maths/sphericalPolynomial";
module "babylonjs/Materials/Textures/baseTexture" {
    interface BaseTexture {
        /**
         * Get the polynomial representation of the texture data.
         * This is mainly use as a fast way to recover IBL Diffuse irradiance data.
         * @see https://learnopengl.com/PBR/IBL/Diffuse-irradiance
         */
        sphericalPolynomial: Nullable<SphericalPolynomial>;
        /**
         * Force recomputation of spherical polynomials.
         * Can be useful if you generate a cubemap multiple times (from a probe for eg) and you need the proper polynomials each time
         */
        forceSphericalPolynomialsRecompute(): void;
    }
}

}
declare module "babylonjs/Materials/Textures/colorGradingTexture" {
import { Nullable } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
import { Matrix } from "babylonjs/Maths/math.vector";
import { BaseTexture } from "babylonjs/Materials/Textures/baseTexture";
import { ThinEngine } from "babylonjs/Engines/thinEngine";
import "babylonjs/Engines/Extensions/engine.rawTexture";
/**
 * This represents a color grading texture. This acts as a lookup table LUT, useful during post process
 * It can help converting any input color in a desired output one. This can then be used to create effects
 * from sepia, black and white to sixties or futuristic rendering...
 *
 * The only supported format is currently 3dl.
 * More information on LUT: https://en.wikipedia.org/wiki/3D_lookup_table
 */
export class ColorGradingTexture extends BaseTexture {
    /**
     * The texture URL.
     */
    url: string;
    /**
     * Empty line regex stored for GC.
     */
    private static _NoneEmptyLineRegex;
    private _textureMatrix;
    private _onLoad;
    /**
     * Instantiates a ColorGradingTexture from the following parameters.
     *
     * @param url The location of the color grading data (currently only supporting 3dl)
     * @param sceneOrEngine The scene or engine the texture will be used in
     * @param onLoad defines a callback triggered when the texture has been loaded
     */
    constructor(url: string, sceneOrEngine: Scene | ThinEngine, onLoad?: Nullable<() => void>);
    /**
     * Fires the onload event from the constructor if requested.
     */
    private _triggerOnLoad;
    /**
     * Returns the texture matrix used in most of the material.
     * This is not used in color grading but keep for troubleshooting purpose (easily swap diffuse by colorgrading to look in).
     */
    getTextureMatrix(): Matrix;
    /**
     * Occurs when the file being loaded is a .3dl LUT file.
     */
    private _load3dlTexture;
    /**
     * Starts the loading process of the texture.
     */
    private _loadTexture;
    /**
     * Clones the color grading texture.
     */
    clone(): ColorGradingTexture;
    /**
     * Called during delayed load for textures.
     */
    delayLoad(): void;
    /**
     * Parses a color grading texture serialized by Babylon.
     * @param parsedTexture The texture information being parsedTexture
     * @param scene The scene to load the texture in
     * @returns A color grading texture
     */
    static Parse(parsedTexture: any, scene: Scene): Nullable<ColorGradingTexture>;
    /**
     * Serializes the LUT texture to json format.
     */
    serialize(): any;
}

}
declare module "babylonjs/Materials/Textures/cubeTexture" {
import { Nullable } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
import { Matrix, Vector3 } from "babylonjs/Maths/math.vector";
import { BaseTexture } from "babylonjs/Materials/Textures/baseTexture";
import { ThinEngine } from "babylonjs/Engines/thinEngine";
import "babylonjs/Engines/Extensions/engine.cubeTexture";
import { Observable } from "babylonjs/Misc/observable";
/**
 * Class for creating a cube texture
 */
export class CubeTexture extends BaseTexture {
    private _delayedOnLoad;
    private _delayedOnError;
    private _lodScale;
    private _lodOffset;
    /**
     * Observable triggered once the texture has been loaded.
     */
    onLoadObservable: Observable<CubeTexture>;
    /**
     * The url of the texture
     */
    url: string;
    /**
     * Gets or sets the center of the bounding box associated with the cube texture.
     * It must define where the camera used to render the texture was set
     * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/reflectionTexture#using-local-cubemap-mode
     */
    boundingBoxPosition: Vector3;
    private _boundingBoxSize;
    /**
     * Gets or sets the size of the bounding box associated with the cube texture
     * When defined, the cubemap will switch to local mode
     * @see https://community.arm.com/graphics/b/blog/posts/reflections-based-on-local-cubemaps-in-unity
     * @example https://www.babylonjs-playground.com/#RNASML
     */
    set boundingBoxSize(value: Vector3);
    /**
     * Returns the bounding box size
     * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/reflectionTexture#using-local-cubemap-mode
     */
    get boundingBoxSize(): Vector3;
    protected _rotationY: number;
    /**
     * Sets texture matrix rotation angle around Y axis in radians.
     */
    set rotationY(value: number);
    /**
     * Gets texture matrix rotation angle around Y axis radians.
     */
    get rotationY(): number;
    /**
     * Are mip maps generated for this texture or not.
     */
    get noMipmap(): boolean;
    private _noMipmap;
    /** @internal */
    _files: Nullable<string[]>;
    protected _forcedExtension: Nullable<string>;
    /**
     * Gets the forced extension (if any)
     */
    get forcedExtension(): Nullable<string>;
    private _extensions;
    private _textureMatrix;
    private _format;
    private _createPolynomials;
    private _loaderOptions;
    private _useSRGBBuffer?;
    /**
     * Creates a cube texture from an array of image urls
     * @param files defines an array of image urls
     * @param scene defines the hosting scene
     * @param noMipmap specifies if mip maps are not used
     * @returns a cube texture
     */
    static CreateFromImages(files: string[], scene: Scene, noMipmap?: boolean): CubeTexture;
    /**
     * Creates and return a texture created from prefilterd data by tools like IBL Baker or Lys.
     * @param url defines the url of the prefiltered texture
     * @param scene defines the scene the texture is attached to
     * @param forcedExtension defines the extension of the file if different from the url
     * @param createPolynomials defines whether or not to create polynomial harmonics from the texture data if necessary
     * @returns the prefiltered texture
     */
    static CreateFromPrefilteredData(url: string, scene: Scene, forcedExtension?: any, createPolynomials?: boolean): CubeTexture;
    /**
     * Creates a cube texture to use with reflection for instance. It can be based upon dds or six images as well
     * as prefiltered data.
     * @param rootUrl defines the url of the texture or the root name of the six images
     * @param sceneOrEngine defines the scene or engine the texture is attached to
     * @param extensions defines the suffixes add to the picture name in case six images are in use like _px.jpg...
     * @param noMipmap defines if mipmaps should be created or not
     * @param files defines the six files to load for the different faces in that order: px, py, pz, nx, ny, nz
     * @param onLoad defines a callback triggered at the end of the file load if no errors occurred
     * @param onError defines a callback triggered in case of error during load
     * @param format defines the internal format to use for the texture once loaded
     * @param prefiltered defines whether or not the texture is created from prefiltered data
     * @param forcedExtension defines the extensions to use (force a special type of file to load) in case it is different from the file name
     * @param createPolynomials defines whether or not to create polynomial harmonics from the texture data if necessary
     * @param lodScale defines the scale applied to environment texture. This manages the range of LOD level used for IBL according to the roughness
     * @param lodOffset defines the offset applied to environment texture. This manages first LOD level used for IBL according to the roughness
     * @param loaderOptions options to be passed to the loader
     * @param useSRGBBuffer Defines if the texture must be loaded in a sRGB GPU buffer (if supported by the GPU) (default: false)
     * @returns the cube texture
     */
    constructor(rootUrl: string, sceneOrEngine: Scene | ThinEngine, extensions?: Nullable<string[]>, noMipmap?: boolean, files?: Nullable<string[]>, onLoad?: Nullable<() => void>, onError?: Nullable<(message?: string, exception?: any) => void>, format?: number, prefiltered?: boolean, forcedExtension?: any, createPolynomials?: boolean, lodScale?: number, lodOffset?: number, loaderOptions?: any, useSRGBBuffer?: boolean);
    /**
     * Get the current class name of the texture useful for serialization or dynamic coding.
     * @returns "CubeTexture"
     */
    getClassName(): string;
    /**
     * Update the url (and optional buffer) of this texture if url was null during construction.
     * @param url the url of the texture
     * @param forcedExtension defines the extension to use
     * @param onLoad callback called when the texture is loaded  (defaults to null)
     * @param prefiltered Defines whether the updated texture is prefiltered or not
     * @param onError callback called if there was an error during the loading process (defaults to null)
     * @param extensions defines the suffixes add to the picture name in case six images are in use like _px.jpg...
     * @param delayLoad defines if the texture should be loaded now (false by default)
     * @param files defines the six files to load for the different faces in that order: px, py, pz, nx, ny, nz
     */
    updateURL(url: string, forcedExtension?: string, onLoad?: Nullable<() => void>, prefiltered?: boolean, onError?: Nullable<(message?: string, exception?: any) => void>, extensions?: Nullable<string[]>, delayLoad?: boolean, files?: Nullable<string[]>): void;
    /**
     * Delays loading of the cube texture
     * @param forcedExtension defines the extension to use
     */
    delayLoad(forcedExtension?: string): void;
    /**
     * Returns the reflection texture matrix
     * @returns the reflection texture matrix
     */
    getReflectionTextureMatrix(): Matrix;
    /**
     * Sets the reflection texture matrix
     * @param value Reflection texture matrix
     */
    setReflectionTextureMatrix(value: Matrix): void;
    private _loadTexture;
    /**
     * Parses text to create a cube texture
     * @param parsedTexture define the serialized text to read from
     * @param scene defines the hosting scene
     * @param rootUrl defines the root url of the cube texture
     * @returns a cube texture
     */
    static Parse(parsedTexture: any, scene: Scene, rootUrl: string): CubeTexture;
    /**
     * Makes a clone, or deep copy, of the cube texture
     * @returns a new cube texture
     */
    clone(): CubeTexture;
}

}
declare module "babylonjs/Materials/Textures/dynamicTexture" {
import { Nullable } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
import { Texture } from "babylonjs/Materials/Textures/texture";
import "babylonjs/Engines/Extensions/engine.dynamicTexture";
import { ICanvasRenderingContext } from "babylonjs/Engines/ICanvas";
/**
 * A class extending Texture allowing drawing on a texture
 * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/dynamicTexture
 */
export class DynamicTexture extends Texture {
    private _generateMipMaps;
    private _canvas;
    private _context;
    /**
     * Creates a DynamicTexture
     * @param name defines the name of the texture
     * @param options provides 3 alternatives for width and height of texture, a canvas, object with width and height properties, number for both width and height
     * @param scene defines the scene where you want the texture
     * @param generateMipMaps defines the use of MinMaps or not (default is false)
     * @param samplingMode defines the sampling mode to use (default is Texture.TRILINEAR_SAMPLINGMODE)
     * @param format defines the texture format to use (default is Engine.TEXTUREFORMAT_RGBA)
     * @param invertY defines if the texture needs to be inverted on the y axis during loading
     */
    constructor(name: string, options: any, scene?: Nullable<Scene>, generateMipMaps?: boolean, samplingMode?: number, format?: number, invertY?: boolean);
    /**
     * Get the current class name of the texture useful for serialization or dynamic coding.
     * @returns "DynamicTexture"
     */
    getClassName(): string;
    /**
     * Gets the current state of canRescale
     */
    get canRescale(): boolean;
    private _recreate;
    /**
     * Scales the texture
     * @param ratio the scale factor to apply to both width and height
     */
    scale(ratio: number): void;
    /**
     * Resizes the texture
     * @param width the new width
     * @param height the new height
     */
    scaleTo(width: number, height: number): void;
    /**
     * Gets the context of the canvas used by the texture
     * @returns the canvas context of the dynamic texture
     */
    getContext(): ICanvasRenderingContext;
    /**
     * Clears the texture
     */
    clear(): void;
    /**
     * Updates the texture
     * @param invertY defines the direction for the Y axis (default is true - y increases downwards)
     * @param premulAlpha defines if alpha is stored as premultiplied (default is false)
     * @param allowGPUOptimization true to allow some specific GPU optimizations (subject to engine feature "allowGPUOptimizationsForGUI" being true)
     */
    update(invertY?: boolean, premulAlpha?: boolean, allowGPUOptimization?: boolean): void;
    /**
     * Draws text onto the texture
     * @param text defines the text to be drawn
     * @param x defines the placement of the text from the left
     * @param y defines the placement of the text from the top when invertY is true and from the bottom when false
     * @param font defines the font to be used with font-style, font-size, font-name
     * @param color defines the color used for the text
     * @param clearColor defines the color for the canvas, use null to not overwrite canvas
     * @param invertY defines the direction for the Y axis (default is true - y increases downwards)
     * @param update defines whether texture is immediately update (default is true)
     */
    drawText(text: string, x: number | null | undefined, y: number | null | undefined, font: string, color: string | null, clearColor: string | null, invertY?: boolean, update?: boolean): void;
    /**
     * Clones the texture
     * @returns the clone of the texture.
     */
    clone(): DynamicTexture;
    /**
     * Serializes the dynamic texture.  The scene should be ready before the dynamic texture is serialized
     * @returns a serialized dynamic texture object
     */
    serialize(): any;
    private static _IsCanvasElement;
    /** @internal */
    _rebuild(): void;
}

}
declare module "babylonjs/Materials/Textures/equiRectangularCubeTexture" {
import { BaseTexture } from "babylonjs/Materials/Textures/baseTexture";
import { Scene } from "babylonjs/scene";
import { Nullable } from "babylonjs/types";
import "babylonjs/Engines/Extensions/engine.rawTexture";
/**
 * This represents a texture coming from an equirectangular image supported by the web browser canvas.
 */
export class EquiRectangularCubeTexture extends BaseTexture {
    /** The six faces of the cube. */
    private static _FacesMapping;
    private _noMipmap;
    private _onLoad;
    private _onError;
    /** The size of the cubemap. */
    private _size;
    /** The buffer of the image. */
    private _buffer;
    /** The width of the input image. */
    private _width;
    /** The height of the input image. */
    private _height;
    /** The URL to the image. */
    url: string;
    /**
     * Instantiates an EquiRectangularCubeTexture from the following parameters.
     * @param url The location of the image
     * @param scene The scene the texture will be used in
     * @param size The cubemap desired size (the more it increases the longer the generation will be)
     * @param noMipmap Forces to not generate the mipmap if true
     * @param gammaSpace Specifies if the texture will be used in gamma or linear space
     * (the PBR material requires those textures in linear space, but the standard material would require them in Gamma space)
     * @param onLoad — defines a callback called when texture is loaded
     * @param onError — defines a callback called if there is an error
     */
    constructor(url: string, scene: Scene, size: number, noMipmap?: boolean, gammaSpace?: boolean, onLoad?: Nullable<() => void>, onError?: Nullable<(message?: string, exception?: any) => void>);
    /**
     * Load the image data, by putting the image on a canvas and extracting its buffer.
     * @param loadTextureCallback
     * @param onError
     */
    private _loadImage;
    /**
     * Convert the image buffer into a cubemap and create a CubeTexture.
     */
    private _loadTexture;
    /**
     * Convert the ArrayBuffer into a Float32Array and drop the transparency channel.
     * @param buffer The ArrayBuffer that should be converted.
     * @returns The buffer as Float32Array.
     */
    private _getFloat32ArrayFromArrayBuffer;
    /**
     * Get the current class name of the texture useful for serialization or dynamic coding.
     * @returns "EquiRectangularCubeTexture"
     */
    getClassName(): string;
    /**
     * Create a clone of the current EquiRectangularCubeTexture and return it.
     * @returns A clone of the current EquiRectangularCubeTexture.
     */
    clone(): EquiRectangularCubeTexture;
}

}
declare module "babylonjs/Materials/Textures/externalTexture" {
import { InternalTexture } from "babylonjs/Materials/Textures/internalTexture";
/**
 * Class used to store an external texture (like GPUExternalTexture in WebGPU)
 */
export class ExternalTexture {
    /**
     * Checks if a texture is an external or internal texture
     * @param texture the external or internal texture
     * @returns true if the texture is an external texture, else false
     */
    static IsExternalTexture(texture: ExternalTexture | InternalTexture): texture is ExternalTexture;
    private _video;
    /**
     * Get the class name of the texture.
     * @returns "ExternalTexture"
     */
    getClassName(): string;
    /**
     * Gets the underlying texture object
     */
    get underlyingResource(): any;
    /**
     * Gets a boolean indicating if the texture uses mipmaps
     */
    useMipMaps: boolean;
    /**
     * The type of the underlying texture is implementation dependent, so return "UNDEFINED" for the type
     */
    readonly type: number;
    /**
     * Gets the unique id of this texture
     */
    readonly uniqueId: number;
    /**
     * Constructs the texture
     * @param video The video the texture should be wrapped around
     */
    constructor(video: HTMLVideoElement);
    /**
     * Get if the texture is ready to be used (downloaded, converted, mip mapped...).
     * @returns true if fully ready
     */
    isReady(): boolean;
    /**
     * Dispose the texture and release its associated resources.
     */
    dispose(): void;
}

}
declare module "babylonjs/Materials/Textures/Filtering/hdrFiltering" {
import { BaseTexture } from "babylonjs/Materials/Textures/baseTexture";
import { ThinEngine } from "babylonjs/Engines/thinEngine";
import { Nullable } from "babylonjs/types";
import "babylonjs/Shaders/hdrFiltering.vertex";
import "babylonjs/Shaders/hdrFiltering.fragment";
/**
 * Options for texture filtering
 */
interface IHDRFilteringOptions {
    /**
     * Scales pixel intensity for the input HDR map.
     */
    hdrScale?: number;
    /**
     * Quality of the filter. Should be `Constants.TEXTURE_FILTERING_QUALITY_OFFLINE` for prefiltering
     */
    quality?: number;
}
/**
 * Filters HDR maps to get correct renderings of PBR reflections
 */
export class HDRFiltering {
    private _engine;
    private _effectRenderer;
    private _effectWrapper;
    private _lodGenerationOffset;
    private _lodGenerationScale;
    /**
     * Quality switch for prefiltering. Should be set to `Constants.TEXTURE_FILTERING_QUALITY_OFFLINE` unless
     * you care about baking speed.
     */
    quality: number;
    /**
     * Scales pixel intensity for the input HDR map.
     */
    hdrScale: number;
    /**
     * Instantiates HDR filter for reflection maps
     *
     * @param engine Thin engine
     * @param options Options
     */
    constructor(engine: ThinEngine, options?: IHDRFilteringOptions);
    private _createRenderTarget;
    private _prefilterInternal;
    private _createEffect;
    /**
     * Get a value indicating if the filter is ready to be used
     * @param texture Texture to filter
     * @returns true if the filter is ready
     */
    isReady(texture: BaseTexture): boolean;
    /**
     * Prefilters a cube texture to have mipmap levels representing roughness values.
     * Prefiltering will be invoked at the end of next rendering pass.
     * This has to be done once the map is loaded, and has not been prefiltered by a third party software.
     * See http://blog.selfshadow.com/publications/s2013-shading-course/karis/s2013_pbs_epic_notes_v2.pdf for more information
     * @param texture Texture to filter
     * @param onFinished Callback when filtering is done
     * @returns Promise called when prefiltering is done
     */
    prefilter(texture: BaseTexture, onFinished?: Nullable<() => void>): Promise<void>;
}
export {};

}
declare module "babylonjs/Materials/Textures/hardwareTextureWrapper" {
/** @internal */
export interface HardwareTextureWrapper {
    underlyingResource: any;
    set(hardwareTexture: any): void;
    setUsage(textureSource: number, generateMipMaps: boolean, isCube: boolean, width: number, height: number): void;
    reset(): void;
    release(): void;
}

}
declare module "babylonjs/Materials/Textures/hdrCubeTexture" {
import { Nullable } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
import { Matrix, Vector3 } from "babylonjs/Maths/math.vector";
import { BaseTexture } from "babylonjs/Materials/Textures/baseTexture";
import { Observable } from "babylonjs/Misc/observable";
import { ThinEngine } from "babylonjs/Engines/thinEngine";
import "babylonjs/Engines/Extensions/engine.rawTexture";
import "babylonjs/Materials/Textures/baseTexture.polynomial";
/**
 * This represents a texture coming from an HDR input.
 *
 * The only supported format is currently panorama picture stored in RGBE format.
 * Example of such files can be found on Poly Haven: https://polyhaven.com/hdris
 */
export class HDRCubeTexture extends BaseTexture {
    private static _FacesMapping;
    private _generateHarmonics;
    private _noMipmap;
    private _prefilterOnLoad;
    private _textureMatrix;
    private _size;
    private _onLoad;
    private _onError;
    /**
     * The texture URL.
     */
    url: string;
    protected _isBlocking: boolean;
    /**
     * Sets whether or not the texture is blocking during loading.
     */
    set isBlocking(value: boolean);
    /**
     * Gets whether or not the texture is blocking during loading.
     */
    get isBlocking(): boolean;
    protected _rotationY: number;
    /**
     * Sets texture matrix rotation angle around Y axis in radians.
     */
    set rotationY(value: number);
    /**
     * Gets texture matrix rotation angle around Y axis radians.
     */
    get rotationY(): number;
    /**
     * Gets or sets the center of the bounding box associated with the cube texture
     * It must define where the camera used to render the texture was set
     */
    boundingBoxPosition: Vector3;
    private _boundingBoxSize;
    /**
     * Gets or sets the size of the bounding box associated with the cube texture
     * When defined, the cubemap will switch to local mode
     * @see https://community.arm.com/graphics/b/blog/posts/reflections-based-on-local-cubemaps-in-unity
     * @example https://www.babylonjs-playground.com/#RNASML
     */
    set boundingBoxSize(value: Vector3);
    get boundingBoxSize(): Vector3;
    /**
     * Observable triggered once the texture has been loaded.
     */
    onLoadObservable: Observable<HDRCubeTexture>;
    /**
     * Instantiates an HDRTexture from the following parameters.
     *
     * @param url The location of the HDR raw data (Panorama stored in RGBE format)
     * @param sceneOrEngine The scene or engine the texture will be used in
     * @param size The cubemap desired size (the more it increases the longer the generation will be)
     * @param noMipmap Forces to not generate the mipmap if true
     * @param generateHarmonics Specifies whether you want to extract the polynomial harmonics during the generation process
     * @param gammaSpace Specifies if the texture will be use in gamma or linear space (the PBR material requires those texture in linear space, but the standard material would require them in Gamma space)
     * @param prefilterOnLoad Prefilters HDR texture to allow use of this texture as a PBR reflection texture.
     * @param onLoad
     * @param onError
     */
    constructor(url: string, sceneOrEngine: Scene | ThinEngine, size: number, noMipmap?: boolean, generateHarmonics?: boolean, gammaSpace?: boolean, prefilterOnLoad?: boolean, onLoad?: Nullable<() => void>, onError?: Nullable<(message?: string, exception?: any) => void>);
    /**
     * Get the current class name of the texture useful for serialization or dynamic coding.
     * @returns "HDRCubeTexture"
     */
    getClassName(): string;
    /**
     * Occurs when the file is raw .hdr file.
     */
    private _loadTexture;
    clone(): HDRCubeTexture;
    delayLoad(): void;
    /**
     * Get the texture reflection matrix used to rotate/transform the reflection.
     * @returns the reflection matrix
     */
    getReflectionTextureMatrix(): Matrix;
    /**
     * Set the texture reflection matrix used to rotate/transform the reflection.
     * @param value Define the reflection matrix to set
     */
    setReflectionTextureMatrix(value: Matrix): void;
    /**
     * Dispose the texture and release its associated resources.
     */
    dispose(): void;
    /**
     * Parses a JSON representation of an HDR Texture in order to create the texture
     * @param parsedTexture Define the JSON representation
     * @param scene Define the scene the texture should be created in
     * @param rootUrl Define the root url in case we need to load relative dependencies
     * @returns the newly created texture after parsing
     */
    static Parse(parsedTexture: any, scene: Scene, rootUrl: string): Nullable<HDRCubeTexture>;
    serialize(): any;
}

}
declare module "babylonjs/Materials/Textures/htmlElementTexture" {
import { Nullable } from "babylonjs/types";
import { BaseTexture } from "babylonjs/Materials/Textures/baseTexture";
import { Matrix } from "babylonjs/Maths/math.vector";
import { Observable } from "babylonjs/Misc/observable";
import "babylonjs/Engines/Extensions/engine.dynamicTexture";
import "babylonjs/Engines/Extensions/engine.videoTexture";
import { ThinEngine } from "babylonjs/Engines/thinEngine";
import { Scene } from "babylonjs/scene";
/**
 * Defines the options related to the creation of an HtmlElementTexture
 */
export interface IHtmlElementTextureOptions {
    /**
     * Defines whether mip maps should be created or not.
     */
    generateMipMaps?: boolean;
    /**
     * Defines the sampling mode of the texture.
     */
    samplingMode?: number;
    /**
     * Defines the associated texture format.
     */
    format?: number;
    /**
     * Defines the engine instance to use the texture with. It is not mandatory if you define a scene.
     */
    engine: Nullable<ThinEngine>;
    /**
     * Defines the scene the texture belongs to. It is not mandatory if you define an engine.
     */
    scene: Nullable<Scene>;
}
/**
 * This represents the smallest workload to use an already existing element (Canvas or Video) as a texture.
 * To be as efficient as possible depending on your constraints nothing aside the first upload
 * is automatically managed.
 * It is a cheap VideoTexture or DynamicTexture if you prefer to keep full control of the elements
 * in your application.
 *
 * As the update is not automatic, you need to call them manually.
 */
export class HtmlElementTexture extends BaseTexture {
    /**
     * The texture URL.
     */
    element: HTMLVideoElement | HTMLCanvasElement;
    /**
     * Observable triggered once the texture has been loaded.
     */
    onLoadObservable: Observable<HtmlElementTexture>;
    private static readonly _DefaultOptions;
    private readonly _format;
    private _textureMatrix;
    private _isVideo;
    private _generateMipMaps;
    private _samplingMode;
    private _externalTexture;
    /**
     * Instantiates a HtmlElementTexture from the following parameters.
     *
     * @param name Defines the name of the texture
     * @param element Defines the video or canvas the texture is filled with
     * @param options Defines the other none mandatory texture creation options
     */
    constructor(name: string, element: HTMLVideoElement | HTMLCanvasElement, options: IHtmlElementTextureOptions);
    private _createInternalTexture;
    /**
     * Returns the texture matrix used in most of the material.
     */
    getTextureMatrix(): Matrix;
    /**
     * Updates the content of the texture.
     * @param invertY Defines whether the texture should be inverted on Y (false by default on video and true on canvas)
     */
    update(invertY?: Nullable<boolean>): void;
    /**
     * Dispose the texture and release its associated resources.
     */
    dispose(): void;
}
export {};

}
declare module "babylonjs/Materials/Textures/index" {
export * from "babylonjs/Materials/Textures/baseTexture";
export * from "babylonjs/Materials/Textures/baseTexture.polynomial";
export * from "babylonjs/Materials/Textures/colorGradingTexture";
export * from "babylonjs/Materials/Textures/cubeTexture";
export * from "babylonjs/Materials/Textures/dynamicTexture";
export * from "babylonjs/Materials/Textures/equiRectangularCubeTexture";
export * from "babylonjs/Materials/Textures/externalTexture";
export * from "babylonjs/Materials/Textures/Filtering/hdrFiltering";
export * from "babylonjs/Materials/Textures/hdrCubeTexture";
export * from "babylonjs/Materials/Textures/htmlElementTexture";
export * from "babylonjs/Materials/Textures/internalTexture";
export * from "babylonjs/Materials/Textures/internalTextureLoader";
export * from "babylonjs/Materials/Textures/Loaders/index";
export * from "babylonjs/Materials/Textures/mirrorTexture";
export * from "babylonjs/Materials/Textures/multiRenderTarget";
export * from "babylonjs/Materials/Textures/Packer/index";
export * from "babylonjs/Materials/Textures/Procedurals/index";
export * from "babylonjs/Materials/Textures/rawCubeTexture";
export * from "babylonjs/Materials/Textures/rawTexture";
export * from "babylonjs/Materials/Textures/rawTexture2DArray";
export * from "babylonjs/Materials/Textures/rawTexture3D";
export * from "babylonjs/Materials/Textures/refractionTexture";
export * from "babylonjs/Materials/Textures/renderTargetTexture";
export * from "babylonjs/Materials/Textures/textureSampler";
export * from "babylonjs/Materials/Textures/texture";
export * from "babylonjs/Materials/Textures/thinTexture";
export * from "babylonjs/Materials/Textures/thinRenderTargetTexture";
export * from "babylonjs/Materials/Textures/videoTexture";
export * from "babylonjs/Materials/Textures/ktx2decoderTypes";

}
declare module "babylonjs/Materials/Textures/internalTexture" {
import { Observable } from "babylonjs/Misc/observable";
import { Nullable, int } from "babylonjs/types";
import { ICanvas, ICanvasRenderingContext } from "babylonjs/Engines/ICanvas";
import { HardwareTextureWrapper } from "babylonjs/Materials/Textures/hardwareTextureWrapper";
import { TextureSampler } from "babylonjs/Materials/Textures/textureSampler";
import { ThinEngine } from "babylonjs/Engines/thinEngine";
import { BaseTexture } from "babylonjs/Materials/Textures/baseTexture";
import { SphericalPolynomial } from "babylonjs/Maths/sphericalPolynomial";
/**
 * Defines the source of the internal texture
 */
export enum InternalTextureSource {
    /**
     * The source of the texture data is unknown
     */
    Unknown = 0,
    /**
     * Texture data comes from an URL
     */
    Url = 1,
    /**
     * Texture data is only used for temporary storage
     */
    Temp = 2,
    /**
     * Texture data comes from raw data (ArrayBuffer)
     */
    Raw = 3,
    /**
     * Texture content is dynamic (video or dynamic texture)
     */
    Dynamic = 4,
    /**
     * Texture content is generated by rendering to it
     */
    RenderTarget = 5,
    /**
     * Texture content is part of a multi render target process
     */
    MultiRenderTarget = 6,
    /**
     * Texture data comes from a cube data file
     */
    Cube = 7,
    /**
     * Texture data comes from a raw cube data
     */
    CubeRaw = 8,
    /**
     * Texture data come from a prefiltered cube data file
     */
    CubePrefiltered = 9,
    /**
     * Texture content is raw 3D data
     */
    Raw3D = 10,
    /**
     * Texture content is raw 2D array data
     */
    Raw2DArray = 11,
    /**
     * Texture content is a depth/stencil texture
     */
    DepthStencil = 12,
    /**
     * Texture data comes from a raw cube data encoded with RGBD
     */
    CubeRawRGBD = 13,
    /**
     * Texture content is a depth texture
     */
    Depth = 14
}
/**
 * Class used to store data associated with WebGL texture data for the engine
 * This class should not be used directly
 */
export class InternalTexture extends TextureSampler {
    /**
     * Defines if the texture is ready
     */
    isReady: boolean;
    /**
     * Defines if the texture is a cube texture
     */
    isCube: boolean;
    /**
     * Defines if the texture contains 3D data
     */
    is3D: boolean;
    /**
     * Defines if the texture contains 2D array data
     */
    is2DArray: boolean;
    /**
     * Defines if the texture contains multiview data
     */
    isMultiview: boolean;
    /**
     * Gets the URL used to load this texture
     */
    url: string;
    /** @internal */
    _originalUrl: string;
    /**
     * Gets a boolean indicating if the texture needs mipmaps generation
     */
    generateMipMaps: boolean;
    /**
     * Gets a boolean indicating if the texture uses mipmaps
     * TODO implements useMipMaps as a separate setting from generateMipMaps
     */
    get useMipMaps(): boolean;
    set useMipMaps(value: boolean);
    /**
     * Gets the number of samples used by the texture (WebGL2+ only)
     */
    samples: number;
    /**
     * Gets the type of the texture (int, float...)
     */
    type: number;
    /**
     * Gets the format of the texture (RGB, RGBA...)
     */
    format: number;
    /**
     * Observable called when the texture is loaded
     */
    onLoadedObservable: Observable<InternalTexture>;
    /**
     * Observable called when the texture load is raising an error
     */
    onErrorObservable: Observable<Partial<{
        message: string;
        exception: any;
    }>>;
    /**
     * If this callback is defined it will be called instead of the default _rebuild function
     */
    onRebuildCallback: Nullable<(internalTexture: InternalTexture) => {
        proxy: Nullable<InternalTexture | Promise<InternalTexture>>;
        isReady: boolean;
        isAsync: boolean;
    }>;
    /**
     * Gets the width of the texture
     */
    width: number;
    /**
     * Gets the height of the texture
     */
    height: number;
    /**
     * Gets the depth of the texture
     */
    depth: number;
    /**
     * Gets the initial width of the texture (It could be rescaled if the current system does not support non power of two textures)
     */
    baseWidth: number;
    /**
     * Gets the initial height of the texture (It could be rescaled if the current system does not support non power of two textures)
     */
    baseHeight: number;
    /**
     * Gets the initial depth of the texture (It could be rescaled if the current system does not support non power of two textures)
     */
    baseDepth: number;
    /**
     * Gets a boolean indicating if the texture is inverted on Y axis
     */
    invertY: boolean;
    /** @internal */
    _invertVScale: boolean;
    /** @internal */
    _associatedChannel: number;
    /** @internal */
    _source: InternalTextureSource;
    /** @internal */
    _buffer: Nullable<string | ArrayBuffer | ArrayBufferView | HTMLImageElement | Blob | ImageBitmap>;
    /** @internal */
    _bufferView: Nullable<ArrayBufferView>;
    /** @internal */
    _bufferViewArray: Nullable<ArrayBufferView[]>;
    /** @internal */
    _bufferViewArrayArray: Nullable<ArrayBufferView[][]>;
    /** @internal */
    _size: number;
    /** @internal */
    _extension: string;
    /** @internal */
    _files: Nullable<string[]>;
    /** @internal */
    _workingCanvas: Nullable<ICanvas>;
    /** @internal */
    _workingContext: Nullable<ICanvasRenderingContext>;
    /** @internal */
    _cachedCoordinatesMode: Nullable<number>;
    /** @internal */
    _isDisabled: boolean;
    /** @internal */
    _compression: Nullable<string>;
    /** @internal */
    _sphericalPolynomial: Nullable<SphericalPolynomial>;
    /** @internal */
    _sphericalPolynomialPromise: Nullable<Promise<SphericalPolynomial>>;
    /** @internal */
    _sphericalPolynomialComputed: boolean;
    /** @internal */
    _lodGenerationScale: number;
    /** @internal */
    _lodGenerationOffset: number;
    /** @internal */
    _useSRGBBuffer: boolean;
    /** @internal */
    _lodTextureHigh: Nullable<BaseTexture>;
    /** @internal */
    _lodTextureMid: Nullable<BaseTexture>;
    /** @internal */
    _lodTextureLow: Nullable<BaseTexture>;
    /** @internal */
    _isRGBD: boolean;
    /** @internal */
    _linearSpecularLOD: boolean;
    /** @internal */
    _irradianceTexture: Nullable<BaseTexture>;
    /** @internal */
    _hardwareTexture: Nullable<HardwareTextureWrapper>;
    /** @internal */
    _maxLodLevel: Nullable<number>;
    /** @internal */
    _references: number;
    /** @internal */
    _gammaSpace: Nullable<boolean>;
    private _engine;
    private _uniqueId;
    /** @internal */
    static _Counter: number;
    /** Gets the unique id of the internal texture */
    get uniqueId(): number;
    /** @internal */
    _setUniqueId(id: number): void;
    /**
     * Gets the Engine the texture belongs to.
     * @returns The babylon engine
     */
    getEngine(): ThinEngine;
    /**
     * Gets the data source type of the texture
     */
    get source(): InternalTextureSource;
    /**
     * Creates a new InternalTexture
     * @param engine defines the engine to use
     * @param source defines the type of data that will be used
     * @param delayAllocation if the texture allocation should be delayed (default: false)
     */
    constructor(engine: ThinEngine, source: InternalTextureSource, delayAllocation?: boolean);
    /**
     * Increments the number of references (ie. the number of Texture that point to it)
     */
    incrementReferences(): void;
    /**
     * Change the size of the texture (not the size of the content)
     * @param width defines the new width
     * @param height defines the new height
     * @param depth defines the new depth (1 by default)
     */
    updateSize(width: int, height: int, depth?: int): void;
    /** @internal */
    _rebuild(): void;
    /**
     * @internal
     */
    _swapAndDie(target: InternalTexture, swapAll?: boolean): void;
    /**
     * Dispose the current allocated resources
     */
    dispose(): void;
}
export {};

}
declare module "babylonjs/Materials/Textures/internalTextureLoader" {
import { Nullable } from "babylonjs/types";
import { InternalTexture } from "babylonjs/Materials/Textures/internalTexture";
/**
 * This represents the required contract to create a new type of texture loader.
 */
export interface IInternalTextureLoader {
    /**
     * Defines whether the loader supports cascade loading the different faces.
     */
    supportCascades: boolean;
    /**
     * This returns if the loader support the current file information.
     * @param extension defines the file extension of the file being loaded
     * @param mimeType defines the optional mime type of the file being loaded
     * @returns true if the loader can load the specified file
     */
    canLoad(extension: string, mimeType?: string): boolean;
    /**
     * Uploads the cube texture data to the WebGL texture. It has already been bound.
     * @param data contains the texture data
     * @param texture defines the BabylonJS internal texture
     * @param createPolynomials will be true if polynomials have been requested
     * @param onLoad defines the callback to trigger once the texture is ready
     * @param onError defines the callback to trigger in case of error
     * @param options options to be passed to the loader
     */
    loadCubeData(data: ArrayBufferView | ArrayBufferView[], texture: InternalTexture, createPolynomials: boolean, onLoad: Nullable<(data?: any) => void>, onError: Nullable<(message?: string, exception?: any) => void>, options?: any): void;
    /**
     * Uploads the 2D texture data to the WebGL texture. It has already been bound once in the callback.
     * @param data contains the texture data
     * @param texture defines the BabylonJS internal texture
     * @param callback defines the method to call once ready to upload
     * @param options options to be passed to the loader
     */
    loadData(data: ArrayBufferView, texture: InternalTexture, callback: (width: number, height: number, loadMipmap: boolean, isCompressed: boolean, done: () => void, loadFailed?: boolean) => void, options?: any): void;
}

}
declare module "babylonjs/Materials/Textures/ktx2decoderTypes" {
export enum SourceTextureFormat {
    ETC1S = 0,
    UASTC4x4 = 1
}
export enum TranscodeTarget {
    ASTC_4X4_RGBA = 0,
    BC7_RGBA = 1,
    BC3_RGBA = 2,
    BC1_RGB = 3,
    PVRTC1_4_RGBA = 4,
    PVRTC1_4_RGB = 5,
    ETC2_RGBA = 6,
    ETC1_RGB = 7,
    RGBA32 = 8,
    R8 = 9,
    RG8 = 10
}
export enum EngineFormat {
    COMPRESSED_RGBA_BPTC_UNORM_EXT = 36492,
    COMPRESSED_RGBA_ASTC_4X4_KHR = 37808,
    COMPRESSED_RGB_S3TC_DXT1_EXT = 33776,
    COMPRESSED_RGBA_S3TC_DXT5_EXT = 33779,
    COMPRESSED_RGBA_PVRTC_4BPPV1_IMG = 35842,
    COMPRESSED_RGB_PVRTC_4BPPV1_IMG = 35840,
    COMPRESSED_RGBA8_ETC2_EAC = 37496,
    COMPRESSED_RGB8_ETC2 = 37492,
    COMPRESSED_RGB_ETC1_WEBGL = 36196,
    RGBA8Format = 32856,
    R8Format = 33321,
    RG8Format = 33323
}
/**
 * Leaf node of a decision tree
 * It defines the transcoding format to use to transcode the texture as well as the corresponding format to use at the engine level when creating the texture
 */
export interface ILeaf {
    /**
     * The format to transcode to
     */
    transcodeFormat: TranscodeTarget;
    /**
     * The format to use when creating the texture at the engine level after it has been transcoded to transcodeFormat
     */
    engineFormat: EngineFormat;
    /**
     * Whether the texture must be rounded to a multiple of 4 (should normally be the case for all compressed formats). Default: true
     */
    roundToMultiple4?: boolean;
}
/**
 * Regular node of a decision tree
 *
 * Each property (except for "yes" and "no"), if not empty, will be checked in order to determine the next node to select.
 * If all checks are successful, the "yes" node will be selected, else the "no" node will be selected.
 */
export interface INode {
    /**
     * The name of the capability to check. Can be one of the following:
     *      astc
     *      bptc
     *      s3tc
     *      pvrtc
     *      etc2
     *      etc1
     */
    cap?: string;
    /**
     * The name of the option to check from the options object passed to the KTX2 decode function. {@link IKTX2DecoderOptions}
     */
    option?: string;
    /**
     * Checks if alpha is present in the texture
     */
    alpha?: boolean;
    /**
     * Checks the currently selected transcoding format.
     */
    transcodeFormat?: TranscodeTarget | TranscodeTarget[];
    /**
     * Checks that the texture is a power of two
     */
    needsPowerOfTwo?: boolean;
    /**
     * The node to select if all checks are successful
     */
    yes?: INode | ILeaf;
    /**
     * The node to select if at least one check is not successful
     */
    no?: INode | ILeaf;
}
/**
 * Decision tree used to determine the transcoding format to use for a given source texture format
 */
export interface IDecisionTree {
    /**
     * textureFormat can be either UASTC or ETC1S
     */
    [textureFormat: string]: INode;
}
/**
 * Result of the KTX2 decode function
 */
export interface IDecodedData {
    /**
     * Width of the texture
     */
    width: number;
    /**
     * Height of the texture
     */
    height: number;
    /**
     * The format to use when creating the texture at the engine level
     * This corresponds to the engineFormat property of the leaf node of the decision tree
     */
    transcodedFormat: number;
    /**
     * List of mipmap levels.
     * The first element is the base level, the last element is the smallest mipmap level (if more than one mipmap level is present)
     */
    mipmaps: Array<IMipmap>;
    /**
     * Whether the texture data is in gamma space or not
     */
    isInGammaSpace: boolean;
    /**
     * Whether the texture has an alpha channel or not
     */
    hasAlpha: boolean;
    /**
     * The name of the transcoder used to transcode the texture
     */
    transcoderName: string;
    /**
     * The errors (if any) encountered during the decoding process
     */
    errors?: string;
}
/**
 * Defines a mipmap level
 */
export interface IMipmap {
    /**
     * The data of the mipmap level
     */
    data: Uint8Array | null;
    /**
     * The width of the mipmap level
     */
    width: number;
    /**
     * The height of the mipmap level
     */
    height: number;
}
/**
 * The compressed texture formats supported by the browser
 */
export interface ICompressedFormatCapabilities {
    /**
     * Whether the browser supports ASTC
     */
    astc?: boolean;
    /**
     * Whether the browser supports BPTC
     */
    bptc?: boolean;
    /**
     * Whether the browser supports S3TC
     */
    s3tc?: boolean;
    /**
     * Whether the browser supports PVRTC
     */
    pvrtc?: boolean;
    /**
     * Whether the browser supports ETC2
     */
    etc2?: boolean;
    /**
     * Whether the browser supports ETC1
     */
    etc1?: boolean;
}
/**
 * Options passed to the KTX2 decode function
 */
export interface IKTX2DecoderOptions {
    /** use RGBA format if ASTC and BC7 are not available as transcoded format */
    useRGBAIfASTCBC7NotAvailableWhenUASTC?: boolean;
    /** force to always use (uncompressed) RGBA for transcoded format */
    forceRGBA?: boolean;
    /** force to always use (uncompressed) R8 for transcoded format */
    forceR8?: boolean;
    /** force to always use (uncompressed) RG8 for transcoded format */
    forceRG8?: boolean;
    /**
     * list of transcoders to bypass when looking for a suitable transcoder. The available transcoders are:
     *      UniversalTranscoder_UASTC_ASTC
     *      UniversalTranscoder_UASTC_BC7
     *      UniversalTranscoder_UASTC_RGBA_UNORM
     *      UniversalTranscoder_UASTC_RGBA_SRGB
     *      UniversalTranscoder_UASTC_R8_UNORM
     *      UniversalTranscoder_UASTC_RG8_UNORM
     *      MSCTranscoder
     */
    bypassTranscoders?: string[];
    /**
     * Custom decision tree to apply after the default decision tree has selected a transcoding format.
     * Allows the user to override the default decision tree selection.
     * The decision tree can use the INode.transcodeFormat property to base its decision on the transcoding format selected by the default decision tree.
     */
    transcodeFormatDecisionTree?: IDecisionTree;
}

}
declare module "babylonjs/Materials/Textures/Loaders/basisTextureLoader" {
import { Nullable } from "babylonjs/types";
import { InternalTexture } from "babylonjs/Materials/Textures/internalTexture";
import { IInternalTextureLoader } from "babylonjs/Materials/Textures/internalTextureLoader";
/**
 * Loader for .basis file format
 */
export class _BasisTextureLoader implements IInternalTextureLoader {
    /**
     * Defines whether the loader supports cascade loading the different faces.
     */
    readonly supportCascades: boolean;
    /**
     * This returns if the loader support the current file information.
     * @param extension defines the file extension of the file being loaded
     * @returns true if the loader can load the specified file
     */
    canLoad(extension: string): boolean;
    /**
     * Uploads the cube texture data to the WebGL texture. It has already been bound.
     * @param data contains the texture data
     * @param texture defines the BabylonJS internal texture
     * @param createPolynomials will be true if polynomials have been requested
     * @param onLoad defines the callback to trigger once the texture is ready
     * @param onError defines the callback to trigger in case of error
     */
    loadCubeData(data: ArrayBufferView | ArrayBufferView[], texture: InternalTexture, createPolynomials: boolean, onLoad: Nullable<(data?: any) => void>, onError: Nullable<(message?: string, exception?: any) => void>): void;
    /**
     * Uploads the 2D texture data to the WebGL texture. It has already been bound once in the callback.
     * @param data contains the texture data
     * @param texture defines the BabylonJS internal texture
     * @param callback defines the method to call once ready to upload
     */
    loadData(data: ArrayBufferView, texture: InternalTexture, callback: (width: number, height: number, loadMipmap: boolean, isCompressed: boolean, done: () => void, failedLoading?: boolean) => void): void;
}

}
declare module "babylonjs/Materials/Textures/Loaders/ddsTextureLoader" {
import { Nullable } from "babylonjs/types";
import { InternalTexture } from "babylonjs/Materials/Textures/internalTexture";
import { IInternalTextureLoader } from "babylonjs/Materials/Textures/internalTextureLoader";
/**
 * Implementation of the DDS Texture Loader.
 * @internal
 */
export class _DDSTextureLoader implements IInternalTextureLoader {
    /**
     * Defines whether the loader supports cascade loading the different faces.
     */
    readonly supportCascades: boolean;
    /**
     * This returns if the loader support the current file information.
     * @param extension defines the file extension of the file being loaded
     * @returns true if the loader can load the specified file
     */
    canLoad(extension: string): boolean;
    /**
     * Uploads the cube texture data to the WebGL texture. It has already been bound.
     * @param imgs contains the cube maps
     * @param texture defines the BabylonJS internal texture
     * @param createPolynomials will be true if polynomials have been requested
     * @param onLoad defines the callback to trigger once the texture is ready
     */
    loadCubeData(imgs: ArrayBufferView | ArrayBufferView[], texture: InternalTexture, createPolynomials: boolean, onLoad: Nullable<(data?: any) => void>): void;
    /**
     * Uploads the 2D texture data to the WebGL texture. It has already been bound once in the callback.
     * @param data contains the texture data
     * @param texture defines the BabylonJS internal texture
     * @param callback defines the method to call once ready to upload
     */
    loadData(data: ArrayBufferView, texture: InternalTexture, callback: (width: number, height: number, loadMipmap: boolean, isCompressed: boolean, done: () => void) => void): void;
}

}
declare module "babylonjs/Materials/Textures/Loaders/envTextureLoader" {
import { Nullable } from "babylonjs/types";
import { InternalTexture } from "babylonjs/Materials/Textures/internalTexture";
import { IInternalTextureLoader } from "babylonjs/Materials/Textures/internalTextureLoader";
/**
 * Implementation of the ENV Texture Loader.
 * @internal
 */
export class _ENVTextureLoader implements IInternalTextureLoader {
    /**
     * Defines whether the loader supports cascade loading the different faces.
     */
    readonly supportCascades: boolean;
    /**
     * This returns if the loader support the current file information.
     * @param extension defines the file extension of the file being loaded
     * @returns true if the loader can load the specified file
     */
    canLoad(extension: string): boolean;
    /**
     * Uploads the cube texture data to the WebGL texture. It has already been bound.
     * @param data contains the texture data
     * @param texture defines the BabylonJS internal texture
     * @param createPolynomials will be true if polynomials have been requested
     * @param onLoad defines the callback to trigger once the texture is ready
     * @param onError defines the callback to trigger in case of error
     */
    loadCubeData(data: ArrayBufferView | ArrayBufferView[], texture: InternalTexture, createPolynomials: boolean, onLoad: Nullable<(data?: any) => void>, onError: Nullable<(message?: string, exception?: any) => void>): void;
    /**
     * Uploads the 2D texture data to the WebGL texture. It has already been bound once in the callback.
     */
    loadData(): void;
}

}
declare module "babylonjs/Materials/Textures/Loaders/hdrTextureLoader" {
import { InternalTexture } from "babylonjs/Materials/Textures/internalTexture";
import { IInternalTextureLoader } from "babylonjs/Materials/Textures/internalTextureLoader";
/**
 * Implementation of the HDR Texture Loader.
 * @internal
 */
export class _HDRTextureLoader implements IInternalTextureLoader {
    /**
     * Defines whether the loader supports cascade loading the different faces.
     */
    readonly supportCascades: boolean;
    /**
     * This returns if the loader support the current file information.
     * @param extension defines the file extension of the file being loaded
     * @returns true if the loader can load the specified file
     */
    canLoad(extension: string): boolean;
    /**
     * Uploads the cube texture data to the WebGL texture. It has already been bound.
     */
    loadCubeData(): void;
    /**
     * Uploads the 2D texture data to the WebGL texture. It has already been bound once in the callback.
     * @param data contains the texture data
     * @param texture defines the BabylonJS internal texture
     * @param callback defines the method to call once ready to upload
     */
    loadData(data: ArrayBufferView, texture: InternalTexture, callback: (width: number, height: number, loadMipmap: boolean, isCompressed: boolean, done: () => void) => void): void;
}

}
declare module "babylonjs/Materials/Textures/Loaders/index" {
export * from "babylonjs/Materials/Textures/Loaders/ddsTextureLoader";
export * from "babylonjs/Materials/Textures/Loaders/envTextureLoader";
export * from "babylonjs/Materials/Textures/Loaders/ktxTextureLoader";
export * from "babylonjs/Materials/Textures/Loaders/tgaTextureLoader";
export * from "babylonjs/Materials/Textures/Loaders/hdrTextureLoader";
export * from "babylonjs/Materials/Textures/Loaders/basisTextureLoader";

}
declare module "babylonjs/Materials/Textures/Loaders/ktxTextureLoader" {
import { Nullable } from "babylonjs/types";
import { InternalTexture } from "babylonjs/Materials/Textures/internalTexture";
import { IInternalTextureLoader } from "babylonjs/Materials/Textures/internalTextureLoader";
/**
 * Implementation of the KTX Texture Loader.
 * @internal
 */
export class _KTXTextureLoader implements IInternalTextureLoader {
    /**
     * Defines whether the loader supports cascade loading the different faces.
     */
    readonly supportCascades: boolean;
    /**
     * This returns if the loader support the current file information.
     * @param extension defines the file extension of the file being loaded
     * @param mimeType defines the optional mime type of the file being loaded
     * @returns true if the loader can load the specified file
     */
    canLoad(extension: string, mimeType?: string): boolean;
    /**
     * Uploads the cube texture data to the WebGL texture. It has already been bound.
     * @param data contains the texture data
     * @param texture defines the BabylonJS internal texture
     * @param createPolynomials will be true if polynomials have been requested
     * @param onLoad defines the callback to trigger once the texture is ready
     */
    loadCubeData(data: ArrayBufferView | ArrayBufferView[], texture: InternalTexture, createPolynomials: boolean, onLoad: Nullable<(data?: any) => void>): void;
    /**
     * Uploads the 2D texture data to the WebGL texture. It has already been bound once in the callback.
     * @param data contains the texture data
     * @param texture defines the BabylonJS internal texture
     * @param callback defines the method to call once ready to upload
     * @param options
     */
    loadData(data: ArrayBufferView, texture: InternalTexture, callback: (width: number, height: number, loadMipmap: boolean, isCompressed: boolean, done: () => void, loadFailed: boolean) => void, options?: any): void;
}

}
declare module "babylonjs/Materials/Textures/Loaders/tgaTextureLoader" {
import { InternalTexture } from "babylonjs/Materials/Textures/internalTexture";
import { IInternalTextureLoader } from "babylonjs/Materials/Textures/internalTextureLoader";
/**
 * Implementation of the TGA Texture Loader.
 * @internal
 */
export class _TGATextureLoader implements IInternalTextureLoader {
    /**
     * Defines whether the loader supports cascade loading the different faces.
     */
    readonly supportCascades: boolean;
    /**
     * This returns if the loader support the current file information.
     * @param extension defines the file extension of the file being loaded
     * @returns true if the loader can load the specified file
     */
    canLoad(extension: string): boolean;
    /**
     * Uploads the cube texture data to the WebGL texture. It has already been bound.
     */
    loadCubeData(): void;
    /**
     * Uploads the 2D texture data to the WebGL texture. It has already been bound once in the callback.
     * @param data contains the texture data
     * @param texture defines the BabylonJS internal texture
     * @param callback defines the method to call once ready to upload
     */
    loadData(data: ArrayBufferView, texture: InternalTexture, callback: (width: number, height: number, loadMipmap: boolean, isCompressed: boolean, done: () => void) => void): void;
}

}
declare module "babylonjs/Materials/Textures/mirrorTexture" {
import { Scene } from "babylonjs/scene";
import { RenderTargetTexture } from "babylonjs/Materials/Textures/renderTargetTexture";
import { Plane } from "babylonjs/Maths/math.plane";
/**
 * Mirror texture can be used to simulate the view from a mirror in a scene.
 * It will dynamically be rendered every frame to adapt to the camera point of view.
 * You can then easily use it as a reflectionTexture on a flat surface.
 * In case the surface is not a plane, please consider relying on reflection probes.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/reflectionTexture#mirrortexture
 */
export class MirrorTexture extends RenderTargetTexture {
    /**
     * Define the reflection plane we want to use. The mirrorPlane is usually set to the constructed reflector.
     * It is possible to directly set the mirrorPlane by directly using a Plane(a, b, c, d) where a, b and c give the plane normal vector (a, b, c) and d is a scalar displacement from the mirrorPlane to the origin. However in all but the very simplest of situations it is more straight forward to set it to the reflector as stated in the doc.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/reflectionTexture#mirrors
     */
    mirrorPlane: Plane;
    /**
     * Define the blur ratio used to blur the reflection if needed.
     */
    set blurRatio(value: number);
    get blurRatio(): number;
    /**
     * Define the adaptive blur kernel used to blur the reflection if needed.
     * This will autocompute the closest best match for the `blurKernel`
     */
    set adaptiveBlurKernel(value: number);
    /**
     * Define the blur kernel used to blur the reflection if needed.
     * Please consider using `adaptiveBlurKernel` as it could find the closest best value for you.
     */
    set blurKernel(value: number);
    /**
     * Define the blur kernel on the X Axis used to blur the reflection if needed.
     * Please consider using `adaptiveBlurKernel` as it could find the closest best value for you.
     */
    set blurKernelX(value: number);
    get blurKernelX(): number;
    /**
     * Define the blur kernel on the Y Axis used to blur the reflection if needed.
     * Please consider using `adaptiveBlurKernel` as it could find the closest best value for you.
     */
    set blurKernelY(value: number);
    get blurKernelY(): number;
    private _autoComputeBlurKernel;
    protected _onRatioRescale(): void;
    private _updateGammaSpace;
    private _imageProcessingConfigChangeObserver;
    private _transformMatrix;
    private _mirrorMatrix;
    private _blurX;
    private _blurY;
    private _adaptiveBlurKernel;
    private _blurKernelX;
    private _blurKernelY;
    private _blurRatio;
    private _sceneUBO;
    private _currentSceneUBO;
    /**
     * Instantiates a Mirror Texture.
     * Mirror texture can be used to simulate the view from a mirror in a scene.
     * It will dynamically be rendered every frame to adapt to the camera point of view.
     * You can then easily use it as a reflectionTexture on a flat surface.
     * In case the surface is not a plane, please consider relying on reflection probes.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/reflectionTexture#mirrors
     * @param name
     * @param size
     * @param scene
     * @param generateMipMaps
     * @param type
     * @param samplingMode
     * @param generateDepthBuffer
     */
    constructor(name: string, size: number | {
        width: number;
        height: number;
    } | {
        ratio: number;
    }, scene?: Scene, generateMipMaps?: boolean, type?: number, samplingMode?: number, generateDepthBuffer?: boolean);
    private _preparePostProcesses;
    /**
     * Clone the mirror texture.
     * @returns the cloned texture
     */
    clone(): MirrorTexture;
    /**
     * Serialize the texture to a JSON representation you could use in Parse later on
     * @returns the serialized JSON representation
     */
    serialize(): any;
    /**
     * Dispose the texture and release its associated resources.
     */
    dispose(): void;
}

}
declare module "babylonjs/Materials/Textures/multiRenderTarget" {
import { Scene } from "babylonjs/scene";
import { Engine } from "babylonjs/Engines/engine";
import { Texture } from "babylonjs/Materials/Textures/texture";
import { RenderTargetTexture } from "babylonjs/Materials/Textures/renderTargetTexture";
import "babylonjs/Engines/Extensions/engine.multiRender";
import { InternalTexture } from "babylonjs/Materials/Textures/internalTexture";
/**
 * Creation options of the multi render target texture.
 */
export interface IMultiRenderTargetOptions {
    /**
     * Define if the texture needs to create mip maps after render.
     */
    generateMipMaps?: boolean;
    /**
     * Define the types of all the draw buffers we want to create
     */
    types?: number[];
    /**
     * Define the sampling modes of all the draw buffers we want to create
     */
    samplingModes?: number[];
    /**
     * Define if sRGB format should be used for each of the draw buffers we want to create
     */
    useSRGBBuffers?: boolean[];
    /**
     * Define if a depth buffer is required
     */
    generateDepthBuffer?: boolean;
    /**
     * Define if a stencil buffer is required
     */
    generateStencilBuffer?: boolean;
    /**
     * Define if a depth texture is required instead of a depth buffer
     */
    generateDepthTexture?: boolean;
    /**
     * Define depth texture format to use
     */
    depthTextureFormat?: number;
    /**
     * Define the number of desired draw buffers
     */
    textureCount?: number;
    /**
     * Define if aspect ratio should be adapted to the texture or stay the scene one
     */
    doNotChangeAspectRatio?: boolean;
    /**
     * Define the default type of the buffers we are creating
     */
    defaultType?: number;
    /**
     * Define the default type of the buffers we are creating
     */
    drawOnlyOnFirstAttachmentByDefault?: boolean;
}
/**
 * A multi render target, like a render target provides the ability to render to a texture.
 * Unlike the render target, it can render to several draw buffers in one draw.
 * This is specially interesting in deferred rendering or for any effects requiring more than
 * just one color from a single pass.
 */
export class MultiRenderTarget extends RenderTargetTexture {
    private _textures;
    private _multiRenderTargetOptions;
    private _count;
    private _drawOnlyOnFirstAttachmentByDefault;
    /**
     * Get if draw buffers are currently supported by the used hardware and browser.
     */
    get isSupported(): boolean;
    /**
     * Get the list of textures generated by the multi render target.
     */
    get textures(): Texture[];
    /**
     * Gets the number of textures in this MRT. This number can be different from `_textures.length` in case a depth texture is generated.
     */
    get count(): number;
    /**
     * Get the depth texture generated by the multi render target if options.generateDepthTexture has been set
     */
    get depthTexture(): Texture;
    /**
     * Set the wrapping mode on U of all the textures we are rendering to.
     * Can be any of the Texture. (CLAMP_ADDRESSMODE, MIRROR_ADDRESSMODE or WRAP_ADDRESSMODE)
     */
    set wrapU(wrap: number);
    /**
     * Set the wrapping mode on V of all the textures we are rendering to.
     * Can be any of the Texture. (CLAMP_ADDRESSMODE, MIRROR_ADDRESSMODE or WRAP_ADDRESSMODE)
     */
    set wrapV(wrap: number);
    /**
     * Instantiate a new multi render target texture.
     * A multi render target, like a render target provides the ability to render to a texture.
     * Unlike the render target, it can render to several draw buffers in one draw.
     * This is specially interesting in deferred rendering or for any effects requiring more than
     * just one color from a single pass.
     * @param name Define the name of the texture
     * @param size Define the size of the buffers to render to
     * @param count Define the number of target we are rendering into
     * @param scene Define the scene the texture belongs to
     * @param options Define the options used to create the multi render target
     * @param textureNames Define the names to set to the textures (if count > 0 - optional)
     */
    constructor(name: string, size: any, count: number, scene?: Scene, options?: IMultiRenderTargetOptions, textureNames?: string[]);
    private _initTypes;
    /**
     * @internal
     */
    _rebuild(forceFullRebuild?: boolean, textureNames?: string[]): void;
    private _createInternalTextures;
    private _releaseTextures;
    private _createTextures;
    /**
     * Replaces an internal texture within the MRT. Useful to share textures between MultiRenderTarget.
     * @param texture The new texture to set in the MRT
     * @param index The index of the texture to replace
     * @param disposePrevious Set to true if the previous internal texture should be disposed
     */
    setInternalTexture(texture: InternalTexture, index: number, disposePrevious?: boolean): void;
    /**
     * Define the number of samples used if MSAA is enabled.
     */
    get samples(): number;
    set samples(value: number);
    /**
     * Resize all the textures in the multi render target.
     * Be careful as it will recreate all the data in the new texture.
     * @param size Define the new size
     */
    resize(size: any): void;
    /**
     * Changes the number of render targets in this MRT
     * Be careful as it will recreate all the data in the new texture.
     * @param count new texture count
     * @param options Specifies texture types and sampling modes for new textures
     * @param textureNames Specifies the names of the textures (optional)
     */
    updateCount(count: number, options?: IMultiRenderTargetOptions, textureNames?: string[]): void;
    protected _unbindFrameBuffer(engine: Engine, faceIndex: number): void;
    /**
     * Dispose the render targets and their associated resources
     * @param doNotDisposeInternalTextures
     */
    dispose(doNotDisposeInternalTextures?: boolean): void;
    /**
     * Release all the underlying texture used as draw buffers.
     */
    releaseInternalTextures(): void;
}

}
declare module "babylonjs/Materials/Textures/MultiviewRenderTarget" {
import { RenderTargetTexture } from "babylonjs/Materials/Textures/renderTargetTexture";
import { Scene } from "babylonjs/scene";
/**
 * Renders to multiple views with a single draw call
 * @see https://www.khronos.org/registry/webgl/extensions/OVR_multiview2/
 */
export class MultiviewRenderTarget extends RenderTargetTexture {
    set samples(value: number);
    get samples(): number;
    /**
     * Creates a multiview render target
     * @param scene scene used with the render target
     * @param size the size of the render target (used for each view)
     */
    constructor(scene?: Scene, size?: number | {
        width: number;
        height: number;
    } | {
        ratio: number;
    });
    /**
     * @internal
     */
    _bindFrameBuffer(): void;
    /**
     * Gets the number of views the corresponding to the texture (eg. a MultiviewRenderTarget will have > 1)
     * @returns the view count
     */
    getViewCount(): number;
}

}
declare module "babylonjs/Materials/Textures/Packer/frame" {
import { Vector2 } from "babylonjs/Maths/math.vector";
/**
 * Defines the basic options interface of a TexturePacker Frame
 */
export interface ITexturePackerFrame {
    /**
     * The frame ID
     */
    id: number;
    /**
     * The frames Scale
     */
    scale: Vector2;
    /**
     * The Frames offset
     */
    offset: Vector2;
}
/**
 * This is a support class for frame Data on texture packer sets.
 */
export class TexturePackerFrame implements ITexturePackerFrame {
    /**
     * The frame ID
     */
    id: number;
    /**
     * The frames Scale
     */
    scale: Vector2;
    /**
     * The Frames offset
     */
    offset: Vector2;
    /**
     * Initializes a texture package frame.
     * @param id The numerical frame identifier
     * @param scale Scalar Vector2 for UV frame
     * @param offset Vector2 for the frame position in UV units.
     * @returns TexturePackerFrame
     */
    constructor(id: number, scale: Vector2, offset: Vector2);
}

}
declare module "babylonjs/Materials/Textures/Packer/index" {
export * from "babylonjs/Materials/Textures/Packer/packer";
export * from "babylonjs/Materials/Textures/Packer/frame";

}
declare module "babylonjs/Materials/Textures/Packer/packer" {
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { Scene } from "babylonjs/scene";
import { Nullable } from "babylonjs/types";
import { Color3, Color4 } from "babylonjs/Maths/math.color";
import { TexturePackerFrame } from "babylonjs/Materials/Textures/Packer/frame";
/**
 * Defines the basic options interface of a TexturePacker
 */
export interface ITexturePackerOptions {
    /**
     * Custom targets for the channels of a texture packer.  Default is all the channels of the Standard Material
     */
    map?: string[];
    /**
     * the UV input targets, as a single value for all meshes. Defaults to VertexBuffer.UVKind
     */
    uvsIn?: string;
    /**
     * the UV output targets, as a single value for all meshes.  Defaults to VertexBuffer.UVKind
     */
    uvsOut?: string;
    /**
     * number representing the layout style. Defaults to LAYOUT_STRIP
     */
    layout?: number;
    /**
     * number of columns if using custom column count layout(2).  This defaults to 4.
     */
    colnum?: number;
    /**
     * flag to update the input meshes to the new packed texture after compilation. Defaults to true.
     */
    updateInputMeshes?: boolean;
    /**
     * boolean flag to dispose all the source textures.  Defaults to true.
     */
    disposeSources?: boolean;
    /**
     * Fills the blank cells in a set to the customFillColor.  Defaults to true.
     */
    fillBlanks?: boolean;
    /**
     * string value representing the context fill style color.  Defaults to 'black'.
     */
    customFillColor?: string;
    /**
     * Width and Height Value of each Frame in the TexturePacker Sets
     */
    frameSize?: number;
    /**
     * Ratio of the value to add padding wise to each cell.  Defaults to 0.0115
     */
    paddingRatio?: number;
    /**
     * Number that declares the fill method for the padding gutter.
     */
    paddingMode?: number;
    /**
     * If in SUBUV_COLOR padding mode what color to use.
     */
    paddingColor?: Color3 | Color4;
}
/**
 * Defines the basic interface of a TexturePacker JSON File
 */
export interface ITexturePackerJSON {
    /**
     * The frame ID
     */
    name: string;
    /**
     * The base64 channel data
     */
    sets: any;
    /**
     * The options of the Packer
     */
    options: ITexturePackerOptions;
    /**
     * The frame data of the Packer
     */
    frames: Array<number>;
}
/**
 * This is a support class that generates a series of packed texture sets.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/materials_introduction
 */
export class TexturePacker {
    /** Packer Layout Constant 0 */
    static readonly LAYOUT_STRIP: number;
    /** Packer Layout Constant 1 */
    static readonly LAYOUT_POWER2: number;
    /** Packer Layout Constant 2 */
    static readonly LAYOUT_COLNUM: number;
    /** Packer Layout Constant 0 */
    static readonly SUBUV_WRAP: number;
    /** Packer Layout Constant 1 */
    static readonly SUBUV_EXTEND: number;
    /** Packer Layout Constant 2 */
    static readonly SUBUV_COLOR: number;
    /** The Name of the Texture Package */
    name: string;
    /** The scene scope of the TexturePacker */
    scene: Scene;
    /** The Meshes to target */
    meshes: AbstractMesh[];
    /** Arguments passed with the Constructor */
    options: ITexturePackerOptions;
    /** The promise that is started upon initialization */
    promise: Nullable<Promise<TexturePacker | string>>;
    /** The Container object for the channel sets that are generated */
    sets: object;
    /** The Container array for the frames that are generated */
    frames: TexturePackerFrame[];
    /** The expected number of textures the system is parsing. */
    private _expecting;
    /** The padding value from Math.ceil(frameSize * paddingRatio) */
    private _paddingValue;
    /**
     * Initializes a texture package series from an array of meshes or a single mesh.
     * @param name The name of the package
     * @param meshes The target meshes to compose the package from
     * @param options The arguments that texture packer should follow while building.
     * @param scene The scene which the textures are scoped to.
     * @returns TexturePacker
     */
    constructor(name: string, meshes: AbstractMesh[], options: ITexturePackerOptions, scene: Scene);
    /**
     * Starts the package process
     * @param resolve The promises resolution function
     * @returns TexturePacker
     */
    private _createFrames;
    /**
     * Calculates the Size of the Channel Sets
     * @returns Vector2
     */
    private _calculateSize;
    /**
     * Calculates the UV data for the frames.
     * @param baseSize the base frameSize
     * @param padding the base frame padding
     * @param dtSize size of the Dynamic Texture for that channel
     * @param dtUnits is 1/dtSize
     * @param update flag to update the input meshes
     */
    private _calculateMeshUVFrames;
    /**
     * Calculates the frames Offset.
     * @param index of the frame
     * @returns Vector2
     */
    private _getFrameOffset;
    /**
     * Updates a Mesh to the frame data
     * @param mesh that is the target
     * @param frameID or the frame index
     */
    private _updateMeshUV;
    /**
     * Updates a Meshes materials to use the texture packer channels
     * @param m is the mesh to target
     * @param force all channels on the packer to be set.
     */
    private _updateTextureReferences;
    /**
     * Public method to set a Mesh to a frame
     * @param m that is the target
     * @param frameID or the frame index
     * @param updateMaterial trigger for if the Meshes attached Material be updated?
     */
    setMeshToFrame(m: AbstractMesh, frameID: number, updateMaterial?: boolean): void;
    /**
     * Starts the async promise to compile the texture packer.
     * @returns Promise<void>
     */
    processAsync(): Promise<void>;
    /**
     * Disposes all textures associated with this packer
     */
    dispose(): void;
    /**
     * Starts the download process for all the channels converting them to base64 data and embedding it all in a JSON file.
     * @param imageType is the image type to use.
     * @param quality of the image if downloading as jpeg, Ranges from >0 to 1.
     */
    download(imageType?: string, quality?: number): void;
    /**
     * Public method to load a texturePacker JSON file.
     * @param data of the JSON file in string format.
     */
    updateFromJSON(data: string): void;
}

}
declare module "babylonjs/Materials/Textures/prePassRenderTarget" {
import { IMultiRenderTargetOptions } from "babylonjs/Materials/Textures/multiRenderTarget";
import { MultiRenderTarget } from "babylonjs/Materials/Textures/multiRenderTarget";
import { Engine } from "babylonjs/Engines/engine";
import { RenderTargetTexture } from "babylonjs/Materials/Textures/renderTargetTexture";
import { Scene } from "babylonjs/scene";
import { PostProcess } from "babylonjs/PostProcesses/postProcess";
import { ImageProcessingPostProcess } from "babylonjs/PostProcesses/imageProcessingPostProcess";
import { Nullable } from "babylonjs/types";
/**
 * A multi render target designed to render the prepass.
 * Prepass is a scene component used to render information in multiple textures
 * alongside with the scene materials rendering.
 * Note : This is an internal class, and you should NOT need to instanciate this.
 * Only the `PrePassRenderer` should instanciate this class.
 * It is more likely that you need a regular `MultiRenderTarget`
 * @internal
 */
export class PrePassRenderTarget extends MultiRenderTarget {
    /**
     * @internal
     */
    _beforeCompositionPostProcesses: PostProcess[];
    /**
     * Image processing post process for composition
     */
    imageProcessingPostProcess: ImageProcessingPostProcess;
    /**
     * @internal
     */
    _engine: Engine;
    /**
     * @internal
     */
    _scene: Scene;
    /**
     * @internal
     */
    _outputPostProcess: Nullable<PostProcess>;
    /**
     * @internal
     */
    _internalTextureDirty: boolean;
    /**
     * Is this render target enabled for prepass rendering
     */
    enabled: boolean;
    /**
     * Render target associated with this prePassRenderTarget
     * If this is `null`, it means this prePassRenderTarget is associated with the scene
     */
    renderTargetTexture: Nullable<RenderTargetTexture>;
    constructor(name: string, renderTargetTexture: Nullable<RenderTargetTexture>, size: any, count: number, scene?: Scene, options?: IMultiRenderTargetOptions | undefined);
    /**
     * Creates a composition effect for this RT
     * @internal
     */
    _createCompositionEffect(): void;
    /**
     * Checks that the size of this RT is still adapted to the desired render size.
     * @internal
     */
    _checkSize(): void;
    /**
     * Changes the number of render targets in this MRT
     * Be careful as it will recreate all the data in the new texture.
     * @param count new texture count
     * @param options Specifies texture types and sampling modes for new textures
     * @param textureNames Specifies the names of the textures (optional)
     */
    updateCount(count: number, options?: IMultiRenderTargetOptions, textureNames?: string[]): void;
    /**
     * Resets the post processes chains applied to this RT.
     * @internal
     */
    _resetPostProcessChain(): void;
    /**
     * Diposes this render target
     */
    dispose(): void;
}

}
declare module "babylonjs/Materials/Textures/Procedurals/customProceduralTexture" {
import { Scene } from "babylonjs/scene";
import { Texture } from "babylonjs/Materials/Textures/texture";
import { ProceduralTexture } from "babylonjs/Materials/Textures/Procedurals/proceduralTexture";
import { TextureSize } from "babylonjs/Materials/Textures/textureCreationOptions";
/**
 * Procedural texturing is a way to programmatically create a texture. There are 2 types of procedural textures: code-only, and code that references some classic 2D images, sometimes called 'refMaps' or 'sampler' images.
 * Custom Procedural textures are the easiest way to create your own procedural in your application.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/proceduralTextures#creating-custom-procedural-textures
 */
export class CustomProceduralTexture extends ProceduralTexture {
    private _animate;
    private _time;
    private _config;
    private _texturePath;
    /**
     * Instantiates a new Custom Procedural Texture.
     * Procedural texturing is a way to programmatically create a texture. There are 2 types of procedural textures: code-only, and code that references some classic 2D images, sometimes called 'refMaps' or 'sampler' images.
     * Custom Procedural textures are the easiest way to create your own procedural in your application.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/proceduralTextures#creating-custom-procedural-textures
     * @param name Define the name of the texture
     * @param texturePath Define the folder path containing all the custom texture related files (config, shaders...)
     * @param size Define the size of the texture to create
     * @param scene Define the scene the texture belongs to
     * @param fallbackTexture Define a fallback texture in case there were issues to create the custom texture
     * @param generateMipMaps Define if the texture should creates mip maps or not
     * @param skipJson Define a boolena indicating that there is no json config file to load
     */
    constructor(name: string, texturePath: string, size: TextureSize, scene: Scene, fallbackTexture?: Texture, generateMipMaps?: boolean, skipJson?: boolean);
    private _loadJson;
    /**
     * Is the texture ready to be used ? (rendered at least once)
     * @returns true if ready, otherwise, false.
     */
    isReady(): boolean;
    /**
     * Render the texture to its associated render target.
     * @param useCameraPostProcess Define if camera post process should be applied to the texture
     */
    render(useCameraPostProcess?: boolean): void;
    /**
     * Update the list of dependant textures samplers in the shader.
     */
    updateTextures(): void;
    /**
     * Update the uniform values of the procedural texture in the shader.
     */
    updateShaderUniforms(): void;
    /**
     * Define if the texture animates or not.
     */
    get animate(): boolean;
    set animate(value: boolean);
}

}
declare module "babylonjs/Materials/Textures/Procedurals/index" {
export * from "babylonjs/Materials/Textures/Procedurals/customProceduralTexture";
export * from "babylonjs/Materials/Textures/Procedurals/noiseProceduralTexture";
export * from "babylonjs/Materials/Textures/Procedurals/proceduralTexture";
export * from "babylonjs/Materials/Textures/Procedurals/proceduralTextureSceneComponent";

}
declare module "babylonjs/Materials/Textures/Procedurals/noiseProceduralTexture" {
import { Nullable } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
import { Texture } from "babylonjs/Materials/Textures/texture";
import { ProceduralTexture } from "babylonjs/Materials/Textures/Procedurals/proceduralTexture";
import "babylonjs/Shaders/noise.fragment";
/**
 * Class used to generate noise procedural textures
 */
export class NoiseProceduralTexture extends ProceduralTexture {
    /** Gets or sets the start time (default is 0) */
    time: number;
    /** Gets or sets a value between 0 and 1 indicating the overall brightness of the texture (default is 0.2) */
    brightness: number;
    /** Defines the number of octaves to process */
    octaves: number;
    /** Defines the level of persistence (0.8 by default) */
    persistence: number;
    /** Gets or sets animation speed factor (default is 1) */
    animationSpeedFactor: number;
    /**
     * Creates a new NoiseProceduralTexture
     * @param name defines the name fo the texture
     * @param size defines the size of the texture (default is 256)
     * @param scene defines the hosting scene
     * @param fallbackTexture defines the texture to use if the NoiseProceduralTexture can't be created
     * @param generateMipMaps defines if mipmaps must be generated (true by default)
     */
    constructor(name: string, size?: number, scene?: Nullable<Scene>, fallbackTexture?: Texture, generateMipMaps?: boolean);
    private _updateShaderUniforms;
    protected _getDefines(): string;
    /**
     * Generate the current state of the procedural texture
     * @param useCameraPostProcess
     */
    render(useCameraPostProcess?: boolean): void;
    /**
     * Serializes this noise procedural texture
     * @returns a serialized noise procedural texture object
     */
    serialize(): any;
    /**
     * Clone the texture.
     * @returns the cloned texture
     */
    clone(): NoiseProceduralTexture;
    /**
     * Creates a NoiseProceduralTexture from parsed noise procedural texture data
     * @param parsedTexture defines parsed texture data
     * @param scene defines the current scene
     * @returns a parsed NoiseProceduralTexture
     */
    static Parse(parsedTexture: any, scene: Scene): NoiseProceduralTexture;
}

}
declare module "babylonjs/Materials/Textures/Procedurals/proceduralTexture" {
import { Observable } from "babylonjs/Misc/observable";
import { Nullable } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
import { Matrix, Vector3, Vector2 } from "babylonjs/Maths/math.vector";
import { Color4, Color3 } from "babylonjs/Maths/math.color";
import { Effect } from "babylonjs/Materials/effect";
import { Texture } from "babylonjs/Materials/Textures/texture";
import "babylonjs/Engines/Extensions/engine.renderTarget";
import "babylonjs/Engines/Extensions/engine.renderTargetCube";
import "babylonjs/Shaders/procedural.vertex";
import { NodeMaterial } from "babylonjs/Materials/Node/nodeMaterial";
import { TextureSize } from "babylonjs/Materials/Textures/textureCreationOptions";
/**
 * Procedural texturing is a way to programmatically create a texture. There are 2 types of procedural textures: code-only, and code that references some classic 2D images, sometimes calmpler' images.
 * This is the base class of any Procedural texture and contains most of the shareable code.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/proceduralTextures
 */
export class ProceduralTexture extends Texture {
    /**
     * Define if the texture is enabled or not (disabled texture will not render)
     */
    isEnabled: boolean;
    /**
     * Define if the texture must be cleared before rendering (default is true)
     */
    autoClear: boolean;
    /**
     * Callback called when the texture is generated
     */
    onGenerated: () => void;
    /**
     * Event raised when the texture is generated
     */
    onGeneratedObservable: Observable<ProceduralTexture>;
    /**
     * Event raised before the texture is generated
     */
    onBeforeGenerationObservable: Observable<ProceduralTexture>;
    /**
     * Gets or sets the node material used to create this texture (null if the texture was manually created)
     */
    nodeMaterialSource: Nullable<NodeMaterial>;
    /** @internal */
    _generateMipMaps: boolean;
    private _drawWrapper;
    /** @internal */
    _textures: {
        [key: string]: Texture;
    };
    /** @internal */
    protected _fallbackTexture: Nullable<Texture>;
    private _size;
    private _textureType;
    private _currentRefreshId;
    private _frameId;
    private _refreshRate;
    private _vertexBuffers;
    private _indexBuffer;
    private _uniforms;
    private _samplers;
    private _fragment;
    private _floats;
    private _ints;
    private _floatsArrays;
    private _colors3;
    private _colors4;
    private _vectors2;
    private _vectors3;
    private _matrices;
    private _fallbackTextureUsed;
    private _fullEngine;
    private _cachedDefines;
    private _contentUpdateId;
    private _contentData;
    private _rtWrapper;
    /**
     * Instantiates a new procedural texture.
     * Procedural texturing is a way to programmatically create a texture. There are 2 types of procedural textures: code-only, and code that references some classic 2D images, sometimes called 'refMaps' or 'sampler' images.
     * This is the base class of any Procedural texture and contains most of the shareable code.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/proceduralTextures
     * @param name  Define the name of the texture
     * @param size Define the size of the texture to create
     * @param fragment Define the fragment shader to use to generate the texture or null if it is defined later
     * @param scene Define the scene the texture belongs to
     * @param fallbackTexture Define a fallback texture in case there were issues to create the custom texture
     * @param generateMipMaps Define if the texture should creates mip maps or not
     * @param isCube Define if the texture is a cube texture or not (this will render each faces of the cube)
     * @param textureType The FBO internal texture type
     */
    constructor(name: string, size: TextureSize, fragment: any, scene: Nullable<Scene>, fallbackTexture?: Nullable<Texture>, generateMipMaps?: boolean, isCube?: boolean, textureType?: number);
    private _createRtWrapper;
    /**
     * The effect that is created when initializing the post process.
     * @returns The created effect corresponding the the postprocess.
     */
    getEffect(): Effect;
    /**
     * @internal*
     */
    _setEffect(effect: Effect): void;
    /**
     * Gets texture content (Use this function wisely as reading from a texture can be slow)
     * @returns an ArrayBufferView promise (Uint8Array or Float32Array)
     */
    getContent(): Nullable<Promise<ArrayBufferView>>;
    private _createIndexBuffer;
    /** @internal */
    _rebuild(): void;
    /**
     * Resets the texture in order to recreate its associated resources.
     * This can be called in case of context loss
     */
    reset(): void;
    protected _getDefines(): string;
    /**
     * Is the texture ready to be used ? (rendered at least once)
     * @returns true if ready, otherwise, false.
     */
    isReady(): boolean;
    /**
     * Resets the refresh counter of the texture and start bak from scratch.
     * Could be useful to regenerate the texture if it is setup to render only once.
     */
    resetRefreshCounter(): void;
    /**
     * Set the fragment shader to use in order to render the texture.
     * @param fragment This can be set to a path (into the shader store) or to a json object containing a fragmentElement property.
     */
    setFragment(fragment: any): void;
    /**
     * Define the refresh rate of the texture or the rendering frequency.
     * Use 0 to render just once, 1 to render on every frame, 2 to render every two frames and so on...
     */
    get refreshRate(): number;
    set refreshRate(value: number);
    /** @internal */
    _shouldRender(): boolean;
    /**
     * Get the size the texture is rendering at.
     * @returns the size (on cube texture it is always squared)
     */
    getRenderSize(): TextureSize;
    /**
     * Resize the texture to new value.
     * @param size Define the new size the texture should have
     * @param generateMipMaps Define whether the new texture should create mip maps
     */
    resize(size: TextureSize, generateMipMaps: boolean): void;
    private _checkUniform;
    /**
     * Set a texture in the shader program used to render.
     * @param name Define the name of the uniform samplers as defined in the shader
     * @param texture Define the texture to bind to this sampler
     * @returns the texture itself allowing "fluent" like uniform updates
     */
    setTexture(name: string, texture: Texture): ProceduralTexture;
    /**
     * Set a float in the shader.
     * @param name Define the name of the uniform as defined in the shader
     * @param value Define the value to give to the uniform
     * @returns the texture itself allowing "fluent" like uniform updates
     */
    setFloat(name: string, value: number): ProceduralTexture;
    /**
     * Set a int in the shader.
     * @param name Define the name of the uniform as defined in the shader
     * @param value Define the value to give to the uniform
     * @returns the texture itself allowing "fluent" like uniform updates
     */
    setInt(name: string, value: number): ProceduralTexture;
    /**
     * Set an array of floats in the shader.
     * @param name Define the name of the uniform as defined in the shader
     * @param value Define the value to give to the uniform
     * @returns the texture itself allowing "fluent" like uniform updates
     */
    setFloats(name: string, value: number[]): ProceduralTexture;
    /**
     * Set a vec3 in the shader from a Color3.
     * @param name Define the name of the uniform as defined in the shader
     * @param value Define the value to give to the uniform
     * @returns the texture itself allowing "fluent" like uniform updates
     */
    setColor3(name: string, value: Color3): ProceduralTexture;
    /**
     * Set a vec4 in the shader from a Color4.
     * @param name Define the name of the uniform as defined in the shader
     * @param value Define the value to give to the uniform
     * @returns the texture itself allowing "fluent" like uniform updates
     */
    setColor4(name: string, value: Color4): ProceduralTexture;
    /**
     * Set a vec2 in the shader from a Vector2.
     * @param name Define the name of the uniform as defined in the shader
     * @param value Define the value to give to the uniform
     * @returns the texture itself allowing "fluent" like uniform updates
     */
    setVector2(name: string, value: Vector2): ProceduralTexture;
    /**
     * Set a vec3 in the shader from a Vector3.
     * @param name Define the name of the uniform as defined in the shader
     * @param value Define the value to give to the uniform
     * @returns the texture itself allowing "fluent" like uniform updates
     */
    setVector3(name: string, value: Vector3): ProceduralTexture;
    /**
     * Set a mat4 in the shader from a MAtrix.
     * @param name Define the name of the uniform as defined in the shader
     * @param value Define the value to give to the uniform
     * @returns the texture itself allowing "fluent" like uniform updates
     */
    setMatrix(name: string, value: Matrix): ProceduralTexture;
    /**
     * Render the texture to its associated render target.
     * @param useCameraPostProcess Define if camera post process should be applied to the texture
     */
    render(useCameraPostProcess?: boolean): void;
    /**
     * Clone the texture.
     * @returns the cloned texture
     */
    clone(): ProceduralTexture;
    /**
     * Dispose the texture and release its associated resources.
     */
    dispose(): void;
}

}
declare module "babylonjs/Materials/Textures/Procedurals/proceduralTextureSceneComponent" {
import { Scene } from "babylonjs/scene";
import { ISceneComponent } from "babylonjs/sceneComponent";
import { ProceduralTexture } from "babylonjs/Materials/Textures/Procedurals/proceduralTexture";
module "babylonjs/abstractScene" {
    interface AbstractScene {
        /**
         * The list of procedural textures added to the scene
         * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/proceduralTextures
         */
        proceduralTextures: Array<ProceduralTexture>;
    }
}
/**
 * Defines the Procedural Texture scene component responsible to manage any Procedural Texture
 * in a given scene.
 */
export class ProceduralTextureSceneComponent implements ISceneComponent {
    /**
     * The component name helpful to identify the component in the list of scene components.
     */
    readonly name: string;
    /**
     * The scene the component belongs to.
     */
    scene: Scene;
    /**
     * Creates a new instance of the component for the given scene
     * @param scene Defines the scene to register the component in
     */
    constructor(scene: Scene);
    /**
     * Registers the component in a given scene
     */
    register(): void;
    /**
     * Rebuilds the elements related to this component in case of
     * context lost for instance.
     */
    rebuild(): void;
    /**
     * Disposes the component and the associated resources.
     */
    dispose(): void;
    private _beforeClear;
}

}
declare module "babylonjs/Materials/Textures/rawCubeTexture" {
import { Nullable } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
import { SphericalPolynomial } from "babylonjs/Maths/sphericalPolynomial";
import { CubeTexture } from "babylonjs/Materials/Textures/cubeTexture";
import "babylonjs/Engines/Extensions/engine.rawTexture";
/**
 * Raw cube texture where the raw buffers are passed in
 */
export class RawCubeTexture extends CubeTexture {
    /**
     * Creates a cube texture where the raw buffers are passed in.
     * @param scene defines the scene the texture is attached to
     * @param data defines the array of data to use to create each face
     * @param size defines the size of the textures
     * @param format defines the format of the data
     * @param type defines the type of the data (like Engine.TEXTURETYPE_UNSIGNED_INT)
     * @param generateMipMaps  defines if the engine should generate the mip levels
     * @param invertY defines if data must be stored with Y axis inverted
     * @param samplingMode defines the required sampling mode (like Texture.NEAREST_SAMPLINGMODE)
     * @param compression defines the compression used (null by default)
     */
    constructor(scene: Scene, data: Nullable<ArrayBufferView[]>, size: number, format?: number, type?: number, generateMipMaps?: boolean, invertY?: boolean, samplingMode?: number, compression?: Nullable<string>);
    /**
     * Updates the raw cube texture.
     * @param data defines the data to store
     * @param format defines the data format
     * @param type defines the type fo the data (Engine.TEXTURETYPE_UNSIGNED_INT by default)
     * @param invertY defines if data must be stored with Y axis inverted
     * @param compression defines the compression used (null by default)
     */
    update(data: ArrayBufferView[], format: number, type: number, invertY: boolean, compression?: Nullable<string>): void;
    /**
     * Updates a raw cube texture with RGBD encoded data.
     * @param data defines the array of data [mipmap][face] to use to create each face
     * @param sphericalPolynomial defines the spherical polynomial for irradiance
     * @param lodScale defines the scale applied to environment texture. This manages the range of LOD level used for IBL according to the roughness
     * @param lodOffset defines the offset applied to environment texture. This manages first LOD level used for IBL according to the roughness
     * @returns a promise that resolves when the operation is complete
     */
    updateRGBDAsync(data: ArrayBufferView[][], sphericalPolynomial?: Nullable<SphericalPolynomial>, lodScale?: number, lodOffset?: number): Promise<void>;
    /**
     * Clones the raw cube texture.
     * @returns a new cube texture
     */
    clone(): CubeTexture;
}

}
declare module "babylonjs/Materials/Textures/rawTexture" {
import { Texture } from "babylonjs/Materials/Textures/texture";
import "babylonjs/Engines/Extensions/engine.rawTexture";
import { Nullable } from "babylonjs/types";
import { ThinEngine } from "babylonjs/Engines/thinEngine";
import { Scene } from "babylonjs/scene";
/**
 * Raw texture can help creating a texture directly from an array of data.
 * This can be super useful if you either get the data from an uncompressed source or
 * if you wish to create your texture pixel by pixel.
 */
export class RawTexture extends Texture {
    /**
     * Define the format of the data (RGB, RGBA... Engine.TEXTUREFORMAT_xxx)
     */
    format: number;
    /**
     * Instantiates a new RawTexture.
     * Raw texture can help creating a texture directly from an array of data.
     * This can be super useful if you either get the data from an uncompressed source or
     * if you wish to create your texture pixel by pixel.
     * @param data define the array of data to use to create the texture (null to create an empty texture)
     * @param width define the width of the texture
     * @param height define the height of the texture
     * @param format define the format of the data (RGB, RGBA... Engine.TEXTUREFORMAT_xxx)
     * @param sceneOrEngine defines the scene or engine the texture will belong to
     * @param generateMipMaps define whether mip maps should be generated or not
     * @param invertY define if the data should be flipped on Y when uploaded to the GPU
     * @param samplingMode define the texture sampling mode (Texture.xxx_SAMPLINGMODE)
     * @param type define the format of the data (int, float... Engine.TEXTURETYPE_xxx)
     * @param creationFlags specific flags to use when creating the texture (Constants.TEXTURE_CREATIONFLAG_STORAGE for storage textures, for eg)
     * @param useSRGBBuffer defines if the texture must be loaded in a sRGB GPU buffer (if supported by the GPU).
     */
    constructor(data: Nullable<ArrayBufferView>, width: number, height: number, 
    /**
     * Define the format of the data (RGB, RGBA... Engine.TEXTUREFORMAT_xxx)
     */
    format: number, sceneOrEngine: Nullable<Scene | ThinEngine>, generateMipMaps?: boolean, invertY?: boolean, samplingMode?: number, type?: number, creationFlags?: number, useSRGBBuffer?: boolean);
    /**
     * Updates the texture underlying data.
     * @param data Define the new data of the texture
     */
    update(data: ArrayBufferView): void;
    /**
     * Creates a luminance texture from some data.
     * @param data Define the texture data
     * @param width Define the width of the texture
     * @param height Define the height of the texture
     * @param sceneOrEngine defines the scene or engine the texture will belong to
     * @param generateMipMaps Define whether or not to create mip maps for the texture
     * @param invertY define if the data should be flipped on Y when uploaded to the GPU
     * @param samplingMode define the texture sampling mode (Texture.xxx_SAMPLINGMODE)
     * @returns the luminance texture
     */
    static CreateLuminanceTexture(data: Nullable<ArrayBufferView>, width: number, height: number, sceneOrEngine: Nullable<Scene | ThinEngine>, generateMipMaps?: boolean, invertY?: boolean, samplingMode?: number): RawTexture;
    /**
     * Creates a luminance alpha texture from some data.
     * @param data Define the texture data
     * @param width Define the width of the texture
     * @param height Define the height of the texture
     * @param sceneOrEngine defines the scene or engine the texture will belong to
     * @param generateMipMaps Define whether or not to create mip maps for the texture
     * @param invertY define if the data should be flipped on Y when uploaded to the GPU
     * @param samplingMode define the texture sampling mode (Texture.xxx_SAMPLINGMODE)
     * @returns the luminance alpha texture
     */
    static CreateLuminanceAlphaTexture(data: Nullable<ArrayBufferView>, width: number, height: number, sceneOrEngine: Nullable<Scene | ThinEngine>, generateMipMaps?: boolean, invertY?: boolean, samplingMode?: number): RawTexture;
    /**
     * Creates an alpha texture from some data.
     * @param data Define the texture data
     * @param width Define the width of the texture
     * @param height Define the height of the texture
     * @param sceneOrEngine defines the scene or engine the texture will belong to
     * @param generateMipMaps Define whether or not to create mip maps for the texture
     * @param invertY define if the data should be flipped on Y when uploaded to the GPU
     * @param samplingMode define the texture sampling mode (Texture.xxx_SAMPLINGMODE)
     * @returns the alpha texture
     */
    static CreateAlphaTexture(data: Nullable<ArrayBufferView>, width: number, height: number, sceneOrEngine: Nullable<Scene | ThinEngine>, generateMipMaps?: boolean, invertY?: boolean, samplingMode?: number): RawTexture;
    /**
     * Creates a RGB texture from some data.
     * @param data Define the texture data
     * @param width Define the width of the texture
     * @param height Define the height of the texture
     * @param sceneOrEngine defines the scene or engine the texture will belong to
     * @param generateMipMaps Define whether or not to create mip maps for the texture
     * @param invertY define if the data should be flipped on Y when uploaded to the GPU
     * @param samplingMode define the texture sampling mode (Texture.xxx_SAMPLINGMODE)
     * @param type define the format of the data (int, float... Engine.TEXTURETYPE_xxx)
     * @param creationFlags specific flags to use when creating the texture (Constants.TEXTURE_CREATIONFLAG_STORAGE for storage textures, for eg)
     * @param useSRGBBuffer defines if the texture must be loaded in a sRGB GPU buffer (if supported by the GPU).
     * @returns the RGB alpha texture
     */
    static CreateRGBTexture(data: Nullable<ArrayBufferView>, width: number, height: number, sceneOrEngine: Nullable<Scene | ThinEngine>, generateMipMaps?: boolean, invertY?: boolean, samplingMode?: number, type?: number, creationFlags?: number, useSRGBBuffer?: boolean): RawTexture;
    /**
     * Creates a RGBA texture from some data.
     * @param data Define the texture data
     * @param width Define the width of the texture
     * @param height Define the height of the texture
     * @param sceneOrEngine defines the scene or engine the texture will belong to
     * @param generateMipMaps Define whether or not to create mip maps for the texture
     * @param invertY define if the data should be flipped on Y when uploaded to the GPU
     * @param samplingMode define the texture sampling mode (Texture.xxx_SAMPLINGMODE)
     * @param type define the format of the data (int, float... Engine.TEXTURETYPE_xxx)
     * @param creationFlags specific flags to use when creating the texture (Constants.TEXTURE_CREATIONFLAG_STORAGE for storage textures, for eg)
     * @param useSRGBBuffer defines if the texture must be loaded in a sRGB GPU buffer (if supported by the GPU).
     * @returns the RGBA texture
     */
    static CreateRGBATexture(data: Nullable<ArrayBufferView>, width: number, height: number, sceneOrEngine: Nullable<Scene | ThinEngine>, generateMipMaps?: boolean, invertY?: boolean, samplingMode?: number, type?: number, creationFlags?: number, useSRGBBuffer?: boolean): RawTexture;
    /**
     * Creates a RGBA storage texture from some data.
     * @param data Define the texture data
     * @param width Define the width of the texture
     * @param height Define the height of the texture
     * @param sceneOrEngine defines the scene or engine the texture will belong to
     * @param generateMipMaps Define whether or not to create mip maps for the texture
     * @param invertY define if the data should be flipped on Y when uploaded to the GPU
     * @param samplingMode define the texture sampling mode (Texture.xxx_SAMPLINGMODE)
     * @param type define the format of the data (int, float... Engine.TEXTURETYPE_xxx)
     * @param useSRGBBuffer defines if the texture must be loaded in a sRGB GPU buffer (if supported by the GPU).
     * @returns the RGBA texture
     */
    static CreateRGBAStorageTexture(data: Nullable<ArrayBufferView>, width: number, height: number, sceneOrEngine: Nullable<Scene | ThinEngine>, generateMipMaps?: boolean, invertY?: boolean, samplingMode?: number, type?: number, useSRGBBuffer?: boolean): RawTexture;
    /**
     * Creates a R texture from some data.
     * @param data Define the texture data
     * @param width Define the width of the texture
     * @param height Define the height of the texture
     * @param sceneOrEngine defines the scene or engine the texture will belong to
     * @param generateMipMaps Define whether or not to create mip maps for the texture
     * @param invertY define if the data should be flipped on Y when uploaded to the GPU
     * @param samplingMode define the texture sampling mode (Texture.xxx_SAMPLINGMODE)
     * @param type define the format of the data (int, float... Engine.TEXTURETYPE_xxx)
     * @returns the R texture
     */
    static CreateRTexture(data: Nullable<ArrayBufferView>, width: number, height: number, sceneOrEngine: Nullable<Scene | ThinEngine>, generateMipMaps?: boolean, invertY?: boolean, samplingMode?: number, type?: number): RawTexture;
    /**
     * Creates a R storage texture from some data.
     * @param data Define the texture data
     * @param width Define the width of the texture
     * @param height Define the height of the texture
     * @param sceneOrEngine defines the scene or engine the texture will belong to
     * @param generateMipMaps Define whether or not to create mip maps for the texture
     * @param invertY define if the data should be flipped on Y when uploaded to the GPU
     * @param samplingMode define the texture sampling mode (Texture.xxx_SAMPLINGMODE)
     * @param type define the format of the data (int, float... Engine.TEXTURETYPE_xxx)
     * @returns the R texture
     */
    static CreateRStorageTexture(data: Nullable<ArrayBufferView>, width: number, height: number, sceneOrEngine: Nullable<Scene | ThinEngine>, generateMipMaps?: boolean, invertY?: boolean, samplingMode?: number, type?: number): RawTexture;
}
export {};

}
declare module "babylonjs/Materials/Textures/rawTexture2DArray" {
import { Texture } from "babylonjs/Materials/Textures/texture";
import "babylonjs/Engines/Extensions/engine.rawTexture";
import { Scene } from "babylonjs/scene";
/**
 * Class used to store 2D array textures containing user data
 */
export class RawTexture2DArray extends Texture {
    /** Gets or sets the texture format to use */
    format: number;
    private _depth;
    /**
     * Gets the number of layers of the texture
     */
    get depth(): number;
    /**
     * Create a new RawTexture2DArray
     * @param data defines the data of the texture
     * @param width defines the width of the texture
     * @param height defines the height of the texture
     * @param depth defines the number of layers of the texture
     * @param format defines the texture format to use
     * @param scene defines the hosting scene
     * @param generateMipMaps defines a boolean indicating if mip levels should be generated (true by default)
     * @param invertY defines if texture must be stored with Y axis inverted
     * @param samplingMode defines the sampling mode to use (Texture.TRILINEAR_SAMPLINGMODE by default)
     * @param textureType defines the texture Type (Engine.TEXTURETYPE_UNSIGNED_INT, Engine.TEXTURETYPE_FLOAT...)
     */
    constructor(data: ArrayBufferView, width: number, height: number, depth: number, 
    /** Gets or sets the texture format to use */
    format: number, scene: Scene, generateMipMaps?: boolean, invertY?: boolean, samplingMode?: number, textureType?: number);
    /**
     * Update the texture with new data
     * @param data defines the data to store in the texture
     */
    update(data: ArrayBufferView): void;
    /**
     * Creates a RGBA texture from some data.
     * @param data Define the texture data
     * @param width Define the width of the texture
     * @param height Define the height of the texture
     * @param depth defines the number of layers of the texture
     * @param scene defines the scene the texture will belong to
     * @param generateMipMaps Define whether or not to create mip maps for the texture
     * @param invertY define if the data should be flipped on Y when uploaded to the GPU
     * @param samplingMode define the texture sampling mode (Texture.xxx_SAMPLINGMODE)
     * @param type define the format of the data (int, float... Engine.TEXTURETYPE_xxx)
     * @returns the RGBA texture
     */
    static CreateRGBATexture(data: ArrayBufferView, width: number, height: number, depth: number, scene: Scene, generateMipMaps?: boolean, invertY?: boolean, samplingMode?: number, type?: number): RawTexture2DArray;
}
export {};

}
declare module "babylonjs/Materials/Textures/rawTexture3D" {
import { Scene } from "babylonjs/scene";
import { Texture } from "babylonjs/Materials/Textures/texture";
import "babylonjs/Engines/Extensions/engine.rawTexture";
/**
 * Class used to store 3D textures containing user data
 */
export class RawTexture3D extends Texture {
    /** Gets or sets the texture format to use */
    format: number;
    /**
     * Create a new RawTexture3D
     * @param data defines the data of the texture
     * @param width defines the width of the texture
     * @param height defines the height of the texture
     * @param depth defines the depth of the texture
     * @param format defines the texture format to use
     * @param scene defines the hosting scene
     * @param generateMipMaps defines a boolean indicating if mip levels should be generated (true by default)
     * @param invertY defines if texture must be stored with Y axis inverted
     * @param samplingMode defines the sampling mode to use (Texture.TRILINEAR_SAMPLINGMODE by default)
     * @param textureType defines the texture Type (Engine.TEXTURETYPE_UNSIGNED_INT, Engine.TEXTURETYPE_FLOAT...)
     */
    constructor(data: ArrayBufferView, width: number, height: number, depth: number, 
    /** Gets or sets the texture format to use */
    format: number, scene: Scene, generateMipMaps?: boolean, invertY?: boolean, samplingMode?: number, textureType?: number);
    /**
     * Update the texture with new data
     * @param data defines the data to store in the texture
     */
    update(data: ArrayBufferView): void;
}

}
declare module "babylonjs/Materials/Textures/refractionTexture" {
import { Scene } from "babylonjs/scene";
import { Plane } from "babylonjs/Maths/math.plane";
import { RenderTargetTexture } from "babylonjs/Materials/Textures/renderTargetTexture";
/**
 * Creates a refraction texture used by refraction channel of the standard material.
 * It is like a mirror but to see through a material.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/reflectionTexture#refractiontexture
 */
export class RefractionTexture extends RenderTargetTexture {
    /**
     * Define the reflection plane we want to use. The refractionPlane is usually set to the constructed refractor.
     * It is possible to directly set the refractionPlane by directly using a Plane(a, b, c, d) where a, b and c give the plane normal vector (a, b, c) and d is a scalar displacement from the refractionPlane to the origin. However in all but the very simplest of situations it is more straight forward to set it to the refractor as stated in the doc.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/reflectionTexture#refraction
     */
    refractionPlane: Plane;
    /**
     * Define how deep under the surface we should see.
     */
    depth: number;
    /**
     * Creates a refraction texture used by refraction channel of the standard material.
     * It is like a mirror but to see through a material.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/reflectionTexture#refraction
     * @param name Define the texture name
     * @param size Define the size of the underlying texture
     * @param scene Define the scene the refraction belongs to
     * @param generateMipMaps Define if we need to generate mips level for the refraction
     */
    constructor(name: string, size: number, scene?: Scene, generateMipMaps?: boolean);
    /**
     * Clone the refraction texture.
     * @returns the cloned texture
     */
    clone(): RefractionTexture;
    /**
     * Serialize the texture to a JSON representation you could use in Parse later on
     * @returns the serialized JSON representation
     */
    serialize(): any;
}

}
declare module "babylonjs/Materials/Textures/renderTargetTexture" {
import { Observable } from "babylonjs/Misc/observable";
import { SmartArray } from "babylonjs/Misc/smartArray";
import { Nullable, Immutable } from "babylonjs/types";
import { Camera } from "babylonjs/Cameras/camera";
import { Scene } from "babylonjs/scene";
import { Matrix, Vector3 } from "babylonjs/Maths/math.vector";
import { Color4 } from "babylonjs/Maths/math.color";
import { RenderTargetCreationOptions, TextureSize } from "babylonjs/Materials/Textures/textureCreationOptions";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { SubMesh } from "babylonjs/Meshes/subMesh";
import { InternalTexture } from "babylonjs/Materials/Textures/internalTexture";
import { Texture } from "babylonjs/Materials/Textures/texture";
import { PostProcess } from "babylonjs/PostProcesses/postProcess";
import { RenderingManager } from "babylonjs/Rendering/renderingManager";
import { IRenderTargetTexture, RenderTargetWrapper } from "babylonjs/Engines/renderTargetWrapper";
import "babylonjs/Engines/Extensions/engine.renderTarget";
import "babylonjs/Engines/Extensions/engine.renderTargetCube";
import { Engine } from "babylonjs/Engines/engine";
import { Material } from "babylonjs/Materials/material";
/**
 * Options for the RenderTargetTexture constructor
 */
export interface RenderTargetTextureOptions {
    /** True (default: false) if mipmaps need to be generated after render */
    generateMipMaps?: boolean;
    /** True (default) to not change the aspect ratio of the scene in the RTT */
    doNotChangeAspectRatio?: boolean;
    /** The type of the buffer in the RTT (byte (default), half float, float...) */
    type?: number;
    /** True (default: false) if a cube texture needs to be created */
    isCube?: boolean;
    /** The sampling mode to be used with the render target (Trilinear (default), Linear, Nearest...) */
    samplingMode?: number;
    /** True (default) to generate a depth buffer */
    generateDepthBuffer?: boolean;
    /** True (default: false) to generate a stencil buffer */
    generateStencilBuffer?: boolean;
    /** True (default: false) if multiple textures need to be created (Draw Buffers) */
    isMulti?: boolean;
    /** The internal format of the buffer in the RTT (RED, RG, RGB, RGBA (default), ALPHA...) */
    format?: number;
    /** True (default: false) if the texture allocation should be delayed */
    delayAllocation?: boolean;
    /** Sample count to use when creating the RTT */
    samples?: number;
    /** specific flags to use when creating the texture (e.g., Constants.TEXTURE_CREATIONFLAG_STORAGE for storage textures) */
    creationFlags?: number;
    /** True (default: false) to indicate that no color target should be created. (e.g., if you only want to write to the depth buffer) */
    noColorAttachment?: boolean;
    /** Specifies the internal texture to use directly instead of creating one (ignores `noColorAttachment` flag when set) **/
    colorAttachment?: InternalTexture;
    /** True (default: false) to create a SRGB texture */
    useSRGBBuffer?: boolean;
}
/**
 * This Helps creating a texture that will be created from a camera in your scene.
 * It is basically a dynamic texture that could be used to create special effects for instance.
 * Actually, It is the base of lot of effects in the framework like post process, shadows, effect layers and rendering pipelines...
 */
export class RenderTargetTexture extends Texture implements IRenderTargetTexture {
    /**
     * The texture will only be rendered once which can be useful to improve performance if everything in your render is static for instance.
     */
    static readonly REFRESHRATE_RENDER_ONCE: number;
    /**
     * The texture will only be rendered rendered every frame and is recommended for dynamic contents.
     */
    static readonly REFRESHRATE_RENDER_ONEVERYFRAME: number;
    /**
     * The texture will be rendered every 2 frames which could be enough if your dynamic objects are not
     * the central point of your effect and can save a lot of performances.
     */
    static readonly REFRESHRATE_RENDER_ONEVERYTWOFRAMES: number;
    /**
     * Use this predicate to dynamically define the list of mesh you want to render.
     * If set, the renderList property will be overwritten.
     */
    renderListPredicate: (AbstractMesh: AbstractMesh) => boolean;
    private _renderList;
    private _unObserveRenderList;
    /**
     * Use this list to define the list of mesh you want to render.
     */
    get renderList(): Nullable<Array<AbstractMesh>>;
    set renderList(value: Nullable<Array<AbstractMesh>>);
    private _renderListHasChanged;
    /**
     * Use this function to overload the renderList array at rendering time.
     * Return null to render with the current renderList, else return the list of meshes to use for rendering.
     * For 2DArray RTT, layerOrFace is the index of the layer that is going to be rendered, else it is the faceIndex of
     * the cube (if the RTT is a cube, else layerOrFace=0).
     * The renderList passed to the function is the current render list (the one that will be used if the function returns null).
     * The length of this list is passed through renderListLength: don't use renderList.length directly because the array can
     * hold dummy elements!
     */
    getCustomRenderList: (layerOrFace: number, renderList: Nullable<Immutable<Array<AbstractMesh>>>, renderListLength: number) => Nullable<Array<AbstractMesh>>;
    /**
     * Define if particles should be rendered in your texture.
     */
    renderParticles: boolean;
    /**
     * Define if sprites should be rendered in your texture.
     */
    renderSprites: boolean;
    /**
     * Force checking the layerMask property even if a custom list of meshes is provided (ie. if renderList is not undefined)
     */
    forceLayerMaskCheck: boolean;
    /**
     * Define the camera used to render the texture.
     */
    activeCamera: Nullable<Camera>;
    /**
     * Override the mesh isReady function with your own one.
     */
    customIsReadyFunction: (mesh: AbstractMesh, refreshRate: number, preWarm?: boolean) => boolean;
    /**
     * Override the render function of the texture with your own one.
     */
    customRenderFunction: (opaqueSubMeshes: SmartArray<SubMesh>, alphaTestSubMeshes: SmartArray<SubMesh>, transparentSubMeshes: SmartArray<SubMesh>, depthOnlySubMeshes: SmartArray<SubMesh>, beforeTransparents?: () => void) => void;
    /**
     * Define if camera post processes should be use while rendering the texture.
     */
    useCameraPostProcesses: boolean;
    /**
     * Define if the camera viewport should be respected while rendering the texture or if the render should be done to the entire texture.
     */
    ignoreCameraViewport: boolean;
    private _postProcessManager;
    /**
     * Post-processes for this render target
     */
    get postProcesses(): PostProcess[];
    private _postProcesses;
    private _resizeObserver;
    private get _prePassEnabled();
    /**
     * An event triggered when the texture is unbind.
     */
    onBeforeBindObservable: Observable<RenderTargetTexture>;
    /**
     * An event triggered when the texture is unbind.
     */
    onAfterUnbindObservable: Observable<RenderTargetTexture>;
    private _onAfterUnbindObserver;
    /**
     * Set a after unbind callback in the texture.
     * This has been kept for backward compatibility and use of onAfterUnbindObservable is recommended.
     */
    set onAfterUnbind(callback: () => void);
    /**
     * An event triggered before rendering the texture
     */
    onBeforeRenderObservable: Observable<number>;
    private _onBeforeRenderObserver;
    /**
     * Set a before render callback in the texture.
     * This has been kept for backward compatibility and use of onBeforeRenderObservable is recommended.
     */
    set onBeforeRender(callback: (faceIndex: number) => void);
    /**
     * An event triggered after rendering the texture
     */
    onAfterRenderObservable: Observable<number>;
    private _onAfterRenderObserver;
    /**
     * Set a after render callback in the texture.
     * This has been kept for backward compatibility and use of onAfterRenderObservable is recommended.
     */
    set onAfterRender(callback: (faceIndex: number) => void);
    /**
     * An event triggered after the texture clear
     */
    onClearObservable: Observable<Engine>;
    private _onClearObserver;
    /**
     * Set a clear callback in the texture.
     * This has been kept for backward compatibility and use of onClearObservable is recommended.
     */
    set onClear(callback: (Engine: Engine) => void);
    /**
     * An event triggered when the texture is resized.
     */
    onResizeObservable: Observable<RenderTargetTexture>;
    /**
     * Define the clear color of the Render Target if it should be different from the scene.
     */
    clearColor: Color4;
    protected _size: TextureSize;
    protected _initialSizeParameter: number | {
        width: number;
        height: number;
    } | {
        ratio: number;
    };
    protected _sizeRatio: Nullable<number>;
    /** @internal */
    _generateMipMaps: boolean;
    /** @internal */
    _cleared: boolean;
    /**
     * Skip the initial clear of the rtt at the beginning of the frame render loop
     */
    skipInitialClear: boolean;
    protected _renderingManager: RenderingManager;
    /** @internal */
    _waitingRenderList?: string[];
    protected _doNotChangeAspectRatio: boolean;
    protected _currentRefreshId: number;
    protected _refreshRate: number;
    protected _textureMatrix: Matrix;
    protected _samples: number;
    protected _renderTargetOptions: RenderTargetCreationOptions;
    private _canRescale;
    protected _renderTarget: Nullable<RenderTargetWrapper>;
    /**
     * Current render pass id of the render target texture. Note it can change over the rendering as there's a separate id for each face of a cube / each layer of an array layer!
     */
    renderPassId: number;
    private _renderPassIds;
    /**
     * Gets the render pass ids used by the render target texture. For a single render target the array length will be 1, for a cube texture it will be 6 and for
     * a 2D texture array it will return an array of ids the size of the 2D texture array
     */
    get renderPassIds(): readonly number[];
    /**
     * Gets the current value of the refreshId counter
     */
    get currentRefreshId(): number;
    /**
     * Sets a specific material to be used to render a mesh/a list of meshes in this render target texture
     * @param mesh mesh or array of meshes
     * @param material material or array of materials to use for this render pass. If undefined is passed, no specific material will be used but the regular material instead (mesh.material). It's possible to provide an array of materials to use a different material for each rendering in the case of a cube texture (6 rendering) and a 2D texture array (as many rendering as the length of the array)
     */
    setMaterialForRendering(mesh: AbstractMesh | AbstractMesh[], material?: Material | Material[]): void;
    private _isCubeData;
    /**
     * Gets render target creation options that were used.
     */
    get renderTargetOptions(): RenderTargetCreationOptions;
    /**
     * Gets the render target wrapper associated with this render target
     */
    get renderTarget(): Nullable<RenderTargetWrapper>;
    protected _onRatioRescale(): void;
    /**
     * Gets or sets the center of the bounding box associated with the texture (when in cube mode)
     * It must define where the camera used to render the texture is set
     */
    boundingBoxPosition: Vector3;
    private _boundingBoxSize;
    /**
     * Gets or sets the size of the bounding box associated with the texture (when in cube mode)
     * When defined, the cubemap will switch to local mode
     * @see https://community.arm.com/graphics/b/blog/posts/reflections-based-on-local-cubemaps-in-unity
     * @example https://www.babylonjs-playground.com/#RNASML
     */
    set boundingBoxSize(value: Vector3);
    get boundingBoxSize(): Vector3;
    /**
     * In case the RTT has been created with a depth texture, get the associated
     * depth texture.
     * Otherwise, return null.
     */
    get depthStencilTexture(): Nullable<InternalTexture>;
    /**
     * Instantiate a render target texture. This is mainly used to render of screen the scene to for instance apply post process
     * or used a shadow, depth texture...
     * @param name The friendly name of the texture
     * @param size The size of the RTT (number if square, or {width: number, height:number} or {ratio:} to define a ratio from the main scene)
     * @param scene The scene the RTT belongs to. Default is the last created scene.
     * @param options The options for creating the render target texture.
     */
    constructor(name: string, size: number | {
        width: number;
        height: number;
        layers?: number;
    } | {
        ratio: number;
    }, scene?: Nullable<Scene>, options?: RenderTargetTextureOptions);
    /**
     * Instantiate a render target texture. This is mainly used to render of screen the scene to for instance apply post process
     * or used a shadow, depth texture...
     * @param name The friendly name of the texture
     * @param size The size of the RTT (number if square, or {width: number, height:number} or {ratio:} to define a ratio from the main scene)
     * @param scene The scene the RTT belongs to. Default is the last created scene
     * @param generateMipMaps True (default: false) if mipmaps need to be generated after render
     * @param doNotChangeAspectRatio True (default) to not change the aspect ratio of the scene in the RTT
     * @param type The type of the buffer in the RTT (byte (default), half float, float...)
     * @param isCube True (default: false) if a cube texture needs to be created
     * @param samplingMode The sampling mode to be used with the render target (Trilinear (default), Linear, Nearest...)
     * @param generateDepthBuffer True (default) to generate a depth buffer
     * @param generateStencilBuffer True (default: false) to generate a stencil buffer
     * @param isMulti True (default: false) if multiple textures need to be created (Draw Buffers)
     * @param format The internal format of the buffer in the RTT (RED, RG, RGB, RGBA (default), ALPHA...)
     * @param delayAllocation True (default: false) if the texture allocation should be delayed
     * @param samples Sample count to use when creating the RTT
     * @param creationFlags specific flags to use when creating the texture (e.g., Constants.TEXTURE_CREATIONFLAG_STORAGE for storage textures)
     * @param noColorAttachment True (default: false) to indicate that no color target should be created. (e.g., if you only want to write to the depth buffer)
     * @param useSRGBBuffer True (default: false) to create a SRGB texture
     */
    constructor(name: string, size: number | {
        width: number;
        height: number;
        layers?: number;
    } | {
        ratio: number;
    }, scene?: Nullable<Scene>, generateMipMaps?: boolean, doNotChangeAspectRatio?: boolean, type?: number, isCube?: boolean, samplingMode?: number, generateDepthBuffer?: boolean, generateStencilBuffer?: boolean, isMulti?: boolean, format?: number, delayAllocation?: boolean, samples?: number, creationFlags?: number, noColorAttachment?: boolean, useSRGBBuffer?: boolean);
    /**
     * Creates a depth stencil texture.
     * This is only available in WebGL 2 or with the depth texture extension available.
     * @param comparisonFunction Specifies the comparison function to set on the texture. If 0 or undefined, the texture is not in comparison mode (default: 0)
     * @param bilinearFiltering Specifies whether or not bilinear filtering is enable on the texture (default: true)
     * @param generateStencil Specifies whether or not a stencil should be allocated in the texture (default: false)
     * @param samples sample count of the depth/stencil texture (default: 1)
     * @param format format of the depth texture (default: Constants.TEXTUREFORMAT_DEPTH32_FLOAT)
     */
    createDepthStencilTexture(comparisonFunction?: number, bilinearFiltering?: boolean, generateStencil?: boolean, samples?: number, format?: number): void;
    private _releaseRenderPassId;
    private _createRenderPassId;
    private _processSizeParameter;
    /**
     * Define the number of samples to use in case of MSAA.
     * It defaults to one meaning no MSAA has been enabled.
     */
    get samples(): number;
    set samples(value: number);
    /**
     * Resets the refresh counter of the texture and start bak from scratch.
     * Could be useful to regenerate the texture if it is setup to render only once.
     */
    resetRefreshCounter(): void;
    /**
     * Define the refresh rate of the texture or the rendering frequency.
     * Use 0 to render just once, 1 to render on every frame, 2 to render every two frames and so on...
     */
    get refreshRate(): number;
    set refreshRate(value: number);
    /**
     * Adds a post process to the render target rendering passes.
     * @param postProcess define the post process to add
     */
    addPostProcess(postProcess: PostProcess): void;
    /**
     * Clear all the post processes attached to the render target
     * @param dispose define if the cleared post processes should also be disposed (false by default)
     */
    clearPostProcesses(dispose?: boolean): void;
    /**
     * Remove one of the post process from the list of attached post processes to the texture
     * @param postProcess define the post process to remove from the list
     */
    removePostProcess(postProcess: PostProcess): void;
    /** @internal */
    _shouldRender(): boolean;
    /**
     * Gets the actual render size of the texture.
     * @returns the width of the render size
     */
    getRenderSize(): number;
    /**
     * Gets the actual render width of the texture.
     * @returns the width of the render size
     */
    getRenderWidth(): number;
    /**
     * Gets the actual render height of the texture.
     * @returns the height of the render size
     */
    getRenderHeight(): number;
    /**
     * Gets the actual number of layers of the texture.
     * @returns the number of layers
     */
    getRenderLayers(): number;
    /**
     * Don't allow this render target texture to rescale. Mainly used to prevent rescaling by the scene optimizer.
     */
    disableRescaling(): void;
    /**
     * Get if the texture can be rescaled or not.
     */
    get canRescale(): boolean;
    /**
     * Resize the texture using a ratio.
     * @param ratio the ratio to apply to the texture size in order to compute the new target size
     */
    scale(ratio: number): void;
    /**
     * Get the texture reflection matrix used to rotate/transform the reflection.
     * @returns the reflection matrix
     */
    getReflectionTextureMatrix(): Matrix;
    /**
     * Resize the texture to a new desired size.
     * Be careful as it will recreate all the data in the new texture.
     * @param size Define the new size. It can be:
     *   - a number for squared texture,
     *   - an object containing { width: number, height: number }
     *   - or an object containing a ratio { ratio: number }
     */
    resize(size: number | {
        width: number;
        height: number;
    } | {
        ratio: number;
    }): void;
    private _defaultRenderListPrepared;
    /**
     * Renders all the objects from the render list into the texture.
     * @param useCameraPostProcess Define if camera post processes should be used during the rendering
     * @param dumpForDebug Define if the rendering result should be dumped (copied) for debugging purpose
     */
    render(useCameraPostProcess?: boolean, dumpForDebug?: boolean): void;
    /**
     * This function will check if the render target texture can be rendered (textures are loaded, shaders are compiled)
     * @returns true if all required resources are ready
     */
    isReadyForRendering(): boolean;
    private _render;
    private _bestReflectionRenderTargetDimension;
    private _prepareRenderingManager;
    /**
     * @internal
     * @param faceIndex face index to bind to if this is a cubetexture
     * @param layer defines the index of the texture to bind in the array
     */
    _bindFrameBuffer(faceIndex?: number, layer?: number): void;
    protected _unbindFrameBuffer(engine: Engine, faceIndex: number): void;
    /**
     * @internal
     */
    _prepareFrame(scene: Scene, faceIndex?: number, layer?: number, useCameraPostProcess?: boolean): void;
    private _renderToTarget;
    /**
     * Overrides the default sort function applied in the rendering group to prepare the meshes.
     * This allowed control for front to back rendering or reversely depending of the special needs.
     *
     * @param renderingGroupId The rendering group id corresponding to its index
     * @param opaqueSortCompareFn The opaque queue comparison function use to sort.
     * @param alphaTestSortCompareFn The alpha test queue comparison function use to sort.
     * @param transparentSortCompareFn The transparent queue comparison function use to sort.
     */
    setRenderingOrder(renderingGroupId: number, opaqueSortCompareFn?: Nullable<(a: SubMesh, b: SubMesh) => number>, alphaTestSortCompareFn?: Nullable<(a: SubMesh, b: SubMesh) => number>, transparentSortCompareFn?: Nullable<(a: SubMesh, b: SubMesh) => number>): void;
    /**
     * Specifies whether or not the stencil and depth buffer are cleared between two rendering groups.
     *
     * @param renderingGroupId The rendering group id corresponding to its index
     * @param autoClearDepthStencil Automatically clears depth and stencil between groups if true.
     */
    setRenderingAutoClearDepthStencil(renderingGroupId: number, autoClearDepthStencil: boolean): void;
    /**
     * Clones the texture.
     * @returns the cloned texture
     */
    clone(): RenderTargetTexture;
    /**
     * Serialize the texture to a JSON representation we can easily use in the respective Parse function.
     * @returns The JSON representation of the texture
     */
    serialize(): any;
    /**
     *  This will remove the attached framebuffer objects. The texture will not be able to be used as render target anymore
     */
    disposeFramebufferObjects(): void;
    /**
     * Release and destroy the underlying lower level texture aka internalTexture.
     */
    releaseInternalTexture(): void;
    /**
     * Dispose the texture and release its associated resources.
     */
    dispose(): void;
    /** @internal */
    _rebuild(): void;
    /**
     * Clear the info related to rendering groups preventing retention point in material dispose.
     */
    freeRenderingGroups(): void;
    /**
     * Gets the number of views the corresponding to the texture (eg. a MultiviewRenderTarget will have > 1)
     * @returns the view count
     */
    getViewCount(): number;
}
export {};

}
declare module "babylonjs/Materials/Textures/texture" {
import { Observable } from "babylonjs/Misc/observable";
import { Nullable } from "babylonjs/types";
import { Matrix } from "babylonjs/Maths/math.vector";
import { BaseTexture } from "babylonjs/Materials/Textures/baseTexture";
import { IInspectable } from "babylonjs/Misc/iInspectable";
import { ThinEngine } from "babylonjs/Engines/thinEngine";
import { InternalTexture } from "babylonjs/Materials/Textures/internalTexture";
import { CubeTexture } from "babylonjs/Materials/Textures/cubeTexture";
import { MirrorTexture } from "babylonjs/Materials/Textures/mirrorTexture";
import { RenderTargetTexture } from "babylonjs/Materials/Textures/renderTargetTexture";
import { Scene } from "babylonjs/scene";
/**
 * Defines the available options when creating a texture
 */
export interface ITextureCreationOptions {
    /** Defines if the texture will require mip maps or not (default: false) */
    noMipmap?: boolean;
    /** Defines if the texture needs to be inverted on the y axis during loading (default: true) */
    invertY?: boolean;
    /** Defines the sampling mode we want for the texture while fetching from it (Texture.NEAREST_SAMPLINGMODE...) (default: Texture.TRILINEAR_SAMPLINGMODE) */
    samplingMode?: number;
    /** Defines a callback triggered when the texture has been loaded (default: null) */
    onLoad?: Nullable<() => void>;
    /** Defines a callback triggered when an error occurred during the loading session (default: null) */
    onError?: Nullable<(message?: string, exception?: any) => void>;
    /** Defines the buffer to load the texture from in case the texture is loaded from a buffer representation (default: null) */
    buffer?: Nullable<string | ArrayBuffer | ArrayBufferView | HTMLImageElement | Blob | ImageBitmap>;
    /** Defines if the buffer we are loading the texture from should be deleted after load (default: false) */
    deleteBuffer?: boolean;
    /** Defines the format of the texture we are trying to load (Engine.TEXTUREFORMAT_RGBA...) (default: ) */
    format?: number;
    /** Defines an optional mime type information (default: undefined) */
    mimeType?: string;
    /** Options to be passed to the loader (default: undefined) */
    loaderOptions?: any;
    /** Specific flags to use when creating the texture (Constants.TEXTURE_CREATIONFLAG_STORAGE for storage textures, for eg) (default: undefined) */
    creationFlags?: number;
    /** Defines if the texture must be loaded in a sRGB GPU buffer (if supported by the GPU) (default: false) */
    useSRGBBuffer?: boolean;
    /** Defines the underlying texture from an already existing one */
    internalTexture?: InternalTexture;
}
/**
 * This represents a texture in babylon. It can be easily loaded from a network, base64 or html input.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/materials_introduction#texture
 */
export class Texture extends BaseTexture {
    /**
     * Gets or sets a general boolean used to indicate that textures containing direct data (buffers) must be saved as part of the serialization process
     */
    static SerializeBuffers: boolean;
    /**
     * Gets or sets a general boolean used to indicate that texture buffers must be saved as part of the serialization process.
     * If no buffer exists, one will be created as base64 string from the internal webgl data.
     */
    static ForceSerializeBuffers: boolean;
    /**
     * This observable will notify when any texture had a loading error
     */
    static OnTextureLoadErrorObservable: Observable<BaseTexture>;
    /** @internal */
    static _SerializeInternalTextureUniqueId: boolean;
    /**
     * @internal
     */
    static _CubeTextureParser: (jsonTexture: any, scene: Scene, rootUrl: string) => CubeTexture;
    /**
     * @internal
     */
    static _CreateMirror: (name: string, renderTargetSize: number, scene: Scene, generateMipMaps: boolean) => MirrorTexture;
    /**
     * @internal
     */
    static _CreateRenderTargetTexture: (name: string, renderTargetSize: number, scene: Scene, generateMipMaps: boolean, creationFlags?: number) => RenderTargetTexture;
    /** nearest is mag = nearest and min = nearest and no mip */
    static readonly NEAREST_SAMPLINGMODE: number;
    /** nearest is mag = nearest and min = nearest and mip = linear */
    static readonly NEAREST_NEAREST_MIPLINEAR: number;
    /** Bilinear is mag = linear and min = linear and no mip */
    static readonly BILINEAR_SAMPLINGMODE: number;
    /** Bilinear is mag = linear and min = linear and mip = nearest */
    static readonly LINEAR_LINEAR_MIPNEAREST: number;
    /** Trilinear is mag = linear and min = linear and mip = linear */
    static readonly TRILINEAR_SAMPLINGMODE: number;
    /** Trilinear is mag = linear and min = linear and mip = linear */
    static readonly LINEAR_LINEAR_MIPLINEAR: number;
    /** mag = nearest and min = nearest and mip = nearest */
    static readonly NEAREST_NEAREST_MIPNEAREST: number;
    /** mag = nearest and min = linear and mip = nearest */
    static readonly NEAREST_LINEAR_MIPNEAREST: number;
    /** mag = nearest and min = linear and mip = linear */
    static readonly NEAREST_LINEAR_MIPLINEAR: number;
    /** mag = nearest and min = linear and mip = none */
    static readonly NEAREST_LINEAR: number;
    /** mag = nearest and min = nearest and mip = none */
    static readonly NEAREST_NEAREST: number;
    /** mag = linear and min = nearest and mip = nearest */
    static readonly LINEAR_NEAREST_MIPNEAREST: number;
    /** mag = linear and min = nearest and mip = linear */
    static readonly LINEAR_NEAREST_MIPLINEAR: number;
    /** mag = linear and min = linear and mip = none */
    static readonly LINEAR_LINEAR: number;
    /** mag = linear and min = nearest and mip = none */
    static readonly LINEAR_NEAREST: number;
    /** Explicit coordinates mode */
    static readonly EXPLICIT_MODE: number;
    /** Spherical coordinates mode */
    static readonly SPHERICAL_MODE: number;
    /** Planar coordinates mode */
    static readonly PLANAR_MODE: number;
    /** Cubic coordinates mode */
    static readonly CUBIC_MODE: number;
    /** Projection coordinates mode */
    static readonly PROJECTION_MODE: number;
    /** Inverse Cubic coordinates mode */
    static readonly SKYBOX_MODE: number;
    /** Inverse Cubic coordinates mode */
    static readonly INVCUBIC_MODE: number;
    /** Equirectangular coordinates mode */
    static readonly EQUIRECTANGULAR_MODE: number;
    /** Equirectangular Fixed coordinates mode */
    static readonly FIXED_EQUIRECTANGULAR_MODE: number;
    /** Equirectangular Fixed Mirrored coordinates mode */
    static readonly FIXED_EQUIRECTANGULAR_MIRRORED_MODE: number;
    /** Texture is not repeating outside of 0..1 UVs */
    static readonly CLAMP_ADDRESSMODE: number;
    /** Texture is repeating outside of 0..1 UVs */
    static readonly WRAP_ADDRESSMODE: number;
    /** Texture is repeating and mirrored */
    static readonly MIRROR_ADDRESSMODE: number;
    /**
     * Gets or sets a boolean which defines if the texture url must be build from the serialized URL instead of just using the name and loading them side by side with the scene file
     */
    static UseSerializedUrlIfAny: boolean;
    /**
     * Define the url of the texture.
     */
    url: Nullable<string>;
    /**
     * Define an offset on the texture to offset the u coordinates of the UVs
     * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/moreMaterials#offsetting
     */
    uOffset: number;
    /**
     * Define an offset on the texture to offset the v coordinates of the UVs
     * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/moreMaterials#offsetting
     */
    vOffset: number;
    /**
     * Define an offset on the texture to scale the u coordinates of the UVs
     * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/moreMaterials#tiling
     */
    uScale: number;
    /**
     * Define an offset on the texture to scale the v coordinates of the UVs
     * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/moreMaterials#tiling
     */
    vScale: number;
    /**
     * Define an offset on the texture to rotate around the u coordinates of the UVs
     * The angle is defined in radians.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/moreMaterials
     */
    uAng: number;
    /**
     * Define an offset on the texture to rotate around the v coordinates of the UVs
     * The angle is defined in radians.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/moreMaterials
     */
    vAng: number;
    /**
     * Define an offset on the texture to rotate around the w coordinates of the UVs (in case of 3d texture)
     * The angle is defined in radians.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/moreMaterials
     */
    wAng: number;
    /**
     * Defines the center of rotation (U)
     */
    uRotationCenter: number;
    /**
     * Defines the center of rotation (V)
     */
    vRotationCenter: number;
    /**
     * Defines the center of rotation (W)
     */
    wRotationCenter: number;
    /**
     * Sets this property to true to avoid deformations when rotating the texture with non-uniform scaling
     */
    homogeneousRotationInUVTransform: boolean;
    /**
     * Are mip maps generated for this texture or not.
     */
    get noMipmap(): boolean;
    /**
     * List of inspectable custom properties (used by the Inspector)
     * @see https://doc.babylonjs.com/toolsAndResources/inspector#extensibility
     */
    inspectableCustomProperties: Nullable<IInspectable[]>;
    private _noMipmap;
    /** @internal */
    _invertY: boolean;
    private _rowGenerationMatrix;
    private _cachedTextureMatrix;
    private _projectionModeMatrix;
    private _t0;
    private _t1;
    private _t2;
    private _cachedUOffset;
    private _cachedVOffset;
    private _cachedUScale;
    private _cachedVScale;
    private _cachedUAng;
    private _cachedVAng;
    private _cachedWAng;
    private _cachedReflectionProjectionMatrixId;
    private _cachedURotationCenter;
    private _cachedVRotationCenter;
    private _cachedWRotationCenter;
    private _cachedHomogeneousRotationInUVTransform;
    private _cachedReflectionTextureMatrix;
    private _cachedReflectionUOffset;
    private _cachedReflectionVOffset;
    private _cachedReflectionUScale;
    private _cachedReflectionVScale;
    private _cachedReflectionCoordinatesMode;
    /** @internal */
    _buffer: Nullable<string | ArrayBuffer | ArrayBufferView | HTMLImageElement | Blob | ImageBitmap>;
    private _deleteBuffer;
    protected _format: Nullable<number>;
    private _delayedOnLoad;
    private _delayedOnError;
    private _mimeType?;
    private _loaderOptions?;
    private _creationFlags?;
    private _useSRGBBuffer?;
    private _forcedExtension?;
    /** Returns the texture mime type if it was defined by a loader (undefined else) */
    get mimeType(): string | undefined;
    /**
     * Observable triggered once the texture has been loaded.
     */
    onLoadObservable: Observable<Texture>;
    protected _isBlocking: boolean;
    /**
     * Is the texture preventing material to render while loading.
     * If false, a default texture will be used instead of the loading one during the preparation step.
     */
    set isBlocking(value: boolean);
    get isBlocking(): boolean;
    /**
     * Gets a boolean indicating if the texture needs to be inverted on the y axis during loading
     */
    get invertY(): boolean;
    /**
     * Instantiates a new texture.
     * This represents a texture in babylon. It can be easily loaded from a network, base64 or html input.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/materials_introduction#texture
     * @param url defines the url of the picture to load as a texture
     * @param sceneOrEngine defines the scene or engine the texture will belong to
     * @param noMipmapOrOptions defines if the texture will require mip maps or not or set of all options to create the texture
     * @param invertY defines if the texture needs to be inverted on the y axis during loading
     * @param samplingMode defines the sampling mode we want for the texture while fetching from it (Texture.NEAREST_SAMPLINGMODE...)
     * @param onLoad defines a callback triggered when the texture has been loaded
     * @param onError defines a callback triggered when an error occurred during the loading session
     * @param buffer defines the buffer to load the texture from in case the texture is loaded from a buffer representation
     * @param deleteBuffer defines if the buffer we are loading the texture from should be deleted after load
     * @param format defines the format of the texture we are trying to load (Engine.TEXTUREFORMAT_RGBA...)
     * @param mimeType defines an optional mime type information
     * @param loaderOptions options to be passed to the loader
     * @param creationFlags specific flags to use when creating the texture (Constants.TEXTURE_CREATIONFLAG_STORAGE for storage textures, for eg)
     * @param forcedExtension defines the extension to use to pick the right loader
     */
    constructor(url: Nullable<string>, sceneOrEngine?: Nullable<Scene | ThinEngine>, noMipmapOrOptions?: boolean | ITextureCreationOptions, invertY?: boolean, samplingMode?: number, onLoad?: Nullable<() => void>, onError?: Nullable<(message?: string, exception?: any) => void>, buffer?: Nullable<string | ArrayBuffer | ArrayBufferView | HTMLImageElement | Blob | ImageBitmap>, deleteBuffer?: boolean, format?: number, mimeType?: string, loaderOptions?: any, creationFlags?: number, forcedExtension?: string);
    /**
     * Update the url (and optional buffer) of this texture if url was null during construction.
     * @param url the url of the texture
     * @param buffer the buffer of the texture (defaults to null)
     * @param onLoad callback called when the texture is loaded  (defaults to null)
     * @param forcedExtension defines the extension to use to pick the right loader
     */
    updateURL(url: string, buffer?: Nullable<string | ArrayBuffer | ArrayBufferView | HTMLImageElement | Blob | ImageBitmap>, onLoad?: () => void, forcedExtension?: string): void;
    /**
     * Finish the loading sequence of a texture flagged as delayed load.
     * @internal
     */
    delayLoad(): void;
    private _prepareRowForTextureGeneration;
    /**
     * Checks if the texture has the same transform matrix than another texture
     * @param texture texture to check against
     * @returns true if the transforms are the same, else false
     */
    checkTransformsAreIdentical(texture: Nullable<Texture>): boolean;
    /**
     * Get the current texture matrix which includes the requested offsetting, tiling and rotation components.
     * @param uBase
     * @returns the transform matrix of the texture.
     */
    getTextureMatrix(uBase?: number): Matrix;
    /**
     * Get the current matrix used to apply reflection. This is useful to rotate an environment texture for instance.
     * @returns The reflection texture transform
     */
    getReflectionTextureMatrix(): Matrix;
    /**
     * Clones the texture.
     * @returns the cloned texture
     */
    clone(): Texture;
    /**
     * Serialize the texture to a JSON representation we can easily use in the respective Parse function.
     * @returns The JSON representation of the texture
     */
    serialize(): any;
    /**
     * Get the current class name of the texture useful for serialization or dynamic coding.
     * @returns "Texture"
     */
    getClassName(): string;
    /**
     * Dispose the texture and release its associated resources.
     */
    dispose(): void;
    /**
     * Parse the JSON representation of a texture in order to recreate the texture in the given scene.
     * @param parsedTexture Define the JSON representation of the texture
     * @param scene Define the scene the parsed texture should be instantiated in
     * @param rootUrl Define the root url of the parsing sequence in the case of relative dependencies
     * @returns The parsed texture if successful
     */
    static Parse(parsedTexture: any, scene: Scene, rootUrl: string): Nullable<BaseTexture>;
    /**
     * Creates a texture from its base 64 representation.
     * @param data Define the base64 payload without the data: prefix
     * @param name Define the name of the texture in the scene useful fo caching purpose for instance
     * @param scene Define the scene the texture should belong to
     * @param noMipmapOrOptions defines if the texture will require mip maps or not or set of all options to create the texture
     * @param invertY define if the texture needs to be inverted on the y axis during loading
     * @param samplingMode define the sampling mode we want for the texture while fetching from it (Texture.NEAREST_SAMPLINGMODE...)
     * @param onLoad define a callback triggered when the texture has been loaded
     * @param onError define a callback triggered when an error occurred during the loading session
     * @param format define the format of the texture we are trying to load (Engine.TEXTUREFORMAT_RGBA...)
     * @param creationFlags specific flags to use when creating the texture (Constants.TEXTURE_CREATIONFLAG_STORAGE for storage textures, for eg)
     * @returns the created texture
     */
    static CreateFromBase64String(data: string, name: string, scene: Scene, noMipmapOrOptions?: boolean | ITextureCreationOptions, invertY?: boolean, samplingMode?: number, onLoad?: Nullable<() => void>, onError?: Nullable<() => void>, format?: number, creationFlags?: number): Texture;
    /**
     * Creates a texture from its data: representation. (data: will be added in case only the payload has been passed in)
     * @param name Define the name of the texture in the scene useful fo caching purpose for instance
     * @param buffer define the buffer to load the texture from in case the texture is loaded from a buffer representation
     * @param scene Define the scene the texture should belong to
     * @param deleteBuffer define if the buffer we are loading the texture from should be deleted after load
     * @param noMipmapOrOptions defines if the texture will require mip maps or not or set of all options to create the texture
     * @param invertY define if the texture needs to be inverted on the y axis during loading
     * @param samplingMode define the sampling mode we want for the texture while fetching from it (Texture.NEAREST_SAMPLINGMODE...)
     * @param onLoad define a callback triggered when the texture has been loaded
     * @param onError define a callback triggered when an error occurred during the loading session
     * @param format define the format of the texture we are trying to load (Engine.TEXTUREFORMAT_RGBA...)
     * @param creationFlags specific flags to use when creating the texture (Constants.TEXTURE_CREATIONFLAG_STORAGE for storage textures, for eg)
     * @returns the created texture
     */
    static LoadFromDataString(name: string, buffer: any, scene: Scene, deleteBuffer?: boolean, noMipmapOrOptions?: boolean | ITextureCreationOptions, invertY?: boolean, samplingMode?: number, onLoad?: Nullable<() => void>, onError?: Nullable<(message?: string, exception?: any) => void>, format?: number, creationFlags?: number): Texture;
}
export {};

}
declare module "babylonjs/Materials/Textures/textureCreationOptions" {
import { InternalTexture } from "babylonjs/Materials/Textures/internalTexture";
/**
 * Define options used to create an internal texture
 */
export interface InternalTextureCreationOptions {
    /**
     * Specifies if mipmaps must be created. If undefined, the value from generateMipMaps is taken instead
     */
    createMipMaps?: boolean;
    /**
     * Specifies if mipmaps must be generated
     */
    generateMipMaps?: boolean;
    /** Defines texture type (int by default) */
    type?: number;
    /** Defines sampling mode (trilinear by default) */
    samplingMode?: number;
    /** Defines format (RGBA by default) */
    format?: number;
    /** Defines sample count (1 by default) */
    samples?: number;
    /** Texture creation flags */
    creationFlags?: number;
    /** Creates the RTT in sRGB space */
    useSRGBBuffer?: boolean;
}
/**
 * Define options used to create a render target texture
 */
export interface RenderTargetCreationOptions extends InternalTextureCreationOptions {
    /** Specifies whether or not a depth should be allocated in the texture (true by default) */
    generateDepthBuffer?: boolean;
    /** Specifies whether or not a stencil should be allocated in the texture (false by default)*/
    generateStencilBuffer?: boolean;
    /** Specifies that no color target should be bound to the render target (useful if you only want to write to the depth buffer, for eg) */
    noColorAttachment?: boolean;
    /** Specifies the internal texture to use directly instead of creating one (ignores `noColorAttachment` flag when set) **/
    colorAttachment?: InternalTexture;
}
/**
 * Define options used to create a depth texture
 */
export interface DepthTextureCreationOptions {
    /** Specifies whether or not a stencil should be allocated in the texture */
    generateStencil?: boolean;
    /** Specifies whether or not bilinear filtering is enable on the texture */
    bilinearFiltering?: boolean;
    /** Specifies the comparison function to set on the texture. If 0 or undefined, the texture is not in comparison mode */
    comparisonFunction?: number;
    /** Specifies if the created texture is a cube texture */
    isCube?: boolean;
    /** Specifies the sample count of the depth/stencil texture texture */
    samples?: number;
    /** Specifies the depth texture format to use */
    depthTextureFormat?: number;
}
/**
 * Type used to define a texture size (either with a number or with a rect width and height)
 */
export type TextureSize = number | {
    width: number;
    height: number;
    layers?: number;
};

}
declare module "babylonjs/Materials/Textures/textureSampler" {
import { Nullable } from "babylonjs/types";
/**
 * Class used to store a texture sampler data
 */
export class TextureSampler {
    /**
     * Gets the sampling mode of the texture
     */
    samplingMode: number;
    /**
     * | Value | Type               | Description |
     * | ----- | ------------------ | ----------- |
     * | 0     | CLAMP_ADDRESSMODE  |             |
     * | 1     | WRAP_ADDRESSMODE   |             |
     * | 2     | MIRROR_ADDRESSMODE |             |
     */
    get wrapU(): Nullable<number>;
    set wrapU(value: Nullable<number>);
    /**
     * | Value | Type               | Description |
     * | ----- | ------------------ | ----------- |
     * | 0     | CLAMP_ADDRESSMODE  |             |
     * | 1     | WRAP_ADDRESSMODE   |             |
     * | 2     | MIRROR_ADDRESSMODE |             |
     */
    get wrapV(): Nullable<number>;
    set wrapV(value: Nullable<number>);
    /**
     * | Value | Type               | Description |
     * | ----- | ------------------ | ----------- |
     * | 0     | CLAMP_ADDRESSMODE  |             |
     * | 1     | WRAP_ADDRESSMODE   |             |
     * | 2     | MIRROR_ADDRESSMODE |             |
     */
    get wrapR(): Nullable<number>;
    set wrapR(value: Nullable<number>);
    /**
     * With compliant hardware and browser (supporting anisotropic filtering)
     * this defines the level of anisotropic filtering in the texture.
     * The higher the better but the slower.
     */
    get anisotropicFilteringLevel(): Nullable<number>;
    set anisotropicFilteringLevel(value: Nullable<number>);
    /**
     * Gets or sets the comparison function (Constants.LESS, Constants.EQUAL, etc). Set 0 to not use a comparison function
     */
    get comparisonFunction(): number;
    set comparisonFunction(value: number);
    private _useMipMaps;
    /**
     * Indicates to use the mip maps (if available on the texture).
     * Thanks to this flag, you can instruct the sampler to not sample the mipmaps even if they exist (and if the sampling mode is set to a value that normally samples the mipmaps!)
     */
    get useMipMaps(): boolean;
    set useMipMaps(value: boolean);
    /** @internal */
    _cachedWrapU: Nullable<number>;
    /** @internal */
    _cachedWrapV: Nullable<number>;
    /** @internal */
    _cachedWrapR: Nullable<number>;
    /** @internal */
    _cachedAnisotropicFilteringLevel: Nullable<number>;
    /** @internal */
    _comparisonFunction: number;
    /**
     * Creates a Sampler instance
     */
    constructor();
    /**
     * Sets all the parameters of the sampler
     * @param wrapU u address mode (default: TEXTURE_WRAP_ADDRESSMODE)
     * @param wrapV v address mode (default: TEXTURE_WRAP_ADDRESSMODE)
     * @param wrapR r address mode (default: TEXTURE_WRAP_ADDRESSMODE)
     * @param anisotropicFilteringLevel anisotropic level (default: 1)
     * @param samplingMode sampling mode (default: Constants.TEXTURE_BILINEAR_SAMPLINGMODE)
     * @param comparisonFunction comparison function (default: 0 - no comparison function)
     * @returns the current sampler instance
     */
    setParameters(wrapU?: number, wrapV?: number, wrapR?: number, anisotropicFilteringLevel?: number, samplingMode?: number, comparisonFunction?: number): TextureSampler;
    /**
     * Compares this sampler with another one
     * @param other sampler to compare with
     * @returns true if the samplers have the same parametres, else false
     */
    compareSampler(other: TextureSampler): boolean;
}

}
declare module "babylonjs/Materials/Textures/thinRenderTargetTexture" {
import { Nullable } from "babylonjs/types";
import { InternalTexture } from "babylonjs/Materials/Textures/internalTexture";
import { ThinEngine } from "babylonjs/Engines/thinEngine";
import { IRenderTargetTexture, RenderTargetWrapper } from "babylonjs/Engines/renderTargetWrapper";
import { ThinTexture } from "babylonjs/Materials/Textures/thinTexture";
import { TextureSize, RenderTargetCreationOptions } from "babylonjs/Materials/Textures/textureCreationOptions";
/**
 * This is a tiny helper class to wrap a RenderTargetWrapper in a texture
 * usable as the input of an effect.
 */
export class ThinRenderTargetTexture extends ThinTexture implements IRenderTargetTexture {
    private readonly _renderTargetOptions;
    private _renderTarget;
    private _size;
    /**
     * Gets the render target wrapper associated with this render target
     */
    get renderTarget(): Nullable<RenderTargetWrapper>;
    /**
     * Instantiates a new ThinRenderTargetTexture.
     * Tiny helper class to wrap a RenderTargetWrapper in a texture.
     * This can be used as an internal texture wrapper in ThinEngine to benefit from the cache and to hold on the associated RTT
     * @param engine Define the internalTexture to wrap
     * @param size Define the size of the RTT to create
     * @param options Define rendertarget options
     */
    constructor(engine: ThinEngine, size: TextureSize, options: RenderTargetCreationOptions);
    /**
     * Resize the texture to a new desired size.
     * Be careful as it will recreate all the data in the new texture.
     * @param size Define the new size. It can be:
     *   - a number for squared texture,
     *   - an object containing { width: number, height: number }
     */
    resize(size: TextureSize): void;
    /**
     * Get the underlying lower level texture from Babylon.
     * @returns the internal texture
     */
    getInternalTexture(): Nullable<InternalTexture>;
    /**
     * Get the class name of the texture.
     * @returns "ThinRenderTargetTexture"
     */
    getClassName(): string;
    /**
     * Dispose the texture and release its associated resources.
     * @param disposeOnlyFramebuffers
     */
    dispose(disposeOnlyFramebuffers?: boolean): void;
}

}
declare module "babylonjs/Materials/Textures/thinTexture" {
import { Nullable } from "babylonjs/types";
import { InternalTexture } from "babylonjs/Materials/Textures/internalTexture";
import { ISize } from "babylonjs/Maths/math.size";
import { ThinEngine } from "babylonjs/Engines/thinEngine";
/**
 * Base class of all the textures in babylon.
 * It groups all the common properties required to work with Thin Engine.
 */
export class ThinTexture {
    protected _wrapU: number;
    /**
     * | Value | Type               | Description |
     * | ----- | ------------------ | ----------- |
     * | 0     | CLAMP_ADDRESSMODE  |             |
     * | 1     | WRAP_ADDRESSMODE   |             |
     * | 2     | MIRROR_ADDRESSMODE |             |
     */
    get wrapU(): number;
    set wrapU(value: number);
    protected _wrapV: number;
    /**
     * | Value | Type               | Description |
     * | ----- | ------------------ | ----------- |
     * | 0     | CLAMP_ADDRESSMODE  |             |
     * | 1     | WRAP_ADDRESSMODE   |             |
     * | 2     | MIRROR_ADDRESSMODE |             |
     */
    get wrapV(): number;
    set wrapV(value: number);
    /**
     * | Value | Type               | Description |
     * | ----- | ------------------ | ----------- |
     * | 0     | CLAMP_ADDRESSMODE  |             |
     * | 1     | WRAP_ADDRESSMODE   |             |
     * | 2     | MIRROR_ADDRESSMODE |             |
     */
    wrapR: number;
    /**
     * With compliant hardware and browser (supporting anisotropic filtering)
     * this defines the level of anisotropic filtering in the texture.
     * The higher the better but the slower. This defaults to 4 as it seems to be the best tradeoff.
     */
    anisotropicFilteringLevel: number;
    /**
     * Define the current state of the loading sequence when in delayed load mode.
     */
    delayLoadState: number;
    /**
     * How a texture is mapped.
     * Unused in thin texture mode.
     */
    get coordinatesMode(): number;
    /**
     * Define if the texture is a cube texture or if false a 2d texture.
     */
    get isCube(): boolean;
    protected set isCube(value: boolean);
    /**
     * Define if the texture is a 3d texture (webgl 2) or if false a 2d texture.
     */
    get is3D(): boolean;
    protected set is3D(value: boolean);
    /**
     * Define if the texture is a 2d array texture (webgl 2) or if false a 2d texture.
     */
    get is2DArray(): boolean;
    protected set is2DArray(value: boolean);
    /**
     * Get the class name of the texture.
     * @returns "ThinTexture"
     */
    getClassName(): string;
    /** @internal */
    _texture: Nullable<InternalTexture>;
    protected _engine: Nullable<ThinEngine>;
    private _cachedSize;
    private _cachedBaseSize;
    /**
     * Instantiates a new ThinTexture.
     * Base class of all the textures in babylon.
     * This can be used as an internal texture wrapper in ThinEngine to benefit from the cache
     * @param internalTexture Define the internalTexture to wrap
     */
    constructor(internalTexture: Nullable<InternalTexture>);
    /**
     * Get if the texture is ready to be used (downloaded, converted, mip mapped...).
     * @returns true if fully ready
     */
    isReady(): boolean;
    /**
     * Triggers the load sequence in delayed load mode.
     */
    delayLoad(): void;
    /**
     * Get the underlying lower level texture from Babylon.
     * @returns the internal texture
     */
    getInternalTexture(): Nullable<InternalTexture>;
    /**
     * Get the size of the texture.
     * @returns the texture size.
     */
    getSize(): ISize;
    /**
     * Get the base size of the texture.
     * It can be different from the size if the texture has been resized for POT for instance
     * @returns the base size
     */
    getBaseSize(): ISize;
    /** @internal */
    protected _initialSamplingMode: number;
    /**
     * Get the current sampling mode associated with the texture.
     */
    get samplingMode(): number;
    /**
     * Update the sampling mode of the texture.
     * Default is Trilinear mode.
     *
     * | Value | Type               | Description |
     * | ----- | ------------------ | ----------- |
     * | 1     | NEAREST_SAMPLINGMODE or NEAREST_NEAREST_MIPLINEAR  | Nearest is: mag = nearest, min = nearest, mip = linear |
     * | 2     | BILINEAR_SAMPLINGMODE or LINEAR_LINEAR_MIPNEAREST | Bilinear is: mag = linear, min = linear, mip = nearest |
     * | 3     | TRILINEAR_SAMPLINGMODE or LINEAR_LINEAR_MIPLINEAR | Trilinear is: mag = linear, min = linear, mip = linear |
     * | 4     | NEAREST_NEAREST_MIPNEAREST |             |
     * | 5    | NEAREST_LINEAR_MIPNEAREST |             |
     * | 6    | NEAREST_LINEAR_MIPLINEAR |             |
     * | 7    | NEAREST_LINEAR |             |
     * | 8    | NEAREST_NEAREST |             |
     * | 9   | LINEAR_NEAREST_MIPNEAREST |             |
     * | 10   | LINEAR_NEAREST_MIPLINEAR |             |
     * | 11   | LINEAR_LINEAR |             |
     * | 12   | LINEAR_NEAREST |             |
     *
     *    > _mag_: magnification filter (close to the viewer)
     *    > _min_: minification filter (far from the viewer)
     *    > _mip_: filter used between mip map levels
     *@param samplingMode Define the new sampling mode of the texture
     */
    updateSamplingMode(samplingMode: number): void;
    /**
     * Release and destroy the underlying lower level texture aka internalTexture.
     */
    releaseInternalTexture(): void;
    /**
     * Dispose the texture and release its associated resources.
     */
    dispose(): void;
}

}
declare module "babylonjs/Materials/Textures/videoTexture" {
import { Observable } from "babylonjs/Misc/observable";
import { Nullable } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
import { Texture } from "babylonjs/Materials/Textures/texture";
import "babylonjs/Engines/Extensions/engine.videoTexture";
import "babylonjs/Engines/Extensions/engine.dynamicTexture";
/**
 * Settings for finer control over video usage
 */
export interface VideoTextureSettings {
    /**
     * Applies `autoplay` to video, if specified
     */
    autoPlay?: boolean;
    /**
     * Applies `muted` to video, if specified
     */
    muted?: boolean;
    /**
     * Applies `loop` to video, if specified
     */
    loop?: boolean;
    /**
     * Automatically updates internal texture from video at every frame in the render loop
     */
    autoUpdateTexture: boolean;
    /**
     * Image src displayed during the video loading or until the user interacts with the video.
     */
    poster?: string;
    /**
     * Defines the associated texture format.
     */
    format?: number;
    /**
     * Notify babylon to not modify any video settings and not control the video's playback.
     * Set this to true if you are controlling the way the video is being played, stopped and paused.
     */
    independentVideoSource?: boolean;
}
/**
 * If you want to display a video in your scene, this is the special texture for that.
 * This special texture works similar to other textures, with the exception of a few parameters.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/videoTexture
 */
export class VideoTexture extends Texture {
    /**
     * Tells whether textures will be updated automatically or user is required to call `updateTexture` manually
     */
    readonly autoUpdateTexture: boolean;
    /**
     * The video instance used by the texture internally
     */
    readonly video: HTMLVideoElement;
    private _externalTexture;
    private _onUserActionRequestedObservable;
    /**
     * Event triggered when a dom action is required by the user to play the video.
     * This happens due to recent changes in browser policies preventing video to auto start.
     */
    get onUserActionRequestedObservable(): Observable<Texture>;
    private _generateMipMaps;
    private _stillImageCaptured;
    private _displayingPosterTexture;
    private _settings;
    private _createInternalTextureOnEvent;
    private _frameId;
    private _currentSrc;
    private _onError?;
    private _errorFound;
    private _processError;
    private _handlePlay;
    /**
     * Creates a video texture.
     * If you want to display a video in your scene, this is the special texture for that.
     * This special texture works similar to other textures, with the exception of a few parameters.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/videoTexture
     * @param name optional name, will detect from video source, if not defined
     * @param src can be used to provide an url, array of urls or an already setup HTML video element.
     * @param scene is obviously the current scene.
     * @param generateMipMaps can be used to turn on mipmaps (Can be expensive for videoTextures because they are often updated).
     * @param invertY is false by default but can be used to invert video on Y axis
     * @param samplingMode controls the sampling method and is set to TRILINEAR_SAMPLINGMODE by default
     * @param settings allows finer control over video usage
     * @param onError defines a callback triggered when an error occurred during the loading session
     * @param format defines the texture format to use (Engine.TEXTUREFORMAT_RGBA by default)
     */
    constructor(name: Nullable<string>, src: string | string[] | HTMLVideoElement, scene: Nullable<Scene>, generateMipMaps?: boolean, invertY?: boolean, samplingMode?: number, settings?: Partial<VideoTextureSettings>, onError?: Nullable<(message?: string, exception?: any) => void>, format?: number);
    /**
     * Get the current class name of the video texture useful for serialization or dynamic coding.
     * @returns "VideoTexture"
     */
    getClassName(): string;
    private _getName;
    private _getVideo;
    private _resizeInternalTexture;
    private _createInternalTexture;
    private _reset;
    /**
     * @internal Internal method to initiate `update`.
     */
    _rebuild(): void;
    /**
     * Update Texture in the `auto` mode. Does not do anything if `settings.autoUpdateTexture` is false.
     */
    update(): void;
    /**
     * Update Texture in `manual` mode. Does not do anything if not visible or paused.
     * @param isVisible Visibility state, detected by user using `scene.getActiveMeshes()` or otherwise.
     */
    updateTexture(isVisible: boolean): void;
    protected _updateInternalTexture: () => void;
    /**
     * Change video content. Changing video instance or setting multiple urls (as in constructor) is not supported.
     * @param url New url.
     */
    updateURL(url: string): void;
    /**
     * Clones the texture.
     * @returns the cloned texture
     */
    clone(): VideoTexture;
    /**
     * Dispose the texture and release its associated resources.
     */
    dispose(): void;
    /**
     * Creates a video texture straight from a stream.
     * @param scene Define the scene the texture should be created in
     * @param stream Define the stream the texture should be created from
     * @param constraints video constraints
     * @param invertY Defines if the video should be stored with invert Y set to true (true by default)
     * @returns The created video texture as a promise
     */
    static CreateFromStreamAsync(scene: Scene, stream: MediaStream, constraints: any, invertY?: boolean): Promise<VideoTexture>;
    /**
     * Creates a video texture straight from your WebCam video feed.
     * @param scene Define the scene the texture should be created in
     * @param constraints Define the constraints to use to create the web cam feed from WebRTC
     * @param audioConstaints Define the audio constraints to use to create the web cam feed from WebRTC
     * @param invertY Defines if the video should be stored with invert Y set to true (true by default)
     * @returns The created video texture as a promise
     */
    static CreateFromWebCamAsync(scene: Scene, constraints: {
        minWidth: number;
        maxWidth: number;
        minHeight: number;
        maxHeight: number;
        deviceId: string;
    } & MediaTrackConstraints, audioConstaints?: boolean | MediaTrackConstraints, invertY?: boolean): Promise<VideoTexture>;
    /**
     * Creates a video texture straight from your WebCam video feed.
     * @param scene Defines the scene the texture should be created in
     * @param onReady Defines a callback to triggered once the texture will be ready
     * @param constraints Defines the constraints to use to create the web cam feed from WebRTC
     * @param audioConstaints Defines the audio constraints to use to create the web cam feed from WebRTC
     * @param invertY Defines if the video should be stored with invert Y set to true (true by default)
     */
    static CreateFromWebCam(scene: Scene, onReady: (videoTexture: VideoTexture) => void, constraints: {
        minWidth: number;
        maxWidth: number;
        minHeight: number;
        maxHeight: number;
        deviceId: string;
    } & MediaTrackConstraints, audioConstaints?: boolean | MediaTrackConstraints, invertY?: boolean): void;
}

}
declare module "babylonjs/Materials/uniformBuffer" {
import { Nullable, FloatArray } from "babylonjs/types";
import { IMatrixLike, IVector3Like, IVector4Like, IColor3Like, IColor4Like } from "babylonjs/Maths/math.like";
import { Effect } from "babylonjs/Materials/effect";
import { ThinTexture } from "babylonjs/Materials/Textures/thinTexture";
import { DataBuffer } from "babylonjs/Buffers/dataBuffer";
import { ThinEngine } from "babylonjs/Engines/thinEngine";
import "babylonjs/Engines/Extensions/engine.uniformBuffer";
/**
 * Uniform buffer objects.
 *
 * Handles blocks of uniform on the GPU.
 *
 * If WebGL 2 is not available, this class falls back on traditional setUniformXXX calls.
 *
 * For more information, please refer to :
 * https://www.khronos.org/opengl/wiki/Uniform_Buffer_Object
 */
export class UniformBuffer {
    /** @internal */
    static _UpdatedUbosInFrame: {
        [name: string]: number;
    };
    private _engine;
    private _buffer;
    private _buffers;
    private _bufferIndex;
    private _createBufferOnWrite;
    private _data;
    private _bufferData;
    private _dynamic?;
    private _uniformLocations;
    private _uniformSizes;
    private _uniformArraySizes;
    private _uniformLocationPointer;
    private _needSync;
    private _noUBO;
    private _currentEffect;
    private _currentEffectName;
    private _name;
    private _currentFrameId;
    private static _MAX_UNIFORM_SIZE;
    private static _TempBuffer;
    private static _TempBufferInt32View;
    private static _TempBufferUInt32View;
    /**
     * Lambda to Update a 3x3 Matrix in a uniform buffer.
     * This is dynamic to allow compat with webgl 1 and 2.
     * You will need to pass the name of the uniform as well as the value.
     */
    updateMatrix3x3: (name: string, matrix: Float32Array) => void;
    /**
     * Lambda to Update a 2x2 Matrix in a uniform buffer.
     * This is dynamic to allow compat with webgl 1 and 2.
     * You will need to pass the name of the uniform as well as the value.
     */
    updateMatrix2x2: (name: string, matrix: Float32Array) => void;
    /**
     * Lambda to Update a single float in a uniform buffer.
     * This is dynamic to allow compat with webgl 1 and 2.
     * You will need to pass the name of the uniform as well as the value.
     */
    updateFloat: (name: string, x: number) => void;
    /**
     * Lambda to Update a vec2 of float in a uniform buffer.
     * This is dynamic to allow compat with webgl 1 and 2.
     * You will need to pass the name of the uniform as well as the value.
     */
    updateFloat2: (name: string, x: number, y: number, suffix?: string) => void;
    /**
     * Lambda to Update a vec3 of float in a uniform buffer.
     * This is dynamic to allow compat with webgl 1 and 2.
     * You will need to pass the name of the uniform as well as the value.
     */
    updateFloat3: (name: string, x: number, y: number, z: number, suffix?: string) => void;
    /**
     * Lambda to Update a vec4 of float in a uniform buffer.
     * This is dynamic to allow compat with webgl 1 and 2.
     * You will need to pass the name of the uniform as well as the value.
     */
    updateFloat4: (name: string, x: number, y: number, z: number, w: number, suffix?: string) => void;
    /**
     * Lambda to Update an array of float in a uniform buffer.
     * This is dynamic to allow compat with webgl 1 and 2.
     * You will need to pass the name of the uniform as well as the value.
     */
    updateFloatArray: (name: string, array: Float32Array) => void;
    /**
     * Lambda to Update an array of number in a uniform buffer.
     * This is dynamic to allow compat with webgl 1 and 2.
     * You will need to pass the name of the uniform as well as the value.
     */
    updateArray: (name: string, array: number[]) => void;
    /**
     * Lambda to Update an array of number in a uniform buffer.
     * This is dynamic to allow compat with webgl 1 and 2.
     * You will need to pass the name of the uniform as well as the value.
     */
    updateIntArray: (name: string, array: Int32Array) => void;
    /**
     * Lambda to Update an array of number in a uniform buffer.
     * This is dynamic to allow compat with webgl 1 and 2.
     * You will need to pass the name of the uniform as well as the value.
     */
    updateUIntArray: (name: string, array: Uint32Array) => void;
    /**
     * Lambda to Update a 4x4 Matrix in a uniform buffer.
     * This is dynamic to allow compat with webgl 1 and 2.
     * You will need to pass the name of the uniform as well as the value.
     */
    updateMatrix: (name: string, mat: IMatrixLike) => void;
    /**
     * Lambda to Update an array of 4x4 Matrix in a uniform buffer.
     * This is dynamic to allow compat with webgl 1 and 2.
     * You will need to pass the name of the uniform as well as the value.
     */
    updateMatrices: (name: string, mat: Float32Array) => void;
    /**
     * Lambda to Update vec3 of float from a Vector in a uniform buffer.
     * This is dynamic to allow compat with webgl 1 and 2.
     * You will need to pass the name of the uniform as well as the value.
     */
    updateVector3: (name: string, vector: IVector3Like) => void;
    /**
     * Lambda to Update vec4 of float from a Vector in a uniform buffer.
     * This is dynamic to allow compat with webgl 1 and 2.
     * You will need to pass the name of the uniform as well as the value.
     */
    updateVector4: (name: string, vector: IVector4Like) => void;
    /**
     * Lambda to Update vec3 of float from a Color in a uniform buffer.
     * This is dynamic to allow compat with webgl 1 and 2.
     * You will need to pass the name of the uniform as well as the value.
     */
    updateColor3: (name: string, color: IColor3Like, suffix?: string) => void;
    /**
     * Lambda to Update vec4 of float from a Color in a uniform buffer.
     * This is dynamic to allow compat with webgl 1 and 2.
     * You will need to pass the name of the uniform as well as the value.
     */
    updateColor4: (name: string, color: IColor3Like, alpha: number, suffix?: string) => void;
    /**
     * Lambda to Update vec4 of float from a Color in a uniform buffer.
     * This is dynamic to allow compat with webgl 1 and 2.
     * You will need to pass the name of the uniform as well as the value.
     */
    updateDirectColor4: (name: string, color: IColor4Like, suffix?: string) => void;
    /**
     * Lambda to Update a int a uniform buffer.
     * This is dynamic to allow compat with webgl 1 and 2.
     * You will need to pass the name of the uniform as well as the value.
     */
    updateInt: (name: string, x: number, suffix?: string) => void;
    /**
     * Lambda to Update a vec2 of int in a uniform buffer.
     * This is dynamic to allow compat with webgl 1 and 2.
     * You will need to pass the name of the uniform as well as the value.
     */
    updateInt2: (name: string, x: number, y: number, suffix?: string) => void;
    /**
     * Lambda to Update a vec3 of int in a uniform buffer.
     * This is dynamic to allow compat with webgl 1 and 2.
     * You will need to pass the name of the uniform as well as the value.
     */
    updateInt3: (name: string, x: number, y: number, z: number, suffix?: string) => void;
    /**
     * Lambda to Update a vec4 of int in a uniform buffer.
     * This is dynamic to allow compat with webgl 1 and 2.
     * You will need to pass the name of the uniform as well as the value.
     */
    updateInt4: (name: string, x: number, y: number, z: number, w: number, suffix?: string) => void;
    /**
     * Lambda to Update a unsigned int a uniform buffer.
     * This is dynamic to allow compat with webgl 1 and 2.
     * You will need to pass the name of the uniform as well as the value.
     */
    updateUInt: (name: string, x: number, suffix?: string) => void;
    /**
     * Lambda to Update a vec2 of unsigned int in a uniform buffer.
     * This is dynamic to allow compat with webgl 1 and 2.
     * You will need to pass the name of the uniform as well as the value.
     */
    updateUInt2: (name: string, x: number, y: number, suffix?: string) => void;
    /**
     * Lambda to Update a vec3 of unsigned int in a uniform buffer.
     * This is dynamic to allow compat with webgl 1 and 2.
     * You will need to pass the name of the uniform as well as the value.
     */
    updateUInt3: (name: string, x: number, y: number, z: number, suffix?: string) => void;
    /**
     * Lambda to Update a vec4 of unsigned int in a uniform buffer.
     * This is dynamic to allow compat with webgl 1 and 2.
     * You will need to pass the name of the uniform as well as the value.
     */
    updateUInt4: (name: string, x: number, y: number, z: number, w: number, suffix?: string) => void;
    /**
     * Instantiates a new Uniform buffer objects.
     *
     * Handles blocks of uniform on the GPU.
     *
     * If WebGL 2 is not available, this class falls back on traditional setUniformXXX calls.
     *
     * For more information, please refer to :
     * @see https://www.khronos.org/opengl/wiki/Uniform_Buffer_Object
     * @param engine Define the engine the buffer is associated with
     * @param data Define the data contained in the buffer
     * @param dynamic Define if the buffer is updatable
     * @param name to assign to the buffer (debugging purpose)
     * @param forceNoUniformBuffer define that this object must not rely on UBO objects
     */
    constructor(engine: ThinEngine, data?: number[], dynamic?: boolean, name?: string, forceNoUniformBuffer?: boolean);
    /**
     * Indicates if the buffer is using the WebGL2 UBO implementation,
     * or just falling back on setUniformXXX calls.
     */
    get useUbo(): boolean;
    /**
     * Indicates if the WebGL underlying uniform buffer is in sync
     * with the javascript cache data.
     */
    get isSync(): boolean;
    /**
     * Indicates if the WebGL underlying uniform buffer is dynamic.
     * Also, a dynamic UniformBuffer will disable cache verification and always
     * update the underlying WebGL uniform buffer to the GPU.
     * @returns if Dynamic, otherwise false
     */
    isDynamic(): boolean;
    /**
     * The data cache on JS side.
     * @returns the underlying data as a float array
     */
    getData(): Float32Array;
    /**
     * The underlying WebGL Uniform buffer.
     * @returns the webgl buffer
     */
    getBuffer(): Nullable<DataBuffer>;
    /**
     * std140 layout specifies how to align data within an UBO structure.
     * See https://khronos.org/registry/OpenGL/specs/gl/glspec45.core.pdf#page=159
     * for specs.
     * @param size
     */
    private _fillAlignment;
    /**
     * Adds an uniform in the buffer.
     * Warning : the subsequents calls of this function must be in the same order as declared in the shader
     * for the layout to be correct ! The addUniform function only handles types like float, vec2, vec3, vec4, mat4,
     * meaning size=1,2,3,4 or 16. It does not handle struct types.
     * @param name Name of the uniform, as used in the uniform block in the shader.
     * @param size Data size, or data directly.
     * @param arraySize The number of elements in the array, 0 if not an array.
     */
    addUniform(name: string, size: number | number[], arraySize?: number): void;
    /**
     * Adds a Matrix 4x4 to the uniform buffer.
     * @param name Name of the uniform, as used in the uniform block in the shader.
     * @param mat A 4x4 matrix.
     */
    addMatrix(name: string, mat: IMatrixLike): void;
    /**
     * Adds a vec2 to the uniform buffer.
     * @param name Name of the uniform, as used in the uniform block in the shader.
     * @param x Define the x component value of the vec2
     * @param y Define the y component value of the vec2
     */
    addFloat2(name: string, x: number, y: number): void;
    /**
     * Adds a vec3 to the uniform buffer.
     * @param name Name of the uniform, as used in the uniform block in the shader.
     * @param x Define the x component value of the vec3
     * @param y Define the y component value of the vec3
     * @param z Define the z component value of the vec3
     */
    addFloat3(name: string, x: number, y: number, z: number): void;
    /**
     * Adds a vec3 to the uniform buffer.
     * @param name Name of the uniform, as used in the uniform block in the shader.
     * @param color Define the vec3 from a Color
     */
    addColor3(name: string, color: IColor3Like): void;
    /**
     * Adds a vec4 to the uniform buffer.
     * @param name Name of the uniform, as used in the uniform block in the shader.
     * @param color Define the rgb components from a Color
     * @param alpha Define the a component of the vec4
     */
    addColor4(name: string, color: IColor3Like, alpha: number): void;
    /**
     * Adds a vec3 to the uniform buffer.
     * @param name Name of the uniform, as used in the uniform block in the shader.
     * @param vector Define the vec3 components from a Vector
     */
    addVector3(name: string, vector: IVector3Like): void;
    /**
     * Adds a Matrix 3x3 to the uniform buffer.
     * @param name Name of the uniform, as used in the uniform block in the shader.
     */
    addMatrix3x3(name: string): void;
    /**
     * Adds a Matrix 2x2 to the uniform buffer.
     * @param name Name of the uniform, as used in the uniform block in the shader.
     */
    addMatrix2x2(name: string): void;
    /**
     * Effectively creates the WebGL Uniform Buffer, once layout is completed with `addUniform`.
     */
    create(): void;
    /** @internal */
    _rebuild(): void;
    /** @internal */
    get _numBuffers(): number;
    /** @internal */
    get _indexBuffer(): number;
    /** Gets the name of this buffer */
    get name(): string;
    private _buffersEqual;
    private _copyBuffer;
    /**
     * Updates the WebGL Uniform Buffer on the GPU.
     * If the `dynamic` flag is set to true, no cache comparison is done.
     * Otherwise, the buffer will be updated only if the cache differs.
     */
    update(): void;
    private _createNewBuffer;
    private _checkNewFrame;
    /**
     * Updates the value of an uniform. The `update` method must be called afterwards to make it effective in the GPU.
     * @param uniformName Define the name of the uniform, as used in the uniform block in the shader.
     * @param data Define the flattened data
     * @param size Define the size of the data.
     */
    updateUniform(uniformName: string, data: FloatArray, size: number): void;
    /**
     * Updates the value of an uniform. The `update` method must be called afterwards to make it effective in the GPU.
     * @param uniformName Define the name of the uniform, as used in the uniform block in the shader.
     * @param data Define the flattened data
     * @param size Define the size of the data.
     */
    updateUniformArray(uniformName: string, data: FloatArray, size: number): void;
    private _valueCache;
    private _cacheMatrix;
    private _updateMatrix3x3ForUniform;
    private _updateMatrix3x3ForEffect;
    private _updateMatrix2x2ForEffect;
    private _updateMatrix2x2ForUniform;
    private _updateFloatForEffect;
    private _updateFloatForUniform;
    private _updateFloat2ForEffect;
    private _updateFloat2ForUniform;
    private _updateFloat3ForEffect;
    private _updateFloat3ForUniform;
    private _updateFloat4ForEffect;
    private _updateFloat4ForUniform;
    private _updateFloatArrayForEffect;
    private _updateFloatArrayForUniform;
    private _updateArrayForEffect;
    private _updateArrayForUniform;
    private _updateIntArrayForEffect;
    private _updateIntArrayForUniform;
    private _updateUIntArrayForEffect;
    private _updateUIntArrayForUniform;
    private _updateMatrixForEffect;
    private _updateMatrixForUniform;
    private _updateMatricesForEffect;
    private _updateMatricesForUniform;
    private _updateVector3ForEffect;
    private _updateVector3ForUniform;
    private _updateVector4ForEffect;
    private _updateVector4ForUniform;
    private _updateColor3ForEffect;
    private _updateColor3ForUniform;
    private _updateColor4ForEffect;
    private _updateDirectColor4ForEffect;
    private _updateColor4ForUniform;
    private _updateDirectColor4ForUniform;
    private _updateIntForEffect;
    private _updateIntForUniform;
    private _updateInt2ForEffect;
    private _updateInt2ForUniform;
    private _updateInt3ForEffect;
    private _updateInt3ForUniform;
    private _updateInt4ForEffect;
    private _updateInt4ForUniform;
    private _updateUIntForEffect;
    private _updateUIntForUniform;
    private _updateUInt2ForEffect;
    private _updateUInt2ForUniform;
    private _updateUInt3ForEffect;
    private _updateUInt3ForUniform;
    private _updateUInt4ForEffect;
    private _updateUInt4ForUniform;
    /**
     * Sets a sampler uniform on the effect.
     * @param name Define the name of the sampler.
     * @param texture Define the texture to set in the sampler
     */
    setTexture(name: string, texture: Nullable<ThinTexture>): void;
    /**
     * Directly updates the value of the uniform in the cache AND on the GPU.
     * @param uniformName Define the name of the uniform, as used in the uniform block in the shader.
     * @param data Define the flattened data
     */
    updateUniformDirectly(uniformName: string, data: FloatArray): void;
    /**
     * Associates an effect to this uniform buffer
     * @param effect Define the effect to associate the buffer to
     * @param name Name of the uniform block in the shader.
     */
    bindToEffect(effect: Effect, name: string): void;
    /**
     * Binds the current (GPU) buffer to the effect
     */
    bindUniformBuffer(): void;
    /**
     * Dissociates the current effect from this uniform buffer
     */
    unbindEffect(): void;
    /**
     * Sets the current state of the class (_bufferIndex, _buffer) to point to the data buffer passed in parameter if this buffer is one of the buffers handled by the class (meaning if it can be found in the _buffers array)
     * This method is meant to be able to update a buffer at any time: just call setDataBuffer to set the class in the right state, call some updateXXX methods and then call udpate() => that will update the GPU buffer on the graphic card
     * @param dataBuffer buffer to look for
     * @returns true if the buffer has been found and the class internal state points to it, else false
     */
    setDataBuffer(dataBuffer: DataBuffer): boolean;
    /**
     * Disposes the uniform buffer.
     */
    dispose(): void;
}

}
declare module "babylonjs/Materials/uniformBufferEffectCommonAccessor" {
import { IColor3Like, IColor4Like, IMatrixLike, IVector3Like, IVector4Like } from "babylonjs/Maths/math.like";
import { Effect } from "babylonjs/Materials/effect";
import { UniformBuffer } from "babylonjs/Materials/uniformBuffer";
/** @internal */
export class UniformBufferEffectCommonAccessor {
    setMatrix3x3: (name: string, matrix: Float32Array) => void;
    setMatrix2x2: (name: string, matrix: Float32Array) => void;
    setFloat: (name: string, x: number) => void;
    setFloat2: (name: string, x: number, y: number, suffix?: string) => void;
    setFloat3: (name: string, x: number, y: number, z: number, suffix?: string) => void;
    setFloat4: (name: string, x: number, y: number, z: number, w: number, suffix?: string) => void;
    setFloatArray: (name: string, array: Float32Array) => void;
    setArray: (name: string, array: number[]) => void;
    setIntArray: (name: string, array: Int32Array) => void;
    setMatrix: (name: string, mat: IMatrixLike) => void;
    setMatrices: (name: string, mat: Float32Array) => void;
    setVector3: (name: string, vector: IVector3Like) => void;
    setVector4: (name: string, vector: IVector4Like) => void;
    setColor3: (name: string, color: IColor3Like, suffix?: string) => void;
    setColor4: (name: string, color: IColor3Like, alpha: number, suffix?: string) => void;
    setDirectColor4: (name: string, color: IColor4Like) => void;
    setInt: (name: string, x: number, suffix?: string) => void;
    setInt2: (name: string, x: number, y: number, suffix?: string) => void;
    setInt3: (name: string, x: number, y: number, z: number, suffix?: string) => void;
    setInt4: (name: string, x: number, y: number, z: number, w: number, suffix?: string) => void;
    private _isUbo;
    constructor(uboOrEffect: UniformBuffer | Effect);
}

}
declare module "babylonjs/Maths/index" {
export * from "babylonjs/Maths/math.scalar";
export * from "babylonjs/Maths/math.functions";
export * from "babylonjs/Maths/math.polar";
export * from "babylonjs/Maths/math";
export * from "babylonjs/Maths/sphericalPolynomial";

}
declare module "babylonjs/Maths/math.axis" {
import { Vector3 } from "babylonjs/Maths/math.vector";
/** Defines supported spaces */
export enum Space {
    /** Local (object) space */
    LOCAL = 0,
    /** World space */
    WORLD = 1,
    /** Bone space */
    BONE = 2
}
/** Defines the 3 main axes */
export class Axis {
    /** X axis */
    static X: Vector3;
    /** Y axis */
    static Y: Vector3;
    /** Z axis */
    static Z: Vector3;
}
/**
 * Defines cartesian components.
 */
export enum Coordinate {
    /** X axis */
    X = 0,
    /** Y axis */
    Y = 1,
    /** Z axis */
    Z = 2
}

}
declare module "babylonjs/Maths/math.color" {
import { DeepImmutable, FloatArray } from "babylonjs/types";
/**
 * Class used to hold a RGB color
 */
export class Color3 {
    /**
     * Defines the red component (between 0 and 1, default is 0)
     */
    r: number;
    /**
     * Defines the green component (between 0 and 1, default is 0)
     */
    g: number;
    /**
     * Defines the blue component (between 0 and 1, default is 0)
     */
    b: number;
    /**
     * Creates a new Color3 object from red, green, blue values, all between 0 and 1
     * @param r defines the red component (between 0 and 1, default is 0)
     * @param g defines the green component (between 0 and 1, default is 0)
     * @param b defines the blue component (between 0 and 1, default is 0)
     */
    constructor(
    /**
     * Defines the red component (between 0 and 1, default is 0)
     */
    r?: number, 
    /**
     * Defines the green component (between 0 and 1, default is 0)
     */
    g?: number, 
    /**
     * Defines the blue component (between 0 and 1, default is 0)
     */
    b?: number);
    /**
     * Creates a string with the Color3 current values
     * @returns the string representation of the Color3 object
     */
    toString(): string;
    /**
     * Returns the string "Color3"
     * @returns "Color3"
     */
    getClassName(): string;
    /**
     * Compute the Color3 hash code
     * @returns an unique number that can be used to hash Color3 objects
     */
    getHashCode(): number;
    /**
     * Stores in the given array from the given starting index the red, green, blue values as successive elements
     * @param array defines the array where to store the r,g,b components
     * @param index defines an optional index in the target array to define where to start storing values
     * @returns the current Color3 object
     */
    toArray(array: FloatArray, index?: number): Color3;
    /**
     * Update the current color with values stored in an array from the starting index of the given array
     * @param array defines the source array
     * @param offset defines an offset in the source array
     * @returns the current Color3 object
     */
    fromArray(array: DeepImmutable<ArrayLike<number>>, offset?: number): Color3;
    /**
     * Returns a new Color4 object from the current Color3 and the given alpha
     * @param alpha defines the alpha component on the new Color4 object (default is 1)
     * @returns a new Color4 object
     */
    toColor4(alpha?: number): Color4;
    /**
     * Returns a new array populated with 3 numeric elements : red, green and blue values
     * @returns the new array
     */
    asArray(): number[];
    /**
     * Returns the luminance value
     * @returns a float value
     */
    toLuminance(): number;
    /**
     * Multiply each Color3 rgb values by the given Color3 rgb values in a new Color3 object
     * @param otherColor defines the second operand
     * @returns the new Color3 object
     */
    multiply(otherColor: DeepImmutable<Color3>): Color3;
    /**
     * Multiply the rgb values of the Color3 and the given Color3 and stores the result in the object "result"
     * @param otherColor defines the second operand
     * @param result defines the Color3 object where to store the result
     * @returns the current Color3
     */
    multiplyToRef(otherColor: DeepImmutable<Color3>, result: Color3): Color3;
    /**
     * Determines equality between Color3 objects
     * @param otherColor defines the second operand
     * @returns true if the rgb values are equal to the given ones
     */
    equals(otherColor: DeepImmutable<Color3>): boolean;
    /**
     * Determines equality between the current Color3 object and a set of r,b,g values
     * @param r defines the red component to check
     * @param g defines the green component to check
     * @param b defines the blue component to check
     * @returns true if the rgb values are equal to the given ones
     */
    equalsFloats(r: number, g: number, b: number): boolean;
    /**
     * Creates a new Color3 with the current Color3 values multiplied by scale
     * @param scale defines the scaling factor to apply
     * @returns a new Color3 object
     */
    scale(scale: number): Color3;
    /**
     * Multiplies the Color3 values by the float "scale"
     * @param scale defines the scaling factor to apply
     * @returns the current updated Color3
     */
    scaleInPlace(scale: number): Color3;
    /**
     * Multiplies the rgb values by scale and stores the result into "result"
     * @param scale defines the scaling factor
     * @param result defines the Color3 object where to store the result
     * @returns the unmodified current Color3
     */
    scaleToRef(scale: number, result: Color3): Color3;
    /**
     * Scale the current Color3 values by a factor and add the result to a given Color3
     * @param scale defines the scale factor
     * @param result defines color to store the result into
     * @returns the unmodified current Color3
     */
    scaleAndAddToRef(scale: number, result: Color3): Color3;
    /**
     * Clamps the rgb values by the min and max values and stores the result into "result"
     * @param min defines minimum clamping value (default is 0)
     * @param max defines maximum clamping value (default is 1)
     * @param result defines color to store the result into
     * @returns the original Color3
     */
    clampToRef(min: number | undefined, max: number | undefined, result: Color3): Color3;
    /**
     * Creates a new Color3 set with the added values of the current Color3 and of the given one
     * @param otherColor defines the second operand
     * @returns the new Color3
     */
    add(otherColor: DeepImmutable<Color3>): Color3;
    /**
     * Stores the result of the addition of the current Color3 and given one rgb values into "result"
     * @param otherColor defines the second operand
     * @param result defines Color3 object to store the result into
     * @returns the unmodified current Color3
     */
    addToRef(otherColor: DeepImmutable<Color3>, result: Color3): Color3;
    /**
     * Returns a new Color3 set with the subtracted values of the given one from the current Color3
     * @param otherColor defines the second operand
     * @returns the new Color3
     */
    subtract(otherColor: DeepImmutable<Color3>): Color3;
    /**
     * Stores the result of the subtraction of given one from the current Color3 rgb values into "result"
     * @param otherColor defines the second operand
     * @param result defines Color3 object to store the result into
     * @returns the unmodified current Color3
     */
    subtractToRef(otherColor: DeepImmutable<Color3>, result: Color3): Color3;
    /**
     * Copy the current object
     * @returns a new Color3 copied the current one
     */
    clone(): Color3;
    /**
     * Copies the rgb values from the source in the current Color3
     * @param source defines the source Color3 object
     * @returns the updated Color3 object
     */
    copyFrom(source: DeepImmutable<Color3>): Color3;
    /**
     * Updates the Color3 rgb values from the given floats
     * @param r defines the red component to read from
     * @param g defines the green component to read from
     * @param b defines the blue component to read from
     * @returns the current Color3 object
     */
    copyFromFloats(r: number, g: number, b: number): Color3;
    /**
     * Updates the Color3 rgb values from the given floats
     * @param r defines the red component to read from
     * @param g defines the green component to read from
     * @param b defines the blue component to read from
     * @returns the current Color3 object
     */
    set(r: number, g: number, b: number): Color3;
    /**
     * Compute the Color3 hexadecimal code as a string
     * @returns a string containing the hexadecimal representation of the Color3 object
     */
    toHexString(): string;
    /**
     * Computes a new Color3 converted from the current one to linear space
     * @returns a new Color3 object
     */
    toLinearSpace(): Color3;
    /**
     * Converts current color in rgb space to HSV values
     * @returns a new color3 representing the HSV values
     */
    toHSV(): Color3;
    /**
     * Converts current color in rgb space to HSV values
     * @param result defines the Color3 where to store the HSV values
     */
    toHSVToRef(result: Color3): void;
    /**
     * Converts the Color3 values to linear space and stores the result in "convertedColor"
     * @param convertedColor defines the Color3 object where to store the linear space version
     * @returns the unmodified Color3
     */
    toLinearSpaceToRef(convertedColor: Color3): Color3;
    /**
     * Computes a new Color3 converted from the current one to gamma space
     * @returns a new Color3 object
     */
    toGammaSpace(): Color3;
    /**
     * Converts the Color3 values to gamma space and stores the result in "convertedColor"
     * @param convertedColor defines the Color3 object where to store the gamma space version
     * @returns the unmodified Color3
     */
    toGammaSpaceToRef(convertedColor: Color3): Color3;
    private static _BlackReadOnly;
    /**
     * Converts Hue, saturation and value to a Color3 (RGB)
     * @param hue defines the hue
     * @param saturation defines the saturation
     * @param value defines the value
     * @param result defines the Color3 where to store the RGB values
     */
    static HSVtoRGBToRef(hue: number, saturation: number, value: number, result: Color3): void;
    /**
     * Converts Hue, saturation and value to a new Color3 (RGB)
     * @param hue defines the hue (value between 0 and 360)
     * @param saturation defines the saturation (value between 0 and 1)
     * @param value defines the value (value between 0 and 1)
     * @returns a new Color3 object
     */
    static FromHSV(hue: number, saturation: number, value: number): Color3;
    /**
     * Creates a new Color3 from the string containing valid hexadecimal values
     * @param hex defines a string containing valid hexadecimal values
     * @returns a new Color3 object
     */
    static FromHexString(hex: string): Color3;
    /**
     * Creates a new Color3 from the starting index of the given array
     * @param array defines the source array
     * @param offset defines an offset in the source array
     * @returns a new Color3 object
     */
    static FromArray(array: DeepImmutable<ArrayLike<number>>, offset?: number): Color3;
    /**
     * Creates a new Color3 from the starting index element of the given array
     * @param array defines the source array to read from
     * @param offset defines the offset in the source array
     * @param result defines the target Color3 object
     */
    static FromArrayToRef(array: DeepImmutable<ArrayLike<number>>, offset: number | undefined, result: Color3): void;
    /**
     * Creates a new Color3 from integer values (< 256)
     * @param r defines the red component to read from (value between 0 and 255)
     * @param g defines the green component to read from (value between 0 and 255)
     * @param b defines the blue component to read from (value between 0 and 255)
     * @returns a new Color3 object
     */
    static FromInts(r: number, g: number, b: number): Color3;
    /**
     * Creates a new Color3 with values linearly interpolated of "amount" between the start Color3 and the end Color3
     * @param start defines the start Color3 value
     * @param end defines the end Color3 value
     * @param amount defines the gradient value between start and end
     * @returns a new Color3 object
     */
    static Lerp(start: DeepImmutable<Color3>, end: DeepImmutable<Color3>, amount: number): Color3;
    /**
     * Creates a new Color3 with values linearly interpolated of "amount" between the start Color3 and the end Color3
     * @param left defines the start value
     * @param right defines the end value
     * @param amount defines the gradient factor
     * @param result defines the Color3 object where to store the result
     */
    static LerpToRef(left: DeepImmutable<Color3>, right: DeepImmutable<Color3>, amount: number, result: Color3): void;
    /**
     * Returns a new Color3 located for "amount" (float) on the Hermite interpolation spline defined by the vectors "value1", "tangent1", "value2", "tangent2"
     * @param value1 defines the first control point
     * @param tangent1 defines the first tangent Color3
     * @param value2 defines the second control point
     * @param tangent2 defines the second tangent Color3
     * @param amount defines the amount on the interpolation spline (between 0 and 1)
     * @returns the new Color3
     */
    static Hermite(value1: DeepImmutable<Color3>, tangent1: DeepImmutable<Color3>, value2: DeepImmutable<Color3>, tangent2: DeepImmutable<Color3>, amount: number): Color3;
    /**
     * Returns a new Color3 which is the 1st derivative of the Hermite spline defined by the colors "value1", "value2", "tangent1", "tangent2".
     * @param value1 defines the first control point
     * @param tangent1 defines the first tangent
     * @param value2 defines the second control point
     * @param tangent2 defines the second tangent
     * @param time define where the derivative must be done
     * @returns 1st derivative
     */
    static Hermite1stDerivative(value1: DeepImmutable<Color3>, tangent1: DeepImmutable<Color3>, value2: DeepImmutable<Color3>, tangent2: DeepImmutable<Color3>, time: number): Color3;
    /**
     * Returns a new Color3 which is the 1st derivative of the Hermite spline defined by the colors "value1", "value2", "tangent1", "tangent2".
     * @param value1 defines the first control point
     * @param tangent1 defines the first tangent
     * @param value2 defines the second control point
     * @param tangent2 defines the second tangent
     * @param time define where the derivative must be done
     * @param result define where to store the derivative
     */
    static Hermite1stDerivativeToRef(value1: DeepImmutable<Color3>, tangent1: DeepImmutable<Color3>, value2: DeepImmutable<Color3>, tangent2: DeepImmutable<Color3>, time: number, result: Color3): void;
    /**
     * Returns a Color3 value containing a red color
     * @returns a new Color3 object
     */
    static Red(): Color3;
    /**
     * Returns a Color3 value containing a green color
     * @returns a new Color3 object
     */
    static Green(): Color3;
    /**
     * Returns a Color3 value containing a blue color
     * @returns a new Color3 object
     */
    static Blue(): Color3;
    /**
     * Returns a Color3 value containing a black color
     * @returns a new Color3 object
     */
    static Black(): Color3;
    /**
     * Gets a Color3 value containing a black color that must not be updated
     */
    static get BlackReadOnly(): DeepImmutable<Color3>;
    /**
     * Returns a Color3 value containing a white color
     * @returns a new Color3 object
     */
    static White(): Color3;
    /**
     * Returns a Color3 value containing a purple color
     * @returns a new Color3 object
     */
    static Purple(): Color3;
    /**
     * Returns a Color3 value containing a magenta color
     * @returns a new Color3 object
     */
    static Magenta(): Color3;
    /**
     * Returns a Color3 value containing a yellow color
     * @returns a new Color3 object
     */
    static Yellow(): Color3;
    /**
     * Returns a Color3 value containing a gray color
     * @returns a new Color3 object
     */
    static Gray(): Color3;
    /**
     * Returns a Color3 value containing a teal color
     * @returns a new Color3 object
     */
    static Teal(): Color3;
    /**
     * Returns a Color3 value containing a random color
     * @returns a new Color3 object
     */
    static Random(): Color3;
}
/**
 * Class used to hold a RBGA color
 */
export class Color4 {
    /**
     * Defines the red component (between 0 and 1, default is 0)
     */
    r: number;
    /**
     * Defines the green component (between 0 and 1, default is 0)
     */
    g: number;
    /**
     * Defines the blue component (between 0 and 1, default is 0)
     */
    b: number;
    /**
     * Defines the alpha component (between 0 and 1, default is 1)
     */
    a: number;
    /**
     * Creates a new Color4 object from red, green, blue values, all between 0 and 1
     * @param r defines the red component (between 0 and 1, default is 0)
     * @param g defines the green component (between 0 and 1, default is 0)
     * @param b defines the blue component (between 0 and 1, default is 0)
     * @param a defines the alpha component (between 0 and 1, default is 1)
     */
    constructor(
    /**
     * Defines the red component (between 0 and 1, default is 0)
     */
    r?: number, 
    /**
     * Defines the green component (between 0 and 1, default is 0)
     */
    g?: number, 
    /**
     * Defines the blue component (between 0 and 1, default is 0)
     */
    b?: number, 
    /**
     * Defines the alpha component (between 0 and 1, default is 1)
     */
    a?: number);
    /**
     * Adds in place the given Color4 values to the current Color4 object
     * @param right defines the second operand
     * @returns the current updated Color4 object
     */
    addInPlace(right: DeepImmutable<Color4>): Color4;
    /**
     * Creates a new array populated with 4 numeric elements : red, green, blue, alpha values
     * @returns the new array
     */
    asArray(): number[];
    /**
     * Stores from the starting index in the given array the Color4 successive values
     * @param array defines the array where to store the r,g,b components
     * @param index defines an optional index in the target array to define where to start storing values
     * @returns the current Color4 object
     */
    toArray(array: FloatArray, index?: number): Color4;
    /**
     * Update the current color with values stored in an array from the starting index of the given array
     * @param array defines the source array
     * @param offset defines an offset in the source array
     * @returns the current Color4 object
     */
    fromArray(array: DeepImmutable<ArrayLike<number>>, offset?: number): Color4;
    /**
     * Determines equality between Color4 objects
     * @param otherColor defines the second operand
     * @returns true if the rgba values are equal to the given ones
     */
    equals(otherColor: DeepImmutable<Color4>): boolean;
    /**
     * Creates a new Color4 set with the added values of the current Color4 and of the given one
     * @param right defines the second operand
     * @returns a new Color4 object
     */
    add(right: DeepImmutable<Color4>): Color4;
    /**
     * Creates a new Color4 set with the subtracted values of the given one from the current Color4
     * @param right defines the second operand
     * @returns a new Color4 object
     */
    subtract(right: DeepImmutable<Color4>): Color4;
    /**
     * Subtracts the given ones from the current Color4 values and stores the results in "result"
     * @param right defines the second operand
     * @param result defines the Color4 object where to store the result
     * @returns the current Color4 object
     */
    subtractToRef(right: DeepImmutable<Color4>, result: Color4): Color4;
    /**
     * Creates a new Color4 with the current Color4 values multiplied by scale
     * @param scale defines the scaling factor to apply
     * @returns a new Color4 object
     */
    scale(scale: number): Color4;
    /**
     * Multiplies the Color4 values by the float "scale"
     * @param scale defines the scaling factor to apply
     * @returns the current updated Color4
     */
    scaleInPlace(scale: number): Color4;
    /**
     * Multiplies the current Color4 values by scale and stores the result in "result"
     * @param scale defines the scaling factor to apply
     * @param result defines the Color4 object where to store the result
     * @returns the current unmodified Color4
     */
    scaleToRef(scale: number, result: Color4): Color4;
    /**
     * Scale the current Color4 values by a factor and add the result to a given Color4
     * @param scale defines the scale factor
     * @param result defines the Color4 object where to store the result
     * @returns the unmodified current Color4
     */
    scaleAndAddToRef(scale: number, result: Color4): Color4;
    /**
     * Clamps the rgb values by the min and max values and stores the result into "result"
     * @param min defines minimum clamping value (default is 0)
     * @param max defines maximum clamping value (default is 1)
     * @param result defines color to store the result into.
     * @returns the current Color4
     */
    clampToRef(min: number | undefined, max: number | undefined, result: Color4): Color4;
    /**
     * Multiply an Color4 value by another and return a new Color4 object
     * @param color defines the Color4 value to multiply by
     * @returns a new Color4 object
     */
    multiply(color: Color4): Color4;
    /**
     * Multiply a Color4 value by another and push the result in a reference value
     * @param color defines the Color4 value to multiply by
     * @param result defines the Color4 to fill the result in
     * @returns the result Color4
     */
    multiplyToRef(color: Color4, result: Color4): Color4;
    /**
     * Creates a string with the Color4 current values
     * @returns the string representation of the Color4 object
     */
    toString(): string;
    /**
     * Returns the string "Color4"
     * @returns "Color4"
     */
    getClassName(): string;
    /**
     * Compute the Color4 hash code
     * @returns an unique number that can be used to hash Color4 objects
     */
    getHashCode(): number;
    /**
     * Creates a new Color4 copied from the current one
     * @returns a new Color4 object
     */
    clone(): Color4;
    /**
     * Copies the given Color4 values into the current one
     * @param source defines the source Color4 object
     * @returns the current updated Color4 object
     */
    copyFrom(source: Color4): Color4;
    /**
     * Copies the given float values into the current one
     * @param r defines the red component to read from
     * @param g defines the green component to read from
     * @param b defines the blue component to read from
     * @param a defines the alpha component to read from
     * @returns the current updated Color4 object
     */
    copyFromFloats(r: number, g: number, b: number, a: number): Color4;
    /**
     * Copies the given float values into the current one
     * @param r defines the red component to read from
     * @param g defines the green component to read from
     * @param b defines the blue component to read from
     * @param a defines the alpha component to read from
     * @returns the current updated Color4 object
     */
    set(r: number, g: number, b: number, a: number): Color4;
    /**
     * Compute the Color4 hexadecimal code as a string
     * @param returnAsColor3 defines if the string should only contains RGB values (off by default)
     * @returns a string containing the hexadecimal representation of the Color4 object
     */
    toHexString(returnAsColor3?: boolean): string;
    /**
     * Computes a new Color4 converted from the current one to linear space
     * @returns a new Color4 object
     */
    toLinearSpace(): Color4;
    /**
     * Converts the Color4 values to linear space and stores the result in "convertedColor"
     * @param convertedColor defines the Color4 object where to store the linear space version
     * @returns the unmodified Color4
     */
    toLinearSpaceToRef(convertedColor: Color4): Color4;
    /**
     * Computes a new Color4 converted from the current one to gamma space
     * @returns a new Color4 object
     */
    toGammaSpace(): Color4;
    /**
     * Converts the Color4 values to gamma space and stores the result in "convertedColor"
     * @param convertedColor defines the Color4 object where to store the gamma space version
     * @returns the unmodified Color4
     */
    toGammaSpaceToRef(convertedColor: Color4): Color4;
    /**
     * Creates a new Color4 from the string containing valid hexadecimal values.
     *
     * A valid hex string is either in the format #RRGGBB or #RRGGBBAA.
     *
     * When a hex string without alpha is passed, the resulting Color4 has
     * its alpha value set to 1.0.
     *
     * An invalid string results in a Color with all its channels set to 0.0,
     * i.e. "transparent black".
     *
     * @param hex defines a string containing valid hexadecimal values
     * @returns a new Color4 object
     */
    static FromHexString(hex: string): Color4;
    /**
     * Creates a new Color4 object set with the linearly interpolated values of "amount" between the left Color4 object and the right Color4 object
     * @param left defines the start value
     * @param right defines the end value
     * @param amount defines the gradient factor
     * @returns a new Color4 object
     */
    static Lerp(left: DeepImmutable<Color4>, right: DeepImmutable<Color4>, amount: number): Color4;
    /**
     * Set the given "result" with the linearly interpolated values of "amount" between the left Color4 object and the right Color4 object
     * @param left defines the start value
     * @param right defines the end value
     * @param amount defines the gradient factor
     * @param result defines the Color4 object where to store data
     */
    static LerpToRef(left: DeepImmutable<Color4>, right: DeepImmutable<Color4>, amount: number, result: Color4): void;
    /**
     * Interpolate between two Color4 using Hermite interpolation
     * @param value1 defines first Color4
     * @param tangent1 defines the incoming tangent
     * @param value2 defines second Color4
     * @param tangent2 defines the outgoing tangent
     * @param amount defines the target Color4
     * @returns the new interpolated Color4
     */
    static Hermite(value1: DeepImmutable<Color4>, tangent1: DeepImmutable<Color4>, value2: DeepImmutable<Color4>, tangent2: DeepImmutable<Color4>, amount: number): Color4;
    /**
     * Returns a new Color4 which is the 1st derivative of the Hermite spline defined by the colors "value1", "value2", "tangent1", "tangent2".
     * @param value1 defines the first control point
     * @param tangent1 defines the first tangent
     * @param value2 defines the second control point
     * @param tangent2 defines the second tangent
     * @param time define where the derivative must be done
     * @returns 1st derivative
     */
    static Hermite1stDerivative(value1: DeepImmutable<Color4>, tangent1: DeepImmutable<Color4>, value2: DeepImmutable<Color4>, tangent2: DeepImmutable<Color4>, time: number): Color4;
    /**
     * Update a Color4 with the 1st derivative of the Hermite spline defined by the colors "value1", "value2", "tangent1", "tangent2".
     * @param value1 defines the first control point
     * @param tangent1 defines the first tangent
     * @param value2 defines the second control point
     * @param tangent2 defines the second tangent
     * @param time define where the derivative must be done
     * @param result define where to store the derivative
     */
    static Hermite1stDerivativeToRef(value1: DeepImmutable<Color4>, tangent1: DeepImmutable<Color4>, value2: DeepImmutable<Color4>, tangent2: DeepImmutable<Color4>, time: number, result: Color4): void;
    /**
     * Creates a new Color4 from a Color3 and an alpha value
     * @param color3 defines the source Color3 to read from
     * @param alpha defines the alpha component (1.0 by default)
     * @returns a new Color4 object
     */
    static FromColor3(color3: DeepImmutable<Color3>, alpha?: number): Color4;
    /**
     * Creates a new Color4 from the starting index element of the given array
     * @param array defines the source array to read from
     * @param offset defines the offset in the source array
     * @returns a new Color4 object
     */
    static FromArray(array: DeepImmutable<ArrayLike<number>>, offset?: number): Color4;
    /**
     * Creates a new Color4 from the starting index element of the given array
     * @param array defines the source array to read from
     * @param offset defines the offset in the source array
     * @param result defines the target Color4 object
     */
    static FromArrayToRef(array: DeepImmutable<ArrayLike<number>>, offset: number | undefined, result: Color4): void;
    /**
     * Creates a new Color3 from integer values (< 256)
     * @param r defines the red component to read from (value between 0 and 255)
     * @param g defines the green component to read from (value between 0 and 255)
     * @param b defines the blue component to read from (value between 0 and 255)
     * @param a defines the alpha component to read from (value between 0 and 255)
     * @returns a new Color3 object
     */
    static FromInts(r: number, g: number, b: number, a: number): Color4;
    /**
     * Check the content of a given array and convert it to an array containing RGBA data
     * If the original array was already containing count * 4 values then it is returned directly
     * @param colors defines the array to check
     * @param count defines the number of RGBA data to expect
     * @returns an array containing count * 4 values (RGBA)
     */
    static CheckColors4(colors: number[], count: number): number[];
}
/**
 * @internal
 */
export class TmpColors {
    static Color3: Color3[];
    static Color4: Color4[];
}

}
declare module "babylonjs/Maths/math.constants" {
/**
 * Constant used to convert a value to gamma space
 * @ignorenaming
 */
export const ToGammaSpace: number;
/**
 * Constant used to convert a value to linear space
 * @ignorenaming
 */
export const ToLinearSpace = 2.2;
/**
 * Constant Golden Ratio value in Babylon.js
 * @ignorenaming
 */
export const PHI: number;
/**
 * Constant used to define the minimal number value in Babylon.js
 * @ignorenaming
 */
const Epsilon = 0.001;
export { Epsilon };

}
declare module "babylonjs/Maths/math" {
export * from "babylonjs/Maths/math.axis";
export * from "babylonjs/Maths/math.color";
export * from "babylonjs/Maths/math.constants";
export * from "babylonjs/Maths/math.frustum";
export * from "babylonjs/Maths/math.path";
export * from "babylonjs/Maths/math.plane";
export * from "babylonjs/Maths/math.size";
export * from "babylonjs/Maths/math.vector";
export * from "babylonjs/Maths/math.vertexFormat";
export * from "babylonjs/Maths/math.viewport";

}
declare module "babylonjs/Maths/math.frustum" {
import { Matrix } from "babylonjs/Maths/math.vector";
import { DeepImmutable } from "babylonjs/types";
import { Plane } from "babylonjs/Maths/math.plane";
/**
 * Represents a camera frustum
 */
export class Frustum {
    /**
     * Gets the planes representing the frustum
     * @param transform matrix to be applied to the returned planes
     * @returns a new array of 6 Frustum planes computed by the given transformation matrix.
     */
    static GetPlanes(transform: DeepImmutable<Matrix>): Plane[];
    /**
     * Gets the near frustum plane transformed by the transform matrix
     * @param transform transformation matrix to be applied to the resulting frustum plane
     * @param frustumPlane the resulting frustum plane
     */
    static GetNearPlaneToRef(transform: DeepImmutable<Matrix>, frustumPlane: Plane): void;
    /**
     * Gets the far frustum plane transformed by the transform matrix
     * @param transform transformation matrix to be applied to the resulting frustum plane
     * @param frustumPlane the resulting frustum plane
     */
    static GetFarPlaneToRef(transform: DeepImmutable<Matrix>, frustumPlane: Plane): void;
    /**
     * Gets the left frustum plane transformed by the transform matrix
     * @param transform transformation matrix to be applied to the resulting frustum plane
     * @param frustumPlane the resulting frustum plane
     */
    static GetLeftPlaneToRef(transform: DeepImmutable<Matrix>, frustumPlane: Plane): void;
    /**
     * Gets the right frustum plane transformed by the transform matrix
     * @param transform transformation matrix to be applied to the resulting frustum plane
     * @param frustumPlane the resulting frustum plane
     */
    static GetRightPlaneToRef(transform: DeepImmutable<Matrix>, frustumPlane: Plane): void;
    /**
     * Gets the top frustum plane transformed by the transform matrix
     * @param transform transformation matrix to be applied to the resulting frustum plane
     * @param frustumPlane the resulting frustum plane
     */
    static GetTopPlaneToRef(transform: DeepImmutable<Matrix>, frustumPlane: Plane): void;
    /**
     * Gets the bottom frustum plane transformed by the transform matrix
     * @param transform transformation matrix to be applied to the resulting frustum plane
     * @param frustumPlane the resulting frustum plane
     */
    static GetBottomPlaneToRef(transform: DeepImmutable<Matrix>, frustumPlane: Plane): void;
    /**
     * Sets the given array "frustumPlanes" with the 6 Frustum planes computed by the given transformation matrix.
     * @param transform transformation matrix to be applied to the resulting frustum planes
     * @param frustumPlanes the resulting frustum planes
     */
    static GetPlanesToRef(transform: DeepImmutable<Matrix>, frustumPlanes: Plane[]): void;
}

}
declare module "babylonjs/Maths/math.functions" {
import { FloatArray, Nullable, IndicesArray } from "babylonjs/types";
import { Vector2 } from "babylonjs/Maths/math.vector";
import { Vector3 } from "babylonjs/Maths/math.vector";
/**
 * Extracts minimum and maximum values from a list of indexed positions
 * @param positions defines the positions to use
 * @param indices defines the indices to the positions
 * @param indexStart defines the start index
 * @param indexCount defines the end index
 * @param bias defines bias value to add to the result
 * @returns minimum and maximum values
 */
export function extractMinAndMaxIndexed(positions: FloatArray, indices: IndicesArray, indexStart: number, indexCount: number, bias?: Nullable<Vector2>): {
    minimum: Vector3;
    maximum: Vector3;
};
/**
 * Extracts minimum and maximum values from a list of positions
 * @param positions defines the positions to use
 * @param start defines the start index in the positions array
 * @param count defines the number of positions to handle
 * @param bias defines bias value to add to the result
 * @param stride defines the stride size to use (distance between two positions in the positions array)
 * @returns minimum and maximum values
 */
export function extractMinAndMax(positions: FloatArray, start: number, count: number, bias?: Nullable<Vector2>, stride?: number): {
    minimum: Vector3;
    maximum: Vector3;
};

}
declare module "babylonjs/Maths/math.isovector" {
import { Vector3 } from "babylonjs/Maths/math.vector";
/**
 * Class representing an isovector a vector containing 2 INTEGER coordinates
 * x axis is horizontal
 * y axis is 60 deg counter clockwise from positive y axis
 * @internal
 */
export class _IsoVector {
    /** defines the first coordinate */
    x: number;
    /** defines the second coordinate */
    y: number;
    /**
     * Creates a new isovector from the given x and y coordinates
     * @param x defines the first coordinate, must be an integer
     * @param y defines the second coordinate, must be an integer
     */
    constructor(
    /** defines the first coordinate */
    x?: number, 
    /** defines the second coordinate */
    y?: number);
    /**
     * Gets a new IsoVector copied from the IsoVector
     * @returns a new IsoVector
     */
    clone(): _IsoVector;
    /**
     * Rotates one IsoVector 60 degrees counter clockwise about another
     * Please note that this is an in place operation
     * @param other an IsoVector a center of rotation
     * @returns the rotated IsoVector
     */
    rotate60About(other: _IsoVector): this;
    /**
     * Rotates one IsoVector 60 degrees clockwise about another
     * Please note that this is an in place operation
     * @param other an IsoVector as center of rotation
     * @returns the rotated IsoVector
     */
    rotateNeg60About(other: _IsoVector): this;
    /**
     * For an equilateral triangle OAB with O at isovector (0, 0) and A at isovector (m, n)
     * Rotates one IsoVector 120 degrees counter clockwise about the center of the triangle
     * Please note that this is an in place operation
     * @param m integer a measure a Primary triangle of order (m, n) m > n
     * @param n >= 0 integer a measure for a Primary triangle of order (m, n)
     * @returns the rotated IsoVector
     */
    rotate120(m: number, n: number): this;
    /**
     * For an equilateral triangle OAB with O at isovector (0, 0) and A at isovector (m, n)
     * Rotates one IsoVector 120 degrees clockwise about the center of the triangle
     * Please note that this is an in place operation
     * @param m integer a measure a Primary triangle of order (m, n) m > n
     * @param n >= 0 integer a measure for a Primary triangle of order (m, n)
     * @returns the rotated IsoVector
     */
    rotateNeg120(m: number, n: number): this;
    /**
     * Transforms an IsoVector to one in Cartesian 3D space based on an isovector
     * @param origin an IsoVector
     * @param isoGridSize
     * @returns Point as a Vector3
     */
    toCartesianOrigin(origin: _IsoVector, isoGridSize: number): Vector3;
    /**
     * Gets a new IsoVector(0, 0)
     * @returns a new IsoVector
     */
    static Zero(): _IsoVector;
}

}
declare module "babylonjs/Maths/math.like" {
import { float, int, DeepImmutable } from "babylonjs/types";
/**
 * @internal
 */
export interface IColor4Like {
    r: float;
    g: float;
    b: float;
    a: float;
}
/**
 * @internal
 */
export interface IColor3Like {
    r: float;
    g: float;
    b: float;
}
export interface IQuaternionLike {
    x: float;
    y: float;
    z: float;
    w: float;
}
/**
 * @internal
 */
export interface IVector4Like {
    x: float;
    y: float;
    z: float;
    w: float;
}
/**
 * @internal
 */
export interface IVector3Like {
    x: float;
    y: float;
    z: float;
}
/**
 * @internal
 */
export interface IVector2Like {
    x: float;
    y: float;
}
/**
 * @internal
 */
export interface IMatrixLike {
    toArray(): DeepImmutable<Float32Array | Array<number>>;
    updateFlag: int;
}
/**
 * @internal
 */
export interface IViewportLike {
    x: float;
    y: float;
    width: float;
    height: float;
}
/**
 * @internal
 */
export interface IPlaneLike {
    normal: IVector3Like;
    d: float;
    normalize(): void;
}

}
declare module "babylonjs/Maths/math.path" {
import { DeepImmutable, Nullable } from "babylonjs/types";
import { Vector2, Vector3 } from "babylonjs/Maths/math.vector";
/**
 * Defines potential orientation for back face culling
 */
export enum Orientation {
    /**
     * Clockwise
     */
    CW = 0,
    /** Counter clockwise */
    CCW = 1
}
/** Class used to represent a Bezier curve */
export class BezierCurve {
    /**
     * Returns the cubic Bezier interpolated value (float) at "t" (float) from the given x1, y1, x2, y2 floats
     * @param t defines the time
     * @param x1 defines the left coordinate on X axis
     * @param y1 defines the left coordinate on Y axis
     * @param x2 defines the right coordinate on X axis
     * @param y2 defines the right coordinate on Y axis
     * @returns the interpolated value
     */
    static Interpolate(t: number, x1: number, y1: number, x2: number, y2: number): number;
}
/**
 * Defines angle representation
 */
export class Angle {
    private _radians;
    /**
     * Creates an Angle object of "radians" radians (float).
     * @param radians the angle in radians
     */
    constructor(radians: number);
    /**
     * Get value in degrees
     * @returns the Angle value in degrees (float)
     */
    degrees(): number;
    /**
     * Get value in radians
     * @returns the Angle value in radians (float)
     */
    radians(): number;
    /**
     * Gets a new Angle object valued with the gradient angle, in radians, of the line joining two points
     * @param a defines first point as the origin
     * @param b defines point
     * @returns a new Angle
     */
    static BetweenTwoPoints(a: DeepImmutable<Vector2>, b: DeepImmutable<Vector2>): Angle;
    /**
     * Gets a new Angle object from the given float in radians
     * @param radians defines the angle value in radians
     * @returns a new Angle
     */
    static FromRadians(radians: number): Angle;
    /**
     * Gets a new Angle object from the given float in degrees
     * @param degrees defines the angle value in degrees
     * @returns a new Angle
     */
    static FromDegrees(degrees: number): Angle;
}
/**
 * This represents an arc in a 2d space.
 */
export class Arc2 {
    /** Defines the start point of the arc */
    startPoint: Vector2;
    /** Defines the mid point of the arc */
    midPoint: Vector2;
    /** Defines the end point of the arc */
    endPoint: Vector2;
    /**
     * Defines the center point of the arc.
     */
    centerPoint: Vector2;
    /**
     * Defines the radius of the arc.
     */
    radius: number;
    /**
     * Defines the angle of the arc (from mid point to end point).
     */
    angle: Angle;
    /**
     * Defines the start angle of the arc (from start point to middle point).
     */
    startAngle: Angle;
    /**
     * Defines the orientation of the arc (clock wise/counter clock wise).
     */
    orientation: Orientation;
    /**
     * Creates an Arc object from the three given points : start, middle and end.
     * @param startPoint Defines the start point of the arc
     * @param midPoint Defines the middle point of the arc
     * @param endPoint Defines the end point of the arc
     */
    constructor(
    /** Defines the start point of the arc */
    startPoint: Vector2, 
    /** Defines the mid point of the arc */
    midPoint: Vector2, 
    /** Defines the end point of the arc */
    endPoint: Vector2);
}
/**
 * Represents a 2D path made up of multiple 2D points
 */
export class Path2 {
    private _points;
    private _length;
    /**
     * If the path start and end point are the same
     */
    closed: boolean;
    /**
     * Creates a Path2 object from the starting 2D coordinates x and y.
     * @param x the starting points x value
     * @param y the starting points y value
     */
    constructor(x: number, y: number);
    /**
     * Adds a new segment until the given coordinates (x, y) to the current Path2.
     * @param x the added points x value
     * @param y the added points y value
     * @returns the updated Path2.
     */
    addLineTo(x: number, y: number): Path2;
    /**
     * Adds _numberOfSegments_ segments according to the arc definition (middle point coordinates, end point coordinates, the arc start point being the current Path2 last point) to the current Path2.
     * @param midX middle point x value
     * @param midY middle point y value
     * @param endX end point x value
     * @param endY end point y value
     * @param numberOfSegments (default: 36)
     * @returns the updated Path2.
     */
    addArcTo(midX: number, midY: number, endX: number, endY: number, numberOfSegments?: number): Path2;
    /**
     * Closes the Path2.
     * @returns the Path2.
     */
    close(): Path2;
    /**
     * Gets the sum of the distance between each sequential point in the path
     * @returns the Path2 total length (float).
     */
    length(): number;
    /**
     * Gets the points which construct the path
     * @returns the Path2 internal array of points.
     */
    getPoints(): Vector2[];
    /**
     * Retreives the point at the distance aways from the starting point
     * @param normalizedLengthPosition the length along the path to retrieve the point from
     * @returns a new Vector2 located at a percentage of the Path2 total length on this path.
     */
    getPointAtLengthPosition(normalizedLengthPosition: number): Vector2;
    /**
     * Creates a new path starting from an x and y position
     * @param x starting x value
     * @param y starting y value
     * @returns a new Path2 starting at the coordinates (x, y).
     */
    static StartingAt(x: number, y: number): Path2;
}
/**
 * Represents a 3D path made up of multiple 3D points
 * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/path3D
 */
export class Path3D {
    /**
     * an array of Vector3, the curve axis of the Path3D
     */
    path: Vector3[];
    private _curve;
    private _distances;
    private _tangents;
    private _normals;
    private _binormals;
    private _raw;
    private _alignTangentsWithPath;
    private readonly _pointAtData;
    /**
     * new Path3D(path, normal, raw)
     * Creates a Path3D. A Path3D is a logical math object, so not a mesh.
     * please read the description in the tutorial : https://doc.babylonjs.com/features/featuresDeepDive/mesh/path3D
     * @param path an array of Vector3, the curve axis of the Path3D
     * @param firstNormal (options) Vector3, the first wanted normal to the curve. Ex (0, 1, 0) for a vertical normal.
     * @param raw (optional, default false) : boolean, if true the returned Path3D isn't normalized. Useful to depict path acceleration or speed.
     * @param alignTangentsWithPath (optional, default false) : boolean, if true the tangents will be aligned with the path.
     */
    constructor(
    /**
     * an array of Vector3, the curve axis of the Path3D
     */
    path: Vector3[], firstNormal?: Nullable<Vector3>, raw?: boolean, alignTangentsWithPath?: boolean);
    /**
     * Returns the Path3D array of successive Vector3 designing its curve.
     * @returns the Path3D array of successive Vector3 designing its curve.
     */
    getCurve(): Vector3[];
    /**
     * Returns the Path3D array of successive Vector3 designing its curve.
     * @returns the Path3D array of successive Vector3 designing its curve.
     */
    getPoints(): Vector3[];
    /**
     * @returns the computed length (float) of the path.
     */
    length(): number;
    /**
     * Returns an array populated with tangent vectors on each Path3D curve point.
     * @returns an array populated with tangent vectors on each Path3D curve point.
     */
    getTangents(): Vector3[];
    /**
     * Returns an array populated with normal vectors on each Path3D curve point.
     * @returns an array populated with normal vectors on each Path3D curve point.
     */
    getNormals(): Vector3[];
    /**
     * Returns an array populated with binormal vectors on each Path3D curve point.
     * @returns an array populated with binormal vectors on each Path3D curve point.
     */
    getBinormals(): Vector3[];
    /**
     * Returns an array populated with distances (float) of the i-th point from the first curve point.
     * @returns an array populated with distances (float) of the i-th point from the first curve point.
     */
    getDistances(): number[];
    /**
     * Returns an interpolated point along this path
     * @param position the position of the point along this path, from 0.0 to 1.0
     * @returns a new Vector3 as the point
     */
    getPointAt(position: number): Vector3;
    /**
     * Returns the tangent vector of an interpolated Path3D curve point at the specified position along this path.
     * @param position the position of the point along this path, from 0.0 to 1.0
     * @param interpolated (optional, default false) : boolean, if true returns an interpolated tangent instead of the tangent of the previous path point.
     * @returns a tangent vector corresponding to the interpolated Path3D curve point, if not interpolated, the tangent is taken from the precomputed tangents array.
     */
    getTangentAt(position: number, interpolated?: boolean): Vector3;
    /**
     * Returns the tangent vector of an interpolated Path3D curve point at the specified position along this path.
     * @param position the position of the point along this path, from 0.0 to 1.0
     * @param interpolated (optional, default false) : boolean, if true returns an interpolated normal instead of the normal of the previous path point.
     * @returns a normal vector corresponding to the interpolated Path3D curve point, if not interpolated, the normal is taken from the precomputed normals array.
     */
    getNormalAt(position: number, interpolated?: boolean): Vector3;
    /**
     * Returns the binormal vector of an interpolated Path3D curve point at the specified position along this path.
     * @param position the position of the point along this path, from 0.0 to 1.0
     * @param interpolated (optional, default false) : boolean, if true returns an interpolated binormal instead of the binormal of the previous path point.
     * @returns a binormal vector corresponding to the interpolated Path3D curve point, if not interpolated, the binormal is taken from the precomputed binormals array.
     */
    getBinormalAt(position: number, interpolated?: boolean): Vector3;
    /**
     * Returns the distance (float) of an interpolated Path3D curve point at the specified position along this path.
     * @param position the position of the point along this path, from 0.0 to 1.0
     * @returns the distance of the interpolated Path3D curve point at the specified position along this path.
     */
    getDistanceAt(position: number): number;
    /**
     * Returns the array index of the previous point of an interpolated point along this path
     * @param position the position of the point to interpolate along this path, from 0.0 to 1.0
     * @returns the array index
     */
    getPreviousPointIndexAt(position: number): number;
    /**
     * Returns the position of an interpolated point relative to the two path points it lies between, from 0.0 (point A) to 1.0 (point B)
     * @param position the position of the point to interpolate along this path, from 0.0 to 1.0
     * @returns the sub position
     */
    getSubPositionAt(position: number): number;
    /**
     * Returns the position of the closest virtual point on this path to an arbitrary Vector3, from 0.0 to 1.0
     * @param target the vector of which to get the closest position to
     * @returns the position of the closest virtual point on this path to the target vector
     */
    getClosestPositionTo(target: Vector3): number;
    /**
     * Returns a sub path (slice) of this path
     * @param start the position of the fist path point, from 0.0 to 1.0, or a negative value, which will get wrapped around from the end of the path to 0.0 to 1.0 values
     * @param end the position of the last path point, from 0.0 to 1.0, or a negative value, which will get wrapped around from the end of the path to 0.0 to 1.0 values
     * @returns a sub path (slice) of this path
     */
    slice(start?: number, end?: number): Path3D;
    /**
     * Forces the Path3D tangent, normal, binormal and distance recomputation.
     * @param path path which all values are copied into the curves points
     * @param firstNormal which should be projected onto the curve
     * @param alignTangentsWithPath (optional, default false) : boolean, if true the tangents will be aligned with the path
     * @returns the same object updated.
     */
    update(path: Vector3[], firstNormal?: Nullable<Vector3>, alignTangentsWithPath?: boolean): Path3D;
    private _compute;
    private _getFirstNonNullVector;
    private _getLastNonNullVector;
    private _normalVector;
    /**
     * Updates the point at data for an interpolated point along this curve
     * @param position the position of the point along this curve, from 0.0 to 1.0
     * @param interpolateTNB
     * @interpolateTNB whether to compute the interpolated tangent, normal and binormal
     * @returns the (updated) point at data
     */
    private _updatePointAtData;
    /**
     * Updates the point at data from the specified parameters
     * @param position where along the path the interpolated point is, from 0.0 to 1.0
     * @param subPosition
     * @param point the interpolated point
     * @param parentIndex the index of an existing curve point that is on, or else positionally the first behind, the interpolated point
     * @param interpolateTNB
     */
    private _setPointAtData;
    /**
     * Updates the point at interpolation matrix for the tangents, normals and binormals
     */
    private _updateInterpolationMatrix;
}
/**
 * A Curve3 object is a logical object, so not a mesh, to handle curves in the 3D geometric space.
 * A Curve3 is designed from a series of successive Vector3.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/drawCurves
 */
export class Curve3 {
    private _points;
    private _length;
    /**
     * Returns a Curve3 object along a Quadratic Bezier curve : https://doc.babylonjs.com/features/featuresDeepDive/mesh/drawCurves#quadratic-bezier-curve
     * @param v0 (Vector3) the origin point of the Quadratic Bezier
     * @param v1 (Vector3) the control point
     * @param v2 (Vector3) the end point of the Quadratic Bezier
     * @param nbPoints (integer) the wanted number of points in the curve
     * @returns the created Curve3
     */
    static CreateQuadraticBezier(v0: DeepImmutable<Vector3>, v1: DeepImmutable<Vector3>, v2: DeepImmutable<Vector3>, nbPoints: number): Curve3;
    /**
     * Returns a Curve3 object along a Cubic Bezier curve : https://doc.babylonjs.com/features/featuresDeepDive/mesh/drawCurves#cubic-bezier-curve
     * @param v0 (Vector3) the origin point of the Cubic Bezier
     * @param v1 (Vector3) the first control point
     * @param v2 (Vector3) the second control point
     * @param v3 (Vector3) the end point of the Cubic Bezier
     * @param nbPoints (integer) the wanted number of points in the curve
     * @returns the created Curve3
     */
    static CreateCubicBezier(v0: DeepImmutable<Vector3>, v1: DeepImmutable<Vector3>, v2: DeepImmutable<Vector3>, v3: DeepImmutable<Vector3>, nbPoints: number): Curve3;
    /**
     * Returns a Curve3 object along a Hermite Spline curve : https://doc.babylonjs.com/features/featuresDeepDive/mesh/drawCurves#hermite-spline
     * @param p1 (Vector3) the origin point of the Hermite Spline
     * @param t1 (Vector3) the tangent vector at the origin point
     * @param p2 (Vector3) the end point of the Hermite Spline
     * @param t2 (Vector3) the tangent vector at the end point
     * @param nSeg (integer) the number of curve segments or nSeg + 1 points in the array
     * @returns the created Curve3
     */
    static CreateHermiteSpline(p1: DeepImmutable<Vector3>, t1: DeepImmutable<Vector3>, p2: DeepImmutable<Vector3>, t2: DeepImmutable<Vector3>, nSeg: number): Curve3;
    /**
     * Returns a Curve3 object along a CatmullRom Spline curve :
     * @param points (array of Vector3) the points the spline must pass through. At least, four points required
     * @param nbPoints (integer) the wanted number of points between each curve control points
     * @param closed (boolean) optional with default false, when true forms a closed loop from the points
     * @returns the created Curve3
     */
    static CreateCatmullRomSpline(points: DeepImmutable<Vector3[]>, nbPoints: number, closed?: boolean): Curve3;
    /**
     * Returns a Curve3 object along an arc through three vector3 points:
     * The three points should not be colinear. When they are the Curve3 is empty.
     * @param first (Vector3) the first point the arc must pass through.
     * @param second (Vector3) the second point the arc must pass through.
     * @param third (Vector3) the third point the arc must pass through.
     * @param steps (number) the larger the number of steps the more detailed the arc.
     * @param closed (boolean) optional with default false, when true forms the chord from the first and third point
     * @param fullCircle Circle (boolean) optional with default false, when true forms the complete circle through the three points
     * @returns the created Curve3
     */
    static ArcThru3Points(first: Vector3, second: Vector3, third: Vector3, steps?: number, closed?: boolean, fullCircle?: boolean): Curve3;
    /**
     * A Curve3 object is a logical object, so not a mesh, to handle curves in the 3D geometric space.
     * A Curve3 is designed from a series of successive Vector3.
     * Tuto : https://doc.babylonjs.com/features/featuresDeepDive/mesh/drawCurves#curve3-object
     * @param points points which make up the curve
     */
    constructor(points: Vector3[]);
    /**
     * @returns the Curve3 stored array of successive Vector3
     */
    getPoints(): Vector3[];
    /**
     * @returns the computed length (float) of the curve.
     */
    length(): number;
    /**
     * Returns a new instance of Curve3 object : var curve = curveA.continue(curveB);
     * This new Curve3 is built by translating and sticking the curveB at the end of the curveA.
     * curveA and curveB keep unchanged.
     * @param curve the curve to continue from this curve
     * @returns the newly constructed curve
     */
    continue(curve: DeepImmutable<Curve3>): Curve3;
    private _computeLength;
}

}
declare module "babylonjs/Maths/math.plane" {
import { DeepImmutable } from "babylonjs/types";
import { Vector3, Matrix } from "babylonjs/Maths/math.vector";
/**
 * Represents a plane by the equation ax + by + cz + d = 0
 */
export class Plane {
    private static _TmpMatrix;
    /**
     * Normal of the plane (a,b,c)
     */
    normal: Vector3;
    /**
     * d component of the plane
     */
    d: number;
    /**
     * Creates a Plane object according to the given floats a, b, c, d and the plane equation : ax + by + cz + d = 0
     * @param a a component of the plane
     * @param b b component of the plane
     * @param c c component of the plane
     * @param d d component of the plane
     */
    constructor(a: number, b: number, c: number, d: number);
    /**
     * @returns the plane coordinates as a new array of 4 elements [a, b, c, d].
     */
    asArray(): number[];
    /**
     * @returns a new plane copied from the current Plane.
     */
    clone(): Plane;
    /**
     * @returns the string "Plane".
     */
    getClassName(): string;
    /**
     * @returns the Plane hash code.
     */
    getHashCode(): number;
    /**
     * Normalize the current Plane in place.
     * @returns the updated Plane.
     */
    normalize(): Plane;
    /**
     * Applies a transformation the plane and returns the result
     * @param transformation the transformation matrix to be applied to the plane
     * @returns a new Plane as the result of the transformation of the current Plane by the given matrix.
     */
    transform(transformation: DeepImmutable<Matrix>): Plane;
    /**
     * Compute the dot product between the point and the plane normal
     * @param point point to calculate the dot product with
     * @returns the dot product (float) of the point coordinates and the plane normal.
     */
    dotCoordinate(point: DeepImmutable<Vector3>): number;
    /**
     * Updates the current Plane from the plane defined by the three given points.
     * @param point1 one of the points used to construct the plane
     * @param point2 one of the points used to construct the plane
     * @param point3 one of the points used to construct the plane
     * @returns the updated Plane.
     */
    copyFromPoints(point1: DeepImmutable<Vector3>, point2: DeepImmutable<Vector3>, point3: DeepImmutable<Vector3>): Plane;
    /**
     * Checks if the plane is facing a given direction (meaning if the plane's normal is pointing in the opposite direction of the given vector).
     * Note that for this function to work as expected you should make sure that:
     *   - direction and the plane normal are normalized
     *   - epsilon is a number just bigger than -1, something like -0.99 for eg
     * @param direction the direction to check if the plane is facing
     * @param epsilon value the dot product is compared against (returns true if dot <= epsilon)
     * @returns True if the plane is facing the given direction
     */
    isFrontFacingTo(direction: DeepImmutable<Vector3>, epsilon: number): boolean;
    /**
     * Calculates the distance to a point
     * @param point point to calculate distance to
     * @returns the signed distance (float) from the given point to the Plane.
     */
    signedDistanceTo(point: DeepImmutable<Vector3>): number;
    /**
     * Creates a plane from an  array
     * @param array the array to create a plane from
     * @returns a new Plane from the given array.
     */
    static FromArray(array: DeepImmutable<ArrayLike<number>>): Plane;
    /**
     * Creates a plane from three points
     * @param point1 point used to create the plane
     * @param point2 point used to create the plane
     * @param point3 point used to create the plane
     * @returns a new Plane defined by the three given points.
     */
    static FromPoints(point1: DeepImmutable<Vector3>, point2: DeepImmutable<Vector3>, point3: DeepImmutable<Vector3>): Plane;
    /**
     * Creates a plane from an origin point and a normal
     * @param origin origin of the plane to be constructed
     * @param normal normal of the plane to be constructed
     * @returns a new Plane the normal vector to this plane at the given origin point.
     * Note : the vector "normal" is updated because normalized.
     */
    static FromPositionAndNormal(origin: DeepImmutable<Vector3>, normal: Vector3): Plane;
    /**
     * Calculates the distance from a plane and a point
     * @param origin origin of the plane to be constructed
     * @param normal normal of the plane to be constructed
     * @param point point to calculate distance to
     * @returns the signed distance between the plane defined by the normal vector at the "origin"" point and the given other point.
     */
    static SignedDistanceToPlaneFromPositionAndNormal(origin: DeepImmutable<Vector3>, normal: DeepImmutable<Vector3>, point: DeepImmutable<Vector3>): number;
}

}
declare module "babylonjs/Maths/math.polar" {
import { DeepImmutable } from "babylonjs/types";
import { Vector2, Vector3 } from "babylonjs/Maths/math.vector";
/**
 * Class used to store (r, theta) vector representation
 */
export class Polar {
    radius: number;
    theta: number;
    /**
     * Creates a new Polar object
     * @param radius the radius of the vector
     * @param theta the angle of the vector
     */
    constructor(radius: number, theta: number);
    /**
     * Gets the class name
     * @returns the string "Polar"
     */
    getClassName(): string;
    /**
     * Converts the current polar to a string
     * @returns the current polar as a string
     */
    toString(): string;
    /**
     * Converts the current polar to an array
     * @reutrns the current polar as an array
     */
    asArray(): number[];
    /**
     * Adds the current Polar and the given Polar and stores the result
     * @param polar the polar to add
     * @param ref the polar to store the result in
     * @returns the updated ref
     */
    addToRef(polar: Polar, ref: Polar): Polar;
    /**
     * Adds the current Polar and the given Polar
     * @param polar the polar to add
     * @returns the sum polar
     */
    add(polar: Polar): Polar;
    /**
     * Adds the given polar to the current polar
     * @param polar the polar to add
     * @returns the current polar
     */
    addInPlace(polar: Polar): this;
    /**
     * Adds the provided values to the current polar
     * @param radius the amount to add to the radius
     * @param theta the amount to add to the theta
     * @returns the current polar
     */
    addInPlaceFromFloats(radius: number, theta: number): this;
    /**
     * Subtracts the given Polar from the current Polar and stores the result
     * @param polar the polar to subtract
     * @param ref the polar to store the result in
     * @returns the updated ref
     */
    subtractToRef(polar: Polar, ref: Polar): Polar;
    /**
     * Subtracts the given Polar from the current Polar
     * @param polar the polar to subtract
     * @returns the difference polar
     */
    subtract(polar: Polar): Polar;
    /**
     * Subtracts the given Polar from the current Polar
     * @param polar the polar to subtract
     * @returns the current polar
     */
    subtractInPlace(polar: Polar): this;
    /**
     * Subtracts the given floats from the current polar
     * @param radius the amount to subtract from the radius
     * @param theta the amount to subtract from the theta
     * @param ref the polar to store the result in
     * @returns the updated ref
     */
    subtractFromFloatsToRef(radius: number, theta: number, ref: Polar): Polar;
    /**
     * Subtracts the given floats from the current polar
     * @param radius the amount to subtract from the radius
     * @param theta the amount to subtract from the theta
     * @returns the difference polar
     */
    subtractFromFloats(radius: number, theta: number): Polar;
    /**
     * Multiplies the given Polar with the current Polar and stores the result
     * @param polar the polar to multiply
     * @param ref the polar to store the result in
     * @returns the updated ref
     */
    multiplyToRef(polar: Polar, ref: Polar): Polar;
    /**
     * Multiplies the given Polar with the current Polar
     * @param polar the polar to multiply
     * @returns the product polar
     */
    multiply(polar: Polar): Polar;
    /**
     * Multiplies the given Polar with the current Polar
     * @param polar the polar to multiply
     * @returns the current polar
     */
    multiplyInPlace(polar: Polar): this;
    /**
     * Divides the current Polar by the given Polar and stores the result
     * @param polar the polar to divide
     * @param ref the polar to store the result in
     * @returns the updated ref
     */
    divideToRef(polar: Polar, ref: Polar): Polar;
    /**
     * Divides the current Polar by the given Polar
     * @param polar the polar to divide
     * @returns the quotient polar
     */
    divide(polar: Polar): Polar;
    /**
     * Divides the current Polar by the given Polar
     * @param polar the polar to divide
     * @returns the current polar
     */
    divideInPlace(polar: Polar): this;
    /**
     * Clones the current polar
     * @returns a clone of the current polar
     */
    clone(): Polar;
    /**
     * Copies the source polar into the current polar
     * @param source the polar to copy from
     * @returns the current polar
     */
    copyFrom(source: Polar): this;
    /**
     * Copies the given values into the current polar
     * @param radius the radius to use
     * @param theta the theta to use
     * @returns the current polar
     */
    copyFromFloats(radius: number, theta: number): this;
    /**
     * Scales the current polar and stores the result
     * @param scale defines the multiplication factor
     * @param ref where to store the result
     * @returns the updated ref
     */
    scaleToRef(scale: number, ref: Polar): Polar;
    /**
     * Scales the current polar and returns a new polar with the scaled coordinates
     * @param scale defines the multiplication factor
     * @returns the scaled polar
     */
    scale(scale: number): Polar;
    /**
     * Scales the current polar
     * @param scale defines the multiplication factor
     * @returns the current polar
     */
    scaleInPlace(scale: number): this;
    /**
     * Sets the values of the current polar
     * @param radius the new radius
     * @param theta the new theta
     * @returns the current polar
     */
    set(radius: number, theta: number): this;
    /**
     * Sets the values of the current polar
     * @param value the new values
     * @returns the current polar
     */
    setAll(value: number): this;
    /**
     * Gets the rectangular coordinates of the current Polar
     * @param ref the reference to assign the result
     * @returns the updated reference
     */
    toVector2ToRef(ref: Vector2): Vector2;
    /**
     * Gets the rectangular coordinates of the current Polar
     * @returns the rectangular coordinates
     */
    toVector2(): Vector2;
    /**
     * Converts a given Vector2 to its polar coordinates
     * @param v the Vector2 to convert
     * @param ref the reference to assign the result
     * @returns the updated reference
     */
    static FromVector2ToRef(v: Vector2, ref: Polar): Polar;
    /**
     * Converts a given Vector2 to its polar coordinates
     * @param v the Vector2 to convert
     * @returns a Polar
     */
    static FromVector2(v: Vector2): Polar;
    /**
     * Converts an array of floats to a polar
     * @param array the array to convert
     * @returns the converted polar
     */
    static FromArray(array: number[]): Polar;
}
/**
 * Class used for (radius, theta, phi) vector representation.
 */
export class Spherical {
    radius: number;
    theta: number;
    phi: number;
    /**
     * @param radius spherical radius
     * @param theta angle from positive y axis to radial line from 0 to PI (vertical)
     * @param phi angle from positive x axis measured anticlockwise from -PI to PI (horizontal)
     */
    constructor(radius: number, theta: number, phi: number);
    /**
     * Gets the class name
     * @returns the string "Spherical"
     */
    getClassName(): string;
    /**
     * Converts the current spherical to a string
     * @returns the current spherical as a string
     */
    toString(): string;
    /**
     * Converts the current spherical to an array
     * @reutrns the current spherical as an array
     */
    asArray(): number[];
    /**
     * Adds the current Spherical and the given Spherical and stores the result
     * @param spherical the spherical to add
     * @param ref the spherical to store the result in
     * @returns the updated ref
     */
    addToRef(spherical: Spherical, ref: Spherical): Spherical;
    /**
     * Adds the current Spherical and the given Spherical
     * @param spherical the spherical to add
     * @returns the sum spherical
     */
    add(spherical: Spherical): Spherical;
    /**
     * Adds the given spherical to the current spherical
     * @param spherical the spherical to add
     * @returns the current spherical
     */
    addInPlace(spherical: Spherical): this;
    /**
     * Adds the provided values to the current spherical
     * @param radius the amount to add to the radius
     * @param theta the amount to add to the theta
     * @param phi the amount to add to the phi
     * @returns the current spherical
     */
    addInPlaceFromFloats(radius: number, theta: number, phi: number): this;
    /**
     * Subtracts the given Spherical from the current Spherical and stores the result
     * @param spherical the spherical to subtract
     * @param ref the spherical to store the result in
     * @returns the updated ref
     */
    subtractToRef(spherical: Spherical, ref: Spherical): Spherical;
    /**
     * Subtracts the given Spherical from the current Spherical
     * @param spherical the spherical to subtract
     * @returns the difference spherical
     */
    subtract(spherical: Spherical): Spherical;
    /**
     * Subtracts the given Spherical from the current Spherical
     * @param spherical the spherical to subtract
     * @returns the current spherical
     */
    subtractInPlace(spherical: Spherical): this;
    /**
     * Subtracts the given floats from the current spherical
     * @param radius the amount to subtract from the radius
     * @param theta the amount to subtract from the theta
     * @param phi the amount to subtract from the phi
     * @param ref the spherical to store the result in
     * @returns the updated ref
     */
    subtractFromFloatsToRef(radius: number, theta: number, phi: number, ref: Spherical): Spherical;
    /**
     * Subtracts the given floats from the current spherical
     * @param radius the amount to subtract from the radius
     * @param theta the amount to subtract from the theta
     * @param phi the amount to subtract from the phi
     * @returns the difference spherical
     */
    subtractFromFloats(radius: number, theta: number, phi: number): Spherical;
    /**
     * Multiplies the given Spherical with the current Spherical and stores the result
     * @param spherical the spherical to multiply
     * @param ref the spherical to store the result in
     * @returns the updated ref
     */
    multiplyToRef(spherical: Spherical, ref: Spherical): Spherical;
    /**
     * Multiplies the given Spherical with the current Spherical
     * @param spherical the spherical to multiply
     * @returns the product spherical
     */
    multiply(spherical: Spherical): Spherical;
    /**
     * Multiplies the given Spherical with the current Spherical
     * @param spherical the spherical to multiply
     * @returns the current spherical
     */
    multiplyInPlace(spherical: Spherical): this;
    /**
     * Divides the current Spherical by the given Spherical and stores the result
     * @param spherical the spherical to divide
     * @param ref the spherical to store the result in
     * @returns the updated ref
     */
    divideToRef(spherical: Spherical, ref: Spherical): Spherical;
    /**
     * Divides the current Spherical by the given Spherical
     * @param spherical the spherical to divide
     * @returns the quotient spherical
     */
    divide(spherical: Spherical): Spherical;
    /**
     * Divides the current Spherical by the given Spherical
     * @param spherical the spherical to divide
     * @returns the current spherical
     */
    divideInPlace(spherical: Spherical): this;
    /**
     * Clones the current spherical
     * @returns a clone of the current spherical
     */
    clone(): Spherical;
    /**
     * Copies the source spherical into the current spherical
     * @param source the spherical to copy from
     * @returns the current spherical
     */
    copyFrom(source: Spherical): this;
    /**
     * Copies the given values into the current spherical
     * @param radius the radius to use
     * @param theta the theta to use
     * @param phi the phi to use
     * @returns the current spherical
     */
    copyFromFloats(radius: number, theta: number, phi: number): this;
    /**
     * Scales the current spherical and stores the result
     * @param scale defines the multiplication factor
     * @param ref where to store the result
     * @returns the updated ref
     */
    scaleToRef(scale: number, ref: Spherical): Spherical;
    /**
     * Scales the current spherical and returns a new spherical with the scaled coordinates
     * @param scale defines the multiplication factor
     * @returns the scaled spherical
     */
    scale(scale: number): Spherical;
    /**
     * Scales the current spherical
     * @param scale defines the multiplication factor
     * @returns the current spherical
     */
    scaleInPlace(scale: number): this;
    /**
     * Sets the values of the current spherical
     * @param radius the new radius
     * @param theta the new theta
     * @param phi the new phi
     * @returns the current spherical
     */
    set(radius: number, theta: number, phi: number): this;
    /**
     * Sets the values of the current spherical
     * @param value the new values
     * @returns the current spherical
     */
    setAll(value: number): this;
    /**
     * Assigns the rectangular coordinates of the current Spherical to a Vector3
     * @param ref the Vector3 to update
     * @returns the updated Vector3
     */
    toVector3ToRef(ref: DeepImmutable<Vector3>): Vector3;
    /**
     * Gets a Vector3 from the current spherical coordinates
     * @returns the (x, y,z) form of the current Spherical
     */
    toVector3(): Vector3;
    /**
     * Assigns the spherical coordinates from a Vector3
     * @param vector the vector to convert
     * @param ref the Spherical to update
     * @returns the updated ref
     */
    static FromVector3ToRef(vector: DeepImmutable<Vector3>, ref: Spherical): Spherical;
    /**
     * Gets a Spherical from a Vector3
     * @param vector defines the vector in (x, y, z) coordinate space
     * @returns a new Spherical
     */
    static FromVector3(vector: DeepImmutable<Vector3>): Spherical;
    /**
     * Converts an array of floats to a spherical
     * @param array the array to convert
     * @returns the converted spherical
     */
    static FromArray(array: number[]): Spherical;
}

}
declare module "babylonjs/Maths/math.scalar" {
/**
 * Scalar computation library
 */
export class Scalar {
    /**
     * Two pi constants convenient for computation.
     */
    static TwoPi: number;
    /**
     * Boolean : true if the absolute difference between a and b is lower than epsilon (default = 1.401298E-45)
     * @param a number
     * @param b number
     * @param epsilon (default = 1.401298E-45)
     * @returns true if the absolute difference between a and b is lower than epsilon (default = 1.401298E-45)
     */
    static WithinEpsilon(a: number, b: number, epsilon?: number): boolean;
    /**
     * Returns a string : the upper case translation of the number i to hexadecimal.
     * @param i number
     * @returns the upper case translation of the number i to hexadecimal.
     */
    static ToHex(i: number): string;
    /**
     * Returns -1 if value is negative and +1 is value is positive.
     * @param value the value
     * @returns the value itself if it's equal to zero.
     */
    static Sign(value: number): number;
    /**
     * Returns the value itself if it's between min and max.
     * Returns min if the value is lower than min.
     * Returns max if the value is greater than max.
     * @param value the value to clmap
     * @param min the min value to clamp to (default: 0)
     * @param max the max value to clamp to (default: 1)
     * @returns the clamped value
     */
    static Clamp(value: number, min?: number, max?: number): number;
    /**
     * the log2 of value.
     * @param value the value to compute log2 of
     * @returns the log2 of value.
     */
    static Log2(value: number): number;
    /**
     * the floor part of a log2 value.
     * @param value the value to compute log2 of
     * @returns the log2 of value.
     */
    static ILog2(value: number): number;
    /**
     * Loops the value, so that it is never larger than length and never smaller than 0.
     *
     * This is similar to the modulo operator but it works with floating point numbers.
     * For example, using 3.0 for t and 2.5 for length, the result would be 0.5.
     * With t = 5 and length = 2.5, the result would be 0.0.
     * Note, however, that the behaviour is not defined for negative numbers as it is for the modulo operator
     * @param value the value
     * @param length the length
     * @returns the looped value
     */
    static Repeat(value: number, length: number): number;
    /**
     * Normalize the value between 0.0 and 1.0 using min and max values
     * @param value value to normalize
     * @param min max to normalize between
     * @param max min to normalize between
     * @returns the normalized value
     */
    static Normalize(value: number, min: number, max: number): number;
    /**
     * Denormalize the value from 0.0 and 1.0 using min and max values
     * @param normalized value to denormalize
     * @param min max to denormalize between
     * @param max min to denormalize between
     * @returns the denormalized value
     */
    static Denormalize(normalized: number, min: number, max: number): number;
    /**
     * Calculates the shortest difference between two given angles given in degrees.
     * @param current current angle in degrees
     * @param target target angle in degrees
     * @returns the delta
     */
    static DeltaAngle(current: number, target: number): number;
    /**
     * PingPongs the value t, so that it is never larger than length and never smaller than 0.
     * @param tx value
     * @param length length
     * @returns The returned value will move back and forth between 0 and length
     */
    static PingPong(tx: number, length: number): number;
    /**
     * Interpolates between min and max with smoothing at the limits.
     *
     * This function interpolates between min and max in a similar way to Lerp. However, the interpolation will gradually speed up
     * from the start and slow down toward the end. This is useful for creating natural-looking animation, fading and other transitions.
     * @param from from
     * @param to to
     * @param tx value
     * @returns the smooth stepped value
     */
    static SmoothStep(from: number, to: number, tx: number): number;
    /**
     * Moves a value current towards target.
     *
     * This is essentially the same as Mathf.Lerp but instead the function will ensure that the speed never exceeds maxDelta.
     * Negative values of maxDelta pushes the value away from target.
     * @param current current value
     * @param target target value
     * @param maxDelta max distance to move
     * @returns resulting value
     */
    static MoveTowards(current: number, target: number, maxDelta: number): number;
    /**
     * Same as MoveTowards but makes sure the values interpolate correctly when they wrap around 360 degrees.
     *
     * Variables current and target are assumed to be in degrees. For optimization reasons, negative values of maxDelta
     *  are not supported and may cause oscillation. To push current away from a target angle, add 180 to that angle instead.
     * @param current current value
     * @param target target value
     * @param maxDelta max distance to move
     * @returns resulting angle
     */
    static MoveTowardsAngle(current: number, target: number, maxDelta: number): number;
    /**
     * Creates a new scalar with values linearly interpolated of "amount" between the start scalar and the end scalar.
     * @param start start value
     * @param end target value
     * @param amount amount to lerp between
     * @returns the lerped value
     */
    static Lerp(start: number, end: number, amount: number): number;
    /**
     * Same as Lerp but makes sure the values interpolate correctly when they wrap around 360 degrees.
     * The parameter t is clamped to the range [0, 1]. Variables a and b are assumed to be in degrees.
     * @param start start value
     * @param end target value
     * @param amount amount to lerp between
     * @returns the lerped value
     */
    static LerpAngle(start: number, end: number, amount: number): number;
    /**
     * Calculates the linear parameter t that produces the interpolant value within the range [a, b].
     * @param a start value
     * @param b target value
     * @param value value between a and b
     * @returns the inverseLerp value
     */
    static InverseLerp(a: number, b: number, value: number): number;
    /**
     * Returns a new scalar located for "amount" (float) on the Hermite spline defined by the scalars "value1", "value3", "tangent1", "tangent2".
     * @see http://mathworld.wolfram.com/HermitePolynomial.html
     * @param value1 defines the first control point
     * @param tangent1 defines the first tangent
     * @param value2 defines the second control point
     * @param tangent2 defines the second tangent
     * @param amount defines the amount on the interpolation spline (between 0 and 1)
     * @returns hermite result
     */
    static Hermite(value1: number, tangent1: number, value2: number, tangent2: number, amount: number): number;
    /**
     * Returns a new scalar which is the 1st derivative of the Hermite spline defined by the scalars "value1", "value2", "tangent1", "tangent2".
     * @param value1 defines the first control point
     * @param tangent1 defines the first tangent
     * @param value2 defines the second control point
     * @param tangent2 defines the second tangent
     * @param time define where the derivative must be done
     * @returns 1st derivative
     */
    static Hermite1stDerivative(value1: number, tangent1: number, value2: number, tangent2: number, time: number): number;
    /**
     * Returns a random float number between and min and max values
     * @param min min value of random
     * @param max max value of random
     * @returns random value
     */
    static RandomRange(min: number, max: number): number;
    /**
     * This function returns percentage of a number in a given range.
     *
     * RangeToPercent(40,20,60) will return 0.5 (50%)
     * RangeToPercent(34,0,100) will return 0.34 (34%)
     * @param number to convert to percentage
     * @param min min range
     * @param max max range
     * @returns the percentage
     */
    static RangeToPercent(number: number, min: number, max: number): number;
    /**
     * This function returns number that corresponds to the percentage in a given range.
     *
     * PercentToRange(0.34,0,100) will return 34.
     * @param percent to convert to number
     * @param min min range
     * @param max max range
     * @returns the number
     */
    static PercentToRange(percent: number, min: number, max: number): number;
    /**
     * Returns the angle converted to equivalent value between -Math.PI and Math.PI radians.
     * @param angle The angle to normalize in radian.
     * @returns The converted angle.
     */
    static NormalizeRadians(angle: number): number;
    /**
     * Returns the highest common factor of two integers.
     * @param a first parameter
     * @param b second parameter
     * @returns HCF of a and b
     */
    static HCF(a: number, b: number): number;
}

}
declare module "babylonjs/Maths/math.size" {
/**
 * Interface for the size containing width and height
 */
export interface ISize {
    /**
     * Width
     */
    width: number;
    /**
     * Height
     */
    height: number;
}
/**
 * Size containing width and height
 */
export class Size implements ISize {
    /**
     * Width
     */
    width: number;
    /**
     * Height
     */
    height: number;
    /**
     * Creates a Size object from the given width and height (floats).
     * @param width width of the new size
     * @param height height of the new size
     */
    constructor(width: number, height: number);
    /**
     * Returns a string with the Size width and height
     * @returns a string with the Size width and height
     */
    toString(): string;
    /**
     * "Size"
     * @returns the string "Size"
     */
    getClassName(): string;
    /**
     * Returns the Size hash code.
     * @returns a hash code for a unique width and height
     */
    getHashCode(): number;
    /**
     * Updates the current size from the given one.
     * @param src the given size
     */
    copyFrom(src: Size): void;
    /**
     * Updates in place the current Size from the given floats.
     * @param width width of the new size
     * @param height height of the new size
     * @returns the updated Size.
     */
    copyFromFloats(width: number, height: number): Size;
    /**
     * Updates in place the current Size from the given floats.
     * @param width width to set
     * @param height height to set
     * @returns the updated Size.
     */
    set(width: number, height: number): Size;
    /**
     * Multiplies the width and height by numbers
     * @param w factor to multiple the width by
     * @param h factor to multiple the height by
     * @returns a new Size set with the multiplication result of the current Size and the given floats.
     */
    multiplyByFloats(w: number, h: number): Size;
    /**
     * Clones the size
     * @returns a new Size copied from the given one.
     */
    clone(): Size;
    /**
     * True if the current Size and the given one width and height are strictly equal.
     * @param other the other size to compare against
     * @returns True if the current Size and the given one width and height are strictly equal.
     */
    equals(other: Size): boolean;
    /**
     * The surface of the Size : width * height (float).
     */
    get surface(): number;
    /**
     * Create a new size of zero
     * @returns a new Size set to (0.0, 0.0)
     */
    static Zero(): Size;
    /**
     * Sums the width and height of two sizes
     * @param otherSize size to add to this size
     * @returns a new Size set as the addition result of the current Size and the given one.
     */
    add(otherSize: Size): Size;
    /**
     * Subtracts the width and height of two
     * @param otherSize size to subtract to this size
     * @returns a new Size set as the subtraction result of  the given one from the current Size.
     */
    subtract(otherSize: Size): Size;
    /**
     * Creates a new Size set at the linear interpolation "amount" between "start" and "end"
     * @param start starting size to lerp between
     * @param end end size to lerp between
     * @param amount amount to lerp between the start and end values
     * @returns a new Size set at the linear interpolation "amount" between "start" and "end"
     */
    static Lerp(start: Size, end: Size, amount: number): Size;
}

}
declare module "babylonjs/Maths/math.vector" {
import { Viewport } from "babylonjs/Maths/math.viewport";
import { DeepImmutable, Nullable, FloatArray, float } from "babylonjs/types";
import { IPlaneLike } from "babylonjs/Maths/math.like";
import { Plane } from "babylonjs/Maths/math.plane";
import { TransformNode } from "babylonjs/Meshes/transformNode";
export type Vector2Constructor<T extends Vector2> = new (...args: ConstructorParameters<typeof Vector2>) => T;
export type Vector3Constructor<T extends Vector3> = new (...args: ConstructorParameters<typeof Vector3>) => T;
export type Vector4Constructor<T extends Vector4> = new (...args: ConstructorParameters<typeof Vector4>) => T;
export type QuaternionConstructor<T extends Quaternion> = new (...args: ConstructorParameters<typeof Quaternion>) => T;
export type MatrixConstructor<T extends Matrix> = new () => T;
/**
 * Class representing a vector containing 2 coordinates
 * Example Playground - Overview -  https://playground.babylonjs.com/#QYBWV4#9
 */
export class Vector2 {
    /** defines the first coordinate */
    x: number;
    /** defines the second coordinate */
    y: number;
    private static _ZeroReadOnly;
    /**
     * Creates a new Vector2 from the given x and y coordinates
     * @param x defines the first coordinate
     * @param y defines the second coordinate
     */
    constructor(
    /** defines the first coordinate */
    x?: number, 
    /** defines the second coordinate */
    y?: number);
    /**
     * Gets a string with the Vector2 coordinates
     * @returns a string with the Vector2 coordinates
     */
    toString(): string;
    /**
     * Gets class name
     * @returns the string "Vector2"
     */
    getClassName(): string;
    /**
     * Gets current vector hash code
     * @returns the Vector2 hash code as a number
     */
    getHashCode(): number;
    /**
     * Sets the Vector2 coordinates in the given array or Float32Array from the given index.
     * Example Playground https://playground.babylonjs.com/#QYBWV4#15
     * @param array defines the source array
     * @param index defines the offset in source array
     * @returns the current Vector2
     */
    toArray(array: FloatArray, index?: number): this;
    /**
     * Update the current vector from an array
     * Example Playground https://playground.babylonjs.com/#QYBWV4#39
     * @param array defines the destination array
     * @param index defines the offset in the destination array
     * @returns the current Vector2
     */
    fromArray(array: FloatArray, index?: number): this;
    /**
     * Copy the current vector to an array
     * Example Playground https://playground.babylonjs.com/#QYBWV4#40
     * @returns a new array with 2 elements: the Vector2 coordinates.
     */
    asArray(): number[];
    /**
     * Sets the Vector2 coordinates with the given Vector2 coordinates
     * Example Playground https://playground.babylonjs.com/#QYBWV4#24
     * @param source defines the source Vector2
     * @returns the current updated Vector2
     */
    copyFrom(source: DeepImmutable<Vector2>): this;
    /**
     * Sets the Vector2 coordinates with the given floats
     * Example Playground https://playground.babylonjs.com/#QYBWV4#25
     * @param x defines the first coordinate
     * @param y defines the second coordinate
     * @returns the current updated Vector2
     */
    copyFromFloats(x: number, y: number): this;
    /**
     * Sets the Vector2 coordinates with the given floats
     * Example Playground https://playground.babylonjs.com/#QYBWV4#62
     * @param x defines the first coordinate
     * @param y defines the second coordinate
     * @returns the current updated Vector2
     */
    set(x: number, y: number): this;
    /**
     * Add another vector with the current one
     * Example Playground https://playground.babylonjs.com/#QYBWV4#11
     * @param otherVector defines the other vector
     * @returns a new Vector2 set with the addition of the current Vector2 and the given one coordinates
     */
    add(otherVector: DeepImmutable<Vector2>): this;
    /**
     * Sets the "result" coordinates with the addition of the current Vector2 and the given one coordinates
     * Example Playground https://playground.babylonjs.com/#QYBWV4#12
     * @param otherVector defines the other vector
     * @param result defines the target vector
     * @returns result input
     */
    addToRef<T extends Vector2>(otherVector: DeepImmutable<Vector2>, result: T): T;
    /**
     * Set the Vector2 coordinates by adding the given Vector2 coordinates
     * Example Playground https://playground.babylonjs.com/#QYBWV4#13
     * @param otherVector defines the other vector
     * @returns the current updated Vector2
     */
    addInPlace(otherVector: DeepImmutable<Vector2>): this;
    /**
     * Gets a new Vector2 by adding the current Vector2 coordinates to the given Vector3 x, y coordinates
     * Example Playground https://playground.babylonjs.com/#QYBWV4#14
     * @param otherVector defines the other vector
     * @returns a new Vector2
     */
    addVector3(otherVector: Vector3): this;
    /**
     * Gets a new Vector2 set with the subtracted coordinates of the given one from the current Vector2
     * Example Playground https://playground.babylonjs.com/#QYBWV4#61
     * @param otherVector defines the other vector
     * @returns a new Vector2
     */
    subtract(otherVector: Vector2): this;
    /**
     * Sets the "result" coordinates with the subtraction of the given one from the current Vector2 coordinates.
     * Example Playground https://playground.babylonjs.com/#QYBWV4#63
     * @param otherVector defines the other vector
     * @param result defines the target vector
     * @returns result input
     */
    subtractToRef<T extends Vector2>(otherVector: DeepImmutable<Vector2>, result: T): T;
    /**
     * Sets the current Vector2 coordinates by subtracting from it the given one coordinates
     * Example Playground https://playground.babylonjs.com/#QYBWV4#88
     * @param otherVector defines the other vector
     * @returns the current updated Vector2
     */
    subtractInPlace(otherVector: DeepImmutable<Vector2>): this;
    /**
     * Multiplies in place the current Vector2 coordinates by the given ones
     * Example Playground https://playground.babylonjs.com/#QYBWV4#43
     * @param otherVector defines the other vector
     * @returns the current updated Vector2
     */
    multiplyInPlace(otherVector: DeepImmutable<Vector2>): this;
    /**
     * Returns a new Vector2 set with the multiplication of the current Vector2 and the given one coordinates
     * Example Playground https://playground.babylonjs.com/#QYBWV4#42
     * @param otherVector defines the other vector
     * @returns a new Vector2
     */
    multiply(otherVector: DeepImmutable<Vector2>): this;
    /**
     * Sets "result" coordinates with the multiplication of the current Vector2 and the given one coordinates
     * Example Playground https://playground.babylonjs.com/#QYBWV4#44
     * @param otherVector defines the other vector
     * @param result defines the target vector
     * @returns result input
     */
    multiplyToRef<T extends Vector2>(otherVector: DeepImmutable<Vector2>, result: T): T;
    /**
     * Gets a new Vector2 set with the Vector2 coordinates multiplied by the given floats
     * Example Playground https://playground.babylonjs.com/#QYBWV4#89
     * @param x defines the first coordinate
     * @param y defines the second coordinate
     * @returns a new Vector2
     */
    multiplyByFloats(x: number, y: number): this;
    /**
     * Returns a new Vector2 set with the Vector2 coordinates divided by the given one coordinates
     * Example Playground https://playground.babylonjs.com/#QYBWV4#27
     * @param otherVector defines the other vector
     * @returns a new Vector2
     */
    divide(otherVector: Vector2): this;
    /**
     * Sets the "result" coordinates with the Vector2 divided by the given one coordinates
     * Example Playground https://playground.babylonjs.com/#QYBWV4#30
     * @param otherVector defines the other vector
     * @param result defines the target vector
     * @returns result input
     */
    divideToRef<T extends Vector2>(otherVector: DeepImmutable<Vector2>, result: T): T;
    /**
     * Divides the current Vector2 coordinates by the given ones
     * Example Playground https://playground.babylonjs.com/#QYBWV4#28
     * @param otherVector defines the other vector
     * @returns the current updated Vector2
     */
    divideInPlace(otherVector: DeepImmutable<Vector2>): this;
    /**
     * Gets a new Vector2 with current Vector2 negated coordinates
     * Example Playground https://playground.babylonjs.com/#QYBWV4#22
     * @returns a new Vector2
     */
    negate(): this;
    /**
     * Negate this vector in place
     * Example Playground https://playground.babylonjs.com/#QYBWV4#23
     * @returns this
     */
    negateInPlace(): this;
    /**
     * Negate the current Vector2 and stores the result in the given vector "result" coordinates
     * Example Playground https://playground.babylonjs.com/#QYBWV4#41
     * @param result defines the Vector3 object where to store the result
     * @returns the result
     */
    negateToRef<T extends Vector2>(result: T): T;
    /**
     * Multiply the Vector2 coordinates by
     * Example Playground https://playground.babylonjs.com/#QYBWV4#59
     * @param scale defines the scaling factor
     * @returns the current updated Vector2
     */
    scaleInPlace(scale: number): this;
    /**
     * Returns a new Vector2 scaled by "scale" from the current Vector2
     * Example Playground https://playground.babylonjs.com/#QYBWV4#52
     * @param scale defines the scaling factor
     * @returns a new Vector2
     */
    scale(scale: number): this;
    /**
     * Scale the current Vector2 values by a factor to a given Vector2
     * Example Playground https://playground.babylonjs.com/#QYBWV4#57
     * @param scale defines the scale factor
     * @param result defines the Vector2 object where to store the result
     * @returns result input
     */
    scaleToRef<T extends Vector2>(scale: number, result: T): T;
    /**
     * Scale the current Vector2 values by a factor and add the result to a given Vector2
     * Example Playground https://playground.babylonjs.com/#QYBWV4#58
     * @param scale defines the scale factor
     * @param result defines the Vector2 object where to store the result
     * @returns result input
     */
    scaleAndAddToRef<T extends Vector2>(scale: number, result: T): T;
    /**
     * Gets a boolean if two vectors are equals
     * Example Playground https://playground.babylonjs.com/#QYBWV4#31
     * @param otherVector defines the other vector
     * @returns true if the given vector coordinates strictly equal the current Vector2 ones
     */
    equals(otherVector: DeepImmutable<Vector2>): boolean;
    /**
     * Gets a boolean if two vectors are equals (using an epsilon value)
     * Example Playground https://playground.babylonjs.com/#QYBWV4#32
     * @param otherVector defines the other vector
     * @param epsilon defines the minimal distance to consider equality
     * @returns true if the given vector coordinates are close to the current ones by a distance of epsilon.
     */
    equalsWithEpsilon(otherVector: DeepImmutable<Vector2>, epsilon?: number): boolean;
    /**
     * Gets a new Vector2 from current Vector2 floored values
     * Example Playground https://playground.babylonjs.com/#QYBWV4#35
     * eg (1.2, 2.31) returns (1, 2)
     * @returns a new Vector2
     */
    floor(): this;
    /**
     * Gets a new Vector2 from current Vector2 fractional values
     * Example Playground https://playground.babylonjs.com/#QYBWV4#34
     * eg (1.2, 2.31) returns (0.2, 0.31)
     * @returns a new Vector2
     */
    fract(): this;
    /**
     * Rotate the current vector into a given result vector
     * Example Playground https://playground.babylonjs.com/#QYBWV4#49
     * @param angle defines the rotation angle
     * @param result defines the result vector where to store the rotated vector
     * @returns result input
     */
    rotateToRef<T extends Vector2>(angle: number, result: T): T;
    /**
     * Gets the length of the vector
     * @returns the vector length (float)
     */
    length(): number;
    /**
     * Gets the vector squared length
     * @returns the vector squared length (float)
     */
    lengthSquared(): number;
    /**
     * Normalize the vector
     * Example Playground https://playground.babylonjs.com/#QYBWV4#48
     * @returns the current updated Vector2
     */
    normalize(): this;
    /**
     * Gets a new Vector2 copied from the Vector2
     * Example Playground https://playground.babylonjs.com/#QYBWV4#20
     * @returns a new Vector2
     */
    clone(): this;
    /**
     * Gets a new Vector2(0, 0)
     * @returns a new Vector2
     */
    static Zero(): Vector2;
    /**
     * Gets a new Vector2(1, 1)
     * @returns a new Vector2
     */
    static One(): Vector2;
    /**
     * Gets a zero Vector2 that must not be updated
     */
    static get ZeroReadOnly(): DeepImmutable<Vector2>;
    /**
     * Gets a new Vector2 set from the given index element of the given array
     * Example Playground https://playground.babylonjs.com/#QYBWV4#79
     * @param array defines the data source
     * @param offset defines the offset in the data source
     * @returns a new Vector2
     */
    static FromArray(array: DeepImmutable<ArrayLike<number>>, offset?: number): Vector2;
    /**
     * Sets "result" from the given index element of the given array
     * Example Playground https://playground.babylonjs.com/#QYBWV4#80
     * @param array defines the data source
     * @param offset defines the offset in the data source
     * @param result defines the target vector
     * @returns result input
     */
    static FromArrayToRef<T extends Vector2>(array: DeepImmutable<ArrayLike<number>>, offset: number, result: T): T;
    /**
     * Gets a new Vector2 located for "amount" (float) on the CatmullRom spline defined by the given four Vector2
     * Example Playground https://playground.babylonjs.com/#QYBWV4#65
     * @param value1 defines 1st point of control
     * @param value2 defines 2nd point of control
     * @param value3 defines 3rd point of control
     * @param value4 defines 4th point of control
     * @param amount defines the interpolation factor
     * @returns a new Vector2
     */
    static CatmullRom<T extends Vector2>(value1: DeepImmutable<T>, value2: DeepImmutable<Vector2>, value3: DeepImmutable<Vector2>, value4: DeepImmutable<Vector2>, amount: number): T;
    /**
     * Returns a new Vector2 set with same the coordinates than "value" ones if the vector "value" is in the square defined by "min" and "max".
     * If a coordinate of "value" is lower than "min" coordinates, the returned Vector2 is given this "min" coordinate.
     * If a coordinate of "value" is greater than "max" coordinates, the returned Vector2 is given this "max" coordinate
     * Example Playground https://playground.babylonjs.com/#QYBWV4#76
     * @param value defines the value to clamp
     * @param min defines the lower limit
     * @param max defines the upper limit
     * @returns a new Vector2
     */
    static Clamp<T extends Vector2>(value: DeepImmutable<T>, min: DeepImmutable<Vector2>, max: DeepImmutable<Vector2>): T;
    /**
     * Returns a new Vector2 located for "amount" (float) on the Hermite spline defined by the vectors "value1", "value2", "tangent1", "tangent2"
     * Example Playground https://playground.babylonjs.com/#QYBWV4#81
     * @param value1 defines the 1st control point
     * @param tangent1 defines the outgoing tangent
     * @param value2 defines the 2nd control point
     * @param tangent2 defines the incoming tangent
     * @param amount defines the interpolation factor
     * @returns a new Vector2
     */
    static Hermite<T extends Vector2>(value1: DeepImmutable<T>, tangent1: DeepImmutable<Vector2>, value2: DeepImmutable<Vector2>, tangent2: DeepImmutable<Vector2>, amount: number): T;
    /**
     * Returns a new Vector2 which is the 1st derivative of the Hermite spline defined by the vectors "value1", "value2", "tangent1", "tangent2".
     * Example Playground https://playground.babylonjs.com/#QYBWV4#82
     * @param value1 defines the first control point
     * @param tangent1 defines the first tangent
     * @param value2 defines the second control point
     * @param tangent2 defines the second tangent
     * @param time define where the derivative must be done
     * @returns 1st derivative
     */
    static Hermite1stDerivative<T extends Vector2>(value1: DeepImmutable<T>, tangent1: DeepImmutable<Vector2>, value2: DeepImmutable<Vector2>, tangent2: DeepImmutable<Vector2>, time: number): T;
    /**
     * Returns a new Vector2 which is the 1st derivative of the Hermite spline defined by the vectors "value1", "value2", "tangent1", "tangent2".
     * Example Playground https://playground.babylonjs.com/#QYBWV4#83
     * @param value1 defines the first control point
     * @param tangent1 defines the first tangent
     * @param value2 defines the second control point
     * @param tangent2 defines the second tangent
     * @param time define where the derivative must be done
     * @param result define where the derivative will be stored
     * @returns result input
     */
    static Hermite1stDerivativeToRef<T extends Vector2>(value1: DeepImmutable<Vector2>, tangent1: DeepImmutable<Vector2>, value2: DeepImmutable<Vector2>, tangent2: DeepImmutable<Vector2>, time: number, result: T): T;
    /**
     * Returns a new Vector2 located for "amount" (float) on the linear interpolation between the vector "start" adn the vector "end".
     * Example Playground https://playground.babylonjs.com/#QYBWV4#84
     * @param start defines the start vector
     * @param end defines the end vector
     * @param amount defines the interpolation factor
     * @returns a new Vector2
     */
    static Lerp<T extends Vector2>(start: DeepImmutable<T>, end: DeepImmutable<Vector2>, amount: number): Vector2;
    /**
     * Gets the dot product of the vector "left" and the vector "right"
     * Example Playground https://playground.babylonjs.com/#QYBWV4#90
     * @param left defines first vector
     * @param right defines second vector
     * @returns the dot product (float)
     */
    static Dot(left: DeepImmutable<Vector2>, right: DeepImmutable<Vector2>): number;
    /**
     * Returns a new Vector2 equal to the normalized given vector
     * Example Playground https://playground.babylonjs.com/#QYBWV4#46
     * @param vector defines the vector to normalize
     * @returns a new Vector2
     */
    static Normalize<T extends Vector2>(vector: DeepImmutable<T>): T;
    /**
     * Normalize a given vector into a second one
     * Example Playground https://playground.babylonjs.com/#QYBWV4#50
     * @param vector defines the vector to normalize
     * @param result defines the vector where to store the result
     * @returns result input
     */
    static NormalizeToRef<T extends Vector2>(vector: DeepImmutable<Vector2>, result: T): T;
    /**
     * Gets a new Vector2 set with the minimal coordinate values from the "left" and "right" vectors
     * Example Playground https://playground.babylonjs.com/#QYBWV4#86
     * @param left defines 1st vector
     * @param right defines 2nd vector
     * @returns a new Vector2
     */
    static Minimize<T extends Vector2>(left: DeepImmutable<T>, right: DeepImmutable<Vector2>): T;
    /**
     * Gets a new Vector2 set with the maximal coordinate values from the "left" and "right" vectors
     * Example Playground https://playground.babylonjs.com/#QYBWV4#86
     * @param left defines 1st vector
     * @param right defines 2nd vector
     * @returns a new Vector2
     */
    static Maximize<T extends Vector2>(left: DeepImmutable<T>, right: DeepImmutable<Vector2>): T;
    /**
     * Gets a new Vector2 set with the transformed coordinates of the given vector by the given transformation matrix
     * Example Playground https://playground.babylonjs.com/#QYBWV4#17
     * @param vector defines the vector to transform
     * @param transformation defines the matrix to apply
     * @returns a new Vector2
     */
    static Transform<T extends Vector2>(vector: DeepImmutable<T>, transformation: DeepImmutable<Matrix>): T;
    /**
     * Transforms the given vector coordinates by the given transformation matrix and stores the result in the vector "result" coordinates
     * Example Playground https://playground.babylonjs.com/#QYBWV4#19
     * @param vector defines the vector to transform
     * @param transformation defines the matrix to apply
     * @param result defines the target vector
     * @returns result input
     */
    static TransformToRef<T extends Vector2>(vector: DeepImmutable<Vector2>, transformation: DeepImmutable<Matrix>, result: T): T;
    /**
     * Determines if a given vector is included in a triangle
     * Example Playground https://playground.babylonjs.com/#QYBWV4#87
     * @param p defines the vector to test
     * @param p0 defines 1st triangle point
     * @param p1 defines 2nd triangle point
     * @param p2 defines 3rd triangle point
     * @returns true if the point "p" is in the triangle defined by the vectors "p0", "p1", "p2"
     */
    static PointInTriangle(p: DeepImmutable<Vector2>, p0: DeepImmutable<Vector2>, p1: DeepImmutable<Vector2>, p2: DeepImmutable<Vector2>): boolean;
    /**
     * Gets the distance between the vectors "value1" and "value2"
     * Example Playground https://playground.babylonjs.com/#QYBWV4#71
     * @param value1 defines first vector
     * @param value2 defines second vector
     * @returns the distance between vectors
     */
    static Distance(value1: DeepImmutable<Vector2>, value2: DeepImmutable<Vector2>): number;
    /**
     * Returns the squared distance between the vectors "value1" and "value2"
     * Example Playground https://playground.babylonjs.com/#QYBWV4#72
     * @param value1 defines first vector
     * @param value2 defines second vector
     * @returns the squared distance between vectors
     */
    static DistanceSquared(value1: DeepImmutable<Vector2>, value2: DeepImmutable<Vector2>): number;
    /**
     * Gets a new Vector2 located at the center of the vectors "value1" and "value2"
     * Example Playground https://playground.babylonjs.com/#QYBWV4#86
     * Example Playground https://playground.babylonjs.com/#QYBWV4#66
     * @param value1 defines first vector
     * @param value2 defines second vector
     * @returns a new Vector2
     */
    static Center<T extends Vector2>(value1: DeepImmutable<T>, value2: DeepImmutable<Vector2>): T;
    /**
     * Gets the center of the vectors "value1" and "value2" and stores the result in the vector "ref"
     * Example Playground https://playground.babylonjs.com/#QYBWV4#66
     * @param value1 defines first vector
     * @param value2 defines second vector
     * @param ref defines third vector
     * @returns ref
     */
    static CenterToRef<T extends Vector2>(value1: DeepImmutable<Vector2>, value2: DeepImmutable<Vector2>, ref: T): T;
    /**
     * Gets the shortest distance (float) between the point "p" and the segment defined by the two points "segA" and "segB".
     * Example Playground https://playground.babylonjs.com/#QYBWV4#77
     * @param p defines the middle point
     * @param segA defines one point of the segment
     * @param segB defines the other point of the segment
     * @returns the shortest distance
     */
    static DistanceOfPointFromSegment(p: DeepImmutable<Vector2>, segA: DeepImmutable<Vector2>, segB: DeepImmutable<Vector2>): number;
}
/**
 * Class used to store (x,y,z) vector representation
 * A Vector3 is the main object used in 3D geometry
 * It can represent either the coordinates of a point the space, either a direction
 * Reminder: js uses a left handed forward facing system
 * Example Playground - Overview - https://playground.babylonjs.com/#R1F8YU
 */
export class Vector3 {
    private static _UpReadOnly;
    private static _DownReadOnly;
    private static _LeftHandedForwardReadOnly;
    private static _RightHandedForwardReadOnly;
    private static _RightReadOnly;
    private static _LeftReadOnly;
    private static _ZeroReadOnly;
    /** @internal */
    _x: number;
    /** @internal */
    _y: number;
    /** @internal */
    _z: number;
    /** @internal */
    _isDirty: boolean;
    /** Gets or sets the x coordinate */
    get x(): number;
    set x(value: number);
    /** Gets or sets the y coordinate */
    get y(): number;
    set y(value: number);
    /** Gets or sets the z coordinate */
    get z(): number;
    set z(value: number);
    /**
     * Creates a new Vector3 object from the given x, y, z (floats) coordinates.
     * @param x defines the first coordinates (on X axis)
     * @param y defines the second coordinates (on Y axis)
     * @param z defines the third coordinates (on Z axis)
     */
    constructor(x?: number, y?: number, z?: number);
    /**
     * Creates a string representation of the Vector3
     * Example Playground https://playground.babylonjs.com/#R1F8YU#67
     * @returns a string with the Vector3 coordinates.
     */
    toString(): string;
    /**
     * Gets the class name
     * @returns the string "Vector3"
     */
    getClassName(): string;
    /**
     * Creates the Vector3 hash code
     * @returns a number which tends to be unique between Vector3 instances
     */
    getHashCode(): number;
    /**
     * Creates an array containing three elements : the coordinates of the Vector3
     * Example Playground https://playground.babylonjs.com/#R1F8YU#10
     * @returns a new array of numbers
     */
    asArray(): number[];
    /**
     * Populates the given array or Float32Array from the given index with the successive coordinates of the Vector3
     * Example Playground https://playground.babylonjs.com/#R1F8YU#65
     * @param array defines the destination array
     * @param index defines the offset in the destination array
     * @returns the current Vector3
     */
    toArray(array: FloatArray, index?: number): this;
    /**
     * Update the current vector from an array
     * Example Playground https://playground.babylonjs.com/#R1F8YU#24
     * @param array defines the destination array
     * @param index defines the offset in the destination array
     * @returns the current Vector3
     */
    fromArray(array: FloatArray, index?: number): this;
    /**
     * Converts the current Vector3 into a quaternion (considering that the Vector3 contains Euler angles representation of a rotation)
     * Example Playground https://playground.babylonjs.com/#R1F8YU#66
     * @returns a new Quaternion object, computed from the Vector3 coordinates
     */
    toQuaternion(): Quaternion;
    /**
     * Adds the given vector to the current Vector3
     * Example Playground https://playground.babylonjs.com/#R1F8YU#4
     * @param otherVector defines the second operand
     * @returns the current updated Vector3
     */
    addInPlace(otherVector: DeepImmutable<Vector3>): this;
    /**
     * Adds the given coordinates to the current Vector3
     * Example Playground https://playground.babylonjs.com/#R1F8YU#5
     * @param x defines the x coordinate of the operand
     * @param y defines the y coordinate of the operand
     * @param z defines the z coordinate of the operand
     * @returns the current updated Vector3
     */
    addInPlaceFromFloats(x: number, y: number, z: number): this;
    /**
     * Gets a new Vector3, result of the addition the current Vector3 and the given vector
     * Example Playground https://playground.babylonjs.com/#R1F8YU#3
     * @param otherVector defines the second operand
     * @returns the resulting Vector3
     */
    add(otherVector: DeepImmutable<Vector3>): this;
    /**
     * Adds the current Vector3 to the given one and stores the result in the vector "result"
     * Example Playground https://playground.babylonjs.com/#R1F8YU#6
     * @param otherVector defines the second operand
     * @param result defines the Vector3 object where to store the result
     * @returns the result
     */
    addToRef<T extends Vector3>(otherVector: DeepImmutable<Vector3>, result: T): T;
    /**
     * Subtract the given vector from the current Vector3
     * Example Playground https://playground.babylonjs.com/#R1F8YU#61
     * @param otherVector defines the second operand
     * @returns the current updated Vector3
     */
    subtractInPlace(otherVector: DeepImmutable<Vector3>): this;
    /**
     * Returns a new Vector3, result of the subtraction of the given vector from the current Vector3
     * Example Playground https://playground.babylonjs.com/#R1F8YU#60
     * @param otherVector defines the second operand
     * @returns the resulting Vector3
     */
    subtract(otherVector: DeepImmutable<Vector3>): this;
    /**
     * Subtracts the given vector from the current Vector3 and stores the result in the vector "result".
     * Example Playground https://playground.babylonjs.com/#R1F8YU#63
     * @param otherVector defines the second operand
     * @param result defines the Vector3 object where to store the result
     * @returns the result
     */
    subtractToRef<T extends Vector3>(otherVector: DeepImmutable<Vector3>, result: T): T;
    /**
     * Returns a new Vector3 set with the subtraction of the given floats from the current Vector3 coordinates
     * Example Playground https://playground.babylonjs.com/#R1F8YU#62
     * @param x defines the x coordinate of the operand
     * @param y defines the y coordinate of the operand
     * @param z defines the z coordinate of the operand
     * @returns the resulting Vector3
     */
    subtractFromFloats(x: number, y: number, z: number): this;
    /**
     * Subtracts the given floats from the current Vector3 coordinates and set the given vector "result" with this result
     * Example Playground https://playground.babylonjs.com/#R1F8YU#64
     * @param x defines the x coordinate of the operand
     * @param y defines the y coordinate of the operand
     * @param z defines the z coordinate of the operand
     * @param result defines the Vector3 object where to store the result
     * @returns the result
     */
    subtractFromFloatsToRef<T extends Vector3>(x: number, y: number, z: number, result: T): T;
    /**
     * Gets a new Vector3 set with the current Vector3 negated coordinates
     * Example Playground https://playground.babylonjs.com/#R1F8YU#35
     * @returns a new Vector3
     */
    negate(): this;
    /**
     * Negate this vector in place
     * Example Playground https://playground.babylonjs.com/#R1F8YU#36
     * @returns this
     */
    negateInPlace(): this;
    /**
     * Negate the current Vector3 and stores the result in the given vector "result" coordinates
     * Example Playground https://playground.babylonjs.com/#R1F8YU#37
     * @param result defines the Vector3 object where to store the result
     * @returns the result
     */
    negateToRef<T extends Vector3 = Vector3>(result: T): T;
    /**
     * Multiplies the Vector3 coordinates by the float "scale"
     * Example Playground https://playground.babylonjs.com/#R1F8YU#56
     * @param scale defines the multiplier factor
     * @returns the current updated Vector3
     */
    scaleInPlace(scale: number): this;
    /**
     * Returns a new Vector3 set with the current Vector3 coordinates multiplied by the float "scale"
     * Example Playground https://playground.babylonjs.com/#R1F8YU#53
     * @param scale defines the multiplier factor
     * @returns a new Vector3
     */
    scale(scale: number): this;
    /**
     * Multiplies the current Vector3 coordinates by the float "scale" and stores the result in the given vector "result" coordinates
     * Example Playground https://playground.babylonjs.com/#R1F8YU#57
     * @param scale defines the multiplier factor
     * @param result defines the Vector3 object where to store the result
     * @returns the result
     */
    scaleToRef<T extends Vector3>(scale: number, result: T): T;
    /**
     * Creates a vector normal (perpendicular) to the current Vector3 and stores the result in the given vector
     * Out of the infinite possibilities the normal chosen is the one formed by rotating the current vector
     * 90 degrees about an axis which lies perpendicular to the current vector
     * and its projection on the xz plane. In the case of a current vector in the xz plane
     * the normal is calculated to be along the y axis.
     * Example Playground https://playground.babylonjs.com/#R1F8YU#230
     * Example Playground https://playground.babylonjs.com/#R1F8YU#231
     * @param result defines the Vector3 object where to store the resultant normal
     * returns the result
     */
    getNormalToRef(result: DeepImmutable<Vector3>): Vector3;
    /**
     * Rotates the vector using the given unit quaternion and stores the new vector in result
     * Example Playground https://playground.babylonjs.com/#R1F8YU#9
     * @param q the unit quaternion representing the rotation
     * @param result the output vector
     * @returns the result
     */
    applyRotationQuaternionToRef<T extends Vector3>(q: Quaternion, result: T): T;
    /**
     * Rotates the vector in place using the given unit quaternion
     * Example Playground https://playground.babylonjs.com/#R1F8YU#8
     * @param q the unit quaternion representing the rotation
     * @returns the current updated Vector3
     */
    applyRotationQuaternionInPlace(q: Quaternion): this;
    /**
     * Rotates the vector using the given unit quaternion and returns the new vector
     * Example Playground https://playground.babylonjs.com/#R1F8YU#7
     * @param q the unit quaternion representing the rotation
     * @returns a new Vector3
     */
    applyRotationQuaternion(q: Quaternion): this;
    /**
     * Scale the current Vector3 values by a factor and add the result to a given Vector3
     * Example Playground https://playground.babylonjs.com/#R1F8YU#55
     * @param scale defines the scale factor
     * @param result defines the Vector3 object where to store the result
     * @returns result input
     */
    scaleAndAddToRef<T extends Vector3>(scale: number, result: T): T;
    /**
     * Projects the current point Vector3 to a plane along a ray starting from a specified origin and passing through the current point Vector3.
     * Example Playground https://playground.babylonjs.com/#R1F8YU#48
     * @param plane defines the plane to project to
     * @param origin defines the origin of the projection ray
     * @returns the projected vector3
     */
    projectOnPlane<T extends Vector3>(plane: Plane, origin: Vector3): T;
    /**
     * Projects the current point Vector3 to a plane along a ray starting from a specified origin and passing through the current point Vector3.
     * Example Playground https://playground.babylonjs.com/#R1F8YU#49
     * @param plane defines the plane to project to
     * @param origin defines the origin of the projection ray
     * @param result defines the Vector3 where to store the result
     * @returns result input
     */
    projectOnPlaneToRef<T extends Vector3>(plane: Plane, origin: Vector3, result: T): T;
    /**
     * Returns true if the current Vector3 and the given vector coordinates are strictly equal
     * Example Playground https://playground.babylonjs.com/#R1F8YU#19
     * @param otherVector defines the second operand
     * @returns true if both vectors are equals
     */
    equals(otherVector: DeepImmutable<Vector3>): boolean;
    /**
     * Returns true if the current Vector3 and the given vector coordinates are distant less than epsilon
     * Example Playground https://playground.babylonjs.com/#R1F8YU#21
     * @param otherVector defines the second operand
     * @param epsilon defines the minimal distance to define values as equals
     * @returns true if both vectors are distant less than epsilon
     */
    equalsWithEpsilon(otherVector: DeepImmutable<Vector3>, epsilon?: number): boolean;
    /**
     * Returns true if the current Vector3 coordinates equals the given floats
     * Example Playground https://playground.babylonjs.com/#R1F8YU#20
     * @param x defines the x coordinate of the operand
     * @param y defines the y coordinate of the operand
     * @param z defines the z coordinate of the operand
     * @returns true if both vectors are equal
     */
    equalsToFloats(x: number, y: number, z: number): boolean;
    /**
     * Multiplies the current Vector3 coordinates by the given ones
     * Example Playground https://playground.babylonjs.com/#R1F8YU#32
     * @param otherVector defines the second operand
     * @returns the current updated Vector3
     */
    multiplyInPlace(otherVector: DeepImmutable<Vector3>): this;
    /**
     * Returns a new Vector3, result of the multiplication of the current Vector3 by the given vector
     * Example Playground https://playground.babylonjs.com/#R1F8YU#31
     * @param otherVector defines the second operand
     * @returns the new Vector3
     */
    multiply(otherVector: DeepImmutable<Vector3>): this;
    /**
     * Multiplies the current Vector3 by the given one and stores the result in the given vector "result"
     * Example Playground https://playground.babylonjs.com/#R1F8YU#33
     * @param otherVector defines the second operand
     * @param result defines the Vector3 object where to store the result
     * @returns the result
     */
    multiplyToRef<T extends Vector3>(otherVector: DeepImmutable<Vector3>, result: T): T;
    /**
     * Returns a new Vector3 set with the result of the multiplication of the current Vector3 coordinates by the given floats
     * Example Playground https://playground.babylonjs.com/#R1F8YU#34
     * @param x defines the x coordinate of the operand
     * @param y defines the y coordinate of the operand
     * @param z defines the z coordinate of the operand
     * @returns the new Vector3
     */
    multiplyByFloats(x: number, y: number, z: number): this;
    /**
     * Returns a new Vector3 set with the result of the division of the current Vector3 coordinates by the given ones
     * Example Playground https://playground.babylonjs.com/#R1F8YU#16
     * @param otherVector defines the second operand
     * @returns the new Vector3
     */
    divide(otherVector: DeepImmutable<Vector3>): this;
    /**
     * Divides the current Vector3 coordinates by the given ones and stores the result in the given vector "result"
     * Example Playground https://playground.babylonjs.com/#R1F8YU#18
     * @param otherVector defines the second operand
     * @param result defines the Vector3 object where to store the result
     * @returns the result
     */
    divideToRef<T extends Vector3>(otherVector: DeepImmutable<Vector3>, result: T): T;
    /**
     * Divides the current Vector3 coordinates by the given ones.
     * Example Playground https://playground.babylonjs.com/#R1F8YU#17
     * @param otherVector defines the second operand
     * @returns the current updated Vector3
     */
    divideInPlace(otherVector: Vector3): this;
    /**
     * Updates the current Vector3 with the minimal coordinate values between its and the given vector ones
     * Example Playground https://playground.babylonjs.com/#R1F8YU#29
     * @param other defines the second operand
     * @returns the current updated Vector3
     */
    minimizeInPlace(other: DeepImmutable<Vector3>): this;
    /**
     * Updates the current Vector3 with the maximal coordinate values between its and the given vector ones.
     * Example Playground https://playground.babylonjs.com/#R1F8YU#27
     * @param other defines the second operand
     * @returns the current updated Vector3
     */
    maximizeInPlace(other: DeepImmutable<Vector3>): this;
    /**
     * Updates the current Vector3 with the minimal coordinate values between its and the given coordinates
     * Example Playground https://playground.babylonjs.com/#R1F8YU#30
     * @param x defines the x coordinate of the operand
     * @param y defines the y coordinate of the operand
     * @param z defines the z coordinate of the operand
     * @returns the current updated Vector3
     */
    minimizeInPlaceFromFloats(x: number, y: number, z: number): this;
    /**
     * Updates the current Vector3 with the maximal coordinate values between its and the given coordinates.
     * Example Playground https://playground.babylonjs.com/#R1F8YU#28
     * @param x defines the x coordinate of the operand
     * @param y defines the y coordinate of the operand
     * @param z defines the z coordinate of the operand
     * @returns the current updated Vector3
     */
    maximizeInPlaceFromFloats(x: number, y: number, z: number): this;
    /**
     * Due to float precision, scale of a mesh could be uniform but float values are off by a small fraction
     * Check if is non uniform within a certain amount of decimal places to account for this
     * @param epsilon the amount the values can differ
     * @returns if the the vector is non uniform to a certain number of decimal places
     */
    isNonUniformWithinEpsilon(epsilon: number): boolean;
    /**
     * Gets a boolean indicating that the vector is non uniform meaning x, y or z are not all the same
     */
    get isNonUniform(): boolean;
    /**
     * Gets a new Vector3 from current Vector3 floored values
     * Example Playground https://playground.babylonjs.com/#R1F8YU#22
     * @returns a new Vector3
     */
    floor(): this;
    /**
     * Gets a new Vector3 from current Vector3 fractional values
     * Example Playground https://playground.babylonjs.com/#R1F8YU#23
     * @returns a new Vector3
     */
    fract(): this;
    /**
     * Gets the length of the Vector3
     * Example Playground https://playground.babylonjs.com/#R1F8YU#25
     * @returns the length of the Vector3
     */
    length(): number;
    /**
     * Gets the squared length of the Vector3
     * Example Playground https://playground.babylonjs.com/#R1F8YU#26
     * @returns squared length of the Vector3
     */
    lengthSquared(): number;
    /**
     * Gets a boolean indicating if the vector contains a zero in one of its components
     * Example Playground https://playground.babylonjs.com/#R1F8YU#1
     */
    get hasAZeroComponent(): boolean;
    /**
     * Normalize the current Vector3.
     * Please note that this is an in place operation.
     * Example Playground https://playground.babylonjs.com/#R1F8YU#122
     * @returns the current updated Vector3
     */
    normalize(): this;
    /**
     * Reorders the x y z properties of the vector in place
     * Example Playground https://playground.babylonjs.com/#R1F8YU#44
     * @param order new ordering of the properties (eg. for vector 1,2,3 with "ZYX" will produce 3,2,1)
     * @returns the current updated vector
     */
    reorderInPlace(order: string): this;
    /**
     * Rotates the vector around 0,0,0 by a quaternion
     * Example Playground https://playground.babylonjs.com/#R1F8YU#47
     * @param quaternion the rotation quaternion
     * @param result vector to store the result
     * @returns the resulting vector
     */
    rotateByQuaternionToRef<T extends Vector3>(quaternion: Quaternion, result: T): T;
    /**
     * Rotates a vector around a given point
     * Example Playground https://playground.babylonjs.com/#R1F8YU#46
     * @param quaternion the rotation quaternion
     * @param point the point to rotate around
     * @param result vector to store the result
     * @returns the resulting vector
     */
    rotateByQuaternionAroundPointToRef<T extends Vector3>(quaternion: Quaternion, point: Vector3, result: T): T;
    /**
     * Returns a new Vector3 as the cross product of the current vector and the "other" one
     * The cross product is then orthogonal to both current and "other"
     * Example Playground https://playground.babylonjs.com/#R1F8YU#14
     * @param other defines the right operand
     * @returns the cross product
     */
    cross(other: Vector3): this;
    /**
     * Normalize the current Vector3 with the given input length.
     * Please note that this is an in place operation.
     * Example Playground https://playground.babylonjs.com/#R1F8YU#123
     * @param len the length of the vector
     * @returns the current updated Vector3
     */
    normalizeFromLength(len: number): this;
    /**
     * Normalize the current Vector3 to a new vector
     * Example Playground https://playground.babylonjs.com/#R1F8YU#124
     * @returns the new Vector3
     */
    normalizeToNew(): this;
    /**
     * Normalize the current Vector3 to the reference
     * Example Playground https://playground.babylonjs.com/#R1F8YU#125
     * @param reference define the Vector3 to update
     * @returns the updated Vector3
     */
    normalizeToRef<T extends Vector3>(reference: T): T;
    /**
     * Creates a new Vector3 copied from the current Vector3
     * Example Playground https://playground.babylonjs.com/#R1F8YU#11
     * @returns the new Vector3
     */
    clone(): this;
    /**
     * Copies the given vector coordinates to the current Vector3 ones
     * Example Playground https://playground.babylonjs.com/#R1F8YU#12
     * @param source defines the source Vector3
     * @returns the current updated Vector3
     */
    copyFrom(source: DeepImmutable<Vector3>): this;
    /**
     * Copies the given floats to the current Vector3 coordinates
     * Example Playground https://playground.babylonjs.com/#R1F8YU#13
     * @param x defines the x coordinate of the operand
     * @param y defines the y coordinate of the operand
     * @param z defines the z coordinate of the operand
     * @returns the current updated Vector3
     */
    copyFromFloats(x: number, y: number, z: number): this;
    /**
     * Copies the given floats to the current Vector3 coordinates
     * Example Playground https://playground.babylonjs.com/#R1F8YU#58
     * @param x defines the x coordinate of the operand
     * @param y defines the y coordinate of the operand
     * @param z defines the z coordinate of the operand
     * @returns the current updated Vector3
     */
    set(x: number, y: number, z: number): this;
    /**
     * Copies the given float to the current Vector3 coordinates
     * Example Playground https://playground.babylonjs.com/#R1F8YU#59
     * @param v defines the x, y and z coordinates of the operand
     * @returns the current updated Vector3
     */
    setAll(v: number): this;
    /**
     * Get the clip factor between two vectors
     * Example Playground https://playground.babylonjs.com/#R1F8YU#126
     * @param vector0 defines the first operand
     * @param vector1 defines the second operand
     * @param axis defines the axis to use
     * @param size defines the size along the axis
     * @returns the clip factor
     */
    static GetClipFactor(vector0: DeepImmutable<Vector3>, vector1: DeepImmutable<Vector3>, axis: DeepImmutable<Vector3>, size: number): number;
    /**
     * Get angle between two vectors
     * Example Playground https://playground.babylonjs.com/#R1F8YU#86
     * @param vector0 the starting point
     * @param vector1 the ending point
     * @param normal direction of the normal
     * @returns the angle between vector0 and vector1
     */
    static GetAngleBetweenVectors(vector0: DeepImmutable<Vector3>, vector1: DeepImmutable<Vector3>, normal: DeepImmutable<Vector3>): number;
    /**
     * Get angle between two vectors projected on a plane
     * Example Playground https://playground.babylonjs.com/#R1F8YU#87
     * Expectation compute time: 0.01 ms (median) and 0.02 ms (percentile 95%)
     * @param vector0 angle between vector0 and vector1
     * @param vector1 angle between vector0 and vector1
     * @param normal Normal of the projection plane
     * @returns the angle in radians (float) between vector0 and vector1 projected on the plane with the specified normal
     */
    static GetAngleBetweenVectorsOnPlane(vector0: Vector3, vector1: Vector3, normal: Vector3): number;
    /**
     * Gets the rotation that aligns the roll axis (Y) to the line joining the start point to the target point and stores it in the ref Vector3
     * Example PG https://playground.babylonjs.com/#R1F8YU#189
     * @param start the starting point
     * @param target the target point
     * @param ref the vector3 to store the result
     * @returns ref in the form (pitch, yaw, 0)
     */
    static PitchYawRollToMoveBetweenPointsToRef<T extends Vector3>(start: Vector3, target: Vector3, ref: T): T;
    /**
     * Gets the rotation that aligns the roll axis (Y) to the line joining the start point to the target point
     * Example PG https://playground.babylonjs.com/#R1F8YU#188
     * @param start the starting point
     * @param target the target point
     * @returns the rotation in the form (pitch, yaw, 0)
     */
    static PitchYawRollToMoveBetweenPoints(start: Vector3, target: Vector3): Vector3;
    /**
     * Slerp between two vectors. See also `SmoothToRef`
     * Slerp is a spherical linear interpolation
     * giving a slow in and out effect
     * Example Playground 1 https://playground.babylonjs.com/#R1F8YU#108
     * Example Playground 2 https://playground.babylonjs.com/#R1F8YU#109
     * @param vector0 Start vector
     * @param vector1 End vector
     * @param slerp amount (will be clamped between 0 and 1)
     * @param result The slerped vector
     */
    static SlerpToRef<T extends Vector3 = Vector3>(vector0: Vector3, vector1: Vector3, slerp: number, result: T): T;
    /**
     * Smooth interpolation between two vectors using Slerp
     * Example Playground https://playground.babylonjs.com/#R1F8YU#110
     * @param source source vector
     * @param goal goal vector
     * @param deltaTime current interpolation frame
     * @param lerpTime total interpolation time
     * @param result the smoothed vector
     */
    static SmoothToRef<T extends Vector3 = Vector3>(source: Vector3, goal: Vector3, deltaTime: number, lerpTime: number, result: T): T;
    /**
     * Returns a new Vector3 set from the index "offset" of the given array
     * Example Playground https://playground.babylonjs.com/#R1F8YU#83
     * @param array defines the source array
     * @param offset defines the offset in the source array
     * @returns the new Vector3
     */
    static FromArray(array: DeepImmutable<ArrayLike<number>>, offset?: number): Vector3;
    /**
     * Returns a new Vector3 set from the index "offset" of the given Float32Array
     * @param array defines the source array
     * @param offset defines the offset in the source array
     * @returns the new Vector3
     * @deprecated Please use FromArray instead.
     */
    static FromFloatArray(array: DeepImmutable<Float32Array>, offset?: number): Vector3;
    /**
     * Sets the given vector "result" with the element values from the index "offset" of the given array
     * Example Playground https://playground.babylonjs.com/#R1F8YU#84
     * @param array defines the source array
     * @param offset defines the offset in the source array
     * @param result defines the Vector3 where to store the result
     * @returns result input
     */
    static FromArrayToRef<T extends Vector3>(array: DeepImmutable<ArrayLike<number>>, offset: number, result: T): T;
    /**
     * Sets the given vector "result" with the element values from the index "offset" of the given Float32Array
     * @param array defines the source array
     * @param offset defines the offset in the source array
     * @param result defines the Vector3 where to store the result
     * @deprecated Please use FromArrayToRef instead.
     */
    static FromFloatArrayToRef<T extends Vector3>(array: DeepImmutable<Float32Array>, offset: number, result: T): T;
    /**
     * Sets the given vector "result" with the given floats.
     * Example Playground https://playground.babylonjs.com/#R1F8YU#85
     * @param x defines the x coordinate of the source
     * @param y defines the y coordinate of the source
     * @param z defines the z coordinate of the source
     * @param result defines the Vector3 where to store the result
     */
    static FromFloatsToRef<T extends Vector3 = Vector3>(x: number, y: number, z: number, result: T): T;
    /**
     * Returns a new Vector3 set to (0.0, 0.0, 0.0)
     * @returns a new empty Vector3
     */
    static Zero(): Vector3;
    /**
     * Returns a new Vector3 set to (1.0, 1.0, 1.0)
     * @returns a new Vector3
     */
    static One(): Vector3;
    /**
     * Returns a new Vector3 set to (0.0, 1.0, 0.0)
     * Example Playground https://playground.babylonjs.com/#R1F8YU#71
     * @returns a new up Vector3
     */
    static Up(): Vector3;
    /**
     * Gets an up Vector3 that must not be updated
     */
    static get UpReadOnly(): DeepImmutable<Vector3>;
    /**
     * Gets a down Vector3 that must not be updated
     */
    static get DownReadOnly(): DeepImmutable<Vector3>;
    /**
     * Gets a right Vector3 that must not be updated
     */
    static get RightReadOnly(): DeepImmutable<Vector3>;
    /**
     * Gets a left Vector3 that must not be updated
     */
    static get LeftReadOnly(): DeepImmutable<Vector3>;
    /**
     * Gets a forward Vector3 that must not be updated
     */
    static get LeftHandedForwardReadOnly(): DeepImmutable<Vector3>;
    /**
     * Gets a forward Vector3 that must not be updated
     */
    static get RightHandedForwardReadOnly(): DeepImmutable<Vector3>;
    /**
     * Gets a zero Vector3 that must not be updated
     */
    static get ZeroReadOnly(): DeepImmutable<Vector3>;
    /**
     * Returns a new Vector3 set to (0.0, -1.0, 0.0)
     * Example Playground https://playground.babylonjs.com/#R1F8YU#71
     * @returns a new down Vector3
     */
    static Down(): Vector3;
    /**
     * Returns a new Vector3 set to (0.0, 0.0, 1.0)
     * Example Playground https://playground.babylonjs.com/#R1F8YU#71
     * @param rightHandedSystem is the scene right-handed (negative z)
     * @returns a new forward Vector3
     */
    static Forward(rightHandedSystem?: boolean): Vector3;
    /**
     * Returns a new Vector3 set to (0.0, 0.0, -1.0)
     * Example Playground https://playground.babylonjs.com/#R1F8YU#71
     * @param rightHandedSystem is the scene right-handed (negative-z)
     * @returns a new Backward Vector3
     */
    static Backward(rightHandedSystem?: boolean): Vector3;
    /**
     * Returns a new Vector3 set to (1.0, 0.0, 0.0)
     * Example Playground https://playground.babylonjs.com/#R1F8YU#71
     * @returns a new right Vector3
     */
    static Right(): Vector3;
    /**
     * Returns a new Vector3 set to (-1.0, 0.0, 0.0)
     * Example Playground https://playground.babylonjs.com/#R1F8YU#71
     * @returns a new left Vector3
     */
    static Left(): Vector3;
    /**
     * Returns a new Vector3 set with the result of the transformation by the given matrix of the given vector.
     * This method computes transformed coordinates only, not transformed direction vectors (ie. it takes translation in account)
     * Example Playground https://playground.babylonjs.com/#R1F8YU#111
     * @param vector defines the Vector3 to transform
     * @param transformation defines the transformation matrix
     * @returns the transformed Vector3
     */
    static TransformCoordinates<T extends Vector3>(vector: DeepImmutable<Vector3>, transformation: DeepImmutable<Matrix>): Vector3;
    /**
     * Sets the given vector "result" coordinates with the result of the transformation by the given matrix of the given vector
     * This method computes transformed coordinates only, not transformed direction vectors (ie. it takes translation in account)
     * Example Playground https://playground.babylonjs.com/#R1F8YU#113
     * @param vector defines the Vector3 to transform
     * @param transformation defines the transformation matrix
     * @param result defines the Vector3 where to store the result
     * @returns result input
     */
    static TransformCoordinatesToRef<T extends Vector3>(vector: DeepImmutable<Vector3>, transformation: DeepImmutable<Matrix>, result: T): T;
    /**
     * Sets the given vector "result" coordinates with the result of the transformation by the given matrix of the given floats (x, y, z)
     * This method computes transformed coordinates only, not transformed direction vectors
     * Example Playground https://playground.babylonjs.com/#R1F8YU#115
     * @param x define the x coordinate of the source vector
     * @param y define the y coordinate of the source vector
     * @param z define the z coordinate of the source vector
     * @param transformation defines the transformation matrix
     * @param result defines the Vector3 where to store the result
     * @returns result input
     */
    static TransformCoordinatesFromFloatsToRef<T extends Vector3>(x: number, y: number, z: number, transformation: DeepImmutable<Matrix>, result: T): T;
    /**
     * Returns a new Vector3 set with the result of the normal transformation by the given matrix of the given vector
     * This methods computes transformed normalized direction vectors only (ie. it does not apply translation)
     * Example Playground https://playground.babylonjs.com/#R1F8YU#112
     * @param vector defines the Vector3 to transform
     * @param transformation defines the transformation matrix
     * @returns the new Vector3
     */
    static TransformNormal(vector: DeepImmutable<Vector3>, transformation: DeepImmutable<Matrix>): Vector3;
    /**
     * Sets the given vector "result" with the result of the normal transformation by the given matrix of the given vector
     * This methods computes transformed normalized direction vectors only (ie. it does not apply translation)
     * Example Playground https://playground.babylonjs.com/#R1F8YU#114
     * @param vector defines the Vector3 to transform
     * @param transformation defines the transformation matrix
     * @param result defines the Vector3 where to store the result
     * @returns result input
     */
    static TransformNormalToRef<T extends Vector3>(vector: DeepImmutable<Vector3>, transformation: DeepImmutable<Matrix>, result: T): T;
    /**
     * Sets the given vector "result" with the result of the normal transformation by the given matrix of the given floats (x, y, z)
     * This methods computes transformed normalized direction vectors only (ie. it does not apply translation)
     * Example Playground https://playground.babylonjs.com/#R1F8YU#116
     * @param x define the x coordinate of the source vector
     * @param y define the y coordinate of the source vector
     * @param z define the z coordinate of the source vector
     * @param transformation defines the transformation matrix
     * @param result defines the Vector3 where to store the result
     * @returns result input
     */
    static TransformNormalFromFloatsToRef<T extends Vector3>(x: number, y: number, z: number, transformation: DeepImmutable<Matrix>, result: T): T;
    /**
     * Returns a new Vector3 located for "amount" on the CatmullRom interpolation spline defined by the vectors "value1", "value2", "value3", "value4"
     * Example Playground https://playground.babylonjs.com/#R1F8YU#69
     * @param value1 defines the first control point
     * @param value2 defines the second control point
     * @param value3 defines the third control point
     * @param value4 defines the fourth control point
     * @param amount defines the amount on the spline to use
     * @returns the new Vector3
     */
    static CatmullRom<T extends Vector3>(value1: DeepImmutable<T>, value2: DeepImmutable<Vector3>, value3: DeepImmutable<Vector3>, value4: DeepImmutable<Vector3>, amount: number): T;
    /**
     * Returns a new Vector3 set with the coordinates of "value", if the vector "value" is in the cube defined by the vectors "min" and "max"
     * If a coordinate value of "value" is lower than one of the "min" coordinate, then this "value" coordinate is set with the "min" one
     * If a coordinate value of "value" is greater than one of the "max" coordinate, then this "value" coordinate is set with the "max" one
     * Example Playground https://playground.babylonjs.com/#R1F8YU#76
     * @param value defines the current value
     * @param min defines the lower range value
     * @param max defines the upper range value
     * @returns the new Vector3
     */
    static Clamp<T extends Vector3>(value: DeepImmutable<T>, min: DeepImmutable<Vector3>, max: DeepImmutable<Vector3>): T;
    /**
     * Sets the given vector "result" with the coordinates of "value", if the vector "value" is in the cube defined by the vectors "min" and "max"
     * If a coordinate value of "value" is lower than one of the "min" coordinate, then this "value" coordinate is set with the "min" one
     * If a coordinate value of "value" is greater than one of the "max" coordinate, then this "value" coordinate is set with the "max" one
     * Example Playground https://playground.babylonjs.com/#R1F8YU#77
     * @param value defines the current value
     * @param min defines the lower range value
     * @param max defines the upper range value
     * @param result defines the Vector3 where to store the result
     * @returns result input
     */
    static ClampToRef<T extends Vector3>(value: DeepImmutable<Vector3>, min: DeepImmutable<Vector3>, max: DeepImmutable<Vector3>, result: T): T;
    /**
     * Checks if a given vector is inside a specific range
     * Example Playground https://playground.babylonjs.com/#R1F8YU#75
     * @param v defines the vector to test
     * @param min defines the minimum range
     * @param max defines the maximum range
     */
    static CheckExtends(v: Vector3, min: Vector3, max: Vector3): void;
    /**
     * Returns a new Vector3 located for "amount" (float) on the Hermite interpolation spline defined by the vectors "value1", "tangent1", "value2", "tangent2"
     * Example Playground https://playground.babylonjs.com/#R1F8YU#89
     * @param value1 defines the first control point
     * @param tangent1 defines the first tangent vector
     * @param value2 defines the second control point
     * @param tangent2 defines the second tangent vector
     * @param amount defines the amount on the interpolation spline (between 0 and 1)
     * @returns the new Vector3
     */
    static Hermite<T extends Vector3>(value1: DeepImmutable<T>, tangent1: DeepImmutable<Vector3>, value2: DeepImmutable<Vector3>, tangent2: DeepImmutable<Vector3>, amount: number): T;
    /**
     * Returns a new Vector3 which is the 1st derivative of the Hermite spline defined by the vectors "value1", "value2", "tangent1", "tangent2".
     * Example Playground https://playground.babylonjs.com/#R1F8YU#90
     * @param value1 defines the first control point
     * @param tangent1 defines the first tangent
     * @param value2 defines the second control point
     * @param tangent2 defines the second tangent
     * @param time define where the derivative must be done
     * @returns 1st derivative
     */
    static Hermite1stDerivative<T extends Vector3>(value1: DeepImmutable<T>, tangent1: DeepImmutable<Vector3>, value2: DeepImmutable<Vector3>, tangent2: DeepImmutable<Vector3>, time: number): T;
    /**
     * Update a Vector3 with the 1st derivative of the Hermite spline defined by the vectors "value1", "value2", "tangent1", "tangent2".
     * Example Playground https://playground.babylonjs.com/#R1F8YU#91
     * @param value1 defines the first control point
     * @param tangent1 defines the first tangent
     * @param value2 defines the second control point
     * @param tangent2 defines the second tangent
     * @param time define where the derivative must be done
     * @param result define where to store the derivative
     * @returns result input
     */
    static Hermite1stDerivativeToRef<T extends Vector3>(value1: DeepImmutable<Vector3>, tangent1: DeepImmutable<Vector3>, value2: DeepImmutable<Vector3>, tangent2: DeepImmutable<Vector3>, time: number, result: T): T;
    /**
     * Returns a new Vector3 located for "amount" (float) on the linear interpolation between the vectors "start" and "end"
     * Example Playground https://playground.babylonjs.com/#R1F8YU#95
     * @param start defines the start value
     * @param end defines the end value
     * @param amount max defines amount between both (between 0 and 1)
     * @returns the new Vector3
     */
    static Lerp<T extends Vector3>(start: DeepImmutable<T>, end: DeepImmutable<Vector3>, amount: number): T;
    /**
     * Sets the given vector "result" with the result of the linear interpolation from the vector "start" for "amount" to the vector "end"
     * Example Playground https://playground.babylonjs.com/#R1F8YU#93
     * @param start defines the start value
     * @param end defines the end value
     * @param amount max defines amount between both (between 0 and 1)
     * @param result defines the Vector3 where to store the result
     * @returns result input
     */
    static LerpToRef<T extends Vector3>(start: DeepImmutable<Vector3>, end: DeepImmutable<Vector3>, amount: number, result: T): T;
    /**
     * Returns the dot product (float) between the vectors "left" and "right"
     * Example Playground https://playground.babylonjs.com/#R1F8YU#82
     * @param left defines the left operand
     * @param right defines the right operand
     * @returns the dot product
     */
    static Dot(left: DeepImmutable<Vector3>, right: DeepImmutable<Vector3>): number;
    /**
     * Returns a new Vector3 as the cross product of the vectors "left" and "right"
     * The cross product is then orthogonal to both "left" and "right"
     * Example Playground https://playground.babylonjs.com/#R1F8YU#15
     * @param left defines the left operand
     * @param right defines the right operand
     * @returns the cross product
     */
    static Cross<T extends Vector3>(left: DeepImmutable<T>, right: DeepImmutable<Vector3>): T;
    /**
     * Sets the given vector "result" with the cross product of "left" and "right"
     * The cross product is then orthogonal to both "left" and "right"
     * Example Playground https://playground.babylonjs.com/#R1F8YU#78
     * @param left defines the left operand
     * @param right defines the right operand
     * @param result defines the Vector3 where to store the result
     * @returns result input
     */
    static CrossToRef<T extends Vector3>(left: DeepImmutable<Vector3>, right: DeepImmutable<Vector3>, result: T): T;
    /**
     * Returns a new Vector3 as the normalization of the given vector
     * Example Playground https://playground.babylonjs.com/#R1F8YU#98
     * @param vector defines the Vector3 to normalize
     * @returns the new Vector3
     */
    static Normalize(vector: DeepImmutable<Vector3>): Vector3;
    /**
     * Sets the given vector "result" with the normalization of the given first vector
     * Example Playground https://playground.babylonjs.com/#R1F8YU#98
     * @param vector defines the Vector3 to normalize
     * @param result defines the Vector3 where to store the result
     * @returns result input
     */
    static NormalizeToRef<T extends Vector3>(vector: DeepImmutable<Vector3>, result: T): T;
    /**
     * Project a Vector3 onto screen space
     * Example Playground https://playground.babylonjs.com/#R1F8YU#101
     * @param vector defines the Vector3 to project
     * @param world defines the world matrix to use
     * @param transform defines the transform (view x projection) matrix to use
     * @param viewport defines the screen viewport to use
     * @returns the new Vector3
     */
    static Project<T extends Vector3>(vector: DeepImmutable<T>, world: DeepImmutable<Matrix>, transform: DeepImmutable<Matrix>, viewport: DeepImmutable<Viewport>): T;
    /**
     * Project a Vector3 onto screen space to reference
     * Example Playground https://playground.babylonjs.com/#R1F8YU#102
     * @param vector defines the Vector3 to project
     * @param world defines the world matrix to use
     * @param transform defines the transform (view x projection) matrix to use
     * @param viewport defines the screen viewport to use
     * @param result the vector in which the screen space will be stored
     * @returns result input
     */
    static ProjectToRef<T extends Vector3>(vector: DeepImmutable<Vector3>, world: DeepImmutable<Matrix>, transform: DeepImmutable<Matrix>, viewport: DeepImmutable<Viewport>, result: T): T;
    /**
     * Reflects a vector off the plane defined by a normalized normal
     * @param inDirection defines the vector direction
     * @param normal defines the normal - Must be normalized
     * @returns the resulting vector
     */
    static Reflect<T extends Vector3>(inDirection: DeepImmutable<Vector3>, normal: DeepImmutable<Vector3>): Vector3;
    /**
     * Reflects a vector off the plane defined by a normalized normal to reference
     * @param inDirection defines the vector direction
     * @param normal defines the normal - Must be normalized
     * @param result defines the Vector3 where to store the result
     * @returns the resulting vector
     */
    static ReflectToRef<T extends Vector3>(inDirection: DeepImmutable<Vector3>, normal: DeepImmutable<Vector3>, ref: T): T;
    /**
     * @internal
     */
    static _UnprojectFromInvertedMatrixToRef<T extends Vector3>(source: DeepImmutable<Vector3>, matrix: DeepImmutable<Matrix>, result: T): T;
    /**
     * Unproject from screen space to object space
     * Example Playground https://playground.babylonjs.com/#R1F8YU#121
     * @param source defines the screen space Vector3 to use
     * @param viewportWidth defines the current width of the viewport
     * @param viewportHeight defines the current height of the viewport
     * @param world defines the world matrix to use (can be set to Identity to go to world space)
     * @param transform defines the transform (view x projection) matrix to use
     * @returns the new Vector3
     */
    static UnprojectFromTransform<T extends Vector3>(source: DeepImmutable<T>, viewportWidth: number, viewportHeight: number, world: DeepImmutable<Matrix>, transform: DeepImmutable<Matrix>): T;
    /**
     * Unproject from screen space to object space
     * Example Playground https://playground.babylonjs.com/#R1F8YU#117
     * @param source defines the screen space Vector3 to use
     * @param viewportWidth defines the current width of the viewport
     * @param viewportHeight defines the current height of the viewport
     * @param world defines the world matrix to use (can be set to Identity to go to world space)
     * @param view defines the view matrix to use
     * @param projection defines the projection matrix to use
     * @returns the new Vector3
     */
    static Unproject<T extends Vector3>(source: DeepImmutable<T>, viewportWidth: number, viewportHeight: number, world: DeepImmutable<Matrix>, view: DeepImmutable<Matrix>, projection: DeepImmutable<Matrix>): T;
    /**
     * Unproject from screen space to object space
     * Example Playground https://playground.babylonjs.com/#R1F8YU#119
     * @param source defines the screen space Vector3 to use
     * @param viewportWidth defines the current width of the viewport
     * @param viewportHeight defines the current height of the viewport
     * @param world defines the world matrix to use (can be set to Identity to go to world space)
     * @param view defines the view matrix to use
     * @param projection defines the projection matrix to use
     * @param result defines the Vector3 where to store the result
     * @returns result input
     */
    static UnprojectToRef<T extends Vector3>(source: DeepImmutable<Vector3>, viewportWidth: number, viewportHeight: number, world: DeepImmutable<Matrix>, view: DeepImmutable<Matrix>, projection: DeepImmutable<Matrix>, result: T): T;
    /**
     * Unproject from screen space to object space
     * Example Playground https://playground.babylonjs.com/#R1F8YU#120
     * @param sourceX defines the screen space x coordinate to use
     * @param sourceY defines the screen space y coordinate to use
     * @param sourceZ defines the screen space z coordinate to use
     * @param viewportWidth defines the current width of the viewport
     * @param viewportHeight defines the current height of the viewport
     * @param world defines the world matrix to use (can be set to Identity to go to world space)
     * @param view defines the view matrix to use
     * @param projection defines the projection matrix to use
     * @param result defines the Vector3 where to store the result
     * @returns result input
     */
    static UnprojectFloatsToRef<T extends Vector3>(sourceX: float, sourceY: float, sourceZ: float, viewportWidth: number, viewportHeight: number, world: DeepImmutable<Matrix>, view: DeepImmutable<Matrix>, projection: DeepImmutable<Matrix>, result: T): T;
    /**
     * Gets the minimal coordinate values between two Vector3
     * Example Playground https://playground.babylonjs.com/#R1F8YU#97
     * @param left defines the first operand
     * @param right defines the second operand
     * @returns the new Vector3
     */
    static Minimize<T extends Vector3>(left: DeepImmutable<T>, right: DeepImmutable<Vector3>): T;
    /**
     * Gets the maximal coordinate values between two Vector3
     * Example Playground https://playground.babylonjs.com/#R1F8YU#96
     * @param left defines the first operand
     * @param right defines the second operand
     * @returns the new Vector3
     */
    static Maximize<T extends Vector3>(left: DeepImmutable<T>, right: DeepImmutable<Vector3>): T;
    /**
     * Returns the distance between the vectors "value1" and "value2"
     * Example Playground https://playground.babylonjs.com/#R1F8YU#81
     * @param value1 defines the first operand
     * @param value2 defines the second operand
     * @returns the distance
     */
    static Distance(value1: DeepImmutable<Vector3>, value2: DeepImmutable<Vector3>): number;
    /**
     * Returns the squared distance between the vectors "value1" and "value2"
     * Example Playground https://playground.babylonjs.com/#R1F8YU#80
     * @param value1 defines the first operand
     * @param value2 defines the second operand
     * @returns the squared distance
     */
    static DistanceSquared(value1: DeepImmutable<Vector3>, value2: DeepImmutable<Vector3>): number;
    /**
     * Projects "vector" on the triangle determined by its extremities "p0", "p1" and "p2", stores the result in "ref"
     * and returns the distance to the projected point.
     * Example Playground https://playground.babylonjs.com/#R1F8YU#104
     * From http://citeseerx.ist.psu.edu/viewdoc/download?doi=10.1.1.104.4264&rep=rep1&type=pdf
     *
     * @param vector the vector to get distance from
     * @param p0 extremity of the triangle
     * @param p1 extremity of the triangle
     * @param p2 extremity of the triangle
     * @param ref variable to store the result to
     * @returns The distance between "ref" and "vector"
     */
    static ProjectOnTriangleToRef(vector: DeepImmutable<Vector3>, p0: DeepImmutable<Vector3>, p1: DeepImmutable<Vector3>, p2: DeepImmutable<Vector3>, ref: Vector3): number;
    /**
     * Returns a new Vector3 located at the center between "value1" and "value2"
     * Example Playground https://playground.babylonjs.com/#R1F8YU#72
     * @param value1 defines the first operand
     * @param value2 defines the second operand
     * @returns the new Vector3
     */
    static Center(value1: DeepImmutable<Vector3>, value2: DeepImmutable<Vector3>): Vector3;
    /**
     * Gets the center of the vectors "value1" and "value2" and stores the result in the vector "ref"
     * Example Playground https://playground.babylonjs.com/#R1F8YU#73
     * @param value1 defines first vector
     * @param value2 defines second vector
     * @param ref defines third vector
     * @returns ref
     */
    static CenterToRef<T extends Vector3>(value1: DeepImmutable<Vector3>, value2: DeepImmutable<Vector3>, ref: T): T;
    /**
     * Given three orthogonal normalized left-handed oriented Vector3 axis in space (target system),
     * RotationFromAxis() returns the rotation Euler angles (ex : rotation.x, rotation.y, rotation.z) to apply
     * to something in order to rotate it from its local system to the given target system
     * Note: axis1, axis2 and axis3 are normalized during this operation
     * Example Playground https://playground.babylonjs.com/#R1F8YU#106
     * @param axis1 defines the first axis
     * @param axis2 defines the second axis
     * @param axis3 defines the third axis
     * @returns a new Vector3
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/transforms/center_origin/target_align
     */
    static RotationFromAxis<T extends Vector3>(axis1: DeepImmutable<T>, axis2: DeepImmutable<Vector3>, axis3: DeepImmutable<Vector3>): T;
    /**
     * The same than RotationFromAxis but updates the given ref Vector3 parameter instead of returning a new Vector3
     * Example Playground https://playground.babylonjs.com/#R1F8YU#107
     * @param axis1 defines the first axis
     * @param axis2 defines the second axis
     * @param axis3 defines the third axis
     * @param ref defines the Vector3 where to store the result
     * @returns result input
     */
    static RotationFromAxisToRef<T extends Vector3>(axis1: DeepImmutable<Vector3>, axis2: DeepImmutable<Vector3>, axis3: DeepImmutable<Vector3>, ref: T): T;
}
/**
 * Vector4 class created for EulerAngle class conversion to Quaternion
 */
export class Vector4 {
    /** x value of the vector */
    x: number;
    /** y value of the vector */
    y: number;
    /** z value of the vector */
    z: number;
    /** w value of the vector */
    w: number;
    private static _ZeroReadOnly;
    /**
     * Creates a Vector4 object from the given floats.
     * @param x x value of the vector
     * @param y y value of the vector
     * @param z z value of the vector
     * @param w w value of the vector
     */
    constructor(
    /** x value of the vector */
    x?: number, 
    /** y value of the vector */
    y?: number, 
    /** z value of the vector */
    z?: number, 
    /** w value of the vector */
    w?: number);
    /**
     * Returns the string with the Vector4 coordinates.
     * @returns a string containing all the vector values
     */
    toString(): string;
    /**
     * Returns the string "Vector4".
     * @returns "Vector4"
     */
    getClassName(): string;
    /**
     * Returns the Vector4 hash code.
     * @returns a unique hash code
     */
    getHashCode(): number;
    /**
     * Returns a new array populated with 4 elements : the Vector4 coordinates.
     * @returns the resulting array
     */
    asArray(): number[];
    /**
     * Populates the given array from the given index with the Vector4 coordinates.
     * @param array array to populate
     * @param index index of the array to start at (default: 0)
     * @returns the Vector4.
     */
    toArray(array: FloatArray, index?: number): this;
    /**
     * Update the current vector from an array
     * @param array defines the destination array
     * @param index defines the offset in the destination array
     * @returns the current Vector3
     */
    fromArray(array: FloatArray, index?: number): this;
    /**
     * Adds the given vector to the current Vector4.
     * @param otherVector the vector to add
     * @returns the updated Vector4.
     */
    addInPlace(otherVector: DeepImmutable<Vector4>): this;
    /**
     * Returns a new Vector4 as the result of the addition of the current Vector4 and the given one.
     * @param otherVector the vector to add
     * @returns the resulting vector
     */
    add(otherVector: DeepImmutable<Vector4>): this;
    /**
     * Updates the given vector "result" with the result of the addition of the current Vector4 and the given one.
     * @param otherVector the vector to add
     * @param result the vector to store the result
     * @returns result input
     */
    addToRef<T extends Vector4>(otherVector: DeepImmutable<Vector4>, result: T): T;
    /**
     * Subtract in place the given vector from the current Vector4.
     * @param otherVector the vector to subtract
     * @returns the updated Vector4.
     */
    subtractInPlace(otherVector: DeepImmutable<Vector4>): this;
    /**
     * Returns a new Vector4 with the result of the subtraction of the given vector from the current Vector4.
     * @param otherVector the vector to add
     * @returns the new vector with the result
     */
    subtract(otherVector: DeepImmutable<Vector4>): this;
    /**
     * Sets the given vector "result" with the result of the subtraction of the given vector from the current Vector4.
     * @param otherVector the vector to subtract
     * @param result the vector to store the result
     * @returns result input
     */
    subtractToRef<T extends Vector4>(otherVector: DeepImmutable<Vector4>, result: T): T;
    /**
     * Returns a new Vector4 set with the result of the subtraction of the given floats from the current Vector4 coordinates.
     */
    /**
     * Returns a new Vector4 set with the result of the subtraction of the given floats from the current Vector4 coordinates.
     * @param x value to subtract
     * @param y value to subtract
     * @param z value to subtract
     * @param w value to subtract
     * @returns new vector containing the result
     */
    subtractFromFloats(x: number, y: number, z: number, w: number): this;
    /**
     * Sets the given vector "result" set with the result of the subtraction of the given floats from the current Vector4 coordinates.
     * @param x value to subtract
     * @param y value to subtract
     * @param z value to subtract
     * @param w value to subtract
     * @param result the vector to store the result in
     * @returns result input
     */
    subtractFromFloatsToRef<T extends Vector4>(x: number, y: number, z: number, w: number, result: T): T;
    /**
     * Returns a new Vector4 set with the current Vector4 negated coordinates.
     * @returns a new vector with the negated values
     */
    negate(): this;
    /**
     * Negate this vector in place
     * @returns this
     */
    negateInPlace(): this;
    /**
     * Negate the current Vector4 and stores the result in the given vector "result" coordinates
     * @param result defines the Vector3 object where to store the result
     * @returns the result
     */
    negateToRef<T extends Vector4>(result: T): T;
    /**
     * Multiplies the current Vector4 coordinates by scale (float).
     * @param scale the number to scale with
     * @returns the updated Vector4.
     */
    scaleInPlace(scale: number): this;
    /**
     * Returns a new Vector4 set with the current Vector4 coordinates multiplied by scale (float).
     * @param scale the number to scale with
     * @returns a new vector with the result
     */
    scale(scale: number): this;
    /**
     * Sets the given vector "result" with the current Vector4 coordinates multiplied by scale (float).
     * @param scale the number to scale with
     * @param result a vector to store the result in
     * @returns result input
     */
    scaleToRef<T extends Vector4>(scale: number, result: T): T;
    /**
     * Scale the current Vector4 values by a factor and add the result to a given Vector4
     * @param scale defines the scale factor
     * @param result defines the Vector4 object where to store the result
     * @returns result input
     */
    scaleAndAddToRef<T extends Vector4>(scale: number, result: T): T;
    /**
     * Boolean : True if the current Vector4 coordinates are stricly equal to the given ones.
     * @param otherVector the vector to compare against
     * @returns true if they are equal
     */
    equals(otherVector: DeepImmutable<Vector4>): boolean;
    /**
     * Boolean : True if the current Vector4 coordinates are each beneath the distance "epsilon" from the given vector ones.
     * @param otherVector vector to compare against
     * @param epsilon (Default: very small number)
     * @returns true if they are equal
     */
    equalsWithEpsilon(otherVector: DeepImmutable<Vector4>, epsilon?: number): boolean;
    /**
     * Boolean : True if the given floats are strictly equal to the current Vector4 coordinates.
     * @param x x value to compare against
     * @param y y value to compare against
     * @param z z value to compare against
     * @param w w value to compare against
     * @returns true if equal
     */
    equalsToFloats(x: number, y: number, z: number, w: number): boolean;
    /**
     * Multiplies in place the current Vector4 by the given one.
     * @param otherVector vector to multiple with
     * @returns the updated Vector4.
     */
    multiplyInPlace(otherVector: Vector4): this;
    /**
     * Returns a new Vector4 set with the multiplication result of the current Vector4 and the given one.
     * @param otherVector vector to multiple with
     * @returns resulting new vector
     */
    multiply(otherVector: DeepImmutable<Vector4>): this;
    /**
     * Updates the given vector "result" with the multiplication result of the current Vector4 and the given one.
     * @param otherVector vector to multiple with
     * @param result vector to store the result
     * @returns result input
     */
    multiplyToRef<T extends Vector4>(otherVector: DeepImmutable<Vector4>, result: T): T;
    /**
     * Returns a new Vector4 set with the multiplication result of the given floats and the current Vector4 coordinates.
     * @param x x value multiply with
     * @param y y value multiply with
     * @param z z value multiply with
     * @param w w value multiply with
     * @returns resulting new vector
     */
    multiplyByFloats(x: number, y: number, z: number, w: number): this;
    /**
     * Returns a new Vector4 set with the division result of the current Vector4 by the given one.
     * @param otherVector vector to devide with
     * @returns resulting new vector
     */
    divide(otherVector: DeepImmutable<Vector4>): this;
    /**
     * Updates the given vector "result" with the division result of the current Vector4 by the given one.
     * @param otherVector vector to devide with
     * @param result vector to store the result
     * @returns result input
     */
    divideToRef<T extends Vector4>(otherVector: DeepImmutable<Vector4>, result: T): T;
    /**
     * Divides the current Vector3 coordinates by the given ones.
     * @param otherVector vector to devide with
     * @returns the updated Vector3.
     */
    divideInPlace(otherVector: DeepImmutable<Vector4>): this;
    /**
     * Updates the Vector4 coordinates with the minimum values between its own and the given vector ones
     * @param other defines the second operand
     * @returns the current updated Vector4
     */
    minimizeInPlace(other: DeepImmutable<Vector4>): this;
    /**
     * Updates the Vector4 coordinates with the maximum values between its own and the given vector ones
     * @param other defines the second operand
     * @returns the current updated Vector4
     */
    maximizeInPlace(other: DeepImmutable<Vector4>): this;
    /**
     * Gets a new Vector4 from current Vector4 floored values
     * @returns a new Vector4
     */
    floor(): this;
    /**
     * Gets a new Vector4 from current Vector4 fractional values
     * @returns a new Vector4
     */
    fract(): this;
    /**
     * Returns the Vector4 length (float).
     * @returns the length
     */
    length(): number;
    /**
     * Returns the Vector4 squared length (float).
     * @returns the length squared
     */
    lengthSquared(): number;
    /**
     * Normalizes in place the Vector4.
     * @returns the updated Vector4.
     */
    normalize(): this;
    /**
     * Returns a new Vector3 from the Vector4 (x, y, z) coordinates.
     * @returns this converted to a new vector3
     */
    toVector3(): Vector3;
    /**
     * Returns a new Vector4 copied from the current one.
     * @returns the new cloned vector
     */
    clone(): this;
    /**
     * Updates the current Vector4 with the given one coordinates.
     * @param source the source vector to copy from
     * @returns the updated Vector4.
     */
    copyFrom(source: DeepImmutable<Vector4>): this;
    /**
     * Updates the current Vector4 coordinates with the given floats.
     * @param x float to copy from
     * @param y float to copy from
     * @param z float to copy from
     * @param w float to copy from
     * @returns the updated Vector4.
     */
    copyFromFloats(x: number, y: number, z: number, w: number): this;
    /**
     * Updates the current Vector4 coordinates with the given floats.
     * @param x float to set from
     * @param y float to set from
     * @param z float to set from
     * @param w float to set from
     * @returns the updated Vector4.
     */
    set(x: number, y: number, z: number, w: number): this;
    /**
     * Copies the given float to the current Vector3 coordinates
     * @param v defines the x, y, z and w coordinates of the operand
     * @returns the current updated Vector3
     */
    setAll(v: number): this;
    /**
     * Returns a new Vector4 set from the starting index of the given array.
     * @param array the array to pull values from
     * @param offset the offset into the array to start at
     * @returns the new vector
     */
    static FromArray(array: DeepImmutable<ArrayLike<number>>, offset?: number): Vector4;
    /**
     * Updates the given vector "result" from the starting index of the given array.
     * @param array the array to pull values from
     * @param offset the offset into the array to start at
     * @param result the vector to store the result in
     * @returns result input
     */
    static FromArrayToRef<T extends Vector4>(array: DeepImmutable<ArrayLike<number>>, offset: number, result: T): T;
    /**
     * Updates the given vector "result" from the starting index of the given Float32Array.
     * @param array the array to pull values from
     * @param offset the offset into the array to start at
     * @param result the vector to store the result in
     * @returns result input
     */
    static FromFloatArrayToRef<T extends Vector4>(array: DeepImmutable<Float32Array>, offset: number, result: T): T;
    /**
     * Updates the given vector "result" coordinates from the given floats.
     * @param x float to set from
     * @param y float to set from
     * @param z float to set from
     * @param w float to set from
     * @param result the vector to the floats in
     * @returns result input
     */
    static FromFloatsToRef<T extends Vector4>(x: number, y: number, z: number, w: number, result: T): T;
    /**
     * Returns a new Vector4 set to (0.0, 0.0, 0.0, 0.0)
     * @returns the new vector
     */
    static Zero(): Vector4;
    /**
     * Returns a new Vector4 set to (1.0, 1.0, 1.0, 1.0)
     * @returns the new vector
     */
    static One(): Vector4;
    /**
     * Gets a zero Vector4 that must not be updated
     */
    static get ZeroReadOnly(): DeepImmutable<Vector4>;
    /**
     * Returns a new normalized Vector4 from the given one.
     * @param vector the vector to normalize
     * @returns the vector
     */
    static Normalize(vector: DeepImmutable<Vector4>): Vector4;
    /**
     * Updates the given vector "result" from the normalization of the given one.
     * @param vector the vector to normalize
     * @param result the vector to store the result in
     * @returns result input
     */
    static NormalizeToRef<T extends Vector4>(vector: DeepImmutable<Vector4>, result: T): T;
    /**
     * Returns a vector with the minimum values from the left and right vectors
     * @param left left vector to minimize
     * @param right right vector to minimize
     * @returns a new vector with the minimum of the left and right vector values
     */
    static Minimize<T extends Vector4>(left: DeepImmutable<T>, right: DeepImmutable<Vector4>): T;
    /**
     * Returns a vector with the maximum values from the left and right vectors
     * @param left left vector to maximize
     * @param right right vector to maximize
     * @returns a new vector with the maximum of the left and right vector values
     */
    static Maximize<T extends Vector4>(left: DeepImmutable<T>, right: DeepImmutable<Vector4>): T;
    /**
     * Returns the distance (float) between the vectors "value1" and "value2".
     * @param value1 value to calulate the distance between
     * @param value2 value to calulate the distance between
     * @returns the distance between the two vectors
     */
    static Distance(value1: DeepImmutable<Vector4>, value2: DeepImmutable<Vector4>): number;
    /**
     * Returns the squared distance (float) between the vectors "value1" and "value2".
     * @param value1 value to calulate the distance between
     * @param value2 value to calulate the distance between
     * @returns the distance between the two vectors squared
     */
    static DistanceSquared(value1: DeepImmutable<Vector4>, value2: DeepImmutable<Vector4>): number;
    /**
     * Returns a new Vector4 located at the center between the vectors "value1" and "value2".
     * @param value1 value to calulate the center between
     * @param value2 value to calulate the center between
     * @returns the center between the two vectors
     */
    static Center(value1: DeepImmutable<Vector4>, value2: DeepImmutable<Vector4>): Vector4;
    /**
     * Gets the center of the vectors "value1" and "value2" and stores the result in the vector "ref"
     * @param value1 defines first vector
     * @param value2 defines second vector
     * @param ref defines third vector
     * @returns ref
     */
    static CenterToRef<T extends Vector4>(value1: DeepImmutable<Vector4>, value2: DeepImmutable<Vector4>, ref: T): T;
    /**
     * Returns a new Vector4 set with the result of the transformation by the given matrix of the given vector.
     * This method computes tranformed coordinates only, not transformed direction vectors (ie. it takes translation in account)
     * The difference with Vector3.TransformCoordinates is that the w component is not used to divide the other coordinates but is returned in the w coordinate instead
     * @param vector defines the Vector3 to transform
     * @param transformation defines the transformation matrix
     * @returns the transformed Vector4
     */
    static TransformCoordinates(vector: DeepImmutable<Vector3>, transformation: DeepImmutable<Matrix>): Vector4;
    /**
     * Sets the given vector "result" coordinates with the result of the transformation by the given matrix of the given vector
     * This method computes tranformed coordinates only, not transformed direction vectors (ie. it takes translation in account)
     * The difference with Vector3.TransformCoordinatesToRef is that the w component is not used to divide the other coordinates but is returned in the w coordinate instead
     * @param vector defines the Vector3 to transform
     * @param transformation defines the transformation matrix
     * @param result defines the Vector4 where to store the result
     * @returns result input
     */
    static TransformCoordinatesToRef<T extends Vector4>(vector: DeepImmutable<Vector3>, transformation: DeepImmutable<Matrix>, result: T): T;
    /**
     * Sets the given vector "result" coordinates with the result of the transformation by the given matrix of the given floats (x, y, z)
     * This method computes tranformed coordinates only, not transformed direction vectors
     * The difference with Vector3.TransformCoordinatesFromFloatsToRef is that the w component is not used to divide the other coordinates but is returned in the w coordinate instead
     * @param x define the x coordinate of the source vector
     * @param y define the y coordinate of the source vector
     * @param z define the z coordinate of the source vector
     * @param transformation defines the transformation matrix
     * @param result defines the Vector4 where to store the result
     * @returns result input
     */
    static TransformCoordinatesFromFloatsToRef<T extends Vector4>(x: number, y: number, z: number, transformation: DeepImmutable<Matrix>, result: T): T;
    /**
     * Returns a new Vector4 set with the result of the normal transformation by the given matrix of the given vector.
     * This methods computes transformed normalized direction vectors only.
     * @param vector the vector to transform
     * @param transformation the transformation matrix to apply
     * @returns the new vector
     */
    static TransformNormal<T extends Vector4>(vector: DeepImmutable<T>, transformation: DeepImmutable<Matrix>): T;
    /**
     * Sets the given vector "result" with the result of the normal transformation by the given matrix of the given vector.
     * This methods computes transformed normalized direction vectors only.
     * @param vector the vector to transform
     * @param transformation the transformation matrix to apply
     * @param result the vector to store the result in
     * @returns result input
     */
    static TransformNormalToRef<T extends Vector4>(vector: DeepImmutable<Vector4>, transformation: DeepImmutable<Matrix>, result: T): T;
    /**
     * Sets the given vector "result" with the result of the normal transformation by the given matrix of the given floats (x, y, z, w).
     * This methods computes transformed normalized direction vectors only.
     * @param x value to transform
     * @param y value to transform
     * @param z value to transform
     * @param w value to transform
     * @param transformation the transformation matrix to apply
     * @param result the vector to store the results in
     * @returns result input
     */
    static TransformNormalFromFloatsToRef<T extends Vector4>(x: number, y: number, z: number, w: number, transformation: DeepImmutable<Matrix>, result: T): T;
    /**
     * Creates a new Vector4 from a Vector3
     * @param source defines the source data
     * @param w defines the 4th component (default is 0)
     * @returns a new Vector4
     */
    static FromVector3(source: Vector3, w?: number): Vector4;
}
/**
 * Class used to store quaternion data
 * Example Playground - Overview - https://playground.babylonjs.com/#L49EJ7#100
 * @see https://en.wikipedia.org/wiki/Quaternion
 * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/transforms
 */
export class Quaternion {
    /** @internal */
    _x: number;
    /** @internal */
    _y: number;
    /** @internal */
    _z: number;
    /** @internal */
    _w: number;
    /** @internal */
    _isDirty: boolean;
    /** Gets or sets the x coordinate */
    get x(): number;
    set x(value: number);
    /** Gets or sets the y coordinate */
    get y(): number;
    set y(value: number);
    /** Gets or sets the z coordinate */
    get z(): number;
    set z(value: number);
    /** Gets or sets the w coordinate */
    get w(): number;
    set w(value: number);
    /**
     * Creates a new Quaternion from the given floats
     * @param x defines the first component (0 by default)
     * @param y defines the second component (0 by default)
     * @param z defines the third component (0 by default)
     * @param w defines the fourth component (1.0 by default)
     */
    constructor(x?: number, y?: number, z?: number, w?: number);
    /**
     * Gets a string representation for the current quaternion
     * @returns a string with the Quaternion coordinates
     */
    toString(): string;
    /**
     * Gets the class name of the quaternion
     * @returns the string "Quaternion"
     */
    getClassName(): string;
    /**
     * Gets a hash code for this quaternion
     * @returns the quaternion hash code
     */
    getHashCode(): number;
    /**
     * Copy the quaternion to an array
     * Example Playground https://playground.babylonjs.com/#L49EJ7#13
     * @returns a new array populated with 4 elements from the quaternion coordinates
     */
    asArray(): number[];
    /**
     * Stores from the starting index in the given array the Quaternion successive values
     * Example Playground https://playground.babylonjs.com/#L49EJ7#59
     * @param array defines the array where to store the x,y,z,w components
     * @param index defines an optional index in the target array to define where to start storing values
     * @returns the current Quaternion object
     */
    toArray(array: FloatArray, index?: number): Quaternion;
    /**
     * Check if two quaternions are equals
     * Example Playground https://playground.babylonjs.com/#L49EJ7#38
     * @param otherQuaternion defines the second operand
     * @returns true if the current quaternion and the given one coordinates are strictly equals
     */
    equals(otherQuaternion: DeepImmutable<Quaternion>): boolean;
    /**
     * Gets a boolean if two quaternions are equals (using an epsilon value)
     * Example Playground https://playground.babylonjs.com/#L49EJ7#37
     * @param otherQuaternion defines the other quaternion
     * @param epsilon defines the minimal distance to consider equality
     * @returns true if the given quaternion coordinates are close to the current ones by a distance of epsilon.
     */
    equalsWithEpsilon(otherQuaternion: DeepImmutable<Quaternion>, epsilon?: number): boolean;
    /**
     * Clone the current quaternion
     * Example Playground https://playground.babylonjs.com/#L49EJ7#12
     * @returns a new quaternion copied from the current one
     */
    clone(): this;
    /**
     * Copy a quaternion to the current one
     * Example Playground https://playground.babylonjs.com/#L49EJ7#86
     * @param other defines the other quaternion
     * @returns the updated current quaternion
     */
    copyFrom(other: DeepImmutable<Quaternion>): this;
    /**
     * Updates the current quaternion with the given float coordinates
     * Example Playground https://playground.babylonjs.com/#L49EJ7#87
     * @param x defines the x coordinate
     * @param y defines the y coordinate
     * @param z defines the z coordinate
     * @param w defines the w coordinate
     * @returns the updated current quaternion
     */
    copyFromFloats(x: number, y: number, z: number, w: number): this;
    /**
     * Updates the current quaternion from the given float coordinates
     * Example Playground https://playground.babylonjs.com/#L49EJ7#56
     * @param x defines the x coordinate
     * @param y defines the y coordinate
     * @param z defines the z coordinate
     * @param w defines the w coordinate
     * @returns the updated current quaternion
     */
    set(x: number, y: number, z: number, w: number): this;
    /**
     * Adds two quaternions
     * Example Playground https://playground.babylonjs.com/#L49EJ7#10
     * @param other defines the second operand
     * @returns a new quaternion as the addition result of the given one and the current quaternion
     */
    add(other: DeepImmutable<Quaternion>): this;
    /**
     * Add a quaternion to the current one
     * Example Playground https://playground.babylonjs.com/#L49EJ7#11
     * @param other defines the quaternion to add
     * @returns the current quaternion
     */
    addInPlace(other: DeepImmutable<Quaternion>): this;
    /**
     * Subtract two quaternions
     * Example Playground https://playground.babylonjs.com/#L49EJ7#57
     * @param other defines the second operand
     * @returns a new quaternion as the subtraction result of the given one from the current one
     */
    subtract(other: Quaternion): this;
    /**
     * Subtract a quaternion to the current one
     * Example Playground https://playground.babylonjs.com/#L49EJ7#58
     * @param other defines the quaternion to subtract
     * @returns the current quaternion
     */
    subtractInPlace(other: DeepImmutable<Quaternion>): this;
    /**
     * Multiplies the current quaternion by a scale factor
     * Example Playground https://playground.babylonjs.com/#L49EJ7#88
     * @param value defines the scale factor
     * @returns a new quaternion set by multiplying the current quaternion coordinates by the float "scale"
     */
    scale(value: number): this;
    /**
     * Scale the current quaternion values by a factor and stores the result to a given quaternion
     * Example Playground https://playground.babylonjs.com/#L49EJ7#89
     * @param scale defines the scale factor
     * @param result defines the Quaternion object where to store the result
     * @returns result input
     */
    scaleToRef<T extends Quaternion>(scale: number, result: T): T;
    /**
     * Multiplies in place the current quaternion by a scale factor
     * Example Playground https://playground.babylonjs.com/#L49EJ7#90
     * @param value defines the scale factor
     * @returns the current modified quaternion
     */
    scaleInPlace(value: number): this;
    /**
     * Scale the current quaternion values by a factor and add the result to a given quaternion
     * Example Playground https://playground.babylonjs.com/#L49EJ7#91
     * @param scale defines the scale factor
     * @param result defines the Quaternion object where to store the result
     * @returns result input
     */
    scaleAndAddToRef<T extends Quaternion>(scale: number, result: T): T;
    /**
     * Multiplies two quaternions
     * Example Playground https://playground.babylonjs.com/#L49EJ7#43
     * @param q1 defines the second operand
     * @returns a new quaternion set as the multiplication result of the current one with the given one "q1"
     */
    multiply(q1: DeepImmutable<Quaternion>): this;
    /**
     * Sets the given "result" as the the multiplication result of the current one with the given one "q1"
     * Example Playground https://playground.babylonjs.com/#L49EJ7#45
     * @param q1 defines the second operand
     * @param result defines the target quaternion
     * @returns the current quaternion
     */
    multiplyToRef<T extends Quaternion>(q1: DeepImmutable<Quaternion>, result: T): T;
    /**
     * Updates the current quaternion with the multiplication of itself with the given one "q1"
     * Example Playground https://playground.babylonjs.com/#L49EJ7#46
     * @param q1 defines the second operand
     * @returns the currentupdated quaternion
     */
    multiplyInPlace(q1: DeepImmutable<Quaternion>): this;
    /**
     * Conjugates the current quaternion and stores the result in the given quaternion
     * Example Playground https://playground.babylonjs.com/#L49EJ7#81
     * @param ref defines the target quaternion
     * @returns result input
     */
    conjugateToRef<T extends Quaternion>(ref: T): T;
    /**
     * Conjugates in place the current quaternion
     * Example Playground https://playground.babylonjs.com/#L49EJ7#82
     * @returns the current updated quaternion
     */
    conjugateInPlace(): this;
    /**
     * Conjugates (1-q) the current quaternion
     * Example Playground https://playground.babylonjs.com/#L49EJ7#83
     * @returns a new quaternion
     */
    conjugate(): this;
    /**
     * Returns the inverse of the current quaternion
     * Example Playground https://playground.babylonjs.com/#L49EJ7#84
     * @returns a new quaternion
     */
    invert(): this;
    /**
     * Invert in place the current quaternion
     * Example Playground https://playground.babylonjs.com/#L49EJ7#85
     * @returns this quaternion
     */
    invertInPlace(): this;
    /**
     * Gets squared length of current quaternion
     * Example Playground https://playground.babylonjs.com/#L49EJ7#29
     * @returns the quaternion length (float)
     */
    lengthSquared(): number;
    /**
     * Gets length of current quaternion
     * Example Playground https://playground.babylonjs.com/#L49EJ7#28
     * @returns the quaternion length (float)
     */
    length(): number;
    /**
     * Normalize in place the current quaternion
     * Example Playground https://playground.babylonjs.com/#L49EJ7#54
     * @returns the current updated quaternion
     */
    normalize(): this;
    /**
     * Normalize a copy of the current quaternion
     * Example Playground https://playground.babylonjs.com/#L49EJ7#55
     * @returns the normalized quaternion
     */
    normalizeToNew(): this;
    /**
     * Returns a new Vector3 set with the Euler angles translated from the current quaternion
     * Example Playground https://playground.babylonjs.com/#L49EJ7#32
     * @returns a new Vector3 containing the Euler angles
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/transforms/center_origin/rotation_conventions
     */
    toEulerAngles(): Vector3;
    /**
     * Sets the given vector3 "result" with the Euler angles translated from the current quaternion
     * Example Playground https://playground.babylonjs.com/#L49EJ7#31
     * @param result defines the vector which will be filled with the Euler angles
     * @returns result input
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/transforms/center_origin/rotation_conventions
     */
    toEulerAnglesToRef<T extends Vector3>(result: T): T;
    /**
     * Updates the given rotation matrix with the current quaternion values
     * Example Playground https://playground.babylonjs.com/#L49EJ7#67
     * @param result defines the target matrix
     * @returns the current unchanged quaternion
     */
    toRotationMatrix<T extends Matrix>(result: T): T;
    /**
     * Updates the current quaternion from the given rotation matrix values
     * Example Playground https://playground.babylonjs.com/#L49EJ7#41
     * @param matrix defines the source matrix
     * @returns the current updated quaternion
     */
    fromRotationMatrix(matrix: DeepImmutable<Matrix>): this;
    /**
     * Creates a new quaternion from a rotation matrix
     * Example Playground https://playground.babylonjs.com/#L49EJ7#101
     * @param matrix defines the source matrix
     * @returns a new quaternion created from the given rotation matrix values
     */
    static FromRotationMatrix(matrix: DeepImmutable<Matrix>): Quaternion;
    /**
     * Updates the given quaternion with the given rotation matrix values
     * Example Playground https://playground.babylonjs.com/#L49EJ7#102
     * @param matrix defines the source matrix
     * @param result defines the target quaternion
     * @returns result input
     */
    static FromRotationMatrixToRef<T extends Quaternion>(matrix: DeepImmutable<Matrix>, result: T): T;
    /**
     * Returns the dot product (float) between the quaternions "left" and "right"
     * Example Playground https://playground.babylonjs.com/#L49EJ7#61
     * @param left defines the left operand
     * @param right defines the right operand
     * @returns the dot product
     */
    static Dot(left: DeepImmutable<Quaternion>, right: DeepImmutable<Quaternion>): number;
    /**
     * Checks if the orientations of two rotation quaternions are close to each other
     * Example Playground https://playground.babylonjs.com/#L49EJ7#60
     * @param quat0 defines the first quaternion to check
     * @param quat1 defines the second quaternion to check
     * @param epsilon defines closeness, 0 same orientation, 1 PI apart, default 0.1
     * @returns true if the two quaternions are close to each other within epsilon
     */
    static AreClose(quat0: DeepImmutable<Quaternion>, quat1: DeepImmutable<Quaternion>, epsilon?: number): boolean;
    /**
     * Smooth interpolation between two quaternions using Slerp
     * Example Playground https://playground.babylonjs.com/#L49EJ7#93
     * @param source source quaternion
     * @param goal goal quaternion
     * @param deltaTime current interpolation frame
     * @param lerpTime total interpolation time
     * @param result the smoothed quaternion
     */
    static SmoothToRef<T extends Quaternion>(source: Quaternion, goal: Quaternion, deltaTime: number, lerpTime: number, result: T): T;
    /**
     * Creates an empty quaternion
     * @returns a new quaternion set to (0.0, 0.0, 0.0)
     */
    static Zero(): Quaternion;
    /**
     * Inverse a given quaternion
     * Example Playground https://playground.babylonjs.com/#L49EJ7#103
     * @param q defines the source quaternion
     * @returns a new quaternion as the inverted current quaternion
     */
    static Inverse<T extends Quaternion>(q: DeepImmutable<T>): T;
    /**
     * Inverse a given quaternion
     * Example Playground https://playground.babylonjs.com/#L49EJ7#104
     * @param q defines the source quaternion
     * @param result the quaternion the result will be stored in
     * @returns the result quaternion
     */
    static InverseToRef<T extends Quaternion>(q: Quaternion, result: T): T;
    /**
     * Creates an identity quaternion
     * @returns the identity quaternion
     */
    static Identity(): Quaternion;
    /**
     * Gets a boolean indicating if the given quaternion is identity
     * @param quaternion defines the quaternion to check
     * @returns true if the quaternion is identity
     */
    static IsIdentity(quaternion: DeepImmutable<Quaternion>): boolean;
    /**
     * Creates a quaternion from a rotation around an axis
     * Example Playground https://playground.babylonjs.com/#L49EJ7#72
     * @param axis defines the axis to use
     * @param angle defines the angle to use
     * @returns a new quaternion created from the given axis (Vector3) and angle in radians (float)
     */
    static RotationAxis(axis: DeepImmutable<Vector3>, angle: number): Quaternion;
    /**
     * Creates a rotation around an axis and stores it into the given quaternion
     * Example Playground https://playground.babylonjs.com/#L49EJ7#73
     * @param axis defines the axis to use
     * @param angle defines the angle to use
     * @param result defines the target quaternion
     * @returns the target quaternion
     */
    static RotationAxisToRef<T extends Quaternion>(axis: DeepImmutable<Vector3>, angle: number, result: T): T;
    /**
     * Creates a new quaternion from data stored into an array
     * Example Playground https://playground.babylonjs.com/#L49EJ7#63
     * @param array defines the data source
     * @param offset defines the offset in the source array where the data starts
     * @returns a new quaternion
     */
    static FromArray(array: DeepImmutable<ArrayLike<number>>, offset?: number): Quaternion;
    /**
     * Updates the given quaternion "result" from the starting index of the given array.
     * Example Playground https://playground.babylonjs.com/#L49EJ7#64
     * @param array the array to pull values from
     * @param offset the offset into the array to start at
     * @param result the quaternion to store the result in
     * @returns result input
     */
    static FromArrayToRef<T extends Quaternion>(array: DeepImmutable<ArrayLike<number>>, offset: number, result: T): T;
    /**
     * Create a quaternion from Euler rotation angles
     * Example Playground https://playground.babylonjs.com/#L49EJ7#33
     * @param x Pitch
     * @param y Yaw
     * @param z Roll
     * @returns the new Quaternion
     */
    static FromEulerAngles(x: number, y: number, z: number): Quaternion;
    /**
     * Updates a quaternion from Euler rotation angles
     * Example Playground https://playground.babylonjs.com/#L49EJ7#34
     * @param x Pitch
     * @param y Yaw
     * @param z Roll
     * @param result the quaternion to store the result
     * @returns the updated quaternion
     */
    static FromEulerAnglesToRef<T extends Quaternion>(x: number, y: number, z: number, result: T): T;
    /**
     * Create a quaternion from Euler rotation vector
     * Example Playground https://playground.babylonjs.com/#L49EJ7#35
     * @param vec the Euler vector (x Pitch, y Yaw, z Roll)
     * @returns the new Quaternion
     */
    static FromEulerVector(vec: DeepImmutable<Vector3>): Quaternion;
    /**
     * Updates a quaternion from Euler rotation vector
     * Example Playground https://playground.babylonjs.com/#L49EJ7#36
     * @param vec the Euler vector (x Pitch, y Yaw, z Roll)
     * @param result the quaternion to store the result
     * @returns the updated quaternion
     */
    static FromEulerVectorToRef<T extends Quaternion>(vec: DeepImmutable<Vector3>, result: T): T;
    /**
     * Updates a quaternion so that it rotates vector vecFrom to vector vecTo
     * Example Playground - https://playground.babylonjs.com/#L49EJ7#70
     * @param vecFrom defines the direction vector from which to rotate
     * @param vecTo defines the direction vector to which to rotate
     * @param result the quaternion to store the result
     * @returns the updated quaternion
     */
    static FromUnitVectorsToRef<T extends Quaternion>(vecFrom: DeepImmutable<Vector3>, vecTo: DeepImmutable<Vector3>, result: T): T;
    /**
     * Creates a new quaternion from the given Euler float angles (y, x, z)
     * Example Playground https://playground.babylonjs.com/#L49EJ7#77
     * @param yaw defines the rotation around Y axis
     * @param pitch defines the rotation around X axis
     * @param roll defines the rotation around Z axis
     * @returns the new quaternion
     */
    static RotationYawPitchRoll(yaw: number, pitch: number, roll: number): Quaternion;
    /**
     * Creates a new rotation from the given Euler float angles (y, x, z) and stores it in the target quaternion
     * Example Playground https://playground.babylonjs.com/#L49EJ7#78
     * @param yaw defines the rotation around Y axis
     * @param pitch defines the rotation around X axis
     * @param roll defines the rotation around Z axis
     * @param result defines the target quaternion
     * @returns result input
     */
    static RotationYawPitchRollToRef<T extends Quaternion>(yaw: number, pitch: number, roll: number, result: T): T;
    /**
     * Creates a new quaternion from the given Euler float angles expressed in z-x-z orientation
     * Example Playground https://playground.babylonjs.com/#L49EJ7#68
     * @param alpha defines the rotation around first axis
     * @param beta defines the rotation around second axis
     * @param gamma defines the rotation around third axis
     * @returns the new quaternion
     */
    static RotationAlphaBetaGamma(alpha: number, beta: number, gamma: number): Quaternion;
    /**
     * Creates a new quaternion from the given Euler float angles expressed in z-x-z orientation and stores it in the target quaternion
     * Example Playground https://playground.babylonjs.com/#L49EJ7#69
     * @param alpha defines the rotation around first axis
     * @param beta defines the rotation around second axis
     * @param gamma defines the rotation around third axis
     * @param result defines the target quaternion
     * @returns result input
     */
    static RotationAlphaBetaGammaToRef<T extends Quaternion>(alpha: number, beta: number, gamma: number, result: T): T;
    /**
     * Creates a new quaternion containing the rotation value to reach the target (axis1, axis2, axis3) orientation as a rotated XYZ system (axis1, axis2 and axis3 are normalized during this operation)
     * Example Playground https://playground.babylonjs.com/#L49EJ7#75
     * @param axis1 defines the first axis
     * @param axis2 defines the second axis
     * @param axis3 defines the third axis
     * @returns the new quaternion
     */
    static RotationQuaternionFromAxis(axis1: DeepImmutable<Vector3>, axis2: DeepImmutable<Vector3>, axis3: DeepImmutable<Vector3>): Quaternion;
    /**
     * Creates a rotation value to reach the target (axis1, axis2, axis3) orientation as a rotated XYZ system (axis1, axis2 and axis3 are normalized during this operation) and stores it in the target quaternion
     * Example Playground https://playground.babylonjs.com/#L49EJ7#76
     * @param axis1 defines the first axis
     * @param axis2 defines the second axis
     * @param axis3 defines the third axis
     * @param ref defines the target quaternion
     * @returns result input
     */
    static RotationQuaternionFromAxisToRef<T extends Quaternion>(axis1: DeepImmutable<Vector3>, axis2: DeepImmutable<Vector3>, axis3: DeepImmutable<Vector3>, ref: T): T;
    /**
     * Creates a new rotation value to orient an object to look towards the given forward direction, the up direction being oriented like "up".
     * This function works in left handed mode
     * Example Playground https://playground.babylonjs.com/#L49EJ7#96
     * @param forward defines the forward direction - Must be normalized and orthogonal to up.
     * @param up defines the up vector for the entity - Must be normalized and orthogonal to forward.
     * @returns A new quaternion oriented toward the specified forward and up.
     */
    static FromLookDirectionLH(forward: DeepImmutable<Vector3>, up: DeepImmutable<Vector3>): Quaternion;
    /**
     * Creates a new rotation value to orient an object to look towards the given forward direction with the up direction being oriented like "up", and stores it in the target quaternion.
     * This function works in left handed mode
     * Example Playground https://playground.babylonjs.com/#L49EJ7#97
     * @param forward defines the forward direction - Must be normalized and orthogonal to up.
     * @param up defines the up vector for the entity - Must be normalized and orthogonal to forward.
     * @param ref defines the target quaternion.
     * @returns result input
     */
    static FromLookDirectionLHToRef<T extends Quaternion>(forward: DeepImmutable<Vector3>, up: DeepImmutable<Vector3>, ref: T): T;
    /**
     * Creates a new rotation value to orient an object to look towards the given forward direction, the up direction being oriented like "up".
     * This function works in right handed mode
     * Example Playground https://playground.babylonjs.com/#L49EJ7#98
     * @param forward defines the forward direction - Must be normalized and orthogonal to up.
     * @param up defines the up vector for the entity - Must be normalized and orthogonal to forward.
     * @returns A new quaternion oriented toward the specified forward and up.
     */
    static FromLookDirectionRH(forward: DeepImmutable<Vector3>, up: DeepImmutable<Vector3>): Quaternion;
    /**
     * Creates a new rotation value to orient an object to look towards the given forward direction with the up direction being oriented like "up", and stores it in the target quaternion.
     * This function works in right handed mode
     * Example Playground https://playground.babylonjs.com/#L49EJ7#105
     * @param forward defines the forward direction - Must be normalized and orthogonal to up.
     * @param up defines the up vector for the entity - Must be normalized and orthogonal to forward.
     * @param ref defines the target quaternion.
     * @returns result input
     */
    static FromLookDirectionRHToRef<T extends Quaternion>(forward: DeepImmutable<Vector3>, up: DeepImmutable<Vector3>, ref: T): T;
    /**
     * Interpolates between two quaternions
     * Example Playground https://playground.babylonjs.com/#L49EJ7#79
     * @param left defines first quaternion
     * @param right defines second quaternion
     * @param amount defines the gradient to use
     * @returns the new interpolated quaternion
     */
    static Slerp(left: DeepImmutable<Quaternion>, right: DeepImmutable<Quaternion>, amount: number): Quaternion;
    /**
     * Interpolates between two quaternions and stores it into a target quaternion
     * Example Playground https://playground.babylonjs.com/#L49EJ7#92
     * @param left defines first quaternion
     * @param right defines second quaternion
     * @param amount defines the gradient to use
     * @param result defines the target quaternion
     * @returns result input
     */
    static SlerpToRef<T extends Quaternion>(left: DeepImmutable<Quaternion>, right: DeepImmutable<Quaternion>, amount: number, result: T): T;
    /**
     * Interpolate between two quaternions using Hermite interpolation
     * Example Playground https://playground.babylonjs.com/#L49EJ7#47
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/drawCurves#hermite-quaternion-spline
     * @param value1 defines first quaternion
     * @param tangent1 defines the incoming tangent
     * @param value2 defines second quaternion
     * @param tangent2 defines the outgoing tangent
     * @param amount defines the target quaternion
     * @returns the new interpolated quaternion
     */
    static Hermite<T extends Quaternion>(value1: DeepImmutable<T>, tangent1: DeepImmutable<Quaternion>, value2: DeepImmutable<Quaternion>, tangent2: DeepImmutable<Quaternion>, amount: number): T;
    /**
     * Returns a new Quaternion which is the 1st derivative of the Hermite spline defined by the quaternions "value1", "value2", "tangent1", "tangent2".
     * Example Playground https://playground.babylonjs.com/#L49EJ7#48
     * @param value1 defines the first control point
     * @param tangent1 defines the first tangent
     * @param value2 defines the second control point
     * @param tangent2 defines the second tangent
     * @param time define where the derivative must be done
     * @returns 1st derivative
     */
    static Hermite1stDerivative<T extends Quaternion>(value1: DeepImmutable<T>, tangent1: DeepImmutable<Quaternion>, value2: DeepImmutable<Quaternion>, tangent2: DeepImmutable<Quaternion>, time: number): T;
    /**
     * Update a Quaternion with the 1st derivative of the Hermite spline defined by the quaternions "value1", "value2", "tangent1", "tangent2".
     * Example Playground https://playground.babylonjs.com/#L49EJ7#49
     * @param value1 defines the first control point
     * @param tangent1 defines the first tangent
     * @param value2 defines the second control point
     * @param tangent2 defines the second tangent
     * @param time define where the derivative must be done
     * @param result define where to store the derivative
     * @returns result input
     */
    static Hermite1stDerivativeToRef<T extends Quaternion>(value1: DeepImmutable<Quaternion>, tangent1: DeepImmutable<Quaternion>, value2: DeepImmutable<Quaternion>, tangent2: DeepImmutable<Quaternion>, time: number, result: T): T;
}
/**
 * Class used to store matrix data (4x4)
 * Note on matrix definitions in Babylon.js for setting values directly
 * rather than using one of the methods available.
 * Matrix size is given by rows x columns.
 * A Vector3 is a 1 X 3 matrix [x, y, z].
 *
 * In Babylon.js multiplying a 1 x 3 matrix by a 4 x 4 matrix
 * is done using BABYLON.Vector4.TransformCoordinates(Vector3, Matrix).
 * and extending the passed Vector3 to a Vector4, V = [x, y, z, 1].
 * Let M be a matrix with elements m(row, column), so that
 * m(2, 3) is the element in row 2 column 3 of M.
 *
 * Multiplication is of the form VM and has the resulting Vector4
 * VM = [xm(0, 0) + ym(1, 0) + zm(2, 0) + m(3, 0), xm(0, 1) + ym(1, 1) + zm(2, 1) + m(3, 1), xm(0, 2) + ym(1, 2) + zm(2, 2) + m(3, 2), xm(0, 3) + ym(1, 3) + zm(2, 3) + m(3, 3)].
 * On the web you will find many examples that use the opposite convention of MV,
 * in which case to make use of the examples you will need to transpose the matrix.
 *
 * Example Playground - Overview Linear Algebra - https://playground.babylonjs.com/#AV9X17
 * Example Playground - Overview Transformation - https://playground.babylonjs.com/#AV9X17#1
 * Example Playground - Overview Projection - https://playground.babylonjs.com/#AV9X17#2
 */
export class Matrix {
    /**
     * Gets the precision of matrix computations
     */
    static get Use64Bits(): boolean;
    private static _UpdateFlagSeed;
    private static _IdentityReadOnly;
    private _isIdentity;
    private _isIdentityDirty;
    private _isIdentity3x2;
    private _isIdentity3x2Dirty;
    /**
     * Gets the update flag of the matrix which is an unique number for the matrix.
     * It will be incremented every time the matrix data change.
     * You can use it to speed the comparison between two versions of the same matrix.
     */
    updateFlag: number;
    private readonly _m;
    /**
     * Gets the internal data of the matrix
     */
    get m(): DeepImmutable<Float32Array | Array<number>>;
    /**
     * Update the updateFlag to indicate that the matrix has been updated
     */
    markAsUpdated(): void;
    private _updateIdentityStatus;
    /**
     * Creates an empty matrix (filled with zeros)
     */
    constructor();
    /**
     * Check if the current matrix is identity
     * @returns true is the matrix is the identity matrix
     */
    isIdentity(): boolean;
    /**
     * Check if the current matrix is identity as a texture matrix (3x2 store in 4x4)
     * @returns true is the matrix is the identity matrix
     */
    isIdentityAs3x2(): boolean;
    /**
     * Gets the determinant of the matrix
     * Example Playground - https://playground.babylonjs.com/#AV9X17#34
     * @returns the matrix determinant
     */
    determinant(): number;
    /**
     * Returns the matrix as a Float32Array or Array<number>
     * Example Playground - https://playground.babylonjs.com/#AV9X17#49
     * @returns the matrix underlying array
     */
    toArray(): DeepImmutable<Float32Array | Array<number>>;
    /**
     * Returns the matrix as a Float32Array or Array<number>
     * Example Playground - https://playground.babylonjs.com/#AV9X17#114
     * @returns the matrix underlying array.
     */
    asArray(): DeepImmutable<Float32Array | Array<number>>;
    /**
     * Inverts the current matrix in place
     * Example Playground - https://playground.babylonjs.com/#AV9X17#118
     * @returns the current inverted matrix
     */
    invert(): this;
    /**
     * Sets all the matrix elements to zero
     * @returns the current matrix
     */
    reset(): this;
    /**
     * Adds the current matrix with a second one
     * Example Playground - https://playground.babylonjs.com/#AV9X17#44
     * @param other defines the matrix to add
     * @returns a new matrix as the addition of the current matrix and the given one
     */
    add(other: DeepImmutable<Matrix>): this;
    /**
     * Sets the given matrix "result" to the addition of the current matrix and the given one
     * Example Playground - https://playground.babylonjs.com/#AV9X17#45
     * @param other defines the matrix to add
     * @param result defines the target matrix
     * @returns result input
     */
    addToRef<T extends Matrix>(other: DeepImmutable<Matrix>, result: T): T;
    /**
     * Adds in place the given matrix to the current matrix
     * Example Playground - https://playground.babylonjs.com/#AV9X17#46
     * @param other defines the second operand
     * @returns the current updated matrix
     */
    addToSelf(other: DeepImmutable<Matrix>): this;
    /**
     * Sets the given matrix to the current inverted Matrix
     * Example Playground - https://playground.babylonjs.com/#AV9X17#119
     * @param other defines the target matrix
     * @returns result input
     */
    invertToRef<T extends Matrix>(other: T): T;
    /**
     * add a value at the specified position in the current Matrix
     * Example Playground - https://playground.babylonjs.com/#AV9X17#47
     * @param index the index of the value within the matrix. between 0 and 15.
     * @param value the value to be added
     * @returns the current updated matrix
     */
    addAtIndex(index: number, value: number): this;
    /**
     * mutiply the specified position in the current Matrix by a value
     * @param index the index of the value within the matrix. between 0 and 15.
     * @param value the value to be added
     * @returns the current updated matrix
     */
    multiplyAtIndex(index: number, value: number): this;
    /**
     * Inserts the translation vector (using 3 floats) in the current matrix
     * Example Playground - https://playground.babylonjs.com/#AV9X17#120
     * @param x defines the 1st component of the translation
     * @param y defines the 2nd component of the translation
     * @param z defines the 3rd component of the translation
     * @returns the current updated matrix
     */
    setTranslationFromFloats(x: number, y: number, z: number): this;
    /**
     * Adds the translation vector (using 3 floats) in the current matrix
     * Example Playground - https://playground.babylonjs.com/#AV9X17#20
     * Example Playground - https://playground.babylonjs.com/#AV9X17#48
     * @param x defines the 1st component of the translation
     * @param y defines the 2nd component of the translation
     * @param z defines the 3rd component of the translation
     * @returns the current updated matrix
     */
    addTranslationFromFloats(x: number, y: number, z: number): this;
    /**
     * Inserts the translation vector in the current matrix
     * Example Playground - https://playground.babylonjs.com/#AV9X17#121
     * @param vector3 defines the translation to insert
     * @returns the current updated matrix
     */
    setTranslation(vector3: DeepImmutable<Vector3>): this;
    /**
     * Gets the translation value of the current matrix
     * Example Playground - https://playground.babylonjs.com/#AV9X17#122
     * @returns a new Vector3 as the extracted translation from the matrix
     */
    getTranslation(): Vector3;
    /**
     * Fill a Vector3 with the extracted translation from the matrix
     * Example Playground - https://playground.babylonjs.com/#AV9X17#123
     * @param result defines the Vector3 where to store the translation
     * @returns the current matrix
     */
    getTranslationToRef<T extends Vector3>(result: T): T;
    /**
     * Remove rotation and scaling part from the matrix
     * @returns the updated matrix
     */
    removeRotationAndScaling(): this;
    /**
     * Multiply two matrices
     * Example Playground - https://playground.babylonjs.com/#AV9X17#15
     * A.multiply(B) means apply B to A so result is B x A
     * @param other defines the second operand
     * @returns a new matrix set with the multiplication result of the current Matrix and the given one
     */
    multiply(other: DeepImmutable<Matrix>): this;
    /**
     * Copy the current matrix from the given one
     * Example Playground - https://playground.babylonjs.com/#AV9X17#21
     * @param other defines the source matrix
     * @returns the current updated matrix
     */
    copyFrom(other: DeepImmutable<Matrix>): this;
    /**
     * Populates the given array from the starting index with the current matrix values
     * @param array defines the target array
     * @param offset defines the offset in the target array where to start storing values
     * @returns the current matrix
     */
    copyToArray(array: Float32Array | Array<number>, offset?: number): this;
    /**
     * Sets the given matrix "result" with the multiplication result of the current Matrix and the given one
     * A.multiplyToRef(B, R) means apply B to A and store in R and R = B x A
     * Example Playground - https://playground.babylonjs.com/#AV9X17#16
     * @param other defines the second operand
     * @param result defines the matrix where to store the multiplication
     * @returns result input
     */
    multiplyToRef<T extends Matrix>(other: DeepImmutable<Matrix>, result: T): T;
    /**
     * Sets the Float32Array "result" from the given index "offset" with the multiplication of the current matrix and the given one
     * @param other defines the second operand
     * @param result defines the array where to store the multiplication
     * @param offset defines the offset in the target array where to start storing values
     * @returns the current matrix
     */
    multiplyToArray(other: DeepImmutable<Matrix>, result: Float32Array | Array<number>, offset: number): this;
    /**
     * Check equality between this matrix and a second one
     * @param value defines the second matrix to compare
     * @returns true is the current matrix and the given one values are strictly equal
     */
    equals(value: DeepImmutable<Matrix>): boolean;
    /**
     * Clone the current matrix
     * Example Playground - https://playground.babylonjs.com/#AV9X17#18
     * @returns a new matrix from the current matrix
     */
    clone(): this;
    /**
     * Returns the name of the current matrix class
     * @returns the string "Matrix"
     */
    getClassName(): string;
    /**
     * Gets the hash code of the current matrix
     * @returns the hash code
     */
    getHashCode(): number;
    /**
     * Decomposes the current Matrix into a translation, rotation and scaling components of the provided node
     * Example Playground - https://playground.babylonjs.com/#AV9X17#13
     * @param node the node to decompose the matrix to
     * @returns true if operation was successful
     */
    decomposeToTransformNode(node: TransformNode): boolean;
    /**
     * Decomposes the current Matrix into a translation, rotation and scaling components
     * Example Playground - https://playground.babylonjs.com/#AV9X17#12
     * @param scale defines the scale vector3 given as a reference to update
     * @param rotation defines the rotation quaternion given as a reference to update
     * @param translation defines the translation vector3 given as a reference to update
     * @param preserveScalingNode Use scaling sign coming from this node. Otherwise scaling sign might change.
     * @returns true if operation was successful
     */
    decompose(scale?: Vector3, rotation?: Quaternion, translation?: Vector3, preserveScalingNode?: TransformNode): boolean;
    /**
     * Gets specific row of the matrix
     * Example Playground - https://playground.babylonjs.com/#AV9X17#36
     * @param index defines the number of the row to get
     * @returns the index-th row of the current matrix as a new Vector4
     */
    getRow(index: number): Nullable<Vector4>;
    /**
     * Gets specific row of the matrix to ref
     * Example Playground - https://playground.babylonjs.com/#AV9X17#36
     * @param index defines the number of the row to get
     * @param rowVector vector to store the index-th row of the current matrix
     * @returns result input
     */
    getRowToRef<T extends Vector4>(index: number, rowVector: T): T;
    /**
     * Sets the index-th row of the current matrix to the vector4 values
     * Example Playground - https://playground.babylonjs.com/#AV9X17#36
     * @param index defines the number of the row to set
     * @param row defines the target vector4
     * @returns the updated current matrix
     */
    setRow(index: number, row: Vector4): this;
    /**
     * Compute the transpose of the matrix
     * Example Playground - https://playground.babylonjs.com/#AV9X17#40
     * @returns the new transposed matrix
     */
    transpose(): this;
    /**
     * Compute the transpose of the matrix and store it in a given matrix
     * Example Playground - https://playground.babylonjs.com/#AV9X17#41
     * @param result defines the target matrix
     * @returns result input
     */
    transposeToRef<T extends Matrix>(result: T): T;
    /**
     * Sets the index-th row of the current matrix with the given 4 x float values
     * Example Playground - https://playground.babylonjs.com/#AV9X17#36
     * @param index defines the row index
     * @param x defines the x component to set
     * @param y defines the y component to set
     * @param z defines the z component to set
     * @param w defines the w component to set
     * @returns the updated current matrix
     */
    setRowFromFloats(index: number, x: number, y: number, z: number, w: number): this;
    /**
     * Compute a new matrix set with the current matrix values multiplied by scale (float)
     * @param scale defines the scale factor
     * @returns a new matrix
     */
    scale(scale: number): this;
    /**
     * Scale the current matrix values by a factor to a given result matrix
     * @param scale defines the scale factor
     * @param result defines the matrix to store the result
     * @returns result input
     */
    scaleToRef<T extends Matrix>(scale: number, result: T): T;
    /**
     * Scale the current matrix values by a factor and add the result to a given matrix
     * @param scale defines the scale factor
     * @param result defines the Matrix to store the result
     * @returns result input
     */
    scaleAndAddToRef<T extends Matrix>(scale: number, result: T): T;
    /**
     * Writes to the given matrix a normal matrix, computed from this one (using values from identity matrix for fourth row and column).
     * Example Playground - https://playground.babylonjs.com/#AV9X17#17
     * @param ref matrix to store the result
     */
    toNormalMatrix<T extends Matrix>(ref: T): T;
    /**
     * Gets only rotation part of the current matrix
     * @returns a new matrix sets to the extracted rotation matrix from the current one
     */
    getRotationMatrix(): this;
    /**
     * Extracts the rotation matrix from the current one and sets it as the given "result"
     * @param result defines the target matrix to store data to
     * @returns result input
     */
    getRotationMatrixToRef<T extends Matrix>(result: T): T;
    /**
     * Toggles model matrix from being right handed to left handed in place and vice versa
     */
    toggleModelMatrixHandInPlace(): this;
    /**
     * Toggles projection matrix from being right handed to left handed in place and vice versa
     */
    toggleProjectionMatrixHandInPlace(): this;
    /**
     * Creates a matrix from an array
     * Example Playground - https://playground.babylonjs.com/#AV9X17#42
     * @param array defines the source array
     * @param offset defines an offset in the source array
     * @returns a new Matrix set from the starting index of the given array
     */
    static FromArray(array: DeepImmutable<ArrayLike<number>>, offset?: number): Matrix;
    /**
     * Copy the content of an array into a given matrix
     * Example Playground - https://playground.babylonjs.com/#AV9X17#43
     * @param array defines the source array
     * @param offset defines an offset in the source array
     * @param result defines the target matrix
     * @returns result input
     */
    static FromArrayToRef<T extends Matrix>(array: DeepImmutable<ArrayLike<number>>, offset: number, result: T): T;
    /**
     * Stores an array into a matrix after having multiplied each component by a given factor
     * Example Playground - https://playground.babylonjs.com/#AV9X17#50
     * @param array defines the source array
     * @param offset defines the offset in the source array
     * @param scale defines the scaling factor
     * @param result defines the target matrix
     * @returns result input
     */
    static FromFloat32ArrayToRefScaled<T extends Matrix>(array: DeepImmutable<Float32Array | Array<number>>, offset: number, scale: number, result: T): T;
    /**
     * Gets an identity matrix that must not be updated
     */
    static get IdentityReadOnly(): DeepImmutable<Matrix>;
    /**
     * Stores a list of values (16) inside a given matrix
     * @param initialM11 defines 1st value of 1st row
     * @param initialM12 defines 2nd value of 1st row
     * @param initialM13 defines 3rd value of 1st row
     * @param initialM14 defines 4th value of 1st row
     * @param initialM21 defines 1st value of 2nd row
     * @param initialM22 defines 2nd value of 2nd row
     * @param initialM23 defines 3rd value of 2nd row
     * @param initialM24 defines 4th value of 2nd row
     * @param initialM31 defines 1st value of 3rd row
     * @param initialM32 defines 2nd value of 3rd row
     * @param initialM33 defines 3rd value of 3rd row
     * @param initialM34 defines 4th value of 3rd row
     * @param initialM41 defines 1st value of 4th row
     * @param initialM42 defines 2nd value of 4th row
     * @param initialM43 defines 3rd value of 4th row
     * @param initialM44 defines 4th value of 4th row
     * @param result defines the target matrix
     * @returns result input
     */
    static FromValuesToRef(initialM11: number, initialM12: number, initialM13: number, initialM14: number, initialM21: number, initialM22: number, initialM23: number, initialM24: number, initialM31: number, initialM32: number, initialM33: number, initialM34: number, initialM41: number, initialM42: number, initialM43: number, initialM44: number, result: Matrix): void;
    /**
     * Creates new matrix from a list of values (16)
     * @param initialM11 defines 1st value of 1st row
     * @param initialM12 defines 2nd value of 1st row
     * @param initialM13 defines 3rd value of 1st row
     * @param initialM14 defines 4th value of 1st row
     * @param initialM21 defines 1st value of 2nd row
     * @param initialM22 defines 2nd value of 2nd row
     * @param initialM23 defines 3rd value of 2nd row
     * @param initialM24 defines 4th value of 2nd row
     * @param initialM31 defines 1st value of 3rd row
     * @param initialM32 defines 2nd value of 3rd row
     * @param initialM33 defines 3rd value of 3rd row
     * @param initialM34 defines 4th value of 3rd row
     * @param initialM41 defines 1st value of 4th row
     * @param initialM42 defines 2nd value of 4th row
     * @param initialM43 defines 3rd value of 4th row
     * @param initialM44 defines 4th value of 4th row
     * @returns the new matrix
     */
    static FromValues(initialM11: number, initialM12: number, initialM13: number, initialM14: number, initialM21: number, initialM22: number, initialM23: number, initialM24: number, initialM31: number, initialM32: number, initialM33: number, initialM34: number, initialM41: number, initialM42: number, initialM43: number, initialM44: number): Matrix;
    /**
     * Creates a new matrix composed by merging scale (vector3), rotation (quaternion) and translation (vector3)
     * Example Playground - https://playground.babylonjs.com/#AV9X17#24
     * @param scale defines the scale vector3
     * @param rotation defines the rotation quaternion
     * @param translation defines the translation vector3
     * @returns a new matrix
     */
    static Compose(scale: DeepImmutable<Vector3>, rotation: DeepImmutable<Quaternion>, translation: DeepImmutable<Vector3>): Matrix;
    /**
     * Sets a matrix to a value composed by merging scale (vector3), rotation (quaternion) and translation (vector3)
     * Example Playground - https://playground.babylonjs.com/#AV9X17#25
     * @param scale defines the scale vector3
     * @param rotation defines the rotation quaternion
     * @param translation defines the translation vector3
     * @param result defines the target matrix
     * @returns result input
     */
    static ComposeToRef<T extends Matrix>(scale: DeepImmutable<Vector3>, rotation: DeepImmutable<Quaternion>, translation: DeepImmutable<Vector3>, result: T): T;
    /**
     * Creates a new identity matrix
     * @returns a new identity matrix
     */
    static Identity(): Matrix;
    /**
     * Creates a new identity matrix and stores the result in a given matrix
     * @param result defines the target matrix
     * @returns result input
     */
    static IdentityToRef<T extends Matrix>(result: T): T;
    /**
     * Creates a new zero matrix
     * @returns a new zero matrix
     */
    static Zero(): Matrix;
    /**
     * Creates a new rotation matrix for "angle" radians around the X axis
     * Example Playground - https://playground.babylonjs.com/#AV9X17#97
     * @param angle defines the angle (in radians) to use
     * @returns the new matrix
     */
    static RotationX(angle: number): Matrix;
    /**
     * Creates a new matrix as the invert of a given matrix
     * Example Playground - https://playground.babylonjs.com/#AV9X17#124
     * @param source defines the source matrix
     * @returns the new matrix
     */
    static Invert<T extends Matrix>(source: DeepImmutable<T>): T;
    /**
     * Creates a new rotation matrix for "angle" radians around the X axis and stores it in a given matrix
     * Example Playground - https://playground.babylonjs.com/#AV9X17#98
     * @param angle defines the angle (in radians) to use
     * @param result defines the target matrix
     * @returns result input
     */
    static RotationXToRef<T extends Matrix>(angle: number, result: T): T;
    /**
     * Creates a new rotation matrix for "angle" radians around the Y axis
     * Example Playground - https://playground.babylonjs.com/#AV9X17#99
     * @param angle defines the angle (in radians) to use
     * @returns the new matrix
     */
    static RotationY(angle: number): Matrix;
    /**
     * Creates a new rotation matrix for "angle" radians around the Y axis and stores it in a given matrix
     * Example Playground - https://playground.babylonjs.com/#AV9X17#100
     * @param angle defines the angle (in radians) to use
     * @param result defines the target matrix
     * @returns result input
     */
    static RotationYToRef<T extends Matrix>(angle: number, result: T): T;
    /**
     * Creates a new rotation matrix for "angle" radians around the Z axis
     * Example Playground - https://playground.babylonjs.com/#AV9X17#101
     * @param angle defines the angle (in radians) to use
     * @returns the new matrix
     */
    static RotationZ(angle: number): Matrix;
    /**
     * Creates a new rotation matrix for "angle" radians around the Z axis and stores it in a given matrix
     * Example Playground - https://playground.babylonjs.com/#AV9X17#102
     * @param angle defines the angle (in radians) to use
     * @param result defines the target matrix
     * @returns result input
     */
    static RotationZToRef<T extends Matrix>(angle: number, result: T): T;
    /**
     * Creates a new rotation matrix for "angle" radians around the given axis
     * Example Playground - https://playground.babylonjs.com/#AV9X17#96
     * @param axis defines the axis to use
     * @param angle defines the angle (in radians) to use
     * @returns the new matrix
     */
    static RotationAxis(axis: DeepImmutable<Vector3>, angle: number): Matrix;
    /**
     * Creates a new rotation matrix for "angle" radians around the given axis and stores it in a given matrix
     * Example Playground - https://playground.babylonjs.com/#AV9X17#94
     * @param axis defines the axis to use
     * @param angle defines the angle (in radians) to use
     * @param result defines the target matrix
     * @returns result input
     */
    static RotationAxisToRef<T extends Matrix>(axis: DeepImmutable<Vector3>, angle: number, result: T): T;
    /**
     * Takes normalised vectors and returns a rotation matrix to align "from" with "to".
     * Taken from http://www.iquilezles.org/www/articles/noacos/noacos.htm
     * Example Playground - https://playground.babylonjs.com/#AV9X17#93
     * @param from defines the vector to align
     * @param to defines the vector to align to
     * @param result defines the target matrix
     * @returns result input
     */
    static RotationAlignToRef<T extends Matrix>(from: DeepImmutable<Vector3>, to: DeepImmutable<Vector3>, result: T): T;
    /**
     * Creates a rotation matrix
     * Example Playground - https://playground.babylonjs.com/#AV9X17#103
     * Example Playground - https://playground.babylonjs.com/#AV9X17#105
     * @param yaw defines the yaw angle in radians (Y axis)
     * @param pitch defines the pitch angle in radians (X axis)
     * @param roll defines the roll angle in radians (Z axis)
     * @returns the new rotation matrix
     */
    static RotationYawPitchRoll(yaw: number, pitch: number, roll: number): Matrix;
    /**
     * Creates a rotation matrix and stores it in a given matrix
     * Example Playground - https://playground.babylonjs.com/#AV9X17#104
     * @param yaw defines the yaw angle in radians (Y axis)
     * @param pitch defines the pitch angle in radians (X axis)
     * @param roll defines the roll angle in radians (Z axis)
     * @param result defines the target matrix
     * @returns result input
     */
    static RotationYawPitchRollToRef<T extends Matrix>(yaw: number, pitch: number, roll: number, result: T): T;
    /**
     * Creates a scaling matrix
     * Example Playground - https://playground.babylonjs.com/#AV9X17#107
     * @param x defines the scale factor on X axis
     * @param y defines the scale factor on Y axis
     * @param z defines the scale factor on Z axis
     * @returns the new matrix
     */
    static Scaling(x: number, y: number, z: number): Matrix;
    /**
     * Creates a scaling matrix and stores it in a given matrix
     * Example Playground - https://playground.babylonjs.com/#AV9X17#108
     * @param x defines the scale factor on X axis
     * @param y defines the scale factor on Y axis
     * @param z defines the scale factor on Z axis
     * @param result defines the target matrix
     * @returns result input
     */
    static ScalingToRef<T extends Matrix>(x: number, y: number, z: number, result: T): T;
    /**
     * Creates a translation matrix
     * Example Playground - https://playground.babylonjs.com/#AV9X17#109
     * @param x defines the translation on X axis
     * @param y defines the translation on Y axis
     * @param z defines the translationon Z axis
     * @returns the new matrix
     */
    static Translation(x: number, y: number, z: number): Matrix;
    /**
     * Creates a translation matrix and stores it in a given matrix
     * Example Playground - https://playground.babylonjs.com/#AV9X17#110
     * @param x defines the translation on X axis
     * @param y defines the translation on Y axis
     * @param z defines the translationon Z axis
     * @param result defines the target matrix
     * @returns result input
     */
    static TranslationToRef<T extends Matrix>(x: number, y: number, z: number, result: T): T;
    /**
     * Returns a new Matrix whose values are the interpolated values for "gradient" (float) between the ones of the matrices "startValue" and "endValue".
     * Example Playground - https://playground.babylonjs.com/#AV9X17#55
     * @param startValue defines the start value
     * @param endValue defines the end value
     * @param gradient defines the gradient factor
     * @returns the new matrix
     */
    static Lerp<T extends Matrix>(startValue: DeepImmutable<T>, endValue: DeepImmutable<Matrix>, gradient: number): T;
    /**
     * Set the given matrix "result" as the interpolated values for "gradient" (float) between the ones of the matrices "startValue" and "endValue".
     * Example Playground - https://playground.babylonjs.com/#AV9X17#54
     * @param startValue defines the start value
     * @param endValue defines the end value
     * @param gradient defines the gradient factor
     * @param result defines the Matrix object where to store data
     * @returns result input
     */
    static LerpToRef<T extends Matrix>(startValue: DeepImmutable<Matrix>, endValue: DeepImmutable<Matrix>, gradient: number, result: T): T;
    /**
     * Builds a new matrix whose values are computed by:
     * * decomposing the the "startValue" and "endValue" matrices into their respective scale, rotation and translation matrices
     * * interpolating for "gradient" (float) the values between each of these decomposed matrices between the start and the end
     * * recomposing a new matrix from these 3 interpolated scale, rotation and translation matrices
     * Example Playground - https://playground.babylonjs.com/#AV9X17#22
     * Example Playground - https://playground.babylonjs.com/#AV9X17#51
     * @param startValue defines the first matrix
     * @param endValue defines the second matrix
     * @param gradient defines the gradient between the two matrices
     * @returns the new matrix
     */
    static DecomposeLerp<T extends Matrix>(startValue: DeepImmutable<T>, endValue: DeepImmutable<Matrix>, gradient: number): T;
    /**
     * Update a matrix to values which are computed by:
     * * decomposing the the "startValue" and "endValue" matrices into their respective scale, rotation and translation matrices
     * * interpolating for "gradient" (float) the values between each of these decomposed matrices between the start and the end
     * * recomposing a new matrix from these 3 interpolated scale, rotation and translation matrices
     * Example Playground - https://playground.babylonjs.com/#AV9X17#23
     * Example Playground - https://playground.babylonjs.com/#AV9X17#53
     * @param startValue defines the first matrix
     * @param endValue defines the second matrix
     * @param gradient defines the gradient between the two matrices
     * @param result defines the target matrix
     * @returns result input
     */
    static DecomposeLerpToRef<T extends Matrix>(startValue: DeepImmutable<Matrix>, endValue: DeepImmutable<Matrix>, gradient: number, result: T): T;
    /**
     * Creates a new matrix that transforms vertices from world space to camera space. It takes three vectors as arguments that together describe the position and orientation of the camera.
     * This function generates a matrix suitable for a left handed coordinate system
     * Example Playground - https://playground.babylonjs.com/#AV9X17#58
     * Example Playground - https://playground.babylonjs.com/#AV9X17#59
     * @param eye defines the final position of the entity
     * @param target defines where the entity should look at
     * @param up defines the up vector for the entity
     * @returns the new matrix
     */
    static LookAtLH(eye: DeepImmutable<Vector3>, target: DeepImmutable<Vector3>, up: DeepImmutable<Vector3>): Matrix;
    /**
     * Sets the given "result" Matrix to a matrix that transforms vertices from world space to camera space. It takes three vectors as arguments that together describe the position and orientation of the camera.
     * This function generates a matrix suitable for a left handed coordinate system
     * Example Playground - https://playground.babylonjs.com/#AV9X17#60
     * Example Playground - https://playground.babylonjs.com/#AV9X17#61
     * @param eye defines the final position of the entity
     * @param target defines where the entity should look at
     * @param up defines the up vector for the entity
     * @param result defines the target matrix
     * @returns result input
     */
    static LookAtLHToRef(eye: DeepImmutable<Vector3>, target: DeepImmutable<Vector3>, up: DeepImmutable<Vector3>, result: Matrix): void;
    /**
     * Creates a new matrix that transforms vertices from world space to camera space. It takes three vectors as arguments that together describe the position and orientation of the camera.
     * This function generates a matrix suitable for a right handed coordinate system
     * Example Playground - https://playground.babylonjs.com/#AV9X17#62
     * Example Playground - https://playground.babylonjs.com/#AV9X17#63
     * @param eye defines the final position of the entity
     * @param target defines where the entity should look at
     * @param up defines the up vector for the entity
     * @returns the new matrix
     */
    static LookAtRH(eye: DeepImmutable<Vector3>, target: DeepImmutable<Vector3>, up: DeepImmutable<Vector3>): Matrix;
    /**
     * Sets the given "result" Matrix to a matrix that transforms vertices from world space to camera space. It takes three vectors as arguments that together describe the position and orientation of the camera.
     * This function generates a matrix suitable for a right handed coordinate system
     * Example Playground - https://playground.babylonjs.com/#AV9X17#64
     * Example Playground - https://playground.babylonjs.com/#AV9X17#65
     * @param eye defines the final position of the entity
     * @param target defines where the entity should look at
     * @param up defines the up vector for the entity
     * @param result defines the target matrix
     * @returns result input
     */
    static LookAtRHToRef<T extends Matrix>(eye: DeepImmutable<Vector3>, target: DeepImmutable<Vector3>, up: DeepImmutable<Vector3>, result: T): T;
    /**
     * Creates a new matrix that transforms vertices from world space to camera space. It takes two vectors as arguments that together describe the orientation of the camera. The position is assumed to be at the origin (0,0,0)
     * This function generates a matrix suitable for a left handed coordinate system
     * Example Playground - https://playground.babylonjs.com/#AV9X17#66
     * @param forward defines the forward direction - Must be normalized and orthogonal to up.
     * @param up defines the up vector for the entity - Must be normalized and orthogonal to forward.
     * @returns the new matrix
     */
    static LookDirectionLH(forward: DeepImmutable<Vector3>, up: DeepImmutable<Vector3>): Matrix;
    /**
     * Sets the given "result" Matrix to a matrix that transforms vertices from world space to camera space. It takes two vectors as arguments that together describe the orientation of the camera. The position is assumed to be at the origin (0,0,0)
     * This function generates a matrix suitable for a left handed coordinate system
     * Example Playground - https://playground.babylonjs.com/#AV9X17#67
     * @param forward defines the forward direction - Must be normalized and orthogonal to up.
     * @param up defines the up vector for the entity - Must be normalized and orthogonal to forward.
     * @param result defines the target matrix
     * @returns result input
     */
    static LookDirectionLHToRef<T extends Matrix>(forward: DeepImmutable<Vector3>, up: DeepImmutable<Vector3>, result: T): T;
    /**
     * Creates a new matrix that transforms vertices from world space to camera space. It takes two vectors as arguments that together describe the orientation of the camera. The position is assumed to be at the origin (0,0,0)
     * This function generates a matrix suitable for a right handed coordinate system
     * Example Playground - https://playground.babylonjs.com/#AV9X17#68
     * @param forward defines the forward direction - Must be normalized and orthogonal to up.
     * @param up defines the up vector for the entity - Must be normalized and orthogonal to forward.
     * @returns the new matrix
     */
    static LookDirectionRH(forward: DeepImmutable<Vector3>, up: DeepImmutable<Vector3>): Matrix;
    /**
     * Sets the given "result" Matrix to a matrix that transforms vertices from world space to camera space. It takes two vectors as arguments that together describe the orientation of the camera. The position is assumed to be at the origin (0,0,0)
     * This function generates a matrix suitable for a right handed coordinate system
     * Example Playground - https://playground.babylonjs.com/#AV9X17#69
     * @param forward defines the forward direction - Must be normalized and orthogonal to up.
     * @param up defines the up vector for the entity - Must be normalized and orthogonal to forward.
     * @param result defines the target matrix
     * @returns result input
     */
    static LookDirectionRHToRef<T extends Matrix>(forward: DeepImmutable<Vector3>, up: DeepImmutable<Vector3>, result: T): T;
    /**
     * Create a left-handed orthographic projection matrix
     * Example Playground - https://playground.babylonjs.com/#AV9X17#70
     * @param width defines the viewport width
     * @param height defines the viewport height
     * @param znear defines the near clip plane
     * @param zfar defines the far clip plane
     * @param halfZRange true to generate NDC coordinates between 0 and 1 instead of -1 and 1 (default: false)
     * @returns a new matrix as a left-handed orthographic projection matrix
     */
    static OrthoLH(width: number, height: number, znear: number, zfar: number, halfZRange?: boolean): Matrix;
    /**
     * Store a left-handed orthographic projection to a given matrix
     * Example Playground - https://playground.babylonjs.com/#AV9X17#71
     * @param width defines the viewport width
     * @param height defines the viewport height
     * @param znear defines the near clip plane
     * @param zfar defines the far clip plane
     * @param result defines the target matrix
     * @param halfZRange true to generate NDC coordinates between 0 and 1 instead of -1 and 1 (default: false)
     * @returns result input
     */
    static OrthoLHToRef<T extends Matrix>(width: number, height: number, znear: number, zfar: number, result: T, halfZRange?: boolean): T;
    /**
     * Create a left-handed orthographic projection matrix
     * Example Playground - https://playground.babylonjs.com/#AV9X17#72
     * @param left defines the viewport left coordinate
     * @param right defines the viewport right coordinate
     * @param bottom defines the viewport bottom coordinate
     * @param top defines the viewport top coordinate
     * @param znear defines the near clip plane
     * @param zfar defines the far clip plane
     * @param halfZRange true to generate NDC coordinates between 0 and 1 instead of -1 and 1 (default: false)
     * @returns a new matrix as a left-handed orthographic projection matrix
     */
    static OrthoOffCenterLH(left: number, right: number, bottom: number, top: number, znear: number, zfar: number, halfZRange?: boolean): Matrix;
    /**
     * Stores a left-handed orthographic projection into a given matrix
     * Example Playground - https://playground.babylonjs.com/#AV9X17#73
     * @param left defines the viewport left coordinate
     * @param right defines the viewport right coordinate
     * @param bottom defines the viewport bottom coordinate
     * @param top defines the viewport top coordinate
     * @param znear defines the near clip plane
     * @param zfar defines the far clip plane
     * @param result defines the target matrix
     * @param halfZRange true to generate NDC coordinates between 0 and 1 instead of -1 and 1 (default: false)
     * @returns result input
     */
    static OrthoOffCenterLHToRef<T extends Matrix>(left: number, right: number, bottom: number, top: number, znear: number, zfar: number, result: T, halfZRange?: boolean): T;
    /**
     * Creates a right-handed orthographic projection matrix
     * Example Playground - https://playground.babylonjs.com/#AV9X17#76
     * @param left defines the viewport left coordinate
     * @param right defines the viewport right coordinate
     * @param bottom defines the viewport bottom coordinate
     * @param top defines the viewport top coordinate
     * @param znear defines the near clip plane
     * @param zfar defines the far clip plane
     * @param halfZRange true to generate NDC coordinates between 0 and 1 instead of -1 and 1 (default: false)
     * @returns a new matrix as a right-handed orthographic projection matrix
     */
    static OrthoOffCenterRH(left: number, right: number, bottom: number, top: number, znear: number, zfar: number, halfZRange?: boolean): Matrix;
    /**
     * Stores a right-handed orthographic projection into a given matrix
     * Example Playground - https://playground.babylonjs.com/#AV9X17#77
     * @param left defines the viewport left coordinate
     * @param right defines the viewport right coordinate
     * @param bottom defines the viewport bottom coordinate
     * @param top defines the viewport top coordinate
     * @param znear defines the near clip plane
     * @param zfar defines the far clip plane
     * @param result defines the target matrix
     * @param halfZRange true to generate NDC coordinates between 0 and 1 instead of -1 and 1 (default: false)
     * @returns result input
     */
    static OrthoOffCenterRHToRef<T extends Matrix>(left: number, right: number, bottom: number, top: number, znear: number, zfar: number, result: T, halfZRange?: boolean): T;
    /**
     * Creates a left-handed perspective projection matrix
     * Example Playground - https://playground.babylonjs.com/#AV9X17#85
     * @param width defines the viewport width
     * @param height defines the viewport height
     * @param znear defines the near clip plane
     * @param zfar defines the far clip plane
     * @param halfZRange true to generate NDC coordinates between 0 and 1 instead of -1 and 1 (default: false)
     * @param projectionPlaneTilt optional tilt angle of the projection plane around the X axis (horizontal)
     * @returns a new matrix as a left-handed perspective projection matrix
     */
    static PerspectiveLH(width: number, height: number, znear: number, zfar: number, halfZRange?: boolean, projectionPlaneTilt?: number): Matrix;
    /**
     * Creates a left-handed perspective projection matrix
     * Example Playground - https://playground.babylonjs.com/#AV9X17#78
     * @param fov defines the horizontal field of view
     * @param aspect defines the aspect ratio
     * @param znear defines the near clip plane
     * @param zfar defines the far clip plane. If 0, assume we are in "infinite zfar" mode
     * @param halfZRange true to generate NDC coordinates between 0 and 1 instead of -1 and 1 (default: false)
     * @param projectionPlaneTilt optional tilt angle of the projection plane around the X axis (horizontal)
     * @param reverseDepthBufferMode true to indicate that we are in a reverse depth buffer mode (meaning znear and zfar have been inverted when calling the function)
     * @returns a new matrix as a left-handed perspective projection matrix
     */
    static PerspectiveFovLH(fov: number, aspect: number, znear: number, zfar: number, halfZRange?: boolean, projectionPlaneTilt?: number, reverseDepthBufferMode?: boolean): Matrix;
    /**
     * Stores a left-handed perspective projection into a given matrix
     * Example Playground - https://playground.babylonjs.com/#AV9X17#81
     * @param fov defines the horizontal field of view
     * @param aspect defines the aspect ratio
     * @param znear defines the near clip plane
     * @param zfar defines the far clip plane. If 0, assume we are in "infinite zfar" mode
     * @param result defines the target matrix
     * @param isVerticalFovFixed defines it the fov is vertically fixed (default) or horizontally
     * @param halfZRange true to generate NDC coordinates between 0 and 1 instead of -1 and 1 (default: false)
     * @param projectionPlaneTilt optional tilt angle of the projection plane around the X axis (horizontal)
     * @param reverseDepthBufferMode true to indicate that we are in a reverse depth buffer mode (meaning znear and zfar have been inverted when calling the function)
     * @returns result input
     */
    static PerspectiveFovLHToRef<T extends Matrix>(fov: number, aspect: number, znear: number, zfar: number, result: T, isVerticalFovFixed?: boolean, halfZRange?: boolean, projectionPlaneTilt?: number, reverseDepthBufferMode?: boolean): T;
    /**
     * Stores a left-handed perspective projection into a given matrix with depth reversed
     * Example Playground - https://playground.babylonjs.com/#AV9X17#89
     * @param fov defines the horizontal field of view
     * @param aspect defines the aspect ratio
     * @param znear defines the near clip plane
     * @param zfar not used as infinity is used as far clip
     * @param result defines the target matrix
     * @param isVerticalFovFixed defines it the fov is vertically fixed (default) or horizontally
     * @param halfZRange true to generate NDC coordinates between 0 and 1 instead of -1 and 1 (default: false)
     * @param projectionPlaneTilt optional tilt angle of the projection plane around the X axis (horizontal)
     * @returns result input
     */
    static PerspectiveFovReverseLHToRef<T extends Matrix>(fov: number, aspect: number, znear: number, zfar: number, result: T, isVerticalFovFixed?: boolean, halfZRange?: boolean, projectionPlaneTilt?: number): T;
    /**
     * Creates a right-handed perspective projection matrix
     * Example Playground - https://playground.babylonjs.com/#AV9X17#83
     * @param fov defines the horizontal field of view
     * @param aspect defines the aspect ratio
     * @param znear defines the near clip plane
     * @param zfar defines the far clip plane. If 0, assume we are in "infinite zfar" mode
     * @param halfZRange true to generate NDC coordinates between 0 and 1 instead of -1 and 1 (default: false)
     * @param projectionPlaneTilt optional tilt angle of the projection plane around the X axis (horizontal)
     * @param reverseDepthBufferMode true to indicate that we are in a reverse depth buffer mode (meaning znear and zfar have been inverted when calling the function)
     * @returns a new matrix as a right-handed perspective projection matrix
     */
    static PerspectiveFovRH(fov: number, aspect: number, znear: number, zfar: number, halfZRange?: boolean, projectionPlaneTilt?: number, reverseDepthBufferMode?: boolean): Matrix;
    /**
     * Stores a right-handed perspective projection into a given matrix
     * Example Playground - https://playground.babylonjs.com/#AV9X17#84
     * @param fov defines the horizontal field of view
     * @param aspect defines the aspect ratio
     * @param znear defines the near clip plane
     * @param zfar defines the far clip plane. If 0, assume we are in "infinite zfar" mode
     * @param result defines the target matrix
     * @param isVerticalFovFixed defines it the fov is vertically fixed (default) or horizontally
     * @param halfZRange true to generate NDC coordinates between 0 and 1 instead of -1 and 1 (default: false)
     * @param projectionPlaneTilt optional tilt angle of the projection plane around the X axis (horizontal)
     * @param reverseDepthBufferMode true to indicate that we are in a reverse depth buffer mode (meaning znear and zfar have been inverted when calling the function)
     * @returns result input
     */
    static PerspectiveFovRHToRef<T extends Matrix>(fov: number, aspect: number, znear: number, zfar: number, result: T, isVerticalFovFixed?: boolean, halfZRange?: boolean, projectionPlaneTilt?: number, reverseDepthBufferMode?: boolean): T;
    /**
     * Stores a right-handed perspective projection into a given matrix
     * Example Playground - https://playground.babylonjs.com/#AV9X17#90
     * @param fov defines the horizontal field of view
     * @param aspect defines the aspect ratio
     * @param znear defines the near clip plane
     * @param zfar not used as infinity is used as far clip
     * @param result defines the target matrix
     * @param isVerticalFovFixed defines it the fov is vertically fixed (default) or horizontally
     * @param halfZRange true to generate NDC coordinates between 0 and 1 instead of -1 and 1 (default: false)
     * @param projectionPlaneTilt optional tilt angle of the projection plane around the X axis (horizontal)
     * @returns result input
     */
    static PerspectiveFovReverseRHToRef<T extends Matrix>(fov: number, aspect: number, znear: number, zfar: number, result: T, isVerticalFovFixed?: boolean, halfZRange?: boolean, projectionPlaneTilt?: number): T;
    /**
     * Stores a perspective projection for WebVR info a given matrix
     * Example Playground - https://playground.babylonjs.com/#AV9X17#92
     * @param fov defines the field of view
     * @param fov.upDegrees
     * @param fov.downDegrees
     * @param fov.leftDegrees
     * @param fov.rightDegrees
     * @param znear defines the near clip plane
     * @param zfar defines the far clip plane
     * @param result defines the target matrix
     * @param rightHanded defines if the matrix must be in right-handed mode (false by default)
     * @param halfZRange true to generate NDC coordinates between 0 and 1 instead of -1 and 1 (default: false)
     * @param projectionPlaneTilt optional tilt angle of the projection plane around the X axis (horizontal)
     * @returns result input
     */
    static PerspectiveFovWebVRToRef<T extends Matrix>(fov: {
        upDegrees: number;
        downDegrees: number;
        leftDegrees: number;
        rightDegrees: number;
    }, znear: number, zfar: number, result: T, rightHanded?: boolean, halfZRange?: boolean, projectionPlaneTilt?: number): T;
    /**
     * Computes a complete transformation matrix
     * Example Playground - https://playground.babylonjs.com/#AV9X17#113
     * @param viewport defines the viewport to use
     * @param world defines the world matrix
     * @param view defines the view matrix
     * @param projection defines the projection matrix
     * @param zmin defines the near clip plane
     * @param zmax defines the far clip plane
     * @returns the transformation matrix
     */
    static GetFinalMatrix<T extends Matrix>(viewport: DeepImmutable<Viewport>, world: DeepImmutable<T>, view: DeepImmutable<Matrix>, projection: DeepImmutable<Matrix>, zmin: number, zmax: number): T;
    /**
     * Extracts a 2x2 matrix from a given matrix and store the result in a Float32Array
     * @param matrix defines the matrix to use
     * @returns a new Float32Array array with 4 elements : the 2x2 matrix extracted from the given matrix
     */
    static GetAsMatrix2x2(matrix: DeepImmutable<Matrix>): Float32Array | Array<number>;
    /**
     * Extracts a 3x3 matrix from a given matrix and store the result in a Float32Array
     * @param matrix defines the matrix to use
     * @returns a new Float32Array array with 9 elements : the 3x3 matrix extracted from the given matrix
     */
    static GetAsMatrix3x3(matrix: DeepImmutable<Matrix>): Float32Array | Array<number>;
    /**
     * Compute the transpose of a given matrix
     * Example Playground - https://playground.babylonjs.com/#AV9X17#111
     * @param matrix defines the matrix to transpose
     * @returns the new matrix
     */
    static Transpose<T extends Matrix>(matrix: DeepImmutable<T>): T;
    /**
     * Compute the transpose of a matrix and store it in a target matrix
     * Example Playground - https://playground.babylonjs.com/#AV9X17#112
     * @param matrix defines the matrix to transpose
     * @param result defines the target matrix
     * @returns result input
     */
    static TransposeToRef<T extends Matrix>(matrix: DeepImmutable<Matrix>, result: T): T;
    /**
     * Computes a reflection matrix from a plane
     * Example Playground - https://playground.babylonjs.com/#AV9X17#87
     * @param plane defines the reflection plane
     * @returns a new matrix
     */
    static Reflection(plane: DeepImmutable<IPlaneLike>): Matrix;
    /**
     * Computes a reflection matrix from a plane
     * Example Playground - https://playground.babylonjs.com/#AV9X17#88
     * @param plane defines the reflection plane
     * @param result defines the target matrix
     * @returns result input
     */
    static ReflectionToRef<T extends Matrix>(plane: DeepImmutable<IPlaneLike>, result: T): T;
    /**
     * Sets the given matrix as a rotation matrix composed from the 3 left handed axes
     * @param xaxis defines the value of the 1st axis
     * @param yaxis defines the value of the 2nd axis
     * @param zaxis defines the value of the 3rd axis
     * @param result defines the target matrix
     * @returns result input
     */
    static FromXYZAxesToRef<T extends Matrix>(xaxis: DeepImmutable<Vector3>, yaxis: DeepImmutable<Vector3>, zaxis: DeepImmutable<Vector3>, result: T): T;
    /**
     * Creates a rotation matrix from a quaternion and stores it in a target matrix
     * @param quat defines the quaternion to use
     * @param result defines the target matrix
     * @returns result input
     */
    static FromQuaternionToRef<T extends Matrix>(quat: DeepImmutable<Quaternion>, result: T): T;
}
/**
 * @internal
 */
export class TmpVectors {
    static Vector2: [Vector2, Vector2, Vector2];
    static Vector3: [Vector3, Vector3, Vector3, Vector3, Vector3, Vector3, Vector3, Vector3, Vector3, Vector3, Vector3, Vector3, Vector3];
    static Vector4: [Vector4, Vector4, Vector4];
    static Quaternion: [Quaternion, Quaternion];
    static Matrix: [Matrix, Matrix, Matrix, Matrix, Matrix, Matrix, Matrix, Matrix];
}

}
declare module "babylonjs/Maths/math.vertexFormat" {
import { Vector3, Vector2 } from "babylonjs/Maths/math.vector";
/**
 * Contains position and normal vectors for a vertex
 */
export class PositionNormalVertex {
    /** the position of the vertex (defaut: 0,0,0) */
    position: Vector3;
    /** the normal of the vertex (defaut: 0,1,0) */
    normal: Vector3;
    /**
     * Creates a PositionNormalVertex
     * @param position the position of the vertex (defaut: 0,0,0)
     * @param normal the normal of the vertex (defaut: 0,1,0)
     */
    constructor(
    /** the position of the vertex (defaut: 0,0,0) */
    position?: Vector3, 
    /** the normal of the vertex (defaut: 0,1,0) */
    normal?: Vector3);
    /**
     * Clones the PositionNormalVertex
     * @returns the cloned PositionNormalVertex
     */
    clone(): PositionNormalVertex;
}
/**
 * Contains position, normal and uv vectors for a vertex
 */
export class PositionNormalTextureVertex {
    /** the position of the vertex (defaut: 0,0,0) */
    position: Vector3;
    /** the normal of the vertex (defaut: 0,1,0) */
    normal: Vector3;
    /** the uv of the vertex (default: 0,0) */
    uv: Vector2;
    /**
     * Creates a PositionNormalTextureVertex
     * @param position the position of the vertex (defaut: 0,0,0)
     * @param normal the normal of the vertex (defaut: 0,1,0)
     * @param uv the uv of the vertex (default: 0,0)
     */
    constructor(
    /** the position of the vertex (defaut: 0,0,0) */
    position?: Vector3, 
    /** the normal of the vertex (defaut: 0,1,0) */
    normal?: Vector3, 
    /** the uv of the vertex (default: 0,0) */
    uv?: Vector2);
    /**
     * Clones the PositionNormalTextureVertex
     * @returns the cloned PositionNormalTextureVertex
     */
    clone(): PositionNormalTextureVertex;
}

}
declare module "babylonjs/Maths/math.viewport" {
/**
 * Class used to represent a viewport on screen
 */
export class Viewport {
    /** viewport left coordinate */
    x: number;
    /** viewport top coordinate */
    y: number;
    /**viewport width */
    width: number;
    /** viewport height */
    height: number;
    /**
     * Creates a Viewport object located at (x, y) and sized (width, height)
     * @param x defines viewport left coordinate
     * @param y defines viewport top coordinate
     * @param width defines the viewport width
     * @param height defines the viewport height
     */
    constructor(
    /** viewport left coordinate */
    x: number, 
    /** viewport top coordinate */
    y: number, 
    /**viewport width */
    width: number, 
    /** viewport height */
    height: number);
    /**
     * Creates a new viewport using absolute sizing (from 0-> width, 0-> height instead of 0->1)
     * @param renderWidth defines the rendering width
     * @param renderHeight defines the rendering height
     * @returns a new Viewport
     */
    toGlobal(renderWidth: number, renderHeight: number): Viewport;
    /**
     * Stores absolute viewport value into a target viewport (from 0-> width, 0-> height instead of 0->1)
     * @param renderWidth defines the rendering width
     * @param renderHeight defines the rendering height
     * @param ref defines the target viewport
     * @returns the current viewport
     */
    toGlobalToRef(renderWidth: number, renderHeight: number, ref: Viewport): Viewport;
    /**
     * Returns a new Viewport copied from the current one
     * @returns a new Viewport
     */
    clone(): Viewport;
}

}
declare module "babylonjs/Maths/sphericalPolynomial" {
import { Vector3 } from "babylonjs/Maths/math.vector";
import { Color3 } from "babylonjs/Maths/math.color";
/**
 * Class representing spherical harmonics coefficients to the 3rd degree
 */
export class SphericalHarmonics {
    /**
     * Defines whether or not the harmonics have been prescaled for rendering.
     */
    preScaled: boolean;
    /**
     * The l0,0 coefficients of the spherical harmonics
     */
    l00: Vector3;
    /**
     * The l1,-1 coefficients of the spherical harmonics
     */
    l1_1: Vector3;
    /**
     * The l1,0 coefficients of the spherical harmonics
     */
    l10: Vector3;
    /**
     * The l1,1 coefficients of the spherical harmonics
     */
    l11: Vector3;
    /**
     * The l2,-2 coefficients of the spherical harmonics
     */
    l2_2: Vector3;
    /**
     * The l2,-1 coefficients of the spherical harmonics
     */
    l2_1: Vector3;
    /**
     * The l2,0 coefficients of the spherical harmonics
     */
    l20: Vector3;
    /**
     * The l2,1 coefficients of the spherical harmonics
     */
    l21: Vector3;
    /**
     * The l2,2 coefficients of the spherical harmonics
     */
    l22: Vector3;
    /**
     * Adds a light to the spherical harmonics
     * @param direction the direction of the light
     * @param color the color of the light
     * @param deltaSolidAngle the delta solid angle of the light
     */
    addLight(direction: Vector3, color: Color3, deltaSolidAngle: number): void;
    /**
     * Scales the spherical harmonics by the given amount
     * @param scale the amount to scale
     */
    scaleInPlace(scale: number): void;
    /**
     * Convert from incident radiance (Li) to irradiance (E) by applying convolution with the cosine-weighted hemisphere.
     *
     * ```
     * E_lm = A_l * L_lm
     * ```
     *
     * In spherical harmonics this convolution amounts to scaling factors for each frequency band.
     * This corresponds to equation 5 in "An Efficient Representation for Irradiance Environment Maps", where
     * the scaling factors are given in equation 9.
     */
    convertIncidentRadianceToIrradiance(): void;
    /**
     * Convert from irradiance to outgoing radiance for Lambertian BDRF, suitable for efficient shader evaluation.
     *
     * ```
     * L = (1/pi) * E * rho
     * ```
     *
     * This is done by an additional scale by 1/pi, so is a fairly trivial operation but important conceptually.
     */
    convertIrradianceToLambertianRadiance(): void;
    /**
     * Integrates the reconstruction coefficients directly in to the SH preventing further
     * required operations at run time.
     *
     * This is simply done by scaling back the SH with Ylm constants parameter.
     * The trigonometric part being applied by the shader at run time.
     */
    preScaleForRendering(): void;
    /**
     * update the spherical harmonics coefficients from the given array
     * @param data defines the 9x3 coefficients (l00, l1-1, l10, l11, l2-2, l2-1, l20, l21, l22)
     * @returns the spherical harmonics (this)
     */
    updateFromArray(data: ArrayLike<ArrayLike<number>>): SphericalHarmonics;
    /**
     * update the spherical harmonics coefficients from the given floats array
     * @param data defines the 9x3 coefficients (l00, l1-1, l10, l11, l2-2, l2-1, l20, l21, l22)
     * @returns the spherical harmonics (this)
     */
    updateFromFloatsArray(data: ArrayLike<number>): SphericalHarmonics;
    /**
     * Constructs a spherical harmonics from an array.
     * @param data defines the 9x3 coefficients (l00, l1-1, l10, l11, l2-2, l2-1, l20, l21, l22)
     * @returns the spherical harmonics
     */
    static FromArray(data: ArrayLike<ArrayLike<number>>): SphericalHarmonics;
    /**
     * Gets the spherical harmonics from polynomial
     * @param polynomial the spherical polynomial
     * @returns the spherical harmonics
     */
    static FromPolynomial(polynomial: SphericalPolynomial): SphericalHarmonics;
}
/**
 * Class representing spherical polynomial coefficients to the 3rd degree
 */
export class SphericalPolynomial {
    private _harmonics;
    /**
     * The spherical harmonics used to create the polynomials.
     */
    get preScaledHarmonics(): SphericalHarmonics;
    /**
     * The x coefficients of the spherical polynomial
     */
    x: Vector3;
    /**
     * The y coefficients of the spherical polynomial
     */
    y: Vector3;
    /**
     * The z coefficients of the spherical polynomial
     */
    z: Vector3;
    /**
     * The xx coefficients of the spherical polynomial
     */
    xx: Vector3;
    /**
     * The yy coefficients of the spherical polynomial
     */
    yy: Vector3;
    /**
     * The zz coefficients of the spherical polynomial
     */
    zz: Vector3;
    /**
     * The xy coefficients of the spherical polynomial
     */
    xy: Vector3;
    /**
     * The yz coefficients of the spherical polynomial
     */
    yz: Vector3;
    /**
     * The zx coefficients of the spherical polynomial
     */
    zx: Vector3;
    /**
     * Adds an ambient color to the spherical polynomial
     * @param color the color to add
     */
    addAmbient(color: Color3): void;
    /**
     * Scales the spherical polynomial by the given amount
     * @param scale the amount to scale
     */
    scaleInPlace(scale: number): void;
    /**
     * Updates the spherical polynomial from harmonics
     * @param harmonics the spherical harmonics
     * @returns the spherical polynomial
     */
    updateFromHarmonics(harmonics: SphericalHarmonics): SphericalPolynomial;
    /**
     * Gets the spherical polynomial from harmonics
     * @param harmonics the spherical harmonics
     * @returns the spherical polynomial
     */
    static FromHarmonics(harmonics: SphericalHarmonics): SphericalPolynomial;
    /**
     * Constructs a spherical polynomial from an array.
     * @param data defines the 9x3 coefficients (x, y, z, xx, yy, zz, yz, zx, xy)
     * @returns the spherical polynomial
     */
    static FromArray(data: ArrayLike<ArrayLike<number>>): SphericalPolynomial;
}

}
declare module "babylonjs/Meshes/abstractMesh" {
import { Observable } from "babylonjs/Misc/observable";
import { Nullable, FloatArray, IndicesArray, DeepImmutable } from "babylonjs/types";
import { Camera } from "babylonjs/Cameras/camera";
import { Scene, IDisposable } from "babylonjs/scene";
import { Vector2 } from "babylonjs/Maths/math.vector";
import { Matrix, Vector3 } from "babylonjs/Maths/math.vector";
import { Node } from "babylonjs/node";
import { IGetSetVerticesData } from "babylonjs/Meshes/mesh.vertexData";
import { TransformNode } from "babylonjs/Meshes/transformNode";
import { SubMesh } from "babylonjs/Meshes/subMesh";
import { PickingInfo } from "babylonjs/Collisions/pickingInfo";
import { ICullable } from "babylonjs/Culling/boundingInfo";
import { BoundingInfo } from "babylonjs/Culling/boundingInfo";
import { Material } from "babylonjs/Materials/material";
import { Light } from "babylonjs/Lights/light";
import { Skeleton } from "babylonjs/Bones/skeleton";
import { MorphTargetManager } from "babylonjs/Morph/morphTargetManager";
import { IBakedVertexAnimationManager } from "babylonjs/BakedVertexAnimation/bakedVertexAnimationManager";
import { IEdgesRenderer } from "babylonjs/Rendering/edgesRenderer";
import { SolidParticle } from "babylonjs/Particles/solidParticle";
import { AbstractActionManager } from "babylonjs/Actions/abstractActionManager";
import { UniformBuffer } from "babylonjs/Materials/uniformBuffer";
import { _MeshCollisionData } from "babylonjs/Collisions/meshCollisionData";
import { RawTexture } from "babylonjs/Materials/Textures/rawTexture";
import { Color3, Color4 } from "babylonjs/Maths/math.color";
import { Plane } from "babylonjs/Maths/math.plane";
import { IParticleSystem } from "babylonjs/Particles/IParticleSystem";
import { Ray } from "babylonjs/Culling/ray";
import { Collider } from "babylonjs/Collisions/collider";
import { TrianglePickingPredicate } from "babylonjs/Culling/ray";
import { RenderingGroup } from "babylonjs/Rendering/renderingGroup";
import { IEdgesRendererOptions } from "babylonjs/Rendering/edgesRenderer";
/** @internal */
class _FacetDataStorage {
    facetPositions: Vector3[];
    facetNormals: Vector3[];
    facetPartitioning: number[][];
    facetNb: number;
    partitioningSubdivisions: number;
    partitioningBBoxRatio: number;
    facetDataEnabled: boolean;
    facetParameters: any;
    bbSize: Vector3;
    subDiv: {
        max: number;
        X: number;
        Y: number;
        Z: number;
    };
    facetDepthSort: boolean;
    facetDepthSortEnabled: boolean;
    depthSortedIndices: IndicesArray;
    depthSortedFacets: {
        ind: number;
        sqDistance: number;
    }[];
    facetDepthSortFunction: (f1: {
        ind: number;
        sqDistance: number;
    }, f2: {
        ind: number;
        sqDistance: number;
    }) => number;
    facetDepthSortFrom: Vector3;
    facetDepthSortOrigin: Vector3;
    invertedMatrix: Matrix;
}
/**
 * @internal
 **/
class _InternalAbstractMeshDataInfo {
    _hasVertexAlpha: boolean;
    _useVertexColors: boolean;
    _numBoneInfluencers: number;
    _applyFog: boolean;
    _receiveShadows: boolean;
    _facetData: _FacetDataStorage;
    _visibility: number;
    _skeleton: Nullable<Skeleton>;
    _layerMask: number;
    _computeBonesUsingShaders: boolean;
    _isActive: boolean;
    _onlyForInstances: boolean;
    _isActiveIntermediate: boolean;
    _onlyForInstancesIntermediate: boolean;
    _actAsRegularMesh: boolean;
    _currentLOD: Nullable<AbstractMesh>;
    _currentLODIsUpToDate: boolean;
    _collisionRetryCount: number;
    _morphTargetManager: Nullable<MorphTargetManager>;
    _renderingGroupId: number;
    _bakedVertexAnimationManager: Nullable<IBakedVertexAnimationManager>;
    _material: Nullable<Material>;
    _materialForRenderPass: Array<Material | undefined>;
    _positions: Nullable<Vector3[]>;
    _pointerOverDisableMeshTesting: boolean;
    _meshCollisionData: _MeshCollisionData;
    _enableDistantPicking: boolean;
}
/**
 * Class used to store all common mesh properties
 */
export class AbstractMesh extends TransformNode implements IDisposable, ICullable, IGetSetVerticesData {
    /** No occlusion */
    static OCCLUSION_TYPE_NONE: number;
    /** Occlusion set to optimistic */
    static OCCLUSION_TYPE_OPTIMISTIC: number;
    /** Occlusion set to strict */
    static OCCLUSION_TYPE_STRICT: number;
    /** Use an accurate occlusion algorithm */
    static OCCLUSION_ALGORITHM_TYPE_ACCURATE: number;
    /** Use a conservative occlusion algorithm */
    static OCCLUSION_ALGORITHM_TYPE_CONSERVATIVE: number;
    /** Default culling strategy : this is an exclusion test and it's the more accurate.
     *  Test order :
     *  Is the bounding sphere outside the frustum ?
     *  If not, are the bounding box vertices outside the frustum ?
     *  It not, then the cullable object is in the frustum.
     */
    static readonly CULLINGSTRATEGY_STANDARD: number;
    /** Culling strategy : Bounding Sphere Only.
     *  This is an exclusion test. It's faster than the standard strategy because the bounding box is not tested.
     *  It's also less accurate than the standard because some not visible objects can still be selected.
     *  Test : is the bounding sphere outside the frustum ?
     *  If not, then the cullable object is in the frustum.
     */
    static readonly CULLINGSTRATEGY_BOUNDINGSPHERE_ONLY: number;
    /** Culling strategy : Optimistic Inclusion.
     *  This in an inclusion test first, then the standard exclusion test.
     *  This can be faster when a cullable object is expected to be almost always in the camera frustum.
     *  This could also be a little slower than the standard test when the tested object center is not the frustum but one of its bounding box vertex is still inside.
     *  Anyway, it's as accurate as the standard strategy.
     *  Test :
     *  Is the cullable object bounding sphere center in the frustum ?
     *  If not, apply the default culling strategy.
     */
    static readonly CULLINGSTRATEGY_OPTIMISTIC_INCLUSION: number;
    /** Culling strategy : Optimistic Inclusion then Bounding Sphere Only.
     *  This in an inclusion test first, then the bounding sphere only exclusion test.
     *  This can be the fastest test when a cullable object is expected to be almost always in the camera frustum.
     *  This could also be a little slower than the BoundingSphereOnly strategy when the tested object center is not in the frustum but its bounding sphere still intersects it.
     *  It's less accurate than the standard strategy and as accurate as the BoundingSphereOnly strategy.
     *  Test :
     *  Is the cullable object bounding sphere center in the frustum ?
     *  If not, apply the Bounding Sphere Only strategy. No Bounding Box is tested here.
     */
    static readonly CULLINGSTRATEGY_OPTIMISTIC_INCLUSION_THEN_BSPHERE_ONLY: number;
    /**
     * No billboard
     */
    static get BILLBOARDMODE_NONE(): number;
    /** Billboard on X axis */
    static get BILLBOARDMODE_X(): number;
    /** Billboard on Y axis */
    static get BILLBOARDMODE_Y(): number;
    /** Billboard on Z axis */
    static get BILLBOARDMODE_Z(): number;
    /** Billboard on all axes */
    static get BILLBOARDMODE_ALL(): number;
    /** Billboard on using position instead of orientation */
    static get BILLBOARDMODE_USE_POSITION(): number;
    /** @internal */
    _internalAbstractMeshDataInfo: _InternalAbstractMeshDataInfo;
    /** @internal */
    _waitingMaterialId: Nullable<string>;
    /**
     * The culling strategy to use to check whether the mesh must be rendered or not.
     * This value can be changed at any time and will be used on the next render mesh selection.
     * The possible values are :
     * - AbstractMesh.CULLINGSTRATEGY_STANDARD
     * - AbstractMesh.CULLINGSTRATEGY_BOUNDINGSPHERE_ONLY
     * - AbstractMesh.CULLINGSTRATEGY_OPTIMISTIC_INCLUSION
     * - AbstractMesh.CULLINGSTRATEGY_OPTIMISTIC_INCLUSION_THEN_BSPHERE_ONLY
     * Please read each static variable documentation to get details about the culling process.
     * */
    cullingStrategy: number;
    /**
     * Gets the number of facets in the mesh
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/facetData#what-is-a-mesh-facet
     */
    get facetNb(): number;
    /**
     * Gets or set the number (integer) of subdivisions per axis in the partitioning space
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/facetData#tweaking-the-partitioning
     */
    get partitioningSubdivisions(): number;
    set partitioningSubdivisions(nb: number);
    /**
     * The ratio (float) to apply to the bounding box size to set to the partitioning space.
     * Ex : 1.01 (default) the partitioning space is 1% bigger than the bounding box
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/facetData#tweaking-the-partitioning
     */
    get partitioningBBoxRatio(): number;
    set partitioningBBoxRatio(ratio: number);
    /**
     * Gets or sets a boolean indicating that the facets must be depth sorted on next call to `updateFacetData()`.
     * Works only for updatable meshes.
     * Doesn't work with multi-materials
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/facetData#facet-depth-sort
     */
    get mustDepthSortFacets(): boolean;
    set mustDepthSortFacets(sort: boolean);
    /**
     * The location (Vector3) where the facet depth sort must be computed from.
     * By default, the active camera position.
     * Used only when facet depth sort is enabled
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/facetData#facet-depth-sort
     */
    get facetDepthSortFrom(): Vector3;
    set facetDepthSortFrom(location: Vector3);
    /** number of collision detection tries. Change this value if not all collisions are detected and handled properly */
    get collisionRetryCount(): number;
    set collisionRetryCount(retryCount: number);
    /**
     * gets a boolean indicating if facetData is enabled
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/facetData#what-is-a-mesh-facet
     */
    get isFacetDataEnabled(): boolean;
    /**
     * Gets or sets the morph target manager
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/morphTargets
     */
    get morphTargetManager(): Nullable<MorphTargetManager>;
    set morphTargetManager(value: Nullable<MorphTargetManager>);
    /**
     * Gets or sets the baked vertex animation manager
     * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/baked_texture_animations
     */
    get bakedVertexAnimationManager(): Nullable<IBakedVertexAnimationManager>;
    set bakedVertexAnimationManager(value: Nullable<IBakedVertexAnimationManager>);
    /** @internal */
    _syncGeometryWithMorphTargetManager(): void;
    /**
     * @internal
     */
    _updateNonUniformScalingState(value: boolean): boolean;
    /**
     * An event triggered when this mesh collides with another one
     */
    onCollideObservable: Observable<AbstractMesh>;
    /** Set a function to call when this mesh collides with another one */
    set onCollide(callback: (collidedMesh?: AbstractMesh) => void);
    /**
     * An event triggered when the collision's position changes
     */
    onCollisionPositionChangeObservable: Observable<Vector3>;
    /** Set a function to call when the collision's position changes */
    set onCollisionPositionChange(callback: () => void);
    /**
     * An event triggered when material is changed
     */
    onMaterialChangedObservable: Observable<AbstractMesh>;
    /**
     * Gets or sets the orientation for POV movement & rotation
     */
    definedFacingForward: boolean;
    /** @internal */
    _occlusionQuery: Nullable<WebGLQuery | number>;
    /** @internal */
    _renderingGroup: Nullable<RenderingGroup>;
    /**
     * Gets or sets mesh visibility between 0 and 1 (default is 1)
     */
    get visibility(): number;
    /**
     * Gets or sets mesh visibility between 0 and 1 (default is 1)
     */
    set visibility(value: number);
    /** Gets or sets the alpha index used to sort transparent meshes
     * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/advanced/transparent_rendering#alpha-index
     */
    alphaIndex: number;
    /**
     * Gets or sets a boolean indicating if the mesh is visible (renderable). Default is true
     */
    isVisible: boolean;
    /**
     * Gets or sets a boolean indicating if the mesh can be picked (by scene.pick for instance or through actions). Default is true
     */
    isPickable: boolean;
    /**
     * Gets or sets a boolean indicating if the mesh can be near picked. Default is false
     */
    isNearPickable: boolean;
    /**
     * Gets or sets a boolean indicating if the mesh can be near grabbed. Default is false
     */
    isNearGrabbable: boolean;
    /** Gets or sets a boolean indicating that bounding boxes of subMeshes must be rendered as well (false by default) */
    showSubMeshesBoundingBox: boolean;
    /** Gets or sets a boolean indicating if the mesh must be considered as a ray blocker for lens flares (false by default)
     * @see https://doc.babylonjs.com/features/featuresDeepDive/environment/lenseFlare
     */
    isBlocker: boolean;
    /**
     * Gets or sets a boolean indicating that pointer move events must be supported on this mesh (false by default)
     */
    enablePointerMoveEvents: boolean;
    /**
     * Gets or sets the property which disables the test that is checking that the mesh under the pointer is the same than the previous time we tested for it (default: false).
     * Set this property to true if you want thin instances picking to be reported accurately when moving over the mesh.
     * Note that setting this property to true will incur some performance penalties when dealing with pointer events for this mesh so use it sparingly.
     */
    get pointerOverDisableMeshTesting(): boolean;
    set pointerOverDisableMeshTesting(disable: boolean);
    /**
     * Specifies the rendering group id for this mesh (0 by default)
     * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/advanced/transparent_rendering#rendering-groups
     */
    get renderingGroupId(): number;
    set renderingGroupId(value: number);
    /** Gets or sets current material */
    get material(): Nullable<Material>;
    set material(value: Nullable<Material>);
    /**
     * Gets the material used to render the mesh in a specific render pass
     * @param renderPassId render pass id
     * @returns material used for the render pass. If no specific material is used for this render pass, undefined is returned (meaning mesh.material is used for this pass)
     */
    getMaterialForRenderPass(renderPassId: number): Material | undefined;
    /**
     * Sets the material to be used to render the mesh in a specific render pass
     * @param renderPassId render pass id
     * @param material material to use for this render pass. If undefined is passed, no specific material will be used for this render pass but the regular material will be used instead (mesh.material)
     */
    setMaterialForRenderPass(renderPassId: number, material?: Material): void;
    /**
     * Gets or sets a boolean indicating that this mesh can receive realtime shadows
     * @see https://doc.babylonjs.com/features/featuresDeepDive/lights/shadows
     */
    get receiveShadows(): boolean;
    set receiveShadows(value: boolean);
    /** Defines color to use when rendering outline */
    outlineColor: Color3;
    /** Define width to use when rendering outline */
    outlineWidth: number;
    /** Defines color to use when rendering overlay */
    overlayColor: Color3;
    /** Defines alpha to use when rendering overlay */
    overlayAlpha: number;
    /** Gets or sets a boolean indicating that this mesh contains vertex color data with alpha values */
    get hasVertexAlpha(): boolean;
    set hasVertexAlpha(value: boolean);
    /** Gets or sets a boolean indicating that this mesh needs to use vertex color data to render (if this kind of vertex data is available in the geometry) */
    get useVertexColors(): boolean;
    set useVertexColors(value: boolean);
    /**
     * Gets or sets a boolean indicating that bone animations must be computed by the CPU (false by default)
     */
    get computeBonesUsingShaders(): boolean;
    set computeBonesUsingShaders(value: boolean);
    /** Gets or sets the number of allowed bone influences per vertex (4 by default) */
    get numBoneInfluencers(): number;
    set numBoneInfluencers(value: number);
    /** Gets or sets a boolean indicating that this mesh will allow fog to be rendered on it (true by default) */
    get applyFog(): boolean;
    set applyFog(value: boolean);
    /** When enabled, decompose picking matrices for better precision with large values for mesh position and scling */
    get enableDistantPicking(): boolean;
    set enableDistantPicking(value: boolean);
    /** Gets or sets a boolean indicating that internal octree (if available) can be used to boost submeshes selection (true by default) */
    useOctreeForRenderingSelection: boolean;
    /** Gets or sets a boolean indicating that internal octree (if available) can be used to boost submeshes picking (true by default) */
    useOctreeForPicking: boolean;
    /** Gets or sets a boolean indicating that internal octree (if available) can be used to boost submeshes collision (true by default) */
    useOctreeForCollisions: boolean;
    /**
     * Gets or sets the current layer mask (default is 0x0FFFFFFF)
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/layerMasksAndMultiCam
     */
    get layerMask(): number;
    set layerMask(value: number);
    /**
     * True if the mesh must be rendered in any case (this will shortcut the frustum clipping phase)
     */
    alwaysSelectAsActiveMesh: boolean;
    /**
     * Gets or sets a boolean indicating that the bounding info does not need to be kept in sync (for performance reason)
     */
    doNotSyncBoundingInfo: boolean;
    /**
     * Gets or sets the current action manager
     * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions
     */
    actionManager: Nullable<AbstractActionManager>;
    /**
     * Gets or sets the ellipsoid used to impersonate this mesh when using collision engine (default is (0.5, 1, 0.5))
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_collisions
     */
    ellipsoid: Vector3;
    /**
     * Gets or sets the ellipsoid offset used to impersonate this mesh when using collision engine (default is (0, 0, 0))
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_collisions
     */
    ellipsoidOffset: Vector3;
    /**
     * Gets or sets a collision mask used to mask collisions (default is -1).
     * A collision between A and B will happen if A.collisionGroup & b.collisionMask !== 0
     */
    get collisionMask(): number;
    set collisionMask(mask: number);
    /**
     * Gets or sets a collision response flag (default is true).
     * when collisionResponse is false, events are still triggered but colliding entity has no response
     * This helps creating trigger volume when user wants collision feedback events but not position/velocity
     * to respond to the collision.
     */
    get collisionResponse(): boolean;
    set collisionResponse(response: boolean);
    /**
     * Gets or sets the current collision group mask (-1 by default).
     * A collision between A and B will happen if A.collisionGroup & b.collisionMask !== 0
     */
    get collisionGroup(): number;
    set collisionGroup(mask: number);
    /**
     * Gets or sets current surrounding meshes (null by default).
     *
     * By default collision detection is tested against every mesh in the scene.
     * It is possible to set surroundingMeshes to a defined list of meshes and then only these specified
     * meshes will be tested for the collision.
     *
     * Note: if set to an empty array no collision will happen when this mesh is moved.
     */
    get surroundingMeshes(): Nullable<AbstractMesh[]>;
    set surroundingMeshes(meshes: Nullable<AbstractMesh[]>);
    /**
     * Defines edge width used when edgesRenderer is enabled
     * @see https://www.babylonjs-playground.com/#10OJSG#13
     */
    edgesWidth: number;
    /**
     * Defines edge color used when edgesRenderer is enabled
     * @see https://www.babylonjs-playground.com/#10OJSG#13
     */
    edgesColor: Color4;
    /** @internal */
    _edgesRenderer: Nullable<IEdgesRenderer>;
    /** @internal */
    _masterMesh: Nullable<AbstractMesh>;
    protected _boundingInfo: Nullable<BoundingInfo>;
    protected _boundingInfoIsDirty: boolean;
    /** @internal */
    _renderId: number;
    /**
     * Gets or sets the list of subMeshes
     * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/multiMaterials
     */
    subMeshes: SubMesh[];
    /** @internal */
    _intersectionsInProgress: AbstractMesh[];
    /** @internal */
    _unIndexed: boolean;
    /** @internal */
    _lightSources: Light[];
    /** Gets the list of lights affecting that mesh */
    get lightSources(): Light[];
    /** @internal */
    get _positions(): Nullable<Vector3[]>;
    /** @internal */
    _waitingData: {
        lods: Nullable<any>;
        actions: Nullable<any>;
        freezeWorldMatrix: Nullable<boolean>;
    };
    /** @internal */
    _bonesTransformMatrices: Nullable<Float32Array>;
    /** @internal */
    _transformMatrixTexture: Nullable<RawTexture>;
    /**
     * Gets or sets a skeleton to apply skinning transformations
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/bonesSkeletons
     */
    set skeleton(value: Nullable<Skeleton>);
    get skeleton(): Nullable<Skeleton>;
    /**
     * An event triggered when the mesh is rebuilt.
     */
    onRebuildObservable: Observable<AbstractMesh>;
    /**
     * The current mesh uniform buffer.
     * @internal Internal use only.
     */
    _uniformBuffer: UniformBuffer;
    /**
     * Creates a new AbstractMesh
     * @param name defines the name of the mesh
     * @param scene defines the hosting scene
     */
    constructor(name: string, scene?: Nullable<Scene>);
    protected _buildUniformLayout(): void;
    /**
     * Transfer the mesh values to its UBO.
     * @param world The world matrix associated with the mesh
     */
    transferToEffect(world: Matrix): void;
    /**
     * Gets the mesh uniform buffer.
     * @returns the uniform buffer of the mesh.
     */
    getMeshUniformBuffer(): UniformBuffer;
    /**
     * Returns the string "AbstractMesh"
     * @returns "AbstractMesh"
     */
    getClassName(): string;
    /**
     * Gets a string representation of the current mesh
     * @param fullDetails defines a boolean indicating if full details must be included
     * @returns a string representation of the current mesh
     */
    toString(fullDetails?: boolean): string;
    /**
     * @internal
     */
    protected _getEffectiveParent(): Nullable<Node>;
    /**
     * @internal
     */
    _getActionManagerForTrigger(trigger?: number, initialCall?: boolean): Nullable<AbstractActionManager>;
    /**
     * @internal
     */
    _rebuild(dispose?: boolean): void;
    /** @internal */
    _resyncLightSources(): void;
    /**
     * @internal
     */
    _resyncLightSource(light: Light): void;
    /** @internal */
    _unBindEffect(): void;
    /**
     * @internal
     */
    _removeLightSource(light: Light, dispose: boolean): void;
    private _markSubMeshesAsDirty;
    /**
     * @internal
     */
    _markSubMeshesAsLightDirty(dispose?: boolean): void;
    /** @internal */
    _markSubMeshesAsAttributesDirty(): void;
    /** @internal */
    _markSubMeshesAsMiscDirty(): void;
    /**
     * Flag the AbstractMesh as dirty (Forcing it to update everything)
     * @param property if set to "rotation" the objects rotationQuaternion will be set to null
     * @returns this AbstractMesh
     */
    markAsDirty(property?: string): AbstractMesh;
    /**
     * Resets the draw wrappers cache for all submeshes of this abstract mesh
     * @param passId If provided, releases only the draw wrapper corresponding to this render pass id
     */
    resetDrawCache(passId?: number): void;
    /**
     * Returns true if the mesh is blocked. Implemented by child classes
     */
    get isBlocked(): boolean;
    /**
     * Returns the mesh itself by default. Implemented by child classes
     * @param camera defines the camera to use to pick the right LOD level
     * @returns the currentAbstractMesh
     */
    getLOD(camera: Camera): Nullable<AbstractMesh>;
    /**
     * Returns 0 by default. Implemented by child classes
     * @returns an integer
     */
    getTotalVertices(): number;
    /**
     * Returns a positive integer : the total number of indices in this mesh geometry.
     * @returns the number of indices or zero if the mesh has no geometry.
     */
    getTotalIndices(): number;
    /**
     * Returns null by default. Implemented by child classes
     * @returns null
     */
    getIndices(): Nullable<IndicesArray>;
    /**
     * Returns the array of the requested vertex data kind. Implemented by child classes
     * @param kind defines the vertex data kind to use
     * @returns null
     */
    getVerticesData(kind: string): Nullable<FloatArray>;
    /**
     * Sets the vertex data of the mesh geometry for the requested `kind`.
     * If the mesh has no geometry, a new Geometry object is set to the mesh and then passed this vertex data.
     * Note that a new underlying VertexBuffer object is created each call.
     * If the `kind` is the `PositionKind`, the mesh BoundingInfo is renewed, so the bounding box and sphere, and the mesh World Matrix is recomputed.
     * @param kind defines vertex data kind:
     * * VertexBuffer.PositionKind
     * * VertexBuffer.UVKind
     * * VertexBuffer.UV2Kind
     * * VertexBuffer.UV3Kind
     * * VertexBuffer.UV4Kind
     * * VertexBuffer.UV5Kind
     * * VertexBuffer.UV6Kind
     * * VertexBuffer.ColorKind
     * * VertexBuffer.MatricesIndicesKind
     * * VertexBuffer.MatricesIndicesExtraKind
     * * VertexBuffer.MatricesWeightsKind
     * * VertexBuffer.MatricesWeightsExtraKind
     * @param data defines the data source
     * @param updatable defines if the data must be flagged as updatable (or static)
     * @param stride defines the vertex stride (size of an entire vertex). Can be null and in this case will be deduced from vertex data kind
     * @returns the current mesh
     */
    setVerticesData(kind: string, data: FloatArray, updatable?: boolean, stride?: number): AbstractMesh;
    /**
     * Updates the existing vertex data of the mesh geometry for the requested `kind`.
     * If the mesh has no geometry, it is simply returned as it is.
     * @param kind defines vertex data kind:
     * * VertexBuffer.PositionKind
     * * VertexBuffer.UVKind
     * * VertexBuffer.UV2Kind
     * * VertexBuffer.UV3Kind
     * * VertexBuffer.UV4Kind
     * * VertexBuffer.UV5Kind
     * * VertexBuffer.UV6Kind
     * * VertexBuffer.ColorKind
     * * VertexBuffer.MatricesIndicesKind
     * * VertexBuffer.MatricesIndicesExtraKind
     * * VertexBuffer.MatricesWeightsKind
     * * VertexBuffer.MatricesWeightsExtraKind
     * @param data defines the data source
     * @param updateExtends If `kind` is `PositionKind` and if `updateExtends` is true, the mesh BoundingInfo is renewed, so the bounding box and sphere, and the mesh World Matrix is recomputed
     * @param makeItUnique If true, a new global geometry is created from this data and is set to the mesh
     * @returns the current mesh
     */
    updateVerticesData(kind: string, data: FloatArray, updateExtends?: boolean, makeItUnique?: boolean): AbstractMesh;
    /**
     * Sets the mesh indices,
     * If the mesh has no geometry, a new Geometry object is created and set to the mesh.
     * @param indices Expects an array populated with integers or a typed array (Int32Array, Uint32Array, Uint16Array)
     * @param totalVertices Defines the total number of vertices
     * @returns the current mesh
     */
    setIndices(indices: IndicesArray, totalVertices: Nullable<number>): AbstractMesh;
    /**
     * Gets a boolean indicating if specific vertex data is present
     * @param kind defines the vertex data kind to use
     * @returns true is data kind is present
     */
    isVerticesDataPresent(kind: string): boolean;
    /**
     * Returns the mesh BoundingInfo object or creates a new one and returns if it was undefined.
     * Note that it returns a shallow bounding of the mesh (i.e. it does not include children).
     * To get the full bounding of all children, call `getHierarchyBoundingVectors` instead.
     * @returns a BoundingInfo
     */
    getBoundingInfo(): BoundingInfo;
    /**
     * Overwrite the current bounding info
     * @param boundingInfo defines the new bounding info
     * @returns the current mesh
     */
    setBoundingInfo(boundingInfo: BoundingInfo): AbstractMesh;
    /**
     * Returns true if there is already a bounding info
     */
    get hasBoundingInfo(): boolean;
    /**
     * Creates a new bounding info for the mesh
     * @param minimum min vector of the bounding box/sphere
     * @param maximum max vector of the bounding box/sphere
     * @param worldMatrix defines the new world matrix
     * @returns the new bounding info
     */
    buildBoundingInfo(minimum: DeepImmutable<Vector3>, maximum: DeepImmutable<Vector3>, worldMatrix?: DeepImmutable<Matrix>): BoundingInfo;
    /**
     * Uniformly scales the mesh to fit inside of a unit cube (1 X 1 X 1 units)
     * @param includeDescendants Use the hierarchy's bounding box instead of the mesh's bounding box. Default is false
     * @param ignoreRotation ignore rotation when computing the scale (ie. object will be axis aligned). Default is false
     * @param predicate predicate that is passed in to getHierarchyBoundingVectors when selecting which object should be included when scaling
     * @returns the current mesh
     */
    normalizeToUnitCube(includeDescendants?: boolean, ignoreRotation?: boolean, predicate?: Nullable<(node: AbstractMesh) => boolean>): AbstractMesh;
    /** Gets a boolean indicating if this mesh has skinning data and an attached skeleton */
    get useBones(): boolean;
    /** @internal */
    _preActivate(): void;
    /**
     * @internal
     */
    _preActivateForIntermediateRendering(renderId: number): void;
    /**
     * @internal
     */
    _activate(renderId: number, intermediateRendering: boolean): boolean;
    /** @internal */
    _postActivate(): void;
    /** @internal */
    _freeze(): void;
    /** @internal */
    _unFreeze(): void;
    /**
     * Gets the current world matrix
     * @returns a Matrix
     */
    getWorldMatrix(): Matrix;
    /** @internal */
    _getWorldMatrixDeterminant(): number;
    /**
     * Gets a boolean indicating if this mesh is an instance or a regular mesh
     */
    get isAnInstance(): boolean;
    /**
     * Gets a boolean indicating if this mesh has instances
     */
    get hasInstances(): boolean;
    /**
     * Gets a boolean indicating if this mesh has thin instances
     */
    get hasThinInstances(): boolean;
    /**
     * Perform relative position change from the point of view of behind the front of the mesh.
     * This is performed taking into account the meshes current rotation, so you do not have to care.
     * Supports definition of mesh facing forward or backward {@link definedFacingForwardSearch | See definedFacingForwardSearch }.
     * @param amountRight defines the distance on the right axis
     * @param amountUp defines the distance on the up axis
     * @param amountForward defines the distance on the forward axis
     * @returns the current mesh
     */
    movePOV(amountRight: number, amountUp: number, amountForward: number): AbstractMesh;
    /**
     * Calculate relative position change from the point of view of behind the front of the mesh.
     * This is performed taking into account the meshes current rotation, so you do not have to care.
     * Supports definition of mesh facing forward or backward {@link definedFacingForwardSearch | See definedFacingForwardSearch }.
     * @param amountRight defines the distance on the right axis
     * @param amountUp defines the distance on the up axis
     * @param amountForward defines the distance on the forward axis
     * @returns the new displacement vector
     */
    calcMovePOV(amountRight: number, amountUp: number, amountForward: number): Vector3;
    /**
     * Perform relative rotation change from the point of view of behind the front of the mesh.
     * Supports definition of mesh facing forward or backward {@link definedFacingForwardSearch | See definedFacingForwardSearch }.
     * @param flipBack defines the flip
     * @param twirlClockwise defines the twirl
     * @param tiltRight defines the tilt
     * @returns the current mesh
     */
    rotatePOV(flipBack: number, twirlClockwise: number, tiltRight: number): AbstractMesh;
    /**
     * Calculate relative rotation change from the point of view of behind the front of the mesh.
     * Supports definition of mesh facing forward or backward {@link definedFacingForwardSearch | See definedFacingForwardSearch }.
     * @param flipBack defines the flip
     * @param twirlClockwise defines the twirl
     * @param tiltRight defines the tilt
     * @returns the new rotation vector
     */
    calcRotatePOV(flipBack: number, twirlClockwise: number, tiltRight: number): Vector3;
    /**
     * This method recomputes and sets a new BoundingInfo to the mesh unless it is locked.
     * This means the mesh underlying bounding box and sphere are recomputed.
     * @param applySkeleton defines whether to apply the skeleton before computing the bounding info
     * @param applyMorph  defines whether to apply the morph target before computing the bounding info
     * @returns the current mesh
     */
    refreshBoundingInfo(applySkeleton?: boolean, applyMorph?: boolean): AbstractMesh;
    /**
     * @internal
     */
    _refreshBoundingInfo(data: Nullable<FloatArray>, bias: Nullable<Vector2>): void;
    /**
     * Internal function to get buffer data and possibly apply morphs and normals
     * @param applySkeleton
     * @param applyMorph
     * @param data
     * @param kind the kind of data you want. Can be Normal or Position
     */
    private _getData;
    /**
     * Get the normals vertex data and optionally apply skeleton and morphing.
     * @param applySkeleton defines whether to apply the skeleton
     * @param applyMorph  defines whether to apply the morph target
     * @returns the normals data
     */
    getNormalsData(applySkeleton?: boolean, applyMorph?: boolean): Nullable<FloatArray>;
    /**
     * Get the position vertex data and optionally apply skeleton and morphing.
     * @param applySkeleton defines whether to apply the skeleton
     * @param applyMorph  defines whether to apply the morph target
     * @param data defines the position data to apply the skeleton and morph to
     * @returns the position data
     */
    getPositionData(applySkeleton?: boolean, applyMorph?: boolean, data?: Nullable<FloatArray>): Nullable<FloatArray>;
    /**
     * @internal
     */
    _getPositionData(applySkeleton: boolean, applyMorph: boolean): Nullable<FloatArray>;
    /** @internal */
    _updateBoundingInfo(): AbstractMesh;
    /**
     * @internal
     */
    _updateSubMeshesBoundingInfo(matrix: DeepImmutable<Matrix>): AbstractMesh;
    /** @internal */
    protected _afterComputeWorldMatrix(): void;
    /**
     * Returns `true` if the mesh is within the frustum defined by the passed array of planes.
     * A mesh is in the frustum if its bounding box intersects the frustum
     * @param frustumPlanes defines the frustum to test
     * @returns true if the mesh is in the frustum planes
     */
    isInFrustum(frustumPlanes: Plane[]): boolean;
    /**
     * Returns `true` if the mesh is completely in the frustum defined be the passed array of planes.
     * A mesh is completely in the frustum if its bounding box it completely inside the frustum.
     * @param frustumPlanes defines the frustum to test
     * @returns true if the mesh is completely in the frustum planes
     */
    isCompletelyInFrustum(frustumPlanes: Plane[]): boolean;
    /**
     * True if the mesh intersects another mesh or a SolidParticle object
     * @param mesh defines a target mesh or SolidParticle to test
     * @param precise Unless the parameter `precise` is set to `true` the intersection is computed according to Axis Aligned Bounding Boxes (AABB), else according to OBB (Oriented BBoxes)
     * @param includeDescendants Can be set to true to test if the mesh defined in parameters intersects with the current mesh or any child meshes
     * @returns true if there is an intersection
     */
    intersectsMesh(mesh: AbstractMesh | SolidParticle, precise?: boolean, includeDescendants?: boolean): boolean;
    /**
     * Returns true if the passed point (Vector3) is inside the mesh bounding box
     * @param point defines the point to test
     * @returns true if there is an intersection
     */
    intersectsPoint(point: Vector3): boolean;
    /**
     * Gets or sets a boolean indicating that this mesh can be used in the collision engine
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_collisions
     */
    get checkCollisions(): boolean;
    set checkCollisions(collisionEnabled: boolean);
    /**
     * Gets Collider object used to compute collisions (not physics)
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_collisions
     */
    get collider(): Nullable<Collider>;
    /**
     * Move the mesh using collision engine
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_collisions
     * @param displacement defines the requested displacement vector
     * @returns the current mesh
     */
    moveWithCollisions(displacement: Vector3): AbstractMesh;
    private _onCollisionPositionChange;
    /**
     * @internal
     */
    _collideForSubMesh(subMesh: SubMesh, transformMatrix: Matrix, collider: Collider): AbstractMesh;
    /**
     * @internal
     */
    _processCollisionsForSubMeshes(collider: Collider, transformMatrix: Matrix): AbstractMesh;
    /** @internal */
    _shouldConvertRHS(): boolean;
    /**
     * @internal
     */
    _checkCollision(collider: Collider): AbstractMesh;
    /** @internal */
    _generatePointsArray(): boolean;
    /**
     * Checks if the passed Ray intersects with the mesh
     * @param ray defines the ray to use
     * @param fastCheck defines if fast mode (but less precise) must be used (false by default)
     * @param trianglePredicate defines an optional predicate used to select faces when a mesh intersection is detected
     * @param onlyBoundingInfo defines a boolean indicating if picking should only happen using bounding info (false by default)
     * @param worldToUse defines the world matrix to use to get the world coordinate of the intersection point
     * @param skipBoundingInfo a boolean indicating if we should skip the bounding info check
     * @returns the picking info
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/interactions/mesh_intersect
     */
    intersects(ray: Ray, fastCheck?: boolean, trianglePredicate?: TrianglePickingPredicate, onlyBoundingInfo?: boolean, worldToUse?: Matrix, skipBoundingInfo?: boolean): PickingInfo;
    /**
     * Clones the current mesh
     * @param name defines the mesh name
     * @param newParent defines the new mesh parent
     * @param doNotCloneChildren defines a boolean indicating that children must not be cloned (false by default)
     * @returns the new mesh
     */
    clone(name: string, newParent: Nullable<Node>, doNotCloneChildren?: boolean): Nullable<AbstractMesh>;
    /**
     * Disposes all the submeshes of the current meshnp
     * @returns the current mesh
     */
    releaseSubMeshes(): AbstractMesh;
    /**
     * Releases resources associated with this abstract mesh.
     * @param doNotRecurse Set to true to not recurse into each children (recurse into each children by default)
     * @param disposeMaterialAndTextures Set to true to also dispose referenced materials and textures (false by default)
     */
    dispose(doNotRecurse?: boolean, disposeMaterialAndTextures?: boolean): void;
    /**
     * Adds the passed mesh as a child to the current mesh
     * @param mesh defines the child mesh
     * @param preserveScalingSign if true, keep scaling sign of child. Otherwise, scaling sign might change.
     * @returns the current mesh
     */
    addChild(mesh: AbstractMesh, preserveScalingSign?: boolean): AbstractMesh;
    /**
     * Removes the passed mesh from the current mesh children list
     * @param mesh defines the child mesh
     * @param preserveScalingSign if true, keep scaling sign of child. Otherwise, scaling sign might change.
     * @returns the current mesh
     */
    removeChild(mesh: AbstractMesh, preserveScalingSign?: boolean): AbstractMesh;
    /** @internal */
    private _initFacetData;
    /**
     * Updates the mesh facetData arrays and the internal partitioning when the mesh is morphed or updated.
     * This method can be called within the render loop.
     * You don't need to call this method by yourself in the render loop when you update/morph a mesh with the methods CreateXXX() as they automatically manage this computation
     * @returns the current mesh
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/facetData
     */
    updateFacetData(): AbstractMesh;
    /**
     * Returns the facetLocalNormals array.
     * The normals are expressed in the mesh local spac
     * @returns an array of Vector3
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/facetData
     */
    getFacetLocalNormals(): Vector3[];
    /**
     * Returns the facetLocalPositions array.
     * The facet positions are expressed in the mesh local space
     * @returns an array of Vector3
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/facetData
     */
    getFacetLocalPositions(): Vector3[];
    /**
     * Returns the facetLocalPartitioning array
     * @returns an array of array of numbers
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/facetData
     */
    getFacetLocalPartitioning(): number[][];
    /**
     * Returns the i-th facet position in the world system.
     * This method allocates a new Vector3 per call
     * @param i defines the facet index
     * @returns a new Vector3
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/facetData
     */
    getFacetPosition(i: number): Vector3;
    /**
     * Sets the reference Vector3 with the i-th facet position in the world system
     * @param i defines the facet index
     * @param ref defines the target vector
     * @returns the current mesh
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/facetData
     */
    getFacetPositionToRef(i: number, ref: Vector3): AbstractMesh;
    /**
     * Returns the i-th facet normal in the world system.
     * This method allocates a new Vector3 per call
     * @param i defines the facet index
     * @returns a new Vector3
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/facetData
     */
    getFacetNormal(i: number): Vector3;
    /**
     * Sets the reference Vector3 with the i-th facet normal in the world system
     * @param i defines the facet index
     * @param ref defines the target vector
     * @returns the current mesh
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/facetData
     */
    getFacetNormalToRef(i: number, ref: Vector3): this;
    /**
     * Returns the facets (in an array) in the same partitioning block than the one the passed coordinates are located (expressed in the mesh local system)
     * @param x defines x coordinate
     * @param y defines y coordinate
     * @param z defines z coordinate
     * @returns the array of facet indexes
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/facetData
     */
    getFacetsAtLocalCoordinates(x: number, y: number, z: number): Nullable<number[]>;
    /**
     * Returns the closest mesh facet index at (x,y,z) World coordinates, null if not found
     * @param x defines x coordinate
     * @param y defines y coordinate
     * @param z defines z coordinate
     * @param projected sets as the (x,y,z) world projection on the facet
     * @param checkFace if true (default false), only the facet "facing" to (x,y,z) or only the ones "turning their backs", according to the parameter "facing" are returned
     * @param facing if facing and checkFace are true, only the facet "facing" to (x, y, z) are returned : positive dot (x, y, z) * facet position. If facing si false and checkFace is true, only the facet "turning their backs" to (x, y, z) are returned : negative dot (x, y, z) * facet position
     * @returns the face index if found (or null instead)
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/facetData
     */
    getClosestFacetAtCoordinates(x: number, y: number, z: number, projected?: Vector3, checkFace?: boolean, facing?: boolean): Nullable<number>;
    /**
     * Returns the closest mesh facet index at (x,y,z) local coordinates, null if not found
     * @param x defines x coordinate
     * @param y defines y coordinate
     * @param z defines z coordinate
     * @param projected sets as the (x,y,z) local projection on the facet
     * @param checkFace if true (default false), only the facet "facing" to (x,y,z) or only the ones "turning their backs", according to the parameter "facing" are returned
     * @param facing if facing and checkFace are true, only the facet "facing" to (x, y, z) are returned : positive dot (x, y, z) * facet position. If facing si false and checkFace is true, only the facet "turning their backs" to (x, y, z) are returned : negative dot (x, y, z) * facet position
     * @returns the face index if found (or null instead)
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/facetData
     */
    getClosestFacetAtLocalCoordinates(x: number, y: number, z: number, projected?: Vector3, checkFace?: boolean, facing?: boolean): Nullable<number>;
    /**
     * Returns the object "parameter" set with all the expected parameters for facetData computation by ComputeNormals()
     * @returns the parameters
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/facetData
     */
    getFacetDataParameters(): any;
    /**
     * Disables the feature FacetData and frees the related memory
     * @returns the current mesh
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/facetData
     */
    disableFacetData(): AbstractMesh;
    /**
     * Updates the AbstractMesh indices array
     * @param indices defines the data source
     * @param offset defines the offset in the index buffer where to store the new data (can be null)
     * @param gpuMemoryOnly defines a boolean indicating that only the GPU memory must be updated leaving the CPU version of the indices unchanged (false by default)
     * @returns the current mesh
     */
    updateIndices(indices: IndicesArray, offset?: number, gpuMemoryOnly?: boolean): AbstractMesh;
    /**
     * Creates new normals data for the mesh
     * @param updatable defines if the normal vertex buffer must be flagged as updatable
     * @returns the current mesh
     */
    createNormals(updatable: boolean): AbstractMesh;
    /**
     * Align the mesh with a normal
     * @param normal defines the normal to use
     * @param upDirection can be used to redefined the up vector to use (will use the (0, 1, 0) by default)
     * @returns the current mesh
     */
    alignWithNormal(normal: Vector3, upDirection?: Vector3): AbstractMesh;
    /** @internal */
    _checkOcclusionQuery(): boolean;
    /**
     * Disables the mesh edge rendering mode
     * @returns the currentAbstractMesh
     */
    disableEdgesRendering(): AbstractMesh;
    /**
     * Enables the edge rendering mode on the mesh.
     * This mode makes the mesh edges visible
     * @param epsilon defines the maximal distance between two angles to detect a face
     * @param checkVerticesInsteadOfIndices indicates that we should check vertex list directly instead of faces
     * @param options options to the edge renderer
     * @returns the currentAbstractMesh
     * @see https://www.babylonjs-playground.com/#19O9TU#0
     */
    enableEdgesRendering(epsilon?: number, checkVerticesInsteadOfIndices?: boolean, options?: IEdgesRendererOptions): AbstractMesh;
    /**
     * This function returns all of the particle systems in the scene that use the mesh as an emitter.
     * @returns an array of particle systems in the scene that use the mesh as an emitter
     */
    getConnectedParticleSystems(): IParticleSystem[];
}
export {};

}
declare module "babylonjs/Meshes/buffer" {
/**
 * This is here for backwards compatibility with 4.2
 * @internal
 */
export { VertexBuffer, Buffer } from "babylonjs/Buffers/buffer";

}
declare module "babylonjs/Meshes/Builders/boxBuilder" {
import { Nullable } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
import { Vector4 } from "babylonjs/Maths/math.vector";
import { Color4 } from "babylonjs/Maths/math.color";
import { Mesh } from "babylonjs/Meshes/mesh";
import { VertexData } from "babylonjs/Meshes/mesh.vertexData";
/**
 * Creates the VertexData for a box
 * @param options an object used to set the following optional parameters for the box, required but can be empty
 * * size sets the width, height and depth of the box to the value of size, optional default 1
 * * width sets the width (x direction) of the box, overwrites the width set by size, optional, default size
 * * height sets the height (y direction) of the box, overwrites the height set by size, optional, default size
 * * depth sets the depth (z direction) of the box, overwrites the depth set by size, optional, default size
 * * faceUV an array of 6 Vector4 elements used to set different images to each box side
 * * faceColors an array of 6 Color3 elements used to set different colors to each box side
 * * sideOrientation optional and takes the values : Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
 * * frontUvs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the front side, optional, default vector4 (0, 0, 1, 1)
 * * backUVs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the back side, optional, default vector4 (0, 0, 1, 1)
 * @param options.size
 * @param options.width
 * @param options.height
 * @param options.depth
 * @param options.faceUV
 * @param options.faceColors
 * @param options.sideOrientation
 * @param options.frontUVs
 * @param options.backUVs
 * @param options.wrap
 * @param options.topBaseAt
 * @param options.bottomBaseAt
 * @returns the VertexData of the box
 */
export function CreateBoxVertexData(options: {
    size?: number;
    width?: number;
    height?: number;
    depth?: number;
    faceUV?: Vector4[];
    faceColors?: Color4[];
    sideOrientation?: number;
    frontUVs?: Vector4;
    backUVs?: Vector4;
    wrap?: boolean;
    topBaseAt?: number;
    bottomBaseAt?: number;
}): VertexData;
/**
 * Creates a box mesh
 * * The parameter `size` sets the size (float) of each box side (default 1)
 * * You can set some different box dimensions by using the parameters `width`, `height` and `depth` (all by default have the same value of `size`)
 * * You can set different colors and different images to each box side by using the parameters `faceColors` (an array of 6 Color3 elements) and `faceUV` (an array of 6 Vector4 elements)
 * * Please read this tutorial : https://doc.babylonjs.com/features/featuresDeepDive/materials/using/texturePerBoxFace
 * * You can also set the mesh side orientation with the values : BABYLON.Mesh.FRONTSIDE (default), BABYLON.Mesh.BACKSIDE or BABYLON.Mesh.DOUBLESIDE
 * * If you create a double-sided mesh, you can choose what parts of the texture image to crop and stick respectively on the front and the back sides with the parameters `frontUVs` and `backUVs` (Vector4). Detail here : https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#side-orientation
 * * The mesh can be set to updatable with the boolean parameter `updatable` (default false) if its internal geometry is supposed to change once created
 * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#box
 * @param name defines the name of the mesh
 * @param options defines the options used to create the mesh
 * @param options.size
 * @param options.width
 * @param options.height
 * @param options.depth
 * @param options.faceUV
 * @param options.faceColors
 * @param options.sideOrientation
 * @param options.frontUVs
 * @param options.backUVs
 * @param options.wrap
 * @param options.topBaseAt
 * @param options.bottomBaseAt
 * @param options.updatable
 * @param scene defines the hosting scene
 * @returns the box mesh
 */
export function CreateBox(name: string, options?: {
    size?: number;
    width?: number;
    height?: number;
    depth?: number;
    faceUV?: Vector4[];
    faceColors?: Color4[];
    sideOrientation?: number;
    frontUVs?: Vector4;
    backUVs?: Vector4;
    wrap?: boolean;
    topBaseAt?: number;
    bottomBaseAt?: number;
    updatable?: boolean;
}, scene?: Nullable<Scene>): Mesh;
/**
 * Class containing static functions to help procedurally build meshes
 * @deprecated please use CreateBox directly
 */
export const BoxBuilder: {
    CreateBox: typeof CreateBox;
};

}
declare module "babylonjs/Meshes/Builders/capsuleBuilder" {
import { VertexData } from "babylonjs/Meshes/mesh.vertexData";
import { Vector3 } from "babylonjs/Maths/math.vector";
import { Mesh } from "babylonjs/Meshes/mesh";
import { Nullable } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
/**
 * Scripts based off of https://github.com/maximeq/three-js-capsule-geometry/blob/master/src/CapsuleBufferGeometry.js
 * @param options the constructors options used to shape the mesh.
 * @returns the capsule VertexData
 * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set/capsule
 */
export function CreateCapsuleVertexData(options?: ICreateCapsuleOptions): VertexData;
/**
 * The options Interface for creating a Capsule Mesh
 */
export interface ICreateCapsuleOptions {
    /** The Orientation of the capsule.  Default : Vector3.Up() */
    orientation?: Vector3;
    /** Number of sub segments on the tube section of the capsule running parallel to orientation. */
    subdivisions?: number;
    /** Number of cylindrical segments on the capsule. */
    tessellation?: number;
    /** Height or Length of the capsule. */
    height?: number;
    /** Radius of the capsule. */
    radius?: number;
    /** Number of sub segments on the cap sections of the capsule running parallel to orientation. */
    capSubdivisions?: number;
    /** Overwrite for the top radius. */
    radiusTop?: number;
    /** Overwrite for the bottom radius. */
    radiusBottom?: number;
    /** Overwrite for the top capSubdivisions. */
    topCapSubdivisions?: number;
    /** Overwrite for the bottom capSubdivisions. */
    bottomCapSubdivisions?: number;
    /** Internal geometry is supposed to change once created. */
    updatable?: boolean;
}
/**
 * Creates a capsule or a pill mesh
 * @param name defines the name of the mesh
 * @param options The constructors options.
 * @param scene The scene the mesh is scoped to.
 * @returns Capsule Mesh
 */
export function CreateCapsule(name: string, options?: ICreateCapsuleOptions, scene?: Nullable<Scene>): Mesh;
/**
 * Class containing static functions to help procedurally build meshes
 * @deprecated please use CreateCapsule directly
 */
export const CapsuleBuilder: {
    CreateCapsule: typeof CreateCapsule;
};

}
declare module "babylonjs/Meshes/Builders/cylinderBuilder" {
import { Vector4 } from "babylonjs/Maths/math.vector";
import { Color4 } from "babylonjs/Maths/math.color";
import { Mesh } from "babylonjs/Meshes/mesh";
import { VertexData } from "babylonjs/Meshes/mesh.vertexData";
import { Scene } from "babylonjs/scene";
import { Nullable } from "babylonjs/types";
/**
 * Creates the VertexData for a cylinder, cone or prism
 * @param options an object used to set the following optional parameters for the box, required but can be empty
 * * height sets the height (y direction) of the cylinder, optional, default 2
 * * diameterTop sets the diameter of the top of the cone, overwrites diameter,  optional, default diameter
 * * diameterBottom sets the diameter of the bottom of the cone, overwrites diameter,  optional, default diameter
 * * diameter sets the diameter of the top and bottom of the cone, optional default 1
 * * tessellation the number of prism sides, 3 for a triangular prism, optional, default 24
 * * subdivisions` the number of rings along the cylinder height, optional, default 1
 * * arc a number from 0 to 1, to create an unclosed cylinder based on the fraction of the circumference given by the arc value, optional, default 1
 * * faceColors an array of Color3 elements used to set different colors to the top, rings and bottom respectively
 * * faceUV an array of Vector4 elements used to set different images to the top, rings and bottom respectively
 * * hasRings when true makes each subdivision independently treated as a face for faceUV and faceColors, optional, default false
 * * enclose when true closes an open cylinder by adding extra flat faces between the height axis and vertical edges, think cut cake
 * * sideOrientation optional and takes the values : Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
 * * frontUvs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the front side, optional, default vector4 (0, 0, 1, 1)
 * * backUVs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the back side, optional, default vector4 (0, 0, 1, 1)
 * @param options.height
 * @param options.diameterTop
 * @param options.diameterBottom
 * @param options.diameter
 * @param options.tessellation
 * @param options.subdivisions
 * @param options.arc
 * @param options.faceColors
 * @param options.faceUV
 * @param options.hasRings
 * @param options.enclose
 * @param options.cap
 * @param options.sideOrientation
 * @param options.frontUVs
 * @param options.backUVs
 * @returns the VertexData of the cylinder, cone or prism
 */
export function CreateCylinderVertexData(options: {
    height?: number;
    diameterTop?: number;
    diameterBottom?: number;
    diameter?: number;
    tessellation?: number;
    subdivisions?: number;
    arc?: number;
    faceColors?: Color4[];
    faceUV?: Vector4[];
    hasRings?: boolean;
    enclose?: boolean;
    cap?: number;
    sideOrientation?: number;
    frontUVs?: Vector4;
    backUVs?: Vector4;
}): VertexData;
/**
 * Creates a cylinder or a cone mesh
 * * The parameter `height` sets the height size (float) of the cylinder/cone (float, default 2).
 * * The parameter `diameter` sets the diameter of the top and bottom cap at once (float, default 1).
 * * The parameters `diameterTop` and `diameterBottom` overwrite the parameter `diameter` and set respectively the top cap and bottom cap diameter (floats, default 1). The parameter "diameterBottom" can't be zero.
 * * The parameter `tessellation` sets the number of cylinder sides (positive integer, default 24). Set it to 3 to get a prism for instance.
 * * The parameter `subdivisions` sets the number of rings along the cylinder height (positive integer, default 1).
 * * The parameter `hasRings` (boolean, default false) makes the subdivisions independent from each other, so they become different faces.
 * * The parameter `enclose`  (boolean, default false) adds two extra faces per subdivision to a sliced cylinder to close it around its height axis.
 * * The parameter `cap` sets the way the cylinder is capped. Possible values : BABYLON.Mesh.NO_CAP, BABYLON.Mesh.CAP_START, BABYLON.Mesh.CAP_END, BABYLON.Mesh.CAP_ALL (default).
 * * The parameter `arc` (float, default 1) is the ratio (max 1) to apply to the circumference to slice the cylinder.
 * * You can set different colors and different images to each box side by using the parameters `faceColors` (an array of n Color3 elements) and `faceUV` (an array of n Vector4 elements).
 * * The value of n is the number of cylinder faces. If the cylinder has only 1 subdivisions, n equals : top face + cylinder surface + bottom face = 3
 * * Now, if the cylinder has 5 independent subdivisions (hasRings = true), n equals : top face + 5 stripe surfaces + bottom face = 2 + 5 = 7
 * * Finally, if the cylinder has 5 independent subdivisions and is enclose, n equals : top face + 5 x (stripe surface + 2 closing faces) + bottom face = 2 + 5 * 3 = 17
 * * Each array (color or UVs) is always ordered the same way : the first element is the bottom cap, the last element is the top cap. The other elements are each a ring surface.
 * * If `enclose` is false, a ring surface is one element.
 * * If `enclose` is true, a ring surface is 3 successive elements in the array : the tubular surface, then the two closing faces.
 * * Example how to set colors and textures on a sliced cylinder : https://www.html5gamedevs.com/topic/17945-creating-a-closed-slice-of-a-cylinder/#comment-106379
 * * You can also set the mesh side orientation with the values : BABYLON.Mesh.FRONTSIDE (default), BABYLON.Mesh.BACKSIDE or BABYLON.Mesh.DOUBLESIDE
 * * If you create a double-sided mesh, you can choose what parts of the texture image to crop and stick respectively on the front and the back sides with the parameters `frontUVs` and `backUVs` (Vector4). Detail here : https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#side-orientation
 * * The mesh can be set to updatable with the boolean parameter `updatable` (default false) if its internal geometry is supposed to change once created.
 * @param name defines the name of the mesh
 * @param options defines the options used to create the mesh
 * @param options.height
 * @param options.diameterTop
 * @param options.diameterBottom
 * @param options.diameter
 * @param options.tessellation
 * @param options.subdivisions
 * @param options.arc
 * @param options.faceColors
 * @param options.faceUV
 * @param options.updatable
 * @param options.hasRings
 * @param options.enclose
 * @param options.cap
 * @param options.sideOrientation
 * @param options.frontUVs
 * @param options.backUVs
 * @param scene defines the hosting scene
 * @returns the cylinder mesh
 * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#cylinder-or-cone
 */
export function CreateCylinder(name: string, options?: {
    height?: number;
    diameterTop?: number;
    diameterBottom?: number;
    diameter?: number;
    tessellation?: number;
    subdivisions?: number;
    arc?: number;
    faceColors?: Color4[];
    faceUV?: Vector4[];
    updatable?: boolean;
    hasRings?: boolean;
    enclose?: boolean;
    cap?: number;
    sideOrientation?: number;
    frontUVs?: Vector4;
    backUVs?: Vector4;
}, scene?: Nullable<Scene>): Mesh;
/**
 * Class containing static functions to help procedurally build meshes
 * @deprecated Please use CreateCylinder directly
 */
export const CylinderBuilder: {
    CreateCylinder: typeof CreateCylinder;
};

}
declare module "babylonjs/Meshes/Builders/decalBuilder" {
import { Vector3 } from "babylonjs/Maths/math.vector";
import { Mesh } from "babylonjs/Meshes/mesh";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
/**
 * Creates a decal mesh.
 * A decal is a mesh usually applied as a model onto the surface of another mesh. So don't forget the parameter `sourceMesh` depicting the decal
 * * The parameter `position` (Vector3, default `(0, 0, 0)`) sets the position of the decal in World coordinates
 * * The parameter `normal` (Vector3, default `Vector3.Up`) sets the normal of the mesh where the decal is applied onto in World coordinates
 * * The parameter `size` (Vector3, default `(1, 1, 1)`) sets the decal scaling
 * * The parameter `angle` (float in radian, default 0) sets the angle to rotate the decal
 * * The parameter `captureUVS` defines if we need to capture the uvs or compute them
 * * The parameter `cullBackFaces` defines if the back faces should be removed from the decal mesh
 * * The parameter `localMode` defines that the computations should be done with the local mesh coordinates instead of the world space coordinates.
 * *    Use this mode if you want the decal to be parented to the sourceMesh and move/rotate with it.
 * Note: Meshes with morph targets are not supported!
 * @param name defines the name of the mesh
 * @param sourceMesh defines the mesh where the decal must be applied
 * @param options defines the options used to create the mesh
 * @param options.position
 * @param options.normal
 * @param options.size
 * @param options.angle
 * @param options.captureUVS
 * @param options.cullBackFaces
 * @param options.localMode
 * @returns the decal mesh
 * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/decals
 */
export function CreateDecal(name: string, sourceMesh: AbstractMesh, options: {
    position?: Vector3;
    normal?: Vector3;
    size?: Vector3;
    angle?: number;
    captureUVS?: boolean;
    cullBackFaces?: boolean;
    localMode?: boolean;
}): Mesh;
/**
 * Class containing static functions to help procedurally build meshes
 * @deprecated use the function directly from the module
 */
export const DecalBuilder: {
    CreateDecal: typeof CreateDecal;
};

}
declare module "babylonjs/Meshes/Builders/discBuilder" {
import { Nullable } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
import { Vector4 } from "babylonjs/Maths/math.vector";
import { Mesh } from "babylonjs/Meshes/mesh";
import { VertexData } from "babylonjs/Meshes/mesh.vertexData";
/**
 * Creates the VertexData of the Disc or regular Polygon
 * @param options an object used to set the following optional parameters for the disc, required but can be empty
 * * radius the radius of the disc, optional default 0.5
 * * tessellation the number of polygon sides, optional, default 64
 * * arc a number from 0 to 1, to create an unclosed polygon based on the fraction of the circumference given by the arc value, optional, default 1
 * * sideOrientation optional and takes the values : Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
 * * frontUvs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the front side, optional, default vector4 (0, 0, 1, 1)
 * * backUVs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the back side, optional, default vector4 (0, 0, 1, 1)
 * @param options.radius
 * @param options.tessellation
 * @param options.arc
 * @param options.sideOrientation
 * @param options.frontUVs
 * @param options.backUVs
 * @returns the VertexData of the box
 */
export function CreateDiscVertexData(options: {
    radius?: number;
    tessellation?: number;
    arc?: number;
    sideOrientation?: number;
    frontUVs?: Vector4;
    backUVs?: Vector4;
}): VertexData;
/**
 * Creates a plane polygonal mesh.  By default, this is a disc
 * * The parameter `radius` sets the radius size (float) of the polygon (default 0.5)
 * * The parameter `tessellation` sets the number of polygon sides (positive integer, default 64). So a tessellation valued to 3 will build a triangle, to 4 a square, etc
 * * You can create an unclosed polygon with the parameter `arc` (positive float, default 1), valued between 0 and 1, what is the ratio of the circumference : 2 x PI x ratio
 * * You can also set the mesh side orientation with the values : BABYLON.Mesh.FRONTSIDE (default), BABYLON.Mesh.BACKSIDE or BABYLON.Mesh.DOUBLESIDE
 * * If you create a double-sided mesh, you can choose what parts of the texture image to crop and stick respectively on the front and the back sides with the parameters `frontUVs` and `backUVs` (Vector4). Detail here : https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#side-orientation
 * * The mesh can be set to updatable with the boolean parameter `updatable` (default false) if its internal geometry is supposed to change once created
 * @param name defines the name of the mesh
 * @param options defines the options used to create the mesh
 * @param options.radius
 * @param options.tessellation
 * @param options.arc
 * @param options.updatable
 * @param options.sideOrientation
 * @param options.frontUVs
 * @param options.backUVs
 * @param scene defines the hosting scene
 * @returns the plane polygonal mesh
 * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#disc-or-regular-polygon
 */
export function CreateDisc(name: string, options?: {
    radius?: number;
    tessellation?: number;
    arc?: number;
    updatable?: boolean;
    sideOrientation?: number;
    frontUVs?: Vector4;
    backUVs?: Vector4;
}, scene?: Nullable<Scene>): Mesh;
/**
 * Class containing static functions to help procedurally build meshes
 * @deprecated please use CreateDisc directly
 */
export const DiscBuilder: {
    CreateDisc: typeof CreateDisc;
};

}
declare module "babylonjs/Meshes/Builders/geodesicBuilder" {
import { Scene } from "babylonjs/scene";
import { Vector4 } from "babylonjs/Maths/math.vector";
import { Color4 } from "babylonjs/Maths/math.color";
import { Mesh } from "babylonjs/Meshes/mesh";
import { Nullable } from "babylonjs/types";
/**
 * Creates the Mesh for a Geodesic Polyhedron
 * @see https://en.wikipedia.org/wiki/Geodesic_polyhedron
 * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/polyhedra/geodesic_poly
 * @param name defines the name of the mesh
 * @param options an object used to set the following optional parameters for the polyhedron, required but can be empty
 * * m number of horizontal steps along an isogrid
 * * n number of angled steps along an isogrid
 * * size the size of the Geodesic, optional default 1
 * * sizeX allows stretching in the x direction, optional, default size
 * * sizeY allows stretching in the y direction, optional, default size
 * * sizeZ allows stretching in the z direction, optional, default size
 * * faceUV an array of Vector4 elements used to set different images to the top, rings and bottom respectively
 * * faceColors an array of Color3 elements used to set different colors to the top, rings and bottom respectively
 * * flat when true creates a flat shaded mesh, optional, default true
 * * subdivisions increasing the subdivisions increases the number of faces, optional, default 4
 * * sideOrientation optional and takes the values : Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
 * * frontUvs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the front side, optional, default vector4 (0, 0, 1, 1)
 * * backUVs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the back side, optional, default vector4 (0, 0, 1, 1)
 * @param options.n
 * @param options.size
 * @param options.sizeX
 * @param options.sizeY
 * @param options.sizeZ
 * @param options.faceUV
 * @param options.faceColors
 * @param options.flat
 * @param options.updatable
 * @param options.sideOrientation
 * @param options.frontUVs
 * @param options.backUVs
 * @param options.m
 * @param scene defines the hosting scene
 * @returns Geodesic mesh
 */
export function CreateGeodesic(name: string, options: {
    m?: number;
    n?: number;
    size?: number;
    sizeX?: number;
    sizeY?: number;
    sizeZ?: number;
    faceUV?: Vector4[];
    faceColors?: Color4[];
    flat?: boolean;
    updatable?: boolean;
    sideOrientation?: number;
    frontUVs?: Vector4;
    backUVs?: Vector4;
}, scene?: Nullable<Scene>): Mesh;

}
declare module "babylonjs/Meshes/Builders/goldbergBuilder" {
import { Scene } from "babylonjs/scene";
import { VertexData } from "babylonjs/Meshes/mesh.vertexData";
import { Nullable } from "babylonjs/types";
import { PolyhedronData } from "babylonjs/Meshes/geodesicMesh";
import { GoldbergMesh } from "babylonjs/Meshes/goldbergMesh";
/**
 * Defines the set of data required to create goldberg vertex data.
 */
export type GoldbergVertexDataOption = {
    /**
     * the size of the Goldberg, optional default 1
     */
    size?: number;
    /**
     * allows stretching in the x direction, optional, default size
     */
    sizeX?: number;
    /**
     * allows stretching in the y direction, optional, default size
     */
    sizeY?: number;
    /**
     * allows stretching in the z direction, optional, default size
     */
    sizeZ?: number;
    /**
     * optional and takes the values : Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
     */
    sideOrientation?: number;
};
/**
 * Defines the set of data required to create a goldberg mesh.
 */
export type GoldbergCreationOption = {
    /**
     * number of horizontal steps along an isogrid
     */
    m?: number;
    /**
     * number of angled steps along an isogrid
     */
    n?: number;
    /**
     * defines if the mesh must be flagged as updatable
     */
    updatable?: boolean;
} & GoldbergVertexDataOption;
/**
 * Creates the Mesh for a Goldberg Polyhedron
 * @param options an object used to set the following optional parameters for the polyhedron, required but can be empty
 * @param goldbergData polyhedronData defining the Goldberg polyhedron
 * @returns GoldbergSphere mesh
 */
export function CreateGoldbergVertexData(options: GoldbergVertexDataOption, goldbergData: PolyhedronData): VertexData;
/**
 * Creates the Mesh for a Goldberg Polyhedron which is made from 12 pentagonal and the rest hexagonal faces
 * @see https://en.wikipedia.org/wiki/Goldberg_polyhedron
 * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/polyhedra/goldberg_poly
 * @param name defines the name of the mesh
 * @param options an object used to set the following optional parameters for the polyhedron, required but can be empty
 * @param scene defines the hosting scene
 * @returns Goldberg mesh
 */
export function CreateGoldberg(name: string, options: GoldbergCreationOption, scene?: Nullable<Scene>): GoldbergMesh;

}
declare module "babylonjs/Meshes/Builders/groundBuilder" {
import { Scene } from "babylonjs/scene";
import { Color3 } from "babylonjs/Maths/math.color";
import { Mesh } from "babylonjs/Meshes/mesh";
import { VertexData } from "babylonjs/Meshes/mesh.vertexData";
import { GroundMesh } from "babylonjs/Meshes/groundMesh";
import { Nullable } from "babylonjs/types";
/**
 * Creates the VertexData for a Ground
 * @param options an object used to set the following optional parameters for the Ground, required but can be empty
 *  - width the width (x direction) of the ground, optional, default 1
 *  - height the height (z direction) of the ground, optional, default 1
 *  - subdivisions the number of subdivisions per side, optional, default 1
 * @param options.width
 * @param options.height
 * @param options.subdivisions
 * @param options.subdivisionsX
 * @param options.subdivisionsY
 * @returns the VertexData of the Ground
 */
export function CreateGroundVertexData(options: {
    width?: number;
    height?: number;
    subdivisions?: number;
    subdivisionsX?: number;
    subdivisionsY?: number;
}): VertexData;
/**
 * Creates the VertexData for a TiledGround by subdividing the ground into tiles
 * @param options an object used to set the following optional parameters for the Ground, required but can be empty
 * * xmin the ground minimum X coordinate, optional, default -1
 * * zmin the ground minimum Z coordinate, optional, default -1
 * * xmax the ground maximum X coordinate, optional, default 1
 * * zmax the ground maximum Z coordinate, optional, default 1
 * * subdivisions a javascript object {w: positive integer, h: positive integer}, `w` and `h` are the numbers of subdivisions on the ground width and height creating 'tiles', default {w: 6, h: 6}
 * * precision a javascript object {w: positive integer, h: positive integer}, `w` and `h` are the numbers of subdivisions on the tile width and height, default {w: 2, h: 2}
 * @param options.xmin
 * @param options.zmin
 * @param options.xmax
 * @param options.zmax
 * @param options.subdivisions
 * @param options.subdivisions.w
 * @param options.subdivisions.h
 * @param options.precision
 * @param options.precision.w
 * @param options.precision.h
 * @returns the VertexData of the TiledGround
 */
export function CreateTiledGroundVertexData(options: {
    xmin: number;
    zmin: number;
    xmax: number;
    zmax: number;
    subdivisions?: {
        w: number;
        h: number;
    };
    precision?: {
        w: number;
        h: number;
    };
}): VertexData;
/**
 * Creates the VertexData of the Ground designed from a heightmap
 * @param options an object used to set the following parameters for the Ground, required and provided by CreateGroundFromHeightMap
 * * width the width (x direction) of the ground
 * * height the height (z direction) of the ground
 * * subdivisions the number of subdivisions per side
 * * minHeight the minimum altitude on the ground, optional, default 0
 * * maxHeight the maximum altitude on the ground, optional default 1
 * * colorFilter the filter to apply to the image pixel colors to compute the height, optional Color3, default (0.3, 0.59, 0.11)
 * * buffer the array holding the image color data
 * * bufferWidth the width of image
 * * bufferHeight the height of image
 * * alphaFilter Remove any data where the alpha channel is below this value, defaults 0 (all data visible)
 * @param options.width
 * @param options.height
 * @param options.subdivisions
 * @param options.minHeight
 * @param options.maxHeight
 * @param options.colorFilter
 * @param options.buffer
 * @param options.bufferWidth
 * @param options.bufferHeight
 * @param options.alphaFilter
 * @returns the VertexData of the Ground designed from a heightmap
 */
export function CreateGroundFromHeightMapVertexData(options: {
    width: number;
    height: number;
    subdivisions: number;
    minHeight: number;
    maxHeight: number;
    colorFilter: Color3;
    buffer: Uint8Array;
    bufferWidth: number;
    bufferHeight: number;
    alphaFilter: number;
}): VertexData;
/**
 * Creates a ground mesh
 * * The parameters `width` and `height` (floats, default 1) set the width and height sizes of the ground
 * * The parameter `subdivisions` (positive integer) sets the number of subdivisions per side
 * * The mesh can be set to updatable with the boolean parameter `updatable` (default false) if its internal geometry is supposed to change once created
 * @param name defines the name of the mesh
 * @param options defines the options used to create the mesh
 * @param options.width
 * @param options.height
 * @param options.subdivisions
 * @param options.subdivisionsX
 * @param options.subdivisionsY
 * @param options.updatable
 * @param scene defines the hosting scene
 * @returns the ground mesh
 * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#ground
 */
export function CreateGround(name: string, options?: {
    width?: number;
    height?: number;
    subdivisions?: number;
    subdivisionsX?: number;
    subdivisionsY?: number;
    updatable?: boolean;
}, scene?: Scene): GroundMesh;
/**
 * Creates a tiled ground mesh
 * * The parameters `xmin` and `xmax` (floats, default -1 and 1) set the ground minimum and maximum X coordinates
 * * The parameters `zmin` and `zmax` (floats, default -1 and 1) set the ground minimum and maximum Z coordinates
 * * The parameter `subdivisions` is a javascript object `{w: positive integer, h: positive integer}` (default `{w: 6, h: 6}`). `w` and `h` are the numbers of subdivisions on the ground width and height. Each subdivision is called a tile
 * * The parameter `precision` is a javascript object `{w: positive integer, h: positive integer}` (default `{w: 2, h: 2}`). `w` and `h` are the numbers of subdivisions on the ground width and height of each tile
 * * The mesh can be set to updatable with the boolean parameter `updatable` (default false) if its internal geometry is supposed to change once created.
 * @param name defines the name of the mesh
 * @param options defines the options used to create the mesh
 * @param options.xmin
 * @param options.zmin
 * @param options.xmax
 * @param options.zmax
 * @param options.subdivisions
 * @param options.subdivisions.w
 * @param options.subdivisions.h
 * @param options.precision
 * @param options.precision.w
 * @param options.precision.h
 * @param options.updatable
 * @param scene defines the hosting scene
 * @returns the tiled ground mesh
 * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#tiled-ground
 */
export function CreateTiledGround(name: string, options: {
    xmin: number;
    zmin: number;
    xmax: number;
    zmax: number;
    subdivisions?: {
        w: number;
        h: number;
    };
    precision?: {
        w: number;
        h: number;
    };
    updatable?: boolean;
}, scene?: Nullable<Scene>): Mesh;
/**
 * Creates a ground mesh from a height map
 * * The parameter `url` sets the URL of the height map image resource.
 * * The parameters `width` and `height` (positive floats, default 10) set the ground width and height sizes.
 * * The parameter `subdivisions` (positive integer, default 1) sets the number of subdivision per side.
 * * The parameter `minHeight` (float, default 0) is the minimum altitude on the ground.
 * * The parameter `maxHeight` (float, default 1) is the maximum altitude on the ground.
 * * The parameter `colorFilter` (optional Color3, default (0.3, 0.59, 0.11) ) is the filter to apply to the image pixel colors to compute the height.
 * * The parameter `onReady` is a javascript callback function that will be called  once the mesh is just built (the height map download can last some time).
 * * The parameter `alphaFilter` will filter any data where the alpha channel is below this value, defaults 0 (all data visible)
 * * The mesh can be set to updatable with the boolean parameter `updatable` (default false) if its internal geometry is supposed to change once created.
 * @param name defines the name of the mesh
 * @param url defines the url to the height map
 * @param options defines the options used to create the mesh
 * @param options.width
 * @param options.height
 * @param options.subdivisions
 * @param options.minHeight
 * @param options.maxHeight
 * @param options.colorFilter
 * @param options.alphaFilter
 * @param options.updatable
 * @param options.onReady
 * @param scene defines the hosting scene
 * @returns the ground mesh
 * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set/height_map
 * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#ground-from-a-height-map
 */
export function CreateGroundFromHeightMap(name: string, url: string, options?: {
    width?: number;
    height?: number;
    subdivisions?: number;
    minHeight?: number;
    maxHeight?: number;
    colorFilter?: Color3;
    alphaFilter?: number;
    updatable?: boolean;
    onReady?: (mesh: GroundMesh) => void;
}, scene?: Nullable<Scene>): GroundMesh;
/**
 * Class containing static functions to help procedurally build meshes
 * @deprecated use the functions directly from the module
 */
export const GroundBuilder: {
    CreateGround: typeof CreateGround;
    CreateGroundFromHeightMap: typeof CreateGroundFromHeightMap;
    CreateTiledGround: typeof CreateTiledGround;
};

}
declare module "babylonjs/Meshes/Builders/hemisphereBuilder" {
import { Mesh } from "babylonjs/Meshes/mesh";
import { Scene } from "babylonjs/scene";
/**
 * Creates a hemisphere mesh
 * @param name defines the name of the mesh
 * @param options defines the options used to create the mesh
 * @param options.segments
 * @param options.diameter
 * @param options.sideOrientation
 * @param scene defines the hosting scene
 * @returns the hemisphere mesh
 */
export function CreateHemisphere(name: string, options?: {
    segments?: number;
    diameter?: number;
    sideOrientation?: number;
}, scene?: Scene): Mesh;
/**
 * Class containing static functions to help procedurally build meshes
 * @deprecated use the function directly from the module
 */
export const HemisphereBuilder: {
    CreateHemisphere: typeof CreateHemisphere;
};

}
declare module "babylonjs/Meshes/Builders/icoSphereBuilder" {
import { Scene } from "babylonjs/scene";
import { Vector4 } from "babylonjs/Maths/math.vector";
import { Mesh } from "babylonjs/Meshes/mesh";
import { VertexData } from "babylonjs/Meshes/mesh.vertexData";
import { Nullable } from "babylonjs/types";
/**
 * Creates the VertexData of the IcoSphere
 * @param options an object used to set the following optional parameters for the IcoSphere, required but can be empty
 * * radius the radius of the IcoSphere, optional default 1
 * * radiusX allows stretching in the x direction, optional, default radius
 * * radiusY allows stretching in the y direction, optional, default radius
 * * radiusZ allows stretching in the z direction, optional, default radius
 * * flat when true creates a flat shaded mesh, optional, default true
 * * subdivisions increasing the subdivisions increases the number of faces, optional, default 4
 * * sideOrientation optional and takes the values : Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
 * * frontUvs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the front side, optional, default vector4 (0, 0, 1, 1)
 * * backUVs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the back side, optional, default vector4 (0, 0, 1, 1)
 * @param options.radius
 * @param options.radiusX
 * @param options.radiusY
 * @param options.radiusZ
 * @param options.flat
 * @param options.subdivisions
 * @param options.sideOrientation
 * @param options.frontUVs
 * @param options.backUVs
 * @returns the VertexData of the IcoSphere
 */
export function CreateIcoSphereVertexData(options: {
    radius?: number;
    radiusX?: number;
    radiusY?: number;
    radiusZ?: number;
    flat?: boolean;
    subdivisions?: number;
    sideOrientation?: number;
    frontUVs?: Vector4;
    backUVs?: Vector4;
}): VertexData;
/**
 * Creates a sphere based upon an icosahedron with 20 triangular faces which can be subdivided
 * * The parameter `radius` sets the radius size (float) of the icosphere (default 1)
 * * You can set some different icosphere dimensions, for instance to build an ellipsoid, by using the parameters `radiusX`, `radiusY` and `radiusZ` (all by default have the same value of `radius`)
 * * The parameter `subdivisions` sets the number of subdivisions (positive integer, default 4). The more subdivisions, the more faces on the icosphere whatever its size
 * * The parameter `flat` (boolean, default true) gives each side its own normals. Set it to false to get a smooth continuous light reflection on the surface
 * * You can also set the mesh side orientation with the values : BABYLON.Mesh.FRONTSIDE (default), BABYLON.Mesh.BACKSIDE or BABYLON.Mesh.DOUBLESIDE
 * * If you create a double-sided mesh, you can choose what parts of the texture image to crop and stick respectively on the front and the back sides with the parameters `frontUVs` and `backUVs` (Vector4). Detail here : https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#side-orientation
 * * The mesh can be set to updatable with the boolean parameter `updatable` (default false) if its internal geometry is supposed to change once created
 * @param name defines the name of the mesh
 * @param options defines the options used to create the mesh
 * @param options.radius
 * @param options.radiusX
 * @param options.radiusY
 * @param options.radiusZ
 * @param options.flat
 * @param options.subdivisions
 * @param options.sideOrientation
 * @param options.frontUVs
 * @param options.backUVs
 * @param options.updatable
 * @param scene defines the hosting scene
 * @returns the icosahedron mesh
 * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/polyhedra#icosphere
 */
export function CreateIcoSphere(name: string, options?: {
    radius?: number;
    radiusX?: number;
    radiusY?: number;
    radiusZ?: number;
    flat?: boolean;
    subdivisions?: number;
    sideOrientation?: number;
    frontUVs?: Vector4;
    backUVs?: Vector4;
    updatable?: boolean;
}, scene?: Nullable<Scene>): Mesh;
/**
 * Class containing static functions to help procedurally build meshes
 * @deprecated use the function directly from the module
 */
export const IcoSphereBuilder: {
    CreateIcoSphere: typeof CreateIcoSphere;
};

}
declare module "babylonjs/Meshes/Builders/index" {
export * from "babylonjs/Meshes/Builders/boxBuilder";
export * from "babylonjs/Meshes/Builders/tiledBoxBuilder";
export * from "babylonjs/Meshes/Builders/discBuilder";
export * from "babylonjs/Meshes/Builders/ribbonBuilder";
export * from "babylonjs/Meshes/Builders/sphereBuilder";
export * from "babylonjs/Meshes/Builders/hemisphereBuilder";
export * from "babylonjs/Meshes/Builders/cylinderBuilder";
export * from "babylonjs/Meshes/Builders/torusBuilder";
export * from "babylonjs/Meshes/Builders/torusKnotBuilder";
export * from "babylonjs/Meshes/Builders/linesBuilder";
export * from "babylonjs/Meshes/Builders/polygonBuilder";
export * from "babylonjs/Meshes/Builders/shapeBuilder";
export * from "babylonjs/Meshes/Builders/latheBuilder";
export * from "babylonjs/Meshes/Builders/planeBuilder";
export * from "babylonjs/Meshes/Builders/tiledPlaneBuilder";
export * from "babylonjs/Meshes/Builders/groundBuilder";
export * from "babylonjs/Meshes/Builders/tubeBuilder";
export * from "babylonjs/Meshes/Builders/polyhedronBuilder";
export * from "babylonjs/Meshes/Builders/geodesicBuilder";
export * from "babylonjs/Meshes/Builders/goldbergBuilder";
export * from "babylonjs/Meshes/Builders/decalBuilder";
export * from "babylonjs/Meshes/Builders/icoSphereBuilder";
export * from "babylonjs/Meshes/Builders/capsuleBuilder";

}
declare module "babylonjs/Meshes/Builders/latheBuilder" {
import { Scene } from "babylonjs/scene";
import { Vector4 } from "babylonjs/Maths/math.vector";
import { Vector3 } from "babylonjs/Maths/math.vector";
import { Mesh } from "babylonjs/Meshes/mesh";
import { Nullable } from "babylonjs/types";
/**
 * Creates lathe mesh.
 * The lathe is a shape with a symmetry axis : a 2D model shape is rotated around this axis to design the lathe
 * * The parameter `shape` is a required array of successive Vector3. This array depicts the shape to be rotated in its local space : the shape must be designed in the xOy plane and will be rotated around the Y axis. It's usually a 2D shape, so the Vector3 z coordinates are often set to zero
 * * The parameter `radius` (positive float, default 1) is the radius value of the lathe
 * * The parameter `tessellation` (positive integer, default 64) is the side number of the lathe
 * * The parameter `clip` (positive integer, default 0) is the number of sides to not create without effecting the general shape of the sides
 * * The parameter `arc` (positive float, default 1) is the ratio of the lathe. 0.5 builds for instance half a lathe, so an opened shape
 * * The parameter `closed` (boolean, default true) opens/closes the lathe circumference. This should be set to false when used with the parameter "arc"
 * * The parameter `cap` sets the way the extruded shape is capped. Possible values : BABYLON.Mesh.NO_CAP (default), BABYLON.Mesh.CAP_START, BABYLON.Mesh.CAP_END, BABYLON.Mesh.CAP_ALL
 * * You can also set the mesh side orientation with the values : BABYLON.Mesh.FRONTSIDE (default), BABYLON.Mesh.BACKSIDE or BABYLON.Mesh.DOUBLESIDE
 * * If you create a double-sided mesh, you can choose what parts of the texture image to crop and stick respectively on the front and the back sides with the parameters `frontUVs` and `backUVs` (Vector4). Detail here : https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#side-orientation
 * * The optional parameter `invertUV` (boolean, default false) swaps in the geometry the U and V coordinates to apply a texture
 * * The mesh can be set to updatable with the boolean parameter `updatable` (default false) if its internal geometry is supposed to change once created
 * @param name defines the name of the mesh
 * @param options defines the options used to create the mesh
 * @param options.shape
 * @param options.radius
 * @param options.tessellation
 * @param options.clip
 * @param options.arc
 * @param options.closed
 * @param options.updatable
 * @param options.sideOrientation
 * @param options.frontUVs
 * @param options.backUVs
 * @param options.cap
 * @param options.invertUV
 * @param scene defines the hosting scene
 * @returns the lathe mesh
 * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/param#lathe
 */
export function CreateLathe(name: string, options: {
    shape: Vector3[];
    radius?: number;
    tessellation?: number;
    clip?: number;
    arc?: number;
    closed?: boolean;
    updatable?: boolean;
    sideOrientation?: number;
    frontUVs?: Vector4;
    backUVs?: Vector4;
    cap?: number;
    invertUV?: boolean;
}, scene?: Nullable<Scene>): Mesh;
/**
 * Class containing static functions to help procedurally build meshes
 * @deprecated use the function direction from the module
 */
export const LatheBuilder: {
    CreateLathe: typeof CreateLathe;
};

}
declare module "babylonjs/Meshes/Builders/linesBuilder" {
import { Vector3 } from "babylonjs/Maths/math.vector";
import { Color4 } from "babylonjs/Maths/math.color";
import { VertexData } from "babylonjs/Meshes/mesh.vertexData";
import { Nullable } from "babylonjs/types";
import { LinesMesh } from "babylonjs/Meshes/linesMesh";
import { Scene } from "babylonjs/scene";
import { Material } from "babylonjs/Materials/material";
/**
 * Creates the VertexData of the LineSystem
 * @param options an object used to set the following optional parameters for the LineSystem, required but can be empty
 *  - lines an array of lines, each line being an array of successive Vector3
 *  - colors an array of line colors, each of the line colors being an array of successive Color4, one per line point
 * @param options.lines
 * @param options.colors
 * @returns the VertexData of the LineSystem
 */
export function CreateLineSystemVertexData(options: {
    lines: Vector3[][];
    colors?: Nullable<Color4[][]>;
}): VertexData;
/**
 * Create the VertexData for a DashedLines
 * @param options an object used to set the following optional parameters for the DashedLines, required but can be empty
 *  - points an array successive Vector3
 *  - dashSize the size of the dashes relative to the dash number, optional, default 3
 *  - gapSize the size of the gap between two successive dashes relative to the dash number, optional, default 1
 *  - dashNb the intended total number of dashes, optional, default 200
 * @param options.points
 * @param options.dashSize
 * @param options.gapSize
 * @param options.dashNb
 * @returns the VertexData for the DashedLines
 */
export function CreateDashedLinesVertexData(options: {
    points: Vector3[];
    dashSize?: number;
    gapSize?: number;
    dashNb?: number;
}): VertexData;
/**
 * Creates a line system mesh. A line system is a pool of many lines gathered in a single mesh
 * * A line system mesh is considered as a parametric shape since it has no predefined original shape. Its shape is determined by the passed array of lines as an input parameter
 * * Like every other parametric shape, it is dynamically updatable by passing an existing instance of LineSystem to this static function
 * * The parameter `lines` is an array of lines, each line being an array of successive Vector3
 * * The optional parameter `instance` is an instance of an existing LineSystem object to be updated with the passed `lines` parameter
 * * The optional parameter `colors` is an array of line colors, each line colors being an array of successive Color4, one per line point
 * * The optional parameter `useVertexAlpha` is to be set to `false` (default `true`) when you don't need the alpha blending (faster)
 * * The optional parameter `material` is the material to use to draw the lines if provided. If not, a default material will be created
 * * Updating a simple Line mesh, you just need to update every line in the `lines` array : https://doc.babylonjs.com/features/featuresDeepDive/mesh/dynamicMeshMorph#lines-and-dashedlines
 * * When updating an instance, remember that only line point positions can change, not the number of points, neither the number of lines
 * * The mesh can be set to updatable with the boolean parameter `updatable` (default false) if its internal geometry is supposed to change once created
 * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/param#line-system
 * @param name defines the name of the new line system
 * @param options defines the options used to create the line system
 * @param options.lines
 * @param options.updatable
 * @param options.instance
 * @param options.colors
 * @param options.useVertexAlpha
 * @param options.material
 * @param scene defines the hosting scene
 * @returns a new line system mesh
 */
export function CreateLineSystem(name: string, options: {
    lines: Vector3[][];
    updatable?: boolean;
    instance?: Nullable<LinesMesh>;
    colors?: Nullable<Color4[][]>;
    useVertexAlpha?: boolean;
    material?: Material;
}, scene: Nullable<Scene>): LinesMesh;
/**
 * Creates a line mesh
 * A line mesh is considered as a parametric shape since it has no predefined original shape. Its shape is determined by the passed array of points as an input parameter
 * * Like every other parametric shape, it is dynamically updatable by passing an existing instance of LineMesh to this static function
 * * The parameter `points` is an array successive Vector3
 * * The optional parameter `instance` is an instance of an existing LineMesh object to be updated with the passed `points` parameter : https://doc.babylonjs.com/features/featuresDeepDive/mesh/dynamicMeshMorph#lines-and-dashedlines
 * * The optional parameter `colors` is an array of successive Color4, one per line point
 * * The optional parameter `useVertexAlpha` is to be set to `false` (default `true`) when you don't need alpha blending (faster)
 * * The optional parameter `material` is the material to use to draw the lines if provided. If not, a default material will be created
 * * When updating an instance, remember that only point positions can change, not the number of points
 * * The mesh can be set to updatable with the boolean parameter `updatable` (default false) if its internal geometry is supposed to change once created
 * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/param#lines
 * @param name defines the name of the new line system
 * @param options defines the options used to create the line system
 * @param options.points
 * @param options.updatable
 * @param options.instance
 * @param options.colors
 * @param options.useVertexAlpha
 * @param options.material
 * @param scene defines the hosting scene
 * @returns a new line mesh
 */
export function CreateLines(name: string, options: {
    points: Vector3[];
    updatable?: boolean;
    instance?: Nullable<LinesMesh>;
    colors?: Color4[];
    useVertexAlpha?: boolean;
    material?: Material;
}, scene?: Nullable<Scene>): LinesMesh;
/**
 * Creates a dashed line mesh
 * * A dashed line mesh is considered as a parametric shape since it has no predefined original shape. Its shape is determined by the passed array of points as an input parameter
 * * Like every other parametric shape, it is dynamically updatable by passing an existing instance of LineMesh to this static function
 * * The parameter `points` is an array successive Vector3
 * * The parameter `dashNb` is the intended total number of dashes (positive integer, default 200)
 * * The parameter `dashSize` is the size of the dashes relatively the dash number (positive float, default 3)
 * * The parameter `gapSize` is the size of the gap between two successive dashes relatively the dash number (positive float, default 1)
 * * The optional parameter `instance` is an instance of an existing LineMesh object to be updated with the passed `points` parameter : https://doc.babylonjs.com/features/featuresDeepDive/mesh/dynamicMeshMorph#lines-and-dashedlines
 * * The optional parameter `useVertexAlpha` is to be set to `false` (default `true`) when you don't need the alpha blending (faster)
 * * The optional parameter `material` is the material to use to draw the lines if provided. If not, a default material will be created
 * * When updating an instance, remember that only point positions can change, not the number of points
 * * The mesh can be set to updatable with the boolean parameter `updatable` (default false) if its internal geometry is supposed to change once created
 * @param name defines the name of the mesh
 * @param options defines the options used to create the mesh
 * @param options.points
 * @param options.dashSize
 * @param options.gapSize
 * @param options.dashNb
 * @param options.updatable
 * @param options.instance
 * @param options.useVertexAlpha
 * @param options.material
 * @param scene defines the hosting scene
 * @returns the dashed line mesh
 * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/param#dashed-lines
 */
export function CreateDashedLines(name: string, options: {
    points: Vector3[];
    dashSize?: number;
    gapSize?: number;
    dashNb?: number;
    updatable?: boolean;
    instance?: LinesMesh;
    useVertexAlpha?: boolean;
    material?: Material;
}, scene?: Nullable<Scene>): LinesMesh;
/**
 * Class containing static functions to help procedurally build meshes
 * @deprecated use the functions directly from the module
 */
export const LinesBuilder: {
    CreateDashedLines: typeof CreateDashedLines;
    CreateLineSystem: typeof CreateLineSystem;
    CreateLines: typeof CreateLines;
};
export {};

}
declare module "babylonjs/Meshes/Builders/planeBuilder" {
import { Scene } from "babylonjs/scene";
import { Vector4 } from "babylonjs/Maths/math.vector";
import { Mesh } from "babylonjs/Meshes/mesh";
import { VertexData } from "babylonjs/Meshes/mesh.vertexData";
import { Nullable } from "babylonjs/types";
import { Plane } from "babylonjs/Maths/math.plane";
/**
 * Creates the VertexData for a Plane
 * @param options an object used to set the following optional parameters for the plane, required but can be empty
 * * size sets the width and height of the plane to the value of size, optional default 1
 * * width sets the width (x direction) of the plane, overwrites the width set by size, optional, default size
 * * height sets the height (y direction) of the plane, overwrites the height set by size, optional, default size
 * * sideOrientation optional and takes the values : Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
 * * frontUvs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the front side, optional, default vector4 (0, 0, 1, 1)
 * * backUVs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the back side, optional, default vector4 (0, 0, 1, 1)
 * @param options.size
 * @param options.width
 * @param options.height
 * @param options.sideOrientation
 * @param options.frontUVs
 * @param options.backUVs
 * @returns the VertexData of the box
 */
export function CreatePlaneVertexData(options: {
    size?: number;
    width?: number;
    height?: number;
    sideOrientation?: number;
    frontUVs?: Vector4;
    backUVs?: Vector4;
}): VertexData;
/**
 * Creates a plane mesh
 * * The parameter `size` sets the size (float) of both sides of the plane at once (default 1)
 * * You can set some different plane dimensions by using the parameters `width` and `height` (both by default have the same value of `size`)
 * * The parameter `sourcePlane` is a Plane instance. It builds a mesh plane from a Math plane
 * * You can also set the mesh side orientation with the values : BABYLON.Mesh.FRONTSIDE (default), BABYLON.Mesh.BACKSIDE or BABYLON.Mesh.DOUBLESIDE
 * * If you create a double-sided mesh, you can choose what parts of the texture image to crop and stick respectively on the front and the back sides with the parameters `frontUVs` and `backUVs` (Vector4). Detail here : https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#side-orientation
 * * The mesh can be set to updatable with the boolean parameter `updatable` (default false) if its internal geometry is supposed to change once created
 * @param name defines the name of the mesh
 * @param options defines the options used to create the mesh
 * @param options.size
 * @param options.width
 * @param options.height
 * @param options.sideOrientation
 * @param options.frontUVs
 * @param options.backUVs
 * @param options.updatable
 * @param options.sourcePlane
 * @param scene defines the hosting scene
 * @returns the plane mesh
 * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#plane
 */
export function CreatePlane(name: string, options?: {
    size?: number;
    width?: number;
    height?: number;
    sideOrientation?: number;
    frontUVs?: Vector4;
    backUVs?: Vector4;
    updatable?: boolean;
    sourcePlane?: Plane;
}, scene?: Nullable<Scene>): Mesh;
/**
 * Class containing static functions to help procedurally build meshes
 * @deprecated use the function directly from the module
 */
export const PlaneBuilder: {
    CreatePlane: typeof CreatePlane;
};

}
declare module "babylonjs/Meshes/Builders/polygonBuilder" {
import { Scene } from "babylonjs/scene";
import { Vector3 } from "babylonjs/Maths/math.vector";
import { Vector4 } from "babylonjs/Maths/math.vector";
import { Color4 } from "babylonjs/Maths/math.color";
import { Mesh } from "babylonjs/Meshes/mesh";
import { VertexData } from "babylonjs/Meshes/mesh.vertexData";
import { Nullable } from "babylonjs/types";
/**
 * Creates the VertexData for an irregular Polygon in the XoZ plane using a mesh built by polygonTriangulation.build()
 * All parameters are provided by CreatePolygon as needed
 * @param polygon a mesh built from polygonTriangulation.build()
 * @param sideOrientation takes the values Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
 * @param fUV an array of Vector4 elements used to set different images to the top, rings and bottom respectively
 * @param fColors an array of Color3 elements used to set different colors to the top, rings and bottom respectively
 * @param frontUVs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the front side, optional, default vector4 (0, 0, 1, 1)
 * @param backUVs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the back side, optional, default vector4 (0, 0, 1, 1)
 * @param wrp a boolean, default false, when true and fUVs used texture is wrapped around all sides, when false texture is applied side
 * @returns the VertexData of the Polygon
 */
export function CreatePolygonVertexData(polygon: Mesh, sideOrientation: number, fUV?: Vector4[], fColors?: Color4[], frontUVs?: Vector4, backUVs?: Vector4, wrp?: boolean): VertexData;
/**
 * Creates a polygon mesh
 * The polygon's shape will depend on the input parameters and is constructed parallel to a ground mesh
 * * The parameter `shape` is a required array of successive Vector3 representing the corners of the polygon in th XoZ plane, that is y = 0 for all vectors
 * * You can set the mesh side orientation with the values : Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
 * * The mesh can be set to updatable with the boolean parameter `updatable` (default false) if its internal geometry is supposed to change once created
 * * If you create a double-sided mesh, you can choose what parts of the texture image to crop and stick respectively on the front and the back sides with the parameters `frontUVs` and `backUVs` (Vector4)
 * * Remember you can only change the shape positions, not their number when updating a polygon
 * @param name defines the name of the mesh
 * @param options defines the options used to create the mesh
 * @param options.shape
 * @param options.holes
 * @param options.depth
 * @param options.smoothingThreshold
 * @param options.faceUV
 * @param options.faceColors
 * @param options.updatable
 * @param options.sideOrientation
 * @param options.frontUVs
 * @param options.backUVs
 * @param options.wrap
 * @param scene defines the hosting scene
 * @param earcutInjection can be used to inject your own earcut reference
 * @returns the polygon mesh
 */
export function CreatePolygon(name: string, options: {
    shape: Vector3[];
    holes?: Vector3[][];
    depth?: number;
    smoothingThreshold?: number;
    faceUV?: Vector4[];
    faceColors?: Color4[];
    updatable?: boolean;
    sideOrientation?: number;
    frontUVs?: Vector4;
    backUVs?: Vector4;
    wrap?: boolean;
}, scene?: Nullable<Scene>, earcutInjection?: any): Mesh;
/**
 * Creates an extruded polygon mesh, with depth in the Y direction.
 * * You can set different colors and different images to the top, bottom and extruded side by using the parameters `faceColors` (an array of 3 Color3 elements) and `faceUV` (an array of 3 Vector4 elements)
 * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/texturePerBoxFace
 * @param name defines the name of the mesh
 * @param options defines the options used to create the mesh
 * @param options.shape
 * @param options.holes
 * @param options.depth
 * @param options.faceUV
 * @param options.faceColors
 * @param options.updatable
 * @param options.sideOrientation
 * @param options.frontUVs
 * @param options.backUVs
 * @param options.wrap
 * @param scene defines the hosting scene
 * @param earcutInjection can be used to inject your own earcut reference
 * @returns the polygon mesh
 */
export function ExtrudePolygon(name: string, options: {
    shape: Vector3[];
    holes?: Vector3[][];
    depth?: number;
    faceUV?: Vector4[];
    faceColors?: Color4[];
    updatable?: boolean;
    sideOrientation?: number;
    frontUVs?: Vector4;
    backUVs?: Vector4;
    wrap?: boolean;
}, scene?: Nullable<Scene>, earcutInjection?: any): Mesh;
/**
 * Class containing static functions to help procedurally build meshes
 * @deprecated use the functions directly from the module
 */
export const PolygonBuilder: {
    ExtrudePolygon: typeof ExtrudePolygon;
    CreatePolygon: typeof CreatePolygon;
};

}
declare module "babylonjs/Meshes/Builders/polyhedronBuilder" {
import { Scene } from "babylonjs/scene";
import { Vector4 } from "babylonjs/Maths/math.vector";
import { Color4 } from "babylonjs/Maths/math.color";
import { Mesh } from "babylonjs/Meshes/mesh";
import { VertexData } from "babylonjs/Meshes/mesh.vertexData";
import { Nullable } from "babylonjs/types";
/**
 * Creates the VertexData for a Polyhedron
 * @param options an object used to set the following optional parameters for the polyhedron, required but can be empty
 * * type provided types are:
 *  * 0 : Tetrahedron, 1 : Octahedron, 2 : Dodecahedron, 3 : Icosahedron, 4 : Rhombicuboctahedron, 5 : Triangular Prism, 6 : Pentagonal Prism, 7 : Hexagonal Prism, 8 : Square Pyramid (J1)
 *  * 9 : Pentagonal Pyramid (J2), 10 : Triangular Dipyramid (J12), 11 : Pentagonal Dipyramid (J13), 12 : Elongated Square Dipyramid (J15), 13 : Elongated Pentagonal Dipyramid (J16), 14 : Elongated Pentagonal Cupola (J20)
 * * size the size of the IcoSphere, optional default 1
 * * sizeX allows stretching in the x direction, optional, default size
 * * sizeY allows stretching in the y direction, optional, default size
 * * sizeZ allows stretching in the z direction, optional, default size
 * * custom a number that overwrites the type to create from an extended set of polyhedron from https://www.babylonjs-playground.com/#21QRSK#15 with minimised editor
 * * faceUV an array of Vector4 elements used to set different images to the top, rings and bottom respectively
 * * faceColors an array of Color3 elements used to set different colors to the top, rings and bottom respectively
 * * flat when true creates a flat shaded mesh, optional, default true
 * * subdivisions increasing the subdivisions increases the number of faces, optional, default 4
 * * sideOrientation optional and takes the values : Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
 * * frontUvs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the front side, optional, default vector4 (0, 0, 1, 1)
 * * backUVs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the back side, optional, default vector4 (0, 0, 1, 1)
 * @param options.type
 * @param options.size
 * @param options.sizeX
 * @param options.sizeY
 * @param options.sizeZ
 * @param options.custom
 * @param options.faceUV
 * @param options.faceColors
 * @param options.flat
 * @param options.sideOrientation
 * @param options.frontUVs
 * @param options.backUVs
 * @returns the VertexData of the Polyhedron
 */
export function CreatePolyhedronVertexData(options: {
    type?: number;
    size?: number;
    sizeX?: number;
    sizeY?: number;
    sizeZ?: number;
    custom?: any;
    faceUV?: Vector4[];
    faceColors?: Color4[];
    flat?: boolean;
    sideOrientation?: number;
    frontUVs?: Vector4;
    backUVs?: Vector4;
}): VertexData;
/**
 * Creates a polyhedron mesh
 * * The parameter `type` (positive integer, max 14, default 0) sets the polyhedron type to build among the 15 embbeded types. Please refer to the type sheet in the tutorial to choose the wanted type
 * * The parameter `size` (positive float, default 1) sets the polygon size
 * * You can overwrite the `size` on each dimension bu using the parameters `sizeX`, `sizeY` or `sizeZ` (positive floats, default to `size` value)
 * * You can build other polyhedron types than the 15 embbeded ones by setting the parameter `custom` (`polyhedronObject`, default null). If you set the parameter `custom`, this overrides the parameter `type`
 * * A `polyhedronObject` is a formatted javascript object. You'll find a full file with pre-set polyhedra here : https://github.com/BabylonJS/Extensions/tree/master/Polyhedron
 * * You can set the color and the UV of each side of the polyhedron with the parameters `faceColors` (Color4, default `(1, 1, 1, 1)`) and faceUV (Vector4, default `(0, 0, 1, 1)`)
 * * To understand how to set `faceUV` or `faceColors`, please read this by considering the right number of faces of your polyhedron, instead of only 6 for the box : https://doc.babylonjs.com/features/featuresDeepDive/materials/using/texturePerBoxFace
 * * The parameter `flat` (boolean, default true). If set to false, it gives the polyhedron a single global face, so less vertices and shared normals. In this case, `faceColors` and `faceUV` are ignored
 * * You can also set the mesh side orientation with the values : BABYLON.Mesh.FRONTSIDE (default), BABYLON.Mesh.BACKSIDE or BABYLON.Mesh.DOUBLESIDE
 * * If you create a double-sided mesh, you can choose what parts of the texture image to crop and stick respectively on the front and the back sides with the parameters `frontUVs` and `backUVs` (Vector4). Detail here : https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#side-orientation
 * * The mesh can be set to updatable with the boolean parameter `updatable` (default false) if its internal geometry is supposed to change once created
 * @param name defines the name of the mesh
 * @param options defines the options used to create the mesh
 * @param options.type
 * @param options.size
 * @param options.sizeX
 * @param options.sizeY
 * @param options.sizeZ
 * @param options.custom
 * @param options.faceUV
 * @param options.faceColors
 * @param options.flat
 * @param options.updatable
 * @param options.sideOrientation
 * @param options.frontUVs
 * @param options.backUVs
 * @param scene defines the hosting scene
 * @returns the polyhedron mesh
 * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/polyhedra
 */
export function CreatePolyhedron(name: string, options?: {
    type?: number;
    size?: number;
    sizeX?: number;
    sizeY?: number;
    sizeZ?: number;
    custom?: any;
    faceUV?: Vector4[];
    faceColors?: Color4[];
    flat?: boolean;
    updatable?: boolean;
    sideOrientation?: number;
    frontUVs?: Vector4;
    backUVs?: Vector4;
}, scene?: Nullable<Scene>): Mesh;
/**
 * Class containing static functions to help procedurally build meshes
 * @deprecated use the function directly from the module
 */
export const PolyhedronBuilder: {
    CreatePolyhedron: typeof CreatePolyhedron;
};

}
declare module "babylonjs/Meshes/Builders/ribbonBuilder" {
import { Nullable } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
import { Vector3, Vector2, Vector4 } from "babylonjs/Maths/math.vector";
import { Color4 } from "babylonjs/Maths/math.color";
import { Mesh } from "babylonjs/Meshes/mesh";
import { VertexData } from "babylonjs/Meshes/mesh.vertexData";
/**
 * Creates the VertexData for a Ribbon
 * @param options an object used to set the following optional parameters for the ribbon, required but can be empty
 * * pathArray array of paths, each of which an array of successive Vector3
 * * closeArray creates a seam between the first and the last paths of the pathArray, optional, default false
 * * closePath creates a seam between the first and the last points of each path of the path array, optional, default false
 * * offset a positive integer, only used when pathArray contains a single path (offset = 10 means the point 1 is joined to the point 11), default rounded half size of the pathArray length
 * * sideOrientation optional and takes the values : Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
 * * frontUvs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the front side, optional, default vector4 (0, 0, 1, 1)
 * * backUVs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the back side, optional, default vector4 (0, 0, 1, 1)
 * * invertUV swaps in the U and V coordinates when applying a texture, optional, default false
 * * uvs a linear array, of length 2 * number of vertices, of custom UV values, optional
 * * colors a linear array, of length 4 * number of vertices, of custom color values, optional
 * @param options.pathArray
 * @param options.closeArray
 * @param options.closePath
 * @param options.offset
 * @param options.sideOrientation
 * @param options.frontUVs
 * @param options.backUVs
 * @param options.invertUV
 * @param options.uvs
 * @param options.colors
 * @returns the VertexData of the ribbon
 */
export function CreateRibbonVertexData(options: {
    pathArray: Vector3[][];
    closeArray?: boolean;
    closePath?: boolean;
    offset?: number;
    sideOrientation?: number;
    frontUVs?: Vector4;
    backUVs?: Vector4;
    invertUV?: boolean;
    uvs?: Vector2[];
    colors?: Color4[];
}): VertexData;
/**
 * Creates a ribbon mesh. The ribbon is a parametric shape.  It has no predefined shape. Its final shape will depend on the input parameters
 * * The parameter `pathArray` is a required array of paths, what are each an array of successive Vector3. The pathArray parameter depicts the ribbon geometry
 * * The parameter `closeArray` (boolean, default false) creates a seam between the first and the last paths of the path array
 * * The parameter `closePath` (boolean, default false) creates a seam between the first and the last points of each path of the path array
 * * The parameter `offset` (positive integer, default : rounded half size of the pathArray length), is taken in account only if the `pathArray` is containing a single path
 * * It's the offset to join the points from the same path. Ex : offset = 10 means the point 1 is joined to the point 11
 * * The optional parameter `instance` is an instance of an existing Ribbon object to be updated with the passed `pathArray` parameter : https://doc.babylonjs.com/features/featuresDeepDive/mesh/dynamicMeshMorph#ribbon
 * * You can also set the mesh side orientation with the values : BABYLON.Mesh.FRONTSIDE (default), BABYLON.Mesh.BACKSIDE or BABYLON.Mesh.DOUBLESIDE
 * * If you create a double-sided mesh, you can choose what parts of the texture image to crop and stick respectively on the front and the back sides with the parameters `frontUVs` and `backUVs` (Vector4). Detail here : https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#side-orientation
 * * The optional parameter `invertUV` (boolean, default false) swaps in the geometry the U and V coordinates to apply a texture
 * * The parameter `uvs` is an optional flat array of `Vector2` to update/set each ribbon vertex with its own custom UV values instead of the computed ones
 * * The parameters `colors` is an optional flat array of `Color4` to set/update each ribbon vertex with its own custom color values
 * * Note that if you use the parameters `uvs` or `colors`, the passed arrays must be populated with the right number of elements, it is to say the number of ribbon vertices. Remember that if you set `closePath` to `true`, there's one extra vertex per path in the geometry
 * * Moreover, you can use the parameter `color` with `instance` (to update the ribbon), only if you previously used it at creation time
 * * The mesh can be set to updatable with the boolean parameter `updatable` (default false) if its internal geometry is supposed to change once created
 * @param name defines the name of the mesh
 * @param options defines the options used to create the mesh
 * @param options.pathArray
 * @param options.closeArray
 * @param options.closePath
 * @param options.offset
 * @param options.updatable
 * @param options.sideOrientation
 * @param options.frontUVs
 * @param options.backUVs
 * @param options.instance
 * @param options.invertUV
 * @param options.uvs
 * @param options.colors
 * @param scene defines the hosting scene
 * @returns the ribbon mesh
 * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/param/ribbon_extra
 * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/param
 */
export function CreateRibbon(name: string, options: {
    pathArray: Vector3[][];
    closeArray?: boolean;
    closePath?: boolean;
    offset?: number;
    updatable?: boolean;
    sideOrientation?: number;
    frontUVs?: Vector4;
    backUVs?: Vector4;
    instance?: Mesh;
    invertUV?: boolean;
    uvs?: Vector2[];
    colors?: Color4[];
}, scene?: Nullable<Scene>): Mesh;
/**
 * Class containing static functions to help procedurally build meshes
 * @deprecated use CreateRibbon directly
 */
export const RibbonBuilder: {
    CreateRibbon: typeof CreateRibbon;
};

}
declare module "babylonjs/Meshes/Builders/shapeBuilder" {
import { Nullable } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
import { Vector4 } from "babylonjs/Maths/math.vector";
import { Vector3 } from "babylonjs/Maths/math.vector";
import { Mesh } from "babylonjs/Meshes/mesh";
/**
 * Creates an extruded shape mesh. The extrusion is a parametric shape. It has no predefined shape. Its final shape will depend on the input parameters.
 * * The parameter `shape` is a required array of successive Vector3. This array depicts the shape to be extruded in its local space : the shape must be designed in the xOy plane and will be extruded along the Z axis.
 * * The parameter `path` is a required array of successive Vector3. This is the axis curve the shape is extruded along.
 * * The parameter `rotation` (float, default 0 radians) is the angle value to rotate the shape each step (each path point), from the former step (so rotation added each step) along the curve.
 * * The parameter `scale` (float, default 1) is the value to scale the shape.
 * * The parameter `closeShape` (boolean, default false) closes the shape when true, since v5.0.0.
 * * The parameter `closePath` (boolean, default false) closes the path when true and no caps, since v5.0.0.
 * * The parameter `cap` sets the way the extruded shape is capped. Possible values : BABYLON.Mesh.NO_CAP (default), BABYLON.Mesh.CAP_START, BABYLON.Mesh.CAP_END, BABYLON.Mesh.CAP_ALL
 * * The optional parameter `instance` is an instance of an existing ExtrudedShape object to be updated with the passed `shape`, `path`, `scale` or `rotation` parameters : https://doc.babylonjs.com/features/featuresDeepDive/mesh/dynamicMeshMorph#extruded-shape
 * * Remember you can only change the shape or path point positions, not their number when updating an extruded shape.
 * * You can also set the mesh side orientation with the values : BABYLON.Mesh.FRONTSIDE (default), BABYLON.Mesh.BACKSIDE or BABYLON.Mesh.DOUBLESIDE
 * * If you create a double-sided mesh, you can choose what parts of the texture image to crop and stick respectively on the front and the back sides with the parameters `frontUVs` and `backUVs` (Vector4). Detail here : https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#side-orientation
 * * The optional parameter `invertUV` (boolean, default false) swaps in the geometry the U and V coordinates to apply a texture.
 * * The mesh can be set to updatable with the boolean parameter `updatable` (default false) if its internal geometry is supposed to change once created.
 * * The optional parameter `firstNormal` (Vector3) defines the direction of the first normal of the supplied path. Consider using this for any path that is straight, and particular for paths in the xy plane.
 * * The optional `adjustFrame` (boolean, default false) will cause the internally generated Path3D tangents, normals, and binormals to be adjusted so that a) they are always well-defined, and b) they do not reverse from one path point to the next. This prevents the extruded shape from being flipped and/or rotated with resulting mesh self-intersections. This is primarily useful for straight paths that can reverse direction.
 * @param name defines the name of the mesh
 * @param options defines the options used to create the mesh
 * @param options.shape
 * @param options.path
 * @param options.scale
 * @param options.rotation
 * @param options.closeShape
 * @param options.closePath
 * @param options.cap
 * @param options.updatable
 * @param options.sideOrientation
 * @param options.frontUVs
 * @param options.backUVs
 * @param options.instance
 * @param options.invertUV
 * @param options.firstNormal
 * @param options.adjustFrame
 * @param scene defines the hosting scene
 * @returns the extruded shape mesh
 * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/param
 * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/param#extruded-shapes
 */
export function ExtrudeShape(name: string, options: {
    shape: Vector3[];
    path: Vector3[];
    scale?: number;
    rotation?: number;
    closeShape?: boolean;
    closePath?: boolean;
    cap?: number;
    updatable?: boolean;
    sideOrientation?: number;
    frontUVs?: Vector4;
    backUVs?: Vector4;
    instance?: Mesh;
    invertUV?: boolean;
    firstNormal?: Vector3;
    adjustFrame?: boolean;
}, scene?: Nullable<Scene>): Mesh;
/**
 * Creates an custom extruded shape mesh.
 * The custom extrusion is a parametric shape. It has no predefined shape. Its final shape will depend on the input parameters.
 * * The parameter `shape` is a required array of successive Vector3. This array depicts the shape to be extruded in its local space : the shape must be designed in the xOy plane and will be extruded along the Z axis.
 * * The parameter `path` is a required array of successive Vector3. This is the axis curve the shape is extruded along.
 * * The parameter `rotationFunction` (JS function) is a custom Javascript function called on each path point. This function is passed the position i of the point in the path and the distance of this point from the beginning of the path
 * * It must returns a float value that will be the rotation in radians applied to the shape on each path point.
 * * The parameter `scaleFunction` (JS function) is a custom Javascript function called on each path point. This function is passed the position i of the point in the path and the distance of this point from the beginning of the path
 * * It must returns a float value that will be the scale value applied to the shape on each path point
 * * The parameter `closeShape` (boolean, default false) closes the shape when true, since v5.0.0.
 * * The parameter `closePath` (boolean, default false) closes the path when true and no caps, since v5.0.0.
 * * The parameter `ribbonClosePath` (boolean, default false) forces the extrusion underlying ribbon to close all the paths in its `pathArray` - depreciated in favor of closeShape
 * * The parameter `ribbonCloseArray` (boolean, default false) forces the extrusion underlying ribbon to close its `pathArray` - depreciated in favor of closePath
 * * The parameter `cap` sets the way the extruded shape is capped. Possible values : BABYLON.Mesh.NO_CAP (default), BABYLON.Mesh.CAP_START, BABYLON.Mesh.CAP_END, BABYLON.Mesh.CAP_ALL
 * * The optional parameter `instance` is an instance of an existing ExtrudedShape object to be updated with the passed `shape`, `path`, `scale` or `rotation` parameters : https://doc.babylonjs.com/features/featuresDeepDive/mesh/dynamicMeshMorph#extruded-shape
 * * Remember you can only change the shape or path point positions, not their number when updating an extruded shape
 * * You can also set the mesh side orientation with the values : BABYLON.Mesh.FRONTSIDE (default), BABYLON.Mesh.BACKSIDE or BABYLON.Mesh.DOUBLESIDE
 * * If you create a double-sided mesh, you can choose what parts of the texture image to crop and stick respectively on the front and the back sides with the parameters `frontUVs` and `backUVs` (Vector4). Detail here : https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#side-orientation
 * * The optional parameter `invertUV` (boolean, default false) swaps in the geometry the U and V coordinates to apply a texture
 * * The mesh can be set to updatable with the boolean parameter `updatable` (default false) if its internal geometry is supposed to change once created
 * * The optional parameter `firstNormal` (Vector3) defines the direction of the first normal of the supplied path. It should be supplied when the path is in the xy plane, and particularly if these sections are straight, because the underlying Path3D object will pick a normal in the xy plane that causes the extrusion to be collapsed into the plane. This should be used for any path that is straight.
 * * The optional `adjustFrame` (boolean, default false) will cause the internally generated Path3D tangents, normals, and binormals to be adjusted so that a) they are always well-defined, and b) they do not reverse from one path point to the next. This prevents the extruded shape from being flipped and/or rotated with resulting mesh self-intersections. This is primarily useful for straight paths that can reverse direction.
 * @param name defines the name of the mesh
 * @param options defines the options used to create the mesh
 * @param options.shape
 * @param options.path
 * @param options.scaleFunction
 * @param options.rotationFunction
 * @param options.ribbonCloseArray
 * @param options.ribbonClosePath
 * @param options.closeShape
 * @param options.closePath
 * @param options.cap
 * @param options.updatable
 * @param options.sideOrientation
 * @param options.frontUVs
 * @param options.backUVs
 * @param options.instance
 * @param options.invertUV
 * @param options.firstNormal
 * @param options.adjustFrame
 * @param scene defines the hosting scene
 * @returns the custom extruded shape mesh
 * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/param#custom-extruded-shapes
 * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/param
 * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/param#extruded-shapes
 */
export function ExtrudeShapeCustom(name: string, options: {
    shape: Vector3[];
    path: Vector3[];
    scaleFunction?: Nullable<{
        (i: number, distance: number): number;
    }>;
    rotationFunction?: Nullable<{
        (i: number, distance: number): number;
    }>;
    ribbonCloseArray?: boolean;
    ribbonClosePath?: boolean;
    closeShape?: boolean;
    closePath?: boolean;
    cap?: number;
    updatable?: boolean;
    sideOrientation?: number;
    frontUVs?: Vector4;
    backUVs?: Vector4;
    instance?: Mesh;
    invertUV?: boolean;
    firstNormal?: Vector3;
    adjustFrame?: boolean;
}, scene?: Nullable<Scene>): Mesh;
/**
 * Class containing static functions to help procedurally build meshes
 * @deprecated please use the functions directly from the module
 */
export const ShapeBuilder: {
    ExtrudeShape: typeof ExtrudeShape;
    ExtrudeShapeCustom: typeof ExtrudeShapeCustom;
};

}
declare module "babylonjs/Meshes/Builders/sphereBuilder" {
import { Vector4 } from "babylonjs/Maths/math.vector";
import { Mesh } from "babylonjs/Meshes/mesh";
import { VertexData } from "babylonjs/Meshes/mesh.vertexData";
import { Scene } from "babylonjs/scene";
import { Nullable } from "babylonjs/types";
/**
 * Creates the VertexData for an ellipsoid, defaults to a sphere
 * @param options an object used to set the following optional parameters for the box, required but can be empty
 * * segments sets the number of horizontal strips optional, default 32
 * * diameter sets the axes dimensions, diameterX, diameterY and diameterZ to the value of diameter, optional default 1
 * * diameterX sets the diameterX (x direction) of the ellipsoid, overwrites the diameterX set by diameter, optional, default diameter
 * * diameterY sets the diameterY (y direction) of the ellipsoid, overwrites the diameterY set by diameter, optional, default diameter
 * * diameterZ sets the diameterZ (z direction) of the ellipsoid, overwrites the diameterZ set by diameter, optional, default diameter
 * * arc a number from 0 to 1, to create an unclosed ellipsoid based on the fraction of the circumference (latitude) given by the arc value, optional, default 1
 * * slice a number from 0 to 1, to create an unclosed ellipsoid based on the fraction of the height (latitude) given by the arc value, optional, default 1
 * * sideOrientation optional and takes the values : Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
 * * frontUvs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the front side, optional, default vector4 (0, 0, 1, 1)
 * * backUVs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the back side, optional, default vector4 (0, 0, 1, 1)
 * @param options.segments
 * @param options.diameter
 * @param options.diameterX
 * @param options.diameterY
 * @param options.diameterZ
 * @param options.arc
 * @param options.slice
 * @param options.sideOrientation
 * @param options.frontUVs
 * @param options.backUVs
 * @param options.dedupTopBottomIndices
 * @returns the VertexData of the ellipsoid
 */
export function CreateSphereVertexData(options: {
    segments?: number;
    diameter?: number;
    diameterX?: number;
    diameterY?: number;
    diameterZ?: number;
    arc?: number;
    slice?: number;
    sideOrientation?: number;
    frontUVs?: Vector4;
    backUVs?: Vector4;
    dedupTopBottomIndices?: boolean;
}): VertexData;
/**
 * Creates a sphere mesh
 * * The parameter `diameter` sets the diameter size (float) of the sphere (default 1)
 * * You can set some different sphere dimensions, for instance to build an ellipsoid, by using the parameters `diameterX`, `diameterY` and `diameterZ` (all by default have the same value of `diameter`)
 * * The parameter `segments` sets the sphere number of horizontal stripes (positive integer, default 32)
 * * You can create an unclosed sphere with the parameter `arc` (positive float, default 1), valued between 0 and 1, what is the ratio of the circumference (latitude) : 2 x PI x ratio
 * * You can create an unclosed sphere on its height with the parameter `slice` (positive float, default1), valued between 0 and 1, what is the height ratio (longitude)
 * * You can also set the mesh side orientation with the values : BABYLON.Mesh.FRONTSIDE (default), BABYLON.Mesh.BACKSIDE or BABYLON.Mesh.DOUBLESIDE
 * * If you create a double-sided mesh, you can choose what parts of the texture image to crop and stick respectively on the front and the back sides with the parameters `frontUVs` and `backUVs` (Vector4). Detail here : https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#side-orientation
 * * The mesh can be set to updatable with the boolean parameter `updatable` (default false) if its internal geometry is supposed to change once created
 * @param name defines the name of the mesh
 * @param options defines the options used to create the mesh
 * @param options.segments
 * @param options.diameter
 * @param options.diameterX
 * @param options.diameterY
 * @param options.diameterZ
 * @param options.arc
 * @param options.slice
 * @param options.sideOrientation
 * @param options.frontUVs
 * @param options.backUVs
 * @param options.updatable
 * @param scene defines the hosting scene
 * @returns the sphere mesh
 * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#sphere
 */
export function CreateSphere(name: string, options?: {
    segments?: number;
    diameter?: number;
    diameterX?: number;
    diameterY?: number;
    diameterZ?: number;
    arc?: number;
    slice?: number;
    sideOrientation?: number;
    frontUVs?: Vector4;
    backUVs?: Vector4;
    updatable?: boolean;
}, scene?: Nullable<Scene>): Mesh;
/**
 * Class containing static functions to help procedurally build meshes
 * @deprecated use CreateSphere directly
 */
export const SphereBuilder: {
    CreateSphere: typeof CreateSphere;
};

}
declare module "babylonjs/Meshes/Builders/tiledBoxBuilder" {
import { Nullable } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
import { Vector4 } from "babylonjs/Maths/math.vector";
import { Color4 } from "babylonjs/Maths/math.color";
import { Mesh } from "babylonjs/Meshes/mesh";
import { VertexData } from "babylonjs/Meshes/mesh.vertexData";
/**
 * Creates the VertexData for a tiled box
 * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set/tiled_box
 * @param options an object used to set the following optional parameters for the tiled box, required but can be empty
 * * pattern sets the rotation or reflection pattern for the tiles,
 * * size of the box
 * * width of the box, overwrites size
 * * height of the box, overwrites size
 * * depth of the box, overwrites size
 * * tileSize sets the size of a tile
 * * tileWidth sets the tile width and overwrites tileSize
 * * tileHeight sets the tile width and overwrites tileSize
 * * faceUV an array of 6 Vector4 elements used to set different images to each box side
 * * faceColors an array of 6 Color3 elements used to set different colors to each box side
 * * alignHorizontal places whole tiles aligned to the center, left or right of a row
 * * alignVertical places whole tiles aligned to the center, left or right of a column
 * @param options.pattern
 * @param options.size
 * @param options.width
 * @param options.height
 * @param options.depth
 * @param options.tileSize
 * @param options.tileWidth
 * @param options.tileHeight
 * @param options.faceUV
 * @param options.faceColors
 * @param options.alignHorizontal
 * @param options.alignVertical
 * @param options.sideOrientation
 * * sideOrientation optional and takes the values : Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
 * @returns the VertexData of the TiledBox
 */
export function CreateTiledBoxVertexData(options: {
    pattern?: number;
    size?: number;
    width?: number;
    height?: number;
    depth?: number;
    tileSize?: number;
    tileWidth?: number;
    tileHeight?: number;
    faceUV?: Vector4[];
    faceColors?: Color4[];
    alignHorizontal?: number;
    alignVertical?: number;
    sideOrientation?: number;
}): VertexData;
/**
 * Creates a tiled box mesh
 * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set/tiled_box
 * @param name defines the name of the mesh
 * @param options an object used to set the following optional parameters for the tiled box, required but can be empty
 * * pattern sets the rotation or reflection pattern for the tiles,
 * * size of the box
 * * width of the box, overwrites size
 * * height of the box, overwrites size
 * * depth of the box, overwrites size
 * * tileSize sets the size of a tile
 * * tileWidth sets the tile width and overwrites tileSize
 * * tileHeight sets the tile width and overwrites tileSize
 * * faceUV an array of 6 Vector4 elements used to set different images to each box side
 * * faceColors an array of 6 Color3 elements used to set different colors to each box side
 * * alignHorizontal places whole tiles aligned to the center, left or right of a row
 * * alignVertical places whole tiles aligned to the center, left or right of a column
 * * sideOrientation optional and takes the values : Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
 * @param options.pattern
 * @param options.width
 * @param options.height
 * @param options.depth
 * @param options.tileSize
 * @param options.tileWidth
 * @param options.tileHeight
 * @param options.alignHorizontal
 * @param options.alignVertical
 * @param options.faceUV
 * @param options.faceColors
 * @param options.sideOrientation
 * @param options.updatable
 * @param scene defines the hosting scene
 * @returns the box mesh
 */
export function CreateTiledBox(name: string, options: {
    pattern?: number;
    width?: number;
    height?: number;
    depth?: number;
    tileSize?: number;
    tileWidth?: number;
    tileHeight?: number;
    alignHorizontal?: number;
    alignVertical?: number;
    faceUV?: Vector4[];
    faceColors?: Color4[];
    sideOrientation?: number;
    updatable?: boolean;
}, scene?: Nullable<Scene>): Mesh;
/**
 * Class containing static functions to help procedurally build meshes
 * @deprecated use CreateTiledBox instead
 */
export const TiledBoxBuilder: {
    CreateTiledBox: typeof CreateTiledBox;
};

}
declare module "babylonjs/Meshes/Builders/tiledPlaneBuilder" {
import { Nullable } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
import { Vector4 } from "babylonjs/Maths/math.vector";
import { Mesh } from "babylonjs/Meshes/mesh";
import { VertexData } from "babylonjs/Meshes/mesh.vertexData";
/**
 * Creates the VertexData for a tiled plane
 * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set/tiled_plane
 * @param options an object used to set the following optional parameters for the tiled plane, required but can be empty
 * * pattern a limited pattern arrangement depending on the number
 * * size of the box
 * * width of the box, overwrites size
 * * height of the box, overwrites size
 * * tileSize sets the width, height and depth of the tile to the value of size, optional default 1
 * * tileWidth sets the width (x direction) of the tile, overwrites the width set by size, optional, default size
 * * tileHeight sets the height (y direction) of the tile, overwrites the height set by size, optional, default size
 * * sideOrientation optional and takes the values : Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
 * * alignHorizontal places whole tiles aligned to the center, left or right of a row
 * * alignVertical places whole tiles aligned to the center, left or right of a column
 * * frontUvs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the front side, optional, default vector4 (0, 0, 1, 1)
 * @param options.pattern
 * @param options.tileSize
 * @param options.tileWidth
 * @param options.tileHeight
 * @param options.size
 * @param options.width
 * @param options.height
 * @param options.alignHorizontal
 * @param options.alignVertical
 * @param options.sideOrientation
 * @param options.frontUVs
 * @param options.backUVs
 * * backUVs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the back side, optional, default vector4 (0, 0, 1, 1)
 * @returns the VertexData of the tiled plane
 */
export function CreateTiledPlaneVertexData(options: {
    pattern?: number;
    tileSize?: number;
    tileWidth?: number;
    tileHeight?: number;
    size?: number;
    width?: number;
    height?: number;
    alignHorizontal?: number;
    alignVertical?: number;
    sideOrientation?: number;
    frontUVs?: Vector4;
    backUVs?: Vector4;
}): VertexData;
/**
 * Creates a tiled plane mesh
 * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set/tiled_plane
 * @param name defines the name of the mesh
 * @param options an object used to set the following optional parameters for the tiled plane, required but can be empty
 * * pattern a limited pattern arrangement depending on the number
 * * size of the box
 * * width of the box, overwrites size
 * * height of the box, overwrites size
 * * tileSize sets the width, height and depth of the tile to the value of size, optional default 1
 * * tileWidth sets the width (x direction) of the tile, overwrites the width set by size, optional, default size
 * * tileHeight sets the height (y direction) of the tile, overwrites the height set by size, optional, default size
 * * sideOrientation optional and takes the values : Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
 * * alignHorizontal places whole tiles aligned to the center, left or right of a row
 * * alignVertical places whole tiles aligned to the center, left or right of a column
 * * frontUvs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the front side, optional, default vector4 (0, 0, 1, 1)
 * * backUVs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the back side, optional, default vector4 (0, 0, 1, 1)
 * @param options.pattern
 * @param options.tileSize
 * @param options.tileWidth
 * @param options.tileHeight
 * @param options.size
 * @param options.width
 * @param options.height
 * @param options.alignHorizontal
 * @param options.alignVertical
 * @param options.sideOrientation
 * @param options.frontUVs
 * @param options.backUVs
 * @param options.updatable
 * @param scene defines the hosting scene
 * @returns the box mesh
 */
export function CreateTiledPlane(name: string, options: {
    pattern?: number;
    tileSize?: number;
    tileWidth?: number;
    tileHeight?: number;
    size?: number;
    width?: number;
    height?: number;
    alignHorizontal?: number;
    alignVertical?: number;
    sideOrientation?: number;
    frontUVs?: Vector4;
    backUVs?: Vector4;
    updatable?: boolean;
}, scene?: Nullable<Scene>): Mesh;
/**
 * Class containing static functions to help procedurally build meshes
 * @deprecated use CreateTiledPlane instead
 */
export const TiledPlaneBuilder: {
    CreateTiledPlane: typeof CreateTiledPlane;
};

}
declare module "babylonjs/Meshes/Builders/torusBuilder" {
import { Vector4 } from "babylonjs/Maths/math.vector";
import { Mesh } from "babylonjs/Meshes/mesh";
import { VertexData } from "babylonjs/Meshes/mesh.vertexData";
import { Scene } from "babylonjs/scene";
/**
 * Creates the VertexData for a torus
 * @param options an object used to set the following optional parameters for the box, required but can be empty
 * * diameter the diameter of the torus, optional default 1
 * * thickness the diameter of the tube forming the torus, optional default 0.5
 * * tessellation the number of prism sides, 3 for a triangular prism, optional, default 24
 * * sideOrientation optional and takes the values : Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
 * * frontUvs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the front side, optional, default vector4 (0, 0, 1, 1)
 * * backUVs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the back side, optional, default vector4 (0, 0, 1, 1)
 * @param options.diameter
 * @param options.thickness
 * @param options.tessellation
 * @param options.sideOrientation
 * @param options.frontUVs
 * @param options.backUVs
 * @returns the VertexData of the torus
 */
export function CreateTorusVertexData(options: {
    diameter?: number;
    thickness?: number;
    tessellation?: number;
    sideOrientation?: number;
    frontUVs?: Vector4;
    backUVs?: Vector4;
}): VertexData;
/**
 * Creates a torus mesh
 * * The parameter `diameter` sets the diameter size (float) of the torus (default 1)
 * * The parameter `thickness` sets the diameter size of the tube of the torus (float, default 0.5)
 * * The parameter `tessellation` sets the number of torus sides (positive integer, default 16)
 * * You can also set the mesh side orientation with the values : BABYLON.Mesh.FRONTSIDE (default), BABYLON.Mesh.BACKSIDE or BABYLON.Mesh.DOUBLESIDE
 * * If you create a double-sided mesh, you can choose what parts of the texture image to crop and stick respectively on the front and the back sides with the parameters `frontUVs` and `backUVs` (Vector4). Detail here : https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#side-orientation
 * * The mesh can be set to updatable with the boolean parameter `updatable` (default false) if its internal geometry is supposed to change once created.
 * @param name defines the name of the mesh
 * @param options defines the options used to create the mesh
 * @param options.diameter
 * @param options.thickness
 * @param options.tessellation
 * @param options.updatable
 * @param options.sideOrientation
 * @param options.frontUVs
 * @param options.backUVs
 * @param scene defines the hosting scene
 * @returns the torus mesh
 * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#torus
 */
export function CreateTorus(name: string, options?: {
    diameter?: number;
    thickness?: number;
    tessellation?: number;
    updatable?: boolean;
    sideOrientation?: number;
    frontUVs?: Vector4;
    backUVs?: Vector4;
}, scene?: Scene): Mesh;
/**
 * Class containing static functions to help procedurally build meshes
 * @deprecated use CreateTorus instead
 */
export const TorusBuilder: {
    CreateTorus: typeof CreateTorus;
};

}
declare module "babylonjs/Meshes/Builders/torusKnotBuilder" {
import { Vector4 } from "babylonjs/Maths/math.vector";
import { Mesh } from "babylonjs/Meshes/mesh";
import { VertexData } from "babylonjs/Meshes/mesh.vertexData";
import { Scene } from "babylonjs/scene";
/**
 * Creates the VertexData for a TorusKnot
 * @param options an object used to set the following optional parameters for the TorusKnot, required but can be empty
 * * radius the radius of the torus knot, optional, default 2
 * * tube the thickness of the tube, optional, default 0.5
 * * radialSegments the number of sides on each tube segments, optional, default 32
 * * tubularSegments the number of tubes to decompose the knot into, optional, default 32
 * * p the number of windings around the z axis, optional,  default 2
 * * q the number of windings around the x axis, optional,  default 3
 * * sideOrientation optional and takes the values : Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
 * * frontUvs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the front side, optional, default vector4 (0, 0, 1, 1)
 * * backUVs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the back side, optional, default vector4 (0, 0, 1, 1)
 * @param options.radius
 * @param options.tube
 * @param options.radialSegments
 * @param options.tubularSegments
 * @param options.p
 * @param options.q
 * @param options.sideOrientation
 * @param options.frontUVs
 * @param options.backUVs
 * @returns the VertexData of the Torus Knot
 */
export function CreateTorusKnotVertexData(options: {
    radius?: number;
    tube?: number;
    radialSegments?: number;
    tubularSegments?: number;
    p?: number;
    q?: number;
    sideOrientation?: number;
    frontUVs?: Vector4;
    backUVs?: Vector4;
}): VertexData;
/**
 * Creates a torus knot mesh
 * * The parameter `radius` sets the global radius size (float) of the torus knot (default 2)
 * * The parameter `radialSegments` sets the number of sides on each tube segments (positive integer, default 32)
 * * The parameter `tubularSegments` sets the number of tubes to decompose the knot into (positive integer, default 32)
 * * The parameters `p` and `q` are the number of windings on each axis (positive integers, default 2 and 3)
 * * You can also set the mesh side orientation with the values : BABYLON.Mesh.FRONTSIDE (default), BABYLON.Mesh.BACKSIDE or BABYLON.Mesh.DOUBLESIDE
 * * If you create a double-sided mesh, you can choose what parts of the texture image to crop and stick respectively on the front and the back sides with the parameters `frontUVs` and `backUVs` (Vector4). Detail here : https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#side-orientation
 * * The mesh can be set to updatable with the boolean parameter `updatable` (default false) if its internal geometry is supposed to change once created.
 * @param name defines the name of the mesh
 * @param options defines the options used to create the mesh
 * @param options.radius
 * @param options.tube
 * @param options.radialSegments
 * @param options.tubularSegments
 * @param options.p
 * @param options.q
 * @param options.updatable
 * @param options.sideOrientation
 * @param options.frontUVs
 * @param options.backUVs
 * @param scene defines the hosting scene
 * @returns the torus knot mesh
 * @see  https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#torus-knot
 */
export function CreateTorusKnot(name: string, options?: {
    radius?: number;
    tube?: number;
    radialSegments?: number;
    tubularSegments?: number;
    p?: number;
    q?: number;
    updatable?: boolean;
    sideOrientation?: number;
    frontUVs?: Vector4;
    backUVs?: Vector4;
}, scene?: Scene): Mesh;
/**
 * Class containing static functions to help procedurally build meshes
 * @deprecated use CreateTorusKnot instead
 */
export const TorusKnotBuilder: {
    CreateTorusKnot: typeof CreateTorusKnot;
};

}
declare module "babylonjs/Meshes/Builders/tubeBuilder" {
import { Nullable } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
import { Vector4 } from "babylonjs/Maths/math.vector";
import { Vector3 } from "babylonjs/Maths/math.vector";
import { Mesh } from "babylonjs/Meshes/mesh";
/**
 * Creates a tube mesh.
 * The tube is a parametric shape. It has no predefined shape. Its final shape will depend on the input parameters
 * * The parameter `path` is a required array of successive Vector3. It is the curve used as the axis of the tube
 * * The parameter `radius` (positive float, default 1) sets the tube radius size
 * * The parameter `tessellation` (positive float, default 64) is the number of sides on the tubular surface
 * * The parameter `radiusFunction` (javascript function, default null) is a vanilla javascript function. If it is not null, it overrides the parameter `radius`
 * * This function is called on each point of the tube path and is passed the index `i` of the i-th point and the distance of this point from the first point of the path. It must return a radius value (positive float)
 * * The parameter `arc` (positive float, maximum 1, default 1) is the ratio to apply to the tube circumference : 2 x PI x arc
 * * The parameter `cap` sets the way the extruded shape is capped. Possible values : BABYLON.Mesh.NO_CAP (default), BABYLON.Mesh.CAP_START, BABYLON.Mesh.CAP_END, BABYLON.Mesh.CAP_ALL
 * * The optional parameter `instance` is an instance of an existing Tube object to be updated with the passed `pathArray` parameter. The `path`Array HAS to have the SAME number of points as the previous one: https://doc.babylonjs.com/features/featuresDeepDive/mesh/dynamicMeshMorph#tube
 * * You can also set the mesh side orientation with the values : BABYLON.Mesh.FRONTSIDE (default), BABYLON.Mesh.BACKSIDE or BABYLON.Mesh.DOUBLESIDE
 * * If you create a double-sided mesh, you can choose what parts of the texture image to crop and stick respectively on the front and the back sides with the parameters `frontUVs` and `backUVs` (Vector4). Detail here : https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#side-orientation
 * * The optional parameter `invertUV` (boolean, default false) swaps in the geometry the U and V coordinates to apply a texture
 * * The mesh can be set to updatable with the boolean parameter `updatable` (default false) if its internal geometry is supposed to change once created. The NUMBER of points CAN'T CHANGE, only their positions.
 * @param name defines the name of the mesh
 * @param options defines the options used to create the mesh
 * @param options.path
 * @param options.radius
 * @param options.tessellation
 * @param options.radiusFunction
 * @param options.cap
 * @param options.arc
 * @param options.updatable
 * @param options.sideOrientation
 * @param options.frontUVs
 * @param options.backUVs
 * @param options.instance
 * @param options.invertUV
 * @param scene defines the hosting scene
 * @returns the tube mesh
 * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/param
 * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#tube
 */
export function CreateTube(name: string, options: {
    path: Vector3[];
    radius?: number;
    tessellation?: number;
    radiusFunction?: {
        (i: number, distance: number): number;
    };
    cap?: number;
    arc?: number;
    updatable?: boolean;
    sideOrientation?: number;
    frontUVs?: Vector4;
    backUVs?: Vector4;
    instance?: Mesh;
    invertUV?: boolean;
}, scene?: Nullable<Scene>): Mesh;
/**
 * Class containing static functions to help procedurally build meshes
 * @deprecated use CreateTube directly
 */
export const TubeBuilder: {
    CreateTube: typeof CreateTube;
};

}
declare module "babylonjs/Meshes/Compression/dracoCompression" {
import { IDisposable } from "babylonjs/scene";
import { VertexData } from "babylonjs/Meshes/mesh.vertexData";
/**
 * Configuration for Draco compression
 */
export interface IDracoCompressionConfiguration {
    /**
     * Configuration for the decoder.
     */
    decoder: {
        /**
         * The url to the WebAssembly module.
         */
        wasmUrl?: string;
        /**
         * The url to the WebAssembly binary.
         */
        wasmBinaryUrl?: string;
        /**
         * The url to the fallback JavaScript module.
         */
        fallbackUrl?: string;
    };
}
/**
 * Draco compression (https://google.github.io/draco/)
 *
 * This class wraps the Draco module.
 *
 * **Encoder**
 *
 * The encoder is not currently implemented.
 *
 * **Decoder**
 *
 * By default, the configuration points to a copy of the Draco decoder files for glTF from the babylon.js preview cdn https://preview.babylonjs.com/draco_wasm_wrapper_gltf.js.
 *
 * To update the configuration, use the following code:
 * ```javascript
 *     DracoCompression.Configuration = {
 *         decoder: {
 *             wasmUrl: "<url to the WebAssembly library>",
 *             wasmBinaryUrl: "<url to the WebAssembly binary>",
 *             fallbackUrl: "<url to the fallback JavaScript library>",
 *         }
 *     };
 * ```
 *
 * Draco has two versions, one for WebAssembly and one for JavaScript. The decoder configuration can be set to only support WebAssembly or only support the JavaScript version.
 * Decoding will automatically fallback to the JavaScript version if WebAssembly version is not configured or if WebAssembly is not supported by the browser.
 * Use `DracoCompression.DecoderAvailable` to determine if the decoder configuration is available for the current context.
 *
 * To decode Draco compressed data, get the default DracoCompression object and call decodeMeshAsync:
 * ```javascript
 *     var vertexData = await DracoCompression.Default.decodeMeshAsync(data);
 * ```
 *
 * @see https://playground.babylonjs.com/#DMZIBD#0
 */
export class DracoCompression implements IDisposable {
    private _workerPoolPromise?;
    private _decoderModulePromise?;
    /**
     * The configuration. Defaults to the following urls:
     * - wasmUrl: "https://preview.babylonjs.com/draco_wasm_wrapper_gltf.js"
     * - wasmBinaryUrl: "https://preview.babylonjs.com/draco_decoder_gltf.wasm"
     * - fallbackUrl: "https://preview.babylonjs.com/draco_decoder_gltf.js"
     */
    static Configuration: IDracoCompressionConfiguration;
    /**
     * Returns true if the decoder configuration is available.
     */
    static get DecoderAvailable(): boolean;
    /**
     * Default number of workers to create when creating the draco compression object.
     */
    static DefaultNumWorkers: number;
    private static GetDefaultNumWorkers;
    private static _Default;
    /**
     * Default instance for the draco compression object.
     */
    static get Default(): DracoCompression;
    /**
     * Constructor
     * @param numWorkers The number of workers for async operations. Specify `0` to disable web workers and run synchronously in the current context.
     */
    constructor(numWorkers?: number);
    /**
     * Stop all async operations and release resources.
     */
    dispose(): void;
    /**
     * Returns a promise that resolves when ready. Call this manually to ensure draco compression is ready before use.
     * @returns a promise that resolves when ready
     */
    whenReadyAsync(): Promise<void>;
    /**
     * Decode Draco compressed mesh data to vertex data.
     * @param data The ArrayBuffer or ArrayBufferView for the Draco compression data
     * @param attributes A map of attributes from vertex buffer kinds to Draco unique ids
     * @param dividers a list of optional dividers for normalization
     * @returns A promise that resolves with the decoded vertex data
     */
    decodeMeshAsync(data: ArrayBuffer | ArrayBufferView, attributes?: {
        [kind: string]: number;
    }, dividers?: {
        [kind: string]: number;
    }): Promise<VertexData>;
}

}
declare module "babylonjs/Meshes/Compression/index" {
export * from "babylonjs/Meshes/Compression/dracoCompression";
export * from "babylonjs/Meshes/Compression/meshoptCompression";

}
declare module "babylonjs/Meshes/Compression/meshoptCompression" {
import { IDisposable } from "babylonjs/scene";
/**
 * Configuration for meshoptimizer compression
 */
export interface IMeshoptCompressionConfiguration {
    /**
     * Configuration for the decoder.
     */
    decoder: {
        /**
         * The url to the meshopt decoder library.
         */
        url: string;
    };
}
/**
 * Meshopt compression (https://github.com/zeux/meshoptimizer)
 *
 * This class wraps the meshopt library from https://github.com/zeux/meshoptimizer/tree/master/js.
 *
 * **Encoder**
 *
 * The encoder is not currently implemented.
 *
 * **Decoder**
 *
 * By default, the configuration points to a copy of the meshopt files on the Babylon.js preview CDN (e.g. https://preview.babylonjs.com/meshopt_decoder.js).
 *
 * To update the configuration, use the following code:
 * ```javascript
 *     MeshoptCompression.Configuration = {
 *         decoder: {
 *             url: "<url to the meshopt decoder library>"
 *         }
 *     };
 * ```
 */
export class MeshoptCompression implements IDisposable {
    private _decoderModulePromise?;
    /**
     * The configuration. Defaults to the following:
     * ```javascript
     * decoder: {
     *   url: "https://preview.babylonjs.com/meshopt_decoder.js"
     * }
     * ```
     */
    static Configuration: IMeshoptCompressionConfiguration;
    private static _Default;
    /**
     * Default instance for the meshoptimizer object.
     */
    static get Default(): MeshoptCompression;
    /**
     * Constructor
     */
    constructor();
    /**
     * Stop all async operations and release resources.
     */
    dispose(): void;
    /**
     * Decode meshopt data.
     * @see https://github.com/zeux/meshoptimizer/tree/master/js#decoder
     * @param source The input data.
     * @param count The number of elements.
     * @param stride The stride in bytes.
     * @param mode The compression mode.
     * @param filter The compression filter.
     * @returns a Promise<Uint8Array> that resolves to the decoded data
     */
    decodeGltfBufferAsync(source: Uint8Array, count: number, stride: number, mode: "ATTRIBUTES" | "TRIANGLES" | "INDICES", filter?: string): Promise<Uint8Array>;
}

}
declare module "babylonjs/Meshes/csg" {
import { Nullable } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
import { Quaternion, Matrix, Vector3 } from "babylonjs/Maths/math.vector";
import { Mesh } from "babylonjs/Meshes/mesh";
import { Material } from "babylonjs/Materials/material";
/**
 * Class for building Constructive Solid Geometry
 */
export class CSG {
    private _polygons;
    /**
     * The world matrix
     */
    matrix: Matrix;
    /**
     * Stores the position
     */
    position: Vector3;
    /**
     * Stores the rotation
     */
    rotation: Vector3;
    /**
     * Stores the rotation quaternion
     */
    rotationQuaternion: Nullable<Quaternion>;
    /**
     * Stores the scaling vector
     */
    scaling: Vector3;
    /**
     * Convert the Mesh to CSG
     * @param mesh The Mesh to convert to CSG
     * @param absolute If true, the final (local) matrix transformation is set to the identity and not to that of `mesh`. It can help when dealing with right-handed meshes (default: false)
     * @returns A new CSG from the Mesh
     */
    static FromMesh(mesh: Mesh, absolute?: boolean): CSG;
    /**
     * Construct a CSG solid from a list of `CSG.Polygon` instances.
     * @param polygons Polygons used to construct a CSG solid
     */
    private static _FromPolygons;
    /**
     * Clones, or makes a deep copy, of the CSG
     * @returns A new CSG
     */
    clone(): CSG;
    /**
     * Unions this CSG with another CSG
     * @param csg The CSG to union against this CSG
     * @returns The unioned CSG
     */
    union(csg: CSG): CSG;
    /**
     * Unions this CSG with another CSG in place
     * @param csg The CSG to union against this CSG
     */
    unionInPlace(csg: CSG): void;
    /**
     * Subtracts this CSG with another CSG
     * @param csg The CSG to subtract against this CSG
     * @returns A new CSG
     */
    subtract(csg: CSG): CSG;
    /**
     * Subtracts this CSG with another CSG in place
     * @param csg The CSG to subtract against this CSG
     */
    subtractInPlace(csg: CSG): void;
    /**
     * Intersect this CSG with another CSG
     * @param csg The CSG to intersect against this CSG
     * @returns A new CSG
     */
    intersect(csg: CSG): CSG;
    /**
     * Intersects this CSG with another CSG in place
     * @param csg The CSG to intersect against this CSG
     */
    intersectInPlace(csg: CSG): void;
    /**
     * Return a new CSG solid with solid and empty space switched. This solid is
     * not modified.
     * @returns A new CSG solid with solid and empty space switched
     */
    inverse(): CSG;
    /**
     * Inverses the CSG in place
     */
    inverseInPlace(): void;
    /**
     * This is used to keep meshes transformations so they can be restored
     * when we build back a Babylon Mesh
     * NB : All CSG operations are performed in world coordinates
     * @param csg The CSG to copy the transform attributes from
     * @returns This CSG
     */
    copyTransformAttributes(csg: CSG): CSG;
    /**
     * Build Raw mesh from CSG
     * Coordinates here are in world space
     * @param name The name of the mesh geometry
     * @param scene The Scene
     * @param keepSubMeshes Specifies if the submeshes should be kept
     * @returns A new Mesh
     */
    buildMeshGeometry(name: string, scene?: Scene, keepSubMeshes?: boolean): Mesh;
    /**
     * Build Mesh from CSG taking material and transforms into account
     * @param name The name of the Mesh
     * @param material The material of the Mesh
     * @param scene The Scene
     * @param keepSubMeshes Specifies if submeshes should be kept
     * @returns The new Mesh
     */
    toMesh(name: string, material?: Nullable<Material>, scene?: Scene, keepSubMeshes?: boolean): Mesh;
}

}
declare module "babylonjs/Meshes/geodesicMesh" {
import { Vector3 } from "babylonjs/Maths/math.vector";
import { _IsoVector } from "babylonjs/Maths/math.isovector";
/**
 * Class representing data for one face OAB of an equilateral icosahedron
 * When O is the isovector (0, 0), A is isovector (m, n)
 * @internal
 */
export class _PrimaryIsoTriangle {
    m: number;
    n: number;
    cartesian: Vector3[];
    vertices: _IsoVector[];
    max: number[];
    min: number[];
    vecToidx: {
        [key: string]: number;
    };
    vertByDist: {
        [key: string]: number[];
    };
    closestTo: number[][];
    innerFacets: string[][];
    isoVecsABOB: _IsoVector[][];
    isoVecsOBOA: _IsoVector[][];
    isoVecsBAOA: _IsoVector[][];
    vertexTypes: number[][];
    coau: number;
    cobu: number;
    coav: number;
    cobv: number;
    IDATA: PolyhedronData;
    /**
     * Creates the PrimaryIsoTriangle Triangle OAB
     * @param m an integer
     * @param n an integer
     */
    setIndices(): void;
    calcCoeffs(): void;
    createInnerFacets(): void;
    edgeVecsABOB(): void;
    mapABOBtoOBOA(): void;
    mapABOBtoBAOA(): void;
    MapToFace(faceNb: number, geodesicData: PolyhedronData): void;
    /**Creates a primary triangle
     * @internal
     */
    build(m: number, n: number): this;
}
/** Builds Polyhedron Data
 * @internal
 */
export class PolyhedronData {
    name: string;
    category: string;
    vertex: number[][];
    face: number[][];
    edgematch: (number | string)[][];
    constructor(name: string, category: string, vertex: number[][], face: number[][]);
}
/**
 * This class Extends the PolyhedronData Class to provide measures for a Geodesic Polyhedron
 */
export class GeodesicData extends PolyhedronData {
    /**
     * @internal
     */
    edgematch: (number | string)[][];
    /**
     * @internal
     */
    adjacentFaces: number[][];
    /**
     * @internal
     */
    sharedNodes: number;
    /**
     * @internal
     */
    poleNodes: number;
    /**
     * @internal
     */
    innerToData(face: number, primTri: _PrimaryIsoTriangle): void;
    /**
     * @internal
     */
    mapABOBtoDATA(faceNb: number, primTri: _PrimaryIsoTriangle): void;
    /**
     * @internal
     */
    mapOBOAtoDATA(faceNb: number, primTri: _PrimaryIsoTriangle): void;
    /**
     * @internal
     */
    mapBAOAtoDATA(faceNb: number, primTri: _PrimaryIsoTriangle): void;
    /**
     * @internal
     */
    orderData(primTri: _PrimaryIsoTriangle): void;
    /**
     * @internal
     */
    setOrder(m: number, faces: number[]): number[];
    /**
     * @internal
     */
    toGoldbergPolyhedronData(): PolyhedronData;
    /**Builds the data for a Geodesic Polyhedron from a primary triangle
     * @param primTri the primary triangle
     * @internal
     */
    static BuildGeodesicData(primTri: _PrimaryIsoTriangle): GeodesicData;
}

}
declare module "babylonjs/Meshes/geometry" {
import { Nullable, FloatArray, DataArray, IndicesArray } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
import { Vector2 } from "babylonjs/Maths/math.vector";
import { Vector3 } from "babylonjs/Maths/math.vector";
import { Engine } from "babylonjs/Engines/engine";
import { IGetSetVerticesData } from "babylonjs/Meshes/mesh.vertexData";
import { VertexData } from "babylonjs/Meshes/mesh.vertexData";
import { VertexBuffer } from "babylonjs/Buffers/buffer";
import { Effect } from "babylonjs/Materials/effect";
import { BoundingInfo } from "babylonjs/Culling/boundingInfo";
import { DataBuffer } from "babylonjs/Buffers/dataBuffer";
import { AbstractScene } from "babylonjs/abstractScene";
import { Mesh } from "babylonjs/Meshes/mesh";
/**
 * Class used to store geometry data (vertex buffers + index buffer)
 */
export class Geometry implements IGetSetVerticesData {
    /**
     * Gets or sets the ID of the geometry
     */
    id: string;
    /**
     * Gets or sets the unique ID of the geometry
     */
    uniqueId: number;
    /**
     * Gets the delay loading state of the geometry (none by default which means not delayed)
     */
    delayLoadState: number;
    /**
     * Gets the file containing the data to load when running in delay load state
     */
    delayLoadingFile: Nullable<string>;
    /**
     * Callback called when the geometry is updated
     */
    onGeometryUpdated: (geometry: Geometry, kind?: string) => void;
    private _scene;
    private _engine;
    private _meshes;
    private _totalVertices;
    /** @internal */
    _loadedUniqueId: string;
    /** @internal */
    _indices: IndicesArray;
    /** @internal */
    _vertexBuffers: {
        [key: string]: VertexBuffer;
    };
    private _isDisposed;
    private _extend;
    private _boundingBias;
    /** @internal */
    _delayInfo: Array<string>;
    private _indexBuffer;
    private _indexBufferIsUpdatable;
    /** @internal */
    _boundingInfo: Nullable<BoundingInfo>;
    /** @internal */
    _delayLoadingFunction: Nullable<(any: any, geometry: Geometry) => void>;
    /** @internal */
    _softwareSkinningFrameId: number;
    private _vertexArrayObjects;
    private _updatable;
    /** @internal */
    _positions: Nullable<Vector3[]>;
    private _positionsCache;
    /** @internal */
    _parentContainer: Nullable<AbstractScene>;
    /**
     *  Gets or sets the Bias Vector to apply on the bounding elements (box/sphere), the max extend is computed as v += v * bias.x + bias.y, the min is computed as v -= v * bias.x + bias.y
     */
    get boundingBias(): Vector2;
    /**
     *  Gets or sets the Bias Vector to apply on the bounding elements (box/sphere), the max extend is computed as v += v * bias.x + bias.y, the min is computed as v -= v * bias.x + bias.y
     */
    set boundingBias(value: Vector2);
    /**
     * Static function used to attach a new empty geometry to a mesh
     * @param mesh defines the mesh to attach the geometry to
     * @returns the new Geometry
     */
    static CreateGeometryForMesh(mesh: Mesh): Geometry;
    /** Get the list of meshes using this geometry */
    get meshes(): Mesh[];
    /**
     * If set to true (false by default), the bounding info applied to the meshes sharing this geometry will be the bounding info defined at the class level
     * and won't be computed based on the vertex positions (which is what we get when useBoundingInfoFromGeometry = false)
     */
    useBoundingInfoFromGeometry: boolean;
    /**
     * Creates a new geometry
     * @param id defines the unique ID
     * @param scene defines the hosting scene
     * @param vertexData defines the VertexData used to get geometry data
     * @param updatable defines if geometry must be updatable (false by default)
     * @param mesh defines the mesh that will be associated with the geometry
     */
    constructor(id: string, scene?: Scene, vertexData?: VertexData, updatable?: boolean, mesh?: Nullable<Mesh>);
    /**
     * Gets the current extend of the geometry
     */
    get extend(): {
        minimum: Vector3;
        maximum: Vector3;
    };
    /**
     * Gets the hosting scene
     * @returns the hosting Scene
     */
    getScene(): Scene;
    /**
     * Gets the hosting engine
     * @returns the hosting Engine
     */
    getEngine(): Engine;
    /**
     * Defines if the geometry is ready to use
     * @returns true if the geometry is ready to be used
     */
    isReady(): boolean;
    /**
     * Gets a value indicating that the geometry should not be serialized
     */
    get doNotSerialize(): boolean;
    /** @internal */
    _rebuild(): void;
    /**
     * Affects all geometry data in one call
     * @param vertexData defines the geometry data
     * @param updatable defines if the geometry must be flagged as updatable (false as default)
     */
    setAllVerticesData(vertexData: VertexData, updatable?: boolean): void;
    /**
     * Set specific vertex data
     * @param kind defines the data kind (Position, normal, etc...)
     * @param data defines the vertex data to use
     * @param updatable defines if the vertex must be flagged as updatable (false as default)
     * @param stride defines the stride to use (0 by default). This value is deduced from the kind value if not specified
     */
    setVerticesData(kind: string, data: FloatArray, updatable?: boolean, stride?: number): void;
    /**
     * Removes a specific vertex data
     * @param kind defines the data kind (Position, normal, etc...)
     */
    removeVerticesData(kind: string): void;
    /**
     * Affect a vertex buffer to the geometry. the vertexBuffer.getKind() function is used to determine where to store the data
     * @param buffer defines the vertex buffer to use
     * @param totalVertices defines the total number of vertices for position kind (could be null)
     * @param disposeExistingBuffer disposes the existing buffer, if any (default: true)
     */
    setVerticesBuffer(buffer: VertexBuffer, totalVertices?: Nullable<number>, disposeExistingBuffer?: boolean): void;
    /**
     * Update a specific vertex buffer
     * This function will directly update the underlying DataBuffer according to the passed numeric array or Float32Array
     * It will do nothing if the buffer is not updatable
     * @param kind defines the data kind (Position, normal, etc...)
     * @param data defines the data to use
     * @param offset defines the offset in the target buffer where to store the data
     * @param useBytes set to true if the offset is in bytes
     */
    updateVerticesDataDirectly(kind: string, data: DataArray, offset: number, useBytes?: boolean): void;
    /**
     * Update a specific vertex buffer
     * This function will create a new buffer if the current one is not updatable
     * @param kind defines the data kind (Position, normal, etc...)
     * @param data defines the data to use
     * @param updateExtends defines if the geometry extends must be recomputed (false by default)
     */
    updateVerticesData(kind: string, data: FloatArray, updateExtends?: boolean): void;
    private _updateBoundingInfo;
    /**
     * @internal
     */
    _bind(effect: Nullable<Effect>, indexToBind?: Nullable<DataBuffer>, overrideVertexBuffers?: {
        [kind: string]: Nullable<VertexBuffer>;
    }, overrideVertexArrayObjects?: {
        [key: string]: WebGLVertexArrayObject;
    }): void;
    /**
     * Gets total number of vertices
     * @returns the total number of vertices
     */
    getTotalVertices(): number;
    /**
     * Gets a specific vertex data attached to this geometry. Float data is constructed if the vertex buffer data cannot be returned directly.
     * @param kind defines the data kind (Position, normal, etc...)
     * @param copyWhenShared defines if the returned array must be cloned upon returning it if the current geometry is shared between multiple meshes
     * @param forceCopy defines a boolean indicating that the returned array must be cloned upon returning it
     * @returns a float array containing vertex data
     */
    getVerticesData(kind: string, copyWhenShared?: boolean, forceCopy?: boolean): Nullable<FloatArray>;
    /**
     * Returns a boolean defining if the vertex data for the requested `kind` is updatable
     * @param kind defines the data kind (Position, normal, etc...)
     * @returns true if the vertex buffer with the specified kind is updatable
     */
    isVertexBufferUpdatable(kind: string): boolean;
    /**
     * Gets a specific vertex buffer
     * @param kind defines the data kind (Position, normal, etc...)
     * @returns a VertexBuffer
     */
    getVertexBuffer(kind: string): Nullable<VertexBuffer>;
    /**
     * Returns all vertex buffers
     * @returns an object holding all vertex buffers indexed by kind
     */
    getVertexBuffers(): Nullable<{
        [key: string]: VertexBuffer;
    }>;
    /**
     * Gets a boolean indicating if specific vertex buffer is present
     * @param kind defines the data kind (Position, normal, etc...)
     * @returns true if data is present
     */
    isVerticesDataPresent(kind: string): boolean;
    /**
     * Gets a list of all attached data kinds (Position, normal, etc...)
     * @returns a list of string containing all kinds
     */
    getVerticesDataKinds(): string[];
    /**
     * Update index buffer
     * @param indices defines the indices to store in the index buffer
     * @param offset defines the offset in the target buffer where to store the data
     * @param gpuMemoryOnly defines a boolean indicating that only the GPU memory must be updated leaving the CPU version of the indices unchanged (false by default)
     */
    updateIndices(indices: IndicesArray, offset?: number, gpuMemoryOnly?: boolean): void;
    /**
     * Creates a new index buffer
     * @param indices defines the indices to store in the index buffer
     * @param totalVertices defines the total number of vertices (could be null)
     * @param updatable defines if the index buffer must be flagged as updatable (false by default)
     */
    setIndices(indices: IndicesArray, totalVertices?: Nullable<number>, updatable?: boolean): void;
    /**
     * Return the total number of indices
     * @returns the total number of indices
     */
    getTotalIndices(): number;
    /**
     * Gets the index buffer array
     * @param copyWhenShared defines if the returned array must be cloned upon returning it if the current geometry is shared between multiple meshes
     * @param forceCopy defines a boolean indicating that the returned array must be cloned upon returning it
     * @returns the index buffer array
     */
    getIndices(copyWhenShared?: boolean, forceCopy?: boolean): Nullable<IndicesArray>;
    /**
     * Gets the index buffer
     * @returns the index buffer
     */
    getIndexBuffer(): Nullable<DataBuffer>;
    /**
     * @internal
     */
    _releaseVertexArrayObject(effect?: Nullable<Effect>): void;
    /**
     * Release the associated resources for a specific mesh
     * @param mesh defines the source mesh
     * @param shouldDispose defines if the geometry must be disposed if there is no more mesh pointing to it
     */
    releaseForMesh(mesh: Mesh, shouldDispose?: boolean): void;
    /**
     * Apply current geometry to a given mesh
     * @param mesh defines the mesh to apply geometry to
     */
    applyToMesh(mesh: Mesh): void;
    private _updateExtend;
    private _applyToMesh;
    private _notifyUpdate;
    /**
     * Load the geometry if it was flagged as delay loaded
     * @param scene defines the hosting scene
     * @param onLoaded defines a callback called when the geometry is loaded
     */
    load(scene: Scene, onLoaded?: () => void): void;
    private _queueLoad;
    /**
     * Invert the geometry to move from a right handed system to a left handed one.
     */
    toLeftHanded(): void;
    /** @internal */
    _resetPointsArrayCache(): void;
    /** @internal */
    _generatePointsArray(): boolean;
    /**
     * Gets a value indicating if the geometry is disposed
     * @returns true if the geometry was disposed
     */
    isDisposed(): boolean;
    private _disposeVertexArrayObjects;
    /**
     * Free all associated resources
     */
    dispose(): void;
    /**
     * Clone the current geometry into a new geometry
     * @param id defines the unique ID of the new geometry
     * @returns a new geometry object
     */
    copy(id: string): Geometry;
    /**
     * Serialize the current geometry info (and not the vertices data) into a JSON object
     * @returns a JSON representation of the current geometry data (without the vertices data)
     */
    serialize(): any;
    private _toNumberArray;
    /**
     * Release any memory retained by the cached data on the Geometry.
     *
     * Call this function to reduce memory footprint of the mesh.
     * Vertex buffers will not store CPU data anymore (this will prevent picking, collisions or physics to work correctly)
     */
    clearCachedData(): void;
    /**
     * Serialize all vertices data into a JSON object
     * @returns a JSON representation of the current geometry data
     */
    serializeVerticeData(): any;
    /**
     * Extracts a clone of a mesh geometry
     * @param mesh defines the source mesh
     * @param id defines the unique ID of the new geometry object
     * @returns the new geometry object
     */
    static ExtractFromMesh(mesh: Mesh, id: string): Nullable<Geometry>;
    /**
     * You should now use Tools.RandomId(), this method is still here for legacy reasons.
     * Implementation from http://stackoverflow.com/questions/105034/how-to-create-a-guid-uuid-in-javascript/2117523#answer-2117523
     * Be aware Math.random() could cause collisions, but:
     * "All but 6 of the 128 bits of the ID are randomly generated, which means that for any two ids, there's a 1 in 2^^122 (or 5.3x10^^36) chance they'll collide"
     * @returns a string containing a new GUID
     */
    static RandomId(): string;
    private static _GetGeometryByLoadedUniqueId;
    /**
     * @internal
     */
    static _ImportGeometry(parsedGeometry: any, mesh: Mesh): void;
    private static _CleanMatricesWeights;
    /**
     * Create a new geometry from persisted data (Using .babylon file format)
     * @param parsedVertexData defines the persisted data
     * @param scene defines the hosting scene
     * @param rootUrl defines the root url to use to load assets (like delayed data)
     * @returns the new geometry object
     */
    static Parse(parsedVertexData: any, scene: Scene, rootUrl: string): Nullable<Geometry>;
}
export {};

}
declare module "babylonjs/Meshes/goldbergMesh" {
import { Scene } from "babylonjs/scene";
import { Vector2 } from "babylonjs/Maths/math.vector";
import { Vector3 } from "babylonjs/Maths/math.vector";
import { Mesh } from "babylonjs/Meshes/mesh";
import { Color4 } from "babylonjs/Maths/math.color";
/**
 * Defines the set of goldberg data used to create the polygon
 */
export type GoldbergData = {
    /**
     * The list of Goldberg faces colors
     */
    faceColors: Color4[];
    /**
     * The list of Goldberg faces centers
     */
    faceCenters: Vector3[];
    /**
     * The list of Goldberg faces Z axis
     */
    faceZaxis: Vector3[];
    /**
     * The list of Goldberg faces Y axis
     */
    faceXaxis: Vector3[];
    /**
     * The list of Goldberg faces X axis
     */
    faceYaxis: Vector3[];
    /**
     * Defines the number of shared faces
     */
    nbSharedFaces: number;
    /**
     * Defines the number of unshared faces
     */
    nbUnsharedFaces: number;
    /**
     * Defines the total number of goldberg faces
     */
    nbFaces: number;
    /**
     * Defines the number of goldberg faces at the pole
     */
    nbFacesAtPole: number;
    /**
     * Defines the number of adjacent faces per goldberg faces
     */
    adjacentFaces: number[][];
};
/**
 * Mesh for a Goldberg Polyhedron which is made from 12 pentagonal and the rest hexagonal faces
 * @see https://en.wikipedia.org/wiki/Goldberg_polyhedron
 */
export class GoldbergMesh extends Mesh {
    /**
     * Defines the specific Goldberg data used in this mesh construction.
     */
    goldbergData: GoldbergData;
    /**
     * Gets the related Goldberg face from pole infos
     * @param poleOrShared Defines the pole index or the shared face index if the fromPole parameter is passed in
     * @param fromPole Defines an optional pole index to find the related info from
     * @returns the goldberg face number
     */
    relatedGoldbergFace(poleOrShared: number, fromPole?: number): number;
    private _changeGoldbergFaceColors;
    /**
     * Set new goldberg face colors
     * @param colorRange the new color to apply to the mesh
     */
    setGoldbergFaceColors(colorRange: (number | Color4)[][]): void;
    /**
     * Updates new goldberg face colors
     * @param colorRange the new color to apply to the mesh
     */
    updateGoldbergFaceColors(colorRange: (number | Color4)[][]): void;
    private _changeGoldbergFaceUVs;
    /**
     * set new goldberg face UVs
     * @param uvRange the new UVs to apply to the mesh
     */
    setGoldbergFaceUVs(uvRange: (number | Vector2)[][]): void;
    /**
     * Updates new goldberg face UVs
     * @param uvRange the new UVs to apply to the mesh
     */
    updateGoldbergFaceUVs(uvRange: (number | Vector2)[][]): void;
    /**
     * Places a mesh on a particular face of the goldberg polygon
     * @param mesh Defines the mesh to position
     * @param face Defines the face to position onto
     * @param position Defines the position relative to the face we are positioning the mesh onto
     */
    placeOnGoldbergFaceAt(mesh: Mesh, face: number, position: Vector3): void;
    /**
     * Serialize current mesh
     * @param serializationObject defines the object which will receive the serialization data
     */
    serialize(serializationObject: any): void;
    /**
     * Parses a serialized goldberg mesh
     * @param parsedMesh the serialized mesh
     * @param scene the scene to create the goldberg mesh in
     * @returns the created goldberg mesh
     */
    static Parse(parsedMesh: any, scene: Scene): GoldbergMesh;
}

}
declare module "babylonjs/Meshes/groundMesh" {
import { Scene } from "babylonjs/scene";
import { Vector3 } from "babylonjs/Maths/math.vector";
import { Mesh } from "babylonjs/Meshes/mesh";
/**
 * Mesh representing the ground
 */
export class GroundMesh extends Mesh {
    /** If octree should be generated */
    generateOctree: boolean;
    private _heightQuads;
    /** @internal */
    _subdivisionsX: number;
    /** @internal */
    _subdivisionsY: number;
    /** @internal */
    _width: number;
    /** @internal */
    _height: number;
    /** @internal */
    _minX: number;
    /** @internal */
    _maxX: number;
    /** @internal */
    _minZ: number;
    /** @internal */
    _maxZ: number;
    constructor(name: string, scene?: Scene);
    /**
     * "GroundMesh"
     * @returns "GroundMesh"
     */
    getClassName(): string;
    /**
     * The minimum of x and y subdivisions
     */
    get subdivisions(): number;
    /**
     * X subdivisions
     */
    get subdivisionsX(): number;
    /**
     * Y subdivisions
     */
    get subdivisionsY(): number;
    /**
     * This function will divide the mesh into submeshes and update an octree to help to select the right submeshes
     * for rendering, picking and collision computations. Please note that you must have a decent number of submeshes
     * to get performance improvements when using an octree.
     * @param chunksCount the number of submeshes the mesh will be divided into
     * @param octreeBlocksSize the maximum size of the octree blocks (Default: 32)
     */
    optimize(chunksCount: number, octreeBlocksSize?: number): void;
    /**
     * Returns a height (y) value in the World system :
     * the ground altitude at the coordinates (x, z) expressed in the World system.
     * @param x x coordinate
     * @param z z coordinate
     * @returns the ground y position if (x, z) are outside the ground surface.
     */
    getHeightAtCoordinates(x: number, z: number): number;
    /**
     * Returns a normalized vector (Vector3) orthogonal to the ground
     * at the ground coordinates (x, z) expressed in the World system.
     * @param x x coordinate
     * @param z z coordinate
     * @returns Vector3(0.0, 1.0, 0.0) if (x, z) are outside the ground surface.
     */
    getNormalAtCoordinates(x: number, z: number): Vector3;
    /**
     * Updates the Vector3 passed a reference with a normalized vector orthogonal to the ground
     * at the ground coordinates (x, z) expressed in the World system.
     * Doesn't update the reference Vector3 if (x, z) are outside the ground surface.
     * @param x x coordinate
     * @param z z coordinate
     * @param ref vector to store the result
     * @returns the GroundMesh.
     */
    getNormalAtCoordinatesToRef(x: number, z: number, ref: Vector3): GroundMesh;
    /**
     * Force the heights to be recomputed for getHeightAtCoordinates() or getNormalAtCoordinates()
     * if the ground has been updated.
     * This can be used in the render loop.
     * @returns the GroundMesh.
     */
    updateCoordinateHeights(): GroundMesh;
    private _getFacetAt;
    private _initHeightQuads;
    private _computeHeightQuads;
    /**
     * Serializes this ground mesh
     * @param serializationObject object to write serialization to
     */
    serialize(serializationObject: any): void;
    /**
     * Parses a serialized ground mesh
     * @param parsedMesh the serialized mesh
     * @param scene the scene to create the ground mesh in
     * @returns the created ground mesh
     */
    static Parse(parsedMesh: any, scene: Scene): GroundMesh;
}

}
declare module "babylonjs/Meshes/index" {
export * from "babylonjs/Meshes/abstractMesh";
export * from "babylonjs/Meshes/Compression/index";
export * from "babylonjs/Meshes/csg";
export * from "babylonjs/Meshes/geometry";
export * from "babylonjs/Meshes/groundMesh";
export * from "babylonjs/Meshes/goldbergMesh";
export * from "babylonjs/Meshes/trailMesh";
export * from "babylonjs/Meshes/instancedMesh";
export * from "babylonjs/Meshes/linesMesh";
export * from "babylonjs/Meshes/mesh";
export * from "babylonjs/Meshes/mesh.vertexData";
export * from "babylonjs/Meshes/meshBuilder";
export * from "babylonjs/Meshes/meshSimplification";
export * from "babylonjs/Meshes/meshSimplificationSceneComponent";
export * from "babylonjs/Meshes/polygonMesh";
export * from "babylonjs/Meshes/geodesicMesh";
export * from "babylonjs/Meshes/subMesh";
export * from "babylonjs/Meshes/subMesh.project";
export * from "babylonjs/Meshes/meshLODLevel";
export * from "babylonjs/Meshes/transformNode";
export * from "babylonjs/Meshes/Builders/index";
export * from "babylonjs/Meshes/WebGL/webGLDataBuffer";
export * from "babylonjs/Meshes/WebGPU/webgpuDataBuffer";
import "babylonjs/Meshes/thinInstanceMesh";
export * from "babylonjs/Meshes/thinInstanceMesh";

}
declare module "babylonjs/Meshes/instancedMesh" {
import { Nullable, FloatArray, IndicesArray } from "babylonjs/types";
import { Vector3 } from "babylonjs/Maths/math.vector";
import { Matrix } from "babylonjs/Maths/math.vector";
import { Camera } from "babylonjs/Cameras/camera";
import { Node } from "babylonjs/node";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { Mesh } from "babylonjs/Meshes/mesh";
import { Material } from "babylonjs/Materials/material";
import { Skeleton } from "babylonjs/Bones/skeleton";
import { TransformNode } from "babylonjs/Meshes/transformNode";
import { Light } from "babylonjs/Lights/light";
import { VertexBuffer } from "babylonjs/Buffers/buffer";
/**
 * Creates an instance based on a source mesh.
 */
export class InstancedMesh extends AbstractMesh {
    private _sourceMesh;
    private _currentLOD;
    private _billboardWorldMatrix;
    /** @internal */
    _indexInSourceMeshInstanceArray: number;
    /** @internal */
    _distanceToCamera: number;
    /** @internal */
    _previousWorldMatrix: Nullable<Matrix>;
    constructor(name: string, source: Mesh);
    /**
     * Returns the string "InstancedMesh".
     */
    getClassName(): string;
    /** Gets the list of lights affecting that mesh */
    get lightSources(): Light[];
    _resyncLightSources(): void;
    _resyncLightSource(): void;
    _removeLightSource(): void;
    /**
     * If the source mesh receives shadows
     */
    get receiveShadows(): boolean;
    /**
     * The material of the source mesh
     */
    get material(): Nullable<Material>;
    /**
     * Visibility of the source mesh
     */
    get visibility(): number;
    /**
     * Skeleton of the source mesh
     */
    get skeleton(): Nullable<Skeleton>;
    /**
     * Rendering ground id of the source mesh
     */
    get renderingGroupId(): number;
    set renderingGroupId(value: number);
    /**
     * Returns the total number of vertices (integer).
     */
    getTotalVertices(): number;
    /**
     * Returns a positive integer : the total number of indices in this mesh geometry.
     * @returns the number of indices or zero if the mesh has no geometry.
     */
    getTotalIndices(): number;
    /**
     * The source mesh of the instance
     */
    get sourceMesh(): Mesh;
    /**
     * Creates a new InstancedMesh object from the mesh model.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/copies/instances
     * @param name defines the name of the new instance
     * @returns a new InstancedMesh
     */
    createInstance(name: string): InstancedMesh;
    /**
     * Is this node ready to be used/rendered
     * @param completeCheck defines if a complete check (including materials and lights) has to be done (false by default)
     * @returns {boolean} is it ready
     */
    isReady(completeCheck?: boolean): boolean;
    /**
     * Returns an array of integers or a typed array (Int32Array, Uint32Array, Uint16Array) populated with the mesh indices.
     * @param kind kind of verticies to retrieve (eg. positions, normals, uvs, etc.)
     * @param copyWhenShared If true (default false) and and if the mesh geometry is shared among some other meshes, the returned array is a copy of the internal one.
     * @returns a float array or a Float32Array of the requested kind of data : positions, normals, uvs, etc.
     */
    getVerticesData(kind: string, copyWhenShared?: boolean): Nullable<FloatArray>;
    /**
     * Sets the vertex data of the mesh geometry for the requested `kind`.
     * If the mesh has no geometry, a new Geometry object is set to the mesh and then passed this vertex data.
     * The `data` are either a numeric array either a Float32Array.
     * The parameter `updatable` is passed as is to the underlying Geometry object constructor (if initially none) or updater.
     * The parameter `stride` is an optional positive integer, it is usually automatically deducted from the `kind` (3 for positions or normals, 2 for UV, etc).
     * Note that a new underlying VertexBuffer object is created each call.
     * If the `kind` is the `PositionKind`, the mesh BoundingInfo is renewed, so the bounding box and sphere, and the mesh World Matrix is recomputed.
     *
     * Possible `kind` values :
     * - VertexBuffer.PositionKind
     * - VertexBuffer.UVKind
     * - VertexBuffer.UV2Kind
     * - VertexBuffer.UV3Kind
     * - VertexBuffer.UV4Kind
     * - VertexBuffer.UV5Kind
     * - VertexBuffer.UV6Kind
     * - VertexBuffer.ColorKind
     * - VertexBuffer.MatricesIndicesKind
     * - VertexBuffer.MatricesIndicesExtraKind
     * - VertexBuffer.MatricesWeightsKind
     * - VertexBuffer.MatricesWeightsExtraKind
     *
     * Returns the Mesh.
     * @param kind
     * @param data
     * @param updatable
     * @param stride
     */
    setVerticesData(kind: string, data: FloatArray, updatable?: boolean, stride?: number): AbstractMesh;
    /**
     * Updates the existing vertex data of the mesh geometry for the requested `kind`.
     * If the mesh has no geometry, it is simply returned as it is.
     * The `data` are either a numeric array either a Float32Array.
     * No new underlying VertexBuffer object is created.
     * If the `kind` is the `PositionKind` and if `updateExtends` is true, the mesh BoundingInfo is renewed, so the bounding box and sphere, and the mesh World Matrix is recomputed.
     * If the parameter `makeItUnique` is true, a new global geometry is created from this positions and is set to the mesh.
     *
     * Possible `kind` values :
     * - VertexBuffer.PositionKind
     * - VertexBuffer.UVKind
     * - VertexBuffer.UV2Kind
     * - VertexBuffer.UV3Kind
     * - VertexBuffer.UV4Kind
     * - VertexBuffer.UV5Kind
     * - VertexBuffer.UV6Kind
     * - VertexBuffer.ColorKind
     * - VertexBuffer.MatricesIndicesKind
     * - VertexBuffer.MatricesIndicesExtraKind
     * - VertexBuffer.MatricesWeightsKind
     * - VertexBuffer.MatricesWeightsExtraKind
     *
     * Returns the Mesh.
     * @param kind
     * @param data
     * @param updateExtends
     * @param makeItUnique
     */
    updateVerticesData(kind: string, data: FloatArray, updateExtends?: boolean, makeItUnique?: boolean): Mesh;
    /**
     * Sets the mesh indices.
     * Expects an array populated with integers or a typed array (Int32Array, Uint32Array, Uint16Array).
     * If the mesh has no geometry, a new Geometry object is created and set to the mesh.
     * This method creates a new index buffer each call.
     * Returns the Mesh.
     * @param indices
     * @param totalVertices
     */
    setIndices(indices: IndicesArray, totalVertices?: Nullable<number>): Mesh;
    /**
     * Boolean : True if the mesh owns the requested kind of data.
     * @param kind
     */
    isVerticesDataPresent(kind: string): boolean;
    /**
     * Returns an array of indices (IndicesArray).
     */
    getIndices(): Nullable<IndicesArray>;
    get _positions(): Nullable<Vector3[]>;
    /**
     * This method recomputes and sets a new BoundingInfo to the mesh unless it is locked.
     * This means the mesh underlying bounding box and sphere are recomputed.
     * @param applySkeleton defines whether to apply the skeleton before computing the bounding info
     * @param applyMorph  defines whether to apply the morph target before computing the bounding info
     * @returns the current mesh
     */
    refreshBoundingInfo(applySkeleton?: boolean, applyMorph?: boolean): InstancedMesh;
    /** @internal */
    _preActivate(): InstancedMesh;
    /**
     * @internal
     */
    _activate(renderId: number, intermediateRendering: boolean): boolean;
    /** @internal */
    _postActivate(): void;
    getWorldMatrix(): Matrix;
    get isAnInstance(): boolean;
    /**
     * Returns the current associated LOD AbstractMesh.
     * @param camera
     */
    getLOD(camera: Camera): AbstractMesh;
    /**
     * @internal
     */
    _preActivateForIntermediateRendering(renderId: number): Mesh;
    /** @internal */
    _syncSubMeshes(): InstancedMesh;
    /** @internal */
    _generatePointsArray(): boolean;
    /** @internal */
    _updateBoundingInfo(): AbstractMesh;
    /**
     * Creates a new InstancedMesh from the current mesh.
     *
     * Returns the clone.
     * @param name the cloned mesh name
     * @param newParent the optional Node to parent the clone to.
     * @param doNotCloneChildren if `true` the model children aren't cloned.
     * @param newSourceMesh if set this mesh will be used as the source mesh instead of ths instance's one
     * @returns the clone
     */
    clone(name: string, newParent?: Nullable<Node>, doNotCloneChildren?: boolean, newSourceMesh?: Mesh): InstancedMesh;
    /**
     * Disposes the InstancedMesh.
     * Returns nothing.
     * @param doNotRecurse
     * @param disposeMaterialAndTextures
     */
    dispose(doNotRecurse?: boolean, disposeMaterialAndTextures?: boolean): void;
    /**
     * @internal
     */
    _serializeAsParent(serializationObject: any): void;
    /**
     * Instantiate (when possible) or clone that node with its hierarchy
     * @param newParent defines the new parent to use for the instance (or clone)
     * @param options defines options to configure how copy is done
     * @param options.doNotInstantiate defines if the model must be instantiated or just cloned
     * @param options.newSourcedMesh newSourcedMesh the new source mesh for the instance (or clone)
     * @param onNewNodeCreated defines an option callback to call when a clone or an instance is created
     * @returns an instance (or a clone) of the current node with its hierarchy
     */
    instantiateHierarchy(newParent?: Nullable<TransformNode>, options?: {
        doNotInstantiate: boolean | ((node: TransformNode) => boolean);
        newSourcedMesh?: Mesh;
    }, onNewNodeCreated?: (source: TransformNode, clone: TransformNode) => void): Nullable<TransformNode>;
}
module "babylonjs/Meshes/mesh" {
    interface Mesh {
        /**
         * Register a custom buffer that will be instanced
         * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/copies/instances#custom-buffers
         * @param kind defines the buffer kind
         * @param stride defines the stride in floats
         */
        registerInstancedBuffer(kind: string, stride: number): void;
        /**
         * Invalidate VertexArrayObjects belonging to the mesh (but not to the Geometry of the mesh).
         */
        _invalidateInstanceVertexArrayObject(): void;
        /**
         * true to use the edge renderer for all instances of this mesh
         */
        edgesShareWithInstances: boolean;
        /** @internal */
        _userInstancedBuffersStorage: {
            data: {
                [key: string]: Float32Array;
            };
            sizes: {
                [key: string]: number;
            };
            vertexBuffers: {
                [key: string]: Nullable<VertexBuffer>;
            };
            strides: {
                [key: string]: number;
            };
            vertexArrayObjects?: {
                [key: string]: WebGLVertexArrayObject;
            };
        };
    }
}
module "babylonjs/Meshes/abstractMesh" {
    interface AbstractMesh {
        /**
         * Object used to store instanced buffers defined by user
         * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/copies/instances#custom-buffers
         */
        instancedBuffers: {
            [key: string]: any;
        };
    }
}

}
declare module "babylonjs/Meshes/linesMesh" {
import { Nullable } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
import { Color3 } from "babylonjs/Maths/math.color";
import { Node } from "babylonjs/node";
import { SubMesh } from "babylonjs/Meshes/subMesh";
import { Mesh } from "babylonjs/Meshes/mesh";
import { InstancedMesh } from "babylonjs/Meshes/instancedMesh";
import { Material } from "babylonjs/Materials/material";
import { Effect } from "babylonjs/Materials/effect";
import "babylonjs/Shaders/color.fragment";
import "babylonjs/Shaders/color.vertex";
/**
 * Line mesh
 * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/param
 */
export class LinesMesh extends Mesh {
    /**
     * If vertex color should be applied to the mesh
     */
    readonly useVertexColor?: boolean | undefined;
    /**
     * If vertex alpha should be applied to the mesh
     */
    readonly useVertexAlpha?: boolean | undefined;
    /**
     * Color of the line (Default: White)
     */
    color: Color3;
    /**
     * Alpha of the line (Default: 1)
     */
    alpha: number;
    /**
     * The intersection Threshold is the margin applied when intersection a segment of the LinesMesh with a Ray.
     * This margin is expressed in world space coordinates, so its value may vary.
     * Default value is 0.1
     */
    intersectionThreshold: number;
    private _lineMaterial;
    private _isShaderMaterial;
    private _color4;
    /**
     * Creates a new LinesMesh
     * @param name defines the name
     * @param scene defines the hosting scene
     * @param parent defines the parent mesh if any
     * @param source defines the optional source LinesMesh used to clone data from
     * @param doNotCloneChildren When cloning, skip cloning child meshes of source, default False.
     * When false, achieved by calling a clone(), also passing False.
     * This will make creation of children, recursive.
     * @param useVertexColor defines if this LinesMesh supports vertex color
     * @param useVertexAlpha defines if this LinesMesh supports vertex alpha
     * @param material material to use to draw the line. If not provided, will create a new one
     */
    constructor(name: string, scene?: Nullable<Scene>, parent?: Nullable<Node>, source?: Nullable<LinesMesh>, doNotCloneChildren?: boolean, 
    /**
     * If vertex color should be applied to the mesh
     */
    useVertexColor?: boolean | undefined, 
    /**
     * If vertex alpha should be applied to the mesh
     */
    useVertexAlpha?: boolean | undefined, material?: Material);
    isReady(): boolean;
    /**
     * Returns the string "LineMesh"
     */
    getClassName(): string;
    /**
     * @internal
     */
    get material(): Material;
    /**
     * @internal
     */
    set material(value: Material);
    /**
     * @internal
     */
    get checkCollisions(): boolean;
    set checkCollisions(value: boolean);
    /**
     * @internal
     */
    _bind(_subMesh: SubMesh, colorEffect: Effect): Mesh;
    /**
     * @internal
     */
    _draw(subMesh: SubMesh, fillMode: number, instancesCount?: number): Mesh;
    /**
     * Disposes of the line mesh
     * @param doNotRecurse If children should be disposed
     * @param disposeMaterialAndTextures This parameter is not used by the LineMesh class
     * @param doNotDisposeMaterial If the material should not be disposed (default: false, meaning the material is disposed)
     */
    dispose(doNotRecurse?: boolean, disposeMaterialAndTextures?: boolean, doNotDisposeMaterial?: boolean): void;
    /**
     * Returns a new LineMesh object cloned from the current one.
     * @param name
     * @param newParent
     * @param doNotCloneChildren
     */
    clone(name: string, newParent?: Nullable<Node>, doNotCloneChildren?: boolean): LinesMesh;
    /**
     * Creates a new InstancedLinesMesh object from the mesh model.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/copies/instances
     * @param name defines the name of the new instance
     * @returns a new InstancedLinesMesh
     */
    createInstance(name: string): InstancedLinesMesh;
    /**
     * Serializes this ground mesh
     * @param serializationObject object to write serialization to
     */
    serialize(serializationObject: any): void;
    /**
     * Parses a serialized ground mesh
     * @param parsedMesh the serialized mesh
     * @param scene the scene to create the ground mesh in
     * @returns the created ground mesh
     */
    static Parse(parsedMesh: any, scene: Scene): LinesMesh;
}
/**
 * Creates an instance based on a source LinesMesh
 */
export class InstancedLinesMesh extends InstancedMesh {
    /**
     * The intersection Threshold is the margin applied when intersection a segment of the LinesMesh with a Ray.
     * This margin is expressed in world space coordinates, so its value may vary.
     * Initialized with the intersectionThreshold value of the source LinesMesh
     */
    intersectionThreshold: number;
    constructor(name: string, source: LinesMesh);
    /**
     * Returns the string "InstancedLinesMesh".
     */
    getClassName(): string;
}

}
declare module "babylonjs/Meshes/mesh" {
import { Observable } from "babylonjs/Misc/observable";
import { IAnimatable } from "babylonjs/Animations/animatable.interface";
import { Nullable, FloatArray, IndicesArray } from "babylonjs/types";
import { Camera } from "babylonjs/Cameras/camera";
import { Scene } from "babylonjs/scene";
import { Matrix, Vector3, Vector2 } from "babylonjs/Maths/math.vector";
import { Engine } from "babylonjs/Engines/engine";
import { Node } from "babylonjs/node";
import { VertexBuffer, Buffer } from "babylonjs/Buffers/buffer";
import { IGetSetVerticesData } from "babylonjs/Meshes/mesh.vertexData";
import { Geometry } from "babylonjs/Meshes/geometry";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { SubMesh } from "babylonjs/Meshes/subMesh";
import { BoundingSphere } from "babylonjs/Culling/boundingSphere";
import { Effect } from "babylonjs/Materials/effect";
import { Material } from "babylonjs/Materials/material";
import { Skeleton } from "babylonjs/Bones/skeleton";
import { MeshLODLevel } from "babylonjs/Meshes/meshLODLevel";
import { Path3D } from "babylonjs/Maths/math.path";
import { Plane } from "babylonjs/Maths/math.plane";
import { TransformNode } from "babylonjs/Meshes/transformNode";
import { GoldbergMesh } from "babylonjs/Meshes/goldbergMesh";
import { InstancedMesh } from "babylonjs/Meshes/instancedMesh";
import { IPhysicsEnabledObject } from "babylonjs/Physics/v1/physicsImpostor";
import { PhysicsImpostor } from "babylonjs/Physics/v1/physicsImpostor";
/**
 * @internal
 **/
export class _CreationDataStorage {
    closePath?: boolean;
    closeArray?: boolean;
    idx: number[];
    dashSize: number;
    gapSize: number;
    path3D: Path3D;
    pathArray: Vector3[][];
    arc: number;
    radius: number;
    cap: number;
    tessellation: number;
}
/**
 * @internal
 **/
class _InstanceDataStorage {
    visibleInstances: any;
    batchCache: _InstancesBatch;
    batchCacheReplacementModeInFrozenMode: _InstancesBatch;
    instancesBufferSize: number;
    instancesBuffer: Nullable<Buffer>;
    instancesPreviousBuffer: Nullable<Buffer>;
    instancesData: Float32Array;
    instancesPreviousData: Float32Array;
    overridenInstanceCount: number;
    isFrozen: boolean;
    forceMatrixUpdates: boolean;
    previousBatch: Nullable<_InstancesBatch>;
    hardwareInstancedRendering: boolean;
    sideOrientation: number;
    manualUpdate: boolean;
    previousManualUpdate: boolean;
    previousRenderId: number;
    masterMeshPreviousWorldMatrix: Nullable<Matrix>;
}
/**
 * @internal
 **/
export class _InstancesBatch {
    mustReturn: boolean;
    visibleInstances: Nullable<import("babylonjs/Meshes/instancedMesh").InstancedMesh[]>[];
    renderSelf: boolean[];
    hardwareInstancedRendering: boolean[];
}
/**
 * @internal
 **/
class _ThinInstanceDataStorage {
    instancesCount: number;
    matrixBuffer: Nullable<Buffer>;
    previousMatrixBuffer: Nullable<Buffer>;
    matrixBufferSize: number;
    matrixData: Nullable<Float32Array>;
    previousMatrixData: Nullable<Float32Array>;
    boundingVectors: Array<Vector3>;
    worldMatrices: Nullable<Matrix[]>;
    masterMeshPreviousWorldMatrix: Nullable<Matrix>;
}
/**
 * Class used to represent renderable models
 */
export class Mesh extends AbstractMesh implements IGetSetVerticesData {
    /**
     * Mesh side orientation : usually the external or front surface
     */
    static readonly FRONTSIDE: number;
    /**
     * Mesh side orientation : usually the internal or back surface
     */
    static readonly BACKSIDE: number;
    /**
     * Mesh side orientation : both internal and external or front and back surfaces
     */
    static readonly DOUBLESIDE: number;
    /**
     * Mesh side orientation : by default, `FRONTSIDE`
     */
    static readonly DEFAULTSIDE: number;
    /**
     * Mesh cap setting : no cap
     */
    static readonly NO_CAP: number;
    /**
     * Mesh cap setting : one cap at the beginning of the mesh
     */
    static readonly CAP_START: number;
    /**
     * Mesh cap setting : one cap at the end of the mesh
     */
    static readonly CAP_END: number;
    /**
     * Mesh cap setting : two caps, one at the beginning  and one at the end of the mesh
     */
    static readonly CAP_ALL: number;
    /**
     * Mesh pattern setting : no flip or rotate
     */
    static readonly NO_FLIP: number;
    /**
     * Mesh pattern setting : flip (reflect in y axis) alternate tiles on each row or column
     */
    static readonly FLIP_TILE: number;
    /**
     * Mesh pattern setting : rotate (180degs) alternate tiles on each row or column
     */
    static readonly ROTATE_TILE: number;
    /**
     * Mesh pattern setting : flip (reflect in y axis) all tiles on alternate rows
     */
    static readonly FLIP_ROW: number;
    /**
     * Mesh pattern setting : rotate (180degs) all tiles on alternate rows
     */
    static readonly ROTATE_ROW: number;
    /**
     * Mesh pattern setting : flip and rotate alternate tiles on each row or column
     */
    static readonly FLIP_N_ROTATE_TILE: number;
    /**
     * Mesh pattern setting : rotate pattern and rotate
     */
    static readonly FLIP_N_ROTATE_ROW: number;
    /**
     * Mesh tile positioning : part tiles same on left/right or top/bottom
     */
    static readonly CENTER: number;
    /**
     * Mesh tile positioning : part tiles on left
     */
    static readonly LEFT: number;
    /**
     * Mesh tile positioning : part tiles on right
     */
    static readonly RIGHT: number;
    /**
     * Mesh tile positioning : part tiles on top
     */
    static readonly TOP: number;
    /**
     * Mesh tile positioning : part tiles on bottom
     */
    static readonly BOTTOM: number;
    /**
     * Indicates that the instanced meshes should be sorted from back to front before rendering if their material is transparent
     */
    static INSTANCEDMESH_SORT_TRANSPARENT: boolean;
    /**
     * Gets the default side orientation.
     * @param orientation the orientation to value to attempt to get
     * @returns the default orientation
     * @internal
     */
    static _GetDefaultSideOrientation(orientation?: number): number;
    private _internalMeshDataInfo;
    /**
     * Determines if the LOD levels are intended to be calculated using screen coverage (surface area ratio) instead of distance.
     */
    get useLODScreenCoverage(): boolean;
    set useLODScreenCoverage(value: boolean);
    /**
     * Will notify when the mesh is completely ready, including materials.
     * Observers added to this observable will be removed once triggered
     */
    onMeshReadyObservable: Observable<Mesh>;
    get computeBonesUsingShaders(): boolean;
    set computeBonesUsingShaders(value: boolean);
    /**
     * An event triggered before rendering the mesh
     */
    get onBeforeRenderObservable(): Observable<Mesh>;
    /**
     * An event triggered before binding the mesh
     */
    get onBeforeBindObservable(): Observable<Mesh>;
    /**
     * An event triggered after rendering the mesh
     */
    get onAfterRenderObservable(): Observable<Mesh>;
    /**
     * An event triggeredbetween rendering pass when using separateCullingPass = true
     */
    get onBetweenPassObservable(): Observable<SubMesh>;
    /**
     * An event triggered before drawing the mesh
     */
    get onBeforeDrawObservable(): Observable<Mesh>;
    private _onBeforeDrawObserver;
    /**
     * Sets a callback to call before drawing the mesh. It is recommended to use onBeforeDrawObservable instead
     */
    set onBeforeDraw(callback: () => void);
    get hasInstances(): boolean;
    get hasThinInstances(): boolean;
    /**
     * Gets the delay loading state of the mesh (when delay loading is turned on)
     * @see https://doc.babylonjs.com/features/featuresDeepDive/importers/incrementalLoading
     */
    delayLoadState: number;
    /**
     * Gets the list of instances created from this mesh
     * it is not supposed to be modified manually.
     * Note also that the order of the InstancedMesh wihin the array is not significant and might change.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/copies/instances
     */
    instances: import("babylonjs/Meshes/instancedMesh").InstancedMesh[];
    /**
     * Gets the file containing delay loading data for this mesh
     */
    delayLoadingFile: string;
    /** @internal */
    _binaryInfo: any;
    /**
     * User defined function used to change how LOD level selection is done
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/LOD
     */
    onLODLevelSelection: (distance: number, mesh: Mesh, selectedLevel: Nullable<Mesh>) => void;
    /** @internal */
    _creationDataStorage: Nullable<_CreationDataStorage>;
    /** @internal */
    _geometry: Nullable<Geometry>;
    /** @internal */
    _delayInfo: Array<string>;
    /** @internal */
    _delayLoadingFunction: (any: any, mesh: Mesh) => void;
    /**
     * Gets or sets the forced number of instances to display.
     * If 0 (default value), the number of instances is not forced and depends on the draw type
     * (regular / instance / thin instances mesh)
     */
    get forcedInstanceCount(): number;
    set forcedInstanceCount(count: number);
    /** @internal */
    _instanceDataStorage: _InstanceDataStorage;
    /** @internal */
    _thinInstanceDataStorage: _ThinInstanceDataStorage;
    /** @internal */
    _shouldGenerateFlatShading: boolean;
    /** @internal */
    _originalBuilderSideOrientation: number;
    /**
     * Use this property to change the original side orientation defined at construction time
     */
    overrideMaterialSideOrientation: Nullable<number>;
    /**
     * Gets or sets a boolean indicating whether to render ignoring the active camera's max z setting. (false by default)
     * Note this will reduce performance when set to true.
     */
    ignoreCameraMaxZ: boolean;
    /**
     * Gets the source mesh (the one used to clone this one from)
     */
    get source(): Nullable<Mesh>;
    /**
     * Gets the list of clones of this mesh
     * The scene must have been constructed with useClonedMeshMap=true for this to work!
     * Note that useClonedMeshMap=true is the default setting
     */
    get cloneMeshMap(): Nullable<{
        [id: string]: Mesh | undefined;
    }>;
    /**
     * Gets or sets a boolean indicating that this mesh does not use index buffer
     */
    get isUnIndexed(): boolean;
    set isUnIndexed(value: boolean);
    /** Gets the array buffer used to store the instanced buffer used for instances' world matrices */
    get worldMatrixInstancedBuffer(): Float32Array;
    /** Gets the array buffer used to store the instanced buffer used for instances' previous world matrices */
    get previousWorldMatrixInstancedBuffer(): Float32Array;
    /** Gets or sets a boolean indicating that the update of the instance buffer of the world matrices is manual */
    get manualUpdateOfWorldMatrixInstancedBuffer(): boolean;
    set manualUpdateOfWorldMatrixInstancedBuffer(value: boolean);
    /** Gets or sets a boolean indicating that the update of the instance buffer of the world matrices is manual */
    get manualUpdateOfPreviousWorldMatrixInstancedBuffer(): boolean;
    set manualUpdateOfPreviousWorldMatrixInstancedBuffer(value: boolean);
    /** Gets or sets a boolean indicating that the update of the instance buffer of the world matrices must be performed in all cases (and notably even in frozen mode) */
    get forceWorldMatrixInstancedBufferUpdate(): boolean;
    set forceWorldMatrixInstancedBufferUpdate(value: boolean);
    /**
     * @constructor
     * @param name The value used by scene.getMeshByName() to do a lookup.
     * @param scene The scene to add this mesh to.
     * @param parent The parent of this mesh, if it has one
     * @param source An optional Mesh from which geometry is shared, cloned.
     * @param doNotCloneChildren When cloning, skip cloning child meshes of source, default False.
     *                  When false, achieved by calling a clone(), also passing False.
     *                  This will make creation of children, recursive.
     * @param clonePhysicsImpostor When cloning, include cloning mesh physics impostor, default True.
     */
    constructor(name: string, scene?: Nullable<Scene>, parent?: Nullable<Node>, source?: Nullable<Mesh>, doNotCloneChildren?: boolean, clonePhysicsImpostor?: boolean);
    instantiateHierarchy(newParent?: Nullable<TransformNode>, options?: {
        doNotInstantiate: boolean | ((node: TransformNode) => boolean);
    }, onNewNodeCreated?: (source: TransformNode, clone: TransformNode) => void): Nullable<TransformNode>;
    /**
     * Gets the class name
     * @returns the string "Mesh".
     */
    getClassName(): string;
    /** @internal */
    get _isMesh(): boolean;
    /**
     * Returns a description of this mesh
     * @param fullDetails define if full details about this mesh must be used
     * @returns a descriptive string representing this mesh
     */
    toString(fullDetails?: boolean): string;
    /** @internal */
    _unBindEffect(): void;
    /**
     * Gets a boolean indicating if this mesh has LOD
     */
    get hasLODLevels(): boolean;
    /**
     * Gets the list of MeshLODLevel associated with the current mesh
     * @returns an array of MeshLODLevel
     */
    getLODLevels(): MeshLODLevel[];
    private _sortLODLevels;
    /**
     * Add a mesh as LOD level triggered at the given distance.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/LOD
     * @param distanceOrScreenCoverage Either distance from the center of the object to show this level or the screen coverage if `useScreenCoverage` is set to `true`.
     * If screen coverage, value is a fraction of the screen's total surface, between 0 and 1.
     * Example Playground for distance https://playground.babylonjs.com/#QE7KM#197
     * Example Playground for screen coverage https://playground.babylonjs.com/#QE7KM#196
     * @param mesh The mesh to be added as LOD level (can be null)
     * @returns This mesh (for chaining)
     */
    addLODLevel(distanceOrScreenCoverage: number, mesh: Nullable<Mesh>): Mesh;
    /**
     * Returns the LOD level mesh at the passed distance or null if not found.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/LOD
     * @param distance The distance from the center of the object to show this level
     * @returns a Mesh or `null`
     */
    getLODLevelAtDistance(distance: number): Nullable<Mesh>;
    /**
     * Remove a mesh from the LOD array
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/LOD
     * @param mesh defines the mesh to be removed
     * @returns This mesh (for chaining)
     */
    removeLODLevel(mesh: Nullable<Mesh>): Mesh;
    /**
     * Returns the registered LOD mesh distant from the parameter `camera` position if any, else returns the current mesh.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/LOD
     * @param camera defines the camera to use to compute distance
     * @param boundingSphere defines a custom bounding sphere to use instead of the one from this mesh
     * @returns This mesh (for chaining)
     */
    getLOD(camera: Camera, boundingSphere?: BoundingSphere): Nullable<AbstractMesh>;
    /**
     * Gets the mesh internal Geometry object
     */
    get geometry(): Nullable<Geometry>;
    /**
     * Returns the total number of vertices within the mesh geometry or zero if the mesh has no geometry.
     * @returns the total number of vertices
     */
    getTotalVertices(): number;
    /**
     * Returns the content of an associated vertex buffer
     * @param kind defines which buffer to read from (positions, indices, normals, etc). Possible `kind` values :
     * - VertexBuffer.PositionKind
     * - VertexBuffer.UVKind
     * - VertexBuffer.UV2Kind
     * - VertexBuffer.UV3Kind
     * - VertexBuffer.UV4Kind
     * - VertexBuffer.UV5Kind
     * - VertexBuffer.UV6Kind
     * - VertexBuffer.ColorKind
     * - VertexBuffer.MatricesIndicesKind
     * - VertexBuffer.MatricesIndicesExtraKind
     * - VertexBuffer.MatricesWeightsKind
     * - VertexBuffer.MatricesWeightsExtraKind
     * @param copyWhenShared defines a boolean indicating that if the mesh geometry is shared among some other meshes, the returned array is a copy of the internal one
     * @param forceCopy defines a boolean forcing the copy of the buffer no matter what the value of copyWhenShared is
     * @returns a FloatArray or null if the mesh has no geometry or no vertex buffer for this kind.
     */
    getVerticesData(kind: string, copyWhenShared?: boolean, forceCopy?: boolean): Nullable<FloatArray>;
    /**
     * Returns the mesh VertexBuffer object from the requested `kind`
     * @param kind defines which buffer to read from (positions, indices, normals, etc). Possible `kind` values :
     * - VertexBuffer.PositionKind
     * - VertexBuffer.NormalKind
     * - VertexBuffer.UVKind
     * - VertexBuffer.UV2Kind
     * - VertexBuffer.UV3Kind
     * - VertexBuffer.UV4Kind
     * - VertexBuffer.UV5Kind
     * - VertexBuffer.UV6Kind
     * - VertexBuffer.ColorKind
     * - VertexBuffer.MatricesIndicesKind
     * - VertexBuffer.MatricesIndicesExtraKind
     * - VertexBuffer.MatricesWeightsKind
     * - VertexBuffer.MatricesWeightsExtraKind
     * @returns a FloatArray or null if the mesh has no vertex buffer for this kind.
     */
    getVertexBuffer(kind: string): Nullable<VertexBuffer>;
    /**
     * Tests if a specific vertex buffer is associated with this mesh
     * @param kind defines which buffer to check (positions, indices, normals, etc). Possible `kind` values :
     * - VertexBuffer.PositionKind
     * - VertexBuffer.NormalKind
     * - VertexBuffer.UVKind
     * - VertexBuffer.UV2Kind
     * - VertexBuffer.UV3Kind
     * - VertexBuffer.UV4Kind
     * - VertexBuffer.UV5Kind
     * - VertexBuffer.UV6Kind
     * - VertexBuffer.ColorKind
     * - VertexBuffer.MatricesIndicesKind
     * - VertexBuffer.MatricesIndicesExtraKind
     * - VertexBuffer.MatricesWeightsKind
     * - VertexBuffer.MatricesWeightsExtraKind
     * @returns a boolean
     */
    isVerticesDataPresent(kind: string): boolean;
    /**
     * Returns a boolean defining if the vertex data for the requested `kind` is updatable.
     * @param kind defines which buffer to check (positions, indices, normals, etc). Possible `kind` values :
     * - VertexBuffer.PositionKind
     * - VertexBuffer.UVKind
     * - VertexBuffer.UV2Kind
     * - VertexBuffer.UV3Kind
     * - VertexBuffer.UV4Kind
     * - VertexBuffer.UV5Kind
     * - VertexBuffer.UV6Kind
     * - VertexBuffer.ColorKind
     * - VertexBuffer.MatricesIndicesKind
     * - VertexBuffer.MatricesIndicesExtraKind
     * - VertexBuffer.MatricesWeightsKind
     * - VertexBuffer.MatricesWeightsExtraKind
     * @returns a boolean
     */
    isVertexBufferUpdatable(kind: string): boolean;
    /**
     * Returns a string which contains the list of existing `kinds` of Vertex Data associated with this mesh.
     * @returns an array of strings
     */
    getVerticesDataKinds(): string[];
    /**
     * Returns a positive integer : the total number of indices in this mesh geometry.
     * @returns the numner of indices or zero if the mesh has no geometry.
     */
    getTotalIndices(): number;
    /**
     * Returns an array of integers or a typed array (Int32Array, Uint32Array, Uint16Array) populated with the mesh indices.
     * @param copyWhenShared If true (default false) and and if the mesh geometry is shared among some other meshes, the returned array is a copy of the internal one.
     * @param forceCopy defines a boolean indicating that the returned array must be cloned upon returning it
     * @returns the indices array or an empty array if the mesh has no geometry
     */
    getIndices(copyWhenShared?: boolean, forceCopy?: boolean): Nullable<IndicesArray>;
    get isBlocked(): boolean;
    /**
     * Determine if the current mesh is ready to be rendered
     * @param completeCheck defines if a complete check (including materials and lights) has to be done (false by default)
     * @param forceInstanceSupport will check if the mesh will be ready when used with instances (false by default)
     * @returns true if all associated assets are ready (material, textures, shaders)
     */
    isReady(completeCheck?: boolean, forceInstanceSupport?: boolean): boolean;
    /**
     * Gets a boolean indicating if the normals aren't to be recomputed on next mesh `positions` array update. This property is pertinent only for updatable parametric shapes.
     */
    get areNormalsFrozen(): boolean;
    /**
     * This function affects parametric shapes on vertex position update only : ribbons, tubes, etc. It has no effect at all on other shapes. It prevents the mesh normals from being recomputed on next `positions` array update.
     * @returns the current mesh
     */
    freezeNormals(): Mesh;
    /**
     * This function affects parametric shapes on vertex position update only : ribbons, tubes, etc. It has no effect at all on other shapes. It reactivates the mesh normals computation if it was previously frozen
     * @returns the current mesh
     */
    unfreezeNormals(): Mesh;
    /**
     * Sets a value overriding the instance count. Only applicable when custom instanced InterleavedVertexBuffer are used rather than InstancedMeshs
     */
    set overridenInstanceCount(count: number);
    /** @internal */
    _preActivate(): Mesh;
    /**
     * @internal
     */
    _preActivateForIntermediateRendering(renderId: number): Mesh;
    /**
     * @internal
     */
    _registerInstanceForRenderId(instance: InstancedMesh, renderId: number): Mesh;
    protected _afterComputeWorldMatrix(): void;
    /** @internal */
    _postActivate(): void;
    /**
     * This method recomputes and sets a new BoundingInfo to the mesh unless it is locked.
     * This means the mesh underlying bounding box and sphere are recomputed.
     * @param applySkeleton defines whether to apply the skeleton before computing the bounding info
     * @param applyMorph  defines whether to apply the morph target before computing the bounding info
     * @returns the current mesh
     */
    refreshBoundingInfo(applySkeleton?: boolean, applyMorph?: boolean): Mesh;
    /**
     * @internal
     */
    _createGlobalSubMesh(force: boolean): Nullable<SubMesh>;
    /**
     * This function will subdivide the mesh into multiple submeshes
     * @param count defines the expected number of submeshes
     */
    subdivide(count: number): void;
    /**
     * Copy a FloatArray into a specific associated vertex buffer
     * @param kind defines which buffer to write to (positions, indices, normals, etc). Possible `kind` values :
     * - VertexBuffer.PositionKind
     * - VertexBuffer.UVKind
     * - VertexBuffer.UV2Kind
     * - VertexBuffer.UV3Kind
     * - VertexBuffer.UV4Kind
     * - VertexBuffer.UV5Kind
     * - VertexBuffer.UV6Kind
     * - VertexBuffer.ColorKind
     * - VertexBuffer.MatricesIndicesKind
     * - VertexBuffer.MatricesIndicesExtraKind
     * - VertexBuffer.MatricesWeightsKind
     * - VertexBuffer.MatricesWeightsExtraKind
     * @param data defines the data source
     * @param updatable defines if the updated vertex buffer must be flagged as updatable
     * @param stride defines the data stride size (can be null)
     * @returns the current mesh
     */
    setVerticesData(kind: string, data: FloatArray, updatable?: boolean, stride?: number): AbstractMesh;
    /**
     * Delete a vertex buffer associated with this mesh
     * @param kind defines which buffer to delete (positions, indices, normals, etc). Possible `kind` values :
     * - VertexBuffer.PositionKind
     * - VertexBuffer.UVKind
     * - VertexBuffer.UV2Kind
     * - VertexBuffer.UV3Kind
     * - VertexBuffer.UV4Kind
     * - VertexBuffer.UV5Kind
     * - VertexBuffer.UV6Kind
     * - VertexBuffer.ColorKind
     * - VertexBuffer.MatricesIndicesKind
     * - VertexBuffer.MatricesIndicesExtraKind
     * - VertexBuffer.MatricesWeightsKind
     * - VertexBuffer.MatricesWeightsExtraKind
     */
    removeVerticesData(kind: string): void;
    /**
     * Flags an associated vertex buffer as updatable
     * @param kind defines which buffer to use (positions, indices, normals, etc). Possible `kind` values :
     * - VertexBuffer.PositionKind
     * - VertexBuffer.UVKind
     * - VertexBuffer.UV2Kind
     * - VertexBuffer.UV3Kind
     * - VertexBuffer.UV4Kind
     * - VertexBuffer.UV5Kind
     * - VertexBuffer.UV6Kind
     * - VertexBuffer.ColorKind
     * - VertexBuffer.MatricesIndicesKind
     * - VertexBuffer.MatricesIndicesExtraKind
     * - VertexBuffer.MatricesWeightsKind
     * - VertexBuffer.MatricesWeightsExtraKind
     * @param updatable defines if the updated vertex buffer must be flagged as updatable
     */
    markVerticesDataAsUpdatable(kind: string, updatable?: boolean): void;
    /**
     * Sets the mesh global Vertex Buffer
     * @param buffer defines the buffer to use
     * @param disposeExistingBuffer disposes the existing buffer, if any (default: true)
     * @returns the current mesh
     */
    setVerticesBuffer(buffer: VertexBuffer, disposeExistingBuffer?: boolean): Mesh;
    /**
     * Update a specific associated vertex buffer
     * @param kind defines which buffer to write to (positions, indices, normals, etc). Possible `kind` values :
     * - VertexBuffer.PositionKind
     * - VertexBuffer.UVKind
     * - VertexBuffer.UV2Kind
     * - VertexBuffer.UV3Kind
     * - VertexBuffer.UV4Kind
     * - VertexBuffer.UV5Kind
     * - VertexBuffer.UV6Kind
     * - VertexBuffer.ColorKind
     * - VertexBuffer.MatricesIndicesKind
     * - VertexBuffer.MatricesIndicesExtraKind
     * - VertexBuffer.MatricesWeightsKind
     * - VertexBuffer.MatricesWeightsExtraKind
     * @param data defines the data source
     * @param updateExtends defines if extends info of the mesh must be updated (can be null). This is mostly useful for "position" kind
     * @param makeItUnique defines if the geometry associated with the mesh must be cloned to make the change only for this mesh (and not all meshes associated with the same geometry)
     * @returns the current mesh
     */
    updateVerticesData(kind: string, data: FloatArray, updateExtends?: boolean, makeItUnique?: boolean): AbstractMesh;
    /**
     * This method updates the vertex positions of an updatable mesh according to the `positionFunction` returned values.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/dynamicMeshMorph#other-shapes-updatemeshpositions
     * @param positionFunction is a simple JS function what is passed the mesh `positions` array. It doesn't need to return anything
     * @param computeNormals is a boolean (default true) to enable/disable the mesh normal recomputation after the vertex position update
     * @returns the current mesh
     */
    updateMeshPositions(positionFunction: (data: FloatArray) => void, computeNormals?: boolean): Mesh;
    /**
     * Creates a un-shared specific occurence of the geometry for the mesh.
     * @returns the current mesh
     */
    makeGeometryUnique(): Mesh;
    /**
     * Set the index buffer of this mesh
     * @param indices defines the source data
     * @param totalVertices defines the total number of vertices referenced by this index data (can be null)
     * @param updatable defines if the updated index buffer must be flagged as updatable (default is false)
     * @returns the current mesh
     */
    setIndices(indices: IndicesArray, totalVertices?: Nullable<number>, updatable?: boolean): AbstractMesh;
    /**
     * Update the current index buffer
     * @param indices defines the source data
     * @param offset defines the offset in the index buffer where to store the new data (can be null)
     * @param gpuMemoryOnly defines a boolean indicating that only the GPU memory must be updated leaving the CPU version of the indices unchanged (false by default)
     * @returns the current mesh
     */
    updateIndices(indices: IndicesArray, offset?: number, gpuMemoryOnly?: boolean): AbstractMesh;
    /**
     * Invert the geometry to move from a right handed system to a left handed one.
     * @returns the current mesh
     */
    toLeftHanded(): Mesh;
    /**
     * @internal
     */
    _bind(subMesh: SubMesh, effect: Effect, fillMode: number): Mesh;
    /**
     * @internal
     */
    _draw(subMesh: SubMesh, fillMode: number, instancesCount?: number): Mesh;
    /**
     * Registers for this mesh a javascript function called just before the rendering process
     * @param func defines the function to call before rendering this mesh
     * @returns the current mesh
     */
    registerBeforeRender(func: (mesh: AbstractMesh) => void): Mesh;
    /**
     * Disposes a previously registered javascript function called before the rendering
     * @param func defines the function to remove
     * @returns the current mesh
     */
    unregisterBeforeRender(func: (mesh: AbstractMesh) => void): Mesh;
    /**
     * Registers for this mesh a javascript function called just after the rendering is complete
     * @param func defines the function to call after rendering this mesh
     * @returns the current mesh
     */
    registerAfterRender(func: (mesh: AbstractMesh) => void): Mesh;
    /**
     * Disposes a previously registered javascript function called after the rendering.
     * @param func defines the function to remove
     * @returns the current mesh
     */
    unregisterAfterRender(func: (mesh: AbstractMesh) => void): Mesh;
    /**
     * @internal
     */
    _getInstancesRenderList(subMeshId: number, isReplacementMode?: boolean): _InstancesBatch;
    /**
     * @internal
     */
    _renderWithInstances(subMesh: SubMesh, fillMode: number, batch: _InstancesBatch, effect: Effect, engine: Engine): Mesh;
    /**
     * @internal
     */
    _renderWithThinInstances(subMesh: SubMesh, fillMode: number, effect: Effect, engine: Engine): void;
    /**
     * @internal
     */
    _processInstancedBuffers(visibleInstances: Nullable<InstancedMesh[]>, renderSelf: boolean): void;
    /**
     * @internal
     */
    _processRendering(renderingMesh: AbstractMesh, subMesh: SubMesh, effect: Effect, fillMode: number, batch: _InstancesBatch, hardwareInstancedRendering: boolean, onBeforeDraw: (isInstance: boolean, world: Matrix, effectiveMaterial?: Material) => void, effectiveMaterial?: Material): Mesh;
    /**
     * @internal
     */
    _rebuild(dispose?: boolean): void;
    /** @internal */
    _freeze(): void;
    /** @internal */
    _unFreeze(): void;
    /**
     * Triggers the draw call for the mesh. Usually, you don't need to call this method by your own because the mesh rendering is handled by the scene rendering manager
     * @param subMesh defines the subMesh to render
     * @param enableAlphaMode defines if alpha mode can be changed
     * @param effectiveMeshReplacement defines an optional mesh used to provide info for the rendering
     * @returns the current mesh
     */
    render(subMesh: SubMesh, enableAlphaMode: boolean, effectiveMeshReplacement?: AbstractMesh): Mesh;
    private _onBeforeDraw;
    /**
     *   Renormalize the mesh and patch it up if there are no weights
     *   Similar to normalization by adding the weights compute the reciprocal and multiply all elements, this wil ensure that everything adds to 1.
     *   However in the case of zero weights then we set just a single influence to 1.
     *   We check in the function for extra's present and if so we use the normalizeSkinWeightsWithExtras rather than the FourWeights version.
     */
    cleanMatrixWeights(): void;
    private _normalizeSkinFourWeights;
    private _normalizeSkinWeightsAndExtra;
    /**
     * ValidateSkinning is used to determine that a mesh has valid skinning data along with skin metrics, if missing weights,
     * or not normalized it is returned as invalid mesh the string can be used for console logs, or on screen messages to let
     * the user know there was an issue with importing the mesh
     * @returns a validation object with skinned, valid and report string
     */
    validateSkinning(): {
        skinned: boolean;
        valid: boolean;
        report: string;
    };
    /** @internal */
    _checkDelayState(): Mesh;
    private _queueLoad;
    /**
     * Returns `true` if the mesh is within the frustum defined by the passed array of planes.
     * A mesh is in the frustum if its bounding box intersects the frustum
     * @param frustumPlanes defines the frustum to test
     * @returns true if the mesh is in the frustum planes
     */
    isInFrustum(frustumPlanes: Plane[]): boolean;
    /**
     * Sets the mesh material by the material or multiMaterial `id` property
     * @param id is a string identifying the material or the multiMaterial
     * @returns the current mesh
     */
    setMaterialById(id: string): Mesh;
    /**
     * Returns as a new array populated with the mesh material and/or skeleton, if any.
     * @returns an array of IAnimatable
     */
    getAnimatables(): IAnimatable[];
    /**
     * Modifies the mesh geometry according to the passed transformation matrix.
     * This method returns nothing, but it really modifies the mesh even if it's originally not set as updatable.
     * The mesh normals are modified using the same transformation.
     * Note that, under the hood, this method sets a new VertexBuffer each call.
     * @param transform defines the transform matrix to use
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/transforms/center_origin/bakingTransforms
     * @returns the current mesh
     */
    bakeTransformIntoVertices(transform: Matrix): Mesh;
    /**
     * Modifies the mesh geometry according to its own current World Matrix.
     * The mesh World Matrix is then reset.
     * This method returns nothing but really modifies the mesh even if it's originally not set as updatable.
     * Note that, under the hood, this method sets a new VertexBuffer each call.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/transforms/center_origin/bakingTransforms
     * @param bakeIndependentlyOfChildren indicates whether to preserve all child nodes' World Matrix during baking
     * @returns the current mesh
     */
    bakeCurrentTransformIntoVertices(bakeIndependentlyOfChildren?: boolean): Mesh;
    /** @internal */
    get _positions(): Nullable<Vector3[]>;
    /** @internal */
    _resetPointsArrayCache(): Mesh;
    /** @internal */
    _generatePointsArray(): boolean;
    /**
     * Returns a new Mesh object generated from the current mesh properties.
     * This method must not get confused with createInstance()
     * @param name is a string, the name given to the new mesh
     * @param newParent can be any Node object (default `null`)
     * @param doNotCloneChildren allows/denies the recursive cloning of the original mesh children if any (default `false`)
     * @param clonePhysicsImpostor allows/denies the cloning in the same time of the original mesh `body` used by the physics engine, if any (default `true`)
     * @returns a new mesh
     */
    clone(name?: string, newParent?: Nullable<Node>, doNotCloneChildren?: boolean, clonePhysicsImpostor?: boolean): Mesh;
    /**
     * Releases resources associated with this mesh.
     * @param doNotRecurse Set to true to not recurse into each children (recurse into each children by default)
     * @param disposeMaterialAndTextures Set to true to also dispose referenced materials and textures (false by default)
     */
    dispose(doNotRecurse?: boolean, disposeMaterialAndTextures?: boolean): void;
    /** @internal */
    _disposeInstanceSpecificData(): void;
    /** @internal */
    _disposeThinInstanceSpecificData(): void;
    /**
     * Modifies the mesh geometry according to a displacement map.
     * A displacement map is a colored image. Each pixel color value (actually a gradient computed from red, green, blue values) will give the displacement to apply to each mesh vertex.
     * The mesh must be set as updatable. Its internal geometry is directly modified, no new buffer are allocated.
     * @param url is a string, the URL from the image file is to be downloaded.
     * @param minHeight is the lower limit of the displacement.
     * @param maxHeight is the upper limit of the displacement.
     * @param onSuccess is an optional Javascript function to be called just after the mesh is modified. It is passed the modified mesh and must return nothing.
     * @param uvOffset is an optional vector2 used to offset UV.
     * @param uvScale is an optional vector2 used to scale UV.
     * @param forceUpdate defines whether or not to force an update of the generated buffers. This is useful to apply on a deserialized model for instance.
     * @returns the Mesh.
     */
    applyDisplacementMap(url: string, minHeight: number, maxHeight: number, onSuccess?: (mesh: Mesh) => void, uvOffset?: Vector2, uvScale?: Vector2, forceUpdate?: boolean): Mesh;
    /**
     * Modifies the mesh geometry according to a displacementMap buffer.
     * A displacement map is a colored image. Each pixel color value (actually a gradient computed from red, green, blue values) will give the displacement to apply to each mesh vertex.
     * The mesh must be set as updatable. Its internal geometry is directly modified, no new buffer are allocated.
     * @param buffer is a `Uint8Array` buffer containing series of `Uint8` lower than 255, the red, green, blue and alpha values of each successive pixel.
     * @param heightMapWidth is the width of the buffer image.
     * @param heightMapHeight is the height of the buffer image.
     * @param minHeight is the lower limit of the displacement.
     * @param maxHeight is the upper limit of the displacement.
     * @param uvOffset is an optional vector2 used to offset UV.
     * @param uvScale is an optional vector2 used to scale UV.
     * @param forceUpdate defines whether or not to force an update of the generated buffers. This is useful to apply on a deserialized model for instance.
     * @returns the Mesh.
     */
    applyDisplacementMapFromBuffer(buffer: Uint8Array, heightMapWidth: number, heightMapHeight: number, minHeight: number, maxHeight: number, uvOffset?: Vector2, uvScale?: Vector2, forceUpdate?: boolean): Mesh;
    /**
     * Modify the mesh to get a flat shading rendering.
     * This means each mesh facet will then have its own normals. Usually new vertices are added in the mesh geometry to get this result.
     * Warning : the mesh is really modified even if not set originally as updatable and, under the hood, a new VertexBuffer is allocated.
     * @returns current mesh
     */
    convertToFlatShadedMesh(): Mesh;
    /**
     * This method removes all the mesh indices and add new vertices (duplication) in order to unfold facets into buffers.
     * In other words, more vertices, no more indices and a single bigger VBO.
     * The mesh is really modified even if not set originally as updatable. Under the hood, a new VertexBuffer is allocated.
     * @returns current mesh
     */
    convertToUnIndexedMesh(): Mesh;
    /**
     * Inverses facet orientations.
     * Warning : the mesh is really modified even if not set originally as updatable. A new VertexBuffer is created under the hood each call.
     * @param flipNormals will also inverts the normals
     * @returns current mesh
     */
    flipFaces(flipNormals?: boolean): Mesh;
    /**
     * Increase the number of facets and hence vertices in a mesh
     * Vertex normals are interpolated from existing vertex normals
     * Warning : the mesh is really modified even if not set originally as updatable. A new VertexBuffer is created under the hood each call.
     * @param numberPerEdge the number of new vertices to add to each edge of a facet, optional default 1
     */
    increaseVertices(numberPerEdge?: number): void;
    /**
     * Force adjacent facets to share vertices and remove any facets that have all vertices in a line
     * This will undo any application of covertToFlatShadedMesh
     * Warning : the mesh is really modified even if not set originally as updatable. A new VertexBuffer is created under the hood each call.
     */
    forceSharedVertices(): void;
    /**
     * @internal
     */
    static _instancedMeshFactory(name: string, mesh: Mesh): InstancedMesh;
    /**
     * @internal
     */
    static _PhysicsImpostorParser(scene: Scene, physicObject: IPhysicsEnabledObject, jsonObject: any): PhysicsImpostor;
    /**
     * Creates a new InstancedMesh object from the mesh model.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/copies/instances
     * @param name defines the name of the new instance
     * @returns a new InstancedMesh
     */
    createInstance(name: string): InstancedMesh;
    /**
     * Synchronises all the mesh instance submeshes to the current mesh submeshes, if any.
     * After this call, all the mesh instances have the same submeshes than the current mesh.
     * @returns the current mesh
     */
    synchronizeInstances(): Mesh;
    /**
     * Optimization of the mesh's indices, in case a mesh has duplicated vertices.
     * The function will only reorder the indices and will not remove unused vertices to avoid problems with submeshes.
     * This should be used together with the simplification to avoid disappearing triangles.
     * @param successCallback an optional success callback to be called after the optimization finished.
     * @returns the current mesh
     */
    optimizeIndices(successCallback?: (mesh?: Mesh) => void): Mesh;
    /**
     * Serialize current mesh
     * @param serializationObject defines the object which will receive the serialization data
     */
    serialize(serializationObject?: any): any;
    /** @internal */
    _syncGeometryWithMorphTargetManager(): void;
    /**
     * @internal
     */
    static _GroundMeshParser: (parsedMesh: any, scene: Scene) => Mesh;
    /**
     * @internal
     */
    static _GoldbergMeshParser: (parsedMesh: any, scene: Scene) => GoldbergMesh;
    /**
     * @internal
     */
    static _LinesMeshParser: (parsedMesh: any, scene: Scene) => Mesh;
    /**
     * Returns a new Mesh object parsed from the source provided.
     * @param parsedMesh is the source
     * @param scene defines the hosting scene
     * @param rootUrl is the root URL to prefix the `delayLoadingFile` property with
     * @returns a new Mesh
     */
    static Parse(parsedMesh: any, scene: Scene, rootUrl: string): Mesh;
    /**
     * Prepare internal position array for software CPU skinning
     * @returns original positions used for CPU skinning. Useful for integrating Morphing with skeletons in same mesh
     */
    setPositionsForCPUSkinning(): Nullable<Float32Array>;
    /**
     * Prepare internal normal array for software CPU skinning
     * @returns original normals used for CPU skinning. Useful for integrating Morphing with skeletons in same mesh.
     */
    setNormalsForCPUSkinning(): Nullable<Float32Array>;
    /**
     * Updates the vertex buffer by applying transformation from the bones
     * @param skeleton defines the skeleton to apply to current mesh
     * @returns the current mesh
     */
    applySkeleton(skeleton: Skeleton): Mesh;
    /**
     * Returns an object containing a min and max Vector3 which are the minimum and maximum vectors of each mesh bounding box from the passed array, in the world coordinates
     * @param meshes defines the list of meshes to scan
     * @returns an object `{min:` Vector3`, max:` Vector3`}`
     */
    static MinMax(meshes: AbstractMesh[]): {
        min: Vector3;
        max: Vector3;
    };
    /**
     * Returns the center of the `{min:` Vector3`, max:` Vector3`}` or the center of MinMax vector3 computed from a mesh array
     * @param meshesOrMinMaxVector could be an array of meshes or a `{min:` Vector3`, max:` Vector3`}` object
     * @returns a vector3
     */
    static Center(meshesOrMinMaxVector: {
        min: Vector3;
        max: Vector3;
    } | AbstractMesh[]): Vector3;
    /**
     * Merge the array of meshes into a single mesh for performance reasons.
     * @param meshes array of meshes with the vertices to merge. Entries cannot be empty meshes.
     * @param disposeSource when true (default), dispose of the vertices from the source meshes.
     * @param allow32BitsIndices when the sum of the vertices > 64k, this must be set to true.
     * @param meshSubclass (optional) can be set to a Mesh where the merged vertices will be inserted.
     * @param subdivideWithSubMeshes when true (false default), subdivide mesh into subMeshes.
     * @param multiMultiMaterials when true (false default), subdivide mesh into subMeshes with multiple materials, ignores subdivideWithSubMeshes.
     * @returns a new mesh
     */
    static MergeMeshes(meshes: Array<Mesh>, disposeSource?: boolean, allow32BitsIndices?: boolean, meshSubclass?: Mesh, subdivideWithSubMeshes?: boolean, multiMultiMaterials?: boolean): Nullable<Mesh>;
    /**
     * Merge the array of meshes into a single mesh for performance reasons.
     * @param meshes array of meshes with the vertices to merge. Entries cannot be empty meshes.
     * @param disposeSource when true (default), dispose of the vertices from the source meshes.
     * @param allow32BitsIndices when the sum of the vertices > 64k, this must be set to true.
     * @param meshSubclass (optional) can be set to a Mesh where the merged vertices will be inserted.
     * @param subdivideWithSubMeshes when true (false default), subdivide mesh into subMeshes.
     * @param multiMultiMaterials when true (false default), subdivide mesh into subMeshes with multiple materials, ignores subdivideWithSubMeshes.
     * @returns a new mesh
     */
    static MergeMeshesAsync(meshes: Array<Mesh>, disposeSource?: boolean, allow32BitsIndices?: boolean, meshSubclass?: Mesh, subdivideWithSubMeshes?: boolean, multiMultiMaterials?: boolean): Promise<any>;
    private static _MergeMeshesCoroutine;
    /**
     * @internal
     */
    addInstance(instance: InstancedMesh): void;
    /**
     * @internal
     */
    removeInstance(instance: InstancedMesh): void;
    /** @internal */
    _shouldConvertRHS(): boolean;
}
import { Color4 } from "babylonjs/Maths/math.color";
import { Vector4 } from "babylonjs/Maths/math.vector";
import { ICreateCapsuleOptions } from "babylonjs/Meshes/Builders/capsuleBuilder";
import { GroundMesh } from "babylonjs/Meshes/groundMesh";
import { LinesMesh } from "babylonjs/Meshes/linesMesh";
module "babylonjs/Meshes/mesh" {
    interface Mesh {
        /**
         * Sets the mesh material by the material or multiMaterial `id` property
         * @param id is a string identifying the material or the multiMaterial
         * @returns the current mesh
         * @deprecated Please use MeshBuilder instead Please use setMaterialById instead
         */
        setMaterialByID(id: string): Mesh;
    }
    namespace Mesh {
        /**
         * Creates a ribbon mesh.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/param
         * @param name defines the name of the mesh to create
         * @param pathArray is a required array of paths, what are each an array of successive Vector3. The pathArray parameter depicts the ribbon geometry.
         * @param closeArray creates a seam between the first and the last paths of the path array (default is false)
         * @param closePath creates a seam between the first and the last points of each path of the path array
         * @param offset is taken in account only if the `pathArray` is containing a single path
         * @param scene defines the hosting scene
         * @param updatable defines if the mesh must be flagged as updatable
         * @param sideOrientation defines the mesh side orientation (https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#side-orientation)
         * @param instance defines an instance of an existing Ribbon object to be updated with the passed `pathArray` parameter (https://doc.babylonjs.com/how_to/How_to_dynamically_morph_a_mesh#ribbon)
         * @returns a new Mesh
         * @deprecated Please use MeshBuilder instead
         */
        function CreateRibbon(name: string, pathArray: Vector3[][], closeArray: boolean, closePath: boolean, offset: number, scene?: Scene, updatable?: boolean, sideOrientation?: number, instance?: Mesh): Mesh;
        /**
         * Creates a plane polygonal mesh.  By default, this is a disc.
         * @param name defines the name of the mesh to create
         * @param radius sets the radius size (float) of the polygon (default 0.5)
         * @param tessellation sets the number of polygon sides (positive integer, default 64). So a tessellation valued to 3 will build a triangle, to 4 a square, etc
         * @param scene defines the hosting scene
         * @param updatable defines if the mesh must be flagged as updatable
         * @param sideOrientation defines the mesh side orientation (https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#side-orientation)
         * @returns a new Mesh
         * @deprecated Please use MeshBuilder instead
         */
        function CreateDisc(name: string, radius: number, tessellation: number, scene: Nullable<Scene>, updatable?: boolean, sideOrientation?: number): Mesh;
        /**
         * Creates a box mesh.
         * @param name defines the name of the mesh to create
         * @param size sets the size (float) of each box side (default 1)
         * @param scene defines the hosting scene
         * @param updatable defines if the mesh must be flagged as updatable
         * @param sideOrientation defines the mesh side orientation (https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#side-orientation)
         * @returns a new Mesh
         * @deprecated Please use MeshBuilder instead
         */
        function CreateBox(name: string, size: number, scene: Nullable<Scene>, updatable?: boolean, sideOrientation?: number): Mesh;
        /**
         * Creates a sphere mesh.
         * @param name defines the name of the mesh to create
         * @param segments sets the sphere number of horizontal stripes (positive integer, default 32)
         * @param diameter sets the diameter size (float) of the sphere (default 1)
         * @param scene defines the hosting scene
         * @param updatable defines if the mesh must be flagged as updatable
         * @param sideOrientation defines the mesh side orientation (https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#side-orientation)
         * @returns a new Mesh
         * @deprecated Please use MeshBuilder instead
         */
        function CreateSphere(name: string, segments: number, diameter: number, scene?: Scene, updatable?: boolean, sideOrientation?: number): Mesh;
        /**
         * Creates a hemisphere mesh.
         * @param name defines the name of the mesh to create
         * @param segments sets the sphere number of horizontal stripes (positive integer, default 32)
         * @param diameter sets the diameter size (float) of the sphere (default 1)
         * @param scene defines the hosting scene
         * @returns a new Mesh
         * @deprecated Please use MeshBuilder instead
         */
        function CreateHemisphere(name: string, segments: number, diameter: number, scene?: Scene): Mesh;
        /**
         * Creates a cylinder or a cone mesh.
         * @param name defines the name of the mesh to create
         * @param height sets the height size (float) of the cylinder/cone (float, default 2)
         * @param diameterTop set the top cap diameter (floats, default 1)
         * @param diameterBottom set the bottom cap diameter (floats, default 1). This value can't be zero
         * @param tessellation sets the number of cylinder sides (positive integer, default 24). Set it to 3 to get a prism for instance
         * @param subdivisions sets the number of rings along the cylinder height (positive integer, default 1)
         * @param scene defines the hosting scene
         * @param updatable defines if the mesh must be flagged as updatable
         * @param sideOrientation defines the mesh side orientation (https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#side-orientation)
         * @returns a new Mesh
         * @deprecated Please use MeshBuilder instead
         */
        function CreateCylinder(name: string, height: number, diameterTop: number, diameterBottom: number, tessellation: number, subdivisions: any, scene?: Scene, updatable?: any, sideOrientation?: number): Mesh;
        /**
         * Creates a torus mesh.
         * @param name defines the name of the mesh to create
         * @param diameter sets the diameter size (float) of the torus (default 1)
         * @param thickness sets the diameter size of the tube of the torus (float, default 0.5)
         * @param tessellation sets the number of torus sides (positive integer, default 16)
         * @param scene defines the hosting scene
         * @param updatable defines if the mesh must be flagged as updatable
         * @param sideOrientation defines the mesh side orientation (https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#side-orientation)
         * @returns a new Mesh
         * @deprecated Please use MeshBuilder instead
         */
        function CreateTorus(name: string, diameter: number, thickness: number, tessellation: number, scene?: Scene, updatable?: boolean, sideOrientation?: number): Mesh;
        /**
         * Creates a torus knot mesh.
         * @param name defines the name of the mesh to create
         * @param radius sets the global radius size (float) of the torus knot (default 2)
         * @param tube sets the diameter size of the tube of the torus (float, default 0.5)
         * @param radialSegments sets the number of sides on each tube segments (positive integer, default 32)
         * @param tubularSegments sets the number of tubes to decompose the knot into (positive integer, default 32)
         * @param p the number of windings on X axis (positive integers, default 2)
         * @param q the number of windings on Y axis (positive integers, default 3)
         * @param scene defines the hosting scene
         * @param updatable defines if the mesh must be flagged as updatable
         * @param sideOrientation defines the mesh side orientation (https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#side-orientation)
         * @returns a new Mesh
         * @deprecated Please use MeshBuilder instead
         */
        function CreateTorusKnot(name: string, radius: number, tube: number, radialSegments: number, tubularSegments: number, p: number, q: number, scene?: Scene, updatable?: boolean, sideOrientation?: number): Mesh;
        /**
         * Creates a line mesh..
         * @param name defines the name of the mesh to create
         * @param points is an array successive Vector3
         * @param scene defines the hosting scene
         * @param updatable defines if the mesh must be flagged as updatable
         * @param instance is an instance of an existing LineMesh object to be updated with the passed `points` parameter (https://doc.babylonjs.com/how_to/How_to_dynamically_morph_a_mesh#lines-and-dashedlines).
         * @returns a new Mesh
         * @deprecated Please use MeshBuilder instead
         */
        function CreateLines(name: string, points: Vector3[], scene: Nullable<Scene>, updatable: boolean, instance?: Nullable<LinesMesh>): LinesMesh;
        /**
         * Creates a dashed line mesh.
         * @param name defines the name of the mesh to create
         * @param points is an array successive Vector3
         * @param dashSize is the size of the dashes relatively the dash number (positive float, default 3)
         * @param gapSize is the size of the gap between two successive dashes relatively the dash number (positive float, default 1)
         * @param dashNb is the intended total number of dashes (positive integer, default 200)
         * @param scene defines the hosting scene
         * @param updatable defines if the mesh must be flagged as updatable
         * @param instance is an instance of an existing LineMesh object to be updated with the passed `points` parameter (https://doc.babylonjs.com/how_to/How_to_dynamically_morph_a_mesh#lines-and-dashedlines)
         * @returns a new Mesh
         * @deprecated Please use MeshBuilder instead
         */
        function CreateDashedLines(name: string, points: Vector3[], dashSize: number, gapSize: number, dashNb: number, scene: Nullable<Scene>, updatable?: boolean, instance?: LinesMesh): LinesMesh;
        /**
         * Creates a polygon mesh.Please consider using the same method from the MeshBuilder class instead
         * The polygon's shape will depend on the input parameters and is constructed parallel to a ground mesh.
         * The parameter `shape` is a required array of successive Vector3 representing the corners of the polygon in th XoZ plane, that is y = 0 for all vectors.
         * You can set the mesh side orientation with the values : Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
         * The mesh can be set to updatable with the boolean parameter `updatable` (default false) if its internal geometry is supposed to change once created.
         * Remember you can only change the shape positions, not their number when updating a polygon.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/param#non-regular-polygon
         * @param name defines the name of the mesh to create
         * @param shape is a required array of successive Vector3 representing the corners of the polygon in th XoZ plane, that is y = 0 for all vectors
         * @param scene defines the hosting scene
         * @param holes is a required array of arrays of successive Vector3 used to defines holes in the polygon
         * @param updatable defines if the mesh must be flagged as updatable
         * @param sideOrientation defines the mesh side orientation (https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#side-orientation)
         * @param earcutInjection can be used to inject your own earcut reference
         * @returns a new Mesh
         * @deprecated Please use MeshBuilder instead
         */
        function CreatePolygon(name: string, shape: Vector3[], scene: Scene, holes?: Vector3[][], updatable?: boolean, sideOrientation?: number, earcutInjection?: any): Mesh;
        /**
         * Creates an extruded polygon mesh, with depth in the Y direction..
         * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/param#extruded-non-regular-polygon
         * @param name defines the name of the mesh to create
         * @param shape is a required array of successive Vector3 representing the corners of the polygon in th XoZ plane, that is y = 0 for all vectors
         * @param depth defines the height of extrusion
         * @param scene defines the hosting scene
         * @param holes is a required array of arrays of successive Vector3 used to defines holes in the polygon
         * @param updatable defines if the mesh must be flagged as updatable
         * @param sideOrientation defines the mesh side orientation (https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#side-orientation)
         * @param earcutInjection can be used to inject your own earcut reference
         * @returns a new Mesh
         * @deprecated Please use MeshBuilder instead
         */
        function ExtrudePolygon(name: string, shape: Vector3[], depth: number, scene: Scene, holes?: Vector3[][], updatable?: boolean, sideOrientation?: number, earcutInjection?: any): Mesh;
        /**
         * Creates an extruded shape mesh.
         * The extrusion is a parametric shape. It has no predefined shape. Its final shape will depend on the input parameters.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/param
         * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/param#extruded-shapes
         * @param name defines the name of the mesh to create
         * @param shape is a required array of successive Vector3. This array depicts the shape to be extruded in its local space : the shape must be designed in the xOy plane and will be extruded along the Z axis
         * @param path is a required array of successive Vector3. This is the axis curve the shape is extruded along
         * @param scale is the value to scale the shape
         * @param rotation is the angle value to rotate the shape each step (each path point), from the former step (so rotation added each step) along the curve
         * @param cap sets the way the extruded shape is capped. Possible values : Mesh.NO_CAP (default), Mesh.CAP_START, Mesh.CAP_END, Mesh.CAP_ALL
         * @param scene defines the hosting scene
         * @param updatable defines if the mesh must be flagged as updatable
         * @param sideOrientation defines the mesh side orientation (https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#side-orientation)
         * @param instance is an instance of an existing ExtrudedShape object to be updated with the passed `shape`, `path`, `scale` or `rotation` parameters (https://doc.babylonjs.com/how_to/How_to_dynamically_morph_a_mesh#extruded-shape)
         * @returns a new Mesh
         * @deprecated Please use MeshBuilder instead
         */
        function ExtrudeShape(name: string, shape: Vector3[], path: Vector3[], scale: number, rotation: number, cap: number, scene: Nullable<Scene>, updatable?: boolean, sideOrientation?: number, instance?: Mesh): Mesh;
        /**
         * Creates an custom extruded shape mesh.
         * The custom extrusion is a parametric shape.
         * It has no predefined shape. Its final shape will depend on the input parameters.
         *
         * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/param#extruded-shapes
         * @param name defines the name of the mesh to create
         * @param shape is a required array of successive Vector3. This array depicts the shape to be extruded in its local space : the shape must be designed in the xOy plane and will be extruded along the Z axis
         * @param path is a required array of successive Vector3. This is the axis curve the shape is extruded along
         * @param scaleFunction is a custom Javascript function called on each path point
         * @param rotationFunction is a custom Javascript function called on each path point
         * @param ribbonCloseArray forces the extrusion underlying ribbon to close all the paths in its `pathArray`
         * @param ribbonClosePath forces the extrusion underlying ribbon to close its `pathArray`
         * @param cap sets the way the extruded shape is capped. Possible values : Mesh.NO_CAP (default), Mesh.CAP_START, Mesh.CAP_END, Mesh.CAP_ALL
         * @param scene defines the hosting scene
         * @param updatable defines if the mesh must be flagged as updatable
         * @param sideOrientation defines the mesh side orientation (https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#side-orientation)
         * @param instance is an instance of an existing ExtrudedShape object to be updated with the passed `shape`, `path`, `scale` or `rotation` parameters (https://doc.babylonjs.com/features/featuresDeepDive/mesh/dynamicMeshMorph#extruded-shape)
         * @returns a new Mesh
         * @deprecated Please use MeshBuilder instead
         */
        function ExtrudeShapeCustom(name: string, shape: Vector3[], path: Vector3[], scaleFunction: Nullable<{
            (i: number, distance: number): number;
        }>, rotationFunction: Nullable<{
            (i: number, distance: number): number;
        }>, ribbonCloseArray: boolean, ribbonClosePath: boolean, cap: number, scene: Scene, updatable?: boolean, sideOrientation?: number, instance?: Mesh): Mesh;
        /**
         * Creates lathe mesh.
         * The lathe is a shape with a symmetry axis : a 2D model shape is rotated around this axis to design the lathe.
         * @param name defines the name of the mesh to create
         * @param shape is a required array of successive Vector3. This array depicts the shape to be rotated in its local space : the shape must be designed in the xOy plane and will be rotated around the Y axis. It's usually a 2D shape, so the Vector3 z coordinates are often set to zero
         * @param radius is the radius value of the lathe
         * @param tessellation is the side number of the lathe.
         * @param scene defines the hosting scene
         * @param updatable defines if the mesh must be flagged as updatable
         * @param sideOrientation defines the mesh side orientation (https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#side-orientation)
         * @returns a new Mesh
         * @deprecated Please use MeshBuilder instead
         */
        function CreateLathe(name: string, shape: Vector3[], radius: number, tessellation: number, scene: Scene, updatable?: boolean, sideOrientation?: number): Mesh;
        /**
         * Creates a plane mesh.
         * @param name defines the name of the mesh to create
         * @param size sets the size (float) of both sides of the plane at once (default 1)
         * @param scene defines the hosting scene
         * @param updatable defines if the mesh must be flagged as updatable
         * @param sideOrientation defines the mesh side orientation (https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#side-orientation)
         * @returns a new Mesh
         * @deprecated Please use MeshBuilder instead
         */
        function CreatePlane(name: string, size: number, scene: Scene, updatable?: boolean, sideOrientation?: number): Mesh;
        /**
         * Creates a ground mesh.
         * @param name defines the name of the mesh to create
         * @param width set the width of the ground
         * @param height set the height of the ground
         * @param subdivisions sets the number of subdivisions per side
         * @param scene defines the hosting scene
         * @param updatable defines if the mesh must be flagged as updatable
         * @returns a new Mesh
         * @deprecated Please use MeshBuilder instead
         */
        function CreateGround(name: string, width: number, height: number, subdivisions: number, scene?: Scene, updatable?: boolean): Mesh;
        /**
         * Creates a tiled ground mesh.
         * @param name defines the name of the mesh to create
         * @param xmin set the ground minimum X coordinate
         * @param zmin set the ground minimum Y coordinate
         * @param xmax set the ground maximum X coordinate
         * @param zmax set the ground maximum Z coordinate
         * @param subdivisions is an object `{w: positive integer, h: positive integer}` (default `{w: 6, h: 6}`). `w` and `h` are the numbers of subdivisions on the ground width and height. Each subdivision is called a tile
         * @param precision is an object `{w: positive integer, h: positive integer}` (default `{w: 2, h: 2}`). `w` and `h` are the numbers of subdivisions on the ground width and height of each tile
         * @param scene defines the hosting scene
         * @param updatable defines if the mesh must be flagged as updatable
         * @returns a new Mesh
         * @deprecated Please use MeshBuilder instead
         */
        function CreateTiledGround(name: string, xmin: number, zmin: number, xmax: number, zmax: number, subdivisions: {
            w: number;
            h: number;
        }, precision: {
            w: number;
            h: number;
        }, scene: Scene, updatable?: boolean): Mesh;
        /**
         * Creates a ground mesh from a height map.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set/height_map
         * @param name defines the name of the mesh to create
         * @param url sets the URL of the height map image resource
         * @param width set the ground width size
         * @param height set the ground height size
         * @param subdivisions sets the number of subdivision per side
         * @param minHeight is the minimum altitude on the ground
         * @param maxHeight is the maximum altitude on the ground
         * @param scene defines the hosting scene
         * @param updatable defines if the mesh must be flagged as updatable
         * @param onReady  is a callback function that will be called  once the mesh is built (the height map download can last some time)
         * @param alphaFilter will filter any data where the alpha channel is below this value, defaults 0 (all data visible)
         * @returns a new Mesh
         * @deprecated Please use MeshBuilder instead
         */
        function CreateGroundFromHeightMap(name: string, url: string, width: number, height: number, subdivisions: number, minHeight: number, maxHeight: number, scene: Scene, updatable?: boolean, onReady?: (mesh: GroundMesh) => void, alphaFilter?: number): GroundMesh;
        /**
         * Creates a tube mesh.
         * The tube is a parametric shape.
         * It has no predefined shape. Its final shape will depend on the input parameters.
         *
         * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/param
         * @param name defines the name of the mesh to create
         * @param path is a required array of successive Vector3. It is the curve used as the axis of the tube
         * @param radius sets the tube radius size
         * @param tessellation is the number of sides on the tubular surface
         * @param radiusFunction is a custom function. If it is not null, it overrides the parameter `radius`. This function is called on each point of the tube path and is passed the index `i` of the i-th point and the distance of this point from the first point of the path
         * @param cap sets the way the extruded shape is capped. Possible values : Mesh.NO_CAP (default), Mesh.CAP_START, Mesh.CAP_END, Mesh.CAP_ALL
         * @param scene defines the hosting scene
         * @param updatable defines if the mesh must be flagged as updatable
         * @param sideOrientation defines the mesh side orientation (https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#side-orientation)
         * @param instance is an instance of an existing Tube object to be updated with the passed `pathArray` parameter (https://doc.babylonjs.com/how_to/How_to_dynamically_morph_a_mesh#tube)
         * @returns a new Mesh
         * @deprecated Please use MeshBuilder instead
         */
        function CreateTube(name: string, path: Vector3[], radius: number, tessellation: number, radiusFunction: {
            (i: number, distance: number): number;
        }, cap: number, scene: Scene, updatable?: boolean, sideOrientation?: number, instance?: Mesh): Mesh;
        /**
         * Creates a polyhedron mesh.
         *.
         * * The parameter `type` (positive integer, max 14, default 0) sets the polyhedron type to build among the 15 embedded types. Please refer to the type sheet in the tutorial to choose the wanted type
         * * The parameter `size` (positive float, default 1) sets the polygon size
         * * You can overwrite the `size` on each dimension bu using the parameters `sizeX`, `sizeY` or `sizeZ` (positive floats, default to `size` value)
         * * You can build other polyhedron types than the 15 embbeded ones by setting the parameter `custom` (`polyhedronObject`, default null). If you set the parameter `custom`, this overwrittes the parameter `type`
         * * A `polyhedronObject` is a formatted javascript object. You'll find a full file with pre-set polyhedra here : https://github.com/BabylonJS/Extensions/tree/master/Polyhedron
         * * You can set the color and the UV of each side of the polyhedron with the parameters `faceColors` (Color4, default `(1, 1, 1, 1)`) and faceUV (Vector4, default `(0, 0, 1, 1)`)
         * * To understand how to set `faceUV` or `faceColors`, please read this by considering the right number of faces of your polyhedron, instead of only 6 for the box : https://doc.babylonjs.com/features/featuresDeepDive/materials/using/texturePerBoxFace
         * * The parameter `flat` (boolean, default true). If set to false, it gives the polyhedron a single global face, so less vertices and shared normals. In this case, `faceColors` and `faceUV` are ignored
         * * You can also set the mesh side orientation with the values : Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
         * * If you create a double-sided mesh, you can choose what parts of the texture image to crop and stick respectively on the front and the back sides with the parameters `frontUVs` and `backUVs` (Vector4). Detail here : https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#side-orientation
         * * The mesh can be set to updatable with the boolean parameter `updatable` (default false) if its internal geometry is supposed to change once created
         * @param name defines the name of the mesh to create
         * @param options defines the options used to create the mesh
         * @param scene defines the hosting scene
         * @returns a new Mesh
         * @deprecated Please use MeshBuilder instead
         */
        function CreatePolyhedron(name: string, options: {
            type?: number;
            size?: number;
            sizeX?: number;
            sizeY?: number;
            sizeZ?: number;
            custom?: any;
            faceUV?: Vector4[];
            faceColors?: Color4[];
            updatable?: boolean;
            sideOrientation?: number;
        }, scene: Scene): Mesh;
        /**
         * Creates a sphere based upon an icosahedron with 20 triangular faces which can be subdivided
         * * The parameter `radius` sets the radius size (float) of the icosphere (default 1)
         * * You can set some different icosphere dimensions, for instance to build an ellipsoid, by using the parameters `radiusX`, `radiusY` and `radiusZ` (all by default have the same value than `radius`)
         * * The parameter `subdivisions` sets the number of subdivisions (positive integer, default 4). The more subdivisions, the more faces on the icosphere whatever its size
         * * The parameter `flat` (boolean, default true) gives each side its own normals. Set it to false to get a smooth continuous light reflection on the surface
         * * You can also set the mesh side orientation with the values : Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
         * * If you create a double-sided mesh, you can choose what parts of the texture image to crop and stick respectively on the front and the back sides with the parameters `frontUVs` and `backUVs` (Vector4). Detail here : https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#side-orientation
         * * The mesh can be set to updatable with the boolean parameter `updatable` (default false) if its internal geometry is supposed to change once created
         * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/polyhedra#icosphere
         * @param name defines the name of the mesh
         * @param options defines the options used to create the mesh
         * @param scene defines the hosting scene
         * @returns a new Mesh
         * @deprecated Please use MeshBuilder instead
         */
        function CreateIcoSphere(name: string, options: {
            radius?: number;
            flat?: boolean;
            subdivisions?: number;
            sideOrientation?: number;
            updatable?: boolean;
        }, scene: Scene): Mesh;
        /**
         * Creates a decal mesh.
         *.
         * A decal is a mesh usually applied as a model onto the surface of another mesh
         * @param name  defines the name of the mesh
         * @param sourceMesh defines the mesh receiving the decal
         * @param position sets the position of the decal in world coordinates
         * @param normal sets the normal of the mesh where the decal is applied onto in world coordinates
         * @param size sets the decal scaling
         * @param angle sets the angle to rotate the decal
         * @returns a new Mesh
         * @deprecated Please use MeshBuilder instead
         */
        function CreateDecal(name: string, sourceMesh: AbstractMesh, position: Vector3, normal: Vector3, size: Vector3, angle: number): Mesh;
        /** Creates a Capsule Mesh
         * @param name defines the name of the mesh.
         * @param options the constructors options used to shape the mesh.
         * @param scene defines the scene the mesh is scoped to.
         * @returns the capsule mesh
         * @see https://doc.babylonjs.com/how_to/capsule_shape
         * @deprecated Please use MeshBuilder instead
         */
        function CreateCapsule(name: string, options: ICreateCapsuleOptions, scene: Scene): Mesh;
        /**
         * Extends a mesh to a Goldberg mesh
         * Warning  the mesh to convert MUST be an import of a perviously exported Goldberg mesh
         * @param mesh the mesh to convert
         * @returns the extended mesh
         * @deprecated Please use ExtendMeshToGoldberg instead
         */
        function ExtendToGoldberg(mesh: Mesh): Mesh;
    }
}
export {};

}
declare module "babylonjs/Meshes/mesh.vertexData" {
import { Nullable, FloatArray, IndicesArray } from "babylonjs/types";
import { Matrix, Vector2 } from "babylonjs/Maths/math.vector";
import { Vector3, Vector4 } from "babylonjs/Maths/math.vector";
import { Color3 } from "babylonjs/Maths/math.color";
import { Color4 } from "babylonjs/Maths/math.color";
import { Coroutine } from "babylonjs/Misc/coroutine";
import { ICreateCapsuleOptions } from "babylonjs/Meshes/Builders/capsuleBuilder";
import { Geometry } from "babylonjs/Meshes/geometry";
import { Mesh } from "babylonjs/Meshes/mesh";
/**
 * Define an interface for all classes that will get and set the data on vertices
 */
export interface IGetSetVerticesData {
    /**
     * Gets a boolean indicating if specific vertex data is present
     * @param kind defines the vertex data kind to use
     * @returns true is data kind is present
     */
    isVerticesDataPresent(kind: string): boolean;
    /**
     * Gets a specific vertex data attached to this geometry. Float data is constructed if the vertex buffer data cannot be returned directly.
     * @param kind defines the data kind (Position, normal, etc...)
     * @param copyWhenShared defines if the returned array must be cloned upon returning it if the current geometry is shared between multiple meshes
     * @param forceCopy defines a boolean indicating that the returned array must be cloned upon returning it
     * @returns a float array containing vertex data
     */
    getVerticesData(kind: string, copyWhenShared?: boolean, forceCopy?: boolean): Nullable<FloatArray>;
    /**
     * Returns an array of integers or a typed array (Int32Array, Uint32Array, Uint16Array) populated with the mesh indices.
     * @param copyWhenShared If true (default false) and and if the mesh geometry is shared among some other meshes, the returned array is a copy of the internal one.
     * @param forceCopy defines a boolean indicating that the returned array must be cloned upon returning it
     * @returns the indices array or an empty array if the mesh has no geometry
     */
    getIndices(copyWhenShared?: boolean, forceCopy?: boolean): Nullable<IndicesArray>;
    /**
     * Set specific vertex data
     * @param kind defines the data kind (Position, normal, etc...)
     * @param data defines the vertex data to use
     * @param updatable defines if the vertex must be flagged as updatable (false as default)
     * @param stride defines the stride to use (0 by default). This value is deduced from the kind value if not specified
     */
    setVerticesData(kind: string, data: FloatArray, updatable: boolean): void;
    /**
     * Update a specific associated vertex buffer
     * @param kind defines which buffer to write to (positions, indices, normals, etc). Possible `kind` values :
     * - VertexBuffer.PositionKind
     * - VertexBuffer.UVKind
     * - VertexBuffer.UV2Kind
     * - VertexBuffer.UV3Kind
     * - VertexBuffer.UV4Kind
     * - VertexBuffer.UV5Kind
     * - VertexBuffer.UV6Kind
     * - VertexBuffer.ColorKind
     * - VertexBuffer.MatricesIndicesKind
     * - VertexBuffer.MatricesIndicesExtraKind
     * - VertexBuffer.MatricesWeightsKind
     * - VertexBuffer.MatricesWeightsExtraKind
     * @param data defines the data source
     * @param updateExtends defines if extends info of the mesh must be updated (can be null). This is mostly useful for "position" kind
     * @param makeItUnique defines if the geometry associated with the mesh must be cloned to make the change only for this mesh (and not all meshes associated with the same geometry)
     */
    updateVerticesData(kind: string, data: FloatArray, updateExtends?: boolean, makeItUnique?: boolean): void;
    /**
     * Creates a new index buffer
     * @param indices defines the indices to store in the index buffer
     * @param totalVertices defines the total number of vertices (could be null)
     * @param updatable defines if the index buffer must be flagged as updatable (false by default)
     */
    setIndices(indices: IndicesArray, totalVertices: Nullable<number>, updatable?: boolean): void;
}
/**
 * This class contains the various kinds of data on every vertex of a mesh used in determining its shape and appearance
 */
export class VertexData {
    /**
     * Mesh side orientation : usually the external or front surface
     */
    static readonly FRONTSIDE: number;
    /**
     * Mesh side orientation : usually the internal or back surface
     */
    static readonly BACKSIDE: number;
    /**
     * Mesh side orientation : both internal and external or front and back surfaces
     */
    static readonly DOUBLESIDE: number;
    /**
     * Mesh side orientation : by default, `FRONTSIDE`
     */
    static readonly DEFAULTSIDE: number;
    /**
     * An array of the x, y, z position of each vertex  [...., x, y, z, .....]
     */
    positions: Nullable<FloatArray>;
    /**
     * An array of the x, y, z normal vector of each vertex  [...., x, y, z, .....]
     */
    normals: Nullable<FloatArray>;
    /**
     * An array of the x, y, z tangent vector of each vertex  [...., x, y, z, .....]
     */
    tangents: Nullable<FloatArray>;
    /**
     * An array of u,v which maps a texture image onto each vertex  [...., u, v, .....]
     */
    uvs: Nullable<FloatArray>;
    /**
     * A second array of u,v which maps a texture image onto each vertex  [...., u, v, .....]
     */
    uvs2: Nullable<FloatArray>;
    /**
     * A third array of u,v which maps a texture image onto each vertex  [...., u, v, .....]
     */
    uvs3: Nullable<FloatArray>;
    /**
     * A fourth array of u,v which maps a texture image onto each vertex  [...., u, v, .....]
     */
    uvs4: Nullable<FloatArray>;
    /**
     * A fifth array of u,v which maps a texture image onto each vertex  [...., u, v, .....]
     */
    uvs5: Nullable<FloatArray>;
    /**
     * A sixth array of u,v which maps a texture image onto each vertex  [...., u, v, .....]
     */
    uvs6: Nullable<FloatArray>;
    /**
     * An array of the r, g, b, a, color of each vertex  [...., r, g, b, a, .....]
     */
    colors: Nullable<FloatArray>;
    /**
     * An array containing the list of indices to the array of matrices produced by bones, each vertex have up to 4 indices (8 if the matricesIndicesExtra is set).
     */
    matricesIndices: Nullable<FloatArray>;
    /**
     * An array containing the list of weights defining the weight of each indexed matrix in the final computation
     */
    matricesWeights: Nullable<FloatArray>;
    /**
     * An array extending the number of possible indices
     */
    matricesIndicesExtra: Nullable<FloatArray>;
    /**
     * An array extending the number of possible weights when the number of indices is extended
     */
    matricesWeightsExtra: Nullable<FloatArray>;
    /**
     * An array of i, j, k the three vertex indices required for each triangular facet  [...., i, j, k .....]
     */
    indices: Nullable<IndicesArray>;
    /**
     * Uses the passed data array to set the set the values for the specified kind of data
     * @param data a linear array of floating numbers
     * @param kind the type of data that is being set, eg positions, colors etc
     */
    set(data: FloatArray, kind: string): void;
    /**
     * Associates the vertexData to the passed Mesh.
     * Sets it as updatable or not (default `false`)
     * @param mesh the mesh the vertexData is applied to
     * @param updatable when used and having the value true allows new data to update the vertexData
     * @returns the VertexData
     */
    applyToMesh(mesh: Mesh, updatable?: boolean): VertexData;
    /**
     * Associates the vertexData to the passed Geometry.
     * Sets it as updatable or not (default `false`)
     * @param geometry the geometry the vertexData is applied to
     * @param updatable when used and having the value true allows new data to update the vertexData
     * @returns VertexData
     */
    applyToGeometry(geometry: Geometry, updatable?: boolean): VertexData;
    /**
     * Updates the associated mesh
     * @param mesh the mesh to be updated
     * @returns VertexData
     */
    updateMesh(mesh: Mesh): VertexData;
    /**
     * Updates the associated geometry
     * @param geometry the geometry to be updated
     * @returns VertexData.
     */
    updateGeometry(geometry: Geometry): VertexData;
    private readonly _applyTo;
    /**
     * @internal
     */
    _applyToCoroutine(meshOrGeometry: IGetSetVerticesData, updatable: boolean | undefined, isAsync: boolean): Coroutine<VertexData>;
    private _update;
    private static _TransformVector3Coordinates;
    private static _TransformVector3Normals;
    private static _TransformVector4Normals;
    private static _FlipFaces;
    /**
     * Transforms each position and each normal of the vertexData according to the passed Matrix
     * @param matrix the transforming matrix
     * @returns the VertexData
     */
    transform(matrix: Matrix): VertexData;
    /**
     * Merges the passed VertexData into the current one
     * @param others the VertexData to be merged into the current one
     * @param use32BitsIndices defines a boolean indicating if indices must be store in a 32 bits array
     * @param forceCloneIndices defines a boolean indicating if indices are forced to be cloned
     * @returns the modified VertexData
     */
    merge(others: VertexData | VertexData[], use32BitsIndices?: boolean, forceCloneIndices?: boolean): VertexData;
    /**
     * @internal
     */
    _mergeCoroutine(transform: Matrix | undefined, vertexDatas: (readonly [vertexData: VertexData, transform?: Matrix])[], use32BitsIndices: boolean | undefined, isAsync: boolean, forceCloneIndices: boolean): Coroutine<VertexData>;
    private static _MergeElement;
    private _validate;
    /**
     * Serializes the VertexData
     * @returns a serialized object
     */
    serialize(): any;
    /**
     * Extracts the vertexData from a mesh
     * @param mesh the mesh from which to extract the VertexData
     * @param copyWhenShared defines if the VertexData must be cloned when shared between multiple meshes, optional, default false
     * @param forceCopy indicating that the VertexData must be cloned, optional, default false
     * @returns the object VertexData associated to the passed mesh
     */
    static ExtractFromMesh(mesh: Mesh, copyWhenShared?: boolean, forceCopy?: boolean): VertexData;
    /**
     * Extracts the vertexData from the geometry
     * @param geometry the geometry from which to extract the VertexData
     * @param copyWhenShared defines if the VertexData must be cloned when the geometry is shared between multiple meshes, optional, default false
     * @param forceCopy indicating that the VertexData must be cloned, optional, default false
     * @returns the object VertexData associated to the passed mesh
     */
    static ExtractFromGeometry(geometry: Geometry, copyWhenShared?: boolean, forceCopy?: boolean): VertexData;
    private static _ExtractFrom;
    /**
     * Creates the VertexData for a Ribbon
     * @param options an object used to set the following optional parameters for the ribbon, required but can be empty
     * * pathArray array of paths, each of which an array of successive Vector3
     * * closeArray creates a seam between the first and the last paths of the pathArray, optional, default false
     * * closePath creates a seam between the first and the last points of each path of the path array, optional, default false
     * * offset a positive integer, only used when pathArray contains a single path (offset = 10 means the point 1 is joined to the point 11), default rounded half size of the pathArray length
     * * sideOrientation optional and takes the values : Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
     * * frontUvs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the front side, optional, default vector4 (0, 0, 1, 1)
     * * backUVs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the back side, optional, default vector4 (0, 0, 1, 1)
     * * invertUV swaps in the U and V coordinates when applying a texture, optional, default false
     * * uvs a linear array, of length 2 * number of vertices, of custom UV values, optional
     * * colors a linear array, of length 4 * number of vertices, of custom color values, optional
     * @param options.pathArray
     * @param options.closeArray
     * @param options.closePath
     * @param options.offset
     * @param options.sideOrientation
     * @param options.frontUVs
     * @param options.backUVs
     * @param options.invertUV
     * @param options.uvs
     * @param options.colors
     * @returns the VertexData of the ribbon
     * @deprecated use CreateRibbonVertexData instead
     */
    static CreateRibbon(options: {
        pathArray: Vector3[][];
        closeArray?: boolean;
        closePath?: boolean;
        offset?: number;
        sideOrientation?: number;
        frontUVs?: Vector4;
        backUVs?: Vector4;
        invertUV?: boolean;
        uvs?: Vector2[];
        colors?: Color4[];
    }): VertexData;
    /**
     * Creates the VertexData for a box
     * @param options an object used to set the following optional parameters for the box, required but can be empty
     * * size sets the width, height and depth of the box to the value of size, optional default 1
     * * width sets the width (x direction) of the box, overwrites the width set by size, optional, default size
     * * height sets the height (y direction) of the box, overwrites the height set by size, optional, default size
     * * depth sets the depth (z direction) of the box, overwrites the depth set by size, optional, default size
     * * faceUV an array of 6 Vector4 elements used to set different images to each box side
     * * faceColors an array of 6 Color3 elements used to set different colors to each box side
     * * sideOrientation optional and takes the values : Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
     * * frontUvs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the front side, optional, default vector4 (0, 0, 1, 1)
     * * backUVs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the back side, optional, default vector4 (0, 0, 1, 1)
     * @param options.size
     * @param options.width
     * @param options.height
     * @param options.depth
     * @param options.faceUV
     * @param options.faceColors
     * @param options.sideOrientation
     * @param options.frontUVs
     * @param options.backUVs
     * @returns the VertexData of the box
     * @deprecated Please use CreateBoxVertexData from the BoxBuilder file instead
     */
    static CreateBox(options: {
        size?: number;
        width?: number;
        height?: number;
        depth?: number;
        faceUV?: Vector4[];
        faceColors?: Color4[];
        sideOrientation?: number;
        frontUVs?: Vector4;
        backUVs?: Vector4;
    }): VertexData;
    /**
     * Creates the VertexData for a tiled box
     * @param options an object used to set the following optional parameters for the box, required but can be empty
     * * faceTiles sets the pattern, tile size and number of tiles for a face
     * * faceUV an array of 6 Vector4 elements used to set different images to each box side
     * * faceColors an array of 6 Color3 elements used to set different colors to each box side
     * * sideOrientation optional and takes the values : Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
     * @param options.pattern
     * @param options.width
     * @param options.height
     * @param options.depth
     * @param options.tileSize
     * @param options.tileWidth
     * @param options.tileHeight
     * @param options.alignHorizontal
     * @param options.alignVertical
     * @param options.faceUV
     * @param options.faceColors
     * @param options.sideOrientation
     * @returns the VertexData of the box
     * @deprecated Please use CreateTiledBoxVertexData instead
     */
    static CreateTiledBox(options: {
        pattern?: number;
        width?: number;
        height?: number;
        depth?: number;
        tileSize?: number;
        tileWidth?: number;
        tileHeight?: number;
        alignHorizontal?: number;
        alignVertical?: number;
        faceUV?: Vector4[];
        faceColors?: Color4[];
        sideOrientation?: number;
    }): VertexData;
    /**
     * Creates the VertexData for a tiled plane
     * @param options an object used to set the following optional parameters for the box, required but can be empty
     * * pattern a limited pattern arrangement depending on the number
     * * tileSize sets the width, height and depth of the tile to the value of size, optional default 1
     * * tileWidth sets the width (x direction) of the tile, overwrites the width set by size, optional, default size
     * * tileHeight sets the height (y direction) of the tile, overwrites the height set by size, optional, default size
     * * sideOrientation optional and takes the values : Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
     * * frontUvs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the front side, optional, default vector4 (0, 0, 1, 1)
     * * backUVs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the back side, optional, default vector4 (0, 0, 1, 1)
     * @param options.pattern
     * @param options.tileSize
     * @param options.tileWidth
     * @param options.tileHeight
     * @param options.size
     * @param options.width
     * @param options.height
     * @param options.alignHorizontal
     * @param options.alignVertical
     * @param options.sideOrientation
     * @param options.frontUVs
     * @param options.backUVs
     * @returns the VertexData of the tiled plane
     * @deprecated use CreateTiledPlaneVertexData instead
     */
    static CreateTiledPlane(options: {
        pattern?: number;
        tileSize?: number;
        tileWidth?: number;
        tileHeight?: number;
        size?: number;
        width?: number;
        height?: number;
        alignHorizontal?: number;
        alignVertical?: number;
        sideOrientation?: number;
        frontUVs?: Vector4;
        backUVs?: Vector4;
    }): VertexData;
    /**
     * Creates the VertexData for an ellipsoid, defaults to a sphere
     * @param options an object used to set the following optional parameters for the box, required but can be empty
     * * segments sets the number of horizontal strips optional, default 32
     * * diameter sets the axes dimensions, diameterX, diameterY and diameterZ to the value of diameter, optional default 1
     * * diameterX sets the diameterX (x direction) of the ellipsoid, overwrites the diameterX set by diameter, optional, default diameter
     * * diameterY sets the diameterY (y direction) of the ellipsoid, overwrites the diameterY set by diameter, optional, default diameter
     * * diameterZ sets the diameterZ (z direction) of the ellipsoid, overwrites the diameterZ set by diameter, optional, default diameter
     * * arc a number from 0 to 1, to create an unclosed ellipsoid based on the fraction of the circumference (latitude) given by the arc value, optional, default 1
     * * slice a number from 0 to 1, to create an unclosed ellipsoid based on the fraction of the height (latitude) given by the arc value, optional, default 1
     * * sideOrientation optional and takes the values : Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
     * * frontUvs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the front side, optional, default vector4 (0, 0, 1, 1)
     * * backUVs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the back side, optional, default vector4 (0, 0, 1, 1)
     * @param options.segments
     * @param options.diameter
     * @param options.diameterX
     * @param options.diameterY
     * @param options.diameterZ
     * @param options.arc
     * @param options.slice
     * @param options.sideOrientation
     * @param options.frontUVs
     * @param options.backUVs
     * @returns the VertexData of the ellipsoid
     * @deprecated use CreateSphereVertexData instead
     */
    static CreateSphere(options: {
        segments?: number;
        diameter?: number;
        diameterX?: number;
        diameterY?: number;
        diameterZ?: number;
        arc?: number;
        slice?: number;
        sideOrientation?: number;
        frontUVs?: Vector4;
        backUVs?: Vector4;
    }): VertexData;
    /**
     * Creates the VertexData for a cylinder, cone or prism
     * @param options an object used to set the following optional parameters for the box, required but can be empty
     * * height sets the height (y direction) of the cylinder, optional, default 2
     * * diameterTop sets the diameter of the top of the cone, overwrites diameter,  optional, default diameter
     * * diameterBottom sets the diameter of the bottom of the cone, overwrites diameter,  optional, default diameter
     * * diameter sets the diameter of the top and bottom of the cone, optional default 1
     * * tessellation the number of prism sides, 3 for a triangular prism, optional, default 24
     * * subdivisions` the number of rings along the cylinder height, optional, default 1
     * * arc a number from 0 to 1, to create an unclosed cylinder based on the fraction of the circumference given by the arc value, optional, default 1
     * * faceColors an array of Color3 elements used to set different colors to the top, rings and bottom respectively
     * * faceUV an array of Vector4 elements used to set different images to the top, rings and bottom respectively
     * * hasRings when true makes each subdivision independently treated as a face for faceUV and faceColors, optional, default false
     * * enclose when true closes an open cylinder by adding extra flat faces between the height axis and vertical edges, think cut cake
     * * sideOrientation optional and takes the values : Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
     * * frontUvs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the front side, optional, default vector4 (0, 0, 1, 1)
     * * backUVs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the back side, optional, default vector4 (0, 0, 1, 1)
     * @param options.height
     * @param options.diameterTop
     * @param options.diameterBottom
     * @param options.diameter
     * @param options.tessellation
     * @param options.subdivisions
     * @param options.arc
     * @param options.faceColors
     * @param options.faceUV
     * @param options.hasRings
     * @param options.enclose
     * @param options.sideOrientation
     * @param options.frontUVs
     * @param options.backUVs
     * @returns the VertexData of the cylinder, cone or prism
     * @deprecated please use CreateCylinderVertexData instead
     */
    static CreateCylinder(options: {
        height?: number;
        diameterTop?: number;
        diameterBottom?: number;
        diameter?: number;
        tessellation?: number;
        subdivisions?: number;
        arc?: number;
        faceColors?: Color4[];
        faceUV?: Vector4[];
        hasRings?: boolean;
        enclose?: boolean;
        sideOrientation?: number;
        frontUVs?: Vector4;
        backUVs?: Vector4;
    }): VertexData;
    /**
     * Creates the VertexData for a torus
     * @param options an object used to set the following optional parameters for the box, required but can be empty
     * * diameter the diameter of the torus, optional default 1
     * * thickness the diameter of the tube forming the torus, optional default 0.5
     * * tessellation the number of prism sides, 3 for a triangular prism, optional, default 24
     * * sideOrientation optional and takes the values : Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
     * * frontUvs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the front side, optional, default vector4 (0, 0, 1, 1)
     * * backUVs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the back side, optional, default vector4 (0, 0, 1, 1)
     * @param options.diameter
     * @param options.thickness
     * @param options.tessellation
     * @param options.sideOrientation
     * @param options.frontUVs
     * @param options.backUVs
     * @returns the VertexData of the torus
     * @deprecated use CreateTorusVertexData instead
     */
    static CreateTorus(options: {
        diameter?: number;
        thickness?: number;
        tessellation?: number;
        sideOrientation?: number;
        frontUVs?: Vector4;
        backUVs?: Vector4;
    }): VertexData;
    /**
     * Creates the VertexData of the LineSystem
     * @param options an object used to set the following optional parameters for the LineSystem, required but can be empty
     *  - lines an array of lines, each line being an array of successive Vector3
     *  - colors an array of line colors, each of the line colors being an array of successive Color4, one per line point
     * @param options.lines
     * @param options.colors
     * @returns the VertexData of the LineSystem
     * @deprecated use CreateLineSystemVertexData instead
     */
    static CreateLineSystem(options: {
        lines: Vector3[][];
        colors?: Nullable<Color4[][]>;
    }): VertexData;
    /**
     * Create the VertexData for a DashedLines
     * @param options an object used to set the following optional parameters for the DashedLines, required but can be empty
     *  - points an array successive Vector3
     *  - dashSize the size of the dashes relative to the dash number, optional, default 3
     *  - gapSize the size of the gap between two successive dashes relative to the dash number, optional, default 1
     *  - dashNb the intended total number of dashes, optional, default 200
     * @param options.points
     * @param options.dashSize
     * @param options.gapSize
     * @param options.dashNb
     * @returns the VertexData for the DashedLines
     * @deprecated use CreateDashedLinesVertexData instead
     */
    static CreateDashedLines(options: {
        points: Vector3[];
        dashSize?: number;
        gapSize?: number;
        dashNb?: number;
    }): VertexData;
    /**
     * Creates the VertexData for a Ground
     * @param options an object used to set the following optional parameters for the Ground, required but can be empty
     *  - width the width (x direction) of the ground, optional, default 1
     *  - height the height (z direction) of the ground, optional, default 1
     *  - subdivisions the number of subdivisions per side, optional, default 1
     * @param options.width
     * @param options.height
     * @param options.subdivisions
     * @param options.subdivisionsX
     * @param options.subdivisionsY
     * @returns the VertexData of the Ground
     * @deprecated Please use CreateGroundVertexData instead
     */
    static CreateGround(options: {
        width?: number;
        height?: number;
        subdivisions?: number;
        subdivisionsX?: number;
        subdivisionsY?: number;
    }): VertexData;
    /**
     * Creates the VertexData for a TiledGround by subdividing the ground into tiles
     * @param options an object used to set the following optional parameters for the Ground, required but can be empty
     * * xmin the ground minimum X coordinate, optional, default -1
     * * zmin the ground minimum Z coordinate, optional, default -1
     * * xmax the ground maximum X coordinate, optional, default 1
     * * zmax the ground maximum Z coordinate, optional, default 1
     * * subdivisions a javascript object {w: positive integer, h: positive integer}, `w` and `h` are the numbers of subdivisions on the ground width and height creating 'tiles', default {w: 6, h: 6}
     * * precision a javascript object {w: positive integer, h: positive integer}, `w` and `h` are the numbers of subdivisions on the tile width and height, default {w: 2, h: 2}
     * @param options.xmin
     * @param options.zmin
     * @param options.xmax
     * @param options.zmax
     * @param options.subdivisions
     * @param options.subdivisions.w
     * @param options.subdivisions.h
     * @param options.precision
     * @param options.precision.w
     * @param options.precision.h
     * @returns the VertexData of the TiledGround
     * @deprecated use CreateTiledGroundVertexData instead
     */
    static CreateTiledGround(options: {
        xmin: number;
        zmin: number;
        xmax: number;
        zmax: number;
        subdivisions?: {
            w: number;
            h: number;
        };
        precision?: {
            w: number;
            h: number;
        };
    }): VertexData;
    /**
     * Creates the VertexData of the Ground designed from a heightmap
     * @param options an object used to set the following parameters for the Ground, required and provided by CreateGroundFromHeightMap
     * * width the width (x direction) of the ground
     * * height the height (z direction) of the ground
     * * subdivisions the number of subdivisions per side
     * * minHeight the minimum altitude on the ground, optional, default 0
     * * maxHeight the maximum altitude on the ground, optional default 1
     * * colorFilter the filter to apply to the image pixel colors to compute the height, optional Color3, default (0.3, 0.59, 0.11)
     * * buffer the array holding the image color data
     * * bufferWidth the width of image
     * * bufferHeight the height of image
     * * alphaFilter Remove any data where the alpha channel is below this value, defaults 0 (all data visible)
     * @param options.width
     * @param options.height
     * @param options.subdivisions
     * @param options.minHeight
     * @param options.maxHeight
     * @param options.colorFilter
     * @param options.buffer
     * @param options.bufferWidth
     * @param options.bufferHeight
     * @param options.alphaFilter
     * @returns the VertexData of the Ground designed from a heightmap
     * @deprecated use CreateGroundFromHeightMapVertexData instead
     */
    static CreateGroundFromHeightMap(options: {
        width: number;
        height: number;
        subdivisions: number;
        minHeight: number;
        maxHeight: number;
        colorFilter: Color3;
        buffer: Uint8Array;
        bufferWidth: number;
        bufferHeight: number;
        alphaFilter: number;
    }): VertexData;
    /**
     * Creates the VertexData for a Plane
     * @param options an object used to set the following optional parameters for the plane, required but can be empty
     * * size sets the width and height of the plane to the value of size, optional default 1
     * * width sets the width (x direction) of the plane, overwrites the width set by size, optional, default size
     * * height sets the height (y direction) of the plane, overwrites the height set by size, optional, default size
     * * sideOrientation optional and takes the values : Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
     * * frontUvs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the front side, optional, default vector4 (0, 0, 1, 1)
     * * backUVs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the back side, optional, default vector4 (0, 0, 1, 1)
     * @param options.size
     * @param options.width
     * @param options.height
     * @param options.sideOrientation
     * @param options.frontUVs
     * @param options.backUVs
     * @returns the VertexData of the box
     * @deprecated use CreatePlaneVertexData instead
     */
    static CreatePlane(options: {
        size?: number;
        width?: number;
        height?: number;
        sideOrientation?: number;
        frontUVs?: Vector4;
        backUVs?: Vector4;
    }): VertexData;
    /**
     * Creates the VertexData of the Disc or regular Polygon
     * @param options an object used to set the following optional parameters for the disc, required but can be empty
     * * radius the radius of the disc, optional default 0.5
     * * tessellation the number of polygon sides, optional, default 64
     * * arc a number from 0 to 1, to create an unclosed polygon based on the fraction of the circumference given by the arc value, optional, default 1
     * * sideOrientation optional and takes the values : Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
     * * frontUvs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the front side, optional, default vector4 (0, 0, 1, 1)
     * * backUVs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the back side, optional, default vector4 (0, 0, 1, 1)
     * @param options.radius
     * @param options.tessellation
     * @param options.arc
     * @param options.sideOrientation
     * @param options.frontUVs
     * @param options.backUVs
     * @returns the VertexData of the box
     * @deprecated use CreateDiscVertexData instead
     */
    static CreateDisc(options: {
        radius?: number;
        tessellation?: number;
        arc?: number;
        sideOrientation?: number;
        frontUVs?: Vector4;
        backUVs?: Vector4;
    }): VertexData;
    /**
     * Creates the VertexData for an irregular Polygon in the XoZ plane using a mesh built by polygonTriangulation.build()
     * All parameters are provided by CreatePolygon as needed
     * @param polygon a mesh built from polygonTriangulation.build()
     * @param sideOrientation takes the values Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
     * @param fUV an array of Vector4 elements used to set different images to the top, rings and bottom respectively
     * @param fColors an array of Color3 elements used to set different colors to the top, rings and bottom respectively
     * @param frontUVs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the front side, optional, default vector4 (0, 0, 1, 1)
     * @param backUVs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the back side, optional, default vector4 (0, 0, 1, 1)
     * @param wrap a boolean, default false, when true and fUVs used texture is wrapped around all sides, when false texture is applied side
     * @returns the VertexData of the Polygon
     * @deprecated use CreatePolygonVertexData instead
     */
    static CreatePolygon(polygon: Mesh, sideOrientation: number, fUV?: Vector4[], fColors?: Color4[], frontUVs?: Vector4, backUVs?: Vector4, wrap?: boolean): VertexData;
    /**
     * Creates the VertexData of the IcoSphere
     * @param options an object used to set the following optional parameters for the IcoSphere, required but can be empty
     * * radius the radius of the IcoSphere, optional default 1
     * * radiusX allows stretching in the x direction, optional, default radius
     * * radiusY allows stretching in the y direction, optional, default radius
     * * radiusZ allows stretching in the z direction, optional, default radius
     * * flat when true creates a flat shaded mesh, optional, default true
     * * subdivisions increasing the subdivisions increases the number of faces, optional, default 4
     * * sideOrientation optional and takes the values : Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
     * * frontUvs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the front side, optional, default vector4 (0, 0, 1, 1)
     * * backUVs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the back side, optional, default vector4 (0, 0, 1, 1)
     * @param options.radius
     * @param options.radiusX
     * @param options.radiusY
     * @param options.radiusZ
     * @param options.flat
     * @param options.subdivisions
     * @param options.sideOrientation
     * @param options.frontUVs
     * @param options.backUVs
     * @returns the VertexData of the IcoSphere
     * @deprecated use CreateIcoSphereVertexData instead
     */
    static CreateIcoSphere(options: {
        radius?: number;
        radiusX?: number;
        radiusY?: number;
        radiusZ?: number;
        flat?: boolean;
        subdivisions?: number;
        sideOrientation?: number;
        frontUVs?: Vector4;
        backUVs?: Vector4;
    }): VertexData;
    /**
     * Creates the VertexData for a Polyhedron
     * @param options an object used to set the following optional parameters for the polyhedron, required but can be empty
     * * type provided types are:
     *  * 0 : Tetrahedron, 1 : Octahedron, 2 : Dodecahedron, 3 : Icosahedron, 4 : Rhombicuboctahedron, 5 : Triangular Prism, 6 : Pentagonal Prism, 7 : Hexagonal Prism, 8 : Square Pyramid (J1)
     *  * 9 : Pentagonal Pyramid (J2), 10 : Triangular Dipyramid (J12), 11 : Pentagonal Dipyramid (J13), 12 : Elongated Square Dipyramid (J15), 13 : Elongated Pentagonal Dipyramid (J16), 14 : Elongated Pentagonal Cupola (J20)
     * * size the size of the IcoSphere, optional default 1
     * * sizeX allows stretching in the x direction, optional, default size
     * * sizeY allows stretching in the y direction, optional, default size
     * * sizeZ allows stretching in the z direction, optional, default size
     * * custom a number that overwrites the type to create from an extended set of polyhedron from https://www.babylonjs-playground.com/#21QRSK#15 with minimised editor
     * * faceUV an array of Vector4 elements used to set different images to the top, rings and bottom respectively
     * * faceColors an array of Color3 elements used to set different colors to the top, rings and bottom respectively
     * * flat when true creates a flat shaded mesh, optional, default true
     * * subdivisions increasing the subdivisions increases the number of faces, optional, default 4
     * * sideOrientation optional and takes the values : Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
     * * frontUvs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the front side, optional, default vector4 (0, 0, 1, 1)
     * * backUVs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the back side, optional, default vector4 (0, 0, 1, 1)
     * @param options.type
     * @param options.size
     * @param options.sizeX
     * @param options.sizeY
     * @param options.sizeZ
     * @param options.custom
     * @param options.faceUV
     * @param options.faceColors
     * @param options.flat
     * @param options.sideOrientation
     * @param options.frontUVs
     * @param options.backUVs
     * @returns the VertexData of the Polyhedron
     * @deprecated use CreatePolyhedronVertexData instead
     */
    static CreatePolyhedron(options: {
        type?: number;
        size?: number;
        sizeX?: number;
        sizeY?: number;
        sizeZ?: number;
        custom?: any;
        faceUV?: Vector4[];
        faceColors?: Color4[];
        flat?: boolean;
        sideOrientation?: number;
        frontUVs?: Vector4;
        backUVs?: Vector4;
    }): VertexData;
    /**
     * Creates the VertexData for a Capsule, inspired from https://github.com/maximeq/three-js-capsule-geometry/blob/master/src/CapsuleBufferGeometry.js
     * @param options an object used to set the following optional parameters for the capsule, required but can be empty
     * @returns the VertexData of the Capsule
     * @deprecated Please use CreateCapsuleVertexData from the capsuleBuilder file instead
     */
    static CreateCapsule(options?: ICreateCapsuleOptions): VertexData;
    /**
     * Creates the VertexData for a TorusKnot
     * @param options an object used to set the following optional parameters for the TorusKnot, required but can be empty
     * * radius the radius of the torus knot, optional, default 2
     * * tube the thickness of the tube, optional, default 0.5
     * * radialSegments the number of sides on each tube segments, optional, default 32
     * * tubularSegments the number of tubes to decompose the knot into, optional, default 32
     * * p the number of windings around the z axis, optional,  default 2
     * * q the number of windings around the x axis, optional,  default 3
     * * sideOrientation optional and takes the values : Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
     * * frontUvs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the front side, optional, default vector4 (0, 0, 1, 1)
     * * backUVs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the back side, optional, default vector4 (0, 0, 1, 1)
     * @param options.radius
     * @param options.tube
     * @param options.radialSegments
     * @param options.tubularSegments
     * @param options.p
     * @param options.q
     * @param options.sideOrientation
     * @param options.frontUVs
     * @param options.backUVs
     * @returns the VertexData of the Torus Knot
     * @deprecated use CreateTorusKnotVertexData instead
     */
    static CreateTorusKnot(options: {
        radius?: number;
        tube?: number;
        radialSegments?: number;
        tubularSegments?: number;
        p?: number;
        q?: number;
        sideOrientation?: number;
        frontUVs?: Vector4;
        backUVs?: Vector4;
    }): VertexData;
    /**
     * Compute normals for given positions and indices
     * @param positions an array of vertex positions, [...., x, y, z, ......]
     * @param indices an array of indices in groups of three for each triangular facet, [...., i, j, k, ......]
     * @param normals an array of vertex normals, [...., x, y, z, ......]
     * @param options an object used to set the following optional parameters for the TorusKnot, optional
     * * facetNormals : optional array of facet normals (vector3)
     * * facetPositions : optional array of facet positions (vector3)
     * * facetPartitioning : optional partitioning array. facetPositions is required for facetPartitioning computation
     * * ratio : optional partitioning ratio / bounding box, required for facetPartitioning computation
     * * bInfo : optional bounding info, required for facetPartitioning computation
     * * bbSize : optional bounding box size data, required for facetPartitioning computation
     * * subDiv : optional partitioning data about subdivisions on  each axis (int), required for facetPartitioning computation
     * * useRightHandedSystem: optional boolean to for right handed system computation
     * * depthSort : optional boolean to enable the facet depth sort computation
     * * distanceTo : optional Vector3 to compute the facet depth from this location
     * * depthSortedFacets : optional array of depthSortedFacets to store the facet distances from the reference location
     * @param options.facetNormals
     * @param options.facetPositions
     * @param options.facetPartitioning
     * @param options.ratio
     * @param options.bInfo
     * @param options.bbSize
     * @param options.subDiv
     * @param options.useRightHandedSystem
     * @param options.depthSort
     * @param options.distanceTo
     * @param options.depthSortedFacets
     */
    static ComputeNormals(positions: any, indices: any, normals: any, options?: {
        facetNormals?: any;
        facetPositions?: any;
        facetPartitioning?: any;
        ratio?: number;
        bInfo?: any;
        bbSize?: Vector3;
        subDiv?: any;
        useRightHandedSystem?: boolean;
        depthSort?: boolean;
        distanceTo?: Vector3;
        depthSortedFacets?: any;
    }): void;
    /**
     * @internal
     */
    static _ComputeSides(sideOrientation: number, positions: FloatArray, indices: FloatArray | IndicesArray, normals: FloatArray, uvs: FloatArray, frontUVs?: Vector4, backUVs?: Vector4): void;
    /**
     * Applies VertexData created from the imported parameters to the geometry
     * @param parsedVertexData the parsed data from an imported file
     * @param geometry the geometry to apply the VertexData to
     */
    static ImportVertexData(parsedVertexData: any, geometry: Geometry): void;
}
export {};

}
declare module "babylonjs/Meshes/meshBuilder" {
import { CreateRibbon } from "babylonjs/Meshes/Builders/ribbonBuilder";
import { CreateDisc } from "babylonjs/Meshes/Builders/discBuilder";
import { CreateBox } from "babylonjs/Meshes/Builders/boxBuilder";
import { CreateTiledBox } from "babylonjs/Meshes/Builders/tiledBoxBuilder";
import { CreateSphere } from "babylonjs/Meshes/Builders/sphereBuilder";
import { CreateCylinder } from "babylonjs/Meshes/Builders/cylinderBuilder";
import { CreateTorus } from "babylonjs/Meshes/Builders/torusBuilder";
import { CreateTorusKnot } from "babylonjs/Meshes/Builders/torusKnotBuilder";
import { CreateDashedLines, CreateLineSystem, CreateLines } from "babylonjs/Meshes/Builders/linesBuilder";
import { CreatePolygon, ExtrudePolygon } from "babylonjs/Meshes/Builders/polygonBuilder";
import { ExtrudeShape, ExtrudeShapeCustom } from "babylonjs/Meshes/Builders/shapeBuilder";
import { CreateLathe } from "babylonjs/Meshes/Builders/latheBuilder";
import { CreatePlane } from "babylonjs/Meshes/Builders/planeBuilder";
import { CreateTiledPlane } from "babylonjs/Meshes/Builders/tiledPlaneBuilder";
import { CreateGround, CreateGroundFromHeightMap, CreateTiledGround } from "babylonjs/Meshes/Builders/groundBuilder";
import { CreateTube } from "babylonjs/Meshes/Builders/tubeBuilder";
import { CreatePolyhedron } from "babylonjs/Meshes/Builders/polyhedronBuilder";
import { CreateIcoSphere } from "babylonjs/Meshes/Builders/icoSphereBuilder";
import { CreateDecal } from "babylonjs/Meshes/Builders/decalBuilder";
import { CreateCapsule } from "babylonjs/Meshes/Builders/capsuleBuilder";
import { CreateGeodesic } from "babylonjs/Meshes/Builders/geodesicBuilder";
import { CreateGoldberg } from "babylonjs/Meshes/Builders/goldbergBuilder";
/**
 * Class containing static functions to help procedurally build meshes
 */
export const MeshBuilder: {
    CreateBox: typeof CreateBox;
    CreateTiledBox: typeof CreateTiledBox;
    CreateSphere: typeof CreateSphere;
    CreateDisc: typeof CreateDisc;
    CreateIcoSphere: typeof CreateIcoSphere;
    CreateRibbon: typeof CreateRibbon;
    CreateCylinder: typeof CreateCylinder;
    CreateTorus: typeof CreateTorus;
    CreateTorusKnot: typeof CreateTorusKnot;
    CreateLineSystem: typeof CreateLineSystem;
    CreateLines: typeof CreateLines;
    CreateDashedLines: typeof CreateDashedLines;
    ExtrudeShape: typeof ExtrudeShape;
    ExtrudeShapeCustom: typeof ExtrudeShapeCustom;
    CreateLathe: typeof CreateLathe;
    CreateTiledPlane: typeof CreateTiledPlane;
    CreatePlane: typeof CreatePlane;
    CreateGround: typeof CreateGround;
    CreateTiledGround: typeof CreateTiledGround;
    CreateGroundFromHeightMap: typeof CreateGroundFromHeightMap;
    CreatePolygon: typeof CreatePolygon;
    ExtrudePolygon: typeof ExtrudePolygon;
    CreateTube: typeof CreateTube;
    CreatePolyhedron: typeof CreatePolyhedron;
    CreateGeodesic: typeof CreateGeodesic;
    CreateGoldberg: typeof CreateGoldberg;
    CreateDecal: typeof CreateDecal;
    CreateCapsule: typeof CreateCapsule;
};

}
declare module "babylonjs/Meshes/meshLODLevel" {
import { Mesh } from "babylonjs/Meshes/mesh";
import { Nullable } from "babylonjs/types";
/**
 * Class used to represent a specific level of detail of a mesh
 * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/LOD
 */
export class MeshLODLevel {
    /** Either distance from the center of the object to show this level or the screen coverage if `useLODScreenCoverage` is set to `true` on the mesh*/
    distanceOrScreenCoverage: number;
    /** Defines the mesh to use to render this level */
    mesh: Nullable<Mesh>;
    /**
     * Creates a new LOD level
     * @param distanceOrScreenCoverage defines either the distance or the screen coverage where this level should start being displayed
     * @param mesh defines the mesh to use to render this level
     */
    constructor(
    /** Either distance from the center of the object to show this level or the screen coverage if `useLODScreenCoverage` is set to `true` on the mesh*/
    distanceOrScreenCoverage: number, 
    /** Defines the mesh to use to render this level */
    mesh: Nullable<Mesh>);
}

}
declare module "babylonjs/Meshes/meshSimplification" {
import { Mesh } from "babylonjs/Meshes/mesh";
/**
 * A simplifier interface for future simplification implementations
 * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/simplifyingMeshes
 */
export interface ISimplifier {
    /**
     * Simplification of a given mesh according to the given settings.
     * Since this requires computation, it is assumed that the function runs async.
     * @param settings The settings of the simplification, including quality and distance
     * @param successCallback A callback that will be called after the mesh was simplified.
     * @param errorCallback in case of an error, this callback will be called. optional.
     */
    simplify(settings: ISimplificationSettings, successCallback: (simplifiedMeshes: Mesh) => void, errorCallback?: () => void): void;
}
/**
 * Expected simplification settings.
 * Quality should be between 0 and 1 (1 being 100%, 0 being 0%)
 * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/simplifyingMeshes
 */
export interface ISimplificationSettings {
    /**
     * Gets or sets the expected quality
     */
    quality: number;
    /**
     * Gets or sets the distance when this optimized version should be used
     */
    distance: number;
    /**
     * Gets an already optimized mesh
     */
    optimizeMesh?: boolean;
}
/**
 * Class used to specify simplification options
 * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/simplifyingMeshes
 */
export class SimplificationSettings implements ISimplificationSettings {
    /** expected quality */
    quality: number;
    /** distance when this optimized version should be used */
    distance: number;
    /** already optimized mesh  */
    optimizeMesh?: boolean | undefined;
    /**
     * Creates a SimplificationSettings
     * @param quality expected quality
     * @param distance distance when this optimized version should be used
     * @param optimizeMesh already optimized mesh
     */
    constructor(
    /** expected quality */
    quality: number, 
    /** distance when this optimized version should be used */
    distance: number, 
    /** already optimized mesh  */
    optimizeMesh?: boolean | undefined);
}
/**
 * Interface used to define a simplification task
 */
export interface ISimplificationTask {
    /**
     * Array of settings
     */
    settings: Array<ISimplificationSettings>;
    /**
     * Simplification type
     */
    simplificationType: SimplificationType;
    /**
     * Mesh to simplify
     */
    mesh: Mesh;
    /**
     * Callback called on success
     */
    successCallback?: () => void;
    /**
     * Defines if parallel processing can be used
     */
    parallelProcessing: boolean;
}
/**
 * Queue used to order the simplification tasks
 * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/simplifyingMeshes
 */
export class SimplificationQueue {
    private _simplificationArray;
    /**
     * Gets a boolean indicating that the process is still running
     */
    running: boolean;
    /**
     * Creates a new queue
     */
    constructor();
    /**
     * Adds a new simplification task
     * @param task defines a task to add
     */
    addTask(task: ISimplificationTask): void;
    /**
     * Execute next task
     */
    executeNext(): void;
    /**
     * Execute a simplification task
     * @param task defines the task to run
     */
    runSimplification(task: ISimplificationTask): void;
    private _getSimplifier;
}
/**
 * The implemented types of simplification
 * At the moment only Quadratic Error Decimation is implemented
 * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/simplifyingMeshes
 */
export enum SimplificationType {
    /** Quadratic error decimation */
    QUADRATIC = 0
}
/**
 * An implementation of the Quadratic Error simplification algorithm.
 * Original paper : http://www1.cs.columbia.edu/~cs4162/html05s/garland97.pdf
 * Ported mostly from QSlim and http://voxels.blogspot.de/2014/05/quadric-mesh-simplification-with-source.html to babylon JS
 * @author RaananW
 * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/simplifyingMeshes
 */
export class QuadraticErrorSimplification implements ISimplifier {
    private _mesh;
    private _triangles;
    private _vertices;
    private _references;
    private _reconstructedMesh;
    /** Gets or sets the number pf sync iterations */
    syncIterations: number;
    /** Gets or sets the aggressiveness of the simplifier */
    aggressiveness: number;
    /** Gets or sets the number of allowed iterations for decimation */
    decimationIterations: number;
    /** Gets or sets the espilon to use for bounding box computation */
    boundingBoxEpsilon: number;
    /**
     * Creates a new QuadraticErrorSimplification
     * @param _mesh defines the target mesh
     */
    constructor(_mesh: Mesh);
    /**
     * Simplification of a given mesh according to the given settings.
     * Since this requires computation, it is assumed that the function runs async.
     * @param settings The settings of the simplification, including quality and distance
     * @param successCallback A callback that will be called after the mesh was simplified.
     */
    simplify(settings: ISimplificationSettings, successCallback: (simplifiedMesh: Mesh) => void): void;
    private _runDecimation;
    private _initWithMesh;
    private _init;
    private _reconstructMesh;
    private _initDecimatedMesh;
    private _isFlipped;
    private _updateTriangles;
    private _identifyBorder;
    private _updateMesh;
    private _vertexError;
    private _calculateError;
}

}
declare module "babylonjs/Meshes/meshSimplificationSceneComponent" {
import { Scene } from "babylonjs/scene";
import { ISimplificationSettings } from "babylonjs/Meshes/meshSimplification";
import { SimplificationQueue, SimplificationType } from "babylonjs/Meshes/meshSimplification";
import { ISceneComponent } from "babylonjs/sceneComponent";
module "babylonjs/scene" {
    interface Scene {
        /** @internal (Backing field) */
        _simplificationQueue: SimplificationQueue;
        /**
         * Gets or sets the simplification queue attached to the scene
         * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/simplifyingMeshes
         */
        simplificationQueue: SimplificationQueue;
    }
}
module "babylonjs/Meshes/mesh" {
    interface Mesh {
        /**
         * Simplify the mesh according to the given array of settings.
         * Function will return immediately and will simplify async
         * @param settings a collection of simplification settings
         * @param parallelProcessing should all levels calculate parallel or one after the other
         * @param simplificationType the type of simplification to run
         * @param successCallback optional success callback to be called after the simplification finished processing all settings
         * @returns the current mesh
         */
        simplify(settings: Array<ISimplificationSettings>, parallelProcessing?: boolean, simplificationType?: SimplificationType, successCallback?: (mesh?: Mesh, submeshIndex?: number) => void): Mesh;
    }
}
/**
 * Defines the simplification queue scene component responsible to help scheduling the various simplification task
 * created in a scene
 */
export class SimplicationQueueSceneComponent implements ISceneComponent {
    /**
     * The component name helpfull to identify the component in the list of scene components.
     */
    readonly name: string;
    /**
     * The scene the component belongs to.
     */
    scene: Scene;
    /**
     * Creates a new instance of the component for the given scene
     * @param scene Defines the scene to register the component in
     */
    constructor(scene: Scene);
    /**
     * Registers the component in a given scene
     */
    register(): void;
    /**
     * Rebuilds the elements related to this component in case of
     * context lost for instance.
     */
    rebuild(): void;
    /**
     * Disposes the component and the associated resources
     */
    dispose(): void;
    private _beforeCameraUpdate;
}

}
declare module "babylonjs/Meshes/polygonMesh" {
import { Scene } from "babylonjs/scene";
import { Vector2 } from "babylonjs/Maths/math.vector";
import { Mesh } from "babylonjs/Meshes/mesh";
import { VertexData } from "babylonjs/Meshes/mesh.vertexData";
import { Path2 } from "babylonjs/Maths/math.path";
/**
 * Polygon
 * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/param#non-regular-polygon
 */
export class Polygon {
    /**
     * Creates a rectangle
     * @param xmin bottom X coord
     * @param ymin bottom Y coord
     * @param xmax top X coord
     * @param ymax top Y coord
     * @returns points that make the resulting rectangle
     */
    static Rectangle(xmin: number, ymin: number, xmax: number, ymax: number): Vector2[];
    /**
     * Creates a circle
     * @param radius radius of circle
     * @param cx scale in x
     * @param cy scale in y
     * @param numberOfSides number of sides that make up the circle
     * @returns points that make the resulting circle
     */
    static Circle(radius: number, cx?: number, cy?: number, numberOfSides?: number): Vector2[];
    /**
     * Creates a polygon from input string
     * @param input Input polygon data
     * @returns the parsed points
     */
    static Parse(input: string): Vector2[];
    /**
     * Starts building a polygon from x and y coordinates
     * @param x x coordinate
     * @param y y coordinate
     * @returns the started path2
     */
    static StartingAt(x: number, y: number): Path2;
}
/**
 * Builds a polygon
 * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/param/polyMeshBuilder
 */
export class PolygonMeshBuilder {
    private _points;
    private _outlinepoints;
    private _holes;
    private _name;
    private _scene;
    private _epoints;
    private _eholes;
    private _addToepoint;
    /**
     * Babylon reference to the earcut plugin.
     */
    bjsEarcut: any;
    /**
     * Creates a PolygonMeshBuilder
     * @param name name of the builder
     * @param contours Path of the polygon
     * @param scene scene to add to when creating the mesh
     * @param earcutInjection can be used to inject your own earcut reference
     */
    constructor(name: string, contours: Path2 | Vector2[] | any, scene?: Scene, earcutInjection?: any);
    /**
     * Adds a hole within the polygon
     * @param hole Array of points defining the hole
     * @returns this
     */
    addHole(hole: Vector2[]): PolygonMeshBuilder;
    /**
     * Creates the polygon
     * @param updatable If the mesh should be updatable
     * @param depth The depth of the mesh created
     * @param smoothingThreshold Dot product threshold for smoothed normals
     * @returns the created mesh
     */
    build(updatable?: boolean, depth?: number, smoothingThreshold?: number): Mesh;
    /**
     * Creates the polygon
     * @param depth The depth of the mesh created
     * @param smoothingThreshold Dot product threshold for smoothed normals
     * @returns the created VertexData
     */
    buildVertexData(depth?: number, smoothingThreshold?: number): VertexData;
    /**
     * Adds a side to the polygon
     * @param positions points that make the polygon
     * @param normals normals of the polygon
     * @param uvs uvs of the polygon
     * @param indices indices of the polygon
     * @param bounds bounds of the polygon
     * @param points points of the polygon
     * @param depth depth of the polygon
     * @param flip flip of the polygon
     * @param smoothingThreshold
     */
    private _addSide;
}

}
declare module "babylonjs/Meshes/subMesh" {
import { Nullable, IndicesArray, DeepImmutable, FloatArray } from "babylonjs/types";
import { Matrix, Vector3 } from "babylonjs/Maths/math.vector";
import { Engine } from "babylonjs/Engines/engine";
import { IntersectionInfo } from "babylonjs/Collisions/intersectionInfo";
import { ICullable } from "babylonjs/Culling/boundingInfo";
import { BoundingInfo } from "babylonjs/Culling/boundingInfo";
import { Effect } from "babylonjs/Materials/effect";
import { DataBuffer } from "babylonjs/Buffers/dataBuffer";
import { Plane } from "babylonjs/Maths/math.plane";
import { DrawWrapper } from "babylonjs/Materials/drawWrapper";
import { IMaterialContext } from "babylonjs/Engines/IMaterialContext";
import { Collider } from "babylonjs/Collisions/collider";
import { Material } from "babylonjs/Materials/material";
import { MaterialDefines } from "babylonjs/Materials/materialDefines";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { Mesh } from "babylonjs/Meshes/mesh";
import { Ray } from "babylonjs/Culling/ray";
import { TrianglePickingPredicate } from "babylonjs/Culling/ray";
/**
 * Defines a subdivision inside a mesh
 */
export class SubMesh implements ICullable {
    /** the material index to use */
    materialIndex: number;
    /** vertex index start */
    verticesStart: number;
    /** vertices count */
    verticesCount: number;
    /** index start */
    indexStart: number;
    /** indices count */
    indexCount: number;
    private _engine;
    /** @internal */
    _drawWrappers: Array<DrawWrapper>;
    private _mainDrawWrapperOverride;
    /**
     * Gets material defines used by the effect associated to the sub mesh
     */
    get materialDefines(): Nullable<MaterialDefines>;
    /**
     * Sets material defines used by the effect associated to the sub mesh
     */
    set materialDefines(defines: Nullable<MaterialDefines>);
    /**
     * @internal
     */
    _getDrawWrapper(passId?: number, createIfNotExisting?: boolean): DrawWrapper | undefined;
    /**
     * @internal
     */
    _removeDrawWrapper(passId: number, disposeWrapper?: boolean): void;
    /**
     * Gets associated (main) effect (possibly the effect override if defined)
     */
    get effect(): Nullable<Effect>;
    /** @internal */
    get _drawWrapper(): DrawWrapper;
    /** @internal */
    get _drawWrapperOverride(): Nullable<DrawWrapper>;
    /**
     * @internal
     */
    _setMainDrawWrapperOverride(wrapper: Nullable<DrawWrapper>): void;
    /**
     * Sets associated effect (effect used to render this submesh)
     * @param effect defines the effect to associate with
     * @param defines defines the set of defines used to compile this effect
     * @param materialContext material context associated to the effect
     * @param resetContext true to reset the draw context
     */
    setEffect(effect: Nullable<Effect>, defines?: Nullable<string | MaterialDefines>, materialContext?: IMaterialContext, resetContext?: boolean): void;
    /**
     * Resets the draw wrappers cache
     * @param passId If provided, releases only the draw wrapper corresponding to this render pass id
     */
    resetDrawCache(passId?: number): void;
    /** @internal */
    _linesIndexCount: number;
    private _mesh;
    private _renderingMesh;
    private _boundingInfo;
    private _linesIndexBuffer;
    /** @internal */
    _lastColliderWorldVertices: Nullable<Vector3[]>;
    /** @internal */
    _trianglePlanes: Plane[];
    /** @internal */
    _lastColliderTransformMatrix: Nullable<Matrix>;
    /** @internal */
    _wasDispatched: boolean;
    /** @internal */
    _renderId: number;
    /** @internal */
    _alphaIndex: number;
    /** @internal */
    _distanceToCamera: number;
    /** @internal */
    _id: number;
    private _currentMaterial;
    /**
     * Add a new submesh to a mesh
     * @param materialIndex defines the material index to use
     * @param verticesStart defines vertex index start
     * @param verticesCount defines vertices count
     * @param indexStart defines index start
     * @param indexCount defines indices count
     * @param mesh defines the parent mesh
     * @param renderingMesh defines an optional rendering mesh
     * @param createBoundingBox defines if bounding box should be created for this submesh
     * @returns the new submesh
     */
    static AddToMesh(materialIndex: number, verticesStart: number, verticesCount: number, indexStart: number, indexCount: number, mesh: AbstractMesh, renderingMesh?: Mesh, createBoundingBox?: boolean): SubMesh;
    /**
     * Creates a new submesh
     * @param materialIndex defines the material index to use
     * @param verticesStart defines vertex index start
     * @param verticesCount defines vertices count
     * @param indexStart defines index start
     * @param indexCount defines indices count
     * @param mesh defines the parent mesh
     * @param renderingMesh defines an optional rendering mesh
     * @param createBoundingBox defines if bounding box should be created for this submesh
     * @param addToMesh defines a boolean indicating that the submesh must be added to the mesh.subMeshes array (true by default)
     */
    constructor(
    /** the material index to use */
    materialIndex: number, 
    /** vertex index start */
    verticesStart: number, 
    /** vertices count */
    verticesCount: number, 
    /** index start */
    indexStart: number, 
    /** indices count */
    indexCount: number, mesh: AbstractMesh, renderingMesh?: Mesh, createBoundingBox?: boolean, addToMesh?: boolean);
    /**
     * Returns true if this submesh covers the entire parent mesh
     * @ignorenaming
     */
    get IsGlobal(): boolean;
    /**
     * Returns the submesh BoundingInfo object
     * @returns current bounding info (or mesh's one if the submesh is global)
     */
    getBoundingInfo(): BoundingInfo;
    /**
     * Sets the submesh BoundingInfo
     * @param boundingInfo defines the new bounding info to use
     * @returns the SubMesh
     */
    setBoundingInfo(boundingInfo: BoundingInfo): SubMesh;
    /**
     * Returns the mesh of the current submesh
     * @returns the parent mesh
     */
    getMesh(): AbstractMesh;
    /**
     * Returns the rendering mesh of the submesh
     * @returns the rendering mesh (could be different from parent mesh)
     */
    getRenderingMesh(): Mesh;
    /**
     * Returns the replacement mesh of the submesh
     * @returns the replacement mesh (could be different from parent mesh)
     */
    getReplacementMesh(): Nullable<AbstractMesh>;
    /**
     * Returns the effective mesh of the submesh
     * @returns the effective mesh (could be different from parent mesh)
     */
    getEffectiveMesh(): AbstractMesh;
    /**
     * Returns the submesh material
     * @param getDefaultMaterial Defines whether or not to get the default material if nothing has been defined.
     * @returns null or the current material
     */
    getMaterial(getDefaultMaterial?: boolean): Nullable<Material>;
    private _isMultiMaterial;
    /**
     * Sets a new updated BoundingInfo object to the submesh
     * @param data defines an optional position array to use to determine the bounding info
     * @returns the SubMesh
     */
    refreshBoundingInfo(data?: Nullable<FloatArray>): SubMesh;
    /**
     * @internal
     */
    _checkCollision(collider: Collider): boolean;
    /**
     * Updates the submesh BoundingInfo
     * @param world defines the world matrix to use to update the bounding info
     * @returns the submesh
     */
    updateBoundingInfo(world: DeepImmutable<Matrix>): SubMesh;
    /**
     * True is the submesh bounding box intersects the frustum defined by the passed array of planes.
     * @param frustumPlanes defines the frustum planes
     * @returns true if the submesh is intersecting with the frustum
     */
    isInFrustum(frustumPlanes: Plane[]): boolean;
    /**
     * True is the submesh bounding box is completely inside the frustum defined by the passed array of planes
     * @param frustumPlanes defines the frustum planes
     * @returns true if the submesh is inside the frustum
     */
    isCompletelyInFrustum(frustumPlanes: Plane[]): boolean;
    /**
     * Renders the submesh
     * @param enableAlphaMode defines if alpha needs to be used
     * @returns the submesh
     */
    render(enableAlphaMode: boolean): SubMesh;
    /**
     * @internal
     */
    _getLinesIndexBuffer(indices: IndicesArray, engine: Engine): DataBuffer;
    /**
     * Checks if the submesh intersects with a ray
     * @param ray defines the ray to test
     * @returns true is the passed ray intersects the submesh bounding box
     */
    canIntersects(ray: Ray): boolean;
    /**
     * Intersects current submesh with a ray
     * @param ray defines the ray to test
     * @param positions defines mesh's positions array
     * @param indices defines mesh's indices array
     * @param fastCheck defines if the first intersection will be used (and not the closest)
     * @param trianglePredicate defines an optional predicate used to select faces when a mesh intersection is detected
     * @returns intersection info or null if no intersection
     */
    intersects(ray: Ray, positions: Vector3[], indices: IndicesArray, fastCheck?: boolean, trianglePredicate?: TrianglePickingPredicate): Nullable<IntersectionInfo>;
    /**
     * @internal
     */
    private _intersectLines;
    /**
     * @internal
     */
    private _intersectUnIndexedLines;
    /**
     * @internal
     */
    private _intersectTriangles;
    /**
     * @internal
     */
    private _intersectUnIndexedTriangles;
    /** @internal */
    _rebuild(): void;
    /**
     * Creates a new submesh from the passed mesh
     * @param newMesh defines the new hosting mesh
     * @param newRenderingMesh defines an optional rendering mesh
     * @returns the new submesh
     */
    clone(newMesh: AbstractMesh, newRenderingMesh?: Mesh): SubMesh;
    /**
     * Release associated resources
     */
    dispose(): void;
    /**
     * Gets the class name
     * @returns the string "SubMesh".
     */
    getClassName(): string;
    /**
     * Creates a new submesh from indices data
     * @param materialIndex the index of the main mesh material
     * @param startIndex the index where to start the copy in the mesh indices array
     * @param indexCount the number of indices to copy then from the startIndex
     * @param mesh the main mesh to create the submesh from
     * @param renderingMesh the optional rendering mesh
     * @param createBoundingBox defines if bounding box should be created for this submesh
     * @returns a new submesh
     */
    static CreateFromIndices(materialIndex: number, startIndex: number, indexCount: number, mesh: AbstractMesh, renderingMesh?: Mesh, createBoundingBox?: boolean): SubMesh;
}
export {};

}
declare module "babylonjs/Meshes/subMesh.project" {
import { Vector3 } from "babylonjs/Maths/math.vector";
import { IndicesArray } from "babylonjs/types";
module "babylonjs/Meshes/subMesh" {
    interface SubMesh {
        /** @internal */
        _projectOnTrianglesToRef(vector: Vector3, positions: Vector3[], indices: IndicesArray, step: number, checkStopper: boolean, ref: Vector3): number;
        /** @internal */
        _projectOnUnIndexedTrianglesToRef(vector: Vector3, positions: Vector3[], indices: IndicesArray, ref: Vector3): number;
        /**
         * Projects a point on this submesh and stores the result in "ref"
         *
         * @param vector point to project
         * @param positions defines mesh's positions array
         * @param indices defines mesh's indices array
         * @param ref vector that will store the result
         * @returns distance from the point and the submesh, or -1 if the mesh rendering mode doesn't support projections
         */
        projectToRef(vector: Vector3, positions: Vector3[], indices: IndicesArray, ref: Vector3): number;
    }
}

}
declare module "babylonjs/Meshes/thinInstanceMesh" {
import { Nullable, DeepImmutableObject } from "babylonjs/types";
import { VertexBuffer, Buffer } from "babylonjs/Buffers/buffer";
import { Matrix } from "babylonjs/Maths/math.vector";
module "babylonjs/Meshes/mesh" {
    interface Mesh {
        /**
         * Gets or sets a boolean defining if we want picking to pick thin instances as well
         */
        thinInstanceEnablePicking: boolean;
        /**
         * Creates a new thin instance
         * @param matrix the matrix or array of matrices (position, rotation, scale) of the thin instance(s) to create
         * @param refresh true to refresh the underlying gpu buffer (default: true). If you do multiple calls to this method in a row, set refresh to true only for the last call to save performance
         * @returns the thin instance index number. If you pass an array of matrices, other instance indexes are index+1, index+2, etc
         */
        thinInstanceAdd(matrix: DeepImmutableObject<Matrix> | Array<DeepImmutableObject<Matrix>>, refresh?: boolean): number;
        /**
         * Adds the transformation (matrix) of the current mesh as a thin instance
         * @param refresh true to refresh the underlying gpu buffer (default: true). If you do multiple calls to this method in a row, set refresh to true only for the last call to save performance
         * @returns the thin instance index number
         */
        thinInstanceAddSelf(refresh?: boolean): number;
        /**
         * Registers a custom attribute to be used with thin instances
         * @param kind name of the attribute
         * @param stride size in floats of the attribute
         */
        thinInstanceRegisterAttribute(kind: string, stride: number): void;
        /**
         * Sets the matrix of a thin instance
         * @param index index of the thin instance
         * @param matrix matrix to set
         * @param refresh true to refresh the underlying gpu buffer (default: true). If you do multiple calls to this method in a row, set refresh to true only for the last call to save performance
         */
        thinInstanceSetMatrixAt(index: number, matrix: DeepImmutableObject<Matrix>, refresh?: boolean): void;
        /**
         * Sets the value of a custom attribute for a thin instance
         * @param kind name of the attribute
         * @param index index of the thin instance
         * @param value value to set
         * @param refresh true to refresh the underlying gpu buffer (default: true). If you do multiple calls to this method in a row, set refresh to true only for the last call to save performance
         */
        thinInstanceSetAttributeAt(kind: string, index: number, value: Array<number>, refresh?: boolean): void;
        /**
         * Gets / sets the number of thin instances to display. Note that you can't set a number higher than what the underlying buffer can handle.
         */
        thinInstanceCount: number;
        /**
         * Sets a buffer to be used with thin instances. This method is a faster way to setup multiple instances than calling thinInstanceAdd repeatedly
         * @param kind name of the attribute. Use "matrix" to setup the buffer of matrices
         * @param buffer buffer to set
         * @param stride size in floats of each value of the buffer
         * @param staticBuffer indicates that the buffer is static, so that you won't change it after it is set (better performances - false by default)
         */
        thinInstanceSetBuffer(kind: string, buffer: Nullable<Float32Array>, stride?: number, staticBuffer?: boolean): void;
        /**
         * Gets the list of world matrices
         * @returns an array containing all the world matrices from the thin instances
         */
        thinInstanceGetWorldMatrices(): Matrix[];
        /**
         * Synchronize the gpu buffers with a thin instance buffer. Call this method if you update later on the buffers passed to thinInstanceSetBuffer
         * @param kind name of the attribute to update. Use "matrix" to update the buffer of matrices
         */
        thinInstanceBufferUpdated(kind: string): void;
        /**
         * Applies a partial update to a buffer directly on the GPU
         * Note that the buffer located on the CPU is NOT updated! It's up to you to update it (or not) with the same data you pass to this method
         * @param kind name of the attribute to update. Use "matrix" to update the buffer of matrices
         * @param data the data to set in the GPU buffer
         * @param offset the offset in the GPU buffer where to update the data
         */
        thinInstancePartialBufferUpdate(kind: string, data: Float32Array, offset: number): void;
        /**
         * Refreshes the bounding info, taking into account all the thin instances defined
         * @param forceRefreshParentInfo true to force recomputing the mesh bounding info and use it to compute the aggregated bounding info
         * @param applySkeleton defines whether to apply the skeleton before computing the bounding info
         * @param applyMorph  defines whether to apply the morph target before computing the bounding info
         */
        thinInstanceRefreshBoundingInfo(forceRefreshParentInfo?: boolean, applySkeleton?: boolean, applyMorph?: boolean): void;
        /** @internal */
        _thinInstanceInitializeUserStorage(): void;
        /** @internal */
        _thinInstanceUpdateBufferSize(kind: string, numInstances?: number): void;
        /** @internal */
        _thinInstanceCreateMatrixBuffer(kind: string, buffer: Nullable<Float32Array>, staticBuffer: boolean): Buffer;
        /** @internal */
        _userThinInstanceBuffersStorage: {
            data: {
                [key: string]: Float32Array;
            };
            sizes: {
                [key: string]: number;
            };
            vertexBuffers: {
                [key: string]: Nullable<VertexBuffer>;
            };
            strides: {
                [key: string]: number;
            };
        };
    }
}

}
declare module "babylonjs/Meshes/trailMesh" {
import { Mesh } from "babylonjs/Meshes/mesh";
import { Scene } from "babylonjs/scene";
import { TransformNode } from "babylonjs/Meshes/transformNode";
/**
 * Class used to create a trail following a mesh
 */
export class TrailMesh extends Mesh {
    private _generator;
    private _autoStart;
    private _running;
    private _diameter;
    private _length;
    private _sectionPolygonPointsCount;
    private _sectionVectors;
    private _sectionNormalVectors;
    private _beforeRenderObserver;
    /**
     * @constructor
     * @param name The value used by scene.getMeshByName() to do a lookup.
     * @param generator The mesh or transform node to generate a trail.
     * @param scene The scene to add this mesh to.
     * @param diameter Diameter of trailing mesh. Default is 1.
     * @param length Length of trailing mesh. Default is 60.
     * @param autoStart Automatically start trailing mesh. Default true.
     */
    constructor(name: string, generator: TransformNode, scene?: Scene, diameter?: number, length?: number, autoStart?: boolean);
    /**
     * "TrailMesh"
     * @returns "TrailMesh"
     */
    getClassName(): string;
    private _createMesh;
    /**
     * Start trailing mesh.
     */
    start(): void;
    /**
     * Stop trailing mesh.
     */
    stop(): void;
    /**
     * Update trailing mesh geometry.
     */
    update(): void;
    /**
     * Returns a new TrailMesh object.
     * @param name is a string, the name given to the new mesh
     * @param newGenerator use new generator object for cloned trail mesh
     * @returns a new mesh
     */
    clone(name: string | undefined, newGenerator: TransformNode): TrailMesh;
    /**
     * Serializes this trail mesh
     * @param serializationObject object to write serialization to
     */
    serialize(serializationObject: any): void;
    /**
     * Parses a serialized trail mesh
     * @param parsedMesh the serialized mesh
     * @param scene the scene to create the trail mesh in
     * @returns the created trail mesh
     */
    static Parse(parsedMesh: any, scene: Scene): TrailMesh;
}

}
declare module "babylonjs/Meshes/transformNode" {
import { DeepImmutable, Nullable } from "babylonjs/types";
import { Observable } from "babylonjs/Misc/observable";
import { Camera } from "babylonjs/Cameras/camera";
import { Scene } from "babylonjs/scene";
import { Quaternion, Matrix, Vector3 } from "babylonjs/Maths/math.vector";
import { Node } from "babylonjs/node";
import { Bone } from "babylonjs/Bones/bone";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { Space } from "babylonjs/Maths/math.axis";
/**
 * A TransformNode is an object that is not rendered but can be used as a center of transformation. This can decrease memory usage and increase rendering speed compared to using an empty mesh as a parent and is less complicated than using a pivot matrix.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/transforms/parent_pivot/transform_node
 */
export class TransformNode extends Node {
    /**
     * Object will not rotate to face the camera
     */
    static BILLBOARDMODE_NONE: number;
    /**
     * Object will rotate to face the camera but only on the x axis
     */
    static BILLBOARDMODE_X: number;
    /**
     * Object will rotate to face the camera but only on the y axis
     */
    static BILLBOARDMODE_Y: number;
    /**
     * Object will rotate to face the camera but only on the z axis
     */
    static BILLBOARDMODE_Z: number;
    /**
     * Object will rotate to face the camera
     */
    static BILLBOARDMODE_ALL: number;
    /**
     * Object will rotate to face the camera's position instead of orientation
     */
    static BILLBOARDMODE_USE_POSITION: number;
    /**
     * Child transform with Billboard flags should or should not apply parent rotation (default if off)
     */
    static BillboardUseParentOrientation: boolean;
    private static _TmpRotation;
    private static _TmpScaling;
    private static _TmpTranslation;
    private _forward;
    private _up;
    private _right;
    private _position;
    private _rotation;
    private _rotationQuaternion;
    protected _scaling: Vector3;
    private _transformToBoneReferal;
    private _currentParentWhenAttachingToBone;
    private _isAbsoluteSynced;
    private _billboardMode;
    /**
     * Gets or sets the billboard mode. Default is 0.
     *
     * | Value | Type | Description |
     * | --- | --- | --- |
     * | 0 | BILLBOARDMODE_NONE |  |
     * | 1 | BILLBOARDMODE_X |  |
     * | 2 | BILLBOARDMODE_Y |  |
     * | 4 | BILLBOARDMODE_Z |  |
     * | 7 | BILLBOARDMODE_ALL |  |
     *
     */
    get billboardMode(): number;
    set billboardMode(value: number);
    private _preserveParentRotationForBillboard;
    /**
     * Gets or sets a boolean indicating that parent rotation should be preserved when using billboards.
     * This could be useful for glTF objects where parent rotation helps converting from right handed to left handed
     */
    get preserveParentRotationForBillboard(): boolean;
    set preserveParentRotationForBillboard(value: boolean);
    private _computeUseBillboardPath;
    /**
     * Multiplication factor on scale x/y/z when computing the world matrix. Eg. for a 1x1x1 cube setting this to 2 will make it a 2x2x2 cube
     */
    scalingDeterminant: number;
    private _infiniteDistance;
    /**
     * Gets or sets the distance of the object to max, often used by skybox
     */
    get infiniteDistance(): boolean;
    set infiniteDistance(value: boolean);
    /**
     * Gets or sets a boolean indicating that non uniform scaling (when at least one component is different from others) should be ignored.
     * By default the system will update normals to compensate
     */
    ignoreNonUniformScaling: boolean;
    /**
     * Gets or sets a boolean indicating that even if rotationQuaternion is defined, you can keep updating rotation property and Babylon.js will just mix both
     */
    reIntegrateRotationIntoRotationQuaternion: boolean;
    /** @internal */
    _poseMatrix: Nullable<Matrix>;
    /** @internal */
    _localMatrix: Matrix;
    private _usePivotMatrix;
    private _absolutePosition;
    private _absoluteScaling;
    private _absoluteRotationQuaternion;
    private _pivotMatrix;
    private _pivotMatrixInverse;
    /** @internal */
    _postMultiplyPivotMatrix: boolean;
    protected _isWorldMatrixFrozen: boolean;
    /** @internal */
    _indexInSceneTransformNodesArray: number;
    /**
     * An event triggered after the world matrix is updated
     */
    onAfterWorldMatrixUpdateObservable: Observable<TransformNode>;
    constructor(name: string, scene?: Nullable<Scene>, isPure?: boolean);
    /**
     * Gets a string identifying the name of the class
     * @returns "TransformNode" string
     */
    getClassName(): string;
    /**
     * Gets or set the node position (default is (0.0, 0.0, 0.0))
     */
    get position(): Vector3;
    set position(newPosition: Vector3);
    /**
     * return true if a pivot has been set
     * @returns true if a pivot matrix is used
     */
    isUsingPivotMatrix(): boolean;
    /**
     * Gets or sets the rotation property : a Vector3 defining the rotation value in radians around each local axis X, Y, Z  (default is (0.0, 0.0, 0.0)).
     * If rotation quaternion is set, this Vector3 will be ignored and copy from the quaternion
     */
    get rotation(): Vector3;
    set rotation(newRotation: Vector3);
    /**
     * Gets or sets the scaling property : a Vector3 defining the node scaling along each local axis X, Y, Z (default is (1.0, 1.0, 1.0)).
     */
    get scaling(): Vector3;
    set scaling(newScaling: Vector3);
    /**
     * Gets or sets the rotation Quaternion property : this a Quaternion object defining the node rotation by using a unit quaternion (undefined by default, but can be null).
     * If set, only the rotationQuaternion is then used to compute the node rotation (ie. node.rotation will be ignored)
     */
    get rotationQuaternion(): Nullable<Quaternion>;
    set rotationQuaternion(quaternion: Nullable<Quaternion>);
    /**
     * The forward direction of that transform in world space.
     */
    get forward(): Vector3;
    /**
     * The up direction of that transform in world space.
     */
    get up(): Vector3;
    /**
     * The right direction of that transform in world space.
     */
    get right(): Vector3;
    /**
     * Copies the parameter passed Matrix into the mesh Pose matrix.
     * @param matrix the matrix to copy the pose from
     * @returns this TransformNode.
     */
    updatePoseMatrix(matrix: Matrix): TransformNode;
    /**
     * Returns the mesh Pose matrix.
     * @returns the pose matrix
     */
    getPoseMatrix(): Matrix;
    /** @internal */
    _isSynchronized(): boolean;
    /** @internal */
    _initCache(): void;
    /**
     * Returns the current mesh absolute position.
     * Returns a Vector3.
     */
    get absolutePosition(): Vector3;
    /**
     * Returns the current mesh absolute scaling.
     * Returns a Vector3.
     */
    get absoluteScaling(): Vector3;
    /**
     * Returns the current mesh absolute rotation.
     * Returns a Quaternion.
     */
    get absoluteRotationQuaternion(): Quaternion;
    /**
     * Sets a new matrix to apply before all other transformation
     * @param matrix defines the transform matrix
     * @returns the current TransformNode
     */
    setPreTransformMatrix(matrix: Matrix): TransformNode;
    /**
     * Sets a new pivot matrix to the current node
     * @param matrix defines the new pivot matrix to use
     * @param postMultiplyPivotMatrix defines if the pivot matrix must be cancelled in the world matrix. When this parameter is set to true (default), the inverse of the pivot matrix is also applied at the end to cancel the transformation effect
     * @returns the current TransformNode
     */
    setPivotMatrix(matrix: DeepImmutable<Matrix>, postMultiplyPivotMatrix?: boolean): TransformNode;
    /**
     * Returns the mesh pivot matrix.
     * Default : Identity.
     * @returns the matrix
     */
    getPivotMatrix(): Matrix;
    /**
     * Instantiate (when possible) or clone that node with its hierarchy
     * @param newParent defines the new parent to use for the instance (or clone)
     * @param options defines options to configure how copy is done
     * @param options.doNotInstantiate defines if the model must be instantiated or just cloned
     * @param onNewNodeCreated defines an option callback to call when a clone or an instance is created
     * @returns an instance (or a clone) of the current node with its hierarchy
     */
    instantiateHierarchy(newParent?: Nullable<TransformNode>, options?: {
        doNotInstantiate: boolean | ((node: TransformNode) => boolean);
    }, onNewNodeCreated?: (source: TransformNode, clone: TransformNode) => void): Nullable<TransformNode>;
    /**
     * Prevents the World matrix to be computed any longer
     * @param newWorldMatrix defines an optional matrix to use as world matrix
     * @param decompose defines whether to decompose the given newWorldMatrix or directly assign
     * @returns the TransformNode.
     */
    freezeWorldMatrix(newWorldMatrix?: Nullable<Matrix>, decompose?: boolean): TransformNode;
    /**
     * Allows back the World matrix computation.
     * @returns the TransformNode.
     */
    unfreezeWorldMatrix(): this;
    /**
     * True if the World matrix has been frozen.
     */
    get isWorldMatrixFrozen(): boolean;
    /**
     * Returns the mesh absolute position in the World.
     * @returns a Vector3.
     */
    getAbsolutePosition(): Vector3;
    /**
     * Sets the mesh absolute position in the World from a Vector3 or an Array(3).
     * @param absolutePosition the absolute position to set
     * @returns the TransformNode.
     */
    setAbsolutePosition(absolutePosition: Vector3): TransformNode;
    /**
     * Sets the mesh position in its local space.
     * @param vector3 the position to set in localspace
     * @returns the TransformNode.
     */
    setPositionWithLocalVector(vector3: Vector3): TransformNode;
    /**
     * Returns the mesh position in the local space from the current World matrix values.
     * @returns a new Vector3.
     */
    getPositionExpressedInLocalSpace(): Vector3;
    /**
     * Translates the mesh along the passed Vector3 in its local space.
     * @param vector3 the distance to translate in localspace
     * @returns the TransformNode.
     */
    locallyTranslate(vector3: Vector3): TransformNode;
    private static _LookAtVectorCache;
    /**
     * Orients a mesh towards a target point. Mesh must be drawn facing user.
     * @param targetPoint the position (must be in same space as current mesh) to look at
     * @param yawCor optional yaw (y-axis) correction in radians
     * @param pitchCor optional pitch (x-axis) correction in radians
     * @param rollCor optional roll (z-axis) correction in radians
     * @param space the chosen space of the target
     * @returns the TransformNode.
     */
    lookAt(targetPoint: Vector3, yawCor?: number, pitchCor?: number, rollCor?: number, space?: Space): TransformNode;
    /**
     * Returns a new Vector3 that is the localAxis, expressed in the mesh local space, rotated like the mesh.
     * This Vector3 is expressed in the World space.
     * @param localAxis axis to rotate
     * @returns a new Vector3 that is the localAxis, expressed in the mesh local space, rotated like the mesh.
     */
    getDirection(localAxis: Vector3): Vector3;
    /**
     * Sets the Vector3 "result" as the rotated Vector3 "localAxis" in the same rotation than the mesh.
     * localAxis is expressed in the mesh local space.
     * result is computed in the World space from the mesh World matrix.
     * @param localAxis axis to rotate
     * @param result the resulting transformnode
     * @returns this TransformNode.
     */
    getDirectionToRef(localAxis: Vector3, result: Vector3): TransformNode;
    /**
     * Sets this transform node rotation to the given local axis.
     * @param localAxis the axis in local space
     * @param yawCor optional yaw (y-axis) correction in radians
     * @param pitchCor optional pitch (x-axis) correction in radians
     * @param rollCor optional roll (z-axis) correction in radians
     * @returns this TransformNode
     */
    setDirection(localAxis: Vector3, yawCor?: number, pitchCor?: number, rollCor?: number): TransformNode;
    /**
     * Sets a new pivot point to the current node
     * @param point defines the new pivot point to use
     * @param space defines if the point is in world or local space (local by default)
     * @returns the current TransformNode
     */
    setPivotPoint(point: Vector3, space?: Space): TransformNode;
    /**
     * Returns a new Vector3 set with the mesh pivot point coordinates in the local space.
     * @returns the pivot point
     */
    getPivotPoint(): Vector3;
    /**
     * Sets the passed Vector3 "result" with the coordinates of the mesh pivot point in the local space.
     * @param result the vector3 to store the result
     * @returns this TransformNode.
     */
    getPivotPointToRef(result: Vector3): TransformNode;
    /**
     * Returns a new Vector3 set with the mesh pivot point World coordinates.
     * @returns a new Vector3 set with the mesh pivot point World coordinates.
     */
    getAbsolutePivotPoint(): Vector3;
    /**
     * Sets the Vector3 "result" coordinates with the mesh pivot point World coordinates.
     * @param result vector3 to store the result
     * @returns this TransformNode.
     */
    getAbsolutePivotPointToRef(result: Vector3): TransformNode;
    /**
     * Flag the transform node as dirty (Forcing it to update everything)
     * @param property if set to "rotation" the objects rotationQuaternion will be set to null
     * @returns this  node
     */
    markAsDirty(property?: string): Node;
    /**
     * Defines the passed node as the parent of the current node.
     * The node will remain exactly where it is and its position / rotation will be updated accordingly.
     * Note that if the mesh has a pivot matrix / point defined it will be applied after the parent was updated.
     * In that case the node will not remain in the same space as it is, as the pivot will be applied.
     * To avoid this, you can set updatePivot to true and the pivot will be updated to identity
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/transforms/parent_pivot/parent
     * @param node the node ot set as the parent
     * @param preserveScalingSign if true, keep scaling sign of child. Otherwise, scaling sign might change.
     * @param updatePivot if true, update the pivot matrix to keep the node in the same space as before
     * @returns this TransformNode.
     */
    setParent(node: Nullable<Node>, preserveScalingSign?: boolean, updatePivot?: boolean): TransformNode;
    private _nonUniformScaling;
    /**
     * True if the scaling property of this object is non uniform eg. (1,2,1)
     */
    get nonUniformScaling(): boolean;
    /**
     * @internal
     */
    _updateNonUniformScalingState(value: boolean): boolean;
    /**
     * Attach the current TransformNode to another TransformNode associated with a bone
     * @param bone Bone affecting the TransformNode
     * @param affectedTransformNode TransformNode associated with the bone
     * @returns this object
     */
    attachToBone(bone: Bone, affectedTransformNode: TransformNode): TransformNode;
    /**
     * Detach the transform node if its associated with a bone
     * @param resetToPreviousParent Indicates if the parent that was in effect when attachToBone was called should be set back or if we should set parent to null instead (defaults to the latter)
     * @returns this object
     */
    detachFromBone(resetToPreviousParent?: boolean): TransformNode;
    private static _RotationAxisCache;
    /**
     * Rotates the mesh around the axis vector for the passed angle (amount) expressed in radians, in the given space.
     * space (default LOCAL) can be either Space.LOCAL, either Space.WORLD.
     * Note that the property `rotationQuaternion` is then automatically updated and the property `rotation` is set to (0,0,0) and no longer used.
     * The passed axis is also normalized.
     * @param axis the axis to rotate around
     * @param amount the amount to rotate in radians
     * @param space Space to rotate in (Default: local)
     * @returns the TransformNode.
     */
    rotate(axis: Vector3, amount: number, space?: Space): TransformNode;
    /**
     * Rotates the mesh around the axis vector for the passed angle (amount) expressed in radians, in world space.
     * Note that the property `rotationQuaternion` is then automatically updated and the property `rotation` is set to (0,0,0) and no longer used.
     * The passed axis is also normalized. .
     * Method is based on http://www.euclideanspace.com/maths/geometry/affine/aroundPoint/index.htm
     * @param point the point to rotate around
     * @param axis the axis to rotate around
     * @param amount the amount to rotate in radians
     * @returns the TransformNode
     */
    rotateAround(point: Vector3, axis: Vector3, amount: number): TransformNode;
    /**
     * Translates the mesh along the axis vector for the passed distance in the given space.
     * space (default LOCAL) can be either Space.LOCAL, either Space.WORLD.
     * @param axis the axis to translate in
     * @param distance the distance to translate
     * @param space Space to rotate in (Default: local)
     * @returns the TransformNode.
     */
    translate(axis: Vector3, distance: number, space?: Space): TransformNode;
    /**
     * Adds a rotation step to the mesh current rotation.
     * x, y, z are Euler angles expressed in radians.
     * This methods updates the current mesh rotation, either mesh.rotation, either mesh.rotationQuaternion if it's set.
     * This means this rotation is made in the mesh local space only.
     * It's useful to set a custom rotation order different from the BJS standard one YXZ.
     * Example : this rotates the mesh first around its local X axis, then around its local Z axis, finally around its local Y axis.
     * ```javascript
     * mesh.addRotation(x1, 0, 0).addRotation(0, 0, z2).addRotation(0, 0, y3);
     * ```
     * Note that `addRotation()` accumulates the passed rotation values to the current ones and computes the .rotation or .rotationQuaternion updated values.
     * Under the hood, only quaternions are used. So it's a little faster is you use .rotationQuaternion because it doesn't need to translate them back to Euler angles.
     * @param x Rotation to add
     * @param y Rotation to add
     * @param z Rotation to add
     * @returns the TransformNode.
     */
    addRotation(x: number, y: number, z: number): TransformNode;
    /**
     * @internal
     */
    protected _getEffectiveParent(): Nullable<Node>;
    /**
     * Returns whether the transform node world matrix computation needs the camera information to be computed.
     * This is the case when the node is a billboard or has an infinite distance for instance.
     * @returns true if the world matrix computation needs the camera information to be computed
     */
    isWorldMatrixCameraDependent(): boolean;
    /**
     * Computes the world matrix of the node
     * @param force defines if the cache version should be invalidated forcing the world matrix to be created from scratch
     * @param camera defines the camera used if different from the scene active camera (This is used with modes like Billboard or infinite distance)
     * @returns the world matrix
     */
    computeWorldMatrix(force?: boolean, camera?: Nullable<Camera>): Matrix;
    /**
     * Resets this nodeTransform's local matrix to Matrix.Identity().
     * @param independentOfChildren indicates if all child nodeTransform's world-space transform should be preserved.
     */
    resetLocalMatrix(independentOfChildren?: boolean): void;
    protected _afterComputeWorldMatrix(): void;
    /**
     * If you'd like to be called back after the mesh position, rotation or scaling has been updated.
     * @param func callback function to add
     *
     * @returns the TransformNode.
     */
    registerAfterWorldMatrixUpdate(func: (mesh: TransformNode) => void): TransformNode;
    /**
     * Removes a registered callback function.
     * @param func callback function to remove
     * @returns the TransformNode.
     */
    unregisterAfterWorldMatrixUpdate(func: (mesh: TransformNode) => void): TransformNode;
    /**
     * Gets the position of the current mesh in camera space
     * @param camera defines the camera to use
     * @returns a position
     */
    getPositionInCameraSpace(camera?: Nullable<Camera>): Vector3;
    /**
     * Returns the distance from the mesh to the active camera
     * @param camera defines the camera to use
     * @returns the distance
     */
    getDistanceToCamera(camera?: Nullable<Camera>): number;
    /**
     * Clone the current transform node
     * @param name Name of the new clone
     * @param newParent New parent for the clone
     * @param doNotCloneChildren Do not clone children hierarchy
     * @returns the new transform node
     */
    clone(name: string, newParent: Nullable<Node>, doNotCloneChildren?: boolean): Nullable<TransformNode>;
    /**
     * Serializes the objects information.
     * @param currentSerializationObject defines the object to serialize in
     * @returns the serialized object
     */
    serialize(currentSerializationObject?: any): any;
    /**
     * Returns a new TransformNode object parsed from the source provided.
     * @param parsedTransformNode is the source.
     * @param scene the scene the object belongs to
     * @param rootUrl is a string, it's the root URL to prefix the `delayLoadingFile` property with
     * @returns a new TransformNode object parsed from the source provided.
     */
    static Parse(parsedTransformNode: any, scene: Scene, rootUrl: string): TransformNode;
    /**
     * Get all child-transformNodes of this node
     * @param directDescendantsOnly defines if true only direct descendants of 'this' will be considered, if false direct and also indirect (children of children, an so on in a recursive manner) descendants of 'this' will be considered
     * @param predicate defines an optional predicate that will be called on every evaluated child, the predicate must return true for a given child to be part of the result, otherwise it will be ignored
     * @returns an array of TransformNode
     */
    getChildTransformNodes(directDescendantsOnly?: boolean, predicate?: (node: Node) => boolean): TransformNode[];
    /**
     * Releases resources associated with this transform node.
     * @param doNotRecurse Set to true to not recurse into each children (recurse into each children by default)
     * @param disposeMaterialAndTextures Set to true to also dispose referenced materials and textures (false by default)
     */
    dispose(doNotRecurse?: boolean, disposeMaterialAndTextures?: boolean): void;
    /**
     * Uniformly scales the mesh to fit inside of a unit cube (1 X 1 X 1 units)
     * @param includeDescendants Use the hierarchy's bounding box instead of the mesh's bounding box. Default is false
     * @param ignoreRotation ignore rotation when computing the scale (ie. object will be axis aligned). Default is false
     * @param predicate predicate that is passed in to getHierarchyBoundingVectors when selecting which object should be included when scaling
     * @returns the current mesh
     */
    normalizeToUnitCube(includeDescendants?: boolean, ignoreRotation?: boolean, predicate?: Nullable<(node: AbstractMesh) => boolean>): TransformNode;
    private _syncAbsoluteScalingAndRotation;
}

}
declare module "babylonjs/Meshes/WebGL/webGLDataBuffer" {
import { DataBuffer } from "babylonjs/Buffers/dataBuffer";
/** @internal */
export class WebGLDataBuffer extends DataBuffer {
    private _buffer;
    constructor(resource: WebGLBuffer);
    get underlyingResource(): any;
}

}
declare module "babylonjs/Meshes/WebGPU/webgpuDataBuffer" {
import { DataBuffer } from "babylonjs/Buffers/dataBuffer";
/** @internal */
export class WebGPUDataBuffer extends DataBuffer {
    private _buffer;
    constructor(resource: GPUBuffer);
    get underlyingResource(): any;
}

}
declare module "babylonjs/Misc/andOrNotEvaluator" {
/**
 * Class used to evaluate queries containing `and` and `or` operators
 */
export class AndOrNotEvaluator {
    /**
     * Evaluate a query
     * @param query defines the query to evaluate
     * @param evaluateCallback defines the callback used to filter result
     * @returns true if the query matches
     */
    static Eval(query: string, evaluateCallback: (val: any) => boolean): boolean;
    private static _HandleParenthesisContent;
    private static _SimplifyNegation;
}

}
declare module "babylonjs/Misc/arrayTools" {
/** @internal */
interface TupleTypes<T> {
    2: [T, T];
    3: [T, T, T];
    4: [T, T, T, T];
    5: [T, T, T, T, T];
    6: [T, T, T, T, T, T];
    7: [T, T, T, T, T, T, T];
    8: [T, T, T, T, T, T, T, T];
    9: [T, T, T, T, T, T, T, T, T];
    10: [T, T, T, T, T, T, T, T, T, T];
    11: [T, T, T, T, T, T, T, T, T, T, T];
    12: [T, T, T, T, T, T, T, T, T, T, T, T];
    13: [T, T, T, T, T, T, T, T, T, T, T, T, T];
    14: [T, T, T, T, T, T, T, T, T, T, T, T, T, T];
    15: [T, T, T, T, T, T, T, T, T, T, T, T, T, T, T];
}
/**
 * Class containing a set of static utilities functions for arrays.
 */
export class ArrayTools {
    /**
     * Returns an array of the given size filled with elements built from the given constructor and the parameters.
     * @param size the number of element to construct and put in the array.
     * @param itemBuilder a callback responsible for creating new instance of item. Called once per array entry.
     * @returns a new array filled with new objects.
     */
    static BuildArray<T>(size: number, itemBuilder: () => T): Array<T>;
    /**
     * Returns a tuple of the given size filled with elements built from the given constructor and the parameters.
     * @param size he number of element to construct and put in the tuple.
     * @param itemBuilder a callback responsible for creating new instance of item. Called once per tuple entry.
     * @returns a new tuple filled with new objects.
     */
    static BuildTuple<T, N extends keyof TupleTypes<unknown>>(size: N, itemBuilder: () => T): TupleTypes<T>[N];
}
/**
 * Defines the callback type used when an observed array function is triggered.
 * @internal
 */
export type _ObserveCallback = (functionName: string, previousLength: number) => void;
/**
 * Observes an array and notifies the given observer when the array is modified.
 * @param array Defines the array to observe
 * @param callback Defines the function to call when the array is modified (in the limit of the observed array functions)
 * @returns A function to call to stop observing the array
 * @internal
 */
export function _ObserveArray<T>(array: T[], callback: _ObserveCallback): () => void;
export {};

}
declare module "babylonjs/Misc/assetsManager" {
import { Scene } from "babylonjs/scene";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { TransformNode } from "babylonjs/Meshes/transformNode";
import { IParticleSystem } from "babylonjs/Particles/IParticleSystem";
import { Skeleton } from "babylonjs/Bones/skeleton";
import { Observable } from "babylonjs/Misc/observable";
import { BaseTexture } from "babylonjs/Materials/Textures/baseTexture";
import { Texture } from "babylonjs/Materials/Textures/texture";
import { CubeTexture } from "babylonjs/Materials/Textures/cubeTexture";
import { HDRCubeTexture } from "babylonjs/Materials/Textures/hdrCubeTexture";
import { EquiRectangularCubeTexture } from "babylonjs/Materials/Textures/equiRectangularCubeTexture";
import { AnimationGroup } from "babylonjs/Animations/animationGroup";
import { AssetContainer } from "babylonjs/assetContainer";
/**
 * Defines the list of states available for a task inside a AssetsManager
 */
export enum AssetTaskState {
    /**
     * Initialization
     */
    INIT = 0,
    /**
     * Running
     */
    RUNNING = 1,
    /**
     * Done
     */
    DONE = 2,
    /**
     * Error
     */
    ERROR = 3
}
/**
 * Define an abstract asset task used with a AssetsManager class to load assets into a scene
 */
export abstract class AbstractAssetTask {
    /**
     * Task name
     */ name: string;
    /**
     * Callback called when the task is successful
     */
    onSuccess: (task: any) => void;
    /**
     * Callback called when the task is not successful
     */
    onError: (task: any, message?: string, exception?: any) => void;
    /**
     * Creates a new AssetsManager
     * @param name defines the name of the task
     */
    constructor(
    /**
     * Task name
     */ name: string);
    private _isCompleted;
    private _taskState;
    private _errorObject;
    /**
     * Get if the task is completed
     */
    get isCompleted(): boolean;
    /**
     * Gets the current state of the task
     */
    get taskState(): AssetTaskState;
    /**
     * Gets the current error object (if task is in error)
     */
    get errorObject(): {
        message?: string;
        exception?: any;
    };
    /**
     * Internal only
     * @internal
     */
    _setErrorObject(message?: string, exception?: any): void;
    /**
     * Execute the current task
     * @param scene defines the scene where you want your assets to be loaded
     * @param onSuccess is a callback called when the task is successfully executed
     * @param onError is a callback called if an error occurs
     */
    run(scene: Scene, onSuccess: () => void, onError: (message?: string, exception?: any) => void): void;
    /**
     * Execute the current task
     * @param scene defines the scene where you want your assets to be loaded
     * @param onSuccess is a callback called when the task is successfully executed
     * @param onError is a callback called if an error occurs
     */
    runTask(scene: Scene, onSuccess: () => void, onError: (message?: string, exception?: any) => void): void;
    /**
     * Reset will set the task state back to INIT, so the next load call of the assets manager will execute this task again.
     * This can be used with failed tasks that have the reason for failure fixed.
     */
    reset(): void;
    private _onErrorCallback;
    private _onDoneCallback;
}
/**
 * Define the interface used by progress events raised during assets loading
 */
export interface IAssetsProgressEvent {
    /**
     * Defines the number of remaining tasks to process
     */
    remainingCount: number;
    /**
     * Defines the total number of tasks
     */
    totalCount: number;
    /**
     * Defines the task that was just processed
     */
    task: AbstractAssetTask;
}
/**
 * Class used to share progress information about assets loading
 */
export class AssetsProgressEvent implements IAssetsProgressEvent {
    /**
     * Defines the number of remaining tasks to process
     */
    remainingCount: number;
    /**
     * Defines the total number of tasks
     */
    totalCount: number;
    /**
     * Defines the task that was just processed
     */
    task: AbstractAssetTask;
    /**
     * Creates a AssetsProgressEvent
     * @param remainingCount defines the number of remaining tasks to process
     * @param totalCount defines the total number of tasks
     * @param task defines the task that was just processed
     */
    constructor(remainingCount: number, totalCount: number, task: AbstractAssetTask);
}
/**
 * Define a task used by AssetsManager to load assets into a container
 */
export class ContainerAssetTask extends AbstractAssetTask {
    /**
     * Defines the name of the task
     */
    name: string;
    /**
     * Defines the list of mesh's names you want to load
     */
    meshesNames: any;
    /**
     * Defines the root url to use as a base to load your meshes and associated resources
     */
    rootUrl: string;
    /**
     * Defines the filename or File of the scene to load from
     */
    sceneFilename: string | File;
    /**
     * Defines the extension to use to load the scene (if not defined, ".babylon" will be used)
     */
    extension?: string | undefined;
    /**
     * Get the loaded asset container
     */
    loadedContainer: AssetContainer;
    /**
     * Gets the list of loaded transforms
     */
    loadedTransformNodes: Array<TransformNode>;
    /**
     * Gets the list of loaded meshes
     */
    loadedMeshes: Array<AbstractMesh>;
    /**
     * Gets the list of loaded particle systems
     */
    loadedParticleSystems: Array<IParticleSystem>;
    /**
     * Gets the list of loaded skeletons
     */
    loadedSkeletons: Array<Skeleton>;
    /**
     * Gets the list of loaded animation groups
     */
    loadedAnimationGroups: Array<AnimationGroup>;
    /**
     * Callback called when the task is successful
     */
    onSuccess: (task: ContainerAssetTask) => void;
    /**
     * Callback called when the task is successful
     */
    onError: (task: ContainerAssetTask, message?: string, exception?: any) => void;
    /**
     * Creates a new ContainerAssetTask
     * @param name defines the name of the task
     * @param meshesNames defines the list of mesh's names you want to load
     * @param rootUrl defines the root url to use as a base to load your meshes and associated resources
     * @param sceneFilename defines the filename or File of the scene to load from
     */
    constructor(
    /**
     * Defines the name of the task
     */
    name: string, 
    /**
     * Defines the list of mesh's names you want to load
     */
    meshesNames: any, 
    /**
     * Defines the root url to use as a base to load your meshes and associated resources
     */
    rootUrl: string, 
    /**
     * Defines the filename or File of the scene to load from
     */
    sceneFilename: string | File, 
    /**
     * Defines the extension to use to load the scene (if not defined, ".babylon" will be used)
     */
    extension?: string | undefined);
    /**
     * Execute the current task
     * @param scene defines the scene where you want your assets to be loaded
     * @param onSuccess is a callback called when the task is successfully executed
     * @param onError is a callback called if an error occurs
     */
    runTask(scene: Scene, onSuccess: () => void, onError: (message?: string, exception?: any) => void): void;
}
/**
 * Define a task used by AssetsManager to load meshes
 */
export class MeshAssetTask extends AbstractAssetTask {
    /**
     * Defines the name of the task
     */
    name: string;
    /**
     * Defines the list of mesh's names you want to load
     */
    meshesNames: any;
    /**
     * Defines the root url to use as a base to load your meshes and associated resources
     */
    rootUrl: string;
    /**
     * Defines the filename or File of the scene to load from
     */
    sceneFilename: string | File;
    /**
     * Defines the extension to use to load the scene (if not defined, ".babylon" will be used)
     */
    extension?: string | undefined;
    /**
     * Gets the list of loaded transforms
     */
    loadedTransformNodes: Array<TransformNode>;
    /**
     * Gets the list of loaded meshes
     */
    loadedMeshes: Array<AbstractMesh>;
    /**
     * Gets the list of loaded particle systems
     */
    loadedParticleSystems: Array<IParticleSystem>;
    /**
     * Gets the list of loaded skeletons
     */
    loadedSkeletons: Array<Skeleton>;
    /**
     * Gets the list of loaded animation groups
     */
    loadedAnimationGroups: Array<AnimationGroup>;
    /**
     * Callback called when the task is successful
     */
    onSuccess: (task: MeshAssetTask) => void;
    /**
     * Callback called when the task is successful
     */
    onError: (task: MeshAssetTask, message?: string, exception?: any) => void;
    /**
     * Creates a new MeshAssetTask
     * @param name defines the name of the task
     * @param meshesNames defines the list of mesh's names you want to load
     * @param rootUrl defines the root url to use as a base to load your meshes and associated resources
     * @param sceneFilename defines the filename or File of the scene to load from
     */
    constructor(
    /**
     * Defines the name of the task
     */
    name: string, 
    /**
     * Defines the list of mesh's names you want to load
     */
    meshesNames: any, 
    /**
     * Defines the root url to use as a base to load your meshes and associated resources
     */
    rootUrl: string, 
    /**
     * Defines the filename or File of the scene to load from
     */
    sceneFilename: string | File, 
    /**
     * Defines the extension to use to load the scene (if not defined, ".babylon" will be used)
     */
    extension?: string | undefined);
    /**
     * Execute the current task
     * @param scene defines the scene where you want your assets to be loaded
     * @param onSuccess is a callback called when the task is successfully executed
     * @param onError is a callback called if an error occurs
     */
    runTask(scene: Scene, onSuccess: () => void, onError: (message?: string, exception?: any) => void): void;
}
/**
 * Define a task used by AssetsManager to load text content
 */
export class TextFileAssetTask extends AbstractAssetTask {
    /**
     * Defines the name of the task
     */
    name: string;
    /**
     * Defines the location of the file to load
     */
    url: string;
    /**
     * Gets the loaded text string
     */
    text: string;
    /**
     * Callback called when the task is successful
     */
    onSuccess: (task: TextFileAssetTask) => void;
    /**
     * Callback called when the task is successful
     */
    onError: (task: TextFileAssetTask, message?: string, exception?: any) => void;
    /**
     * Creates a new TextFileAssetTask object
     * @param name defines the name of the task
     * @param url defines the location of the file to load
     */
    constructor(
    /**
     * Defines the name of the task
     */
    name: string, 
    /**
     * Defines the location of the file to load
     */
    url: string);
    /**
     * Execute the current task
     * @param scene defines the scene where you want your assets to be loaded
     * @param onSuccess is a callback called when the task is successfully executed
     * @param onError is a callback called if an error occurs
     */
    runTask(scene: Scene, onSuccess: () => void, onError: (message?: string, exception?: any) => void): void;
}
/**
 * Define a task used by AssetsManager to load binary data
 */
export class BinaryFileAssetTask extends AbstractAssetTask {
    /**
     * Defines the name of the task
     */
    name: string;
    /**
     * Defines the location of the file to load
     */
    url: string;
    /**
     * Gets the loaded data (as an array buffer)
     */
    data: ArrayBuffer;
    /**
     * Callback called when the task is successful
     */
    onSuccess: (task: BinaryFileAssetTask) => void;
    /**
     * Callback called when the task is successful
     */
    onError: (task: BinaryFileAssetTask, message?: string, exception?: any) => void;
    /**
     * Creates a new BinaryFileAssetTask object
     * @param name defines the name of the new task
     * @param url defines the location of the file to load
     */
    constructor(
    /**
     * Defines the name of the task
     */
    name: string, 
    /**
     * Defines the location of the file to load
     */
    url: string);
    /**
     * Execute the current task
     * @param scene defines the scene where you want your assets to be loaded
     * @param onSuccess is a callback called when the task is successfully executed
     * @param onError is a callback called if an error occurs
     */
    runTask(scene: Scene, onSuccess: () => void, onError: (message?: string, exception?: any) => void): void;
}
/**
 * Define a task used by AssetsManager to load images
 */
export class ImageAssetTask extends AbstractAssetTask {
    /**
     * Defines the name of the task
     */
    name: string;
    /**
     * Defines the location of the image to load
     */
    url: string;
    /**
     * Gets the loaded images
     */
    image: HTMLImageElement;
    /**
     * Callback called when the task is successful
     */
    onSuccess: (task: ImageAssetTask) => void;
    /**
     * Callback called when the task is successful
     */
    onError: (task: ImageAssetTask, message?: string, exception?: any) => void;
    /**
     * Creates a new ImageAssetTask
     * @param name defines the name of the task
     * @param url defines the location of the image to load
     */
    constructor(
    /**
     * Defines the name of the task
     */
    name: string, 
    /**
     * Defines the location of the image to load
     */
    url: string);
    /**
     * Execute the current task
     * @param scene defines the scene where you want your assets to be loaded
     * @param onSuccess is a callback called when the task is successfully executed
     * @param onError is a callback called if an error occurs
     */
    runTask(scene: Scene, onSuccess: () => void, onError: (message?: string, exception?: any) => void): void;
}
/**
 * Defines the interface used by texture loading tasks
 */
export interface ITextureAssetTask<TEX extends BaseTexture> {
    /**
     * Gets the loaded texture
     */
    texture: TEX;
}
/**
 * Define a task used by AssetsManager to load 2D textures
 */
export class TextureAssetTask extends AbstractAssetTask implements ITextureAssetTask<Texture> {
    /**
     * Defines the name of the task
     */
    name: string;
    /**
     * Defines the location of the file to load
     */
    url: string;
    /**
     * Defines if mipmap should not be generated (default is false)
     */
    noMipmap?: boolean | undefined;
    /**
     * Defines if texture must be inverted on Y axis (default is true)
     */
    invertY: boolean;
    /**
     * Defines the sampling mode to use (default is Texture.TRILINEAR_SAMPLINGMODE)
     */
    samplingMode: number;
    /**
     * Gets the loaded texture
     */
    texture: Texture;
    /**
     * Callback called when the task is successful
     */
    onSuccess: (task: TextureAssetTask) => void;
    /**
     * Callback called when the task is successful
     */
    onError: (task: TextureAssetTask, message?: string, exception?: any) => void;
    /**
     * Creates a new TextureAssetTask object
     * @param name defines the name of the task
     * @param url defines the location of the file to load
     * @param noMipmap defines if mipmap should not be generated (default is false)
     * @param invertY defines if texture must be inverted on Y axis (default is true)
     * @param samplingMode defines the sampling mode to use (default is Texture.TRILINEAR_SAMPLINGMODE)
     */
    constructor(
    /**
     * Defines the name of the task
     */
    name: string, 
    /**
     * Defines the location of the file to load
     */
    url: string, 
    /**
     * Defines if mipmap should not be generated (default is false)
     */
    noMipmap?: boolean | undefined, 
    /**
     * Defines if texture must be inverted on Y axis (default is true)
     */
    invertY?: boolean, 
    /**
     * Defines the sampling mode to use (default is Texture.TRILINEAR_SAMPLINGMODE)
     */
    samplingMode?: number);
    /**
     * Execute the current task
     * @param scene defines the scene where you want your assets to be loaded
     * @param onSuccess is a callback called when the task is successfully executed
     * @param onError is a callback called if an error occurs
     */
    runTask(scene: Scene, onSuccess: () => void, onError: (message?: string, exception?: any) => void): void;
}
/**
 * Define a task used by AssetsManager to load cube textures
 */
export class CubeTextureAssetTask extends AbstractAssetTask implements ITextureAssetTask<CubeTexture> {
    /**
     * Defines the name of the task
     */
    name: string;
    /**
     * Defines the location of the files to load (You have to specify the folder where the files are + filename with no extension)
     */
    url: string;
    /**
     * Defines the extensions to use to load files (["_px", "_py", "_pz", "_nx", "_ny", "_nz"] by default)
     */
    extensions?: string[] | undefined;
    /**
     * Defines if mipmaps should not be generated (default is false)
     */
    noMipmap?: boolean | undefined;
    /**
     * Defines the explicit list of files (undefined by default)
     */
    files?: string[] | undefined;
    /**
     * Defines the prefiltered texture option (default is false)
     */
    prefiltered?: boolean | undefined;
    /**
     * Gets the loaded texture
     */
    texture: CubeTexture;
    /**
     * Callback called when the task is successful
     */
    onSuccess: (task: CubeTextureAssetTask) => void;
    /**
     * Callback called when the task is successful
     */
    onError: (task: CubeTextureAssetTask, message?: string, exception?: any) => void;
    /**
     * Creates a new CubeTextureAssetTask
     * @param name defines the name of the task
     * @param url defines the location of the files to load (You have to specify the folder where the files are + filename with no extension)
     * @param extensions defines the extensions to use to load files (["_px", "_py", "_pz", "_nx", "_ny", "_nz"] by default)
     * @param noMipmap defines if mipmaps should not be generated (default is false)
     * @param files defines the explicit list of files (undefined by default)
     * @param prefiltered
     */
    constructor(
    /**
     * Defines the name of the task
     */
    name: string, 
    /**
     * Defines the location of the files to load (You have to specify the folder where the files are + filename with no extension)
     */
    url: string, 
    /**
     * Defines the extensions to use to load files (["_px", "_py", "_pz", "_nx", "_ny", "_nz"] by default)
     */
    extensions?: string[] | undefined, 
    /**
     * Defines if mipmaps should not be generated (default is false)
     */
    noMipmap?: boolean | undefined, 
    /**
     * Defines the explicit list of files (undefined by default)
     */
    files?: string[] | undefined, 
    /**
     * Defines the prefiltered texture option (default is false)
     */
    prefiltered?: boolean | undefined);
    /**
     * Execute the current task
     * @param scene defines the scene where you want your assets to be loaded
     * @param onSuccess is a callback called when the task is successfully executed
     * @param onError is a callback called if an error occurs
     */
    runTask(scene: Scene, onSuccess: () => void, onError: (message?: string, exception?: any) => void): void;
}
/**
 * Define a task used by AssetsManager to load HDR cube textures
 */
export class HDRCubeTextureAssetTask extends AbstractAssetTask implements ITextureAssetTask<HDRCubeTexture> {
    /**
     * Defines the name of the task
     */
    name: string;
    /**
     * Defines the location of the file to load
     */
    url: string;
    /**
     * Defines the desired size (the more it increases the longer the generation will be)
     */
    size: number;
    /**
     * Defines if mipmaps should not be generated (default is false)
     */
    noMipmap: boolean;
    /**
     * Specifies whether you want to extract the polynomial harmonics during the generation process (default is true)
     */
    generateHarmonics: boolean;
    /**
     * Specifies if the texture will be use in gamma or linear space (the PBR material requires those texture in linear space, but the standard material would require them in Gamma space) (default is false)
     */
    gammaSpace: boolean;
    /**
     * Internal Use Only
     */
    reserved: boolean;
    /**
     * Gets the loaded texture
     */
    texture: HDRCubeTexture;
    /**
     * Callback called when the task is successful
     */
    onSuccess: (task: HDRCubeTextureAssetTask) => void;
    /**
     * Callback called when the task is successful
     */
    onError: (task: HDRCubeTextureAssetTask, message?: string, exception?: any) => void;
    /**
     * Creates a new HDRCubeTextureAssetTask object
     * @param name defines the name of the task
     * @param url defines the location of the file to load
     * @param size defines the desired size (the more it increases the longer the generation will be) If the size is omitted this implies you are using a preprocessed cubemap.
     * @param noMipmap defines if mipmaps should not be generated (default is false)
     * @param generateHarmonics specifies whether you want to extract the polynomial harmonics during the generation process (default is true)
     * @param gammaSpace specifies if the texture will be use in gamma or linear space (the PBR material requires those texture in linear space, but the standard material would require them in Gamma space) (default is false)
     * @param reserved Internal use only
     */
    constructor(
    /**
     * Defines the name of the task
     */
    name: string, 
    /**
     * Defines the location of the file to load
     */
    url: string, 
    /**
     * Defines the desired size (the more it increases the longer the generation will be)
     */
    size: number, 
    /**
     * Defines if mipmaps should not be generated (default is false)
     */
    noMipmap?: boolean, 
    /**
     * Specifies whether you want to extract the polynomial harmonics during the generation process (default is true)
     */
    generateHarmonics?: boolean, 
    /**
     * Specifies if the texture will be use in gamma or linear space (the PBR material requires those texture in linear space, but the standard material would require them in Gamma space) (default is false)
     */
    gammaSpace?: boolean, 
    /**
     * Internal Use Only
     */
    reserved?: boolean);
    /**
     * Execute the current task
     * @param scene defines the scene where you want your assets to be loaded
     * @param onSuccess is a callback called when the task is successfully executed
     * @param onError is a callback called if an error occurs
     */
    runTask(scene: Scene, onSuccess: () => void, onError: (message?: string, exception?: any) => void): void;
}
/**
 * Define a task used by AssetsManager to load Equirectangular cube textures
 */
export class EquiRectangularCubeTextureAssetTask extends AbstractAssetTask implements ITextureAssetTask<EquiRectangularCubeTexture> {
    /**
     * Defines the name of the task
     */
    name: string;
    /**
     * Defines the location of the file to load
     */
    url: string;
    /**
     * Defines the desired size (the more it increases the longer the generation will be)
     */
    size: number;
    /**
     * Defines if mipmaps should not be generated (default is false)
     */
    noMipmap: boolean;
    /**
     * Specifies if the texture will be use in gamma or linear space (the PBR material requires those texture in linear space,
     * but the standard material would require them in Gamma space) (default is true)
     */
    gammaSpace: boolean;
    /**
     * Gets the loaded texture
     */
    texture: EquiRectangularCubeTexture;
    /**
     * Callback called when the task is successful
     */
    onSuccess: (task: EquiRectangularCubeTextureAssetTask) => void;
    /**
     * Callback called when the task is successful
     */
    onError: (task: EquiRectangularCubeTextureAssetTask, message?: string, exception?: any) => void;
    /**
     * Creates a new EquiRectangularCubeTextureAssetTask object
     * @param name defines the name of the task
     * @param url defines the location of the file to load
     * @param size defines the desired size (the more it increases the longer the generation will be)
     * If the size is omitted this implies you are using a preprocessed cubemap.
     * @param noMipmap defines if mipmaps should not be generated (default is false)
     * @param gammaSpace specifies if the texture will be used in gamma or linear space
     * (the PBR material requires those texture in linear space, but the standard material would require them in Gamma space)
     * (default is true)
     */
    constructor(
    /**
     * Defines the name of the task
     */
    name: string, 
    /**
     * Defines the location of the file to load
     */
    url: string, 
    /**
     * Defines the desired size (the more it increases the longer the generation will be)
     */
    size: number, 
    /**
     * Defines if mipmaps should not be generated (default is false)
     */
    noMipmap?: boolean, 
    /**
     * Specifies if the texture will be use in gamma or linear space (the PBR material requires those texture in linear space,
     * but the standard material would require them in Gamma space) (default is true)
     */
    gammaSpace?: boolean);
    /**
     * Execute the current task
     * @param scene defines the scene where you want your assets to be loaded
     * @param onSuccess is a callback called when the task is successfully executed
     * @param onError is a callback called if an error occurs
     */
    runTask(scene: Scene, onSuccess: () => void, onError: (message?: string, exception?: any) => void): void;
}
/**
 * This class can be used to easily import assets into a scene
 * @see https://doc.babylonjs.com/features/featuresDeepDive/importers/assetManager
 */
export class AssetsManager {
    private _scene;
    private _isLoading;
    protected _tasks: AbstractAssetTask[];
    protected _waitingTasksCount: number;
    protected _totalTasksCount: number;
    /**
     * Callback called when all tasks are processed
     */
    onFinish: (tasks: AbstractAssetTask[]) => void;
    /**
     * Callback called when a task is successful
     */
    onTaskSuccess: (task: AbstractAssetTask) => void;
    /**
     * Callback called when a task had an error
     */
    onTaskError: (task: AbstractAssetTask) => void;
    /**
     * Callback called when a task is done (whatever the result is)
     */
    onProgress: (remainingCount: number, totalCount: number, task: AbstractAssetTask) => void;
    /**
     * Observable called when all tasks are processed
     */
    onTaskSuccessObservable: Observable<AbstractAssetTask>;
    /**
     * Observable called when a task had an error
     */
    onTaskErrorObservable: Observable<AbstractAssetTask>;
    /**
     * Observable called when all tasks were executed
     */
    onTasksDoneObservable: Observable<AbstractAssetTask[]>;
    /**
     * Observable called when a task is done (whatever the result is)
     */
    onProgressObservable: Observable<IAssetsProgressEvent>;
    /**
     * Gets or sets a boolean defining if the AssetsManager should use the default loading screen
     * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/customLoadingScreen
     */
    useDefaultLoadingScreen: boolean;
    /**
     * Gets or sets a boolean defining if the AssetsManager should automatically hide the loading screen
     * when all assets have been downloaded.
     * If set to false, you need to manually call in hideLoadingUI() once your scene is ready.
     */
    autoHideLoadingUI: boolean;
    /**
     * Creates a new AssetsManager
     * @param scene defines the scene to work on
     */
    constructor(scene?: Scene);
    /**
     * Add a ContainerAssetTask to the list of active tasks
     * @param taskName defines the name of the new task
     * @param meshesNames defines the name of meshes to load
     * @param rootUrl defines the root url to use to locate files
     * @param sceneFilename defines the filename of the scene file or the File itself
     * @param extension defines the extension to use to load the file
     * @returns a new ContainerAssetTask object
     */
    addContainerTask(taskName: string, meshesNames: any, rootUrl: string, sceneFilename: string | File, extension?: string): ContainerAssetTask;
    /**
     * Add a MeshAssetTask to the list of active tasks
     * @param taskName defines the name of the new task
     * @param meshesNames defines the name of meshes to load
     * @param rootUrl defines the root url to use to locate files
     * @param sceneFilename defines the filename of the scene file or the File itself
     * @param extension defines the extension to use to load the file
     * @returns a new MeshAssetTask object
     */
    addMeshTask(taskName: string, meshesNames: any, rootUrl: string, sceneFilename: string | File, extension?: string): MeshAssetTask;
    /**
     * Add a TextFileAssetTask to the list of active tasks
     * @param taskName defines the name of the new task
     * @param url defines the url of the file to load
     * @returns a new TextFileAssetTask object
     */
    addTextFileTask(taskName: string, url: string): TextFileAssetTask;
    /**
     * Add a BinaryFileAssetTask to the list of active tasks
     * @param taskName defines the name of the new task
     * @param url defines the url of the file to load
     * @returns a new BinaryFileAssetTask object
     */
    addBinaryFileTask(taskName: string, url: string): BinaryFileAssetTask;
    /**
     * Add a ImageAssetTask to the list of active tasks
     * @param taskName defines the name of the new task
     * @param url defines the url of the file to load
     * @returns a new ImageAssetTask object
     */
    addImageTask(taskName: string, url: string): ImageAssetTask;
    /**
     * Add a TextureAssetTask to the list of active tasks
     * @param taskName defines the name of the new task
     * @param url defines the url of the file to load
     * @param noMipmap defines if the texture must not receive mipmaps (false by default)
     * @param invertY defines if you want to invert Y axis of the loaded texture (true by default)
     * @param samplingMode defines the sampling mode to use (Texture.TRILINEAR_SAMPLINGMODE by default)
     * @returns a new TextureAssetTask object
     */
    addTextureTask(taskName: string, url: string, noMipmap?: boolean, invertY?: boolean, samplingMode?: number): TextureAssetTask;
    /**
     * Add a CubeTextureAssetTask to the list of active tasks
     * @param taskName defines the name of the new task
     * @param url defines the url of the file to load
     * @param extensions defines the extension to use to load the cube map (can be null)
     * @param noMipmap defines if the texture must not receive mipmaps (false by default)
     * @param files defines the list of files to load (can be null)
     * @param prefiltered defines the prefiltered texture option (default is false)
     * @returns a new CubeTextureAssetTask object
     */
    addCubeTextureTask(taskName: string, url: string, extensions?: string[], noMipmap?: boolean, files?: string[], prefiltered?: boolean): CubeTextureAssetTask;
    /**
     *
     * Add a HDRCubeTextureAssetTask to the list of active tasks
     * @param taskName defines the name of the new task
     * @param url defines the url of the file to load
     * @param size defines the size you want for the cubemap (can be null)
     * @param noMipmap defines if the texture must not receive mipmaps (false by default)
     * @param generateHarmonics defines if you want to automatically generate (true by default)
     * @param gammaSpace specifies if the texture will be use in gamma or linear space (the PBR material requires those texture in linear space, but the standard material would require them in Gamma space) (default is false)
     * @param reserved Internal use only
     * @returns a new HDRCubeTextureAssetTask object
     */
    addHDRCubeTextureTask(taskName: string, url: string, size: number, noMipmap?: boolean, generateHarmonics?: boolean, gammaSpace?: boolean, reserved?: boolean): HDRCubeTextureAssetTask;
    /**
     *
     * Add a EquiRectangularCubeTextureAssetTask to the list of active tasks
     * @param taskName defines the name of the new task
     * @param url defines the url of the file to load
     * @param size defines the size you want for the cubemap (can be null)
     * @param noMipmap defines if the texture must not receive mipmaps (false by default)
     * @param gammaSpace Specifies if the texture will be used in gamma or linear space
     * (the PBR material requires those textures in linear space, but the standard material would require them in Gamma space)
     * @returns a new EquiRectangularCubeTextureAssetTask object
     */
    addEquiRectangularCubeTextureAssetTask(taskName: string, url: string, size: number, noMipmap?: boolean, gammaSpace?: boolean): EquiRectangularCubeTextureAssetTask;
    /**
     * Remove a task from the assets manager.
     * @param task the task to remove
     */
    removeTask(task: AbstractAssetTask): void;
    private _decreaseWaitingTasksCount;
    private _runTask;
    private _formatTaskErrorMessage;
    /**
     * Reset the AssetsManager and remove all tasks
     * @returns the current instance of the AssetsManager
     */
    reset(): AssetsManager;
    /**
     * Start the loading process
     * @returns the current instance of the AssetsManager
     */
    load(): AssetsManager;
    /**
     * Start the loading process as an async operation
     * @returns a promise returning the list of failed tasks
     */
    loadAsync(): Promise<void>;
}

}
declare module "babylonjs/Misc/basis" {
import { InternalTexture } from "babylonjs/Materials/Textures/internalTexture";
import { Engine } from "babylonjs/Engines/engine";
/**
 * Info about the .basis files
 */
class BasisFileInfo {
    /**
     * If the file has alpha
     */
    hasAlpha: boolean;
    /**
     * Info about each image of the basis file
     */
    images: Array<{
        levels: Array<{
            width: number;
            height: number;
            transcodedPixels: ArrayBufferView;
        }>;
    }>;
}
/**
 * Result of transcoding a basis file
 */
class TranscodeResult {
    /**
     * Info about the .basis file
     */
    fileInfo: BasisFileInfo;
    /**
     * Format to use when loading the file
     */
    format: number;
}
/**
 * Configuration options for the Basis transcoder
 */
export class BasisTranscodeConfiguration {
    /**
     * Supported compression formats used to determine the supported output format of the transcoder
     */
    supportedCompressionFormats?: {
        /**
         * etc1 compression format
         */
        etc1?: boolean;
        /**
         * s3tc compression format
         */
        s3tc?: boolean;
        /**
         * pvrtc compression format
         */
        pvrtc?: boolean;
        /**
         * etc2 compression format
         */
        etc2?: boolean;
        /**
         * astc compression format
         */
        astc?: boolean;
        /**
         * bc7 compression format
         */
        bc7?: boolean;
    };
    /**
     * If mipmap levels should be loaded for transcoded images (Default: true)
     */
    loadMipmapLevels?: boolean;
    /**
     * Index of a single image to load (Default: all images)
     */
    loadSingleImage?: number;
}
/**
 * Used to load .Basis files
 * See https://github.com/BinomialLLC/basis_universal/tree/master/webgl
 */
export const BasisToolsOptions: {
    /**
     * URL to use when loading the basis transcoder
     */
    JSModuleURL: string;
    /**
     * URL to use when loading the wasm module for the transcoder
     */
    WasmModuleURL: string;
};
/**
 * Get the internal format to be passed to texImage2D corresponding to the .basis format value
 * @param basisFormat format chosen from GetSupportedTranscodeFormat
 * @param engine
 * @returns internal format corresponding to the Basis format
 */
export const GetInternalFormatFromBasisFormat: (basisFormat: number, engine: Engine) => number;
/**
 * Transcodes a loaded image file to compressed pixel data
 * @param data image data to transcode
 * @param config configuration options for the transcoding
 * @returns a promise resulting in the transcoded image
 */
export const TranscodeAsync: (data: ArrayBuffer | ArrayBufferView, config: BasisTranscodeConfiguration) => Promise<TranscodeResult>;
/**
 * Loads a texture from the transcode result
 * @param texture texture load to
 * @param transcodeResult the result of transcoding the basis file to load from
 */
export const LoadTextureFromTranscodeResult: (texture: InternalTexture, transcodeResult: TranscodeResult) => void;
/**
 * Used to load .Basis files
 * See https://github.com/BinomialLLC/basis_universal/tree/master/webgl
 */
export const BasisTools: {
    /**
     * URL to use when loading the basis transcoder
     */
    JSModuleURL: string;
    /**
     * URL to use when loading the wasm module for the transcoder
     */
    WasmModuleURL: string;
    /**
     * Get the internal format to be passed to texImage2D corresponding to the .basis format value
     * @param basisFormat format chosen from GetSupportedTranscodeFormat
     * @returns internal format corresponding to the Basis format
     */
    GetInternalFormatFromBasisFormat: (basisFormat: number, engine: Engine) => number;
    /**
     * Transcodes a loaded image file to compressed pixel data
     * @param data image data to transcode
     * @param config configuration options for the transcoding
     * @returns a promise resulting in the transcoded image
     */
    TranscodeAsync: (data: ArrayBuffer | ArrayBufferView, config: BasisTranscodeConfiguration) => Promise<TranscodeResult>;
    /**
     * Loads a texture from the transcode result
     * @param texture texture load to
     * @param transcodeResult the result of transcoding the basis file to load from
     */
    LoadTextureFromTranscodeResult: (texture: InternalTexture, transcodeResult: TranscodeResult) => void;
};
export {};

}
declare module "babylonjs/Misc/brdfTextureTools" {
import { BaseTexture } from "babylonjs/Materials/Textures/baseTexture";
import { Scene } from "babylonjs/scene";
/**
 * Gets a default environment BRDF for MS-BRDF Height Correlated BRDF
 * @param scene defines the hosting scene
 * @returns the environment BRDF texture
 */
export const GetEnvironmentBRDFTexture: (scene: Scene) => BaseTexture;
/**
 * Class used to host texture specific utilities
 */
export const BRDFTextureTools: {
    /**
     * Gets a default environment BRDF for MS-BRDF Height Correlated BRDF
     * @param scene defines the hosting scene
     * @returns the environment BRDF texture
     */
    GetEnvironmentBRDFTexture: (scene: Scene) => BaseTexture;
};

}
declare module "babylonjs/Misc/codeStringParsingTools" {
/**
 * Extracts the characters between two markers (for eg, between "(" and ")"). The function handles nested markers as well as markers inside strings (delimited by ", ' or `) and comments
 * @param markerOpen opening marker
 * @param markerClose closing marker
 * @param block code block to parse
 * @param startIndex starting index in block where the extraction must start. The character at block[startIndex] should be the markerOpen character!
 * @returns index of the last character for the extraction (or -1 if the string is invalid - no matching closing marker found). The string to extract (without the markers) is the string between startIndex + 1 and the returned value (exclusive)
 */
export function ExtractBetweenMarkers(markerOpen: string, markerClose: string, block: string, startIndex: number): number;
/**
 * Parses a string and skip whitespaces
 * @param s string to parse
 * @param index index where to start parsing
 * @returns the index after all whitespaces have been skipped
 */
export function SkipWhitespaces(s: string, index: number): number;
/**
 * Checks if a character is an identifier character (meaning, if it is 0-9, A-Z, a-z or _)
 * @param c character to check
 * @returns true if the character is an identifier character
 */
export function IsIdentifierChar(c: string): boolean;
/**
 * Removes the comments of a code block
 * @param block code block to parse
 * @returns block with the comments removed
 */
export function RemoveComments(block: string): string;
/**
 * Finds the first occurrence of a character in a string going backward
 * @param s the string to parse
 * @param index starting index in the string
 * @param c the character to find
 * @returns the index of the character if found, else -1
 */
export function FindBackward(s: string, index: number, c: string): number;
/**
 * Escapes a string so that it is usable as a regular expression
 * @param s string to escape
 * @returns escaped string
 */
export function EscapeRegExp(s: string): string;

}
declare module "babylonjs/Misc/copyTextureToTexture" {
import { ThinEngine } from "babylonjs/Engines/thinEngine";
import { InternalTexture } from "babylonjs/Materials/Textures/internalTexture";
import { IRenderTargetTexture, RenderTargetWrapper } from "babylonjs/Engines/renderTargetWrapper";
import { ThinTexture } from "babylonjs/Materials/Textures/thinTexture";
import "babylonjs/Shaders/copyTextureToTexture.fragment";
/**
 * Conversion modes available when copying a texture into another one
 */
export enum ConversionMode {
    None = 0,
    ToLinearSpace = 1,
    ToGammaSpace = 2
}
/**
 * Class used for fast copy from one texture to another
 */
export class CopyTextureToTexture {
    private _engine;
    private _isDepthTexture;
    private _renderer;
    private _effectWrapper;
    private _source;
    private _conversion;
    private _textureIsInternal;
    /**
     * Constructs a new instance of the class
     * @param engine The engine to use for the copy
     * @param isDepthTexture True means that we should write (using gl_FragDepth) into the depth texture attached to the destination (default: false)
     */
    constructor(engine: ThinEngine, isDepthTexture?: boolean);
    /**
     * Indicates if the effect is ready to be used for the copy
     * @returns true if "copy" can be called without delay, else false
     */
    isReady(): boolean;
    /**
     * Copy one texture into another
     * @param source The source texture
     * @param destination The destination texture
     * @param conversion The conversion mode that should be applied when copying
     * @returns
     */
    copy(source: InternalTexture | ThinTexture, destination: RenderTargetWrapper | IRenderTargetTexture, conversion?: ConversionMode): boolean;
    /**
     * Releases all the resources used by the class
     */
    dispose(): void;
}

}
declare module "babylonjs/Misc/copyTools" {
import { ISize } from "babylonjs/Maths/math.size";
import { Nullable } from "babylonjs/types";
import { BaseTexture } from "babylonjs/Materials/Textures/baseTexture";
/**
 * Transform some pixel data to a base64 string
 * @param pixels defines the pixel data to transform to base64
 * @param size defines the width and height of the (texture) data
 * @param invertY true if the data must be inverted for the Y coordinate during the conversion
 * @returns The base64 encoded string or null
 */
export function GenerateBase64StringFromPixelData(pixels: ArrayBufferView, size: ISize, invertY?: boolean): Nullable<string>;
/**
 * Reads the pixels stored in the webgl texture and returns them as a base64 string
 * @param texture defines the texture to read pixels from
 * @param faceIndex defines the face of the texture to read (in case of cube texture)
 * @param level defines the LOD level of the texture to read (in case of Mip Maps)
 * @returns The base64 encoded string or null
 */
export function GenerateBase64StringFromTexture(texture: BaseTexture, faceIndex?: number, level?: number): Nullable<string>;
/**
 * Reads the pixels stored in the webgl texture and returns them as a base64 string
 * @param texture defines the texture to read pixels from
 * @param faceIndex defines the face of the texture to read (in case of cube texture)
 * @param level defines the LOD level of the texture to read (in case of Mip Maps)
 * @returns The base64 encoded string or null wrapped in a promise
 */
export function GenerateBase64StringFromTextureAsync(texture: BaseTexture, faceIndex?: number, level?: number): Promise<Nullable<string>>;
/**
 * Class used to host copy specific utilities
 * (Back-compat)
 */
export const CopyTools: {
    /**
     * Transform some pixel data to a base64 string
     * @param pixels defines the pixel data to transform to base64
     * @param size defines the width and height of the (texture) data
     * @param invertY true if the data must be inverted for the Y coordinate during the conversion
     * @returns The base64 encoded string or null
     */
    GenerateBase64StringFromPixelData: typeof GenerateBase64StringFromPixelData;
    /**
     * Reads the pixels stored in the webgl texture and returns them as a base64 string
     * @param texture defines the texture to read pixels from
     * @param faceIndex defines the face of the texture to read (in case of cube texture)
     * @param level defines the LOD level of the texture to read (in case of Mip Maps)
     * @returns The base64 encoded string or null
     */
    GenerateBase64StringFromTexture: typeof GenerateBase64StringFromTexture;
    /**
     * Reads the pixels stored in the webgl texture and returns them as a base64 string
     * @param texture defines the texture to read pixels from
     * @param faceIndex defines the face of the texture to read (in case of cube texture)
     * @param level defines the LOD level of the texture to read (in case of Mip Maps)
     * @returns The base64 encoded string or null wrapped in a promise
     */
    GenerateBase64StringFromTextureAsync: typeof GenerateBase64StringFromTextureAsync;
};
export {};

}
declare module "babylonjs/Misc/coroutine" {
/**
 * A Coroutine<T> is the intersection of:
 * 1. An Iterator that yields void, returns a T, and is not passed values with calls to next.
 * 2. An IterableIterator of void (since it only yields void).
 */
type CoroutineBase<TStep, TReturn> = Iterator<TStep, TReturn, void> & IterableIterator<TStep>;
/** @internal */
export type Coroutine<T> = CoroutineBase<void, T>;
/** @internal */
export type AsyncCoroutine<T> = CoroutineBase<void | Promise<void>, T>;
/** @internal */
export type CoroutineStep<T> = IteratorResult<void, T>;
/** @internal */
export type CoroutineScheduler<T> = (coroutine: AsyncCoroutine<T>, onStep: (stepResult: CoroutineStep<T>) => void, onError: (stepError: any) => void) => void;
/**
 * @internal
 */
export function inlineScheduler<T>(coroutine: AsyncCoroutine<T>, onStep: (stepResult: CoroutineStep<T>) => void, onError: (stepError: any) => void): void;
/**
 * @internal
 */
export function createYieldingScheduler<T>(yieldAfterMS?: number): (coroutine: AsyncCoroutine<T>, onStep: (stepResult: CoroutineStep<T>) => void, onError: (stepError: any) => void) => void;
/**
 * @internal
 */
export function runCoroutine<T>(coroutine: AsyncCoroutine<T>, scheduler: CoroutineScheduler<T>, onSuccess: (result: T) => void, onError: (error: any) => void, abortSignal?: AbortSignal): void;
/**
 * @internal
 */
export function runCoroutineSync<T>(coroutine: Coroutine<T>, abortSignal?: AbortSignal): T;
/**
 * @internal
 */
export function runCoroutineAsync<T>(coroutine: AsyncCoroutine<T>, scheduler: CoroutineScheduler<T>, abortSignal?: AbortSignal): Promise<T>;
/**
 * Given a function that returns a Coroutine<T>, produce a function with the same parameters that returns a T.
 * The returned function runs the coroutine synchronously.
 * @param coroutineFactory A function that returns a Coroutine<T>.
 * @param abortSignal
 * @returns A function that runs the coroutine synchronously.
 * @internal
 */
export function makeSyncFunction<TParams extends unknown[], TReturn>(coroutineFactory: (...params: TParams) => Coroutine<TReturn>, abortSignal?: AbortSignal): (...params: TParams) => TReturn;
/**
 * Given a function that returns a Coroutine<T>, product a function with the same parameters that returns a Promise<T>.
 * The returned function runs the coroutine asynchronously, yield control of the execution context occasionally to enable a more responsive experience.
 * @param coroutineFactory A function that returns a Coroutine<T>.
 * @param scheduler
 * @param abortSignal
 * @returns A function that runs the coroutine asynchronously.
 * @internal
 */
export function makeAsyncFunction<TParams extends unknown[], TReturn>(coroutineFactory: (...params: TParams) => AsyncCoroutine<TReturn>, scheduler: CoroutineScheduler<TReturn>, abortSignal?: AbortSignal): (...params: TParams) => Promise<TReturn>;
export {};

}
declare module "babylonjs/Misc/customAnimationFrameRequester" {
/**
 * Interface for any object that can request an animation frame
 */
export interface ICustomAnimationFrameRequester {
    /**
     * This function will be called when the render loop is ready. If this is not populated, the engine's renderloop function will be called
     */
    renderFunction?: Function;
    /**
     * Called to request the next frame to render to
     * @see https://developer.mozilla.org/en-US/docs/Web/API/window/requestAnimationFrame
     */
    requestAnimationFrame: Function;
    /**
     * You can pass this value to cancelAnimationFrame() to cancel the refresh callback request
     * @see https://developer.mozilla.org/en-US/docs/Web/API/window/requestAnimationFrame#Return_value
     */
    requestID?: number;
}

}
declare module "babylonjs/Misc/dataReader" {
/**
 * Interface for a data buffer
 */
export interface IDataBuffer {
    /**
     * Reads bytes from the data buffer.
     * @param byteOffset The byte offset to read
     * @param byteLength The byte length to read
     * @returns A promise that resolves when the bytes are read
     */
    readAsync(byteOffset: number, byteLength: number): Promise<ArrayBufferView>;
    /**
     * The byte length of the buffer.
     */
    readonly byteLength: number;
}
/**
 * Utility class for reading from a data buffer
 */
export class DataReader {
    /**
     * The data buffer associated with this data reader.
     */
    readonly buffer: IDataBuffer;
    /**
     * The current byte offset from the beginning of the data buffer.
     */
    byteOffset: number;
    private _dataView;
    private _dataByteOffset;
    /**
     * Constructor
     * @param buffer The buffer to read
     */
    constructor(buffer: IDataBuffer);
    /**
     * Loads the given byte length.
     * @param byteLength The byte length to load
     * @returns A promise that resolves when the load is complete
     */
    loadAsync(byteLength: number): Promise<void>;
    /**
     * Read a unsigned 32-bit integer from the currently loaded data range.
     * @returns The 32-bit integer read
     */
    readUint32(): number;
    /**
     * Read a byte array from the currently loaded data range.
     * @param byteLength The byte length to read
     * @returns The byte array read
     */
    readUint8Array(byteLength: number): Uint8Array;
    /**
     * Read a string from the currently loaded data range.
     * @param byteLength The byte length to read
     * @returns The string read
     */
    readString(byteLength: number): string;
    /**
     * Skips the given byte length the currently loaded data range.
     * @param byteLength The byte length to skip
     */
    skipBytes(byteLength: number): void;
}

}
declare module "babylonjs/Misc/dataStorage" {
/**
 * Class for storing data to local storage if available or in-memory storage otherwise
 */
export class DataStorage {
    private static _Storage;
    private static _GetStorage;
    /**
     * Reads a string from the data storage
     * @param key The key to read
     * @param defaultValue The value if the key doesn't exist
     * @returns The string value
     */
    static ReadString(key: string, defaultValue: string): string;
    /**
     * Writes a string to the data storage
     * @param key The key to write
     * @param value The value to write
     */
    static WriteString(key: string, value: string): void;
    /**
     * Reads a boolean from the data storage
     * @param key The key to read
     * @param defaultValue The value if the key doesn't exist
     * @returns The boolean value
     */
    static ReadBoolean(key: string, defaultValue: boolean): boolean;
    /**
     * Writes a boolean to the data storage
     * @param key The key to write
     * @param value The value to write
     */
    static WriteBoolean(key: string, value: boolean): void;
    /**
     * Reads a number from the data storage
     * @param key The key to read
     * @param defaultValue The value if the key doesn't exist
     * @returns The number value
     */
    static ReadNumber(key: string, defaultValue: number): number;
    /**
     * Writes a number to the data storage
     * @param key The key to write
     * @param value The value to write
     */
    static WriteNumber(key: string, value: number): void;
}

}
declare module "babylonjs/Misc/dds" {
import { SphericalPolynomial } from "babylonjs/Maths/sphericalPolynomial";
import { InternalTexture } from "babylonjs/Materials/Textures/internalTexture";
import { Nullable } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
import { ThinEngine } from "babylonjs/Engines/thinEngine";
import "babylonjs/Engines/Extensions/engine.cubeTexture";
/**
 * Direct draw surface info
 * @see https://docs.microsoft.com/en-us/windows/desktop/direct3ddds/dx-graphics-dds-pguide
 */
export interface DDSInfo {
    /**
     * Width of the texture
     */
    width: number;
    /**
     * Width of the texture
     */
    height: number;
    /**
     * Number of Mipmaps for the texture
     * @see https://en.wikipedia.org/wiki/Mipmap
     */
    mipmapCount: number;
    /**
     * If the textures format is a known fourCC format
     * @see https://www.fourcc.org/
     */
    isFourCC: boolean;
    /**
     * If the texture is an RGB format eg. DXGI_FORMAT_B8G8R8X8_UNORM format
     */
    isRGB: boolean;
    /**
     * If the texture is a lumincance format
     */
    isLuminance: boolean;
    /**
     * If this is a cube texture
     * @see https://docs.microsoft.com/en-us/windows/desktop/direct3ddds/dds-file-layout-for-cubic-environment-maps
     */
    isCube: boolean;
    /**
     * If the texture is a compressed format eg. FOURCC_DXT1
     */
    isCompressed: boolean;
    /**
     * The dxgiFormat of the texture
     * @see https://docs.microsoft.com/en-us/windows/desktop/api/dxgiformat/ne-dxgiformat-dxgi_format
     */
    dxgiFormat: number;
    /**
     * Texture type eg. Engine.TEXTURETYPE_UNSIGNED_INT, Engine.TEXTURETYPE_FLOAT
     */
    textureType: number;
    /**
     * Sphericle polynomial created for the dds texture
     */
    sphericalPolynomial?: SphericalPolynomial;
}
/**
 * Class used to provide DDS decompression tools
 */
export class DDSTools {
    /**
     * Gets or sets a boolean indicating that LOD info is stored in alpha channel (false by default)
     */
    static StoreLODInAlphaChannel: boolean;
    /**
     * Gets DDS information from an array buffer
     * @param data defines the array buffer view to read data from
     * @returns the DDS information
     */
    static GetDDSInfo(data: ArrayBufferView): DDSInfo;
    private static _GetHalfFloatAsFloatRGBAArrayBuffer;
    private static _GetHalfFloatRGBAArrayBuffer;
    private static _GetFloatRGBAArrayBuffer;
    private static _GetFloatAsHalfFloatRGBAArrayBuffer;
    private static _GetFloatAsUIntRGBAArrayBuffer;
    private static _GetHalfFloatAsUIntRGBAArrayBuffer;
    private static _GetRGBAArrayBuffer;
    private static _ExtractLongWordOrder;
    private static _GetRGBArrayBuffer;
    private static _GetLuminanceArrayBuffer;
    /**
     * Uploads DDS Levels to a Babylon Texture
     * @internal
     */
    static UploadDDSLevels(engine: ThinEngine, texture: InternalTexture, data: ArrayBufferView, info: DDSInfo, loadMipmaps: boolean, faces: number, lodIndex?: number, currentFace?: number, destTypeMustBeFilterable?: boolean): void;
}
module "babylonjs/Engines/thinEngine" {
    interface ThinEngine {
        /**
         * Create a cube texture from prefiltered data (ie. the mipmaps contain ready to use data for PBR reflection)
         * @param rootUrl defines the url where the file to load is located
         * @param scene defines the current scene
         * @param lodScale defines scale to apply to the mip map selection
         * @param lodOffset defines offset to apply to the mip map selection
         * @param onLoad defines an optional callback raised when the texture is loaded
         * @param onError defines an optional callback raised if there is an issue to load the texture
         * @param format defines the format of the data
         * @param forcedExtension defines the extension to use to pick the right loader
         * @param createPolynomials defines wheter or not to create polynomails harmonics for the texture
         * @returns the cube texture as an InternalTexture
         */
        createPrefilteredCubeTexture(rootUrl: string, scene: Nullable<Scene>, lodScale: number, lodOffset: number, onLoad?: Nullable<(internalTexture: Nullable<InternalTexture>) => void>, onError?: Nullable<(message?: string, exception?: any) => void>, format?: number, forcedExtension?: any, createPolynomials?: boolean): InternalTexture;
    }
}

}
declare module "babylonjs/Misc/decorators" {
import { Nullable } from "babylonjs/types";
import { IAnimatable } from "babylonjs/Animations/animatable.interface";
import { Scene } from "babylonjs/scene";
import { ImageProcessingConfiguration } from "babylonjs/Materials/imageProcessingConfiguration";
import { FresnelParameters } from "babylonjs/Materials/fresnelParameters";
import { ColorCurves } from "babylonjs/Materials/colorCurves";
import { BaseTexture } from "babylonjs/Materials/Textures/baseTexture";
export function expandToProperty(callback: string, targetKey?: Nullable<string>): (target: any, propertyKey: string) => void;
export function serialize(sourceName?: string): (target: any, propertyKey: string | symbol) => void;
export function serializeAsTexture(sourceName?: string): (target: any, propertyKey: string | symbol) => void;
export function serializeAsColor3(sourceName?: string): (target: any, propertyKey: string | symbol) => void;
export function serializeAsFresnelParameters(sourceName?: string): (target: any, propertyKey: string | symbol) => void;
export function serializeAsVector2(sourceName?: string): (target: any, propertyKey: string | symbol) => void;
export function serializeAsVector3(sourceName?: string): (target: any, propertyKey: string | symbol) => void;
export function serializeAsMeshReference(sourceName?: string): (target: any, propertyKey: string | symbol) => void;
export function serializeAsColorCurves(sourceName?: string): (target: any, propertyKey: string | symbol) => void;
export function serializeAsColor4(sourceName?: string): (target: any, propertyKey: string | symbol) => void;
export function serializeAsImageProcessingConfiguration(sourceName?: string): (target: any, propertyKey: string | symbol) => void;
export function serializeAsQuaternion(sourceName?: string): (target: any, propertyKey: string | symbol) => void;
export function serializeAsMatrix(sourceName?: string): (target: any, propertyKey: string | symbol) => void;
/**
 * Decorator used to define property that can be serialized as reference to a camera
 * @param sourceName defines the name of the property to decorate
 */
export function serializeAsCameraReference(sourceName?: string): (target: any, propertyKey: string | symbol) => void;
/**
 * Class used to help serialization objects
 */
export class SerializationHelper {
    /**
     * Gets or sets a boolean to indicate if the UniqueId property should be serialized
     */
    static AllowLoadingUniqueId: boolean;
    /**
     * @internal
     */
    static _ImageProcessingConfigurationParser: (sourceProperty: any) => ImageProcessingConfiguration;
    /**
     * @internal
     */
    static _FresnelParametersParser: (sourceProperty: any) => FresnelParameters;
    /**
     * @internal
     */
    static _ColorCurvesParser: (sourceProperty: any) => ColorCurves;
    /**
     * @internal
     */
    static _TextureParser: (sourceProperty: any, scene: Scene, rootUrl: string) => Nullable<BaseTexture>;
    /**
     * Appends the serialized animations from the source animations
     * @param source Source containing the animations
     * @param destination Target to store the animations
     */
    static AppendSerializedAnimations(source: IAnimatable, destination: any): void;
    /**
     * Static function used to serialized a specific entity
     * @param entity defines the entity to serialize
     * @param serializationObject defines the optional target object where serialization data will be stored
     * @returns a JSON compatible object representing the serialization of the entity
     */
    static Serialize<T>(entity: T, serializationObject?: any): any;
    /**
     * Creates a new entity from a serialization data object
     * @param creationFunction defines a function used to instanciated the new entity
     * @param source defines the source serialization data
     * @param scene defines the hosting scene
     * @param rootUrl defines the root url for resources
     * @returns a new entity
     */
    static Parse<T>(creationFunction: () => T, source: any, scene: Nullable<Scene>, rootUrl?: Nullable<string>): T;
    /**
     * Clones an object
     * @param creationFunction defines the function used to instanciate the new object
     * @param source defines the source object
     * @returns the cloned object
     */
    static Clone<T>(creationFunction: () => T, source: T): T;
    /**
     * Instanciates a new object based on a source one (some data will be shared between both object)
     * @param creationFunction defines the function used to instanciate the new object
     * @param source defines the source object
     * @returns the new object
     */
    static Instanciate<T>(creationFunction: () => T, source: T): T;
}
/**
 * Decorator used to redirect a function to a native implementation if available.
 * @internal
 */
export function nativeOverride<T extends (...params: any[]) => boolean>(target: any, propertyKey: string, descriptor: TypedPropertyDescriptor<(...params: Parameters<T>) => unknown>, predicate?: T): void;
export namespace nativeOverride {
    var filter: <T extends (...params: any) => boolean>(predicate: T) => (target: any, propertyKey: string, descriptor: TypedPropertyDescriptor<(...params: Parameters<T>) => unknown>) => void;
}
export {};

}
declare module "babylonjs/Misc/deepCopier" {
/**
 * Class containing a set of static utilities functions for deep copy.
 */
export class DeepCopier {
    /**
     * Tries to copy an object by duplicating every property
     * @param source defines the source object
     * @param destination defines the target object
     * @param doNotCopyList defines a list of properties to avoid
     * @param mustCopyList defines a list of properties to copy (even if they start with _)
     */
    static DeepCopy(source: any, destination: any, doNotCopyList?: string[], mustCopyList?: string[]): void;
}

}
declare module "babylonjs/Misc/deferred" {
/**
 * Wrapper class for promise with external resolve and reject.
 */
export class Deferred<T> {
    /**
     * The promise associated with this deferred object.
     */
    readonly promise: Promise<T>;
    private _resolve;
    private _reject;
    /**
     * The resolve method of the promise associated with this deferred object.
     */
    get resolve(): (value: T | PromiseLike<T>) => void;
    /**
     * The reject method of the promise associated with this deferred object.
     */
    get reject(): (reason?: any) => void;
    /**
     * Constructor for this deferred object.
     */
    constructor();
}

}
declare module "babylonjs/Misc/depthReducer" {
import { Nullable } from "babylonjs/types";
import { RenderTargetTexture } from "babylonjs/Materials/Textures/renderTargetTexture";
import { Camera } from "babylonjs/Cameras/camera";
import { DepthRenderer } from "babylonjs/Rendering/depthRenderer";
import { MinMaxReducer } from "babylonjs/Misc/minMaxReducer";
/**
 * This class is a small wrapper around the MinMaxReducer class to compute the min/max values of a depth texture
 */
export class DepthReducer extends MinMaxReducer {
    private _depthRenderer;
    private _depthRendererId;
    /**
     * Gets the depth renderer used for the computation.
     * Note that the result is null if you provide your own renderer when calling setDepthRenderer.
     */
    get depthRenderer(): Nullable<DepthRenderer>;
    /**
     * Creates a depth reducer
     * @param camera The camera used to render the depth texture
     */
    constructor(camera: Camera);
    /**
     * Sets the depth renderer to use to generate the depth map
     * @param depthRenderer The depth renderer to use. If not provided, a new one will be created automatically
     * @param type The texture type of the depth map (default: TEXTURETYPE_HALF_FLOAT)
     * @param forceFullscreenViewport Forces the post processes used for the reduction to be applied without taking into account viewport (defaults to true)
     */
    setDepthRenderer(depthRenderer?: Nullable<DepthRenderer>, type?: number, forceFullscreenViewport?: boolean): void;
    /**
     * @internal
     */
    setSourceTexture(sourceTexture: RenderTargetTexture, depthRedux: boolean, type?: number, forceFullscreenViewport?: boolean): void;
    /**
     * Activates the reduction computation.
     * When activated, the observers registered in onAfterReductionPerformed are
     * called after the computation is performed
     */
    activate(): void;
    /**
     * Deactivates the reduction computation.
     */
    deactivate(): void;
    /**
     * Disposes the depth reducer
     * @param disposeAll true to dispose all the resources. You should always call this function with true as the parameter (or without any parameter as it is the default one). This flag is meant to be used internally.
     */
    dispose(disposeAll?: boolean): void;
}

}
declare module "babylonjs/Misc/devTools" {
/**
 * @internal
 */
export function _WarnImport(name: string): string;

}
declare module "babylonjs/Misc/domManagement" {
/**
 * Checks if the window object exists
 * @returns true if the window object exists
 */
export function IsWindowObjectExist(): boolean;
/**
 * Checks if the navigator object exists
 * @returns true if the navigator object exists
 */
export function IsNavigatorAvailable(): boolean;
/**
 * Check if the document object exists
 * @returns true if the document object exists
 */
export function IsDocumentAvailable(): boolean;
/**
 * Extracts text content from a DOM element hierarchy
 * @param element defines the root element
 * @returns a string
 */
export function GetDOMTextContent(element: HTMLElement): string;
/**
 * Sets of helpers dealing with the DOM and some of the recurrent functions needed in
 * Babylon.js
 */
export const DomManagement: {
    /**
     * Checks if the window object exists
     * @returns true if the window object exists
     */
    IsWindowObjectExist: typeof IsWindowObjectExist;
    /**
     * Checks if the navigator object exists
     * @returns true if the navigator object exists
     */
    IsNavigatorAvailable: typeof IsNavigatorAvailable;
    /**
     * Check if the document object exists
     * @returns true if the document object exists
     */
    IsDocumentAvailable: typeof IsDocumentAvailable;
    /**
     * Extracts text content from a DOM element hierarchy
     * @param element defines the root element
     * @returns a string
     */
    GetDOMTextContent: typeof GetDOMTextContent;
};

}
declare module "babylonjs/Misc/dumpTools" {
import { Engine } from "babylonjs/Engines/engine";
/**
 * Class containing a set of static utilities functions to dump data from a canvas
 */
export class DumpTools {
    private static _DumpToolsEngine;
    private static _CreateDumpRenderer;
    /**
     * Dumps the current bound framebuffer
     * @param width defines the rendering width
     * @param height defines the rendering height
     * @param engine defines the hosting engine
     * @param successCallback defines the callback triggered once the data are available
     * @param mimeType defines the mime type of the result
     * @param fileName defines the filename to download. If present, the result will automatically be downloaded
     * @returns a void promise
     */
    static DumpFramebuffer(width: number, height: number, engine: Engine, successCallback?: (data: string) => void, mimeType?: string, fileName?: string): Promise<void>;
    /**
     * Dumps an array buffer
     * @param width defines the rendering width
     * @param height defines the rendering height
     * @param data the data array
     * @param mimeType defines the mime type of the result
     * @param fileName defines the filename to download. If present, the result will automatically be downloaded
     * @param invertY true to invert the picture in the Y dimension
     * @param toArrayBuffer true to convert the data to an ArrayBuffer (encoded as `mimeType`) instead of a base64 string
     * @param quality defines the quality of the result
     * @returns a promise that resolve to the final data
     */
    static DumpDataAsync(width: number, height: number, data: ArrayBufferView, mimeType?: string, fileName?: string, invertY?: boolean, toArrayBuffer?: boolean, quality?: number): Promise<string | ArrayBuffer>;
    /**
     * Dumps an array buffer
     * @param width defines the rendering width
     * @param height defines the rendering height
     * @param data the data array
     * @param successCallback defines the callback triggered once the data are available
     * @param mimeType defines the mime type of the result
     * @param fileName defines the filename to download. If present, the result will automatically be downloaded
     * @param invertY true to invert the picture in the Y dimension
     * @param toArrayBuffer true to convert the data to an ArrayBuffer (encoded as `mimeType`) instead of a base64 string
     * @param quality defines the quality of the result
     */
    static DumpData(width: number, height: number, data: ArrayBufferView, successCallback?: (data: string | ArrayBuffer) => void, mimeType?: string, fileName?: string, invertY?: boolean, toArrayBuffer?: boolean, quality?: number): void;
    /**
     * Dispose the dump tools associated resources
     */
    static Dispose(): void;
}

}
declare module "babylonjs/Misc/environmentTextureTools" {
import { Nullable } from "babylonjs/types";
import { SphericalPolynomial } from "babylonjs/Maths/sphericalPolynomial";
import { InternalTexture } from "babylonjs/Materials/Textures/internalTexture";
import { BaseTexture } from "babylonjs/Materials/Textures/baseTexture";
import "babylonjs/Engines/Extensions/engine.renderTargetCube";
import "babylonjs/Engines/Extensions/engine.readTexture";
import "babylonjs/Materials/Textures/baseTexture.polynomial";
import "babylonjs/Shaders/rgbdEncode.fragment";
import "babylonjs/Shaders/rgbdDecode.fragment";
/**
 * Raw texture data and descriptor sufficient for WebGL texture upload
 */
export type EnvironmentTextureInfo = EnvironmentTextureInfoV1 | EnvironmentTextureInfoV2;
/**
 * v1 of EnvironmentTextureInfo
 */
interface EnvironmentTextureInfoV1 {
    /**
     * Version of the environment map
     */
    version: 1;
    /**
     * Width of image
     */
    width: number;
    /**
     * Irradiance information stored in the file.
     */
    irradiance: any;
    /**
     * Specular information stored in the file.
     */
    specular: any;
}
/**
 * v2 of EnvironmentTextureInfo
 */
interface EnvironmentTextureInfoV2 {
    /**
     * Version of the environment map
     */
    version: 2;
    /**
     * Width of image
     */
    width: number;
    /**
     * Irradiance information stored in the file.
     */
    irradiance: any;
    /**
     * Specular information stored in the file.
     */
    specular: any;
    /**
     * The mime type used to encode the image data.
     */
    imageType: string;
}
/**
 * Defines One Image in the file. It requires only the position in the file
 * as well as the length.
 */
interface BufferImageData {
    /**
     * Length of the image data.
     */
    length: number;
    /**
     * Position of the data from the null terminator delimiting the end of the JSON.
     */
    position: number;
}
/**
 * Defines the specular data enclosed in the file.
 * This corresponds to the version 1 of the data.
 */
export interface EnvironmentTextureSpecularInfoV1 {
    /**
     * Defines where the specular Payload is located. It is a runtime value only not stored in the file.
     */
    specularDataPosition?: number;
    /**
     * This contains all the images data needed to reconstruct the cubemap.
     */
    mipmaps: Array<BufferImageData>;
    /**
     * Defines the scale applied to environment texture. This manages the range of LOD level used for IBL according to the roughness.
     */
    lodGenerationScale: number;
}
/**
 * Options for creating environment textures
 */
export interface CreateEnvTextureOptions {
    /**
     * The mime type of encoded images.
     */
    imageType?: string;
    /**
     * the image quality of encoded WebP images.
     */
    imageQuality?: number;
}
/**
 * Gets the environment info from an env file.
 * @param data The array buffer containing the .env bytes.
 * @returns the environment file info (the json header) if successfully parsed, normalized to the latest supported version.
 */
export function GetEnvInfo(data: ArrayBufferView): Nullable<EnvironmentTextureInfoV2>;
/**
 * Normalizes any supported version of the environment file info to the latest version
 * @param info environment file info on any supported version
 * @returns environment file info in the latest supported version
 * @private
 */
export function normalizeEnvInfo(info: EnvironmentTextureInfo): EnvironmentTextureInfoV2;
/**
 * Creates an environment texture from a loaded cube texture.
 * @param texture defines the cube texture to convert in env file
 * @param options options for the conversion process
 * @param options.imageType the mime type for the encoded images, with support for "image/png" (default) and "image/webp"
 * @param options.imageQuality the image quality of encoded WebP images.
 * @returns a promise containing the environment data if successful.
 */
export function CreateEnvTextureAsync(texture: BaseTexture, options?: CreateEnvTextureOptions): Promise<ArrayBuffer>;
/**
 * Creates the ArrayBufferViews used for initializing environment texture image data.
 * @param data the image data
 * @param info parameters that determine what views will be created for accessing the underlying buffer
 * @returns the views described by info providing access to the underlying buffer
 */
export function CreateImageDataArrayBufferViews(data: ArrayBufferView, info: EnvironmentTextureInfo): Array<Array<ArrayBufferView>>;
/**
 * Uploads the texture info contained in the env file to the GPU.
 * @param texture defines the internal texture to upload to
 * @param data defines the data to load
 * @param info defines the texture info retrieved through the GetEnvInfo method
 * @returns a promise
 */
export function UploadEnvLevelsAsync(texture: InternalTexture, data: ArrayBufferView, info: EnvironmentTextureInfo): Promise<void>;
/**
 * Uploads the levels of image data to the GPU.
 * @param texture defines the internal texture to upload to
 * @param imageData defines the array buffer views of image data [mipmap][face]
 * @param imageType the mime type of the image data
 * @returns a promise
 */
export function UploadLevelsAsync(texture: InternalTexture, imageData: ArrayBufferView[][], imageType?: string): Promise<void>;
/**
 * Uploads spherical polynomials information to the texture.
 * @param texture defines the texture we are trying to upload the information to
 * @param info defines the environment texture info retrieved through the GetEnvInfo method
 */
export function UploadEnvSpherical(texture: InternalTexture, info: EnvironmentTextureInfo): void;
/**
 * @internal
 */
export function _UpdateRGBDAsync(internalTexture: InternalTexture, data: ArrayBufferView[][], sphericalPolynomial: Nullable<SphericalPolynomial>, lodScale: number, lodOffset: number): Promise<InternalTexture>;
/**
 * Sets of helpers addressing the serialization and deserialization of environment texture
 * stored in a BabylonJS env file.
 * Those files are usually stored as .env files.
 */
export const EnvironmentTextureTools: {
    /**
     * Gets the environment info from an env file.
     * @param data The array buffer containing the .env bytes.
     * @returns the environment file info (the json header) if successfully parsed, normalized to the latest supported version.
     */
    GetEnvInfo: typeof GetEnvInfo;
    /**
     * Creates an environment texture from a loaded cube texture.
     * @param texture defines the cube texture to convert in env file
     * @param options options for the conversion process
     * @param options.imageType the mime type for the encoded images, with support for "image/png" (default) and "image/webp"
     * @param options.imageQuality the image quality of encoded WebP images.
     * @returns a promise containing the environment data if successful.
     */
    CreateEnvTextureAsync: typeof CreateEnvTextureAsync;
    /**
     * Creates the ArrayBufferViews used for initializing environment texture image data.
     * @param data the image data
     * @param info parameters that determine what views will be created for accessing the underlying buffer
     * @returns the views described by info providing access to the underlying buffer
     */
    CreateImageDataArrayBufferViews: typeof CreateImageDataArrayBufferViews;
    /**
     * Uploads the texture info contained in the env file to the GPU.
     * @param texture defines the internal texture to upload to
     * @param data defines the data to load
     * @param info defines the texture info retrieved through the GetEnvInfo method
     * @returns a promise
     */
    UploadEnvLevelsAsync: typeof UploadEnvLevelsAsync;
    /**
     * Uploads the levels of image data to the GPU.
     * @param texture defines the internal texture to upload to
     * @param imageData defines the array buffer views of image data [mipmap][face]
     * @param imageType the mime type of the image data
     * @returns a promise
     */
    UploadLevelsAsync: typeof UploadLevelsAsync;
    /**
     * Uploads spherical polynomials information to the texture.
     * @param texture defines the texture we are trying to upload the information to
     * @param info defines the environment texture info retrieved through the GetEnvInfo method
     */
    UploadEnvSpherical: typeof UploadEnvSpherical;
};
export {};

}
declare module "babylonjs/Misc/error" {
/**
 * Base error. Due to limitations of typedoc-check and missing documentation
 * in lib.es5.d.ts, cannot extend Error directly for RuntimeError.
 * @ignore
 */
export abstract class BaseError extends Error {
    protected static _setPrototypeOf: (o: any, proto: object | null) => any;
}
/**
 * Error codes for BaseError
 */
export const ErrorCodes: {
    /** Invalid or empty mesh vertex positions. */
    readonly MeshInvalidPositionsError: 0;
    /** Unsupported texture found. */
    readonly UnsupportedTextureError: 1000;
    /** Unexpected magic number found in GLTF file header. */
    readonly GLTFLoaderUnexpectedMagicError: 2000;
    /** SceneLoader generic error code. Ideally wraps the inner exception. */
    readonly SceneLoaderError: 3000;
    /** Load file error */
    readonly LoadFileError: 4000;
    /** Request file error */
    readonly RequestFileError: 4001;
    /** Read file error */
    readonly ReadFileError: 4002;
};
/**
 * Error code type
 */
export type ErrorCodesType = typeof ErrorCodes[keyof typeof ErrorCodes];
/**
 * Application runtime error
 */
export class RuntimeError extends BaseError {
    /**
     * The error code
     */
    errorCode: ErrorCodesType;
    /**
     * The error that caused this outer error
     */
    innerError?: Error;
    /**
     * Creates a new RuntimeError
     * @param message defines the message of the error
     * @param errorCode the error code
     * @param innerError the error that caused the outer error
     */
    constructor(message: string, errorCode: ErrorCodesType, innerError?: Error);
}

}
declare module "babylonjs/Misc/fileRequest" {
import { Observable } from "babylonjs/Misc/observable";
/**
 * File request interface
 */
export interface IFileRequest {
    /**
     * Raised when the request is complete (success or error).
     */
    onCompleteObservable: Observable<IFileRequest>;
    /**
     * Aborts the request for a file.
     */
    abort: () => void;
}

}
declare module "babylonjs/Misc/filesInput" {
import { Engine } from "babylonjs/Engines/engine";
import { Scene } from "babylonjs/scene";
import { ISceneLoaderProgressEvent } from "babylonjs/Loading/sceneLoader";
import { Nullable } from "babylonjs/types";
/**
 * Class used to help managing file picking and drag-n-drop
 */
export class FilesInput {
    /**
     * List of files ready to be loaded
     */
    static get FilesToLoad(): {
        [key: string]: File;
    };
    /**
     * Callback called when a file is processed
     */
    onProcessFileCallback: (file: File, name: string, extension: string, setSceneFileToLoad: (sceneFile: File) => void) => boolean;
    /**
     * Function used when loading the scene file
     * @param sceneFile
     * @param onProgress
     */
    loadAsync: (sceneFile: File, onProgress: Nullable<(event: ISceneLoaderProgressEvent) => void>) => Promise<Scene>;
    private _engine;
    private _currentScene;
    private _sceneLoadedCallback;
    private _progressCallback;
    private _additionalRenderLoopLogicCallback;
    private _textureLoadingCallback;
    private _startingProcessingFilesCallback;
    private _onReloadCallback;
    private _errorCallback;
    private _elementToMonitor;
    private _sceneFileToLoad;
    private _filesToLoad;
    /**
     * Creates a new FilesInput
     * @param engine defines the rendering engine
     * @param scene defines the hosting scene
     * @param sceneLoadedCallback callback called when scene is loaded
     * @param progressCallback callback called to track progress
     * @param additionalRenderLoopLogicCallback callback called to add user logic to the rendering loop
     * @param textureLoadingCallback callback called when a texture is loading
     * @param startingProcessingFilesCallback callback called when the system is about to process all files
     * @param onReloadCallback callback called when a reload is requested
     * @param errorCallback callback call if an error occurs
     */
    constructor(engine: Engine, scene: Nullable<Scene>, sceneLoadedCallback: Nullable<(sceneFile: File, scene: Scene) => void>, progressCallback: Nullable<(progress: ISceneLoaderProgressEvent) => void>, additionalRenderLoopLogicCallback: Nullable<() => void>, textureLoadingCallback: Nullable<(remaining: number) => void>, startingProcessingFilesCallback: Nullable<(files?: File[]) => void>, onReloadCallback: Nullable<(sceneFile: File) => void>, errorCallback: Nullable<(sceneFile: File, scene: Nullable<Scene>, message: string) => void>);
    private _dragEnterHandler;
    private _dragOverHandler;
    private _dropHandler;
    /**
     * Calls this function to listen to drag'n'drop events on a specific DOM element
     * @param elementToMonitor defines the DOM element to track
     */
    monitorElementForDragNDrop(elementToMonitor: HTMLElement): void;
    /** Gets the current list of files to load */
    get filesToLoad(): File[];
    /**
     * Release all associated resources
     */
    dispose(): void;
    private _renderFunction;
    private _drag;
    private _drop;
    private _traverseFolder;
    private _processFiles;
    /**
     * Load files from a drop event
     * @param event defines the drop event to use as source
     */
    loadFiles(event: any): void;
    private _processReload;
    /**
     * Reload the current scene from the loaded files
     */
    reload(): void;
}

}
declare module "babylonjs/Misc/filesInputStore" {
/**
 * Class used to help managing file picking and drag'n'drop
 * File Storage
 */
export class FilesInputStore {
    /**
     * List of files ready to be loaded
     */
    static FilesToLoad: {
        [key: string]: File;
    };
}

}
declare module "babylonjs/Misc/fileTools" {
import { WebRequest } from "babylonjs/Misc/webRequest";
import { Nullable } from "babylonjs/types";
import { IOfflineProvider } from "babylonjs/Offline/IOfflineProvider";
import { IFileRequest } from "babylonjs/Misc/fileRequest";
import { RuntimeError } from "babylonjs/Misc/error";
/** @ignore */
export class LoadFileError extends RuntimeError {
    request?: WebRequest;
    file?: File;
    /**
     * Creates a new LoadFileError
     * @param message defines the message of the error
     * @param object defines the optional web request
     */
    constructor(message: string, object?: WebRequest | File);
}
/** @ignore */
export class RequestFileError extends RuntimeError {
    request: WebRequest;
    /**
     * Creates a new LoadFileError
     * @param message defines the message of the error
     * @param request defines the optional web request
     */
    constructor(message: string, request: WebRequest);
}
/** @ignore */
export class ReadFileError extends RuntimeError {
    file: File;
    /**
     * Creates a new ReadFileError
     * @param message defines the message of the error
     * @param file defines the optional file
     */
    constructor(message: string, file: File);
}
/**
 * @internal
 */
export const FileToolsOptions: {
    DefaultRetryStrategy: (url: string, request: WebRequest, retryIndex: number) => number;
    BaseUrl: string;
    CorsBehavior: string | ((url: string | string[]) => string);
    PreprocessUrl: (url: string) => string;
};
/**
 * Sets the cors behavior on a dom element. This will add the required Tools.CorsBehavior to the element.
 * @param url define the url we are trying
 * @param element define the dom element where to configure the cors policy
 * @internal
 */
export const SetCorsBehavior: (url: string | string[], element: {
    crossOrigin: string | null;
}) => void;
/**
 * Loads an image as an HTMLImageElement.
 * @param input url string, ArrayBuffer, or Blob to load
 * @param onLoad callback called when the image successfully loads
 * @param onError callback called when the image fails to load
 * @param offlineProvider offline provider for caching
 * @param mimeType optional mime type
 * @param imageBitmapOptions
 * @returns the HTMLImageElement of the loaded image
 * @internal
 */
export const LoadImage: (input: string | ArrayBuffer | ArrayBufferView | Blob, onLoad: (img: HTMLImageElement | ImageBitmap) => void, onError: (message?: string, exception?: any) => void, offlineProvider: Nullable<IOfflineProvider>, mimeType?: string, imageBitmapOptions?: ImageBitmapOptions) => Nullable<HTMLImageElement>;
/**
 * Reads a file from a File object
 * @param file defines the file to load
 * @param onSuccess defines the callback to call when data is loaded
 * @param onProgress defines the callback to call during loading process
 * @param useArrayBuffer defines a boolean indicating that data must be returned as an ArrayBuffer
 * @param onError defines the callback to call when an error occurs
 * @returns a file request object
 * @internal
 */
export const ReadFile: (file: File, onSuccess: (data: any) => void, onProgress?: ((ev: ProgressEvent) => any) | undefined, useArrayBuffer?: boolean, onError?: ((error: ReadFileError) => void) | undefined) => IFileRequest;
/**
 * Loads a file from a url, a data url, or a file url
 * @param fileOrUrl file, url, data url, or file url to load
 * @param onSuccess callback called when the file successfully loads
 * @param onProgress callback called while file is loading (if the server supports this mode)
 * @param offlineProvider defines the offline provider for caching
 * @param useArrayBuffer defines a boolean indicating that date must be returned as ArrayBuffer
 * @param onError callback called when the file fails to load
 * @param onOpened
 * @returns a file request object
 * @internal
 */
export const LoadFile: (fileOrUrl: File | string, onSuccess: (data: string | ArrayBuffer, responseURL?: string, contentType?: Nullable<string>) => void, onProgress?: ((ev: ProgressEvent) => void) | undefined, offlineProvider?: IOfflineProvider, useArrayBuffer?: boolean, onError?: ((request?: WebRequest, exception?: LoadFileError) => void) | undefined, onOpened?: ((request: WebRequest) => void) | undefined) => IFileRequest;
/**
 * Loads a file from a url
 * @param url url to load
 * @param onSuccess callback called when the file successfully loads
 * @param onProgress callback called while file is loading (if the server supports this mode)
 * @param offlineProvider defines the offline provider for caching
 * @param useArrayBuffer defines a boolean indicating that date must be returned as ArrayBuffer
 * @param onError callback called when the file fails to load
 * @param onOpened callback called when the web request is opened
 * @returns a file request object
 * @internal
 */
export const RequestFile: (url: string, onSuccess?: ((data: string | ArrayBuffer, request?: WebRequest) => void) | undefined, onProgress?: ((event: ProgressEvent) => void) | undefined, offlineProvider?: IOfflineProvider, useArrayBuffer?: boolean, onError?: ((error: RequestFileError) => void) | undefined, onOpened?: ((request: WebRequest) => void) | undefined) => IFileRequest;
/**
 * Checks if the loaded document was accessed via `file:`-Protocol.
 * @returns boolean
 * @internal
 */
export const IsFileURL: () => boolean;
/**
 * Test if the given uri is a valid base64 data url
 * @param uri The uri to test
 * @returns True if the uri is a base64 data url or false otherwise
 * @internal
 */
export const IsBase64DataUrl: (uri: string) => boolean;
export const TestBase64DataUrl: (uri: string) => {
    match: boolean;
    type: string;
};
/**
 * Decode the given base64 uri.
 * @param uri The uri to decode
 * @returns The decoded base64 data.
 * @internal
 */
export function DecodeBase64UrlToBinary(uri: string): ArrayBuffer;
/**
 * Decode the given base64 uri into a UTF-8 encoded string.
 * @param uri The uri to decode
 * @returns The decoded base64 data.
 * @internal
 */
export const DecodeBase64UrlToString: (uri: string) => string;
/**
* FileTools defined as any.
* This should not be imported or used in future releases or in any module in the framework
* @internal
* @deprecated import the needed function from fileTools.ts
*/
export let FileTools: {
    DecodeBase64UrlToBinary: (uri: string) => ArrayBuffer;
    DecodeBase64UrlToString: (uri: string) => string;
    DefaultRetryStrategy: any;
    BaseUrl: any;
    CorsBehavior: any;
    PreprocessUrl: any;
    IsBase64DataUrl: (uri: string) => boolean;
    IsFileURL: () => boolean;
    LoadFile: (fileOrUrl: string | File, onSuccess: (data: string | ArrayBuffer, responseURL?: string | undefined) => void, onProgress?: ((ev: ProgressEvent<EventTarget>) => void) | undefined, offlineProvider?: IOfflineProvider | undefined, useArrayBuffer?: boolean | undefined, onError?: ((request?: WebRequest | undefined, exception?: LoadFileError | undefined) => void) | undefined, onOpened?: ((request: WebRequest) => void) | undefined) => IFileRequest;
    LoadImage: (input: string | ArrayBuffer | Blob | ArrayBufferView, onLoad: (img: HTMLImageElement | ImageBitmap) => void, onError: (message?: string | undefined, exception?: any) => void, offlineProvider: Nullable<IOfflineProvider>, mimeType?: string | undefined, imageBitmapOptions?: ImageBitmapOptions | undefined) => Nullable<HTMLImageElement>;
    ReadFile: (file: File, onSuccess: (data: any) => void, onProgress?: ((ev: ProgressEvent<EventTarget>) => any) | undefined, useArrayBuffer?: boolean | undefined, onError?: ((error: ReadFileError) => void) | undefined) => IFileRequest;
    RequestFile: (url: string, onSuccess: (data: string | ArrayBuffer, request?: WebRequest | undefined) => void, onProgress?: ((event: ProgressEvent<EventTarget>) => void) | undefined, offlineProvider?: IOfflineProvider | undefined, useArrayBuffer?: boolean | undefined, onError?: ((error: RequestFileError) => void) | undefined, onOpened?: ((request: WebRequest) => void) | undefined) => IFileRequest;
    SetCorsBehavior: (url: string | string[], element: {
        crossOrigin: string | null;
    }) => void;
};
/**
 * @param DecodeBase64UrlToBinary
 * @param DecodeBase64UrlToString
 * @param FileToolsOptions
 * @internal
 */
export const _injectLTSFileTools: (DecodeBase64UrlToBinary: (uri: string) => ArrayBuffer, DecodeBase64UrlToString: (uri: string) => string, FileToolsOptions: {
    DefaultRetryStrategy: any;
    BaseUrl: any;
    CorsBehavior: any;
    PreprocessUrl: any;
}, IsBase64DataUrl: (uri: string) => boolean, IsFileURL: () => boolean, LoadFile: (fileOrUrl: string | File, onSuccess: (data: string | ArrayBuffer, responseURL?: string | undefined) => void, onProgress?: ((ev: ProgressEvent<EventTarget>) => void) | undefined, offlineProvider?: IOfflineProvider | undefined, useArrayBuffer?: boolean | undefined, onError?: ((request?: WebRequest | undefined, exception?: LoadFileError | undefined) => void) | undefined, onOpened?: ((request: WebRequest) => void) | undefined) => IFileRequest, LoadImage: (input: string | ArrayBuffer | ArrayBufferView | Blob, onLoad: (img: HTMLImageElement | ImageBitmap) => void, onError: (message?: string | undefined, exception?: any) => void, offlineProvider: Nullable<IOfflineProvider>, mimeType?: string, imageBitmapOptions?: ImageBitmapOptions | undefined) => Nullable<HTMLImageElement>, ReadFile: (file: File, onSuccess: (data: any) => void, onProgress?: ((ev: ProgressEvent<EventTarget>) => any) | undefined, useArrayBuffer?: boolean | undefined, onError?: ((error: ReadFileError) => void) | undefined) => IFileRequest, RequestFile: (url: string, onSuccess: (data: string | ArrayBuffer, request?: WebRequest | undefined) => void, onProgress?: ((event: ProgressEvent<EventTarget>) => void) | undefined, offlineProvider?: IOfflineProvider | undefined, useArrayBuffer?: boolean | undefined, onError?: ((error: RequestFileError) => void) | undefined, onOpened?: ((request: WebRequest) => void) | undefined) => IFileRequest, SetCorsBehavior: (url: string | string[], element: {
    crossOrigin: string | null;
}) => void) => void;

}
declare module "babylonjs/Misc/gradients" {
import { Color3 } from "babylonjs/Maths/math.color";
import { Color4 } from "babylonjs/Maths/math.color";
/** Interface used by value gradients (color, factor, ...) */
export interface IValueGradient {
    /**
     * Gets or sets the gradient value (between 0 and 1)
     */
    gradient: number;
}
/** Class used to store color4 gradient */
export class ColorGradient implements IValueGradient {
    /**
     * Gets or sets the gradient value (between 0 and 1)
     */
    gradient: number;
    /**
     * Gets or sets first associated color
     */
    color1: Color4;
    /**
     * Gets or sets second associated color
     */
    color2?: Color4 | undefined;
    /**
     * Creates a new color4 gradient
     * @param gradient gets or sets the gradient value (between 0 and 1)
     * @param color1 gets or sets first associated color
     * @param color2 gets or sets first second color
     */
    constructor(
    /**
     * Gets or sets the gradient value (between 0 and 1)
     */
    gradient: number, 
    /**
     * Gets or sets first associated color
     */
    color1: Color4, 
    /**
     * Gets or sets second associated color
     */
    color2?: Color4 | undefined);
    /**
     * Will get a color picked randomly between color1 and color2.
     * If color2 is undefined then color1 will be used
     * @param result defines the target Color4 to store the result in
     */
    getColorToRef(result: Color4): void;
}
/** Class used to store color 3 gradient */
export class Color3Gradient implements IValueGradient {
    /**
     * Gets or sets the gradient value (between 0 and 1)
     */
    gradient: number;
    /**
     * Gets or sets the associated color
     */
    color: Color3;
    /**
     * Creates a new color3 gradient
     * @param gradient gets or sets the gradient value (between 0 and 1)
     * @param color gets or sets associated color
     */
    constructor(
    /**
     * Gets or sets the gradient value (between 0 and 1)
     */
    gradient: number, 
    /**
     * Gets or sets the associated color
     */
    color: Color3);
}
/** Class used to store factor gradient */
export class FactorGradient implements IValueGradient {
    /**
     * Gets or sets the gradient value (between 0 and 1)
     */
    gradient: number;
    /**
     * Gets or sets first associated factor
     */
    factor1: number;
    /**
     * Gets or sets second associated factor
     */
    factor2?: number | undefined;
    /**
     * Creates a new factor gradient
     * @param gradient gets or sets the gradient value (between 0 and 1)
     * @param factor1 gets or sets first associated factor
     * @param factor2 gets or sets second associated factor
     */
    constructor(
    /**
     * Gets or sets the gradient value (between 0 and 1)
     */
    gradient: number, 
    /**
     * Gets or sets first associated factor
     */
    factor1: number, 
    /**
     * Gets or sets second associated factor
     */
    factor2?: number | undefined);
    /**
     * Will get a number picked randomly between factor1 and factor2.
     * If factor2 is undefined then factor1 will be used
     * @returns the picked number
     */
    getFactor(): number;
}
/**
 * Helper used to simplify some generic gradient tasks
 */
export class GradientHelper {
    /**
     * Gets the current gradient from an array of IValueGradient
     * @param ratio defines the current ratio to get
     * @param gradients defines the array of IValueGradient
     * @param updateFunc defines the callback function used to get the final value from the selected gradients
     */
    static GetCurrentGradient(ratio: number, gradients: IValueGradient[], updateFunc: (current: IValueGradient, next: IValueGradient, scale: number) => void): void;
}

}
declare module "babylonjs/Misc/guid" {
/**
 * Implementation from http://stackoverflow.com/questions/105034/how-to-create-a-guid-uuid-in-javascript/2117523#answer-2117523
 * Be aware Math.random() could cause collisions, but:
 * "All but 6 of the 128 bits of the ID are randomly generated, which means that for any two ids, there's a 1 in 2^^122 (or 5.3x10^^36) chance they'll collide"
 * @returns a pseudo random id
 */
export function RandomGUID(): string;
/**
 * Class used to manipulate GUIDs
 */
export const GUID: {
    /**
     * Implementation from http://stackoverflow.com/questions/105034/how-to-create-a-guid-uuid-in-javascript/2117523#answer-2117523
     * Be aware Math.random() could cause collisions, but:
     * "All but 6 of the 128 bits of the ID are randomly generated, which means that for any two ids, there's a 1 in 2^^122 (or 5.3x10^^36) chance they'll collide"
     * @returns a pseudo random id
     */
    RandomId: typeof RandomGUID;
};

}
declare module "babylonjs/Misc/HighDynamicRange/cubemapToSphericalPolynomial" {
import { SphericalPolynomial } from "babylonjs/Maths/sphericalPolynomial";
import { BaseTexture } from "babylonjs/Materials/Textures/baseTexture";
import { Nullable } from "babylonjs/types";
import { CubeMapInfo } from "babylonjs/Misc/HighDynamicRange/panoramaToCubemap";
/**
 * Helper class dealing with the extraction of spherical polynomial dataArray
 * from a cube map.
 */
export class CubeMapToSphericalPolynomialTools {
    private static _FileFaces;
    /**
     * Converts a texture to the according Spherical Polynomial data.
     * This extracts the first 3 orders only as they are the only one used in the lighting.
     *
     * @param texture The texture to extract the information from.
     * @returns The Spherical Polynomial data.
     */
    static ConvertCubeMapTextureToSphericalPolynomial(texture: BaseTexture): Nullable<Promise<SphericalPolynomial>>;
    /**
     * Compute the area on the unit sphere of the rectangle defined by (x,y) and the origin
     * See https://www.rorydriscoll.com/2012/01/15/cubemap-texel-solid-angle/
     * @param x
     * @param y
     */
    private static _AreaElement;
    /**
     * Converts a cubemap to the according Spherical Polynomial data.
     * This extracts the first 3 orders only as they are the only one used in the lighting.
     *
     * @param cubeInfo The Cube map to extract the information from.
     * @returns The Spherical Polynomial data.
     */
    static ConvertCubeMapToSphericalPolynomial(cubeInfo: CubeMapInfo): SphericalPolynomial;
}

}
declare module "babylonjs/Misc/HighDynamicRange/hdr" {
import { CubeMapInfo } from "babylonjs/Misc/HighDynamicRange/panoramaToCubemap";
/**
 * Header information of HDR texture files.
 */
export interface HDRInfo {
    /**
     * The height of the texture in pixels.
     */
    height: number;
    /**
     * The width of the texture in pixels.
     */
    width: number;
    /**
     * The index of the beginning of the data in the binary file.
     */
    dataPosition: number;
}
/**
 * This groups tools to convert HDR texture to native colors array.
 */
export class HDRTools {
    private static _Ldexp;
    private static _Rgbe2float;
    private static _ReadStringLine;
    /**
     * Reads header information from an RGBE texture stored in a native array.
     * More information on this format are available here:
     * https://en.wikipedia.org/wiki/RGBE_image_format
     *
     * @param uint8array The binary file stored in  native array.
     * @returns The header information.
     */
    static RGBE_ReadHeader(uint8array: Uint8Array): HDRInfo;
    /**
     * Returns the cubemap information (each faces texture data) extracted from an RGBE texture.
     * This RGBE texture needs to store the information as a panorama.
     *
     * More information on this format are available here:
     * https://en.wikipedia.org/wiki/RGBE_image_format
     *
     * @param buffer The binary file stored in an array buffer.
     * @param size The expected size of the extracted cubemap.
     * @returns The Cube Map information.
     */
    static GetCubeMapTextureData(buffer: ArrayBuffer, size: number): CubeMapInfo;
    /**
     * Returns the pixels data extracted from an RGBE texture.
     * This pixels will be stored left to right up to down in the R G B order in one array.
     *
     * More information on this format are available here:
     * https://en.wikipedia.org/wiki/RGBE_image_format
     *
     * @param uint8array The binary file stored in an array buffer.
     * @param hdrInfo The header information of the file.
     * @returns The pixels data in RGB right to left up to down order.
     */
    static RGBE_ReadPixels(uint8array: Uint8Array, hdrInfo: HDRInfo): Float32Array;
    private static _RGBEReadPixelsRLE;
    private static _RGBEReadPixelsNOTRLE;
}

}
declare module "babylonjs/Misc/HighDynamicRange/index" {
export * from "babylonjs/Misc/HighDynamicRange/cubemapToSphericalPolynomial";
export * from "babylonjs/Misc/HighDynamicRange/hdr";
export * from "babylonjs/Misc/HighDynamicRange/panoramaToCubemap";

}
declare module "babylonjs/Misc/HighDynamicRange/panoramaToCubemap" {
import { Nullable } from "babylonjs/types";
/**
 * CubeMap information grouping all the data for each faces as well as the cubemap size.
 */
export interface CubeMapInfo {
    /**
     * The pixel array for the front face.
     * This is stored in format, left to right, up to down format.
     */
    front: Nullable<ArrayBufferView>;
    /**
     * The pixel array for the back face.
     * This is stored in format, left to right, up to down format.
     */
    back: Nullable<ArrayBufferView>;
    /**
     * The pixel array for the left face.
     * This is stored in format, left to right, up to down format.
     */
    left: Nullable<ArrayBufferView>;
    /**
     * The pixel array for the right face.
     * This is stored in format, left to right, up to down format.
     */
    right: Nullable<ArrayBufferView>;
    /**
     * The pixel array for the up face.
     * This is stored in format, left to right, up to down format.
     */
    up: Nullable<ArrayBufferView>;
    /**
     * The pixel array for the down face.
     * This is stored in format, left to right, up to down format.
     */
    down: Nullable<ArrayBufferView>;
    /**
     * The size of the cubemap stored.
     *
     * Each faces will be size * size pixels.
     */
    size: number;
    /**
     * The format of the texture.
     *
     * RGBA, RGB.
     */
    format: number;
    /**
     * The type of the texture data.
     *
     * UNSIGNED_INT, FLOAT.
     */
    type: number;
    /**
     * Specifies whether the texture is in gamma space.
     */
    gammaSpace: boolean;
}
/**
 * Helper class useful to convert panorama picture to their cubemap representation in 6 faces.
 */
export class PanoramaToCubeMapTools {
    private static FACE_LEFT;
    private static FACE_RIGHT;
    private static FACE_FRONT;
    private static FACE_BACK;
    private static FACE_DOWN;
    private static FACE_UP;
    /**
     * Converts a panorama stored in RGB right to left up to down format into a cubemap (6 faces).
     *
     * @param float32Array The source data.
     * @param inputWidth The width of the input panorama.
     * @param inputHeight The height of the input panorama.
     * @param size The willing size of the generated cubemap (each faces will be size * size pixels)
     * @returns The cubemap data
     */
    static ConvertPanoramaToCubemap(float32Array: Float32Array, inputWidth: number, inputHeight: number, size: number): CubeMapInfo;
    private static CreateCubemapTexture;
    private static CalcProjectionSpherical;
}

}
declare module "babylonjs/Misc/iInspectable" {
/**
 * Enum that determines the text-wrapping mode to use.
 */
export enum InspectableType {
    /**
     * Checkbox for booleans
     */
    Checkbox = 0,
    /**
     * Sliders for numbers
     */
    Slider = 1,
    /**
     * Vector3
     */
    Vector3 = 2,
    /**
     * Quaternions
     */
    Quaternion = 3,
    /**
     * Color3
     */
    Color3 = 4,
    /**
     * String
     */
    String = 5,
    /**
     * Button
     */
    Button = 6,
    /**
     * Options
     */
    Options = 7,
    /**
     * Tab
     */
    Tab = 8,
    /**
     * File button
     */
    FileButton = 9,
    /**
     * Vector2
     */
    Vector2 = 10
}
/**
 * Interface used to define custom inspectable options in "Options" mode.
 * This interface is used by the inspector to display the list of options
 */
export interface IInspectableOptions {
    /**
     * Defines the visible part of the option
     */
    label: string;
    /**
     * Defines the value part of the option (returned through the callback)
     */
    value: number | string;
    /**
     * Defines if the option should be selected or not
     */
    selected?: boolean;
}
/**
 * Interface used to define custom inspectable properties.
 * This interface is used by the inspector to display custom property grids
 * @see https://doc.babylonjs.com/toolsAndResources/inspector#extensibility
 */
export interface IInspectable {
    /**
     * Gets the label to display
     */
    label: string;
    /**
     * Gets the name of the property to edit
     */
    propertyName: string;
    /**
     * Gets the type of the editor to use
     */
    type: InspectableType;
    /**
     * Gets the minimum value of the property when using in "slider" mode
     */
    min?: number;
    /**
     * Gets the maximum value of the property when using in "slider" mode
     */
    max?: number;
    /**
     * Gets the setp to use when using in "slider" mode
     */
    step?: number;
    /**
     * Gets the callback function when using "Button" mode
     */
    callback?: () => void;
    /**
     * Gets the callback function when using "FileButton" mode
     */
    fileCallback?: (file: File) => void;
    /**
     * Gets the list of options when using "Option" mode
     */
    options?: IInspectableOptions[];
    /**
     * Gets the extensions to accept when using "FileButton" mode.
     * The value should be a comma separated string with the list of extensions to accept e.g., ".jpg, .png, .tga, .dds, .env".
     */
    accept?: string;
}

}
declare module "babylonjs/Misc/index" {
export * from "babylonjs/Misc/andOrNotEvaluator";
export * from "babylonjs/Misc/assetsManager";
export * from "babylonjs/Misc/basis";
export * from "babylonjs/Misc/dds";
export * from "babylonjs/Misc/decorators";
export * from "babylonjs/Misc/deferred";
export * from "babylonjs/Misc/environmentTextureTools";
export * from "babylonjs/Misc/meshExploder";
export * from "babylonjs/Misc/filesInput";
export * from "babylonjs/Misc/HighDynamicRange/index";
export * from "babylonjs/Misc/khronosTextureContainer";
export * from "babylonjs/Misc/observable";
export * from "babylonjs/Misc/observable.extensions";
export * from "babylonjs/Misc/performanceMonitor";
export * from "babylonjs/Misc/sceneOptimizer";
export * from "babylonjs/Misc/sceneSerializer";
export * from "babylonjs/Misc/smartArray";
export * from "babylonjs/Misc/stringDictionary";
export * from "babylonjs/Misc/tags";
export * from "babylonjs/Misc/textureTools";
export * from "babylonjs/Misc/tga";
export * from "babylonjs/Misc/tools";
export * from "babylonjs/Misc/videoRecorder";
export * from "babylonjs/Misc/virtualJoystick";
export * from "babylonjs/Misc/workerPool";
export * from "babylonjs/Misc/logger";
export * from "babylonjs/Misc/typeStore";
export * from "babylonjs/Misc/filesInputStore";
export * from "babylonjs/Misc/deepCopier";
export * from "babylonjs/Misc/pivotTools";
export * from "babylonjs/Misc/precisionDate";
export * from "babylonjs/Misc/screenshotTools";
export * from "babylonjs/Misc/webRequest";
export * from "babylonjs/Misc/iInspectable";
export * from "babylonjs/Misc/brdfTextureTools";
export * from "babylonjs/Misc/rgbdTextureTools";
export * from "babylonjs/Misc/gradients";
export * from "babylonjs/Misc/perfCounter";
export * from "babylonjs/Misc/fileRequest";
export * from "babylonjs/Misc/customAnimationFrameRequester";
export * from "babylonjs/Misc/retryStrategy";
export * from "babylonjs/Misc/interfaces/screenshotSize";
export * from "babylonjs/Misc/interfaces/iPerfViewer";
export * from "babylonjs/Misc/fileTools";
export * from "babylonjs/Misc/stringTools";
export * from "babylonjs/Misc/dataReader";
export * from "babylonjs/Misc/minMaxReducer";
export * from "babylonjs/Misc/depthReducer";
export * from "babylonjs/Misc/dataStorage";
export * from "babylonjs/Misc/sceneRecorder";
export * from "babylonjs/Misc/khronosTextureContainer2";
export * from "babylonjs/Misc/trajectoryClassifier";
export * from "babylonjs/Misc/timer";
export * from "babylonjs/Misc/copyTools";
export * from "babylonjs/Misc/reflector";
export * from "babylonjs/Misc/domManagement";
export * from "babylonjs/Misc/pressureObserverWrapper";
export * from "babylonjs/Misc/PerformanceViewer/index";
export * from "babylonjs/Misc/coroutine";
export * from "babylonjs/Misc/guid";
export * from "babylonjs/Misc/error";
export * from "babylonjs/Misc/observableCoroutine";
export * from "babylonjs/Misc/copyTextureToTexture";
export * from "babylonjs/Misc/dumpTools";

}
declare module "babylonjs/Misc/instantiationTools" {
/**
 * Class used to enable instantiation of objects by class name
 */
export class InstantiationTools {
    /**
     * Use this object to register external classes like custom textures or material
     * to allow the loaders to instantiate them
     */
    static RegisteredExternalClasses: {
        [key: string]: Object;
    };
    /**
     * Tries to instantiate a new object from a given class name
     * @param className defines the class name to instantiate
     * @returns the new object or null if the system was not able to do the instantiation
     */
    static Instantiate(className: string): any;
}

}
declare module "babylonjs/Misc/interfaces/iClipPlanesHolder" {
import { Nullable } from "babylonjs/types";
import { Plane } from "babylonjs/Maths/math";
/**
 * Interface used to define entities containing multiple clip planes
 */
export interface IClipPlanesHolder {
    /**
     * Gets or sets the active clipplane 1
     */
    clipPlane: Nullable<Plane>;
    /**
     * Gets or sets the active clipplane 2
     */
    clipPlane2: Nullable<Plane>;
    /**
     * Gets or sets the active clipplane 3
     */
    clipPlane3: Nullable<Plane>;
    /**
     * Gets or sets the active clipplane 4
     */
    clipPlane4: Nullable<Plane>;
    /**
     * Gets or sets the active clipplane 5
     */
    clipPlane5: Nullable<Plane>;
    /**
     * Gets or sets the active clipplane 6
     */
    clipPlane6: Nullable<Plane>;
}

}
declare module "babylonjs/Misc/interfaces/iPerfViewer" {
import { DynamicFloat32Array } from "babylonjs/Misc/PerformanceViewer/dynamicFloat32Array";
/**
 * Defines the shape of a collection of datasets that our graphing service uses for drawing purposes.
 */
export interface IPerfDatasets {
    /**
     * The ids of our dataset.
     */
    ids: string[];
    /**
     * The data to be processed by the performance graph. Each slice will be of the form of [timestamp, numberOfPoints, value1, value2...]
     */
    data: DynamicFloat32Array;
    /**
     * A list of starting indices for each slice of data collected. Used for fast access of an arbitrary slice inside the data array.
     */
    startingIndices: DynamicFloat32Array;
}
/**
 * Defines the shape of a the metadata the graphing service uses for drawing purposes.
 */
export interface IPerfMetadata {
    /**
     * The color of the line to be drawn.
     */
    color?: string;
    /**
     * Specifies if data should be hidden, falsey by default.
     */
    hidden?: boolean;
    /**
     * Specifies the category of the data
     */
    category?: string;
}
/**
 * Defines the shape of a custom user registered event.
 */
export interface IPerfCustomEvent {
    /**
     * The name of the event.
     */
    name: string;
    /**
     * The value for the event, if set we will use it as the value, otherwise we will count the number of occurrences.
     */
    value?: number;
}

}
declare module "babylonjs/Misc/interfaces/iWebRequest" {
/**
 * Interface used to define the mechanism to get data from the network
 */
export interface IWebRequest {
    /**
     * Returns client's response url
     */
    responseURL: string;
    /**
     * Returns client's status
     */
    status: number;
    /**
     * Returns client's status as a text
     */
    statusText: string;
}

}
declare module "babylonjs/Misc/interfaces/screenshotSize" {
/**
 * Interface for screenshot methods with describe argument called `size` as object with options
 * @link https://doc.babylonjs.com/api/classes/babylon.screenshottools
 */
export interface IScreenshotSize {
    /**
     * number in pixels for canvas height
     */
    height?: number;
    /**
     * multiplier allowing render at a higher or lower resolution
     * If value is defined then height and width will be ignored and taken from camera
     */
    precision?: number;
    /**
     * number in pixels for canvas width
     */
    width?: number;
}

}
declare module "babylonjs/Misc/khronosTextureContainer" {
import { InternalTexture } from "babylonjs/Materials/Textures/internalTexture";
/**
 * for description see https://www.khronos.org/opengles/sdk/tools/KTX/
 * for file layout see https://www.khronos.org/opengles/sdk/tools/KTX/file_format_spec/
 */
export class KhronosTextureContainer {
    /** contents of the KTX container file */
    data: ArrayBufferView;
    private static HEADER_LEN;
    private static COMPRESSED_2D;
    private static COMPRESSED_3D;
    private static TEX_2D;
    private static TEX_3D;
    /**
     * Gets the openGL type
     */
    glType: number;
    /**
     * Gets the openGL type size
     */
    glTypeSize: number;
    /**
     * Gets the openGL format
     */
    glFormat: number;
    /**
     * Gets the openGL internal format
     */
    glInternalFormat: number;
    /**
     * Gets the base internal format
     */
    glBaseInternalFormat: number;
    /**
     * Gets image width in pixel
     */
    pixelWidth: number;
    /**
     * Gets image height in pixel
     */
    pixelHeight: number;
    /**
     * Gets image depth in pixels
     */
    pixelDepth: number;
    /**
     * Gets the number of array elements
     */
    numberOfArrayElements: number;
    /**
     * Gets the number of faces
     */
    numberOfFaces: number;
    /**
     * Gets the number of mipmap levels
     */
    numberOfMipmapLevels: number;
    /**
     * Gets the bytes of key value data
     */
    bytesOfKeyValueData: number;
    /**
     * Gets the load type
     */
    loadType: number;
    /**
     * If the container has been made invalid (eg. constructor failed to correctly load array buffer)
     */
    isInvalid: boolean;
    /**
     * Creates a new KhronosTextureContainer
     * @param data contents of the KTX container file
     * @param facesExpected should be either 1 or 6, based whether a cube texture or or
     */
    constructor(
    /** contents of the KTX container file */
    data: ArrayBufferView, facesExpected: number);
    /**
     * Uploads KTX content to a Babylon Texture.
     * It is assumed that the texture has already been created & is currently bound
     * @internal
     */
    uploadLevels(texture: InternalTexture, loadMipmaps: boolean): void;
    private _upload2DCompressedLevels;
    /**
     * Checks if the given data starts with a KTX file identifier.
     * @param data the data to check
     * @returns true if the data is a KTX file or false otherwise
     */
    static IsValid(data: ArrayBufferView): boolean;
}

}
declare module "babylonjs/Misc/khronosTextureContainer2" {
import { InternalTexture } from "babylonjs/Materials/Textures/internalTexture";
import { ThinEngine } from "babylonjs/Engines/thinEngine";
import { Nullable } from "babylonjs/types";
import { IDecodedData, IKTX2DecoderOptions } from "babylonjs/Materials/Textures/ktx2decoderTypes";
/**
 * Class that defines the default KTX2 decoder options.
 *
 * This class is useful for providing options to the KTX2 decoder to control how the source data is transcoded.
 */
export class DefaultKTX2DecoderOptions {
    private _isDirty;
    /**
     * Gets the dirty flag
     */
    get isDirty(): boolean;
    private _useRGBAIfASTCBC7NotAvailableWhenUASTC?;
    /**
     * force a (uncompressed) RGBA transcoded format if transcoding a UASTC source format and ASTC + BC7 are not available as a compressed transcoded format
     */
    get useRGBAIfASTCBC7NotAvailableWhenUASTC(): boolean | undefined;
    set useRGBAIfASTCBC7NotAvailableWhenUASTC(value: boolean | undefined);
    private _useRGBAIfOnlyBC1BC3AvailableWhenUASTC;
    /**
     * force a (uncompressed) RGBA transcoded format if transcoding a UASTC source format and only BC1 or BC3 are available as a compressed transcoded format.
     * This property is true by default to favor speed over memory, because currently transcoding from UASTC to BC1/3 is slow because the transcoder transcodes
     * to uncompressed and then recompresses the texture
     */
    get useRGBAIfOnlyBC1BC3AvailableWhenUASTC(): boolean;
    set useRGBAIfOnlyBC1BC3AvailableWhenUASTC(value: boolean);
    private _forceRGBA?;
    /**
     * force to always use (uncompressed) RGBA for transcoded format
     */
    get forceRGBA(): boolean | undefined;
    set forceRGBA(value: boolean | undefined);
    private _forceR8?;
    /**
     * force to always use (uncompressed) R8 for transcoded format
     */
    get forceR8(): boolean | undefined;
    set forceR8(value: boolean | undefined);
    private _forceRG8?;
    /**
     * force to always use (uncompressed) RG8 for transcoded format
     */
    get forceRG8(): boolean | undefined;
    set forceRG8(value: boolean | undefined);
    private _bypassTranscoders?;
    /**
     * list of transcoders to bypass when looking for a suitable transcoder. The available transcoders are:
     *      UniversalTranscoder_UASTC_ASTC
     *      UniversalTranscoder_UASTC_BC7
     *      UniversalTranscoder_UASTC_RGBA_UNORM
     *      UniversalTranscoder_UASTC_RGBA_SRGB
     *      UniversalTranscoder_UASTC_R8_UNORM
     *      UniversalTranscoder_UASTC_RG8_UNORM
     *      MSCTranscoder
     */
    get bypassTranscoders(): string[] | undefined;
    set bypassTranscoders(value: string[] | undefined);
    private _ktx2DecoderOptions;
    /** @internal */
    _getKTX2DecoderOptions(): IKTX2DecoderOptions;
}
/**
 * Class for loading KTX2 files
 */
export class KhronosTextureContainer2 {
    private static _WorkerPoolPromise?;
    private static _DecoderModulePromise?;
    /**
     * URLs to use when loading the KTX2 decoder module as well as its dependencies
     * If a url is null, the default url is used (pointing to https://preview.babylonjs.com)
     * Note that jsDecoderModule can't be null and that the other dependencies will only be loaded if necessary
     * Urls you can change:
     *     URLConfig.jsDecoderModule
     *     URLConfig.wasmUASTCToASTC
     *     URLConfig.wasmUASTCToBC7
     *     URLConfig.wasmUASTCToRGBA_UNORM
     *     URLConfig.wasmUASTCToRGBA_SRGB
     *     URLConfig.wasmUASTCToR8_UNORM
     *     URLConfig.wasmUASTCToRG8_UNORM
     *     URLConfig.jsMSCTranscoder
     *     URLConfig.wasmMSCTranscoder
     *     URLConfig.wasmZSTDDecoder
     * You can see their default values in this PG: https://playground.babylonjs.com/#EIJH8L#29
     */
    static URLConfig: {
        jsDecoderModule: string;
        wasmUASTCToASTC: Nullable<string>;
        wasmUASTCToBC7: Nullable<string>;
        wasmUASTCToRGBA_UNORM: Nullable<string>;
        wasmUASTCToRGBA_SRGB: Nullable<string>;
        wasmUASTCToR8_UNORM: Nullable<string>;
        wasmUASTCToRG8_UNORM: Nullable<string>;
        jsMSCTranscoder: Nullable<string>;
        wasmMSCTranscoder: Nullable<string>;
        wasmZSTDDecoder: Nullable<string>;
    };
    /**
     * Default number of workers used to handle data decoding
     */
    static DefaultNumWorkers: number;
    /**
     * Default configuration for the KTX2 decoder.
     * The options defined in this way have priority over those passed when creating a KTX2 texture with new Texture(...).
     */
    static DefaultDecoderOptions: DefaultKTX2DecoderOptions;
    private static GetDefaultNumWorkers;
    private _engine;
    private static _Initialize;
    /**
     * Constructor
     * @param engine The engine to use
     * @param numWorkers The number of workers for async operations. Specify `0` to disable web workers and run synchronously in the current context.
     */
    constructor(engine: ThinEngine, numWorkers?: number);
    /**
     * @internal
     */
    uploadAsync(data: ArrayBufferView, internalTexture: InternalTexture, options?: IKTX2DecoderOptions & IDecodedData): Promise<void>;
    protected _createTexture(data: IDecodedData, internalTexture: InternalTexture, options?: IKTX2DecoderOptions & IDecodedData): void;
    /**
     * Checks if the given data starts with a KTX2 file identifier.
     * @param data the data to check
     * @returns true if the data is a KTX2 file or false otherwise
     */
    static IsValid(data: ArrayBufferView): boolean;
}

}
declare module "babylonjs/Misc/logger" {
/**
 * Logger used throughout the application to allow configuration of
 * the log level required for the messages.
 */
export class Logger {
    /**
     * No log
     */
    static readonly NoneLogLevel: number;
    /**
     * Only message logs
     */
    static readonly MessageLogLevel: number;
    /**
     * Only warning logs
     */
    static readonly WarningLogLevel: number;
    /**
     * Only error logs
     */
    static readonly ErrorLogLevel: number;
    /**
     * All logs
     */
    static readonly AllLogLevel: number;
    /**
     * Message to display when a message has been logged too many times
     */
    static MessageLimitReached: string;
    private static _LogCache;
    private static _LogLimitOutputs;
    private static _Levels;
    /**
     * Gets a value indicating the number of loading errors
     * @ignorenaming
     */
    static errorsCount: number;
    /**
     * Callback called when a new log is added
     */
    static OnNewCacheEntry: (entry: string) => void;
    private static _CheckLimit;
    private static _GenerateLimitMessage;
    private static _AddLogEntry;
    private static _FormatMessage;
    private static _LogDisabled;
    private static _LogEnabled;
    /**
     * Log a message to the console
     */
    static Log: (message: string, limit?: number) => void;
    /**
     * Write a warning message to the console
     */
    static Warn: (message: string, limit?: number) => void;
    /**
     * Write an error message to the console
     */
    static Error: (message: string, limit?: number) => void;
    /**
     * Gets current log cache (list of logs)
     */
    static get LogCache(): string;
    /**
     * Clears the log cache
     */
    static ClearLogCache(): void;
    /**
     * Sets the current log level (MessageLogLevel / WarningLogLevel / ErrorLogLevel)
     */
    static set LogLevels(level: number);
}

}
declare module "babylonjs/Misc/meshExploder" {
import { Mesh } from "babylonjs/Meshes/mesh";
/**
 * Class used to explode meshes (ie. to have a center and move them away from that center to better see the overall organization)
 */
export class MeshExploder {
    private _centerMesh;
    private _meshes;
    private _meshesOrigins;
    private _toCenterVectors;
    private _scaledDirection;
    private _newPosition;
    private _centerPosition;
    /**
     * Explodes meshes from a center mesh.
     * @param meshes The meshes to explode.
     * @param centerMesh The mesh to be center of explosion.
     */
    constructor(meshes: Array<Mesh>, centerMesh?: Mesh);
    private _setCenterMesh;
    /**
     * Get class name
     * @returns "MeshExploder"
     */
    getClassName(): string;
    /**
     * "Exploded meshes"
     * @returns Array of meshes with the centerMesh at index 0.
     */
    getMeshes(): Array<Mesh>;
    /**
     * Explodes meshes giving a specific direction
     * @param direction Number to multiply distance of each mesh's origin from center. Use a negative number to implode, or zero to reset.
     */
    explode(direction?: number): void;
}

}
declare module "babylonjs/Misc/minMaxReducer" {
import { Nullable } from "babylonjs/types";
import { RenderTargetTexture } from "babylonjs/Materials/Textures/renderTargetTexture";
import { Camera } from "babylonjs/Cameras/camera";
import { Observer } from "babylonjs/Misc/observable";
import { Observable } from "babylonjs/Misc/observable";
import { PostProcess } from "babylonjs/PostProcesses/postProcess";
import { PostProcessManager } from "babylonjs/PostProcesses/postProcessManager";
import { ThinEngine } from "babylonjs/Engines/thinEngine";
import "babylonjs/Shaders/minmaxRedux.fragment";
/**
 * This class computes a min/max reduction from a texture: it means it computes the minimum
 * and maximum values from all values of the texture.
 * It is performed on the GPU for better performances, thanks to a succession of post processes.
 * The source values are read from the red channel of the texture.
 */
export class MinMaxReducer {
    /**
     * Observable triggered when the computation has been performed
     */
    onAfterReductionPerformed: Observable<{
        min: number;
        max: number;
    }>;
    protected _camera: Camera;
    protected _sourceTexture: Nullable<RenderTargetTexture>;
    protected _reductionSteps: Nullable<Array<PostProcess>>;
    protected _postProcessManager: PostProcessManager;
    protected _onAfterUnbindObserver: Nullable<Observer<RenderTargetTexture>>;
    protected _forceFullscreenViewport: boolean;
    protected _onContextRestoredObserver: Nullable<Observer<ThinEngine>>;
    /**
     * Creates a min/max reducer
     * @param camera The camera to use for the post processes
     */
    constructor(camera: Camera);
    /**
     * Gets the texture used to read the values from.
     */
    get sourceTexture(): Nullable<RenderTargetTexture>;
    /**
     * Sets the source texture to read the values from.
     * One must indicate if the texture is a depth texture or not through the depthRedux parameter
     * because in such textures '1' value must not be taken into account to compute the maximum
     * as this value is used to clear the texture.
     * Note that the computation is not activated by calling this function, you must call activate() for that!
     * @param sourceTexture The texture to read the values from. The values should be in the red channel.
     * @param depthRedux Indicates if the texture is a depth texture or not
     * @param type The type of the textures created for the reduction (defaults to TEXTURETYPE_HALF_FLOAT)
     * @param forceFullscreenViewport Forces the post processes used for the reduction to be applied without taking into account viewport (defaults to true)
     */
    setSourceTexture(sourceTexture: RenderTargetTexture, depthRedux: boolean, type?: number, forceFullscreenViewport?: boolean): void;
    /**
     * Defines the refresh rate of the computation.
     * Use 0 to compute just once, 1 to compute on every frame, 2 to compute every two frames and so on...
     */
    get refreshRate(): number;
    set refreshRate(value: number);
    protected _activated: boolean;
    /**
     * Gets the activation status of the reducer
     */
    get activated(): boolean;
    /**
     * Activates the reduction computation.
     * When activated, the observers registered in onAfterReductionPerformed are
     * called after the computation is performed
     */
    activate(): void;
    /**
     * Deactivates the reduction computation.
     */
    deactivate(): void;
    /**
     * Disposes the min/max reducer
     * @param disposeAll true to dispose all the resources. You should always call this function with true as the parameter (or without any parameter as it is the default one). This flag is meant to be used internally.
     */
    dispose(disposeAll?: boolean): void;
}

}
declare module "babylonjs/Misc/observable" {
import { Nullable } from "babylonjs/types";
/**
 * A class serves as a medium between the observable and its observers
 */
export class EventState {
    /**
     * Create a new EventState
     * @param mask defines the mask associated with this state
     * @param skipNextObservers defines a flag which will instruct the observable to skip following observers when set to true
     * @param target defines the original target of the state
     * @param currentTarget defines the current target of the state
     */
    constructor(mask: number, skipNextObservers?: boolean, target?: any, currentTarget?: any);
    /**
     * Initialize the current event state
     * @param mask defines the mask associated with this state
     * @param skipNextObservers defines a flag which will instruct the observable to skip following observers when set to true
     * @param target defines the original target of the state
     * @param currentTarget defines the current target of the state
     * @returns the current event state
     */
    initialize(mask: number, skipNextObservers?: boolean, target?: any, currentTarget?: any): EventState;
    /**
     * An Observer can set this property to true to prevent subsequent observers of being notified
     */
    skipNextObservers: boolean;
    /**
     * Get the mask value that were used to trigger the event corresponding to this EventState object
     */
    mask: number;
    /**
     * The object that originally notified the event
     */
    target?: any;
    /**
     * The current object in the bubbling phase
     */
    currentTarget?: any;
    /**
     * This will be populated with the return value of the last function that was executed.
     * If it is the first function in the callback chain it will be the event data.
     */
    lastReturnValue?: any;
    /**
     * User defined information that will be sent to observers
     */
    userInfo?: any;
}
/**
 * Represent an Observer registered to a given Observable object.
 */
export class Observer<T> {
    /**
     * Defines the callback to call when the observer is notified
     */
    callback: (eventData: T, eventState: EventState) => void;
    /**
     * Defines the mask of the observer (used to filter notifications)
     */
    mask: number;
    /**
     * Defines the current scope used to restore the JS context
     */
    scope: any;
    /** @internal */
    _willBeUnregistered: boolean;
    /**
     * Gets or sets a property defining that the observer as to be unregistered after the next notification
     */
    unregisterOnNextCall: boolean;
    /**
     * Creates a new observer
     * @param callback defines the callback to call when the observer is notified
     * @param mask defines the mask of the observer (used to filter notifications)
     * @param scope defines the current scope used to restore the JS context
     */
    constructor(
    /**
     * Defines the callback to call when the observer is notified
     */
    callback: (eventData: T, eventState: EventState) => void, 
    /**
     * Defines the mask of the observer (used to filter notifications)
     */
    mask: number, 
    /**
     * Defines the current scope used to restore the JS context
     */
    scope?: any);
}
/**
 * The Observable class is a simple implementation of the Observable pattern.
 *
 * There's one slight particularity though: a given Observable can notify its observer using a particular mask value, only the Observers registered with this mask value will be notified.
 * This enable a more fine grained execution without having to rely on multiple different Observable objects.
 * For instance you may have a given Observable that have four different types of notifications: Move (mask = 0x01), Stop (mask = 0x02), Turn Right (mask = 0X04), Turn Left (mask = 0X08).
 * A given observer can register itself with only Move and Stop (mask = 0x03), then it will only be notified when one of these two occurs and will never be for Turn Left/Right.
 */
export class Observable<T> {
    private _observers;
    private _numObserversMarkedAsDeleted;
    /**
     * @internal
     */
    _eventState: EventState;
    private _onObserverAdded;
    /**
     * Create an observable from a Promise.
     * @param promise a promise to observe for fulfillment.
     * @param onErrorObservable an observable to notify if a promise was rejected.
     * @returns the new Observable
     */
    static FromPromise<T, E = Error>(promise: Promise<T>, onErrorObservable?: Observable<E>): Observable<T>;
    /**
     * Gets the list of observers
     * Note that observers that were recently deleted may still be present in the list because they are only really deleted on the next javascript tick!
     */
    get observers(): Array<Observer<T>>;
    /**
     * Creates a new observable
     * @param onObserverAdded defines a callback to call when a new observer is added
     */
    constructor(onObserverAdded?: (observer: Observer<T>) => void);
    /**
     * Create a new Observer with the specified callback
     * @param callback the callback that will be executed for that Observer
     * @param mask the mask used to filter observers
     * @param insertFirst if true the callback will be inserted at the first position, hence executed before the others ones. If false (default behavior) the callback will be inserted at the last position, executed after all the others already present.
     * @param scope optional scope for the callback to be called from
     * @param unregisterOnFirstCall defines if the observer as to be unregistered after the next notification
     * @returns the new observer created for the callback
     */
    add(callback: (eventData: T, eventState: EventState) => void, mask?: number, insertFirst?: boolean, scope?: any, unregisterOnFirstCall?: boolean): Nullable<Observer<T>>;
    /**
     * Create a new Observer with the specified callback and unregisters after the next notification
     * @param callback the callback that will be executed for that Observer
     * @returns the new observer created for the callback
     */
    addOnce(callback: (eventData: T, eventState: EventState) => void): Nullable<Observer<T>>;
    /**
     * Remove an Observer from the Observable object
     * @param observer the instance of the Observer to remove
     * @returns false if it doesn't belong to this Observable
     */
    remove(observer: Nullable<Observer<T>>): boolean;
    /**
     * Remove a callback from the Observable object
     * @param callback the callback to remove
     * @param scope optional scope. If used only the callbacks with this scope will be removed
     * @returns false if it doesn't belong to this Observable
     */
    removeCallback(callback: (eventData: T, eventState: EventState) => void, scope?: any): boolean;
    /**
     * @internal
     */
    _deferUnregister(observer: Observer<T>): void;
    private _remove;
    /**
     * Moves the observable to the top of the observer list making it get called first when notified
     * @param observer the observer to move
     */
    makeObserverTopPriority(observer: Observer<T>): void;
    /**
     * Moves the observable to the bottom of the observer list making it get called last when notified
     * @param observer the observer to move
     */
    makeObserverBottomPriority(observer: Observer<T>): void;
    /**
     * Notify all Observers by calling their respective callback with the given data
     * Will return true if all observers were executed, false if an observer set skipNextObservers to true, then prevent the subsequent ones to execute
     * @param eventData defines the data to send to all observers
     * @param mask defines the mask of the current notification (observers with incompatible mask (ie mask & observer.mask === 0) will not be notified)
     * @param target defines the original target of the state
     * @param currentTarget defines the current target of the state
     * @param userInfo defines any user info to send to observers
     * @returns false if the complete observer chain was not processed (because one observer set the skipNextObservers to true)
     */
    notifyObservers(eventData: T, mask?: number, target?: any, currentTarget?: any, userInfo?: any): boolean;
    /**
     * Notify a specific observer
     * @param observer defines the observer to notify
     * @param eventData defines the data to be sent to each callback
     * @param mask is used to filter observers defaults to -1
     */
    notifyObserver(observer: Observer<T>, eventData: T, mask?: number): void;
    /**
     * Gets a boolean indicating if the observable has at least one observer
     * @returns true is the Observable has at least one Observer registered
     */
    hasObservers(): boolean;
    /**
     * Clear the list of observers
     */
    clear(): void;
    /**
     * Clone the current observable
     * @returns a new observable
     */
    clone(): Observable<T>;
    /**
     * Does this observable handles observer registered with a given mask
     * @param mask defines the mask to be tested
     * @returns whether or not one observer registered with the given mask is handled
     **/
    hasSpecificMask(mask?: number): boolean;
}

}
declare module "babylonjs/Misc/observable.extensions" {
import { EventState } from "babylonjs/Misc/observable";
import { Observable } from "babylonjs/Misc/observable";
/**
 * Represent a list of observers registered to multiple Observables object.
 */
export class MultiObserver<T> {
    private _observers;
    private _observables;
    /**
     * Release associated resources
     */
    dispose(): void;
    /**
     * Raise a callback when one of the observable will notify
     * @param observables defines a list of observables to watch
     * @param callback defines the callback to call on notification
     * @param mask defines the mask used to filter notifications
     * @param scope defines the current scope used to restore the JS context
     * @returns the new MultiObserver
     */
    static Watch<T>(observables: Observable<T>[], callback: (eventData: T, eventState: EventState) => void, mask?: number, scope?: any): MultiObserver<T>;
}
module "babylonjs/Misc/observable" {
    interface Observable<T> {
        /**
         * Calling this will execute each callback, expecting it to be a promise or return a value.
         * If at any point in the chain one function fails, the promise will fail and the execution will not continue.
         * This is useful when a chain of events (sometimes async events) is needed to initialize a certain object
         * and it is crucial that all callbacks will be executed.
         * The order of the callbacks is kept, callbacks are not executed parallel.
         *
         * @param eventData The data to be sent to each callback
         * @param mask is used to filter observers defaults to -1
         * @param target defines the callback target (see EventState)
         * @param currentTarget defines he current object in the bubbling phase
         * @param userInfo defines any user info to send to observers
         * @returns {Promise<T>} will return a Promise than resolves when all callbacks executed successfully.
         */
        notifyObserversWithPromise(eventData: T, mask?: number, target?: any, currentTarget?: any, userInfo?: any): Promise<T>;
    }
}

}
declare module "babylonjs/Misc/observableCoroutine" {
import { AsyncCoroutine, CoroutineScheduler } from "babylonjs/Misc/coroutine";
module "babylonjs/Misc/observable" {
    interface Observable<T> {
        /**
         * Internal observable-based coroutine scheduler instance.
         */
        _coroutineScheduler?: CoroutineScheduler<void>;
        /**
         * Internal disposal method for observable-based coroutine scheduler instance.
         */
        _coroutineSchedulerDispose?: () => void;
        /**
         * Runs a coroutine asynchronously on this observable
         * @param coroutine the iterator resulting from having started the coroutine
         * @returns a promise which will be resolved when the coroutine finishes or rejected if the coroutine is cancelled
         */
        runCoroutineAsync(coroutine: AsyncCoroutine<void>): Promise<void>;
        /**
         * Cancels all coroutines currently running on this observable
         */
        cancelAllCoroutines(): void;
    }
}

}
declare module "babylonjs/Misc/perfCounter" {
/**
 * This class is used to track a performance counter which is number based.
 * The user has access to many properties which give statistics of different nature.
 *
 * The implementer can track two kinds of Performance Counter: time and count.
 * For time you can optionally call fetchNewFrame() to notify the start of a new frame to monitor, then call beginMonitoring() to start and endMonitoring() to record the lapsed time. endMonitoring takes a newFrame parameter for you to specify if the monitored time should be set for a new frame or accumulated to the current frame being monitored.
 * For count you first have to call fetchNewFrame() to notify the start of a new frame to monitor, then call addCount() how many time required to increment the count value you monitor.
 */
export class PerfCounter {
    /**
     * Gets or sets a global boolean to turn on and off all the counters
     */
    static Enabled: boolean;
    /**
     * Returns the smallest value ever
     */
    get min(): number;
    /**
     * Returns the biggest value ever
     */
    get max(): number;
    /**
     * Returns the average value since the performance counter is running
     */
    get average(): number;
    /**
     * Returns the average value of the last second the counter was monitored
     */
    get lastSecAverage(): number;
    /**
     * Returns the current value
     */
    get current(): number;
    /**
     * Gets the accumulated total
     */
    get total(): number;
    /**
     * Gets the total value count
     */
    get count(): number;
    /**
     * Creates a new counter
     */
    constructor();
    /**
     * Call this method to start monitoring a new frame.
     * This scenario is typically used when you accumulate monitoring time many times for a single frame, you call this method at the start of the frame, then beginMonitoring to start recording and endMonitoring(false) to accumulated the recorded time to the PerfCounter or addCount() to accumulate a monitored count.
     */
    fetchNewFrame(): void;
    /**
     * Call this method to monitor a count of something (e.g. mesh drawn in viewport count)
     * @param newCount the count value to add to the monitored count
     * @param fetchResult true when it's the last time in the frame you add to the counter and you wish to update the statistics properties (min/max/average), false if you only want to update statistics.
     */
    addCount(newCount: number, fetchResult: boolean): void;
    /**
     * Start monitoring this performance counter
     */
    beginMonitoring(): void;
    /**
     * Compute the time lapsed since the previous beginMonitoring() call.
     * @param newFrame true by default to fetch the result and monitor a new frame, if false the time monitored will be added to the current frame counter
     */
    endMonitoring(newFrame?: boolean): void;
    private _fetchResult;
    private _startMonitoringTime;
    private _min;
    private _max;
    private _average;
    private _current;
    private _totalValueCount;
    private _totalAccumulated;
    private _lastSecAverage;
    private _lastSecAccumulated;
    private _lastSecTime;
    private _lastSecValueCount;
}

}
declare module "babylonjs/Misc/performanceMonitor" {
/**
 * Performance monitor tracks rolling average frame-time and frame-time variance over a user defined sliding-window
 */
export class PerformanceMonitor {
    private _enabled;
    private _rollingFrameTime;
    private _lastFrameTimeMs;
    /**
     * constructor
     * @param frameSampleSize The number of samples required to saturate the sliding window
     */
    constructor(frameSampleSize?: number);
    /**
     * Samples current frame
     * @param timeMs A timestamp in milliseconds of the current frame to compare with other frames
     */
    sampleFrame(timeMs?: number): void;
    /**
     * Returns the average frame time in milliseconds over the sliding window (or the subset of frames sampled so far)
     */
    get averageFrameTime(): number;
    /**
     * Returns the variance frame time in milliseconds over the sliding window (or the subset of frames sampled so far)
     */
    get averageFrameTimeVariance(): number;
    /**
     * Returns the frame time of the most recent frame
     */
    get instantaneousFrameTime(): number;
    /**
     * Returns the average framerate in frames per second over the sliding window (or the subset of frames sampled so far)
     */
    get averageFPS(): number;
    /**
     * Returns the average framerate in frames per second using the most recent frame time
     */
    get instantaneousFPS(): number;
    /**
     * Returns true if enough samples have been taken to completely fill the sliding window
     */
    get isSaturated(): boolean;
    /**
     * Enables contributions to the sliding window sample set
     */
    enable(): void;
    /**
     * Disables contributions to the sliding window sample set
     * Samples will not be interpolated over the disabled period
     */
    disable(): void;
    /**
     * Returns true if sampling is enabled
     */
    get isEnabled(): boolean;
    /**
     * Resets performance monitor
     */
    reset(): void;
}
/**
 * RollingAverage
 *
 * Utility to efficiently compute the rolling average and variance over a sliding window of samples
 */
export class RollingAverage {
    /**
     * Current average
     */
    average: number;
    /**
     * Current variance
     */
    variance: number;
    protected _samples: Array<number>;
    protected _sampleCount: number;
    protected _pos: number;
    protected _m2: number;
    /**
     * constructor
     * @param length The number of samples required to saturate the sliding window
     */
    constructor(length: number);
    /**
     * Adds a sample to the sample set
     * @param v The sample value
     */
    add(v: number): void;
    /**
     * Returns previously added values or null if outside of history or outside the sliding window domain
     * @param i Index in history. For example, pass 0 for the most recent value and 1 for the value before that
     * @returns Value previously recorded with add() or null if outside of range
     */
    history(i: number): number;
    /**
     * Returns true if enough samples have been taken to completely fill the sliding window
     * @returns true if sample-set saturated
     */
    isSaturated(): boolean;
    /**
     * Resets the rolling average (equivalent to 0 samples taken so far)
     */
    reset(): void;
    /**
     * Wraps a value around the sample range boundaries
     * @param i Position in sample range, for example if the sample length is 5, and i is -3, then 2 will be returned.
     * @returns Wrapped position in sample range
     */
    protected _wrapPosition(i: number): number;
}

}
declare module "babylonjs/Misc/PerformanceViewer/dynamicFloat32Array" {
/**
 * A class acting as a dynamic float32array used in the performance viewer
 */
export class DynamicFloat32Array {
    private _view;
    private _itemLength;
    /**
     * Creates a new DynamicFloat32Array with the desired item capacity.
     * @param itemCapacity The initial item capacity you would like to set for the array.
     */
    constructor(itemCapacity: number);
    /**
     * The number of items currently in the array.
     */
    get itemLength(): number;
    /**
     * Gets value at index, NaN if no such index exists.
     * @param index the index to get the value at.
     * @returns the value at the index provided.
     */
    at(index: number): number;
    /**
     * Gets a view of the original array from start to end (exclusive of end).
     * @param start starting index.
     * @param end ending index.
     * @returns a subarray of the original array.
     */
    subarray(start: number, end: number): Float32Array;
    /**
     * Pushes items to the end of the array.
     * @param item The item to push into the array.
     */
    push(item: number): void;
    /**
     * Grows the array by the growth factor when necessary.
     */
    private _growArray;
}

}
declare module "babylonjs/Misc/PerformanceViewer/index" {
export * from "babylonjs/Misc/PerformanceViewer/performanceViewerCollector";
export * from "babylonjs/Misc/PerformanceViewer/performanceViewerCollectionStrategies";
export * from "babylonjs/Misc/PerformanceViewer/dynamicFloat32Array";
export * from "babylonjs/Misc/PerformanceViewer/performanceViewerSceneExtension";

}
declare module "babylonjs/Misc/PerformanceViewer/performanceViewerCollectionStrategies" {
import { Scene } from "babylonjs/scene";
/**
 * Defines the general structure of what is necessary for a collection strategy.
 */
export interface IPerfViewerCollectionStrategy {
    /**
     * The id of the strategy.
     */
    id: string;
    /**
     * Function which gets the data for the strategy.
     */
    getData: () => number;
    /**
     * Function which does any necessary cleanup. Called when performanceViewerCollector.dispose() is called.
     */
    dispose: () => void;
}
/**
 * Initializer callback for a strategy
 */
export type PerfStrategyInitialization = (scene: Scene) => IPerfViewerCollectionStrategy;
/**
 * Defines the predefined strategies used in the performance viewer.
 */
export class PerfCollectionStrategy {
    /**
     * Gets the initializer for the strategy used for collection of fps metrics
     * @returns the initializer for the fps strategy
     */
    static FpsStrategy(): PerfStrategyInitialization;
    /**
     * Gets the initializer for the strategy used for collection of thermal utilization metrics.
     * Needs the experimental pressure API.
     * @returns the initializer for the thermal utilization strategy
     */
    static ThermalStrategy(): PerfStrategyInitialization;
    /**
     * Gets the initializer for the strategy used for collection of power supply utilization metrics.
     * Needs the experimental pressure API.
     * @returns the initializer for the power supply utilization strategy
     */
    static PowerSupplyStrategy(): PerfStrategyInitialization;
    /**
     * Gets the initializer for the strategy used for collection of pressure metrics.
     * Needs the experimental pressure API.
     * @returns the initializer for the pressure strategy
     */
    static PressureStrategy(): PerfStrategyInitialization;
    private static _PressureStrategy;
    /**
     * Gets the initializer for the strategy used for collection of total meshes metrics.
     * @returns the initializer for the total meshes strategy
     */
    static TotalMeshesStrategy(): PerfStrategyInitialization;
    /**
     * Gets the initializer for the strategy used for collection of active meshes metrics.
     * @returns the initializer for the active meshes strategy
     */
    static ActiveMeshesStrategy(): PerfStrategyInitialization;
    /**
     * Gets the initializer for the strategy used for collection of active indices metrics.
     * @returns the initializer for the active indices strategy
     */
    static ActiveIndicesStrategy(): PerfStrategyInitialization;
    /**
     * Gets the initializer for the strategy used for collection of active faces metrics.
     * @returns the initializer for the active faces strategy
     */
    static ActiveFacesStrategy(): PerfStrategyInitialization;
    /**
     * Gets the initializer for the strategy used for collection of active bones metrics.
     * @returns the initializer for the active bones strategy
     */
    static ActiveBonesStrategy(): PerfStrategyInitialization;
    /**
     * Gets the initializer for the strategy used for collection of active particles metrics.
     * @returns the initializer for the active particles strategy
     */
    static ActiveParticlesStrategy(): PerfStrategyInitialization;
    /**
     * Gets the initializer for the strategy used for collection of draw calls metrics.
     * @returns the initializer for the draw calls strategy
     */
    static DrawCallsStrategy(): PerfStrategyInitialization;
    /**
     * Gets the initializer for the strategy used for collection of total lights metrics.
     * @returns the initializer for the total lights strategy
     */
    static TotalLightsStrategy(): PerfStrategyInitialization;
    /**
     * Gets the initializer for the strategy used for collection of total vertices metrics.
     * @returns the initializer for the total vertices strategy
     */
    static TotalVerticesStrategy(): PerfStrategyInitialization;
    /**
     * Gets the initializer for the strategy used for collection of total materials metrics.
     * @returns the initializer for the total materials strategy
     */
    static TotalMaterialsStrategy(): PerfStrategyInitialization;
    /**
     * Gets the initializer for the strategy used for collection of total textures metrics.
     * @returns the initializer for the total textures strategy
     */
    static TotalTexturesStrategy(): PerfStrategyInitialization;
    /**
     * Gets the initializer for the strategy used for collection of absolute fps metrics.
     * @returns the initializer for the absolute fps strategy
     */
    static AbsoluteFpsStrategy(): PerfStrategyInitialization;
    /**
     * Gets the initializer for the strategy used for collection of meshes selection time metrics.
     * @returns the initializer for the meshes selection time strategy
     */
    static MeshesSelectionStrategy(): PerfStrategyInitialization;
    /**
     * Gets the initializer for the strategy used for collection of render targets time metrics.
     * @returns the initializer for the render targets time strategy
     */
    static RenderTargetsStrategy(): PerfStrategyInitialization;
    /**
     * Gets the initializer for the strategy used for collection of particles time metrics.
     * @returns the initializer for the particles time strategy
     */
    static ParticlesStrategy(): PerfStrategyInitialization;
    /**
     * Gets the initializer for the strategy used for collection of sprites time metrics.
     * @returns the initializer for the sprites time strategy
     */
    static SpritesStrategy(): PerfStrategyInitialization;
    /**
     * Gets the initializer for the strategy used for collection of animations time metrics.
     * @returns the initializer for the animations time strategy
     */
    static AnimationsStrategy(): PerfStrategyInitialization;
    /**
     * Gets the initializer for the strategy used for collection of physics time metrics.
     * @returns the initializer for the physics time strategy
     */
    static PhysicsStrategy(): PerfStrategyInitialization;
    /**
     * Gets the initializer for the strategy used for collection of render time metrics.
     * @returns the initializer for the render time strategy
     */
    static RenderStrategy(): PerfStrategyInitialization;
    /**
     * Gets the initializer for the strategy used for collection of total frame time metrics.
     * @returns the initializer for the total frame time strategy
     */
    static FrameTotalStrategy(): PerfStrategyInitialization;
    /**
     * Gets the initializer for the strategy used for collection of inter-frame time metrics.
     * @returns the initializer for the inter-frame time strategy
     */
    static InterFrameStrategy(): PerfStrategyInitialization;
    /**
     * Gets the initializer for the strategy used for collection of gpu frame time metrics.
     * @returns the initializer for the gpu frame time strategy
     */
    static GpuFrameTimeStrategy(): PerfStrategyInitialization;
}

}
declare module "babylonjs/Misc/PerformanceViewer/performanceViewerCollector" {
import { Scene } from "babylonjs/scene";
import { IPerfCustomEvent, IPerfDatasets, IPerfMetadata } from "babylonjs/Misc/interfaces/iPerfViewer";
import { Observable } from "babylonjs/Misc/observable";
import { PerfStrategyInitialization } from "babylonjs/Misc/PerformanceViewer/performanceViewerCollectionStrategies";
/**
 * Callback strategy and optional category for data collection
 */
interface IPerformanceViewerStrategyParameter {
    /**
     * The strategy for collecting data. Available strategies are located on the PerfCollectionStrategy class
     */
    strategyCallback: PerfStrategyInitialization;
    /**
     * Category for displaying this strategy on the viewer. Can be undefined or an empty string, in which case the strategy will be displayed on top
     */
    category?: string;
    /**
     * Starts hidden
     */
    hidden?: boolean;
}
/**
 * The collector class handles the collection and storage of data into the appropriate array.
 * The collector also handles notifying any observers of any updates.
 */
export class PerformanceViewerCollector {
    private _scene;
    private _datasetMeta;
    private _strategies;
    private _startingTimestamp;
    private _hasLoadedData;
    private _isStarted;
    private readonly _customEventObservable;
    private readonly _eventRestoreSet;
    /**
     * Datastructure containing the collected datasets. Warning: you should not modify the values in here, data will be of the form [timestamp, numberOfPoints, value1, value2..., timestamp, etc...]
     */
    readonly datasets: IPerfDatasets;
    /**
     * An observable you can attach to get deltas in the dataset. Subscribing to this will increase memory consumption slightly, and may hurt performance due to increased garbage collection needed.
     * Updates of slices will be of the form [timestamp, numberOfPoints, value1, value2...].
     */
    readonly datasetObservable: Observable<number[]>;
    /**
     * An observable you can attach to get the most updated map of metadatas.
     */
    readonly metadataObservable: Observable<Map<string, IPerfMetadata>>;
    /**
     * The offset for when actual data values start appearing inside a slice.
     */
    static get SliceDataOffset(): number;
    /**
     * The offset for the value of the number of points inside a slice.
     */
    static get NumberOfPointsOffset(): number;
    /**
     * Handles the creation of a performance viewer collector.
     * @param _scene the scene to collect on.
     * @param _enabledStrategyCallbacks the list of data to collect with callbacks for initialization purposes.
     */
    constructor(_scene: Scene, _enabledStrategyCallbacks?: IPerformanceViewerStrategyParameter[]);
    /**
     * Registers a custom string event which will be callable via sendEvent. This method returns an event object which will contain the id of the event.
     * The user can set a value optionally, which will be used in the sendEvent method. If the value is set, we will record this value at the end of each frame,
     * if not we will increment our counter and record the value of the counter at the end of each frame. The value recorded is 0 if no sendEvent method is called, within a frame.
     * @param name The name of the event to register
     * @param forceUpdate if the code should force add an event, and replace the last one.
     * @param category the category for that event
     * @returns The event registered, used in sendEvent
     */
    registerEvent(name: string, forceUpdate?: boolean, category?: string): IPerfCustomEvent | undefined;
    /**
     * Lets the perf collector handle an event, occurences or event value depending on if the event.value params is set.
     * @param event the event to handle an occurence for
     */
    sendEvent(event: IPerfCustomEvent): void;
    /**
     * This event restores all custom string events if necessary.
     */
    private _restoreStringEvents;
    /**
     * This method adds additional collection strategies for data collection purposes.
     * @param strategyCallbacks the list of data to collect with callbacks.
     */
    addCollectionStrategies(...strategyCallbacks: IPerformanceViewerStrategyParameter[]): void;
    /**
     * Gets a 6 character hexcode representing the colour from a passed in string.
     * @param id the string to get a hex code for.
     * @returns a hexcode hashed from the id.
     */
    private _getHexColorFromId;
    /**
     * Collects data for every dataset by using the appropriate strategy. This is called every frame.
     * This method will then notify all observers with the latest slice.
     */
    private _collectDataAtFrame;
    /**
     * Collects and then sends the latest slice to any observers by using the appropriate strategy when the user wants.
     * The slice will be of the form [timestamp, numberOfPoints, value1, value2...]
     * This method does not add onto the collected data accessible via the datasets variable.
     */
    getCurrentSlice(): void;
    /**
     * Updates a property for a dataset's metadata with the value provided.
     * @param id the id of the dataset which needs its metadata updated.
     * @param prop the property to update.
     * @param value the value to update the property with.
     */
    updateMetadata<T extends keyof IPerfMetadata>(id: string, prop: T, value: IPerfMetadata[T]): void;
    /**
     * Completely clear, data, ids, and strategies saved to this performance collector.
     * @param preserveStringEventsRestore if it should preserve the string events, by default will clear string events registered when called.
     */
    clear(preserveStringEventsRestore?: boolean): void;
    /**
     * Accessor which lets the caller know if the performance collector has data loaded from a file or not!
     * Call clear() to reset this value.
     * @returns true if the data is loaded from a file, false otherwise.
     */
    get hasLoadedData(): boolean;
    /**
     * Given a string containing file data, this function parses the file data into the datasets object.
     * It returns a boolean to indicate if this object was successfully loaded with the data.
     * @param data string content representing the file data.
     * @param keepDatasetMeta if it should use reuse the existing dataset metadata
     * @returns true if the data was successfully loaded, false otherwise.
     */
    loadFromFileData(data: string, keepDatasetMeta?: boolean): boolean;
    /**
     * Exports the datasets inside of the collector to a csv.
     */
    exportDataToCsv(): void;
    /**
     * Starts the realtime collection of data.
     * @param shouldPreserve optional boolean param, if set will preserve the dataset between calls of start.
     */
    start(shouldPreserve?: boolean): void;
    /**
     * Stops the collection of data.
     */
    stop(): void;
    /**
     * Returns if the perf collector has been started or not.
     */
    get isStarted(): boolean;
    /**
     * Disposes of the object
     */
    dispose(): void;
}
export {};

}
declare module "babylonjs/Misc/PerformanceViewer/performanceViewerSceneExtension" {
export {};

}
declare module "babylonjs/Misc/pivotTools" {
import { TransformNode } from "babylonjs/Meshes/transformNode";
/**
 * Class containing a set of static utilities functions for managing Pivots
 * @internal
 */
export class PivotTools {
    private static _PivotCached;
    private static _OldPivotPoint;
    private static _PivotTranslation;
    private static _PivotTmpVector;
    private static _PivotPostMultiplyPivotMatrix;
    /**
     * @internal
     */
    static _RemoveAndStorePivotPoint(mesh: TransformNode): void;
    /**
     * @internal
     */
    static _RestorePivotPoint(mesh: TransformNode): void;
}

}
declare module "babylonjs/Misc/precisionDate" {
/**
 * Class containing a set of static utilities functions for precision date
 */
export class PrecisionDate {
    /**
     * Gets either window.performance.now() if supported or Date.now() else
     */
    static get Now(): number;
}

}
declare module "babylonjs/Misc/pressureObserverWrapper" {
import { Observable } from "babylonjs/Misc/observable";
/**
 * A wrapper for the experimental pressure api which allows a callback to be called whenever certain thresholds are met.
 */
export class PressureObserverWrapper {
    private _observer;
    private _currentState;
    /**
     * An event triggered when the cpu usage/speed meets certain thresholds.
     * Note: pressure is an experimental API.
     */
    onPressureChanged: Observable<PressureRecord[]>;
    /**
     * A pressure observer will call this callback, whenever these thresholds are met.
     * @param options An object containing the thresholds used to decide what value to to return for each update property (average of start and end of a threshold boundary).
     */
    constructor(options?: PressureObserverOptions);
    /**
     * Returns true if PressureObserver is available for use, false otherwise.
     */
    static get IsAvailable(): boolean;
    /**
     * Method that must be called to begin observing changes, and triggering callbacks.
     * @param source defines the source to observe
     */
    observe(source: PressureSource): void;
    /**
     * Method that must be called to stop observing changes and triggering callbacks (cleanup function).
     * @param source defines the source to unobserve
     */
    unobserve(source: PressureSource): void;
    /**
     * Release the associated resources.
     */
    dispose(): void;
}

}
declare module "babylonjs/Misc/reflector" {
import { Scene } from "babylonjs/scene";
/**
 * Class used to connect with the reflector zone of the sandbox via the reflector bridge
 * @since 5.0.0
 */
export class Reflector {
    private static readonly _SERVER_PREFIX;
    private _scene;
    private _webSocket;
    /**
     * Constructs a reflector object.
     * @param scene The scene to use
     * @param hostname The hostname of the reflector bridge
     * @param port The port of the reflector bridge
     */
    constructor(scene: Scene, hostname: string, port: number);
    /**
     * Closes the reflector connection
     */
    close(): void;
    private _handleServerMessage;
    private _handleClientMessage;
}

}
declare module "babylonjs/Misc/retryStrategy" {
import { WebRequest } from "babylonjs/Misc/webRequest";
/**
 * Class used to define a retry strategy when error happens while loading assets
 */
export class RetryStrategy {
    /**
     * Function used to defines an exponential back off strategy
     * @param maxRetries defines the maximum number of retries (3 by default)
     * @param baseInterval defines the interval between retries
     * @returns the strategy function to use
     */
    static ExponentialBackoff(maxRetries?: number, baseInterval?: number): (url: string, request: WebRequest, retryIndex: number) => number;
}

}
declare module "babylonjs/Misc/rgbdTextureTools" {
import "babylonjs/Shaders/rgbdDecode.fragment";
import "babylonjs/Engines/Extensions/engine.renderTarget";
import { Texture } from "babylonjs/Materials/Textures/texture";
import { InternalTexture } from "babylonjs/Materials/Textures/internalTexture";
import { Scene } from "babylonjs/scene";
/**
 * Class used to host RGBD texture specific utilities
 */
export class RGBDTextureTools {
    /**
     * Expand the RGBD Texture from RGBD to Half Float if possible.
     * @param texture the texture to expand.
     */
    static ExpandRGBDTexture(texture: Texture): void;
    /**
     * Encode the texture to RGBD if possible.
     * @param internalTexture the texture to encode
     * @param scene the scene hosting the texture
     * @param outputTextureType type of the texture in which the encoding is performed
     * @returns a promise with the internalTexture having its texture replaced by the result of the processing
     */
    static EncodeTextureToRGBD(internalTexture: InternalTexture, scene: Scene, outputTextureType?: number): Promise<InternalTexture>;
}
export {};

}
declare module "babylonjs/Misc/sceneOptimizer" {
import { Scene, IDisposable } from "babylonjs/scene";
import { Observable } from "babylonjs/Misc/observable";
/**
 * Defines the root class used to create scene optimization to use with SceneOptimizer
 * @description More details at https://doc.babylonjs.com/features/featuresDeepDive/scene/sceneOptimizer
 */
export class SceneOptimization {
    /**
     * Defines the priority of this optimization (0 by default which means first in the list)
     */
    priority: number;
    /**
     * Gets a string describing the action executed by the current optimization
     * @returns description string
     */
    getDescription(): string;
    /**
     * This function will be called by the SceneOptimizer when its priority is reached in order to apply the change required by the current optimization
     * @param scene defines the current scene where to apply this optimization
     * @param optimizer defines the current optimizer
     * @returns true if everything that can be done was applied
     */
    apply(scene: Scene, optimizer: SceneOptimizer): boolean;
    /**
     * Creates the SceneOptimization object
     * @param priority defines the priority of this optimization (0 by default which means first in the list)
     */
    constructor(
    /**
     * Defines the priority of this optimization (0 by default which means first in the list)
     */
    priority?: number);
}
/**
 * Defines an optimization used to reduce the size of render target textures
 * @description More details at https://doc.babylonjs.com/features/featuresDeepDive/scene/sceneOptimizer
 */
export class TextureOptimization extends SceneOptimization {
    /**
     * Defines the priority of this optimization (0 by default which means first in the list)
     */
    priority: number;
    /**
     * Defines the maximum sized allowed for textures (1024 is the default value). If a texture is bigger, it will be scaled down using a factor defined by the step parameter
     */
    maximumSize: number;
    /**
     * Defines the factor (0.5 by default) used to scale down textures bigger than maximum sized allowed.
     */
    step: number;
    /**
     * Gets a string describing the action executed by the current optimization
     * @returns description string
     */
    getDescription(): string;
    /**
     * Creates the TextureOptimization object
     * @param priority defines the priority of this optimization (0 by default which means first in the list)
     * @param maximumSize defines the maximum sized allowed for textures (1024 is the default value). If a texture is bigger, it will be scaled down using a factor defined by the step parameter
     * @param step defines the factor (0.5 by default) used to scale down textures bigger than maximum sized allowed.
     */
    constructor(
    /**
     * Defines the priority of this optimization (0 by default which means first in the list)
     */
    priority?: number, 
    /**
     * Defines the maximum sized allowed for textures (1024 is the default value). If a texture is bigger, it will be scaled down using a factor defined by the step parameter
     */
    maximumSize?: number, 
    /**
     * Defines the factor (0.5 by default) used to scale down textures bigger than maximum sized allowed.
     */
    step?: number);
    /**
     * This function will be called by the SceneOptimizer when its priority is reached in order to apply the change required by the current optimization
     * @param scene defines the current scene where to apply this optimization
     * @param optimizer defines the current optimizer
     * @returns true if everything that can be done was applied
     */
    apply(scene: Scene, optimizer: SceneOptimizer): boolean;
}
/**
 * Defines an optimization used to increase or decrease the rendering resolution
 * @description More details at https://doc.babylonjs.com/features/featuresDeepDive/scene/sceneOptimizer
 */
export class HardwareScalingOptimization extends SceneOptimization {
    /**
     * Defines the priority of this optimization (0 by default which means first in the list)
     */
    priority: number;
    /**
     * Defines the maximum scale to use (2 by default)
     */
    maximumScale: number;
    /**
     * Defines the step to use between two passes (0.5 by default)
     */
    step: number;
    private _currentScale;
    private _directionOffset;
    /**
     * Gets a string describing the action executed by the current optimization
     * @returns description string
     */
    getDescription(): string;
    /**
     * Creates the HardwareScalingOptimization object
     * @param priority defines the priority of this optimization (0 by default which means first in the list)
     * @param maximumScale defines the maximum scale to use (2 by default)
     * @param step defines the step to use between two passes (0.5 by default)
     */
    constructor(
    /**
     * Defines the priority of this optimization (0 by default which means first in the list)
     */
    priority?: number, 
    /**
     * Defines the maximum scale to use (2 by default)
     */
    maximumScale?: number, 
    /**
     * Defines the step to use between two passes (0.5 by default)
     */
    step?: number);
    /**
     * This function will be called by the SceneOptimizer when its priority is reached in order to apply the change required by the current optimization
     * @param scene defines the current scene where to apply this optimization
     * @param optimizer defines the current optimizer
     * @returns true if everything that can be done was applied
     */
    apply(scene: Scene, optimizer: SceneOptimizer): boolean;
}
/**
 * Defines an optimization used to remove shadows
 * @description More details at https://doc.babylonjs.com/features/featuresDeepDive/scene/sceneOptimizer
 */
export class ShadowsOptimization extends SceneOptimization {
    /**
     * Gets a string describing the action executed by the current optimization
     * @returns description string
     */
    getDescription(): string;
    /**
     * This function will be called by the SceneOptimizer when its priority is reached in order to apply the change required by the current optimization
     * @param scene defines the current scene where to apply this optimization
     * @param optimizer defines the current optimizer
     * @returns true if everything that can be done was applied
     */
    apply(scene: Scene, optimizer: SceneOptimizer): boolean;
}
/**
 * Defines an optimization used to turn post-processes off
 * @description More details at https://doc.babylonjs.com/features/featuresDeepDive/scene/sceneOptimizer
 */
export class PostProcessesOptimization extends SceneOptimization {
    /**
     * Gets a string describing the action executed by the current optimization
     * @returns description string
     */
    getDescription(): string;
    /**
     * This function will be called by the SceneOptimizer when its priority is reached in order to apply the change required by the current optimization
     * @param scene defines the current scene where to apply this optimization
     * @param optimizer defines the current optimizer
     * @returns true if everything that can be done was applied
     */
    apply(scene: Scene, optimizer: SceneOptimizer): boolean;
}
/**
 * Defines an optimization used to turn lens flares off
 * @description More details at https://doc.babylonjs.com/features/featuresDeepDive/scene/sceneOptimizer
 */
export class LensFlaresOptimization extends SceneOptimization {
    /**
     * Gets a string describing the action executed by the current optimization
     * @returns description string
     */
    getDescription(): string;
    /**
     * This function will be called by the SceneOptimizer when its priority is reached in order to apply the change required by the current optimization
     * @param scene defines the current scene where to apply this optimization
     * @param optimizer defines the current optimizer
     * @returns true if everything that can be done was applied
     */
    apply(scene: Scene, optimizer: SceneOptimizer): boolean;
}
/**
 * Defines an optimization based on user defined callback.
 * @description More details at https://doc.babylonjs.com/features/featuresDeepDive/scene/sceneOptimizer
 */
export class CustomOptimization extends SceneOptimization {
    /**
     * Callback called to apply the custom optimization.
     */
    onApply: (scene: Scene, optimizer: SceneOptimizer) => boolean;
    /**
     * Callback called to get custom description
     */
    onGetDescription: () => string;
    /**
     * Gets a string describing the action executed by the current optimization
     * @returns description string
     */
    getDescription(): string;
    /**
     * This function will be called by the SceneOptimizer when its priority is reached in order to apply the change required by the current optimization
     * @param scene defines the current scene where to apply this optimization
     * @param optimizer defines the current optimizer
     * @returns true if everything that can be done was applied
     */
    apply(scene: Scene, optimizer: SceneOptimizer): boolean;
}
/**
 * Defines an optimization used to turn particles off
 * @description More details at https://doc.babylonjs.com/features/featuresDeepDive/scene/sceneOptimizer
 */
export class ParticlesOptimization extends SceneOptimization {
    /**
     * Gets a string describing the action executed by the current optimization
     * @returns description string
     */
    getDescription(): string;
    /**
     * This function will be called by the SceneOptimizer when its priority is reached in order to apply the change required by the current optimization
     * @param scene defines the current scene where to apply this optimization
     * @param optimizer defines the current optimizer
     * @returns true if everything that can be done was applied
     */
    apply(scene: Scene, optimizer: SceneOptimizer): boolean;
}
/**
 * Defines an optimization used to turn render targets off
 * @description More details at https://doc.babylonjs.com/features/featuresDeepDive/scene/sceneOptimizer
 */
export class RenderTargetsOptimization extends SceneOptimization {
    /**
     * Gets a string describing the action executed by the current optimization
     * @returns description string
     */
    getDescription(): string;
    /**
     * This function will be called by the SceneOptimizer when its priority is reached in order to apply the change required by the current optimization
     * @param scene defines the current scene where to apply this optimization
     * @param optimizer defines the current optimizer
     * @returns true if everything that can be done was applied
     */
    apply(scene: Scene, optimizer: SceneOptimizer): boolean;
}
/**
 * Defines an optimization used to merge meshes with compatible materials
 * @description More details at https://doc.babylonjs.com/features/featuresDeepDive/scene/sceneOptimizer
 */
export class MergeMeshesOptimization extends SceneOptimization {
    private static _UpdateSelectionTree;
    /**
     * Gets or sets a boolean which defines if optimization octree has to be updated
     */
    static get UpdateSelectionTree(): boolean;
    /**
     * Gets or sets a boolean which defines if optimization octree has to be updated
     */
    static set UpdateSelectionTree(value: boolean);
    /**
     * Gets a string describing the action executed by the current optimization
     * @returns description string
     */
    getDescription(): string;
    private _canBeMerged;
    /**
     * This function will be called by the SceneOptimizer when its priority is reached in order to apply the change required by the current optimization
     * @param scene defines the current scene where to apply this optimization
     * @param optimizer defines the current optimizer
     * @param updateSelectionTree defines that the selection octree has to be updated (false by default)
     * @returns true if everything that can be done was applied
     */
    apply(scene: Scene, optimizer: SceneOptimizer, updateSelectionTree?: boolean): boolean;
}
/**
 * Defines a list of options used by SceneOptimizer
 * @description More details at https://doc.babylonjs.com/features/featuresDeepDive/scene/sceneOptimizer
 */
export class SceneOptimizerOptions {
    /**
     * Defines the target frame rate to reach (60 by default)
     */
    targetFrameRate: number;
    /**
     * Defines the interval between two checks (2000ms by default)
     */
    trackerDuration: number;
    /**
     * Gets the list of optimizations to apply
     */
    optimizations: SceneOptimization[];
    /**
     * Creates a new list of options used by SceneOptimizer
     * @param targetFrameRate defines the target frame rate to reach (60 by default)
     * @param trackerDuration defines the interval between two checks (2000ms by default)
     */
    constructor(
    /**
     * Defines the target frame rate to reach (60 by default)
     */
    targetFrameRate?: number, 
    /**
     * Defines the interval between two checks (2000ms by default)
     */
    trackerDuration?: number);
    /**
     * Add a new optimization
     * @param optimization defines the SceneOptimization to add to the list of active optimizations
     * @returns the current SceneOptimizerOptions
     */
    addOptimization(optimization: SceneOptimization): SceneOptimizerOptions;
    /**
     * Add a new custom optimization
     * @param onApply defines the callback called to apply the custom optimization (true if everything that can be done was applied)
     * @param onGetDescription defines the callback called to get the description attached with the optimization.
     * @param priority defines the priority of this optimization (0 by default which means first in the list)
     * @returns the current SceneOptimizerOptions
     */
    addCustomOptimization(onApply: (scene: Scene, optimizer: SceneOptimizer) => boolean, onGetDescription: () => string, priority?: number): SceneOptimizerOptions;
    /**
     * Creates a list of pre-defined optimizations aimed to reduce the visual impact on the scene
     * @param targetFrameRate defines the target frame rate (60 by default)
     * @returns a SceneOptimizerOptions object
     */
    static LowDegradationAllowed(targetFrameRate?: number): SceneOptimizerOptions;
    /**
     * Creates a list of pre-defined optimizations aimed to have a moderate impact on the scene visual
     * @param targetFrameRate defines the target frame rate (60 by default)
     * @returns a SceneOptimizerOptions object
     */
    static ModerateDegradationAllowed(targetFrameRate?: number): SceneOptimizerOptions;
    /**
     * Creates a list of pre-defined optimizations aimed to have a big impact on the scene visual
     * @param targetFrameRate defines the target frame rate (60 by default)
     * @returns a SceneOptimizerOptions object
     */
    static HighDegradationAllowed(targetFrameRate?: number): SceneOptimizerOptions;
}
/**
 * Class used to run optimizations in order to reach a target frame rate
 * @description More details at https://doc.babylonjs.com/features/featuresDeepDive/scene/sceneOptimizer
 */
export class SceneOptimizer implements IDisposable {
    private _isRunning;
    private _options;
    private _scene;
    private _currentPriorityLevel;
    private _targetFrameRate;
    private _trackerDuration;
    private _currentFrameRate;
    private _sceneDisposeObserver;
    private _improvementMode;
    /**
     * Defines an observable called when the optimizer reaches the target frame rate
     */
    onSuccessObservable: Observable<SceneOptimizer>;
    /**
     * Defines an observable called when the optimizer enables an optimization
     */
    onNewOptimizationAppliedObservable: Observable<SceneOptimization>;
    /**
     * Defines an observable called when the optimizer is not able to reach the target frame rate
     */
    onFailureObservable: Observable<SceneOptimizer>;
    /**
     * Gets or sets a boolean indicating if the optimizer is in improvement mode
     */
    get isInImprovementMode(): boolean;
    set isInImprovementMode(value: boolean);
    /**
     * Gets the current priority level (0 at start)
     */
    get currentPriorityLevel(): number;
    /**
     * Gets the current frame rate checked by the SceneOptimizer
     */
    get currentFrameRate(): number;
    /**
     * Gets or sets the current target frame rate (60 by default)
     */
    get targetFrameRate(): number;
    /**
     * Gets or sets the current target frame rate (60 by default)
     */
    set targetFrameRate(value: number);
    /**
     * Gets or sets the current interval between two checks (every 2000ms by default)
     */
    get trackerDuration(): number;
    /**
     * Gets or sets the current interval between two checks (every 2000ms by default)
     */
    set trackerDuration(value: number);
    /**
     * Gets the list of active optimizations
     */
    get optimizations(): SceneOptimization[];
    /**
     * Creates a new SceneOptimizer
     * @param scene defines the scene to work on
     * @param options defines the options to use with the SceneOptimizer
     * @param autoGeneratePriorities defines if priorities must be generated and not read from SceneOptimization property (true by default)
     * @param improvementMode defines if the scene optimizer must run the maximum optimization while staying over a target frame instead of trying to reach the target framerate (false by default)
     */
    constructor(scene: Scene, options?: SceneOptimizerOptions, autoGeneratePriorities?: boolean, improvementMode?: boolean);
    /**
     * Stops the current optimizer
     */
    stop(): void;
    /**
     * Reset the optimizer to initial step (current priority level = 0)
     */
    reset(): void;
    /**
     * Start the optimizer. By default it will try to reach a specific framerate
     * but if the optimizer is set with improvementMode === true then it will run all optimization while frame rate is above the target frame rate
     */
    start(): void;
    private _checkCurrentState;
    /**
     * Release all resources
     */
    dispose(): void;
    /**
     * Helper function to create a SceneOptimizer with one single line of code
     * @param scene defines the scene to work on
     * @param options defines the options to use with the SceneOptimizer
     * @param onSuccess defines a callback to call on success
     * @param onFailure defines a callback to call on failure
     * @returns the new SceneOptimizer object
     */
    static OptimizeAsync(scene: Scene, options?: SceneOptimizerOptions, onSuccess?: () => void, onFailure?: () => void): SceneOptimizer;
}

}
declare module "babylonjs/Misc/sceneRecorder" {
import { Scene } from "babylonjs/scene";
/**
 * Class used to record delta files between 2 scene states
 */
export class SceneRecorder {
    private _trackedScene;
    private _savedJSON;
    /**
     * Track a given scene. This means the current scene state will be considered the original state
     * @param scene defines the scene to track
     */
    track(scene: Scene): void;
    /**
     * Get the delta between current state and original state
     * @returns a any containing the delta
     */
    getDelta(): any;
    private _compareArray;
    private _compareObjects;
    private _compareCollections;
    private static GetShadowGeneratorById;
    /**
     * Apply a given delta to a given scene
     * @param deltaJSON defines the JSON containing the delta
     * @param scene defines the scene to apply the delta to
     */
    static ApplyDelta(deltaJSON: any | string, scene: Scene): void;
    private static _ApplyPropertiesToEntity;
    private static _ApplyDeltaForEntity;
}

}
declare module "babylonjs/Misc/sceneSerializer" {
import { Scene } from "babylonjs/scene";
/**
 * Class used to serialize a scene into a string
 */
export class SceneSerializer {
    /**
     * Clear cache used by a previous serialization
     */
    static ClearCache(): void;
    /**
     * Serialize a scene into a JSON compatible object
     * Note that if the current engine does not support synchronous texture reading (like WebGPU), you should use SerializeAsync instead
     * as else you may not retrieve the proper base64 encoded texture data (when using the Texture.ForceSerializeBuffers flag)
     * @param scene defines the scene to serialize
     * @returns a JSON compatible object
     */
    static Serialize(scene: Scene): any;
    private static _Serialize;
    /**
     * Serialize a scene into a JSON compatible object
     * @param scene defines the scene to serialize
     * @returns a JSON promise compatible object
     */
    static SerializeAsync(scene: Scene): Promise<any>;
    private static _CollectPromises;
    /**
     * Serialize a mesh into a JSON compatible object
     * @param toSerialize defines the mesh to serialize
     * @param withParents defines if parents must be serialized as well
     * @param withChildren defines if children must be serialized as well
     * @returns a JSON compatible object
     */
    static SerializeMesh(toSerialize: any, withParents?: boolean, withChildren?: boolean): any;
}

}
declare module "babylonjs/Misc/screenshotTools" {
import { Camera } from "babylonjs/Cameras/camera";
import { IScreenshotSize } from "babylonjs/Misc/interfaces/screenshotSize";
import { Engine } from "babylonjs/Engines/engine";
/**
 * Captures a screenshot of the current rendering
 * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/renderToPNG
 * @param engine defines the rendering engine
 * @param camera defines the source camera
 * @param size This parameter can be set to a single number or to an object with the
 * following (optional) properties: precision, width, height. If a single number is passed,
 * it will be used for both width and height. If an object is passed, the screenshot size
 * will be derived from the parameters. The precision property is a multiplier allowing
 * rendering at a higher or lower resolution
 * @param successCallback defines the callback receives a single parameter which contains the
 * screenshot as a string of base64-encoded characters. This string can be assigned to the
 * src parameter of an <img> to display it
 * @param mimeType defines the MIME type of the screenshot image (default: image/png).
 * Check your browser for supported MIME types
 * @param forceDownload force the system to download the image even if a successCallback is provided
 */
export function CreateScreenshot(engine: Engine, camera: Camera, size: IScreenshotSize | number, successCallback?: (data: string) => void, mimeType?: string, forceDownload?: boolean): void;
/**
 * Captures a screenshot of the current rendering
 * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/renderToPNG
 * @param engine defines the rendering engine
 * @param camera defines the source camera
 * @param size This parameter can be set to a single number or to an object with the
 * following (optional) properties: precision, width, height. If a single number is passed,
 * it will be used for both width and height. If an object is passed, the screenshot size
 * will be derived from the parameters. The precision property is a multiplier allowing
 * rendering at a higher or lower resolution
 * @param mimeType defines the MIME type of the screenshot image (default: image/png).
 * Check your browser for supported MIME types
 * @returns screenshot as a string of base64-encoded characters. This string can be assigned
 * to the src parameter of an <img> to display it
 */
export function CreateScreenshotAsync(engine: Engine, camera: Camera, size: IScreenshotSize | number, mimeType?: string): Promise<string>;
/**
 * Captures a screenshot of the current rendering for a specific size. This will render the entire canvas but will generate a blink (due to canvas resize)
 * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/renderToPNG
 * @param engine defines the rendering engine
 * @param camera defines the source camera
 * @param width defines the expected width
 * @param height defines the expected height
 * @param mimeType defines the MIME type of the screenshot image (default: image/png).
 * Check your browser for supported MIME types
 * @returns screenshot as a string of base64-encoded characters. This string can be assigned
 * to the src parameter of an <img> to display it
 */
export function CreateScreenshotWithResizeAsync(engine: Engine, camera: Camera, width: number, height: number, mimeType?: string): Promise<void>;
/**
 * Generates an image screenshot from the specified camera.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/renderToPNG
 * @param engine The engine to use for rendering
 * @param camera The camera to use for rendering
 * @param size This parameter can be set to a single number or to an object with the
 * following (optional) properties: precision, width, height. If a single number is passed,
 * it will be used for both width and height. If an object is passed, the screenshot size
 * will be derived from the parameters. The precision property is a multiplier allowing
 * rendering at a higher or lower resolution
 * @param successCallback The callback receives a single parameter which contains the
 * screenshot as a string of base64-encoded characters. This string can be assigned to the
 * src parameter of an <img> to display it
 * @param mimeType The MIME type of the screenshot image (default: image/png).
 * Check your browser for supported MIME types
 * @param samples Texture samples (default: 1)
 * @param antialiasing Whether antialiasing should be turned on or not (default: false)
 * @param fileName A name for for the downloaded file.
 * @param renderSprites Whether the sprites should be rendered or not (default: false)
 * @param enableStencilBuffer Whether the stencil buffer should be enabled or not (default: false)
 * @param useLayerMask if the camera's layer mask should be used to filter what should be rendered (default: true)
 */
export function CreateScreenshotUsingRenderTarget(engine: Engine, camera: Camera, size: IScreenshotSize | number, successCallback?: (data: string) => void, mimeType?: string, samples?: number, antialiasing?: boolean, fileName?: string, renderSprites?: boolean, enableStencilBuffer?: boolean, useLayerMask?: boolean): void;
/**
 * Generates an image screenshot from the specified camera.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/renderToPNG
 * @param engine The engine to use for rendering
 * @param camera The camera to use for rendering
 * @param size This parameter can be set to a single number or to an object with the
 * following (optional) properties: precision, width, height. If a single number is passed,
 * it will be used for both width and height. If an object is passed, the screenshot size
 * will be derived from the parameters. The precision property is a multiplier allowing
 * rendering at a higher or lower resolution
 * @param mimeType The MIME type of the screenshot image (default: image/png).
 * Check your browser for supported MIME types
 * @param samples Texture samples (default: 1)
 * @param antialiasing Whether antialiasing should be turned on or not (default: false)
 * @param fileName A name for for the downloaded file.
 * @param renderSprites Whether the sprites should be rendered or not (default: false)
 * @param enableStencilBuffer Whether the stencil buffer should be enabled or not (default: false)
 * @param useLayerMask if the camera's layer mask should be used to filter what should be rendered (default: true)
 * @returns screenshot as a string of base64-encoded characters. This string can be assigned
 * to the src parameter of an <img> to display it
 */
export function CreateScreenshotUsingRenderTargetAsync(engine: Engine, camera: Camera, size: IScreenshotSize | number, mimeType?: string, samples?: number, antialiasing?: boolean, fileName?: string, renderSprites?: boolean, enableStencilBuffer?: boolean, useLayerMask?: boolean): Promise<string>;
/**
 * Class containing a set of static utilities functions for screenshots
 */
export const ScreenshotTools: {
    /**
     * Captures a screenshot of the current rendering
     * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/renderToPNG
     * @param engine defines the rendering engine
     * @param camera defines the source camera
     * @param size This parameter can be set to a single number or to an object with the
     * following (optional) properties: precision, width, height. If a single number is passed,
     * it will be used for both width and height. If an object is passed, the screenshot size
     * will be derived from the parameters. The precision property is a multiplier allowing
     * rendering at a higher or lower resolution
     * @param successCallback defines the callback receives a single parameter which contains the
     * screenshot as a string of base64-encoded characters. This string can be assigned to the
     * src parameter of an <img> to display it
     * @param mimeType defines the MIME type of the screenshot image (default: image/png).
     * Check your browser for supported MIME types
     * @param forceDownload force the system to download the image even if a successCallback is provided
     */
    CreateScreenshot: typeof CreateScreenshot;
    /**
     * Captures a screenshot of the current rendering
     * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/renderToPNG
     * @param engine defines the rendering engine
     * @param camera defines the source camera
     * @param size This parameter can be set to a single number or to an object with the
     * following (optional) properties: precision, width, height. If a single number is passed,
     * it will be used for both width and height. If an object is passed, the screenshot size
     * will be derived from the parameters. The precision property is a multiplier allowing
     * rendering at a higher or lower resolution
     * @param mimeType defines the MIME type of the screenshot image (default: image/png).
     * Check your browser for supported MIME types
     * @returns screenshot as a string of base64-encoded characters. This string can be assigned
     * to the src parameter of an <img> to display it
     */
    CreateScreenshotAsync: typeof CreateScreenshotAsync;
    /**
     * Captures a screenshot of the current rendering for a specific size. This will render the entire canvas but will generate a blink (due to canvas resize)
     * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/renderToPNG
     * @param engine defines the rendering engine
     * @param camera defines the source camera
     * @param width defines the expected width
     * @param height defines the expected height
     * @param mimeType defines the MIME type of the screenshot image (default: image/png).
     * Check your browser for supported MIME types
     * @returns screenshot as a string of base64-encoded characters. This string can be assigned
     * to the src parameter of an <img> to display it
     */
    CreateScreenshotWithResizeAsync: typeof CreateScreenshotWithResizeAsync;
    /**
     * Generates an image screenshot from the specified camera.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/renderToPNG
     * @param engine The engine to use for rendering
     * @param camera The camera to use for rendering
     * @param size This parameter can be set to a single number or to an object with the
     * following (optional) properties: precision, width, height. If a single number is passed,
     * it will be used for both width and height. If an object is passed, the screenshot size
     * will be derived from the parameters. The precision property is a multiplier allowing
     * rendering at a higher or lower resolution
     * @param successCallback The callback receives a single parameter which contains the
     * screenshot as a string of base64-encoded characters. This string can be assigned to the
     * src parameter of an <img> to display it
     * @param mimeType The MIME type of the screenshot image (default: image/png).
     * Check your browser for supported MIME types
     * @param samples Texture samples (default: 1)
     * @param antialiasing Whether antialiasing should be turned on or not (default: false)
     * @param fileName A name for for the downloaded file.
     * @param renderSprites Whether the sprites should be rendered or not (default: false)
     * @param enableStencilBuffer Whether the stencil buffer should be enabled or not (default: false)
     */
    CreateScreenshotUsingRenderTarget: typeof CreateScreenshotUsingRenderTarget;
    /**
     * Generates an image screenshot from the specified camera.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/renderToPNG
     * @param engine The engine to use for rendering
     * @param camera The camera to use for rendering
     * @param size This parameter can be set to a single number or to an object with the
     * following (optional) properties: precision, width, height. If a single number is passed,
     * it will be used for both width and height. If an object is passed, the screenshot size
     * will be derived from the parameters. The precision property is a multiplier allowing
     * rendering at a higher or lower resolution
     * @param mimeType The MIME type of the screenshot image (default: image/png).
     * Check your browser for supported MIME types
     * @param samples Texture samples (default: 1)
     * @param antialiasing Whether antialiasing should be turned on or not (default: false)
     * @param fileName A name for for the downloaded file.
     * @param renderSprites Whether the sprites should be rendered or not (default: false)
     * @returns screenshot as a string of base64-encoded characters. This string can be assigned
     * to the src parameter of an <img> to display it
     */
    CreateScreenshotUsingRenderTargetAsync: typeof CreateScreenshotUsingRenderTargetAsync;
};
export {};

}
declare module "babylonjs/Misc/smartArray" {
/**
 * Defines an array and its length.
 * It can be helpful to group result from both Arrays and smart arrays in one structure.
 */
export interface ISmartArrayLike<T> {
    /**
     * The data of the array.
     */
    data: Array<T>;
    /**
     * The active length of the array.
     */
    length: number;
}
/**
 * Defines an GC Friendly array where the backfield array do not shrink to prevent over allocations.
 */
export class SmartArray<T> implements ISmartArrayLike<T> {
    /**
     * The full set of data from the array.
     */
    data: Array<T>;
    /**
     * The active length of the array.
     */
    length: number;
    protected _id: number;
    /**
     * Instantiates a Smart Array.
     * @param capacity defines the default capacity of the array.
     */
    constructor(capacity: number);
    /**
     * Pushes a value at the end of the active data.
     * @param value defines the object to push in the array.
     */
    push(value: T): void;
    /**
     * Iterates over the active data and apply the lambda to them.
     * @param func defines the action to apply on each value.
     */
    forEach(func: (content: T) => void): void;
    /**
     * Sorts the full sets of data.
     * @param compareFn defines the comparison function to apply.
     */
    sort(compareFn: (a: T, b: T) => number): void;
    /**
     * Resets the active data to an empty array.
     */
    reset(): void;
    /**
     * Releases all the data from the array as well as the array.
     */
    dispose(): void;
    /**
     * Concats the active data with a given array.
     * @param array defines the data to concatenate with.
     */
    concat(array: any): void;
    /**
     * Returns the position of a value in the active data.
     * @param value defines the value to find the index for
     * @returns the index if found in the active data otherwise -1
     */
    indexOf(value: T): number;
    /**
     * Returns whether an element is part of the active data.
     * @param value defines the value to look for
     * @returns true if found in the active data otherwise false
     */
    contains(value: T): boolean;
    private static _GlobalId;
}
/**
 * Defines an GC Friendly array where the backfield array do not shrink to prevent over allocations.
 * The data in this array can only be present once
 */
export class SmartArrayNoDuplicate<T> extends SmartArray<T> {
    private _duplicateId;
    /**
     * Pushes a value at the end of the active data.
     * THIS DOES NOT PREVENT DUPPLICATE DATA
     * @param value defines the object to push in the array.
     */
    push(value: T): void;
    /**
     * Pushes a value at the end of the active data.
     * If the data is already present, it won t be added again
     * @param value defines the object to push in the array.
     * @returns true if added false if it was already present
     */
    pushNoDuplicate(value: T): boolean;
    /**
     * Resets the active data to an empty array.
     */
    reset(): void;
    /**
     * Concats the active data with a given array.
     * This ensures no duplicate will be present in the result.
     * @param array defines the data to concatenate with.
     */
    concatWithNoDuplicate(array: any): void;
}

}
declare module "babylonjs/Misc/stringDictionary" {
import { Nullable } from "babylonjs/types";
/**
 * This class implement a typical dictionary using a string as key and the generic type T as value.
 * The underlying implementation relies on an associative array to ensure the best performances.
 * The value can be anything including 'null' but except 'undefined'
 */
export class StringDictionary<T> {
    /**
     * This will clear this dictionary and copy the content from the 'source' one.
     * If the T value is a custom object, it won't be copied/cloned, the same object will be used
     * @param source the dictionary to take the content from and copy to this dictionary
     */
    copyFrom(source: StringDictionary<T>): void;
    /**
     * Get a value based from its key
     * @param key the given key to get the matching value from
     * @returns the value if found, otherwise undefined is returned
     */
    get(key: string): T | undefined;
    /**
     * Get a value from its key or add it if it doesn't exist.
     * This method will ensure you that a given key/data will be present in the dictionary.
     * @param key the given key to get the matching value from
     * @param factory the factory that will create the value if the key is not present in the dictionary.
     * The factory will only be invoked if there's no data for the given key.
     * @returns the value corresponding to the key.
     */
    getOrAddWithFactory(key: string, factory: (key: string) => T): T;
    /**
     * Get a value from its key if present in the dictionary otherwise add it
     * @param key the key to get the value from
     * @param val if there's no such key/value pair in the dictionary add it with this value
     * @returns the value corresponding to the key
     */
    getOrAdd(key: string, val: T): T;
    /**
     * Check if there's a given key in the dictionary
     * @param key the key to check for
     * @returns true if the key is present, false otherwise
     */
    contains(key: string): boolean;
    /**
     * Add a new key and its corresponding value
     * @param key the key to add
     * @param value the value corresponding to the key
     * @returns true if the operation completed successfully, false if we couldn't insert the key/value because there was already this key in the dictionary
     */
    add(key: string, value: T): boolean;
    /**
     * Update a specific value associated to a key
     * @param key defines the key to use
     * @param value defines the value to store
     * @returns true if the value was updated (or false if the key was not found)
     */
    set(key: string, value: T): boolean;
    /**
     * Get the element of the given key and remove it from the dictionary
     * @param key defines the key to search
     * @returns the value associated with the key or null if not found
     */
    getAndRemove(key: string): Nullable<T>;
    /**
     * Remove a key/value from the dictionary.
     * @param key the key to remove
     * @returns true if the item was successfully deleted, false if no item with such key exist in the dictionary
     */
    remove(key: string): boolean;
    /**
     * Clear the whole content of the dictionary
     */
    clear(): void;
    /**
     * Gets the current count
     */
    get count(): number;
    /**
     * Execute a callback on each key/val of the dictionary.
     * Note that you can remove any element in this dictionary in the callback implementation
     * @param callback the callback to execute on a given key/value pair
     */
    forEach(callback: (key: string, val: T) => void): void;
    /**
     * Execute a callback on every occurrence of the dictionary until it returns a valid TRes object.
     * If the callback returns null or undefined the method will iterate to the next key/value pair
     * Note that you can remove any element in this dictionary in the callback implementation
     * @param callback the callback to execute, if it return a valid T instanced object the enumeration will stop and the object will be returned
     * @returns the first item
     */
    first<TRes>(callback: (key: string, val: T) => TRes): NonNullable<TRes> | null;
    private _count;
    private _data;
}

}
declare module "babylonjs/Misc/stringTools" {
/**
 * Checks for a matching suffix at the end of a string (for ES5 and lower)
 * @param str Source string
 * @param suffix Suffix to search for in the source string
 * @returns Boolean indicating whether the suffix was found (true) or not (false)
 * @deprecated Please use native string function instead
 */
export const EndsWith: (str: string, suffix: string) => boolean;
/**
 * Checks for a matching suffix at the beginning of a string (for ES5 and lower)
 * @param str Source string
 * @param suffix Suffix to search for in the source string
 * @returns Boolean indicating whether the suffix was found (true) or not (false)
 * @deprecated Please use native string function instead
 */
export const StartsWith: (str: string, suffix: string) => boolean;
/**
 * Decodes a buffer into a string
 * @param buffer The buffer to decode
 * @returns The decoded string
 */
export const Decode: (buffer: Uint8Array | Uint16Array) => string;
/**
 * Encode a buffer to a base64 string
 * @param buffer defines the buffer to encode
 * @returns the encoded string
 */
export const EncodeArrayBufferToBase64: (buffer: ArrayBuffer | ArrayBufferView) => string;
/**
 * Converts a given base64 string as an ASCII encoded stream of data
 * @param base64Data The base64 encoded string to decode
 * @returns Decoded ASCII string
 */
export const DecodeBase64ToString: (base64Data: string) => string;
/**
 * Converts a given base64 string into an ArrayBuffer of raw byte data
 * @param base64Data The base64 encoded string to decode
 * @returns ArrayBuffer of byte data
 */
export const DecodeBase64ToBinary: (base64Data: string) => ArrayBuffer;
/**
 * Converts a number to string and pads with preceding zeroes until it is of specified length.
 * @param num the number to convert and pad
 * @param length the expected length of the string
 * @returns the padded string
 */
export const PadNumber: (num: number, length: number) => string;
/**
 * Helper to manipulate strings
 */
export const StringTools: {
    EndsWith: (str: string, suffix: string) => boolean;
    StartsWith: (str: string, suffix: string) => boolean;
    Decode: (buffer: Uint8Array | Uint16Array) => string;
    EncodeArrayBufferToBase64: (buffer: ArrayBuffer | ArrayBufferView) => string;
    DecodeBase64ToString: (base64Data: string) => string;
    DecodeBase64ToBinary: (base64Data: string) => ArrayBuffer;
    PadNumber: (num: number, length: number) => string;
};

}
declare module "babylonjs/Misc/tags" {
/**
 * Class used to store custom tags
 */
export class Tags {
    /**
     * Adds support for tags on the given object
     * @param obj defines the object to use
     */
    static EnableFor(obj: any): void;
    /**
     * Removes tags support
     * @param obj defines the object to use
     */
    static DisableFor(obj: any): void;
    /**
     * Gets a boolean indicating if the given object has tags
     * @param obj defines the object to use
     * @returns a boolean
     */
    static HasTags(obj: any): boolean;
    /**
     * Gets the tags available on a given object
     * @param obj defines the object to use
     * @param asString defines if the tags must be returned as a string instead of an array of strings
     * @returns the tags
     */
    static GetTags(obj: any, asString?: boolean): any;
    /**
     * Adds tags to an object
     * @param obj defines the object to use
     * @param tagsString defines the tag string. The tags 'true' and 'false' are reserved and cannot be used as tags.
     * A tag cannot start with '||', '&&', and '!'. It cannot contain whitespaces
     */
    static AddTagsTo(obj: any, tagsString: string): void;
    /**
     * @internal
     */
    static _AddTagTo(obj: any, tag: string): void;
    /**
     * Removes specific tags from a specific object
     * @param obj defines the object to use
     * @param tagsString defines the tags to remove
     */
    static RemoveTagsFrom(obj: any, tagsString: string): void;
    /**
     * @internal
     */
    static _RemoveTagFrom(obj: any, tag: string): void;
    /**
     * Defines if tags hosted on an object match a given query
     * @param obj defines the object to use
     * @param tagsQuery defines the tag query
     * @returns a boolean
     */
    static MatchesQuery(obj: any, tagsQuery: string): boolean;
}

}
declare module "babylonjs/Misc/textureTools" {
import { InternalTexture } from "babylonjs/Materials/Textures/internalTexture";
import { Texture } from "babylonjs/Materials/Textures/texture";
import { Scene } from "babylonjs/scene";
/**
 * Uses the GPU to create a copy texture rescaled at a given size
 * @param texture Texture to copy from
 * @param width defines the desired width
 * @param height defines the desired height
 * @param useBilinearMode defines if bilinear mode has to be used
 * @returns the generated texture
 */
export function CreateResizedCopy(texture: Texture, width: number, height: number, useBilinearMode?: boolean): Texture;
/**
 * Apply a post process to a texture
 * @param postProcessName name of the fragment post process
 * @param internalTexture the texture to encode
 * @param scene the scene hosting the texture
 * @param type type of the output texture. If not provided, use the one from internalTexture
 * @param samplingMode sampling mode to use to sample the source texture. If not provided, use the one from internalTexture
 * @param format format of the output texture. If not provided, use the one from internalTexture
 * @returns a promise with the internalTexture having its texture replaced by the result of the processing
 */
export function ApplyPostProcess(postProcessName: string, internalTexture: InternalTexture, scene: Scene, type?: number, samplingMode?: number, format?: number): Promise<InternalTexture>;
/**
 * Converts a number to half float
 * @param value number to convert
 * @returns converted number
 */
export function ToHalfFloat(value: number): number;
/**
 * Converts a half float to a number
 * @param value half float to convert
 * @returns converted half float
 */
export function FromHalfFloat(value: number): number;
/**
 * Class used to host texture specific utilities
 */
export const TextureTools: {
    /**
     * Uses the GPU to create a copy texture rescaled at a given size
     * @param texture Texture to copy from
     * @param width defines the desired width
     * @param height defines the desired height
     * @param useBilinearMode defines if bilinear mode has to be used
     * @returns the generated texture
     */
    CreateResizedCopy: typeof CreateResizedCopy;
    /**
     * Apply a post process to a texture
     * @param postProcessName name of the fragment post process
     * @param internalTexture the texture to encode
     * @param scene the scene hosting the texture
     * @param type type of the output texture. If not provided, use the one from internalTexture
     * @param samplingMode sampling mode to use to sample the source texture. If not provided, use the one from internalTexture
     * @param format format of the output texture. If not provided, use the one from internalTexture
     * @returns a promise with the internalTexture having its texture replaced by the result of the processing
     */
    ApplyPostProcess: typeof ApplyPostProcess;
    /**
     * Converts a number to half float
     * @param value number to convert
     * @returns converted number
     */
    ToHalfFloat: typeof ToHalfFloat;
    /**
     * Converts a half float to a number
     * @param value half float to convert
     * @returns converted half float
     */
    FromHalfFloat: typeof FromHalfFloat;
};

}
declare module "babylonjs/Misc/tga" {
import { InternalTexture } from "babylonjs/Materials/Textures/internalTexture";
/**
 * Gets the header of a TGA file
 * @param data defines the TGA data
 * @returns the header
 */
export function GetTGAHeader(data: Uint8Array): any;
/**
 * Uploads TGA content to a Babylon Texture
 * @internal
 */
export function UploadContent(texture: InternalTexture, data: Uint8Array): void;
/**
 * @internal
 */
function _getImageData8bits(header: any, palettes: Uint8Array, pixel_data: Uint8Array, y_start: number, y_step: number, y_end: number, x_start: number, x_step: number, x_end: number): Uint8Array;
/**
 * @internal
 */
function _getImageData16bits(header: any, palettes: Uint8Array, pixel_data: Uint8Array, y_start: number, y_step: number, y_end: number, x_start: number, x_step: number, x_end: number): Uint8Array;
/**
 * @internal
 */
function _getImageData24bits(header: any, palettes: Uint8Array, pixel_data: Uint8Array, y_start: number, y_step: number, y_end: number, x_start: number, x_step: number, x_end: number): Uint8Array;
/**
 * @internal
 */
function _getImageData32bits(header: any, palettes: Uint8Array, pixel_data: Uint8Array, y_start: number, y_step: number, y_end: number, x_start: number, x_step: number, x_end: number): Uint8Array;
/**
 * @internal
 */
function _getImageDataGrey8bits(header: any, palettes: Uint8Array, pixel_data: Uint8Array, y_start: number, y_step: number, y_end: number, x_start: number, x_step: number, x_end: number): Uint8Array;
/**
 * @internal
 */
function _getImageDataGrey16bits(header: any, palettes: Uint8Array, pixel_data: Uint8Array, y_start: number, y_step: number, y_end: number, x_start: number, x_step: number, x_end: number): Uint8Array;
/**
 * Based on jsTGALoader - Javascript loader for TGA file
 * By Vincent Thibault
 * @see http://blog.robrowser.com/javascript-tga-loader.html
 */
export const TGATools: {
    /**
     * Gets the header of a TGA file
     * @param data defines the TGA data
     * @returns the header
     */
    GetTGAHeader: typeof GetTGAHeader;
    /**
     * Uploads TGA content to a Babylon Texture
     * @internal
     */
    UploadContent: typeof UploadContent;
    /** @internal */
    _getImageData8bits: typeof _getImageData8bits;
    /** @internal */
    _getImageData16bits: typeof _getImageData16bits;
    /** @internal */
    _getImageData24bits: typeof _getImageData24bits;
    /** @internal */
    _getImageData32bits: typeof _getImageData32bits;
    /** @internal */
    _getImageDataGrey8bits: typeof _getImageDataGrey8bits;
    /** @internal */
    _getImageDataGrey16bits: typeof _getImageDataGrey16bits;
};
export {};

}
declare module "babylonjs/Misc/timer" {
import { Observer } from "babylonjs/Misc/observable";
import { Observable } from "babylonjs/Misc/observable";
import { Nullable } from "babylonjs/types";
import { IDisposable } from "babylonjs/scene";
/**
 * Construction options for a timer
 */
export interface ITimerOptions<T> {
    /**
     * Time-to-end
     */
    timeout: number;
    /**
     * The context observable is used to calculate time deltas and provides the context of the timer's callbacks. Will usually be OnBeforeRenderObservable.
     * Countdown calculation is done ONLY when the observable is notifying its observers, meaning that if
     * you choose an observable that doesn't trigger too often, the wait time might extend further than the requested max time
     */
    contextObservable: Observable<T>;
    /**
     * Optional parameters when adding an observer to the observable
     */
    observableParameters?: {
        mask?: number;
        insertFirst?: boolean;
        scope?: any;
    };
    /**
     * An optional break condition that will stop the times prematurely. In this case onEnded will not be triggered!
     */
    breakCondition?: (data?: ITimerData<T>) => boolean;
    /**
     * Will be triggered when the time condition has met
     */
    onEnded?: (data: ITimerData<any>) => void;
    /**
     * Will be triggered when the break condition has met (prematurely ended)
     */
    onAborted?: (data: ITimerData<any>) => void;
    /**
     * Optional function to execute on each tick (or count)
     */
    onTick?: (data: ITimerData<any>) => void;
}
/**
 * An interface defining the data sent by the timer
 */
export interface ITimerData<T> {
    /**
     * When did it start
     */
    startTime: number;
    /**
     * Time now
     */
    currentTime: number;
    /**
     * Time passed since started
     */
    deltaTime: number;
    /**
     * How much is completed, in [0.0...1.0].
     * Note that this CAN be higher than 1 due to the fact that we don't actually measure time but delta between observable calls
     */
    completeRate: number;
    /**
     * What the registered observable sent in the last count
     */
    payload: T;
}
/**
 * The current state of the timer
 */
export enum TimerState {
    /**
     * Timer initialized, not yet started
     */
    INIT = 0,
    /**
     * Timer started and counting
     */
    STARTED = 1,
    /**
     * Timer ended (whether aborted or time reached)
     */
    ENDED = 2
}
/**
 * A simple version of the timer. Will take options and start the timer immediately after calling it
 *
 * @param options options with which to initialize this timer
 */
export function setAndStartTimer(options: ITimerOptions<any>): Nullable<Observer<any>>;
/**
 * An advanced implementation of a timer class
 */
export class AdvancedTimer<T = any> implements IDisposable {
    /**
     * Will notify each time the timer calculates the remaining time
     */
    onEachCountObservable: Observable<ITimerData<T>>;
    /**
     * Will trigger when the timer was aborted due to the break condition
     */
    onTimerAbortedObservable: Observable<ITimerData<T>>;
    /**
     * Will trigger when the timer ended successfully
     */
    onTimerEndedObservable: Observable<ITimerData<T>>;
    /**
     * Will trigger when the timer state has changed
     */
    onStateChangedObservable: Observable<TimerState>;
    private _observer;
    private _contextObservable;
    private _observableParameters;
    private _startTime;
    private _timer;
    private _state;
    private _breakCondition;
    private _timeToEnd;
    private _breakOnNextTick;
    /**
     * Will construct a new advanced timer based on the options provided. Timer will not start until start() is called.
     * @param options construction options for this advanced timer
     */
    constructor(options: ITimerOptions<T>);
    /**
     * set a breaking condition for this timer. Default is to never break during count
     * @param predicate the new break condition. Returns true to break, false otherwise
     */
    set breakCondition(predicate: (data: ITimerData<T>) => boolean);
    /**
     * Reset ALL associated observables in this advanced timer
     */
    clearObservables(): void;
    /**
     * Will start a new iteration of this timer. Only one instance of this timer can run at a time.
     *
     * @param timeToEnd how much time to measure until timer ended
     */
    start(timeToEnd?: number): void;
    /**
     * Will force a stop on the next tick.
     */
    stop(): void;
    /**
     * Dispose this timer, clearing all resources
     */
    dispose(): void;
    private _setState;
    private _tick;
    private _stop;
}

}
declare module "babylonjs/Misc/timingTools" {
/**
 * Class used to provide helper for timing
 */
export class TimingTools {
    /**
     * Polyfill for setImmediate
     * @param action defines the action to execute after the current execution block
     */
    static SetImmediate(action: () => void): void;
}

}
declare module "babylonjs/Misc/tools" {
import { Nullable } from "babylonjs/types";
import { GetDOMTextContent, IsWindowObjectExist } from "babylonjs/Misc/domManagement";
import { WebRequest } from "babylonjs/Misc/webRequest";
import { IFileRequest } from "babylonjs/Misc/fileRequest";
import { ReadFileError } from "babylonjs/Misc/fileTools";
import { IOfflineProvider } from "babylonjs/Offline/IOfflineProvider";
import { IScreenshotSize } from "babylonjs/Misc/interfaces/screenshotSize";
import { Engine } from "babylonjs/Engines/engine";
import { Camera } from "babylonjs/Cameras/camera";
import { IColor4Like } from "babylonjs/Maths/math.like";
/**
 * Class containing a set of static utilities functions
 */
export class Tools {
    /**
     * Gets or sets the base URL to use to load assets
     */
    static get BaseUrl(): string;
    static set BaseUrl(value: string);
    /**
     * Enable/Disable Custom HTTP Request Headers globally.
     * default = false
     * @see CustomRequestHeaders
     */
    static UseCustomRequestHeaders: boolean;
    /**
     * Custom HTTP Request Headers to be sent with XMLHttpRequests
     * i.e. when loading files, where the server/service expects an Authorization header
     */
    static CustomRequestHeaders: {
        [key: string]: string;
    };
    /**
     * Gets or sets the retry strategy to apply when an error happens while loading an asset
     */
    static get DefaultRetryStrategy(): (url: string, request: WebRequest, retryIndex: number) => number;
    static set DefaultRetryStrategy(strategy: (url: string, request: WebRequest, retryIndex: number) => number);
    /**
     * Default behaviour for cors in the application.
     * It can be a string if the expected behavior is identical in the entire app.
     * Or a callback to be able to set it per url or on a group of them (in case of Video source for instance)
     */
    static get CorsBehavior(): string | ((url: string | string[]) => string);
    static set CorsBehavior(value: string | ((url: string | string[]) => string));
    /**
     * Gets or sets a global variable indicating if fallback texture must be used when a texture cannot be loaded
     * @ignorenaming
     */
    static get UseFallbackTexture(): boolean;
    static set UseFallbackTexture(value: boolean);
    /**
     * Use this object to register external classes like custom textures or material
     * to allow the loaders to instantiate them
     */
    static get RegisteredExternalClasses(): {
        [key: string]: Object;
    };
    static set RegisteredExternalClasses(classes: {
        [key: string]: Object;
    });
    /**
     * Texture content used if a texture cannot loaded
     * @ignorenaming
     */
    static get fallbackTexture(): string;
    static set fallbackTexture(value: string);
    /**
     * Read the content of a byte array at a specified coordinates (taking in account wrapping)
     * @param u defines the coordinate on X axis
     * @param v defines the coordinate on Y axis
     * @param width defines the width of the source data
     * @param height defines the height of the source data
     * @param pixels defines the source byte array
     * @param color defines the output color
     */
    static FetchToRef(u: number, v: number, width: number, height: number, pixels: Uint8Array, color: IColor4Like): void;
    /**
     * Interpolates between a and b via alpha
     * @param a The lower value (returned when alpha = 0)
     * @param b The upper value (returned when alpha = 1)
     * @param alpha The interpolation-factor
     * @returns The mixed value
     */
    static Mix(a: number, b: number, alpha: number): number;
    /**
     * Tries to instantiate a new object from a given class name
     * @param className defines the class name to instantiate
     * @returns the new object or null if the system was not able to do the instantiation
     */
    static Instantiate(className: string): any;
    /**
     * Polyfill for setImmediate
     * @param action defines the action to execute after the current execution block
     */
    static SetImmediate(action: () => void): void;
    /**
     * Function indicating if a number is an exponent of 2
     * @param value defines the value to test
     * @returns true if the value is an exponent of 2
     */
    static IsExponentOfTwo(value: number): boolean;
    private static _TmpFloatArray;
    /**
     * Returns the nearest 32-bit single precision float representation of a Number
     * @param value A Number.  If the parameter is of a different type, it will get converted
     * to a number or to NaN if it cannot be converted
     * @returns number
     */
    static FloatRound(value: number): number;
    /**
     * Extracts the filename from a path
     * @param path defines the path to use
     * @returns the filename
     */
    static GetFilename(path: string): string;
    /**
     * Extracts the "folder" part of a path (everything before the filename).
     * @param uri The URI to extract the info from
     * @param returnUnchangedIfNoSlash Do not touch the URI if no slashes are present
     * @returns The "folder" part of the path
     */
    static GetFolderPath(uri: string, returnUnchangedIfNoSlash?: boolean): string;
    /**
     * Extracts text content from a DOM element hierarchy
     * Back Compat only, please use GetDOMTextContent instead.
     */
    static GetDOMTextContent: typeof GetDOMTextContent;
    /**
     * Convert an angle in radians to degrees
     * @param angle defines the angle to convert
     * @returns the angle in degrees
     */
    static ToDegrees(angle: number): number;
    /**
     * Convert an angle in degrees to radians
     * @param angle defines the angle to convert
     * @returns the angle in radians
     */
    static ToRadians(angle: number): number;
    /**
     * Smooth angle changes (kind of low-pass filter), in particular for device orientation "shaking"
     * Use trigonometric functions to avoid discontinuity (0/360, -180/180)
     * @param previousAngle defines last angle value, in degrees
     * @param newAngle defines new angle value, in degrees
     * @param smoothFactor defines smoothing sensitivity; min 0: no smoothing, max 1: new data ignored
     * @returns the angle in degrees
     */
    static SmoothAngleChange(previousAngle: number, newAngle: number, smoothFactor?: number): number;
    /**
     * Returns an array if obj is not an array
     * @param obj defines the object to evaluate as an array
     * @param allowsNullUndefined defines a boolean indicating if obj is allowed to be null or undefined
     * @returns either obj directly if obj is an array or a new array containing obj
     */
    static MakeArray(obj: any, allowsNullUndefined?: boolean): Nullable<Array<any>>;
    /**
     * Gets the pointer prefix to use
     * @param engine defines the engine we are finding the prefix for
     * @returns "pointer" if touch is enabled. Else returns "mouse"
     */
    static GetPointerPrefix(engine: Engine): string;
    /**
     * Sets the cors behavior on a dom element. This will add the required Tools.CorsBehavior to the element.
     * @param url define the url we are trying
     * @param element define the dom element where to configure the cors policy
     * @param element.crossOrigin
     */
    static SetCorsBehavior(url: string | string[], element: {
        crossOrigin: string | null;
    }): void;
    /**
     * Sets the referrerPolicy behavior on a dom element.
     * @param referrerPolicy define the referrer policy to use
     * @param element define the dom element where to configure the referrer policy
     * @param element.referrerPolicy
     */
    static SetReferrerPolicyBehavior(referrerPolicy: Nullable<ReferrerPolicy>, element: {
        referrerPolicy: string | null;
    }): void;
    /**
     * Removes unwanted characters from an url
     * @param url defines the url to clean
     * @returns the cleaned url
     */
    static CleanUrl(url: string): string;
    /**
     * Gets or sets a function used to pre-process url before using them to load assets
     */
    static get PreprocessUrl(): (url: string) => string;
    static set PreprocessUrl(processor: (url: string) => string);
    /**
     * Loads an image as an HTMLImageElement.
     * @param input url string, ArrayBuffer, or Blob to load
     * @param onLoad callback called when the image successfully loads
     * @param onError callback called when the image fails to load
     * @param offlineProvider offline provider for caching
     * @param mimeType optional mime type
     * @param imageBitmapOptions optional the options to use when creating an ImageBitmap
     * @returns the HTMLImageElement of the loaded image
     */
    static LoadImage(input: string | ArrayBuffer | Blob, onLoad: (img: HTMLImageElement | ImageBitmap) => void, onError: (message?: string, exception?: any) => void, offlineProvider: Nullable<IOfflineProvider>, mimeType?: string, imageBitmapOptions?: ImageBitmapOptions): Nullable<HTMLImageElement>;
    /**
     * Loads a file from a url
     * @param url url string, ArrayBuffer, or Blob to load
     * @param onSuccess callback called when the file successfully loads
     * @param onProgress callback called while file is loading (if the server supports this mode)
     * @param offlineProvider defines the offline provider for caching
     * @param useArrayBuffer defines a boolean indicating that date must be returned as ArrayBuffer
     * @param onError callback called when the file fails to load
     * @returns a file request object
     */
    static LoadFile(url: string, onSuccess: (data: string | ArrayBuffer, responseURL?: string) => void, onProgress?: (data: any) => void, offlineProvider?: IOfflineProvider, useArrayBuffer?: boolean, onError?: (request?: WebRequest, exception?: any) => void): IFileRequest;
    /**
     * Loads a file from a url
     * @param url the file url to load
     * @param useArrayBuffer defines a boolean indicating that date must be returned as ArrayBuffer
     * @returns a promise containing an ArrayBuffer corresponding to the loaded file
     */
    static LoadFileAsync(url: string, useArrayBuffer?: boolean): Promise<ArrayBuffer | string>;
    /**
     * Load a script (identified by an url). When the url returns, the
     * content of this file is added into a new script element, attached to the DOM (body element)
     * @param scriptUrl defines the url of the script to laod
     * @param onSuccess defines the callback called when the script is loaded
     * @param onError defines the callback to call if an error occurs
     * @param scriptId defines the id of the script element
     */
    static LoadScript(scriptUrl: string, onSuccess: () => void, onError?: (message?: string, exception?: any) => void, scriptId?: string): void;
    /**
     * Load an asynchronous script (identified by an url). When the url returns, the
     * content of this file is added into a new script element, attached to the DOM (body element)
     * @param scriptUrl defines the url of the script to laod
     * @returns a promise request object
     */
    static LoadScriptAsync(scriptUrl: string): Promise<void>;
    /**
     * Loads a file from a blob
     * @param fileToLoad defines the blob to use
     * @param callback defines the callback to call when data is loaded
     * @param progressCallback defines the callback to call during loading process
     * @returns a file request object
     */
    static ReadFileAsDataURL(fileToLoad: Blob, callback: (data: any) => void, progressCallback: (ev: ProgressEvent) => any): IFileRequest;
    /**
     * Reads a file from a File object
     * @param file defines the file to load
     * @param onSuccess defines the callback to call when data is loaded
     * @param onProgress defines the callback to call during loading process
     * @param useArrayBuffer defines a boolean indicating that data must be returned as an ArrayBuffer
     * @param onError defines the callback to call when an error occurs
     * @returns a file request object
     */
    static ReadFile(file: File, onSuccess: (data: any) => void, onProgress?: (ev: ProgressEvent) => any, useArrayBuffer?: boolean, onError?: (error: ReadFileError) => void): IFileRequest;
    /**
     * Creates a data url from a given string content
     * @param content defines the content to convert
     * @returns the new data url link
     */
    static FileAsURL(content: string): string;
    /**
     * Format the given number to a specific decimal format
     * @param value defines the number to format
     * @param decimals defines the number of decimals to use
     * @returns the formatted string
     */
    static Format(value: number, decimals?: number): string;
    /**
     * Tries to copy an object by duplicating every property
     * @param source defines the source object
     * @param destination defines the target object
     * @param doNotCopyList defines a list of properties to avoid
     * @param mustCopyList defines a list of properties to copy (even if they start with _)
     */
    static DeepCopy(source: any, destination: any, doNotCopyList?: string[], mustCopyList?: string[]): void;
    /**
     * Gets a boolean indicating if the given object has no own property
     * @param obj defines the object to test
     * @returns true if object has no own property
     */
    static IsEmpty(obj: any): boolean;
    /**
     * Function used to register events at window level
     * @param windowElement defines the Window object to use
     * @param events defines the events to register
     */
    static RegisterTopRootEvents(windowElement: Window, events: {
        name: string;
        handler: Nullable<(e: FocusEvent) => any>;
    }[]): void;
    /**
     * Function used to unregister events from window level
     * @param windowElement defines the Window object to use
     * @param events defines the events to unregister
     */
    static UnregisterTopRootEvents(windowElement: Window, events: {
        name: string;
        handler: Nullable<(e: FocusEvent) => any>;
    }[]): void;
    /**
     * Dumps the current bound framebuffer
     * @param width defines the rendering width
     * @param height defines the rendering height
     * @param engine defines the hosting engine
     * @param successCallback defines the callback triggered once the data are available
     * @param mimeType defines the mime type of the result
     * @param fileName defines the filename to download. If present, the result will automatically be downloaded
     * @returns a void promise
     */
    static DumpFramebuffer(width: number, height: number, engine: Engine, successCallback?: (data: string) => void, mimeType?: string, fileName?: string): Promise<void>;
    /**
     * Dumps an array buffer
     * @param width defines the rendering width
     * @param height defines the rendering height
     * @param data the data array
     * @param successCallback defines the callback triggered once the data are available
     * @param mimeType defines the mime type of the result
     * @param fileName defines the filename to download. If present, the result will automatically be downloaded
     * @param invertY true to invert the picture in the Y dimension
     * @param toArrayBuffer true to convert the data to an ArrayBuffer (encoded as `mimeType`) instead of a base64 string
     * @param quality defines the quality of the result
     */
    static DumpData(width: number, height: number, data: ArrayBufferView, successCallback?: (data: string | ArrayBuffer) => void, mimeType?: string, fileName?: string, invertY?: boolean, toArrayBuffer?: boolean, quality?: number): void;
    /**
     * Dumps an array buffer
     * @param width defines the rendering width
     * @param height defines the rendering height
     * @param data the data array
     * @param mimeType defines the mime type of the result
     * @param fileName defines the filename to download. If present, the result will automatically be downloaded
     * @param invertY true to invert the picture in the Y dimension
     * @param toArrayBuffer true to convert the data to an ArrayBuffer (encoded as `mimeType`) instead of a base64 string
     * @param quality defines the quality of the result
     * @returns a promise that resolve to the final data
     */
    static DumpDataAsync(width: number, height: number, data: ArrayBufferView, mimeType?: string, fileName?: string, invertY?: boolean, toArrayBuffer?: boolean, quality?: number): Promise<string | ArrayBuffer>;
    /**
     * Converts the canvas data to blob.
     * This acts as a polyfill for browsers not supporting the to blob function.
     * @param canvas Defines the canvas to extract the data from
     * @param successCallback Defines the callback triggered once the data are available
     * @param mimeType Defines the mime type of the result
     * @param quality defines the quality of the result
     */
    static ToBlob(canvas: HTMLCanvasElement, successCallback: (blob: Nullable<Blob>) => void, mimeType?: string, quality?: number): void;
    /**
     * Download a Blob object
     * @param blob the Blob object
     * @param fileName the file name to download
     * @returns
     */
    static DownloadBlob(blob: Blob, fileName?: string): void;
    /**
     * Encodes the canvas data to base 64 or automatically download the result if filename is defined
     * @param canvas canvas to get the data from.
     * @param successCallback defines the callback triggered once the data are available
     * @param mimeType defines the mime type of the result
     * @param fileName defines he filename to download. If present, the result will automatically be downloaded
     * @param quality defines the quality of the result
     */
    static EncodeScreenshotCanvasData(canvas: HTMLCanvasElement, successCallback?: (data: string) => void, mimeType?: string, fileName?: string, quality?: number): void;
    /**
     * Downloads a blob in the browser
     * @param blob defines the blob to download
     * @param fileName defines the name of the downloaded file
     */
    static Download(blob: Blob, fileName: string): void;
    /**
     * Will return the right value of the noPreventDefault variable
     * Needed to keep backwards compatibility to the old API.
     *
     * @param args arguments passed to the attachControl function
     * @returns the correct value for noPreventDefault
     */
    static BackCompatCameraNoPreventDefault(args: IArguments): boolean;
    /**
     * Captures a screenshot of the current rendering
     * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/renderToPNG
     * @param engine defines the rendering engine
     * @param camera defines the source camera
     * @param size This parameter can be set to a single number or to an object with the
     * following (optional) properties: precision, width, height. If a single number is passed,
     * it will be used for both width and height. If an object is passed, the screenshot size
     * will be derived from the parameters. The precision property is a multiplier allowing
     * rendering at a higher or lower resolution
     * @param successCallback defines the callback receives a single parameter which contains the
     * screenshot as a string of base64-encoded characters. This string can be assigned to the
     * src parameter of an <img> to display it
     * @param mimeType defines the MIME type of the screenshot image (default: image/png).
     * Check your browser for supported MIME types
     */
    static CreateScreenshot(engine: Engine, camera: Camera, size: IScreenshotSize | number, successCallback?: (data: string) => void, mimeType?: string): void;
    /**
     * Captures a screenshot of the current rendering
     * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/renderToPNG
     * @param engine defines the rendering engine
     * @param camera defines the source camera
     * @param size This parameter can be set to a single number or to an object with the
     * following (optional) properties: precision, width, height. If a single number is passed,
     * it will be used for both width and height. If an object is passed, the screenshot size
     * will be derived from the parameters. The precision property is a multiplier allowing
     * rendering at a higher or lower resolution
     * @param mimeType defines the MIME type of the screenshot image (default: image/png).
     * Check your browser for supported MIME types
     * @returns screenshot as a string of base64-encoded characters. This string can be assigned
     * to the src parameter of an <img> to display it
     */
    static CreateScreenshotAsync(engine: Engine, camera: Camera, size: IScreenshotSize | number, mimeType?: string): Promise<string>;
    /**
     * Generates an image screenshot from the specified camera.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/renderToPNG
     * @param engine The engine to use for rendering
     * @param camera The camera to use for rendering
     * @param size This parameter can be set to a single number or to an object with the
     * following (optional) properties: precision, width, height. If a single number is passed,
     * it will be used for both width and height. If an object is passed, the screenshot size
     * will be derived from the parameters. The precision property is a multiplier allowing
     * rendering at a higher or lower resolution
     * @param successCallback The callback receives a single parameter which contains the
     * screenshot as a string of base64-encoded characters. This string can be assigned to the
     * src parameter of an <img> to display it
     * @param mimeType The MIME type of the screenshot image (default: image/png).
     * Check your browser for supported MIME types
     * @param samples Texture samples (default: 1)
     * @param antialiasing Whether antialiasing should be turned on or not (default: false)
     * @param fileName A name for for the downloaded file.
     */
    static CreateScreenshotUsingRenderTarget(engine: Engine, camera: Camera, size: IScreenshotSize | number, successCallback?: (data: string) => void, mimeType?: string, samples?: number, antialiasing?: boolean, fileName?: string): void;
    /**
     * Generates an image screenshot from the specified camera.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/renderToPNG
     * @param engine The engine to use for rendering
     * @param camera The camera to use for rendering
     * @param size This parameter can be set to a single number or to an object with the
     * following (optional) properties: precision, width, height. If a single number is passed,
     * it will be used for both width and height. If an object is passed, the screenshot size
     * will be derived from the parameters. The precision property is a multiplier allowing
     * rendering at a higher or lower resolution
     * @param mimeType The MIME type of the screenshot image (default: image/png).
     * Check your browser for supported MIME types
     * @param samples Texture samples (default: 1)
     * @param antialiasing Whether antialiasing should be turned on or not (default: false)
     * @param fileName A name for for the downloaded file.
     * @returns screenshot as a string of base64-encoded characters. This string can be assigned
     * to the src parameter of an <img> to display it
     */
    static CreateScreenshotUsingRenderTargetAsync(engine: Engine, camera: Camera, size: IScreenshotSize | number, mimeType?: string, samples?: number, antialiasing?: boolean, fileName?: string): Promise<string>;
    /**
     * Implementation from http://stackoverflow.com/questions/105034/how-to-create-a-guid-uuid-in-javascript/2117523#answer-2117523
     * Be aware Math.random() could cause collisions, but:
     * "All but 6 of the 128 bits of the ID are randomly generated, which means that for any two ids, there's a 1 in 2^^122 (or 5.3x10^^36) chance they'll collide"
     * @returns a pseudo random id
     */
    static RandomId(): string;
    /**
     * Test if the given uri is a base64 string
     * @deprecated Please use FileTools.IsBase64DataUrl instead.
     * @param uri The uri to test
     * @returns True if the uri is a base64 string or false otherwise
     */
    static IsBase64(uri: string): boolean;
    /**
     * Decode the given base64 uri.
     * @deprecated Please use FileTools.DecodeBase64UrlToBinary instead.
     * @param uri The uri to decode
     * @returns The decoded base64 data.
     */
    static DecodeBase64(uri: string): ArrayBuffer;
    static GetAbsoluteUrl: (url: string) => string;
    /**
     * No log
     */
    static readonly NoneLogLevel: number;
    /**
     * Only message logs
     */
    static readonly MessageLogLevel: number;
    /**
     * Only warning logs
     */
    static readonly WarningLogLevel: number;
    /**
     * Only error logs
     */
    static readonly ErrorLogLevel: number;
    /**
     * All logs
     */
    static readonly AllLogLevel: number;
    /**
     * Gets a value indicating the number of loading errors
     * @ignorenaming
     */
    static get errorsCount(): number;
    /**
     * Callback called when a new log is added
     */
    static OnNewCacheEntry: (entry: string) => void;
    /**
     * Log a message to the console
     * @param message defines the message to log
     */
    static Log(message: string): void;
    /**
     * Write a warning message to the console
     * @param message defines the message to log
     */
    static Warn(message: string): void;
    /**
     * Write an error message to the console
     * @param message defines the message to log
     */
    static Error(message: string): void;
    /**
     * Gets current log cache (list of logs)
     */
    static get LogCache(): string;
    /**
     * Clears the log cache
     */
    static ClearLogCache(): void;
    /**
     * Sets the current log level (MessageLogLevel / WarningLogLevel / ErrorLogLevel)
     */
    static set LogLevels(level: number);
    /**
     * Checks if the window object exists
     * Back Compat only, please use IsWindowObjectExist instead.
     */
    static IsWindowObjectExist: typeof IsWindowObjectExist;
    /**
     * No performance log
     */
    static readonly PerformanceNoneLogLevel: number;
    /**
     * Use user marks to log performance
     */
    static readonly PerformanceUserMarkLogLevel: number;
    /**
     * Log performance to the console
     */
    static readonly PerformanceConsoleLogLevel: number;
    private static _Performance;
    /**
     * Sets the current performance log level
     */
    static set PerformanceLogLevel(level: number);
    private static _StartPerformanceCounterDisabled;
    private static _EndPerformanceCounterDisabled;
    private static _StartUserMark;
    private static _EndUserMark;
    private static _StartPerformanceConsole;
    private static _EndPerformanceConsole;
    /**
     * Starts a performance counter
     */
    static StartPerformanceCounter: (counterName: string, condition?: boolean) => void;
    /**
     * Ends a specific performance counter
     */
    static EndPerformanceCounter: (counterName: string, condition?: boolean) => void;
    /**
     * Gets either window.performance.now() if supported or Date.now() else
     */
    static get Now(): number;
    /**
     * This method will return the name of the class used to create the instance of the given object.
     * It will works only on Javascript basic data types (number, string, ...) and instance of class declared with the @className decorator.
     * @param object the object to get the class name from
     * @param isType defines if the object is actually a type
     * @returns the name of the class, will be "object" for a custom data type not using the @className decorator
     */
    static GetClassName(object: any, isType?: boolean): string;
    /**
     * Gets the first element of an array satisfying a given predicate
     * @param array defines the array to browse
     * @param predicate defines the predicate to use
     * @returns null if not found or the element
     */
    static First<T>(array: Array<T>, predicate: (item: T) => boolean): Nullable<T>;
    /**
     * This method will return the name of the full name of the class, including its owning module (if any).
     * It will works only on Javascript basic data types (number, string, ...) and instance of class declared with the @className decorator or implementing a method getClassName():string (in which case the module won't be specified).
     * @param object the object to get the class name from
     * @param isType defines if the object is actually a type
     * @returns a string that can have two forms: "moduleName.className" if module was specified when the class' Name was registered or "className" if there was not module specified.
     * @ignorenaming
     */
    static getFullClassName(object: any, isType?: boolean): Nullable<string>;
    /**
     * Returns a promise that resolves after the given amount of time.
     * @param delay Number of milliseconds to delay
     * @returns Promise that resolves after the given amount of time
     */
    static DelayAsync(delay: number): Promise<void>;
    /**
     * Utility function to detect if the current user agent is Safari
     * @returns whether or not the current user agent is safari
     */
    static IsSafari(): boolean;
}
/**
 * Use this className as a decorator on a given class definition to add it a name and optionally its module.
 * You can then use the Tools.getClassName(obj) on an instance to retrieve its class name.
 * This method is the only way to get it done in all cases, even if the .js file declaring the class is minified
 * @param name The name of the class, case should be preserved
 * @param module The name of the Module hosting the class, optional, but strongly recommended to specify if possible. Case should be preserved.
 */
export function className(name: string, module?: string): (target: Object) => void;
/**
 * An implementation of a loop for asynchronous functions.
 */
export class AsyncLoop {
    /**
     * Defines the number of iterations for the loop
     */
    iterations: number;
    /**
     * Defines the current index of the loop.
     */
    index: number;
    private _done;
    private _fn;
    private _successCallback;
    /**
     * Constructor.
     * @param iterations the number of iterations.
     * @param func the function to run each iteration
     * @param successCallback the callback that will be called upon successful execution
     * @param offset starting offset.
     */
    constructor(
    /**
     * Defines the number of iterations for the loop
     */
    iterations: number, func: (asyncLoop: AsyncLoop) => void, successCallback: () => void, offset?: number);
    /**
     * Execute the next iteration. Must be called after the last iteration was finished.
     */
    executeNext(): void;
    /**
     * Break the loop and run the success callback.
     */
    breakLoop(): void;
    /**
     * Create and run an async loop.
     * @param iterations the number of iterations.
     * @param fn the function to run each iteration
     * @param successCallback the callback that will be called upon successful execution
     * @param offset starting offset.
     * @returns the created async loop object
     */
    static Run(iterations: number, fn: (asyncLoop: AsyncLoop) => void, successCallback: () => void, offset?: number): AsyncLoop;
    /**
     * A for-loop that will run a given number of iterations synchronous and the rest async.
     * @param iterations total number of iterations
     * @param syncedIterations number of synchronous iterations in each async iteration.
     * @param fn the function to call each iteration.
     * @param callback a success call back that will be called when iterating stops.
     * @param breakFunction a break condition (optional)
     * @param timeout timeout settings for the setTimeout function. default - 0.
     * @returns the created async loop object
     */
    static SyncAsyncForLoop(iterations: number, syncedIterations: number, fn: (iteration: number) => void, callback: () => void, breakFunction?: () => boolean, timeout?: number): AsyncLoop;
}

}
declare module "babylonjs/Misc/trajectoryClassifier" {
import { DeepImmutable, Nullable } from "babylonjs/types";
import { Vector3 } from "babylonjs/Maths/math.vector";
/**
 * A 3D trajectory consisting of an order list of vectors describing a
 * path of motion through 3D space.
 */
export class Trajectory {
    private _points;
    private readonly _segmentLength;
    /**
     * Serialize to JSON.
     * @returns serialized JSON string
     */
    serialize(): string;
    /**
     * Deserialize from JSON.
     * @param json serialized JSON string
     * @returns deserialized Trajectory
     */
    static Deserialize(json: string): Trajectory;
    /**
     * Create a new empty Trajectory.
     * @param segmentLength radius of discretization for Trajectory points
     */
    constructor(segmentLength?: number);
    /**
     * Get the length of the Trajectory.
     * @returns length of the Trajectory
     */
    getLength(): number;
    /**
     * Append a new point to the Trajectory.
     * NOTE: This implementation has many allocations.
     * @param point point to append to the Trajectory
     */
    add(point: DeepImmutable<Vector3>): void;
    /**
     * Create a new Trajectory with a segment length chosen to make it
     * probable that the new Trajectory will have a specified number of
     * segments. This operation is imprecise.
     * @param targetResolution number of segments desired
     * @returns new Trajectory with approximately the requested number of segments
     */
    resampleAtTargetResolution(targetResolution: number): Trajectory;
    /**
     * Convert Trajectory segments into tokenized representation. This
     * representation is an array of numbers where each nth number is the
     * index of the token which is most similar to the nth segment of the
     * Trajectory.
     * @param tokens list of vectors which serve as discrete tokens
     * @returns list of indices of most similar token per segment
     */
    tokenize(tokens: DeepImmutable<Vector3[]>): number[];
    private static _ForwardDir;
    private static _InverseFromVec;
    private static _UpDir;
    private static _FromToVec;
    private static _LookMatrix;
    /**
     * Transform the rotation (i.e., direction) of a segment to isolate
     * the relative transformation represented by the segment. This operation
     * may or may not succeed due to singularities in the equations that define
     * motion relativity in this context.
     * @param priorVec the origin of the prior segment
     * @param fromVec the origin of the current segment
     * @param toVec the destination of the current segment
     * @param result reference to output variable
     * @returns whether or not transformation was successful
     */
    private static _TransformSegmentDirToRef;
    private static _BestMatch;
    private static _Score;
    private static _BestScore;
    /**
     * Determine which token vector is most similar to the
     * segment vector.
     * @param segment segment vector
     * @param tokens token vector list
     * @returns index of the most similar token to the segment
     */
    private static _TokenizeSegment;
}
/**
 * Class representing a set of known, named trajectories to which Trajectories can be
 * added and using which Trajectories can be recognized.
 */
export class TrajectoryClassifier {
    private _maximumAllowableMatchCost;
    private _vector3Alphabet;
    private _levenshteinAlphabet;
    private _nameToDescribedTrajectory;
    /**
     * Serialize to JSON.
     * @returns JSON serialization
     */
    serialize(): string;
    /**
     * Deserialize from JSON.
     * @param json JSON serialization
     * @returns deserialized TrajectorySet
     */
    static Deserialize(json: string): TrajectoryClassifier;
    /**
     * Initialize a new empty TrajectorySet with auto-generated Alphabets.
     * VERY naive, need to be generating these things from known
     * sets. Better version later, probably eliminating this one.
     * @returns auto-generated TrajectorySet
     */
    static Generate(): TrajectoryClassifier;
    private constructor();
    /**
     * Add a new Trajectory to the set with a given name.
     * @param trajectory new Trajectory to be added
     * @param classification name to which to add the Trajectory
     */
    addTrajectoryToClassification(trajectory: Trajectory, classification: string): void;
    /**
     * Remove a known named trajectory and all Trajectories associated with it.
     * @param classification name to remove
     * @returns whether anything was removed
     */
    deleteClassification(classification: string): boolean;
    /**
     * Attempt to recognize a Trajectory from among all the classifications
     * already known to the classifier.
     * @param trajectory Trajectory to be recognized
     * @returns classification of Trajectory if recognized, null otherwise
     */
    classifyTrajectory(trajectory: Trajectory): Nullable<string>;
}

}
declare module "babylonjs/Misc/typeStore" {
/**
 * @internal
 */
export function RegisterClass(className: string, type: Object): void;
/**
 * @internal
 */
export function GetClass(fqdn: string): any;

}
declare module "babylonjs/Misc/uniqueIdGenerator" {
/**
 * Helper class used to generate session unique ID
 */
export class UniqueIdGenerator {
    private static _UniqueIdCounter;
    /**
     * Gets an unique (relatively to the current scene) Id
     */
    static get UniqueId(): number;
}

}
declare module "babylonjs/Misc/videoRecorder" {
import { Nullable } from "babylonjs/types";
import { Engine } from "babylonjs/Engines/engine";
/**
 * This represents the different options available for the video capture.
 */
export interface VideoRecorderOptions {
    /** Defines the mime type of the video. */
    mimeType: string;
    /** Defines the FPS the video should be recorded at. */
    fps: number;
    /** Defines the chunk size for the recording data. */
    recordChunckSize: number;
    /** The audio tracks to attach to the recording. */
    audioTracks?: MediaStreamTrack[];
}
/**
 * This can help with recording videos from BabylonJS.
 * This is based on the available WebRTC functionalities of the browser.
 *
 * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/renderToVideo
 */
export class VideoRecorder {
    private static readonly _DefaultOptions;
    /**
     * Returns whether or not the VideoRecorder is available in your browser.
     * @param engine Defines the Babylon Engine.
     * @returns true if supported otherwise false.
     */
    static IsSupported(engine: Engine): boolean;
    private readonly _options;
    private _canvas;
    private _mediaRecorder;
    private _recordedChunks;
    private _fileName;
    private _resolve;
    private _reject;
    /**
     * True when a recording is already in progress.
     */
    get isRecording(): boolean;
    /**
     * Create a new VideoCapture object which can help converting what you see in Babylon to a video file.
     * @param engine Defines the BabylonJS Engine you wish to record.
     * @param options Defines options that can be used to customize the capture.
     */
    constructor(engine: Engine, options?: Partial<VideoRecorderOptions>);
    /**
     * Stops the current recording before the default capture timeout passed in the startRecording function.
     */
    stopRecording(): void;
    /**
     * Starts recording the canvas for a max duration specified in parameters.
     * @param fileName Defines the name of the file to be downloaded when the recording stop.
     * If null no automatic download will start and you can rely on the promise to get the data back.
     * @param maxDuration Defines the maximum recording time in seconds.
     * It defaults to 7 seconds. A value of zero will not stop automatically, you would need to call stopRecording manually.
     * @returns A promise callback at the end of the recording with the video data in Blob.
     */
    startRecording(fileName?: Nullable<string>, maxDuration?: number): Promise<Blob>;
    /**
     * Releases internal resources used during the recording.
     */
    dispose(): void;
    private _handleDataAvailable;
    private _handleError;
    private _handleStop;
}

}
declare module "babylonjs/Misc/virtualJoystick" {
import { Nullable } from "babylonjs/types";
import { Vector3 } from "babylonjs/Maths/math.vector";
/**
 * Defines the potential axis of a Joystick
 */
export enum JoystickAxis {
    /** X axis */
    X = 0,
    /** Y axis */
    Y = 1,
    /** Z axis */
    Z = 2
}
/**
 * Represents the different customization options available
 * for VirtualJoystick
 */
interface VirtualJoystickCustomizations {
    /**
     * Size of the joystick's puck
     */
    puckSize: number;
    /**
     * Size of the joystick's container
     */
    containerSize: number;
    /**
     * Color of the joystick && puck
     */
    color: string;
    /**
     * Image URL for the joystick's puck
     */
    puckImage?: string;
    /**
     * Image URL for the joystick's container
     */
    containerImage?: string;
    /**
     * Defines the unmoving position of the joystick container
     */
    position?: {
        x: number;
        y: number;
    };
    /**
     * Defines whether or not the joystick container is always visible
     */
    alwaysVisible: boolean;
    /**
     * Defines whether or not to limit the movement of the puck to the joystick's container
     */
    limitToContainer: boolean;
}
/**
 * Class used to define virtual joystick (used in touch mode)
 */
export class VirtualJoystick {
    /**
     * Gets or sets a boolean indicating that left and right values must be inverted
     */
    reverseLeftRight: boolean;
    /**
     * Gets or sets a boolean indicating that up and down values must be inverted
     */
    reverseUpDown: boolean;
    /**
     * Gets the offset value for the position (ie. the change of the position value)
     */
    deltaPosition: Vector3;
    /**
     * Gets a boolean indicating if the virtual joystick was pressed
     */
    pressed: boolean;
    /**
     * Canvas the virtual joystick will render onto, default z-index of this is 5
     */
    static Canvas: Nullable<HTMLCanvasElement>;
    /**
     * boolean indicating whether or not the joystick's puck's movement should be limited to the joystick's container area
     */
    limitToContainer: boolean;
    private static _GlobalJoystickIndex;
    private static _AlwaysVisibleSticks;
    private static _VJCanvasContext;
    private static _VJCanvasWidth;
    private static _VJCanvasHeight;
    private static _HalfWidth;
    private static _GetDefaultOptions;
    private _action;
    private _axisTargetedByLeftAndRight;
    private _axisTargetedByUpAndDown;
    private _joystickSensibility;
    private _inversedSensibility;
    private _joystickPointerId;
    private _joystickColor;
    private _joystickPointerPos;
    private _joystickPreviousPointerPos;
    private _joystickPointerStartPos;
    private _deltaJoystickVector;
    private _leftJoystick;
    private _touches;
    private _joystickPosition;
    private _alwaysVisible;
    private _puckImage;
    private _containerImage;
    private _released;
    private _joystickPuckSize;
    private _joystickContainerSize;
    private _clearPuckSize;
    private _clearContainerSize;
    private _clearPuckSizeOffset;
    private _clearContainerSizeOffset;
    private _onPointerDownHandlerRef;
    private _onPointerMoveHandlerRef;
    private _onPointerUpHandlerRef;
    private _onResize;
    /**
     * Creates a new virtual joystick
     * @param leftJoystick defines that the joystick is for left hand (false by default)
     * @param customizations Defines the options we want to customize the VirtualJoystick
     */
    constructor(leftJoystick?: boolean, customizations?: Partial<VirtualJoystickCustomizations>);
    /**
     * Defines joystick sensibility (ie. the ratio between a physical move and virtual joystick position change)
     * @param newJoystickSensibility defines the new sensibility
     */
    setJoystickSensibility(newJoystickSensibility: number): void;
    private _onPointerDown;
    private _onPointerMove;
    private _onPointerUp;
    /**
     * Change the color of the virtual joystick
     * @param newColor a string that must be a CSS color value (like "red") or the hexa value (like "#FF0000")
     */
    setJoystickColor(newColor: string): void;
    /**
     * Size of the joystick's container
     */
    set containerSize(newSize: number);
    get containerSize(): number;
    /**
     * Size of the joystick's puck
     */
    set puckSize(newSize: number);
    get puckSize(): number;
    /**
     * Clears the set position of the joystick
     */
    clearPosition(): void;
    /**
     * Defines whether or not the joystick container is always visible
     */
    set alwaysVisible(value: boolean);
    get alwaysVisible(): boolean;
    /**
     * Sets the constant position of the Joystick container
     * @param x X axis coordinate
     * @param y Y axis coordinate
     */
    setPosition(x: number, y: number): void;
    /**
     * Defines a callback to call when the joystick is touched
     * @param action defines the callback
     */
    setActionOnTouch(action: () => any): void;
    /**
     * Defines which axis you'd like to control for left & right
     * @param axis defines the axis to use
     */
    setAxisForLeftRight(axis: JoystickAxis): void;
    /**
     * Defines which axis you'd like to control for up & down
     * @param axis defines the axis to use
     */
    setAxisForUpDown(axis: JoystickAxis): void;
    /**
     * Clears the canvas from the previous puck / container draw
     */
    private _clearPreviousDraw;
    /**
     * Loads `urlPath` to be used for the container's image
     * @param urlPath defines the urlPath of an image to use
     */
    setContainerImage(urlPath: string): void;
    /**
     * Loads `urlPath` to be used for the puck's image
     * @param urlPath defines the urlPath of an image to use
     */
    setPuckImage(urlPath: string): void;
    /**
     * Draws the Virtual Joystick's container
     */
    private _drawContainer;
    /**
     * Draws the Virtual Joystick's puck
     */
    private _drawPuck;
    private _drawVirtualJoystick;
    /**
     * Release internal HTML canvas
     */
    releaseCanvas(): void;
}
export {};

}
declare module "babylonjs/Misc/webRequest" {
import { IWebRequest } from "babylonjs/Misc/interfaces/iWebRequest";
import { Nullable } from "babylonjs/types";
/**
 * Extended version of XMLHttpRequest with support for customizations (headers, ...)
 */
export class WebRequest implements IWebRequest {
    private readonly _xhr;
    /**
     * Custom HTTP Request Headers to be sent with XMLHttpRequests
     * i.e. when loading files, where the server/service expects an Authorization header
     */
    static CustomRequestHeaders: {
        [key: string]: string;
    };
    /**
     * Add callback functions in this array to update all the requests before they get sent to the network
     */
    static CustomRequestModifiers: ((request: XMLHttpRequest, url: string) => void)[];
    static SkipRequestModificationForBabylonCDN: boolean;
    private _requestURL;
    private _injectCustomRequestHeaders;
    private _shouldSkipRequestModifications;
    /**
     * Gets or sets a function to be called when loading progress changes
     */
    get onprogress(): ((this: XMLHttpRequest, ev: ProgressEvent) => any) | null;
    set onprogress(value: ((this: XMLHttpRequest, ev: ProgressEvent) => any) | null);
    /**
     * Returns client's state
     */
    get readyState(): number;
    /**
     * Returns client's status
     */
    get status(): number;
    /**
     * Returns client's status as a text
     */
    get statusText(): string;
    /**
     * Returns client's response
     */
    get response(): any;
    /**
     * Returns client's response url
     */
    get responseURL(): string;
    /**
     * Returns client's response as text
     */
    get responseText(): string;
    /**
     * Gets or sets the expected response type
     */
    get responseType(): XMLHttpRequestResponseType;
    set responseType(value: XMLHttpRequestResponseType);
    /**
     * Gets or sets the timeout value in milliseconds
     */
    get timeout(): number;
    set timeout(value: number);
    /** @internal */
    addEventListener<K extends keyof XMLHttpRequestEventMap>(type: K, listener: (this: XMLHttpRequest, ev: XMLHttpRequestEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;
    /** @internal */
    removeEventListener<K extends keyof XMLHttpRequestEventMap>(type: K, listener: (this: XMLHttpRequest, ev: XMLHttpRequestEventMap[K]) => any, options?: boolean | EventListenerOptions): void;
    /**
     * Cancels any network activity
     */
    abort(): void;
    /**
     * Initiates the request. The optional argument provides the request body. The argument is ignored if request method is GET or HEAD
     * @param body defines an optional request body
     */
    send(body?: Document | XMLHttpRequestBodyInit | null): void;
    /**
     * Sets the request method, request URL
     * @param method defines the method to use (GET, POST, etc..)
     * @param url defines the url to connect with
     */
    open(method: string, url: string): void;
    /**
     * Sets the value of a request header.
     * @param name The name of the header whose value is to be set
     * @param value The value to set as the body of the header
     */
    setRequestHeader(name: string, value: string): void;
    /**
     * Get the string containing the text of a particular header's value.
     * @param name The name of the header
     * @returns The string containing the text of the given header name
     */
    getResponseHeader(name: string): Nullable<string>;
}

}
declare module "babylonjs/Misc/workerPool" {
import { IDisposable } from "babylonjs/scene";
/** @ignore */
interface WorkerInfo {
    workerPromise: Promise<Worker>;
    idle: boolean;
    timeoutId?: ReturnType<typeof setTimeout>;
}
/**
 * Helper class to push actions to a pool of workers.
 */
export class WorkerPool implements IDisposable {
    protected _workerInfos: Array<WorkerInfo>;
    protected _pendingActions: ((worker: Worker, onComplete: () => void) => void)[];
    /**
     * Constructor
     * @param workers Array of workers to use for actions
     */
    constructor(workers: Array<Worker>);
    /**
     * Terminates all workers and clears any pending actions.
     */
    dispose(): void;
    /**
     * Pushes an action to the worker pool. If all the workers are active, the action will be
     * pended until a worker has completed its action.
     * @param action The action to perform. Call onComplete when the action is complete.
     */
    push(action: (worker: Worker, onComplete: () => void) => void): void;
    protected _executeOnIdleWorker(action: (worker: Worker, onComplete: () => void) => void): boolean;
    protected _execute(workerInfo: WorkerInfo, action: (worker: Worker, onComplete: () => void) => void): void;
}
/**
 * Options for AutoReleaseWorkerPool
 */
export interface AutoReleaseWorkerPoolOptions {
    /**
     * Idle time elapsed before workers are terminated.
     */
    idleTimeElapsedBeforeRelease: number;
}
/**
 * Similar to the WorkerPool class except it creates and destroys workers automatically with a maximum of `maxWorkers` workers.
 * Workers are terminated when it is idle for at least `idleTimeElapsedBeforeRelease` milliseconds.
 */
export class AutoReleaseWorkerPool extends WorkerPool {
    /**
     * Default options for the constructor.
     * Override to change the defaults.
     */
    static DefaultOptions: AutoReleaseWorkerPoolOptions;
    private readonly _maxWorkers;
    private readonly _createWorkerAsync;
    private readonly _options;
    constructor(maxWorkers: number, createWorkerAsync: () => Promise<Worker>, options?: AutoReleaseWorkerPoolOptions);
    push(action: (worker: Worker, onComplete: () => void) => void): void;
    protected _execute(workerInfo: WorkerInfo, action: (worker: Worker, onComplete: () => void) => void): void;
}
export {};

}
declare module "babylonjs/Morph/index" {
export * from "babylonjs/Morph/morphTarget";
export * from "babylonjs/Morph/morphTargetManager";

}
declare module "babylonjs/Morph/morphTarget" {
import { IAnimatable } from "babylonjs/Animations/animatable.interface";
import { Observable } from "babylonjs/Misc/observable";
import { Nullable, FloatArray } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { AnimationPropertiesOverride } from "babylonjs/Animations/animationPropertiesOverride";
/**
 * Defines a target to use with MorphTargetManager
 * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/morphTargets
 */
export class MorphTarget implements IAnimatable {
    /** defines the name of the target */
    name: string;
    /**
     * Gets or sets the list of animations
     */
    animations: import("babylonjs/Animations/animation").Animation[];
    private _scene;
    private _positions;
    private _normals;
    private _tangents;
    private _uvs;
    private _influence;
    private _uniqueId;
    /**
     * Observable raised when the influence changes
     */
    onInfluenceChanged: Observable<boolean>;
    /** @internal */
    _onDataLayoutChanged: Observable<void>;
    /**
     * Gets or sets the influence of this target (ie. its weight in the overall morphing)
     */
    get influence(): number;
    set influence(influence: number);
    /**
     * Gets or sets the id of the morph Target
     */
    id: string;
    private _animationPropertiesOverride;
    /**
     * Gets or sets the animation properties override
     */
    get animationPropertiesOverride(): Nullable<AnimationPropertiesOverride>;
    set animationPropertiesOverride(value: Nullable<AnimationPropertiesOverride>);
    /**
     * Creates a new MorphTarget
     * @param name defines the name of the target
     * @param influence defines the influence to use
     * @param scene defines the scene the morphtarget belongs to
     */
    constructor(
    /** defines the name of the target */
    name: string, influence?: number, scene?: Nullable<Scene>);
    /**
     * Gets the unique ID of this manager
     */
    get uniqueId(): number;
    /**
     * Gets a boolean defining if the target contains position data
     */
    get hasPositions(): boolean;
    /**
     * Gets a boolean defining if the target contains normal data
     */
    get hasNormals(): boolean;
    /**
     * Gets a boolean defining if the target contains tangent data
     */
    get hasTangents(): boolean;
    /**
     * Gets a boolean defining if the target contains texture coordinates data
     */
    get hasUVs(): boolean;
    /**
     * Affects position data to this target
     * @param data defines the position data to use
     */
    setPositions(data: Nullable<FloatArray>): void;
    /**
     * Gets the position data stored in this target
     * @returns a FloatArray containing the position data (or null if not present)
     */
    getPositions(): Nullable<FloatArray>;
    /**
     * Affects normal data to this target
     * @param data defines the normal data to use
     */
    setNormals(data: Nullable<FloatArray>): void;
    /**
     * Gets the normal data stored in this target
     * @returns a FloatArray containing the normal data (or null if not present)
     */
    getNormals(): Nullable<FloatArray>;
    /**
     * Affects tangent data to this target
     * @param data defines the tangent data to use
     */
    setTangents(data: Nullable<FloatArray>): void;
    /**
     * Gets the tangent data stored in this target
     * @returns a FloatArray containing the tangent data (or null if not present)
     */
    getTangents(): Nullable<FloatArray>;
    /**
     * Affects texture coordinates data to this target
     * @param data defines the texture coordinates data to use
     */
    setUVs(data: Nullable<FloatArray>): void;
    /**
     * Gets the texture coordinates data stored in this target
     * @returns a FloatArray containing the texture coordinates data (or null if not present)
     */
    getUVs(): Nullable<FloatArray>;
    /**
     * Clone the current target
     * @returns a new MorphTarget
     */
    clone(): MorphTarget;
    /**
     * Serializes the current target into a Serialization object
     * @returns the serialized object
     */
    serialize(): any;
    /**
     * Returns the string "MorphTarget"
     * @returns "MorphTarget"
     */
    getClassName(): string;
    /**
     * Creates a new target from serialized data
     * @param serializationObject defines the serialized data to use
     * @param scene defines the hosting scene
     * @returns a new MorphTarget
     */
    static Parse(serializationObject: any, scene?: Scene): MorphTarget;
    /**
     * Creates a MorphTarget from mesh data
     * @param mesh defines the source mesh
     * @param name defines the name to use for the new target
     * @param influence defines the influence to attach to the target
     * @returns a new MorphTarget
     */
    static FromMesh(mesh: AbstractMesh, name?: string, influence?: number): MorphTarget;
}

}
declare module "babylonjs/Morph/morphTargetManager" {
import { Nullable } from "babylonjs/types";
import { IDisposable, Scene } from "babylonjs/scene";
import { MorphTarget } from "babylonjs/Morph/morphTarget";
import { Effect } from "babylonjs/Materials/effect";
import { RawTexture2DArray } from "babylonjs/Materials/Textures/rawTexture2DArray";
import { AbstractScene } from "babylonjs/abstractScene";
/**
 * This class is used to deform meshes using morphing between different targets
 * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/morphTargets
 */
export class MorphTargetManager implements IDisposable {
    /** Enable storing morph target data into textures when set to true (true by default) */
    static EnableTextureStorage: boolean;
    /** Maximum number of active morph targets supported in the "vertex attribute" mode (i.e., not the "texture" mode) */
    static MaxActiveMorphTargetsInVertexAttributeMode: number;
    private _targets;
    private _targetInfluenceChangedObservers;
    private _targetDataLayoutChangedObservers;
    private _activeTargets;
    private _scene;
    private _influences;
    private _morphTargetTextureIndices;
    private _supportsNormals;
    private _supportsTangents;
    private _supportsUVs;
    private _vertexCount;
    private _textureVertexStride;
    private _textureWidth;
    private _textureHeight;
    private _uniqueId;
    private _tempInfluences;
    private _canUseTextureForTargets;
    private _blockCounter;
    /** @internal */
    _parentContainer: Nullable<AbstractScene>;
    /** @internal */
    _targetStoreTexture: Nullable<RawTexture2DArray>;
    /**
     * Gets or sets a boolean indicating if influencers must be optimized (eg. recompiling the shader if less influencers are used)
     */
    optimizeInfluencers: boolean;
    /**
     * Gets or sets a boolean indicating if normals must be morphed
     */
    enableNormalMorphing: boolean;
    /**
     * Gets or sets a boolean indicating if tangents must be morphed
     */
    enableTangentMorphing: boolean;
    /**
     * Gets or sets a boolean indicating if UV must be morphed
     */
    enableUVMorphing: boolean;
    /**
     * Sets a boolean indicating that adding new target or updating an existing target will not update the underlying data buffers
     */
    set areUpdatesFrozen(block: boolean);
    get areUpdatesFrozen(): boolean;
    /**
     * Creates a new MorphTargetManager
     * @param scene defines the current scene
     */
    constructor(scene?: Nullable<Scene>);
    /**
     * Gets the unique ID of this manager
     */
    get uniqueId(): number;
    /**
     * Gets the number of vertices handled by this manager
     */
    get vertexCount(): number;
    /**
     * Gets a boolean indicating if this manager supports morphing of normals
     */
    get supportsNormals(): boolean;
    /**
     * Gets a boolean indicating if this manager supports morphing of tangents
     */
    get supportsTangents(): boolean;
    /**
     * Gets a boolean indicating if this manager supports morphing of texture coordinates
     */
    get supportsUVs(): boolean;
    /**
     * Gets the number of targets stored in this manager
     */
    get numTargets(): number;
    /**
     * Gets the number of influencers (ie. the number of targets with influences > 0)
     */
    get numInfluencers(): number;
    /**
     * Gets the list of influences (one per target)
     */
    get influences(): Float32Array;
    private _useTextureToStoreTargets;
    /**
     * Gets or sets a boolean indicating that targets should be stored as a texture instead of using vertex attributes (default is true).
     * Please note that this option is not available if the hardware does not support it
     */
    get useTextureToStoreTargets(): boolean;
    set useTextureToStoreTargets(value: boolean);
    /**
     * Gets a boolean indicating that the targets are stored into a texture (instead of as attributes)
     */
    get isUsingTextureForTargets(): boolean;
    /**
     * Gets the active target at specified index. An active target is a target with an influence > 0
     * @param index defines the index to check
     * @returns the requested target
     */
    getActiveTarget(index: number): MorphTarget;
    /**
     * Gets the target at specified index
     * @param index defines the index to check
     * @returns the requested target
     */
    getTarget(index: number): MorphTarget;
    /**
     * Add a new target to this manager
     * @param target defines the target to add
     */
    addTarget(target: MorphTarget): void;
    /**
     * Removes a target from the manager
     * @param target defines the target to remove
     */
    removeTarget(target: MorphTarget): void;
    /**
     * @internal
     */
    _bind(effect: Effect): void;
    /**
     * Clone the current manager
     * @returns a new MorphTargetManager
     */
    clone(): MorphTargetManager;
    /**
     * Serializes the current manager into a Serialization object
     * @returns the serialized object
     */
    serialize(): any;
    private _syncActiveTargets;
    /**
     * Synchronize the targets with all the meshes using this morph target manager
     */
    synchronize(): void;
    /**
     * Release all resources
     */
    dispose(): void;
    /**
     * Creates a new MorphTargetManager from serialized data
     * @param serializationObject defines the serialized data
     * @param scene defines the hosting scene
     * @returns the new MorphTargetManager
     */
    static Parse(serializationObject: any, scene: Scene): MorphTargetManager;
}

}
declare module "babylonjs/Navigation/INavigationEngine" {
import { TransformNode } from "babylonjs/Meshes/transformNode";
import { Vector3 } from "babylonjs/Maths/math";
import { Mesh } from "babylonjs/Meshes/mesh";
import { Scene } from "babylonjs/scene";
/**
 * Navigation plugin interface to add navigation constrained by a navigation mesh
 */
export interface INavigationEnginePlugin {
    /**
     * plugin name
     */
    name: string;
    /**
     * Creates a navigation mesh
     * @param meshes array of all the geometry used to compute the navigation mesh
     * @param parameters bunch of parameters used to filter geometry
     */
    createNavMesh(meshes: Array<Mesh>, parameters: INavMeshParameters): void;
    /**
     * Create a navigation mesh debug mesh
     * @param scene is where the mesh will be added
     * @returns debug display mesh
     */
    createDebugNavMesh(scene: Scene): Mesh;
    /**
     * Get a navigation mesh constrained position, closest to the parameter position
     * @param position world position
     * @returns the closest point to position constrained by the navigation mesh
     */
    getClosestPoint(position: Vector3): Vector3;
    /**
     * Get a navigation mesh constrained position, closest to the parameter position
     * @param position world position
     * @param result output the closest point to position constrained by the navigation mesh
     */
    getClosestPointToRef(position: Vector3, result: Vector3): void;
    /**
     * Get a navigation mesh constrained position, within a particular radius
     * @param position world position
     * @param maxRadius the maximum distance to the constrained world position
     * @returns the closest point to position constrained by the navigation mesh
     */
    getRandomPointAround(position: Vector3, maxRadius: number): Vector3;
    /**
     * Get a navigation mesh constrained position, within a particular radius
     * @param position world position
     * @param maxRadius the maximum distance to the constrained world position
     * @param result output the closest point to position constrained by the navigation mesh
     */
    getRandomPointAroundToRef(position: Vector3, maxRadius: number, result: Vector3): void;
    /**
     * Compute the final position from a segment made of destination-position
     * @param position world position
     * @param destination world position
     * @returns the resulting point along the navmesh
     */
    moveAlong(position: Vector3, destination: Vector3): Vector3;
    /**
     * Compute the final position from a segment made of destination-position
     * @param position world position
     * @param destination world position
     * @param result output the resulting point along the navmesh
     */
    moveAlongToRef(position: Vector3, destination: Vector3, result: Vector3): void;
    /**
     * Compute a navigation path from start to end. Returns an empty array if no path can be computed
     * @param start world position
     * @param end world position
     * @returns array containing world position composing the path
     */
    computePath(start: Vector3, end: Vector3): Vector3[];
    /**
     * If this plugin is supported
     * @returns true if plugin is supported
     */
    isSupported(): boolean;
    /**
     * Create a new Crowd so you can add agents
     * @param maxAgents the maximum agent count in the crowd
     * @param maxAgentRadius the maximum radius an agent can have
     * @param scene to attach the crowd to
     * @returns the crowd you can add agents to
     */
    createCrowd(maxAgents: number, maxAgentRadius: number, scene: Scene): ICrowd;
    /**
     * Set the Bounding box extent for doing spatial queries (getClosestPoint, getRandomPointAround, ...)
     * The queries will try to find a solution within those bounds
     * default is (1,1,1)
     * @param extent x,y,z value that define the extent around the queries point of reference
     */
    setDefaultQueryExtent(extent: Vector3): void;
    /**
     * Get the Bounding box extent specified by setDefaultQueryExtent
     * @returns the box extent values
     */
    getDefaultQueryExtent(): Vector3;
    /**
     * build the navmesh from a previously saved state using getNavmeshData
     * @param data the Uint8Array returned by getNavmeshData
     */
    buildFromNavmeshData(data: Uint8Array): void;
    /**
     * returns the navmesh data that can be used later. The navmesh must be built before retrieving the data
     * @returns data the Uint8Array that can be saved and reused
     */
    getNavmeshData(): Uint8Array;
    /**
     * Get the Bounding box extent result specified by setDefaultQueryExtent
     * @param result output the box extent values
     */
    getDefaultQueryExtentToRef(result: Vector3): void;
    /**
     * Set the time step of the navigation tick update.
     * Default is 1/60.
     * A value of 0 will disable fixed time update
     * @param newTimeStep the new timestep to apply to this world.
     */
    setTimeStep(newTimeStep: number): void;
    /**
     * Get the time step of the navigation tick update.
     * @returns the current time step
     */
    getTimeStep(): number;
    /**
     * If delta time in navigation tick update is greater than the time step
     * a number of sub iterations are done. If more iterations are need to reach deltatime
     * they will be discarded.
     * A value of 0 will set to no maximum and update will use as many substeps as needed
     * @param newStepCount the maximum number of iterations
     */
    setMaximumSubStepCount(newStepCount: number): void;
    /**
     * Get the maximum number of iterations per navigation tick update
     * @returns the maximum number of iterations
     */
    getMaximumSubStepCount(): number;
    /**
     * Creates a cylinder obstacle and add it to the navigation
     * @param position world position
     * @param radius cylinder radius
     * @param height cylinder height
     * @returns the obstacle freshly created
     */
    addCylinderObstacle(position: Vector3, radius: number, height: number): IObstacle;
    /**
     * Creates an oriented box obstacle and add it to the navigation
     * @param position world position
     * @param extent box size
     * @param angle angle in radians of the box orientation on Y axis
     * @returns the obstacle freshly created
     */
    addBoxObstacle(position: Vector3, extent: Vector3, angle: number): IObstacle;
    /**
     * Removes an obstacle created by addCylinderObstacle or addBoxObstacle
     * @param obstacle obstacle to remove from the navigation
     */
    removeObstacle(obstacle: IObstacle): void;
    /**
     * Release all resources
     */
    dispose(): void;
}
/**
 * Obstacle interface
 */
export interface IObstacle {
}
/**
 * Crowd Interface. A Crowd is a collection of moving agents constrained by a navigation mesh
 */
export interface ICrowd {
    /**
     * Add a new agent to the crowd with the specified parameter a corresponding transformNode.
     * You can attach anything to that node. The node position is updated in the scene update tick.
     * @param pos world position that will be constrained by the navigation mesh
     * @param parameters agent parameters
     * @param transform hooked to the agent that will be update by the scene
     * @returns agent index
     */
    addAgent(pos: Vector3, parameters: IAgentParameters, transform: TransformNode): number;
    /**
     * Returns the agent position in world space
     * @param index agent index returned by addAgent
     * @returns world space position
     */
    getAgentPosition(index: number): Vector3;
    /**
     * Gets the agent position result in world space
     * @param index agent index returned by addAgent
     * @param result output world space position
     */
    getAgentPositionToRef(index: number, result: Vector3): void;
    /**
     * Gets the agent velocity in world space
     * @param index agent index returned by addAgent
     * @returns world space velocity
     */
    getAgentVelocity(index: number): Vector3;
    /**
     * Gets the agent velocity result in world space
     * @param index agent index returned by addAgent
     * @param result output world space velocity
     */
    getAgentVelocityToRef(index: number, result: Vector3): void;
    /**
     * Gets the agent next target point on the path
     * @param index agent index returned by addAgent
     * @returns world space position
     */
    getAgentNextTargetPath(index: number): Vector3;
    /**
     * Gets the agent state
     * @param index agent index returned by addAgent
     * @returns agent state
     */
    getAgentState(index: number): number;
    /**
     * returns true if the agent in over an off mesh link connection
     * @param index agent index returned by addAgent
     * @returns true if over an off mesh link connection
     */
    overOffmeshConnection(index: number): boolean;
    /**
     * Gets the agent next target point on the path
     * @param index agent index returned by addAgent
     * @param result output world space position
     */
    getAgentNextTargetPathToRef(index: number, result: Vector3): void;
    /**
     * remove a particular agent previously created
     * @param index agent index returned by addAgent
     */
    removeAgent(index: number): void;
    /**
     * get the list of all agents attached to this crowd
     * @returns list of agent indices
     */
    getAgents(): number[];
    /**
     * Tick update done by the Scene. Agent position/velocity/acceleration is updated by this function
     * @param deltaTime in seconds
     */
    update(deltaTime: number): void;
    /**
     * Asks a particular agent to go to a destination. That destination is constrained by the navigation mesh
     * @param index agent index returned by addAgent
     * @param destination targeted world position
     */
    agentGoto(index: number, destination: Vector3): void;
    /**
     * Teleport the agent to a new position
     * @param index agent index returned by addAgent
     * @param destination targeted world position
     */
    agentTeleport(index: number, destination: Vector3): void;
    /**
     * Update agent parameters
     * @param index agent index returned by addAgent
     * @param parameters agent parameters
     */
    updateAgentParameters(index: number, parameters: IAgentParameters): void;
    /**
     * Set the Bounding box extent for doing spatial queries (getClosestPoint, getRandomPointAround, ...)
     * The queries will try to find a solution within those bounds
     * default is (1,1,1)
     * @param extent x,y,z value that define the extent around the queries point of reference
     */
    setDefaultQueryExtent(extent: Vector3): void;
    /**
     * Get the Bounding box extent specified by setDefaultQueryExtent
     * @returns the box extent values
     */
    getDefaultQueryExtent(): Vector3;
    /**
     * Get the Bounding box extent result specified by setDefaultQueryExtent
     * @param result output the box extent values
     */
    getDefaultQueryExtentToRef(result: Vector3): void;
    /**
     * Get the next corner points composing the path (max 4 points)
     * @param index agent index returned by addAgent
     * @returns array containing world position composing the path
     */
    getCorners(index: number): Vector3[];
    /**
     * Release all resources
     */
    dispose(): void;
}
/**
 * Configures an agent
 */
export interface IAgentParameters {
    /**
     *  Agent radius. [Limit: >= 0]
     */
    radius: number;
    /**
     * Agent height. [Limit: > 0]
     */
    height: number;
    /**
     *  Maximum allowed acceleration. [Limit: >= 0]
     */
    maxAcceleration: number;
    /**
     * Maximum allowed speed. [Limit: >= 0]
     */
    maxSpeed: number;
    /**
     * Defines how close a collision element must be before it is considered for steering behaviors. [Limits: > 0]
     */
    collisionQueryRange: number;
    /**
     * The path visibility optimization range. [Limit: > 0]
     */
    pathOptimizationRange: number;
    /**
     * How aggressive the agent manager should be at avoiding collisions with this agent. [Limit: >= 0]
     */
    separationWeight: number;
    /**
     * Observers will be notified when agent gets inside the virtual circle with this Radius around destination point.
     * Default is agent radius
     */
    reachRadius?: number;
}
/**
 * Configures the navigation mesh creation
 */
export interface INavMeshParameters {
    /**
     * The xz-plane cell size to use for fields. [Limit: > 0] [Units: wu]
     */
    cs: number;
    /**
     * The y-axis cell size to use for fields. [Limit: > 0] [Units: wu]
     */
    ch: number;
    /**
     * The maximum slope that is considered walkable. [Limits: 0 <= value < 90] [Units: Degrees]
     */
    walkableSlopeAngle: number;
    /**
     * Minimum floor to 'ceiling' height that will still allow the floor area to
     * be considered walkable. [Limit: >= 3] [Units: vx]
     */
    walkableHeight: number;
    /**
     * Maximum ledge height that is considered to still be traversable. [Limit: >=0] [Units: vx]
     */
    walkableClimb: number;
    /**
     * The distance to erode/shrink the walkable area of the heightfield away from
     * obstructions.  [Limit: >=0] [Units: vx]
     */
    walkableRadius: number;
    /**
     * The maximum allowed length for contour edges along the border of the mesh. [Limit: >=0] [Units: vx]
     */
    maxEdgeLen: number;
    /**
     * The maximum distance a simplified contour's border edges should deviate
     * the original raw contour. [Limit: >=0] [Units: vx]
     */
    maxSimplificationError: number;
    /**
     * The minimum number of cells allowed to form isolated island areas. [Limit: >=0] [Units: vx]
     */
    minRegionArea: number;
    /**
     * Any regions with a span count smaller than this value will, if possible,
     * be merged with larger regions. [Limit: >=0] [Units: vx]
     */
    mergeRegionArea: number;
    /**
     * The maximum number of vertices allowed for polygons generated during the
     * contour to polygon conversion process. [Limit: >= 3]
     */
    maxVertsPerPoly: number;
    /**
     * Sets the sampling distance to use when generating the detail mesh.
     * (For height detail only.) [Limits: 0 or >= 0.9] [Units: wu]
     */
    detailSampleDist: number;
    /**
     * The maximum distance the detail mesh surface should deviate from heightfield
     * data. (For height detail only.) [Limit: >=0] [Units: wu]
     */
    detailSampleMaxError: number;
    /**
     * If using obstacles, the navmesh must be subdivided internaly by tiles.
     * This member defines the tile cube side length in world units.
     * If no obstacles are needed, leave it undefined or 0.
     */
    tileSize?: number;
    /**
     * The size of the non-navigable border around the heightfield.
     */
    borderSize?: number;
}

}
declare module "babylonjs/Navigation/index" {
export * from "babylonjs/Navigation/INavigationEngine";
export * from "babylonjs/Navigation/Plugins/index";

}
declare module "babylonjs/Navigation/Plugins/index" {
export * from "babylonjs/Navigation/Plugins/recastJSPlugin";

}
declare module "babylonjs/Navigation/Plugins/recastJSPlugin" {
import { INavigationEnginePlugin, ICrowd, IAgentParameters, INavMeshParameters, IObstacle } from "babylonjs/Navigation/INavigationEngine";
import { Mesh } from "babylonjs/Meshes/mesh";
import { Scene } from "babylonjs/scene";
import { Vector3 } from "babylonjs/Maths/math";
import { TransformNode } from "babylonjs/Meshes/transformNode";
import { Observable } from "babylonjs/Misc/observable";
/**
 * RecastJS navigation plugin
 */
export class RecastJSPlugin implements INavigationEnginePlugin {
    /**
     * Reference to the Recast library
     */
    bjsRECAST: any;
    /**
     * plugin name
     */
    name: string;
    /**
     * the first navmesh created. We might extend this to support multiple navmeshes
     */
    navMesh: any;
    private _maximumSubStepCount;
    private _timeStep;
    private _timeFactor;
    private _tempVec1;
    private _tempVec2;
    private _worker;
    /**
     * Initializes the recastJS plugin
     * @param recastInjection can be used to inject your own recast reference
     */
    constructor(recastInjection?: any);
    /**
     * Set worker URL to be used when generating a new navmesh
     * @param workerURL url string
     * @returns boolean indicating if worker is created
     */
    setWorkerURL(workerURL: string): boolean;
    /**
     * Set the time step of the navigation tick update.
     * Default is 1/60.
     * A value of 0 will disable fixed time update
     * @param newTimeStep the new timestep to apply to this world.
     */
    setTimeStep(newTimeStep?: number): void;
    /**
     * Get the time step of the navigation tick update.
     * @returns the current time step
     */
    getTimeStep(): number;
    /**
     * If delta time in navigation tick update is greater than the time step
     * a number of sub iterations are done. If more iterations are need to reach deltatime
     * they will be discarded.
     * A value of 0 will set to no maximum and update will use as many substeps as needed
     * @param newStepCount the maximum number of iterations
     */
    setMaximumSubStepCount(newStepCount?: number): void;
    /**
     * Get the maximum number of iterations per navigation tick update
     * @returns the maximum number of iterations
     */
    getMaximumSubStepCount(): number;
    /**
     * Time factor applied when updating crowd agents (default 1). A value of 0 will pause crowd updates.
     * @param value the time factor applied at update
     */
    set timeFactor(value: number);
    /**
     * Get the time factor used for crowd agent update
     * @returns the time factor
     */
    get timeFactor(): number;
    /**
     * Creates a navigation mesh
     * @param meshes array of all the geometry used to compute the navigation mesh
     * @param parameters bunch of parameters used to filter geometry
     * @param completion callback when data is available from the worker. Not used without a worker
     */
    createNavMesh(meshes: Array<Mesh>, parameters: INavMeshParameters, completion?: (navmeshData: Uint8Array) => void): void;
    /**
     * Create a navigation mesh debug mesh
     * @param scene is where the mesh will be added
     * @returns debug display mesh
     */
    createDebugNavMesh(scene: Scene): Mesh;
    /**
     * Get a navigation mesh constrained position, closest to the parameter position
     * @param position world position
     * @returns the closest point to position constrained by the navigation mesh
     */
    getClosestPoint(position: Vector3): Vector3;
    /**
     * Get a navigation mesh constrained position, closest to the parameter position
     * @param position world position
     * @param result output the closest point to position constrained by the navigation mesh
     */
    getClosestPointToRef(position: Vector3, result: Vector3): void;
    /**
     * Get a navigation mesh constrained position, within a particular radius
     * @param position world position
     * @param maxRadius the maximum distance to the constrained world position
     * @returns the closest point to position constrained by the navigation mesh
     */
    getRandomPointAround(position: Vector3, maxRadius: number): Vector3;
    /**
     * Get a navigation mesh constrained position, within a particular radius
     * @param position world position
     * @param maxRadius the maximum distance to the constrained world position
     * @param result output the closest point to position constrained by the navigation mesh
     */
    getRandomPointAroundToRef(position: Vector3, maxRadius: number, result: Vector3): void;
    /**
     * Compute the final position from a segment made of destination-position
     * @param position world position
     * @param destination world position
     * @returns the resulting point along the navmesh
     */
    moveAlong(position: Vector3, destination: Vector3): Vector3;
    /**
     * Compute the final position from a segment made of destination-position
     * @param position world position
     * @param destination world position
     * @param result output the resulting point along the navmesh
     */
    moveAlongToRef(position: Vector3, destination: Vector3, result: Vector3): void;
    /**
     * Compute a navigation path from start to end. Returns an empty array if no path can be computed
     * @param start world position
     * @param end world position
     * @returns array containing world position composing the path
     */
    computePath(start: Vector3, end: Vector3): Vector3[];
    /**
     * Create a new Crowd so you can add agents
     * @param maxAgents the maximum agent count in the crowd
     * @param maxAgentRadius the maximum radius an agent can have
     * @param scene to attach the crowd to
     * @returns the crowd you can add agents to
     */
    createCrowd(maxAgents: number, maxAgentRadius: number, scene: Scene): ICrowd;
    /**
     * Set the Bounding box extent for doing spatial queries (getClosestPoint, getRandomPointAround, ...)
     * The queries will try to find a solution within those bounds
     * default is (1,1,1)
     * @param extent x,y,z value that define the extent around the queries point of reference
     */
    setDefaultQueryExtent(extent: Vector3): void;
    /**
     * Get the Bounding box extent specified by setDefaultQueryExtent
     * @returns the box extent values
     */
    getDefaultQueryExtent(): Vector3;
    /**
     * build the navmesh from a previously saved state using getNavmeshData
     * @param data the Uint8Array returned by getNavmeshData
     */
    buildFromNavmeshData(data: Uint8Array): void;
    /**
     * returns the navmesh data that can be used later. The navmesh must be built before retrieving the data
     * @returns data the Uint8Array that can be saved and reused
     */
    getNavmeshData(): Uint8Array;
    /**
     * Get the Bounding box extent result specified by setDefaultQueryExtent
     * @param result output the box extent values
     */
    getDefaultQueryExtentToRef(result: Vector3): void;
    /**
     * Disposes
     */
    dispose(): void;
    /**
     * Creates a cylinder obstacle and add it to the navigation
     * @param position world position
     * @param radius cylinder radius
     * @param height cylinder height
     * @returns the obstacle freshly created
     */
    addCylinderObstacle(position: Vector3, radius: number, height: number): IObstacle;
    /**
     * Creates an oriented box obstacle and add it to the navigation
     * @param position world position
     * @param extent box size
     * @param angle angle in radians of the box orientation on Y axis
     * @returns the obstacle freshly created
     */
    addBoxObstacle(position: Vector3, extent: Vector3, angle: number): IObstacle;
    /**
     * Removes an obstacle created by addCylinderObstacle or addBoxObstacle
     * @param obstacle obstacle to remove from the navigation
     */
    removeObstacle(obstacle: IObstacle): void;
    /**
     * If this plugin is supported
     * @returns true if plugin is supported
     */
    isSupported(): boolean;
}
/**
 * Recast detour crowd implementation
 */
export class RecastJSCrowd implements ICrowd {
    /**
     * Recast/detour plugin
     */
    bjsRECASTPlugin: RecastJSPlugin;
    /**
     * Link to the detour crowd
     */
    recastCrowd: any;
    /**
     * One transform per agent
     */
    transforms: TransformNode[];
    /**
     * All agents created
     */
    agents: number[];
    /**
     * agents reach radius
     */
    reachRadii: number[];
    /**
     * true when a destination is active for an agent and notifier hasn't been notified of reach
     */
    private _agentDestinationArmed;
    /**
     * agent current target
     */
    private _agentDestination;
    /**
     * Link to the scene is kept to unregister the crowd from the scene
     */
    private _scene;
    /**
     * Observer for crowd updates
     */
    private _onBeforeAnimationsObserver;
    /**
     * Fires each time an agent is in reach radius of its destination
     */
    onReachTargetObservable: Observable<{
        agentIndex: number;
        destination: Vector3;
    }>;
    /**
     * Constructor
     * @param plugin recastJS plugin
     * @param maxAgents the maximum agent count in the crowd
     * @param maxAgentRadius the maximum radius an agent can have
     * @param scene to attach the crowd to
     * @returns the crowd you can add agents to
     */
    constructor(plugin: RecastJSPlugin, maxAgents: number, maxAgentRadius: number, scene: Scene);
    /**
     * Add a new agent to the crowd with the specified parameter a corresponding transformNode.
     * You can attach anything to that node. The node position is updated in the scene update tick.
     * @param pos world position that will be constrained by the navigation mesh
     * @param parameters agent parameters
     * @param transform hooked to the agent that will be update by the scene
     * @returns agent index
     */
    addAgent(pos: Vector3, parameters: IAgentParameters, transform: TransformNode): number;
    /**
     * Returns the agent position in world space
     * @param index agent index returned by addAgent
     * @returns world space position
     */
    getAgentPosition(index: number): Vector3;
    /**
     * Returns the agent position result in world space
     * @param index agent index returned by addAgent
     * @param result output world space position
     */
    getAgentPositionToRef(index: number, result: Vector3): void;
    /**
     * Returns the agent velocity in world space
     * @param index agent index returned by addAgent
     * @returns world space velocity
     */
    getAgentVelocity(index: number): Vector3;
    /**
     * Returns the agent velocity result in world space
     * @param index agent index returned by addAgent
     * @param result output world space velocity
     */
    getAgentVelocityToRef(index: number, result: Vector3): void;
    /**
     * Returns the agent next target point on the path
     * @param index agent index returned by addAgent
     * @returns world space position
     */
    getAgentNextTargetPath(index: number): Vector3;
    /**
     * Returns the agent next target point on the path
     * @param index agent index returned by addAgent
     * @param result output world space position
     */
    getAgentNextTargetPathToRef(index: number, result: Vector3): void;
    /**
     * Gets the agent state
     * @param index agent index returned by addAgent
     * @returns agent state
     */
    getAgentState(index: number): number;
    /**
     * returns true if the agent in over an off mesh link connection
     * @param index agent index returned by addAgent
     * @returns true if over an off mesh link connection
     */
    overOffmeshConnection(index: number): boolean;
    /**
     * Asks a particular agent to go to a destination. That destination is constrained by the navigation mesh
     * @param index agent index returned by addAgent
     * @param destination targeted world position
     */
    agentGoto(index: number, destination: Vector3): void;
    /**
     * Teleport the agent to a new position
     * @param index agent index returned by addAgent
     * @param destination targeted world position
     */
    agentTeleport(index: number, destination: Vector3): void;
    /**
     * Update agent parameters
     * @param index agent index returned by addAgent
     * @param parameters agent parameters
     */
    updateAgentParameters(index: number, parameters: IAgentParameters): void;
    /**
     * remove a particular agent previously created
     * @param index agent index returned by addAgent
     */
    removeAgent(index: number): void;
    /**
     * get the list of all agents attached to this crowd
     * @returns list of agent indices
     */
    getAgents(): number[];
    /**
     * Tick update done by the Scene. Agent position/velocity/acceleration is updated by this function
     * @param deltaTime in seconds
     */
    update(deltaTime: number): void;
    /**
     * Set the Bounding box extent for doing spatial queries (getClosestPoint, getRandomPointAround, ...)
     * The queries will try to find a solution within those bounds
     * default is (1,1,1)
     * @param extent x,y,z value that define the extent around the queries point of reference
     */
    setDefaultQueryExtent(extent: Vector3): void;
    /**
     * Get the Bounding box extent specified by setDefaultQueryExtent
     * @returns the box extent values
     */
    getDefaultQueryExtent(): Vector3;
    /**
     * Get the Bounding box extent result specified by setDefaultQueryExtent
     * @param result output the box extent values
     */
    getDefaultQueryExtentToRef(result: Vector3): void;
    /**
     * Get the next corner points composing the path (max 4 points)
     * @param index agent index returned by addAgent
     * @returns array containing world position composing the path
     */
    getCorners(index: number): Vector3[];
    /**
     * Release all resources
     */
    dispose(): void;
}

}
declare module "babylonjs/node" {
import { Scene } from "babylonjs/scene";
import { Nullable } from "babylonjs/types";
import { Matrix, Vector3 } from "babylonjs/Maths/math.vector";
import { Engine } from "babylonjs/Engines/engine";
import { IBehaviorAware, Behavior } from "babylonjs/Behaviors/behavior";
import { Observable } from "babylonjs/Misc/observable";
import { AbstractActionManager } from "babylonjs/Actions/abstractActionManager";
import { IInspectable } from "babylonjs/Misc/iInspectable";
import { AbstractScene } from "babylonjs/abstractScene";
import { IAccessibilityTag } from "babylonjs/IAccessibilityTag";
import { Animatable } from "babylonjs/Animations/animatable";
import { AnimationPropertiesOverride } from "babylonjs/Animations/animationPropertiesOverride";
import { Animation } from "babylonjs/Animations/animation";
import { AnimationRange } from "babylonjs/Animations/animationRange";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
/**
 * Defines how a node can be built from a string name.
 */
export type NodeConstructor = (name: string, scene: Scene, options?: any) => () => Node;
/**
 * Node is the basic class for all scene objects (Mesh, Light, Camera.)
 */
export class Node implements IBehaviorAware<Node> {
    protected _isDirty: boolean;
    /**
     * @internal
     */
    static _AnimationRangeFactory: (_name: string, _from: number, _to: number) => AnimationRange;
    private static _NodeConstructors;
    /**
     * Add a new node constructor
     * @param type defines the type name of the node to construct
     * @param constructorFunc defines the constructor function
     */
    static AddNodeConstructor(type: string, constructorFunc: NodeConstructor): void;
    /**
     * Returns a node constructor based on type name
     * @param type defines the type name
     * @param name defines the new node name
     * @param scene defines the hosting scene
     * @param options defines optional options to transmit to constructors
     * @returns the new constructor or null
     */
    static Construct(type: string, name: string, scene: Scene, options?: any): Nullable<() => Node>;
    private _nodeDataStorage;
    /**
     * Gets or sets the name of the node
     */
    name: string;
    /**
     * Gets or sets the id of the node
     */
    id: string;
    /**
     * Gets or sets the unique id of the node
     */
    uniqueId: number;
    /**
     * Gets or sets a string used to store user defined state for the node
     */
    state: string;
    /**
     * Gets or sets an object used to store user defined information for the node
     */
    metadata: any;
    /** @internal */
    _internalMetadata: any;
    /**
     * For internal use only. Please do not use.
     */
    reservedDataStore: any;
    /**
     * List of inspectable custom properties (used by the Inspector)
     * @see https://doc.babylonjs.com/toolsAndResources/inspector#extensibility
     */
    inspectableCustomProperties: IInspectable[];
    /**
     * Gets or sets the accessibility tag to describe the node for accessibility purpose.
     */
    set accessibilityTag(value: Nullable<IAccessibilityTag>);
    get accessibilityTag(): Nullable<IAccessibilityTag>;
    protected _accessibilityTag: Nullable<IAccessibilityTag>;
    onAccessibilityTagChangedObservable: Observable<Nullable<IAccessibilityTag>>;
    /**
     * Gets or sets a boolean used to define if the node must be serialized
     */
    get doNotSerialize(): boolean;
    set doNotSerialize(value: boolean);
    /** @internal */
    _parentContainer: Nullable<AbstractScene>;
    /**
     * Gets a list of Animations associated with the node
     */
    animations: import("babylonjs/Animations/animation").Animation[];
    protected _ranges: {
        [name: string]: Nullable<AnimationRange>;
    };
    /**
     * Callback raised when the node is ready to be used
     */
    onReady: Nullable<(node: Node) => void>;
    /** @internal */
    _currentRenderId: number;
    private _parentUpdateId;
    /** @internal */
    _childUpdateId: number;
    /** @internal */
    _waitingParentId: Nullable<string>;
    /** @internal */
    _waitingParentInstanceIndex: Nullable<string>;
    /** @internal */
    _waitingParsedUniqueId: Nullable<number>;
    /** @internal */
    _scene: Scene;
    /** @internal */
    _cache: any;
    protected _parentNode: Nullable<Node>;
    /** @internal */
    protected _children: Nullable<Node[]>;
    /** @internal */
    _worldMatrix: Matrix;
    /** @internal */
    _worldMatrixDeterminant: number;
    /** @internal */
    _worldMatrixDeterminantIsDirty: boolean;
    /**
     * Gets a boolean indicating if the node has been disposed
     * @returns true if the node was disposed
     */
    isDisposed(): boolean;
    /**
     * Gets or sets the parent of the node (without keeping the current position in the scene)
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/transforms/parent_pivot/parent
     */
    set parent(parent: Nullable<Node>);
    get parent(): Nullable<Node>;
    /**
     * @internal
     */
    _serializeAsParent(serializationObject: any): void;
    /** @internal */
    _addToSceneRootNodes(): void;
    /** @internal */
    _removeFromSceneRootNodes(): void;
    private _animationPropertiesOverride;
    /**
     * Gets or sets the animation properties override
     */
    get animationPropertiesOverride(): Nullable<AnimationPropertiesOverride>;
    set animationPropertiesOverride(value: Nullable<AnimationPropertiesOverride>);
    /**
     * Gets a string identifying the name of the class
     * @returns "Node" string
     */
    getClassName(): string;
    /** @internal */
    readonly _isNode: boolean;
    /**
     * An event triggered when the mesh is disposed
     */
    onDisposeObservable: Observable<Node>;
    private _onDisposeObserver;
    /**
     * Sets a callback that will be raised when the node will be disposed
     */
    set onDispose(callback: () => void);
    /**
     * An event triggered when the enabled state of the node changes
     */
    get onEnabledStateChangedObservable(): Observable<boolean>;
    /**
     * An event triggered when the node is cloned
     */
    get onClonedObservable(): Observable<Node>;
    /**
     * Creates a new Node
     * @param name the name and id to be given to this node
     * @param scene the scene this node will be added to
     */
    constructor(name: string, scene?: Nullable<Scene>);
    /**
     * Gets the scene of the node
     * @returns a scene
     */
    getScene(): Scene;
    /**
     * Gets the engine of the node
     * @returns a Engine
     */
    getEngine(): Engine;
    private _behaviors;
    /**
     * Attach a behavior to the node
     * @see https://doc.babylonjs.com/features/featuresDeepDive/behaviors
     * @param behavior defines the behavior to attach
     * @param attachImmediately defines that the behavior must be attached even if the scene is still loading
     * @returns the current Node
     */
    addBehavior(behavior: Behavior<Node>, attachImmediately?: boolean): Node;
    /**
     * Remove an attached behavior
     * @see https://doc.babylonjs.com/features/featuresDeepDive/behaviors
     * @param behavior defines the behavior to attach
     * @returns the current Node
     */
    removeBehavior(behavior: Behavior<Node>): Node;
    /**
     * Gets the list of attached behaviors
     * @see https://doc.babylonjs.com/features/featuresDeepDive/behaviors
     */
    get behaviors(): Behavior<Node>[];
    /**
     * Gets an attached behavior by name
     * @param name defines the name of the behavior to look for
     * @see https://doc.babylonjs.com/features/featuresDeepDive/behaviors
     * @returns null if behavior was not found else the requested behavior
     */
    getBehaviorByName(name: string): Nullable<Behavior<Node>>;
    /**
     * Returns the latest update of the World matrix
     * @returns a Matrix
     */
    getWorldMatrix(): Matrix;
    /** @internal */
    _getWorldMatrixDeterminant(): number;
    /**
     * Returns directly the latest state of the mesh World matrix.
     * A Matrix is returned.
     */
    get worldMatrixFromCache(): Matrix;
    /** @internal */
    _initCache(): void;
    /**
     * @internal
     */
    updateCache(force?: boolean): void;
    /**
     * @internal
     */
    _getActionManagerForTrigger(trigger?: number, _initialCall?: boolean): Nullable<AbstractActionManager>;
    /**
     * @internal
     */
    _updateCache(_ignoreParentClass?: boolean): void;
    /** @internal */
    _isSynchronized(): boolean;
    /** @internal */
    _markSyncedWithParent(): void;
    /** @internal */
    isSynchronizedWithParent(): boolean;
    /** @internal */
    isSynchronized(): boolean;
    /**
     * Is this node ready to be used/rendered
     * @param _completeCheck defines if a complete check (including materials and lights) has to be done (false by default)
     * @returns true if the node is ready
     */
    isReady(_completeCheck?: boolean): boolean;
    /**
     * Flag the  node as dirty (Forcing it to update everything)
     * @param _property helps children apply precise "dirtyfication"
     * @returns this node
     */
    markAsDirty(_property?: string): Node;
    /**
     * Is this node enabled?
     * If the node has a parent, all ancestors will be checked and false will be returned if any are false (not enabled), otherwise will return true
     * @param checkAncestors indicates if this method should check the ancestors. The default is to check the ancestors. If set to false, the method will return the value of this node without checking ancestors
     * @returns whether this node (and its parent) is enabled
     */
    isEnabled(checkAncestors?: boolean): boolean;
    /** @internal */
    protected _syncParentEnabledState(): void;
    /**
     * Set the enabled state of this node
     * @param value defines the new enabled state
     */
    setEnabled(value: boolean): void;
    /**
     * Is this node a descendant of the given node?
     * The function will iterate up the hierarchy until the ancestor was found or no more parents defined
     * @param ancestor defines the parent node to inspect
     * @returns a boolean indicating if this node is a descendant of the given node
     */
    isDescendantOf(ancestor: Node): boolean;
    /**
     * @internal
     */
    _getDescendants(results: Node[], directDescendantsOnly?: boolean, predicate?: (node: Node) => boolean): void;
    /**
     * Will return all nodes that have this node as ascendant
     * @param directDescendantsOnly defines if true only direct descendants of 'this' will be considered, if false direct and also indirect (children of children, an so on in a recursive manner) descendants of 'this' will be considered
     * @param predicate defines an optional predicate that will be called on every evaluated child, the predicate must return true for a given child to be part of the result, otherwise it will be ignored
     * @returns all children nodes of all types
     */
    getDescendants<T extends Node>(directDescendantsOnly?: boolean, predicate?: (node: Node) => node is T): T[];
    /**
     * Will return all nodes that have this node as ascendant
     * @param directDescendantsOnly defines if true only direct descendants of 'this' will be considered, if false direct and also indirect (children of children, an so on in a recursive manner) descendants of 'this' will be considered
     * @param predicate defines an optional predicate that will be called on every evaluated child, the predicate must return true for a given child to be part of the result, otherwise it will be ignored
     * @returns all children nodes of all types
     */
    getDescendants(directDescendantsOnly?: boolean, predicate?: (node: Node) => boolean): Node[];
    /**
     * Get all child-meshes of this node
     * @param directDescendantsOnly defines if true only direct descendants of 'this' will be considered, if false direct and also indirect (children of children, an so on in a recursive manner) descendants of 'this' will be considered (Default: false)
     * @param predicate defines an optional predicate that will be called on every evaluated child, the predicate must return true for a given child to be part of the result, otherwise it will be ignored
     * @returns an array of AbstractMesh
     */
    getChildMeshes<T extends AbstractMesh>(directDescendantsOnly?: boolean, predicate?: (node: Node) => node is T): T[];
    /**
     * Get all child-meshes of this node
     * @param directDescendantsOnly defines if true only direct descendants of 'this' will be considered, if false direct and also indirect (children of children, an so on in a recursive manner) descendants of 'this' will be considered (Default: false)
     * @param predicate defines an optional predicate that will be called on every evaluated child, the predicate must return true for a given child to be part of the result, otherwise it will be ignored
     * @returns an array of AbstractMesh
     */
    getChildMeshes(directDescendantsOnly?: boolean, predicate?: (node: Node) => boolean): AbstractMesh[];
    /**
     * Get all direct children of this node
     * @param predicate defines an optional predicate that will be called on every evaluated child, the predicate must return true for a given child to be part of the result, otherwise it will be ignored
     * @param directDescendantsOnly defines if true only direct descendants of 'this' will be considered, if false direct and also indirect (children of children, an so on in a recursive manner) descendants of 'this' will be considered (Default: true)
     * @returns an array of Node
     */
    getChildren<T extends Node>(predicate?: (node: Node) => node is T, directDescendantsOnly?: boolean): T[];
    /**
     * Get all direct children of this node
     * @param predicate defines an optional predicate that will be called on every evaluated child, the predicate must return true for a given child to be part of the result, otherwise it will be ignored
     * @param directDescendantsOnly defines if true only direct descendants of 'this' will be considered, if false direct and also indirect (children of children, an so on in a recursive manner) descendants of 'this' will be considered (Default: true)
     * @returns an array of Node
     */
    getChildren(predicate?: (node: Node) => boolean, directDescendantsOnly?: boolean): Node[];
    /**
     * @internal
     */
    _setReady(state: boolean): void;
    /**
     * Get an animation by name
     * @param name defines the name of the animation to look for
     * @returns null if not found else the requested animation
     */
    getAnimationByName(name: string): Nullable<Animation>;
    /**
     * Creates an animation range for this node
     * @param name defines the name of the range
     * @param from defines the starting key
     * @param to defines the end key
     */
    createAnimationRange(name: string, from: number, to: number): void;
    /**
     * Delete a specific animation range
     * @param name defines the name of the range to delete
     * @param deleteFrames defines if animation frames from the range must be deleted as well
     */
    deleteAnimationRange(name: string, deleteFrames?: boolean): void;
    /**
     * Get an animation range by name
     * @param name defines the name of the animation range to look for
     * @returns null if not found else the requested animation range
     */
    getAnimationRange(name: string): Nullable<AnimationRange>;
    /**
     * Gets the list of all animation ranges defined on this node
     * @returns an array
     */
    getAnimationRanges(): Nullable<AnimationRange>[];
    /**
     * Will start the animation sequence
     * @param name defines the range frames for animation sequence
     * @param loop defines if the animation should loop (false by default)
     * @param speedRatio defines the speed factor in which to run the animation (1 by default)
     * @param onAnimationEnd defines a function to be executed when the animation ended (undefined by default)
     * @returns the object created for this animation. If range does not exist, it will return null
     */
    beginAnimation(name: string, loop?: boolean, speedRatio?: number, onAnimationEnd?: () => void): Nullable<Animatable>;
    /**
     * Serialize animation ranges into a JSON compatible object
     * @returns serialization object
     */
    serializeAnimationRanges(): any;
    /**
     * Computes the world matrix of the node
     * @param _force defines if the cache version should be invalidated forcing the world matrix to be created from scratch
     * @returns the world matrix
     */
    computeWorldMatrix(_force?: boolean): Matrix;
    /**
     * Releases resources associated with this node.
     * @param doNotRecurse Set to true to not recurse into each children (recurse into each children by default)
     * @param disposeMaterialAndTextures Set to true to also dispose referenced materials and textures (false by default)
     */
    dispose(doNotRecurse?: boolean, disposeMaterialAndTextures?: boolean): void;
    /**
     * Parse animation range data from a serialization object and store them into a given node
     * @param node defines where to store the animation ranges
     * @param parsedNode defines the serialization object to read data from
     * @param _scene defines the hosting scene
     */
    static ParseAnimationRanges(node: Node, parsedNode: any, _scene: Scene): void;
    /**
     * Return the minimum and maximum world vectors of the entire hierarchy under current node
     * @param includeDescendants Include bounding info from descendants as well (true by default)
     * @param predicate defines a callback function that can be customize to filter what meshes should be included in the list used to compute the bounding vectors
     * @returns the new bounding vectors
     */
    getHierarchyBoundingVectors(includeDescendants?: boolean, predicate?: Nullable<(abstractMesh: AbstractMesh) => boolean>): {
        min: Vector3;
        max: Vector3;
    };
}
export {};

}
declare module "babylonjs/Offline/database" {
import { IOfflineProvider } from "babylonjs/Offline/IOfflineProvider";
/**
 * Class used to enable access to IndexedDB
 * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/optimizeCached
 */
export class Database implements IOfflineProvider {
    private _currentSceneUrl;
    private _db;
    private _enableSceneOffline;
    private _enableTexturesOffline;
    private _manifestVersionFound;
    private _mustUpdateRessources;
    private _hasReachedQuota;
    private _isSupported;
    private _idbFactory;
    /** Gets a boolean indicating if the user agent supports blob storage (this value will be updated after creating the first Database object) */
    private static _IsUASupportingBlobStorage;
    /**
     * Gets a boolean indicating if Database storage is enabled (off by default)
     */
    static IDBStorageEnabled: boolean;
    /**
     * Gets a boolean indicating if scene must be saved in the database
     */
    get enableSceneOffline(): boolean;
    /**
     * Gets a boolean indicating if textures must be saved in the database
     */
    get enableTexturesOffline(): boolean;
    /**
     * Creates a new Database
     * @param urlToScene defines the url to load the scene
     * @param callbackManifestChecked defines the callback to use when manifest is checked
     * @param disableManifestCheck defines a boolean indicating that we want to skip the manifest validation (it will be considered validated and up to date)
     */
    constructor(urlToScene: string, callbackManifestChecked: (checked: boolean) => any, disableManifestCheck?: boolean);
    private static _ParseURL;
    private static _ReturnFullUrlLocation;
    private _checkManifestFile;
    /**
     * Open the database and make it available
     * @param successCallback defines the callback to call on success
     * @param errorCallback defines the callback to call on error
     */
    open(successCallback: () => void, errorCallback: () => void): void;
    /**
     * Loads an image from the database
     * @param url defines the url to load from
     * @param image defines the target DOM image
     */
    loadImage(url: string, image: HTMLImageElement): void;
    private _loadImageFromDBAsync;
    private _saveImageIntoDBAsync;
    private _checkVersionFromDB;
    private _loadVersionFromDBAsync;
    private _saveVersionIntoDBAsync;
    /**
     * Loads a file from database
     * @param url defines the URL to load from
     * @param sceneLoaded defines a callback to call on success
     * @param progressCallBack defines a callback to call when progress changed
     * @param errorCallback defines a callback to call on error
     * @param useArrayBuffer defines a boolean to use array buffer instead of text string
     */
    loadFile(url: string, sceneLoaded: (data: any) => void, progressCallBack?: (data: any) => void, errorCallback?: () => void, useArrayBuffer?: boolean): void;
    private _loadFileAsync;
    private _saveFileAsync;
    /**
     * Validates if xhr data is correct
     * @param xhr defines the request to validate
     * @param dataType defines the expected data type
     * @returns true if data is correct
     */
    private static _ValidateXHRData;
}

}
declare module "babylonjs/Offline/index" {
export * from "babylonjs/Offline/database";
export * from "babylonjs/Offline/IOfflineProvider";

}
declare module "babylonjs/Offline/IOfflineProvider" {
/**
 * Class used to enable access to offline support
 * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/optimizeCached
 */
export interface IOfflineProvider {
    /**
     * Gets a boolean indicating if scene must be saved in the database
     */
    enableSceneOffline: boolean;
    /**
     * Gets a boolean indicating if textures must be saved in the database
     */
    enableTexturesOffline: boolean;
    /**
     * Open the offline support and make it available
     * @param successCallback defines the callback to call on success
     * @param errorCallback defines the callback to call on error
     */
    open(successCallback: () => void, errorCallback: () => void): void;
    /**
     * Loads an image from the offline support
     * @param url defines the url to load from
     * @param image defines the target DOM image
     */
    loadImage(url: string, image: HTMLImageElement): void;
    /**
     * Loads a file from offline support
     * @param url defines the URL to load from
     * @param sceneLoaded defines a callback to call on success
     * @param progressCallBack defines a callback to call when progress changed
     * @param errorCallback defines a callback to call on error
     * @param useArrayBuffer defines a boolean to use array buffer instead of text string
     */
    loadFile(url: string, sceneLoaded: (data: any) => void, progressCallBack?: (data: any) => void, errorCallback?: () => void, useArrayBuffer?: boolean): void;
}

}
declare module "babylonjs/Particles/baseParticleSystem" {
import { Nullable } from "babylonjs/types";
import { Vector2, Vector3 } from "babylonjs/Maths/math.vector";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { ImageProcessingConfiguration } from "babylonjs/Materials/imageProcessingConfiguration";
import { ImageProcessingConfigurationDefines } from "babylonjs/Materials/imageProcessingConfiguration";
import { ColorGradient, FactorGradient, Color3Gradient, IValueGradient } from "babylonjs/Misc/gradients";
import { IParticleEmitterType } from "babylonjs/Particles/EmitterTypes/index";
import { BoxParticleEmitter, PointParticleEmitter, HemisphericParticleEmitter, SphereParticleEmitter, SphereDirectedParticleEmitter, CylinderParticleEmitter, CylinderDirectedParticleEmitter, ConeParticleEmitter } from "babylonjs/Particles/EmitterTypes/index";
import { BaseTexture } from "babylonjs/Materials/Textures/baseTexture";
import { Color4 } from "babylonjs/Maths/math.color";
import { ThinEngine } from "babylonjs/Engines/thinEngine";
import "babylonjs/Engines/Extensions/engine.dynamicBuffer";
import { IClipPlanesHolder } from "babylonjs/Misc/interfaces/iClipPlanesHolder";
import { Plane } from "babylonjs/Maths/math.plane";
import { Animation } from "babylonjs/Animations/animation";
import { Scene } from "babylonjs/scene";
import { ProceduralTexture } from "babylonjs/Materials/Textures/Procedurals/proceduralTexture";
import { RawTexture } from "babylonjs/Materials/Textures/rawTexture";
/**
 * This represents the base class for particle system in Babylon.
 * Particles are often small sprites used to simulate hard-to-reproduce phenomena like fire, smoke, water, or abstract visual effects like magic glitter and faery dust.
 * Particles can take different shapes while emitted like box, sphere, cone or you can write your custom function.
 * @example https://doc.babylonjs.com/features/featuresDeepDive/particles/particle_system/particle_system_intro
 */
export class BaseParticleSystem implements IClipPlanesHolder {
    /**
     * Source color is added to the destination color without alpha affecting the result
     */
    static BLENDMODE_ONEONE: number;
    /**
     * Blend current color and particle color using particle’s alpha
     */
    static BLENDMODE_STANDARD: number;
    /**
     * Add current color and particle color multiplied by particle’s alpha
     */
    static BLENDMODE_ADD: number;
    /**
     * Multiply current color with particle color
     */
    static BLENDMODE_MULTIPLY: number;
    /**
     * Multiply current color with particle color then add current color and particle color multiplied by particle’s alpha
     */
    static BLENDMODE_MULTIPLYADD: number;
    /**
     * List of animations used by the particle system.
     */
    animations: Animation[];
    /**
     * Gets or sets the unique id of the particle system
     */
    uniqueId: number;
    /**
     * The id of the Particle system.
     */
    id: string;
    /**
     * The friendly name of the Particle system.
     */
    name: string;
    /**
     * Snippet ID if the particle system was created from the snippet server
     */
    snippetId: string;
    /**
     * The rendering group used by the Particle system to chose when to render.
     */
    renderingGroupId: number;
    /**
     * The emitter represents the Mesh or position we are attaching the particle system to.
     */
    emitter: Nullable<AbstractMesh | Vector3>;
    /**
     * The maximum number of particles to emit per frame
     */
    emitRate: number;
    /**
     * If you want to launch only a few particles at once, that can be done, as well.
     */
    manualEmitCount: number;
    /**
     * The overall motion speed (0.01 is default update speed, faster updates = faster animation)
     */
    updateSpeed: number;
    /**
     * The amount of time the particle system is running (depends of the overall update speed).
     */
    targetStopDuration: number;
    /**
     * Specifies whether the particle system will be disposed once it reaches the end of the animation.
     */
    disposeOnStop: boolean;
    /**
     * Minimum power of emitting particles.
     */
    minEmitPower: number;
    /**
     * Maximum power of emitting particles.
     */
    maxEmitPower: number;
    /**
     * Minimum life time of emitting particles.
     */
    minLifeTime: number;
    /**
     * Maximum life time of emitting particles.
     */
    maxLifeTime: number;
    /**
     * Minimum Size of emitting particles.
     */
    minSize: number;
    /**
     * Maximum Size of emitting particles.
     */
    maxSize: number;
    /**
     * Minimum scale of emitting particles on X axis.
     */
    minScaleX: number;
    /**
     * Maximum scale of emitting particles on X axis.
     */
    maxScaleX: number;
    /**
     * Minimum scale of emitting particles on Y axis.
     */
    minScaleY: number;
    /**
     * Maximum scale of emitting particles on Y axis.
     */
    maxScaleY: number;
    /**
     * Gets or sets the minimal initial rotation in radians.
     */
    minInitialRotation: number;
    /**
     * Gets or sets the maximal initial rotation in radians.
     */
    maxInitialRotation: number;
    /**
     * Minimum angular speed of emitting particles (Z-axis rotation for each particle).
     */
    minAngularSpeed: number;
    /**
     * Maximum angular speed of emitting particles (Z-axis rotation for each particle).
     */
    maxAngularSpeed: number;
    /**
     * The texture used to render each particle. (this can be a spritesheet)
     */
    particleTexture: Nullable<BaseTexture>;
    /**
     * The layer mask we are rendering the particles through.
     */
    layerMask: number;
    /**
     * This can help using your own shader to render the particle system.
     * The according effect will be created
     */
    customShader: any;
    /**
     * By default particle system starts as soon as they are created. This prevents the
     * automatic start to happen and let you decide when to start emitting particles.
     */
    preventAutoStart: boolean;
    /** @internal */
    _wasDispatched: boolean;
    protected _rootUrl: string;
    private _noiseTexture;
    /**
     * Gets or sets a texture used to add random noise to particle positions
     */
    get noiseTexture(): Nullable<ProceduralTexture>;
    set noiseTexture(value: Nullable<ProceduralTexture>);
    /** Gets or sets the strength to apply to the noise value (default is (10, 10, 10)) */
    noiseStrength: Vector3;
    /**
     * Callback triggered when the particle animation is ending.
     */
    onAnimationEnd: Nullable<() => void>;
    /**
     * Blend mode use to render the particle, it can be either ParticleSystem.BLENDMODE_ONEONE or ParticleSystem.BLENDMODE_STANDARD.
     */
    blendMode: number;
    /**
     * Forces the particle to write their depth information to the depth buffer. This can help preventing other draw calls
     * to override the particles.
     */
    forceDepthWrite: boolean;
    /** Gets or sets a value indicating how many cycles (or frames) must be executed before first rendering (this value has to be set before starting the system). Default is 0 */
    preWarmCycles: number;
    /** Gets or sets a value indicating the time step multiplier to use in pre-warm mode (default is 1) */
    preWarmStepOffset: number;
    /**
     * If using a spritesheet (isAnimationSheetEnabled) defines the speed of the sprite loop (default is 1 meaning the animation will play once during the entire particle lifetime)
     */
    spriteCellChangeSpeed: number;
    /**
     * If using a spritesheet (isAnimationSheetEnabled) defines the first sprite cell to display
     */
    startSpriteCellID: number;
    /**
     * If using a spritesheet (isAnimationSheetEnabled) defines the last sprite cell to display
     */
    endSpriteCellID: number;
    /**
     * If using a spritesheet (isAnimationSheetEnabled), defines the sprite cell width to use
     */
    spriteCellWidth: number;
    /**
     * If using a spritesheet (isAnimationSheetEnabled), defines the sprite cell height to use
     */
    spriteCellHeight: number;
    /**
     * If using a spritesheet (isAnimationSheetEnabled), defines wether the sprite animation is looping
     */
    spriteCellLoop: boolean;
    /**
     * This allows the system to random pick the start cell ID between startSpriteCellID and endSpriteCellID
     */
    spriteRandomStartCell: boolean;
    /** Gets or sets a Vector2 used to move the pivot (by default (0,0)) */
    translationPivot: Vector2;
    /** @internal */
    _isAnimationSheetEnabled: boolean;
    /**
     * Gets or sets a boolean indicating that hosted animations (in the system.animations array) must be started when system.start() is called
     */
    beginAnimationOnStart: boolean;
    /**
     * Gets or sets the frame to start the animation from when beginAnimationOnStart is true
     */
    beginAnimationFrom: number;
    /**
     * Gets or sets the frame to end the animation on when beginAnimationOnStart is true
     */
    beginAnimationTo: number;
    /**
     * Gets or sets a boolean indicating if animations must loop when beginAnimationOnStart is true
     */
    beginAnimationLoop: boolean;
    /**
     * Gets or sets a world offset applied to all particles
     */
    worldOffset: Vector3;
    /**
     * Gets or sets the active clipplane 1
     */
    clipPlane: Nullable<Plane>;
    /**
     * Gets or sets the active clipplane 2
     */
    clipPlane2: Nullable<Plane>;
    /**
     * Gets or sets the active clipplane 3
     */
    clipPlane3: Nullable<Plane>;
    /**
     * Gets or sets the active clipplane 4
     */
    clipPlane4: Nullable<Plane>;
    /**
     * Gets or sets the active clipplane 5
     */
    clipPlane5: Nullable<Plane>;
    /**
     * Gets or sets the active clipplane 6
     */
    clipPlane6: Nullable<Plane>;
    /**
     * Gets or sets whether an animation sprite sheet is enabled or not on the particle system
     */
    get isAnimationSheetEnabled(): boolean;
    set isAnimationSheetEnabled(value: boolean);
    private _useLogarithmicDepth;
    /**
     * Gets or sets a boolean enabling the use of logarithmic depth buffers, which is good for wide depth buffers.
     */
    get useLogarithmicDepth(): boolean;
    set useLogarithmicDepth(value: boolean);
    /**
     * Get hosting scene
     * @returns the scene
     */
    getScene(): Nullable<Scene>;
    /**
     * You can use gravity if you want to give an orientation to your particles.
     */
    gravity: Vector3;
    protected _colorGradients: Nullable<Array<ColorGradient>>;
    protected _sizeGradients: Nullable<Array<FactorGradient>>;
    protected _lifeTimeGradients: Nullable<Array<FactorGradient>>;
    protected _angularSpeedGradients: Nullable<Array<FactorGradient>>;
    protected _velocityGradients: Nullable<Array<FactorGradient>>;
    protected _limitVelocityGradients: Nullable<Array<FactorGradient>>;
    protected _dragGradients: Nullable<Array<FactorGradient>>;
    protected _emitRateGradients: Nullable<Array<FactorGradient>>;
    protected _startSizeGradients: Nullable<Array<FactorGradient>>;
    protected _rampGradients: Nullable<Array<Color3Gradient>>;
    protected _colorRemapGradients: Nullable<Array<FactorGradient>>;
    protected _alphaRemapGradients: Nullable<Array<FactorGradient>>;
    protected _hasTargetStopDurationDependantGradient(): boolean | null;
    /**
     * Defines the delay in milliseconds before starting the system (0 by default)
     */
    startDelay: number;
    /**
     * Gets the current list of drag gradients.
     * You must use addDragGradient and removeDragGradient to update this list
     * @returns the list of drag gradients
     */
    getDragGradients(): Nullable<Array<FactorGradient>>;
    /** Gets or sets a value indicating the damping to apply if the limit velocity factor is reached */
    limitVelocityDamping: number;
    /**
     * Gets the current list of limit velocity gradients.
     * You must use addLimitVelocityGradient and removeLimitVelocityGradient to update this list
     * @returns the list of limit velocity gradients
     */
    getLimitVelocityGradients(): Nullable<Array<FactorGradient>>;
    /**
     * Gets the current list of color gradients.
     * You must use addColorGradient and removeColorGradient to update this list
     * @returns the list of color gradients
     */
    getColorGradients(): Nullable<Array<ColorGradient>>;
    /**
     * Gets the current list of size gradients.
     * You must use addSizeGradient and removeSizeGradient to update this list
     * @returns the list of size gradients
     */
    getSizeGradients(): Nullable<Array<FactorGradient>>;
    /**
     * Gets the current list of color remap gradients.
     * You must use addColorRemapGradient and removeColorRemapGradient to update this list
     * @returns the list of color remap gradients
     */
    getColorRemapGradients(): Nullable<Array<FactorGradient>>;
    /**
     * Gets the current list of alpha remap gradients.
     * You must use addAlphaRemapGradient and removeAlphaRemapGradient to update this list
     * @returns the list of alpha remap gradients
     */
    getAlphaRemapGradients(): Nullable<Array<FactorGradient>>;
    /**
     * Gets the current list of life time gradients.
     * You must use addLifeTimeGradient and removeLifeTimeGradient to update this list
     * @returns the list of life time gradients
     */
    getLifeTimeGradients(): Nullable<Array<FactorGradient>>;
    /**
     * Gets the current list of angular speed gradients.
     * You must use addAngularSpeedGradient and removeAngularSpeedGradient to update this list
     * @returns the list of angular speed gradients
     */
    getAngularSpeedGradients(): Nullable<Array<FactorGradient>>;
    /**
     * Gets the current list of velocity gradients.
     * You must use addVelocityGradient and removeVelocityGradient to update this list
     * @returns the list of velocity gradients
     */
    getVelocityGradients(): Nullable<Array<FactorGradient>>;
    /**
     * Gets the current list of start size gradients.
     * You must use addStartSizeGradient and removeStartSizeGradient to update this list
     * @returns the list of start size gradients
     */
    getStartSizeGradients(): Nullable<Array<FactorGradient>>;
    /**
     * Gets the current list of emit rate gradients.
     * You must use addEmitRateGradient and removeEmitRateGradient to update this list
     * @returns the list of emit rate gradients
     */
    getEmitRateGradients(): Nullable<Array<FactorGradient>>;
    /**
     * Random direction of each particle after it has been emitted, between direction1 and direction2 vectors.
     * This only works when particleEmitterTyps is a BoxParticleEmitter
     */
    get direction1(): Vector3;
    set direction1(value: Vector3);
    /**
     * Random direction of each particle after it has been emitted, between direction1 and direction2 vectors.
     * This only works when particleEmitterTyps is a BoxParticleEmitter
     */
    get direction2(): Vector3;
    set direction2(value: Vector3);
    /**
     * Minimum box point around our emitter. Our emitter is the center of particles source, but if you want your particles to emit from more than one point, then you can tell it to do so.
     * This only works when particleEmitterTyps is a BoxParticleEmitter
     */
    get minEmitBox(): Vector3;
    set minEmitBox(value: Vector3);
    /**
     * Maximum box point around our emitter. Our emitter is the center of particles source, but if you want your particles to emit from more than one point, then you can tell it to do so.
     * This only works when particleEmitterTyps is a BoxParticleEmitter
     */
    get maxEmitBox(): Vector3;
    set maxEmitBox(value: Vector3);
    /**
     * Random color of each particle after it has been emitted, between color1 and color2 vectors
     */
    color1: Color4;
    /**
     * Random color of each particle after it has been emitted, between color1 and color2 vectors
     */
    color2: Color4;
    /**
     * Color the particle will have at the end of its lifetime
     */
    colorDead: Color4;
    /**
     * An optional mask to filter some colors out of the texture, or filter a part of the alpha channel
     */
    textureMask: Color4;
    /**
     * The particle emitter type defines the emitter used by the particle system.
     * It can be for example box, sphere, or cone...
     */
    particleEmitterType: IParticleEmitterType;
    /** @internal */
    _isSubEmitter: boolean;
    /** @internal */
    _billboardMode: number;
    /**
     * Gets or sets the billboard mode to use when isBillboardBased = true.
     * Value can be: ParticleSystem.BILLBOARDMODE_ALL, ParticleSystem.BILLBOARDMODE_Y, ParticleSystem.BILLBOARDMODE_STRETCHED
     */
    get billboardMode(): number;
    set billboardMode(value: number);
    /** @internal */
    _isBillboardBased: boolean;
    /**
     * Gets or sets a boolean indicating if the particles must be rendered as billboard or aligned with the direction
     */
    get isBillboardBased(): boolean;
    set isBillboardBased(value: boolean);
    /**
     * The scene the particle system belongs to.
     */
    protected _scene: Nullable<Scene>;
    /**
     * The engine the particle system belongs to.
     */
    protected _engine: ThinEngine;
    /**
     * Local cache of defines for image processing.
     */
    protected _imageProcessingConfigurationDefines: ImageProcessingConfigurationDefines;
    /**
     * Default configuration related to image processing available in the standard Material.
     */
    protected _imageProcessingConfiguration: Nullable<ImageProcessingConfiguration>;
    /**
     * Gets the image processing configuration used either in this material.
     */
    get imageProcessingConfiguration(): Nullable<ImageProcessingConfiguration>;
    /**
     * Sets the Default image processing configuration used either in the this material.
     *
     * If sets to null, the scene one is in use.
     */
    set imageProcessingConfiguration(value: Nullable<ImageProcessingConfiguration>);
    /**
     * Attaches a new image processing configuration to the Standard Material.
     * @param configuration
     */
    protected _attachImageProcessingConfiguration(configuration: Nullable<ImageProcessingConfiguration>): void;
    /** @internal */
    protected _reset(): void;
    /**
     * @internal
     */
    protected _removeGradientAndTexture(gradient: number, gradients: Nullable<IValueGradient[]>, texture: Nullable<RawTexture>): BaseParticleSystem;
    /**
     * Instantiates a particle system.
     * Particles are often small sprites used to simulate hard-to-reproduce phenomena like fire, smoke, water, or abstract visual effects like magic glitter and faery dust.
     * @param name The name of the particle system
     */
    constructor(name: string);
    /**
     * Creates a Point Emitter for the particle system (emits directly from the emitter position)
     * @param direction1 Particles are emitted between the direction1 and direction2 from within the box
     * @param direction2 Particles are emitted between the direction1 and direction2 from within the box
     * @returns the emitter
     */
    createPointEmitter(direction1: Vector3, direction2: Vector3): PointParticleEmitter;
    /**
     * Creates a Hemisphere Emitter for the particle system (emits along the hemisphere radius)
     * @param radius The radius of the hemisphere to emit from
     * @param radiusRange The range of the hemisphere to emit from [0-1] 0 Surface Only, 1 Entire Radius
     * @returns the emitter
     */
    createHemisphericEmitter(radius?: number, radiusRange?: number): HemisphericParticleEmitter;
    /**
     * Creates a Sphere Emitter for the particle system (emits along the sphere radius)
     * @param radius The radius of the sphere to emit from
     * @param radiusRange The range of the sphere to emit from [0-1] 0 Surface Only, 1 Entire Radius
     * @returns the emitter
     */
    createSphereEmitter(radius?: number, radiusRange?: number): SphereParticleEmitter;
    /**
     * Creates a Directed Sphere Emitter for the particle system (emits between direction1 and direction2)
     * @param radius The radius of the sphere to emit from
     * @param direction1 Particles are emitted between the direction1 and direction2 from within the sphere
     * @param direction2 Particles are emitted between the direction1 and direction2 from within the sphere
     * @returns the emitter
     */
    createDirectedSphereEmitter(radius?: number, direction1?: Vector3, direction2?: Vector3): SphereDirectedParticleEmitter;
    /**
     * Creates a Cylinder Emitter for the particle system (emits from the cylinder to the particle position)
     * @param radius The radius of the emission cylinder
     * @param height The height of the emission cylinder
     * @param radiusRange The range of emission [0-1] 0 Surface only, 1 Entire Radius
     * @param directionRandomizer How much to randomize the particle direction [0-1]
     * @returns the emitter
     */
    createCylinderEmitter(radius?: number, height?: number, radiusRange?: number, directionRandomizer?: number): CylinderParticleEmitter;
    /**
     * Creates a Directed Cylinder Emitter for the particle system (emits between direction1 and direction2)
     * @param radius The radius of the cylinder to emit from
     * @param height The height of the emission cylinder
     * @param radiusRange the range of the emission cylinder [0-1] 0 Surface only, 1 Entire Radius (1 by default)
     * @param direction1 Particles are emitted between the direction1 and direction2 from within the cylinder
     * @param direction2 Particles are emitted between the direction1 and direction2 from within the cylinder
     * @returns the emitter
     */
    createDirectedCylinderEmitter(radius?: number, height?: number, radiusRange?: number, direction1?: Vector3, direction2?: Vector3): CylinderDirectedParticleEmitter;
    /**
     * Creates a Cone Emitter for the particle system (emits from the cone to the particle position)
     * @param radius The radius of the cone to emit from
     * @param angle The base angle of the cone
     * @returns the emitter
     */
    createConeEmitter(radius?: number, angle?: number): ConeParticleEmitter;
    /**
     * Creates a Box Emitter for the particle system. (emits between direction1 and direction2 from withing the box defined by minEmitBox and maxEmitBox)
     * @param direction1 Particles are emitted between the direction1 and direction2 from within the box
     * @param direction2 Particles are emitted between the direction1 and direction2 from within the box
     * @param minEmitBox Particles are emitted from the box between minEmitBox and maxEmitBox
     * @param maxEmitBox  Particles are emitted from the box between minEmitBox and maxEmitBox
     * @returns the emitter
     */
    createBoxEmitter(direction1: Vector3, direction2: Vector3, minEmitBox: Vector3, maxEmitBox: Vector3): BoxParticleEmitter;
}

}
declare module "babylonjs/Particles/cloudPoint" {
import { Nullable } from "babylonjs/types";
import { Matrix } from "babylonjs/Maths/math";
import { Color4, Vector2, Vector3, Quaternion } from "babylonjs/Maths/math";
import { Mesh } from "babylonjs/Meshes/mesh";
import { BoundingInfo } from "babylonjs/Culling/boundingInfo";
import { PointsCloudSystem } from "babylonjs/Particles/pointsCloudSystem";
/**
 * Represents one particle of a points cloud system.
 */
export class CloudPoint {
    /**
     * particle global index
     */
    idx: number;
    /**
     * The color of the particle
     */
    color: Nullable<Color4>;
    /**
     * The world space position of the particle.
     */
    position: Vector3;
    /**
     * The world space rotation of the particle. (Not use if rotationQuaternion is set)
     */
    rotation: Vector3;
    /**
     * The world space rotation quaternion of the particle.
     */
    rotationQuaternion: Nullable<Quaternion>;
    /**
     * The uv of the particle.
     */
    uv: Nullable<Vector2>;
    /**
     * The current speed of the particle.
     */
    velocity: Vector3;
    /**
     * The pivot point in the particle local space.
     */
    pivot: Vector3;
    /**
     * Must the particle be translated from its pivot point in its local space ?
     * In this case, the pivot point is set at the origin of the particle local space and the particle is translated.
     * Default : false
     */
    translateFromPivot: boolean;
    /**
     * Index of this particle in the global "positions" array (Internal use)
     * @internal
     */
    _pos: number;
    /**
     * @internal Index of this particle in the global "indices" array (Internal use)
     */
    _ind: number;
    /**
     * Group this particle belongs to
     */
    _group: PointsGroup;
    /**
     * Group id of this particle
     */
    groupId: number;
    /**
     * Index of the particle in its group id (Internal use)
     */
    idxInGroup: number;
    /**
     * @internal Particle BoundingInfo object (Internal use)
     */
    _boundingInfo: BoundingInfo;
    /**
     * @internal Reference to the PCS that the particle belongs to (Internal use)
     */
    _pcs: PointsCloudSystem;
    /**
     * @internal Still set as invisible in order to skip useless computations (Internal use)
     */
    _stillInvisible: boolean;
    /**
     * @internal Last computed particle rotation matrix
     */
    _rotationMatrix: number[];
    /**
     * Parent particle Id, if any.
     * Default null.
     */
    parentId: Nullable<number>;
    /**
     * @internal Internal global position in the PCS.
     */
    _globalPosition: Vector3;
    /**
     * Creates a Point Cloud object.
     * Don't create particles manually, use instead the PCS internal tools like _addParticle()
     * @param particleIndex (integer) is the particle index in the PCS pool. It's also the particle identifier.
     * @param group (PointsGroup) is the group the particle belongs to
     * @param groupId (integer) is the group identifier in the PCS.
     * @param idxInGroup (integer) is the index of the particle in the current point group (ex: the 10th point of addPoints(30))
     * @param pcs defines the PCS it is associated to
     */
    constructor(particleIndex: number, group: PointsGroup, groupId: number, idxInGroup: number, pcs: PointsCloudSystem);
    /**
     * get point size
     */
    get size(): Vector3;
    /**
     * Set point size
     */
    set size(scale: Vector3);
    /**
     * Legacy support, changed quaternion to rotationQuaternion
     */
    get quaternion(): Nullable<Quaternion>;
    /**
     * Legacy support, changed quaternion to rotationQuaternion
     */
    set quaternion(q: Nullable<Quaternion>);
    /**
     * Returns a boolean. True if the particle intersects a mesh, else false
     * The intersection is computed on the particle position and Axis Aligned Bounding Box (AABB) or Sphere
     * @param target is the object (point or mesh) what the intersection is computed against
     * @param isSphere is boolean flag when false (default) bounding box of mesh is used, when true the bounding sphere is used
     * @returns true if it intersects
     */
    intersectsMesh(target: Mesh, isSphere: boolean): boolean;
    /**
     * get the rotation matrix of the particle
     * @internal
     */
    getRotationMatrix(m: Matrix): void;
}
/**
 * Represents a group of points in a points cloud system
 *  * PCS internal tool, don't use it manually.
 */
export class PointsGroup {
    /**
     * Get or set the groupId
     * @deprecated Please use groupId instead
     */
    get groupID(): number;
    set groupID(groupID: number);
    /**
     * The group id
     * @internal
     */
    groupId: number;
    /**
     * image data for group (internal use)
     * @internal
     */
    _groupImageData: Nullable<ArrayBufferView>;
    /**
     * Image Width (internal use)
     * @internal
     */
    _groupImgWidth: number;
    /**
     * Image Height (internal use)
     * @internal
     */
    _groupImgHeight: number;
    /**
     * Custom position function (internal use)
     * @internal
     */
    _positionFunction: Nullable<(particle: CloudPoint, i?: number, s?: number) => void>;
    /**
     * density per facet for surface points
     * @internal
     */
    _groupDensity: number[];
    /**
     * Only when points are colored by texture carries pointer to texture list array
     * @internal
     */
    _textureNb: number;
    /**
     * Creates a points group object. This is an internal reference to produce particles for the PCS.
     * PCS internal tool, don't use it manually.
     * @internal
     */
    constructor(id: number, posFunction: Nullable<(particle: CloudPoint, i?: number, s?: number) => void>);
}

}
declare module "babylonjs/Particles/computeShaderParticleSystem" {
import { ThinEngine } from "babylonjs/Engines/thinEngine";
import { IGPUParticleSystemPlatform } from "babylonjs/Particles/IGPUParticleSystemPlatform";
import { Buffer, VertexBuffer } from "babylonjs/Buffers/buffer";
import { GPUParticleSystem } from "babylonjs/Particles/gpuParticleSystem";
import { DataArray } from "babylonjs/types";
import { DataBuffer } from "babylonjs/Buffers/dataBuffer";
import { UniformBufferEffectCommonAccessor } from "babylonjs/Materials/uniformBufferEffectCommonAccessor";
import { Effect } from "babylonjs/Materials/effect";
import "babylonjs/ShadersWGSL/gpuUpdateParticles.compute";
/** @internal */
export class ComputeShaderParticleSystem implements IGPUParticleSystemPlatform {
    private _parent;
    private _engine;
    private _updateComputeShader;
    private _simParamsComputeShader;
    private _bufferComputeShader;
    private _renderVertexBuffers;
    readonly alignDataInBuffer: boolean;
    constructor(parent: GPUParticleSystem, engine: ThinEngine);
    isUpdateBufferCreated(): boolean;
    isUpdateBufferReady(): boolean;
    createUpdateBuffer(defines: string): UniformBufferEffectCommonAccessor;
    createVertexBuffers(updateBuffer: Buffer, renderVertexBuffers: {
        [key: string]: VertexBuffer;
    }): void;
    createParticleBuffer(data: number[]): DataArray | DataBuffer;
    bindDrawBuffers(index: number, effect: Effect): void;
    preUpdateParticleBuffer(): void;
    updateParticleBuffer(index: number, targetBuffer: Buffer, currentActiveCount: number): void;
    releaseBuffers(): void;
    releaseVertexBuffers(): void;
}

}
declare module "babylonjs/Particles/EmitterTypes/boxParticleEmitter" {
import { Matrix } from "babylonjs/Maths/math.vector";
import { Vector3 } from "babylonjs/Maths/math.vector";
import { Particle } from "babylonjs/Particles/particle";
import { IParticleEmitterType } from "babylonjs/Particles/EmitterTypes/IParticleEmitterType";
import { UniformBufferEffectCommonAccessor } from "babylonjs/Materials/uniformBufferEffectCommonAccessor";
import { UniformBuffer } from "babylonjs/Materials/uniformBuffer";
/**
 * Particle emitter emitting particles from the inside of a box.
 * It emits the particles randomly between 2 given directions.
 */
export class BoxParticleEmitter implements IParticleEmitterType {
    /**
     * Random direction of each particle after it has been emitted, between direction1 and direction2 vectors.
     */
    direction1: Vector3;
    /**
     * Random direction of each particle after it has been emitted, between direction1 and direction2 vectors.
     */
    direction2: Vector3;
    /**
     * Minimum box point around our emitter. Our emitter is the center of particles source, but if you want your particles to emit from more than one point, then you can tell it to do so.
     */
    minEmitBox: Vector3;
    /**
     * Maximum box point around our emitter. Our emitter is the center of particles source, but if you want your particles to emit from more than one point, then you can tell it to do so.
     */
    maxEmitBox: Vector3;
    /**
     * Creates a new instance BoxParticleEmitter
     */
    constructor();
    /**
     * Called by the particle System when the direction is computed for the created particle.
     * @param worldMatrix is the world matrix of the particle system
     * @param directionToUpdate is the direction vector to update with the result
     * @param particle is the particle we are computed the direction for
     * @param isLocal defines if the direction should be set in local space
     */
    startDirectionFunction(worldMatrix: Matrix, directionToUpdate: Vector3, particle: Particle, isLocal: boolean): void;
    /**
     * Called by the particle System when the position is computed for the created particle.
     * @param worldMatrix is the world matrix of the particle system
     * @param positionToUpdate is the position vector to update with the result
     * @param particle is the particle we are computed the position for
     * @param isLocal defines if the position should be set in local space
     */
    startPositionFunction(worldMatrix: Matrix, positionToUpdate: Vector3, particle: Particle, isLocal: boolean): void;
    /**
     * Clones the current emitter and returns a copy of it
     * @returns the new emitter
     */
    clone(): BoxParticleEmitter;
    /**
     * Called by the GPUParticleSystem to setup the update shader
     * @param uboOrEffect defines the update shader
     */
    applyToShader(uboOrEffect: UniformBufferEffectCommonAccessor): void;
    /**
     * Creates the structure of the ubo for this particle emitter
     * @param ubo ubo to create the structure for
     */
    buildUniformLayout(ubo: UniformBuffer): void;
    /**
     * Returns a string to use to update the GPU particles update shader
     * @returns a string containing the defines string
     */
    getEffectDefines(): string;
    /**
     * Returns the string "BoxParticleEmitter"
     * @returns a string containing the class name
     */
    getClassName(): string;
    /**
     * Serializes the particle system to a JSON object.
     * @returns the JSON object
     */
    serialize(): any;
    /**
     * Parse properties from a JSON object
     * @param serializationObject defines the JSON object
     */
    parse(serializationObject: any): void;
}

}
declare module "babylonjs/Particles/EmitterTypes/coneParticleEmitter" {
import { Matrix } from "babylonjs/Maths/math.vector";
import { Vector3 } from "babylonjs/Maths/math.vector";
import { Particle } from "babylonjs/Particles/particle";
import { IParticleEmitterType } from "babylonjs/Particles/EmitterTypes/IParticleEmitterType";
import { UniformBufferEffectCommonAccessor } from "babylonjs/Materials/uniformBufferEffectCommonAccessor";
import { UniformBuffer } from "babylonjs/Materials/uniformBuffer";
/**
 * Particle emitter emitting particles from the inside of a cone.
 * It emits the particles alongside the cone volume from the base to the particle.
 * The emission direction might be randomized.
 */
export class ConeParticleEmitter implements IParticleEmitterType {
    /** defines how much to randomize the particle direction [0-1] (default is 0) */
    directionRandomizer: number;
    private _radius;
    private _angle;
    private _height;
    /**
     * Gets or sets a value indicating where on the radius the start position should be picked (1 = everywhere, 0 = only surface)
     */
    radiusRange: number;
    /**
     * Gets or sets a value indicating where on the height the start position should be picked (1 = everywhere, 0 = only surface)
     */
    heightRange: number;
    /**
     * Gets or sets a value indicating if all the particles should be emitted from the spawn point only (the base of the cone)
     */
    emitFromSpawnPointOnly: boolean;
    /**
     * Gets or sets the radius of the emission cone
     */
    get radius(): number;
    set radius(value: number);
    /**
     * Gets or sets the angle of the emission cone
     */
    get angle(): number;
    set angle(value: number);
    private _buildHeight;
    /**
     * Creates a new instance ConeParticleEmitter
     * @param radius the radius of the emission cone (1 by default)
     * @param angle the cone base angle (PI by default)
     * @param directionRandomizer defines how much to randomize the particle direction [0-1] (default is 0)
     */
    constructor(radius?: number, angle?: number, 
    /** defines how much to randomize the particle direction [0-1] (default is 0) */
    directionRandomizer?: number);
    /**
     * Called by the particle System when the direction is computed for the created particle.
     * @param worldMatrix is the world matrix of the particle system
     * @param directionToUpdate is the direction vector to update with the result
     * @param particle is the particle we are computed the direction for
     * @param isLocal defines if the direction should be set in local space
     */
    startDirectionFunction(worldMatrix: Matrix, directionToUpdate: Vector3, particle: Particle, isLocal: boolean): void;
    /**
     * Called by the particle System when the position is computed for the created particle.
     * @param worldMatrix is the world matrix of the particle system
     * @param positionToUpdate is the position vector to update with the result
     * @param particle is the particle we are computed the position for
     * @param isLocal defines if the position should be set in local space
     */
    startPositionFunction(worldMatrix: Matrix, positionToUpdate: Vector3, particle: Particle, isLocal: boolean): void;
    /**
     * Clones the current emitter and returns a copy of it
     * @returns the new emitter
     */
    clone(): ConeParticleEmitter;
    /**
     * Called by the GPUParticleSystem to setup the update shader
     * @param uboOrEffect defines the update shader
     */
    applyToShader(uboOrEffect: UniformBufferEffectCommonAccessor): void;
    /**
     * Creates the structure of the ubo for this particle emitter
     * @param ubo ubo to create the structure for
     */
    buildUniformLayout(ubo: UniformBuffer): void;
    /**
     * Returns a string to use to update the GPU particles update shader
     * @returns a string containing the defines string
     */
    getEffectDefines(): string;
    /**
     * Returns the string "ConeParticleEmitter"
     * @returns a string containing the class name
     */
    getClassName(): string;
    /**
     * Serializes the particle system to a JSON object.
     * @returns the JSON object
     */
    serialize(): any;
    /**
     * Parse properties from a JSON object
     * @param serializationObject defines the JSON object
     */
    parse(serializationObject: any): void;
}

}
declare module "babylonjs/Particles/EmitterTypes/customParticleEmitter" {
import { Matrix } from "babylonjs/Maths/math.vector";
import { Vector3 } from "babylonjs/Maths/math.vector";
import { Particle } from "babylonjs/Particles/particle";
import { IParticleEmitterType } from "babylonjs/Particles/EmitterTypes/IParticleEmitterType";
import { Nullable } from "babylonjs/types";
import { UniformBufferEffectCommonAccessor } from "babylonjs/Materials/uniformBufferEffectCommonAccessor";
import { UniformBuffer } from "babylonjs/Materials/uniformBuffer";
/**
 * Particle emitter emitting particles from a custom list of positions.
 */
export class CustomParticleEmitter implements IParticleEmitterType {
    /**
     * Gets or sets the position generator that will create the initial position of each particle.
     * Index will be provided when used with GPU particle. Particle will be provided when used with CPU particles
     */
    particlePositionGenerator: (index: number, particle: Nullable<Particle>, outPosition: Vector3) => void;
    /**
     * Gets or sets the destination generator that will create the final destination of each particle.
     *  * Index will be provided when used with GPU particle. Particle will be provided when used with CPU particles
     */
    particleDestinationGenerator: (index: number, particle: Nullable<Particle>, outDestination: Vector3) => void;
    /**
     * Creates a new instance CustomParticleEmitter
     */
    constructor();
    /**
     * Called by the particle System when the direction is computed for the created particle.
     * @param worldMatrix is the world matrix of the particle system
     * @param directionToUpdate is the direction vector to update with the result
     * @param particle is the particle we are computed the direction for
     * @param isLocal defines if the direction should be set in local space
     */
    startDirectionFunction(worldMatrix: Matrix, directionToUpdate: Vector3, particle: Particle, isLocal: boolean): void;
    /**
     * Called by the particle System when the position is computed for the created particle.
     * @param worldMatrix is the world matrix of the particle system
     * @param positionToUpdate is the position vector to update with the result
     * @param particle is the particle we are computed the position for
     * @param isLocal defines if the position should be set in local space
     */
    startPositionFunction(worldMatrix: Matrix, positionToUpdate: Vector3, particle: Particle, isLocal: boolean): void;
    /**
     * Clones the current emitter and returns a copy of it
     * @returns the new emitter
     */
    clone(): CustomParticleEmitter;
    /**
     * Called by the GPUParticleSystem to setup the update shader
     * @param uboOrEffect defines the update shader
     */
    applyToShader(uboOrEffect: UniformBufferEffectCommonAccessor): void;
    /**
     * Creates the structure of the ubo for this particle emitter
     * @param ubo ubo to create the structure for
     */
    buildUniformLayout(ubo: UniformBuffer): void;
    /**
     * Returns a string to use to update the GPU particles update shader
     * @returns a string containing the defines string
     */
    getEffectDefines(): string;
    /**
     * Returns the string "PointParticleEmitter"
     * @returns a string containing the class name
     */
    getClassName(): string;
    /**
     * Serializes the particle system to a JSON object.
     * @returns the JSON object
     */
    serialize(): any;
    /**
     * Parse properties from a JSON object
     * @param serializationObject defines the JSON object
     */
    parse(serializationObject: any): void;
}

}
declare module "babylonjs/Particles/EmitterTypes/cylinderParticleEmitter" {
import { Matrix } from "babylonjs/Maths/math.vector";
import { Vector3 } from "babylonjs/Maths/math.vector";
import { Particle } from "babylonjs/Particles/particle";
import { IParticleEmitterType } from "babylonjs/Particles/EmitterTypes/IParticleEmitterType";
import { UniformBufferEffectCommonAccessor } from "babylonjs/Materials/uniformBufferEffectCommonAccessor";
import { UniformBuffer } from "babylonjs/Materials/uniformBuffer";
/**
 * Particle emitter emitting particles from the inside of a cylinder.
 * It emits the particles alongside the cylinder radius. The emission direction might be randomized.
 */
export class CylinderParticleEmitter implements IParticleEmitterType {
    /**
     * The radius of the emission cylinder.
     */
    radius: number;
    /**
     * The height of the emission cylinder.
     */
    height: number;
    /**
     * The range of emission [0-1] 0 Surface only, 1 Entire Radius.
     */
    radiusRange: number;
    /**
     * How much to randomize the particle direction [0-1].
     */
    directionRandomizer: number;
    private _tempVector;
    /**
     * Creates a new instance CylinderParticleEmitter
     * @param radius the radius of the emission cylinder (1 by default)
     * @param height the height of the emission cylinder (1 by default)
     * @param radiusRange the range of the emission cylinder [0-1] 0 Surface only, 1 Entire Radius (1 by default)
     * @param directionRandomizer defines how much to randomize the particle direction [0-1]
     */
    constructor(
    /**
     * The radius of the emission cylinder.
     */
    radius?: number, 
    /**
     * The height of the emission cylinder.
     */
    height?: number, 
    /**
     * The range of emission [0-1] 0 Surface only, 1 Entire Radius.
     */
    radiusRange?: number, 
    /**
     * How much to randomize the particle direction [0-1].
     */
    directionRandomizer?: number);
    /**
     * Called by the particle System when the direction is computed for the created particle.
     * @param worldMatrix is the world matrix of the particle system
     * @param directionToUpdate is the direction vector to update with the result
     * @param particle is the particle we are computed the direction for
     * @param isLocal defines if the direction should be set in local space
     * @param inverseWorldMatrix defines the inverted world matrix to use if isLocal is false
     */
    startDirectionFunction(worldMatrix: Matrix, directionToUpdate: Vector3, particle: Particle, isLocal: boolean, inverseWorldMatrix: Matrix): void;
    /**
     * Called by the particle System when the position is computed for the created particle.
     * @param worldMatrix is the world matrix of the particle system
     * @param positionToUpdate is the position vector to update with the result
     * @param particle is the particle we are computed the position for
     * @param isLocal defines if the position should be set in local space
     */
    startPositionFunction(worldMatrix: Matrix, positionToUpdate: Vector3, particle: Particle, isLocal: boolean): void;
    /**
     * Clones the current emitter and returns a copy of it
     * @returns the new emitter
     */
    clone(): CylinderParticleEmitter;
    /**
     * Called by the GPUParticleSystem to setup the update shader
     * @param uboOrEffect defines the update shader
     */
    applyToShader(uboOrEffect: UniformBufferEffectCommonAccessor): void;
    /**
     * Creates the structure of the ubo for this particle emitter
     * @param ubo ubo to create the structure for
     */
    buildUniformLayout(ubo: UniformBuffer): void;
    /**
     * Returns a string to use to update the GPU particles update shader
     * @returns a string containing the defines string
     */
    getEffectDefines(): string;
    /**
     * Returns the string "CylinderParticleEmitter"
     * @returns a string containing the class name
     */
    getClassName(): string;
    /**
     * Serializes the particle system to a JSON object.
     * @returns the JSON object
     */
    serialize(): any;
    /**
     * Parse properties from a JSON object
     * @param serializationObject defines the JSON object
     */
    parse(serializationObject: any): void;
}
/**
 * Particle emitter emitting particles from the inside of a cylinder.
 * It emits the particles randomly between two vectors.
 */
export class CylinderDirectedParticleEmitter extends CylinderParticleEmitter {
    /**
     * The min limit of the emission direction.
     */
    direction1: Vector3;
    /**
     * The max limit of the emission direction.
     */
    direction2: Vector3;
    /**
     * Creates a new instance CylinderDirectedParticleEmitter
     * @param radius the radius of the emission cylinder (1 by default)
     * @param height the height of the emission cylinder (1 by default)
     * @param radiusRange the range of the emission cylinder [0-1] 0 Surface only, 1 Entire Radius (1 by default)
     * @param direction1 the min limit of the emission direction (up vector by default)
     * @param direction2 the max limit of the emission direction (up vector by default)
     */
    constructor(radius?: number, height?: number, radiusRange?: number, 
    /**
     * The min limit of the emission direction.
     */
    direction1?: Vector3, 
    /**
     * The max limit of the emission direction.
     */
    direction2?: Vector3);
    /**
     * Called by the particle System when the direction is computed for the created particle.
     * @param worldMatrix is the world matrix of the particle system
     * @param directionToUpdate is the direction vector to update with the result
     */
    startDirectionFunction(worldMatrix: Matrix, directionToUpdate: Vector3): void;
    /**
     * Clones the current emitter and returns a copy of it
     * @returns the new emitter
     */
    clone(): CylinderDirectedParticleEmitter;
    /**
     * Called by the GPUParticleSystem to setup the update shader
     * @param uboOrEffect defines the update shader
     */
    applyToShader(uboOrEffect: UniformBufferEffectCommonAccessor): void;
    /**
     * Creates the structure of the ubo for this particle emitter
     * @param ubo ubo to create the structure for
     */
    buildUniformLayout(ubo: UniformBuffer): void;
    /**
     * Returns a string to use to update the GPU particles update shader
     * @returns a string containing the defines string
     */
    getEffectDefines(): string;
    /**
     * Returns the string "CylinderDirectedParticleEmitter"
     * @returns a string containing the class name
     */
    getClassName(): string;
    /**
     * Serializes the particle system to a JSON object.
     * @returns the JSON object
     */
    serialize(): any;
    /**
     * Parse properties from a JSON object
     * @param serializationObject defines the JSON object
     */
    parse(serializationObject: any): void;
}

}
declare module "babylonjs/Particles/EmitterTypes/hemisphericParticleEmitter" {
import { Matrix } from "babylonjs/Maths/math.vector";
import { Vector3 } from "babylonjs/Maths/math.vector";
import { Particle } from "babylonjs/Particles/particle";
import { IParticleEmitterType } from "babylonjs/Particles/EmitterTypes/IParticleEmitterType";
import { UniformBufferEffectCommonAccessor } from "babylonjs/Materials/uniformBufferEffectCommonAccessor";
import { UniformBuffer } from "babylonjs/Materials/uniformBuffer";
/**
 * Particle emitter emitting particles from the inside of a hemisphere.
 * It emits the particles alongside the hemisphere radius. The emission direction might be randomized.
 */
export class HemisphericParticleEmitter implements IParticleEmitterType {
    /**
     * The radius of the emission hemisphere.
     */
    radius: number;
    /**
     * The range of emission [0-1] 0 Surface only, 1 Entire Radius.
     */
    radiusRange: number;
    /**
     * How much to randomize the particle direction [0-1].
     */
    directionRandomizer: number;
    /**
     * Creates a new instance HemisphericParticleEmitter
     * @param radius the radius of the emission hemisphere (1 by default)
     * @param radiusRange the range of the emission hemisphere [0-1] 0 Surface only, 1 Entire Radius (1 by default)
     * @param directionRandomizer defines how much to randomize the particle direction [0-1]
     */
    constructor(
    /**
     * The radius of the emission hemisphere.
     */
    radius?: number, 
    /**
     * The range of emission [0-1] 0 Surface only, 1 Entire Radius.
     */
    radiusRange?: number, 
    /**
     * How much to randomize the particle direction [0-1].
     */
    directionRandomizer?: number);
    /**
     * Called by the particle System when the direction is computed for the created particle.
     * @param worldMatrix is the world matrix of the particle system
     * @param directionToUpdate is the direction vector to update with the result
     * @param particle is the particle we are computed the direction for
     * @param isLocal defines if the direction should be set in local space
     */
    startDirectionFunction(worldMatrix: Matrix, directionToUpdate: Vector3, particle: Particle, isLocal: boolean): void;
    /**
     * Called by the particle System when the position is computed for the created particle.
     * @param worldMatrix is the world matrix of the particle system
     * @param positionToUpdate is the position vector to update with the result
     * @param particle is the particle we are computed the position for
     * @param isLocal defines if the position should be set in local space
     */
    startPositionFunction(worldMatrix: Matrix, positionToUpdate: Vector3, particle: Particle, isLocal: boolean): void;
    /**
     * Clones the current emitter and returns a copy of it
     * @returns the new emitter
     */
    clone(): HemisphericParticleEmitter;
    /**
     * Called by the GPUParticleSystem to setup the update shader
     * @param uboOrEffect defines the update shader
     */
    applyToShader(uboOrEffect: UniformBufferEffectCommonAccessor): void;
    /**
     * Creates the structure of the ubo for this particle emitter
     * @param ubo ubo to create the structure for
     */
    buildUniformLayout(ubo: UniformBuffer): void;
    /**
     * Returns a string to use to update the GPU particles update shader
     * @returns a string containing the defines string
     */
    getEffectDefines(): string;
    /**
     * Returns the string "HemisphericParticleEmitter"
     * @returns a string containing the class name
     */
    getClassName(): string;
    /**
     * Serializes the particle system to a JSON object.
     * @returns the JSON object
     */
    serialize(): any;
    /**
     * Parse properties from a JSON object
     * @param serializationObject defines the JSON object
     */
    parse(serializationObject: any): void;
}

}
declare module "babylonjs/Particles/EmitterTypes/index" {
export * from "babylonjs/Particles/EmitterTypes/boxParticleEmitter";
export * from "babylonjs/Particles/EmitterTypes/coneParticleEmitter";
export * from "babylonjs/Particles/EmitterTypes/cylinderParticleEmitter";
export * from "babylonjs/Particles/EmitterTypes/hemisphericParticleEmitter";
export * from "babylonjs/Particles/EmitterTypes/IParticleEmitterType";
export * from "babylonjs/Particles/EmitterTypes/pointParticleEmitter";
export * from "babylonjs/Particles/EmitterTypes/sphereParticleEmitter";
export * from "babylonjs/Particles/EmitterTypes/customParticleEmitter";
export * from "babylonjs/Particles/EmitterTypes/meshParticleEmitter";

}
declare module "babylonjs/Particles/EmitterTypes/IParticleEmitterType" {
import { Vector3, Matrix } from "babylonjs/Maths/math.vector";
import { Particle } from "babylonjs/Particles/particle";
import { Nullable } from "babylonjs/types";
import { UniformBufferEffectCommonAccessor } from "babylonjs/Materials/uniformBufferEffectCommonAccessor";
import { UniformBuffer } from "babylonjs/Materials/uniformBuffer";
import { Scene } from "babylonjs/scene";
/**
 * Particle emitter represents a volume emitting particles.
 * This is the responsibility of the implementation to define the volume shape like cone/sphere/box.
 */
export interface IParticleEmitterType {
    /**
     * Called by the particle System when the direction is computed for the created particle.
     * @param worldMatrix is the world matrix of the particle system
     * @param directionToUpdate is the direction vector to update with the result
     * @param particle is the particle we are computed the direction for
     * @param isLocal defines if the direction should be set in local space
     * @param inverseWorldMatrix defines the inverted world matrix to use if isLocal is false
     */
    startDirectionFunction(worldMatrix: Matrix, directionToUpdate: Vector3, particle: Particle, isLocal: boolean, inverseWorldMatrix: Matrix): void;
    /**
     * Called by the particle System when the position is computed for the created particle.
     * @param worldMatrix is the world matrix of the particle system
     * @param positionToUpdate is the position vector to update with the result
     * @param particle is the particle we are computed the position for
     * @param isLocal defines if the position should be set in local space
     */
    startPositionFunction(worldMatrix: Matrix, positionToUpdate: Vector3, particle: Particle, isLocal: boolean): void;
    /**
     * Clones the current emitter and returns a copy of it
     * @returns the new emitter
     */
    clone(): IParticleEmitterType;
    /**
     * Called by the GPUParticleSystem to setup the update shader
     * @param uboOrEffect defines the update shader
     */
    applyToShader(uboOrEffect: UniformBufferEffectCommonAccessor): void;
    /**
     * Creates the structure of the ubo for this particle emitter
     * @param ubo ubo to create the structure for
     */
    buildUniformLayout(ubo: UniformBuffer): void;
    /**
     * Returns a string to use to update the GPU particles update shader
     * @returns the effect defines string
     */
    getEffectDefines(): string;
    /**
     * Returns a string representing the class name
     * @returns a string containing the class name
     */
    getClassName(): string;
    /**
     * Serializes the particle system to a JSON object.
     * @returns the JSON object
     */
    serialize(): any;
    /**
     * Parse properties from a JSON object
     * @param serializationObject defines the JSON object
     * @param scene defines the hosting scene
     */
    parse(serializationObject: any, scene: Nullable<Scene>): void;
}
export {};

}
declare module "babylonjs/Particles/EmitterTypes/meshParticleEmitter" {
import { Matrix } from "babylonjs/Maths/math.vector";
import { Vector3 } from "babylonjs/Maths/math.vector";
import { Particle } from "babylonjs/Particles/particle";
import { IParticleEmitterType } from "babylonjs/Particles/EmitterTypes/IParticleEmitterType";
import { Nullable } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { UniformBufferEffectCommonAccessor } from "babylonjs/Materials/uniformBufferEffectCommonAccessor";
import { UniformBuffer } from "babylonjs/Materials/uniformBuffer";
/**
 * Particle emitter emitting particles from the inside of a box.
 * It emits the particles randomly between 2 given directions.
 */
export class MeshParticleEmitter implements IParticleEmitterType {
    private _indices;
    private _positions;
    private _normals;
    private _storedNormal;
    private _mesh;
    /**
     * Random direction of each particle after it has been emitted, between direction1 and direction2 vectors.
     */
    direction1: Vector3;
    /**
     * Random direction of each particle after it has been emitted, between direction1 and direction2 vectors.
     */
    direction2: Vector3;
    /**
     * Gets or sets a boolean indicating that particle directions must be built from mesh face normals
     */
    useMeshNormalsForDirection: boolean;
    /** Defines the mesh to use as source */
    get mesh(): Nullable<AbstractMesh>;
    set mesh(value: Nullable<AbstractMesh>);
    /**
     * Creates a new instance MeshParticleEmitter
     * @param mesh defines the mesh to use as source
     */
    constructor(mesh?: Nullable<AbstractMesh>);
    /**
     * Called by the particle System when the direction is computed for the created particle.
     * @param worldMatrix is the world matrix of the particle system
     * @param directionToUpdate is the direction vector to update with the result
     * @param particle is the particle we are computed the direction for
     * @param isLocal defines if the direction should be set in local space
     */
    startDirectionFunction(worldMatrix: Matrix, directionToUpdate: Vector3, particle: Particle, isLocal: boolean): void;
    /**
     * Called by the particle System when the position is computed for the created particle.
     * @param worldMatrix is the world matrix of the particle system
     * @param positionToUpdate is the position vector to update with the result
     * @param particle is the particle we are computed the position for
     * @param isLocal defines if the position should be set in local space
     */
    startPositionFunction(worldMatrix: Matrix, positionToUpdate: Vector3, particle: Particle, isLocal: boolean): void;
    /**
     * Clones the current emitter and returns a copy of it
     * @returns the new emitter
     */
    clone(): MeshParticleEmitter;
    /**
     * Called by the GPUParticleSystem to setup the update shader
     * @param uboOrEffect defines the update shader
     */
    applyToShader(uboOrEffect: UniformBufferEffectCommonAccessor): void;
    /**
     * Creates the structure of the ubo for this particle emitter
     * @param ubo ubo to create the structure for
     */
    buildUniformLayout(ubo: UniformBuffer): void;
    /**
     * Returns a string to use to update the GPU particles update shader
     * @returns a string containing the defines string
     */
    getEffectDefines(): string;
    /**
     * Returns the string "BoxParticleEmitter"
     * @returns a string containing the class name
     */
    getClassName(): string;
    /**
     * Serializes the particle system to a JSON object.
     * @returns the JSON object
     */
    serialize(): any;
    /**
     * Parse properties from a JSON object
     * @param serializationObject defines the JSON object
     * @param scene defines the hosting scene
     */
    parse(serializationObject: any, scene: Nullable<Scene>): void;
}

}
declare module "babylonjs/Particles/EmitterTypes/pointParticleEmitter" {
import { Matrix } from "babylonjs/Maths/math.vector";
import { Vector3 } from "babylonjs/Maths/math.vector";
import { Particle } from "babylonjs/Particles/particle";
import { IParticleEmitterType } from "babylonjs/Particles/EmitterTypes/IParticleEmitterType";
import { UniformBufferEffectCommonAccessor } from "babylonjs/Materials/uniformBufferEffectCommonAccessor";
import { UniformBuffer } from "babylonjs/Materials/uniformBuffer";
/**
 * Particle emitter emitting particles from a point.
 * It emits the particles randomly between 2 given directions.
 */
export class PointParticleEmitter implements IParticleEmitterType {
    /**
     * Random direction of each particle after it has been emitted, between direction1 and direction2 vectors.
     */
    direction1: Vector3;
    /**
     * Random direction of each particle after it has been emitted, between direction1 and direction2 vectors.
     */
    direction2: Vector3;
    /**
     * Creates a new instance PointParticleEmitter
     */
    constructor();
    /**
     * Called by the particle System when the direction is computed for the created particle.
     * @param worldMatrix is the world matrix of the particle system
     * @param directionToUpdate is the direction vector to update with the result
     * @param particle is the particle we are computed the direction for
     * @param isLocal defines if the direction should be set in local space
     */
    startDirectionFunction(worldMatrix: Matrix, directionToUpdate: Vector3, particle: Particle, isLocal: boolean): void;
    /**
     * Called by the particle System when the position is computed for the created particle.
     * @param worldMatrix is the world matrix of the particle system
     * @param positionToUpdate is the position vector to update with the result
     * @param particle is the particle we are computed the position for
     * @param isLocal defines if the position should be set in local space
     */
    startPositionFunction(worldMatrix: Matrix, positionToUpdate: Vector3, particle: Particle, isLocal: boolean): void;
    /**
     * Clones the current emitter and returns a copy of it
     * @returns the new emitter
     */
    clone(): PointParticleEmitter;
    /**
     * Called by the GPUParticleSystem to setup the update shader
     * @param uboOrEffect defines the update shader
     */
    applyToShader(uboOrEffect: UniformBufferEffectCommonAccessor): void;
    /**
     * Creates the structure of the ubo for this particle emitter
     * @param ubo ubo to create the structure for
     */
    buildUniformLayout(ubo: UniformBuffer): void;
    /**
     * Returns a string to use to update the GPU particles update shader
     * @returns a string containing the defines string
     */
    getEffectDefines(): string;
    /**
     * Returns the string "PointParticleEmitter"
     * @returns a string containing the class name
     */
    getClassName(): string;
    /**
     * Serializes the particle system to a JSON object.
     * @returns the JSON object
     */
    serialize(): any;
    /**
     * Parse properties from a JSON object
     * @param serializationObject defines the JSON object
     */
    parse(serializationObject: any): void;
}

}
declare module "babylonjs/Particles/EmitterTypes/sphereParticleEmitter" {
import { Matrix } from "babylonjs/Maths/math.vector";
import { Vector3 } from "babylonjs/Maths/math.vector";
import { Particle } from "babylonjs/Particles/particle";
import { IParticleEmitterType } from "babylonjs/Particles/EmitterTypes/IParticleEmitterType";
import { UniformBufferEffectCommonAccessor } from "babylonjs/Materials/uniformBufferEffectCommonAccessor";
import { UniformBuffer } from "babylonjs/Materials/uniformBuffer";
/**
 * Particle emitter emitting particles from the inside of a sphere.
 * It emits the particles alongside the sphere radius. The emission direction might be randomized.
 */
export class SphereParticleEmitter implements IParticleEmitterType {
    /**
     * The radius of the emission sphere.
     */
    radius: number;
    /**
     * The range of emission [0-1] 0 Surface only, 1 Entire Radius.
     */
    radiusRange: number;
    /**
     * How much to randomize the particle direction [0-1].
     */
    directionRandomizer: number;
    /**
     * Creates a new instance SphereParticleEmitter
     * @param radius the radius of the emission sphere (1 by default)
     * @param radiusRange the range of the emission sphere [0-1] 0 Surface only, 1 Entire Radius (1 by default)
     * @param directionRandomizer defines how much to randomize the particle direction [0-1]
     */
    constructor(
    /**
     * The radius of the emission sphere.
     */
    radius?: number, 
    /**
     * The range of emission [0-1] 0 Surface only, 1 Entire Radius.
     */
    radiusRange?: number, 
    /**
     * How much to randomize the particle direction [0-1].
     */
    directionRandomizer?: number);
    /**
     * Called by the particle System when the direction is computed for the created particle.
     * @param worldMatrix is the world matrix of the particle system
     * @param directionToUpdate is the direction vector to update with the result
     * @param particle is the particle we are computed the direction for
     * @param isLocal defines if the direction should be set in local space
     */
    startDirectionFunction(worldMatrix: Matrix, directionToUpdate: Vector3, particle: Particle, isLocal: boolean): void;
    /**
     * Called by the particle System when the position is computed for the created particle.
     * @param worldMatrix is the world matrix of the particle system
     * @param positionToUpdate is the position vector to update with the result
     * @param particle is the particle we are computed the position for
     * @param isLocal defines if the position should be set in local space
     */
    startPositionFunction(worldMatrix: Matrix, positionToUpdate: Vector3, particle: Particle, isLocal: boolean): void;
    /**
     * Clones the current emitter and returns a copy of it
     * @returns the new emitter
     */
    clone(): SphereParticleEmitter;
    /**
     * Called by the GPUParticleSystem to setup the update shader
     * @param uboOrEffect defines the update shader
     */
    applyToShader(uboOrEffect: UniformBufferEffectCommonAccessor): void;
    /**
     * Creates the structure of the ubo for this particle emitter
     * @param ubo ubo to create the structure for
     */
    buildUniformLayout(ubo: UniformBuffer): void;
    /**
     * Returns a string to use to update the GPU particles update shader
     * @returns a string containing the defines string
     */
    getEffectDefines(): string;
    /**
     * Returns the string "SphereParticleEmitter"
     * @returns a string containing the class name
     */
    getClassName(): string;
    /**
     * Serializes the particle system to a JSON object.
     * @returns the JSON object
     */
    serialize(): any;
    /**
     * Parse properties from a JSON object
     * @param serializationObject defines the JSON object
     */
    parse(serializationObject: any): void;
}
/**
 * Particle emitter emitting particles from the inside of a sphere.
 * It emits the particles randomly between two vectors.
 */
export class SphereDirectedParticleEmitter extends SphereParticleEmitter {
    /**
     * The min limit of the emission direction.
     */
    direction1: Vector3;
    /**
     * The max limit of the emission direction.
     */
    direction2: Vector3;
    /**
     * Creates a new instance SphereDirectedParticleEmitter
     * @param radius the radius of the emission sphere (1 by default)
     * @param direction1 the min limit of the emission direction (up vector by default)
     * @param direction2 the max limit of the emission direction (up vector by default)
     */
    constructor(radius?: number, 
    /**
     * The min limit of the emission direction.
     */
    direction1?: Vector3, 
    /**
     * The max limit of the emission direction.
     */
    direction2?: Vector3);
    /**
     * Called by the particle System when the direction is computed for the created particle.
     * @param worldMatrix is the world matrix of the particle system
     * @param directionToUpdate is the direction vector to update with the result
     */
    startDirectionFunction(worldMatrix: Matrix, directionToUpdate: Vector3): void;
    /**
     * Clones the current emitter and returns a copy of it
     * @returns the new emitter
     */
    clone(): SphereDirectedParticleEmitter;
    /**
     * Called by the GPUParticleSystem to setup the update shader
     * @param uboOrEffect defines the update shader
     */
    applyToShader(uboOrEffect: UniformBufferEffectCommonAccessor): void;
    /**
     * Creates the structure of the ubo for this particle emitter
     * @param ubo ubo to create the structure for
     */
    buildUniformLayout(ubo: UniformBuffer): void;
    /**
     * Returns a string to use to update the GPU particles update shader
     * @returns a string containing the defines string
     */
    getEffectDefines(): string;
    /**
     * Returns the string "SphereDirectedParticleEmitter"
     * @returns a string containing the class name
     */
    getClassName(): string;
    /**
     * Serializes the particle system to a JSON object.
     * @returns the JSON object
     */
    serialize(): any;
    /**
     * Parse properties from a JSON object
     * @param serializationObject defines the JSON object
     */
    parse(serializationObject: any): void;
}

}
declare module "babylonjs/Particles/gpuParticleSystem" {
import { Immutable, Nullable } from "babylonjs/types";
import { Color3Gradient, IValueGradient } from "babylonjs/Misc/gradients";
import { Observable } from "babylonjs/Misc/observable";
import { Matrix } from "babylonjs/Maths/math.vector";
import { Color4 } from "babylonjs/Maths/math.color";
import { VertexBuffer } from "babylonjs/Buffers/buffer";
import { IParticleSystem } from "babylonjs/Particles/IParticleSystem";
import { BaseParticleSystem } from "babylonjs/Particles/baseParticleSystem";
import { IDisposable } from "babylonjs/scene";
import { Effect } from "babylonjs/Materials/effect";
import { RawTexture } from "babylonjs/Materials/Textures/rawTexture";
import { IAnimatable } from "babylonjs/Animations/animatable.interface";
import { ThinEngine } from "babylonjs/Engines/thinEngine";
import { DataBuffer } from "babylonjs/Buffers/dataBuffer";
import { DrawWrapper } from "babylonjs/Materials/drawWrapper";
import { Scene } from "babylonjs/scene";
import "babylonjs/Shaders/gpuRenderParticles.fragment";
import "babylonjs/Shaders/gpuRenderParticles.vertex";
/**
 * This represents a GPU particle system in Babylon
 * This is the fastest particle system in Babylon as it uses the GPU to update the individual particle data
 * @see https://www.babylonjs-playground.com/#PU4WYI#4
 */
export class GPUParticleSystem extends BaseParticleSystem implements IDisposable, IParticleSystem, IAnimatable {
    /**
     * The layer mask we are rendering the particles through.
     */
    layerMask: number;
    private _capacity;
    private _activeCount;
    private _currentActiveCount;
    private _accumulatedCount;
    private _updateBuffer;
    private _buffer0;
    private _buffer1;
    private _spriteBuffer;
    private _renderVertexBuffers;
    private _targetIndex;
    private _sourceBuffer;
    private _targetBuffer;
    private _currentRenderId;
    private _currentRenderingCameraUniqueId;
    private _started;
    private _stopped;
    private _timeDelta;
    /** @internal */
    _randomTexture: RawTexture;
    /** @internal */
    _randomTexture2: RawTexture;
    /** Indicates that the update of particles is done in the animate function (and not in render). Default: false */
    updateInAnimate: boolean;
    private _attributesStrideSize;
    private _cachedUpdateDefines;
    private _randomTextureSize;
    private _actualFrame;
    private _drawWrappers;
    private _customWrappers;
    private readonly _rawTextureWidth;
    private _platform;
    /**
     * Gets a boolean indicating if the GPU particles can be rendered on current browser
     */
    static get IsSupported(): boolean;
    /**
     * An event triggered when the system is disposed.
     */
    onDisposeObservable: Observable<IParticleSystem>;
    /**
     * An event triggered when the system is stopped
     */
    onStoppedObservable: Observable<IParticleSystem>;
    /**
     * Gets the maximum number of particles active at the same time.
     * @returns The max number of active particles.
     */
    getCapacity(): number;
    /**
     * Forces the particle to write their depth information to the depth buffer. This can help preventing other draw calls
     * to override the particles.
     */
    forceDepthWrite: boolean;
    /**
     * Gets or set the number of active particles
     */
    get activeParticleCount(): number;
    set activeParticleCount(value: number);
    private _preWarmDone;
    /**
     * Specifies if the particles are updated in emitter local space or world space.
     */
    isLocal: boolean;
    /** Indicates that the particle system is GPU based */
    readonly isGPU: boolean;
    /** Gets or sets a matrix to use to compute projection */
    defaultProjectionMatrix: Matrix;
    /**
     * Is this system ready to be used/rendered
     * @returns true if the system is ready
     */
    isReady(): boolean;
    /**
     * Gets if the system has been started. (Note: this will still be true after stop is called)
     * @returns True if it has been started, otherwise false.
     */
    isStarted(): boolean;
    /**
     * Gets if the system has been stopped. (Note: rendering is still happening but the system is frozen)
     * @returns True if it has been stopped, otherwise false.
     */
    isStopped(): boolean;
    /**
     * Gets a boolean indicating that the system is stopping
     * @returns true if the system is currently stopping
     */
    isStopping(): boolean;
    /**
     * Gets the number of particles active at the same time.
     * @returns The number of active particles.
     */
    getActiveCount(): number;
    /**
     * Starts the particle system and begins to emit
     * @param delay defines the delay in milliseconds before starting the system (this.startDelay by default)
     */
    start(delay?: number): void;
    /**
     * Stops the particle system.
     */
    stop(): void;
    /**
     * Remove all active particles
     */
    reset(): void;
    /**
     * Returns the string "GPUParticleSystem"
     * @returns a string containing the class name
     */
    getClassName(): string;
    /**
     * Gets the custom effect used to render the particles
     * @param blendMode Blend mode for which the effect should be retrieved
     * @returns The effect
     */
    getCustomEffect(blendMode?: number): Nullable<Effect>;
    private _getCustomDrawWrapper;
    /**
     * Sets the custom effect used to render the particles
     * @param effect The effect to set
     * @param blendMode Blend mode for which the effect should be set
     */
    setCustomEffect(effect: Nullable<Effect>, blendMode?: number): void;
    /** @internal */
    protected _onBeforeDrawParticlesObservable: Nullable<Observable<Nullable<Effect>>>;
    /**
     * Observable that will be called just before the particles are drawn
     */
    get onBeforeDrawParticlesObservable(): Observable<Nullable<Effect>>;
    /**
     * Gets the name of the particle vertex shader
     */
    get vertexShaderName(): string;
    /**
     * Gets the vertex buffers used by the particle system
     * Should be called after render() has been called for the current frame so that the buffers returned are the ones that have been updated
     * in the current frame (there's a ping-pong between two sets of buffers - for a given frame, one set is used as the source and the other as the destination)
     */
    get vertexBuffers(): Immutable<{
        [key: string]: VertexBuffer;
    }>;
    /**
     * Gets the index buffer used by the particle system (null for GPU particle systems)
     */
    get indexBuffer(): Nullable<DataBuffer>;
    /** @internal */
    _colorGradientsTexture: RawTexture;
    protected _removeGradientAndTexture(gradient: number, gradients: Nullable<IValueGradient[]>, texture: RawTexture): BaseParticleSystem;
    /**
     * Adds a new color gradient
     * @param gradient defines the gradient to use (between 0 and 1)
     * @param color1 defines the color to affect to the specified gradient
     * @returns the current particle system
     */
    addColorGradient(gradient: number, color1: Color4): GPUParticleSystem;
    private _refreshColorGradient;
    /** Force the system to rebuild all gradients that need to be resync */
    forceRefreshGradients(): void;
    /**
     * Remove a specific color gradient
     * @param gradient defines the gradient to remove
     * @returns the current particle system
     */
    removeColorGradient(gradient: number): GPUParticleSystem;
    /**
     * Resets the draw wrappers cache
     */
    resetDrawCache(): void;
    /** @internal */
    _angularSpeedGradientsTexture: RawTexture;
    /** @internal */
    _sizeGradientsTexture: RawTexture;
    /** @internal */
    _velocityGradientsTexture: RawTexture;
    /** @internal */
    _limitVelocityGradientsTexture: RawTexture;
    /** @internal */
    _dragGradientsTexture: RawTexture;
    private _addFactorGradient;
    /**
     * Adds a new size gradient
     * @param gradient defines the gradient to use (between 0 and 1)
     * @param factor defines the size factor to affect to the specified gradient
     * @returns the current particle system
     */
    addSizeGradient(gradient: number, factor: number): GPUParticleSystem;
    /**
     * Remove a specific size gradient
     * @param gradient defines the gradient to remove
     * @returns the current particle system
     */
    removeSizeGradient(gradient: number): GPUParticleSystem;
    private _refreshFactorGradient;
    /**
     * Adds a new angular speed gradient
     * @param gradient defines the gradient to use (between 0 and 1)
     * @param factor defines the angular speed to affect to the specified gradient
     * @returns the current particle system
     */
    addAngularSpeedGradient(gradient: number, factor: number): GPUParticleSystem;
    /**
     * Remove a specific angular speed gradient
     * @param gradient defines the gradient to remove
     * @returns the current particle system
     */
    removeAngularSpeedGradient(gradient: number): GPUParticleSystem;
    /**
     * Adds a new velocity gradient
     * @param gradient defines the gradient to use (between 0 and 1)
     * @param factor defines the velocity to affect to the specified gradient
     * @returns the current particle system
     */
    addVelocityGradient(gradient: number, factor: number): GPUParticleSystem;
    /**
     * Remove a specific velocity gradient
     * @param gradient defines the gradient to remove
     * @returns the current particle system
     */
    removeVelocityGradient(gradient: number): GPUParticleSystem;
    /**
     * Adds a new limit velocity gradient
     * @param gradient defines the gradient to use (between 0 and 1)
     * @param factor defines the limit velocity value to affect to the specified gradient
     * @returns the current particle system
     */
    addLimitVelocityGradient(gradient: number, factor: number): GPUParticleSystem;
    /**
     * Remove a specific limit velocity gradient
     * @param gradient defines the gradient to remove
     * @returns the current particle system
     */
    removeLimitVelocityGradient(gradient: number): GPUParticleSystem;
    /**
     * Adds a new drag gradient
     * @param gradient defines the gradient to use (between 0 and 1)
     * @param factor defines the drag value to affect to the specified gradient
     * @returns the current particle system
     */
    addDragGradient(gradient: number, factor: number): GPUParticleSystem;
    /**
     * Remove a specific drag gradient
     * @param gradient defines the gradient to remove
     * @returns the current particle system
     */
    removeDragGradient(gradient: number): GPUParticleSystem;
    /**
     * Not supported by GPUParticleSystem
     * @returns the current particle system
     */
    addEmitRateGradient(): IParticleSystem;
    /**
     * Not supported by GPUParticleSystem
     * @returns the current particle system
     */
    removeEmitRateGradient(): IParticleSystem;
    /**
     * Not supported by GPUParticleSystem
     * @returns the current particle system
     */
    addStartSizeGradient(): IParticleSystem;
    /**
     * Not supported by GPUParticleSystem
     * @returns the current particle system
     */
    removeStartSizeGradient(): IParticleSystem;
    /**
     * Not supported by GPUParticleSystem
     * @returns the current particle system
     */
    addColorRemapGradient(): IParticleSystem;
    /**
     * Not supported by GPUParticleSystem
     * @returns the current particle system
     */
    removeColorRemapGradient(): IParticleSystem;
    /**
     * Not supported by GPUParticleSystem
     * @returns the current particle system
     */
    addAlphaRemapGradient(): IParticleSystem;
    /**
     * Not supported by GPUParticleSystem
     * @returns the current particle system
     */
    removeAlphaRemapGradient(): IParticleSystem;
    /**
     * Not supported by GPUParticleSystem
     * @returns the current particle system
     */
    addRampGradient(): IParticleSystem;
    /**
     * Not supported by GPUParticleSystem
     * @returns the current particle system
     */
    removeRampGradient(): IParticleSystem;
    /**
     * Not supported by GPUParticleSystem
     * @returns the list of ramp gradients
     */
    getRampGradients(): Nullable<Array<Color3Gradient>>;
    /**
     * Not supported by GPUParticleSystem
     * Gets or sets a boolean indicating that ramp gradients must be used
     * @see https://doc.babylonjs.com/features/featuresDeepDive/particles/particle_system/particle_system_intro#ramp-gradients
     */
    get useRampGradients(): boolean;
    set useRampGradients(value: boolean);
    /**
     * Not supported by GPUParticleSystem
     * @returns the current particle system
     */
    addLifeTimeGradient(): IParticleSystem;
    /**
     * Not supported by GPUParticleSystem
     * @returns the current particle system
     */
    removeLifeTimeGradient(): IParticleSystem;
    /**
     * Instantiates a GPU particle system.
     * Particles are often small sprites used to simulate hard-to-reproduce phenomena like fire, smoke, water, or abstract visual effects like magic glitter and faery dust.
     * @param name The name of the particle system
     * @param options The options used to create the system
     * @param sceneOrEngine The scene the particle system belongs to or the engine to use if no scene
     * @param customEffect a custom effect used to change the way particles are rendered by default
     * @param isAnimationSheetEnabled Must be true if using a spritesheet to animate the particles texture
     */
    constructor(name: string, options: Partial<{
        capacity: number;
        randomTextureSize: number;
    }>, sceneOrEngine: Scene | ThinEngine, customEffect?: Nullable<Effect>, isAnimationSheetEnabled?: boolean);
    protected _reset(): void;
    private _createVertexBuffers;
    private _initialize;
    /** @internal */
    _recreateUpdateEffect(): boolean;
    /**
     * @internal
     */
    _getWrapper(blendMode: number): DrawWrapper;
    /**
     * @internal
     */
    static _GetAttributeNamesOrOptions(hasColorGradients?: boolean, isAnimationSheetEnabled?: boolean, isBillboardBased?: boolean, isBillboardStretched?: boolean): string[];
    /**
     * @internal
     */
    static _GetEffectCreationOptions(isAnimationSheetEnabled?: boolean, useLogarithmicDepth?: boolean): string[];
    /**
     * Fill the defines array according to the current settings of the particle system
     * @param defines Array to be updated
     * @param blendMode blend mode to take into account when updating the array
     */
    fillDefines(defines: Array<string>, blendMode?: number): void;
    /**
     * Fill the uniforms, attributes and samplers arrays according to the current settings of the particle system
     * @param uniforms Uniforms array to fill
     * @param attributes Attributes array to fill
     * @param samplers Samplers array to fill
     */
    fillUniformsAttributesAndSamplerNames(uniforms: Array<string>, attributes: Array<string>, samplers: Array<string>): void;
    /**
     * Animates the particle system for the current frame by emitting new particles and or animating the living ones.
     * @param preWarm defines if we are in the pre-warmimg phase
     */
    animate(preWarm?: boolean): void;
    private _createFactorGradientTexture;
    private _createSizeGradientTexture;
    private _createAngularSpeedGradientTexture;
    private _createVelocityGradientTexture;
    private _createLimitVelocityGradientTexture;
    private _createDragGradientTexture;
    private _createColorGradientTexture;
    private _render;
    /** @internal */
    _update(emitterWM?: Matrix): void;
    /**
     * Renders the particle system in its current state
     * @param preWarm defines if the system should only update the particles but not render them
     * @param forceUpdateOnly if true, force to only update the particles and never display them (meaning, even if preWarm=false, when forceUpdateOnly=true the particles won't be displayed)
     * @returns the current number of particles
     */
    render(preWarm?: boolean, forceUpdateOnly?: boolean): number;
    /**
     * Rebuilds the particle system
     */
    rebuild(): void;
    private _releaseBuffers;
    /**
     * Disposes the particle system and free the associated resources
     * @param disposeTexture defines if the particule texture must be disposed as well (true by default)
     */
    dispose(disposeTexture?: boolean): void;
    /**
     * Clones the particle system.
     * @param name The name of the cloned object
     * @param newEmitter The new emitter to use
     * @param cloneTexture Also clone the textures if true
     * @returns the cloned particle system
     */
    clone(name: string, newEmitter: any, cloneTexture?: boolean): GPUParticleSystem;
    /**
     * Serializes the particle system to a JSON object
     * @param serializeTexture defines if the texture must be serialized as well
     * @returns the JSON object
     */
    serialize(serializeTexture?: boolean): any;
    /**
     * Parses a JSON object to create a GPU particle system.
     * @param parsedParticleSystem The JSON object to parse
     * @param sceneOrEngine The scene or the engine to create the particle system in
     * @param rootUrl The root url to use to load external dependencies like texture
     * @param doNotStart Ignore the preventAutoStart attribute and does not start
     * @param capacity defines the system capacity (if null or undefined the sotred capacity will be used)
     * @returns the parsed GPU particle system
     */
    static Parse(parsedParticleSystem: any, sceneOrEngine: Scene | ThinEngine, rootUrl: string, doNotStart?: boolean, capacity?: number): GPUParticleSystem;
}
export {};

}
declare module "babylonjs/Particles/IGPUParticleSystemPlatform" {
import { Buffer, VertexBuffer } from "babylonjs/Buffers/buffer";
import { DataBuffer } from "babylonjs/Buffers/dataBuffer";
import { Effect } from "babylonjs/Materials/effect";
import { UniformBufferEffectCommonAccessor } from "babylonjs/Materials/uniformBufferEffectCommonAccessor";
import { DataArray } from "babylonjs/types";
/** @internal */
export interface IGPUParticleSystemPlatform {
    alignDataInBuffer: boolean;
    isUpdateBufferCreated: () => boolean;
    isUpdateBufferReady: () => boolean;
    createUpdateBuffer: (defines: string) => UniformBufferEffectCommonAccessor;
    createVertexBuffers: (updateBuffer: Buffer, renderVertexBuffers: {
        [key: string]: VertexBuffer;
    }) => void;
    createParticleBuffer: (data: number[]) => DataArray | DataBuffer;
    bindDrawBuffers: (index: number, effect: Effect) => void;
    preUpdateParticleBuffer: () => void;
    updateParticleBuffer: (index: number, targetBuffer: Buffer, currentActiveCount: number) => void;
    releaseBuffers: () => void;
    releaseVertexBuffers: () => void;
}

}
declare module "babylonjs/Particles/index" {
export * from "babylonjs/Particles/baseParticleSystem";
export * from "babylonjs/Particles/EmitterTypes/index";
export * from "babylonjs/Particles/webgl2ParticleSystem";
export * from "babylonjs/Particles/computeShaderParticleSystem";
export * from "babylonjs/Particles/gpuParticleSystem";
export * from "babylonjs/Particles/IParticleSystem";
export * from "babylonjs/Particles/particle";
export * from "babylonjs/Particles/particleHelper";
export * from "babylonjs/Particles/particleSystem";
import "babylonjs/Particles/particleSystemComponent";
export * from "babylonjs/Particles/particleSystemComponent";
export * from "babylonjs/Particles/particleSystemSet";
export * from "babylonjs/Particles/solidParticle";
export * from "babylonjs/Particles/solidParticleSystem";
export * from "babylonjs/Particles/cloudPoint";
export * from "babylonjs/Particles/pointsCloudSystem";
export * from "babylonjs/Particles/subEmitter";

}
declare module "babylonjs/Particles/IParticleSystem" {
import { Immutable, Nullable } from "babylonjs/types";
import { Vector2, Vector3, Matrix } from "babylonjs/Maths/math.vector";
import { Color3, Color4 } from "babylonjs/Maths/math.color";
import { BaseTexture } from "babylonjs/Materials/Textures/baseTexture";
import { BoxParticleEmitter, IParticleEmitterType, PointParticleEmitter, HemisphericParticleEmitter, SphereParticleEmitter, SphereDirectedParticleEmitter, CylinderParticleEmitter, ConeParticleEmitter } from "babylonjs/Particles/EmitterTypes/index";
import { Scene } from "babylonjs/scene";
import { ColorGradient, FactorGradient, Color3Gradient } from "babylonjs/Misc/gradients";
import { Effect } from "babylonjs/Materials/effect";
import { Observable } from "babylonjs/Misc/observable";
import { VertexBuffer } from "babylonjs/Buffers/buffer";
import { DataBuffer } from "babylonjs/Buffers/dataBuffer";
import { Animation } from "babylonjs/Animations/animation";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
/**
 * Interface representing a particle system in Babylon.js.
 * This groups the common functionalities that needs to be implemented in order to create a particle system.
 * A particle system represents a way to manage particles from their emission to their animation and rendering.
 */
export interface IParticleSystem {
    /**
     * List of animations used by the particle system.
     */
    animations: Animation[];
    /**
     * The id of the Particle system.
     */
    id: string;
    /**
     * The name of the Particle system.
     */
    name: string;
    /**
     * The emitter represents the Mesh or position we are attaching the particle system to.
     */
    emitter: Nullable<AbstractMesh | Vector3>;
    /**
     * Gets or sets a boolean indicating if the particles must be rendered as billboard or aligned with the direction
     */
    isBillboardBased: boolean;
    /**
     * The rendering group used by the Particle system to chose when to render.
     */
    renderingGroupId: number;
    /**
     * The layer mask we are rendering the particles through.
     */
    layerMask: number;
    /**
     * The overall motion speed (0.01 is default update speed, faster updates = faster animation)
     */
    updateSpeed: number;
    /**
     * The amount of time the particle system is running (depends of the overall update speed).
     */
    targetStopDuration: number;
    /**
     * The texture used to render each particle. (this can be a spritesheet)
     */
    particleTexture: Nullable<BaseTexture>;
    /**
     * Blend mode use to render the particle, it can be either ParticleSystem.BLENDMODE_ONEONE, ParticleSystem.BLENDMODE_STANDARD or ParticleSystem.BLENDMODE_ADD.
     */
    blendMode: number;
    /**
     * Minimum life time of emitting particles.
     */
    minLifeTime: number;
    /**
     * Maximum life time of emitting particles.
     */
    maxLifeTime: number;
    /**
     * Minimum Size of emitting particles.
     */
    minSize: number;
    /**
     * Maximum Size of emitting particles.
     */
    maxSize: number;
    /**
     * Minimum scale of emitting particles on X axis.
     */
    minScaleX: number;
    /**
     * Maximum scale of emitting particles on X axis.
     */
    maxScaleX: number;
    /**
     * Minimum scale of emitting particles on Y axis.
     */
    minScaleY: number;
    /**
     * Maximum scale of emitting particles on Y axis.
     */
    maxScaleY: number;
    /**
     * Random color of each particle after it has been emitted, between color1 and color2 vectors.
     */
    color1: Color4;
    /**
     * Random color of each particle after it has been emitted, between color1 and color2 vectors.
     */
    color2: Color4;
    /**
     * Color the particle will have at the end of its lifetime.
     */
    colorDead: Color4;
    /**
     * The maximum number of particles to emit per frame until we reach the activeParticleCount value
     */
    emitRate: number;
    /**
     * You can use gravity if you want to give an orientation to your particles.
     */
    gravity: Vector3;
    /**
     * Minimum power of emitting particles.
     */
    minEmitPower: number;
    /**
     * Maximum power of emitting particles.
     */
    maxEmitPower: number;
    /**
     * Minimum angular speed of emitting particles (Z-axis rotation for each particle).
     */
    minAngularSpeed: number;
    /**
     * Maximum angular speed of emitting particles (Z-axis rotation for each particle).
     */
    maxAngularSpeed: number;
    /**
     * Gets or sets the minimal initial rotation in radians.
     */
    minInitialRotation: number;
    /**
     * Gets or sets the maximal initial rotation in radians.
     */
    maxInitialRotation: number;
    /**
     * The particle emitter type defines the emitter used by the particle system.
     * It can be for example box, sphere, or cone...
     */
    particleEmitterType: Nullable<IParticleEmitterType>;
    /**
     * Defines the delay in milliseconds before starting the system (0 by default)
     */
    startDelay: number;
    /**
     * Gets or sets a value indicating how many cycles (or frames) must be executed before first rendering (this value has to be set before starting the system). Default is 0
     */
    preWarmCycles: number;
    /**
     * Gets or sets a value indicating the time step multiplier to use in pre-warm mode (default is 1)
     */
    preWarmStepOffset: number;
    /**
     * If using a spritesheet (isAnimationSheetEnabled) defines the speed of the sprite loop (default is 1 meaning the animation will play once during the entire particle lifetime)
     */
    spriteCellChangeSpeed: number;
    /**
     * If using a spritesheet (isAnimationSheetEnabled) defines the first sprite cell to display
     */
    startSpriteCellID: number;
    /**
     * If using a spritesheet (isAnimationSheetEnabled) defines the last sprite cell to display
     */
    endSpriteCellID: number;
    /**
     * If using a spritesheet (isAnimationSheetEnabled), defines whether the sprite animation is looping
     */
    spriteCellLoop: boolean;
    /**
     * If using a spritesheet (isAnimationSheetEnabled), defines the sprite cell width to use
     */
    spriteCellWidth: number;
    /**
     * If using a spritesheet (isAnimationSheetEnabled), defines the sprite cell height to use
     */
    spriteCellHeight: number;
    /**
     * This allows the system to random pick the start cell ID between startSpriteCellID and endSpriteCellID
     */
    spriteRandomStartCell: boolean;
    /**
     * Gets or sets a boolean indicating if a spritesheet is used to animate the particles texture
     */
    isAnimationSheetEnabled: boolean;
    /** Gets or sets a Vector2 used to move the pivot (by default (0,0)) */
    translationPivot: Vector2;
    /**
     * Gets or sets a texture used to add random noise to particle positions
     */
    noiseTexture: Nullable<BaseTexture>;
    /** Gets or sets the strength to apply to the noise value (default is (10, 10, 10)) */
    noiseStrength: Vector3;
    /**
     * Gets or sets the billboard mode to use when isBillboardBased = true.
     * Value can be: ParticleSystem.BILLBOARDMODE_ALL, ParticleSystem.BILLBOARDMODE_Y, ParticleSystem.BILLBOARDMODE_STRETCHED
     */
    billboardMode: number;
    /**
     * Gets or sets a boolean enabling the use of logarithmic depth buffers, which is good for wide depth buffers.
     */
    useLogarithmicDepth: boolean;
    /** Gets or sets a value indicating the damping to apply if the limit velocity factor is reached */
    limitVelocityDamping: number;
    /**
     * Gets or sets a boolean indicating that hosted animations (in the system.animations array) must be started when system.start() is called
     */
    beginAnimationOnStart: boolean;
    /**
     * Gets or sets the frame to start the animation from when beginAnimationOnStart is true
     */
    beginAnimationFrom: number;
    /**
     * Gets or sets the frame to end the animation on when beginAnimationOnStart is true
     */
    beginAnimationTo: number;
    /**
     * Gets or sets a boolean indicating if animations must loop when beginAnimationOnStart is true
     */
    beginAnimationLoop: boolean;
    /**
     * Specifies whether the particle system will be disposed once it reaches the end of the animation.
     */
    disposeOnStop: boolean;
    /**
     * If you want to launch only a few particles at once, that can be done, as well.
     */
    manualEmitCount: number;
    /**
     * Specifies if the particles are updated in emitter local space or world space
     */
    isLocal: boolean;
    /** Snippet ID if the particle system was created from the snippet server */
    snippetId: string;
    /** Gets or sets a matrix to use to compute projection */
    defaultProjectionMatrix: Matrix;
    /** Indicates that the update of particles is done in the animate function (and not in render) */
    updateInAnimate: boolean;
    /** @internal */
    _wasDispatched: boolean;
    /**
     * Gets the maximum number of particles active at the same time.
     * @returns The max number of active particles.
     */
    getCapacity(): number;
    /**
     * Gets the number of particles active at the same time.
     * @returns The number of active particles.
     */
    getActiveCount(): number;
    /**
     * Gets if the system has been started. (Note: this will still be true after stop is called)
     * @returns True if it has been started, otherwise false.
     */
    isStarted(): boolean;
    /**
     * Animates the particle system for this frame.
     */
    animate(): void;
    /**
     * Renders the particle system in its current state.
     * @returns the current number of particles
     */
    render(): number;
    /**
     * Dispose the particle system and frees its associated resources.
     * @param disposeTexture defines if the particle texture must be disposed as well (true by default)
     */
    dispose(disposeTexture?: boolean): void;
    /**
     * An event triggered when the system is disposed
     */
    onDisposeObservable: Observable<IParticleSystem>;
    /**
     * An event triggered when the system is stopped
     */
    onStoppedObservable: Observable<IParticleSystem>;
    /**
     * Clones the particle system.
     * @param name The name of the cloned object
     * @param newEmitter The new emitter to use
     * @returns the cloned particle system
     */
    clone(name: string, newEmitter: any): Nullable<IParticleSystem>;
    /**
     * Serializes the particle system to a JSON object
     * @param serializeTexture defines if the texture must be serialized as well
     * @returns the JSON object
     */
    serialize(serializeTexture: boolean): any;
    /**
     * Rebuild the particle system
     */
    rebuild(): void;
    /** Force the system to rebuild all gradients that need to be resync */
    forceRefreshGradients(): void;
    /**
     * Starts the particle system and begins to emit
     * @param delay defines the delay in milliseconds before starting the system (0 by default)
     */
    start(delay?: number): void;
    /**
     * Stops the particle system.
     */
    stop(): void;
    /**
     * Remove all active particles
     */
    reset(): void;
    /**
     * Gets a boolean indicating that the system is stopping
     * @returns true if the system is currently stopping
     */
    isStopping(): boolean;
    /**
     * Is this system ready to be used/rendered
     * @returns true if the system is ready
     */
    isReady(): boolean;
    /**
     * Returns the string "ParticleSystem"
     * @returns a string containing the class name
     */
    getClassName(): string;
    /**
     * Gets the custom effect used to render the particles
     * @param blendMode Blend mode for which the effect should be retrieved
     * @returns The effect
     */
    getCustomEffect(blendMode: number): Nullable<Effect>;
    /**
     * Sets the custom effect used to render the particles
     * @param effect The effect to set
     * @param blendMode Blend mode for which the effect should be set
     */
    setCustomEffect(effect: Nullable<Effect>, blendMode: number): void;
    /**
     * Fill the defines array according to the current settings of the particle system
     * @param defines Array to be updated
     * @param blendMode blend mode to take into account when updating the array
     */
    fillDefines(defines: Array<string>, blendMode: number): void;
    /**
     * Fill the uniforms, attributes and samplers arrays according to the current settings of the particle system
     * @param uniforms Uniforms array to fill
     * @param attributes Attributes array to fill
     * @param samplers Samplers array to fill
     */
    fillUniformsAttributesAndSamplerNames(uniforms: Array<string>, attributes: Array<string>, samplers: Array<string>): void;
    /**
     * Observable that will be called just before the particles are drawn
     */
    onBeforeDrawParticlesObservable: Observable<Nullable<Effect>>;
    /**
     * Gets the name of the particle vertex shader
     */
    vertexShaderName: string;
    /**
     * Gets the vertex buffers used by the particle system
     */
    vertexBuffers: Immutable<{
        [key: string]: VertexBuffer;
    }>;
    /**
     * Gets the index buffer used by the particle system (or null if no index buffer is used)
     */
    indexBuffer: Nullable<DataBuffer>;
    /**
     * Adds a new color gradient
     * @param gradient defines the gradient to use (between 0 and 1)
     * @param color1 defines the color to affect to the specified gradient
     * @param color2 defines an additional color used to define a range ([color, color2]) with main color to pick the final color from
     * @returns the current particle system
     */
    addColorGradient(gradient: number, color1: Color4, color2?: Color4): IParticleSystem;
    /**
     * Remove a specific color gradient
     * @param gradient defines the gradient to remove
     * @returns the current particle system
     */
    removeColorGradient(gradient: number): IParticleSystem;
    /**
     * Adds a new size gradient
     * @param gradient defines the gradient to use (between 0 and 1)
     * @param factor defines the size factor to affect to the specified gradient
     * @param factor2 defines an additional factor used to define a range ([factor, factor2]) with main value to pick the final value from
     * @returns the current particle system
     */
    addSizeGradient(gradient: number, factor: number, factor2?: number): IParticleSystem;
    /**
     * Remove a specific size gradient
     * @param gradient defines the gradient to remove
     * @returns the current particle system
     */
    removeSizeGradient(gradient: number): IParticleSystem;
    /**
     * Gets the current list of color gradients.
     * You must use addColorGradient and removeColorGradient to update this list
     * @returns the list of color gradients
     */
    getColorGradients(): Nullable<Array<ColorGradient>>;
    /**
     * Gets the current list of size gradients.
     * You must use addSizeGradient and removeSizeGradient to update this list
     * @returns the list of size gradients
     */
    getSizeGradients(): Nullable<Array<FactorGradient>>;
    /**
     * Gets the current list of angular speed gradients.
     * You must use addAngularSpeedGradient and removeAngularSpeedGradient to update this list
     * @returns the list of angular speed gradients
     */
    getAngularSpeedGradients(): Nullable<Array<FactorGradient>>;
    /**
     * Adds a new angular speed gradient
     * @param gradient defines the gradient to use (between 0 and 1)
     * @param factor defines the angular speed to affect to the specified gradient
     * @param factor2 defines an additional factor used to define a range ([factor, factor2]) with main value to pick the final value from
     * @returns the current particle system
     */
    addAngularSpeedGradient(gradient: number, factor: number, factor2?: number): IParticleSystem;
    /**
     * Remove a specific angular speed gradient
     * @param gradient defines the gradient to remove
     * @returns the current particle system
     */
    removeAngularSpeedGradient(gradient: number): IParticleSystem;
    /**
     * Gets the current list of velocity gradients.
     * You must use addVelocityGradient and removeVelocityGradient to update this list
     * @returns the list of velocity gradients
     */
    getVelocityGradients(): Nullable<Array<FactorGradient>>;
    /**
     * Adds a new velocity gradient
     * @param gradient defines the gradient to use (between 0 and 1)
     * @param factor defines the velocity to affect to the specified gradient
     * @param factor2 defines an additional factor used to define a range ([factor, factor2]) with main value to pick the final value from
     * @returns the current particle system
     */
    addVelocityGradient(gradient: number, factor: number, factor2?: number): IParticleSystem;
    /**
     * Remove a specific velocity gradient
     * @param gradient defines the gradient to remove
     * @returns the current particle system
     */
    removeVelocityGradient(gradient: number): IParticleSystem;
    /**
     * Gets the current list of limit velocity gradients.
     * You must use addLimitVelocityGradient and removeLimitVelocityGradient to update this list
     * @returns the list of limit velocity gradients
     */
    getLimitVelocityGradients(): Nullable<Array<FactorGradient>>;
    /**
     * Adds a new limit velocity gradient
     * @param gradient defines the gradient to use (between 0 and 1)
     * @param factor defines the limit velocity to affect to the specified gradient
     * @param factor2 defines an additional factor used to define a range ([factor, factor2]) with main value to pick the final value from
     * @returns the current particle system
     */
    addLimitVelocityGradient(gradient: number, factor: number, factor2?: number): IParticleSystem;
    /**
     * Remove a specific limit velocity gradient
     * @param gradient defines the gradient to remove
     * @returns the current particle system
     */
    removeLimitVelocityGradient(gradient: number): IParticleSystem;
    /**
     * Adds a new drag gradient
     * @param gradient defines the gradient to use (between 0 and 1)
     * @param factor defines the drag to affect to the specified gradient
     * @param factor2 defines an additional factor used to define a range ([factor, factor2]) with main value to pick the final value from
     * @returns the current particle system
     */
    addDragGradient(gradient: number, factor: number, factor2?: number): IParticleSystem;
    /**
     * Remove a specific drag gradient
     * @param gradient defines the gradient to remove
     * @returns the current particle system
     */
    removeDragGradient(gradient: number): IParticleSystem;
    /**
     * Gets the current list of drag gradients.
     * You must use addDragGradient and removeDragGradient to update this list
     * @returns the list of drag gradients
     */
    getDragGradients(): Nullable<Array<FactorGradient>>;
    /**
     * Adds a new emit rate gradient (please note that this will only work if you set the targetStopDuration property)
     * @param gradient defines the gradient to use (between 0 and 1)
     * @param factor defines the emit rate to affect to the specified gradient
     * @param factor2 defines an additional factor used to define a range ([factor, factor2]) with main value to pick the final value from
     * @returns the current particle system
     */
    addEmitRateGradient(gradient: number, factor: number, factor2?: number): IParticleSystem;
    /**
     * Remove a specific emit rate gradient
     * @param gradient defines the gradient to remove
     * @returns the current particle system
     */
    removeEmitRateGradient(gradient: number): IParticleSystem;
    /**
     * Gets the current list of emit rate gradients.
     * You must use addEmitRateGradient and removeEmitRateGradient to update this list
     * @returns the list of emit rate gradients
     */
    getEmitRateGradients(): Nullable<Array<FactorGradient>>;
    /**
     * Adds a new start size gradient (please note that this will only work if you set the targetStopDuration property)
     * @param gradient defines the gradient to use (between 0 and 1)
     * @param factor defines the start size to affect to the specified gradient
     * @param factor2 defines an additional factor used to define a range ([factor, factor2]) with main value to pick the final value from
     * @returns the current particle system
     */
    addStartSizeGradient(gradient: number, factor: number, factor2?: number): IParticleSystem;
    /**
     * Remove a specific start size gradient
     * @param gradient defines the gradient to remove
     * @returns the current particle system
     */
    removeStartSizeGradient(gradient: number): IParticleSystem;
    /**
     * Gets the current list of start size gradients.
     * You must use addStartSizeGradient and removeStartSizeGradient to update this list
     * @returns the list of start size gradients
     */
    getStartSizeGradients(): Nullable<Array<FactorGradient>>;
    /**
     * Adds a new life time gradient
     * @param gradient defines the gradient to use (between 0 and 1)
     * @param factor defines the life time factor to affect to the specified gradient
     * @param factor2 defines an additional factor used to define a range ([factor, factor2]) with main value to pick the final value from
     * @returns the current particle system
     */
    addLifeTimeGradient(gradient: number, factor: number, factor2?: number): IParticleSystem;
    /**
     * Remove a specific life time gradient
     * @param gradient defines the gradient to remove
     * @returns the current particle system
     */
    removeLifeTimeGradient(gradient: number): IParticleSystem;
    /**
     * Gets the current list of life time gradients.
     * You must use addLifeTimeGradient and removeLifeTimeGradient to update this list
     * @returns the list of life time gradients
     */
    getLifeTimeGradients(): Nullable<Array<FactorGradient>>;
    /**
     * Gets the current list of color gradients.
     * You must use addColorGradient and removeColorGradient to update this list
     * @returns the list of color gradients
     */
    getColorGradients(): Nullable<Array<ColorGradient>>;
    /**
     * Adds a new ramp gradient used to remap particle colors
     * @param gradient defines the gradient to use (between 0 and 1)
     * @param color defines the color to affect to the specified gradient
     * @returns the current particle system
     */
    addRampGradient(gradient: number, color: Color3): IParticleSystem;
    /**
     * Gets the current list of ramp gradients.
     * You must use addRampGradient and removeRampGradient to update this list
     * @returns the list of ramp gradients
     */
    getRampGradients(): Nullable<Array<Color3Gradient>>;
    /** Gets or sets a boolean indicating that ramp gradients must be used
     * @see https://doc.babylonjs.com/features/featuresDeepDive/particles/particle_system/ramps_and_blends
     */
    useRampGradients: boolean;
    /**
     * Adds a new color remap gradient
     * @param gradient defines the gradient to use (between 0 and 1)
     * @param min defines the color remap minimal range
     * @param max defines the color remap maximal range
     * @returns the current particle system
     */
    addColorRemapGradient(gradient: number, min: number, max: number): IParticleSystem;
    /**
     * Gets the current list of color remap gradients.
     * You must use addColorRemapGradient and removeColorRemapGradient to update this list
     * @returns the list of color remap gradients
     */
    getColorRemapGradients(): Nullable<Array<FactorGradient>>;
    /**
     * Adds a new alpha remap gradient
     * @param gradient defines the gradient to use (between 0 and 1)
     * @param min defines the alpha remap minimal range
     * @param max defines the alpha remap maximal range
     * @returns the current particle system
     */
    addAlphaRemapGradient(gradient: number, min: number, max: number): IParticleSystem;
    /**
     * Gets the current list of alpha remap gradients.
     * You must use addAlphaRemapGradient and removeAlphaRemapGradient to update this list
     * @returns the list of alpha remap gradients
     */
    getAlphaRemapGradients(): Nullable<Array<FactorGradient>>;
    /**
     * Creates a Point Emitter for the particle system (emits directly from the emitter position)
     * @param direction1 Particles are emitted between the direction1 and direction2 from within the box
     * @param direction2 Particles are emitted between the direction1 and direction2 from within the box
     * @returns the emitter
     */
    createPointEmitter(direction1: Vector3, direction2: Vector3): PointParticleEmitter;
    /**
     * Creates a Hemisphere Emitter for the particle system (emits along the hemisphere radius)
     * @param radius The radius of the hemisphere to emit from
     * @param radiusRange The range of the hemisphere to emit from [0-1] 0 Surface Only, 1 Entire Radius
     * @returns the emitter
     */
    createHemisphericEmitter(radius: number, radiusRange: number): HemisphericParticleEmitter;
    /**
     * Creates a Sphere Emitter for the particle system (emits along the sphere radius)
     * @param radius The radius of the sphere to emit from
     * @param radiusRange The range of the sphere to emit from [0-1] 0 Surface Only, 1 Entire Radius
     * @returns the emitter
     */
    createSphereEmitter(radius: number, radiusRange: number): SphereParticleEmitter;
    /**
     * Creates a Directed Sphere Emitter for the particle system (emits between direction1 and direction2)
     * @param radius The radius of the sphere to emit from
     * @param direction1 Particles are emitted between the direction1 and direction2 from within the sphere
     * @param direction2 Particles are emitted between the direction1 and direction2 from within the sphere
     * @returns the emitter
     */
    createDirectedSphereEmitter(radius: number, direction1: Vector3, direction2: Vector3): SphereDirectedParticleEmitter;
    /**
     * Creates a Cylinder Emitter for the particle system (emits from the cylinder to the particle position)
     * @param radius The radius of the emission cylinder
     * @param height The height of the emission cylinder
     * @param radiusRange The range of emission [0-1] 0 Surface only, 1 Entire Radius
     * @param directionRandomizer How much to randomize the particle direction [0-1]
     * @returns the emitter
     */
    createCylinderEmitter(radius: number, height: number, radiusRange: number, directionRandomizer: number): CylinderParticleEmitter;
    /**
     * Creates a Directed Cylinder Emitter for the particle system (emits between direction1 and direction2)
     * @param radius The radius of the cylinder to emit from
     * @param height The height of the emission cylinder
     * @param radiusRange the range of the emission cylinder [0-1] 0 Surface only, 1 Entire Radius (1 by default)
     * @param direction1 Particles are emitted between the direction1 and direction2 from within the cylinder
     * @param direction2 Particles are emitted between the direction1 and direction2 from within the cylinder
     * @returns the emitter
     */
    createDirectedCylinderEmitter(radius: number, height: number, radiusRange: number, direction1: Vector3, direction2: Vector3): SphereDirectedParticleEmitter;
    /**
     * Creates a Cone Emitter for the particle system (emits from the cone to the particle position)
     * @param radius The radius of the cone to emit from
     * @param angle The base angle of the cone
     * @returns the emitter
     */
    createConeEmitter(radius: number, angle: number): ConeParticleEmitter;
    /**
     * Creates a Box Emitter for the particle system. (emits between direction1 and direction2 from withing the box defined by minEmitBox and maxEmitBox)
     * @param direction1 Particles are emitted between the direction1 and direction2 from within the box
     * @param direction2 Particles are emitted between the direction1 and direction2 from within the box
     * @param minEmitBox Particles are emitted from the box between minEmitBox and maxEmitBox
     * @param maxEmitBox  Particles are emitted from the box between minEmitBox and maxEmitBox
     * @returns the emitter
     */
    createBoxEmitter(direction1: Vector3, direction2: Vector3, minEmitBox: Vector3, maxEmitBox: Vector3): BoxParticleEmitter;
    /**
     * Get hosting scene
     * @returns the scene
     */
    getScene(): Nullable<Scene>;
}
export {};

}
declare module "babylonjs/Particles/particle" {
import { Nullable } from "babylonjs/types";
import { Vector2, Vector3, Vector4 } from "babylonjs/Maths/math.vector";
import { Color4 } from "babylonjs/Maths/math.color";
import { ParticleSystem } from "babylonjs/Particles/particleSystem";
import { SubEmitter } from "babylonjs/Particles/subEmitter";
import { ColorGradient, FactorGradient } from "babylonjs/Misc/gradients";
/**
 * A particle represents one of the element emitted by a particle system.
 * This is mainly define by its coordinates, direction, velocity and age.
 */
export class Particle {
    /**
     * The particle system the particle belongs to.
     */
    particleSystem: ParticleSystem;
    private static _Count;
    /**
     * Unique ID of the particle
     */
    id: number;
    /**
     * The world position of the particle in the scene.
     */
    position: Vector3;
    /**
     * The world direction of the particle in the scene.
     */
    direction: Vector3;
    /**
     * The color of the particle.
     */
    color: Color4;
    /**
     * The color change of the particle per step.
     */
    colorStep: Color4;
    /**
     * Defines how long will the life of the particle be.
     */
    lifeTime: number;
    /**
     * The current age of the particle.
     */
    age: number;
    /**
     * The current size of the particle.
     */
    size: number;
    /**
     * The current scale of the particle.
     */
    scale: Vector2;
    /**
     * The current angle of the particle.
     */
    angle: number;
    /**
     * Defines how fast is the angle changing.
     */
    angularSpeed: number;
    /**
     * Defines the cell index used by the particle to be rendered from a sprite.
     */
    cellIndex: number;
    /**
     * The information required to support color remapping
     */
    remapData: Vector4;
    /** @internal */
    _randomCellOffset?: number;
    /** @internal */
    _initialDirection: Nullable<Vector3>;
    /** @internal */
    _attachedSubEmitters: Nullable<Array<SubEmitter>>;
    /** @internal */
    _initialStartSpriteCellID: number;
    /** @internal */
    _initialEndSpriteCellID: number;
    /** @internal */
    _initialSpriteCellLoop: boolean;
    /** @internal */
    _currentColorGradient: Nullable<ColorGradient>;
    /** @internal */
    _currentColor1: Color4;
    /** @internal */
    _currentColor2: Color4;
    /** @internal */
    _currentSizeGradient: Nullable<FactorGradient>;
    /** @internal */
    _currentSize1: number;
    /** @internal */
    _currentSize2: number;
    /** @internal */
    _currentAngularSpeedGradient: Nullable<FactorGradient>;
    /** @internal */
    _currentAngularSpeed1: number;
    /** @internal */
    _currentAngularSpeed2: number;
    /** @internal */
    _currentVelocityGradient: Nullable<FactorGradient>;
    /** @internal */
    _currentVelocity1: number;
    /** @internal */
    _currentVelocity2: number;
    /** @internal */
    _currentLimitVelocityGradient: Nullable<FactorGradient>;
    /** @internal */
    _currentLimitVelocity1: number;
    /** @internal */
    _currentLimitVelocity2: number;
    /** @internal */
    _currentDragGradient: Nullable<FactorGradient>;
    /** @internal */
    _currentDrag1: number;
    /** @internal */
    _currentDrag2: number;
    /** @internal */
    _randomNoiseCoordinates1: Vector3;
    /** @internal */
    _randomNoiseCoordinates2: Vector3;
    /** @internal */
    _localPosition?: Vector3;
    /**
     * Creates a new instance Particle
     * @param particleSystem the particle system the particle belongs to
     */
    constructor(
    /**
     * The particle system the particle belongs to.
     */
    particleSystem: ParticleSystem);
    private _updateCellInfoFromSystem;
    /**
     * Defines how the sprite cell index is updated for the particle
     */
    updateCellIndex(): void;
    /**
     * @internal
     */
    _inheritParticleInfoToSubEmitter(subEmitter: SubEmitter): void;
    /** @internal */
    _inheritParticleInfoToSubEmitters(): void;
    /** @internal */
    _reset(): void;
    /**
     * Copy the properties of particle to another one.
     * @param other the particle to copy the information to.
     */
    copyTo(other: Particle): void;
}

}
declare module "babylonjs/Particles/particleHelper" {
import { Nullable } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
import { Vector3 } from "babylonjs/Maths/math.vector";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { IParticleSystem } from "babylonjs/Particles/IParticleSystem";
import { ParticleSystemSet } from "babylonjs/Particles/particleSystemSet";
/**
 * This class is made for on one-liner static method to help creating particle system set.
 */
export class ParticleHelper {
    /**
     * Gets or sets base Assets URL
     */
    static BaseAssetsUrl: string;
    /** Define the Url to load snippets */
    static SnippetUrl: string;
    /**
     * Create a default particle system that you can tweak
     * @param emitter defines the emitter to use
     * @param capacity defines the system capacity (default is 500 particles)
     * @param scene defines the hosting scene
     * @param useGPU defines if a GPUParticleSystem must be created (default is false)
     * @returns the new Particle system
     */
    static CreateDefault(emitter: Nullable<AbstractMesh | Vector3>, capacity?: number, scene?: Scene, useGPU?: boolean): IParticleSystem;
    /**
     * This is the main static method (one-liner) of this helper to create different particle systems
     * @param type This string represents the type to the particle system to create
     * @param scene The scene where the particle system should live
     * @param gpu If the system will use gpu
     * @param capacity defines the system capacity (if null or undefined the sotred capacity will be used)
     * @returns the ParticleSystemSet created
     */
    static CreateAsync(type: string, scene: Nullable<Scene>, gpu?: boolean, capacity?: number): Promise<ParticleSystemSet>;
    /**
     * Static function used to export a particle system to a ParticleSystemSet variable.
     * Please note that the emitter shape is not exported
     * @param systems defines the particle systems to export
     * @returns the created particle system set
     */
    static ExportSet(systems: IParticleSystem[]): ParticleSystemSet;
    /**
     * Creates a particle system from a snippet saved in a remote file
     * @param name defines the name of the particle system to create (can be null or empty to use the one from the json data)
     * @param url defines the url to load from
     * @param scene defines the hosting scene
     * @param gpu If the system will use gpu
     * @param rootUrl defines the root URL to use to load textures and relative dependencies
     * @param capacity defines the system capacity (if null or undefined the sotred capacity will be used)
     * @returns a promise that will resolve to the new particle system
     */
    static ParseFromFileAsync(name: Nullable<string>, url: string, scene: Scene, gpu?: boolean, rootUrl?: string, capacity?: number): Promise<IParticleSystem>;
    /**
     * Creates a particle system from a snippet saved by the particle system editor
     * @param snippetId defines the snippet to load (can be set to _BLANK to create a default one)
     * @param scene defines the hosting scene
     * @param gpu If the system will use gpu
     * @param rootUrl defines the root URL to use to load textures and relative dependencies
     * @param capacity defines the system capacity (if null or undefined the sotred capacity will be used)
     * @returns a promise that will resolve to the new particle system
     */
    static ParseFromSnippetAsync(snippetId: string, scene: Scene, gpu?: boolean, rootUrl?: string, capacity?: number): Promise<IParticleSystem>;
    /**
     * Creates a particle system from a snippet saved by the particle system editor
     * @deprecated Please use ParseFromSnippetAsync instead
     * @param snippetId defines the snippet to load (can be set to _BLANK to create a default one)
     * @param scene defines the hosting scene
     * @param gpu If the system will use gpu
     * @param rootUrl defines the root URL to use to load textures and relative dependencies
     * @param capacity defines the system capacity (if null or undefined the sotred capacity will be used)
     * @returns a promise that will resolve to the new particle system
     */
    static CreateFromSnippetAsync: typeof ParticleHelper.ParseFromSnippetAsync;
}

}
declare module "babylonjs/Particles/particleSystem" {
import { Immutable, Nullable } from "babylonjs/types";
import { FactorGradient, Color3Gradient } from "babylonjs/Misc/gradients";
import { Observable } from "babylonjs/Misc/observable";
import { Vector3, Matrix } from "babylonjs/Maths/math.vector";
import { VertexBuffer } from "babylonjs/Buffers/buffer";
import { Effect } from "babylonjs/Materials/effect";
import { IDisposable } from "babylonjs/scene";
import { IParticleSystem } from "babylonjs/Particles/IParticleSystem";
import { BaseParticleSystem } from "babylonjs/Particles/baseParticleSystem";
import { Particle } from "babylonjs/Particles/particle";
import { SubEmitter } from "babylonjs/Particles/subEmitter";
import { IAnimatable } from "babylonjs/Animations/animatable.interface";
import "babylonjs/Shaders/particles.fragment";
import "babylonjs/Shaders/particles.vertex";
import { DataBuffer } from "babylonjs/Buffers/dataBuffer";
import { Color4, Color3 } from "babylonjs/Maths/math.color";
import { ThinEngine } from "babylonjs/Engines/thinEngine";
import "babylonjs/Engines/Extensions/engine.alpha";
import { Scene } from "babylonjs/scene";
/**
 * This represents a particle system in Babylon.
 * Particles are often small sprites used to simulate hard-to-reproduce phenomena like fire, smoke, water, or abstract visual effects like magic glitter and faery dust.
 * Particles can take different shapes while emitted like box, sphere, cone or you can write your custom function.
 * @example https://doc.babylonjs.com/features/featuresDeepDive/particles/particle_system/particle_system_intro
 */
export class ParticleSystem extends BaseParticleSystem implements IDisposable, IAnimatable, IParticleSystem {
    /**
     * Billboard mode will only apply to Y axis
     */
    static readonly BILLBOARDMODE_Y: number;
    /**
     * Billboard mode will apply to all axes
     */
    static readonly BILLBOARDMODE_ALL: number;
    /**
     * Special billboard mode where the particle will be biilboard to the camera but rotated to align with direction
     */
    static readonly BILLBOARDMODE_STRETCHED: number;
    /**
     * Special billboard mode where the particle will be billboard to the camera but only around the axis of the direction of particle emission
     */
    static readonly BILLBOARDMODE_STRETCHED_LOCAL: number;
    /**
     * This function can be defined to provide custom update for active particles.
     * This function will be called instead of regular update (age, position, color, etc.).
     * Do not forget that this function will be called on every frame so try to keep it simple and fast :)
     */
    updateFunction: (particles: Particle[]) => void;
    private _emitterWorldMatrix;
    private _emitterInverseWorldMatrix;
    /**
     * This function can be defined to specify initial direction for every new particle.
     * It by default use the emitterType defined function
     */
    startDirectionFunction: (worldMatrix: Matrix, directionToUpdate: Vector3, particle: Particle, isLocal: boolean) => void;
    /**
     * This function can be defined to specify initial position for every new particle.
     * It by default use the emitterType defined function
     */
    startPositionFunction: (worldMatrix: Matrix, positionToUpdate: Vector3, particle: Particle, isLocal: boolean) => void;
    /**
     * @internal
     */
    _inheritedVelocityOffset: Vector3;
    /**
     * An event triggered when the system is disposed
     */
    onDisposeObservable: Observable<IParticleSystem>;
    /**
     * An event triggered when the system is stopped
     */
    onStoppedObservable: Observable<IParticleSystem>;
    private _onDisposeObserver;
    /**
     * Sets a callback that will be triggered when the system is disposed
     */
    set onDispose(callback: () => void);
    private _particles;
    private _epsilon;
    private _capacity;
    private _stockParticles;
    private _newPartsExcess;
    private _vertexData;
    private _vertexBuffer;
    private _vertexBuffers;
    private _spriteBuffer;
    private _indexBuffer;
    private _drawWrappers;
    private _customWrappers;
    private _scaledColorStep;
    private _colorDiff;
    private _scaledDirection;
    private _scaledGravity;
    private _currentRenderId;
    private _alive;
    private _useInstancing;
    private _vertexArrayObject;
    private _started;
    private _stopped;
    private _actualFrame;
    private _scaledUpdateSpeed;
    private _vertexBufferSize;
    /** @internal */
    _currentEmitRateGradient: Nullable<FactorGradient>;
    /** @internal */
    _currentEmitRate1: number;
    /** @internal */
    _currentEmitRate2: number;
    /** @internal */
    _currentStartSizeGradient: Nullable<FactorGradient>;
    /** @internal */
    _currentStartSize1: number;
    /** @internal */
    _currentStartSize2: number;
    /** Indicates that the update of particles is done in the animate function */
    readonly updateInAnimate: boolean;
    private readonly _rawTextureWidth;
    private _rampGradientsTexture;
    private _useRampGradients;
    /** Gets or sets a matrix to use to compute projection */
    defaultProjectionMatrix: Matrix;
    /** Gets or sets a matrix to use to compute view */
    defaultViewMatrix: Matrix;
    /** Gets or sets a boolean indicating that ramp gradients must be used
     * @see https://doc.babylonjs.com/features/featuresDeepDive/particles/particle_system/particle_system_intro#ramp-gradients
     */
    get useRampGradients(): boolean;
    set useRampGradients(value: boolean);
    /**
     * The Sub-emitters templates that will be used to generate the sub particle system to be associated with the system, this property is used by the root particle system only.
     * When a particle is spawned, an array will be chosen at random and all the emitters in that array will be attached to the particle.  (Default: [])
     */
    subEmitters: Array<ParticleSystem | SubEmitter | Array<SubEmitter>>;
    private _subEmitters;
    /**
     * @internal
     * If the particle systems emitter should be disposed when the particle system is disposed
     */
    _disposeEmitterOnDispose: boolean;
    /**
     * The current active Sub-systems, this property is used by the root particle system only.
     */
    activeSubSystems: Array<ParticleSystem>;
    /**
     * Specifies if the particles are updated in emitter local space or world space
     */
    isLocal: boolean;
    /** Indicates that the particle system is CPU based */
    readonly isGPU: boolean;
    private _rootParticleSystem;
    /**
     * Gets the current list of active particles
     */
    get particles(): Particle[];
    /**
     * Gets the number of particles active at the same time.
     * @returns The number of active particles.
     */
    getActiveCount(): number;
    /**
     * Returns the string "ParticleSystem"
     * @returns a string containing the class name
     */
    getClassName(): string;
    /**
     * Gets a boolean indicating that the system is stopping
     * @returns true if the system is currently stopping
     */
    isStopping(): boolean;
    /**
     * Gets the custom effect used to render the particles
     * @param blendMode Blend mode for which the effect should be retrieved
     * @returns The effect
     */
    getCustomEffect(blendMode?: number): Nullable<Effect>;
    private _getCustomDrawWrapper;
    /**
     * Sets the custom effect used to render the particles
     * @param effect The effect to set
     * @param blendMode Blend mode for which the effect should be set
     */
    setCustomEffect(effect: Nullable<Effect>, blendMode?: number): void;
    /** @internal */
    private _onBeforeDrawParticlesObservable;
    /**
     * Observable that will be called just before the particles are drawn
     */
    get onBeforeDrawParticlesObservable(): Observable<Nullable<Effect>>;
    /**
     * Gets the name of the particle vertex shader
     */
    get vertexShaderName(): string;
    /**
     * Gets the vertex buffers used by the particle system
     */
    get vertexBuffers(): Immutable<{
        [key: string]: VertexBuffer;
    }>;
    /**
     * Gets the index buffer used by the particle system (or null if no index buffer is used (if _useInstancing=true))
     */
    get indexBuffer(): Nullable<DataBuffer>;
    /**
     * Instantiates a particle system.
     * Particles are often small sprites used to simulate hard-to-reproduce phenomena like fire, smoke, water, or abstract visual effects like magic glitter and faery dust.
     * @param name The name of the particle system
     * @param capacity The max number of particles alive at the same time
     * @param sceneOrEngine The scene the particle system belongs to or the engine to use if no scene
     * @param customEffect a custom effect used to change the way particles are rendered by default
     * @param isAnimationSheetEnabled Must be true if using a spritesheet to animate the particles texture
     * @param epsilon Offset used to render the particles
     */
    constructor(name: string, capacity: number, sceneOrEngine: Scene | ThinEngine, customEffect?: Nullable<Effect>, isAnimationSheetEnabled?: boolean, epsilon?: number);
    private _addFactorGradient;
    private _removeFactorGradient;
    /**
     * Adds a new life time gradient
     * @param gradient defines the gradient to use (between 0 and 1)
     * @param factor defines the life time factor to affect to the specified gradient
     * @param factor2 defines an additional factor used to define a range ([factor, factor2]) with main value to pick the final value from
     * @returns the current particle system
     */
    addLifeTimeGradient(gradient: number, factor: number, factor2?: number): IParticleSystem;
    /**
     * Remove a specific life time gradient
     * @param gradient defines the gradient to remove
     * @returns the current particle system
     */
    removeLifeTimeGradient(gradient: number): IParticleSystem;
    /**
     * Adds a new size gradient
     * @param gradient defines the gradient to use (between 0 and 1)
     * @param factor defines the size factor to affect to the specified gradient
     * @param factor2 defines an additional factor used to define a range ([factor, factor2]) with main value to pick the final value from
     * @returns the current particle system
     */
    addSizeGradient(gradient: number, factor: number, factor2?: number): IParticleSystem;
    /**
     * Remove a specific size gradient
     * @param gradient defines the gradient to remove
     * @returns the current particle system
     */
    removeSizeGradient(gradient: number): IParticleSystem;
    /**
     * Adds a new color remap gradient
     * @param gradient defines the gradient to use (between 0 and 1)
     * @param min defines the color remap minimal range
     * @param max defines the color remap maximal range
     * @returns the current particle system
     */
    addColorRemapGradient(gradient: number, min: number, max: number): IParticleSystem;
    /**
     * Remove a specific color remap gradient
     * @param gradient defines the gradient to remove
     * @returns the current particle system
     */
    removeColorRemapGradient(gradient: number): IParticleSystem;
    /**
     * Adds a new alpha remap gradient
     * @param gradient defines the gradient to use (between 0 and 1)
     * @param min defines the alpha remap minimal range
     * @param max defines the alpha remap maximal range
     * @returns the current particle system
     */
    addAlphaRemapGradient(gradient: number, min: number, max: number): IParticleSystem;
    /**
     * Remove a specific alpha remap gradient
     * @param gradient defines the gradient to remove
     * @returns the current particle system
     */
    removeAlphaRemapGradient(gradient: number): IParticleSystem;
    /**
     * Adds a new angular speed gradient
     * @param gradient defines the gradient to use (between 0 and 1)
     * @param factor defines the angular speed  to affect to the specified gradient
     * @param factor2 defines an additional factor used to define a range ([factor, factor2]) with main value to pick the final value from
     * @returns the current particle system
     */
    addAngularSpeedGradient(gradient: number, factor: number, factor2?: number): IParticleSystem;
    /**
     * Remove a specific angular speed gradient
     * @param gradient defines the gradient to remove
     * @returns the current particle system
     */
    removeAngularSpeedGradient(gradient: number): IParticleSystem;
    /**
     * Adds a new velocity gradient
     * @param gradient defines the gradient to use (between 0 and 1)
     * @param factor defines the velocity to affect to the specified gradient
     * @param factor2 defines an additional factor used to define a range ([factor, factor2]) with main value to pick the final value from
     * @returns the current particle system
     */
    addVelocityGradient(gradient: number, factor: number, factor2?: number): IParticleSystem;
    /**
     * Remove a specific velocity gradient
     * @param gradient defines the gradient to remove
     * @returns the current particle system
     */
    removeVelocityGradient(gradient: number): IParticleSystem;
    /**
     * Adds a new limit velocity gradient
     * @param gradient defines the gradient to use (between 0 and 1)
     * @param factor defines the limit velocity value to affect to the specified gradient
     * @param factor2 defines an additional factor used to define a range ([factor, factor2]) with main value to pick the final value from
     * @returns the current particle system
     */
    addLimitVelocityGradient(gradient: number, factor: number, factor2?: number): IParticleSystem;
    /**
     * Remove a specific limit velocity gradient
     * @param gradient defines the gradient to remove
     * @returns the current particle system
     */
    removeLimitVelocityGradient(gradient: number): IParticleSystem;
    /**
     * Adds a new drag gradient
     * @param gradient defines the gradient to use (between 0 and 1)
     * @param factor defines the drag value to affect to the specified gradient
     * @param factor2 defines an additional factor used to define a range ([factor, factor2]) with main value to pick the final value from
     * @returns the current particle system
     */
    addDragGradient(gradient: number, factor: number, factor2?: number): IParticleSystem;
    /**
     * Remove a specific drag gradient
     * @param gradient defines the gradient to remove
     * @returns the current particle system
     */
    removeDragGradient(gradient: number): IParticleSystem;
    /**
     * Adds a new emit rate gradient (please note that this will only work if you set the targetStopDuration property)
     * @param gradient defines the gradient to use (between 0 and 1)
     * @param factor defines the emit rate value to affect to the specified gradient
     * @param factor2 defines an additional factor used to define a range ([factor, factor2]) with main value to pick the final value from
     * @returns the current particle system
     */
    addEmitRateGradient(gradient: number, factor: number, factor2?: number): IParticleSystem;
    /**
     * Remove a specific emit rate gradient
     * @param gradient defines the gradient to remove
     * @returns the current particle system
     */
    removeEmitRateGradient(gradient: number): IParticleSystem;
    /**
     * Adds a new start size gradient (please note that this will only work if you set the targetStopDuration property)
     * @param gradient defines the gradient to use (between 0 and 1)
     * @param factor defines the start size value to affect to the specified gradient
     * @param factor2 defines an additional factor used to define a range ([factor, factor2]) with main value to pick the final value from
     * @returns the current particle system
     */
    addStartSizeGradient(gradient: number, factor: number, factor2?: number): IParticleSystem;
    /**
     * Remove a specific start size gradient
     * @param gradient defines the gradient to remove
     * @returns the current particle system
     */
    removeStartSizeGradient(gradient: number): IParticleSystem;
    private _createRampGradientTexture;
    /**
     * Gets the current list of ramp gradients.
     * You must use addRampGradient and removeRampGradient to update this list
     * @returns the list of ramp gradients
     */
    getRampGradients(): Nullable<Array<Color3Gradient>>;
    /** Force the system to rebuild all gradients that need to be resync */
    forceRefreshGradients(): void;
    private _syncRampGradientTexture;
    /**
     * Adds a new ramp gradient used to remap particle colors
     * @param gradient defines the gradient to use (between 0 and 1)
     * @param color defines the color to affect to the specified gradient
     * @returns the current particle system
     */
    addRampGradient(gradient: number, color: Color3): ParticleSystem;
    /**
     * Remove a specific ramp gradient
     * @param gradient defines the gradient to remove
     * @returns the current particle system
     */
    removeRampGradient(gradient: number): ParticleSystem;
    /**
     * Adds a new color gradient
     * @param gradient defines the gradient to use (between 0 and 1)
     * @param color1 defines the color to affect to the specified gradient
     * @param color2 defines an additional color used to define a range ([color, color2]) with main color to pick the final color from
     * @returns this particle system
     */
    addColorGradient(gradient: number, color1: Color4, color2?: Color4): IParticleSystem;
    /**
     * Remove a specific color gradient
     * @param gradient defines the gradient to remove
     * @returns this particle system
     */
    removeColorGradient(gradient: number): IParticleSystem;
    /**
     * Resets the draw wrappers cache
     */
    resetDrawCache(): void;
    private _fetchR;
    protected _reset(): void;
    private _resetEffect;
    private _createVertexBuffers;
    private _createIndexBuffer;
    /**
     * Gets the maximum number of particles active at the same time.
     * @returns The max number of active particles.
     */
    getCapacity(): number;
    /**
     * Gets whether there are still active particles in the system.
     * @returns True if it is alive, otherwise false.
     */
    isAlive(): boolean;
    /**
     * Gets if the system has been started. (Note: this will still be true after stop is called)
     * @returns True if it has been started, otherwise false.
     */
    isStarted(): boolean;
    private _prepareSubEmitterInternalArray;
    /**
     * Starts the particle system and begins to emit
     * @param delay defines the delay in milliseconds before starting the system (this.startDelay by default)
     */
    start(delay?: number): void;
    /**
     * Stops the particle system.
     * @param stopSubEmitters if true it will stop the current system and all created sub-Systems if false it will stop the current root system only, this param is used by the root particle system only. the default value is true.
     */
    stop(stopSubEmitters?: boolean): void;
    /**
     * Remove all active particles
     */
    reset(): void;
    /**
     * @internal (for internal use only)
     */
    _appendParticleVertex(index: number, particle: Particle, offsetX: number, offsetY: number): void;
    /**
     * "Recycles" one of the particle by copying it back to the "stock" of particles and removing it from the active list.
     * Its lifetime will start back at 0.
     * @param particle
     */
    recycleParticle: (particle: Particle) => void;
    private _stopSubEmitters;
    private _createParticle;
    private _removeFromRoot;
    private _emitFromParticle;
    private _update;
    /**
     * @internal
     */
    static _GetAttributeNamesOrOptions(isAnimationSheetEnabled?: boolean, isBillboardBased?: boolean, useRampGradients?: boolean): string[];
    /**
     * @internal
     */
    static _GetEffectCreationOptions(isAnimationSheetEnabled?: boolean, useLogarithmicDepth?: boolean): string[];
    /**
     * Fill the defines array according to the current settings of the particle system
     * @param defines Array to be updated
     * @param blendMode blend mode to take into account when updating the array
     */
    fillDefines(defines: Array<string>, blendMode: number): void;
    /**
     * Fill the uniforms, attributes and samplers arrays according to the current settings of the particle system
     * @param uniforms Uniforms array to fill
     * @param attributes Attributes array to fill
     * @param samplers Samplers array to fill
     */
    fillUniformsAttributesAndSamplerNames(uniforms: Array<string>, attributes: Array<string>, samplers: Array<string>): void;
    /**
     * @internal
     */
    private _getWrapper;
    /**
     * Animates the particle system for the current frame by emitting new particles and or animating the living ones.
     * @param preWarmOnly will prevent the system from updating the vertex buffer (default is false)
     */
    animate(preWarmOnly?: boolean): void;
    private _appendParticleVertices;
    /**
     * Rebuilds the particle system.
     */
    rebuild(): void;
    /**
     * Is this system ready to be used/rendered
     * @returns true if the system is ready
     */
    isReady(): boolean;
    private _render;
    /**
     * Renders the particle system in its current state.
     * @returns the current number of particles
     */
    render(): number;
    /**
     * Disposes the particle system and free the associated resources
     * @param disposeTexture defines if the particle texture must be disposed as well (true by default)
     */
    dispose(disposeTexture?: boolean): void;
    /**
     * Clones the particle system.
     * @param name The name of the cloned object
     * @param newEmitter The new emitter to use
     * @param cloneTexture Also clone the textures if true
     * @returns the cloned particle system
     */
    clone(name: string, newEmitter: any, cloneTexture?: boolean): ParticleSystem;
    /**
     * Serializes the particle system to a JSON object
     * @param serializeTexture defines if the texture must be serialized as well
     * @returns the JSON object
     */
    serialize(serializeTexture?: boolean): any;
    /**
     * @internal
     */
    static _Serialize(serializationObject: any, particleSystem: IParticleSystem, serializeTexture: boolean): void;
    /**
     * @internal
     */
    static _Parse(parsedParticleSystem: any, particleSystem: IParticleSystem, sceneOrEngine: Scene | ThinEngine, rootUrl: string): void;
    /**
     * Parses a JSON object to create a particle system.
     * @param parsedParticleSystem The JSON object to parse
     * @param sceneOrEngine The scene or the engine to create the particle system in
     * @param rootUrl The root url to use to load external dependencies like texture
     * @param doNotStart Ignore the preventAutoStart attribute and does not start
     * @param capacity defines the system capacity (if null or undefined the sotred capacity will be used)
     * @returns the Parsed particle system
     */
    static Parse(parsedParticleSystem: any, sceneOrEngine: Scene | ThinEngine, rootUrl: string, doNotStart?: boolean, capacity?: number): ParticleSystem;
}
export {};

}
declare module "babylonjs/Particles/particleSystemComponent" {
import { IParticleSystem } from "babylonjs/Particles/IParticleSystem";
import { Effect } from "babylonjs/Materials/effect";
import "babylonjs/Shaders/particles.vertex";
import { EffectFallbacks } from "babylonjs/Materials/effectFallbacks";
module "babylonjs/Engines/engine" {
    interface Engine {
        /**
         * Create an effect to use with particle systems.
         * Please note that some parameters like animation sheets or not being billboard are not supported in this configuration, except if you pass
         * the particle system for which you want to create a custom effect in the last parameter
         * @param fragmentName defines the base name of the effect (The name of file without .fragment.fx)
         * @param uniformsNames defines a list of attribute names
         * @param samplers defines an array of string used to represent textures
         * @param defines defines the string containing the defines to use to compile the shaders
         * @param fallbacks defines the list of potential fallbacks to use if shader compilation fails
         * @param onCompiled defines a function to call when the effect creation is successful
         * @param onError defines a function to call when the effect creation has failed
         * @param particleSystem the particle system you want to create the effect for
         * @returns the new Effect
         */
        createEffectForParticles(fragmentName: string, uniformsNames: string[], samplers: string[], defines: string, fallbacks?: EffectFallbacks, onCompiled?: (effect: Effect) => void, onError?: (effect: Effect, errors: string) => void, particleSystem?: IParticleSystem): Effect;
    }
}
module "babylonjs/Meshes/mesh" {
    interface Mesh {
        /**
         * Returns an array populated with IParticleSystem objects whose the mesh is the emitter
         * @returns an array of IParticleSystem
         */
        getEmittedParticleSystems(): IParticleSystem[];
        /**
         * Returns an array populated with IParticleSystem objects whose the mesh or its children are the emitter
         * @returns an array of IParticleSystem
         */
        getHierarchyEmittedParticleSystems(): IParticleSystem[];
    }
}

}
declare module "babylonjs/Particles/particleSystemSet" {
import { Nullable } from "babylonjs/types";
import { Color3 } from "babylonjs/Maths/math.color";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { IParticleSystem } from "babylonjs/Particles/IParticleSystem";
import { Scene, IDisposable } from "babylonjs/scene";
import { Vector3 } from "babylonjs/Maths/math.vector";
/**
 * Represents a set of particle systems working together to create a specific effect
 */
export class ParticleSystemSet implements IDisposable {
    /**
     * Gets or sets base Assets URL
     */
    static BaseAssetsUrl: string;
    private _emitterCreationOptions;
    private _emitterNode;
    private _emitterNodeIsOwned;
    /**
     * Gets the particle system list
     */
    systems: IParticleSystem[];
    /**
     * Gets or sets the emitter node used with this set
     */
    get emitterNode(): Nullable<AbstractMesh | Vector3>;
    set emitterNode(value: Nullable<AbstractMesh | Vector3>);
    /**
     * Creates a new emitter mesh as a sphere
     * @param options defines the options used to create the sphere
     * @param options.diameter
     * @param options.segments
     * @param options.color
     * @param renderingGroupId defines the renderingGroupId to use for the sphere
     * @param scene defines the hosting scene
     */
    setEmitterAsSphere(options: {
        diameter: number;
        segments: number;
        color: Color3;
    }, renderingGroupId: number, scene: Scene): void;
    /**
     * Starts all particle systems of the set
     * @param emitter defines an optional mesh to use as emitter for the particle systems
     */
    start(emitter?: AbstractMesh): void;
    /**
     * Release all associated resources
     */
    dispose(): void;
    /**
     * Serialize the set into a JSON compatible object
     * @param serializeTexture defines if the texture must be serialized as well
     * @returns a JSON compatible representation of the set
     */
    serialize(serializeTexture?: boolean): any;
    /**
     * Parse a new ParticleSystemSet from a serialized source
     * @param data defines a JSON compatible representation of the set
     * @param scene defines the hosting scene
     * @param gpu defines if we want GPU particles or CPU particles
     * @param capacity defines the system capacity (if null or undefined the sotred capacity will be used)
     * @returns a new ParticleSystemSet
     */
    static Parse(data: any, scene: Scene, gpu?: boolean, capacity?: number): ParticleSystemSet;
}

}
declare module "babylonjs/Particles/pointsCloudSystem" {
import { Color4 } from "babylonjs/Maths/math";
import { Mesh } from "babylonjs/Meshes/mesh";
import { Scene, IDisposable } from "babylonjs/scene";
import { CloudPoint } from "babylonjs/Particles/cloudPoint";
import { Material } from "babylonjs/Materials/material";
/** Defines the 4 color options */
export enum PointColor {
    /** color value */
    Color = 2,
    /** uv value */
    UV = 1,
    /** random value */
    Random = 0,
    /** stated value */
    Stated = 3
}
/**
 * The PointCloudSystem (PCS) is a single updatable mesh. The points corresponding to the vertices of this big mesh.
 * As it is just a mesh, the PointCloudSystem has all the same properties as any other BJS mesh : not more, not less. It can be scaled, rotated, translated, enlighted, textured, moved, etc.

 * The PointCloudSystem is also a particle system, with each point being a particle. It provides some methods to manage the particles.
 * However it is behavior agnostic. This means it has no emitter, no particle physics, no particle recycler. You have to implement your own behavior.
 *
 * Full documentation here : TO BE ENTERED
 */
export class PointsCloudSystem implements IDisposable {
    /**
     *  The PCS array of cloud point objects. Just access each particle as with any classic array.
     *  Example : var p = SPS.particles[i];
     */
    particles: CloudPoint[];
    /**
     * The PCS total number of particles. Read only. Use PCS.counter instead if you need to set your own value.
     */
    nbParticles: number;
    /**
     * This a counter for your own usage. It's not set by any SPS functions.
     */
    counter: number;
    /**
     * The PCS name. This name is also given to the underlying mesh.
     */
    name: string;
    /**
     * The PCS mesh. It's a standard BJS Mesh, so all the methods from the Mesh class are available.
     */
    mesh?: Mesh;
    /**
     * This empty object is intended to store some PCS specific or temporary values in order to lower the Garbage Collector activity.
     * Please read :
     */
    vars: any;
    /**
     * @internal
     */
    _size: number;
    private _scene;
    private _promises;
    private _positions;
    private _indices;
    private _normals;
    private _colors;
    private _uvs;
    private _indices32;
    private _positions32;
    private _colors32;
    private _uvs32;
    private _updatable;
    private _isVisibilityBoxLocked;
    private _alwaysVisible;
    private _groups;
    private _groupCounter;
    private _computeParticleColor;
    private _computeParticleTexture;
    private _computeParticleRotation;
    private _computeBoundingBox;
    private _isReady;
    /**
     * Gets the particle positions computed by the Point Cloud System
     */
    get positions(): Float32Array;
    /**
     * Gets the particle colors computed by the Point Cloud System
     */
    get colors(): Float32Array;
    /**
     * Gets the particle uvs computed by the Point Cloud System
     */
    get uvs(): Float32Array;
    /**
     * Creates a PCS (Points Cloud System) object
     * @param name (String) is the PCS name, this will be the underlying mesh name
     * @param pointSize (number) is the size for each point. Has no effect on a WebGPU engine.
     * @param scene (Scene) is the scene in which the PCS is added
     * @param options defines the options of the PCS e.g.
     * * updatable (optional boolean, default true) : if the PCS must be updatable or immutable
     * @param options.updatable
     */
    constructor(name: string, pointSize: number, scene: Scene, options?: {
        updatable?: boolean;
    });
    /**
     * Builds the PCS underlying mesh. Returns a standard Mesh.
     * If no points were added to the PCS, the returned mesh is just a single point.
     * @param material The material to use to render the mesh. If not provided, will create a default one
     * @returns a promise for the created mesh
     */
    buildMeshAsync(material?: Material): Promise<Mesh>;
    /**
     * @internal
     */
    private _buildMesh;
    private _addParticle;
    private _randomUnitVector;
    private _getColorIndicesForCoord;
    private _setPointsColorOrUV;
    private _colorFromTexture;
    private _calculateDensity;
    /**
     * Adds points to the PCS in random positions within a unit sphere
     * @param nb (positive integer) the number of particles to be created from this model
     * @param pointFunction is an optional javascript function to be called for each particle on PCS creation
     * @returns the number of groups in the system
     */
    addPoints(nb: number, pointFunction?: any): number;
    /**
     * Adds points to the PCS from the surface of the model shape
     * @param mesh is any Mesh object that will be used as a surface model for the points
     * @param nb (positive integer) the number of particles to be created from this model
     * @param colorWith determines whether a point is colored using color (default), uv, random, stated or none (invisible)
     * @param color (color4) to be used when colorWith is stated or color (number) when used to specify texture position
     * @param range (number from 0 to 1) to determine the variation in shape and tone for a stated color
     * @returns the number of groups in the system
     */
    addSurfacePoints(mesh: Mesh, nb: number, colorWith?: number, color?: Color4 | number, range?: number): number;
    /**
     * Adds points to the PCS inside the model shape
     * @param mesh is any Mesh object that will be used as a surface model for the points
     * @param nb (positive integer) the number of particles to be created from this model
     * @param colorWith determines whether a point is colored using color (default), uv, random, stated or none (invisible)
     * @param color (color4) to be used when colorWith is stated or color (number) when used to specify texture position
     * @param range (number from 0 to 1) to determine the variation in shape and tone for a stated color
     * @returns the number of groups in the system
     */
    addVolumePoints(mesh: Mesh, nb: number, colorWith?: number, color?: Color4 | number, range?: number): number;
    /**
     *  Sets all the particles : this method actually really updates the mesh according to the particle positions, rotations, colors, textures, etc.
     *  This method calls `updateParticle()` for each particle of the SPS.
     *  For an animated SPS, it is usually called within the render loop.
     * @param start The particle index in the particle array where to start to compute the particle property values _(default 0)_
     * @param end The particle index in the particle array where to stop to compute the particle property values _(default nbParticle - 1)_
     * @param update If the mesh must be finally updated on this call after all the particle computations _(default true)_
     * @returns the PCS.
     */
    setParticles(start?: number, end?: number, update?: boolean): PointsCloudSystem;
    /**
     * Disposes the PCS.
     */
    dispose(): void;
    /**
     * Visibility helper : Recomputes the visible size according to the mesh bounding box
     * doc :
     * @returns the PCS.
     */
    refreshVisibleSize(): PointsCloudSystem;
    /**
     * Visibility helper : Sets the size of a visibility box, this sets the underlying mesh bounding box.
     * @param size the size (float) of the visibility box
     * note : this doesn't lock the PCS mesh bounding box.
     * doc :
     */
    setVisibilityBox(size: number): void;
    /**
     * Gets whether the PCS is always visible or not
     * doc :
     */
    get isAlwaysVisible(): boolean;
    /**
     * Sets the PCS as always visible or not
     * doc :
     */
    set isAlwaysVisible(val: boolean);
    /**
     * Tells to `setParticles()` to compute the particle rotations or not
     * Default value : false. The PCS is faster when it's set to false
     * Note : particle rotations are only applied to parent particles
     * Note : the particle rotations aren't stored values, so setting `computeParticleRotation` to false will prevents the particle to rotate
     */
    set computeParticleRotation(val: boolean);
    /**
     * Tells to `setParticles()` to compute the particle colors or not.
     * Default value : true. The PCS is faster when it's set to false.
     * Note : the particle colors are stored values, so setting `computeParticleColor` to false will keep yet the last colors set.
     */
    set computeParticleColor(val: boolean);
    set computeParticleTexture(val: boolean);
    /**
     * Gets if `setParticles()` computes the particle colors or not.
     * Default value : false. The PCS is faster when it's set to false.
     * Note : the particle colors are stored values, so setting `computeParticleColor` to false will keep yet the last colors set.
     */
    get computeParticleColor(): boolean;
    /**
     * Gets if `setParticles()` computes the particle textures or not.
     * Default value : false. The PCS is faster when it's set to false.
     * Note : the particle textures are stored values, so setting `computeParticleTexture` to false will keep yet the last colors set.
     */
    get computeParticleTexture(): boolean;
    /**
     * Tells to `setParticles()` to compute or not the mesh bounding box when computing the particle positions.
     */
    set computeBoundingBox(val: boolean);
    /**
     * Gets if `setParticles()` computes or not the mesh bounding box when computing the particle positions.
     */
    get computeBoundingBox(): boolean;
    /**
     * This function does nothing. It may be overwritten to set all the particle first values.
     * The PCS doesn't call this function, you may have to call it by your own.
     * doc :
     */
    initParticles(): void;
    /**
     * This function does nothing. It may be overwritten to recycle a particle
     * The PCS doesn't call this function, you can to call it
     * doc :
     * @param particle The particle to recycle
     * @returns the recycled particle
     */
    recycleParticle(particle: CloudPoint): CloudPoint;
    /**
     * Updates a particle : this function should  be overwritten by the user.
     * It is called on each particle by `setParticles()`. This is the place to code each particle behavior.
     * doc :
     * @example : just set a particle position or velocity and recycle conditions
     * @param particle The particle to update
     * @returns the updated particle
     */
    updateParticle(particle: CloudPoint): CloudPoint;
    /**
     * This will be called before any other treatment by `setParticles()` and will be passed three parameters.
     * This does nothing and may be overwritten by the user.
     * @param start the particle index in the particle array where to start to iterate, same than the value passed to setParticle()
     * @param stop the particle index in the particle array where to stop to iterate, same than the value passed to setParticle()
     * @param update the boolean update value actually passed to setParticles()
     */
    beforeUpdateParticles(start?: number, stop?: number, update?: boolean): void;
    /**
     * This will be called  by `setParticles()` after all the other treatments and just before the actual mesh update.
     * This will be passed three parameters.
     * This does nothing and may be overwritten by the user.
     * @param start the particle index in the particle array where to start to iterate, same than the value passed to setParticle()
     * @param stop the particle index in the particle array where to stop to iterate, same than the value passed to setParticle()
     * @param update the boolean update value actually passed to setParticles()
     */
    afterUpdateParticles(start?: number, stop?: number, update?: boolean): void;
}

}
declare module "babylonjs/Particles/solidParticle" {
import { Nullable } from "babylonjs/types";
import { Matrix } from "babylonjs/Maths/math.vector";
import { Vector3, Quaternion, Vector4, Vector2 } from "babylonjs/Maths/math.vector";
import { Color4 } from "babylonjs/Maths/math.color";
import { Mesh } from "babylonjs/Meshes/mesh";
import { BoundingInfo } from "babylonjs/Culling/boundingInfo";
import { SolidParticleSystem } from "babylonjs/Particles/solidParticleSystem";
import { Plane } from "babylonjs/Maths/math.plane";
import { Material } from "babylonjs/Materials/material";
/**
 * Represents one particle of a solid particle system.
 */
export class SolidParticle {
    /**
     * particle global index
     */
    idx: number;
    /**
     * particle identifier
     */
    id: number;
    /**
     * The color of the particle
     */
    color: Nullable<Color4>;
    /**
     * The world space position of the particle.
     */
    position: Vector3;
    /**
     * The world space rotation of the particle. (Not use if rotationQuaternion is set)
     */
    rotation: Vector3;
    /**
     * The world space rotation quaternion of the particle.
     */
    rotationQuaternion: Nullable<Quaternion>;
    /**
     * The scaling of the particle.
     */
    scaling: Vector3;
    /**
     * The uvs of the particle.
     */
    uvs: Vector4;
    /**
     * The current speed of the particle.
     */
    velocity: Vector3;
    /**
     * The pivot point in the particle local space.
     */
    pivot: Vector3;
    /**
     * Must the particle be translated from its pivot point in its local space ?
     * In this case, the pivot point is set at the origin of the particle local space and the particle is translated.
     * Default : false
     */
    translateFromPivot: boolean;
    /**
     * Is the particle active or not ?
     */
    alive: boolean;
    /**
     * Is the particle visible or not ?
     */
    isVisible: boolean;
    /**
     * Index of this particle in the global "positions" array (Internal use)
     * @internal
     */
    _pos: number;
    /**
     * @internal Index of this particle in the global "indices" array (Internal use)
     */
    _ind: number;
    /**
     * @internal ModelShape of this particle (Internal use)
     */
    _model: ModelShape;
    /**
     * ModelShape id of this particle
     */
    shapeId: number;
    /**
     * Index of the particle in its shape id
     */
    idxInShape: number;
    /**
     * @internal Reference to the shape model BoundingInfo object (Internal use)
     */
    _modelBoundingInfo: BoundingInfo;
    private _boundingInfo;
    /**
     * @internal Reference to the SPS what the particle belongs to (Internal use)
     */
    _sps: SolidParticleSystem;
    /**
     * @internal Still set as invisible in order to skip useless computations (Internal use)
     */
    _stillInvisible: boolean;
    /**
     * @internal Last computed particle rotation matrix
     */
    _rotationMatrix: number[];
    /**
     * Parent particle Id, if any.
     * Default null.
     */
    parentId: Nullable<number>;
    /**
     * The particle material identifier (integer) when MultiMaterials are enabled in the SPS.
     */
    materialIndex: Nullable<number>;
    /**
     * Custom object or properties.
     */
    props: Nullable<any>;
    /**
     * The culling strategy to use to check whether the solid particle must be culled or not when using isInFrustum().
     * The possible values are :
     * - AbstractMesh.CULLINGSTRATEGY_STANDARD
     * - AbstractMesh.CULLINGSTRATEGY_BOUNDINGSPHERE_ONLY
     * - AbstractMesh.CULLINGSTRATEGY_OPTIMISTIC_INCLUSION
     * - AbstractMesh.CULLINGSTRATEGY_OPTIMISTIC_INCLUSION_THEN_BSPHERE_ONLY
     * The default value for solid particles is AbstractMesh.CULLINGSTRATEGY_BOUNDINGSPHERE_ONLY
     * Please read each static variable documentation in the class AbstractMesh to get details about the culling process.
     * */
    cullingStrategy: number;
    /**
     * @internal Internal global position in the SPS.
     */
    _globalPosition: Vector3;
    /**
     * Particle BoundingInfo object
     * @returns a BoundingInfo
     */
    getBoundingInfo(): BoundingInfo;
    /**
     * Returns true if there is already a bounding info
     */
    get hasBoundingInfo(): boolean;
    /**
     * Creates a Solid Particle object.
     * Don't create particles manually, use instead the Solid Particle System internal tools like _addParticle()
     * @param particleIndex (integer) is the particle index in the Solid Particle System pool.
     * @param particleId (integer) is the particle identifier. Unless some particles are removed from the SPS, it's the same value than the particle idx.
     * @param positionIndex (integer) is the starting index of the particle vertices in the SPS "positions" array.
     * @param indiceIndex (integer) is the starting index of the particle indices in the SPS "indices" array.
     * @param model (ModelShape) is a reference to the model shape on what the particle is designed.
     * @param shapeId (integer) is the model shape identifier in the SPS.
     * @param idxInShape (integer) is the index of the particle in the current model (ex: the 10th box of addShape(box, 30))
     * @param sps defines the sps it is associated to
     * @param modelBoundingInfo is the reference to the model BoundingInfo used for intersection computations.
     * @param materialIndex is the particle material identifier (integer) when the MultiMaterials are enabled in the SPS.
     */
    constructor(particleIndex: number, particleId: number, positionIndex: number, indiceIndex: number, model: Nullable<ModelShape>, shapeId: number, idxInShape: number, sps: SolidParticleSystem, modelBoundingInfo?: Nullable<BoundingInfo>, materialIndex?: Nullable<number>);
    /**
     * Copies the particle property values into the existing target : position, rotation, scaling, uvs, colors, pivot, parent, visibility, alive
     * @param target the particle target
     * @returns the current particle
     */
    copyToRef(target: SolidParticle): SolidParticle;
    /**
     * Legacy support, changed scale to scaling
     */
    get scale(): Vector3;
    /**
     * Legacy support, changed scale to scaling
     */
    set scale(scale: Vector3);
    /**
     * Legacy support, changed quaternion to rotationQuaternion
     */
    get quaternion(): Nullable<Quaternion>;
    /**
     * Legacy support, changed quaternion to rotationQuaternion
     */
    set quaternion(q: Nullable<Quaternion>);
    /**
     * Returns a boolean. True if the particle intersects another particle or another mesh, else false.
     * The intersection is computed on the particle bounding sphere and Axis Aligned Bounding Box (AABB)
     * @param target is the object (solid particle or mesh) what the intersection is computed against.
     * @returns true if it intersects
     */
    intersectsMesh(target: Mesh | SolidParticle): boolean;
    /**
     * Returns `true` if the solid particle is within the frustum defined by the passed array of planes.
     * A particle is in the frustum if its bounding box intersects the frustum
     * @param frustumPlanes defines the frustum to test
     * @returns true if the particle is in the frustum planes
     */
    isInFrustum(frustumPlanes: Plane[]): boolean;
    /**
     * get the rotation matrix of the particle
     * @internal
     */
    getRotationMatrix(m: Matrix): void;
}
/**
 * Represents the shape of the model used by one particle of a solid particle system.
 * SPS internal tool, don't use it manually.
 */
export class ModelShape {
    /**
     * Get or set the shapeId
     * @deprecated Please use shapeId instead
     */
    get shapeID(): number;
    set shapeID(shapeID: number);
    /**
     * The shape id
     * @internal
     */
    shapeId: number;
    /**
     * flat array of model positions (internal use)
     * @internal
     */
    _shape: Vector3[];
    /**
     * flat array of model UVs (internal use)
     * @internal
     */
    _shapeUV: number[];
    /**
     * color array of the model
     * @internal
     */
    _shapeColors: number[];
    /**
     * indices array of the model
     * @internal
     */
    _indices: number[];
    /**
     * normals array of the model
     * @internal
     */
    _normals: number[];
    /**
     * length of the shape in the model indices array (internal use)
     * @internal
     */
    _indicesLength: number;
    /**
     * Custom position function (internal use)
     * @internal
     */
    _positionFunction: Nullable<(particle: SolidParticle, i: number, s: number) => void>;
    /**
     * Custom vertex function (internal use)
     * @internal
     */
    _vertexFunction: Nullable<(particle: SolidParticle, vertex: Vector3, i: number) => void>;
    /**
     * Model material (internal use)
     * @internal
     */
    _material: Nullable<Material>;
    /**
     * Creates a ModelShape object. This is an internal simplified reference to a mesh used as for a model to replicate particles from by the SPS.
     * SPS internal tool, don't use it manually.
     * @internal
     */
    constructor(id: number, shape: Vector3[], indices: number[], normals: number[], colors: number[], shapeUV: number[], posFunction: Nullable<(particle: SolidParticle, i: number, s: number) => void>, vtxFunction: Nullable<(particle: SolidParticle, vertex: Vector3, i: number) => void>, material: Nullable<Material>);
}
/**
 * Represents a Depth Sorted Particle in the solid particle system.
 * @internal
 */
export class DepthSortedParticle {
    /**
     * Particle index
     */
    idx: number;
    /**
     * Index of the particle in the "indices" array
     */
    ind: number;
    /**
     * Length of the particle shape in the "indices" array
     */
    indicesLength: number;
    /**
     * Squared distance from the particle to the camera
     */
    sqDistance: number;
    /**
     * Material index when used with MultiMaterials
     */
    materialIndex: number;
    /**
     * Creates a new sorted particle
     * @param idx
     * @param ind
     * @param indLength
     * @param materialIndex
     */
    constructor(idx: number, ind: number, indLength: number, materialIndex: number);
}
/**
 * Represents a solid particle vertex
 */
export class SolidParticleVertex {
    /**
     * Vertex position
     */
    position: Vector3;
    /**
     * Vertex color
     */
    color: Color4;
    /**
     * Vertex UV
     */
    uv: Vector2;
    /**
     * Creates a new solid particle vertex
     */
    constructor();
    /** Vertex x coordinate */
    get x(): number;
    set x(val: number);
    /** Vertex y coordinate */
    get y(): number;
    set y(val: number);
    /** Vertex z coordinate */
    get z(): number;
    set z(val: number);
}

}
declare module "babylonjs/Particles/solidParticleSystem" {
import { Nullable, IndicesArray } from "babylonjs/types";
import { Vector3 } from "babylonjs/Maths/math.vector";
import { Color4 } from "babylonjs/Maths/math.color";
import { Mesh } from "babylonjs/Meshes/mesh";
import { Scene, IDisposable } from "babylonjs/scene";
import { DepthSortedParticle, SolidParticle, ModelShape, SolidParticleVertex } from "babylonjs/Particles/solidParticle";
import { TargetCamera } from "babylonjs/Cameras/targetCamera";
import { BoundingInfo } from "babylonjs/Culling/boundingInfo";
import { Material } from "babylonjs/Materials/material";
import { MultiMaterial } from "babylonjs/Materials/multiMaterial";
import { PickingInfo } from "babylonjs/Collisions/pickingInfo";
/**
 * The SPS is a single updatable mesh. The solid particles are simply separate parts or faces fo this big mesh.
 *As it is just a mesh, the SPS has all the same properties than any other BJS mesh : not more, not less. It can be scaled, rotated, translated, enlighted, textured, moved, etc.

 * The SPS is also a particle system. It provides some methods to manage the particles.
 * However it is behavior agnostic. This means it has no emitter, no particle physics, no particle recycler. You have to implement your own behavior.
 *
 * Full documentation here : https://doc.babylonjs.com/features/featuresDeepDive/particles/solid_particle_system/sps_intro
 */
export class SolidParticleSystem implements IDisposable {
    /**
     *  The SPS array of Solid Particle objects. Just access each particle as with any classic array.
     *  Example : var p = SPS.particles[i];
     */
    particles: SolidParticle[];
    /**
     * The SPS total number of particles. Read only. Use SPS.counter instead if you need to set your own value.
     */
    nbParticles: number;
    /**
     * If the particles must ever face the camera (default false). Useful for planar particles.
     */
    billboard: boolean;
    /**
     * Recompute normals when adding a shape
     */
    recomputeNormals: boolean;
    /**
     * This a counter ofr your own usage. It's not set by any SPS functions.
     */
    counter: number;
    /**
     * The SPS name. This name is also given to the underlying mesh.
     */
    name: string;
    /**
     * The SPS mesh. It's a standard BJS Mesh, so all the methods from the Mesh class are available.
     */
    mesh: Mesh;
    /**
     * This empty object is intended to store some SPS specific or temporary values in order to lower the Garbage Collector activity.
     * Please read : https://doc.babylonjs.com/features/featuresDeepDive/particles/solid_particle_system/optimize_sps#limit-garbage-collection
     */
    vars: any;
    /**
     * This array is populated when the SPS is set as 'pickable'.
     * Each key of this array is a `faceId` value that you can get from a pickResult object.
     * Each element of this array is an object `{idx: int, faceId: int}`.
     * `idx` is the picked particle index in the `SPS.particles` array
     * `faceId` is the picked face index counted within this particle.
     * This array is the first element of the pickedBySubMesh array : sps.pickBySubMesh[0].
     * It's not pertinent to use it when using a SPS with the support for MultiMaterial enabled.
     * Use the method SPS.pickedParticle(pickingInfo) instead.
     * Please read : https://doc.babylonjs.com/features/featuresDeepDive/particles/solid_particle_system/picking_sps
     */
    pickedParticles: {
        idx: number;
        faceId: number;
    }[];
    /**
     * This array is populated when the SPS is set as 'pickable'
     * Each key of this array is a submesh index.
     * Each element of this array is a second array defined like this :
     * Each key of this second array is a `faceId` value that you can get from a pickResult object.
     * Each element of this second array is an object `{idx: int, faceId: int}`.
     * `idx` is the picked particle index in the `SPS.particles` array
     * `faceId` is the picked face index counted within this particle.
     * It's better to use the method SPS.pickedParticle(pickingInfo) rather than using directly this array.
     * Please read : https://doc.babylonjs.com/features/featuresDeepDive/particles/solid_particle_system/picking_sps
     */
    pickedBySubMesh: {
        idx: number;
        faceId: number;
    }[][];
    /**
     * This array is populated when `enableDepthSort` is set to true.
     * Each element of this array is an instance of the class DepthSortedParticle.
     */
    depthSortedParticles: DepthSortedParticle[];
    /**
     * If the particle intersection must be computed only with the bounding sphere (no bounding box computation, so faster). (Internal use only)
     * @internal
     */
    _bSphereOnly: boolean;
    /**
     * A number to multiply the bounding sphere radius by in order to reduce it for instance. (Internal use only)
     * @internal
     */
    _bSphereRadiusFactor: number;
    protected _scene: Scene;
    protected _positions: number[];
    protected _indices: number[];
    protected _normals: number[];
    protected _colors: number[];
    protected _uvs: number[];
    protected _indices32: IndicesArray;
    protected _positions32: Float32Array;
    protected _normals32: Float32Array;
    protected _fixedNormal32: Float32Array;
    protected _colors32: Float32Array;
    protected _uvs32: Float32Array;
    protected _index: number;
    protected _updatable: boolean;
    protected _pickable: boolean;
    protected _isVisibilityBoxLocked: boolean;
    protected _alwaysVisible: boolean;
    protected _depthSort: boolean;
    protected _expandable: boolean;
    protected _shapeCounter: number;
    protected _copy: SolidParticle;
    protected _color: Color4;
    protected _computeParticleColor: boolean;
    protected _computeParticleTexture: boolean;
    protected _computeParticleRotation: boolean;
    protected _computeParticleVertex: boolean;
    protected _computeBoundingBox: boolean;
    protected _autoFixFaceOrientation: boolean;
    protected _depthSortParticles: boolean;
    protected _camera: TargetCamera;
    protected _mustUnrotateFixedNormals: boolean;
    protected _particlesIntersect: boolean;
    protected _needs32Bits: boolean;
    protected _isNotBuilt: boolean;
    protected _lastParticleId: number;
    protected _idxOfId: number[];
    protected _multimaterialEnabled: boolean;
    protected _useModelMaterial: boolean;
    protected _indicesByMaterial: number[];
    protected _materialIndexes: number[];
    protected _depthSortFunction: (p1: DepthSortedParticle, p2: DepthSortedParticle) => number;
    protected _materialSortFunction: (p1: DepthSortedParticle, p2: DepthSortedParticle) => number;
    protected _materials: Material[];
    protected _multimaterial: MultiMaterial;
    protected _materialIndexesById: any;
    protected _defaultMaterial: Material;
    protected _autoUpdateSubMeshes: boolean;
    protected _tmpVertex: SolidParticleVertex;
    protected _recomputeInvisibles: boolean;
    /**
     * Creates a SPS (Solid Particle System) object.
     * @param name (String) is the SPS name, this will be the underlying mesh name.
     * @param scene (Scene) is the scene in which the SPS is added.
     * @param options defines the options of the sps e.g.
     * * updatable (optional boolean, default true) : if the SPS must be updatable or immutable.
     * * isPickable (optional boolean, default false) : if the solid particles must be pickable.
     * * enableDepthSort (optional boolean, default false) : if the solid particles must be sorted in the geometry according to their distance to the camera.
     * * useModelMaterial (optional boolean, default false) : if the model materials must be used to create the SPS multimaterial. This enables the multimaterial supports of the SPS.
     * * enableMultiMaterial (optional boolean, default false) : if the solid particles can be given different materials.
     * * expandable (optional boolean, default false) : if particles can still be added after the initial SPS mesh creation.
     * * particleIntersection (optional boolean, default false) : if the solid particle intersections must be computed.
     * * boundingSphereOnly (optional boolean, default false) : if the particle intersection must be computed only with the bounding sphere (no bounding box computation, so faster).
     * * bSphereRadiusFactor (optional float, default 1.0) : a number to multiply the bounding sphere radius by in order to reduce it for instance.
     * * computeBoundingBox (optional boolean, default false): if the bounding box of the entire SPS will be computed (for occlusion detection, for example). If it is false, the bounding box will be the bounding box of the first particle.
     * * autoFixFaceOrientation (optional boolean, default false): if the particle face orientations will be flipped for transformations that change orientation (scale (-1, 1, 1), for example)
     * @param options.updatable
     * @param options.isPickable
     * @param options.enableDepthSort
     * @param options.particleIntersection
     * @param options.boundingSphereOnly
     * @param options.bSphereRadiusFactor
     * @param options.expandable
     * @param options.useModelMaterial
     * @param options.enableMultiMaterial
     * @param options.computeBoundingBox
     * @param options.autoFixFaceOrientation
     * @example bSphereRadiusFactor = 1.0 / Math.sqrt(3.0) => the bounding sphere exactly matches a spherical mesh.
     */
    constructor(name: string, scene: Scene, options?: {
        updatable?: boolean;
        isPickable?: boolean;
        enableDepthSort?: boolean;
        particleIntersection?: boolean;
        boundingSphereOnly?: boolean;
        bSphereRadiusFactor?: number;
        expandable?: boolean;
        useModelMaterial?: boolean;
        enableMultiMaterial?: boolean;
        computeBoundingBox?: boolean;
        autoFixFaceOrientation?: boolean;
    });
    /**
     * Builds the SPS underlying mesh. Returns a standard Mesh.
     * If no model shape was added to the SPS, the returned mesh is just a single triangular plane.
     * @returns the created mesh
     */
    buildMesh(): Mesh;
    /**
     * Digests the mesh and generates as many solid particles in the system as wanted. Returns the SPS.
     * These particles will have the same geometry than the mesh parts and will be positioned at the same localisation than the mesh original places.
     * Thus the particles generated from `digest()` have their property `position` set yet.
     * @param mesh ( Mesh ) is the mesh to be digested
     * @param options {facetNb} (optional integer, default 1) is the number of mesh facets per particle, this parameter is overridden by the parameter `number` if any
     * {delta} (optional integer, default 0) is the random extra number of facets per particle , each particle will have between `facetNb` and `facetNb + delta` facets
     * {number} (optional positive integer) is the wanted number of particles : each particle is built with `mesh_total_facets / number` facets
     * {storage} (optional existing array) is an array where the particles will be stored for a further use instead of being inserted in the SPS.
     * @param options.facetNb
     * @param options.number
     * @param options.delta
     * @param options.storage
     * @returns the current SPS
     */
    digest(mesh: Mesh, options?: {
        facetNb?: number;
        number?: number;
        delta?: number;
        storage?: [];
    }): SolidParticleSystem;
    /**
     * Unrotate the fixed normals in case the mesh was built with pre-rotated particles, ex : use of positionFunction in addShape()
     * @internal
     */
    protected _unrotateFixedNormals(): void;
    /**
     * Resets the temporary working copy particle
     * @internal
     */
    protected _resetCopy(): void;
    /**
     * Inserts the shape model geometry in the global SPS mesh by updating the positions, indices, normals, colors, uvs arrays
     * @param p the current index in the positions array to be updated
     * @param ind the current index in the indices array
     * @param shape a Vector3 array, the shape geometry
     * @param positions the positions array to be updated
     * @param meshInd the shape indices array
     * @param indices the indices array to be updated
     * @param meshUV the shape uv array
     * @param uvs the uv array to be updated
     * @param meshCol the shape color array
     * @param colors the color array to be updated
     * @param meshNor the shape normals array
     * @param normals the normals array to be updated
     * @param idx the particle index
     * @param idxInShape the particle index in its shape
     * @param options the addShape() method  passed options
     * @param model
     * @model the particle model
     * @internal
     */
    protected _meshBuilder(p: number, ind: number, shape: Vector3[], positions: number[], meshInd: IndicesArray, indices: number[], meshUV: number[] | Float32Array, uvs: number[], meshCol: number[] | Float32Array, colors: number[], meshNor: number[] | Float32Array, normals: number[], idx: number, idxInShape: number, options: any, model: ModelShape): SolidParticle;
    /**
     * Returns a shape Vector3 array from positions float array
     * @param positions float array
     * @returns a vector3 array
     * @internal
     */
    protected _posToShape(positions: number[] | Float32Array): Vector3[];
    /**
     * Returns a shapeUV array from a float uvs (array deep copy)
     * @param uvs as a float array
     * @returns a shapeUV array
     * @internal
     */
    protected _uvsToShapeUV(uvs: number[] | Float32Array): number[];
    /**
     * Adds a new particle object in the particles array
     * @param idx particle index in particles array
     * @param id particle id
     * @param idxpos positionIndex : the starting index of the particle vertices in the SPS "positions" array
     * @param idxind indiceIndex : he starting index of the particle indices in the SPS "indices" array
     * @param model particle ModelShape object
     * @param shapeId model shape identifier
     * @param idxInShape index of the particle in the current model
     * @param bInfo model bounding info object
     * @param storage target storage array, if any
     * @internal
     */
    protected _addParticle(idx: number, id: number, idxpos: number, idxind: number, model: ModelShape, shapeId: number, idxInShape: number, bInfo?: Nullable<BoundingInfo>, storage?: Nullable<[]>): SolidParticle;
    /**
     * Adds some particles to the SPS from the model shape. Returns the shape id.
     * Please read the doc : https://doc.babylonjs.com/features/featuresDeepDive/particles/solid_particle_system/immutable_sps
     * @param mesh is any Mesh object that will be used as a model for the solid particles.
     * @param nb (positive integer) the number of particles to be created from this model
     * @param options {positionFunction} is an optional javascript function to called for each particle on SPS creation.
     * {vertexFunction} is an optional javascript function to called for each vertex of each particle on SPS creation
     * {storage} (optional existing array) is an array where the particles will be stored for a further use instead of being inserted in the SPS.
     * @param options.positionFunction
     * @param options.vertexFunction
     * @param options.storage
     * @returns the number of shapes in the system
     */
    addShape(mesh: Mesh, nb: number, options?: {
        positionFunction?: any;
        vertexFunction?: any;
        storage?: [];
    }): number;
    /**
     * Rebuilds a particle back to its just built status : if needed, recomputes the custom positions and vertices
     * @internal
     */
    protected _rebuildParticle(particle: SolidParticle, reset?: boolean): void;
    /**
     * Rebuilds the whole mesh and updates the VBO : custom positions and vertices are recomputed if needed.
     * @param reset boolean, default false : if the particles must be reset at position and rotation zero, scaling 1, color white, initial UVs and not parented.
     * @returns the SPS.
     */
    rebuildMesh(reset?: boolean): SolidParticleSystem;
    /** Removes the particles from the start-th to the end-th included from an expandable SPS (required).
     *  Returns an array with the removed particles.
     *  If the number of particles to remove is lower than zero or greater than the global remaining particle number, then an empty array is returned.
     *  The SPS can't be empty so at least one particle needs to remain in place.
     *  Under the hood, the VertexData array, so the VBO buffer, is recreated each call.
     * @param start index of the first particle to remove
     * @param end index of the last particle to remove (included)
     * @returns an array populated with the removed particles
     */
    removeParticles(start: number, end: number): SolidParticle[];
    /**
     * Inserts some pre-created particles in the solid particle system so that they can be managed by setParticles().
     * @param solidParticleArray an array populated with Solid Particles objects
     * @returns the SPS
     */
    insertParticlesFromArray(solidParticleArray: SolidParticle[]): SolidParticleSystem;
    /**
     * Creates a new particle and modifies the SPS mesh geometry :
     * - calls _meshBuilder() to increase the SPS mesh geometry step by step
     * - calls _addParticle() to populate the particle array
     * factorized code from addShape() and insertParticlesFromArray()
     * @param idx particle index in the particles array
     * @param i particle index in its shape
     * @param modelShape particle ModelShape object
     * @param shape shape vertex array
     * @param meshInd shape indices array
     * @param meshUV shape uv array
     * @param meshCol shape color array
     * @param meshNor shape normals array
     * @param bbInfo shape bounding info
     * @param storage target particle storage
     * @param options
     * @options addShape() passed options
     * @internal
     */
    protected _insertNewParticle(idx: number, i: number, modelShape: ModelShape, shape: Vector3[], meshInd: IndicesArray, meshUV: number[] | Float32Array, meshCol: number[] | Float32Array, meshNor: number[] | Float32Array, bbInfo: Nullable<BoundingInfo>, storage: Nullable<[]>, options: any): Nullable<SolidParticle>;
    /**
     *  Sets all the particles : this method actually really updates the mesh according to the particle positions, rotations, colors, textures, etc.
     *  This method calls `updateParticle()` for each particle of the SPS.
     *  For an animated SPS, it is usually called within the render loop.
     * This methods does nothing if called on a non updatable or not yet built SPS. Example : buildMesh() not called after having added or removed particles from an expandable SPS.
     * @param start The particle index in the particle array where to start to compute the particle property values _(default 0)_
     * @param end The particle index in the particle array where to stop to compute the particle property values _(default nbParticle - 1)_
     * @param update If the mesh must be finally updated on this call after all the particle computations _(default true)_
     * @returns the SPS.
     */
    setParticles(start?: number, end?: number, update?: boolean): SolidParticleSystem;
    /**
     * Disposes the SPS.
     */
    dispose(): void;
    /** Returns an object {idx: number faceId: number} for the picked particle from the passed pickingInfo object.
     * idx is the particle index in the SPS
     * faceId is the picked face index counted within this particle.
     * Returns null if the pickInfo can't identify a picked particle.
     * @param pickingInfo (PickingInfo object)
     * @returns {idx: number, faceId: number} or null
     */
    pickedParticle(pickingInfo: PickingInfo): Nullable<{
        idx: number;
        faceId: number;
    }>;
    /**
     * Returns a SolidParticle object from its identifier : particle.id
     * @param id (integer) the particle Id
     * @returns the searched particle or null if not found in the SPS.
     */
    getParticleById(id: number): Nullable<SolidParticle>;
    /**
     * Returns a new array populated with the particles having the passed shapeId.
     * @param shapeId (integer) the shape identifier
     * @returns a new solid particle array
     */
    getParticlesByShapeId(shapeId: number): SolidParticle[];
    /**
     * Populates the passed array "ref" with the particles having the passed shapeId.
     * @param shapeId the shape identifier
     * @returns the SPS
     * @param ref
     */
    getParticlesByShapeIdToRef(shapeId: number, ref: SolidParticle[]): SolidParticleSystem;
    /**
     * Computes the required SubMeshes according the materials assigned to the particles.
     * @returns the solid particle system.
     * Does nothing if called before the SPS mesh is built.
     */
    computeSubMeshes(): SolidParticleSystem;
    /**
     * Sorts the solid particles by material when MultiMaterial is enabled.
     * Updates the indices32 array.
     * Updates the indicesByMaterial array.
     * Updates the mesh indices array.
     * @returns the SPS
     * @internal
     */
    protected _sortParticlesByMaterial(): SolidParticleSystem;
    /**
     * Sets the material indexes by id materialIndexesById[id] = materialIndex
     * @internal
     */
    protected _setMaterialIndexesById(): void;
    /**
     * Returns an array with unique values of Materials from the passed array
     * @param array the material array to be checked and filtered
     * @internal
     */
    protected _filterUniqueMaterialId(array: Material[]): Material[];
    /**
     * Sets a new Standard Material as _defaultMaterial if not already set.
     * @internal
     */
    protected _setDefaultMaterial(): Material;
    /**
     * Visibility helper : Recomputes the visible size according to the mesh bounding box
     * doc : https://doc.babylonjs.com/features/featuresDeepDive/particles/solid_particle_system/sps_visibility
     * @returns the SPS.
     */
    refreshVisibleSize(): SolidParticleSystem;
    /**
     * Visibility helper : Sets the size of a visibility box, this sets the underlying mesh bounding box.
     * @param size the size (float) of the visibility box
     * note : this doesn't lock the SPS mesh bounding box.
     * doc : https://doc.babylonjs.com/features/featuresDeepDive/particles/solid_particle_system/sps_visibility
     */
    setVisibilityBox(size: number): void;
    /**
     * Gets whether the SPS as always visible or not
     * doc : https://doc.babylonjs.com/features/featuresDeepDive/particles/solid_particle_system/sps_visibility
     */
    get isAlwaysVisible(): boolean;
    /**
     * Sets the SPS as always visible or not
     * doc : https://doc.babylonjs.com/features/featuresDeepDive/particles/solid_particle_system/sps_visibility
     */
    set isAlwaysVisible(val: boolean);
    /**
     * Sets the SPS visibility box as locked or not. This enables/disables the underlying mesh bounding box updates.
     * doc : https://doc.babylonjs.com/features/featuresDeepDive/particles/solid_particle_system/sps_visibility
     */
    set isVisibilityBoxLocked(val: boolean);
    /**
     * Gets if the SPS visibility box as locked or not. This enables/disables the underlying mesh bounding box updates.
     * doc : https://doc.babylonjs.com/features/featuresDeepDive/particles/solid_particle_system/sps_visibility
     */
    get isVisibilityBoxLocked(): boolean;
    /**
     * Tells to `setParticles()` to compute the particle rotations or not.
     * Default value : true. The SPS is faster when it's set to false.
     * Note : the particle rotations aren't stored values, so setting `computeParticleRotation` to false will prevents the particle to rotate.
     */
    set computeParticleRotation(val: boolean);
    /**
     * Tells to `setParticles()` to compute the particle colors or not.
     * Default value : true. The SPS is faster when it's set to false.
     * Note : the particle colors are stored values, so setting `computeParticleColor` to false will keep yet the last colors set.
     */
    set computeParticleColor(val: boolean);
    set computeParticleTexture(val: boolean);
    /**
     * Tells to `setParticles()` to call the vertex function for each vertex of each particle, or not.
     * Default value : false. The SPS is faster when it's set to false.
     * Note : the particle custom vertex positions aren't stored values.
     */
    set computeParticleVertex(val: boolean);
    /**
     * Tells to `setParticles()` to compute or not the mesh bounding box when computing the particle positions.
     */
    set computeBoundingBox(val: boolean);
    /**
     * Tells to `setParticles()` to sort or not the distance between each particle and the camera.
     * Skipped when `enableDepthSort` is set to `false` (default) at construction time.
     * Default : `true`
     */
    set depthSortParticles(val: boolean);
    /**
     * Gets if `setParticles()` computes the particle rotations or not.
     * Default value : true. The SPS is faster when it's set to false.
     * Note : the particle rotations aren't stored values, so setting `computeParticleRotation` to false will prevents the particle to rotate.
     */
    get computeParticleRotation(): boolean;
    /**
     * Gets if `setParticles()` computes the particle colors or not.
     * Default value : true. The SPS is faster when it's set to false.
     * Note : the particle colors are stored values, so setting `computeParticleColor` to false will keep yet the last colors set.
     */
    get computeParticleColor(): boolean;
    /**
     * Gets if `setParticles()` computes the particle textures or not.
     * Default value : true. The SPS is faster when it's set to false.
     * Note : the particle textures are stored values, so setting `computeParticleTexture` to false will keep yet the last colors set.
     */
    get computeParticleTexture(): boolean;
    /**
     * Gets if `setParticles()` calls the vertex function for each vertex of each particle, or not.
     * Default value : false. The SPS is faster when it's set to false.
     * Note : the particle custom vertex positions aren't stored values.
     */
    get computeParticleVertex(): boolean;
    /**
     * Gets if `setParticles()` computes or not the mesh bounding box when computing the particle positions.
     */
    get computeBoundingBox(): boolean;
    /**
     * Gets if `setParticles()` sorts or not the distance between each particle and the camera.
     * Skipped when `enableDepthSort` is set to `false` (default) at construction time.
     * Default : `true`
     */
    get depthSortParticles(): boolean;
    /**
     * Gets if the SPS is created as expandable at construction time.
     * Default : `false`
     */
    get expandable(): boolean;
    /**
     * Gets if the SPS supports the Multi Materials
     */
    get multimaterialEnabled(): boolean;
    /**
     * Gets if the SPS uses the model materials for its own multimaterial.
     */
    get useModelMaterial(): boolean;
    /**
     * The SPS used material array.
     */
    get materials(): Material[];
    /**
     * Sets the SPS MultiMaterial from the passed materials.
     * Note : the passed array is internally copied and not used then by reference.
     * @param materials an array of material objects. This array indexes are the materialIndex values of the particles.
     */
    setMultiMaterial(materials: Material[]): void;
    /**
     * The SPS computed multimaterial object
     */
    get multimaterial(): MultiMaterial;
    set multimaterial(mm: MultiMaterial);
    /**
     * If the subMeshes must be updated on the next call to setParticles()
     */
    get autoUpdateSubMeshes(): boolean;
    set autoUpdateSubMeshes(val: boolean);
    /**
     * This function does nothing. It may be overwritten to set all the particle first values.
     * The SPS doesn't call this function, you may have to call it by your own.
     * doc : https://doc.babylonjs.com/features/featuresDeepDive/particles/solid_particle_system/manage_sps_particles
     */
    initParticles(): void;
    /**
     * This function does nothing. It may be overwritten to recycle a particle.
     * The SPS doesn't call this function, you may have to call it by your own.
     * doc : https://doc.babylonjs.com/features/featuresDeepDive/particles/solid_particle_system/manage_sps_particles
     * @param particle The particle to recycle
     * @returns the recycled particle
     */
    recycleParticle(particle: SolidParticle): SolidParticle;
    /**
     * Updates a particle : this function should  be overwritten by the user.
     * It is called on each particle by `setParticles()`. This is the place to code each particle behavior.
     * doc : https://doc.babylonjs.com/features/featuresDeepDive/particles/solid_particle_system/manage_sps_particles
     * @example : just set a particle position or velocity and recycle conditions
     * @param particle The particle to update
     * @returns the updated particle
     */
    updateParticle(particle: SolidParticle): SolidParticle;
    /**
     * Updates a vertex of a particle : it can be overwritten by the user.
     * This will be called on each vertex particle by `setParticles()` if `computeParticleVertex` is set to true only.
     * @param particle the current particle
     * @param vertex the current vertex of the current particle : a SolidParticleVertex object
     * @param pt the index of the current vertex in the particle shape
     * doc : https://doc.babylonjs.com/features/featuresDeepDive/particles/solid_particle_system/sps_vertices
     * @example : just set a vertex particle position or color
     * @returns the sps
     */
    updateParticleVertex(particle: SolidParticle, vertex: SolidParticleVertex, pt: number): SolidParticleSystem;
    /**
     * This will be called before any other treatment by `setParticles()` and will be passed three parameters.
     * This does nothing and may be overwritten by the user.
     * @param start the particle index in the particle array where to stop to iterate, same than the value passed to setParticle()
     * @param stop the particle index in the particle array where to stop to iterate, same than the value passed to setParticle()
     * @param update the boolean update value actually passed to setParticles()
     */
    beforeUpdateParticles(start?: number, stop?: number, update?: boolean): void;
    /**
     * This will be called  by `setParticles()` after all the other treatments and just before the actual mesh update.
     * This will be passed three parameters.
     * This does nothing and may be overwritten by the user.
     * @param start the particle index in the particle array where to stop to iterate, same than the value passed to setParticle()
     * @param stop the particle index in the particle array where to stop to iterate, same than the value passed to setParticle()
     * @param update the boolean update value actually passed to setParticles()
     */
    afterUpdateParticles(start?: number, stop?: number, update?: boolean): void;
}

}
declare module "babylonjs/Particles/subEmitter" {
import { ThinEngine } from "babylonjs/Engines/thinEngine";
import { Scene } from "babylonjs/scene";
import { ParticleSystem } from "babylonjs/Particles/particleSystem";
/**
 * Type of sub emitter
 */
export enum SubEmitterType {
    /**
     * Attached to the particle over it's lifetime
     */
    ATTACHED = 0,
    /**
     * Created when the particle dies
     */
    END = 1
}
/**
 * Sub emitter class used to emit particles from an existing particle
 */
export class SubEmitter {
    /**
     * the particle system to be used by the sub emitter
     */
    particleSystem: ParticleSystem;
    /**
     * Type of the submitter (Default: END)
     */
    type: SubEmitterType;
    /**
     * If the particle should inherit the direction from the particle it's attached to. (+Y will face the direction the particle is moving) (Default: false)
     * Note: This only is supported when using an emitter of type Mesh
     */
    inheritDirection: boolean;
    /**
     * How much of the attached particles speed should be added to the sub emitted particle (default: 0)
     */
    inheritedVelocityAmount: number;
    /**
     * Creates a sub emitter
     * @param particleSystem the particle system to be used by the sub emitter
     */
    constructor(
    /**
     * the particle system to be used by the sub emitter
     */
    particleSystem: ParticleSystem);
    /**
     * Clones the sub emitter
     * @returns the cloned sub emitter
     */
    clone(): SubEmitter;
    /**
     * Serialize current object to a JSON object
     * @param serializeTexture defines if the texture must be serialized as well
     * @returns the serialized object
     */
    serialize(serializeTexture?: boolean): any;
    /**
     * @internal
     */
    static _ParseParticleSystem(system: any, sceneOrEngine: Scene | ThinEngine, rootUrl: string, doNotStart?: boolean): ParticleSystem;
    /**
     * Creates a new SubEmitter from a serialized JSON version
     * @param serializationObject defines the JSON object to read from
     * @param sceneOrEngine defines the hosting scene or the hosting engine
     * @param rootUrl defines the rootUrl for data loading
     * @returns a new SubEmitter
     */
    static Parse(serializationObject: any, sceneOrEngine: Scene | ThinEngine, rootUrl: string): SubEmitter;
    /** Release associated resources */
    dispose(): void;
}
export {};

}
declare module "babylonjs/Particles/webgl2ParticleSystem" {
import { VertexBuffer, Buffer } from "babylonjs/Buffers/buffer";
import { ThinEngine } from "babylonjs/Engines/thinEngine";
import { IGPUParticleSystemPlatform } from "babylonjs/Particles/IGPUParticleSystemPlatform";
import { GPUParticleSystem } from "babylonjs/Particles/gpuParticleSystem";
import { DataArray } from "babylonjs/types";
import { DataBuffer } from "babylonjs/Buffers/dataBuffer";
import { UniformBufferEffectCommonAccessor } from "babylonjs/Materials/uniformBufferEffectCommonAccessor";
import "babylonjs/Shaders/gpuUpdateParticles.fragment";
import "babylonjs/Shaders/gpuUpdateParticles.vertex";
/** @internal */
export class WebGL2ParticleSystem implements IGPUParticleSystemPlatform {
    private _parent;
    private _engine;
    private _updateEffect;
    private _updateEffectOptions;
    private _renderVAO;
    private _updateVAO;
    readonly alignDataInBuffer: boolean;
    constructor(parent: GPUParticleSystem, engine: ThinEngine);
    isUpdateBufferCreated(): boolean;
    isUpdateBufferReady(): boolean;
    createUpdateBuffer(defines: string): UniformBufferEffectCommonAccessor;
    createVertexBuffers(updateBuffer: Buffer, renderVertexBuffers: {
        [key: string]: VertexBuffer;
    }): void;
    createParticleBuffer(data: number[]): DataArray | DataBuffer;
    bindDrawBuffers(index: number): void;
    preUpdateParticleBuffer(): void;
    updateParticleBuffer(index: number, targetBuffer: Buffer, currentActiveCount: number): void;
    releaseBuffers(): void;
    releaseVertexBuffers(): void;
    private _createUpdateVAO;
}

}
declare module "babylonjs/Physics/index" {
export * from "babylonjs/Physics/v1/index";
export * from "babylonjs/Physics/v2/index";
export * from "babylonjs/Physics/physicsEngineComponent";
export * from "babylonjs/Physics/v1/physicsEngineComponent";
export * from "babylonjs/Physics/physicsHelper";

}
declare module "babylonjs/Physics/IPhysicsEngine" {
import { Vector3 } from "babylonjs/Maths/math.vector";
import { PhysicsRaycastResult } from "babylonjs/Physics/physicsRaycastResult";
import { IPhysicsEnginePlugin as IPhysicsEnginePluginV1 } from "babylonjs/Physics/v1/IPhysicsEnginePlugin";
import { IPhysicsEnginePluginV2 } from "babylonjs/Physics/v2/IPhysicsEnginePlugin";
/**
 * Interface used to define a physics engine
 * @see https://doc.babylonjs.com/features/featuresDeepDive/physics/usingPhysicsEngine
 */
export interface IPhysicsEngine {
    /**
     * Gets the gravity vector used by the simulation
     */
    gravity: Vector3;
    /**
     *
     */
    getPluginVersion(): number;
    /**
     * Sets the gravity vector used by the simulation
     * @param gravity defines the gravity vector to use
     */
    setGravity(gravity: Vector3): void;
    /**
     * Set the time step of the physics engine.
     * Default is 1/60.
     * To slow it down, enter 1/600 for example.
     * To speed it up, 1/30
     * @param newTimeStep the new timestep to apply to this world.
     */
    setTimeStep(newTimeStep: number): void;
    /**
     * Get the time step of the physics engine.
     * @returns the current time step
     */
    getTimeStep(): number;
    /**
     * Set the sub time step of the physics engine.
     * Default is 0 meaning there is no sub steps
     * To increase physics resolution precision, set a small value (like 1 ms)
     * @param subTimeStep defines the new sub timestep used for physics resolution.
     */
    setSubTimeStep(subTimeStep: number): void;
    /**
     * Get the sub time step of the physics engine.
     * @returns the current sub time step
     */
    getSubTimeStep(): number;
    /**
     * Release all resources
     */
    dispose(): void;
    /**
     * Gets the name of the current physics plugin
     * @returns the name of the plugin
     */
    getPhysicsPluginName(): string;
    /**
     * Gets the current plugin used to run the simulation
     * @returns current plugin
     */
    getPhysicsPlugin(): IPhysicsEnginePluginV1 | IPhysicsEnginePluginV2 | null;
    /**
     * Does a raycast in the physics world
     * @param from when should the ray start?
     * @param to when should the ray end?
     * @returns PhysicsRaycastResult
     */
    raycast(from: Vector3, to: Vector3): PhysicsRaycastResult;
    /**
     * Called by the scene. No need to call it.
     * @param delta defines the timespan between frames
     */
    _step(delta: number): void;
}

}
declare module "babylonjs/Physics/joinedPhysicsEngineComponent" {
import { Nullable } from "babylonjs/types";
import { Observable } from "babylonjs/Misc/observable";
import { Vector3 } from "babylonjs/Maths/math.vector";
import { ISceneComponent } from "babylonjs/sceneComponent";
import { Scene } from "babylonjs/scene";
import { IPhysicsEngine } from "babylonjs/Physics/IPhysicsEngine";
import { IPhysicsEnginePlugin as IPhysicsEnginePluginV1 } from "babylonjs/Physics/v1/IPhysicsEnginePlugin";
import { IPhysicsEnginePluginV2 } from "babylonjs/Physics/v2/IPhysicsEnginePlugin";
module "babylonjs/scene" {
    /**
     *
     */
    interface Scene {
        /** @internal (Backing field) */
        _physicsEngine: Nullable<IPhysicsEngine>;
        /** @internal */
        _physicsTimeAccumulator: number;
        /**
         * Gets the current physics engine
         * @returns a IPhysicsEngine or null if none attached
         */
        getPhysicsEngine(): Nullable<IPhysicsEngine>;
        /**
         * Enables physics to the current scene
         * @param gravity defines the scene's gravity for the physics engine. defaults to real earth gravity : (0, -9.81, 0)
         * @param plugin defines the physics engine to be used. defaults to CannonJS.
         * @returns a boolean indicating if the physics engine was initialized
         */
        enablePhysics(gravity?: Nullable<Vector3>, plugin?: IPhysicsEnginePluginV1 | IPhysicsEnginePluginV2): boolean;
        /**
         * Disables and disposes the physics engine associated with the scene
         */
        disablePhysicsEngine(): void;
        /**
         * Gets a boolean indicating if there is an active physics engine
         * @returns a boolean indicating if there is an active physics engine
         */
        isPhysicsEnabled(): boolean;
        /**
         * Deletes a physics compound impostor
         * @param compound defines the compound to delete
         */
        deleteCompoundImpostor(compound: any): void;
        /**
         * An event triggered when physic simulation is about to be run
         */
        onBeforePhysicsObservable: Observable<Scene>;
        /**
         * An event triggered when physic simulation has been done
         */
        onAfterPhysicsObservable: Observable<Scene>;
    }
}
/**
 * Defines the physics engine scene component responsible to manage a physics engine
 */
export class PhysicsEngineSceneComponent implements ISceneComponent {
    /**
     * The component name helpful to identify the component in the list of scene components.
     */
    readonly name: string;
    /**
     * The scene the component belongs to.
     */
    scene: Scene;
    /**
     * Creates a new instance of the component for the given scene
     * @param scene Defines the scene to register the component in
     */
    constructor(scene: Scene);
    /**
     * Registers the component in a given scene
     */
    register(): void;
    /**
     * Rebuilds the elements related to this component in case of
     * context lost for instance.
     */
    rebuild(): void;
    /**
     * Disposes the component and the associated resources
     */
    dispose(): void;
}

}
declare module "babylonjs/Physics/physicsEngine" {
export { PhysicsEngine } from "babylonjs/Physics/v1/physicsEngine";

}
declare module "babylonjs/Physics/physicsEngineComponent" {
import "babylonjs/Physics/joinedPhysicsEngineComponent";
import "babylonjs/Physics/v1/physicsEngineComponent";

}
declare module "babylonjs/Physics/physicsHelper" {
import { Nullable } from "babylonjs/types";
import { Vector3 } from "babylonjs/Maths/math.vector";
import { Mesh } from "babylonjs/Meshes/mesh";
import { Scene } from "babylonjs/scene";
import { PhysicsImpostor } from "babylonjs/Physics/v1/physicsImpostor";
import { PhysicsBody } from "babylonjs/Physics/v2/physicsBody";
/**
 * A helper for physics simulations
 * @see https://doc.babylonjs.com/features/featuresDeepDive/physics/usingPhysicsEngine#further-functionality-of-the-impostor-class
 */
export class PhysicsHelper {
    private _scene;
    private _physicsEngine;
    private _hitData;
    /**
     * Initializes the Physics helper
     * @param scene Babylon.js scene
     */
    constructor(scene: Scene);
    /**
     * Applies a radial explosion impulse
     * @param origin the origin of the explosion
     * @param radiusOrEventOptions the radius or the options of radial explosion
     * @param strength the explosion strength
     * @param falloff possible options: Constant & Linear. Defaults to Constant
     * @returns A physics radial explosion event, or null
     */
    applyRadialExplosionImpulse(origin: Vector3, radiusOrEventOptions: number | PhysicsRadialExplosionEventOptions, strength?: number, falloff?: PhysicsRadialImpulseFalloff): Nullable<PhysicsRadialExplosionEvent>;
    /**
     * Applies a radial explosion force
     * @param origin the origin of the explosion
     * @param radiusOrEventOptions the radius or the options of radial explosion
     * @param strength the explosion strength
     * @param falloff possible options: Constant & Linear. Defaults to Constant
     * @returns A physics radial explosion event, or null
     */
    applyRadialExplosionForce(origin: Vector3, radiusOrEventOptions: number | PhysicsRadialExplosionEventOptions, strength?: number, falloff?: PhysicsRadialImpulseFalloff): Nullable<PhysicsRadialExplosionEvent>;
    /**
     * Creates a gravitational field
     * @param origin the origin of the gravitational field
     * @param radiusOrEventOptions the radius or the options of radial gravitational field
     * @param strength the gravitational field strength
     * @param falloff possible options: Constant & Linear. Defaults to Constant
     * @returns A physics gravitational field event, or null
     */
    gravitationalField(origin: Vector3, radiusOrEventOptions: number | PhysicsRadialExplosionEventOptions, strength?: number, falloff?: PhysicsRadialImpulseFalloff): Nullable<PhysicsGravitationalFieldEvent>;
    /**
     * Creates a physics updraft event
     * @param origin the origin of the updraft
     * @param radiusOrEventOptions the radius or the options of the updraft
     * @param strength the strength of the updraft
     * @param height the height of the updraft
     * @param updraftMode possible options: Center & Perpendicular. Defaults to Center
     * @returns A physics updraft event, or null
     */
    updraft(origin: Vector3, radiusOrEventOptions: number | PhysicsUpdraftEventOptions, strength?: number, height?: number, updraftMode?: PhysicsUpdraftMode): Nullable<PhysicsUpdraftEvent>;
    /**
     * Creates a physics vortex event
     * @param origin the of the vortex
     * @param radiusOrEventOptions the radius or the options of the vortex
     * @param strength the strength of the vortex
     * @param height   the height of the vortex
     * @returns a Physics vortex event, or null
     * A physics vortex event or null
     */
    vortex(origin: Vector3, radiusOrEventOptions: number | PhysicsVortexEventOptions, strength?: number, height?: number): Nullable<PhysicsVortexEvent>;
    private _copyPhysicsHitData;
}
/**
 * Represents a physics radial explosion event
 */
class PhysicsRadialExplosionEvent {
    private _scene;
    private _options;
    private _sphere;
    private _dataFetched;
    /**
     * Initializes a radial explosion event
     * @param _scene BabylonJS scene
     * @param _options The options for the vortex event
     */
    constructor(_scene: Scene, _options: PhysicsRadialExplosionEventOptions);
    /**
     * Returns the data related to the radial explosion event (sphere).
     * @returns The radial explosion event data
     */
    getData(): PhysicsRadialExplosionEventData;
    private _getHitData;
    /**
     * Returns the force and contact point of the body or false, if the body is not affected by the force/impulse.
     * @param body A physics body
     * @param origin the origin of the explosion
     * @returns A physics force and contact point, or null
     */
    getBodyHitData(body: PhysicsBody, origin: Vector3, data: PhysicsHitData): boolean;
    /**
     * Returns the force and contact point of the impostor or false, if the impostor is not affected by the force/impulse.
     * @param impostor A physics imposter
     * @param origin the origin of the explosion
     * @returns A physics force and contact point, or null
     */
    getImpostorHitData(impostor: PhysicsImpostor, origin: Vector3, data: PhysicsHitData): boolean;
    /**
     * Triggers affected impostors callbacks
     * @param affectedImpostorsWithData defines the list of affected impostors (including associated data)
     */
    triggerAffectedImpostorsCallback(affectedImpostorsWithData: Array<PhysicsAffectedImpostorWithData>): void;
    /**
     * Triggers affected bodies callbacks
     * @param affectedBodiesWithData defines the list of affected bodies (including associated data)
     */
    triggerAffectedBodiesCallback(affectedBodiesWithData: Array<PhysicsAffectedBodyWithData>): void;
    /**
     * Disposes the sphere.
     * @param force Specifies if the sphere should be disposed by force
     */
    dispose(force?: boolean): void;
    /*** Helpers ***/
    private _prepareSphere;
    private _intersectsWithSphere;
}
/**
 * Represents a gravitational field event
 */
class PhysicsGravitationalFieldEvent {
    private _physicsHelper;
    private _scene;
    private _origin;
    private _options;
    private _tickCallback;
    private _sphere;
    private _dataFetched;
    /**
     * Initializes the physics gravitational field event
     * @param _physicsHelper A physics helper
     * @param _scene BabylonJS scene
     * @param _origin The origin position of the gravitational field event
     * @param _options The options for the vortex event
     */
    constructor(_physicsHelper: PhysicsHelper, _scene: Scene, _origin: Vector3, _options: PhysicsRadialExplosionEventOptions);
    /**
     * Returns the data related to the gravitational field event (sphere).
     * @returns A gravitational field event
     */
    getData(): PhysicsGravitationalFieldEventData;
    /**
     * Enables the gravitational field.
     */
    enable(): void;
    /**
     * Disables the gravitational field.
     */
    disable(): void;
    /**
     * Disposes the sphere.
     * @param force The force to dispose from the gravitational field event
     */
    dispose(force?: boolean): void;
    private _tick;
}
/**
 * Represents a physics updraft event
 */
class PhysicsUpdraftEvent {
    private _scene;
    private _origin;
    private _options;
    private _physicsEngine;
    private _originTop;
    private _originDirection;
    private _tickCallback;
    private _cylinder;
    private _cylinderPosition;
    private _dataFetched;
    private static hitData;
    /**
     * Initializes the physics updraft event
     * @param _scene BabylonJS scene
     * @param _origin The origin position of the updraft
     * @param _options The options for the updraft event
     */
    constructor(_scene: Scene, _origin: Vector3, _options: PhysicsUpdraftEventOptions);
    /**
     * Returns the data related to the updraft event (cylinder).
     * @returns A physics updraft event
     */
    getData(): PhysicsUpdraftEventData;
    /**
     * Enables the updraft.
     */
    enable(): void;
    /**
     * Disables the updraft.
     */
    disable(): void;
    /**
     * Disposes the cylinder.
     * @param force Specifies if the updraft should be disposed by force
     */
    dispose(force?: boolean): void;
    private _getHitData;
    private _getBodyHitData;
    private _getImpostorHitData;
    private _tick;
    /*** Helpers ***/
    private _prepareCylinder;
    private _intersectsWithCylinder;
}
/**
 * Represents a physics vortex event
 */
class PhysicsVortexEvent {
    private _scene;
    private _origin;
    private _options;
    private _physicsEngine;
    private _originTop;
    private _tickCallback;
    private _cylinder;
    private _cylinderPosition;
    private _dataFetched;
    private static originOnPlane;
    private static hitData;
    /**
     * Initializes the physics vortex event
     * @param _scene The BabylonJS scene
     * @param _origin The origin position of the vortex
     * @param _options The options for the vortex event
     */
    constructor(_scene: Scene, _origin: Vector3, _options: PhysicsVortexEventOptions);
    /**
     * Returns the data related to the vortex event (cylinder).
     * @returns The physics vortex event data
     */
    getData(): PhysicsVortexEventData;
    /**
     * Enables the vortex.
     */
    enable(): void;
    /**
     * Disables the cortex.
     */
    disable(): void;
    /**
     * Disposes the sphere.
     * @param force
     */
    dispose(force?: boolean): void;
    private _getHitData;
    private _getBodyHitData;
    private _getImpostorHitData;
    private _tick;
    /*** Helpers ***/
    private _prepareCylinder;
    private _intersectsWithCylinder;
}
/**
 * Options fot the radial explosion event
 * @see https://doc.babylonjs.com/features/featuresDeepDive/physics/usingPhysicsEngine#further-functionality-of-the-impostor-class
 */
export class PhysicsRadialExplosionEventOptions {
    /**
     * The radius of the sphere for the radial explosion.
     */
    radius: number;
    /**
     * The strength of the explosion.
     */
    strength: number;
    /**
     * The strength of the force in correspondence to the distance of the affected object
     */
    falloff: PhysicsRadialImpulseFalloff;
    /**
     * Sphere options for the radial explosion.
     */
    sphere: {
        segments: number;
        diameter: number;
    };
    /**
     * Sphere options for the radial explosion.
     */
    affectedImpostorsCallback: (affectedImpostorsWithData: Array<PhysicsAffectedImpostorWithData>) => void;
    /**
     * Sphere options for the radial explosion.
     */
    affectedBodiesCallback: (affectedBodiesWithData: Array<PhysicsAffectedBodyWithData>) => void;
}
/**
 * Options fot the updraft event
 * @see https://doc.babylonjs.com/features/featuresDeepDive/physics/usingPhysicsEngine#further-functionality-of-the-impostor-class
 */
export class PhysicsUpdraftEventOptions {
    /**
     * The radius of the cylinder for the vortex
     */
    radius: number;
    /**
     * The strength of the updraft.
     */
    strength: number;
    /**
     * The height of the cylinder for the updraft.
     */
    height: number;
    /**
     * The mode for the the updraft.
     */
    updraftMode: PhysicsUpdraftMode;
}
/**
 * Options fot the vortex event
 * @see https://doc.babylonjs.com/features/featuresDeepDive/physics/usingPhysicsEngine#further-functionality-of-the-impostor-class
 */
export class PhysicsVortexEventOptions {
    /**
     * The radius of the cylinder for the vortex
     */
    radius: number;
    /**
     * The strength of the vortex.
     */
    strength: number;
    /**
     * The height of the cylinder for the vortex.
     */
    height: number;
    /**
     * At which distance, relative to the radius the centripetal forces should kick in? Range: 0-1
     */
    centripetalForceThreshold: number;
    /**
     * This multiplier determines with how much force the objects will be pushed sideways/around the vortex, when below the threshold.
     */
    centripetalForceMultiplier: number;
    /**
     * This multiplier determines with how much force the objects will be pushed sideways/around the vortex, when above the threshold.
     */
    centrifugalForceMultiplier: number;
    /**
     * This multiplier determines with how much force the objects will be pushed upwards, when in the vortex.
     */
    updraftForceMultiplier: number;
}
/**
 * The strength of the force in correspondence to the distance of the affected object
 * @see https://doc.babylonjs.com/features/featuresDeepDive/physics/usingPhysicsEngine#further-functionality-of-the-impostor-class
 */
export enum PhysicsRadialImpulseFalloff {
    /** Defines that impulse is constant in strength across it's whole radius */
    Constant = 0,
    /** Defines that impulse gets weaker if it's further from the origin */
    Linear = 1
}
/**
 * The strength of the force in correspondence to the distance of the affected object
 * @see https://doc.babylonjs.com/features/featuresDeepDive/physics/usingPhysicsEngine#further-functionality-of-the-impostor-class
 */
export enum PhysicsUpdraftMode {
    /** Defines that the upstream forces will pull towards the top center of the cylinder */
    Center = 0,
    /** Defines that once a impostor is inside the cylinder, it will shoot out perpendicular from the ground of the cylinder */
    Perpendicular = 1
}
/**
 * Interface for a physics hit data
 * @see https://doc.babylonjs.com/features/featuresDeepDive/physics/usingPhysicsEngine#further-functionality-of-the-impostor-class
 */
export interface PhysicsHitData {
    /**
     * The force applied at the contact point
     */
    force: Vector3;
    /**
     * The contact point
     */
    contactPoint: Vector3;
    /**
     * The distance from the origin to the contact point
     */
    distanceFromOrigin: number;
}
/**
 * Interface for radial explosion event data
 * @see https://doc.babylonjs.com/features/featuresDeepDive/physics/usingPhysicsEngine#further-functionality-of-the-impostor-class
 */
export interface PhysicsRadialExplosionEventData {
    /**
     * A sphere used for the radial explosion event
     */
    sphere: Mesh;
}
/**
 * Interface for gravitational field event data
 * @see https://doc.babylonjs.com/features/featuresDeepDive/physics/usingPhysicsEngine#further-functionality-of-the-impostor-class
 */
export interface PhysicsGravitationalFieldEventData {
    /**
     * A sphere mesh used for the gravitational field event
     */
    sphere: Mesh;
}
/**
 * Interface for updraft event data
 * @see https://doc.babylonjs.com/features/featuresDeepDive/physics/usingPhysicsEngine#further-functionality-of-the-impostor-class
 */
export interface PhysicsUpdraftEventData {
    /**
     * A cylinder used for the updraft event
     */
    cylinder: Mesh;
}
/**
 * Interface for vortex event data
 * @see https://doc.babylonjs.com/features/featuresDeepDive/physics/usingPhysicsEngine#further-functionality-of-the-impostor-class
 */
export interface PhysicsVortexEventData {
    /**
     * A cylinder used for the vortex event
     */
    cylinder: Mesh;
}
/**
 * Interface for an affected physics impostor
 * @see https://doc.babylonjs.com/features/featuresDeepDive/physics/usingPhysicsEngine#further-functionality-of-the-impostor-class
 */
export interface PhysicsAffectedImpostorWithData {
    /**
     * The impostor affected by the effect
     */
    impostor: PhysicsImpostor;
    /**
     * The data about the hit/force from the explosion
     */
    hitData: PhysicsHitData;
}
/**
 * Interface for an affected physics body
 * @see
 */
export interface PhysicsAffectedBodyWithData {
    /**
     * The impostor affected by the effect
     */
    body: PhysicsBody;
    /**
     * The data about the hit/force from the explosion
     */
    hitData: PhysicsHitData;
}
export {};

}
declare module "babylonjs/Physics/physicsImpostor" {
export * from "babylonjs/Physics/v1/physicsImpostor";

}
declare module "babylonjs/Physics/physicsJoint" {
export * from "babylonjs/Physics/v1/physicsJoint";

}
declare module "babylonjs/Physics/physicsRaycastResult" {
import { Vector3 } from "babylonjs/Maths/math.vector";
/**
 * Holds the data for the raycast result
 * @see https://doc.babylonjs.com/features/featuresDeepDive/physics/usingPhysicsEngine
 */
export class PhysicsRaycastResult {
    private _hasHit;
    private _hitDistance;
    private _hitNormalWorld;
    private _hitPointWorld;
    private _rayFromWorld;
    private _rayToWorld;
    /**
     * Gets if there was a hit
     */
    get hasHit(): boolean;
    /**
     * Gets the distance from the hit
     */
    get hitDistance(): number;
    /**
     * Gets the hit normal/direction in the world
     */
    get hitNormalWorld(): Vector3;
    /**
     * Gets the hit point in the world
     */
    get hitPointWorld(): Vector3;
    /**
     * Gets the ray "start point" of the ray in the world
     */
    get rayFromWorld(): Vector3;
    /**
     * Gets the ray "end point" of the ray in the world
     */
    get rayToWorld(): Vector3;
    /**
     * Sets the hit data (normal & point in world space)
     * @param hitNormalWorld defines the normal in world space
     * @param hitPointWorld defines the point in world space
     */
    setHitData(hitNormalWorld: IXYZ, hitPointWorld: IXYZ): void;
    /**
     * Sets the distance from the start point to the hit point
     * @param distance
     */
    setHitDistance(distance: number): void;
    /**
     * Calculates the distance manually
     */
    calculateHitDistance(): void;
    /**
     * Resets all the values to default
     * @param from The from point on world space
     * @param to The to point on world space
     */
    reset(from?: Vector3, to?: Vector3): void;
}
/**
 * Interface for the size containing width and height
 */
interface IXYZ {
    /**
     * X
     */
    x: number;
    /**
     * Y
     */
    y: number;
    /**
     * Z
     */
    z: number;
}
export {};

}
declare module "babylonjs/Physics/Plugins/ammoJSPlugin" {
export * from "babylonjs/Physics/v1/Plugins/ammoJSPlugin";

}
declare module "babylonjs/Physics/Plugins/cannonJSPlugin" {
export * from "babylonjs/Physics/v1/Plugins/cannonJSPlugin";

}
declare module "babylonjs/Physics/Plugins/oimoJSPlugin" {
export * from "babylonjs/Physics/v1/Plugins/oimoJSPlugin";

}
declare module "babylonjs/Physics/v1/index" {
export * from "babylonjs/Physics/v1/IPhysicsEnginePlugin";
export * from "babylonjs/Physics/v1/physicsEngine";
export * from "babylonjs/Physics/v1/physicsEngineComponent";
export * from "babylonjs/Physics/v1/physicsImpostor";
export * from "babylonjs/Physics/v1/physicsJoint";
export * from "babylonjs/Physics/v1/Plugins/index";

}
declare module "babylonjs/Physics/v1/IPhysicsEnginePlugin" {
import { Nullable } from "babylonjs/types";
import { Vector3, Quaternion } from "babylonjs/Maths/math.vector";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { PhysicsImpostor } from "babylonjs/Physics/v1/physicsImpostor";
import { PhysicsJoint, IMotorEnabledJoint } from "babylonjs/Physics/v1/physicsJoint";
import { PhysicsRaycastResult } from "babylonjs/Physics/physicsRaycastResult";
/**
 * Interface used to describe a physics joint
 */
export interface PhysicsImpostorJoint {
    /** Defines the main impostor to which the joint is linked */
    mainImpostor: PhysicsImpostor;
    /** Defines the impostor that is connected to the main impostor using this joint */
    connectedImpostor: PhysicsImpostor;
    /** Defines the joint itself */
    joint: PhysicsJoint;
}
/** @internal */
export interface IPhysicsEnginePlugin {
    /**
     *
     */
    world: any;
    /**
     *
     */
    name: string;
    setGravity(gravity: Vector3): void;
    setTimeStep(timeStep: number): void;
    getTimeStep(): number;
    executeStep(delta: number, impostors: Array<PhysicsImpostor>): void;
    getPluginVersion(): number;
    applyImpulse(impostor: PhysicsImpostor, force: Vector3, contactPoint: Vector3): void;
    applyForce(impostor: PhysicsImpostor, force: Vector3, contactPoint: Vector3): void;
    generatePhysicsBody(impostor: PhysicsImpostor): void;
    removePhysicsBody(impostor: PhysicsImpostor): void;
    generateJoint(joint: PhysicsImpostorJoint): void;
    removeJoint(joint: PhysicsImpostorJoint): void;
    isSupported(): boolean;
    setTransformationFromPhysicsBody(impostor: PhysicsImpostor): void;
    setPhysicsBodyTransformation(impostor: PhysicsImpostor, newPosition: Vector3, newRotation: Quaternion): void;
    setLinearVelocity(impostor: PhysicsImpostor, velocity: Nullable<Vector3>): void;
    setAngularVelocity(impostor: PhysicsImpostor, velocity: Nullable<Vector3>): void;
    getLinearVelocity(impostor: PhysicsImpostor): Nullable<Vector3>;
    getAngularVelocity(impostor: PhysicsImpostor): Nullable<Vector3>;
    setBodyMass(impostor: PhysicsImpostor, mass: number): void;
    getBodyMass(impostor: PhysicsImpostor): number;
    getBodyFriction(impostor: PhysicsImpostor): number;
    setBodyFriction(impostor: PhysicsImpostor, friction: number): void;
    getBodyRestitution(impostor: PhysicsImpostor): number;
    setBodyRestitution(impostor: PhysicsImpostor, restitution: number): void;
    getBodyPressure?(impostor: PhysicsImpostor): number;
    setBodyPressure?(impostor: PhysicsImpostor, pressure: number): void;
    getBodyStiffness?(impostor: PhysicsImpostor): number;
    setBodyStiffness?(impostor: PhysicsImpostor, stiffness: number): void;
    getBodyVelocityIterations?(impostor: PhysicsImpostor): number;
    setBodyVelocityIterations?(impostor: PhysicsImpostor, velocityIterations: number): void;
    getBodyPositionIterations?(impostor: PhysicsImpostor): number;
    setBodyPositionIterations?(impostor: PhysicsImpostor, positionIterations: number): void;
    appendAnchor?(impostor: PhysicsImpostor, otherImpostor: PhysicsImpostor, width: number, height: number, influence: number, noCollisionBetweenLinkedBodies: boolean): void;
    appendHook?(impostor: PhysicsImpostor, otherImpostor: PhysicsImpostor, length: number, influence: number, noCollisionBetweenLinkedBodies: boolean): void;
    sleepBody(impostor: PhysicsImpostor): void;
    wakeUpBody(impostor: PhysicsImpostor): void;
    raycast(from: Vector3, to: Vector3): PhysicsRaycastResult;
    raycastToRef(from: Vector3, to: Vector3, result: PhysicsRaycastResult): void;
    updateDistanceJoint(joint: PhysicsJoint, maxDistance: number, minDistance?: number): void;
    setMotor(joint: IMotorEnabledJoint, speed: number, maxForce?: number, motorIndex?: number): void;
    setLimit(joint: IMotorEnabledJoint, upperLimit: number, lowerLimit?: number, motorIndex?: number): void;
    getRadius(impostor: PhysicsImpostor): number;
    getBoxSizeToRef(impostor: PhysicsImpostor, result: Vector3): void;
    syncMeshWithImpostor(mesh: AbstractMesh, impostor: PhysicsImpostor): void;
    dispose(): void;
}

}
declare module "babylonjs/Physics/v1/physicsEngine" {
import { Nullable } from "babylonjs/types";
import { Vector3 } from "babylonjs/Maths/math.vector";
import { IPhysicsEnginePlugin } from "babylonjs/Physics/v1/IPhysicsEnginePlugin";
import { IPhysicsEngine } from "babylonjs/Physics/IPhysicsEngine";
import { PhysicsImpostor, IPhysicsEnabledObject } from "babylonjs/Physics/v1/physicsImpostor";
import { PhysicsJoint } from "babylonjs/Physics/v1/physicsJoint";
import { PhysicsRaycastResult } from "babylonjs/Physics/physicsRaycastResult";
/**
 * Class used to control physics engine
 * @see https://doc.babylonjs.com/features/featuresDeepDive/physics/usingPhysicsEngine
 */
export class PhysicsEngine implements IPhysicsEngine {
    private _physicsPlugin;
    /**
     * Global value used to control the smallest number supported by the simulation
     */
    private _impostors;
    private _joints;
    private _subTimeStep;
    private _uniqueIdCounter;
    /**
     * Gets the gravity vector used by the simulation
     */
    gravity: Vector3;
    /**
     *
     * @returns version
     */
    getPluginVersion(): number;
    /**
     * Factory used to create the default physics plugin.
     * @returns The default physics plugin
     */
    static DefaultPluginFactory(): IPhysicsEnginePlugin;
    /**
     * Creates a new Physics Engine
     * @param gravity defines the gravity vector used by the simulation
     * @param _physicsPlugin defines the plugin to use (CannonJS by default)
     */
    constructor(gravity: Nullable<Vector3>, _physicsPlugin?: IPhysicsEnginePlugin);
    /**
     * Sets the gravity vector used by the simulation
     * @param gravity defines the gravity vector to use
     */
    setGravity(gravity: Vector3): void;
    /**
     * Set the time step of the physics engine.
     * Default is 1/60.
     * To slow it down, enter 1/600 for example.
     * To speed it up, 1/30
     * @param newTimeStep defines the new timestep to apply to this world.
     */
    setTimeStep(newTimeStep?: number): void;
    /**
     * Get the time step of the physics engine.
     * @returns the current time step
     */
    getTimeStep(): number;
    /**
     * Set the sub time step of the physics engine.
     * Default is 0 meaning there is no sub steps
     * To increase physics resolution precision, set a small value (like 1 ms)
     * @param subTimeStep defines the new sub timestep used for physics resolution.
     */
    setSubTimeStep(subTimeStep?: number): void;
    /**
     * Get the sub time step of the physics engine.
     * @returns the current sub time step
     */
    getSubTimeStep(): number;
    /**
     * Release all resources
     */
    dispose(): void;
    /**
     * Gets the name of the current physics plugin
     * @returns the name of the plugin
     */
    getPhysicsPluginName(): string;
    /**
     * Adding a new impostor for the impostor tracking.
     * This will be done by the impostor itself.
     * @param impostor the impostor to add
     */
    addImpostor(impostor: PhysicsImpostor): void;
    /**
     * Remove an impostor from the engine.
     * This impostor and its mesh will not longer be updated by the physics engine.
     * @param impostor the impostor to remove
     */
    removeImpostor(impostor: PhysicsImpostor): void;
    /**
     * Add a joint to the physics engine
     * @param mainImpostor defines the main impostor to which the joint is added.
     * @param connectedImpostor defines the impostor that is connected to the main impostor using this joint
     * @param joint defines the joint that will connect both impostors.
     */
    addJoint(mainImpostor: PhysicsImpostor, connectedImpostor: PhysicsImpostor, joint: PhysicsJoint): void;
    /**
     * Removes a joint from the simulation
     * @param mainImpostor defines the impostor used with the joint
     * @param connectedImpostor defines the other impostor connected to the main one by the joint
     * @param joint defines the joint to remove
     */
    removeJoint(mainImpostor: PhysicsImpostor, connectedImpostor: PhysicsImpostor, joint: PhysicsJoint): void;
    /**
     * Called by the scene. No need to call it.
     * @param delta defines the timespan between frames
     */
    _step(delta: number): void;
    /**
     * Gets the current plugin used to run the simulation
     * @returns current plugin
     */
    getPhysicsPlugin(): IPhysicsEnginePlugin;
    /**
     * Gets the list of physic impostors
     * @returns an array of PhysicsImpostor
     */
    getImpostors(): Array<PhysicsImpostor>;
    /**
     * Gets the impostor for a physics enabled object
     * @param object defines the object impersonated by the impostor
     * @returns the PhysicsImpostor or null if not found
     */
    getImpostorForPhysicsObject(object: IPhysicsEnabledObject): Nullable<PhysicsImpostor>;
    /**
     * Gets the impostor for a physics body object
     * @param body defines physics body used by the impostor
     * @returns the PhysicsImpostor or null if not found
     */
    getImpostorWithPhysicsBody(body: any): Nullable<PhysicsImpostor>;
    /**
     * Does a raycast in the physics world
     * @param from when should the ray start?
     * @param to when should the ray end?
     * @returns PhysicsRaycastResult
     */
    raycast(from: Vector3, to: Vector3): PhysicsRaycastResult;
    /**
     * Does a raycast in the physics world
     * @param from when should the ray start?
     * @param to when should the ray end?
     * @param result resulting PhysicsRaycastResult
     */
    raycastToRef(from: Vector3, to: Vector3, result: PhysicsRaycastResult): void;
}

}
declare module "babylonjs/Physics/v1/physicsEngineComponent" {
import { Nullable } from "babylonjs/types";
import { Observer } from "babylonjs/Misc/observable";
import { Vector3 } from "babylonjs/Maths/math.vector";
import { Mesh } from "babylonjs/Meshes/mesh";
import { Node } from "babylonjs/node";
import { PhysicsImpostor } from "babylonjs/Physics/v1/physicsImpostor";
module "babylonjs/Meshes/abstractMesh" {
    /**
     *
     */
    interface AbstractMesh {
        /** @internal */
        _physicsImpostor: Nullable<PhysicsImpostor>;
        /**
         * Gets or sets impostor used for physic simulation
         * @see https://doc.babylonjs.com/features/featuresDeepDive/physics
         */
        physicsImpostor: Nullable<PhysicsImpostor>;
        /**
         * Gets the current physics impostor
         * @see https://doc.babylonjs.com/features/featuresDeepDive/physics
         * @returns a physics impostor or null
         */
        getPhysicsImpostor(): Nullable<PhysicsImpostor>;
        /** Apply a physic impulse to the mesh
         * @param force defines the force to apply
         * @param contactPoint defines where to apply the force
         * @returns the current mesh
         * @see https://doc.babylonjs.com/features/featuresDeepDive/physics/usingPhysicsEngine
         */
        applyImpulse(force: Vector3, contactPoint: Vector3): AbstractMesh;
        /**
         * Creates a physic joint between two meshes
         * @param otherMesh defines the other mesh to use
         * @param pivot1 defines the pivot to use on this mesh
         * @param pivot2 defines the pivot to use on the other mesh
         * @param options defines additional options (can be plugin dependent)
         * @returns the current mesh
         * @see https://www.babylonjs-playground.com/#0BS5U0#0
         */
        setPhysicsLinkWith(otherMesh: Mesh, pivot1: Vector3, pivot2: Vector3, options?: any): AbstractMesh;
        /** @internal */
        _disposePhysicsObserver: Nullable<Observer<Node>>;
    }
}

}
declare module "babylonjs/Physics/v1/physicsImpostor" {
import { Nullable, IndicesArray } from "babylonjs/types";
import { Matrix } from "babylonjs/Maths/math.vector";
import { Vector3, Quaternion } from "babylonjs/Maths/math.vector";
import { TransformNode } from "babylonjs/Meshes/transformNode";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { Scene } from "babylonjs/scene";
import { Bone } from "babylonjs/Bones/bone";
import { BoundingInfo } from "babylonjs/Culling/boundingInfo";
import { PhysicsJointData } from "babylonjs/Physics/v1/physicsJoint";
import { PhysicsJoint } from "babylonjs/Physics/v1/physicsJoint";
import { Space } from "babylonjs/Maths/math.axis";
/**
 * The interface for the physics imposter parameters
 * @see https://doc.babylonjs.com/features/featuresDeepDive/physics/usingPhysicsEngine
 */
export interface PhysicsImpostorParameters {
    /**
     * The mass of the physics imposter
     */
    mass: number;
    /**
     * The friction of the physics imposter
     */
    friction?: number;
    /**
     * The coefficient of restitution of the physics imposter
     */
    restitution?: number;
    /**
     * The native options of the physics imposter
     */
    nativeOptions?: any;
    /**
     * Specifies if the parent should be ignored
     */
    ignoreParent?: boolean;
    /**
     * Specifies if bi-directional transformations should be disabled
     */
    disableBidirectionalTransformation?: boolean;
    /**
     * The pressure inside the physics imposter, soft object only
     */
    pressure?: number;
    /**
     * The stiffness the physics imposter, soft object only
     */
    stiffness?: number;
    /**
     * The number of iterations used in maintaining consistent vertex velocities, soft object only
     */
    velocityIterations?: number;
    /**
     * The number of iterations used in maintaining consistent vertex positions, soft object only
     */
    positionIterations?: number;
    /**
     * The number used to fix points on a cloth (0, 1, 2, 4, 8) or rope (0, 1, 2) only
     * 0 None, 1, back left or top, 2, back right or bottom, 4, front left, 8, front right
     * Add to fix multiple points
     */
    fixedPoints?: number;
    /**
     * The collision margin around a soft object
     */
    margin?: number;
    /**
     * The collision margin around a soft object
     */
    damping?: number;
    /**
     * The path for a rope based on an extrusion
     */
    path?: any;
    /**
     * The shape of an extrusion used for a rope based on an extrusion
     */
    shape?: any;
}
/**
 * Interface for a physics-enabled object
 * @see https://doc.babylonjs.com/features/featuresDeepDive/physics/usingPhysicsEngine
 */
export interface IPhysicsEnabledObject {
    /**
     * The position of the physics-enabled object
     */
    position: Vector3;
    /**
     * The rotation of the physics-enabled object
     */
    rotationQuaternion: Nullable<Quaternion>;
    /**
     * The scale of the physics-enabled object
     */
    scaling: Vector3;
    /**
     * The rotation of the physics-enabled object
     */
    rotation?: Vector3;
    /**
     * The parent of the physics-enabled object
     */
    parent?: any;
    /**
     * The bounding info of the physics-enabled object
     * @returns The bounding info of the physics-enabled object
     */
    getBoundingInfo(): BoundingInfo;
    /**
     * Computes the world matrix
     * @param force Specifies if the world matrix should be computed by force
     * @returns A world matrix
     */
    computeWorldMatrix(force: boolean): Matrix;
    /**
     * Gets the world matrix
     * @returns A world matrix
     */
    getWorldMatrix?(): Matrix;
    /**
     * Gets the child meshes
     * @param directDescendantsOnly Specifies if only direct-descendants should be obtained
     * @returns An array of abstract meshes
     */
    getChildMeshes?(directDescendantsOnly?: boolean): Array<AbstractMesh>;
    /**
     * Gets the vertex data
     * @param kind The type of vertex data
     * @returns A nullable array of numbers, or a float32 array
     */
    getVerticesData(kind: string): Nullable<Array<number> | Float32Array>;
    /**
     * Gets the indices from the mesh
     * @returns A nullable array of index arrays
     */
    getIndices?(): Nullable<IndicesArray>;
    /**
     * Gets the scene from the mesh
     * @returns the indices array or null
     */
    getScene?(): Scene;
    /**
     * Gets the absolute position from the mesh
     * @returns the absolute position
     */
    getAbsolutePosition(): Vector3;
    /**
     * Gets the absolute pivot point from the mesh
     * @returns the absolute pivot point
     */
    getAbsolutePivotPoint(): Vector3;
    /**
     * Rotates the mesh
     * @param axis The axis of rotation
     * @param amount The amount of rotation
     * @param space The space of the rotation
     * @returns The rotation transform node
     */
    rotate(axis: Vector3, amount: number, space?: Space): TransformNode;
    /**
     * Translates the mesh
     * @param axis The axis of translation
     * @param distance The distance of translation
     * @param space The space of the translation
     * @returns The transform node
     */
    translate(axis: Vector3, distance: number, space?: Space): TransformNode;
    /**
     * Sets the absolute position of the mesh
     * @param absolutePosition The absolute position of the mesh
     * @returns The transform node
     */
    setAbsolutePosition(absolutePosition: Vector3): TransformNode;
    /**
     * Gets the class name of the mesh
     * @returns The class name
     */
    getClassName(): string;
}
/**
 * Represents a physics imposter
 * @see https://doc.babylonjs.com/features/featuresDeepDive/physics/usingPhysicsEngine
 */
export class PhysicsImpostor {
    /**
     * The physics-enabled object used as the physics imposter
     */
    object: IPhysicsEnabledObject;
    /**
     * The type of the physics imposter
     */
    type: number;
    private _options;
    private _scene?;
    /**
     * The default object size of the imposter
     */
    static DEFAULT_OBJECT_SIZE: Vector3;
    /**
     * The identity quaternion of the imposter
     */
    static IDENTITY_QUATERNION: Quaternion;
    /** @internal */
    _pluginData: any;
    private _physicsEngine;
    private _physicsBody;
    private _bodyUpdateRequired;
    private _onBeforePhysicsStepCallbacks;
    private _onAfterPhysicsStepCallbacks;
    /** @internal */
    _onPhysicsCollideCallbacks: Array<{
        callback: (collider: PhysicsImpostor, collidedAgainst: PhysicsImpostor, point: Nullable<Vector3>, distance: number, impulse: number, normal: Nullable<Vector3>) => void;
        otherImpostors: Array<PhysicsImpostor>;
    }>;
    private _deltaPosition;
    private _deltaRotation;
    private _deltaRotationConjugated;
    /** @internal */
    _isFromLine: boolean;
    private _parent;
    private _isDisposed;
    private static _TmpVecs;
    private static _TmpQuat;
    /**
     * Specifies if the physics imposter is disposed
     */
    get isDisposed(): boolean;
    /**
     * Gets the mass of the physics imposter
     */
    get mass(): number;
    set mass(value: number);
    /**
     * Gets the coefficient of friction
     */
    get friction(): number;
    /**
     * Sets the coefficient of friction
     */
    set friction(value: number);
    /**
     * Gets the coefficient of restitution
     */
    get restitution(): number;
    /**
     * Sets the coefficient of restitution
     */
    set restitution(value: number);
    /**
     * Gets the pressure of a soft body; only supported by the AmmoJSPlugin
     */
    get pressure(): number;
    /**
     * Sets the pressure of a soft body; only supported by the AmmoJSPlugin
     */
    set pressure(value: number);
    /**
     * Gets the stiffness of a soft body; only supported by the AmmoJSPlugin
     */
    get stiffness(): number;
    /**
     * Sets the stiffness of a soft body; only supported by the AmmoJSPlugin
     */
    set stiffness(value: number);
    /**
     * Gets the velocityIterations of a soft body; only supported by the AmmoJSPlugin
     */
    get velocityIterations(): number;
    /**
     * Sets the velocityIterations of a soft body; only supported by the AmmoJSPlugin
     */
    set velocityIterations(value: number);
    /**
     * Gets the positionIterations of a soft body; only supported by the AmmoJSPlugin
     */
    get positionIterations(): number;
    /**
     * Sets the positionIterations of a soft body; only supported by the AmmoJSPlugin
     */
    set positionIterations(value: number);
    /**
     * The unique id of the physics imposter
     * set by the physics engine when adding this impostor to the array
     */
    uniqueId: number;
    /**
     * @internal
     */
    soft: boolean;
    /**
     * @internal
     */
    segments: number;
    private _joints;
    /**
     * Initializes the physics imposter
     * @param object The physics-enabled object used as the physics imposter
     * @param type The type of the physics imposter. Types are available as static members of this class.
     * @param _options The options for the physics imposter
     * @param _scene The Babylon scene
     */
    constructor(
    /**
     * The physics-enabled object used as the physics imposter
     */
    object: IPhysicsEnabledObject, 
    /**
     * The type of the physics imposter
     */
    type: number, _options?: PhysicsImpostorParameters, _scene?: Scene | undefined);
    /**
     * This function will completely initialize this impostor.
     * It will create a new body - but only if this mesh has no parent.
     * If it has, this impostor will not be used other than to define the impostor
     * of the child mesh.
     * @internal
     */
    _init(): void;
    private _getPhysicsParent;
    /**
     * Should a new body be generated.
     * @returns boolean specifying if body initialization is required
     */
    isBodyInitRequired(): boolean;
    /**
     * Sets the updated scaling
     */
    setScalingUpdated(): void;
    /**
     * Force a regeneration of this or the parent's impostor's body.
     * Use with caution - This will remove all previously-instantiated joints.
     */
    forceUpdate(): void;
    /**
     * Gets the body that holds this impostor. Either its own, or its parent.
     */
    get physicsBody(): any;
    /**
     * Get the parent of the physics imposter
     * @returns Physics imposter or null
     */
    get parent(): Nullable<PhysicsImpostor>;
    /**
     * Sets the parent of the physics imposter
     */
    set parent(value: Nullable<PhysicsImpostor>);
    /**
     * Set the physics body. Used mainly by the physics engine/plugin
     */
    set physicsBody(physicsBody: any);
    /**
     * Resets the update flags
     */
    resetUpdateFlags(): void;
    /**
     * Gets the object extents
     * @returns the object extents
     */
    getObjectExtents(): Vector3;
    /**
     * Gets the object center
     * @returns The object center
     */
    getObjectCenter(): Vector3;
    /**
     * Get a specific parameter from the options parameters
     * @param paramName The object parameter name
     * @returns The object parameter
     */
    getParam(paramName: string): any;
    /**
     * Sets a specific parameter in the options given to the physics plugin
     * @param paramName The parameter name
     * @param value The value of the parameter
     */
    setParam(paramName: string, value: number): void;
    /**
     * Specifically change the body's mass. Won't recreate the physics body object
     * @param mass The mass of the physics imposter
     */
    setMass(mass: number): void;
    /**
     * Gets the linear velocity
     * @returns  linear velocity or null
     */
    getLinearVelocity(): Nullable<Vector3>;
    /**
     * Sets the linear velocity
     * @param velocity  linear velocity or null
     */
    setLinearVelocity(velocity: Nullable<Vector3>): void;
    /**
     * Gets the angular velocity
     * @returns angular velocity or null
     */
    getAngularVelocity(): Nullable<Vector3>;
    /**
     * Sets the angular velocity
     * @param velocity The velocity or null
     */
    setAngularVelocity(velocity: Nullable<Vector3>): void;
    /**
     * Execute a function with the physics plugin native code
     * Provide a function the will have two variables - the world object and the physics body object
     * @param func The function to execute with the physics plugin native code
     */
    executeNativeFunction(func: (world: any, physicsBody: any) => void): void;
    /**
     * Register a function that will be executed before the physics world is stepping forward
     * @param func The function to execute before the physics world is stepped forward
     */
    registerBeforePhysicsStep(func: (impostor: PhysicsImpostor) => void): void;
    /**
     * Unregister a function that will be executed before the physics world is stepping forward
     * @param func The function to execute before the physics world is stepped forward
     */
    unregisterBeforePhysicsStep(func: (impostor: PhysicsImpostor) => void): void;
    /**
     * Register a function that will be executed after the physics step
     * @param func The function to execute after physics step
     */
    registerAfterPhysicsStep(func: (impostor: PhysicsImpostor) => void): void;
    /**
     * Unregisters a function that will be executed after the physics step
     * @param func The function to execute after physics step
     */
    unregisterAfterPhysicsStep(func: (impostor: PhysicsImpostor) => void): void;
    /**
     * register a function that will be executed when this impostor collides against a different body
     * @param collideAgainst Physics imposter, or array of physics imposters to collide against
     * @param func Callback that is executed on collision
     */
    registerOnPhysicsCollide(collideAgainst: PhysicsImpostor | Array<PhysicsImpostor>, func: (collider: PhysicsImpostor, collidedAgainst: PhysicsImpostor, point: Nullable<Vector3>) => void): void;
    /**
     * Unregisters the physics imposter's collision callback
     * @param collideAgainst The physics object to collide against
     * @param func Callback to execute on collision
     */
    unregisterOnPhysicsCollide(collideAgainst: PhysicsImpostor | Array<PhysicsImpostor>, func: (collider: PhysicsImpostor, collidedAgainst: PhysicsImpostor | Array<PhysicsImpostor>, point: Nullable<Vector3>) => void): void;
    private _tmpQuat;
    private _tmpQuat2;
    /**
     * Get the parent rotation
     * @returns The parent rotation
     */
    getParentsRotation(): Quaternion;
    /**
     * this function is executed by the physics engine.
     */
    beforeStep: () => void;
    /**
     * this function is executed by the physics engine
     */
    afterStep: () => void;
    /**
     * Legacy collision detection event support
     */
    onCollideEvent: Nullable<(collider: PhysicsImpostor, collidedWith: PhysicsImpostor) => void>;
    /**
     *
     * @param e
     * @returns
     */
    onCollide: (e: {
        body: any;
        point: Nullable<Vector3>;
        distance: number;
        impulse: number;
        normal: Nullable<Vector3>;
    }) => void;
    /**
     * Apply a force
     * @param force The force to apply
     * @param contactPoint The contact point for the force
     * @returns The physics imposter
     */
    applyForce(force: Vector3, contactPoint: Vector3): PhysicsImpostor;
    /**
     * Apply an impulse
     * @param force The impulse force
     * @param contactPoint The contact point for the impulse force
     * @returns The physics imposter
     */
    applyImpulse(force: Vector3, contactPoint: Vector3): PhysicsImpostor;
    /**
     * A help function to create a joint
     * @param otherImpostor A physics imposter used to create a joint
     * @param jointType The type of joint
     * @param jointData The data for the joint
     * @returns The physics imposter
     */
    createJoint(otherImpostor: PhysicsImpostor, jointType: number, jointData: PhysicsJointData): PhysicsImpostor;
    /**
     * Add a joint to this impostor with a different impostor
     * @param otherImpostor A physics imposter used to add a joint
     * @param joint The joint to add
     * @returns The physics imposter
     */
    addJoint(otherImpostor: PhysicsImpostor, joint: PhysicsJoint): PhysicsImpostor;
    /**
     * Add an anchor to a cloth impostor
     * @param otherImpostor rigid impostor to anchor to
     * @param width ratio across width from 0 to 1
     * @param height ratio up height from 0 to 1
     * @param influence the elasticity between cloth impostor and anchor from 0, very stretchy to 1, little stretch
     * @param noCollisionBetweenLinkedBodies when true collisions between cloth impostor and anchor are ignored; default false
     * @returns impostor the soft imposter
     */
    addAnchor(otherImpostor: PhysicsImpostor, width: number, height: number, influence: number, noCollisionBetweenLinkedBodies: boolean): PhysicsImpostor;
    /**
     * Add a hook to a rope impostor
     * @param otherImpostor rigid impostor to anchor to
     * @param length ratio across rope from 0 to 1
     * @param influence the elasticity between rope impostor and anchor from 0, very stretchy to 1, little stretch
     * @param noCollisionBetweenLinkedBodies when true collisions between soft impostor and anchor are ignored; default false
     * @returns impostor the rope imposter
     */
    addHook(otherImpostor: PhysicsImpostor, length: number, influence: number, noCollisionBetweenLinkedBodies: boolean): PhysicsImpostor;
    /**
     * Will keep this body still, in a sleep mode.
     * @returns the physics imposter
     */
    sleep(): PhysicsImpostor;
    /**
     * Wake the body up.
     * @returns The physics imposter
     */
    wakeUp(): PhysicsImpostor;
    /**
     * Clones the physics imposter
     * @param newObject The physics imposter clones to this physics-enabled object
     * @returns A nullable physics imposter
     */
    clone(newObject: IPhysicsEnabledObject): Nullable<PhysicsImpostor>;
    /**
     * Disposes the physics imposter
     */
    dispose(): void;
    /**
     * Sets the delta position
     * @param position The delta position amount
     */
    setDeltaPosition(position: Vector3): void;
    /**
     * Sets the delta rotation
     * @param rotation The delta rotation amount
     */
    setDeltaRotation(rotation: Quaternion): void;
    /**
     * Gets the box size of the physics imposter and stores the result in the input parameter
     * @param result Stores the box size
     * @returns The physics imposter
     */
    getBoxSizeToRef(result: Vector3): PhysicsImpostor;
    /**
     * Gets the radius of the physics imposter
     * @returns Radius of the physics imposter
     */
    getRadius(): number;
    /**
     * Sync a bone with this impostor
     * @param bone The bone to sync to the impostor.
     * @param boneMesh The mesh that the bone is influencing.
     * @param jointPivot The pivot of the joint / bone in local space.
     * @param distToJoint Optional distance from the impostor to the joint.
     * @param adjustRotation Optional quaternion for adjusting the local rotation of the bone.
     */
    syncBoneWithImpostor(bone: Bone, boneMesh: AbstractMesh, jointPivot: Vector3, distToJoint?: number, adjustRotation?: Quaternion): void;
    /**
     * Sync impostor to a bone
     * @param bone The bone that the impostor will be synced to.
     * @param boneMesh The mesh that the bone is influencing.
     * @param jointPivot The pivot of the joint / bone in local space.
     * @param distToJoint Optional distance from the impostor to the joint.
     * @param adjustRotation Optional quaternion for adjusting the local rotation of the bone.
     * @param boneAxis Optional vector3 axis the bone is aligned with
     */
    syncImpostorWithBone(bone: Bone, boneMesh: AbstractMesh, jointPivot: Vector3, distToJoint?: number, adjustRotation?: Quaternion, boneAxis?: Vector3): void;
    /**
     * No-Imposter type
     */
    static NoImpostor: number;
    /**
     * Sphere-Imposter type
     */
    static SphereImpostor: number;
    /**
     * Box-Imposter type
     */
    static BoxImpostor: number;
    /**
     * Plane-Imposter type
     */
    static PlaneImpostor: number;
    /**
     * Mesh-imposter type (Only available to objects with vertices data)
     */
    static MeshImpostor: number;
    /**
     * Capsule-Impostor type (Ammo.js plugin only)
     */
    static CapsuleImpostor: number;
    /**
     * Cylinder-Imposter type
     */
    static CylinderImpostor: number;
    /**
     * Particle-Imposter type
     */
    static ParticleImpostor: number;
    /**
     * Heightmap-Imposter type
     */
    static HeightmapImpostor: number;
    /**
     * ConvexHull-Impostor type (Ammo.js plugin only)
     */
    static ConvexHullImpostor: number;
    /**
     * Custom-Imposter type (Ammo.js plugin only)
     */
    static CustomImpostor: number;
    /**
     * Rope-Imposter type
     */
    static RopeImpostor: number;
    /**
     * Cloth-Imposter type
     */
    static ClothImpostor: number;
    /**
     * Softbody-Imposter type
     */
    static SoftbodyImpostor: number;
}

}
declare module "babylonjs/Physics/v1/physicsJoint" {
import { Vector3 } from "babylonjs/Maths/math.vector";
import { IPhysicsEnginePlugin } from "babylonjs/Physics/v1/IPhysicsEnginePlugin";
/**
 * Interface for Physics-Joint data
 * @see https://doc.babylonjs.com/features/featuresDeepDive/physics/usingPhysicsEngine
 */
export interface PhysicsJointData {
    /**
     * The main pivot of the joint
     */
    mainPivot?: Vector3;
    /**
     * The connected pivot of the joint
     */
    connectedPivot?: Vector3;
    /**
     * The main axis of the joint
     */
    mainAxis?: Vector3;
    /**
     * The connected axis of the joint
     */
    connectedAxis?: Vector3;
    /**
     * The collision of the joint
     */
    collision?: boolean;
    /**
     * Native Oimo/Cannon/Energy data
     */
    nativeParams?: any;
}
/**
 * This is a holder class for the physics joint created by the physics plugin
 * It holds a set of functions to control the underlying joint
 * @see https://doc.babylonjs.com/features/featuresDeepDive/physics/usingPhysicsEngine
 */
export class PhysicsJoint {
    /**
     * The type of the physics joint
     */
    type: number;
    /**
     * The data for the physics joint
     */
    jointData: PhysicsJointData;
    private _physicsJoint;
    protected _physicsPlugin: IPhysicsEnginePlugin;
    /**
     * Initializes the physics joint
     * @param type The type of the physics joint
     * @param jointData The data for the physics joint
     */
    constructor(
    /**
     * The type of the physics joint
     */
    type: number, 
    /**
     * The data for the physics joint
     */
    jointData: PhysicsJointData);
    /**
     * Gets the physics joint
     */
    get physicsJoint(): any;
    /**
     * Sets the physics joint
     */
    set physicsJoint(newJoint: any);
    /**
     * Sets the physics plugin
     */
    set physicsPlugin(physicsPlugin: IPhysicsEnginePlugin);
    /**
     * Execute a function that is physics-plugin specific.
     * @param {Function} func the function that will be executed.
     *                        It accepts two parameters: the physics world and the physics joint
     */
    executeNativeFunction(func: (world: any, physicsJoint: any) => void): void;
    /**
     * Distance-Joint type
     */
    static DistanceJoint: number;
    /**
     * Hinge-Joint type
     */
    static HingeJoint: number;
    /**
     * Ball-and-Socket joint type
     */
    static BallAndSocketJoint: number;
    /**
     * Wheel-Joint type
     */
    static WheelJoint: number;
    /**
     * Slider-Joint type
     */
    static SliderJoint: number;
    /**
     * Prismatic-Joint type
     */
    static PrismaticJoint: number;
    /**
     * Universal-Joint type
     * ENERGY FTW! (compare with this - @see http://ode-wiki.org/wiki/index.php?title=Manual:_Joint_Types_and_Functions)
     */
    static UniversalJoint: number;
    /**
     * Hinge-Joint 2 type
     */
    static Hinge2Joint: number;
    /**
     * Point to Point Joint type.  Similar to a Ball-Joint.  Different in parameters
     */
    static PointToPointJoint: number;
    /**
     * Spring-Joint type
     */
    static SpringJoint: number;
    /**
     * Lock-Joint type
     */
    static LockJoint: number;
}
/**
 * A class representing a physics distance joint
 * @see https://doc.babylonjs.com/features/featuresDeepDive/physics/usingPhysicsEngine
 */
export class DistanceJoint extends PhysicsJoint {
    /**
     *
     * @param jointData The data for the Distance-Joint
     */
    constructor(jointData: DistanceJointData);
    /**
     * Update the predefined distance.
     * @param maxDistance The maximum preferred distance
     * @param minDistance The minimum preferred distance
     */
    updateDistance(maxDistance: number, minDistance?: number): void;
}
/**
 * Represents a Motor-Enabled Joint
 * @see https://doc.babylonjs.com/features/featuresDeepDive/physics/usingPhysicsEngine
 */
export class MotorEnabledJoint extends PhysicsJoint implements IMotorEnabledJoint {
    /**
     * Initializes the Motor-Enabled Joint
     * @param type The type of the joint
     * @param jointData The physical joint data for the joint
     */
    constructor(type: number, jointData: PhysicsJointData);
    /**
     * Set the motor values.
     * Attention, this function is plugin specific. Engines won't react 100% the same.
     * @param force the force to apply
     * @param maxForce max force for this motor.
     */
    setMotor(force?: number, maxForce?: number): void;
    /**
     * Set the motor's limits.
     * Attention, this function is plugin specific. Engines won't react 100% the same.
     * @param upperLimit The upper limit of the motor
     * @param lowerLimit The lower limit of the motor
     */
    setLimit(upperLimit: number, lowerLimit?: number): void;
}
/**
 * This class represents a single physics Hinge-Joint
 * @see https://doc.babylonjs.com/features/featuresDeepDive/physics/usingPhysicsEngine
 */
export class HingeJoint extends MotorEnabledJoint {
    /**
     * Initializes the Hinge-Joint
     * @param jointData The joint data for the Hinge-Joint
     */
    constructor(jointData: PhysicsJointData);
    /**
     * Set the motor values.
     * Attention, this function is plugin specific. Engines won't react 100% the same.
     * @param {number} force the force to apply
     * @param {number} maxForce max force for this motor.
     */
    setMotor(force?: number, maxForce?: number): void;
    /**
     * Set the motor's limits.
     * Attention, this function is plugin specific. Engines won't react 100% the same.
     * @param upperLimit The upper limit of the motor
     * @param lowerLimit The lower limit of the motor
     */
    setLimit(upperLimit: number, lowerLimit?: number): void;
}
/**
 * This class represents a dual hinge physics joint (same as wheel joint)
 * @see https://doc.babylonjs.com/features/featuresDeepDive/physics/usingPhysicsEngine
 */
export class Hinge2Joint extends MotorEnabledJoint {
    /**
     * Initializes the Hinge2-Joint
     * @param jointData The joint data for the Hinge2-Joint
     */
    constructor(jointData: PhysicsJointData);
    /**
     * Set the motor values.
     * Attention, this function is plugin specific. Engines won't react 100% the same.
     * @param targetSpeed the speed the motor is to reach
     * @param maxForce max force for this motor.
     * @param motorIndex motor's index, 0 or 1.
     */
    setMotor(targetSpeed?: number, maxForce?: number, motorIndex?: number): void;
    /**
     * Set the motor limits.
     * Attention, this function is plugin specific. Engines won't react 100% the same.
     * @param upperLimit the upper limit
     * @param lowerLimit lower limit
     * @param motorIndex the motor's index, 0 or 1.
     */
    setLimit(upperLimit: number, lowerLimit?: number, motorIndex?: number): void;
}
/**
 * Interface for a motor enabled joint
 * @see https://doc.babylonjs.com/features/featuresDeepDive/physics/usingPhysicsEngine
 */
export interface IMotorEnabledJoint {
    /**
     * Physics joint
     */
    physicsJoint: any;
    /**
     * Sets the motor of the motor-enabled joint
     * @param force The force of the motor
     * @param maxForce The maximum force of the motor
     * @param motorIndex The index of the motor
     */
    setMotor(force?: number, maxForce?: number, motorIndex?: number): void;
    /**
     * Sets the limit of the motor
     * @param upperLimit The upper limit of the motor
     * @param lowerLimit The lower limit of the motor
     * @param motorIndex The index of the motor
     */
    setLimit(upperLimit: number, lowerLimit?: number, motorIndex?: number): void;
}
/**
 * Joint data for a Distance-Joint
 * @see https://doc.babylonjs.com/features/featuresDeepDive/physics/usingPhysicsEngine
 */
export interface DistanceJointData extends PhysicsJointData {
    /**
     * Max distance the 2 joint objects can be apart
     */
    maxDistance: number;
}
/**
 * Joint data from a spring joint
 * @see https://doc.babylonjs.com/features/featuresDeepDive/physics/usingPhysicsEngine
 */
export interface SpringJointData extends PhysicsJointData {
    /**
     * Length of the spring
     */
    length: number;
    /**
     * Stiffness of the spring
     */
    stiffness: number;
    /**
     * Damping of the spring
     */
    damping: number;
    /** this callback will be called when applying the force to the impostors. */
    forceApplicationCallback: () => void;
}

}
declare module "babylonjs/Physics/v1/Plugins/ammoJSPlugin" {
import { Quaternion, Vector3 } from "babylonjs/Maths/math.vector";
import { IPhysicsEnginePlugin, PhysicsImpostorJoint } from "babylonjs/Physics/v1/IPhysicsEnginePlugin";
import { PhysicsImpostor } from "babylonjs/Physics/v1/physicsImpostor";
import { IMotorEnabledJoint } from "babylonjs/Physics/v1/physicsJoint";
import { Nullable } from "babylonjs/types";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { PhysicsRaycastResult } from "babylonjs/Physics/physicsRaycastResult";
/**
 * AmmoJS Physics plugin
 * @see https://doc.babylonjs.com/features/featuresDeepDive/physics/usingPhysicsEngine
 * @see https://github.com/kripken/ammo.js/
 */
export class AmmoJSPlugin implements IPhysicsEnginePlugin {
    private _useDeltaForWorldStep;
    /**
     * Reference to the Ammo library
     */
    bjsAMMO: any;
    /**
     * Created ammoJS world which physics bodies are added to
     */
    world: any;
    /**
     * Name of the plugin
     */
    name: string;
    private _timeStep;
    private _fixedTimeStep;
    private _maxSteps;
    private _tmpQuaternion;
    private _tmpAmmoTransform;
    private _tmpAmmoQuaternion;
    private _tmpAmmoConcreteContactResultCallback;
    private _collisionConfiguration;
    private _dispatcher;
    private _overlappingPairCache;
    private _solver;
    private _softBodySolver;
    private _tmpAmmoVectorA;
    private _tmpAmmoVectorB;
    private _tmpAmmoVectorC;
    private _tmpAmmoVectorD;
    private _tmpContactCallbackResult;
    private _tmpAmmoVectorRCA;
    private _tmpAmmoVectorRCB;
    private _raycastResult;
    private _tmpContactPoint;
    private _tmpContactNormal;
    private _tmpContactDistance;
    private _tmpContactImpulse;
    private _tmpVec3;
    private static readonly _DISABLE_COLLISION_FLAG;
    private static readonly _KINEMATIC_FLAG;
    private static readonly _DISABLE_DEACTIVATION_FLAG;
    /**
     * Initializes the ammoJS plugin
     * @param _useDeltaForWorldStep if the time between frames should be used when calculating physics steps (Default: true)
     * @param ammoInjection can be used to inject your own ammo reference
     * @param overlappingPairCache can be used to specify your own overlapping pair cache
     */
    constructor(_useDeltaForWorldStep?: boolean, ammoInjection?: any, overlappingPairCache?: any);
    /**
     *
     * @returns plugin version
     */
    getPluginVersion(): number;
    /**
     * Sets the gravity of the physics world (m/(s^2))
     * @param gravity Gravity to set
     */
    setGravity(gravity: Vector3): void;
    /**
     * Amount of time to step forward on each frame (only used if useDeltaForWorldStep is false in the constructor)
     * @param timeStep timestep to use in seconds
     */
    setTimeStep(timeStep: number): void;
    /**
     * Increment to step forward in the physics engine (If timeStep is set to 1/60 and fixedTimeStep is set to 1/120 the physics engine should run 2 steps per frame) (Default: 1/60)
     * @param fixedTimeStep fixedTimeStep to use in seconds
     */
    setFixedTimeStep(fixedTimeStep: number): void;
    /**
     * Sets the maximum number of steps by the physics engine per frame (Default: 5)
     * @param maxSteps the maximum number of steps by the physics engine per frame
     */
    setMaxSteps(maxSteps: number): void;
    /**
     * Gets the current timestep (only used if useDeltaForWorldStep is false in the constructor)
     * @returns the current timestep in seconds
     */
    getTimeStep(): number;
    /**
     * The create custom shape handler function to be called when using BABYLON.PhysicsImposter.CustomImpostor
     */
    onCreateCustomShape: (impostor: PhysicsImpostor) => any;
    /**
     * The create custom mesh impostor handler function to support building custom mesh impostor vertex data
     */
    onCreateCustomMeshImpostor: (impostor: PhysicsImpostor) => any;
    /**
     * The create custom convex hull impostor handler function to support building custom convex hull impostor vertex data
     */
    onCreateCustomConvexHullImpostor: (impostor: PhysicsImpostor) => any;
    private _isImpostorInContact;
    private _isImpostorPairInContact;
    private _stepSimulation;
    /**
     * Moves the physics simulation forward delta seconds and updates the given physics imposters
     * Prior to the step the imposters physics location is set to the position of the babylon meshes
     * After the step the babylon meshes are set to the position of the physics imposters
     * @param delta amount of time to step forward
     * @param impostors array of imposters to update before/after the step
     */
    executeStep(delta: number, impostors: Array<PhysicsImpostor>): void;
    /**
     * Update babylon mesh to match physics world object
     * @param impostor imposter to match
     */
    private _afterSoftStep;
    /**
     * Update babylon mesh vertices vertices to match physics world softbody or cloth
     * @param impostor imposter to match
     */
    private _ropeStep;
    /**
     * Update babylon mesh vertices vertices to match physics world softbody or cloth
     * @param impostor imposter to match
     */
    private _softbodyOrClothStep;
    private _tmpMatrix;
    /**
     * Applies an impulse on the imposter
     * @param impostor imposter to apply impulse to
     * @param force amount of force to be applied to the imposter
     * @param contactPoint the location to apply the impulse on the imposter
     */
    applyImpulse(impostor: PhysicsImpostor, force: Vector3, contactPoint: Vector3): void;
    /**
     * Applies a force on the imposter
     * @param impostor imposter to apply force
     * @param force amount of force to be applied to the imposter
     * @param contactPoint the location to apply the force on the imposter
     */
    applyForce(impostor: PhysicsImpostor, force: Vector3, contactPoint: Vector3): void;
    /**
     * Creates a physics body using the plugin
     * @param impostor the imposter to create the physics body on
     */
    generatePhysicsBody(impostor: PhysicsImpostor): void;
    /**
     * Removes the physics body from the imposter and disposes of the body's memory
     * @param impostor imposter to remove the physics body from
     */
    removePhysicsBody(impostor: PhysicsImpostor): void;
    /**
     * Generates a joint
     * @param impostorJoint the imposter joint to create the joint with
     */
    generateJoint(impostorJoint: PhysicsImpostorJoint): void;
    /**
     * Removes a joint
     * @param impostorJoint the imposter joint to remove the joint from
     */
    removeJoint(impostorJoint: PhysicsImpostorJoint): void;
    private _addMeshVerts;
    /**
     * Initialise the soft body vertices to match its object's (mesh) vertices
     * Softbody vertices (nodes) are in world space and to match this
     * The object's position and rotation is set to zero and so its vertices are also then set in world space
     * @param impostor to create the softbody for
     */
    private _softVertexData;
    /**
     * Create an impostor's soft body
     * @param impostor to create the softbody for
     */
    private _createSoftbody;
    /**
     * Create cloth for an impostor
     * @param impostor to create the softbody for
     */
    private _createCloth;
    /**
     * Create rope for an impostor
     * @param impostor to create the softbody for
     */
    private _createRope;
    /**
     * Create a custom physics impostor shape using the plugin's onCreateCustomShape handler
     * @param impostor to create the custom physics shape for
     */
    private _createCustom;
    private _addHullVerts;
    private _createShape;
    /**
     * Sets the mesh body position/rotation from the babylon impostor
     * @param impostor imposter containing the physics body and babylon object
     */
    setTransformationFromPhysicsBody(impostor: PhysicsImpostor): void;
    /**
     * Sets the babylon object's position/rotation from the physics body's position/rotation
     * @param impostor imposter containing the physics body and babylon object
     * @param newPosition new position
     * @param newRotation new rotation
     */
    setPhysicsBodyTransformation(impostor: PhysicsImpostor, newPosition: Vector3, newRotation: Quaternion): void;
    /**
     * If this plugin is supported
     * @returns true if its supported
     */
    isSupported(): boolean;
    /**
     * Sets the linear velocity of the physics body
     * @param impostor imposter to set the velocity on
     * @param velocity velocity to set
     */
    setLinearVelocity(impostor: PhysicsImpostor, velocity: Vector3): void;
    /**
     * Sets the angular velocity of the physics body
     * @param impostor imposter to set the velocity on
     * @param velocity velocity to set
     */
    setAngularVelocity(impostor: PhysicsImpostor, velocity: Vector3): void;
    /**
     * gets the linear velocity
     * @param impostor imposter to get linear velocity from
     * @returns linear velocity
     */
    getLinearVelocity(impostor: PhysicsImpostor): Nullable<Vector3>;
    /**
     * gets the angular velocity
     * @param impostor imposter to get angular velocity from
     * @returns angular velocity
     */
    getAngularVelocity(impostor: PhysicsImpostor): Nullable<Vector3>;
    /**
     * Sets the mass of physics body
     * @param impostor imposter to set the mass on
     * @param mass mass to set
     */
    setBodyMass(impostor: PhysicsImpostor, mass: number): void;
    /**
     * Gets the mass of the physics body
     * @param impostor imposter to get the mass from
     * @returns mass
     */
    getBodyMass(impostor: PhysicsImpostor): number;
    /**
     * Gets friction of the impostor
     * @param impostor impostor to get friction from
     * @returns friction value
     */
    getBodyFriction(impostor: PhysicsImpostor): number;
    /**
     * Sets friction of the impostor
     * @param impostor impostor to set friction on
     * @param friction friction value
     */
    setBodyFriction(impostor: PhysicsImpostor, friction: number): void;
    /**
     * Gets restitution of the impostor
     * @param impostor impostor to get restitution from
     * @returns restitution value
     */
    getBodyRestitution(impostor: PhysicsImpostor): number;
    /**
     * Sets restitution of the impostor
     * @param impostor impostor to set resitution on
     * @param restitution resitution value
     */
    setBodyRestitution(impostor: PhysicsImpostor, restitution: number): void;
    /**
     * Gets pressure inside the impostor
     * @param impostor impostor to get pressure from
     * @returns pressure value
     */
    getBodyPressure(impostor: PhysicsImpostor): number;
    /**
     * Sets pressure inside a soft body impostor
     * Cloth and rope must remain 0 pressure
     * @param impostor impostor to set pressure on
     * @param pressure pressure value
     */
    setBodyPressure(impostor: PhysicsImpostor, pressure: number): void;
    /**
     * Gets stiffness of the impostor
     * @param impostor impostor to get stiffness from
     * @returns pressure value
     */
    getBodyStiffness(impostor: PhysicsImpostor): number;
    /**
     * Sets stiffness of the impostor
     * @param impostor impostor to set stiffness on
     * @param stiffness stiffness value from 0 to 1
     */
    setBodyStiffness(impostor: PhysicsImpostor, stiffness: number): void;
    /**
     * Gets velocityIterations of the impostor
     * @param impostor impostor to get velocity iterations from
     * @returns velocityIterations value
     */
    getBodyVelocityIterations(impostor: PhysicsImpostor): number;
    /**
     * Sets velocityIterations of the impostor
     * @param impostor impostor to set velocity iterations on
     * @param velocityIterations velocityIterations value
     */
    setBodyVelocityIterations(impostor: PhysicsImpostor, velocityIterations: number): void;
    /**
     * Gets positionIterations of the impostor
     * @param impostor impostor to get position iterations from
     * @returns positionIterations value
     */
    getBodyPositionIterations(impostor: PhysicsImpostor): number;
    /**
     * Sets positionIterations of the impostor
     * @param impostor impostor to set position on
     * @param positionIterations positionIterations value
     */
    setBodyPositionIterations(impostor: PhysicsImpostor, positionIterations: number): void;
    /**
     * Append an anchor to a cloth object
     * @param impostor is the cloth impostor to add anchor to
     * @param otherImpostor is the rigid impostor to anchor to
     * @param width ratio across width from 0 to 1
     * @param height ratio up height from 0 to 1
     * @param influence the elasticity between cloth impostor and anchor from 0, very stretchy to 1, little stretch
     * @param noCollisionBetweenLinkedBodies when true collisions between soft impostor and anchor are ignored; default false
     */
    appendAnchor(impostor: PhysicsImpostor, otherImpostor: PhysicsImpostor, width: number, height: number, influence?: number, noCollisionBetweenLinkedBodies?: boolean): void;
    /**
     * Append an hook to a rope object
     * @param impostor is the rope impostor to add hook to
     * @param otherImpostor is the rigid impostor to hook to
     * @param length ratio along the rope from 0 to 1
     * @param influence the elasticity between soft impostor and anchor from 0, very stretchy to 1, little stretch
     * @param noCollisionBetweenLinkedBodies when true collisions between soft impostor and anchor are ignored; default false
     */
    appendHook(impostor: PhysicsImpostor, otherImpostor: PhysicsImpostor, length: number, influence?: number, noCollisionBetweenLinkedBodies?: boolean): void;
    /**
     * Sleeps the physics body and stops it from being active
     * @param impostor impostor to sleep
     */
    sleepBody(impostor: PhysicsImpostor): void;
    /**
     * Activates the physics body
     * @param impostor impostor to activate
     */
    wakeUpBody(impostor: PhysicsImpostor): void;
    /**
     * Updates the distance parameters of the joint
     */
    updateDistanceJoint(): void;
    /**
     * Sets a motor on the joint
     * @param joint joint to set motor on
     * @param speed speed of the motor
     * @param maxForce maximum force of the motor
     */
    setMotor(joint: IMotorEnabledJoint, speed?: number, maxForce?: number): void;
    /**
     * Sets the motors limit
     */
    setLimit(): void;
    /**
     * Syncs the position and rotation of a mesh with the impostor
     * @param mesh mesh to sync
     * @param impostor impostor to update the mesh with
     */
    syncMeshWithImpostor(mesh: AbstractMesh, impostor: PhysicsImpostor): void;
    /**
     * Gets the radius of the impostor
     * @param impostor impostor to get radius from
     * @returns the radius
     */
    getRadius(impostor: PhysicsImpostor): number;
    /**
     * Gets the box size of the impostor
     * @param impostor impostor to get box size from
     * @param result the resulting box size
     */
    getBoxSizeToRef(impostor: PhysicsImpostor, result: Vector3): void;
    /**
     * Disposes of the impostor
     */
    dispose(): void;
    /**
     * Does a raycast in the physics world
     * @param from where should the ray start?
     * @param to where should the ray end?
     * @returns PhysicsRaycastResult
     */
    raycast(from: Vector3, to: Vector3): PhysicsRaycastResult;
    /**
     * Does a raycast in the physics world
     * @param from when should the ray start?
     * @param to when should the ray end?
     * @param result resulting PhysicsRaycastResult
     */
    raycastToRef(from: Vector3, to: Vector3, result: PhysicsRaycastResult): void;
}

}
declare module "babylonjs/Physics/v1/Plugins/cannonJSPlugin" {
import { Nullable } from "babylonjs/types";
import { Vector3, Quaternion } from "babylonjs/Maths/math.vector";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { IPhysicsEnginePlugin, PhysicsImpostorJoint } from "babylonjs/Physics/v1/IPhysicsEnginePlugin";
import { PhysicsImpostor } from "babylonjs/Physics/v1/physicsImpostor";
import { IMotorEnabledJoint } from "babylonjs/Physics/v1/physicsJoint";
import { PhysicsJoint } from "babylonjs/Physics/v1/physicsJoint";
import { PhysicsRaycastResult } from "babylonjs/Physics/physicsRaycastResult";
/** @internal */
export class CannonJSPlugin implements IPhysicsEnginePlugin {
    private _useDeltaForWorldStep;
    world: any;
    name: string;
    private _physicsMaterials;
    private _fixedTimeStep;
    private _cannonRaycastResult;
    private _raycastResult;
    private _physicsBodiesToRemoveAfterStep;
    private _firstFrame;
    private _tmpQuaternion;
    BJSCANNON: any;
    constructor(_useDeltaForWorldStep?: boolean, iterations?: number, cannonInjection?: any);
    /**
     *
     * @returns plugin version
     */
    getPluginVersion(): number;
    setGravity(gravity: Vector3): void;
    setTimeStep(timeStep: number): void;
    getTimeStep(): number;
    executeStep(delta: number, impostors: Array<PhysicsImpostor>): void;
    private _removeMarkedPhysicsBodiesFromWorld;
    applyImpulse(impostor: PhysicsImpostor, force: Vector3, contactPoint: Vector3): void;
    applyForce(impostor: PhysicsImpostor, force: Vector3, contactPoint: Vector3): void;
    generatePhysicsBody(impostor: PhysicsImpostor): void;
    private _processChildMeshes;
    removePhysicsBody(impostor: PhysicsImpostor): void;
    generateJoint(impostorJoint: PhysicsImpostorJoint): void;
    removeJoint(impostorJoint: PhysicsImpostorJoint): void;
    private _addMaterial;
    private _checkWithEpsilon;
    private _createShape;
    private _createHeightmap;
    private _minus90X;
    private _plus90X;
    private _tmpPosition;
    private _tmpDeltaPosition;
    private _tmpUnityRotation;
    private _updatePhysicsBodyTransformation;
    setTransformationFromPhysicsBody(impostor: PhysicsImpostor): void;
    setPhysicsBodyTransformation(impostor: PhysicsImpostor, newPosition: Vector3, newRotation: Quaternion): void;
    isSupported(): boolean;
    setLinearVelocity(impostor: PhysicsImpostor, velocity: Vector3): void;
    setAngularVelocity(impostor: PhysicsImpostor, velocity: Vector3): void;
    getLinearVelocity(impostor: PhysicsImpostor): Nullable<Vector3>;
    getAngularVelocity(impostor: PhysicsImpostor): Nullable<Vector3>;
    setBodyMass(impostor: PhysicsImpostor, mass: number): void;
    getBodyMass(impostor: PhysicsImpostor): number;
    getBodyFriction(impostor: PhysicsImpostor): number;
    setBodyFriction(impostor: PhysicsImpostor, friction: number): void;
    getBodyRestitution(impostor: PhysicsImpostor): number;
    setBodyRestitution(impostor: PhysicsImpostor, restitution: number): void;
    sleepBody(impostor: PhysicsImpostor): void;
    wakeUpBody(impostor: PhysicsImpostor): void;
    updateDistanceJoint(joint: PhysicsJoint, maxDistance: number): void;
    setMotor(joint: IMotorEnabledJoint, speed?: number, maxForce?: number, motorIndex?: number): void;
    setLimit(joint: IMotorEnabledJoint, minForce: number, maxForce?: number): void;
    syncMeshWithImpostor(mesh: AbstractMesh, impostor: PhysicsImpostor): void;
    getRadius(impostor: PhysicsImpostor): number;
    getBoxSizeToRef(impostor: PhysicsImpostor, result: Vector3): void;
    dispose(): void;
    private _extendNamespace;
    /**
     * Does a raycast in the physics world
     * @param from when should the ray start?
     * @param to when should the ray end?
     * @returns PhysicsRaycastResult
     */
    raycast(from: Vector3, to: Vector3): PhysicsRaycastResult;
    /**
     * Does a raycast in the physics world
     * @param from when should the ray start?
     * @param to when should the ray end?
     * @param result resulting PhysicsRaycastResult
     */
    raycastToRef(from: Vector3, to: Vector3, result: PhysicsRaycastResult): void;
}

}
declare module "babylonjs/Physics/v1/Plugins/index" {
export * from "babylonjs/Physics/v1/Plugins/cannonJSPlugin";
export * from "babylonjs/Physics/v1/Plugins/ammoJSPlugin";
export * from "babylonjs/Physics/v1/Plugins/oimoJSPlugin";

}
declare module "babylonjs/Physics/v1/Plugins/oimoJSPlugin" {
import { PhysicsImpostor } from "babylonjs/Physics/v1/physicsImpostor";
import { IMotorEnabledJoint } from "babylonjs/Physics/v1/physicsJoint";
import { PhysicsJoint } from "babylonjs/Physics/v1/physicsJoint";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { Vector3, Quaternion } from "babylonjs/Maths/math.vector";
import { Nullable } from "babylonjs/types";
import { PhysicsRaycastResult } from "babylonjs/Physics/physicsRaycastResult";
import { IPhysicsEnginePlugin, PhysicsImpostorJoint } from "babylonjs/Physics/v1/IPhysicsEnginePlugin";
/** @internal */
export class OimoJSPlugin implements IPhysicsEnginePlugin {
    private _useDeltaForWorldStep;
    world: any;
    name: string;
    BJSOIMO: any;
    private _raycastResult;
    private _fixedTimeStep;
    constructor(_useDeltaForWorldStep?: boolean, iterations?: number, oimoInjection?: any);
    /**
     *
     * @returns plugin version
     */
    getPluginVersion(): number;
    setGravity(gravity: Vector3): void;
    setTimeStep(timeStep: number): void;
    getTimeStep(): number;
    private _tmpImpostorsArray;
    executeStep(delta: number, impostors: Array<PhysicsImpostor>): void;
    applyImpulse(impostor: PhysicsImpostor, force: Vector3, contactPoint: Vector3): void;
    applyForce(impostor: PhysicsImpostor, force: Vector3, contactPoint: Vector3): void;
    generatePhysicsBody(impostor: PhysicsImpostor): void;
    private _tmpPositionVector;
    removePhysicsBody(impostor: PhysicsImpostor): void;
    generateJoint(impostorJoint: PhysicsImpostorJoint): void;
    removeJoint(impostorJoint: PhysicsImpostorJoint): void;
    isSupported(): boolean;
    setTransformationFromPhysicsBody(impostor: PhysicsImpostor): void;
    setPhysicsBodyTransformation(impostor: PhysicsImpostor, newPosition: Vector3, newRotation: Quaternion): void;
    setLinearVelocity(impostor: PhysicsImpostor, velocity: Vector3): void;
    setAngularVelocity(impostor: PhysicsImpostor, velocity: Vector3): void;
    getLinearVelocity(impostor: PhysicsImpostor): Nullable<Vector3>;
    getAngularVelocity(impostor: PhysicsImpostor): Nullable<Vector3>;
    setBodyMass(impostor: PhysicsImpostor, mass: number): void;
    getBodyMass(impostor: PhysicsImpostor): number;
    getBodyFriction(impostor: PhysicsImpostor): number;
    setBodyFriction(impostor: PhysicsImpostor, friction: number): void;
    getBodyRestitution(impostor: PhysicsImpostor): number;
    setBodyRestitution(impostor: PhysicsImpostor, restitution: number): void;
    sleepBody(impostor: PhysicsImpostor): void;
    wakeUpBody(impostor: PhysicsImpostor): void;
    updateDistanceJoint(joint: PhysicsJoint, maxDistance: number, minDistance?: number): void;
    setMotor(joint: IMotorEnabledJoint, speed: number, force?: number, motorIndex?: number): void;
    setLimit(joint: IMotorEnabledJoint, upperLimit: number, lowerLimit?: number, motorIndex?: number): void;
    syncMeshWithImpostor(mesh: AbstractMesh, impostor: PhysicsImpostor): void;
    getRadius(impostor: PhysicsImpostor): number;
    getBoxSizeToRef(impostor: PhysicsImpostor, result: Vector3): void;
    dispose(): void;
    /**
     * Does a raycast in the physics world
     * @param from when should the ray start?
     * @param to when should the ray end?
     * @returns PhysicsRaycastResult
     */
    raycast(from: Vector3, to: Vector3): PhysicsRaycastResult;
    /**
     * Does a raycast in the physics world
     * @param from when should the ray start?
     * @param to when should the ray end?
     * @param result resulting PhysicsRaycastResult
     */
    raycastToRef(from: Vector3, to: Vector3, result: PhysicsRaycastResult): void;
}

}
declare module "babylonjs/Physics/v2/index" {
export { PhysicsEngine as PhysicsEngineV2 } from "babylonjs/Physics/v2/physicsEngine";
export * from "babylonjs/Physics/v2/physicsBody";
export * from "babylonjs/Physics/v2/physicsShape";
export * from "babylonjs/Physics/v2/physicsConstraint";
export * from "babylonjs/Physics/v2/physicsMaterial";
export * from "babylonjs/Physics/v2/physicsAggregate";
export * from "babylonjs/Physics/v2/IPhysicsEnginePlugin";

}
declare module "babylonjs/Physics/v2/IPhysicsEnginePlugin" {
import { Vector3, Quaternion } from "babylonjs/Maths/math.vector";
import { PhysicsRaycastResult } from "babylonjs/Physics/physicsRaycastResult";
import { PhysicsBody } from "babylonjs/Physics/v2/physicsBody";
import { PhysicsShape } from "babylonjs/Physics/v2/physicsShape";
import { PhysicsConstraint } from "babylonjs/Physics/v2/physicsConstraint";
import { BoundingBox } from "babylonjs/Culling/boundingBox";
import { TransformNode } from "babylonjs/Meshes/transformNode";
import { PhysicsMaterial } from "babylonjs/Physics/v2/physicsMaterial";
import { Mesh } from "babylonjs/Meshes/mesh";
import { Nullable } from "babylonjs/types";
/** @internal */
export enum ConstraintAxisLimitMode {
    FREE = 0,
    LIMITED = 1,
    LOCKED = 2,
    NONE = 3
}
/** @internal */
export enum ConstraintAxis {
    LINEAR_X = 0,
    LINEAR_Y = 1,
    LINEAR_Z = 2,
    ANGULAR_X = 3,
    ANGULAR_Y = 4,
    ANGULAR_Z = 5,
    LINEAR_DISTANCE = 6
}
/** @internal */
export enum ConstraintType {
    BALL_AND_SOCKET = 0,
    DISTANCE = 1,
    HINGE = 2,
    SLIDER = 3,
    LOCK = 4
}
/** @internal */
export enum ShapeType {
    SPHERE = 0,
    CAPSULE = 1,
    CYLINDER = 2,
    BOX = 3,
    CONVEX_HULL = 4,
    CONTAINER = 5,
    MESH = 6,
    HEIGHTFIELD = 7
}
/** @internal */
export enum ConstraintMotorType {
    NONE = 0,
    VELOCITY = 1,
    POSITION = 2
}
/** @internal */
export interface PhysicsShapeParameters {
    center?: Vector3;
    radius?: number;
    pointA?: Vector3;
    pointB?: Vector3;
    rotation?: Quaternion;
    extents?: Vector3;
    mesh?: Mesh;
    includeChildMeshes?: boolean;
}
/** @internal */
export interface PhysicsConstraintParameters {
    pivotA?: Vector3;
    pivotB?: Vector3;
    axisA?: Vector3;
    axisB?: Vector3;
}
/**
 *
 */
/** @internal */
export interface MassProperties {
    /**
     *
     */
    centerOfMass: Vector3;
    /**
     *
     */
    mass: number;
    /**
     *
     */
    inertia: Vector3;
    /**
     *
     */
    inertiaOrientation: Quaternion;
}
/** @internal */
export interface IPhysicsEnginePluginV2 {
    /**
     *
     */
    world: any;
    /**
     *
     */
    name: string;
    setGravity(gravity: Vector3): void;
    setTimeStep(timeStep: number): void;
    getTimeStep(): number;
    executeStep(delta: number, bodies: Array<PhysicsBody>): void;
    getPluginVersion(): number;
    registerOnCollide(func: (collider: PhysicsBody, collidedAgainst: PhysicsBody, point: Nullable<Vector3>) => void): void;
    unregisterOnCollide(func: (collider: PhysicsBody, collidedAgainst: PhysicsBody, point: Nullable<Vector3>) => void): void;
    initBody(body: PhysicsBody, position: Vector3, orientation: Quaternion): void;
    initBodyInstances(body: PhysicsBody, mesh: Mesh): void;
    removeBody(body: PhysicsBody): void;
    sync(body: PhysicsBody): void;
    syncTransform(body: PhysicsBody, transformNode: TransformNode): void;
    setShape(body: PhysicsBody, shape: PhysicsShape): void;
    getShape(body: PhysicsBody): PhysicsShape;
    setFilterGroup(body: PhysicsBody, group: number): void;
    getFilterGroup(body: PhysicsBody): number;
    setEventMask(body: PhysicsBody, eventMask: number): void;
    getEventMask(body: PhysicsBody): number;
    setMassProperties(body: PhysicsBody, massProps: MassProperties): void;
    getMassProperties(body: PhysicsBody): MassProperties;
    setLinearDamping(body: PhysicsBody, damping: number): void;
    getLinearDamping(body: PhysicsBody): number;
    setAngularDamping(body: PhysicsBody, damping: number): void;
    getAngularDamping(body: PhysicsBody): number;
    setLinearVelocity(body: PhysicsBody, linVel: Vector3): void;
    getLinearVelocityToRef(body: PhysicsBody, linVel: Vector3): void;
    applyImpulse(body: PhysicsBody, location: Vector3, impulse: Vector3): void;
    applyForce(body: PhysicsBody, location: Vector3, force: Vector3): void;
    setAngularVelocity(body: PhysicsBody, angVel: Vector3): void;
    getAngularVelocityToRef(body: PhysicsBody, angVel: Vector3): void;
    getBodyGeometry(body: PhysicsBody): {};
    disposeBody(body: PhysicsBody): void;
    registerOnBodyCollide(body: PhysicsBody, func: (collider: PhysicsBody, collidedAgainst: PhysicsBody, point: Nullable<Vector3>) => void): void;
    unregisterOnBodyCollide(body: PhysicsBody, func: (collider: PhysicsBody, collidedAgainst: PhysicsBody, point: Nullable<Vector3>) => void): void;
    initShape(shape: PhysicsShape, type: ShapeType, options: PhysicsShapeParameters): void;
    setFilterLayer(shape: PhysicsShape, layer: number): void;
    getFilterLayer(shape: PhysicsShape): number;
    setMaterial(shape: PhysicsShape, material: PhysicsMaterial): void;
    getMaterial(shape: PhysicsShape): PhysicsMaterial;
    setDensity(shape: PhysicsShape, density: number): void;
    getDensity(shape: PhysicsShape): number;
    addChild(shape: PhysicsShape, newChild: PhysicsShape, childTransform: TransformNode): void;
    removeChild(shape: PhysicsShape, childIndex: number): void;
    getNumChildren(shape: PhysicsShape): number;
    getBoundingBox(shape: PhysicsShape): BoundingBox;
    disposeShape(shape: PhysicsShape): void;
    initMaterial(material: PhysicsMaterial): void;
    setFriction(material: PhysicsMaterial, friction: number): void;
    getFriction(material: PhysicsMaterial): number;
    setRestitution(material: PhysicsMaterial, restitution: number): void;
    getRestitution(material: PhysicsMaterial): number;
    disposeMaterial(material: PhysicsMaterial): void;
    initConstraint(constraint: PhysicsConstraint, type: ConstraintType, options: PhysicsConstraintParameters): void;
    setParentBody(constraint: PhysicsConstraint, body: PhysicsBody): void;
    getParentBody(constraint: PhysicsConstraint): PhysicsBody;
    setChildBody(constraint: PhysicsConstraint, body: PhysicsBody): void;
    getChildBody(constraint: PhysicsConstraint): PhysicsBody;
    setAnchorInParent(constraint: PhysicsConstraint, pivot: Vector3, axisX: Vector3, axisY: Vector3): void;
    setAnchorInChild(constraint: PhysicsConstraint, pivot: Vector3, axisX: Vector3, axisY: Vector3): void;
    setEnabled(constraint: PhysicsConstraint, isEnabled: boolean): void;
    getEnabled(constraint: PhysicsConstraint): boolean;
    setCollisionsEnabled(constraint: PhysicsConstraint, isEnabled: boolean): void;
    getCollisionsEnabled(constraint: PhysicsConstraint): boolean;
    setAxisFriction(constraint: PhysicsConstraint, axis: ConstraintAxis, friction: number): void;
    getAxisFriction(constraint: PhysicsConstraint, axis: ConstraintAxis): number;
    setAxisMode(constraint: PhysicsConstraint, axis: ConstraintAxis, limitMode: ConstraintAxisLimitMode): void;
    getAxisMode(constraint: PhysicsConstraint, axis: ConstraintAxis): ConstraintAxisLimitMode;
    setAxisMinLimit(constraint: PhysicsConstraint, axis: ConstraintAxis, minLimit: number): void;
    getAxisMinLimit(constraint: PhysicsConstraint, axis: ConstraintAxis): number;
    setAxisMaxLimit(constraint: PhysicsConstraint, axis: ConstraintAxis, limit: number): void;
    getAxisMaxLimit(constraint: PhysicsConstraint, axis: ConstraintAxis): number;
    setAxisMotorType(constraint: PhysicsConstraint, axis: ConstraintAxis, motorType: ConstraintMotorType): void;
    getAxisMotorType(constraint: PhysicsConstraint, axis: ConstraintAxis): ConstraintMotorType;
    setAxisMotorTarget(constraint: PhysicsConstraint, axis: ConstraintAxis, target: number): void;
    getAxisMotorTarget(constraint: PhysicsConstraint, axis: ConstraintAxis): number;
    setAxisMotorMaxForce(constraint: PhysicsConstraint, axis: ConstraintAxis, maxForce: number): void;
    getAxisMotorMaxForce(constraint: PhysicsConstraint, axis: ConstraintAxis): number;
    disposeConstraint(constraint: PhysicsConstraint): void;
    raycast(from: Vector3, to: Vector3, result: PhysicsRaycastResult): void;
    dispose(): void;
}

}
declare module "babylonjs/Physics/v2/physicsAggregate" {
import { PhysicsBody } from "babylonjs/Physics/v2/physicsBody";
import { PhysicsMaterial } from "babylonjs/Physics/v2/physicsMaterial";
import { PhysicsShape } from "babylonjs/Physics/v2/physicsShape";
import { Scene } from "babylonjs/scene";
import { TransformNode } from "babylonjs/Meshes/transformNode";
import { Vector3 } from "babylonjs/Maths/math.vector";
/**
 * The interface for the physics aggregate parameters
 */
export interface PhysicsAggregateParameters {
    /**
     * The mass of the physics aggregate
     */
    mass: number;
    /**
     * The friction of the physics aggregate
     */
    friction?: number;
    /**
     * The coefficient of restitution of the physics aggregate
     */
    restitution?: number;
    /**
     * The native options of the physics aggregate
     */
    nativeOptions?: any;
    /**
     * Specifies if the parent should be ignored
     */
    ignoreParent?: boolean;
    /**
     * Specifies if bi-directional transformations should be disabled
     */
    disableBidirectionalTransformation?: boolean;
    /**
     * The pressure inside the physics aggregate, soft object only
     */
    pressure?: number;
    /**
     * The stiffness the physics aggregate, soft object only
     */
    stiffness?: number;
    /**
     * The number of iterations used in maintaining consistent vertex velocities, soft object only
     */
    velocityIterations?: number;
    /**
     * The number of iterations used in maintaining consistent vertex positions, soft object only
     */
    positionIterations?: number;
    /**
     * The number used to fix points on a cloth (0, 1, 2, 4, 8) or rope (0, 1, 2) only
     * 0 None, 1, back left or top, 2, back right or bottom, 4, front left, 8, front right
     * Add to fix multiple points
     */
    fixedPoints?: number;
    /**
     * The collision margin around a soft object
     */
    margin?: number;
    /**
     * The collision margin around a soft object
     */
    damping?: number;
    /**
     * The path for a rope based on an extrusion
     */
    path?: any;
    /**
     * The shape of an extrusion used for a rope based on an extrusion
     */
    shape?: any;
    /**
     * Radius for sphere, cylinder and capsule
     */
    radius?: number;
    /**
     * Starting point for cylinder/capsule
     */
    pointA?: Vector3;
    /**
     * Ending point for cylinder/capsule
     */
    pointB?: Vector3;
    /**
     * Extents for box
     */
    extents?: Vector3;
}
/**
 * Helper class to create and interact with a PhysicsAggregate.
 * This is a transition object that works like Physics Plugin V1 Impostors.
 * This helper instanciate all mandatory physics objects to get a body/shape and material.
 * It's less efficient that handling body and shapes independently but for prototyping or
 * a small numbers of physics objects, it's good enough.
 */
export class PhysicsAggregate {
    /**
     * The physics-enabled object used as the physics aggregate
     */
    transformNode: TransformNode;
    /**
     * The type of the physics aggregate
     */
    type: number;
    private _options;
    private _scene?;
    /**
     * The body that is associated with this aggregate
     */
    body: PhysicsBody;
    /**
     * The shape that is associated with this aggregate
     */
    shape: PhysicsShape;
    /**
     * The material that is associated with this aggregate
     */
    material: PhysicsMaterial;
    constructor(
    /**
     * The physics-enabled object used as the physics aggregate
     */
    transformNode: TransformNode, 
    /**
     * The type of the physics aggregate
     */
    type: number, _options?: PhysicsAggregateParameters, _scene?: Scene | undefined);
    private _addSizeOptions;
    /**
     * Releases the body, shape and material
     */
    dispose(): void;
}

}
declare module "babylonjs/Physics/v2/physicsBody" {
import { MassProperties } from "babylonjs/Physics/v2/IPhysicsEnginePlugin";
import { PhysicsShape } from "babylonjs/Physics/v2/physicsShape";
import { Vector3 } from "babylonjs/Maths/math.vector";
import { Scene } from "babylonjs/scene";
import { TransformNode } from "babylonjs/Meshes";
import { Nullable } from "babylonjs/types";
/**
 * PhysicsBody is useful for creating a physics body that can be used in a physics engine. It allows
 * the user to set the mass and velocity of the body, which can then be used to calculate the
 * motion of the body in the physics engine.
 */
export class PhysicsBody {
    /**
     * V2 Physics plugin private data for single Transform
     */
    _pluginData: any;
    /**
     * V2 Physics plugin private data for instances
     */
    _pluginDataInstances: Array<any>;
    /**
     * The V2 plugin used to create and manage this Physics Body
     */
    private _physicsPlugin;
    /**
     * The engine used to create and manage this Physics Body
     */
    private _physicsEngine;
    /**
     * The transform node associated with this Physics Body
     */
    transformNode: TransformNode;
    /**
     * Disable pre-step that consists in updating Physics Body from Transform Node Translation/Orientation.
     * True by default for maximum performance.
     */
    disablePreStep: boolean;
    private static _DEFAULT_OBJECT_SIZE;
    private static _IDENTITY_QUATERNION;
    /**
     * Constructs a new physics body for the given node.
     * @param transformNode - The Transform Node to construct the physics body for.
     * @param scene - The scene containing the physics engine.
     *
     * This code is useful for creating a physics body for a given Transform Node in a scene.
     * It checks the version of the physics engine and the physics plugin, and initializes the body accordingly.
     * It also sets the node's rotation quaternion if it is not already set. Finally, it adds the body to the physics engine.
     */
    constructor(transformNode: TransformNode, scene: Scene);
    /**
     * Sets the shape of the physics body.
     * @param shape - The shape of the physics body.
     *
     * This method is useful for setting the shape of the physics body, which is necessary for the physics engine to accurately simulate the body's behavior.
     * The shape is used to calculate the body's mass, inertia, and other properties.
     */
    setShape(shape: PhysicsShape): void;
    /**
     * Retrieves the physics shape associated with this object.
     *
     * @returns The physics shape associated with this object, or `undefined` if no
     * shape is associated.
     *
     * This method is useful for retrieving the physics shape associated with this object,
     * which can be used to apply physical forces to the object or to detect collisions.
     */
    getShape(): PhysicsShape | undefined;
    /**
     * Sets the filter group of the physics body.
     * @param group - The filter group of the physics body.
     *
     * This method is useful for setting the filter group of the physics body.
     * The filter group is used to determine which bodies should collide with each other.
     * This allows for more control over the physics engine and can be used to create more realistic simulations.
     */
    setFilterGroup(group: number): void;
    /**
     * Gets the filter group of the physics engine.
     *
     * @returns The filter group of the physics engine.
     *
     * This method is useful for getting the filter group of the physics engine,
     * which is used to determine which objects will interact with each other.
     * This is important for creating realistic physics simulations.
     */
    getFilterGroup(): number;
    /**
     * Sets the event mask for the physics engine.
     *
     * @param eventMask - A bitmask that determines which events will be sent to the physics engine.
     *
     * This method is useful for setting the event mask for the physics engine, which determines which events
     * will be sent to the physics engine. This allows the user to control which events the physics engine will respond to.
     */
    setEventMask(eventMask: number): void;
    /**
     * Gets the event mask of the physics engine.
     *
     * @returns The event mask of the physics engine.
     *
     * This method is useful for getting the event mask of the physics engine,
     * which is used to determine which events the engine will respond to.
     * This is important for ensuring that the engine is responding to the correct events and not
     * wasting resources on unnecessary events.
     */
    getEventMask(): number;
    /**
     * Sets the mass properties of the physics object.
     *
     * @param massProps - The mass properties to set.
     *
     * This method is useful for setting the mass properties of a physics object, such as its mass,
     * inertia, and center of mass. This is important for accurately simulating the physics of the object in the physics engine.
     */
    setMassProperties(massProps: MassProperties): void;
    /**
     * Retrieves the mass properties of the object.
     *
     * @returns The mass properties of the object, or `undefined` if the physics
     * plugin does not support mass properties.
     *
     * This method is useful for physics simulations, as it allows the user to
     * retrieve the mass properties of the object, such as its mass, center of mass,
     * and moment of inertia. This information is necessary for accurate physics
     * simulations.
     */
    getMassProperties(): MassProperties | undefined;
    /**
     * Sets the linear damping of the physics body.
     *
     * @param damping - The linear damping value.
     *
     * This method is useful for controlling the linear damping of the physics body,
     * which is the rate at which the body's velocity decreases over time. This is useful for simulating
     * the effects of air resistance or other forms of friction.
     */
    setLinearDamping(damping: number): void;
    /**
     * Gets the linear damping of the physics body.
     * @returns The linear damping of the physics body.
     *
     * This method is useful for retrieving the linear damping of the physics body, which is the amount of
     * resistance the body has to linear motion. This is useful for simulating realistic physics behavior
     * in a game.
     */
    getLinearDamping(): number;
    /**
     * Sets the angular damping of the physics body.
     * @param damping The angular damping of the body.
     *
     * This method is useful for controlling the angular velocity of a physics body.
     * By setting the damping, the body's angular velocity will be reduced over time, simulating the effect of friction.
     * This can be used to create realistic physical behavior in a physics engine.
     */
    setAngularDamping(damping: number): void;
    /**
     * Gets the angular damping of the physics body.
     *
     * @returns The angular damping of the physics body.
     *
     * This method is useful for getting the angular damping of the physics body,
     * which is the rate of reduction of the angular velocity over time.
     * This is important for simulating realistic physics behavior in a game.
     */
    getAngularDamping(): number;
    /**
     * Sets the linear velocity of the physics object.
     * @param linVel - The linear velocity to set.
     *
     * This method is useful for setting the linear velocity of a physics object,
     * which is necessary for simulating realistic physics in a game engine.
     * By setting the linear velocity, the physics object will move in the direction and speed specified by the vector.
     * This allows for realistic physics simulations, such as simulating the motion of a ball rolling down a hill.
     */
    setLinearVelocity(linVel: Vector3): void;
    /**
     * Gets the linear velocity of the physics body and stores it in the given vector3.
     * @param linVel - The vector3 to store the linear velocity in.
     *
     * This method is useful for getting the linear velocity of a physics body in a physics engine.
     * This can be used to determine the speed and direction of the body, which can be used to calculate the motion of the body.*/
    getLinearVelocityToRef(linVel: Vector3): void;
    /**
     * Sets the angular velocity of the physics object.
     * @param angVel - The angular velocity to set.
     *
     * This method is useful for setting the angular velocity of a physics object, which is necessary for
     * simulating realistic physics behavior. The angular velocity is used to determine the rate of rotation of the object,
     * which is important for simulating realistic motion.
     */
    setAngularVelocity(angVel: Vector3): void;
    /**
     * Gets the angular velocity of the physics body and stores it in the given vector3.
     * @param angVel - The vector3 to store the angular velocity in.
     *
     * This method is useful for getting the angular velocity of a physics body, which can be used to determine the body's
     * rotational speed. This information can be used to create realistic physics simulations.
     */
    getAngularVelocityToRef(angVel: Vector3): void;
    /**
     * Applies an impulse to the physics object.
     *
     * @param location The location of the impulse.
     * @param impulse The impulse vector.
     *
     * This method is useful for applying an impulse to a physics object, which can be used to simulate physical forces such as gravity,
     * collisions, and explosions. This can be used to create realistic physics simulations in a game or other application.
     */
    applyImpulse(location: Vector3, impulse: Vector3): void;
    /**
     * Applies a force to the physics object.
     *
     * @param location The location of the force.
     * @param force The force vector.
     *
     * This method is useful for applying a force to a physics object, which can be used to simulate physical forces such as gravity,
     * collisions, and explosions. This can be used to create realistic physics simulations in a game or other application.
     */
    applyForce(location: Vector3, force: Vector3): void;
    /**
     * Retrieves the geometry of the body from the physics plugin.
     *
     * @returns The geometry of the body.
     *
     * This method is useful for retrieving the geometry of the body from the physics plugin, which can be used for various physics calculations.
     */
    getGeometry(): {};
    /**
     * Register a collision callback that is called when the body collides
     * Filtering by body is inefficient. It's more preferable to register a collision callback for the entire world
     * and do the filtering on the user side.
     */
    registerOnCollide(func: (collider: PhysicsBody, collidedAgainst: PhysicsBody, point: Nullable<Vector3>) => void): void;
    /**
     * Unregister a collision callback that is called when the body collides
     */
    unregisterOnCollide(func: (collider: PhysicsBody, collidedAgainst: PhysicsBody, point: Nullable<Vector3>) => void): void;
    /**
     * Gets the object extents
     * @returns the object extents
     */
    getObjectExtents(): Vector3;
    /**
     * return geometric center of the associated mesh
     */
    getObjectCenter(): Vector3;
    /**
     * Disposes the body from the physics engine.
     *
     * This method is useful for cleaning up the physics engine when a body is no longer needed. Disposing the body will free up resources and prevent memory leaks.
     */
    dispose(): void;
}

}
declare module "babylonjs/Physics/v2/physicsConstraint" {
import { Scene } from "babylonjs/scene";
import { Vector3 } from "babylonjs/Maths/math.vector";
import { IPhysicsEnginePluginV2, ConstraintAxis, PhysicsConstraintParameters, ConstraintAxisLimitMode, ConstraintMotorType } from "babylonjs/Physics/v2/IPhysicsEnginePlugin";
import { ConstraintType } from "babylonjs/Physics/v2/IPhysicsEnginePlugin";
import { PhysicsBody } from "babylonjs/Physics/v2/physicsBody";
/**
 * This is a holder class for the physics constraint created by the physics plugin
 * It holds a set of functions to control the underlying constraint
 * @see https://doc.babylonjs.com/features/featuresDeepDive/physics/usingPhysicsEngine
 */
export class PhysicsConstraint {
    /**
     * V2 Physics plugin private data for a physics material
     */
    _pluginData: any;
    /**
     * The V2 plugin used to create and manage this Physics Body
     */
    protected _physicsPlugin: IPhysicsEnginePluginV2;
    /**
     * Constructs a new constraint for the physics constraint.
     * @param type The type of constraint to create.
     * @param options The options for the constraint.
     * @param scene The scene the constraint belongs to.
     *
     * This code is useful for creating a new constraint for the physics engine. It checks if the scene has a physics engine, and if the plugin version is correct.
     * If all checks pass, it initializes the constraint with the given type and options.
     */
    constructor(type: ConstraintType, options: PhysicsConstraintParameters, scene: Scene);
    /**
     * Sets the parent body of this body.
     * @param body - The parent body to set.
     *
     * This method is useful for setting the parent body of a physics body in a physics engine.
     * This allows the engine to accurately simulate the motion of the body in relation to its parent body.
     * For example, if the parent body is a planet, the engine can accurately simulate the motion of the body in relation to the planet's gravity.
     */
    setParentBody(body: PhysicsBody): void;
    /**
     * Retrieves the parent body of the current physics constraint.
     *
     * @returns The parent body of the current physics constraint, or `undefined` if the
     * current constraint does not have a parent body.
     */
    getParentBody(): PhysicsBody | undefined;
    /**
     * Sets the child body of the current body.
     *
     * @param body - The child body to set.
     *
     * This method is useful for setting the child body of the current body in a physics engine.
     * This allows for the creation of complex structures of bodies that interact with each other in a realistic way.
     */
    setChildBody(body: PhysicsBody): void;
    /**
     * Retrieves the child body of the current physics constraint.
     *
     * @returns The child body of the current physics constraint, or `undefined` if no
     * child body is present.
     */
    getChildBody(): PhysicsBody | undefined;
    /**
     *
     * @param pivot +
     * @param axisX
     * @param axisY
     */
    setAnchorInParent(pivot: Vector3, axisX: Vector3, axisY: Vector3): void;
    /**
     *
     * @param pivot
     * @param axisX
     * @param axisY
     */
    setAnchorInChild(pivot: Vector3, axisX: Vector3, axisY: Vector3): void;
    /**
     *
     * @param isEnabled
     */
    setEnabled(isEnabled: boolean): void;
    /**
     *
     * @returns
     */
    getEnabled(): boolean;
    /**
     *
     * @param isEnabled
     */
    setCollisionsEnabled(isEnabled: boolean): void;
    /**
     *
     * @returns
     */
    getCollisionsEnabled(): boolean;
    /**
     *
     * @param axis
     * @param friction
     */
    setAxisFriction(axis: ConstraintAxis, friction: number): void;
    /**
     *
     * @param axis
     * @returns
     */
    getAxisFriction(axis: ConstraintAxis): number;
    /**
     *
     * @param axis
     * @param limitMode
     */
    setAxisMode(axis: ConstraintAxis, limitMode: ConstraintAxisLimitMode): void;
    /**
     *
     * @param axis
     */
    getAxisMode(axis: ConstraintAxis): ConstraintAxisLimitMode;
    /**
     *
     */
    setAxisMinLimit(axis: ConstraintAxis, minLimit: number): void;
    /**
     *
     */
    getAxisMinLimit(axis: ConstraintAxis): number;
    /**
     *
     */
    setAxisMaxLimit(axis: ConstraintAxis, limit: number): void;
    /**
     *
     */
    getAxisMaxLimit(axis: ConstraintAxis): number;
    /**
     *
     */
    setAxisMotorType(axis: ConstraintAxis, motorType: ConstraintMotorType): void;
    /**
     *
     */
    getAxisMotorType(axis: ConstraintAxis): ConstraintMotorType;
    /**
     *
     */
    setAxisMotorTarget(axis: ConstraintAxis, target: number): void;
    /**
     *
     */
    getAxisMotorTarget(axis: ConstraintAxis): number;
    /**
     *
     */
    setAxisMotorMaxForce(axis: ConstraintAxis, maxForce: number): void;
    /**
     *
     */
    getAxisMotorMaxForce(axis: ConstraintAxis): number;
    /**
     *
     */
    dispose(): void;
}
/**
 *
 */
/** @internal */
export class PhysicsConstraintBallAndSocket extends PhysicsConstraint {
    /** @internal */
    constructor(pivotA: Vector3, pivotB: Vector3, axisA: Vector3, axisB: Vector3, scene: Scene);
}
/**
 *
 */
/** @internal */
export class PhysicsConstraintDistance extends PhysicsConstraint {
    /** @internal */
    constructor(pivotA: Vector3, pivotB: Vector3, axisA: Vector3, axisB: Vector3, scene: Scene);
}
/**
 *
 */
/** @internal */
export class PhysicsConstraintHinge extends PhysicsConstraint {
    /** @internal */
    constructor(pivotA: Vector3, pivotB: Vector3, axisA: Vector3, axisB: Vector3, scene: Scene);
}
/**
 *
 */
/** @internal */
export class PhysicsConstraintSlider extends PhysicsConstraint {
    /** @internal */
    constructor(pivotA: Vector3, pivotB: Vector3, axisA: Vector3, axisB: Vector3, scene: Scene);
}
/**
 *
 */
/** @internal */
export class PhysicsConstraintLock extends PhysicsConstraint {
    /** @internal */
    constructor(pivotA: Vector3, pivotB: Vector3, axisA: Vector3, axisB: Vector3, scene: Scene);
}

}
declare module "babylonjs/Physics/v2/physicsEngine" {
import { Nullable } from "babylonjs/types";
import { Vector3 } from "babylonjs/Maths/math.vector";
import { IPhysicsEngine } from "babylonjs/Physics/IPhysicsEngine";
import { IPhysicsEnginePluginV2 } from "babylonjs/Physics/v2/IPhysicsEnginePlugin";
import { PhysicsRaycastResult } from "babylonjs/Physics/physicsRaycastResult";
import { PhysicsBody } from "babylonjs/Physics/v2/physicsBody";
/**
 * Class used to control physics engine
 * @see https://doc.babylonjs.com/features/featuresDeepDive/physics/usingPhysicsEngine
 */
/** @internal */
export class PhysicsEngine implements IPhysicsEngine {
    private _physicsPlugin;
    /** @internal */
    private _physicsBodies;
    private _subTimeStep;
    /**
     * Gets the gravity vector used by the simulation
     */
    gravity: Vector3;
    /**
     *
     * @returns physics plugin version
     */
    getPluginVersion(): number;
    /**
     * Factory used to create the default physics plugin.
     * @returns The default physics plugin
     */
    static DefaultPluginFactory(): IPhysicsEnginePluginV2;
    /**
     * Creates a new Physics Engine
     * @param gravity defines the gravity vector used by the simulation
     * @param _physicsPlugin defines the plugin to use (CannonJS by default)
     */
    constructor(gravity: Nullable<Vector3>, _physicsPlugin?: IPhysicsEnginePluginV2);
    /**
     * Sets the gravity vector used by the simulation
     * @param gravity defines the gravity vector to use
     */
    setGravity(gravity: Vector3): void;
    /**
     * Set the time step of the physics engine.
     * Default is 1/60.
     * To slow it down, enter 1/600 for example.
     * To speed it up, 1/30
     * @param newTimeStep defines the new timestep to apply to this world.
     */
    setTimeStep(newTimeStep?: number): void;
    /**
     * Get the time step of the physics engine.
     * @returns the current time step
     */
    getTimeStep(): number;
    /**
     * Set the sub time step of the physics engine.
     * Default is 0 meaning there is no sub steps
     * To increase physics resolution precision, set a small value (like 1 ms)
     * @param subTimeStep defines the new sub timestep used for physics resolution.
     */
    setSubTimeStep(subTimeStep?: number): void;
    /**
     * Get the sub time step of the physics engine.
     * @returns the current sub time step
     */
    getSubTimeStep(): number;
    /**
     * Release all resources
     */
    dispose(): void;
    /**
     * Gets the name of the current physics plugin
     * @returns the name of the plugin
     */
    getPhysicsPluginName(): string;
    /**
     * Adding a new impostor for the impostor tracking.
     * This will be done by the impostor itself.
     * @param impostor the impostor to add
     */
    /**
     * Called by the scene. No need to call it.
     * @param delta defines the timespan between frames
     */
    _step(delta: number): void;
    /**
     * Add a body as an active component of this engine
     * @param body
     */
    addBody(physicsBody: PhysicsBody): void;
    /**
     * Removes a particular body from this engine
     */
    removeBody(physicsBody: PhysicsBody): void;
    /**
     * Returns an array of bodies added to this engine

     */
    getBodies(): Array<PhysicsBody>;
    /**
     * Gets the current plugin used to run the simulation
     * @returns current plugin
     */
    getPhysicsPlugin(): IPhysicsEnginePluginV2;
    /**
     * Does a raycast in the physics world
     * @param from when should the ray start?
     * @param to when should the ray end?
     * @param result resulting PhysicsRaycastResult
     */
    raycastToRef(from: Vector3, to: Vector3, result: PhysicsRaycastResult): void;
    /**
     * Does a raycast in the physics world
     * @param from when should the ray start?
     * @param to when should the ray end?
     * @returns PhysicsRaycastResult
     */
    raycast(from: Vector3, to: Vector3): PhysicsRaycastResult;
}

}
declare module "babylonjs/Physics/v2/physicsEngineComponent" {
import { Nullable } from "babylonjs/types";
import { Observer } from "babylonjs/Misc/observable";
import { Vector3 } from "babylonjs/Maths/math.vector";
import { Node } from "babylonjs/node";
import { PhysicsBody } from "babylonjs/Physics/v2/physicsBody";
import "babylonjs/Physics/joinedPhysicsEngineComponent";
module "babylonjs/Meshes/transformNode" {
    /**
     *
     */
    /** @internal */
    interface TransformNode {
        /** @internal */
        _physicsBody: Nullable<PhysicsBody>;
        /**
         * @see
         */
        physicsBody: Nullable<PhysicsBody>;
        /**
         *
         */
        getPhysicsBody(): Nullable<PhysicsBody>;
        /** Apply a physic impulse to the mesh
         * @param force defines the force to apply
         * @param contactPoint defines where to apply the force
         * @returns the current mesh
         */
        applyImpulse(force: Vector3, contactPoint: Vector3): TransformNode;
        /** @internal */
        _disposePhysicsObserver: Nullable<Observer<Node>>;
    }
}

}
declare module "babylonjs/Physics/v2/physicsMaterial" {
import { Scene } from "babylonjs/scene";
import { IPhysicsEnginePluginV2 } from "babylonjs/Physics/v2/IPhysicsEnginePlugin";
/**
 * Physics material class
 * Helps setting friction and restitution that are used to compute responding forces in collision response
 */
export class PhysicsMaterial {
    /**
     * V2 Physics plugin private data for a physics material
     */
    _pluginData: any;
    /**
     * The V2 plugin used to create and manage this Physics Body
     */
    protected _physicsPlugin: IPhysicsEnginePluginV2;
    /**
     * Constructor of the physics material.
     * @param friction - The friction of the material.
     * @param restitution - The restitution of the material.
     * @param scene - The scene to which the physics engine belongs.
     *
     * This code is useful for initializing the physics material with the
     * given friction, restitution and scene. It also checks if the
     * physics engine and plugin are available and if the plugin version
     * is correct. If all conditions are met, it initializes the material
     * with the physics plugin.
     */
    constructor(friction: number, restitution: number, scene: Scene);
    /**
     * Sets the friction of the physics engine.
     * @param friction - The friction to set.
     *
     * This method is useful for setting the friction of the physics engine, which is important for simulating realistic physics.
     * The friction determines how much an object will slow down when it is in contact with another object.
     * This is important for simulating realistic physics, such as when an object slides across a surface.
     */
    setFriction(friction: number): void;
    /**
     * Gets the friction of the physics engine.
     *
     * @returns The friction of the physics engine.
     *
     * This method is useful for getting the friction of the physics engine, which is used to calculate the force of friction between two objects.
     * Knowing the friction of the engine can help to accurately simulate the physical behavior of objects in the engine.
     */
    getFriction(): number;
    /**
     * Sets the restitution of the physics body.
     * @param restitution A number between 0 and 1 that represents the restitution of the body.
     *
     * This method is useful for setting the restitution of a physics body, which is the amount of energy that is retained after a collision.
     * A restitution of 0 means that no energy is retained, while a restitution of 1 means that all energy is retained.
     * Setting the restitution of a body can help to create realistic physics simulations.
     */
    setRestitution(restitution: number): void;
    /**
     * Gets the restitution of the physics engine.
     * @returns The restitution of the physics engine.
     *
     * This method is useful for retrieving the restitution of the physics engine, which is the amount of energy that is preserved after two objects collide.
     * Knowing the restitution of the physics engine can help to accurately simulate the behavior of objects in the physics engine.
     */
    getRestitution(): number;
    /**
     * Disposes the material.
     *
     * This method is useful for cleaning up the material when it is no longer needed.
     * It calls the disposeMaterial method of the physics plugin, which is responsible for disposing the material and freeing up any resources associated with it.
     * This ensures that the material is properly disposed of and does not cause any memory leaks.
     */
    dispose(): void;
}

}
declare module "babylonjs/Physics/v2/physicsShape" {
import { TransformNode } from "babylonjs/Meshes/transformNode";
import { BoundingBox } from "babylonjs/Culling/boundingBox";
import { ShapeType } from "babylonjs/Physics/v2/IPhysicsEnginePlugin";
import { PhysicsShapeParameters } from "babylonjs/Physics/v2/IPhysicsEnginePlugin";
import { PhysicsMaterial } from "babylonjs/Physics/v2/physicsMaterial";
import { Vector3 } from "babylonjs/Maths/math.vector";
import { Quaternion } from "babylonjs/Maths/math.vector";
import { Mesh } from "babylonjs/Meshes/mesh";
import { Scene } from "babylonjs/scene";
/**
 * PhysicsShape class.
 * This class is useful for creating a physics shape that can be used in a physics engine.
 * A Physic Shape determine how collision are computed. It must be attached to a body.
 */
export class PhysicsShape {
    /**
     * V2 Physics plugin private data for single shape
     */
    _pluginData: any;
    /**
     * The V2 plugin used to create and manage this Physics Body
     */
    private _physicsPlugin;
    private _type;
    /**
     * Constructs a new physics shape.
     * @param type The type of the shape.
     * @param options The options of the shape.
     * @param scene The scene the shape belongs to.
     *
     * This code is useful for creating a new physics shape with the given type, options, and scene.
     * It also checks that the physics engine and plugin version are correct.
     * If not, it throws an error. This ensures that the shape is created with the correct parameters and is compatible with the physics engine.
     */
    constructor(type: number, options: PhysicsShapeParameters | undefined, scene: Scene);
    /**
     *
     */
    get type(): ShapeType;
    /**
     *
     * @param layer
     */
    setFilterLayer(layer: number): void;
    /**
     *
     * @returns
     */
    getFilterLayer(): number;
    /**
     *
     * @param materialId
     */
    setMaterial(material: PhysicsMaterial): void;
    /**
     *
     * @returns
     */
    getMaterial(): PhysicsMaterial | undefined;
    /**
     *
     * @param density
     */
    setDensity(density: number): void;
    /**
     *
     */
    getDensity(): number;
    /**
     *
     * @param newChild
     * @param childTransform
     */
    addChild(newChild: PhysicsShape, childTransform: TransformNode): void;
    /**
     *
     * @param childIndex
     */
    removeChild(childIndex: number): void;
    /**
     *
     * @returns
     */
    getNumChildren(): number;
    /**
     *
     */
    getBoundingBox(): BoundingBox;
    /**
     *
     */
    dispose(): void;
}
/**
 * Helper object to create a sphere shape
 */
export class PhysicsShapeSphere extends PhysicsShape {
    /** @internal */
    /**
     * Constructor for the Sphere Shape
     * @param center local center of the sphere
     * @param radius radius
     * @param scene scene to attach to
     */
    constructor(center: Vector3, radius: number, scene: Scene);
}
/***
 * Helper object to create a capsule shape
 */
export class PhysicsShapeCapsule extends PhysicsShape {
    /**
     *
     * @param pointA Starting point that defines the capsule segment
     * @param pointB ending point of that same segment
     * @param radius radius
     * @param scene scene to attach to
     */
    constructor(pointA: Vector3, pointB: Vector3, radius: number, scene: Scene);
}
/**
 * Helper object to create a cylinder shape
 */
export class PhysicsShapeCylinder extends PhysicsShape {
    /**
     *
     * @param pointA Starting point that defines the cylinder segment
     * @param pointB ending point of that same segment
     * @param radius radius
     * @param scene scene to attach to
     */
    constructor(pointA: Vector3, pointB: Vector3, radius: number, scene: Scene);
}
/**
 * Helper object to create a box shape
 */
export class PhysicsShapeBox extends PhysicsShape {
    /**
     *
     * @param center local center of the sphere
     * @param rotation local orientation
     * @param extents size of the box in each direction
     * @param scene scene to attach to
     */
    constructor(center: Vector3, rotation: Quaternion, extents: Vector3, scene: Scene);
}
/**
 * Helper object to create a convex hull shape
 */
export class PhysicsShapeConvexHull extends PhysicsShape {
    /**
     *
     * @param mesh the mesh to be used as topology infos for the convex hull
     * @param scene scene to attach to
     */
    constructor(mesh: Mesh, scene: Scene);
}
/**
 * Helper object to create a mesh shape
 */
export class PhysicsShapeMesh extends PhysicsShape {
    /**
     *
     * @param mesh the mesh topology that will be used to create the shape
     * @param scene scene to attach to
     */
    constructor(mesh: Mesh, scene: Scene);
}
/**
 * A shape container holds a variable number of shapes. Use AddChild to append to newly created parent container.
 */
export class PhysicsShapeContainer extends PhysicsShape {
    /**
     * Constructor of the Shape container
     * @param scene scene to attach to
     */
    constructor(scene: Scene);
}

}
declare module "babylonjs/Physics/v2/Plugins/index" {

}
declare module "babylonjs/PostProcesses/anaglyphPostProcess" {
import { Engine } from "babylonjs/Engines/engine";
import { PostProcessOptions } from "babylonjs/PostProcesses/postProcess";
import { PostProcess } from "babylonjs/PostProcesses/postProcess";
import { Camera } from "babylonjs/Cameras/camera";
import "babylonjs/Shaders/anaglyph.fragment";
/**
 * Postprocess used to generate anaglyphic rendering
 */
export class AnaglyphPostProcess extends PostProcess {
    private _passedProcess;
    /**
     * Gets a string identifying the name of the class
     * @returns "AnaglyphPostProcess" string
     */
    getClassName(): string;
    /**
     * Creates a new AnaglyphPostProcess
     * @param name defines postprocess name
     * @param options defines creation options or target ratio scale
     * @param rigCameras defines cameras using this postprocess
     * @param samplingMode defines required sampling mode (BABYLON.Texture.NEAREST_SAMPLINGMODE by default)
     * @param engine defines hosting engine
     * @param reusable defines if the postprocess will be reused multiple times per frame
     */
    constructor(name: string, options: number | PostProcessOptions, rigCameras: Camera[], samplingMode?: number, engine?: Engine, reusable?: boolean);
}

}
declare module "babylonjs/PostProcesses/blackAndWhitePostProcess" {
import { PostProcessOptions } from "babylonjs/PostProcesses/postProcess";
import { PostProcess } from "babylonjs/PostProcesses/postProcess";
import { Camera } from "babylonjs/Cameras/camera";
import { Engine } from "babylonjs/Engines/engine";
import "babylonjs/Shaders/blackAndWhite.fragment";
import { Nullable } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
/**
 * Post process used to render in black and white
 */
export class BlackAndWhitePostProcess extends PostProcess {
    /**
     * Linear about to convert he result to black and white (default: 1)
     */
    degree: number;
    /**
     * Gets a string identifying the name of the class
     * @returns "BlackAndWhitePostProcess" string
     */
    getClassName(): string;
    /**
     * Creates a black and white post process
     * @see https://doc.babylonjs.com/features/featuresDeepDive/postProcesses/usePostProcesses#black-and-white
     * @param name The name of the effect.
     * @param options The required width/height ratio to downsize to before computing the render pass.
     * @param camera The camera to apply the render pass to.
     * @param samplingMode The sampling mode to be used when computing the pass. (default: 0)
     * @param engine The engine which the post process will be applied. (default: current engine)
     * @param reusable If the post process can be reused on the same frame. (default: false)
     */
    constructor(name: string, options: number | PostProcessOptions, camera: Nullable<Camera>, samplingMode?: number, engine?: Engine, reusable?: boolean);
    /**
     * @internal
     */
    static _Parse(parsedPostProcess: any, targetCamera: Camera, scene: Scene, rootUrl: string): Nullable<BlackAndWhitePostProcess>;
}
export {};

}
declare module "babylonjs/PostProcesses/bloomEffect" {
import { PostProcessRenderEffect } from "babylonjs/PostProcesses/RenderPipeline/postProcessRenderEffect";
import { PostProcess } from "babylonjs/PostProcesses/postProcess";
import { ExtractHighlightsPostProcess } from "babylonjs/PostProcesses/extractHighlightsPostProcess";
import { Camera } from "babylonjs/Cameras/camera";
import { Scene } from "babylonjs/scene";
/**
 * The bloom effect spreads bright areas of an image to simulate artifacts seen in cameras
 */
export class BloomEffect extends PostProcessRenderEffect {
    private _bloomScale;
    /**
     * @internal Internal
     */
    _effects: Array<PostProcess>;
    /**
     * @internal Internal
     */
    _downscale: ExtractHighlightsPostProcess;
    private _blurX;
    private _blurY;
    private _merge;
    /**
     * The luminance threshold to find bright areas of the image to bloom.
     */
    get threshold(): number;
    set threshold(value: number);
    /**
     * The strength of the bloom.
     */
    get weight(): number;
    set weight(value: number);
    /**
     * Specifies the size of the bloom blur kernel, relative to the final output size
     */
    get kernel(): number;
    set kernel(value: number);
    /**
     * Creates a new instance of @see BloomEffect
     * @param scene The scene the effect belongs to.
     * @param _bloomScale The ratio of the blur texture to the input texture that should be used to compute the bloom.
     * @param bloomWeight The the strength of bloom.
     * @param bloomKernel The size of the kernel to be used when applying the blur.
     * @param pipelineTextureType The type of texture to be used when performing the post processing.
     * @param blockCompilation If compilation of the shader should not be done in the constructor. The updateEffect method can be used to compile the shader at a later time. (default: false)
     */
    constructor(scene: Scene, _bloomScale: number, bloomWeight: number, bloomKernel: number, pipelineTextureType?: number, blockCompilation?: boolean);
    /**
     * Disposes each of the internal effects for a given camera.
     * @param camera The camera to dispose the effect on.
     */
    disposeEffects(camera: Camera): void;
    /**
     * @internal Internal
     */
    _updateEffects(): void;
    /**
     * Internal
     * @returns if all the contained post processes are ready.
     * @internal
     */
    _isReady(): boolean;
}

}
declare module "babylonjs/PostProcesses/bloomMergePostProcess" {
import { PostProcessOptions } from "babylonjs/PostProcesses/postProcess";
import { PostProcess } from "babylonjs/PostProcesses/postProcess";
import { Nullable } from "babylonjs/types";
import { Engine } from "babylonjs/Engines/engine";
import { Camera } from "babylonjs/Cameras/camera";
import "babylonjs/Shaders/bloomMerge.fragment";
/**
 * The BloomMergePostProcess merges blurred images with the original based on the values of the circle of confusion.
 */
export class BloomMergePostProcess extends PostProcess {
    /** Weight of the bloom to be added to the original input. */
    weight: number;
    /**
     * Gets a string identifying the name of the class
     * @returns "BloomMergePostProcess" string
     */
    getClassName(): string;
    /**
     * Creates a new instance of @see BloomMergePostProcess
     * @param name The name of the effect.
     * @param originalFromInput Post process which's input will be used for the merge.
     * @param blurred Blurred highlights post process which's output will be used.
     * @param weight Weight of the bloom to be added to the original input.
     * @param options The required width/height ratio to downsize to before computing the render pass.
     * @param camera The camera to apply the render pass to.
     * @param samplingMode The sampling mode to be used when computing the pass. (default: 0)
     * @param engine The engine which the post process will be applied. (default: current engine)
     * @param reusable If the post process can be reused on the same frame. (default: false)
     * @param textureType Type of textures used when performing the post process. (default: 0)
     * @param blockCompilation If compilation of the shader should not be done in the constructor. The updateEffect method can be used to compile the shader at a later time. (default: false)
     */
    constructor(name: string, originalFromInput: PostProcess, blurred: PostProcess, 
    /** Weight of the bloom to be added to the original input. */
    weight: number, options: number | PostProcessOptions, camera: Nullable<Camera>, samplingMode?: number, engine?: Engine, reusable?: boolean, textureType?: number, blockCompilation?: boolean);
}

}
declare module "babylonjs/PostProcesses/blurPostProcess" {
import { Vector2 } from "babylonjs/Maths/math.vector";
import { Nullable } from "babylonjs/types";
import { PostProcessOptions } from "babylonjs/PostProcesses/postProcess";
import { PostProcess } from "babylonjs/PostProcesses/postProcess";
import { Camera } from "babylonjs/Cameras/camera";
import { Effect } from "babylonjs/Materials/effect";
import { Engine } from "babylonjs/Engines/engine";
import "babylonjs/Shaders/kernelBlur.fragment";
import "babylonjs/Shaders/kernelBlur.vertex";
import { Scene } from "babylonjs/scene";
/**
 * The Blur Post Process which blurs an image based on a kernel and direction.
 * Can be used twice in x and y directions to perform a gaussian blur in two passes.
 */
export class BlurPostProcess extends PostProcess {
    private _blockCompilation;
    protected _kernel: number;
    protected _idealKernel: number;
    protected _packedFloat: boolean;
    private _staticDefines;
    /** The direction in which to blur the image. */
    direction: Vector2;
    /**
     * Sets the length in pixels of the blur sample region
     */
    set kernel(v: number);
    /**
     * Gets the length in pixels of the blur sample region
     */
    get kernel(): number;
    /**
     * Sets whether or not the blur needs to unpack/repack floats
     */
    set packedFloat(v: boolean);
    /**
     * Gets whether or not the blur is unpacking/repacking floats
     */
    get packedFloat(): boolean;
    /**
     * Gets a string identifying the name of the class
     * @returns "BlurPostProcess" string
     */
    getClassName(): string;
    /**
     * Creates a new instance BlurPostProcess
     * @param name The name of the effect.
     * @param direction The direction in which to blur the image.
     * @param kernel The size of the kernel to be used when computing the blur. eg. Size of 3 will blur the center pixel by 2 pixels surrounding it.
     * @param options The required width/height ratio to downsize to before computing the render pass. (Use 1.0 for full size)
     * @param camera The camera to apply the render pass to.
     * @param samplingMode The sampling mode to be used when computing the pass. (default: 0)
     * @param engine The engine which the post process will be applied. (default: current engine)
     * @param reusable If the post process can be reused on the same frame. (default: false)
     * @param textureType Type of textures used when performing the post process. (default: 0)
     * @param defines
     * @param _blockCompilation If compilation of the shader should not be done in the constructor. The updateEffect method can be used to compile the shader at a later time. (default: false)
     * @param textureFormat Format of textures used when performing the post process. (default: TEXTUREFORMAT_RGBA)
     */
    constructor(name: string, direction: Vector2, kernel: number, options: number | PostProcessOptions, camera: Nullable<Camera>, samplingMode?: number, engine?: Engine, reusable?: boolean, textureType?: number, defines?: string, _blockCompilation?: boolean, textureFormat?: number);
    /**
     * Updates the effect with the current post process compile time values and recompiles the shader.
     * @param defines Define statements that should be added at the beginning of the shader. (default: null)
     * @param uniforms Set of uniform variables that will be passed to the shader. (default: null)
     * @param samplers Set of Texture2D variables that will be passed to the shader. (default: null)
     * @param indexParameters The index parameters to be used for babylons include syntax "#include<kernelBlurVaryingDeclaration>[0..varyingCount]". (default: undefined) See usage in babylon.blurPostProcess.ts and kernelBlur.vertex.fx
     * @param onCompiled Called when the shader has been compiled.
     * @param onError Called if there is an error when compiling a shader.
     */
    updateEffect(defines?: Nullable<string>, uniforms?: Nullable<string[]>, samplers?: Nullable<string[]>, indexParameters?: any, onCompiled?: (effect: Effect) => void, onError?: (effect: Effect, errors: string) => void): void;
    protected _updateParameters(onCompiled?: (effect: Effect) => void, onError?: (effect: Effect, errors: string) => void): void;
    /**
     * Best kernels are odd numbers that when divided by 2, their integer part is even, so 5, 9 or 13.
     * Other odd kernels optimize correctly but require proportionally more samples, even kernels are
     * possible but will produce minor visual artifacts. Since each new kernel requires a new shader we
     * want to minimize kernel changes, having gaps between physical kernels is helpful in that regard.
     * The gaps between physical kernels are compensated for in the weighting of the samples
     * @param idealKernel Ideal blur kernel.
     * @returns Nearest best kernel.
     */
    protected _nearestBestKernel(idealKernel: number): number;
    /**
     * Calculates the value of a Gaussian distribution with sigma 3 at a given point.
     * @param x The point on the Gaussian distribution to sample.
     * @returns the value of the Gaussian function at x.
     */
    protected _gaussianWeight(x: number): number;
    /**
     * Generates a string that can be used as a floating point number in GLSL.
     * @param x Value to print.
     * @param decimalFigures Number of decimal places to print the number to (excluding trailing 0s).
     * @returns GLSL float string.
     */
    protected _glslFloat(x: number, decimalFigures?: number): string;
    /**
     * @internal
     */
    static _Parse(parsedPostProcess: any, targetCamera: Camera, scene: Scene, rootUrl: string): Nullable<BlurPostProcess>;
}
export {};

}
declare module "babylonjs/PostProcesses/chromaticAberrationPostProcess" {
import { Vector2 } from "babylonjs/Maths/math.vector";
import { Nullable } from "babylonjs/types";
import { PostProcessOptions } from "babylonjs/PostProcesses/postProcess";
import { PostProcess } from "babylonjs/PostProcesses/postProcess";
import { Camera } from "babylonjs/Cameras/camera";
import { Engine } from "babylonjs/Engines/engine";
import "babylonjs/Shaders/chromaticAberration.fragment";
import { Scene } from "babylonjs/scene";
/**
 * The ChromaticAberrationPostProcess separates the rgb channels in an image to produce chromatic distortion around the edges of the screen
 */
export class ChromaticAberrationPostProcess extends PostProcess {
    /**
     * The amount of separation of rgb channels (default: 30)
     */
    aberrationAmount: number;
    /**
     * The amount the effect will increase for pixels closer to the edge of the screen. (default: 0)
     */
    radialIntensity: number;
    /**
     * The normalized direction in which the rgb channels should be separated. If set to 0,0 radial direction will be used. (default: Vector2(0.707,0.707))
     */
    direction: Vector2;
    /**
     * The center position where the radialIntensity should be around. [0.5,0.5 is center of screen, 1,1 is top right corner] (default: Vector2(0.5 ,0.5))
     */
    centerPosition: Vector2;
    /** The width of the screen to apply the effect on */
    screenWidth: number;
    /** The height of the screen to apply the effect on */
    screenHeight: number;
    /**
     * Gets a string identifying the name of the class
     * @returns "ChromaticAberrationPostProcess" string
     */
    getClassName(): string;
    /**
     * Creates a new instance ChromaticAberrationPostProcess
     * @param name The name of the effect.
     * @param screenWidth The width of the screen to apply the effect on.
     * @param screenHeight The height of the screen to apply the effect on.
     * @param options The required width/height ratio to downsize to before computing the render pass.
     * @param camera The camera to apply the render pass to.
     * @param samplingMode The sampling mode to be used when computing the pass. (default: 0)
     * @param engine The engine which the post process will be applied. (default: current engine)
     * @param reusable If the post process can be reused on the same frame. (default: false)
     * @param textureType Type of textures used when performing the post process. (default: 0)
     * @param blockCompilation If compilation of the shader should not be done in the constructor. The updateEffect method can be used to compile the shader at a later time. (default: false)
     */
    constructor(name: string, screenWidth: number, screenHeight: number, options: number | PostProcessOptions, camera: Nullable<Camera>, samplingMode?: number, engine?: Engine, reusable?: boolean, textureType?: number, blockCompilation?: boolean);
    /**
     * @internal
     */
    static _Parse(parsedPostProcess: any, targetCamera: Camera, scene: Scene, rootUrl: string): Nullable<ChromaticAberrationPostProcess>;
}
export {};

}
declare module "babylonjs/PostProcesses/circleOfConfusionPostProcess" {
import { Nullable } from "babylonjs/types";
import { Engine } from "babylonjs/Engines/engine";
import { PostProcessOptions } from "babylonjs/PostProcesses/postProcess";
import { PostProcess } from "babylonjs/PostProcesses/postProcess";
import { RenderTargetTexture } from "babylonjs/Materials/Textures/renderTargetTexture";
import { Camera } from "babylonjs/Cameras/camera";
import "babylonjs/Shaders/circleOfConfusion.fragment";
/**
 * The CircleOfConfusionPostProcess computes the circle of confusion value for each pixel given required lens parameters. See https://en.wikipedia.org/wiki/Circle_of_confusion
 */
export class CircleOfConfusionPostProcess extends PostProcess {
    /**
     * Max lens size in scene units/1000 (eg. millimeter). Standard cameras are 50mm. (default: 50) The diameter of the resulting aperture can be computed by lensSize/fStop.
     */
    lensSize: number;
    /**
     * F-Stop of the effect's camera. The diameter of the resulting aperture can be computed by lensSize/fStop. (default: 1.4)
     */
    fStop: number;
    /**
     * Distance away from the camera to focus on in scene units/1000 (eg. millimeter). (default: 2000)
     */
    focusDistance: number;
    /**
     * Focal length of the effect's camera in scene units/1000 (eg. millimeter). (default: 50)
     */
    focalLength: number;
    /**
     * Gets a string identifying the name of the class
     * @returns "CircleOfConfusionPostProcess" string
     */
    getClassName(): string;
    private _depthTexture;
    /**
     * Creates a new instance CircleOfConfusionPostProcess
     * @param name The name of the effect.
     * @param depthTexture The depth texture of the scene to compute the circle of confusion. This must be set in order for this to function but may be set after initialization if needed.
     * @param options The required width/height ratio to downsize to before computing the render pass.
     * @param camera The camera to apply the render pass to.
     * @param samplingMode The sampling mode to be used when computing the pass. (default: 0)
     * @param engine The engine which the post process will be applied. (default: current engine)
     * @param reusable If the post process can be reused on the same frame. (default: false)
     * @param textureType Type of textures used when performing the post process. (default: 0)
     * @param blockCompilation If compilation of the shader should not be done in the constructor. The updateEffect method can be used to compile the shader at a later time. (default: false)
     */
    constructor(name: string, depthTexture: Nullable<RenderTargetTexture>, options: number | PostProcessOptions, camera: Nullable<Camera>, samplingMode?: number, engine?: Engine, reusable?: boolean, textureType?: number, blockCompilation?: boolean);
    /**
     * Depth texture to be used to compute the circle of confusion. This must be set here or in the constructor in order for the post process to function.
     */
    set depthTexture(value: RenderTargetTexture);
}

}
declare module "babylonjs/PostProcesses/colorCorrectionPostProcess" {
import { PostProcessOptions } from "babylonjs/PostProcesses/postProcess";
import { PostProcess } from "babylonjs/PostProcesses/postProcess";
import { Engine } from "babylonjs/Engines/engine";
import { Camera } from "babylonjs/Cameras/camera";
import "babylonjs/Shaders/colorCorrection.fragment";
import { Nullable } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
/**
 *
 * This post-process allows the modification of rendered colors by using
 * a 'look-up table' (LUT). This effect is also called Color Grading.
 *
 * The object needs to be provided an url to a texture containing the color
 * look-up table: the texture must be 256 pixels wide and 16 pixels high.
 * Use an image editing software to tweak the LUT to match your needs.
 *
 * For an example of a color LUT, see here:
 * @see http://udn.epicgames.com/Three/rsrc/Three/ColorGrading/RGBTable16x1.png
 * For explanations on color grading, see here:
 * @see http://udn.epicgames.com/Three/ColorGrading.html
 *
 */
export class ColorCorrectionPostProcess extends PostProcess {
    private _colorTableTexture;
    /**
     * Gets the color table url used to create the LUT texture
     */
    colorTableUrl: string;
    /**
     * Gets a string identifying the name of the class
     * @returns "ColorCorrectionPostProcess" string
     */
    getClassName(): string;
    constructor(name: string, colorTableUrl: string, options: number | PostProcessOptions, camera: Nullable<Camera>, samplingMode?: number, engine?: Engine, reusable?: boolean);
    /**
     * @internal
     */
    static _Parse(parsedPostProcess: any, targetCamera: Camera, scene: Scene, rootUrl: string): Nullable<ColorCorrectionPostProcess>;
}
export {};

}
declare module "babylonjs/PostProcesses/convolutionPostProcess" {
import { PostProcessOptions } from "babylonjs/PostProcesses/postProcess";
import { PostProcess } from "babylonjs/PostProcesses/postProcess";
import { Nullable } from "babylonjs/types";
import { Camera } from "babylonjs/Cameras/camera";
import { Engine } from "babylonjs/Engines/engine";
import "babylonjs/Shaders/convolution.fragment";
import { Scene } from "babylonjs/scene";
/**
 * The ConvolutionPostProcess applies a 3x3 kernel to every pixel of the
 * input texture to perform effects such as edge detection or sharpening
 * See http://en.wikipedia.org/wiki/Kernel_(image_processing)
 */
export class ConvolutionPostProcess extends PostProcess {
    /** Array of 9 values corresponding to the 3x3 kernel to be applied */
    kernel: number[];
    /**
     * Gets a string identifying the name of the class
     * @returns "ConvolutionPostProcess" string
     */
    getClassName(): string;
    /**
     * Creates a new instance ConvolutionPostProcess
     * @param name The name of the effect.
     * @param kernel Array of 9 values corresponding to the 3x3 kernel to be applied
     * @param options The required width/height ratio to downsize to before computing the render pass.
     * @param camera The camera to apply the render pass to.
     * @param samplingMode The sampling mode to be used when computing the pass. (default: 0)
     * @param engine The engine which the post process will be applied. (default: current engine)
     * @param reusable If the post process can be reused on the same frame. (default: false)
     * @param textureType Type of textures used when performing the post process. (default: 0)
     */
    constructor(name: string, kernel: number[], options: number | PostProcessOptions, camera: Nullable<Camera>, samplingMode?: number, engine?: Engine, reusable?: boolean, textureType?: number);
    /**
     * @internal
     */
    static _Parse(parsedPostProcess: any, targetCamera: Camera, scene: Scene, rootUrl: string): Nullable<ConvolutionPostProcess>;
    /**
     * Edge detection 0 see https://en.wikipedia.org/wiki/Kernel_(image_processing)
     */
    static EdgeDetect0Kernel: number[];
    /**
     * Edge detection 1 see https://en.wikipedia.org/wiki/Kernel_(image_processing)
     */
    static EdgeDetect1Kernel: number[];
    /**
     * Edge detection 2 see https://en.wikipedia.org/wiki/Kernel_(image_processing)
     */
    static EdgeDetect2Kernel: number[];
    /**
     * Kernel to sharpen an image see https://en.wikipedia.org/wiki/Kernel_(image_processing)
     */
    static SharpenKernel: number[];
    /**
     * Kernel to emboss an image see https://en.wikipedia.org/wiki/Kernel_(image_processing)
     */
    static EmbossKernel: number[];
    /**
     * Kernel to blur an image see https://en.wikipedia.org/wiki/Kernel_(image_processing)
     */
    static GaussianKernel: number[];
}
export {};

}
declare module "babylonjs/PostProcesses/depthOfFieldBlurPostProcess" {
import { Nullable } from "babylonjs/types";
import { Vector2 } from "babylonjs/Maths/math.vector";
import { Camera } from "babylonjs/Cameras/camera";
import { PostProcess, PostProcessOptions } from "babylonjs/PostProcesses/postProcess";
import { BlurPostProcess } from "babylonjs/PostProcesses/blurPostProcess";
import { Engine } from "babylonjs/Engines/engine";
import { Scene } from "babylonjs/scene";
/**
 * The DepthOfFieldBlurPostProcess applied a blur in a give direction.
 * This blur differs from the standard BlurPostProcess as it attempts to avoid blurring pixels
 * based on samples that have a large difference in distance than the center pixel.
 * See section 2.6.2 http://fileadmin.cs.lth.se/cs/education/edan35/lectures/12dof.pdf
 */
export class DepthOfFieldBlurPostProcess extends BlurPostProcess {
    /**
     * The direction the blur should be applied
     */
    direction: Vector2;
    /**
     * Gets a string identifying the name of the class
     * @returns "DepthOfFieldBlurPostProcess" string
     */
    getClassName(): string;
    /**
     * Creates a new instance DepthOfFieldBlurPostProcess
     * @param name The name of the effect.
     * @param scene The scene the effect belongs to.
     * @param direction The direction the blur should be applied.
     * @param kernel The size of the kernel used to blur.
     * @param options The required width/height ratio to downsize to before computing the render pass.
     * @param camera The camera to apply the render pass to.
     * @param circleOfConfusion The circle of confusion + depth map to be used to avoid blurring across edges
     * @param imageToBlur The image to apply the blur to (default: Current rendered frame)
     * @param samplingMode The sampling mode to be used when computing the pass. (default: 0)
     * @param engine The engine which the post process will be applied. (default: current engine)
     * @param reusable If the post process can be reused on the same frame. (default: false)
     * @param textureType Type of textures used when performing the post process. (default: 0)
     * @param blockCompilation If compilation of the shader should not be done in the constructor. The updateEffect method can be used to compile the shader at a later time. (default: false)
     * @param textureFormat Format of textures used when performing the post process. (default: TEXTUREFORMAT_RGBA)
     */
    constructor(name: string, scene: Scene, direction: Vector2, kernel: number, options: number | PostProcessOptions, camera: Nullable<Camera>, circleOfConfusion: PostProcess, imageToBlur?: Nullable<PostProcess>, samplingMode?: number, engine?: Engine, reusable?: boolean, textureType?: number, blockCompilation?: boolean, textureFormat?: number);
}

}
declare module "babylonjs/PostProcesses/depthOfFieldEffect" {
import { Nullable } from "babylonjs/types";
import { Camera } from "babylonjs/Cameras/camera";
import { RenderTargetTexture } from "babylonjs/Materials/Textures/renderTargetTexture";
import { PostProcess } from "babylonjs/PostProcesses/postProcess";
import { PostProcessRenderEffect } from "babylonjs/PostProcesses/RenderPipeline/postProcessRenderEffect";
import { DepthOfFieldBlurPostProcess } from "babylonjs/PostProcesses/depthOfFieldBlurPostProcess";
import { Scene } from "babylonjs/scene";
/**
 * Specifies the level of max blur that should be applied when using the depth of field effect
 */
export enum DepthOfFieldEffectBlurLevel {
    /**
     * Subtle blur
     */
    Low = 0,
    /**
     * Medium blur
     */
    Medium = 1,
    /**
     * Large blur
     */
    High = 2
}
/**
 * The depth of field effect applies a blur to objects that are closer or further from where the camera is focusing.
 */
export class DepthOfFieldEffect extends PostProcessRenderEffect {
    private _circleOfConfusion;
    /**
     * @internal Internal, blurs from high to low
     */
    _depthOfFieldBlurX: Array<DepthOfFieldBlurPostProcess>;
    private _depthOfFieldBlurY;
    private _dofMerge;
    /**
     * @internal Internal post processes in depth of field effect
     */
    _effects: Array<PostProcess>;
    /**
     * The focal the length of the camera used in the effect in scene units/1000 (eg. millimeter)
     */
    set focalLength(value: number);
    get focalLength(): number;
    /**
     * F-Stop of the effect's camera. The diameter of the resulting aperture can be computed by lensSize/fStop. (default: 1.4)
     */
    set fStop(value: number);
    get fStop(): number;
    /**
     * Distance away from the camera to focus on in scene units/1000 (eg. millimeter). (default: 2000)
     */
    set focusDistance(value: number);
    get focusDistance(): number;
    /**
     * Max lens size in scene units/1000 (eg. millimeter). Standard cameras are 50mm. (default: 50) The diameter of the resulting aperture can be computed by lensSize/fStop.
     */
    set lensSize(value: number);
    get lensSize(): number;
    /**
     * Creates a new instance DepthOfFieldEffect
     * @param scene The scene the effect belongs to.
     * @param depthTexture The depth texture of the scene to compute the circle of confusion.This must be set in order for this to function but may be set after initialization if needed.
     * @param blurLevel
     * @param pipelineTextureType The type of texture to be used when performing the post processing.
     * @param blockCompilation If compilation of the shader should not be done in the constructor. The updateEffect method can be used to compile the shader at a later time. (default: false)
     */
    constructor(scene: Scene, depthTexture: Nullable<RenderTargetTexture>, blurLevel?: DepthOfFieldEffectBlurLevel, pipelineTextureType?: number, blockCompilation?: boolean);
    /**
     * Get the current class name of the current effect
     * @returns "DepthOfFieldEffect"
     */
    getClassName(): string;
    /**
     * Depth texture to be used to compute the circle of confusion. This must be set here or in the constructor in order for the post process to function.
     */
    set depthTexture(value: RenderTargetTexture);
    /**
     * Disposes each of the internal effects for a given camera.
     * @param camera The camera to dispose the effect on.
     */
    disposeEffects(camera: Camera): void;
    /**
     * @internal Internal
     */
    _updateEffects(): void;
    /**
     * Internal
     * @returns if all the contained post processes are ready.
     * @internal
     */
    _isReady(): boolean;
}

}
declare module "babylonjs/PostProcesses/depthOfFieldMergePostProcess" {
import { Nullable } from "babylonjs/types";
import { Camera } from "babylonjs/Cameras/camera";
import { Effect } from "babylonjs/Materials/effect";
import { PostProcessOptions } from "babylonjs/PostProcesses/postProcess";
import { PostProcess } from "babylonjs/PostProcesses/postProcess";
import { Engine } from "babylonjs/Engines/engine";
import "babylonjs/Shaders/depthOfFieldMerge.fragment";
/**
 * Options to be set when merging outputs from the default pipeline.
 */
export class DepthOfFieldMergePostProcessOptions {
    /**
     * The original image to merge on top of
     */
    originalFromInput: PostProcess;
    /**
     * Parameters to perform the merge of the depth of field effect
     */
    depthOfField?: {
        circleOfConfusion: PostProcess;
        blurSteps: Array<PostProcess>;
    };
    /**
     * Parameters to perform the merge of bloom effect
     */
    bloom?: {
        blurred: PostProcess;
        weight: number;
    };
}
/**
 * The DepthOfFieldMergePostProcess merges blurred images with the original based on the values of the circle of confusion.
 */
export class DepthOfFieldMergePostProcess extends PostProcess {
    private _blurSteps;
    /**
     * Gets a string identifying the name of the class
     * @returns "DepthOfFieldMergePostProcess" string
     */
    getClassName(): string;
    /**
     * Creates a new instance of DepthOfFieldMergePostProcess
     * @param name The name of the effect.
     * @param originalFromInput Post process which's input will be used for the merge.
     * @param circleOfConfusion Circle of confusion post process which's output will be used to blur each pixel.
     * @param _blurSteps Blur post processes from low to high which will be mixed with the original image.
     * @param options The required width/height ratio to downsize to before computing the render pass.
     * @param camera The camera to apply the render pass to.
     * @param samplingMode The sampling mode to be used when computing the pass. (default: 0)
     * @param engine The engine which the post process will be applied. (default: current engine)
     * @param reusable If the post process can be reused on the same frame. (default: false)
     * @param textureType Type of textures used when performing the post process. (default: 0)
     * @param blockCompilation If compilation of the shader should not be done in the constructor. The updateEffect method can be used to compile the shader at a later time. (default: false)
     */
    constructor(name: string, originalFromInput: PostProcess, circleOfConfusion: PostProcess, _blurSteps: Array<PostProcess>, options: number | PostProcessOptions, camera: Nullable<Camera>, samplingMode?: number, engine?: Engine, reusable?: boolean, textureType?: number, blockCompilation?: boolean);
    /**
     * Updates the effect with the current post process compile time values and recompiles the shader.
     * @param defines Define statements that should be added at the beginning of the shader. (default: null)
     * @param uniforms Set of uniform variables that will be passed to the shader. (default: null)
     * @param samplers Set of Texture2D variables that will be passed to the shader. (default: null)
     * @param indexParameters The index parameters to be used for babylons include syntax "#include<kernelBlurVaryingDeclaration>[0..varyingCount]". (default: undefined) See usage in babylon.blurPostProcess.ts and kernelBlur.vertex.fx
     * @param onCompiled Called when the shader has been compiled.
     * @param onError Called if there is an error when compiling a shader.
     */
    updateEffect(defines?: Nullable<string>, uniforms?: Nullable<string[]>, samplers?: Nullable<string[]>, indexParameters?: any, onCompiled?: (effect: Effect) => void, onError?: (effect: Effect, errors: string) => void): void;
}

}
declare module "babylonjs/PostProcesses/displayPassPostProcess" {
import { Nullable } from "babylonjs/types";
import { Camera } from "babylonjs/Cameras/camera";
import { PostProcessOptions } from "babylonjs/PostProcesses/postProcess";
import { PostProcess } from "babylonjs/PostProcesses/postProcess";
import { Engine } from "babylonjs/Engines/engine";
import "babylonjs/Shaders/displayPass.fragment";
import { Scene } from "babylonjs/scene";
/**
 * DisplayPassPostProcess which produces an output the same as it's input
 */
export class DisplayPassPostProcess extends PostProcess {
    /**
     * Gets a string identifying the name of the class
     * @returns "DisplayPassPostProcess" string
     */
    getClassName(): string;
    /**
     * Creates the DisplayPassPostProcess
     * @param name The name of the effect.
     * @param options The required width/height ratio to downsize to before computing the render pass.
     * @param camera The camera to apply the render pass to.
     * @param samplingMode The sampling mode to be used when computing the pass. (default: 0)
     * @param engine The engine which the post process will be applied. (default: current engine)
     * @param reusable If the post process can be reused on the same frame. (default: false)
     */
    constructor(name: string, options: number | PostProcessOptions, camera: Nullable<Camera>, samplingMode?: number, engine?: Engine, reusable?: boolean);
    /**
     * @internal
     */
    static _Parse(parsedPostProcess: any, targetCamera: Camera, scene: Scene, rootUrl: string): Nullable<DisplayPassPostProcess>;
}
export {};

}
declare module "babylonjs/PostProcesses/extractHighlightsPostProcess" {
import { Nullable } from "babylonjs/types";
import { Camera } from "babylonjs/Cameras/camera";
import { PostProcessOptions } from "babylonjs/PostProcesses/postProcess";
import { PostProcess } from "babylonjs/PostProcesses/postProcess";
import { Engine } from "babylonjs/Engines/engine";
import "babylonjs/Shaders/extractHighlights.fragment";
/**
 * The extract highlights post process sets all pixels to black except pixels above the specified luminance threshold. Used as the first step for a bloom effect.
 */
export class ExtractHighlightsPostProcess extends PostProcess {
    /**
     * The luminance threshold, pixels below this value will be set to black.
     */
    threshold: number;
    /** @internal */
    _exposure: number;
    /**
     * Post process which has the input texture to be used when performing highlight extraction
     * @internal
     */
    _inputPostProcess: Nullable<PostProcess>;
    /**
     * Gets a string identifying the name of the class
     * @returns "ExtractHighlightsPostProcess" string
     */
    getClassName(): string;
    constructor(name: string, options: number | PostProcessOptions, camera: Nullable<Camera>, samplingMode?: number, engine?: Engine, reusable?: boolean, textureType?: number, blockCompilation?: boolean);
}

}
declare module "babylonjs/PostProcesses/filterPostProcess" {
import { Nullable } from "babylonjs/types";
import { Matrix } from "babylonjs/Maths/math.vector";
import { Camera } from "babylonjs/Cameras/camera";
import { PostProcessOptions } from "babylonjs/PostProcesses/postProcess";
import { PostProcess } from "babylonjs/PostProcesses/postProcess";
import { Engine } from "babylonjs/Engines/engine";
import "babylonjs/Shaders/filter.fragment";
import { Scene } from "babylonjs/scene";
/**
 * Applies a kernel filter to the image
 */
export class FilterPostProcess extends PostProcess {
    /** The matrix to be applied to the image */
    kernelMatrix: Matrix;
    /**
     * Gets a string identifying the name of the class
     * @returns "FilterPostProcess" string
     */
    getClassName(): string;
    /**
     *
     * @param name The name of the effect.
     * @param kernelMatrix The matrix to be applied to the image
     * @param options The required width/height ratio to downsize to before computing the render pass.
     * @param camera The camera to apply the render pass to.
     * @param samplingMode The sampling mode to be used when computing the pass. (default: 0)
     * @param engine The engine which the post process will be applied. (default: current engine)
     * @param reusable If the post process can be reused on the same frame. (default: false)
     */
    constructor(name: string, kernelMatrix: Matrix, options: number | PostProcessOptions, camera: Nullable<Camera>, samplingMode?: number, engine?: Engine, reusable?: boolean);
    /**
     * @internal
     */
    static _Parse(parsedPostProcess: any, targetCamera: Camera, scene: Scene, rootUrl: string): Nullable<FilterPostProcess>;
}
export {};

}
declare module "babylonjs/PostProcesses/fxaaPostProcess" {
import { Nullable } from "babylonjs/types";
import { Camera } from "babylonjs/Cameras/camera";
import { PostProcessOptions } from "babylonjs/PostProcesses/postProcess";
import { PostProcess } from "babylonjs/PostProcesses/postProcess";
import { Engine } from "babylonjs/Engines/engine";
import "babylonjs/Shaders/fxaa.fragment";
import "babylonjs/Shaders/fxaa.vertex";
import { Scene } from "babylonjs/scene";
/**
 * Fxaa post process
 * @see https://doc.babylonjs.com/features/featuresDeepDive/postProcesses/usePostProcesses#fxaa
 */
export class FxaaPostProcess extends PostProcess {
    /**
     * Gets a string identifying the name of the class
     * @returns "FxaaPostProcess" string
     */
    getClassName(): string;
    constructor(name: string, options: number | PostProcessOptions, camera?: Nullable<Camera>, samplingMode?: number, engine?: Engine, reusable?: boolean, textureType?: number);
    private _getDefines;
    /**
     * @internal
     */
    static _Parse(parsedPostProcess: any, targetCamera: Camera, scene: Scene, rootUrl: string): FxaaPostProcess;
}
export {};

}
declare module "babylonjs/PostProcesses/grainPostProcess" {
import { Nullable } from "babylonjs/types";
import { Camera } from "babylonjs/Cameras/camera";
import { PostProcessOptions } from "babylonjs/PostProcesses/postProcess";
import { PostProcess } from "babylonjs/PostProcesses/postProcess";
import { Engine } from "babylonjs/Engines/engine";
import "babylonjs/Shaders/grain.fragment";
import { Scene } from "babylonjs/scene";
/**
 * The GrainPostProcess adds noise to the image at mid luminance levels
 */
export class GrainPostProcess extends PostProcess {
    /**
     * The intensity of the grain added (default: 30)
     */
    intensity: number;
    /**
     * If the grain should be randomized on every frame
     */
    animated: boolean;
    /**
     * Gets a string identifying the name of the class
     * @returns "GrainPostProcess" string
     */
    getClassName(): string;
    /**
     * Creates a new instance of @see GrainPostProcess
     * @param name The name of the effect.
     * @param options The required width/height ratio to downsize to before computing the render pass.
     * @param camera The camera to apply the render pass to.
     * @param samplingMode The sampling mode to be used when computing the pass. (default: 0)
     * @param engine The engine which the post process will be applied. (default: current engine)
     * @param reusable If the post process can be reused on the same frame. (default: false)
     * @param textureType Type of textures used when performing the post process. (default: 0)
     * @param blockCompilation If compilation of the shader should not be done in the constructor. The updateEffect method can be used to compile the shader at a later time. (default: false)
     */
    constructor(name: string, options: number | PostProcessOptions, camera: Nullable<Camera>, samplingMode?: number, engine?: Engine, reusable?: boolean, textureType?: number, blockCompilation?: boolean);
    /**
     * @internal
     */
    static _Parse(parsedPostProcess: any, targetCamera: Camera, scene: Scene, rootUrl: string): GrainPostProcess;
}
export {};

}
declare module "babylonjs/PostProcesses/highlightsPostProcess" {
import { Nullable } from "babylonjs/types";
import { Camera } from "babylonjs/Cameras/camera";
import { PostProcessOptions } from "babylonjs/PostProcesses/postProcess";
import { PostProcess } from "babylonjs/PostProcesses/postProcess";
import { Engine } from "babylonjs/Engines/engine";
import "babylonjs/Shaders/highlights.fragment";
/**
 * Extracts highlights from the image
 * @see https://doc.babylonjs.com/features/featuresDeepDive/postProcesses/usePostProcesses
 */
export class HighlightsPostProcess extends PostProcess {
    /**
     * Gets a string identifying the name of the class
     * @returns "HighlightsPostProcess" string
     */
    getClassName(): string;
    /**
     * Extracts highlights from the image
     * @see https://doc.babylonjs.com/features/featuresDeepDive/postProcesses/usePostProcesses
     * @param name The name of the effect.
     * @param options The required width/height ratio to downsize to before computing the render pass.
     * @param camera The camera to apply the render pass to.
     * @param samplingMode The sampling mode to be used when computing the pass. (default: 0)
     * @param engine The engine which the post process will be applied. (default: current engine)
     * @param reusable If the post process can be reused on the same frame. (default: false)
     * @param textureType Type of texture for the post process (default: Engine.TEXTURETYPE_UNSIGNED_INT)
     */
    constructor(name: string, options: number | PostProcessOptions, camera: Nullable<Camera>, samplingMode?: number, engine?: Engine, reusable?: boolean, textureType?: number);
}

}
declare module "babylonjs/PostProcesses/imageProcessingPostProcess" {
import { Nullable } from "babylonjs/types";
import { Color4 } from "babylonjs/Maths/math.color";
import { Camera } from "babylonjs/Cameras/camera";
import { BaseTexture } from "babylonjs/Materials/Textures/baseTexture";
import { ColorCurves } from "babylonjs/Materials/colorCurves";
import { ImageProcessingConfiguration } from "babylonjs/Materials/imageProcessingConfiguration";
import { PostProcessOptions } from "babylonjs/PostProcesses/postProcess";
import { PostProcess } from "babylonjs/PostProcesses/postProcess";
import { Engine } from "babylonjs/Engines/engine";
import "babylonjs/Shaders/imageProcessing.fragment";
import "babylonjs/Shaders/postprocess.vertex";
/**
 * ImageProcessingPostProcess
 * @see https://doc.babylonjs.com/features/featuresDeepDive/postProcesses/usePostProcesses#imageprocessing
 */
export class ImageProcessingPostProcess extends PostProcess {
    /**
     * Default configuration related to image processing available in the PBR Material.
     */
    protected _imageProcessingConfiguration: ImageProcessingConfiguration;
    /**
     * Gets the image processing configuration used either in this material.
     */
    get imageProcessingConfiguration(): ImageProcessingConfiguration;
    /**
     * Sets the Default image processing configuration used either in the this material.
     *
     * If sets to null, the scene one is in use.
     */
    set imageProcessingConfiguration(value: ImageProcessingConfiguration);
    /**
     * Keep track of the image processing observer to allow dispose and replace.
     */
    private _imageProcessingObserver;
    /**
     * Attaches a new image processing configuration to the PBR Material.
     * @param configuration
     * @param doNotBuild
     */
    protected _attachImageProcessingConfiguration(configuration: Nullable<ImageProcessingConfiguration>, doNotBuild?: boolean): void;
    /**
     * If the post process is supported.
     */
    get isSupported(): boolean;
    /**
     * Gets Color curves setup used in the effect if colorCurvesEnabled is set to true .
     */
    get colorCurves(): Nullable<ColorCurves>;
    /**
     * Sets Color curves setup used in the effect if colorCurvesEnabled is set to true .
     */
    set colorCurves(value: Nullable<ColorCurves>);
    /**
     * Gets whether the color curves effect is enabled.
     */
    get colorCurvesEnabled(): boolean;
    /**
     * Sets whether the color curves effect is enabled.
     */
    set colorCurvesEnabled(value: boolean);
    /**
     * Gets Color grading LUT texture used in the effect if colorGradingEnabled is set to true.
     */
    get colorGradingTexture(): Nullable<BaseTexture>;
    /**
     * Sets Color grading LUT texture used in the effect if colorGradingEnabled is set to true.
     */
    set colorGradingTexture(value: Nullable<BaseTexture>);
    /**
     * Gets whether the color grading effect is enabled.
     */
    get colorGradingEnabled(): boolean;
    /**
     * Gets whether the color grading effect is enabled.
     */
    set colorGradingEnabled(value: boolean);
    /**
     * Gets exposure used in the effect.
     */
    get exposure(): number;
    /**
     * Sets exposure used in the effect.
     */
    set exposure(value: number);
    /**
     * Gets whether tonemapping is enabled or not.
     */
    get toneMappingEnabled(): boolean;
    /**
     * Sets whether tonemapping is enabled or not
     */
    set toneMappingEnabled(value: boolean);
    /**
     * Gets the type of tone mapping effect.
     */
    get toneMappingType(): number;
    /**
     * Sets the type of tone mapping effect.
     */
    set toneMappingType(value: number);
    /**
     * Gets contrast used in the effect.
     */
    get contrast(): number;
    /**
     * Sets contrast used in the effect.
     */
    set contrast(value: number);
    /**
     * Gets Vignette stretch size.
     */
    get vignetteStretch(): number;
    /**
     * Sets Vignette stretch size.
     */
    set vignetteStretch(value: number);
    /**
     * Gets Vignette center X Offset.
     * @deprecated use vignetteCenterX instead
     */
    get vignetteCentreX(): number;
    /**
     * Sets Vignette center X Offset.
     * @deprecated use vignetteCenterX instead
     */
    set vignetteCentreX(value: number);
    /**
     * Gets Vignette center Y Offset.
     * @deprecated use vignetteCenterY instead
     */
    get vignetteCentreY(): number;
    /**
     * Sets Vignette center Y Offset.
     * @deprecated use vignetteCenterY instead
     */
    set vignetteCentreY(value: number);
    /**
     * Vignette center Y Offset.
     */
    get vignetteCenterY(): number;
    set vignetteCenterY(value: number);
    /**
     * Vignette center X Offset.
     */
    get vignetteCenterX(): number;
    set vignetteCenterX(value: number);
    /**
     * Gets Vignette weight or intensity of the vignette effect.
     */
    get vignetteWeight(): number;
    /**
     * Sets Vignette weight or intensity of the vignette effect.
     */
    set vignetteWeight(value: number);
    /**
     * Gets Color of the vignette applied on the screen through the chosen blend mode (vignetteBlendMode)
     * if vignetteEnabled is set to true.
     */
    get vignetteColor(): Color4;
    /**
     * Sets Color of the vignette applied on the screen through the chosen blend mode (vignetteBlendMode)
     * if vignetteEnabled is set to true.
     */
    set vignetteColor(value: Color4);
    /**
     * Gets Camera field of view used by the Vignette effect.
     */
    get vignetteCameraFov(): number;
    /**
     * Sets Camera field of view used by the Vignette effect.
     */
    set vignetteCameraFov(value: number);
    /**
     * Gets the vignette blend mode allowing different kind of effect.
     */
    get vignetteBlendMode(): number;
    /**
     * Sets the vignette blend mode allowing different kind of effect.
     */
    set vignetteBlendMode(value: number);
    /**
     * Gets whether the vignette effect is enabled.
     */
    get vignetteEnabled(): boolean;
    /**
     * Sets whether the vignette effect is enabled.
     */
    set vignetteEnabled(value: boolean);
    /**
     * Gets intensity of the dithering effect.
     */
    get ditheringIntensity(): number;
    /**
     * Sets intensity of the dithering effect.
     */
    set ditheringIntensity(value: number);
    /**
     * Gets whether the dithering effect is enabled.
     */
    get ditheringEnabled(): boolean;
    /**
     * Sets whether the dithering effect is enabled.
     */
    set ditheringEnabled(value: boolean);
    private _fromLinearSpace;
    /**
     * Gets whether the input of the processing is in Gamma or Linear Space.
     */
    get fromLinearSpace(): boolean;
    /**
     * Sets whether the input of the processing is in Gamma or Linear Space.
     */
    set fromLinearSpace(value: boolean);
    /**
     * Defines cache preventing GC.
     */
    private _defines;
    constructor(name: string, options: number | PostProcessOptions, camera?: Nullable<Camera>, samplingMode?: number, engine?: Engine, reusable?: boolean, textureType?: number, imageProcessingConfiguration?: ImageProcessingConfiguration);
    /**
     *  "ImageProcessingPostProcess"
     * @returns "ImageProcessingPostProcess"
     */
    getClassName(): string;
    /**
     * @internal
     */
    _updateParameters(): void;
    dispose(camera?: Camera): void;
}

}
declare module "babylonjs/PostProcesses/index" {
export * from "babylonjs/PostProcesses/anaglyphPostProcess";
export * from "babylonjs/PostProcesses/blackAndWhitePostProcess";
export * from "babylonjs/PostProcesses/bloomEffect";
export * from "babylonjs/PostProcesses/bloomMergePostProcess";
export * from "babylonjs/PostProcesses/blurPostProcess";
export * from "babylonjs/PostProcesses/chromaticAberrationPostProcess";
export * from "babylonjs/PostProcesses/circleOfConfusionPostProcess";
export * from "babylonjs/PostProcesses/colorCorrectionPostProcess";
export * from "babylonjs/PostProcesses/convolutionPostProcess";
export * from "babylonjs/PostProcesses/depthOfFieldBlurPostProcess";
export * from "babylonjs/PostProcesses/depthOfFieldEffect";
export * from "babylonjs/PostProcesses/depthOfFieldMergePostProcess";
export * from "babylonjs/PostProcesses/displayPassPostProcess";
export * from "babylonjs/PostProcesses/extractHighlightsPostProcess";
export * from "babylonjs/PostProcesses/filterPostProcess";
export * from "babylonjs/PostProcesses/fxaaPostProcess";
export * from "babylonjs/PostProcesses/grainPostProcess";
export * from "babylonjs/PostProcesses/highlightsPostProcess";
export * from "babylonjs/PostProcesses/imageProcessingPostProcess";
export * from "babylonjs/PostProcesses/motionBlurPostProcess";
export * from "babylonjs/PostProcesses/passPostProcess";
export * from "babylonjs/PostProcesses/postProcess";
export * from "babylonjs/PostProcesses/postProcessManager";
export * from "babylonjs/PostProcesses/refractionPostProcess";
export * from "babylonjs/PostProcesses/RenderPipeline/index";
export * from "babylonjs/PostProcesses/sharpenPostProcess";
export * from "babylonjs/PostProcesses/stereoscopicInterlacePostProcess";
export * from "babylonjs/PostProcesses/tonemapPostProcess";
export * from "babylonjs/PostProcesses/volumetricLightScatteringPostProcess";
export * from "babylonjs/PostProcesses/vrDistortionCorrectionPostProcess";
export * from "babylonjs/PostProcesses/vrMultiviewToSingleviewPostProcess";
export * from "babylonjs/PostProcesses/screenSpaceReflectionPostProcess";
export * from "babylonjs/PostProcesses/screenSpaceCurvaturePostProcess";

}
declare module "babylonjs/PostProcesses/motionBlurPostProcess" {
import { Nullable } from "babylonjs/types";
import { Camera } from "babylonjs/Cameras/camera";
import { PostProcessOptions } from "babylonjs/PostProcesses/postProcess";
import { PostProcess } from "babylonjs/PostProcesses/postProcess";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import "babylonjs/Animations/animatable";
import "babylonjs/Rendering/geometryBufferRendererSceneComponent";
import "babylonjs/Shaders/motionBlur.fragment";
import { Engine } from "babylonjs/Engines/engine";
import { Scene } from "babylonjs/scene";
/**
 * The Motion Blur Post Process which blurs an image based on the objects velocity in scene.
 * Velocity can be affected by each object's rotation, position and scale depending on the transformation speed.
 * As an example, all you have to do is to create the post-process:
 *  var mb = new BABYLON.MotionBlurPostProcess(
 *      'mb', // The name of the effect.
 *      scene, // The scene containing the objects to blur according to their velocity.
 *      1.0, // The required width/height ratio to downsize to before computing the render pass.
 *      camera // The camera to apply the render pass to.
 * );
 * Then, all objects moving, rotating and/or scaling will be blurred depending on the transformation speed.
 */
export class MotionBlurPostProcess extends PostProcess {
    /**
     * Defines how much the image is blurred by the movement. Default value is equal to 1
     */
    motionStrength: number;
    /**
     * Gets the number of iterations are used for motion blur quality. Default value is equal to 32
     */
    get motionBlurSamples(): number;
    /**
     * Sets the number of iterations to be used for motion blur quality
     */
    set motionBlurSamples(samples: number);
    private _motionBlurSamples;
    /**
     * Gets whether or not the motion blur post-process is in object based mode.
     */
    get isObjectBased(): boolean;
    /**
     * Sets whether or not the motion blur post-process is in object based mode.
     */
    set isObjectBased(value: boolean);
    private _isObjectBased;
    private _forceGeometryBuffer;
    private get _geometryBufferRenderer();
    private get _prePassRenderer();
    private _invViewProjection;
    private _previousViewProjection;
    /**
     * Gets a string identifying the name of the class
     * @returns "MotionBlurPostProcess" string
     */
    getClassName(): string;
    /**
     * Creates a new instance MotionBlurPostProcess
     * @param name The name of the effect.
     * @param scene The scene containing the objects to blur according to their velocity.
     * @param options The required width/height ratio to downsize to before computing the render pass.
     * @param camera The camera to apply the render pass to.
     * @param samplingMode The sampling mode to be used when computing the pass. (default: 0)
     * @param engine The engine which the post process will be applied. (default: current engine)
     * @param reusable If the post process can be reused on the same frame. (default: false)
     * @param textureType Type of textures used when performing the post process. (default: 0)
     * @param blockCompilation If compilation of the shader should not be done in the constructor. The updateEffect method can be used to compile the shader at a later time. (default: true)
     * @param forceGeometryBuffer If this post process should use geometry buffer instead of prepass (default: false)
     */
    constructor(name: string, scene: Scene, options: number | PostProcessOptions, camera: Nullable<Camera>, samplingMode?: number, engine?: Engine, reusable?: boolean, textureType?: number, blockCompilation?: boolean, forceGeometryBuffer?: boolean);
    /**
     * Excludes the given skinned mesh from computing bones velocities.
     * Computing bones velocities can have a cost and that cost. The cost can be saved by calling this function and by passing the skinned mesh reference to ignore.
     * @param skinnedMesh The mesh containing the skeleton to ignore when computing the velocity map.
     */
    excludeSkinnedMesh(skinnedMesh: AbstractMesh): void;
    /**
     * Removes the given skinned mesh from the excluded meshes to integrate bones velocities while rendering the velocity map.
     * @param skinnedMesh The mesh containing the skeleton that has been ignored previously.
     * @see excludeSkinnedMesh to exclude a skinned mesh from bones velocity computation.
     */
    removeExcludedSkinnedMesh(skinnedMesh: AbstractMesh): void;
    /**
     * Disposes the post process.
     * @param camera The camera to dispose the post process on.
     */
    dispose(camera?: Camera): void;
    /**
     * Called on the mode changed (object based or screen based).
     */
    private _applyMode;
    /**
     * Called on the effect is applied when the motion blur post-process is in object based mode.
     * @param effect
     */
    private _onApplyObjectBased;
    /**
     * Called on the effect is applied when the motion blur post-process is in screen based mode.
     * @param effect
     */
    private _onApplyScreenBased;
    /**
     * Called on the effect must be updated (changed mode, samples count, etc.).
     */
    private _updateEffect;
    /**
     * @internal
     */
    static _Parse(parsedPostProcess: any, targetCamera: Camera, scene: Scene, rootUrl: string): Nullable<MotionBlurPostProcess>;
}
export {};

}
declare module "babylonjs/PostProcesses/passPostProcess" {
import { Nullable } from "babylonjs/types";
import { Camera } from "babylonjs/Cameras/camera";
import { PostProcessOptions } from "babylonjs/PostProcesses/postProcess";
import { PostProcess } from "babylonjs/PostProcesses/postProcess";
import { Engine } from "babylonjs/Engines/engine";
import "babylonjs/Shaders/pass.fragment";
import "babylonjs/Shaders/passCube.fragment";
import { Scene } from "babylonjs/scene";
/**
 * PassPostProcess which produces an output the same as it's input
 */
export class PassPostProcess extends PostProcess {
    /**
     * Gets a string identifying the name of the class
     * @returns "PassPostProcess" string
     */
    getClassName(): string;
    /**
     * Creates the PassPostProcess
     * @param name The name of the effect.
     * @param options The required width/height ratio to downsize to before computing the render pass.
     * @param camera The camera to apply the render pass to.
     * @param samplingMode The sampling mode to be used when computing the pass. (default: 0)
     * @param engine The engine which the post process will be applied. (default: current engine)
     * @param reusable If the post process can be reused on the same frame. (default: false)
     * @param textureType The type of texture to be used when performing the post processing.
     * @param blockCompilation If compilation of the shader should not be done in the constructor. The updateEffect method can be used to compile the shader at a later time. (default: false)
     */
    constructor(name: string, options: number | PostProcessOptions, camera?: Nullable<Camera>, samplingMode?: number, engine?: Engine, reusable?: boolean, textureType?: number, blockCompilation?: boolean);
    /**
     * @internal
     */
    static _Parse(parsedPostProcess: any, targetCamera: Camera, scene: Scene, rootUrl: string): PassPostProcess;
}
/**
 * PassCubePostProcess which produces an output the same as it's input (which must be a cube texture)
 */
export class PassCubePostProcess extends PostProcess {
    private _face;
    /**
     * Gets or sets the cube face to display.
     *  * 0 is +X
     *  * 1 is -X
     *  * 2 is +Y
     *  * 3 is -Y
     *  * 4 is +Z
     *  * 5 is -Z
     */
    get face(): number;
    set face(value: number);
    /**
     * Gets a string identifying the name of the class
     * @returns "PassCubePostProcess" string
     */
    getClassName(): string;
    /**
     * Creates the PassCubePostProcess
     * @param name The name of the effect.
     * @param options The required width/height ratio to downsize to before computing the render pass.
     * @param camera The camera to apply the render pass to.
     * @param samplingMode The sampling mode to be used when computing the pass. (default: 0)
     * @param engine The engine which the post process will be applied. (default: current engine)
     * @param reusable If the post process can be reused on the same frame. (default: false)
     * @param textureType The type of texture to be used when performing the post processing.
     * @param blockCompilation If compilation of the shader should not be done in the constructor. The updateEffect method can be used to compile the shader at a later time. (default: false)
     */
    constructor(name: string, options: number | PostProcessOptions, camera?: Nullable<Camera>, samplingMode?: number, engine?: Engine, reusable?: boolean, textureType?: number, blockCompilation?: boolean);
    /**
     * @internal
     */
    static _Parse(parsedPostProcess: any, targetCamera: Camera, scene: Scene, rootUrl: string): PassCubePostProcess;
}
export {};

}
declare module "babylonjs/PostProcesses/postProcess" {
import { Nullable } from "babylonjs/types";
import { SmartArray } from "babylonjs/Misc/smartArray";
import { Observable } from "babylonjs/Misc/observable";
import { Vector2 } from "babylonjs/Maths/math.vector";
import { Camera } from "babylonjs/Cameras/camera";
import { Effect } from "babylonjs/Materials/effect";
import "babylonjs/Shaders/postprocess.vertex";
import { IInspectable } from "babylonjs/Misc/iInspectable";
import { Engine } from "babylonjs/Engines/engine";
import { Color4 } from "babylonjs/Maths/math.color";
import "babylonjs/Engines/Extensions/engine.renderTarget";
import { NodeMaterial } from "babylonjs/Materials/Node/nodeMaterial";
import { AbstractScene } from "babylonjs/abstractScene";
import { RenderTargetWrapper } from "babylonjs/Engines/renderTargetWrapper";
import { ShaderLanguage } from "babylonjs/Materials/shaderLanguage";
import { Scene } from "babylonjs/scene";
import { InternalTexture } from "babylonjs/Materials/Textures/internalTexture";
import { PrePassRenderer } from "babylonjs/Rendering/prePassRenderer";
import { PrePassEffectConfiguration } from "babylonjs/Rendering/prePassEffectConfiguration";
/**
 * Allows for custom processing of the shader code used by a post process
 */
export type PostProcessCustomShaderCodeProcessing = {
    /**
     * If provided, will be called two times with the vertex and fragment code so that this code can be updated after the #include have been processed
     */
    processCodeAfterIncludes?: (postProcessName: string, shaderType: string, code: string) => string;
    /**
     * If provided, will be called two times with the vertex and fragment code so that this code can be updated before it is compiled by the GPU
     */
    processFinalCode?: (postProcessName: string, shaderType: string, code: string) => string;
    /**
     * If provided, will be called before creating the effect to collect additional custom bindings (defines, uniforms, samplers)
     */
    defineCustomBindings?: (postProcessName: string, defines: Nullable<string>, uniforms: string[], samplers: string[]) => Nullable<string>;
    /**
     * If provided, will be called when binding inputs to the shader code to allow the user to add custom bindings
     */
    bindCustomBindings?: (postProcessName: string, effect: Effect) => void;
};
/**
 * Size options for a post process
 */
export type PostProcessOptions = {
    width: number;
    height: number;
};
/**
 * PostProcess can be used to apply a shader to a texture after it has been rendered
 * See https://doc.babylonjs.com/features/featuresDeepDive/postProcesses/usePostProcesses
 */
export class PostProcess {
    /** @internal */
    _parentContainer: Nullable<AbstractScene>;
    private static _CustomShaderCodeProcessing;
    /**
     * Registers a shader code processing with a post process name.
     * @param postProcessName name of the post process. Use null for the fallback shader code processing. This is the shader code processing that will be used in case no specific shader code processing has been associated to a post process name
     * @param customShaderCodeProcessing shader code processing to associate to the post process name
     * @returns
     */
    static RegisterShaderCodeProcessing(postProcessName: Nullable<string>, customShaderCodeProcessing?: PostProcessCustomShaderCodeProcessing): void;
    private static _GetShaderCodeProcessing;
    /**
     * Gets or sets the unique id of the post process
     */
    uniqueId: number;
    /** Name of the PostProcess. */
    name: string;
    /**
     * Width of the texture to apply the post process on
     */
    width: number;
    /**
     * Height of the texture to apply the post process on
     */
    height: number;
    /**
     * Gets the node material used to create this postprocess (null if the postprocess was manually created)
     */
    nodeMaterialSource: Nullable<NodeMaterial>;
    /**
     * Internal, reference to the location where this postprocess was output to. (Typically the texture on the next postprocess in the chain)
     * @internal
     */
    _outputTexture: Nullable<RenderTargetWrapper>;
    /**
     * Sampling mode used by the shader
     * See https://doc.babylonjs.com/classes/3.1/texture
     */
    renderTargetSamplingMode: number;
    /**
     * Clear color to use when screen clearing
     */
    clearColor: Color4;
    /**
     * If the buffer needs to be cleared before applying the post process. (default: true)
     * Should be set to false if shader will overwrite all previous pixels.
     */
    autoClear: boolean;
    /**
     * Type of alpha mode to use when performing the post process (default: Engine.ALPHA_DISABLE)
     */
    alphaMode: number;
    /**
     * Sets the setAlphaBlendConstants of the babylon engine
     */
    alphaConstants: Color4;
    /**
     * Animations to be used for the post processing
     */
    animations: import("babylonjs/Animations/animation").Animation[];
    /**
     * Enable Pixel Perfect mode where texture is not scaled to be power of 2.
     * Can only be used on a single postprocess or on the last one of a chain. (default: false)
     */
    enablePixelPerfectMode: boolean;
    /**
     * Force the postprocess to be applied without taking in account viewport
     */
    forceFullscreenViewport: boolean;
    /**
     * List of inspectable custom properties (used by the Inspector)
     * @see https://doc.babylonjs.com/toolsAndResources/inspector#extensibility
     */
    inspectableCustomProperties: IInspectable[];
    /**
     * Scale mode for the post process (default: Engine.SCALEMODE_FLOOR)
     *
     * | Value | Type                                | Description |
     * | ----- | ----------------------------------- | ----------- |
     * | 1     | SCALEMODE_FLOOR                     | [engine.scalemode_floor](https://doc.babylonjs.com/api/classes/babylon.engine#scalemode_floor) |
     * | 2     | SCALEMODE_NEAREST                   | [engine.scalemode_nearest](https://doc.babylonjs.com/api/classes/babylon.engine#scalemode_nearest) |
     * | 3     | SCALEMODE_CEILING                   | [engine.scalemode_ceiling](https://doc.babylonjs.com/api/classes/babylon.engine#scalemode_ceiling) |
     *
     */
    scaleMode: number;
    /**
     * Force textures to be a power of two (default: false)
     */
    alwaysForcePOT: boolean;
    private _samples;
    /**
     * Number of sample textures (default: 1)
     */
    get samples(): number;
    set samples(n: number);
    /**
     * Modify the scale of the post process to be the same as the viewport (default: false)
     */
    adaptScaleToCurrentViewport: boolean;
    private _camera;
    protected _scene: Scene;
    private _engine;
    private _options;
    private _reusable;
    private _renderId;
    private _textureType;
    private _textureFormat;
    private _shaderLanguage;
    /**
     * if externalTextureSamplerBinding is true, the "apply" method won't bind the textureSampler texture, it is expected to be done by the "outside" (by the onApplyObservable observer most probably).
     * counter-productive in some cases because if the texture bound by "apply" is different from the currently texture bound, (the one set by the onApplyObservable observer, for eg) some
     * internal structures (materialContext) will be dirtified, which may impact performances
     */
    externalTextureSamplerBinding: boolean;
    /**
     * Smart array of input and output textures for the post process.
     * @internal
     */
    _textures: SmartArray<RenderTargetWrapper>;
    /**
     * Smart array of input and output textures for the post process.
     * @internal
     */
    private _textureCache;
    /**
     * The index in _textures that corresponds to the output texture.
     * @internal
     */
    _currentRenderTextureInd: number;
    private _drawWrapper;
    private _samplers;
    private _fragmentUrl;
    private _vertexUrl;
    private _parameters;
    protected _postProcessDefines: Nullable<string>;
    private _scaleRatio;
    protected _indexParameters: any;
    private _shareOutputWithPostProcess;
    private _texelSize;
    /** @internal */
    _forcedOutputTexture: Nullable<RenderTargetWrapper>;
    /**
     * Prepass configuration in case this post process needs a texture from prepass
     * @internal
     */
    _prePassEffectConfiguration: PrePassEffectConfiguration;
    /**
     * Returns the fragment url or shader name used in the post process.
     * @returns the fragment url or name in the shader store.
     */
    getEffectName(): string;
    /**
     * An event triggered when the postprocess is activated.
     */
    onActivateObservable: Observable<Camera>;
    private _onActivateObserver;
    /**
     * A function that is added to the onActivateObservable
     */
    set onActivate(callback: Nullable<(camera: Camera) => void>);
    /**
     * An event triggered when the postprocess changes its size.
     */
    onSizeChangedObservable: Observable<PostProcess>;
    private _onSizeChangedObserver;
    /**
     * A function that is added to the onSizeChangedObservable
     */
    set onSizeChanged(callback: (postProcess: PostProcess) => void);
    /**
     * An event triggered when the postprocess applies its effect.
     */
    onApplyObservable: Observable<Effect>;
    private _onApplyObserver;
    /**
     * A function that is added to the onApplyObservable
     */
    set onApply(callback: (effect: Effect) => void);
    /**
     * An event triggered before rendering the postprocess
     */
    onBeforeRenderObservable: Observable<Effect>;
    private _onBeforeRenderObserver;
    /**
     * A function that is added to the onBeforeRenderObservable
     */
    set onBeforeRender(callback: (effect: Effect) => void);
    /**
     * An event triggered after rendering the postprocess
     */
    onAfterRenderObservable: Observable<Effect>;
    private _onAfterRenderObserver;
    /**
     * A function that is added to the onAfterRenderObservable
     */
    set onAfterRender(callback: (efect: Effect) => void);
    /**
     * The input texture for this post process and the output texture of the previous post process. When added to a pipeline the previous post process will
     * render it's output into this texture and this texture will be used as textureSampler in the fragment shader of this post process.
     */
    get inputTexture(): RenderTargetWrapper;
    set inputTexture(value: RenderTargetWrapper);
    /**
     * Since inputTexture should always be defined, if we previously manually set `inputTexture`,
     * the only way to unset it is to use this function to restore its internal state
     */
    restoreDefaultInputTexture(): void;
    /**
     * Gets the camera which post process is applied to.
     * @returns The camera the post process is applied to.
     */
    getCamera(): Camera;
    /**
     * Gets the texel size of the postprocess.
     * See https://en.wikipedia.org/wiki/Texel_(graphics)
     */
    get texelSize(): Vector2;
    /**
     * Creates a new instance PostProcess
     * @param name The name of the PostProcess.
     * @param fragmentUrl The url of the fragment shader to be used.
     * @param parameters Array of the names of uniform non-sampler2D variables that will be passed to the shader.
     * @param samplers Array of the names of uniform sampler2D variables that will be passed to the shader.
     * @param options The required width/height ratio to downsize to before computing the render pass. (Use 1.0 for full size)
     * @param camera The camera to apply the render pass to.
     * @param samplingMode The sampling mode to be used when computing the pass. (default: 0)
     * @param engine The engine which the post process will be applied. (default: current engine)
     * @param reusable If the post process can be reused on the same frame. (default: false)
     * @param defines String of defines that will be set when running the fragment shader. (default: null)
     * @param textureType Type of textures used when performing the post process. (default: 0)
     * @param vertexUrl The url of the vertex shader to be used. (default: "postprocess")
     * @param indexParameters The index parameters to be used for babylons include syntax "#include<kernelBlurVaryingDeclaration>[0..varyingCount]". (default: undefined) See usage in babylon.blurPostProcess.ts and kernelBlur.vertex.fx
     * @param blockCompilation If the shader should not be compiled immediatly. (default: false)
     * @param textureFormat Format of textures used when performing the post process. (default: TEXTUREFORMAT_RGBA)
     */
    constructor(name: string, fragmentUrl: string, parameters: Nullable<string[]>, samplers: Nullable<string[]>, options: number | PostProcessOptions, camera: Nullable<Camera>, samplingMode?: number, engine?: Engine, reusable?: boolean, defines?: Nullable<string>, textureType?: number, vertexUrl?: string, indexParameters?: any, blockCompilation?: boolean, textureFormat?: number, shaderLanguage?: ShaderLanguage);
    /**
     * Gets a string identifying the name of the class
     * @returns "PostProcess" string
     */
    getClassName(): string;
    /**
     * Gets the engine which this post process belongs to.
     * @returns The engine the post process was enabled with.
     */
    getEngine(): Engine;
    /**
     * The effect that is created when initializing the post process.
     * @returns The created effect corresponding the the postprocess.
     */
    getEffect(): Effect;
    /**
     * To avoid multiple redundant textures for multiple post process, the output the output texture for this post process can be shared with another.
     * @param postProcess The post process to share the output with.
     * @returns This post process.
     */
    shareOutputWith(postProcess: PostProcess): PostProcess;
    /**
     * Reverses the effect of calling shareOutputWith and returns the post process back to its original state.
     * This should be called if the post process that shares output with this post process is disabled/disposed.
     */
    useOwnOutput(): void;
    /**
     * Updates the effect with the current post process compile time values and recompiles the shader.
     * @param defines Define statements that should be added at the beginning of the shader. (default: null)
     * @param uniforms Set of uniform variables that will be passed to the shader. (default: null)
     * @param samplers Set of Texture2D variables that will be passed to the shader. (default: null)
     * @param indexParameters The index parameters to be used for babylons include syntax "#include<kernelBlurVaryingDeclaration>[0..varyingCount]". (default: undefined) See usage in babylon.blurPostProcess.ts and kernelBlur.vertex.fx
     * @param onCompiled Called when the shader has been compiled.
     * @param onError Called if there is an error when compiling a shader.
     * @param vertexUrl The url of the vertex shader to be used (default: the one given at construction time)
     * @param fragmentUrl The url of the fragment shader to be used (default: the one given at construction time)
     */
    updateEffect(defines?: Nullable<string>, uniforms?: Nullable<string[]>, samplers?: Nullable<string[]>, indexParameters?: any, onCompiled?: (effect: Effect) => void, onError?: (effect: Effect, errors: string) => void, vertexUrl?: string, fragmentUrl?: string): void;
    /**
     * The post process is reusable if it can be used multiple times within one frame.
     * @returns If the post process is reusable
     */
    isReusable(): boolean;
    /** invalidate frameBuffer to hint the postprocess to create a depth buffer */
    markTextureDirty(): void;
    private _createRenderTargetTexture;
    private _flushTextureCache;
    private _resize;
    /**
     * Activates the post process by intializing the textures to be used when executed. Notifies onActivateObservable.
     * When this post process is used in a pipeline, this is call will bind the input texture of this post process to the output of the previous.
     * @param camera The camera that will be used in the post process. This camera will be used when calling onActivateObservable.
     * @param sourceTexture The source texture to be inspected to get the width and height if not specified in the post process constructor. (default: null)
     * @param forceDepthStencil If true, a depth and stencil buffer will be generated. (default: false)
     * @returns The render target wrapper that was bound to be written to.
     */
    activate(camera: Nullable<Camera>, sourceTexture?: Nullable<InternalTexture>, forceDepthStencil?: boolean): RenderTargetWrapper;
    /**
     * If the post process is supported.
     */
    get isSupported(): boolean;
    /**
     * The aspect ratio of the output texture.
     */
    get aspectRatio(): number;
    /**
     * Get a value indicating if the post-process is ready to be used
     * @returns true if the post-process is ready (shader is compiled)
     */
    isReady(): boolean;
    /**
     * Binds all textures and uniforms to the shader, this will be run on every pass.
     * @returns the effect corresponding to this post process. Null if not compiled or not ready.
     */
    apply(): Nullable<Effect>;
    private _disposeTextures;
    private _disposeTextureCache;
    /**
     * Sets the required values to the prepass renderer.
     * @param prePassRenderer defines the prepass renderer to setup.
     * @returns true if the pre pass is needed.
     */
    setPrePassRenderer(prePassRenderer: PrePassRenderer): boolean;
    /**
     * Disposes the post process.
     * @param camera The camera to dispose the post process on.
     */
    dispose(camera?: Camera): void;
    /**
     * Serializes the post process to a JSON object
     * @returns the JSON object
     */
    serialize(): any;
    /**
     * Clones this post process
     * @returns a new post process similar to this one
     */
    clone(): Nullable<PostProcess>;
    /**
     * Creates a material from parsed material data
     * @param parsedPostProcess defines parsed post process data
     * @param scene defines the hosting scene
     * @param rootUrl defines the root URL to use to load textures
     * @returns a new post process
     */
    static Parse(parsedPostProcess: any, scene: Scene, rootUrl: string): Nullable<PostProcess>;
    /**
     * @internal
     */
    static _Parse(parsedPostProcess: any, targetCamera: Camera, scene: Scene, rootUrl: string): Nullable<PostProcess>;
}
export {};

}
declare module "babylonjs/PostProcesses/postProcessManager" {
import { Nullable } from "babylonjs/types";
import { InternalTexture } from "babylonjs/Materials/Textures/internalTexture";
import { PostProcess } from "babylonjs/PostProcesses/postProcess";
import { RenderTargetWrapper } from "babylonjs/Engines/renderTargetWrapper";
import { Scene } from "babylonjs/scene";
/**
 * PostProcessManager is used to manage one or more post processes or post process pipelines
 * See https://doc.babylonjs.com/features/featuresDeepDive/postProcesses/usePostProcesses
 */
export class PostProcessManager {
    private _scene;
    private _indexBuffer;
    private _vertexBuffers;
    /**
     * Creates a new instance PostProcess
     * @param scene The scene that the post process is associated with.
     */
    constructor(scene: Scene);
    private _prepareBuffers;
    private _buildIndexBuffer;
    /**
     * Rebuilds the vertex buffers of the manager.
     * @internal
     */
    _rebuild(): void;
    /**
     * Prepares a frame to be run through a post process.
     * @param sourceTexture The input texture to the post processes. (default: null)
     * @param postProcesses An array of post processes to be run. (default: null)
     * @returns True if the post processes were able to be run.
     * @internal
     */
    _prepareFrame(sourceTexture?: Nullable<InternalTexture>, postProcesses?: Nullable<PostProcess[]>): boolean;
    /**
     * Manually render a set of post processes to a texture.
     * Please note, the frame buffer won't be unbound after the call in case you have more render to do.
     * @param postProcesses An array of post processes to be run.
     * @param targetTexture The render target wrapper to render to.
     * @param forceFullscreenViewport force gl.viewport to be full screen eg. 0,0,textureWidth,textureHeight
     * @param faceIndex defines the face to render to if a cubemap is defined as the target
     * @param lodLevel defines which lod of the texture to render to
     * @param doNotBindFrambuffer If set to true, assumes that the framebuffer has been bound previously
     */
    directRender(postProcesses: PostProcess[], targetTexture?: Nullable<RenderTargetWrapper>, forceFullscreenViewport?: boolean, faceIndex?: number, lodLevel?: number, doNotBindFrambuffer?: boolean): void;
    /**
     * Finalize the result of the output of the postprocesses.
     * @param doNotPresent If true the result will not be displayed to the screen.
     * @param targetTexture The render target wrapper to render to.
     * @param faceIndex The index of the face to bind the target texture to.
     * @param postProcesses The array of post processes to render.
     * @param forceFullscreenViewport force gl.viewport to be full screen eg. 0,0,textureWidth,textureHeight (default: false)
     * @internal
     */
    _finalizeFrame(doNotPresent?: boolean, targetTexture?: RenderTargetWrapper, faceIndex?: number, postProcesses?: Array<PostProcess>, forceFullscreenViewport?: boolean): void;
    /**
     * Disposes of the post process manager.
     */
    dispose(): void;
}
export {};

}
declare module "babylonjs/PostProcesses/refractionPostProcess" {
import { Color3 } from "babylonjs/Maths/math.color";
import { Camera } from "babylonjs/Cameras/camera";
import { Texture } from "babylonjs/Materials/Textures/texture";
import { PostProcessOptions } from "babylonjs/PostProcesses/postProcess";
import { PostProcess } from "babylonjs/PostProcesses/postProcess";
import { Engine } from "babylonjs/Engines/engine";
import "babylonjs/Shaders/refraction.fragment";
import { Nullable } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
/**
 * Post process which applies a refraction texture
 * @see https://doc.babylonjs.com/features/featuresDeepDive/postProcesses/usePostProcesses#refraction
 */
export class RefractionPostProcess extends PostProcess {
    private _refTexture;
    private _ownRefractionTexture;
    /** the base color of the refraction (used to taint the rendering) */
    color: Color3;
    /** simulated refraction depth */
    depth: number;
    /** the coefficient of the base color (0 to remove base color tainting) */
    colorLevel: number;
    /** Gets the url used to load the refraction texture */
    refractionTextureUrl: string;
    /**
     * Gets or sets the refraction texture
     * Please note that you are responsible for disposing the texture if you set it manually
     */
    get refractionTexture(): Texture;
    set refractionTexture(value: Texture);
    /**
     * Gets a string identifying the name of the class
     * @returns "RefractionPostProcess" string
     */
    getClassName(): string;
    /**
     * Initializes the RefractionPostProcess
     * @see https://doc.babylonjs.com/features/featuresDeepDive/postProcesses/usePostProcesses#refraction
     * @param name The name of the effect.
     * @param refractionTextureUrl Url of the refraction texture to use
     * @param color the base color of the refraction (used to taint the rendering)
     * @param depth simulated refraction depth
     * @param colorLevel the coefficient of the base color (0 to remove base color tainting)
     * @param options The required width/height ratio to downsize to before computing the render pass.
     * @param camera The camera to apply the render pass to.
     * @param samplingMode The sampling mode to be used when computing the pass. (default: 0)
     * @param engine The engine which the post process will be applied. (default: current engine)
     * @param reusable If the post process can be reused on the same frame. (default: false)
     */
    constructor(name: string, refractionTextureUrl: string, color: Color3, depth: number, colorLevel: number, options: number | PostProcessOptions, camera: Nullable<Camera>, samplingMode?: number, engine?: Engine, reusable?: boolean);
    /**
     * Disposes of the post process
     * @param camera Camera to dispose post process on
     */
    dispose(camera: Camera): void;
    /**
     * @internal
     */
    static _Parse(parsedPostProcess: any, targetCamera: Camera, scene: Scene, rootUrl: string): RefractionPostProcess;
}
export {};

}
declare module "babylonjs/PostProcesses/RenderPipeline/index" {
export * from "babylonjs/PostProcesses/RenderPipeline/Pipelines/index";
export * from "babylonjs/PostProcesses/RenderPipeline/postProcessRenderEffect";
export * from "babylonjs/PostProcesses/RenderPipeline/postProcessRenderPipeline";
export * from "babylonjs/PostProcesses/RenderPipeline/postProcessRenderPipelineManager";
export * from "babylonjs/PostProcesses/RenderPipeline/postProcessRenderPipelineManagerSceneComponent";

}
declare module "babylonjs/PostProcesses/RenderPipeline/Pipelines/defaultRenderingPipeline" {
import { Nullable } from "babylonjs/types";
import { Observable } from "babylonjs/Misc/observable";
import { IAnimatable } from "babylonjs/Animations/animatable.interface";
import { Camera } from "babylonjs/Cameras/camera";
import { IDisposable, Scene } from "babylonjs/scene";
import { GlowLayer } from "babylonjs/Layers/glowLayer";
import { SharpenPostProcess } from "babylonjs/PostProcesses/sharpenPostProcess";
import { ImageProcessingPostProcess } from "babylonjs/PostProcesses/imageProcessingPostProcess";
import { ChromaticAberrationPostProcess } from "babylonjs/PostProcesses/chromaticAberrationPostProcess";
import { GrainPostProcess } from "babylonjs/PostProcesses/grainPostProcess";
import { FxaaPostProcess } from "babylonjs/PostProcesses/fxaaPostProcess";
import { PostProcessRenderPipeline } from "babylonjs/PostProcesses/RenderPipeline/postProcessRenderPipeline";
import { DepthOfFieldEffect, DepthOfFieldEffectBlurLevel } from "babylonjs/PostProcesses/depthOfFieldEffect";
import "babylonjs/PostProcesses/RenderPipeline/postProcessRenderPipelineManagerSceneComponent";
import { Animation } from "babylonjs/Animations/animation";
/**
 * The default rendering pipeline can be added to a scene to apply common post processing effects such as anti-aliasing or depth of field.
 * See https://doc.babylonjs.com/features/featuresDeepDive/postProcesses/defaultRenderingPipeline
 */
export class DefaultRenderingPipeline extends PostProcessRenderPipeline implements IDisposable, IAnimatable {
    private _scene;
    private _camerasToBeAttached;
    /**
     * ID of the sharpen post process,
     */
    private readonly SharpenPostProcessId;
    /**
     * @ignore
     * ID of the image processing post process;
     */
    readonly ImageProcessingPostProcessId: string;
    /**
     * @ignore
     * ID of the Fast Approximate Anti-Aliasing post process;
     */
    readonly FxaaPostProcessId: string;
    /**
     * ID of the chromatic aberration post process,
     */
    private readonly ChromaticAberrationPostProcessId;
    /**
     * ID of the grain post process
     */
    private readonly GrainPostProcessId;
    /**
     * Sharpen post process which will apply a sharpen convolution to enhance edges
     */
    sharpen: SharpenPostProcess;
    private _sharpenEffect;
    private bloom;
    /**
     * Depth of field effect, applies a blur based on how far away objects are from the focus distance.
     */
    depthOfField: DepthOfFieldEffect;
    /**
     * The Fast Approximate Anti-Aliasing post process which attempts to remove aliasing from an image.
     */
    fxaa: FxaaPostProcess;
    /**
     * Image post processing pass used to perform operations such as tone mapping or color grading.
     */
    imageProcessing: ImageProcessingPostProcess;
    /**
     * Chromatic aberration post process which will shift rgb colors in the image
     */
    chromaticAberration: ChromaticAberrationPostProcess;
    private _chromaticAberrationEffect;
    /**
     * Grain post process which add noise to the image
     */
    grain: GrainPostProcess;
    private _grainEffect;
    /**
     * Glow post process which adds a glow to emissive areas of the image
     */
    private _glowLayer;
    /**
     * Animations which can be used to tweak settings over a period of time
     */
    animations: Animation[];
    private _imageProcessingConfigurationObserver;
    private _sharpenEnabled;
    private _bloomEnabled;
    private _depthOfFieldEnabled;
    private _depthOfFieldBlurLevel;
    private _fxaaEnabled;
    private _imageProcessingEnabled;
    private _defaultPipelineTextureType;
    private _bloomScale;
    private _chromaticAberrationEnabled;
    private _grainEnabled;
    private _buildAllowed;
    /**
     * Enable or disable automatic building of the pipeline when effects are enabled and disabled.
     * If false, you will have to manually call prepare() to update the pipeline.
     */
    get automaticBuild(): boolean;
    set automaticBuild(value: boolean);
    /**
     * This is triggered each time the pipeline has been built.
     */
    onBuildObservable: Observable<DefaultRenderingPipeline>;
    /**
     * Gets active scene
     */
    get scene(): Scene;
    /**
     * Enable or disable the sharpen process from the pipeline
     */
    set sharpenEnabled(enabled: boolean);
    get sharpenEnabled(): boolean;
    private _resizeObserver;
    private _hardwareScaleLevel;
    private _bloomKernel;
    /**
     * Specifies the size of the bloom blur kernel, relative to the final output size
     */
    get bloomKernel(): number;
    set bloomKernel(value: number);
    /**
     * Specifies the weight of the bloom in the final rendering
     */
    private _bloomWeight;
    /**
     * Specifies the luma threshold for the area that will be blurred by the bloom
     */
    private _bloomThreshold;
    private _hdr;
    /**
     * The strength of the bloom.
     */
    set bloomWeight(value: number);
    get bloomWeight(): number;
    /**
     * The luminance threshold to find bright areas of the image to bloom.
     */
    set bloomThreshold(value: number);
    get bloomThreshold(): number;
    /**
     * The scale of the bloom, lower value will provide better performance.
     */
    set bloomScale(value: number);
    get bloomScale(): number;
    /**
     * Enable or disable the bloom from the pipeline
     */
    set bloomEnabled(enabled: boolean);
    get bloomEnabled(): boolean;
    private _rebuildBloom;
    /**
     * If the depth of field is enabled.
     */
    get depthOfFieldEnabled(): boolean;
    set depthOfFieldEnabled(enabled: boolean);
    /**
     * Blur level of the depth of field effect. (Higher blur will effect performance)
     */
    get depthOfFieldBlurLevel(): DepthOfFieldEffectBlurLevel;
    set depthOfFieldBlurLevel(value: DepthOfFieldEffectBlurLevel);
    /**
     * If the anti aliasing is enabled.
     */
    set fxaaEnabled(enabled: boolean);
    get fxaaEnabled(): boolean;
    private _samples;
    /**
     * MSAA sample count, setting this to 4 will provide 4x anti aliasing. (default: 1)
     */
    set samples(sampleCount: number);
    get samples(): number;
    /**
     * If image processing is enabled.
     */
    set imageProcessingEnabled(enabled: boolean);
    get imageProcessingEnabled(): boolean;
    /**
     * If glow layer is enabled. (Adds a glow effect to emmissive materials)
     */
    set glowLayerEnabled(enabled: boolean);
    get glowLayerEnabled(): boolean;
    /**
     * Gets the glow layer (or null if not defined)
     */
    get glowLayer(): Nullable<GlowLayer>;
    /**
     * Enable or disable the chromaticAberration process from the pipeline
     */
    set chromaticAberrationEnabled(enabled: boolean);
    get chromaticAberrationEnabled(): boolean;
    /**
     * Enable or disable the grain process from the pipeline
     */
    set grainEnabled(enabled: boolean);
    get grainEnabled(): boolean;
    /**
     * Instantiates a DefaultRenderingPipeline.
     * @param name The rendering pipeline name (default: "")
     * @param hdr If high dynamic range textures should be used (default: true)
     * @param scene The scene linked to this pipeline (default: the last created scene)
     * @param cameras The array of cameras that the rendering pipeline will be attached to (default: scene.cameras)
     * @param automaticBuild If false, you will have to manually call prepare() to update the pipeline (default: true)
     */
    constructor(name?: string, hdr?: boolean, scene?: Scene, cameras?: Camera[], automaticBuild?: boolean);
    /**
     * Get the class name
     * @returns "DefaultRenderingPipeline"
     */
    getClassName(): string;
    /**
     * Force the compilation of the entire pipeline.
     */
    prepare(): void;
    private _hasCleared;
    private _prevPostProcess;
    private _prevPrevPostProcess;
    private _setAutoClearAndTextureSharing;
    private _depthOfFieldSceneObserver;
    private _activeCameraChangedObserver;
    private _activeCamerasChangedObserver;
    private _buildPipeline;
    private _disposePostProcesses;
    /**
     * Adds a camera to the pipeline
     * @param camera the camera to be added
     */
    addCamera(camera: Camera): void;
    /**
     * Removes a camera from the pipeline
     * @param camera the camera to remove
     */
    removeCamera(camera: Camera): void;
    /**
     * Dispose of the pipeline and stop all post processes
     */
    dispose(): void;
    /**
     * Serialize the rendering pipeline (Used when exporting)
     * @returns the serialized object
     */
    serialize(): any;
    /**
     * Parse the serialized pipeline
     * @param source Source pipeline.
     * @param scene The scene to load the pipeline to.
     * @param rootUrl The URL of the serialized pipeline.
     * @returns An instantiated pipeline from the serialized object.
     */
    static Parse(source: any, scene: Scene, rootUrl: string): DefaultRenderingPipeline;
}
export {};

}
declare module "babylonjs/PostProcesses/RenderPipeline/Pipelines/index" {
export * from "babylonjs/PostProcesses/RenderPipeline/Pipelines/defaultRenderingPipeline";
export * from "babylonjs/PostProcesses/RenderPipeline/Pipelines/lensRenderingPipeline";
export * from "babylonjs/PostProcesses/RenderPipeline/Pipelines/ssao2RenderingPipeline";
export * from "babylonjs/PostProcesses/RenderPipeline/Pipelines/ssaoRenderingPipeline";
export * from "babylonjs/PostProcesses/RenderPipeline/Pipelines/standardRenderingPipeline";

}
declare module "babylonjs/PostProcesses/RenderPipeline/Pipelines/lensRenderingPipeline" {
import { Camera } from "babylonjs/Cameras/camera";
import { PostProcessRenderPipeline } from "babylonjs/PostProcesses/RenderPipeline/postProcessRenderPipeline";
import { Scene } from "babylonjs/scene";
import "babylonjs/PostProcesses/RenderPipeline/postProcessRenderPipelineManagerSceneComponent";
import "babylonjs/Shaders/chromaticAberration.fragment";
import "babylonjs/Shaders/lensHighlights.fragment";
import "babylonjs/Shaders/depthOfField.fragment";
/**
 * BABYLON.JS Chromatic Aberration GLSL Shader
 * Author: Olivier Guyot
 * Separates very slightly R, G and B colors on the edges of the screen
 * Inspired by Francois Tarlier & Martins Upitis
 */
export class LensRenderingPipeline extends PostProcessRenderPipeline {
    /**
     * @ignore
     * The chromatic aberration PostProcess id in the pipeline
     */
    LensChromaticAberrationEffect: string;
    /**
     * @ignore
     * The highlights enhancing PostProcess id in the pipeline
     */
    HighlightsEnhancingEffect: string;
    /**
     * @ignore
     * The depth-of-field PostProcess id in the pipeline
     */
    LensDepthOfFieldEffect: string;
    private _scene;
    private _depthTexture;
    private _grainTexture;
    private _chromaticAberrationPostProcess;
    private _highlightsPostProcess;
    private _depthOfFieldPostProcess;
    private _edgeBlur;
    private _grainAmount;
    private _chromaticAberration;
    private _distortion;
    private _highlightsGain;
    private _highlightsThreshold;
    private _dofDistance;
    private _dofAperture;
    private _dofDarken;
    private _dofPentagon;
    private _blurNoise;
    /**
     * @constructor
     *
     * Effect parameters are as follow:
     * {
     *      chromatic_aberration: number;       // from 0 to x (1 for realism)
     *      edge_blur: number;                  // from 0 to x (1 for realism)
     *      distortion: number;                 // from 0 to x (1 for realism), note that this will effect the pointer position precision
     *      grain_amount: number;               // from 0 to 1
     *      grain_texture: BABYLON.Texture;     // texture to use for grain effect; if unset, use random B&W noise
     *      dof_focus_distance: number;         // depth-of-field: focus distance; unset to disable (disabled by default)
     *      dof_aperture: number;               // depth-of-field: focus blur bias (default: 1)
     *      dof_darken: number;                 // depth-of-field: darken that which is out of focus (from 0 to 1, disabled by default)
     *      dof_pentagon: boolean;              // depth-of-field: makes a pentagon-like "bokeh" effect
     *      dof_gain: number;                   // depth-of-field: highlights gain; unset to disable (disabled by default)
     *      dof_threshold: number;              // depth-of-field: highlights threshold (default: 1)
     *      blur_noise: boolean;                // add a little bit of noise to the blur (default: true)
     * }
     * Note: if an effect parameter is unset, effect is disabled
     *
     * @param name The rendering pipeline name
     * @param parameters - An object containing all parameters (see above)
     * @param scene The scene linked to this pipeline
     * @param ratio The size of the postprocesses (0.5 means that your postprocess will have a width = canvas.width 0.5 and a height = canvas.height 0.5)
     * @param cameras The array of cameras that the rendering pipeline will be attached to
     */
    constructor(name: string, parameters: any, scene: Scene, ratio?: number, cameras?: Camera[]);
    /**
     * Get the class name
     * @returns "LensRenderingPipeline"
     */
    getClassName(): string;
    /**
     * Gets associated scene
     */
    get scene(): Scene;
    /**
     * Gets or sets the edge blur
     */
    get edgeBlur(): number;
    set edgeBlur(value: number);
    /**
     * Gets or sets the grain amount
     */
    get grainAmount(): number;
    set grainAmount(value: number);
    /**
     * Gets or sets the chromatic aberration amount
     */
    get chromaticAberration(): number;
    set chromaticAberration(value: number);
    /**
     * Gets or sets the depth of field aperture
     */
    get dofAperture(): number;
    set dofAperture(value: number);
    /**
     * Gets or sets the edge distortion
     */
    get edgeDistortion(): number;
    set edgeDistortion(value: number);
    /**
     * Gets or sets the depth of field distortion
     */
    get dofDistortion(): number;
    set dofDistortion(value: number);
    /**
     * Gets or sets the darken out of focus amount
     */
    get darkenOutOfFocus(): number;
    set darkenOutOfFocus(value: number);
    /**
     * Gets or sets a boolean indicating if blur noise is enabled
     */
    get blurNoise(): boolean;
    set blurNoise(value: boolean);
    /**
     * Gets or sets a boolean indicating if pentagon bokeh is enabled
     */
    get pentagonBokeh(): boolean;
    set pentagonBokeh(value: boolean);
    /**
     * Gets or sets the highlight grain amount
     */
    get highlightsGain(): number;
    set highlightsGain(value: number);
    /**
     * Gets or sets the highlight threshold
     */
    get highlightsThreshold(): number;
    set highlightsThreshold(value: number);
    /**
     * Sets the amount of blur at the edges
     * @param amount blur amount
     */
    setEdgeBlur(amount: number): void;
    /**
     * Sets edge blur to 0
     */
    disableEdgeBlur(): void;
    /**
     * Sets the amount of grain
     * @param amount Amount of grain
     */
    setGrainAmount(amount: number): void;
    /**
     * Set grain amount to 0
     */
    disableGrain(): void;
    /**
     * Sets the chromatic aberration amount
     * @param amount amount of chromatic aberration
     */
    setChromaticAberration(amount: number): void;
    /**
     * Sets chromatic aberration amount to 0
     */
    disableChromaticAberration(): void;
    /**
     * Sets the EdgeDistortion amount
     * @param amount amount of EdgeDistortion
     */
    setEdgeDistortion(amount: number): void;
    /**
     * Sets edge distortion to 0
     */
    disableEdgeDistortion(): void;
    /**
     * Sets the FocusDistance amount
     * @param amount amount of FocusDistance
     */
    setFocusDistance(amount: number): void;
    /**
     * Disables depth of field
     */
    disableDepthOfField(): void;
    /**
     * Sets the Aperture amount
     * @param amount amount of Aperture
     */
    setAperture(amount: number): void;
    /**
     * Sets the DarkenOutOfFocus amount
     * @param amount amount of DarkenOutOfFocus
     */
    setDarkenOutOfFocus(amount: number): void;
    private _pentagonBokehIsEnabled;
    /**
     * Creates a pentagon bokeh effect
     */
    enablePentagonBokeh(): void;
    /**
     * Disables the pentagon bokeh effect
     */
    disablePentagonBokeh(): void;
    /**
     * Enables noise blur
     */
    enableNoiseBlur(): void;
    /**
     * Disables noise blur
     */
    disableNoiseBlur(): void;
    /**
     * Sets the HighlightsGain amount
     * @param amount amount of HighlightsGain
     */
    setHighlightsGain(amount: number): void;
    /**
     * Sets the HighlightsThreshold amount
     * @param amount amount of HighlightsThreshold
     */
    setHighlightsThreshold(amount: number): void;
    /**
     * Disables highlights
     */
    disableHighlights(): void;
    /**
     * Removes the internal pipeline assets and detaches the pipeline from the scene cameras
     * @param disableDepthRender If the scene's depth rendering should be disabled (default: false)
     */
    dispose(disableDepthRender?: boolean): void;
    private _createChromaticAberrationPostProcess;
    private _createHighlightsPostProcess;
    private _createDepthOfFieldPostProcess;
    private _createGrainTexture;
}

}
declare module "babylonjs/PostProcesses/RenderPipeline/Pipelines/ssao2RenderingPipeline" {
import { Camera } from "babylonjs/Cameras/camera";
import { PostProcessRenderPipeline } from "babylonjs/PostProcesses/RenderPipeline/postProcessRenderPipeline";
import { Scene } from "babylonjs/scene";
import "babylonjs/PostProcesses/RenderPipeline/postProcessRenderPipelineManagerSceneComponent";
import "babylonjs/Shaders/ssao2.fragment";
import "babylonjs/Shaders/ssaoCombine.fragment";
/**
 * Render pipeline to produce ssao effect
 */
export class SSAO2RenderingPipeline extends PostProcessRenderPipeline {
    /**
     * @ignore
     * The PassPostProcess id in the pipeline that contains the original scene color
     */
    SSAOOriginalSceneColorEffect: string;
    /**
     * @ignore
     * The SSAO PostProcess id in the pipeline
     */
    SSAORenderEffect: string;
    /**
     * @ignore
     * The horizontal blur PostProcess id in the pipeline
     */
    SSAOBlurHRenderEffect: string;
    /**
     * @ignore
     * The vertical blur PostProcess id in the pipeline
     */
    SSAOBlurVRenderEffect: string;
    /**
     * @ignore
     * The PostProcess id in the pipeline that combines the SSAO-Blur output with the original scene color (SSAOOriginalSceneColorEffect)
     */
    SSAOCombineRenderEffect: string;
    /**
     * The output strength of the SSAO post-process. Default value is 1.0.
     */
    totalStrength: number;
    /**
     * Maximum depth value to still render AO. A smooth falloff makes the dimming more natural, so there will be no abrupt shading change.
     */
    maxZ: number;
    /**
     * In order to save performances, SSAO radius is clamped on close geometry. This ratio changes by how much
     */
    minZAspect: number;
    private _samples;
    /**
     * Number of samples used for the SSAO calculations. Default value is 8
     */
    set samples(n: number);
    get samples(): number;
    private _textureSamples;
    /**
     * Number of samples to use for antialiasing
     */
    set textureSamples(n: number);
    get textureSamples(): number;
    /**
     * Force rendering the geometry through geometry buffer
     */
    private _forceGeometryBuffer;
    private get _geometryBufferRenderer();
    private get _prePassRenderer();
    /**
     * Ratio object used for SSAO ratio and blur ratio
     */
    private _ratio;
    /**
     * Dynamically generated sphere sampler.
     */
    private _sampleSphere;
    /**
     * Blur filter offsets
     */
    private _samplerOffsets;
    private _expensiveBlur;
    /**
     * If bilateral blur should be used
     */
    set expensiveBlur(b: boolean);
    get expensiveBlur(): boolean;
    /**
     * The radius around the analyzed pixel used by the SSAO post-process. Default value is 2.0
     */
    radius: number;
    /**
     * The base color of the SSAO post-process
     * The final result is "base + ssao" between [0, 1]
     */
    base: number;
    /**
     *  Support test.
     */
    static get IsSupported(): boolean;
    private _scene;
    private _randomTexture;
    private _originalColorPostProcess;
    private _ssaoPostProcess;
    private _blurHPostProcess;
    private _blurVPostProcess;
    private _ssaoCombinePostProcess;
    /**
     * Gets active scene
     */
    get scene(): Scene;
    /**
     * @constructor
     * @param name The rendering pipeline name
     * @param scene The scene linked to this pipeline
     * @param ratio The size of the postprocesses. Can be a number shared between passes or an object for more precision: { ssaoRatio: 0.5, blurRatio: 1.0 }
     * @param cameras The array of cameras that the rendering pipeline will be attached to
     * @param forceGeometryBuffer Set to true if you want to use the legacy geometry buffer renderer
     * @param textureType The texture type used by the different post processes created by SSAO (default: Constants.TEXTURETYPE_UNSIGNED_INT)
     */
    constructor(name: string, scene: Scene, ratio: any, cameras?: Camera[], forceGeometryBuffer?: boolean, textureType?: number);
    /**
     * Get the class name
     * @returns "SSAO2RenderingPipeline"
     */
    getClassName(): string;
    /**
     * Removes the internal pipeline assets and detaches the pipeline from the scene cameras
     * @param disableGeometryBufferRenderer
     */
    dispose(disableGeometryBufferRenderer?: boolean): void;
    private _createBlurPostProcess;
    /** @internal */
    _rebuild(): void;
    private _bits;
    private _radicalInverse_VdC;
    private _hammersley;
    private _hemisphereSample_uniform;
    private _generateHemisphere;
    private _getDefinesForSSAO;
    private static readonly ORTHO_DEPTH_PROJECTION;
    private static readonly PERSPECTIVE_DEPTH_PROJECTION;
    private _createSSAOPostProcess;
    private _createSSAOCombinePostProcess;
    private _createRandomTexture;
    /**
     * Serialize the rendering pipeline (Used when exporting)
     * @returns the serialized object
     */
    serialize(): any;
    /**
     * Parse the serialized pipeline
     * @param source Source pipeline.
     * @param scene The scene to load the pipeline to.
     * @param rootUrl The URL of the serialized pipeline.
     * @returns An instantiated pipeline from the serialized object.
     */
    static Parse(source: any, scene: Scene, rootUrl: string): SSAO2RenderingPipeline;
}

}
declare module "babylonjs/PostProcesses/RenderPipeline/Pipelines/ssaoRenderingPipeline" {
import { Camera } from "babylonjs/Cameras/camera";
import { PostProcessRenderPipeline } from "babylonjs/PostProcesses/RenderPipeline/postProcessRenderPipeline";
import { Scene } from "babylonjs/scene";
import "babylonjs/PostProcesses/RenderPipeline/postProcessRenderPipelineManagerSceneComponent";
import "babylonjs/Shaders/ssao.fragment";
import "babylonjs/Shaders/ssaoCombine.fragment";
/**
 * Render pipeline to produce ssao effect
 */
export class SSAORenderingPipeline extends PostProcessRenderPipeline {
    /**
     * @ignore
     * The PassPostProcess id in the pipeline that contains the original scene color
     */
    SSAOOriginalSceneColorEffect: string;
    /**
     * @ignore
     * The SSAO PostProcess id in the pipeline
     */
    SSAORenderEffect: string;
    /**
     * @ignore
     * The horizontal blur PostProcess id in the pipeline
     */
    SSAOBlurHRenderEffect: string;
    /**
     * @ignore
     * The vertical blur PostProcess id in the pipeline
     */
    SSAOBlurVRenderEffect: string;
    /**
     * @ignore
     * The PostProcess id in the pipeline that combines the SSAO-Blur output with the original scene color (SSAOOriginalSceneColorEffect)
     */
    SSAOCombineRenderEffect: string;
    /**
     * The output strength of the SSAO post-process. Default value is 1.0.
     */
    totalStrength: number;
    /**
     * The radius around the analyzed pixel used by the SSAO post-process. Default value is 0.0006
     */
    radius: number;
    /**
     * Related to fallOff, used to interpolate SSAO samples (first interpolate function input) based on the occlusion difference of each pixel
     * Must not be equal to fallOff and superior to fallOff.
     * Default value is 0.0075
     */
    area: number;
    /**
     * Related to area, used to interpolate SSAO samples (second interpolate function input) based on the occlusion difference of each pixel
     * Must not be equal to area and inferior to area.
     * Default value is 0.000001
     */
    fallOff: number;
    /**
     * The base color of the SSAO post-process
     * The final result is "base + ssao" between [0, 1]
     */
    base: number;
    private _scene;
    private _randomTexture;
    private _originalColorPostProcess;
    private _ssaoPostProcess;
    private _blurHPostProcess;
    private _blurVPostProcess;
    private _ssaoCombinePostProcess;
    private _firstUpdate;
    /**
     * Gets active scene
     */
    get scene(): Scene;
    /**
     * @constructor
     * @param name - The rendering pipeline name
     * @param scene - The scene linked to this pipeline
     * @param ratio - The size of the postprocesses. Can be a number shared between passes or an object for more precision: { ssaoRatio: 0.5, combineRatio: 1.0 }
     * @param cameras - The array of cameras that the rendering pipeline will be attached to
     */
    constructor(name: string, scene: Scene, ratio: any, cameras?: Camera[]);
    /**
     * @internal
     */
    _attachCameras(cameras: any, unique: boolean): void;
    /**
     * Get the class name
     * @returns "SSAORenderingPipeline"
     */
    getClassName(): string;
    /**
     * Removes the internal pipeline assets and detaches the pipeline from the scene cameras
     * @param disableDepthRender
     */
    dispose(disableDepthRender?: boolean): void;
    private _createBlurPostProcess;
    /** @internal */
    _rebuild(): void;
    private _createSSAOPostProcess;
    private _createSSAOCombinePostProcess;
    private _createRandomTexture;
}

}
declare module "babylonjs/PostProcesses/RenderPipeline/Pipelines/standardRenderingPipeline" {
import { Nullable } from "babylonjs/types";
import { IAnimatable } from "babylonjs/Animations/animatable.interface";
import { Camera } from "babylonjs/Cameras/camera";
import { Texture } from "babylonjs/Materials/Textures/texture";
import { PostProcess } from "babylonjs/PostProcesses/postProcess";
import { PostProcessRenderPipeline } from "babylonjs/PostProcesses/RenderPipeline/postProcessRenderPipeline";
import { BlurPostProcess } from "babylonjs/PostProcesses/blurPostProcess";
import { FxaaPostProcess } from "babylonjs/PostProcesses/fxaaPostProcess";
import { IDisposable, Scene } from "babylonjs/scene";
import { SpotLight } from "babylonjs/Lights/spotLight";
import { DirectionalLight } from "babylonjs/Lights/directionalLight";
import { ScreenSpaceReflectionPostProcess } from "babylonjs/PostProcesses/screenSpaceReflectionPostProcess";
import { Animation } from "babylonjs/Animations/animation";
import "babylonjs/PostProcesses/RenderPipeline/postProcessRenderPipelineManagerSceneComponent";
import "babylonjs/Shaders/standard.fragment";
/**
 * Standard rendering pipeline
 * Default pipeline should be used going forward but the standard pipeline will be kept for backwards compatibility.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/postProcesses/standardRenderingPipeline
 */
export class StandardRenderingPipeline extends PostProcessRenderPipeline implements IDisposable, IAnimatable {
    /**
     * Public members
     */
    /**
     * Post-process which contains the original scene color before the pipeline applies all the effects
     */
    originalPostProcess: Nullable<PostProcess>;
    /**
     * Post-process used to down scale an image x4
     */
    downSampleX4PostProcess: Nullable<PostProcess>;
    /**
     * Post-process used to calculate the illuminated surfaces controlled by a threshold
     */
    brightPassPostProcess: Nullable<PostProcess>;
    /**
     * Post-process array storing all the horizontal blur post-processes used by the pipeline
     */
    blurHPostProcesses: PostProcess[];
    /**
     * Post-process array storing all the vertical blur post-processes used by the pipeline
     */
    blurVPostProcesses: PostProcess[];
    /**
     * Post-process used to add colors of 2 textures (typically brightness + real scene color)
     */
    textureAdderPostProcess: Nullable<PostProcess>;
    /**
     * Post-process used to create volumetric lighting effect
     */
    volumetricLightPostProcess: Nullable<PostProcess>;
    /**
     * Post-process used to smooth the previous volumetric light post-process on the X axis
     */
    volumetricLightSmoothXPostProcess: Nullable<BlurPostProcess>;
    /**
     * Post-process used to smooth the previous volumetric light post-process on the Y axis
     */
    volumetricLightSmoothYPostProcess: Nullable<BlurPostProcess>;
    /**
     * Post-process used to merge the volumetric light effect and the real scene color
     */
    volumetricLightMergePostProces: Nullable<PostProcess>;
    /**
     * Post-process used to store the final volumetric light post-process (attach/detach for debug purpose)
     */
    volumetricLightFinalPostProcess: Nullable<PostProcess>;
    /**
     * Base post-process used to calculate the average luminance of the final image for HDR
     */
    luminancePostProcess: Nullable<PostProcess>;
    /**
     * Post-processes used to create down sample post-processes in order to get
     * the average luminance of the final image for HDR
     * Array of length "StandardRenderingPipeline.LuminanceSteps"
     */
    luminanceDownSamplePostProcesses: PostProcess[];
    /**
     * Post-process used to create a HDR effect (light adaptation)
     */
    hdrPostProcess: Nullable<PostProcess>;
    /**
     * Post-process used to store the final texture adder post-process (attach/detach for debug purpose)
     */
    textureAdderFinalPostProcess: Nullable<PostProcess>;
    /**
     * Post-process used to store the final lens flare post-process (attach/detach for debug purpose)
     */
    lensFlareFinalPostProcess: Nullable<PostProcess>;
    /**
     * Post-process used to merge the final HDR post-process and the real scene color
     */
    hdrFinalPostProcess: Nullable<PostProcess>;
    /**
     * Post-process used to create a lens flare effect
     */
    lensFlarePostProcess: Nullable<PostProcess>;
    /**
     * Post-process that merges the result of the lens flare post-process and the real scene color
     */
    lensFlareComposePostProcess: Nullable<PostProcess>;
    /**
     * Post-process used to create a motion blur effect
     */
    motionBlurPostProcess: Nullable<PostProcess>;
    /**
     * Post-process used to create a depth of field effect
     */
    depthOfFieldPostProcess: Nullable<PostProcess>;
    /**
     * The Fast Approximate Anti-Aliasing post process which attempts to remove aliasing from an image.
     */
    fxaaPostProcess: Nullable<FxaaPostProcess>;
    /**
     * Post-process used to simulate realtime reflections using the screen space and geometry renderer.
     */
    screenSpaceReflectionPostProcess: Nullable<ScreenSpaceReflectionPostProcess>;
    /**
     * Represents the brightness threshold in order to configure the illuminated surfaces
     */
    brightThreshold: number;
    /**
     * Configures the blur intensity used for surexposed surfaces are highlighted surfaces (light halo)
     */
    blurWidth: number;
    /**
     * Sets if the blur for highlighted surfaces must be only horizontal
     */
    horizontalBlur: boolean;
    /**
     * Gets the overall exposure used by the pipeline
     */
    get exposure(): number;
    /**
     * Sets the overall exposure used by the pipeline
     */
    set exposure(value: number);
    /**
     * Texture used typically to simulate "dirty" on camera lens
     */
    lensTexture: Nullable<Texture>;
    /**
     * Represents the offset coefficient based on Rayleigh principle. Typically in interval [-0.2, 0.2]
     */
    volumetricLightCoefficient: number;
    /**
     * The overall power of volumetric lights, typically in interval [0, 10] maximum
     */
    volumetricLightPower: number;
    /**
     * Used the set the blur intensity to smooth the volumetric lights
     */
    volumetricLightBlurScale: number;
    /**
     * Light (spot or directional) used to generate the volumetric lights rays
     * The source light must have a shadow generate so the pipeline can get its
     * depth map
     */
    sourceLight: Nullable<SpotLight | DirectionalLight>;
    /**
     * For eye adaptation, represents the minimum luminance the eye can see
     */
    hdrMinimumLuminance: number;
    /**
     * For eye adaptation, represents the decrease luminance speed
     */
    hdrDecreaseRate: number;
    /**
     * For eye adaptation, represents the increase luminance speed
     */
    hdrIncreaseRate: number;
    /**
     * Gets whether or not the exposure of the overall pipeline should be automatically adjusted by the HDR post-process
     */
    get hdrAutoExposure(): boolean;
    /**
     * Sets whether or not the exposure of the overall pipeline should be automatically adjusted by the HDR post-process
     */
    set hdrAutoExposure(value: boolean);
    /**
     * Lens color texture used by the lens flare effect. Mandatory if lens flare effect enabled
     */
    lensColorTexture: Nullable<Texture>;
    /**
     * The overall strength for the lens flare effect
     */
    lensFlareStrength: number;
    /**
     * Dispersion coefficient for lens flare ghosts
     */
    lensFlareGhostDispersal: number;
    /**
     * Main lens flare halo width
     */
    lensFlareHaloWidth: number;
    /**
     * Based on the lens distortion effect, defines how much the lens flare result
     * is distorted
     */
    lensFlareDistortionStrength: number;
    /**
     * Configures the blur intensity used for for lens flare (halo)
     */
    lensFlareBlurWidth: number;
    /**
     * Lens star texture must be used to simulate rays on the flares and is available
     * in the documentation
     */
    lensStarTexture: Nullable<Texture>;
    /**
     * As the "lensTexture" (can be the same texture or different), it is used to apply the lens
     * flare effect by taking account of the dirt texture
     */
    lensFlareDirtTexture: Nullable<Texture>;
    /**
     * Represents the focal length for the depth of field effect
     */
    depthOfFieldDistance: number;
    /**
     * Represents the blur intensity for the blurred part of the depth of field effect
     */
    depthOfFieldBlurWidth: number;
    /**
     * Gets how much the image is blurred by the movement while using the motion blur post-process
     */
    get motionStrength(): number;
    /**
     * Sets how much the image is blurred by the movement while using the motion blur post-process
     */
    set motionStrength(strength: number);
    /**
     * Gets whether or not the motion blur post-process is object based or screen based.
     */
    get objectBasedMotionBlur(): boolean;
    /**
     * Sets whether or not the motion blur post-process should be object based or screen based
     */
    set objectBasedMotionBlur(value: boolean);
    /**
     * List of animations for the pipeline (IAnimatable implementation)
     */
    animations: Animation[];
    /**
     * Private members
     */
    private _scene;
    private _currentDepthOfFieldSource;
    private _basePostProcess;
    private _fixedExposure;
    private _currentExposure;
    private _hdrAutoExposure;
    private _hdrCurrentLuminance;
    private _motionStrength;
    private _isObjectBasedMotionBlur;
    private _floatTextureType;
    private _camerasToBeAttached;
    private _ratio;
    private _bloomEnabled;
    private _depthOfFieldEnabled;
    private _vlsEnabled;
    private _lensFlareEnabled;
    private _hdrEnabled;
    private _motionBlurEnabled;
    private _fxaaEnabled;
    private _screenSpaceReflectionsEnabled;
    private _motionBlurSamples;
    private _volumetricLightStepsCount;
    private _samples;
    /**
     * @ignore
     * Specifies if the bloom pipeline is enabled
     */
    get BloomEnabled(): boolean;
    set BloomEnabled(enabled: boolean);
    /**
     * @ignore
     * Specifies if the depth of field pipeline is enabled
     */
    get DepthOfFieldEnabled(): boolean;
    set DepthOfFieldEnabled(enabled: boolean);
    /**
     * @ignore
     * Specifies if the lens flare pipeline is enabled
     */
    get LensFlareEnabled(): boolean;
    set LensFlareEnabled(enabled: boolean);
    /**
     * @ignore
     * Specifies if the HDR pipeline is enabled
     */
    get HDREnabled(): boolean;
    set HDREnabled(enabled: boolean);
    /**
     * @ignore
     * Specifies if the volumetric lights scattering effect is enabled
     */
    get VLSEnabled(): boolean;
    set VLSEnabled(enabled: boolean);
    /**
     * @ignore
     * Specifies if the motion blur effect is enabled
     */
    get MotionBlurEnabled(): boolean;
    set MotionBlurEnabled(enabled: boolean);
    /**
     * Specifies if anti-aliasing is enabled
     */
    get fxaaEnabled(): boolean;
    set fxaaEnabled(enabled: boolean);
    /**
     * Specifies if screen space reflections are enabled.
     */
    get screenSpaceReflectionsEnabled(): boolean;
    set screenSpaceReflectionsEnabled(enabled: boolean);
    /**
     * Specifies the number of steps used to calculate the volumetric lights
     * Typically in interval [50, 200]
     */
    get volumetricLightStepsCount(): number;
    set volumetricLightStepsCount(count: number);
    /**
     * Specifies the number of samples used for the motion blur effect
     * Typically in interval [16, 64]
     */
    get motionBlurSamples(): number;
    set motionBlurSamples(samples: number);
    /**
     * Specifies MSAA sample count, setting this to 4 will provide 4x anti aliasing. (default: 1)
     */
    get samples(): number;
    set samples(sampleCount: number);
    /**
     * Default pipeline should be used going forward but the standard pipeline will be kept for backwards compatibility.
     * @constructor
     * @param name The rendering pipeline name
     * @param scene The scene linked to this pipeline
     * @param ratio The size of the postprocesses (0.5 means that your postprocess will have a width = canvas.width 0.5 and a height = canvas.height 0.5)
     * @param originalPostProcess the custom original color post-process. Must be "reusable". Can be null.
     * @param cameras The array of cameras that the rendering pipeline will be attached to
     */
    constructor(name: string, scene: Scene, ratio: number, originalPostProcess?: Nullable<PostProcess>, cameras?: Camera[]);
    private _buildPipeline;
    private _createDownSampleX4PostProcess;
    private _createBrightPassPostProcess;
    private _createBlurPostProcesses;
    private _createTextureAdderPostProcess;
    private _createVolumetricLightPostProcess;
    private _createLuminancePostProcesses;
    private _createHdrPostProcess;
    private _createLensFlarePostProcess;
    private _createDepthOfFieldPostProcess;
    private _createMotionBlurPostProcess;
    private _getDepthTexture;
    private _disposePostProcesses;
    /**
     * Dispose of the pipeline and stop all post processes
     */
    dispose(): void;
    /**
     * Serialize the rendering pipeline (Used when exporting)
     * @returns the serialized object
     */
    serialize(): any;
    /**
     * Parse the serialized pipeline
     * @param source Source pipeline.
     * @param scene The scene to load the pipeline to.
     * @param rootUrl The URL of the serialized pipeline.
     * @returns An instantiated pipeline from the serialized object.
     */
    static Parse(source: any, scene: Scene, rootUrl: string): StandardRenderingPipeline;
    /**
     * Luminance steps
     */
    static LuminanceSteps: number;
}
export {};

}
declare module "babylonjs/PostProcesses/RenderPipeline/postProcessRenderEffect" {
import { Nullable } from "babylonjs/types";
import { Camera } from "babylonjs/Cameras/camera";
import { PostProcess } from "babylonjs/PostProcesses/postProcess";
import { Engine } from "babylonjs/Engines/engine";
/**
 * This represents a set of one or more post processes in Babylon.
 * A post process can be used to apply a shader to a texture after it is rendered.
 * @example https://doc.babylonjs.com/features/featuresDeepDive/postProcesses/postProcessRenderPipeline
 */
export class PostProcessRenderEffect {
    private _postProcesses;
    private _getPostProcesses;
    private _singleInstance;
    private _cameras;
    private _indicesForCamera;
    /**
     * Name of the effect
     * @internal
     */
    _name: string;
    /**
     * Instantiates a post process render effect.
     * A post process can be used to apply a shader to a texture after it is rendered.
     * @param engine The engine the effect is tied to
     * @param name The name of the effect
     * @param getPostProcesses A function that returns a set of post processes which the effect will run in order to be run.
     * @param singleInstance False if this post process can be run on multiple cameras. (default: true)
     */
    constructor(engine: Engine, name: string, getPostProcesses: () => Nullable<PostProcess | Array<PostProcess>>, singleInstance?: boolean);
    /**
     * Checks if all the post processes in the effect are supported.
     */
    get isSupported(): boolean;
    /**
     * Updates the current state of the effect
     * @internal
     */
    _update(): void;
    /**
     * Attaches the effect on cameras
     * @param cameras The camera to attach to.
     * @internal
     */
    _attachCameras(cameras: Camera): void;
    /**
     * Attaches the effect on cameras
     * @param cameras The camera to attach to.
     * @internal
     */
    _attachCameras(cameras: Camera[]): void;
    /**
     * Detaches the effect on cameras
     * @param cameras The camera to detach from.
     * @internal
     */
    _detachCameras(cameras: Camera): void;
    /**
     * Detaches the effect on cameras
     * @param cameras The camera to detach from.
     * @internal
     */
    _detachCameras(cameras: Camera[]): void;
    /**
     * Enables the effect on given cameras
     * @param cameras The camera to enable.
     * @internal
     */
    _enable(cameras: Camera): void;
    /**
     * Enables the effect on given cameras
     * @param cameras The camera to enable.
     * @internal
     */
    _enable(cameras: Nullable<Camera[]>): void;
    /**
     * Disables the effect on the given cameras
     * @param cameras The camera to disable.
     * @internal
     */
    _disable(cameras: Camera): void;
    /**
     * Disables the effect on the given cameras
     * @param cameras The camera to disable.
     * @internal
     */
    _disable(cameras: Nullable<Camera[]>): void;
    /**
     * Gets a list of the post processes contained in the effect.
     * @param camera The camera to get the post processes on.
     * @returns The list of the post processes in the effect.
     */
    getPostProcesses(camera?: Camera): Nullable<Array<PostProcess>>;
}

}
declare module "babylonjs/PostProcesses/RenderPipeline/postProcessRenderPipeline" {
import { Nullable } from "babylonjs/types";
import { Camera } from "babylonjs/Cameras/camera";
import { Engine } from "babylonjs/Engines/engine";
import { PostProcessRenderEffect } from "babylonjs/PostProcesses/RenderPipeline/postProcessRenderEffect";
import { IInspectable } from "babylonjs/Misc/iInspectable";
import { PrePassRenderer } from "babylonjs/Rendering/prePassRenderer";
/**
 * PostProcessRenderPipeline
 * @see https://doc.babylonjs.com/features/featuresDeepDive/postProcesses/postProcessRenderPipeline
 */
export class PostProcessRenderPipeline {
    private _engine;
    protected _renderEffects: {
        [key: string]: PostProcessRenderEffect;
    };
    protected _renderEffectsForIsolatedPass: PostProcessRenderEffect[];
    /**
     * List of inspectable custom properties (used by the Inspector)
     * @see https://doc.babylonjs.com/toolsAndResources/inspector#extensibility
     */
    inspectableCustomProperties: IInspectable[];
    /**
     * @internal
     */
    protected _cameras: Camera[];
    /** @internal */
    _name: string;
    /**
     * Gets pipeline name
     */
    get name(): string;
    /** Gets the list of attached cameras */
    get cameras(): Camera[];
    /**
     * Initializes a PostProcessRenderPipeline
     * @param _engine engine to add the pipeline to
     * @param name name of the pipeline
     */
    constructor(_engine: Engine, name: string);
    /**
     * Gets the class name
     * @returns "PostProcessRenderPipeline"
     */
    getClassName(): string;
    /**
     * If all the render effects in the pipeline are supported
     */
    get isSupported(): boolean;
    /**
     * Adds an effect to the pipeline
     * @param renderEffect the effect to add
     */
    addEffect(renderEffect: PostProcessRenderEffect): void;
    /** @internal */
    _rebuild(): void;
    /** @internal */
    _enableEffect(renderEffectName: string, cameras: Camera): void;
    /** @internal */
    _enableEffect(renderEffectName: string, cameras: Camera[]): void;
    /** @internal */
    _disableEffect(renderEffectName: string, cameras: Nullable<Camera[]>): void;
    /** @internal */
    _disableEffect(renderEffectName: string, cameras: Nullable<Camera[]>): void;
    /** @internal */
    _attachCameras(cameras: Camera, unique: boolean): void;
    /** @internal */
    _attachCameras(cameras: Camera[], unique: boolean): void;
    /** @internal */
    _detachCameras(cameras: Camera): void;
    /** @internal */
    _detachCameras(cameras: Nullable<Camera[]>): void;
    /** @internal */
    _update(): void;
    /** @internal */
    _reset(): void;
    protected _enableMSAAOnFirstPostProcess(sampleCount: number): boolean;
    /**
     * Sets the required values to the prepass renderer.
     * @param prePassRenderer defines the prepass renderer to setup.
     * @returns true if the pre pass is needed.
     */
    setPrePassRenderer(prePassRenderer: PrePassRenderer): boolean;
    /**
     * Disposes of the pipeline
     */
    dispose(): void;
}
export {};

}
declare module "babylonjs/PostProcesses/RenderPipeline/postProcessRenderPipelineManager" {
import { Camera } from "babylonjs/Cameras/camera";
import { PostProcessRenderPipeline } from "babylonjs/PostProcesses/RenderPipeline/postProcessRenderPipeline";
/**
 * PostProcessRenderPipelineManager class
 * @see https://doc.babylonjs.com/features/featuresDeepDive/postProcesses/postProcessRenderPipeline
 */
export class PostProcessRenderPipelineManager {
    private _renderPipelines;
    /**
     * Initializes a PostProcessRenderPipelineManager
     * @see https://doc.babylonjs.com/features/featuresDeepDive/postProcesses/postProcessRenderPipeline
     */
    constructor();
    /**
     * Gets the list of supported render pipelines
     */
    get supportedPipelines(): PostProcessRenderPipeline[];
    /**
     * Adds a pipeline to the manager
     * @param renderPipeline The pipeline to add
     */
    addPipeline(renderPipeline: PostProcessRenderPipeline): void;
    /**
     * Attaches a camera to the pipeline
     * @param renderPipelineName The name of the pipeline to attach to
     * @param cameras the camera to attach
     * @param unique if the camera can be attached multiple times to the pipeline
     */
    attachCamerasToRenderPipeline(renderPipelineName: string, cameras: any | Camera[] | Camera, unique?: boolean): void;
    /**
     * Detaches a camera from the pipeline
     * @param renderPipelineName The name of the pipeline to detach from
     * @param cameras the camera to detach
     */
    detachCamerasFromRenderPipeline(renderPipelineName: string, cameras: any | Camera[] | Camera): void;
    /**
     * Enables an effect by name on a pipeline
     * @param renderPipelineName the name of the pipeline to enable the effect in
     * @param renderEffectName the name of the effect to enable
     * @param cameras the cameras that the effect should be enabled on
     */
    enableEffectInPipeline(renderPipelineName: string, renderEffectName: string, cameras: any | Camera[] | Camera): void;
    /**
     * Disables an effect by name on a pipeline
     * @param renderPipelineName the name of the pipeline to disable the effect in
     * @param renderEffectName the name of the effect to disable
     * @param cameras the cameras that the effect should be disabled on
     */
    disableEffectInPipeline(renderPipelineName: string, renderEffectName: string, cameras: any | Camera[] | Camera): void;
    /**
     * Updates the state of all contained render pipelines and disposes of any non supported pipelines
     */
    update(): void;
    /** @internal */
    _rebuild(): void;
    /**
     * Disposes of the manager and pipelines
     */
    dispose(): void;
}

}
declare module "babylonjs/PostProcesses/RenderPipeline/postProcessRenderPipelineManagerSceneComponent" {
import { ISceneComponent } from "babylonjs/sceneComponent";
import { PostProcessRenderPipelineManager } from "babylonjs/PostProcesses/RenderPipeline/postProcessRenderPipelineManager";
import { Scene } from "babylonjs/scene";
module "babylonjs/scene" {
    interface Scene {
        /** @internal (Backing field) */
        _postProcessRenderPipelineManager: PostProcessRenderPipelineManager;
        /**
         * Gets the postprocess render pipeline manager
         * @see https://doc.babylonjs.com/features/featuresDeepDive/postProcesses/postProcessRenderPipeline
         * @see https://doc.babylonjs.com/features/featuresDeepDive/postProcesses/defaultRenderingPipeline
         */
        readonly postProcessRenderPipelineManager: PostProcessRenderPipelineManager;
    }
}
/**
 * Defines the Render Pipeline scene component responsible to rendering pipelines
 */
export class PostProcessRenderPipelineManagerSceneComponent implements ISceneComponent {
    /**
     * The component name helpful to identify the component in the list of scene components.
     */
    readonly name: string;
    /**
     * The scene the component belongs to.
     */
    scene: Scene;
    /**
     * Creates a new instance of the component for the given scene
     * @param scene Defines the scene to register the component in
     */
    constructor(scene: Scene);
    /**
     * Registers the component in a given scene
     */
    register(): void;
    /**
     * Rebuilds the elements related to this component in case of
     * context lost for instance.
     */
    rebuild(): void;
    /**
     * Disposes the component and the associated resources
     */
    dispose(): void;
    private _gatherRenderTargets;
}

}
declare module "babylonjs/PostProcesses/screenSpaceCurvaturePostProcess" {
import { Nullable } from "babylonjs/types";
import { Camera } from "babylonjs/Cameras/camera";
import { PostProcessOptions } from "babylonjs/PostProcesses/postProcess";
import { PostProcess } from "babylonjs/PostProcesses/postProcess";
import "babylonjs/Rendering/geometryBufferRendererSceneComponent";
import "babylonjs/Shaders/screenSpaceCurvature.fragment";
import { Engine } from "babylonjs/Engines/engine";
import { Scene } from "babylonjs/scene";
/**
 * The Screen Space curvature effect can help highlighting ridge and valley of a model.
 */
export class ScreenSpaceCurvaturePostProcess extends PostProcess {
    /**
     * Defines how much ridge the curvature effect displays.
     */
    ridge: number;
    /**
     * Defines how much valley the curvature effect displays.
     */
    valley: number;
    private _geometryBufferRenderer;
    /**
     * Gets a string identifying the name of the class
     * @returns "ScreenSpaceCurvaturePostProcess" string
     */
    getClassName(): string;
    /**
     * Creates a new instance ScreenSpaceCurvaturePostProcess
     * @param name The name of the effect.
     * @param scene The scene containing the objects to blur according to their velocity.
     * @param options The required width/height ratio to downsize to before computing the render pass.
     * @param camera The camera to apply the render pass to.
     * @param samplingMode The sampling mode to be used when computing the pass. (default: 0)
     * @param engine The engine which the post process will be applied. (default: current engine)
     * @param reusable If the post process can be reused on the same frame. (default: false)
     * @param textureType Type of textures used when performing the post process. (default: 0)
     * @param blockCompilation If compilation of the shader should not be done in the constructor. The updateEffect method can be used to compile the shader at a later time. (default: false)
     */
    constructor(name: string, scene: Scene, options: number | PostProcessOptions, camera: Nullable<Camera>, samplingMode?: number, engine?: Engine, reusable?: boolean, textureType?: number, blockCompilation?: boolean);
    /**
     * Support test.
     */
    static get IsSupported(): boolean;
    /**
     * @internal
     */
    static _Parse(parsedPostProcess: any, targetCamera: Camera, scene: Scene, rootUrl: string): ScreenSpaceCurvaturePostProcess;
}
export {};

}
declare module "babylonjs/PostProcesses/screenSpaceReflectionPostProcess" {
import { Nullable } from "babylonjs/types";
import { Camera } from "babylonjs/Cameras/camera";
import { PostProcessOptions } from "babylonjs/PostProcesses/postProcess";
import { PostProcess } from "babylonjs/PostProcesses/postProcess";
import "babylonjs/Shaders/screenSpaceReflection.fragment";
import { Engine } from "babylonjs/Engines/engine";
import { Scene } from "babylonjs/scene";
/**
 * The ScreenSpaceReflectionPostProcess performs realtime reflections using only and only the available informations on the screen (positions and normals).
 * Basically, the screen space reflection post-process will compute reflections according the material's reflectivity.
 */
export class ScreenSpaceReflectionPostProcess extends PostProcess {
    /**
     * Gets or sets a reflection threshold mainly used to adjust the reflection's height.
     */
    threshold: number;
    /**
     * Gets or sets the current reflection strength. 1.0 is an ideal value but can be increased/decreased for particular results.
     */
    strength: number;
    /**
     * Gets or sets the falloff exponent used while computing fresnel. More the exponent is high, more the reflections will be discrete.
     */
    reflectionSpecularFalloffExponent: number;
    /**
     * Gets or sets the step size used to iterate until the effect finds the color of the reflection's pixel. Typically in interval [0.1, 1.0]
     */
    step: number;
    /**
     * Gets or sets the factor applied when computing roughness. Default value is 0.2.
     */
    roughnessFactor: number;
    private _forceGeometryBuffer;
    private get _geometryBufferRenderer();
    private get _prePassRenderer();
    private _enableSmoothReflections;
    private _reflectionSamples;
    private _smoothSteps;
    private _isSceneRightHanded;
    /**
     * Gets a string identifying the name of the class
     * @returns "ScreenSpaceReflectionPostProcess" string
     */
    getClassName(): string;
    /**
     * Creates a new instance of ScreenSpaceReflectionPostProcess.
     * @param name The name of the effect.
     * @param scene The scene containing the objects to calculate reflections.
     * @param options The required width/height ratio to downsize to before computing the render pass.
     * @param camera The camera to apply the render pass to.
     * @param samplingMode The sampling mode to be used when computing the pass. (default: 0)
     * @param engine The engine which the post process will be applied. (default: current engine)
     * @param reusable If the post process can be reused on the same frame. (default: false)
     * @param textureType Type of textures used when performing the post process. (default: 0)
     * @param blockCompilation If compilation of the shader should not be done in the constructor. The updateEffect method can be used to compile the shader at a later time. (default: true)
     * @param forceGeometryBuffer If this post process should use geometry buffer instead of prepass (default: false)
     */
    constructor(name: string, scene: Scene, options: number | PostProcessOptions, camera: Nullable<Camera>, samplingMode?: number, engine?: Engine, reusable?: boolean, textureType?: number, blockCompilation?: boolean, forceGeometryBuffer?: boolean);
    /**
     * Gets whether or not smoothing reflections is enabled.
     * Enabling smoothing will require more GPU power and can generate a drop in FPS.
     */
    get enableSmoothReflections(): boolean;
    /**
     * Sets whether or not smoothing reflections is enabled.
     * Enabling smoothing will require more GPU power and can generate a drop in FPS.
     */
    set enableSmoothReflections(enabled: boolean);
    /**
     * Gets the number of samples taken while computing reflections. More samples count is high,
     * more the post-process wil require GPU power and can generate a drop in FPS. Basically in interval [25, 100].
     */
    get reflectionSamples(): number;
    /**
     * Sets the number of samples taken while computing reflections. More samples count is high,
     * more the post-process wil require GPU power and can generate a drop in FPS. Basically in interval [25, 100].
     */
    set reflectionSamples(samples: number);
    /**
     * Gets the number of samples taken while smoothing reflections. More samples count is high,
     * more the post-process will require GPU power and can generate a drop in FPS.
     * Default value (5.0) work pretty well in all cases but can be adjusted.
     */
    get smoothSteps(): number;
    set smoothSteps(steps: number);
    private _updateEffectDefines;
    /**
     * @internal
     */
    static _Parse(parsedPostProcess: any, targetCamera: Camera, scene: Scene, rootUrl: string): ScreenSpaceReflectionPostProcess;
}
export {};

}
declare module "babylonjs/PostProcesses/sharpenPostProcess" {
import { Nullable } from "babylonjs/types";
import { Camera } from "babylonjs/Cameras/camera";
import { PostProcessOptions } from "babylonjs/PostProcesses/postProcess";
import { PostProcess } from "babylonjs/PostProcesses/postProcess";
import "babylonjs/Shaders/sharpen.fragment";
import { Engine } from "babylonjs/Engines/engine";
import { Scene } from "babylonjs/scene";
/**
 * The SharpenPostProcess applies a sharpen kernel to every pixel
 * See http://en.wikipedia.org/wiki/Kernel_(image_processing)
 */
export class SharpenPostProcess extends PostProcess {
    /**
     * How much of the original color should be applied. Setting this to 0 will display edge detection. (default: 1)
     */
    colorAmount: number;
    /**
     * How much sharpness should be applied (default: 0.3)
     */
    edgeAmount: number;
    /**
     * Gets a string identifying the name of the class
     * @returns "SharpenPostProcess" string
     */
    getClassName(): string;
    /**
     * Creates a new instance ConvolutionPostProcess
     * @param name The name of the effect.
     * @param options The required width/height ratio to downsize to before computing the render pass.
     * @param camera The camera to apply the render pass to.
     * @param samplingMode The sampling mode to be used when computing the pass. (default: 0)
     * @param engine The engine which the post process will be applied. (default: current engine)
     * @param reusable If the post process can be reused on the same frame. (default: false)
     * @param textureType Type of textures used when performing the post process. (default: 0)
     * @param blockCompilation If compilation of the shader should not be done in the constructor. The updateEffect method can be used to compile the shader at a later time. (default: false)
     */
    constructor(name: string, options: number | PostProcessOptions, camera: Nullable<Camera>, samplingMode?: number, engine?: Engine, reusable?: boolean, textureType?: number, blockCompilation?: boolean);
    /**
     * @internal
     */
    static _Parse(parsedPostProcess: any, targetCamera: Camera, scene: Scene, rootUrl: string): SharpenPostProcess;
}
export {};

}
declare module "babylonjs/PostProcesses/stereoscopicInterlacePostProcess" {
import { Camera } from "babylonjs/Cameras/camera";
import { PostProcess } from "babylonjs/PostProcesses/postProcess";
import { Engine } from "babylonjs/Engines/engine";
import "babylonjs/Shaders/stereoscopicInterlace.fragment";
/**
 * StereoscopicInterlacePostProcessI used to render stereo views from a rigged camera with support for alternate line interlacing
 */
export class StereoscopicInterlacePostProcessI extends PostProcess {
    private _stepSize;
    private _passedProcess;
    /**
     * Gets a string identifying the name of the class
     * @returns "StereoscopicInterlacePostProcessI" string
     */
    getClassName(): string;
    /**
     * Initializes a StereoscopicInterlacePostProcessI
     * @param name The name of the effect.
     * @param rigCameras The rig cameras to be applied to the post process
     * @param isStereoscopicHoriz If the rendered results are horizontal or vertical
     * @param isStereoscopicInterlaced If the rendered results are alternate line interlaced
     * @param samplingMode The sampling mode to be used when computing the pass. (default: 0)
     * @param engine The engine which the post process will be applied. (default: current engine)
     * @param reusable If the post process can be reused on the same frame. (default: false)
     */
    constructor(name: string, rigCameras: Camera[], isStereoscopicHoriz: boolean, isStereoscopicInterlaced: boolean, samplingMode?: number, engine?: Engine, reusable?: boolean);
}
/**
 * StereoscopicInterlacePostProcess used to render stereo views from a rigged camera
 */
export class StereoscopicInterlacePostProcess extends PostProcess {
    private _stepSize;
    private _passedProcess;
    /**
     * Gets a string identifying the name of the class
     * @returns "StereoscopicInterlacePostProcess" string
     */
    getClassName(): string;
    /**
     * Initializes a StereoscopicInterlacePostProcess
     * @param name The name of the effect.
     * @param rigCameras The rig cameras to be applied to the post process
     * @param isStereoscopicHoriz If the rendered results are horizontal or vertical
     * @param samplingMode The sampling mode to be used when computing the pass. (default: 0)
     * @param engine The engine which the post process will be applied. (default: current engine)
     * @param reusable If the post process can be reused on the same frame. (default: false)
     */
    constructor(name: string, rigCameras: Camera[], isStereoscopicHoriz: boolean, samplingMode?: number, engine?: Engine, reusable?: boolean);
}

}
declare module "babylonjs/PostProcesses/subSurfaceScatteringPostProcess" {
import { Nullable } from "babylonjs/types";
import { Camera } from "babylonjs/Cameras/camera";
import { PostProcessOptions } from "babylonjs/PostProcesses/postProcess";
import { PostProcess } from "babylonjs/PostProcesses/postProcess";
import { Engine } from "babylonjs/Engines/engine";
import { Scene } from "babylonjs/scene";
import "babylonjs/Shaders/imageProcessing.fragment";
import "babylonjs/Shaders/subSurfaceScattering.fragment";
import "babylonjs/Shaders/postprocess.vertex";
/**
 * Sub surface scattering post process
 */
export class SubSurfaceScatteringPostProcess extends PostProcess {
    /**
     * Gets a string identifying the name of the class
     * @returns "SubSurfaceScatteringPostProcess" string
     */
    getClassName(): string;
    constructor(name: string, scene: Scene, options: number | PostProcessOptions, camera?: Nullable<Camera>, samplingMode?: number, engine?: Engine, reusable?: boolean, textureType?: number);
}

}
declare module "babylonjs/PostProcesses/tonemapPostProcess" {
import { Camera } from "babylonjs/Cameras/camera";
import { PostProcess } from "babylonjs/PostProcesses/postProcess";
import "babylonjs/Shaders/tonemap.fragment";
import { Nullable } from "babylonjs/types";
import { Engine } from "babylonjs/Engines/engine";
/** Defines operator used for tonemapping */
export enum TonemappingOperator {
    /** Hable */
    Hable = 0,
    /** Reinhard */
    Reinhard = 1,
    /** HejiDawson */
    HejiDawson = 2,
    /** Photographic */
    Photographic = 3
}
/**
 * Defines a post process to apply tone mapping
 */
export class TonemapPostProcess extends PostProcess {
    private _operator;
    /** Defines the required exposure adjustment */
    exposureAdjustment: number;
    /**
     * Gets a string identifying the name of the class
     * @returns "TonemapPostProcess" string
     */
    getClassName(): string;
    /**
     * Creates a new TonemapPostProcess
     * @param name defines the name of the postprocess
     * @param _operator defines the operator to use
     * @param exposureAdjustment defines the required exposure adjustment
     * @param camera defines the camera to use (can be null)
     * @param samplingMode defines the required sampling mode (BABYLON.Texture.BILINEAR_SAMPLINGMODE by default)
     * @param engine defines the hosting engine (can be ignore if camera is set)
     * @param textureFormat defines the texture format to use (BABYLON.Engine.TEXTURETYPE_UNSIGNED_INT by default)
     * @param reusable If the post process can be reused on the same frame. (default: false)
     */
    constructor(name: string, _operator: TonemappingOperator, 
    /** Defines the required exposure adjustment */
    exposureAdjustment: number, camera: Nullable<Camera>, samplingMode?: number, engine?: Engine, textureFormat?: number, reusable?: boolean);
}
export {};

}
declare module "babylonjs/PostProcesses/volumetricLightScatteringPostProcess" {
import { Vector3 } from "babylonjs/Maths/math.vector";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { Mesh } from "babylonjs/Meshes/mesh";
import { Camera } from "babylonjs/Cameras/camera";
import { RenderTargetTexture } from "babylonjs/Materials/Textures/renderTargetTexture";
import { PostProcess } from "babylonjs/PostProcesses/postProcess";
import { Scene } from "babylonjs/scene";
import "babylonjs/Shaders/depth.vertex";
import "babylonjs/Shaders/volumetricLightScattering.fragment";
import "babylonjs/Shaders/volumetricLightScatteringPass.vertex";
import "babylonjs/Shaders/volumetricLightScatteringPass.fragment";
import { Nullable } from "babylonjs/types";
import { Engine } from "babylonjs/Engines/engine";
/**
 *  Inspired by https://developer.nvidia.com/gpugems/gpugems3/part-ii-light-and-shadows/chapter-13-volumetric-light-scattering-post-process
 */
export class VolumetricLightScatteringPostProcess extends PostProcess {
    private _volumetricLightScatteringRTT;
    private _viewPort;
    private _screenCoordinates;
    /**
     * If not undefined, the mesh position is computed from the attached node position
     */
    attachedNode: {
        position: Vector3;
    };
    /**
     * Custom position of the mesh. Used if "useCustomMeshPosition" is set to "true"
     */
    customMeshPosition: Vector3;
    /**
     * Set if the post-process should use a custom position for the light source (true) or the internal mesh position (false)
     */
    useCustomMeshPosition: boolean;
    /**
     * If the post-process should inverse the light scattering direction
     */
    invert: boolean;
    /**
     * The internal mesh used by the post-process
     */
    mesh: Mesh;
    /**
     * @internal
     * VolumetricLightScatteringPostProcess.useDiffuseColor is no longer used, use the mesh material directly instead
     */
    get useDiffuseColor(): boolean;
    set useDiffuseColor(useDiffuseColor: boolean);
    /**
     * Array containing the excluded meshes not rendered in the internal pass
     */
    excludedMeshes: AbstractMesh[];
    /**
     * Array containing the only meshes rendered in the internal pass.
     * If this array is not empty, only the meshes from this array are rendered in the internal pass
     */
    includedMeshes: AbstractMesh[];
    /**
     * Controls the overall intensity of the post-process
     */
    exposure: number;
    /**
     * Dissipates each sample's contribution in range [0, 1]
     */
    decay: number;
    /**
     * Controls the overall intensity of each sample
     */
    weight: number;
    /**
     * Controls the density of each sample
     */
    density: number;
    /**
     * @constructor
     * @param name The post-process name
     * @param ratio The size of the post-process and/or internal pass (0.5 means that your postprocess will have a width = canvas.width 0.5 and a height = canvas.height 0.5)
     * @param camera The camera that the post-process will be attached to
     * @param mesh The mesh used to create the light scattering
     * @param samples The post-process quality, default 100
     * @param samplingMode The post-process filtering mode
     * @param engine The babylon engine
     * @param reusable If the post-process is reusable
     * @param scene The constructor needs a scene reference to initialize internal components. If "camera" is null a "scene" must be provided
     */
    constructor(name: string, ratio: any, camera: Nullable<Camera>, mesh?: Mesh, samples?: number, samplingMode?: number, engine?: Engine, reusable?: boolean, scene?: Scene);
    /**
     * Returns the string "VolumetricLightScatteringPostProcess"
     * @returns "VolumetricLightScatteringPostProcess"
     */
    getClassName(): string;
    private _isReady;
    /**
     * Sets the new light position for light scattering effect
     * @param position The new custom light position
     */
    setCustomMeshPosition(position: Vector3): void;
    /**
     * Returns the light position for light scattering effect
     * @returns Vector3 The custom light position
     */
    getCustomMeshPosition(): Vector3;
    /**
     * Disposes the internal assets and detaches the post-process from the camera
     * @param camera
     */
    dispose(camera: Camera): void;
    /**
     * Returns the render target texture used by the post-process
     * @returns the render target texture used by the post-process
     */
    getPass(): RenderTargetTexture;
    private _meshExcluded;
    private _createPass;
    private _updateMeshScreenCoordinates;
    /**
     * Creates a default mesh for the Volumeric Light Scattering post-process
     * @param name The mesh name
     * @param scene The scene where to create the mesh
     * @returns the default mesh
     */
    static CreateDefaultMesh(name: string, scene: Scene): Mesh;
}
export {};

}
declare module "babylonjs/PostProcesses/vrDistortionCorrectionPostProcess" {
import { Camera } from "babylonjs/Cameras/camera";
import { VRCameraMetrics } from "babylonjs/Cameras/VR/vrCameraMetrics";
import { PostProcess } from "babylonjs/PostProcesses/postProcess";
import "babylonjs/Shaders/vrDistortionCorrection.fragment";
import { Nullable } from "babylonjs/types";
/**
 * VRDistortionCorrectionPostProcess used for mobile VR
 */
export class VRDistortionCorrectionPostProcess extends PostProcess {
    private _isRightEye;
    private _distortionFactors;
    private _postProcessScaleFactor;
    private _lensCenterOffset;
    private _scaleIn;
    private _scaleFactor;
    private _lensCenter;
    /**
     * Gets a string identifying the name of the class
     * @returns "VRDistortionCorrectionPostProcess" string
     */
    getClassName(): string;
    /**
     * Initializes the VRDistortionCorrectionPostProcess
     * @param name The name of the effect.
     * @param camera The camera to apply the render pass to.
     * @param isRightEye If this is for the right eye distortion
     * @param vrMetrics All the required metrics for the VR camera
     */
    constructor(name: string, camera: Nullable<Camera>, isRightEye: boolean, vrMetrics: VRCameraMetrics);
}

}
declare module "babylonjs/PostProcesses/vrMultiviewToSingleviewPostProcess" {
import { Camera } from "babylonjs/Cameras/camera";
import { PostProcess } from "babylonjs/PostProcesses/postProcess";
import "babylonjs/Shaders/vrMultiviewToSingleview.fragment";
import "babylonjs/Engines/Extensions/engine.multiview";
import { Nullable } from "babylonjs/types";
/**
 * VRMultiviewToSingleview used to convert multiview texture arrays to standard textures for scenarios such as webVR
 * This will not be used for webXR as it supports displaying texture arrays directly
 */
export class VRMultiviewToSingleviewPostProcess extends PostProcess {
    /**
     * Gets a string identifying the name of the class
     * @returns "VRMultiviewToSingleviewPostProcess" string
     */
    getClassName(): string;
    /**
     * Initializes a VRMultiviewToSingleview
     * @param name name of the post process
     * @param camera camera to be applied to
     * @param scaleFactor scaling factor to the size of the output texture
     */
    constructor(name: string, camera: Nullable<Camera>, scaleFactor: number);
}

}
declare module "babylonjs/Probes/index" {
export * from "babylonjs/Probes/reflectionProbe";

}
declare module "babylonjs/Probes/reflectionProbe" {
import { RenderTargetTexture } from "babylonjs/Materials/Textures/renderTargetTexture";
import { Vector3 } from "babylonjs/Maths/math.vector";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { Nullable } from "babylonjs/types";
import { AbstractScene } from "babylonjs/abstractScene";
import { Scene } from "babylonjs/scene";
module "babylonjs/abstractScene" {
    interface AbstractScene {
        /**
         * The list of reflection probes added to the scene
         * @see https://doc.babylonjs.com/features/featuresDeepDive/environment/reflectionProbes
         */
        reflectionProbes: Array<ReflectionProbe>;
        /**
         * Removes the given reflection probe from this scene.
         * @param toRemove The reflection probe to remove
         * @returns The index of the removed reflection probe
         */
        removeReflectionProbe(toRemove: ReflectionProbe): number;
        /**
         * Adds the given reflection probe to this scene.
         * @param newReflectionProbe The reflection probe to add
         */
        addReflectionProbe(newReflectionProbe: ReflectionProbe): void;
    }
}
/**
 * Class used to generate realtime reflection / refraction cube textures
 * @see https://doc.babylonjs.com/features/featuresDeepDive/environment/reflectionProbes
 */
export class ReflectionProbe {
    /** defines the name of the probe */
    name: string;
    private _scene;
    private _renderTargetTexture;
    private _projectionMatrix;
    private _viewMatrix;
    private _target;
    private _add;
    private _attachedMesh;
    private _invertYAxis;
    private _sceneUBOs;
    private _currentSceneUBO;
    /** Gets or sets probe position (center of the cube map) */
    position: Vector3;
    /**
     * Gets or sets an object used to store user defined information for the reflection probe.
     */
    metadata: any;
    /** @internal */
    _parentContainer: Nullable<AbstractScene>;
    /**
     * Creates a new reflection probe
     * @param name defines the name of the probe
     * @param size defines the texture resolution (for each face)
     * @param scene defines the hosting scene
     * @param generateMipMaps defines if mip maps should be generated automatically (true by default)
     * @param useFloat defines if HDR data (float data) should be used to store colors (false by default)
     * @param linearSpace defines if the probe should be generated in linear space or not (false by default)
     */
    constructor(
    /** defines the name of the probe */
    name: string, size: number, scene: Scene, generateMipMaps?: boolean, useFloat?: boolean, linearSpace?: boolean);
    /** Gets or sets the number of samples to use for multi-sampling (0 by default). Required WebGL2 */
    get samples(): number;
    set samples(value: number);
    /** Gets or sets the refresh rate to use (on every frame by default) */
    get refreshRate(): number;
    set refreshRate(value: number);
    /**
     * Gets the hosting scene
     * @returns a Scene
     */
    getScene(): Scene;
    /** Gets the internal CubeTexture used to render to */
    get cubeTexture(): RenderTargetTexture;
    /** Gets the list of meshes to render */
    get renderList(): Nullable<AbstractMesh[]>;
    /**
     * Attach the probe to a specific mesh (Rendering will be done from attached mesh's position)
     * @param mesh defines the mesh to attach to
     */
    attachToMesh(mesh: Nullable<AbstractMesh>): void;
    /**
     * Specifies whether or not the stencil and depth buffer are cleared between two rendering groups
     * @param renderingGroupId The rendering group id corresponding to its index
     * @param autoClearDepthStencil Automatically clears depth and stencil between groups if true.
     */
    setRenderingAutoClearDepthStencil(renderingGroupId: number, autoClearDepthStencil: boolean): void;
    /**
     * Clean all associated resources
     */
    dispose(): void;
    /**
     * Converts the reflection probe information to a readable string for debug purpose.
     * @param fullDetails Supports for multiple levels of logging within scene loading
     * @returns the human readable reflection probe info
     */
    toString(fullDetails?: boolean): string;
    /**
     * Get the class name of the refection probe.
     * @returns "ReflectionProbe"
     */
    getClassName(): string;
    /**
     * Serialize the reflection probe to a JSON representation we can easily use in the respective Parse function.
     * @returns The JSON representation of the texture
     */
    serialize(): any;
    /**
     * Parse the JSON representation of a reflection probe in order to recreate the reflection probe in the given scene.
     * @param parsedReflectionProbe Define the JSON representation of the reflection probe
     * @param scene Define the scene the parsed reflection probe should be instantiated in
     * @param rootUrl Define the root url of the parsing sequence in the case of relative dependencies
     * @returns The parsed reflection probe if successful
     */
    static Parse(parsedReflectionProbe: any, scene: Scene, rootUrl: string): Nullable<ReflectionProbe>;
}

}
declare module "babylonjs/Rendering/boundingBoxRenderer" {
import { Scene } from "babylonjs/scene";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { SmartArray } from "babylonjs/Misc/smartArray";
import { ISceneComponent } from "babylonjs/sceneComponent";
import { BoundingBox } from "babylonjs/Culling/boundingBox";
import { Color3 } from "babylonjs/Maths/math.color";
import { Observable } from "babylonjs/Misc/observable";
import "babylonjs/Shaders/boundingBoxRenderer.fragment";
import "babylonjs/Shaders/boundingBoxRenderer.vertex";
module "babylonjs/scene" {
    interface Scene {
        /** @internal (Backing field) */
        _boundingBoxRenderer: BoundingBoxRenderer;
        /** @internal (Backing field) */
        _forceShowBoundingBoxes: boolean;
        /**
         * Gets or sets a boolean indicating if all bounding boxes must be rendered
         */
        forceShowBoundingBoxes: boolean;
        /**
         * Gets the bounding box renderer associated with the scene
         * @returns a BoundingBoxRenderer
         */
        getBoundingBoxRenderer(): BoundingBoxRenderer;
    }
}
module "babylonjs/Meshes/abstractMesh" {
    interface AbstractMesh {
        /** @internal (Backing field) */
        _showBoundingBox: boolean;
        /**
         * Gets or sets a boolean indicating if the bounding box must be rendered as well (false by default)
         */
        showBoundingBox: boolean;
    }
}
/**
 * Component responsible of rendering the bounding box of the meshes in a scene.
 * This is usually used through the mesh.showBoundingBox or the scene.forceShowBoundingBoxes properties
 */
export class BoundingBoxRenderer implements ISceneComponent {
    /**
     * The component name helpful to identify the component in the list of scene components.
     */
    readonly name: string;
    /**
     * The scene the component belongs to.
     */
    scene: Scene;
    /**
     * Color of the bounding box lines placed in front of an object
     */
    frontColor: Color3;
    /**
     * Color of the bounding box lines placed behind an object
     */
    backColor: Color3;
    /**
     * Defines if the renderer should show the back lines or not
     */
    showBackLines: boolean;
    /**
     * Observable raised before rendering a bounding box
     */
    onBeforeBoxRenderingObservable: Observable<BoundingBox>;
    /**
     * Observable raised after rendering a bounding box
     */
    onAfterBoxRenderingObservable: Observable<BoundingBox>;
    /**
     * Observable raised after resources are created
     */
    onResourcesReadyObservable: Observable<BoundingBoxRenderer>;
    /**
     * When false, no bounding boxes will be rendered
     */
    enabled: boolean;
    /**
     * @internal
     */
    renderList: SmartArray<BoundingBox>;
    private _colorShader;
    private _colorShaderForOcclusionQuery;
    private _vertexBuffers;
    private _indexBuffer;
    private _fillIndexBuffer;
    private _fillIndexData;
    private _uniformBufferFront;
    private _uniformBufferBack;
    private _renderPassIdForOcclusionQuery;
    /**
     * Instantiates a new bounding box renderer in a scene.
     * @param scene the scene the  renderer renders in
     */
    constructor(scene: Scene);
    private _buildUniformLayout;
    /**
     * Registers the component in a given scene
     */
    register(): void;
    private _evaluateSubMesh;
    private _preActiveMesh;
    private _prepareResources;
    private _createIndexBuffer;
    /**
     * Rebuilds the elements related to this component in case of
     * context lost for instance.
     */
    rebuild(): void;
    /**
     * @internal
     */
    reset(): void;
    /**
     * Render the bounding boxes of a specific rendering group
     * @param renderingGroupId defines the rendering group to render
     */
    render(renderingGroupId: number): void;
    private _createWrappersForBoundingBox;
    /**
     * In case of occlusion queries, we can render the occlusion bounding box through this method
     * @param mesh Define the mesh to render the occlusion bounding box for
     */
    renderOcclusionBoundingBox(mesh: AbstractMesh): void;
    /**
     * Dispose and release the resources attached to this renderer.
     */
    dispose(): void;
}

}
declare module "babylonjs/Rendering/depthPeelingRenderer" {
import { Effect } from "babylonjs/Materials/effect";
import { SubMesh } from "babylonjs/Meshes/subMesh";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { SmartArray } from "babylonjs/Misc/smartArray";
import { Scene } from "babylonjs/scene";
import { PrePassRenderer } from "babylonjs/Rendering/prePassRenderer";
import "babylonjs/Shaders/postprocess.vertex";
import "babylonjs/Shaders/oitFinal.fragment";
import "babylonjs/Shaders/oitBackBlend.fragment";
/**
 * The depth peeling renderer that performs
 * Order independant transparency (OIT).
 * This should not be instanciated directly, as it is part of a scene component
 */
export class DepthPeelingRenderer {
    private _scene;
    private _engine;
    private _depthMrts;
    private _thinTextures;
    private _colorMrts;
    private _blendBackMrt;
    private _outputRT;
    private _blendBackEffectWrapper;
    private _blendBackEffectWrapperPingPong;
    private _finalEffectWrapper;
    private _effectRenderer;
    private _currentPingPongState;
    private _prePassEffectConfiguration;
    private _blendBackTexture;
    private _layoutCacheFormat;
    private _layoutCache;
    private _renderPassIds;
    private _candidateSubMeshes;
    private _excludedSubMeshes;
    private _excludedMeshes;
    private static _DEPTH_CLEAR_VALUE;
    private static _MIN_DEPTH;
    private static _MAX_DEPTH;
    private _colorCache;
    private _passCount;
    /**
     * Number of depth peeling passes. As we are using dual depth peeling, each pass two levels of transparency are processed.
     */
    get passCount(): number;
    set passCount(count: number);
    private _useRenderPasses;
    /**
     * Instructs the renderer to use render passes. It is an optimization that makes the rendering faster for some engines (like WebGPU) but that consumes more memory, so it is disabled by default.
     */
    get useRenderPasses(): boolean;
    set useRenderPasses(usePasses: boolean);
    /**
     * Add a mesh in the exclusion list to prevent it to be handled by the depth peeling renderer
     * @param mesh The mesh to exclude from the depth peeling renderer
     */
    addExcludedMesh(mesh: AbstractMesh): void;
    /**
     * Remove a mesh from the exclusion list of the depth peeling renderer
     * @param mesh The mesh to remove
     */
    removeExcludedMesh(mesh: AbstractMesh): void;
    /**
     * Instanciates the depth peeling renderer
     * @param scene Scene to attach to
     * @param passCount Number of depth layers to peel
     * @returns The depth peeling renderer
     */
    constructor(scene: Scene, passCount?: number);
    private _createRenderPassIds;
    private _releaseRenderPassIds;
    private _createTextures;
    private _disposeTextures;
    private _updateTextures;
    private _updateTextureReferences;
    private _createEffects;
    /**
     * Links to the prepass renderer
     * @param prePassRenderer The scene PrePassRenderer
     */
    setPrePassRenderer(prePassRenderer: PrePassRenderer): void;
    /**
     * Binds depth peeling textures on an effect
     * @param effect The effect to bind textures on
     */
    bind(effect: Effect): void;
    private _renderSubMeshes;
    private _finalCompose;
    /**
     * Renders transparent submeshes with depth peeling
     * @param transparentSubMeshes List of transparent meshes to render
     * @returns The array of submeshes that could not be handled by this renderer
     */
    render(transparentSubMeshes: SmartArray<SubMesh>): SmartArray<SubMesh>;
    /**
     * Disposes the depth peeling renderer and associated ressources
     */
    dispose(): void;
}

}
declare module "babylonjs/Rendering/depthPeelingSceneComponent" {
import { Scene } from "babylonjs/scene";
import { ISceneComponent } from "babylonjs/sceneComponent";
import { Nullable } from "babylonjs/types";
import { DepthPeelingRenderer } from "babylonjs/Rendering/depthPeelingRenderer";
module "babylonjs/scene" {
    interface Scene {
        /**
         * The depth peeling renderer
         */
        depthPeelingRenderer: Nullable<DepthPeelingRenderer>;
        /** @internal (Backing field) */
        _depthPeelingRenderer: Nullable<DepthPeelingRenderer>;
        /**
         * Flag to indicate if we want to use order independent transparency, despite the performance hit
         */
        useOrderIndependentTransparency: boolean;
        /** @internal */
        _useOrderIndependentTransparency: boolean;
    }
}
/**
 * Scene component to render order independent transparency with depth peeling
 */
export class DepthPeelingSceneComponent implements ISceneComponent {
    /**
     * The component name helpful to identify the component in the list of scene components.
     */
    readonly name: string;
    /**
     * The scene the component belongs to.
     */
    scene: Scene;
    /**
     * Creates a new instance of the component for the given scene
     * @param scene Defines the scene to register the component in
     */
    constructor(scene: Scene);
    /**
     * Registers the component in a given scene
     */
    register(): void;
    /**
     * Rebuilds the elements related to this component in case of
     * context lost for instance.
     */
    rebuild(): void;
    /**
     * Disposes the component and the associated resources.
     */
    dispose(): void;
}

}
declare module "babylonjs/Rendering/depthRenderer" {
import { Nullable } from "babylonjs/types";
import { SubMesh } from "babylonjs/Meshes/subMesh";
import { Scene } from "babylonjs/scene";
import { RenderTargetTexture } from "babylonjs/Materials/Textures/renderTargetTexture";
import { Camera } from "babylonjs/Cameras/camera";
import "babylonjs/Shaders/depth.fragment";
import "babylonjs/Shaders/depth.vertex";
import { Material } from "babylonjs/Materials/material";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
/**
 * This represents a depth renderer in Babylon.
 * A depth renderer will render to it's depth map every frame which can be displayed or used in post processing
 */
export class DepthRenderer {
    private _scene;
    private _depthMap;
    private readonly _storeNonLinearDepth;
    private readonly _clearColor;
    /** Get if the depth renderer is using packed depth or not */
    readonly isPacked: boolean;
    private _camera;
    /** Enable or disable the depth renderer. When disabled, the depth texture is not updated */
    enabled: boolean;
    /** Force writing the transparent objects into the depth map */
    forceDepthWriteTransparentMeshes: boolean;
    /**
     * Specifies that the depth renderer will only be used within
     * the camera it is created for.
     * This can help forcing its rendering during the camera processing.
     */
    useOnlyInActiveCamera: boolean;
    /**
     * @internal
     */
    static _SceneComponentInitialization: (scene: Scene) => void;
    /**
     * Sets a specific material to be used to render a mesh/a list of meshes by the depth renderer
     * @param mesh mesh or array of meshes
     * @param material material to use by the depth render when rendering the mesh(es). If undefined is passed, the specific material created by the depth renderer will be used.
     */
    setMaterialForRendering(mesh: AbstractMesh | AbstractMesh[], material?: Material): void;
    /**
     * Instantiates a depth renderer
     * @param scene The scene the renderer belongs to
     * @param type The texture type of the depth map (default: Engine.TEXTURETYPE_FLOAT)
     * @param camera The camera to be used to render the depth map (default: scene's active camera)
     * @param storeNonLinearDepth Defines whether the depth is stored linearly like in Babylon Shadows or directly like glFragCoord.z
     * @param samplingMode The sampling mode to be used with the render target (Linear, Nearest...)
     */
    constructor(scene: Scene, type?: number, camera?: Nullable<Camera>, storeNonLinearDepth?: boolean, samplingMode?: number);
    /**
     * Creates the depth rendering effect and checks if the effect is ready.
     * @param subMesh The submesh to be used to render the depth map of
     * @param useInstances If multiple world instances should be used
     * @returns if the depth renderer is ready to render the depth map
     */
    isReady(subMesh: SubMesh, useInstances: boolean): boolean;
    /**
     * Gets the texture which the depth map will be written to.
     * @returns The depth map texture
     */
    getDepthMap(): RenderTargetTexture;
    /**
     * Disposes of the depth renderer.
     */
    dispose(): void;
}
export {};

}
declare module "babylonjs/Rendering/depthRendererSceneComponent" {
import { Nullable } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
import { DepthRenderer } from "babylonjs/Rendering/depthRenderer";
import { Camera } from "babylonjs/Cameras/camera";
import { ISceneComponent } from "babylonjs/sceneComponent";
module "babylonjs/scene" {
    interface Scene {
        /** @internal (Backing field) */
        _depthRenderer: {
            [id: string]: DepthRenderer;
        };
        /**
         * Creates a depth renderer a given camera which contains a depth map which can be used for post processing.
         * @param camera The camera to create the depth renderer on (default: scene's active camera)
         * @param storeNonLinearDepth Defines whether the depth is stored linearly like in Babylon Shadows or directly like glFragCoord.z
         * @param force32bitsFloat Forces 32 bits float when supported (else 16 bits float is prioritized over 32 bits float if supported)
         * @param samplingMode The sampling mode to be used with the render target (Linear, Nearest...)
         * @returns the created depth renderer
         */
        enableDepthRenderer(camera?: Nullable<Camera>, storeNonLinearDepth?: boolean, force32bitsFloat?: boolean, samplingMode?: number): DepthRenderer;
        /**
         * Disables a depth renderer for a given camera
         * @param camera The camera to disable the depth renderer on (default: scene's active camera)
         */
        disableDepthRenderer(camera?: Nullable<Camera>): void;
    }
}
/**
 * Defines the Depth Renderer scene component responsible to manage a depth buffer useful
 * in several rendering techniques.
 */
export class DepthRendererSceneComponent implements ISceneComponent {
    /**
     * The component name helpful to identify the component in the list of scene components.
     */
    readonly name: string;
    /**
     * The scene the component belongs to.
     */
    scene: Scene;
    /**
     * Creates a new instance of the component for the given scene
     * @param scene Defines the scene to register the component in
     */
    constructor(scene: Scene);
    /**
     * Registers the component in a given scene
     */
    register(): void;
    /**
     * Rebuilds the elements related to this component in case of
     * context lost for instance.
     */
    rebuild(): void;
    /**
     * Disposes the component and the associated resources
     */
    dispose(): void;
    private _gatherRenderTargets;
    private _gatherActiveCameraRenderTargets;
}

}
declare module "babylonjs/Rendering/edgesRenderer" {
import { Immutable, Nullable } from "babylonjs/types";
import { VertexBuffer } from "babylonjs/Buffers/buffer";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { Matrix } from "babylonjs/Maths/math.vector";
import { Vector3 } from "babylonjs/Maths/math.vector";
import { IDisposable } from "babylonjs/scene";
import { ShaderMaterial } from "babylonjs/Materials/shaderMaterial";
import "babylonjs/Shaders/line.fragment";
import "babylonjs/Shaders/line.vertex";
import { DataBuffer } from "babylonjs/Buffers/dataBuffer";
import { SmartArray } from "babylonjs/Misc/smartArray";
import { DrawWrapper } from "babylonjs/Materials/drawWrapper";
module "babylonjs/scene" {
    interface Scene {
        /** @internal */
        _edgeRenderLineShader: Nullable<ShaderMaterial>;
    }
}
module "babylonjs/Meshes/abstractMesh" {
    interface AbstractMesh {
        /**
         * Gets the edgesRenderer associated with the mesh
         */
        edgesRenderer: Nullable<EdgesRenderer>;
    }
}
module "babylonjs/Meshes/linesMesh" {
    interface LinesMesh {
        /**
         * Enables the edge rendering mode on the mesh.
         * This mode makes the mesh edges visible
         * @param epsilon defines the maximal distance between two angles to detect a face
         * @param checkVerticesInsteadOfIndices indicates that we should check vertex list directly instead of faces
         * @returns the currentAbstractMesh
         * @see https://www.babylonjs-playground.com/#19O9TU#0
         */
        enableEdgesRendering(epsilon?: number, checkVerticesInsteadOfIndices?: boolean): AbstractMesh;
    }
}
module "babylonjs/Meshes/linesMesh" {
    interface InstancedLinesMesh {
        /**
         * Enables the edge rendering mode on the mesh.
         * This mode makes the mesh edges visible
         * @param epsilon defines the maximal distance between two angles to detect a face
         * @param checkVerticesInsteadOfIndices indicates that we should check vertex list directly instead of faces
         * @returns the current InstancedLinesMesh
         * @see https://www.babylonjs-playground.com/#19O9TU#0
         */
        enableEdgesRendering(epsilon?: number, checkVerticesInsteadOfIndices?: boolean): InstancedLinesMesh;
    }
}
/**
 * Defines the minimum contract an Edges renderer should follow.
 */
export interface IEdgesRenderer extends IDisposable {
    /**
     * Gets or sets a boolean indicating if the edgesRenderer is active
     */
    isEnabled: boolean;
    /**
     * Renders the edges of the attached mesh,
     */
    render(): void;
    /**
     * Checks whether or not the edges renderer is ready to render.
     * @returns true if ready, otherwise false.
     */
    isReady(): boolean;
    /**
     * List of instances to render in case the source mesh has instances
     */
    customInstances: SmartArray<Matrix>;
}
/**
 * Defines the additional options of the edges renderer
 */
export interface IEdgesRendererOptions {
    /**
     * Gets or sets a boolean indicating that the alternate edge finder algorithm must be used
     * If not defined, the default value is true
     */
    useAlternateEdgeFinder?: boolean;
    /**
     * Gets or sets a boolean indicating that the vertex merger fast processing must be used.
     * If not defined, the default value is true.
     * You should normally leave it undefined (or set it to true), except if you see some artifacts in the edges rendering (can happen with complex geometries)
     * This option is used only if useAlternateEdgeFinder = true
     */
    useFastVertexMerger?: boolean;
    /**
     * During edges processing, the vertices are merged if they are close enough: epsilonVertexMerge is the limit within which vertices are considered to be equal.
     * The default value is 1e-6
     * This option is used only if useAlternateEdgeFinder = true
     */
    epsilonVertexMerge?: number;
    /**
     * Gets or sets a boolean indicating that tessellation should be applied before finding the edges. You may need to activate this option if your geometry is a bit
     * unusual, like having a vertex of a triangle in-between two vertices of an edge of another triangle. It happens often when using CSG to construct meshes.
     * This option is used only if useAlternateEdgeFinder = true
     */
    applyTessellation?: boolean;
    /**
     * The limit under which 3 vertices are considered to be aligned. 3 vertices PQR are considered aligned if distance(PQ) + distance(QR) - distance(PR) < epsilonVertexAligned
     * The default value is 1e-6
     * This option is used only if useAlternateEdgeFinder = true
     */
    epsilonVertexAligned?: number;
    /**
     * Gets or sets a boolean indicating that degenerated triangles should not be processed.
     * Degenerated triangles are triangles that have 2 or 3 vertices with the same coordinates
     */
    removeDegeneratedTriangles?: boolean;
}
/**
 * This class is used to generate edges of the mesh that could then easily be rendered in a scene.
 */
export class EdgesRenderer implements IEdgesRenderer {
    /**
     * Define the size of the edges with an orthographic camera
     */
    edgesWidthScalerForOrthographic: number;
    /**
     * Define the size of the edges with a perspective camera
     */
    edgesWidthScalerForPerspective: number;
    protected _source: AbstractMesh;
    protected _linesPositions: number[];
    protected _linesNormals: number[];
    protected _linesIndices: number[];
    protected _epsilon: number;
    protected _indicesCount: number;
    protected _drawWrapper?: DrawWrapper;
    protected _lineShader: ShaderMaterial;
    protected _ib: DataBuffer;
    protected _buffers: {
        [key: string]: Nullable<VertexBuffer>;
    };
    protected _buffersForInstances: {
        [key: string]: Nullable<VertexBuffer>;
    };
    protected _checkVerticesInsteadOfIndices: boolean;
    protected _options: Nullable<IEdgesRendererOptions>;
    private _meshRebuildObserver;
    private _meshDisposeObserver;
    /** Gets or sets a boolean indicating if the edgesRenderer is active */
    isEnabled: boolean;
    /** Gets the vertices generated by the edge renderer */
    get linesPositions(): Immutable<Array<number>>;
    /** Gets the normals generated by the edge renderer */
    get linesNormals(): Immutable<Array<number>>;
    /** Gets the indices generated by the edge renderer */
    get linesIndices(): Immutable<Array<number>>;
    /**
     * Gets or sets the shader used to draw the lines
     */
    get lineShader(): ShaderMaterial;
    set lineShader(shader: ShaderMaterial);
    /**
     * List of instances to render in case the source mesh has instances
     */
    customInstances: SmartArray<Matrix>;
    private static _GetShader;
    /**
     * Creates an instance of the EdgesRenderer. It is primarily use to display edges of a mesh.
     * Beware when you use this class with complex objects as the adjacencies computation can be really long
     * @param  source Mesh used to create edges
     * @param  epsilon sum of angles in adjacency to check for edge
     * @param  checkVerticesInsteadOfIndices bases the edges detection on vertices vs indices. Note that this parameter is not used if options.useAlternateEdgeFinder = true
     * @param  generateEdgesLines - should generate Lines or only prepare resources.
     * @param  options The options to apply when generating the edges
     */
    constructor(source: AbstractMesh, epsilon?: number, checkVerticesInsteadOfIndices?: boolean, generateEdgesLines?: boolean, options?: IEdgesRendererOptions);
    protected _prepareRessources(): void;
    /** @internal */
    _rebuild(): void;
    /**
     * Releases the required resources for the edges renderer
     */
    dispose(): void;
    protected _processEdgeForAdjacencies(pa: number, pb: number, p0: number, p1: number, p2: number): number;
    protected _processEdgeForAdjacenciesWithVertices(pa: Vector3, pb: Vector3, p0: Vector3, p1: Vector3, p2: Vector3): number;
    /**
     * Checks if the pair of p0 and p1 is en edge
     * @param faceIndex
     * @param edge
     * @param faceNormals
     * @param  p0
     * @param  p1
     * @private
     */
    protected _checkEdge(faceIndex: number, edge: number, faceNormals: Array<Vector3>, p0: Vector3, p1: Vector3): void;
    /**
     * push line into the position, normal and index buffer
     * @param p0
     * @param p1
     * @param offset
     * @protected
     */
    protected createLine(p0: Vector3, p1: Vector3, offset: number): void;
    /**
     * See https://playground.babylonjs.com/#R3JR6V#1 for a visual display of the algorithm
     * @param edgePoints
     * @param indexTriangle
     * @param indices
     * @param remapVertexIndices
     */
    private _tessellateTriangle;
    private _generateEdgesLinesAlternate;
    /**
     * Generates lines edges from adjacencjes
     * @private
     */
    _generateEdgesLines(): void;
    /**
     * Checks whether or not the edges renderer is ready to render.
     * @returns true if ready, otherwise false.
     */
    isReady(): boolean;
    /**
     * Renders the edges of the attached mesh,
     */
    render(): void;
}
/**
 * LineEdgesRenderer for LineMeshes to remove unnecessary triangulation
 */
export class LineEdgesRenderer extends EdgesRenderer {
    /**
     * This constructor turns off auto generating edges line in Edges Renderer to make it here.
     * @param  source LineMesh used to generate edges
     * @param  epsilon not important (specified angle for edge detection)
     * @param  checkVerticesInsteadOfIndices not important for LineMesh
     */
    constructor(source: AbstractMesh, epsilon?: number, checkVerticesInsteadOfIndices?: boolean);
    /**
     * Generate edges for each line in LinesMesh. Every Line should be rendered as edge.
     */
    _generateEdgesLines(): void;
}

}
declare module "babylonjs/Rendering/fluidRenderer/fluidRenderer" {
import { Scene } from "babylonjs/scene";
import { FloatArray, Nullable } from "babylonjs/types";
import { Camera } from "babylonjs/Cameras/camera";
import { IParticleSystem } from "babylonjs/Particles/IParticleSystem";
import { ISceneComponent } from "babylonjs/sceneComponent";
import { FluidRenderingObject } from "babylonjs/Rendering/fluidRenderer/fluidRenderingObject";
import { FluidRenderingTargetRenderer } from "babylonjs/Rendering/fluidRenderer/fluidRenderingTargetRenderer";
import "babylonjs/Shaders/fluidRenderingParticleDepth.vertex";
import "babylonjs/Shaders/fluidRenderingParticleDepth.fragment";
import "babylonjs/Shaders/fluidRenderingParticleThickness.vertex";
import "babylonjs/Shaders/fluidRenderingParticleThickness.fragment";
import "babylonjs/Shaders/fluidRenderingParticleDiffuse.vertex";
import "babylonjs/Shaders/fluidRenderingParticleDiffuse.fragment";
import "babylonjs/Shaders/fluidRenderingBilateralBlur.fragment";
import "babylonjs/Shaders/fluidRenderingStandardBlur.fragment";
import "babylonjs/Shaders/fluidRenderingRender.fragment";
module "babylonjs/abstractScene" {
    interface AbstractScene {
        /** @internal (Backing field) */
        _fluidRenderer: Nullable<FluidRenderer>;
        /**
         * Gets or Sets the fluid renderer associated to the scene.
         */
        fluidRenderer: Nullable<FluidRenderer>;
        /**
         * Enables the fluid renderer and associates it with the scene
         * @returns the FluidRenderer
         */
        enableFluidRenderer(): Nullable<FluidRenderer>;
        /**
         * Disables the fluid renderer associated with the scene
         */
        disableFluidRenderer(): void;
    }
}
/**
 * Defines the fluid renderer scene component responsible to render objects as fluids
 */
export class FluidRendererSceneComponent implements ISceneComponent {
    /**
     * The component name helpful to identify the component in the list of scene components.
     */
    readonly name: string;
    /**
     * The scene the component belongs to.
     */
    scene: Scene;
    /**
     * Creates a new instance of the component for the given scene
     * @param scene Defines the scene to register the component in
     */
    constructor(scene: Scene);
    /**
     * Registers the component in a given scene
     */
    register(): void;
    private _gatherActiveCameraRenderTargets;
    private _afterCameraDraw;
    /**
     * Rebuilds the elements related to this component in case of
     * context lost for instance.
     */
    rebuild(): void;
    /**
     * Disposes the component and the associated resources
     */
    dispose(): void;
}
/**
 * An object rendered as a fluid.
 * It consists of the object itself as well as the render target renderer (which is used to generate the textures (render target) needed for fluid rendering)
 */
export interface IFluidRenderingRenderObject {
    /** object rendered as a fluid */
    object: FluidRenderingObject;
    /** target renderer used to render the fluid object */
    targetRenderer: FluidRenderingTargetRenderer;
}
/**
 * Class responsible for fluid rendering.
 * It is implementing the method described in https://developer.download.nvidia.com/presentations/2010/gdc/Direct3D_Effects.pdf
 */
export class FluidRenderer {
    /** @internal */
    static _SceneComponentInitialization(scene: Scene): void;
    private _scene;
    private _engine;
    private _onEngineResizeObserver;
    private _cameras;
    /** Retrieves all the render objects managed by the class */
    readonly renderObjects: Array<IFluidRenderingRenderObject>;
    /** Retrieves all the render target renderers managed by the class */
    readonly targetRenderers: FluidRenderingTargetRenderer[];
    /**
     * Initializes the class
     * @param scene Scene in which the objects are part of
     */
    constructor(scene: Scene);
    /**
     * Reinitializes the class
     * Can be used if you change the object priority (FluidRenderingObject.priority), to make sure the objects are rendered in the right order
     */
    recreate(): void;
    /**
     * Gets the render object corresponding to a particle system (null if the particle system is not rendered as a fluid)
     * @param ps The particle system
     * @returns the render object corresponding to this particle system if any, otherwise null
     */
    getRenderObjectFromParticleSystem(ps: IParticleSystem): Nullable<IFluidRenderingRenderObject>;
    /**
     * Adds a particle system to the fluid renderer.
     * Note that you should not normally call this method directly, as you can simply use the renderAsFluid property of the ParticleSystem/GPUParticleSystem class
     * @param ps particle system
     * @param generateDiffuseTexture True if you want to generate a diffuse texture from the particle system and use it as part of the fluid rendering (default: false)
     * @param targetRenderer The target renderer used to display the particle system as a fluid. If not provided, the method will create a new one
     * @param camera The camera used by the target renderer (if the target renderer is created by the method)
     * @returns the render object corresponding to the particle system
     */
    addParticleSystem(ps: IParticleSystem, generateDiffuseTexture?: boolean, targetRenderer?: FluidRenderingTargetRenderer, camera?: Camera): IFluidRenderingRenderObject;
    /**
     * Adds a custom particle set to the fluid renderer.
     * @param buffers The list of buffers (should contain at least a "position" buffer!)
     * @param numParticles Number of particles in each buffer
     * @param generateDiffuseTexture True if you want to generate a diffuse texture from buffers and use it as part of the fluid rendering (default: false). For the texture to be generated correctly, you need a "color" buffer in the set!
     * @param targetRenderer The target renderer used to display the particle system as a fluid. If not provided, the method will create a new one
     * @param camera The camera used by the target renderer (if the target renderer is created by the method)
     * @returns the render object corresponding to the custom particle set
     */
    addCustomParticles(buffers: {
        [key: string]: FloatArray;
    }, numParticles: number, generateDiffuseTexture?: boolean, targetRenderer?: FluidRenderingTargetRenderer, camera?: Camera): IFluidRenderingRenderObject;
    /**
     * Removes a render object from the fluid renderer
     * @param renderObject the render object to remove
     * @param removeUnusedTargetRenderer True to remove/dispose of the target renderer if it's not used anymore (default: true)
     * @returns True if the render object has been found and released, else false
     */
    removeRenderObject(renderObject: IFluidRenderingRenderObject, removeUnusedTargetRenderer?: boolean): boolean;
    private _sortRenderingObjects;
    private _removeUnusedTargetRenderers;
    private _getParticleSystemIndex;
    private _initialize;
    private _setParticleSizeForRenderTargets;
    private _setUseVelocityForRenderObject;
    /** @internal */
    _prepareRendering(): void;
    /** @internal */
    _render(forCamera?: Camera): void;
    /**
     * Disposes of all the ressources used by the class
     */
    dispose(): void;
}

}
declare module "babylonjs/Rendering/fluidRenderer/fluidRenderingDepthTextureCopy" {
import { Engine } from "babylonjs/Engines/engine";
import { RenderTargetWrapper } from "babylonjs/Engines/renderTargetWrapper";
import { InternalTexture } from "babylonjs/Materials/Textures/internalTexture";
/** @internal */
export class FluidRenderingDepthTextureCopy {
    private _engine;
    private _depthRTWrapper;
    private _copyTextureToTexture;
    get depthRTWrapper(): RenderTargetWrapper;
    constructor(engine: Engine, width: number, height: number, samples?: number);
    copy(source: InternalTexture): boolean;
    dispose(): void;
}

}
declare module "babylonjs/Rendering/fluidRenderer/fluidRenderingObject" {
import { VertexBuffer } from "babylonjs/Buffers/buffer";
import { DataBuffer } from "babylonjs/Buffers/dataBuffer";
import { Engine } from "babylonjs/Engines/engine";
import { EffectWrapper } from "babylonjs/Materials/effectRenderer";
import { Observable } from "babylonjs/Misc/observable";
import { Scene } from "babylonjs/scene";
import { Nullable } from "babylonjs/types";
/**
 * Defines the base object used for fluid rendering.
 * It is based on a list of vertices (particles)
 */
export abstract class FluidRenderingObject {
    protected _scene: Scene;
    protected _engine: Engine;
    protected _effectsAreDirty: boolean;
    protected _depthEffectWrapper: Nullable<EffectWrapper>;
    protected _thicknessEffectWrapper: Nullable<EffectWrapper>;
    /** Defines the priority of the object. Objects will be rendered in ascending order of priority */
    priority: number;
    protected _particleSize: number;
    /** Observable triggered when the size of the particle is changed */
    onParticleSizeChanged: Observable<FluidRenderingObject>;
    /** Gets or sets the size of the particle */
    get particleSize(): number;
    set particleSize(size: number);
    /** Defines the alpha value of a particle */
    particleThicknessAlpha: number;
    /** Indicates if the object uses instancing or not */
    get useInstancing(): boolean;
    private _useVelocity;
    /** Indicates if velocity of particles should be used when rendering the object. The vertex buffer set must contain a "velocity" buffer for this to work! */
    get useVelocity(): boolean;
    set useVelocity(use: boolean);
    private _hasVelocity;
    /**
     * Gets the vertex buffers
     */
    abstract get vertexBuffers(): {
        [key: string]: VertexBuffer;
    };
    /**
     * Gets the index buffer (or null if the object is using instancing)
     */
    get indexBuffer(): Nullable<DataBuffer>;
    /**
     * Gets the name of the class
     */
    getClassName(): string;
    /**
     * Instantiates a fluid rendering object
     * @param scene The scene the object is part of
     */
    constructor(scene: Scene);
    protected _createEffects(): void;
    /**
     * Indicates if the object is ready to be rendered
     * @returns True if everything is ready for the object to be rendered, otherwise false
     */
    isReady(): boolean;
    /**
     * Gets the number of particles (vertices) of this object
     * @returns The number of particles
     */
    abstract get numParticles(): number;
    /**
     * Render the depth texture for this object
     */
    renderDepthTexture(): void;
    /**
     * Render the thickness texture for this object
     */
    renderThicknessTexture(): void;
    /**
     * Render the diffuse texture for this object
     */
    renderDiffuseTexture(): void;
    /**
     * Releases the ressources used by the class
     */
    dispose(): void;
}

}
declare module "babylonjs/Rendering/fluidRenderer/fluidRenderingObjectCustomParticles" {
import { VertexBuffer } from "babylonjs/Buffers/buffer";
import { Scene } from "babylonjs/scene";
import { FloatArray } from "babylonjs/types";
import { FluidRenderingObject } from "babylonjs/Rendering/fluidRenderer/fluidRenderingObject";
/**
 * Defines a rendering object based on a list of custom buffers
 * The list must contain at least a "position" buffer!
 */
export class FluidRenderingObjectCustomParticles extends FluidRenderingObject {
    private _numParticles;
    private _diffuseEffectWrapper;
    private _vertexBuffers;
    /**
     * Gets the name of the class
     */
    getClassName(): string;
    /**
     * Gets the vertex buffers
     */
    get vertexBuffers(): {
        [key: string]: VertexBuffer;
    };
    /**
     * Creates a new instance of the class
     * @param scene The scene the particles should be rendered into
     * @param buffers The list of buffers (must contain at least one "position" buffer!). Note that you don't have to pass all (or any!) buffers at once in the constructor, you can use the addBuffers method to add more later.
     * @param numParticles Number of vertices to take into account from the buffers
     */
    constructor(scene: Scene, buffers: {
        [key: string]: FloatArray;
    }, numParticles: number);
    /**
     * Add some new buffers
     * @param buffers List of buffers
     */
    addBuffers(buffers: {
        [key: string]: FloatArray;
    }): void;
    protected _createEffects(): void;
    /**
     * Indicates if the object is ready to be rendered
     * @returns True if everything is ready for the object to be rendered, otherwise false
     */
    isReady(): boolean;
    /**
     * Gets the number of particles in this object
     * @returns The number of particles
     */
    get numParticles(): number;
    /**
     * Sets the number of particles in this object
     * @param num The number of particles to take into account
     */
    setNumParticles(num: number): void;
    /**
     * Render the diffuse texture for this object
     */
    renderDiffuseTexture(): void;
    /**
     * Releases the ressources used by the class
     */
    dispose(): void;
}

}
declare module "babylonjs/Rendering/fluidRenderer/fluidRenderingObjectParticleSystem" {
import { VertexBuffer } from "babylonjs/Buffers/buffer";
import { DataBuffer } from "babylonjs/Buffers/dataBuffer";
import { IParticleSystem } from "babylonjs/Particles/IParticleSystem";
import { Scene } from "babylonjs/scene";
import { Nullable } from "babylonjs/types";
import { FluidRenderingObject } from "babylonjs/Rendering/fluidRenderer/fluidRenderingObject";
/**
 * Defines a rendering object based on a particle system
 */
export class FluidRenderingObjectParticleSystem extends FluidRenderingObject {
    private _particleSystem;
    private _originalRender;
    private _blendMode;
    private _onBeforeDrawParticleObserver;
    private _updateInAnimate;
    /** Gets the particle system */
    get particleSystem(): IParticleSystem;
    /**
     * Gets the name of the class
     */
    getClassName(): string;
    private _useTrueRenderingForDiffuseTexture;
    /**
     * Gets or sets a boolean indicating that the diffuse texture should be generated based on the regular rendering of the particle system (default: true).
     * Sometimes, generating the diffuse texture this way may be sub-optimal. In that case, you can disable this property, in which case the particle system will be
     * rendered using a ALPHA_COMBINE mode instead of the one used by the particle system.
     */
    get useTrueRenderingForDiffuseTexture(): boolean;
    set useTrueRenderingForDiffuseTexture(use: boolean);
    /**
     * Gets the vertex buffers
     */
    get vertexBuffers(): {
        [key: string]: VertexBuffer;
    };
    /**
     * Gets the index buffer (or null if the object is using instancing)
     */
    get indexBuffer(): Nullable<DataBuffer>;
    /**
     * Creates a new instance of the class
     * @param scene The scene the particle system is part of
     * @param ps The particle system
     */
    constructor(scene: Scene, ps: IParticleSystem);
    /**
     * Indicates if the object is ready to be rendered
     * @returns True if everything is ready for the object to be rendered, otherwise false
     */
    isReady(): boolean;
    /**
     * Gets the number of particles in this particle system
     * @returns The number of particles
     */
    get numParticles(): number;
    /**
     * Render the diffuse texture for this object
     */
    renderDiffuseTexture(): void;
    /**
     * Releases the ressources used by the class
     */
    dispose(): void;
}

}
declare module "babylonjs/Rendering/fluidRenderer/fluidRenderingTargetRenderer" {
import { Camera } from "babylonjs/Cameras/camera";
import { Engine } from "babylonjs/Engines/engine";
import { BaseTexture } from "babylonjs/Materials/Textures/baseTexture";
import { InternalTexture } from "babylonjs/Materials/Textures/internalTexture";
import { Color3, Color4 } from "babylonjs/Maths/math.color";
import { Matrix, Vector3 } from "babylonjs/Maths/math.vector";
import { Observable } from "babylonjs/Misc/observable";
import { PostProcess } from "babylonjs/PostProcesses/postProcess";
import { Scene } from "babylonjs/scene";
import { Nullable } from "babylonjs/types";
import { FluidRenderingObject } from "babylonjs/Rendering/fluidRenderer/fluidRenderingObject";
import { FluidRenderingTextures } from "babylonjs/Rendering/fluidRenderer/fluidRenderingTextures";
/**
 * Textures that can be displayed as a debugging tool
 */
export enum FluidRenderingDebug {
    DepthTexture = 0,
    DepthBlurredTexture = 1,
    ThicknessTexture = 2,
    ThicknessBlurredTexture = 3,
    DiffuseTexture = 4,
    Normals = 5,
    DiffuseRendering = 6
}
/**
 * Class used to render an object as a fluid thanks to different render target textures (depth, thickness, diffuse)
 */
export class FluidRenderingTargetRenderer {
    protected _scene: Scene;
    protected _camera: Nullable<Camera>;
    protected _engine: Engine;
    protected _invProjectionMatrix: Matrix;
    protected _depthClearColor: Color4;
    protected _thicknessClearColor: Color4;
    protected _needInitialization: boolean;
    /**
     * Returns true if the class needs to be reinitialized (because of changes in parameterization)
     */
    get needInitialization(): boolean;
    private _generateDiffuseTexture;
    /**
     * Gets or sets a boolean indicating that the diffuse texture should be generated and used for the rendering
     */
    get generateDiffuseTexture(): boolean;
    set generateDiffuseTexture(generate: boolean);
    /**
     * Fluid color. Not used if generateDiffuseTexture is true
     */
    fluidColor: Color3;
    /**
     * Density of the fluid (positive number). The higher the value, the more opaque the fluid.
     */
    density: number;
    /**
     * Strength of the refraction (positive number, but generally between 0 and 0.3).
     */
    refractionStrength: number;
    /**
     * Strength of the fresnel effect (value between 0 and 1). Lower the value if you want to soften the specular effect
     */
    fresnelClamp: number;
    /**
     * Strength of the specular power (positive number). Increase the value to make the specular effect more concentrated
     */
    specularPower: number;
    /**
     * Minimum thickness of the particles (positive number). If useFixedThickness is true, minimumThickness is the thickness used
     */
    minimumThickness: number;
    /**
     * Direction of the light. The fluid is assumed to be lit by a directional light
     */
    dirLight: Vector3;
    private _debugFeature;
    /**
     * Gets or sets the feature (texture) to be debugged. Not used if debug is false
     */
    get debugFeature(): FluidRenderingDebug;
    set debugFeature(feature: FluidRenderingDebug);
    private _debug;
    /**
     * Gets or sets a boolean indicating if we should display a specific texture (given by debugFeature) for debugging purpose
     */
    get debug(): boolean;
    set debug(debug: boolean);
    private _environmentMap?;
    /**
     * Gets or sets the environment map used for the reflection part of the shading
     * If null, no map will be used. If undefined, the scene.environmentMap will be used (if defined)
     */
    get environmentMap(): Nullable<BaseTexture> | undefined;
    set environmentMap(map: Nullable<BaseTexture> | undefined);
    private _enableBlurDepth;
    /**
     * Gets or sets a boolean indicating that the depth texture should be blurred
     */
    get enableBlurDepth(): boolean;
    set enableBlurDepth(enable: boolean);
    private _blurDepthSizeDivisor;
    /**
     * Gets or sets the depth size divisor (positive number, generally between 1 and 4), which is used as a divisor when creating the texture used for blurring the depth
     * For eg. if blurDepthSizeDivisor=2, the texture used to blur the depth will be half the size of the depth texture
     */
    get blurDepthSizeDivisor(): number;
    set blurDepthSizeDivisor(scale: number);
    private _blurDepthFilterSize;
    /**
     * Size of the kernel used to filter the depth blur texture (positive number, generally between 1 and 20 - higher values will require more processing power from the GPU)
     */
    get blurDepthFilterSize(): number;
    set blurDepthFilterSize(filterSize: number);
    private _blurDepthNumIterations;
    /**
     * Number of blurring iterations used to generate the depth blur texture (positive number, generally between 1 and 10 - higher values will require more processing power from the GPU)
     */
    get blurDepthNumIterations(): number;
    set blurDepthNumIterations(numIterations: number);
    private _blurDepthMaxFilterSize;
    /**
     * Maximum size of the kernel used to blur the depth texture (positive number, generally between 1 and 200 - higher values will require more processing power from the GPU when the particles are larger on screen)
     */
    get blurDepthMaxFilterSize(): number;
    set blurDepthMaxFilterSize(maxFilterSize: number);
    private _blurDepthDepthScale;
    /**
     * Depth weight in the calculation when applying the bilateral blur to generate the depth blur texture (positive number, generally between 0 and 100)
     */
    get blurDepthDepthScale(): number;
    set blurDepthDepthScale(scale: number);
    private _enableBlurThickness;
    /**
     * Gets or sets a boolean indicating that the thickness texture should be blurred
     */
    get enableBlurThickness(): boolean;
    set enableBlurThickness(enable: boolean);
    private _blurThicknessSizeDivisor;
    /**
     * Gets or sets the thickness size divisor (positive number, generally between 1 and 4), which is used as a divisor when creating the texture used for blurring the thickness
     * For eg. if blurThicknessSizeDivisor=2, the texture used to blur the thickness will be half the size of the thickness texture
     */
    get blurThicknessSizeDivisor(): number;
    set blurThicknessSizeDivisor(scale: number);
    private _blurThicknessFilterSize;
    /**
     * Size of the kernel used to filter the thickness blur texture (positive number, generally between 1 and 20 - higher values will require more processing power from the GPU)
     */
    get blurThicknessFilterSize(): number;
    set blurThicknessFilterSize(filterSize: number);
    private _blurThicknessNumIterations;
    /**
     * Number of blurring iterations used to generate the thickness blur texture (positive number, generally between 1 and 10 - higher values will require more processing power from the GPU)
     */
    get blurThicknessNumIterations(): number;
    set blurThicknessNumIterations(numIterations: number);
    private _useFixedThickness;
    /**
     * Gets or sets a boolean indicating that a fixed thickness should be used instead of generating a thickness texture
     */
    get useFixedThickness(): boolean;
    set useFixedThickness(use: boolean);
    /** @internal */
    _bgDepthTexture: Nullable<InternalTexture>;
    /** @internal */
    _onUseVelocityChanged: Observable<FluidRenderingTargetRenderer>;
    private _useVelocity;
    /**
     * Gets or sets a boolean indicating that the velocity should be used when rendering the particles as a fluid.
     * Note: the vertex buffers must contain a "velocity" buffer for this to work!
     */
    get useVelocity(): boolean;
    set useVelocity(use: boolean);
    private _depthMapSize;
    /**
     * Defines the size of the depth texture.
     * If null, the texture will have the size of the screen
     */
    get depthMapSize(): Nullable<number>;
    set depthMapSize(size: Nullable<number>);
    private _thicknessMapSize;
    /**
     * Defines the size of the thickness texture.
     * If null, the texture will have the size of the screen
     */
    get thicknessMapSize(): Nullable<number>;
    set thicknessMapSize(size: Nullable<number>);
    private _diffuseMapSize;
    /**
     * Defines the size of the diffuse texture.
     * If null, the texture will have the size of the screen
     */
    get diffuseMapSize(): Nullable<number>;
    set diffuseMapSize(size: Nullable<number>);
    private _samples;
    /**
     * Gets or sets the number of samples used by MSAA
     * Note: changing this value in WebGL does not work because depth/stencil textures can't be created with MSAA (see https://github.com/BabylonJS/Babylon.js/issues/12444)
     */
    get samples(): number;
    set samples(samples: number);
    /**
     * Gets the camera used for the rendering
     */
    get camera(): Nullable<Camera>;
    /** @internal */
    _renderPostProcess: Nullable<PostProcess>;
    /** @internal */
    _depthRenderTarget: Nullable<FluidRenderingTextures>;
    /** @internal */
    _diffuseRenderTarget: Nullable<FluidRenderingTextures>;
    /** @internal */
    _thicknessRenderTarget: Nullable<FluidRenderingTextures>;
    /**
     * Creates an instance of the class
     * @param scene Scene used to render the fluid object into
     * @param camera Camera used to render the fluid object. If not provided, use the active camera of the scene instead
     */
    constructor(scene: Scene, camera?: Camera);
    /** @internal */
    _initialize(): void;
    protected _setBlurParameters(renderTarget?: Nullable<FluidRenderingTextures>): void;
    protected _setBlurDepthParameters(): void;
    protected _setBlurThicknessParameters(): void;
    protected _initializeRenderTarget(renderTarget: FluidRenderingTextures): void;
    protected _createLiquidRenderingPostProcess(): void;
    /** @internal */
    _clearTargets(): void;
    /** @internal */
    _render(fluidObject: FluidRenderingObject): void;
    /**
     * Releases all the ressources used by the class
     * @param onlyPostProcesses If true, releases only the ressources used by the render post processes
     */
    dispose(onlyPostProcesses?: boolean): void;
}

}
declare module "babylonjs/Rendering/fluidRenderer/fluidRenderingTextures" {
import { Camera } from "babylonjs/Cameras/camera";
import { Engine } from "babylonjs/Engines/engine";
import { RenderTargetWrapper } from "babylonjs/Engines/renderTargetWrapper";
import { Texture } from "babylonjs/Materials/Textures/texture";
import { ThinTexture } from "babylonjs/Materials/Textures/thinTexture";
import { PostProcess } from "babylonjs/PostProcesses/postProcess";
import { Scene } from "babylonjs/scene";
import { Nullable } from "babylonjs/types";
import { Observable } from "babylonjs/Misc/observable";
/** @internal */
export class FluidRenderingTextures {
    protected _name: string;
    protected _scene: Scene;
    protected _camera: Nullable<Camera>;
    protected _engine: Engine;
    protected _width: number;
    protected _height: number;
    protected _blurTextureSizeX: number;
    protected _blurTextureSizeY: number;
    protected _textureType: number;
    protected _textureFormat: number;
    protected _blurTextureType: number;
    protected _blurTextureFormat: number;
    protected _useStandardBlur: boolean;
    protected _generateDepthBuffer: boolean;
    protected _samples: number;
    protected _postProcessRunningIndex: number;
    protected _rt: Nullable<RenderTargetWrapper>;
    protected _texture: Nullable<Texture>;
    protected _rtBlur: Nullable<RenderTargetWrapper>;
    protected _textureBlurred: Nullable<Texture>;
    protected _blurPostProcesses: Nullable<PostProcess[]>;
    enableBlur: boolean;
    blurSizeDivisor: number;
    blurFilterSize: number;
    private _blurNumIterations;
    get blurNumIterations(): number;
    set blurNumIterations(numIterations: number);
    blurMaxFilterSize: number;
    blurDepthScale: number;
    particleSize: number;
    onDisposeObservable: Observable<FluidRenderingTextures>;
    get renderTarget(): Nullable<RenderTargetWrapper>;
    get renderTargetBlur(): Nullable<RenderTargetWrapper>;
    get texture(): Nullable<Texture>;
    get textureBlur(): Nullable<Texture>;
    constructor(name: string, scene: Scene, width: number, height: number, blurTextureSizeX: number, blurTextureSizeY: number, textureType?: number, textureFormat?: number, blurTextureType?: number, blurTextureFormat?: number, useStandardBlur?: boolean, camera?: Nullable<Camera>, generateDepthBuffer?: boolean, samples?: number);
    initialize(): void;
    applyBlurPostProcesses(): void;
    protected _createRenderTarget(): void;
    protected _createBlurPostProcesses(textureBlurSource: ThinTexture, textureType: number, textureFormat: number, blurSizeDivisor: number, debugName: string, useStandardBlur?: boolean): [RenderTargetWrapper, Texture, PostProcess[]];
    private _fixReusablePostProcess;
    private _getProjectedParticleConstant;
    private _getDepthThreshold;
    dispose(): void;
}

}
declare module "babylonjs/Rendering/fluidRenderer/index" {
export * from "babylonjs/Rendering/fluidRenderer/fluidRenderer";
export * from "babylonjs/Rendering/fluidRenderer/fluidRenderingObject";
export * from "babylonjs/Rendering/fluidRenderer/fluidRenderingObjectParticleSystem";
export * from "babylonjs/Rendering/fluidRenderer/fluidRenderingObjectCustomParticles";
export * from "babylonjs/Rendering/fluidRenderer/fluidRenderingTargetRenderer";

}
declare module "babylonjs/Rendering/geometryBufferRenderer" {
import { Matrix } from "babylonjs/Maths/math.vector";
import { SubMesh } from "babylonjs/Meshes/subMesh";
import { InternalTexture } from "babylonjs/Materials/Textures/internalTexture";
import { MultiRenderTarget } from "babylonjs/Materials/Textures/multiRenderTarget";
import { PrePassRenderer } from "babylonjs/Rendering/prePassRenderer";
import { Scene } from "babylonjs/scene";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { Nullable } from "babylonjs/types";
import "babylonjs/Shaders/geometry.fragment";
import "babylonjs/Shaders/geometry.vertex";
/** @internal */
interface ISavedTransformationMatrix {
    world: Matrix;
    viewProjection: Matrix;
}
/**
 * This renderer is helpful to fill one of the render target with a geometry buffer.
 */
export class GeometryBufferRenderer {
    /**
     * Constant used to retrieve the depth texture index in the G-Buffer textures array
     * using getIndex(GeometryBufferRenderer.DEPTH_TEXTURE_INDEX)
     */
    static readonly DEPTH_TEXTURE_TYPE: number;
    /**
     * Constant used to retrieve the normal texture index in the G-Buffer textures array
     * using getIndex(GeometryBufferRenderer.NORMAL_TEXTURE_INDEX)
     */
    static readonly NORMAL_TEXTURE_TYPE: number;
    /**
     * Constant used to retrieve the position texture index in the G-Buffer textures array
     * using getIndex(GeometryBufferRenderer.POSITION_TEXTURE_INDEX)
     */
    static readonly POSITION_TEXTURE_TYPE: number;
    /**
     * Constant used to retrieve the velocity texture index in the G-Buffer textures array
     * using getIndex(GeometryBufferRenderer.VELOCITY_TEXTURE_INDEX)
     */
    static readonly VELOCITY_TEXTURE_TYPE: number;
    /**
     * Constant used to retrieve the reflectivity texture index in the G-Buffer textures array
     * using the getIndex(GeometryBufferRenderer.REFLECTIVITY_TEXTURE_TYPE)
     */
    static readonly REFLECTIVITY_TEXTURE_TYPE: number;
    /**
     * Dictionary used to store the previous transformation matrices of each rendered mesh
     * in order to compute objects velocities when enableVelocity is set to "true"
     * @internal
     */
    _previousTransformationMatrices: {
        [index: number]: ISavedTransformationMatrix;
    };
    /**
     * Dictionary used to store the previous bones transformation matrices of each rendered mesh
     * in order to compute objects velocities when enableVelocity is set to "true"
     * @internal
     */
    _previousBonesTransformationMatrices: {
        [index: number]: Float32Array;
    };
    /**
     * Array used to store the ignored skinned meshes while computing velocity map (typically used by the motion blur post-process).
     * Avoids computing bones velocities and computes only mesh's velocity itself (position, rotation, scaling).
     */
    excludedSkinnedMeshesFromVelocity: AbstractMesh[];
    /** Gets or sets a boolean indicating if transparent meshes should be rendered */
    renderTransparentMeshes: boolean;
    private _scene;
    private _resizeObserver;
    private _multiRenderTarget;
    private _ratio;
    private _enablePosition;
    private _enableVelocity;
    private _enableReflectivity;
    private _depthFormat;
    private _positionIndex;
    private _velocityIndex;
    private _reflectivityIndex;
    private _depthIndex;
    private _normalIndex;
    private _linkedWithPrePass;
    private _prePassRenderer;
    private _attachments;
    private _useUbo;
    protected _cachedDefines: string;
    /**
     * @internal
     * Sets up internal structures to share outputs with PrePassRenderer
     * This method should only be called by the PrePassRenderer itself
     */
    _linkPrePassRenderer(prePassRenderer: PrePassRenderer): void;
    /**
     * @internal
     * Separates internal structures from PrePassRenderer so the geometry buffer can now operate by itself.
     * This method should only be called by the PrePassRenderer itself
     */
    _unlinkPrePassRenderer(): void;
    /**
     * @internal
     * Resets the geometry buffer layout
     */
    _resetLayout(): void;
    /**
     * @internal
     * Replaces a texture in the geometry buffer renderer
     * Useful when linking textures of the prepass renderer
     */
    _forceTextureType(geometryBufferType: number, index: number): void;
    /**
     * @internal
     * Sets texture attachments
     * Useful when linking textures of the prepass renderer
     */
    _setAttachments(attachments: number[]): void;
    /**
     * @internal
     * Replaces the first texture which is hard coded as a depth texture in the geometry buffer
     * Useful when linking textures of the prepass renderer
     */
    _linkInternalTexture(internalTexture: InternalTexture): void;
    /**
     * Gets the render list (meshes to be rendered) used in the G buffer.
     */
    get renderList(): Nullable<AbstractMesh[]>;
    /**
     * Set the render list (meshes to be rendered) used in the G buffer.
     */
    set renderList(meshes: Nullable<AbstractMesh[]>);
    /**
     * Gets whether or not G buffer are supported by the running hardware.
     * This requires draw buffer supports
     */
    get isSupported(): boolean;
    /**
     * Returns the index of the given texture type in the G-Buffer textures array
     * @param textureType The texture type constant. For example GeometryBufferRenderer.POSITION_TEXTURE_INDEX
     * @returns the index of the given texture type in the G-Buffer textures array
     */
    getTextureIndex(textureType: number): number;
    /**
     * Gets a boolean indicating if objects positions are enabled for the G buffer.
     */
    get enablePosition(): boolean;
    /**
     * Sets whether or not objects positions are enabled for the G buffer.
     */
    set enablePosition(enable: boolean);
    /**
     * Gets a boolean indicating if objects velocities are enabled for the G buffer.
     */
    get enableVelocity(): boolean;
    /**
     * Sets whether or not objects velocities are enabled for the G buffer.
     */
    set enableVelocity(enable: boolean);
    /**
     * Gets a boolean indicating if objects reflectivity are enabled in the G buffer.
     */
    get enableReflectivity(): boolean;
    /**
     * Sets whether or not objects reflectivity are enabled for the G buffer.
     * For Metallic-Roughness workflow with ORM texture, we assume that ORM texture is defined according to the default layout:
     * pbr.useRoughnessFromMetallicTextureAlpha = false;
     * pbr.useRoughnessFromMetallicTextureGreen = true;
     * pbr.useMetallnessFromMetallicTextureBlue = true;
     */
    set enableReflectivity(enable: boolean);
    /**
     * Gets the scene associated with the buffer.
     */
    get scene(): Scene;
    /**
     * Gets the ratio used by the buffer during its creation.
     * How big is the buffer related to the main canvas.
     */
    get ratio(): number;
    /**
     * @internal
     */
    static _SceneComponentInitialization: (scene: Scene) => void;
    /**
     * Creates a new G Buffer for the scene
     * @param scene The scene the buffer belongs to
     * @param ratio How big is the buffer related to the main canvas (default: 1)
     * @param depthFormat Format of the depth texture (default: Constants.TEXTUREFORMAT_DEPTH16)
     */
    constructor(scene: Scene, ratio?: number, depthFormat?: number);
    /**
     * Checks whether everything is ready to render a submesh to the G buffer.
     * @param subMesh the submesh to check readiness for
     * @param useInstances is the mesh drawn using instance or not
     * @returns true if ready otherwise false
     */
    isReady(subMesh: SubMesh, useInstances: boolean): boolean;
    /**
     * Gets the current underlying G Buffer.
     * @returns the buffer
     */
    getGBuffer(): MultiRenderTarget;
    /**
     * Gets the number of samples used to render the buffer (anti aliasing).
     */
    get samples(): number;
    /**
     * Sets the number of samples used to render the buffer (anti aliasing).
     */
    set samples(value: number);
    /**
     * Disposes the renderer and frees up associated resources.
     */
    dispose(): void;
    private _assignRenderTargetIndices;
    protected _createRenderTargets(): void;
    private _copyBonesTransformationMatrices;
}
export {};

}
declare module "babylonjs/Rendering/geometryBufferRendererSceneComponent" {
import { Nullable } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
import { ISceneComponent } from "babylonjs/sceneComponent";
import { GeometryBufferRenderer } from "babylonjs/Rendering/geometryBufferRenderer";
module "babylonjs/scene" {
    interface Scene {
        /** @internal (Backing field) */
        _geometryBufferRenderer: Nullable<GeometryBufferRenderer>;
        /**
         * Gets or Sets the current geometry buffer associated to the scene.
         */
        geometryBufferRenderer: Nullable<GeometryBufferRenderer>;
        /**
         * Enables a GeometryBufferRender and associates it with the scene
         * @param ratio defines the scaling ratio to apply to the renderer (1 by default which means same resolution)
         * @param depthFormat Format of the depth texture (default: Constants.TEXTUREFORMAT_DEPTH16)
         * @returns the GeometryBufferRenderer
         */
        enableGeometryBufferRenderer(ratio?: number, depthFormat?: number): Nullable<GeometryBufferRenderer>;
        /**
         * Disables the GeometryBufferRender associated with the scene
         */
        disableGeometryBufferRenderer(): void;
    }
}
/**
 * Defines the Geometry Buffer scene component responsible to manage a G-Buffer useful
 * in several rendering techniques.
 */
export class GeometryBufferRendererSceneComponent implements ISceneComponent {
    /**
     * The component name helpful to identify the component in the list of scene components.
     */
    readonly name: string;
    /**
     * The scene the component belongs to.
     */
    scene: Scene;
    /**
     * Creates a new instance of the component for the given scene
     * @param scene Defines the scene to register the component in
     */
    constructor(scene: Scene);
    /**
     * Registers the component in a given scene
     */
    register(): void;
    /**
     * Rebuilds the elements related to this component in case of
     * context lost for instance.
     */
    rebuild(): void;
    /**
     * Disposes the component and the associated resources
     */
    dispose(): void;
    private _gatherRenderTargets;
}

}
declare module "babylonjs/Rendering/index" {
export * from "babylonjs/Rendering/boundingBoxRenderer";
export * from "babylonjs/Rendering/depthRenderer";
export * from "babylonjs/Rendering/depthRendererSceneComponent";
export * from "babylonjs/Rendering/depthPeelingRenderer";
export * from "babylonjs/Rendering/depthPeelingSceneComponent";
export * from "babylonjs/Rendering/edgesRenderer";
export * from "babylonjs/Rendering/geometryBufferRenderer";
export * from "babylonjs/Rendering/geometryBufferRendererSceneComponent";
export * from "babylonjs/Rendering/prePassRenderer";
export * from "babylonjs/Rendering/prePassRendererSceneComponent";
export * from "babylonjs/Rendering/subSurfaceSceneComponent";
export * from "babylonjs/Rendering/outlineRenderer";
export * from "babylonjs/Rendering/renderingGroup";
export * from "babylonjs/Rendering/renderingManager";
export * from "babylonjs/Rendering/utilityLayerRenderer";
export * from "babylonjs/Rendering/fluidRenderer/index";

}
declare module "babylonjs/Rendering/motionBlurConfiguration" {
import { PrePassEffectConfiguration } from "babylonjs/Rendering/prePassEffectConfiguration";
/**
 * Contains all parameters needed for the prepass to perform
 * motion blur
 */
export class MotionBlurConfiguration implements PrePassEffectConfiguration {
    /**
     * Is motion blur enabled
     */
    enabled: boolean;
    /**
     * Name of the configuration
     */
    name: string;
    /**
     * Textures that should be present in the MRT for this effect to work
     */
    readonly texturesRequired: number[];
}

}
declare module "babylonjs/Rendering/outlineRenderer" {
import { SubMesh } from "babylonjs/Meshes/subMesh";
import { _InstancesBatch } from "babylonjs/Meshes/mesh";
import { Scene } from "babylonjs/scene";
import { ISceneComponent } from "babylonjs/sceneComponent";
import "babylonjs/Shaders/outline.fragment";
import "babylonjs/Shaders/outline.vertex";
module "babylonjs/scene" {
    interface Scene {
        /** @internal */
        _outlineRenderer: OutlineRenderer;
        /**
         * Gets the outline renderer associated with the scene
         * @returns a OutlineRenderer
         */
        getOutlineRenderer(): OutlineRenderer;
    }
}
module "babylonjs/Meshes/abstractMesh" {
    interface AbstractMesh {
        /** @internal (Backing field) */
        _renderOutline: boolean;
        /**
         * Gets or sets a boolean indicating if the outline must be rendered as well
         * @see https://www.babylonjs-playground.com/#10WJ5S#3
         */
        renderOutline: boolean;
        /** @internal (Backing field) */
        _renderOverlay: boolean;
        /**
         * Gets or sets a boolean indicating if the overlay must be rendered as well
         * @see https://www.babylonjs-playground.com/#10WJ5S#2
         */
        renderOverlay: boolean;
    }
}
/**
 * This class is responsible to draw the outline/overlay of meshes.
 * It should not be used directly but through the available method on mesh.
 */
export class OutlineRenderer implements ISceneComponent {
    /**
     * Stencil value used to avoid outline being seen within the mesh when the mesh is transparent
     */
    private static _StencilReference;
    /**
     * The name of the component. Each component must have a unique name.
     */
    name: string;
    /**
     * The scene the component belongs to.
     */
    scene: Scene;
    /**
     * Defines a zOffset default Factor to prevent zFighting between the overlay and the mesh.
     */
    zOffset: number;
    /**
     * Defines a zOffset default Unit to prevent zFighting between the overlay and the mesh.
     */
    zOffsetUnits: number;
    private _engine;
    private _savedDepthWrite;
    private _passIdForDrawWrapper;
    /**
     * Instantiates a new outline renderer. (There could be only one per scene).
     * @param scene Defines the scene it belongs to
     */
    constructor(scene: Scene);
    /**
     * Register the component to one instance of a scene.
     */
    register(): void;
    /**
     * Rebuilds the elements related to this component in case of
     * context lost for instance.
     */
    rebuild(): void;
    /**
     * Disposes the component and the associated resources.
     */
    dispose(): void;
    /**
     * Renders the outline in the canvas.
     * @param subMesh Defines the sumesh to render
     * @param batch Defines the batch of meshes in case of instances
     * @param useOverlay Defines if the rendering is for the overlay or the outline
     * @param renderPassId Render pass id to use to render the mesh
     */
    render(subMesh: SubMesh, batch: _InstancesBatch, useOverlay?: boolean, renderPassId?: number): void;
    /**
     * Returns whether or not the outline renderer is ready for a given submesh.
     * All the dependencies e.g. submeshes, texture, effect... mus be ready
     * @param subMesh Defines the submesh to check readiness for
     * @param useInstances Defines whether wee are trying to render instances or not
     * @param renderPassId Render pass id to use to render the mesh
     * @returns true if ready otherwise false
     */
    isReady(subMesh: SubMesh, useInstances: boolean, renderPassId?: number): boolean;
    private _beforeRenderingMesh;
    private _afterRenderingMesh;
}

}
declare module "babylonjs/Rendering/prePassEffectConfiguration" {
import { PostProcess } from "babylonjs/PostProcesses/postProcess";
/**
 * Interface for defining prepass effects in the prepass post-process pipeline
 */
export interface PrePassEffectConfiguration {
    /**
     * Name of the effect
     */
    name: string;
    /**
     * Post process to attach for this effect
     */
    postProcess?: PostProcess;
    /**
     * Textures required in the MRT
     */
    texturesRequired: number[];
    /**
     * Is the effect enabled
     */
    enabled: boolean;
    /**
     * Does the output of this prepass need to go through imageprocessing
     */
    needsImageProcessing?: boolean;
    /**
     * Disposes the effect configuration
     */
    dispose?: () => void;
    /**
     * Creates the associated post process
     */
    createPostProcess?: () => PostProcess;
}

}
declare module "babylonjs/Rendering/prePassRenderer" {
import { PrePassRenderTarget } from "babylonjs/Materials/Textures/prePassRenderTarget";
import { Scene } from "babylonjs/scene";
import { Effect } from "babylonjs/Materials/effect";
import { Nullable } from "babylonjs/types";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { Camera } from "babylonjs/Cameras/camera";
import { Material } from "babylonjs/Materials/material";
import { SubMesh } from "babylonjs/Meshes/subMesh";
import { PrePassEffectConfiguration } from "babylonjs/Rendering/prePassEffectConfiguration";
import { RenderTargetTexture } from "babylonjs/Materials/Textures/renderTargetTexture";
/**
 * Renders a pre pass of the scene
 * This means every mesh in the scene will be rendered to a render target texture
 * And then this texture will be composited to the rendering canvas with post processes
 * It is necessary for effects like subsurface scattering or deferred shading
 */
export class PrePassRenderer {
    /**
     * @internal
     */
    static _SceneComponentInitialization: (scene: Scene) => void;
    /**
     * To save performance, we can excluded skinned meshes from the prepass
     */
    excludedSkinnedMesh: AbstractMesh[];
    /**
     * Force material to be excluded from the prepass
     * Can be useful when `useGeometryBufferFallback` is set to `true`
     * and you don't want a material to show in the effect.
     */
    excludedMaterials: Material[];
    private _scene;
    private _engine;
    /**
     * Number of textures in the multi render target texture where the scene is directly rendered
     */
    mrtCount: number;
    private _mrtFormats;
    private _mrtLayout;
    private _mrtNames;
    private _textureIndices;
    private _multiRenderAttachments;
    private _defaultAttachments;
    private _clearAttachments;
    /**
     * Returns the index of a texture in the multi render target texture array.
     * @param type Texture type
     * @returns The index
     */
    getIndex(type: number): number;
    /**
     * How many samples are used for MSAA of the scene render target
     */
    get samples(): number;
    set samples(n: number);
    private static _TextureFormats;
    private _isDirty;
    /**
     * The render target where the scene is directly rendered
     */
    defaultRT: PrePassRenderTarget;
    /**
     * Configuration for prepass effects
     */
    private _effectConfigurations;
    /**
     * @returns the prepass render target for the rendering pass.
     * If we are currently rendering a render target, it returns the PrePassRenderTarget
     * associated with that render target. Otherwise, it returns the scene default PrePassRenderTarget
     */
    getRenderTarget(): PrePassRenderTarget;
    /**
     * @internal
     * Managed by the scene component
     * @param prePassRenderTarget
     */
    _setRenderTarget(prePassRenderTarget: Nullable<PrePassRenderTarget>): void;
    /**
     * Returns true if the currently rendered prePassRenderTarget is the one
     * associated with the scene.
     */
    get currentRTisSceneRT(): boolean;
    private _geometryBuffer;
    /**
     * Prevents the PrePassRenderer from using the GeometryBufferRenderer as a fallback
     */
    doNotUseGeometryRendererFallback: boolean;
    private _refreshGeometryBufferRendererLink;
    private _currentTarget;
    /**
     * All the render targets generated by prepass
     */
    renderTargets: PrePassRenderTarget[];
    private readonly _clearColor;
    private _enabled;
    private _needsCompositionForThisPass;
    private _postProcessesSourceForThisPass;
    /**
     * Indicates if the prepass is enabled
     */
    get enabled(): boolean;
    /**
     * Set to true to disable gamma transform in PrePass.
     * Can be useful in case you already proceed to gamma transform on a material level
     * and your post processes don't need to be in linear color space.
     */
    disableGammaTransform: boolean;
    /**
     * Instantiates a prepass renderer
     * @param scene The scene
     */
    constructor(scene: Scene);
    /**
     * Creates a new PrePassRenderTarget
     * This should be the only way to instantiate a `PrePassRenderTarget`
     * @param name Name of the `PrePassRenderTarget`
     * @param renderTargetTexture RenderTarget the `PrePassRenderTarget` will be attached to.
     * Can be `null` if the created `PrePassRenderTarget` is attached to the scene (default framebuffer).
     * @internal
     */
    _createRenderTarget(name: string, renderTargetTexture: Nullable<RenderTargetTexture>): PrePassRenderTarget;
    /**
     * Indicates if rendering a prepass is supported
     */
    get isSupported(): boolean;
    /**
     * Sets the proper output textures to draw in the engine.
     * @param effect The effect that is drawn. It can be or not be compatible with drawing to several output textures.
     * @param subMesh Submesh on which the effect is applied
     */
    bindAttachmentsForEffect(effect: Effect, subMesh: SubMesh): void;
    private _reinitializeAttachments;
    private _resetLayout;
    private _updateGeometryBufferLayout;
    /**
     * Restores attachments for single texture draw.
     */
    restoreAttachments(): void;
    /**
     * @internal
     */
    _beforeDraw(camera?: Camera, faceIndex?: number, layer?: number): void;
    private _prepareFrame;
    /**
     * Sets an intermediary texture between prepass and postprocesses. This texture
     * will be used as input for post processes
     * @param rt
     * @returns true if there are postprocesses that will use this texture,
     * false if there is no postprocesses - and the function has no effect
     */
    setCustomOutput(rt: RenderTargetTexture): boolean;
    private _renderPostProcesses;
    /**
     * @internal
     */
    _afterDraw(faceIndex?: number, layer?: number): void;
    /**
     * Clears the current prepass render target (in the sense of settings pixels to the scene clear color value)
     * @internal
     */
    _clear(): void;
    private _bindFrameBuffer;
    private _setEnabled;
    private _setRenderTargetEnabled;
    /**
     * Adds an effect configuration to the prepass render target.
     * If an effect has already been added, it won't add it twice and will return the configuration
     * already present.
     * @param cfg the effect configuration
     * @returns the effect configuration now used by the prepass
     */
    addEffectConfiguration(cfg: PrePassEffectConfiguration): PrePassEffectConfiguration;
    private _enable;
    private _disable;
    private _getPostProcessesSource;
    private _setupOutputForThisPass;
    private _linkInternalTexture;
    /**
     * @internal
     */
    _unlinkInternalTexture(prePassRenderTarget: PrePassRenderTarget): void;
    private _needsImageProcessing;
    private _hasImageProcessing;
    /**
     * Internal, gets the first post proces.
     * @param postProcesses
     * @returns the first post process to be run on this camera.
     */
    private _getFirstPostProcess;
    /**
     * Marks the prepass renderer as dirty, triggering a check if the prepass is necessary for the next rendering.
     */
    markAsDirty(): void;
    /**
     * Enables a texture on the MultiRenderTarget for prepass
     * @param types
     */
    private _enableTextures;
    private _update;
    private _markAllMaterialsAsPrePassDirty;
    /**
     * Disposes the prepass renderer.
     */
    dispose(): void;
}

}
declare module "babylonjs/Rendering/prePassRendererSceneComponent" {
import { Nullable } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
import { ISceneComponent } from "babylonjs/sceneComponent";
import { PrePassRenderer } from "babylonjs/Rendering/prePassRenderer";
import { PrePassRenderTarget } from "babylonjs/Materials/Textures/prePassRenderTarget";
module "babylonjs/abstractScene" {
    interface AbstractScene {
        /** @internal (Backing field) */
        _prePassRenderer: Nullable<PrePassRenderer>;
        /**
         * Gets or Sets the current prepass renderer associated to the scene.
         */
        prePassRenderer: Nullable<PrePassRenderer>;
        /**
         * Enables the prepass and associates it with the scene
         * @returns the PrePassRenderer
         */
        enablePrePassRenderer(): Nullable<PrePassRenderer>;
        /**
         * Disables the prepass associated with the scene
         */
        disablePrePassRenderer(): void;
    }
}
module "babylonjs/Materials/Textures/renderTargetTexture" {
    interface RenderTargetTexture {
        /** @internal */
        _prePassRenderTarget: Nullable<PrePassRenderTarget>;
    }
}
/**
 * Defines the Geometry Buffer scene component responsible to manage a G-Buffer useful
 * in several rendering techniques.
 */
export class PrePassRendererSceneComponent implements ISceneComponent {
    /**
     * The component name helpful to identify the component in the list of scene components.
     */
    readonly name: string;
    /**
     * The scene the component belongs to.
     */
    scene: Scene;
    /**
     * Creates a new instance of the component for the given scene
     * @param scene Defines the scene to register the component in
     */
    constructor(scene: Scene);
    /**
     * Registers the component in a given scene
     */
    register(): void;
    private _beforeRenderTargetDraw;
    private _afterRenderTargetDraw;
    private _beforeRenderTargetClearStage;
    private _beforeCameraDraw;
    private _afterCameraDraw;
    private _beforeClearStage;
    private _beforeRenderingMeshStage;
    private _afterRenderingMeshStage;
    /**
     * Rebuilds the elements related to this component in case of
     * context lost for instance.
     */
    rebuild(): void;
    /**
     * Disposes the component and the associated resources
     */
    dispose(): void;
}

}
declare module "babylonjs/Rendering/renderingGroup" {
import { SmartArray, SmartArrayNoDuplicate } from "babylonjs/Misc/smartArray";
import { SubMesh } from "babylonjs/Meshes/subMesh";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { Nullable } from "babylonjs/types";
import { IParticleSystem } from "babylonjs/Particles/IParticleSystem";
import { IEdgesRenderer } from "babylonjs/Rendering/edgesRenderer";
import { ISpriteManager } from "babylonjs/Sprites/spriteManager";
import { Material } from "babylonjs/Materials/material";
import { Scene } from "babylonjs/scene";
/**
 * This represents the object necessary to create a rendering group.
 * This is exclusively used and created by the rendering manager.
 * To modify the behavior, you use the available helpers in your scene or meshes.
 * @internal
 */
export class RenderingGroup {
    index: number;
    private static _ZeroVector;
    private _scene;
    private _opaqueSubMeshes;
    private _transparentSubMeshes;
    private _alphaTestSubMeshes;
    private _depthOnlySubMeshes;
    private _particleSystems;
    private _spriteManagers;
    private _opaqueSortCompareFn;
    private _alphaTestSortCompareFn;
    private _transparentSortCompareFn;
    private _renderOpaque;
    private _renderAlphaTest;
    private _renderTransparent;
    /** @internal */
    _empty: boolean;
    /** @internal */
    _edgesRenderers: SmartArrayNoDuplicate<IEdgesRenderer>;
    onBeforeTransparentRendering: () => void;
    /**
     * Set the opaque sort comparison function.
     * If null the sub meshes will be render in the order they were created
     */
    set opaqueSortCompareFn(value: Nullable<(a: SubMesh, b: SubMesh) => number>);
    /**
     * Set the alpha test sort comparison function.
     * If null the sub meshes will be render in the order they were created
     */
    set alphaTestSortCompareFn(value: Nullable<(a: SubMesh, b: SubMesh) => number>);
    /**
     * Set the transparent sort comparison function.
     * If null the sub meshes will be render in the order they were created
     */
    set transparentSortCompareFn(value: Nullable<(a: SubMesh, b: SubMesh) => number>);
    /**
     * Creates a new rendering group.
     * @param index The rendering group index
     * @param scene
     * @param opaqueSortCompareFn The opaque sort comparison function. If null no order is applied
     * @param alphaTestSortCompareFn The alpha test sort comparison function. If null no order is applied
     * @param transparentSortCompareFn The transparent sort comparison function. If null back to front + alpha index sort is applied
     */
    constructor(index: number, scene: Scene, opaqueSortCompareFn?: Nullable<(a: SubMesh, b: SubMesh) => number>, alphaTestSortCompareFn?: Nullable<(a: SubMesh, b: SubMesh) => number>, transparentSortCompareFn?: Nullable<(a: SubMesh, b: SubMesh) => number>);
    /**
     * Render all the sub meshes contained in the group.
     * @param customRenderFunction Used to override the default render behaviour of the group.
     * @param renderSprites
     * @param renderParticles
     * @param activeMeshes
     * @returns true if rendered some submeshes.
     */
    render(customRenderFunction: Nullable<(opaqueSubMeshes: SmartArray<SubMesh>, transparentSubMeshes: SmartArray<SubMesh>, alphaTestSubMeshes: SmartArray<SubMesh>, depthOnlySubMeshes: SmartArray<SubMesh>) => void>, renderSprites: boolean, renderParticles: boolean, activeMeshes: Nullable<AbstractMesh[]>): void;
    /**
     * Renders the opaque submeshes in the order from the opaqueSortCompareFn.
     * @param subMeshes The submeshes to render
     */
    private _renderOpaqueSorted;
    /**
     * Renders the opaque submeshes in the order from the alphatestSortCompareFn.
     * @param subMeshes The submeshes to render
     */
    private _renderAlphaTestSorted;
    /**
     * Renders the opaque submeshes in the order from the transparentSortCompareFn.
     * @param subMeshes The submeshes to render
     */
    private _renderTransparentSorted;
    /**
     * Renders the submeshes in a specified order.
     * @param subMeshes The submeshes to sort before render
     * @param sortCompareFn The comparison function use to sort
     * @param camera The camera position use to preprocess the submeshes to help sorting
     * @param transparent Specifies to activate blending if true
     */
    private static _RenderSorted;
    /**
     * Build in function which can be applied to ensure meshes of a special queue (opaque, alpha test, transparent)
     * are rendered back to front if in the same alpha index.
     *
     * @param a The first submesh
     * @param b The second submesh
     * @returns The result of the comparison
     */
    static defaultTransparentSortCompare(a: SubMesh, b: SubMesh): number;
    /**
     * Build in function which can be applied to ensure meshes of a special queue (opaque, alpha test, transparent)
     * are rendered back to front.
     *
     * @param a The first submesh
     * @param b The second submesh
     * @returns The result of the comparison
     */
    static backToFrontSortCompare(a: SubMesh, b: SubMesh): number;
    /**
     * Build in function which can be applied to ensure meshes of a special queue (opaque, alpha test, transparent)
     * are rendered front to back (prevent overdraw).
     *
     * @param a The first submesh
     * @param b The second submesh
     * @returns The result of the comparison
     */
    static frontToBackSortCompare(a: SubMesh, b: SubMesh): number;
    /**
     * Build in function which can be applied to ensure meshes of a special queue (opaque, alpha test, transparent)
     * are grouped by material then geometry.
     *
     * @param a The first submesh
     * @param b The second submesh
     * @returns The result of the comparison
     */
    static PainterSortCompare(a: SubMesh, b: SubMesh): number;
    /**
     * Resets the different lists of submeshes to prepare a new frame.
     */
    prepare(): void;
    /**
     * Resets the different lists of sprites to prepare a new frame.
     */
    prepareSprites(): void;
    dispose(): void;
    /**
     * Inserts the submesh in its correct queue depending on its material.
     * @param subMesh The submesh to dispatch
     * @param [mesh] Optional reference to the submeshes's mesh. Provide if you have an exiting reference to improve performance.
     * @param [material] Optional reference to the submeshes's material. Provide if you have an exiting reference to improve performance.
     */
    dispatch(subMesh: SubMesh, mesh?: AbstractMesh, material?: Nullable<Material>): void;
    dispatchSprites(spriteManager: ISpriteManager): void;
    dispatchParticles(particleSystem: IParticleSystem): void;
    private _renderParticles;
    private _renderSprites;
}

}
declare module "babylonjs/Rendering/renderingManager" {
import { Nullable } from "babylonjs/types";
import { SmartArray } from "babylonjs/Misc/smartArray";
import { ISpriteManager } from "babylonjs/Sprites/spriteManager";
import { IParticleSystem } from "babylonjs/Particles/IParticleSystem";
import { RenderingGroup } from "babylonjs/Rendering/renderingGroup";
import { Scene } from "babylonjs/scene";
import { Camera } from "babylonjs/Cameras/camera";
import { Material } from "babylonjs/Materials/material";
import { SubMesh } from "babylonjs/Meshes/subMesh";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
/**
 * Interface describing the different options available in the rendering manager
 * regarding Auto Clear between groups.
 */
export interface IRenderingManagerAutoClearSetup {
    /**
     * Defines whether or not autoclear is enable.
     */
    autoClear: boolean;
    /**
     * Defines whether or not to autoclear the depth buffer.
     */
    depth: boolean;
    /**
     * Defines whether or not to autoclear the stencil buffer.
     */
    stencil: boolean;
}
/**
 * This class is used by the onRenderingGroupObservable
 */
export class RenderingGroupInfo {
    /**
     * The Scene that being rendered
     */
    scene: Scene;
    /**
     * The camera currently used for the rendering pass
     */
    camera: Nullable<Camera>;
    /**
     * The ID of the renderingGroup being processed
     */
    renderingGroupId: number;
}
/**
 * This is the manager responsible of all the rendering for meshes sprites and particles.
 * It is enable to manage the different groups as well as the different necessary sort functions.
 * This should not be used directly aside of the few static configurations
 */
export class RenderingManager {
    /**
     * The max id used for rendering groups (not included)
     */
    static MAX_RENDERINGGROUPS: number;
    /**
     * The min id used for rendering groups (included)
     */
    static MIN_RENDERINGGROUPS: number;
    /**
     * Used to globally prevent autoclearing scenes.
     */
    static AUTOCLEAR: boolean;
    /**
     * @internal
     */
    _useSceneAutoClearSetup: boolean;
    private _scene;
    private _renderingGroups;
    private _depthStencilBufferAlreadyCleaned;
    private _autoClearDepthStencil;
    private _customOpaqueSortCompareFn;
    private _customAlphaTestSortCompareFn;
    private _customTransparentSortCompareFn;
    private _renderingGroupInfo;
    private _maintainStateBetweenFrames;
    /**
     * Gets or sets a boolean indicating that the manager will not reset between frames.
     * This means that if a mesh becomes invisible or transparent it will not be visible until this boolean is set to false again.
     * By default, the rendering manager will dispatch all active meshes per frame (moving them to the transparent, opaque or alpha testing lists).
     * By turning this property on, you will accelerate the rendering by keeping all these lists unchanged between frames.
     */
    get maintainStateBetweenFrames(): boolean;
    set maintainStateBetweenFrames(value: boolean);
    /**
     * Instantiates a new rendering group for a particular scene
     * @param scene Defines the scene the groups belongs to
     */
    constructor(scene: Scene);
    /**
     * Gets the rendering group with the specified id.
     */
    getRenderingGroup(id: number): RenderingGroup;
    private _clearDepthStencilBuffer;
    /**
     * Renders the entire managed groups. This is used by the scene or the different render targets.
     * @internal
     */
    render(customRenderFunction: Nullable<(opaqueSubMeshes: SmartArray<SubMesh>, transparentSubMeshes: SmartArray<SubMesh>, alphaTestSubMeshes: SmartArray<SubMesh>, depthOnlySubMeshes: SmartArray<SubMesh>) => void>, activeMeshes: Nullable<AbstractMesh[]>, renderParticles: boolean, renderSprites: boolean): void;
    /**
     * Resets the different information of the group to prepare a new frame
     * @internal
     */
    reset(): void;
    /**
     * Resets the sprites information of the group to prepare a new frame
     * @internal
     */
    resetSprites(): void;
    /**
     * Dispose and release the group and its associated resources.
     * @internal
     */
    dispose(): void;
    /**
     * Clear the info related to rendering groups preventing retention points during dispose.
     */
    freeRenderingGroups(): void;
    private _prepareRenderingGroup;
    /**
     * Add a sprite manager to the rendering manager in order to render it this frame.
     * @param spriteManager Define the sprite manager to render
     */
    dispatchSprites(spriteManager: ISpriteManager): void;
    /**
     * Add a particle system to the rendering manager in order to render it this frame.
     * @param particleSystem Define the particle system to render
     */
    dispatchParticles(particleSystem: IParticleSystem): void;
    /**
     * Add a submesh to the manager in order to render it this frame
     * @param subMesh The submesh to dispatch
     * @param mesh Optional reference to the submeshes's mesh. Provide if you have an exiting reference to improve performance.
     * @param material Optional reference to the submeshes's material. Provide if you have an exiting reference to improve performance.
     */
    dispatch(subMesh: SubMesh, mesh?: AbstractMesh, material?: Nullable<Material>): void;
    /**
     * Overrides the default sort function applied in the rendering group to prepare the meshes.
     * This allowed control for front to back rendering or reversely depending of the special needs.
     *
     * @param renderingGroupId The rendering group id corresponding to its index
     * @param opaqueSortCompareFn The opaque queue comparison function use to sort.
     * @param alphaTestSortCompareFn The alpha test queue comparison function use to sort.
     * @param transparentSortCompareFn The transparent queue comparison function use to sort.
     */
    setRenderingOrder(renderingGroupId: number, opaqueSortCompareFn?: Nullable<(a: SubMesh, b: SubMesh) => number>, alphaTestSortCompareFn?: Nullable<(a: SubMesh, b: SubMesh) => number>, transparentSortCompareFn?: Nullable<(a: SubMesh, b: SubMesh) => number>): void;
    /**
     * Specifies whether or not the stencil and depth buffer are cleared between two rendering groups.
     *
     * @param renderingGroupId The rendering group id corresponding to its index
     * @param autoClearDepthStencil Automatically clears depth and stencil between groups if true.
     * @param depth Automatically clears depth between groups if true and autoClear is true.
     * @param stencil Automatically clears stencil between groups if true and autoClear is true.
     */
    setRenderingAutoClearDepthStencil(renderingGroupId: number, autoClearDepthStencil: boolean, depth?: boolean, stencil?: boolean): void;
    /**
     * Gets the current auto clear configuration for one rendering group of the rendering
     * manager.
     * @param index the rendering group index to get the information for
     * @returns The auto clear setup for the requested rendering group
     */
    getAutoClearDepthStencilSetup(index: number): IRenderingManagerAutoClearSetup;
}
export {};

}
declare module "babylonjs/Rendering/screenSpaceReflectionsConfiguration" {
import { PrePassEffectConfiguration } from "babylonjs/Rendering/prePassEffectConfiguration";
/**
 * Contains all parameters needed for the prepass to perform
 * screen space reflections
 */
export class ScreenSpaceReflectionsConfiguration implements PrePassEffectConfiguration {
    /**
     * Is ssr enabled
     */
    enabled: boolean;
    /**
     * Name of the configuration
     */
    name: string;
    /**
     * Textures that should be present in the MRT for this effect to work
     */
    readonly texturesRequired: number[];
}

}
declare module "babylonjs/Rendering/ssao2Configuration" {
import { PrePassEffectConfiguration } from "babylonjs/Rendering/prePassEffectConfiguration";
/**
 * Contains all parameters needed for the prepass to perform
 * screen space subsurface scattering
 */
export class SSAO2Configuration implements PrePassEffectConfiguration {
    /**
     * Is subsurface enabled
     */
    enabled: boolean;
    /**
     * Name of the configuration
     */
    name: string;
    /**
     * Textures that should be present in the MRT for this effect to work
     */
    readonly texturesRequired: number[];
}

}
declare module "babylonjs/Rendering/subSurfaceConfiguration" {
import { Scene } from "babylonjs/scene";
import { Color3 } from "babylonjs/Maths/math.color";
import { SubSurfaceScatteringPostProcess } from "babylonjs/PostProcesses/subSurfaceScatteringPostProcess";
import { PrePassEffectConfiguration } from "babylonjs/Rendering/prePassEffectConfiguration";
/**
 * Contains all parameters needed for the prepass to perform
 * screen space subsurface scattering
 */
export class SubSurfaceConfiguration implements PrePassEffectConfiguration {
    /**
     * @internal
     */
    static _SceneComponentInitialization: (scene: Scene) => void;
    private _ssDiffusionS;
    private _ssFilterRadii;
    private _ssDiffusionD;
    /**
     * Post process to attach for screen space subsurface scattering
     */
    postProcess: SubSurfaceScatteringPostProcess;
    /**
     * Diffusion profile color for subsurface scattering
     */
    get ssDiffusionS(): number[];
    /**
     * Diffusion profile max color channel value for subsurface scattering
     */
    get ssDiffusionD(): number[];
    /**
     * Diffusion profile filter radius for subsurface scattering
     */
    get ssFilterRadii(): number[];
    /**
     * Is subsurface enabled
     */
    enabled: boolean;
    /**
     * Does the output of this prepass need to go through imageprocessing
     */
    needsImageProcessing: boolean;
    /**
     * Name of the configuration
     */
    name: string;
    /**
     * Diffusion profile colors for subsurface scattering
     * You can add one diffusion color using `addDiffusionProfile` on `scene.prePassRenderer`
     * See ...
     * Note that you can only store up to 5 of them
     */
    ssDiffusionProfileColors: Color3[];
    /**
     * Defines the ratio real world => scene units.
     * Used for subsurface scattering
     */
    metersPerUnit: number;
    /**
     * Textures that should be present in the MRT for this effect to work
     */
    readonly texturesRequired: number[];
    private _scene;
    /**
     * Builds a subsurface configuration object
     * @param scene The scene
     */
    constructor(scene: Scene);
    /**
     * Adds a new diffusion profile.
     * Useful for more realistic subsurface scattering on diverse materials.
     * @param color The color of the diffusion profile. Should be the average color of the material.
     * @returns The index of the diffusion profile for the material subsurface configuration
     */
    addDiffusionProfile(color: Color3): number;
    /**
     * Creates the sss post process
     * @returns The created post process
     */
    createPostProcess(): SubSurfaceScatteringPostProcess;
    /**
     * Deletes all diffusion profiles.
     * Note that in order to render subsurface scattering, you should have at least 1 diffusion profile.
     */
    clearAllDiffusionProfiles(): void;
    /**
     * Disposes this object
     */
    dispose(): void;
    /**
     * @internal
     * https://zero-radiance.github.io/post/sampling-diffusion/
     *
     * Importance sample the normalized diffuse reflectance profile for the computed value of 's'.
     * ------------------------------------------------------------------------------------
     * R[r, phi, s]   = s * (Exp[-r * s] + Exp[-r * s / 3]) / (8 * Pi * r)
     * PDF[r, phi, s] = r * R[r, phi, s]
     * CDF[r, s]      = 1 - 1/4 * Exp[-r * s] - 3/4 * Exp[-r * s / 3]
     * ------------------------------------------------------------------------------------
     * We importance sample the color channel with the widest scattering distance.
     */
    getDiffusionProfileParameters(color: Color3): number;
    /**
     * Performs sampling of a Normalized Burley diffusion profile in polar coordinates.
     * 'u' is the random number (the value of the CDF): [0, 1).
     * rcp(s) = 1 / ShapeParam = ScatteringDistance.
     * Returns the sampled radial distance, s.t. (u = 0 -> r = 0) and (u = 1 -> r = Inf).
     * @param u
     * @param rcpS
     */
    private _sampleBurleyDiffusionProfile;
}

}
declare module "babylonjs/Rendering/subSurfaceSceneComponent" {
import { Nullable } from "babylonjs/types";
import { Scene } from "babylonjs/scene";
import { ISceneSerializableComponent } from "babylonjs/sceneComponent";
import { SubSurfaceConfiguration } from "babylonjs/Rendering/subSurfaceConfiguration";
module "babylonjs/abstractScene" {
    interface AbstractScene {
        /** @internal (Backing field) */
        _subSurfaceConfiguration: Nullable<SubSurfaceConfiguration>;
        /**
         * Gets or Sets the current prepass renderer associated to the scene.
         */
        subSurfaceConfiguration: Nullable<SubSurfaceConfiguration>;
        /**
         * Enables the subsurface effect for prepass
         * @returns the SubSurfaceConfiguration
         */
        enableSubSurfaceForPrePass(): Nullable<SubSurfaceConfiguration>;
        /**
         * Disables the subsurface effect for prepass
         */
        disableSubSurfaceForPrePass(): void;
    }
}
/**
 * Defines the Geometry Buffer scene component responsible to manage a G-Buffer useful
 * in several rendering techniques.
 */
export class SubSurfaceSceneComponent implements ISceneSerializableComponent {
    /**
     * The component name helpful to identify the component in the list of scene components.
     */
    readonly name: string;
    /**
     * The scene the component belongs to.
     */
    scene: Scene;
    /**
     * Creates a new instance of the component for the given scene
     * @param scene Defines the scene to register the component in
     */
    constructor(scene: Scene);
    /**
     * Registers the component in a given scene
     */
    register(): void;
    /**
     * Serializes the component data to the specified json object
     * @param serializationObject The object to serialize to
     */
    serialize(serializationObject: any): void;
    /**
     * Adds all the elements from the container to the scene
     */
    addFromContainer(): void;
    /**
     * Removes all the elements in the container from the scene
     */
    removeFromContainer(): void;
    /**
     * Rebuilds the elements related to this component in case of
     * context lost for instance.
     */
    rebuild(): void;
    /**
     * Disposes the component and the associated resources
     */
    dispose(): void;
}

}
declare module "babylonjs/Rendering/utilityLayerRenderer" {
import { IDisposable } from "babylonjs/scene";
import { Scene } from "babylonjs/scene";
import { Nullable } from "babylonjs/types";
import { Observable } from "babylonjs/Misc/observable";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { HemisphericLight } from "babylonjs/Lights/hemisphericLight";
import { Camera } from "babylonjs/Cameras/camera";
/**
 * Renders a layer on top of an existing scene
 */
export class UtilityLayerRenderer implements IDisposable {
    /** the original scene that will be rendered on top of */
    originalScene: Scene;
    private _pointerCaptures;
    private _lastPointerEvents;
    /** @internal */
    static _DefaultUtilityLayer: Nullable<UtilityLayerRenderer>;
    /** @internal */
    static _DefaultKeepDepthUtilityLayer: Nullable<UtilityLayerRenderer>;
    private _sharedGizmoLight;
    private _renderCamera;
    /**
     * Gets the camera that is used to render the utility layer (when not set, this will be the last active camera)
     * @param getRigParentIfPossible if the current active camera is a rig camera, should its parent camera be returned
     * @returns the camera that is used when rendering the utility layer
     */
    getRenderCamera(getRigParentIfPossible?: boolean): Camera;
    /**
     * Sets the camera that should be used when rendering the utility layer (If set to null the last active camera will be used)
     * @param cam the camera that should be used when rendering the utility layer
     */
    setRenderCamera(cam: Nullable<Camera>): void;
    /**
     * @internal
     * Light which used by gizmos to get light shading
     */
    _getSharedGizmoLight(): HemisphericLight;
    /**
     * If the picking should be done on the utility layer prior to the actual scene (Default: true)
     */
    pickUtilitySceneFirst: boolean;
    /**
     * A shared utility layer that can be used to overlay objects into a scene (Depth map of the previous scene is cleared before drawing on top of it)
     */
    static get DefaultUtilityLayer(): UtilityLayerRenderer;
    /**
     * Creates an utility layer, and set it as a default utility layer
     * @param scene associated scene
     * @internal
     */
    static _CreateDefaultUtilityLayerFromScene(scene: Scene): UtilityLayerRenderer;
    /**
     * A shared utility layer that can be used to embed objects into a scene (Depth map of the previous scene is not cleared before drawing on top of it)
     */
    static get DefaultKeepDepthUtilityLayer(): UtilityLayerRenderer;
    /**
     * The scene that is rendered on top of the original scene
     */
    utilityLayerScene: Scene;
    /**
     *  If the utility layer should automatically be rendered on top of existing scene
     */
    shouldRender: boolean;
    /**
     * If set to true, only pointer down onPointerObservable events will be blocked when picking is occluded by original scene
     */
    onlyCheckPointerDownEvents: boolean;
    /**
     * If set to false, only pointerUp, pointerDown and pointerMove will be sent to the utilityLayerScene (false by default)
     */
    processAllEvents: boolean;
    /**
     * Set to false to disable picking
     */
    pickingEnabled: boolean;
    /**
     * Observable raised when the pointer moves from the utility layer scene to the main scene
     */
    onPointerOutObservable: Observable<number>;
    /** Gets or sets a predicate that will be used to indicate utility meshes present in the main scene */
    mainSceneTrackerPredicate: (mesh: Nullable<AbstractMesh>) => boolean;
    private _afterRenderObserver;
    private _sceneDisposeObserver;
    private _originalPointerObserver;
    /**
     * Instantiates a UtilityLayerRenderer
     * @param originalScene the original scene that will be rendered on top of
     * @param handleEvents boolean indicating if the utility layer should handle events
     */
    constructor(
    /** the original scene that will be rendered on top of */
    originalScene: Scene, handleEvents?: boolean);
    private _notifyObservers;
    /**
     * Renders the utility layers scene on top of the original scene
     */
    render(): void;
    /**
     * Disposes of the renderer
     */
    dispose(): void;
    private _updateCamera;
}

}
declare module "babylonjs/scene" {
import { Nullable } from "babylonjs/types";
import { IAnimatable } from "babylonjs/Animations/animatable.interface";
import { Observable } from "babylonjs/Misc/observable";
import { ISmartArrayLike } from "babylonjs/Misc/smartArray";
import { SmartArrayNoDuplicate, SmartArray } from "babylonjs/Misc/smartArray";
import { Vector2, Vector4 } from "babylonjs/Maths/math.vector";
import { Vector3, Matrix } from "babylonjs/Maths/math.vector";
import { IParticleSystem } from "babylonjs/Particles/IParticleSystem";
import { AbstractScene } from "babylonjs/abstractScene";
import { ImageProcessingConfiguration } from "babylonjs/Materials/imageProcessingConfiguration";
import { UniformBuffer } from "babylonjs/Materials/uniformBuffer";
import { PickingInfo } from "babylonjs/Collisions/pickingInfo";
import { ICollisionCoordinator } from "babylonjs/Collisions/collisionCoordinator";
import { PointerEventTypes, PointerInfoPre, PointerInfo } from "babylonjs/Events/pointerEvents";
import { KeyboardInfoPre, KeyboardInfo } from "babylonjs/Events/keyboardEvents";
import { PostProcessManager } from "babylonjs/PostProcesses/postProcessManager";
import { IOfflineProvider } from "babylonjs/Offline/IOfflineProvider";
import { RenderingGroupInfo, IRenderingManagerAutoClearSetup } from "babylonjs/Rendering/renderingManager";
import { RenderingManager } from "babylonjs/Rendering/renderingManager";
import { ISceneComponent, ISceneSerializableComponent, SimpleStageAction, RenderTargetsStageAction, RenderTargetStageAction, MeshStageAction, EvaluateSubMeshStageAction, PreActiveMeshStageAction, CameraStageAction, RenderingGroupStageAction, RenderingMeshStageAction, PointerMoveStageAction, PointerUpDownStageAction, CameraStageFrameBufferAction } from "babylonjs/sceneComponent";
import { Stage } from "babylonjs/sceneComponent";
import { Engine } from "babylonjs/Engines/engine";
import { AbstractActionManager } from "babylonjs/Actions/abstractActionManager";
import { WebRequest } from "babylonjs/Misc/webRequest";
import { InputManager } from "babylonjs/Inputs/scene.inputManager";
import { PerfCounter } from "babylonjs/Misc/perfCounter";
import { IFileRequest } from "babylonjs/Misc/fileRequest";
import { Color4, Color3 } from "babylonjs/Maths/math.color";
import { Plane } from "babylonjs/Maths/math.plane";
import { LoadFileError, RequestFileError, ReadFileError } from "babylonjs/Misc/fileTools";
import { IClipPlanesHolder } from "babylonjs/Misc/interfaces/iClipPlanesHolder";
import { IPointerEvent } from "babylonjs/Events/deviceInputEvents";
import { Ray } from "babylonjs/Culling/ray";
import { TrianglePickingPredicate } from "babylonjs/Culling/ray";
import { Animation } from "babylonjs/Animations/animation";
import { Animatable } from "babylonjs/Animations/animatable";
import { AnimationGroup } from "babylonjs/Animations/animationGroup";
import { AnimationPropertiesOverride } from "babylonjs/Animations/animationPropertiesOverride";
import { Collider } from "babylonjs/Collisions/collider";
import { PostProcess } from "babylonjs/PostProcesses/postProcess";
import { Material } from "babylonjs/Materials/material";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { Light } from "babylonjs/Lights/light";
import { Camera } from "babylonjs/Cameras/camera";
import { MultiMaterial } from "babylonjs/Materials/multiMaterial";
import { BaseTexture } from "babylonjs/Materials/Textures/baseTexture";
import { TransformNode } from "babylonjs/Meshes/transformNode";
import { Skeleton } from "babylonjs/Bones/skeleton";
import { Bone } from "babylonjs/Bones/bone";
import { SubMesh } from "babylonjs/Meshes/subMesh";
import { Mesh } from "babylonjs/Meshes/mesh";
import { Node } from "babylonjs/node";
import { Geometry } from "babylonjs/Meshes/geometry";
import { MorphTargetManager } from "babylonjs/Morph/morphTargetManager";
import { Effect } from "babylonjs/Materials/effect";
import { MorphTarget } from "babylonjs/Morph/morphTarget";
import { PerformanceViewerCollector } from "babylonjs/Misc/PerformanceViewer/performanceViewerCollector";
/**
 * Define an interface for all classes that will hold resources
 */
export interface IDisposable {
    /**
     * Releases all held resources
     */
    dispose(): void;
}
/** Interface defining initialization parameters for Scene class */
export interface SceneOptions {
    /**
     * Defines that scene should keep up-to-date a map of geometry to enable fast look-up by uniqueId
     * It will improve performance when the number of geometries becomes important.
     */
    useGeometryUniqueIdsMap?: boolean;
    /**
     * Defines that each material of the scene should keep up-to-date a map of referencing meshes for fast disposing
     * It will improve performance when the number of mesh becomes important, but might consume a bit more memory
     */
    useMaterialMeshMap?: boolean;
    /**
     * Defines that each mesh of the scene should keep up-to-date a map of referencing cloned meshes for fast disposing
     * It will improve performance when the number of mesh becomes important, but might consume a bit more memory
     */
    useClonedMeshMap?: boolean;
    /** Defines if the creation of the scene should impact the engine (Eg. UtilityLayer's scene) */
    virtual?: boolean;
}
/**
 * Define how the scene should favor performance over ease of use
 */
export enum ScenePerformancePriority {
    /** Default mode. No change. Performance will be treated as less important than backward compatibility */
    BackwardCompatible = 0,
    /** Some performance options will be turned on trying to strike a balance between perf and ease of use */
    Intermediate = 1,
    /** Performance will be top priority */
    Aggressive = 2
}
/**
 * Represents a scene to be rendered by the engine.
 * @see https://doc.babylonjs.com/features/featuresDeepDive/scene
 */
export class Scene extends AbstractScene implements IAnimatable, IClipPlanesHolder {
    /** The fog is deactivated */
    static readonly FOGMODE_NONE: number;
    /** The fog density is following an exponential function */
    static readonly FOGMODE_EXP: number;
    /** The fog density is following an exponential function faster than FOGMODE_EXP */
    static readonly FOGMODE_EXP2: number;
    /** The fog density is following a linear function. */
    static readonly FOGMODE_LINEAR: number;
    /**
     * Gets or sets the minimum deltatime when deterministic lock step is enabled
     * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/advanced_animations#deterministic-lockstep
     */
    static MinDeltaTime: number;
    /**
     * Gets or sets the maximum deltatime when deterministic lock step is enabled
     * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/advanced_animations#deterministic-lockstep
     */
    static MaxDeltaTime: number;
    /**
     * Factory used to create the default material.
     * @param scene The scene to create the material for
     * @returns The default material
     */
    static DefaultMaterialFactory(scene: Scene): Material;
    /**
     * Factory used to create the a collision coordinator.
     * @returns The collision coordinator
     */
    static CollisionCoordinatorFactory(): ICollisionCoordinator;
    /** @internal */
    _inputManager: InputManager;
    /** Define this parameter if you are using multiple cameras and you want to specify which one should be used for pointer position */
    cameraToUseForPointers: Nullable<Camera>;
    /** @internal */
    readonly _isScene: boolean;
    /** @internal */
    _blockEntityCollection: boolean;
    /**
     * Gets or sets a boolean that indicates if the scene must clear the render buffer before rendering a frame
     */
    autoClear: boolean;
    /**
     * Gets or sets a boolean that indicates if the scene must clear the depth and stencil buffers before rendering a frame
     */
    autoClearDepthAndStencil: boolean;
    /**
     * Defines the color used to clear the render buffer (Default is (0.2, 0.2, 0.3, 1.0))
     */
    clearColor: Color4;
    /**
     * Defines the color used to simulate the ambient color (Default is (0, 0, 0))
     */
    ambientColor: Color3;
    /**
     * This is use to store the default BRDF lookup for PBR materials in your scene.
     * It should only be one of the following (if not the default embedded one):
     * * For uncorrelated BRDF (pbr.brdf.useEnergyConservation = false and pbr.brdf.useSmithVisibilityHeightCorrelated = false) : https://assets.babylonjs.com/environments/uncorrelatedBRDF.dds
     * * For correlated BRDF (pbr.brdf.useEnergyConservation = false and pbr.brdf.useSmithVisibilityHeightCorrelated = true) : https://assets.babylonjs.com/environments/correlatedBRDF.dds
     * * For correlated multi scattering BRDF (pbr.brdf.useEnergyConservation = true and pbr.brdf.useSmithVisibilityHeightCorrelated = true) : https://assets.babylonjs.com/environments/correlatedMSBRDF.dds
     * The material properties need to be setup according to the type of texture in use.
     */
    environmentBRDFTexture: BaseTexture;
    /**
     * Texture used in all pbr material as the reflection texture.
     * As in the majority of the scene they are the same (exception for multi room and so on),
     * this is easier to reference from here than from all the materials.
     */
    get environmentTexture(): Nullable<BaseTexture>;
    /**
     * Texture used in all pbr material as the reflection texture.
     * As in the majority of the scene they are the same (exception for multi room and so on),
     * this is easier to set here than in all the materials.
     */
    set environmentTexture(value: Nullable<BaseTexture>);
    /**
     * Intensity of the environment in all pbr material.
     * This dims or reinforces the IBL lighting overall (reflection and diffuse).
     * As in the majority of the scene they are the same (exception for multi room and so on),
     * this is easier to reference from here than from all the materials.
     */
    environmentIntensity: number;
    /** @internal */
    protected _imageProcessingConfiguration: ImageProcessingConfiguration;
    /**
     * Default image processing configuration used either in the rendering
     * Forward main pass or through the imageProcessingPostProcess if present.
     * As in the majority of the scene they are the same (exception for multi camera),
     * this is easier to reference from here than from all the materials and post process.
     *
     * No setter as we it is a shared configuration, you can set the values instead.
     */
    get imageProcessingConfiguration(): ImageProcessingConfiguration;
    private _performancePriority;
    /**
     * Gets or sets a value indicating how to treat performance relatively to ease of use and backward compatibility
     */
    get performancePriority(): ScenePerformancePriority;
    set performancePriority(value: ScenePerformancePriority);
    private _forceWireframe;
    /**
     * Gets or sets a boolean indicating if all rendering must be done in wireframe
     */
    set forceWireframe(value: boolean);
    get forceWireframe(): boolean;
    private _skipFrustumClipping;
    /**
     * Gets or sets a boolean indicating if we should skip the frustum clipping part of the active meshes selection
     */
    set skipFrustumClipping(value: boolean);
    get skipFrustumClipping(): boolean;
    private _forcePointsCloud;
    /**
     * Gets or sets a boolean indicating if all rendering must be done in point cloud
     */
    set forcePointsCloud(value: boolean);
    get forcePointsCloud(): boolean;
    /**
     * Gets or sets the active clipplane 1
     */
    clipPlane: Nullable<Plane>;
    /**
     * Gets or sets the active clipplane 2
     */
    clipPlane2: Nullable<Plane>;
    /**
     * Gets or sets the active clipplane 3
     */
    clipPlane3: Nullable<Plane>;
    /**
     * Gets or sets the active clipplane 4
     */
    clipPlane4: Nullable<Plane>;
    /**
     * Gets or sets the active clipplane 5
     */
    clipPlane5: Nullable<Plane>;
    /**
     * Gets or sets the active clipplane 6
     */
    clipPlane6: Nullable<Plane>;
    /**
     * Gets or sets a boolean indicating if animations are enabled
     */
    animationsEnabled: boolean;
    private _animationPropertiesOverride;
    /**
     * Gets or sets the animation properties override
     */
    get animationPropertiesOverride(): Nullable<AnimationPropertiesOverride>;
    set animationPropertiesOverride(value: Nullable<AnimationPropertiesOverride>);
    /**
     * Gets or sets a boolean indicating if a constant deltatime has to be used
     * This is mostly useful for testing purposes when you do not want the animations to scale with the framerate
     */
    useConstantAnimationDeltaTime: boolean;
    /**
     * Gets or sets a boolean indicating if the scene must keep the meshUnderPointer property updated
     * Please note that it requires to run a ray cast through the scene on every frame
     */
    constantlyUpdateMeshUnderPointer: boolean;
    /**
     * Defines the HTML cursor to use when hovering over interactive elements
     */
    hoverCursor: string;
    /**
     * Defines the HTML default cursor to use (empty by default)
     */
    defaultCursor: string;
    /**
     * Defines whether cursors are handled by the scene.
     */
    doNotHandleCursors: boolean;
    /**
     * This is used to call preventDefault() on pointer down
     * in order to block unwanted artifacts like system double clicks
     */
    preventDefaultOnPointerDown: boolean;
    /**
     * This is used to call preventDefault() on pointer up
     * in order to block unwanted artifacts like system double clicks
     */
    preventDefaultOnPointerUp: boolean;
    /**
     * Gets or sets user defined metadata
     */
    metadata: any;
    /**
     * For internal use only. Please do not use.
     */
    reservedDataStore: any;
    /**
     * Gets the name of the plugin used to load this scene (null by default)
     */
    loadingPluginName: string;
    /**
     * Use this array to add regular expressions used to disable offline support for specific urls
     */
    disableOfflineSupportExceptionRules: RegExp[];
    /**
     * An event triggered when the scene is disposed.
     */
    onDisposeObservable: Observable<Scene>;
    private _onDisposeObserver;
    /** Sets a function to be executed when this scene is disposed. */
    set onDispose(callback: () => void);
    /**
     * An event triggered before rendering the scene (right after animations and physics)
     */
    onBeforeRenderObservable: Observable<Scene>;
    private _onBeforeRenderObserver;
    /** Sets a function to be executed before rendering this scene */
    set beforeRender(callback: Nullable<() => void>);
    /**
     * An event triggered after rendering the scene
     */
    onAfterRenderObservable: Observable<Scene>;
    /**
     * An event triggered after rendering the scene for an active camera (When scene.render is called this will be called after each camera)
     * This is triggered for each "sub" camera in a Camera Rig unlike onAfterCameraRenderObservable
     */
    onAfterRenderCameraObservable: Observable<import("babylonjs/Cameras/camera").Camera>;
    private _onAfterRenderObserver;
    /** Sets a function to be executed after rendering this scene */
    set afterRender(callback: Nullable<() => void>);
    /**
     * An event triggered before animating the scene
     */
    onBeforeAnimationsObservable: Observable<Scene>;
    /**
     * An event triggered after animations processing
     */
    onAfterAnimationsObservable: Observable<Scene>;
    /**
     * An event triggered before draw calls are ready to be sent
     */
    onBeforeDrawPhaseObservable: Observable<Scene>;
    /**
     * An event triggered after draw calls have been sent
     */
    onAfterDrawPhaseObservable: Observable<Scene>;
    /**
     * An event triggered when the scene is ready
     */
    onReadyObservable: Observable<Scene>;
    /**
     * An event triggered before rendering a camera
     */
    onBeforeCameraRenderObservable: Observable<import("babylonjs/Cameras/camera").Camera>;
    private _onBeforeCameraRenderObserver;
    /** Sets a function to be executed before rendering a camera*/
    set beforeCameraRender(callback: () => void);
    /**
     * An event triggered after rendering a camera
     * This is triggered for the full rig Camera only unlike onAfterRenderCameraObservable
     */
    onAfterCameraRenderObservable: Observable<import("babylonjs/Cameras/camera").Camera>;
    private _onAfterCameraRenderObserver;
    /** Sets a function to be executed after rendering a camera*/
    set afterCameraRender(callback: () => void);
    /**
     * An event triggered when active meshes evaluation is about to start
     */
    onBeforeActiveMeshesEvaluationObservable: Observable<Scene>;
    /**
     * An event triggered when active meshes evaluation is done
     */
    onAfterActiveMeshesEvaluationObservable: Observable<Scene>;
    /**
     * An event triggered when particles rendering is about to start
     * Note: This event can be trigger more than once per frame (because particles can be rendered by render target textures as well)
     */
    onBeforeParticlesRenderingObservable: Observable<Scene>;
    /**
     * An event triggered when particles rendering is done
     * Note: This event can be trigger more than once per frame (because particles can be rendered by render target textures as well)
     */
    onAfterParticlesRenderingObservable: Observable<Scene>;
    /**
     * An event triggered when SceneLoader.Append or SceneLoader.Load or SceneLoader.ImportMesh were successfully executed
     */
    onDataLoadedObservable: Observable<Scene>;
    /**
     * An event triggered when a camera is created
     */
    onNewCameraAddedObservable: Observable<import("babylonjs/Cameras/camera").Camera>;
    /**
     * An event triggered when a camera is removed
     */
    onCameraRemovedObservable: Observable<import("babylonjs/Cameras/camera").Camera>;
    /**
     * An event triggered when a light is created
     */
    onNewLightAddedObservable: Observable<import("babylonjs/Lights/light").Light>;
    /**
     * An event triggered when a light is removed
     */
    onLightRemovedObservable: Observable<import("babylonjs/Lights/light").Light>;
    /**
     * An event triggered when a geometry is created
     */
    onNewGeometryAddedObservable: Observable<import("babylonjs/Meshes/geometry").Geometry>;
    /**
     * An event triggered when a geometry is removed
     */
    onGeometryRemovedObservable: Observable<import("babylonjs/Meshes/geometry").Geometry>;
    /**
     * An event triggered when a transform node is created
     */
    onNewTransformNodeAddedObservable: Observable<import("babylonjs/Meshes/transformNode").TransformNode>;
    /**
     * An event triggered when a transform node is removed
     */
    onTransformNodeRemovedObservable: Observable<import("babylonjs/Meshes/transformNode").TransformNode>;
    /**
     * An event triggered when a mesh is created
     */
    onNewMeshAddedObservable: Observable<import("babylonjs/Meshes/abstractMesh").AbstractMesh>;
    /**
     * An event triggered when a mesh is removed
     */
    onMeshRemovedObservable: Observable<import("babylonjs/Meshes/abstractMesh").AbstractMesh>;
    /**
     * An event triggered when a skeleton is created
     */
    onNewSkeletonAddedObservable: Observable<import("babylonjs/Bones/skeleton").Skeleton>;
    /**
     * An event triggered when a skeleton is removed
     */
    onSkeletonRemovedObservable: Observable<import("babylonjs/Bones/skeleton").Skeleton>;
    /**
     * An event triggered when a material is created
     */
    onNewMaterialAddedObservable: Observable<import("babylonjs/Materials/material").Material>;
    /**
     * An event triggered when a multi material is created
     */
    onNewMultiMaterialAddedObservable: Observable<import("babylonjs/Materials/multiMaterial").MultiMaterial>;
    /**
     * An event triggered when a material is removed
     */
    onMaterialRemovedObservable: Observable<import("babylonjs/Materials/material").Material>;
    /**
     * An event triggered when a multi material is removed
     */
    onMultiMaterialRemovedObservable: Observable<import("babylonjs/Materials/multiMaterial").MultiMaterial>;
    /**
     * An event triggered when a texture is created
     */
    onNewTextureAddedObservable: Observable<import("babylonjs/Materials/Textures/baseTexture").BaseTexture>;
    /**
     * An event triggered when a texture is removed
     */
    onTextureRemovedObservable: Observable<import("babylonjs/Materials/Textures/baseTexture").BaseTexture>;
    /**
     * An event triggered when render targets are about to be rendered
     * Can happen multiple times per frame.
     */
    onBeforeRenderTargetsRenderObservable: Observable<Scene>;
    /**
     * An event triggered when render targets were rendered.
     * Can happen multiple times per frame.
     */
    onAfterRenderTargetsRenderObservable: Observable<Scene>;
    /**
     * An event triggered before calculating deterministic simulation step
     */
    onBeforeStepObservable: Observable<Scene>;
    /**
     * An event triggered after calculating deterministic simulation step
     */
    onAfterStepObservable: Observable<Scene>;
    /**
     * An event triggered when the activeCamera property is updated
     */
    onActiveCameraChanged: Observable<Scene>;
    /**
     * An event triggered when the activeCameras property is updated
     */
    onActiveCamerasChanged: Observable<Scene>;
    /**
     * This Observable will be triggered before rendering each renderingGroup of each rendered camera.
     * The RenderingGroupInfo class contains all the information about the context in which the observable is called
     * If you wish to register an Observer only for a given set of renderingGroup, use the mask with a combination of the renderingGroup index elevated to the power of two (1 for renderingGroup 0, 2 for renderingrOup1, 4 for 2 and 8 for 3)
     */
    onBeforeRenderingGroupObservable: Observable<RenderingGroupInfo>;
    /**
     * This Observable will be triggered after rendering each renderingGroup of each rendered camera.
     * The RenderingGroupInfo class contains all the information about the context in which the observable is called
     * If you wish to register an Observer only for a given set of renderingGroup, use the mask with a combination of the renderingGroup index elevated to the power of two (1 for renderingGroup 0, 2 for renderingrOup1, 4 for 2 and 8 for 3)
     */
    onAfterRenderingGroupObservable: Observable<RenderingGroupInfo>;
    /**
     * This Observable will when a mesh has been imported into the scene.
     */
    onMeshImportedObservable: Observable<import("babylonjs/Meshes/abstractMesh").AbstractMesh>;
    /**
     * This Observable will when an animation file has been imported into the scene.
     */
    onAnimationFileImportedObservable: Observable<Scene>;
    /**
     * Gets or sets a user defined funtion to select LOD from a mesh and a camera.
     * By default this function is undefined and Babylon.js will select LOD based on distance to camera
     */
    customLODSelector: (mesh: AbstractMesh, camera: Camera) => Nullable<AbstractMesh>;
    /** @internal */
    _registeredForLateAnimationBindings: SmartArrayNoDuplicate<any>;
    /**
     * Gets or sets a predicate used to select candidate meshes for a pointer down event
     */
    pointerDownPredicate: (Mesh: AbstractMesh) => boolean;
    /**
     * Gets or sets a predicate used to select candidate meshes for a pointer up event
     */
    pointerUpPredicate: (Mesh: AbstractMesh) => boolean;
    /**
     * Gets or sets a predicate used to select candidate meshes for a pointer move event
     */
    pointerMovePredicate: (Mesh: AbstractMesh) => boolean;
    /**
     * Gets or sets a boolean indicating if the user want to entirely skip the picking phase when a pointer move event occurs.
     */
    skipPointerMovePicking: boolean;
    /**
     * Gets or sets a boolean indicating if the user want to entirely skip the picking phase when a pointer down event occurs.
     */
    skipPointerDownPicking: boolean;
    /**
     * Gets or sets a boolean indicating if the user want to entirely skip the picking phase when a pointer up event occurs.  Off by default.
     */
    skipPointerUpPicking: boolean;
    /** Callback called when a pointer move is detected */
    onPointerMove?: (evt: IPointerEvent, pickInfo: PickingInfo, type: PointerEventTypes) => void;
    /** Callback called when a pointer down is detected  */
    onPointerDown?: (evt: IPointerEvent, pickInfo: PickingInfo, type: PointerEventTypes) => void;
    /** Callback called when a pointer up is detected  */
    onPointerUp?: (evt: IPointerEvent, pickInfo: Nullable<PickingInfo>, type: PointerEventTypes) => void;
    /** Callback called when a pointer pick is detected */
    onPointerPick?: (evt: IPointerEvent, pickInfo: PickingInfo) => void;
    /**
     * Gets or sets a predicate used to select candidate faces for a pointer move event
     */
    pointerMoveTrianglePredicate: ((p0: Vector3, p1: Vector3, p2: Vector3, ray: Ray) => boolean) | undefined;
    /**
     * This observable event is triggered when any ponter event is triggered. It is registered during Scene.attachControl() and it is called BEFORE the 3D engine process anything (mesh/sprite picking for instance).
     * You have the possibility to skip the process and the call to onPointerObservable by setting PointerInfoPre.skipOnPointerObservable to true
     */
    onPrePointerObservable: Observable<PointerInfoPre>;
    /**
     * Observable event triggered each time an input event is received from the rendering canvas
     */
    onPointerObservable: Observable<PointerInfo>;
    /**
     * Gets the pointer coordinates without any translation (ie. straight out of the pointer event)
     */
    get unTranslatedPointer(): Vector2;
    /**
     * Gets or sets the distance in pixel that you have to move to prevent some events. Default is 10 pixels
     */
    static get DragMovementThreshold(): number;
    static set DragMovementThreshold(value: number);
    /**
     * Time in milliseconds to wait to raise long press events if button is still pressed. Default is 500 ms
     */
    static get LongPressDelay(): number;
    static set LongPressDelay(value: number);
    /**
     * Time in milliseconds to wait to raise long press events if button is still pressed. Default is 300 ms
     */
    static get DoubleClickDelay(): number;
    static set DoubleClickDelay(value: number);
    /** If you need to check double click without raising a single click at first click, enable this flag */
    static get ExclusiveDoubleClickMode(): boolean;
    static set ExclusiveDoubleClickMode(value: boolean);
    /**
     * Bind the current view position to an effect.
     * @param effect The effect to be bound
     * @param variableName name of the shader variable that will hold the eye position
     * @param isVector3 true to indicates that variableName is a Vector3 and not a Vector4
     * @returns the computed eye position
     */
    bindEyePosition(effect: Nullable<Effect>, variableName?: string, isVector3?: boolean): Vector4;
    /**
     * Update the scene ubo before it can be used in rendering processing
     * @returns the scene UniformBuffer
     */
    finalizeSceneUbo(): UniformBuffer;
    /** @internal */
    _mirroredCameraPosition: Nullable<Vector3>;
    /**
     * This observable event is triggered when any keyboard event si raised and registered during Scene.attachControl()
     * You have the possibility to skip the process and the call to onKeyboardObservable by setting KeyboardInfoPre.skipOnPointerObservable to true
     */
    onPreKeyboardObservable: Observable<KeyboardInfoPre>;
    /**
     * Observable event triggered each time an keyboard event is received from the hosting window
     */
    onKeyboardObservable: Observable<KeyboardInfo>;
    private _useRightHandedSystem;
    /**
     * Gets or sets a boolean indicating if the scene must use right-handed coordinates system
     */
    set useRightHandedSystem(value: boolean);
    get useRightHandedSystem(): boolean;
    private _timeAccumulator;
    private _currentStepId;
    private _currentInternalStep;
    /**
     * Sets the step Id used by deterministic lock step
     * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/advanced_animations#deterministic-lockstep
     * @param newStepId defines the step Id
     */
    setStepId(newStepId: number): void;
    /**
     * Gets the step Id used by deterministic lock step
     * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/advanced_animations#deterministic-lockstep
     * @returns the step Id
     */
    getStepId(): number;
    /**
     * Gets the internal step used by deterministic lock step
     * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/advanced_animations#deterministic-lockstep
     * @returns the internal step
     */
    getInternalStep(): number;
    private _fogEnabled;
    /**
     * Gets or sets a boolean indicating if fog is enabled on this scene
     * @see https://doc.babylonjs.com/features/featuresDeepDive/environment/environment_introduction#fog
     * (Default is true)
     */
    set fogEnabled(value: boolean);
    get fogEnabled(): boolean;
    private _fogMode;
    /**
     * Gets or sets the fog mode to use
     * @see https://doc.babylonjs.com/features/featuresDeepDive/environment/environment_introduction#fog
     * | mode | value |
     * | --- | --- |
     * | FOGMODE_NONE | 0 |
     * | FOGMODE_EXP | 1 |
     * | FOGMODE_EXP2 | 2 |
     * | FOGMODE_LINEAR | 3 |
     */
    set fogMode(value: number);
    get fogMode(): number;
    /**
     * Gets or sets the fog color to use
     * @see https://doc.babylonjs.com/features/featuresDeepDive/environment/environment_introduction#fog
     * (Default is Color3(0.2, 0.2, 0.3))
     */
    fogColor: Color3;
    /**
     * Gets or sets the fog density to use
     * @see https://doc.babylonjs.com/features/featuresDeepDive/environment/environment_introduction#fog
     * (Default is 0.1)
     */
    fogDensity: number;
    /**
     * Gets or sets the fog start distance to use
     * @see https://doc.babylonjs.com/features/featuresDeepDive/environment/environment_introduction#fog
     * (Default is 0)
     */
    fogStart: number;
    /**
     * Gets or sets the fog end distance to use
     * @see https://doc.babylonjs.com/features/featuresDeepDive/environment/environment_introduction#fog
     * (Default is 1000)
     */
    fogEnd: number;
    /**
     * Flag indicating that the frame buffer binding is handled by another component
     */
    get prePass(): boolean;
    /**
     * Flag indicating if we need to store previous matrices when rendering
     */
    needsPreviousWorldMatrices: boolean;
    private _shadowsEnabled;
    /**
     * Gets or sets a boolean indicating if shadows are enabled on this scene
     */
    set shadowsEnabled(value: boolean);
    get shadowsEnabled(): boolean;
    private _lightsEnabled;
    /**
     * Gets or sets a boolean indicating if lights are enabled on this scene
     */
    set lightsEnabled(value: boolean);
    get lightsEnabled(): boolean;
    private _activeCameras;
    private _unObserveActiveCameras;
    /** All of the active cameras added to this scene. */
    get activeCameras(): Nullable<Camera[]>;
    set activeCameras(cameras: Nullable<Camera[]>);
    /** @internal */
    _activeCamera: Nullable<Camera>;
    /** Gets or sets the current active camera */
    get activeCamera(): Nullable<Camera>;
    set activeCamera(value: Nullable<Camera>);
    private _defaultMaterial;
    /** The default material used on meshes when no material is affected */
    get defaultMaterial(): Material;
    /** The default material used on meshes when no material is affected */
    set defaultMaterial(value: Material);
    private _texturesEnabled;
    /**
     * Gets or sets a boolean indicating if textures are enabled on this scene
     */
    set texturesEnabled(value: boolean);
    get texturesEnabled(): boolean;
    /**
     * Gets or sets a boolean indicating if physic engines are enabled on this scene
     */
    physicsEnabled: boolean;
    /**
     * Gets or sets a boolean indicating if particles are enabled on this scene
     */
    particlesEnabled: boolean;
    /**
     * Gets or sets a boolean indicating if sprites are enabled on this scene
     */
    spritesEnabled: boolean;
    private _skeletonsEnabled;
    /**
     * Gets or sets a boolean indicating if skeletons are enabled on this scene
     */
    set skeletonsEnabled(value: boolean);
    get skeletonsEnabled(): boolean;
    /**
     * Gets or sets a boolean indicating if lens flares are enabled on this scene
     */
    lensFlaresEnabled: boolean;
    /**
     * Gets or sets a boolean indicating if collisions are enabled on this scene
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_collisions
     */
    collisionsEnabled: boolean;
    private _collisionCoordinator;
    /** @internal */
    get collisionCoordinator(): ICollisionCoordinator;
    /**
     * Defines the gravity applied to this scene (used only for collisions)
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_collisions
     */
    gravity: Vector3;
    /**
     * Gets or sets a boolean indicating if postprocesses are enabled on this scene
     */
    postProcessesEnabled: boolean;
    /**
     * Gets the current postprocess manager
     */
    postProcessManager: PostProcessManager;
    /**
     * Gets or sets a boolean indicating if render targets are enabled on this scene
     */
    renderTargetsEnabled: boolean;
    /**
     * Gets or sets a boolean indicating if next render targets must be dumped as image for debugging purposes
     * We recommend not using it and instead rely on Spector.js: http://spector.babylonjs.com
     */
    dumpNextRenderTargets: boolean;
    /**
     * The list of user defined render targets added to the scene
     */
    customRenderTargets: import("babylonjs/Materials/Textures/renderTargetTexture").RenderTargetTexture[];
    /**
     * Defines if texture loading must be delayed
     * If true, textures will only be loaded when they need to be rendered
     */
    useDelayedTextureLoading: boolean;
    /**
     * Gets the list of meshes imported to the scene through SceneLoader
     */
    importedMeshesFiles: string[];
    /**
     * Gets or sets a boolean indicating if probes are enabled on this scene
     */
    probesEnabled: boolean;
    /**
     * Gets or sets the current offline provider to use to store scene data
     * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/optimizeCached
     */
    offlineProvider: IOfflineProvider;
    /**
     * Gets or sets the action manager associated with the scene
     * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions
     */
    actionManager: AbstractActionManager;
    private _meshesForIntersections;
    /**
     * Gets or sets a boolean indicating if procedural textures are enabled on this scene
     */
    proceduralTexturesEnabled: boolean;
    private _engine;
    private _totalVertices;
    /** @internal */
    _activeIndices: PerfCounter;
    /** @internal */
    _activeParticles: PerfCounter;
    /** @internal */
    _activeBones: PerfCounter;
    private _animationRatio;
    /** @internal */
    _animationTimeLast: number;
    /** @internal */
    _animationTime: number;
    /**
     * Gets or sets a general scale for animation speed
     * @see https://www.babylonjs-playground.com/#IBU2W7#3
     */
    animationTimeScale: number;
    /** @internal */
    _cachedMaterial: Nullable<Material>;
    /** @internal */
    _cachedEffect: Nullable<Effect>;
    /** @internal */
    _cachedVisibility: Nullable<number>;
    private _renderId;
    private _frameId;
    private _executeWhenReadyTimeoutId;
    private _intermediateRendering;
    private _defaultFrameBufferCleared;
    private _viewUpdateFlag;
    private _projectionUpdateFlag;
    /** @internal */
    _toBeDisposed: Nullable<IDisposable>[];
    private _activeRequests;
    /** @internal */
    _pendingData: any[];
    private _isDisposed;
    /**
     * Gets or sets a boolean indicating that all submeshes of active meshes must be rendered
     * Use this boolean to avoid computing frustum clipping on submeshes (This could help when you are CPU bound)
     */
    dispatchAllSubMeshesOfActiveMeshes: boolean;
    private _activeMeshes;
    private _processedMaterials;
    private _renderTargets;
    private _materialsRenderTargets;
    /** @internal */
    _activeParticleSystems: SmartArray<IParticleSystem>;
    private _activeSkeletons;
    private _softwareSkinnedMeshes;
    private _renderingManager;
    /**
     * Gets the scene's rendering manager
     */
    get renderingManager(): RenderingManager;
    /** @internal */
    _activeAnimatables: import("babylonjs/Animations/animatable").Animatable[];
    private _transformMatrix;
    private _sceneUbo;
    /** @internal */
    _viewMatrix: Matrix;
    /** @internal */
    _projectionMatrix: Matrix;
    /** @internal */
    _forcedViewPosition: Nullable<Vector3>;
    /** @internal */
    _frustumPlanes: Plane[];
    /**
     * Gets the list of frustum planes (built from the active camera)
     */
    get frustumPlanes(): Plane[];
    /**
     * Gets or sets a boolean indicating if lights must be sorted by priority (off by default)
     * This is useful if there are more lights that the maximum simulteanous authorized
     */
    requireLightSorting: boolean;
    /** @internal */
    readonly useMaterialMeshMap: boolean;
    /** @internal */
    readonly useClonedMeshMap: boolean;
    private _externalData;
    private _uid;
    /**
     * @internal
     * Backing store of defined scene components.
     */
    _components: ISceneComponent[];
    /**
     * @internal
     * Backing store of defined scene components.
     */
    _serializableComponents: ISceneSerializableComponent[];
    /**
     * List of components to register on the next registration step.
     */
    private _transientComponents;
    /**
     * Registers the transient components if needed.
     */
    private _registerTransientComponents;
    /**
     * @internal
     * Add a component to the scene.
     * Note that the ccomponent could be registered on th next frame if this is called after
     * the register component stage.
     * @param component Defines the component to add to the scene
     */
    _addComponent(component: ISceneComponent): void;
    /**
     * @internal
     * Gets a component from the scene.
     * @param name defines the name of the component to retrieve
     * @returns the component or null if not present
     */
    _getComponent(name: string): Nullable<ISceneComponent>;
    /**
     * @internal
     * Defines the actions happening before camera updates.
     */
    _beforeCameraUpdateStage: Stage<SimpleStageAction>;
    /**
     * @internal
     * Defines the actions happening before clear the canvas.
     */
    _beforeClearStage: Stage<SimpleStageAction>;
    /**
     * @internal
     * Defines the actions happening before clear the canvas.
     */
    _beforeRenderTargetClearStage: Stage<RenderTargetStageAction>;
    /**
     * @internal
     * Defines the actions when collecting render targets for the frame.
     */
    _gatherRenderTargetsStage: Stage<RenderTargetsStageAction>;
    /**
     * @internal
     * Defines the actions happening for one camera in the frame.
     */
    _gatherActiveCameraRenderTargetsStage: Stage<RenderTargetsStageAction>;
    /**
     * @internal
     * Defines the actions happening during the per mesh ready checks.
     */
    _isReadyForMeshStage: Stage<MeshStageAction>;
    /**
     * @internal
     * Defines the actions happening before evaluate active mesh checks.
     */
    _beforeEvaluateActiveMeshStage: Stage<SimpleStageAction>;
    /**
     * @internal
     * Defines the actions happening during the evaluate sub mesh checks.
     */
    _evaluateSubMeshStage: Stage<EvaluateSubMeshStageAction>;
    /**
     * @internal
     * Defines the actions happening during the active mesh stage.
     */
    _preActiveMeshStage: Stage<PreActiveMeshStageAction>;
    /**
     * @internal
     * Defines the actions happening during the per camera render target step.
     */
    _cameraDrawRenderTargetStage: Stage<CameraStageFrameBufferAction>;
    /**
     * @internal
     * Defines the actions happening just before the active camera is drawing.
     */
    _beforeCameraDrawStage: Stage<CameraStageAction>;
    /**
     * @internal
     * Defines the actions happening just before a render target is drawing.
     */
    _beforeRenderTargetDrawStage: Stage<RenderTargetStageAction>;
    /**
     * @internal
     * Defines the actions happening just before a rendering group is drawing.
     */
    _beforeRenderingGroupDrawStage: Stage<RenderingGroupStageAction>;
    /**
     * @internal
     * Defines the actions happening just before a mesh is drawing.
     */
    _beforeRenderingMeshStage: Stage<RenderingMeshStageAction>;
    /**
     * @internal
     * Defines the actions happening just after a mesh has been drawn.
     */
    _afterRenderingMeshStage: Stage<RenderingMeshStageAction>;
    /**
     * @internal
     * Defines the actions happening just after a rendering group has been drawn.
     */
    _afterRenderingGroupDrawStage: Stage<RenderingGroupStageAction>;
    /**
     * @internal
     * Defines the actions happening just after the active camera has been drawn.
     */
    _afterCameraDrawStage: Stage<CameraStageAction>;
    /**
     * @internal
     * Defines the actions happening just after the post processing
     */
    _afterCameraPostProcessStage: Stage<CameraStageAction>;
    /**
     * @internal
     * Defines the actions happening just after a render target has been drawn.
     */
    _afterRenderTargetDrawStage: Stage<RenderTargetStageAction>;
    /**
     * Defines the actions happening just after the post processing on a render target
     */
    _afterRenderTargetPostProcessStage: Stage<RenderTargetStageAction>;
    /**
     * @internal
     * Defines the actions happening just after rendering all cameras and computing intersections.
     */
    _afterRenderStage: Stage<SimpleStageAction>;
    /**
     * @internal
     * Defines the actions happening when a pointer move event happens.
     */
    _pointerMoveStage: Stage<PointerMoveStageAction>;
    /**
     * @internal
     * Defines the actions happening when a pointer down event happens.
     */
    _pointerDownStage: Stage<PointerUpDownStageAction>;
    /**
     * @internal
     * Defines the actions happening when a pointer up event happens.
     */
    _pointerUpStage: Stage<PointerUpDownStageAction>;
    /**
     * an optional map from Geometry Id to Geometry index in the 'geometries' array
     */
    private _geometriesByUniqueId;
    /**
     * Creates a new Scene
     * @param engine defines the engine to use to render this scene
     * @param options defines the scene options
     */
    constructor(engine: Engine, options?: SceneOptions);
    /**
     * Gets a string identifying the name of the class
     * @returns "Scene" string
     */
    getClassName(): string;
    private _defaultMeshCandidates;
    /**
     * @internal
     */
    _getDefaultMeshCandidates(): ISmartArrayLike<AbstractMesh>;
    private _defaultSubMeshCandidates;
    /**
     * @internal
     */
    _getDefaultSubMeshCandidates(mesh: AbstractMesh): ISmartArrayLike<SubMesh>;
    /**
     * Sets the default candidate providers for the scene.
     * This sets the getActiveMeshCandidates, getActiveSubMeshCandidates, getIntersectingSubMeshCandidates
     * and getCollidingSubMeshCandidates to their default function
     */
    setDefaultCandidateProviders(): void;
    /**
     * Gets the mesh that is currently under the pointer
     */
    get meshUnderPointer(): Nullable<AbstractMesh>;
    /**
     * Gets or sets the current on-screen X position of the pointer
     */
    get pointerX(): number;
    set pointerX(value: number);
    /**
     * Gets or sets the current on-screen Y position of the pointer
     */
    get pointerY(): number;
    set pointerY(value: number);
    /**
     * Gets the cached material (ie. the latest rendered one)
     * @returns the cached material
     */
    getCachedMaterial(): Nullable<Material>;
    /**
     * Gets the cached effect (ie. the latest rendered one)
     * @returns the cached effect
     */
    getCachedEffect(): Nullable<Effect>;
    /**
     * Gets the cached visibility state (ie. the latest rendered one)
     * @returns the cached visibility state
     */
    getCachedVisibility(): Nullable<number>;
    /**
     * Gets a boolean indicating if the current material / effect / visibility must be bind again
     * @param material defines the current material
     * @param effect defines the current effect
     * @param visibility defines the current visibility state
     * @returns true if one parameter is not cached
     */
    isCachedMaterialInvalid(material: Material, effect: Effect, visibility?: number): boolean;
    /**
     * Gets the engine associated with the scene
     * @returns an Engine
     */
    getEngine(): Engine;
    /**
     * Gets the total number of vertices rendered per frame
     * @returns the total number of vertices rendered per frame
     */
    getTotalVertices(): number;
    /**
     * Gets the performance counter for total vertices
     * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/optimize_your_scene#instrumentation
     */
    get totalVerticesPerfCounter(): PerfCounter;
    /**
     * Gets the total number of active indices rendered per frame (You can deduce the number of rendered triangles by dividing this number by 3)
     * @returns the total number of active indices rendered per frame
     */
    getActiveIndices(): number;
    /**
     * Gets the performance counter for active indices
     * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/optimize_your_scene#instrumentation
     */
    get totalActiveIndicesPerfCounter(): PerfCounter;
    /**
     * Gets the total number of active particles rendered per frame
     * @returns the total number of active particles rendered per frame
     */
    getActiveParticles(): number;
    /**
     * Gets the performance counter for active particles
     * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/optimize_your_scene#instrumentation
     */
    get activeParticlesPerfCounter(): PerfCounter;
    /**
     * Gets the total number of active bones rendered per frame
     * @returns the total number of active bones rendered per frame
     */
    getActiveBones(): number;
    /**
     * Gets the performance counter for active bones
     * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/optimize_your_scene#instrumentation
     */
    get activeBonesPerfCounter(): PerfCounter;
    /**
     * Gets the array of active meshes
     * @returns an array of AbstractMesh
     */
    getActiveMeshes(): SmartArray<AbstractMesh>;
    /**
     * Gets the animation ratio (which is 1.0 is the scene renders at 60fps and 2 if the scene renders at 30fps, etc.)
     * @returns a number
     */
    getAnimationRatio(): number;
    /**
     * Gets an unique Id for the current render phase
     * @returns a number
     */
    getRenderId(): number;
    /**
     * Gets an unique Id for the current frame
     * @returns a number
     */
    getFrameId(): number;
    /** Call this function if you want to manually increment the render Id*/
    incrementRenderId(): void;
    private _createUbo;
    /**
     * Use this method to simulate a pointer move on a mesh
     * The pickResult parameter can be obtained from a scene.pick or scene.pickWithRay
     * @param pickResult pickingInfo of the object wished to simulate pointer event on
     * @param pointerEventInit pointer event state to be used when simulating the pointer event (eg. pointer id for multitouch)
     * @returns the current scene
     */
    simulatePointerMove(pickResult: PickingInfo, pointerEventInit?: PointerEventInit): Scene;
    /**
     * Use this method to simulate a pointer down on a mesh
     * The pickResult parameter can be obtained from a scene.pick or scene.pickWithRay
     * @param pickResult pickingInfo of the object wished to simulate pointer event on
     * @param pointerEventInit pointer event state to be used when simulating the pointer event (eg. pointer id for multitouch)
     * @returns the current scene
     */
    simulatePointerDown(pickResult: PickingInfo, pointerEventInit?: PointerEventInit): Scene;
    /**
     * Use this method to simulate a pointer up on a mesh
     * The pickResult parameter can be obtained from a scene.pick or scene.pickWithRay
     * @param pickResult pickingInfo of the object wished to simulate pointer event on
     * @param pointerEventInit pointer event state to be used when simulating the pointer event (eg. pointer id for multitouch)
     * @param doubleTap indicates that the pointer up event should be considered as part of a double click (false by default)
     * @returns the current scene
     */
    simulatePointerUp(pickResult: PickingInfo, pointerEventInit?: PointerEventInit, doubleTap?: boolean): Scene;
    /**
     * Gets a boolean indicating if the current pointer event is captured (meaning that the scene has already handled the pointer down)
     * @param pointerId defines the pointer id to use in a multi-touch scenario (0 by default)
     * @returns true if the pointer was captured
     */
    isPointerCaptured(pointerId?: number): boolean;
    /**
     * Attach events to the canvas (To handle actionManagers triggers and raise onPointerMove, onPointerDown and onPointerUp
     * @param attachUp defines if you want to attach events to pointerup
     * @param attachDown defines if you want to attach events to pointerdown
     * @param attachMove defines if you want to attach events to pointermove
     */
    attachControl(attachUp?: boolean, attachDown?: boolean, attachMove?: boolean): void;
    /** Detaches all event handlers*/
    detachControl(): void;
    /**
     * This function will check if the scene can be rendered (textures are loaded, shaders are compiled)
     * Delay loaded resources are not taking in account
     * @param checkRenderTargets true to also check that the meshes rendered as part of a render target are ready (default: true)
     * @returns true if all required resources are ready
     */
    isReady(checkRenderTargets?: boolean): boolean;
    /** Resets all cached information relative to material (including effect and visibility) */
    resetCachedMaterial(): void;
    /**
     * Registers a function to be called before every frame render
     * @param func defines the function to register
     */
    registerBeforeRender(func: () => void): void;
    /**
     * Unregisters a function called before every frame render
     * @param func defines the function to unregister
     */
    unregisterBeforeRender(func: () => void): void;
    /**
     * Registers a function to be called after every frame render
     * @param func defines the function to register
     */
    registerAfterRender(func: () => void): void;
    /**
     * Unregisters a function called after every frame render
     * @param func defines the function to unregister
     */
    unregisterAfterRender(func: () => void): void;
    private _executeOnceBeforeRender;
    /**
     * The provided function will run before render once and will be disposed afterwards.
     * A timeout delay can be provided so that the function will be executed in N ms.
     * The timeout is using the browser's native setTimeout so time percision cannot be guaranteed.
     * @param func The function to be executed.
     * @param timeout optional delay in ms
     */
    executeOnceBeforeRender(func: () => void, timeout?: number): void;
    /**
     * This function can help adding any object to the list of data awaited to be ready in order to check for a complete scene loading.
     * @param data defines the object to wait for
     */
    addPendingData(data: any): void;
    /**
     * Remove a pending data from the loading list which has previously been added with addPendingData.
     * @param data defines the object to remove from the pending list
     */
    removePendingData(data: any): void;
    /**
     * Returns the number of items waiting to be loaded
     * @returns the number of items waiting to be loaded
     */
    getWaitingItemsCount(): number;
    /**
     * Returns a boolean indicating if the scene is still loading data
     */
    get isLoading(): boolean;
    /**
     * Registers a function to be executed when the scene is ready
     * @param {Function} func - the function to be executed
     * @param checkRenderTargets true to also check that the meshes rendered as part of a render target are ready (default: false)
     */
    executeWhenReady(func: () => void, checkRenderTargets?: boolean): void;
    /**
     * Returns a promise that resolves when the scene is ready
     * @param checkRenderTargets true to also check that the meshes rendered as part of a render target are ready (default: false)
     * @returns A promise that resolves when the scene is ready
     */
    whenReadyAsync(checkRenderTargets?: boolean): Promise<void>;
    /**
     * @internal
     */
    _checkIsReady(checkRenderTargets?: boolean): void;
    /**
     * Gets all animatable attached to the scene
     */
    get animatables(): Animatable[];
    /**
     * Resets the last animation time frame.
     * Useful to override when animations start running when loading a scene for the first time.
     */
    resetLastAnimationTimeFrame(): void;
    /**
     * Gets the current view matrix
     * @returns a Matrix
     */
    getViewMatrix(): Matrix;
    /**
     * Gets the current projection matrix
     * @returns a Matrix
     */
    getProjectionMatrix(): Matrix;
    /**
     * Gets the current transform matrix
     * @returns a Matrix made of View * Projection
     */
    getTransformMatrix(): Matrix;
    /**
     * Sets the current transform matrix
     * @param viewL defines the View matrix to use
     * @param projectionL defines the Projection matrix to use
     * @param viewR defines the right View matrix to use (if provided)
     * @param projectionR defines the right Projection matrix to use (if provided)
     */
    setTransformMatrix(viewL: Matrix, projectionL: Matrix, viewR?: Matrix, projectionR?: Matrix): void;
    /**
     * Gets the uniform buffer used to store scene data
     * @returns a UniformBuffer
     */
    getSceneUniformBuffer(): UniformBuffer;
    /**
     * Creates a scene UBO
     * @param name name of the uniform buffer (optional, for debugging purpose only)
     * @returns a new ubo
     */
    createSceneUniformBuffer(name?: string): UniformBuffer;
    /**
     * Sets the scene ubo
     * @param ubo the ubo to set for the scene
     */
    setSceneUniformBuffer(ubo: UniformBuffer): void;
    /**
     * Gets an unique (relatively to the current scene) Id
     * @returns an unique number for the scene
     */
    getUniqueId(): number;
    /**
     * Add a mesh to the list of scene's meshes
     * @param newMesh defines the mesh to add
     * @param recursive if all child meshes should also be added to the scene
     */
    addMesh(newMesh: AbstractMesh, recursive?: boolean): void;
    /**
     * Remove a mesh for the list of scene's meshes
     * @param toRemove defines the mesh to remove
     * @param recursive if all child meshes should also be removed from the scene
     * @returns the index where the mesh was in the mesh list
     */
    removeMesh(toRemove: AbstractMesh, recursive?: boolean): number;
    /**
     * Add a transform node to the list of scene's transform nodes
     * @param newTransformNode defines the transform node to add
     */
    addTransformNode(newTransformNode: TransformNode): void;
    /**
     * Remove a transform node for the list of scene's transform nodes
     * @param toRemove defines the transform node to remove
     * @returns the index where the transform node was in the transform node list
     */
    removeTransformNode(toRemove: TransformNode): number;
    /**
     * Remove a skeleton for the list of scene's skeletons
     * @param toRemove defines the skeleton to remove
     * @returns the index where the skeleton was in the skeleton list
     */
    removeSkeleton(toRemove: Skeleton): number;
    /**
     * Remove a morph target for the list of scene's morph targets
     * @param toRemove defines the morph target to remove
     * @returns the index where the morph target was in the morph target list
     */
    removeMorphTargetManager(toRemove: MorphTargetManager): number;
    /**
     * Remove a light for the list of scene's lights
     * @param toRemove defines the light to remove
     * @returns the index where the light was in the light list
     */
    removeLight(toRemove: Light): number;
    /**
     * Remove a camera for the list of scene's cameras
     * @param toRemove defines the camera to remove
     * @returns the index where the camera was in the camera list
     */
    removeCamera(toRemove: Camera): number;
    /**
     * Remove a particle system for the list of scene's particle systems
     * @param toRemove defines the particle system to remove
     * @returns the index where the particle system was in the particle system list
     */
    removeParticleSystem(toRemove: IParticleSystem): number;
    /**
     * Remove a animation for the list of scene's animations
     * @param toRemove defines the animation to remove
     * @returns the index where the animation was in the animation list
     */
    removeAnimation(toRemove: Animation): number;
    /**
     * Will stop the animation of the given target
     * @param target - the target
     * @param animationName - the name of the animation to stop (all animations will be stopped if both this and targetMask are empty)
     * @param targetMask - a function that determines if the animation should be stopped based on its target (all animations will be stopped if both this and animationName are empty)
     */
    stopAnimation(target: any, animationName?: string, targetMask?: (target: any) => boolean): void;
    /**
     * Removes the given animation group from this scene.
     * @param toRemove The animation group to remove
     * @returns The index of the removed animation group
     */
    removeAnimationGroup(toRemove: AnimationGroup): number;
    /**
     * Removes the given multi-material from this scene.
     * @param toRemove The multi-material to remove
     * @returns The index of the removed multi-material
     */
    removeMultiMaterial(toRemove: MultiMaterial): number;
    /**
     * Removes the given material from this scene.
     * @param toRemove The material to remove
     * @returns The index of the removed material
     */
    removeMaterial(toRemove: Material): number;
    /**
     * Removes the given action manager from this scene.
     * @deprecated
     * @param toRemove The action manager to remove
     * @returns The index of the removed action manager
     */
    removeActionManager(toRemove: AbstractActionManager): number;
    /**
     * Removes the given texture from this scene.
     * @param toRemove The texture to remove
     * @returns The index of the removed texture
     */
    removeTexture(toRemove: BaseTexture): number;
    /**
     * Adds the given light to this scene
     * @param newLight The light to add
     */
    addLight(newLight: Light): void;
    /**
     * Sorts the list list based on light priorities
     */
    sortLightsByPriority(): void;
    /**
     * Adds the given camera to this scene
     * @param newCamera The camera to add
     */
    addCamera(newCamera: Camera): void;
    /**
     * Adds the given skeleton to this scene
     * @param newSkeleton The skeleton to add
     */
    addSkeleton(newSkeleton: Skeleton): void;
    /**
     * Adds the given particle system to this scene
     * @param newParticleSystem The particle system to add
     */
    addParticleSystem(newParticleSystem: IParticleSystem): void;
    /**
     * Adds the given animation to this scene
     * @param newAnimation The animation to add
     */
    addAnimation(newAnimation: Animation): void;
    /**
     * Adds the given animation group to this scene.
     * @param newAnimationGroup The animation group to add
     */
    addAnimationGroup(newAnimationGroup: AnimationGroup): void;
    /**
     * Adds the given multi-material to this scene
     * @param newMultiMaterial The multi-material to add
     */
    addMultiMaterial(newMultiMaterial: MultiMaterial): void;
    /**
     * Adds the given material to this scene
     * @param newMaterial The material to add
     */
    addMaterial(newMaterial: Material): void;
    /**
     * Adds the given morph target to this scene
     * @param newMorphTargetManager The morph target to add
     */
    addMorphTargetManager(newMorphTargetManager: MorphTargetManager): void;
    /**
     * Adds the given geometry to this scene
     * @param newGeometry The geometry to add
     */
    addGeometry(newGeometry: Geometry): void;
    /**
     * Adds the given action manager to this scene
     * @deprecated
     * @param newActionManager The action manager to add
     */
    addActionManager(newActionManager: AbstractActionManager): void;
    /**
     * Adds the given texture to this scene.
     * @param newTexture The texture to add
     */
    addTexture(newTexture: BaseTexture): void;
    /**
     * Switch active camera
     * @param newCamera defines the new active camera
     * @param attachControl defines if attachControl must be called for the new active camera (default: true)
     */
    switchActiveCamera(newCamera: Camera, attachControl?: boolean): void;
    /**
     * sets the active camera of the scene using its Id
     * @param id defines the camera's Id
     * @returns the new active camera or null if none found.
     */
    setActiveCameraById(id: string): Nullable<Camera>;
    /**
     * sets the active camera of the scene using its name
     * @param name defines the camera's name
     * @returns the new active camera or null if none found.
     */
    setActiveCameraByName(name: string): Nullable<Camera>;
    /**
     * get an animation group using its name
     * @param name defines the material's name
     * @returns the animation group or null if none found.
     */
    getAnimationGroupByName(name: string): Nullable<AnimationGroup>;
    private _getMaterial;
    /**
     * Get a material using its unique id
     * @param uniqueId defines the material's unique id
     * @param allowMultiMaterials determines whether multimaterials should be considered
     * @returns the material or null if none found.
     */
    getMaterialByUniqueID(uniqueId: number, allowMultiMaterials?: boolean): Nullable<Material>;
    /**
     * get a material using its id
     * @param id defines the material's Id
     * @param allowMultiMaterials determines whether multimaterials should be considered
     * @returns the material or null if none found.
     */
    getMaterialById(id: string, allowMultiMaterials?: boolean): Nullable<Material>;
    /**
     * Gets a material using its name
     * @param name defines the material's name
     * @param allowMultiMaterials determines whether multimaterials should be considered
     * @returns the material or null if none found.
     */
    getMaterialByName(name: string, allowMultiMaterials?: boolean): Nullable<Material>;
    /**
     * Gets a last added material using a given id
     * @param id defines the material's id
     * @param allowMultiMaterials determines whether multimaterials should be considered
     * @returns the last material with the given id or null if none found.
     */
    getLastMaterialById(id: string, allowMultiMaterials?: boolean): Nullable<Material>;
    /**
     * Get a texture using its unique id
     * @param uniqueId defines the texture's unique id
     * @returns the texture or null if none found.
     */
    getTextureByUniqueId(uniqueId: number): Nullable<BaseTexture>;
    /**
     * Gets a texture using its name
     * @param name defines the texture's name
     * @returns the texture or null if none found.
     */
    getTextureByName(name: string): Nullable<BaseTexture>;
    /**
     * Gets a camera using its Id
     * @param id defines the Id to look for
     * @returns the camera or null if not found
     */
    getCameraById(id: string): Nullable<Camera>;
    /**
     * Gets a camera using its unique Id
     * @param uniqueId defines the unique Id to look for
     * @returns the camera or null if not found
     */
    getCameraByUniqueId(uniqueId: number): Nullable<Camera>;
    /**
     * Gets a camera using its name
     * @param name defines the camera's name
     * @returns the camera or null if none found.
     */
    getCameraByName(name: string): Nullable<Camera>;
    /**
     * Gets a bone using its Id
     * @param id defines the bone's Id
     * @returns the bone or null if not found
     */
    getBoneById(id: string): Nullable<Bone>;
    /**
     * Gets a bone using its id
     * @param name defines the bone's name
     * @returns the bone or null if not found
     */
    getBoneByName(name: string): Nullable<Bone>;
    /**
     * Gets a light node using its name
     * @param name defines the the light's name
     * @returns the light or null if none found.
     */
    getLightByName(name: string): Nullable<Light>;
    /**
     * Gets a light node using its Id
     * @param id defines the light's Id
     * @returns the light or null if none found.
     */
    getLightById(id: string): Nullable<Light>;
    /**
     * Gets a light node using its scene-generated unique Id
     * @param uniqueId defines the light's unique Id
     * @returns the light or null if none found.
     */
    getLightByUniqueId(uniqueId: number): Nullable<Light>;
    /**
     * Gets a particle system by Id
     * @param id defines the particle system Id
     * @returns the corresponding system or null if none found
     */
    getParticleSystemById(id: string): Nullable<IParticleSystem>;
    /**
     * Gets a geometry using its Id
     * @param id defines the geometry's Id
     * @returns the geometry or null if none found.
     */
    getGeometryById(id: string): Nullable<Geometry>;
    private _getGeometryByUniqueId;
    /**
     * Add a new geometry to this scene
     * @param geometry defines the geometry to be added to the scene.
     * @param force defines if the geometry must be pushed even if a geometry with this id already exists
     * @returns a boolean defining if the geometry was added or not
     */
    pushGeometry(geometry: Geometry, force?: boolean): boolean;
    /**
     * Removes an existing geometry
     * @param geometry defines the geometry to be removed from the scene
     * @returns a boolean defining if the geometry was removed or not
     */
    removeGeometry(geometry: Geometry): boolean;
    /**
     * Gets the list of geometries attached to the scene
     * @returns an array of Geometry
     */
    getGeometries(): Geometry[];
    /**
     * Gets the first added mesh found of a given Id
     * @param id defines the Id to search for
     * @returns the mesh found or null if not found at all
     */
    getMeshById(id: string): Nullable<AbstractMesh>;
    /**
     * Gets a list of meshes using their Id
     * @param id defines the Id to search for
     * @returns a list of meshes
     */
    getMeshesById(id: string): Array<AbstractMesh>;
    /**
     * Gets the first added transform node found of a given Id
     * @param id defines the Id to search for
     * @returns the found transform node or null if not found at all.
     */
    getTransformNodeById(id: string): Nullable<TransformNode>;
    /**
     * Gets a transform node with its auto-generated unique Id
     * @param uniqueId defines the unique Id to search for
     * @returns the found transform node or null if not found at all.
     */
    getTransformNodeByUniqueId(uniqueId: number): Nullable<TransformNode>;
    /**
     * Gets a list of transform nodes using their Id
     * @param id defines the Id to search for
     * @returns a list of transform nodes
     */
    getTransformNodesById(id: string): Array<TransformNode>;
    /**
     * Gets a mesh with its auto-generated unique Id
     * @param uniqueId defines the unique Id to search for
     * @returns the found mesh or null if not found at all.
     */
    getMeshByUniqueId(uniqueId: number): Nullable<AbstractMesh>;
    /**
     * Gets a the last added mesh using a given Id
     * @param id defines the Id to search for
     * @returns the found mesh or null if not found at all.
     */
    getLastMeshById(id: string): Nullable<AbstractMesh>;
    /**
     * Gets a the last added node (Mesh, Camera, Light) using a given Id
     * @param id defines the Id to search for
     * @returns the found node or null if not found at all
     */
    getLastEntryById(id: string): Nullable<Node>;
    /**
     * Gets a node (Mesh, Camera, Light) using a given Id
     * @param id defines the Id to search for
     * @returns the found node or null if not found at all
     */
    getNodeById(id: string): Nullable<Node>;
    /**
     * Gets a node (Mesh, Camera, Light) using a given name
     * @param name defines the name to search for
     * @returns the found node or null if not found at all.
     */
    getNodeByName(name: string): Nullable<Node>;
    /**
     * Gets a mesh using a given name
     * @param name defines the name to search for
     * @returns the found mesh or null if not found at all.
     */
    getMeshByName(name: string): Nullable<AbstractMesh>;
    /**
     * Gets a transform node using a given name
     * @param name defines the name to search for
     * @returns the found transform node or null if not found at all.
     */
    getTransformNodeByName(name: string): Nullable<TransformNode>;
    /**
     * Gets a skeleton using a given Id (if many are found, this function will pick the last one)
     * @param id defines the Id to search for
     * @returns the found skeleton or null if not found at all.
     */
    getLastSkeletonById(id: string): Nullable<Skeleton>;
    /**
     * Gets a skeleton using a given auto generated unique id
     * @param  uniqueId defines the unique id to search for
     * @returns the found skeleton or null if not found at all.
     */
    getSkeletonByUniqueId(uniqueId: number): Nullable<Skeleton>;
    /**
     * Gets a skeleton using a given id (if many are found, this function will pick the first one)
     * @param id defines the id to search for
     * @returns the found skeleton or null if not found at all.
     */
    getSkeletonById(id: string): Nullable<Skeleton>;
    /**
     * Gets a skeleton using a given name
     * @param name defines the name to search for
     * @returns the found skeleton or null if not found at all.
     */
    getSkeletonByName(name: string): Nullable<Skeleton>;
    /**
     * Gets a morph target manager  using a given id (if many are found, this function will pick the last one)
     * @param id defines the id to search for
     * @returns the found morph target manager or null if not found at all.
     */
    getMorphTargetManagerById(id: number): Nullable<MorphTargetManager>;
    /**
     * Gets a morph target using a given id (if many are found, this function will pick the first one)
     * @param id defines the id to search for
     * @returns the found morph target or null if not found at all.
     */
    getMorphTargetById(id: string): Nullable<MorphTarget>;
    /**
     * Gets a morph target using a given name (if many are found, this function will pick the first one)
     * @param name defines the name to search for
     * @returns the found morph target or null if not found at all.
     */
    getMorphTargetByName(name: string): Nullable<MorphTarget>;
    /**
     * Gets a post process using a given name (if many are found, this function will pick the first one)
     * @param name defines the name to search for
     * @returns the found post process or null if not found at all.
     */
    getPostProcessByName(name: string): Nullable<PostProcess>;
    /**
     * Gets a boolean indicating if the given mesh is active
     * @param mesh defines the mesh to look for
     * @returns true if the mesh is in the active list
     */
    isActiveMesh(mesh: AbstractMesh): boolean;
    /**
     * Return a unique id as a string which can serve as an identifier for the scene
     */
    get uid(): string;
    /**
     * Add an externally attached data from its key.
     * This method call will fail and return false, if such key already exists.
     * If you don't care and just want to get the data no matter what, use the more convenient getOrAddExternalDataWithFactory() method.
     * @param key the unique key that identifies the data
     * @param data the data object to associate to the key for this Engine instance
     * @returns true if no such key were already present and the data was added successfully, false otherwise
     */
    addExternalData<T extends Object>(key: string, data: T): boolean;
    /**
     * Get an externally attached data from its key
     * @param key the unique key that identifies the data
     * @returns the associated data, if present (can be null), or undefined if not present
     */
    getExternalData<T>(key: string): Nullable<T>;
    /**
     * Get an externally attached data from its key, create it using a factory if it's not already present
     * @param key the unique key that identifies the data
     * @param factory the factory that will be called to create the instance if and only if it doesn't exists
     * @returns the associated data, can be null if the factory returned null.
     */
    getOrAddExternalDataWithFactory<T extends Object>(key: string, factory: (k: string) => T): T;
    /**
     * Remove an externally attached data from the Engine instance
     * @param key the unique key that identifies the data
     * @returns true if the data was successfully removed, false if it doesn't exist
     */
    removeExternalData(key: string): boolean;
    private _evaluateSubMesh;
    /**
     * Clear the processed materials smart array preventing retention point in material dispose.
     */
    freeProcessedMaterials(): void;
    private _preventFreeActiveMeshesAndRenderingGroups;
    /** Gets or sets a boolean blocking all the calls to freeActiveMeshes and freeRenderingGroups
     * It can be used in order to prevent going through methods freeRenderingGroups and freeActiveMeshes several times to improve performance
     * when disposing several meshes in a row or a hierarchy of meshes.
     * When used, it is the responsibility of the user to blockfreeActiveMeshesAndRenderingGroups back to false.
     */
    get blockfreeActiveMeshesAndRenderingGroups(): boolean;
    set blockfreeActiveMeshesAndRenderingGroups(value: boolean);
    /**
     * Clear the active meshes smart array preventing retention point in mesh dispose.
     */
    freeActiveMeshes(): void;
    /**
     * Clear the info related to rendering groups preventing retention points during dispose.
     */
    freeRenderingGroups(): void;
    /** @internal */
    _isInIntermediateRendering(): boolean;
    /**
     * Lambda returning the list of potentially active meshes.
     */
    getActiveMeshCandidates: () => ISmartArrayLike<AbstractMesh>;
    /**
     * Lambda returning the list of potentially active sub meshes.
     */
    getActiveSubMeshCandidates: (mesh: AbstractMesh) => ISmartArrayLike<SubMesh>;
    /**
     * Lambda returning the list of potentially intersecting sub meshes.
     */
    getIntersectingSubMeshCandidates: (mesh: AbstractMesh, localRay: Ray) => ISmartArrayLike<SubMesh>;
    /**
     * Lambda returning the list of potentially colliding sub meshes.
     */
    getCollidingSubMeshCandidates: (mesh: AbstractMesh, collider: Collider) => ISmartArrayLike<SubMesh>;
    /** @internal */
    _activeMeshesFrozen: boolean;
    /** @internal */
    _activeMeshesFrozenButKeepClipping: boolean;
    private _skipEvaluateActiveMeshesCompletely;
    /**
     * Use this function to stop evaluating active meshes. The current list will be keep alive between frames
     * @param skipEvaluateActiveMeshes defines an optional boolean indicating that the evaluate active meshes step must be completely skipped
     * @param onSuccess optional success callback
     * @param onError optional error callback
     * @param freezeMeshes defines if meshes should be frozen (true by default)
     * @param keepFrustumCulling defines if you want to keep running the frustum clipping (false by default)
     * @returns the current scene
     */
    freezeActiveMeshes(skipEvaluateActiveMeshes?: boolean, onSuccess?: () => void, onError?: (message: string) => void, freezeMeshes?: boolean, keepFrustumCulling?: boolean): Scene;
    /**
     * Use this function to restart evaluating active meshes on every frame
     * @returns the current scene
     */
    unfreezeActiveMeshes(): Scene;
    private _executeActiveContainerCleanup;
    private _evaluateActiveMeshes;
    private _activeMesh;
    /**
     * Update the transform matrix to update from the current active camera
     * @param force defines a boolean used to force the update even if cache is up to date
     */
    updateTransformMatrix(force?: boolean): void;
    private _bindFrameBuffer;
    private _clearFrameBuffer;
    /** @internal */
    _allowPostProcessClearColor: boolean;
    /**
     * @internal
     */
    _renderForCamera(camera: Camera, rigParent?: Camera, bindFrameBuffer?: boolean): void;
    private _processSubCameras;
    private _checkIntersections;
    /**
     * @internal
     */
    _advancePhysicsEngineStep(step: number): void;
    /**
     * User updatable function that will return a deterministic frame time when engine is in deterministic lock step mode
     */
    getDeterministicFrameTime: () => number;
    /** @internal */
    _animate(): void;
    /** Execute all animations (for a frame) */
    animate(): void;
    private _clear;
    private _checkCameraRenderTarget;
    /**
     * Resets the draw wrappers cache of all meshes
     * @param passId If provided, releases only the draw wrapper corresponding to this render pass id
     */
    resetDrawCache(passId?: number): void;
    /**
     * Render the scene
     * @param updateCameras defines a boolean indicating if cameras must update according to their inputs (true by default)
     * @param ignoreAnimations defines a boolean indicating if animations should not be executed (false by default)
     */
    render(updateCameras?: boolean, ignoreAnimations?: boolean): void;
    /**
     * Freeze all materials
     * A frozen material will not be updatable but should be faster to render
     * Note: multimaterials will not be frozen, but their submaterials will
     */
    freezeMaterials(): void;
    /**
     * Unfreeze all materials
     * A frozen material will not be updatable but should be faster to render
     */
    unfreezeMaterials(): void;
    /**
     * Releases all held resources
     */
    dispose(): void;
    private _disposeList;
    /**
     * Gets if the scene is already disposed
     */
    get isDisposed(): boolean;
    /**
     * Call this function to reduce memory footprint of the scene.
     * Vertex buffers will not store CPU data anymore (this will prevent picking, collisions or physics to work correctly)
     */
    clearCachedVertexData(): void;
    /**
     * This function will remove the local cached buffer data from texture.
     * It will save memory but will prevent the texture from being rebuilt
     */
    cleanCachedTextureBuffer(): void;
    /**
     * Get the world extend vectors with an optional filter
     *
     * @param filterPredicate the predicate - which meshes should be included when calculating the world size
     * @returns {{ min: Vector3; max: Vector3 }} min and max vectors
     */
    getWorldExtends(filterPredicate?: (mesh: AbstractMesh) => boolean): {
        min: Vector3;
        max: Vector3;
    };
    /**
     * Creates a ray that can be used to pick in the scene
     * @param x defines the x coordinate of the origin (on-screen)
     * @param y defines the y coordinate of the origin (on-screen)
     * @param world defines the world matrix to use if you want to pick in object space (instead of world space)
     * @param camera defines the camera to use for the picking
     * @param cameraViewSpace defines if picking will be done in view space (false by default)
     * @returns a Ray
     */
    createPickingRay(x: number, y: number, world: Nullable<Matrix>, camera: Nullable<Camera>, cameraViewSpace?: boolean): Ray;
    /**
     * Creates a ray that can be used to pick in the scene
     * @param x defines the x coordinate of the origin (on-screen)
     * @param y defines the y coordinate of the origin (on-screen)
     * @param world defines the world matrix to use if you want to pick in object space (instead of world space)
     * @param result defines the ray where to store the picking ray
     * @param camera defines the camera to use for the picking
     * @param cameraViewSpace defines if picking will be done in view space (false by default)
     * @param enableDistantPicking defines if picking should handle large values for mesh position/scaling (false by default)
     * @returns the current scene
     */
    createPickingRayToRef(x: number, y: number, world: Nullable<Matrix>, result: Ray, camera: Nullable<Camera>, cameraViewSpace?: boolean, enableDistantPicking?: boolean): Scene;
    /**
     * Creates a ray that can be used to pick in the scene
     * @param x defines the x coordinate of the origin (on-screen)
     * @param y defines the y coordinate of the origin (on-screen)
     * @param camera defines the camera to use for the picking
     * @returns a Ray
     */
    createPickingRayInCameraSpace(x: number, y: number, camera?: Camera): Ray;
    /**
     * Creates a ray that can be used to pick in the scene
     * @param x defines the x coordinate of the origin (on-screen)
     * @param y defines the y coordinate of the origin (on-screen)
     * @param result defines the ray where to store the picking ray
     * @param camera defines the camera to use for the picking
     * @returns the current scene
     */
    createPickingRayInCameraSpaceToRef(x: number, y: number, result: Ray, camera?: Camera): Scene;
    /** @internal */
    get _pickingAvailable(): boolean;
    /** @internal */
    _registeredActions: number;
    /** Launch a ray to try to pick a mesh in the scene
     * @param x position on screen
     * @param y position on screen
     * @param predicate Predicate function used to determine eligible meshes. Can be set to null. In this case, a mesh must be enabled, visible and with isPickable set to true
     * @param fastCheck defines if the first intersection will be used (and not the closest)
     * @param camera to use for computing the picking ray. Can be set to null. In this case, the scene.activeCamera will be used
     * @param trianglePredicate defines an optional predicate used to select faces when a mesh intersection is detected
     * @returns a PickingInfo
     */
    pick(x: number, y: number, predicate?: (mesh: AbstractMesh) => boolean, fastCheck?: boolean, camera?: Nullable<Camera>, trianglePredicate?: TrianglePickingPredicate): PickingInfo;
    /** Launch a ray to try to pick a mesh in the scene using only bounding information of the main mesh (not using submeshes)
     * @param x position on screen
     * @param y position on screen
     * @param predicate Predicate function used to determine eligible meshes. Can be set to null. In this case, a mesh must be enabled, visible and with isPickable set to true
     * @param fastCheck defines if the first intersection will be used (and not the closest)
     * @param camera to use for computing the picking ray. Can be set to null. In this case, the scene.activeCamera will be used
     * @returns a PickingInfo (Please note that some info will not be set like distance, bv, bu and everything that cannot be capture by only using bounding infos)
     */
    pickWithBoundingInfo(x: number, y: number, predicate?: (mesh: AbstractMesh) => boolean, fastCheck?: boolean, camera?: Nullable<Camera>): Nullable<PickingInfo>;
    /** Use the given ray to pick a mesh in the scene
     * @param ray The ray to use to pick meshes
     * @param predicate Predicate function used to determine eligible meshes. Can be set to null. In this case, a mesh must have isPickable set to true
     * @param fastCheck defines if the first intersection will be used (and not the closest)
     * @param trianglePredicate defines an optional predicate used to select faces when a mesh intersection is detected
     * @returns a PickingInfo
     */
    pickWithRay(ray: Ray, predicate?: (mesh: AbstractMesh) => boolean, fastCheck?: boolean, trianglePredicate?: TrianglePickingPredicate): Nullable<PickingInfo>;
    /**
     * Launch a ray to try to pick a mesh in the scene
     * @param x X position on screen
     * @param y Y position on screen
     * @param predicate Predicate function used to determine eligible meshes. Can be set to null. In this case, a mesh must be enabled, visible and with isPickable set to true
     * @param camera camera to use for computing the picking ray. Can be set to null. In this case, the scene.activeCamera will be used
     * @param trianglePredicate defines an optional predicate used to select faces when a mesh intersection is detected
     * @returns an array of PickingInfo
     */
    multiPick(x: number, y: number, predicate?: (mesh: AbstractMesh) => boolean, camera?: Camera, trianglePredicate?: TrianglePickingPredicate): Nullable<PickingInfo[]>;
    /**
     * Launch a ray to try to pick a mesh in the scene
     * @param ray Ray to use
     * @param predicate Predicate function used to determine eligible meshes. Can be set to null. In this case, a mesh must be enabled, visible and with isPickable set to true
     * @param trianglePredicate defines an optional predicate used to select faces when a mesh intersection is detected
     * @returns an array of PickingInfo
     */
    multiPickWithRay(ray: Ray, predicate?: (mesh: AbstractMesh) => boolean, trianglePredicate?: TrianglePickingPredicate): Nullable<PickingInfo[]>;
    /**
     * Force the value of meshUnderPointer
     * @param mesh defines the mesh to use
     * @param pointerId optional pointer id when using more than one pointer
     * @param pickResult optional pickingInfo data used to find mesh
     */
    setPointerOverMesh(mesh: Nullable<AbstractMesh>, pointerId?: number, pickResult?: Nullable<PickingInfo>): void;
    /**
     * Gets the mesh under the pointer
     * @returns a Mesh or null if no mesh is under the pointer
     */
    getPointerOverMesh(): Nullable<AbstractMesh>;
    /** @internal */
    _rebuildGeometries(): void;
    /** @internal */
    _rebuildTextures(): void;
    private _getByTags;
    /**
     * Get a list of meshes by tags
     * @param tagsQuery defines the tags query to use
     * @param forEach defines a predicate used to filter results
     * @returns an array of Mesh
     */
    getMeshesByTags(tagsQuery: string, forEach?: (mesh: AbstractMesh) => void): Mesh[];
    /**
     * Get a list of cameras by tags
     * @param tagsQuery defines the tags query to use
     * @param forEach defines a predicate used to filter results
     * @returns an array of Camera
     */
    getCamerasByTags(tagsQuery: string, forEach?: (camera: Camera) => void): Camera[];
    /**
     * Get a list of lights by tags
     * @param tagsQuery defines the tags query to use
     * @param forEach defines a predicate used to filter results
     * @returns an array of Light
     */
    getLightsByTags(tagsQuery: string, forEach?: (light: Light) => void): Light[];
    /**
     * Get a list of materials by tags
     * @param tagsQuery defines the tags query to use
     * @param forEach defines a predicate used to filter results
     * @returns an array of Material
     */
    getMaterialByTags(tagsQuery: string, forEach?: (material: Material) => void): Material[];
    /**
     * Get a list of transform nodes by tags
     * @param tagsQuery defines the tags query to use
     * @param forEach defines a predicate used to filter results
     * @returns an array of TransformNode
     */
    getTransformNodesByTags(tagsQuery: string, forEach?: (transform: TransformNode) => void): TransformNode[];
    /**
     * Overrides the default sort function applied in the rendering group to prepare the meshes.
     * This allowed control for front to back rendering or reversly depending of the special needs.
     *
     * @param renderingGroupId The rendering group id corresponding to its index
     * @param opaqueSortCompareFn The opaque queue comparison function use to sort.
     * @param alphaTestSortCompareFn The alpha test queue comparison function use to sort.
     * @param transparentSortCompareFn The transparent queue comparison function use to sort.
     */
    setRenderingOrder(renderingGroupId: number, opaqueSortCompareFn?: Nullable<(a: SubMesh, b: SubMesh) => number>, alphaTestSortCompareFn?: Nullable<(a: SubMesh, b: SubMesh) => number>, transparentSortCompareFn?: Nullable<(a: SubMesh, b: SubMesh) => number>): void;
    /**
     * Specifies whether or not the stencil and depth buffer are cleared between two rendering groups.
     *
     * @param renderingGroupId The rendering group id corresponding to its index
     * @param autoClearDepthStencil Automatically clears depth and stencil between groups if true.
     * @param depth Automatically clears depth between groups if true and autoClear is true.
     * @param stencil Automatically clears stencil between groups if true and autoClear is true.
     */
    setRenderingAutoClearDepthStencil(renderingGroupId: number, autoClearDepthStencil: boolean, depth?: boolean, stencil?: boolean): void;
    /**
     * Gets the current auto clear configuration for one rendering group of the rendering
     * manager.
     * @param index the rendering group index to get the information for
     * @returns The auto clear setup for the requested rendering group
     */
    getAutoClearDepthStencilSetup(index: number): IRenderingManagerAutoClearSetup;
    private _blockMaterialDirtyMechanism;
    /** Gets or sets a boolean blocking all the calls to markAllMaterialsAsDirty (ie. the materials won't be updated if they are out of sync) */
    get blockMaterialDirtyMechanism(): boolean;
    set blockMaterialDirtyMechanism(value: boolean);
    /**
     * Will flag all materials as dirty to trigger new shader compilation
     * @param flag defines the flag used to specify which material part must be marked as dirty
     * @param predicate If not null, it will be used to specify if a material has to be marked as dirty
     */
    markAllMaterialsAsDirty(flag: number, predicate?: (mat: Material) => boolean): void;
    /**
     * @internal
     */
    _loadFile(fileOrUrl: File | string, onSuccess: (data: string | ArrayBuffer, responseURL?: string) => void, onProgress?: (ev: ProgressEvent) => void, useOfflineSupport?: boolean, useArrayBuffer?: boolean, onError?: (request?: WebRequest, exception?: LoadFileError) => void, onOpened?: (request: WebRequest) => void): IFileRequest;
    /**
     * @internal
     */
    _loadFileAsync(fileOrUrl: File | string, onProgress?: (data: any) => void, useOfflineSupport?: boolean, useArrayBuffer?: boolean, onOpened?: (request: WebRequest) => void): Promise<string | ArrayBuffer>;
    /**
     * @internal
     */
    _requestFile(url: string, onSuccess: (data: string | ArrayBuffer, request?: WebRequest) => void, onProgress?: (ev: ProgressEvent) => void, useOfflineSupport?: boolean, useArrayBuffer?: boolean, onError?: (error: RequestFileError) => void, onOpened?: (request: WebRequest) => void): IFileRequest;
    /**
     * @internal
     */
    _requestFileAsync(url: string, onProgress?: (ev: ProgressEvent) => void, useOfflineSupport?: boolean, useArrayBuffer?: boolean, onOpened?: (request: WebRequest) => void): Promise<string | ArrayBuffer>;
    /**
     * @internal
     */
    _readFile(file: File, onSuccess: (data: string | ArrayBuffer) => void, onProgress?: (ev: ProgressEvent) => any, useArrayBuffer?: boolean, onError?: (error: ReadFileError) => void): IFileRequest;
    /**
     * @internal
     */
    _readFileAsync(file: File, onProgress?: (ev: ProgressEvent) => any, useArrayBuffer?: boolean): Promise<string | ArrayBuffer>;
    /**
     * Internal perfCollector instance used for sharing between inspector and playground.
     * Marked as protected to allow sharing between prototype extensions, but disallow access at toplevel.
     */
    protected _perfCollector: Nullable<PerformanceViewerCollector>;
    /**
     * This method gets the performance collector belonging to the scene, which is generally shared with the inspector.
     * @returns the perf collector belonging to the scene.
     */
    getPerfCollector(): PerformanceViewerCollector;
}
module "babylonjs/scene" {
    interface Scene {
        /**
         * Sets the active camera of the scene using its Id
         * @param id defines the camera's Id
         * @returns the new active camera or null if none found.
         * @deprecated Please use setActiveCameraById instead
         */
        setActiveCameraByID(id: string): Nullable<Camera>;
        /**
         * Get a material using its id
         * @param id defines the material's Id
         * @returns the material or null if none found.
         * @deprecated Please use getMaterialById instead
         */
        getMaterialByID(id: string): Nullable<Material>;
        /**
         * Gets a the last added material using a given id
         * @param id defines the material's Id
         * @returns the last material with the given id or null if none found.
         * @deprecated Please use getLastMaterialById instead
         */
        getLastMaterialByID(id: string): Nullable<Material>;
        /**
         * Get a texture using its unique id
         * @param uniqueId defines the texture's unique id
         * @returns the texture or null if none found.
         * @deprecated Please use getTextureByUniqueId instead
         */
        getTextureByUniqueID(uniqueId: number): Nullable<BaseTexture>;
        /**
         * Gets a camera using its Id
         * @param id defines the Id to look for
         * @returns the camera or null if not found
         * @deprecated Please use getCameraById instead
         */
        getCameraByID(id: string): Nullable<Camera>;
        /**
         * Gets a camera using its unique Id
         * @param uniqueId defines the unique Id to look for
         * @returns the camera or null if not found
         * @deprecated Please use getCameraByUniqueId instead
         */
        getCameraByUniqueID(uniqueId: number): Nullable<Camera>;
        /**
         * Gets a bone using its Id
         * @param id defines the bone's Id
         * @returns the bone or null if not found
         * @deprecated Please use getBoneById instead
         */
        getBoneByID(id: string): Nullable<Bone>;
        /**
         * Gets a light node using its Id
         * @param id defines the light's Id
         * @returns the light or null if none found.
         * @deprecated Please use getLightById instead
         */
        getLightByID(id: string): Nullable<Light>;
        /**
         * Gets a light node using its scene-generated unique Id
         * @param uniqueId defines the light's unique Id
         * @returns the light or null if none found.
         * @deprecated Please use getLightByUniqueId instead
         */
        getLightByUniqueID(uniqueId: number): Nullable<Light>;
        /**
         * Gets a particle system by Id
         * @param id defines the particle system Id
         * @returns the corresponding system or null if none found
         * @deprecated Please use getParticleSystemById instead
         */
        getParticleSystemByID(id: string): Nullable<IParticleSystem>;
        /**
         * Gets a geometry using its Id
         * @param id defines the geometry's Id
         * @returns the geometry or null if none found.
         * @deprecated Please use getGeometryById instead
         */
        getGeometryByID(id: string): Nullable<Geometry>;
        /**
         * Gets the first added mesh found of a given Id
         * @param id defines the Id to search for
         * @returns the mesh found or null if not found at all
         * @deprecated Please use getMeshById instead
         */
        getMeshByID(id: string): Nullable<AbstractMesh>;
        /**
         * Gets a mesh with its auto-generated unique Id
         * @param uniqueId defines the unique Id to search for
         * @returns the found mesh or null if not found at all.
         * @deprecated Please use getMeshByUniqueId instead
         */
        getMeshByUniqueID(uniqueId: number): Nullable<AbstractMesh>;
        /**
         * Gets a the last added mesh using a given Id
         * @param id defines the Id to search for
         * @returns the found mesh or null if not found at all.
         * @deprecated Please use getLastMeshById instead
         */
        getLastMeshByID(id: string): Nullable<AbstractMesh>;
        /**
         * Gets a list of meshes using their Id
         * @param id defines the Id to search for
         * @returns a list of meshes
         * @deprecated Please use getMeshesById instead
         */
        getMeshesByID(id: string): Array<AbstractMesh>;
        /**
         * Gets the first added transform node found of a given Id
         * @param id defines the Id to search for
         * @returns the found transform node or null if not found at all.
         * @deprecated Please use getTransformNodeById instead
         */
        getTransformNodeByID(id: string): Nullable<TransformNode>;
        /**
         * Gets a transform node with its auto-generated unique Id
         * @param uniqueId defines the unique Id to search for
         * @returns the found transform node or null if not found at all.
         * @deprecated Please use getTransformNodeByUniqueId instead
         */
        getTransformNodeByUniqueID(uniqueId: number): Nullable<TransformNode>;
        /**
         * Gets a list of transform nodes using their Id
         * @param id defines the Id to search for
         * @returns a list of transform nodes
         * @deprecated Please use getTransformNodesById instead
         */
        getTransformNodesByID(id: string): Array<TransformNode>;
        /**
         * Gets a node (Mesh, Camera, Light) using a given Id
         * @param id defines the Id to search for
         * @returns the found node or null if not found at all
         * @deprecated Please use getNodeById instead
         */
        getNodeByID(id: string): Nullable<Node>;
        /**
         * Gets a the last added node (Mesh, Camera, Light) using a given Id
         * @param id defines the Id to search for
         * @returns the found node or null if not found at all
         * @deprecated Please use getLastEntryById instead
         */
        getLastEntryByID(id: string): Nullable<Node>;
        /**
         * Gets a skeleton using a given Id (if many are found, this function will pick the last one)
         * @param id defines the Id to search for
         * @returns the found skeleton or null if not found at all.
         * @deprecated Please use getLastSkeletonById instead
         */
        getLastSkeletonByID(id: string): Nullable<Skeleton>;
    }
}
export {};

}
declare module "babylonjs/sceneComponent" {
import { Scene } from "babylonjs/scene";
import { SmartArrayNoDuplicate } from "babylonjs/Misc/smartArray";
import { Nullable } from "babylonjs/types";
import { PickingInfo } from "babylonjs/Collisions/pickingInfo";
import { AbstractScene } from "babylonjs/abstractScene";
import { IPointerEvent } from "babylonjs/Events/deviceInputEvents";
import { Mesh } from "babylonjs/Meshes/mesh";
import { Effect } from "babylonjs/Materials/effect";
import { Camera } from "babylonjs/Cameras/camera";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { SubMesh } from "babylonjs/Meshes/subMesh";
import { RenderTargetTexture } from "babylonjs/Materials/Textures/renderTargetTexture";
/**
 * Groups all the scene component constants in one place to ease maintenance.
 * @internal
 */
export class SceneComponentConstants {
    static readonly NAME_EFFECTLAYER: string;
    static readonly NAME_LAYER: string;
    static readonly NAME_LENSFLARESYSTEM: string;
    static readonly NAME_BOUNDINGBOXRENDERER: string;
    static readonly NAME_PARTICLESYSTEM: string;
    static readonly NAME_GAMEPAD: string;
    static readonly NAME_SIMPLIFICATIONQUEUE: string;
    static readonly NAME_GEOMETRYBUFFERRENDERER: string;
    static readonly NAME_PREPASSRENDERER: string;
    static readonly NAME_DEPTHRENDERER: string;
    static readonly NAME_DEPTHPEELINGRENDERER: string;
    static readonly NAME_POSTPROCESSRENDERPIPELINEMANAGER: string;
    static readonly NAME_SPRITE: string;
    static readonly NAME_SUBSURFACE: string;
    static readonly NAME_OUTLINERENDERER: string;
    static readonly NAME_PROCEDURALTEXTURE: string;
    static readonly NAME_SHADOWGENERATOR: string;
    static readonly NAME_OCTREE: string;
    static readonly NAME_PHYSICSENGINE: string;
    static readonly NAME_AUDIO: string;
    static readonly NAME_FLUIDRENDERER: string;
    static readonly STEP_ISREADYFORMESH_EFFECTLAYER: number;
    static readonly STEP_BEFOREEVALUATEACTIVEMESH_BOUNDINGBOXRENDERER: number;
    static readonly STEP_EVALUATESUBMESH_BOUNDINGBOXRENDERER: number;
    static readonly STEP_PREACTIVEMESH_BOUNDINGBOXRENDERER: number;
    static readonly STEP_CAMERADRAWRENDERTARGET_EFFECTLAYER: number;
    static readonly STEP_BEFORECAMERADRAW_PREPASS: number;
    static readonly STEP_BEFORECAMERADRAW_EFFECTLAYER: number;
    static readonly STEP_BEFORECAMERADRAW_LAYER: number;
    static readonly STEP_BEFORERENDERTARGETDRAW_PREPASS: number;
    static readonly STEP_BEFORERENDERTARGETDRAW_LAYER: number;
    static readonly STEP_BEFORERENDERINGMESH_PREPASS: number;
    static readonly STEP_BEFORERENDERINGMESH_OUTLINE: number;
    static readonly STEP_AFTERRENDERINGMESH_PREPASS: number;
    static readonly STEP_AFTERRENDERINGMESH_OUTLINE: number;
    static readonly STEP_AFTERRENDERINGGROUPDRAW_EFFECTLAYER_DRAW: number;
    static readonly STEP_AFTERRENDERINGGROUPDRAW_BOUNDINGBOXRENDERER: number;
    static readonly STEP_BEFORECAMERAUPDATE_SIMPLIFICATIONQUEUE: number;
    static readonly STEP_BEFORECAMERAUPDATE_GAMEPAD: number;
    static readonly STEP_BEFORECLEAR_PROCEDURALTEXTURE: number;
    static readonly STEP_BEFORECLEAR_PREPASS: number;
    static readonly STEP_BEFORERENDERTARGETCLEAR_PREPASS: number;
    static readonly STEP_AFTERRENDERTARGETDRAW_PREPASS: number;
    static readonly STEP_AFTERRENDERTARGETDRAW_LAYER: number;
    static readonly STEP_AFTERCAMERADRAW_PREPASS: number;
    static readonly STEP_AFTERCAMERADRAW_EFFECTLAYER: number;
    static readonly STEP_AFTERCAMERADRAW_LENSFLARESYSTEM: number;
    static readonly STEP_AFTERCAMERADRAW_EFFECTLAYER_DRAW: number;
    static readonly STEP_AFTERCAMERADRAW_LAYER: number;
    static readonly STEP_AFTERCAMERADRAW_FLUIDRENDERER: number;
    static readonly STEP_AFTERCAMERAPOSTPROCESS_LAYER: number;
    static readonly STEP_AFTERRENDERTARGETPOSTPROCESS_LAYER: number;
    static readonly STEP_AFTERRENDER_AUDIO: number;
    static readonly STEP_GATHERRENDERTARGETS_DEPTHRENDERER: number;
    static readonly STEP_GATHERRENDERTARGETS_GEOMETRYBUFFERRENDERER: number;
    static readonly STEP_GATHERRENDERTARGETS_SHADOWGENERATOR: number;
    static readonly STEP_GATHERRENDERTARGETS_POSTPROCESSRENDERPIPELINEMANAGER: number;
    static readonly STEP_GATHERACTIVECAMERARENDERTARGETS_DEPTHRENDERER: number;
    static readonly STEP_GATHERACTIVECAMERARENDERTARGETS_FLUIDRENDERER: number;
    static readonly STEP_POINTERMOVE_SPRITE: number;
    static readonly STEP_POINTERDOWN_SPRITE: number;
    static readonly STEP_POINTERUP_SPRITE: number;
}
/**
 * This represents a scene component.
 *
 * This is used to decouple the dependency the scene is having on the different workloads like
 * layers, post processes...
 */
export interface ISceneComponent {
    /**
     * The name of the component. Each component must have a unique name.
     */
    name: string;
    /**
     * The scene the component belongs to.
     */
    scene: Scene;
    /**
     * Register the component to one instance of a scene.
     */
    register(): void;
    /**
     * Rebuilds the elements related to this component in case of
     * context lost for instance.
     */
    rebuild(): void;
    /**
     * Disposes the component and the associated ressources.
     */
    dispose(): void;
}
/**
 * This represents a SERIALIZABLE scene component.
 *
 * This extends Scene Component to add Serialization methods on top.
 */
export interface ISceneSerializableComponent extends ISceneComponent {
    /**
     * Adds all the elements from the container to the scene
     * @param container the container holding the elements
     */
    addFromContainer(container: AbstractScene): void;
    /**
     * Removes all the elements in the container from the scene
     * @param container contains the elements to remove
     * @param dispose if the removed element should be disposed (default: false)
     */
    removeFromContainer(container: AbstractScene, dispose?: boolean): void;
    /**
     * Serializes the component data to the specified json object
     * @param serializationObject The object to serialize to
     */
    serialize(serializationObject: any): void;
}
/**
 * Strong typing of a Mesh related stage step action
 */
export type MeshStageAction = (mesh: AbstractMesh, hardwareInstancedRendering: boolean) => boolean;
/**
 * Strong typing of a Evaluate Sub Mesh related stage step action
 */
export type EvaluateSubMeshStageAction = (mesh: AbstractMesh, subMesh: SubMesh) => void;
/**
 * Strong typing of a pre active Mesh related stage step action
 */
export type PreActiveMeshStageAction = (mesh: AbstractMesh) => void;
/**
 * Strong typing of a Camera related stage step action
 */
export type CameraStageAction = (camera: Camera) => void;
/**
 * Strong typing of a Camera Frame buffer related stage step action
 */
export type CameraStageFrameBufferAction = (camera: Camera) => boolean;
/**
 * Strong typing of a Render Target related stage step action
 */
export type RenderTargetStageAction = (renderTarget: RenderTargetTexture, faceIndex?: number, layer?: number) => void;
/**
 * Strong typing of a RenderingGroup related stage step action
 */
export type RenderingGroupStageAction = (renderingGroupId: number) => void;
/**
 * Strong typing of a Mesh Render related stage step action
 */
export type RenderingMeshStageAction = (mesh: Mesh, subMesh: SubMesh, batch: any, effect: Nullable<Effect>) => void;
/**
 * Strong typing of a simple stage step action
 */
export type SimpleStageAction = () => void;
/**
 * Strong typing of a render target action.
 */
export type RenderTargetsStageAction = (renderTargets: SmartArrayNoDuplicate<RenderTargetTexture>) => void;
/**
 * Strong typing of a pointer move action.
 */
export type PointerMoveStageAction = (unTranslatedPointerX: number, unTranslatedPointerY: number, pickResult: Nullable<PickingInfo>, isMeshPicked: boolean, element: Nullable<HTMLElement>) => Nullable<PickingInfo>;
/**
 * Strong typing of a pointer up/down action.
 */
export type PointerUpDownStageAction = (unTranslatedPointerX: number, unTranslatedPointerY: number, pickResult: Nullable<PickingInfo>, evt: IPointerEvent, doubleClick: boolean) => Nullable<PickingInfo>;
/**
 * Representation of a stage in the scene (Basically a list of ordered steps)
 * @internal
 */
export class Stage<T extends Function> extends Array<{
    index: number;
    component: ISceneComponent;
    action: T;
}> {
    /**
     * Hide ctor from the rest of the world.
     * @param items The items to add.
     */
    private constructor();
    /**
     * Creates a new Stage.
     * @returns A new instance of a Stage
     */
    static Create<T extends Function>(): Stage<T>;
    /**
     * Registers a step in an ordered way in the targeted stage.
     * @param index Defines the position to register the step in
     * @param component Defines the component attached to the step
     * @param action Defines the action to launch during the step
     */
    registerStep(index: number, component: ISceneComponent, action: T): void;
    /**
     * Clears all the steps from the stage.
     */
    clear(): void;
}
export {};

}
declare module "babylonjs/Shaders/anaglyph.fragment" {
/** @internal */
export const anaglyphPixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/background.fragment" {
import "babylonjs/Shaders/ShadersInclude/backgroundFragmentDeclaration";
import "babylonjs/Shaders/ShadersInclude/backgroundUboDeclaration";
import "babylonjs/Shaders/ShadersInclude/helperFunctions";
import "babylonjs/Shaders/ShadersInclude/reflectionFunction";
import "babylonjs/Shaders/ShadersInclude/imageProcessingDeclaration";
import "babylonjs/Shaders/ShadersInclude/lightFragmentDeclaration";
import "babylonjs/Shaders/ShadersInclude/lightUboDeclaration";
import "babylonjs/Shaders/ShadersInclude/lightsFragmentFunctions";
import "babylonjs/Shaders/ShadersInclude/shadowsFragmentFunctions";
import "babylonjs/Shaders/ShadersInclude/imageProcessingFunctions";
import "babylonjs/Shaders/ShadersInclude/clipPlaneFragmentDeclaration";
import "babylonjs/Shaders/ShadersInclude/fogFragmentDeclaration";
import "babylonjs/Shaders/ShadersInclude/clipPlaneFragment";
import "babylonjs/Shaders/ShadersInclude/lightFragment";
import "babylonjs/Shaders/ShadersInclude/fogFragment";
/** @internal */
export const backgroundPixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/background.vertex" {
import "babylonjs/Shaders/ShadersInclude/backgroundVertexDeclaration";
import "babylonjs/Shaders/ShadersInclude/backgroundUboDeclaration";
import "babylonjs/Shaders/ShadersInclude/helperFunctions";
import "babylonjs/Shaders/ShadersInclude/bonesDeclaration";
import "babylonjs/Shaders/ShadersInclude/bakedVertexAnimationDeclaration";
import "babylonjs/Shaders/ShadersInclude/instancesDeclaration";
import "babylonjs/Shaders/ShadersInclude/clipPlaneVertexDeclaration";
import "babylonjs/Shaders/ShadersInclude/fogVertexDeclaration";
import "babylonjs/Shaders/ShadersInclude/lightVxFragmentDeclaration";
import "babylonjs/Shaders/ShadersInclude/lightVxUboDeclaration";
import "babylonjs/Shaders/ShadersInclude/instancesVertex";
import "babylonjs/Shaders/ShadersInclude/bonesVertex";
import "babylonjs/Shaders/ShadersInclude/bakedVertexAnimation";
import "babylonjs/Shaders/ShadersInclude/clipPlaneVertex";
import "babylonjs/Shaders/ShadersInclude/fogVertex";
import "babylonjs/Shaders/ShadersInclude/shadowsVertex";
/** @internal */
export const backgroundVertexShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/blackAndWhite.fragment" {
/** @internal */
export const blackAndWhitePixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/bloomMerge.fragment" {
/** @internal */
export const bloomMergePixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/blur.fragment" {
/** @internal */
export const blurPixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/boundingBoxRenderer.fragment" {
import "babylonjs/Shaders/ShadersInclude/boundingBoxRendererFragmentDeclaration";
import "babylonjs/Shaders/ShadersInclude/boundingBoxRendererUboDeclaration";
/** @internal */
export const boundingBoxRendererPixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/boundingBoxRenderer.vertex" {
import "babylonjs/Shaders/ShadersInclude/boundingBoxRendererVertexDeclaration";
import "babylonjs/Shaders/ShadersInclude/boundingBoxRendererUboDeclaration";
/** @internal */
export const boundingBoxRendererVertexShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/chromaticAberration.fragment" {
/** @internal */
export const chromaticAberrationPixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/circleOfConfusion.fragment" {
/** @internal */
export const circleOfConfusionPixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/clearQuad.fragment" {
/** @internal */
export const clearQuadPixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/clearQuad.vertex" {
/** @internal */
export const clearQuadVertexShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/color.fragment" {
import "babylonjs/Shaders/ShadersInclude/clipPlaneFragmentDeclaration";
import "babylonjs/Shaders/ShadersInclude/clipPlaneFragment";
/** @internal */
export const colorPixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/color.vertex" {
import "babylonjs/Shaders/ShadersInclude/bonesDeclaration";
import "babylonjs/Shaders/ShadersInclude/bakedVertexAnimationDeclaration";
import "babylonjs/Shaders/ShadersInclude/clipPlaneVertexDeclaration";
import "babylonjs/Shaders/ShadersInclude/instancesDeclaration";
import "babylonjs/Shaders/ShadersInclude/instancesVertex";
import "babylonjs/Shaders/ShadersInclude/bonesVertex";
import "babylonjs/Shaders/ShadersInclude/bakedVertexAnimation";
import "babylonjs/Shaders/ShadersInclude/clipPlaneVertex";
import "babylonjs/Shaders/ShadersInclude/vertexColorMixing";
/** @internal */
export const colorVertexShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/colorCorrection.fragment" {
/** @internal */
export const colorCorrectionPixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/convolution.fragment" {
/** @internal */
export const convolutionPixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/copyTextureToTexture.fragment" {
import "babylonjs/Shaders/ShadersInclude/helperFunctions";
/** @internal */
export const copyTextureToTexturePixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/default.fragment" {
import "babylonjs/Shaders/ShadersInclude/defaultFragmentDeclaration";
import "babylonjs/Shaders/ShadersInclude/defaultUboDeclaration";
import "babylonjs/Shaders/ShadersInclude/prePassDeclaration";
import "babylonjs/Shaders/ShadersInclude/oitDeclaration";
import "babylonjs/Shaders/ShadersInclude/mainUVVaryingDeclaration";
import "babylonjs/Shaders/ShadersInclude/helperFunctions";
import "babylonjs/Shaders/ShadersInclude/lightFragmentDeclaration";
import "babylonjs/Shaders/ShadersInclude/lightUboDeclaration";
import "babylonjs/Shaders/ShadersInclude/lightsFragmentFunctions";
import "babylonjs/Shaders/ShadersInclude/shadowsFragmentFunctions";
import "babylonjs/Shaders/ShadersInclude/samplerFragmentDeclaration";
import "babylonjs/Shaders/ShadersInclude/fresnelFunction";
import "babylonjs/Shaders/ShadersInclude/reflectionFunction";
import "babylonjs/Shaders/ShadersInclude/imageProcessingDeclaration";
import "babylonjs/Shaders/ShadersInclude/imageProcessingFunctions";
import "babylonjs/Shaders/ShadersInclude/bumpFragmentMainFunctions";
import "babylonjs/Shaders/ShadersInclude/bumpFragmentFunctions";
import "babylonjs/Shaders/ShadersInclude/clipPlaneFragmentDeclaration";
import "babylonjs/Shaders/ShadersInclude/logDepthDeclaration";
import "babylonjs/Shaders/ShadersInclude/fogFragmentDeclaration";
import "babylonjs/Shaders/ShadersInclude/clipPlaneFragment";
import "babylonjs/Shaders/ShadersInclude/bumpFragment";
import "babylonjs/Shaders/ShadersInclude/depthPrePass";
import "babylonjs/Shaders/ShadersInclude/lightFragment";
import "babylonjs/Shaders/ShadersInclude/logDepthFragment";
import "babylonjs/Shaders/ShadersInclude/fogFragment";
import "babylonjs/Shaders/ShadersInclude/oitFragment";
/** @internal */
export const defaultPixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/default.vertex" {
import "babylonjs/Shaders/ShadersInclude/defaultVertexDeclaration";
import "babylonjs/Shaders/ShadersInclude/defaultUboDeclaration";
import "babylonjs/Shaders/ShadersInclude/uvAttributeDeclaration";
import "babylonjs/Shaders/ShadersInclude/helperFunctions";
import "babylonjs/Shaders/ShadersInclude/bonesDeclaration";
import "babylonjs/Shaders/ShadersInclude/bakedVertexAnimationDeclaration";
import "babylonjs/Shaders/ShadersInclude/instancesDeclaration";
import "babylonjs/Shaders/ShadersInclude/prePassVertexDeclaration";
import "babylonjs/Shaders/ShadersInclude/mainUVVaryingDeclaration";
import "babylonjs/Shaders/ShadersInclude/samplerVertexDeclaration";
import "babylonjs/Shaders/ShadersInclude/bumpVertexDeclaration";
import "babylonjs/Shaders/ShadersInclude/clipPlaneVertexDeclaration";
import "babylonjs/Shaders/ShadersInclude/fogVertexDeclaration";
import "babylonjs/Shaders/ShadersInclude/lightVxFragmentDeclaration";
import "babylonjs/Shaders/ShadersInclude/lightVxUboDeclaration";
import "babylonjs/Shaders/ShadersInclude/morphTargetsVertexGlobalDeclaration";
import "babylonjs/Shaders/ShadersInclude/morphTargetsVertexDeclaration";
import "babylonjs/Shaders/ShadersInclude/logDepthDeclaration";
import "babylonjs/Shaders/ShadersInclude/morphTargetsVertexGlobal";
import "babylonjs/Shaders/ShadersInclude/morphTargetsVertex";
import "babylonjs/Shaders/ShadersInclude/instancesVertex";
import "babylonjs/Shaders/ShadersInclude/bonesVertex";
import "babylonjs/Shaders/ShadersInclude/bakedVertexAnimation";
import "babylonjs/Shaders/ShadersInclude/prePassVertex";
import "babylonjs/Shaders/ShadersInclude/uvVariableDeclaration";
import "babylonjs/Shaders/ShadersInclude/samplerVertexImplementation";
import "babylonjs/Shaders/ShadersInclude/bumpVertex";
import "babylonjs/Shaders/ShadersInclude/clipPlaneVertex";
import "babylonjs/Shaders/ShadersInclude/fogVertex";
import "babylonjs/Shaders/ShadersInclude/shadowsVertex";
import "babylonjs/Shaders/ShadersInclude/vertexColorMixing";
import "babylonjs/Shaders/ShadersInclude/pointCloudVertex";
import "babylonjs/Shaders/ShadersInclude/logDepthVertex";
/** @internal */
export const defaultVertexShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/depth.fragment" {
import "babylonjs/Shaders/ShadersInclude/clipPlaneFragmentDeclaration";
import "babylonjs/Shaders/ShadersInclude/packingFunctions";
import "babylonjs/Shaders/ShadersInclude/clipPlaneFragment";
/** @internal */
export const depthPixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/depth.vertex" {
import "babylonjs/Shaders/ShadersInclude/bonesDeclaration";
import "babylonjs/Shaders/ShadersInclude/bakedVertexAnimationDeclaration";
import "babylonjs/Shaders/ShadersInclude/morphTargetsVertexGlobalDeclaration";
import "babylonjs/Shaders/ShadersInclude/morphTargetsVertexDeclaration";
import "babylonjs/Shaders/ShadersInclude/clipPlaneVertexDeclaration";
import "babylonjs/Shaders/ShadersInclude/instancesDeclaration";
import "babylonjs/Shaders/ShadersInclude/morphTargetsVertexGlobal";
import "babylonjs/Shaders/ShadersInclude/morphTargetsVertex";
import "babylonjs/Shaders/ShadersInclude/instancesVertex";
import "babylonjs/Shaders/ShadersInclude/bonesVertex";
import "babylonjs/Shaders/ShadersInclude/bakedVertexAnimation";
import "babylonjs/Shaders/ShadersInclude/clipPlaneVertex";
/** @internal */
export const depthVertexShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/depthBoxBlur.fragment" {
/** @internal */
export const depthBoxBlurPixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/depthOfField.fragment" {
/** @internal */
export const depthOfFieldPixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/depthOfFieldMerge.fragment" {
/** @internal */
export const depthOfFieldMergePixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/displayPass.fragment" {
/** @internal */
export const displayPassPixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/extractHighlights.fragment" {
import "babylonjs/Shaders/ShadersInclude/helperFunctions";
/** @internal */
export const extractHighlightsPixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/filter.fragment" {
/** @internal */
export const filterPixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/fluidRenderingBilateralBlur.fragment" {
/** @internal */
export const fluidRenderingBilateralBlurPixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/fluidRenderingParticleDepth.fragment" {
/** @internal */
export const fluidRenderingParticleDepthPixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/fluidRenderingParticleDepth.vertex" {
/** @internal */
export const fluidRenderingParticleDepthVertexShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/fluidRenderingParticleDiffuse.fragment" {
/** @internal */
export const fluidRenderingParticleDiffusePixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/fluidRenderingParticleDiffuse.vertex" {
/** @internal */
export const fluidRenderingParticleDiffuseVertexShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/fluidRenderingParticleThickness.fragment" {
/** @internal */
export const fluidRenderingParticleThicknessPixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/fluidRenderingParticleThickness.vertex" {
/** @internal */
export const fluidRenderingParticleThicknessVertexShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/fluidRenderingRender.fragment" {
/** @internal */
export const fluidRenderingRenderPixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/fluidRenderingStandardBlur.fragment" {
/** @internal */
export const fluidRenderingStandardBlurPixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/fxaa.fragment" {
/** @internal */
export const fxaaPixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/fxaa.vertex" {
/** @internal */
export const fxaaVertexShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/geometry.fragment" {
import "babylonjs/Shaders/ShadersInclude/mrtFragmentDeclaration";
import "babylonjs/Shaders/ShadersInclude/bumpFragmentMainFunctions";
import "babylonjs/Shaders/ShadersInclude/bumpFragmentFunctions";
import "babylonjs/Shaders/ShadersInclude/helperFunctions";
import "babylonjs/Shaders/ShadersInclude/bumpFragment";
/** @internal */
export const geometryPixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/geometry.vertex" {
import "babylonjs/Shaders/ShadersInclude/bonesDeclaration";
import "babylonjs/Shaders/ShadersInclude/bakedVertexAnimationDeclaration";
import "babylonjs/Shaders/ShadersInclude/morphTargetsVertexGlobalDeclaration";
import "babylonjs/Shaders/ShadersInclude/morphTargetsVertexDeclaration";
import "babylonjs/Shaders/ShadersInclude/instancesDeclaration";
import "babylonjs/Shaders/ShadersInclude/geometryVertexDeclaration";
import "babylonjs/Shaders/ShadersInclude/geometryUboDeclaration";
import "babylonjs/Shaders/ShadersInclude/morphTargetsVertexGlobal";
import "babylonjs/Shaders/ShadersInclude/morphTargetsVertex";
import "babylonjs/Shaders/ShadersInclude/instancesVertex";
import "babylonjs/Shaders/ShadersInclude/bonesVertex";
import "babylonjs/Shaders/ShadersInclude/bakedVertexAnimation";
import "babylonjs/Shaders/ShadersInclude/bumpVertex";
/** @internal */
export const geometryVertexShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/glowBlurPostProcess.fragment" {
/** @internal */
export const glowBlurPostProcessPixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/glowMapGeneration.fragment" {
import "babylonjs/Shaders/ShadersInclude/helperFunctions";
import "babylonjs/Shaders/ShadersInclude/clipPlaneFragmentDeclaration";
import "babylonjs/Shaders/ShadersInclude/clipPlaneFragment";
/** @internal */
export const glowMapGenerationPixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/glowMapGeneration.vertex" {
import "babylonjs/Shaders/ShadersInclude/bonesDeclaration";
import "babylonjs/Shaders/ShadersInclude/bakedVertexAnimationDeclaration";
import "babylonjs/Shaders/ShadersInclude/morphTargetsVertexGlobalDeclaration";
import "babylonjs/Shaders/ShadersInclude/morphTargetsVertexDeclaration";
import "babylonjs/Shaders/ShadersInclude/clipPlaneVertexDeclaration";
import "babylonjs/Shaders/ShadersInclude/instancesDeclaration";
import "babylonjs/Shaders/ShadersInclude/morphTargetsVertexGlobal";
import "babylonjs/Shaders/ShadersInclude/morphTargetsVertex";
import "babylonjs/Shaders/ShadersInclude/instancesVertex";
import "babylonjs/Shaders/ShadersInclude/bonesVertex";
import "babylonjs/Shaders/ShadersInclude/bakedVertexAnimation";
import "babylonjs/Shaders/ShadersInclude/clipPlaneVertex";
/** @internal */
export const glowMapGenerationVertexShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/glowMapMerge.fragment" {
/** @internal */
export const glowMapMergePixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/glowMapMerge.vertex" {
/** @internal */
export const glowMapMergeVertexShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/gpuRenderParticles.fragment" {
import "babylonjs/Shaders/ShadersInclude/clipPlaneFragmentDeclaration2";
import "babylonjs/Shaders/ShadersInclude/imageProcessingDeclaration";
import "babylonjs/Shaders/ShadersInclude/logDepthDeclaration";
import "babylonjs/Shaders/ShadersInclude/helperFunctions";
import "babylonjs/Shaders/ShadersInclude/imageProcessingFunctions";
import "babylonjs/Shaders/ShadersInclude/clipPlaneFragment";
import "babylonjs/Shaders/ShadersInclude/logDepthFragment";
/** @internal */
export const gpuRenderParticlesPixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/gpuRenderParticles.vertex" {
import "babylonjs/Shaders/ShadersInclude/clipPlaneVertexDeclaration2";
import "babylonjs/Shaders/ShadersInclude/logDepthDeclaration";
import "babylonjs/Shaders/ShadersInclude/clipPlaneVertex";
import "babylonjs/Shaders/ShadersInclude/logDepthVertex";
/** @internal */
export const gpuRenderParticlesVertexShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/gpuUpdateParticles.fragment" {
/** @internal */
export const gpuUpdateParticlesPixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/gpuUpdateParticles.vertex" {
/** @internal */
export const gpuUpdateParticlesVertexShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/grain.fragment" {
import "babylonjs/Shaders/ShadersInclude/helperFunctions";
/** @internal */
export const grainPixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/hdrFiltering.fragment" {
import "babylonjs/Shaders/ShadersInclude/helperFunctions";
import "babylonjs/Shaders/ShadersInclude/importanceSampling";
import "babylonjs/Shaders/ShadersInclude/pbrBRDFFunctions";
import "babylonjs/Shaders/ShadersInclude/hdrFilteringFunctions";
/** @internal */
export const hdrFilteringPixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/hdrFiltering.vertex" {
/** @internal */
export const hdrFilteringVertexShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/highlights.fragment" {
/** @internal */
export const highlightsPixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/imageProcessing.fragment" {
import "babylonjs/Shaders/ShadersInclude/imageProcessingDeclaration";
import "babylonjs/Shaders/ShadersInclude/helperFunctions";
import "babylonjs/Shaders/ShadersInclude/imageProcessingFunctions";
/** @internal */
export const imageProcessingPixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/kernelBlur.fragment" {
import "babylonjs/Shaders/ShadersInclude/kernelBlurVaryingDeclaration";
import "babylonjs/Shaders/ShadersInclude/packingFunctions";
import "babylonjs/Shaders/ShadersInclude/kernelBlurFragment";
import "babylonjs/Shaders/ShadersInclude/kernelBlurFragment2";
/** @internal */
export const kernelBlurPixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/kernelBlur.vertex" {
import "babylonjs/Shaders/ShadersInclude/kernelBlurVaryingDeclaration";
import "babylonjs/Shaders/ShadersInclude/kernelBlurVertex";
/** @internal */
export const kernelBlurVertexShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/layer.fragment" {
import "babylonjs/Shaders/ShadersInclude/helperFunctions";
/** @internal */
export const layerPixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/layer.vertex" {
/** @internal */
export const layerVertexShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/lensFlare.fragment" {
/** @internal */
export const lensFlarePixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/lensFlare.vertex" {
/** @internal */
export const lensFlareVertexShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/lensHighlights.fragment" {
/** @internal */
export const lensHighlightsPixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/line.fragment" {
import "babylonjs/Shaders/ShadersInclude/clipPlaneFragmentDeclaration";
import "babylonjs/Shaders/ShadersInclude/clipPlaneFragment";
/** @internal */
export const linePixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/line.vertex" {
import "babylonjs/Shaders/ShadersInclude/instancesDeclaration";
import "babylonjs/Shaders/ShadersInclude/clipPlaneVertexDeclaration";
import "babylonjs/Shaders/ShadersInclude/instancesVertex";
import "babylonjs/Shaders/ShadersInclude/clipPlaneVertex";
/** @internal */
export const lineVertexShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/minmaxRedux.fragment" {
/** @internal */
export const minmaxReduxPixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/motionBlur.fragment" {
/** @internal */
export const motionBlurPixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/noise.fragment" {
/** @internal */
export const noisePixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/oitBackBlend.fragment" {
/** @internal */
export const oitBackBlendPixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/oitFinal.fragment" {
/** @internal */
export const oitFinalPixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/outline.fragment" {
import "babylonjs/Shaders/ShadersInclude/clipPlaneFragmentDeclaration";
import "babylonjs/Shaders/ShadersInclude/logDepthDeclaration";
import "babylonjs/Shaders/ShadersInclude/clipPlaneFragment";
import "babylonjs/Shaders/ShadersInclude/logDepthFragment";
/** @internal */
export const outlinePixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/outline.vertex" {
import "babylonjs/Shaders/ShadersInclude/bonesDeclaration";
import "babylonjs/Shaders/ShadersInclude/bakedVertexAnimationDeclaration";
import "babylonjs/Shaders/ShadersInclude/morphTargetsVertexGlobalDeclaration";
import "babylonjs/Shaders/ShadersInclude/morphTargetsVertexDeclaration";
import "babylonjs/Shaders/ShadersInclude/clipPlaneVertexDeclaration";
import "babylonjs/Shaders/ShadersInclude/instancesDeclaration";
import "babylonjs/Shaders/ShadersInclude/logDepthDeclaration";
import "babylonjs/Shaders/ShadersInclude/morphTargetsVertexGlobal";
import "babylonjs/Shaders/ShadersInclude/morphTargetsVertex";
import "babylonjs/Shaders/ShadersInclude/instancesVertex";
import "babylonjs/Shaders/ShadersInclude/bonesVertex";
import "babylonjs/Shaders/ShadersInclude/bakedVertexAnimation";
import "babylonjs/Shaders/ShadersInclude/clipPlaneVertex";
import "babylonjs/Shaders/ShadersInclude/logDepthVertex";
/** @internal */
export const outlineVertexShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/particles.fragment" {
import "babylonjs/Shaders/ShadersInclude/clipPlaneFragmentDeclaration";
import "babylonjs/Shaders/ShadersInclude/imageProcessingDeclaration";
import "babylonjs/Shaders/ShadersInclude/logDepthDeclaration";
import "babylonjs/Shaders/ShadersInclude/helperFunctions";
import "babylonjs/Shaders/ShadersInclude/imageProcessingFunctions";
import "babylonjs/Shaders/ShadersInclude/clipPlaneFragment";
import "babylonjs/Shaders/ShadersInclude/logDepthFragment";
/** @internal */
export const particlesPixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/particles.vertex" {
import "babylonjs/Shaders/ShadersInclude/clipPlaneVertexDeclaration";
import "babylonjs/Shaders/ShadersInclude/logDepthDeclaration";
import "babylonjs/Shaders/ShadersInclude/clipPlaneVertex";
import "babylonjs/Shaders/ShadersInclude/logDepthVertex";
/** @internal */
export const particlesVertexShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/pass.fragment" {
/** @internal */
export const passPixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/passCube.fragment" {
/** @internal */
export const passCubePixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/pbr.fragment" {
import "babylonjs/Shaders/ShadersInclude/prePassDeclaration";
import "babylonjs/Shaders/ShadersInclude/oitDeclaration";
import "babylonjs/Shaders/ShadersInclude/pbrFragmentDeclaration";
import "babylonjs/Shaders/ShadersInclude/pbrUboDeclaration";
import "babylonjs/Shaders/ShadersInclude/pbrFragmentExtraDeclaration";
import "babylonjs/Shaders/ShadersInclude/lightFragmentDeclaration";
import "babylonjs/Shaders/ShadersInclude/lightUboDeclaration";
import "babylonjs/Shaders/ShadersInclude/pbrFragmentSamplersDeclaration";
import "babylonjs/Shaders/ShadersInclude/imageProcessingDeclaration";
import "babylonjs/Shaders/ShadersInclude/clipPlaneFragmentDeclaration";
import "babylonjs/Shaders/ShadersInclude/logDepthDeclaration";
import "babylonjs/Shaders/ShadersInclude/fogFragmentDeclaration";
import "babylonjs/Shaders/ShadersInclude/helperFunctions";
import "babylonjs/Shaders/ShadersInclude/subSurfaceScatteringFunctions";
import "babylonjs/Shaders/ShadersInclude/importanceSampling";
import "babylonjs/Shaders/ShadersInclude/pbrHelperFunctions";
import "babylonjs/Shaders/ShadersInclude/imageProcessingFunctions";
import "babylonjs/Shaders/ShadersInclude/shadowsFragmentFunctions";
import "babylonjs/Shaders/ShadersInclude/harmonicsFunctions";
import "babylonjs/Shaders/ShadersInclude/pbrDirectLightingSetupFunctions";
import "babylonjs/Shaders/ShadersInclude/pbrDirectLightingFalloffFunctions";
import "babylonjs/Shaders/ShadersInclude/pbrBRDFFunctions";
import "babylonjs/Shaders/ShadersInclude/hdrFilteringFunctions";
import "babylonjs/Shaders/ShadersInclude/pbrDirectLightingFunctions";
import "babylonjs/Shaders/ShadersInclude/pbrIBLFunctions";
import "babylonjs/Shaders/ShadersInclude/bumpFragmentMainFunctions";
import "babylonjs/Shaders/ShadersInclude/bumpFragmentFunctions";
import "babylonjs/Shaders/ShadersInclude/reflectionFunction";
import "babylonjs/Shaders/ShadersInclude/pbrBlockAlbedoOpacity";
import "babylonjs/Shaders/ShadersInclude/pbrBlockReflectivity";
import "babylonjs/Shaders/ShadersInclude/pbrBlockAmbientOcclusion";
import "babylonjs/Shaders/ShadersInclude/pbrBlockAlphaFresnel";
import "babylonjs/Shaders/ShadersInclude/pbrBlockAnisotropic";
import "babylonjs/Shaders/ShadersInclude/pbrBlockReflection";
import "babylonjs/Shaders/ShadersInclude/pbrBlockSheen";
import "babylonjs/Shaders/ShadersInclude/pbrBlockClearcoat";
import "babylonjs/Shaders/ShadersInclude/pbrBlockIridescence";
import "babylonjs/Shaders/ShadersInclude/pbrBlockSubSurface";
import "babylonjs/Shaders/ShadersInclude/clipPlaneFragment";
import "babylonjs/Shaders/ShadersInclude/pbrBlockNormalGeometric";
import "babylonjs/Shaders/ShadersInclude/bumpFragment";
import "babylonjs/Shaders/ShadersInclude/pbrBlockNormalFinal";
import "babylonjs/Shaders/ShadersInclude/depthPrePass";
import "babylonjs/Shaders/ShadersInclude/pbrBlockLightmapInit";
import "babylonjs/Shaders/ShadersInclude/pbrBlockGeometryInfo";
import "babylonjs/Shaders/ShadersInclude/pbrBlockReflectance0";
import "babylonjs/Shaders/ShadersInclude/pbrBlockReflectance";
import "babylonjs/Shaders/ShadersInclude/pbrBlockDirectLighting";
import "babylonjs/Shaders/ShadersInclude/lightFragment";
import "babylonjs/Shaders/ShadersInclude/pbrBlockFinalLitComponents";
import "babylonjs/Shaders/ShadersInclude/pbrBlockFinalUnlitComponents";
import "babylonjs/Shaders/ShadersInclude/pbrBlockFinalColorComposition";
import "babylonjs/Shaders/ShadersInclude/logDepthFragment";
import "babylonjs/Shaders/ShadersInclude/fogFragment";
import "babylonjs/Shaders/ShadersInclude/pbrBlockImageProcessing";
import "babylonjs/Shaders/ShadersInclude/oitFragment";
import "babylonjs/Shaders/ShadersInclude/pbrDebug";
/** @internal */
export const pbrPixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/pbr.vertex" {
import "babylonjs/Shaders/ShadersInclude/pbrVertexDeclaration";
import "babylonjs/Shaders/ShadersInclude/pbrUboDeclaration";
import "babylonjs/Shaders/ShadersInclude/uvAttributeDeclaration";
import "babylonjs/Shaders/ShadersInclude/mainUVVaryingDeclaration";
import "babylonjs/Shaders/ShadersInclude/helperFunctions";
import "babylonjs/Shaders/ShadersInclude/bonesDeclaration";
import "babylonjs/Shaders/ShadersInclude/bakedVertexAnimationDeclaration";
import "babylonjs/Shaders/ShadersInclude/instancesDeclaration";
import "babylonjs/Shaders/ShadersInclude/prePassVertexDeclaration";
import "babylonjs/Shaders/ShadersInclude/samplerVertexDeclaration";
import "babylonjs/Shaders/ShadersInclude/harmonicsFunctions";
import "babylonjs/Shaders/ShadersInclude/bumpVertexDeclaration";
import "babylonjs/Shaders/ShadersInclude/clipPlaneVertexDeclaration";
import "babylonjs/Shaders/ShadersInclude/fogVertexDeclaration";
import "babylonjs/Shaders/ShadersInclude/lightVxFragmentDeclaration";
import "babylonjs/Shaders/ShadersInclude/lightVxUboDeclaration";
import "babylonjs/Shaders/ShadersInclude/morphTargetsVertexGlobalDeclaration";
import "babylonjs/Shaders/ShadersInclude/morphTargetsVertexDeclaration";
import "babylonjs/Shaders/ShadersInclude/logDepthDeclaration";
import "babylonjs/Shaders/ShadersInclude/morphTargetsVertexGlobal";
import "babylonjs/Shaders/ShadersInclude/morphTargetsVertex";
import "babylonjs/Shaders/ShadersInclude/instancesVertex";
import "babylonjs/Shaders/ShadersInclude/bonesVertex";
import "babylonjs/Shaders/ShadersInclude/bakedVertexAnimation";
import "babylonjs/Shaders/ShadersInclude/prePassVertex";
import "babylonjs/Shaders/ShadersInclude/uvVariableDeclaration";
import "babylonjs/Shaders/ShadersInclude/samplerVertexImplementation";
import "babylonjs/Shaders/ShadersInclude/bumpVertex";
import "babylonjs/Shaders/ShadersInclude/clipPlaneVertex";
import "babylonjs/Shaders/ShadersInclude/fogVertex";
import "babylonjs/Shaders/ShadersInclude/shadowsVertex";
import "babylonjs/Shaders/ShadersInclude/vertexColorMixing";
import "babylonjs/Shaders/ShadersInclude/logDepthVertex";
/** @internal */
export const pbrVertexShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/postprocess.vertex" {
/** @internal */
export const postprocessVertexShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/procedural.vertex" {
/** @internal */
export const proceduralVertexShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/refraction.fragment" {
/** @internal */
export const refractionPixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/rgbdDecode.fragment" {
import "babylonjs/Shaders/ShadersInclude/helperFunctions";
/** @internal */
export const rgbdDecodePixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/rgbdEncode.fragment" {
import "babylonjs/Shaders/ShadersInclude/helperFunctions";
/** @internal */
export const rgbdEncodePixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/screenSpaceCurvature.fragment" {
/** @internal */
export const screenSpaceCurvaturePixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/screenSpaceReflection.fragment" {
/** @internal */
export const screenSpaceReflectionPixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/backgroundFragmentDeclaration" {
/** @internal */
export const backgroundFragmentDeclaration: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/backgroundUboDeclaration" {
import "babylonjs/Shaders/ShadersInclude/sceneUboDeclaration";
/** @internal */
export const backgroundUboDeclaration: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/backgroundVertexDeclaration" {
/** @internal */
export const backgroundVertexDeclaration: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/bakedVertexAnimation" {
/** @internal */
export const bakedVertexAnimation: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/bakedVertexAnimationDeclaration" {
/** @internal */
export const bakedVertexAnimationDeclaration: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/bayerDitherFunctions" {
/** @internal */
export const bayerDitherFunctions: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/bonesDeclaration" {
/** @internal */
export const bonesDeclaration: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/bonesVertex" {
/** @internal */
export const bonesVertex: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/boundingBoxRendererFragmentDeclaration" {
/** @internal */
export const boundingBoxRendererFragmentDeclaration: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/boundingBoxRendererUboDeclaration" {
/** @internal */
export const boundingBoxRendererUboDeclaration: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/boundingBoxRendererVertexDeclaration" {
/** @internal */
export const boundingBoxRendererVertexDeclaration: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/bumpFragment" {
/** @internal */
export const bumpFragment: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/bumpFragmentFunctions" {
import "babylonjs/Shaders/ShadersInclude/samplerFragmentDeclaration";
/** @internal */
export const bumpFragmentFunctions: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/bumpFragmentMainFunctions" {
/** @internal */
export const bumpFragmentMainFunctions: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/bumpVertex" {
/** @internal */
export const bumpVertex: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/bumpVertexDeclaration" {
/** @internal */
export const bumpVertexDeclaration: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/clipPlaneFragment" {
/** @internal */
export const clipPlaneFragment: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/clipPlaneFragmentDeclaration" {
/** @internal */
export const clipPlaneFragmentDeclaration: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/clipPlaneFragmentDeclaration2" {
/** @internal */
export const clipPlaneFragmentDeclaration2: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/clipPlaneVertex" {
/** @internal */
export const clipPlaneVertex: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/clipPlaneVertexDeclaration" {
/** @internal */
export const clipPlaneVertexDeclaration: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/clipPlaneVertexDeclaration2" {
/** @internal */
export const clipPlaneVertexDeclaration2: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/defaultFragmentDeclaration" {
/** @internal */
export const defaultFragmentDeclaration: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/defaultUboDeclaration" {
import "babylonjs/Shaders/ShadersInclude/sceneUboDeclaration";
import "babylonjs/Shaders/ShadersInclude/meshUboDeclaration";
/** @internal */
export const defaultUboDeclaration: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/defaultVertexDeclaration" {
/** @internal */
export const defaultVertexDeclaration: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/depthPrePass" {
/** @internal */
export const depthPrePass: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/diffusionProfile" {
/** @internal */
export const diffusionProfile: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/fibonacci" {
/** @internal */
export const fibonacci: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/fogFragment" {
/** @internal */
export const fogFragment: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/fogFragmentDeclaration" {
/** @internal */
export const fogFragmentDeclaration: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/fogVertex" {
/** @internal */
export const fogVertex: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/fogVertexDeclaration" {
/** @internal */
export const fogVertexDeclaration: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/fresnelFunction" {
/** @internal */
export const fresnelFunction: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/geometryUboDeclaration" {
import "babylonjs/Shaders/ShadersInclude/sceneUboDeclaration";
/** @internal */
export const geometryUboDeclaration: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/geometryVertexDeclaration" {
/** @internal */
export const geometryVertexDeclaration: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/harmonicsFunctions" {
/** @internal */
export const harmonicsFunctions: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/hdrFilteringFunctions" {
/** @internal */
export const hdrFilteringFunctions: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/helperFunctions" {
/** @internal */
export const helperFunctions: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/imageProcessingCompatibility" {
/** @internal */
export const imageProcessingCompatibility: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/imageProcessingDeclaration" {
/** @internal */
export const imageProcessingDeclaration: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/imageProcessingFunctions" {
/** @internal */
export const imageProcessingFunctions: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/importanceSampling" {
/** @internal */
export const importanceSampling: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/instancesDeclaration" {
/** @internal */
export const instancesDeclaration: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/instancesVertex" {
/** @internal */
export const instancesVertex: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/kernelBlurFragment" {
/** @internal */
export const kernelBlurFragment: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/kernelBlurFragment2" {
/** @internal */
export const kernelBlurFragment2: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/kernelBlurVaryingDeclaration" {
/** @internal */
export const kernelBlurVaryingDeclaration: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/kernelBlurVertex" {
/** @internal */
export const kernelBlurVertex: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/lightFragment" {
/** @internal */
export const lightFragment: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/lightFragmentDeclaration" {
/** @internal */
export const lightFragmentDeclaration: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/lightsFragmentFunctions" {
/** @internal */
export const lightsFragmentFunctions: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/lightUboDeclaration" {
/** @internal */
export const lightUboDeclaration: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/lightVxFragmentDeclaration" {
/** @internal */
export const lightVxFragmentDeclaration: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/lightVxUboDeclaration" {
/** @internal */
export const lightVxUboDeclaration: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/logDepthDeclaration" {
/** @internal */
export const logDepthDeclaration: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/logDepthFragment" {
/** @internal */
export const logDepthFragment: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/logDepthVertex" {
/** @internal */
export const logDepthVertex: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/mainUVVaryingDeclaration" {
/** @internal */
export const mainUVVaryingDeclaration: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/meshFragmentDeclaration" {
/** @internal */
export const meshFragmentDeclaration: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/meshUboDeclaration" {
/** @internal */
export const meshUboDeclaration: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/meshVertexDeclaration" {
/** @internal */
export const meshVertexDeclaration: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/morphTargetsVertex" {
/** @internal */
export const morphTargetsVertex: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/morphTargetsVertexDeclaration" {
/** @internal */
export const morphTargetsVertexDeclaration: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/morphTargetsVertexGlobal" {
/** @internal */
export const morphTargetsVertexGlobal: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/morphTargetsVertexGlobalDeclaration" {
/** @internal */
export const morphTargetsVertexGlobalDeclaration: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/mrtFragmentDeclaration" {
/** @internal */
export const mrtFragmentDeclaration: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/oitDeclaration" {
/** @internal */
export const oitDeclaration: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/oitFragment" {
/** @internal */
export const oitFragment: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/packingFunctions" {
/** @internal */
export const packingFunctions: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/pbrBlockAlbedoOpacity" {
/** @internal */
export const pbrBlockAlbedoOpacity: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/pbrBlockAlphaFresnel" {
/** @internal */
export const pbrBlockAlphaFresnel: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/pbrBlockAmbientOcclusion" {
/** @internal */
export const pbrBlockAmbientOcclusion: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/pbrBlockAnisotropic" {
/** @internal */
export const pbrBlockAnisotropic: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/pbrBlockClearcoat" {
/** @internal */
export const pbrBlockClearcoat: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/pbrBlockDirectLighting" {
/** @internal */
export const pbrBlockDirectLighting: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/pbrBlockFinalColorComposition" {
/** @internal */
export const pbrBlockFinalColorComposition: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/pbrBlockFinalLitComponents" {
/** @internal */
export const pbrBlockFinalLitComponents: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/pbrBlockFinalUnlitComponents" {
/** @internal */
export const pbrBlockFinalUnlitComponents: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/pbrBlockGeometryInfo" {
/** @internal */
export const pbrBlockGeometryInfo: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/pbrBlockImageProcessing" {
/** @internal */
export const pbrBlockImageProcessing: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/pbrBlockIridescence" {
/** @internal */
export const pbrBlockIridescence: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/pbrBlockLightmapInit" {
/** @internal */
export const pbrBlockLightmapInit: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/pbrBlockNormalFinal" {
/** @internal */
export const pbrBlockNormalFinal: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/pbrBlockNormalGeometric" {
/** @internal */
export const pbrBlockNormalGeometric: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/pbrBlockReflectance" {
/** @internal */
export const pbrBlockReflectance: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/pbrBlockReflectance0" {
/** @internal */
export const pbrBlockReflectance0: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/pbrBlockReflection" {
/** @internal */
export const pbrBlockReflection: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/pbrBlockReflectivity" {
/** @internal */
export const pbrBlockReflectivity: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/pbrBlockSheen" {
/** @internal */
export const pbrBlockSheen: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/pbrBlockSubSurface" {
/** @internal */
export const pbrBlockSubSurface: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/pbrBRDFFunctions" {
/** @internal */
export const pbrBRDFFunctions: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/pbrDebug" {
/** @internal */
export const pbrDebug: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/pbrDirectLightingFalloffFunctions" {
/** @internal */
export const pbrDirectLightingFalloffFunctions: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/pbrDirectLightingFunctions" {
/** @internal */
export const pbrDirectLightingFunctions: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/pbrDirectLightingSetupFunctions" {
/** @internal */
export const pbrDirectLightingSetupFunctions: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/pbrFragmentDeclaration" {
/** @internal */
export const pbrFragmentDeclaration: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/pbrFragmentExtraDeclaration" {
import "babylonjs/Shaders/ShadersInclude/mainUVVaryingDeclaration";
/** @internal */
export const pbrFragmentExtraDeclaration: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/pbrFragmentSamplersDeclaration" {
import "babylonjs/Shaders/ShadersInclude/samplerFragmentDeclaration";
import "babylonjs/Shaders/ShadersInclude/samplerFragmentAlternateDeclaration";
/** @internal */
export const pbrFragmentSamplersDeclaration: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/pbrHelperFunctions" {
/** @internal */
export const pbrHelperFunctions: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/pbrIBLFunctions" {
/** @internal */
export const pbrIBLFunctions: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/pbrUboDeclaration" {
import "babylonjs/Shaders/ShadersInclude/sceneUboDeclaration";
import "babylonjs/Shaders/ShadersInclude/meshUboDeclaration";
/** @internal */
export const pbrUboDeclaration: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/pbrVertexDeclaration" {
/** @internal */
export const pbrVertexDeclaration: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/pointCloudVertex" {
/** @internal */
export const pointCloudVertex: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/pointCloudVertexDeclaration" {
/** @internal */
export const pointCloudVertexDeclaration: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/prePassDeclaration" {
/** @internal */
export const prePassDeclaration: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/prePassVertex" {
/** @internal */
export const prePassVertex: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/prePassVertexDeclaration" {
/** @internal */
export const prePassVertexDeclaration: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/reflectionFunction" {
/** @internal */
export const reflectionFunction: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/samplerFragmentAlternateDeclaration" {
/** @internal */
export const samplerFragmentAlternateDeclaration: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/samplerFragmentDeclaration" {
/** @internal */
export const samplerFragmentDeclaration: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/samplerVertexDeclaration" {
/** @internal */
export const samplerVertexDeclaration: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/samplerVertexImplementation" {
/** @internal */
export const samplerVertexImplementation: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/sceneFragmentDeclaration" {
/** @internal */
export const sceneFragmentDeclaration: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/sceneUboDeclaration" {
/** @internal */
export const sceneUboDeclaration: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/sceneVertexDeclaration" {
/** @internal */
export const sceneVertexDeclaration: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/shadowMapFragment" {
/** @internal */
export const shadowMapFragment: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/shadowMapFragmentExtraDeclaration" {
import "babylonjs/Shaders/ShadersInclude/packingFunctions";
import "babylonjs/Shaders/ShadersInclude/bayerDitherFunctions";
/** @internal */
export const shadowMapFragmentExtraDeclaration: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/shadowMapFragmentSoftTransparentShadow" {
/** @internal */
export const shadowMapFragmentSoftTransparentShadow: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/shadowMapUboDeclaration" {
import "babylonjs/Shaders/ShadersInclude/sceneUboDeclaration";
import "babylonjs/Shaders/ShadersInclude/meshUboDeclaration";
/** @internal */
export const shadowMapUboDeclaration: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/shadowMapVertexDeclaration" {
import "babylonjs/Shaders/ShadersInclude/sceneVertexDeclaration";
import "babylonjs/Shaders/ShadersInclude/meshVertexDeclaration";
/** @internal */
export const shadowMapVertexDeclaration: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/shadowMapVertexExtraDeclaration" {
/** @internal */
export const shadowMapVertexExtraDeclaration: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/shadowMapVertexMetric" {
/** @internal */
export const shadowMapVertexMetric: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/shadowMapVertexNormalBias" {
/** @internal */
export const shadowMapVertexNormalBias: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/shadowsFragmentFunctions" {
/** @internal */
export const shadowsFragmentFunctions: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/shadowsVertex" {
/** @internal */
export const shadowsVertex: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/subSurfaceScatteringFunctions" {
/** @internal */
export const subSurfaceScatteringFunctions: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/uvAttributeDeclaration" {
/** @internal */
export const uvAttributeDeclaration: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/uvVariableDeclaration" {
/** @internal */
export const uvVariableDeclaration: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ShadersInclude/vertexColorMixing" {
/** @internal */
export const vertexColorMixing: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/shadowMap.fragment" {
import "babylonjs/Shaders/ShadersInclude/shadowMapFragmentExtraDeclaration";
import "babylonjs/Shaders/ShadersInclude/clipPlaneFragmentDeclaration";
import "babylonjs/Shaders/ShadersInclude/clipPlaneFragment";
import "babylonjs/Shaders/ShadersInclude/shadowMapFragment";
/** @internal */
export const shadowMapPixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/shadowMap.vertex" {
import "babylonjs/Shaders/ShadersInclude/bonesDeclaration";
import "babylonjs/Shaders/ShadersInclude/bakedVertexAnimationDeclaration";
import "babylonjs/Shaders/ShadersInclude/morphTargetsVertexGlobalDeclaration";
import "babylonjs/Shaders/ShadersInclude/morphTargetsVertexDeclaration";
import "babylonjs/Shaders/ShadersInclude/helperFunctions";
import "babylonjs/Shaders/ShadersInclude/shadowMapVertexDeclaration";
import "babylonjs/Shaders/ShadersInclude/shadowMapUboDeclaration";
import "babylonjs/Shaders/ShadersInclude/shadowMapVertexExtraDeclaration";
import "babylonjs/Shaders/ShadersInclude/clipPlaneVertexDeclaration";
import "babylonjs/Shaders/ShadersInclude/morphTargetsVertexGlobal";
import "babylonjs/Shaders/ShadersInclude/morphTargetsVertex";
import "babylonjs/Shaders/ShadersInclude/instancesVertex";
import "babylonjs/Shaders/ShadersInclude/bonesVertex";
import "babylonjs/Shaders/ShadersInclude/bakedVertexAnimation";
import "babylonjs/Shaders/ShadersInclude/shadowMapVertexNormalBias";
import "babylonjs/Shaders/ShadersInclude/shadowMapVertexMetric";
import "babylonjs/Shaders/ShadersInclude/clipPlaneVertex";
/** @internal */
export const shadowMapVertexShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/sharpen.fragment" {
/** @internal */
export const sharpenPixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/spriteMap.fragment" {
/** @internal */
export const spriteMapPixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/spriteMap.vertex" {
/** @internal */
export const spriteMapVertexShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/sprites.fragment" {
import "babylonjs/Shaders/ShadersInclude/fogFragmentDeclaration";
import "babylonjs/Shaders/ShadersInclude/fogFragment";
import "babylonjs/Shaders/ShadersInclude/imageProcessingCompatibility";
/** @internal */
export const spritesPixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/sprites.vertex" {
import "babylonjs/Shaders/ShadersInclude/fogVertexDeclaration";
/** @internal */
export const spritesVertexShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ssao.fragment" {
/** @internal */
export const ssaoPixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ssao2.fragment" {
/** @internal */
export const ssao2PixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/ssaoCombine.fragment" {
/** @internal */
export const ssaoCombinePixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/standard.fragment" {
import "babylonjs/Shaders/ShadersInclude/packingFunctions";
/** @internal */
export const standardPixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/stereoscopicInterlace.fragment" {
/** @internal */
export const stereoscopicInterlacePixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/subSurfaceScattering.fragment" {
import "babylonjs/Shaders/ShadersInclude/fibonacci";
import "babylonjs/Shaders/ShadersInclude/helperFunctions";
import "babylonjs/Shaders/ShadersInclude/subSurfaceScatteringFunctions";
import "babylonjs/Shaders/ShadersInclude/diffusionProfile";
/** @internal */
export const subSurfaceScatteringPixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/tonemap.fragment" {
/** @internal */
export const tonemapPixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/volumetricLightScattering.fragment" {
/** @internal */
export const volumetricLightScatteringPixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/volumetricLightScatteringPass.fragment" {
/** @internal */
export const volumetricLightScatteringPassPixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/volumetricLightScatteringPass.vertex" {
import "babylonjs/Shaders/ShadersInclude/bonesDeclaration";
import "babylonjs/Shaders/ShadersInclude/bakedVertexAnimationDeclaration";
import "babylonjs/Shaders/ShadersInclude/morphTargetsVertexGlobalDeclaration";
import "babylonjs/Shaders/ShadersInclude/morphTargetsVertexDeclaration";
import "babylonjs/Shaders/ShadersInclude/instancesDeclaration";
import "babylonjs/Shaders/ShadersInclude/morphTargetsVertexGlobal";
import "babylonjs/Shaders/ShadersInclude/morphTargetsVertex";
import "babylonjs/Shaders/ShadersInclude/instancesVertex";
import "babylonjs/Shaders/ShadersInclude/bonesVertex";
import "babylonjs/Shaders/ShadersInclude/bakedVertexAnimation";
/** @internal */
export const volumetricLightScatteringPassVertexShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/vrDistortionCorrection.fragment" {
/** @internal */
export const vrDistortionCorrectionPixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Shaders/vrMultiviewToSingleview.fragment" {
/** @internal */
export const vrMultiviewToSingleviewPixelShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/ShadersWGSL/gpuUpdateParticles.compute" {
/** @internal */
export const gpuUpdateParticlesComputeShader: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/ShadersWGSL/ShadersInclude/bakedVertexAnimation" {
/** @internal */
export const bakedVertexAnimation: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/ShadersWGSL/ShadersInclude/bakedVertexAnimationDeclaration" {
/** @internal */
export const bakedVertexAnimationDeclaration: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/ShadersWGSL/ShadersInclude/bonesDeclaration" {
/** @internal */
export const bonesDeclaration: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/ShadersWGSL/ShadersInclude/bonesVertex" {
/** @internal */
export const bonesVertex: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/ShadersWGSL/ShadersInclude/clipPlaneFragment" {
/** @internal */
export const clipPlaneFragment: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/ShadersWGSL/ShadersInclude/clipPlaneFragmentDeclaration" {
/** @internal */
export const clipPlaneFragmentDeclaration: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/ShadersWGSL/ShadersInclude/clipPlaneVertex" {
/** @internal */
export const clipPlaneVertex: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/ShadersWGSL/ShadersInclude/clipPlaneVertexDeclaration" {
/** @internal */
export const clipPlaneVertexDeclaration: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/ShadersWGSL/ShadersInclude/instancesDeclaration" {
/** @internal */
export const instancesDeclaration: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/ShadersWGSL/ShadersInclude/instancesVertex" {
/** @internal */
export const instancesVertex: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/ShadersWGSL/ShadersInclude/meshUboDeclaration" {
/** @internal */
export const meshUboDeclaration: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/ShadersWGSL/ShadersInclude/morphTargetsVertex" {
/** @internal */
export const morphTargetsVertex: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/ShadersWGSL/ShadersInclude/morphTargetsVertexDeclaration" {
/** @internal */
export const morphTargetsVertexDeclaration: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/ShadersWGSL/ShadersInclude/morphTargetsVertexGlobal" {
/** @internal */
export const morphTargetsVertexGlobal: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/ShadersWGSL/ShadersInclude/morphTargetsVertexGlobalDeclaration" {
/** @internal */
export const morphTargetsVertexGlobalDeclaration: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/ShadersWGSL/ShadersInclude/sceneUboDeclaration" {
/** @internal */
export const sceneUboDeclaration: {
    name: string;
    shader: string;
};

}
declare module "babylonjs/Sprites/index" {
export * from "babylonjs/Sprites/sprite";
export * from "babylonjs/Sprites/ISprites";
export * from "babylonjs/Sprites/spriteManager";
export * from "babylonjs/Sprites/spriteMap";
export * from "babylonjs/Sprites/spritePackedManager";
export * from "babylonjs/Sprites/spriteSceneComponent";

}
declare module "babylonjs/Sprites/ISprites" {
/**
 * Defines the basic options interface of a Sprite Frame Source Size.
 */
export interface ISpriteJSONSpriteSourceSize {
    /**
     * number of the original width of the Frame
     */
    w: number;
    /**
     * number of the original height of the Frame
     */
    h: number;
}
/**
 * Defines the basic options interface of a Sprite Frame Data.
 */
export interface ISpriteJSONSpriteFrameData {
    /**
     * number of the x offset of the Frame
     */
    x: number;
    /**
     * number of the y offset of the Frame
     */
    y: number;
    /**
     * number of the width of the Frame
     */
    w: number;
    /**
     * number of the height of the Frame
     */
    h: number;
}
/**
 * Defines the basic options interface of a JSON Sprite.
 */
export interface ISpriteJSONSprite {
    /**
     * string name of the Frame
     */
    filename: string;
    /**
     * ISpriteJSONSpriteFrame basic object of the frame data
     */
    frame: ISpriteJSONSpriteFrameData;
    /**
     * boolean to flag is the frame was rotated.
     */
    rotated: boolean;
    /**
     * boolean to flag is the frame was trimmed.
     */
    trimmed: boolean;
    /**
     * ISpriteJSONSpriteFrame basic object of the source data
     */
    spriteSourceSize: ISpriteJSONSpriteFrameData;
    /**
     * ISpriteJSONSpriteFrame basic object of the source data
     */
    sourceSize: ISpriteJSONSpriteSourceSize;
}
/**
 * Defines the basic options interface of a JSON atlas.
 */
export interface ISpriteJSONAtlas {
    /**
     * Array of objects that contain the frame data.
     */
    frames: Array<ISpriteJSONSprite>;
    /**
     * object basic object containing the sprite meta data.
     */
    meta?: object;
}

}
declare module "babylonjs/Sprites/sprite" {
import { Vector3 } from "babylonjs/Maths/math.vector";
import { Nullable } from "babylonjs/types";
import { ActionManager } from "babylonjs/Actions/actionManager";
import { ISpriteManager, SpriteManager } from "babylonjs/Sprites/spriteManager";
import { Color4 } from "babylonjs/Maths/math.color";
import { Observable } from "babylonjs/Misc/observable";
import { IAnimatable } from "babylonjs/Animations/animatable.interface";
import { ThinSprite } from "babylonjs/Sprites/thinSprite";
import { Animation } from "babylonjs/Animations/animation";
/**
 * Class used to represent a sprite
 * @see https://doc.babylonjs.com/features/featuresDeepDive/sprites
 */
export class Sprite extends ThinSprite implements IAnimatable {
    /** defines the name */
    name: string;
    /** Gets or sets the current world position */
    position: Vector3;
    /** Gets or sets the main color */
    color: Color4;
    /** Gets or sets a boolean indicating that this sprite should be disposed after animation ends */
    disposeWhenFinishedAnimating: boolean;
    /** Gets the list of attached animations */
    animations: Nullable<Array<Animation>>;
    /** Gets or sets a boolean indicating if the sprite can be picked */
    isPickable: boolean;
    /** Gets or sets a boolean indicating that sprite texture alpha will be used for precise picking (false by default) */
    useAlphaForPicking: boolean;
    /**
     * Gets or sets the associated action manager
     */
    actionManager: Nullable<ActionManager>;
    /**
     * An event triggered when the control has been disposed
     */
    onDisposeObservable: Observable<Sprite>;
    private _manager;
    private _onAnimationEnd;
    /**
     * Gets or sets the sprite size
     */
    get size(): number;
    set size(value: number);
    /**
     * Gets or sets the unique id of the sprite
     */
    uniqueId: number;
    /**
     * Gets the manager of this sprite
     */
    get manager(): ISpriteManager;
    /**
     * Creates a new Sprite
     * @param name defines the name
     * @param manager defines the manager
     */
    constructor(
    /** defines the name */
    name: string, manager: ISpriteManager);
    /**
     * Returns the string "Sprite"
     * @returns "Sprite"
     */
    getClassName(): string;
    /** Gets or sets the initial key for the animation (setting it will restart the animation)  */
    get fromIndex(): number;
    set fromIndex(value: number);
    /** Gets or sets the end key for the animation (setting it will restart the animation)  */
    get toIndex(): number;
    set toIndex(value: number);
    /** Gets or sets a boolean indicating if the animation is looping (setting it will restart the animation)  */
    get loopAnimation(): boolean;
    set loopAnimation(value: boolean);
    /** Gets or sets the delay between cell changes (setting it will restart the animation)  */
    get delay(): number;
    set delay(value: number);
    /**
     * Starts an animation
     * @param from defines the initial key
     * @param to defines the end key
     * @param loop defines if the animation must loop
     * @param delay defines the start delay (in ms)
     * @param onAnimationEnd defines a callback to call when animation ends
     */
    playAnimation(from: number, to: number, loop: boolean, delay: number, onAnimationEnd?: Nullable<() => void>): void;
    private _endAnimation;
    /** Release associated resources */
    dispose(): void;
    /**
     * Serializes the sprite to a JSON object
     * @returns the JSON object
     */
    serialize(): any;
    /**
     * Parses a JSON object to create a new sprite
     * @param parsedSprite The JSON object to parse
     * @param manager defines the hosting manager
     * @returns the new sprite
     */
    static Parse(parsedSprite: any, manager: SpriteManager): Sprite;
}
export {};

}
declare module "babylonjs/Sprites/spriteManager" {
import { IDisposable, Scene } from "babylonjs/scene";
import { Nullable } from "babylonjs/types";
import { Observable } from "babylonjs/Misc/observable";
import { Sprite } from "babylonjs/Sprites/sprite";
import { PickingInfo } from "babylonjs/Collisions/pickingInfo";
import { Camera } from "babylonjs/Cameras/camera";
import { Texture } from "babylonjs/Materials/Textures/texture";
import { Ray } from "babylonjs/Culling/ray";
/**
 * Defines the minimum interface to fulfill in order to be a sprite manager.
 */
export interface ISpriteManager extends IDisposable {
    /**
     * Gets manager's name
     */
    name: string;
    /**
     * Restricts the camera to viewing objects with the same layerMask.
     * A camera with a layerMask of 1 will render spriteManager.layerMask & camera.layerMask!== 0
     */
    layerMask: number;
    /**
     * Gets or sets a boolean indicating if the mesh can be picked (by scene.pick for instance or through actions). Default is true
     */
    isPickable: boolean;
    /**
     * Gets the hosting scene
     */
    scene: Scene;
    /**
     * Specifies the rendering group id for this mesh (0 by default)
     * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/advanced/transparent_rendering#rendering-groups
     */
    renderingGroupId: number;
    /**
     * Defines the list of sprites managed by the manager.
     */
    sprites: Array<Sprite>;
    /**
     * Gets or sets the spritesheet texture
     */
    texture: Texture;
    /** Defines the default width of a cell in the spritesheet */
    cellWidth: number;
    /** Defines the default height of a cell in the spritesheet */
    cellHeight: number;
    /** @internal */
    _wasDispatched: boolean;
    /**
     * Tests the intersection of a sprite with a specific ray.
     * @param ray The ray we are sending to test the collision
     * @param camera The camera space we are sending rays in
     * @param predicate A predicate allowing excluding sprites from the list of object to test
     * @param fastCheck defines if the first intersection will be used (and not the closest)
     * @returns picking info or null.
     */
    intersects(ray: Ray, camera: Camera, predicate?: (sprite: Sprite) => boolean, fastCheck?: boolean): Nullable<PickingInfo>;
    /**
     * Intersects the sprites with a ray
     * @param ray defines the ray to intersect with
     * @param camera defines the current active camera
     * @param predicate defines a predicate used to select candidate sprites
     * @returns null if no hit or a PickingInfo array
     */
    multiIntersects(ray: Ray, camera: Camera, predicate?: (sprite: Sprite) => boolean): Nullable<PickingInfo[]>;
    /**
     * Renders the list of sprites on screen.
     */
    render(): void;
    /**
     * Rebuilds the manager (after a context lost, for eg)
     */
    rebuild(): void;
}
/**
 * Class used to manage multiple sprites on the same spritesheet
 * @see https://doc.babylonjs.com/features/featuresDeepDive/sprites
 */
export class SpriteManager implements ISpriteManager {
    /** defines the manager's name */
    name: string;
    /** Define the Url to load snippets */
    static SnippetUrl: string;
    /** Snippet ID if the manager was created from the snippet server */
    snippetId: string;
    /** Gets the list of sprites */
    sprites: Sprite[];
    /** Gets or sets the rendering group id (0 by default) */
    renderingGroupId: number;
    /** Gets or sets camera layer mask */
    layerMask: number;
    /** Gets or sets a boolean indicating if the sprites are pickable */
    isPickable: boolean;
    /**
     * Gets or sets an object used to store user defined information for the sprite manager
     */
    metadata: any;
    /** @internal */
    _wasDispatched: boolean;
    /**
     * An event triggered when the manager is disposed.
     */
    onDisposeObservable: Observable<SpriteManager>;
    /**
     * Callback called when the manager is disposed
     */
    set onDispose(callback: () => void);
    /**
     * Gets or sets the unique id of the sprite
     */
    uniqueId: number;
    /**
     * Gets the array of sprites
     */
    get children(): Sprite[];
    /**
     * Gets the hosting scene
     */
    get scene(): Scene;
    /**
     * Gets the capacity of the manager
     */
    get capacity(): number;
    /**
     * Gets or sets the spritesheet texture
     */
    get texture(): Texture;
    set texture(value: Texture);
    /** Defines the default width of a cell in the spritesheet */
    get cellWidth(): number;
    set cellWidth(value: number);
    /** Defines the default height of a cell in the spritesheet */
    get cellHeight(): number;
    set cellHeight(value: number);
    /** Gets or sets a boolean indicating if the manager must consider scene fog when rendering */
    get fogEnabled(): boolean;
    set fogEnabled(value: boolean);
    /**
     * Blend mode use to render the particle, it can be any of
     * the static Constants.ALPHA_x properties provided in this class.
     * Default value is Constants.ALPHA_COMBINE
     */
    get blendMode(): number;
    set blendMode(blendMode: number);
    private _disableDepthWrite;
    /** Disables writing to the depth buffer when rendering the sprites.
     *  It can be handy to disable depth writing when using textures without alpha channel
     *  and setting some specific blend modes.
     */
    get disableDepthWrite(): boolean;
    set disableDepthWrite(value: boolean);
    private _spriteRenderer;
    /** Associative array from JSON sprite data file */
    private _cellData;
    /** Array of sprite names from JSON sprite data file */
    private _spriteMap;
    /** True when packed cell data from JSON file is ready*/
    private _packedAndReady;
    private _textureContent;
    private _onDisposeObserver;
    private _fromPacked;
    private _scene;
    /**
     * Creates a new sprite manager
     * @param name defines the manager's name
     * @param imgUrl defines the sprite sheet url
     * @param capacity defines the maximum allowed number of sprites
     * @param cellSize defines the size of a sprite cell
     * @param scene defines the hosting scene
     * @param epsilon defines the epsilon value to align texture (0.01 by default)
     * @param samplingMode defines the sampling mode to use with spritesheet
     * @param fromPacked set to false; do not alter
     * @param spriteJSON null otherwise a JSON object defining sprite sheet data; do not alter
     */
    constructor(
    /** defines the manager's name */
    name: string, imgUrl: string, capacity: number, cellSize: any, scene: Scene, epsilon?: number, samplingMode?: number, fromPacked?: boolean, spriteJSON?: any | null);
    /**
     * Returns the string "SpriteManager"
     * @returns "SpriteManager"
     */
    getClassName(): string;
    private _makePacked;
    private _checkTextureAlpha;
    /**
     * Intersects the sprites with a ray
     * @param ray defines the ray to intersect with
     * @param camera defines the current active camera
     * @param predicate defines a predicate used to select candidate sprites
     * @param fastCheck defines if a fast check only must be done (the first potential sprite is will be used and not the closer)
     * @returns null if no hit or a PickingInfo
     */
    intersects(ray: Ray, camera: Camera, predicate?: (sprite: Sprite) => boolean, fastCheck?: boolean): Nullable<PickingInfo>;
    /**
     * Intersects the sprites with a ray
     * @param ray defines the ray to intersect with
     * @param camera defines the current active camera
     * @param predicate defines a predicate used to select candidate sprites
     * @returns null if no hit or a PickingInfo array
     */
    multiIntersects(ray: Ray, camera: Camera, predicate?: (sprite: Sprite) => boolean): Nullable<PickingInfo[]>;
    /**
     * Render all child sprites
     */
    render(): void;
    private _customUpdate;
    /**
     * Rebuilds the manager (after a context lost, for eg)
     */
    rebuild(): void;
    /**
     * Release associated resources
     */
    dispose(): void;
    /**
     * Serializes the sprite manager to a JSON object
     * @param serializeTexture defines if the texture must be serialized as well
     * @returns the JSON object
     */
    serialize(serializeTexture?: boolean): any;
    /**
     * Parses a JSON object to create a new sprite manager.
     * @param parsedManager The JSON object to parse
     * @param scene The scene to create the sprite manager
     * @param rootUrl The root url to use to load external dependencies like texture
     * @returns the new sprite manager
     */
    static Parse(parsedManager: any, scene: Scene, rootUrl: string): SpriteManager;
    /**
     * Creates a sprite manager from a snippet saved in a remote file
     * @param name defines the name of the sprite manager to create (can be null or empty to use the one from the json data)
     * @param url defines the url to load from
     * @param scene defines the hosting scene
     * @param rootUrl defines the root URL to use to load textures and relative dependencies
     * @returns a promise that will resolve to the new sprite manager
     */
    static ParseFromFileAsync(name: Nullable<string>, url: string, scene: Scene, rootUrl?: string): Promise<SpriteManager>;
    /**
     * Creates a sprite manager from a snippet saved by the sprite editor
     * @param snippetId defines the snippet to load (can be set to _BLANK to create a default one)
     * @param scene defines the hosting scene
     * @param rootUrl defines the root URL to use to load textures and relative dependencies
     * @returns a promise that will resolve to the new sprite manager
     */
    static ParseFromSnippetAsync(snippetId: string, scene: Scene, rootUrl?: string): Promise<SpriteManager>;
    /**
     * Creates a sprite manager from a snippet saved by the sprite editor
     * @deprecated Please use ParseFromSnippetAsync instead
     * @param snippetId defines the snippet to load (can be set to _BLANK to create a default one)
     * @param scene defines the hosting scene
     * @param rootUrl defines the root URL to use to load textures and relative dependencies
     * @returns a promise that will resolve to the new sprite manager
     */
    static CreateFromSnippetAsync: typeof SpriteManager.ParseFromSnippetAsync;
}
export {};

}
declare module "babylonjs/Sprites/spriteMap" {
import { IDisposable, Scene } from "babylonjs/scene";
import { Vector2, Vector3 } from "babylonjs/Maths/math.vector";
import { Texture } from "babylonjs/Materials/Textures/texture";
import { RawTexture } from "babylonjs/Materials/Textures/rawTexture";
import { ISpriteJSONSprite, ISpriteJSONAtlas } from "babylonjs/Sprites/ISprites";
import "babylonjs/Shaders/spriteMap.fragment";
import "babylonjs/Shaders/spriteMap.vertex";
/**
 * Defines the basic options interface of a SpriteMap
 */
export interface ISpriteMapOptions {
    /**
     * Vector2 of the number of cells in the grid.
     */
    stageSize?: Vector2;
    /**
     * Vector2 of the size of the output plane in World Units.
     */
    outputSize?: Vector2;
    /**
     * Vector3 of the position of the output plane in World Units.
     */
    outputPosition?: Vector3;
    /**
     * Vector3 of the rotation of the output plane.
     */
    outputRotation?: Vector3;
    /**
     * number of layers that the system will reserve in resources.
     */
    layerCount?: number;
    /**
     * number of max animation frames a single cell will reserve in resources.
     */
    maxAnimationFrames?: number;
    /**
     * number cell index of the base tile when the system compiles.
     */
    baseTile?: number;
    /**
     * boolean flip the sprite after its been repositioned by the framing data.
     */
    flipU?: boolean;
    /**
     * Vector3 scalar of the global RGB values of the SpriteMap.
     */
    colorMultiply?: Vector3;
}
/**
 * Defines the IDisposable interface in order to be cleanable from resources.
 */
export interface ISpriteMap extends IDisposable {
    /**
     * String name of the SpriteMap.
     */
    name: string;
    /**
     * The JSON Array file from a https://www.codeandweb.com/texturepacker export.  Or similar structure.
     */
    atlasJSON: ISpriteJSONAtlas;
    /**
     * Texture of the SpriteMap.
     */
    spriteSheet: Texture;
    /**
     * The parameters to initialize the SpriteMap with.
     */
    options: ISpriteMapOptions;
}
/**
 * Class used to manage a grid restricted sprite deployment on an Output plane.
 */
export class SpriteMap implements ISpriteMap {
    /** The Name of the spriteMap */
    name: string;
    /** The JSON file with the frame and meta data */
    atlasJSON: ISpriteJSONAtlas;
    /** The systems Sprite Sheet Texture */
    spriteSheet: Texture;
    /** Arguments passed with the Constructor */
    options: ISpriteMapOptions;
    /** Public Sprite Storage array, parsed from atlasJSON */
    sprites: Array<ISpriteJSONSprite>;
    /** Returns the Number of Sprites in the System */
    get spriteCount(): number;
    /** Returns the Position of Output Plane*/
    get position(): Vector3;
    /** Returns the Position of Output Plane*/
    set position(v: Vector3);
    /** Returns the Rotation of Output Plane*/
    get rotation(): Vector3;
    /** Returns the Rotation of Output Plane*/
    set rotation(v: Vector3);
    /** Sets the AnimationMap*/
    get animationMap(): RawTexture;
    /** Sets the AnimationMap*/
    set animationMap(v: RawTexture);
    /** Scene that the SpriteMap was created in */
    private _scene;
    /** Texture Buffer of Float32 that holds tile frame data*/
    private _frameMap;
    /** Texture Buffers of Float32 that holds tileMap data*/
    private _tileMaps;
    /** Texture Buffer of Float32 that holds Animation Data*/
    private _animationMap;
    /** Custom ShaderMaterial Central to the System*/
    private _material;
    /** Custom ShaderMaterial Central to the System*/
    private _output;
    /** Systems Time Ticker*/
    private _time;
    /**
     * Creates a new SpriteMap
     * @param name defines the SpriteMaps Name
     * @param atlasJSON is the JSON file that controls the Sprites Frames and Meta
     * @param spriteSheet is the Texture that the Sprites are on.
     * @param options a basic deployment configuration
     * @param scene The Scene that the map is deployed on
     */
    constructor(name: string, atlasJSON: ISpriteJSONAtlas, spriteSheet: Texture, options: ISpriteMapOptions, scene: Scene);
    /**
     * Returns tileID location
     * @returns Vector2 the cell position ID
     */
    getTileID(): Vector2;
    /**
     * Gets the UV location of the mouse over the SpriteMap.
     * @returns Vector2 the UV position of the mouse interaction
     */
    getMousePosition(): Vector2;
    /**
     * Creates the "frame" texture Buffer
     * -------------------------------------
     * Structure of frames
     *  "filename": "Falling-Water-2.png",
     * "frame": {"x":69,"y":103,"w":24,"h":32},
     * "rotated": true,
     * "trimmed": true,
     * "spriteSourceSize": {"x":4,"y":0,"w":24,"h":32},
     * "sourceSize": {"w":32,"h":32}
     * @returns RawTexture of the frameMap
     */
    private _createFrameBuffer;
    /**
     * Creates the tileMap texture Buffer
     * @param buffer normally and array of numbers, or a false to generate from scratch
     * @param _layer indicates what layer for a logic trigger dealing with the baseTile.  The system uses this
     * @returns RawTexture of the tileMap
     */
    private _createTileBuffer;
    /**
     * Modifies the data of the tileMaps
     * @param _layer is the ID of the layer you want to edit on the SpriteMap
     * @param pos is the iVector2 Coordinates of the Tile
     * @param tile The SpriteIndex of the new Tile
     */
    changeTiles(_layer: number | undefined, pos: Vector2 | Vector2[], tile?: number): void;
    /**
     * Creates the animationMap texture Buffer
     * @param buffer normally and array of numbers, or a false to generate from scratch
     * @returns RawTexture of the animationMap
     */
    private _createTileAnimationBuffer;
    /**
     * Modifies the data of the animationMap
     * @param cellID is the Index of the Sprite
     * @param _frame is the target Animation frame
     * @param toCell is the Target Index of the next frame of the animation
     * @param time is a value between 0-1 that is the trigger for when the frame should change tiles
     * @param speed is a global scalar of the time variable on the map.
     */
    addAnimationToTile(cellID?: number, _frame?: number, toCell?: number, time?: number, speed?: number): void;
    /**
     * Exports the .tilemaps file
     */
    saveTileMaps(): void;
    /**
     * Imports the .tilemaps file
     * @param url of the .tilemaps file
     */
    loadTileMaps(url: string): void;
    /**
     * Release associated resources
     */
    dispose(): void;
}

}
declare module "babylonjs/Sprites/spritePackedManager" {
import { SpriteManager } from "babylonjs/Sprites/spriteManager";
import { Scene } from "babylonjs/scene";
/**
 * Class used to manage multiple sprites of different sizes on the same spritesheet
 * @see https://doc.babylonjs.com/features/featuresDeepDive/sprites
 */
export class SpritePackedManager extends SpriteManager {
    /** defines the packed manager's name */
    name: string;
    /**
     * Creates a new sprite manager from a packed sprite sheet
     * @param name defines the manager's name
     * @param imgUrl defines the sprite sheet url
     * @param capacity defines the maximum allowed number of sprites
     * @param scene defines the hosting scene
     * @param spriteJSON null otherwise a JSON object defining sprite sheet data
     * @param epsilon defines the epsilon value to align texture (0.01 by default)
     * @param samplingMode defines the sampling mode to use with spritesheet
     * @param fromPacked set to true; do not alter
     */
    constructor(
    /** defines the packed manager's name */
    name: string, imgUrl: string, capacity: number, scene: Scene, spriteJSON?: string | null, epsilon?: number, samplingMode?: number);
}

}
declare module "babylonjs/Sprites/spriteRenderer" {
import { Nullable } from "babylonjs/types";
import { IMatrixLike } from "babylonjs/Maths/math.like";
import { ThinEngine } from "babylonjs/Engines/thinEngine";
import { ThinSprite } from "babylonjs/Sprites/thinSprite";
import { ISize } from "babylonjs/Maths/math.size";
import { ThinTexture } from "babylonjs/Materials/Textures/thinTexture";
import { Scene } from "babylonjs/scene";
import "babylonjs/Engines/Extensions/engine.alpha";
import "babylonjs/Engines/Extensions/engine.dynamicBuffer";
import "babylonjs/Shaders/sprites.fragment";
import "babylonjs/Shaders/sprites.vertex";
/**
 * Class used to render sprites.
 *
 * It can be used either to render Sprites or ThinSprites with ThinEngine only.
 */
export class SpriteRenderer {
    /**
     * Defines the texture of the spritesheet
     */
    texture: Nullable<ThinTexture>;
    /**
     * Defines the default width of a cell in the spritesheet
     */
    cellWidth: number;
    /**
     * Defines the default height of a cell in the spritesheet
     */
    cellHeight: number;
    /**
     * Blend mode use to render the particle, it can be any of
     * the static Constants.ALPHA_x properties provided in this class.
     * Default value is Constants.ALPHA_COMBINE
     */
    blendMode: number;
    /**
     * Gets or sets a boolean indicating if alpha mode is automatically
     * reset.
     */
    autoResetAlpha: boolean;
    /**
     * Disables writing to the depth buffer when rendering the sprites.
     * It can be handy to disable depth writing when using textures without alpha channel
     * and setting some specific blend modes.
     */
    disableDepthWrite: boolean;
    /**
     * Gets or sets a boolean indicating if the manager must consider scene fog when rendering
     */
    fogEnabled: boolean;
    /**
     * Gets the capacity of the manager
     */
    get capacity(): number;
    private readonly _engine;
    private readonly _useVAO;
    private readonly _useInstancing;
    private readonly _scene;
    private readonly _capacity;
    private readonly _epsilon;
    private _vertexBufferSize;
    private _vertexData;
    private _buffer;
    private _vertexBuffers;
    private _spriteBuffer;
    private _indexBuffer;
    private _drawWrapperBase;
    private _drawWrapperFog;
    private _drawWrapperDepth;
    private _drawWrapperFogDepth;
    private _vertexArrayObject;
    /**
     * Creates a new sprite Renderer
     * @param engine defines the engine the renderer works with
     * @param capacity defines the maximum allowed number of sprites
     * @param epsilon defines the epsilon value to align texture (0.01 by default)
     * @param scene defines the hosting scene
     */
    constructor(engine: ThinEngine, capacity: number, epsilon?: number, scene?: Nullable<Scene>);
    /**
     * Render all child sprites
     * @param sprites defines the list of sprites to render
     * @param deltaTime defines the time since last frame
     * @param viewMatrix defines the viewMatrix to use to render the sprites
     * @param projectionMatrix defines the projectionMatrix to use to render the sprites
     * @param customSpriteUpdate defines a custom function to update the sprites data before they render
     */
    render(sprites: ThinSprite[], deltaTime: number, viewMatrix: IMatrixLike, projectionMatrix: IMatrixLike, customSpriteUpdate?: Nullable<(sprite: ThinSprite, baseSize: ISize) => void>): void;
    private _appendSpriteVertex;
    private _buildIndexBuffer;
    /**
     * Rebuilds the renderer (after a context lost, for eg)
     */
    rebuild(): void;
    /**
     * Release associated resources
     */
    dispose(): void;
}
export {};

}
declare module "babylonjs/Sprites/spriteSceneComponent" {
import { Nullable } from "babylonjs/types";
import { Observable } from "babylonjs/Misc/observable";
import { Scene } from "babylonjs/scene";
import { Sprite } from "babylonjs/Sprites/sprite";
import { ISpriteManager } from "babylonjs/Sprites/spriteManager";
import { Ray } from "babylonjs/Culling/ray";
import { Camera } from "babylonjs/Cameras/camera";
import { PickingInfo } from "babylonjs/Collisions/pickingInfo";
import { ISceneComponent } from "babylonjs/sceneComponent";
module "babylonjs/scene" {
    interface Scene {
        /** @internal */
        _pointerOverSprite: Nullable<Sprite>;
        /** @internal */
        _pickedDownSprite: Nullable<Sprite>;
        /** @internal */
        _tempSpritePickingRay: Nullable<Ray>;
        /**
         * All of the sprite managers added to this scene
         * @see https://doc.babylonjs.com/features/featuresDeepDive/sprites
         */
        spriteManagers: Array<ISpriteManager>;
        /**
         * An event triggered when sprites rendering is about to start
         * Note: This event can be trigger more than once per frame (because sprites can be rendered by render target textures as well)
         */
        onBeforeSpritesRenderingObservable: Observable<Scene>;
        /**
         * An event triggered when sprites rendering is done
         * Note: This event can be trigger more than once per frame (because sprites can be rendered by render target textures as well)
         */
        onAfterSpritesRenderingObservable: Observable<Scene>;
        /** @internal */
        _internalPickSprites(ray: Ray, predicate?: (sprite: Sprite) => boolean, fastCheck?: boolean, camera?: Camera): Nullable<PickingInfo>;
        /** Launch a ray to try to pick a sprite in the scene
         * @param x position on screen
         * @param y position on screen
         * @param predicate Predicate function used to determine eligible sprites. Can be set to null. In this case, a sprite must have isPickable set to true
         * @param fastCheck defines if the first intersection will be used (and not the closest)
         * @param camera camera to use for computing the picking ray. Can be set to null. In this case, the scene.activeCamera will be used
         * @returns a PickingInfo
         */
        pickSprite(x: number, y: number, predicate?: (sprite: Sprite) => boolean, fastCheck?: boolean, camera?: Camera): Nullable<PickingInfo>;
        /** Use the given ray to pick a sprite in the scene
         * @param ray The ray (in world space) to use to pick meshes
         * @param predicate Predicate function used to determine eligible sprites. Can be set to null. In this case, a sprite must have isPickable set to true
         * @param fastCheck defines if the first intersection will be used (and not the closest)
         * @param camera camera to use. Can be set to null. In this case, the scene.activeCamera will be used
         * @returns a PickingInfo
         */
        pickSpriteWithRay(ray: Ray, predicate?: (sprite: Sprite) => boolean, fastCheck?: boolean, camera?: Camera): Nullable<PickingInfo>;
        /** @internal */
        _internalMultiPickSprites(ray: Ray, predicate?: (sprite: Sprite) => boolean, camera?: Camera): Nullable<PickingInfo[]>;
        /** Launch a ray to try to pick sprites in the scene
         * @param x position on screen
         * @param y position on screen
         * @param predicate Predicate function used to determine eligible sprites. Can be set to null. In this case, a sprite must have isPickable set to true
         * @param camera camera to use for computing the picking ray. Can be set to null. In this case, the scene.activeCamera will be used
         * @returns a PickingInfo array
         */
        multiPickSprite(x: number, y: number, predicate?: (sprite: Sprite) => boolean, camera?: Camera): Nullable<PickingInfo[]>;
        /** Use the given ray to pick sprites in the scene
         * @param ray The ray (in world space) to use to pick meshes
         * @param predicate Predicate function used to determine eligible sprites. Can be set to null. In this case, a sprite must have isPickable set to true
         * @param camera camera to use. Can be set to null. In this case, the scene.activeCamera will be used
         * @returns a PickingInfo array
         */
        multiPickSpriteWithRay(ray: Ray, predicate?: (sprite: Sprite) => boolean, camera?: Camera): Nullable<PickingInfo[]>;
        /**
         * Force the sprite under the pointer
         * @param sprite defines the sprite to use
         */
        setPointerOverSprite(sprite: Nullable<Sprite>): void;
        /**
         * Gets the sprite under the pointer
         * @returns a Sprite or null if no sprite is under the pointer
         */
        getPointerOverSprite(): Nullable<Sprite>;
    }
}
/**
 * Defines the sprite scene component responsible to manage sprites
 * in a given scene.
 */
export class SpriteSceneComponent implements ISceneComponent {
    /**
     * The component name helpfull to identify the component in the list of scene components.
     */
    readonly name: string;
    /**
     * The scene the component belongs to.
     */
    scene: Scene;
    /** @internal */
    private _spritePredicate;
    /**
     * Creates a new instance of the component for the given scene
     * @param scene Defines the scene to register the component in
     */
    constructor(scene: Scene);
    /**
     * Registers the component in a given scene
     */
    register(): void;
    /**
     * Rebuilds the elements related to this component in case of
     * context lost for instance.
     */
    rebuild(): void;
    /**
     * Disposes the component and the associated resources.
     */
    dispose(): void;
    private _pickSpriteButKeepRay;
    private _pointerMove;
    private _pointerDown;
    private _pointerUp;
}

}
declare module "babylonjs/Sprites/thinSprite" {
import { IVector3Like, IColor4Like } from "babylonjs/Maths/math.like";
import { Nullable } from "babylonjs/types";
/**
 * ThinSprite Class used to represent a thin sprite
 * This is the base class for sprites but can also directly be used with ThinEngine
 * @see https://doc.babylonjs.com/features/featuresDeepDive/sprites
 */
export class ThinSprite {
    /** Gets or sets the cell index in the sprite sheet */
    cellIndex: number;
    /** Gets or sets the cell reference in the sprite sheet, uses sprite's filename when added to sprite sheet */
    cellRef: string;
    /** Gets or sets the current world position */
    position: IVector3Like;
    /** Gets or sets the main color */
    color: IColor4Like;
    /** Gets or sets the width */
    width: number;
    /** Gets or sets the height */
    height: number;
    /** Gets or sets rotation angle */
    angle: number;
    /** Gets or sets a boolean indicating if UV coordinates should be inverted in U axis */
    invertU: boolean;
    /** Gets or sets a boolean indicating if UV coordinates should be inverted in B axis */
    invertV: boolean;
    /** Gets or sets a boolean indicating if the sprite is visible (renderable). Default is true */
    isVisible: boolean;
    /**
     * Returns a boolean indicating if the animation is started
     */
    get animationStarted(): boolean;
    /** Gets the initial key for the animation (setting it will restart the animation)  */
    get fromIndex(): number;
    /** Gets or sets the end key for the animation (setting it will restart the animation)  */
    get toIndex(): number;
    /** Gets or sets a boolean indicating if the animation is looping (setting it will restart the animation)  */
    get loopAnimation(): boolean;
    /** Gets or sets the delay between cell changes (setting it will restart the animation)  */
    get delay(): number;
    /** @internal */
    _xOffset: number;
    /** @internal */
    _yOffset: number;
    /** @internal */
    _xSize: number;
    /** @internal */
    _ySize: number;
    private _animationStarted;
    protected _loopAnimation: boolean;
    protected _fromIndex: number;
    protected _toIndex: number;
    protected _delay: number;
    private _direction;
    private _time;
    private _onBaseAnimationEnd;
    /**
     * Creates a new Thin Sprite
     */
    constructor();
    /**
     * Starts an animation
     * @param from defines the initial key
     * @param to defines the end key
     * @param loop defines if the animation must loop
     * @param delay defines the start delay (in ms)
     * @param onAnimationEnd defines a callback for when the animation ends
     */
    playAnimation(from: number, to: number, loop: boolean, delay: number, onAnimationEnd: Nullable<() => void>): void;
    /** Stops current animation (if any) */
    stopAnimation(): void;
    /**
     * @internal
     */
    _animate(deltaTime: number): void;
}

}
declare module "babylonjs/States/alphaCullingState" {
import { Nullable } from "babylonjs/types";
/**
 * @internal
 **/
export class AlphaState {
    _blendFunctionParameters: Nullable<number>[];
    _blendEquationParameters: Nullable<number>[];
    _blendConstants: Nullable<number>[];
    _isBlendConstantsDirty: boolean;
    private _alphaBlend;
    private _isAlphaBlendDirty;
    private _isBlendFunctionParametersDirty;
    private _isBlendEquationParametersDirty;
    /**
     * Initializes the state.
     */
    constructor();
    get isDirty(): boolean;
    get alphaBlend(): boolean;
    set alphaBlend(value: boolean);
    setAlphaBlendConstants(r: number, g: number, b: number, a: number): void;
    setAlphaBlendFunctionParameters(value0: number, value1: number, value2: number, value3: number): void;
    setAlphaEquationParameters(rgb: number, alpha: number): void;
    reset(): void;
    apply(gl: WebGLRenderingContext): void;
}

}
declare module "babylonjs/States/depthCullingState" {
import { Nullable } from "babylonjs/types";
/**
 * @internal
 **/
export class DepthCullingState {
    protected _isDepthTestDirty: boolean;
    protected _isDepthMaskDirty: boolean;
    protected _isDepthFuncDirty: boolean;
    protected _isCullFaceDirty: boolean;
    protected _isCullDirty: boolean;
    protected _isZOffsetDirty: boolean;
    protected _isFrontFaceDirty: boolean;
    protected _depthTest: boolean;
    protected _depthMask: boolean;
    protected _depthFunc: Nullable<number>;
    protected _cull: Nullable<boolean>;
    protected _cullFace: Nullable<number>;
    protected _zOffset: number;
    protected _zOffsetUnits: number;
    protected _frontFace: Nullable<number>;
    /**
     * Initializes the state.
     * @param reset
     */
    constructor(reset?: boolean);
    get isDirty(): boolean;
    get zOffset(): number;
    set zOffset(value: number);
    get zOffsetUnits(): number;
    set zOffsetUnits(value: number);
    get cullFace(): Nullable<number>;
    set cullFace(value: Nullable<number>);
    get cull(): Nullable<boolean>;
    set cull(value: Nullable<boolean>);
    get depthFunc(): Nullable<number>;
    set depthFunc(value: Nullable<number>);
    get depthMask(): boolean;
    set depthMask(value: boolean);
    get depthTest(): boolean;
    set depthTest(value: boolean);
    get frontFace(): Nullable<number>;
    set frontFace(value: Nullable<number>);
    reset(): void;
    apply(gl: WebGLRenderingContext): void;
}

}
declare module "babylonjs/States/index" {
export * from "babylonjs/States/alphaCullingState";
export * from "babylonjs/States/depthCullingState";
export * from "babylonjs/States/stencilState";
export * from "babylonjs/States/stencilStateComposer";

}
declare module "babylonjs/States/IStencilState" {
/** @internal */
export interface IStencilState {
    enabled: boolean;
    mask: number;
    func: number;
    funcRef: number;
    funcMask: number;
    opStencilDepthPass: number;
    opStencilFail: number;
    opDepthFail: number;
    reset(): void;
}

}
declare module "babylonjs/States/stencilState" {
import { IStencilState } from "babylonjs/States/IStencilState";
/**
 * @internal
 **/
export class StencilState implements IStencilState {
    /** Passed to depthFunction or stencilFunction to specify depth or stencil tests will always pass. i.e. Pixels will be drawn in the order they are drawn */
    static readonly ALWAYS: number;
    /** Passed to stencilOperation to specify that stencil value must be kept */
    static readonly KEEP: number;
    /** Passed to stencilOperation to specify that stencil value must be replaced */
    static readonly REPLACE: number;
    constructor();
    reset(): void;
    func: number;
    get stencilFunc(): number;
    set stencilFunc(value: number);
    funcRef: number;
    get stencilFuncRef(): number;
    set stencilFuncRef(value: number);
    funcMask: number;
    get stencilFuncMask(): number;
    set stencilFuncMask(value: number);
    opStencilFail: number;
    get stencilOpStencilFail(): number;
    set stencilOpStencilFail(value: number);
    opDepthFail: number;
    get stencilOpDepthFail(): number;
    set stencilOpDepthFail(value: number);
    opStencilDepthPass: number;
    get stencilOpStencilDepthPass(): number;
    set stencilOpStencilDepthPass(value: number);
    mask: number;
    get stencilMask(): number;
    set stencilMask(value: number);
    enabled: boolean;
    get stencilTest(): boolean;
    set stencilTest(value: boolean);
}

}
declare module "babylonjs/States/stencilStateComposer" {
import { IStencilState } from "babylonjs/States/IStencilState";
/**
 * @internal
 **/
export class StencilStateComposer {
    protected _isStencilTestDirty: boolean;
    protected _isStencilMaskDirty: boolean;
    protected _isStencilFuncDirty: boolean;
    protected _isStencilOpDirty: boolean;
    protected _enabled: boolean;
    protected _mask: number;
    protected _func: number;
    protected _funcRef: number;
    protected _funcMask: number;
    protected _opStencilFail: number;
    protected _opDepthFail: number;
    protected _opStencilDepthPass: number;
    stencilGlobal: IStencilState;
    stencilMaterial: IStencilState | undefined;
    useStencilGlobalOnly: boolean;
    get isDirty(): boolean;
    get func(): number;
    set func(value: number);
    get funcRef(): number;
    set funcRef(value: number);
    get funcMask(): number;
    set funcMask(value: number);
    get opStencilFail(): number;
    set opStencilFail(value: number);
    get opDepthFail(): number;
    set opDepthFail(value: number);
    get opStencilDepthPass(): number;
    set opStencilDepthPass(value: number);
    get mask(): number;
    set mask(value: number);
    get enabled(): boolean;
    set enabled(value: boolean);
    constructor(reset?: boolean);
    reset(): void;
    apply(gl?: WebGLRenderingContext): void;
}

}
declare module "babylonjs/types" {
/** Alias type for value that can be null */
export type Nullable<T> = T | null;
/**
 * Alias type for number that are floats
 * @ignorenaming
 */
export type float = number;
/**
 * Alias type for number that are doubles.
 * @ignorenaming
 */
export type double = number;
/**
 * Alias type for number that are integer
 * @ignorenaming
 */
export type int = number;
/** Alias type for number array or Float32Array */
export type FloatArray = number[] | Float32Array;
/** Alias type for number array or Float32Array or Int32Array or Uint32Array or Uint16Array */
export type IndicesArray = number[] | Int32Array | Uint32Array | Uint16Array;
/**
 * Alias for types that can be used by a Buffer or VertexBuffer.
 */
export type DataArray = number[] | ArrayBuffer | ArrayBufferView;
/**
 * Alias type for primitive types
 * @ignorenaming
 */
type Primitive = undefined | null | boolean | string | number | Function | Element;
/**
 * Type modifier to make all the properties of an object Readonly
 */
export type Immutable<T> = T extends Primitive ? T : T extends Array<infer U> ? ReadonlyArray<U> : DeepImmutable<T>;
/**
 * Type modifier to make all the properties of an object Readonly recursively
 */
export type DeepImmutable<T> = T extends Primitive ? T : T extends Array<infer U> ? DeepImmutableArray<U> : DeepImmutableObject<T>;
/**
 * Type modifier to make object properties readonly.
 */
export type DeepImmutableObject<T> = {
    readonly [K in keyof T]: DeepImmutable<T[K]>;
};
/** @internal */
interface DeepImmutableArray<T> extends ReadonlyArray<DeepImmutable<T>> {
}
export {};
/** @internal */

}
declare module "babylonjs/XR/features/index" {
export * from "babylonjs/XR/features/WebXRAbstractFeature";
export * from "babylonjs/XR/features/WebXRHitTestLegacy";
export * from "babylonjs/XR/features/WebXRAnchorSystem";
export * from "babylonjs/XR/features/WebXRPlaneDetector";
export * from "babylonjs/XR/features/WebXRBackgroundRemover";
export * from "babylonjs/XR/features/WebXRControllerTeleportation";
export * from "babylonjs/XR/features/WebXRControllerPointerSelection";
export * from "babylonjs/XR/features/WebXRControllerPhysics";
export * from "babylonjs/XR/features/WebXRHitTest";
export * from "babylonjs/XR/features/WebXRFeaturePointSystem";
export * from "babylonjs/XR/features/WebXRHandTracking";
export * from "babylonjs/XR/features/WebXRMeshDetector";
export * from "babylonjs/XR/features/WebXRImageTracking";
export * from "babylonjs/XR/features/WebXRNearInteraction";
export * from "babylonjs/XR/features/WebXRDOMOverlay";
export * from "babylonjs/XR/features/WebXRControllerMovement";
export * from "babylonjs/XR/features/WebXRLightEstimation";
export * from "babylonjs/XR/features/WebXREyeTracking";
export * from "babylonjs/XR/features/WebXRWalkingLocomotion";
export * from "babylonjs/XR/features/WebXRLayers";

}
declare module "babylonjs/XR/features/WebXRAbstractFeature" {
import { IWebXRFeature } from "babylonjs/XR/webXRFeaturesManager";
import { Observable, EventState } from "babylonjs/Misc/observable";
import { WebXRSessionManager } from "babylonjs/XR/webXRSessionManager";
/**
 * This is the base class for all WebXR features.
 * Since most features require almost the same resources and callbacks, this class can be used to simplify the development
 * Note that since the features manager is using the `IWebXRFeature` you are in no way obligated to use this class
 */
export abstract class WebXRAbstractFeature implements IWebXRFeature {
    protected _xrSessionManager: WebXRSessionManager;
    private _attached;
    private _removeOnDetach;
    /**
     * Is this feature disposed?
     */
    isDisposed: boolean;
    /**
     * Should auto-attach be disabled?
     */
    disableAutoAttach: boolean;
    /**
     * The name of the native xr feature name (like anchor, hit-test, or hand-tracking)
     */
    xrNativeFeatureName: string;
    /**
     * Construct a new (abstract) WebXR feature
     * @param _xrSessionManager the xr session manager for this feature
     */
    constructor(_xrSessionManager: WebXRSessionManager);
    /**
     * Is this feature attached
     */
    get attached(): boolean;
    /**
     * attach this feature
     *
     * @param force should attachment be forced (even when already attached)
     * @returns true if successful, false is failed or already attached
     */
    attach(force?: boolean): boolean;
    /**
     * detach this feature.
     *
     * @returns true if successful, false if failed or already detached
     */
    detach(): boolean;
    /**
     * Dispose this feature and all of the resources attached
     */
    dispose(): void;
    /**
     * This function will be executed during before enabling the feature and can be used to not-allow enabling it.
     * Note that at this point the session has NOT started, so this is purely checking if the browser supports it
     *
     * @returns whether or not the feature is compatible in this environment
     */
    isCompatible(): boolean;
    /**
     * This is used to register callbacks that will automatically be removed when detach is called.
     * @param observable the observable to which the observer will be attached
     * @param callback the callback to register
     */
    protected _addNewAttachObserver<T>(observable: Observable<T>, callback: (eventData: T, eventState: EventState) => void): void;
    /**
     * Code in this function will be executed on each xrFrame received from the browser.
     * This function will not execute after the feature is detached.
     * @param _xrFrame the current frame
     */
    protected abstract _onXRFrame(_xrFrame: XRFrame): void;
}

}
declare module "babylonjs/XR/features/WebXRAnchorSystem" {
import { WebXRSessionManager } from "babylonjs/XR/webXRSessionManager";
import { Observable } from "babylonjs/Misc/observable";
import { Matrix, Vector3, Quaternion } from "babylonjs/Maths/math.vector";
import { TransformNode } from "babylonjs/Meshes/transformNode";
import { WebXRAbstractFeature } from "babylonjs/XR/features/WebXRAbstractFeature";
import { IWebXRHitResult } from "babylonjs/XR/features/WebXRHitTest";
/**
 * Configuration options of the anchor system
 */
export interface IWebXRAnchorSystemOptions {
    /**
     * a node that will be used to convert local to world coordinates
     */
    worldParentNode?: TransformNode;
    /**
     * If set to true a reference of the created anchors will be kept until the next session starts
     * If not defined, anchors will be removed from the array when the feature is detached or the session ended.
     */
    doNotRemoveAnchorsOnSessionEnded?: boolean;
}
/**
 * A babylon container for an XR Anchor
 */
export interface IWebXRAnchor {
    /**
     * A babylon-assigned ID for this anchor
     */
    id: number;
    /**
     * Transformation matrix to apply to an object attached to this anchor
     */
    transformationMatrix: Matrix;
    /**
     * The native anchor object
     */
    xrAnchor: XRAnchor;
    /**
     * if defined, this object will be constantly updated by the anchor's position and rotation
     */
    attachedNode?: TransformNode;
    /**
     * Remove this anchor from the scene
     */
    remove(): void;
}
/**
 * An implementation of the anchor system for WebXR.
 * For further information see https://github.com/immersive-web/anchors/
 */
export class WebXRAnchorSystem extends WebXRAbstractFeature {
    private _options;
    private _lastFrameDetected;
    private _trackedAnchors;
    private _referenceSpaceForFrameAnchors;
    private _futureAnchors;
    /**
     * The module's name
     */
    static readonly Name: string;
    /**
     * The (Babylon) version of this module.
     * This is an integer representing the implementation version.
     * This number does not correspond to the WebXR specs version
     */
    static readonly Version: number;
    /**
     * Observers registered here will be executed when a new anchor was added to the session
     */
    onAnchorAddedObservable: Observable<IWebXRAnchor>;
    /**
     * Observers registered here will be executed when an anchor was removed from the session
     */
    onAnchorRemovedObservable: Observable<IWebXRAnchor>;
    /**
     * Observers registered here will be executed when an existing anchor updates
     * This can execute N times every frame
     */
    onAnchorUpdatedObservable: Observable<IWebXRAnchor>;
    /**
     * Set the reference space to use for anchor creation, when not using a hit test.
     * Will default to the session's reference space if not defined
     */
    set referenceSpaceForFrameAnchors(referenceSpace: XRReferenceSpace);
    /**
     * constructs a new anchor system
     * @param _xrSessionManager an instance of WebXRSessionManager
     * @param _options configuration object for this feature
     */
    constructor(_xrSessionManager: WebXRSessionManager, _options?: IWebXRAnchorSystemOptions);
    private _tmpVector;
    private _tmpQuaternion;
    private _populateTmpTransformation;
    /**
     * Create a new anchor point using a hit test result at a specific point in the scene
     * An anchor is tracked only after it is added to the trackerAnchors in xrFrame. The promise returned here does not yet guaranty that.
     * Use onAnchorAddedObservable to get newly added anchors if you require tracking guaranty.
     *
     * @param hitTestResult The hit test result to use for this anchor creation
     * @param position an optional position offset for this anchor
     * @param rotationQuaternion an optional rotation offset for this anchor
     * @returns A promise that fulfills when babylon has created the corresponding WebXRAnchor object and tracking has begun
     */
    addAnchorPointUsingHitTestResultAsync(hitTestResult: IWebXRHitResult, position?: Vector3, rotationQuaternion?: Quaternion): Promise<IWebXRAnchor>;
    /**
     * Add a new anchor at a specific position and rotation
     * This function will add a new anchor per default in the next available frame. Unless forced, the createAnchor function
     * will be called in the next xrFrame loop to make sure that the anchor can be created correctly.
     * An anchor is tracked only after it is added to the trackerAnchors in xrFrame. The promise returned here does not yet guaranty that.
     * Use onAnchorAddedObservable to get newly added anchors if you require tracking guaranty.
     *
     * @param position the position in which to add an anchor
     * @param rotationQuaternion an optional rotation for the anchor transformation
     * @param forceCreateInCurrentFrame force the creation of this anchor in the current frame. Must be called inside xrFrame loop!
     * @returns A promise that fulfills when babylon has created the corresponding WebXRAnchor object and tracking has begun
     */
    addAnchorAtPositionAndRotationAsync(position: Vector3, rotationQuaternion?: Quaternion, forceCreateInCurrentFrame?: boolean): Promise<IWebXRAnchor>;
    /**
     * Get the list of anchors currently being tracked by the system
     */
    get anchors(): IWebXRAnchor[];
    /**
     * detach this feature.
     * Will usually be called by the features manager
     *
     * @returns true if successful.
     */
    detach(): boolean;
    /**
     * Dispose this feature and all of the resources attached
     */
    dispose(): void;
    protected _onXRFrame(frame: XRFrame): void;
    /**
     * avoiding using Array.find for global support.
     * @param xrAnchor the plane to find in the array
     */
    private _findIndexInAnchorArray;
    private _updateAnchorWithXRFrame;
    private _createAnchorAtTransformation;
}

}
declare module "babylonjs/XR/features/WebXRBackgroundRemover" {
import { WebXRSessionManager } from "babylonjs/XR/webXRSessionManager";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { Observable } from "babylonjs/Misc/observable";
import { WebXRAbstractFeature } from "babylonjs/XR/features/WebXRAbstractFeature";
/**
 * Options interface for the background remover plugin
 */
export interface IWebXRBackgroundRemoverOptions {
    /**
     * Further background meshes to disable when entering AR
     */
    backgroundMeshes?: AbstractMesh[];
    /**
     * flags to configure the removal of the environment helper.
     * If not set, the entire background will be removed. If set, flags should be set as well.
     */
    environmentHelperRemovalFlags?: {
        /**
         * Should the skybox be removed (default false)
         */
        skyBox?: boolean;
        /**
         * Should the ground be removed (default false)
         */
        ground?: boolean;
    };
    /**
     * don't disable the environment helper
     */
    ignoreEnvironmentHelper?: boolean;
}
/**
 * A module that will automatically disable background meshes when entering AR and will enable them when leaving AR.
 */
export class WebXRBackgroundRemover extends WebXRAbstractFeature {
    /**
     * read-only options to be used in this module
     */
    readonly options: IWebXRBackgroundRemoverOptions;
    /**
     * The module's name
     */
    static readonly Name: string;
    /**
     * The (Babylon) version of this module.
     * This is an integer representing the implementation version.
     * This number does not correspond to the WebXR specs version
     */
    static readonly Version: number;
    /**
     * registered observers will be triggered when the background state changes
     */
    onBackgroundStateChangedObservable: Observable<boolean>;
    /**
     * constructs a new background remover module
     * @param _xrSessionManager the session manager for this module
     * @param options read-only options to be used in this module
     */
    constructor(_xrSessionManager: WebXRSessionManager, 
    /**
     * read-only options to be used in this module
     */
    options?: IWebXRBackgroundRemoverOptions);
    /**
     * attach this feature
     * Will usually be called by the features manager
     *
     * @returns true if successful.
     */
    attach(): boolean;
    /**
     * detach this feature.
     * Will usually be called by the features manager
     *
     * @returns true if successful.
     */
    detach(): boolean;
    /**
     * Dispose this feature and all of the resources attached
     */
    dispose(): void;
    protected _onXRFrame(_xrFrame: XRFrame): void;
    private _setBackgroundState;
}

}
declare module "babylonjs/XR/features/WebXRControllerMovement" {
import { WebXRSessionManager } from "babylonjs/XR/webXRSessionManager";
import { Nullable } from "babylonjs/types";
import { WebXRInput } from "babylonjs/XR/webXRInput";
import { WebXRInputSource } from "babylonjs/XR/webXRInputSource";
import { IWebXRMotionControllerAxesValue, IWebXRMotionControllerComponentChangesValues } from "babylonjs/XR/motionController/webXRControllerComponent";
import { WebXRControllerComponent } from "babylonjs/XR/motionController/webXRControllerComponent";
import { Quaternion } from "babylonjs/Maths/math.vector";
import { WebXRAbstractFeature } from "babylonjs/XR/features/WebXRAbstractFeature";
import { MotionControllerComponentType } from "babylonjs/XR/motionController/webXRAbstractMotionController";
/**
 * The options container for the controller movement module
 */
export interface IWebXRControllerMovementOptions {
    /**
     * Override default behaviour and provide your own movement controls
     */
    customRegistrationConfigurations?: WebXRControllerMovementRegistrationConfiguration[];
    /**
     * Is movement enabled
     */
    movementEnabled?: boolean;
    /**
     * Camera direction follows view pose and movement by default will move independently of the viewer's pose.
     */
    movementOrientationFollowsViewerPose: boolean;
    /**
     * Movement speed factor (default is 1.0)
     */
    movementSpeed?: number;
    /**
     * Minimum threshold the controller's thumbstick/touchpad must pass before being recognized for movement (avoids jitter/unintentional movement)
     */
    movementThreshold?: number;
    /**
     * Is rotation enabled
     */
    rotationEnabled?: boolean;
    /**
     * Minimum threshold the controller's thumstick/touchpad must pass before being recognized for rotation (avoids jitter/unintentional rotation)
     */
    rotationThreshold?: number;
    /**
     * Movement speed factor (default is 1.0)
     */
    rotationSpeed?: number;
    /**
     * Babylon XR Input class for controller
     */
    xrInput: WebXRInput;
}
/**
 * Feature context is used in handlers and on each XR frame to control the camera movement/direction.
 */
export type WebXRControllerMovementFeatureContext = {
    movementEnabled: boolean;
    movementOrientationFollowsViewerPose: boolean;
    movementSpeed: number;
    movementThreshold: number;
    rotationEnabled: boolean;
    rotationSpeed: number;
    rotationThreshold: number;
};
/**
 * Current state of Movements shared across components and handlers.
 */
export type WebXRControllerMovementState = {
    moveX: number;
    moveY: number;
    rotateX: number;
    rotateY: number;
};
/**
 * Button of Axis Handler must be specified.
 */
export type WebXRControllerMovementRegistrationConfiguration = {
    /**
     * handlers are filtered to these types only
     */
    allowedComponentTypes?: MotionControllerComponentType[];
    /**
     * For registering movement to specific hand only.  Useful if your app has a "main hand" and "off hand" for determining the functionality of a controller.
     */
    forceHandedness?: XRHandedness;
    /**
     * For main component only (useful for buttons and may not trigger axis changes).
     */
    mainComponentOnly?: boolean;
    /**
     * Additional predicate to apply to controllers to restrict a handler being added.
     */
    componentSelectionPredicate?: (xrController: WebXRInputSource) => Nullable<WebXRControllerComponent>;
} & ({
    /**
     * Called when axis changes occur.
     */
    axisChangedHandler: (axes: IWebXRMotionControllerAxesValue, movementState: WebXRControllerMovementState, featureContext: WebXRControllerMovementFeatureContext, xrInput: WebXRInput) => void;
} | {
    /**
     * Called when the button state changes.
     */
    buttonChangedhandler: (pressed: IWebXRMotionControllerComponentChangesValues<boolean>, movementState: WebXRControllerMovementState, featureContext: WebXRControllerMovementFeatureContext, xrInput: WebXRInput) => void;
});
/**
 * This is a movement feature to be used with WebXR-enabled motion controllers.
 * When enabled and attached, the feature will allow a user to move around and rotate in the scene using
 * the input of the attached controllers.
 */
export class WebXRControllerMovement extends WebXRAbstractFeature {
    private _controllers;
    private _currentRegistrationConfigurations;
    private _featureContext;
    private _movementDirection;
    private _movementState;
    private _xrInput;
    private _tmpRotationMatrix;
    private _tmpTranslationDirection;
    private _tmpMovementTranslation;
    /**
     * The module's name
     */
    static readonly Name: string;
    /**
     * Standard controller configurations.
     */
    static readonly REGISTRATIONS: {
        [key: string]: WebXRControllerMovementRegistrationConfiguration[];
    };
    /**
     * The (Babylon) version of this module.
     * This is an integer representing the implementation version.
     * This number does not correspond to the webxr specs version
     */
    static readonly Version: number;
    /**
     * Current movement direction.  Will be null before XR Frames have been processed.
     */
    get movementDirection(): Nullable<Quaternion>;
    /**
     * Is movement enabled
     */
    get movementEnabled(): boolean;
    /**
     * Sets whether movement is enabled or not
     * @param enabled is movement enabled
     */
    set movementEnabled(enabled: boolean);
    /**
     * If movement follows viewer pose
     */
    get movementOrientationFollowsViewerPose(): boolean;
    /**
     * Sets whether movement follows viewer pose
     * @param followsPose is movement should follow viewer pose
     */
    set movementOrientationFollowsViewerPose(followsPose: boolean);
    /**
     * Gets movement speed
     */
    get movementSpeed(): number;
    /**
     * Sets movement speed
     * @param movementSpeed movement speed
     */
    set movementSpeed(movementSpeed: number);
    /**
     * Gets minimum threshold the controller's thumbstick/touchpad must pass before being recognized for movement (avoids jitter/unintentional movement)
     */
    get movementThreshold(): number;
    /**
     * Sets minimum threshold the controller's thumbstick/touchpad must pass before being recognized for movement (avoids jitter/unintentional movement)
     * @param movementThreshold new threshold
     */
    set movementThreshold(movementThreshold: number);
    /**
     * Is rotation enabled
     */
    get rotationEnabled(): boolean;
    /**
     * Sets whether rotation is enabled or not
     * @param enabled is rotation enabled
     */
    set rotationEnabled(enabled: boolean);
    /**
     * Gets rotation speed factor
     */
    get rotationSpeed(): number;
    /**
     * Sets rotation speed factor (1.0 is default)
     * @param rotationSpeed new rotation speed factor
     */
    set rotationSpeed(rotationSpeed: number);
    /**
     * Gets minimum threshold the controller's thumbstick/touchpad must pass before being recognized for rotation (avoids jitter/unintentional rotation)
     */
    get rotationThreshold(): number;
    /**
     * Sets minimum threshold the controller's thumbstick/touchpad must pass before being recognized for rotation (avoids jitter/unintentional rotation)
     * @param threshold new threshold
     */
    set rotationThreshold(threshold: number);
    /**
     * constructs a new movement controller system
     * @param _xrSessionManager an instance of WebXRSessionManager
     * @param options configuration object for this feature
     */
    constructor(_xrSessionManager: WebXRSessionManager, options: IWebXRControllerMovementOptions);
    attach(): boolean;
    detach(): boolean;
    /**
     * Occurs on every XR frame.
     * @param _xrFrame
     */
    protected _onXRFrame(_xrFrame: XRFrame): void;
    private _attachController;
    private _detachController;
}

}
declare module "babylonjs/XR/features/WebXRControllerPhysics" {
import { WebXRAbstractFeature } from "babylonjs/XR/features/WebXRAbstractFeature";
import { WebXRInputSource } from "babylonjs/XR/webXRInputSource";
import { PhysicsImpostor } from "babylonjs/Physics/v1/physicsImpostor";
import { WebXRInput } from "babylonjs/XR/webXRInput";
import { WebXRSessionManager } from "babylonjs/XR/webXRSessionManager";
import { Nullable } from "babylonjs/types";
/**
 * Options for the controller physics feature
 */
export class IWebXRControllerPhysicsOptions {
    /**
     * Should the headset get its own impostor
     */
    enableHeadsetImpostor?: boolean;
    /**
     * Optional parameters for the headset impostor
     */
    headsetImpostorParams?: {
        /**
         * The type of impostor to create. Default is sphere
         */
        impostorType: number;
        /**
         * the size of the impostor. Defaults to 10cm
         */
        impostorSize?: number | {
            width: number;
            height: number;
            depth: number;
        };
        /**
         * Friction definitions
         */
        friction?: number;
        /**
         * Restitution
         */
        restitution?: number;
    };
    /**
     * The physics properties of the future impostors
     */
    physicsProperties?: {
        /**
         * If set to true, a mesh impostor will be created when the controller mesh was loaded
         * Note that this requires a physics engine that supports mesh impostors!
         */
        useControllerMesh?: boolean;
        /**
         * The type of impostor to create. Default is sphere
         */
        impostorType?: number;
        /**
         * the size of the impostor. Defaults to 10cm
         */
        impostorSize?: number | {
            width: number;
            height: number;
            depth: number;
        };
        /**
         * Friction definitions
         */
        friction?: number;
        /**
         * Restitution
         */
        restitution?: number;
    };
    /**
     * the xr input to use with this pointer selection
     */
    xrInput: WebXRInput;
}
/**
 * Add physics impostor to your webxr controllers,
 * including naive calculation of their linear and angular velocity
 */
export class WebXRControllerPhysics extends WebXRAbstractFeature {
    private readonly _options;
    private _attachController;
    private _createPhysicsImpostor;
    private _controllers;
    private _debugMode;
    private _delta;
    private _headsetImpostor?;
    private _headsetMesh?;
    private _lastTimestamp;
    private _tmpQuaternion;
    private _tmpVector;
    /**
     * The module's name
     */
    static readonly Name: string;
    /**
     * The (Babylon) version of this module.
     * This is an integer representing the implementation version.
     * This number does not correspond to the webxr specs version
     */
    static readonly Version: number;
    /**
     * Construct a new Controller Physics Feature
     * @param _xrSessionManager the corresponding xr session manager
     * @param _options options to create this feature with
     */
    constructor(_xrSessionManager: WebXRSessionManager, _options: IWebXRControllerPhysicsOptions);
    /**
     * @internal
     * enable debugging - will show console outputs and the impostor mesh
     */
    _enablePhysicsDebug(): void;
    /**
     * Manually add a controller (if no xrInput was provided or physics engine was not enabled)
     * @param xrController the controller to add
     */
    addController(xrController: WebXRInputSource): void;
    /**
     * attach this feature
     * Will usually be called by the features manager
     *
     * @returns true if successful.
     */
    attach(): boolean;
    /**
     * detach this feature.
     * Will usually be called by the features manager
     *
     * @returns true if successful.
     */
    detach(): boolean;
    /**
     * Get the headset impostor, if enabled
     * @returns the impostor
     */
    getHeadsetImpostor(): PhysicsImpostor | undefined;
    /**
     * Get the physics impostor of a specific controller.
     * The impostor is not attached to a mesh because a mesh for each controller is not obligatory
     * @param controller the controller or the controller id of which to get the impostor
     * @returns the impostor or null
     */
    getImpostorForController(controller: WebXRInputSource | string): Nullable<PhysicsImpostor>;
    /**
     * Update the physics properties provided in the constructor
     * @param newProperties the new properties object
     * @param newProperties.impostorType
     * @param newProperties.impostorSize
     * @param newProperties.friction
     * @param newProperties.restitution
     */
    setPhysicsProperties(newProperties: {
        impostorType?: number;
        impostorSize?: number | {
            width: number;
            height: number;
            depth: number;
        };
        friction?: number;
        restitution?: number;
    }): void;
    protected _onXRFrame(_xrFrame: any): void;
    private _detachController;
}

}
declare module "babylonjs/XR/features/WebXRControllerPointerSelection" {
import { WebXRSessionManager } from "babylonjs/XR/webXRSessionManager";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { WebXRInput } from "babylonjs/XR/webXRInput";
import { WebXRInputSource } from "babylonjs/XR/webXRInputSource";
import { Scene } from "babylonjs/scene";
import { Nullable } from "babylonjs/types";
import { Color3 } from "babylonjs/Maths/math.color";
import { WebXRAbstractFeature } from "babylonjs/XR/features/WebXRAbstractFeature";
import { WebXRCamera } from "babylonjs/XR/webXRCamera";
import { Mesh } from "babylonjs/Meshes/mesh";
/**
 * Options interface for the pointer selection module
 */
export interface IWebXRControllerPointerSelectionOptions {
    /**
     * if provided, this scene will be used to render meshes.
     */
    customUtilityLayerScene?: Scene;
    /**
     * Disable the pointer up event when the xr controller in screen and gaze mode is disposed (meaning - when the user removed the finger from the screen)
     * If not disabled, the last picked point will be used to execute a pointer up event
     * If disabled, pointer up event will be triggered right after the pointer down event.
     * Used in screen and gaze target ray mode only
     */
    disablePointerUpOnTouchOut: boolean;
    /**
     * For gaze mode for tracked-pointer / controllers (time to select instead of button press)
     */
    forceGazeMode: boolean;
    /**
     * Factor to be applied to the pointer-moved function in the gaze mode. How sensitive should the gaze mode be when checking if the pointer moved
     * to start a new countdown to the pointer down event.
     * Defaults to 1.
     */
    gazeModePointerMovedFactor?: number;
    /**
     * Different button type to use instead of the main component
     */
    overrideButtonId?: string;
    /**
     *  use this rendering group id for the meshes (optional)
     */
    renderingGroupId?: number;
    /**
     * The amount of time in milliseconds it takes between pick found something to a pointer down event.
     * Used in gaze modes. Tracked pointer uses the trigger, screen uses touch events
     * 3000 means 3 seconds between pointing at something and selecting it
     */
    timeToSelect?: number;
    /**
     * Should meshes created here be added to a utility layer or the main scene
     */
    useUtilityLayer?: boolean;
    /**
     * Optional WebXR camera to be used for gaze selection
     */
    gazeCamera?: WebXRCamera;
    /**
     * the xr input to use with this pointer selection
     */
    xrInput: WebXRInput;
    /**
     * Should the scene pointerX and pointerY update be disabled
     * This is required for fullscreen AR GUI, but might slow down other experiences.
     * Disable in VR, if not needed.
     * The first rig camera (left eye) will be used to calculate the projection
     */
    disableScenePointerVectorUpdate: boolean;
    /**
     * Enable pointer selection on all controllers instead of switching between them
     */
    enablePointerSelectionOnAllControllers?: boolean;
    /**
     * The preferred hand to give the pointer selection to. This will be prioritized when the controller initialize.
     * If switch is enabled, it will still allow the user to switch between the different controllers
     */
    preferredHandedness?: XRHandedness;
    /**
     * Disable switching the pointer selection from one controller to the other.
     * If the preferred hand is set it will be fixed on this hand, and if not it will be fixed on the first controller added to the scene
     */
    disableSwitchOnClick?: boolean;
    /**
     * The maximum distance of the pointer selection feature. Defaults to 100.
     */
    maxPointerDistance?: number;
    /**
     * A function that will be called when a new selection mesh is generated.
     * This function should return a mesh that will be used as the selection mesh.
     * The default is a torus with a 0.01 diameter and 0.0075 thickness .
     */
    customSelectionMeshGenerator?: () => Mesh;
    /**
     * A function that will be called when a new laser pointer mesh is generated.
     * This function should return a mesh that will be used as the laser pointer mesh.
     * The height (y) of the mesh must be 1.
     */
    customLasterPointerMeshGenerator?: () => AbstractMesh;
}
/**
 * A module that will enable pointer selection for motion controllers of XR Input Sources
 */
export class WebXRControllerPointerSelection extends WebXRAbstractFeature {
    private readonly _options;
    private static _IdCounter;
    private _attachController;
    private _controllers;
    private _scene;
    private _tmpVectorForPickCompare;
    private _attachedController;
    /**
     * The module's name
     */
    static readonly Name: string;
    /**
     * The (Babylon) version of this module.
     * This is an integer representing the implementation version.
     * This number does not correspond to the WebXR specs version
     */
    static readonly Version: number;
    /**
     * Disable lighting on the laser pointer (so it will always be visible)
     */
    disablePointerLighting: boolean;
    /**
     * Disable lighting on the selection mesh (so it will always be visible)
     */
    disableSelectionMeshLighting: boolean;
    /**
     * Should the laser pointer be displayed
     */
    displayLaserPointer: boolean;
    /**
     * Should the selection mesh be displayed (The ring at the end of the laser pointer)
     */
    displaySelectionMesh: boolean;
    /**
     * This color will be set to the laser pointer when selection is triggered
     */
    laserPointerPickedColor: Color3;
    /**
     * Default color of the laser pointer
     */
    laserPointerDefaultColor: Color3;
    /**
     * default color of the selection ring
     */
    selectionMeshDefaultColor: Color3;
    /**
     * This color will be applied to the selection ring when selection is triggered
     */
    selectionMeshPickedColor: Color3;
    /**
     * Optional filter to be used for ray selection.  This predicate shares behavior with
     * scene.pointerMovePredicate which takes priority if it is also assigned.
     */
    raySelectionPredicate: (mesh: AbstractMesh) => boolean;
    /**
     * constructs a new background remover module
     * @param _xrSessionManager the session manager for this module
     * @param _options read-only options to be used in this module
     */
    constructor(_xrSessionManager: WebXRSessionManager, _options: IWebXRControllerPointerSelectionOptions);
    /**
     * attach this feature
     * Will usually be called by the features manager
     *
     * @returns true if successful.
     */
    attach(): boolean;
    /**
     * detach this feature.
     * Will usually be called by the features manager
     *
     * @returns true if successful.
     */
    detach(): boolean;
    /**
     * Will get the mesh under a specific pointer.
     * `scene.meshUnderPointer` will only return one mesh - either left or right.
     * @param controllerId the controllerId to check
     * @returns The mesh under pointer or null if no mesh is under the pointer
     */
    getMeshUnderPointer(controllerId: string): Nullable<AbstractMesh>;
    /**
     * Get the xr controller that correlates to the pointer id in the pointer event
     *
     * @param id the pointer id to search for
     * @returns the controller that correlates to this id or null if not found
     */
    getXRControllerByPointerId(id: number): Nullable<WebXRInputSource>;
    /**
     * @internal
     */
    _getPointerSelectionDisabledByPointerId(id: number): boolean;
    /**
     * @internal
     */
    _setPointerSelectionDisabledByPointerId(id: number, state: boolean): void;
    private _identityMatrix;
    private _screenCoordinatesRef;
    private _viewportRef;
    protected _onXRFrame(_xrFrame: XRFrame): void;
    private get _utilityLayerScene();
    private _attachGazeMode;
    private _attachScreenRayMode;
    private _attachTrackedPointerRayMode;
    private _convertNormalToDirectionOfRay;
    private _detachController;
    private _generateNewMeshPair;
    private _pickingMoved;
    private _updatePointerDistance;
    private _augmentPointerInit;
    /** @internal */
    get lasterPointerDefaultColor(): Color3;
}

}
declare module "babylonjs/XR/features/WebXRControllerTeleportation" {
import { IWebXRFeature } from "babylonjs/XR/webXRFeaturesManager";
import { Observable } from "babylonjs/Misc/observable";
import { WebXRSessionManager } from "babylonjs/XR/webXRSessionManager";
import { Nullable } from "babylonjs/types";
import { WebXRInput } from "babylonjs/XR/webXRInput";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { Vector3 } from "babylonjs/Maths/math.vector";
import { Material } from "babylonjs/Materials/material";
import { PickingInfo } from "babylonjs/Collisions/pickingInfo";
import { WebXRAbstractFeature } from "babylonjs/XR/features/WebXRAbstractFeature";
import { Scene } from "babylonjs/scene";
/**
 * The options container for the teleportation module
 */
export interface IWebXRTeleportationOptions {
    /**
     * if provided, this scene will be used to render meshes.
     */
    customUtilityLayerScene?: Scene;
    /**
     * Values to configure the default target mesh
     */
    defaultTargetMeshOptions?: {
        /**
         * Fill color of the teleportation area
         */
        teleportationFillColor?: string;
        /**
         * Border color for the teleportation area
         */
        teleportationBorderColor?: string;
        /**
         * Disable the mesh's animation sequence
         */
        disableAnimation?: boolean;
        /**
         * Disable lighting on the material or the ring and arrow
         */
        disableLighting?: boolean;
        /**
         * Override the default material of the torus and arrow
         */
        torusArrowMaterial?: Material;
        /**
         * Override the default material of the Landing Zone
         */
        teleportationCircleMaterial?: Material;
    };
    /**
     * A list of meshes to use as floor meshes.
     * Meshes can be added and removed after initializing the feature using the
     * addFloorMesh and removeFloorMesh functions
     * If empty, rotation will still work
     */
    floorMeshes?: AbstractMesh[];
    /**
     *  use this rendering group id for the meshes (optional)
     */
    renderingGroupId?: number;
    /**
     * Should teleportation move only to snap points
     */
    snapPointsOnly?: boolean;
    /**
     * An array of points to which the teleportation will snap to.
     * If the teleportation ray is in the proximity of one of those points, it will be corrected to this point.
     */
    snapPositions?: Vector3[];
    /**
     * How close should the teleportation ray be in order to snap to position.
     * Default to 0.8 units (meters)
     */
    snapToPositionRadius?: number;
    /**
     * Provide your own teleportation mesh instead of babylon's wonderful doughnut.
     * If you want to support rotation, make sure your mesh has a direction indicator.
     *
     * When left untouched, the default mesh will be initialized.
     */
    teleportationTargetMesh?: AbstractMesh;
    /**
     * If main component is used (no thumbstick), how long should the "long press" take before teleport
     */
    timeToTeleport?: number;
    /**
     * Disable using the thumbstick and use the main component (usually trigger) on long press.
     * This will be automatically true if the controller doesn't have a thumbstick or touchpad.
     */
    useMainComponentOnly?: boolean;
    /**
     * Should meshes created here be added to a utility layer or the main scene
     */
    useUtilityLayer?: boolean;
    /**
     * Babylon XR Input class for controller
     */
    xrInput: WebXRInput;
    /**
     * Meshes that the teleportation ray cannot go through
     */
    pickBlockerMeshes?: AbstractMesh[];
    /**
     * Should teleport work only on a specific hand?
     */
    forceHandedness?: XRHandedness;
    /**
     * If provided, this function will be used to generate the ray mesh instead of the lines mesh being used per default
     */
    generateRayPathMesh?: (points: Vector3[], pickingInfo: PickingInfo) => AbstractMesh;
}
/**
 * This is a teleportation feature to be used with WebXR-enabled motion controllers.
 * When enabled and attached, the feature will allow a user to move around and rotate in the scene using
 * the input of the attached controllers.
 */
export class WebXRMotionControllerTeleportation extends WebXRAbstractFeature {
    private _options;
    private _controllers;
    private _currentTeleportationControllerId;
    private _floorMeshes;
    private _quadraticBezierCurve;
    private _selectionFeature;
    private _snapToPositions;
    private _snappedToPoint;
    private _teleportationRingMaterial?;
    private _tmpRay;
    private _tmpVector;
    private _tmpQuaternion;
    /**
     * Skip the next teleportation. This can be controlled by the user to prevent the user from teleportation
     * to sections that are not yet "unlocked", but should still show the teleportation mesh.
     */
    skipNextTeleportation: boolean;
    /**
     * The module's name
     */
    static readonly Name: string;
    /**
     * The (Babylon) version of this module.
     * This is an integer representing the implementation version.
     * This number does not correspond to the webxr specs version
     */
    static readonly Version: number;
    /**
     * Is movement backwards enabled
     */
    backwardsMovementEnabled: boolean;
    /**
     * Distance to travel when moving backwards
     */
    backwardsTeleportationDistance: number;
    /**
     * The distance from the user to the inspection point in the direction of the controller
     * A higher number will allow the user to move further
     * defaults to 5 (meters, in xr units)
     */
    parabolicCheckRadius: number;
    /**
     * Should the module support parabolic ray on top of direct ray
     * If enabled, the user will be able to point "at the sky" and move according to predefined radius distance
     * Very helpful when moving between floors / different heights
     */
    parabolicRayEnabled: boolean;
    /**
     * The second type of ray - straight line.
     * Should it be enabled or should the parabolic line be the only one.
     */
    straightRayEnabled: boolean;
    /**
     * How much rotation should be applied when rotating right and left
     */
    rotationAngle: number;
    /**
     * This observable will notify when the target mesh position was updated.
     * The picking info it provides contains the point to which the target mesh will move ()
     */
    onTargetMeshPositionUpdatedObservable: Observable<PickingInfo>;
    /**
     * Is teleportation enabled. Can be used to allow rotation only.
     */
    teleportationEnabled: boolean;
    private _rotationEnabled;
    /**
     * Is rotation enabled when moving forward?
     * Disabling this feature will prevent the user from deciding the direction when teleporting
     */
    get rotationEnabled(): boolean;
    /**
     * Sets whether rotation is enabled or not
     * @param enabled is rotation enabled when teleportation is shown
     */
    set rotationEnabled(enabled: boolean);
    /**
     * Exposes the currently set teleportation target mesh.
     */
    get teleportationTargetMesh(): Nullable<AbstractMesh>;
    /**
     * constructs a new teleportation system
     * @param _xrSessionManager an instance of WebXRSessionManager
     * @param _options configuration object for this feature
     */
    constructor(_xrSessionManager: WebXRSessionManager, _options: IWebXRTeleportationOptions);
    /**
     * Get the snapPointsOnly flag
     */
    get snapPointsOnly(): boolean;
    /**
     * Sets the snapPointsOnly flag
     * @param snapToPoints should teleportation be exclusively to snap points
     */
    set snapPointsOnly(snapToPoints: boolean);
    /**
     * Add a new mesh to the floor meshes array
     * @param mesh the mesh to use as floor mesh
     */
    addFloorMesh(mesh: AbstractMesh): void;
    /**
     * Add a mesh to the list of meshes blocking the teleportation ray
     * @param mesh The mesh to add to the teleportation-blocking meshes
     */
    addBlockerMesh(mesh: AbstractMesh): void;
    /**
     * Add a new snap-to point to fix teleportation to this position
     * @param newSnapPoint The new Snap-To point
     */
    addSnapPoint(newSnapPoint: Vector3): void;
    attach(): boolean;
    detach(): boolean;
    dispose(): void;
    /**
     * Remove a mesh from the floor meshes array
     * @param mesh the mesh to remove
     */
    removeFloorMesh(mesh: AbstractMesh): void;
    /**
     * Remove a mesh from the blocker meshes array
     * @param mesh the mesh to remove
     */
    removeBlockerMesh(mesh: AbstractMesh): void;
    /**
     * Remove a mesh from the floor meshes array using its name
     * @param name the mesh name to remove
     */
    removeFloorMeshByName(name: string): void;
    /**
     * This function will iterate through the array, searching for this point or equal to it. It will then remove it from the snap-to array
     * @param snapPointToRemove the point (or a clone of it) to be removed from the array
     * @returns was the point found and removed or not
     */
    removeSnapPoint(snapPointToRemove: Vector3): boolean;
    /**
     * This function sets a selection feature that will be disabled when
     * the forward ray is shown and will be reattached when hidden.
     * This is used to remove the selection rays when moving.
     * @param selectionFeature the feature to disable when forward movement is enabled
     */
    setSelectionFeature(selectionFeature: Nullable<IWebXRFeature>): void;
    protected _onXRFrame(_xrFrame: XRFrame): void;
    private _attachController;
    private _createDefaultTargetMesh;
    private _detachController;
    private _findClosestSnapPointWithRadius;
    private _setTargetMeshPosition;
    private _setTargetMeshVisibility;
    private _showParabolicPath;
    private _teleportForward;
}

}
declare module "babylonjs/XR/features/WebXRDOMOverlay" {
import { Nullable } from "babylonjs/types";
import { WebXRSessionManager } from "babylonjs/XR/webXRSessionManager";
import { WebXRAbstractFeature } from "babylonjs/XR/features/WebXRAbstractFeature";
/**
 * Options for DOM Overlay feature
 */
export interface IWebXRDomOverlayOptions {
    /**
     * DOM Element or document query selector string for overlay.
     *
     * NOTE: UA may make this element background transparent in XR.
     */
    element: Element | string;
    /**
     * Supress XR Select events on container element (DOM blocks interaction to scene).
     */
    supressXRSelectEvents?: boolean;
}
/**
 * Type of DOM overlay provided by UA.
 */
type WebXRDomOverlayType = 
/**
 * Covers the entire physical screen for a screen-based device, for example handheld AR
 */
"screen"
/**
 * Appears as a floating rectangle in space
 */
 | "floating"
/**
 * Follows the user’s head movement consistently, appearing similar to a HUD
 */
 | "head-locked";
/**
 * DOM Overlay Feature
 *
 * @since 5.0.0
 */
export class WebXRDomOverlay extends WebXRAbstractFeature {
    /**
     * options to use when constructing this feature
     */
    readonly options: IWebXRDomOverlayOptions;
    /**
     * Type of overlay - non-null when available
     */
    private _domOverlayType;
    /**
     * Event Listener to supress "beforexrselect" events.
     */
    private _beforeXRSelectListener;
    /**
     * Element used for overlay
     */
    private _element;
    /**
     * The module's name
     */
    static readonly Name: string;
    /**
     * The (Babylon) version of this module.
     * This is an integer representing the implementation version.
     * This number does not correspond to the WebXR specs version
     */
    static readonly Version: number;
    /**
     * Creates a new instance of the dom-overlay feature
     * @param _xrSessionManager an instance of WebXRSessionManager
     * @param options options to use when constructing this feature
     */
    constructor(_xrSessionManager: WebXRSessionManager, 
    /**
     * options to use when constructing this feature
     */
    options: IWebXRDomOverlayOptions);
    /**
     * attach this feature
     * Will usually be called by the features manager
     *
     * @returns true if successful.
     */
    attach(): boolean;
    /**
     * The type of DOM overlay (null when not supported).  Provided by UA and remains unchanged for duration of session.
     */
    get domOverlayType(): Nullable<WebXRDomOverlayType>;
    /**
     * Dispose this feature and all of the resources attached
     */
    dispose(): void;
    protected _onXRFrame(_xrFrame: XRFrame): void;
    /**
     * Extends the session init object if needed
     * @returns augmentation object for the xr session init object.
     */
    getXRSessionInitExtension(): Promise<Partial<XRSessionInit>>;
}
export {};

}
declare module "babylonjs/XR/features/WebXREyeTracking" {
import { WebXRAbstractFeature } from "babylonjs/XR/features/WebXRAbstractFeature";
import { WebXRSessionManager } from "babylonjs/XR/webXRSessionManager";
import { Observable } from "babylonjs/Misc/observable";
import { Ray } from "babylonjs/Culling/ray";
import { Nullable } from "babylonjs/types";
/**
 * The WebXR Eye Tracking feature grabs eye data from the device and provides it in an easy-access format.
 * Currently only enabled for BabylonNative applications.
 */
export class WebXREyeTracking extends WebXRAbstractFeature {
    private _latestEyeSpace;
    private _gazeRay;
    /**
     * The module's name
     */
    static readonly Name: string;
    /**
     * The (Babylon) version of this module.
     * This is an integer representing the implementation version.
     * This number does not correspond to the WebXR specs version
     */
    static readonly Version: number;
    /**
     * This observable will notify registered observers when eye tracking starts
     */
    readonly onEyeTrackingStartedObservable: Observable<Ray>;
    /**
     * This observable will notify registered observers when eye tracking ends
     */
    readonly onEyeTrackingEndedObservable: Observable<void>;
    /**
     * This observable will notify registered observers on each frame that has valid tracking
     */
    readonly onEyeTrackingFrameUpdateObservable: Observable<Ray>;
    /**
     * Creates a new instance of the XR eye tracking feature.
     * @param _xrSessionManager An instance of WebXRSessionManager.
     */
    constructor(_xrSessionManager: WebXRSessionManager);
    /**
     * Dispose this feature and all of the resources attached.
     */
    dispose(): void;
    /**
     * Returns whether the gaze data is valid or not
     * @returns true if the data is valid
     */
    get isEyeGazeValid(): boolean;
    /**
     * Get a reference to the gaze ray. This data is valid while eye tracking persists, and will be set to null when gaze data is no longer available
     * @returns a reference to the gaze ray if it exists and is valid, returns null otherwise.
     */
    getEyeGaze(): Nullable<Ray>;
    protected _onXRFrame(frame: XRFrame): void;
    private _eyeTrackingStartListener;
    private _eyeTrackingEndListener;
    private _init;
}

}
declare module "babylonjs/XR/features/WebXRFeaturePointSystem" {
import { WebXRSessionManager } from "babylonjs/XR/webXRSessionManager";
import { Observable } from "babylonjs/Misc/observable";
import { Vector3 } from "babylonjs/Maths/math.vector";
import { WebXRAbstractFeature } from "babylonjs/XR/features/WebXRAbstractFeature";
/**
 * A babylon interface for a "WebXR" feature point.
 * Represents the position and confidence value of a given feature point.
 */
export interface IWebXRFeaturePoint {
    /**
     * Represents the position of the feature point in world space.
     */
    position: Vector3;
    /**
     * Represents the confidence value of the feature point in world space. 0 being least confident, and 1 being most confident.
     */
    confidenceValue: number;
}
/**
 * The feature point system is used to detect feature points from real world geometry.
 * This feature is currently experimental and only supported on BabylonNative, and should not be used in the browser.
 * The newly introduced API can be seen in webxr.nativeextensions.d.ts and described in FeaturePoints.md.
 */
export class WebXRFeaturePointSystem extends WebXRAbstractFeature {
    private _enabled;
    private _featurePointCloud;
    /**
     * The module's name
     */
    static readonly Name: string;
    /**
     * The (Babylon) version of this module.
     * This is an integer representing the implementation version.
     * This number does not correspond to the WebXR specs version
     */
    static readonly Version: number;
    /**
     * Observers registered here will be executed whenever new feature points are added (on XRFrame while the session is tracking).
     * Will notify the observers about which feature points have been added.
     */
    readonly onFeaturePointsAddedObservable: Observable<number[]>;
    /**
     * Observers registered here will be executed whenever a feature point has been updated (on XRFrame while the session is tracking).
     * Will notify the observers about which feature points have been updated.
     */
    readonly onFeaturePointsUpdatedObservable: Observable<number[]>;
    /**
     * The current feature point cloud maintained across frames.
     */
    get featurePointCloud(): Array<IWebXRFeaturePoint>;
    /**
     * construct the feature point system
     * @param _xrSessionManager an instance of xr Session manager
     */
    constructor(_xrSessionManager: WebXRSessionManager);
    /**
     * Detach this feature.
     * Will usually be called by the features manager
     *
     * @returns true if successful.
     */
    detach(): boolean;
    /**
     * Dispose this feature and all of the resources attached
     */
    dispose(): void;
    /**
     * On receiving a new XR frame if this feature is attached notify observers new feature point data is available.
     * @param frame
     */
    protected _onXRFrame(frame: XRFrame): void;
    /**
     * Initializes the feature. If the feature point feature is not available for this environment do not mark the feature as enabled.
     */
    private _init;
}

}
declare module "babylonjs/XR/features/WebXRHandTracking" {
import { WebXRAbstractFeature } from "babylonjs/XR/features/WebXRAbstractFeature";
import { WebXRSessionManager } from "babylonjs/XR/webXRSessionManager";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { Mesh } from "babylonjs/Meshes/mesh";
import { WebXRInput } from "babylonjs/XR/webXRInput";
import { WebXRInputSource } from "babylonjs/XR/webXRInputSource";
import { Nullable } from "babylonjs/types";
import { IDisposable } from "babylonjs/scene";
import { Observable } from "babylonjs/Misc/observable";
import { InstancedMesh } from "babylonjs/Meshes/instancedMesh";
import { Color3 } from "babylonjs/Maths/math.color";
/**
 * Configuration interface for the hand tracking feature
 */
export interface IWebXRHandTrackingOptions {
    /**
     * The xrInput that will be used as source for new hands
     */
    xrInput: WebXRInput;
    /**
     * Configuration object for the joint meshes.
     */
    jointMeshes?: {
        /**
         * Should the meshes created be invisible (defaults to false).
         */
        invisible?: boolean;
        /**
         * A source mesh to be used to create instances. Defaults to an icosphere with two subdivisions and smooth lighting.
         * This mesh will be the source for all other (25) meshes.
         * It should have the general size of a single unit, as the instances will be scaled according to the provided radius.
         */
        sourceMesh?: Mesh;
        /**
         * This function will be called after a mesh was created for a specific joint.
         * Using this function you can either manipulate the instance or return a new mesh.
         * When returning a new mesh the instance created before will be disposed.
         * @param meshInstance An instance of the original joint mesh being used for the joint.
         * @param jointId The joint's index, see https://immersive-web.github.io/webxr-hand-input/#skeleton-joints-section for more info.
         * @param hand Which hand ("left", "right") the joint will be on.
         */
        onHandJointMeshGenerated?: (meshInstance: InstancedMesh, jointId: number, hand: XRHandedness) => AbstractMesh | undefined;
        /**
         * Should the source mesh stay visible (defaults to false).
         */
        keepOriginalVisible?: boolean;
        /**
         * Should each instance have its own physics impostor
         */
        enablePhysics?: boolean;
        /**
         * If enabled, override default physics properties
         */
        physicsProps?: {
            friction?: number;
            restitution?: number;
            impostorType?: number;
        };
        /**
         * Scale factor for all joint meshes (defaults to 1)
         */
        scaleFactor?: number;
    };
    /**
     * Configuration object for the hand meshes.
     */
    handMeshes?: {
        /**
         * Should the default hand mesh be disabled. In this case, the spheres will be visible (unless set invisible).
         */
        disableDefaultMeshes?: boolean;
        /**
         * Rigged hand meshes that will be tracked to the user's hands. This will override the default hand mesh.
         */
        customMeshes?: {
            right: AbstractMesh;
            left: AbstractMesh;
        };
        /**
         * Are the meshes prepared for a left-handed system. Default hand meshes are right-handed.
         */
        meshesUseLeftHandedCoordinates?: boolean;
        /**
         * If a hand mesh was provided, this array will define what axis will update which node. This will override the default hand mesh
         */
        customRigMappings?: {
            right: XRHandMeshRigMapping;
            left: XRHandMeshRigMapping;
        };
        /**
         * Override the colors of the hand meshes.
         */
        customColors?: {
            base?: Color3;
            fresnel?: Color3;
            fingerColor?: Color3;
            tipFresnel?: Color3;
        };
    };
}
/**
 * Parts of the hands divided to writs and finger names
 */
export enum HandPart {
    /**
     * HandPart - Wrist
     */
    WRIST = "wrist",
    /**
     * HandPart - The thumb
     */
    THUMB = "thumb",
    /**
     * HandPart - Index finger
     */
    INDEX = "index",
    /**
     * HandPart - Middle finger
     */
    MIDDLE = "middle",
    /**
     * HandPart - Ring finger
     */
    RING = "ring",
    /**
     * HandPart - Little finger
     */
    LITTLE = "little"
}
/**
 * Joints of the hand as defined by the WebXR specification.
 * https://immersive-web.github.io/webxr-hand-input/#skeleton-joints-section
 */
export enum WebXRHandJoint {
    /** Wrist */
    WRIST = "wrist",
    /** Thumb near wrist */
    THUMB_METACARPAL = "thumb-metacarpal",
    /** Thumb first knuckle */
    THUMB_PHALANX_PROXIMAL = "thumb-phalanx-proximal",
    /** Thumb second knuckle */
    THUMB_PHALANX_DISTAL = "thumb-phalanx-distal",
    /** Thumb tip */
    THUMB_TIP = "thumb-tip",
    /** Index finger near wrist */
    INDEX_FINGER_METACARPAL = "index-finger-metacarpal",
    /** Index finger first knuckle */
    INDEX_FINGER_PHALANX_PROXIMAL = "index-finger-phalanx-proximal",
    /** Index finger second knuckle */
    INDEX_FINGER_PHALANX_INTERMEDIATE = "index-finger-phalanx-intermediate",
    /** Index finger third knuckle */
    INDEX_FINGER_PHALANX_DISTAL = "index-finger-phalanx-distal",
    /** Index finger tip */
    INDEX_FINGER_TIP = "index-finger-tip",
    /** Middle finger near wrist */
    MIDDLE_FINGER_METACARPAL = "middle-finger-metacarpal",
    /** Middle finger first knuckle */
    MIDDLE_FINGER_PHALANX_PROXIMAL = "middle-finger-phalanx-proximal",
    /** Middle finger second knuckle */
    MIDDLE_FINGER_PHALANX_INTERMEDIATE = "middle-finger-phalanx-intermediate",
    /** Middle finger third knuckle */
    MIDDLE_FINGER_PHALANX_DISTAL = "middle-finger-phalanx-distal",
    /** Middle finger tip */
    MIDDLE_FINGER_TIP = "middle-finger-tip",
    /** Ring finger near wrist */
    RING_FINGER_METACARPAL = "ring-finger-metacarpal",
    /** Ring finger first knuckle */
    RING_FINGER_PHALANX_PROXIMAL = "ring-finger-phalanx-proximal",
    /** Ring finger second knuckle */
    RING_FINGER_PHALANX_INTERMEDIATE = "ring-finger-phalanx-intermediate",
    /** Ring finger third knuckle */
    RING_FINGER_PHALANX_DISTAL = "ring-finger-phalanx-distal",
    /** Ring finger tip */
    RING_FINGER_TIP = "ring-finger-tip",
    /** Pinky finger near wrist */
    PINKY_FINGER_METACARPAL = "pinky-finger-metacarpal",
    /** Pinky finger first knuckle */
    PINKY_FINGER_PHALANX_PROXIMAL = "pinky-finger-phalanx-proximal",
    /** Pinky finger second knuckle */
    PINKY_FINGER_PHALANX_INTERMEDIATE = "pinky-finger-phalanx-intermediate",
    /** Pinky finger third knuckle */
    PINKY_FINGER_PHALANX_DISTAL = "pinky-finger-phalanx-distal",
    /** Pinky finger tip */
    PINKY_FINGER_TIP = "pinky-finger-tip"
}
/** A type encapsulating a dictionary mapping WebXR joints to bone names in a rigged hand mesh.  */
export type XRHandMeshRigMapping = {
    [webXRJointName in WebXRHandJoint]: string;
};
/**
 * Representing a single hand (with its corresponding native XRHand object)
 */
export class WebXRHand implements IDisposable {
    /** The controller to which the hand correlates. */
    readonly xrController: WebXRInputSource;
    private readonly _jointMeshes;
    private _handMesh;
    /** An optional rig mapping for the hand mesh. If not provided (but a hand mesh is provided),
     * it will be assumed that the hand mesh's bones are named directly after the WebXR bone names. */
    readonly rigMapping: Nullable<XRHandMeshRigMapping>;
    private readonly _leftHandedMeshes;
    private readonly _jointsInvisible;
    private readonly _jointScaleFactor;
    private _scene;
    /**
     * Transform nodes that will directly receive the transforms from the WebXR matrix data.
     */
    private _jointTransforms;
    /**
     * The float array that will directly receive the transform matrix data from WebXR.
     */
    private _jointTransformMatrices;
    private _tempJointMatrix;
    /**
     * The float array that will directly receive the joint radii from WebXR.
     */
    private _jointRadii;
    /**
     * Get the hand mesh.
     */
    get handMesh(): Nullable<AbstractMesh>;
    /**
     * Get meshes of part of the hand.
     * @param part The part of hand to get.
     * @returns An array of meshes that correlate to the hand part requested.
     */
    getHandPartMeshes(part: HandPart): AbstractMesh[];
    /**
     * Retrieves a mesh linked to a named joint in the hand.
     * @param jointName The name of the joint.
     * @returns An AbstractMesh whose position corresponds with the joint position.
     */
    getJointMesh(jointName: WebXRHandJoint): AbstractMesh;
    /**
     * Construct a new hand object
     * @param xrController The controller to which the hand correlates.
     * @param _jointMeshes The meshes to be used to track the hand joints.
     * @param _handMesh An optional hand mesh.
     * @param rigMapping An optional rig mapping for the hand mesh.
     *                   If not provided (but a hand mesh is provided),
     *                   it will be assumed that the hand mesh's bones are named
     *                   directly after the WebXR bone names.
     * @param _leftHandedMeshes Are the hand meshes left-handed-system meshes
     * @param _jointsInvisible Are the tracked joint meshes visible
     * @param _jointScaleFactor Scale factor for all joint meshes
     */
    constructor(
    /** The controller to which the hand correlates. */
    xrController: WebXRInputSource, _jointMeshes: AbstractMesh[], _handMesh: Nullable<AbstractMesh>, 
    /** An optional rig mapping for the hand mesh. If not provided (but a hand mesh is provided),
     * it will be assumed that the hand mesh's bones are named directly after the WebXR bone names. */
    rigMapping: Nullable<XRHandMeshRigMapping>, _leftHandedMeshes?: boolean, _jointsInvisible?: boolean, _jointScaleFactor?: number);
    /**
     * Sets the current hand mesh to render for the WebXRHand.
     * @param handMesh The rigged hand mesh that will be tracked to the user's hand.
     * @param rigMapping The mapping from XRHandJoint to bone names to use with the mesh.
     */
    setHandMesh(handMesh: AbstractMesh, rigMapping: Nullable<XRHandMeshRigMapping>): void;
    /**
     * Update this hand from the latest xr frame.
     * @param xrFrame The latest frame received from WebXR.
     * @param referenceSpace The current viewer reference space.
     */
    updateFromXRFrame(xrFrame: XRFrame, referenceSpace: XRReferenceSpace): void;
    /**
     * Dispose this Hand object
     */
    dispose(): void;
}
/**
 * WebXR Hand Joint tracking feature, available for selected browsers and devices
 */
export class WebXRHandTracking extends WebXRAbstractFeature {
    /** Options to use when constructing this feature. */
    readonly options: IWebXRHandTrackingOptions;
    /**
     * The module's name
     */
    static readonly Name: string;
    /**
     * The (Babylon) version of this module.
     * This is an integer representing the implementation version.
     * This number does not correspond to the WebXR specs version
     */
    static readonly Version: number;
    /** The base URL for the default hand model. */
    static DEFAULT_HAND_MODEL_BASE_URL: string;
    /** The filename to use for the default right hand model. */
    static DEFAULT_HAND_MODEL_RIGHT_FILENAME: string;
    /** The filename to use for the default left hand model. */
    static DEFAULT_HAND_MODEL_LEFT_FILENAME: string;
    /** The URL pointing to the default hand model NodeMaterial shader. */
    static DEFAULT_HAND_MODEL_SHADER_URL: string;
    private static readonly _ICOSPHERE_PARAMS;
    private static _RightHandGLB;
    private static _LeftHandGLB;
    private static _GenerateTrackedJointMeshes;
    private static _GenerateDefaultHandMeshesAsync;
    /**
     * Generates a mapping from XRHandJoint to bone name for the default hand mesh.
     * @param handedness The handedness being mapped for.
     */
    private static _GenerateDefaultHandMeshRigMapping;
    private _attachedHands;
    private _trackingHands;
    private _handResources;
    /**
     * This observable will notify registered observers when a new hand object was added and initialized
     */
    onHandAddedObservable: Observable<WebXRHand>;
    /**
     * This observable will notify its observers right before the hand object is disposed
     */
    onHandRemovedObservable: Observable<WebXRHand>;
    /**
     * Check if the needed objects are defined.
     * This does not mean that the feature is enabled, but that the objects needed are well defined.
     */
    isCompatible(): boolean;
    /**
     * Get the hand object according to the controller id
     * @param controllerId the controller id to which we want to get the hand
     * @returns null if not found or the WebXRHand object if found
     */
    getHandByControllerId(controllerId: string): Nullable<WebXRHand>;
    /**
     * Get a hand object according to the requested handedness
     * @param handedness the handedness to request
     * @returns null if not found or the WebXRHand object if found
     */
    getHandByHandedness(handedness: XRHandedness): Nullable<WebXRHand>;
    /**
     * Creates a new instance of the XR hand tracking feature.
     * @param _xrSessionManager An instance of WebXRSessionManager.
     * @param options Options to use when constructing this feature.
     */
    constructor(_xrSessionManager: WebXRSessionManager, 
    /** Options to use when constructing this feature. */
    options: IWebXRHandTrackingOptions);
    /**
     * Attach this feature.
     * Will usually be called by the features manager.
     *
     * @returns true if successful.
     */
    attach(): boolean;
    protected _onXRFrame(_xrFrame: XRFrame): void;
    private _attachHand;
    private _detachHandById;
    private _detachHand;
    /**
     * Detach this feature.
     * Will usually be called by the features manager.
     *
     * @returns true if successful.
     */
    detach(): boolean;
    /**
     * Dispose this feature and all of the resources attached.
     */
    dispose(): void;
}

}
declare module "babylonjs/XR/features/WebXRHitTest" {
import { WebXRSessionManager } from "babylonjs/XR/webXRSessionManager";
import { Observable } from "babylonjs/Misc/observable";
import { Vector3, Quaternion } from "babylonjs/Maths/math.vector";
import { WebXRAbstractFeature } from "babylonjs/XR/features/WebXRAbstractFeature";
import { IWebXRLegacyHitTestOptions, IWebXRLegacyHitResult, IWebXRHitTestFeature } from "babylonjs/XR/features/WebXRHitTestLegacy";
/**
 * Options used for hit testing (version 2)
 */
export interface IWebXRHitTestOptions extends IWebXRLegacyHitTestOptions {
    /**
     * Do not create a permanent hit test. Will usually be used when only
     * transient inputs are needed.
     */
    disablePermanentHitTest?: boolean;
    /**
     * Enable transient (for example touch-based) hit test inspections
     */
    enableTransientHitTest?: boolean;
    /**
     * Override the default transient hit test profile (generic-touchscreen).
     */
    transientHitTestProfile?: string;
    /**
     * Offset ray for the permanent hit test
     */
    offsetRay?: Vector3;
    /**
     * Offset ray for the transient hit test
     */
    transientOffsetRay?: Vector3;
    /**
     * Instead of using viewer space for hit tests, use the reference space defined in the session manager
     */
    useReferenceSpace?: boolean;
    /**
     * Override the default entity type(s) of the hit-test result
     */
    entityTypes?: XRHitTestTrackableType[];
}
/**
 * Interface defining the babylon result of hit-test
 */
export interface IWebXRHitResult extends IWebXRLegacyHitResult {
    /**
     * The input source that generated this hit test (if transient)
     */
    inputSource?: XRInputSource;
    /**
     * Is this a transient hit test
     */
    isTransient?: boolean;
    /**
     * Position of the hit test result
     */
    position: Vector3;
    /**
     * Rotation of the hit test result
     */
    rotationQuaternion: Quaternion;
    /**
     * The native hit test result
     */
    xrHitResult: XRHitTestResult;
}
/**
 * The currently-working hit-test module.
 * Hit test (or Ray-casting) is used to interact with the real world.
 * For further information read here - https://github.com/immersive-web/hit-test
 *
 * Tested on chrome (mobile) 80.
 */
export class WebXRHitTest extends WebXRAbstractFeature implements IWebXRHitTestFeature<IWebXRHitResult> {
    /**
     * options to use when constructing this feature
     */
    readonly options: IWebXRHitTestOptions;
    private _tmpMat;
    private _tmpPos;
    private _tmpQuat;
    private _transientXrHitTestSource;
    private _xrHitTestSource;
    private _initHitTestSource;
    /**
     * The module's name
     */
    static readonly Name: string;
    /**
     * The (Babylon) version of this module.
     * This is an integer representing the implementation version.
     * This number does not correspond to the WebXR specs version
     */
    static readonly Version: number;
    /**
     * When set to true, each hit test will have its own position/rotation objects
     * When set to false, position and rotation objects will be reused for each hit test. It is expected that
     * the developers will clone them or copy them as they see fit.
     */
    autoCloneTransformation: boolean;
    /**
     * Triggered when new babylon (transformed) hit test results are available
     * Note - this will be called when results come back from the device. It can be an empty array!!
     */
    onHitTestResultObservable: Observable<IWebXRHitResult[]>;
    /**
     * Use this to temporarily pause hit test checks.
     */
    paused: boolean;
    /**
     * Creates a new instance of the hit test feature
     * @param _xrSessionManager an instance of WebXRSessionManager
     * @param options options to use when constructing this feature
     */
    constructor(_xrSessionManager: WebXRSessionManager, 
    /**
     * options to use when constructing this feature
     */
    options?: IWebXRHitTestOptions);
    /**
     * attach this feature
     * Will usually be called by the features manager
     *
     * @returns true if successful.
     */
    attach(): boolean;
    /**
     * detach this feature.
     * Will usually be called by the features manager
     *
     * @returns true if successful.
     */
    detach(): boolean;
    /**
     * Dispose this feature and all of the resources attached
     */
    dispose(): void;
    protected _onXRFrame(frame: XRFrame): void;
    private _processWebXRHitTestResult;
}

}
declare module "babylonjs/XR/features/WebXRHitTestLegacy" {
import { IWebXRFeature } from "babylonjs/XR/webXRFeaturesManager";
import { WebXRSessionManager } from "babylonjs/XR/webXRSessionManager";
import { Observable } from "babylonjs/Misc/observable";
import { Matrix } from "babylonjs/Maths/math.vector";
import { TransformNode } from "babylonjs/Meshes/transformNode";
import { WebXRAbstractFeature } from "babylonjs/XR/features/WebXRAbstractFeature";
/**
 * An interface for all Hit test features
 */
export interface IWebXRHitTestFeature<T extends IWebXRLegacyHitResult> extends IWebXRFeature {
    /**
     * Triggered when new babylon (transformed) hit test results are available
     */
    onHitTestResultObservable: Observable<T[]>;
}
/**
 * Options used for hit testing
 */
export interface IWebXRLegacyHitTestOptions {
    /**
     * Only test when user interacted with the scene. Default - hit test every frame
     */
    testOnPointerDownOnly?: boolean;
    /**
     * The node to use to transform the local results to world coordinates
     */
    worldParentNode?: TransformNode;
}
/**
 * Interface defining the babylon result of raycasting/hit-test
 */
export interface IWebXRLegacyHitResult {
    /**
     * Transformation matrix that can be applied to a node that will put it in the hit point location
     */
    transformationMatrix: Matrix;
    /**
     * The native hit test result
     */
    xrHitResult: XRHitResult | XRHitTestResult;
}
/**
 * The currently-working hit-test module.
 * Hit test (or Ray-casting) is used to interact with the real world.
 * For further information read here - https://github.com/immersive-web/hit-test
 */
export class WebXRHitTestLegacy extends WebXRAbstractFeature implements IWebXRHitTestFeature<IWebXRLegacyHitResult> {
    /**
     * options to use when constructing this feature
     */
    readonly options: IWebXRLegacyHitTestOptions;
    private _direction;
    private _mat;
    private _onSelectEnabled;
    private _origin;
    /**
     * The module's name
     */
    static readonly Name: string;
    /**
     * The (Babylon) version of this module.
     * This is an integer representing the implementation version.
     * This number does not correspond to the WebXR specs version
     */
    static readonly Version: number;
    /**
     * Populated with the last native XR Hit Results
     */
    lastNativeXRHitResults: XRHitResult[];
    /**
     * Triggered when new babylon (transformed) hit test results are available
     */
    onHitTestResultObservable: Observable<IWebXRLegacyHitResult[]>;
    /**
     * Creates a new instance of the (legacy version) hit test feature
     * @param _xrSessionManager an instance of WebXRSessionManager
     * @param options options to use when constructing this feature
     */
    constructor(_xrSessionManager: WebXRSessionManager, 
    /**
     * options to use when constructing this feature
     */
    options?: IWebXRLegacyHitTestOptions);
    /**
     * execute a hit test with an XR Ray
     *
     * @param xrSession a native xrSession that will execute this hit test
     * @param xrRay the ray (position and direction) to use for ray-casting
     * @param referenceSpace native XR reference space to use for the hit-test
     * @param filter filter function that will filter the results
     * @returns a promise that resolves with an array of native XR hit result in xr coordinates system
     */
    static XRHitTestWithRay(xrSession: XRSession, xrRay: XRRay, referenceSpace: XRReferenceSpace, filter?: (result: XRHitResult) => boolean): Promise<XRHitResult[]>;
    /**
     * Execute a hit test on the current running session using a select event returned from a transient input (such as touch)
     * @param event the (select) event to use to select with
     * @param referenceSpace the reference space to use for this hit test
     * @returns a promise that resolves with an array of native XR hit result in xr coordinates system
     */
    static XRHitTestWithSelectEvent(event: XRInputSourceEvent, referenceSpace: XRReferenceSpace): Promise<XRHitResult[]>;
    /**
     * attach this feature
     * Will usually be called by the features manager
     *
     * @returns true if successful.
     */
    attach(): boolean;
    /**
     * detach this feature.
     * Will usually be called by the features manager
     *
     * @returns true if successful.
     */
    detach(): boolean;
    /**
     * Dispose this feature and all of the resources attached
     */
    dispose(): void;
    protected _onXRFrame(frame: XRFrame): void;
    private _onHitTestResults;
    private _onSelect;
}

}
declare module "babylonjs/XR/features/WebXRImageTracking" {
import { WebXRSessionManager } from "babylonjs/XR/webXRSessionManager";
import { Observable } from "babylonjs/Misc/observable";
import { WebXRAbstractFeature } from "babylonjs/XR/features/WebXRAbstractFeature";
import { Matrix } from "babylonjs/Maths/math.vector";
import { Nullable } from "babylonjs/types";
/**
 * Options interface for the background remover plugin
 */
export interface IWebXRImageTrackingOptions {
    /**
     * A required array with images to track
     */
    images: {
        /**
         * The source of the image. can be a URL or an image bitmap
         */
        src: string | ImageBitmap;
        /**
         * The estimated width in the real world (in meters)
         */
        estimatedRealWorldWidth: number;
    }[];
}
/**
 * An object representing an image tracked by the system
 */
export interface IWebXRTrackedImage {
    /**
     * The ID of this image (which is the same as the position in the array that was used to initialize the feature)
     */
    id: number;
    /**
     * Is the transformation provided emulated. If it is, the system "guesses" its real position. Otherwise it can be considered as exact position.
     */
    emulated?: boolean;
    /**
     * Just in case it is needed - the image bitmap that is being tracked
     */
    originalBitmap: ImageBitmap;
    /**
     * The native XR result image tracking result, untouched
     */
    xrTrackingResult?: XRImageTrackingResult;
    /**
     * Width in real world (meters)
     */
    realWorldWidth?: number;
    /**
     * A transformation matrix of this current image in the current reference space.
     */
    transformationMatrix: Matrix;
    /**
     * The width/height ratio of this image. can be used to calculate the size of the detected object/image
     */
    ratio?: number;
}
/**
 * Image tracking for immersive AR sessions.
 * Providing a list of images and their estimated widths will enable tracking those images in the real world.
 */
export class WebXRImageTracking extends WebXRAbstractFeature {
    /**
     * read-only options to be used in this module
     */
    readonly options: IWebXRImageTrackingOptions;
    /**
     * The module's name
     */
    static readonly Name: string;
    /**
     * The (Babylon) version of this module.
     * This is an integer representing the implementation version.
     * This number does not correspond to the WebXR specs version
     */
    static readonly Version: number;
    /**
     * This will be triggered if the underlying system deems an image untrackable.
     * The index is the index of the image from the array used to initialize the feature.
     */
    onUntrackableImageFoundObservable: Observable<number>;
    /**
     * An image was deemed trackable, and the system will start tracking it.
     */
    onTrackableImageFoundObservable: Observable<IWebXRTrackedImage>;
    /**
     * The image was found and its state was updated.
     */
    onTrackedImageUpdatedObservable: Observable<IWebXRTrackedImage>;
    private _trackableScoreStatus;
    private _trackedImages;
    private _originalTrackingRequest;
    /**
     * constructs the image tracking feature
     * @param _xrSessionManager the session manager for this module
     * @param options read-only options to be used in this module
     */
    constructor(_xrSessionManager: WebXRSessionManager, 
    /**
     * read-only options to be used in this module
     */
    options: IWebXRImageTrackingOptions);
    /**
     * attach this feature
     * Will usually be called by the features manager
     *
     * @returns true if successful.
     */
    attach(): boolean;
    /**
     * detach this feature.
     * Will usually be called by the features manager
     *
     * @returns true if successful.
     */
    detach(): boolean;
    /**
     * Get a tracked image by its ID.
     *
     * @param id the id of the image to load (position in the init array)
     * @returns a trackable image, if exists in this location
     */
    getTrackedImageById(id: number): Nullable<IWebXRTrackedImage>;
    /**
     * Dispose this feature and all of the resources attached
     */
    dispose(): void;
    /**
     * Extends the session init object if needed
     * @returns augmentation object fo the xr session init object.
     */
    getXRSessionInitExtension(): Promise<Partial<XRSessionInit>>;
    protected _onXRFrame(_xrFrame: XRFrame): void;
    private _checkScoresAsync;
}

}
declare module "babylonjs/XR/features/WebXRLayers" {
import { WebXRSessionManager } from "babylonjs/XR/webXRSessionManager";
import { WebXRAbstractFeature } from "babylonjs/XR/features/WebXRAbstractFeature";
import { WebXRLayerRenderTargetTextureProvider } from "babylonjs/XR/webXRRenderTargetTextureProvider";
import { WebXRLayerType } from "babylonjs/XR/webXRLayerWrapper";
import { WebXRLayerWrapper } from "babylonjs/XR/webXRLayerWrapper";
import { WebXRWebGLLayerWrapper } from "babylonjs/XR/webXRWebGLLayer";
/**
 * Wraps xr composition layers.
 * @internal
 */
export class WebXRCompositionLayerWrapper extends WebXRLayerWrapper {
    getWidth: () => number;
    getHeight: () => number;
    readonly layer: XRCompositionLayer;
    readonly layerType: WebXRLayerType;
    readonly isMultiview: boolean;
    createRTTProvider: (xrSessionManager: WebXRSessionManager) => WebXRLayerRenderTargetTextureProvider;
    constructor(getWidth: () => number, getHeight: () => number, layer: XRCompositionLayer, layerType: WebXRLayerType, isMultiview: boolean, createRTTProvider: (xrSessionManager: WebXRSessionManager) => WebXRLayerRenderTargetTextureProvider);
}
/**
 * Wraps xr projection layers.
 * @internal
 */
export class WebXRProjectionLayerWrapper extends WebXRCompositionLayerWrapper {
    readonly layer: XRProjectionLayer;
    constructor(layer: XRProjectionLayer, isMultiview: boolean, xrGLBinding: XRWebGLBinding);
}
/**
 * Configuration options of the layers feature
 */
export interface IWebXRLayersOptions {
    /**
     * Whether to try initializing the base projection layer as a multiview render target, if multiview is supported.
     * Defaults to false.
     */
    preferMultiviewOnInit?: boolean;
}
/**
 * Exposes the WebXR Layers API.
 */
export class WebXRLayers extends WebXRAbstractFeature {
    private readonly _options;
    /**
     * The module's name
     */
    static readonly Name: string;
    /**
     * The (Babylon) version of this module.
     * This is an integer representing the implementation version.
     * This number does not correspond to the WebXR specs version
     */
    static readonly Version: number;
    /**
     * Already-created layers
     */
    private _existingLayers;
    private _glContext;
    private _xrWebGLBinding;
    constructor(_xrSessionManager: WebXRSessionManager, _options?: IWebXRLayersOptions);
    /**
     * Attach this feature.
     * Will usually be called by the features manager.
     *
     * @returns true if successful.
     */
    attach(): boolean;
    detach(): boolean;
    /**
     * Creates a new XRWebGLLayer.
     * @param params an object providing configuration options for the new XRWebGLLayer
     * @returns the XRWebGLLayer
     */
    createXRWebGLLayer(params?: XRWebGLLayerInit): WebXRWebGLLayerWrapper;
    /**
     * Creates a new XRProjectionLayer.
     * @param params an object providing configuration options for the new XRProjectionLayer.
     * @param multiview whether the projection layer should render with multiview.
     * @returns the projection layer
     */
    createProjectionLayer(params?: XRProjectionLayerInit, multiview?: boolean): WebXRProjectionLayerWrapper;
    /**
     * Add a new layer to the already-existing list of layers
     * @param wrappedLayer the new layer to add to the existing ones
     */
    addXRSessionLayer(wrappedLayer: WebXRLayerWrapper): void;
    /**
     * Sets the layers to be used by the XR session.
     * Note that you must call this function with any layers you wish to render to
     * since it adds them to the XR session's render state
     * (replacing any layers that were added in a previous call to setXRSessionLayers or updateRenderState).
     * This method also sets up the session manager's render target texture provider
     * as the first layer in the array, which feeds the WebXR camera(s) attached to the session.
     * @param wrappedLayers An array of WebXRLayerWrapper, usually returned from the WebXRLayers createLayer functions.
     */
    setXRSessionLayers(wrappedLayers: Array<WebXRLayerWrapper>): void;
    isCompatible(): boolean;
    /**
     * Dispose this feature and all of the resources attached.
     */
    dispose(): void;
    protected _onXRFrame(_xrFrame: XRFrame): void;
}

}
declare module "babylonjs/XR/features/WebXRLightEstimation" {
import { Observable } from "babylonjs/Misc/observable";
import { Nullable } from "babylonjs/types";
import { WebXRSessionManager } from "babylonjs/XR/webXRSessionManager";
import { WebXRAbstractFeature } from "babylonjs/XR/features/WebXRAbstractFeature";
import { Color3 } from "babylonjs/Maths/math.color";
import { Vector3 } from "babylonjs/Maths/math.vector";
import { DirectionalLight } from "babylonjs/Lights/directionalLight";
import { BaseTexture } from "babylonjs/Materials/Textures/baseTexture";
import { SphericalHarmonics } from "babylonjs/Maths/sphericalPolynomial";
/**
 * Options for Light Estimation feature
 */
export interface IWebXRLightEstimationOptions {
    /**
     * Disable the cube map reflection feature. In this case only light direction and color will be updated
     */
    disableCubeMapReflection?: boolean;
    /**
     * Should the scene's env texture be set to the cube map reflection texture
     * Note that this doesn't work is disableCubeMapReflection if set to false
     */
    setSceneEnvironmentTexture?: boolean;
    /**
     * How often should the cubemap update in ms.
     * If not set the cubemap will be updated every time the underlying system updates the environment texture.
     */
    cubeMapPollInterval?: number;
    /**
     * How often should the light estimation properties update in ms.
     * If not set the light estimation properties will be updated on every frame (depending on the underlying system)
     */
    lightEstimationPollInterval?: number;
    /**
     * Should a directional light source be created.
     * If created, this light source will be updated whenever the light estimation values change
     */
    createDirectionalLightSource?: boolean;
    /**
     * Define the format to be used for the light estimation texture.
     */
    reflectionFormat?: XRReflectionFormat;
    /**
     * Should the light estimation's needed vectors be constructed on each frame.
     * Use this when you use those vectors and don't want their values to change outside of the light estimation feature
     */
    disableVectorReuse?: boolean;
    /**
     * disable applying the spherical polynomial to the cube map texture
     */
    disableSphericalPolynomial?: boolean;
}
/**
 * An interface describing the result of a light estimation
 */
export interface IWebXRLightEstimation {
    /**
     * The intensity of the light source
     */
    lightIntensity: number;
    /**
     * Color of light source
     */
    lightColor: Color3;
    /**
     * The direction from the light source
     */
    lightDirection: Vector3;
    /**
     * Spherical harmonics coefficients of the light source
     */
    sphericalHarmonics: SphericalHarmonics;
}
/**
 * Light Estimation Feature
 *
 * @since 5.0.0
 */
export class WebXRLightEstimation extends WebXRAbstractFeature {
    /**
     * options to use when constructing this feature
     */
    readonly options: IWebXRLightEstimationOptions;
    private _canvasContext;
    private _reflectionCubeMap;
    private _xrLightEstimate;
    private _xrLightProbe;
    private _xrWebGLBinding;
    private _lightDirection;
    private _lightColor;
    private _intensity;
    private _sphericalHarmonics;
    private _cubeMapPollTime;
    private _lightEstimationPollTime;
    /**
     * The module's name
     */
    static readonly Name: string;
    /**
     * The (Babylon) version of this module.
     * This is an integer representing the implementation version.
     * This number does not correspond to the WebXR specs version
     */
    static readonly Version: number;
    /**
     * ARCore's reflection cube map size is 16x16.
     * Once other systems support this feature we will need to change this to be dynamic.
     * see https://github.com/immersive-web/lighting-estimation/blob/main/lighting-estimation-explainer.md#cube-map-open-questions
     */
    private _reflectionCubeMapTextureSize;
    /**
     * If createDirectionalLightSource is set to true this light source will be created automatically.
     * Otherwise this can be set with an external directional light source.
     * This light will be updated whenever the light estimation values change.
     */
    directionalLight: Nullable<DirectionalLight>;
    /**
     * This observable will notify when the reflection cube map is updated.
     */
    onReflectionCubeMapUpdatedObservable: Observable<BaseTexture>;
    /**
     * Creates a new instance of the light estimation feature
     * @param _xrSessionManager an instance of WebXRSessionManager
     * @param options options to use when constructing this feature
     */
    constructor(_xrSessionManager: WebXRSessionManager, 
    /**
     * options to use when constructing this feature
     */
    options: IWebXRLightEstimationOptions);
    /**
     * While the estimated cube map is expected to update over time to better reflect the user's environment as they move around those changes are unlikely to happen with every XRFrame.
     * Since creating and processing the cube map is potentially expensive, especially if mip maps are needed, you can listen to the onReflectionCubeMapUpdatedObservable to determine
     * when it has been updated.
     */
    get reflectionCubeMapTexture(): Nullable<BaseTexture>;
    /**
     * The most recent light estimate.  Available starting on the first frame where the device provides a light probe.
     */
    get xrLightingEstimate(): Nullable<IWebXRLightEstimation>;
    private _getCanvasContext;
    private _getXRGLBinding;
    /**
     * Event Listener for "reflectionchange" events.
     */
    private _updateReflectionCubeMap;
    /**
     * attach this feature
     * Will usually be called by the features manager
     *
     * @returns true if successful.
     */
    attach(): boolean;
    /**
     * detach this feature.
     * Will usually be called by the features manager
     *
     * @returns true if successful.
     */
    detach(): boolean;
    /**
     * Dispose this feature and all of the resources attached
     */
    dispose(): void;
    protected _onXRFrame(_xrFrame: XRFrame): void;
}

}
declare module "babylonjs/XR/features/WebXRMeshDetector" {
import { WebXRAbstractFeature } from "babylonjs/XR/features/WebXRAbstractFeature";
import { WebXRSessionManager } from "babylonjs/XR/webXRSessionManager";
import { TransformNode } from "babylonjs/Meshes/transformNode";
import { Matrix } from "babylonjs/Maths/math";
import { Observable } from "babylonjs/Misc/observable";
/**
 * Options used in the mesh detector module
 */
export interface IWebXRMeshDetectorOptions {
    /**
     * The node to use to transform the local results to world coordinates
     */
    worldParentNode?: TransformNode;
    /**
     * If set to true a reference of the created meshes will be kept until the next session starts
     * If not defined, meshes will be removed from the array when the feature is detached or the session ended.
     */
    doNotRemoveMeshesOnSessionEnded?: boolean;
    /**
     * Preferred detector configuration, not all preferred options will be supported by all platforms.
     */
    preferredDetectorOptions?: XRGeometryDetectorOptions;
    /**
     * If set to true, WebXRMeshDetector will convert coordinate systems for meshes.
     * If not defined, mesh conversions from right handed to left handed coordinate systems won't be conducted.
     * Right handed mesh data will be available through IWebXRVertexData.xrMesh.
     */
    convertCoordinateSystems?: boolean;
}
/**
 * A babylon interface for a XR mesh's vertex data.
 *
 * Currently not supported by WebXR, available only with BabylonNative
 */
export interface IWebXRVertexData {
    /**
     * A babylon-assigned ID for this mesh
     */
    id: number;
    /**
     * Data required for constructing a mesh in Babylon.js.
     */
    xrMesh: XRMesh;
    /**
     * The node to use to transform the local results to world coordinates.
     * WorldParentNode will only exist if it was declared in the IWebXRMeshDetectorOptions.
     */
    worldParentNode?: TransformNode;
    /**
     * An array of vertex positions in babylon space. right/left hand system is taken into account.
     * Positions will only be calculated if convertCoordinateSystems is set to true in the IWebXRMeshDetectorOptions.
     */
    positions?: Float32Array;
    /**
     * An array of indices in babylon space. Indices have a counterclockwise winding order.
     * Indices will only be populated if convertCoordinateSystems is set to true in the IWebXRMeshDetectorOptions.
     */
    indices?: Uint32Array;
    /**
     * An array of vertex normals in babylon space. right/left hand system is taken into account.
     * Normals will not be calculated if convertCoordinateSystems is undefined in the IWebXRMeshDetectorOptions.
     * Different platforms may or may not support mesh normals when convertCoordinateSystems is set to true.
     */
    normals?: Float32Array;
    /**
     * A transformation matrix to apply on the mesh that will be built using the meshDefinition.
     * Local vs. World are decided if worldParentNode was provided or not in the options when constructing the module.
     * TransformationMatrix will only be calculated if convertCoordinateSystems is set to true in the IWebXRMeshDetectorOptions.
     */
    transformationMatrix?: Matrix;
}
/**
 * The mesh detector is used to detect meshes in the real world when in AR
 */
export class WebXRMeshDetector extends WebXRAbstractFeature {
    private _options;
    private _detectedMeshes;
    /**
     * The module's name
     */
    static readonly Name: string;
    /**
     * The (Babylon) version of this module.
     * This is an integer representing the implementation version.
     * This number does not correspond to the WebXR specs version
     */
    static readonly Version: number;
    /**
     * Observers registered here will be executed when a new mesh was added to the session
     */
    onMeshAddedObservable: Observable<IWebXRVertexData>;
    /**
     * Observers registered here will be executed when a mesh is no longer detected in the session
     */
    onMeshRemovedObservable: Observable<IWebXRVertexData>;
    /**
     * Observers registered here will be executed when an existing mesh updates
     */
    onMeshUpdatedObservable: Observable<IWebXRVertexData>;
    constructor(_xrSessionManager: WebXRSessionManager, _options?: IWebXRMeshDetectorOptions);
    detach(): boolean;
    dispose(): void;
    protected _onXRFrame(frame: XRFrame): void;
    private _init;
    private _updateVertexDataWithXRMesh;
}

}
declare module "babylonjs/XR/features/WebXRNearInteraction" {
import { WebXRControllerPointerSelection } from "babylonjs/XR/features/WebXRControllerPointerSelection";
import { WebXRSessionManager } from "babylonjs/XR/webXRSessionManager";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { WebXRInput } from "babylonjs/XR/webXRInput";
import { WebXRInputSource } from "babylonjs/XR/webXRInputSource";
import { Scene } from "babylonjs/scene";
import { Nullable } from "babylonjs/types";
import { PickingInfo } from "babylonjs/Collisions/pickingInfo";
import { WebXRAbstractFeature } from "babylonjs/XR/features/WebXRAbstractFeature";
import { BoundingSphere } from "babylonjs/Culling/boundingSphere";
import { Color3 } from "babylonjs/Maths/math.color";
import { Material } from "babylonjs/Materials/material";
import "babylonjs/Meshes/subMesh.project";
/**
 * Where should the near interaction mesh be attached to when using a motion controller for near interaction
 */
export enum WebXRNearControllerMode {
    /**
     * Motion controllers will not support near interaction
     */
    DISABLED = 0,
    /**
     * The interaction point for motion controllers will be inside of them
     */
    CENTERED_ON_CONTROLLER = 1,
    /**
     * The interaction point for motion controllers will be in front of the controller
     */
    CENTERED_IN_FRONT = 2
}
/**
 * Options interface for the near interaction module
 */
export interface IWebXRNearInteractionOptions {
    /**
     * If provided, this scene will be used to render meshes.
     */
    customUtilityLayerScene?: Scene;
    /**
     * Should meshes created here be added to a utility layer or the main scene
     */
    useUtilityLayer?: boolean;
    /**
     * The xr input to use with this near interaction
     */
    xrInput: WebXRInput;
    /**
     * Enable near interaction on all controllers instead of switching between them
     */
    enableNearInteractionOnAllControllers?: boolean;
    /**
     * The preferred hand to give the near interaction to. This will be prioritized when the controller initialize.
     * If switch is enabled, it will still allow the user to switch between the different controllers
     */
    preferredHandedness?: XRHandedness;
    /**
     * Disable switching the near interaction from one controller to the other.
     * If the preferred hand is set it will be fixed on this hand, and if not it will be fixed on the first controller added to the scene
     */
    disableSwitchOnClick?: boolean;
    /**
     * Far interaction feature to toggle when near interaction takes precedence
     */
    farInteractionFeature?: WebXRControllerPointerSelection;
    /**
     * Near interaction mode for motion controllers
     */
    nearInteractionControllerMode?: WebXRNearControllerMode;
    /**
     * Optional material for the motion controller orb, if enabled
     */
    motionControllerOrbMaterial?: Material;
}
/**
 * A module that will enable near interaction near interaction for hands and motion controllers of XR Input Sources
 */
export class WebXRNearInteraction extends WebXRAbstractFeature {
    private readonly _options;
    private static _IdCounter;
    private _tmpRay;
    private _attachController;
    private _controllers;
    private _scene;
    private _attachedController;
    private _farInteractionFeature;
    /**
     * The module's name
     */
    static readonly Name: string;
    /**
     * The (Babylon) version of this module.
     * This is an integer representing the implementation version.
     * This number does not correspond to the WebXR specs version
     */
    static readonly Version: number;
    /**
     * default color of the selection ring
     */
    selectionMeshDefaultColor: Color3;
    /**
     * This color will be applied to the selection ring when selection is triggered
     */
    selectionMeshPickedColor: Color3;
    /**
     * constructs a new background remover module
     * @param _xrSessionManager the session manager for this module
     * @param _options read-only options to be used in this module
     */
    constructor(_xrSessionManager: WebXRSessionManager, _options: IWebXRNearInteractionOptions);
    /**
     * Attach this feature
     * Will usually be called by the features manager
     *
     * @returns true if successful.
     */
    attach(): boolean;
    /**
     * Detach this feature.
     * Will usually be called by the features manager
     *
     * @returns true if successful.
     */
    detach(): boolean;
    /**
     * Will get the mesh under a specific pointer.
     * `scene.meshUnderPointer` will only return one mesh - either left or right.
     * @param controllerId the controllerId to check
     * @returns The mesh under pointer or null if no mesh is under the pointer
     */
    getMeshUnderPointer(controllerId: string): Nullable<AbstractMesh>;
    /**
     * Get the xr controller that correlates to the pointer id in the pointer event
     *
     * @param id the pointer id to search for
     * @returns the controller that correlates to this id or null if not found
     */
    getXRControllerByPointerId(id: number): Nullable<WebXRInputSource>;
    /**
     * This function sets webXRControllerPointerSelection feature that will be disabled when
     * the hover range is reached for a mesh and will be reattached when not in hover range.
     * This is used to remove the selection rays when moving.
     * @param farInteractionFeature the feature to disable when finger is in hover range for a mesh
     */
    setFarInteractionFeature(farInteractionFeature: Nullable<WebXRControllerPointerSelection>): void;
    /**
     * Filter used for near interaction pick and hover
     * @param mesh
     */
    private _nearPickPredicate;
    /**
     * Filter used for near interaction grab
     * @param mesh
     */
    private _nearGrabPredicate;
    /**
     * Filter used for any near interaction
     * @param mesh
     */
    private _nearInteractionPredicate;
    private _controllerAvailablePredicate;
    private _handleTransitionAnimation;
    private readonly _hoverRadius;
    private readonly _pickRadius;
    private readonly _controllerPickRadius;
    private readonly _nearGrabLengthScale;
    private _processTouchPoint;
    protected _onXRFrame(_xrFrame: XRFrame): void;
    private get _utilityLayerScene();
    private _generateVisualCue;
    private _isControllerReadyForNearInteraction;
    private _attachNearInteractionMode;
    private _detachController;
    private _generateNewTouchPointMesh;
    private _pickWithSphere;
    /**
     * Picks a mesh with a sphere
     * @param mesh the mesh to pick
     * @param sphere picking sphere in world coordinates
     * @param skipBoundingInfo a boolean indicating if we should skip the bounding info check
     * @returns the picking info
     */
    static PickMeshWithSphere(mesh: AbstractMesh, sphere: BoundingSphere, skipBoundingInfo?: boolean): PickingInfo;
}

}
declare module "babylonjs/XR/features/WebXRPlaneDetector" {
import { TransformNode } from "babylonjs/Meshes/transformNode";
import { WebXRSessionManager } from "babylonjs/XR/webXRSessionManager";
import { Observable } from "babylonjs/Misc/observable";
import { Vector3, Matrix } from "babylonjs/Maths/math.vector";
import { WebXRAbstractFeature } from "babylonjs/XR/features/WebXRAbstractFeature";
/**
 * Options used in the plane detector module
 */
export interface IWebXRPlaneDetectorOptions {
    /**
     * The node to use to transform the local results to world coordinates
     */
    worldParentNode?: TransformNode;
    /**
     * If set to true a reference of the created planes will be kept until the next session starts
     * If not defined, planes will be removed from the array when the feature is detached or the session ended.
     */
    doNotRemovePlanesOnSessionEnded?: boolean;
    /**
     * Preferred detector configuration, not all preferred options will be supported by all platforms.
     */
    preferredDetectorOptions?: XRGeometryDetectorOptions;
}
/**
 * A babylon interface for a WebXR plane.
 * A Plane is actually a polygon, built from N points in space
 *
 * Supported in chrome 79, not supported in canary 81 ATM
 */
export interface IWebXRPlane {
    /**
     * a babylon-assigned ID for this polygon
     */
    id: number;
    /**
     * an array of vector3 points in babylon space. right/left hand system is taken into account.
     */
    polygonDefinition: Array<Vector3>;
    /**
     * A transformation matrix to apply on the mesh that will be built using the polygonDefinition
     * Local vs. World are decided if worldParentNode was provided or not in the options when constructing the module
     */
    transformationMatrix: Matrix;
    /**
     * the native xr-plane object
     */
    xrPlane: XRPlane;
}
/**
 * The plane detector is used to detect planes in the real world when in AR
 * For more information see https://github.com/immersive-web/real-world-geometry/
 */
export class WebXRPlaneDetector extends WebXRAbstractFeature {
    private _options;
    private _detectedPlanes;
    private _enabled;
    private _lastFrameDetected;
    /**
     * The module's name
     */
    static readonly Name: string;
    /**
     * The (Babylon) version of this module.
     * This is an integer representing the implementation version.
     * This number does not correspond to the WebXR specs version
     */
    static readonly Version: number;
    /**
     * Observers registered here will be executed when a new plane was added to the session
     */
    onPlaneAddedObservable: Observable<IWebXRPlane>;
    /**
     * Observers registered here will be executed when a plane is no longer detected in the session
     */
    onPlaneRemovedObservable: Observable<IWebXRPlane>;
    /**
     * Observers registered here will be executed when an existing plane updates (for example - expanded)
     * This can execute N times every frame
     */
    onPlaneUpdatedObservable: Observable<IWebXRPlane>;
    /**
     * construct a new Plane Detector
     * @param _xrSessionManager an instance of xr Session manager
     * @param _options configuration to use when constructing this feature
     */
    constructor(_xrSessionManager: WebXRSessionManager, _options?: IWebXRPlaneDetectorOptions);
    /**
     * detach this feature.
     * Will usually be called by the features manager
     *
     * @returns true if successful.
     */
    detach(): boolean;
    /**
     * Dispose this feature and all of the resources attached
     */
    dispose(): void;
    /**
     * Check if the needed objects are defined.
     * This does not mean that the feature is enabled, but that the objects needed are well defined.
     */
    isCompatible(): boolean;
    protected _onXRFrame(frame: XRFrame): void;
    private _init;
    private _updatePlaneWithXRPlane;
    /**
     * avoiding using Array.find for global support.
     * @param xrPlane the plane to find in the array
     */
    private _findIndexInPlaneArray;
}

}
declare module "babylonjs/XR/features/WebXRWalkingLocomotion" {
import { TransformNode } from "babylonjs/Meshes/transformNode";
import { WebXRCamera } from "babylonjs/XR/webXRCamera";
import { WebXRSessionManager } from "babylonjs/XR/webXRSessionManager";
import { WebXRAbstractFeature } from "babylonjs/XR/features/WebXRAbstractFeature";
/**
 * Options for the walking locomotion feature.
 */
export interface IWebXRWalkingLocomotionOptions {
    /**
     * The target to be moved by walking locomotion. This should be the transform node
     * which is the root of the XR space (i.e., the WebXRCamera's parent node). However,
     * for simple cases and legacy purposes, articulating the WebXRCamera itself is also
     * supported as a deprecated feature.
     */
    locomotionTarget: WebXRCamera | TransformNode;
}
/**
 * A module that will enable VR locomotion by detecting when the user walks in place.
 */
export class WebXRWalkingLocomotion extends WebXRAbstractFeature {
    /**
     * The module's name.
     */
    static get Name(): string;
    /**
     * The (Babylon) version of this module.
     * This is an integer representing the implementation version.
     * This number has no external basis.
     */
    static get Version(): number;
    private _sessionManager;
    private _up;
    private _forward;
    private _position;
    private _movement;
    private _walker;
    private _locomotionTarget;
    private _isLocomotionTargetWebXRCamera;
    /**
     * The target to be articulated by walking locomotion.
     * When the walking locomotion feature detects walking in place, this element's
     * X and Z coordinates will be modified to reflect locomotion. This target should
     * be either the XR space's origin (i.e., the parent node of the WebXRCamera) or
     * the WebXRCamera itself. Note that the WebXRCamera path will modify the position
     * of the WebXRCamera directly and is thus discouraged.
     */
    get locomotionTarget(): WebXRCamera | TransformNode;
    /**
     * The target to be articulated by walking locomotion.
     * When the walking locomotion feature detects walking in place, this element's
     * X and Z coordinates will be modified to reflect locomotion. This target should
     * be either the XR space's origin (i.e., the parent node of the WebXRCamera) or
     * the WebXRCamera itself. Note that the WebXRCamera path will modify the position
     * of the WebXRCamera directly and is thus discouraged.
     */
    set locomotionTarget(locomotionTarget: WebXRCamera | TransformNode);
    /**
     * Construct a new Walking Locomotion feature.
     * @param sessionManager manager for the current XR session
     * @param options creation options, prominently including the vector target for locomotion
     */
    constructor(sessionManager: WebXRSessionManager, options: IWebXRWalkingLocomotionOptions);
    /**
     * Checks whether this feature is compatible with the current WebXR session.
     * Walking locomotion is only compatible with "immersive-vr" sessions.
     * @returns true if compatible, false otherwise
     */
    isCompatible(): boolean;
    /**
     * Attaches the feature.
     * Typically called automatically by the features manager.
     * @returns true if attach succeeded, false otherwise
     */
    attach(): boolean;
    /**
     * Detaches the feature.
     * Typically called automatically by the features manager.
     * @returns true if detach succeeded, false otherwise
     */
    detach(): boolean;
    protected _onXRFrame(frame: XRFrame): void;
}

}
declare module "babylonjs/XR/index" {
export * from "babylonjs/XR/webXRCamera";
export * from "babylonjs/XR/webXREnterExitUI";
export * from "babylonjs/XR/webXRExperienceHelper";
export * from "babylonjs/XR/webXRInput";
export * from "babylonjs/XR/webXRInputSource";
export * from "babylonjs/XR/webXRManagedOutputCanvas";
export * from "babylonjs/XR/webXRTypes";
export * from "babylonjs/XR/webXRSessionManager";
export * from "babylonjs/XR/webXRDefaultExperience";
export * from "babylonjs/XR/webXRFeaturesManager";
export * from "babylonjs/XR/features/index";
export * from "babylonjs/XR/motionController/index";
export * from "babylonjs/XR/native/index";

}
declare module "babylonjs/XR/motionController/index" {
export * from "babylonjs/XR/motionController/webXRAbstractMotionController";
export * from "babylonjs/XR/motionController/webXRControllerComponent";
export * from "babylonjs/XR/motionController/webXRGenericHandController";
export * from "babylonjs/XR/motionController/webXRGenericMotionController";
export * from "babylonjs/XR/motionController/webXRMicrosoftMixedRealityController";
export * from "babylonjs/XR/motionController/webXRMotionControllerManager";
export * from "babylonjs/XR/motionController/webXROculusTouchMotionController";
export * from "babylonjs/XR/motionController/webXRHTCViveMotionController";
export * from "babylonjs/XR/motionController/webXRProfiledMotionController";

}
declare module "babylonjs/XR/motionController/webXRAbstractMotionController" {
import { IDisposable, Scene } from "babylonjs/scene";
import { WebXRControllerComponent } from "babylonjs/XR/motionController/webXRControllerComponent";
import { Observable } from "babylonjs/Misc/observable";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { Nullable } from "babylonjs/types";
/**
 * Handedness type in xrInput profiles. These can be used to define layouts in the Layout Map.
 */
export type MotionControllerHandedness = "none" | "left" | "right";
/**
 * The type of components available in motion controllers.
 * This is not the name of the component.
 */
export type MotionControllerComponentType = "trigger" | "squeeze" | "touchpad" | "thumbstick" | "button";
/**
 * The state of a controller component
 */
export type MotionControllerComponentStateType = "default" | "touched" | "pressed";
/**
 * The schema of motion controller layout.
 * No object will be initialized using this interface
 * This is used just to define the profile.
 */
export interface IMotionControllerLayout {
    /**
     * Path to load the assets. Usually relative to the base path
     */
    assetPath: string;
    /**
     * Available components (unsorted)
     */
    components: {
        /**
         * A map of component Ids
         */
        [componentId: string]: {
            /**
             * The type of input the component outputs
             */
            type: MotionControllerComponentType;
            /**
             * The indices of this component in the gamepad object
             */
            gamepadIndices: {
                /**
                 * Index of button
                 */
                button?: number;
                /**
                 * If available, index of x-axis
                 */
                xAxis?: number;
                /**
                 * If available, index of y-axis
                 */
                yAxis?: number;
            };
            /**
             * The mesh's root node name
             */
            rootNodeName: string;
            /**
             * Animation definitions for this model
             */
            visualResponses: {
                [stateKey: string]: {
                    /**
                     * What property will be animated
                     */
                    componentProperty: "xAxis" | "yAxis" | "button" | "state";
                    /**
                     * What states influence this visual response
                     */
                    states: MotionControllerComponentStateType[];
                    /**
                     * Type of animation - movement or visibility
                     */
                    valueNodeProperty: "transform" | "visibility";
                    /**
                     * Base node name to move. Its position will be calculated according to the min and max nodes
                     */
                    valueNodeName?: string;
                    /**
                     * Minimum movement node
                     */
                    minNodeName?: string;
                    /**
                     * Max movement node
                     */
                    maxNodeName?: string;
                };
            };
            /**
             * If touch enabled, what is the name of node to display user feedback
             */
            touchPointNodeName?: string;
        };
    };
    /**
     * Is it xr standard mapping or not
     */
    gamepadMapping: "" | "xr-standard";
    /**
     * Base root node of this entire model
     */
    rootNodeName: string;
    /**
     * Defines the main button component id
     */
    selectComponentId: string;
}
/**
 * A definition for the layout map in the input profile
 */
export interface IMotionControllerLayoutMap {
    /**
     * Layouts with handedness type as a key
     */
    [handedness: string]: IMotionControllerLayout;
}
/**
 * The XR Input profile schema
 * Profiles can be found here:
 * https://github.com/immersive-web/webxr-input-profiles/tree/master/packages/registry/profiles
 */
export interface IMotionControllerProfile {
    /**
     * fallback profiles for this profileId
     */
    fallbackProfileIds: string[];
    /**
     * The layout map, with handedness as key
     */
    layouts: IMotionControllerLayoutMap;
    /**
     * The id of this profile
     * correlates to the profile(s) in the xrInput.profiles array
     */
    profileId: string;
}
/**
 * A helper-interface for the 3 meshes needed for controller button animation
 * The meshes are provided to the _lerpButtonTransform function to calculate the current position of the value mesh
 */
export interface IMotionControllerButtonMeshMap {
    /**
     * the mesh that defines the pressed value mesh position.
     * This is used to find the max-position of this button
     */
    pressedMesh: AbstractMesh;
    /**
     * the mesh that defines the unpressed value mesh position.
     * This is used to find the min (or initial) position of this button
     */
    unpressedMesh: AbstractMesh;
    /**
     * The mesh that will be changed when value changes
     */
    valueMesh: AbstractMesh;
}
/**
 * A helper-interface for the 3 meshes needed for controller axis animation.
 * This will be expanded when touchpad animations are fully supported
 * The meshes are provided to the _lerpAxisTransform function to calculate the current position of the value mesh
 */
export interface IMotionControllerMeshMap {
    /**
     * the mesh that defines the maximum value mesh position.
     */
    maxMesh?: AbstractMesh;
    /**
     * the mesh that defines the minimum value mesh position.
     */
    minMesh?: AbstractMesh;
    /**
     * The mesh that will be changed when axis value changes
     */
    valueMesh?: AbstractMesh;
}
/**
 * The elements needed for change-detection of the gamepad objects in motion controllers
 */
export interface IMinimalMotionControllerObject {
    /**
     * Available axes of this controller
     */
    axes: number[];
    /**
     * An array of available buttons
     */
    buttons: Array<{
        /**
         * Value of the button/trigger
         */
        value: number;
        /**
         * If the button/trigger is currently touched
         */
        touched: boolean;
        /**
         * If the button/trigger is currently pressed
         */
        pressed: boolean;
    }>;
    /**
     * EXPERIMENTAL haptic support.
     */
    hapticActuators?: Array<{
        pulse: (value: number, duration: number) => Promise<boolean>;
    }>;
}
/**
 * An Abstract Motion controller
 * This class receives an xrInput and a profile layout and uses those to initialize the components
 * Each component has an observable to check for changes in value and state
 */
export abstract class WebXRAbstractMotionController implements IDisposable {
    protected scene: Scene;
    protected layout: IMotionControllerLayout;
    /**
     * The gamepad object correlating to this controller
     */
    gamepadObject: IMinimalMotionControllerObject;
    /**
     * handedness (left/right/none) of this controller
     */
    handedness: MotionControllerHandedness;
    /**
     * @internal
     */
    _doNotLoadControllerMesh: boolean;
    private _controllerCache?;
    private _initComponent;
    private _modelReady;
    /**
     * A map of components (WebXRControllerComponent) in this motion controller
     * Components have a ComponentType and can also have both button and axis definitions
     */
    readonly components: {
        [id: string]: WebXRControllerComponent;
    };
    /**
     * Disable the model's animation. Can be set at any time.
     */
    disableAnimation: boolean;
    /**
     * Observers registered here will be triggered when the model of this controller is done loading
     */
    onModelLoadedObservable: Observable<WebXRAbstractMotionController>;
    /**
     * The profile id of this motion controller
     */
    abstract profileId: string;
    /**
     * The root mesh of the model. It is null if the model was not yet initialized
     */
    rootMesh: Nullable<AbstractMesh>;
    /**
     * constructs a new abstract motion controller
     * @param scene the scene to which the model of the controller will be added
     * @param layout The profile layout to load
     * @param gamepadObject The gamepad object correlating to this controller
     * @param handedness handedness (left/right/none) of this controller
     * @param _doNotLoadControllerMesh set this flag to ignore the mesh loading
     * @param _controllerCache a cache holding controller models already loaded in this session
     */
    constructor(scene: Scene, layout: IMotionControllerLayout, 
    /**
     * The gamepad object correlating to this controller
     */
    gamepadObject: IMinimalMotionControllerObject, 
    /**
     * handedness (left/right/none) of this controller
     */
    handedness: MotionControllerHandedness, 
    /**
     * @internal
     */
    _doNotLoadControllerMesh?: boolean, _controllerCache?: {
        filename: string;
        path: string;
        meshes: AbstractMesh[];
    }[] | undefined);
    /**
     * Dispose this controller, the model mesh and all its components
     */
    dispose(): void;
    /**
     * Returns all components of specific type
     * @param type the type to search for
     * @returns an array of components with this type
     */
    getAllComponentsOfType(type: MotionControllerComponentType): WebXRControllerComponent[];
    /**
     * get a component based an its component id as defined in layout.components
     * @param id the id of the component
     * @returns the component correlates to the id or undefined if not found
     */
    getComponent(id: string): WebXRControllerComponent;
    /**
     * Get the list of components available in this motion controller
     * @returns an array of strings correlating to available components
     */
    getComponentIds(): string[];
    /**
     * Get the first component of specific type
     * @param type type of component to find
     * @returns a controller component or null if not found
     */
    getComponentOfType(type: MotionControllerComponentType): Nullable<WebXRControllerComponent>;
    /**
     * Get the main (Select) component of this controller as defined in the layout
     * @returns the main component of this controller
     */
    getMainComponent(): WebXRControllerComponent;
    /**
     * Loads the model correlating to this controller
     * When the mesh is loaded, the onModelLoadedObservable will be triggered
     * @returns A promise fulfilled with the result of the model loading
     */
    loadModel(): Promise<boolean>;
    /**
     * Update this model using the current XRFrame
     * @param xrFrame the current xr frame to use and update the model
     */
    updateFromXRFrame(xrFrame: XRFrame): void;
    /**
     * Backwards compatibility due to a deeply-integrated typo
     */
    get handness(): MotionControllerHandedness;
    /**
     * Pulse (vibrate) this controller
     * If the controller does not support pulses, this function will fail silently and return Promise<false> directly after called
     * Consecutive calls to this function will cancel the last pulse call
     *
     * @param value the strength of the pulse in 0.0...1.0 range
     * @param duration Duration of the pulse in milliseconds
     * @param hapticActuatorIndex optional index of actuator (will usually be 0)
     * @returns a promise that will send true when the pulse has ended and false if the device doesn't support pulse or an error accrued
     */
    pulse(value: number, duration: number, hapticActuatorIndex?: number): Promise<boolean>;
    protected _getChildByName(node: AbstractMesh, name: string): AbstractMesh | undefined;
    protected _getImmediateChildByName(node: AbstractMesh, name: string): AbstractMesh | undefined;
    /**
     * Moves the axis on the controller mesh based on its current state
     * @param axisMap
     * @param axisValue the value of the axis which determines the meshes new position
     * @internal
     */
    protected _lerpTransform(axisMap: IMotionControllerMeshMap, axisValue: number, fixValueCoordinates?: boolean): void;
    /**
     * Update the model itself with the current frame data
     * @param xrFrame the frame to use for updating the model mesh
     */
    protected updateModel(xrFrame: XRFrame): void;
    /**
     * Get the filename and path for this controller's model
     * @returns a map of filename and path
     */
    protected abstract _getFilenameAndPath(): {
        filename: string;
        path: string;
    };
    /**
     * This function is called before the mesh is loaded. It checks for loading constraints.
     * For example, this function can check if the GLB loader is available
     * If this function returns false, the generic controller will be loaded instead
     * @returns Is the client ready to load the mesh
     */
    protected abstract _getModelLoadingConstraints(): boolean;
    /**
     * This function will be called after the model was successfully loaded and can be used
     * for mesh transformations before it is available for the user
     * @param meshes the loaded meshes
     */
    protected abstract _processLoadedModel(meshes: AbstractMesh[]): void;
    /**
     * Set the root mesh for this controller. Important for the WebXR controller class
     * @param meshes the loaded meshes
     */
    protected abstract _setRootMesh(meshes: AbstractMesh[]): void;
    /**
     * A function executed each frame that updates the mesh (if needed)
     * @param xrFrame the current xrFrame
     */
    protected abstract _updateModel(xrFrame: XRFrame): void;
    private _getGenericFilenameAndPath;
    private _getGenericParentMesh;
}

}
declare module "babylonjs/XR/motionController/webXRControllerComponent" {
import { IMinimalMotionControllerObject, MotionControllerComponentType } from "babylonjs/XR/motionController/webXRAbstractMotionController";
import { Observable } from "babylonjs/Misc/observable";
import { IDisposable } from "babylonjs/scene";
/**
 * X-Y values for axes in WebXR
 */
export interface IWebXRMotionControllerAxesValue {
    /**
     * The value of the x axis
     */
    x: number;
    /**
     * The value of the y-axis
     */
    y: number;
}
/**
 * changed / previous values for the values of this component
 */
export interface IWebXRMotionControllerComponentChangesValues<T> {
    /**
     * current (this frame) value
     */
    current: T;
    /**
     * previous (last change) value
     */
    previous: T;
}
/**
 * Represents changes in the component between current frame and last values recorded
 */
export interface IWebXRMotionControllerComponentChanges {
    /**
     * will be populated with previous and current values if axes changed
     */
    axes?: IWebXRMotionControllerComponentChangesValues<IWebXRMotionControllerAxesValue>;
    /**
     * will be populated with previous and current values if pressed changed
     */
    pressed?: IWebXRMotionControllerComponentChangesValues<boolean>;
    /**
     * will be populated with previous and current values if touched changed
     */
    touched?: IWebXRMotionControllerComponentChangesValues<boolean>;
    /**
     * will be populated with previous and current values if value changed
     */
    value?: IWebXRMotionControllerComponentChangesValues<number>;
}
/**
 * This class represents a single component (for example button or thumbstick) of a motion controller
 */
export class WebXRControllerComponent implements IDisposable {
    /**
     * the id of this component
     */
    id: string;
    /**
     * the type of the component
     */
    type: MotionControllerComponentType;
    private _buttonIndex;
    private _axesIndices;
    private _axes;
    private _changes;
    private _currentValue;
    private _hasChanges;
    private _pressed;
    private _touched;
    /**
     * button component type
     */
    static BUTTON_TYPE: MotionControllerComponentType;
    /**
     * squeeze component type
     */
    static SQUEEZE_TYPE: MotionControllerComponentType;
    /**
     * Thumbstick component type
     */
    static THUMBSTICK_TYPE: MotionControllerComponentType;
    /**
     * Touchpad component type
     */
    static TOUCHPAD_TYPE: MotionControllerComponentType;
    /**
     * trigger component type
     */
    static TRIGGER_TYPE: MotionControllerComponentType;
    /**
     * If axes are available for this component (like a touchpad or thumbstick) the observers will be notified when
     * the axes data changes
     */
    onAxisValueChangedObservable: Observable<{
        x: number;
        y: number;
    }>;
    /**
     * Observers registered here will be triggered when the state of a button changes
     * State change is either pressed / touched / value
     */
    onButtonStateChangedObservable: Observable<WebXRControllerComponent>;
    /**
     * Creates a new component for a motion controller.
     * It is created by the motion controller itself
     *
     * @param id the id of this component
     * @param type the type of the component
     * @param _buttonIndex index in the buttons array of the gamepad
     * @param _axesIndices indices of the values in the axes array of the gamepad
     */
    constructor(
    /**
     * the id of this component
     */
    id: string, 
    /**
     * the type of the component
     */
    type: MotionControllerComponentType, _buttonIndex?: number, _axesIndices?: number[]);
    /**
     * The current axes data. If this component has no axes it will still return an object { x: 0, y: 0 }
     */
    get axes(): IWebXRMotionControllerAxesValue;
    /**
     * Get the changes. Elements will be populated only if they changed with their previous and current value
     */
    get changes(): IWebXRMotionControllerComponentChanges;
    /**
     * Return whether or not the component changed the last frame
     */
    get hasChanges(): boolean;
    /**
     * is the button currently pressed
     */
    get pressed(): boolean;
    /**
     * is the button currently touched
     */
    get touched(): boolean;
    /**
     * Get the current value of this component
     */
    get value(): number;
    /**
     * Dispose this component
     */
    dispose(): void;
    /**
     * Are there axes correlating to this component
     * @returns true is axes data is available
     */
    isAxes(): boolean;
    /**
     * Is this component a button (hence - pressable)
     * @returns true if can be pressed
     */
    isButton(): boolean;
    /**
     * update this component using the gamepad object it is in. Called on every frame
     * @param nativeController the native gamepad controller object
     */
    update(nativeController: IMinimalMotionControllerObject): void;
}

}
declare module "babylonjs/XR/motionController/webXRGenericHandController" {
import { IMinimalMotionControllerObject, MotionControllerHandedness } from "babylonjs/XR/motionController/webXRAbstractMotionController";
import { WebXRAbstractMotionController } from "babylonjs/XR/motionController/webXRAbstractMotionController";
import { Scene } from "babylonjs/scene";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
/**
 * A generic hand controller class that supports select and a secondary grasp
 */
export class WebXRGenericHandController extends WebXRAbstractMotionController {
    profileId: string;
    /**
     * Create a new hand controller object, without loading a controller model
     * @param scene the scene to use to create this controller
     * @param gamepadObject the corresponding gamepad object
     * @param handedness the handedness of the controller
     */
    constructor(scene: Scene, gamepadObject: IMinimalMotionControllerObject, handedness: MotionControllerHandedness);
    protected _getFilenameAndPath(): {
        filename: string;
        path: string;
    };
    protected _getModelLoadingConstraints(): boolean;
    protected _processLoadedModel(_meshes: AbstractMesh[]): void;
    protected _setRootMesh(meshes: AbstractMesh[]): void;
    protected _updateModel(): void;
}

}
declare module "babylonjs/XR/motionController/webXRGenericMotionController" {
import { IMinimalMotionControllerObject, MotionControllerHandedness } from "babylonjs/XR/motionController/webXRAbstractMotionController";
import { WebXRAbstractMotionController } from "babylonjs/XR/motionController/webXRAbstractMotionController";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { Scene } from "babylonjs/scene";
/**
 * A generic trigger-only motion controller for WebXR
 */
export class WebXRGenericTriggerMotionController extends WebXRAbstractMotionController {
    /**
     * Static version of the profile id of this controller
     */
    static ProfileId: string;
    profileId: string;
    constructor(scene: Scene, gamepadObject: IMinimalMotionControllerObject, handedness: MotionControllerHandedness);
    protected _getFilenameAndPath(): {
        filename: string;
        path: string;
    };
    protected _getModelLoadingConstraints(): boolean;
    protected _processLoadedModel(meshes: AbstractMesh[]): void;
    protected _setRootMesh(meshes: AbstractMesh[]): void;
    protected _updateModel(): void;
}

}
declare module "babylonjs/XR/motionController/webXRHTCViveMotionController" {
import { IMinimalMotionControllerObject, MotionControllerHandedness } from "babylonjs/XR/motionController/webXRAbstractMotionController";
import { WebXRAbstractMotionController } from "babylonjs/XR/motionController/webXRAbstractMotionController";
import { Scene } from "babylonjs/scene";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
/**
 * The motion controller class for the standard HTC-Vive controllers
 */
export class WebXRHTCViveMotionController extends WebXRAbstractMotionController {
    private _modelRootNode;
    /**
     * The base url used to load the left and right controller models
     */
    static MODEL_BASE_URL: string;
    /**
     * File name for the controller model.
     */
    static MODEL_FILENAME: string;
    profileId: string;
    /**
     * Create a new Vive motion controller object
     * @param scene the scene to use to create this controller
     * @param gamepadObject the corresponding gamepad object
     * @param handedness the handedness of the controller
     */
    constructor(scene: Scene, gamepadObject: IMinimalMotionControllerObject, handedness: MotionControllerHandedness);
    protected _getFilenameAndPath(): {
        filename: string;
        path: string;
    };
    protected _getModelLoadingConstraints(): boolean;
    protected _processLoadedModel(_meshes: AbstractMesh[]): void;
    protected _setRootMesh(meshes: AbstractMesh[]): void;
    protected _updateModel(): void;
}

}
declare module "babylonjs/XR/motionController/webXRMicrosoftMixedRealityController" {
import { IMinimalMotionControllerObject, MotionControllerHandedness } from "babylonjs/XR/motionController/webXRAbstractMotionController";
import { WebXRAbstractMotionController } from "babylonjs/XR/motionController/webXRAbstractMotionController";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { Scene } from "babylonjs/scene";
/**
 * The motion controller class for all microsoft mixed reality controllers
 */
export class WebXRMicrosoftMixedRealityController extends WebXRAbstractMotionController {
    protected readonly _mapping: {
        defaultButton: {
            valueNodeName: string;
            unpressedNodeName: string;
            pressedNodeName: string;
        };
        defaultAxis: {
            valueNodeName: string;
            minNodeName: string;
            maxNodeName: string;
        };
        buttons: {
            "xr-standard-trigger": {
                rootNodeName: string;
                componentProperty: string;
                states: string[];
            };
            "xr-standard-squeeze": {
                rootNodeName: string;
                componentProperty: string;
                states: string[];
            };
            "xr-standard-touchpad": {
                rootNodeName: string;
                labelAnchorNodeName: string;
                touchPointNodeName: string;
            };
            "xr-standard-thumbstick": {
                rootNodeName: string;
                componentProperty: string;
                states: string[];
            };
        };
        axes: {
            "xr-standard-touchpad": {
                "x-axis": {
                    rootNodeName: string;
                };
                "y-axis": {
                    rootNodeName: string;
                };
            };
            "xr-standard-thumbstick": {
                "x-axis": {
                    rootNodeName: string;
                };
                "y-axis": {
                    rootNodeName: string;
                };
            };
        };
    };
    /**
     * The base url used to load the left and right controller models
     */
    static MODEL_BASE_URL: string;
    /**
     * The name of the left controller model file
     */
    static MODEL_LEFT_FILENAME: string;
    /**
     * The name of the right controller model file
     */
    static MODEL_RIGHT_FILENAME: string;
    profileId: string;
    constructor(scene: Scene, gamepadObject: IMinimalMotionControllerObject, handedness: MotionControllerHandedness);
    protected _getFilenameAndPath(): {
        filename: string;
        path: string;
    };
    protected _getModelLoadingConstraints(): boolean;
    protected _processLoadedModel(_meshes: AbstractMesh[]): void;
    protected _setRootMesh(meshes: AbstractMesh[]): void;
    protected _updateModel(): void;
}

}
declare module "babylonjs/XR/motionController/webXRMotionControllerManager" {
import { WebXRAbstractMotionController } from "babylonjs/XR/motionController/webXRAbstractMotionController";
import { Scene } from "babylonjs/scene";
/**
 * A construction function type to create a new controller based on an xrInput object
 */
export type MotionControllerConstructor = (xrInput: XRInputSource, scene: Scene) => WebXRAbstractMotionController;
/**
 * Motion controller manager is managing the different webxr profiles and makes sure the right
 * controller is being loaded.
 */
export class WebXRMotionControllerManager {
    private static _AvailableControllers;
    private static _Fallbacks;
    private static _ProfileLoadingPromises;
    private static _ProfilesList;
    /**
     * The base URL of the online controller repository. Can be changed at any time.
     */
    static BaseRepositoryUrl: string;
    /**
     * Which repository gets priority - local or online
     */
    static PrioritizeOnlineRepository: boolean;
    /**
     * Use the online repository, or use only locally-defined controllers
     */
    static UseOnlineRepository: boolean;
    /**
     * Disable the controller cache and load the models each time a new WebXRProfileMotionController is loaded.
     * Defaults to true.
     */
    static DisableControllerCache: boolean;
    /**
     * Clear the cache used for profile loading and reload when requested again
     */
    static ClearProfilesCache(): void;
    /**
     * Register the default fallbacks.
     * This function is called automatically when this file is imported.
     */
    static DefaultFallbacks(): void;
    /**
     * Find a fallback profile if the profile was not found. There are a few predefined generic profiles.
     * @param profileId the profile to which a fallback needs to be found
     * @returns an array with corresponding fallback profiles
     */
    static FindFallbackWithProfileId(profileId: string): string[];
    /**
     * When acquiring a new xrInput object (usually by the WebXRInput class), match it with the correct profile.
     * The order of search:
     *
     * 1) Iterate the profiles array of the xr input and try finding a corresponding motion controller
     * 2) (If not found) search in the gamepad id and try using it (legacy versions only)
     * 3) search for registered fallbacks (should be redundant, nonetheless it makes sense to check)
     * 4) return the generic trigger controller if none were found
     *
     * @param xrInput the xrInput to which a new controller is initialized
     * @param scene the scene to which the model will be added
     * @param forceProfile force a certain profile for this controller
     * @returns A promise that fulfils with the motion controller class for this profile id or the generic standard class if none was found
     */
    static GetMotionControllerWithXRInput(xrInput: XRInputSource, scene: Scene, forceProfile?: string): Promise<WebXRAbstractMotionController>;
    /**
     * Register a new controller based on its profile. This function will be called by the controller classes themselves.
     *
     * If you are missing a profile, make sure it is imported in your source, otherwise it will not register.
     *
     * @param type the profile type to register
     * @param constructFunction the function to be called when loading this profile
     */
    static RegisterController(type: string, constructFunction: MotionControllerConstructor): void;
    /**
     * Register a fallback to a specific profile.
     * @param profileId the profileId that will receive the fallbacks
     * @param fallbacks A list of fallback profiles
     */
    static RegisterFallbacksForProfileId(profileId: string, fallbacks: string[]): void;
    /**
     * Will update the list of profiles available in the repository
     * @returns a promise that resolves to a map of profiles available online
     */
    static UpdateProfilesList(): Promise<{
        [profile: string]: string;
    }>;
    /**
     * Clear the controller's cache (usually happens at the end of a session)
     */
    static ClearControllerCache(): void;
    private static _LoadProfileFromRepository;
    private static _LoadProfilesFromAvailableControllers;
}

}
declare module "babylonjs/XR/motionController/webXROculusTouchMotionController" {
import { IMinimalMotionControllerObject, MotionControllerHandedness } from "babylonjs/XR/motionController/webXRAbstractMotionController";
import { WebXRAbstractMotionController } from "babylonjs/XR/motionController/webXRAbstractMotionController";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { Scene } from "babylonjs/scene";
/**
 * The motion controller class for oculus touch (quest, rift).
 * This class supports legacy mapping as well the standard xr mapping
 */
export class WebXROculusTouchMotionController extends WebXRAbstractMotionController {
    private _forceLegacyControllers;
    private _modelRootNode;
    /**
     * The base url used to load the left and right controller models
     */
    static MODEL_BASE_URL: string;
    /**
     * The name of the left controller model file
     */
    static MODEL_LEFT_FILENAME: string;
    /**
     * The name of the right controller model file
     */
    static MODEL_RIGHT_FILENAME: string;
    /**
     * Base Url for the Quest controller model.
     */
    static QUEST_MODEL_BASE_URL: string;
    profileId: string;
    constructor(scene: Scene, gamepadObject: IMinimalMotionControllerObject, handedness: MotionControllerHandedness, _legacyMapping?: boolean, _forceLegacyControllers?: boolean);
    protected _getFilenameAndPath(): {
        filename: string;
        path: string;
    };
    protected _getModelLoadingConstraints(): boolean;
    protected _processLoadedModel(_meshes: AbstractMesh[]): void;
    protected _setRootMesh(meshes: AbstractMesh[]): void;
    protected _updateModel(): void;
    /**
     * Is this the new type of oculus touch. At the moment both have the same profile and it is impossible to differentiate
     * between the touch and touch 2.
     */
    private _isQuest;
}

}
declare module "babylonjs/XR/motionController/webXRProfiledMotionController" {
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { IMotionControllerProfile } from "babylonjs/XR/motionController/webXRAbstractMotionController";
import { WebXRAbstractMotionController } from "babylonjs/XR/motionController/webXRAbstractMotionController";
import { Scene } from "babylonjs/scene";
/**
 * A profiled motion controller has its profile loaded from an online repository.
 * The class is responsible of loading the model, mapping the keys and enabling model-animations
 */
export class WebXRProfiledMotionController extends WebXRAbstractMotionController {
    private _repositoryUrl;
    private controllerCache?;
    private _buttonMeshMapping;
    private _touchDots;
    /**
     * The profile ID of this controller. Will be populated when the controller initializes.
     */
    profileId: string;
    constructor(scene: Scene, xrInput: XRInputSource, _profile: IMotionControllerProfile, _repositoryUrl: string, controllerCache?: {
        filename: string;
        path: string;
        meshes: AbstractMesh[];
    }[] | undefined);
    dispose(): void;
    protected _getFilenameAndPath(): {
        filename: string;
        path: string;
    };
    protected _getModelLoadingConstraints(): boolean;
    protected _processLoadedModel(_meshes: AbstractMesh[]): void;
    protected _setRootMesh(meshes: AbstractMesh[]): void;
    protected _updateModel(_xrFrame: XRFrame): void;
}

}
declare module "babylonjs/XR/native/index" {
export * from "babylonjs/XR/native/nativeXRRenderTarget";
export * from "babylonjs/XR/native/nativeXRFrame";

}
declare module "babylonjs/XR/native/nativeXRFrame" {
/** @internal */
interface INativeXRFrame extends XRFrame {
    getPoseData: (space: XRSpace, baseSpace: XRReferenceSpace, vectorBuffer: ArrayBuffer, matrixBuffer: ArrayBuffer) => XRPose;
    _imageTrackingResults?: XRImageTrackingResult[];
}
/** @internal */
export class NativeXRFrame implements XRFrame {
    private _nativeImpl;
    private readonly _xrTransform;
    private readonly _xrPose;
    private readonly _xrPoseVectorData;
    get session(): XRSession;
    constructor(_nativeImpl: INativeXRFrame);
    getPose(space: XRSpace, baseSpace: XRReferenceSpace): XRPose | undefined;
    readonly fillPoses: any;
    readonly getViewerPose: any;
    readonly getHitTestResults: any;
    readonly getHitTestResultsForTransientInput: () => never;
    get trackedAnchors(): XRAnchorSet | undefined;
    readonly createAnchor: any;
    get worldInformation(): XRWorldInformation | undefined;
    get detectedPlanes(): XRPlaneSet | undefined;
    readonly getJointPose: any;
    readonly fillJointRadii: any;
    readonly getLightEstimate: () => never;
    get featurePointCloud(): number[] | undefined;
    readonly getImageTrackingResults: () => XRImageTrackingResult[];
}
export {};

}
declare module "babylonjs/XR/native/nativeXRRenderTarget" {
import { RenderTargetTexture } from "babylonjs/Materials/Textures/renderTargetTexture";
import { Viewport } from "babylonjs/Maths/math.viewport";
import { Nullable } from "babylonjs/types";
import { WebXRLayerWrapper } from "babylonjs/XR/webXRLayerWrapper";
import { WebXRLayerRenderTargetTextureProvider } from "babylonjs/XR/webXRRenderTargetTextureProvider";
import { WebXRSessionManager } from "babylonjs/XR/webXRSessionManager";
import { WebXRRenderTarget } from "babylonjs/XR/webXRTypes";
/**
 * Wraps XRWebGLLayer's created by Babylon Native.
 * @internal
 */
export class NativeXRLayerWrapper extends WebXRLayerWrapper {
    readonly layer: XRWebGLLayer;
    constructor(layer: XRWebGLLayer);
}
/**
 * Provides render target textures for layers created by Babylon Native.
 * @internal
 */
export class NativeXRLayerRenderTargetTextureProvider extends WebXRLayerRenderTargetTextureProvider {
    readonly layerWrapper: NativeXRLayerWrapper;
    private _nativeRTTProvider;
    private _nativeLayer;
    constructor(sessionManager: WebXRSessionManager, layerWrapper: NativeXRLayerWrapper);
    trySetViewportForView(viewport: Viewport): boolean;
    getRenderTargetTextureForEye(eye: XREye): Nullable<RenderTargetTexture>;
    getRenderTargetTextureForView(view: XRView): Nullable<RenderTargetTexture>;
    getFramebufferDimensions(): Nullable<{
        framebufferWidth: number;
        framebufferHeight: number;
    }>;
}
/**
 * Creates the xr layer that will be used as the xr session's base layer.
 * @internal
 */
export class NativeXRRenderTarget implements WebXRRenderTarget {
    canvasContext: WebGLRenderingContext;
    xrLayer: Nullable<XRWebGLLayer>;
    private _nativeRenderTarget;
    constructor(_xrSessionManager: WebXRSessionManager);
    initializeXRLayerAsync(xrSession: XRSession): Promise<XRWebGLLayer>;
    dispose(): void;
}

}
declare module "babylonjs/XR/webXRCamera" {
import { Vector3 } from "babylonjs/Maths/math.vector";
import { Scene } from "babylonjs/scene";
import { Camera } from "babylonjs/Cameras/camera";
import { FreeCamera } from "babylonjs/Cameras/freeCamera";
import { WebXRSessionManager } from "babylonjs/XR/webXRSessionManager";
import { Observable } from "babylonjs/Misc/observable";
import { WebXRTrackingState } from "babylonjs/XR/webXRTypes";
/**
 * WebXR Camera which holds the views for the xrSession
 * @see https://doc.babylonjs.com/features/featuresDeepDive/webXR/webXRCamera
 */
export class WebXRCamera extends FreeCamera {
    private _xrSessionManager;
    private static _ScaleReadOnly;
    private _firstFrame;
    private _referenceQuaternion;
    private _referencedPosition;
    private _trackingState;
    /**
     * Observable raised before camera teleportation
     */
    onBeforeCameraTeleport: Observable<Vector3>;
    /**
     *  Observable raised after camera teleportation
     */
    onAfterCameraTeleport: Observable<Vector3>;
    /**
     * Notifies when the camera's tracking state has changed.
     * Notice - will also be triggered when tracking has started (at the beginning of the session)
     */
    onTrackingStateChanged: Observable<WebXRTrackingState>;
    /**
     * Should position compensation execute on first frame.
     * This is used when copying the position from a native (non XR) camera
     */
    compensateOnFirstFrame: boolean;
    /**
     * The last XRViewerPose from the current XRFrame
     * @internal
     */
    _lastXRViewerPose?: XRViewerPose;
    /**
     * Creates a new webXRCamera, this should only be set at the camera after it has been updated by the xrSessionManager
     * @param name the name of the camera
     * @param scene the scene to add the camera to
     * @param _xrSessionManager a constructed xr session manager
     */
    constructor(name: string, scene: Scene, _xrSessionManager: WebXRSessionManager);
    /**
     * Get the current XR tracking state of the camera
     */
    get trackingState(): WebXRTrackingState;
    private _setTrackingState;
    /**
     * Return the user's height, unrelated to the current ground.
     * This will be the y position of this camera, when ground level is 0.
     */
    get realWorldHeight(): number;
    /** @internal */
    _updateForDualEyeDebugging(): void;
    /**
     * Sets this camera's transformation based on a non-vr camera
     * @param otherCamera the non-vr camera to copy the transformation from
     * @param resetToBaseReferenceSpace should XR reset to the base reference space
     */
    setTransformationFromNonVRCamera(otherCamera?: Camera, resetToBaseReferenceSpace?: boolean): void;
    /**
     * Gets the current instance class name ("WebXRCamera").
     * @returns the class name
     */
    getClassName(): string;
    dispose(): void;
    private _updateFromXRSession;
    private _updateNumberOfRigCameras;
    private _updateReferenceSpace;
}

}
declare module "babylonjs/XR/webXRDefaultExperience" {
import { WebXRExperienceHelper } from "babylonjs/XR/webXRExperienceHelper";
import { Scene } from "babylonjs/scene";
import { IWebXRInputOptions } from "babylonjs/XR/webXRInput";
import { WebXRInput } from "babylonjs/XR/webXRInput";
import { IWebXRControllerPointerSelectionOptions } from "babylonjs/XR/features/WebXRControllerPointerSelection";
import { WebXRControllerPointerSelection } from "babylonjs/XR/features/WebXRControllerPointerSelection";
import { IWebXRNearInteractionOptions } from "babylonjs/XR/features/WebXRNearInteraction";
import { WebXRNearInteraction } from "babylonjs/XR/features/WebXRNearInteraction";
import { WebXRRenderTarget } from "babylonjs/XR/webXRTypes";
import { WebXREnterExitUIOptions } from "babylonjs/XR/webXREnterExitUI";
import { WebXREnterExitUI } from "babylonjs/XR/webXREnterExitUI";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { WebXRManagedOutputCanvasOptions } from "babylonjs/XR/webXRManagedOutputCanvas";
import { IWebXRTeleportationOptions } from "babylonjs/XR/features/WebXRControllerTeleportation";
import { WebXRMotionControllerTeleportation } from "babylonjs/XR/features/WebXRControllerTeleportation";
/**
 * Options for the default xr helper
 */
export class WebXRDefaultExperienceOptions {
    /**
     * Enable or disable default UI to enter XR
     */
    disableDefaultUI?: boolean;
    /**
     * Should pointer selection not initialize.
     * Note that disabling pointer selection also disables teleportation.
     * Defaults to false.
     */
    disablePointerSelection?: boolean;
    /**
     * Should teleportation not initialize. Defaults to false.
     */
    disableTeleportation?: boolean;
    /**
     * Should nearInteraction not initialize. Defaults to false.
     */
    disableNearInteraction?: boolean;
    /**
     * Floor meshes that will be used for teleport
     */
    floorMeshes?: Array<AbstractMesh>;
    /**
     * If set to true, the first frame will not be used to reset position
     * The first frame is mainly used when copying transformation from the old camera
     * Mainly used in AR
     */
    ignoreNativeCameraTransformation?: boolean;
    /**
     * Optional configuration for the XR input object
     */
    inputOptions?: Partial<IWebXRInputOptions>;
    /**
     * optional configuration for pointer selection
     */
    pointerSelectionOptions?: Partial<IWebXRControllerPointerSelectionOptions>;
    /**
     * optional configuration for near interaction
     */
    nearInteractionOptions?: Partial<IWebXRNearInteractionOptions>;
    /**
     * optional configuration for teleportation
     */
    teleportationOptions?: Partial<IWebXRTeleportationOptions>;
    /**
     * optional configuration for the output canvas
     */
    outputCanvasOptions?: WebXRManagedOutputCanvasOptions;
    /**
     * optional UI options. This can be used among other to change session mode and reference space type
     */
    uiOptions?: Partial<WebXREnterExitUIOptions>;
    /**
     * When loading teleportation and pointer select, use stable versions instead of latest.
     */
    useStablePlugins?: boolean;
    /**
     * An optional rendering group id that will be set globally for teleportation, pointer selection and default controller meshes
     */
    renderingGroupId?: number;
    /**
     * A list of optional features to init the session with
     * If set to true, all features we support will be added
     */
    optionalFeatures?: boolean | string[];
}
/**
 * Default experience which provides a similar setup to the previous webVRExperience
 */
export class WebXRDefaultExperience {
    /**
     * Base experience
     */
    baseExperience: WebXRExperienceHelper;
    /**
     * Enables ui for entering/exiting xr
     */
    enterExitUI: WebXREnterExitUI;
    /**
     * Input experience extension
     */
    input: WebXRInput;
    /**
     * Enables laser pointer and selection
     */
    pointerSelection: WebXRControllerPointerSelection;
    /**
     * Default target xr should render to
     */
    renderTarget: WebXRRenderTarget;
    /**
     * Enables teleportation
     */
    teleportation: WebXRMotionControllerTeleportation;
    /**
     * Enables near interaction for hands/controllers
     */
    nearInteraction: WebXRNearInteraction;
    private constructor();
    /**
     * Creates the default xr experience
     * @param scene scene
     * @param options options for basic configuration
     * @returns resulting WebXRDefaultExperience
     */
    static CreateAsync(scene: Scene, options?: WebXRDefaultExperienceOptions): Promise<WebXRDefaultExperience>;
    /**
     * Disposes of the experience helper
     */
    dispose(): void;
}

}
declare module "babylonjs/XR/webXREnterExitUI" {
import { Nullable } from "babylonjs/types";
import { Observable } from "babylonjs/Misc/observable";
import { IDisposable, Scene } from "babylonjs/scene";
import { WebXRExperienceHelper } from "babylonjs/XR/webXRExperienceHelper";
import { WebXRRenderTarget } from "babylonjs/XR/webXRTypes";
/**
 * Button which can be used to enter a different mode of XR
 */
export class WebXREnterExitUIButton {
    /** button element */
    element: HTMLElement;
    /** XR initialization options for the button */
    sessionMode: XRSessionMode;
    /** Reference space type */
    referenceSpaceType: XRReferenceSpaceType;
    /**
     * Creates a WebXREnterExitUIButton
     * @param element button element
     * @param sessionMode XR initialization session mode
     * @param referenceSpaceType the type of reference space to be used
     */
    constructor(
    /** button element */
    element: HTMLElement, 
    /** XR initialization options for the button */
    sessionMode: XRSessionMode, 
    /** Reference space type */
    referenceSpaceType: XRReferenceSpaceType);
    /**
     * Extendable function which can be used to update the button's visuals when the state changes
     * @param activeButton the current active button in the UI
     */
    update(activeButton: Nullable<WebXREnterExitUIButton>): void;
}
/**
 * Options to create the webXR UI
 */
export class WebXREnterExitUIOptions {
    /**
     * User provided buttons to enable/disable WebXR. The system will provide default if not set
     */
    customButtons?: Array<WebXREnterExitUIButton>;
    /**
     * A reference space type to use when creating the default button.
     * Default is local-floor
     */
    referenceSpaceType?: XRReferenceSpaceType;
    /**
     * Context to enter xr with
     */
    renderTarget?: Nullable<WebXRRenderTarget>;
    /**
     * A session mode to use when creating the default button.
     * Default is immersive-vr
     */
    sessionMode?: XRSessionMode;
    /**
     * A list of optional features to init the session with
     */
    optionalFeatures?: string[];
    /**
     * A list of optional features to init the session with
     */
    requiredFeatures?: string[];
    /**
     * If set, the `sessiongranted` event will not be registered. `sessiongranted` is used to move seamlessly between WebXR experiences.
     * If set to true the user will be forced to press the "enter XR" button even if sessiongranted event was triggered.
     * If not set and a sessiongranted event was triggered, the XR session will start automatically.
     */
    ignoreSessionGrantedEvent?: boolean;
    /**
     * If defined, this function will be executed if the UI encounters an error when entering XR
     */
    onError?: (error: any) => void;
}
/**
 * UI to allow the user to enter/exit XR mode
 */
export class WebXREnterExitUI implements IDisposable {
    private _scene;
    /** version of the options passed to this UI */
    options: WebXREnterExitUIOptions;
    private _activeButton;
    private _buttons;
    private _helper;
    private _renderTarget?;
    /**
     * The HTML Div Element to which buttons are added.
     */
    readonly overlay: HTMLDivElement;
    /**
     * Fired every time the active button is changed.
     *
     * When xr is entered via a button that launches xr that button will be the callback parameter
     *
     * When exiting xr the callback parameter will be null)
     */
    activeButtonChangedObservable: Observable<Nullable<WebXREnterExitUIButton>>;
    /**
     * Construct a new EnterExit UI class
     *
     * @param _scene babylon scene object to use
     * @param options (read-only) version of the options passed to this UI
     */
    constructor(_scene: Scene, 
    /** version of the options passed to this UI */
    options: WebXREnterExitUIOptions);
    /**
     * Set the helper to be used with this UI component.
     * The UI is bound to an experience helper. If not provided the UI can still be used but the events should be registered by the developer.
     *
     * @param helper the experience helper to attach
     * @param renderTarget an optional render target (in case it is created outside of the helper scope)
     * @returns a promise that resolves when the ui is ready
     */
    setHelperAsync(helper: WebXRExperienceHelper, renderTarget?: WebXRRenderTarget): Promise<void>;
    /**
     * Creates UI to allow the user to enter/exit XR mode
     * @param scene the scene to add the ui to
     * @param helper the xr experience helper to enter/exit xr with
     * @param options options to configure the UI
     * @returns the created ui
     */
    static CreateAsync(scene: Scene, helper: WebXRExperienceHelper, options: WebXREnterExitUIOptions): Promise<WebXREnterExitUI>;
    private _enterXRWithButtonIndex;
    /**
     * Disposes of the XR UI component
     */
    dispose(): void;
    private _onSessionGranted;
    private _updateButtons;
}

}
declare module "babylonjs/XR/webXRExperienceHelper" {
import { Observable } from "babylonjs/Misc/observable";
import { IDisposable, Scene } from "babylonjs/scene";
import { WebXRSessionManager } from "babylonjs/XR/webXRSessionManager";
import { WebXRCamera } from "babylonjs/XR/webXRCamera";
import { WebXRRenderTarget } from "babylonjs/XR/webXRTypes";
import { WebXRState } from "babylonjs/XR/webXRTypes";
import { WebXRFeaturesManager } from "babylonjs/XR/webXRFeaturesManager";
/**
 * Options for setting up XR spectator camera.
 */
export interface WebXRSpectatorModeOption {
    /**
     * Expected refresh rate (frames per sec) for a spectator camera.
     */
    fps?: number;
    /**
     * The index of rigCameras array in a WebXR camera.
     */
    preferredCameraIndex?: number;
}
/**
 * Base set of functionality needed to create an XR experience (WebXRSessionManager, Camera, StateManagement, etc.)
 * @see https://doc.babylonjs.com/features/featuresDeepDive/webXR/webXRExperienceHelpers
 */
export class WebXRExperienceHelper implements IDisposable {
    private _scene;
    private _nonVRCamera;
    private _attachedToElement;
    private _spectatorCamera;
    private _originalSceneAutoClear;
    private _supported;
    private _spectatorMode;
    private _lastTimestamp;
    /**
     * Camera used to render xr content
     */
    camera: WebXRCamera;
    /** A features manager for this xr session */
    featuresManager: WebXRFeaturesManager;
    /**
     * Observers registered here will be triggered after the camera's initial transformation is set
     * This can be used to set a different ground level or an extra rotation.
     *
     * Note that ground level is considered to be at 0. The height defined by the XR camera will be added
     * to the position set after this observable is done executing.
     */
    onInitialXRPoseSetObservable: Observable<WebXRCamera>;
    /**
     * Fires when the state of the experience helper has changed
     */
    onStateChangedObservable: Observable<WebXRState>;
    /** Session manager used to keep track of xr session */
    sessionManager: WebXRSessionManager;
    /**
     * The current state of the XR experience (eg. transitioning, in XR or not in XR)
     */
    state: WebXRState;
    /**
     * Creates a WebXRExperienceHelper
     * @param _scene The scene the helper should be created in
     */
    private constructor();
    /**
     * Creates the experience helper
     * @param scene the scene to attach the experience helper to
     * @returns a promise for the experience helper
     */
    static CreateAsync(scene: Scene): Promise<WebXRExperienceHelper>;
    /**
     * Disposes of the experience helper
     */
    dispose(): void;
    /**
     * Enters XR mode (This must be done within a user interaction in most browsers eg. button click)
     * @param sessionMode options for the XR session
     * @param referenceSpaceType frame of reference of the XR session
     * @param renderTarget the output canvas that will be used to enter XR mode
     * @param sessionCreationOptions optional XRSessionInit object to init the session with
     * @returns promise that resolves after xr mode has entered
     */
    enterXRAsync(sessionMode: XRSessionMode, referenceSpaceType: XRReferenceSpaceType, renderTarget?: WebXRRenderTarget, sessionCreationOptions?: XRSessionInit): Promise<WebXRSessionManager>;
    /**
     * Exits XR mode and returns the scene to its original state
     * @returns promise that resolves after xr mode has exited
     */
    exitXRAsync(): Promise<void>;
    /**
     * Enable spectator mode for desktop VR experiences.
     * When spectator mode is enabled a camera will be attached to the desktop canvas and will
     * display the first rig camera's view on the desktop canvas.
     * Please note that this will degrade performance, as it requires another camera render.
     * It is also not recommended to enable this in devices like the quest, as it brings no benefit there.
     * @param options giving WebXRSpectatorModeOption for specutator camera to setup when the spectator mode is enabled.
     */
    enableSpectatorMode(options?: WebXRSpectatorModeOption): void;
    /**
     * Disable spectator mode for desktop VR experiences.
     */
    disableSpecatatorMode(): void;
    private _switchSpectatorMode;
    private _nonXRToXRCamera;
    private _setState;
}

}
declare module "babylonjs/XR/webXRFeaturesManager" {
import { WebXRSessionManager } from "babylonjs/XR/webXRSessionManager";
import { IDisposable } from "babylonjs/scene";
/**
 * Defining the interface required for a (webxr) feature
 */
export interface IWebXRFeature extends IDisposable {
    /**
     * Is this feature attached
     */
    attached: boolean;
    /**
     * Should auto-attach be disabled?
     */
    disableAutoAttach: boolean;
    /**
     * Attach the feature to the session
     * Will usually be called by the features manager
     *
     * @param force should attachment be forced (even when already attached)
     * @returns true if successful.
     */
    attach(force?: boolean): boolean;
    /**
     * Detach the feature from the session
     * Will usually be called by the features manager
     *
     * @returns true if successful.
     */
    detach(): boolean;
    /**
     * This function will be executed during before enabling the feature and can be used to not-allow enabling it.
     * Note that at this point the session has NOT started, so this is purely checking if the browser supports it
     *
     * @returns whether or not the feature is compatible in this environment
     */
    isCompatible(): boolean;
    /**
     * Was this feature disposed;
     */
    isDisposed: boolean;
    /**
     * The name of the native xr feature name, if applicable (like anchor, hit-test, or hand-tracking)
     */
    xrNativeFeatureName?: string;
    /**
     * A list of (Babylon WebXR) features this feature depends on
     */
    dependsOn?: string[];
    /**
     * If this feature requires to extend the XRSessionInit object, this function will return the partial XR session init object
     */
    getXRSessionInitExtension?: () => Promise<Partial<XRSessionInit>>;
}
/**
 * A list of the currently available features without referencing them
 */
export class WebXRFeatureName {
    /**
     * The name of the anchor system feature
     */
    static readonly ANCHOR_SYSTEM: string;
    /**
     * The name of the background remover feature
     */
    static readonly BACKGROUND_REMOVER: string;
    /**
     * The name of the hit test feature
     */
    static readonly HIT_TEST: string;
    /**
     * The name of the mesh detection feature
     */
    static readonly MESH_DETECTION: string;
    /**
     * physics impostors for xr controllers feature
     */
    static readonly PHYSICS_CONTROLLERS: string;
    /**
     * The name of the plane detection feature
     */
    static readonly PLANE_DETECTION: string;
    /**
     * The name of the pointer selection feature
     */
    static readonly POINTER_SELECTION: string;
    /**
     * The name of the teleportation feature
     */
    static readonly TELEPORTATION: string;
    /**
     * The name of the feature points feature.
     */
    static readonly FEATURE_POINTS: string;
    /**
     * The name of the hand tracking feature.
     */
    static readonly HAND_TRACKING: string;
    /**
     * The name of the image tracking feature
     */
    static readonly IMAGE_TRACKING: string;
    /**
     * The name of the near interaction feature
     */
    static readonly NEAR_INTERACTION: string;
    /**
     * The name of the DOM overlay feature
     */
    static readonly DOM_OVERLAY: string;
    /**
     * The name of the movement feature
     */
    static readonly MOVEMENT: string;
    /**
     * The name of the light estimation feature
     */
    static readonly LIGHT_ESTIMATION: string;
    /**
     * The name of the eye tracking feature
     */
    static readonly EYE_TRACKING: string;
    /**
     * The name of the walking locomotion feature
     */
    static readonly WALKING_LOCOMOTION: string;
    /**
     * The name of the composition layers feature
     */
    static readonly LAYERS: string;
}
/**
 * Defining the constructor of a feature. Used to register the modules.
 */
export type WebXRFeatureConstructor = (xrSessionManager: WebXRSessionManager, options?: any) => () => IWebXRFeature;
/**
 * The WebXR features manager is responsible of enabling or disabling features required for the current XR session.
 * It is mainly used in AR sessions.
 *
 * A feature can have a version that is defined by Babylon (and does not correspond with the webxr version).
 */
export class WebXRFeaturesManager implements IDisposable {
    private _xrSessionManager;
    private static readonly _AvailableFeatures;
    private _features;
    /**
     * The key is the feature to check and the value is the feature that conflicts.
     */
    private static readonly _ConflictingFeatures;
    /**
     * constructs a new features manages.
     *
     * @param _xrSessionManager an instance of WebXRSessionManager
     */
    constructor(_xrSessionManager: WebXRSessionManager);
    /**
     * Used to register a module. After calling this function a developer can use this feature in the scene.
     * Mainly used internally.
     *
     * @param featureName the name of the feature to register
     * @param constructorFunction the function used to construct the module
     * @param version the (babylon) version of the module
     * @param stable is that a stable version of this module
     */
    static AddWebXRFeature(featureName: string, constructorFunction: WebXRFeatureConstructor, version?: number, stable?: boolean): void;
    /**
     * Returns a constructor of a specific feature.
     *
     * @param featureName the name of the feature to construct
     * @param version the version of the feature to load
     * @param xrSessionManager the xrSessionManager. Used to construct the module
     * @param options optional options provided to the module.
     * @returns a function that, when called, will return a new instance of this feature
     */
    static ConstructFeature(featureName: string, version: number | undefined, xrSessionManager: WebXRSessionManager, options?: any): () => IWebXRFeature;
    /**
     * Can be used to return the list of features currently registered
     *
     * @returns an Array of available features
     */
    static GetAvailableFeatures(): string[];
    /**
     * Gets the versions available for a specific feature
     * @param featureName the name of the feature
     * @returns an array with the available versions
     */
    static GetAvailableVersions(featureName: string): string[];
    /**
     * Return the latest unstable version of this feature
     * @param featureName the name of the feature to search
     * @returns the version number. if not found will return -1
     */
    static GetLatestVersionOfFeature(featureName: string): number;
    /**
     * Return the latest stable version of this feature
     * @param featureName the name of the feature to search
     * @returns the version number. if not found will return -1
     */
    static GetStableVersionOfFeature(featureName: string): number;
    /**
     * Attach a feature to the current session. Mainly used when session started to start the feature effect.
     * Can be used during a session to start a feature
     * @param featureName the name of feature to attach
     */
    attachFeature(featureName: string): void;
    /**
     * Can be used inside a session or when the session ends to detach a specific feature
     * @param featureName the name of the feature to detach
     */
    detachFeature(featureName: string): void;
    /**
     * Used to disable an already-enabled feature
     * The feature will be disposed and will be recreated once enabled.
     * @param featureName the feature to disable
     * @returns true if disable was successful
     */
    disableFeature(featureName: string | {
        Name: string;
    }): boolean;
    /**
     * dispose this features manager
     */
    dispose(): void;
    /**
     * Enable a feature using its name and a version. This will enable it in the scene, and will be responsible to attach it when the session starts.
     * If used twice, the old version will be disposed and a new one will be constructed. This way you can re-enable with different configuration.
     *
     * @param featureName the name of the feature to load or the class of the feature
     * @param version optional version to load. if not provided the latest version will be enabled
     * @param moduleOptions options provided to the module. Ses the module documentation / constructor
     * @param attachIfPossible if set to true (default) the feature will be automatically attached, if it is currently possible
     * @param required is this feature required to the app. If set to true the session init will fail if the feature is not available.
     * @returns a new constructed feature or throws an error if feature not found or conflicts with another enabled feature.
     */
    enableFeature(featureName: string | {
        Name: string;
    }, version?: number | string, moduleOptions?: any, attachIfPossible?: boolean, required?: boolean): IWebXRFeature;
    /**
     * get the implementation of an enabled feature.
     * @param featureName the name of the feature to load
     * @returns the feature class, if found
     */
    getEnabledFeature(featureName: string): IWebXRFeature;
    /**
     * Get the list of enabled features
     * @returns an array of enabled features
     */
    getEnabledFeatures(): string[];
    /**
     * This function will extend the session creation configuration object with enabled features.
     * If, for example, the anchors feature is enabled, it will be automatically added to the optional or required features list,
     * according to the defined "required" variable, provided during enableFeature call
     * @param xrSessionInit the xr Session init object to extend
     *
     * @returns an extended XRSessionInit object
     */
    _extendXRSessionInitObject(xrSessionInit: XRSessionInit): Promise<XRSessionInit>;
}

}
declare module "babylonjs/XR/webXRInput" {
import { Observable } from "babylonjs/Misc/observable";
import { IDisposable } from "babylonjs/scene";
import { IWebXRControllerOptions } from "babylonjs/XR/webXRInputSource";
import { WebXRInputSource } from "babylonjs/XR/webXRInputSource";
import { WebXRSessionManager } from "babylonjs/XR/webXRSessionManager";
import { WebXRCamera } from "babylonjs/XR/webXRCamera";
/**
 * The schema for initialization options of the XR Input class
 */
export interface IWebXRInputOptions {
    /**
     * If set to true no model will be automatically loaded
     */
    doNotLoadControllerMeshes?: boolean;
    /**
     * If set, this profile will be used for all controllers loaded (for example "microsoft-mixed-reality")
     * If not found, the xr input profile data will be used.
     * Profiles are defined here - https://github.com/immersive-web/webxr-input-profiles/
     */
    forceInputProfile?: string;
    /**
     * Do not send a request to the controller repository to load the profile.
     *
     * Instead, use the controllers available in babylon itself.
     */
    disableOnlineControllerRepository?: boolean;
    /**
     * A custom URL for the controllers repository
     */
    customControllersRepositoryURL?: string;
    /**
     * Should the controller model's components not move according to the user input
     */
    disableControllerAnimation?: boolean;
    /**
     * Optional options to pass to the controller. Will be overridden by the Input options where applicable
     */
    controllerOptions?: IWebXRControllerOptions;
}
/**
 * XR input used to track XR inputs such as controllers/rays
 */
export class WebXRInput implements IDisposable {
    /**
     * the xr session manager for this session
     */
    xrSessionManager: WebXRSessionManager;
    /**
     * the WebXR camera for this session. Mainly used for teleportation
     */
    xrCamera: WebXRCamera;
    private readonly _options;
    /**
     * XR controllers being tracked
     */
    controllers: Array<WebXRInputSource>;
    private _frameObserver;
    private _sessionEndedObserver;
    private _sessionInitObserver;
    /**
     * Event when a controller has been connected/added
     */
    onControllerAddedObservable: Observable<WebXRInputSource>;
    /**
     * Event when a controller has been removed/disconnected
     */
    onControllerRemovedObservable: Observable<WebXRInputSource>;
    /**
     * Initializes the WebXRInput
     * @param xrSessionManager the xr session manager for this session
     * @param xrCamera the WebXR camera for this session. Mainly used for teleportation
     * @param _options = initialization options for this xr input
     */
    constructor(
    /**
     * the xr session manager for this session
     */
    xrSessionManager: WebXRSessionManager, 
    /**
     * the WebXR camera for this session. Mainly used for teleportation
     */
    xrCamera: WebXRCamera, _options?: IWebXRInputOptions);
    private _onInputSourcesChange;
    private _addAndRemoveControllers;
    /**
     * Disposes of the object
     */
    dispose(): void;
}

}
declare module "babylonjs/XR/webXRInputSource" {
import { Observable } from "babylonjs/Misc/observable";
import { AbstractMesh } from "babylonjs/Meshes/abstractMesh";
import { Ray } from "babylonjs/Culling/ray";
import { Scene } from "babylonjs/scene";
import { WebXRAbstractMotionController } from "babylonjs/XR/motionController/webXRAbstractMotionController";
import { WebXRCamera } from "babylonjs/XR/webXRCamera";
/**
 * Configuration options for the WebXR controller creation
 */
export interface IWebXRControllerOptions {
    /**
     * Should the controller mesh be animated when a user interacts with it
     * The pressed buttons / thumbstick and touchpad animations will be disabled
     */
    disableMotionControllerAnimation?: boolean;
    /**
     * Do not load the controller mesh, in case a different mesh needs to be loaded.
     */
    doNotLoadControllerMesh?: boolean;
    /**
     * Force a specific controller type for this controller.
     * This can be used when creating your own profile or when testing different controllers
     */
    forceControllerProfile?: string;
    /**
     * Defines a rendering group ID for meshes that will be loaded.
     * This is for the default controllers only.
     */
    renderingGroupId?: number;
}
/**
 * Represents an XR controller
 */
export class WebXRInputSource {
    private _scene;
    /** The underlying input source for the controller  */
    inputSource: XRInputSource;
    private _options;
    private _tmpVector;
    private _uniqueId;
    private _disposed;
    /**
     * Represents the part of the controller that is held. This may not exist if the controller is the head mounted display itself, if that's the case only the pointer from the head will be available
     */
    grip?: AbstractMesh;
    /**
     * If available, this is the gamepad object related to this controller.
     * Using this object it is possible to get click events and trackpad changes of the
     * webxr controller that is currently being used.
     */
    motionController?: WebXRAbstractMotionController;
    /**
     * Event that fires when the controller is removed/disposed.
     * The object provided as event data is this controller, after associated assets were disposed.
     * uniqueId is still available.
     */
    onDisposeObservable: Observable<WebXRInputSource>;
    /**
     * Will be triggered when the mesh associated with the motion controller is done loading.
     * It is also possible that this will never trigger (!) if no mesh was loaded, or if the developer decides to load a different mesh
     * A shortened version of controller -> motion controller -> on mesh loaded.
     */
    onMeshLoadedObservable: Observable<AbstractMesh>;
    /**
     * Observers registered here will trigger when a motion controller profile was assigned to this xr controller
     */
    onMotionControllerInitObservable: Observable<WebXRAbstractMotionController>;
    /**
     * Pointer which can be used to select objects or attach a visible laser to
     */
    pointer: AbstractMesh;
    /**
     * The last XRPose the was calculated on the current XRFrame
     * @internal
     */
    _lastXRPose?: XRPose;
    /**
     * Creates the input source object
     * @see https://doc.babylonjs.com/features/featuresDeepDive/webXR/webXRInputControllerSupport
     * @param _scene the scene which the controller should be associated to
     * @param inputSource the underlying input source for the controller
     * @param _options options for this controller creation
     */
    constructor(_scene: Scene, 
    /** The underlying input source for the controller  */
    inputSource: XRInputSource, _options?: IWebXRControllerOptions);
    /**
     * Get this controllers unique id
     */
    get uniqueId(): string;
    /**
     * Disposes of the object
     */
    dispose(): void;
    /**
     * Gets a world space ray coming from the pointer or grip
     * @param result the resulting ray
     * @param gripIfAvailable use the grip mesh instead of the pointer, if available
     */
    getWorldPointerRayToRef(result: Ray, gripIfAvailable?: boolean): void;
    /**
     * Updates the controller pose based on the given XRFrame
     * @param xrFrame xr frame to update the pose with
     * @param referenceSpace reference space to use
     * @param xrCamera the xr camera, used for parenting
     */
    updateFromXRFrame(xrFrame: XRFrame, referenceSpace: XRReferenceSpace, xrCamera: WebXRCamera): void;
}

}
declare module "babylonjs/XR/webXRLayerWrapper" {
import { Nullable } from "babylonjs/types";
import { WebXRLayerRenderTargetTextureProvider } from "babylonjs/XR/webXRRenderTargetTextureProvider";
import { WebXRSessionManager } from "babylonjs/XR/webXRSessionManager";
/** Covers all supported subclasses of WebXR's XRCompositionLayer */
export type WebXRCompositionLayerType = "XRProjectionLayer";
/** Covers all supported subclasses of WebXR's XRLayer */
export type WebXRLayerType = "XRWebGLLayer" | WebXRCompositionLayerType;
/**
 * Wrapper over subclasses of XRLayer.
 * @internal
 */
export class WebXRLayerWrapper {
    /** The width of the layer's framebuffer. */
    getWidth: () => number;
    /** The height of the layer's framebuffer. */
    getHeight: () => number;
    /** The XR layer that this WebXRLayerWrapper wraps. */
    readonly layer: XRLayer;
    /** The type of XR layer that is being wrapped. */
    readonly layerType: WebXRLayerType;
    /** Create a render target provider for the wrapped layer. */
    createRenderTargetTextureProvider: (xrSessionManager: WebXRSessionManager) => WebXRLayerRenderTargetTextureProvider;
    /**
     * Check if fixed foveation is supported on this device
     */
    get isFixedFoveationSupported(): boolean;
    /**
     * Get the fixed foveation currently set, as specified by the webxr specs
     * If this returns null, then fixed foveation is not supported
     */
    get fixedFoveation(): Nullable<number>;
    /**
     * Set the fixed foveation to the specified value, as specified by the webxr specs
     * This value will be normalized to be between 0 and 1, 1 being max foveation, 0 being no foveation
     */
    set fixedFoveation(value: Nullable<number>);
    protected constructor(
    /** The width of the layer's framebuffer. */
    getWidth: () => number, 
    /** The height of the layer's framebuffer. */
    getHeight: () => number, 
    /** The XR layer that this WebXRLayerWrapper wraps. */
    layer: XRLayer, 
    /** The type of XR layer that is being wrapped. */
    layerType: WebXRLayerType, 
    /** Create a render target provider for the wrapped layer. */
    createRenderTargetTextureProvider: (xrSessionManager: WebXRSessionManager) => WebXRLayerRenderTargetTextureProvider);
}

}
declare module "babylonjs/XR/webXRManagedOutputCanvas" {
import { Nullable } from "babylonjs/types";
import { ThinEngine } from "babylonjs/Engines/thinEngine";
import { WebXRRenderTarget } from "babylonjs/XR/webXRTypes";
import { WebXRSessionManager } from "babylonjs/XR/webXRSessionManager";
import { Observable } from "babylonjs/Misc/observable";
/**
 * Configuration object for WebXR output canvas
 */
export class WebXRManagedOutputCanvasOptions {
    /**
     * An optional canvas in case you wish to create it yourself and provide it here.
     * If not provided, a new canvas will be created
     */
    canvasElement?: HTMLCanvasElement;
    /**
     * Options for this XR Layer output
     */
    canvasOptions?: XRWebGLLayerInit;
    /**
     * CSS styling for a newly created canvas (if not provided)
     */
    newCanvasCssStyle?: string;
    /**
     * Get the default values of the configuration object
     * @param engine defines the engine to use (can be null)
     * @returns default values of this configuration object
     */
    static GetDefaults(engine?: ThinEngine): WebXRManagedOutputCanvasOptions;
}
/**
 * Creates a canvas that is added/removed from the webpage when entering/exiting XR
 */
export class WebXRManagedOutputCanvas implements WebXRRenderTarget {
    private _options;
    private _canvas;
    private _engine;
    private _originalCanvasSize;
    /**
     * Rendering context of the canvas which can be used to display/mirror xr content
     */
    canvasContext: WebGLRenderingContext;
    /**
     * xr layer for the canvas
     */
    xrLayer: Nullable<XRWebGLLayer>;
    private _xrLayerWrapper;
    /**
     * Observers registered here will be triggered when the xr layer was initialized
     */
    onXRLayerInitObservable: Observable<XRWebGLLayer>;
    /**
     * Initializes the canvas to be added/removed upon entering/exiting xr
     * @param _xrSessionManager The XR Session manager
     * @param _options optional configuration for this canvas output. defaults will be used if not provided
     */
    constructor(_xrSessionManager: WebXRSessionManager, _options?: WebXRManagedOutputCanvasOptions);
    /**
     * Disposes of the object
     */
    dispose(): void;
    /**
     * Initializes a XRWebGLLayer to be used as the session's baseLayer.
     * @param xrSession xr session
     * @returns a promise that will resolve once the XR Layer has been created
     */
    initializeXRLayerAsync(xrSession: XRSession): Promise<XRWebGLLayer>;
    private _addCanvas;
    private _removeCanvas;
    private _setCanvasSize;
    private _setManagedOutputCanvas;
}

}
declare module "babylonjs/XR/webXRRenderTargetTextureProvider" {
import { RenderTargetTexture } from "babylonjs/Materials/Textures/renderTargetTexture";
import { Viewport } from "babylonjs/Maths/math.viewport";
import { IDisposable, Scene } from "babylonjs/scene";
import { Nullable } from "babylonjs/types";
import { WebXRLayerWrapper } from "babylonjs/XR/webXRLayerWrapper";
/**
 * An interface for objects that provide render target textures for XR rendering.
 */
export interface IWebXRRenderTargetTextureProvider extends IDisposable {
    /**
     * Attempts to set the framebuffer-size-normalized viewport to be rendered this frame for this view.
     * In the event of a failure, the supplied viewport is not updated.
     * @param viewport the viewport to which the view will be rendered
     * @param view the view for which to set the viewport
     * @returns whether the operation was successful
     */
    trySetViewportForView(viewport: Viewport, view: XRView): boolean;
    /**
     * Gets the correct render target texture to be rendered this frame for this eye
     * @param eye the eye for which to get the render target
     * @returns the render target for the specified eye or null if not available
     */
    getRenderTargetTextureForEye(eye: XREye): Nullable<RenderTargetTexture>;
    /**
     * Gets the correct render target texture to be rendered this frame for this view
     * @param view the view for which to get the render target
     * @returns the render target for the specified view or null if not available
     */
    getRenderTargetTextureForView(view: XRView): Nullable<RenderTargetTexture>;
}
/**
 * Provides render target textures and other important rendering information for a given XRLayer.
 * @internal
 */
export abstract class WebXRLayerRenderTargetTextureProvider implements IWebXRRenderTargetTextureProvider {
    private readonly _scene;
    readonly layerWrapper: WebXRLayerWrapper;
    abstract trySetViewportForView(viewport: Viewport, view: XRView): boolean;
    abstract getRenderTargetTextureForEye(eye: XREye): Nullable<RenderTargetTexture>;
    abstract getRenderTargetTextureForView(view: XRView): Nullable<RenderTargetTexture>;
    protected _renderTargetTextures: RenderTargetTexture[];
    protected _framebufferDimensions: Nullable<{
        framebufferWidth: number;
        framebufferHeight: number;
    }>;
    private _engine;
    constructor(_scene: Scene, layerWrapper: WebXRLayerWrapper);
    private _createInternalTexture;
    protected _createRenderTargetTexture(width: number, height: number, framebuffer: Nullable<WebGLFramebuffer>, colorTexture?: WebGLTexture, depthStencilTexture?: WebGLTexture, multiview?: boolean): RenderTargetTexture;
    protected _destroyRenderTargetTexture(renderTargetTexture: RenderTargetTexture): void;
    getFramebufferDimensions(): Nullable<{
        framebufferWidth: number;
        framebufferHeight: number;
    }>;
    dispose(): void;
}

}
declare module "babylonjs/XR/webXRSessionManager" {
import { Observable } from "babylonjs/Misc/observable";
import { Nullable } from "babylonjs/types";
import { IDisposable, Scene } from "babylonjs/scene";
import { RenderTargetTexture } from "babylonjs/Materials/Textures/renderTargetTexture";
import { WebXRRenderTarget } from "babylonjs/XR/webXRTypes";
import { WebXRManagedOutputCanvasOptions } from "babylonjs/XR/webXRManagedOutputCanvas";
import { IWebXRRenderTargetTextureProvider } from "babylonjs/XR/webXRRenderTargetTextureProvider";
import { Viewport } from "babylonjs/Maths/math.viewport";
import { WebXRLayerWrapper } from "babylonjs/XR/webXRLayerWrapper";
/**
 * Manages an XRSession to work with Babylon's engine
 * @see https://doc.babylonjs.com/features/featuresDeepDive/webXR/webXRSessionManagers
 */
export class WebXRSessionManager implements IDisposable, IWebXRRenderTargetTextureProvider {
    /** The scene which the session should be created for */
    scene: Scene;
    private _engine;
    private _referenceSpace;
    private _baseLayerWrapper;
    private _baseLayerRTTProvider;
    private _xrNavigator;
    private _sessionMode;
    private _onEngineDisposedObserver;
    /**
     * The base reference space from which the session started. good if you want to reset your
     * reference space
     */
    baseReferenceSpace: XRReferenceSpace;
    /**
     * Current XR frame
     */
    currentFrame: Nullable<XRFrame>;
    /** WebXR timestamp updated every frame */
    currentTimestamp: number;
    /**
     * Used just in case of a failure to initialize an immersive session.
     * The viewer reference space is compensated using this height, creating a kind of "viewer-floor" reference space
     */
    defaultHeightCompensation: number;
    /**
     * Fires every time a new xrFrame arrives which can be used to update the camera
     */
    onXRFrameObservable: Observable<XRFrame>;
    /**
     * Fires when the reference space changed
     */
    onXRReferenceSpaceChanged: Observable<XRReferenceSpace>;
    /**
     * Fires when the xr session is ended either by the device or manually done
     */
    onXRSessionEnded: Observable<any>;
    /**
     * Fires when the xr session is initialized: right after requestSession was called and returned with a successful result
     */
    onXRSessionInit: Observable<XRSession>;
    /**
     * Underlying xr session
     */
    session: XRSession;
    /**
     * The viewer (head position) reference space. This can be used to get the XR world coordinates
     * or get the offset the player is currently at.
     */
    viewerReferenceSpace: XRReferenceSpace;
    /**
     * Are we currently in the XR loop?
     */
    inXRFrameLoop: boolean;
    /**
     * Are we in an XR session?
     */
    inXRSession: boolean;
    /**
     * Constructs a WebXRSessionManager, this must be initialized within a user action before usage
     * @param scene The scene which the session should be created for
     */
    constructor(
    /** The scene which the session should be created for */
    scene: Scene);
    /**
     * The current reference space used in this session. This reference space can constantly change!
     * It is mainly used to offset the camera's position.
     */
    get referenceSpace(): XRReferenceSpace;
    /**
     * Set a new reference space and triggers the observable
     */
    set referenceSpace(newReferenceSpace: XRReferenceSpace);
    /**
     * The mode for the managed XR session
     */
    get sessionMode(): XRSessionMode;
    /**
     * Disposes of the session manager
     * This should be called explicitly by the dev, if required.
     */
    dispose(): void;
    /**
     * Stops the xrSession and restores the render loop
     * @returns Promise which resolves after it exits XR
     */
    exitXRAsync(): Promise<void>;
    /**
     * Attempts to set the framebuffer-size-normalized viewport to be rendered this frame for this view.
     * In the event of a failure, the supplied viewport is not updated.
     * @param viewport the viewport to which the view will be rendered
     * @param view the view for which to set the viewport
     * @returns whether the operation was successful
     */
    trySetViewportForView(viewport: Viewport, view: XRView): boolean;
    /**
     * Gets the correct render target texture to be rendered this frame for this eye
     * @param eye the eye for which to get the render target
     * @returns the render target for the specified eye or null if not available
     */
    getRenderTargetTextureForEye(eye: XREye): Nullable<RenderTargetTexture>;
    /**
     * Gets the correct render target texture to be rendered this frame for this view
     * @param view the view for which to get the render target
     * @returns the render target for the specified view or null if not available
     */
    getRenderTargetTextureForView(view: XRView): Nullable<RenderTargetTexture>;
    /**
     * Creates a WebXRRenderTarget object for the XR session
     * @param options optional options to provide when creating a new render target
     * @returns a WebXR render target to which the session can render
     */
    getWebXRRenderTarget(options?: WebXRManagedOutputCanvasOptions): WebXRRenderTarget;
    /**
     * Initializes the manager
     * After initialization enterXR can be called to start an XR session
     * @returns Promise which resolves after it is initialized
     */
    initializeAsync(): Promise<void>;
    /**
     * Initializes an xr session
     * @param xrSessionMode mode to initialize
     * @param xrSessionInit defines optional and required values to pass to the session builder
     * @returns a promise which will resolve once the session has been initialized
     */
    initializeSessionAsync(xrSessionMode?: XRSessionMode, xrSessionInit?: XRSessionInit): Promise<XRSession>;
    /**
     * Checks if a session would be supported for the creation options specified
     * @param sessionMode session mode to check if supported eg. immersive-vr
     * @returns A Promise that resolves to true if supported and false if not
     */
    isSessionSupportedAsync(sessionMode: XRSessionMode): Promise<boolean>;
    /**
     * Resets the reference space to the one started the session
     */
    resetReferenceSpace(): void;
    /**
     * Starts rendering to the xr layer
     */
    runXRRenderLoop(): void;
    /**
     * Sets the reference space on the xr session
     * @param referenceSpaceType space to set
     * @returns a promise that will resolve once the reference space has been set
     */
    setReferenceSpaceTypeAsync(referenceSpaceType?: XRReferenceSpaceType): Promise<XRReferenceSpace>;
    /**
     * Updates the render state of the session.
     * Note that this is deprecated in favor of WebXRSessionManager.updateRenderState().
     * @param state state to set
     * @returns a promise that resolves once the render state has been updated
     * @deprecated
     */
    updateRenderStateAsync(state: XRRenderState): Promise<void>;
    /**
     * @internal
     */
    _setBaseLayerWrapper(baseLayerWrapper: Nullable<WebXRLayerWrapper>): void;
    /**
     * Updates the render state of the session
     * @param state state to set
     */
    updateRenderState(state: XRRenderStateInit): void;
    /**
     * Returns a promise that resolves with a boolean indicating if the provided session mode is supported by this browser
     * @param sessionMode defines the session to test
     * @returns a promise with boolean as final value
     */
    static IsSessionSupportedAsync(sessionMode: XRSessionMode): Promise<boolean>;
    /**
     * Returns true if Babylon.js is using the BabylonNative backend, otherwise false
     */
    get isNative(): boolean;
    /**
     * The current frame rate as reported by the device
     */
    get currentFrameRate(): number | undefined;
    /**
     * A list of supported frame rates (only available in-session!
     */
    get supportedFrameRates(): Float32Array | undefined;
    /**
     * Set the framerate of the session.
     * @param rate the new framerate. This value needs to be in the supportedFrameRates array
     * @returns a promise that resolves once the framerate has been set
     */
    updateTargetFrameRate(rate: number): Promise<void>;
    /**
     * Run a callback in the xr render loop
     * @param callback the callback to call when in XR Frame
     * @param ignoreIfNotInSession if no session is currently running, run it first thing on the next session
     */
    runInXRFrame(callback: () => void, ignoreIfNotInSession?: boolean): void;
    /**
     * Check if fixed foveation is supported on this device
     */
    get isFixedFoveationSupported(): boolean;
    /**
     * Get the fixed foveation currently set, as specified by the webxr specs
     * If this returns null, then fixed foveation is not supported
     */
    get fixedFoveation(): Nullable<number>;
    /**
     * Set the fixed foveation to the specified value, as specified by the webxr specs
     * This value will be normalized to be between 0 and 1, 1 being max foveation, 0 being no foveation
     */
    set fixedFoveation(value: Nullable<number>);
}

}
declare module "babylonjs/XR/webXRTypes" {
import { Nullable } from "babylonjs/types";
import { IDisposable } from "babylonjs/scene";
/**
 * States of the webXR experience
 */
export enum WebXRState {
    /**
     * Transitioning to being in XR mode
     */
    ENTERING_XR = 0,
    /**
     * Transitioning to non XR mode
     */
    EXITING_XR = 1,
    /**
     * In XR mode and presenting
     */
    IN_XR = 2,
    /**
     * Not entered XR mode
     */
    NOT_IN_XR = 3
}
/**
 * The state of the XR camera's tracking
 */
export enum WebXRTrackingState {
    /**
     * No transformation received, device is not being tracked
     */
    NOT_TRACKING = 0,
    /**
     * Tracking lost - using emulated position
     */
    TRACKING_LOST = 1,
    /**
     * Transformation tracking works normally
     */
    TRACKING = 2
}
/**
 * Abstraction of the XR render target
 */
export interface WebXRRenderTarget extends IDisposable {
    /**
     * xrpresent context of the canvas which can be used to display/mirror xr content
     */
    canvasContext: WebGLRenderingContext;
    /**
     * xr layer for the canvas
     */
    xrLayer: Nullable<XRWebGLLayer>;
    /**
     * Initializes a XRWebGLLayer to be used as the session's baseLayer.
     * @param xrSession xr session
     * @returns a promise that will resolve once the XR Layer has been created
     */
    initializeXRLayerAsync(xrSession: XRSession): Promise<XRWebGLLayer>;
}

}
declare module "babylonjs/XR/webXRWebGLLayer" {
import { RenderTargetTexture } from "babylonjs/Materials/Textures/renderTargetTexture";
import { Viewport } from "babylonjs/Maths/math.viewport";
import { Scene } from "babylonjs/scene";
import { Nullable } from "babylonjs/types";
import { WebXRLayerWrapper } from "babylonjs/XR/webXRLayerWrapper";
import { WebXRLayerRenderTargetTextureProvider } from "babylonjs/XR/webXRRenderTargetTextureProvider";
/**
 * Wraps xr webgl layers.
 * @internal
 */
export class WebXRWebGLLayerWrapper extends WebXRLayerWrapper {
    readonly layer: XRWebGLLayer;
    /**
     * @param layer is the layer to be wrapped.
     * @returns a new WebXRLayerWrapper wrapping the provided XRWebGLLayer.
     */
    constructor(layer: XRWebGLLayer);
}
/**
 * Provides render target textures and other important rendering information for a given XRWebGLLayer.
 * @internal
 */
export class WebXRWebGLLayerRenderTargetTextureProvider extends WebXRLayerRenderTargetTextureProvider {
    readonly layerWrapper: WebXRWebGLLayerWrapper;
    protected _framebufferDimensions: {
        framebufferWidth: number;
        framebufferHeight: number;
    };
    private _rtt;
    private _framebuffer;
    private _layer;
    constructor(scene: Scene, layerWrapper: WebXRWebGLLayerWrapper);
    trySetViewportForView(viewport: Viewport, view: XRView): boolean;
    getRenderTargetTextureForEye(eye: XREye): Nullable<RenderTargetTexture>;
    getRenderTargetTextureForView(view: XRView): Nullable<RenderTargetTexture>;
}

}

declare module "babylonjs" {
    export * from "babylonjs/Legacy/legacy";
}


declare module BABYLON {
    /**
     * Defines how the parser contract is defined.
     * These parsers are used to parse a list of specific assets (like particle systems, etc..)
     */
    export type BabylonFileParser = (parsedData: any, scene: Scene, container: AssetContainer, rootUrl: string) => void;
    /**
     * Defines how the individual parser contract is defined.
     * These parser can parse an individual asset
     */
    export type IndividualBabylonFileParser = (parsedData: any, scene: Scene, rootUrl: string) => any;
    /**
     * Base class of the scene acting as a container for the different elements composing a scene.
     * This class is dynamically extended by the different components of the scene increasing
     * flexibility and reducing coupling
     */
    export abstract class AbstractScene {
        /**
         * Stores the list of available parsers in the application.
         */
        private static _BabylonFileParsers;
        /**
         * Stores the list of available individual parsers in the application.
         */
        private static _IndividualBabylonFileParsers;
        /**
         * Adds a parser in the list of available ones
         * @param name Defines the name of the parser
         * @param parser Defines the parser to add
         */
        static AddParser(name: string, parser: BabylonFileParser): void;
        /**
         * Gets a general parser from the list of available ones
         * @param name Defines the name of the parser
         * @returns the requested parser or null
         */
        static GetParser(name: string): Nullable<BabylonFileParser>;
        /**
         * Adds n individual parser in the list of available ones
         * @param name Defines the name of the parser
         * @param parser Defines the parser to add
         */
        static AddIndividualParser(name: string, parser: IndividualBabylonFileParser): void;
        /**
         * Gets an individual parser from the list of available ones
         * @param name Defines the name of the parser
         * @returns the requested parser or null
         */
        static GetIndividualParser(name: string): Nullable<IndividualBabylonFileParser>;
        /**
         * Parser json data and populate both a scene and its associated container object
         * @param jsonData Defines the data to parse
         * @param scene Defines the scene to parse the data for
         * @param container Defines the container attached to the parsing sequence
         * @param rootUrl Defines the root url of the data
         */
        static Parse(jsonData: any, scene: Scene, container: AssetContainer, rootUrl: string): void;
        /**
         * Gets the list of root nodes (ie. nodes with no parent)
         */
        rootNodes: Node[];
        /** All of the cameras added to this scene
         * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras
         */
        cameras: Camera[];
        /**
         * All of the lights added to this scene
         * @see https://doc.babylonjs.com/features/featuresDeepDive/lights/lights_introduction
         */
        lights: Light[];
        /**
         * All of the (abstract) meshes added to this scene
         */
        meshes: AbstractMesh[];
        /**
         * The list of skeletons added to the scene
         * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/bonesSkeletons
         */
        skeletons: Skeleton[];
        /**
         * All of the particle systems added to this scene
         * @see https://doc.babylonjs.com/features/featuresDeepDive/particles/particle_system/particle_system_intro
         */
        particleSystems: IParticleSystem[];
        /**
         * Gets a list of Animations associated with the scene
         */
        animations: Animation[];
        /**
         * All of the animation groups added to this scene
         * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/groupAnimations
         */
        animationGroups: AnimationGroup[];
        /**
         * All of the multi-materials added to this scene
         * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/multiMaterials
         */
        multiMaterials: MultiMaterial[];
        /**
         * All of the materials added to this scene
         * In the context of a Scene, it is not supposed to be modified manually.
         * Any addition or removal should be done using the addMaterial and removeMaterial Scene methods.
         * Note also that the order of the Material within the array is not significant and might change.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/materials_introduction
         */
        materials: Material[];
        /**
         * The list of morph target managers added to the scene
         * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/dynamicMeshMorph
         */
        morphTargetManagers: MorphTargetManager[];
        /**
         * The list of geometries used in the scene.
         */
        geometries: Geometry[];
        /**
         * All of the transform nodes added to this scene
         * In the context of a Scene, it is not supposed to be modified manually.
         * Any addition or removal should be done using the addTransformNode and removeTransformNode Scene methods.
         * Note also that the order of the TransformNode within the array is not significant and might change.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/transforms/parent_pivot/transform_node
         */
        transformNodes: TransformNode[];
        /**
         * ActionManagers available on the scene.
         * @deprecated
         */
        actionManagers: AbstractActionManager[];
        /**
         * Textures to keep.
         */
        textures: BaseTexture[];
        /** @internal */
        protected _environmentTexture: Nullable<BaseTexture>;
        /**
         * Texture used in all pbr material as the reflection texture.
         * As in the majority of the scene they are the same (exception for multi room and so on),
         * this is easier to reference from here than from all the materials.
         */
        get environmentTexture(): Nullable<BaseTexture>;
        set environmentTexture(value: Nullable<BaseTexture>);
        /**
         * The list of postprocesses added to the scene
         */
        postProcesses: PostProcess[];
        /**
         * @returns all meshes, lights, cameras, transformNodes and bones
         */
        getNodes(): Array<Node>;
    }


    /**
     * Abstract class used to decouple action Manager from scene and meshes.
     * Do not instantiate.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions
     */
    export abstract class AbstractActionManager implements IDisposable {
        /** Gets the list of active triggers */
        static Triggers: {
            [key: string]: number;
        };
        /** Gets the cursor to use when hovering items */
        hoverCursor: string;
        /** Gets the list of actions */
        actions: IAction[];
        /**
         * Gets or sets a boolean indicating that the manager is recursive meaning that it can trigger action from children
         */
        isRecursive: boolean;
        /**
         * Releases all associated resources
         */
        abstract dispose(): void;
        /**
         * Does this action manager has pointer triggers
         */
        abstract get hasPointerTriggers(): boolean;
        /**
         * Does this action manager has pick triggers
         */
        abstract get hasPickTriggers(): boolean;
        /**
         * Process a specific trigger
         * @param trigger defines the trigger to process
         * @param evt defines the event details to be processed
         */
        abstract processTrigger(trigger: number, evt?: IActionEvent): void;
        /**
         * Does this action manager handles actions of any of the given triggers
         * @param triggers defines the triggers to be tested
         * @returns a boolean indicating whether one (or more) of the triggers is handled
         */
        abstract hasSpecificTriggers(triggers: number[]): boolean;
        /**
         * Does this action manager handles actions of any of the given triggers. This function takes two arguments for
         * speed.
         * @param triggerA defines the trigger to be tested
         * @param triggerB defines the trigger to be tested
         * @returns a boolean indicating whether one (or more) of the triggers is handled
         */
        abstract hasSpecificTriggers2(triggerA: number, triggerB: number): boolean;
        /**
         * Does this action manager handles actions of a given trigger
         * @param trigger defines the trigger to be tested
         * @param parameterPredicate defines an optional predicate to filter triggers by parameter
         * @returns whether the trigger is handled
         */
        abstract hasSpecificTrigger(trigger: number, parameterPredicate?: (parameter: any) => boolean): boolean;
        /**
         * Serialize this manager to a JSON object
         * @param name defines the property name to store this manager
         * @returns a JSON representation of this manager
         */
        abstract serialize(name: string): any;
        /**
         * Registers an action to this action manager
         * @param action defines the action to be registered
         * @returns the action amended (prepared) after registration
         */
        abstract registerAction(action: IAction): Nullable<IAction>;
        /**
         * Unregisters an action to this action manager
         * @param action defines the action to be unregistered
         * @returns a boolean indicating whether the action has been unregistered
         */
        abstract unregisterAction(action: IAction): Boolean;
        /**
         * Does exist one action manager with at least one trigger
         **/
        static get HasTriggers(): boolean;
        /**
         * Does exist one action manager with at least one pick trigger
         **/
        static get HasPickTriggers(): boolean;
        /**
         * Does exist one action manager that handles actions of a given trigger
         * @param trigger defines the trigger to be tested
         * @returns a boolean indicating whether the trigger is handled by at least one action manager
         **/
        static HasSpecificTrigger(trigger: number): boolean;
    }


    /**
     * Interface used to define Action
     */
    export interface IAction {
        /**
         * Trigger for the action
         */
        trigger: number;
        /** Options of the trigger */
        triggerOptions: any;
        /**
         * Gets the trigger parameters
         * @returns the trigger parameters
         */
        getTriggerParameter(): any;
        /**
         * Internal only - executes current action event
         * @internal
         */
        _executeCurrent(evt?: ActionEvent): void;
        /**
         * Serialize placeholder for child classes
         * @param parent of child
         * @returns the serialized object
         */
        serialize(parent: any): any;
        /**
         * Internal only
         * @internal
         */
        _prepare(): void;
        /**
         * Internal only - manager for action
         * @internal
         */
        _actionManager: Nullable<AbstractActionManager>;
        /**
         * Adds action to chain of actions, may be a DoNothingAction
         * @param action defines the next action to execute
         * @returns The action passed in
         * @see https://www.babylonjs-playground.com/#1T30HR#0
         */
        then(action: IAction): IAction;
    }
    /**
     * The action to be carried out following a trigger
     * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#available-actions
     */
    export class Action implements IAction {
        /** the trigger, with or without parameters, for the action */
        triggerOptions: any;
        /**
         * Trigger for the action
         */
        trigger: number;
        /**
         * Internal only - manager for action
         * @internal
         */
        _actionManager: ActionManager;
        private _nextActiveAction;
        private _child;
        private _condition?;
        private _triggerParameter;
        /**
         * An event triggered prior to action being executed.
         */
        onBeforeExecuteObservable: Observable<Action>;
        /**
         * Creates a new Action
         * @param triggerOptions the trigger, with or without parameters, for the action
         * @param condition an optional determinant of action
         */
        constructor(
        /** the trigger, with or without parameters, for the action */
        triggerOptions: any, condition?: Condition);
        /**
         * Internal only
         * @internal
         */
        _prepare(): void;
        /**
         * Gets the trigger parameter
         * @returns the trigger parameter
         */
        getTriggerParameter(): any;
        /**
         * Sets the trigger parameter
         * @param value defines the new trigger parameter
         */
        setTriggerParameter(value: any): void;
        /**
         * Internal only - Returns if the current condition allows to run the action
         * @internal
         */
        _evaluateConditionForCurrentFrame(): boolean;
        /**
         * Internal only - executes current action event
         * @internal
         */
        _executeCurrent(evt?: ActionEvent): void;
        /**
         * Execute placeholder for child classes
         * @param evt optional action event
         */
        execute(evt?: ActionEvent): void;
        /**
         * Skips to next active action
         */
        skipToNextActiveAction(): void;
        /**
         * Adds action to chain of actions, may be a DoNothingAction
         * @param action defines the next action to execute
         * @returns The action passed in
         * @see https://www.babylonjs-playground.com/#1T30HR#0
         */
        then(action: Action): Action;
        /**
         * Internal only
         * @internal
         */
        _getProperty(propertyPath: string): string;
        /**
         * @internal
         */
        _getEffectiveTarget(target: any, propertyPath: string): any;
        /**
         * Serialize placeholder for child classes
         * @param parent of child
         * @returns the serialized object
         */
        serialize(parent: any): any;
        /**
         * Internal only called by serialize
         * @internal
         */
        protected _serialize(serializedAction: any, parent?: any): any;
        /**
         * Internal only
         * @internal
         */
        static _SerializeValueAsString: (value: any) => string;
        /**
         * Internal only
         * @internal
         */
        static _GetTargetProperty: (target: Scene | Node | Material) => {
            name: string;
            targetType: string;
            value: string;
        };
    }


    /**
     * Interface used to define ActionEvent
     */
    export interface IActionEvent {
        /** The mesh or sprite that triggered the action */
        source: any;
        /** The X mouse cursor position at the time of the event */
        pointerX: number;
        /** The Y mouse cursor position at the time of the event */
        pointerY: number;
        /** The mesh that is currently pointed at (can be null) */
        meshUnderPointer: Nullable<AbstractMesh>;
        /** the original (browser) event that triggered the ActionEvent */
        sourceEvent?: any;
        /** additional data for the event */
        additionalData?: any;
    }
    /**
     * ActionEvent is the event being sent when an action is triggered.
     */
    export class ActionEvent implements IActionEvent {
        /** The mesh or sprite that triggered the action */
        source: any;
        /** The X mouse cursor position at the time of the event */
        pointerX: number;
        /** The Y mouse cursor position at the time of the event */
        pointerY: number;
        /** The mesh that is currently pointed at (can be null) */
        meshUnderPointer: Nullable<AbstractMesh>;
        /** the original (browser) event that triggered the ActionEvent */
        sourceEvent?: any;
        /** additional data for the event */
        additionalData?: any;
        /**
         * Creates a new ActionEvent
         * @param source The mesh or sprite that triggered the action
         * @param pointerX The X mouse cursor position at the time of the event
         * @param pointerY The Y mouse cursor position at the time of the event
         * @param meshUnderPointer The mesh that is currently pointed at (can be null)
         * @param sourceEvent the original (browser) event that triggered the ActionEvent
         * @param additionalData additional data for the event
         */
        constructor(
        /** The mesh or sprite that triggered the action */
        source: any, 
        /** The X mouse cursor position at the time of the event */
        pointerX: number, 
        /** The Y mouse cursor position at the time of the event */
        pointerY: number, 
        /** The mesh that is currently pointed at (can be null) */
        meshUnderPointer: Nullable<AbstractMesh>, 
        /** the original (browser) event that triggered the ActionEvent */
        sourceEvent?: any, 
        /** additional data for the event */
        additionalData?: any);
        /**
         * Helper function to auto-create an ActionEvent from a source mesh.
         * @param source The source mesh that triggered the event
         * @param evt The original (browser) event
         * @param additionalData additional data for the event
         * @returns the new ActionEvent
         */
        static CreateNew(source: AbstractMesh, evt?: any, additionalData?: any): ActionEvent;
        /**
         * Helper function to auto-create an ActionEvent from a source sprite
         * @param source The source sprite that triggered the event
         * @param scene Scene associated with the sprite
         * @param evt The original (browser) event
         * @param additionalData additional data for the event
         * @returns the new ActionEvent
         */
        static CreateNewFromSprite(source: Sprite, scene: Scene, evt?: any, additionalData?: any): ActionEvent;
        /**
         * Helper function to auto-create an ActionEvent from a scene. If triggered by a mesh use ActionEvent.CreateNew
         * @param scene the scene where the event occurred
         * @param evt The original (browser) event
         * @returns the new ActionEvent
         */
        static CreateNewFromScene(scene: Scene, evt: any): ActionEvent;
        /**
         * Helper function to auto-create an ActionEvent from a primitive
         * @param prim defines the target primitive
         * @param pointerPos defines the pointer position
         * @param evt The original (browser) event
         * @param additionalData additional data for the event
         * @returns the new ActionEvent
         */
        static CreateNewFromPrimitive(prim: any, pointerPos: Vector2, evt?: Event, additionalData?: any): ActionEvent;
    }


    /**
     * Action Manager manages all events to be triggered on a given mesh or the global scene.
     * A single scene can have many Action Managers to handle predefined actions on specific meshes.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions
     */
    export class ActionManager extends AbstractActionManager {
        /**
         * Nothing
         * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
         */
        static readonly NothingTrigger = 0;
        /**
         * On pick
         * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
         */
        static readonly OnPickTrigger = 1;
        /**
         * On left pick
         * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
         */
        static readonly OnLeftPickTrigger = 2;
        /**
         * On right pick
         * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
         */
        static readonly OnRightPickTrigger = 3;
        /**
         * On center pick
         * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
         */
        static readonly OnCenterPickTrigger = 4;
        /**
         * On pick down
         * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
         */
        static readonly OnPickDownTrigger = 5;
        /**
         * On double pick
         * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
         */
        static readonly OnDoublePickTrigger = 6;
        /**
         * On pick up
         * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
         */
        static readonly OnPickUpTrigger = 7;
        /**
         * On pick out.
         * This trigger will only be raised if you also declared a OnPickDown
         * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
         */
        static readonly OnPickOutTrigger = 16;
        /**
         * On long press
         * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
         */
        static readonly OnLongPressTrigger = 8;
        /**
         * On pointer over
         * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
         */
        static readonly OnPointerOverTrigger = 9;
        /**
         * On pointer out
         * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
         */
        static readonly OnPointerOutTrigger = 10;
        /**
         * On every frame
         * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
         */
        static readonly OnEveryFrameTrigger = 11;
        /**
         * On intersection enter
         * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
         */
        static readonly OnIntersectionEnterTrigger = 12;
        /**
         * On intersection exit
         * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
         */
        static readonly OnIntersectionExitTrigger = 13;
        /**
         * On key down
         * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
         */
        static readonly OnKeyDownTrigger = 14;
        /**
         * On key up
         * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
         */
        static readonly OnKeyUpTrigger = 15;
        private _scene;
        /**
         * Creates a new action manager
         * @param scene defines the hosting scene
         */
        constructor(scene?: Nullable<Scene>);
        /**
         * Releases all associated resources
         */
        dispose(): void;
        /**
         * Gets hosting scene
         * @returns the hosting scene
         */
        getScene(): Scene;
        /**
         * Does this action manager handles actions of any of the given triggers
         * @param triggers defines the triggers to be tested
         * @returns a boolean indicating whether one (or more) of the triggers is handled
         */
        hasSpecificTriggers(triggers: number[]): boolean;
        /**
         * Does this action manager handles actions of any of the given triggers. This function takes two arguments for
         * speed.
         * @param triggerA defines the trigger to be tested
         * @param triggerB defines the trigger to be tested
         * @returns a boolean indicating whether one (or more) of the triggers is handled
         */
        hasSpecificTriggers2(triggerA: number, triggerB: number): boolean;
        /**
         * Does this action manager handles actions of a given trigger
         * @param trigger defines the trigger to be tested
         * @param parameterPredicate defines an optional predicate to filter triggers by parameter
         * @returns whether the trigger is handled
         */
        hasSpecificTrigger(trigger: number, parameterPredicate?: (parameter: any) => boolean): boolean;
        /**
         * Does this action manager has pointer triggers
         */
        get hasPointerTriggers(): boolean;
        /**
         * Does this action manager has pick triggers
         */
        get hasPickTriggers(): boolean;
        /**
         * Registers an action to this action manager
         * @param action defines the action to be registered
         * @returns the action amended (prepared) after registration
         */
        registerAction(action: IAction): Nullable<IAction>;
        /**
         * Unregisters an action to this action manager
         * @param action defines the action to be unregistered
         * @returns a boolean indicating whether the action has been unregistered
         */
        unregisterAction(action: IAction): Boolean;
        /**
         * Process a specific trigger
         * @param trigger defines the trigger to process
         * @param evt defines the event details to be processed
         */
        processTrigger(trigger: number, evt?: IActionEvent): void;
        /**
         * @internal
         */
        _getEffectiveTarget(target: any, propertyPath: string): any;
        /**
         * @internal
         */
        _getProperty(propertyPath: string): string;
        /**
         * Serialize this manager to a JSON object
         * @param name defines the property name to store this manager
         * @returns a JSON representation of this manager
         */
        serialize(name: string): any;
        /**
         * Creates a new ActionManager from a JSON data
         * @param parsedActions defines the JSON data to read from
         * @param object defines the hosting mesh
         * @param scene defines the hosting scene
         */
        static Parse(parsedActions: any, object: Nullable<AbstractMesh>, scene: Scene): void;
        /**
         * Get a trigger name by index
         * @param trigger defines the trigger index
         * @returns a trigger name
         */
        static GetTriggerName(trigger: number): string;
    }


    /**
     * A Condition applied to an Action
     */
    export class Condition {
        /**
         * Internal only - manager for action
         * @internal
         */
        _actionManager: ActionManager;
        /**
         * @internal
         */
        _evaluationId: number;
        /**
         * @internal
         */
        _currentResult: boolean;
        /**
         * Creates a new Condition
         * @param actionManager the manager of the action the condition is applied to
         */
        constructor(actionManager: ActionManager);
        /**
         * Check if the current condition is valid
         * @returns a boolean
         */
        isValid(): boolean;
        /**
         * @internal
         */
        _getProperty(propertyPath: string): string;
        /**
         * @internal
         */
        _getEffectiveTarget(target: any, propertyPath: string): any;
        /**
         * Serialize placeholder for child classes
         * @returns the serialized object
         */
        serialize(): any;
        /**
         * @internal
         */
        protected _serialize(serializedCondition: any): any;
    }
    /**
     * Defines specific conditional operators as extensions of Condition
     */
    export class ValueCondition extends Condition {
        /** path to specify the property of the target the conditional operator uses  */
        propertyPath: string;
        /** the value compared by the conditional operator against the current value of the property */
        value: any;
        /** the conditional operator, default ValueCondition.IsEqual */
        operator: number;
        private static _IsEqual;
        private static _IsDifferent;
        private static _IsGreater;
        private static _IsLesser;
        /**
         * returns the number for IsEqual
         */
        static get IsEqual(): number;
        /**
         * Returns the number for IsDifferent
         */
        static get IsDifferent(): number;
        /**
         * Returns the number for IsGreater
         */
        static get IsGreater(): number;
        /**
         * Returns the number for IsLesser
         */
        static get IsLesser(): number;
        /**
         * Internal only The action manager for the condition
         * @internal
         */
        _actionManager: ActionManager;
        private _target;
        private _effectiveTarget;
        private _property;
        /**
         * Creates a new ValueCondition
         * @param actionManager manager for the action the condition applies to
         * @param target for the action
         * @param propertyPath path to specify the property of the target the conditional operator uses
         * @param value the value compared by the conditional operator against the current value of the property
         * @param operator the conditional operator, default ValueCondition.IsEqual
         */
        constructor(actionManager: ActionManager, target: any, 
        /** path to specify the property of the target the conditional operator uses  */
        propertyPath: string, 
        /** the value compared by the conditional operator against the current value of the property */
        value: any, 
        /** the conditional operator, default ValueCondition.IsEqual */
        operator?: number);
        /**
         * Compares the given value with the property value for the specified conditional operator
         * @returns the result of the comparison
         */
        isValid(): boolean;
        /**
         * Serialize the ValueCondition into a JSON compatible object
         * @returns serialization object
         */
        serialize(): any;
        /**
         * Gets the name of the conditional operator for the ValueCondition
         * @param operator the conditional operator
         * @returns the name
         */
        static GetOperatorName(operator: number): string;
    }
    /**
     * Defines a predicate condition as an extension of Condition
     */
    export class PredicateCondition extends Condition {
        /** defines the predicate function used to validate the condition */
        predicate: () => boolean;
        /**
         * Internal only - manager for action
         * @internal
         */
        _actionManager: ActionManager;
        /**
         * Creates a new PredicateCondition
         * @param actionManager manager for the action the condition applies to
         * @param predicate defines the predicate function used to validate the condition
         */
        constructor(actionManager: ActionManager, 
        /** defines the predicate function used to validate the condition */
        predicate: () => boolean);
        /**
         * @returns the validity of the predicate condition
         */
        isValid(): boolean;
    }
    /**
     * Defines a state condition as an extension of Condition
     */
    export class StateCondition extends Condition {
        /** Value to compare with target state  */
        value: string;
        /**
         * Internal only - manager for action
         * @internal
         */
        _actionManager: ActionManager;
        private _target;
        /**
         * Creates a new StateCondition
         * @param actionManager manager for the action the condition applies to
         * @param target of the condition
         * @param value to compare with target state
         */
        constructor(actionManager: ActionManager, target: any, 
        /** Value to compare with target state  */
        value: string);
        /**
         * Gets a boolean indicating if the current condition is met
         * @returns the validity of the state
         */
        isValid(): boolean;
        /**
         * Serialize the StateCondition into a JSON compatible object
         * @returns serialization object
         */
        serialize(): any;
    }


    /**
     * This defines an action responsible to toggle a boolean once triggered.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions
     */
    export class SwitchBooleanAction extends Action {
        /**
         * The path to the boolean property in the target object
         */
        propertyPath: string;
        private _target;
        private _effectiveTarget;
        private _property;
        /**
         * Instantiate the action
         * @param triggerOptions defines the trigger options
         * @param target defines the object containing the boolean
         * @param propertyPath defines the path to the boolean property in the target object
         * @param condition defines the trigger related conditions
         */
        constructor(triggerOptions: any, target: any, propertyPath: string, condition?: Condition);
        /** @internal */
        _prepare(): void;
        /**
         * Execute the action toggle the boolean value.
         */
        execute(): void;
        /**
         * Serializes the actions and its related information.
         * @param parent defines the object to serialize in
         * @returns the serialized object
         */
        serialize(parent: any): any;
    }
    /**
     * This defines an action responsible to set a the state field of the target
     *  to a desired value once triggered.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions
     */
    export class SetStateAction extends Action {
        /**
         * The value to store in the state field.
         */
        value: string;
        private _target;
        /**
         * Instantiate the action
         * @param triggerOptions defines the trigger options
         * @param target defines the object containing the state property
         * @param value defines the value to store in the state field
         * @param condition defines the trigger related conditions
         */
        constructor(triggerOptions: any, target: any, value: string, condition?: Condition);
        /**
         * Execute the action and store the value on the target state property.
         */
        execute(): void;
        /**
         * Serializes the actions and its related information.
         * @param parent defines the object to serialize in
         * @returns the serialized object
         */
        serialize(parent: any): any;
    }
    /**
     * This defines an action responsible to set a property of the target
     *  to a desired value once triggered.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions
     */
    export class SetValueAction extends Action {
        /**
         * The path of the property to set in the target.
         */
        propertyPath: string;
        /**
         * The value to set in the property
         */
        value: any;
        private _target;
        private _effectiveTarget;
        private _property;
        /**
         * Instantiate the action
         * @param triggerOptions defines the trigger options
         * @param target defines the object containing the property
         * @param propertyPath defines the path of the property to set in the target
         * @param value defines the value to set in the property
         * @param condition defines the trigger related conditions
         */
        constructor(triggerOptions: any, target: any, propertyPath: string, value: any, condition?: Condition);
        /** @internal */
        _prepare(): void;
        /**
         * Execute the action and set the targeted property to the desired value.
         */
        execute(): void;
        /**
         * Serializes the actions and its related information.
         * @param parent defines the object to serialize in
         * @returns the serialized object
         */
        serialize(parent: any): any;
    }
    /**
     * This defines an action responsible to increment the target value
     *  to a desired value once triggered.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions
     */
    export class IncrementValueAction extends Action {
        /**
         * The path of the property to increment in the target.
         */
        propertyPath: string;
        /**
         * The value we should increment the property by.
         */
        value: any;
        private _target;
        private _effectiveTarget;
        private _property;
        /**
         * Instantiate the action
         * @param triggerOptions defines the trigger options
         * @param target defines the object containing the property
         * @param propertyPath defines the path of the property to increment in the target
         * @param value defines the value value we should increment the property by
         * @param condition defines the trigger related conditions
         */
        constructor(triggerOptions: any, target: any, propertyPath: string, value: any, condition?: Condition);
        /** @internal */
        _prepare(): void;
        /**
         * Execute the action and increment the target of the value amount.
         */
        execute(): void;
        /**
         * Serializes the actions and its related information.
         * @param parent defines the object to serialize in
         * @returns the serialized object
         */
        serialize(parent: any): any;
    }
    /**
     * This defines an action responsible to start an animation once triggered.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions
     */
    export class PlayAnimationAction extends Action {
        /**
         * Where the animation should start (animation frame)
         */
        from: number;
        /**
         * Where the animation should stop (animation frame)
         */
        to: number;
        /**
         * Define if the animation should loop or stop after the first play.
         */
        loop?: boolean;
        private _target;
        /**
         * Instantiate the action
         * @param triggerOptions defines the trigger options
         * @param target defines the target animation or animation name
         * @param from defines from where the animation should start (animation frame)
         * @param to defines where the animation should stop (animation frame)
         * @param loop defines if the animation should loop or stop after the first play
         * @param condition defines the trigger related conditions
         */
        constructor(triggerOptions: any, target: any, from: number, to: number, loop?: boolean, condition?: Condition);
        /** @internal */
        _prepare(): void;
        /**
         * Execute the action and play the animation.
         */
        execute(): void;
        /**
         * Serializes the actions and its related information.
         * @param parent defines the object to serialize in
         * @returns the serialized object
         */
        serialize(parent: any): any;
    }
    /**
     * This defines an action responsible to stop an animation once triggered.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions
     */
    export class StopAnimationAction extends Action {
        private _target;
        /**
         * Instantiate the action
         * @param triggerOptions defines the trigger options
         * @param target defines the target animation or animation name
         * @param condition defines the trigger related conditions
         */
        constructor(triggerOptions: any, target: any, condition?: Condition);
        /** @internal */
        _prepare(): void;
        /**
         * Execute the action and stop the animation.
         */
        execute(): void;
        /**
         * Serializes the actions and its related information.
         * @param parent defines the object to serialize in
         * @returns the serialized object
         */
        serialize(parent: any): any;
    }
    /**
     * This defines an action responsible that does nothing once triggered.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions
     */
    export class DoNothingAction extends Action {
        /**
         * Instantiate the action
         * @param triggerOptions defines the trigger options
         * @param condition defines the trigger related conditions
         */
        constructor(triggerOptions?: any, condition?: Condition);
        /**
         * Execute the action and do nothing.
         */
        execute(): void;
        /**
         * Serializes the actions and its related information.
         * @param parent defines the object to serialize in
         * @returns the serialized object
         */
        serialize(parent: any): any;
    }
    /**
     * This defines an action responsible to trigger several actions once triggered.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions
     */
    export class CombineAction extends Action {
        /**
         * The list of aggregated animations to run.
         */
        children: Action[];
        /**
         * defines if the children actions conditions should be check before execution
         */
        enableChildrenConditions: boolean;
        /**
         * Instantiate the action
         * @param triggerOptions defines the trigger options
         * @param children defines the list of aggregated animations to run
         * @param condition defines the trigger related conditions
         * @param enableChildrenConditions defines if the children actions conditions should be check before execution
         */
        constructor(triggerOptions: any, children: Action[], condition?: Condition, enableChildrenConditions?: boolean);
        /** @internal */
        _prepare(): void;
        /**
         * Execute the action and executes all the aggregated actions.
         * @param evt
         */
        execute(evt: ActionEvent): void;
        /**
         * Serializes the actions and its related information.
         * @param parent defines the object to serialize in
         * @returns the serialized object
         */
        serialize(parent: any): any;
    }
    /**
     * This defines an action responsible to run code (external event) once triggered.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions
     */
    export class ExecuteCodeAction extends Action {
        /**
         * The callback function to run.
         */
        func: (evt: ActionEvent) => void;
        /**
         * Instantiate the action
         * @param triggerOptions defines the trigger options
         * @param func defines the callback function to run
         * @param condition defines the trigger related conditions
         */
        constructor(triggerOptions: any, func: (evt: ActionEvent) => void, condition?: Condition);
        /**
         * Execute the action and run the attached code.
         * @param evt
         */
        execute(evt: ActionEvent): void;
    }
    /**
     * This defines an action responsible to set the parent property of the target once triggered.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions
     */
    export class SetParentAction extends Action {
        private _parent;
        private _target;
        /**
         * Instantiate the action
         * @param triggerOptions defines the trigger options
         * @param target defines the target containing the parent property
         * @param parent defines from where the animation should start (animation frame)
         * @param condition defines the trigger related conditions
         */
        constructor(triggerOptions: any, target: any, parent: any, condition?: Condition);
        /** @internal */
        _prepare(): void;
        /**
         * Execute the action and set the parent property.
         */
        execute(): void;
        /**
         * Serializes the actions and its related information.
         * @param parent defines the object to serialize in
         * @returns the serialized object
         */
        serialize(parent: any): any;
    }


    /**
     * This defines an action helpful to play a defined sound on a triggered action.
     */
    export class PlaySoundAction extends Action {
        private _sound;
        /**
         * Instantiate the action
         * @param triggerOptions defines the trigger options
         * @param sound defines the sound to play
         * @param condition defines the trigger related conditions
         */
        constructor(triggerOptions: any, sound: Sound, condition?: Condition);
        /** @internal */
        _prepare(): void;
        /**
         * Execute the action and play the sound.
         */
        execute(): void;
        /**
         * Serializes the actions and its related information.
         * @param parent defines the object to serialize in
         * @returns the serialized object
         */
        serialize(parent: any): any;
    }
    /**
     * This defines an action helpful to stop a defined sound on a triggered action.
     */
    export class StopSoundAction extends Action {
        private _sound;
        /**
         * Instantiate the action
         * @param triggerOptions defines the trigger options
         * @param sound defines the sound to stop
         * @param condition defines the trigger related conditions
         */
        constructor(triggerOptions: any, sound: Sound, condition?: Condition);
        /** @internal */
        _prepare(): void;
        /**
         * Execute the action and stop the sound.
         */
        execute(): void;
        /**
         * Serializes the actions and its related information.
         * @param parent defines the object to serialize in
         * @returns the serialized object
         */
        serialize(parent: any): any;
    }




    /**
     * This defines an action responsible to change the value of a property
     * by interpolating between its current value and the newly set one once triggered.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions
     */
    export class InterpolateValueAction extends Action {
        /**
         * Defines the path of the property where the value should be interpolated
         */
        propertyPath: string;
        /**
         * Defines the target value at the end of the interpolation.
         */
        value: any;
        /**
         * Defines the time it will take for the property to interpolate to the value.
         */
        duration: number;
        /**
         * Defines if the other scene animations should be stopped when the action has been triggered
         */
        stopOtherAnimations?: boolean;
        /**
         * Defines a callback raised once the interpolation animation has been done.
         */
        onInterpolationDone?: () => void;
        /**
         * Observable triggered once the interpolation animation has been done.
         */
        onInterpolationDoneObservable: Observable<InterpolateValueAction>;
        private _target;
        private _effectiveTarget;
        private _property;
        /**
         * Instantiate the action
         * @param triggerOptions defines the trigger options
         * @param target defines the object containing the value to interpolate
         * @param propertyPath defines the path to the property in the target object
         * @param value defines the target value at the end of the interpolation
         * @param duration defines the time it will take for the property to interpolate to the value.
         * @param condition defines the trigger related conditions
         * @param stopOtherAnimations defines if the other scene animations should be stopped when the action has been triggered
         * @param onInterpolationDone defines a callback raised once the interpolation animation has been done
         */
        constructor(triggerOptions: any, target: any, propertyPath: string, value: any, duration?: number, condition?: Condition, stopOtherAnimations?: boolean, onInterpolationDone?: () => void);
        /** @internal */
        _prepare(): void;
        /**
         * Execute the action starts the value interpolation.
         */
        execute(): void;
        /**
         * Serializes the actions and its related information.
         * @param parent defines the object to serialize in
         * @returns the serialized object
         */
        serialize(parent: any): any;
    }


    /**
     * Class used to store an actual running animation
     */
    export class Animatable {
        /** defines the target object */
        target: any;
        /** defines the starting frame number (default is 0) */
        fromFrame: number;
        /** defines the ending frame number (default is 100) */
        toFrame: number;
        /** defines if the animation must loop (default is false)  */
        loopAnimation: boolean;
        /** defines a callback to call when animation ends if it is not looping */
        onAnimationEnd?: Nullable<() => void> | undefined;
        /** defines a callback to call when animation loops */
        onAnimationLoop?: Nullable<() => void> | undefined;
        /** defines whether the animation should be evaluated additively */
        isAdditive: boolean;
        private _localDelayOffset;
        private _pausedDelay;
        private _manualJumpDelay;
        private _runtimeAnimations;
        private _paused;
        private _scene;
        private _speedRatio;
        private _weight;
        private _syncRoot;
        private _frameToSyncFromJump;
        private _goToFrame;
        /**
         * Gets or sets a boolean indicating if the animatable must be disposed and removed at the end of the animation.
         * This will only apply for non looping animation (default is true)
         */
        disposeOnEnd: boolean;
        /**
         * Gets a boolean indicating if the animation has started
         */
        animationStarted: boolean;
        /**
         * Observer raised when the animation ends
         */
        onAnimationEndObservable: Observable<Animatable>;
        /**
         * Observer raised when the animation loops
         */
        onAnimationLoopObservable: Observable<Animatable>;
        /**
         * Gets the root Animatable used to synchronize and normalize animations
         */
        get syncRoot(): Nullable<Animatable>;
        /**
         * Gets the current frame of the first RuntimeAnimation
         * Used to synchronize Animatables
         */
        get masterFrame(): number;
        /**
         * Gets or sets the animatable weight (-1.0 by default meaning not weighted)
         */
        get weight(): number;
        set weight(value: number);
        /**
         * Gets or sets the speed ratio to apply to the animatable (1.0 by default)
         */
        get speedRatio(): number;
        set speedRatio(value: number);
        /**
         * Creates a new Animatable
         * @param scene defines the hosting scene
         * @param target defines the target object
         * @param fromFrame defines the starting frame number (default is 0)
         * @param toFrame defines the ending frame number (default is 100)
         * @param loopAnimation defines if the animation must loop (default is false)
         * @param speedRatio defines the factor to apply to animation speed (default is 1)
         * @param onAnimationEnd defines a callback to call when animation ends if it is not looping
         * @param animations defines a group of animation to add to the new Animatable
         * @param onAnimationLoop defines a callback to call when animation loops
         * @param isAdditive defines whether the animation should be evaluated additively
         */
        constructor(scene: Scene, 
        /** defines the target object */
        target: any, 
        /** defines the starting frame number (default is 0) */
        fromFrame?: number, 
        /** defines the ending frame number (default is 100) */
        toFrame?: number, 
        /** defines if the animation must loop (default is false)  */
        loopAnimation?: boolean, speedRatio?: number, 
        /** defines a callback to call when animation ends if it is not looping */
        onAnimationEnd?: Nullable<() => void> | undefined, animations?: Animation[], 
        /** defines a callback to call when animation loops */
        onAnimationLoop?: Nullable<() => void> | undefined, 
        /** defines whether the animation should be evaluated additively */
        isAdditive?: boolean);
        /**
         * Synchronize and normalize current Animatable with a source Animatable
         * This is useful when using animation weights and when animations are not of the same length
         * @param root defines the root Animatable to synchronize with (null to stop synchronizing)
         * @returns the current Animatable
         */
        syncWith(root: Nullable<Animatable>): Animatable;
        /**
         * Gets the list of runtime animations
         * @returns an array of RuntimeAnimation
         */
        getAnimations(): RuntimeAnimation[];
        /**
         * Adds more animations to the current animatable
         * @param target defines the target of the animations
         * @param animations defines the new animations to add
         */
        appendAnimations(target: any, animations: Animation[]): void;
        /**
         * Gets the source animation for a specific property
         * @param property defines the property to look for
         * @returns null or the source animation for the given property
         */
        getAnimationByTargetProperty(property: string): Nullable<Animation>;
        /**
         * Gets the runtime animation for a specific property
         * @param property defines the property to look for
         * @returns null or the runtime animation for the given property
         */
        getRuntimeAnimationByTargetProperty(property: string): Nullable<RuntimeAnimation>;
        /**
         * Resets the animatable to its original state
         */
        reset(): void;
        /**
         * Allows the animatable to blend with current running animations
         * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/advanced_animations#animation-blending
         * @param blendingSpeed defines the blending speed to use
         */
        enableBlending(blendingSpeed: number): void;
        /**
         * Disable animation blending
         * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/advanced_animations#animation-blending
         */
        disableBlending(): void;
        /**
         * Jump directly to a given frame
         * @param frame defines the frame to jump to
         */
        goToFrame(frame: number): void;
        /**
         * Pause the animation
         */
        pause(): void;
        /**
         * Restart the animation
         */
        restart(): void;
        private _raiseOnAnimationEnd;
        /**
         * Stop and delete the current animation
         * @param animationName defines a string used to only stop some of the runtime animations instead of all
         * @param targetMask - a function that determines if the animation should be stopped based on its target (all animations will be stopped if both this and animationName are empty)
         */
        stop(animationName?: string, targetMask?: (target: any) => boolean): void;
        /**
         * Wait asynchronously for the animation to end
         * @returns a promise which will be fulfilled when the animation ends
         */
        waitAsync(): Promise<Animatable>;
        /**
         * @internal
         */
        _animate(delay: number): boolean;
    }
        interface Scene {
            /** @internal */
            _registerTargetForLateAnimationBinding(runtimeAnimation: RuntimeAnimation, originalValue: any): void;
            /** @internal */
            _processLateAnimationBindingsForMatrices(holder: {
                totalWeight: number;
                totalAdditiveWeight: number;
                animations: RuntimeAnimation[];
                additiveAnimations: RuntimeAnimation[];
                originalValue: Matrix;
            }): any;
            /** @internal */
            _processLateAnimationBindingsForQuaternions(holder: {
                totalWeight: number;
                totalAdditiveWeight: number;
                animations: RuntimeAnimation[];
                additiveAnimations: RuntimeAnimation[];
                originalValue: Quaternion;
            }, refQuaternion: Quaternion): Quaternion;
            /** @internal */
            _processLateAnimationBindings(): void;
            /**
             * Will start the animation sequence of a given target
             * @param target defines the target
             * @param from defines from which frame should animation start
             * @param to defines until which frame should animation run.
             * @param weight defines the weight to apply to the animation (1.0 by default)
             * @param loop defines if the animation loops
             * @param speedRatio defines the speed in which to run the animation (1.0 by default)
             * @param onAnimationEnd defines the function to be executed when the animation ends
             * @param animatable defines an animatable object. If not provided a new one will be created from the given params
             * @param targetMask defines if the target should be animated if animations are present (this is called recursively on descendant animatables regardless of return value)
             * @param onAnimationLoop defines the callback to call when an animation loops
             * @param isAdditive defines whether the animation should be evaluated additively (false by default)
             * @returns the animatable object created for this animation
             */
            beginWeightedAnimation(target: any, from: number, to: number, weight: number, loop?: boolean, speedRatio?: number, onAnimationEnd?: () => void, animatable?: Animatable, targetMask?: (target: any) => boolean, onAnimationLoop?: () => void, isAdditive?: boolean): Animatable;
            /**
             * Will start the animation sequence of a given target
             * @param target defines the target
             * @param from defines from which frame should animation start
             * @param to defines until which frame should animation run.
             * @param loop defines if the animation loops
             * @param speedRatio defines the speed in which to run the animation (1.0 by default)
             * @param onAnimationEnd defines the function to be executed when the animation ends
             * @param animatable defines an animatable object. If not provided a new one will be created from the given params
             * @param stopCurrent defines if the current animations must be stopped first (true by default)
             * @param targetMask defines if the target should be animate if animations are present (this is called recursively on descendant animatables regardless of return value)
             * @param onAnimationLoop defines the callback to call when an animation loops
             * @param isAdditive defines whether the animation should be evaluated additively (false by default)
             * @returns the animatable object created for this animation
             */
            beginAnimation(target: any, from: number, to: number, loop?: boolean, speedRatio?: number, onAnimationEnd?: () => void, animatable?: Animatable, stopCurrent?: boolean, targetMask?: (target: any) => boolean, onAnimationLoop?: () => void, isAdditive?: boolean): Animatable;
            /**
             * Will start the animation sequence of a given target and its hierarchy
             * @param target defines the target
             * @param directDescendantsOnly if true only direct descendants will be used, if false direct and also indirect (children of children, an so on in a recursive manner) descendants will be used.
             * @param from defines from which frame should animation start
             * @param to defines until which frame should animation run.
             * @param loop defines if the animation loops
             * @param speedRatio defines the speed in which to run the animation (1.0 by default)
             * @param onAnimationEnd defines the function to be executed when the animation ends
             * @param animatable defines an animatable object. If not provided a new one will be created from the given params
             * @param stopCurrent defines if the current animations must be stopped first (true by default)
             * @param targetMask defines if the target should be animated if animations are present (this is called recursively on descendant animatables regardless of return value)
             * @param onAnimationLoop defines the callback to call when an animation loops
             * @param isAdditive defines whether the animation should be evaluated additively (false by default)
             * @returns the list of created animatables
             */
            beginHierarchyAnimation(target: any, directDescendantsOnly: boolean, from: number, to: number, loop?: boolean, speedRatio?: number, onAnimationEnd?: () => void, animatable?: Animatable, stopCurrent?: boolean, targetMask?: (target: any) => boolean, onAnimationLoop?: () => void, isAdditive?: boolean): Animatable[];
            /**
             * Begin a new animation on a given node
             * @param target defines the target where the animation will take place
             * @param animations defines the list of animations to start
             * @param from defines the initial value
             * @param to defines the final value
             * @param loop defines if you want animation to loop (off by default)
             * @param speedRatio defines the speed ratio to apply to all animations
             * @param onAnimationEnd defines the callback to call when an animation ends (will be called once per node)
             * @param onAnimationLoop defines the callback to call when an animation loops
             * @param isAdditive defines whether the animation should be evaluated additively (false by default)
             * @returns the list of created animatables
             */
            beginDirectAnimation(target: any, animations: Animation[], from: number, to: number, loop?: boolean, speedRatio?: number, onAnimationEnd?: () => void, onAnimationLoop?: () => void, isAdditive?: boolean): Animatable;
            /**
             * Begin a new animation on a given node and its hierarchy
             * @param target defines the root node where the animation will take place
             * @param directDescendantsOnly if true only direct descendants will be used, if false direct and also indirect (children of children, an so on in a recursive manner) descendants will be used.
             * @param animations defines the list of animations to start
             * @param from defines the initial value
             * @param to defines the final value
             * @param loop defines if you want animation to loop (off by default)
             * @param speedRatio defines the speed ratio to apply to all animations
             * @param onAnimationEnd defines the callback to call when an animation ends (will be called once per node)
             * @param onAnimationLoop defines the callback to call when an animation loops
             * @param isAdditive defines whether the animation should be evaluated additively (false by default)
             * @returns the list of animatables created for all nodes
             */
            beginDirectHierarchyAnimation(target: Node, directDescendantsOnly: boolean, animations: Animation[], from: number, to: number, loop?: boolean, speedRatio?: number, onAnimationEnd?: () => void, onAnimationLoop?: () => void, isAdditive?: boolean): Animatable[];
            /**
             * Gets the animatable associated with a specific target
             * @param target defines the target of the animatable
             * @returns the required animatable if found
             */
            getAnimatableByTarget(target: any): Nullable<Animatable>;
            /**
             * Gets all animatables associated with a given target
             * @param target defines the target to look animatables for
             * @returns an array of Animatables
             */
            getAllAnimatablesByTarget(target: any): Array<Animatable>;
            /**
             * Stops and removes all animations that have been applied to the scene
             */
            stopAllAnimations(): void;
            /**
             * Gets the current delta time used by animation engine
             */
            deltaTime: number;
        }
        interface Bone {
            /**
             * Copy an animation range from another bone
             * @param source defines the source bone
             * @param rangeName defines the range name to copy
             * @param frameOffset defines the frame offset
             * @param rescaleAsRequired defines if rescaling must be applied if required
             * @param skelDimensionsRatio defines the scaling ratio
             * @returns true if operation was successful
             */
            copyAnimationRange(source: Bone, rangeName: string, frameOffset: number, rescaleAsRequired: boolean, skelDimensionsRatio: Nullable<Vector3>): boolean;
        }


    /**
     * Interface containing an array of animations
     */
    export interface IAnimatable {
        /**
         * Array of animations
         */
        animations: Nullable<Array<Animation>>;
    }


    /**
     * @internal
     */
    export class _IAnimationState {
        key: number;
        repeatCount: number;
        workValue?: any;
        loopMode?: number;
        offsetValue?: any;
        highLimitValue?: any;
    }
    /**
     * Class used to store any kind of animation
     */
    export class Animation {
        /**Name of the animation */
        name: string;
        /**Property to animate */
        targetProperty: string;
        /**The frames per second of the animation */
        framePerSecond: number;
        /**The data type of the animation */
        dataType: number;
        /**The loop mode of the animation */
        loopMode?: number | undefined;
        /**Specifies if blending should be enabled */
        enableBlending?: boolean | undefined;
        private static _UniqueIdGenerator;
        /**
         * Use matrix interpolation instead of using direct key value when animating matrices
         */
        static AllowMatricesInterpolation: boolean;
        /**
         * When matrix interpolation is enabled, this boolean forces the system to use Matrix.DecomposeLerp instead of Matrix.Lerp. Interpolation is more precise but slower
         */
        static AllowMatrixDecomposeForInterpolation: boolean;
        /**
         * Gets or sets the unique id of the animation (the uniqueness is solely among other animations)
         */
        uniqueId: number;
        /** Define the Url to load snippets */
        static SnippetUrl: string;
        /** Snippet ID if the animation was created from the snippet server */
        snippetId: string;
        /**
         * Stores the key frames of the animation
         */
        private _keys;
        /**
         * Stores the easing function of the animation
         */
        private _easingFunction;
        /**
         * @internal Internal use only
         */
        _runtimeAnimations: RuntimeAnimation[];
        /**
         * The set of event that will be linked to this animation
         */
        private _events;
        /**
         * Stores an array of target property paths
         */
        targetPropertyPath: string[];
        /**
         * Stores the blending speed of the animation
         */
        blendingSpeed: number;
        /**
         * Stores the animation ranges for the animation
         */
        private _ranges;
        /**
         * @internal Internal use
         */
        static _PrepareAnimation(name: string, targetProperty: string, framePerSecond: number, totalFrame: number, from: any, to: any, loopMode?: number, easingFunction?: EasingFunction): Nullable<Animation>;
        /**
         * Sets up an animation
         * @param property The property to animate
         * @param animationType The animation type to apply
         * @param framePerSecond The frames per second of the animation
         * @param easingFunction The easing function used in the animation
         * @returns The created animation
         */
        static CreateAnimation(property: string, animationType: number, framePerSecond: number, easingFunction: EasingFunction): Animation;
        /**
         * Create and start an animation on a node
         * @param name defines the name of the global animation that will be run on all nodes
         * @param target defines the target where the animation will take place
         * @param targetProperty defines property to animate
         * @param framePerSecond defines the number of frame per second yo use
         * @param totalFrame defines the number of frames in total
         * @param from defines the initial value
         * @param to defines the final value
         * @param loopMode defines which loop mode you want to use (off by default)
         * @param easingFunction defines the easing function to use (linear by default)
         * @param onAnimationEnd defines the callback to call when animation end
         * @param scene defines the hosting scene
         * @returns the animatable created for this animation
         */
        static CreateAndStartAnimation(name: string, target: any, targetProperty: string, framePerSecond: number, totalFrame: number, from: any, to: any, loopMode?: number, easingFunction?: EasingFunction, onAnimationEnd?: () => void, scene?: Scene): Nullable<Animatable>;
        /**
         * Create and start an animation on a node and its descendants
         * @param name defines the name of the global animation that will be run on all nodes
         * @param node defines the root node where the animation will take place
         * @param directDescendantsOnly if true only direct descendants will be used, if false direct and also indirect (children of children, an so on in a recursive manner) descendants will be used
         * @param targetProperty defines property to animate
         * @param framePerSecond defines the number of frame per second to use
         * @param totalFrame defines the number of frames in total
         * @param from defines the initial value
         * @param to defines the final value
         * @param loopMode defines which loop mode you want to use (off by default)
         * @param easingFunction defines the easing function to use (linear by default)
         * @param onAnimationEnd defines the callback to call when an animation ends (will be called once per node)
         * @returns the list of animatables created for all nodes
         * @example https://www.babylonjs-playground.com/#MH0VLI
         */
        static CreateAndStartHierarchyAnimation(name: string, node: Node, directDescendantsOnly: boolean, targetProperty: string, framePerSecond: number, totalFrame: number, from: any, to: any, loopMode?: number, easingFunction?: EasingFunction, onAnimationEnd?: () => void): Nullable<Animatable[]>;
        /**
         * Creates a new animation, merges it with the existing animations and starts it
         * @param name Name of the animation
         * @param node Node which contains the scene that begins the animations
         * @param targetProperty Specifies which property to animate
         * @param framePerSecond The frames per second of the animation
         * @param totalFrame The total number of frames
         * @param from The frame at the beginning of the animation
         * @param to The frame at the end of the animation
         * @param loopMode Specifies the loop mode of the animation
         * @param easingFunction (Optional) The easing function of the animation, which allow custom mathematical formulas for animations
         * @param onAnimationEnd Callback to run once the animation is complete
         * @returns Nullable animation
         */
        static CreateMergeAndStartAnimation(name: string, node: Node, targetProperty: string, framePerSecond: number, totalFrame: number, from: any, to: any, loopMode?: number, easingFunction?: EasingFunction, onAnimationEnd?: () => void): Nullable<Animatable>;
        /**
         * Convert the keyframes for all animations belonging to the group to be relative to a given reference frame.
         * @param sourceAnimation defines the Animation containing keyframes to convert
         * @param referenceFrame defines the frame that keyframes in the range will be relative to
         * @param range defines the name of the AnimationRange belonging to the Animation to convert
         * @param cloneOriginal defines whether or not to clone the animation and convert the clone or convert the original animation (default is false)
         * @param clonedName defines the name of the resulting cloned Animation if cloneOriginal is true
         * @returns a new Animation if cloneOriginal is true or the original Animation if cloneOriginal is false
         */
        static MakeAnimationAdditive(sourceAnimation: Animation, referenceFrame?: number, range?: string, cloneOriginal?: boolean, clonedName?: string): Animation;
        /**
         * Transition property of an host to the target Value
         * @param property The property to transition
         * @param targetValue The target Value of the property
         * @param host The object where the property to animate belongs
         * @param scene Scene used to run the animation
         * @param frameRate Framerate (in frame/s) to use
         * @param transition The transition type we want to use
         * @param duration The duration of the animation, in milliseconds
         * @param onAnimationEnd Callback trigger at the end of the animation
         * @returns Nullable animation
         */
        static TransitionTo(property: string, targetValue: any, host: any, scene: Scene, frameRate: number, transition: Animation, duration: number, onAnimationEnd?: Nullable<() => void>): Nullable<Animatable>;
        /**
         * Return the array of runtime animations currently using this animation
         */
        get runtimeAnimations(): RuntimeAnimation[];
        /**
         * Specifies if any of the runtime animations are currently running
         */
        get hasRunningRuntimeAnimations(): boolean;
        /**
         * Initializes the animation
         * @param name Name of the animation
         * @param targetProperty Property to animate
         * @param framePerSecond The frames per second of the animation
         * @param dataType The data type of the animation
         * @param loopMode The loop mode of the animation
         * @param enableBlending Specifies if blending should be enabled
         */
        constructor(
        /**Name of the animation */
        name: string, 
        /**Property to animate */
        targetProperty: string, 
        /**The frames per second of the animation */
        framePerSecond: number, 
        /**The data type of the animation */
        dataType: number, 
        /**The loop mode of the animation */
        loopMode?: number | undefined, 
        /**Specifies if blending should be enabled */
        enableBlending?: boolean | undefined);
        /**
         * Converts the animation to a string
         * @param fullDetails support for multiple levels of logging within scene loading
         * @returns String form of the animation
         */
        toString(fullDetails?: boolean): string;
        /**
         * Add an event to this animation
         * @param event Event to add
         */
        addEvent(event: AnimationEvent): void;
        /**
         * Remove all events found at the given frame
         * @param frame The frame to remove events from
         */
        removeEvents(frame: number): void;
        /**
         * Retrieves all the events from the animation
         * @returns Events from the animation
         */
        getEvents(): AnimationEvent[];
        /**
         * Creates an animation range
         * @param name Name of the animation range
         * @param from Starting frame of the animation range
         * @param to Ending frame of the animation
         */
        createRange(name: string, from: number, to: number): void;
        /**
         * Deletes an animation range by name
         * @param name Name of the animation range to delete
         * @param deleteFrames Specifies if the key frames for the range should also be deleted (true) or not (false)
         */
        deleteRange(name: string, deleteFrames?: boolean): void;
        /**
         * Gets the animation range by name, or null if not defined
         * @param name Name of the animation range
         * @returns Nullable animation range
         */
        getRange(name: string): Nullable<AnimationRange>;
        /**
         * Gets the key frames from the animation
         * @returns The key frames of the animation
         */
        getKeys(): Array<IAnimationKey>;
        /**
         * Gets the highest frame rate of the animation
         * @returns Highest frame rate of the animation
         */
        getHighestFrame(): number;
        /**
         * Gets the easing function of the animation
         * @returns Easing function of the animation
         */
        getEasingFunction(): Nullable<IEasingFunction>;
        /**
         * Sets the easing function of the animation
         * @param easingFunction A custom mathematical formula for animation
         */
        setEasingFunction(easingFunction: Nullable<IEasingFunction>): void;
        /**
         * Interpolates a scalar linearly
         * @param startValue Start value of the animation curve
         * @param endValue End value of the animation curve
         * @param gradient Scalar amount to interpolate
         * @returns Interpolated scalar value
         */
        floatInterpolateFunction(startValue: number, endValue: number, gradient: number): number;
        /**
         * Interpolates a scalar cubically
         * @param startValue Start value of the animation curve
         * @param outTangent End tangent of the animation
         * @param endValue End value of the animation curve
         * @param inTangent Start tangent of the animation curve
         * @param gradient Scalar amount to interpolate
         * @returns Interpolated scalar value
         */
        floatInterpolateFunctionWithTangents(startValue: number, outTangent: number, endValue: number, inTangent: number, gradient: number): number;
        /**
         * Interpolates a quaternion using a spherical linear interpolation
         * @param startValue Start value of the animation curve
         * @param endValue End value of the animation curve
         * @param gradient Scalar amount to interpolate
         * @returns Interpolated quaternion value
         */
        quaternionInterpolateFunction(startValue: Quaternion, endValue: Quaternion, gradient: number): Quaternion;
        /**
         * Interpolates a quaternion cubically
         * @param startValue Start value of the animation curve
         * @param outTangent End tangent of the animation curve
         * @param endValue End value of the animation curve
         * @param inTangent Start tangent of the animation curve
         * @param gradient Scalar amount to interpolate
         * @returns Interpolated quaternion value
         */
        quaternionInterpolateFunctionWithTangents(startValue: Quaternion, outTangent: Quaternion, endValue: Quaternion, inTangent: Quaternion, gradient: number): Quaternion;
        /**
         * Interpolates a Vector3 linearly
         * @param startValue Start value of the animation curve
         * @param endValue End value of the animation curve
         * @param gradient Scalar amount to interpolate (value between 0 and 1)
         * @returns Interpolated scalar value
         */
        vector3InterpolateFunction(startValue: Vector3, endValue: Vector3, gradient: number): Vector3;
        /**
         * Interpolates a Vector3 cubically
         * @param startValue Start value of the animation curve
         * @param outTangent End tangent of the animation
         * @param endValue End value of the animation curve
         * @param inTangent Start tangent of the animation curve
         * @param gradient Scalar amount to interpolate (value between 0 and 1)
         * @returns InterpolatedVector3 value
         */
        vector3InterpolateFunctionWithTangents(startValue: Vector3, outTangent: Vector3, endValue: Vector3, inTangent: Vector3, gradient: number): Vector3;
        /**
         * Interpolates a Vector2 linearly
         * @param startValue Start value of the animation curve
         * @param endValue End value of the animation curve
         * @param gradient Scalar amount to interpolate (value between 0 and 1)
         * @returns Interpolated Vector2 value
         */
        vector2InterpolateFunction(startValue: Vector2, endValue: Vector2, gradient: number): Vector2;
        /**
         * Interpolates a Vector2 cubically
         * @param startValue Start value of the animation curve
         * @param outTangent End tangent of the animation
         * @param endValue End value of the animation curve
         * @param inTangent Start tangent of the animation curve
         * @param gradient Scalar amount to interpolate (value between 0 and 1)
         * @returns Interpolated Vector2 value
         */
        vector2InterpolateFunctionWithTangents(startValue: Vector2, outTangent: Vector2, endValue: Vector2, inTangent: Vector2, gradient: number): Vector2;
        /**
         * Interpolates a size linearly
         * @param startValue Start value of the animation curve
         * @param endValue End value of the animation curve
         * @param gradient Scalar amount to interpolate
         * @returns Interpolated Size value
         */
        sizeInterpolateFunction(startValue: Size, endValue: Size, gradient: number): Size;
        /**
         * Interpolates a Color3 linearly
         * @param startValue Start value of the animation curve
         * @param endValue End value of the animation curve
         * @param gradient Scalar amount to interpolate
         * @returns Interpolated Color3 value
         */
        color3InterpolateFunction(startValue: Color3, endValue: Color3, gradient: number): Color3;
        /**
         * Interpolates a Color3 cubically
         * @param startValue Start value of the animation curve
         * @param outTangent End tangent of the animation
         * @param endValue End value of the animation curve
         * @param inTangent Start tangent of the animation curve
         * @param gradient Scalar amount to interpolate
         * @returns interpolated value
         */
        color3InterpolateFunctionWithTangents(startValue: Color3, outTangent: Color3, endValue: Color3, inTangent: Color3, gradient: number): Color3;
        /**
         * Interpolates a Color4 linearly
         * @param startValue Start value of the animation curve
         * @param endValue End value of the animation curve
         * @param gradient Scalar amount to interpolate
         * @returns Interpolated Color3 value
         */
        color4InterpolateFunction(startValue: Color4, endValue: Color4, gradient: number): Color4;
        /**
         * Interpolates a Color4 cubically
         * @param startValue Start value of the animation curve
         * @param outTangent End tangent of the animation
         * @param endValue End value of the animation curve
         * @param inTangent Start tangent of the animation curve
         * @param gradient Scalar amount to interpolate
         * @returns interpolated value
         */
        color4InterpolateFunctionWithTangents(startValue: Color4, outTangent: Color4, endValue: Color4, inTangent: Color4, gradient: number): Color4;
        /**
         * @internal Internal use only
         */
        _getKeyValue(value: any): any;
        /**
         * Evaluate the animation value at a given frame
         * @param currentFrame defines the frame where we want to evaluate the animation
         * @returns the animation value
         */
        evaluate(currentFrame: number): any;
        /**
         * @internal Internal use only
         */
        _interpolate(currentFrame: number, state: _IAnimationState): any;
        /**
         * Defines the function to use to interpolate matrices
         * @param startValue defines the start matrix
         * @param endValue defines the end matrix
         * @param gradient defines the gradient between both matrices
         * @param result defines an optional target matrix where to store the interpolation
         * @returns the interpolated matrix
         */
        matrixInterpolateFunction(startValue: Matrix, endValue: Matrix, gradient: number, result?: Matrix): Matrix;
        /**
         * Makes a copy of the animation
         * @returns Cloned animation
         */
        clone(): Animation;
        /**
         * Sets the key frames of the animation
         * @param values The animation key frames to set
         */
        setKeys(values: Array<IAnimationKey>): void;
        /**
         * Serializes the animation to an object
         * @returns Serialized object
         */
        serialize(): any;
        /**
         * Float animation type
         */
        static readonly ANIMATIONTYPE_FLOAT = 0;
        /**
         * Vector3 animation type
         */
        static readonly ANIMATIONTYPE_VECTOR3 = 1;
        /**
         * Quaternion animation type
         */
        static readonly ANIMATIONTYPE_QUATERNION = 2;
        /**
         * Matrix animation type
         */
        static readonly ANIMATIONTYPE_MATRIX = 3;
        /**
         * Color3 animation type
         */
        static readonly ANIMATIONTYPE_COLOR3 = 4;
        /**
         * Color3 animation type
         */
        static readonly ANIMATIONTYPE_COLOR4 = 7;
        /**
         * Vector2 animation type
         */
        static readonly ANIMATIONTYPE_VECTOR2 = 5;
        /**
         * Size animation type
         */
        static readonly ANIMATIONTYPE_SIZE = 6;
        /**
         * Relative Loop Mode
         */
        static readonly ANIMATIONLOOPMODE_RELATIVE = 0;
        /**
         * Cycle Loop Mode
         */
        static readonly ANIMATIONLOOPMODE_CYCLE = 1;
        /**
         * Constant Loop Mode
         */
        static readonly ANIMATIONLOOPMODE_CONSTANT = 2;
        /**
         * @internal
         */
        static _UniversalLerp(left: any, right: any, amount: number): any;
        /**
         * Parses an animation object and creates an animation
         * @param parsedAnimation Parsed animation object
         * @returns Animation object
         */
        static Parse(parsedAnimation: any): Animation;
        /**
         * Appends the serialized animations from the source animations
         * @param source Source containing the animations
         * @param destination Target to store the animations
         */
        static AppendSerializedAnimations(source: IAnimatable, destination: any): void;
        /**
         * Creates a new animation or an array of animations from a snippet saved in a remote file
         * @param name defines the name of the animation to create (can be null or empty to use the one from the json data)
         * @param url defines the url to load from
         * @returns a promise that will resolve to the new animation or an array of animations
         */
        static ParseFromFileAsync(name: Nullable<string>, url: string): Promise<Animation | Array<Animation>>;
        /**
         * Creates an animation or an array of animations from a snippet saved by the Inspector
         * @param snippetId defines the snippet to load
         * @returns a promise that will resolve to the new animation or a new array of animations
         */
        static ParseFromSnippetAsync(snippetId: string): Promise<Animation | Array<Animation>>;
        /**
         * Creates an animation or an array of animations from a snippet saved by the Inspector
         * @deprecated Please use ParseFromSnippetAsync instead
         * @param snippetId defines the snippet to load
         * @returns a promise that will resolve to the new animation or a new array of animations
         */
        static CreateFromSnippetAsync: typeof Animation.ParseFromSnippetAsync;
    }


    /**
     * Composed of a frame, and an action function
     */
    export class AnimationEvent {
        /** The frame for which the event is triggered **/
        frame: number;
        /** The event to perform when triggered **/
        action: (currentFrame: number) => void;
        /** Specifies if the event should be triggered only once**/
        onlyOnce?: boolean | undefined;
        /**
         * Specifies if the animation event is done
         */
        isDone: boolean;
        /**
         * Initializes the animation event
         * @param frame The frame for which the event is triggered
         * @param action The event to perform when triggered
         * @param onlyOnce Specifies if the event should be triggered only once
         */
        constructor(
        /** The frame for which the event is triggered **/
        frame: number, 
        /** The event to perform when triggered **/
        action: (currentFrame: number) => void, 
        /** Specifies if the event should be triggered only once**/
        onlyOnce?: boolean | undefined);
        /** @internal */
        _clone(): AnimationEvent;
    }


    /**
     * This class defines the direct association between an animation and a target
     */
    export class TargetedAnimation {
        /**
         * Animation to perform
         */
        animation: Animation;
        /**
         * Target to animate
         */
        target: any;
        /**
         * Returns the string "TargetedAnimation"
         * @returns "TargetedAnimation"
         */
        getClassName(): string;
        /**
         * Serialize the object
         * @returns the JSON object representing the current entity
         */
        serialize(): any;
    }
    /**
     * Use this class to create coordinated animations on multiple targets
     */
    export class AnimationGroup implements IDisposable {
        /** The name of the animation group */
        name: string;
        private _scene;
        private _targetedAnimations;
        private _animatables;
        private _from;
        private _to;
        private _isStarted;
        private _isPaused;
        private _speedRatio;
        private _loopAnimation;
        private _isAdditive;
        /** @internal */
        _parentContainer: Nullable<AbstractScene>;
        /**
         * Gets or sets the unique id of the node
         */
        uniqueId: number;
        /**
         * This observable will notify when one animation have ended
         */
        onAnimationEndObservable: Observable<TargetedAnimation>;
        /**
         * Observer raised when one animation loops
         */
        onAnimationLoopObservable: Observable<TargetedAnimation>;
        /**
         * Observer raised when all animations have looped
         */
        onAnimationGroupLoopObservable: Observable<AnimationGroup>;
        /**
         * This observable will notify when all animations have ended.
         */
        onAnimationGroupEndObservable: Observable<AnimationGroup>;
        /**
         * This observable will notify when all animations have paused.
         */
        onAnimationGroupPauseObservable: Observable<AnimationGroup>;
        /**
         * This observable will notify when all animations are playing.
         */
        onAnimationGroupPlayObservable: Observable<AnimationGroup>;
        /**
         * Gets or sets an object used to store user defined information for the node
         */
        metadata: any;
        /**
         * Gets the first frame
         */
        get from(): number;
        /**
         * Gets the last frame
         */
        get to(): number;
        /**
         * Define if the animations are started
         */
        get isStarted(): boolean;
        /**
         * Gets a value indicating that the current group is playing
         */
        get isPlaying(): boolean;
        /**
         * Gets or sets the speed ratio to use for all animations
         */
        get speedRatio(): number;
        /**
         * Gets or sets the speed ratio to use for all animations
         */
        set speedRatio(value: number);
        /**
         * Gets or sets if all animations should loop or not
         */
        get loopAnimation(): boolean;
        set loopAnimation(value: boolean);
        /**
         * Gets or sets if all animations should be evaluated additively
         */
        get isAdditive(): boolean;
        set isAdditive(value: boolean);
        /**
         * Gets the targeted animations for this animation group
         */
        get targetedAnimations(): Array<TargetedAnimation>;
        /**
         * returning the list of animatables controlled by this animation group.
         */
        get animatables(): Array<Animatable>;
        /**
         * Gets the list of target animations
         */
        get children(): TargetedAnimation[];
        /**
         * Instantiates a new Animation Group.
         * This helps managing several animations at once.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/groupAnimations
         * @param name Defines the name of the group
         * @param scene Defines the scene the group belongs to
         */
        constructor(
        /** The name of the animation group */
        name: string, scene?: Nullable<Scene>);
        /**
         * Add an animation (with its target) in the group
         * @param animation defines the animation we want to add
         * @param target defines the target of the animation
         * @returns the TargetedAnimation object
         */
        addTargetedAnimation(animation: Animation, target: any): TargetedAnimation;
        /**
         * This function will normalize every animation in the group to make sure they all go from beginFrame to endFrame
         * It can add constant keys at begin or end
         * @param beginFrame defines the new begin frame for all animations or the smallest begin frame of all animations if null (defaults to null)
         * @param endFrame defines the new end frame for all animations or the largest end frame of all animations if null (defaults to null)
         * @returns the animation group
         */
        normalize(beginFrame?: Nullable<number>, endFrame?: Nullable<number>): AnimationGroup;
        private _animationLoopCount;
        private _animationLoopFlags;
        private _processLoop;
        /**
         * Start all animations on given targets
         * @param loop defines if animations must loop
         * @param speedRatio defines the ratio to apply to animation speed (1 by default)
         * @param from defines the from key (optional)
         * @param to defines the to key (optional)
         * @param isAdditive defines the additive state for the resulting animatables (optional)
         * @returns the current animation group
         */
        start(loop?: boolean, speedRatio?: number, from?: number, to?: number, isAdditive?: boolean): AnimationGroup;
        /**
         * Pause all animations
         * @returns the animation group
         */
        pause(): AnimationGroup;
        /**
         * Play all animations to initial state
         * This function will start() the animations if they were not started or will restart() them if they were paused
         * @param loop defines if animations must loop
         * @returns the animation group
         */
        play(loop?: boolean): AnimationGroup;
        /**
         * Reset all animations to initial state
         * @returns the animation group
         */
        reset(): AnimationGroup;
        /**
         * Restart animations from key 0
         * @returns the animation group
         */
        restart(): AnimationGroup;
        /**
         * Stop all animations
         * @returns the animation group
         */
        stop(): AnimationGroup;
        /**
         * Set animation weight for all animatables
         * @param weight defines the weight to use
         * @returns the animationGroup
         * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/advanced_animations#animation-weights
         */
        setWeightForAllAnimatables(weight: number): AnimationGroup;
        /**
         * Synchronize and normalize all animatables with a source animatable
         * @param root defines the root animatable to synchronize with (null to stop synchronizing)
         * @returns the animationGroup
         * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/advanced_animations#animation-weights
         */
        syncAllAnimationsWith(root: Nullable<Animatable>): AnimationGroup;
        /**
         * Goes to a specific frame in this animation group
         * @param frame the frame number to go to
         * @returns the animationGroup
         */
        goToFrame(frame: number): AnimationGroup;
        /**
         * Dispose all associated resources
         */
        dispose(): void;
        private _checkAnimationGroupEnded;
        /**
         * Clone the current animation group and returns a copy
         * @param newName defines the name of the new group
         * @param targetConverter defines an optional function used to convert current animation targets to new ones
         * @param cloneAnimations defines if the animations should be cloned or referenced
         * @returns the new animation group
         */
        clone(newName: string, targetConverter?: (oldTarget: any) => any, cloneAnimations?: boolean): AnimationGroup;
        /**
         * Serializes the animationGroup to an object
         * @returns Serialized object
         */
        serialize(): any;
        /**
         * Returns a new AnimationGroup object parsed from the source provided.
         * @param parsedAnimationGroup defines the source
         * @param scene defines the scene that will receive the animationGroup
         * @returns a new AnimationGroup
         */
        static Parse(parsedAnimationGroup: any, scene: Scene): AnimationGroup;
        /**
         * Convert the keyframes for all animations belonging to the group to be relative to a given reference frame.
         * @param sourceAnimationGroup defines the AnimationGroup containing animations to convert
         * @param referenceFrame defines the frame that keyframes in the range will be relative to
         * @param range defines the name of the AnimationRange belonging to the animations in the group to convert
         * @param cloneOriginal defines whether or not to clone the group and convert the clone or convert the original group (default is false)
         * @param clonedName defines the name of the resulting cloned AnimationGroup if cloneOriginal is true
         * @returns a new AnimationGroup if cloneOriginal is true or the original AnimationGroup if cloneOriginal is false
         */
        static MakeAnimationAdditive(sourceAnimationGroup: AnimationGroup, referenceFrame?: number, range?: string, cloneOriginal?: boolean, clonedName?: string): AnimationGroup;
        /**
         * Returns the string "AnimationGroup"
         * @returns "AnimationGroup"
         */
        getClassName(): string;
        /**
         * Creates a detailed string about the object
         * @param fullDetails defines if the output string will support multiple levels of logging within scene loading
         * @returns a string representing the object
         */
        toString(fullDetails?: boolean): string;
    }


    /**
     * Defines an interface which represents an animation key frame
     */
    export interface IAnimationKey {
        /**
         * Frame of the key frame
         */
        frame: number;
        /**
         * Value at the specifies key frame
         */
        value: any;
        /**
         * The input tangent for the cubic hermite spline
         */
        inTangent?: any;
        /**
         * The output tangent for the cubic hermite spline
         */
        outTangent?: any;
        /**
         * The animation interpolation type
         */
        interpolation?: AnimationKeyInterpolation;
        /**
         * Property defined by UI tools to link (or not ) the tangents
         */
        lockedTangent?: boolean;
    }
    /**
     * Enum for the animation key frame interpolation type
     */
    export enum AnimationKeyInterpolation {
        /**
         * Use tangents to interpolate between start and end values.
         */
        NONE = 0,
        /**
         * Do not interpolate between keys and use the start key value only. Tangents are ignored
         */
        STEP = 1
    }


    /**
     * Class used to override all child animations of a given target
     */
    export class AnimationPropertiesOverride {
        /**
         * Gets or sets a value indicating if animation blending must be used
         */
        enableBlending: boolean;
        /**
         * Gets or sets the blending speed to use when enableBlending is true
         */
        blendingSpeed: number;
        /**
         * Gets or sets the default loop mode to use
         */
        loopMode: number;
    }


    /**
     * Represents the range of an animation
     */
    export class AnimationRange {
        /**The name of the animation range**/
        name: string;
        /**The starting frame of the animation */
        from: number;
        /**The ending frame of the animation*/
        to: number;
        /**
         * Initializes the range of an animation
         * @param name The name of the animation range
         * @param from The starting frame of the animation
         * @param to The ending frame of the animation
         */
        constructor(
        /**The name of the animation range**/
        name: string, 
        /**The starting frame of the animation */
        from: number, 
        /**The ending frame of the animation*/
        to: number);
        /**
         * Makes a copy of the animation range
         * @returns A copy of the animation range
         */
        clone(): AnimationRange;
    }


    /**
     * This represents the main contract an easing function should follow.
     * Easing functions are used throughout the animation system.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/advanced_animations#easing-functions
     */
    export interface IEasingFunction {
        /**
         * Given an input gradient between 0 and 1, this returns the corresponding value
         * of the easing function.
         * The link below provides some of the most common examples of easing functions.
         * @see https://easings.net/
         * @param gradient Defines the value between 0 and 1 we want the easing value for
         * @returns the corresponding value on the curve defined by the easing function
         */
        ease(gradient: number): number;
    }
    /**
     * Base class used for every default easing function.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/advanced_animations#easing-functions
     */
    export class EasingFunction implements IEasingFunction {
        /**
         * Interpolation follows the mathematical formula associated with the easing function.
         */
        static readonly EASINGMODE_EASEIN = 0;
        /**
         * Interpolation follows 100% interpolation minus the output of the formula associated with the easing function.
         */
        static readonly EASINGMODE_EASEOUT = 1;
        /**
         * Interpolation uses EaseIn for the first half of the animation and EaseOut for the second half.
         */
        static readonly EASINGMODE_EASEINOUT = 2;
        private _easingMode;
        /**
         * Sets the easing mode of the current function.
         * @param easingMode Defines the willing mode (EASINGMODE_EASEIN, EASINGMODE_EASEOUT or EASINGMODE_EASEINOUT)
         */
        setEasingMode(easingMode: number): void;
        /**
         * Gets the current easing mode.
         * @returns the easing mode
         */
        getEasingMode(): number;
        /**
         * @internal
         */
        easeInCore(gradient: number): number;
        /**
         * Given an input gradient between 0 and 1, this returns the corresponding value
         * of the easing function.
         * @param gradient Defines the value between 0 and 1 we want the easing value for
         * @returns the corresponding value on the curve defined by the easing function
         */
        ease(gradient: number): number;
    }
    /**
     * Easing function with a circle shape (see link below).
     * @see https://easings.net/#easeInCirc
     * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/advanced_animations#easing-functions
     */
    export class CircleEase extends EasingFunction implements IEasingFunction {
        /**
         * @internal
         */
        easeInCore(gradient: number): number;
    }
    /**
     * Easing function with a ease back shape (see link below).
     * @see https://easings.net/#easeInBack
     * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/advanced_animations#easing-functions
     */
    export class BackEase extends EasingFunction implements IEasingFunction {
        /** Defines the amplitude of the function */
        amplitude: number;
        /**
         * Instantiates a back ease easing
         * @see https://easings.net/#easeInBack
         * @param amplitude Defines the amplitude of the function
         */
        constructor(
        /** Defines the amplitude of the function */
        amplitude?: number);
        /**
         * @internal
         */
        easeInCore(gradient: number): number;
    }
    /**
     * Easing function with a bouncing shape (see link below).
     * @see https://easings.net/#easeInBounce
     * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/advanced_animations#easing-functions
     */
    export class BounceEase extends EasingFunction implements IEasingFunction {
        /** Defines the number of bounces */
        bounces: number;
        /** Defines the amplitude of the bounce */
        bounciness: number;
        /**
         * Instantiates a bounce easing
         * @see https://easings.net/#easeInBounce
         * @param bounces Defines the number of bounces
         * @param bounciness Defines the amplitude of the bounce
         */
        constructor(
        /** Defines the number of bounces */
        bounces?: number, 
        /** Defines the amplitude of the bounce */
        bounciness?: number);
        /**
         * @internal
         */
        easeInCore(gradient: number): number;
    }
    /**
     * Easing function with a power of 3 shape (see link below).
     * @see https://easings.net/#easeInCubic
     * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/advanced_animations#easing-functions
     */
    export class CubicEase extends EasingFunction implements IEasingFunction {
        /**
         * @internal
         */
        easeInCore(gradient: number): number;
    }
    /**
     * Easing function with an elastic shape (see link below).
     * @see https://easings.net/#easeInElastic
     * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/advanced_animations#easing-functions
     */
    export class ElasticEase extends EasingFunction implements IEasingFunction {
        /** Defines the number of oscillations*/
        oscillations: number;
        /** Defines the amplitude of the oscillations*/
        springiness: number;
        /**
         * Instantiates an elastic easing function
         * @see https://easings.net/#easeInElastic
         * @param oscillations Defines the number of oscillations
         * @param springiness Defines the amplitude of the oscillations
         */
        constructor(
        /** Defines the number of oscillations*/
        oscillations?: number, 
        /** Defines the amplitude of the oscillations*/
        springiness?: number);
        /**
         * @internal
         */
        easeInCore(gradient: number): number;
    }
    /**
     * Easing function with an exponential shape (see link below).
     * @see https://easings.net/#easeInExpo
     * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/advanced_animations#easing-functions
     */
    export class ExponentialEase extends EasingFunction implements IEasingFunction {
        /** Defines the exponent of the function */
        exponent: number;
        /**
         * Instantiates an exponential easing function
         * @see https://easings.net/#easeInExpo
         * @param exponent Defines the exponent of the function
         */
        constructor(
        /** Defines the exponent of the function */
        exponent?: number);
        /**
         * @internal
         */
        easeInCore(gradient: number): number;
    }
    /**
     * Easing function with a power shape (see link below).
     * @see https://easings.net/#easeInQuad
     * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/advanced_animations#easing-functions
     */
    export class PowerEase extends EasingFunction implements IEasingFunction {
        /** Defines the power of the function */
        power: number;
        /**
         * Instantiates an power base easing function
         * @see https://easings.net/#easeInQuad
         * @param power Defines the power of the function
         */
        constructor(
        /** Defines the power of the function */
        power?: number);
        /**
         * @internal
         */
        easeInCore(gradient: number): number;
    }
    /**
     * Easing function with a power of 2 shape (see link below).
     * @see https://easings.net/#easeInQuad
     * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/advanced_animations#easing-functions
     */
    export class QuadraticEase extends EasingFunction implements IEasingFunction {
        /**
         * @internal
         */
        easeInCore(gradient: number): number;
    }
    /**
     * Easing function with a power of 4 shape (see link below).
     * @see https://easings.net/#easeInQuart
     * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/advanced_animations#easing-functions
     */
    export class QuarticEase extends EasingFunction implements IEasingFunction {
        /**
         * @internal
         */
        easeInCore(gradient: number): number;
    }
    /**
     * Easing function with a power of 5 shape (see link below).
     * @see https://easings.net/#easeInQuint
     * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/advanced_animations#easing-functions
     */
    export class QuinticEase extends EasingFunction implements IEasingFunction {
        /**
         * @internal
         */
        easeInCore(gradient: number): number;
    }
    /**
     * Easing function with a sin shape (see link below).
     * @see https://easings.net/#easeInSine
     * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/advanced_animations#easing-functions
     */
    export class SineEase extends EasingFunction implements IEasingFunction {
        /**
         * @internal
         */
        easeInCore(gradient: number): number;
    }
    /**
     * Easing function with a bezier shape (see link below).
     * @see http://cubic-bezier.com/#.17,.67,.83,.67
     * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/advanced_animations#easing-functions
     */
    export class BezierCurveEase extends EasingFunction implements IEasingFunction {
        /** Defines the x component of the start tangent in the bezier curve */
        x1: number;
        /** Defines the y component of the start tangent in the bezier curve */
        y1: number;
        /** Defines the x component of the end tangent in the bezier curve */
        x2: number;
        /** Defines the y component of the end tangent in the bezier curve */
        y2: number;
        /**
         * Instantiates a bezier function
         * @see http://cubic-bezier.com/#.17,.67,.83,.67
         * @param x1 Defines the x component of the start tangent in the bezier curve
         * @param y1 Defines the y component of the start tangent in the bezier curve
         * @param x2 Defines the x component of the end tangent in the bezier curve
         * @param y2 Defines the y component of the end tangent in the bezier curve
         */
        constructor(
        /** Defines the x component of the start tangent in the bezier curve */
        x1?: number, 
        /** Defines the y component of the start tangent in the bezier curve */
        y1?: number, 
        /** Defines the x component of the end tangent in the bezier curve */
        x2?: number, 
        /** Defines the y component of the end tangent in the bezier curve */
        y2?: number);
        /**
         * @internal
         */
        easeInCore(gradient: number): number;
    }




    /**
     * A cursor which tracks a point on a path
     */
    export class PathCursor {
        private _path;
        /**
         * Stores path cursor callbacks for when an onchange event is triggered
         */
        private _onchange;
        /**
         * The value of the path cursor
         */
        value: number;
        /**
         * The animation array of the path cursor
         */
        animations: Animation[];
        /**
         * Initializes the path cursor
         * @param _path The path to track
         */
        constructor(_path: Path2);
        /**
         * Gets the cursor point on the path
         * @returns A point on the path cursor at the cursor location
         */
        getPoint(): Vector3;
        /**
         * Moves the cursor ahead by the step amount
         * @param step The amount to move the cursor forward
         * @returns This path cursor
         */
        moveAhead(step?: number): PathCursor;
        /**
         * Moves the cursor behind by the step amount
         * @param step The amount to move the cursor back
         * @returns This path cursor
         */
        moveBack(step?: number): PathCursor;
        /**
         * Moves the cursor by the step amount
         * If the step amount is greater than one, an exception is thrown
         * @param step The amount to move the cursor
         * @returns This path cursor
         */
        move(step: number): PathCursor;
        /**
         * Ensures that the value is limited between zero and one
         * @returns This path cursor
         */
        private _ensureLimits;
        /**
         * Runs onchange callbacks on change (used by the animation engine)
         * @returns This path cursor
         */
        private _raiseOnChange;
        /**
         * Executes a function on change
         * @param f A path cursor onchange callback
         * @returns This path cursor
         */
        onchange(f: (cursor: PathCursor) => void): PathCursor;
    }


    /**
     * Defines a runtime animation
     */
    export class RuntimeAnimation {
        private _events;
        /**
         * The current frame of the runtime animation
         */
        private _currentFrame;
        /**
         * The animation used by the runtime animation
         */
        private _animation;
        /**
         * The target of the runtime animation
         */
        private _target;
        /**
         * The initiating animatable
         */
        private _host;
        /**
         * The original value of the runtime animation
         */
        private _originalValue;
        /**
         * The original blend value of the runtime animation
         */
        private _originalBlendValue;
        /**
         * The offsets cache of the runtime animation
         */
        private _offsetsCache;
        /**
         * The high limits cache of the runtime animation
         */
        private _highLimitsCache;
        /**
         * Specifies if the runtime animation has been stopped
         */
        private _stopped;
        /**
         * The blending factor of the runtime animation
         */
        private _blendingFactor;
        /**
         * The BabylonJS scene
         */
        private _scene;
        /**
         * The current value of the runtime animation
         */
        private _currentValue;
        /** @internal */
        _animationState: _IAnimationState;
        /**
         * The active target of the runtime animation
         */
        private _activeTargets;
        private _currentActiveTarget;
        private _directTarget;
        /**
         * The target path of the runtime animation
         */
        private _targetPath;
        /**
         * The weight of the runtime animation
         */
        private _weight;
        /**
         * The ratio offset of the runtime animation
         */
        private _ratioOffset;
        /**
         * The previous delay of the runtime animation
         */
        private _previousDelay;
        /**
         * The previous ratio of the runtime animation
         */
        private _previousRatio;
        private _enableBlending;
        private _keys;
        private _minFrame;
        private _maxFrame;
        private _minValue;
        private _maxValue;
        private _targetIsArray;
        /**
         * Gets the current frame of the runtime animation
         */
        get currentFrame(): number;
        /**
         * Gets the weight of the runtime animation
         */
        get weight(): number;
        /**
         * Gets the current value of the runtime animation
         */
        get currentValue(): any;
        /**
         * Gets or sets the target path of the runtime animation
         */
        get targetPath(): string;
        /**
         * Gets the actual target of the runtime animation
         */
        get target(): any;
        /**
         * Gets the additive state of the runtime animation
         */
        get isAdditive(): boolean;
        /** @internal */
        _onLoop: () => void;
        /**
         * Create a new RuntimeAnimation object
         * @param target defines the target of the animation
         * @param animation defines the source animation object
         * @param scene defines the hosting scene
         * @param host defines the initiating Animatable
         */
        constructor(target: any, animation: Animation, scene: Scene, host: Animatable);
        private _preparePath;
        /**
         * Gets the animation from the runtime animation
         */
        get animation(): Animation;
        /**
         * Resets the runtime animation to the beginning
         * @param restoreOriginal defines whether to restore the target property to the original value
         */
        reset(restoreOriginal?: boolean): void;
        /**
         * Specifies if the runtime animation is stopped
         * @returns Boolean specifying if the runtime animation is stopped
         */
        isStopped(): boolean;
        /**
         * Disposes of the runtime animation
         */
        dispose(): void;
        /**
         * Apply the interpolated value to the target
         * @param currentValue defines the value computed by the animation
         * @param weight defines the weight to apply to this value (Defaults to 1.0)
         */
        setValue(currentValue: any, weight: number): void;
        private _getOriginalValues;
        private _setValue;
        /**
         * Gets the loop pmode of the runtime animation
         * @returns Loop Mode
         */
        private _getCorrectLoopMode;
        /**
         * Move the current animation to a given frame
         * @param frame defines the frame to move to
         */
        goToFrame(frame: number): void;
        /**
         * @internal Internal use only
         */
        _prepareForSpeedRatioChange(newSpeedRatio: number): void;
        /**
         * Execute the current animation
         * @param delay defines the delay to add to the current frame
         * @param from defines the lower bound of the animation range
         * @param to defines the upper bound of the animation range
         * @param loop defines if the current animation must loop
         * @param speedRatio defines the current speed ratio
         * @param weight defines the weight of the animation (default is -1 so no weight)
         * @returns a boolean indicating if the animation is running
         */
        animate(delay: number, from: number, to: number, loop: boolean, speedRatio: number, weight?: number): boolean;
    }


    /**
     * Set of assets to keep when moving a scene into an asset container.
     */
    export class KeepAssets extends AbstractScene {
    }
    /**
     * Class used to store the output of the AssetContainer.instantiateAllMeshesToScene function
     */
    export class InstantiatedEntries {
        /**
         * List of new root nodes (eg. nodes with no parent)
         */
        rootNodes: TransformNode[];
        /**
         * List of new skeletons
         */
        skeletons: Skeleton[];
        /**
         * List of new animation groups
         */
        animationGroups: AnimationGroup[];
        /**
         * Disposes the instantiated entries from the scene
         */
        dispose(): void;
    }
    /**
     * Container with a set of assets that can be added or removed from a scene.
     */
    export class AssetContainer extends AbstractScene {
        private _wasAddedToScene;
        private _onContextRestoredObserver;
        /**
         * The scene the AssetContainer belongs to.
         */
        scene: Scene;
        /**
         * Instantiates an AssetContainer.
         * @param scene The scene the AssetContainer belongs to.
         */
        constructor(scene?: Nullable<Scene>);
        /**
         * Given a list of nodes, return a topological sorting of them.
         * @param nodes
         */
        private _topologicalSort;
        private _addNodeAndDescendantsToList;
        /**
         * Check if a specific node is contained in this asset container.
         * @param node
         */
        private _isNodeInContainer;
        /**
         * For every node in the scene, check if its parent node is also in the scene.
         */
        private _isValidHierarchy;
        /**
         * Instantiate or clone all meshes and add the new ones to the scene.
         * Skeletons and animation groups will all be cloned
         * @param nameFunction defines an optional function used to get new names for clones
         * @param cloneMaterials defines an optional boolean that defines if materials must be cloned as well (false by default)
         * @param options defines an optional list of options to control how to instantiate / clone models
         * @param options.doNotInstantiate defines if the model must be instantiated or just cloned
         * @param options.predicate defines a predicate used to filter whih mesh to instantiate/clone
         * @returns a list of rootNodes, skeletons and animation groups that were duplicated
         */
        instantiateModelsToScene(nameFunction?: (sourceName: string) => string, cloneMaterials?: boolean, options?: {
            doNotInstantiate?: boolean | ((node: TransformNode) => boolean);
            predicate?: (entity: any) => boolean;
        }): InstantiatedEntries;
        /**
         * Adds all the assets from the container to the scene.
         */
        addAllToScene(): void;
        /**
         * Adds assets from the container to the scene.
         * @param predicate defines a predicate used to select which entity will be added (can be null)
         */
        addToScene(predicate?: Nullable<(entity: any) => boolean>): void;
        /**
         * Removes all the assets in the container from the scene
         */
        removeAllFromScene(): void;
        /**
         * Removes assets in the container from the scene
         * @param predicate defines a predicate used to select which entity will be added (can be null)
         */
        removeFromScene(predicate?: Nullable<(entity: any) => boolean>): void;
        /**
         * Disposes all the assets in the container
         */
        dispose(): void;
        private _moveAssets;
        /**
         * Removes all the assets contained in the scene and adds them to the container.
         * @param keepAssets Set of assets to keep in the scene. (default: empty)
         */
        moveAllFromScene(keepAssets?: KeepAssets): void;
        /**
         * Adds all meshes in the asset container to a root mesh that can be used to position all the contained meshes. The root mesh is then added to the front of the meshes in the assetContainer.
         * @returns the root mesh
         */
        createRootMesh(): Mesh;
        /**
         * Merge animations (direct and animation groups) from this asset container into a scene
         * @param scene is the instance of BABYLON.Scene to append to (default: last created scene)
         * @param animatables set of animatables to retarget to a node from the scene
         * @param targetConverter defines a function used to convert animation targets from the asset container to the scene (default: search node by name)
         * @returns an array of the new AnimationGroup added to the scene (empty array if none)
         */
        mergeAnimationsTo(scene: Nullable<Scene> | undefined, animatables: Animatable[], targetConverter?: Nullable<(target: any) => Nullable<Node>>): AnimationGroup[];
    }


    /**
     * Class used to work with sound analyzer using fast fourier transform (FFT)
     * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic
     */
    export class Analyser {
        /**
         * Gets or sets the smoothing
         * @ignorenaming
         */
        SMOOTHING: number;
        /**
         * Gets or sets the FFT table size
         * @ignorenaming
         */
        FFT_SIZE: number;
        /**
         * Gets or sets the bar graph amplitude
         * @ignorenaming
         */
        BARGRAPHAMPLITUDE: number;
        /**
         * Gets or sets the position of the debug canvas
         * @ignorenaming
         */
        DEBUGCANVASPOS: {
            x: number;
            y: number;
        };
        /**
         * Gets or sets the debug canvas size
         * @ignorenaming
         */
        DEBUGCANVASSIZE: {
            width: number;
            height: number;
        };
        private _byteFreqs;
        private _byteTime;
        private _floatFreqs;
        private _webAudioAnalyser;
        private _debugCanvas;
        private _debugCanvasContext;
        private _scene;
        private _registerFunc;
        private _audioEngine;
        /**
         * Creates a new analyser
         * @param scene defines hosting scene
         */
        constructor(scene?: Nullable<Scene>);
        /**
         * Get the number of data values you will have to play with for the visualization
         * @see https://developer.mozilla.org/en-US/docs/Web/API/AnalyserNode/frequencyBinCount
         * @returns a number
         */
        getFrequencyBinCount(): number;
        /**
         * Gets the current frequency data as a byte array
         * @see https://developer.mozilla.org/en-US/docs/Web/API/AnalyserNode/getByteFrequencyData
         * @returns a Uint8Array
         */
        getByteFrequencyData(): Uint8Array;
        /**
         * Gets the current waveform as a byte array
         * @see https://developer.mozilla.org/en-US/docs/Web/API/AnalyserNode/getByteTimeDomainData
         * @returns a Uint8Array
         */
        getByteTimeDomainData(): Uint8Array;
        /**
         * Gets the current frequency data as a float array
         * @see https://developer.mozilla.org/en-US/docs/Web/API/AnalyserNode/getByteFrequencyData
         * @returns a Float32Array
         */
        getFloatFrequencyData(): Float32Array;
        /**
         * Renders the debug canvas
         */
        drawDebugCanvas(): void;
        /**
         * Stops rendering the debug canvas and removes it
         */
        stopDebugCanvas(): void;
        /**
         * Connects two audio nodes
         * @param inputAudioNode defines first node to connect
         * @param outputAudioNode defines second node to connect
         */
        connectAudioNodes(inputAudioNode: AudioNode, outputAudioNode: AudioNode): void;
        /**
         * Releases all associated resources
         */
        dispose(): void;
    }


    /**
     * This represents the default audio engine used in babylon.
     * It is responsible to play, synchronize and analyse sounds throughout the  application.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic
     */
    export class AudioEngine implements IAudioEngine {
        private _audioContext;
        private _audioContextInitialized;
        private _muteButton;
        private _hostElement;
        private _audioDestination;
        /**
         * Gets whether the current host supports Web Audio and thus could create AudioContexts.
         */
        canUseWebAudio: boolean;
        /**
         * The master gain node defines the global audio volume of your audio engine.
         */
        masterGain: GainNode;
        /**
         * Defines if Babylon should emit a warning if WebAudio is not supported.
         * @ignoreNaming
         */
        WarnedWebAudioUnsupported: boolean;
        /**
         * Gets whether or not mp3 are supported by your browser.
         */
        isMP3supported: boolean;
        /**
         * Gets whether or not ogg are supported by your browser.
         */
        isOGGsupported: boolean;
        /**
         * Gets whether audio has been unlocked on the device.
         * Some Browsers have strong restrictions about Audio and won t autoplay unless
         * a user interaction has happened.
         */
        unlocked: boolean;
        /**
         * Defines if the audio engine relies on a custom unlocked button.
         * In this case, the embedded button will not be displayed.
         */
        useCustomUnlockedButton: boolean;
        /**
         * Event raised when audio has been unlocked on the browser.
         */
        onAudioUnlockedObservable: Observable<IAudioEngine>;
        /**
         * Event raised when audio has been locked on the browser.
         */
        onAudioLockedObservable: Observable<IAudioEngine>;
        /**
         * Gets the current AudioContext if available.
         */
        get audioContext(): Nullable<AudioContext>;
        private _connectedAnalyser;
        /**
         * Instantiates a new audio engine.
         *
         * There should be only one per page as some browsers restrict the number
         * of audio contexts you can create.
         * @param hostElement defines the host element where to display the mute icon if necessary
         * @param audioContext defines the audio context to be used by the audio engine
         * @param audioDestination defines the audio destination node to be used by audio engine
         */
        constructor(hostElement?: Nullable<HTMLElement>, audioContext?: Nullable<AudioContext>, audioDestination?: Nullable<AudioDestinationNode | MediaStreamAudioDestinationNode>);
        /**
         * Flags the audio engine in Locked state.
         * This happens due to new browser policies preventing audio to autoplay.
         */
        lock(): void;
        /**
         * Unlocks the audio engine once a user action has been done on the dom.
         * This is helpful to resume play once browser policies have been satisfied.
         */
        unlock(): void;
        private _resumeAudioContext;
        private _initializeAudioContext;
        private _tryToRun;
        private _triggerRunningState;
        private _triggerSuspendedState;
        private _displayMuteButton;
        private _moveButtonToTopLeft;
        private _onResize;
        private _hideMuteButton;
        /**
         * Destroy and release the resources associated with the audio context.
         */
        dispose(): void;
        /**
         * Gets the global volume sets on the master gain.
         * @returns the global volume if set or -1 otherwise
         */
        getGlobalVolume(): number;
        /**
         * Sets the global volume of your experience (sets on the master gain).
         * @param newVolume Defines the new global volume of the application
         */
        setGlobalVolume(newVolume: number): void;
        /**
         * Connect the audio engine to an audio analyser allowing some amazing
         * synchronization between the sounds/music and your visualization (VuMeter for instance).
         * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic#using-the-analyser
         * @param analyser The analyser to connect to the engine
         */
        connectToAnalyser(analyser: Analyser): void;
    }


        interface AbstractScene {
            /**
             * The list of sounds used in the scene.
             */
            sounds: Nullable<Array<Sound>>;
        }
        interface Scene {
            /**
             * @internal
             * Backing field
             */
            _mainSoundTrack: SoundTrack;
            /**
             * The main sound track played by the scene.
             * It contains your primary collection of sounds.
             */
            mainSoundTrack: SoundTrack;
            /**
             * The list of sound tracks added to the scene
             * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic
             */
            soundTracks: Nullable<Array<SoundTrack>>;
            /**
             * Gets a sound using a given name
             * @param name defines the name to search for
             * @returns the found sound or null if not found at all.
             */
            getSoundByName(name: string): Nullable<Sound>;
            /**
             * Gets or sets if audio support is enabled
             * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic
             */
            audioEnabled: boolean;
            /**
             * Gets or sets if audio will be output to headphones
             * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic
             */
            headphone: boolean;
            /**
             * Gets or sets custom audio listener position provider
             * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic
             */
            audioListenerPositionProvider: Nullable<() => Vector3>;
            /**
             * Gets or sets custom audio listener rotation provider
             * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic
             */
            audioListenerRotationProvider: Nullable<() => Vector3>;
            /**
             * Gets or sets a refresh rate when using 3D audio positioning
             */
            audioPositioningRefreshRate: number;
        }
    /**
     * Defines the sound scene component responsible to manage any sounds
     * in a given scene.
     */
    export class AudioSceneComponent implements ISceneSerializableComponent {
        private static _CameraDirection;
        /**
         * The component name helpful to identify the component in the list of scene components.
         */
        readonly name = "Audio";
        /**
         * The scene the component belongs to.
         */
        scene: Scene;
        private _audioEnabled;
        /**
         * Gets whether audio is enabled or not.
         * Please use related enable/disable method to switch state.
         */
        get audioEnabled(): boolean;
        private _headphone;
        /**
         * Gets whether audio is outputting to headphone or not.
         * Please use the according Switch methods to change output.
         */
        get headphone(): boolean;
        /**
         * Gets or sets a refresh rate when using 3D audio positioning
         */
        audioPositioningRefreshRate: number;
        /**
         * Gets or Sets a custom listener position for all sounds in the scene
         * By default, this is the position of the first active camera
         */
        audioListenerPositionProvider: Nullable<() => Vector3>;
        /**
         * Gets or Sets a custom listener rotation for all sounds in the scene
         * By default, this is the rotation of the first active camera
         */
        audioListenerRotationProvider: Nullable<() => Vector3>;
        /**
         * Creates a new instance of the component for the given scene
         * @param scene Defines the scene to register the component in
         */
        constructor(scene?: Nullable<Scene>);
        /**
         * Registers the component in a given scene
         */
        register(): void;
        /**
         * Rebuilds the elements related to this component in case of
         * context lost for instance.
         */
        rebuild(): void;
        /**
         * Serializes the component data to the specified json object
         * @param serializationObject The object to serialize to
         */
        serialize(serializationObject: any): void;
        /**
         * Adds all the elements from the container to the scene
         * @param container the container holding the elements
         */
        addFromContainer(container: AbstractScene): void;
        /**
         * Removes all the elements in the container from the scene
         * @param container contains the elements to remove
         * @param dispose if the removed element should be disposed (default: false)
         */
        removeFromContainer(container: AbstractScene, dispose?: boolean): void;
        /**
         * Disposes the component and the associated resources.
         */
        dispose(): void;
        /**
         * Disables audio in the associated scene.
         */
        disableAudio(): void;
        /**
         * Enables audio in the associated scene.
         */
        enableAudio(): void;
        /**
         * Switch audio to headphone output.
         */
        switchAudioModeForHeadphones(): void;
        /**
         * Switch audio to normal speakers.
         */
        switchAudioModeForNormalSpeakers(): void;
        private _cachedCameraDirection;
        private _cachedCameraPosition;
        private _lastCheck;
        private _invertMatrixTemp;
        private _cameraDirectionTemp;
        private _afterRender;
    }




    /**
     * This represents an audio engine and it is responsible
     * to play, synchronize and analyse sounds throughout the application.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic
     */
    export interface IAudioEngine extends IDisposable {
        /**
         * Gets whether the current host supports Web Audio and thus could create AudioContexts.
         */
        readonly canUseWebAudio: boolean;
        /**
         * Gets the current AudioContext if available.
         */
        readonly audioContext: Nullable<AudioContext>;
        /**
         * The master gain node defines the global audio volume of your audio engine.
         */
        readonly masterGain: GainNode;
        /**
         * Gets whether or not mp3 are supported by your browser.
         */
        readonly isMP3supported: boolean;
        /**
         * Gets whether or not ogg are supported by your browser.
         */
        readonly isOGGsupported: boolean;
        /**
         * Defines if Babylon should emit a warning if WebAudio is not supported.
         * @ignoreNaming
         */
        WarnedWebAudioUnsupported: boolean;
        /**
         * Defines if the audio engine relies on a custom unlocked button.
         * In this case, the embedded button will not be displayed.
         */
        useCustomUnlockedButton: boolean;
        /**
         * Gets whether or not the audio engine is unlocked (require first a user gesture on some browser).
         */
        readonly unlocked: boolean;
        /**
         * Event raised when audio has been unlocked on the browser.
         */
        onAudioUnlockedObservable: Observable<IAudioEngine>;
        /**
         * Event raised when audio has been locked on the browser.
         */
        onAudioLockedObservable: Observable<IAudioEngine>;
        /**
         * Flags the audio engine in Locked state.
         * This happens due to new browser policies preventing audio to autoplay.
         */
        lock(): void;
        /**
         * Unlocks the audio engine once a user action has been done on the dom.
         * This is helpful to resume play once browser policies have been satisfied.
         */
        unlock(): void;
        /**
         * Gets the global volume sets on the master gain.
         * @returns the global volume if set or -1 otherwise
         */
        getGlobalVolume(): number;
        /**
         * Sets the global volume of your experience (sets on the master gain).
         * @param newVolume Defines the new global volume of the application
         */
        setGlobalVolume(newVolume: number): void;
        /**
         * Connect the audio engine to an audio analyser allowing some amazing
         * synchronization between the sounds/music and your visualization (VuMeter for instance).
         * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic#using-the-analyser
         * @param analyser The analyser to connect to the engine
         */
        connectToAnalyser(analyser: Analyser): void;
    }


    /**
     * Interface used to define options for the Audio Engine
     * @since 5.0.0
     */
    export interface IAudioEngineOptions {
        /**
         * Specifies an existing Audio Context for the audio engine
         */
        audioContext?: AudioContext;
        /**
         * Specifies a destination node for the audio engine
         */
        audioDestination?: AudioDestinationNode | MediaStreamAudioDestinationNode;
    }


    /**
     * Interface used to define options for Sound class
     */
    export interface ISoundOptions {
        /**
         * Does the sound autoplay once loaded.
         */
        autoplay?: boolean;
        /**
         * Does the sound loop after it finishes playing once.
         */
        loop?: boolean;
        /**
         * Sound's volume
         */
        volume?: number;
        /**
         * Is it a spatial sound?
         */
        spatialSound?: boolean;
        /**
         * Maximum distance to hear that sound
         */
        maxDistance?: number;
        /**
         * Uses user defined attenuation function
         */
        useCustomAttenuation?: boolean;
        /**
         * Define the roll off factor of spatial sounds.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic#creating-a-spatial-3d-sound
         */
        rolloffFactor?: number;
        /**
         * Define the reference distance the sound should be heard perfectly.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic#creating-a-spatial-3d-sound
         */
        refDistance?: number;
        /**
         * Define the distance attenuation model the sound will follow.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic#creating-a-spatial-3d-sound
         */
        distanceModel?: string;
        /**
         * Defines the playback speed (1 by default)
         */
        playbackRate?: number;
        /**
         * Defines if the sound is from a streaming source
         */
        streaming?: boolean;
        /**
         * Defines an optional length (in seconds) inside the sound file
         */
        length?: number;
        /**
         * Defines an optional offset (in seconds) inside the sound file
         */
        offset?: number;
        /**
         * If true, URLs will not be required to state the audio file codec to use.
         */
        skipCodecCheck?: boolean;
    }


    /**
     * Defines a sound that can be played in the application.
     * The sound can either be an ambient track or a simple sound played in reaction to a user action.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic
     */
    export class Sound {
        /**
         * The name of the sound in the scene.
         */
        name: string;
        /**
         * Does the sound autoplay once loaded.
         */
        autoplay: boolean;
        private _loop;
        /**
         * Does the sound loop after it finishes playing once.
         */
        get loop(): boolean;
        set loop(value: boolean);
        /**
         * Does the sound use a custom attenuation curve to simulate the falloff
         * happening when the source gets further away from the camera.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic#creating-your-own-custom-attenuation-function
         */
        useCustomAttenuation: boolean;
        /**
         * The sound track id this sound belongs to.
         */
        soundTrackId: number;
        /**
         * Is this sound currently played.
         */
        isPlaying: boolean;
        /**
         * Is this sound currently paused.
         */
        isPaused: boolean;
        /**
         * Define the reference distance the sound should be heard perfectly.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic#creating-a-spatial-3d-sound
         */
        refDistance: number;
        /**
         * Define the roll off factor of spatial sounds.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic#creating-a-spatial-3d-sound
         */
        rolloffFactor: number;
        /**
         * Define the max distance the sound should be heard (intensity just became 0 at this point).
         * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic#creating-a-spatial-3d-sound
         */
        maxDistance: number;
        /**
         * Define the distance attenuation model the sound will follow.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic#creating-a-spatial-3d-sound
         */
        distanceModel: string;
        /**
         * @internal
         * Back Compat
         **/
        onended: () => any;
        /**
         * Gets or sets an object used to store user defined information for the sound.
         */
        metadata: any;
        /**
         * Observable event when the current playing sound finishes.
         */
        onEndedObservable: Observable<Sound>;
        /**
         * Gets the current time for the sound.
         */
        get currentTime(): number;
        /**
         * Does this sound enables spatial sound.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic#creating-a-spatial-3d-sound
         */
        get spatialSound(): boolean;
        /**
         * Does this sound enables spatial sound.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic#creating-a-spatial-3d-sound
         */
        set spatialSound(newValue: boolean);
        private _spatialSound;
        private _panningModel;
        private _playbackRate;
        private _streaming;
        private _startTime;
        private _currentTime;
        private _position;
        private _localDirection;
        private _volume;
        private _isReadyToPlay;
        private _isDirectional;
        private _readyToPlayCallback;
        private _audioBuffer;
        private _soundSource;
        private _streamingSource;
        private _soundPanner;
        private _soundGain;
        private _inputAudioNode;
        private _outputAudioNode;
        private _coneInnerAngle;
        private _coneOuterAngle;
        private _coneOuterGain;
        private _scene;
        private _connectedTransformNode;
        private _customAttenuationFunction;
        private _registerFunc;
        private _isOutputConnected;
        private _htmlAudioElement;
        private _urlType;
        private _length?;
        private _offset?;
        /**
         * @internal
         */
        static _SceneComponentInitialization: (scene: Scene) => void;
        /**
         * Create a sound and attach it to a scene
         * @param name Name of your sound
         * @param urlOrArrayBuffer Url to the sound to load async or ArrayBuffer, it also works with MediaStreams and AudioBuffers
         * @param scene defines the scene the sound belongs to
         * @param readyToPlayCallback Provide a callback function if you'd like to load your code once the sound is ready to be played
         * @param options Objects to provide with the current available options: autoplay, loop, volume, spatialSound, maxDistance, rolloffFactor, refDistance, distanceModel, panningModel, streaming
         */
        constructor(name: string, urlOrArrayBuffer: any, scene?: Nullable<Scene>, readyToPlayCallback?: Nullable<() => void>, options?: ISoundOptions);
        /**
         * Release the sound and its associated resources
         */
        dispose(): void;
        /**
         * Gets if the sounds is ready to be played or not.
         * @returns true if ready, otherwise false
         */
        isReady(): boolean;
        /**
         * Get the current class name.
         * @returns current class name
         */
        getClassName(): string;
        private _audioBufferLoaded;
        private _soundLoaded;
        /**
         * Sets the data of the sound from an audiobuffer
         * @param audioBuffer The audioBuffer containing the data
         */
        setAudioBuffer(audioBuffer: AudioBuffer): void;
        /**
         * Updates the current sounds options such as maxdistance, loop...
         * @param options A JSON object containing values named as the object properties
         */
        updateOptions(options: ISoundOptions): void;
        private _createSpatialParameters;
        private _updateSpatialParameters;
        /**
         * Switch the panning model to HRTF:
         * Renders a stereo output of higher quality than equalpower — it uses a convolution with measured impulse responses from human subjects.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic#creating-a-spatial-3d-sound
         */
        switchPanningModelToHRTF(): void;
        /**
         * Switch the panning model to Equal Power:
         * Represents the equal-power panning algorithm, generally regarded as simple and efficient. equalpower is the default value.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic#creating-a-spatial-3d-sound
         */
        switchPanningModelToEqualPower(): void;
        private _switchPanningModel;
        /**
         * Connect this sound to a sound track audio node like gain...
         * @param soundTrackAudioNode the sound track audio node to connect to
         */
        connectToSoundTrackAudioNode(soundTrackAudioNode: AudioNode): void;
        /**
         * Transform this sound into a directional source
         * @param coneInnerAngle Size of the inner cone in degree
         * @param coneOuterAngle Size of the outer cone in degree
         * @param coneOuterGain Volume of the sound outside the outer cone (between 0.0 and 1.0)
         */
        setDirectionalCone(coneInnerAngle: number, coneOuterAngle: number, coneOuterGain: number): void;
        /**
         * Gets or sets the inner angle for the directional cone.
         */
        get directionalConeInnerAngle(): number;
        /**
         * Gets or sets the inner angle for the directional cone.
         */
        set directionalConeInnerAngle(value: number);
        /**
         * Gets or sets the outer angle for the directional cone.
         */
        get directionalConeOuterAngle(): number;
        /**
         * Gets or sets the outer angle for the directional cone.
         */
        set directionalConeOuterAngle(value: number);
        /**
         * Sets the position of the emitter if spatial sound is enabled
         * @param newPosition Defines the new position
         */
        setPosition(newPosition: Vector3): void;
        /**
         * Sets the local direction of the emitter if spatial sound is enabled
         * @param newLocalDirection Defines the new local direction
         */
        setLocalDirectionToMesh(newLocalDirection: Vector3): void;
        private _updateDirection;
        /** @internal */
        updateDistanceFromListener(): void;
        /**
         * Sets a new custom attenuation function for the sound.
         * @param callback Defines the function used for the attenuation
         * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic#creating-your-own-custom-attenuation-function
         */
        setAttenuationFunction(callback: (currentVolume: number, currentDistance: number, maxDistance: number, refDistance: number, rolloffFactor: number) => number): void;
        /**
         * Play the sound
         * @param time (optional) Start the sound after X seconds. Start immediately (0) by default.
         * @param offset (optional) Start the sound at a specific time in seconds
         * @param length (optional) Sound duration (in seconds)
         */
        play(time?: number, offset?: number, length?: number): void;
        private _onended;
        /**
         * Stop the sound
         * @param time (optional) Stop the sound after X seconds. Stop immediately (0) by default.
         */
        stop(time?: number): void;
        /**
         * Put the sound in pause
         */
        pause(): void;
        /**
         * Sets a dedicated volume for this sounds
         * @param newVolume Define the new volume of the sound
         * @param time Define time for gradual change to new volume
         */
        setVolume(newVolume: number, time?: number): void;
        /**
         * Set the sound play back rate
         * @param newPlaybackRate Define the playback rate the sound should be played at
         */
        setPlaybackRate(newPlaybackRate: number): void;
        /**
         * Gets the sound play back rate.
         * @returns the  play back rate of the sound
         */
        getPlaybackRate(): number;
        /**
         * Gets the volume of the sound.
         * @returns the volume of the sound
         */
        getVolume(): number;
        /**
         * Attach the sound to a dedicated mesh
         * @param transformNode The transform node to connect the sound with
         * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic#attaching-a-sound-to-a-mesh
         */
        attachToMesh(transformNode: TransformNode): void;
        /**
         * Detach the sound from the previously attached mesh
         * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic#attaching-a-sound-to-a-mesh
         */
        detachFromMesh(): void;
        private _onRegisterAfterWorldMatrixUpdate;
        /**
         * Clone the current sound in the scene.
         * @returns the new sound clone
         */
        clone(): Nullable<Sound>;
        /**
         * Gets the current underlying audio buffer containing the data
         * @returns the audio buffer
         */
        getAudioBuffer(): Nullable<AudioBuffer>;
        /**
         * Gets the WebAudio AudioBufferSourceNode, lets you keep track of and stop instances of this Sound.
         * @returns the source node
         */
        getSoundSource(): Nullable<AudioBufferSourceNode>;
        /**
         * Gets the WebAudio GainNode, gives you precise control over the gain of instances of this Sound.
         * @returns the gain node
         */
        getSoundGain(): Nullable<GainNode>;
        /**
         * Serializes the Sound in a JSON representation
         * @returns the JSON representation of the sound
         */
        serialize(): any;
        /**
         * Parse a JSON representation of a sound to instantiate in a given scene
         * @param parsedSound Define the JSON representation of the sound (usually coming from the serialize method)
         * @param scene Define the scene the new parsed sound should be created in
         * @param rootUrl Define the rooturl of the load in case we need to fetch relative dependencies
         * @param sourceSound Define a sound place holder if do not need to instantiate a new one
         * @returns the newly parsed sound
         */
        static Parse(parsedSound: any, scene: Scene, rootUrl: string, sourceSound?: Sound): Sound;
        private _setOffset;
    }


    /**
     * Options allowed during the creation of a sound track.
     */
    export interface ISoundTrackOptions {
        /**
         * The volume the sound track should take during creation
         */
        volume?: number;
        /**
         * Define if the sound track is the main sound track of the scene
         */
        mainTrack?: boolean;
    }
    /**
     * It could be useful to isolate your music & sounds on several tracks to better manage volume on a grouped instance of sounds.
     * It will be also used in a future release to apply effects on a specific track.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic#using-sound-tracks
     */
    export class SoundTrack {
        /**
         * The unique identifier of the sound track in the scene.
         */
        id: number;
        /**
         * The list of sounds included in the sound track.
         */
        soundCollection: Array<Sound>;
        private _outputAudioNode;
        private _scene;
        private _connectedAnalyser;
        private _options;
        private _isInitialized;
        /**
         * Creates a new sound track.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic#using-sound-tracks
         * @param scene Define the scene the sound track belongs to
         * @param options
         */
        constructor(scene?: Nullable<Scene>, options?: ISoundTrackOptions);
        private _initializeSoundTrackAudioGraph;
        /**
         * Release the sound track and its associated resources
         */
        dispose(): void;
        /**
         * Adds a sound to this sound track
         * @param sound define the sound to add
         * @ignoreNaming
         */
        addSound(sound: Sound): void;
        /**
         * Removes a sound to this sound track
         * @param sound define the sound to remove
         * @ignoreNaming
         */
        removeSound(sound: Sound): void;
        /**
         * Set a global volume for the full sound track.
         * @param newVolume Define the new volume of the sound track
         */
        setVolume(newVolume: number): void;
        /**
         * Switch the panning model to HRTF:
         * Renders a stereo output of higher quality than equalpower — it uses a convolution with measured impulse responses from human subjects.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic#creating-a-spatial-3d-sound
         */
        switchPanningModelToHRTF(): void;
        /**
         * Switch the panning model to Equal Power:
         * Represents the equal-power panning algorithm, generally regarded as simple and efficient. equalpower is the default value.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic#creating-a-spatial-3d-sound
         */
        switchPanningModelToEqualPower(): void;
        /**
         * Connect the sound track to an audio analyser allowing some amazing
         * synchronization between the sounds/music and your visualization (VuMeter for instance).
         * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic#using-the-analyser
         * @param analyser The analyser to connect to the engine
         */
        connectToAnalyser(analyser: Analyser): void;
    }


    /**
     * Wraps one or more Sound objects and selects one with random weight for playback.
     */
    export class WeightedSound {
        /** When true a Sound will be selected and played when the current playing Sound completes. */
        loop: boolean;
        private _coneInnerAngle;
        private _coneOuterAngle;
        private _volume;
        /** A Sound is currently playing. */
        isPlaying: boolean;
        /** A Sound is currently paused. */
        isPaused: boolean;
        private _sounds;
        private _weights;
        private _currentIndex?;
        /**
         * Creates a new WeightedSound from the list of sounds given.
         * @param loop When true a Sound will be selected and played when the current playing Sound completes.
         * @param sounds Array of Sounds that will be selected from.
         * @param weights Array of number values for selection weights; length must equal sounds, values will be normalized to 1
         */
        constructor(loop: boolean, sounds: Sound[], weights: number[]);
        /**
         * The size of cone in degrees for a directional sound in which there will be no attenuation.
         */
        get directionalConeInnerAngle(): number;
        /**
         * The size of cone in degrees for a directional sound in which there will be no attenuation.
         */
        set directionalConeInnerAngle(value: number);
        /**
         * Size of cone in degrees for a directional sound outside of which there will be no sound.
         * Listener angles between innerAngle and outerAngle will falloff linearly.
         */
        get directionalConeOuterAngle(): number;
        /**
         * Size of cone in degrees for a directional sound outside of which there will be no sound.
         * Listener angles between innerAngle and outerAngle will falloff linearly.
         */
        set directionalConeOuterAngle(value: number);
        /**
         * Playback volume.
         */
        get volume(): number;
        /**
         * Playback volume.
         */
        set volume(value: number);
        private _onended;
        /**
         * Suspend playback
         */
        pause(): void;
        /**
         * Stop playback
         */
        stop(): void;
        /**
         * Start playback.
         * @param startOffset Position the clip head at a specific time in seconds.
         */
        play(startOffset?: number): void;
    }


    /**
     * Interface for baked vertex animation texture, see BakedVertexAnimationManager
     * @since 5.0
     */
    export interface IBakedVertexAnimationManager {
        /**
         * The vertex animation texture
         */
        texture: Nullable<BaseTexture>;
        /**
         * Gets or sets a boolean indicating if the edgesRenderer is active
         */
        isEnabled: boolean;
        /**
         * The animation parameters for the mesh. See setAnimationParameters()
         */
        animationParameters: Vector4;
        /**
         * The time counter, to pick the correct animation frame.
         */
        time: number;
        /**
         * Binds to the effect.
         * @param effect The effect to bind to.
         * @param useInstances True when it's an instance.
         */
        bind(effect: Effect, useInstances: boolean): void;
        /**
         * Sets animation parameters.
         * @param startFrame The first frame of the animation.
         * @param endFrame The last frame of the animation.
         * @param offset The offset when starting the animation.
         * @param speedFramesPerSecond The frame rate.
         */
        setAnimationParameters(startFrame: number, endFrame: number, offset: number, speedFramesPerSecond: number): void;
        /**
         * Disposes the resources of the manager.
         * @param forceDisposeTextures - Forces the disposal of all textures.
         */
        dispose(forceDisposeTextures?: boolean): void;
        /**
         * Get the current class name useful for serialization or dynamic coding.
         * @returns "BakedVertexAnimationManager"
         */
        getClassName(): string;
    }
    /**
     * This class is used to animate meshes using a baked vertex animation texture
     * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/baked_texture_animations
     * @since 5.0
     */
    export class BakedVertexAnimationManager implements IBakedVertexAnimationManager {
        private _scene;
        private _texture;
        /**
         * The vertex animation texture
         */
        texture: Nullable<BaseTexture>;
        private _isEnabled;
        /**
         * Enable or disable the vertex animation manager
         */
        isEnabled: boolean;
        /**
         * The animation parameters for the mesh. See setAnimationParameters()
         */
        animationParameters: Vector4;
        /**
         * The time counter, to pick the correct animation frame.
         */
        time: number;
        /**
         * Creates a new BakedVertexAnimationManager
         * @param scene defines the current scene
         */
        constructor(scene?: Nullable<Scene>);
        /** @internal */
        _markSubMeshesAsAttributesDirty(): void;
        /**
         * Binds to the effect.
         * @param effect The effect to bind to.
         * @param useInstances True when it's an instance.
         */
        bind(effect: Effect, useInstances?: boolean): void;
        /**
         * Clone the current manager
         * @returns a new BakedVertexAnimationManager
         */
        clone(): BakedVertexAnimationManager;
        /**
         * Sets animation parameters.
         * @param startFrame The first frame of the animation.
         * @param endFrame The last frame of the animation.
         * @param offset The offset when starting the animation.
         * @param speedFramesPerSecond The frame rate.
         */
        setAnimationParameters(startFrame: number, endFrame: number, offset?: number, speedFramesPerSecond?: number): void;
        /**
         * Disposes the resources of the manager.
         * @param forceDisposeTextures - Forces the disposal of all textures.
         */
        dispose(forceDisposeTextures?: boolean): void;
        /**
         * Get the current class name useful for serialization or dynamic coding.
         * @returns "BakedVertexAnimationManager"
         */
        getClassName(): string;
        /**
         * Makes a duplicate of the current instance into another one.
         * @param vatMap define the instance where to copy the info
         */
        copyTo(vatMap: BakedVertexAnimationManager): void;
        /**
         * Serializes this vertex animation instance
         * @returns - An object with the serialized instance.
         */
        serialize(): any;
        /**
         * Parses a vertex animation setting from a serialized object.
         * @param source - Serialized object.
         * @param scene Defines the scene we are parsing for
         * @param rootUrl Defines the rootUrl to load from
         */
        parse(source: any, scene: Scene, rootUrl: string): void;
    }




    /**
     * Class to bake vertex animation textures.
     * @since 5.0
     */
    export class VertexAnimationBaker {
        private _scene;
        private _mesh;
        /**
         * Create a new VertexAnimationBaker object which can help baking animations into a texture.
         * @param scene Defines the scene the VAT belongs to
         * @param mesh Defines the mesh the VAT belongs to
         */
        constructor(scene: Scene, mesh: Mesh);
        /**
         * Bakes the animation into the texture. This should be called once, when the
         * scene starts, so the VAT is generated and associated to the mesh.
         * @param ranges Defines the ranges in the animation that will be baked.
         * @returns The array of matrix transforms for each vertex (columns) and frame (rows), as a Float32Array.
         */
        bakeVertexData(ranges: AnimationRange[]): Promise<Float32Array>;
        /**
         * Runs an animation frame and stores its vertex data
         *
         * @param vertexData The array to save data to.
         * @param frameIndex Current frame in the skeleton animation to render.
         * @param textureIndex Current index of the texture data.
         */
        private _executeAnimationFrame;
        /**
         * Builds a vertex animation texture given the vertexData in an array.
         * @param vertexData The vertex animation data. You can generate it with bakeVertexData().
         * @returns The vertex animation texture to be used with BakedVertexAnimationManager.
         */
        textureFromBakedVertexData(vertexData: Float32Array): RawTexture;
        /**
         * Serializes our vertexData to an object, with a nice string for the vertexData.
         * @param vertexData The vertex array data.
         * @returns This object serialized to a JS dict.
         */
        serializeBakedVertexDataToObject(vertexData: Float32Array): Record<string, any>;
        /**
         * Loads previously baked data.
         * @param data The object as serialized by serializeBakedVertexDataToObject()
         * @returns The array of matrix transforms for each vertex (columns) and frame (rows), as a Float32Array.
         */
        loadBakedVertexDataFromObject(data: Record<string, any>): Float32Array;
        /**
         * Serializes our vertexData to a JSON string, with a nice string for the vertexData.
         * Should be called right after bakeVertexData().
         * @param vertexData The vertex array data.
         * @returns This object serialized to a safe string.
         */
        serializeBakedVertexDataToJSON(vertexData: Float32Array): string;
        /**
         * Loads previously baked data in string format.
         * @param json The json string as serialized by serializeBakedVertexDataToJSON().
         * @returns The array of matrix transforms for each vertex (columns) and frame (rows), as a Float32Array.
         */
        loadBakedVertexDataFromJSON(json: string): Float32Array;
    }


    /**
     * Interface used to define a behavior
     */
    export interface Behavior<T> {
        /** gets or sets behavior's name */
        name: string;
        /**
         * Function called when the behavior needs to be initialized (after attaching it to a target)
         */
        init(): void;
        /**
         * Called when the behavior is attached to a target
         * @param target defines the target where the behavior is attached to
         */
        attach(target: T): void;
        /**
         * Called when the behavior is detached from its target
         */
        detach(): void;
    }
    /**
     * Interface implemented by classes supporting behaviors
     */
    export interface IBehaviorAware<T> {
        /**
         * Attach a behavior
         * @param behavior defines the behavior to attach
         * @returns the current host
         */
        addBehavior(behavior: Behavior<T>): T;
        /**
         * Remove a behavior from the current object
         * @param behavior defines the behavior to detach
         * @returns the current host
         */
        removeBehavior(behavior: Behavior<T>): T;
        /**
         * Gets a behavior using its name to search
         * @param name defines the name to search
         * @returns the behavior or null if not found
         */
        getBehaviorByName(name: string): Nullable<Behavior<T>>;
    }


    /**
     * The autoRotation behavior (AutoRotationBehavior) is designed to create a smooth rotation of an ArcRotateCamera when there is no user interaction.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/behaviors/cameraBehaviors#autorotation-behavior
     */
    export class AutoRotationBehavior implements Behavior<ArcRotateCamera> {
        /**
         * Gets the name of the behavior.
         */
        get name(): string;
        private _zoomStopsAnimation;
        private _idleRotationSpeed;
        private _idleRotationWaitTime;
        private _idleRotationSpinupTime;
        targetAlpha: Nullable<number>;
        /**
         * Sets the flag that indicates if user zooming should stop animation.
         */
        set zoomStopsAnimation(flag: boolean);
        /**
         * Gets the flag that indicates if user zooming should stop animation.
         */
        get zoomStopsAnimation(): boolean;
        /**
         * Sets the default speed at which the camera rotates around the model.
         */
        set idleRotationSpeed(speed: number);
        /**
         * Gets the default speed at which the camera rotates around the model.
         */
        get idleRotationSpeed(): number;
        /**
         * Sets the time (in milliseconds) to wait after user interaction before the camera starts rotating.
         */
        set idleRotationWaitTime(time: number);
        /**
         * Gets the time (milliseconds) to wait after user interaction before the camera starts rotating.
         */
        get idleRotationWaitTime(): number;
        /**
         * Sets the time (milliseconds) to take to spin up to the full idle rotation speed.
         */
        set idleRotationSpinupTime(time: number);
        /**
         * Gets the time (milliseconds) to take to spin up to the full idle rotation speed.
         */
        get idleRotationSpinupTime(): number;
        /**
         * Gets a value indicating if the camera is currently rotating because of this behavior
         */
        get rotationInProgress(): boolean;
        private _onPrePointerObservableObserver;
        private _onAfterCheckInputsObserver;
        private _attachedCamera;
        private _isPointerDown;
        private _lastFrameTime;
        private _lastInteractionTime;
        private _cameraRotationSpeed;
        /**
         * Initializes the behavior.
         */
        init(): void;
        /**
         * Attaches the behavior to its arc rotate camera.
         * @param camera Defines the camera to attach the behavior to
         */
        attach(camera: ArcRotateCamera): void;
        /**
         * Detaches the behavior from its current arc rotate camera.
         */
        detach(): void;
        /**
         * Force-reset the last interaction time
         * @param customTime an optional time that will be used instead of the current last interaction time. For example `Date.now()`
         */
        resetLastInteractionTime(customTime?: number): void;
        /**
         * Returns true if camera alpha reaches the target alpha
         * @returns true if camera alpha reaches the target alpha
         */
        private _reachTargetAlpha;
        /**
         * Returns true if user is scrolling.
         * @returns true if user is scrolling.
         */
        private _userIsZooming;
        private _lastFrameRadius;
        private _shouldAnimationStopForInteraction;
        /**
         *  Applies any current user interaction to the camera. Takes into account maximum alpha rotation.
         */
        private _applyUserInteraction;
        private _userIsMoving;
    }


    /**
     * Add a bouncing effect to an ArcRotateCamera when reaching a specified minimum and maximum radius
     * @see https://doc.babylonjs.com/features/featuresDeepDive/behaviors/cameraBehaviors#bouncing-behavior
     */
    export class BouncingBehavior implements Behavior<ArcRotateCamera> {
        /**
         * Gets the name of the behavior.
         */
        get name(): string;
        /**
         * The easing function used by animations
         */
        static EasingFunction: BackEase;
        /**
         * The easing mode used by animations
         */
        static EasingMode: number;
        /**
         * The duration of the animation, in milliseconds
         */
        transitionDuration: number;
        /**
         * Length of the distance animated by the transition when lower radius is reached
         */
        lowerRadiusTransitionRange: number;
        /**
         * Length of the distance animated by the transition when upper radius is reached
         */
        upperRadiusTransitionRange: number;
        private _autoTransitionRange;
        /**
         * Gets a value indicating if the lowerRadiusTransitionRange and upperRadiusTransitionRange are defined automatically
         */
        get autoTransitionRange(): boolean;
        /**
         * Sets a value indicating if the lowerRadiusTransitionRange and upperRadiusTransitionRange are defined automatically
         * Transition ranges will be set to 5% of the bounding box diagonal in world space
         */
        set autoTransitionRange(value: boolean);
        private _attachedCamera;
        private _onAfterCheckInputsObserver;
        private _onMeshTargetChangedObserver;
        /**
         * Initializes the behavior.
         */
        init(): void;
        /**
         * Attaches the behavior to its arc rotate camera.
         * @param camera Defines the camera to attach the behavior to
         */
        attach(camera: ArcRotateCamera): void;
        /**
         * Detaches the behavior from its current arc rotate camera.
         */
        detach(): void;
        private _radiusIsAnimating;
        private _radiusBounceTransition;
        private _animatables;
        private _cachedWheelPrecision;
        /**
         * Checks if the camera radius is at the specified limit. Takes into account animation locks.
         * @param radiusLimit The limit to check against.
         * @returns Bool to indicate if at limit.
         */
        private _isRadiusAtLimit;
        /**
         * Applies an animation to the radius of the camera, extending by the radiusDelta.
         * @param radiusDelta The delta by which to animate to. Can be negative.
         */
        private _applyBoundRadiusAnimation;
        /**
         * Removes all animation locks. Allows new animations to be added to any of the camera properties.
         */
        protected _clearAnimationLocks(): void;
        /**
         * Stops and removes all animations that have been applied to the camera
         */
        stopAllAnimations(): void;
    }


    /**
     * The framing behavior (FramingBehavior) is designed to automatically position an ArcRotateCamera when its target is set to a mesh. It is also useful if you want to prevent the camera to go under a virtual horizontal plane.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/behaviors/cameraBehaviors#framing-behavior
     */
    export class FramingBehavior implements Behavior<ArcRotateCamera> {
        /**
         * Gets the name of the behavior.
         */
        get name(): string;
        /**
         * An event triggered when the animation to zoom on target mesh has ended
         */
        onTargetFramingAnimationEndObservable: Observable<void>;
        private _mode;
        private _radiusScale;
        private _positionScale;
        private _defaultElevation;
        private _elevationReturnTime;
        private _elevationReturnWaitTime;
        private _zoomStopsAnimation;
        private _framingTime;
        /**
         * The easing function used by animations
         */
        static EasingFunction: ExponentialEase;
        /**
         * The easing mode used by animations
         */
        static EasingMode: number;
        /**
         * Sets the current mode used by the behavior
         */
        set mode(mode: number);
        /**
         * Gets current mode used by the behavior.
         */
        get mode(): number;
        /**
         * Sets the scale applied to the radius (1 by default)
         */
        set radiusScale(radius: number);
        /**
         * Gets the scale applied to the radius
         */
        get radiusScale(): number;
        /**
         * Sets the scale to apply on Y axis to position camera focus. 0.5 by default which means the center of the bounding box.
         */
        set positionScale(scale: number);
        /**
         * Gets the scale to apply on Y axis to position camera focus. 0.5 by default which means the center of the bounding box.
         */
        get positionScale(): number;
        /**
         * Sets the angle above/below the horizontal plane to return to when the return to default elevation idle
         * behaviour is triggered, in radians.
         */
        set defaultElevation(elevation: number);
        /**
         * Gets the angle above/below the horizontal plane to return to when the return to default elevation idle
         * behaviour is triggered, in radians.
         */
        get defaultElevation(): number;
        /**
         * Sets the time (in milliseconds) taken to return to the default beta position.
         * Negative value indicates camera should not return to default.
         */
        set elevationReturnTime(speed: number);
        /**
         * Gets the time (in milliseconds) taken to return to the default beta position.
         * Negative value indicates camera should not return to default.
         */
        get elevationReturnTime(): number;
        /**
         * Sets the delay (in milliseconds) taken before the camera returns to the default beta position.
         */
        set elevationReturnWaitTime(time: number);
        /**
         * Gets the delay (in milliseconds) taken before the camera returns to the default beta position.
         */
        get elevationReturnWaitTime(): number;
        /**
         * Sets the flag that indicates if user zooming should stop animation.
         */
        set zoomStopsAnimation(flag: boolean);
        /**
         * Gets the flag that indicates if user zooming should stop animation.
         */
        get zoomStopsAnimation(): boolean;
        /**
         * Sets the transition time when framing the mesh, in milliseconds
         */
        set framingTime(time: number);
        /**
         * Gets the transition time when framing the mesh, in milliseconds
         */
        get framingTime(): number;
        /**
         * Define if the behavior should automatically change the configured
         * camera limits and sensibilities.
         */
        autoCorrectCameraLimitsAndSensibility: boolean;
        private _onPrePointerObservableObserver;
        private _onAfterCheckInputsObserver;
        private _onMeshTargetChangedObserver;
        private _attachedCamera;
        private _isPointerDown;
        private _lastInteractionTime;
        /**
         * Initializes the behavior.
         */
        init(): void;
        /**
         * Attaches the behavior to its arc rotate camera.
         * @param camera Defines the camera to attach the behavior to
         */
        attach(camera: ArcRotateCamera): void;
        /**
         * Detaches the behavior from its current arc rotate camera.
         */
        detach(): void;
        private _animatables;
        private _betaIsAnimating;
        private _betaTransition;
        private _radiusTransition;
        private _vectorTransition;
        /**
         * Targets the given mesh and updates zoom level accordingly.
         * @param mesh  The mesh to target.
         * @param focusOnOriginXZ Determines if the camera should focus on 0 in the X and Z axis instead of the mesh
         * @param onAnimationEnd Callback triggered at the end of the framing animation
         */
        zoomOnMesh(mesh: AbstractMesh, focusOnOriginXZ?: boolean, onAnimationEnd?: Nullable<() => void>): void;
        /**
         * Targets the given mesh with its children and updates zoom level accordingly.
         * @param mesh  The mesh to target.
         * @param focusOnOriginXZ Determines if the camera should focus on 0 in the X and Z axis instead of the mesh
         * @param onAnimationEnd Callback triggered at the end of the framing animation
         */
        zoomOnMeshHierarchy(mesh: AbstractMesh, focusOnOriginXZ?: boolean, onAnimationEnd?: Nullable<() => void>): void;
        /**
         * Targets the given meshes with their children and updates zoom level accordingly.
         * @param meshes  The mesh to target.
         * @param focusOnOriginXZ Determines if the camera should focus on 0 in the X and Z axis instead of the mesh
         * @param onAnimationEnd Callback triggered at the end of the framing animation
         */
        zoomOnMeshesHierarchy(meshes: AbstractMesh[], focusOnOriginXZ?: boolean, onAnimationEnd?: Nullable<() => void>): void;
        /**
         * Targets the bounding box info defined by its extends and updates zoom level accordingly.
         * @param minimumWorld Determines the smaller position of the bounding box extend
         * @param maximumWorld Determines the bigger position of the bounding box extend
         * @param focusOnOriginXZ Determines if the camera should focus on 0 in the X and Z axis instead of the mesh
         * @param onAnimationEnd Callback triggered at the end of the framing animation
         */
        zoomOnBoundingInfo(minimumWorld: Vector3, maximumWorld: Vector3, focusOnOriginXZ?: boolean, onAnimationEnd?: Nullable<() => void>): void;
        /**
         * Calculates the lowest radius for the camera based on the bounding box of the mesh.
         * @param minimumWorld
         * @param maximumWorld
         * @returns The minimum distance from the primary mesh's center point at which the camera must be kept in order
         *		 to fully enclose the mesh in the viewing frustum.
         */
        protected _calculateLowerRadiusFromModelBoundingSphere(minimumWorld: Vector3, maximumWorld: Vector3): number;
        /**
         * Keeps the camera above the ground plane. If the user pulls the camera below the ground plane, the camera
         * is automatically returned to its default position (expected to be above ground plane).
         */
        private _maintainCameraAboveGround;
        /**
         * Returns the frustum slope based on the canvas ratio and camera FOV
         * @returns The frustum slope represented as a Vector2 with X and Y slopes
         */
        private _getFrustumSlope;
        /**
         * Removes all animation locks. Allows new animations to be added to any of the arcCamera properties.
         */
        private _clearAnimationLocks;
        /**
         *  Applies any current user interaction to the camera. Takes into account maximum alpha rotation.
         */
        private _applyUserInteraction;
        /**
         * Stops and removes all animations that have been applied to the camera
         */
        stopAllAnimations(): void;
        /**
         * Gets a value indicating if the user is moving the camera
         */
        get isUserIsMoving(): boolean;
        /**
         * The camera can move all the way towards the mesh.
         */
        static IgnoreBoundsSizeMode: number;
        /**
         * The camera is not allowed to zoom closer to the mesh than the point at which the adjusted bounding sphere touches the frustum sides
         */
        static FitFrustumSidesMode: number;
    }






    /**
     * A behavior that when attached to a mesh will will place a specified node on the meshes face pointing towards the camera
     */
    export class AttachToBoxBehavior implements Behavior<Mesh> {
        private _ui;
        /**
         *  The name of the behavior
         */
        name: string;
        /**
         * The distance away from the face of the mesh that the UI should be attached to (default: 0.15)
         */
        distanceAwayFromFace: number;
        /**
         * The distance from the bottom of the face that the UI should be attached to (default: 0.15)
         */
        distanceAwayFromBottomOfFace: number;
        private _faceVectors;
        private _target;
        private _scene;
        private _onRenderObserver;
        private _tmpMatrix;
        private _tmpVector;
        /**
         * Creates the AttachToBoxBehavior, used to attach UI to the closest face of the box to a camera
         * @param _ui The transform node that should be attached to the mesh
         */
        constructor(_ui: TransformNode);
        /**
         *  Initializes the behavior
         */
        init(): void;
        private _closestFace;
        private _zeroVector;
        private _lookAtTmpMatrix;
        private _lookAtToRef;
        /**
         * Attaches the AttachToBoxBehavior to the passed in mesh
         * @param target The mesh that the specified node will be attached to
         */
        attach(target: Mesh): void;
        /**
         *  Detaches the behavior from the mesh
         */
        detach(): void;
    }


    /**
     * Data store to track virtual pointers movement
     */
    type VirtualMeshInfo = {
        dragging: boolean;
        moving: boolean;
        dragMesh: AbstractMesh;
        originMesh: AbstractMesh;
        pivotMesh: AbstractMesh;
        startingPivotPosition: Vector3;
        startingPivotOrientation: Quaternion;
        startingPosition: Vector3;
        startingOrientation: Quaternion;
        lastOriginPosition: Vector3;
        lastDragPosition: Vector3;
    };
    /**
     * Base behavior for six degrees of freedom interactions in XR experiences.
     * Creates virtual meshes that are dragged around
     * And observables for position/rotation changes
     */
    export class BaseSixDofDragBehavior implements Behavior<Mesh> {
        protected static _virtualScene: Scene;
        private _pointerObserver;
        private _attachedToElement;
        protected _virtualMeshesInfo: {
            [id: number]: VirtualMeshInfo;
        };
        private _tmpVector;
        private _tmpQuaternion;
        protected _dragType: {
            NONE: number;
            DRAG: number;
            DRAG_WITH_CONTROLLER: number;
            NEAR_DRAG: number;
        };
        protected _scene: Scene;
        protected _moving: boolean;
        protected _ownerNode: TransformNode;
        protected _dragging: number;
        /**
         * The list of child meshes that can receive drag events
         * If `null`, all child meshes will receive drag event
         */
        draggableMeshes: Nullable<AbstractMesh[]>;
        /**
         * How much faster the object should move when the controller is moving towards it. This is useful to bring objects that are far away from the user to them faster. Set this to 0 to avoid any speed increase. (Default: 3)
         */
        zDragFactor: number;
        /**
         * The id of the pointer that is currently interacting with the behavior (-1 when no pointer is active)
         */
        get currentDraggingPointerId(): number;
        set currentDraggingPointerId(value: number);
        /**
         * In case of multipointer interaction, all pointer ids currently active are stored here
         */
        currentDraggingPointerIds: number[];
        /**
         * Get or set the currentDraggingPointerId
         * @deprecated Please use currentDraggingPointerId instead
         */
        get currentDraggingPointerID(): number;
        set currentDraggingPointerID(currentDraggingPointerID: number);
        /**
        /**
         * If camera controls should be detached during the drag
         */
        detachCameraControls: boolean;
        /**
         * Fires each time a drag starts
         */
        onDragStartObservable: Observable<{
            position: Vector3;
        }>;
        /**
         * Fires each time a drag happens
         */
        onDragObservable: Observable<{
            delta: Vector3;
            position: Vector3;
            pickInfo: PickingInfo;
        }>;
        /**
         *  Fires each time a drag ends (eg. mouse release after drag)
         */
        onDragEndObservable: Observable<{}>;
        /**
         * Should the behavior allow simultaneous pointers to interact with the owner node.
         */
        allowMultiPointer: boolean;
        /**
         *  The name of the behavior
         */
        get name(): string;
        /**
         *  Returns true if the attached mesh is currently moving with this behavior
         */
        get isMoving(): boolean;
        /**
         *  Initializes the behavior
         */
        init(): void;
        /**
         * In the case of multiple active cameras, the cameraToUseForPointers should be used if set instead of active camera
         */
        private get _pointerCamera();
        private _createVirtualMeshInfo;
        protected _resetVirtualMeshesPosition(): void;
        private _pointerUpdate2D;
        private _pointerUpdateXR;
        /**
         * Attaches the scale behavior the passed in mesh
         * @param ownerNode The mesh that will be scaled around once attached
         */
        attach(ownerNode: TransformNode): void;
        private _applyZOffset;
        protected _targetDragStart(worldPosition: Vector3, worldRotation: Quaternion, pointerId: number): void;
        protected _targetDrag(worldDeltaPosition: Vector3, worldDeltaRotation: Quaternion, pointerId: number): void;
        protected _targetDragEnd(pointerId: number): void;
        protected _reattachCameraControls(): void;
        /**
         * Detaches the behavior from the mesh
         */
        detach(): void;
    }


    /**
     * A behavior that when attached to a mesh will allow the mesh to fade in and out
     */
    export class FadeInOutBehavior implements Behavior<Mesh> {
        /**
         * Time in milliseconds to delay before fading in (Default: 0)
         */
        fadeInDelay: number;
        /**
         * Time in milliseconds to delay before fading out (Default: 0)
         */
        fadeOutDelay: number;
        /**
         * Time in milliseconds for the mesh to fade in (Default: 300)
         */
        fadeInTime: number;
        /**
         * Time in milliseconds for the mesh to fade out (Default: 300)
         */
        fadeOutTime: number;
        /**
         * Time in milliseconds to delay before fading in (Default: 0)
         * Will set both fade in and out delay to the same value
         */
        get delay(): number;
        set delay(value: number);
        private _millisecondsPerFrame;
        private _hovered;
        private _hoverValue;
        private _ownerNode;
        private _onBeforeRenderObserver;
        private _delay;
        private _time;
        /**
         * Instantiates the FadeInOutBehavior
         */
        constructor();
        /**
         *  The name of the behavior
         */
        get name(): string;
        /**
         *  Initializes the behavior
         */
        init(): void;
        /**
         * Attaches the fade behavior on the passed in mesh
         * @param ownerNode The mesh that will be faded in/out once attached
         */
        attach(ownerNode: Mesh): void;
        /**
         *  Detaches the behavior from the mesh
         */
        detach(): void;
        /**
         * Triggers the mesh to begin fading in (or out)
         * @param fadeIn if the object should fade in or out (true to fade in)
         */
        fadeIn(fadeIn?: boolean): void;
        /**
         * Triggers the mesh to begin fading out
         */
        fadeOut(): void;
        private _update;
        private _setAllVisibility;
        private _attachObserver;
        private _detachObserver;
    }


    /**
     * A behavior that when attached to a mesh will follow a camera
     * @since 5.0.0
     */
    export class FollowBehavior implements Behavior<TransformNode> {
        private _scene;
        private _tmpQuaternion;
        private _tmpVectors;
        private _tmpMatrix;
        private _tmpInvertView;
        private _tmpForward;
        private _tmpNodeForward;
        private _tmpPosition;
        private _followedCamera;
        private _onBeforeRender;
        private _workingPosition;
        private _workingQuaternion;
        private _lastTick;
        private _recenterNextUpdate;
        /**
         * Attached node of this behavior
         */
        attachedNode: Nullable<TransformNode>;
        /**
         * Set to false if the node should strictly follow the camera without any interpolation time
         */
        interpolatePose: boolean;
        /**
         * Rate of interpolation of position and rotation of the attached node.
         * Higher values will give a slower interpolation.
         */
        lerpTime: number;
        /**
         * If the behavior should ignore the pitch and roll of the camera.
         */
        ignoreCameraPitchAndRoll: boolean;
        /**
         * Pitch offset from camera (relative to Max Distance)
         * Is only effective if `ignoreCameraPitchAndRoll` is set to `true`.
         */
        pitchOffset: number;
        /**
         * The vertical angle from the camera forward axis to the owner will not exceed this value
         */
        maxViewVerticalDegrees: number;
        /**
         * The horizontal angle from the camera forward axis to the owner will not exceed this value
         */
        maxViewHorizontalDegrees: number;
        /**
         * The attached node will not reorient until the angle between its forward vector and the vector to the camera is greater than this value
         */
        orientToCameraDeadzoneDegrees: number;
        /**
         * Option to ignore distance clamping
         */
        ignoreDistanceClamp: boolean;
        /**
         * Option to ignore angle clamping
         */
        ignoreAngleClamp: boolean;
        /**
         * Max vertical distance between the attachedNode and camera
         */
        verticalMaxDistance: number;
        /**
         *  Default distance from eye to attached node, i.e. the sphere radius
         */
        defaultDistance: number;
        /**
         *  Max distance from eye to attached node, i.e. the sphere radius
         */
        maximumDistance: number;
        /**
         *  Min distance from eye to attached node, i.e. the sphere radius
         */
        minimumDistance: number;
        /**
         * Ignore vertical movement and lock the Y position of the object.
         */
        useFixedVerticalOffset: boolean;
        /**
         * Fixed vertical position offset distance.
         */
        fixedVerticalOffset: number;
        /**
         * Enables/disables the behavior
         * @internal
         */
        _enabled: boolean;
        /**
         * The camera that should be followed by this behavior
         */
        get followedCamera(): Nullable<Camera>;
        set followedCamera(camera: Nullable<Camera>);
        /**
         *  The name of the behavior
         */
        get name(): string;
        /**
         *  Initializes the behavior
         */
        init(): void;
        /**
         * Attaches the follow behavior
         * @param ownerNode The mesh that will be following once attached
         * @param followedCamera The camera that should be followed by the node
         */
        attach(ownerNode: TransformNode, followedCamera?: Camera): void;
        /**
         *  Detaches the behavior from the mesh
         */
        detach(): void;
        /**
         * Recenters the attached node in front of the camera on the next update
         */
        recenter(): void;
        private _angleBetweenVectorAndPlane;
        private _length2D;
        private _distanceClamp;
        private _applyVerticalClamp;
        private _toOrientationQuatToRef;
        private _applyPitchOffset;
        private _angularClamp;
        private _orientationClamp;
        private _passedOrientationDeadzone;
        private _updateLeashing;
        private _updateTransformToGoal;
        private _addObservables;
        private _removeObservables;
    }


    /**
     * Zones around the hand
     */
    export enum HandConstraintZone {
        /**
         * Above finger tips
         */
        ABOVE_FINGER_TIPS = 0,
        /**
         * Next to the thumb
         */
        RADIAL_SIDE = 1,
        /**
         * Next to the pinky finger
         */
        ULNAR_SIDE = 2,
        /**
         * Below the wrist
         */
        BELOW_WRIST = 3
    }
    /**
     * Orientations for the hand zones and for the attached node
     */
    export enum HandConstraintOrientation {
        /**
         * Orientation is towards the camera
         */
        LOOK_AT_CAMERA = 0,
        /**
         * Orientation is determined by the rotation of the palm
         */
        HAND_ROTATION = 1
    }
    /**
     * Orientations for the hand zones and for the attached node
     */
    export enum HandConstraintVisibility {
        /**
         * Constraint is always visible
         */
        ALWAYS_VISIBLE = 0,
        /**
         * Constraint is only visible when the palm is up
         */
        PALM_UP = 1,
        /**
         * Constraint is only visible when the user is looking at the constraint.
         * Uses XR Eye Tracking if enabled/available, otherwise uses camera direction
         */
        GAZE_FOCUS = 2,
        /**
         * Constraint is only visible when the palm is up and the user is looking at it
         */
        PALM_AND_GAZE = 3
    }
    /**
     * Hand constraint behavior that makes the attached `TransformNode` follow hands in XR experiences.
     * @since 5.0.0
     */
    export class HandConstraintBehavior implements Behavior<TransformNode> {
        private _scene;
        private _node;
        private _eyeTracking;
        private _handTracking;
        private _sceneRenderObserver;
        private _zoneAxis;
        /**
         * Sets the HandConstraintVisibility level for the hand constraint
         */
        handConstraintVisibility: HandConstraintVisibility;
        /**
         * A number from 0.0 to 1.0, marking how restricted the direction the palm faces is for the attached node to be enabled.
         * A 1 means the palm must be directly facing the user before the node is enabled, a 0 means it is always enabled.
         * Used with HandConstraintVisibility.PALM_UP
         */
        palmUpStrictness: number;
        /**
         * The radius in meters around the center of the hand that the user must gaze inside for the attached node to be enabled and appear.
         * Used with HandConstraintVisibility.GAZE_FOCUS
         */
        gazeProximityRadius: number;
        /**
         * Offset distance from the hand in meters
         */
        targetOffset: number;
        /**
         * Where to place the node regarding the center of the hand.
         */
        targetZone: HandConstraintZone;
        /**
         * Orientation mode of the 4 zones around the hand
         */
        zoneOrientationMode: HandConstraintOrientation;
        /**
         * Orientation mode of the node attached to this behavior
         */
        nodeOrientationMode: HandConstraintOrientation;
        /**
         * Set the hand this behavior should follow. If set to "none", it will follow any visible hand (prioritising the left one).
         */
        handedness: XRHandedness;
        /**
         * Rate of interpolation of position and rotation of the attached node.
         * Higher values will give a slower interpolation.
         */
        lerpTime: number;
        /**
         * Builds a hand constraint behavior
         */
        constructor();
        /** gets or sets behavior's name */
        get name(): string;
        /** Enable the behavior */
        enable(): void;
        /** Disable the behavior */
        disable(): void;
        private _getHandPose;
        /**
         * Initializes the hand constraint behavior
         */
        init(): void;
        /**
         * Attaches the hand constraint to a `TransformNode`
         * @param node defines the node to attach the behavior to
         */
        attach(node: TransformNode): void;
        private _setVisibility;
        /**
         * Detaches the behavior from the `TransformNode`
         */
        detach(): void;
        /**
         * Links the behavior to the XR experience in which to retrieve hand transform information.
         * @param xr xr experience
         */
        linkToXRExperience(xr: WebXRExperienceHelper | WebXRFeaturesManager): void;
    }




    /**
     * A behavior that when attached to a mesh will allow the mesh to be scaled
     */
    export class MultiPointerScaleBehavior implements Behavior<Mesh> {
        private _dragBehaviorA;
        private _dragBehaviorB;
        private _startDistance;
        private _initialScale;
        private _targetScale;
        private _ownerNode;
        private _sceneRenderObserver;
        /**
         * Instantiate a new behavior that when attached to a mesh will allow the mesh to be scaled
         */
        constructor();
        /**
         *  The name of the behavior
         */
        get name(): string;
        /**
         *  Initializes the behavior
         */
        init(): void;
        private _getCurrentDistance;
        /**
         * Attaches the scale behavior the passed in mesh
         * @param ownerNode The mesh that will be scaled around once attached
         */
        attach(ownerNode: Mesh): void;
        /**
         *  Detaches the behavior from the mesh
         */
        detach(): void;
    }


    /**
     * A behavior that when attached to a mesh will allow the mesh to be dragged around the screen based on pointer events
     */
    export class PointerDragBehavior implements Behavior<AbstractMesh> {
        private static _AnyMouseId;
        /**
         * Abstract mesh the behavior is set on
         */
        attachedNode: AbstractMesh;
        protected _dragPlane: Mesh;
        private _scene;
        private _pointerObserver;
        private _beforeRenderObserver;
        private static _PlaneScene;
        private _useAlternatePickedPointAboveMaxDragAngleDragSpeed;
        private _activeDragButton;
        private _activePointerInfo;
        /**
         * The maximum tolerated angle between the drag plane and dragging pointer rays to trigger pointer events. Set to 0 to allow any angle (default: 0)
         */
        maxDragAngle: number;
        /**
         * Butttons that can be used to initiate a drag
         */
        dragButtons: number[];
        /**
         * @internal
         */
        _useAlternatePickedPointAboveMaxDragAngle: boolean;
        /**
         * Get or set the currentDraggingPointerId
         * @deprecated Please use currentDraggingPointerId instead
         */
        get currentDraggingPointerID(): number;
        set currentDraggingPointerID(currentDraggingPointerID: number);
        /**
         * The id of the pointer that is currently interacting with the behavior (-1 when no pointer is active)
         */
        currentDraggingPointerId: number;
        /**
         * The last position where the pointer hit the drag plane in world space
         */
        lastDragPosition: Vector3;
        /**
         * If the behavior is currently in a dragging state
         */
        dragging: boolean;
        /**
         * The distance towards the target drag position to move each frame. This can be useful to avoid jitter. Set this to 1 for no delay. (Default: 0.2)
         */
        dragDeltaRatio: number;
        /**
         * If the drag plane orientation should be updated during the dragging (Default: true)
         */
        updateDragPlane: boolean;
        private _debugMode;
        private _moving;
        /**
         *  Fires each time the attached mesh is dragged with the pointer
         *  * delta between last drag position and current drag position in world space
         *  * dragDistance along the drag axis
         *  * dragPlaneNormal normal of the current drag plane used during the drag
         *  * dragPlanePoint in world space where the drag intersects the drag plane
         *
         *  (if validatedDrag is used, the position of the attached mesh might not equal dragPlanePoint)
         */
        onDragObservable: Observable<{
            delta: Vector3;
            dragPlanePoint: Vector3;
            dragPlaneNormal: Vector3;
            dragDistance: number;
            pointerId: number;
            pointerInfo: Nullable<PointerInfo>;
        }>;
        /**
         *  Fires each time a drag begins (eg. mouse down on mesh)
         *  * dragPlanePoint in world space where the drag intersects the drag plane
         *
         *  (if validatedDrag is used, the position of the attached mesh might not equal dragPlanePoint)
         */
        onDragStartObservable: Observable<{
            dragPlanePoint: Vector3;
            pointerId: number;
            pointerInfo: Nullable<PointerInfo>;
        }>;
        /**
         *  Fires each time a drag ends (eg. mouse release after drag)
         *  * dragPlanePoint in world space where the drag intersects the drag plane
         *
         *  (if validatedDrag is used, the position of the attached mesh might not equal dragPlanePoint)
         */
        onDragEndObservable: Observable<{
            dragPlanePoint: Vector3;
            pointerId: number;
            pointerInfo: Nullable<PointerInfo>;
        }>;
        /**
         *  Fires each time behavior enabled state changes
         */
        onEnabledObservable: Observable<boolean>;
        /**
         *  If the attached mesh should be moved when dragged
         */
        moveAttached: boolean;
        /**
         *  If the drag behavior will react to drag events (Default: true)
         */
        set enabled(value: boolean);
        get enabled(): boolean;
        private _enabled;
        /**
         * If pointer events should start and release the drag (Default: true)
         */
        startAndReleaseDragOnPointerEvents: boolean;
        /**
         * If camera controls should be detached during the drag
         */
        detachCameraControls: boolean;
        /**
         * If set, the drag plane/axis will be rotated based on the attached mesh's world rotation (Default: true)
         */
        useObjectOrientationForDragging: boolean;
        private _options;
        /**
         * Gets the options used by the behavior
         */
        get options(): {
            dragAxis?: Vector3;
            dragPlaneNormal?: Vector3;
        };
        /**
         * Sets the options used by the behavior
         */
        set options(options: {
            dragAxis?: Vector3;
            dragPlaneNormal?: Vector3;
        });
        /**
         * Creates a pointer drag behavior that can be attached to a mesh
         * @param options The drag axis or normal of the plane that will be dragged across. If no options are specified the drag plane will always face the ray's origin (eg. camera)
         * @param options.dragAxis
         * @param options.dragPlaneNormal
         */
        constructor(options?: {
            dragAxis?: Vector3;
            dragPlaneNormal?: Vector3;
        });
        /**
         * Predicate to determine if it is valid to move the object to a new position when it is moved
         * @param targetPosition
         */
        validateDrag: (targetPosition: Vector3) => boolean;
        /**
         *  The name of the behavior
         */
        get name(): string;
        /**
         *  Initializes the behavior
         */
        init(): void;
        private _tmpVector;
        private _alternatePickedPoint;
        private _worldDragAxis;
        private _targetPosition;
        private _attachedToElement;
        /**
         * Attaches the drag behavior the passed in mesh
         * @param ownerNode The mesh that will be dragged around once attached
         * @param predicate Predicate to use for pick filtering
         */
        attach(ownerNode: AbstractMesh, predicate?: (m: AbstractMesh) => boolean): void;
        /**
         * Force release the drag action by code.
         */
        releaseDrag(): void;
        private _startDragRay;
        private _lastPointerRay;
        /**
         * Simulates the start of a pointer drag event on the behavior
         * @param pointerId pointerID of the pointer that should be simulated (Default: Any mouse pointer ID)
         * @param fromRay initial ray of the pointer to be simulated (Default: Ray from camera to attached mesh)
         * @param startPickedPoint picked point of the pointer to be simulated (Default: attached mesh position)
         */
        startDrag(pointerId?: number, fromRay?: Ray, startPickedPoint?: Vector3): void;
        protected _startDrag(pointerId: number, fromRay?: Ray, startPickedPoint?: Vector3): void;
        private _dragDelta;
        protected _moveDrag(ray: Ray): void;
        private _pickWithRayOnDragPlane;
        private _pointA;
        private _pointC;
        private _localAxis;
        private _lookAt;
        private _updateDragPlanePosition;
        /**
         *  Detaches the behavior from the mesh
         */
        detach(): void;
    }


    /**
     * A behavior that when attached to a mesh will allow the mesh to be dragged around based on directions and origin of the pointer's ray
     */
    export class SixDofDragBehavior extends BaseSixDofDragBehavior {
        private _sceneRenderObserver;
        private _virtualTransformNode;
        protected _targetPosition: Vector3;
        protected _targetOrientation: Quaternion;
        protected _targetScaling: Vector3;
        protected _startingPosition: Vector3;
        protected _startingOrientation: Quaternion;
        protected _startingScaling: Vector3;
        /**
         * Fires when position is updated
         */
        onPositionChangedObservable: Observable<{
            position: Vector3;
        }>;
        /**
         * The distance towards the target drag position to move each frame. This can be useful to avoid jitter. Set this to 1 for no delay. (Default: 0.2)
         */
        dragDeltaRatio: number;
        /**
         * If the object should rotate to face the drag origin
         */
        rotateDraggedObject: boolean;
        /**
         * If `rotateDraggedObject` is set to `true`, this parameter determines if we are only rotating around the y axis (yaw)
         */
        rotateAroundYOnly: boolean;
        /**
         * Should the behavior rotate 1:1 with the motion controller, when one is used.
         */
        rotateWithMotionController: boolean;
        /**
         *  The name of the behavior
         */
        get name(): string;
        /**
         * Use this flag to update the target but not move the owner node towards the target
         */
        disableMovement: boolean;
        /**
         * Should the object rotate towards the camera when we start dragging it
         */
        faceCameraOnDragStart: boolean;
        /**
         * Attaches the six DoF drag behavior
         * @param ownerNode The mesh that will be dragged around once attached
         */
        attach(ownerNode: Mesh): void;
        private _getPositionOffsetAround;
        private _onePointerPositionUpdated;
        private _twoPointersPositionUpdated;
        protected _targetDragStart(): void;
        protected _targetDrag(worldDeltaPosition: Vector3, worldDeltaRotation: Quaternion): void;
        protected _targetDragEnd(): void;
        /**
         *  Detaches the behavior from the mesh
         */
        detach(): void;
    }


    /**
     * A behavior that allows a transform node to stick to a surface position/orientation
     * @since 5.0.0
     */
    export class SurfaceMagnetismBehavior implements Behavior<Mesh> {
        private _scene;
        private _attachedMesh;
        private _attachPointLocalOffset;
        private _pointerObserver;
        private _workingPosition;
        private _workingQuaternion;
        private _lastTick;
        private _onBeforeRender;
        private _hit;
        /**
         * Distance offset from the hit point to place the target at, along the hit normal.
         */
        hitNormalOffset: number;
        /**
         * Name of the behavior
         */
        get name(): string;
        /**
         * Spatial mapping meshes to collide with
         */
        meshes: AbstractMesh[];
        /**
         * Function called when the behavior needs to be initialized (after attaching it to a target)
         */
        init(): void;
        /**
         * Set to false if the node should strictly follow the camera without any interpolation time
         */
        interpolatePose: boolean;
        /**
         * Rate of interpolation of position and rotation of the attached node.
         * Higher values will give a slower interpolation.
         */
        lerpTime: number;
        /**
         * If true, pitch and roll are omitted.
         */
        keepOrientationVertical: boolean;
        /**
         * Is this behavior reacting to pointer events
         */
        enabled: boolean;
        /**
         * Maximum distance for the node to stick to the surface
         */
        maxStickingDistance: number;
        /**
         * Attaches the behavior to a transform node
         * @param target defines the target where the behavior is attached to
         * @param scene the scene
         */
        attach(target: Mesh, scene?: Scene): void;
        /**
         * Detaches the behavior
         */
        detach(): void;
        private _getTargetPose;
        /**
         * Updates the attach point with the current geometry extents of the attached mesh
         */
        updateAttachPoint(): void;
        /**
         * Finds the intersection point of the given ray onto the meshes and updates the target.
         * Transformation will be interpolated according to `interpolatePose` and `lerpTime` properties.
         * If no mesh of `meshes` are hit, this does nothing.
         * @param pickInfo The input pickingInfo that will be used to intersect the meshes
         * @returns a boolean indicating if we found a hit to stick to
         */
        findAndUpdateTarget(pickInfo: PickingInfo): boolean;
        private _getAttachPointOffsetToRef;
        private _updateTransformToGoal;
        private _addObservables;
        private _removeObservables;
    }


    /**
     * Class used to store bone information
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/bonesSkeletons
     */
    export class Bone extends Node {
        /**
         * defines the bone name
         */
        name: string;
        private static _TmpVecs;
        private static _TmpQuat;
        private static _TmpMats;
        /**
         * Gets the list of child bones
         */
        children: Bone[];
        /** Gets the animations associated with this bone */
        animations: Animation[];
        /**
         * Gets or sets bone length
         */
        length: number;
        /**
         * @internal Internal only
         * Set this value to map this bone to a different index in the transform matrices
         * Set this value to -1 to exclude the bone from the transform matrices
         */
        _index: Nullable<number>;
        private _skeleton;
        private _localMatrix;
        private _restPose;
        private _baseMatrix;
        private _absoluteTransform;
        private _invertedAbsoluteTransform;
        private _scalingDeterminant;
        private _worldTransform;
        private _localScaling;
        private _localRotation;
        private _localPosition;
        private _needToDecompose;
        private _needToCompose;
        /** @internal */
        _linkedTransformNode: Nullable<TransformNode>;
        /** @internal */
        _waitingTransformNodeId: Nullable<string>;
        /** @internal */
        get _matrix(): Matrix;
        /** @internal */
        set _matrix(value: Matrix);
        /**
         * Create a new bone
         * @param name defines the bone name
         * @param skeleton defines the parent skeleton
         * @param parentBone defines the parent (can be null if the bone is the root)
         * @param localMatrix defines the local matrix
         * @param restPose defines the rest pose matrix
         * @param baseMatrix defines the base matrix
         * @param index defines index of the bone in the hierarchy
         */
        constructor(
        /**
         * defines the bone name
         */
        name: string, skeleton: Skeleton, parentBone?: Nullable<Bone>, localMatrix?: Nullable<Matrix>, restPose?: Nullable<Matrix>, baseMatrix?: Nullable<Matrix>, index?: Nullable<number>);
        /**
         * Gets the current object class name.
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the parent skeleton
         * @returns a skeleton
         */
        getSkeleton(): Skeleton;
        get parent(): Bone;
        /**
         * Gets parent bone
         * @returns a bone or null if the bone is the root of the bone hierarchy
         */
        getParent(): Nullable<Bone>;
        /**
         * Returns an array containing the root bones
         * @returns an array containing the root bones
         */
        getChildren(): Array<Bone>;
        /**
         * Gets the node index in matrix array generated for rendering
         * @returns the node index
         */
        getIndex(): number;
        set parent(newParent: Nullable<Bone>);
        /**
         * Sets the parent bone
         * @param parent defines the parent (can be null if the bone is the root)
         * @param updateDifferenceMatrix defines if the difference matrix must be updated
         */
        setParent(parent: Nullable<Bone>, updateDifferenceMatrix?: boolean): void;
        /**
         * Gets the local matrix
         * @returns a matrix
         */
        getLocalMatrix(): Matrix;
        /**
         * Gets the base matrix (initial matrix which remains unchanged)
         * @returns the base matrix (as known as bind pose matrix)
         */
        getBaseMatrix(): Matrix;
        /**
         * Gets the rest pose matrix
         * @returns a matrix
         */
        getRestPose(): Matrix;
        /**
         * Sets the rest pose matrix
         * @param matrix the local-space rest pose to set for this bone
         */
        setRestPose(matrix: Matrix): void;
        /**
         * Gets the bind pose matrix
         * @returns the bind pose matrix
         * @deprecated Please use getBaseMatrix instead
         */
        getBindPose(): Matrix;
        /**
         * Sets the bind pose matrix
         * @param matrix the local-space bind pose to set for this bone
         * @deprecated Please use updateMatrix instead
         */
        setBindPose(matrix: Matrix): void;
        /**
         * Gets a matrix used to store world matrix (ie. the matrix sent to shaders)
         */
        getWorldMatrix(): Matrix;
        /**
         * Sets the local matrix to rest pose matrix
         */
        returnToRest(): void;
        /**
         * Gets the inverse of the absolute transform matrix.
         * This matrix will be multiplied by local matrix to get the difference matrix (ie. the difference between original state and current state)
         * @returns a matrix
         */
        getInvertedAbsoluteTransform(): Matrix;
        /**
         * Gets the absolute transform matrix (ie base matrix * parent world matrix)
         * @returns a matrix
         */
        getAbsoluteTransform(): Matrix;
        /**
         * Links with the given transform node.
         * The local matrix of this bone is copied from the transform node every frame.
         * @param transformNode defines the transform node to link to
         */
        linkTransformNode(transformNode: Nullable<TransformNode>): void;
        /**
         * Gets the node used to drive the bone's transformation
         * @returns a transform node or null
         */
        getTransformNode(): Nullable<TransformNode>;
        /** Gets or sets current position (in local space) */
        get position(): Vector3;
        set position(newPosition: Vector3);
        /** Gets or sets current rotation (in local space) */
        get rotation(): Vector3;
        set rotation(newRotation: Vector3);
        /** Gets or sets current rotation quaternion (in local space) */
        get rotationQuaternion(): Quaternion;
        set rotationQuaternion(newRotation: Quaternion);
        /** Gets or sets current scaling (in local space) */
        get scaling(): Vector3;
        set scaling(newScaling: Vector3);
        /**
         * Gets the animation properties override
         */
        get animationPropertiesOverride(): Nullable<AnimationPropertiesOverride>;
        private _decompose;
        private _compose;
        /**
         * Update the base and local matrices
         * @param matrix defines the new base or local matrix
         * @param updateDifferenceMatrix defines if the difference matrix must be updated
         * @param updateLocalMatrix defines if the local matrix should be updated
         */
        updateMatrix(matrix: Matrix, updateDifferenceMatrix?: boolean, updateLocalMatrix?: boolean): void;
        /**
         * @internal
         */
        _updateDifferenceMatrix(rootMatrix?: Matrix, updateChildren?: boolean): void;
        /**
         * Flag the bone as dirty (Forcing it to update everything)
         * @returns this bone
         */
        markAsDirty(): Bone;
        /** @internal */
        _markAsDirtyAndCompose(): void;
        private _markAsDirtyAndDecompose;
        /**
         * Translate the bone in local or world space
         * @param vec The amount to translate the bone
         * @param space The space that the translation is in
         * @param tNode The TransformNode that this bone is attached to. This is only used in world space
         */
        translate(vec: Vector3, space?: Space, tNode?: TransformNode): void;
        /**
         * Set the position of the bone in local or world space
         * @param position The position to set the bone
         * @param space The space that the position is in
         * @param tNode The TransformNode that this bone is attached to.  This is only used in world space
         */
        setPosition(position: Vector3, space?: Space, tNode?: TransformNode): void;
        /**
         * Set the absolute position of the bone (world space)
         * @param position The position to set the bone
         * @param tNode The TransformNode that this bone is attached to
         */
        setAbsolutePosition(position: Vector3, tNode?: TransformNode): void;
        /**
         * Scale the bone on the x, y and z axes (in local space)
         * @param x The amount to scale the bone on the x axis
         * @param y The amount to scale the bone on the y axis
         * @param z The amount to scale the bone on the z axis
         * @param scaleChildren sets this to true if children of the bone should be scaled as well (false by default)
         */
        scale(x: number, y: number, z: number, scaleChildren?: boolean): void;
        /**
         * Set the bone scaling in local space
         * @param scale defines the scaling vector
         */
        setScale(scale: Vector3): void;
        /**
         * Gets the current scaling in local space
         * @returns the current scaling vector
         */
        getScale(): Vector3;
        /**
         * Gets the current scaling in local space and stores it in a target vector
         * @param result defines the target vector
         */
        getScaleToRef(result: Vector3): void;
        /**
         * Set the yaw, pitch, and roll of the bone in local or world space
         * @param yaw The rotation of the bone on the y axis
         * @param pitch The rotation of the bone on the x axis
         * @param roll The rotation of the bone on the z axis
         * @param space The space that the axes of rotation are in
         * @param tNode The TransformNode that this bone is attached to.  This is only used in world space
         */
        setYawPitchRoll(yaw: number, pitch: number, roll: number, space?: Space, tNode?: TransformNode): void;
        /**
         * Add a rotation to the bone on an axis in local or world space
         * @param axis The axis to rotate the bone on
         * @param amount The amount to rotate the bone
         * @param space The space that the axis is in
         * @param tNode The TransformNode that this bone is attached to. This is only used in world space
         */
        rotate(axis: Vector3, amount: number, space?: Space, tNode?: TransformNode): void;
        /**
         * Set the rotation of the bone to a particular axis angle in local or world space
         * @param axis The axis to rotate the bone on
         * @param angle The angle that the bone should be rotated to
         * @param space The space that the axis is in
         * @param tNode The TransformNode that this bone is attached to.  This is only used in world space
         */
        setAxisAngle(axis: Vector3, angle: number, space?: Space, tNode?: TransformNode): void;
        /**
         * Set the euler rotation of the bone in local or world space
         * @param rotation The euler rotation that the bone should be set to
         * @param space The space that the rotation is in
         * @param tNode The TransformNode that this bone is attached to. This is only used in world space
         */
        setRotation(rotation: Vector3, space?: Space, tNode?: TransformNode): void;
        /**
         * Set the quaternion rotation of the bone in local or world space
         * @param quat The quaternion rotation that the bone should be set to
         * @param space The space that the rotation is in
         * @param tNode The TransformNode that this bone is attached to. This is only used in world space
         */
        setRotationQuaternion(quat: Quaternion, space?: Space, tNode?: TransformNode): void;
        /**
         * Set the rotation matrix of the bone in local or world space
         * @param rotMat The rotation matrix that the bone should be set to
         * @param space The space that the rotation is in
         * @param tNode The TransformNode that this bone is attached to. This is only used in world space
         */
        setRotationMatrix(rotMat: Matrix, space?: Space, tNode?: TransformNode): void;
        private _rotateWithMatrix;
        private _getNegativeRotationToRef;
        /**
         * Get the position of the bone in local or world space
         * @param space The space that the returned position is in
         * @param tNode The TransformNode that this bone is attached to. This is only used in world space
         * @returns The position of the bone
         */
        getPosition(space?: Space, tNode?: Nullable<TransformNode>): Vector3;
        /**
         * Copy the position of the bone to a vector3 in local or world space
         * @param space The space that the returned position is in
         * @param tNode The TransformNode that this bone is attached to. This is only used in world space
         * @param result The vector3 to copy the position to
         */
        getPositionToRef(space: Space | undefined, tNode: Nullable<TransformNode>, result: Vector3): void;
        /**
         * Get the absolute position of the bone (world space)
         * @param tNode The TransformNode that this bone is attached to
         * @returns The absolute position of the bone
         */
        getAbsolutePosition(tNode?: Nullable<TransformNode>): Vector3;
        /**
         * Copy the absolute position of the bone (world space) to the result param
         * @param tNode The TransformNode that this bone is attached to
         * @param result The vector3 to copy the absolute position to
         */
        getAbsolutePositionToRef(tNode: TransformNode, result: Vector3): void;
        /**
         * Compute the absolute transforms of this bone and its children
         */
        computeAbsoluteTransforms(): void;
        /**
         * Get the world direction from an axis that is in the local space of the bone
         * @param localAxis The local direction that is used to compute the world direction
         * @param tNode The TransformNode that this bone is attached to
         * @returns The world direction
         */
        getDirection(localAxis: Vector3, tNode?: Nullable<TransformNode>): Vector3;
        /**
         * Copy the world direction to a vector3 from an axis that is in the local space of the bone
         * @param localAxis The local direction that is used to compute the world direction
         * @param tNode The TransformNode that this bone is attached to
         * @param result The vector3 that the world direction will be copied to
         */
        getDirectionToRef(localAxis: Vector3, tNode: Nullable<TransformNode> | undefined, result: Vector3): void;
        /**
         * Get the euler rotation of the bone in local or world space
         * @param space The space that the rotation should be in
         * @param tNode The TransformNode that this bone is attached to.  This is only used in world space
         * @returns The euler rotation
         */
        getRotation(space?: Space, tNode?: Nullable<TransformNode>): Vector3;
        /**
         * Copy the euler rotation of the bone to a vector3.  The rotation can be in either local or world space
         * @param space The space that the rotation should be in
         * @param tNode The TransformNode that this bone is attached to.  This is only used in world space
         * @param result The vector3 that the rotation should be copied to
         */
        getRotationToRef(space: Space | undefined, tNode: Nullable<TransformNode> | undefined, result: Vector3): void;
        /**
         * Get the quaternion rotation of the bone in either local or world space
         * @param space The space that the rotation should be in
         * @param tNode The TransformNode that this bone is attached to.  This is only used in world space
         * @returns The quaternion rotation
         */
        getRotationQuaternion(space?: Space, tNode?: Nullable<TransformNode>): Quaternion;
        /**
         * Copy the quaternion rotation of the bone to a quaternion.  The rotation can be in either local or world space
         * @param space The space that the rotation should be in
         * @param tNode The TransformNode that this bone is attached to.  This is only used in world space
         * @param result The quaternion that the rotation should be copied to
         */
        getRotationQuaternionToRef(space: Space | undefined, tNode: Nullable<TransformNode> | undefined, result: Quaternion): void;
        /**
         * Get the rotation matrix of the bone in local or world space
         * @param space The space that the rotation should be in
         * @param tNode The TransformNode that this bone is attached to.  This is only used in world space
         * @returns The rotation matrix
         */
        getRotationMatrix(space: Space | undefined, tNode: TransformNode): Matrix;
        /**
         * Copy the rotation matrix of the bone to a matrix.  The rotation can be in either local or world space
         * @param space The space that the rotation should be in
         * @param tNode The TransformNode that this bone is attached to.  This is only used in world space
         * @param result The quaternion that the rotation should be copied to
         */
        getRotationMatrixToRef(space: Space | undefined, tNode: TransformNode, result: Matrix): void;
        /**
         * Get the world position of a point that is in the local space of the bone
         * @param position The local position
         * @param tNode The TransformNode that this bone is attached to
         * @returns The world position
         */
        getAbsolutePositionFromLocal(position: Vector3, tNode?: Nullable<TransformNode>): Vector3;
        /**
         * Get the world position of a point that is in the local space of the bone and copy it to the result param
         * @param position The local position
         * @param tNode The TransformNode that this bone is attached to
         * @param result The vector3 that the world position should be copied to
         */
        getAbsolutePositionFromLocalToRef(position: Vector3, tNode: Nullable<TransformNode> | undefined, result: Vector3): void;
        /**
         * Get the local position of a point that is in world space
         * @param position The world position
         * @param tNode The TransformNode that this bone is attached to
         * @returns The local position
         */
        getLocalPositionFromAbsolute(position: Vector3, tNode?: Nullable<TransformNode>): Vector3;
        /**
         * Get the local position of a point that is in world space and copy it to the result param
         * @param position The world position
         * @param tNode The TransformNode that this bone is attached to
         * @param result The vector3 that the local position should be copied to
         */
        getLocalPositionFromAbsoluteToRef(position: Vector3, tNode: Nullable<TransformNode> | undefined, result: Vector3): void;
        /**
         * Set the current local matrix as the restPose for this bone.
         */
        setCurrentPoseAsRest(): void;
    }


    /**
     * Class used to apply inverse kinematics to bones
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/bonesSkeletons#boneikcontroller
     */
    export class BoneIKController {
        private static _TmpVecs;
        private static _TmpQuat;
        private static _TmpMats;
        /**
         * Gets or sets the target TransformNode
         * Name kept as mesh for back compatibility
         */
        targetMesh: TransformNode;
        /** Gets or sets the mesh used as pole */
        poleTargetMesh: TransformNode;
        /**
         * Gets or sets the bone used as pole
         */
        poleTargetBone: Nullable<Bone>;
        /**
         * Gets or sets the target position
         */
        targetPosition: Vector3;
        /**
         * Gets or sets the pole target position
         */
        poleTargetPosition: Vector3;
        /**
         * Gets or sets the pole target local offset
         */
        poleTargetLocalOffset: Vector3;
        /**
         * Gets or sets the pole angle
         */
        poleAngle: number;
        /**
         * Gets or sets the TransformNode associated with the controller
         * Name kept as mesh for back compatibility
         */
        mesh: TransformNode;
        /**
         * The amount to slerp (spherical linear interpolation) to the target.  Set this to a value between 0 and 1 (a value of 1 disables slerp)
         */
        slerpAmount: number;
        private _bone1Quat;
        private _bone1Mat;
        private _bone2Ang;
        private _bone1;
        private _bone2;
        private _bone1Length;
        private _bone2Length;
        private _maxAngle;
        private _maxReach;
        private _rightHandedSystem;
        private _bendAxis;
        private _slerping;
        private _adjustRoll;
        private _notEnoughInformation;
        /**
         * Gets or sets maximum allowed angle
         */
        get maxAngle(): number;
        set maxAngle(value: number);
        /**
         * Creates a new BoneIKController
         * @param mesh defines the TransformNode to control
         * @param bone defines the bone to control. The bone needs to have a parent bone. It also needs to have a length greater than 0 or a children we can use to infer its length.
         * @param options defines options to set up the controller
         * @param options.targetMesh
         * @param options.poleTargetMesh
         * @param options.poleTargetBone
         * @param options.poleTargetLocalOffset
         * @param options.poleAngle
         * @param options.bendAxis
         * @param options.maxAngle
         * @param options.slerpAmount
         */
        constructor(mesh: TransformNode, bone: Bone, options?: {
            targetMesh?: TransformNode;
            poleTargetMesh?: TransformNode;
            poleTargetBone?: Bone;
            poleTargetLocalOffset?: Vector3;
            poleAngle?: number;
            bendAxis?: Vector3;
            maxAngle?: number;
            slerpAmount?: number;
        });
        private _setMaxAngle;
        /**
         * Force the controller to update the bones
         */
        update(): void;
        private _updateLinkedTransformRotation;
    }


    /**
     * Class used to make a bone look toward a point in space
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/bonesSkeletons#bonelookcontroller
     */
    export class BoneLookController {
        private static _TmpVecs;
        private static _TmpQuat;
        private static _TmpMats;
        /**
         * The target Vector3 that the bone will look at
         */
        target: Vector3;
        /**
         * The TransformNode that the bone is attached to
         * Name kept as mesh for back compatibility
         */
        mesh: TransformNode;
        /**
         * The bone that will be looking to the target
         */
        bone: Bone;
        /**
         * The up axis of the coordinate system that is used when the bone is rotated
         */
        upAxis: Vector3;
        /**
         * The space that the up axis is in - Space.BONE, Space.LOCAL (default), or Space.WORLD
         */
        upAxisSpace: Space;
        /**
         * Used to make an adjustment to the yaw of the bone
         */
        adjustYaw: number;
        /**
         * Used to make an adjustment to the pitch of the bone
         */
        adjustPitch: number;
        /**
         * Used to make an adjustment to the roll of the bone
         */
        adjustRoll: number;
        /**
         * The amount to slerp (spherical linear interpolation) to the target.  Set this to a value between 0 and 1 (a value of 1 disables slerp)
         */
        slerpAmount: number;
        private _minYaw;
        private _maxYaw;
        private _minPitch;
        private _maxPitch;
        private _minYawSin;
        private _minYawCos;
        private _maxYawSin;
        private _maxYawCos;
        private _midYawConstraint;
        private _minPitchTan;
        private _maxPitchTan;
        private _boneQuat;
        private _slerping;
        private _transformYawPitch;
        private _transformYawPitchInv;
        private _firstFrameSkipped;
        private _yawRange;
        private _fowardAxis;
        /**
         * Gets or sets the minimum yaw angle that the bone can look to
         */
        get minYaw(): number;
        set minYaw(value: number);
        /**
         * Gets or sets the maximum yaw angle that the bone can look to
         */
        get maxYaw(): number;
        set maxYaw(value: number);
        /**
         * Gets or sets the minimum pitch angle that the bone can look to
         */
        get minPitch(): number;
        set minPitch(value: number);
        /**
         * Gets or sets the maximum pitch angle that the bone can look to
         */
        get maxPitch(): number;
        set maxPitch(value: number);
        /**
         * Create a BoneLookController
         * @param mesh the TransformNode that the bone belongs to
         * @param bone the bone that will be looking to the target
         * @param target the target Vector3 to look at
         * @param options optional settings:
         * * maxYaw: the maximum angle the bone will yaw to
         * * minYaw: the minimum angle the bone will yaw to
         * * maxPitch: the maximum angle the bone will pitch to
         * * minPitch: the minimum angle the bone will yaw to
         * * slerpAmount: set the between 0 and 1 to make the bone slerp to the target.
         * * upAxis: the up axis of the coordinate system
         * * upAxisSpace: the space that the up axis is in - Space.BONE, Space.LOCAL (default), or Space.WORLD.
         * * yawAxis: set yawAxis if the bone does not yaw on the y axis
         * * pitchAxis: set pitchAxis if the bone does not pitch on the x axis
         * * adjustYaw: used to make an adjustment to the yaw of the bone
         * * adjustPitch: used to make an adjustment to the pitch of the bone
         * * adjustRoll: used to make an adjustment to the roll of the bone
         * @param options.maxYaw
         * @param options.minYaw
         * @param options.maxPitch
         * @param options.minPitch
         * @param options.slerpAmount
         * @param options.upAxis
         * @param options.upAxisSpace
         * @param options.yawAxis
         * @param options.pitchAxis
         * @param options.adjustYaw
         * @param options.adjustPitch
         * @param options.adjustRoll
         **/
        constructor(mesh: TransformNode, bone: Bone, target: Vector3, options?: {
            maxYaw?: number;
            minYaw?: number;
            maxPitch?: number;
            minPitch?: number;
            slerpAmount?: number;
            upAxis?: Vector3;
            upAxisSpace?: Space;
            yawAxis?: Vector3;
            pitchAxis?: Vector3;
            adjustYaw?: number;
            adjustPitch?: number;
            adjustRoll?: number;
        });
        /**
         * Update the bone to look at the target.  This should be called before the scene is rendered (use scene.registerBeforeRender())
         */
        update(): void;
        private _getAngleDiff;
        private _getAngleBetween;
        private _isAngleBetween;
        private _updateLinkedTransformRotation;
    }




    /**
     * Class used to handle skinning animations
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/bonesSkeletons
     */
    export class Skeleton implements IAnimatable {
        /** defines the skeleton name */
        name: string;
        /** defines the skeleton Id */
        id: string;
        /**
         * Defines the list of child bones
         */
        bones: Bone[];
        /**
         * Defines an estimate of the dimension of the skeleton at rest
         */
        dimensionsAtRest: Vector3;
        /**
         * Defines a boolean indicating if the root matrix is provided by meshes or by the current skeleton (this is the default value)
         */
        needInitialSkinMatrix: boolean;
        /**
         * Gets the list of animations attached to this skeleton
         */
        animations: Array<Animation>;
        private _scene;
        private _isDirty;
        private _transformMatrices;
        private _transformMatrixTexture;
        private _meshesWithPoseMatrix;
        private _animatables;
        private _identity;
        private _synchronizedWithMesh;
        private _ranges;
        private _absoluteTransformIsDirty;
        private _canUseTextureForBones;
        private _uniqueId;
        /** @internal */
        _numBonesWithLinkedTransformNode: number;
        /** @internal */
        _hasWaitingData: Nullable<boolean>;
        /** @internal */
        _parentContainer: Nullable<AbstractScene>;
        /**
         * Specifies if the skeleton should be serialized
         */
        doNotSerialize: boolean;
        private _useTextureToStoreBoneMatrices;
        /**
         * Gets or sets a boolean indicating that bone matrices should be stored as a texture instead of using shader uniforms (default is true).
         * Please note that this option is not available if the hardware does not support it
         */
        get useTextureToStoreBoneMatrices(): boolean;
        set useTextureToStoreBoneMatrices(value: boolean);
        private _animationPropertiesOverride;
        /**
         * Gets or sets the animation properties override
         */
        get animationPropertiesOverride(): Nullable<AnimationPropertiesOverride>;
        set animationPropertiesOverride(value: Nullable<AnimationPropertiesOverride>);
        /**
         * List of inspectable custom properties (used by the Inspector)
         * @see https://doc.babylonjs.com/toolsAndResources/inspector#extensibility
         */
        inspectableCustomProperties: IInspectable[];
        /**
         * An observable triggered before computing the skeleton's matrices
         */
        onBeforeComputeObservable: Observable<Skeleton>;
        /**
         * Gets a boolean indicating that the skeleton effectively stores matrices into a texture
         */
        get isUsingTextureForMatrices(): boolean;
        /**
         * Gets the unique ID of this skeleton
         */
        get uniqueId(): number;
        /**
         * Creates a new skeleton
         * @param name defines the skeleton name
         * @param id defines the skeleton Id
         * @param scene defines the hosting scene
         */
        constructor(
        /** defines the skeleton name */
        name: string, 
        /** defines the skeleton Id */
        id: string, scene: Scene);
        /**
         * Gets the current object class name.
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Returns an array containing the root bones
         * @returns an array containing the root bones
         */
        getChildren(): Array<Bone>;
        /**
         * Gets the list of transform matrices to send to shaders (one matrix per bone)
         * @param mesh defines the mesh to use to get the root matrix (if needInitialSkinMatrix === true)
         * @returns a Float32Array containing matrices data
         */
        getTransformMatrices(mesh: AbstractMesh): Float32Array;
        /**
         * Gets the list of transform matrices to send to shaders inside a texture (one matrix per bone)
         * @param mesh defines the mesh to use to get the root matrix (if needInitialSkinMatrix === true)
         * @returns a raw texture containing the data
         */
        getTransformMatrixTexture(mesh: AbstractMesh): Nullable<RawTexture>;
        /**
         * Gets the current hosting scene
         * @returns a scene object
         */
        getScene(): Scene;
        /**
         * Gets a string representing the current skeleton data
         * @param fullDetails defines a boolean indicating if we want a verbose version
         * @returns a string representing the current skeleton data
         */
        toString(fullDetails?: boolean): string;
        /**
         * Get bone's index searching by name
         * @param name defines bone's name to search for
         * @returns the indice of the bone. Returns -1 if not found
         */
        getBoneIndexByName(name: string): number;
        /**
         * Create a new animation range
         * @param name defines the name of the range
         * @param from defines the start key
         * @param to defines the end key
         */
        createAnimationRange(name: string, from: number, to: number): void;
        /**
         * Delete a specific animation range
         * @param name defines the name of the range
         * @param deleteFrames defines if frames must be removed as well
         */
        deleteAnimationRange(name: string, deleteFrames?: boolean): void;
        /**
         * Gets a specific animation range
         * @param name defines the name of the range to look for
         * @returns the requested animation range or null if not found
         */
        getAnimationRange(name: string): Nullable<AnimationRange>;
        /**
         * Gets the list of all animation ranges defined on this skeleton
         * @returns an array
         */
        getAnimationRanges(): Nullable<AnimationRange>[];
        /**
         * Copy animation range from a source skeleton.
         * This is not for a complete retargeting, only between very similar skeleton's with only possible bone length differences
         * @param source defines the source skeleton
         * @param name defines the name of the range to copy
         * @param rescaleAsRequired defines if rescaling must be applied if required
         * @returns true if operation was successful
         */
        copyAnimationRange(source: Skeleton, name: string, rescaleAsRequired?: boolean): boolean;
        /**
         * Forces the skeleton to go to rest pose
         */
        returnToRest(): void;
        private _getHighestAnimationFrame;
        /**
         * Begin a specific animation range
         * @param name defines the name of the range to start
         * @param loop defines if looping must be turned on (false by default)
         * @param speedRatio defines the speed ratio to apply (1 by default)
         * @param onAnimationEnd defines a callback which will be called when animation will end
         * @returns a new animatable
         */
        beginAnimation(name: string, loop?: boolean, speedRatio?: number, onAnimationEnd?: () => void): Nullable<Animatable>;
        /**
         * Convert the keyframes for a range of animation on a skeleton to be relative to a given reference frame.
         * @param skeleton defines the Skeleton containing the animation range to convert
         * @param referenceFrame defines the frame that keyframes in the range will be relative to
         * @param range defines the name of the AnimationRange belonging to the Skeleton to convert
         * @returns the original skeleton
         */
        static MakeAnimationAdditive(skeleton: Skeleton, referenceFrame: number | undefined, range: string): Nullable<Skeleton>;
        /** @internal */
        _markAsDirty(): void;
        /**
         * @internal
         */
        _registerMeshWithPoseMatrix(mesh: AbstractMesh): void;
        /**
         * @internal
         */
        _unregisterMeshWithPoseMatrix(mesh: AbstractMesh): void;
        private _computeTransformMatrices;
        /**
         * Build all resources required to render a skeleton
         */
        prepare(): void;
        /**
         * Gets the list of animatables currently running for this skeleton
         * @returns an array of animatables
         */
        getAnimatables(): IAnimatable[];
        /**
         * Clone the current skeleton
         * @param name defines the name of the new skeleton
         * @param id defines the id of the new skeleton
         * @returns the new skeleton
         */
        clone(name: string, id?: string): Skeleton;
        /**
         * Enable animation blending for this skeleton
         * @param blendingSpeed defines the blending speed to apply
         * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/advanced_animations#animation-blending
         */
        enableBlending(blendingSpeed?: number): void;
        /**
         * Releases all resources associated with the current skeleton
         */
        dispose(): void;
        /**
         * Serialize the skeleton in a JSON object
         * @returns a JSON object
         */
        serialize(): any;
        /**
         * Creates a new skeleton from serialized data
         * @param parsedSkeleton defines the serialized data
         * @param scene defines the hosting scene
         * @returns a new skeleton
         */
        static Parse(parsedSkeleton: any, scene: Scene): Skeleton;
        /**
         * Compute all node absolute transforms
         * @param forceUpdate defines if computation must be done even if cache is up to date
         */
        computeAbsoluteTransforms(forceUpdate?: boolean): void;
        /**
         * Gets the root pose matrix
         * @returns a matrix
         */
        getPoseMatrix(): Nullable<Matrix>;
        /**
         * Sorts bones per internal index
         */
        sortBones(): void;
        private _sortBones;
        /**
         * Set the current local matrix as the restPose for all bones in the skeleton.
         */
        setCurrentPoseAsRest(): void;
    }


    /**
     * Class used to store data that will be store in GPU memory
     */
    export class Buffer {
        private _engine;
        private _buffer;
        /** @internal */
        _data: Nullable<DataArray>;
        private _updatable;
        private _instanced;
        private _divisor;
        private _isAlreadyOwned;
        /**
         * Gets the byte stride.
         */
        readonly byteStride: number;
        /**
         * Constructor
         * @param engine the engine
         * @param data the data to use for this buffer
         * @param updatable whether the data is updatable
         * @param stride the stride (optional)
         * @param postponeInternalCreation whether to postpone creating the internal WebGL buffer (optional)
         * @param instanced whether the buffer is instanced (optional)
         * @param useBytes set to true if the stride in in bytes (optional)
         * @param divisor sets an optional divisor for instances (1 by default)
         */
        constructor(engine: any, data: DataArray | DataBuffer, updatable: boolean, stride?: number, postponeInternalCreation?: boolean, instanced?: boolean, useBytes?: boolean, divisor?: number);
        /**
         * Create a new VertexBuffer based on the current buffer
         * @param kind defines the vertex buffer kind (position, normal, etc.)
         * @param offset defines offset in the buffer (0 by default)
         * @param size defines the size in floats of attributes (position is 3 for instance)
         * @param stride defines the stride size in floats in the buffer (the offset to apply to reach next value when data is interleaved)
         * @param instanced defines if the vertex buffer contains indexed data
         * @param useBytes defines if the offset and stride are in bytes     *
         * @param divisor sets an optional divisor for instances (1 by default)
         * @returns the new vertex buffer
         */
        createVertexBuffer(kind: string, offset: number, size: number, stride?: number, instanced?: boolean, useBytes?: boolean, divisor?: number): VertexBuffer;
        /**
         * Gets a boolean indicating if the Buffer is updatable?
         * @returns true if the buffer is updatable
         */
        isUpdatable(): boolean;
        /**
         * Gets current buffer's data
         * @returns a DataArray or null
         */
        getData(): Nullable<DataArray>;
        /**
         * Gets underlying native buffer
         * @returns underlying native buffer
         */
        getBuffer(): Nullable<DataBuffer>;
        /**
         * Gets the stride in float32 units (i.e. byte stride / 4).
         * May not be an integer if the byte stride is not divisible by 4.
         * @returns the stride in float32 units
         * @deprecated Please use byteStride instead.
         */
        getStrideSize(): number;
        /**
         * Store data into the buffer. Creates the buffer if not used already.
         * If the buffer was already used, it will be updated only if it is updatable, otherwise it will do nothing.
         * @param data defines the data to store
         */
        create(data?: Nullable<DataArray>): void;
        /** @internal */
        _rebuild(): void;
        /**
         * Update current buffer data
         * @param data defines the data to store
         */
        update(data: DataArray): void;
        /**
         * Updates the data directly.
         * @param data the new data
         * @param offset the new offset
         * @param vertexCount the vertex count (optional)
         * @param useBytes set to true if the offset is in bytes
         */
        updateDirectly(data: DataArray, offset: number, vertexCount?: number, useBytes?: boolean): void;
        /** @internal */
        _increaseReferences(): void;
        /**
         * Release all resources
         */
        dispose(): void;
    }
    /**
     * Specialized buffer used to store vertex data
     */
    export class VertexBuffer {
        private static _Counter;
        /** @internal */
        _buffer: Buffer;
        /** @internal */
        _validOffsetRange: boolean;
        private _kind;
        private _size;
        private _ownsBuffer;
        private _instanced;
        private _instanceDivisor;
        /**
         * The byte type.
         */
        static readonly BYTE = 5120;
        /**
         * The unsigned byte type.
         */
        static readonly UNSIGNED_BYTE = 5121;
        /**
         * The short type.
         */
        static readonly SHORT = 5122;
        /**
         * The unsigned short type.
         */
        static readonly UNSIGNED_SHORT = 5123;
        /**
         * The integer type.
         */
        static readonly INT = 5124;
        /**
         * The unsigned integer type.
         */
        static readonly UNSIGNED_INT = 5125;
        /**
         * The float type.
         */
        static readonly FLOAT = 5126;
        /**
         * Gets or sets the instance divisor when in instanced mode
         */
        get instanceDivisor(): number;
        set instanceDivisor(value: number);
        /**
         * Gets the byte stride.
         */
        readonly byteStride: number;
        /**
         * Gets the byte offset.
         */
        readonly byteOffset: number;
        /**
         * Gets whether integer data values should be normalized into a certain range when being casted to a float.
         */
        readonly normalized: boolean;
        /**
         * Gets the data type of each component in the array.
         */
        readonly type: number;
        /**
         * Gets the unique id of this vertex buffer
         */
        readonly uniqueId: number;
        /**
         * Gets a hash code representing the format (type, normalized, size, instanced, stride) of this buffer
         * All buffers with the same format will have the same hash code
         */
        readonly hashCode: number;
        /**
         * Constructor
         * @param engine the engine
         * @param data the data to use for this vertex buffer
         * @param kind the vertex buffer kind
         * @param updatable whether the data is updatable
         * @param postponeInternalCreation whether to postpone creating the internal WebGL buffer (optional)
         * @param stride the stride (optional)
         * @param instanced whether the buffer is instanced (optional)
         * @param offset the offset of the data (optional)
         * @param size the number of components (optional)
         * @param type the type of the component (optional)
         * @param normalized whether the data contains normalized data (optional)
         * @param useBytes set to true if stride and offset are in bytes (optional)
         * @param divisor defines the instance divisor to use (1 by default)
         * @param takeBufferOwnership defines if the buffer should be released when the vertex buffer is disposed
         */
        constructor(engine: any, data: DataArray | Buffer | DataBuffer, kind: string, updatable: boolean, postponeInternalCreation?: boolean, stride?: number, instanced?: boolean, offset?: number, size?: number, type?: number, normalized?: boolean, useBytes?: boolean, divisor?: number, takeBufferOwnership?: boolean);
        private _computeHashCode;
        /** @internal */
        _rebuild(): void;
        /**
         * Returns the kind of the VertexBuffer (string)
         * @returns a string
         */
        getKind(): string;
        /**
         * Gets a boolean indicating if the VertexBuffer is updatable?
         * @returns true if the buffer is updatable
         */
        isUpdatable(): boolean;
        /**
         * Gets current buffer's data
         * @returns a DataArray or null
         */
        getData(): Nullable<DataArray>;
        /**
         * Gets current buffer's data as a float array. Float data is constructed if the vertex buffer data cannot be returned directly.
         * @param totalVertices number of vertices in the buffer to take into account
         * @param forceCopy defines a boolean indicating that the returned array must be cloned upon returning it
         * @returns a float array containing vertex data
         */
        getFloatData(totalVertices: number, forceCopy?: boolean): Nullable<FloatArray>;
        /**
         * Gets underlying native buffer
         * @returns underlying native buffer
         */
        getBuffer(): Nullable<DataBuffer>;
        /**
         * Gets the stride in float32 units (i.e. byte stride / 4).
         * May not be an integer if the byte stride is not divisible by 4.
         * @returns the stride in float32 units
         * @deprecated Please use byteStride instead.
         */
        getStrideSize(): number;
        /**
         * Returns the offset as a multiple of the type byte length.
         * @returns the offset in bytes
         * @deprecated Please use byteOffset instead.
         */
        getOffset(): number;
        /**
         * Returns the number of components or the byte size per vertex attribute
         * @param sizeInBytes If true, returns the size in bytes or else the size in number of components of the vertex attribute (default: false)
         * @returns the number of components
         */
        getSize(sizeInBytes?: boolean): number;
        /**
         * Gets a boolean indicating is the internal buffer of the VertexBuffer is instanced
         * @returns true if this buffer is instanced
         */
        getIsInstanced(): boolean;
        /**
         * Returns the instancing divisor, zero for non-instanced (integer).
         * @returns a number
         */
        getInstanceDivisor(): number;
        /**
         * Store data into the buffer. If the buffer was already used it will be either recreated or updated depending on isUpdatable property
         * @param data defines the data to store
         */
        create(data?: DataArray): void;
        /**
         * Updates the underlying buffer according to the passed numeric array or Float32Array.
         * This function will create a new buffer if the current one is not updatable
         * @param data defines the data to store
         */
        update(data: DataArray): void;
        /**
         * Updates directly the underlying WebGLBuffer according to the passed numeric array or Float32Array.
         * Returns the directly updated WebGLBuffer.
         * @param data the new data
         * @param offset the new offset
         * @param useBytes set to true if the offset is in bytes
         */
        updateDirectly(data: DataArray, offset: number, useBytes?: boolean): void;
        /**
         * Disposes the VertexBuffer and the underlying WebGLBuffer.
         */
        dispose(): void;
        /**
         * Enumerates each value of this vertex buffer as numbers.
         * @param count the number of values to enumerate
         * @param callback the callback function called for each value
         */
        forEach(count: number, callback: (value: number, index: number) => void): void;
        /**
         * Positions
         */
        static readonly PositionKind = "position";
        /**
         * Normals
         */
        static readonly NormalKind = "normal";
        /**
         * Tangents
         */
        static readonly TangentKind = "tangent";
        /**
         * Texture coordinates
         */
        static readonly UVKind = "uv";
        /**
         * Texture coordinates 2
         */
        static readonly UV2Kind = "uv2";
        /**
         * Texture coordinates 3
         */
        static readonly UV3Kind = "uv3";
        /**
         * Texture coordinates 4
         */
        static readonly UV4Kind = "uv4";
        /**
         * Texture coordinates 5
         */
        static readonly UV5Kind = "uv5";
        /**
         * Texture coordinates 6
         */
        static readonly UV6Kind = "uv6";
        /**
         * Colors
         */
        static readonly ColorKind = "color";
        /**
         * Instance Colors
         */
        static readonly ColorInstanceKind = "instanceColor";
        /**
         * Matrix indices (for bones)
         */
        static readonly MatricesIndicesKind = "matricesIndices";
        /**
         * Matrix weights (for bones)
         */
        static readonly MatricesWeightsKind = "matricesWeights";
        /**
         * Additional matrix indices (for bones)
         */
        static readonly MatricesIndicesExtraKind = "matricesIndicesExtra";
        /**
         * Additional matrix weights (for bones)
         */
        static readonly MatricesWeightsExtraKind = "matricesWeightsExtra";
        /**
         * Deduces the stride given a kind.
         * @param kind The kind string to deduce
         * @returns The deduced stride
         */
        static DeduceStride(kind: string): number;
        /**
         * Gets the byte length of the given type.
         * @param type the type
         * @returns the number of bytes
         */
        static GetTypeByteLength(type: number): number;
        /**
         * Enumerates each value of the given parameters as numbers.
         * @param data the data to enumerate
         * @param byteOffset the byte offset of the data
         * @param byteStride the byte stride of the data
         * @param componentCount the number of components per element
         * @param componentType the type of the component
         * @param count the number of values to enumerate
         * @param normalized whether the data is normalized
         * @param callback the callback function called for each value
         */
        static ForEach(data: DataArray, byteOffset: number, byteStride: number, componentCount: number, componentType: number, count: number, normalized: boolean, callback: (value: number, index: number) => void): void;
        private static _GetFloatValue;
    }


    /**
     * Class used to store gfx data (like WebGLBuffer)
     */
    export class DataBuffer {
        private static _Counter;
        /**
         * Gets or sets the number of objects referencing this buffer
         */
        references: number;
        /** Gets or sets the size of the underlying buffer */
        capacity: number;
        /**
         * Gets or sets a boolean indicating if the buffer contains 32bits indices
         */
        is32Bits: boolean;
        /**
         * Gets the underlying buffer
         */
        get underlyingResource(): any;
        /**
         * Gets the unique id of this buffer
         */
        readonly uniqueId: number;
        /**
         * Constructs the buffer
         */
        constructor();
    }




    /**
     * This class is a small wrapper around a native buffer that can be read and/or written
     */
    export class StorageBuffer {
        private _engine;
        private _buffer;
        private _bufferSize;
        private _creationFlags;
        /**
         * Creates a new storage buffer instance
         * @param engine The engine the buffer will be created inside
         * @param size The size of the buffer in bytes
         * @param creationFlags flags to use when creating the buffer (see Constants.BUFFER_CREATIONFLAG_XXX). The BUFFER_CREATIONFLAG_STORAGE flag will be automatically added.
         */
        constructor(engine: ThinEngine, size: number, creationFlags?: number);
        private _create;
        /** @internal */
        _rebuild(): void;
        /**
         * Gets underlying native buffer
         * @returns underlying native buffer
         */
        getBuffer(): DataBuffer;
        /**
         * Updates the storage buffer
         * @param data the data used to update the storage buffer
         * @param byteOffset the byte offset of the data (optional)
         * @param byteLength the byte length of the data (optional)
         */
        update(data: DataArray, byteOffset?: number, byteLength?: number): void;
        /**
         * Reads data from the storage buffer
         * @param offset The offset in the storage buffer to start reading from (default: 0)
         * @param size  The number of bytes to read from the storage buffer (default: capacity of the buffer)
         * @param buffer The buffer to write the data we have read from the storage buffer to (optional)
         * @returns If not undefined, returns the (promise) buffer (as provided by the 4th parameter) filled with the data, else it returns a (promise) Uint8Array with the data read from the storage buffer
         */
        read(offset?: number, size?: number, buffer?: ArrayBufferView): Promise<ArrayBufferView>;
        /**
         * Disposes the storage buffer
         */
        dispose(): void;
    }


    /**
     * This represents an orbital type of camera.
     *
     * This camera always points towards a given target position and can be rotated around that target with the target as the centre of rotation. It can be controlled with cursors and mouse, or with touch events.
     * Think of this camera as one orbiting its target position, or more imaginatively as a spy satellite orbiting the earth. Its position relative to the target (earth) can be set by three parameters, alpha (radians) the longitudinal rotation, beta (radians) the latitudinal rotation and radius the distance from the target position.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_introduction#arc-rotate-camera
     */
    export class ArcRotateCamera extends TargetCamera {
        /**
         * Defines the rotation angle of the camera along the longitudinal axis.
         */
        alpha: number;
        /**
         * Defines the rotation angle of the camera along the latitudinal axis.
         */
        beta: number;
        /**
         * Defines the radius of the camera from it s target point.
         */
        radius: number;
        /**
         * Defines an override value to use as the parameter to setTarget.
         * This allows the parameter to be specified when animating the target (e.g. using FramingBehavior).
         */
        overrideCloneAlphaBetaRadius: Nullable<boolean>;
        protected _target: Vector3;
        protected _targetHost: Nullable<AbstractMesh>;
        /**
         * Defines the target point of the camera.
         * The camera looks towards it from the radius distance.
         */
        get target(): Vector3;
        set target(value: Vector3);
        /**
         * Defines the target mesh of the camera.
         * The camera looks towards it from the radius distance.
         * Please note that setting a target host will disable panning.
         */
        get targetHost(): Nullable<AbstractMesh>;
        set targetHost(value: Nullable<AbstractMesh>);
        /**
         * Return the current target position of the camera. This value is expressed in local space.
         * @returns the target position
         */
        getTarget(): Vector3;
        /**
         * Define the current local position of the camera in the scene
         */
        get position(): Vector3;
        set position(newPosition: Vector3);
        protected _upToYMatrix: Matrix;
        protected _yToUpMatrix: Matrix;
        /**
         * The vector the camera should consider as up. (default is Vector3(0, 1, 0) as returned by Vector3.Up())
         * Setting this will copy the given vector to the camera's upVector, and set rotation matrices to and from Y up.
         * DO NOT set the up vector using copyFrom or copyFromFloats, as this bypasses setting the above matrices.
         */
        set upVector(vec: Vector3);
        get upVector(): Vector3;
        /**
         * Sets the Y-up to camera up-vector rotation matrix, and the up-vector to Y-up rotation matrix.
         */
        setMatUp(): void;
        /**
         * Current inertia value on the longitudinal axis.
         * The bigger this number the longer it will take for the camera to stop.
         */
        inertialAlphaOffset: number;
        /**
         * Current inertia value on the latitudinal axis.
         * The bigger this number the longer it will take for the camera to stop.
         */
        inertialBetaOffset: number;
        /**
         * Current inertia value on the radius axis.
         * The bigger this number the longer it will take for the camera to stop.
         */
        inertialRadiusOffset: number;
        /**
         * Minimum allowed angle on the longitudinal axis.
         * This can help limiting how the Camera is able to move in the scene.
         */
        lowerAlphaLimit: Nullable<number>;
        /**
         * Maximum allowed angle on the longitudinal axis.
         * This can help limiting how the Camera is able to move in the scene.
         */
        upperAlphaLimit: Nullable<number>;
        /**
         * Minimum allowed angle on the latitudinal axis.
         * This can help limiting how the Camera is able to move in the scene.
         */
        lowerBetaLimit: Nullable<number>;
        /**
         * Maximum allowed angle on the latitudinal axis.
         * This can help limiting how the Camera is able to move in the scene.
         */
        upperBetaLimit: Nullable<number>;
        /**
         * Minimum allowed distance of the camera to the target (The camera can not get closer).
         * This can help limiting how the Camera is able to move in the scene.
         */
        lowerRadiusLimit: Nullable<number>;
        /**
         * Maximum allowed distance of the camera to the target (The camera can not get further).
         * This can help limiting how the Camera is able to move in the scene.
         */
        upperRadiusLimit: Nullable<number>;
        /**
         * Defines the current inertia value used during panning of the camera along the X axis.
         */
        inertialPanningX: number;
        /**
         * Defines the current inertia value used during panning of the camera along the Y axis.
         */
        inertialPanningY: number;
        /**
         * Defines the distance used to consider the camera in pan mode vs pinch/zoom.
         * Basically if your fingers moves away from more than this distance you will be considered
         * in pinch mode.
         */
        pinchToPanMaxDistance: number;
        /**
         * Defines the maximum distance the camera can pan.
         * This could help keeping the camera always in your scene.
         */
        panningDistanceLimit: Nullable<number>;
        /**
         * Defines the target of the camera before panning.
         */
        panningOriginTarget: Vector3;
        /**
         * Defines the value of the inertia used during panning.
         * 0 would mean stop inertia and one would mean no deceleration at all.
         */
        panningInertia: number;
        /**
         * Gets or Set the pointer angular sensibility  along the X axis or how fast is the camera rotating.
         */
        get angularSensibilityX(): number;
        set angularSensibilityX(value: number);
        /**
         * Gets or Set the pointer angular sensibility along the Y axis or how fast is the camera rotating.
         */
        get angularSensibilityY(): number;
        set angularSensibilityY(value: number);
        /**
         * Gets or Set the pointer pinch precision or how fast is the camera zooming.
         */
        get pinchPrecision(): number;
        set pinchPrecision(value: number);
        /**
         * Gets or Set the pointer pinch delta percentage or how fast is the camera zooming.
         * It will be used instead of pinchDeltaPrecision if different from 0.
         * It defines the percentage of current camera.radius to use as delta when pinch zoom is used.
         */
        get pinchDeltaPercentage(): number;
        set pinchDeltaPercentage(value: number);
        /**
         * Gets or Set the pointer use natural pinch zoom to override the pinch precision
         * and pinch delta percentage.
         * When useNaturalPinchZoom is true, multi touch zoom will zoom in such
         * that any object in the plane at the camera's target point will scale
         * perfectly with finger motion.
         */
        get useNaturalPinchZoom(): boolean;
        set useNaturalPinchZoom(value: boolean);
        /**
         * Gets or Set the pointer panning sensibility or how fast is the camera moving.
         */
        get panningSensibility(): number;
        set panningSensibility(value: number);
        /**
         * Gets or Set the list of keyboard keys used to control beta angle in a positive direction.
         */
        get keysUp(): number[];
        set keysUp(value: number[]);
        /**
         * Gets or Set the list of keyboard keys used to control beta angle in a negative direction.
         */
        get keysDown(): number[];
        set keysDown(value: number[]);
        /**
         * Gets or Set the list of keyboard keys used to control alpha angle in a negative direction.
         */
        get keysLeft(): number[];
        set keysLeft(value: number[]);
        /**
         * Gets or Set the list of keyboard keys used to control alpha angle in a positive direction.
         */
        get keysRight(): number[];
        set keysRight(value: number[]);
        /**
         * Gets or Set the mouse wheel precision or how fast is the camera zooming.
         */
        get wheelPrecision(): number;
        set wheelPrecision(value: number);
        /**
         * Gets or Set the boolean value that controls whether or not the mouse wheel
         * zooms to the location of the mouse pointer or not.  The default is false.
         */
        get zoomToMouseLocation(): boolean;
        set zoomToMouseLocation(value: boolean);
        /**
         * Gets or Set the mouse wheel delta percentage or how fast is the camera zooming.
         * It will be used instead of pinchDeltaPrecision if different from 0.
         * It defines the percentage of current camera.radius to use as delta when pinch zoom is used.
         */
        get wheelDeltaPercentage(): number;
        set wheelDeltaPercentage(value: number);
        /**
         * Defines how much the radius should be scaled while zooming on a particular mesh (through the zoomOn function)
         */
        zoomOnFactor: number;
        /**
         * Defines a screen offset for the camera position.
         */
        targetScreenOffset: Vector2;
        /**
         * Allows the camera to be completely reversed.
         * If false the camera can not arrive upside down.
         */
        allowUpsideDown: boolean;
        /**
         * Define if double tap/click is used to restore the previously saved state of the camera.
         */
        useInputToRestoreState: boolean;
        /** @internal */
        _viewMatrix: Matrix;
        /** @internal */
        _useCtrlForPanning: boolean;
        /** @internal */
        _panningMouseButton: number;
        /**
         * Defines the input associated to the camera.
         */
        inputs: ArcRotateCameraInputsManager;
        /** @internal */
        _reset: () => void;
        /**
         * Defines the allowed panning axis.
         */
        panningAxis: Vector3;
        protected _transformedDirection: Vector3;
        /**
         * Defines if camera will eliminate transform on y axis.
         */
        mapPanning: boolean;
        private _bouncingBehavior;
        /**
         * Gets the bouncing behavior of the camera if it has been enabled.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/behaviors/cameraBehaviors#bouncing-behavior
         */
        get bouncingBehavior(): Nullable<BouncingBehavior>;
        /**
         * Defines if the bouncing behavior of the camera is enabled on the camera.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/behaviors/cameraBehaviors#bouncing-behavior
         */
        get useBouncingBehavior(): boolean;
        set useBouncingBehavior(value: boolean);
        private _framingBehavior;
        /**
         * Gets the framing behavior of the camera if it has been enabled.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/behaviors/cameraBehaviors#framing-behavior
         */
        get framingBehavior(): Nullable<FramingBehavior>;
        /**
         * Defines if the framing behavior of the camera is enabled on the camera.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/behaviors/cameraBehaviors#framing-behavior
         */
        get useFramingBehavior(): boolean;
        set useFramingBehavior(value: boolean);
        private _autoRotationBehavior;
        /**
         * Gets the auto rotation behavior of the camera if it has been enabled.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/behaviors/cameraBehaviors#autorotation-behavior
         */
        get autoRotationBehavior(): Nullable<AutoRotationBehavior>;
        /**
         * Defines if the auto rotation behavior of the camera is enabled on the camera.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/behaviors/cameraBehaviors#autorotation-behavior
         */
        get useAutoRotationBehavior(): boolean;
        set useAutoRotationBehavior(value: boolean);
        /**
         * Observable triggered when the mesh target has been changed on the camera.
         */
        onMeshTargetChangedObservable: Observable<Nullable<AbstractMesh>>;
        /**
         * Event raised when the camera is colliding with a mesh.
         */
        onCollide: (collidedMesh: AbstractMesh) => void;
        /**
         * Defines whether the camera should check collision with the objects oh the scene.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_collisions#how-can-i-do-this-
         */
        checkCollisions: boolean;
        /**
         * Defines the collision radius of the camera.
         * This simulates a sphere around the camera.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_collisions#arcrotatecamera
         */
        collisionRadius: Vector3;
        protected _collider: Collider;
        protected _previousPosition: Vector3;
        protected _collisionVelocity: Vector3;
        protected _newPosition: Vector3;
        protected _previousAlpha: number;
        protected _previousBeta: number;
        protected _previousRadius: number;
        protected _collisionTriggered: boolean;
        protected _targetBoundingCenter: Nullable<Vector3>;
        private _computationVector;
        /**
         * Instantiates a new ArcRotateCamera in a given scene
         * @param name Defines the name of the camera
         * @param alpha Defines the camera rotation along the longitudinal axis
         * @param beta Defines the camera rotation along the latitudinal axis
         * @param radius Defines the camera distance from its target
         * @param target Defines the camera target
         * @param scene Defines the scene the camera belongs to
         * @param setActiveOnSceneIfNoneActive Defines whether the camera should be marked as active if not other active cameras have been defined
         */
        constructor(name: string, alpha: number, beta: number, radius: number, target: Vector3, scene?: Scene, setActiveOnSceneIfNoneActive?: boolean);
        /** @internal */
        _initCache(): void;
        /**
         * @internal
         */
        _updateCache(ignoreParentClass?: boolean): void;
        protected _getTargetPosition(): Vector3;
        private _storedAlpha;
        private _storedBeta;
        private _storedRadius;
        private _storedTarget;
        private _storedTargetScreenOffset;
        /**
         * Stores the current state of the camera (alpha, beta, radius and target)
         * @returns the camera itself
         */
        storeState(): Camera;
        /**
         * @internal
         * Restored camera state. You must call storeState() first
         */
        _restoreStateValues(): boolean;
        /** @internal */
        _isSynchronizedViewMatrix(): boolean;
        /**
         * Attach the input controls to a specific dom element to get the input from.
         * @param noPreventDefault Defines whether event caught by the controls should call preventdefault() (https://developer.mozilla.org/en-US/docs/Web/API/Event/preventDefault)
         */
        attachControl(noPreventDefault?: boolean): void;
        /**
         * Attach the input controls to a specific dom element to get the input from.
         * @param ignored defines an ignored parameter kept for backward compatibility.
         * @param noPreventDefault Defines whether event caught by the controls should call preventdefault() (https://developer.mozilla.org/en-US/docs/Web/API/Event/preventDefault)
         */
        attachControl(ignored: any, noPreventDefault?: boolean): void;
        /**
         * Attached controls to the current camera.
         * @param noPreventDefault Defines whether event caught by the controls should call preventdefault() (https://developer.mozilla.org/en-US/docs/Web/API/Event/preventDefault)
         * @param useCtrlForPanning  Defines whether ctrl is used for panning within the controls
         */
        attachControl(noPreventDefault: boolean, useCtrlForPanning: boolean): void;
        /**
         * Attached controls to the current camera.
         * @param ignored defines an ignored parameter kept for backward compatibility.
         * @param noPreventDefault Defines whether event caught by the controls should call preventdefault() (https://developer.mozilla.org/en-US/docs/Web/API/Event/preventDefault)
         * @param useCtrlForPanning  Defines whether ctrl is used for panning within the controls
         */
        attachControl(ignored: any, noPreventDefault: boolean, useCtrlForPanning: boolean): void;
        /**
         * Attached controls to the current camera.
         * @param noPreventDefault Defines whether event caught by the controls should call preventdefault() (https://developer.mozilla.org/en-US/docs/Web/API/Event/preventDefault)
         * @param useCtrlForPanning  Defines whether ctrl is used for panning within the controls
         * @param panningMouseButton Defines whether panning is allowed through mouse click button
         */
        attachControl(noPreventDefault: boolean, useCtrlForPanning: boolean, panningMouseButton: number): void;
        /**
         * Detach the current controls from the specified dom element.
         */
        detachControl(): void;
        /** @internal */
        _checkInputs(): void;
        protected _checkLimits(): void;
        /**
         * Rebuilds angles (alpha, beta) and radius from the give position and target
         */
        rebuildAnglesAndRadius(): void;
        /**
         * Use a position to define the current camera related information like alpha, beta and radius
         * @param position Defines the position to set the camera at
         */
        setPosition(position: Vector3): void;
        /**
         * Defines the target the camera should look at.
         * This will automatically adapt alpha beta and radius to fit within the new target.
         * Please note that setting a target as a mesh will disable panning.
         * @param target Defines the new target as a Vector or a mesh
         * @param toBoundingCenter In case of a mesh target, defines whether to target the mesh position or its bounding information center
         * @param allowSamePosition If false, prevents reapplying the new computed position if it is identical to the current one (optim)
         * @param cloneAlphaBetaRadius If true, replicate the current setup (alpha, beta, radius) on the new target
         */
        setTarget(target: AbstractMesh | Vector3, toBoundingCenter?: boolean, allowSamePosition?: boolean, cloneAlphaBetaRadius?: boolean): void;
        /** @internal */
        _getViewMatrix(): Matrix;
        protected _onCollisionPositionChange: (collisionId: number, newPosition: Vector3, collidedMesh?: Nullable<AbstractMesh>) => void;
        /**
         * Zooms on a mesh to be at the min distance where we could see it fully in the current viewport.
         * @param meshes Defines the mesh to zoom on
         * @param doNotUpdateMaxZ Defines whether or not maxZ should be updated whilst zooming on the mesh (this can happen if the mesh is big and the maxradius pretty small for instance)
         */
        zoomOn(meshes?: AbstractMesh[], doNotUpdateMaxZ?: boolean): void;
        /**
         * Focus on a mesh or a bounding box. This adapts the target and maxRadius if necessary but does not update the current radius.
         * The target will be changed but the radius
         * @param meshesOrMinMaxVectorAndDistance Defines the mesh or bounding info to focus on
         * @param doNotUpdateMaxZ Defines whether or not maxZ should be updated whilst zooming on the mesh (this can happen if the mesh is big and the maxradius pretty small for instance)
         */
        focusOn(meshesOrMinMaxVectorAndDistance: AbstractMesh[] | {
            min: Vector3;
            max: Vector3;
            distance: number;
        }, doNotUpdateMaxZ?: boolean): void;
        /**
         * @override
         * Override Camera.createRigCamera
         */
        createRigCamera(name: string, cameraIndex: number): Camera;
        /**
         * @internal
         * @override
         * Override Camera._updateRigCameras
         */
        _updateRigCameras(): void;
        /**
         * Destroy the camera and release the current resources hold by it.
         */
        dispose(): void;
        /**
         * Gets the current object class name.
         * @returns the class name
         */
        getClassName(): string;
    }


    /**
     * Default Inputs manager for the ArcRotateCamera.
     * It groups all the default supported inputs for ease of use.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/customizingCameraInputs
     */
    export class ArcRotateCameraInputsManager extends CameraInputsManager<ArcRotateCamera> {
        /**
         * Instantiates a new ArcRotateCameraInputsManager.
         * @param camera Defines the camera the inputs belong to
         */
        constructor(camera: ArcRotateCamera);
        /**
         * Add mouse wheel input support to the input manager.
         * @returns the current input manager
         */
        addMouseWheel(): ArcRotateCameraInputsManager;
        /**
         * Add pointers input support to the input manager.
         * @returns the current input manager
         */
        addPointers(): ArcRotateCameraInputsManager;
        /**
         * Add keyboard input support to the input manager.
         * @returns the current input manager
         */
        addKeyboard(): ArcRotateCameraInputsManager;
    }


    /**
     * This is the base class of all the camera used in the application.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras
     */
    export class Camera extends Node {
        /**
         * @internal
         */
        static _CreateDefaultParsedCamera: (name: string, scene: Scene) => Camera;
        /**
         * This is the default projection mode used by the cameras.
         * It helps recreating a feeling of perspective and better appreciate depth.
         * This is the best way to simulate real life cameras.
         */
        static readonly PERSPECTIVE_CAMERA = 0;
        /**
         * This helps creating camera with an orthographic mode.
         * Orthographic is commonly used in engineering as a means to produce object specifications that communicate dimensions unambiguously, each line of 1 unit length (cm, meter..whatever) will appear to have the same length everywhere on the drawing. This allows the drafter to dimension only a subset of lines and let the reader know that other lines of that length on the drawing are also that length in reality. Every parallel line in the drawing is also parallel in the object.
         */
        static readonly ORTHOGRAPHIC_CAMERA = 1;
        /**
         * This is the default FOV mode for perspective cameras.
         * This setting aligns the upper and lower bounds of the viewport to the upper and lower bounds of the camera frustum.
         */
        static readonly FOVMODE_VERTICAL_FIXED = 0;
        /**
         * This setting aligns the left and right bounds of the viewport to the left and right bounds of the camera frustum.
         */
        static readonly FOVMODE_HORIZONTAL_FIXED = 1;
        /**
         * This specifies there is no need for a camera rig.
         * Basically only one eye is rendered corresponding to the camera.
         */
        static readonly RIG_MODE_NONE = 0;
        /**
         * Simulates a camera Rig with one blue eye and one red eye.
         * This can be use with 3d blue and red glasses.
         */
        static readonly RIG_MODE_STEREOSCOPIC_ANAGLYPH = 10;
        /**
         * Defines that both eyes of the camera will be rendered side by side with a parallel target.
         */
        static readonly RIG_MODE_STEREOSCOPIC_SIDEBYSIDE_PARALLEL = 11;
        /**
         * Defines that both eyes of the camera will be rendered side by side with a none parallel target.
         */
        static readonly RIG_MODE_STEREOSCOPIC_SIDEBYSIDE_CROSSEYED = 12;
        /**
         * Defines that both eyes of the camera will be rendered over under each other.
         */
        static readonly RIG_MODE_STEREOSCOPIC_OVERUNDER = 13;
        /**
         * Defines that both eyes of the camera will be rendered on successive lines interlaced for passive 3d monitors.
         */
        static readonly RIG_MODE_STEREOSCOPIC_INTERLACED = 14;
        /**
         * Defines that both eyes of the camera should be renderered in a VR mode (carbox).
         */
        static readonly RIG_MODE_VR = 20;
        /**
         * Defines that both eyes of the camera should be renderered in a VR mode (webVR).
         */
        static readonly RIG_MODE_WEBVR = 21;
        /**
         * Custom rig mode allowing rig cameras to be populated manually with any number of cameras
         */
        static readonly RIG_MODE_CUSTOM = 22;
        /**
         * Defines if by default attaching controls should prevent the default javascript event to continue.
         */
        static ForceAttachControlToAlwaysPreventDefault: boolean;
        /**
         * Define the input manager associated with the camera.
         */
        inputs: CameraInputsManager<Camera>;
        /** @internal */
        _position: Vector3;
        /**
         * Define the current local position of the camera in the scene
         */
        get position(): Vector3;
        set position(newPosition: Vector3);
        protected _upVector: Vector3;
        /**
         * The vector the camera should consider as up.
         * (default is Vector3(0, 1, 0) aka Vector3.Up())
         */
        set upVector(vec: Vector3);
        get upVector(): Vector3;
        /**
         * The screen area in scene units squared
         */
        get screenArea(): number;
        /**
         * Define the current limit on the left side for an orthographic camera
         * In scene unit
         */
        private _orthoLeft;
        set orthoLeft(value: Nullable<number>);
        get orthoLeft(): Nullable<number>;
        /**
         * Define the current limit on the right side for an orthographic camera
         * In scene unit
         */
        private _orthoRight;
        set orthoRight(value: Nullable<number>);
        get orthoRight(): Nullable<number>;
        /**
         * Define the current limit on the bottom side for an orthographic camera
         * In scene unit
         */
        private _orthoBottom;
        set orthoBottom(value: Nullable<number>);
        get orthoBottom(): Nullable<number>;
        /**
         * Define the current limit on the top side for an orthographic camera
         * In scene unit
         */
        private _orthoTop;
        set orthoTop(value: Nullable<number>);
        get orthoTop(): Nullable<number>;
        /**
         * Field Of View is set in Radians. (default is 0.8)
         */
        fov: number;
        /**
         * Projection plane tilt around the X axis (horizontal), set in Radians. (default is 0)
         * Can be used to make vertical lines in world space actually vertical on the screen.
         * See https://forum.babylonjs.com/t/add-vertical-shift-to-3ds-max-exporter-babylon-cameras/17480
         */
        projectionPlaneTilt: number;
        /**
         * Define the minimum distance the camera can see from.
         * This is important to note that the depth buffer are not infinite and the closer it starts
         * the more your scene might encounter depth fighting issue.
         */
        minZ: number;
        /**
         * Define the maximum distance the camera can see to.
         * This is important to note that the depth buffer are not infinite and the further it end
         * the more your scene might encounter depth fighting issue.
         */
        maxZ: number;
        /**
         * Define the default inertia of the camera.
         * This helps giving a smooth feeling to the camera movement.
         */
        inertia: number;
        /**
         * Define the mode of the camera (Camera.PERSPECTIVE_CAMERA or Camera.ORTHOGRAPHIC_CAMERA)
         */
        private _mode;
        set mode(mode: number);
        get mode(): number;
        /**
         * Define whether the camera is intermediate.
         * This is useful to not present the output directly to the screen in case of rig without post process for instance
         */
        isIntermediate: boolean;
        /**
         * Define the viewport of the camera.
         * This correspond to the portion of the screen the camera will render to in normalized 0 to 1 unit.
         */
        viewport: Viewport;
        /**
         * Restricts the camera to viewing objects with the same layerMask.
         * A camera with a layerMask of 1 will render mesh.layerMask & camera.layerMask!== 0
         */
        layerMask: number;
        /**
         * fovMode sets the camera frustum bounds to the viewport bounds. (default is FOVMODE_VERTICAL_FIXED)
         */
        fovMode: number;
        /**
         * Rig mode of the camera.
         * This is useful to create the camera with two "eyes" instead of one to create VR or stereoscopic scenes.
         * This is normally controlled byt the camera themselves as internal use.
         */
        cameraRigMode: number;
        /**
         * Defines the distance between both "eyes" in case of a RIG
         */
        interaxialDistance: number;
        /**
         * Defines if stereoscopic rendering is done side by side or over under.
         */
        isStereoscopicSideBySide: boolean;
        /**
         * Defines the list of custom render target which are rendered to and then used as the input to this camera's render. Eg. display another camera view on a TV in the main scene
         * This is pretty helpful if you wish to make a camera render to a texture you could reuse somewhere
         * else in the scene. (Eg. security camera)
         *
         * To change the final output target of the camera, camera.outputRenderTarget should be used instead (eg. webXR renders to a render target corresponding to an HMD)
         */
        customRenderTargets: RenderTargetTexture[];
        /**
         * When set, the camera will render to this render target instead of the default canvas
         *
         * If the desire is to use the output of a camera as a texture in the scene consider using camera.customRenderTargets instead
         */
        outputRenderTarget: Nullable<RenderTargetTexture>;
        /**
         * Observable triggered when the camera view matrix has changed.
         */
        onViewMatrixChangedObservable: Observable<Camera>;
        /**
         * Observable triggered when the camera Projection matrix has changed.
         */
        onProjectionMatrixChangedObservable: Observable<Camera>;
        /**
         * Observable triggered when the inputs have been processed.
         */
        onAfterCheckInputsObservable: Observable<Camera>;
        /**
         * Observable triggered when reset has been called and applied to the camera.
         */
        onRestoreStateObservable: Observable<Camera>;
        /**
         * Is this camera a part of a rig system?
         */
        isRigCamera: boolean;
        /**
         * If isRigCamera set to true this will be set with the parent camera.
         * The parent camera is not (!) necessarily the .parent of this camera (like in the case of XR)
         */
        rigParent?: Camera;
        /**
         * Render pass id used by the camera to render into the main framebuffer
         */
        renderPassId: number;
        /** @internal */
        _cameraRigParams: any;
        /** @internal */
        _rigCameras: Camera[];
        /** @internal */
        _rigPostProcess: Nullable<PostProcess>;
        protected _webvrViewMatrix: Matrix;
        /** @internal */
        _skipRendering: boolean;
        /** @internal */
        _projectionMatrix: Matrix;
        /** @internal */
        _postProcesses: Nullable<PostProcess>[];
        /** @internal */
        _activeMeshes: SmartArray<AbstractMesh>;
        protected _globalPosition: Vector3;
        /** @internal */
        _computedViewMatrix: Matrix;
        private _doNotComputeProjectionMatrix;
        private _transformMatrix;
        private _frustumPlanes;
        private _refreshFrustumPlanes;
        private _storedFov;
        private _stateStored;
        private _absoluteRotation;
        /**
         * Instantiates a new camera object.
         * This should not be used directly but through the inherited cameras: ArcRotate, Free...
         * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras
         * @param name Defines the name of the camera in the scene
         * @param position Defines the position of the camera
         * @param scene Defines the scene the camera belongs too
         * @param setActiveOnSceneIfNoneActive Defines if the camera should be set as active after creation if no other camera have been defined in the scene
         */
        constructor(name: string, position: Vector3, scene?: Scene, setActiveOnSceneIfNoneActive?: boolean);
        /**
         * Store current camera state (fov, position, etc..)
         * @returns the camera
         */
        storeState(): Camera;
        /**
         * Restores the camera state values if it has been stored. You must call storeState() first
         */
        protected _restoreStateValues(): boolean;
        /**
         * Restored camera state. You must call storeState() first.
         * @returns true if restored and false otherwise
         */
        restoreState(): boolean;
        /**
         * Gets the class name of the camera.
         * @returns the class name
         */
        getClassName(): string;
        /** @internal */
        readonly _isCamera = true;
        /**
         * Gets a string representation of the camera useful for debug purpose.
         * @param fullDetails Defines that a more verbose level of logging is required
         * @returns the string representation
         */
        toString(fullDetails?: boolean): string;
        /**
         * Automatically tilts the projection plane, using `projectionPlaneTilt`, to correct the perspective effect on vertical lines.
         */
        applyVerticalCorrection(): void;
        /**
         * Gets the current world space position of the camera.
         */
        get globalPosition(): Vector3;
        /**
         * Gets the list of active meshes this frame (meshes no culled or excluded by lod s in the frame)
         * @returns the active meshe list
         */
        getActiveMeshes(): SmartArray<AbstractMesh>;
        /**
         * Check whether a mesh is part of the current active mesh list of the camera
         * @param mesh Defines the mesh to check
         * @returns true if active, false otherwise
         */
        isActiveMesh(mesh: Mesh): boolean;
        /**
         * Is this camera ready to be used/rendered
         * @param completeCheck defines if a complete check (including post processes) has to be done (false by default)
         * @returns true if the camera is ready
         */
        isReady(completeCheck?: boolean): boolean;
        /** @internal */
        _initCache(): void;
        /**
         * @internal
         */
        _updateCache(ignoreParentClass?: boolean): void;
        /** @internal */
        _isSynchronized(): boolean;
        /** @internal */
        _isSynchronizedViewMatrix(): boolean;
        /** @internal */
        _isSynchronizedProjectionMatrix(): boolean;
        /**
         * Attach the input controls to a specific dom element to get the input from.
         * @param noPreventDefault Defines whether event caught by the controls should call preventdefault() (https://developer.mozilla.org/en-US/docs/Web/API/Event/preventDefault)
         */
        attachControl(noPreventDefault?: boolean): void;
        /**
         * Attach the input controls to a specific dom element to get the input from.
         * @param ignored defines an ignored parameter kept for backward compatibility.
         * @param noPreventDefault Defines whether event caught by the controls should call preventdefault() (https://developer.mozilla.org/en-US/docs/Web/API/Event/preventDefault)
         * BACK COMPAT SIGNATURE ONLY.
         */
        attachControl(ignored: any, noPreventDefault?: boolean): void;
        /**
         * Detach the current controls from the specified dom element.
         */
        detachControl(): void;
        /**
         * Detach the current controls from the specified dom element.
         * @param ignored defines an ignored parameter kept for backward compatibility.
         */
        detachControl(ignored?: any): void;
        /**
         * Update the camera state according to the different inputs gathered during the frame.
         */
        update(): void;
        /** @internal */
        _checkInputs(): void;
        /** @internal */
        get rigCameras(): Camera[];
        /**
         * Gets the post process used by the rig cameras
         */
        get rigPostProcess(): Nullable<PostProcess>;
        /**
         * Internal, gets the first post process.
         * @returns the first post process to be run on this camera.
         */
        _getFirstPostProcess(): Nullable<PostProcess>;
        private _cascadePostProcessesToRigCams;
        /**
         * Attach a post process to the camera.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/postProcesses/usePostProcesses#attach-postprocess
         * @param postProcess The post process to attach to the camera
         * @param insertAt The position of the post process in case several of them are in use in the scene
         * @returns the position the post process has been inserted at
         */
        attachPostProcess(postProcess: PostProcess, insertAt?: Nullable<number>): number;
        /**
         * Detach a post process to the camera.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/postProcesses/usePostProcesses#attach-postprocess
         * @param postProcess The post process to detach from the camera
         */
        detachPostProcess(postProcess: PostProcess): void;
        /**
         * Gets the current world matrix of the camera
         */
        getWorldMatrix(): Matrix;
        /** @internal */
        _getViewMatrix(): Matrix;
        /**
         * Gets the current view matrix of the camera.
         * @param force forces the camera to recompute the matrix without looking at the cached state
         * @returns the view matrix
         */
        getViewMatrix(force?: boolean): Matrix;
        /**
         * Freeze the projection matrix.
         * It will prevent the cache check of the camera projection compute and can speed up perf
         * if no parameter of the camera are meant to change
         * @param projection Defines manually a projection if necessary
         */
        freezeProjectionMatrix(projection?: Matrix): void;
        /**
         * Unfreeze the projection matrix if it has previously been freezed by freezeProjectionMatrix.
         */
        unfreezeProjectionMatrix(): void;
        /**
         * Gets the current projection matrix of the camera.
         * @param force forces the camera to recompute the matrix without looking at the cached state
         * @returns the projection matrix
         */
        getProjectionMatrix(force?: boolean): Matrix;
        /**
         * Gets the transformation matrix (ie. the multiplication of view by projection matrices)
         * @returns a Matrix
         */
        getTransformationMatrix(): Matrix;
        private _updateFrustumPlanes;
        /**
         * Checks if a cullable object (mesh...) is in the camera frustum
         * This checks the bounding box center. See isCompletelyInFrustum for a full bounding check
         * @param target The object to check
         * @param checkRigCameras If the rig cameras should be checked (eg. with webVR camera both eyes should be checked) (Default: false)
         * @returns true if the object is in frustum otherwise false
         */
        isInFrustum(target: ICullable, checkRigCameras?: boolean): boolean;
        /**
         * Checks if a cullable object (mesh...) is in the camera frustum
         * Unlike isInFrustum this checks the full bounding box
         * @param target The object to check
         * @returns true if the object is in frustum otherwise false
         */
        isCompletelyInFrustum(target: ICullable): boolean;
        /**
         * Gets a ray in the forward direction from the camera.
         * @param length Defines the length of the ray to create
         * @param transform Defines the transform to apply to the ray, by default the world matrix is used to create a workd space ray
         * @param origin Defines the start point of the ray which defaults to the camera position
         * @returns the forward ray
         */
        getForwardRay(length?: number, transform?: Matrix, origin?: Vector3): Ray;
        /**
         * Gets a ray in the forward direction from the camera.
         * @param refRay the ray to (re)use when setting the values
         * @param length Defines the length of the ray to create
         * @param transform Defines the transform to apply to the ray, by default the world matrx is used to create a workd space ray
         * @param origin Defines the start point of the ray which defaults to the camera position
         * @returns the forward ray
         */
        getForwardRayToRef(refRay: Ray, length?: number, transform?: Matrix, origin?: Vector3): Ray;
        /**
         * Releases resources associated with this node.
         * @param doNotRecurse Set to true to not recurse into each children (recurse into each children by default)
         * @param disposeMaterialAndTextures Set to true to also dispose referenced materials and textures (false by default)
         */
        dispose(doNotRecurse?: boolean, disposeMaterialAndTextures?: boolean): void;
        /** @internal */
        _isLeftCamera: boolean;
        /**
         * Gets the left camera of a rig setup in case of Rigged Camera
         */
        get isLeftCamera(): boolean;
        /** @internal */
        _isRightCamera: boolean;
        /**
         * Gets the right camera of a rig setup in case of Rigged Camera
         */
        get isRightCamera(): boolean;
        /**
         * Gets the left camera of a rig setup in case of Rigged Camera
         */
        get leftCamera(): Nullable<FreeCamera>;
        /**
         * Gets the right camera of a rig setup in case of Rigged Camera
         */
        get rightCamera(): Nullable<FreeCamera>;
        /**
         * Gets the left camera target of a rig setup in case of Rigged Camera
         * @returns the target position
         */
        getLeftTarget(): Nullable<Vector3>;
        /**
         * Gets the right camera target of a rig setup in case of Rigged Camera
         * @returns the target position
         */
        getRightTarget(): Nullable<Vector3>;
        /**
         * @internal
         */
        setCameraRigMode(mode: number, rigParams: any): void;
        protected _setRigMode(rigParams: any): void;
        /** @internal */
        _getVRProjectionMatrix(): Matrix;
        protected _updateCameraRotationMatrix(): void;
        protected _updateWebVRCameraRotationMatrix(): void;
        /**
         * This function MUST be overwritten by the different WebVR cameras available.
         * The context in which it is running is the RIG camera. So 'this' is the TargetCamera, left or right.
         * @internal
         */
        _getWebVRProjectionMatrix(): Matrix;
        /**
         * This function MUST be overwritten by the different WebVR cameras available.
         * The context in which it is running is the RIG camera. So 'this' is the TargetCamera, left or right.
         * @internal
         */
        _getWebVRViewMatrix(): Matrix;
        /**
         * @internal
         */
        setCameraRigParameter(name: string, value: any): void;
        /**
         * needs to be overridden by children so sub has required properties to be copied
         * @internal
         */
        createRigCamera(name: string, cameraIndex: number): Nullable<Camera>;
        /**
         * May need to be overridden by children
         * @internal
         */
        _updateRigCameras(): void;
        /** @internal */
        _setupInputs(): void;
        /**
         * Serialiaze the camera setup to a json representation
         * @returns the JSON representation
         */
        serialize(): any;
        /**
         * Clones the current camera.
         * @param name The cloned camera name
         * @param newParent The cloned camera's new parent (none by default)
         * @returns the cloned camera
         */
        clone(name: string, newParent?: Nullable<Node>): Camera;
        /**
         * Gets the direction of the camera relative to a given local axis.
         * @param localAxis Defines the reference axis to provide a relative direction.
         * @returns the direction
         */
        getDirection(localAxis: Vector3): Vector3;
        /**
         * Returns the current camera absolute rotation
         */
        get absoluteRotation(): Quaternion;
        /**
         * Gets the direction of the camera relative to a given local axis into a passed vector.
         * @param localAxis Defines the reference axis to provide a relative direction.
         * @param result Defines the vector to store the result in
         */
        getDirectionToRef(localAxis: Vector3, result: Vector3): void;
        /**
         * Gets a camera constructor for a given camera type
         * @param type The type of the camera to construct (should be equal to one of the camera class name)
         * @param name The name of the camera the result will be able to instantiate
         * @param scene The scene the result will construct the camera in
         * @param interaxial_distance In case of stereoscopic setup, the distance between both eyes
         * @param isStereoscopicSideBySide In case of stereoscopic setup, should the sereo be side b side
         * @returns a factory method to construct the camera
         */
        static GetConstructorFromName(type: string, name: string, scene: Scene, interaxial_distance?: number, isStereoscopicSideBySide?: boolean): () => Camera;
        /**
         * Compute the world  matrix of the camera.
         * @returns the camera world matrix
         */
        computeWorldMatrix(): Matrix;
        /**
         * Parse a JSON and creates the camera from the parsed information
         * @param parsedCamera The JSON to parse
         * @param scene The scene to instantiate the camera in
         * @returns the newly constructed camera
         */
        static Parse(parsedCamera: any, scene: Scene): Camera;
    }


    /**
     * @ignore
     * This is a list of all the different input types that are available in the application.
     * Fo instance: ArcRotateCameraGamepadInput...
     */
    export var CameraInputTypes: {};
    /**
     * This is the contract to implement in order to create a new input class.
     * Inputs are dealing with listening to user actions and moving the camera accordingly.
     */
    export interface ICameraInput<TCamera extends Camera> {
        /**
         * Defines the camera the input is attached to.
         */
        camera: Nullable<TCamera>;
        /**
         * Gets the class name of the current input.
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Get the friendly name associated with the input class.
         * @returns the input friendly name
         */
        getSimpleName(): string;
        /**
         * Attach the input controls to a specific dom element to get the input from.
         * @param noPreventDefault Defines whether event caught by the controls should call preventdefault() (https://developer.mozilla.org/en-US/docs/Web/API/Event/preventDefault)
         */
        attachControl(noPreventDefault?: boolean): void;
        /**
         * Detach the current controls from the specified dom element.
         */
        detachControl(): void;
        /**
         * Update the current camera state depending on the inputs that have been used this frame.
         * This is a dynamically created lambda to avoid the performance penalty of looping for inputs in the render loop.
         */
        checkInputs?: () => void;
    }
    /**
     * Represents a map of input types to input instance or input index to input instance.
     */
    export interface CameraInputsMap<TCamera extends Camera> {
        /**
         * Accessor to the input by input type.
         */
        [name: string]: ICameraInput<TCamera>;
        /**
         * Accessor to the input by input index.
         */
        [idx: number]: ICameraInput<TCamera>;
    }
    /**
     * This represents the input manager used within a camera.
     * It helps dealing with all the different kind of input attached to a camera.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/customizingCameraInputs
     */
    export class CameraInputsManager<TCamera extends Camera> {
        /**
         * Defines the list of inputs attached to the camera.
         */
        attached: CameraInputsMap<TCamera>;
        /**
         * Defines the dom element the camera is collecting inputs from.
         * This is null if the controls have not been attached.
         */
        attachedToElement: boolean;
        /**
         * Defines whether event caught by the controls should call preventdefault() (https://developer.mozilla.org/en-US/docs/Web/API/Event/preventDefault)
         */
        noPreventDefault: boolean;
        /**
         * Defined the camera the input manager belongs to.
         */
        camera: TCamera;
        /**
         * Update the current camera state depending on the inputs that have been used this frame.
         * This is a dynamically created lambda to avoid the performance penalty of looping for inputs in the render loop.
         */
        checkInputs: () => void;
        /**
         * Instantiate a new Camera Input Manager.
         * @param camera Defines the camera the input manager belongs to
         */
        constructor(camera: TCamera);
        /**
         * Add an input method to a camera
         * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/customizingCameraInputs
         * @param input Camera input method
         */
        add(input: ICameraInput<TCamera>): void;
        /**
         * Remove a specific input method from a camera
         * example: camera.inputs.remove(camera.inputs.attached.mouse);
         * @param inputToRemove camera input method
         */
        remove(inputToRemove: ICameraInput<TCamera>): void;
        /**
         * Remove a specific input type from a camera
         * example: camera.inputs.remove("ArcRotateCameraGamepadInput");
         * @param inputType the type of the input to remove
         */
        removeByType(inputType: string): void;
        private _addCheckInputs;
        /**
         * Attach the input controls to the currently attached dom element to listen the events from.
         * @param input Defines the input to attach
         */
        attachInput(input: ICameraInput<TCamera>): void;
        /**
         * Attach the current manager inputs controls to a specific dom element to listen the events from.
         * @param noPreventDefault Defines whether event caught by the controls should call preventdefault() (https://developer.mozilla.org/en-US/docs/Web/API/Event/preventDefault)
         */
        attachElement(noPreventDefault?: boolean): void;
        /**
         * Detach the current manager inputs controls from a specific dom element.
         * @param disconnect Defines whether the input should be removed from the current list of attached inputs
         */
        detachElement(disconnect?: boolean): void;
        /**
         * Rebuild the dynamic inputCheck function from the current list of
         * defined inputs in the manager.
         */
        rebuildInputCheck(): void;
        /**
         * Remove all attached input methods from a camera
         */
        clear(): void;
        /**
         * Serialize the current input manager attached to a camera.
         * This ensures than once parsed,
         * the input associated to the camera will be identical to the current ones
         * @param serializedCamera Defines the camera serialization JSON the input serialization should write to
         */
        serialize(serializedCamera: any): void;
        /**
         * Parses an input manager serialized JSON to restore the previous list of inputs
         * and states associated to a camera.
         * @param parsedCamera Defines the JSON to parse
         */
        parse(parsedCamera: any): void;
    }


    /**
     * This is a camera specifically designed to react to device orientation events such as a modern mobile device
     * being tilted forward or back and left or right.
     */
    export class DeviceOrientationCamera extends FreeCamera {
        private _initialQuaternion;
        private _quaternionCache;
        private _tmpDragQuaternion;
        private _disablePointerInputWhenUsingDeviceOrientation;
        /**
         * Creates a new device orientation camera
         * @param name The name of the camera
         * @param position The start position camera
         * @param scene The scene the camera belongs to
         */
        constructor(name: string, position: Vector3, scene?: Scene);
        /**
         * Gets or sets a boolean indicating that pointer input must be disabled on first orientation sensor update (Default: true)
         */
        get disablePointerInputWhenUsingDeviceOrientation(): boolean;
        set disablePointerInputWhenUsingDeviceOrientation(value: boolean);
        private _dragFactor;
        /**
         * Enabled turning on the y axis when the orientation sensor is active
         * @param dragFactor the factor that controls the turn speed (default: 1/300)
         */
        enableHorizontalDragging(dragFactor?: number): void;
        /**
         * Gets the current instance class name ("DeviceOrientationCamera").
         * This helps avoiding instanceof at run time.
         * @returns the class name
         */
        getClassName(): string;
        /**
         * @internal
         * Checks and applies the current values of the inputs to the camera. (Internal use only)
         */
        _checkInputs(): void;
        /**
         * Reset the camera to its default orientation on the specified axis only.
         * @param axis The axis to reset
         */
        resetToCurrentRotation(axis?: Axis): void;
    }


    /**
     * This is a flying camera, designed for 3D movement and rotation in all directions,
     * such as in a 3D Space Shooter or a Flight Simulator.
     */
    export class FlyCamera extends TargetCamera {
        /**
         * Define the collision ellipsoid of the camera.
         * This is helpful for simulating a camera body, like a player's body.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_collisions#arcrotatecamera
         */
        ellipsoid: Vector3;
        /**
         * Define an offset for the position of the ellipsoid around the camera.
         * This can be helpful if the camera is attached away from the player's body center,
         * such as at its head.
         */
        ellipsoidOffset: Vector3;
        /**
         * Enable or disable collisions of the camera with the rest of the scene objects.
         */
        checkCollisions: boolean;
        /**
         * Enable or disable gravity on the camera.
         */
        applyGravity: boolean;
        /**
         * Define the current direction the camera is moving to.
         */
        cameraDirection: Vector3;
        /**
         * Define the current local rotation of the camera as a quaternion to prevent Gimbal lock.
         * This overrides and empties cameraRotation.
         */
        rotationQuaternion: Quaternion;
        /**
         * Track Roll to maintain the wanted Rolling when looking around.
         */
        _trackRoll: number;
        /**
         * Slowly correct the Roll to its original value after a Pitch+Yaw rotation.
         */
        rollCorrect: number;
        /**
         * Mimic a banked turn, Rolling the camera when Yawing.
         * It's recommended to use rollCorrect = 10 for faster banking correction.
         */
        bankedTurn: boolean;
        /**
         * Limit in radians for how much Roll banking will add. (Default: 90°)
         */
        bankedTurnLimit: number;
        /**
         * Value of 0 disables the banked Roll.
         * Value of 1 is equal to the Yaw angle in radians.
         */
        bankedTurnMultiplier: number;
        /**
         * The inputs manager loads all the input sources, such as keyboard and mouse.
         */
        inputs: FlyCameraInputsManager;
        /**
         * Gets the input sensibility for mouse input.
         * Higher values reduce sensitivity.
         */
        get angularSensibility(): number;
        /**
         * Sets the input sensibility for a mouse input.
         * Higher values reduce sensitivity.
         */
        set angularSensibility(value: number);
        /**
         * Get the keys for camera movement forward.
         */
        get keysForward(): number[];
        /**
         * Set the keys for camera movement forward.
         */
        set keysForward(value: number[]);
        /**
         * Get the keys for camera movement backward.
         */
        get keysBackward(): number[];
        set keysBackward(value: number[]);
        /**
         * Get the keys for camera movement up.
         */
        get keysUp(): number[];
        /**
         * Set the keys for camera movement up.
         */
        set keysUp(value: number[]);
        /**
         * Get the keys for camera movement down.
         */
        get keysDown(): number[];
        /**
         * Set the keys for camera movement down.
         */
        set keysDown(value: number[]);
        /**
         * Get the keys for camera movement left.
         */
        get keysLeft(): number[];
        /**
         * Set the keys for camera movement left.
         */
        set keysLeft(value: number[]);
        /**
         * Set the keys for camera movement right.
         */
        get keysRight(): number[];
        /**
         * Set the keys for camera movement right.
         */
        set keysRight(value: number[]);
        /**
         * Event raised when the camera collides with a mesh in the scene.
         */
        onCollide: (collidedMesh: AbstractMesh) => void;
        private _collider;
        private _needMoveForGravity;
        private _oldPosition;
        private _diffPosition;
        private _newPosition;
        /** @internal */
        _localDirection: Vector3;
        /** @internal */
        _transformedDirection: Vector3;
        /**
         * Instantiates a FlyCamera.
         * This is a flying camera, designed for 3D movement and rotation in all directions,
         * such as in a 3D Space Shooter or a Flight Simulator.
         * @param name Define the name of the camera in the scene.
         * @param position Define the starting position of the camera in the scene.
         * @param scene Define the scene the camera belongs to.
         * @param setActiveOnSceneIfNoneActive Defines whether the camera should be marked as active, if no other camera has been defined as active.
         */
        constructor(name: string, position: Vector3, scene?: Scene, setActiveOnSceneIfNoneActive?: boolean);
        /**
         * Attach the input controls to a specific dom element to get the input from.
         * @param noPreventDefault Defines whether event caught by the controls should call preventdefault() (https://developer.mozilla.org/en-US/docs/Web/API/Event/preventDefault)
         */
        attachControl(noPreventDefault?: boolean): void;
        /**
         * Detach a control from the HTML DOM element.
         * The camera will stop reacting to that input.
         */
        detachControl(): void;
        private _collisionMask;
        /**
         * Get the mask that the camera ignores in collision events.
         */
        get collisionMask(): number;
        /**
         * Set the mask that the camera ignores in collision events.
         */
        set collisionMask(mask: number);
        /**
         * @internal
         */
        _collideWithWorld(displacement: Vector3): void;
        /**
         * @internal
         */
        private _onCollisionPositionChange;
        /** @internal */
        _checkInputs(): void;
        /** @internal */
        _decideIfNeedsToMove(): boolean;
        /** @internal */
        _updatePosition(): void;
        /**
         * Restore the Roll to its target value at the rate specified.
         * @param rate - Higher means slower restoring.
         * @internal
         */
        restoreRoll(rate: number): void;
        /**
         * Destroy the camera and release the current resources held by it.
         */
        dispose(): void;
        /**
         * Get the current object class name.
         * @returns the class name.
         */
        getClassName(): string;
    }


    /**
     * Default Inputs manager for the FlyCamera.
     * It groups all the default supported inputs for ease of use.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/customizingCameraInputs
     */
    export class FlyCameraInputsManager extends CameraInputsManager<FlyCamera> {
        /**
         * Instantiates a new FlyCameraInputsManager.
         * @param camera Defines the camera the inputs belong to.
         */
        constructor(camera: FlyCamera);
        /**
         * Add keyboard input support to the input manager.
         * @returns the new FlyCameraKeyboardMoveInput().
         */
        addKeyboard(): FlyCameraInputsManager;
        /**
         * Add mouse input support to the input manager.
         * @returns the new FlyCameraMouseInput().
         */
        addMouse(): FlyCameraInputsManager;
    }


    /**
     * A follow camera takes a mesh as a target and follows it as it moves. Both a free camera version followCamera and
     * an arc rotate version arcFollowCamera are available.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_introduction#followcamera
     */
    export class FollowCamera extends TargetCamera {
        /**
         * Distance the follow camera should follow an object at
         */
        radius: number;
        /**
         * Minimum allowed distance of the camera to the axis of rotation
         * (The camera can not get closer).
         * This can help limiting how the Camera is able to move in the scene.
         */
        lowerRadiusLimit: Nullable<number>;
        /**
         * Maximum allowed distance of the camera to the axis of rotation
         * (The camera can not get further).
         * This can help limiting how the Camera is able to move in the scene.
         */
        upperRadiusLimit: Nullable<number>;
        /**
         * Define a rotation offset between the camera and the object it follows
         */
        rotationOffset: number;
        /**
         * Minimum allowed angle to camera position relative to target object.
         * This can help limiting how the Camera is able to move in the scene.
         */
        lowerRotationOffsetLimit: Nullable<number>;
        /**
         * Maximum allowed angle to camera position relative to target object.
         * This can help limiting how the Camera is able to move in the scene.
         */
        upperRotationOffsetLimit: Nullable<number>;
        /**
         * Define a height offset between the camera and the object it follows.
         * It can help following an object from the top (like a car chasing a plane)
         */
        heightOffset: number;
        /**
         * Minimum allowed height of camera position relative to target object.
         * This can help limiting how the Camera is able to move in the scene.
         */
        lowerHeightOffsetLimit: Nullable<number>;
        /**
         * Maximum allowed height of camera position relative to target object.
         * This can help limiting how the Camera is able to move in the scene.
         */
        upperHeightOffsetLimit: Nullable<number>;
        /**
         * Define how fast the camera can accelerate to follow it s target.
         */
        cameraAcceleration: number;
        /**
         * Define the speed limit of the camera following an object.
         */
        maxCameraSpeed: number;
        /**
         * Define the target of the camera.
         */
        lockedTarget: Nullable<AbstractMesh>;
        /**
         * Defines the input associated with the camera.
         */
        inputs: FollowCameraInputsManager;
        /**
         * Instantiates the follow camera.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_introduction#followcamera
         * @param name Define the name of the camera in the scene
         * @param position Define the position of the camera
         * @param scene Define the scene the camera belong to
         * @param lockedTarget Define the target of the camera
         */
        constructor(name: string, position: Vector3, scene?: Scene, lockedTarget?: Nullable<AbstractMesh>);
        private _follow;
        /**
         * Attach the input controls to a specific dom element to get the input from.
         * @param noPreventDefault Defines whether event caught by the controls should call preventdefault() (https://developer.mozilla.org/en-US/docs/Web/API/Event/preventDefault)
         */
        attachControl(noPreventDefault?: boolean): void;
        /**
         * Detach the current controls from the specified dom element.
         */
        detachControl(): void;
        /** @internal */
        _checkInputs(): void;
        private _checkLimits;
        /**
         * Gets the camera class name.
         * @returns the class name
         */
        getClassName(): string;
    }
    /**
     * Arc Rotate version of the follow camera.
     * It still follows a Defined mesh but in an Arc Rotate Camera fashion.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_introduction#followcamera
     */
    export class ArcFollowCamera extends TargetCamera {
        /** The longitudinal angle of the camera */
        alpha: number;
        /** The latitudinal angle of the camera */
        beta: number;
        /** The radius of the camera from its target */
        radius: number;
        private _cartesianCoordinates;
        /** Define the camera target (the mesh it should follow) */
        private _meshTarget;
        /**
         * Instantiates a new ArcFollowCamera
         * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_introduction#followcamera
         * @param name Define the name of the camera
         * @param alpha Define the rotation angle of the camera around the longitudinal axis
         * @param beta Define the rotation angle of the camera around the elevation axis
         * @param radius Define the radius of the camera from its target point
         * @param target Define the target of the camera
         * @param scene Define the scene the camera belongs to
         */
        constructor(name: string, 
        /** The longitudinal angle of the camera */
        alpha: number, 
        /** The latitudinal angle of the camera */
        beta: number, 
        /** The radius of the camera from its target */
        radius: number, 
        /** Define the camera target (the mesh it should follow) */
        target: Nullable<AbstractMesh>, scene: Scene);
        /**
         * Sets the mesh to follow with this camera.
         * @param target the target to follow
         */
        setMeshTarget(target: Nullable<AbstractMesh>): void;
        private _follow;
        /** @internal */
        _checkInputs(): void;
        /**
         * Returns the class name of the object.
         * It is mostly used internally for serialization purposes.
         */
        getClassName(): string;
    }


    /**
     * Default Inputs manager for the FollowCamera.
     * It groups all the default supported inputs for ease of use.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/customizingCameraInputs
     */
    export class FollowCameraInputsManager extends CameraInputsManager<FollowCamera> {
        /**
         * Instantiates a new FollowCameraInputsManager.
         * @param camera Defines the camera the inputs belong to
         */
        constructor(camera: FollowCamera);
        /**
         * Add keyboard input support to the input manager.
         * @returns the current input manager
         */
        addKeyboard(): FollowCameraInputsManager;
        /**
         * Add mouse wheel input support to the input manager.
         * @returns the current input manager
         */
        addMouseWheel(): FollowCameraInputsManager;
        /**
         * Add pointers input support to the input manager.
         * @returns the current input manager
         */
        addPointers(): FollowCameraInputsManager;
        /**
         * Add orientation input support to the input manager.
         * @returns the current input manager
         */
        addVRDeviceOrientation(): FollowCameraInputsManager;
    }


    /**
     * This represents a free type of camera. It can be useful in First Person Shooter game for instance.
     * Please consider using the new UniversalCamera instead as it adds more functionality like the gamepad.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_introduction#universal-camera
     */
    export class FreeCamera extends TargetCamera {
        /**
         * Define the collision ellipsoid of the camera.
         * This is helpful to simulate a camera body like the player body around the camera
         * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_collisions#arcrotatecamera
         */
        ellipsoid: Vector3;
        /**
         * Define an offset for the position of the ellipsoid around the camera.
         * This can be helpful to determine the center of the body near the gravity center of the body
         * instead of its head.
         */
        ellipsoidOffset: Vector3;
        /**
         * Enable or disable collisions of the camera with the rest of the scene objects.
         */
        checkCollisions: boolean;
        /**
         * Enable or disable gravity on the camera.
         */
        applyGravity: boolean;
        /**
         * Define the input manager associated to the camera.
         */
        inputs: FreeCameraInputsManager;
        /**
         * Gets the input sensibility for a mouse input. (default is 2000.0)
         * Higher values reduce sensitivity.
         */
        get angularSensibility(): number;
        /**
         * Sets the input sensibility for a mouse input. (default is 2000.0)
         * Higher values reduce sensitivity.
         */
        set angularSensibility(value: number);
        /**
         * Gets or Set the list of keyboard keys used to control the forward move of the camera.
         */
        get keysUp(): number[];
        set keysUp(value: number[]);
        /**
         * Gets or Set the list of keyboard keys used to control the upward move of the camera.
         */
        get keysUpward(): number[];
        set keysUpward(value: number[]);
        /**
         * Gets or Set the list of keyboard keys used to control the backward move of the camera.
         */
        get keysDown(): number[];
        set keysDown(value: number[]);
        /**
         * Gets or Set the list of keyboard keys used to control the downward move of the camera.
         */
        get keysDownward(): number[];
        set keysDownward(value: number[]);
        /**
         * Gets or Set the list of keyboard keys used to control the left strafe move of the camera.
         */
        get keysLeft(): number[];
        set keysLeft(value: number[]);
        /**
         * Gets or Set the list of keyboard keys used to control the right strafe move of the camera.
         */
        get keysRight(): number[];
        set keysRight(value: number[]);
        /**
         * Gets or Set the list of keyboard keys used to control the left rotation move of the camera.
         */
        get keysRotateLeft(): number[];
        set keysRotateLeft(value: number[]);
        /**
         * Gets or Set the list of keyboard keys used to control the right rotation move of the camera.
         */
        get keysRotateRight(): number[];
        set keysRotateRight(value: number[]);
        /**
         * Event raised when the camera collide with a mesh in the scene.
         */
        onCollide: (collidedMesh: AbstractMesh) => void;
        private _collider;
        private _needMoveForGravity;
        private _oldPosition;
        private _diffPosition;
        private _newPosition;
        /** @internal */
        _localDirection: Vector3;
        /** @internal */
        _transformedDirection: Vector3;
        /**
         * Instantiates a Free Camera.
         * This represents a free type of camera. It can be useful in First Person Shooter game for instance.
         * Please consider using the new UniversalCamera instead as it adds more functionality like touch to this camera.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_introduction#universal-camera
         * @param name Define the name of the camera in the scene
         * @param position Define the start position of the camera in the scene
         * @param scene Define the scene the camera belongs to
         * @param setActiveOnSceneIfNoneActive Defines whether the camera should be marked as active if not other active cameras have been defined
         */
        constructor(name: string, position: Vector3, scene?: Scene, setActiveOnSceneIfNoneActive?: boolean);
        /**
         * Attach the input controls to a specific dom element to get the input from.
         * @param noPreventDefault Defines whether event caught by the controls should call preventdefault() (https://developer.mozilla.org/en-US/docs/Web/API/Event/preventDefault)
         */
        attachControl(noPreventDefault?: boolean): void;
        /**
         * Attach the input controls to a specific dom element to get the input from.
         * @param ignored defines an ignored parameter kept for backward compatibility.
         * @param noPreventDefault Defines whether event caught by the controls should call preventdefault() (https://developer.mozilla.org/en-US/docs/Web/API/Event/preventDefault)
         * BACK COMPAT SIGNATURE ONLY.
         */
        attachControl(ignored: any, noPreventDefault?: boolean): void;
        /**
         * Detach the current controls from the specified dom element.
         */
        detachControl(): void;
        private _collisionMask;
        /**
         * Define a collision mask to limit the list of object the camera can collide with
         */
        get collisionMask(): number;
        set collisionMask(mask: number);
        /**
         * @internal
         */
        _collideWithWorld(displacement: Vector3): void;
        private _onCollisionPositionChange;
        /** @internal */
        _checkInputs(): void;
        /** @internal */
        _decideIfNeedsToMove(): boolean;
        /** @internal */
        _updatePosition(): void;
        /**
         * Destroy the camera and release the current resources hold by it.
         */
        dispose(): void;
        /**
         * Gets the current object class name.
         * @returns the class name
         */
        getClassName(): string;
    }


    /**
     * Default Inputs manager for the FreeCamera.
     * It groups all the default supported inputs for ease of use.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/customizingCameraInputs
     */
    export class FreeCameraInputsManager extends CameraInputsManager<FreeCamera> {
        /**
         * @internal
         */
        _mouseInput: Nullable<FreeCameraMouseInput>;
        /**
         * @internal
         */
        _mouseWheelInput: Nullable<FreeCameraMouseWheelInput>;
        /**
         * Instantiates a new FreeCameraInputsManager.
         * @param camera Defines the camera the inputs belong to
         */
        constructor(camera: FreeCamera);
        /**
         * Add keyboard input support to the input manager.
         * @returns the current input manager
         */
        addKeyboard(): FreeCameraInputsManager;
        /**
         * Add mouse input support to the input manager.
         * @param touchEnabled if the FreeCameraMouseInput should support touch (default: true)
         * @returns the current input manager
         */
        addMouse(touchEnabled?: boolean): FreeCameraInputsManager;
        /**
         * Removes the mouse input support from the manager
         * @returns the current input manager
         */
        removeMouse(): FreeCameraInputsManager;
        /**
         * Add mouse wheel input support to the input manager.
         * @returns the current input manager
         */
        addMouseWheel(): FreeCameraInputsManager;
        /**
         * Removes the mouse wheel input support from the manager
         * @returns the current input manager
         */
        removeMouseWheel(): FreeCameraInputsManager;
        /**
         * Add touch input support to the input manager.
         * @returns the current input manager
         */
        addTouch(): FreeCameraInputsManager;
        /**
         * Remove all attached input methods from a camera
         */
        clear(): void;
    }


    /**
     * This represents a FPS type of camera. This is only here for back compat purpose.
     * Please use the UniversalCamera instead as both are identical.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_introduction#universal-camera
     */
    export class GamepadCamera extends UniversalCamera {
        /**
         * Instantiates a new Gamepad Camera
         * This represents a FPS type of camera. This is only here for back compat purpose.
         * Please use the UniversalCamera instead as both are identical.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_introduction#universal-camera
         * @param name Define the name of the camera in the scene
         * @param position Define the start position of the camera in the scene
         * @param scene Define the scene the camera belongs to
         */
        constructor(name: string, position: Vector3, scene?: Scene);
        /**
         * Gets the current object class name.
         * @returns the class name
         */
        getClassName(): string;
    }




    /**
     * Manage the gamepad inputs to control an arc rotate camera.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/customizingCameraInputs
     */
    export class ArcRotateCameraGamepadInput implements ICameraInput<ArcRotateCamera> {
        /**
         * Defines the camera the input is attached to.
         */
        camera: ArcRotateCamera;
        /**
         * Defines the gamepad the input is gathering event from.
         */
        gamepad: Nullable<Gamepad>;
        /**
         * Defines the gamepad rotation sensibility.
         * This is the threshold from when rotation starts to be accounted for to prevent jittering.
         */
        gamepadRotationSensibility: number;
        /**
         * Defines the gamepad move sensibility.
         * This is the threshold from when moving starts to be accounted for for to prevent jittering.
         */
        gamepadMoveSensibility: number;
        private _yAxisScale;
        /**
         * Gets or sets a boolean indicating that Yaxis (for right stick) should be inverted
         */
        get invertYAxis(): boolean;
        set invertYAxis(value: boolean);
        private _onGamepadConnectedObserver;
        private _onGamepadDisconnectedObserver;
        /**
         * Attach the input controls to a specific dom element to get the input from.
         */
        attachControl(): void;
        /**
         * Detach the current controls from the specified dom element.
         */
        detachControl(): void;
        /**
         * Update the current camera state depending on the inputs that have been used this frame.
         * This is a dynamically created lambda to avoid the performance penalty of looping for inputs in the render loop.
         */
        checkInputs(): void;
        /**
         * Gets the class name of the current intput.
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Get the friendly name associated with the input class.
         * @returns the input friendly name
         */
        getSimpleName(): string;
    }


    /**
     * Manage the keyboard inputs to control the movement of an arc rotate camera.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/customizingCameraInputs
     */
    export class ArcRotateCameraKeyboardMoveInput implements ICameraInput<ArcRotateCamera> {
        /**
         * Defines the camera the input is attached to.
         */
        camera: ArcRotateCamera;
        /**
         * Defines the list of key codes associated with the up action (increase alpha)
         */
        keysUp: number[];
        /**
         * Defines the list of key codes associated with the down action (decrease alpha)
         */
        keysDown: number[];
        /**
         * Defines the list of key codes associated with the left action (increase beta)
         */
        keysLeft: number[];
        /**
         * Defines the list of key codes associated with the right action (decrease beta)
         */
        keysRight: number[];
        /**
         * Defines the list of key codes associated with the reset action.
         * Those keys reset the camera to its last stored state (with the method camera.storeState())
         */
        keysReset: number[];
        /**
         * Defines the panning sensibility of the inputs.
         * (How fast is the camera panning)
         */
        panningSensibility: number;
        /**
         * Defines the zooming sensibility of the inputs.
         * (How fast is the camera zooming)
         */
        zoomingSensibility: number;
        /**
         * Defines whether maintaining the alt key down switch the movement mode from
         * orientation to zoom.
         */
        useAltToZoom: boolean;
        /**
         * Rotation speed of the camera
         */
        angularSpeed: number;
        private _keys;
        private _ctrlPressed;
        private _altPressed;
        private _onCanvasBlurObserver;
        private _onKeyboardObserver;
        private _engine;
        private _scene;
        /**
         * Attach the input controls to a specific dom element to get the input from.
         * @param noPreventDefault Defines whether event caught by the controls should call preventdefault() (https://developer.mozilla.org/en-US/docs/Web/API/Event/preventDefault)
         */
        attachControl(noPreventDefault?: boolean): void;
        /**
         * Detach the current controls from the specified dom element.
         */
        detachControl(): void;
        /**
         * Update the current camera state depending on the inputs that have been used this frame.
         * This is a dynamically created lambda to avoid the performance penalty of looping for inputs in the render loop.
         */
        checkInputs(): void;
        /**
         * Gets the class name of the current input.
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Get the friendly name associated with the input class.
         * @returns the input friendly name
         */
        getSimpleName(): string;
    }


    /**
     * Manage the mouse wheel inputs to control an arc rotate camera.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/customizingCameraInputs
     */
    export class ArcRotateCameraMouseWheelInput implements ICameraInput<ArcRotateCamera> {
        /**
         * Defines the camera the input is attached to.
         */
        camera: ArcRotateCamera;
        /**
         * Gets or Set the mouse wheel precision or how fast is the camera zooming.
         */
        wheelPrecision: number;
        /**
         * Gets or Set the boolean value that controls whether or not the mouse wheel
         * zooms to the location of the mouse pointer or not.  The default is false.
         */
        zoomToMouseLocation: boolean;
        /**
         * wheelDeltaPercentage will be used instead of wheelPrecision if different from 0.
         * It defines the percentage of current camera.radius to use as delta when wheel is used.
         */
        wheelDeltaPercentage: number;
        /**
         * If set, this function will be used to set the radius delta that will be added to the current camera radius
         */
        customComputeDeltaFromMouseWheel: Nullable<(wheelDelta: number, input: ArcRotateCameraMouseWheelInput, event: IWheelEvent) => number>;
        private _wheel;
        private _observer;
        private _hitPlane;
        protected _computeDeltaFromMouseWheelLegacyEvent(mouseWheelDelta: number, radius: number): number;
        /**
         * Attach the input controls to a specific dom element to get the input from.
         * @param noPreventDefault Defines whether event caught by the controls should call preventdefault() (https://developer.mozilla.org/en-US/docs/Web/API/Event/preventDefault)
         */
        attachControl(noPreventDefault?: boolean): void;
        /**
         * Detach the current controls from the specified dom element.
         */
        detachControl(): void;
        /**
         * Update the current camera state depending on the inputs that have been used this frame.
         * This is a dynamically created lambda to avoid the performance penalty of looping for inputs in the render loop.
         */
        checkInputs(): void;
        /**
         * Gets the class name of the current input.
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Get the friendly name associated with the input class.
         * @returns the input friendly name
         */
        getSimpleName(): string;
        private _updateHitPlane;
        private _getPosition;
        private _inertialPanning;
        private _zoomToMouse;
        private _zeroIfClose;
    }


    /**
     * Manage the pointers inputs to control an arc rotate camera.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/customizingCameraInputs
     */
    export class ArcRotateCameraPointersInput extends BaseCameraPointersInput {
        /**
         * Defines the camera the input is attached to.
         */
        camera: ArcRotateCamera;
        /**
         * The minimum radius used for pinch, to avoid radius lock at 0
         */
        static MinimumRadiusForPinch: number;
        /**
         * Gets the class name of the current input.
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Defines the buttons associated with the input to handle camera move.
         */
        buttons: number[];
        /**
         * Defines the pointer angular sensibility  along the X axis or how fast is
         * the camera rotating.
         */
        angularSensibilityX: number;
        /**
         * Defines the pointer angular sensibility along the Y axis or how fast is
         * the camera rotating.
         */
        angularSensibilityY: number;
        /**
         * Defines the pointer pinch precision or how fast is the camera zooming.
         */
        pinchPrecision: number;
        /**
         * pinchDeltaPercentage will be used instead of pinchPrecision if different
         * from 0.
         * It defines the percentage of current camera.radius to use as delta when
         * pinch zoom is used.
         */
        pinchDeltaPercentage: number;
        /**
         * When useNaturalPinchZoom is true, multi touch zoom will zoom in such
         * that any object in the plane at the camera's target point will scale
         * perfectly with finger motion.
         * Overrides pinchDeltaPercentage and pinchPrecision.
         */
        useNaturalPinchZoom: boolean;
        /**
         * Defines whether zoom (2 fingers pinch) is enabled through multitouch
         */
        pinchZoom: boolean;
        /**
         * Defines the pointer panning sensibility or how fast is the camera moving.
         */
        panningSensibility: number;
        /**
         * Defines whether panning (2 fingers swipe) is enabled through multitouch.
         */
        multiTouchPanning: boolean;
        /**
         * Defines whether panning is enabled for both pan (2 fingers swipe) and
         * zoom (pinch) through multitouch.
         */
        multiTouchPanAndZoom: boolean;
        /**
         * Revers pinch action direction.
         */
        pinchInwards: boolean;
        private _isPanClick;
        private _twoFingerActivityCount;
        private _isPinching;
        /**
         * Move camera from multi touch panning positions.
         * @param previousMultiTouchPanPosition
         * @param multiTouchPanPosition
         */
        private _computeMultiTouchPanning;
        /**
         * Move camera from pinch zoom distances.
         * @param previousPinchSquaredDistance
         * @param pinchSquaredDistance
         */
        private _computePinchZoom;
        /**
         * Called on pointer POINTERMOVE event if only a single touch is active.
         * @param point
         * @param offsetX
         * @param offsetY
         */
        onTouch(point: Nullable<PointerTouch>, offsetX: number, offsetY: number): void;
        /**
         * Called on pointer POINTERDOUBLETAP event.
         */
        onDoubleTap(): void;
        /**
         * Called on pointer POINTERMOVE event if multiple touches are active.
         * @param pointA
         * @param pointB
         * @param previousPinchSquaredDistance
         * @param pinchSquaredDistance
         * @param previousMultiTouchPanPosition
         * @param multiTouchPanPosition
         */
        onMultiTouch(pointA: Nullable<PointerTouch>, pointB: Nullable<PointerTouch>, previousPinchSquaredDistance: number, pinchSquaredDistance: number, previousMultiTouchPanPosition: Nullable<PointerTouch>, multiTouchPanPosition: Nullable<PointerTouch>): void;
        /**
         * Called each time a new POINTERDOWN event occurs. Ie, for each button
         * press.
         * @param evt
         */
        onButtonDown(evt: IPointerEvent): void;
        /**
         * Called each time a new POINTERUP event occurs. Ie, for each button
         * release.
         */
        onButtonUp(): void;
        /**
         * Called when window becomes inactive.
         */
        onLostFocus(): void;
    }


        interface ArcRotateCameraInputsManager {
            /**
             * Add orientation input support to the input manager.
             * @returns the current input manager
             */
            addVRDeviceOrientation(): ArcRotateCameraInputsManager;
        }
    /**
     * Manage the device orientation inputs (gyroscope) to control an arc rotate camera.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/customizingCameraInputs
     */
    export class ArcRotateCameraVRDeviceOrientationInput implements ICameraInput<ArcRotateCamera> {
        /**
         * Defines the camera the input is attached to.
         */
        camera: ArcRotateCamera;
        /**
         * Defines a correction factor applied on the alpha value retrieved from the orientation events.
         */
        alphaCorrection: number;
        /**
         * Defines a correction factor applied on the gamma value retrieved from the orientation events.
         */
        gammaCorrection: number;
        private _alpha;
        private _gamma;
        private _dirty;
        private _deviceOrientationHandler;
        /**
         * Instantiate a new ArcRotateCameraVRDeviceOrientationInput.
         */
        constructor();
        /**
         * Attach the input controls to a specific dom element to get the input from.
         * @param noPreventDefault Defines whether event caught by the controls should call preventdefault() (https://developer.mozilla.org/en-US/docs/Web/API/Event/preventDefault)
         */
        attachControl(noPreventDefault?: boolean): void;
        /**
         * @internal
         */
        _onOrientationEvent(evt: DeviceOrientationEvent): void;
        /**
         * Update the current camera state depending on the inputs that have been used this frame.
         * This is a dynamically created lambda to avoid the performance penalty of looping for inputs in the render loop.
         */
        checkInputs(): void;
        /**
         * Detach the current controls from the specified dom element.
         */
        detachControl(): void;
        /**
         * Gets the class name of the current input.
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Get the friendly name associated with the input class.
         * @returns the input friendly name
         */
        getSimpleName(): string;
    }


    /**
     * Base class for mouse wheel input..
     * See FollowCameraMouseWheelInput in src/Cameras/Inputs/freeCameraMouseWheelInput.ts
     * for example usage.
     */
    export abstract class BaseCameraMouseWheelInput implements ICameraInput<Camera> {
        /**
         * Defines the camera the input is attached to.
         */
        abstract camera: Camera;
        /**
         * How fast is the camera moves in relation to X axis mouseWheel events.
         * Use negative value to reverse direction.
         */
        wheelPrecisionX: number;
        /**
         * How fast is the camera moves in relation to Y axis mouseWheel events.
         * Use negative value to reverse direction.
         */
        wheelPrecisionY: number;
        /**
         * How fast is the camera moves in relation to Z axis mouseWheel events.
         * Use negative value to reverse direction.
         */
        wheelPrecisionZ: number;
        /**
         * Observable for when a mouse wheel move event occurs.
         */
        onChangedObservable: Observable<{
            wheelDeltaX: number;
            wheelDeltaY: number;
            wheelDeltaZ: number;
        }>;
        private _wheel;
        private _observer;
        /**
         * Attach the input controls to a specific dom element to get the input from.
         * @param noPreventDefault Defines whether event caught by the controls
         *   should call preventdefault().
         *   (https://developer.mozilla.org/en-US/docs/Web/API/Event/preventDefault)
         */
        attachControl(noPreventDefault?: boolean): void;
        /**
         * Detach the current controls from the specified dom element.
         */
        detachControl(): void;
        /**
         * Called for each rendered frame.
         */
        checkInputs(): void;
        /**
         * Gets the class name of the current input.
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Get the friendly name associated with the input class.
         * @returns the input friendly name
         */
        getSimpleName(): string;
        /**
         * Incremental value of multiple mouse wheel movements of the X axis.
         * Should be zero-ed when read.
         */
        protected _wheelDeltaX: number;
        /**
         * Incremental value of multiple mouse wheel movements of the Y axis.
         * Should be zero-ed when read.
         */
        protected _wheelDeltaY: number;
        /**
         * Incremental value of multiple mouse wheel movements of the Z axis.
         * Should be zero-ed when read.
         */
        protected _wheelDeltaZ: number;
        /**
         * Firefox uses a different scheme to report scroll distances to other
         * browsers. Rather than use complicated methods to calculate the exact
         * multiple we need to apply, let's just cheat and use a constant.
         * https://developer.mozilla.org/en-US/docs/Web/API/WheelEvent/deltaMode
         * https://stackoverflow.com/questions/20110224/what-is-the-height-of-a-line-in-a-wheel-event-deltamode-dom-delta-line
         */
        private readonly _ffMultiplier;
        /**
         * Different event attributes for wheel data fall into a few set ranges.
         * Some relevant but dated date here:
         * https://stackoverflow.com/questions/5527601/normalizing-mousewheel-speed-across-browsers
         */
        private readonly _normalize;
    }


    /**
     * Base class for Camera Pointer Inputs.
     * See FollowCameraPointersInput in src/Cameras/Inputs/followCameraPointersInput.ts
     * for example usage.
     */
    export abstract class BaseCameraPointersInput implements ICameraInput<Camera> {
        /**
         * Defines the camera the input is attached to.
         */
        abstract camera: Camera;
        /**
         * Whether keyboard modifier keys are pressed at time of last mouse event.
         */
        protected _altKey: boolean;
        protected _ctrlKey: boolean;
        protected _metaKey: boolean;
        protected _shiftKey: boolean;
        /**
         * Which mouse buttons were pressed at time of last mouse event.
         * https://developer.mozilla.org/en-US/docs/Web/API/MouseEvent/buttons
         */
        protected _buttonsPressed: number;
        private _currentActiveButton;
        private _contextMenuBind;
        /**
         * Defines the buttons associated with the input to handle camera move.
         */
        buttons: number[];
        /**
         * Attach the input controls to a specific dom element to get the input from.
         * @param noPreventDefault Defines whether event caught by the controls should call preventdefault() (https://developer.mozilla.org/en-US/docs/Web/API/Event/preventDefault)
         */
        attachControl(noPreventDefault?: boolean): void;
        /**
         * Detach the current controls from the specified dom element.
         */
        detachControl(): void;
        /**
         * Gets the class name of the current input.
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Get the friendly name associated with the input class.
         * @returns the input friendly name
         */
        getSimpleName(): string;
        /**
         * Called on pointer POINTERDOUBLETAP event.
         * Override this method to provide functionality on POINTERDOUBLETAP event.
         * @param type
         */
        onDoubleTap(type: string): void;
        /**
         * Called on pointer POINTERMOVE event if only a single touch is active.
         * Override this method to provide functionality.
         * @param point
         * @param offsetX
         * @param offsetY
         */
        onTouch(point: Nullable<PointerTouch>, offsetX: number, offsetY: number): void;
        /**
         * Called on pointer POINTERMOVE event if multiple touches are active.
         * Override this method to provide functionality.
         * @param _pointA
         * @param _pointB
         * @param previousPinchSquaredDistance
         * @param pinchSquaredDistance
         * @param previousMultiTouchPanPosition
         * @param multiTouchPanPosition
         */
        onMultiTouch(_pointA: Nullable<PointerTouch>, _pointB: Nullable<PointerTouch>, previousPinchSquaredDistance: number, pinchSquaredDistance: number, previousMultiTouchPanPosition: Nullable<PointerTouch>, multiTouchPanPosition: Nullable<PointerTouch>): void;
        /**
         * Called on JS contextmenu event.
         * Override this method to provide functionality.
         * @param evt
         */
        onContextMenu(evt: PointerEvent): void;
        /**
         * Called each time a new POINTERDOWN event occurs. Ie, for each button
         * press.
         * Override this method to provide functionality.
         * @param evt
         */
        onButtonDown(evt: IPointerEvent): void;
        /**
         * Called each time a new POINTERUP event occurs. Ie, for each button
         * release.
         * Override this method to provide functionality.
         * @param evt
         */
        onButtonUp(evt: IPointerEvent): void;
        /**
         * Called when window becomes inactive.
         * Override this method to provide functionality.
         */
        onLostFocus(): void;
        private _pointerInput;
        private _observer;
        private _onLostFocus;
        private _pointA;
        private _pointB;
    }


    /**
     * Listen to keyboard events to control the camera.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/customizingCameraInputs
     */
    export class FlyCameraKeyboardInput implements ICameraInput<FlyCamera> {
        /**
         * Defines the camera the input is attached to.
         */
        camera: FlyCamera;
        /**
         * The list of keyboard keys used to control the forward move of the camera.
         */
        keysForward: number[];
        /**
         * The list of keyboard keys used to control the backward move of the camera.
         */
        keysBackward: number[];
        /**
         * The list of keyboard keys used to control the forward move of the camera.
         */
        keysUp: number[];
        /**
         * The list of keyboard keys used to control the backward move of the camera.
         */
        keysDown: number[];
        /**
         * The list of keyboard keys used to control the right strafe move of the camera.
         */
        keysRight: number[];
        /**
         * The list of keyboard keys used to control the left strafe move of the camera.
         */
        keysLeft: number[];
        private _keys;
        private _onCanvasBlurObserver;
        private _onKeyboardObserver;
        private _engine;
        private _scene;
        /**
         * Attach the input controls to a specific dom element to get the input from.
         * @param noPreventDefault Defines whether event caught by the controls should call preventdefault() (https://developer.mozilla.org/en-US/docs/Web/API/Event/preventDefault)
         */
        attachControl(noPreventDefault?: boolean): void;
        /**
         * Detach the current controls from the specified dom element.
         */
        detachControl(): void;
        /**
         * Gets the class name of the current input.
         * @returns the class name
         */
        getClassName(): string;
        /**
         * @internal
         */
        _onLostFocus(): void;
        /**
         * Get the friendly name associated with the input class.
         * @returns the input friendly name
         */
        getSimpleName(): string;
        /**
         * Update the current camera state depending on the inputs that have been used this frame.
         * This is a dynamically created lambda to avoid the performance penalty of looping for inputs in the render loop.
         */
        checkInputs(): void;
    }


    /**
     * Listen to mouse events to control the camera.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/customizingCameraInputs
     */
    export class FlyCameraMouseInput implements ICameraInput<FlyCamera> {
        /**
         * Defines the camera the input is attached to.
         */
        camera: FlyCamera;
        /**
         * Defines if touch is enabled. (Default is true.)
         */
        touchEnabled: boolean;
        /**
         * Defines the buttons associated with the input to handle camera rotation.
         */
        buttons: number[];
        /**
         * Assign buttons for Yaw control.
         */
        buttonsYaw: number[];
        /**
         * Assign buttons for Pitch control.
         */
        buttonsPitch: number[];
        /**
         * Assign buttons for Roll control.
         */
        buttonsRoll: number[];
        /**
         * Detect if any button is being pressed while mouse is moved.
         * -1 = Mouse locked.
         * 0 = Left button.
         * 1 = Middle Button.
         * 2 = Right Button.
         */
        activeButton: number;
        /**
         * Defines the pointer's angular sensibility, to control the camera rotation speed.
         * Higher values reduce its sensitivity.
         */
        angularSensibility: number;
        private _observer;
        private _rollObserver;
        private _previousPosition;
        private _noPreventDefault;
        private _element;
        /**
         * Listen to mouse events to control the camera.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/customizingCameraInputs
         */
        constructor();
        /**
         * Attach the mouse control to the HTML DOM element.
         * @param noPreventDefault Defines whether events caught by the controls should call preventdefault().
         */
        attachControl(noPreventDefault?: boolean): void;
        /**
         * Detach the current controls from the specified dom element.
         */
        detachControl(): void;
        /**
         * Gets the class name of the current input.
         * @returns the class name.
         */
        getClassName(): string;
        /**
         * Get the friendly name associated with the input class.
         * @returns the input's friendly name.
         */
        getSimpleName(): string;
        private _pointerInput;
        private _onMouseMove;
        /**
         * Rotate camera by mouse offset.
         * @param offsetX
         * @param offsetY
         */
        private _rotateCamera;
    }


    /**
     * Manage the keyboard inputs to control the movement of a follow camera.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/customizingCameraInputs
     */
    export class FollowCameraKeyboardMoveInput implements ICameraInput<FollowCamera> {
        /**
         * Defines the camera the input is attached to.
         */
        camera: FollowCamera;
        /**
         * Defines the list of key codes associated with the up action (increase heightOffset)
         */
        keysHeightOffsetIncr: number[];
        /**
         * Defines the list of key codes associated with the down action (decrease heightOffset)
         */
        keysHeightOffsetDecr: number[];
        /**
         * Defines whether the Alt modifier key is required to move up/down (alter heightOffset)
         */
        keysHeightOffsetModifierAlt: boolean;
        /**
         * Defines whether the Ctrl modifier key is required to move up/down (alter heightOffset)
         */
        keysHeightOffsetModifierCtrl: boolean;
        /**
         * Defines whether the Shift modifier key is required to move up/down (alter heightOffset)
         */
        keysHeightOffsetModifierShift: boolean;
        /**
         * Defines the list of key codes associated with the left action (increase rotationOffset)
         */
        keysRotationOffsetIncr: number[];
        /**
         * Defines the list of key codes associated with the right action (decrease rotationOffset)
         */
        keysRotationOffsetDecr: number[];
        /**
         * Defines whether the Alt modifier key is required to move left/right (alter rotationOffset)
         */
        keysRotationOffsetModifierAlt: boolean;
        /**
         * Defines whether the Ctrl modifier key is required to move left/right (alter rotationOffset)
         */
        keysRotationOffsetModifierCtrl: boolean;
        /**
         * Defines whether the Shift modifier key is required to move left/right (alter rotationOffset)
         */
        keysRotationOffsetModifierShift: boolean;
        /**
         * Defines the list of key codes associated with the zoom-in action (decrease radius)
         */
        keysRadiusIncr: number[];
        /**
         * Defines the list of key codes associated with the zoom-out action (increase radius)
         */
        keysRadiusDecr: number[];
        /**
         * Defines whether the Alt modifier key is required to zoom in/out (alter radius value)
         */
        keysRadiusModifierAlt: boolean;
        /**
         * Defines whether the Ctrl modifier key is required to zoom in/out (alter radius value)
         */
        keysRadiusModifierCtrl: boolean;
        /**
         * Defines whether the Shift modifier key is required to zoom in/out (alter radius value)
         */
        keysRadiusModifierShift: boolean;
        /**
         * Defines the rate of change of heightOffset.
         */
        heightSensibility: number;
        /**
         * Defines the rate of change of rotationOffset.
         */
        rotationSensibility: number;
        /**
         * Defines the rate of change of radius.
         */
        radiusSensibility: number;
        private _keys;
        private _ctrlPressed;
        private _altPressed;
        private _shiftPressed;
        private _onCanvasBlurObserver;
        private _onKeyboardObserver;
        private _engine;
        private _scene;
        /**
         * Attach the input controls to a specific dom element to get the input from.
         * @param noPreventDefault Defines whether event caught by the controls should call preventdefault() (https://developer.mozilla.org/en-US/docs/Web/API/Event/preventDefault)
         */
        attachControl(noPreventDefault?: boolean): void;
        /**
         * Detach the current controls from the specified dom element.
         */
        detachControl(): void;
        /**
         * Update the current camera state depending on the inputs that have been used this frame.
         * This is a dynamically created lambda to avoid the performance penalty of looping for inputs in the render loop.
         */
        checkInputs(): void;
        /**
         * Gets the class name of the current input.
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Get the friendly name associated with the input class.
         * @returns the input friendly name
         */
        getSimpleName(): string;
        /**
         * Check if the pressed modifier keys (Alt/Ctrl/Shift) match those configured to
         * allow modification of the heightOffset value.
         */
        private _modifierHeightOffset;
        /**
         * Check if the pressed modifier keys (Alt/Ctrl/Shift) match those configured to
         * allow modification of the rotationOffset value.
         */
        private _modifierRotationOffset;
        /**
         * Check if the pressed modifier keys (Alt/Ctrl/Shift) match those configured to
         * allow modification of the radius value.
         */
        private _modifierRadius;
    }


    /**
     * Manage the mouse wheel inputs to control a follow camera.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/customizingCameraInputs
     */
    export class FollowCameraMouseWheelInput implements ICameraInput<FollowCamera> {
        /**
         * Defines the camera the input is attached to.
         */
        camera: FollowCamera;
        /**
         * Moue wheel controls zoom. (Mouse wheel modifies camera.radius value.)
         */
        axisControlRadius: boolean;
        /**
         * Moue wheel controls height. (Mouse wheel modifies camera.heightOffset value.)
         */
        axisControlHeight: boolean;
        /**
         * Moue wheel controls angle. (Mouse wheel modifies camera.rotationOffset value.)
         */
        axisControlRotation: boolean;
        /**
         * Gets or Set the mouse wheel precision or how fast is the camera moves in
         * relation to mouseWheel events.
         */
        wheelPrecision: number;
        /**
         * wheelDeltaPercentage will be used instead of wheelPrecision if different from 0.
         * It defines the percentage of current camera.radius to use as delta when wheel is used.
         */
        wheelDeltaPercentage: number;
        private _wheel;
        private _observer;
        /**
         * Attach the input controls to a specific dom element to get the input from.
         * @param noPreventDefault Defines whether event caught by the controls should call preventdefault() (https://developer.mozilla.org/en-US/docs/Web/API/Event/preventDefault)
         */
        attachControl(noPreventDefault?: boolean): void;
        /**
         * Detach the current controls from the specified dom element.
         */
        detachControl(): void;
        /**
         * Gets the class name of the current input.
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Get the friendly name associated with the input class.
         * @returns the input friendly name
         */
        getSimpleName(): string;
    }


    /**
     * Manage the pointers inputs to control an follow camera.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/customizingCameraInputs
     */
    export class FollowCameraPointersInput extends BaseCameraPointersInput {
        /**
         * Defines the camera the input is attached to.
         */
        camera: FollowCamera;
        /**
         * Gets the class name of the current input.
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Defines the pointer angular sensibility along the X axis or how fast is
         * the camera rotating.
         * A negative number will reverse the axis direction.
         */
        angularSensibilityX: number;
        /**
         * Defines the pointer angular sensibility along the Y axis or how fast is
         * the camera rotating.
         * A negative number will reverse the axis direction.
         */
        angularSensibilityY: number;
        /**
         * Defines the pointer pinch precision or how fast is the camera zooming.
         * A negative number will reverse the axis direction.
         */
        pinchPrecision: number;
        /**
         * pinchDeltaPercentage will be used instead of pinchPrecision if different
         * from 0.
         * It defines the percentage of current camera.radius to use as delta when
         * pinch zoom is used.
         */
        pinchDeltaPercentage: number;
        /**
         * Pointer X axis controls zoom. (X axis modifies camera.radius value.)
         */
        axisXControlRadius: boolean;
        /**
         * Pointer X axis controls height. (X axis modifies camera.heightOffset value.)
         */
        axisXControlHeight: boolean;
        /**
         * Pointer X axis controls angle. (X axis modifies camera.rotationOffset value.)
         */
        axisXControlRotation: boolean;
        /**
         * Pointer Y axis controls zoom. (Y axis modifies camera.radius value.)
         */
        axisYControlRadius: boolean;
        /**
         * Pointer Y axis controls height. (Y axis modifies camera.heightOffset value.)
         */
        axisYControlHeight: boolean;
        /**
         * Pointer Y axis controls angle. (Y axis modifies camera.rotationOffset value.)
         */
        axisYControlRotation: boolean;
        /**
         * Pinch controls zoom. (Pinch modifies camera.radius value.)
         */
        axisPinchControlRadius: boolean;
        /**
         * Pinch controls height. (Pinch modifies camera.heightOffset value.)
         */
        axisPinchControlHeight: boolean;
        /**
         * Pinch controls angle. (Pinch modifies camera.rotationOffset value.)
         */
        axisPinchControlRotation: boolean;
        /**
         * Log error messages if basic misconfiguration has occurred.
         */
        warningEnable: boolean;
        onTouch(pointA: Nullable<PointerTouch>, offsetX: number, offsetY: number): void;
        onMultiTouch(pointA: Nullable<PointerTouch>, pointB: Nullable<PointerTouch>, previousPinchSquaredDistance: number, pinchSquaredDistance: number, previousMultiTouchPanPosition: Nullable<PointerTouch>, multiTouchPanPosition: Nullable<PointerTouch>): void;
        private _warningCounter;
        private _warning;
    }


        interface FreeCameraInputsManager {
            /**
             * @internal
             */
            _deviceOrientationInput: Nullable<FreeCameraDeviceOrientationInput>;
            /**
             * Add orientation input support to the input manager.
             * @param smoothFactor deviceOrientation smoothing. 0: no smoothing, 1: new data ignored, 0.9 recommended for smoothing
             * @returns the current input manager
             */
            addDeviceOrientation(smoothFactor?: number): FreeCameraInputsManager;
        }
    /**
     * Takes information about the orientation of the device as reported by the deviceorientation event to orient the camera.
     * Screen rotation is taken into account.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/customizingCameraInputs
     */
    export class FreeCameraDeviceOrientationInput implements ICameraInput<FreeCamera> {
        private _camera;
        private _screenOrientationAngle;
        private _constantTranform;
        private _screenQuaternion;
        private _alpha;
        private _beta;
        private _gamma;
        /** alpha+beta+gamma smoothing. 0: no smoothing, 1: new data ignored, 0.9 recommended for smoothing */
        smoothFactor: number;
        /**
         * Can be used to detect if a device orientation sensor is available on a device
         * @param timeout amount of time in milliseconds to wait for a response from the sensor (default: infinite)
         * @returns a promise that will resolve on orientation change
         */
        static WaitForOrientationChangeAsync(timeout?: number): Promise<void>;
        /**
         * @internal
         */
        _onDeviceOrientationChangedObservable: Observable<void>;
        /**
         * Instantiates a new input
         * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/customizingCameraInputs
         */
        constructor();
        /**
         * Define the camera controlled by the input.
         */
        get camera(): FreeCamera;
        set camera(camera: FreeCamera);
        /**
         * Attach the input controls to a specific dom element to get the input from.
         */
        attachControl(): void;
        private _orientationChanged;
        private _deviceOrientation;
        /**
         * Detach the current controls from the specified dom element.
         */
        detachControl(): void;
        /**
         * Update the current camera state depending on the inputs that have been used this frame.
         * This is a dynamically created lambda to avoid the performance penalty of looping for inputs in the render loop.
         */
        checkInputs(): void;
        /**
         * Gets the class name of the current input.
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Get the friendly name associated with the input class.
         * @returns the input friendly name
         */
        getSimpleName(): string;
    }


    /**
     * Manage the gamepad inputs to control a free camera.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/customizingCameraInputs
     */
    export class FreeCameraGamepadInput implements ICameraInput<FreeCamera> {
        /**
         * Define the camera the input is attached to.
         */
        camera: FreeCamera;
        /**
         * Define the Gamepad controlling the input
         */
        gamepad: Nullable<Gamepad>;
        /**
         * Defines the gamepad rotation sensibility.
         * This is the threshold from when rotation starts to be accounted for to prevent jittering.
         */
        gamepadAngularSensibility: number;
        /**
         * Defines the gamepad move sensibility.
         * This is the threshold from when moving starts to be accounted for for to prevent jittering.
         */
        gamepadMoveSensibility: number;
        /**
         * Defines the minimum value at which any analog stick input is ignored.
         * Note: This value should only be a value between 0 and 1.
         */
        deadzoneDelta: number;
        private _yAxisScale;
        /**
         * Gets or sets a boolean indicating that Yaxis (for right stick) should be inverted
         */
        get invertYAxis(): boolean;
        set invertYAxis(value: boolean);
        private _onGamepadConnectedObserver;
        private _onGamepadDisconnectedObserver;
        private _cameraTransform;
        private _deltaTransform;
        private _vector3;
        private _vector2;
        /**
         * Attach the input controls to a specific dom element to get the input from.
         */
        attachControl(): void;
        /**
         * Detach the current controls from the specified dom element.
         */
        detachControl(): void;
        /**
         * Update the current camera state depending on the inputs that have been used this frame.
         * This is a dynamically created lambda to avoid the performance penalty of looping for inputs in the render loop.
         */
        checkInputs(): void;
        /**
         * Gets the class name of the current input.
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Get the friendly name associated with the input class.
         * @returns the input friendly name
         */
        getSimpleName(): string;
    }


    /**
     * Manage the keyboard inputs to control the movement of a free camera.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/customizingCameraInputs
     */
    export class FreeCameraKeyboardMoveInput implements ICameraInput<FreeCamera> {
        /**
         * Defines the camera the input is attached to.
         */
        camera: FreeCamera;
        /**
         * Gets or Set the list of keyboard keys used to control the forward move of the camera.
         */
        keysUp: number[];
        /**
         * Gets or Set the list of keyboard keys used to control the upward move of the camera.
         */
        keysUpward: number[];
        /**
         * Gets or Set the list of keyboard keys used to control the backward move of the camera.
         */
        keysDown: number[];
        /**
         * Gets or Set the list of keyboard keys used to control the downward move of the camera.
         */
        keysDownward: number[];
        /**
         * Gets or Set the list of keyboard keys used to control the left strafe move of the camera.
         */
        keysLeft: number[];
        /**
         * Gets or Set the list of keyboard keys used to control the right strafe move of the camera.
         */
        keysRight: number[];
        /**
         * Defines the pointer angular sensibility  along the X and Y axis or how fast is the camera rotating.
         */
        rotationSpeed: number;
        /**
         * Gets or Set the list of keyboard keys used to control the left rotation move of the camera.
         */
        keysRotateLeft: number[];
        /**
         * Gets or Set the list of keyboard keys used to control the right rotation move of the camera.
         */
        keysRotateRight: number[];
        private _keys;
        private _onCanvasBlurObserver;
        private _onKeyboardObserver;
        private _engine;
        private _scene;
        /**
         * Attach the input controls to a specific dom element to get the input from.
         * @param noPreventDefault Defines whether event caught by the controls should call preventdefault() (https://developer.mozilla.org/en-US/docs/Web/API/Event/preventDefault)
         */
        attachControl(noPreventDefault?: boolean): void;
        /**
         * Detach the current controls from the specified dom element.
         */
        detachControl(): void;
        /**
         * Update the current camera state depending on the inputs that have been used this frame.
         * This is a dynamically created lambda to avoid the performance penalty of looping for inputs in the render loop.
         */
        checkInputs(): void;
        /**
         * Gets the class name of the current input.
         * @returns the class name
         */
        getClassName(): string;
        /** @internal */
        _onLostFocus(): void;
        /**
         * Get the friendly name associated with the input class.
         * @returns the input friendly name
         */
        getSimpleName(): string;
        private _getLocalRotation;
    }


    /**
     * Manage the mouse inputs to control the movement of a free camera.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/customizingCameraInputs
     */
    export class FreeCameraMouseInput implements ICameraInput<FreeCamera> {
        /**
         * Define if touch is enabled in the mouse input
         */
        touchEnabled: boolean;
        /**
         * Defines the camera the input is attached to.
         */
        camera: FreeCamera;
        /**
         * Defines the buttons associated with the input to handle camera move.
         */
        buttons: number[];
        /**
         * Defines the pointer angular sensibility  along the X and Y axis or how fast is the camera rotating.
         */
        angularSensibility: number;
        private _pointerInput;
        private _onMouseMove;
        private _observer;
        private _previousPosition;
        /**
         * Observable for when a pointer move event occurs containing the move offset
         */
        onPointerMovedObservable: Observable<{
            offsetX: number;
            offsetY: number;
        }>;
        /**
         * @internal
         * If the camera should be rotated automatically based on pointer movement
         */
        _allowCameraRotation: boolean;
        private _currentActiveButton;
        private _activePointerId;
        private _contextMenuBind;
        /**
         * Manage the mouse inputs to control the movement of a free camera.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/customizingCameraInputs
         * @param touchEnabled Defines if touch is enabled or not
         */
        constructor(
        /**
         * Define if touch is enabled in the mouse input
         */
        touchEnabled?: boolean);
        /**
         * Attach the input controls to a specific dom element to get the input from.
         * @param noPreventDefault Defines whether event caught by the controls should call preventdefault() (https://developer.mozilla.org/en-US/docs/Web/API/Event/preventDefault)
         */
        attachControl(noPreventDefault?: boolean): void;
        /**
         * Called on JS contextmenu event.
         * Override this method to provide functionality.
         * @param evt
         */
        onContextMenu(evt: PointerEvent): void;
        /**
         * Detach the current controls from the specified dom element.
         */
        detachControl(): void;
        /**
         * Gets the class name of the current input.
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Get the friendly name associated with the input class.
         * @returns the input friendly name
         */
        getSimpleName(): string;
    }


    /**
     * Manage the mouse wheel inputs to control a free camera.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/customizingCameraInputs
     */
    export class FreeCameraMouseWheelInput extends BaseCameraMouseWheelInput {
        /**
         * Defines the camera the input is attached to.
         */
        camera: FreeCamera;
        /**
         * Gets the class name of the current input.
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Set which movement axis (relative to camera's orientation) the mouse
         * wheel's X axis controls.
         * @param axis The axis to be moved. Set null to clear.
         */
        set wheelXMoveRelative(axis: Nullable<Coordinate>);
        /**
         * Get the configured movement axis (relative to camera's orientation) the
         * mouse wheel's X axis controls.
         * @returns The configured axis or null if none.
         */
        get wheelXMoveRelative(): Nullable<Coordinate>;
        /**
         * Set which movement axis (relative to camera's orientation) the mouse
         * wheel's Y axis controls.
         * @param axis The axis to be moved. Set null to clear.
         */
        set wheelYMoveRelative(axis: Nullable<Coordinate>);
        /**
         * Get the configured movement axis (relative to camera's orientation) the
         * mouse wheel's Y axis controls.
         * @returns The configured axis or null if none.
         */
        get wheelYMoveRelative(): Nullable<Coordinate>;
        /**
         * Set which movement axis (relative to camera's orientation) the mouse
         * wheel's Z axis controls.
         * @param axis The axis to be moved. Set null to clear.
         */
        set wheelZMoveRelative(axis: Nullable<Coordinate>);
        /**
         * Get the configured movement axis (relative to camera's orientation) the
         * mouse wheel's Z axis controls.
         * @returns The configured axis or null if none.
         */
        get wheelZMoveRelative(): Nullable<Coordinate>;
        /**
         * Set which rotation axis (relative to camera's orientation) the mouse
         * wheel's X axis controls.
         * @param axis The axis to be moved. Set null to clear.
         */
        set wheelXRotateRelative(axis: Nullable<Coordinate>);
        /**
         * Get the configured rotation axis (relative to camera's orientation) the
         * mouse wheel's X axis controls.
         * @returns The configured axis or null if none.
         */
        get wheelXRotateRelative(): Nullable<Coordinate>;
        /**
         * Set which rotation axis (relative to camera's orientation) the mouse
         * wheel's Y axis controls.
         * @param axis The axis to be moved. Set null to clear.
         */
        set wheelYRotateRelative(axis: Nullable<Coordinate>);
        /**
         * Get the configured rotation axis (relative to camera's orientation) the
         * mouse wheel's Y axis controls.
         * @returns The configured axis or null if none.
         */
        get wheelYRotateRelative(): Nullable<Coordinate>;
        /**
         * Set which rotation axis (relative to camera's orientation) the mouse
         * wheel's Z axis controls.
         * @param axis The axis to be moved. Set null to clear.
         */
        set wheelZRotateRelative(axis: Nullable<Coordinate>);
        /**
         * Get the configured rotation axis (relative to camera's orientation) the
         * mouse wheel's Z axis controls.
         * @returns The configured axis or null if none.
         */
        get wheelZRotateRelative(): Nullable<Coordinate>;
        /**
         * Set which movement axis (relative to the scene) the mouse wheel's X axis
         * controls.
         * @param axis The axis to be moved. Set null to clear.
         */
        set wheelXMoveScene(axis: Nullable<Coordinate>);
        /**
         * Get the configured movement axis (relative to the scene) the mouse wheel's
         * X axis controls.
         * @returns The configured axis or null if none.
         */
        get wheelXMoveScene(): Nullable<Coordinate>;
        /**
         * Set which movement axis (relative to the scene) the mouse wheel's Y axis
         * controls.
         * @param axis The axis to be moved. Set null to clear.
         */
        set wheelYMoveScene(axis: Nullable<Coordinate>);
        /**
         * Get the configured movement axis (relative to the scene) the mouse wheel's
         * Y axis controls.
         * @returns The configured axis or null if none.
         */
        get wheelYMoveScene(): Nullable<Coordinate>;
        /**
         * Set which movement axis (relative to the scene) the mouse wheel's Z axis
         * controls.
         * @param axis The axis to be moved. Set null to clear.
         */
        set wheelZMoveScene(axis: Nullable<Coordinate>);
        /**
         * Get the configured movement axis (relative to the scene) the mouse wheel's
         * Z axis controls.
         * @returns The configured axis or null if none.
         */
        get wheelZMoveScene(): Nullable<Coordinate>;
        /**
         * Called for each rendered frame.
         */
        checkInputs(): void;
        private _moveRelative;
        private _rotateRelative;
        private _moveScene;
        /**
         * These are set to the desired default behaviour.
         */
        private _wheelXAction;
        private _wheelXActionCoordinate;
        private _wheelYAction;
        private _wheelYActionCoordinate;
        private _wheelZAction;
        private _wheelZActionCoordinate;
        /**
         * Update the camera according to any configured properties for the 3
         * mouse-wheel axis.
         */
        private _updateCamera;
        /**
         * Update one property of the camera.
         * @param value
         * @param cameraProperty
         * @param coordinate
         */
        private _updateCameraProperty;
    }


    /**
     * Manage the touch inputs to control the movement of a free camera.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/customizingCameraInputs
     */
    export class FreeCameraTouchInput implements ICameraInput<FreeCamera> {
        /**
         * Define if mouse events can be treated as touch events
         */
        allowMouse: boolean;
        /**
         * Defines the camera the input is attached to.
         */
        camera: FreeCamera;
        /**
         * Defines the touch sensibility for rotation.
         * The lower the faster.
         */
        touchAngularSensibility: number;
        /**
         * Defines the touch sensibility for move.
         * The lower the faster.
         */
        touchMoveSensibility: number;
        /**
         * Swap touch actions so that one touch is used for rotation and multiple for movement
         */
        singleFingerRotate: boolean;
        private _offsetX;
        private _offsetY;
        private _pointerPressed;
        private _pointerInput?;
        private _observer;
        private _onLostFocus;
        private _isSafari;
        /**
         * Manage the touch inputs to control the movement of a free camera.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/customizingCameraInputs
         * @param allowMouse Defines if mouse events can be treated as touch events
         */
        constructor(
        /**
         * Define if mouse events can be treated as touch events
         */
        allowMouse?: boolean);
        /**
         * Attach the input controls to a specific dom element to get the input from.
         * @param noPreventDefault Defines whether event caught by the controls should call preventdefault() (https://developer.mozilla.org/en-US/docs/Web/API/Event/preventDefault)
         */
        attachControl(noPreventDefault?: boolean): void;
        /**
         * Detach the current controls from the specified dom element.
         */
        detachControl(): void;
        /**
         * Update the current camera state depending on the inputs that have been used this frame.
         * This is a dynamically created lambda to avoid the performance penalty of looping for inputs in the render loop.
         */
        checkInputs(): void;
        /**
         * Gets the class name of the current input.
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Get the friendly name associated with the input class.
         * @returns the input friendly name
         */
        getSimpleName(): string;
    }


        interface FreeCameraInputsManager {
            /**
             * Add virtual joystick input support to the input manager.
             * @returns the current input manager
             */
            addVirtualJoystick(): FreeCameraInputsManager;
        }
    /**
     * Manage the Virtual Joystick inputs to control the movement of a free camera.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/customizingCameraInputs
     */
    export class FreeCameraVirtualJoystickInput implements ICameraInput<FreeCamera> {
        /**
         * Defines the camera the input is attached to.
         */
        camera: FreeCamera;
        private _leftjoystick;
        private _rightjoystick;
        /**
         * Gets the left stick of the virtual joystick.
         * @returns The virtual Joystick
         */
        getLeftJoystick(): VirtualJoystick;
        /**
         * Gets the right stick of the virtual joystick.
         * @returns The virtual Joystick
         */
        getRightJoystick(): VirtualJoystick;
        /**
         * Update the current camera state depending on the inputs that have been used this frame.
         * This is a dynamically created lambda to avoid the performance penalty of looping for inputs in the render loop.
         */
        checkInputs(): void;
        /**
         * Attach the input controls to a specific dom element to get the input from.
         */
        attachControl(): void;
        /**
         * Detach the current controls from the specified dom element.
         */
        detachControl(): void;
        /**
         * Gets the class name of the current input.
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Get the friendly name associated with the input class.
         * @returns the input friendly name
         */
        getSimpleName(): string;
    }






    /**
     * @internal
     */
    export function setStereoscopicAnaglyphRigMode(camera: Camera): void;


    /**
     * @internal
     */
    export function setStereoscopicRigMode(camera: Camera): void;


    /**
     * @internal
     */
    export function setVRRigMode(camera: Camera, rigParams: any): void;


    /**
     * @internal
     */
    export function setWebVRRigMode(camera: Camera, rigParams: any): void;


    /**
     * Camera used to simulate anaglyphic rendering (based on ArcRotateCamera)
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_introduction#anaglyph-cameras
     */
    export class AnaglyphArcRotateCamera extends ArcRotateCamera {
        /**
         * Creates a new AnaglyphArcRotateCamera
         * @param name defines camera name
         * @param alpha defines alpha angle (in radians)
         * @param beta defines beta angle (in radians)
         * @param radius defines radius
         * @param target defines camera target
         * @param interaxialDistance defines distance between each color axis
         * @param scene defines the hosting scene
         */
        constructor(name: string, alpha: number, beta: number, radius: number, target: Vector3, interaxialDistance: number, scene?: Scene);
        /**
         * Gets camera class name
         * @returns AnaglyphArcRotateCamera
         */
        getClassName(): string;
        protected _setRigMode: any;
    }


    /**
     * Camera used to simulate anaglyphic rendering (based on FreeCamera)
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_introduction#anaglyph-cameras
     */
    export class AnaglyphFreeCamera extends FreeCamera {
        /**
         * Creates a new AnaglyphFreeCamera
         * @param name defines camera name
         * @param position defines initial position
         * @param interaxialDistance defines distance between each color axis
         * @param scene defines the hosting scene
         */
        constructor(name: string, position: Vector3, interaxialDistance: number, scene?: Scene);
        /**
         * Gets camera class name
         * @returns AnaglyphFreeCamera
         */
        getClassName(): string;
        protected _setRigMode: any;
    }


    /**
     * Camera used to simulate anaglyphic rendering (based on GamepadCamera)
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_introduction#anaglyph-cameras
     */
    export class AnaglyphGamepadCamera extends GamepadCamera {
        /**
         * Creates a new AnaglyphGamepadCamera
         * @param name defines camera name
         * @param position defines initial position
         * @param interaxialDistance defines distance between each color axis
         * @param scene defines the hosting scene
         */
        constructor(name: string, position: Vector3, interaxialDistance: number, scene?: Scene);
        /**
         * Gets camera class name
         * @returns AnaglyphGamepadCamera
         */
        getClassName(): string;
        protected _setRigMode: any;
    }


    /**
     * Camera used to simulate anaglyphic rendering (based on UniversalCamera)
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_introduction#anaglyph-cameras
     */
    export class AnaglyphUniversalCamera extends UniversalCamera {
        /**
         * Creates a new AnaglyphUniversalCamera
         * @param name defines camera name
         * @param position defines initial position
         * @param interaxialDistance defines distance between each color axis
         * @param scene defines the hosting scene
         */
        constructor(name: string, position: Vector3, interaxialDistance: number, scene?: Scene);
        /**
         * Gets camera class name
         * @returns AnaglyphUniversalCamera
         */
        getClassName(): string;
        protected _setRigMode: any;
    }




    /**
     * Camera used to simulate stereoscopic rendering (based on ArcRotateCamera)
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras
     */
    export class StereoscopicArcRotateCamera extends ArcRotateCamera {
        /**
         * Creates a new StereoscopicArcRotateCamera
         * @param name defines camera name
         * @param alpha defines alpha angle (in radians)
         * @param beta defines beta angle (in radians)
         * @param radius defines radius
         * @param target defines camera target
         * @param interaxialDistance defines distance between each color axis
         * @param isStereoscopicSideBySide defines is stereoscopic is done side by side or over under
         * @param scene defines the hosting scene
         */
        constructor(name: string, alpha: number, beta: number, radius: number, target: Vector3, interaxialDistance: number, isStereoscopicSideBySide: boolean, scene?: Scene);
        /**
         * Gets camera class name
         * @returns StereoscopicArcRotateCamera
         */
        getClassName(): string;
        protected _setRigMode: any;
    }


    /**
     * Camera used to simulate stereoscopic rendering (based on FreeCamera)
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras
     */
    export class StereoscopicFreeCamera extends FreeCamera {
        /**
         * Creates a new StereoscopicFreeCamera
         * @param name defines camera name
         * @param position defines initial position
         * @param interaxialDistance defines distance between each color axis
         * @param isStereoscopicSideBySide defines is stereoscopic is done side by side or over under
         * @param scene defines the hosting scene
         */
        constructor(name: string, position: Vector3, interaxialDistance: number, isStereoscopicSideBySide: boolean, scene?: Scene);
        /**
         * Gets camera class name
         * @returns StereoscopicFreeCamera
         */
        getClassName(): string;
        protected _setRigMode: any;
    }


    /**
     * Camera used to simulate stereoscopic rendering (based on GamepadCamera)
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras
     */
    export class StereoscopicGamepadCamera extends GamepadCamera {
        /**
         * Creates a new StereoscopicGamepadCamera
         * @param name defines camera name
         * @param position defines initial position
         * @param interaxialDistance defines distance between each color axis
         * @param isStereoscopicSideBySide defines is stereoscopic is done side by side or over under
         * @param scene defines the hosting scene
         */
        constructor(name: string, position: Vector3, interaxialDistance: number, isStereoscopicSideBySide: boolean, scene?: Scene);
        /**
         * Gets camera class name
         * @returns StereoscopicGamepadCamera
         */
        getClassName(): string;
        protected _setRigMode: any;
    }


    /**
     * Camera used to simulate stereoscopic rendering on real screens (based on UniversalCamera)
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras
     */
    export class StereoscopicScreenUniversalCamera extends UniversalCamera {
        private _distanceToProjectionPlane;
        private _distanceBetweenEyes;
        set distanceBetweenEyes(newValue: number);
        /**
         * distance between the eyes
         */
        get distanceBetweenEyes(): number;
        set distanceToProjectionPlane(newValue: number);
        /**
         * Distance to projection plane (should be the same units the like distance between the eyes)
         */
        get distanceToProjectionPlane(): number;
        /**
         * Creates a new StereoscopicScreenUniversalCamera
         * @param name defines camera name
         * @param position defines initial position
         * @param scene defines the hosting scene
         * @param distanceToProjectionPlane defines distance between each color axis. The rig cameras will receive this as their negative z position!
         * @param distanceBetweenEyes defines is stereoscopic is done side by side or over under
         */
        constructor(name: string, position: Vector3, scene?: Scene, distanceToProjectionPlane?: number, distanceBetweenEyes?: number);
        /**
         * Gets camera class name
         * @returns StereoscopicScreenUniversalCamera
         */
        getClassName(): string;
        /**
         * @internal
         */
        createRigCamera(name: string): Nullable<Camera>;
        /**
         * @internal
         */
        _updateRigCameras(): void;
        private _updateCamera;
        protected _setRigMode(): void;
    }


    /**
     * Camera used to simulate stereoscopic rendering (based on UniversalCamera)
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras
     */
    export class StereoscopicUniversalCamera extends UniversalCamera {
        /**
         * Creates a new StereoscopicUniversalCamera
         * @param name defines camera name
         * @param position defines initial position
         * @param interaxialDistance defines distance between each color axis
         * @param isStereoscopicSideBySide defines is stereoscopic is done side by side or over under
         * @param scene defines the hosting scene
         */
        constructor(name: string, position: Vector3, interaxialDistance: number, isStereoscopicSideBySide: boolean, scene?: Scene);
        /**
         * Gets camera class name
         * @returns StereoscopicUniversalCamera
         */
        getClassName(): string;
        protected _setRigMode: any;
    }


    /**
     * A target camera takes a mesh or position as a target and continues to look at it while it moves.
     * This is the base of the follow, arc rotate cameras and Free camera
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras
     */
    export class TargetCamera extends Camera {
        private static _RigCamTransformMatrix;
        private static _TargetTransformMatrix;
        private static _TargetFocalPoint;
        private _tmpUpVector;
        private _tmpTargetVector;
        /**
         * Define the current direction the camera is moving to
         */
        cameraDirection: Vector3;
        /**
         * Define the current rotation the camera is rotating to
         */
        cameraRotation: Vector2;
        /** Gets or sets a boolean indicating that the scaling of the parent hierarchy will not be taken in account by the camera */
        ignoreParentScaling: boolean;
        /**
         * When set, the up vector of the camera will be updated by the rotation of the camera
         */
        updateUpVectorFromRotation: boolean;
        private _tmpQuaternion;
        /**
         * Define the current rotation of the camera
         */
        rotation: Vector3;
        /**
         * Define the current rotation of the camera as a quaternion to prevent Gimbal lock
         */
        rotationQuaternion: Quaternion;
        /**
         * Define the current speed of the camera
         */
        speed: number;
        /**
         * Add constraint to the camera to prevent it to move freely in all directions and
         * around all axis.
         */
        noRotationConstraint: boolean;
        /**
         * Reverses mouselook direction to 'natural' panning as opposed to traditional direct
         * panning
         */
        invertRotation: boolean;
        /**
         * Speed multiplier for inverse camera panning
         */
        inverseRotationSpeed: number;
        /**
         * Define the current target of the camera as an object or a position.
         * Please note that locking a target will disable panning.
         */
        lockedTarget: any;
        /** @internal */
        _currentTarget: Vector3;
        /** @internal */
        _initialFocalDistance: number;
        /** @internal */
        _viewMatrix: Matrix;
        /** @internal */
        _camMatrix: Matrix;
        /** @internal */
        _cameraTransformMatrix: Matrix;
        /** @internal */
        _cameraRotationMatrix: Matrix;
        /** @internal */
        _referencePoint: Vector3;
        /** @internal */
        _transformedReferencePoint: Vector3;
        /** @internal */
        _reset: () => void;
        private _defaultUp;
        /**
         * Instantiates a target camera that takes a mesh or position as a target and continues to look at it while it moves.
         * This is the base of the follow, arc rotate cameras and Free camera
         * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras
         * @param name Defines the name of the camera in the scene
         * @param position Defines the start position of the camera in the scene
         * @param scene Defines the scene the camera belongs to
         * @param setActiveOnSceneIfNoneActive Defines whether the camera should be marked as active if not other active cameras have been defined
         */
        constructor(name: string, position: Vector3, scene?: Scene, setActiveOnSceneIfNoneActive?: boolean);
        /**
         * Gets the position in front of the camera at a given distance.
         * @param distance The distance from the camera we want the position to be
         * @returns the position
         */
        getFrontPosition(distance: number): Vector3;
        /** @internal */
        _getLockedTargetPosition(): Nullable<Vector3>;
        private _storedPosition;
        private _storedRotation;
        private _storedRotationQuaternion;
        /**
         * Store current camera state of the camera (fov, position, rotation, etc..)
         * @returns the camera
         */
        storeState(): Camera;
        /**
         * Restored camera state. You must call storeState() first
         * @returns whether it was successful or not
         * @internal
         */
        _restoreStateValues(): boolean;
        /** @internal */
        _initCache(): void;
        /**
         * @internal
         */
        _updateCache(ignoreParentClass?: boolean): void;
        /** @internal */
        _isSynchronizedViewMatrix(): boolean;
        /** @internal */
        _computeLocalCameraSpeed(): number;
        /**
         * Defines the target the camera should look at.
         * @param target Defines the new target as a Vector
         */
        setTarget(target: Vector3): void;
        /**
         * Defines the target point of the camera.
         * The camera looks towards it form the radius distance.
         */
        get target(): Vector3;
        set target(value: Vector3);
        /**
         * Return the current target position of the camera. This value is expressed in local space.
         * @returns the target position
         */
        getTarget(): Vector3;
        /** @internal */
        _decideIfNeedsToMove(): boolean;
        /** @internal */
        _updatePosition(): void;
        /** @internal */
        _checkInputs(): void;
        protected _updateCameraRotationMatrix(): void;
        /**
         * Update the up vector to apply the rotation of the camera (So if you changed the camera rotation.z this will let you update the up vector as well)
         * @returns the current camera
         */
        private _rotateUpVectorWithCameraRotationMatrix;
        private _cachedRotationZ;
        private _cachedQuaternionRotationZ;
        /** @internal */
        _getViewMatrix(): Matrix;
        protected _computeViewMatrix(position: Vector3, target: Vector3, up: Vector3): void;
        /**
         * @internal
         */
        createRigCamera(name: string, cameraIndex: number): Nullable<Camera>;
        /**
         * @internal
         */
        _updateRigCameras(): void;
        private _getRigCamPositionAndTarget;
        /**
         * Gets the current object class name.
         * @returns the class name
         */
        getClassName(): string;
    }


    /**
     * This represents a FPS type of camera controlled by touch.
     * This is like a universal camera minus the Gamepad controls.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_introduction#universal-camera
     */
    export class TouchCamera extends FreeCamera {
        /**
         * Defines the touch sensibility for rotation.
         * The higher the faster.
         */
        get touchAngularSensibility(): number;
        set touchAngularSensibility(value: number);
        /**
         * Defines the touch sensibility for move.
         * The higher the faster.
         */
        get touchMoveSensibility(): number;
        set touchMoveSensibility(value: number);
        /**
         * Instantiates a new touch camera.
         * This represents a FPS type of camera controlled by touch.
         * This is like a universal camera minus the Gamepad controls.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_introduction#universal-camera
         * @param name Define the name of the camera in the scene
         * @param position Define the start position of the camera in the scene
         * @param scene Define the scene the camera belongs to
         */
        constructor(name: string, position: Vector3, scene?: Scene);
        /**
         * Gets the current object class name.
         * @returns the class name
         */
        getClassName(): string;
        /** @internal */
        _setupInputs(): void;
    }


    /**
     * The Universal Camera is the one to choose for first person shooter type games, and works with all the keyboard, mouse, touch and gamepads. This replaces the earlier Free Camera,
     * which still works and will still be found in many Playgrounds.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_introduction#universal-camera
     */
    export class UniversalCamera extends TouchCamera {
        /**
         * Defines the gamepad rotation sensibility.
         * This is the threshold from when rotation starts to be accounted for to prevent jittering.
         */
        get gamepadAngularSensibility(): number;
        set gamepadAngularSensibility(value: number);
        /**
         * Defines the gamepad move sensibility.
         * This is the threshold from when moving starts to be accounted for to prevent jittering.
         */
        get gamepadMoveSensibility(): number;
        set gamepadMoveSensibility(value: number);
        /**
         * The Universal Camera is the one to choose for first person shooter type games, and works with all the keyboard, mouse, touch and gamepads. This replaces the earlier Free Camera,
         * which still works and will still be found in many Playgrounds.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_introduction#universal-camera
         * @param name Define the name of the camera in the scene
         * @param position Define the start position of the camera in the scene
         * @param scene Define the scene the camera belongs to
         */
        constructor(name: string, position: Vector3, scene?: Scene);
        /**
         * Gets the current object class name.
         * @returns the class name
         */
        getClassName(): string;
    }


    /**
     * This represents a free type of camera. It can be useful in First Person Shooter game for instance.
     * It is identical to the Free Camera and simply adds by default a virtual joystick.
     * Virtual Joysticks are on-screen 2D graphics that are used to control the camera or other scene items.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_introduction#virtual-joysticks-camera
     */
    export class VirtualJoysticksCamera extends FreeCamera {
        /**
         * Instantiates a VirtualJoysticksCamera. It can be useful in First Person Shooter game for instance.
         * It is identical to the Free Camera and simply adds by default a virtual joystick.
         * Virtual Joysticks are on-screen 2D graphics that are used to control the camera or other scene items.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_introduction#virtual-joysticks-camera
         * @param name Define the name of the camera in the scene
         * @param position Define the start position of the camera in the scene
         * @param scene Define the scene the camera belongs to
         */
        constructor(name: string, position: Vector3, scene?: Scene);
        /**
         * Gets the current object class name.
         * @returns the class name
         */
        getClassName(): string;
    }




    /**
     * This represents all the required metrics to create a VR camera.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_introduction#device-orientation-camera
     */
    export class VRCameraMetrics {
        /**
         * Define the horizontal resolution off the screen.
         */
        hResolution: number;
        /**
         * Define the vertical resolution off the screen.
         */
        vResolution: number;
        /**
         * Define the horizontal screen size.
         */
        hScreenSize: number;
        /**
         * Define the vertical screen size.
         */
        vScreenSize: number;
        /**
         * Define the vertical screen center position.
         */
        vScreenCenter: number;
        /**
         * Define the distance of the eyes to the screen.
         */
        eyeToScreenDistance: number;
        /**
         * Define the distance between both lenses
         */
        lensSeparationDistance: number;
        /**
         * Define the distance between both viewer's eyes.
         */
        interpupillaryDistance: number;
        /**
         * Define the distortion factor of the VR postprocess.
         * Please, touch with care.
         */
        distortionK: number[];
        /**
         * Define the chromatic aberration correction factors for the VR post process.
         */
        chromaAbCorrection: number[];
        /**
         * Define the scale factor of the post process.
         * The smaller the better but the slower.
         */
        postProcessScaleFactor: number;
        /**
         * Define an offset for the lens center.
         */
        lensCenterOffset: number;
        /**
         * Define if the current vr camera should compensate the distortion of the lens or not.
         */
        compensateDistortion: boolean;
        /**
         * Defines if multiview should be enabled when rendering (Default: false)
         */
        multiviewEnabled: boolean;
        /**
         * Gets the rendering aspect ratio based on the provided resolutions.
         */
        get aspectRatio(): number;
        /**
         * Gets the aspect ratio based on the FOV, scale factors, and real screen sizes.
         */
        get aspectRatioFov(): number;
        /**
         * @internal
         */
        get leftHMatrix(): Matrix;
        /**
         * @internal
         */
        get rightHMatrix(): Matrix;
        /**
         * @internal
         */
        get leftPreViewMatrix(): Matrix;
        /**
         * @internal
         */
        get rightPreViewMatrix(): Matrix;
        /**
         * Get the default VRMetrics based on the most generic setup.
         * @returns the default vr metrics
         */
        static GetDefault(): VRCameraMetrics;
    }


    /**
     * Camera used to simulate VR rendering (based on ArcRotateCamera)
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_introduction#vr-device-orientation-cameras
     */
    export class VRDeviceOrientationArcRotateCamera extends ArcRotateCamera {
        /**
         * Creates a new VRDeviceOrientationArcRotateCamera
         * @param name defines camera name
         * @param alpha defines the camera rotation along the longitudinal axis
         * @param beta defines the camera rotation along the latitudinal axis
         * @param radius defines the camera distance from its target
         * @param target defines the camera target
         * @param scene defines the scene the camera belongs to
         * @param compensateDistortion defines if the camera needs to compensate the lens distortion
         * @param vrCameraMetrics defines the vr metrics associated to the camera
         */
        constructor(name: string, alpha: number, beta: number, radius: number, target: Vector3, scene?: Scene, compensateDistortion?: boolean, vrCameraMetrics?: VRCameraMetrics);
        /**
         * Gets camera class name
         * @returns VRDeviceOrientationArcRotateCamera
         */
        getClassName(): string;
        protected _setRigMode: any;
    }


    /**
     * Camera used to simulate VR rendering (based on FreeCamera)
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_introduction#vr-device-orientation-cameras
     */
    export class VRDeviceOrientationFreeCamera extends DeviceOrientationCamera {
        /**
         * Creates a new VRDeviceOrientationFreeCamera
         * @param name defines camera name
         * @param position defines the start position of the camera
         * @param scene defines the scene the camera belongs to
         * @param compensateDistortion defines if the camera needs to compensate the lens distortion
         * @param vrCameraMetrics defines the vr metrics associated to the camera
         */
        constructor(name: string, position: Vector3, scene?: Scene, compensateDistortion?: boolean, vrCameraMetrics?: VRCameraMetrics);
        /**
         * Gets camera class name
         * @returns VRDeviceOrientationFreeCamera
         */
        getClassName(): string;
        protected _setRigMode: any;
    }


    /**
     * Camera used to simulate VR rendering (based on VRDeviceOrientationFreeCamera)
     * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_introduction#vr-device-orientation-cameras
     */
    export class VRDeviceOrientationGamepadCamera extends VRDeviceOrientationFreeCamera {
        /**
         * Creates a new VRDeviceOrientationGamepadCamera
         * @param name defines camera name
         * @param position defines the start position of the camera
         * @param scene defines the scene the camera belongs to
         * @param compensateDistortion defines if the camera needs to compensate the lens distortion
         * @param vrCameraMetrics defines the vr metrics associated to the camera
         */
        constructor(name: string, position: Vector3, scene?: Scene, compensateDistortion?: boolean, vrCameraMetrics?: VRCameraMetrics);
        /**
         * Gets camera class name
         * @returns VRDeviceOrientationGamepadCamera
         */
        getClassName(): string;
        protected _setRigMode: any;
    }


    /**
     * Options to modify the vr teleportation behavior.
     */
    export interface VRTeleportationOptions {
        /**
         * The name of the mesh which should be used as the teleportation floor. (default: null)
         */
        floorMeshName?: string;
        /**
         * A list of meshes to be used as the teleportation floor. (default: empty)
         */
        floorMeshes?: Mesh[];
        /**
         * The teleportation mode. (default: TELEPORTATIONMODE_CONSTANTTIME)
         */
        teleportationMode?: number;
        /**
         * The duration of the animation in ms, apply when animationMode is TELEPORTATIONMODE_CONSTANTTIME. (default 122ms)
         */
        teleportationTime?: number;
        /**
         * The speed of the animation in distance/sec, apply when animationMode is TELEPORTATIONMODE_CONSTANTSPEED. (default 20 units / sec)
         */
        teleportationSpeed?: number;
        /**
         * The easing function used in the animation or null for Linear. (default CircleEase)
         */
        easingFunction?: EasingFunction;
    }
    /**
     * Options to modify the vr experience helper's behavior.
     */
    export interface VRExperienceHelperOptions extends WebVROptions {
        /**
         * Create a DeviceOrientationCamera to be used as your out of vr camera. (default: true)
         */
        createDeviceOrientationCamera?: boolean;
        /**
         * Create a VRDeviceOrientationFreeCamera to be used for VR when no external HMD is found. (default: true)
         */
        createFallbackVRDeviceOrientationFreeCamera?: boolean;
        /**
         * Uses the main button on the controller to toggle the laser casted. (default: true)
         */
        laserToggle?: boolean;
        /**
         * A list of meshes to be used as the teleportation floor. If specified, teleportation will be enabled (default: undefined)
         */
        floorMeshes?: Mesh[];
        /**
         * Distortion metrics for the fallback vrDeviceOrientationCamera (default: VRCameraMetrics.Default)
         */
        vrDeviceOrientationCameraMetrics?: VRCameraMetrics;
        /**
         * Defines if WebXR should be used instead of WebVR (if available)
         */
        useXR?: boolean;
    }
    /**
     * Event containing information after VR has been entered
     */
    export class OnAfterEnteringVRObservableEvent {
        /**
         * If entering vr was successful
         */
        success: boolean;
    }
    /**
     * Helps to quickly add VR support to an existing scene.
     * See https://doc.babylonjs.com/features/featuresDeepDive/cameras/webVRHelper
     * @deprecated
     */
    export class VRExperienceHelper {
        /** Options to modify the vr experience helper's behavior. */
        webVROptions: VRExperienceHelperOptions;
        private _scene;
        private _position;
        private _btnVR;
        private _btnVRDisplayed;
        private _webVRsupported;
        private _webVRready;
        private _webVRrequesting;
        private _webVRpresenting;
        private _hasEnteredVR;
        private _fullscreenVRpresenting;
        private _inputElement;
        private _webVRCamera;
        private _vrDeviceOrientationCamera;
        private _deviceOrientationCamera;
        private _existingCamera;
        private _onKeyDown;
        private _onVrDisplayPresentChangeBind;
        private _onVRDisplayChangedBind;
        private _onVRRequestPresentStart;
        private _onVRRequestPresentComplete;
        /**
         * Gets or sets a boolean indicating that gaze can be enabled even if pointer lock is not engage (useful on iOS where fullscreen mode and pointer lock are not supported)
         */
        enableGazeEvenWhenNoPointerLock: boolean;
        /**
         * Gets or sets a boolean indicating that the VREXperienceHelper will exit VR if double tap is detected
         */
        exitVROnDoubleTap: boolean;
        /**
         * Observable raised right before entering VR.
         */
        onEnteringVRObservable: Observable<VRExperienceHelper>;
        /**
         * Observable raised when entering VR has completed.
         */
        onAfterEnteringVRObservable: Observable<OnAfterEnteringVRObservableEvent>;
        /**
         * Observable raised when exiting VR.
         */
        onExitingVRObservable: Observable<VRExperienceHelper>;
        /**
         * Observable raised when controller mesh is loaded.
         */
        onControllerMeshLoadedObservable: Observable<WebVRController>;
        /** Return this.onEnteringVRObservable
         * Note: This one is for backward compatibility. Please use onEnteringVRObservable directly
         */
        get onEnteringVR(): Observable<VRExperienceHelper>;
        /** Return this.onExitingVRObservable
         * Note: This one is for backward compatibility. Please use onExitingVRObservable directly
         */
        get onExitingVR(): Observable<VRExperienceHelper>;
        /** Return this.onControllerMeshLoadedObservable
         * Note: This one is for backward compatibility. Please use onControllerMeshLoadedObservable directly
         */
        get onControllerMeshLoaded(): Observable<WebVRController>;
        private _rayLength;
        private _useCustomVRButton;
        private _teleportationRequested;
        private _teleportActive;
        private _floorMeshName;
        private _floorMeshesCollection;
        private _teleportationMode;
        private _teleportationTime;
        private _teleportationSpeed;
        private _teleportationEasing;
        private _rotationAllowed;
        private _teleportBackwardsVector;
        private _teleportationTarget;
        private _isDefaultTeleportationTarget;
        private _postProcessMove;
        private _teleportationFillColor;
        private _teleportationBorderColor;
        private _rotationAngle;
        private _haloCenter;
        private _cameraGazer;
        private _padSensibilityUp;
        private _padSensibilityDown;
        private _leftController;
        private _rightController;
        private _gazeColor;
        private _laserColor;
        private _pickedLaserColor;
        private _pickedGazeColor;
        /**
         * Observable raised when a new mesh is selected based on meshSelectionPredicate
         */
        onNewMeshSelected: Observable<AbstractMesh>;
        /**
         * Observable raised when a new mesh is selected based on meshSelectionPredicate.
         * This observable will provide the mesh and the controller used to select the mesh
         */
        onMeshSelectedWithController: Observable<{
            mesh: AbstractMesh;
            controller: WebVRController;
        }>;
        /**
         * Observable raised when a new mesh is picked based on meshSelectionPredicate
         */
        onNewMeshPicked: Observable<PickingInfo>;
        private _circleEase;
        /**
         * Observable raised before camera teleportation
         */
        onBeforeCameraTeleport: Observable<Vector3>;
        /**
         *  Observable raised after camera teleportation
         */
        onAfterCameraTeleport: Observable<Vector3>;
        /**
         * Observable raised when current selected mesh gets unselected
         */
        onSelectedMeshUnselected: Observable<AbstractMesh>;
        private _raySelectionPredicate;
        /**
         * To be optionally changed by user to define custom ray selection
         */
        raySelectionPredicate: (mesh: AbstractMesh) => boolean;
        /**
         * To be optionally changed by user to define custom selection logic (after ray selection)
         */
        meshSelectionPredicate: (mesh: AbstractMesh) => boolean;
        /**
         * Set teleportation enabled. If set to false camera teleportation will be disabled but camera rotation will be kept.
         */
        teleportationEnabled: boolean;
        private _defaultHeight;
        private _teleportationInitialized;
        private _interactionsEnabled;
        private _interactionsRequested;
        private _displayGaze;
        private _displayLaserPointer;
        /**
         * The mesh used to display where the user is going to teleport.
         */
        get teleportationTarget(): Mesh;
        /**
         * Sets the mesh to be used to display where the user is going to teleport.
         */
        set teleportationTarget(value: Mesh);
        /**
         * The mesh used to display where the user is selecting, this mesh will be cloned and set as the gazeTracker for the left and right controller
         * when set bakeCurrentTransformIntoVertices will be called on the mesh.
         * See https://doc.babylonjs.com/features/featuresDeepDive/mesh/transforms/center_origin/bakingTransforms
         */
        get gazeTrackerMesh(): Mesh;
        set gazeTrackerMesh(value: Mesh);
        /**
         * If the gaze trackers scale should be updated to be constant size when pointing at near/far meshes
         */
        updateGazeTrackerScale: boolean;
        /**
         * If the gaze trackers color should be updated when selecting meshes
         */
        updateGazeTrackerColor: boolean;
        /**
         * If the controller laser color should be updated when selecting meshes
         */
        updateControllerLaserColor: boolean;
        /**
         * The gaze tracking mesh corresponding to the left controller
         */
        get leftControllerGazeTrackerMesh(): Nullable<Mesh>;
        /**
         * The gaze tracking mesh corresponding to the right controller
         */
        get rightControllerGazeTrackerMesh(): Nullable<Mesh>;
        /**
         * If the ray of the gaze should be displayed.
         */
        get displayGaze(): boolean;
        /**
         * Sets if the ray of the gaze should be displayed.
         */
        set displayGaze(value: boolean);
        /**
         * If the ray of the LaserPointer should be displayed.
         */
        get displayLaserPointer(): boolean;
        /**
         * Sets if the ray of the LaserPointer should be displayed.
         */
        set displayLaserPointer(value: boolean);
        /**
         * The deviceOrientationCamera used as the camera when not in VR.
         */
        get deviceOrientationCamera(): Nullable<DeviceOrientationCamera>;
        /**
         * Based on the current WebVR support, returns the current VR camera used.
         */
        get currentVRCamera(): Nullable<Camera>;
        /**
         * The webVRCamera which is used when in VR.
         */
        get webVRCamera(): WebVRFreeCamera;
        /**
         * The deviceOrientationCamera that is used as a fallback when vr device is not connected.
         */
        get vrDeviceOrientationCamera(): Nullable<VRDeviceOrientationFreeCamera>;
        /**
         * The html button that is used to trigger entering into VR.
         */
        get vrButton(): Nullable<HTMLButtonElement>;
        private get _teleportationRequestInitiated();
        /**
         * Defines whether or not Pointer lock should be requested when switching to
         * full screen.
         */
        requestPointerLockOnFullScreen: boolean;
        /**
         * If asking to force XR, this will be populated with the default xr experience
         */
        xr: WebXRDefaultExperience;
        /**
         * Was the XR test done already. If this is true AND this.xr exists, xr is initialized.
         * If this is true and no this.xr, xr exists but is not supported, using WebVR.
         */
        xrTestDone: boolean;
        /**
         * Instantiates a VRExperienceHelper.
         * Helps to quickly add VR support to an existing scene.
         * @param scene The scene the VRExperienceHelper belongs to.
         * @param webVROptions Options to modify the vr experience helper's behavior.
         */
        constructor(scene: Scene, 
        /** Options to modify the vr experience helper's behavior. */
        webVROptions?: VRExperienceHelperOptions);
        private _completeVRInit;
        private _onDefaultMeshLoaded;
        private _onResize;
        private _onFullscreenChange;
        /**
         * Gets a value indicating if we are currently in VR mode.
         */
        get isInVRMode(): boolean;
        private _onVrDisplayPresentChange;
        private _onVRDisplayChanged;
        private _moveButtonToBottomRight;
        private _displayVRButton;
        private _updateButtonVisibility;
        private _cachedAngularSensibility;
        /**
         * Attempt to enter VR. If a headset is connected and ready, will request present on that.
         * Otherwise, will use the fullscreen API.
         */
        enterVR(): void;
        /**
         * Attempt to exit VR, or fullscreen.
         */
        exitVR(): void;
        /**
         * The position of the vr experience helper.
         */
        get position(): Vector3;
        /**
         * Sets the position of the vr experience helper.
         */
        set position(value: Vector3);
        /**
         * Enables controllers and user interactions such as selecting and object or clicking on an object.
         */
        enableInteractions(): void;
        private get _noControllerIsActive();
        private _beforeRender;
        private _isTeleportationFloor;
        /**
         * Adds a floor mesh to be used for teleportation.
         * @param floorMesh the mesh to be used for teleportation.
         */
        addFloorMesh(floorMesh: Mesh): void;
        /**
         * Removes a floor mesh from being used for teleportation.
         * @param floorMesh the mesh to be removed.
         */
        removeFloorMesh(floorMesh: Mesh): void;
        /**
         * Enables interactions and teleportation using the VR controllers and gaze.
         * @param vrTeleportationOptions options to modify teleportation behavior.
         */
        enableTeleportation(vrTeleportationOptions?: VRTeleportationOptions): void;
        private _onNewGamepadConnected;
        private _tryEnableInteractionOnController;
        private _onNewGamepadDisconnected;
        private _enableInteractionOnController;
        private _checkTeleportWithRay;
        private _checkRotate;
        private _checkTeleportBackwards;
        private _enableTeleportationOnController;
        private _createTeleportationCircles;
        private _displayTeleportationTarget;
        private _hideTeleportationTarget;
        private _rotateCamera;
        private _moveTeleportationSelectorTo;
        private _workingVector;
        private _workingQuaternion;
        private _workingMatrix;
        /**
         * Time Constant Teleportation Mode
         */
        static readonly TELEPORTATIONMODE_CONSTANTTIME = 0;
        /**
         * Speed Constant Teleportation Mode
         */
        static readonly TELEPORTATIONMODE_CONSTANTSPEED = 1;
        /**
         * Teleports the users feet to the desired location
         * @param location The location where the user's feet should be placed
         */
        teleportCamera(location: Vector3): void;
        private _convertNormalToDirectionOfRay;
        private _castRayAndSelectObject;
        private _notifySelectedMeshUnselected;
        /**
         * Permanently set new colors for the laser pointer
         * @param color the new laser color
         * @param pickedColor the new laser color when picked mesh detected
         */
        setLaserColor(color: Color3, pickedColor?: Color3): void;
        /**
         * Set lighting enabled / disabled on the laser pointer of both controllers
         * @param enabled should the lighting be enabled on the laser pointer
         */
        setLaserLightingState(enabled?: boolean): void;
        /**
         * Permanently set new colors for the gaze pointer
         * @param color the new gaze color
         * @param pickedColor the new gaze color when picked mesh detected
         */
        setGazeColor(color: Color3, pickedColor?: Color3): void;
        /**
         * Sets the color of the laser ray from the vr controllers.
         * @param color new color for the ray.
         */
        changeLaserColor(color: Color3): void;
        /**
         * Sets the color of the ray from the vr headsets gaze.
         * @param color new color for the ray.
         */
        changeGazeColor(color: Color3): void;
        /**
         * Exits VR and disposes of the vr experience helper
         */
        dispose(): void;
        /**
         * Gets the name of the VRExperienceHelper class
         * @returns "VRExperienceHelper"
         */
        getClassName(): string;
    }


    /**
     * This is a copy of VRPose. See https://developer.mozilla.org/en-US/docs/Web/API/VRPose
     * IMPORTANT!! The data is right-hand data.
     * @export
     * @interface DevicePose
     */
    export interface DevicePose {
        /**
         * The position of the device, values in array are [x,y,z].
         */
        readonly position: Nullable<Float32Array>;
        /**
         * The linearVelocity of the device, values in array are [x,y,z].
         */
        readonly linearVelocity: Nullable<Float32Array>;
        /**
         * The linearAcceleration of the device, values in array are [x,y,z].
         */
        readonly linearAcceleration: Nullable<Float32Array>;
        /**
         * The orientation of the device in a quaternion array, values in array are [x,y,z,w].
         */
        readonly orientation: Nullable<Float32Array>;
        /**
         * The angularVelocity of the device, values in array are [x,y,z].
         */
        readonly angularVelocity: Nullable<Float32Array>;
        /**
         * The angularAcceleration of the device, values in array are [x,y,z].
         */
        readonly angularAcceleration: Nullable<Float32Array>;
    }
    /**
     * Interface representing a pose controlled object in Babylon.
     * A pose controlled object has both regular pose values as well as pose values
     * from an external device such as a VR head mounted display
     */
    export interface PoseControlled {
        /**
         * The position of the object in babylon space.
         */
        position: Vector3;
        /**
         * The rotation quaternion of the object in babylon space.
         */
        rotationQuaternion: Quaternion;
        /**
         * The position of the device in babylon space.
         */
        devicePosition?: Vector3;
        /**
         * The rotation quaternion of the device in babylon space.
         */
        deviceRotationQuaternion: Quaternion;
        /**
         * The raw pose coming from the device.
         */
        rawPose: Nullable<DevicePose>;
        /**
         * The scale of the device to be used when translating from device space to babylon space.
         */
        deviceScaleFactor: number;
        /**
         * Updates the poseControlled values based on the input device pose.
         * @param poseData the pose data to update the object with
         */
        updateFromDevice(poseData: DevicePose): void;
    }
    /**
     * Set of options to customize the webVRCamera
     */
    export interface WebVROptions {
        /**
         * Sets if the webVR camera should be tracked to the vrDevice. (default: true)
         */
        trackPosition?: boolean;
        /**
         * Sets the scale of the vrDevice in babylon space. (default: 1)
         */
        positionScale?: number;
        /**
         * If there are more than one VRDisplays, this will choose the display matching this name. (default: pick first vrDisplay)
         */
        displayName?: string;
        /**
         * Should the native controller meshes be initialized. (default: true)
         */
        controllerMeshes?: boolean;
        /**
         * Creating a default HemiLight only on controllers. (default: true)
         */
        defaultLightingOnControllers?: boolean;
        /**
         * If you don't want to use the default VR button of the helper. (default: false)
         */
        useCustomVRButton?: boolean;
        /**
         * If you'd like to provide your own button to the VRHelper. (default: standard babylon vr button)
         */
        customVRButton?: HTMLButtonElement;
        /**
         * To change the length of the ray for gaze/controllers. Will be scaled by positionScale. (default: 100)
         */
        rayLength?: number;
        /**
         * To change the default offset from the ground to account for user's height in meters. Will be scaled by positionScale. (default: 1.7)
         */
        defaultHeight?: number;
        /**
         * If multiview should be used if available (default: false)
         */
        useMultiview?: boolean;
    }
    /**
     * This represents a WebVR camera.
     * The WebVR camera is Babylon's simple interface to interaction with Windows Mixed Reality, HTC Vive and Oculus Rift.
     * @deprecated Use WebXR instead - https://doc.babylonjs.com/features/featuresDeepDive/webXR
     * @example https://doc.babylonjs.com/features/featuresDeepDive/cameras/webVRCamera
     */
    export class WebVRFreeCamera extends FreeCamera implements PoseControlled {
        private _webVROptions;
        /**
         * @internal
         * The vrDisplay tied to the camera. See https://developer.mozilla.org/en-US/docs/Web/API/VRDisplay
         */
        _vrDevice: any;
        /**
         * The rawPose of the vrDevice.
         */
        rawPose: Nullable<DevicePose>;
        private _onVREnabled;
        private _specsVersion;
        private _attached;
        private _frameData;
        protected _descendants: Array<Node>;
        private _deviceRoomPosition;
        /** @internal */
        _deviceRoomRotationQuaternion: Quaternion;
        private _standingMatrix;
        /**
         * Represents device position in babylon space.
         */
        devicePosition: Vector3;
        /**
         * Represents device rotation in babylon space.
         */
        deviceRotationQuaternion: Quaternion;
        /**
         * The scale of the device to be used when translating from device space to babylon space.
         */
        deviceScaleFactor: number;
        private _deviceToWorld;
        private _worldToDevice;
        /**
         * References to the webVR controllers for the vrDevice.
         */
        controllers: Array<WebVRController>;
        /**
         * Emits an event when a controller is attached.
         */
        onControllersAttachedObservable: Observable<WebVRController[]>;
        /**
         * Emits an event when a controller's mesh has been loaded;
         */
        onControllerMeshLoadedObservable: Observable<WebVRController>;
        /**
         * Emits an event when the HMD's pose has been updated.
         */
        onPoseUpdatedFromDeviceObservable: Observable<any>;
        private _poseSet;
        /**
         * If the rig cameras be used as parent instead of this camera.
         */
        rigParenting: boolean;
        private _lightOnControllers;
        private _defaultHeight?;
        /**
         * Instantiates a WebVRFreeCamera.
         * @param name The name of the WebVRFreeCamera
         * @param position The starting anchor position for the camera
         * @param scene The scene the camera belongs to
         * @param _webVROptions a set of customizable options for the webVRCamera
         */
        constructor(name: string, position: Vector3, scene?: Scene, _webVROptions?: WebVROptions);
        protected _setRigMode: any;
        /**
         * Gets the device distance from the ground in meters.
         * @returns the distance in meters from the vrDevice to ground in device space. If standing matrix is not supported for the vrDevice 0 is returned.
         */
        deviceDistanceToRoomGround(): number;
        /**
         * Enables the standing matrix when supported. This can be used to position the user's view the correct height from the ground.
         * @param callback will be called when the standing matrix is set. Callback parameter is if the standing matrix is supported.
         */
        useStandingMatrix(callback?: (bool: boolean) => void): void;
        /**
         * Enables the standing matrix when supported. This can be used to position the user's view the correct height from the ground.
         * @returns A promise with a boolean set to if the standing matrix is supported.
         */
        useStandingMatrixAsync(): Promise<boolean>;
        /**
         * Disposes the camera
         */
        dispose(): void;
        /**
         * Gets a vrController by name.
         * @param name The name of the controller to retrieve
         * @returns the controller matching the name specified or null if not found
         */
        getControllerByName(name: string): Nullable<WebVRController>;
        private _leftController;
        /**
         * The controller corresponding to the users left hand.
         */
        get leftController(): Nullable<WebVRController>;
        private _rightController;
        /**
         * The controller corresponding to the users right hand.
         */
        get rightController(): Nullable<WebVRController>;
        /**
         * Casts a ray forward from the vrCamera's gaze.
         * @param length Length of the ray (default: 100)
         * @returns the ray corresponding to the gaze
         */
        getForwardRay(length?: number): Ray;
        /**
         * @internal
         * Updates the camera based on device's frame data
         */
        _checkInputs(): void;
        /**
         * Updates the poseControlled values based on the input device pose.
         * @param poseData Pose coming from the device
         */
        updateFromDevice(poseData: DevicePose): void;
        private _detachIfAttached;
        /**
         * WebVR's attach control will start broadcasting frames to the device.
         * Note that in certain browsers (chrome for example) this function must be called
         * within a user-interaction callback. Example:
         * <pre> scene.onPointerDown = function() { camera.attachControl(canvas); }</pre>
         *
         * @param noPreventDefault prevent the default html element operation when attaching the vrDevice
         */
        attachControl(noPreventDefault?: boolean): void;
        /**
         * Detach the current controls from the specified dom element.
         */
        detachControl(): void;
        /**
         * @returns the name of this class
         */
        getClassName(): string;
        /**
         * Calls resetPose on the vrDisplay
         * See: https://developer.mozilla.org/en-US/docs/Web/API/VRDisplay/resetPose
         */
        resetToCurrentRotation(): void;
        /**
         * @internal
         * Updates the rig cameras (left and right eye)
         */
        _updateRigCameras(): void;
        private _workingVector;
        private _oneVector;
        private _workingMatrix;
        private _updateCacheCalled;
        private _correctPositionIfNotTrackPosition;
        /**
         * @internal
         * Updates the cached values of the camera
         * @param ignoreParentClass ignores updating the parent class's cache (default: false)
         */
        _updateCache(ignoreParentClass?: boolean): void;
        /**
         * @internal
         * Get current device position in babylon world
         */
        _computeDevicePosition(): void;
        /**
         * Updates the current device position and rotation in the babylon world
         */
        update(): void;
        /**
         * @internal
         * Gets the view matrix of this camera (Always set to identity as left and right eye cameras contain the actual view matrix)
         * @returns an identity matrix
         */
        _getViewMatrix(): Matrix;
        private _tmpMatrix;
        /**
         * This function is called by the two RIG cameras.
         * 'this' is the left or right camera (and NOT (!!!) the WebVRFreeCamera instance)
         * @internal
         */
        _getWebVRViewMatrix(): Matrix;
        /** @internal */
        _getWebVRProjectionMatrix(): Matrix;
        private _onGamepadConnectedObserver;
        private _onGamepadDisconnectedObserver;
        private _updateCacheWhenTrackingDisabledObserver;
        /**
         * Initializes the controllers and their meshes
         */
        initControllers(): void;
    }


    /** @internal */
    export class Collider {
        /** Define if a collision was found */
        collisionFound: boolean;
        /**
         * Define last intersection point in local space
         */
        intersectionPoint: Vector3;
        /**
         * Define last collided mesh
         */
        collidedMesh: Nullable<AbstractMesh>;
        /**
         * If true, it check for double sided faces and only returns 1 collision instead of 2
         */
        static DoubleSidedCheck: boolean;
        private _collisionPoint;
        private _planeIntersectionPoint;
        private _tempVector;
        private _tempVector2;
        private _tempVector3;
        private _tempVector4;
        private _edge;
        private _baseToVertex;
        private _destinationPoint;
        private _slidePlaneNormal;
        private _displacementVector;
        /** @internal */
        _radius: Vector3;
        /** @internal */
        _retry: number;
        private _velocity;
        private _basePoint;
        private _epsilon;
        /** @internal */
        _velocityWorldLength: number;
        /** @internal */
        _basePointWorld: Vector3;
        private _velocityWorld;
        private _normalizedVelocity;
        /** @internal */
        _initialVelocity: Vector3;
        /** @internal */
        _initialPosition: Vector3;
        private _nearestDistance;
        private _collisionMask;
        private _velocitySquaredLength;
        private _nearestDistanceSquared;
        get collisionMask(): number;
        set collisionMask(mask: number);
        /**
         * Gets the plane normal used to compute the sliding response (in local space)
         */
        get slidePlaneNormal(): Vector3;
        /**
         * @internal
         */
        _initialize(source: Vector3, dir: Vector3, e: number): void;
        /**
         * @internal
         */
        _checkPointInTriangle(point: Vector3, pa: Vector3, pb: Vector3, pc: Vector3, n: Vector3): boolean;
        /**
         * @internal
         */
        _canDoCollision(sphereCenter: Vector3, sphereRadius: number, vecMin: Vector3, vecMax: Vector3): boolean;
        /**
         * @internal
         */
        _testTriangle(faceIndex: number, trianglePlaneArray: Array<Plane>, p1: Vector3, p2: Vector3, p3: Vector3, hasMaterial: boolean, hostMesh: AbstractMesh): void;
        /**
         * @internal
         */
        _collide(trianglePlaneArray: Array<Plane>, pts: Vector3[], indices: IndicesArray, indexStart: number, indexEnd: number, decal: number, hasMaterial: boolean, hostMesh: AbstractMesh, invertTriangles?: boolean, triangleStrip?: boolean): void;
        /**
         * @internal
         */
        _getResponse(pos: Vector3, vel: Vector3): void;
    }


    /** @internal */
    export interface ICollisionCoordinator {
        createCollider(): Collider;
        getNewPosition(position: Vector3, displacement: Vector3, collider: Collider, maximumRetry: number, excludedMesh: Nullable<AbstractMesh>, onNewPosition: (collisionIndex: number, newPosition: Vector3, collidedMesh: Nullable<AbstractMesh>) => void, collisionIndex: number): void;
        init(scene: Scene): void;
    }
    /** @internal */
    export class DefaultCollisionCoordinator implements ICollisionCoordinator {
        private _scene;
        private _scaledPosition;
        private _scaledVelocity;
        private _finalPosition;
        getNewPosition(position: Vector3, displacement: Vector3, collider: Collider, maximumRetry: number, excludedMesh: AbstractMesh, onNewPosition: (collisionIndex: number, newPosition: Vector3, collidedMesh: Nullable<AbstractMesh>) => void, collisionIndex: number): void;
        createCollider(): Collider;
        init(scene: Scene): void;
        private _collideWithWorld;
    }




    /**
     * @internal
     */
    export class IntersectionInfo {
        bu: Nullable<number>;
        bv: Nullable<number>;
        distance: number;
        faceId: number;
        subMeshId: number;
        constructor(bu: Nullable<number>, bv: Nullable<number>, distance: number);
    }


    /**
     * @internal
     */
    export class _MeshCollisionData {
        _checkCollisions: boolean;
        _collisionMask: number;
        _collisionGroup: number;
        _surroundingMeshes: Nullable<AbstractMesh[]>;
        _collider: Nullable<Collider>;
        _oldPositionForCollisions: Vector3;
        _diffPositionForCollisions: Vector3;
        _onCollideObserver: Nullable<Observer<AbstractMesh>>;
        _onCollisionPositionChangeObserver: Nullable<Observer<Vector3>>;
        _collisionResponse: boolean;
    }


    /**
     * Information about the result of picking within a scene
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/interactions/picking_collisions
     */
    export class PickingInfo {
        /**
         * If the pick collided with an object
         */
        hit: boolean;
        /**
         * Distance away where the pick collided
         */
        distance: number;
        /**
         * The location of pick collision
         */
        pickedPoint: Nullable<Vector3>;
        /**
         * The mesh corresponding the pick collision
         */
        pickedMesh: Nullable<AbstractMesh>;
        /** (See getTextureCoordinates) The barycentric U coordinate that is used when calculating the texture coordinates of the collision.*/
        bu: number;
        /** (See getTextureCoordinates) The barycentric V coordinate that is used when calculating the texture coordinates of the collision.*/
        bv: number;
        /** The index of the face on the mesh that was picked, or the index of the Line if the picked Mesh is a LinesMesh */
        faceId: number;
        /** The index of the face on the subMesh that was picked, or the index of the Line if the picked Mesh is a LinesMesh */
        subMeshFaceId: number;
        /** Id of the submesh that was picked */
        subMeshId: number;
        /** If a sprite was picked, this will be the sprite the pick collided with */
        pickedSprite: Nullable<Sprite>;
        /** If we are picking a mesh with thin instance, this will give you the picked thin instance */
        thinInstanceIndex: number;
        /**
         * The ray that was used to perform the picking.
         */
        ray: Nullable<Ray>;
        /**
         * If a mesh was used to do the picking (eg. 6dof controller) as a "near interaction", this will be populated.
         */
        originMesh: Nullable<AbstractMesh>;
        /**
         * The aim-space transform of the input used for picking, if it is an XR input source.
         */
        aimTransform: Nullable<TransformNode>;
        /**
         * The grip-space transform of the input used for picking, if it is an XR input source.
         * Some XR sources, such as input coming from head mounted displays, do not have this.
         */
        gripTransform: Nullable<TransformNode>;
        /**
         * Gets the normal corresponding to the face the pick collided with
         * @param useWorldCoordinates If the resulting normal should be relative to the world (default: false)
         * @param useVerticesNormals If the vertices normals should be used to calculate the normal instead of the normal map (default: true)
         * @returns The normal corresponding to the face the pick collided with
         * @remarks Note that the returned normal will always point towards the picking ray.
         */
        getNormal(useWorldCoordinates?: boolean, useVerticesNormals?: boolean): Nullable<Vector3>;
        /**
         * Gets the texture coordinates of where the pick occurred
         * @returns The vector containing the coordinates of the texture
         */
        getTextureCoordinates(): Nullable<Vector2>;
    }


    /**
     * Options used to control default behaviors regarding compatibility support
     */
    export class CompatibilityOptions {
        /**
         * Defines if the system should use OpenGL convention for UVs when creating geometry or loading .babylon files (false by default)
         */
        static UseOpenGLOrientationForUV: boolean;
    }




    /**
     * Options to be used when creating a compute effect.
     */
    export interface IComputeEffectCreationOptions {
        /**
         * Define statements that will be set in the shader.
         */
        defines: any;
        /**
         * The name of the entry point in the shader source (default: "main")
         */
        entryPoint?: string;
        /**
         * Callback that will be called when the shader is compiled.
         */
        onCompiled: Nullable<(effect: ComputeEffect) => void>;
        /**
         * Callback that will be called if an error occurs during shader compilation.
         */
        onError: Nullable<(effect: ComputeEffect, errors: string) => void>;
        /**
         * If provided, will be called with the shader code so that this code can be updated before it is compiled by the GPU
         */
        processFinalCode?: Nullable<(code: string) => string>;
    }
    /**
     * Effect wrapping a compute shader and let execute (dispatch) the shader
     */
    export class ComputeEffect {
        private static _UniqueIdSeed;
        /**
         * Enable logging of the shader code when a compilation error occurs
         */
        static LogShaderCodeOnCompilationError: boolean;
        /**
         * Name of the effect.
         */
        name: any;
        /**
         * String container all the define statements that should be set on the shader.
         */
        defines: string;
        /**
         * Callback that will be called when the shader is compiled.
         */
        onCompiled: Nullable<(effect: ComputeEffect) => void>;
        /**
         * Callback that will be called if an error occurs during shader compilation.
         */
        onError: Nullable<(effect: ComputeEffect, errors: string) => void>;
        /**
         * Unique ID of the effect.
         */
        uniqueId: number;
        /**
         * Observable that will be called when the shader is compiled.
         * It is recommended to use executeWhenCompile() or to make sure that scene.isReady() is called to get this observable raised.
         */
        onCompileObservable: Observable<ComputeEffect>;
        /**
         * Observable that will be called if an error occurs during shader compilation.
         */
        onErrorObservable: Observable<ComputeEffect>;
        /**
         * Observable that will be called when effect is bound.
         */
        onBindObservable: Observable<ComputeEffect>;
        /**
         * @internal
         * Specifies if the effect was previously ready
         */
        _wasPreviouslyReady: boolean;
        private _engine;
        private _isReady;
        private _compilationError;
        /** @internal */
        _key: string;
        private _computeSourceCodeOverride;
        /** @internal */
        _pipelineContext: Nullable<IComputePipelineContext>;
        /** @internal */
        _computeSourceCode: string;
        private _rawComputeSourceCode;
        private _entryPoint;
        private _shaderLanguage;
        private _shaderStore;
        private _shaderRepository;
        private _includeShaderStore;
        /**
         * Creates a compute effect that can be used to execute a compute shader
         * @param baseName Name of the effect
         * @param options Set of all options to create the effect
         * @param engine The engine the effect is created for
         * @param key Effect Key identifying uniquely compiled shader variants
         */
        constructor(baseName: any, options: IComputeEffectCreationOptions, engine: Engine, key?: string);
        private _useFinalCode;
        /**
         * Unique key for this effect
         */
        get key(): string;
        /**
         * If the effect has been compiled and prepared.
         * @returns if the effect is compiled and prepared.
         */
        isReady(): boolean;
        private _isReadyInternal;
        /**
         * The engine the effect was initialized with.
         * @returns the engine.
         */
        getEngine(): Engine;
        /**
         * The pipeline context for this effect
         * @returns the associated pipeline context
         */
        getPipelineContext(): Nullable<IComputePipelineContext>;
        /**
         * The error from the last compilation.
         * @returns the error string.
         */
        getCompilationError(): string;
        /**
         * Adds a callback to the onCompiled observable and call the callback immediately if already ready.
         * @param func The callback to be used.
         */
        executeWhenCompiled(func: (effect: ComputeEffect) => void): void;
        private _checkIsReady;
        private _loadShader;
        /**
         * Gets the compute shader source code of this effect
         */
        get computeSourceCode(): string;
        /**
         * Gets the compute shader source code before it has been processed by the preprocessor
         */
        get rawComputeSourceCode(): string;
        /**
         * Prepares the effect
         * @internal
         */
        _prepareEffect(): void;
        private _getShaderCodeAndErrorLine;
        private _processCompilationErrors;
        /**
         * Release all associated resources.
         **/
        dispose(): void;
        /**
         * This function will add a new compute shader to the shader store
         * @param name the name of the shader
         * @param computeShader compute shader content
         */
        static RegisterShader(name: string, computeShader: string): void;
    }


    /**
     * Defines the options associated with the creation of a compute shader.
     */
    export interface IComputeShaderOptions {
        /**
         * list of bindings mapping (key is property name, value is binding location)
         * Must be provided because browsers don't support reflection for wgsl shaders yet (so there's no way to query the binding/group from a variable name)
         * TODO: remove this when browsers support reflection for wgsl shaders
         */
        bindingsMapping: ComputeBindingMapping;
        /**
         * The list of defines used in the shader
         */
        defines?: string[];
        /**
         * The name of the entry point in the shader source (default: "main")
         */
        entryPoint?: string;
        /**
         * If provided, will be called with the shader code so that this code can be updated before it is compiled by the GPU
         */
        processFinalCode?: Nullable<(code: string) => string>;
    }
    /**
     * The ComputeShader object lets you execute a compute shader on your GPU (if supported by the engine)
     */
    export class ComputeShader {
        private _engine;
        private _shaderPath;
        private _options;
        private _effect;
        private _cachedDefines;
        private _bindings;
        private _samplers;
        private _context;
        private _contextIsDirty;
        /**
         * Gets the unique id of the compute shader
         */
        readonly uniqueId: number;
        /**
         * The name of the shader
         */
        name: string;
        /**
         * The options used to create the shader
         */
        get options(): IComputeShaderOptions;
        /**
         * The shaderPath used to create the shader
         */
        get shaderPath(): any;
        /**
         * Callback triggered when the shader is compiled
         */
        onCompiled: Nullable<(effect: ComputeEffect) => void>;
        /**
         * Callback triggered when an error occurs
         */
        onError: Nullable<(effect: ComputeEffect, errors: string) => void>;
        /**
         * Instantiates a new compute shader.
         * @param name Defines the name of the compute shader in the scene
         * @param engine Defines the engine the compute shader belongs to
         * @param shaderPath Defines  the route to the shader code in one of three ways:
         *  * object: { compute: "custom" }, used with ShaderStore.ShadersStoreWGSL["customComputeShader"]
         *  * object: { computeElement: "HTMLElementId" }, used with shader code in script tags
         *  * object: { computeSource: "compute shader code string" using with string containing the shader code
         *  * string: try first to find the code in ShaderStore.ShadersStoreWGSL[shaderPath + "ComputeShader"]. If not, assumes it is a file with name shaderPath.compute.fx in index.html folder.
         * @param options Define the options used to create the shader
         */
        constructor(name: string, engine: ThinEngine, shaderPath: any, options?: Partial<IComputeShaderOptions>);
        /**
         * Gets the current class name of the material e.g. "ComputeShader"
         * Mainly use in serialization.
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Binds a texture to the shader
         * @param name Binding name of the texture
         * @param texture Texture to bind
         * @param bindSampler Bind the sampler corresponding to the texture (default: true). The sampler will be bound just before the binding index of the texture
         */
        setTexture(name: string, texture: BaseTexture, bindSampler?: boolean): void;
        /**
         * Binds a storage texture to the shader
         * @param name Binding name of the texture
         * @param texture Texture to bind
         */
        setStorageTexture(name: string, texture: BaseTexture): void;
        /**
         * Binds a uniform buffer to the shader
         * @param name Binding name of the buffer
         * @param buffer Buffer to bind
         */
        setUniformBuffer(name: string, buffer: UniformBuffer): void;
        /**
         * Binds a storage buffer to the shader
         * @param name Binding name of the buffer
         * @param buffer Buffer to bind
         */
        setStorageBuffer(name: string, buffer: StorageBuffer): void;
        /**
         * Binds a texture sampler to the shader
         * @param name Binding name of the sampler
         * @param sampler Sampler to bind
         */
        setTextureSampler(name: string, sampler: TextureSampler): void;
        /**
         * Specifies that the compute shader is ready to be executed (the compute effect and all the resources are ready)
         * @returns true if the compute shader is ready to be executed
         */
        isReady(): boolean;
        /**
         * Dispatches (executes) the compute shader
         * @param x Number of workgroups to execute on the X dimension
         * @param y Number of workgroups to execute on the Y dimension (default: 1)
         * @param z Number of workgroups to execute on the Z dimension (default: 1)
         * @returns True if the dispatch could be done, else false (meaning either the compute effect or at least one of the bound resources was not ready)
         */
        dispatch(x: number, y?: number, z?: number): boolean;
        /**
         * Waits for the compute shader to be ready and executes it
         * @param x Number of workgroups to execute on the X dimension
         * @param y Number of workgroups to execute on the Y dimension (default: 1)
         * @param z Number of workgroups to execute on the Z dimension (default: 1)
         * @param delay Delay between the retries while the shader is not ready (in milliseconds - 10 by default)
         * @returns A promise that is resolved once the shader has been sent to the GPU. Note that it does not mean that the shader execution itself is finished!
         */
        dispatchWhenReady(x: number, y?: number, z?: number, delay?: number): Promise<void>;
        /**
         * Serializes this compute shader in a JSON representation
         * @returns the serialized compute shader object
         */
        serialize(): any;
        /**
         * Creates a compute shader from parsed compute shader data
         * @param source defines the JSON representation of the compute shader
         * @param scene defines the hosting scene
         * @param rootUrl defines the root URL to use to load textures and relative dependencies
         * @returns a new compute shader
         */
        static Parse(source: any, scene: Scene, rootUrl: string): ComputeShader;
    }


    /** @internal */
    export interface IComputeContext {
        clear(): void;
    }


    /**
     * Class used to store and describe the pipeline context associated with a compute effect
     */
    export interface IComputePipelineContext {
        /**
         * Gets a boolean indicating that this pipeline context is supporting asynchronous creating
         */
        isAsync: boolean;
        /**
         * Gets a boolean indicating that the context is ready to be used (like shader / pipeline are compiled and ready for instance)
         */
        isReady: boolean;
        /** @internal */
        _name?: string;
        /** @internal */
        _getComputeShaderCode(): string | null;
        /** Releases the resources associated with the pipeline. */
        dispose(): void;
    }




    /**
     * Class used to store bounding box information
     */
    export class BoundingBox implements ICullable {
        /**
         * Gets the 8 vectors representing the bounding box in local space
         */
        readonly vectors: Vector3[];
        /**
         * Gets the center of the bounding box in local space
         */
        readonly center: Vector3;
        /**
         * Gets the center of the bounding box in world space
         */
        readonly centerWorld: Vector3;
        /**
         * Gets the extend size in local space
         */
        readonly extendSize: Vector3;
        /**
         * Gets the extend size in world space
         */
        readonly extendSizeWorld: Vector3;
        /**
         * Gets the OBB (object bounding box) directions
         */
        readonly directions: Vector3[];
        /**
         * Gets the 8 vectors representing the bounding box in world space
         */
        readonly vectorsWorld: Vector3[];
        /**
         * Gets the minimum vector in world space
         */
        readonly minimumWorld: Vector3;
        /**
         * Gets the maximum vector in world space
         */
        readonly maximumWorld: Vector3;
        /**
         * Gets the minimum vector in local space
         */
        readonly minimum: Vector3;
        /**
         * Gets the maximum vector in local space
         */
        readonly maximum: Vector3;
        private _worldMatrix;
        private static readonly _TmpVector3;
        /**
         * @internal
         */
        _tag: number;
        /** @internal */
        _drawWrapperFront: Nullable<DrawWrapper>;
        /** @internal */
        _drawWrapperBack: Nullable<DrawWrapper>;
        /**
         * Creates a new bounding box
         * @param min defines the minimum vector (in local space)
         * @param max defines the maximum vector (in local space)
         * @param worldMatrix defines the new world matrix
         */
        constructor(min: DeepImmutable<Vector3>, max: DeepImmutable<Vector3>, worldMatrix?: DeepImmutable<Matrix>);
        /**
         * Recreates the entire bounding box from scratch as if we call the constructor in place
         * @param min defines the new minimum vector (in local space)
         * @param max defines the new maximum vector (in local space)
         * @param worldMatrix defines the new world matrix
         */
        reConstruct(min: DeepImmutable<Vector3>, max: DeepImmutable<Vector3>, worldMatrix?: DeepImmutable<Matrix>): void;
        /**
         * Scale the current bounding box by applying a scale factor
         * @param factor defines the scale factor to apply
         * @returns the current bounding box
         */
        scale(factor: number): BoundingBox;
        /**
         * Gets the world matrix of the bounding box
         * @returns a matrix
         */
        getWorldMatrix(): DeepImmutable<Matrix>;
        /**
         * @internal
         */
        _update(world: DeepImmutable<Matrix>): void;
        /**
         * Tests if the bounding box is intersecting the frustum planes
         * @param frustumPlanes defines the frustum planes to test
         * @returns true if there is an intersection
         */
        isInFrustum(frustumPlanes: Array<DeepImmutable<Plane>>): boolean;
        /**
         * Tests if the bounding box is entirely inside the frustum planes
         * @param frustumPlanes defines the frustum planes to test
         * @returns true if there is an inclusion
         */
        isCompletelyInFrustum(frustumPlanes: Array<DeepImmutable<Plane>>): boolean;
        /**
         * Tests if a point is inside the bounding box
         * @param point defines the point to test
         * @returns true if the point is inside the bounding box
         */
        intersectsPoint(point: DeepImmutable<Vector3>): boolean;
        /**
         * Tests if the bounding box intersects with a bounding sphere
         * @param sphere defines the sphere to test
         * @returns true if there is an intersection
         */
        intersectsSphere(sphere: DeepImmutable<BoundingSphere>): boolean;
        /**
         * Tests if the bounding box intersects with a box defined by a min and max vectors
         * @param min defines the min vector to use
         * @param max defines the max vector to use
         * @returns true if there is an intersection
         */
        intersectsMinMax(min: DeepImmutable<Vector3>, max: DeepImmutable<Vector3>): boolean;
        /**
         * Disposes the resources of the class
         */
        dispose(): void;
        /**
         * Tests if two bounding boxes are intersections
         * @param box0 defines the first box to test
         * @param box1 defines the second box to test
         * @returns true if there is an intersection
         */
        static Intersects(box0: DeepImmutable<BoundingBox>, box1: DeepImmutable<BoundingBox>): boolean;
        /**
         * Tests if a bounding box defines by a min/max vectors intersects a sphere
         * @param minPoint defines the minimum vector of the bounding box
         * @param maxPoint defines the maximum vector of the bounding box
         * @param sphereCenter defines the sphere center
         * @param sphereRadius defines the sphere radius
         * @returns true if there is an intersection
         */
        static IntersectsSphere(minPoint: DeepImmutable<Vector3>, maxPoint: DeepImmutable<Vector3>, sphereCenter: DeepImmutable<Vector3>, sphereRadius: number): boolean;
        /**
         * Tests if a bounding box defined with 8 vectors is entirely inside frustum planes
         * @param boundingVectors defines an array of 8 vectors representing a bounding box
         * @param frustumPlanes defines the frustum planes to test
         * @returns true if there is an inclusion
         */
        static IsCompletelyInFrustum(boundingVectors: Array<DeepImmutable<Vector3>>, frustumPlanes: Array<DeepImmutable<Plane>>): boolean;
        /**
         * Tests if a bounding box defined with 8 vectors intersects frustum planes
         * @param boundingVectors defines an array of 8 vectors representing a bounding box
         * @param frustumPlanes defines the frustum planes to test
         * @returns true if there is an intersection
         */
        static IsInFrustum(boundingVectors: Array<DeepImmutable<Vector3>>, frustumPlanes: Array<DeepImmutable<Plane>>): boolean;
    }


    /**
     * Interface for cullable objects
     * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/materials_introduction#back-face-culling
     */
    export interface ICullable {
        /**
         * Checks if the object or part of the object is in the frustum
         * @param frustumPlanes Camera near/planes
         * @returns true if the object is in frustum otherwise false
         */
        isInFrustum(frustumPlanes: Plane[]): boolean;
        /**
         * Checks if a cullable object (mesh...) is in the camera frustum
         * Unlike isInFrustum this checks the full bounding box
         * @param frustumPlanes Camera near/planes
         * @returns true if the object is in frustum otherwise false
         */
        isCompletelyInFrustum(frustumPlanes: Plane[]): boolean;
    }
    /**
     * Info for a bounding data of a mesh
     */
    export class BoundingInfo implements ICullable {
        /**
         * Bounding box for the mesh
         */
        readonly boundingBox: BoundingBox;
        /**
         * Bounding sphere for the mesh
         */
        readonly boundingSphere: BoundingSphere;
        private _isLocked;
        private static readonly _TmpVector3;
        /**
         * Constructs bounding info
         * @param minimum min vector of the bounding box/sphere
         * @param maximum max vector of the bounding box/sphere
         * @param worldMatrix defines the new world matrix
         */
        constructor(minimum: DeepImmutable<Vector3>, maximum: DeepImmutable<Vector3>, worldMatrix?: DeepImmutable<Matrix>);
        /**
         * Recreates the entire bounding info from scratch as if we call the constructor in place
         * @param min defines the new minimum vector (in local space)
         * @param max defines the new maximum vector (in local space)
         * @param worldMatrix defines the new world matrix
         */
        reConstruct(min: DeepImmutable<Vector3>, max: DeepImmutable<Vector3>, worldMatrix?: DeepImmutable<Matrix>): void;
        /**
         * min vector of the bounding box/sphere
         */
        get minimum(): Vector3;
        /**
         * max vector of the bounding box/sphere
         */
        get maximum(): Vector3;
        /**
         * If the info is locked and won't be updated to avoid perf overhead
         */
        get isLocked(): boolean;
        set isLocked(value: boolean);
        /**
         * Updates the bounding sphere and box
         * @param world world matrix to be used to update
         */
        update(world: DeepImmutable<Matrix>): void;
        /**
         * Recreate the bounding info to be centered around a specific point given a specific extend.
         * @param center New center of the bounding info
         * @param extend New extend of the bounding info
         * @returns the current bounding info
         */
        centerOn(center: DeepImmutable<Vector3>, extend: DeepImmutable<Vector3>): BoundingInfo;
        /**
         * Grows the bounding info to include the given point.
         * @param point The point that will be included in the current bounding info
         * @returns the current bounding info
         */
        encapsulate(point: Vector3): BoundingInfo;
        /**
         * Grows the bounding info to encapsulate the given bounding info.
         * @param toEncapsulate The bounding info that will be encapsulated in the current bounding info
         * @returns the current bounding info
         */
        encapsulateBoundingInfo(toEncapsulate: BoundingInfo): BoundingInfo;
        /**
         * Scale the current bounding info by applying a scale factor
         * @param factor defines the scale factor to apply
         * @returns the current bounding info
         */
        scale(factor: number): BoundingInfo;
        /**
         * Returns `true` if the bounding info is within the frustum defined by the passed array of planes.
         * @param frustumPlanes defines the frustum to test
         * @param strategy defines the strategy to use for the culling (default is BABYLON.AbstractMesh.CULLINGSTRATEGY_STANDARD)
         * The different strategies available are:
         * * BABYLON.AbstractMesh.CULLINGSTRATEGY_STANDARD most accurate but slower @see https://doc.babylonjs.com/typedoc/classes/BABYLON.AbstractMesh#CULLINGSTRATEGY_STANDARD
         * * BABYLON.AbstractMesh.CULLINGSTRATEGY_BOUNDINGSPHERE_ONLY faster but less accurate @see https://doc.babylonjs.com/typedoc/classes/BABYLON.AbstractMesh#CULLINGSTRATEGY_BOUNDINGSPHERE_ONLY
         * * BABYLON.AbstractMesh.CULLINGSTRATEGY_OPTIMISTIC_INCLUSION can be faster if always visible @see https://doc.babylonjs.com/typedoc/classes/BABYLON.AbstractMesh#CULLINGSTRATEGY_OPTIMISTIC_INCLUSION
         * * BABYLON.AbstractMesh.CULLINGSTRATEGY_OPTIMISTIC_INCLUSION_THEN_BSPHERE_ONLY can be faster if always visible @see https://doc.babylonjs.com/typedoc/classes/BABYLON.AbstractMesh#CULLINGSTRATEGY_OPTIMISTIC_INCLUSION_THEN_BSPHERE_ONLY
         * @returns true if the bounding info is in the frustum planes
         */
        isInFrustum(frustumPlanes: Array<DeepImmutable<Plane>>, strategy?: number): boolean;
        /**
         * Gets the world distance between the min and max points of the bounding box
         */
        get diagonalLength(): number;
        /**
         * Checks if a cullable object (mesh...) is in the camera frustum
         * Unlike isInFrustum this checks the full bounding box
         * @param frustumPlanes Camera near/planes
         * @returns true if the object is in frustum otherwise false
         */
        isCompletelyInFrustum(frustumPlanes: Array<DeepImmutable<Plane>>): boolean;
        /**
         * @internal
         */
        _checkCollision(collider: Collider): boolean;
        /**
         * Checks if a point is inside the bounding box and bounding sphere or the mesh
         * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/interactions/mesh_intersect
         * @param point the point to check intersection with
         * @returns if the point intersects
         */
        intersectsPoint(point: DeepImmutable<Vector3>): boolean;
        /**
         * Checks if another bounding info intersects the bounding box and bounding sphere or the mesh
         * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/interactions/mesh_intersect
         * @param boundingInfo the bounding info to check intersection with
         * @param precise if the intersection should be done using OBB
         * @returns if the bounding info intersects
         */
        intersects(boundingInfo: DeepImmutable<BoundingInfo>, precise: boolean): boolean;
    }


    /**
     * Class used to store bounding sphere information
     */
    export class BoundingSphere {
        /**
         * Gets the center of the bounding sphere in local space
         */
        readonly center: Vector3;
        /**
         * Radius of the bounding sphere in local space
         */
        radius: number;
        /**
         * Gets the center of the bounding sphere in world space
         */
        readonly centerWorld: Vector3;
        /**
         * Radius of the bounding sphere in world space
         */
        radiusWorld: number;
        /**
         * Gets the minimum vector in local space
         */
        readonly minimum: Vector3;
        /**
         * Gets the maximum vector in local space
         */
        readonly maximum: Vector3;
        private _worldMatrix;
        private static readonly _TmpVector3;
        /**
         * Creates a new bounding sphere
         * @param min defines the minimum vector (in local space)
         * @param max defines the maximum vector (in local space)
         * @param worldMatrix defines the new world matrix
         */
        constructor(min: DeepImmutable<Vector3>, max: DeepImmutable<Vector3>, worldMatrix?: DeepImmutable<Matrix>);
        /**
         * Recreates the entire bounding sphere from scratch as if we call the constructor in place
         * @param min defines the new minimum vector (in local space)
         * @param max defines the new maximum vector (in local space)
         * @param worldMatrix defines the new world matrix
         */
        reConstruct(min: DeepImmutable<Vector3>, max: DeepImmutable<Vector3>, worldMatrix?: DeepImmutable<Matrix>): void;
        /**
         * Scale the current bounding sphere by applying a scale factor
         * @param factor defines the scale factor to apply
         * @returns the current bounding box
         */
        scale(factor: number): BoundingSphere;
        /**
         * Gets the world matrix of the bounding box
         * @returns a matrix
         */
        getWorldMatrix(): DeepImmutable<Matrix>;
        /**
         * @internal
         */
        _update(worldMatrix: DeepImmutable<Matrix>): void;
        /**
         * Tests if the bounding sphere is intersecting the frustum planes
         * @param frustumPlanes defines the frustum planes to test
         * @returns true if there is an intersection
         */
        isInFrustum(frustumPlanes: Array<DeepImmutable<Plane>>): boolean;
        /**
         * Tests if the bounding sphere center is in between the frustum planes.
         * Used for optimistic fast inclusion.
         * @param frustumPlanes defines the frustum planes to test
         * @returns true if the sphere center is in between the frustum planes
         */
        isCenterInFrustum(frustumPlanes: Array<DeepImmutable<Plane>>): boolean;
        /**
         * Tests if a point is inside the bounding sphere
         * @param point defines the point to test
         * @returns true if the point is inside the bounding sphere
         */
        intersectsPoint(point: DeepImmutable<Vector3>): boolean;
        /**
         * Checks if two sphere intersect
         * @param sphere0 sphere 0
         * @param sphere1 sphere 1
         * @returns true if the spheres intersect
         */
        static Intersects(sphere0: DeepImmutable<BoundingSphere>, sphere1: DeepImmutable<BoundingSphere>): boolean;
        /**
         * Creates a sphere from a center and a radius
         * @param center The center
         * @param radius radius
         * @param matrix Optional worldMatrix
         * @returns The sphere
         */
        static CreateFromCenterAndRadius(center: DeepImmutable<Vector3>, radius: number, matrix?: DeepImmutable<Matrix>): BoundingSphere;
    }






    /**
     * Octrees are a really powerful data structure that can quickly select entities based on space coordinates.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/optimizeOctrees
     */
    export class Octree<T> {
        /** Defines the maximum depth (sub-levels) for your octree. Default value is 2, which means 8 8 8 = 512 blocks :) (This parameter takes precedence over capacity.) */
        maxDepth: number;
        /**
         * Blocks within the octree containing objects
         */
        blocks: Array<OctreeBlock<T>>;
        /**
         * Content stored in the octree
         */
        dynamicContent: T[];
        private _maxBlockCapacity;
        private _selectionContent;
        private _creationFunc;
        /**
         * Creates a octree
         * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/optimizeOctrees
         * @param creationFunc function to be used to instantiate the octree
         * @param maxBlockCapacity defines the maximum number of meshes you want on your octree's leaves (default: 64)
         * @param maxDepth defines the maximum depth (sub-levels) for your octree. Default value is 2, which means 8 8 8 = 512 blocks :) (This parameter takes precedence over capacity.)
         */
        constructor(creationFunc: (entry: T, block: OctreeBlock<T>) => void, maxBlockCapacity?: number, 
        /** Defines the maximum depth (sub-levels) for your octree. Default value is 2, which means 8 8 8 = 512 blocks :) (This parameter takes precedence over capacity.) */
        maxDepth?: number);
        /**
         * Updates the octree by adding blocks for the passed in meshes within the min and max world parameters
         * @param worldMin worldMin for the octree blocks var blockSize = new Vector3((worldMax.x - worldMin.x) / 2, (worldMax.y - worldMin.y) / 2, (worldMax.z - worldMin.z) / 2);
         * @param worldMax worldMax for the octree blocks var blockSize = new Vector3((worldMax.x - worldMin.x) / 2, (worldMax.y - worldMin.y) / 2, (worldMax.z - worldMin.z) / 2);
         * @param entries meshes to be added to the octree blocks
         */
        update(worldMin: Vector3, worldMax: Vector3, entries: T[]): void;
        /**
         * Adds a mesh to the octree
         * @param entry Mesh to add to the octree
         */
        addMesh(entry: T): void;
        /**
         * Remove an element from the octree
         * @param entry defines the element to remove
         */
        removeMesh(entry: T): void;
        /**
         * Selects an array of meshes within the frustum
         * @param frustumPlanes The frustum planes to use which will select all meshes within it
         * @param allowDuplicate If duplicate objects are allowed in the resulting object array
         * @returns array of meshes within the frustum
         */
        select(frustumPlanes: Plane[], allowDuplicate?: boolean): SmartArray<T>;
        /**
         * Test if the octree intersect with the given bounding sphere and if yes, then add its content to the selection array
         * @param sphereCenter defines the bounding sphere center
         * @param sphereRadius defines the bounding sphere radius
         * @param allowDuplicate defines if the selection array can contains duplicated entries
         * @returns an array of objects that intersect the sphere
         */
        intersects(sphereCenter: Vector3, sphereRadius: number, allowDuplicate?: boolean): SmartArray<T>;
        /**
         * Test if the octree intersect with the given ray and if yes, then add its content to resulting array
         * @param ray defines the ray to test with
         * @returns array of intersected objects
         */
        intersectsRay(ray: Ray): SmartArray<T>;
        /**
         * Adds a mesh into the octree block if it intersects the block
         * @param entry
         * @param block
         */
        static CreationFuncForMeshes: (entry: AbstractMesh, block: OctreeBlock<AbstractMesh>) => void;
        /**
         * Adds a submesh into the octree block if it intersects the block
         * @param entry
         * @param block
         */
        static CreationFuncForSubMeshes: (entry: SubMesh, block: OctreeBlock<SubMesh>) => void;
    }


    /**
     * Contains an array of blocks representing the octree
     */
    export interface IOctreeContainer<T> {
        /**
         * Blocks within the octree
         */
        blocks: Array<OctreeBlock<T>>;
    }
    /**
     * Class used to store a cell in an octree
     * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/optimizeOctrees
     */
    export class OctreeBlock<T> {
        /**
         * Gets the content of the current block
         */
        entries: T[];
        /**
         * Gets the list of block children
         */
        blocks: Array<OctreeBlock<T>>;
        private _depth;
        private _maxDepth;
        private _capacity;
        private _minPoint;
        private _maxPoint;
        private _boundingVectors;
        private _creationFunc;
        /**
         * Creates a new block
         * @param minPoint defines the minimum vector (in world space) of the block's bounding box
         * @param maxPoint defines the maximum vector (in world space) of the block's bounding box
         * @param capacity defines the maximum capacity of this block (if capacity is reached the block will be split into sub blocks)
         * @param depth defines the current depth of this block in the octree
         * @param maxDepth defines the maximal depth allowed (beyond this value, the capacity is ignored)
         * @param creationFunc defines a callback to call when an element is added to the block
         */
        constructor(minPoint: Vector3, maxPoint: Vector3, capacity: number, depth: number, maxDepth: number, creationFunc: (entry: T, block: OctreeBlock<T>) => void);
        /**
         * Gets the maximum capacity of this block (if capacity is reached the block will be split into sub blocks)
         */
        get capacity(): number;
        /**
         * Gets the minimum vector (in world space) of the block's bounding box
         */
        get minPoint(): Vector3;
        /**
         * Gets the maximum vector (in world space) of the block's bounding box
         */
        get maxPoint(): Vector3;
        /**
         * Add a new element to this block
         * @param entry defines the element to add
         */
        addEntry(entry: T): void;
        /**
         * Remove an element from this block
         * @param entry defines the element to remove
         */
        removeEntry(entry: T): void;
        /**
         * Add an array of elements to this block
         * @param entries defines the array of elements to add
         */
        addEntries(entries: T[]): void;
        /**
         * Test if the current block intersects the frustum planes and if yes, then add its content to the selection array
         * @param frustumPlanes defines the frustum planes to test
         * @param selection defines the array to store current content if selection is positive
         * @param allowDuplicate defines if the selection array can contains duplicated entries
         */
        select(frustumPlanes: Plane[], selection: SmartArrayNoDuplicate<T>, allowDuplicate?: boolean): void;
        /**
         * Test if the current block intersect with the given bounding sphere and if yes, then add its content to the selection array
         * @param sphereCenter defines the bounding sphere center
         * @param sphereRadius defines the bounding sphere radius
         * @param selection defines the array to store current content if selection is positive
         * @param allowDuplicate defines if the selection array can contains duplicated entries
         */
        intersects(sphereCenter: Vector3, sphereRadius: number, selection: SmartArrayNoDuplicate<T>, allowDuplicate?: boolean): void;
        /**
         * Test if the current block intersect with the given ray and if yes, then add its content to the selection array
         * @param ray defines the ray to test with
         * @param selection defines the array to store current content if selection is positive
         */
        intersectsRay(ray: Ray, selection: SmartArrayNoDuplicate<T>): void;
        /**
         * Subdivide the content into child blocks (this block will then be empty)
         */
        createInnerBlocks(): void;
        /**
         * @internal
         */
        static _CreateBlocks<T>(worldMin: Vector3, worldMax: Vector3, entries: T[], maxBlockCapacity: number, currentDepth: number, maxDepth: number, target: IOctreeContainer<T>, creationFunc: (entry: T, block: OctreeBlock<T>) => void): void;
    }


        interface Scene {
            /**
             * @internal
             * Backing Filed
             */
            _selectionOctree: Octree<AbstractMesh>;
            /**
             * Gets the octree used to boost mesh selection (picking)
             * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/optimizeOctrees
             */
            selectionOctree: Octree<AbstractMesh>;
            /**
             * Creates or updates the octree used to boost selection (picking)
             * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/optimizeOctrees
             * @param maxCapacity defines the maximum capacity per leaf
             * @param maxDepth defines the maximum depth of the octree
             * @returns an octree of AbstractMesh
             */
            createOrUpdateSelectionOctree(maxCapacity?: number, maxDepth?: number): Octree<AbstractMesh>;
        }
        interface AbstractMesh {
            /**
             * @internal
             * Backing Field
             */
            _submeshesOctree: Octree<SubMesh>;
            /**
             * This function will create an octree to help to select the right submeshes for rendering, picking and collision computations.
             * Please note that you must have a decent number of submeshes to get performance improvements when using an octree
             * @param maxCapacity defines the maximum size of each block (64 by default)
             * @param maxDepth defines the maximum depth to use (no more than 2 levels by default)
             * @returns the new octree
             * @see https://www.babylonjs-playground.com/#NA4OQ#12
             * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/optimizeOctrees
             */
            createOrUpdateSubmeshesOctree(maxCapacity?: number, maxDepth?: number): Octree<SubMesh>;
        }
    /**
     * Defines the octree scene component responsible to manage any octrees
     * in a given scene.
     */
    export class OctreeSceneComponent {
        /**
         * The component name help to identify the component in the list of scene components.
         */
        readonly name = "Octree";
        /**
         * The scene the component belongs to.
         */
        scene: Scene;
        /**
         * Indicates if the meshes have been checked to make sure they are isEnabled()
         */
        readonly checksIsEnabled = true;
        /**
         * Creates a new instance of the component for the given scene
         * @param scene Defines the scene to register the component in
         */
        constructor(scene?: Scene);
        /**
         * Registers the component in a given scene
         */
        register(): void;
        /**
         * Return the list of active meshes
         * @returns the list of active meshes
         */
        getActiveMeshCandidates(): ISmartArrayLike<AbstractMesh>;
        /**
         * Return the list of active sub meshes
         * @param mesh The mesh to get the candidates sub meshes from
         * @returns the list of active sub meshes
         */
        getActiveSubMeshCandidates(mesh: AbstractMesh): ISmartArrayLike<SubMesh>;
        private _tempRay;
        /**
         * Return the list of sub meshes intersecting with a given local ray
         * @param mesh defines the mesh to find the submesh for
         * @param localRay defines the ray in local space
         * @returns the list of intersecting sub meshes
         */
        getIntersectingSubMeshCandidates(mesh: AbstractMesh, localRay: Ray): ISmartArrayLike<SubMesh>;
        /**
         * Return the list of sub meshes colliding with a collider
         * @param mesh defines the mesh to find the submesh for
         * @param collider defines the collider to evaluate the collision against
         * @returns the list of colliding sub meshes
         */
        getCollidingSubMeshCandidates(mesh: AbstractMesh, collider: Collider): ISmartArrayLike<SubMesh>;
        /**
         * Rebuilds the elements related to this component in case of
         * context lost for instance.
         */
        rebuild(): void;
        /**
         * Disposes the component and the associated resources.
         */
        dispose(): void;
    }


    /**
     * Class representing a ray with position and direction
     */
    export class Ray {
        /** origin point */
        origin: Vector3;
        /** direction */
        direction: Vector3;
        /** length of the ray */
        length: number;
        private static readonly _TmpVector3;
        private static _RayDistant;
        private _tmpRay;
        /**
         * Creates a new ray
         * @param origin origin point
         * @param direction direction
         * @param length length of the ray
         */
        constructor(
        /** origin point */
        origin: Vector3, 
        /** direction */
        direction: Vector3, 
        /** length of the ray */
        length?: number);
        /**
         * Clone the current ray
         * @returns a new ray
         */
        clone(): Ray;
        /**
         * Checks if the ray intersects a box
         * This does not account for the ray length by design to improve perfs.
         * @param minimum bound of the box
         * @param maximum bound of the box
         * @param intersectionTreshold extra extend to be added to the box in all direction
         * @returns if the box was hit
         */
        intersectsBoxMinMax(minimum: DeepImmutable<Vector3>, maximum: DeepImmutable<Vector3>, intersectionTreshold?: number): boolean;
        /**
         * Checks if the ray intersects a box
         * This does not account for the ray lenght by design to improve perfs.
         * @param box the bounding box to check
         * @param intersectionTreshold extra extend to be added to the BoundingBox in all direction
         * @returns if the box was hit
         */
        intersectsBox(box: DeepImmutable<BoundingBox>, intersectionTreshold?: number): boolean;
        /**
         * If the ray hits a sphere
         * @param sphere the bounding sphere to check
         * @param intersectionTreshold extra extend to be added to the BoundingSphere in all direction
         * @returns true if it hits the sphere
         */
        intersectsSphere(sphere: DeepImmutable<BoundingSphere>, intersectionTreshold?: number): boolean;
        /**
         * If the ray hits a triange
         * @param vertex0 triangle vertex
         * @param vertex1 triangle vertex
         * @param vertex2 triangle vertex
         * @returns intersection information if hit
         */
        intersectsTriangle(vertex0: DeepImmutable<Vector3>, vertex1: DeepImmutable<Vector3>, vertex2: DeepImmutable<Vector3>): Nullable<IntersectionInfo>;
        /**
         * Checks if ray intersects a plane
         * @param plane the plane to check
         * @returns the distance away it was hit
         */
        intersectsPlane(plane: DeepImmutable<Plane>): Nullable<number>;
        /**
         * Calculate the intercept of a ray on a given axis
         * @param axis to check 'x' | 'y' | 'z'
         * @param offset from axis interception (i.e. an offset of 1y is intercepted above ground)
         * @returns a vector containing the coordinates where 'axis' is equal to zero (else offset), or null if there is no intercept.
         */
        intersectsAxis(axis: string, offset?: number): Nullable<Vector3>;
        /**
         * Checks if ray intersects a mesh
         * @param mesh the mesh to check
         * @param fastCheck defines if the first intersection will be used (and not the closest)
         * @returns picking info of the intersection
         */
        intersectsMesh(mesh: DeepImmutable<AbstractMesh>, fastCheck?: boolean): PickingInfo;
        /**
         * Checks if ray intersects a mesh
         * @param meshes the meshes to check
         * @param fastCheck defines if the first intersection will be used (and not the closest)
         * @param results array to store result in
         * @returns Array of picking infos
         */
        intersectsMeshes(meshes: Array<DeepImmutable<AbstractMesh>>, fastCheck?: boolean, results?: Array<PickingInfo>): Array<PickingInfo>;
        private _comparePickingInfo;
        private static _Smallnum;
        private static _Rayl;
        /**
         * Intersection test between the ray and a given segment within a given tolerance (threshold)
         * @param sega the first point of the segment to test the intersection against
         * @param segb the second point of the segment to test the intersection against
         * @param threshold the tolerance margin, if the ray doesn't intersect the segment but is close to the given threshold, the intersection is successful
         * @returns the distance from the ray origin to the intersection point if there's intersection, or -1 if there's no intersection
         */
        intersectionSegment(sega: DeepImmutable<Vector3>, segb: DeepImmutable<Vector3>, threshold: number): number;
        /**
         * Update the ray from viewport position
         * @param x position
         * @param y y position
         * @param viewportWidth viewport width
         * @param viewportHeight viewport height
         * @param world world matrix
         * @param view view matrix
         * @param projection projection matrix
         * @param enableDistantPicking defines if picking should handle large values for mesh position/scaling (false by default)
         * @returns this ray updated
         */
        update(x: number, y: number, viewportWidth: number, viewportHeight: number, world: DeepImmutable<Matrix>, view: DeepImmutable<Matrix>, projection: DeepImmutable<Matrix>, enableDistantPicking?: boolean): Ray;
        /**
         * Creates a ray with origin and direction of 0,0,0
         * @returns the new ray
         */
        static Zero(): Ray;
        /**
         * Creates a new ray from screen space and viewport
         * @param x position
         * @param y y position
         * @param viewportWidth viewport width
         * @param viewportHeight viewport height
         * @param world world matrix
         * @param view view matrix
         * @param projection projection matrix
         * @returns new ray
         */
        static CreateNew(x: number, y: number, viewportWidth: number, viewportHeight: number, world: DeepImmutable<Matrix>, view: DeepImmutable<Matrix>, projection: DeepImmutable<Matrix>): Ray;
        /**
         * Function will create a new transformed ray starting from origin and ending at the end point. Ray's length will be set, and ray will be
         * transformed to the given world matrix.
         * @param origin The origin point
         * @param end The end point
         * @param world a matrix to transform the ray to. Default is the identity matrix.
         * @returns the new ray
         */
        static CreateNewFromTo(origin: Vector3, end: Vector3, world?: DeepImmutable<Matrix>): Ray;
        /**
         * Transforms a ray by a matrix
         * @param ray ray to transform
         * @param matrix matrix to apply
         * @returns the resulting new ray
         */
        static Transform(ray: DeepImmutable<Ray>, matrix: DeepImmutable<Matrix>): Ray;
        /**
         * Transforms a ray by a matrix
         * @param ray ray to transform
         * @param matrix matrix to apply
         * @param result ray to store result in
         */
        static TransformToRef(ray: DeepImmutable<Ray>, matrix: DeepImmutable<Matrix>, result: Ray): void;
        /**
         * Unproject a ray from screen space to object space
         * @param sourceX defines the screen space x coordinate to use
         * @param sourceY defines the screen space y coordinate to use
         * @param viewportWidth defines the current width of the viewport
         * @param viewportHeight defines the current height of the viewport
         * @param world defines the world matrix to use (can be set to Identity to go to world space)
         * @param view defines the view matrix to use
         * @param projection defines the projection matrix to use
         */
        unprojectRayToRef(sourceX: float, sourceY: float, viewportWidth: number, viewportHeight: number, world: DeepImmutable<Matrix>, view: DeepImmutable<Matrix>, projection: DeepImmutable<Matrix>): void;
    }
    /**
     * Type used to define predicate used to select faces when a mesh intersection is detected
     */
    export type TrianglePickingPredicate = (p0: Vector3, p1: Vector3, p2: Vector3, ray: Ray, i0: number, i1: number, i2: number) => boolean;
        interface Scene {
            /** @internal */
            _tempPickingRay: Nullable<Ray>;
            /** @internal */
            _cachedRayForTransform: Ray;
            /** @internal */
            _pickWithRayInverseMatrix: Matrix;
            /** @internal */
            _internalPick(rayFunction: (world: Matrix, enableDistantPicking: boolean) => Ray, predicate?: (mesh: AbstractMesh) => boolean, fastCheck?: boolean, onlyBoundingInfo?: boolean, trianglePredicate?: TrianglePickingPredicate): PickingInfo;
            /** @internal */
            _internalMultiPick(rayFunction: (world: Matrix, enableDistantPicking: boolean) => Ray, predicate?: (mesh: AbstractMesh) => boolean, trianglePredicate?: TrianglePickingPredicate): Nullable<PickingInfo[]>;
            /** @internal */
            _internalPickForMesh(pickingInfo: Nullable<PickingInfo>, rayFunction: (world: Matrix, enableDistantPicking: boolean) => Ray, mesh: AbstractMesh, world: Matrix, fastCheck?: boolean, onlyBoundingInfo?: boolean, trianglePredicate?: TrianglePickingPredicate, skipBoundingInfo?: boolean): Nullable<PickingInfo>;
        }



}
declare module BABYLON.Debug {
        /**
     * The Axes viewer will show 3 axes in a specific point in space
     * @see https://doc.babylonjs.com/toolsAndResources/utilities/World_Axes
     */
    export class AxesViewer {
        private _xAxis;
        private _yAxis;
        private _zAxis;
        private _scaleLinesFactor;
        private _instanced;
        /**
         * Gets the hosting scene
         */
        scene: Nullable<Scene>;
        /**
         * Gets or sets a number used to scale line length
         */
        scaleLines: number;
        /** Gets the node hierarchy used to render x-axis */
        get xAxis(): TransformNode;
        /** Gets the node hierarchy used to render y-axis */
        get yAxis(): TransformNode;
        /** Gets the node hierarchy used to render z-axis */
        get zAxis(): TransformNode;
        /**
         * Creates a new AxesViewer
         * @param scene defines the hosting scene
         * @param scaleLines defines a number used to scale line length (1 by default)
         * @param renderingGroupId defines a number used to set the renderingGroupId of the meshes (2 by default)
         * @param xAxis defines the node hierarchy used to render the x-axis
         * @param yAxis defines the node hierarchy used to render the y-axis
         * @param zAxis defines the node hierarchy used to render the z-axis
         * @param lineThickness The line thickness to use when creating the arrow. defaults to 1.
         */
        constructor(scene?: Scene, scaleLines?: number, renderingGroupId?: Nullable<number>, xAxis?: TransformNode, yAxis?: TransformNode, zAxis?: TransformNode, lineThickness?: number);
        /**
         * Force the viewer to update
         * @param position defines the position of the viewer
         * @param xaxis defines the x axis of the viewer
         * @param yaxis defines the y axis of the viewer
         * @param zaxis defines the z axis of the viewer
         */
        update(position: Vector3, xaxis: Vector3, yaxis: Vector3, zaxis: Vector3): void;
        /**
         * Creates an instance of this axes viewer.
         * @returns a new axes viewer with instanced meshes
         */
        createInstance(): AxesViewer;
        /** Releases resources */
        dispose(): void;
        private static _SetRenderingGroupId;
    }



}
declare module BABYLON {

}
declare module BABYLON.Debug {
        /**
     * The BoneAxesViewer will attach 3 axes to a specific bone of a specific mesh
     * @see demo here: https://www.babylonjs-playground.com/#0DE8F4#8
     */
    export class BoneAxesViewer extends BABYLON.Debug.AxesViewer {
        /**
         * Gets or sets the target mesh where to display the axes viewer
         */
        mesh: Nullable<Mesh>;
        /**
         * Gets or sets the target bone where to display the axes viewer
         */
        bone: Nullable<Bone>;
        /** Gets current position */
        pos: Vector3;
        /** Gets direction of X axis */
        xaxis: Vector3;
        /** Gets direction of Y axis */
        yaxis: Vector3;
        /** Gets direction of Z axis */
        zaxis: Vector3;
        /**
         * Creates a new BoneAxesViewer
         * @param scene defines the hosting scene
         * @param bone defines the target bone
         * @param mesh defines the target mesh
         * @param scaleLines defines a scaling factor for line length (1 by default)
         */
        constructor(scene: Scene, bone: Bone, mesh: Mesh, scaleLines?: number);
        /**
         * Force the viewer to update
         */
        update(): void;
        /** Releases resources */
        dispose(): void;
    }



}
declare module BABYLON {
    /**
     * Interface used to define scene explorer extensibility option
     */
    export interface IExplorerExtensibilityOption {
        /**
         * Define the option label
         */
        label: string;
        /**
         * Defines the action to execute on click
         */
        action: (entity: any) => void;
    }
    /**
     * Defines a group of actions associated with a predicate to use when extending the Inspector scene explorer
     */
    export interface IExplorerExtensibilityGroup {
        /**
         * Defines a predicate to test if a given type mut be extended
         */
        predicate: (entity: any) => boolean;
        /**
         * Gets the list of options added to a type
         */
        entries: IExplorerExtensibilityOption[];
    }
    /**
     * Defines a new node that will be displayed as top level node in the explorer
     */
    export interface IExplorerAdditionalChild {
        /**
         * Gets the name of the additional node
         */
        name: string;
        /**
         * Function used to return the class name of the child node
         */
        getClassName(): string;
        /**
         * List of inspectable custom properties (used by the Inspector)
         * @see https://doc.babylonjs.com/toolsAndResources/inspector#extensibility
         */
        inspectableCustomProperties: IInspectable[];
    }
    /**
     * Defines a new node that will be displayed as top level node in the explorer
     */
    export interface IExplorerAdditionalNode {
        /**
         * Gets the name of the additional node
         */
        name: string;
        /**
         * Function used to return the list of child entries
         */
        getContent(): IExplorerAdditionalChild[];
    }
    /**
     * Interface used to define the options to use to create the Inspector
     */
    export interface IInspectorOptions {
        /**
         * Display in overlay mode (default: false)
         */
        overlay?: boolean;
        /**
         * HTML element to use as root (the parent of the rendering canvas will be used as default value)
         */
        globalRoot?: HTMLElement;
        /**
         * Display the Scene explorer
         */
        showExplorer?: boolean;
        /**
         * Display the property inspector
         */
        showInspector?: boolean;
        /**
         * Display in embed mode (both panes on the right)
         */
        embedMode?: boolean;
        /**
         * let the Inspector handles resize of the canvas when panes are resized (default to true)
         */
        handleResize?: boolean;
        /**
         * Allow the panes to popup (default: true)
         */
        enablePopup?: boolean;
        /**
         * Allow the panes to be closed by users (default: true)
         */
        enableClose?: boolean;
        /**
         * Optional list of extensibility entries
         */
        explorerExtensibility?: IExplorerExtensibilityGroup[];
        /**
         * Optional list of additional top level nodes
         */
        additionalNodes?: IExplorerAdditionalNode[];
        /**
         * Optional URL to get the inspector script from (by default it uses the babylonjs CDN).
         */
        inspectorURL?: string;
        /**
         * Optional initial tab (default to DebugLayerTab.Properties)
         */
        initialTab?: DebugLayerTab;
        /**
         * Optional camera to use to render the gizmos from the inspector (default to the scene.activeCamera or the latest from scene.activeCameras)
         */
        gizmoCamera?: Camera;
    }
        interface Scene {
            /**
             * @internal
             * Backing field
             */
            _debugLayer: DebugLayer;
            /**
             * Gets the debug layer (aka Inspector) associated with the scene
             * @see https://doc.babylonjs.com/toolsAndResources/inspector
             */
            debugLayer: DebugLayer;
        }
    /**
     * Enum of inspector action tab
     */
    export enum DebugLayerTab {
        /**
         * Properties tag (default)
         */
        Properties = 0,
        /**
         * Debug tab
         */
        Debug = 1,
        /**
         * Statistics tab
         */
        Statistics = 2,
        /**
         * Tools tab
         */
        Tools = 3,
        /**
         * Settings tab
         */
        Settings = 4
    }
    /**
     * The debug layer (aka Inspector) is the go to tool in order to better understand
     * what is happening in your scene
     * @see https://doc.babylonjs.com/toolsAndResources/inspector
     */
    export class DebugLayer {
        /**
         * Define the url to get the inspector script from.
         * By default it uses the babylonjs CDN.
         * @ignoreNaming
         */
        static InspectorURL: string;
        private _scene;
        private BJSINSPECTOR;
        private _onPropertyChangedObservable?;
        /**
         * Observable triggered when a property is changed through the inspector.
         */
        get onPropertyChangedObservable(): any;
        private _onSelectionChangedObservable?;
        /**
         * Observable triggered when the selection is changed through the inspector.
         */
        get onSelectionChangedObservable(): any;
        /**
         * Instantiates a new debug layer.
         * The debug layer (aka Inspector) is the go to tool in order to better understand
         * what is happening in your scene
         * @see https://doc.babylonjs.com/toolsAndResources/inspector
         * @param scene Defines the scene to inspect
         */
        constructor(scene?: Scene);
        /**
         * Creates the inspector window.
         * @param config
         */
        private _createInspector;
        /**
         * Select a specific entity in the scene explorer and highlight a specific block in that entity property grid
         * @param entity defines the entity to select
         * @param lineContainerTitles defines the specific blocks to highlight (could be a string or an array of strings)
         */
        select(entity: any, lineContainerTitles?: string | string[]): void;
        /** Get the inspector from bundle or global */
        private _getGlobalInspector;
        /**
         * Get if the inspector is visible or not.
         * @returns true if visible otherwise, false
         */
        isVisible(): boolean;
        /**
         * Hide the inspector and close its window.
         */
        hide(): void;
        /**
         * Update the scene in the inspector
         */
        setAsActiveScene(): void;
        /**
         * Launch the debugLayer.
         * @param config Define the configuration of the inspector
         * @returns a promise fulfilled when the debug layer is visible
         */
        show(config?: IInspectorOptions): Promise<DebugLayer>;
    }


    /**
     * Class used to render a debug view of the frustum for a directional light
     * @see https://playground.babylonjs.com/#7EFGSG#4
     * @since 5.0.0
     */
    export class DirectionalLightFrustumViewer {
        private _scene;
        private _light;
        private _camera;
        private _inverseViewMatrix;
        private _visible;
        private _rootNode;
        private _lightHelperFrustumMeshes;
        private _nearLinesPoints;
        private _farLinesPoints;
        private _trLinesPoints;
        private _brLinesPoints;
        private _tlLinesPoints;
        private _blLinesPoints;
        private _nearPlaneVertices;
        private _farPlaneVertices;
        private _rightPlaneVertices;
        private _leftPlaneVertices;
        private _topPlaneVertices;
        private _bottomPlaneVertices;
        private _oldPosition;
        private _oldDirection;
        private _oldAutoCalc;
        private _oldMinZ;
        private _oldMaxZ;
        private _transparency;
        /**
         * Gets or sets the transparency of the frustum planes
         */
        get transparency(): number;
        set transparency(alpha: number);
        private _showLines;
        /**
         * true to display the edges of the frustum
         */
        get showLines(): boolean;
        set showLines(show: boolean);
        private _showPlanes;
        /**
         * true to display the planes of the frustum
         */
        get showPlanes(): boolean;
        set showPlanes(show: boolean);
        /**
         * Creates a new frustum viewer
         * @param light directional light to display the frustum for
         * @param camera camera used to retrieve the minZ / maxZ values if the shadowMinZ/shadowMaxZ values of the light are not setup
         */
        constructor(light: DirectionalLight, camera: Camera);
        /**
         * Shows the frustum
         */
        show(): void;
        /**
         * Hides the frustum
         */
        hide(): void;
        /**
         * Updates the frustum.
         * Call this method to update the frustum view if the light has changed position/direction
         */
        update(): void;
        /**
         * Dispose of the class / remove the frustum view
         */
        dispose(): void;
        protected _createGeometry(): void;
        protected _getInvertViewMatrix(): Matrix;
    }




    /**
     * Defines the options associated with the creation of a SkeletonViewer.
     */
    export interface ISkeletonViewerOptions {
        /** Should the system pause animations before building the Viewer? */
        pauseAnimations: boolean;
        /** Should the system return the skeleton to rest before building? */
        returnToRest: boolean;
        /** public Display Mode of the Viewer */
        displayMode: number;
        /** Flag to toggle if the Viewer should use the CPU for animations or not? */
        displayOptions: ISkeletonViewerDisplayOptions;
        /** Flag to toggle if the Viewer should use the CPU for animations or not? */
        computeBonesUsingShaders: boolean;
        /** Flag ignore non weighted bones */
        useAllBones: boolean;
    }
    /**
     * Defines how to display the various bone meshes for the viewer.
     */
    export interface ISkeletonViewerDisplayOptions {
        /** How far down to start tapering the bone spurs */
        midStep?: number;
        /** How big is the midStep? */
        midStepFactor?: number;
        /** Base for the Sphere Size */
        sphereBaseSize?: number;
        /** The ratio of the sphere to the longest bone in units */
        sphereScaleUnit?: number;
        /** Ratio for the Sphere Size */
        sphereFactor?: number;
        /** Whether a spur should attach its far end to the child bone position */
        spurFollowsChild?: boolean;
        /** Whether to show local axes or not  */
        showLocalAxes?: boolean;
        /** Length of each local axis */
        localAxesSize?: number;
    }
    /**
     * Defines the constructor options for the BoneWeight Shader.
     */
    export interface IBoneWeightShaderOptions {
        /** Skeleton to Map */
        skeleton: Skeleton;
        /** Colors for Uninfluenced bones */
        colorBase?: Color3;
        /** Colors for 0.0-0.25 Weight bones */
        colorZero?: Color3;
        /** Color for 0.25-0.5 Weight Influence */
        colorQuarter?: Color3;
        /** Color for 0.5-0.75 Weight Influence */
        colorHalf?: Color3;
        /** Color for 0.75-1 Weight Influence */
        colorFull?: Color3;
        /** Color for Zero Weight Influence */
        targetBoneIndex?: number;
    }
    /**
     * Simple structure of the gradient steps for the Color Map.
     */
    export interface ISkeletonMapShaderColorMapKnot {
        /** Color of the Knot */
        color: Color3;
        /** Location of the Knot */
        location: number;
    }
    /**
     * Defines the constructor options for the SkeletonMap Shader.
     */
    export interface ISkeletonMapShaderOptions {
        /** Skeleton to Map */
        skeleton: Skeleton;
        /** Array of ColorMapKnots that make the gradient must be ordered with knot[i].location < knot[i+1].location*/
        colorMap?: ISkeletonMapShaderColorMapKnot[];
    }



}
declare module BABYLON.Debug {
        /**
     * Used to show the physics impostor around the specific mesh
     */
    export class PhysicsViewer {
        /** @internal */
        protected _impostors: Array<Nullable<PhysicsImpostor>>;
        /** @internal */
        protected _meshes: Array<Nullable<AbstractMesh>>;
        /** @internal */
        protected _bodies: Array<Nullable<PhysicsBody>>;
        /** @internal */
        protected _bodyMeshes: Array<Nullable<AbstractMesh>>;
        /** @internal */
        protected _scene: Nullable<Scene>;
        /** @internal */
        protected _numMeshes: number;
        /** @internal */
        protected _numBodies: number;
        /** @internal */
        protected _physicsEnginePlugin: BABYLON.IPhysicsEnginePlugin | IPhysicsEnginePluginV2 | null;
        private _renderFunction;
        private _utilityLayer;
        private _debugBoxMesh;
        private _debugSphereMesh;
        private _debugCapsuleMesh;
        private _debugCylinderMesh;
        private _debugMaterial;
        private _debugMeshMeshes;
        /**
         * Creates a new PhysicsViewer
         * @param scene defines the hosting scene
         */
        constructor(scene?: Scene);
        /**
         * Updates the debug meshes of the physics engine.
         *
         * This code is useful for synchronizing the debug meshes of the physics engine with the physics impostor and mesh.
         * It checks if the impostor is disposed and if the plugin version is 1, then it syncs the mesh with the impostor.
         * This ensures that the debug meshes are up to date with the physics engine.
         */
        protected _updateDebugMeshes(): void;
        /**
         * Updates the debug meshes of the physics engine.
         *
         * This method is useful for synchronizing the debug meshes with the physics impostors.
         * It iterates through the impostors and meshes, and if the plugin version is 1, it syncs the mesh with the impostor.
         * This ensures that the debug meshes accurately reflect the physics impostors, which is important for debugging the physics engine.
         */
        protected _updateDebugMeshesV1(): void;
        /**
         * Updates the debug meshes of the physics engine for V2 plugin.
         *
         * This method is useful for synchronizing the debug meshes of the physics engine with the current state of the bodies.
         * It iterates through the bodies array and updates the debug meshes with the current transform of each body.
         * This ensures that the debug meshes accurately reflect the current state of the physics engine.
         */
        protected _updateDebugMeshesV2(): void;
        /**
         * Renders a specified physic impostor
         * @param impostor defines the impostor to render
         * @param targetMesh defines the mesh represented by the impostor
         * @returns the new debug mesh used to render the impostor
         */
        showImpostor(impostor: PhysicsImpostor, targetMesh?: Mesh): Nullable<AbstractMesh>;
        /**
         * Shows a debug mesh for a given physics body.
         * @param body The physics body to show.
         * @returns The debug mesh, or null if the body is already shown.
         *
         * This function is useful for visualizing the physics body in the scene.
         * It creates a debug mesh for the given body and adds it to the scene.
         * It also registers a before render function to update the debug mesh position and rotation.
         */
        showBody(body: PhysicsBody): Nullable<AbstractMesh>;
        /**
         * Hides an impostor from the scene.
         * @param impostor - The impostor to hide.
         *
         * This method is useful for hiding an impostor from the scene. It removes the
         * impostor from the utility layer scene, disposes the mesh, and removes the
         * impostor from the list of impostors. If the impostor is the last one in the
         * list, it also unregisters the render function.
         */
        hideImpostor(impostor: Nullable<PhysicsImpostor>): void;
        /**
         * Hides a body from the physics engine.
         * @param body - The body to hide.
         *
         * This function is useful for hiding a body from the physics engine.
         * It removes the body from the utility layer scene and disposes the mesh associated with it.
         * It also unregisters the render function if the number of bodies is 0.
         * This is useful for hiding a body from the physics engine without deleting it.
         */
        hideBody(body: Nullable<PhysicsBody>): void;
        private _getDebugMaterial;
        private _getDebugBoxMesh;
        private _getDebugSphereMesh;
        private _getDebugCapsuleMesh;
        private _getDebugCylinderMesh;
        private _getDebugMeshMesh;
        private _getDebugMesh;
        /**
         * Creates a debug mesh for a given physics body
         * @param body The physics body to create the debug mesh for
         * @returns The created debug mesh or null if the utility layer is not available
         *
         * This code is useful for creating a debug mesh for a given physics body.
         * It creates a Mesh object with a VertexData object containing the positions and indices
         * of the geometry of the body. The mesh is then assigned a debug material from the utility layer scene.
         * This allows for visualizing the physics body in the scene.
         */
        private _getDebugBodyMesh;
        /**
         * Clean up physics debug display
         */
        dispose(): void;
    }



}
declare module BABYLON {
    /**
     * As raycast might be hard to debug, the RayHelper can help rendering the different rays
     * in order to better appreciate the issue one might have.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/interactions/picking_collisions#debugging
     */
    export class RayHelper {
        /**
         * Defines the ray we are currently tryin to visualize.
         */
        ray: Nullable<Ray>;
        private _renderPoints;
        private _renderLine;
        private _renderFunction;
        private _scene;
        private _onAfterRenderObserver;
        private _onAfterStepObserver;
        private _attachedToMesh;
        private _meshSpaceDirection;
        private _meshSpaceOrigin;
        /**
         * Helper function to create a colored helper in a scene in one line.
         * @param ray Defines the ray we are currently tryin to visualize
         * @param scene Defines the scene the ray is used in
         * @param color Defines the color we want to see the ray in
         * @returns The newly created ray helper.
         */
        static CreateAndShow(ray: Ray, scene: Scene, color: Color3): RayHelper;
        /**
         * Instantiate a new ray helper.
         * As raycast might be hard to debug, the RayHelper can help rendering the different rays
         * in order to better appreciate the issue one might have.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/interactions/picking_collisions#debugging
         * @param ray Defines the ray we are currently tryin to visualize
         */
        constructor(ray: Ray);
        /**
         * Shows the ray we are willing to debug.
         * @param scene Defines the scene the ray needs to be rendered in
         * @param color Defines the color the ray needs to be rendered in
         */
        show(scene: Scene, color?: Color3): void;
        /**
         * Hides the ray we are debugging.
         */
        hide(): void;
        private _render;
        /**
         * Attach a ray helper to a mesh so that we can easily see its orientation for instance or information like its normals.
         * @param mesh Defines the mesh we want the helper attached to
         * @param meshSpaceDirection Defines the direction of the Ray in mesh space (local space of the mesh node)
         * @param meshSpaceOrigin Defines the origin of the Ray in mesh space (local space of the mesh node)
         * @param length Defines the length of the ray
         */
        attachToMesh(mesh: AbstractMesh, meshSpaceDirection?: Vector3, meshSpaceOrigin?: Vector3, length?: number): void;
        /**
         * Detach the ray helper from the mesh it has previously been attached to.
         */
        detachFromMesh(): void;
        private _updateToMesh;
        /**
         * Dispose the helper and release its associated resources.
         */
        dispose(): void;
    }



}
declare module BABYLON.Debug {
        /**
     * Class used to render a debug view of a given skeleton
     * @see http://www.babylonjs-playground.com/#1BZJVJ#8
     */
    export class SkeletonViewer {
        /** defines the skeleton to render */
        skeleton: Skeleton;
        /** defines the mesh attached to the skeleton */
        mesh: AbstractMesh;
        /** defines a boolean indicating if bones matrices must be forced to update before rendering (true by default)  */
        autoUpdateBonesMatrices: boolean;
        /** defines the rendering group id to use with the viewer */
        renderingGroupId: number;
        /** is the options for the viewer */
        options: Partial<ISkeletonViewerOptions>;
        /** public Display constants BABYLON.SkeletonViewer.DISPLAY_LINES */
        static readonly DISPLAY_LINES = 0;
        /** public Display constants BABYLON.SkeletonViewer.DISPLAY_SPHERES */
        static readonly DISPLAY_SPHERES = 1;
        /** public Display constants BABYLON.SkeletonViewer.DISPLAY_SPHERE_AND_SPURS */
        static readonly DISPLAY_SPHERE_AND_SPURS = 2;
        /** public static method to create a BoneWeight Shader
         * @param options The constructor options
         * @param scene The scene that the shader is scoped to
         * @returns The created ShaderMaterial
         * @see http://www.babylonjs-playground.com/#1BZJVJ#395
         */
        static CreateBoneWeightShader(options: IBoneWeightShaderOptions, scene: Scene): ShaderMaterial;
        /** public static method to create a BoneWeight Shader
         * @param options The constructor options
         * @param scene The scene that the shader is scoped to
         * @returns The created ShaderMaterial
         */
        static CreateSkeletonMapShader(options: ISkeletonMapShaderOptions, scene: Scene): ShaderMaterial;
        /** private static method to create a BoneWeight Shader
         * @param size The size of the buffer to create (usually the bone count)
         * @param colorMap The gradient data to generate
         * @param scene The scene that the shader is scoped to
         * @returns an Array of floats from the color gradient values
         */
        private static _CreateBoneMapColorBuffer;
        /** If SkeletonViewer scene scope. */
        private _scene;
        /** Gets or sets the color used to render the skeleton */
        color: Color3;
        /** Array of the points of the skeleton fo the line view. */
        private _debugLines;
        /** The SkeletonViewers Mesh. */
        private _debugMesh;
        /** The local axes Meshes. */
        private _localAxes;
        /** If SkeletonViewer is enabled. */
        private _isEnabled;
        /** If SkeletonViewer is ready. */
        private _ready;
        /** SkeletonViewer render observable. */
        private _obs;
        /** The Utility Layer to render the gizmos in. */
        private _utilityLayer;
        private _boneIndices;
        /** Gets the Scene. */
        get scene(): Scene;
        /** Gets the utilityLayer. */
        get utilityLayer(): Nullable<UtilityLayerRenderer>;
        /** Checks Ready Status. */
        get isReady(): Boolean;
        /** Sets Ready Status. */
        set ready(value: boolean);
        /** Gets the debugMesh */
        get debugMesh(): Nullable<AbstractMesh> | Nullable<LinesMesh>;
        /** Sets the debugMesh */
        set debugMesh(value: Nullable<AbstractMesh> | Nullable<LinesMesh>);
        /** Gets the displayMode */
        get displayMode(): number;
        /** Sets the displayMode */
        set displayMode(value: number);
        /**
         * Creates a new SkeletonViewer
         * @param skeleton defines the skeleton to render
         * @param mesh defines the mesh attached to the skeleton
         * @param scene defines the hosting scene
         * @param autoUpdateBonesMatrices defines a boolean indicating if bones matrices must be forced to update before rendering (true by default)
         * @param renderingGroupId defines the rendering group id to use with the viewer
         * @param options All of the extra constructor options for the SkeletonViewer
         */
        constructor(
        /** defines the skeleton to render */
        skeleton: Skeleton, 
        /** defines the mesh attached to the skeleton */
        mesh: AbstractMesh, 
        /** The Scene scope*/
        scene: Scene, 
        /** defines a boolean indicating if bones matrices must be forced to update before rendering (true by default)  */
        autoUpdateBonesMatrices?: boolean, 
        /** defines the rendering group id to use with the viewer */
        renderingGroupId?: number, 
        /** is the options for the viewer */
        options?: Partial<ISkeletonViewerOptions>);
        /** The Dynamic bindings for the update functions */
        private _bindObs;
        /** Update the viewer to sync with current skeleton state, only used to manually update. */
        update(): void;
        /** Gets or sets a boolean indicating if the viewer is enabled */
        set isEnabled(value: boolean);
        get isEnabled(): boolean;
        private _getBonePosition;
        private _getLinesForBonesWithLength;
        private _getLinesForBonesNoLength;
        /**
         * function to revert the mesh and scene back to the initial state.
         * @param animationState
         */
        private _revert;
        /**
         * function to get the absolute bind pose of a bone by accumulating transformations up the bone hierarchy.
         * @param bone
         * @param matrix
         */
        private _getAbsoluteBindPoseToRef;
        /**
         * function to build and bind sphere joint points and spur bone representations.
         * @param spheresOnly
         */
        private _buildSpheresAndSpurs;
        private _buildLocalAxes;
        /** Update the viewer to sync with current skeleton state, only used for the line display. */
        private _displayLinesUpdate;
        /** Changes the displayMode of the skeleton viewer
         * @param mode The displayMode numerical value
         */
        changeDisplayMode(mode: number): void;
        /** Sets a display option of the skeleton viewer
         *
         * | Option           | Type    | Default | Description |
         * | ---------------- | ------- | ------- | ----------- |
         * | midStep          | float   | 0.235   | A percentage between a bone and its child that determines the widest part of a spur. Only used when `displayMode` is set to `DISPLAY_SPHERE_AND_SPURS`. |
         * | midStepFactor    | float   | 0.15    | Mid step width expressed as a factor of the length. A value of 0.5 makes the spur width half of the spur length. Only used when `displayMode` is set to `DISPLAY_SPHERE_AND_SPURS`. |
         * | sphereBaseSize   | float   | 2       | Sphere base size. Only used when `displayMode` is set to `DISPLAY_SPHERE_AND_SPURS`. |
         * | sphereScaleUnit  | float   | 0.865   | Sphere scale factor used to scale spheres in relation to the longest bone. Only used when `displayMode` is set to `DISPLAY_SPHERE_AND_SPURS`. |
         * | spurFollowsChild | boolean | false   | Whether a spur should attach its far end to the child bone. |
         * | showLocalAxes    | boolean | false   | Displays local axes on all bones. |
         * | localAxesSize    | float   | 0.075   | Determines the length of each local axis. |
         *
         * @param option String of the option name
         * @param value The numerical option value
         */
        changeDisplayOptions(option: string, value: number): void;
        /** Release associated resources */
        dispose(): void;
    }



}
declare module BABYLON {
    /**
     * Class to wrap DeviceInputSystem data into an event object
     */
    export class DeviceEventFactory {
        /**
         * Create device input events based on provided type and slot
         *
         * @param deviceType Type of device
         * @param deviceSlot "Slot" or index that device is referenced in
         * @param inputIndex Id of input to be checked
         * @param currentState Current value for given input
         * @param deviceInputSystem Reference to DeviceInputSystem
         * @param elementToAttachTo HTMLElement to reference as target for inputs
         * @returns IUIEvent object
         */
        static CreateDeviceEvent(deviceType: DeviceType, deviceSlot: number, inputIndex: number, currentState: Nullable<number>, deviceInputSystem: IDeviceInputSystem, elementToAttachTo?: any): IUIEvent;
        /**
         * Creates pointer event
         *
         * @param deviceType Type of device
         * @param deviceSlot "Slot" or index that device is referenced in
         * @param inputIndex Id of input to be checked
         * @param currentState Current value for given input
         * @param deviceInputSystem Reference to DeviceInputSystem
         * @param elementToAttachTo HTMLElement to reference as target for inputs
         * @returns IUIEvent object (Pointer)
         */
        private static _CreatePointerEvent;
        /**
         * Create Mouse Wheel Event
         * @param deviceType Type of device
         * @param deviceSlot "Slot" or index that device is referenced in
         * @param inputIndex Id of input to be checked
         * @param currentState Current value for given input
         * @param deviceInputSystem Reference to DeviceInputSystem
         * @param elementToAttachTo HTMLElement to reference as target for inputs
         * @returns IUIEvent object (Wheel)
         */
        private static _CreateWheelEvent;
        /**
         * Create Mouse Event
         * @param deviceType Type of device
         * @param deviceSlot "Slot" or index that device is referenced in
         * @param inputIndex Id of input to be checked
         * @param currentState Current value for given input
         * @param deviceInputSystem Reference to DeviceInputSystem
         * @param elementToAttachTo HTMLElement to reference as target for inputs
         * @returns IUIEvent object (Mouse)
         */
        private static _CreateMouseEvent;
        /**
         * Create Keyboard Event
         * @param inputIndex Id of input to be checked
         * @param currentState Current value for given input
         * @param deviceInputSystem Reference to DeviceInputSystem
         * @param elementToAttachTo HTMLElement to reference as target for inputs
         * @returns IEvent object (Keyboard)
         */
        private static _CreateKeyboardEvent;
        /**
         * Add parameters for non-character keys (Ctrl, Alt, Meta, Shift)
         * @param evt Event object to add parameters to
         * @param deviceInputSystem DeviceInputSystem to pull values from
         */
        private static _CheckNonCharacterKeys;
        /**
         * Create base event object
         * @param elementToAttachTo Value to use as event target
         * @returns
         */
        private static _CreateEvent;
    }




    /**
     * Enum for Device Types
     */
    export enum DeviceType {
        /** Generic */
        Generic = 0,
        /** Keyboard */
        Keyboard = 1,
        /** Mouse */
        Mouse = 2,
        /** Touch Pointers */
        Touch = 3,
        /** PS4 Dual Shock */
        DualShock = 4,
        /** Xbox */
        Xbox = 5,
        /** Switch Controller */
        Switch = 6,
        /** PS5 DualSense */
        DualSense = 7
    }
    /**
     * Enum for All Pointers (Touch/Mouse)
     */
    export enum PointerInput {
        /** Horizontal Axis (Not used in events/observables; only in polling) */
        Horizontal = 0,
        /** Vertical Axis (Not used in events/observables; only in polling) */
        Vertical = 1,
        /** Left Click or Touch */
        LeftClick = 2,
        /** Middle Click */
        MiddleClick = 3,
        /** Right Click */
        RightClick = 4,
        /** Browser Back */
        BrowserBack = 5,
        /** Browser Forward */
        BrowserForward = 6,
        /** Mouse Wheel X */
        MouseWheelX = 7,
        /** Mouse Wheel Y */
        MouseWheelY = 8,
        /** Mouse Wheel Z */
        MouseWheelZ = 9,
        /** Used in events/observables to identify if x/y changes occurred */
        Move = 12
    }
    /** @internal */
    export enum NativePointerInput {
        /** Horizontal Axis */
        Horizontal = 0,
        /** Vertical Axis */
        Vertical = 1,
        /** Left Click or Touch */
        LeftClick = 2,
        /** Middle Click */
        MiddleClick = 3,
        /** Right Click */
        RightClick = 4,
        /** Browser Back */
        BrowserBack = 5,
        /** Browser Forward */
        BrowserForward = 6,
        /** Mouse Wheel X */
        MouseWheelX = 7,
        /** Mouse Wheel Y */
        MouseWheelY = 8,
        /** Mouse Wheel Z */
        MouseWheelZ = 9,
        /** Delta X */
        DeltaHorizontal = 10,
        /** Delta Y */
        DeltaVertical = 11
    }
    /**
     * Enum for Dual Shock Gamepad
     */
    export enum DualShockInput {
        /** Cross */
        Cross = 0,
        /** Circle */
        Circle = 1,
        /** Square */
        Square = 2,
        /** Triangle */
        Triangle = 3,
        /** L1 */
        L1 = 4,
        /** R1 */
        R1 = 5,
        /** L2 */
        L2 = 6,
        /** R2 */
        R2 = 7,
        /** Share */
        Share = 8,
        /** Options */
        Options = 9,
        /** L3 */
        L3 = 10,
        /** R3 */
        R3 = 11,
        /** DPadUp */
        DPadUp = 12,
        /** DPadDown */
        DPadDown = 13,
        /** DPadLeft */
        DPadLeft = 14,
        /** DRight */
        DPadRight = 15,
        /** Home */
        Home = 16,
        /** TouchPad */
        TouchPad = 17,
        /** LStickXAxis */
        LStickXAxis = 18,
        /** LStickYAxis */
        LStickYAxis = 19,
        /** RStickXAxis */
        RStickXAxis = 20,
        /** RStickYAxis */
        RStickYAxis = 21
    }
    /**
     * Enum for Dual Sense Gamepad
     */
    export enum DualSenseInput {
        /** Cross */
        Cross = 0,
        /** Circle */
        Circle = 1,
        /** Square */
        Square = 2,
        /** Triangle */
        Triangle = 3,
        /** L1 */
        L1 = 4,
        /** R1 */
        R1 = 5,
        /** L2 */
        L2 = 6,
        /** R2 */
        R2 = 7,
        /** Create */
        Create = 8,
        /** Options */
        Options = 9,
        /** L3 */
        L3 = 10,
        /** R3 */
        R3 = 11,
        /** DPadUp */
        DPadUp = 12,
        /** DPadDown */
        DPadDown = 13,
        /** DPadLeft */
        DPadLeft = 14,
        /** DRight */
        DPadRight = 15,
        /** Home */
        Home = 16,
        /** TouchPad */
        TouchPad = 17,
        /** LStickXAxis */
        LStickXAxis = 18,
        /** LStickYAxis */
        LStickYAxis = 19,
        /** RStickXAxis */
        RStickXAxis = 20,
        /** RStickYAxis */
        RStickYAxis = 21
    }
    /**
     * Enum for Xbox Gamepad
     */
    export enum XboxInput {
        /** A */
        A = 0,
        /** B */
        B = 1,
        /** X */
        X = 2,
        /** Y */
        Y = 3,
        /** LB */
        LB = 4,
        /** RB */
        RB = 5,
        /** LT */
        LT = 6,
        /** RT */
        RT = 7,
        /** Back */
        Back = 8,
        /** Start */
        Start = 9,
        /** LS */
        LS = 10,
        /** RS */
        RS = 11,
        /** DPadUp */
        DPadUp = 12,
        /** DPadDown */
        DPadDown = 13,
        /** DPadLeft */
        DPadLeft = 14,
        /** DRight */
        DPadRight = 15,
        /** Home */
        Home = 16,
        /** LStickXAxis */
        LStickXAxis = 17,
        /** LStickYAxis */
        LStickYAxis = 18,
        /** RStickXAxis */
        RStickXAxis = 19,
        /** RStickYAxis */
        RStickYAxis = 20
    }
    /**
     * Enum for Switch (Pro/JoyCon L+R) Gamepad
     */
    export enum SwitchInput {
        /** B */
        B = 0,
        /** A */
        A = 1,
        /** Y */
        Y = 2,
        /** X */
        X = 3,
        /** L */
        L = 4,
        /** R */
        R = 5,
        /** ZL */
        ZL = 6,
        /** ZR */
        ZR = 7,
        /** Minus */
        Minus = 8,
        /** Plus */
        Plus = 9,
        /** LS */
        LS = 10,
        /** RS */
        RS = 11,
        /** DPadUp */
        DPadUp = 12,
        /** DPadDown */
        DPadDown = 13,
        /** DPadLeft */
        DPadLeft = 14,
        /** DRight */
        DPadRight = 15,
        /** Home */
        Home = 16,
        /** Capture */
        Capture = 17,
        /** LStickXAxis */
        LStickXAxis = 18,
        /** LStickYAxis */
        LStickYAxis = 19,
        /** RStickXAxis */
        RStickXAxis = 20,
        /** RStickYAxis */
        RStickYAxis = 21
    }


    /**
     * Subset of DeviceInput that only handles pointers and keyboard
     */
    export type DeviceSourceEvent<T extends DeviceType> = T extends DeviceType.Keyboard ? IKeyboardEvent : T extends DeviceType.Mouse ? IWheelEvent | IPointerEvent : T extends DeviceType.Touch ? IPointerEvent : never;
    /**
     * Class that handles all input for a specific device
     */
    export class DeviceSource<T extends DeviceType> {
        /** Type of device */
        readonly deviceType: T;
        /** "Slot" or index that device is referenced in */
        readonly deviceSlot: number;
        /**
         * Observable to handle device input changes per device
         */
        readonly onInputChangedObservable: Observable<DeviceSourceEvent<T>>;
        private readonly _deviceInputSystem;
        /**
         * Default Constructor
         * @param deviceInputSystem - Reference to DeviceInputSystem
         * @param deviceType - Type of device
         * @param deviceSlot - "Slot" or index that device is referenced in
         */
        constructor(deviceInputSystem: IDeviceInputSystem, 
        /** Type of device */
        deviceType: T, 
        /** "Slot" or index that device is referenced in */
        deviceSlot?: number);
        /**
         * Get input for specific input
         * @param inputIndex - index of specific input on device
         * @returns Input value from DeviceInputSystem
         */
        getInput(inputIndex: DeviceInput<T>): number;
    }


    /**
     * Class to keep track of devices
     */
    export class DeviceSourceManager implements IDisposable, IObservableManager {
        /**
         * Observable to be triggered when after a device is connected, any new observers added will be triggered against already connected devices
         */
        readonly onDeviceConnectedObservable: Observable<DeviceSourceType>;
        /**
         * Observable to be triggered when after a device is disconnected
         */
        readonly onDeviceDisconnectedObservable: Observable<DeviceSourceType>;
        private _engine;
        private _onDisposeObserver;
        private readonly _devices;
        private readonly _firstDevice;
        /**
         * Gets a DeviceSource, given a type and slot
         * @param deviceType - Type of Device
         * @param deviceSlot - Slot or ID of device
         * @returns DeviceSource
         */
        getDeviceSource<T extends DeviceType>(deviceType: T, deviceSlot?: number): Nullable<DeviceSource<T>>;
        /**
         * Gets an array of DeviceSource objects for a given device type
         * @param deviceType - Type of Device
         * @returns All available DeviceSources of a given type
         */
        getDeviceSources<T extends DeviceType>(deviceType: T): ReadonlyArray<DeviceSource<T>>;
        /**
         * Default constructor
         * @param engine - Used to get canvas (if applicable)
         */
        constructor(engine: Engine);
        /**
         * Dispose of DeviceSourceManager
         */
        dispose(): void;
        /**
         * @param deviceSource - Source to add
         * @internal
         */
        _addDevice(deviceSource: DeviceSourceType): void;
        /**
         * @param deviceType - DeviceType
         * @param deviceSlot - DeviceSlot
         * @internal
         */
        _removeDevice(deviceType: DeviceType, deviceSlot: number): void;
        /**
         * @param deviceType - DeviceType
         * @param deviceSlot - DeviceSlot
         * @param eventData - Event
         * @internal
         */
        _onInputChanged<T extends DeviceType>(deviceType: T, deviceSlot: number, eventData: IUIEvent): void;
        private _updateFirstDevices;
    }


    /**
     * Type to handle enforcement of inputs
     */
    export type DeviceInput<T extends DeviceType> = T extends DeviceType.Keyboard | DeviceType.Generic ? number : T extends DeviceType.Mouse | DeviceType.Touch ? Exclude<PointerInput, PointerInput.Move | PointerInput.MouseWheelX | PointerInput.MouseWheelY | PointerInput.MouseWheelZ> : T extends DeviceType.DualShock ? DualShockInput : T extends DeviceType.Xbox ? XboxInput : T extends DeviceType.Switch ? SwitchInput : T extends DeviceType.DualSense ? DualSenseInput : never;


    /**
     * Interface for DeviceInputSystem implementations (JS and Native)
     */
    export interface IDeviceInputSystem extends IDisposable {
        /**
         * Checks for current device input value, given an id and input index. Throws exception if requested device not initialized.
         * @param deviceType Enum specifying device type
         * @param deviceSlot "Slot" or index that device is referenced in
         * @param inputIndex Id of input to be checked
         * @returns Current value of input
         */
        pollInput(deviceType: DeviceType, deviceSlot: number, inputIndex: number): number;
        /**
         * Check for a specific device in the DeviceInputSystem
         * @param deviceType Type of device to check for
         * @returns bool with status of device's existence
         */
        isDeviceAvailable(deviceType: DeviceType): boolean;
    }


    type Distribute<T> = T extends DeviceType ? DeviceSource<T> : never;
    export type DeviceSourceType = Distribute<DeviceType>;
        interface Engine {
            /** @internal */
            _deviceSourceManager?: InternalDeviceSourceManager;
        }
    /** @internal */
    export interface IObservableManager {
        onDeviceConnectedObservable: Observable<DeviceSourceType>;
        onDeviceDisconnectedObservable: Observable<DeviceSourceType>;
        _onInputChanged(deviceType: DeviceType, deviceSlot: number, eventData: IUIEvent): void;
        _addDevice(deviceSource: DeviceSource<DeviceType>): void;
        _removeDevice(deviceType: DeviceType, deviceSlot: number): void;
    }
    /** @internal */
    export class InternalDeviceSourceManager implements IDisposable {
        readonly _deviceInputSystem: IDeviceInputSystem;
        private readonly _devices;
        private readonly _registeredManagers;
        _refCount: number;
        constructor(engine: Engine);
        readonly registerManager: (manager: IObservableManager) => void;
        readonly unregisterManager: (manager: IObservableManager) => void;
        dispose(): void;
    }


    /** @internal */
    export class NativeDeviceInputSystem implements IDeviceInputSystem {
        private readonly _nativeInput;
        constructor(onDeviceConnected: (deviceType: DeviceType, deviceSlot: number) => void, onDeviceDisconnected: (deviceType: DeviceType, deviceSlot: number) => void, onInputChanged: (deviceType: DeviceType, deviceSlot: number, eventData: IUIEvent) => void);
        /**
         * Checks for current device input value, given an id and input index. Throws exception if requested device not initialized.
         * @param deviceType Enum specifying device type
         * @param deviceSlot "Slot" or index that device is referenced in
         * @param inputIndex Id of input to be checked
         * @returns Current value of input
         */
        pollInput(deviceType: DeviceType, deviceSlot: number, inputIndex: number): number;
        /**
         * Check for a specific device in the DeviceInputSystem
         * @param deviceType Type of device to check for
         * @returns bool with status of device's existence
         */
        isDeviceAvailable(deviceType: DeviceType): boolean;
        /**
         * Dispose of all the observables
         */
        dispose(): void;
        /**
         * For versions of BabylonNative that don't have the NativeInput plugin initialized, create a dummy version
         * @returns Object with dummy functions
         */
        private _createDummyNativeInput;
    }


    /** @internal */
    export class WebDeviceInputSystem implements IDeviceInputSystem {
        private _inputs;
        private _gamepads;
        private _keyboardActive;
        private _pointerActive;
        private _elementToAttachTo;
        private _metaKeys;
        private readonly _engine;
        private readonly _usingSafari;
        private readonly _usingMacOS;
        private _onDeviceConnected;
        private _onDeviceDisconnected;
        private _onInputChanged;
        private _keyboardDownEvent;
        private _keyboardUpEvent;
        private _keyboardBlurEvent;
        private _pointerMoveEvent;
        private _pointerDownEvent;
        private _pointerUpEvent;
        private _pointerCancelEvent;
        private _pointerWheelEvent;
        private _pointerBlurEvent;
        private _wheelEventName;
        private _eventsAttached;
        private _mouseId;
        private readonly _isUsingFirefox;
        private _activeTouchIds;
        private _maxTouchPoints;
        private _pointerInputClearObserver;
        private _gamepadConnectedEvent;
        private _gamepadDisconnectedEvent;
        private _eventPrefix;
        constructor(engine: Engine, onDeviceConnected: (deviceType: DeviceType, deviceSlot: number) => void, onDeviceDisconnected: (deviceType: DeviceType, deviceSlot: number) => void, onInputChanged: (deviceType: DeviceType, deviceSlot: number, eventData: IUIEvent) => void);
        /**
         * Checks for current device input value, given an id and input index. Throws exception if requested device not initialized.
         * @param deviceType Enum specifying device type
         * @param deviceSlot "Slot" or index that device is referenced in
         * @param inputIndex Id of input to be checked
         * @returns Current value of input
         */
        pollInput(deviceType: DeviceType, deviceSlot: number, inputIndex: number): number;
        /**
         * Check for a specific device in the DeviceInputSystem
         * @param deviceType Type of device to check for
         * @returns bool with status of device's existence
         */
        isDeviceAvailable(deviceType: DeviceType): boolean;
        /**
         * Dispose of all the eventlisteners
         */
        dispose(): void;
        /**
         * Enable listening for user input events
         */
        private _enableEvents;
        /**
         * Disable listening for user input events
         */
        private _disableEvents;
        /**
         * Checks for existing connections to devices and register them, if necessary
         * Currently handles gamepads and mouse
         */
        private _checkForConnectedDevices;
        /**
         * Add a gamepad to the DeviceInputSystem
         * @param gamepad A single DOM Gamepad object
         */
        private _addGamePad;
        /**
         * Add pointer device to DeviceInputSystem
         * @param deviceType Type of Pointer to add
         * @param deviceSlot Pointer ID (0 for mouse, pointerId for Touch)
         * @param currentX Current X at point of adding
         * @param currentY Current Y at point of adding
         */
        private _addPointerDevice;
        /**
         * Add device and inputs to device array
         * @param deviceType Enum specifying device type
         * @param deviceSlot "Slot" or index that device is referenced in
         * @param numberOfInputs Number of input entries to create for given device
         */
        private _registerDevice;
        /**
         * Given a specific device name, remove that device from the device map
         * @param deviceType Enum specifying device type
         * @param deviceSlot "Slot" or index that device is referenced in
         */
        private _unregisterDevice;
        /**
         * Handle all actions that come from keyboard interaction
         */
        private _handleKeyActions;
        /**
         * Handle all actions that come from pointer interaction
         */
        private _handlePointerActions;
        /**
         * Handle all actions that come from gamepad interaction
         */
        private _handleGamepadActions;
        /**
         * Update all non-event based devices with each frame
         * @param deviceType Enum specifying device type
         * @param deviceSlot "Slot" or index that device is referenced in
         * @param inputIndex Id of input to be checked
         */
        private _updateDevice;
        /**
         * Gets DeviceType from the device name
         * @param deviceName Name of Device from DeviceInputSystem
         * @returns DeviceType enum value
         */
        private _getGamepadDeviceType;
        /**
         * Get DeviceType from a given pointer/mouse/touch event.
         * @param evt PointerEvent to evaluate
         * @returns DeviceType interpreted from event
         */
        private _getPointerType;
    }


    /** Defines the cross module used constants to avoid circular dependencies */
    export class Constants {
        /** Defines that alpha blending is disabled */
        static readonly ALPHA_DISABLE = 0;
        /** Defines that alpha blending is SRC ALPHA * SRC + DEST */
        static readonly ALPHA_ADD = 1;
        /** Defines that alpha blending is SRC ALPHA * SRC + (1 - SRC ALPHA) * DEST */
        static readonly ALPHA_COMBINE = 2;
        /** Defines that alpha blending is DEST - SRC * DEST */
        static readonly ALPHA_SUBTRACT = 3;
        /** Defines that alpha blending is SRC * DEST */
        static readonly ALPHA_MULTIPLY = 4;
        /** Defines that alpha blending is SRC ALPHA * SRC + (1 - SRC) * DEST */
        static readonly ALPHA_MAXIMIZED = 5;
        /** Defines that alpha blending is SRC + DEST */
        static readonly ALPHA_ONEONE = 6;
        /** Defines that alpha blending is SRC + (1 - SRC ALPHA) * DEST */
        static readonly ALPHA_PREMULTIPLIED = 7;
        /**
         * Defines that alpha blending is SRC + (1 - SRC ALPHA) * DEST
         * Alpha will be set to (1 - SRC ALPHA) * DEST ALPHA
         */
        static readonly ALPHA_PREMULTIPLIED_PORTERDUFF = 8;
        /** Defines that alpha blending is CST * SRC + (1 - CST) * DEST */
        static readonly ALPHA_INTERPOLATE = 9;
        /**
         * Defines that alpha blending is SRC + (1 - SRC) * DEST
         * Alpha will be set to SRC ALPHA + (1 - SRC ALPHA) * DEST ALPHA
         */
        static readonly ALPHA_SCREENMODE = 10;
        /**
         * Defines that alpha blending is SRC + DST
         * Alpha will be set to SRC ALPHA + DST ALPHA
         */
        static readonly ALPHA_ONEONE_ONEONE = 11;
        /**
         * Defines that alpha blending is SRC * DST ALPHA + DST
         * Alpha will be set to 0
         */
        static readonly ALPHA_ALPHATOCOLOR = 12;
        /**
         * Defines that alpha blending is SRC * (1 - DST) + DST * (1 - SRC)
         */
        static readonly ALPHA_REVERSEONEMINUS = 13;
        /**
         * Defines that alpha blending is SRC + DST * (1 - SRC ALPHA)
         * Alpha will be set to SRC ALPHA + DST ALPHA * (1 - SRC ALPHA)
         */
        static readonly ALPHA_SRC_DSTONEMINUSSRCALPHA = 14;
        /**
         * Defines that alpha blending is SRC + DST
         * Alpha will be set to SRC ALPHA
         */
        static readonly ALPHA_ONEONE_ONEZERO = 15;
        /**
         * Defines that alpha blending is SRC * (1 - DST) + DST * (1 - SRC)
         * Alpha will be set to DST ALPHA
         */
        static readonly ALPHA_EXCLUSION = 16;
        /**
         * Defines that alpha blending is SRC * SRC ALPHA + DST * (1 - SRC ALPHA)
         * Alpha will be set to SRC ALPHA + (1 - SRC ALPHA) * DST ALPHA
         */
        static readonly ALPHA_LAYER_ACCUMULATE = 17;
        /** Defines that alpha blending equation a SUM */
        static readonly ALPHA_EQUATION_ADD = 0;
        /** Defines that alpha blending equation a SUBSTRACTION */
        static readonly ALPHA_EQUATION_SUBSTRACT = 1;
        /** Defines that alpha blending equation a REVERSE SUBSTRACTION */
        static readonly ALPHA_EQUATION_REVERSE_SUBTRACT = 2;
        /** Defines that alpha blending equation a MAX operation */
        static readonly ALPHA_EQUATION_MAX = 3;
        /** Defines that alpha blending equation a MIN operation */
        static readonly ALPHA_EQUATION_MIN = 4;
        /**
         * Defines that alpha blending equation a DARKEN operation:
         * It takes the min of the src and sums the alpha channels.
         */
        static readonly ALPHA_EQUATION_DARKEN = 5;
        /** Defines that the resource is not delayed*/
        static readonly DELAYLOADSTATE_NONE = 0;
        /** Defines that the resource was successfully delay loaded */
        static readonly DELAYLOADSTATE_LOADED = 1;
        /** Defines that the resource is currently delay loading */
        static readonly DELAYLOADSTATE_LOADING = 2;
        /** Defines that the resource is delayed and has not started loading */
        static readonly DELAYLOADSTATE_NOTLOADED = 4;
        /** Passed to depthFunction or stencilFunction to specify depth or stencil tests will never pass. i.e. Nothing will be drawn */
        static readonly NEVER = 512;
        /** Passed to depthFunction or stencilFunction to specify depth or stencil tests will always pass. i.e. Pixels will be drawn in the order they are drawn */
        static readonly ALWAYS = 519;
        /** Passed to depthFunction or stencilFunction to specify depth or stencil tests will pass if the new depth value is less than the stored value */
        static readonly LESS = 513;
        /** Passed to depthFunction or stencilFunction to specify depth or stencil tests will pass if the new depth value is equals to the stored value */
        static readonly EQUAL = 514;
        /** Passed to depthFunction or stencilFunction to specify depth or stencil tests will pass if the new depth value is less than or equal to the stored value */
        static readonly LEQUAL = 515;
        /** Passed to depthFunction or stencilFunction to specify depth or stencil tests will pass if the new depth value is greater than the stored value */
        static readonly GREATER = 516;
        /** Passed to depthFunction or stencilFunction to specify depth or stencil tests will pass if the new depth value is greater than or equal to the stored value */
        static readonly GEQUAL = 518;
        /** Passed to depthFunction or stencilFunction to specify depth or stencil tests will pass if the new depth value is not equal to the stored value */
        static readonly NOTEQUAL = 517;
        /** Passed to stencilOperation to specify that stencil value must be kept */
        static readonly KEEP = 7680;
        /** Passed to stencilOperation to specify that stencil value must be zero */
        static readonly ZERO = 0;
        /** Passed to stencilOperation to specify that stencil value must be replaced */
        static readonly REPLACE = 7681;
        /** Passed to stencilOperation to specify that stencil value must be incremented */
        static readonly INCR = 7682;
        /** Passed to stencilOperation to specify that stencil value must be decremented */
        static readonly DECR = 7683;
        /** Passed to stencilOperation to specify that stencil value must be inverted */
        static readonly INVERT = 5386;
        /** Passed to stencilOperation to specify that stencil value must be incremented with wrapping */
        static readonly INCR_WRAP = 34055;
        /** Passed to stencilOperation to specify that stencil value must be decremented with wrapping */
        static readonly DECR_WRAP = 34056;
        /** Texture is not repeating outside of 0..1 UVs */
        static readonly TEXTURE_CLAMP_ADDRESSMODE = 0;
        /** Texture is repeating outside of 0..1 UVs */
        static readonly TEXTURE_WRAP_ADDRESSMODE = 1;
        /** Texture is repeating and mirrored */
        static readonly TEXTURE_MIRROR_ADDRESSMODE = 2;
        /** Flag to create a storage texture */
        static readonly TEXTURE_CREATIONFLAG_STORAGE = 1;
        /** ALPHA */
        static readonly TEXTUREFORMAT_ALPHA = 0;
        /** LUMINANCE */
        static readonly TEXTUREFORMAT_LUMINANCE = 1;
        /** LUMINANCE_ALPHA */
        static readonly TEXTUREFORMAT_LUMINANCE_ALPHA = 2;
        /** RGB */
        static readonly TEXTUREFORMAT_RGB = 4;
        /** RGBA */
        static readonly TEXTUREFORMAT_RGBA = 5;
        /** RED */
        static readonly TEXTUREFORMAT_RED = 6;
        /** RED (2nd reference) */
        static readonly TEXTUREFORMAT_R = 6;
        /** RG */
        static readonly TEXTUREFORMAT_RG = 7;
        /** RED_INTEGER */
        static readonly TEXTUREFORMAT_RED_INTEGER = 8;
        /** RED_INTEGER (2nd reference) */
        static readonly TEXTUREFORMAT_R_INTEGER = 8;
        /** RG_INTEGER */
        static readonly TEXTUREFORMAT_RG_INTEGER = 9;
        /** RGB_INTEGER */
        static readonly TEXTUREFORMAT_RGB_INTEGER = 10;
        /** RGBA_INTEGER */
        static readonly TEXTUREFORMAT_RGBA_INTEGER = 11;
        /** BGRA */
        static readonly TEXTUREFORMAT_BGRA = 12;
        /** Depth 24 bits + Stencil 8 bits */
        static readonly TEXTUREFORMAT_DEPTH24_STENCIL8 = 13;
        /** Depth 32 bits float */
        static readonly TEXTUREFORMAT_DEPTH32_FLOAT = 14;
        /** Depth 16 bits */
        static readonly TEXTUREFORMAT_DEPTH16 = 15;
        /** Depth 24 bits */
        static readonly TEXTUREFORMAT_DEPTH24 = 16;
        /** Depth 24 bits unorm + Stencil 8 bits */
        static readonly TEXTUREFORMAT_DEPTH24UNORM_STENCIL8 = 17;
        /** Depth 32 bits float + Stencil 8 bits */
        static readonly TEXTUREFORMAT_DEPTH32FLOAT_STENCIL8 = 18;
        /** Compressed BC7 */
        static readonly TEXTUREFORMAT_COMPRESSED_RGBA_BPTC_UNORM = 36492;
        /** Compressed BC7 (SRGB) */
        static readonly TEXTUREFORMAT_COMPRESSED_SRGB_ALPHA_BPTC_UNORM = 36493;
        /** Compressed BC6 unsigned float */
        static readonly TEXTUREFORMAT_COMPRESSED_RGB_BPTC_UNSIGNED_FLOAT = 36495;
        /** Compressed BC6 signed float */
        static readonly TEXTUREFORMAT_COMPRESSED_RGB_BPTC_SIGNED_FLOAT = 36494;
        /** Compressed BC3 */
        static readonly TEXTUREFORMAT_COMPRESSED_RGBA_S3TC_DXT5 = 33779;
        /** Compressed BC3 (SRGB) */
        static readonly TEXTUREFORMAT_COMPRESSED_SRGB_ALPHA_S3TC_DXT5_EXT = 35919;
        /** Compressed BC2 */
        static readonly TEXTUREFORMAT_COMPRESSED_RGBA_S3TC_DXT3 = 33778;
        /** Compressed BC2 (SRGB) */
        static readonly TEXTUREFORMAT_COMPRESSED_SRGB_ALPHA_S3TC_DXT3_EXT = 35918;
        /** Compressed BC1 (RGBA) */
        static readonly TEXTUREFORMAT_COMPRESSED_RGBA_S3TC_DXT1 = 33777;
        /** Compressed BC1 (RGB) */
        static readonly TEXTUREFORMAT_COMPRESSED_RGB_S3TC_DXT1 = 33776;
        /** Compressed BC1 (SRGB+A) */
        static readonly TEXTUREFORMAT_COMPRESSED_SRGB_ALPHA_S3TC_DXT1_EXT = 35917;
        /** Compressed BC1 (SRGB) */
        static readonly TEXTUREFORMAT_COMPRESSED_SRGB_S3TC_DXT1_EXT = 35916;
        /** Compressed ASTC 4x4 */
        static readonly TEXTUREFORMAT_COMPRESSED_RGBA_ASTC_4x4 = 37808;
        /** Compressed ASTC 4x4 (SRGB) */
        static readonly TEXTUREFORMAT_COMPRESSED_SRGB8_ALPHA8_ASTC_4x4_KHR = 37840;
        /** Compressed ETC1 (RGB) */
        static readonly TEXTUREFORMAT_COMPRESSED_RGB_ETC1_WEBGL = 36196;
        /** Compressed ETC2 (RGB) */
        static readonly TEXTUREFORMAT_COMPRESSED_RGB8_ETC2 = 37492;
        /** Compressed ETC2 (SRGB) */
        static readonly TEXTUREFORMAT_COMPRESSED_SRGB8_ETC2 = 37493;
        /** Compressed ETC2 (RGB+A1) */
        static readonly TEXTUREFORMAT_COMPRESSED_RGB8_PUNCHTHROUGH_ALPHA1_ETC2 = 37494;
        /** Compressed ETC2 (SRGB+A1)*/
        static readonly TEXTUREFORMAT_COMPRESSED_SRGB8_PUNCHTHROUGH_ALPHA1_ETC2 = 37495;
        /** Compressed ETC2 (RGB+A) */
        static readonly TEXTUREFORMAT_COMPRESSED_RGBA8_ETC2_EAC = 37496;
        /** Compressed ETC2 (SRGB+1) */
        static readonly TEXTUREFORMAT_COMPRESSED_SRGB8_ALPHA8_ETC2_EAC = 37497;
        /** UNSIGNED_BYTE */
        static readonly TEXTURETYPE_UNSIGNED_BYTE = 0;
        /** UNSIGNED_BYTE (2nd reference) */
        static readonly TEXTURETYPE_UNSIGNED_INT = 0;
        /** FLOAT */
        static readonly TEXTURETYPE_FLOAT = 1;
        /** HALF_FLOAT */
        static readonly TEXTURETYPE_HALF_FLOAT = 2;
        /** BYTE */
        static readonly TEXTURETYPE_BYTE = 3;
        /** SHORT */
        static readonly TEXTURETYPE_SHORT = 4;
        /** UNSIGNED_SHORT */
        static readonly TEXTURETYPE_UNSIGNED_SHORT = 5;
        /** INT */
        static readonly TEXTURETYPE_INT = 6;
        /** UNSIGNED_INT */
        static readonly TEXTURETYPE_UNSIGNED_INTEGER = 7;
        /** UNSIGNED_SHORT_4_4_4_4 */
        static readonly TEXTURETYPE_UNSIGNED_SHORT_4_4_4_4 = 8;
        /** UNSIGNED_SHORT_5_5_5_1 */
        static readonly TEXTURETYPE_UNSIGNED_SHORT_5_5_5_1 = 9;
        /** UNSIGNED_SHORT_5_6_5 */
        static readonly TEXTURETYPE_UNSIGNED_SHORT_5_6_5 = 10;
        /** UNSIGNED_INT_2_10_10_10_REV */
        static readonly TEXTURETYPE_UNSIGNED_INT_2_10_10_10_REV = 11;
        /** UNSIGNED_INT_24_8 */
        static readonly TEXTURETYPE_UNSIGNED_INT_24_8 = 12;
        /** UNSIGNED_INT_10F_11F_11F_REV */
        static readonly TEXTURETYPE_UNSIGNED_INT_10F_11F_11F_REV = 13;
        /** UNSIGNED_INT_5_9_9_9_REV */
        static readonly TEXTURETYPE_UNSIGNED_INT_5_9_9_9_REV = 14;
        /** FLOAT_32_UNSIGNED_INT_24_8_REV */
        static readonly TEXTURETYPE_FLOAT_32_UNSIGNED_INT_24_8_REV = 15;
        /** UNDEFINED */
        static readonly TEXTURETYPE_UNDEFINED = 16;
        /** nearest is mag = nearest and min = nearest and no mip */
        static readonly TEXTURE_NEAREST_SAMPLINGMODE = 1;
        /** mag = nearest and min = nearest and mip = none */
        static readonly TEXTURE_NEAREST_NEAREST = 1;
        /** Bilinear is mag = linear and min = linear and no mip */
        static readonly TEXTURE_BILINEAR_SAMPLINGMODE = 2;
        /** mag = linear and min = linear and mip = none */
        static readonly TEXTURE_LINEAR_LINEAR = 2;
        /** Trilinear is mag = linear and min = linear and mip = linear */
        static readonly TEXTURE_TRILINEAR_SAMPLINGMODE = 3;
        /** Trilinear is mag = linear and min = linear and mip = linear */
        static readonly TEXTURE_LINEAR_LINEAR_MIPLINEAR = 3;
        /** mag = nearest and min = nearest and mip = nearest */
        static readonly TEXTURE_NEAREST_NEAREST_MIPNEAREST = 4;
        /** mag = nearest and min = linear and mip = nearest */
        static readonly TEXTURE_NEAREST_LINEAR_MIPNEAREST = 5;
        /** mag = nearest and min = linear and mip = linear */
        static readonly TEXTURE_NEAREST_LINEAR_MIPLINEAR = 6;
        /** mag = nearest and min = linear and mip = none */
        static readonly TEXTURE_NEAREST_LINEAR = 7;
        /** nearest is mag = nearest and min = nearest and mip = linear */
        static readonly TEXTURE_NEAREST_NEAREST_MIPLINEAR = 8;
        /** mag = linear and min = nearest and mip = nearest */
        static readonly TEXTURE_LINEAR_NEAREST_MIPNEAREST = 9;
        /** mag = linear and min = nearest and mip = linear */
        static readonly TEXTURE_LINEAR_NEAREST_MIPLINEAR = 10;
        /** Bilinear is mag = linear and min = linear and mip = nearest */
        static readonly TEXTURE_LINEAR_LINEAR_MIPNEAREST = 11;
        /** mag = linear and min = nearest and mip = none */
        static readonly TEXTURE_LINEAR_NEAREST = 12;
        /** Explicit coordinates mode */
        static readonly TEXTURE_EXPLICIT_MODE = 0;
        /** Spherical coordinates mode */
        static readonly TEXTURE_SPHERICAL_MODE = 1;
        /** Planar coordinates mode */
        static readonly TEXTURE_PLANAR_MODE = 2;
        /** Cubic coordinates mode */
        static readonly TEXTURE_CUBIC_MODE = 3;
        /** Projection coordinates mode */
        static readonly TEXTURE_PROJECTION_MODE = 4;
        /** Skybox coordinates mode */
        static readonly TEXTURE_SKYBOX_MODE = 5;
        /** Inverse Cubic coordinates mode */
        static readonly TEXTURE_INVCUBIC_MODE = 6;
        /** Equirectangular coordinates mode */
        static readonly TEXTURE_EQUIRECTANGULAR_MODE = 7;
        /** Equirectangular Fixed coordinates mode */
        static readonly TEXTURE_FIXED_EQUIRECTANGULAR_MODE = 8;
        /** Equirectangular Fixed Mirrored coordinates mode */
        static readonly TEXTURE_FIXED_EQUIRECTANGULAR_MIRRORED_MODE = 9;
        /** Offline (baking) quality for texture filtering */
        static readonly TEXTURE_FILTERING_QUALITY_OFFLINE = 4096;
        /** High quality for texture filtering */
        static readonly TEXTURE_FILTERING_QUALITY_HIGH = 64;
        /** Medium quality for texture filtering */
        static readonly TEXTURE_FILTERING_QUALITY_MEDIUM = 16;
        /** Low quality for texture filtering */
        static readonly TEXTURE_FILTERING_QUALITY_LOW = 8;
        /** Defines that texture rescaling will use a floor to find the closer power of 2 size */
        static readonly SCALEMODE_FLOOR = 1;
        /** Defines that texture rescaling will look for the nearest power of 2 size */
        static readonly SCALEMODE_NEAREST = 2;
        /** Defines that texture rescaling will use a ceil to find the closer power of 2 size */
        static readonly SCALEMODE_CEILING = 3;
        /**
         * The dirty texture flag value
         */
        static readonly MATERIAL_TextureDirtyFlag = 1;
        /**
         * The dirty light flag value
         */
        static readonly MATERIAL_LightDirtyFlag = 2;
        /**
         * The dirty fresnel flag value
         */
        static readonly MATERIAL_FresnelDirtyFlag = 4;
        /**
         * The dirty attribute flag value
         */
        static readonly MATERIAL_AttributesDirtyFlag = 8;
        /**
         * The dirty misc flag value
         */
        static readonly MATERIAL_MiscDirtyFlag = 16;
        /**
         * The dirty prepass flag value
         */
        static readonly MATERIAL_PrePassDirtyFlag = 32;
        /**
         * The all dirty flag value
         */
        static readonly MATERIAL_AllDirtyFlag = 63;
        /**
         * Returns the triangle fill mode
         */
        static readonly MATERIAL_TriangleFillMode = 0;
        /**
         * Returns the wireframe mode
         */
        static readonly MATERIAL_WireFrameFillMode = 1;
        /**
         * Returns the point fill mode
         */
        static readonly MATERIAL_PointFillMode = 2;
        /**
         * Returns the point list draw mode
         */
        static readonly MATERIAL_PointListDrawMode = 3;
        /**
         * Returns the line list draw mode
         */
        static readonly MATERIAL_LineListDrawMode = 4;
        /**
         * Returns the line loop draw mode
         */
        static readonly MATERIAL_LineLoopDrawMode = 5;
        /**
         * Returns the line strip draw mode
         */
        static readonly MATERIAL_LineStripDrawMode = 6;
        /**
         * Returns the triangle strip draw mode
         */
        static readonly MATERIAL_TriangleStripDrawMode = 7;
        /**
         * Returns the triangle fan draw mode
         */
        static readonly MATERIAL_TriangleFanDrawMode = 8;
        /**
         * Stores the clock-wise side orientation
         */
        static readonly MATERIAL_ClockWiseSideOrientation = 0;
        /**
         * Stores the counter clock-wise side orientation
         */
        static readonly MATERIAL_CounterClockWiseSideOrientation = 1;
        /**
         * Nothing
         * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
         */
        static readonly ACTION_NothingTrigger = 0;
        /**
         * On pick
         * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
         */
        static readonly ACTION_OnPickTrigger = 1;
        /**
         * On left pick
         * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
         */
        static readonly ACTION_OnLeftPickTrigger = 2;
        /**
         * On right pick
         * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
         */
        static readonly ACTION_OnRightPickTrigger = 3;
        /**
         * On center pick
         * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
         */
        static readonly ACTION_OnCenterPickTrigger = 4;
        /**
         * On pick down
         * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
         */
        static readonly ACTION_OnPickDownTrigger = 5;
        /**
         * On double pick
         * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
         */
        static readonly ACTION_OnDoublePickTrigger = 6;
        /**
         * On pick up
         * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
         */
        static readonly ACTION_OnPickUpTrigger = 7;
        /**
         * On pick out.
         * This trigger will only be raised if you also declared a OnPickDown
         * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
         */
        static readonly ACTION_OnPickOutTrigger = 16;
        /**
         * On long press
         * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
         */
        static readonly ACTION_OnLongPressTrigger = 8;
        /**
         * On pointer over
         * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
         */
        static readonly ACTION_OnPointerOverTrigger = 9;
        /**
         * On pointer out
         * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
         */
        static readonly ACTION_OnPointerOutTrigger = 10;
        /**
         * On every frame
         * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
         */
        static readonly ACTION_OnEveryFrameTrigger = 11;
        /**
         * On intersection enter
         * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
         */
        static readonly ACTION_OnIntersectionEnterTrigger = 12;
        /**
         * On intersection exit
         * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
         */
        static readonly ACTION_OnIntersectionExitTrigger = 13;
        /**
         * On key down
         * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
         */
        static readonly ACTION_OnKeyDownTrigger = 14;
        /**
         * On key up
         * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions#triggers
         */
        static readonly ACTION_OnKeyUpTrigger = 15;
        /**
         * Billboard mode will only apply to Y axis
         */
        static readonly PARTICLES_BILLBOARDMODE_Y = 2;
        /**
         * Billboard mode will apply to all axes
         */
        static readonly PARTICLES_BILLBOARDMODE_ALL = 7;
        /**
         * Special billboard mode where the particle will be biilboard to the camera but rotated to align with direction
         */
        static readonly PARTICLES_BILLBOARDMODE_STRETCHED = 8;
        /**
         * Special billboard mode where the particle will be billboard to the camera but only around the axis of the direction of particle emission
         */
        static readonly PARTICLES_BILLBOARDMODE_STRETCHED_LOCAL = 9;
        /** Default culling strategy : this is an exclusion test and it's the more accurate.
         *  Test order :
         *  Is the bounding sphere outside the frustum ?
         *  If not, are the bounding box vertices outside the frustum ?
         *  It not, then the cullable object is in the frustum.
         */
        static readonly MESHES_CULLINGSTRATEGY_STANDARD = 0;
        /** Culling strategy : Bounding Sphere Only.
         *  This is an exclusion test. It's faster than the standard strategy because the bounding box is not tested.
         *  It's also less accurate than the standard because some not visible objects can still be selected.
         *  Test : is the bounding sphere outside the frustum ?
         *  If not, then the cullable object is in the frustum.
         */
        static readonly MESHES_CULLINGSTRATEGY_BOUNDINGSPHERE_ONLY = 1;
        /** Culling strategy : Optimistic Inclusion.
         *  This in an inclusion test first, then the standard exclusion test.
         *  This can be faster when a cullable object is expected to be almost always in the camera frustum.
         *  This could also be a little slower than the standard test when the tested object center is not the frustum but one of its bounding box vertex is still inside.
         *  Anyway, it's as accurate as the standard strategy.
         *  Test :
         *  Is the cullable object bounding sphere center in the frustum ?
         *  If not, apply the default culling strategy.
         */
        static readonly MESHES_CULLINGSTRATEGY_OPTIMISTIC_INCLUSION = 2;
        /** Culling strategy : Optimistic Inclusion then Bounding Sphere Only.
         *  This in an inclusion test first, then the bounding sphere only exclusion test.
         *  This can be the fastest test when a cullable object is expected to be almost always in the camera frustum.
         *  This could also be a little slower than the BoundingSphereOnly strategy when the tested object center is not in the frustum but its bounding sphere still intersects it.
         *  It's less accurate than the standard strategy and as accurate as the BoundingSphereOnly strategy.
         *  Test :
         *  Is the cullable object bounding sphere center in the frustum ?
         *  If not, apply the Bounding Sphere Only strategy. No Bounding Box is tested here.
         */
        static readonly MESHES_CULLINGSTRATEGY_OPTIMISTIC_INCLUSION_THEN_BSPHERE_ONLY = 3;
        /**
         * No logging while loading
         */
        static readonly SCENELOADER_NO_LOGGING = 0;
        /**
         * Minimal logging while loading
         */
        static readonly SCENELOADER_MINIMAL_LOGGING = 1;
        /**
         * Summary logging while loading
         */
        static readonly SCENELOADER_SUMMARY_LOGGING = 2;
        /**
         * Detailed logging while loading
         */
        static readonly SCENELOADER_DETAILED_LOGGING = 3;
        /**
         * Constant used to retrieve the irradiance texture index in the textures array in the prepass
         * using getIndex(Constants.PREPASS_IRRADIANCE_TEXTURE_TYPE)
         */
        static readonly PREPASS_IRRADIANCE_TEXTURE_TYPE = 0;
        /**
         * Constant used to retrieve the position texture index in the textures array in the prepass
         * using getIndex(Constants.PREPASS_POSITION_TEXTURE_INDEX)
         */
        static readonly PREPASS_POSITION_TEXTURE_TYPE = 1;
        /**
         * Constant used to retrieve the velocity texture index in the textures array in the prepass
         * using getIndex(Constants.PREPASS_VELOCITY_TEXTURE_INDEX)
         */
        static readonly PREPASS_VELOCITY_TEXTURE_TYPE = 2;
        /**
         * Constant used to retrieve the reflectivity texture index in the textures array in the prepass
         * using the getIndex(Constants.PREPASS_REFLECTIVITY_TEXTURE_TYPE)
         */
        static readonly PREPASS_REFLECTIVITY_TEXTURE_TYPE = 3;
        /**
         * Constant used to retrieve the lit color texture index in the textures array in the prepass
         * using the getIndex(Constants.PREPASS_COLOR_TEXTURE_TYPE)
         */
        static readonly PREPASS_COLOR_TEXTURE_TYPE = 4;
        /**
         * Constant used to retrieve depth index in the textures array in the prepass
         * using the getIndex(Constants.PREPASS_DEPTH_TEXTURE_TYPE)
         */
        static readonly PREPASS_DEPTH_TEXTURE_TYPE = 5;
        /**
         * Constant used to retrieve normal index in the textures array in the prepass
         * using the getIndex(Constants.PREPASS_NORMAL_TEXTURE_TYPE)
         */
        static readonly PREPASS_NORMAL_TEXTURE_TYPE = 6;
        /**
         * Constant used to retrieve albedo index in the textures array in the prepass
         * using the getIndex(Constants.PREPASS_ALBEDO_SQRT_TEXTURE_TYPE)
         */
        static readonly PREPASS_ALBEDO_SQRT_TEXTURE_TYPE = 7;
        /** Flag to create a readable buffer (the buffer can be the source of a copy) */
        static readonly BUFFER_CREATIONFLAG_READ = 1;
        /** Flag to create a writable buffer (the buffer can be the destination of a copy) */
        static readonly BUFFER_CREATIONFLAG_WRITE = 2;
        /** Flag to create a readable and writable buffer */
        static readonly BUFFER_CREATIONFLAG_READWRITE = 3;
        /** Flag to create a buffer suitable to be used as a uniform buffer */
        static readonly BUFFER_CREATIONFLAG_UNIFORM = 4;
        /** Flag to create a buffer suitable to be used as a vertex buffer */
        static readonly BUFFER_CREATIONFLAG_VERTEX = 8;
        /** Flag to create a buffer suitable to be used as an index buffer */
        static readonly BUFFER_CREATIONFLAG_INDEX = 16;
        /** Flag to create a buffer suitable to be used as a storage buffer */
        static readonly BUFFER_CREATIONFLAG_STORAGE = 32;
        /**
         * Prefixes used by the engine for sub mesh draw wrappers
         */
        /** @internal */
        static readonly RENDERPASS_MAIN = 0;
        /**
         * Constant used as key code for Alt key
         */
        static readonly INPUT_ALT_KEY = 18;
        /**
         * Constant used as key code for Ctrl key
         */
        static readonly INPUT_CTRL_KEY = 17;
        /**
         * Constant used as key code for Meta key (Left Win, Left Cmd)
         */
        static readonly INPUT_META_KEY1 = 91;
        /**
         * Constant used as key code for Meta key (Right Win)
         */
        static readonly INPUT_META_KEY2 = 92;
        /**
         * Constant used as key code for Meta key (Right Win, Right Cmd)
         */
        static readonly INPUT_META_KEY3 = 93;
        /**
         * Constant used as key code for Shift key
         */
        static readonly INPUT_SHIFT_KEY = 16;
        /** Standard snapshot rendering. In this mode, some form of dynamic behavior is possible (for eg, uniform buffers are still updated) */
        static readonly SNAPSHOTRENDERING_STANDARD = 0;
        /** Fast snapshot rendering. In this mode, everything is static and only some limited form of dynamic behaviour is possible */
        static readonly SNAPSHOTRENDERING_FAST = 1;
        /**
         * This is the default projection mode used by the cameras.
         * It helps recreating a feeling of perspective and better appreciate depth.
         * This is the best way to simulate real life cameras.
         */
        static readonly PERSPECTIVE_CAMERA = 0;
        /**
         * This helps creating camera with an orthographic mode.
         * Orthographic is commonly used in engineering as a means to produce object specifications that communicate dimensions unambiguously, each line of 1 unit length (cm, meter..whatever) will appear to have the same length everywhere on the drawing. This allows the drafter to dimension only a subset of lines and let the reader know that other lines of that length on the drawing are also that length in reality. Every parallel line in the drawing is also parallel in the object.
         */
        static readonly ORTHOGRAPHIC_CAMERA = 1;
        /**
         * This is the default FOV mode for perspective cameras.
         * This setting aligns the upper and lower bounds of the viewport to the upper and lower bounds of the camera frustum.
         */
        static readonly FOVMODE_VERTICAL_FIXED = 0;
        /**
         * This setting aligns the left and right bounds of the viewport to the left and right bounds of the camera frustum.
         */
        static readonly FOVMODE_HORIZONTAL_FIXED = 1;
        /**
         * This specifies there is no need for a camera rig.
         * Basically only one eye is rendered corresponding to the camera.
         */
        static readonly RIG_MODE_NONE = 0;
        /**
         * Simulates a camera Rig with one blue eye and one red eye.
         * This can be use with 3d blue and red glasses.
         */
        static readonly RIG_MODE_STEREOSCOPIC_ANAGLYPH = 10;
        /**
         * Defines that both eyes of the camera will be rendered side by side with a parallel target.
         */
        static readonly RIG_MODE_STEREOSCOPIC_SIDEBYSIDE_PARALLEL = 11;
        /**
         * Defines that both eyes of the camera will be rendered side by side with a none parallel target.
         */
        static readonly RIG_MODE_STEREOSCOPIC_SIDEBYSIDE_CROSSEYED = 12;
        /**
         * Defines that both eyes of the camera will be rendered over under each other.
         */
        static readonly RIG_MODE_STEREOSCOPIC_OVERUNDER = 13;
        /**
         * Defines that both eyes of the camera will be rendered on successive lines interlaced for passive 3d monitors.
         */
        static readonly RIG_MODE_STEREOSCOPIC_INTERLACED = 14;
        /**
         * Defines that both eyes of the camera should be renderered in a VR mode (carbox).
         */
        static readonly RIG_MODE_VR = 20;
        /**
         * Defines that both eyes of the camera should be renderered in a VR mode (webVR).
         */
        static readonly RIG_MODE_WEBVR = 21;
        /**
         * Custom rig mode allowing rig cameras to be populated manually with any number of cameras
         */
        static readonly RIG_MODE_CUSTOM = 22;
        /**
         * Maximum number of uv sets supported
         */
        static readonly MAX_SUPPORTED_UV_SETS = 6;
        /**
         * GL constants
         */
        /** Alpha blend equation: ADD */
        static readonly GL_ALPHA_EQUATION_ADD = 32774;
        /** Alpha equation: MIN */
        static readonly GL_ALPHA_EQUATION_MIN = 32775;
        /** Alpha equation: MAX */
        static readonly GL_ALPHA_EQUATION_MAX = 32776;
        /** Alpha equation: SUBTRACT */
        static readonly GL_ALPHA_EQUATION_SUBTRACT = 32778;
        /** Alpha equation: REVERSE_SUBTRACT */
        static readonly GL_ALPHA_EQUATION_REVERSE_SUBTRACT = 32779;
        /** Alpha blend function: SRC */
        static readonly GL_ALPHA_FUNCTION_SRC = 768;
        /** Alpha blend function: ONE_MINUS_SRC */
        static readonly GL_ALPHA_FUNCTION_ONE_MINUS_SRC_COLOR = 769;
        /** Alpha blend function: SRC_ALPHA */
        static readonly GL_ALPHA_FUNCTION_SRC_ALPHA = 770;
        /** Alpha blend function: ONE_MINUS_SRC_ALPHA */
        static readonly GL_ALPHA_FUNCTION_ONE_MINUS_SRC_ALPHA = 771;
        /** Alpha blend function: DST_ALPHA */
        static readonly GL_ALPHA_FUNCTION_DST_ALPHA = 772;
        /** Alpha blend function: ONE_MINUS_DST_ALPHA */
        static readonly GL_ALPHA_FUNCTION_ONE_MINUS_DST_ALPHA = 773;
        /** Alpha blend function: ONE_MINUS_DST */
        static readonly GL_ALPHA_FUNCTION_DST_COLOR = 774;
        /** Alpha blend function: ONE_MINUS_DST */
        static readonly GL_ALPHA_FUNCTION_ONE_MINUS_DST_COLOR = 775;
        /** Alpha blend function: SRC_ALPHA_SATURATED */
        static readonly GL_ALPHA_FUNCTION_SRC_ALPHA_SATURATED = 776;
        /** Alpha blend function: CONSTANT */
        static readonly GL_ALPHA_FUNCTION_CONSTANT_COLOR = 32769;
        /** Alpha blend function: ONE_MINUS_CONSTANT */
        static readonly GL_ALPHA_FUNCTION_ONE_MINUS_CONSTANT_COLOR = 32770;
        /** Alpha blend function: CONSTANT_ALPHA */
        static readonly GL_ALPHA_FUNCTION_CONSTANT_ALPHA = 32771;
        /** Alpha blend function: ONE_MINUS_CONSTANT_ALPHA */
        static readonly GL_ALPHA_FUNCTION_ONE_MINUS_CONSTANT_ALPHA = 32772;
        /** URL to the snippet server. Points to the public snippet server by default */
        static SnippetUrl: string;
    }


    /**
     * Defines the interface used by display changed events
     */
    export interface IDisplayChangedEventArgs {
        /** Gets the vrDisplay object (if any) */
        vrDisplay: Nullable<any>;
        /** Gets a boolean indicating if webVR is supported */
        vrSupported: boolean;
    }
    /**
     * Defines the interface used by objects containing a viewport (like a camera)
     */
    interface IViewportOwnerLike {
        /**
         * Gets or sets the viewport
         */
        viewport: IViewportLike;
    }
    /**
     * The engine class is responsible for interfacing with all lower-level APIs such as WebGL and Audio
     */
    export class Engine extends ThinEngine {
        /** Defines that alpha blending is disabled */
        static readonly ALPHA_DISABLE = 0;
        /** Defines that alpha blending to SRC ALPHA * SRC + DEST */
        static readonly ALPHA_ADD = 1;
        /** Defines that alpha blending to SRC ALPHA * SRC + (1 - SRC ALPHA) * DEST */
        static readonly ALPHA_COMBINE = 2;
        /** Defines that alpha blending to DEST - SRC * DEST */
        static readonly ALPHA_SUBTRACT = 3;
        /** Defines that alpha blending to SRC * DEST */
        static readonly ALPHA_MULTIPLY = 4;
        /** Defines that alpha blending to SRC ALPHA * SRC + (1 - SRC) * DEST */
        static readonly ALPHA_MAXIMIZED = 5;
        /** Defines that alpha blending to SRC + DEST */
        static readonly ALPHA_ONEONE = 6;
        /** Defines that alpha blending to SRC + (1 - SRC ALPHA) * DEST */
        static readonly ALPHA_PREMULTIPLIED = 7;
        /**
         * Defines that alpha blending to SRC + (1 - SRC ALPHA) * DEST
         * Alpha will be set to (1 - SRC ALPHA) * DEST ALPHA
         */
        static readonly ALPHA_PREMULTIPLIED_PORTERDUFF = 8;
        /** Defines that alpha blending to CST * SRC + (1 - CST) * DEST */
        static readonly ALPHA_INTERPOLATE = 9;
        /**
         * Defines that alpha blending to SRC + (1 - SRC) * DEST
         * Alpha will be set to SRC ALPHA + (1 - SRC ALPHA) * DEST ALPHA
         */
        static readonly ALPHA_SCREENMODE = 10;
        /** Defines that the resource is not delayed*/
        static readonly DELAYLOADSTATE_NONE = 0;
        /** Defines that the resource was successfully delay loaded */
        static readonly DELAYLOADSTATE_LOADED = 1;
        /** Defines that the resource is currently delay loading */
        static readonly DELAYLOADSTATE_LOADING = 2;
        /** Defines that the resource is delayed and has not started loading */
        static readonly DELAYLOADSTATE_NOTLOADED = 4;
        /** Passed to depthFunction or stencilFunction to specify depth or stencil tests will never pass. i.e. Nothing will be drawn */
        static readonly NEVER = 512;
        /** Passed to depthFunction or stencilFunction to specify depth or stencil tests will always pass. i.e. Pixels will be drawn in the order they are drawn */
        static readonly ALWAYS = 519;
        /** Passed to depthFunction or stencilFunction to specify depth or stencil tests will pass if the new depth value is less than the stored value */
        static readonly LESS = 513;
        /** Passed to depthFunction or stencilFunction to specify depth or stencil tests will pass if the new depth value is equals to the stored value */
        static readonly EQUAL = 514;
        /** Passed to depthFunction or stencilFunction to specify depth or stencil tests will pass if the new depth value is less than or equal to the stored value */
        static readonly LEQUAL = 515;
        /** Passed to depthFunction or stencilFunction to specify depth or stencil tests will pass if the new depth value is greater than the stored value */
        static readonly GREATER = 516;
        /** Passed to depthFunction or stencilFunction to specify depth or stencil tests will pass if the new depth value is greater than or equal to the stored value */
        static readonly GEQUAL = 518;
        /** Passed to depthFunction or stencilFunction to specify depth or stencil tests will pass if the new depth value is not equal to the stored value */
        static readonly NOTEQUAL = 517;
        /** Passed to stencilOperation to specify that stencil value must be kept */
        static readonly KEEP = 7680;
        /** Passed to stencilOperation to specify that stencil value must be replaced */
        static readonly REPLACE = 7681;
        /** Passed to stencilOperation to specify that stencil value must be incremented */
        static readonly INCR = 7682;
        /** Passed to stencilOperation to specify that stencil value must be decremented */
        static readonly DECR = 7683;
        /** Passed to stencilOperation to specify that stencil value must be inverted */
        static readonly INVERT = 5386;
        /** Passed to stencilOperation to specify that stencil value must be incremented with wrapping */
        static readonly INCR_WRAP = 34055;
        /** Passed to stencilOperation to specify that stencil value must be decremented with wrapping */
        static readonly DECR_WRAP = 34056;
        /** Texture is not repeating outside of 0..1 UVs */
        static readonly TEXTURE_CLAMP_ADDRESSMODE = 0;
        /** Texture is repeating outside of 0..1 UVs */
        static readonly TEXTURE_WRAP_ADDRESSMODE = 1;
        /** Texture is repeating and mirrored */
        static readonly TEXTURE_MIRROR_ADDRESSMODE = 2;
        /** ALPHA */
        static readonly TEXTUREFORMAT_ALPHA = 0;
        /** LUMINANCE */
        static readonly TEXTUREFORMAT_LUMINANCE = 1;
        /** LUMINANCE_ALPHA */
        static readonly TEXTUREFORMAT_LUMINANCE_ALPHA = 2;
        /** RGB */
        static readonly TEXTUREFORMAT_RGB = 4;
        /** RGBA */
        static readonly TEXTUREFORMAT_RGBA = 5;
        /** RED */
        static readonly TEXTUREFORMAT_RED = 6;
        /** RED (2nd reference) */
        static readonly TEXTUREFORMAT_R = 6;
        /** RG */
        static readonly TEXTUREFORMAT_RG = 7;
        /** RED_INTEGER */
        static readonly TEXTUREFORMAT_RED_INTEGER = 8;
        /** RED_INTEGER (2nd reference) */
        static readonly TEXTUREFORMAT_R_INTEGER = 8;
        /** RG_INTEGER */
        static readonly TEXTUREFORMAT_RG_INTEGER = 9;
        /** RGB_INTEGER */
        static readonly TEXTUREFORMAT_RGB_INTEGER = 10;
        /** RGBA_INTEGER */
        static readonly TEXTUREFORMAT_RGBA_INTEGER = 11;
        /** UNSIGNED_BYTE */
        static readonly TEXTURETYPE_UNSIGNED_BYTE = 0;
        /** UNSIGNED_BYTE (2nd reference) */
        static readonly TEXTURETYPE_UNSIGNED_INT = 0;
        /** FLOAT */
        static readonly TEXTURETYPE_FLOAT = 1;
        /** HALF_FLOAT */
        static readonly TEXTURETYPE_HALF_FLOAT = 2;
        /** BYTE */
        static readonly TEXTURETYPE_BYTE = 3;
        /** SHORT */
        static readonly TEXTURETYPE_SHORT = 4;
        /** UNSIGNED_SHORT */
        static readonly TEXTURETYPE_UNSIGNED_SHORT = 5;
        /** INT */
        static readonly TEXTURETYPE_INT = 6;
        /** UNSIGNED_INT */
        static readonly TEXTURETYPE_UNSIGNED_INTEGER = 7;
        /** UNSIGNED_SHORT_4_4_4_4 */
        static readonly TEXTURETYPE_UNSIGNED_SHORT_4_4_4_4 = 8;
        /** UNSIGNED_SHORT_5_5_5_1 */
        static readonly TEXTURETYPE_UNSIGNED_SHORT_5_5_5_1 = 9;
        /** UNSIGNED_SHORT_5_6_5 */
        static readonly TEXTURETYPE_UNSIGNED_SHORT_5_6_5 = 10;
        /** UNSIGNED_INT_2_10_10_10_REV */
        static readonly TEXTURETYPE_UNSIGNED_INT_2_10_10_10_REV = 11;
        /** UNSIGNED_INT_24_8 */
        static readonly TEXTURETYPE_UNSIGNED_INT_24_8 = 12;
        /** UNSIGNED_INT_10F_11F_11F_REV */
        static readonly TEXTURETYPE_UNSIGNED_INT_10F_11F_11F_REV = 13;
        /** UNSIGNED_INT_5_9_9_9_REV */
        static readonly TEXTURETYPE_UNSIGNED_INT_5_9_9_9_REV = 14;
        /** FLOAT_32_UNSIGNED_INT_24_8_REV */
        static readonly TEXTURETYPE_FLOAT_32_UNSIGNED_INT_24_8_REV = 15;
        /** nearest is mag = nearest and min = nearest and mip = none */
        static readonly TEXTURE_NEAREST_SAMPLINGMODE = 1;
        /** Bilinear is mag = linear and min = linear and mip = nearest */
        static readonly TEXTURE_BILINEAR_SAMPLINGMODE = 2;
        /** Trilinear is mag = linear and min = linear and mip = linear */
        static readonly TEXTURE_TRILINEAR_SAMPLINGMODE = 3;
        /** nearest is mag = nearest and min = nearest and mip = linear */
        static readonly TEXTURE_NEAREST_NEAREST_MIPLINEAR = 8;
        /** Bilinear is mag = linear and min = linear and mip = nearest */
        static readonly TEXTURE_LINEAR_LINEAR_MIPNEAREST = 11;
        /** Trilinear is mag = linear and min = linear and mip = linear */
        static readonly TEXTURE_LINEAR_LINEAR_MIPLINEAR = 3;
        /** mag = nearest and min = nearest and mip = nearest */
        static readonly TEXTURE_NEAREST_NEAREST_MIPNEAREST = 4;
        /** mag = nearest and min = linear and mip = nearest */
        static readonly TEXTURE_NEAREST_LINEAR_MIPNEAREST = 5;
        /** mag = nearest and min = linear and mip = linear */
        static readonly TEXTURE_NEAREST_LINEAR_MIPLINEAR = 6;
        /** mag = nearest and min = linear and mip = none */
        static readonly TEXTURE_NEAREST_LINEAR = 7;
        /** mag = nearest and min = nearest and mip = none */
        static readonly TEXTURE_NEAREST_NEAREST = 1;
        /** mag = linear and min = nearest and mip = nearest */
        static readonly TEXTURE_LINEAR_NEAREST_MIPNEAREST = 9;
        /** mag = linear and min = nearest and mip = linear */
        static readonly TEXTURE_LINEAR_NEAREST_MIPLINEAR = 10;
        /** mag = linear and min = linear and mip = none */
        static readonly TEXTURE_LINEAR_LINEAR = 2;
        /** mag = linear and min = nearest and mip = none */
        static readonly TEXTURE_LINEAR_NEAREST = 12;
        /** Explicit coordinates mode */
        static readonly TEXTURE_EXPLICIT_MODE = 0;
        /** Spherical coordinates mode */
        static readonly TEXTURE_SPHERICAL_MODE = 1;
        /** Planar coordinates mode */
        static readonly TEXTURE_PLANAR_MODE = 2;
        /** Cubic coordinates mode */
        static readonly TEXTURE_CUBIC_MODE = 3;
        /** Projection coordinates mode */
        static readonly TEXTURE_PROJECTION_MODE = 4;
        /** Skybox coordinates mode */
        static readonly TEXTURE_SKYBOX_MODE = 5;
        /** Inverse Cubic coordinates mode */
        static readonly TEXTURE_INVCUBIC_MODE = 6;
        /** Equirectangular coordinates mode */
        static readonly TEXTURE_EQUIRECTANGULAR_MODE = 7;
        /** Equirectangular Fixed coordinates mode */
        static readonly TEXTURE_FIXED_EQUIRECTANGULAR_MODE = 8;
        /** Equirectangular Fixed Mirrored coordinates mode */
        static readonly TEXTURE_FIXED_EQUIRECTANGULAR_MIRRORED_MODE = 9;
        /** Defines that texture rescaling will use a floor to find the closer power of 2 size */
        static readonly SCALEMODE_FLOOR = 1;
        /** Defines that texture rescaling will look for the nearest power of 2 size */
        static readonly SCALEMODE_NEAREST = 2;
        /** Defines that texture rescaling will use a ceil to find the closer power of 2 size */
        static readonly SCALEMODE_CEILING = 3;
        /**
         * Returns the current npm package of the sdk
         */
        static get NpmPackage(): string;
        /**
         * Returns the current version of the framework
         */
        static get Version(): string;
        /** Gets the list of created engines */
        static get Instances(): Engine[];
        /**
         * Gets the latest created engine
         */
        static get LastCreatedEngine(): Nullable<Engine>;
        /**
         * Gets the latest created scene
         */
        static get LastCreatedScene(): Nullable<Scene>;
        /** @internal */
        /**
         * Engine abstraction for loading and creating an image bitmap from a given source string.
         * @param imageSource source to load the image from.
         * @param options An object that sets options for the image's extraction.
         * @returns ImageBitmap.
         */
        _createImageBitmapFromSource(imageSource: string, options?: ImageBitmapOptions): Promise<ImageBitmap>;
        /**
         * Engine abstraction for createImageBitmap
         * @param image source for image
         * @param options An object that sets options for the image's extraction.
         * @returns ImageBitmap
         */
        createImageBitmap(image: ImageBitmapSource, options?: ImageBitmapOptions): Promise<ImageBitmap>;
        /**
         * Resize an image and returns the image data as an uint8array
         * @param image image to resize
         * @param bufferWidth destination buffer width
         * @param bufferHeight destination buffer height
         * @returns an uint8array containing RGBA values of bufferWidth * bufferHeight size
         */
        resizeImageBitmap(image: HTMLImageElement | ImageBitmap, bufferWidth: number, bufferHeight: number): Uint8Array;
        /**
         * Will flag all materials in all scenes in all engines as dirty to trigger new shader compilation
         * @param flag defines which part of the materials must be marked as dirty
         * @param predicate defines a predicate used to filter which materials should be affected
         */
        static MarkAllMaterialsAsDirty(flag: number, predicate?: (mat: Material) => boolean): void;
        /**
         * Method called to create the default loading screen.
         * This can be overridden in your own app.
         * @param canvas The rendering canvas element
         * @returns The loading screen
         */
        static DefaultLoadingScreenFactory(canvas: HTMLCanvasElement): ILoadingScreen;
        /**
         * Method called to create the default rescale post process on each engine.
         */
        static _RescalePostProcessFactory: Nullable<(engine: Engine) => PostProcess>;
        /**
         * Gets or sets a boolean to enable/disable IndexedDB support and avoid XHR on .manifest
         **/
        enableOfflineSupport: boolean;
        /**
         * Gets or sets a boolean to enable/disable checking manifest if IndexedDB support is enabled (js will always consider the database is up to date)
         **/
        disableManifestCheck: boolean;
        /**
         * Gets or sets a boolean to enable/disable the context menu (right-click) from appearing on the main canvas
         */
        disableContextMenu: boolean;
        /**
         * Gets the list of created scenes
         */
        scenes: Scene[];
        /** @internal */
        _virtualScenes: Scene[];
        /**
         * Event raised when a new scene is created
         */
        onNewSceneAddedObservable: Observable<Scene>;
        /**
         * Gets the list of created postprocesses
         */
        postProcesses: PostProcess[];
        /**
         * Gets a boolean indicating if the pointer is currently locked
         */
        isPointerLock: boolean;
        /**
         * Observable event triggered each time the rendering canvas is resized
         */
        onResizeObservable: Observable<Engine>;
        /**
         * Observable event triggered each time the canvas loses focus
         */
        onCanvasBlurObservable: Observable<Engine>;
        /**
         * Observable event triggered each time the canvas gains focus
         */
        onCanvasFocusObservable: Observable<Engine>;
        /**
         * Observable event triggered each time the canvas receives pointerout event
         */
        onCanvasPointerOutObservable: Observable<PointerEvent>;
        /**
         * Observable raised when the engine begins a new frame
         */
        onBeginFrameObservable: Observable<Engine>;
        /**
         * If set, will be used to request the next animation frame for the render loop
         */
        customAnimationFrameRequester: Nullable<ICustomAnimationFrameRequester>;
        /**
         * Observable raised when the engine ends the current frame
         */
        onEndFrameObservable: Observable<Engine>;
        /**
         * Observable raised when the engine is about to compile a shader
         */
        onBeforeShaderCompilationObservable: Observable<Engine>;
        /**
         * Observable raised when the engine has just compiled a shader
         */
        onAfterShaderCompilationObservable: Observable<Engine>;
        /**
         * Gets the audio engine
         * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic
         * @ignorenaming
         */
        static audioEngine: Nullable<IAudioEngine>;
        /**
         * Default AudioEngine factory responsible of creating the Audio Engine.
         * By default, this will create a BabylonJS Audio Engine if the workload has been embedded.
         */
        static AudioEngineFactory: (hostElement: Nullable<HTMLElement>, audioContext: Nullable<AudioContext>, audioDestination: Nullable<AudioDestinationNode | MediaStreamAudioDestinationNode>) => IAudioEngine;
        /**
         * Default offline support factory responsible of creating a tool used to store data locally.
         * By default, this will create a Database object if the workload has been embedded.
         */
        static OfflineProviderFactory: (urlToScene: string, callbackManifestChecked: (checked: boolean) => any, disableManifestCheck: boolean) => IOfflineProvider;
        private _loadingScreen;
        private _pointerLockRequested;
        private _rescalePostProcess;
        protected _deterministicLockstep: boolean;
        protected _lockstepMaxSteps: number;
        protected _timeStep: number;
        protected get _supportsHardwareTextureRescaling(): boolean;
        private _fps;
        private _deltaTime;
        /** @internal */
        _drawCalls: PerfCounter;
        /** Gets or sets the tab index to set to the rendering canvas. 1 is the minimum value to set to be able to capture keyboard events */
        canvasTabIndex: number;
        /**
         * Turn this value on if you want to pause FPS computation when in background
         */
        disablePerformanceMonitorInBackground: boolean;
        private _performanceMonitor;
        /**
         * Gets the performance monitor attached to this engine
         * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/optimize_your_scene#engineinstrumentation
         */
        get performanceMonitor(): PerformanceMonitor;
        private _onFocus;
        private _onBlur;
        private _onCanvasPointerOut;
        private _onCanvasBlur;
        private _onCanvasFocus;
        private _onCanvasContextMenu;
        private _onFullscreenChange;
        private _onPointerLockChange;
        protected _compatibilityMode: boolean;
        /**
         * (WebGPU only) True (default) to be in compatibility mode, meaning rendering all existing scenes without artifacts (same rendering than WebGL).
         * Setting the property to false will improve performances but may not work in some scenes if some precautions are not taken.
         * See https://doc.babylonjs.com/setup/support/webGPU/webGPUOptimization/webGPUNonCompatibilityMode for more details
         */
        get compatibilityMode(): boolean;
        set compatibilityMode(mode: boolean);
        /**
         * Gets the HTML element used to attach event listeners
         * @returns a HTML element
         */
        getInputElement(): Nullable<HTMLElement>;
        /**
         * Creates a new engine
         * @param canvasOrContext defines the canvas or WebGL context to use for rendering. If you provide a WebGL context, Babylon.js will not hook events on the canvas (like pointers, keyboards, etc...) so no event observables will be available. This is mostly used when Babylon.js is used as a plugin on a system which already used the WebGL context
         * @param antialias defines enable antialiasing (default: false)
         * @param options defines further options to be sent to the getContext() function
         * @param adaptToDeviceRatio defines whether to adapt to the device's viewport characteristics (default: false)
         */
        constructor(canvasOrContext: Nullable<HTMLCanvasElement | OffscreenCanvas | WebGLRenderingContext | WebGL2RenderingContext>, antialias?: boolean, options?: EngineOptions, adaptToDeviceRatio?: boolean);
        protected _initGLContext(): void;
        /**
         * Shared initialization across engines types.
         * @param canvas The canvas associated with this instance of the engine.
         * @param doNotHandleTouchAction Defines that engine should ignore modifying touch action attribute and style
         * @param audioEngine Defines if an audio engine should be created by default
         */
        protected _sharedInit(canvas: HTMLCanvasElement, doNotHandleTouchAction: boolean, audioEngine: boolean): void;
        /**
         * Gets current aspect ratio
         * @param viewportOwner defines the camera to use to get the aspect ratio
         * @param useScreen defines if screen size must be used (or the current render target if any)
         * @returns a number defining the aspect ratio
         */
        getAspectRatio(viewportOwner: IViewportOwnerLike, useScreen?: boolean): number;
        /**
         * Gets current screen aspect ratio
         * @returns a number defining the aspect ratio
         */
        getScreenAspectRatio(): number;
        /**
         * Gets the client rect of the HTML canvas attached with the current webGL context
         * @returns a client rectangle
         */
        getRenderingCanvasClientRect(): Nullable<ClientRect>;
        /**
         * Gets the client rect of the HTML element used for events
         * @returns a client rectangle
         */
        getInputElementClientRect(): Nullable<ClientRect>;
        /**
         * Gets a boolean indicating that the engine is running in deterministic lock step mode
         * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/advanced_animations#deterministic-lockstep
         * @returns true if engine is in deterministic lock step mode
         */
        isDeterministicLockStep(): boolean;
        /**
         * Gets the max steps when engine is running in deterministic lock step
         * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/advanced_animations#deterministic-lockstep
         * @returns the max steps
         */
        getLockstepMaxSteps(): number;
        /**
         * Returns the time in ms between steps when using deterministic lock step.
         * @returns time step in (ms)
         */
        getTimeStep(): number;
        /**
         * Force the mipmap generation for the given render target texture
         * @param texture defines the render target texture to use
         * @param unbind defines whether or not to unbind the texture after generation. Defaults to true.
         */
        generateMipMapsForCubemap(texture: InternalTexture, unbind?: boolean): void;
        /** States */
        /**
         * Gets a boolean indicating if depth writing is enabled
         * @returns the current depth writing state
         */
        getDepthWrite(): boolean;
        /**
         * Enable or disable depth writing
         * @param enable defines the state to set
         */
        setDepthWrite(enable: boolean): void;
        /**
         * Gets a boolean indicating if stencil buffer is enabled
         * @returns the current stencil buffer state
         */
        getStencilBuffer(): boolean;
        /**
         * Enable or disable the stencil buffer
         * @param enable defines if the stencil buffer must be enabled or disabled
         */
        setStencilBuffer(enable: boolean): void;
        /**
         * Gets the current stencil mask
         * @returns a number defining the new stencil mask to use
         */
        getStencilMask(): number;
        /**
         * Sets the current stencil mask
         * @param mask defines the new stencil mask to use
         */
        setStencilMask(mask: number): void;
        /**
         * Gets the current stencil function
         * @returns a number defining the stencil function to use
         */
        getStencilFunction(): number;
        /**
         * Gets the current stencil reference value
         * @returns a number defining the stencil reference value to use
         */
        getStencilFunctionReference(): number;
        /**
         * Gets the current stencil mask
         * @returns a number defining the stencil mask to use
         */
        getStencilFunctionMask(): number;
        /**
         * Sets the current stencil function
         * @param stencilFunc defines the new stencil function to use
         */
        setStencilFunction(stencilFunc: number): void;
        /**
         * Sets the current stencil reference
         * @param reference defines the new stencil reference to use
         */
        setStencilFunctionReference(reference: number): void;
        /**
         * Sets the current stencil mask
         * @param mask defines the new stencil mask to use
         */
        setStencilFunctionMask(mask: number): void;
        /**
         * Gets the current stencil operation when stencil fails
         * @returns a number defining stencil operation to use when stencil fails
         */
        getStencilOperationFail(): number;
        /**
         * Gets the current stencil operation when depth fails
         * @returns a number defining stencil operation to use when depth fails
         */
        getStencilOperationDepthFail(): number;
        /**
         * Gets the current stencil operation when stencil passes
         * @returns a number defining stencil operation to use when stencil passes
         */
        getStencilOperationPass(): number;
        /**
         * Sets the stencil operation to use when stencil fails
         * @param operation defines the stencil operation to use when stencil fails
         */
        setStencilOperationFail(operation: number): void;
        /**
         * Sets the stencil operation to use when depth fails
         * @param operation defines the stencil operation to use when depth fails
         */
        setStencilOperationDepthFail(operation: number): void;
        /**
         * Sets the stencil operation to use when stencil passes
         * @param operation defines the stencil operation to use when stencil passes
         */
        setStencilOperationPass(operation: number): void;
        /**
         * Sets a boolean indicating if the dithering state is enabled or disabled
         * @param value defines the dithering state
         */
        setDitheringState(value: boolean): void;
        /**
         * Sets a boolean indicating if the rasterizer state is enabled or disabled
         * @param value defines the rasterizer state
         */
        setRasterizerState(value: boolean): void;
        /**
         * Gets the current depth function
         * @returns a number defining the depth function
         */
        getDepthFunction(): Nullable<number>;
        /**
         * Sets the current depth function
         * @param depthFunc defines the function to use
         */
        setDepthFunction(depthFunc: number): void;
        /**
         * Sets the current depth function to GREATER
         */
        setDepthFunctionToGreater(): void;
        /**
         * Sets the current depth function to GEQUAL
         */
        setDepthFunctionToGreaterOrEqual(): void;
        /**
         * Sets the current depth function to LESS
         */
        setDepthFunctionToLess(): void;
        /**
         * Sets the current depth function to LEQUAL
         */
        setDepthFunctionToLessOrEqual(): void;
        private _cachedStencilBuffer;
        private _cachedStencilFunction;
        private _cachedStencilMask;
        private _cachedStencilOperationPass;
        private _cachedStencilOperationFail;
        private _cachedStencilOperationDepthFail;
        private _cachedStencilReference;
        /**
         * Caches the the state of the stencil buffer
         */
        cacheStencilState(): void;
        /**
         * Restores the state of the stencil buffer
         */
        restoreStencilState(): void;
        /**
         * Directly set the WebGL Viewport
         * @param x defines the x coordinate of the viewport (in screen space)
         * @param y defines the y coordinate of the viewport (in screen space)
         * @param width defines the width of the viewport (in screen space)
         * @param height defines the height of the viewport (in screen space)
         * @returns the current viewport Object (if any) that is being replaced by this call. You can restore this viewport later on to go back to the original state
         */
        setDirectViewport(x: number, y: number, width: number, height: number): Nullable<IViewportLike>;
        /**
         * Executes a scissor clear (ie. a clear on a specific portion of the screen)
         * @param x defines the x-coordinate of the bottom left corner of the clear rectangle
         * @param y defines the y-coordinate of the corner of the clear rectangle
         * @param width defines the width of the clear rectangle
         * @param height defines the height of the clear rectangle
         * @param clearColor defines the clear color
         */
        scissorClear(x: number, y: number, width: number, height: number, clearColor: IColor4Like): void;
        /**
         * Enable scissor test on a specific rectangle (ie. render will only be executed on a specific portion of the screen)
         * @param x defines the x-coordinate of the bottom left corner of the clear rectangle
         * @param y defines the y-coordinate of the corner of the clear rectangle
         * @param width defines the width of the clear rectangle
         * @param height defines the height of the clear rectangle
         */
        enableScissor(x: number, y: number, width: number, height: number): void;
        /**
         * Disable previously set scissor test rectangle
         */
        disableScissor(): void;
        /**
         * @internal
         */
        _reportDrawCall(numDrawCalls?: number): void;
        /**
         * Initializes a webVR display and starts listening to display change events
         * The onVRDisplayChangedObservable will be notified upon these changes
         * @returns The onVRDisplayChangedObservable
         */
        initWebVR(): Observable<IDisplayChangedEventArgs>;
        /** @internal */
        _prepareVRComponent(): void;
        /**
         * @internal
         */
        _connectVREvents(canvas?: HTMLCanvasElement, document?: any): void;
        /** @internal */
        _submitVRFrame(): void;
        /**
         * Call this function to leave webVR mode
         * Will do nothing if webVR is not supported or if there is no webVR device
         * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/webVRCamera
         */
        disableVR(): void;
        /**
         * Gets a boolean indicating that the system is in VR mode and is presenting
         * @returns true if VR mode is engaged
         */
        isVRPresenting(): boolean;
        /** @internal */
        _requestVRFrame(): void;
        /**
         * @internal
         */
        _loadFileAsync(url: string, offlineProvider?: IOfflineProvider, useArrayBuffer?: boolean): Promise<string | ArrayBuffer>;
        /**
         * Gets the source code of the vertex shader associated with a specific webGL program
         * @param program defines the program to use
         * @returns a string containing the source code of the vertex shader associated with the program
         */
        getVertexShaderSource(program: WebGLProgram): Nullable<string>;
        /**
         * Gets the source code of the fragment shader associated with a specific webGL program
         * @param program defines the program to use
         * @returns a string containing the source code of the fragment shader associated with the program
         */
        getFragmentShaderSource(program: WebGLProgram): Nullable<string>;
        /**
         * Sets a depth stencil texture from a render target to the according uniform.
         * @param channel The texture channel
         * @param uniform The uniform to set
         * @param texture The render target texture containing the depth stencil texture to apply
         * @param name The texture name
         */
        setDepthStencilTexture(channel: number, uniform: Nullable<WebGLUniformLocation>, texture: Nullable<RenderTargetTexture>, name?: string): void;
        /**
         * Sets a texture to the webGL context from a postprocess
         * @param channel defines the channel to use
         * @param postProcess defines the source postprocess
         * @param name name of the channel
         */
        setTextureFromPostProcess(channel: number, postProcess: Nullable<PostProcess>, name: string): void;
        /**
         * Binds the output of the passed in post process to the texture channel specified
         * @param channel The channel the texture should be bound to
         * @param postProcess The post process which's output should be bound
         * @param name name of the channel
         */
        setTextureFromPostProcessOutput(channel: number, postProcess: Nullable<PostProcess>, name: string): void;
        protected _rebuildBuffers(): void;
        /** @internal */
        _renderFrame(): void;
        _renderLoop(): void;
        /** @internal */
        _renderViews(): boolean;
        /**
         * Toggle full screen mode
         * @param requestPointerLock defines if a pointer lock should be requested from the user
         */
        switchFullscreen(requestPointerLock: boolean): void;
        /**
         * Enters full screen mode
         * @param requestPointerLock defines if a pointer lock should be requested from the user
         */
        enterFullscreen(requestPointerLock: boolean): void;
        /**
         * Exits full screen mode
         */
        exitFullscreen(): void;
        /**
         * Enters Pointerlock mode
         */
        enterPointerlock(): void;
        /**
         * Exits Pointerlock mode
         */
        exitPointerlock(): void;
        /**
         * Begin a new frame
         */
        beginFrame(): void;
        /**
         * End the current frame
         */
        endFrame(): void;
        /**
         * Resize the view according to the canvas' size
         * @param forceSetSize true to force setting the sizes of the underlying canvas
         */
        resize(forceSetSize?: boolean): void;
        /**
         * Force a specific size of the canvas
         * @param width defines the new canvas' width
         * @param height defines the new canvas' height
         * @param forceSetSize true to force setting the sizes of the underlying canvas
         * @returns true if the size was changed
         */
        setSize(width: number, height: number, forceSetSize?: boolean): boolean;
        _deletePipelineContext(pipelineContext: IPipelineContext): void;
        createShaderProgram(pipelineContext: IPipelineContext, vertexCode: string, fragmentCode: string, defines: Nullable<string>, context?: WebGLRenderingContext, transformFeedbackVaryings?: Nullable<string[]>): WebGLProgram;
        protected _createShaderProgram(pipelineContext: WebGLPipelineContext, vertexShader: WebGLShader, fragmentShader: WebGLShader, context: WebGLRenderingContext, transformFeedbackVaryings?: Nullable<string[]>): WebGLProgram;
        /**
         * @internal
         */
        _releaseTexture(texture: InternalTexture): void;
        /**
         * @internal
         */
        _releaseRenderTargetWrapper(rtWrapper: RenderTargetWrapper): void;
        protected static _RenderPassIdCounter: number;
        /**
         * Gets or sets the current render pass id
         */
        currentRenderPassId: number;
        private _renderPassNames;
        /**
         * Gets the names of the render passes that are currently created
         * @returns list of the render pass names
         */
        getRenderPassNames(): string[];
        /**
         * Gets the name of the current render pass
         * @returns name of the current render pass
         */
        getCurrentRenderPassName(): string;
        /**
         * Creates a render pass id
         * @param name Name of the render pass (for debug purpose only)
         * @returns the id of the new render pass
         */
        createRenderPassId(name?: string): number;
        /**
         * Releases a render pass id
         * @param id id of the render pass to release
         */
        releaseRenderPassId(id: number): void;
        /**
         * @internal
         * Rescales a texture
         * @param source input texture
         * @param destination destination texture
         * @param scene scene to use to render the resize
         * @param internalFormat format to use when resizing
         * @param onComplete callback to be called when resize has completed
         */
        _rescaleTexture(source: InternalTexture, destination: InternalTexture, scene: Nullable<any>, internalFormat: number, onComplete: () => void): void;
        /**
         * Gets the current framerate
         * @returns a number representing the framerate
         */
        getFps(): number;
        /**
         * Gets the time spent between current and previous frame
         * @returns a number representing the delta time in ms
         */
        getDeltaTime(): number;
        private _measureFps;
        /**
         * Wraps an external web gl texture in a Babylon texture.
         * @param texture defines the external texture
         * @returns the babylon internal texture
         */
        wrapWebGLTexture(texture: WebGLTexture): InternalTexture;
        /**
         * @internal
         */
        _uploadImageToTexture(texture: InternalTexture, image: HTMLImageElement | ImageBitmap, faceIndex?: number, lod?: number): void;
        /**
         * Updates a depth texture Comparison Mode and Function.
         * If the comparison Function is equal to 0, the mode will be set to none.
         * Otherwise, this only works in webgl 2 and requires a shadow sampler in the shader.
         * @param texture The texture to set the comparison function for
         * @param comparisonFunction The comparison function to set, 0 if no comparison required
         */
        updateTextureComparisonFunction(texture: InternalTexture, comparisonFunction: number): void;
        /**
         * Creates a webGL buffer to use with instantiation
         * @param capacity defines the size of the buffer
         * @returns the webGL buffer
         */
        createInstancesBuffer(capacity: number): DataBuffer;
        /**
         * Delete a webGL buffer used with instantiation
         * @param buffer defines the webGL buffer to delete
         */
        deleteInstancesBuffer(buffer: WebGLBuffer): void;
        private _clientWaitAsync;
        /**
         * @internal
         */
        _readPixelsAsync(x: number, y: number, w: number, h: number, format: number, type: number, outputBuffer: ArrayBufferView): Promise<ArrayBufferView> | null;
        dispose(): void;
        private _disableTouchAction;
        /**
         * Display the loading screen
         * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/customLoadingScreen
         */
        displayLoadingUI(): void;
        /**
         * Hide the loading screen
         * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/customLoadingScreen
         */
        hideLoadingUI(): void;
        /**
         * Gets the current loading screen object
         * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/customLoadingScreen
         */
        get loadingScreen(): ILoadingScreen;
        /**
         * Sets the current loading screen object
         * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/customLoadingScreen
         */
        set loadingScreen(loadingScreen: ILoadingScreen);
        /**
         * Sets the current loading screen text
         * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/customLoadingScreen
         */
        set loadingUIText(text: string);
        /**
         * Sets the current loading screen background color
         * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/customLoadingScreen
         */
        set loadingUIBackgroundColor(color: string);
        /**
         * creates and returns a new video element
         * @param constraints video constraints
         * @returns video element
         */
        createVideoElement(constraints: MediaTrackConstraints): any;
        /** Pointerlock and fullscreen */
        /**
         * Ask the browser to promote the current element to pointerlock mode
         * @param element defines the DOM element to promote
         */
        static _RequestPointerlock(element: HTMLElement): void;
        /**
         * Asks the browser to exit pointerlock mode
         */
        static _ExitPointerlock(): void;
        /**
         * Ask the browser to promote the current element to fullscreen rendering mode
         * @param element defines the DOM element to promote
         */
        static _RequestFullscreen(element: HTMLElement): void;
        /**
         * Asks the browser to exit fullscreen mode
         */
        static _ExitFullscreen(): void;
        /**
         * Get Font size information
         * @param font font name
         * @returns an object containing ascent, height and descent
         */
        getFontOffset(font: string): {
            ascent: number;
            height: number;
            descent: number;
        };
    }


    /**
     * Interface used to describe the capabilities of the engine relatively to the current browser
     */
    export interface EngineCapabilities {
        /** Maximum textures units per fragment shader */
        maxTexturesImageUnits: number;
        /** Maximum texture units per vertex shader */
        maxVertexTextureImageUnits: number;
        /** Maximum textures units in the entire pipeline */
        maxCombinedTexturesImageUnits: number;
        /** Maximum texture size */
        maxTextureSize: number;
        /** Maximum texture samples */
        maxSamples?: number;
        /** Maximum cube texture size */
        maxCubemapTextureSize: number;
        /** Maximum render texture size */
        maxRenderTextureSize: number;
        /** Maximum number of vertex attributes */
        maxVertexAttribs: number;
        /** Maximum number of varyings */
        maxVaryingVectors: number;
        /** Maximum number of uniforms per vertex shader */
        maxVertexUniformVectors: number;
        /** Maximum number of uniforms per fragment shader */
        maxFragmentUniformVectors: number;
        /** Defines if standard derivatives (dx/dy) are supported */
        standardDerivatives: boolean;
        /** Defines if s3tc texture compression is supported */
        s3tc?: WEBGL_compressed_texture_s3tc;
        /** Defines if s3tc sRGB texture compression is supported */
        s3tc_srgb?: WEBGL_compressed_texture_s3tc_srgb;
        /** Defines if pvrtc texture compression is supported */
        pvrtc: any;
        /** Defines if etc1 texture compression is supported */
        etc1: any;
        /** Defines if etc2 texture compression is supported */
        etc2: any;
        /** Defines if astc texture compression is supported */
        astc: any;
        /** Defines if bptc texture compression is supported */
        bptc: any;
        /** Defines if float textures are supported */
        textureFloat: boolean;
        /** Defines if vertex array objects are supported */
        vertexArrayObject: boolean;
        /** Gets the webgl extension for anisotropic filtering (null if not supported) */
        textureAnisotropicFilterExtension?: EXT_texture_filter_anisotropic;
        /** Gets the maximum level of anisotropy supported */
        maxAnisotropy: number;
        /** Defines if instancing is supported */
        instancedArrays: boolean;
        /** Defines if 32 bits indices are supported */
        uintIndices: boolean;
        /** Defines if high precision shaders are supported */
        highPrecisionShaderSupported: boolean;
        /** Defines if depth reading in the fragment shader is supported */
        fragmentDepthSupported: boolean;
        /** Defines if float texture linear filtering is supported*/
        textureFloatLinearFiltering: boolean;
        /** Defines if rendering to float textures is supported */
        textureFloatRender: boolean;
        /** Defines if half float textures are supported*/
        textureHalfFloat: boolean;
        /** Defines if half float texture linear filtering is supported*/
        textureHalfFloatLinearFiltering: boolean;
        /** Defines if rendering to half float textures is supported */
        textureHalfFloatRender: boolean;
        /** Defines if textureLOD shader command is supported */
        textureLOD: boolean;
        /** Defines if draw buffers extension is supported */
        drawBuffersExtension: boolean;
        /** Defines if depth textures are supported */
        depthTextureExtension: boolean;
        /** Defines if float color buffer are supported */
        colorBufferFloat: boolean;
        /** Gets disjoint timer query extension (null if not supported) */
        timerQuery?: EXT_disjoint_timer_query;
        /** Defines if timestamp can be used with timer query */
        canUseTimestampForTimerQuery: boolean;
        /** Defines if occlusion queries are supported by the engine */
        supportOcclusionQuery: boolean;
        /** Defines if multiview is supported (https://www.khronos.org/registry/webgl/extensions/WEBGL_multiview/) */
        multiview?: any;
        /** Defines if oculus multiview is supported (https://developer.oculus.com/documentation/oculus-browser/latest/concepts/browser-multiview/) */
        oculusMultiview?: any;
        /** Function used to let the system compiles shaders in background */
        parallelShaderCompile?: {
            COMPLETION_STATUS_KHR: number;
        };
        /** Max number of texture samples for MSAA */
        maxMSAASamples: number;
        /** Defines if the blend min max extension is supported */
        blendMinMax: boolean;
        /** In some iOS + WebGL1, gl_InstanceID (and gl_InstanceIDEXT) is undefined even if instancedArrays is true. So don't use gl_InstanceID in those cases */
        canUseGLInstanceID: boolean;
        /** Defines if gl_vertexID is available */
        canUseGLVertexID: boolean;
        /** Defines if compute shaders are supported by the engine */
        supportComputeShaders: boolean;
        /** Defines if sRGB texture formats are supported */
        supportSRGBBuffers: boolean;
        /** Defines if transform feedbacks are supported */
        supportTransformFeedbacks: boolean;
        /** Defines if texture max level are supported */
        textureMaxLevel: boolean;
        /** Defines the maximum layer count for a 2D Texture array. */
        texture2DArrayMaxLayerCount: number;
    }


    /**
     * Helper class to create the best engine depending on the current hardware
     */
    export class EngineFactory {
        /**
         * Creates an engine based on the capabilities of the underlying hardware
         * @param canvas Defines the canvas to use to display the result
         * @param options Defines the options passed to the engine to create the context dependencies
         * @returns a promise that resolves with the created engine
         */
        static CreateAsync(canvas: HTMLCanvasElement, options: any): Promise<Engine>;
    }


    /** @internal */
    export interface EngineFeatures {
        /** Force using Bitmap when Bitmap or HTMLImageElement can be used */
        forceBitmapOverHTMLImageElement: boolean;
        /** Indicates that the engine support rendering to as well as copying to lod float textures */
        supportRenderAndCopyToLodForFloatTextures: boolean;
        /** Indicates that the engine support handling depth/stencil textures */
        supportDepthStencilTexture: boolean;
        /** Indicates that the engine support shadow samplers */
        supportShadowSamplers: boolean;
        /** Indicates to check the matrix bytes per bytes to know if it has changed or not. If false, only the updateFlag of the matrix is checked */
        uniformBufferHardCheckMatrix: boolean;
        /** Indicates that prefiltered mipmaps can be generated in some processes (for eg when loading an HDR cube texture) */
        allowTexturePrefiltering: boolean;
        /** Indicates to track the usage of ubos and to create new ones as necessary during a frame duration */
        trackUbosInFrame: boolean;
        /** Indicates that the current content of a ubo should be compared to the content of the corresponding GPU buffer and the GPU buffer updated only if different. Requires trackUbosInFrame to be true */
        checkUbosContentBeforeUpload: boolean;
        /** Indicates that the Cascaded Shadow Map technic is supported */
        supportCSM: boolean;
        /** Indicates that the textures transcoded by the basis transcoder must have power of 2 width and height */
        basisNeedsPOT: boolean;
        /** Indicates that the engine supports 3D textures */
        support3DTextures: boolean;
        /** Indicates that constants need a type suffix in shaders (used by realtime filtering...) */
        needTypeSuffixInShaderConstants: boolean;
        /** Indicates that MSAA is supported */
        supportMSAA: boolean;
        /** Indicates that SSAO2 is supported */
        supportSSAO2: boolean;
        /** Indicates that some additional texture formats are supported (like TEXTUREFORMAT_R for eg) */
        supportExtendedTextureFormats: boolean;
        /** Indicates that the switch/case construct is supported in shaders */
        supportSwitchCaseInShader: boolean;
        /** Indicates that synchronous texture reading is supported */
        supportSyncTextureRead: boolean;
        /** Indicates that y should be inverted when dealing with bitmaps (notably in environment tools) */
        needsInvertingBitmap: boolean;
        /** Indicates that the engine should cache the bound UBO */
        useUBOBindingCache: boolean;
        /** Indicates that the inliner should be run over every shader code */
        needShaderCodeInlining: boolean;
        /** Indicates that even if we don't have to update the properties of a uniform buffer (because of some optimzations in the material) we still need to bind the uniform buffer themselves */
        needToAlwaysBindUniformBuffers: boolean;
        /**  Indicates that the engine supports render passes */
        supportRenderPasses: boolean;
        /**  Indicates that the engine supports sprite instancing */
        supportSpriteInstancing: boolean;
        /** @internal */
        _collectUbosUpdatedInFrame: boolean;
    }


    /**
     * The engine store class is responsible to hold all the instances of Engine and Scene created
     * during the life time of the application.
     */
    export class EngineStore {
        /** Gets the list of created engines */
        static Instances: Engine[];
        /** @internal */
        static _LastCreatedScene: Nullable<Scene>;
        /**
         * Gets the latest created engine
         */
        static get LastCreatedEngine(): Nullable<Engine>;
        /**
         * Gets the latest created scene
         */
        static get LastCreatedScene(): Nullable<Scene>;
        /**
         * Gets or sets a global variable indicating if fallback texture must be used when a texture cannot be loaded
         * @ignorenaming
         */
        static UseFallbackTexture: boolean;
        /**
         * Texture content used if a texture cannot loaded
         * @ignorenaming
         */
        static FallbackTexture: string;
    }


        interface ThinEngine {
            /**
             * Sets alpha constants used by some alpha blending modes
             * @param r defines the red component
             * @param g defines the green component
             * @param b defines the blue component
             * @param a defines the alpha component
             */
            setAlphaConstants(r: number, g: number, b: number, a: number): void;
            /**
             * Sets the current alpha mode
             * @param mode defines the mode to use (one of the Engine.ALPHA_XXX)
             * @param noDepthWriteChange defines if depth writing state should remains unchanged (false by default)
             * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/advanced/transparent_rendering
             */
            setAlphaMode(mode: number, noDepthWriteChange?: boolean): void;
            /**
             * Gets the current alpha mode
             * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/advanced/transparent_rendering
             * @returns the current alpha mode
             */
            getAlphaMode(): number;
            /**
             * Sets the current alpha equation
             * @param equation defines the equation to use (one of the Engine.ALPHA_EQUATION_XXX)
             */
            setAlphaEquation(equation: number): void;
            /**
             * Gets the current alpha equation.
             * @returns the current alpha equation
             */
            getAlphaEquation(): number;
        }


    /**
     * Type used to locate a resource in a compute shader.
     * TODO: remove this when browsers support reflection for wgsl shaders
     */
    export type ComputeBindingLocation = {
        group: number;
        binding: number;
    };
    /**
     * Type used to lookup a resource and retrieve its binding location
     * TODO: remove this when browsers support reflection for wgsl shaders
     */
    export type ComputeBindingMapping = {
        [key: string]: ComputeBindingLocation;
    };
    /** @internal */
    export enum ComputeBindingType {
        Texture = 0,
        StorageTexture = 1,
        UniformBuffer = 2,
        StorageBuffer = 3,
        TextureWithoutSampler = 4,
        Sampler = 5
    }
    /** @internal */
    export type ComputeBindingList = {
        [key: string]: {
            type: ComputeBindingType;
            object: any;
            indexInGroupEntries?: number;
        };
    };
        interface ThinEngine {
            /**
             * Creates a new compute effect
             * @param baseName Name of the effect
             * @param options Options used to create the effect
             * @returns The new compute effect
             */
            createComputeEffect(baseName: any, options: IComputeEffectCreationOptions): ComputeEffect;
            /**
             * Creates a new compute pipeline context
             * @returns the new pipeline
             */
            createComputePipelineContext(): IComputePipelineContext;
            /**
             * Creates a new compute context
             * @returns the new context
             */
            createComputeContext(): IComputeContext | undefined;
            /**
             * Dispatches a compute shader
             * @param effect The compute effect
             * @param context The compute context
             * @param bindings The list of resources to bind to the shader
             * @param x The number of workgroups to execute on the X dimension
             * @param y The number of workgroups to execute on the Y dimension
             * @param z The number of workgroups to execute on the Z dimension
             * @param bindingsMapping list of bindings mapping (key is property name, value is binding location)
             */
            computeDispatch(effect: ComputeEffect, context: IComputeContext, bindings: ComputeBindingList, x: number, y?: number, z?: number, bindingsMapping?: ComputeBindingMapping): void;
            /**
             * Gets a boolean indicating if all created compute effects are ready
             * @returns true if all effects are ready
             */
            areAllComputeEffectsReady(): boolean;
            /**
             * Forces the engine to release all cached compute effects. This means that next effect compilation will have to be done completely even if a similar effect was already compiled
             */
            releaseComputeEffects(): void;
            /** @internal */
            _prepareComputePipelineContext(pipelineContext: IComputePipelineContext, computeSourceCode: string, rawComputeSourceCode: string, defines: Nullable<string>, entryPoint: string): void;
            /** @internal */
            _rebuildComputeEffects(): void;
            /** @internal */
            _executeWhenComputeStateIsCompiled(pipelineContext: IComputePipelineContext, action: () => void): void;
            /** @internal */
            _releaseComputeEffect(effect: ComputeEffect): void;
            /** @internal */
            _deleteComputePipelineContext(pipelineContext: IComputePipelineContext): void;
        }


        interface ThinEngine {
            /**
             * Creates a depth stencil cube texture.
             * This is only available in WebGL 2.
             * @param size The size of face edge in the cube texture.
             * @param options The options defining the cube texture.
             * @param rtWrapper The render target wrapper for which the depth/stencil texture must be created
             * @returns The cube texture
             */
            _createDepthStencilCubeTexture(size: number, options: DepthTextureCreationOptions, rtWrapper: RenderTargetWrapper): InternalTexture;
            /**
             * Creates a cube texture
             * @param rootUrl defines the url where the files to load is located
             * @param scene defines the current scene
             * @param files defines the list of files to load (1 per face)
             * @param noMipmap defines a boolean indicating that no mipmaps shall be generated (false by default)
             * @param onLoad defines an optional callback raised when the texture is loaded
             * @param onError defines an optional callback raised if there is an issue to load the texture
             * @param format defines the format of the data
             * @param forcedExtension defines the extension to use to pick the right loader
             * @param createPolynomials if a polynomial sphere should be created for the cube texture
             * @param lodScale defines the scale applied to environment texture. This manages the range of LOD level used for IBL according to the roughness
             * @param lodOffset defines the offset applied to environment texture. This manages first LOD level used for IBL according to the roughness
             * @param fallback defines texture to use while falling back when (compressed) texture file not found.
             * @param loaderOptions options to be passed to the loader
             * @param useSRGBBuffer defines if the texture must be loaded in a sRGB GPU buffer (if supported by the GPU).
             * @returns the cube texture as an InternalTexture
             */
            createCubeTexture(rootUrl: string, scene: Nullable<Scene>, files: Nullable<string[]>, noMipmap: boolean | undefined, onLoad: Nullable<(data?: any) => void>, onError: Nullable<(message?: string, exception?: any) => void>, format: number | undefined, forcedExtension: any, createPolynomials: boolean, lodScale: number, lodOffset: number, fallback: Nullable<InternalTexture>, loaderOptions: any, useSRGBBuffer: boolean): InternalTexture;
            /**
             * Creates a cube texture
             * @param rootUrl defines the url where the files to load is located
             * @param scene defines the current scene
             * @param files defines the list of files to load (1 per face)
             * @param noMipmap defines a boolean indicating that no mipmaps shall be generated (false by default)
             * @param onLoad defines an optional callback raised when the texture is loaded
             * @param onError defines an optional callback raised if there is an issue to load the texture
             * @param format defines the format of the data
             * @param forcedExtension defines the extension to use to pick the right loader
             * @returns the cube texture as an InternalTexture
             */
            createCubeTexture(rootUrl: string, scene: Nullable<Scene>, files: Nullable<string[]>, noMipmap: boolean, onLoad: Nullable<(data?: any) => void>, onError: Nullable<(message?: string, exception?: any) => void>, format: number | undefined, forcedExtension: any): InternalTexture;
            /**
             * Creates a cube texture
             * @param rootUrl defines the url where the files to load is located
             * @param scene defines the current scene
             * @param files defines the list of files to load (1 per face)
             * @param noMipmap defines a boolean indicating that no mipmaps shall be generated (false by default)
             * @param onLoad defines an optional callback raised when the texture is loaded
             * @param onError defines an optional callback raised if there is an issue to load the texture
             * @param format defines the format of the data
             * @param forcedExtension defines the extension to use to pick the right loader
             * @param createPolynomials if a polynomial sphere should be created for the cube texture
             * @param lodScale defines the scale applied to environment texture. This manages the range of LOD level used for IBL according to the roughness
             * @param lodOffset defines the offset applied to environment texture. This manages first LOD level used for IBL according to the roughness
             * @returns the cube texture as an InternalTexture
             */
            createCubeTexture(rootUrl: string, scene: Nullable<Scene>, files: Nullable<string[]>, noMipmap: boolean, onLoad: Nullable<(data?: any) => void>, onError: Nullable<(message?: string, exception?: any) => void>, format: number | undefined, forcedExtension: any, createPolynomials: boolean, lodScale: number, lodOffset: number): InternalTexture;
            /** @internal */
            createCubeTextureBase(rootUrl: string, scene: Nullable<Scene>, files: Nullable<string[]>, noMipmap: boolean, onLoad: Nullable<(data?: any) => void>, onError: Nullable<(message?: string, exception?: any) => void>, format: number | undefined, forcedExtension: any, createPolynomials: boolean, lodScale: number, lodOffset: number, fallback: Nullable<InternalTexture>, beforeLoadCubeDataCallback: Nullable<(texture: InternalTexture, data: ArrayBufferView | ArrayBufferView[]) => void>, imageHandler: Nullable<(texture: InternalTexture, imgs: HTMLImageElement[] | ImageBitmap[]) => void>, useSRGBBuffer: boolean): InternalTexture;
            /** @internal */
            _partialLoadFile(url: string, index: number, loadedFiles: ArrayBuffer[], onfinish: (files: ArrayBuffer[]) => void, onErrorCallBack: Nullable<(message?: string, exception?: any) => void>): void;
            /** @internal */
            _cascadeLoadFiles(scene: Nullable<Scene>, onfinish: (images: ArrayBuffer[]) => void, files: string[], onError: Nullable<(message?: string, exception?: any) => void>): void;
            /** @internal */
            _cascadeLoadImgs(scene: Nullable<Scene>, texture: InternalTexture, onfinish: Nullable<(texture: InternalTexture, images: HTMLImageElement[] | ImageBitmap[]) => void>, files: string[], onError: Nullable<(message?: string, exception?: any) => void>, mimeType?: string): void;
            /** @internal */
            _partialLoadImg(url: string, index: number, loadedImages: HTMLImageElement[] | ImageBitmap[], scene: Nullable<Scene>, texture: InternalTexture, onfinish: Nullable<(texture: InternalTexture, images: HTMLImageElement[] | ImageBitmap[]) => void>, onErrorCallBack: Nullable<(message?: string, exception?: any) => void>, mimeType?: string): void;
            /**
             * @internal
             */
            _setCubeMapTextureParams(texture: InternalTexture, loadMipmap: boolean, maxLevel?: number): void;
        }


        interface ThinEngine {
            /** @internal */
            _debugPushGroup(groupName: string, targetObject?: number): void;
            /** @internal */
            _debugPopGroup(targetObject?: number): void;
            /** @internal */
            _debugInsertMarker(text: string, targetObject?: number): void;
            /** @internal */
            _debugFlushPendingCommands(): void;
        }


        interface ThinEngine {
            /**
             * Update a dynamic index buffer
             * @param indexBuffer defines the target index buffer
             * @param indices defines the data to update
             * @param offset defines the offset in the target index buffer where update should start
             */
            updateDynamicIndexBuffer(indexBuffer: DataBuffer, indices: IndicesArray, offset?: number): void;
            /**
             * Updates a dynamic vertex buffer.
             * @param vertexBuffer the vertex buffer to update
             * @param data the data used to update the vertex buffer
             * @param byteOffset the byte offset of the data
             * @param byteLength the byte length of the data
             */
            updateDynamicVertexBuffer(vertexBuffer: DataBuffer, data: DataArray, byteOffset?: number, byteLength?: number): void;
        }


        interface ThinEngine {
            /**
             * Creates a dynamic texture
             * @param width defines the width of the texture
             * @param height defines the height of the texture
             * @param generateMipMaps defines if the engine should generate the mip levels
             * @param samplingMode defines the required sampling mode (Texture.NEAREST_SAMPLINGMODE by default)
             * @returns the dynamic texture inside an InternalTexture
             */
            createDynamicTexture(width: number, height: number, generateMipMaps: boolean, samplingMode: number): InternalTexture;
            /**
             * Update the content of a dynamic texture
             * @param texture defines the texture to update
             * @param source defines the source containing the data
             * @param invertY defines if data must be stored with Y axis inverted
             * @param premulAlpha defines if alpha is stored as premultiplied
             * @param format defines the format of the data
             * @param forceBindTexture if the texture should be forced to be bound eg. after a graphics context loss (Default: false)
             * @param allowGPUOptimization true to allow some specific GPU optimizations (subject to engine feature "allowGPUOptimizationsForGUI" being true)
             */
            updateDynamicTexture(texture: Nullable<InternalTexture>, source: ImageBitmap | ImageData | HTMLImageElement | HTMLCanvasElement | HTMLVideoElement | OffscreenCanvas | ICanvas, invertY?: boolean, premulAlpha?: boolean, format?: number, forceBindTexture?: boolean, allowGPUOptimization?: boolean): void;
        }


        interface ThinEngine {
            /**
             * Creates an external texture
             * @param video video element
             * @returns the external texture, or null if external textures are not supported by the engine
             */
            createExternalTexture(video: HTMLVideoElement): Nullable<ExternalTexture>;
            /**
             * Sets an internal texture to the according uniform.
             * @param name The name of the uniform in the effect
             * @param texture The texture to apply
             */
            setExternalTexture(name: string, texture: Nullable<ExternalTexture>): void;
        }


        interface ThinEngine {
            /**
             * Unbind a list of render target textures from the webGL context
             * This is used only when drawBuffer extension or webGL2 are active
             * @param rtWrapper defines the render target wrapper to unbind
             * @param disableGenerateMipMaps defines a boolean indicating that mipmaps must not be generated
             * @param onBeforeUnbind defines a function which will be called before the effective unbind
             */
            unBindMultiColorAttachmentFramebuffer(rtWrapper: RenderTargetWrapper, disableGenerateMipMaps: boolean, onBeforeUnbind?: () => void): void;
            /**
             * Create a multi render target texture
             * @see https://doc.babylonjs.com/setup/support/webGL2#multiple-render-target
             * @param size defines the size of the texture
             * @param options defines the creation options
             * @param initializeBuffers if set to true, the engine will make an initializing call of drawBuffers
             * @returns a new render target wrapper ready to render textures
             */
            createMultipleRenderTarget(size: TextureSize, options: IMultiRenderTargetOptions, initializeBuffers?: boolean): RenderTargetWrapper;
            /**
             * Update the sample count for a given multiple render target texture
             * @see https://doc.babylonjs.com/setup/support/webGL2#multisample-render-targets
             * @param rtWrapper defines the render target wrapper to update
             * @param samples defines the sample count to set
             * @param initializeBuffers if set to true, the engine will make an initializing call of drawBuffers
             * @returns the effective sample count (could be 0 if multisample render targets are not supported)
             */
            updateMultipleRenderTargetTextureSampleCount(rtWrapper: Nullable<RenderTargetWrapper>, samples: number, initializeBuffers?: boolean): number;
            /**
             * Select a subsets of attachments to draw to.
             * @param attachments gl attachments
             */
            bindAttachments(attachments: number[]): void;
            /**
             * Creates a layout object to draw/clear on specific textures in a MRT
             * @param textureStatus textureStatus[i] indicates if the i-th is active
             * @returns A layout to be fed to the engine, calling `bindAttachments`.
             */
            buildTextureLayout(textureStatus: boolean[]): number[];
            /**
             * Restores the webgl state to only draw on the main color attachment
             * when the frame buffer associated is the canvas frame buffer
             */
            restoreSingleAttachment(): void;
            /**
             * Restores the webgl state to only draw on the main color attachment
             * when the frame buffer associated is not the canvas frame buffer
             */
            restoreSingleAttachmentForRenderTarget(): void;
        }


        interface Engine {
            /**
             * Creates a new multiview render target
             * @param width defines the width of the texture
             * @param height defines the height of the texture
             * @returns the created multiview render target wrapper
             */
            createMultiviewRenderTargetTexture(width: number, height: number): RenderTargetWrapper;
            /**
             * Binds a multiview render target wrapper to be drawn to
             * @param multiviewTexture render target wrapper to bind
             */
            bindMultiviewFramebuffer(multiviewTexture: RenderTargetWrapper): void;
        }
        interface Camera {
            /**
             * @internal
             * For cameras that cannot use multiview images to display directly. (e.g. webVR camera will render to multiview texture, then copy to each eye texture and go from there)
             */
            _useMultiviewToSingleView: boolean;
            /**
             * @internal
             * For cameras that cannot use multiview images to display directly. (e.g. webVR camera will render to multiview texture, then copy to each eye texture and go from there)
             */
            _multiviewTexture: Nullable<RenderTargetTexture>;
            /**
             * @internal
             * For WebXR cameras that are rendering to multiview texture arrays.
             */
            _renderingMultiview: boolean;
            /**
             * @internal
             * ensures the multiview texture of the camera exists and has the specified width/height
             * @param width height to set on the multiview texture
             * @param height width to set on the multiview texture
             */
            _resizeOrCreateMultiviewTexture(width: number, height: number): void;
        }
        interface Scene {
            /** @internal */
            _transformMatrixR: Matrix;
            /** @internal */
            _multiviewSceneUbo: Nullable<UniformBuffer>;
            /** @internal */
            _createMultiviewUbo(): void;
            /** @internal */
            _updateMultiviewUbo(viewR?: Matrix, projectionR?: Matrix): void;
            /** @internal */
            _renderMultiviewToSingleView(camera: Camera): void;
        }


    /** @internal */
    export type OcclusionQuery = WebGLQuery | number;
    /** @internal */
    export class _OcclusionDataStorage {
        /** @internal */
        occlusionInternalRetryCounter: number;
        /** @internal */
        isOcclusionQueryInProgress: boolean;
        /** @internal */
        isOccluded: boolean;
        /** @internal */
        occlusionRetryCount: number;
        /** @internal */
        occlusionType: number;
        /** @internal */
        occlusionQueryAlgorithmType: number;
        /** @internal */
        forceRenderingWhenOccluded: boolean;
    }
        interface Engine {
            /**
             * Create a new webGL query (you must be sure that queries are supported by checking getCaps() function)
             * @returns the new query
             */
            createQuery(): OcclusionQuery;
            /**
             * Delete and release a webGL query
             * @param query defines the query to delete
             * @returns the current engine
             */
            deleteQuery(query: OcclusionQuery): Engine;
            /**
             * Check if a given query has resolved and got its value
             * @param query defines the query to check
             * @returns true if the query got its value
             */
            isQueryResultAvailable(query: OcclusionQuery): boolean;
            /**
             * Gets the value of a given query
             * @param query defines the query to check
             * @returns the value of the query
             */
            getQueryResult(query: OcclusionQuery): number;
            /**
             * Initiates an occlusion query
             * @param algorithmType defines the algorithm to use
             * @param query defines the query to use
             * @returns the current engine
             * @see https://doc.babylonjs.com/features/featuresDeepDive/occlusionQueries
             */
            beginOcclusionQuery(algorithmType: number, query: OcclusionQuery): boolean;
            /**
             * Ends an occlusion query
             * @see https://doc.babylonjs.com/features/featuresDeepDive/occlusionQueries
             * @param algorithmType defines the algorithm to use
             * @returns the current engine
             */
            endOcclusionQuery(algorithmType: number): Engine;
            /**
             * Starts a time query (used to measure time spent by the GPU on a specific frame)
             * Please note that only one query can be issued at a time
             * @returns a time token used to track the time span
             */
            startTimeQuery(): Nullable<_TimeToken>;
            /**
             * Ends a time query
             * @param token defines the token used to measure the time span
             * @returns the time spent (in ns)
             */
            endTimeQuery(token: _TimeToken): int;
            /**
             * Get the performance counter associated with the frame time computation
             * @returns the perf counter
             */
            getGPUFrameTimeCounter(): PerfCounter;
            /**
             * Enable or disable the GPU frame time capture
             * @param value True to enable, false to disable
             */
            captureGPUFrameTime(value: boolean): void;
            /** @internal */
            _currentNonTimestampToken: Nullable<_TimeToken>;
            /** @internal */
            _captureGPUFrameTime: boolean;
            /** @internal */
            _gpuFrameTimeToken: Nullable<_TimeToken>;
            /** @internal */
            _gpuFrameTime: PerfCounter;
            /** @internal */
            _onBeginFrameObserver: Nullable<Observer<Engine>>;
            /** @internal */
            _onEndFrameObserver: Nullable<Observer<Engine>>;
            /** @internal */
            _createTimeQuery(): WebGLQuery;
            /** @internal */
            _deleteTimeQuery(query: WebGLQuery): void;
            /** @internal */
            _getGlAlgorithmType(algorithmType: number): number;
            /** @internal */
            _getTimeQueryResult(query: WebGLQuery): any;
            /** @internal */
            _getTimeQueryAvailability(query: WebGLQuery): any;
        }
        interface AbstractMesh {
            /**
             * Backing filed
             * @internal
             */
            __occlusionDataStorage: _OcclusionDataStorage;
            /**
             * Access property
             * @internal
             */
            _occlusionDataStorage: _OcclusionDataStorage;
            /**
             * This number indicates the number of allowed retries before stop the occlusion query, this is useful if the occlusion query is taking long time before to the query result is retrieved, the query result indicates if the object is visible within the scene or not and based on that Babylon.Js engine decides to show or hide the object.
             * The default value is -1 which means don't break the query and wait till the result
             * @see https://doc.babylonjs.com/features/featuresDeepDive/occlusionQueries
             */
            occlusionRetryCount: number;
            /**
             * This property is responsible for starting the occlusion query within the Mesh or not, this property is also used to determine what should happen when the occlusionRetryCount is reached. It has supports 3 values:
             * * OCCLUSION_TYPE_NONE (Default Value): this option means no occlusion query within the Mesh.
             * * OCCLUSION_TYPE_OPTIMISTIC: this option is means use occlusion query and if occlusionRetryCount is reached and the query is broken show the mesh.
             * * OCCLUSION_TYPE_STRICT: this option is means use occlusion query and if occlusionRetryCount is reached and the query is broken restore the last state of the mesh occlusion if the mesh was visible then show the mesh if was hidden then hide don't show.
             * @see https://doc.babylonjs.com/features/featuresDeepDive/occlusionQueries
             */
            occlusionType: number;
            /**
             * This property determines the type of occlusion query algorithm to run in WebGl, you can use:
             * * AbstractMesh.OCCLUSION_ALGORITHM_TYPE_ACCURATE which is mapped to GL_ANY_SAMPLES_PASSED.
             * * AbstractMesh.OCCLUSION_ALGORITHM_TYPE_CONSERVATIVE (Default Value) which is mapped to GL_ANY_SAMPLES_PASSED_CONSERVATIVE which is a false positive algorithm that is faster than GL_ANY_SAMPLES_PASSED but less accurate.
             * @see https://doc.babylonjs.com/features/featuresDeepDive/occlusionQueries
             */
            occlusionQueryAlgorithmType: number;
            /**
             * Gets or sets whether the mesh is occluded or not, it is used also to set the initial state of the mesh to be occluded or not
             * @see https://doc.babylonjs.com/features/featuresDeepDive/occlusionQueries
             */
            isOccluded: boolean;
            /**
             * Flag to check the progress status of the query
             * @see https://doc.babylonjs.com/features/featuresDeepDive/occlusionQueries
             */
            isOcclusionQueryInProgress: boolean;
            /**
             * Flag to force rendering the mesh even if occluded
             * @see https://doc.babylonjs.com/features/featuresDeepDive/occlusionQueries
             */
            forceRenderingWhenOccluded: boolean;
        }


        interface ThinEngine {
            /**
             * Creates a raw texture
             * @param data defines the data to store in the texture
             * @param width defines the width of the texture
             * @param height defines the height of the texture
             * @param format defines the format of the data
             * @param generateMipMaps defines if the engine should generate the mip levels
             * @param invertY defines if data must be stored with Y axis inverted
             * @param samplingMode defines the required sampling mode (Texture.NEAREST_SAMPLINGMODE by default)
             * @param compression defines the compression used (null by default)
             * @param type defines the type fo the data (Engine.TEXTURETYPE_UNSIGNED_INT by default)
             * @param creationFlags specific flags to use when creating the texture (Constants.TEXTURE_CREATIONFLAG_STORAGE for storage textures, for eg)
             * @param useSRGBBuffer defines if the texture must be loaded in a sRGB GPU buffer (if supported by the GPU).
             * @returns the raw texture inside an InternalTexture
             */
            createRawTexture(data: Nullable<ArrayBufferView>, width: number, height: number, format: number, generateMipMaps: boolean, invertY: boolean, samplingMode: number, compression: Nullable<string>, type: number, creationFlags?: number, useSRGBBuffer?: boolean): InternalTexture;
            /**
             * Update a raw texture
             * @param texture defines the texture to update
             * @param data defines the data to store in the texture
             * @param format defines the format of the data
             * @param invertY defines if data must be stored with Y axis inverted
             */
            updateRawTexture(texture: Nullable<InternalTexture>, data: Nullable<ArrayBufferView>, format: number, invertY: boolean): void;
            /**
             * Update a raw texture
             * @param texture defines the texture to update
             * @param data defines the data to store in the texture
             * @param format defines the format of the data
             * @param invertY defines if data must be stored with Y axis inverted
             * @param compression defines the compression used (null by default)
             * @param type defines the type fo the data (Engine.TEXTURETYPE_UNSIGNED_INT by default)
             * @param useSRGBBuffer defines if the texture must be loaded in a sRGB GPU buffer (if supported by the GPU).
             */
            updateRawTexture(texture: Nullable<InternalTexture>, data: Nullable<ArrayBufferView>, format: number, invertY: boolean, compression: Nullable<string>, type: number, useSRGBBuffer: boolean): void;
            /**
             * Creates a new raw cube texture
             * @param data defines the array of data to use to create each face
             * @param size defines the size of the textures
             * @param format defines the format of the data
             * @param type defines the type of the data (like Engine.TEXTURETYPE_UNSIGNED_INT)
             * @param generateMipMaps  defines if the engine should generate the mip levels
             * @param invertY defines if data must be stored with Y axis inverted
             * @param samplingMode defines the required sampling mode (like Texture.NEAREST_SAMPLINGMODE)
             * @param compression defines the compression used (null by default)
             * @returns the cube texture as an InternalTexture
             */
            createRawCubeTexture(data: Nullable<ArrayBufferView[]>, size: number, format: number, type: number, generateMipMaps: boolean, invertY: boolean, samplingMode: number, compression: Nullable<string>): InternalTexture;
            /**
             * Update a raw cube texture
             * @param texture defines the texture to update
             * @param data defines the data to store
             * @param format defines the data format
             * @param type defines the type fo the data (Engine.TEXTURETYPE_UNSIGNED_INT by default)
             * @param invertY defines if data must be stored with Y axis inverted
             */
            updateRawCubeTexture(texture: InternalTexture, data: ArrayBufferView[], format: number, type: number, invertY: boolean): void;
            /**
             * Update a raw cube texture
             * @param texture defines the texture to update
             * @param data defines the data to store
             * @param format defines the data format
             * @param type defines the type fo the data (Engine.TEXTURETYPE_UNSIGNED_INT by default)
             * @param invertY defines if data must be stored with Y axis inverted
             * @param compression defines the compression used (null by default)
             */
            updateRawCubeTexture(texture: InternalTexture, data: ArrayBufferView[], format: number, type: number, invertY: boolean, compression: Nullable<string>): void;
            /**
             * Update a raw cube texture
             * @param texture defines the texture to update
             * @param data defines the data to store
             * @param format defines the data format
             * @param type defines the type fo the data (Engine.TEXTURETYPE_UNSIGNED_INT by default)
             * @param invertY defines if data must be stored with Y axis inverted
             * @param compression defines the compression used (null by default)
             * @param level defines which level of the texture to update
             */
            updateRawCubeTexture(texture: InternalTexture, data: ArrayBufferView[], format: number, type: number, invertY: boolean, compression: Nullable<string>, level: number): void;
            /**
             * Creates a new raw cube texture from a specified url
             * @param url defines the url where the data is located
             * @param scene defines the current scene
             * @param size defines the size of the textures
             * @param format defines the format of the data
             * @param type defines the type fo the data (like Engine.TEXTURETYPE_UNSIGNED_INT)
             * @param noMipmap defines if the engine should avoid generating the mip levels
             * @param callback defines a callback used to extract texture data from loaded data
             * @param mipmapGenerator defines to provide an optional tool to generate mip levels
             * @param onLoad defines a callback called when texture is loaded
             * @param onError defines a callback called if there is an error
             * @returns the cube texture as an InternalTexture
             */
            createRawCubeTextureFromUrl(url: string, scene: Nullable<Scene>, size: number, format: number, type: number, noMipmap: boolean, callback: (ArrayBuffer: ArrayBuffer) => Nullable<ArrayBufferView[]>, mipmapGenerator: Nullable<(faces: ArrayBufferView[]) => ArrayBufferView[][]>, onLoad: Nullable<() => void>, onError: Nullable<(message?: string, exception?: any) => void>): InternalTexture;
            /**
             * Creates a new raw cube texture from a specified url
             * @param url defines the url where the data is located
             * @param scene defines the current scene
             * @param size defines the size of the textures
             * @param format defines the format of the data
             * @param type defines the type fo the data (like Engine.TEXTURETYPE_UNSIGNED_INT)
             * @param noMipmap defines if the engine should avoid generating the mip levels
             * @param callback defines a callback used to extract texture data from loaded data
             * @param mipmapGenerator defines to provide an optional tool to generate mip levels
             * @param onLoad defines a callback called when texture is loaded
             * @param onError defines a callback called if there is an error
             * @param samplingMode defines the required sampling mode (like Texture.NEAREST_SAMPLINGMODE)
             * @param invertY defines if data must be stored with Y axis inverted
             * @returns the cube texture as an InternalTexture
             */
            createRawCubeTextureFromUrl(url: string, scene: Nullable<Scene>, size: number, format: number, type: number, noMipmap: boolean, callback: (ArrayBuffer: ArrayBuffer) => Nullable<ArrayBufferView[]>, mipmapGenerator: Nullable<(faces: ArrayBufferView[]) => ArrayBufferView[][]>, onLoad: Nullable<() => void>, onError: Nullable<(message?: string, exception?: any) => void>, samplingMode: number, invertY: boolean): InternalTexture;
            /**
             * Creates a new raw 3D texture
             * @param data defines the data used to create the texture
             * @param width defines the width of the texture
             * @param height defines the height of the texture
             * @param depth defines the depth of the texture
             * @param format defines the format of the texture
             * @param generateMipMaps defines if the engine must generate mip levels
             * @param invertY defines if data must be stored with Y axis inverted
             * @param samplingMode defines the required sampling mode (like Texture.NEAREST_SAMPLINGMODE)
             * @param compression defines the compressed used (can be null)
             * @param textureType defines the compressed used (can be null)
             * @param creationFlags specific flags to use when creating the texture (Constants.TEXTURE_CREATIONFLAG_STORAGE for storage textures, for eg)
             * @returns a new raw 3D texture (stored in an InternalTexture)
             */
            createRawTexture3D(data: Nullable<ArrayBufferView>, width: number, height: number, depth: number, format: number, generateMipMaps: boolean, invertY: boolean, samplingMode: number, compression: Nullable<string>, textureType: number, creationFlags?: number): InternalTexture;
            /**
             * Update a raw 3D texture
             * @param texture defines the texture to update
             * @param data defines the data to store
             * @param format defines the data format
             * @param invertY defines if data must be stored with Y axis inverted
             */
            updateRawTexture3D(texture: InternalTexture, data: Nullable<ArrayBufferView>, format: number, invertY: boolean): void;
            /**
             * Update a raw 3D texture
             * @param texture defines the texture to update
             * @param data defines the data to store
             * @param format defines the data format
             * @param invertY defines if data must be stored with Y axis inverted
             * @param compression defines the used compression (can be null)
             * @param textureType defines the texture Type (Engine.TEXTURETYPE_UNSIGNED_INT, Engine.TEXTURETYPE_FLOAT...)
             */
            updateRawTexture3D(texture: InternalTexture, data: Nullable<ArrayBufferView>, format: number, invertY: boolean, compression: Nullable<string>, textureType: number): void;
            /**
             * Creates a new raw 2D array texture
             * @param data defines the data used to create the texture
             * @param width defines the width of the texture
             * @param height defines the height of the texture
             * @param depth defines the number of layers of the texture
             * @param format defines the format of the texture
             * @param generateMipMaps defines if the engine must generate mip levels
             * @param invertY defines if data must be stored with Y axis inverted
             * @param samplingMode defines the required sampling mode (like Texture.NEAREST_SAMPLINGMODE)
             * @param compression defines the compressed used (can be null)
             * @param textureType defines the compressed used (can be null)
             * @param creationFlags specific flags to use when creating the texture (Constants.TEXTURE_CREATIONFLAG_STORAGE for storage textures, for eg)
             * @returns a new raw 2D array texture (stored in an InternalTexture)
             */
            createRawTexture2DArray(data: Nullable<ArrayBufferView>, width: number, height: number, depth: number, format: number, generateMipMaps: boolean, invertY: boolean, samplingMode: number, compression: Nullable<string>, textureType: number, creationFlags?: number): InternalTexture;
            /**
             * Update a raw 2D array texture
             * @param texture defines the texture to update
             * @param data defines the data to store
             * @param format defines the data format
             * @param invertY defines if data must be stored with Y axis inverted
             */
            updateRawTexture2DArray(texture: InternalTexture, data: Nullable<ArrayBufferView>, format: number, invertY: boolean): void;
            /**
             * Update a raw 2D array texture
             * @param texture defines the texture to update
             * @param data defines the data to store
             * @param format defines the data format
             * @param invertY defines if data must be stored with Y axis inverted
             * @param compression defines the used compression (can be null)
             * @param textureType defines the texture Type (Engine.TEXTURETYPE_UNSIGNED_INT, Engine.TEXTURETYPE_FLOAT...)
             */
            updateRawTexture2DArray(texture: InternalTexture, data: Nullable<ArrayBufferView>, format: number, invertY: boolean, compression: Nullable<string>, textureType: number): void;
        }


        interface ThinEngine {
            /** @internal */
            _readTexturePixels(texture: InternalTexture, width: number, height: number, faceIndex?: number, level?: number, buffer?: Nullable<ArrayBufferView>, flushRenderer?: boolean, noDataConversion?: boolean, x?: number, y?: number): Promise<ArrayBufferView>;
            /** @internal */
            _readTexturePixelsSync(texture: InternalTexture, width: number, height: number, faceIndex?: number, level?: number, buffer?: Nullable<ArrayBufferView>, flushRenderer?: boolean, noDataConversion?: boolean, x?: number, y?: number): ArrayBufferView;
        }
    /**
     * Allocate a typed array depending on a texture type. Optionally can copy existing data in the buffer.
     * @param type type of the texture
     * @param sizeOrDstBuffer size of the array OR an existing buffer that will be used as the destination of the copy (if copyBuffer is provided)
     * @param sizeInBytes true if the size of the array is given in bytes, false if it is the number of elements of the array
     * @param copyBuffer if provided, buffer to copy into the destination buffer (either a newly allocated buffer if sizeOrDstBuffer is a number or use sizeOrDstBuffer as the destination buffer otherwise)
     * @returns the allocated buffer or sizeOrDstBuffer if the latter is an ArrayBuffer
     */
    export function allocateAndCopyTypedBuffer(type: number, sizeOrDstBuffer: number | ArrayBuffer, sizeInBytes?: boolean, copyBuffer?: ArrayBuffer): ArrayBufferView;


    /**
     * Type used to define a texture size (either with a number or with a rect width and height)
     * @deprecated please use TextureSize instead
     */
    export type RenderTargetTextureSize = TextureSize;
        interface ThinEngine {
            /**
             * Creates a new render target texture
             * @param size defines the size of the texture
             * @param options defines the options used to create the texture
             * @returns a new render target wrapper ready to render texture
             */
            createRenderTargetTexture(size: TextureSize, options: boolean | RenderTargetCreationOptions): RenderTargetWrapper;
            /**
             * Creates a depth stencil texture.
             * This is only available in WebGL 2 or with the depth texture extension available.
             * @param size The size of face edge in the texture.
             * @param options The options defining the texture.
             * @param rtWrapper The render target wrapper for which the depth/stencil texture must be created
             * @returns The texture
             */
            createDepthStencilTexture(size: TextureSize, options: DepthTextureCreationOptions, rtWrapper: RenderTargetWrapper): InternalTexture;
            /**
             * Updates the sample count of a render target texture
             * @see https://doc.babylonjs.com/setup/support/webGL2#multisample-render-targets
             * @param rtWrapper defines the render target wrapper to update
             * @param samples defines the sample count to set
             * @returns the effective sample count (could be 0 if multisample render targets are not supported)
             */
            updateRenderTargetTextureSampleCount(rtWrapper: Nullable<RenderTargetWrapper>, samples: number): number;
            /** @internal */
            _createDepthStencilTexture(size: TextureSize, options: DepthTextureCreationOptions, rtWrapper: RenderTargetWrapper): InternalTexture;
            /** @internal */
            _createHardwareRenderTargetWrapper(isMulti: boolean, isCube: boolean, size: TextureSize): RenderTargetWrapper;
        }


        interface ThinEngine {
            /**
             * Creates a new render target cube wrapper
             * @param size defines the size of the texture
             * @param options defines the options used to create the texture
             * @returns a new render target cube wrapper
             */
            createRenderTargetCubeTexture(size: number, options?: RenderTargetCreationOptions): RenderTargetWrapper;
        }


        interface ThinEngine {
            /**
             * Creates a storage buffer
             * @param data the data for the storage buffer or the size of the buffer
             * @param creationFlags flags to use when creating the buffer (see Constants.BUFFER_CREATIONFLAG_XXX). The BUFFER_CREATIONFLAG_STORAGE flag will be automatically added
             * @returns the new buffer
             */
            createStorageBuffer(data: DataArray | number, creationFlags: number): DataBuffer;
            /**
             * Updates a storage buffer
             * @param buffer the storage buffer to update
             * @param data the data used to update the storage buffer
             * @param byteOffset the byte offset of the data
             * @param byteLength the byte length of the data
             */
            updateStorageBuffer(buffer: DataBuffer, data: DataArray, byteOffset?: number, byteLength?: number): void;
            /**
             * Read data from a storage buffer
             * @param storageBuffer The storage buffer to read from
             * @param offset The offset in the storage buffer to start reading from (default: 0)
             * @param size  The number of bytes to read from the storage buffer (default: capacity of the buffer)
             * @param buffer The buffer to write the data we have read from the storage buffer to (optional)
             * @returns If not undefined, returns the (promise) buffer (as provided by the 4th parameter) filled with the data, else it returns a (promise) Uint8Array with the data read from the storage buffer
             */
            readFromStorageBuffer(storageBuffer: DataBuffer, offset?: number, size?: number, buffer?: ArrayBufferView): Promise<ArrayBufferView>;
            /**
             * Sets a storage buffer in the shader
             * @param name Defines the name of the storage buffer as defined in the shader
             * @param buffer Defines the value to give to the uniform
             */
            setStorageBuffer(name: string, buffer: Nullable<StorageBuffer>): void;
        }


        interface ThinEngine {
            /**
             * Sets a texture sampler to the according uniform.
             * @param name The name of the uniform in the effect
             * @param sampler The sampler to apply
             */
            setTextureSampler(name: string, sampler: Nullable<TextureSampler>): void;
        }


        interface Engine {
            /** @internal */
            _excludedCompressedTextures: string[];
            /** @internal */
            _textureFormatInUse: string;
            /**
             * Gets the list of texture formats supported
             */
            readonly texturesSupported: Array<string>;
            /**
             * Gets the texture format in use
             */
            readonly textureFormatInUse: Nullable<string>;
            /**
             * Set the compressed texture extensions or file names to skip.
             *
             * @param skippedFiles defines the list of those texture files you want to skip
             * Example: [".dds", ".env", "myfile.png"]
             */
            setCompressedTextureExclusions(skippedFiles: Array<string>): void;
            /**
             * Set the compressed texture format to use, based on the formats you have, and the formats
             * supported by the hardware / browser.
             *
             * Khronos Texture Container (.ktx) files are used to support this.  This format has the
             * advantage of being specifically designed for OpenGL.  Header elements directly correspond
             * to API arguments needed to compressed textures.  This puts the burden on the container
             * generator to house the arcane code for determining these for current & future formats.
             *
             * for description see https://www.khronos.org/opengles/sdk/tools/KTX/
             * for file layout see https://www.khronos.org/opengles/sdk/tools/KTX/file_format_spec/
             *
             * Note: The result of this call is not taken into account when a texture is base64.
             *
             * @param formatsAvailable defines the list of those format families you have created
             * on your server.  Syntax: '-' + format family + '.ktx'.  (Case and order do not matter.)
             *
             * Current families are astc, dxt, pvrtc, etc2, & etc1.
             * @returns The extension selected.
             */
            setTextureFormatToUse(formatsAvailable: Array<string>): Nullable<string>;
        }


    /** @internal */
    export var _forceTransformFeedbackToBundle: boolean;
        interface Engine {
            /**
             * Creates a webGL transform feedback object
             * Please makes sure to check webGLVersion property to check if you are running webGL 2+
             * @returns the webGL transform feedback object
             */
            createTransformFeedback(): WebGLTransformFeedback;
            /**
             * Delete a webGL transform feedback object
             * @param value defines the webGL transform feedback object to delete
             */
            deleteTransformFeedback(value: WebGLTransformFeedback): void;
            /**
             * Bind a webGL transform feedback object to the webgl context
             * @param value defines the webGL transform feedback object to bind
             */
            bindTransformFeedback(value: Nullable<WebGLTransformFeedback>): void;
            /**
             * Begins a transform feedback operation
             * @param usePoints defines if points or triangles must be used
             */
            beginTransformFeedback(usePoints: boolean): void;
            /**
             * Ends a transform feedback operation
             */
            endTransformFeedback(): void;
            /**
             * Specify the varyings to use with transform feedback
             * @param program defines the associated webGL program
             * @param value defines the list of strings representing the varying names
             */
            setTranformFeedbackVaryings(program: WebGLProgram, value: string[]): void;
            /**
             * Bind a webGL buffer for a transform feedback operation
             * @param value defines the webGL buffer to bind
             */
            bindTransformFeedbackBuffer(value: Nullable<DataBuffer>): void;
        }


        interface ThinEngine {
            /**
             * Create an uniform buffer
             * @see https://doc.babylonjs.com/setup/support/webGL2#uniform-buffer-objets
             * @param elements defines the content of the uniform buffer
             * @returns the webGL uniform buffer
             */
            createUniformBuffer(elements: FloatArray): DataBuffer;
            /**
             * Create a dynamic uniform buffer
             * @see https://doc.babylonjs.com/setup/support/webGL2#uniform-buffer-objets
             * @param elements defines the content of the uniform buffer
             * @returns the webGL uniform buffer
             */
            createDynamicUniformBuffer(elements: FloatArray): DataBuffer;
            /**
             * Update an existing uniform buffer
             * @see https://doc.babylonjs.com/setup/support/webGL2#uniform-buffer-objets
             * @param uniformBuffer defines the target uniform buffer
             * @param elements defines the content to update
             * @param offset defines the offset in the uniform buffer where update should start
             * @param count defines the size of the data to update
             */
            updateUniformBuffer(uniformBuffer: DataBuffer, elements: FloatArray, offset?: number, count?: number): void;
            /**
             * Bind an uniform buffer to the current webGL context
             * @param buffer defines the buffer to bind
             */
            bindUniformBuffer(buffer: Nullable<DataBuffer>): void;
            /**
             * Bind a buffer to the current webGL context at a given location
             * @param buffer defines the buffer to bind
             * @param location defines the index where to bind the buffer
             * @param name Name of the uniform variable to bind
             */
            bindUniformBufferBase(buffer: DataBuffer, location: number, name: string): void;
            /**
             * Bind a specific block at a given index in a specific shader program
             * @param pipelineContext defines the pipeline context to use
             * @param blockName defines the block name
             * @param index defines the index where to bind the block
             */
            bindUniformBlock(pipelineContext: IPipelineContext, blockName: string, index: number): void;
        }


        interface ThinEngine {
            /**
             * Update a video texture
             * @param texture defines the texture to update
             * @param video defines the video element to use
             * @param invertY defines if data must be stored with Y axis inverted
             */
            updateVideoTexture(texture: Nullable<InternalTexture>, video: HTMLVideoElement | Nullable<ExternalTexture>, invertY: boolean): void;
        }


    /**
     * Class used to define an additional view for the engine
     * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/multiCanvas
     */
    export class EngineView {
        /**
         * A randomly generated unique id
         */
        readonly id: string;
        /** Defines the canvas where to render the view */
        target: HTMLCanvasElement;
        /** Defines an optional camera used to render the view (will use active camera else) */
        camera?: Camera;
        /** Indicates if the destination view canvas should be cleared before copying the parent canvas. Can help if the scene clear color has alpha < 1 */
        clearBeforeCopy?: boolean;
        /** Indicates if the view is enabled (true by default) */
        enabled: boolean;
        /** Defines a custom function to handle canvas size changes. (the canvas to render into is provided to the callback) */
        customResize?: (canvas: HTMLCanvasElement) => void;
    }
        interface Engine {
            /** @internal */
            _inputElement: Nullable<HTMLElement>;
            /**
             * Gets or sets the  HTML element to use for attaching events
             */
            inputElement: Nullable<HTMLElement>;
            /**
             * Observable to handle when a change to inputElement occurs
             * @internal
             */
            _onEngineViewChanged?: () => void;
            /**
             * Will be triggered before the view renders
             */
            readonly onBeforeViewRenderObservable: Observable<EngineView>;
            /**
             * Will be triggered after the view rendered
             */
            readonly onAfterViewRenderObservable: Observable<EngineView>;
            /**
             * Gets the current engine view
             * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/multiCanvas
             */
            activeView: Nullable<EngineView>;
            /** Gets or sets the list of views */
            views: EngineView[];
            /**
             * Register a new child canvas
             * @param canvas defines the canvas to register
             * @param camera defines an optional camera to use with this canvas (it will overwrite the scene.camera for this view)
             * @param clearBeforeCopy Indicates if the destination view canvas should be cleared before copying the parent canvas. Can help if the scene clear color has alpha < 1
             * @returns the associated view
             */
            registerView(canvas: HTMLCanvasElement, camera?: Camera, clearBeforeCopy?: boolean): EngineView;
            /**
             * Remove a registered child canvas
             * @param canvas defines the canvas to remove
             * @returns the current engine
             */
            unRegisterView(canvas: HTMLCanvasElement): Engine;
            /**
             * @internal
             */
            _renderViewStep(view: EngineView): boolean;
        }


    /**
     * Interface used to define additional presentation attributes
     */
    export interface IVRPresentationAttributes {
        /**
         * Defines a boolean indicating that we want to get 72hz mode on Oculus Browser (default is off eg. 60hz)
         */
        highRefreshRate: boolean;
        /**
         * Enables foveation in VR to improve perf. 0 none, 1 low, 2 medium, 3 high (Default is 1)
         */
        foveationLevel: number;
    }
        interface Engine {
            /** @internal */
            _vrDisplay: any;
            /** @internal */
            _vrSupported: boolean;
            /** @internal */
            _oldSize: Size;
            /** @internal */
            _oldHardwareScaleFactor: number;
            /** @internal */
            _vrExclusivePointerMode: boolean;
            /** @internal */
            _webVRInitPromise: Promise<IDisplayChangedEventArgs>;
            /** @internal */
            _onVRDisplayPointerRestricted: () => void;
            /** @internal */
            _onVRDisplayPointerUnrestricted: () => void;
            /** @internal */
            _onVrDisplayConnect: Nullable<(display: any) => void>;
            /** @internal */
            _onVrDisplayDisconnect: Nullable<() => void>;
            /** @internal */
            _onVrDisplayPresentChange: Nullable<() => void>;
            /**
             * Observable signaled when VR display mode changes
             */
            onVRDisplayChangedObservable: Observable<IDisplayChangedEventArgs>;
            /**
             * Observable signaled when VR request present is complete
             */
            onVRRequestPresentComplete: Observable<boolean>;
            /**
             * Observable signaled when VR request present starts
             */
            onVRRequestPresentStart: Observable<Engine>;
            /**
             * Gets a boolean indicating that the engine is currently in VR exclusive mode for the pointers
             * @see https://docs.microsoft.com/en-us/microsoft-edge/webvr/essentials#mouse-input
             */
            isInVRExclusivePointerMode: boolean;
            /**
             * Gets a boolean indicating if a webVR device was detected
             * @returns true if a webVR device was detected
             */
            isVRDevicePresent(): boolean;
            /**
             * Gets the current webVR device
             * @returns the current webVR device (or null)
             */
            getVRDevice(): any;
            /**
             * Initializes a webVR display and starts listening to display change events
             * The onVRDisplayChangedObservable will be notified upon these changes
             * @returns A promise containing a VRDisplay and if vr is supported
             */
            initWebVRAsync(): Promise<IDisplayChangedEventArgs>;
            /** @internal */
            _getVRDisplaysAsync(): Promise<IDisplayChangedEventArgs>;
            /**
             * Gets or sets the presentation attributes used to configure VR rendering
             */
            vrPresentationAttributes?: IVRPresentationAttributes;
            /**
             * Call this function to switch to webVR mode
             * Will do nothing if webVR is not supported or if there is no webVR device
             * @param options the webvr options provided to the camera. mainly used for multiview
             * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/webVRCamera
             */
            enableVR(options: WebVROptions): void;
            /** @internal */
            _onVRFullScreenTriggered(): void;
        }




    /**
     * Class used to abstract a canvas
     */
    export interface ICanvas {
        /**
         * Canvas width.
         */
        width: number;
        /**
         * Canvas height.
         */
        height: number;
        /**
         * returns a drawing context on the canvas.
         * @param contextType context identifier.
         * @param contextAttributes context attributes.
         * @returns ICanvasRenderingContext object.
         */
        getContext(contextType: string, contextAttributes?: any): ICanvasRenderingContext;
        /**
         * returns a data URI containing a representation of the image in the format specified by the type parameter.
         * @param mime the image format.
         * @returns string containing the requested data URI.
         */
        toDataURL(mime: string): string;
    }
    /**
     * Class used to abstract am image to use with the canvas and its context
     */
    export interface IImage {
        /**
         * onload callback.
         */
        onload: ((this: GlobalEventHandlers, ev: Event) => any) | null;
        /**
         * Error callback.
         */
        onerror: ((this: GlobalEventHandlers, ev: Event) => any) | null;
        /**
         * Image source.
         */
        src: string;
        /**
         * Image width.
         */
        readonly width: number;
        /**
         * Image height.
         */
        readonly height: number;
        /**
         * The original height of the image resource before sizing.
         */
        readonly naturalHeight: number;
        /**
         * The original width of the image resource before sizing.
         */
        readonly naturalWidth: number;
        /**
         * provides support for CORS, defining how the element handles crossorigin requests,
         * thereby enabling the configuration of the CORS requests for the element's fetched data.
         */
        crossOrigin: string | null;
        /**
         * provides support for referrer policy on xhr load request,
         * it is used to control the request header.
         */
        referrerPolicy: string;
    }
    /**
     * Class used to abstract a canvas gradient
     */
    export interface ICanvasGradient {
        /**
         * adds a new color stop, defined by an offset and a color, to a given canvas gradient.
         * @param offset A number between 0 and 1, inclusive, representing the position of the color stop. 0 represents the start of the gradient and 1 represents the end.
         * @param color value representing the color of the stop.
         */
        addColorStop(offset: number, color: string): void;
    }
    /**
     * Class used to abstract a text measurement
     */
    export interface ITextMetrics {
        /**
         * Text width.
         */
        readonly width: number;
        /**
         * distance (in pixels) parallel to the baseline from the alignment point given by the CanvasRenderingContext2D.textAlign
         * property to the left side of the bounding rectangle of the given text
         */
        readonly actualBoundingBoxLeft: number;
        /**
         * distance (in pixels) parallel to the baseline from the alignment point given by the CanvasRenderingContext2D.textAlign
         * property to the right side of the bounding rectangle of the given text
         */
        readonly actualBoundingBoxRight: number;
    }
    /**
     * Class used to abstract canvas rendering
     */
    export interface ICanvasRenderingContext {
        /**
         * Defines the type of corners where two lines meet. Possible values: round, bevel, miter (default).
         */
        lineJoin: string;
        /**
         * Miter limit ratio. Default 10.
         */
        miterLimit: number;
        /**
         * Font setting. Default value 10px sans-serif.
         */
        font: string;
        /**
         * Color or style to use for the lines around shapes. Default #000 (black).
         */
        strokeStyle: string | ICanvasGradient;
        /**
         * Color or style to use inside shapes. Default #000 (black).
         */
        fillStyle: string | ICanvasGradient;
        /**
         * Alpha value that is applied to shapes and images before they are composited onto the canvas. Default 1.0 (opaque).
         */
        globalAlpha: number;
        /**
         * Color of the shadow. Default: fully-transparent black.
         */
        shadowColor: string;
        /**
         * Specifies the blurring effect. Default: 0.
         */
        shadowBlur: number;
        /**
         * Horizontal distance the shadow will be offset. Default: 0.
         */
        shadowOffsetX: number;
        /**
         * Vertical distance the shadow will be offset. Default: 0.
         */
        shadowOffsetY: number;
        /**
         * Width of lines. Default 1.0.
         */
        lineWidth: number;
        /**
         * canvas is a read-only reference to ICanvas.
         */
        readonly canvas: ICanvas;
        /**
         * Sets all pixels in the rectangle defined by starting point (x, y) and size (width, height) to transparent black, erasing any previously drawn content.
         * @param x The x-axis coordinate of the rectangle's starting point.
         * @param y The y-axis coordinate of the rectangle's starting point.
         * @param width The rectangle's width. Positive values are to the right, and negative to the left.
         * @param height The rectangle's height. Positive values are down, and negative are up.
         */
        clearRect(x: number, y: number, width: number, height: number): void;
        /**
         * Saves the current drawing style state using a stack so you can revert any change you make to it using restore().
         */
        save(): void;
        /**
         * Restores the drawing style state to the last element on the 'state stack' saved by save().
         */
        restore(): void;
        /**
         * Draws a filled rectangle at (x, y) position whose size is determined by width and height.
         * @param x The x-axis coordinate of the rectangle's starting point.
         * @param y The y-axis coordinate of the rectangle's starting point.
         * @param width The rectangle's width. Positive values are to the right, and negative to the left.
         * @param height The rectangle's height. Positive values are down, and negative are up.
         */
        fillRect(x: number, y: number, width: number, height: number): void;
        /**
         * Adds a scaling transformation to the canvas units by x horizontally and by y vertically.
         * @param x Scaling factor in the horizontal direction. A negative value flips pixels across the vertical axis. A value of 1 results in no horizontal scaling.
         * @param y Scaling factor in the vertical direction. A negative value flips pixels across the horizontal axis. A value of 1 results in no vertical scaling.
         */
        scale(x: number, y: number): void;
        /**
         * Adds a rotation to the transformation matrix. The angle argument represents a clockwise rotation angle and is expressed in radians.
         * @param angle The rotation angle, clockwise in radians. You can use degree * Math.PI / 180 to calculate a radian from a degree.
         */
        rotate(angle: number): void;
        /**
         * Adds a translation transformation by moving the canvas and its origin x horizontally and y vertically on the grid.
         * @param x Distance to move in the horizontal direction. Positive values are to the right, and negative to the left.
         * @param y Distance to move in the vertical direction. Positive values are down, and negative are up.
         */
        translate(x: number, y: number): void;
        /**
         * Paints a rectangle which has a starting point at (x, y) and has a w width and an h height onto the canvas, using the current stroke style.
         * @param x The x-axis coordinate of the rectangle's starting point.
         * @param y The y-axis coordinate of the rectangle's starting point.
         * @param width The rectangle's width. Positive values are to the right, and negative to the left.
         * @param height The rectangle's height. Positive values are down, and negative are up.
         */
        strokeRect(x: number, y: number, width: number, height: number): void;
        /**
         * Creates a path for a rectangle at position (x, y) with a size that is determined by width and height.
         * @param x The x-axis coordinate of the rectangle's starting point.
         * @param y The y-axis coordinate of the rectangle's starting point.
         * @param width The rectangle's width. Positive values are to the right, and negative to the left.
         * @param height The rectangle's height. Positive values are down, and negative are up.
         */
        rect(x: number, y: number, width: number, height: number): void;
        /**
         * Creates a clipping path from the current sub-paths. Everything drawn after clip() is called appears inside the clipping path only.
         */
        clip(): void;
        /**
         * Paints data from the given ImageData object onto the bitmap. If a dirty rectangle is provided, only the pixels from that rectangle are painted.
         * @param imageData An ImageData object containing the array of pixel values.
         * @param dx Horizontal position (x coordinate) at which to place the image data in the destination canvas.
         * @param dy Vertical position (y coordinate) at which to place the image data in the destination canvas.
         */
        putImageData(imageData: ImageData, dx: number, dy: number): void;
        /**
         * Adds a circular arc to the current path.
         * @param x The horizontal coordinate of the arc's center.
         * @param y The vertical coordinate of the arc's center.
         * @param radius The arc's radius. Must be positive.
         * @param startAngle The angle at which the arc starts in radians, measured from the positive x-axis.
         * @param endAngle The angle at which the arc ends in radians, measured from the positive x-axis.
         * @param anticlockwise An optional Boolean. If true, draws the arc counter-clockwise between the start and end angles. The default is false (clockwise).
         */
        arc(x: number, y: number, radius: number, startAngle: number, endAngle: number, anticlockwise?: boolean): void;
        /**
         * Starts a new path by emptying the list of sub-paths. Call this method when you want to create a new path.
         */
        beginPath(): void;
        /**
         * Causes the point of the pen to move back to the start of the current sub-path. It tries to draw a straight line from the current point to the start.
         * If the shape has already been closed or has only one point, this function does nothing.
         */
        closePath(): void;
        /**
         * Moves the starting point of a new sub-path to the (x, y) coordinates.
         * @param x The x-axis (horizontal) coordinate of the point.
         * @param y The y-axis (vertical) coordinate of the point.
         */
        moveTo(x: number, y: number): void;
        /**
         * Connects the last point in the current sub-path to the specified (x, y) coordinates with a straight line.
         * @param x The x-axis coordinate of the line's end point.
         * @param y The y-axis coordinate of the line's end point.
         */
        lineTo(x: number, y: number): void;
        /**
         * Adds a quadratic Bézier curve to the current path.
         * @param cpx The x-axis coordinate of the control point.
         * @param cpy The y-axis coordinate of the control point.
         * @param x The x-axis coordinate of the end point.
         * @param y The y-axis coordinate of the end point.
         */
        quadraticCurveTo(cpx: number, cpy: number, x: number, y: number): void;
        /**
         * Returns a TextMetrics object.
         * @param text The text String to measure.
         * @returns ITextMetrics A ITextMetrics object.
         */
        measureText(text: string): ITextMetrics;
        /**
         * Strokes the current sub-paths with the current stroke style.
         */
        stroke(): void;
        /**
         * Fills the current sub-paths with the current fill style.
         */
        fill(): void;
        /**
         * Draws the specified image. This method is available in multiple formats, providing a great deal of flexibility in its use.
         * @param image An element to draw into the context.
         * @param sx The x-axis coordinate of the top left corner of the sub-rectangle of the source image to draw into the destination context.
         * @param sy The y-axis coordinate of the top left corner of the sub-rectangle of the source image to draw into the destination context.
         * @param sWidth The width of the sub-rectangle of the source image to draw into the destination context. If not specified, the entire rectangle from the coordinates specified by sx and sy to the bottom-right corner of the image is used.
         * @param sHeight The height of the sub-rectangle of the source image to draw into the destination context.
         * @param dx The x-axis coordinate in the destination canvas at which to place the top-left corner of the source image.
         * @param dy The y-axis coordinate in the destination canvas at which to place the top-left corner of the source image.
         * @param dWidth The width to draw the image in the destination canvas. This allows scaling of the drawn image. If not specified, the image is not scaled in width when drawn.
         * @param dHeight The height to draw the image in the destination canvas. This allows scaling of the drawn image. If not specified, the image is not scaled in height when drawn.
         */
        drawImage(image: any, sx: number, sy: number, sWidth: number, sHeight: number, dx: number, dy: number, dWidth: number, dHeight: number): void;
        /**
         * Draws the specified image. This method is available in multiple formats, providing a great deal of flexibility in its use.
         * @param image An element to draw into the context.
         * @param dx The x-axis coordinate in the destination canvas at which to place the top-left corner of the source image.
         * @param dy The y-axis coordinate in the destination canvas at which to place the top-left corner of the source image.
         * @param dWidth The width to draw the image in the destination canvas. This allows scaling of the drawn image. If not specified, the image is not scaled in width when drawn.
         * @param dHeight The height to draw the image in the destination canvas. This allows scaling of the drawn image. If not specified, the image is not scaled in height when drawn.
         */
        drawImage(image: any, dx: number, dy: number, dWidth: number, dHeight: number): void;
        /**
         * Draws the specified image. This method is available in multiple formats, providing a great deal of flexibility in its use.
         * @param image An element to draw into the context.
         * @param dx The x-axis coordinate in the destination canvas at which to place the top-left corner of the source image.
         * @param dy The y-axis coordinate in the destination canvas at which to place the top-left corner of the source image.
         */
        drawImage(image: any, dx: number, dy: number): void;
        /**
         * Returns an ImageData object representing the underlying pixel data for the area of the canvas denoted by the rectangle which starts at (sx, sy) and has an sw width and sh height.
         * @param sx The x-axis coordinate of the top-left corner of the rectangle from which the ImageData will be extracted.
         * @param sy The y-axis coordinate of the top-left corner of the rectangle from which the ImageData will be extracted.
         * @param sw The width of the rectangle from which the ImageData will be extracted. Positive values are to the right, and negative to the left.
         * @param sh The height of the rectangle from which the ImageData will be extracted. Positive values are down, and negative are up.
         * @returns ImageData An ImageData object containing the image data for the rectangle of the canvas specified.
         */
        getImageData(sx: number, sy: number, sw: number, sh: number): ImageData;
        /**
         * Sets the current line dash pattern.
         * @param segments An Array of numbers that specify distances to alternately draw a line and a gap (in coordinate space units).
         */
        setLineDash(segments: Array<number>): void;
        /**
         * Draws (fills) a given text at the given (x, y) position.
         * @param text A String specifying the text string to render into the context. The text is rendered using the settings specified by font, textAlign, textBaseline, and direction.
         * @param x The x-axis coordinate of the point at which to begin drawing the text, in pixels.
         * @param y The y-axis coordinate of the baseline on which to begin drawing the text, in pixels.
         * @param maxWidth The maximum number of pixels wide the text may be once rendered. If not specified, there is no limit to the width of the text.
         */
        fillText(text: string, x: number, y: number, maxWidth?: number): void;
        /**
         * Draws (strokes) a given text at the given (x, y) position.
         * @param text A String specifying the text string to render into the context. The text is rendered using the settings specified by font, textAlign, textBaseline, and direction.
         * @param x The x-axis coordinate of the point at which to begin drawing the text, in pixels.
         * @param y The y-axis coordinate of the baseline on which to begin drawing the text, in pixels.
         * @param maxWidth The maximum number of pixels wide the text may be once rendered. If not specified, there is no limit to the width of the text.
         */
        strokeText(text: string, x: number, y: number, maxWidth?: number): void;
        /**
         * Creates a linear gradient along the line given by the coordinates represented by the parameters.
         * @param x0 The x-axis coordinate of the start point.
         * @param y0 The y-axis coordinate of the start point.
         * @param x1 The x-axis coordinate of the end point.
         * @param y1 The y-axis coordinate of the end point.
         * @returns ICanvasGradient A linear ICanvasGradient initialized with the specified line.
         */
        createLinearGradient(x0: number, y0: number, x1: number, y1: number): ICanvasGradient;
        /**
         * Creates a linear gradient along the line given by the coordinates represented by the parameters.
         * @param x0 The x-axis coordinate of the start circle.
         * @param y0 The y-axis coordinate of the start circle.
         * @param r0 The radius of the start circle. Must be non-negative and finite.
         * @param x1 The x-axis coordinate of the end point.
         * @param y1 The y-axis coordinate of the end point.
         * @param r1 The radius of the end circle. Must be non-negative and finite.
         * @returns ICanvasGradient A linear ICanvasGradient initialized with the two specified circles.
         */
        createRadialGradient(x0: number, y0: number, r0: number, x1: number, y1: number, r1: number): ICanvasGradient;
        /**
         * Resets the current transform to matrix composed with a, b, c, d, e, f.
         * @param a Horizontal scaling. A value of 1 results in no scaling.
         * @param b Vertical skewing.
         * @param c Horizontal skewing.
         * @param d Vertical scaling. A value of 1 results in no scaling.
         * @param e Horizontal translation (moving).
         * @param f Vertical translation (moving).
         */
        setTransform(a: number, b: number, c: number, d: number, e: number, f: number): void;
    }


    /** @internal */
    export interface IDrawContext {
        uniqueId: number;
        useInstancing: boolean;
        reset(): void;
        dispose(): void;
    }


    /** @internal */
    export interface IMaterialContext {
        uniqueId: number;
        reset(): void;
    }




    /**
     * Interface for attribute information associated with buffer instantiation
     */
    export interface InstancingAttributeInfo {
        /**
         * Name of the GLSL attribute
         * if attribute index is not specified, this is used to retrieve the index from the effect
         */
        attributeName: string;
        /**
         * Index/offset of the attribute in the vertex shader
         * if not specified, this will be computes from the name.
         */
        index?: number;
        /**
         * size of the attribute, 1, 2, 3 or 4
         */
        attributeSize: number;
        /**
         * Offset of the data in the Vertex Buffer acting as the instancing buffer
         */
        offset: number;
        /**
         * Modifies the rate at which generic vertex attributes advance when rendering multiple instances
         * default to 1
         */
        divisor?: number;
        /**
         * type of the attribute, gl.BYTE, gl.UNSIGNED_BYTE, gl.SHORT, gl.UNSIGNED_SHORT, gl.FIXED, gl.FLOAT.
         * default is FLOAT
         */
        attributeType?: number;
        /**
         * normalization of fixed-point data. behavior unclear, use FALSE, default is FALSE
         */
        normalized?: boolean;
    }


    /**
     * Class used to store and describe the pipeline context associated with an effect
     */
    export interface IPipelineContext {
        /**
         * Gets a boolean indicating that this pipeline context is supporting asynchronous creating
         */
        isAsync: boolean;
        /**
         * Gets a boolean indicating that the context is ready to be used (like shaders / pipelines are compiled and ready for instance)
         */
        isReady: boolean;
        /** @internal */
        _name?: string;
        /** @internal */
        _getVertexShaderCode(): string | null;
        /** @internal */
        _getFragmentShaderCode(): string | null;
        /** @internal */
        _handlesSpectorRebuildCallback(onCompiled: (compiledObject: any) => void): void;
        /** @internal */
        _fillEffectInformation(effect: Effect, uniformBuffersNames: {
            [key: string]: number;
        }, uniformsNames: string[], uniforms: {
            [key: string]: Nullable<WebGLUniformLocation>;
        }, samplerList: string[], samplers: {
            [key: string]: number;
        }, attributesNames: string[], attributes: number[]): void;
        /** Releases the resources associated with the pipeline. */
        dispose(): void;
        /**
         * Sets an integer value on a uniform variable.
         * @param uniformName Name of the variable.
         * @param value Value to be set.
         */
        setInt(uniformName: string, value: number): void;
        /**
         * Sets an int2 value on a uniform variable.
         * @param uniformName Name of the variable.
         * @param x First int in int2.
         * @param y Second int in int2.
         */
        setInt2(uniformName: string, x: number, y: number): void;
        /**
         * Sets an int3 value on a uniform variable.
         * @param uniformName Name of the variable.
         * @param x First int in int3.
         * @param y Second int in int3.
         * @param z Third int in int3.
         */
        setInt3(uniformName: string, x: number, y: number, z: number): void;
        /**
         * Sets an int4 value on a uniform variable.
         * @param uniformName Name of the variable.
         * @param x First int in int4.
         * @param y Second int in int4.
         * @param z Third int in int4.
         * @param w Fourth int in int4.
         */
        setInt4(uniformName: string, x: number, y: number, z: number, w: number): void;
        /**
         * Sets an int array on a uniform variable.
         * @param uniformName Name of the variable.
         * @param array array to be set.
         */
        setIntArray(uniformName: string, array: Int32Array): void;
        /**
         * Sets an int array 2 on a uniform variable. (Array is specified as single array eg. [1,2,3,4] will result in [[1,2],[3,4]] in the shader)
         * @param uniformName Name of the variable.
         * @param array array to be set.
         */
        setIntArray2(uniformName: string, array: Int32Array): void;
        /**
         * Sets an int array 3 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6] will result in [[1,2,3],[4,5,6]] in the shader)
         * @param uniformName Name of the variable.
         * @param array array to be set.
         */
        setIntArray3(uniformName: string, array: Int32Array): void;
        /**
         * Sets an int array 4 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6,7,8] will result in [[1,2,3,4],[5,6,7,8]] in the shader)
         * @param uniformName Name of the variable.
         * @param array array to be set.
         */
        setIntArray4(uniformName: string, array: Int32Array): void;
        /**
         * Sets an unsigned integer value on a uniform variable.
         * @param uniformName Name of the variable.
         * @param value Value to be set.
         */
        setUInt(uniformName: string, value: number): void;
        /**
         * Sets an unsigned int2 value on a uniform variable.
         * @param uniformName Name of the variable.
         * @param x First unsigned int in uint2.
         * @param y Second unsigned int in uint2.
         */
        setUInt2(uniformName: string, x: number, y: number): void;
        /**
         * Sets an unsigned int3 value on a uniform variable.
         * @param uniformName Name of the variable.
         * @param x First unsigned int in uint3.
         * @param y Second unsigned int in uint3.
         * @param z Third unsigned int in uint3.
         */
        setUInt3(uniformName: string, x: number, y: number, z: number): void;
        /**
         * Sets an unsigned int4 value on a uniform variable.
         * @param uniformName Name of the variable.
         * @param x First unsigned int in uint4.
         * @param y Second unsigned int in uint4.
         * @param z Third unsigned int in uint4.
         * @param w Fourth unsigned int in uint4.
         */
        setUInt4(uniformName: string, x: number, y: number, z: number, w: number): void;
        /**
         * Sets an unsigned int array on a uniform variable.
         * @param uniformName Name of the variable.
         * @param array array to be set.
         */
        setUIntArray(uniformName: string, array: Uint32Array): void;
        /**
         * Sets an unsigned int array 2 on a uniform variable. (Array is specified as single array eg. [1,2,3,4] will result in [[1,2],[3,4]] in the shader)
         * @param uniformName Name of the variable.
         * @param array array to be set.
         */
        setUIntArray2(uniformName: string, array: Uint32Array): void;
        /**
         * Sets an unsigned int array 3 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6] will result in [[1,2,3],[4,5,6]] in the shader)
         * @param uniformName Name of the variable.
         * @param array array to be set.
         */
        setUIntArray3(uniformName: string, array: Uint32Array): void;
        /**
         * Sets an unsigned int array 4 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6,7,8] will result in [[1,2,3,4],[5,6,7,8]] in the shader)
         * @param uniformName Name of the variable.
         * @param array array to be set.
         */
        setUIntArray4(uniformName: string, array: Uint32Array): void;
        /**
         * Sets an array on a uniform variable.
         * @param uniformName Name of the variable.
         * @param array array to be set.
         */
        setArray(uniformName: string, array: number[] | Float32Array): void;
        /**
         * Sets an array 2 on a uniform variable. (Array is specified as single array eg. [1,2,3,4] will result in [[1,2],[3,4]] in the shader)
         * @param uniformName Name of the variable.
         * @param array array to be set.
         */
        setArray2(uniformName: string, array: number[] | Float32Array): void;
        /**
         * Sets an array 3 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6] will result in [[1,2,3],[4,5,6]] in the shader)
         * @param uniformName Name of the variable.
         * @param array array to be set.
         */
        setArray3(uniformName: string, array: number[] | Float32Array): void;
        /**
         * Sets an array 4 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6,7,8] will result in [[1,2,3,4],[5,6,7,8]] in the shader)
         * @param uniformName Name of the variable.
         * @param array array to be set.
         */
        setArray4(uniformName: string, array: number[] | Float32Array): void;
        /**
         * Sets matrices on a uniform variable.
         * @param uniformName Name of the variable.
         * @param matrices matrices to be set.
         */
        setMatrices(uniformName: string, matrices: Float32Array): void;
        /**
         * Sets matrix on a uniform variable.
         * @param uniformName Name of the variable.
         * @param matrix matrix to be set.
         */
        setMatrix(uniformName: string, matrix: IMatrixLike): void;
        /**
         * Sets a 3x3 matrix on a uniform variable. (Specified as [1,2,3,4,5,6,7,8,9] will result in [1,2,3][4,5,6][7,8,9] matrix)
         * @param uniformName Name of the variable.
         * @param matrix matrix to be set.
         */
        setMatrix3x3(uniformName: string, matrix: Float32Array): void;
        /**
         * Sets a 2x2 matrix on a uniform variable. (Specified as [1,2,3,4] will result in [1,2][3,4] matrix)
         * @param uniformName Name of the variable.
         * @param matrix matrix to be set.
         */
        setMatrix2x2(uniformName: string, matrix: Float32Array): void;
        /**
         * Sets a float on a uniform variable.
         * @param uniformName Name of the variable.
         * @param value value to be set.
         */
        setFloat(uniformName: string, value: number): void;
        /**
         * Sets a Vector2 on a uniform variable.
         * @param uniformName Name of the variable.
         * @param vector2 vector2 to be set.
         */
        setVector2(uniformName: string, vector2: IVector2Like): void;
        /**
         * Sets a float2 on a uniform variable.
         * @param uniformName Name of the variable.
         * @param x First float in float2.
         * @param y Second float in float2.
         */
        setFloat2(uniformName: string, x: number, y: number): void;
        /**
         * Sets a Vector3 on a uniform variable.
         * @param uniformName Name of the variable.
         * @param vector3 Value to be set.
         */
        setVector3(uniformName: string, vector3: IVector3Like): void;
        /**
         * Sets a float3 on a uniform variable.
         * @param uniformName Name of the variable.
         * @param x First float in float3.
         * @param y Second float in float3.
         * @param z Third float in float3.
         */
        setFloat3(uniformName: string, x: number, y: number, z: number): void;
        /**
         * Sets a Vector4 on a uniform variable.
         * @param uniformName Name of the variable.
         * @param vector4 Value to be set.
         */
        setVector4(uniformName: string, vector4: IVector4Like): void;
        /**
         * Sets a Quaternion on a uniform variable.
         * @param uniformName Name of the variable.
         * @param quaternion Value to be set.
         */
        setQuaternion(uniformName: string, quaternion: IQuaternionLike): void;
        /**
         * Sets a float4 on a uniform variable.
         * @param uniformName Name of the variable.
         * @param x First float in float4.
         * @param y Second float in float4.
         * @param z Third float in float4.
         * @param w Fourth float in float4.
         */
        setFloat4(uniformName: string, x: number, y: number, z: number, w: number): void;
        /**
         * Sets a Color3 on a uniform variable.
         * @param uniformName Name of the variable.
         * @param color3 Value to be set.
         */
        setColor3(uniformName: string, color3: IColor3Like): void;
        /**
         * Sets a Color4 on a uniform variable.
         * @param uniformName Name of the variable.
         * @param color3 Value to be set.
         * @param alpha Alpha value to be set.
         */
        setColor4(uniformName: string, color3: IColor3Like, alpha: number): void;
        /**
         * Sets a Color4 on a uniform variable
         * @param uniformName defines the name of the variable
         * @param color4 defines the value to be set
         */
        setDirectColor4(uniformName: string, color4: IColor4Like): void;
    }




    /** @internal */
    export type NativeData = Uint32Array;
    /** @internal */
    export class NativeDataStream {
        private readonly _uint32s;
        private readonly _int32s;
        private readonly _float32s;
        private readonly _length;
        private _position;
        private readonly _nativeDataStream;
        private static readonly DEFAULT_BUFFER_SIZE;
        constructor();
        writeUint32(value: number): void;
        writeInt32(value: number): void;
        writeFloat32(value: number): void;
        writeUint32Array(values: Uint32Array): void;
        writeInt32Array(values: Int32Array): void;
        writeFloat32Array(values: Float32Array): void;
        writeNativeData(handle: NativeData): void;
        writeBoolean(value: boolean): void;
        private _flushIfNecessary;
        private _flush;
    }


    /** @internal */
    export class NativeHardwareTexture implements HardwareTextureWrapper {
        private readonly _engine;
        private _nativeTexture;
        get underlyingResource(): Nullable<NativeTexture>;
        constructor(existingTexture: NativeTexture, engine: INativeEngine);
        setUsage(): void;
        set(hardwareTexture: NativeTexture): void;
        reset(): void;
        release(): void;
    }


    export type NativeTexture = NativeData;
    export type NativeFramebuffer = NativeData;
    export type NativeVertexArrayObject = NativeData;
    export type NativeProgram = NativeData;
    export type NativeUniform = NativeData;
    /** @internal */
    export interface INativeEngine {
        dispose(): void;
        requestAnimationFrame(callback: () => void): void;
        createVertexArray(): NativeData;
        createIndexBuffer(bytes: ArrayBuffer, byteOffset: number, byteLength: number, is32Bits: boolean, dynamic: boolean): NativeData;
        recordIndexBuffer(vertexArray: NativeData, indexBuffer: NativeData): void;
        updateDynamicIndexBuffer(buffer: NativeData, bytes: ArrayBuffer, byteOffset: number, byteLength: number, startIndex: number): void;
        createVertexBuffer(bytes: ArrayBuffer, byteOffset: number, byteLength: number, dynamic: boolean): NativeData;
        recordVertexBuffer(vertexArray: NativeData, vertexBuffer: NativeData, location: number, byteOffset: number, byteStride: number, numElements: number, type: number, normalized: boolean, instanceDivisor: number): void;
        updateDynamicVertexBuffer(vertexBuffer: NativeData, bytes: ArrayBuffer, byteOffset: number, byteLength: number): void;
        createProgram(vertexShader: string, fragmentShader: string): NativeProgram;
        getUniforms(shaderProgram: NativeProgram, uniformsNames: string[]): WebGLUniformLocation[];
        getAttributes(shaderProgram: NativeProgram, attributeNames: string[]): number[];
        createTexture(): NativeTexture;
        initializeTexture(texture: NativeTexture, width: number, height: number, hasMips: boolean, format: number, renderTarget: boolean, srgb: boolean): void;
        loadTexture(texture: NativeTexture, data: ArrayBufferView, generateMips: boolean, invertY: boolean, srgb: boolean, onSuccess: () => void, onError: () => void): void;
        loadRawTexture(texture: NativeTexture, data: ArrayBufferView, width: number, height: number, format: number, generateMips: boolean, invertY: boolean): void;
        loadRawTexture2DArray(texture: NativeTexture, data: Nullable<ArrayBufferView>, width: number, height: number, depth: number, format: number, generateMipMaps: boolean, invertY: boolean): void;
        loadCubeTexture(texture: NativeTexture, data: Array<ArrayBufferView>, generateMips: boolean, invertY: boolean, srgb: boolean, onSuccess: () => void, onError: () => void): void;
        loadCubeTextureWithMips(texture: NativeTexture, data: Array<Array<ArrayBufferView>>, invertY: boolean, srgb: boolean, onSuccess: () => void, onError: () => void): void;
        getTextureWidth(texture: NativeTexture): number;
        getTextureHeight(texture: NativeTexture): number;
        copyTexture(desination: NativeTexture, source: NativeTexture): void;
        deleteTexture(texture: NativeTexture): void;
        readTexture(texture: NativeTexture, mipLevel: number, x: number, y: number, width: number, height: number, buffer: Nullable<ArrayBuffer>, bufferOffset: number, bufferLength: number): Promise<ArrayBuffer>;
        createImageBitmap(data: ArrayBufferView | IImage): ImageBitmap;
        resizeImageBitmap(image: ImageBitmap, bufferWidth: number, bufferHeight: number): Uint8Array;
        createFrameBuffer(texture: Nullable<NativeTexture>, width: number, height: number, generateStencilBuffer: boolean, generateDepthBuffer: boolean): NativeFramebuffer;
        getRenderWidth(): number;
        getRenderHeight(): number;
        getHardwareScalingLevel(): number;
        setHardwareScalingLevel(level: number): void;
        setViewPort(x: number, y: number, width: number, height: number): void;
        setCommandDataStream(dataStream: NativeDataStream): void;
        submitCommands(): void;
    }
    /** @internal */
    interface INativeEngineConstructor {
        prototype: INativeEngine;
        new (): INativeEngine;
        readonly PROTOCOL_VERSION: number;
        readonly CAPS_LIMITS_MAX_TEXTURE_SIZE: number;
        readonly CAPS_LIMITS_MAX_TEXTURE_LAYERS: number;
        readonly TEXTURE_NEAREST_NEAREST: number;
        readonly TEXTURE_LINEAR_LINEAR: number;
        readonly TEXTURE_LINEAR_LINEAR_MIPLINEAR: number;
        readonly TEXTURE_NEAREST_NEAREST_MIPNEAREST: number;
        readonly TEXTURE_NEAREST_LINEAR_MIPNEAREST: number;
        readonly TEXTURE_NEAREST_LINEAR_MIPLINEAR: number;
        readonly TEXTURE_NEAREST_LINEAR: number;
        readonly TEXTURE_NEAREST_NEAREST_MIPLINEAR: number;
        readonly TEXTURE_LINEAR_NEAREST_MIPNEAREST: number;
        readonly TEXTURE_LINEAR_NEAREST_MIPLINEAR: number;
        readonly TEXTURE_LINEAR_LINEAR_MIPNEAREST: number;
        readonly TEXTURE_LINEAR_NEAREST: number;
        readonly DEPTH_TEST_LESS: number;
        readonly DEPTH_TEST_LEQUAL: number;
        readonly DEPTH_TEST_EQUAL: number;
        readonly DEPTH_TEST_GEQUAL: number;
        readonly DEPTH_TEST_GREATER: number;
        readonly DEPTH_TEST_NOTEQUAL: number;
        readonly DEPTH_TEST_NEVER: number;
        readonly DEPTH_TEST_ALWAYS: number;
        readonly ADDRESS_MODE_WRAP: number;
        readonly ADDRESS_MODE_MIRROR: number;
        readonly ADDRESS_MODE_CLAMP: number;
        readonly ADDRESS_MODE_BORDER: number;
        readonly ADDRESS_MODE_MIRROR_ONCE: number;
        readonly TEXTURE_FORMAT_RGB8: number;
        readonly TEXTURE_FORMAT_RGBA8: number;
        readonly TEXTURE_FORMAT_RGBA16F: number;
        readonly TEXTURE_FORMAT_RGBA32F: number;
        readonly ATTRIB_TYPE_INT8: number;
        readonly ATTRIB_TYPE_UINT8: number;
        readonly ATTRIB_TYPE_INT16: number;
        readonly ATTRIB_TYPE_UINT16: number;
        readonly ATTRIB_TYPE_FLOAT: number;
        readonly ALPHA_DISABLE: number;
        readonly ALPHA_ADD: number;
        readonly ALPHA_COMBINE: number;
        readonly ALPHA_SUBTRACT: number;
        readonly ALPHA_MULTIPLY: number;
        readonly ALPHA_MAXIMIZED: number;
        readonly ALPHA_ONEONE: number;
        readonly ALPHA_PREMULTIPLIED: number;
        readonly ALPHA_PREMULTIPLIED_PORTERDUFF: number;
        readonly ALPHA_INTERPOLATE: number;
        readonly ALPHA_SCREENMODE: number;
        readonly STENCIL_TEST_LESS: number;
        readonly STENCIL_TEST_LEQUAL: number;
        readonly STENCIL_TEST_EQUAL: number;
        readonly STENCIL_TEST_GEQUAL: number;
        readonly STENCIL_TEST_GREATER: number;
        readonly STENCIL_TEST_NOTEQUAL: number;
        readonly STENCIL_TEST_NEVER: number;
        readonly STENCIL_TEST_ALWAYS: number;
        readonly STENCIL_OP_FAIL_S_ZERO: number;
        readonly STENCIL_OP_FAIL_S_KEEP: number;
        readonly STENCIL_OP_FAIL_S_REPLACE: number;
        readonly STENCIL_OP_FAIL_S_INCR: number;
        readonly STENCIL_OP_FAIL_S_INCRSAT: number;
        readonly STENCIL_OP_FAIL_S_DECR: number;
        readonly STENCIL_OP_FAIL_S_DECRSAT: number;
        readonly STENCIL_OP_FAIL_S_INVERT: number;
        readonly STENCIL_OP_FAIL_Z_ZERO: number;
        readonly STENCIL_OP_FAIL_Z_KEEP: number;
        readonly STENCIL_OP_FAIL_Z_REPLACE: number;
        readonly STENCIL_OP_FAIL_Z_INCR: number;
        readonly STENCIL_OP_FAIL_Z_INCRSAT: number;
        readonly STENCIL_OP_FAIL_Z_DECR: number;
        readonly STENCIL_OP_FAIL_Z_DECRSAT: number;
        readonly STENCIL_OP_FAIL_Z_INVERT: number;
        readonly STENCIL_OP_PASS_Z_ZERO: number;
        readonly STENCIL_OP_PASS_Z_KEEP: number;
        readonly STENCIL_OP_PASS_Z_REPLACE: number;
        readonly STENCIL_OP_PASS_Z_INCR: number;
        readonly STENCIL_OP_PASS_Z_INCRSAT: number;
        readonly STENCIL_OP_PASS_Z_DECR: number;
        readonly STENCIL_OP_PASS_Z_DECRSAT: number;
        readonly STENCIL_OP_PASS_Z_INVERT: number;
        readonly COMMAND_DELETEVERTEXARRAY: NativeData;
        readonly COMMAND_DELETEINDEXBUFFER: NativeData;
        readonly COMMAND_DELETEVERTEXBUFFER: NativeData;
        readonly COMMAND_SETPROGRAM: NativeData;
        readonly COMMAND_SETMATRIX: NativeData;
        readonly COMMAND_SETMATRIX3X3: NativeData;
        readonly COMMAND_SETMATRIX2X2: NativeData;
        readonly COMMAND_SETMATRICES: NativeData;
        readonly COMMAND_SETINT: NativeData;
        readonly COMMAND_SETINTARRAY: NativeData;
        readonly COMMAND_SETINTARRAY2: NativeData;
        readonly COMMAND_SETINTARRAY3: NativeData;
        readonly COMMAND_SETINTARRAY4: NativeData;
        readonly COMMAND_SETFLOATARRAY: NativeData;
        readonly COMMAND_SETFLOATARRAY2: NativeData;
        readonly COMMAND_SETFLOATARRAY3: NativeData;
        readonly COMMAND_SETFLOATARRAY4: NativeData;
        readonly COMMAND_SETTEXTURESAMPLING: NativeData;
        readonly COMMAND_SETTEXTUREWRAPMODE: NativeData;
        readonly COMMAND_SETTEXTUREANISOTROPICLEVEL: NativeData;
        readonly COMMAND_SETTEXTURE: NativeData;
        readonly COMMAND_BINDVERTEXARRAY: NativeData;
        readonly COMMAND_SETSTATE: NativeData;
        readonly COMMAND_DELETEPROGRAM: NativeData;
        readonly COMMAND_SETZOFFSET: NativeData;
        readonly COMMAND_SETZOFFSETUNITS: NativeData;
        readonly COMMAND_SETDEPTHTEST: NativeData;
        readonly COMMAND_SETDEPTHWRITE: NativeData;
        readonly COMMAND_SETCOLORWRITE: NativeData;
        readonly COMMAND_SETBLENDMODE: NativeData;
        readonly COMMAND_SETFLOAT: NativeData;
        readonly COMMAND_SETFLOAT2: NativeData;
        readonly COMMAND_SETFLOAT3: NativeData;
        readonly COMMAND_SETFLOAT4: NativeData;
        readonly COMMAND_BINDFRAMEBUFFER: NativeData;
        readonly COMMAND_UNBINDFRAMEBUFFER: NativeData;
        readonly COMMAND_DELETEFRAMEBUFFER: NativeData;
        readonly COMMAND_DRAWINDEXED: NativeData;
        readonly COMMAND_DRAW: NativeData;
        readonly COMMAND_CLEAR: NativeData;
        readonly COMMAND_SETSTENCIL: NativeData;
        readonly COMMAND_SETVIEWPORT: NativeData;
    }
    /** @internal */
    export interface INativeCamera {
        createVideo(constraints: MediaTrackConstraints): any;
        updateVideoTexture(texture: Nullable<InternalTexture>, video: HTMLVideoElement, invertY: boolean): void;
    }
    /** @internal */
    interface INativeCameraConstructor {
        prototype: INativeCamera;
        new (): INativeCamera;
    }
    /** @internal */
    interface INativeCanvasConstructor {
        prototype: ICanvas;
        new (): ICanvas;
        loadTTFAsync(fontName: string, buffer: ArrayBuffer): void;
    }
    /** @internal */
    interface INativeImageConstructor {
        prototype: IImage;
        new (): IImage;
    }
    /** @internal */
    interface IDeviceInputSystemConstructor {
        prototype: IDeviceInputSystem;
        new (onDeviceConnected: (deviceType: DeviceType, deviceSlot: number) => void, onDeviceDisconnected: (deviceType: DeviceType, deviceSlot: number) => void, onInputChanged: (deviceType: DeviceType, deviceSlot: number, inputIndex: number, currentState: number) => void): IDeviceInputSystem;
    }
    /** @internal */
    export interface INativeDataStream {
        writeBuffer(buffer: ArrayBuffer, length: number): void;
    }
    /** @internal */
    interface INativeDataStreamConstructor {
        prototype: INativeDataStream;
        new (requestFlushCallback: () => void): INativeDataStream;
        readonly VALIDATION_ENABLED: boolean;
        readonly VALIDATION_UINT_32: number;
        readonly VALIDATION_INT_32: number;
        readonly VALIDATION_FLOAT_32: number;
        readonly VALIDATION_UINT_32_ARRAY: number;
        readonly VALIDATION_INT_32_ARRAY: number;
        readonly VALIDATION_FLOAT_32_ARRAY: number;
        readonly VALIDATION_NATIVE_DATA: number;
        readonly VALIDATION_BOOLEAN: number;
    }
    /** @internal */
    export interface INative {
        Engine: INativeEngineConstructor;
        Camera: INativeCameraConstructor;
        Canvas: INativeCanvasConstructor;
        Image: INativeImageConstructor;
        XMLHttpRequest: any;
        DeviceInputSystem: IDeviceInputSystemConstructor;
        NativeDataStream: INativeDataStreamConstructor;
    }


    export class NativePipelineContext implements IPipelineContext {
        isAsync: boolean;
        isReady: boolean;
        _getVertexShaderCode(): string | null;
        _getFragmentShaderCode(): string | null;
        _handlesSpectorRebuildCallback(onCompiled: (compiledObject: any) => void): void;
        nativeProgram: any;
        private _engine;
        private _valueCache;
        private _uniforms;
        constructor(engine: NativeEngine);
        _fillEffectInformation(effect: Effect, uniformBuffersNames: {
            [key: string]: number;
        }, uniformsNames: string[], uniforms: {
            [key: string]: Nullable<WebGLUniformLocation>;
        }, samplerList: string[], samplers: {
            [key: string]: number;
        }, attributesNames: string[], attributes: number[]): void;
        /**
         * Release all associated resources.
         **/
        dispose(): void;
        /**
         * @internal
         */
        _cacheMatrix(uniformName: string, matrix: IMatrixLike): boolean;
        /**
         * @internal
         */
        _cacheFloat2(uniformName: string, x: number, y: number): boolean;
        /**
         * @internal
         */
        _cacheFloat3(uniformName: string, x: number, y: number, z: number): boolean;
        /**
         * @internal
         */
        _cacheFloat4(uniformName: string, x: number, y: number, z: number, w: number): boolean;
        /**
         * Sets an integer value on a uniform variable.
         * @param uniformName Name of the variable.
         * @param value Value to be set.
         */
        setInt(uniformName: string, value: number): void;
        /**
         * Sets a int2 on a uniform variable.
         * @param uniformName Name of the variable.
         * @param x First int in int2.
         * @param y Second int in int2.
         */
        setInt2(uniformName: string, x: number, y: number): void;
        /**
         * Sets a int3 on a uniform variable.
         * @param uniformName Name of the variable.
         * @param x First int in int3.
         * @param y Second int in int3.
         * @param z Third int in int3.
         */
        setInt3(uniformName: string, x: number, y: number, z: number): void;
        /**
         * Sets a int4 on a uniform variable.
         * @param uniformName Name of the variable.
         * @param x First int in int4.
         * @param y Second int in int4.
         * @param z Third int in int4.
         * @param w Fourth int in int4.
         */
        setInt4(uniformName: string, x: number, y: number, z: number, w: number): void;
        /**
         * Sets an int array on a uniform variable.
         * @param uniformName Name of the variable.
         * @param array array to be set.
         */
        setIntArray(uniformName: string, array: Int32Array): void;
        /**
         * Sets an int array 2 on a uniform variable. (Array is specified as single array eg. [1,2,3,4] will result in [[1,2],[3,4]] in the shader)
         * @param uniformName Name of the variable.
         * @param array array to be set.
         */
        setIntArray2(uniformName: string, array: Int32Array): void;
        /**
         * Sets an int array 3 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6] will result in [[1,2,3],[4,5,6]] in the shader)
         * @param uniformName Name of the variable.
         * @param array array to be set.
         */
        setIntArray3(uniformName: string, array: Int32Array): void;
        /**
         * Sets an int array 4 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6,7,8] will result in [[1,2,3,4],[5,6,7,8]] in the shader)
         * @param uniformName Name of the variable.
         * @param array array to be set.
         */
        setIntArray4(uniformName: string, array: Int32Array): void;
        /**
         * Sets an unsigned integer value on a uniform variable.
         * @param uniformName Name of the variable.
         * @param value Value to be set.
         */
        setUInt(uniformName: string, value: number): void;
        /**
         * Sets a unsigned int2 on a uniform variable.
         * @param uniformName Name of the variable.
         * @param x First unsigned int in uint2.
         * @param y Second unsigned int in uint2.
         */
        setUInt2(uniformName: string, x: number, y: number): void;
        /**
         * Sets a unsigned int3 on a uniform variable.
         * @param uniformName Name of the variable.
         * @param x First unsigned int in uint3.
         * @param y Second unsigned int in uint3.
         * @param z Third unsigned int in uint3.
         */
        setUInt3(uniformName: string, x: number, y: number, z: number): void;
        /**
         * Sets a unsigned int4 on a uniform variable.
         * @param uniformName Name of the variable.
         * @param x First unsigned int in uint4.
         * @param y Second unsigned int in uint4.
         * @param z Third unsigned int in uint4.
         * @param w Fourth unsigned int in uint4.
         */
        setUInt4(uniformName: string, x: number, y: number, z: number, w: number): void;
        /**
         * Sets an unsigned int array on a uniform variable.
         * @param uniformName Name of the variable.
         * @param array array to be set.
         */
        setUIntArray(uniformName: string, array: Uint32Array): void;
        /**
         * Sets an unsigned int array 2 on a uniform variable. (Array is specified as single array eg. [1,2,3,4] will result in [[1,2],[3,4]] in the shader)
         * @param uniformName Name of the variable.
         * @param array array to be set.
         */
        setUIntArray2(uniformName: string, array: Uint32Array): void;
        /**
         * Sets an unsigned int array 3 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6] will result in [[1,2,3],[4,5,6]] in the shader)
         * @param uniformName Name of the variable.
         * @param array array to be set.
         */
        setUIntArray3(uniformName: string, array: Uint32Array): void;
        /**
         * Sets an unsigned int array 4 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6,7,8] will result in [[1,2,3,4],[5,6,7,8]] in the shader)
         * @param uniformName Name of the variable.
         * @param array array to be set.
         */
        setUIntArray4(uniformName: string, array: Uint32Array): void;
        /**
         * Sets an float array on a uniform variable.
         * @param uniformName Name of the variable.
         * @param array array to be set.
         */
        setFloatArray(uniformName: string, array: Float32Array): void;
        /**
         * Sets an float array 2 on a uniform variable. (Array is specified as single array eg. [1,2,3,4] will result in [[1,2],[3,4]] in the shader)
         * @param uniformName Name of the variable.
         * @param array array to be set.
         */
        setFloatArray2(uniformName: string, array: Float32Array): void;
        /**
         * Sets an float array 3 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6] will result in [[1,2,3],[4,5,6]] in the shader)
         * @param uniformName Name of the variable.
         * @param array array to be set.
         */
        setFloatArray3(uniformName: string, array: Float32Array): void;
        /**
         * Sets an float array 4 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6,7,8] will result in [[1,2,3,4],[5,6,7,8]] in the shader)
         * @param uniformName Name of the variable.
         * @param array array to be set.
         */
        setFloatArray4(uniformName: string, array: Float32Array): void;
        /**
         * Sets an array on a uniform variable.
         * @param uniformName Name of the variable.
         * @param array array to be set.
         */
        setArray(uniformName: string, array: number[]): void;
        /**
         * Sets an array 2 on a uniform variable. (Array is specified as single array eg. [1,2,3,4] will result in [[1,2],[3,4]] in the shader)
         * @param uniformName Name of the variable.
         * @param array array to be set.
         */
        setArray2(uniformName: string, array: number[]): void;
        /**
         * Sets an array 3 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6] will result in [[1,2,3],[4,5,6]] in the shader)
         * @param uniformName Name of the variable.
         * @param array array to be set.
         * @returns this effect.
         */
        setArray3(uniformName: string, array: number[]): void;
        /**
         * Sets an array 4 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6,7,8] will result in [[1,2,3,4],[5,6,7,8]] in the shader)
         * @param uniformName Name of the variable.
         * @param array array to be set.
         */
        setArray4(uniformName: string, array: number[]): void;
        /**
         * Sets matrices on a uniform variable.
         * @param uniformName Name of the variable.
         * @param matrices matrices to be set.
         */
        setMatrices(uniformName: string, matrices: Float32Array): void;
        /**
         * Sets matrix on a uniform variable.
         * @param uniformName Name of the variable.
         * @param matrix matrix to be set.
         */
        setMatrix(uniformName: string, matrix: IMatrixLike): void;
        /**
         * Sets a 3x3 matrix on a uniform variable. (Specified as [1,2,3,4,5,6,7,8,9] will result in [1,2,3][4,5,6][7,8,9] matrix)
         * @param uniformName Name of the variable.
         * @param matrix matrix to be set.
         */
        setMatrix3x3(uniformName: string, matrix: Float32Array): void;
        /**
         * Sets a 2x2 matrix on a uniform variable. (Specified as [1,2,3,4] will result in [1,2][3,4] matrix)
         * @param uniformName Name of the variable.
         * @param matrix matrix to be set.
         */
        setMatrix2x2(uniformName: string, matrix: Float32Array): void;
        /**
         * Sets a float on a uniform variable.
         * @param uniformName Name of the variable.
         * @param value value to be set.
         * @returns this effect.
         */
        setFloat(uniformName: string, value: number): void;
        /**
         * Sets a boolean on a uniform variable.
         * @param uniformName Name of the variable.
         * @param bool value to be set.
         */
        setBool(uniformName: string, bool: boolean): void;
        /**
         * Sets a Vector2 on a uniform variable.
         * @param uniformName Name of the variable.
         * @param vector2 vector2 to be set.
         */
        setVector2(uniformName: string, vector2: IVector2Like): void;
        /**
         * Sets a float2 on a uniform variable.
         * @param uniformName Name of the variable.
         * @param x First float in float2.
         * @param y Second float in float2.
         */
        setFloat2(uniformName: string, x: number, y: number): void;
        /**
         * Sets a Vector3 on a uniform variable.
         * @param uniformName Name of the variable.
         * @param vector3 Value to be set.
         */
        setVector3(uniformName: string, vector3: IVector3Like): void;
        /**
         * Sets a float3 on a uniform variable.
         * @param uniformName Name of the variable.
         * @param x First float in float3.
         * @param y Second float in float3.
         * @param z Third float in float3.
         */
        setFloat3(uniformName: string, x: number, y: number, z: number): void;
        /**
         * Sets a Vector4 on a uniform variable.
         * @param uniformName Name of the variable.
         * @param vector4 Value to be set.
         */
        setVector4(uniformName: string, vector4: IVector4Like): void;
        /**
         * Sets a Quaternion on a uniform variable.
         * @param uniformName Name of the variable.
         * @param quaternion Value to be set.
         */
        setQuaternion(uniformName: string, quaternion: IQuaternionLike): void;
        /**
         * Sets a float4 on a uniform variable.
         * @param uniformName Name of the variable.
         * @param x First float in float4.
         * @param y Second float in float4.
         * @param z Third float in float4.
         * @param w Fourth float in float4.
         * @returns this effect.
         */
        setFloat4(uniformName: string, x: number, y: number, z: number, w: number): void;
        /**
         * Sets a Color3 on a uniform variable.
         * @param uniformName Name of the variable.
         * @param color3 Value to be set.
         */
        setColor3(uniformName: string, color3: IColor3Like): void;
        /**
         * Sets a Color4 on a uniform variable.
         * @param uniformName Name of the variable.
         * @param color3 Value to be set.
         * @param alpha Alpha value to be set.
         */
        setColor4(uniformName: string, color3: IColor3Like, alpha: number): void;
        /**
         * Sets a Color4 on a uniform variable
         * @param uniformName defines the name of the variable
         * @param color4 defines the value to be set
         */
        setDirectColor4(uniformName: string, color4: IColor4Like): void;
    }


    export class NativeRenderTargetWrapper extends RenderTargetWrapper {
        readonly _engine: NativeEngine;
        private __framebuffer;
        private __framebufferDepthStencil;
        get _framebuffer(): Nullable<NativeFramebuffer>;
        set _framebuffer(framebuffer: Nullable<NativeFramebuffer>);
        get _framebufferDepthStencil(): Nullable<NativeFramebuffer>;
        set _framebufferDepthStencil(framebufferDepthStencil: Nullable<NativeFramebuffer>);
        constructor(isMulti: boolean, isCube: boolean, size: TextureSize, engine: NativeEngine);
        dispose(disposeOnlyFramebuffers?: boolean): void;
    }


    export class ValidatedNativeDataStream extends NativeDataStream {
        constructor();
        writeUint32(value: number): void;
        writeInt32(value: number): void;
        writeFloat32(value: number): void;
        writeUint32Array(values: Uint32Array): void;
        writeInt32Array(values: Int32Array): void;
        writeFloat32Array(values: Float32Array): void;
        writeNativeData(handle: NativeData): void;
        writeBoolean(value: boolean): void;
    }


    /**
     * Returns _native only after it has been defined by BabylonNative.
     * @internal
     */
    export function AcquireNativeObjectAsync(): Promise<INative>;
    /**
     * Registers a constructor on the _native object. See NativeXRFrame for an example.
     * @internal
     */
    export function RegisterNativeTypeAsync<Type>(typeName: string, constructor: Type): Promise<void>;
    /**
     * Container for accessors for natively-stored mesh data buffers.
     */
    class NativeDataBuffer extends DataBuffer {
        /**
         * Accessor value used to identify/retrieve a natively-stored index buffer.
         */
        nativeIndexBuffer?: NativeData;
        /**
         * Accessor value used to identify/retrieve a natively-stored vertex buffer.
         */
        nativeVertexBuffer?: NativeData;
    }
    /**
     * Options to create the Native engine
     */
    export interface NativeEngineOptions {
        /**
         * defines whether to adapt to the device's viewport characteristics (default: false)
         */
        adaptToDeviceRatio?: boolean;
    }
    /** @internal */
    export class NativeEngine extends Engine {
        private static readonly PROTOCOL_VERSION;
        private readonly _engine;
        private readonly _camera;
        private readonly _commandBufferEncoder;
        private _boundBuffersVertexArray;
        private _currentDepthTest;
        private _stencilTest;
        private _stencilMask;
        private _stencilFunc;
        private _stencilFuncRef;
        private _stencilFuncMask;
        private _stencilOpStencilFail;
        private _stencilOpDepthFail;
        private _stencilOpStencilDepthPass;
        private _zOffset;
        private _zOffsetUnits;
        private _depthWrite;
        getHardwareScalingLevel(): number;
        setHardwareScalingLevel(level: number): void;
        constructor(options?: NativeEngineOptions);
        dispose(): void;
        /** @internal */
        static _createNativeDataStream(): NativeDataStream;
        /**
         * Can be used to override the current requestAnimationFrame requester.
         * @internal
         */
        protected _queueNewFrame(bindedRenderFunction: any, requester?: any): number;
        /**
         * Override default engine behavior.
         * @param framebuffer
         */
        _bindUnboundFramebuffer(framebuffer: Nullable<WebGLFramebuffer>): void;
        /**
         * Gets host document
         * @returns the host document object
         */
        getHostDocument(): Nullable<Document>;
        clear(color: Nullable<IColor4Like>, backBuffer: boolean, depth: boolean, stencil?: boolean): void;
        createIndexBuffer(indices: IndicesArray, updateable?: boolean): NativeDataBuffer;
        createVertexBuffer(vertices: DataArray, updateable?: boolean): NativeDataBuffer;
        protected _recordVertexArrayObject(vertexArray: any, vertexBuffers: {
            [key: string]: VertexBuffer;
        }, indexBuffer: Nullable<NativeDataBuffer>, effect: Effect, overrideVertexBuffers?: {
            [kind: string]: Nullable<VertexBuffer>;
        }): void;
        bindBuffers(vertexBuffers: {
            [key: string]: VertexBuffer;
        }, indexBuffer: Nullable<NativeDataBuffer>, effect: Effect): void;
        recordVertexArrayObject(vertexBuffers: {
            [key: string]: VertexBuffer;
        }, indexBuffer: Nullable<NativeDataBuffer>, effect: Effect, overrideVertexBuffers?: {
            [kind: string]: Nullable<VertexBuffer>;
        }): WebGLVertexArrayObject;
        private _deleteVertexArray;
        bindVertexArrayObject(vertexArray: WebGLVertexArrayObject): void;
        releaseVertexArrayObject(vertexArray: WebGLVertexArrayObject): void;
        getAttributes(pipelineContext: IPipelineContext, attributesNames: string[]): number[];
        /**
         * Draw a list of indexed primitives
         * @param fillMode defines the primitive to use
         * @param indexStart defines the starting index
         * @param indexCount defines the number of index to draw
         * @param instancesCount defines the number of instances to draw (if instantiation is enabled)
         */
        drawElementsType(fillMode: number, indexStart: number, indexCount: number, instancesCount?: number): void;
        /**
         * Draw a list of unindexed primitives
         * @param fillMode defines the primitive to use
         * @param verticesStart defines the index of first vertex to draw
         * @param verticesCount defines the count of vertices to draw
         * @param instancesCount defines the number of instances to draw (if instantiation is enabled)
         */
        drawArraysType(fillMode: number, verticesStart: number, verticesCount: number, instancesCount?: number): void;
        createPipelineContext(): IPipelineContext;
        createMaterialContext(): IMaterialContext | undefined;
        createDrawContext(): IDrawContext | undefined;
        _preparePipelineContext(pipelineContext: IPipelineContext, vertexSourceCode: string, fragmentSourceCode: string, createAsRaw: boolean, _rawVertexSourceCode: string, _rawFragmentSourceCode: string, _rebuildRebind: any, defines: Nullable<string>): void;
        /**
         * @internal
         */
        _isRenderingStateCompiled(pipelineContext: IPipelineContext): boolean;
        /**
         * @internal
         */
        _executeWhenRenderingStateIsCompiled(pipelineContext: IPipelineContext, action: () => void): void;
        createRawShaderProgram(): WebGLProgram;
        createShaderProgram(_pipelineContext: IPipelineContext, vertexCode: string, fragmentCode: string, defines: Nullable<string>): WebGLProgram;
        /**
         * Inline functions in shader code that are marked to be inlined
         * @param code code to inline
         * @returns inlined code
         */
        inlineShaderCode(code: string): string;
        protected _setProgram(program: WebGLProgram): void;
        _deletePipelineContext(pipelineContext: IPipelineContext): void;
        getUniforms(pipelineContext: IPipelineContext, uniformsNames: string[]): WebGLUniformLocation[];
        bindUniformBlock(pipelineContext: IPipelineContext, blockName: string, index: number): void;
        bindSamplers(effect: Effect): void;
        getRenderWidth(useScreen?: boolean): number;
        getRenderHeight(useScreen?: boolean): number;
        setViewport(viewport: IViewportLike, requiredWidth?: number, requiredHeight?: number): void;
        setState(culling: boolean, zOffset?: number, force?: boolean, reverseSide?: boolean, cullBackFaces?: boolean, stencil?: IStencilState, zOffsetUnits?: number): void;
        /**
         * Gets the client rect of native canvas.  Needed for InputManager.
         * @returns a client rectangle
         */
        getInputElementClientRect(): Nullable<DOMRect>;
        /**
         * Set the z offset Factor to apply to current rendering
         * @param value defines the offset to apply
         */
        setZOffset(value: number): void;
        /**
         * Gets the current value of the zOffset Factor
         * @returns the current zOffset Factor state
         */
        getZOffset(): number;
        /**
         * Set the z offset Units to apply to current rendering
         * @param value defines the offset to apply
         */
        setZOffsetUnits(value: number): void;
        /**
         * Gets the current value of the zOffset Units
         * @returns the current zOffset Units state
         */
        getZOffsetUnits(): number;
        /**
         * Enable or disable depth buffering
         * @param enable defines the state to set
         */
        setDepthBuffer(enable: boolean): void;
        /**
         * Gets a boolean indicating if depth writing is enabled
         * @returns the current depth writing state
         */
        getDepthWrite(): boolean;
        getDepthFunction(): Nullable<number>;
        setDepthFunction(depthFunc: number): void;
        /**
         * Enable or disable depth writing
         * @param enable defines the state to set
         */
        setDepthWrite(enable: boolean): void;
        /**
         * Enable or disable color writing
         * @param enable defines the state to set
         */
        setColorWrite(enable: boolean): void;
        /**
         * Gets a boolean indicating if color writing is enabled
         * @returns the current color writing state
         */
        getColorWrite(): boolean;
        private applyStencil;
        private _setStencil;
        /**
         * Enable or disable the stencil buffer
         * @param enable defines if the stencil buffer must be enabled or disabled
         */
        setStencilBuffer(enable: boolean): void;
        /**
         * Gets a boolean indicating if stencil buffer is enabled
         * @returns the current stencil buffer state
         */
        getStencilBuffer(): boolean;
        /**
         * Gets the current stencil operation when stencil passes
         * @returns a number defining stencil operation to use when stencil passes
         */
        getStencilOperationPass(): number;
        /**
         * Sets the stencil operation to use when stencil passes
         * @param operation defines the stencil operation to use when stencil passes
         */
        setStencilOperationPass(operation: number): void;
        /**
         * Sets the current stencil mask
         * @param mask defines the new stencil mask to use
         */
        setStencilMask(mask: number): void;
        /**
         * Sets the current stencil function
         * @param stencilFunc defines the new stencil function to use
         */
        setStencilFunction(stencilFunc: number): void;
        /**
         * Sets the current stencil reference
         * @param reference defines the new stencil reference to use
         */
        setStencilFunctionReference(reference: number): void;
        /**
         * Sets the current stencil mask
         * @param mask defines the new stencil mask to use
         */
        setStencilFunctionMask(mask: number): void;
        /**
         * Sets the stencil operation to use when stencil fails
         * @param operation defines the stencil operation to use when stencil fails
         */
        setStencilOperationFail(operation: number): void;
        /**
         * Sets the stencil operation to use when depth fails
         * @param operation defines the stencil operation to use when depth fails
         */
        setStencilOperationDepthFail(operation: number): void;
        /**
         * Gets the current stencil mask
         * @returns a number defining the new stencil mask to use
         */
        getStencilMask(): number;
        /**
         * Gets the current stencil function
         * @returns a number defining the stencil function to use
         */
        getStencilFunction(): number;
        /**
         * Gets the current stencil reference value
         * @returns a number defining the stencil reference value to use
         */
        getStencilFunctionReference(): number;
        /**
         * Gets the current stencil mask
         * @returns a number defining the stencil mask to use
         */
        getStencilFunctionMask(): number;
        /**
         * Gets the current stencil operation when stencil fails
         * @returns a number defining stencil operation to use when stencil fails
         */
        getStencilOperationFail(): number;
        /**
         * Gets the current stencil operation when depth fails
         * @returns a number defining stencil operation to use when depth fails
         */
        getStencilOperationDepthFail(): number;
        /**
         * Sets alpha constants used by some alpha blending modes
         * @param r defines the red component
         * @param g defines the green component
         * @param b defines the blue component
         * @param a defines the alpha component
         */
        setAlphaConstants(r: number, g: number, b: number, a: number): void;
        /**
         * Sets the current alpha mode
         * @param mode defines the mode to use (one of the BABYLON.Constants.ALPHA_XXX)
         * @param noDepthWriteChange defines if depth writing state should remains unchanged (false by default)
         * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/advanced/transparent_rendering
         */
        setAlphaMode(mode: number, noDepthWriteChange?: boolean): void;
        /**
         * Gets the current alpha mode
         * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/advanced/transparent_rendering
         * @returns the current alpha mode
         */
        getAlphaMode(): number;
        setInt(uniform: WebGLUniformLocation, int: number): boolean;
        setIntArray(uniform: WebGLUniformLocation, array: Int32Array): boolean;
        setIntArray2(uniform: WebGLUniformLocation, array: Int32Array): boolean;
        setIntArray3(uniform: WebGLUniformLocation, array: Int32Array): boolean;
        setIntArray4(uniform: WebGLUniformLocation, array: Int32Array): boolean;
        setFloatArray(uniform: WebGLUniformLocation, array: Float32Array): boolean;
        setFloatArray2(uniform: WebGLUniformLocation, array: Float32Array): boolean;
        setFloatArray3(uniform: WebGLUniformLocation, array: Float32Array): boolean;
        setFloatArray4(uniform: WebGLUniformLocation, array: Float32Array): boolean;
        setArray(uniform: WebGLUniformLocation, array: number[]): boolean;
        setArray2(uniform: WebGLUniformLocation, array: number[]): boolean;
        setArray3(uniform: WebGLUniformLocation, array: number[]): boolean;
        setArray4(uniform: WebGLUniformLocation, array: number[]): boolean;
        setMatrices(uniform: WebGLUniformLocation, matrices: Float32Array): boolean;
        setMatrix3x3(uniform: WebGLUniformLocation, matrix: Float32Array): boolean;
        setMatrix2x2(uniform: WebGLUniformLocation, matrix: Float32Array): boolean;
        setFloat(uniform: WebGLUniformLocation, value: number): boolean;
        setFloat2(uniform: WebGLUniformLocation, x: number, y: number): boolean;
        setFloat3(uniform: WebGLUniformLocation, x: number, y: number, z: number): boolean;
        setFloat4(uniform: WebGLUniformLocation, x: number, y: number, z: number, w: number): boolean;
        setColor3(uniform: WebGLUniformLocation, color3: IColor3Like): boolean;
        setColor4(uniform: WebGLUniformLocation, color3: IColor3Like, alpha: number): boolean;
        wipeCaches(bruteForce?: boolean): void;
        protected _createTexture(): WebGLTexture;
        protected _deleteTexture(texture: Nullable<WebGLTexture>): void;
        /**
         * Update the content of a dynamic texture
         * @param texture defines the texture to update
         * @param canvas defines the canvas containing the source
         * @param invertY defines if data must be stored with Y axis inverted
         * @param premulAlpha defines if alpha is stored as premultiplied
         * @param format defines the format of the data
         */
        updateDynamicTexture(texture: Nullable<InternalTexture>, canvas: any, invertY: boolean, premulAlpha?: boolean, format?: number): void;
        createDynamicTexture(width: number, height: number, generateMipMaps: boolean, samplingMode: number): InternalTexture;
        createVideoElement(constraints: MediaTrackConstraints): any;
        updateVideoTexture(texture: Nullable<InternalTexture>, video: HTMLVideoElement, invertY: boolean): void;
        createRawTexture(data: Nullable<ArrayBufferView>, width: number, height: number, format: number, generateMipMaps: boolean, invertY: boolean, samplingMode: number, compression?: Nullable<string>, type?: number, creationFlags?: number, useSRGBBuffer?: boolean): InternalTexture;
        createRawTexture2DArray(data: Nullable<ArrayBufferView>, width: number, height: number, depth: number, format: number, generateMipMaps: boolean, invertY: boolean, samplingMode: number, compression?: Nullable<string>, textureType?: number): InternalTexture;
        updateRawTexture(texture: Nullable<InternalTexture>, bufferView: Nullable<ArrayBufferView>, format: number, invertY: boolean, compression?: Nullable<string>, type?: number, useSRGBBuffer?: boolean): void;
        /**
         * Usually called from Texture.ts.
         * Passed information to create a NativeTexture
         * @param url defines a value which contains one of the following:
         * * A conventional http URL, e.g. 'http://...' or 'file://...'
         * * A base64 string of in-line texture data, e.g. 'data:image/jpg;base64,/...'
         * * An indicator that data being passed using the buffer parameter, e.g. 'data:mytexture.jpg'
         * @param noMipmap defines a boolean indicating that no mipmaps shall be generated.  Ignored for compressed textures.  They must be in the file
         * @param invertY when true, image is flipped when loaded.  You probably want true. Certain compressed textures may invert this if their default is inverted (eg. ktx)
         * @param scene needed for loading to the correct scene
         * @param samplingMode mode with should be used sample / access the texture (Default: Texture.TRILINEAR_SAMPLINGMODE)
         * @param onLoad optional callback to be called upon successful completion
         * @param onError optional callback to be called upon failure
         * @param buffer a source of a file previously fetched as either a base64 string, an ArrayBuffer (compressed or image format), HTMLImageElement (image format), or a Blob
         * @param fallback an internal argument in case the function must be called again, due to etc1 not having alpha capabilities
         * @param format internal format.  Default: RGB when extension is '.jpg' else RGBA.  Ignored for compressed textures
         * @param forcedExtension defines the extension to use to pick the right loader
         * @param mimeType defines an optional mime type
         * @param loaderOptions options to be passed to the loader
         * @param creationFlags specific flags to use when creating the texture (Constants.TEXTURE_CREATIONFLAG_STORAGE for storage textures, for eg)
         * @param useSRGBBuffer defines if the texture must be loaded in a sRGB GPU buffer (if supported by the GPU).
         * @returns a InternalTexture for assignment back into BABYLON.Texture
         */
        createTexture(url: Nullable<string>, noMipmap: boolean, invertY: boolean, scene: Nullable<ISceneLike>, samplingMode?: number, onLoad?: Nullable<(texture: InternalTexture) => void>, onError?: Nullable<(message: string, exception: any) => void>, buffer?: Nullable<string | ArrayBuffer | ArrayBufferView | HTMLImageElement | Blob | ImageBitmap>, fallback?: Nullable<InternalTexture>, format?: Nullable<number>, forcedExtension?: Nullable<string>, mimeType?: string, loaderOptions?: any, creationFlags?: number, useSRGBBuffer?: boolean): InternalTexture;
        /**
         * Wraps an external native texture in a Babylon texture.
         * @param texture defines the external texture
         * @returns the babylon internal texture
         */
        wrapNativeTexture(texture: any): InternalTexture;
        /**
         * Wraps an external web gl texture in a Babylon texture.
         * @returns the babylon internal texture
         */
        wrapWebGLTexture(): InternalTexture;
        _createDepthStencilTexture(size: TextureSize, options: DepthTextureCreationOptions, rtWrapper: RenderTargetWrapper): InternalTexture;
        /**
         * @internal
         */
        _releaseFramebufferObjects(framebuffer: Nullable<NativeFramebuffer>): void;
        /**
         * @internal Engine abstraction for loading and creating an image bitmap from a given source string.
         * @param imageSource source to load the image from.
         * @param options An object that sets options for the image's extraction.
         * @returns ImageBitmap
         */
        _createImageBitmapFromSource(imageSource: string, options?: ImageBitmapOptions): Promise<ImageBitmap>;
        /**
         * Engine abstraction for createImageBitmap
         * @param image source for image
         * @param options An object that sets options for the image's extraction.
         * @returns ImageBitmap
         */
        createImageBitmap(image: ImageBitmapSource, options?: ImageBitmapOptions): Promise<ImageBitmap>;
        /**
         * Resize an image and returns the image data as an uint8array
         * @param image image to resize
         * @param bufferWidth destination buffer width
         * @param bufferHeight destination buffer height
         * @returns an uint8array containing RGBA values of bufferWidth * bufferHeight size
         */
        resizeImageBitmap(image: ImageBitmap, bufferWidth: number, bufferHeight: number): Uint8Array;
        /**
         * Creates a cube texture
         * @param rootUrl defines the url where the files to load is located
         * @param scene defines the current scene
         * @param files defines the list of files to load (1 per face)
         * @param noMipmap defines a boolean indicating that no mipmaps shall be generated (false by default)
         * @param onLoad defines an optional callback raised when the texture is loaded
         * @param onError defines an optional callback raised if there is an issue to load the texture
         * @param format defines the format of the data
         * @param forcedExtension defines the extension to use to pick the right loader
         * @param createPolynomials if a polynomial sphere should be created for the cube texture
         * @param lodScale defines the scale applied to environment texture. This manages the range of LOD level used for IBL according to the roughness
         * @param lodOffset defines the offset applied to environment texture. This manages first LOD level used for IBL according to the roughness
         * @param fallback defines texture to use while falling back when (compressed) texture file not found.
         * @param loaderOptions options to be passed to the loader
         * @param useSRGBBuffer defines if the texture must be loaded in a sRGB GPU buffer (if supported by the GPU).
         * @returns the cube texture as an InternalTexture
         */
        createCubeTexture(rootUrl: string, scene: Nullable<Scene>, files: Nullable<string[]>, noMipmap?: boolean, onLoad?: Nullable<(data?: any) => void>, onError?: Nullable<(message?: string, exception?: any) => void>, format?: number, forcedExtension?: any, createPolynomials?: boolean, lodScale?: number, lodOffset?: number, fallback?: Nullable<InternalTexture>, loaderOptions?: any, useSRGBBuffer?: boolean): InternalTexture;
        /** @internal */
        _createHardwareTexture(): HardwareTextureWrapper;
        /** @internal */
        _createHardwareRenderTargetWrapper(isMulti: boolean, isCube: boolean, size: TextureSize): RenderTargetWrapper;
        /** @internal */
        _createInternalTexture(size: TextureSize, options: boolean | InternalTextureCreationOptions, _delayGPUTextureCreation?: boolean, source?: InternalTextureSource): InternalTexture;
        createRenderTargetTexture(size: number | {
            width: number;
            height: number;
        }, options: boolean | RenderTargetCreationOptions): RenderTargetWrapper;
        updateTextureSamplingMode(samplingMode: number, texture: InternalTexture): void;
        bindFramebuffer(texture: RenderTargetWrapper, faceIndex?: number, requiredWidth?: number, requiredHeight?: number, forceFullscreenViewport?: boolean): void;
        unBindFramebuffer(texture: RenderTargetWrapper, disableGenerateMipMaps?: boolean, onBeforeUnbind?: () => void): void;
        createDynamicVertexBuffer(data: DataArray): DataBuffer;
        updateDynamicIndexBuffer(indexBuffer: DataBuffer, indices: IndicesArray, offset?: number): void;
        updateDynamicVertexBuffer(vertexBuffer: DataBuffer, verticies: DataArray, byteOffset?: number, byteLength?: number): void;
        protected _setTexture(channel: number, texture: Nullable<BaseTexture>, isPartOfTextureArray?: boolean, depthStencilTexture?: boolean): boolean;
        private _setTextureSampling;
        private _setTextureWrapMode;
        private _setTextureCore;
        private _updateAnisotropicLevel;
        private _getAddressMode;
        /**
         * @internal
         */
        _bindTexture(channel: number, texture: InternalTexture): void;
        protected _deleteBuffer(buffer: NativeDataBuffer): void;
        /**
         * Create a canvas
         * @param width width
         * @param height height
         * @returns ICanvas interface
         */
        createCanvas(width: number, height: number): ICanvas;
        /**
         * Create an image to use with canvas
         * @returns IImage interface
         */
        createCanvasImage(): IImage;
        /**
         * Update a portion of an internal texture
         * @param texture defines the texture to update
         * @param imageData defines the data to store into the texture
         * @param xOffset defines the x coordinates of the update rectangle
         * @param yOffset defines the y coordinates of the update rectangle
         * @param width defines the width of the update rectangle
         * @param height defines the height of the update rectangle
         * @param faceIndex defines the face index if texture is a cube (0 by default)
         * @param lod defines the lod level to update (0 by default)
         * @param generateMipMaps defines whether to generate mipmaps or not
         */
        updateTextureData(texture: InternalTexture, imageData: ArrayBufferView, xOffset: number, yOffset: number, width: number, height: number, faceIndex?: number, lod?: number, generateMipMaps?: boolean): void;
        /**
         * @internal
         */
        _uploadCompressedDataToTextureDirectly(texture: InternalTexture, internalFormat: number, width: number, height: number, data: ArrayBufferView, faceIndex?: number, lod?: number): void;
        /**
         * @internal
         */
        _uploadDataToTextureDirectly(texture: InternalTexture, imageData: ArrayBufferView, faceIndex?: number, lod?: number): void;
        /**
         * @internal
         */
        _uploadArrayBufferViewToTexture(texture: InternalTexture, imageData: ArrayBufferView, faceIndex?: number, lod?: number): void;
        /**
         * @internal
         */
        _uploadImageToTexture(texture: InternalTexture, image: HTMLImageElement, faceIndex?: number, lod?: number): void;
        private _getNativeSamplingMode;
        private _getStencilFunc;
        private _getStencilOpFail;
        private _getStencilDepthFail;
        private _getStencilDepthPass;
        private _getNativeTextureFormat;
        private _getNativeAlphaMode;
        private _getNativeAttribType;
        getFontOffset(font: string): {
            ascent: number;
            height: number;
            descent: number;
        };
        _readTexturePixels(texture: InternalTexture, width: number, height: number, faceIndex?: number, level?: number, buffer?: Nullable<ArrayBufferView>, _flushRenderer?: boolean, _noDataConversion?: boolean, x?: number, y?: number): Promise<ArrayBufferView>;
    }


    /**
     * Options to create the null engine
     */
    export class NullEngineOptions {
        /**
         * Render width (Default: 512)
         */
        renderWidth: number;
        /**
         * Render height (Default: 256)
         */
        renderHeight: number;
        /**
         * Texture size (Default: 512)
         */
        textureSize: number;
        /**
         * If delta time between frames should be constant
         * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/advanced_animations#deterministic-lockstep
         */
        deterministicLockstep: boolean;
        /**
         * Maximum about of steps between frames (Default: 4)
         * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/advanced_animations#deterministic-lockstep
         */
        lockstepMaxSteps: number;
        /**
         * Make the matrix computations to be performed in 64 bits instead of 32 bits. False by default
         */
        useHighPrecisionMatrix?: boolean;
    }
    /**
     * The null engine class provides support for headless version of babylon.js.
     * This can be used in server side scenario or for testing purposes
     */
    export class NullEngine extends Engine {
        private _options;
        /**
         * Gets a boolean indicating that the engine is running in deterministic lock step mode
         * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/advanced_animations#deterministic-lockstep
         * @returns true if engine is in deterministic lock step mode
         */
        isDeterministicLockStep(): boolean;
        /**
         * Gets the max steps when engine is running in deterministic lock step
         * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/advanced_animations#deterministic-lockstep
         * @returns the max steps
         */
        getLockstepMaxSteps(): number;
        /**
         * Gets the current hardware scaling level.
         * By default the hardware scaling level is computed from the window device ratio.
         * if level = 1 then the engine will render at the exact resolution of the canvas. If level = 0.5 then the engine will render at twice the size of the canvas.
         * @returns a number indicating the current hardware scaling level
         */
        getHardwareScalingLevel(): number;
        constructor(options?: NullEngineOptions);
        /**
         * Creates a vertex buffer
         * @param vertices the data for the vertex buffer
         * @returns the new WebGL static buffer
         */
        createVertexBuffer(vertices: FloatArray): DataBuffer;
        /**
         * Creates a new index buffer
         * @param indices defines the content of the index buffer
         * @returns a new webGL buffer
         */
        createIndexBuffer(indices: IndicesArray): DataBuffer;
        /**
         * Clear the current render buffer or the current render target (if any is set up)
         * @param color defines the color to use
         * @param backBuffer defines if the back buffer must be cleared
         * @param depth defines if the depth buffer must be cleared
         * @param stencil defines if the stencil buffer must be cleared
         */
        clear(color: IColor4Like, backBuffer: boolean, depth: boolean, stencil?: boolean): void;
        /**
         * Gets the current render width
         * @param useScreen defines if screen size must be used (or the current render target if any)
         * @returns a number defining the current render width
         */
        getRenderWidth(useScreen?: boolean): number;
        /**
         * Gets the current render height
         * @param useScreen defines if screen size must be used (or the current render target if any)
         * @returns a number defining the current render height
         */
        getRenderHeight(useScreen?: boolean): number;
        /**
         * Set the WebGL's viewport
         * @param viewport defines the viewport element to be used
         * @param requiredWidth defines the width required for rendering. If not provided the rendering canvas' width is used
         * @param requiredHeight defines the height required for rendering. If not provided the rendering canvas' height is used
         */
        setViewport(viewport: IViewportLike, requiredWidth?: number, requiredHeight?: number): void;
        createShaderProgram(pipelineContext: IPipelineContext, vertexCode: string, fragmentCode: string, defines: string, context?: WebGLRenderingContext): WebGLProgram;
        /**
         * Gets the list of webGL uniform locations associated with a specific program based on a list of uniform names
         * @param pipelineContext defines the pipeline context to use
         * @param uniformsNames defines the list of uniform names
         * @returns an array of webGL uniform locations
         */
        getUniforms(pipelineContext: IPipelineContext, uniformsNames: string[]): Nullable<WebGLUniformLocation>[];
        /**
         * Gets the lsit of active attributes for a given webGL program
         * @param pipelineContext defines the pipeline context to use
         * @param attributesNames defines the list of attribute names to get
         * @returns an array of indices indicating the offset of each attribute
         */
        getAttributes(pipelineContext: IPipelineContext, attributesNames: string[]): number[];
        /**
         * Binds an effect to the webGL context
         * @param effect defines the effect to bind
         */
        bindSamplers(effect: Effect): void;
        /**
         * Activates an effect, making it the current one (ie. the one used for rendering)
         * @param effect defines the effect to activate
         */
        enableEffect(effect: Nullable<Effect | DrawWrapper>): void;
        /**
         * Set various states to the webGL context
         * @param culling defines culling state: true to enable culling, false to disable it
         * @param zOffset defines the value to apply to zOffset (0 by default)
         * @param force defines if states must be applied even if cache is up to date
         * @param reverseSide defines if culling must be reversed (CCW if false, CW if true)
         * @param cullBackFaces true to cull back faces, false to cull front faces (if culling is enabled)
         * @param stencil stencil states to set
         * @param zOffsetUnits defines the value to apply to zOffsetUnits (0 by default)
         */
        setState(culling: boolean, zOffset?: number, force?: boolean, reverseSide?: boolean, cullBackFaces?: boolean, stencil?: IStencilState, zOffsetUnits?: number): void;
        /**
         * Set the value of an uniform to an array of int32
         * @param uniform defines the webGL uniform location where to store the value
         * @param array defines the array of int32 to store
         * @returns true if value was set
         */
        setIntArray(uniform: WebGLUniformLocation, array: Int32Array): boolean;
        /**
         * Set the value of an uniform to an array of int32 (stored as vec2)
         * @param uniform defines the webGL uniform location where to store the value
         * @param array defines the array of int32 to store
         * @returns true if value was set
         */
        setIntArray2(uniform: WebGLUniformLocation, array: Int32Array): boolean;
        /**
         * Set the value of an uniform to an array of int32 (stored as vec3)
         * @param uniform defines the webGL uniform location where to store the value
         * @param array defines the array of int32 to store
         * @returns true if value was set
         */
        setIntArray3(uniform: WebGLUniformLocation, array: Int32Array): boolean;
        /**
         * Set the value of an uniform to an array of int32 (stored as vec4)
         * @param uniform defines the webGL uniform location where to store the value
         * @param array defines the array of int32 to store
         * @returns true if value was set
         */
        setIntArray4(uniform: WebGLUniformLocation, array: Int32Array): boolean;
        /**
         * Set the value of an uniform to an array of float32
         * @param uniform defines the webGL uniform location where to store the value
         * @param array defines the array of float32 to store
         * @returns true if value was set
         */
        setFloatArray(uniform: WebGLUniformLocation, array: Float32Array): boolean;
        /**
         * Set the value of an uniform to an array of float32 (stored as vec2)
         * @param uniform defines the webGL uniform location where to store the value
         * @param array defines the array of float32 to store
         * @returns true if value was set
         */
        setFloatArray2(uniform: WebGLUniformLocation, array: Float32Array): boolean;
        /**
         * Set the value of an uniform to an array of float32 (stored as vec3)
         * @param uniform defines the webGL uniform location where to store the value
         * @param array defines the array of float32 to store
         * @returns true if value was set
         */
        setFloatArray3(uniform: WebGLUniformLocation, array: Float32Array): boolean;
        /**
         * Set the value of an uniform to an array of float32 (stored as vec4)
         * @param uniform defines the webGL uniform location where to store the value
         * @param array defines the array of float32 to store
         * @returns true if value was set
         */
        setFloatArray4(uniform: WebGLUniformLocation, array: Float32Array): boolean;
        /**
         * Set the value of an uniform to an array of number
         * @param uniform defines the webGL uniform location where to store the value
         * @param array defines the array of number to store
         * @returns true if value was set
         */
        setArray(uniform: WebGLUniformLocation, array: number[]): boolean;
        /**
         * Set the value of an uniform to an array of number (stored as vec2)
         * @param uniform defines the webGL uniform location where to store the value
         * @param array defines the array of number to store
         * @returns true if value was set
         */
        setArray2(uniform: WebGLUniformLocation, array: number[]): boolean;
        /**
         * Set the value of an uniform to an array of number (stored as vec3)
         * @param uniform defines the webGL uniform location where to store the value
         * @param array defines the array of number to store
         * @returns true if value was set
         */
        setArray3(uniform: WebGLUniformLocation, array: number[]): boolean;
        /**
         * Set the value of an uniform to an array of number (stored as vec4)
         * @param uniform defines the webGL uniform location where to store the value
         * @param array defines the array of number to store
         * @returns true if value was set
         */
        setArray4(uniform: WebGLUniformLocation, array: number[]): boolean;
        /**
         * Set the value of an uniform to an array of float32 (stored as matrices)
         * @param uniform defines the webGL uniform location where to store the value
         * @param matrices defines the array of float32 to store
         * @returns true if value was set
         */
        setMatrices(uniform: WebGLUniformLocation, matrices: Float32Array): boolean;
        /**
         * Set the value of an uniform to a matrix (3x3)
         * @param uniform defines the webGL uniform location where to store the value
         * @param matrix defines the Float32Array representing the 3x3 matrix to store
         * @returns true if value was set
         */
        setMatrix3x3(uniform: WebGLUniformLocation, matrix: Float32Array): boolean;
        /**
         * Set the value of an uniform to a matrix (2x2)
         * @param uniform defines the webGL uniform location where to store the value
         * @param matrix defines the Float32Array representing the 2x2 matrix to store
         * @returns true if value was set
         */
        setMatrix2x2(uniform: WebGLUniformLocation, matrix: Float32Array): boolean;
        /**
         * Set the value of an uniform to a number (float)
         * @param uniform defines the webGL uniform location where to store the value
         * @param value defines the float number to store
         * @returns true if value was set
         */
        setFloat(uniform: WebGLUniformLocation, value: number): boolean;
        /**
         * Set the value of an uniform to a vec2
         * @param uniform defines the webGL uniform location where to store the value
         * @param x defines the 1st component of the value
         * @param y defines the 2nd component of the value
         * @returns true if value was set
         */
        setFloat2(uniform: WebGLUniformLocation, x: number, y: number): boolean;
        /**
         * Set the value of an uniform to a vec3
         * @param uniform defines the webGL uniform location where to store the value
         * @param x defines the 1st component of the value
         * @param y defines the 2nd component of the value
         * @param z defines the 3rd component of the value
         * @returns true if value was set
         */
        setFloat3(uniform: WebGLUniformLocation, x: number, y: number, z: number): boolean;
        /**
         * Set the value of an uniform to a boolean
         * @param uniform defines the webGL uniform location where to store the value
         * @param bool defines the boolean to store
         * @returns true if value was set
         */
        setBool(uniform: WebGLUniformLocation, bool: number): boolean;
        /**
         * Set the value of an uniform to a vec4
         * @param uniform defines the webGL uniform location where to store the value
         * @param x defines the 1st component of the value
         * @param y defines the 2nd component of the value
         * @param z defines the 3rd component of the value
         * @param w defines the 4th component of the value
         * @returns true if value was set
         */
        setFloat4(uniform: WebGLUniformLocation, x: number, y: number, z: number, w: number): boolean;
        /**
         * Sets the current alpha mode
         * @param mode defines the mode to use (one of the Engine.ALPHA_XXX)
         * @param noDepthWriteChange defines if depth writing state should remains unchanged (false by default)
         * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/advanced/transparent_rendering
         */
        setAlphaMode(mode: number, noDepthWriteChange?: boolean): void;
        /**
         * Bind webGl buffers directly to the webGL context
         * @param vertexBuffers defines the vertex buffer to bind
         * @param indexBuffer defines the index buffer to bind
         * @param effect defines the effect associated with the vertex buffer
         */
        bindBuffers(vertexBuffers: {
            [key: string]: VertexBuffer;
        }, indexBuffer: DataBuffer, effect: Effect): void;
        /**
         * Force the entire cache to be cleared
         * You should not have to use this function unless your engine needs to share the webGL context with another engine
         * @param bruteForce defines a boolean to force clearing ALL caches (including stencil, detoh and alpha states)
         */
        wipeCaches(bruteForce?: boolean): void;
        /**
         * Send a draw order
         * @param useTriangles defines if triangles must be used to draw (else wireframe will be used)
         * @param indexStart defines the starting index
         * @param indexCount defines the number of index to draw
         * @param instancesCount defines the number of instances to draw (if instantiation is enabled)
         */
        draw(useTriangles: boolean, indexStart: number, indexCount: number, instancesCount?: number): void;
        /**
         * Draw a list of indexed primitives
         * @param fillMode defines the primitive to use
         * @param indexStart defines the starting index
         * @param indexCount defines the number of index to draw
         * @param instancesCount defines the number of instances to draw (if instantiation is enabled)
         */
        drawElementsType(fillMode: number, indexStart: number, indexCount: number, instancesCount?: number): void;
        /**
         * Draw a list of unindexed primitives
         * @param fillMode defines the primitive to use
         * @param verticesStart defines the index of first vertex to draw
         * @param verticesCount defines the count of vertices to draw
         * @param instancesCount defines the number of instances to draw (if instantiation is enabled)
         */
        drawArraysType(fillMode: number, verticesStart: number, verticesCount: number, instancesCount?: number): void;
        /** @internal */
        protected _createTexture(): WebGLTexture;
        /**
         * @internal
         */
        _releaseTexture(texture: InternalTexture): void;
        /**
         * Usually called from Texture.ts.
         * Passed information to create a WebGLTexture
         * @param urlArg defines a value which contains one of the following:
         * * A conventional http URL, e.g. 'http://...' or 'file://...'
         * * A base64 string of in-line texture data, e.g. 'data:image/jpg;base64,/...'
         * * An indicator that data being passed using the buffer parameter, e.g. 'data:mytexture.jpg'
         * @param noMipmap defines a boolean indicating that no mipmaps shall be generated.  Ignored for compressed textures.  They must be in the file
         * @param invertY when true, image is flipped when loaded.  You probably want true. Certain compressed textures may invert this if their default is inverted (eg. ktx)
         * @param scene needed for loading to the correct scene
         * @param samplingMode mode with should be used sample / access the texture (Default: Texture.TRILINEAR_SAMPLINGMODE)
         * @param onLoad optional callback to be called upon successful completion
         * @param onError optional callback to be called upon failure
         * @param buffer a source of a file previously fetched as either a base64 string, an ArrayBuffer (compressed or image format), HTMLImageElement (image format), or a Blob
         * @param fallback an internal argument in case the function must be called again, due to etc1 not having alpha capabilities
         * @param format internal format.  Default: RGB when extension is '.jpg' else RGBA.  Ignored for compressed textures
         * @param forcedExtension defines the extension to use to pick the right loader
         * @param mimeType defines an optional mime type
         * @returns a InternalTexture for assignment back into BABYLON.Texture
         */
        createTexture(urlArg: Nullable<string>, noMipmap: boolean, invertY: boolean, scene: Nullable<ISceneLike>, samplingMode?: number, onLoad?: Nullable<(texture: InternalTexture) => void>, onError?: Nullable<(message: string, exception: any) => void>, buffer?: Nullable<string | ArrayBuffer | ArrayBufferView | HTMLImageElement | Blob | ImageBitmap>, fallback?: Nullable<InternalTexture>, format?: Nullable<number>, forcedExtension?: Nullable<string>, mimeType?: string): InternalTexture;
        /**
         * @internal
         */
        _createHardwareRenderTargetWrapper(isMulti: boolean, isCube: boolean, size: number | {
            width: number;
            height: number;
            layers?: number;
        }): RenderTargetWrapper;
        /**
         * Creates a new render target wrapper
         * @param size defines the size of the texture
         * @param options defines the options used to create the texture
         * @returns a new render target wrapper
         */
        createRenderTargetTexture(size: any, options: boolean | RenderTargetCreationOptions): RenderTargetWrapper;
        /**
         * Creates a new render target wrapper
         * @param size defines the size of the texture
         * @param options defines the options used to create the texture
         * @returns a new render target wrapper
         */
        createRenderTargetCubeTexture(size: number, options?: RenderTargetCreationOptions): RenderTargetWrapper;
        /**
         * Update the sampling mode of a given texture
         * @param samplingMode defines the required sampling mode
         * @param texture defines the texture to update
         */
        updateTextureSamplingMode(samplingMode: number, texture: InternalTexture): void;
        /**
         * Creates a raw texture
         * @param data defines the data to store in the texture
         * @param width defines the width of the texture
         * @param height defines the height of the texture
         * @param format defines the format of the data
         * @param generateMipMaps defines if the engine should generate the mip levels
         * @param invertY defines if data must be stored with Y axis inverted
         * @param samplingMode defines the required sampling mode (Texture.NEAREST_SAMPLINGMODE by default)
         * @param compression defines the compression used (null by default)
         * @param type defines the type fo the data (Engine.TEXTURETYPE_UNSIGNED_INT by default)
         * @param creationFlags specific flags to use when creating the texture (Constants.TEXTURE_CREATIONFLAG_STORAGE for storage textures, for eg)
         * @param useSRGBBuffer defines if the texture must be loaded in a sRGB GPU buffer (if supported by the GPU).
         * @returns the raw texture inside an InternalTexture
         */
        createRawTexture(data: Nullable<ArrayBufferView>, width: number, height: number, format: number, generateMipMaps: boolean, invertY: boolean, samplingMode: number, compression?: Nullable<string>, type?: number, creationFlags?: number, useSRGBBuffer?: boolean): InternalTexture;
        /**
         * Update a raw texture
         * @param texture defines the texture to update
         * @param data defines the data to store in the texture
         * @param format defines the format of the data
         * @param invertY defines if data must be stored with Y axis inverted
         * @param compression defines the compression used (null by default)
         * @param type defines the type fo the data (Engine.TEXTURETYPE_UNSIGNED_INT by default)
         * @param useSRGBBuffer defines if the texture must be loaded in a sRGB GPU buffer (if supported by the GPU).
         */
        updateRawTexture(texture: Nullable<InternalTexture>, data: Nullable<ArrayBufferView>, format: number, invertY: boolean, compression?: Nullable<string>, type?: number, useSRGBBuffer?: boolean): void;
        /**
         * Binds the frame buffer to the specified texture.
         * @param rtWrapper The render target wrapper to render to
         * @param faceIndex The face of the texture to render to in case of cube texture
         * @param requiredWidth The width of the target to render to
         * @param requiredHeight The height of the target to render to
         * @param forceFullscreenViewport Forces the viewport to be the entire texture/screen if true
         */
        bindFramebuffer(rtWrapper: RenderTargetWrapper, faceIndex?: number, requiredWidth?: number, requiredHeight?: number, forceFullscreenViewport?: boolean): void;
        /**
         * Unbind the current render target texture from the webGL context
         * @param rtWrapper defines the render target wrapper to unbind
         * @param disableGenerateMipMaps defines a boolean indicating that mipmaps must not be generated
         * @param onBeforeUnbind defines a function which will be called before the effective unbind
         */
        unBindFramebuffer(rtWrapper: RenderTargetWrapper, disableGenerateMipMaps?: boolean, onBeforeUnbind?: () => void): void;
        /**
         * Creates a dynamic vertex buffer
         * @param vertices the data for the dynamic vertex buffer
         * @returns the new WebGL dynamic buffer
         */
        createDynamicVertexBuffer(vertices: FloatArray): DataBuffer;
        /**
         * Update the content of a dynamic texture
         * @param texture defines the texture to update
         * @param canvas defines the canvas containing the source
         * @param invertY defines if data must be stored with Y axis inverted
         * @param premulAlpha defines if alpha is stored as premultiplied
         * @param format defines the format of the data
         */
        updateDynamicTexture(texture: Nullable<InternalTexture>, canvas: HTMLCanvasElement, invertY: boolean, premulAlpha?: boolean, format?: number): void;
        /**
         * Gets a boolean indicating if all created effects are ready
         * @returns true if all effects are ready
         */
        areAllEffectsReady(): boolean;
        /**
         * @internal
         * Get the current error code of the webGL context
         * @returns the error code
         * @see https://developer.mozilla.org/en-US/docs/Web/API/WebGLRenderingContext/getError
         */
        getError(): number;
        /** @internal */
        _getUnpackAlignement(): number;
        /**
         * @internal
         */
        _unpackFlipY(value: boolean): void;
        /**
         * Update a dynamic index buffer
         * @param indexBuffer defines the target index buffer
         * @param indices defines the data to update
         * @param offset defines the offset in the target index buffer where update should start
         */
        updateDynamicIndexBuffer(indexBuffer: WebGLBuffer, indices: IndicesArray, offset?: number): void;
        /**
         * Updates a dynamic vertex buffer.
         * @param vertexBuffer the vertex buffer to update
         * @param vertices the data used to update the vertex buffer
         * @param byteOffset the byte offset of the data (optional)
         * @param byteLength the byte length of the data (optional)
         */
        updateDynamicVertexBuffer(vertexBuffer: WebGLBuffer, vertices: FloatArray, byteOffset?: number, byteLength?: number): void;
        /**
         * @internal
         */
        _bindTextureDirectly(target: number, texture: InternalTexture): boolean;
        /**
         * @internal
         */
        _bindTexture(channel: number, texture: InternalTexture): void;
        protected _deleteBuffer(buffer: WebGLBuffer): void;
        /**
         * Force the engine to release all cached effects. This means that next effect compilation will have to be done completely even if a similar effect was already compiled
         */
        releaseEffects(): void;
        displayLoadingUI(): void;
        hideLoadingUI(): void;
        set loadingUIText(_: string);
        /**
         * @internal
         */
        _uploadCompressedDataToTextureDirectly(texture: InternalTexture, internalFormat: number, width: number, height: number, data: ArrayBufferView, faceIndex?: number, lod?: number): void;
        /**
         * @internal
         */
        _uploadDataToTextureDirectly(texture: InternalTexture, imageData: ArrayBufferView, faceIndex?: number, lod?: number): void;
        /**
         * @internal
         */
        _uploadArrayBufferViewToTexture(texture: InternalTexture, imageData: ArrayBufferView, faceIndex?: number, lod?: number): void;
        /**
         * @internal
         */
        _uploadImageToTexture(texture: InternalTexture, image: HTMLImageElement, faceIndex?: number, lod?: number): void;
    }


    /** @internal */
    export class PerformanceConfigurator {
        /** @internal */
        static MatrixUse64Bits: boolean;
        /** @internal */
        static MatrixTrackPrecisionChange: boolean;
        /** @internal */
        static MatrixCurrentType: any;
        /** @internal */
        static MatrixTrackedMatrices: Array<any> | null;
        /**
         * @internal
         */
        static SetMatrixPrecision(use64bits: boolean): void;
    }






    /** @internal */
    export class ShaderDefineAndOperator extends ShaderDefineExpression {
        leftOperand: ShaderDefineExpression;
        rightOperand: ShaderDefineExpression;
        isTrue(preprocessors: {
            [key: string]: string;
        }): boolean;
    }


    /** @internal */
    export class ShaderDefineArithmeticOperator extends ShaderDefineExpression {
        define: string;
        operand: string;
        testValue: string;
        constructor(define: string, operand: string, testValue: string);
        isTrue(preprocessors: {
            [key: string]: string;
        }): boolean;
    }


    /** @internal */
    export class ShaderDefineIsDefinedOperator extends ShaderDefineExpression {
        define: string;
        not: boolean;
        constructor(define: string, not?: boolean);
        isTrue(preprocessors: {
            [key: string]: string;
        }): boolean;
    }


    /** @internal */
    export class ShaderDefineOrOperator extends ShaderDefineExpression {
        leftOperand: ShaderDefineExpression;
        rightOperand: ShaderDefineExpression;
        isTrue(preprocessors: {
            [key: string]: string;
        }): boolean;
    }


    /** @internal */
    export class ShaderDefineExpression {
        isTrue(preprocessors: {
            [key: string]: string;
        }): boolean;
        private static _OperatorPriority;
        private static _Stack;
        static postfixToInfix(postfix: string[]): string;
        static infixToPostfix(infix: string): string[];
    }




    /** @internal */
    export interface IShaderProcessor {
        shaderLanguage: ShaderLanguage;
        uniformRegexp?: RegExp;
        uniformBufferRegexp?: RegExp;
        textureRegexp?: RegExp;
        noPrecision?: boolean;
        parseGLES3?: boolean;
        attributeKeywordName?: string;
        varyingVertexKeywordName?: string;
        varyingFragmentKeywordName?: string;
        preProcessShaderCode?: (code: string, isFragment: boolean) => string;
        attributeProcessor?: (attribute: string, preProcessors: {
            [key: string]: string;
        }, processingContext: Nullable<ShaderProcessingContext>) => string;
        varyingProcessor?: (varying: string, isFragment: boolean, preProcessors: {
            [key: string]: string;
        }, processingContext: Nullable<ShaderProcessingContext>) => string;
        uniformProcessor?: (uniform: string, isFragment: boolean, preProcessors: {
            [key: string]: string;
        }, processingContext: Nullable<ShaderProcessingContext>) => string;
        uniformBufferProcessor?: (uniformBuffer: string, isFragment: boolean, processingContext: Nullable<ShaderProcessingContext>) => string;
        textureProcessor?: (texture: string, isFragment: boolean, preProcessors: {
            [key: string]: string;
        }, processingContext: Nullable<ShaderProcessingContext>) => string;
        endOfUniformBufferProcessor?: (closingBracketLine: string, isFragment: boolean, processingContext: Nullable<ShaderProcessingContext>) => string;
        lineProcessor?: (line: string, isFragment: boolean, processingContext: Nullable<ShaderProcessingContext>) => string;
        preProcessor?: (code: string, defines: string[], isFragment: boolean, processingContext: Nullable<ShaderProcessingContext>) => string;
        postProcessor?: (code: string, defines: string[], isFragment: boolean, processingContext: Nullable<ShaderProcessingContext>, engine: ThinEngine) => string;
        initializeShaders?: (processingContext: Nullable<ShaderProcessingContext>) => void;
        finalizeShaders?: (vertexCode: string, fragmentCode: string, processingContext: Nullable<ShaderProcessingContext>) => {
            vertexCode: string;
            fragmentCode: string;
        };
    }


    /** @internal */
    export class ShaderCodeConditionNode extends ShaderCodeNode {
        process(preprocessors: {
            [key: string]: string;
        }, options: ProcessingOptions): string;
    }


    /** @internal */
    export class ShaderCodeCursor {
        private _lines;
        lineIndex: number;
        get currentLine(): string;
        get canRead(): boolean;
        set lines(value: string[]);
    }


    /**
     * Class used to inline functions in shader code
     */
    export class ShaderCodeInliner {
        private static readonly _RegexpFindFunctionNameAndType;
        private _sourceCode;
        private _functionDescr;
        private _numMaxIterations;
        /** Gets or sets the token used to mark the functions to inline */
        inlineToken: string;
        /** Gets or sets the debug mode */
        debug: boolean;
        /** Gets the code after the inlining process */
        get code(): string;
        /**
         * Initializes the inliner
         * @param sourceCode shader code source to inline
         * @param numMaxIterations maximum number of iterations (used to detect recursive calls)
         */
        constructor(sourceCode: string, numMaxIterations?: number);
        /**
         * Start the processing of the shader code
         */
        processCode(): void;
        private _collectFunctions;
        private _processInlining;
        private _replaceFunctionCallsByCode;
        private _replaceNames;
    }


    /** @internal */
    export class ShaderCodeNode {
        line: string;
        children: ShaderCodeNode[];
        additionalDefineKey?: string;
        additionalDefineValue?: string;
        isValid(preprocessors: {
            [key: string]: string;
        }): boolean;
        process(preprocessors: {
            [key: string]: string;
        }, options: ProcessingOptions): string;
    }


    /** @internal */
    export class ShaderCodeTestNode extends ShaderCodeNode {
        testExpression: ShaderDefineExpression;
        isValid(preprocessors: {
            [key: string]: string;
        }): boolean;
    }


    /**
     * Function for custom code generation
     */
    export type ShaderCustomProcessingFunction = (shaderType: string, code: string) => string;
    /** @internal */
    export interface ShaderProcessingContext {
    }
    /** @internal */
    export interface ProcessingOptions {
        defines: string[];
        indexParameters: any;
        isFragment: boolean;
        shouldUseHighPrecisionShader: boolean;
        supportsUniformBuffers: boolean;
        shadersRepository: string;
        includesShadersStore: {
            [key: string]: string;
        };
        processor: Nullable<IShaderProcessor>;
        version: string;
        platformName: string;
        lookForClosingBracketForUniformBuffer?: boolean;
        processingContext: Nullable<ShaderProcessingContext>;
        isNDCHalfZRange: boolean;
        useReverseDepthBuffer: boolean;
        processCodeAfterIncludes?: ShaderCustomProcessingFunction;
    }


    /** @internal */
    export class ShaderProcessor {
        static Initialize(options: ProcessingOptions): void;
        static Process(sourceCode: string, options: ProcessingOptions, callback: (migratedCode: string, codeBeforeMigration: string) => void, engine: ThinEngine): void;
        static PreProcess(sourceCode: string, options: ProcessingOptions, callback: (migratedCode: string, codeBeforeMigration: string) => void, engine: ThinEngine): void;
        static Finalize(vertexCode: string, fragmentCode: string, options: ProcessingOptions): {
            vertexCode: string;
            fragmentCode: string;
        };
        private static _ProcessPrecision;
        private static _ExtractOperation;
        private static _BuildSubExpression;
        private static _BuildExpression;
        private static _MoveCursorWithinIf;
        private static _MoveCursor;
        private static _EvaluatePreProcessors;
        private static _PreparePreProcessors;
        private static _ProcessShaderConversion;
        private static _ApplyPreProcessing;
        private static _ProcessIncludes;
        /**
         * Loads a file from a url
         * @param url url to load
         * @param onSuccess callback called when the file successfully loads
         * @param onProgress callback called while file is loading (if the server supports this mode)
         * @param offlineProvider defines the offline provider for caching
         * @param useArrayBuffer defines a boolean indicating that date must be returned as ArrayBuffer
         * @param onError callback called when the file fails to load
         * @returns a file request object
         * @internal
         */
        static _FileToolsLoadFile(url: string, onSuccess: (data: string | ArrayBuffer, responseURL?: string) => void, onProgress?: (ev: ProgressEvent) => void, offlineProvider?: IOfflineProvider, useArrayBuffer?: boolean, onError?: (request?: WebRequest, exception?: LoadFileError) => void): IFileRequest;
    }


    /**
     * An interface enforcing the renderTarget accessor to used by render target textures.
     */
    export interface IRenderTargetTexture {
        /**
         * Entry point to access the wrapper on a texture.
         */
        renderTarget: Nullable<RenderTargetWrapper>;
    }
    /**
     * Wrapper around a render target (either single or multi textures)
     */
    export class RenderTargetWrapper {
        protected _engine: ThinEngine;
        private _size;
        private _isCube;
        private _isMulti;
        private _textures;
        /** @internal */
        _samples: number;
        /** @internal */
        _attachments: Nullable<number[]>;
        /** @internal */
        _generateStencilBuffer: boolean;
        /** @internal */
        _generateDepthBuffer: boolean;
        /** @internal */
        _depthStencilTexture: Nullable<InternalTexture>;
        /** @internal */
        _depthStencilTextureWithStencil: boolean;
        /**
         * Gets the depth/stencil texture (if created by a createDepthStencilTexture() call)
         */
        get depthStencilTexture(): Nullable<InternalTexture>;
        /**
         * Indicates if the depth/stencil texture has a stencil aspect
         */
        get depthStencilTextureWithStencil(): boolean;
        /**
         * Defines if the render target wrapper is for a cube texture or if false a 2d texture
         */
        get isCube(): boolean;
        /**
         * Defines if the render target wrapper is for a single or multi target render wrapper
         */
        get isMulti(): boolean;
        /**
         * Defines if the render target wrapper is for a single or an array of textures
         */
        get is2DArray(): boolean;
        /**
         * Gets the size of the render target wrapper (used for cubes, as width=height in this case)
         */
        get size(): number;
        /**
         * Gets the width of the render target wrapper
         */
        get width(): number;
        /**
         * Gets the height of the render target wrapper
         */
        get height(): number;
        /**
         * Gets the number of layers of the render target wrapper (only used if is2DArray is true)
         */
        get layers(): number;
        /**
         * Gets the render texture. If this is a multi render target, gets the first texture
         */
        get texture(): Nullable<InternalTexture>;
        /**
         * Gets the list of render textures. If we are not in a multi render target, the list will be null (use the texture getter instead)
         */
        get textures(): Nullable<InternalTexture[]>;
        /**
         * Gets the sample count of the render target
         */
        get samples(): number;
        /**
         * Sets the sample count of the render target
         * @param value sample count
         * @param initializeBuffers If set to true, the engine will make an initializing call to drawBuffers (only used when isMulti=true).
         * @param force true to force calling the update sample count engine function even if the current sample count is equal to value
         * @returns the sample count that has been set
         */
        setSamples(value: number, initializeBuffers?: boolean, force?: boolean): number;
        /**
         * Initializes the render target wrapper
         * @param isMulti true if the wrapper is a multi render target
         * @param isCube true if the wrapper should render to a cube texture
         * @param size size of the render target (width/height/layers)
         * @param engine engine used to create the render target
         */
        constructor(isMulti: boolean, isCube: boolean, size: TextureSize, engine: ThinEngine);
        /**
         * Sets the render target texture(s)
         * @param textures texture(s) to set
         */
        setTextures(textures: Nullable<InternalTexture> | Nullable<InternalTexture[]>): void;
        /**
         * Set a texture in the textures array
         * @param texture the texture to set
         * @param index the index in the textures array to set
         * @param disposePrevious If this function should dispose the previous texture
         */
        setTexture(texture: InternalTexture, index?: number, disposePrevious?: boolean): void;
        /**
         * Creates the depth/stencil texture
         * @param comparisonFunction Comparison function to use for the texture
         * @param bilinearFiltering true if bilinear filtering should be used when sampling the texture
         * @param generateStencil true if the stencil aspect should also be created
         * @param samples sample count to use when creating the texture
         * @param format format of the depth texture
         * @returns the depth/stencil created texture
         */
        createDepthStencilTexture(comparisonFunction?: number, bilinearFiltering?: boolean, generateStencil?: boolean, samples?: number, format?: number): InternalTexture;
        /**
         * Shares the depth buffer of this render target with another render target.
         * @internal
         * @param renderTarget Destination renderTarget
         */
        _shareDepth(renderTarget: RenderTargetWrapper): void;
        /**
         * @internal
         */
        _swapAndDie(target: InternalTexture): void;
        protected _cloneRenderTargetWrapper(): Nullable<RenderTargetWrapper>;
        protected _swapRenderTargetWrapper(target: RenderTargetWrapper): void;
        /** @internal */
        _rebuild(): void;
        /**
         * Releases the internal render textures
         */
        releaseTextures(): void;
        /**
         * Disposes the whole render target wrapper
         * @param disposeOnlyFramebuffers true if only the frame buffers should be released (used for the WebGL engine). If false, all the textures will also be released
         */
        dispose(disposeOnlyFramebuffers?: boolean): void;
    }


    /**
     * Defines the shader related stores and directory
     */
    export class ShaderStore {
        /**
         * Gets or sets the relative url used to load shaders if using the engine in non-minified mode
         */
        static ShadersRepository: string;
        /**
         * Store of each shader (The can be looked up using effect.key)
         */
        static ShadersStore: {
            [key: string]: string;
        };
        /**
         * Store of each included file for a shader (The can be looked up using effect.key)
         */
        static IncludesShadersStore: {
            [key: string]: string;
        };
        /**
         * Gets or sets the relative url used to load shaders (WGSL) if using the engine in non-minified mode
         */
        static ShadersRepositoryWGSL: string;
        /**
         * Store of each shader  (WGSL)
         */
        static ShadersStoreWGSL: {
            [key: string]: string;
        };
        /**
         * Store of each included file for a shader (WGSL)
         */
        static IncludesShadersStoreWGSL: {
            [key: string]: string;
        };
        /**
         * Gets the shaders repository path for a given shader language
         * @param shaderLanguage the shader language
         * @returns the path to the shaders repository
         */
        static GetShadersRepository(shaderLanguage?: ShaderLanguage): string;
        /**
         * Gets the shaders store of a given shader language
         * @param shaderLanguage the shader language
         * @returns the shaders store
         */
        static GetShadersStore(shaderLanguage?: ShaderLanguage): {
            [key: string]: string;
        };
        /**
         * Gets the include shaders store of a given shader language
         * @param shaderLanguage the shader language
         * @returns the include shaders store
         */
        static GetIncludesShadersStore(shaderLanguage?: ShaderLanguage): {
            [key: string]: string;
        };
    }


    /**
     * Defines the interface used by objects working like Scene
     * @internal
     */
    export interface ISceneLike {
        addPendingData(data: any): void;
        removePendingData(data: any): void;
        offlineProvider: IOfflineProvider;
    }
    /**
     * Information about the current host
     */
    export interface HostInformation {
        /**
         * Defines if the current host is a mobile
         */
        isMobile: boolean;
    }
    /** Interface defining initialization parameters for Engine class */
    export interface EngineOptions extends WebGLContextAttributes {
        /**
         * Defines if the engine should no exceed a specified device ratio
         * @see https://developer.mozilla.org/en-US/docs/Web/API/Window/devicePixelRatio
         */
        limitDeviceRatio?: number;
        /**
         * Defines if webvr should be enabled automatically
         * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/webVRCamera
         */
        autoEnableWebVR?: boolean;
        /**
         * Defines if webgl2 should be turned off even if supported
         * @see https://doc.babylonjs.com/setup/support/webGL2
         */
        disableWebGL2Support?: boolean;
        /**
         * Defines if webaudio should be initialized as well
         * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic
         */
        audioEngine?: boolean;
        /**
         * Specifies options for the audio engine
         */
        audioEngineOptions?: IAudioEngineOptions;
        /**
         * Defines if animations should run using a deterministic lock step
         * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/advanced_animations#deterministic-lockstep
         */
        deterministicLockstep?: boolean;
        /** Defines the maximum steps to use with deterministic lock step mode */
        lockstepMaxSteps?: number;
        /** Defines the seconds between each deterministic lock step */
        timeStep?: number;
        /**
         * Defines that engine should ignore context lost events
         * If this event happens when this parameter is true, you will have to reload the page to restore rendering
         */
        doNotHandleContextLost?: boolean;
        /**
         * Defines that engine should ignore modifying touch action attribute and style
         * If not handle, you might need to set it up on your side for expected touch devices behavior.
         */
        doNotHandleTouchAction?: boolean;
        /**
         * Defines that engine should compile shaders with high precision floats (if supported). True by default
         */
        useHighPrecisionFloats?: boolean;
        /**
         * Make the canvas XR Compatible for XR sessions
         */
        xrCompatible?: boolean;
        /**
         * Make the matrix computations to be performed in 64 bits instead of 32 bits. False by default
         */
        useHighPrecisionMatrix?: boolean;
        /**
         * Will prevent the system from falling back to software implementation if a hardware device cannot be created
         */
        failIfMajorPerformanceCaveat?: boolean;
        /**
         * Defines whether to adapt to the device's viewport characteristics (default: false)
         */
        adaptToDeviceRatio?: boolean;
        /**
         * If sRGB Buffer support is not set during construction, use this value to force a specific state
         * This is added due to an issue when processing textures in chrome/edge/firefox
         * This will not influence NativeEngine and WebGPUEngine which set the behavior to true during construction.
         */
        forceSRGBBufferSupportState?: boolean;
        /**
         * True if the more expensive but exact conversions should be used for transforming colors to and from linear space within shaders.
         * Otherwise, the default is to use a cheaper approximation.
         */
        useExactSrgbConversions?: boolean;
    }
    /**
     * The base engine class (root of all engines)
     */
    export class ThinEngine {
        /** Use this array to turn off some WebGL2 features on known buggy browsers version */
        static ExceptionList: ({
            key: string;
            capture: string;
            captureConstraint: number;
            targets: string[];
        } | {
            key: string;
            capture: null;
            captureConstraint: null;
            targets: string[];
        })[];
        /** @internal */
        static _TextureLoaders: IInternalTextureLoader[];
        /**
         * Returns the current npm package of the sdk
         */
        static get NpmPackage(): string;
        /**
         * Returns the current version of the framework
         */
        static get Version(): string;
        /**
         * Returns a string describing the current engine
         */
        get description(): string;
        /** @internal */
        protected _name: string;
        /**
         * Gets or sets the name of the engine
         */
        get name(): string;
        set name(value: string);
        /**
         * Returns the version of the engine
         */
        get version(): number;
        /**
         * Gets or sets the epsilon value used by collision engine
         */
        static CollisionsEpsilon: number;
        /**
         * Gets or sets the relative url used to load shaders if using the engine in non-minified mode
         */
        static get ShadersRepository(): string;
        static set ShadersRepository(value: string);
        protected _shaderProcessor: Nullable<IShaderProcessor>;
        /**
         * @internal
         */
        _getShaderProcessor(shaderLanguage: ShaderLanguage): Nullable<IShaderProcessor>;
        /**
         * Gets or sets a boolean that indicates if textures must be forced to power of 2 size even if not required
         */
        forcePOTTextures: boolean;
        /**
         * Gets a boolean indicating if the engine is currently rendering in fullscreen mode
         */
        isFullscreen: boolean;
        /**
         * Gets or sets a boolean indicating if back faces must be culled. If false, front faces are culled instead (true by default)
         * If non null, this takes precedence over the value from the material
         */
        cullBackFaces: Nullable<boolean>;
        /**
         * Gets or sets a boolean indicating if the engine must keep rendering even if the window is not in foreground
         */
        renderEvenInBackground: boolean;
        /**
         * Gets or sets a boolean indicating that cache can be kept between frames
         */
        preventCacheWipeBetweenFrames: boolean;
        /** Gets or sets a boolean indicating if the engine should validate programs after compilation */
        validateShaderPrograms: boolean;
        private _useReverseDepthBuffer;
        /**
         * Gets or sets a boolean indicating if depth buffer should be reverse, going from far to near.
         * This can provide greater z depth for distant objects.
         */
        get useReverseDepthBuffer(): boolean;
        set useReverseDepthBuffer(useReverse: boolean);
        /**
         * Indicates if the z range in NDC space is 0..1 (value: true) or -1..1 (value: false)
         */
        readonly isNDCHalfZRange = false;
        /**
         * Indicates that the origin of the texture/framebuffer space is the bottom left corner. If false, the origin is top left
         */
        readonly hasOriginBottomLeft = true;
        /**
         * Gets or sets a boolean indicating that uniform buffers must be disabled even if they are supported
         */
        disableUniformBuffers: boolean;
        /**
         * An event triggered when the engine is disposed.
         */
        readonly onDisposeObservable: Observable<ThinEngine>;
        private _frameId;
        /**
         * Gets the current frame id
         */
        get frameId(): number;
        /**
         * The time (in milliseconds elapsed since the current page has been loaded) when the engine was initialized
         */
        readonly startTime: number;
        /** @internal */
        _uniformBuffers: UniformBuffer[];
        /** @internal */
        _storageBuffers: StorageBuffer[];
        /**
         * Gets a boolean indicating that the engine supports uniform buffers
         * @see https://doc.babylonjs.com/setup/support/webGL2#uniform-buffer-objets
         */
        get supportsUniformBuffers(): boolean;
        /** @internal */
        _gl: WebGL2RenderingContext;
        /** @internal */
        _webGLVersion: number;
        protected _renderingCanvas: Nullable<HTMLCanvasElement>;
        protected _windowIsBackground: boolean;
        protected _creationOptions: EngineOptions;
        protected _audioContext: Nullable<AudioContext>;
        protected _audioDestination: Nullable<AudioDestinationNode | MediaStreamAudioDestinationNode>;
        /**
         * Gets the options used for engine creation
         * @returns EngineOptions object
         */
        getCreationOptions(): EngineOptions;
        protected _highPrecisionShadersAllowed: boolean;
        /** @internal */
        get _shouldUseHighPrecisionShader(): boolean;
        /**
         * Gets a boolean indicating that only power of 2 textures are supported
         * Please note that you can still use non power of 2 textures but in this case the engine will forcefully convert them
         */
        get needPOTTextures(): boolean;
        /** @internal */
        _badOS: boolean;
        /** @internal */
        _badDesktopOS: boolean;
        /** @internal */
        _hardwareScalingLevel: number;
        /** @internal */
        _caps: EngineCapabilities;
        /** @internal */
        _features: EngineFeatures;
        protected _isStencilEnable: boolean;
        private _glVersion;
        private _glRenderer;
        private _glVendor;
        /** @internal */
        _videoTextureSupported: boolean;
        protected _renderingQueueLaunched: boolean;
        protected _activeRenderLoops: (() => void)[];
        /**
         * Gets the list of current active render loop functions
         * @returns an array with the current render loop functions
         */
        get activeRenderLoops(): Array<() => void>;
        /**
         * Observable signaled when a context lost event is raised
         */
        onContextLostObservable: Observable<ThinEngine>;
        /**
         * Observable signaled when a context restored event is raised
         */
        onContextRestoredObservable: Observable<ThinEngine>;
        private _onContextLost;
        private _onContextRestored;
        protected _contextWasLost: boolean;
        /** @internal */
        _doNotHandleContextLost: boolean;
        /**
         * Gets or sets a boolean indicating if resources should be retained to be able to handle context lost events
         * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/optimize_your_scene#handling-webgl-context-lost
         */
        get doNotHandleContextLost(): boolean;
        set doNotHandleContextLost(value: boolean);
        /**
         * Gets or sets a boolean indicating that vertex array object must be disabled even if they are supported
         */
        disableVertexArrayObjects: boolean;
        /** @internal */
        protected _colorWrite: boolean;
        /** @internal */
        protected _colorWriteChanged: boolean;
        /** @internal */
        protected _depthCullingState: DepthCullingState;
        /** @internal */
        protected _stencilStateComposer: StencilStateComposer;
        /** @internal */
        protected _stencilState: StencilState;
        /** @internal */
        _alphaState: AlphaState;
        /** @internal */
        _alphaMode: number;
        /** @internal */
        _alphaEquation: number;
        /** @internal */
        _internalTexturesCache: InternalTexture[];
        /** @internal */
        _renderTargetWrapperCache: RenderTargetWrapper[];
        /** @internal */
        protected _activeChannel: number;
        private _currentTextureChannel;
        /** @internal */
        protected _boundTexturesCache: {
            [key: string]: Nullable<InternalTexture>;
        };
        protected _currentEffect: Nullable<Effect>;
        /** @internal */
        _currentDrawContext: IDrawContext;
        /** @internal */
        _currentMaterialContext: IMaterialContext;
        /** @internal */
        protected _currentProgram: Nullable<WebGLProgram>;
        protected _compiledEffects: {
            [key: string]: Effect;
        };
        private _vertexAttribArraysEnabled;
        /** @internal */
        protected _cachedViewport: Nullable<IViewportLike>;
        private _cachedVertexArrayObject;
        /** @internal */
        protected _cachedVertexBuffers: any;
        /** @internal */
        protected _cachedIndexBuffer: Nullable<DataBuffer>;
        /** @internal */
        protected _cachedEffectForVertexBuffers: Nullable<Effect>;
        /** @internal */
        _currentRenderTarget: Nullable<RenderTargetWrapper>;
        private _uintIndicesCurrentlySet;
        protected _currentBoundBuffer: Nullable<DataBuffer>[];
        /** @internal */
        _currentFramebuffer: Nullable<WebGLFramebuffer>;
        /** @internal */
        _dummyFramebuffer: Nullable<WebGLFramebuffer>;
        private _currentBufferPointers;
        private _currentInstanceLocations;
        private _currentInstanceBuffers;
        private _textureUnits;
        /** @internal */
        _workingCanvas: Nullable<ICanvas>;
        /** @internal */
        _workingContext: Nullable<ICanvasRenderingContext>;
        /** @internal */
        _boundRenderFunction: any;
        private _vaoRecordInProgress;
        private _mustWipeVertexAttributes;
        private _emptyTexture;
        private _emptyCubeTexture;
        private _emptyTexture3D;
        private _emptyTexture2DArray;
        /** @internal */
        _frameHandler: number;
        private _nextFreeTextureSlots;
        private _maxSimultaneousTextures;
        private _maxMSAASamplesOverride;
        private _activeRequests;
        /**
         * If set to true zooming in and out in the browser will rescale the hardware-scaling correctly.
         */
        adaptToDeviceRatio: boolean;
        /** @internal */
        private _lastDevicePixelRatio;
        /** @internal */
        _transformTextureUrl: Nullable<(url: string) => string>;
        /**
         * Gets information about the current host
         */
        hostInformation: HostInformation;
        protected get _supportsHardwareTextureRescaling(): boolean;
        private _framebufferDimensionsObject;
        /**
         * sets the object from which width and height will be taken from when getting render width and height
         * Will fallback to the gl object
         * @param dimensions the framebuffer width and height that will be used.
         */
        set framebufferDimensionsObject(dimensions: Nullable<{
            framebufferWidth: number;
            framebufferHeight: number;
        }>);
        /**
         * Gets the current viewport
         */
        get currentViewport(): Nullable<IViewportLike>;
        /**
         * Gets the default empty texture
         */
        get emptyTexture(): InternalTexture;
        /**
         * Gets the default empty 3D texture
         */
        get emptyTexture3D(): InternalTexture;
        /**
         * Gets the default empty 2D array texture
         */
        get emptyTexture2DArray(): InternalTexture;
        /**
         * Gets the default empty cube texture
         */
        get emptyCubeTexture(): InternalTexture;
        /**
         * Defines whether the engine has been created with the premultipliedAlpha option on or not.
         */
        premultipliedAlpha: boolean;
        /**
         * Observable event triggered before each texture is initialized
         */
        onBeforeTextureInitObservable: Observable<Texture>;
        /** @internal */
        protected _isWebGPU: boolean;
        /**
         * Gets a boolean indicating if the engine runs in WebGPU or not.
         */
        get isWebGPU(): boolean;
        /** @internal */
        protected _shaderPlatformName: string;
        /**
         * Gets the shader platform name used by the effects.
         */
        get shaderPlatformName(): string;
        /**
         * Enables or disables the snapshot rendering mode
         * Note that the WebGL engine does not support snapshot rendering so setting the value won't have any effect for this engine
         */
        get snapshotRendering(): boolean;
        set snapshotRendering(activate: boolean);
        protected _snapshotRenderingMode: number;
        /**
         * Gets or sets the snapshot rendering mode
         */
        get snapshotRenderingMode(): number;
        set snapshotRenderingMode(mode: number);
        protected _useExactSrgbConversions: boolean;
        /**
         * Gets a boolean indicating if the exact sRGB conversions or faster approximations are used for converting to and from linear space.
         */
        get useExactSrgbConversions(): boolean;
        /**
         * Creates a new snapshot at the next frame using the current snapshotRenderingMode
         */
        snapshotRenderingReset(): void;
        private _checkForMobile;
        private static _CreateCanvas;
        /**
         * Create a canvas. This method is overridden by other engines
         * @param width width
         * @param height height
         * @returns ICanvas interface
         */
        createCanvas(width: number, height: number): ICanvas;
        /**
         * Create an image to use with canvas
         * @returns IImage interface
         */
        createCanvasImage(): IImage;
        /**
         * Creates a new engine
         * @param canvasOrContext defines the canvas or WebGL context to use for rendering. If you provide a WebGL context, Babylon.js will not hook events on the canvas (like pointers, keyboards, etc...) so no event observables will be available. This is mostly used when Babylon.js is used as a plugin on a system which already used the WebGL context
         * @param antialias defines enable antialiasing (default: false)
         * @param options defines further options to be sent to the getContext() function
         * @param adaptToDeviceRatio defines whether to adapt to the device's viewport characteristics (default: false)
         */
        constructor(canvasOrContext: Nullable<HTMLCanvasElement | OffscreenCanvas | WebGLRenderingContext | WebGL2RenderingContext>, antialias?: boolean, options?: EngineOptions, adaptToDeviceRatio?: boolean);
        protected _restoreEngineAfterContextLost(initEngine: () => void): void;
        /**
         * Shared initialization across engines types.
         * @param canvas The canvas associated with this instance of the engine.
         * @param doNotHandleTouchAction Defines that engine should ignore modifying touch action attribute and style
         * @param audioEngine Defines if an audio engine should be created by default
         */
        protected _sharedInit(canvas: HTMLCanvasElement, doNotHandleTouchAction: boolean, audioEngine: boolean): void;
        /**
         * @internal
         */
        _getShaderProcessingContext(shaderLanguage: ShaderLanguage): Nullable<ShaderProcessingContext>;
        private _rebuildInternalTextures;
        private _rebuildRenderTargetWrappers;
        private _rebuildEffects;
        /**
         * Gets a boolean indicating if all created effects are ready
         * @returns true if all effects are ready
         */
        areAllEffectsReady(): boolean;
        protected _rebuildBuffers(): void;
        protected _initGLContext(): void;
        protected _initFeatures(): void;
        /**
         * Gets version of the current webGL context
         * Keep it for back compat - use version instead
         */
        get webGLVersion(): number;
        /**
         * Gets a string identifying the name of the class
         * @returns "Engine" string
         */
        getClassName(): string;
        /**
         * Returns true if the stencil buffer has been enabled through the creation option of the context.
         */
        get isStencilEnable(): boolean;
        /** @internal */
        _prepareWorkingCanvas(): void;
        /**
         * Reset the texture cache to empty state
         */
        resetTextureCache(): void;
        /**
         * Gets an object containing information about the current engine context
         * @returns an object containing the vendor, the renderer and the version of the current engine context
         */
        getInfo(): {
            vendor: string;
            renderer: string;
            version: string;
        };
        /**
         * Gets an object containing information about the current webGL context
         * @returns an object containing the vendor, the renderer and the version of the current webGL context
         */
        getGlInfo(): {
            vendor: string;
            renderer: string;
            version: string;
        };
        /**
         * Defines the hardware scaling level.
         * By default the hardware scaling level is computed from the window device ratio.
         * if level = 1 then the engine will render at the exact resolution of the canvas. If level = 0.5 then the engine will render at twice the size of the canvas.
         * @param level defines the level to use
         */
        setHardwareScalingLevel(level: number): void;
        /**
         * Gets the current hardware scaling level.
         * By default the hardware scaling level is computed from the window device ratio.
         * if level = 1 then the engine will render at the exact resolution of the canvas. If level = 0.5 then the engine will render at twice the size of the canvas.
         * @returns a number indicating the current hardware scaling level
         */
        getHardwareScalingLevel(): number;
        /**
         * Gets the list of loaded textures
         * @returns an array containing all loaded textures
         */
        getLoadedTexturesCache(): InternalTexture[];
        /**
         * Gets the object containing all engine capabilities
         * @returns the EngineCapabilities object
         */
        getCaps(): EngineCapabilities;
        /**
         * stop executing a render loop function and remove it from the execution array
         * @param renderFunction defines the function to be removed. If not provided all functions will be removed.
         */
        stopRenderLoop(renderFunction?: () => void): void;
        /** @internal */
        _renderLoop(): void;
        /**
         * Gets the HTML canvas attached with the current webGL context
         * @returns a HTML canvas
         */
        getRenderingCanvas(): Nullable<HTMLCanvasElement>;
        /**
         * Gets the audio context specified in engine initialization options
         * @returns an Audio Context
         */
        getAudioContext(): Nullable<AudioContext>;
        /**
         * Gets the audio destination specified in engine initialization options
         * @returns an audio destination node
         */
        getAudioDestination(): Nullable<AudioDestinationNode | MediaStreamAudioDestinationNode>;
        /**
         * Gets host window
         * @returns the host window object
         */
        getHostWindow(): Nullable<Window>;
        /**
         * Gets the current render width
         * @param useScreen defines if screen size must be used (or the current render target if any)
         * @returns a number defining the current render width
         */
        getRenderWidth(useScreen?: boolean): number;
        /**
         * Gets the current render height
         * @param useScreen defines if screen size must be used (or the current render target if any)
         * @returns a number defining the current render height
         */
        getRenderHeight(useScreen?: boolean): number;
        /**
         * Can be used to override the current requestAnimationFrame requester.
         * @internal
         */
        protected _queueNewFrame(bindedRenderFunction: any, requester?: any): number;
        /**
         * Register and execute a render loop. The engine can have more than one render function
         * @param renderFunction defines the function to continuously execute
         */
        runRenderLoop(renderFunction: () => void): void;
        /**
         * Clear the current render buffer or the current render target (if any is set up)
         * @param color defines the color to use
         * @param backBuffer defines if the back buffer must be cleared
         * @param depth defines if the depth buffer must be cleared
         * @param stencil defines if the stencil buffer must be cleared
         */
        clear(color: Nullable<IColor4Like>, backBuffer: boolean, depth: boolean, stencil?: boolean): void;
        protected _viewportCached: {
            x: number;
            y: number;
            z: number;
            w: number;
        };
        /**
         * @internal
         */
        _viewport(x: number, y: number, width: number, height: number): void;
        /**
         * Set the WebGL's viewport
         * @param viewport defines the viewport element to be used
         * @param requiredWidth defines the width required for rendering. If not provided the rendering canvas' width is used
         * @param requiredHeight defines the height required for rendering. If not provided the rendering canvas' height is used
         */
        setViewport(viewport: IViewportLike, requiredWidth?: number, requiredHeight?: number): void;
        /**
         * Begin a new frame
         */
        beginFrame(): void;
        /**
         * Enf the current frame
         */
        endFrame(): void;
        /**
         * Resize the view according to the canvas' size
         * @param forceSetSize true to force setting the sizes of the underlying canvas
         */
        resize(forceSetSize?: boolean): void;
        /**
         * Force a specific size of the canvas
         * @param width defines the new canvas' width
         * @param height defines the new canvas' height
         * @param forceSetSize true to force setting the sizes of the underlying canvas
         * @returns true if the size was changed
         */
        setSize(width: number, height: number, forceSetSize?: boolean): boolean;
        /**
         * Binds the frame buffer to the specified texture.
         * @param texture The render target wrapper to render to
         * @param faceIndex The face of the texture to render to in case of cube texture
         * @param requiredWidth The width of the target to render to
         * @param requiredHeight The height of the target to render to
         * @param forceFullscreenViewport Forces the viewport to be the entire texture/screen if true
         * @param lodLevel defines the lod level to bind to the frame buffer
         * @param layer defines the 2d array index to bind to frame buffer to
         */
        bindFramebuffer(texture: RenderTargetWrapper, faceIndex?: number, requiredWidth?: number, requiredHeight?: number, forceFullscreenViewport?: boolean, lodLevel?: number, layer?: number): void;
        /**
         * Set various states to the webGL context
         * @param culling defines culling state: true to enable culling, false to disable it
         * @param zOffset defines the value to apply to zOffset (0 by default)
         * @param force defines if states must be applied even if cache is up to date
         * @param reverseSide defines if culling must be reversed (CCW if false, CW if true)
         * @param cullBackFaces true to cull back faces, false to cull front faces (if culling is enabled)
         * @param stencil stencil states to set
         * @param zOffsetUnits defines the value to apply to zOffsetUnits (0 by default)
         */
        setState(culling: boolean, zOffset?: number, force?: boolean, reverseSide?: boolean, cullBackFaces?: boolean, stencil?: IStencilState, zOffsetUnits?: number): void;
        /**
         * Gets a boolean indicating if depth testing is enabled
         * @returns the current state
         */
        getDepthBuffer(): boolean;
        /**
         * Enable or disable depth buffering
         * @param enable defines the state to set
         */
        setDepthBuffer(enable: boolean): void;
        /**
         * Set the z offset Factor to apply to current rendering
         * @param value defines the offset to apply
         */
        setZOffset(value: number): void;
        /**
         * Gets the current value of the zOffset Factor
         * @returns the current zOffset Factor state
         */
        getZOffset(): number;
        /**
         * Set the z offset Units to apply to current rendering
         * @param value defines the offset to apply
         */
        setZOffsetUnits(value: number): void;
        /**
         * Gets the current value of the zOffset Units
         * @returns the current zOffset Units state
         */
        getZOffsetUnits(): number;
        /**
         * @internal
         */
        _bindUnboundFramebuffer(framebuffer: Nullable<WebGLFramebuffer>): void;
        /** @internal */
        _currentFrameBufferIsDefaultFrameBuffer(): boolean;
        /**
         * Generates the mipmaps for a texture
         * @param texture texture to generate the mipmaps for
         */
        generateMipmaps(texture: InternalTexture): void;
        /**
         * Unbind the current render target texture from the webGL context
         * @param texture defines the render target wrapper to unbind
         * @param disableGenerateMipMaps defines a boolean indicating that mipmaps must not be generated
         * @param onBeforeUnbind defines a function which will be called before the effective unbind
         */
        unBindFramebuffer(texture: RenderTargetWrapper, disableGenerateMipMaps?: boolean, onBeforeUnbind?: () => void): void;
        /**
         * Force a webGL flush (ie. a flush of all waiting webGL commands)
         */
        flushFramebuffer(): void;
        /**
         * Unbind the current render target and bind the default framebuffer
         */
        restoreDefaultFramebuffer(): void;
        /** @internal */
        protected _resetVertexBufferBinding(): void;
        /**
         * Creates a vertex buffer
         * @param data the data for the vertex buffer
         * @returns the new WebGL static buffer
         */
        createVertexBuffer(data: DataArray): DataBuffer;
        private _createVertexBuffer;
        /**
         * Creates a dynamic vertex buffer
         * @param data the data for the dynamic vertex buffer
         * @returns the new WebGL dynamic buffer
         */
        createDynamicVertexBuffer(data: DataArray): DataBuffer;
        protected _resetIndexBufferBinding(): void;
        /**
         * Creates a new index buffer
         * @param indices defines the content of the index buffer
         * @param updatable defines if the index buffer must be updatable
         * @returns a new webGL buffer
         */
        createIndexBuffer(indices: IndicesArray, updatable?: boolean): DataBuffer;
        protected _normalizeIndexData(indices: IndicesArray): Uint16Array | Uint32Array;
        /**
         * Bind a webGL buffer to the webGL context
         * @param buffer defines the buffer to bind
         */
        bindArrayBuffer(buffer: Nullable<DataBuffer>): void;
        protected bindIndexBuffer(buffer: Nullable<DataBuffer>): void;
        private _bindBuffer;
        /**
         * update the bound buffer with the given data
         * @param data defines the data to update
         */
        updateArrayBuffer(data: Float32Array): void;
        private _vertexAttribPointer;
        /**
         * @internal
         */
        _bindIndexBufferWithCache(indexBuffer: Nullable<DataBuffer>): void;
        private _bindVertexBuffersAttributes;
        /**
         * Records a vertex array object
         * @see https://doc.babylonjs.com/setup/support/webGL2#vertex-array-objects
         * @param vertexBuffers defines the list of vertex buffers to store
         * @param indexBuffer defines the index buffer to store
         * @param effect defines the effect to store
         * @param overrideVertexBuffers defines optional list of avertex buffers that overrides the entries in vertexBuffers
         * @returns the new vertex array object
         */
        recordVertexArrayObject(vertexBuffers: {
            [key: string]: VertexBuffer;
        }, indexBuffer: Nullable<DataBuffer>, effect: Effect, overrideVertexBuffers?: {
            [kind: string]: Nullable<VertexBuffer>;
        }): WebGLVertexArrayObject;
        /**
         * Bind a specific vertex array object
         * @see https://doc.babylonjs.com/setup/support/webGL2#vertex-array-objects
         * @param vertexArrayObject defines the vertex array object to bind
         * @param indexBuffer defines the index buffer to bind
         */
        bindVertexArrayObject(vertexArrayObject: WebGLVertexArrayObject, indexBuffer: Nullable<DataBuffer>): void;
        /**
         * Bind webGl buffers directly to the webGL context
         * @param vertexBuffer defines the vertex buffer to bind
         * @param indexBuffer defines the index buffer to bind
         * @param vertexDeclaration defines the vertex declaration to use with the vertex buffer
         * @param vertexStrideSize defines the vertex stride of the vertex buffer
         * @param effect defines the effect associated with the vertex buffer
         */
        bindBuffersDirectly(vertexBuffer: DataBuffer, indexBuffer: DataBuffer, vertexDeclaration: number[], vertexStrideSize: number, effect: Effect): void;
        private _unbindVertexArrayObject;
        /**
         * Bind a list of vertex buffers to the webGL context
         * @param vertexBuffers defines the list of vertex buffers to bind
         * @param indexBuffer defines the index buffer to bind
         * @param effect defines the effect associated with the vertex buffers
         * @param overrideVertexBuffers defines optional list of avertex buffers that overrides the entries in vertexBuffers
         */
        bindBuffers(vertexBuffers: {
            [key: string]: Nullable<VertexBuffer>;
        }, indexBuffer: Nullable<DataBuffer>, effect: Effect, overrideVertexBuffers?: {
            [kind: string]: Nullable<VertexBuffer>;
        }): void;
        /**
         * Unbind all instance attributes
         */
        unbindInstanceAttributes(): void;
        /**
         * Release and free the memory of a vertex array object
         * @param vao defines the vertex array object to delete
         */
        releaseVertexArrayObject(vao: WebGLVertexArrayObject): void;
        /**
         * @internal
         */
        _releaseBuffer(buffer: DataBuffer): boolean;
        protected _deleteBuffer(buffer: DataBuffer): void;
        /**
         * Update the content of a webGL buffer used with instantiation and bind it to the webGL context
         * @param instancesBuffer defines the webGL buffer to update and bind
         * @param data defines the data to store in the buffer
         * @param offsetLocations defines the offsets or attributes information used to determine where data must be stored in the buffer
         */
        updateAndBindInstancesBuffer(instancesBuffer: DataBuffer, data: Float32Array, offsetLocations: number[] | InstancingAttributeInfo[]): void;
        /**
         * Bind the content of a webGL buffer used with instantiation
         * @param instancesBuffer defines the webGL buffer to bind
         * @param attributesInfo defines the offsets or attributes information used to determine where data must be stored in the buffer
         * @param computeStride defines Whether to compute the strides from the info or use the default 0
         */
        bindInstancesBuffer(instancesBuffer: DataBuffer, attributesInfo: InstancingAttributeInfo[], computeStride?: boolean): void;
        /**
         * Disable the instance attribute corresponding to the name in parameter
         * @param name defines the name of the attribute to disable
         */
        disableInstanceAttributeByName(name: string): void;
        /**
         * Disable the instance attribute corresponding to the location in parameter
         * @param attributeLocation defines the attribute location of the attribute to disable
         */
        disableInstanceAttribute(attributeLocation: number): void;
        /**
         * Disable the attribute corresponding to the location in parameter
         * @param attributeLocation defines the attribute location of the attribute to disable
         */
        disableAttributeByIndex(attributeLocation: number): void;
        /**
         * Send a draw order
         * @param useTriangles defines if triangles must be used to draw (else wireframe will be used)
         * @param indexStart defines the starting index
         * @param indexCount defines the number of index to draw
         * @param instancesCount defines the number of instances to draw (if instantiation is enabled)
         */
        draw(useTriangles: boolean, indexStart: number, indexCount: number, instancesCount?: number): void;
        /**
         * Draw a list of points
         * @param verticesStart defines the index of first vertex to draw
         * @param verticesCount defines the count of vertices to draw
         * @param instancesCount defines the number of instances to draw (if instantiation is enabled)
         */
        drawPointClouds(verticesStart: number, verticesCount: number, instancesCount?: number): void;
        /**
         * Draw a list of unindexed primitives
         * @param useTriangles defines if triangles must be used to draw (else wireframe will be used)
         * @param verticesStart defines the index of first vertex to draw
         * @param verticesCount defines the count of vertices to draw
         * @param instancesCount defines the number of instances to draw (if instantiation is enabled)
         */
        drawUnIndexed(useTriangles: boolean, verticesStart: number, verticesCount: number, instancesCount?: number): void;
        /**
         * Draw a list of indexed primitives
         * @param fillMode defines the primitive to use
         * @param indexStart defines the starting index
         * @param indexCount defines the number of index to draw
         * @param instancesCount defines the number of instances to draw (if instantiation is enabled)
         */
        drawElementsType(fillMode: number, indexStart: number, indexCount: number, instancesCount?: number): void;
        /**
         * Draw a list of unindexed primitives
         * @param fillMode defines the primitive to use
         * @param verticesStart defines the index of first vertex to draw
         * @param verticesCount defines the count of vertices to draw
         * @param instancesCount defines the number of instances to draw (if instantiation is enabled)
         */
        drawArraysType(fillMode: number, verticesStart: number, verticesCount: number, instancesCount?: number): void;
        private _drawMode;
        /** @internal */
        protected _reportDrawCall(): void;
        /**
         * @internal
         */
        _releaseEffect(effect: Effect): void;
        /**
         * @internal
         */
        _deletePipelineContext(pipelineContext: IPipelineContext): void;
        /** @internal */
        _getGlobalDefines(defines?: {
            [key: string]: string;
        }): string | undefined;
        /**
         * Create a new effect (used to store vertex/fragment shaders)
         * @param baseName defines the base name of the effect (The name of file without .fragment.fx or .vertex.fx)
         * @param attributesNamesOrOptions defines either a list of attribute names or an IEffectCreationOptions object
         * @param uniformsNamesOrEngine defines either a list of uniform names or the engine to use
         * @param samplers defines an array of string used to represent textures
         * @param defines defines the string containing the defines to use to compile the shaders
         * @param fallbacks defines the list of potential fallbacks to use if shader compilation fails
         * @param onCompiled defines a function to call when the effect creation is successful
         * @param onError defines a function to call when the effect creation has failed
         * @param indexParameters defines an object containing the index values to use to compile shaders (like the maximum number of simultaneous lights)
         * @param shaderLanguage the language the shader is written in (default: GLSL)
         * @returns the new Effect
         */
        createEffect(baseName: any, attributesNamesOrOptions: string[] | IEffectCreationOptions, uniformsNamesOrEngine: string[] | ThinEngine, samplers?: string[], defines?: string, fallbacks?: IEffectFallbacks, onCompiled?: Nullable<(effect: Effect) => void>, onError?: Nullable<(effect: Effect, errors: string) => void>, indexParameters?: any, shaderLanguage?: ShaderLanguage): Effect;
        protected static _ConcatenateShader(source: string, defines: Nullable<string>, shaderVersion?: string): string;
        private _compileShader;
        private _compileRawShader;
        /**
         * @internal
         */
        _getShaderSource(shader: WebGLShader): Nullable<string>;
        /**
         * Directly creates a webGL program
         * @param pipelineContext  defines the pipeline context to attach to
         * @param vertexCode defines the vertex shader code to use
         * @param fragmentCode defines the fragment shader code to use
         * @param context defines the webGL context to use (if not set, the current one will be used)
         * @param transformFeedbackVaryings defines the list of transform feedback varyings to use
         * @returns the new webGL program
         */
        createRawShaderProgram(pipelineContext: IPipelineContext, vertexCode: string, fragmentCode: string, context?: WebGLRenderingContext, transformFeedbackVaryings?: Nullable<string[]>): WebGLProgram;
        /**
         * Creates a webGL program
         * @param pipelineContext  defines the pipeline context to attach to
         * @param vertexCode  defines the vertex shader code to use
         * @param fragmentCode defines the fragment shader code to use
         * @param defines defines the string containing the defines to use to compile the shaders
         * @param context defines the webGL context to use (if not set, the current one will be used)
         * @param transformFeedbackVaryings defines the list of transform feedback varyings to use
         * @returns the new webGL program
         */
        createShaderProgram(pipelineContext: IPipelineContext, vertexCode: string, fragmentCode: string, defines: Nullable<string>, context?: WebGLRenderingContext, transformFeedbackVaryings?: Nullable<string[]>): WebGLProgram;
        /**
         * Inline functions in shader code that are marked to be inlined
         * @param code code to inline
         * @returns inlined code
         */
        inlineShaderCode(code: string): string;
        /**
         * Creates a new pipeline context
         * @param shaderProcessingContext defines the shader processing context used during the processing if available
         * @returns the new pipeline
         */
        createPipelineContext(shaderProcessingContext: Nullable<ShaderProcessingContext>): IPipelineContext;
        /**
         * Creates a new material context
         * @returns the new context
         */
        createMaterialContext(): IMaterialContext | undefined;
        /**
         * Creates a new draw context
         * @returns the new context
         */
        createDrawContext(): IDrawContext | undefined;
        protected _createShaderProgram(pipelineContext: WebGLPipelineContext, vertexShader: WebGLShader, fragmentShader: WebGLShader, context: WebGLRenderingContext, transformFeedbackVaryings?: Nullable<string[]>): WebGLProgram;
        protected _finalizePipelineContext(pipelineContext: WebGLPipelineContext): void;
        /**
         * @internal
         */
        _preparePipelineContext(pipelineContext: IPipelineContext, vertexSourceCode: string, fragmentSourceCode: string, createAsRaw: boolean, rawVertexSourceCode: string, rawFragmentSourceCode: string, rebuildRebind: any, defines: Nullable<string>, transformFeedbackVaryings: Nullable<string[]>, key: string): void;
        /**
         * @internal
         */
        _isRenderingStateCompiled(pipelineContext: IPipelineContext): boolean;
        /**
         * @internal
         */
        _executeWhenRenderingStateIsCompiled(pipelineContext: IPipelineContext, action: () => void): void;
        /**
         * Gets the list of webGL uniform locations associated with a specific program based on a list of uniform names
         * @param pipelineContext defines the pipeline context to use
         * @param uniformsNames defines the list of uniform names
         * @returns an array of webGL uniform locations
         */
        getUniforms(pipelineContext: IPipelineContext, uniformsNames: string[]): Nullable<WebGLUniformLocation>[];
        /**
         * Gets the list of active attributes for a given webGL program
         * @param pipelineContext defines the pipeline context to use
         * @param attributesNames defines the list of attribute names to get
         * @returns an array of indices indicating the offset of each attribute
         */
        getAttributes(pipelineContext: IPipelineContext, attributesNames: string[]): number[];
        /**
         * Activates an effect, making it the current one (ie. the one used for rendering)
         * @param effect defines the effect to activate
         */
        enableEffect(effect: Nullable<Effect | DrawWrapper>): void;
        /**
         * Set the value of an uniform to a number (int)
         * @param uniform defines the webGL uniform location where to store the value
         * @param value defines the int number to store
         * @returns true if the value was set
         */
        setInt(uniform: Nullable<WebGLUniformLocation>, value: number): boolean;
        /**
         * Set the value of an uniform to a int2
         * @param uniform defines the webGL uniform location where to store the value
         * @param x defines the 1st component of the value
         * @param y defines the 2nd component of the value
         * @returns true if the value was set
         */
        setInt2(uniform: Nullable<WebGLUniformLocation>, x: number, y: number): boolean;
        /**
         * Set the value of an uniform to a int3
         * @param uniform defines the webGL uniform location where to store the value
         * @param x defines the 1st component of the value
         * @param y defines the 2nd component of the value
         * @param z defines the 3rd component of the value
         * @returns true if the value was set
         */
        setInt3(uniform: Nullable<WebGLUniformLocation>, x: number, y: number, z: number): boolean;
        /**
         * Set the value of an uniform to a int4
         * @param uniform defines the webGL uniform location where to store the value
         * @param x defines the 1st component of the value
         * @param y defines the 2nd component of the value
         * @param z defines the 3rd component of the value
         * @param w defines the 4th component of the value
         * @returns true if the value was set
         */
        setInt4(uniform: Nullable<WebGLUniformLocation>, x: number, y: number, z: number, w: number): boolean;
        /**
         * Set the value of an uniform to an array of int32
         * @param uniform defines the webGL uniform location where to store the value
         * @param array defines the array of int32 to store
         * @returns true if the value was set
         */
        setIntArray(uniform: Nullable<WebGLUniformLocation>, array: Int32Array): boolean;
        /**
         * Set the value of an uniform to an array of int32 (stored as vec2)
         * @param uniform defines the webGL uniform location where to store the value
         * @param array defines the array of int32 to store
         * @returns true if the value was set
         */
        setIntArray2(uniform: Nullable<WebGLUniformLocation>, array: Int32Array): boolean;
        /**
         * Set the value of an uniform to an array of int32 (stored as vec3)
         * @param uniform defines the webGL uniform location where to store the value
         * @param array defines the array of int32 to store
         * @returns true if the value was set
         */
        setIntArray3(uniform: Nullable<WebGLUniformLocation>, array: Int32Array): boolean;
        /**
         * Set the value of an uniform to an array of int32 (stored as vec4)
         * @param uniform defines the webGL uniform location where to store the value
         * @param array defines the array of int32 to store
         * @returns true if the value was set
         */
        setIntArray4(uniform: Nullable<WebGLUniformLocation>, array: Int32Array): boolean;
        /**
         * Set the value of an uniform to a number (unsigned int)
         * @param uniform defines the webGL uniform location where to store the value
         * @param value defines the unsigned int number to store
         * @returns true if the value was set
         */
        setUInt(uniform: Nullable<WebGLUniformLocation>, value: number): boolean;
        /**
         * Set the value of an uniform to a unsigned int2
         * @param uniform defines the webGL uniform location where to store the value
         * @param x defines the 1st component of the value
         * @param y defines the 2nd component of the value
         * @returns true if the value was set
         */
        setUInt2(uniform: Nullable<WebGLUniformLocation>, x: number, y: number): boolean;
        /**
         * Set the value of an uniform to a unsigned int3
         * @param uniform defines the webGL uniform location where to store the value
         * @param x defines the 1st component of the value
         * @param y defines the 2nd component of the value
         * @param z defines the 3rd component of the value
         * @returns true if the value was set
         */
        setUInt3(uniform: Nullable<WebGLUniformLocation>, x: number, y: number, z: number): boolean;
        /**
         * Set the value of an uniform to a unsigned int4
         * @param uniform defines the webGL uniform location where to store the value
         * @param x defines the 1st component of the value
         * @param y defines the 2nd component of the value
         * @param z defines the 3rd component of the value
         * @param w defines the 4th component of the value
         * @returns true if the value was set
         */
        setUInt4(uniform: Nullable<WebGLUniformLocation>, x: number, y: number, z: number, w: number): boolean;
        /**
         * Set the value of an uniform to an array of unsigned int32
         * @param uniform defines the webGL uniform location where to store the value
         * @param array defines the array of unsigned int32 to store
         * @returns true if the value was set
         */
        setUIntArray(uniform: Nullable<WebGLUniformLocation>, array: Uint32Array): boolean;
        /**
         * Set the value of an uniform to an array of unsigned int32 (stored as vec2)
         * @param uniform defines the webGL uniform location where to store the value
         * @param array defines the array of unsigned int32 to store
         * @returns true if the value was set
         */
        setUIntArray2(uniform: Nullable<WebGLUniformLocation>, array: Uint32Array): boolean;
        /**
         * Set the value of an uniform to an array of unsigned int32 (stored as vec3)
         * @param uniform defines the webGL uniform location where to store the value
         * @param array defines the array of unsigned int32 to store
         * @returns true if the value was set
         */
        setUIntArray3(uniform: Nullable<WebGLUniformLocation>, array: Uint32Array): boolean;
        /**
         * Set the value of an uniform to an array of unsigned int32 (stored as vec4)
         * @param uniform defines the webGL uniform location where to store the value
         * @param array defines the array of unsigned int32 to store
         * @returns true if the value was set
         */
        setUIntArray4(uniform: Nullable<WebGLUniformLocation>, array: Uint32Array): boolean;
        /**
         * Set the value of an uniform to an array of number
         * @param uniform defines the webGL uniform location where to store the value
         * @param array defines the array of number to store
         * @returns true if the value was set
         */
        setArray(uniform: Nullable<WebGLUniformLocation>, array: number[] | Float32Array): boolean;
        /**
         * Set the value of an uniform to an array of number (stored as vec2)
         * @param uniform defines the webGL uniform location where to store the value
         * @param array defines the array of number to store
         * @returns true if the value was set
         */
        setArray2(uniform: Nullable<WebGLUniformLocation>, array: number[] | Float32Array): boolean;
        /**
         * Set the value of an uniform to an array of number (stored as vec3)
         * @param uniform defines the webGL uniform location where to store the value
         * @param array defines the array of number to store
         * @returns true if the value was set
         */
        setArray3(uniform: Nullable<WebGLUniformLocation>, array: number[] | Float32Array): boolean;
        /**
         * Set the value of an uniform to an array of number (stored as vec4)
         * @param uniform defines the webGL uniform location where to store the value
         * @param array defines the array of number to store
         * @returns true if the value was set
         */
        setArray4(uniform: Nullable<WebGLUniformLocation>, array: number[] | Float32Array): boolean;
        /**
         * Set the value of an uniform to an array of float32 (stored as matrices)
         * @param uniform defines the webGL uniform location where to store the value
         * @param matrices defines the array of float32 to store
         * @returns true if the value was set
         */
        setMatrices(uniform: Nullable<WebGLUniformLocation>, matrices: Float32Array): boolean;
        /**
         * Set the value of an uniform to a matrix (3x3)
         * @param uniform defines the webGL uniform location where to store the value
         * @param matrix defines the Float32Array representing the 3x3 matrix to store
         * @returns true if the value was set
         */
        setMatrix3x3(uniform: Nullable<WebGLUniformLocation>, matrix: Float32Array): boolean;
        /**
         * Set the value of an uniform to a matrix (2x2)
         * @param uniform defines the webGL uniform location where to store the value
         * @param matrix defines the Float32Array representing the 2x2 matrix to store
         * @returns true if the value was set
         */
        setMatrix2x2(uniform: Nullable<WebGLUniformLocation>, matrix: Float32Array): boolean;
        /**
         * Set the value of an uniform to a number (float)
         * @param uniform defines the webGL uniform location where to store the value
         * @param value defines the float number to store
         * @returns true if the value was transferred
         */
        setFloat(uniform: Nullable<WebGLUniformLocation>, value: number): boolean;
        /**
         * Set the value of an uniform to a vec2
         * @param uniform defines the webGL uniform location where to store the value
         * @param x defines the 1st component of the value
         * @param y defines the 2nd component of the value
         * @returns true if the value was set
         */
        setFloat2(uniform: Nullable<WebGLUniformLocation>, x: number, y: number): boolean;
        /**
         * Set the value of an uniform to a vec3
         * @param uniform defines the webGL uniform location where to store the value
         * @param x defines the 1st component of the value
         * @param y defines the 2nd component of the value
         * @param z defines the 3rd component of the value
         * @returns true if the value was set
         */
        setFloat3(uniform: Nullable<WebGLUniformLocation>, x: number, y: number, z: number): boolean;
        /**
         * Set the value of an uniform to a vec4
         * @param uniform defines the webGL uniform location where to store the value
         * @param x defines the 1st component of the value
         * @param y defines the 2nd component of the value
         * @param z defines the 3rd component of the value
         * @param w defines the 4th component of the value
         * @returns true if the value was set
         */
        setFloat4(uniform: Nullable<WebGLUniformLocation>, x: number, y: number, z: number, w: number): boolean;
        /**
         * Apply all cached states (depth, culling, stencil and alpha)
         */
        applyStates(): void;
        /**
         * Enable or disable color writing
         * @param enable defines the state to set
         */
        setColorWrite(enable: boolean): void;
        /**
         * Gets a boolean indicating if color writing is enabled
         * @returns the current color writing state
         */
        getColorWrite(): boolean;
        /**
         * Gets the depth culling state manager
         */
        get depthCullingState(): DepthCullingState;
        /**
         * Gets the alpha state manager
         */
        get alphaState(): AlphaState;
        /**
         * Gets the stencil state manager
         */
        get stencilState(): StencilState;
        /**
         * Gets the stencil state composer
         */
        get stencilStateComposer(): StencilStateComposer;
        /**
         * Clears the list of texture accessible through engine.
         * This can help preventing texture load conflict due to name collision.
         */
        clearInternalTexturesCache(): void;
        /**
         * Force the entire cache to be cleared
         * You should not have to use this function unless your engine needs to share the webGL context with another engine
         * @param bruteForce defines a boolean to force clearing ALL caches (including stencil, detoh and alpha states)
         */
        wipeCaches(bruteForce?: boolean): void;
        /**
         * @internal
         */
        _getSamplingParameters(samplingMode: number, generateMipMaps: boolean): {
            min: number;
            mag: number;
        };
        /** @internal */
        protected _createTexture(): WebGLTexture;
        /** @internal */
        _createHardwareTexture(): HardwareTextureWrapper;
        /**
         * Creates an internal texture without binding it to a framebuffer
         * @internal
         * @param size defines the size of the texture
         * @param options defines the options used to create the texture
         * @param delayGPUTextureCreation true to delay the texture creation the first time it is really needed. false to create it right away
         * @param source source type of the texture
         * @returns a new internal texture
         */
        _createInternalTexture(size: TextureSize, options: boolean | InternalTextureCreationOptions, delayGPUTextureCreation?: boolean, source?: InternalTextureSource): InternalTexture;
        /**
         * @internal
         */
        _getUseSRGBBuffer(useSRGBBuffer: boolean, noMipmap: boolean): boolean;
        protected _createTextureBase(url: Nullable<string>, noMipmap: boolean, invertY: boolean, scene: Nullable<ISceneLike>, samplingMode: number | undefined, onLoad: Nullable<(texture: InternalTexture) => void> | undefined, onError: Nullable<(message: string, exception: any) => void> | undefined, prepareTexture: (texture: InternalTexture, extension: string, scene: Nullable<ISceneLike>, img: HTMLImageElement | ImageBitmap | {
            width: number;
            height: number;
        }, invertY: boolean, noMipmap: boolean, isCompressed: boolean, processFunction: (width: number, height: number, img: HTMLImageElement | ImageBitmap | {
            width: number;
            height: number;
        }, extension: string, texture: InternalTexture, continuationCallback: () => void) => boolean, samplingMode: number) => void, prepareTextureProcessFunction: (width: number, height: number, img: HTMLImageElement | ImageBitmap | {
            width: number;
            height: number;
        }, extension: string, texture: InternalTexture, continuationCallback: () => void) => boolean, buffer?: Nullable<string | ArrayBuffer | ArrayBufferView | HTMLImageElement | Blob | ImageBitmap>, fallback?: Nullable<InternalTexture>, format?: Nullable<number>, forcedExtension?: Nullable<string>, mimeType?: string, loaderOptions?: any, useSRGBBuffer?: boolean): InternalTexture;
        /**
         * Usually called from Texture.ts.
         * Passed information to create a WebGLTexture
         * @param url defines a value which contains one of the following:
         * * A conventional http URL, e.g. 'http://...' or 'file://...'
         * * A base64 string of in-line texture data, e.g. 'data:image/jpg;base64,/...'
         * * An indicator that data being passed using the buffer parameter, e.g. 'data:mytexture.jpg'
         * @param noMipmap defines a boolean indicating that no mipmaps shall be generated.  Ignored for compressed textures.  They must be in the file
         * @param invertY when true, image is flipped when loaded.  You probably want true. Certain compressed textures may invert this if their default is inverted (eg. ktx)
         * @param scene needed for loading to the correct scene
         * @param samplingMode mode with should be used sample / access the texture (Default: Texture.TRILINEAR_SAMPLINGMODE)
         * @param onLoad optional callback to be called upon successful completion
         * @param onError optional callback to be called upon failure
         * @param buffer a source of a file previously fetched as either a base64 string, an ArrayBuffer (compressed or image format), HTMLImageElement (image format), or a Blob
         * @param fallback an internal argument in case the function must be called again, due to etc1 not having alpha capabilities
         * @param format internal format.  Default: RGB when extension is '.jpg' else RGBA.  Ignored for compressed textures
         * @param forcedExtension defines the extension to use to pick the right loader
         * @param mimeType defines an optional mime type
         * @param loaderOptions options to be passed to the loader
         * @param creationFlags specific flags to use when creating the texture (Constants.TEXTURE_CREATIONFLAG_STORAGE for storage textures, for eg)
         * @param useSRGBBuffer defines if the texture must be loaded in a sRGB GPU buffer (if supported by the GPU).
         * @returns a InternalTexture for assignment back into BABYLON.Texture
         */
        createTexture(url: Nullable<string>, noMipmap: boolean, invertY: boolean, scene: Nullable<ISceneLike>, samplingMode?: number, onLoad?: Nullable<(texture: InternalTexture) => void>, onError?: Nullable<(message: string, exception: any) => void>, buffer?: Nullable<string | ArrayBuffer | ArrayBufferView | HTMLImageElement | Blob | ImageBitmap>, fallback?: Nullable<InternalTexture>, format?: Nullable<number>, forcedExtension?: Nullable<string>, mimeType?: string, loaderOptions?: any, creationFlags?: number, useSRGBBuffer?: boolean): InternalTexture;
        /**
         * Loads an image as an HTMLImageElement.
         * @param input url string, ArrayBuffer, or Blob to load
         * @param onLoad callback called when the image successfully loads
         * @param onError callback called when the image fails to load
         * @param offlineProvider offline provider for caching
         * @param mimeType optional mime type
         * @param imageBitmapOptions optional the options to use when creating an ImageBitmap
         * @returns the HTMLImageElement of the loaded image
         * @internal
         */
        static _FileToolsLoadImage(input: string | ArrayBuffer | ArrayBufferView | Blob, onLoad: (img: HTMLImageElement | ImageBitmap) => void, onError: (message?: string, exception?: any) => void, offlineProvider: Nullable<IOfflineProvider>, mimeType?: string, imageBitmapOptions?: ImageBitmapOptions): Nullable<HTMLImageElement>;
        /**
         * @internal
         */
        _rescaleTexture(source: InternalTexture, destination: InternalTexture, scene: Nullable<any>, internalFormat: number, onComplete: () => void): void;
        private _unpackFlipYCached;
        /**
         * In case you are sharing the context with other applications, it might
         * be interested to not cache the unpack flip y state to ensure a consistent
         * value would be set.
         */
        enableUnpackFlipYCached: boolean;
        /**
         * @internal
         */
        _unpackFlipY(value: boolean): void;
        /** @internal */
        _getUnpackAlignement(): number;
        private _getTextureTarget;
        /**
         * Update the sampling mode of a given texture
         * @param samplingMode defines the required sampling mode
         * @param texture defines the texture to update
         * @param generateMipMaps defines whether to generate mipmaps for the texture
         */
        updateTextureSamplingMode(samplingMode: number, texture: InternalTexture, generateMipMaps?: boolean): void;
        /**
         * Update the dimensions of a texture
         * @param texture texture to update
         * @param width new width of the texture
         * @param height new height of the texture
         * @param depth new depth of the texture
         */
        updateTextureDimensions(texture: InternalTexture, width: number, height: number, depth?: number): void;
        /**
         * Update the sampling mode of a given texture
         * @param texture defines the texture to update
         * @param wrapU defines the texture wrap mode of the u coordinates
         * @param wrapV defines the texture wrap mode of the v coordinates
         * @param wrapR defines the texture wrap mode of the r coordinates
         */
        updateTextureWrappingMode(texture: InternalTexture, wrapU: Nullable<number>, wrapV?: Nullable<number>, wrapR?: Nullable<number>): void;
        /**
         * @internal
         */
        _setupDepthStencilTexture(internalTexture: InternalTexture, size: number | {
            width: number;
            height: number;
            layers?: number;
        }, generateStencil: boolean, bilinearFiltering: boolean, comparisonFunction: number, samples?: number): void;
        /**
         * @internal
         */
        _uploadCompressedDataToTextureDirectly(texture: InternalTexture, internalFormat: number, width: number, height: number, data: ArrayBufferView, faceIndex?: number, lod?: number): void;
        /**
         * @internal
         */
        _uploadDataToTextureDirectly(texture: InternalTexture, imageData: ArrayBufferView, faceIndex?: number, lod?: number, babylonInternalFormat?: number, useTextureWidthAndHeight?: boolean): void;
        /**
         * Update a portion of an internal texture
         * @param texture defines the texture to update
         * @param imageData defines the data to store into the texture
         * @param xOffset defines the x coordinates of the update rectangle
         * @param yOffset defines the y coordinates of the update rectangle
         * @param width defines the width of the update rectangle
         * @param height defines the height of the update rectangle
         * @param faceIndex defines the face index if texture is a cube (0 by default)
         * @param lod defines the lod level to update (0 by default)
         * @param generateMipMaps defines whether to generate mipmaps or not
         */
        updateTextureData(texture: InternalTexture, imageData: ArrayBufferView, xOffset: number, yOffset: number, width: number, height: number, faceIndex?: number, lod?: number, generateMipMaps?: boolean): void;
        /**
         * @internal
         */
        _uploadArrayBufferViewToTexture(texture: InternalTexture, imageData: ArrayBufferView, faceIndex?: number, lod?: number): void;
        protected _prepareWebGLTextureContinuation(texture: InternalTexture, scene: Nullable<ISceneLike>, noMipmap: boolean, isCompressed: boolean, samplingMode: number): void;
        private _prepareWebGLTexture;
        /**
         * @internal
         */
        _setupFramebufferDepthAttachments(generateStencilBuffer: boolean, generateDepthBuffer: boolean, width: number, height: number, samples?: number): Nullable<WebGLRenderbuffer>;
        /**
         * @internal
         */
        _createRenderBuffer(width: number, height: number, samples: number, internalFormat: number, msInternalFormat: number, attachment: number, unbindBuffer?: boolean): Nullable<WebGLRenderbuffer>;
        _updateRenderBuffer(renderBuffer: Nullable<WebGLRenderbuffer>, width: number, height: number, samples: number, internalFormat: number, msInternalFormat: number, attachment: number, unbindBuffer?: boolean): Nullable<WebGLRenderbuffer>;
        /**
         * @internal
         */
        _releaseTexture(texture: InternalTexture): void;
        /**
         * @internal
         */
        _releaseRenderTargetWrapper(rtWrapper: RenderTargetWrapper): void;
        protected _deleteTexture(texture: Nullable<WebGLTexture>): void;
        protected _setProgram(program: WebGLProgram): void;
        protected _boundUniforms: {
            [key: number]: WebGLUniformLocation;
        };
        /**
         * Binds an effect to the webGL context
         * @param effect defines the effect to bind
         */
        bindSamplers(effect: Effect): void;
        private _activateCurrentTexture;
        /**
         * @internal
         */
        _bindTextureDirectly(target: number, texture: Nullable<InternalTexture>, forTextureDataUpdate?: boolean, force?: boolean): boolean;
        /**
         * @internal
         */
        _bindTexture(channel: number, texture: Nullable<InternalTexture>, name: string): void;
        /**
         * Unbind all textures from the webGL context
         */
        unbindAllTextures(): void;
        /**
         * Sets a texture to the according uniform.
         * @param channel The texture channel
         * @param uniform The uniform to set
         * @param texture The texture to apply
         * @param name The name of the uniform in the effect
         */
        setTexture(channel: number, uniform: Nullable<WebGLUniformLocation>, texture: Nullable<ThinTexture>, name: string): void;
        private _bindSamplerUniformToChannel;
        private _getTextureWrapMode;
        protected _setTexture(channel: number, texture: Nullable<ThinTexture>, isPartOfTextureArray?: boolean, depthStencilTexture?: boolean, name?: string): boolean;
        /**
         * Sets an array of texture to the webGL context
         * @param channel defines the channel where the texture array must be set
         * @param uniform defines the associated uniform location
         * @param textures defines the array of textures to bind
         * @param name name of the channel
         */
        setTextureArray(channel: number, uniform: Nullable<WebGLUniformLocation>, textures: ThinTexture[], name: string): void;
        /**
         * @internal
         */
        _setAnisotropicLevel(target: number, internalTexture: InternalTexture, anisotropicFilteringLevel: number): void;
        private _setTextureParameterFloat;
        private _setTextureParameterInteger;
        /**
         * Unbind all vertex attributes from the webGL context
         */
        unbindAllAttributes(): void;
        /**
         * Force the engine to release all cached effects. This means that next effect compilation will have to be done completely even if a similar effect was already compiled
         */
        releaseEffects(): void;
        /**
         * Dispose and release all associated resources
         */
        dispose(): void;
        /**
         * Attach a new callback raised when context lost event is fired
         * @param callback defines the callback to call
         */
        attachContextLostEvent(callback: (event: WebGLContextEvent) => void): void;
        /**
         * Attach a new callback raised when context restored event is fired
         * @param callback defines the callback to call
         */
        attachContextRestoredEvent(callback: (event: WebGLContextEvent) => void): void;
        /**
         * Get the current error code of the webGL context
         * @returns the error code
         * @see https://developer.mozilla.org/en-US/docs/Web/API/WebGLRenderingContext/getError
         */
        getError(): number;
        private _canRenderToFloatFramebuffer;
        private _canRenderToHalfFloatFramebuffer;
        private _canRenderToFramebuffer;
        /**
         * @internal
         */
        _getWebGLTextureType(type: number): number;
        /**
         * @internal
         */
        _getInternalFormat(format: number, useSRGBBuffer?: boolean): number;
        /**
         * @internal
         */
        _getRGBABufferInternalSizedFormat(type: number, format?: number, useSRGBBuffer?: boolean): number;
        /**
         * @internal
         */
        _getRGBAMultiSampleBufferFormat(type: number): number;
        /**
         * @internal
         */
        _loadFile(url: string, onSuccess: (data: string | ArrayBuffer, responseURL?: string) => void, onProgress?: (data: any) => void, offlineProvider?: IOfflineProvider, useArrayBuffer?: boolean, onError?: (request?: IWebRequest, exception?: any) => void): IFileRequest;
        /**
         * Loads a file from a url
         * @param url url to load
         * @param onSuccess callback called when the file successfully loads
         * @param onProgress callback called while file is loading (if the server supports this mode)
         * @param offlineProvider defines the offline provider for caching
         * @param useArrayBuffer defines a boolean indicating that date must be returned as ArrayBuffer
         * @param onError callback called when the file fails to load
         * @returns a file request object
         * @internal
         */
        static _FileToolsLoadFile(url: string, onSuccess: (data: string | ArrayBuffer, responseURL?: string) => void, onProgress?: (ev: ProgressEvent) => void, offlineProvider?: IOfflineProvider, useArrayBuffer?: boolean, onError?: (request?: WebRequest, exception?: LoadFileError) => void): IFileRequest;
        /**
         * Reads pixels from the current frame buffer. Please note that this function can be slow
         * @param x defines the x coordinate of the rectangle where pixels must be read
         * @param y defines the y coordinate of the rectangle where pixels must be read
         * @param width defines the width of the rectangle where pixels must be read
         * @param height defines the height of the rectangle where pixels must be read
         * @param hasAlpha defines whether the output should have alpha or not (defaults to true)
         * @param flushRenderer true to flush the renderer from the pending commands before reading the pixels
         * @returns a ArrayBufferView promise (Uint8Array) containing RGBA colors
         */
        readPixels(x: number, y: number, width: number, height: number, hasAlpha?: boolean, flushRenderer?: boolean): Promise<ArrayBufferView>;
        private static _IsSupported;
        private static _HasMajorPerformanceCaveat;
        /**
         * Gets a Promise<boolean> indicating if the engine can be instantiated (ie. if a webGL context can be found)
         */
        static get IsSupportedAsync(): Promise<boolean>;
        /**
         * Gets a boolean indicating if the engine can be instantiated (ie. if a webGL context can be found)
         */
        static get IsSupported(): boolean;
        /**
         * Gets a boolean indicating if the engine can be instantiated (ie. if a webGL context can be found)
         * @returns true if the engine can be created
         * @ignorenaming
         */
        static isSupported(): boolean;
        /**
         * Gets a boolean indicating if the engine can be instantiated on a performant device (ie. if a webGL context can be found and it does not use a slow implementation)
         */
        static get HasMajorPerformanceCaveat(): boolean;
        /**
         * Find the next highest power of two.
         * @param x Number to start search from.
         * @returns Next highest power of two.
         */
        static CeilingPOT(x: number): number;
        /**
         * Find the next lowest power of two.
         * @param x Number to start search from.
         * @returns Next lowest power of two.
         */
        static FloorPOT(x: number): number;
        /**
         * Find the nearest power of two.
         * @param x Number to start search from.
         * @returns Next nearest power of two.
         */
        static NearestPOT(x: number): number;
        /**
         * Get the closest exponent of two
         * @param value defines the value to approximate
         * @param max defines the maximum value to return
         * @param mode defines how to define the closest value
         * @returns closest exponent of two of the given value
         */
        static GetExponentOfTwo(value: number, max: number, mode?: number): number;
        /**
         * Queue a new function into the requested animation frame pool (ie. this function will be executed by the browser (or the javascript engine) for the next frame)
         * @param func - the function to be called
         * @param requester - the object that will request the next frame. Falls back to window.
         * @returns frame number
         */
        static QueueNewFrame(func: () => void, requester?: any): number;
        /**
         * Gets host document
         * @returns the host document object
         */
        getHostDocument(): Nullable<Document>;
    }


    /** @internal */
    export class WebGL2ShaderProcessor implements IShaderProcessor {
        shaderLanguage: ShaderLanguage;
        attributeProcessor(attribute: string): string;
        varyingProcessor(varying: string, isFragment: boolean): string;
        postProcessor(code: string, defines: string[], isFragment: boolean): string;
    }


    /** @internal */
    export class WebGLHardwareTexture implements HardwareTextureWrapper {
        private _webGLTexture;
        private _context;
        _MSAARenderBuffer: Nullable<WebGLRenderbuffer>;
        get underlyingResource(): Nullable<WebGLTexture>;
        constructor(existingTexture: Nullable<WebGLTexture> | undefined, context: WebGLRenderingContext);
        setUsage(): void;
        set(hardwareTexture: WebGLTexture): void;
        reset(): void;
        release(): void;
    }


    /** @internal */
    export class WebGLPipelineContext implements IPipelineContext {
        private _valueCache;
        private _uniforms;
        engine: ThinEngine;
        program: Nullable<WebGLProgram>;
        context?: WebGLRenderingContext;
        vertexShader?: WebGLShader;
        fragmentShader?: WebGLShader;
        isParallelCompiled: boolean;
        onCompiled?: () => void;
        transformFeedback?: WebGLTransformFeedback | null;
        vertexCompilationError: Nullable<string>;
        fragmentCompilationError: Nullable<string>;
        programLinkError: Nullable<string>;
        programValidationError: Nullable<string>;
        constructor();
        get isAsync(): boolean;
        get isReady(): boolean;
        _handlesSpectorRebuildCallback(onCompiled: (program: WebGLProgram) => void): void;
        _fillEffectInformation(effect: Effect, uniformBuffersNames: {
            [key: string]: number;
        }, uniformsNames: string[], uniforms: {
            [key: string]: Nullable<WebGLUniformLocation>;
        }, samplerList: string[], samplers: {
            [key: string]: number;
        }, attributesNames: string[], attributes: number[]): void;
        /**
         * Release all associated resources.
         **/
        dispose(): void;
        /**
         * @internal
         */
        _cacheMatrix(uniformName: string, matrix: IMatrixLike): boolean;
        /**
         * @internal
         */
        _cacheFloatN(_uniformName: string, _x: number, _y?: number, _z?: number, _w?: number): boolean;
        /**
         * @internal
         */
        _cacheFloat2(uniformName: string, x: number, y: number): boolean;
        /**
         * @internal
         */
        _cacheFloat3(uniformName: string, x: number, y: number, z: number): boolean;
        /**
         * @internal
         */
        _cacheFloat4(uniformName: string, x: number, y: number, z: number, w: number): boolean;
        /**
         * Sets an integer value on a uniform variable.
         * @param uniformName Name of the variable.
         * @param value Value to be set.
         */
        setInt: (uniformName: string, value: number) => void;
        /**
         * Sets a int2 on a uniform variable.
         * @param uniformName Name of the variable.
         * @param x First int in int2.
         * @param y Second int in int2.
         */
        setInt2: (uniformName: string, x: number, y: number) => void;
        /**
         * Sets a int3 on a uniform variable.
         * @param uniformName Name of the variable.
         * @param x First int in int3.
         * @param y Second int in int3.
         * @param z Third int in int3.
         */
        setInt3: (uniformName: string, x: number, y: number, z: number) => void;
        /**
         * Sets a int4 on a uniform variable.
         * @param uniformName Name of the variable.
         * @param x First int in int4.
         * @param y Second int in int4.
         * @param z Third int in int4.
         * @param w Fourth int in int4.
         */
        setInt4: (uniformName: string, x: number, y: number, z: number, w: number) => void;
        /**
         * Sets an int array on a uniform variable.
         * @param uniformName Name of the variable.
         * @param array array to be set.
         */
        setIntArray: (uniformName: string, array: Int32Array) => void;
        /**
         * Sets an int array 2 on a uniform variable. (Array is specified as single array eg. [1,2,3,4] will result in [[1,2],[3,4]] in the shader)
         * @param uniformName Name of the variable.
         * @param array array to be set.
         */
        setIntArray2: (uniformName: string, array: Int32Array) => void;
        /**
         * Sets an int array 3 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6] will result in [[1,2,3],[4,5,6]] in the shader)
         * @param uniformName Name of the variable.
         * @param array array to be set.
         */
        setIntArray3: (uniformName: string, array: Int32Array) => void;
        /**
         * Sets an int array 4 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6,7,8] will result in [[1,2,3,4],[5,6,7,8]] in the shader)
         * @param uniformName Name of the variable.
         * @param array array to be set.
         */
        setIntArray4: (uniformName: string, array: Int32Array) => void;
        /**
         * Sets an unsigned integer value on a uniform variable.
         * @param uniformName Name of the variable.
         * @param value Value to be set.
         */
        setUInt: (uniformName: string, value: number) => void;
        /**
         * Sets a unsigned int2 on a uniform variable.
         * @param uniformName Name of the variable.
         * @param x First unsigned int in uint2.
         * @param y Second unsigned int in uint2.
         */
        setUInt2: (uniformName: string, x: number, y: number) => void;
        /**
         * Sets a unsigned int3 on a uniform variable.
         * @param uniformName Name of the variable.
         * @param x First unsigned int in uint3.
         * @param y Second unsigned int in uint3.
         * @param z Third unsigned int in uint3.
         */
        setUInt3: (uniformName: string, x: number, y: number, z: number) => void;
        /**
         * Sets a unsigned int4 on a uniform variable.
         * @param uniformName Name of the variable.
         * @param x First unsigned int in uint4.
         * @param y Second unsigned int in uint4.
         * @param z Third unsigned int in uint4.
         * @param w Fourth unsigned int in uint4.
         */
        setUInt4: (uniformName: string, x: number, y: number, z: number, w: number) => void;
        /**
         * Sets an unsigned int array on a uniform variable.
         * @param uniformName Name of the variable.
         * @param array array to be set.
         */
        setUIntArray: (uniformName: string, array: Uint32Array) => void;
        /**
         * Sets an unsigned int array 2 on a uniform variable. (Array is specified as single array eg. [1,2,3,4] will result in [[1,2],[3,4]] in the shader)
         * @param uniformName Name of the variable.
         * @param array array to be set.
         */
        setUIntArray2: (uniformName: string, array: Uint32Array) => void;
        /**
         * Sets an unsigned int array 3 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6] will result in [[1,2,3],[4,5,6]] in the shader)
         * @param uniformName Name of the variable.
         * @param array array to be set.
         */
        setUIntArray3: (uniformName: string, array: Uint32Array) => void;
        /**
         * Sets an unsigned int array 4 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6,7,8] will result in [[1,2,3,4],[5,6,7,8]] in the shader)
         * @param uniformName Name of the variable.
         * @param array array to be set.
         */
        setUIntArray4: (uniformName: string, array: Uint32Array) => void;
        /**
         * Sets an array on a uniform variable.
         * @param uniformName Name of the variable.
         * @param array array to be set.
         */
        setArray: (uniformName: string, array: number[]) => void;
        /**
         * Sets an array 2 on a uniform variable. (Array is specified as single array eg. [1,2,3,4] will result in [[1,2],[3,4]] in the shader)
         * @param uniformName Name of the variable.
         * @param array array to be set.
         */
        setArray2: (uniformName: string, array: number[]) => void;
        /**
         * Sets an array 3 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6] will result in [[1,2,3],[4,5,6]] in the shader)
         * @param uniformName Name of the variable.
         * @param array array to be set.
         * @returns this effect.
         */
        setArray3: (uniformName: string, array: number[]) => void;
        /**
         * Sets an array 4 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6,7,8] will result in [[1,2,3,4],[5,6,7,8]] in the shader)
         * @param uniformName Name of the variable.
         * @param array array to be set.
         */
        setArray4: (uniformName: string, array: number[]) => void;
        /**
         * Sets matrices on a uniform variable.
         * @param uniformName Name of the variable.
         * @param matrices matrices to be set.
         */
        setMatrices: (uniformName: string, matrices: Float32Array) => void;
        /**
         * Sets matrix on a uniform variable.
         * @param uniformName Name of the variable.
         * @param matrix matrix to be set.
         */
        setMatrix(uniformName: string, matrix: IMatrixLike): void;
        /**
         * Sets a 3x3 matrix on a uniform variable. (Specified as [1,2,3,4,5,6,7,8,9] will result in [1,2,3][4,5,6][7,8,9] matrix)
         * @param uniformName Name of the variable.
         * @param matrix matrix to be set.
         */
        setMatrix3x3: (uniformName: string, matrix: Float32Array) => void;
        /**
         * Sets a 2x2 matrix on a uniform variable. (Specified as [1,2,3,4] will result in [1,2][3,4] matrix)
         * @param uniformName Name of the variable.
         * @param matrix matrix to be set.
         */
        setMatrix2x2: (uniformName: string, matrix: Float32Array) => void;
        /**
         * Sets a float on a uniform variable.
         * @param uniformName Name of the variable.
         * @param value value to be set.
         * @returns this effect.
         */
        setFloat: (uniformName: string, value: number) => void;
        /**
         * Sets a Vector2 on a uniform variable.
         * @param uniformName Name of the variable.
         * @param vector2 vector2 to be set.
         */
        setVector2(uniformName: string, vector2: IVector2Like): void;
        /**
         * Sets a float2 on a uniform variable.
         * @param uniformName Name of the variable.
         * @param x First float in float2.
         * @param y Second float in float2.
         */
        setFloat2: (uniformName: string, x: number, y: number) => void;
        /**
         * Sets a Vector3 on a uniform variable.
         * @param uniformName Name of the variable.
         * @param vector3 Value to be set.
         */
        setVector3(uniformName: string, vector3: IVector3Like): void;
        /**
         * Sets a float3 on a uniform variable.
         * @param uniformName Name of the variable.
         * @param x First float in float3.
         * @param y Second float in float3.
         * @param z Third float in float3.
         */
        setFloat3: (uniformName: string, x: number, y: number, z: number) => void;
        /**
         * Sets a Vector4 on a uniform variable.
         * @param uniformName Name of the variable.
         * @param vector4 Value to be set.
         */
        setVector4(uniformName: string, vector4: IVector4Like): void;
        /**
         * Sets a Quaternion on a uniform variable.
         * @param uniformName Name of the variable.
         * @param quaternion Value to be set.
         */
        setQuaternion(uniformName: string, quaternion: IQuaternionLike): void;
        /**
         * Sets a float4 on a uniform variable.
         * @param uniformName Name of the variable.
         * @param x First float in float4.
         * @param y Second float in float4.
         * @param z Third float in float4.
         * @param w Fourth float in float4.
         * @returns this effect.
         */
        setFloat4: (uniformName: string, x: number, y: number, z: number, w: number) => void;
        /**
         * Sets a Color3 on a uniform variable.
         * @param uniformName Name of the variable.
         * @param color3 Value to be set.
         */
        setColor3(uniformName: string, color3: IColor3Like): void;
        /**
         * Sets a Color4 on a uniform variable.
         * @param uniformName Name of the variable.
         * @param color3 Value to be set.
         * @param alpha Alpha value to be set.
         */
        setColor4(uniformName: string, color3: IColor3Like, alpha: number): void;
        /**
         * Sets a Color4 on a uniform variable
         * @param uniformName defines the name of the variable
         * @param color4 defines the value to be set
         */
        setDirectColor4(uniformName: string, color4: IColor4Like): void;
        _getVertexShaderCode(): string | null;
        _getFragmentShaderCode(): string | null;
    }


    /** @internal */
    export class WebGLRenderTargetWrapper extends RenderTargetWrapper {
        private _context;
        _framebuffer: Nullable<WebGLFramebuffer>;
        _depthStencilBuffer: Nullable<WebGLRenderbuffer>;
        _MSAAFramebuffer: Nullable<WebGLFramebuffer>;
        _colorTextureArray: Nullable<WebGLTexture>;
        _depthStencilTextureArray: Nullable<WebGLTexture>;
        constructor(isMulti: boolean, isCube: boolean, size: TextureSize, engine: ThinEngine, context: WebGLRenderingContext);
        protected _cloneRenderTargetWrapper(): Nullable<RenderTargetWrapper>;
        protected _swapRenderTargetWrapper(target: WebGLRenderTargetWrapper): void;
        /**
         * Shares the depth buffer of this render target with another render target.
         * @internal
         * @param renderTarget Destination renderTarget
         */
        _shareDepth(renderTarget: WebGLRenderTargetWrapper): void;
        /**
         * Binds a texture to this render target on a specific attachment
         * @param texture The texture to bind to the framebuffer
         * @param attachmentIndex Index of the attachment
         * @param faceIndex The face of the texture to render to in case of cube texture
         * @param lodLevel defines the lod level to bind to the frame buffer
         */
        private _bindTextureRenderTarget;
        /**
         * Set a texture in the textures array
         * @param texture the texture to set
         * @param index the index in the textures array to set
         * @param disposePrevious If this function should dispose the previous texture
         */
        setTexture(texture: InternalTexture, index?: number, disposePrevious?: boolean): void;
        dispose(disposeOnlyFramebuffers?: boolean): void;
    }


    /** @internal */
    export class WebGLShaderProcessor implements IShaderProcessor {
        shaderLanguage: ShaderLanguage;
        postProcessor(code: string, defines: string[], isFragment: boolean, processingContext: Nullable<ShaderProcessingContext>, engine: ThinEngine): string;
    }




        interface WebGPUEngine {
            /** @internal */
            _createComputePipelineStageDescriptor(computeShader: string, defines: Nullable<string>, entryPoint: string): GPUProgrammableStage;
        }










        interface Effect {
            /**
             * Sets an external texture on the engine to be used in the shader.
             * @param name Name of the external texture variable.
             * @param texture Texture to set.
             */
            setExternalTexture(name: string, texture: Nullable<ExternalTexture>): void;
        }














        interface Effect {
            /**
             * Sets a storage buffer on the engine to be used in the shader.
             * @param name Name of the storage buffer variable.
             * @param buffer Storage buffer to set.
             */
            setStorageBuffer(name: string, buffer: Nullable<StorageBuffer>): void;
        }


        interface Effect {
            /**
             * Sets a sampler on the engine to be used in the shader.
             * @param name Name of the sampler variable.
             * @param sampler Sampler to set.
             */
            setTextureSampler(name: string, sampler: Nullable<TextureSampler>): void;
        }








    /** @internal */
    export class WebGPUBufferManager {
        private _device;
        private _deferredReleaseBuffers;
        private static _IsGPUBuffer;
        constructor(device: GPUDevice);
        createRawBuffer(viewOrSize: ArrayBufferView | number, flags: GPUBufferUsageFlags, mappedAtCreation?: boolean): GPUBuffer;
        createBuffer(viewOrSize: ArrayBufferView | number, flags: GPUBufferUsageFlags): WebGPUDataBuffer;
        setRawData(buffer: GPUBuffer, dstByteOffset: number, src: ArrayBufferView, srcByteOffset: number, byteLength: number): void;
        setSubData(dataBuffer: WebGPUDataBuffer, dstByteOffset: number, src: ArrayBufferView, srcByteOffset?: number, byteLength?: number): void;
        private _getHalfFloatAsFloatRGBAArrayBuffer;
        readDataFromBuffer(gpuBuffer: GPUBuffer, size: number, width: number, height: number, bytesPerRow: number, bytesPerRowAligned: number, type?: number, offset?: number, buffer?: Nullable<ArrayBufferView>, destroyBuffer?: boolean, noDataConversion?: boolean): Promise<ArrayBufferView>;
        releaseBuffer(buffer: DataBuffer | GPUBuffer): boolean;
        destroyDeferredBuffers(): void;
    }


    /** @internal */
    interface IWebGPURenderItem {
        run(renderPass: GPURenderPassEncoder): void;
        clone(): IWebGPURenderItem;
    }
    /** @internal */
    export class WebGPURenderItemViewport implements IWebGPURenderItem {
        x: number;
        y: number;
        w: number;
        h: number;
        constructor(x: number, y: number, w: number, h: number);
        run(renderPass: GPURenderPassEncoder): void;
        clone(): WebGPURenderItemViewport;
    }
    /** @internal */
    export class WebGPURenderItemScissor implements IWebGPURenderItem {
        x: number;
        y: number;
        w: number;
        h: number;
        constructor(x: number, y: number, w: number, h: number);
        run(renderPass: GPURenderPassEncoder): void;
        clone(): WebGPURenderItemScissor;
    }
    /** @internal */
    export class WebGPURenderItemStencilRef implements IWebGPURenderItem {
        ref: number;
        constructor(ref: number);
        run(renderPass: GPURenderPassEncoder): void;
        clone(): WebGPURenderItemStencilRef;
    }
    /** @internal */
    export class WebGPURenderItemBlendColor implements IWebGPURenderItem {
        color: Nullable<number>[];
        constructor(color: Nullable<number>[]);
        run(renderPass: GPURenderPassEncoder): void;
        clone(): WebGPURenderItemBlendColor;
    }
    /** @internal */
    export class WebGPURenderItemBeginOcclusionQuery implements IWebGPURenderItem {
        query: number;
        constructor(query: number);
        run(renderPass: GPURenderPassEncoder): void;
        clone(): WebGPURenderItemBeginOcclusionQuery;
    }
    /** @internal */
    export class WebGPURenderItemEndOcclusionQuery implements IWebGPURenderItem {
        constructor();
        run(renderPass: GPURenderPassEncoder): void;
        clone(): WebGPURenderItemEndOcclusionQuery;
    }
    /** @internal */
    export class WebGPUBundleList {
        private _device;
        private _bundleEncoder;
        private _list;
        private _listLength;
        private _currentItemIsBundle;
        private _currentBundleList;
        numDrawCalls: number;
        constructor(device: GPUDevice);
        addBundle(bundle?: GPURenderBundle): void;
        private _finishBundle;
        addItem(item: IWebGPURenderItem): void;
        getBundleEncoder(colorFormats: (GPUTextureFormat | null)[], depthStencilFormat: GPUTextureFormat | undefined, sampleCount: number): GPURenderBundleEncoder;
        close(): void;
        run(renderPass: GPURenderPassEncoder): void;
        reset(): void;
        clone(): WebGPUBundleList;
    }


    /** @internal */
    export class WebGPUCacheBindGroups {
        static NumBindGroupsCreatedTotal: number;
        static NumBindGroupsCreatedLastFrame: number;
        static NumBindGroupsLookupLastFrame: number;
        static NumBindGroupsNoLookupLastFrame: number;
        private static _Cache;
        private static _NumBindGroupsCreatedCurrentFrame;
        private static _NumBindGroupsLookupCurrentFrame;
        private static _NumBindGroupsNoLookupCurrentFrame;
        private _device;
        private _cacheSampler;
        private _engine;
        disabled: boolean;
        static get Statistics(): {
            totalCreated: number;
            lastFrameCreated: number;
            lookupLastFrame: number;
            noLookupLastFrame: number;
        };
        constructor(device: GPUDevice, cacheSampler: WebGPUCacheSampler, engine: WebGPUEngine);
        endFrame(): void;
        /**
         * Cache is currently based on the uniform/storage buffers, samplers and textures used by the binding groups.
         * Note that all uniform buffers have an offset of 0 in Babylon and we don't have a use case where we would have the same buffer used with different capacity values:
         * that means we don't need to factor in the offset/size of the buffer in the cache, only the id
         * @param webgpuPipelineContext
         * @param drawContext
         * @param materialContext
         */
        getBindGroups(webgpuPipelineContext: WebGPUPipelineContext, drawContext: WebGPUDrawContext, materialContext: WebGPUMaterialContext): GPUBindGroup[];
    }


    /** @internal */
    export abstract class WebGPUCacheRenderPipeline {
        static NumCacheHitWithoutHash: number;
        static NumCacheHitWithHash: number;
        static NumCacheMiss: number;
        static NumPipelineCreationLastFrame: number;
        disabled: boolean;
        private static _NumPipelineCreationCurrentFrame;
        protected _states: number[];
        protected _statesLength: number;
        protected _stateDirtyLowestIndex: number;
        lastStateDirtyLowestIndex: number;
        private _device;
        private _isDirty;
        private _emptyVertexBuffer;
        private _parameter;
        private _kMaxVertexBufferStride;
        private _shaderId;
        private _alphaToCoverageEnabled;
        private _frontFace;
        private _cullEnabled;
        private _cullFace;
        private _clampDepth;
        private _rasterizationState;
        private _depthBias;
        private _depthBiasClamp;
        private _depthBiasSlopeScale;
        private _colorFormat;
        private _webgpuColorFormat;
        private _mrtAttachments1;
        private _mrtAttachments2;
        private _mrtFormats;
        private _mrtEnabledMask;
        private _alphaBlendEnabled;
        private _alphaBlendFuncParams;
        private _alphaBlendEqParams;
        private _writeMask;
        private _colorStates;
        private _depthStencilFormat;
        private _webgpuDepthStencilFormat;
        private _depthTestEnabled;
        private _depthWriteEnabled;
        private _depthCompare;
        private _stencilEnabled;
        private _stencilFrontCompare;
        private _stencilFrontDepthFailOp;
        private _stencilFrontPassOp;
        private _stencilFrontFailOp;
        private _stencilReadMask;
        private _stencilWriteMask;
        private _depthStencilState;
        private _vertexBuffers;
        private _overrideVertexBuffers;
        private _indexBuffer;
        private _textureState;
        private _useTextureStage;
        constructor(device: GPUDevice, emptyVertexBuffer: VertexBuffer, useTextureStage: boolean);
        reset(): void;
        protected abstract _getRenderPipeline(param: {
            token: any;
            pipeline: Nullable<GPURenderPipeline>;
        }): void;
        protected abstract _setRenderPipeline(param: {
            token: any;
            pipeline: Nullable<GPURenderPipeline>;
        }): void;
        readonly vertexBuffers: VertexBuffer[];
        get colorFormats(): (GPUTextureFormat | null)[];
        readonly mrtAttachments: number[];
        readonly mrtTextureArray: InternalTexture[];
        readonly mrtTextureCount: number;
        getRenderPipeline(fillMode: number, effect: Effect, sampleCount: number, textureState?: number): GPURenderPipeline;
        endFrame(): void;
        setAlphaToCoverage(enabled: boolean): void;
        setFrontFace(frontFace: number): void;
        setCullEnabled(enabled: boolean): void;
        setCullFace(cullFace: number): void;
        setClampDepth(clampDepth: boolean): void;
        resetDepthCullingState(): void;
        setDepthCullingState(cullEnabled: boolean, frontFace: number, cullFace: number, zOffset: number, zOffsetUnits: number, depthTestEnabled: boolean, depthWriteEnabled: boolean, depthCompare: Nullable<number>): void;
        setDepthBias(depthBias: number): void;
        setDepthBiasSlopeScale(depthBiasSlopeScale: number): void;
        setColorFormat(format: GPUTextureFormat | null): void;
        setMRTAttachments(attachments: number[]): void;
        setMRT(textureArray: InternalTexture[], textureCount?: number): void;
        setAlphaBlendEnabled(enabled: boolean): void;
        setAlphaBlendFactors(factors: Array<Nullable<number>>, operations: Array<Nullable<number>>): void;
        setWriteMask(mask: number): void;
        setDepthStencilFormat(format: GPUTextureFormat | undefined): void;
        setDepthTestEnabled(enabled: boolean): void;
        setDepthWriteEnabled(enabled: boolean): void;
        setDepthCompare(func: Nullable<number>): void;
        setStencilEnabled(enabled: boolean): void;
        setStencilCompare(func: Nullable<number>): void;
        setStencilDepthFailOp(op: Nullable<number>): void;
        setStencilPassOp(op: Nullable<number>): void;
        setStencilFailOp(op: Nullable<number>): void;
        setStencilReadMask(mask: number): void;
        setStencilWriteMask(mask: number): void;
        resetStencilState(): void;
        setStencilState(stencilEnabled: boolean, compare: Nullable<number>, depthFailOp: Nullable<number>, passOp: Nullable<number>, failOp: Nullable<number>, readMask: number, writeMask: number): void;
        setBuffers(vertexBuffers: Nullable<{
            [key: string]: Nullable<VertexBuffer>;
        }>, indexBuffer: Nullable<DataBuffer>, overrideVertexBuffers: Nullable<{
            [key: string]: Nullable<VertexBuffer>;
        }>): void;
        private static _GetTopology;
        private static _GetAphaBlendOperation;
        private static _GetAphaBlendFactor;
        private static _GetCompareFunction;
        private static _GetStencilOpFunction;
        private static _GetVertexInputDescriptorFormat;
        private _getAphaBlendState;
        private _getColorBlendState;
        private _setShaderStage;
        private _setRasterizationState;
        private _setColorStates;
        private _setDepthStencilState;
        private _setVertexState;
        private _setTextureState;
        private _createPipelineLayout;
        private _createPipelineLayoutWithTextureStage;
        private _getVertexInputDescriptor;
        private _createRenderPipeline;
    }


    /**
     * Class not used, WebGPUCacheRenderPipelineTree is faster
     * @internal
     */
    export class WebGPUCacheRenderPipelineString extends WebGPUCacheRenderPipeline {
        private static _Cache;
        protected _getRenderPipeline(param: {
            token: any;
            pipeline: Nullable<GPURenderPipeline>;
        }): void;
        protected _setRenderPipeline(param: {
            token: any;
            pipeline: Nullable<GPURenderPipeline>;
        }): void;
    }


    /** @internal */
    class NodeState {
        values: {
            [id: number]: NodeState;
        };
        pipeline: GPURenderPipeline;
        constructor();
        count(): [number, number];
    }
    /** @internal */
    export class WebGPUCacheRenderPipelineTree extends WebGPUCacheRenderPipeline {
        private static _Cache;
        private _nodeStack;
        static GetNodeCounts(): {
            nodeCount: number;
            pipelineCount: number;
        };
        static _GetPipelines(node: NodeState, pipelines: Array<Array<number>>, curPath: Array<number>, curPathLen: number): void;
        static GetPipelines(): Array<Array<number>>;
        constructor(device: GPUDevice, emptyVertexBuffer: VertexBuffer, useTextureStage: boolean);
        protected _getRenderPipeline(param: {
            token: any;
            pipeline: Nullable<GPURenderPipeline>;
        }): void;
        protected _setRenderPipeline(param: {
            token: NodeState;
            pipeline: Nullable<GPURenderPipeline>;
        }): void;
    }


    /** @internal */
    export class WebGPUCacheSampler {
        private _samplers;
        private _device;
        disabled: boolean;
        constructor(device: GPUDevice);
        static GetSamplerHashCode(sampler: TextureSampler): number;
        private static _GetSamplerFilterDescriptor;
        private static _GetWrappingMode;
        private static _GetSamplerWrappingDescriptor;
        private static _GetSamplerDescriptor;
        static GetCompareFunction(compareFunction: Nullable<number>): GPUCompareFunction;
        getSampler(sampler: TextureSampler, bypassCache?: boolean, hash?: number): GPUSampler;
    }


    /** @internal */
    export class WebGPUClearQuad {
        private _device;
        private _engine;
        private _cacheRenderPipeline;
        private _effect;
        private _bindGroups;
        private _depthTextureFormat;
        private _bundleCache;
        private _keyTemp;
        setDepthStencilFormat(format: GPUTextureFormat | undefined): void;
        setColorFormat(format: GPUTextureFormat | null): void;
        setMRTAttachments(attachments: number[], textureArray: InternalTexture[], textureCount: number): void;
        constructor(device: GPUDevice, engine: WebGPUEngine, emptyVertexBuffer: VertexBuffer);
        clear(renderPass: Nullable<GPURenderPassEncoder>, clearColor?: Nullable<IColor4Like>, clearDepth?: boolean, clearStencil?: boolean, sampleCount?: number): Nullable<GPURenderBundle>;
    }


    /** @internal */
    export class WebGPUComputeContext implements IComputeContext {
        private static _Counter;
        readonly uniqueId: number;
        private _device;
        private _cacheSampler;
        private _bindGroups;
        private _bindGroupEntries;
        getBindGroups(bindings: ComputeBindingList, computePipeline: GPUComputePipeline, bindingsMapping?: ComputeBindingMapping): GPUBindGroup[];
        constructor(device: GPUDevice, cacheSampler: WebGPUCacheSampler);
        clear(): void;
    }


    /** @internal */
    export class WebGPUComputePipelineContext implements IComputePipelineContext {
        engine: WebGPUEngine;
        sources: {
            compute: string;
            rawCompute: string;
        };
        stage: Nullable<GPUProgrammableStage>;
        computePipeline: GPUComputePipeline;
        get isAsync(): boolean;
        get isReady(): boolean;
        /** @internal */
        _name: string;
        constructor(engine: WebGPUEngine);
        _getComputeShaderCode(): string | null;
        dispose(): void;
    }


    /** @internal */
    export enum PredefinedColorSpace {
        SRGB = "srgb"
    }
    /** @internal */
    export enum PowerPreference {
        LowPower = "low-power",
        HighPerformance = "high-performance"
    }
    /** @internal */
    export enum FeatureName {
        DepthClipControl = "depth-clip-control",
        Depth24UnormStencil8 = "depth24unorm-stencil8",
        Depth32FloatStencil8 = "depth32float-stencil8",
        TextureCompressionBC = "texture-compression-bc",
        TextureCompressionETC2 = "texture-compression-etc2",
        TextureCompressionASTC = "texture-compression-astc",
        TimestampQuery = "timestamp-query",
        IndirectFirstInstance = "indirect-first-instance",
        ShaderF16 = "shader-f16",
        BGRA8UnormStorage = "bgra8unorm-storage"
    }
    /** @internal */
    export enum BufferUsage {
        MapRead = 1,
        MapWrite = 2,
        CopySrc = 4,
        CopyDst = 8,
        Index = 16,
        Vertex = 32,
        Uniform = 64,
        Storage = 128,
        Indirect = 256,
        QueryResolve = 512
    }
    /** @internal */
    export enum MapMode {
        Read = 1,
        Write = 2
    }
    /** @internal */
    export enum TextureDimension {
        E1d = "1d",
        E2d = "2d",
        E3d = "3d"
    }
    /** @internal */
    export enum TextureUsage {
        CopySrc = 1,
        CopyDst = 2,
        TextureBinding = 4,
        StorageBinding = 8,
        RenderAttachment = 16
    }
    /** @internal */
    export enum TextureViewDimension {
        E1d = "1d",
        E2d = "2d",
        E2dArray = "2d-array",
        Cube = "cube",
        CubeArray = "cube-array",
        E3d = "3d"
    }
    /** @internal */
    export enum TextureAspect {
        All = "all",
        StencilOnly = "stencil-only",
        DepthOnly = "depth-only"
    }
    /**
     * Comments taken from https://github.com/gfx-rs/wgpu/blob/master/wgpu-types/src/lib.rs
     * @internal
     */
    export enum TextureFormat {
        R8Unorm = "r8unorm",
        R8Snorm = "r8snorm",
        R8Uint = "r8uint",
        R8Sint = "r8sint",
        R16Uint = "r16uint",
        R16Sint = "r16sint",
        R16Float = "r16float",
        RG8Unorm = "rg8unorm",
        RG8Snorm = "rg8snorm",
        RG8Uint = "rg8uint",
        RG8Sint = "rg8sint",
        R32Uint = "r32uint",
        R32Sint = "r32sint",
        R32Float = "r32float",
        RG16Uint = "rg16uint",
        RG16Sint = "rg16sint",
        RG16Float = "rg16float",
        RGBA8Unorm = "rgba8unorm",
        RGBA8UnormSRGB = "rgba8unorm-srgb",
        RGBA8Snorm = "rgba8snorm",
        RGBA8Uint = "rgba8uint",
        RGBA8Sint = "rgba8sint",
        BGRA8Unorm = "bgra8unorm",
        BGRA8UnormSRGB = "bgra8unorm-srgb",
        RGB9E5UFloat = "rgb9e5ufloat",
        RGB10A2Unorm = "rgb10a2unorm",
        RG11B10UFloat = "rg11b10ufloat",
        RG32Uint = "rg32uint",
        RG32Sint = "rg32sint",
        RG32Float = "rg32float",
        RGBA16Uint = "rgba16uint",
        RGBA16Sint = "rgba16sint",
        RGBA16Float = "rgba16float",
        RGBA32Uint = "rgba32uint",
        RGBA32Sint = "rgba32sint",
        RGBA32Float = "rgba32float",
        Stencil8 = "stencil8",
        Depth16Unorm = "depth16unorm",
        Depth24Plus = "depth24plus",
        Depth24PlusStencil8 = "depth24plus-stencil8",
        Depth32Float = "depth32float",
        BC1RGBAUnorm = "bc1-rgba-unorm",
        BC1RGBAUnormSRGB = "bc1-rgba-unorm-srgb",
        BC2RGBAUnorm = "bc2-rgba-unorm",
        BC2RGBAUnormSRGB = "bc2-rgba-unorm-srgb",
        BC3RGBAUnorm = "bc3-rgba-unorm",
        BC3RGBAUnormSRGB = "bc3-rgba-unorm-srgb",
        BC4RUnorm = "bc4-r-unorm",
        BC4RSnorm = "bc4-r-snorm",
        BC5RGUnorm = "bc5-rg-unorm",
        BC5RGSnorm = "bc5-rg-snorm",
        BC6HRGBUFloat = "bc6h-rgb-ufloat",
        BC6HRGBFloat = "bc6h-rgb-float",
        BC7RGBAUnorm = "bc7-rgba-unorm",
        BC7RGBAUnormSRGB = "bc7-rgba-unorm-srgb",
        ETC2RGB8Unorm = "etc2-rgb8unorm",
        ETC2RGB8UnormSRGB = "etc2-rgb8unorm-srgb",
        ETC2RGB8A1Unorm = "etc2-rgb8a1unorm",
        ETC2RGB8A1UnormSRGB = "etc2-rgb8a1unorm-srgb",
        ETC2RGBA8Unorm = "etc2-rgba8unorm",
        ETC2RGBA8UnormSRGB = "etc2-rgba8unorm-srgb",
        EACR11Unorm = "eac-r11unorm",
        EACR11Snorm = "eac-r11snorm",
        EACRG11Unorm = "eac-rg11unorm",
        EACRG11Snorm = "eac-rg11snorm",
        ASTC4x4Unorm = "astc-4x4-unorm",
        ASTC4x4UnormSRGB = "astc-4x4-unorm-srgb",
        ASTC5x4Unorm = "astc-5x4-unorm",
        ASTC5x4UnormSRGB = "astc-5x4-unorm-srgb",
        ASTC5x5Unorm = "astc-5x5-unorm",
        ASTC5x5UnormSRGB = "astc-5x5-unorm-srgb",
        ASTC6x5Unorm = "astc-6x5-unorm",
        ASTC6x5UnormSRGB = "astc-6x5-unorm-srgb",
        ASTC6x6Unorm = "astc-6x6-unorm",
        ASTC6x6UnormSRGB = "astc-6x6-unorm-srgb",
        ASTC8x5Unorm = "astc-8x5-unorm",
        ASTC8x5UnormSRGB = "astc-8x5-unorm-srgb",
        ASTC8x6Unorm = "astc-8x6-unorm",
        ASTC8x6UnormSRGB = "astc-8x6-unorm-srgb",
        ASTC8x8Unorm = "astc-8x8-unorm",
        ASTC8x8UnormSRGB = "astc-8x8-unorm-srgb",
        ASTC10x5Unorm = "astc-10x5-unorm",
        ASTC10x5UnormSRGB = "astc-10x5-unorm-srgb",
        ASTC10x6Unorm = "astc-10x6-unorm",
        ASTC10x6UnormSRGB = "astc-10x6-unorm-srgb",
        ASTC10x8Unorm = "astc-10x8-unorm",
        ASTC10x8UnormSRGB = "astc-10x8-unorm-srgb",
        ASTC10x10Unorm = "astc-10x10-unorm",
        ASTC10x10UnormSRGB = "astc-10x10-unorm-srgb",
        ASTC12x10Unorm = "astc-12x10-unorm",
        ASTC12x10UnormSRGB = "astc-12x10-unorm-srgb",
        ASTC12x12Unorm = "astc-12x12-unorm",
        ASTC12x12UnormSRGB = "astc-12x12-unorm-srgb",
        Depth24UnormStencil8 = "depth24unorm-stencil8",
        Depth32FloatStencil8 = "depth32float-stencil8"
    }
    /** @internal */
    export enum AddressMode {
        ClampToEdge = "clamp-to-edge",
        Repeat = "repeat",
        MirrorRepeat = "mirror-repeat"
    }
    /** @internal */
    export enum FilterMode {
        Nearest = "nearest",
        Linear = "linear"
    }
    /** @internal */
    export enum CompareFunction {
        Never = "never",
        Less = "less",
        Equal = "equal",
        LessEqual = "less-equal",
        Greater = "greater",
        NotEqual = "not-equal",
        GreaterEqual = "greater-equal",
        Always = "always"
    }
    /** @internal */
    export enum ShaderStage {
        Vertex = 1,
        Fragment = 2,
        Compute = 4
    }
    /** @internal */
    export enum BufferBindingType {
        Uniform = "uniform",
        Storage = "storage",
        ReadOnlyStorage = "read-only-storage"
    }
    /** @internal */
    export enum SamplerBindingType {
        Filtering = "filtering",
        NonFiltering = "non-filtering",
        Comparison = "comparison"
    }
    /** @internal */
    export enum TextureSampleType {
        Float = "float",
        UnfilterableFloat = "unfilterable-float",
        Depth = "depth",
        Sint = "sint",
        Uint = "uint"
    }
    /** @internal */
    export enum StorageTextureAccess {
        WriteOnly = "write-only"
    }
    /** @internal */
    export enum CompilationMessageType {
        Error = "error",
        Warning = "warning",
        Info = "info"
    }
    /** @internal */
    export enum AutoLayoutMode {
        Auto = "auto"
    }
    /** @internal */
    export enum PrimitiveTopology {
        PointList = "point-list",
        LineList = "line-list",
        LineStrip = "line-strip",
        TriangleList = "triangle-list",
        TriangleStrip = "triangle-strip"
    }
    /** @internal */
    export enum FrontFace {
        CCW = "ccw",
        CW = "cw"
    }
    /** @internal */
    export enum CullMode {
        None = "none",
        Front = "front",
        Back = "back"
    }
    /** @internal */
    export enum ColorWriteFlags {
        Red = 1,
        Green = 2,
        Blue = 4,
        Alpha = 8,
        All = 15
    }
    /** @internal */
    export enum BlendFactor {
        Zero = "zero",
        One = "one",
        Src = "src",
        OneMinusSrc = "one-minus-src",
        SrcAlpha = "src-alpha",
        OneMinusSrcAlpha = "one-minus-src-alpha",
        Dst = "dst",
        OneMinusDst = "one-minus-dst",
        DstAlpha = "dst-alpha",
        OneMinusDstAlpha = "one-minus-dst-alpha",
        SrcAlphaSaturated = "src-alpha-saturated",
        Constant = "constant",
        OneMinusConstant = "one-minus-constant"
    }
    /** @internal */
    export enum BlendOperation {
        Add = "add",
        Subtract = "subtract",
        ReverseSubtract = "reverse-subtract",
        Min = "min",
        Max = "max"
    }
    /** @internal */
    export enum StencilOperation {
        Keep = "keep",
        Zero = "zero",
        Replace = "replace",
        Invert = "invert",
        IncrementClamp = "increment-clamp",
        DecrementClamp = "decrement-clamp",
        IncrementWrap = "increment-wrap",
        DecrementWrap = "decrement-wrap"
    }
    /** @internal */
    export enum IndexFormat {
        Uint16 = "uint16",
        Uint32 = "uint32"
    }
    /** @internal */
    export enum VertexFormat {
        Uint8x2 = "uint8x2",
        Uint8x4 = "uint8x4",
        Sint8x2 = "sint8x2",
        Sint8x4 = "sint8x4",
        Unorm8x2 = "unorm8x2",
        Unorm8x4 = "unorm8x4",
        Snorm8x2 = "snorm8x2",
        Snorm8x4 = "snorm8x4",
        Uint16x2 = "uint16x2",
        Uint16x4 = "uint16x4",
        Sint16x2 = "sint16x2",
        Sint16x4 = "sint16x4",
        Unorm16x2 = "unorm16x2",
        Unorm16x4 = "unorm16x4",
        Snorm16x2 = "snorm16x2",
        Snorm16x4 = "snorm16x4",
        Float16x2 = "float16x2",
        Float16x4 = "float16x4",
        Float32 = "float32",
        Float32x2 = "float32x2",
        Float32x3 = "float32x3",
        Float32x4 = "float32x4",
        Uint32 = "uint32",
        Uint32x2 = "uint32x2",
        Uint32x3 = "uint32x3",
        Uint32x4 = "uint32x4",
        Sint32 = "sint32",
        Sint32x2 = "sint32x2",
        Sint32x3 = "sint32x3",
        Sint32x4 = "sint32x4"
    }
    /** @internal */
    export enum InputStepMode {
        Vertex = "vertex",
        Instance = "instance"
    }
    /** @internal */
    export enum ComputePassTimestampLocation {
        Beginning = "beginning",
        End = "end"
    }
    /** @internal */
    export enum RenderPassTimestampLocation {
        Beginning = "beginning",
        End = "end"
    }
    /** @internal */
    export enum LoadOp {
        Load = "load",
        Clear = "clear"
    }
    /** @internal */
    export enum StoreOp {
        Store = "store",
        Discard = "discard"
    }
    /** @internal */
    export enum QueryType {
        Occlusion = "occlusion",
        Timestamp = "timestamp"
    }
    /** @internal */
    export enum CanvasAlphaMode {
        Opaque = "opaque",
        Premultiplied = "premultiplied"
    }
    /** @internal */
    export enum DeviceLostReason {
        Destroyed = "destroyed"
    }
    /** @internal */
    export enum ErrorFilter {
        OutOfMemory = "out-of-memory",
        Validation = "validation"
    }


    /**
     * @internal
     **/
    export class WebGPUDepthCullingState extends DepthCullingState {
        private _cache;
        /**
         * Initializes the state.
         * @param cache
         */
        constructor(cache: WebGPUCacheRenderPipeline);
        get zOffset(): number;
        set zOffset(value: number);
        get zOffsetUnits(): number;
        set zOffsetUnits(value: number);
        get cullFace(): Nullable<number>;
        set cullFace(value: Nullable<number>);
        get cull(): Nullable<boolean>;
        set cull(value: Nullable<boolean>);
        get depthFunc(): Nullable<number>;
        set depthFunc(value: Nullable<number>);
        get depthMask(): boolean;
        set depthMask(value: boolean);
        get depthTest(): boolean;
        set depthTest(value: boolean);
        get frontFace(): Nullable<number>;
        set frontFace(value: Nullable<number>);
        reset(): void;
        apply(): void;
    }


    /** @internal */
    export class WebGPUDrawContext implements IDrawContext {
        private static _Counter;
        fastBundle?: GPURenderBundle;
        bindGroups?: GPUBindGroup[];
        uniqueId: number;
        buffers: {
            [name: string]: Nullable<WebGPUDataBuffer>;
        };
        indirectDrawBuffer?: GPUBuffer;
        private _materialContextUpdateId;
        private _bufferManager;
        private _useInstancing;
        private _indirectDrawData?;
        private _currentInstanceCount;
        private _isDirty;
        isDirty(materialContextUpdateId: number): boolean;
        resetIsDirty(materialContextUpdateId: number): void;
        get useInstancing(): boolean;
        set useInstancing(use: boolean);
        constructor(bufferManager: WebGPUBufferManager);
        reset(): void;
        setBuffer(name: string, buffer: Nullable<WebGPUDataBuffer>): void;
        setIndirectData(indexOrVertexCount: number, instanceCount: number, firstIndexOrVertex: number): void;
        dispose(): void;
    }


    /**
     * Nothing specific to WebGPU in this class, but the spec is not final yet so let's remove it later on
     * if it is not needed
     * @internal
     **/
    export class WebGPUExternalTexture extends ExternalTexture {
        constructor(video: HTMLVideoElement);
    }


    /** @internal */
    export class WebGPUHardwareTexture implements HardwareTextureWrapper {
        /**
         * List of bundles collected in the snapshot rendering mode when the texture is a render target texture
         * The index in this array is the current layer we are rendering into
         * @internal
         */
        _bundleLists: WebGPUBundleList[];
        /**
         * Current layer we are rendering into when in snapshot rendering mode (if the texture is a render target texture)
         * @internal
         */
        _currentLayer: number;
        /**
         * Cache of RenderPassDescriptor and BindGroup used when generating mipmaps (see WebGPUTextureHelper.generateMipmaps)
         * @internal
         */
        _mipmapGenRenderPassDescr: GPURenderPassDescriptor[][];
        /** @internal */
        _mipmapGenBindGroup: GPUBindGroup[][];
        /**
         * Cache for the invertYPreMultiplyAlpha function (see WebGPUTextureHelper)
         * @internal
         */
        _copyInvertYTempTexture?: GPUTexture;
        /** @internal */
        _copyInvertYRenderPassDescr: GPURenderPassDescriptor;
        /** @internal */
        _copyInvertYBindGroup: GPUBindGroup;
        /** @internal */
        _copyInvertYBindGroupWithOfst: GPUBindGroup;
        private _webgpuTexture;
        private _webgpuMSAATexture;
        get underlyingResource(): Nullable<GPUTexture>;
        get msaaTexture(): Nullable<GPUTexture>;
        set msaaTexture(texture: Nullable<GPUTexture>);
        view: Nullable<GPUTextureView>;
        viewForWriting: Nullable<GPUTextureView>;
        format: GPUTextureFormat;
        textureUsages: number;
        textureAdditionalUsages: number;
        constructor(existingTexture?: Nullable<GPUTexture>);
        set(hardwareTexture: GPUTexture): void;
        setUsage(textureSource: number, generateMipMaps: boolean, isCube: boolean, width: number, height: number): void;
        createView(descriptor?: GPUTextureViewDescriptor, createViewForWriting?: boolean): void;
        reset(): void;
        release(): void;
    }


    /** @internal */
    interface IWebGPUMaterialContextSamplerCache {
        sampler: Nullable<TextureSampler>;
        hashCode: number;
    }
    /** @internal */
    interface IWebGPUMaterialContextTextureCache {
        texture: Nullable<InternalTexture | ExternalTexture>;
        isFloatTexture: boolean;
        isExternalTexture: boolean;
    }
    /** @internal */
    export class WebGPUMaterialContext implements IMaterialContext {
        private static _Counter;
        uniqueId: number;
        updateId: number;
        isDirty: boolean;
        samplers: {
            [name: string]: Nullable<IWebGPUMaterialContextSamplerCache>;
        };
        textures: {
            [name: string]: Nullable<IWebGPUMaterialContextTextureCache>;
        };
        get forceBindGroupCreation(): boolean;
        get hasFloatTextures(): boolean;
        protected _numFloatTextures: number;
        protected _numExternalTextures: number;
        constructor();
        reset(): void;
        setSampler(name: string, sampler: Nullable<TextureSampler>): void;
        setTexture(name: string, texture: Nullable<InternalTexture | ExternalTexture>): void;
    }


    /** @internal */
    export class WebGPUOcclusionQuery {
        private _engine;
        private _device;
        private _bufferManager;
        private _currentTotalIndices;
        private _countIncrement;
        private _querySet;
        private _availableIndices;
        private _lastBuffer;
        private _frameLastBuffer;
        get querySet(): GPUQuerySet;
        get hasQueries(): boolean;
        get canBeginQuery(): boolean;
        constructor(engine: WebGPUEngine, device: GPUDevice, bufferManager: WebGPUBufferManager, startCount?: number, incrementCount?: number);
        createQuery(): number;
        deleteQuery(index: number): void;
        isQueryResultAvailable(index: number): boolean;
        getQueryResult(index: number): number;
        private _retrieveQueryBuffer;
        private _allocateNewIndices;
        private _delayQuerySetDispose;
        dispose(): void;
    }


    /** @internal */
    export interface IWebGPURenderPipelineStageDescriptor {
        vertexStage: GPUProgrammableStage;
        fragmentStage?: GPUProgrammableStage;
    }
    /** @internal */
    export class WebGPUPipelineContext implements IPipelineContext {
        engine: WebGPUEngine;
        shaderProcessingContext: WebGPUShaderProcessingContext;
        protected _leftOverUniformsByName: {
            [name: string]: string;
        };
        sources: {
            vertex: string;
            fragment: string;
            rawVertex: string;
            rawFragment: string;
        };
        stages: Nullable<IWebGPURenderPipelineStageDescriptor>;
        bindGroupLayouts: GPUBindGroupLayout[];
        /**
         * Stores the left-over uniform buffer
         */
        uniformBuffer: Nullable<UniformBuffer>;
        onCompiled?: () => void;
        get isAsync(): boolean;
        get isReady(): boolean;
        /** @internal */
        _name: string;
        constructor(shaderProcessingContext: WebGPUShaderProcessingContext, engine: WebGPUEngine);
        _handlesSpectorRebuildCallback(): void;
        _fillEffectInformation(effect: Effect, uniformBuffersNames: {
            [key: string]: number;
        }, uniformsNames: string[], uniforms: {
            [key: string]: Nullable<WebGLUniformLocation>;
        }, samplerList: string[], samplers: {
            [key: string]: number;
        }, attributesNames: string[], attributes: number[]): void;
        /** @internal */
        /**
         * Build the uniform buffer used in the material.
         */
        buildUniformLayout(): void;
        /**
         * Release all associated resources.
         **/
        dispose(): void;
        /**
         * Sets an integer value on a uniform variable.
         * @param uniformName Name of the variable.
         * @param value Value to be set.
         */
        setInt(uniformName: string, value: number): void;
        /**
         * Sets an int2 value on a uniform variable.
         * @param uniformName Name of the variable.
         * @param x First int in int2.
         * @param y Second int in int2.
         */
        setInt2(uniformName: string, x: number, y: number): void;
        /**
         * Sets an int3 value on a uniform variable.
         * @param uniformName Name of the variable.
         * @param x First int in int3.
         * @param y Second int in int3.
         * @param z Third int in int3.
         */
        setInt3(uniformName: string, x: number, y: number, z: number): void;
        /**
         * Sets an int4 value on a uniform variable.
         * @param uniformName Name of the variable.
         * @param x First int in int4.
         * @param y Second int in int4.
         * @param z Third int in int4.
         * @param w Fourth int in int4.
         */
        setInt4(uniformName: string, x: number, y: number, z: number, w: number): void;
        /**
         * Sets an int array on a uniform variable.
         * @param uniformName Name of the variable.
         * @param array array to be set.
         */
        setIntArray(uniformName: string, array: Int32Array): void;
        /**
         * Sets an int array 2 on a uniform variable. (Array is specified as single array eg. [1,2,3,4] will result in [[1,2],[3,4]] in the shader)
         * @param uniformName Name of the variable.
         * @param array array to be set.
         */
        setIntArray2(uniformName: string, array: Int32Array): void;
        /**
         * Sets an int array 3 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6] will result in [[1,2,3],[4,5,6]] in the shader)
         * @param uniformName Name of the variable.
         * @param array array to be set.
         */
        setIntArray3(uniformName: string, array: Int32Array): void;
        /**
         * Sets an int array 4 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6,7,8] will result in [[1,2,3,4],[5,6,7,8]] in the shader)
         * @param uniformName Name of the variable.
         * @param array array to be set.
         */
        setIntArray4(uniformName: string, array: Int32Array): void;
        /**
         * Sets an unsigned integer value on a uniform variable.
         * @param uniformName Name of the variable.
         * @param value Value to be set.
         */
        setUInt(uniformName: string, value: number): void;
        /**
         * Sets an unsigned int2 value on a uniform variable.
         * @param uniformName Name of the variable.
         * @param x First unsigned int in uint2.
         * @param y Second unsigned int in uint2.
         */
        setUInt2(uniformName: string, x: number, y: number): void;
        /**
         * Sets an unsigned int3 value on a uniform variable.
         * @param uniformName Name of the variable.
         * @param x First unsigned int in uint3.
         * @param y Second unsigned int in uint3.
         * @param z Third unsigned int in uint3.
         */
        setUInt3(uniformName: string, x: number, y: number, z: number): void;
        /**
         * Sets an unsigned int4 value on a uniform variable.
         * @param uniformName Name of the variable.
         * @param x First unsigned int in uint4.
         * @param y Second unsigned int in uint4.
         * @param z Third unsigned int in uint4.
         * @param w Fourth unsigned int in uint4.
         */
        setUInt4(uniformName: string, x: number, y: number, z: number, w: number): void;
        /**
         * Sets an unsigned int array on a uniform variable.
         * @param uniformName Name of the variable.
         * @param array array to be set.
         */
        setUIntArray(uniformName: string, array: Uint32Array): void;
        /**
         * Sets an unsigned int array 2 on a uniform variable. (Array is specified as single array eg. [1,2,3,4] will result in [[1,2],[3,4]] in the shader)
         * @param uniformName Name of the variable.
         * @param array array to be set.
         */
        setUIntArray2(uniformName: string, array: Uint32Array): void;
        /**
         * Sets an unsigned int array 3 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6] will result in [[1,2,3],[4,5,6]] in the shader)
         * @param uniformName Name of the variable.
         * @param array array to be set.
         */
        setUIntArray3(uniformName: string, array: Uint32Array): void;
        /**
         * Sets an unsigned int array 4 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6,7,8] will result in [[1,2,3,4],[5,6,7,8]] in the shader)
         * @param uniformName Name of the variable.
         * @param array array to be set.
         */
        setUIntArray4(uniformName: string, array: Uint32Array): void;
        /**
         * Sets an array on a uniform variable.
         * @param uniformName Name of the variable.
         * @param array array to be set.
         */
        setArray(uniformName: string, array: number[]): void;
        /**
         * Sets an array 2 on a uniform variable. (Array is specified as single array eg. [1,2,3,4] will result in [[1,2],[3,4]] in the shader)
         * @param uniformName Name of the variable.
         * @param array array to be set.
         */
        setArray2(uniformName: string, array: number[]): void;
        /**
         * Sets an array 3 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6] will result in [[1,2,3],[4,5,6]] in the shader)
         * @param uniformName Name of the variable.
         * @param array array to be set.
         * @returns this effect.
         */
        setArray3(uniformName: string, array: number[]): void;
        /**
         * Sets an array 4 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6,7,8] will result in [[1,2,3,4],[5,6,7,8]] in the shader)
         * @param uniformName Name of the variable.
         * @param array array to be set.
         */
        setArray4(uniformName: string, array: number[]): void;
        /**
         * Sets matrices on a uniform variable.
         * @param uniformName Name of the variable.
         * @param matrices matrices to be set.
         */
        setMatrices(uniformName: string, matrices: Float32Array): void;
        /**
         * Sets matrix on a uniform variable.
         * @param uniformName Name of the variable.
         * @param matrix matrix to be set.
         */
        setMatrix(uniformName: string, matrix: IMatrixLike): void;
        /**
         * Sets a 3x3 matrix on a uniform variable. (Specified as [1,2,3,4,5,6,7,8,9] will result in [1,2,3][4,5,6][7,8,9] matrix)
         * @param uniformName Name of the variable.
         * @param matrix matrix to be set.
         */
        setMatrix3x3(uniformName: string, matrix: Float32Array): void;
        /**
         * Sets a 2x2 matrix on a uniform variable. (Specified as [1,2,3,4] will result in [1,2][3,4] matrix)
         * @param uniformName Name of the variable.
         * @param matrix matrix to be set.
         */
        setMatrix2x2(uniformName: string, matrix: Float32Array): void;
        /**
         * Sets a float on a uniform variable.
         * @param uniformName Name of the variable.
         * @param value value to be set.
         * @returns this effect.
         */
        setFloat(uniformName: string, value: number): void;
        /**
         * Sets a Vector2 on a uniform variable.
         * @param uniformName Name of the variable.
         * @param vector2 vector2 to be set.
         */
        setVector2(uniformName: string, vector2: IVector2Like): void;
        /**
         * Sets a float2 on a uniform variable.
         * @param uniformName Name of the variable.
         * @param x First float in float2.
         * @param y Second float in float2.
         */
        setFloat2(uniformName: string, x: number, y: number): void;
        /**
         * Sets a Vector3 on a uniform variable.
         * @param uniformName Name of the variable.
         * @param vector3 Value to be set.
         */
        setVector3(uniformName: string, vector3: IVector3Like): void;
        /**
         * Sets a float3 on a uniform variable.
         * @param uniformName Name of the variable.
         * @param x First float in float3.
         * @param y Second float in float3.
         * @param z Third float in float3.
         */
        setFloat3(uniformName: string, x: number, y: number, z: number): void;
        /**
         * Sets a Vector4 on a uniform variable.
         * @param uniformName Name of the variable.
         * @param vector4 Value to be set.
         */
        setVector4(uniformName: string, vector4: IVector4Like): void;
        /**
         * Sets a Quaternion on a uniform variable.
         * @param uniformName Name of the variable.
         * @param quaternion Value to be set.
         */
        setQuaternion(uniformName: string, quaternion: IQuaternionLike): void;
        /**
         * Sets a float4 on a uniform variable.
         * @param uniformName Name of the variable.
         * @param x First float in float4.
         * @param y Second float in float4.
         * @param z Third float in float4.
         * @param w Fourth float in float4.
         * @returns this effect.
         */
        setFloat4(uniformName: string, x: number, y: number, z: number, w: number): void;
        /**
         * Sets a Color3 on a uniform variable.
         * @param uniformName Name of the variable.
         * @param color3 Value to be set.
         */
        setColor3(uniformName: string, color3: IColor3Like): void;
        /**
         * Sets a Color4 on a uniform variable.
         * @param uniformName Name of the variable.
         * @param color3 Value to be set.
         * @param alpha Alpha value to be set.
         */
        setColor4(uniformName: string, color3: IColor3Like, alpha: number): void;
        /**
         * Sets a Color4 on a uniform variable
         * @param uniformName defines the name of the variable
         * @param color4 defines the value to be set
         */
        setDirectColor4(uniformName: string, color4: IColor4Like): void;
        _getVertexShaderCode(): string | null;
        _getFragmentShaderCode(): string | null;
    }


    /** @internal */
    export class WebGPUQuerySet {
        private _device;
        private _bufferManager;
        private _count;
        private _canUseMultipleBuffers;
        private _querySet;
        private _queryBuffer;
        private _dstBuffers;
        get querySet(): GPUQuerySet;
        constructor(count: number, type: QueryType, device: GPUDevice, bufferManager: WebGPUBufferManager, canUseMultipleBuffers?: boolean);
        private _getBuffer;
        readValues(firstQuery?: number, queryCount?: number): Promise<BigUint64Array | null>;
        readValue(firstQuery?: number): Promise<number | null>;
        readTwoValuesAndSubtract(firstQuery?: number): Promise<number | null>;
        dispose(): void;
    }


    /** @internal */
    export class WebGPURenderPassWrapper {
        renderPassDescriptor: Nullable<GPURenderPassDescriptor>;
        renderPass: Nullable<GPURenderPassEncoder>;
        colorAttachmentViewDescriptor: Nullable<GPUTextureViewDescriptor>;
        depthAttachmentViewDescriptor: Nullable<GPUTextureViewDescriptor>;
        colorAttachmentGPUTextures: (WebGPUHardwareTexture | null)[];
        depthTextureFormat: GPUTextureFormat | undefined;
        constructor();
        reset(fullReset?: boolean): void;
    }


    /** @internal */
    export class WebGPURenderTargetWrapper extends RenderTargetWrapper {
        /** @internal */
        _defaultAttachments: number[];
    }


    /** @internal */
    export interface WebGPUBindingInfo {
        groupIndex: number;
        bindingIndex: number;
    }
    /** @internal */
    export interface WebGPUTextureDescription {
        autoBindSampler?: boolean;
        isTextureArray: boolean;
        isStorageTexture: boolean;
        textures: Array<WebGPUBindingInfo>;
        sampleType?: GPUTextureSampleType;
    }
    /** @internal */
    export interface WebGPUSamplerDescription {
        binding: WebGPUBindingInfo;
        type: GPUSamplerBindingType;
    }
    /** @internal */
    export interface WebGPUBufferDescription {
        binding: WebGPUBindingInfo;
    }
    /** @internal */
    export interface WebGPUBindGroupLayoutEntryInfo {
        name: string;
        index: number;
        nameInArrayOfTexture?: string;
    }
    /**
     * @internal
     */
    export class WebGPUShaderProcessingContext implements ShaderProcessingContext {
        /** @internal */
        static _SimplifiedKnownBindings: boolean;
        protected static _SimplifiedKnownUBOs: {
            [key: string]: WebGPUBufferDescription;
        };
        protected static _KnownUBOs: {
            [key: string]: WebGPUBufferDescription;
        };
        static get KnownUBOs(): {
            [key: string]: WebGPUBufferDescription;
        };
        shaderLanguage: ShaderLanguage;
        uboNextBindingIndex: number;
        freeGroupIndex: number;
        freeBindingIndex: number;
        availableVaryings: {
            [key: string]: number;
        };
        availableAttributes: {
            [key: string]: number;
        };
        availableBuffers: {
            [key: string]: WebGPUBufferDescription;
        };
        availableTextures: {
            [key: string]: WebGPUTextureDescription;
        };
        availableSamplers: {
            [key: string]: WebGPUSamplerDescription;
        };
        leftOverUniforms: {
            name: string;
            type: string;
            length: number;
        }[];
        orderedAttributes: string[];
        bindGroupLayoutEntries: GPUBindGroupLayoutEntry[][];
        bindGroupLayoutEntryInfo: WebGPUBindGroupLayoutEntryInfo[][];
        bindGroupEntries: GPUBindGroupEntry[][];
        bufferNames: string[];
        textureNames: string[];
        samplerNames: string[];
        attributeNamesFromEffect: string[];
        attributeLocationsFromEffect: number[];
        private _attributeNextLocation;
        private _varyingNextLocation;
        constructor(shaderLanguage: ShaderLanguage);
        private _findStartingGroupBinding;
        getAttributeNextLocation(dataType: string, arrayLength?: number): number;
        getVaryingNextLocation(dataType: string, arrayLength?: number): number;
        getNextFreeUBOBinding(): {
            groupIndex: number;
            bindingIndex: number;
        };
        private _getNextFreeBinding;
    }


    /** @internal */
    export abstract class WebGPUShaderProcessor implements IShaderProcessor {
        static readonly AutoSamplerSuffix = "Sampler";
        static readonly LeftOvertUBOName = "LeftOver";
        static readonly InternalsUBOName = "Internals";
        static UniformSizes: {
            [type: string]: number;
        };
        protected static _SamplerFunctionByWebGLSamplerType: {
            [key: string]: string;
        };
        protected static _TextureTypeByWebGLSamplerType: {
            [key: string]: string;
        };
        protected static _GpuTextureViewDimensionByWebGPUTextureType: {
            [key: string]: GPUTextureViewDimension;
        };
        protected static _SamplerTypeByWebGLSamplerType: {
            [key: string]: string;
        };
        protected static _IsComparisonSamplerByWebGPUSamplerType: {
            [key: string]: boolean;
        };
        shaderLanguage: ShaderLanguage;
        protected _webgpuProcessingContext: WebGPUShaderProcessingContext;
        protected abstract _getArraySize(name: string, type: string, preProcessors: {
            [key: string]: string;
        }): [string, string, number];
        protected abstract _generateLeftOverUBOCode(name: string, uniformBufferDescription: WebGPUBufferDescription): string;
        protected _addUniformToLeftOverUBO(name: string, uniformType: string, preProcessors: {
            [key: string]: string;
        }): void;
        protected _buildLeftOverUBO(): string;
        protected _collectBindingNames(): void;
        protected _preCreateBindGroupEntries(): void;
        protected _addTextureBindingDescription(name: string, textureInfo: WebGPUTextureDescription, textureIndex: number, dimension: Nullable<GPUTextureViewDimension>, format: Nullable<GPUTextureFormat>, isVertex: boolean): void;
        protected _addSamplerBindingDescription(name: string, samplerInfo: WebGPUSamplerDescription, isVertex: boolean): void;
        protected _addBufferBindingDescription(name: string, uniformBufferInfo: WebGPUBufferDescription, bufferType: GPUBufferBindingType, isVertex: boolean): void;
        protected _injectStartingAndEndingCode(code: string, mainFuncDecl: string, startingCode?: string, endingCode?: string): string;
    }


    /** @internal */
    export class WebGPUShaderProcessorGLSL extends WebGPUShaderProcessor {
        protected _missingVaryings: Array<string>;
        protected _textureArrayProcessing: Array<string>;
        protected _preProcessors: {
            [key: string]: string;
        };
        protected _vertexIsGLES3: boolean;
        protected _fragmentIsGLES3: boolean;
        shaderLanguage: ShaderLanguage;
        parseGLES3: boolean;
        attributeKeywordName: string | undefined;
        varyingVertexKeywordName: string | undefined;
        varyingFragmentKeywordName: string | undefined;
        protected _getArraySize(name: string, type: string, preProcessors: {
            [key: string]: string;
        }): [string, string, number];
        initializeShaders(processingContext: Nullable<ShaderProcessingContext>): void;
        preProcessShaderCode(code: string, isFragment: boolean): string;
        varyingProcessor(varying: string, isFragment: boolean, preProcessors: {
            [key: string]: string;
        }): string;
        attributeProcessor(attribute: string, preProcessors: {
            [key: string]: string;
        }): string;
        uniformProcessor(uniform: string, isFragment: boolean, preProcessors: {
            [key: string]: string;
        }): string;
        uniformBufferProcessor(uniformBuffer: string, isFragment: boolean): string;
        postProcessor(code: string, defines: string[], isFragment: boolean, processingContext: Nullable<ShaderProcessingContext>, engine: ThinEngine): string;
        private _applyTextureArrayProcessing;
        protected _generateLeftOverUBOCode(name: string, uniformBufferDescription: WebGPUBufferDescription): string;
        finalizeShaders(vertexCode: string, fragmentCode: string): {
            vertexCode: string;
            fragmentCode: string;
        };
    }


    /** @internal */
    export class WebGPUShaderProcessorWGSL extends WebGPUShaderProcessor {
        protected _attributesWGSL: string[];
        protected _attributesDeclWGSL: string[];
        protected _attributeNamesWGSL: string[];
        protected _varyingsWGSL: string[];
        protected _varyingsDeclWGSL: string[];
        protected _varyingNamesWGSL: string[];
        protected _stridedUniformArrays: string[];
        shaderLanguage: ShaderLanguage;
        uniformRegexp: RegExp;
        textureRegexp: RegExp;
        noPrecision: boolean;
        protected _getArraySize(name: string, uniformType: string, preProcessors: {
            [key: string]: string;
        }): [string, string, number];
        initializeShaders(processingContext: Nullable<ShaderProcessingContext>): void;
        preProcessShaderCode(code: string): string;
        varyingProcessor(varying: string, isFragment: boolean, preProcessors: {
            [key: string]: string;
        }): string;
        attributeProcessor(attribute: string, preProcessors: {
            [key: string]: string;
        }): string;
        uniformProcessor(uniform: string, isFragment: boolean, preProcessors: {
            [key: string]: string;
        }): string;
        textureProcessor(texture: string, isFragment: boolean, preProcessors: {
            [key: string]: string;
        }): string;
        postProcessor(code: string): string;
        finalizeShaders(vertexCode: string, fragmentCode: string): {
            vertexCode: string;
            fragmentCode: string;
        };
        protected _generateLeftOverUBOCode(name: string, uniformBufferDescription: WebGPUBufferDescription): string;
        private _processSamplers;
        private _processCustomBuffers;
        private _processStridedUniformArrays;
    }


    /** @internal */
    export class WebGPUSnapshotRendering {
        private _engine;
        private _record;
        private _play;
        private _mainPassBundleList;
        private _modeSaved;
        private _bundleList;
        private _bundleListRenderTarget;
        private _enabled;
        private _mode;
        constructor(engine: WebGPUEngine, renderingMode: number, bundleList: WebGPUBundleList, bundleListRenderTarget: WebGPUBundleList);
        get enabled(): boolean;
        get play(): boolean;
        get record(): boolean;
        set enabled(activate: boolean);
        get mode(): number;
        set mode(mode: number);
        endMainRenderPass(): void;
        endRenderTargetPass(currentRenderPass: GPURenderPassEncoder, gpuWrapper: WebGPUHardwareTexture): boolean;
        endFrame(mainRenderPass: Nullable<GPURenderPassEncoder>): void;
        reset(): void;
    }


    /**
     * @internal
     **/
    export class WebGPUStencilStateComposer extends StencilStateComposer {
        private _cache;
        constructor(cache: WebGPUCacheRenderPipeline);
        get func(): number;
        set func(value: number);
        get funcMask(): number;
        set funcMask(value: number);
        get opStencilFail(): number;
        set opStencilFail(value: number);
        get opDepthFail(): number;
        set opDepthFail(value: number);
        get opStencilDepthPass(): number;
        set opStencilDepthPass(value: number);
        get mask(): number;
        set mask(value: number);
        get enabled(): boolean;
        set enabled(value: boolean);
        reset(): void;
        apply(): void;
    }


    /**
     * Map a (renderable) texture format (GPUTextureFormat) to an index for fast lookup (in caches for eg)
     */
    export var renderableTextureFormatToIndex: {
        [name: string]: number;
    };
    /** @internal */
    export class WebGPUTextureHelper {
        private _device;
        private _glslang;
        private _tintWASM;
        private _bufferManager;
        private _mipmapSampler;
        private _videoSampler;
        private _ubCopyWithOfst;
        private _pipelines;
        private _compiledShaders;
        private _videoPipelines;
        private _videoCompiledShaders;
        private _deferredReleaseTextures;
        private _commandEncoderForCreation;
        static ComputeNumMipmapLevels(width: number, height: number): number;
        constructor(device: GPUDevice, glslang: any, tintWASM: Nullable<WebGPUTintWASM>, bufferManager: WebGPUBufferManager);
        private _getPipeline;
        private _getVideoPipeline;
        private static _GetTextureTypeFromFormat;
        private static _GetBlockInformationFromFormat;
        private static _IsHardwareTexture;
        private static _IsInternalTexture;
        static IsImageBitmap(imageBitmap: ImageBitmap | {
            width: number;
            height: number;
        }): imageBitmap is ImageBitmap;
        static IsImageBitmapArray(imageBitmap: ImageBitmap[] | {
            width: number;
            height: number;
        }): imageBitmap is ImageBitmap[];
        setCommandEncoder(encoder: GPUCommandEncoder): void;
        static IsCompressedFormat(format: GPUTextureFormat): boolean;
        static GetWebGPUTextureFormat(type: number, format: number, useSRGBBuffer?: boolean): GPUTextureFormat;
        static GetNumChannelsFromWebGPUTextureFormat(format: GPUTextureFormat): number;
        static HasStencilAspect(format: GPUTextureFormat): boolean;
        static HasDepthAndStencilAspects(format: GPUTextureFormat): boolean;
        copyVideoToTexture(video: ExternalTexture, texture: InternalTexture, format: GPUTextureFormat, invertY?: boolean, commandEncoder?: GPUCommandEncoder): void;
        invertYPreMultiplyAlpha(gpuOrHdwTexture: GPUTexture | WebGPUHardwareTexture, width: number, height: number, format: GPUTextureFormat, invertY?: boolean, premultiplyAlpha?: boolean, faceIndex?: number, mipLevel?: number, layers?: number, ofstX?: number, ofstY?: number, rectWidth?: number, rectHeight?: number, commandEncoder?: GPUCommandEncoder, allowGPUOptimization?: boolean): void;
        copyWithInvertY(srcTextureView: GPUTextureView, format: GPUTextureFormat, renderPassDescriptor: GPURenderPassDescriptor, commandEncoder?: GPUCommandEncoder): void;
        createTexture(imageBitmap: ImageBitmap | {
            width: number;
            height: number;
            layers: number;
        }, hasMipmaps?: boolean, generateMipmaps?: boolean, invertY?: boolean, premultiplyAlpha?: boolean, is3D?: boolean, format?: GPUTextureFormat, sampleCount?: number, commandEncoder?: GPUCommandEncoder, usage?: number, additionalUsages?: number): GPUTexture;
        createCubeTexture(imageBitmaps: ImageBitmap[] | {
            width: number;
            height: number;
        }, hasMipmaps?: boolean, generateMipmaps?: boolean, invertY?: boolean, premultiplyAlpha?: boolean, format?: GPUTextureFormat, sampleCount?: number, commandEncoder?: GPUCommandEncoder, usage?: number, additionalUsages?: number): GPUTexture;
        generateCubeMipmaps(gpuTexture: GPUTexture | WebGPUHardwareTexture, format: GPUTextureFormat, mipLevelCount: number, commandEncoder?: GPUCommandEncoder): void;
        generateMipmaps(gpuOrHdwTexture: GPUTexture | WebGPUHardwareTexture, format: GPUTextureFormat, mipLevelCount: number, faceIndex?: number, commandEncoder?: GPUCommandEncoder): void;
        createGPUTextureForInternalTexture(texture: InternalTexture, width?: number, height?: number, depth?: number, creationFlags?: number): WebGPUHardwareTexture;
        createMSAATexture(texture: InternalTexture, samples: number): void;
        updateCubeTextures(imageBitmaps: ImageBitmap[] | Uint8Array[], gpuTexture: GPUTexture, width: number, height: number, format: GPUTextureFormat, invertY?: boolean, premultiplyAlpha?: boolean, offsetX?: number, offsetY?: number): void;
        updateTexture(imageBitmap: ImageBitmap | Uint8Array | HTMLCanvasElement | OffscreenCanvas, texture: GPUTexture | InternalTexture, width: number, height: number, layers: number, format: GPUTextureFormat, faceIndex?: number, mipLevel?: number, invertY?: boolean, premultiplyAlpha?: boolean, offsetX?: number, offsetY?: number, allowGPUOptimization?: boolean): void;
        readPixels(texture: GPUTexture, x: number, y: number, width: number, height: number, format: GPUTextureFormat, faceIndex?: number, mipLevel?: number, buffer?: Nullable<ArrayBufferView>, noDataConversion?: boolean): Promise<ArrayBufferView>;
        releaseTexture(texture: InternalTexture | GPUTexture): void;
        destroyDeferredTextures(): void;
    }


    /** @internal */
    export class WebGPUTimestampQuery {
        private _device;
        private _bufferManager;
        private _enabled;
        private _gpuFrameTimeCounter;
        private _measureDuration;
        private _measureDurationState;
        get gpuFrameTimeCounter(): PerfCounter;
        constructor(device: GPUDevice, bufferManager: WebGPUBufferManager);
        get enable(): boolean;
        set enable(value: boolean);
        startFrame(commandEncoder: GPUCommandEncoder): void;
        endFrame(commandEncoder: GPUCommandEncoder): void;
    }
    /** @internal */
    export class WebGPUDurationMeasure {
        private _querySet;
        constructor(device: GPUDevice, bufferManager: WebGPUBufferManager);
        start(encoder: GPUCommandEncoder): void;
        stop(encoder: GPUCommandEncoder): Promise<number | null>;
        dispose(): void;
    }


    /**
     * Options to load the associated Twgsl library
     */
    export interface TwgslOptions {
        /**
         * Defines an existing instance of Twgsl (useful in modules who do not access the global instance).
         */
        twgsl?: any;
        /**
         * Defines the URL of the twgsl JS File.
         */
        jsPath?: string;
        /**
         * Defines the URL of the twgsl WASM File.
         */
        wasmPath?: string;
    }
    /** @internal */
    export class WebGPUTintWASM {
        private static readonly _TWgslDefaultOptions;
        static ShowWGSLShaderCode: boolean;
        private _twgsl;
        initTwgsl(twgslOptions?: TwgslOptions): Promise<void>;
        convertSpirV2WGSL(code: Uint32Array): string;
    }


    /**
     * Options to load the associated Glslang library
     */
    export interface GlslangOptions {
        /**
         * Defines an existing instance of Glslang (useful in modules who do not access the global instance).
         */
        glslang?: any;
        /**
         * Defines the URL of the glslang JS File.
         */
        jsPath?: string;
        /**
         * Defines the URL of the glslang WASM File.
         */
        wasmPath?: string;
    }
    /**
     * Options to create the WebGPU engine
     */
    export interface WebGPUEngineOptions extends GPURequestAdapterOptions {
        /**
         * If delta time between frames should be constant
         * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/advanced_animations#deterministic-lockstep
         */
        deterministicLockstep?: boolean;
        /**
         * Maximum about of steps between frames (Default: 4)
         * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/advanced_animations#deterministic-lockstep
         */
        lockstepMaxSteps?: number;
        /**
         * Defines the seconds between each deterministic lock step
         */
        timeStep?: number;
        /**
         * Defines that engine should ignore context lost events
         * If this event happens when this parameter is true, you will have to reload the page to restore rendering
         */
        doNotHandleContextLost?: boolean;
        /**
         * Defines that engine should ignore modifying touch action attribute and style
         * If not handle, you might need to set it up on your side for expected touch devices behavior.
         */
        doNotHandleTouchAction?: boolean;
        /**
         * Defines if webaudio should be initialized as well
         * @see https://doc.babylonjs.com/features/featuresDeepDive/audio/playingSoundsMusic
         */
        audioEngine?: boolean;
        /**
         * Defines the category of adapter to use.
         * Is it the discrete or integrated device.
         */
        powerPreference?: GPUPowerPreference;
        /**
         * Defines the device descriptor used to create a device.
         */
        deviceDescriptor?: GPUDeviceDescriptor;
        /**
         * Defines the requested Swap Chain Format.
         */
        swapChainFormat?: GPUTextureFormat;
        /**
         * Defines whether MSAA is enabled on the canvas.
         */
        antialiasing?: boolean;
        /**
         * Defines whether the stencil buffer should be enabled.
         */
        stencil?: boolean;
        /**
         * Defines whether we should generate debug markers in the gpu command lists (can be seen with PIX for eg)
         */
        enableGPUDebugMarkers?: boolean;
        /**
         * Options to load the associated Glslang library
         */
        glslangOptions?: GlslangOptions;
        /**
         * Options to load the associated Twgsl library
         */
        twgslOptions?: TwgslOptions;
        /**
         * Defines if the engine should no exceed a specified device ratio
         * @see https://developer.mozilla.org/en-US/docs/Web/API/Window/devicePixelRatio
         */
        limitDeviceRatio?: number;
        /**
         * Defines whether to adapt to the device's viewport characteristics (default: false)
         */
        adaptToDeviceRatio?: boolean;
        /**
         * Defines whether the canvas should be created in "premultiplied" mode (if false, the canvas is created in the "opaque" mode) (true by default)
         */
        premultipliedAlpha?: boolean;
        /**
         * Make the matrix computations to be performed in 64 bits instead of 32 bits. False by default
         */
        useHighPrecisionMatrix?: boolean;
    }
    /**
     * The web GPU engine class provides support for WebGPU version of babylon.js.
     * @since 5.0.0
     */
    export class WebGPUEngine extends Engine {
        private static readonly _GLSLslangDefaultOptions;
        /** true to enable using TintWASM to convert Spir-V to WGSL */
        static UseTWGSL: boolean;
        private readonly _uploadEncoderDescriptor;
        private readonly _renderEncoderDescriptor;
        private readonly _renderTargetEncoderDescriptor;
        /** @internal */
        readonly _clearDepthValue = 1;
        /** @internal */
        readonly _clearReverseDepthValue = 0;
        /** @internal */
        readonly _clearStencilValue = 0;
        private readonly _defaultSampleCount;
        private _canvas;
        /** @internal */
        _options: WebGPUEngineOptions;
        private _glslang;
        private _tintWASM;
        private _adapter;
        private _adapterSupportedExtensions;
        /** @internal */
        _device: GPUDevice;
        private _deviceEnabledExtensions;
        private _context;
        private _mainPassSampleCount;
        /** @internal */
        _textureHelper: WebGPUTextureHelper;
        /** @internal */
        _bufferManager: WebGPUBufferManager;
        private _clearQuad;
        /** @internal */
        _cacheSampler: WebGPUCacheSampler;
        /** @internal */
        _cacheRenderPipeline: WebGPUCacheRenderPipeline;
        private _cacheBindGroups;
        private _emptyVertexBuffer;
        /** @internal */
        _mrtAttachments: number[];
        /** @internal */
        _timestampQuery: WebGPUTimestampQuery;
        /** @internal */
        _occlusionQuery: WebGPUOcclusionQuery;
        /** @internal */
        _compiledComputeEffects: {
            [key: string]: ComputeEffect;
        };
        /** @internal */
        _counters: {
            numEnableEffects: number;
            numEnableDrawWrapper: number;
            numBundleCreationNonCompatMode: number;
            numBundleReuseNonCompatMode: number;
        };
        /**
         * Counters from last frame
         */
        readonly countersLastFrame: {
            numEnableEffects: number;
            numEnableDrawWrapper: number;
            numBundleCreationNonCompatMode: number;
            numBundleReuseNonCompatMode: number;
        };
        /**
         * Max number of uncaptured error messages to log
         */
        numMaxUncapturedErrors: number;
        private _mainTexture;
        private _depthTexture;
        private _mainTextureExtends;
        private _depthTextureFormat;
        private _colorFormat;
        /** @internal */
        _ubInvertY: WebGPUDataBuffer;
        /** @internal */
        _ubDontInvertY: WebGPUDataBuffer;
        /** @internal */
        _uploadEncoder: GPUCommandEncoder;
        /** @internal */
        _renderEncoder: GPUCommandEncoder;
        /** @internal */
        _renderTargetEncoder: GPUCommandEncoder;
        private _commandBuffers;
        /** @internal */
        _currentRenderPass: Nullable<GPURenderPassEncoder>;
        /** @internal */
        _mainRenderPassWrapper: WebGPURenderPassWrapper;
        /** @internal */
        _rttRenderPassWrapper: WebGPURenderPassWrapper;
        /** @internal */
        _pendingDebugCommands: Array<[string, Nullable<string>]>;
        /** @internal */
        _bundleList: WebGPUBundleList;
        /** @internal */
        _bundleListRenderTarget: WebGPUBundleList;
        /** @internal */
        _onAfterUnbindFrameBufferObservable: Observable<WebGPUEngine>;
        private _defaultDrawContext;
        private _defaultMaterialContext;
        /** @internal */
        _currentDrawContext: WebGPUDrawContext;
        /** @internal */
        _currentMaterialContext: WebGPUMaterialContext;
        private _currentOverrideVertexBuffers;
        private _currentIndexBuffer;
        private _colorWriteLocal;
        private _forceEnableEffect;
        /** @internal */
        dbgShowShaderCode: boolean;
        /** @internal */
        dbgSanityChecks: boolean;
        /** @internal */
        dbgVerboseLogsForFirstFrames: boolean;
        /** @internal */
        dbgVerboseLogsNumFrames: number;
        /** @internal */
        dbgLogIfNotDrawWrapper: boolean;
        /** @internal */
        dbgShowEmptyEnableEffectCalls: boolean;
        private _snapshotRendering;
        /**
         * Gets or sets the snapshot rendering mode
         */
        get snapshotRenderingMode(): number;
        set snapshotRenderingMode(mode: number);
        /**
         * Creates a new snapshot at the next frame using the current snapshotRenderingMode
         */
        snapshotRenderingReset(): void;
        /**
         * Enables or disables the snapshot rendering mode
         * Note that the WebGL engine does not support snapshot rendering so setting the value won't have any effect for this engine
         */
        get snapshotRendering(): boolean;
        set snapshotRendering(activate: boolean);
        /**
         * Sets this to true to disable the cache for the samplers. You should do it only for testing purpose!
         */
        get disableCacheSamplers(): boolean;
        set disableCacheSamplers(disable: boolean);
        /**
         * Sets this to true to disable the cache for the render pipelines. You should do it only for testing purpose!
         */
        get disableCacheRenderPipelines(): boolean;
        set disableCacheRenderPipelines(disable: boolean);
        /**
         * Sets this to true to disable the cache for the bind groups. You should do it only for testing purpose!
         */
        get disableCacheBindGroups(): boolean;
        set disableCacheBindGroups(disable: boolean);
        /**
         * Gets a Promise<boolean> indicating if the engine can be instantiated (ie. if a WebGPU context can be found)
         */
        static get IsSupportedAsync(): Promise<boolean>;
        /**
         * Not supported by WebGPU, you should call IsSupportedAsync instead!
         */
        static get IsSupported(): boolean;
        /**
         * Gets a boolean indicating that the engine supports uniform buffers
         */
        get supportsUniformBuffers(): boolean;
        /** Gets the supported extensions by the WebGPU adapter */
        get supportedExtensions(): Immutable<GPUFeatureName[]>;
        /** Gets the currently enabled extensions on the WebGPU device */
        get enabledExtensions(): Immutable<GPUFeatureName[]>;
        /**
         * Returns a string describing the current engine
         */
        get description(): string;
        /**
         * Returns the version of the engine
         */
        get version(): number;
        /**
         * Gets an object containing information about the current engine context
         * @returns an object containing the vendor, the renderer and the version of the current engine context
         */
        getInfo(): {
            vendor: string;
            renderer: string;
            version: string;
        };
        /**
         * (WebGPU only) True (default) to be in compatibility mode, meaning rendering all existing scenes without artifacts (same rendering than WebGL).
         * Setting the property to false will improve performances but may not work in some scenes if some precautions are not taken.
         * See https://doc.babylonjs.com/setup/support/webGPU/webGPUOptimization/webGPUNonCompatibilityMode for more details
         */
        get compatibilityMode(): boolean;
        set compatibilityMode(mode: boolean);
        /** @internal */
        get currentSampleCount(): number;
        /**
         * Create a new instance of the gpu engine asynchronously
         * @param canvas Defines the canvas to use to display the result
         * @param options Defines the options passed to the engine to create the GPU context dependencies
         * @returns a promise that resolves with the created engine
         */
        static CreateAsync(canvas: HTMLCanvasElement, options?: WebGPUEngineOptions): Promise<WebGPUEngine>;
        /**
         * Create a new instance of the gpu engine.
         * @param canvas Defines the canvas to use to display the result
         * @param options Defines the options passed to the engine to create the GPU context dependencies
         */
        constructor(canvas: HTMLCanvasElement, options?: WebGPUEngineOptions);
        /**
         * Initializes the WebGPU context and dependencies.
         * @param glslangOptions Defines the GLSLang compiler options if necessary
         * @param twgslOptions Defines the Twgsl compiler options if necessary
         * @returns a promise notifying the readiness of the engine.
         */
        initAsync(glslangOptions?: GlslangOptions, twgslOptions?: TwgslOptions): Promise<void>;
        private _initGlslang;
        private _initializeLimits;
        private _initializeContextAndSwapChain;
        private _initializeMainAttachments;
        private _configureContext;
        /**
         * Force a specific size of the canvas
         * @param width defines the new canvas' width
         * @param height defines the new canvas' height
         * @param forceSetSize true to force setting the sizes of the underlying canvas
         * @returns true if the size was changed
         */
        setSize(width: number, height: number, forceSetSize?: boolean): boolean;
        private _shaderProcessorWGSL;
        /**
         * @internal
         */
        _getShaderProcessor(shaderLanguage: ShaderLanguage): Nullable<IShaderProcessor>;
        /**
         * @internal
         */
        _getShaderProcessingContext(shaderLanguage: ShaderLanguage): Nullable<ShaderProcessingContext>;
        /** @internal */
        applyStates(): void;
        /**
         * Force the entire cache to be cleared
         * You should not have to use this function unless your engine needs to share the WebGPU context with another engine
         * @param bruteForce defines a boolean to force clearing ALL caches (including stencil, detoh and alpha states)
         */
        wipeCaches(bruteForce?: boolean): void;
        /**
         * Enable or disable color writing
         * @param enable defines the state to set
         */
        setColorWrite(enable: boolean): void;
        /**
         * Gets a boolean indicating if color writing is enabled
         * @returns the current color writing state
         */
        getColorWrite(): boolean;
        private _viewportsCurrent;
        private _resetCurrentViewport;
        private _mustUpdateViewport;
        private _applyViewport;
        /**
         * @internal
         */
        _viewport(x: number, y: number, width: number, height: number): void;
        private _scissorsCurrent;
        protected _scissorCached: {
            x: number;
            y: number;
            z: number;
            w: number;
        };
        private _resetCurrentScissor;
        private _mustUpdateScissor;
        private _applyScissor;
        private _scissorIsActive;
        enableScissor(x: number, y: number, width: number, height: number): void;
        disableScissor(): void;
        private _stencilRefsCurrent;
        private _resetCurrentStencilRef;
        private _mustUpdateStencilRef;
        /**
         * @internal
         */
        _applyStencilRef(renderPass: GPURenderPassEncoder): void;
        private _blendColorsCurrent;
        private _resetCurrentColorBlend;
        private _mustUpdateBlendColor;
        private _applyBlendColor;
        /**
         * Clear the current render buffer or the current render target (if any is set up)
         * @param color defines the color to use
         * @param backBuffer defines if the back buffer must be cleared
         * @param depth defines if the depth buffer must be cleared
         * @param stencil defines if the stencil buffer must be cleared
         */
        clear(color: Nullable<IColor4Like>, backBuffer: boolean, depth: boolean, stencil?: boolean): void;
        private _clearFullQuad;
        /**
         * Creates a vertex buffer
         * @param data the data for the vertex buffer
         * @returns the new buffer
         */
        createVertexBuffer(data: DataArray): DataBuffer;
        /**
         * Creates a vertex buffer
         * @param data the data for the dynamic vertex buffer
         * @returns the new buffer
         */
        createDynamicVertexBuffer(data: DataArray): DataBuffer;
        /**
         * Creates a new index buffer
         * @param indices defines the content of the index buffer
         * @returns a new buffer
         */
        createIndexBuffer(indices: IndicesArray): DataBuffer;
        /**
         * @internal
         */
        _createBuffer(data: DataArray | number, creationFlags: number): DataBuffer;
        /**
         * @internal
         */
        bindBuffersDirectly(): void;
        /**
         * @internal
         */
        updateAndBindInstancesBuffer(): void;
        /**
         * Bind a list of vertex buffers with the engine
         * @param vertexBuffers defines the list of vertex buffers to bind
         * @param indexBuffer defines the index buffer to bind
         * @param effect defines the effect associated with the vertex buffers
         * @param overrideVertexBuffers defines optional list of avertex buffers that overrides the entries in vertexBuffers
         */
        bindBuffers(vertexBuffers: {
            [key: string]: Nullable<VertexBuffer>;
        }, indexBuffer: Nullable<DataBuffer>, effect: Effect, overrideVertexBuffers?: {
            [kind: string]: Nullable<VertexBuffer>;
        }): void;
        /**
         * @internal
         */
        _releaseBuffer(buffer: DataBuffer): boolean;
        /**
         * Create a new effect (used to store vertex/fragment shaders)
         * @param baseName defines the base name of the effect (The name of file without .fragment.fx or .vertex.fx)
         * @param attributesNamesOrOptions defines either a list of attribute names or an IEffectCreationOptions object
         * @param uniformsNamesOrEngine defines either a list of uniform names or the engine to use
         * @param samplers defines an array of string used to represent textures
         * @param defines defines the string containing the defines to use to compile the shaders
         * @param fallbacks defines the list of potential fallbacks to use if shader compilation fails
         * @param onCompiled defines a function to call when the effect creation is successful
         * @param onError defines a function to call when the effect creation has failed
         * @param indexParameters defines an object containing the index values to use to compile shaders (like the maximum number of simultaneous lights)
         * @param shaderLanguage the language the shader is written in (default: GLSL)
         * @returns the new Effect
         */
        createEffect(baseName: any, attributesNamesOrOptions: string[] | IEffectCreationOptions, uniformsNamesOrEngine: string[] | Engine, samplers?: string[], defines?: string, fallbacks?: EffectFallbacks, onCompiled?: Nullable<(effect: Effect) => void>, onError?: Nullable<(effect: Effect, errors: string) => void>, indexParameters?: any, shaderLanguage?: ShaderLanguage): Effect;
        private _compileRawShaderToSpirV;
        private _compileShaderToSpirV;
        private _getWGSLShader;
        private _createPipelineStageDescriptor;
        private _compileRawPipelineStageDescriptor;
        private _compilePipelineStageDescriptor;
        /**
         * @internal
         */
        createRawShaderProgram(): WebGLProgram;
        /**
         * @internal
         */
        createShaderProgram(): WebGLProgram;
        /**
         * Inline functions in shader code that are marked to be inlined
         * @param code code to inline
         * @returns inlined code
         */
        inlineShaderCode(code: string): string;
        /**
         * Creates a new pipeline context
         * @param shaderProcessingContext defines the shader processing context used during the processing if available
         * @returns the new pipeline
         */
        createPipelineContext(shaderProcessingContext: Nullable<ShaderProcessingContext>): IPipelineContext;
        /**
         * Creates a new material context
         * @returns the new context
         */
        createMaterialContext(): WebGPUMaterialContext | undefined;
        /**
         * Creates a new draw context
         * @returns the new context
         */
        createDrawContext(): WebGPUDrawContext | undefined;
        /**
         * @internal
         */
        _preparePipelineContext(pipelineContext: IPipelineContext, vertexSourceCode: string, fragmentSourceCode: string, createAsRaw: boolean, rawVertexSourceCode: string, rawFragmentSourceCode: string, rebuildRebind: any, defines: Nullable<string>): void;
        /**
         * Gets the list of active attributes for a given WebGPU program
         * @param pipelineContext defines the pipeline context to use
         * @param attributesNames defines the list of attribute names to get
         * @returns an array of indices indicating the offset of each attribute
         */
        getAttributes(pipelineContext: IPipelineContext, attributesNames: string[]): number[];
        /**
         * Activates an effect, making it the current one (ie. the one used for rendering)
         * @param effect defines the effect to activate
         */
        enableEffect(effect: Nullable<Effect | DrawWrapper>): void;
        /**
         * @internal
         */
        _releaseEffect(effect: Effect): void;
        /**
         * Force the engine to release all cached effects. This means that next effect compilation will have to be done completely even if a similar effect was already compiled
         */
        releaseEffects(): void;
        _deletePipelineContext(pipelineContext: IPipelineContext): void;
        /**
         * Gets a boolean indicating that only power of 2 textures are supported
         * Please note that you can still use non power of 2 textures but in this case the engine will forcefully convert them
         */
        get needPOTTextures(): boolean;
        /** @internal */
        _createHardwareTexture(): HardwareTextureWrapper;
        /**
         * @internal
         */
        _releaseTexture(texture: InternalTexture): void;
        /**
         * @internal
         */
        _getRGBABufferInternalSizedFormat(): number;
        updateTextureComparisonFunction(texture: InternalTexture, comparisonFunction: number): void;
        /**
         * Creates an internal texture without binding it to a framebuffer
         * @internal
         * @param size defines the size of the texture
         * @param options defines the options used to create the texture
         * @param delayGPUTextureCreation true to delay the texture creation the first time it is really needed. false to create it right away
         * @param source source type of the texture
         * @returns a new internal texture
         */
        _createInternalTexture(size: TextureSize, options: boolean | InternalTextureCreationOptions, delayGPUTextureCreation?: boolean, source?: InternalTextureSource): InternalTexture;
        /**
         * Usually called from Texture.ts.
         * Passed information to create a hardware texture
         * @param url defines a value which contains one of the following:
         * * A conventional http URL, e.g. 'http://...' or 'file://...'
         * * A base64 string of in-line texture data, e.g. 'data:image/jpg;base64,/...'
         * * An indicator that data being passed using the buffer parameter, e.g. 'data:mytexture.jpg'
         * @param noMipmap defines a boolean indicating that no mipmaps shall be generated.  Ignored for compressed textures.  They must be in the file
         * @param invertY when true, image is flipped when loaded.  You probably want true. Certain compressed textures may invert this if their default is inverted (eg. ktx)
         * @param scene needed for loading to the correct scene
         * @param samplingMode mode with should be used sample / access the texture (Default: Texture.TRILINEAR_SAMPLINGMODE)
         * @param onLoad optional callback to be called upon successful completion
         * @param onError optional callback to be called upon failure
         * @param buffer a source of a file previously fetched as either a base64 string, an ArrayBuffer (compressed or image format), HTMLImageElement (image format), or a Blob
         * @param fallback an internal argument in case the function must be called again, due to etc1 not having alpha capabilities
         * @param format internal format.  Default: RGB when extension is '.jpg' else RGBA.  Ignored for compressed textures
         * @param forcedExtension defines the extension to use to pick the right loader
         * @param mimeType defines an optional mime type
         * @param loaderOptions options to be passed to the loader
         * @param creationFlags specific flags to use when creating the texture (Constants.TEXTURE_CREATIONFLAG_STORAGE for storage textures, for eg)
         * @param useSRGBBuffer defines if the texture must be loaded in a sRGB GPU buffer (if supported by the GPU).
         * @returns a InternalTexture for assignment back into BABYLON.Texture
         */
        createTexture(url: Nullable<string>, noMipmap: boolean, invertY: boolean, scene: Nullable<ISceneLike>, samplingMode?: number, onLoad?: Nullable<(texture: InternalTexture) => void>, onError?: Nullable<(message: string, exception: any) => void>, buffer?: Nullable<string | ArrayBuffer | ArrayBufferView | HTMLImageElement | Blob | ImageBitmap>, fallback?: Nullable<InternalTexture>, format?: Nullable<number>, forcedExtension?: Nullable<string>, mimeType?: string, loaderOptions?: any, creationFlags?: number, useSRGBBuffer?: boolean): InternalTexture;
        /**
         * Wraps an external web gpu texture in a Babylon texture.
         * @param texture defines the external texture
         * @returns the babylon internal texture
         */
        wrapWebGPUTexture(texture: GPUTexture): InternalTexture;
        /**
         * Wraps an external web gl texture in a Babylon texture.
         * @returns the babylon internal texture
         */
        wrapWebGLTexture(): InternalTexture;
        generateMipMapsForCubemap(texture: InternalTexture): void;
        /**
         * Update the sampling mode of a given texture
         * @param samplingMode defines the required sampling mode
         * @param texture defines the texture to update
         * @param generateMipMaps defines whether to generate mipmaps for the texture
         */
        updateTextureSamplingMode(samplingMode: number, texture: InternalTexture, generateMipMaps?: boolean): void;
        /**
         * Update the sampling mode of a given texture
         * @param texture defines the texture to update
         * @param wrapU defines the texture wrap mode of the u coordinates
         * @param wrapV defines the texture wrap mode of the v coordinates
         * @param wrapR defines the texture wrap mode of the r coordinates
         */
        updateTextureWrappingMode(texture: InternalTexture, wrapU: Nullable<number>, wrapV?: Nullable<number>, wrapR?: Nullable<number>): void;
        /**
         * Update the dimensions of a texture
         * @param texture texture to update
         * @param width new width of the texture
         * @param height new height of the texture
         * @param depth new depth of the texture
         */
        updateTextureDimensions(texture: InternalTexture, width: number, height: number, depth?: number): void;
        /**
         * @internal
         */
        _setInternalTexture(name: string, texture: Nullable<InternalTexture | ExternalTexture>, baseName?: string): void;
        /**
         * Sets a texture to the according uniform.
         * @param channel The texture channel
         * @param unused unused parameter
         * @param texture The texture to apply
         * @param name The name of the uniform in the effect
         */
        setTexture(channel: number, unused: Nullable<WebGLUniformLocation>, texture: Nullable<BaseTexture>, name: string): void;
        /**
         * Sets an array of texture to the WebGPU context
         * @param channel defines the channel where the texture array must be set
         * @param unused unused parameter
         * @param textures defines the array of textures to bind
         * @param name name of the channel
         */
        setTextureArray(channel: number, unused: Nullable<WebGLUniformLocation>, textures: BaseTexture[], name: string): void;
        protected _setTexture(channel: number, texture: Nullable<BaseTexture>, isPartOfTextureArray?: boolean, depthStencilTexture?: boolean, name?: string, baseName?: string): boolean;
        /**
         * @internal
         */
        _setAnisotropicLevel(target: number, internalTexture: InternalTexture, anisotropicFilteringLevel: number): void;
        /**
         * @internal
         */
        _bindTexture(channel: number, texture: InternalTexture, name: string): void;
        /**
         * Generates the mipmaps for a texture
         * @param texture texture to generate the mipmaps for
         */
        generateMipmaps(texture: InternalTexture): void;
        /**
         * @internal
         */
        _generateMipmaps(texture: InternalTexture, commandEncoder?: GPUCommandEncoder): void;
        /**
         * Update a portion of an internal texture
         * @param texture defines the texture to update
         * @param imageData defines the data to store into the texture
         * @param xOffset defines the x coordinates of the update rectangle
         * @param yOffset defines the y coordinates of the update rectangle
         * @param width defines the width of the update rectangle
         * @param height defines the height of the update rectangle
         * @param faceIndex defines the face index if texture is a cube (0 by default)
         * @param lod defines the lod level to update (0 by default)
         * @param generateMipMaps defines whether to generate mipmaps or not
         */
        updateTextureData(texture: InternalTexture, imageData: ArrayBufferView, xOffset: number, yOffset: number, width: number, height: number, faceIndex?: number, lod?: number, generateMipMaps?: boolean): void;
        /**
         * @internal
         */
        _uploadCompressedDataToTextureDirectly(texture: InternalTexture, internalFormat: number, width: number, height: number, imageData: ArrayBufferView, faceIndex?: number, lod?: number): void;
        /**
         * @internal
         */
        _uploadDataToTextureDirectly(texture: InternalTexture, imageData: ArrayBufferView, faceIndex?: number, lod?: number, babylonInternalFormat?: number, useTextureWidthAndHeight?: boolean): void;
        /**
         * @internal
         */
        _uploadArrayBufferViewToTexture(texture: InternalTexture, imageData: ArrayBufferView, faceIndex?: number, lod?: number): void;
        /**
         * @internal
         */
        _uploadImageToTexture(texture: InternalTexture, image: HTMLImageElement | ImageBitmap, faceIndex?: number, lod?: number): void;
        /**
         * Reads pixels from the current frame buffer. Please note that this function can be slow
         * @param x defines the x coordinate of the rectangle where pixels must be read
         * @param y defines the y coordinate of the rectangle where pixels must be read
         * @param width defines the width of the rectangle where pixels must be read
         * @param height defines the height of the rectangle where pixels must be read
         * @param hasAlpha defines whether the output should have alpha or not (defaults to true)
         * @param flushRenderer true to flush the renderer from the pending commands before reading the pixels
         * @returns a ArrayBufferView promise (Uint8Array) containing RGBA colors
         */
        readPixels(x: number, y: number, width: number, height: number, hasAlpha?: boolean, flushRenderer?: boolean): Promise<ArrayBufferView>;
        /**
         * Begin a new frame
         */
        beginFrame(): void;
        /**
         * End the current frame
         */
        endFrame(): void;
        /**
         * Force a WebGPU flush (ie. a flush of all waiting commands)
         * @param reopenPass true to reopen at the end of the function the pass that was active when entering the function
         */
        flushFramebuffer(reopenPass?: boolean): void;
        /** @internal */
        _currentFrameBufferIsDefaultFrameBuffer(): boolean;
        private _startRenderTargetRenderPass;
        /** @internal */
        _endRenderTargetRenderPass(): void;
        private _getCurrentRenderPass;
        /** @internal */
        _getCurrentRenderPassIndex(): number;
        private _startMainRenderPass;
        private _endMainRenderPass;
        /**
         * Binds the frame buffer to the specified texture.
         * @param texture The render target wrapper to render to
         * @param faceIndex The face of the texture to render to in case of cube texture
         * @param requiredWidth The width of the target to render to
         * @param requiredHeight The height of the target to render to
         * @param forceFullscreenViewport Forces the viewport to be the entire texture/screen if true
         * @param lodLevel defines the lod level to bind to the frame buffer
         * @param layer defines the 2d array index to bind to frame buffer to
         */
        bindFramebuffer(texture: RenderTargetWrapper, faceIndex?: number, requiredWidth?: number, requiredHeight?: number, forceFullscreenViewport?: boolean, lodLevel?: number, layer?: number): void;
        /**
         * Unbind the current render target texture from the WebGPU context
         * @param texture defines the render target wrapper to unbind
         * @param disableGenerateMipMaps defines a boolean indicating that mipmaps must not be generated
         * @param onBeforeUnbind defines a function which will be called before the effective unbind
         */
        unBindFramebuffer(texture: RenderTargetWrapper, disableGenerateMipMaps?: boolean, onBeforeUnbind?: () => void): void;
        /**
         * Unbind the current render target and bind the default framebuffer
         */
        restoreDefaultFramebuffer(): void;
        /**
         * @internal
         */
        _setColorFormat(wrapper: WebGPURenderPassWrapper): void;
        /**
         * @internal
         */
        _setDepthTextureFormat(wrapper: WebGPURenderPassWrapper): void;
        setDitheringState(): void;
        setRasterizerState(): void;
        /**
         * Set various states to the webGL context
         * @param culling defines culling state: true to enable culling, false to disable it
         * @param zOffset defines the value to apply to zOffset (0 by default)
         * @param force defines if states must be applied even if cache is up to date
         * @param reverseSide defines if culling must be reversed (CCW if false, CW if true)
         * @param cullBackFaces true to cull back faces, false to cull front faces (if culling is enabled)
         * @param stencil stencil states to set
         * @param zOffsetUnits defines the value to apply to zOffsetUnits (0 by default)
         */
        setState(culling: boolean, zOffset?: number, force?: boolean, reverseSide?: boolean, cullBackFaces?: boolean, stencil?: IStencilState, zOffsetUnits?: number): void;
        private _applyRenderPassChanges;
        private _draw;
        /**
         * Draw a list of indexed primitives
         * @param fillMode defines the primitive to use
         * @param indexStart defines the starting index
         * @param indexCount defines the number of index to draw
         * @param instancesCount defines the number of instances to draw (if instantiation is enabled)
         */
        drawElementsType(fillMode: number, indexStart: number, indexCount: number, instancesCount?: number): void;
        /**
         * Draw a list of unindexed primitives
         * @param fillMode defines the primitive to use
         * @param verticesStart defines the index of first vertex to draw
         * @param verticesCount defines the count of vertices to draw
         * @param instancesCount defines the number of instances to draw (if instantiation is enabled)
         */
        drawArraysType(fillMode: number, verticesStart: number, verticesCount: number, instancesCount?: number): void;
        /**
         * Dispose and release all associated resources
         */
        dispose(): void;
        /**
         * Gets the current render width
         * @param useScreen defines if screen size must be used (or the current render target if any)
         * @returns a number defining the current render width
         */
        getRenderWidth(useScreen?: boolean): number;
        /**
         * Gets the current render height
         * @param useScreen defines if screen size must be used (or the current render target if any)
         * @returns a number defining the current render height
         */
        getRenderHeight(useScreen?: boolean): number;
        /**
         * Gets the HTML canvas attached with the current WebGPU context
         * @returns a HTML canvas
         */
        getRenderingCanvas(): Nullable<HTMLCanvasElement>;
        /**
         * Get the current error code of the WebGPU context
         * @returns the error code
         */
        getError(): number;
        /**
         * @internal
         */
        bindSamplers(): void;
        /**
         * @internal
         */
        _bindTextureDirectly(): boolean;
        /**
         * Gets a boolean indicating if all created effects are ready
         * @returns always true - No parallel shader compilation
         */
        areAllEffectsReady(): boolean;
        /**
         * @internal
         */
        _executeWhenRenderingStateIsCompiled(pipelineContext: IPipelineContext, action: () => void): void;
        /**
         * @internal
         */
        _isRenderingStateCompiled(): boolean;
        /** @internal */
        _getUnpackAlignement(): number;
        /**
         * @internal
         */
        _unpackFlipY(): void;
        /**
         * @internal
         */
        _bindUnboundFramebuffer(): void;
        /**
         * @internal
         */
        _getSamplingParameters(): {
            min: number;
            mag: number;
        };
        /**
         * @internal
         */
        getUniforms(): Nullable<WebGLUniformLocation>[];
        /**
         * @internal
         */
        setIntArray(): boolean;
        /**
         * @internal
         */
        setIntArray2(): boolean;
        /**
         * @internal
         */
        setIntArray3(): boolean;
        /**
         * @internal
         */
        setIntArray4(): boolean;
        /**
         * @internal
         */
        setArray(): boolean;
        /**
         * @internal
         */
        setArray2(): boolean;
        /**
         * @internal
         */
        setArray3(): boolean;
        /**
         * @internal
         */
        setArray4(): boolean;
        /**
         * @internal
         */
        setMatrices(): boolean;
        /**
         * @internal
         */
        setMatrix3x3(): boolean;
        /**
         * @internal
         */
        setMatrix2x2(): boolean;
        /**
         * @internal
         */
        setFloat(): boolean;
        /**
         * @internal
         */
        setFloat2(): boolean;
        /**
         * @internal
         */
        setFloat3(): boolean;
        /**
         * @internal
         */
        setFloat4(): boolean;
    }


    /**
     * Gather the list of clipboard event types as constants.
     */
    export class ClipboardEventTypes {
        /**
         * The clipboard event is fired when a copy command is active (pressed).
         */
        static readonly COPY = 1;
        /**
         *  The clipboard event is fired when a cut command is active (pressed).
         */
        static readonly CUT = 2;
        /**
         * The clipboard event is fired when a paste command is active (pressed).
         */
        static readonly PASTE = 3;
    }
    /**
     * This class is used to store clipboard related info for the onClipboardObservable event.
     */
    export class ClipboardInfo {
        /**
         * Defines the type of event (BABYLON.ClipboardEventTypes)
         */
        type: number;
        /**
         * Defines the related dom event
         */
        event: ClipboardEvent;
        /**
         *Creates an instance of ClipboardInfo.
         * @param type Defines the type of event (BABYLON.ClipboardEventTypes)
         * @param event Defines the related dom event
         */
        constructor(
        /**
         * Defines the type of event (BABYLON.ClipboardEventTypes)
         */
        type: number, 
        /**
         * Defines the related dom event
         */
        event: ClipboardEvent);
        /**
         *  Get the clipboard event's type from the keycode.
         * @param keyCode Defines the keyCode for the current keyboard event.
         * @returns {number}
         */
        static GetTypeFromCharacter(keyCode: number): number;
    }


    /**
     * Event Types
     */
    export enum DeviceInputEventType {
        /** PointerMove */
        PointerMove = 0,
        /** PointerDown */
        PointerDown = 1,
        /** PointerUp */
        PointerUp = 2
    }
    /**
     * Native friendly interface for Event Object
     */
    export interface IUIEvent {
        /**
         * Input array index
         */
        inputIndex: number;
        /**
         * Current target for an event
         */
        currentTarget?: any;
        /**
         * Alias for target
         * @deprecated Use target instead
         */
        srcElement?: any;
        /**
         * Type of event
         */
        type: string;
        /**
         * Reference to object where object was dispatched
         */
        target: any;
        /**
         * Tells user agent what to do when not explicitly handled
         */
        preventDefault: () => void;
    }
    /**
     * Native friendly interface for KeyboardEvent Object
     */
    export interface IKeyboardEvent extends IUIEvent {
        /**
         * Status of Alt key being pressed
         */
        altKey: boolean;
        /**
         * Unicode value of character pressed
         * @deprecated Required for event, use keyCode instead.
         */
        charCode?: number;
        /**
         * Code for key based on layout
         */
        code: string;
        /**
         * Status of Ctrl key being pressed
         */
        ctrlKey: boolean;
        /**
         * String representation of key
         */
        key: string;
        /**
         * ASCII value of key
         * @deprecated Used with DeviceSourceManager
         */
        keyCode: number;
        /**
         * Status of Meta key (eg. Windows key) being pressed
         */
        metaKey: boolean;
        /**
         * Status of Shift key being pressed
         */
        shiftKey: boolean;
    }
    /**
     * Native friendly interface for MouseEvent Object
     */
    export interface IMouseEvent extends IUIEvent {
        /**
         * Subset of possible PointerInput values for events, excluding ones that CANNOT be in events organically
         */
        inputIndex: Exclude<PointerInput, PointerInput.Horizontal | PointerInput.Vertical>;
        /**
         * Status of Alt key being pressed
         */
        altKey: boolean;
        /**
         * Value of single mouse button pressed
         */
        button: number;
        /**
         * Value of all mouse buttons pressed
         */
        buttons: number;
        /**
         * Current X coordinate
         */
        clientX: number;
        /**
         * Current Y coordinate
         */
        clientY: number;
        /**
         * Status of Ctrl key being pressed
         */
        ctrlKey: boolean;
        /**
         * Provides current click count
         */
        detail?: number;
        /**
         * Status of Meta key (eg. Windows key) being pressed
         */
        metaKey: boolean;
        /**
         * Delta of movement on X axis
         */
        movementX: number;
        /**
         * Delta of movement on Y axis
         */
        movementY: number;
        /**
         * Delta of movement on X axis
         * @deprecated Use 'movementX' instead
         */
        mozMovementX?: number;
        /**
         * Delta of movement on Y axis
         * @deprecated Use 'movementY' instead
         */
        mozMovementY?: number;
        /**
         * Delta of movement on X axis
         * @deprecated Use 'movementX' instead
         */
        msMovementX?: number;
        /**
         * Delta of movement on Y axis
         * @deprecated Use 'movementY' instead
         */
        msMovementY?: number;
        /**
         * Current coordinate of X within container
         */
        offsetX: number;
        /**
         * Current coordinate of Y within container
         */
        offsetY: number;
        /**
         * Horizontal coordinate of event
         */
        pageX: number;
        /**
         * Vertical coordinate of event
         */
        pageY: number;
        /**
         * Status of Shift key being pressed
         */
        shiftKey: boolean;
        /**
         * Delta of movement on X axis
         * @deprecated Use 'movementX' instead
         */
        webkitMovementX?: number;
        /**
         * Delta of movement on Y axis
         * @deprecated Use 'movementY' instead
         */
        webkitMovementY?: number;
        /**
         * Alias of clientX
         */
        x: number;
        /**
         * Alias of clientY
         */
        y: number;
    }
    /**
     * Native friendly interface for PointerEvent Object
     */
    export interface IPointerEvent extends IMouseEvent {
        /**
         * Subset of possible PointerInput values for events, excluding ones that CANNOT be in events organically and mouse wheel values
         */
        inputIndex: Exclude<PointerInput, PointerInput.Horizontal | PointerInput.Vertical | PointerInput.MouseWheelX | PointerInput.MouseWheelY | PointerInput.MouseWheelZ>;
        /**
         * Pointer Event ID
         */
        pointerId: number;
        /**
         * Type of pointer
         */
        pointerType: string;
    }
    /**
     * Native friendly interface for WheelEvent Object
     */
    export interface IWheelEvent extends IMouseEvent {
        /**
         * Subset of possible PointerInput values for events that can only be used with mouse wheel
         */
        inputIndex: PointerInput.MouseWheelX | PointerInput.MouseWheelY | PointerInput.MouseWheelZ;
        /**
         * Units for delta value
         */
        deltaMode: number;
        /**
         * Horizontal scroll delta
         */
        deltaX: number;
        /**
         * Vertical scroll delta
         */
        deltaY: number;
        /**
         * Z-Axis scroll delta
         */
        deltaZ: number;
        /**
         * WheelDelta (From MouseWheel Event)
         * @deprecated
         */
        wheelDelta?: number;
    }
    /**
     * Constants used for Events
     */
    export class EventConstants {
        /**
         * Pixel delta for Wheel Events (Default)
         */
        static DOM_DELTA_PIXEL: number;
        /**
         * Line delta for Wheel Events
         */
        static DOM_DELTA_LINE: number;
        /**
         * Page delta for Wheel Events
         */
        static DOM_DELTA_PAGE: number;
    }




    /**
     * Gather the list of keyboard event types as constants.
     */
    export class KeyboardEventTypes {
        /**
         * The keydown event is fired when a key becomes active (pressed).
         */
        static readonly KEYDOWN = 1;
        /**
         * The keyup event is fired when a key has been released.
         */
        static readonly KEYUP = 2;
    }
    /**
     * This class is used to store keyboard related info for the onKeyboardObservable event.
     */
    export class KeyboardInfo {
        /**
         * Defines the type of event (KeyboardEventTypes)
         */
        type: number;
        /**
         * Defines the related dom event
         */
        event: IKeyboardEvent;
        /**
         * Instantiates a new keyboard info.
         * This class is used to store keyboard related info for the onKeyboardObservable event.
         * @param type Defines the type of event (KeyboardEventTypes)
         * @param event Defines the related dom event
         */
        constructor(
        /**
         * Defines the type of event (KeyboardEventTypes)
         */
        type: number, 
        /**
         * Defines the related dom event
         */
        event: IKeyboardEvent);
    }
    /**
     * This class is used to store keyboard related info for the onPreKeyboardObservable event.
     * Set the skipOnKeyboardObservable property to true if you want the engine to stop any process after this event is triggered, even not calling onKeyboardObservable
     */
    export class KeyboardInfoPre extends KeyboardInfo {
        /**
         * Defines the type of event (KeyboardEventTypes)
         */
        type: number;
        /**
         * Defines the related dom event
         */
        event: IKeyboardEvent;
        /**
         * Defines whether the engine should skip the next onKeyboardObservable associated to this pre.
         */
        skipOnKeyboardObservable: boolean;
        /**
         * Defines whether the engine should skip the next onKeyboardObservable associated to this pre.
         * @deprecated use skipOnKeyboardObservable property instead
         */
        get skipOnPointerObservable(): boolean;
        set skipOnPointerObservable(value: boolean);
        /**
         * Instantiates a new keyboard pre info.
         * This class is used to store keyboard related info for the onPreKeyboardObservable event.
         * @param type Defines the type of event (KeyboardEventTypes)
         * @param event Defines the related dom event
         */
        constructor(
        /**
         * Defines the type of event (KeyboardEventTypes)
         */
        type: number, 
        /**
         * Defines the related dom event
         */
        event: IKeyboardEvent);
    }


    /**
     * Gather the list of pointer event types as constants.
     */
    export class PointerEventTypes {
        /**
         * The pointerdown event is fired when a pointer becomes active. For mouse, it is fired when the device transitions from no buttons depressed to at least one button depressed. For touch, it is fired when physical contact is made with the digitizer. For pen, it is fired when the stylus makes physical contact with the digitizer.
         */
        static readonly POINTERDOWN = 1;
        /**
         * The pointerup event is fired when a pointer is no longer active.
         */
        static readonly POINTERUP = 2;
        /**
         * The pointermove event is fired when a pointer changes coordinates.
         */
        static readonly POINTERMOVE = 4;
        /**
         * The pointerwheel event is fired when a mouse wheel has been rotated.
         */
        static readonly POINTERWHEEL = 8;
        /**
         * The pointerpick event is fired when a mesh or sprite has been picked by the pointer.
         */
        static readonly POINTERPICK = 16;
        /**
         * The pointertap event is fired when a the object has been touched and released without drag.
         */
        static readonly POINTERTAP = 32;
        /**
         * The pointerdoubletap event is fired when a the object has been touched and released twice without drag.
         */
        static readonly POINTERDOUBLETAP = 64;
    }
    /**
     * Base class of pointer info types.
     */
    export class PointerInfoBase {
        /**
         * Defines the type of event (PointerEventTypes)
         */
        type: number;
        /**
         * Defines the related dom event
         */
        event: IMouseEvent;
        /**
         * Instantiates the base class of pointers info.
         * @param type Defines the type of event (PointerEventTypes)
         * @param event Defines the related dom event
         */
        constructor(
        /**
         * Defines the type of event (PointerEventTypes)
         */
        type: number, 
        /**
         * Defines the related dom event
         */
        event: IMouseEvent);
    }
    /**
     * This class is used to store pointer related info for the onPrePointerObservable event.
     * Set the skipOnPointerObservable property to true if you want the engine to stop any process after this event is triggered, even not calling onPointerObservable
     */
    export class PointerInfoPre extends PointerInfoBase {
        /**
         * Ray from a pointer if available (eg. 6dof controller)
         */
        ray: Nullable<Ray>;
        /**
         * Defines picking info coming from a near interaction (proximity instead of ray-based picking)
         */
        nearInteractionPickingInfo: Nullable<PickingInfo>;
        /**
         * The original picking info that was used to trigger the pointer event
         */
        originalPickingInfo: Nullable<PickingInfo>;
        /**
         * Defines the local position of the pointer on the canvas.
         */
        localPosition: Vector2;
        /**
         * Defines whether the engine should skip the next OnPointerObservable associated to this pre.
         */
        skipOnPointerObservable: boolean;
        /**
         * Instantiates a PointerInfoPre to store pointer related info to the onPrePointerObservable event.
         * @param type Defines the type of event (PointerEventTypes)
         * @param event Defines the related dom event
         * @param localX Defines the local x coordinates of the pointer when the event occured
         * @param localY Defines the local y coordinates of the pointer when the event occured
         */
        constructor(type: number, event: IMouseEvent, localX: number, localY: number);
    }
    /**
     * This type contains all the data related to a pointer event in Babylon.js.
     * The event member is an instance of PointerEvent for all types except PointerWheel and is of type MouseWheelEvent when type equals PointerWheel. The different event types can be found in the PointerEventTypes class.
     */
    export class PointerInfo extends PointerInfoBase {
        private _pickInfo;
        private _inputManager;
        /**
         * Defines the picking info associated with this PointerInfo object (if applicable)
         */
        get pickInfo(): Nullable<PickingInfo>;
        /**
         * Instantiates a PointerInfo to store pointer related info to the onPointerObservable event.
         * @param type Defines the type of event (PointerEventTypes)
         * @param event Defines the related dom event
         * @param pickInfo Defines the picking info associated to the info (if any)
         * @param inputManager Defines the InputManager to use if there is no pickInfo
         */
        constructor(type: number, event: IMouseEvent, pickInfo: Nullable<PickingInfo>, inputManager?: Nullable<InputManager>);
        /**
         * Generates the picking info if needed
         */
        /** @internal */
        _generatePickInfo(): void;
    }
    /**
     * Data relating to a touch event on the screen.
     */
    export interface PointerTouch {
        /**
         * X coordinate of touch.
         */
        x: number;
        /**
         * Y coordinate of touch.
         */
        y: number;
        /**
         * Id of touch. Unique for each finger.
         */
        pointerId: number;
        /**
         * Event type passed from DOM.
         */
        type: any;
    }


    /**
     * Google Daydream controller
     */
    export class DaydreamController extends WebVRController {
        /**
         * Base Url for the controller model.
         */
        static MODEL_BASE_URL: string;
        /**
         * File name for the controller model.
         */
        static MODEL_FILENAME: string;
        /**
         * Gamepad Id prefix used to identify Daydream Controller.
         */
        static readonly GAMEPAD_ID_PREFIX: string;
        /**
         * Creates a new DaydreamController from a gamepad
         * @param vrGamepad the gamepad that the controller should be created from
         */
        constructor(vrGamepad: any);
        /**
         * Implements abstract method on WebVRController class, loading controller meshes and calling this.attachToMesh if successful.
         * @param scene scene in which to add meshes
         * @param meshLoaded optional callback function that will be called if the mesh loads successfully.
         */
        initControllerMesh(scene: Scene, meshLoaded?: (mesh: AbstractMesh) => void): void;
        /**
         * Called once for each button that changed state since the last frame
         * @param buttonIdx Which button index changed
         * @param state New state of the button
         */
        protected _handleButtonChange(buttonIdx: number, state: ExtendedGamepadButton): void;
    }


    /**
     * Gear VR Controller
     */
    export class GearVRController extends WebVRController {
        /**
         * Base Url for the controller model.
         */
        static MODEL_BASE_URL: string;
        /**
         * File name for the controller model.
         */
        static MODEL_FILENAME: string;
        /**
         * Gamepad Id prefix used to identify this controller.
         */
        static readonly GAMEPAD_ID_PREFIX: string;
        private readonly _buttonIndexToObservableNameMap;
        /**
         * Creates a new GearVRController from a gamepad
         * @param vrGamepad the gamepad that the controller should be created from
         */
        constructor(vrGamepad: any);
        /**
         * Implements abstract method on WebVRController class, loading controller meshes and calling this.attachToMesh if successful.
         * @param scene scene in which to add meshes
         * @param meshLoaded optional callback function that will be called if the mesh loads successfully.
         */
        initControllerMesh(scene: Scene, meshLoaded?: (mesh: AbstractMesh) => void): void;
        /**
         * Called once for each button that changed state since the last frame
         * @param buttonIdx Which button index changed
         * @param state New state of the button
         */
        protected _handleButtonChange(buttonIdx: number, state: ExtendedGamepadButton): void;
    }


    /**
     * Generic Controller
     */
    export class GenericController extends WebVRController {
        /**
         * Base Url for the controller model.
         */
        static readonly MODEL_BASE_URL: string;
        /**
         * File name for the controller model.
         */
        static readonly MODEL_FILENAME: string;
        /**
         * Creates a new GenericController from a gamepad
         * @param vrGamepad the gamepad that the controller should be created from
         */
        constructor(vrGamepad: any);
        /**
         * Implements abstract method on WebVRController class, loading controller meshes and calling this.attachToMesh if successful.
         * @param scene scene in which to add meshes
         * @param meshLoaded optional callback function that will be called if the mesh loads successfully.
         */
        initControllerMesh(scene: Scene, meshLoaded?: (mesh: AbstractMesh) => void): void;
        /**
         * Called once for each button that changed state since the last frame
         * @param buttonIdx Which button index changed
         * @param state New state of the button
         */
        protected _handleButtonChange(buttonIdx: number, state: ExtendedGamepadButton): void;
    }




    /**
     * Oculus Touch Controller
     */
    export class OculusTouchController extends WebVRController {
        /**
         * Base Url for the controller model.
         */
        static MODEL_BASE_URL: string;
        /**
         * File name for the left controller model.
         */
        static MODEL_LEFT_FILENAME: string;
        /**
         * File name for the right controller model.
         */
        static MODEL_RIGHT_FILENAME: string;
        /**
         * Base Url for the Quest controller model.
         */
        static QUEST_MODEL_BASE_URL: string;
        /**
         * @internal
         * If the controllers are running on a device that needs the updated Quest controller models
         */
        static _IsQuest: boolean;
        /**
         * Fired when the secondary trigger on this controller is modified
         */
        onSecondaryTriggerStateChangedObservable: Observable<ExtendedGamepadButton>;
        /**
         * Fired when the thumb rest on this controller is modified
         */
        onThumbRestChangedObservable: Observable<ExtendedGamepadButton>;
        /**
         * Creates a new OculusTouchController from a gamepad
         * @param vrGamepad the gamepad that the controller should be created from
         */
        constructor(vrGamepad: any);
        /**
         * Implements abstract method on WebVRController class, loading controller meshes and calling this.attachToMesh if successful.
         * @param scene scene in which to add meshes
         * @param meshLoaded optional callback function that will be called if the mesh loads successfully.
         */
        initControllerMesh(scene: Scene, meshLoaded?: (mesh: AbstractMesh) => void): void;
        /**
         * Fired when the A button on this controller is modified
         */
        get onAButtonStateChangedObservable(): Observable<ExtendedGamepadButton>;
        /**
         * Fired when the B button on this controller is modified
         */
        get onBButtonStateChangedObservable(): Observable<ExtendedGamepadButton>;
        /**
         * Fired when the X button on this controller is modified
         */
        get onXButtonStateChangedObservable(): Observable<ExtendedGamepadButton>;
        /**
         * Fired when the Y button on this controller is modified
         */
        get onYButtonStateChangedObservable(): Observable<ExtendedGamepadButton>;
        /**
         * Called once for each button that changed state since the last frame
         * 0) thumb stick (touch, press, value = pressed (0,1)). value is in this.leftStick
         * 1) index trigger (touch (?), press (only when value > 0.1), value 0 to 1)
         * 2) secondary trigger (same)
         * 3) A (right) X (left), touch, pressed = value
         * 4) B / Y
         * 5) thumb rest
         * @param buttonIdx Which button index changed
         * @param state New state of the button
         */
        protected _handleButtonChange(buttonIdx: number, state: ExtendedGamepadButton): void;
    }


    /**
     * Defines the types of pose enabled controllers that are supported
     */
    export enum PoseEnabledControllerType {
        /**
         * HTC Vive
         */
        VIVE = 0,
        /**
         * Oculus Rift
         */
        OCULUS = 1,
        /**
         * Windows mixed reality
         */
        WINDOWS = 2,
        /**
         * Samsung gear VR
         */
        GEAR_VR = 3,
        /**
         * Google Daydream
         */
        DAYDREAM = 4,
        /**
         * Generic
         */
        GENERIC = 5
    }
    /**
     * Defines the MutableGamepadButton interface for the state of a gamepad button
     */
    export interface MutableGamepadButton {
        /**
         * Value of the button/trigger
         */
        value: number;
        /**
         * If the button/trigger is currently touched
         */
        touched: boolean;
        /**
         * If the button/trigger is currently pressed
         */
        pressed: boolean;
    }
    /**
     * Defines the ExtendedGamepadButton interface for a gamepad button which includes state provided by a pose controller
     * @internal
     */
    export interface ExtendedGamepadButton extends GamepadButton {
        /**
         * If the button/trigger is currently pressed
         */
        readonly pressed: boolean;
        /**
         * If the button/trigger is currently touched
         */
        readonly touched: boolean;
        /**
         * Value of the button/trigger
         */
        readonly value: number;
    }
    /** @internal */
    export interface _GamePadFactory {
        /**
         * Returns whether or not the current gamepad can be created for this type of controller.
         * @param gamepadInfo Defines the gamepad info as received from the controller APIs.
         * @returns true if it can be created, otherwise false
         */
        canCreate(gamepadInfo: any): boolean;
        /**
         * Creates a new instance of the Gamepad.
         * @param gamepadInfo Defines the gamepad info as received from the controller APIs.
         * @returns the new gamepad instance
         */
        create(gamepadInfo: any): Gamepad;
    }
    /**
     * Defines the PoseEnabledControllerHelper object that is used initialize a gamepad as the controller type it is specified as (eg. windows mixed reality controller)
     */
    export class PoseEnabledControllerHelper {
        /** @internal */
        static _ControllerFactories: _GamePadFactory[];
        /** @internal */
        static _DefaultControllerFactory: Nullable<(gamepadInfo: any) => Gamepad>;
        /**
         * Initializes a gamepad as the controller type it is specified as (eg. windows mixed reality controller)
         * @param vrGamepad the gamepad to initialized
         * @returns a vr controller of the type the gamepad identified as
         */
        static InitiateController(vrGamepad: any): Gamepad;
    }
    /**
     * Defines the PoseEnabledController object that contains state of a vr capable controller
     */
    export class PoseEnabledController extends Gamepad implements PoseControlled {
        /**
         * If the controller is used in a webXR session
         */
        isXR: boolean;
        private _deviceRoomPosition;
        private _deviceRoomRotationQuaternion;
        /**
         * The device position in babylon space
         */
        devicePosition: Vector3;
        /**
         * The device rotation in babylon space
         */
        deviceRotationQuaternion: Quaternion;
        /**
         * The scale factor of the device in babylon space
         */
        deviceScaleFactor: number;
        /**
         * (Likely devicePosition should be used instead) The device position in its room space
         */
        position: Vector3;
        /**
         * (Likely deviceRotationQuaternion should be used instead) The device rotation in its room space
         */
        rotationQuaternion: Quaternion;
        /**
         * The type of controller (Eg. Windows mixed reality)
         */
        controllerType: PoseEnabledControllerType;
        protected _calculatedPosition: Vector3;
        private _calculatedRotation;
        /**
         * The raw pose from the device
         */
        rawPose: DevicePose;
        private _trackPosition;
        private _maxRotationDistFromHeadset;
        private _draggedRoomRotation;
        /**
         * @internal
         */
        _disableTrackPosition(fixedPosition: Vector3): void;
        /**
         * Internal, the mesh attached to the controller
         * @internal
         */
        _mesh: Nullable<AbstractMesh>;
        private _poseControlledCamera;
        private _leftHandSystemQuaternion;
        /**
         * Internal, matrix used to convert room space to babylon space
         * @internal
         */
        _deviceToWorld: Matrix;
        /**
         * Node to be used when casting a ray from the controller
         * @internal
         */
        _pointingPoseNode: Nullable<TransformNode>;
        /**
         * Name of the child mesh that can be used to cast a ray from the controller
         */
        static readonly POINTING_POSE = "POINTING_POSE";
        /**
         * Creates a new PoseEnabledController from a gamepad
         * @param browserGamepad the gamepad that the PoseEnabledController should be created from
         */
        constructor(browserGamepad: any);
        private _workingMatrix;
        /**
         * Updates the state of the pose enabled controller and mesh based on the current position and rotation of the controller
         */
        update(): void;
        /**
         * Updates only the pose device and mesh without doing any button event checking
         */
        protected _updatePoseAndMesh(): void;
        /**
         * Updates the state of the pose enbaled controller based on the raw pose data from the device
         * @param poseData raw pose fromthe device
         */
        updateFromDevice(poseData: DevicePose): void;
        /**
         * @internal
         */
        _meshAttachedObservable: Observable<AbstractMesh>;
        /**
         * Attaches a mesh to the controller
         * @param mesh the mesh to be attached
         */
        attachToMesh(mesh: AbstractMesh): void;
        /**
         * Attaches the controllers mesh to a camera
         * @param camera the camera the mesh should be attached to
         */
        attachToPoseControlledCamera(camera: TargetCamera): void;
        /**
         * Disposes of the controller
         */
        dispose(): void;
        /**
         * The mesh that is attached to the controller
         */
        get mesh(): Nullable<AbstractMesh>;
        /**
         * Gets the ray of the controller in the direction the controller is pointing
         * @param length the length the resulting ray should be
         * @returns a ray in the direction the controller is pointing
         */
        getForwardRay(length?: number): Ray;
    }


    /**
     * Vive Controller
     */
    export class ViveController extends WebVRController {
        /**
         * Base Url for the controller model.
         */
        static MODEL_BASE_URL: string;
        /**
         * File name for the controller model.
         */
        static MODEL_FILENAME: string;
        /**
         * Creates a new ViveController from a gamepad
         * @param vrGamepad the gamepad that the controller should be created from
         */
        constructor(vrGamepad: any);
        /**
         * Implements abstract method on WebVRController class, loading controller meshes and calling this.attachToMesh if successful.
         * @param scene scene in which to add meshes
         * @param meshLoaded optional callback function that will be called if the mesh loads successfully.
         */
        initControllerMesh(scene: Scene, meshLoaded?: (mesh: AbstractMesh) => void): void;
        /**
         * Fired when the left button on this controller is modified
         */
        get onLeftButtonStateChangedObservable(): Observable<ExtendedGamepadButton>;
        /**
         * Fired when the right button on this controller is modified
         */
        get onRightButtonStateChangedObservable(): Observable<ExtendedGamepadButton>;
        /**
         * Fired when the menu button on this controller is modified
         */
        get onMenuButtonStateChangedObservable(): Observable<ExtendedGamepadButton>;
        /**
         * Called once for each button that changed state since the last frame
         * Vive mapping:
         * 0: touchpad
         * 1: trigger
         * 2: left AND right buttons
         * 3: menu button
         * @param buttonIdx Which button index changed
         * @param state New state of the button
         */
        protected _handleButtonChange(buttonIdx: number, state: ExtendedGamepadButton): void;
    }


    /**
     * Defines the WebVRController object that represents controllers tracked in 3D space
     * @deprecated Use WebXR instead
     */
    export abstract class WebVRController extends PoseEnabledController {
        /**
         * Internal, the default controller model for the controller
         */
        protected _defaultModel: Nullable<AbstractMesh>;
        /**
         * Fired when the trigger state has changed
         */
        onTriggerStateChangedObservable: Observable<ExtendedGamepadButton>;
        /**
         * Fired when the main button state has changed
         */
        onMainButtonStateChangedObservable: Observable<ExtendedGamepadButton>;
        /**
         * Fired when the secondary button state has changed
         */
        onSecondaryButtonStateChangedObservable: Observable<ExtendedGamepadButton>;
        /**
         * Fired when the pad state has changed
         */
        onPadStateChangedObservable: Observable<ExtendedGamepadButton>;
        /**
         * Fired when controllers stick values have changed
         */
        onPadValuesChangedObservable: Observable<StickValues>;
        /**
         * Array of button available on the controller
         */
        protected _buttons: Array<MutableGamepadButton>;
        private _onButtonStateChange;
        /**
         * Fired when a controller button's state has changed
         * @param callback the callback containing the button that was modified
         */
        onButtonStateChange(callback: (controlledIndex: number, buttonIndex: number, state: ExtendedGamepadButton) => void): void;
        /**
         * X and Y axis corresponding to the controllers joystick
         */
        pad: StickValues;
        /**
         * 'left' or 'right', see https://w3c.github.io/gamepad/extensions.html#gamepadhand-enum
         */
        hand: string;
        /**
         * The default controller model for the controller
         */
        get defaultModel(): Nullable<AbstractMesh>;
        /**
         * Creates a new WebVRController from a gamepad
         * @param vrGamepad the gamepad that the WebVRController should be created from
         */
        constructor(vrGamepad: any);
        /**
         * Updates the state of the controller and mesh based on the current position and rotation of the controller
         */
        update(): void;
        /**
         * Function to be called when a button is modified
         */
        protected abstract _handleButtonChange(buttonIdx: number, value: ExtendedGamepadButton, changes: GamepadButtonChanges): void;
        /**
         * Loads a mesh and attaches it to the controller
         * @param scene the scene the mesh should be added to
         * @param meshLoaded callback for when the mesh has been loaded
         */
        abstract initControllerMesh(scene: Scene, meshLoaded?: (mesh: AbstractMesh) => void): void;
        private _setButtonValue;
        private _changes;
        private _checkChanges;
        /**
         * Disposes of th webVRController
         */
        dispose(): void;
    }


    /**
     * Defines the WindowsMotionController object that the state of the windows motion controller
     */
    export class WindowsMotionController extends WebVRController {
        /**
         * The base url used to load the left and right controller models
         */
        static MODEL_BASE_URL: string;
        /**
         * The name of the left controller model file
         */
        static MODEL_LEFT_FILENAME: string;
        /**
         * The name of the right controller model file
         */
        static MODEL_RIGHT_FILENAME: string;
        /**
         * The controller name prefix for this controller type
         */
        static readonly GAMEPAD_ID_PREFIX: string;
        /**
         * The controller id pattern for this controller type
         */
        private static readonly GAMEPAD_ID_PATTERN;
        private _loadedMeshInfo;
        protected readonly _mapping: {
            buttons: string[];
            buttonMeshNames: {
                trigger: string;
                menu: string;
                grip: string;
                thumbstick: string;
                trackpad: string;
            };
            buttonObservableNames: {
                trigger: string;
                menu: string;
                grip: string;
                thumbstick: string;
                trackpad: string;
            };
            axisMeshNames: string[];
            pointingPoseMeshName: string;
        };
        /**
         * Fired when the trackpad on this controller is clicked
         */
        onTrackpadChangedObservable: Observable<ExtendedGamepadButton>;
        /**
         * Fired when the trackpad on this controller is modified
         */
        onTrackpadValuesChangedObservable: Observable<StickValues>;
        /**
         * The current x and y values of this controller's trackpad
         */
        trackpad: StickValues;
        /**
         * Creates a new WindowsMotionController from a gamepad
         * @param vrGamepad the gamepad that the controller should be created from
         */
        constructor(vrGamepad: any);
        /**
         * Fired when the trigger on this controller is modified
         */
        get onTriggerButtonStateChangedObservable(): Observable<ExtendedGamepadButton>;
        /**
         * Fired when the menu button on this controller is modified
         */
        get onMenuButtonStateChangedObservable(): Observable<ExtendedGamepadButton>;
        /**
         * Fired when the grip button on this controller is modified
         */
        get onGripButtonStateChangedObservable(): Observable<ExtendedGamepadButton>;
        /**
         * Fired when the thumbstick button on this controller is modified
         */
        get onThumbstickButtonStateChangedObservable(): Observable<ExtendedGamepadButton>;
        /**
         * Fired when the touchpad button on this controller is modified
         */
        get onTouchpadButtonStateChangedObservable(): Observable<ExtendedGamepadButton>;
        /**
         * Fired when the touchpad values on this controller are modified
         */
        get onTouchpadValuesChangedObservable(): Observable<StickValues>;
        protected _updateTrackpad(): void;
        /**
         * Called once per frame by the engine.
         */
        update(): void;
        /**
         * Called once for each button that changed state since the last frame
         * @param buttonIdx Which button index changed
         * @param state New state of the button
         */
        protected _handleButtonChange(buttonIdx: number, state: ExtendedGamepadButton): void;
        /**
         * Moves the buttons on the controller mesh based on their current state
         * @param buttonName the name of the button to move
         * @param buttonValue the value of the button which determines the buttons new position
         */
        protected _lerpButtonTransform(buttonName: string, buttonValue: number): void;
        /**
         * Moves the axis on the controller mesh based on its current state
         * @param axis the index of the axis
         * @param axisValue the value of the axis which determines the meshes new position
         * @internal
         */
        protected _lerpAxisTransform(axis: number, axisValue: number): void;
        /**
         * Implements abstract method on WebVRController class, loading controller meshes and calling this.attachToMesh if successful.
         * @param scene scene in which to add meshes
         * @param meshLoaded optional callback function that will be called if the mesh loads successfully.
         * @param forceDefault
         */
        initControllerMesh(scene: Scene, meshLoaded?: (mesh: AbstractMesh) => void, forceDefault?: boolean): void;
        /**
         * Takes a list of meshes (as loaded from the glTF file) and finds the root node, as well as nodes that
         * can be transformed by button presses and axes values, based on this._mapping.
         *
         * @param scene scene in which the meshes exist
         * @param meshes list of meshes that make up the controller model to process
         * @returns structured view of the given meshes, with mapping of buttons and axes to meshes that can be transformed.
         */
        private _processModel;
        private _createMeshInfo;
        /**
         * Gets the ray of the controller in the direction the controller is pointing
         * @param length the length the resulting ray should be
         * @returns a ray in the direction the controller is pointing
         */
        getForwardRay(length?: number): Ray;
        /**
         * Disposes of the controller
         */
        dispose(): void;
    }
    /**
     * This class represents a new windows motion controller in XR.
     */
    export class XRWindowsMotionController extends WindowsMotionController {
        /**
         * Changing the original WIndowsMotionController mapping to fir the new mapping
         */
        protected readonly _mapping: {
            buttons: string[];
            buttonMeshNames: {
                trigger: string;
                menu: string;
                grip: string;
                thumbstick: string;
                trackpad: string;
            };
            buttonObservableNames: {
                trigger: string;
                menu: string;
                grip: string;
                thumbstick: string;
                trackpad: string;
            };
            axisMeshNames: string[];
            pointingPoseMeshName: string;
        };
        /**
         * Construct a new XR-Based windows motion controller
         *
         * @param gamepadInfo the gamepad object from the browser
         */
        constructor(gamepadInfo: any);
        /**
         * holds the thumbstick values (X,Y)
         */
        thumbstickValues: StickValues;
        /**
         * Fired when the thumbstick on this controller is clicked
         */
        onThumbstickStateChangedObservable: Observable<ExtendedGamepadButton>;
        /**
         * Fired when the thumbstick on this controller is modified
         */
        onThumbstickValuesChangedObservable: Observable<StickValues>;
        /**
         * Fired when the touchpad button on this controller is modified
         */
        onTrackpadChangedObservable: Observable<ExtendedGamepadButton>;
        /**
         * Fired when the touchpad values on this controller are modified
         */
        onTrackpadValuesChangedObservable: Observable<StickValues>;
        /**
         * Fired when the thumbstick button on this controller is modified
         * here to prevent breaking changes
         */
        get onThumbstickButtonStateChangedObservable(): Observable<ExtendedGamepadButton>;
        /**
         * updating the thumbstick(!) and not the trackpad.
         * This is named this way due to the difference between WebVR and XR and to avoid
         * changing the parent class.
         */
        protected _updateTrackpad(): void;
        /**
         * Disposes the class with joy
         */
        dispose(): void;
    }


    /**
     * Defines supported buttons for DualShock compatible gamepads
     */
    export enum DualShockButton {
        /** Cross */
        Cross = 0,
        /** Circle */
        Circle = 1,
        /** Square */
        Square = 2,
        /** Triangle */
        Triangle = 3,
        /** L1 */
        L1 = 4,
        /** R1 */
        R1 = 5,
        /** Share */
        Share = 8,
        /** Options */
        Options = 9,
        /** Left stick */
        LeftStick = 10,
        /** Right stick */
        RightStick = 11
    }
    /** Defines values for DualShock DPad  */
    export enum DualShockDpad {
        /** Up */
        Up = 12,
        /** Down */
        Down = 13,
        /** Left */
        Left = 14,
        /** Right */
        Right = 15
    }
    /**
     * Defines a DualShock gamepad
     */
    export class DualShockPad extends Gamepad {
        private _leftTrigger;
        private _rightTrigger;
        private _onlefttriggerchanged;
        private _onrighttriggerchanged;
        private _onbuttondown;
        private _onbuttonup;
        private _ondpaddown;
        private _ondpadup;
        /** Observable raised when a button is pressed */
        onButtonDownObservable: Observable<DualShockButton>;
        /** Observable raised when a button is released */
        onButtonUpObservable: Observable<DualShockButton>;
        /** Observable raised when a pad is pressed */
        onPadDownObservable: Observable<DualShockDpad>;
        /** Observable raised when a pad is released */
        onPadUpObservable: Observable<DualShockDpad>;
        private _buttonCross;
        private _buttonCircle;
        private _buttonSquare;
        private _buttonTriangle;
        private _buttonShare;
        private _buttonOptions;
        private _buttonL1;
        private _buttonR1;
        private _buttonLeftStick;
        private _buttonRightStick;
        private _dPadUp;
        private _dPadDown;
        private _dPadLeft;
        private _dPadRight;
        /**
         * Creates a new DualShock gamepad object
         * @param id defines the id of this gamepad
         * @param index defines its index
         * @param gamepad defines the internal HTML gamepad object
         */
        constructor(id: string, index: number, gamepad: any);
        /**
         * Defines the callback to call when left trigger is pressed
         * @param callback defines the callback to use
         */
        onlefttriggerchanged(callback: (value: number) => void): void;
        /**
         * Defines the callback to call when right trigger is pressed
         * @param callback defines the callback to use
         */
        onrighttriggerchanged(callback: (value: number) => void): void;
        /**
         * Gets the left trigger value
         */
        get leftTrigger(): number;
        /**
         * Sets the left trigger value
         */
        set leftTrigger(newValue: number);
        /**
         * Gets the right trigger value
         */
        get rightTrigger(): number;
        /**
         * Sets the right trigger value
         */
        set rightTrigger(newValue: number);
        /**
         * Defines the callback to call when a button is pressed
         * @param callback defines the callback to use
         */
        onbuttondown(callback: (buttonPressed: DualShockButton) => void): void;
        /**
         * Defines the callback to call when a button is released
         * @param callback defines the callback to use
         */
        onbuttonup(callback: (buttonReleased: DualShockButton) => void): void;
        /**
         * Defines the callback to call when a pad is pressed
         * @param callback defines the callback to use
         */
        ondpaddown(callback: (dPadPressed: DualShockDpad) => void): void;
        /**
         * Defines the callback to call when a pad is released
         * @param callback defines the callback to use
         */
        ondpadup(callback: (dPadReleased: DualShockDpad) => void): void;
        private _setButtonValue;
        private _setDPadValue;
        /**
         * Gets the value of the `Cross` button
         */
        get buttonCross(): number;
        /**
         * Sets the value of the `Cross` button
         */
        set buttonCross(value: number);
        /**
         * Gets the value of the `Circle` button
         */
        get buttonCircle(): number;
        /**
         * Sets the value of the `Circle` button
         */
        set buttonCircle(value: number);
        /**
         * Gets the value of the `Square` button
         */
        get buttonSquare(): number;
        /**
         * Sets the value of the `Square` button
         */
        set buttonSquare(value: number);
        /**
         * Gets the value of the `Triangle` button
         */
        get buttonTriangle(): number;
        /**
         * Sets the value of the `Triangle` button
         */
        set buttonTriangle(value: number);
        /**
         * Gets the value of the `Options` button
         */
        get buttonOptions(): number;
        /**
         * Sets the value of the `Options` button
         */
        set buttonOptions(value: number);
        /**
         * Gets the value of the `Share` button
         */
        get buttonShare(): number;
        /**
         * Sets the value of the `Share` button
         */
        set buttonShare(value: number);
        /**
         * Gets the value of the `L1` button
         */
        get buttonL1(): number;
        /**
         * Sets the value of the `L1` button
         */
        set buttonL1(value: number);
        /**
         * Gets the value of the `R1` button
         */
        get buttonR1(): number;
        /**
         * Sets the value of the `R1` button
         */
        set buttonR1(value: number);
        /**
         * Gets the value of the Left joystick
         */
        get buttonLeftStick(): number;
        /**
         * Sets the value of the Left joystick
         */
        set buttonLeftStick(value: number);
        /**
         * Gets the value of the Right joystick
         */
        get buttonRightStick(): number;
        /**
         * Sets the value of the Right joystick
         */
        set buttonRightStick(value: number);
        /**
         * Gets the value of D-pad up
         */
        get dPadUp(): number;
        /**
         * Sets the value of D-pad up
         */
        set dPadUp(value: number);
        /**
         * Gets the value of D-pad down
         */
        get dPadDown(): number;
        /**
         * Sets the value of D-pad down
         */
        set dPadDown(value: number);
        /**
         * Gets the value of D-pad left
         */
        get dPadLeft(): number;
        /**
         * Sets the value of D-pad left
         */
        set dPadLeft(value: number);
        /**
         * Gets the value of D-pad right
         */
        get dPadRight(): number;
        /**
         * Sets the value of D-pad right
         */
        set dPadRight(value: number);
        /**
         * Force the gamepad to synchronize with device values
         */
        update(): void;
        /**
         * Disposes the gamepad
         */
        dispose(): void;
    }


    /**
     * Represents a gamepad control stick position
     */
    export class StickValues {
        /**
         * The x component of the control stick
         */
        x: number;
        /**
         * The y component of the control stick
         */
        y: number;
        /**
         * Initializes the gamepad x and y control stick values
         * @param x The x component of the gamepad control stick value
         * @param y The y component of the gamepad control stick value
         */
        constructor(
        /**
         * The x component of the control stick
         */
        x: number, 
        /**
         * The y component of the control stick
         */
        y: number);
    }
    /**
     * An interface which manages callbacks for gamepad button changes
     */
    export interface GamepadButtonChanges {
        /**
         * Called when a gamepad has been changed
         */
        changed: boolean;
        /**
         * Called when a gamepad press event has been triggered
         */
        pressChanged: boolean;
        /**
         * Called when a touch event has been triggered
         */
        touchChanged: boolean;
        /**
         * Called when a value has changed
         */
        valueChanged: boolean;
    }
    /**
     * Represents a gamepad
     */
    export class Gamepad {
        /**
         * The id of the gamepad
         */
        id: string;
        /**
         * The index of the gamepad
         */
        index: number;
        /**
         * The browser gamepad
         */
        browserGamepad: any;
        /**
         * Specifies what type of gamepad this represents
         */
        type: number;
        private _leftStick;
        private _rightStick;
        /** @internal */
        _isConnected: boolean;
        private _leftStickAxisX;
        private _leftStickAxisY;
        private _rightStickAxisX;
        private _rightStickAxisY;
        /**
         * Triggered when the left control stick has been changed
         */
        private _onleftstickchanged;
        /**
         * Triggered when the right control stick has been changed
         */
        private _onrightstickchanged;
        /**
         * Represents a gamepad controller
         */
        static GAMEPAD: number;
        /**
         * Represents a generic controller
         */
        static GENERIC: number;
        /**
         * Represents an XBox controller
         */
        static XBOX: number;
        /**
         * Represents a pose-enabled controller
         */
        static POSE_ENABLED: number;
        /**
         * Represents an Dual Shock controller
         */
        static DUALSHOCK: number;
        /**
         * Specifies whether the left control stick should be Y-inverted
         */
        protected _invertLeftStickY: boolean;
        /**
         * Specifies if the gamepad has been connected
         */
        get isConnected(): boolean;
        /**
         * Initializes the gamepad
         * @param id The id of the gamepad
         * @param index The index of the gamepad
         * @param browserGamepad The browser gamepad
         * @param leftStickX The x component of the left joystick
         * @param leftStickY The y component of the left joystick
         * @param rightStickX The x component of the right joystick
         * @param rightStickY The y component of the right joystick
         */
        constructor(
        /**
         * The id of the gamepad
         */
        id: string, 
        /**
         * The index of the gamepad
         */
        index: number, 
        /**
         * The browser gamepad
         */
        browserGamepad: any, leftStickX?: number, leftStickY?: number, rightStickX?: number, rightStickY?: number);
        /**
         * Callback triggered when the left joystick has changed
         * @param callback
         */
        onleftstickchanged(callback: (values: StickValues) => void): void;
        /**
         * Callback triggered when the right joystick has changed
         * @param callback
         */
        onrightstickchanged(callback: (values: StickValues) => void): void;
        /**
         * Gets the left joystick
         */
        get leftStick(): StickValues;
        /**
         * Sets the left joystick values
         */
        set leftStick(newValues: StickValues);
        /**
         * Gets the right joystick
         */
        get rightStick(): StickValues;
        /**
         * Sets the right joystick value
         */
        set rightStick(newValues: StickValues);
        /**
         * Updates the gamepad joystick positions
         */
        update(): void;
        /**
         * Disposes the gamepad
         */
        dispose(): void;
    }
    /**
     * Represents a generic gamepad
     */
    export class GenericPad extends Gamepad {
        private _buttons;
        private _onbuttondown;
        private _onbuttonup;
        /**
         * Observable triggered when a button has been pressed
         */
        onButtonDownObservable: Observable<number>;
        /**
         * Observable triggered when a button has been released
         */
        onButtonUpObservable: Observable<number>;
        /**
         * Callback triggered when a button has been pressed
         * @param callback Called when a button has been pressed
         */
        onbuttondown(callback: (buttonPressed: number) => void): void;
        /**
         * Callback triggered when a button has been released
         * @param callback Called when a button has been released
         */
        onbuttonup(callback: (buttonReleased: number) => void): void;
        /**
         * Initializes the generic gamepad
         * @param id The id of the generic gamepad
         * @param index The index of the generic gamepad
         * @param browserGamepad The browser gamepad
         */
        constructor(id: string, index: number, browserGamepad: any);
        private _setButtonValue;
        /**
         * Updates the generic gamepad
         */
        update(): void;
        /**
         * Disposes the generic gamepad
         */
        dispose(): void;
    }


    /**
     * Manager for handling gamepads
     */
    export class GamepadManager {
        private _scene?;
        private _babylonGamepads;
        private _oneGamepadConnected;
        /** @internal */
        _isMonitoring: boolean;
        private _gamepadEventSupported;
        private _gamepadSupport?;
        /**
         * observable to be triggered when the gamepad controller has been connected
         */
        onGamepadConnectedObservable: Observable<Gamepad>;
        /**
         * observable to be triggered when the gamepad controller has been disconnected
         */
        onGamepadDisconnectedObservable: Observable<Gamepad>;
        private _onGamepadConnectedEvent;
        private _onGamepadDisconnectedEvent;
        /**
         * Initializes the gamepad manager
         * @param _scene BabylonJS scene
         */
        constructor(_scene?: Scene | undefined);
        /**
         * The gamepads in the game pad manager
         */
        get gamepads(): Gamepad[];
        /**
         * Get the gamepad controllers based on type
         * @param type The type of gamepad controller
         * @returns Nullable gamepad
         */
        getGamepadByType(type?: number): Nullable<Gamepad>;
        /**
         * Disposes the gamepad manager
         */
        dispose(): void;
        private _addNewGamepad;
        private _startMonitoringGamepads;
        private _stopMonitoringGamepads;
        private _loggedErrors;
        /** @internal */
        _checkGamepadsStatus(): void;
        private _updateGamepadObjects;
    }


        interface Scene {
            /** @internal */
            _gamepadManager: Nullable<GamepadManager>;
            /**
             * Gets the gamepad manager associated with the scene
             * @see https://doc.babylonjs.com/features/featuresDeepDive/input/gamepads
             */
            gamepadManager: GamepadManager;
        }
        /**
         * Interface representing a free camera inputs manager
         */
        interface FreeCameraInputsManager {
            /**
             * Adds gamepad input support to the FreeCameraInputsManager.
             * @returns the FreeCameraInputsManager
             */
            addGamepad(): FreeCameraInputsManager;
        }
        /**
         * Interface representing an arc rotate camera inputs manager
         */
        interface ArcRotateCameraInputsManager {
            /**
             * Adds gamepad input support to the ArcRotateCamera InputManager.
             * @returns the camera inputs manager
             */
            addGamepad(): ArcRotateCameraInputsManager;
        }
    /**
     * Defines the gamepad scene component responsible to manage gamepads in a given scene
     */
    export class GamepadSystemSceneComponent implements ISceneComponent {
        /**
         * The component name helpfull to identify the component in the list of scene components.
         */
        readonly name = "Gamepad";
        /**
         * The scene the component belongs to.
         */
        scene: Scene;
        /**
         * Creates a new instance of the component for the given scene
         * @param scene Defines the scene to register the component in
         */
        constructor(scene: Scene);
        /**
         * Registers the component in a given scene
         */
        register(): void;
        /**
         * Rebuilds the elements related to this component in case of
         * context lost for instance.
         */
        rebuild(): void;
        /**
         * Disposes the component and the associated resources
         */
        dispose(): void;
        private _beforeCameraUpdate;
    }




    /**
     * Defines supported buttons for XBox360 compatible gamepads
     */
    export enum Xbox360Button {
        /** A */
        A = 0,
        /** B */
        B = 1,
        /** X */
        X = 2,
        /** Y */
        Y = 3,
        /** Left button */
        LB = 4,
        /** Right button */
        RB = 5,
        /** Back */
        Back = 8,
        /** Start */
        Start = 9,
        /** Left stick */
        LeftStick = 10,
        /** Right stick */
        RightStick = 11
    }
    /** Defines values for XBox360 DPad  */
    export enum Xbox360Dpad {
        /** Up */
        Up = 12,
        /** Down */
        Down = 13,
        /** Left */
        Left = 14,
        /** Right */
        Right = 15
    }
    /**
     * Defines a XBox360 gamepad
     */
    export class Xbox360Pad extends Gamepad {
        private _leftTrigger;
        private _rightTrigger;
        private _onlefttriggerchanged;
        private _onrighttriggerchanged;
        private _onbuttondown;
        private _onbuttonup;
        private _ondpaddown;
        private _ondpadup;
        /** Observable raised when a button is pressed */
        onButtonDownObservable: Observable<Xbox360Button>;
        /** Observable raised when a button is released */
        onButtonUpObservable: Observable<Xbox360Button>;
        /** Observable raised when a pad is pressed */
        onPadDownObservable: Observable<Xbox360Dpad>;
        /** Observable raised when a pad is released */
        onPadUpObservable: Observable<Xbox360Dpad>;
        private _buttonA;
        private _buttonB;
        private _buttonX;
        private _buttonY;
        private _buttonBack;
        private _buttonStart;
        private _buttonLB;
        private _buttonRB;
        private _buttonLeftStick;
        private _buttonRightStick;
        private _dPadUp;
        private _dPadDown;
        private _dPadLeft;
        private _dPadRight;
        private _isXboxOnePad;
        /**
         * Creates a new XBox360 gamepad object
         * @param id defines the id of this gamepad
         * @param index defines its index
         * @param gamepad defines the internal HTML gamepad object
         * @param xboxOne defines if it is a XBox One gamepad
         */
        constructor(id: string, index: number, gamepad: any, xboxOne?: boolean);
        /**
         * Defines the callback to call when left trigger is pressed
         * @param callback defines the callback to use
         */
        onlefttriggerchanged(callback: (value: number) => void): void;
        /**
         * Defines the callback to call when right trigger is pressed
         * @param callback defines the callback to use
         */
        onrighttriggerchanged(callback: (value: number) => void): void;
        /**
         * Gets the left trigger value
         */
        get leftTrigger(): number;
        /**
         * Sets the left trigger value
         */
        set leftTrigger(newValue: number);
        /**
         * Gets the right trigger value
         */
        get rightTrigger(): number;
        /**
         * Sets the right trigger value
         */
        set rightTrigger(newValue: number);
        /**
         * Defines the callback to call when a button is pressed
         * @param callback defines the callback to use
         */
        onbuttondown(callback: (buttonPressed: Xbox360Button) => void): void;
        /**
         * Defines the callback to call when a button is released
         * @param callback defines the callback to use
         */
        onbuttonup(callback: (buttonReleased: Xbox360Button) => void): void;
        /**
         * Defines the callback to call when a pad is pressed
         * @param callback defines the callback to use
         */
        ondpaddown(callback: (dPadPressed: Xbox360Dpad) => void): void;
        /**
         * Defines the callback to call when a pad is released
         * @param callback defines the callback to use
         */
        ondpadup(callback: (dPadReleased: Xbox360Dpad) => void): void;
        private _setButtonValue;
        private _setDPadValue;
        /**
         * Gets the value of the `A` button
         */
        get buttonA(): number;
        /**
         * Sets the value of the `A` button
         */
        set buttonA(value: number);
        /**
         * Gets the value of the `B` button
         */
        get buttonB(): number;
        /**
         * Sets the value of the `B` button
         */
        set buttonB(value: number);
        /**
         * Gets the value of the `X` button
         */
        get buttonX(): number;
        /**
         * Sets the value of the `X` button
         */
        set buttonX(value: number);
        /**
         * Gets the value of the `Y` button
         */
        get buttonY(): number;
        /**
         * Sets the value of the `Y` button
         */
        set buttonY(value: number);
        /**
         * Gets the value of the `Start` button
         */
        get buttonStart(): number;
        /**
         * Sets the value of the `Start` button
         */
        set buttonStart(value: number);
        /**
         * Gets the value of the `Back` button
         */
        get buttonBack(): number;
        /**
         * Sets the value of the `Back` button
         */
        set buttonBack(value: number);
        /**
         * Gets the value of the `Left` button
         */
        get buttonLB(): number;
        /**
         * Sets the value of the `Left` button
         */
        set buttonLB(value: number);
        /**
         * Gets the value of the `Right` button
         */
        get buttonRB(): number;
        /**
         * Sets the value of the `Right` button
         */
        set buttonRB(value: number);
        /**
         * Gets the value of the Left joystick
         */
        get buttonLeftStick(): number;
        /**
         * Sets the value of the Left joystick
         */
        set buttonLeftStick(value: number);
        /**
         * Gets the value of the Right joystick
         */
        get buttonRightStick(): number;
        /**
         * Sets the value of the Right joystick
         */
        set buttonRightStick(value: number);
        /**
         * Gets the value of D-pad up
         */
        get dPadUp(): number;
        /**
         * Sets the value of D-pad up
         */
        set dPadUp(value: number);
        /**
         * Gets the value of D-pad down
         */
        get dPadDown(): number;
        /**
         * Sets the value of D-pad down
         */
        set dPadDown(value: number);
        /**
         * Gets the value of D-pad left
         */
        get dPadLeft(): number;
        /**
         * Sets the value of D-pad left
         */
        set dPadLeft(value: number);
        /**
         * Gets the value of D-pad right
         */
        get dPadRight(): number;
        /**
         * Sets the value of D-pad right
         */
        set dPadRight(value: number);
        /**
         * Force the gamepad to synchronize with device values
         */
        update(): void;
        /**
         * Disposes the gamepad
         */
        dispose(): void;
    }


    /**
     * Interface for axis drag gizmo
     */
    export interface IAxisDragGizmo extends IGizmo {
        /** Drag behavior responsible for the gizmos dragging interactions */
        dragBehavior: PointerDragBehavior;
        /** Drag distance in babylon units that the gizmo will snap to when dragged */
        snapDistance: number;
        /**
         * Event that fires each time the gizmo snaps to a new location.
         * * snapDistance is the the change in distance
         */
        onSnapObservable: Observable<{
            snapDistance: number;
        }>;
        /** If the gizmo is enabled */
        isEnabled: boolean;
        /** Default material used to render when gizmo is not disabled or hovered */
        coloredMaterial: StandardMaterial;
        /** Material used to render when gizmo is hovered with mouse*/
        hoverMaterial: StandardMaterial;
        /** Material used to render when gizmo is disabled. typically grey.*/
        disableMaterial: StandardMaterial;
    }
    /**
     * Single axis drag gizmo
     */
    export class AxisDragGizmo extends Gizmo implements IAxisDragGizmo {
        /**
         * Drag behavior responsible for the gizmos dragging interactions
         */
        dragBehavior: PointerDragBehavior;
        protected _pointerObserver: Nullable<Observer<PointerInfo>>;
        /**
         * Drag distance in babylon units that the gizmo will snap to when dragged (Default: 0)
         */
        snapDistance: number;
        /**
         * Event that fires each time the gizmo snaps to a new location.
         * * snapDistance is the the change in distance
         */
        onSnapObservable: Observable<{
            snapDistance: number;
        }>;
        protected _isEnabled: boolean;
        protected _parent: Nullable<PositionGizmo>;
        protected _gizmoMesh: Mesh;
        protected _coloredMaterial: StandardMaterial;
        protected _hoverMaterial: StandardMaterial;
        protected _disableMaterial: StandardMaterial;
        protected _dragging: boolean;
        /** Default material used to render when gizmo is not disabled or hovered */
        get coloredMaterial(): StandardMaterial;
        /** Material used to render when gizmo is hovered with mouse*/
        get hoverMaterial(): StandardMaterial;
        /** Material used to render when gizmo is disabled. typically grey.*/
        get disableMaterial(): StandardMaterial;
        /**
         * @internal
         */
        static _CreateArrow(scene: Scene, material: StandardMaterial, thickness?: number, isCollider?: boolean): TransformNode;
        /**
         * @internal
         */
        static _CreateArrowInstance(scene: Scene, arrow: TransformNode): TransformNode;
        /**
         * Creates an AxisDragGizmo
         * @param dragAxis The axis which the gizmo will be able to drag on
         * @param color The color of the gizmo
         * @param gizmoLayer The utility layer the gizmo will be added to
         * @param parent
         * @param thickness display gizmo axis thickness
         */
        constructor(dragAxis: Vector3, color?: Color3, gizmoLayer?: UtilityLayerRenderer, parent?: Nullable<PositionGizmo>, thickness?: number);
        protected _attachedNodeChanged(value: Nullable<Node>): void;
        /**
         * If the gizmo is enabled
         */
        set isEnabled(value: boolean);
        get isEnabled(): boolean;
        /**
         * Disposes of the gizmo
         */
        dispose(): void;
    }


    /**
     * Interface for axis scale gizmo
     */
    export interface IAxisScaleGizmo extends IGizmo {
        /** Drag behavior responsible for the gizmos dragging interactions */
        dragBehavior: PointerDragBehavior;
        /** Drag distance in babylon units that the gizmo will snap to when dragged */
        snapDistance: number;
        /**
         * Event that fires each time the gizmo snaps to a new location.
         * * snapDistance is the the change in distance
         */
        onSnapObservable: Observable<{
            snapDistance: number;
        }>;
        /** If the scaling operation should be done on all axis */
        uniformScaling: boolean;
        /** Custom sensitivity value for the drag strength */
        sensitivity: number;
        /** The magnitude of the drag strength (scaling factor) */
        dragScale: number;
        /** If the gizmo is enabled */
        isEnabled: boolean;
        /** Default material used to render when gizmo is not disabled or hovered */
        coloredMaterial: StandardMaterial;
        /** Material used to render when gizmo is hovered with mouse*/
        hoverMaterial: StandardMaterial;
        /** Material used to render when gizmo is disabled. typically grey.*/
        disableMaterial: StandardMaterial;
    }
    /**
     * Single axis scale gizmo
     */
    export class AxisScaleGizmo extends Gizmo implements IAxisScaleGizmo {
        /**
         * Drag behavior responsible for the gizmos dragging interactions
         */
        dragBehavior: PointerDragBehavior;
        protected _pointerObserver: Nullable<Observer<PointerInfo>>;
        /**
         * Scale distance in babylon units that the gizmo will snap to when dragged (Default: 0)
         */
        snapDistance: number;
        /**
         * Event that fires each time the gizmo snaps to a new location.
         * * snapDistance is the the change in distance
         */
        onSnapObservable: Observable<{
            snapDistance: number;
        }>;
        /**
         * If the scaling operation should be done on all axis (default: false)
         */
        uniformScaling: boolean;
        /**
         * Custom sensitivity value for the drag strength
         */
        sensitivity: number;
        /**
         * The magnitude of the drag strength (scaling factor)
         */
        dragScale: number;
        protected _isEnabled: boolean;
        protected _parent: Nullable<ScaleGizmo>;
        protected _gizmoMesh: Mesh;
        protected _coloredMaterial: StandardMaterial;
        protected _hoverMaterial: StandardMaterial;
        protected _disableMaterial: StandardMaterial;
        protected _dragging: boolean;
        private _tmpVector;
        /** Default material used to render when gizmo is not disabled or hovered */
        get coloredMaterial(): StandardMaterial;
        /** Material used to render when gizmo is hovered with mouse*/
        get hoverMaterial(): StandardMaterial;
        /** Material used to render when gizmo is disabled. typically grey.*/
        get disableMaterial(): StandardMaterial;
        /**
         * Creates an AxisScaleGizmo
         * @param dragAxis The axis which the gizmo will be able to scale on
         * @param color The color of the gizmo
         * @param gizmoLayer The utility layer the gizmo will be added to
         * @param parent
         * @param thickness display gizmo axis thickness
         */
        constructor(dragAxis: Vector3, color?: Color3, gizmoLayer?: UtilityLayerRenderer, parent?: Nullable<ScaleGizmo>, thickness?: number);
        /**
         * Create Geometry for Gizmo
         * @param parentMesh
         * @param thickness
         * @param isCollider
         */
        protected _createGizmoMesh(parentMesh: AbstractMesh, thickness: number, isCollider?: boolean): {
            arrowMesh: Mesh;
            arrowTail: Mesh;
        };
        protected _attachedNodeChanged(value: Nullable<Node>): void;
        /**
         * If the gizmo is enabled
         */
        set isEnabled(value: boolean);
        get isEnabled(): boolean;
        /**
         * Disposes of the gizmo
         */
        dispose(): void;
        /**
         * Disposes and replaces the current meshes in the gizmo with the specified mesh
         * @param mesh The mesh to replace the default mesh of the gizmo
         * @param useGizmoMaterial If the gizmo's default material should be used (default: false)
         */
        setCustomMesh(mesh: Mesh, useGizmoMaterial?: boolean): void;
    }


    /**
     * Interface for bounding box gizmo
     */
    export interface IBoundingBoxGizmo extends IGizmo {
        /**
         * If child meshes should be ignored when calculating the bounding box. This should be set to true to avoid perf hits with heavily nested meshes.
         */
        ignoreChildren: boolean;
        /**
         * Returns true if a descendant should be included when computing the bounding box. When null, all descendants are included. If ignoreChildren is set this will be ignored.
         */
        includeChildPredicate: Nullable<(abstractMesh: AbstractMesh) => boolean>;
        /** The size of the rotation spheres attached to the bounding box */
        rotationSphereSize: number;
        /** The size of the scale boxes attached to the bounding box */
        scaleBoxSize: number;
        /**
         * If set, the rotation spheres and scale boxes will increase in size based on the distance away from the camera to have a consistent screen size
         * Note : fixedDragMeshScreenSize takes precedence over fixedDragMeshBoundsSize if both are true
         */
        fixedDragMeshScreenSize: boolean;
        /**
         * If set, the rotation spheres and scale boxes will increase in size based on the size of the bounding box
         * Note : fixedDragMeshScreenSize takes precedence over fixedDragMeshBoundsSize if both are true
         */
        fixedDragMeshBoundsSize: boolean;
        /**
         * The distance away from the object which the draggable meshes should appear world sized when fixedDragMeshScreenSize is set to true
         */
        fixedDragMeshScreenSizeDistanceFactor: number;
        /** Fired when a rotation sphere or scale box is dragged */
        onDragStartObservable: Observable<{}>;
        /** Fired when a scale box is dragged */
        onScaleBoxDragObservable: Observable<{}>;
        /** Fired when a scale box drag is ended */
        onScaleBoxDragEndObservable: Observable<{}>;
        /** Fired when a rotation sphere is dragged */
        onRotationSphereDragObservable: Observable<{}>;
        /** Fired when a rotation sphere drag is ended */
        onRotationSphereDragEndObservable: Observable<{}>;
        /** Relative bounding box pivot used when scaling the attached node. */
        scalePivot: Nullable<Vector3>;
        /** Scale factor vector used for masking some axis */
        axisFactor: Vector3;
        /** Scale factor scalar affecting all axes' drag speed */
        scaleDragSpeed: number;
        /**
         * Sets the color of the bounding box gizmo
         * @param color the color to set
         */
        setColor(color: Color3): void;
        /** Returns an array containing all boxes used for scaling (in increasing x, y and z orders) */
        getScaleBoxes(): AbstractMesh[];
        /** Updates the bounding box information for the Gizmo */
        updateBoundingBox(): void;
        /**
         * Enables rotation on the specified axis and disables rotation on the others
         * @param axis The list of axis that should be enabled (eg. "xy" or "xyz")
         */
        setEnabledRotationAxis(axis: string): void;
        /**
         * Enables/disables scaling
         * @param enable if scaling should be enabled
         * @param homogeneousScaling defines if scaling should only be homogeneous
         */
        setEnabledScaling(enable: boolean, homogeneousScaling?: boolean): void;
        /** Enables a pointer drag behavior on the bounding box of the gizmo */
        enableDragBehavior(): void;
        /** Default material used to render when gizmo is not disabled or hovered */
        coloredMaterial: StandardMaterial;
        /** Material used to render when gizmo is hovered with mouse*/
        hoverMaterial: StandardMaterial;
    }
    /**
     * Bounding box gizmo
     */
    export class BoundingBoxGizmo extends Gizmo implements IBoundingBoxGizmo {
        protected _lineBoundingBox: AbstractMesh;
        protected _rotateSpheresParent: AbstractMesh;
        protected _scaleBoxesParent: AbstractMesh;
        protected _boundingDimensions: Vector3;
        protected _renderObserver: Nullable<Observer<Scene>>;
        protected _pointerObserver: Nullable<Observer<PointerInfo>>;
        protected _scaleDragSpeed: number;
        private _tmpQuaternion;
        private _tmpVector;
        private _tmpRotationMatrix;
        /**
         * If child meshes should be ignored when calculating the bounding box. This should be set to true to avoid perf hits with heavily nested meshes (Default: false)
         */
        ignoreChildren: boolean;
        /**
         * Returns true if a descendant should be included when computing the bounding box. When null, all descendants are included. If ignoreChildren is set this will be ignored. (Default: null)
         */
        includeChildPredicate: Nullable<(abstractMesh: AbstractMesh) => boolean>;
        /**
         * The size of the rotation spheres attached to the bounding box (Default: 0.1)
         */
        rotationSphereSize: number;
        /**
         * The size of the scale boxes attached to the bounding box (Default: 0.1)
         */
        scaleBoxSize: number;
        /**
         * If set, the rotation spheres and scale boxes will increase in size based on the distance away from the camera to have a consistent screen size (Default: false)
         * Note : fixedDragMeshScreenSize takes precedence over fixedDragMeshBoundsSize if both are true
         */
        fixedDragMeshScreenSize: boolean;
        /**
         * If set, the rotation spheres and scale boxes will increase in size based on the size of the bounding box
         * Note : fixedDragMeshScreenSize takes precedence over fixedDragMeshBoundsSize if both are true
         */
        fixedDragMeshBoundsSize: boolean;
        /**
         * The distance away from the object which the draggable meshes should appear world sized when fixedDragMeshScreenSize is set to true (default: 10)
         */
        fixedDragMeshScreenSizeDistanceFactor: number;
        /**
         * Fired when a rotation sphere or scale box is dragged
         */
        onDragStartObservable: Observable<{}>;
        /**
         * Fired when a scale box is dragged
         */
        onScaleBoxDragObservable: Observable<{}>;
        /**
         * Fired when a scale box drag is ended
         */
        onScaleBoxDragEndObservable: Observable<{}>;
        /**
         * Fired when a rotation sphere is dragged
         */
        onRotationSphereDragObservable: Observable<{}>;
        /**
         * Fired when a rotation sphere drag is ended
         */
        onRotationSphereDragEndObservable: Observable<{}>;
        /**
         * Relative bounding box pivot used when scaling the attached node. When null object with scale from the opposite corner. 0.5,0.5,0.5 for center and 0.5,0,0.5 for bottom (Default: null)
         */
        scalePivot: Nullable<Vector3>;
        /**
         * Scale factor used for masking some axis
         */
        protected _axisFactor: Vector3;
        /**
         * Sets the axis factor
         * @param factor the Vector3 value
         */
        set axisFactor(factor: Vector3);
        /**
         * Gets the axis factor
         * @returns the Vector3 factor value
         */
        get axisFactor(): Vector3;
        /**
         * Sets scale drag speed value
         * @param value the new speed value
         */
        set scaleDragSpeed(value: number);
        /**
         * Gets scale drag speed
         * @returns the scale speed number
         */
        get scaleDragSpeed(): number;
        /**
         * Mesh used as a pivot to rotate the attached node
         */
        protected _anchorMesh: AbstractMesh;
        protected _existingMeshScale: Vector3;
        protected _dragMesh: Nullable<Mesh>;
        protected _pointerDragBehavior: PointerDragBehavior;
        protected _coloredMaterial: StandardMaterial;
        protected _hoverColoredMaterial: StandardMaterial;
        /** Default material used to render when gizmo is not disabled or hovered */
        get coloredMaterial(): StandardMaterial;
        /** Material used to render when gizmo is hovered with mouse*/
        get hoverMaterial(): StandardMaterial;
        /**
         * Get the pointerDragBehavior
         */
        get pointerDragBehavior(): PointerDragBehavior;
        /**
         * Sets the color of the bounding box gizmo
         * @param color the color to set
         */
        setColor(color: Color3): void;
        /**
         * Creates an BoundingBoxGizmo
         * @param color The color of the gizmo
         * @param gizmoLayer The utility layer the gizmo will be added to
         */
        constructor(color?: Color3, gizmoLayer?: UtilityLayerRenderer);
        protected _attachedNodeChanged(value: Nullable<AbstractMesh>): void;
        protected _selectNode(selectedMesh: Nullable<Mesh>): void;
        protected _unhoverMeshOnTouchUp(pointerInfo: Nullable<PointerInfo>, selectedMesh: AbstractMesh): void;
        /**
         * returns an array containing all boxes used for scaling (in increasing x, y and z orders)
         */
        getScaleBoxes(): AbstractMesh[];
        /**
         * Updates the bounding box information for the Gizmo
         */
        updateBoundingBox(): void;
        protected _updateRotationSpheres(): void;
        protected _updateScaleBoxes(): void;
        /**
         * Enables rotation on the specified axis and disables rotation on the others
         * @param axis The list of axis that should be enabled (eg. "xy" or "xyz")
         */
        setEnabledRotationAxis(axis: string): void;
        /**
         * Enables/disables scaling
         * @param enable if scaling should be enabled
         * @param homogeneousScaling defines if scaling should only be homogeneous
         */
        setEnabledScaling(enable: boolean, homogeneousScaling?: boolean): void;
        protected _updateDummy(): void;
        /**
         * Enables a pointer drag behavior on the bounding box of the gizmo
         */
        enableDragBehavior(): void;
        /**
         * Disposes of the gizmo
         */
        dispose(): void;
        /**
         * Makes a mesh not pickable and wraps the mesh inside of a bounding box mesh that is pickable. (This is useful to avoid picking within complex geometry)
         * @param mesh the mesh to wrap in the bounding box mesh and make not pickable
         * @returns the bounding box mesh with the passed in mesh as a child
         */
        static MakeNotPickableAndWrapInBoundingBox(mesh: Mesh): Mesh;
        /**
         * CustomMeshes are not supported by this gizmo
         */
        setCustomMesh(): void;
    }


    /**
     * Interface for camera gizmo
     */
    export interface ICameraGizmo extends IGizmo {
        /** Event that fires each time the gizmo is clicked */
        onClickedObservable: Observable<Camera>;
        /** A boolean indicating if frustum lines must be rendered */
        displayFrustum: boolean;
        /** The camera that the gizmo is attached to */
        camera: Nullable<Camera>;
        /** The material used to render the camera gizmo */
        readonly material: StandardMaterial;
    }
    /**
     * Gizmo that enables viewing a camera
     */
    export class CameraGizmo extends Gizmo implements ICameraGizmo {
        protected _cameraMesh: Mesh;
        protected _cameraLinesMesh: Mesh;
        protected _material: StandardMaterial;
        protected _pointerObserver: Nullable<Observer<PointerInfo>>;
        /**
         * Event that fires each time the gizmo is clicked
         */
        onClickedObservable: Observable<Camera>;
        /**
         * Creates a CameraGizmo
         * @param gizmoLayer The utility layer the gizmo will be added to
         */
        constructor(gizmoLayer?: UtilityLayerRenderer);
        protected _camera: Nullable<Camera>;
        /** Gets or sets a boolean indicating if frustum lines must be rendered (true by default)) */
        get displayFrustum(): boolean;
        set displayFrustum(value: boolean);
        /**
         * The camera that the gizmo is attached to
         */
        set camera(camera: Nullable<Camera>);
        get camera(): Nullable<Camera>;
        /**
         * Gets the material used to render the camera gizmo
         */
        get material(): StandardMaterial;
        /**
         * @internal
         * Updates the gizmo to match the attached mesh's position/rotation
         */
        protected _update(): void;
        private static _Scale;
        private _invProjection;
        /**
         * Disposes of the camera gizmo
         */
        dispose(): void;
        private static _CreateCameraMesh;
        private static _CreateCameraFrustum;
    }


    /**
     * Cache built by each axis. Used for managing state between all elements of gizmo for enhanced UI
     */
    export interface GizmoAxisCache {
        /** Mesh used to render the Gizmo */
        gizmoMeshes: Mesh[];
        /** Mesh used to detect user interaction with Gizmo */
        colliderMeshes: Mesh[];
        /** Material used to indicate color of gizmo mesh */
        material: StandardMaterial;
        /** Material used to indicate hover state of the Gizmo */
        hoverMaterial: StandardMaterial;
        /** Material used to indicate disabled state of the Gizmo */
        disableMaterial: StandardMaterial;
        /** Used to indicate Active state of the Gizmo */
        active: boolean;
        /** DragBehavior */
        dragBehavior: PointerDragBehavior;
    }
    /**
     * Interface for basic gizmo
     */
    export interface IGizmo extends IDisposable {
        /** True when the mouse pointer is hovered a gizmo mesh */
        readonly isHovered: boolean;
        /** The root mesh of the gizmo */
        _rootMesh: Mesh;
        /** Ratio for the scale of the gizmo */
        scaleRatio: number;
        /**
         * Mesh that the gizmo will be attached to. (eg. on a drag gizmo the mesh that will be dragged)
         * * When set, interactions will be enabled
         */
        attachedMesh: Nullable<AbstractMesh>;
        /**
         * Node that the gizmo will be attached to. (eg. on a drag gizmo the mesh, bone or NodeTransform that will be dragged)
         * * When set, interactions will be enabled
         */
        attachedNode: Nullable<Node>;
        /**
         * If set the gizmo's rotation will be updated to match the attached mesh each frame (Default: true)
         */
        updateGizmoRotationToMatchAttachedMesh: boolean;
        /** The utility layer the gizmo will be added to */
        gizmoLayer: UtilityLayerRenderer;
        /**
         * If set the gizmo's position will be updated to match the attached mesh each frame (Default: true)
         */
        updateGizmoPositionToMatchAttachedMesh: boolean;
        /**
         * When set, the gizmo will always appear the same size no matter where the camera is (default: true)
         */
        updateScale: boolean;
        /**
         * posture that the gizmo will be display
         * When set null, default value will be used (Quaternion(0, 0, 0, 1))
         */
        customRotationQuaternion: Nullable<Quaternion>;
        /** Disposes and replaces the current meshes in the gizmo with the specified mesh */
        setCustomMesh(mesh: Mesh): void;
    }
    /**
     * Renders gizmos on top of an existing scene which provide controls for position, rotation, etc.
     */
    export class Gizmo implements IGizmo {
        /** The utility layer the gizmo will be added to */
        gizmoLayer: UtilityLayerRenderer;
        /**
         * The root mesh of the gizmo
         */
        _rootMesh: Mesh;
        protected _attachedMesh: Nullable<AbstractMesh>;
        protected _attachedNode: Nullable<Node>;
        protected _customRotationQuaternion: Nullable<Quaternion>;
        /**
         * Ratio for the scale of the gizmo (Default: 1)
         */
        protected _scaleRatio: number;
        /**
         * boolean updated by pointermove when a gizmo mesh is hovered
         */
        protected _isHovered: boolean;
        /**
         * When enabled, any gizmo operation will perserve scaling sign. Default is off.
         * Only valid for TransformNode derived classes (Mesh, AbstractMesh, ...)
         */
        static PreserveScaling: boolean;
        /**
         * Ratio for the scale of the gizmo (Default: 1)
         */
        set scaleRatio(value: number);
        get scaleRatio(): number;
        /**
         * True when the mouse pointer is hovered a gizmo mesh
         */
        get isHovered(): boolean;
        /**
         * If a custom mesh has been set (Default: false)
         */
        protected _customMeshSet: boolean;
        /**
         * Mesh that the gizmo will be attached to. (eg. on a drag gizmo the mesh that will be dragged)
         * * When set, interactions will be enabled
         */
        get attachedMesh(): Nullable<AbstractMesh>;
        set attachedMesh(value: Nullable<AbstractMesh>);
        /**
         * Node that the gizmo will be attached to. (eg. on a drag gizmo the mesh, bone or NodeTransform that will be dragged)
         * * When set, interactions will be enabled
         */
        get attachedNode(): Nullable<Node>;
        set attachedNode(value: Nullable<Node>);
        /**
         * Disposes and replaces the current meshes in the gizmo with the specified mesh
         * @param mesh The mesh to replace the default mesh of the gizmo
         */
        setCustomMesh(mesh: Mesh): void;
        protected _updateGizmoRotationToMatchAttachedMesh: boolean;
        protected _updateGizmoPositionToMatchAttachedMesh: boolean;
        protected _updateScale: boolean;
        /**
         * If set the gizmo's rotation will be updated to match the attached mesh each frame (Default: true)
         * NOTE: This is only possible for meshes with uniform scaling, as otherwise it's not possible to decompose the rotation
         */
        set updateGizmoRotationToMatchAttachedMesh(value: boolean);
        get updateGizmoRotationToMatchAttachedMesh(): boolean;
        /**
         * If set the gizmo's position will be updated to match the attached mesh each frame (Default: true)
         */
        set updateGizmoPositionToMatchAttachedMesh(value: boolean);
        get updateGizmoPositionToMatchAttachedMesh(): boolean;
        /**
         * When set, the gizmo will always appear the same size no matter where the camera is (default: true)
         */
        set updateScale(value: boolean);
        get updateScale(): boolean;
        protected _interactionsEnabled: boolean;
        protected _attachedNodeChanged(value: Nullable<Node>): void;
        protected _beforeRenderObserver: Nullable<Observer<Scene>>;
        private _rightHandtoLeftHandMatrix;
        /**
         * Creates a gizmo
         * @param gizmoLayer The utility layer the gizmo will be added to
         */
        constructor(
        /** The utility layer the gizmo will be added to */
        gizmoLayer?: UtilityLayerRenderer);
        /**
         * posture that the gizmo will be display
         * When set null, default value will be used (Quaternion(0, 0, 0, 1))
         */
        get customRotationQuaternion(): Nullable<Quaternion>;
        set customRotationQuaternion(customRotationQuaternion: Nullable<Quaternion>);
        /**
         * Updates the gizmo to match the attached mesh's position/rotation
         */
        protected _update(): void;
        /**
         * Handle position/translation when using an attached node using pivot
         */
        protected _handlePivot(): void;
        /**
         * computes the rotation/scaling/position of the transform once the Node world matrix has changed.
         */
        protected _matrixChanged(): void;
        /**
         * refresh gizmo mesh material
         * @param gizmoMeshes
         * @param material material to apply
         */
        protected _setGizmoMeshMaterial(gizmoMeshes: Mesh[], material: StandardMaterial): void;
        /**
         * Subscribes to pointer up, down, and hover events. Used for responsive gizmos.
         * @param gizmoLayer The utility layer the gizmo will be added to
         * @param gizmoAxisCache Gizmo axis definition used for reactive gizmo UI
         * @returns {Observer<PointerInfo>} pointerObserver
         */
        static GizmoAxisPointerObserver(gizmoLayer: UtilityLayerRenderer, gizmoAxisCache: Map<Mesh, GizmoAxisCache>): Observer<PointerInfo>;
        /**
         * Disposes of the gizmo
         */
        dispose(): void;
    }


    /**
     * Helps setup gizmo's in the scene to rotate/scale/position nodes
     */
    export class GizmoManager implements IDisposable {
        private _scene;
        /**
         * Gizmo's created by the gizmo manager, gizmo will be null until gizmo has been enabled for the first time
         */
        gizmos: {
            positionGizmo: Nullable<IPositionGizmo>;
            rotationGizmo: Nullable<IRotationGizmo>;
            scaleGizmo: Nullable<IScaleGizmo>;
            boundingBoxGizmo: Nullable<IBoundingBoxGizmo>;
        };
        /** When true, the gizmo will be detached from the current object when a pointer down occurs with an empty picked mesh */
        clearGizmoOnEmptyPointerEvent: boolean;
        /** When true (default), picking to attach a new mesh is enabled. This works in sync with inspector autopicking. */
        enableAutoPicking: boolean;
        /** Fires an event when the manager is attached to a mesh */
        onAttachedToMeshObservable: Observable<Nullable<AbstractMesh>>;
        /** Fires an event when the manager is attached to a node */
        onAttachedToNodeObservable: Observable<Nullable<Node>>;
        protected _gizmosEnabled: {
            positionGizmo: boolean;
            rotationGizmo: boolean;
            scaleGizmo: boolean;
            boundingBoxGizmo: boolean;
        };
        protected _pointerObservers: Observer<PointerInfo>[];
        protected _attachedMesh: Nullable<AbstractMesh>;
        protected _attachedNode: Nullable<Node>;
        protected _boundingBoxColor: Color3;
        protected _defaultUtilityLayer: UtilityLayerRenderer;
        protected _defaultKeepDepthUtilityLayer: UtilityLayerRenderer;
        protected _thickness: number;
        protected _scaleRatio: number;
        /** Node Caching for quick lookup */
        private _gizmoAxisCache;
        /**
         * When bounding box gizmo is enabled, this can be used to track drag/end events
         */
        boundingBoxDragBehavior: SixDofDragBehavior;
        /**
         * Array of meshes which will have the gizmo attached when a pointer selected them. If null, all meshes are attachable. (Default: null)
         */
        attachableMeshes: Nullable<Array<AbstractMesh>>;
        /**
         * Array of nodes which will have the gizmo attached when a pointer selected them. If null, all nodes are attachable. (Default: null)
         */
        attachableNodes: Nullable<Array<Node>>;
        /**
         * If pointer events should perform attaching/detaching a gizmo, if false this can be done manually via attachToMesh/attachToNode. (Default: true)
         */
        usePointerToAttachGizmos: boolean;
        /**
         * Utility layer that the bounding box gizmo belongs to
         */
        get keepDepthUtilityLayer(): UtilityLayerRenderer;
        /**
         * Utility layer that all gizmos besides bounding box belong to
         */
        get utilityLayer(): UtilityLayerRenderer;
        /**
         * True when the mouse pointer is hovering a gizmo mesh
         */
        get isHovered(): boolean;
        /**
         * Ratio for the scale of the gizmo (Default: 1)
         */
        set scaleRatio(value: number);
        get scaleRatio(): number;
        /**
         * Instantiates a gizmo manager
         * @param _scene the scene to overlay the gizmos on top of
         * @param thickness display gizmo axis thickness
         * @param utilityLayer the layer where gizmos are rendered
         * @param keepDepthUtilityLayer the layer where occluded gizmos are rendered
         */
        constructor(_scene: Scene, thickness?: number, utilityLayer?: UtilityLayerRenderer, keepDepthUtilityLayer?: UtilityLayerRenderer);
        /**
         * Subscribes to pointer down events, for attaching and detaching mesh
         * @param scene The scene layer the observer will be added to
         */
        private _attachToMeshPointerObserver;
        /**
         * Attaches a set of gizmos to the specified mesh
         * @param mesh The mesh the gizmo's should be attached to
         */
        attachToMesh(mesh: Nullable<AbstractMesh>): void;
        /**
         * Attaches a set of gizmos to the specified node
         * @param node The node the gizmo's should be attached to
         */
        attachToNode(node: Nullable<Node>): void;
        /**
         * If the position gizmo is enabled
         */
        set positionGizmoEnabled(value: boolean);
        get positionGizmoEnabled(): boolean;
        /**
         * If the rotation gizmo is enabled
         */
        set rotationGizmoEnabled(value: boolean);
        get rotationGizmoEnabled(): boolean;
        /**
         * If the scale gizmo is enabled
         */
        set scaleGizmoEnabled(value: boolean);
        get scaleGizmoEnabled(): boolean;
        /**
         * If the boundingBox gizmo is enabled
         */
        set boundingBoxGizmoEnabled(value: boolean);
        get boundingBoxGizmoEnabled(): boolean;
        /**
         * Builds Gizmo Axis Cache to enable features such as hover state preservation and graying out other axis during manipulation
         * @param gizmoAxisCache Gizmo axis definition used for reactive gizmo UI
         */
        addToAxisCache(gizmoAxisCache: Map<Mesh, GizmoAxisCache>): void;
        /**
         * Disposes of the gizmo manager
         */
        dispose(): void;
    }




    /**
     * Interface for light gizmo
     */
    export interface ILightGizmo extends IGizmo {
        /** Event that fires each time the gizmo is clicked */
        onClickedObservable: Observable<Light>;
        /** The light that the gizmo is attached to */
        light: Nullable<Light>;
        /** The material used to render the light gizmo */
        readonly material: StandardMaterial;
    }
    /**
     * Gizmo that enables viewing a light
     */
    export class LightGizmo extends Gizmo implements ILightGizmo {
        protected _lightMesh: Mesh;
        protected _material: StandardMaterial;
        protected _cachedPosition: Vector3;
        protected _cachedForward: Vector3;
        protected _attachedMeshParent: TransformNode;
        protected _pointerObserver: Nullable<Observer<PointerInfo>>;
        /**
         * Event that fires each time the gizmo is clicked
         */
        onClickedObservable: Observable<Light>;
        /**
         * Creates a LightGizmo
         * @param gizmoLayer The utility layer the gizmo will be added to
         */
        constructor(gizmoLayer?: UtilityLayerRenderer);
        protected _light: Nullable<Light>;
        /**
         * Override attachedNode because lightgizmo only support attached mesh
         * It will return the attached mesh (if any) and setting an attached node will log
         * a warning
         */
        get attachedNode(): Nullable<Node>;
        set attachedNode(value: Nullable<Node>);
        /**
         * The light that the gizmo is attached to
         */
        set light(light: Nullable<Light>);
        get light(): Nullable<Light>;
        /**
         * Gets the material used to render the light gizmo
         */
        get material(): StandardMaterial;
        /**
         * @internal
         * Updates the gizmo to match the attached mesh's position/rotation
         */
        protected _update(): void;
        private static _Scale;
        /**
         * Creates the lines for a light mesh
         * @param levels
         * @param scene
         */
        private static _CreateLightLines;
        /**
         * Disposes of the light gizmo
         */
        dispose(): void;
        private static _CreateHemisphericLightMesh;
        private static _CreatePointLightMesh;
        private static _CreateSpotLightMesh;
        private static _CreateDirectionalLightMesh;
    }


    /**
     * Interface for plane drag gizmo
     */
    export interface IPlaneDragGizmo extends IGizmo {
        /** Drag behavior responsible for the gizmos dragging interactions */
        dragBehavior: PointerDragBehavior;
        /** Drag distance in babylon units that the gizmo will snap to when dragged */
        snapDistance: number;
        /**
         * Event that fires each time the gizmo snaps to a new location.
         * * snapDistance is the the change in distance
         */
        onSnapObservable: Observable<{
            snapDistance: number;
        }>;
        /** If the gizmo is enabled */
        isEnabled: boolean;
        /** Default material used to render when gizmo is not disabled or hovered */
        coloredMaterial: StandardMaterial;
        /** Material used to render when gizmo is hovered with mouse*/
        hoverMaterial: StandardMaterial;
        /** Material used to render when gizmo is disabled. typically grey.*/
        disableMaterial: StandardMaterial;
    }
    /**
     * Single plane drag gizmo
     */
    export class PlaneDragGizmo extends Gizmo implements IPlaneDragGizmo {
        /**
         * Drag behavior responsible for the gizmos dragging interactions
         */
        dragBehavior: PointerDragBehavior;
        protected _pointerObserver: Nullable<Observer<PointerInfo>>;
        /**
         * Drag distance in babylon units that the gizmo will snap to when dragged (Default: 0)
         */
        snapDistance: number;
        /**
         * Event that fires each time the gizmo snaps to a new location.
         * * snapDistance is the the change in distance
         */
        onSnapObservable: Observable<{
            snapDistance: number;
        }>;
        protected _gizmoMesh: TransformNode;
        protected _coloredMaterial: StandardMaterial;
        protected _hoverMaterial: StandardMaterial;
        protected _disableMaterial: StandardMaterial;
        protected _isEnabled: boolean;
        protected _parent: Nullable<PositionGizmo>;
        protected _dragging: boolean;
        /** Default material used to render when gizmo is not disabled or hovered */
        get coloredMaterial(): StandardMaterial;
        /** Material used to render when gizmo is hovered with mouse*/
        get hoverMaterial(): StandardMaterial;
        /** Material used to render when gizmo is disabled. typically grey.*/
        get disableMaterial(): StandardMaterial;
        /**
         * @internal
         */
        static _CreatePlane(scene: Scene, material: StandardMaterial): TransformNode;
        /**
         * Creates a PlaneDragGizmo
         * @param dragPlaneNormal The axis normal to which the gizmo will be able to drag on
         * @param color The color of the gizmo
         * @param gizmoLayer The utility layer the gizmo will be added to
         * @param parent
         */
        constructor(dragPlaneNormal: Vector3, color?: Color3, gizmoLayer?: UtilityLayerRenderer, parent?: Nullable<PositionGizmo>);
        protected _attachedNodeChanged(value: Nullable<Node>): void;
        /**
         * If the gizmo is enabled
         */
        set isEnabled(value: boolean);
        get isEnabled(): boolean;
        /**
         * Disposes of the gizmo
         */
        dispose(): void;
    }


    /**
     * Interface for plane rotation gizmo
     */
    export interface IPlaneRotationGizmo extends IGizmo {
        /** Drag behavior responsible for the gizmos dragging interactions */
        dragBehavior: PointerDragBehavior;
        /** Drag distance in babylon units that the gizmo will snap to when dragged */
        snapDistance: number;
        /**
         * Event that fires each time the gizmo snaps to a new location.
         * * snapDistance is the the change in distance
         */
        onSnapObservable: Observable<{
            snapDistance: number;
        }>;
        /** Accumulated relative angle value for rotation on the axis. */
        angle: number;
        /** If the gizmo is enabled */
        isEnabled: boolean;
        /** Default material used to render when gizmo is not disabled or hovered */
        coloredMaterial: StandardMaterial;
        /** Material used to render when gizmo is hovered with mouse*/
        hoverMaterial: StandardMaterial;
        /** Material used to render when gizmo is disabled. typically grey.*/
        disableMaterial: StandardMaterial;
    }
    /**
     * Single plane rotation gizmo
     */
    export class PlaneRotationGizmo extends Gizmo implements IPlaneRotationGizmo {
        /**
         * Drag behavior responsible for the gizmos dragging interactions
         */
        dragBehavior: PointerDragBehavior;
        protected _pointerObserver: Nullable<Observer<PointerInfo>>;
        /**
         * Rotation distance in radians that the gizmo will snap to (Default: 0)
         */
        snapDistance: number;
        /**
         * Event that fires each time the gizmo snaps to a new location.
         * * snapDistance is the the change in distance
         */
        onSnapObservable: Observable<{
            snapDistance: number;
        }>;
        /**
         * The maximum angle between the camera and the rotation allowed for interaction
         * If a rotation plane appears 'flat', a lower value allows interaction.
         */
        static MaxDragAngle: number;
        /**
         * Accumulated relative angle value for rotation on the axis. Reset to 0 when a dragStart occurs
         */
        angle: number;
        /** Default material used to render when gizmo is not disabled or hovered */
        get coloredMaterial(): StandardMaterial;
        /** Material used to render when gizmo is hovered with mouse*/
        get hoverMaterial(): StandardMaterial;
        /** Material used to render when gizmo is disabled. typically grey.*/
        get disableMaterial(): StandardMaterial;
        protected _isEnabled: boolean;
        protected _parent: Nullable<RotationGizmo>;
        protected _coloredMaterial: StandardMaterial;
        protected _hoverMaterial: StandardMaterial;
        protected _disableMaterial: StandardMaterial;
        protected _gizmoMesh: Mesh;
        protected _rotationDisplayPlane: Mesh;
        protected _dragging: boolean;
        protected _angles: Vector3;
        protected static _RotationGizmoVertexShader: string;
        protected static _RotationGizmoFragmentShader: string;
        protected _rotationShaderMaterial: ShaderMaterial;
        /**
         * Creates a PlaneRotationGizmo
         * @param planeNormal The normal of the plane which the gizmo will be able to rotate on
         * @param color The color of the gizmo
         * @param gizmoLayer The utility layer the gizmo will be added to
         * @param tessellation Amount of tessellation to be used when creating rotation circles
         * @param parent
         * @param useEulerRotation Use and update Euler angle instead of quaternion
         * @param thickness display gizmo axis thickness
         */
        constructor(planeNormal: Vector3, color?: Color3, gizmoLayer?: UtilityLayerRenderer, tessellation?: number, parent?: Nullable<RotationGizmo>, useEulerRotation?: boolean, thickness?: number);
        /**
         * Create Geometry for Gizmo
         * @param parentMesh
         * @param thickness
         * @param tessellation
         */
        protected _createGizmoMesh(parentMesh: AbstractMesh, thickness: number, tessellation: number): {
            rotationMesh: Mesh;
            collider: Mesh;
        };
        protected _attachedNodeChanged(value: Nullable<Node>): void;
        /**
         * If the gizmo is enabled
         */
        set isEnabled(value: boolean);
        get isEnabled(): boolean;
        /**
         * Disposes of the gizmo
         */
        dispose(): void;
    }


    /**
     * Interface for position gizmo
     */
    export interface IPositionGizmo extends IGizmo {
        /** Internal gizmo used for interactions on the x axis */
        xGizmo: IAxisDragGizmo;
        /** Internal gizmo used for interactions on the y axis */
        yGizmo: IAxisDragGizmo;
        /** Internal gizmo used for interactions on the z axis */
        zGizmo: IAxisDragGizmo;
        /** Internal gizmo used for interactions on the yz plane */
        xPlaneGizmo: IPlaneDragGizmo;
        /** Internal gizmo used for interactions on the xz plane */
        yPlaneGizmo: IPlaneDragGizmo;
        /** Internal gizmo used for interactions on the xy plane */
        zPlaneGizmo: IPlaneDragGizmo;
        /** Fires an event when any of it's sub gizmos are dragged */
        onDragStartObservable: Observable<unknown>;
        /** Fires an event when any of it's sub gizmos are released from dragging */
        onDragEndObservable: Observable<unknown>;
        /**
         * If the planar drag gizmo is enabled
         * setting this will enable/disable XY, XZ and YZ planes regardless of individual gizmo settings.
         */
        planarGizmoEnabled: boolean;
        /** Drag distance in babylon units that the gizmo will snap to when dragged */
        snapDistance: number;
        /**
         * Builds Gizmo Axis Cache to enable features such as hover state preservation and graying out other axis during manipulation
         * @param mesh Axis gizmo mesh
         * @param cache Gizmo axis definition used for reactive gizmo UI
         */
        addToAxisCache(mesh: Mesh, cache: GizmoAxisCache): void;
    }
    /**
     * Gizmo that enables dragging a mesh along 3 axis
     */
    export class PositionGizmo extends Gizmo implements IPositionGizmo {
        /**
         * Internal gizmo used for interactions on the x axis
         */
        xGizmo: IAxisDragGizmo;
        /**
         * Internal gizmo used for interactions on the y axis
         */
        yGizmo: IAxisDragGizmo;
        /**
         * Internal gizmo used for interactions on the z axis
         */
        zGizmo: IAxisDragGizmo;
        /**
         * Internal gizmo used for interactions on the yz plane
         */
        xPlaneGizmo: IPlaneDragGizmo;
        /**
         * Internal gizmo used for interactions on the xz plane
         */
        yPlaneGizmo: IPlaneDragGizmo;
        /**
         * Internal gizmo used for interactions on the xy plane
         */
        zPlaneGizmo: IPlaneDragGizmo;
        /**
         * protected variables
         */
        protected _meshAttached: Nullable<AbstractMesh>;
        protected _nodeAttached: Nullable<Node>;
        protected _snapDistance: number;
        protected _observables: Observer<PointerInfo>[];
        /** Node Caching for quick lookup */
        protected _gizmoAxisCache: Map<Mesh, GizmoAxisCache>;
        /** Fires an event when any of it's sub gizmos are dragged */
        onDragStartObservable: Observable<unknown>;
        /** Fires an event when any of it's sub gizmos are released from dragging */
        onDragEndObservable: Observable<unknown>;
        /**
         * If set to true, planar drag is enabled
         */
        protected _planarGizmoEnabled: boolean;
        get attachedMesh(): Nullable<AbstractMesh>;
        set attachedMesh(mesh: Nullable<AbstractMesh>);
        get attachedNode(): Nullable<Node>;
        set attachedNode(node: Nullable<Node>);
        /**
         * True when the mouse pointer is hovering a gizmo mesh
         */
        get isHovered(): boolean;
        /**
         * Creates a PositionGizmo
         * @param gizmoLayer The utility layer the gizmo will be added to
          @param thickness display gizmo axis thickness
         * @param gizmoManager
         */
        constructor(gizmoLayer?: UtilityLayerRenderer, thickness?: number, gizmoManager?: GizmoManager);
        /**
         * If the planar drag gizmo is enabled
         * setting this will enable/disable XY, XZ and YZ planes regardless of individual gizmo settings.
         */
        set planarGizmoEnabled(value: boolean);
        get planarGizmoEnabled(): boolean;
        /**
         * If set the gizmo's rotation will be updated to match the attached mesh each frame (Default: true)
         * NOTE: This is only possible for meshes with uniform scaling, as otherwise it's not possible to decompose the rotation
         */
        set updateGizmoRotationToMatchAttachedMesh(value: boolean);
        get updateGizmoRotationToMatchAttachedMesh(): boolean;
        set updateGizmoPositionToMatchAttachedMesh(value: boolean);
        get updateGizmoPositionToMatchAttachedMesh(): boolean;
        set updateScale(value: boolean);
        get updateScale(): boolean;
        /**
         * Drag distance in babylon units that the gizmo will snap to when dragged (Default: 0)
         */
        set snapDistance(value: number);
        get snapDistance(): number;
        /**
         * Ratio for the scale of the gizmo (Default: 1)
         */
        set scaleRatio(value: number);
        get scaleRatio(): number;
        /**
         * Builds Gizmo Axis Cache to enable features such as hover state preservation and graying out other axis during manipulation
         * @param mesh Axis gizmo mesh
         * @param cache Gizmo axis definition used for reactive gizmo UI
         */
        addToAxisCache(mesh: Mesh, cache: GizmoAxisCache): void;
        /**
         * Disposes of the gizmo
         */
        dispose(): void;
        /**
         * CustomMeshes are not supported by this gizmo
         */
        setCustomMesh(): void;
    }


    /**
     * Interface for rotation gizmo
     */
    export interface IRotationGizmo extends IGizmo {
        /** Internal gizmo used for interactions on the x axis */
        xGizmo: IPlaneRotationGizmo;
        /** Internal gizmo used for interactions on the y axis */
        yGizmo: IPlaneRotationGizmo;
        /** Internal gizmo used for interactions on the z axis */
        zGizmo: IPlaneRotationGizmo;
        /** Fires an event when any of it's sub gizmos are dragged */
        onDragStartObservable: Observable<unknown>;
        /** Fires an event when any of it's sub gizmos are released from dragging */
        onDragEndObservable: Observable<unknown>;
        /** Drag distance in babylon units that the gizmo will snap to when dragged */
        snapDistance: number;
        /**
         * Builds Gizmo Axis Cache to enable features such as hover state preservation and graying out other axis during manipulation
         * @param mesh Axis gizmo mesh
         * @param cache Gizmo axis definition used for reactive gizmo UI
         */
        addToAxisCache(mesh: Mesh, cache: GizmoAxisCache): void;
    }
    /**
     * Options for each individual plane rotation gizmo contained within RotationGizmo
     * @since 5.0.0
     */
    export interface PlaneRotationGizmoOptions {
        /**
         * Color to use for the plane rotation gizmo
         */
        color?: Color3;
    }
    /**
     * Additional options for each rotation gizmo
     */
    export interface RotationGizmoOptions {
        /**
         * When set, the gizmo will always appear the same size no matter where the camera is (default: true)
         */
        updateScale?: boolean;
        /**
         * Specific options for xGizmo
         */
        xOptions?: PlaneRotationGizmoOptions;
        /**
         * Specific options for yGizmo
         */
        yOptions?: PlaneRotationGizmoOptions;
        /**
         * Specific options for zGizmo
         */
        zOptions?: PlaneRotationGizmoOptions;
    }
    /**
     * Gizmo that enables rotating a mesh along 3 axis
     */
    export class RotationGizmo extends Gizmo implements IRotationGizmo {
        /**
         * Internal gizmo used for interactions on the x axis
         */
        xGizmo: IPlaneRotationGizmo;
        /**
         * Internal gizmo used for interactions on the y axis
         */
        yGizmo: IPlaneRotationGizmo;
        /**
         * Internal gizmo used for interactions on the z axis
         */
        zGizmo: IPlaneRotationGizmo;
        /** Fires an event when any of it's sub gizmos are dragged */
        onDragStartObservable: Observable<unknown>;
        /** Fires an event when any of it's sub gizmos are released from dragging */
        onDragEndObservable: Observable<unknown>;
        protected _meshAttached: Nullable<AbstractMesh>;
        protected _nodeAttached: Nullable<Node>;
        protected _observables: Observer<PointerInfo>[];
        /** Node Caching for quick lookup */
        protected _gizmoAxisCache: Map<Mesh, GizmoAxisCache>;
        get attachedMesh(): Nullable<AbstractMesh>;
        set attachedMesh(mesh: Nullable<AbstractMesh>);
        get attachedNode(): Nullable<Node>;
        set attachedNode(node: Nullable<Node>);
        protected _checkBillboardTransform(): void;
        /**
         * True when the mouse pointer is hovering a gizmo mesh
         */
        get isHovered(): boolean;
        /**
         * Creates a RotationGizmo
         * @param gizmoLayer The utility layer the gizmo will be added to
         * @param tessellation Amount of tessellation to be used when creating rotation circles
         * @param useEulerRotation Use and update Euler angle instead of quaternion
         * @param thickness display gizmo axis thickness
         * @param gizmoManager Gizmo manager
         * @param options More options
         */
        constructor(gizmoLayer?: UtilityLayerRenderer, tessellation?: number, useEulerRotation?: boolean, thickness?: number, gizmoManager?: GizmoManager, options?: RotationGizmoOptions);
        /**
         * If set the gizmo's rotation will be updated to match the attached mesh each frame (Default: true)
         * NOTE: This is only possible for meshes with uniform scaling, as otherwise it's not possible to decompose the rotation
         */
        set updateGizmoRotationToMatchAttachedMesh(value: boolean);
        get updateGizmoRotationToMatchAttachedMesh(): boolean;
        set updateGizmoPositionToMatchAttachedMesh(value: boolean);
        get updateGizmoPositionToMatchAttachedMesh(): boolean;
        set updateScale(value: boolean);
        get updateScale(): boolean;
        /**
         * Drag distance in babylon units that the gizmo will snap to when dragged (Default: 0)
         */
        set snapDistance(value: number);
        get snapDistance(): number;
        /**
         * Ratio for the scale of the gizmo (Default: 1)
         */
        set scaleRatio(value: number);
        get scaleRatio(): number;
        /**
         * Builds Gizmo Axis Cache to enable features such as hover state preservation and graying out other axis during manipulation
         * @param mesh Axis gizmo mesh
         * @param cache Gizmo axis definition used for reactive gizmo UI
         */
        addToAxisCache(mesh: Mesh, cache: GizmoAxisCache): void;
        /**
         * Disposes of the gizmo
         */
        dispose(): void;
        /**
         * CustomMeshes are not supported by this gizmo
         */
        setCustomMesh(): void;
    }


    /**
     * Interface for scale gizmo
     */
    export interface IScaleGizmo extends IGizmo {
        /** Internal gizmo used for interactions on the x axis */
        xGizmo: IAxisScaleGizmo;
        /** Internal gizmo used for interactions on the y axis */
        yGizmo: IAxisScaleGizmo;
        /** Internal gizmo used for interactions on the z axis */
        zGizmo: IAxisScaleGizmo;
        /** Internal gizmo used to scale all axis equally*/
        uniformScaleGizmo: IAxisScaleGizmo;
        /** Fires an event when any of it's sub gizmos are dragged */
        onDragStartObservable: Observable<unknown>;
        /** Fires an event when any of it's sub gizmos are released from dragging */
        onDragEndObservable: Observable<unknown>;
        /** Drag distance in babylon units that the gizmo will snap to when dragged */
        snapDistance: number;
        /** Sensitivity factor for dragging */
        sensitivity: number;
        /**
         * Builds Gizmo Axis Cache to enable features such as hover state preservation and graying out other axis during manipulation
         * @param mesh Axis gizmo mesh
         * @param cache Gizmo axis definition used for reactive gizmo UI
         */
        addToAxisCache(mesh: Mesh, cache: GizmoAxisCache): void;
        /** Default material used to render when gizmo is not disabled or hovered */
        coloredMaterial: StandardMaterial;
        /** Material used to render when gizmo is hovered with mouse*/
        hoverMaterial: StandardMaterial;
        /** Material used to render when gizmo is disabled. typically grey.*/
        disableMaterial: StandardMaterial;
    }
    /**
     * Gizmo that enables scaling a mesh along 3 axis
     */
    export class ScaleGizmo extends Gizmo implements IScaleGizmo {
        /**
         * Internal gizmo used for interactions on the x axis
         */
        xGizmo: IAxisScaleGizmo;
        /**
         * Internal gizmo used for interactions on the y axis
         */
        yGizmo: IAxisScaleGizmo;
        /**
         * Internal gizmo used for interactions on the z axis
         */
        zGizmo: IAxisScaleGizmo;
        /**
         * Internal gizmo used to scale all axis equally
         */
        uniformScaleGizmo: IAxisScaleGizmo;
        protected _meshAttached: Nullable<AbstractMesh>;
        protected _nodeAttached: Nullable<Node>;
        protected _snapDistance: number;
        protected _uniformScalingMesh: Mesh;
        protected _octahedron: Mesh;
        protected _sensitivity: number;
        protected _coloredMaterial: StandardMaterial;
        protected _hoverMaterial: StandardMaterial;
        protected _disableMaterial: StandardMaterial;
        protected _observables: Observer<PointerInfo>[];
        /** Node Caching for quick lookup */
        protected _gizmoAxisCache: Map<Mesh, GizmoAxisCache>;
        /** Default material used to render when gizmo is not disabled or hovered */
        get coloredMaterial(): StandardMaterial;
        /** Material used to render when gizmo is hovered with mouse*/
        get hoverMaterial(): StandardMaterial;
        /** Material used to render when gizmo is disabled. typically grey.*/
        get disableMaterial(): StandardMaterial;
        /** Fires an event when any of it's sub gizmos are dragged */
        onDragStartObservable: Observable<unknown>;
        /** Fires an event when any of it's sub gizmos are released from dragging */
        onDragEndObservable: Observable<unknown>;
        get attachedMesh(): Nullable<AbstractMesh>;
        set attachedMesh(mesh: Nullable<AbstractMesh>);
        get attachedNode(): Nullable<Node>;
        set attachedNode(node: Nullable<Node>);
        set updateScale(value: boolean);
        get updateScale(): boolean;
        /**
         * True when the mouse pointer is hovering a gizmo mesh
         */
        get isHovered(): boolean;
        /**
         * Creates a ScaleGizmo
         * @param gizmoLayer The utility layer the gizmo will be added to
         * @param thickness display gizmo axis thickness
         * @param gizmoManager
         */
        constructor(gizmoLayer?: UtilityLayerRenderer, thickness?: number, gizmoManager?: GizmoManager);
        /** Create Geometry for Gizmo */
        protected _createUniformScaleMesh(): AxisScaleGizmo;
        set updateGizmoRotationToMatchAttachedMesh(value: boolean);
        get updateGizmoRotationToMatchAttachedMesh(): boolean;
        /**
         * Drag distance in babylon units that the gizmo will snap to when dragged (Default: 0)
         */
        set snapDistance(value: number);
        get snapDistance(): number;
        /**
         * Ratio for the scale of the gizmo (Default: 1)
         */
        set scaleRatio(value: number);
        get scaleRatio(): number;
        /**
         * Sensitivity factor for dragging (Default: 1)
         */
        set sensitivity(value: number);
        get sensitivity(): number;
        /**
         * Builds Gizmo Axis Cache to enable features such as hover state preservation and graying out other axis during manipulation
         * @param mesh Axis gizmo mesh
         * @param cache Gizmo axis definition used for reactive gizmo UI
         */
        addToAxisCache(mesh: Mesh, cache: GizmoAxisCache): void;
        /**
         * Disposes of the gizmo
         */
        dispose(): void;
    }


    /**
     * Represents the different options available during the creation of
     * a Environment helper.
     *
     * This can control the default ground, skybox and image processing setup of your scene.
     */
    export interface IEnvironmentHelperOptions {
        /**
         * Specifies whether or not to create a ground.
         * True by default.
         */
        createGround: boolean;
        /**
         * Specifies the ground size.
         * 15 by default.
         */
        groundSize: number;
        /**
         * The texture used on the ground for the main color.
         * Comes from the BabylonJS CDN by default.
         *
         * Remarks: Can be either a texture or a url.
         */
        groundTexture: string | BaseTexture;
        /**
         * The color mixed in the ground texture by default.
         * BabylonJS clearColor by default.
         */
        groundColor: Color3;
        /**
         * Specifies the ground opacity.
         * 1 by default.
         */
        groundOpacity: number;
        /**
         * Enables the ground to receive shadows.
         * True by default.
         */
        enableGroundShadow: boolean;
        /**
         * Helps preventing the shadow to be fully black on the ground.
         * 0.5 by default.
         */
        groundShadowLevel: number;
        /**
         * Creates a mirror texture attach to the ground.
         * false by default.
         */
        enableGroundMirror: boolean;
        /**
         * Specifies the ground mirror size ratio.
         * 0.3 by default as the default kernel is 64.
         */
        groundMirrorSizeRatio: number;
        /**
         * Specifies the ground mirror blur kernel size.
         * 64 by default.
         */
        groundMirrorBlurKernel: number;
        /**
         * Specifies the ground mirror visibility amount.
         * 1 by default
         */
        groundMirrorAmount: number;
        /**
         * Specifies the ground mirror reflectance weight.
         * This uses the standard weight of the background material to setup the fresnel effect
         * of the mirror.
         * 1 by default.
         */
        groundMirrorFresnelWeight: number;
        /**
         * Specifies the ground mirror Falloff distance.
         * This can helps reducing the size of the reflection.
         * 0 by Default.
         */
        groundMirrorFallOffDistance: number;
        /**
         * Specifies the ground mirror texture type.
         * Unsigned Int by Default.
         */
        groundMirrorTextureType: number;
        /**
         * Specifies a bias applied to the ground vertical position to prevent z-fighting with
         * the shown objects.
         */
        groundYBias: number;
        /**
         * Specifies whether or not to create a skybox.
         * True by default.
         */
        createSkybox: boolean;
        /**
         * Specifies the skybox size.
         * 20 by default.
         */
        skyboxSize: number;
        /**
         * The texture used on the skybox for the main color.
         * Comes from the BabylonJS CDN by default.
         *
         * Remarks: Can be either a texture or a url.
         */
        skyboxTexture: string | BaseTexture;
        /**
         * The color mixed in the skybox texture by default.
         * BabylonJS clearColor by default.
         */
        skyboxColor: Color3;
        /**
         * The background rotation around the Y axis of the scene.
         * This helps aligning the key lights of your scene with the background.
         * 0 by default.
         */
        backgroundYRotation: number;
        /**
         * Compute automatically the size of the elements to best fit with the scene.
         */
        sizeAuto: boolean;
        /**
         * Default position of the rootMesh if autoSize is not true.
         */
        rootPosition: Vector3;
        /**
         * Sets up the image processing in the scene.
         * true by default.
         */
        setupImageProcessing: boolean;
        /**
         * The texture used as your environment texture in the scene.
         * Comes from the BabylonJS CDN by default and in use if setupImageProcessing is true.
         *
         * Remarks: Can be either a texture or a url.
         */
        environmentTexture: string | BaseTexture;
        /**
         * The value of the exposure to apply to the scene.
         * 0.6 by default if setupImageProcessing is true.
         */
        cameraExposure: number;
        /**
         * The value of the contrast to apply to the scene.
         * 1.6 by default if setupImageProcessing is true.
         */
        cameraContrast: number;
        /**
         * Specifies whether or not tonemapping should be enabled in the scene.
         * true by default if setupImageProcessing is true.
         */
        toneMappingEnabled: boolean;
    }
    /**
     * The Environment helper class can be used to add a fully featured none expensive background to your scene.
     * It includes by default a skybox and a ground relying on the BackgroundMaterial.
     * It also helps with the default setup of your imageProcessing configuration.
     */
    export class EnvironmentHelper {
        /**
         * Default ground texture URL.
         */
        private static _GroundTextureCDNUrl;
        /**
         * Default skybox texture URL.
         */
        private static _SkyboxTextureCDNUrl;
        /**
         * Default environment texture URL.
         */
        private static _EnvironmentTextureCDNUrl;
        /**
         * Creates the default options for the helper.
         */
        private static _GetDefaultOptions;
        private _rootMesh;
        /**
         * Gets the root mesh created by the helper.
         */
        get rootMesh(): Mesh;
        private _skybox;
        /**
         * Gets the skybox created by the helper.
         */
        get skybox(): Nullable<Mesh>;
        private _skyboxTexture;
        /**
         * Gets the skybox texture created by the helper.
         */
        get skyboxTexture(): Nullable<BaseTexture>;
        private _skyboxMaterial;
        /**
         * Gets the skybox material created by the helper.
         */
        get skyboxMaterial(): Nullable<BackgroundMaterial>;
        private _ground;
        /**
         * Gets the ground mesh created by the helper.
         */
        get ground(): Nullable<Mesh>;
        private _groundTexture;
        /**
         * Gets the ground texture created by the helper.
         */
        get groundTexture(): Nullable<BaseTexture>;
        private _groundMirror;
        /**
         * Gets the ground mirror created by the helper.
         */
        get groundMirror(): Nullable<MirrorTexture>;
        /**
         * Gets the ground mirror render list to helps pushing the meshes
         * you wish in the ground reflection.
         */
        get groundMirrorRenderList(): Nullable<AbstractMesh[]>;
        private _groundMaterial;
        /**
         * Gets the ground material created by the helper.
         */
        get groundMaterial(): Nullable<BackgroundMaterial>;
        /**
         * Stores the creation options.
         */
        private readonly _scene;
        private _options;
        /**
         * This observable will be notified with any error during the creation of the environment,
         * mainly texture creation errors.
         */
        onErrorObservable: Observable<{
            message?: string;
            exception?: any;
        }>;
        /**
         * constructor
         * @param options Defines the options we want to customize the helper
         * @param scene The scene to add the material to
         */
        constructor(options: Partial<IEnvironmentHelperOptions>, scene: Scene);
        /**
         * Updates the background according to the new options
         * @param options
         */
        updateOptions(options: Partial<IEnvironmentHelperOptions>): void;
        /**
         * Sets the primary color of all the available elements.
         * @param color the main color to affect to the ground and the background
         */
        setMainColor(color: Color3): void;
        /**
         * Setup the image processing according to the specified options.
         */
        private _setupImageProcessing;
        /**
         * Setup the environment texture according to the specified options.
         */
        private _setupEnvironmentTexture;
        /**
         * Setup the background according to the specified options.
         */
        private _setupBackground;
        /**
         * Get the scene sizes according to the setup.
         */
        private _getSceneSize;
        /**
         * Setup the ground according to the specified options.
         * @param sceneSize
         */
        private _setupGround;
        /**
         * Setup the ground material according to the specified options.
         */
        private _setupGroundMaterial;
        /**
         * Setup the ground diffuse texture according to the specified options.
         */
        private _setupGroundDiffuseTexture;
        /**
         * Setup the ground mirror texture according to the specified options.
         * @param sceneSize
         */
        private _setupGroundMirrorTexture;
        /**
         * Setup the ground to receive the mirror texture.
         */
        private _setupMirrorInGroundMaterial;
        /**
         * Setup the skybox according to the specified options.
         * @param sceneSize
         */
        private _setupSkybox;
        /**
         * Setup the skybox material according to the specified options.
         */
        private _setupSkyboxMaterial;
        /**
         * Setup the skybox reflection texture according to the specified options.
         */
        private _setupSkyboxReflectionTexture;
        private _errorHandler;
        /**
         * Dispose all the elements created by the Helper.
         */
        dispose(): void;
    }




    /**
     * Display a 360 degree photo on an approximately spherical surface, useful for VR applications or skyboxes.
     * As a subclass of TransformNode, this allow parenting to the camera with different locations in the scene.
     * This class achieves its effect with a Texture and a correctly configured BackgroundMaterial on an inverted sphere.
     * Potential additions to this helper include zoom and and non-infinite distance rendering effects.
     */
    export class PhotoDome extends TextureDome<Texture> {
        /**
         * Define the image as a Monoscopic panoramic 360 image.
         */
        static readonly MODE_MONOSCOPIC = 0;
        /**
         * Define the image as a Stereoscopic TopBottom/OverUnder panoramic 360 image.
         */
        static readonly MODE_TOPBOTTOM = 1;
        /**
         * Define the image as a Stereoscopic Side by Side panoramic 360 image.
         */
        static readonly MODE_SIDEBYSIDE = 2;
        /**
         * Gets or sets the texture being displayed on the sphere
         */
        get photoTexture(): Texture;
        /**
         * sets the texture being displayed on the sphere
         */
        set photoTexture(value: Texture);
        /**
         * Gets the current video mode for the video. It can be:
         * * TextureDome.MODE_MONOSCOPIC : Define the texture source as a Monoscopic panoramic 360.
         * * TextureDome.MODE_TOPBOTTOM  : Define the texture source as a Stereoscopic TopBottom/OverUnder panoramic 360.
         * * TextureDome.MODE_SIDEBYSIDE : Define the texture source as a Stereoscopic Side by Side panoramic 360.
         */
        get imageMode(): number;
        /**
         * Sets the current video mode for the video. It can be:
         * * TextureDome.MODE_MONOSCOPIC : Define the texture source as a Monoscopic panoramic 360.
         * * TextureDome.MODE_TOPBOTTOM  : Define the texture source as a Stereoscopic TopBottom/OverUnder panoramic 360.
         * * TextureDome.MODE_SIDEBYSIDE : Define the texture source as a Stereoscopic Side by Side panoramic 360.
         */
        set imageMode(value: number);
        protected _initTexture(urlsOrElement: string, scene: Scene, options: any): Texture;
    }


    /** @internal */
    export var _forceSceneHelpersToBundle: boolean;
        interface Scene {
            /**
             * Creates a default light for the scene.
             * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/fastBuildWorld#create-default-light
             * @param replace has the default false, when true replaces the existing lights in the scene with a hemispheric light
             */
            createDefaultLight(replace?: boolean): void;
            /**
             * Creates a default camera for the scene.
             * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/fastBuildWorld#create-default-camera
             * @param createArcRotateCamera has the default false which creates a free camera, when true creates an arc rotate camera
             * @param replace has default false, when true replaces the active camera in the scene
             * @param attachCameraControls has default false, when true attaches camera controls to the canvas.
             */
            createDefaultCamera(createArcRotateCamera?: boolean, replace?: boolean, attachCameraControls?: boolean): void;
            /**
             * Creates a default camera and a default light.
             * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/fastBuildWorld#create-default-camera-or-light
             * @param createArcRotateCamera has the default false which creates a free camera, when true creates an arc rotate camera
             * @param replace has the default false, when true replaces the active camera/light in the scene
             * @param attachCameraControls has the default false, when true attaches camera controls to the canvas.
             */
            createDefaultCameraOrLight(createArcRotateCamera?: boolean, replace?: boolean, attachCameraControls?: boolean): void;
            /**
             * Creates a new sky box
             * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/fastBuildWorld#create-default-skybox
             * @param environmentTexture defines the texture to use as environment texture
             * @param pbr has default false which requires the StandardMaterial to be used, when true PBRMaterial must be used
             * @param scale defines the overall scale of the skybox
             * @param blur is only available when pbr is true, default is 0, no blur, maximum value is 1
             * @param setGlobalEnvTexture has default true indicating that scene.environmentTexture must match the current skybox texture
             * @returns a new mesh holding the sky box
             */
            createDefaultSkybox(environmentTexture?: BaseTexture, pbr?: boolean, scale?: number, blur?: number, setGlobalEnvTexture?: boolean): Nullable<Mesh>;
            /**
             * Creates a new environment
             * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/fastBuildWorld#create-default-environment
             * @param options defines the options you can use to configure the environment
             * @returns the new EnvironmentHelper
             */
            createDefaultEnvironment(options?: Partial<IEnvironmentHelperOptions>): Nullable<EnvironmentHelper>;
            /**
             * Creates a new VREXperienceHelper
             * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/webVRHelper
             * @param webVROptions defines the options used to create the new VREXperienceHelper
             * @deprecated Please use createDefaultXRExperienceAsync instead
             * @returns a new VREXperienceHelper
             */
            createDefaultVRExperience(webVROptions?: VRExperienceHelperOptions): VRExperienceHelper;
            /**
             * Creates a new WebXRDefaultExperience
             * @see https://doc.babylonjs.com/features/featuresDeepDive/webXR/introToWebXR
             * @param options experience options
             * @returns a promise for a new WebXRDefaultExperience
             */
            createDefaultXRExperienceAsync(options?: WebXRDefaultExperienceOptions): Promise<WebXRDefaultExperience>;
        }


    /**
     * Display a 360/180 degree texture on an approximately spherical surface, useful for VR applications or skyboxes.
     * As a subclass of TransformNode, this allow parenting to the camera or multiple textures with different locations in the scene.
     * This class achieves its effect with a Texture and a correctly configured BackgroundMaterial on an inverted sphere.
     * Potential additions to this helper include zoom and and non-infinite distance rendering effects.
     */
    export abstract class TextureDome<T extends Texture> extends TransformNode {
        protected onError: Nullable<(message?: string, exception?: any) => void>;
        /**
         * Define the source as a Monoscopic panoramic 360/180.
         */
        static readonly MODE_MONOSCOPIC = 0;
        /**
         * Define the source as a Stereoscopic TopBottom/OverUnder panoramic 360/180.
         */
        static readonly MODE_TOPBOTTOM = 1;
        /**
         * Define the source as a Stereoscopic Side by Side panoramic 360/180.
         */
        static readonly MODE_SIDEBYSIDE = 2;
        private _halfDome;
        private _crossEye;
        protected _useDirectMapping: boolean;
        /**
         * The texture being displayed on the sphere
         */
        protected _texture: T;
        /**
         * Gets the texture being displayed on the sphere
         */
        get texture(): T;
        /**
         * Sets the texture being displayed on the sphere
         */
        set texture(newTexture: T);
        /**
         * The skybox material
         */
        protected _material: BackgroundMaterial;
        /**
         * The surface used for the dome
         */
        protected _mesh: Mesh;
        /**
         * Gets the mesh used for the dome.
         */
        get mesh(): Mesh;
        /**
         * A mesh that will be used to mask the back of the dome in case it is a 180 degree movie.
         */
        private _halfDomeMask;
        /**
         * The current fov(field of view) multiplier, 0.0 - 2.0. Defaults to 1.0. Lower values "zoom in" and higher values "zoom out".
         * Also see the options.resolution property.
         */
        get fovMultiplier(): number;
        set fovMultiplier(value: number);
        protected _textureMode: number;
        /**
         * Gets or set the current texture mode for the texture. It can be:
         * * TextureDome.MODE_MONOSCOPIC : Define the texture source as a Monoscopic panoramic 360.
         * * TextureDome.MODE_TOPBOTTOM  : Define the texture source as a Stereoscopic TopBottom/OverUnder panoramic 360.
         * * TextureDome.MODE_SIDEBYSIDE : Define the texture source as a Stereoscopic Side by Side panoramic 360.
         */
        get textureMode(): number;
        /**
         * Sets the current texture mode for the texture. It can be:
         * * TextureDome.MODE_MONOSCOPIC : Define the texture source as a Monoscopic panoramic 360.
         * * TextureDome.MODE_TOPBOTTOM  : Define the texture source as a Stereoscopic TopBottom/OverUnder panoramic 360.
         * * TextureDome.MODE_SIDEBYSIDE : Define the texture source as a Stereoscopic Side by Side panoramic 360.
         */
        set textureMode(value: number);
        /**
         * Is it a 180 degrees dome (half dome) or 360 texture (full dome)
         */
        get halfDome(): boolean;
        /**
         * Set the halfDome mode. If set, only the front (180 degrees) will be displayed and the back will be blacked out.
         */
        set halfDome(enabled: boolean);
        /**
         * Set the cross-eye mode. If set, images that can be seen when crossing eyes will render correctly
         */
        set crossEye(enabled: boolean);
        /**
         * Is it a cross-eye texture?
         */
        get crossEye(): boolean;
        /**
         * The background material of this dome.
         */
        get material(): BackgroundMaterial;
        /**
         * Oberserver used in Stereoscopic VR Mode.
         */
        private _onBeforeCameraRenderObserver;
        /**
         * Observable raised when an error occurred while loading the texture
         */
        onLoadErrorObservable: Observable<string>;
        /**
         * Observable raised when the texture finished loading
         */
        onLoadObservable: Observable<void>;
        /**
         * Create an instance of this class and pass through the parameters to the relevant classes- Texture, StandardMaterial, and Mesh.
         * @param name Element's name, child elements will append suffixes for their own names.
         * @param textureUrlOrElement defines the url(s) or the (video) HTML element to use
         * @param options An object containing optional or exposed sub element properties
         * @param options.resolution
         * @param options.clickToPlay
         * @param options.autoPlay
         * @param options.loop
         * @param options.size
         * @param options.poster
         * @param options.faceForward
         * @param options.useDirectMapping
         * @param options.halfDomeMode
         * @param options.crossEyeMode
         * @param options.generateMipMaps
         * @param options.mesh
         * @param scene
         * @param onError
         */
        constructor(name: string, textureUrlOrElement: string | string[] | HTMLVideoElement, options: {
            resolution?: number;
            clickToPlay?: boolean;
            autoPlay?: boolean;
            loop?: boolean;
            size?: number;
            poster?: string;
            faceForward?: boolean;
            useDirectMapping?: boolean;
            halfDomeMode?: boolean;
            crossEyeMode?: boolean;
            generateMipMaps?: boolean;
            mesh?: Mesh;
        }, scene: Scene, onError?: Nullable<(message?: string, exception?: any) => void>);
        protected abstract _initTexture(urlsOrElement: string | string[] | HTMLElement, scene: Scene, options: any): T;
        protected _changeTextureMode(value: number): void;
        /**
         * Releases resources associated with this node.
         * @param doNotRecurse Set to true to not recurse into each children (recurse into each children by default)
         * @param disposeMaterialAndTextures Set to true to also dispose referenced materials and textures (false by default)
         */
        dispose(doNotRecurse?: boolean, disposeMaterialAndTextures?: boolean): void;
    }


    /**
     * Display a 360/180 degree video on an approximately spherical surface, useful for VR applications or skyboxes.
     * As a subclass of TransformNode, this allow parenting to the camera or multiple videos with different locations in the scene.
     * This class achieves its effect with a VideoTexture and a correctly configured BackgroundMaterial on an inverted sphere.
     * Potential additions to this helper include zoom and and non-infinite distance rendering effects.
     */
    export class VideoDome extends TextureDome<VideoTexture> {
        /**
         * Define the video source as a Monoscopic panoramic 360 video.
         */
        static readonly MODE_MONOSCOPIC = 0;
        /**
         * Define the video source as a Stereoscopic TopBottom/OverUnder panoramic 360 video.
         */
        static readonly MODE_TOPBOTTOM = 1;
        /**
         * Define the video source as a Stereoscopic Side by Side panoramic 360 video.
         */
        static readonly MODE_SIDEBYSIDE = 2;
        /**
         * Get the video texture associated with this video dome
         */
        get videoTexture(): VideoTexture;
        /**
         * Get the video mode of this dome
         */
        get videoMode(): number;
        /**
         * Set the video mode of this dome.
         * @see textureMode
         */
        set videoMode(value: number);
        private _pointerObserver;
        private _textureObserver;
        protected _initTexture(urlsOrElement: string | string[] | HTMLVideoElement, scene: Scene, options: any): VideoTexture;
        /**
         * Releases resources associated with this node.
         * @param doNotRecurse Set to true to not recurse into each children (recurse into each children by default)
         * @param disposeMaterialAndTextures Set to true to also dispose referenced materials and textures (false by default)
         */
        dispose(doNotRecurse?: boolean, disposeMaterialAndTextures?: boolean): void;
    }


    /**
     * Define an interface for a node to indicate it's info for accessibility.
     * By default, Node type doesn't imply accessibility info unless this tag is assigned. Whereas GUI controls already indicate accessibility info, but one can override the info using this tag.
     */
    export interface IAccessibilityTag {
        /**
         * A string as alt text of the node, describing what the node is/does, for accessibility purpose.
         */
        description?: string;
        /**
         * Customize the event of the accessible object.
         * This will be applied on the generated HTML twin node.
         */
        eventHandler?: {
            [key in keyof HTMLElementEventMap]: (e?: Event) => void;
        };
        /**
         * ARIA roles and attributes to customize accessibility support.
         * If you use BabylonJS's accessibility html twin renderer, and want to override the default behavior (not suggested), this can be your way.
         * Learn more about ARIA: https://developer.mozilla.org/en-US/docs/Web/Accessibility/ARIA
         */
        role?: AcceptedRole;
        aria?: {
            [key in AcceptedARIA]: any;
        };
    }
    type AcceptedRole = "toolbar" | "tooltip" | "feed" | "math" | "presentation" | "none" | "note" | "application" | "article" | "cell" | "columnheader" | "definition" | "directory" | "document" | "figure" | "group" | "heading" | "img" | "list" | "listitem" | "meter" | "row" | "rowgroup" | "rowheader" | "separator" | "table" | "term" | "scrollbar" | "searchbox" | "separator" | "slider" | "spinbutton" | "switch" | "tab" | "tabpanel" | "treeitem" | "combobox" | "menu" | "menubar" | "tablist" | "tree" | "treegrid" | "banner" | "complementary" | "contentinfo" | "form" | "main" | "navigation" | "region" | "search" | "alert" | "log" | "marquee" | "status" | "timer" | "alertdialog" | "dialog";
    type AcceptedARIA = "aria-autocomplete" | "aria-checked" | "aria-disabled" | "aria-errormessage" | "aria-expanded" | "aria-haspopup" | "aria-hidden" | "aria-invalid" | "aria-label" | "aria-level" | "aria-modal" | "aria-multiline" | "aria-multiselectable" | "aria-orientation" | "aria-placeholder" | "aria-pressed" | "aria-readonly" | "aria-required" | "aria-selected" | "aria-sort" | "aria-valuemax" | "aria-valuemin" | "aria-valuenow" | "aria-valuetext" | "aria-busy" | "aria-live" | "aria-relevant" | "aria-atomic" | "aria-dropeffect" | "aria-grabbed" | "aria-activedescendant" | "aria-colcount" | "aria-colindex" | "aria-colspan" | "aria-controls" | "aria-describedby" | "aria-description" | "aria-details" | "aria-errormessage" | "aria-flowto" | "aria-labelledby" | "aria-owns" | "aria-posinset" | "aria-rowcount" | "aria-rowindex" | "aria-rowspan" | "aria-setsize";




    /**
     * Class used to manage all inputs for the scene.
     */
    export class InputManager {
        /** The distance in pixel that you have to move to prevent some events */
        static DragMovementThreshold: number;
        /** Time in milliseconds to wait to raise long press events if button is still pressed */
        static LongPressDelay: number;
        /** Time in milliseconds with two consecutive clicks will be considered as a double click */
        static DoubleClickDelay: number;
        /** If you need to check double click without raising a single click at first click, enable this flag */
        static ExclusiveDoubleClickMode: boolean;
        /** This is a defensive check to not allow control attachment prior to an already active one. If already attached, previous control is unattached before attaching the new one. */
        private _alreadyAttached;
        private _alreadyAttachedTo;
        private _onPointerMove;
        private _onPointerDown;
        private _onPointerUp;
        private _initClickEvent;
        private _initActionManager;
        private _delayedSimpleClick;
        private _delayedSimpleClickTimeout;
        private _previousDelayedSimpleClickTimeout;
        private _meshPickProceed;
        private _previousButtonPressed;
        private _currentPickResult;
        private _previousPickResult;
        private _totalPointersPressed;
        private _doubleClickOccured;
        private _isSwiping;
        private _swipeButtonPressed;
        private _skipPointerTap;
        private _isMultiTouchGesture;
        private _pointerOverMesh;
        private _pickedDownMesh;
        private _pickedUpMesh;
        private _pointerX;
        private _pointerY;
        private _unTranslatedPointerX;
        private _unTranslatedPointerY;
        private _startingPointerPosition;
        private _previousStartingPointerPosition;
        private _startingPointerTime;
        private _previousStartingPointerTime;
        private _pointerCaptures;
        private _meshUnderPointerId;
        private _movePointerInfo;
        private _cameraObserverCount;
        private _onKeyDown;
        private _onKeyUp;
        private _scene;
        private _deviceSourceManager;
        /**
         * Creates a new InputManager
         * @param scene - defines the hosting scene
         */
        constructor(scene?: Scene);
        /**
         * Gets the mesh that is currently under the pointer
         * @returns Mesh that the pointer is pointer is hovering over
         */
        get meshUnderPointer(): Nullable<AbstractMesh>;
        /**
         * When using more than one pointer (for example in XR) you can get the mesh under the specific pointer
         * @param pointerId - the pointer id to use
         * @returns The mesh under this pointer id or null if not found
         */
        getMeshUnderPointerByPointerId(pointerId: number): Nullable<AbstractMesh>;
        /**
         * Gets the pointer coordinates in 2D without any translation (ie. straight out of the pointer event)
         * @returns Vector with X/Y values directly from pointer event
         */
        get unTranslatedPointer(): Vector2;
        /**
         * Gets or sets the current on-screen X position of the pointer
         * @returns Translated X with respect to screen
         */
        get pointerX(): number;
        set pointerX(value: number);
        /**
         * Gets or sets the current on-screen Y position of the pointer
         * @returns Translated Y with respect to screen
         */
        get pointerY(): number;
        set pointerY(value: number);
        private _updatePointerPosition;
        private _processPointerMove;
        /** @internal */
        _setRayOnPointerInfo(pickInfo: Nullable<PickingInfo>, event: IMouseEvent): void;
        /** @internal */
        _addCameraPointerObserver(observer: (p: PointerInfo, s: EventState) => void, mask?: number): Nullable<Observer<PointerInfo>>;
        /** @internal */
        _removeCameraPointerObserver(observer: Observer<PointerInfo>): boolean;
        private _checkForPicking;
        private _checkPrePointerObservable;
        /** @internal */
        _pickMove(pointerId: number): PickingInfo;
        private _setCursorAndPointerOverMesh;
        /**
         * Use this method to simulate a pointer move on a mesh
         * The pickResult parameter can be obtained from a scene.pick or scene.pickWithRay
         * @param pickResult - pickingInfo of the object wished to simulate pointer event on
         * @param pointerEventInit - pointer event state to be used when simulating the pointer event (eg. pointer id for multitouch)
         */
        simulatePointerMove(pickResult: PickingInfo, pointerEventInit?: PointerEventInit): void;
        /**
         * Use this method to simulate a pointer down on a mesh
         * The pickResult parameter can be obtained from a scene.pick or scene.pickWithRay
         * @param pickResult - pickingInfo of the object wished to simulate pointer event on
         * @param pointerEventInit - pointer event state to be used when simulating the pointer event (eg. pointer id for multitouch)
         */
        simulatePointerDown(pickResult: PickingInfo, pointerEventInit?: PointerEventInit): void;
        private _processPointerDown;
        /**
         * @internal
         * @internals Boolean if delta for pointer exceeds drag movement threshold
         */
        _isPointerSwiping(): boolean;
        /**
         * Use this method to simulate a pointer up on a mesh
         * The pickResult parameter can be obtained from a scene.pick or scene.pickWithRay
         * @param pickResult - pickingInfo of the object wished to simulate pointer event on
         * @param pointerEventInit - pointer event state to be used when simulating the pointer event (eg. pointer id for multitouch)
         * @param doubleTap - indicates that the pointer up event should be considered as part of a double click (false by default)
         */
        simulatePointerUp(pickResult: PickingInfo, pointerEventInit?: PointerEventInit, doubleTap?: boolean): void;
        private _processPointerUp;
        /**
         * Gets a boolean indicating if the current pointer event is captured (meaning that the scene has already handled the pointer down)
         * @param pointerId - defines the pointer id to use in a multi-touch scenario (0 by default)
         * @returns true if the pointer was captured
         */
        isPointerCaptured(pointerId?: number): boolean;
        /**
         * Attach events to the canvas (To handle actionManagers triggers and raise onPointerMove, onPointerDown and onPointerUp
         * @param attachUp - defines if you want to attach events to pointerup
         * @param attachDown - defines if you want to attach events to pointerdown
         * @param attachMove - defines if you want to attach events to pointermove
         * @param elementToAttachTo - defines the target DOM element to attach to (will use the canvas by default)
         */
        attachControl(attachUp?: boolean, attachDown?: boolean, attachMove?: boolean, elementToAttachTo?: Nullable<HTMLElement>): void;
        /**
         * Detaches all event handlers
         */
        detachControl(): void;
        /**
         * Force the value of meshUnderPointer
         * @param mesh - defines the mesh to use
         * @param pointerId - optional pointer id when using more than one pointer. Defaults to 0
         * @param pickResult - optional pickingInfo data used to find mesh
         */
        setPointerOverMesh(mesh: Nullable<AbstractMesh>, pointerId?: number, pickResult?: Nullable<PickingInfo>): void;
        /**
         * Gets the mesh under the pointer
         * @returns a Mesh or null if no mesh is under the pointer
         */
        getPointerOverMesh(): Nullable<AbstractMesh>;
        /**
         * @param mesh - Mesh to invalidate
         * @internal
         */
        _invalidateMesh(mesh: AbstractMesh): void;
    }


    /**
     * This class can be used to get instrumentation data from a Babylon engine
     * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/optimize_your_scene#engineinstrumentation
     */
    export class EngineInstrumentation implements IDisposable {
        /**
         * Define the instrumented engine.
         */
        engine: Engine;
        private _captureGPUFrameTime;
        private _captureShaderCompilationTime;
        private _shaderCompilationTime;
        private _onBeginFrameObserver;
        private _onEndFrameObserver;
        private _onBeforeShaderCompilationObserver;
        private _onAfterShaderCompilationObserver;
        /**
         * Gets the perf counter used for GPU frame time
         */
        get gpuFrameTimeCounter(): PerfCounter;
        /**
         * Gets the GPU frame time capture status
         */
        get captureGPUFrameTime(): boolean;
        /**
         * Enable or disable the GPU frame time capture
         */
        set captureGPUFrameTime(value: boolean);
        /**
         * Gets the perf counter used for shader compilation time
         */
        get shaderCompilationTimeCounter(): PerfCounter;
        /**
         * Gets the shader compilation time capture status
         */
        get captureShaderCompilationTime(): boolean;
        /**
         * Enable or disable the shader compilation time capture
         */
        set captureShaderCompilationTime(value: boolean);
        /**
         * Instantiates a new engine instrumentation.
         * This class can be used to get instrumentation data from a Babylon engine
         * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/optimize_your_scene#engineinstrumentation
         * @param engine Defines the engine to instrument
         */
        constructor(
        /**
         * Define the instrumented engine.
         */
        engine: Engine);
        /**
         * Dispose and release associated resources.
         */
        dispose(): void;
    }




    /**
     * This class can be used to get instrumentation data from a Babylon engine
     * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/optimize_your_scene#sceneinstrumentation
     */
    export class SceneInstrumentation implements IDisposable {
        /**
         * Defines the scene to instrument
         */
        scene: Scene;
        private _captureActiveMeshesEvaluationTime;
        private _activeMeshesEvaluationTime;
        private _captureRenderTargetsRenderTime;
        private _renderTargetsRenderTime;
        private _captureFrameTime;
        private _frameTime;
        private _captureRenderTime;
        private _renderTime;
        private _captureInterFrameTime;
        private _interFrameTime;
        private _captureParticlesRenderTime;
        private _particlesRenderTime;
        private _captureSpritesRenderTime;
        private _spritesRenderTime;
        private _capturePhysicsTime;
        private _physicsTime;
        private _captureAnimationsTime;
        private _animationsTime;
        private _captureCameraRenderTime;
        private _cameraRenderTime;
        private _onBeforeActiveMeshesEvaluationObserver;
        private _onAfterActiveMeshesEvaluationObserver;
        private _onBeforeRenderTargetsRenderObserver;
        private _onAfterRenderTargetsRenderObserver;
        private _onAfterRenderObserver;
        private _onBeforeDrawPhaseObserver;
        private _onAfterDrawPhaseObserver;
        private _onBeforeAnimationsObserver;
        private _onBeforeParticlesRenderingObserver;
        private _onAfterParticlesRenderingObserver;
        private _onBeforeSpritesRenderingObserver;
        private _onAfterSpritesRenderingObserver;
        private _onBeforePhysicsObserver;
        private _onAfterPhysicsObserver;
        private _onAfterAnimationsObserver;
        private _onBeforeCameraRenderObserver;
        private _onAfterCameraRenderObserver;
        /**
         * Gets the perf counter used for active meshes evaluation time
         */
        get activeMeshesEvaluationTimeCounter(): PerfCounter;
        /**
         * Gets the active meshes evaluation time capture status
         */
        get captureActiveMeshesEvaluationTime(): boolean;
        /**
         * Enable or disable the active meshes evaluation time capture
         */
        set captureActiveMeshesEvaluationTime(value: boolean);
        /**
         * Gets the perf counter used for render targets render time
         */
        get renderTargetsRenderTimeCounter(): PerfCounter;
        /**
         * Gets the render targets render time capture status
         */
        get captureRenderTargetsRenderTime(): boolean;
        /**
         * Enable or disable the render targets render time capture
         */
        set captureRenderTargetsRenderTime(value: boolean);
        /**
         * Gets the perf counter used for particles render time
         */
        get particlesRenderTimeCounter(): PerfCounter;
        /**
         * Gets the particles render time capture status
         */
        get captureParticlesRenderTime(): boolean;
        /**
         * Enable or disable the particles render time capture
         */
        set captureParticlesRenderTime(value: boolean);
        /**
         * Gets the perf counter used for sprites render time
         */
        get spritesRenderTimeCounter(): PerfCounter;
        /**
         * Gets the sprites render time capture status
         */
        get captureSpritesRenderTime(): boolean;
        /**
         * Enable or disable the sprites render time capture
         */
        set captureSpritesRenderTime(value: boolean);
        /**
         * Gets the perf counter used for physics time
         */
        get physicsTimeCounter(): PerfCounter;
        /**
         * Gets the physics time capture status
         */
        get capturePhysicsTime(): boolean;
        /**
         * Enable or disable the physics time capture
         */
        set capturePhysicsTime(value: boolean);
        /**
         * Gets the perf counter used for animations time
         */
        get animationsTimeCounter(): PerfCounter;
        /**
         * Gets the animations time capture status
         */
        get captureAnimationsTime(): boolean;
        /**
         * Enable or disable the animations time capture
         */
        set captureAnimationsTime(value: boolean);
        /**
         * Gets the perf counter used for frame time capture
         */
        get frameTimeCounter(): PerfCounter;
        /**
         * Gets the frame time capture status
         */
        get captureFrameTime(): boolean;
        /**
         * Enable or disable the frame time capture
         */
        set captureFrameTime(value: boolean);
        /**
         * Gets the perf counter used for inter-frames time capture
         */
        get interFrameTimeCounter(): PerfCounter;
        /**
         * Gets the inter-frames time capture status
         */
        get captureInterFrameTime(): boolean;
        /**
         * Enable or disable the inter-frames time capture
         */
        set captureInterFrameTime(value: boolean);
        /**
         * Gets the perf counter used for render time capture
         */
        get renderTimeCounter(): PerfCounter;
        /**
         * Gets the render time capture status
         */
        get captureRenderTime(): boolean;
        /**
         * Enable or disable the render time capture
         */
        set captureRenderTime(value: boolean);
        /**
         * Gets the perf counter used for camera render time capture
         */
        get cameraRenderTimeCounter(): PerfCounter;
        /**
         * Gets the camera render time capture status
         */
        get captureCameraRenderTime(): boolean;
        /**
         * Enable or disable the camera render time capture
         */
        set captureCameraRenderTime(value: boolean);
        /**
         * Gets the perf counter used for draw calls
         */
        get drawCallsCounter(): PerfCounter;
        /**
         * Instantiates a new scene instrumentation.
         * This class can be used to get instrumentation data from a Babylon engine
         * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/optimize_your_scene#sceneinstrumentation
         * @param scene Defines the scene to instrument
         */
        constructor(
        /**
         * Defines the scene to instrument
         */
        scene: Scene);
        /**
         * Dispose and release associated resources.
         */
        dispose(): void;
    }


    /**
     * @internal
     **/
    export class _TimeToken {
        _startTimeQuery: Nullable<WebGLQuery>;
        _endTimeQuery: Nullable<WebGLQuery>;
        _timeElapsedQuery: Nullable<WebGLQuery>;
        _timeElapsedQueryEnded: boolean;
    }


    /**
     * Effect layer options. This helps customizing the behaviour
     * of the effect layer.
     */
    export interface IEffectLayerOptions {
        /**
         * Multiplication factor apply to the canvas size to compute the render target size
         * used to generated the objects (the smaller the faster). Default: 0.5
         */
        mainTextureRatio: number;
        /**
         * Enforces a fixed size texture to ensure effect stability across devices. Default: undefined
         */
        mainTextureFixedSize?: number;
        /**
         * Alpha blending mode used to apply the blur. Default depends of the implementation. Default: ALPHA_COMBINE
         */
        alphaBlendingMode: number;
        /**
         * The camera attached to the layer. Default: null
         */
        camera: Nullable<Camera>;
        /**
         * The rendering group to draw the layer in. Default: -1
         */
        renderingGroupId: number;
        /**
         * The type of the main texture. Default: TEXTURETYPE_UNSIGNED_INT
         */
        mainTextureType: number;
    }
    /**
     * The effect layer Helps adding post process effect blended with the main pass.
     *
     * This can be for instance use to generate glow or highlight effects on the scene.
     *
     * The effect layer class can not be used directly and is intented to inherited from to be
     * customized per effects.
     */
    export abstract class EffectLayer {
        private _vertexBuffers;
        private _indexBuffer;
        private _effectLayerOptions;
        private _mergeDrawWrapper;
        protected _scene: Scene;
        protected _engine: Engine;
        protected _maxSize: number;
        protected _mainTextureDesiredSize: ISize;
        protected _mainTexture: RenderTargetTexture;
        protected _shouldRender: boolean;
        protected _postProcesses: PostProcess[];
        protected _textures: BaseTexture[];
        protected _emissiveTextureAndColor: {
            texture: Nullable<BaseTexture>;
            color: Color4;
        };
        /**
         * The name of the layer
         */
        name: string;
        /**
         * The clear color of the texture used to generate the glow map.
         */
        neutralColor: Color4;
        /**
         * Specifies whether the highlight layer is enabled or not.
         */
        isEnabled: boolean;
        /**
         * Gets the camera attached to the layer.
         */
        get camera(): Nullable<Camera>;
        /**
         * Gets the rendering group id the layer should render in.
         */
        get renderingGroupId(): number;
        set renderingGroupId(renderingGroupId: number);
        /**
         * Specifies if the bounding boxes should be rendered normally or if they should undergo the effect of the layer
         */
        disableBoundingBoxesFromEffectLayer: boolean;
        /**
         * An event triggered when the effect layer has been disposed.
         */
        onDisposeObservable: Observable<EffectLayer>;
        /**
         * An event triggered when the effect layer is about rendering the main texture with the glowy parts.
         */
        onBeforeRenderMainTextureObservable: Observable<EffectLayer>;
        /**
         * An event triggered when the generated texture is being merged in the scene.
         */
        onBeforeComposeObservable: Observable<EffectLayer>;
        /**
         * An event triggered when the mesh is rendered into the effect render target.
         */
        onBeforeRenderMeshToEffect: Observable<AbstractMesh>;
        /**
         * An event triggered after the mesh has been rendered into the effect render target.
         */
        onAfterRenderMeshToEffect: Observable<AbstractMesh>;
        /**
         * An event triggered when the generated texture has been merged in the scene.
         */
        onAfterComposeObservable: Observable<EffectLayer>;
        /**
         * An event triggered when the effect layer changes its size.
         */
        onSizeChangedObservable: Observable<EffectLayer>;
        /**
         * Gets the main texture where the effect is rendered
         */
        get mainTexture(): RenderTargetTexture;
        /**
         * @internal
         */
        static _SceneComponentInitialization: (scene: Scene) => void;
        private _materialForRendering;
        /**
         * Sets a specific material to be used to render a mesh/a list of meshes in the layer
         * @param mesh mesh or array of meshes
         * @param material material to use by the layer when rendering the mesh(es). If undefined is passed, the specific material created by the layer will be used.
         */
        setMaterialForRendering(mesh: AbstractMesh | AbstractMesh[], material?: Material): void;
        /**
         * Instantiates a new effect Layer and references it in the scene.
         * @param name The name of the layer
         * @param scene The scene to use the layer in
         */
        constructor(
        /** The Friendly of the effect in the scene */
        name: string, scene?: Scene);
        /**
         * Get the effect name of the layer.
         * @returns The effect name
         */
        abstract getEffectName(): string;
        /**
         * Checks for the readiness of the element composing the layer.
         * @param subMesh the mesh to check for
         * @param useInstances specify whether or not to use instances to render the mesh
         * @returns true if ready otherwise, false
         */
        abstract isReady(subMesh: SubMesh, useInstances: boolean): boolean;
        /**
         * Returns whether or not the layer needs stencil enabled during the mesh rendering.
         * @returns true if the effect requires stencil during the main canvas render pass.
         */
        abstract needStencil(): boolean;
        /**
         * Create the merge effect. This is the shader use to blit the information back
         * to the main canvas at the end of the scene rendering.
         * @returns The effect containing the shader used to merge the effect on the  main canvas
         */
        protected abstract _createMergeEffect(): Effect;
        /**
         * Creates the render target textures and post processes used in the effect layer.
         */
        protected abstract _createTextureAndPostProcesses(): void;
        /**
         * Implementation specific of rendering the generating effect on the main canvas.
         * @param effect The effect used to render through
         * @param renderNum Index of the _internalRender call (0 for the first time _internalRender is called, 1 for the second time, etc. _internalRender is called the number of times returned by _numInternalDraws())
         */
        protected abstract _internalRender(effect: Effect, renderIndex: number): void;
        /**
         * Sets the required values for both the emissive texture and and the main color.
         */
        protected abstract _setEmissiveTextureAndColor(mesh: Mesh, subMesh: SubMesh, material: Material): void;
        /**
         * Free any resources and references associated to a mesh.
         * Internal use
         * @param mesh The mesh to free.
         */
        abstract _disposeMesh(mesh: Mesh): void;
        /**
         * Serializes this layer (Glow or Highlight for example)
         * @returns a serialized layer object
         */
        abstract serialize?(): any;
        /**
         * Number of times _internalRender will be called. Some effect layers need to render the mesh several times, so they should override this method with the number of times the mesh should be rendered
         * @returns Number of times a mesh must be rendered in the layer
         */
        protected _numInternalDraws(): number;
        /**
         * Initializes the effect layer with the required options.
         * @param options Sets of none mandatory options to use with the layer (see IEffectLayerOptions for more information)
         */
        protected _init(options: Partial<IEffectLayerOptions>): void;
        /**
         * Generates the index buffer of the full screen quad blending to the main canvas.
         */
        private _generateIndexBuffer;
        /**
         * Generates the vertex buffer of the full screen quad blending to the main canvas.
         */
        private _generateVertexBuffer;
        /**
         * Sets the main texture desired size which is the closest power of two
         * of the engine canvas size.
         */
        private _setMainTextureSize;
        /**
         * Creates the main texture for the effect layer.
         */
        protected _createMainTexture(): void;
        /**
         * Adds specific effects defines.
         * @param defines The defines to add specifics to.
         */
        protected _addCustomEffectDefines(defines: string[]): void;
        /**
         * Checks for the readiness of the element composing the layer.
         * @param subMesh the mesh to check for
         * @param useInstances specify whether or not to use instances to render the mesh
         * @param emissiveTexture the associated emissive texture used to generate the glow
         * @returns true if ready otherwise, false
         */
        protected _isReady(subMesh: SubMesh, useInstances: boolean, emissiveTexture: Nullable<BaseTexture>): boolean;
        /**
         * Renders the glowing part of the scene by blending the blurred glowing meshes on top of the rendered scene.
         */
        render(): void;
        /**
         * Determine if a given mesh will be used in the current effect.
         * @param mesh mesh to test
         * @returns true if the mesh will be used
         */
        hasMesh(mesh: AbstractMesh): boolean;
        /**
         * Returns true if the layer contains information to display, otherwise false.
         * @returns true if the glow layer should be rendered
         */
        shouldRender(): boolean;
        /**
         * Returns true if the mesh should render, otherwise false.
         * @param mesh The mesh to render
         * @returns true if it should render otherwise false
         */
        protected _shouldRenderMesh(mesh: AbstractMesh): boolean;
        /**
         * Returns true if the mesh can be rendered, otherwise false.
         * @param mesh The mesh to render
         * @param material The material used on the mesh
         * @returns true if it can be rendered otherwise false
         */
        protected _canRenderMesh(mesh: AbstractMesh, material: Material): boolean;
        /**
         * Returns true if the mesh should render, otherwise false.
         * @returns true if it should render otherwise false
         */
        protected _shouldRenderEmissiveTextureForMesh(): boolean;
        /**
         * Renders the submesh passed in parameter to the generation map.
         * @param subMesh
         * @param enableAlphaMode
         */
        protected _renderSubMesh(subMesh: SubMesh, enableAlphaMode?: boolean): void;
        /**
         * Defines whether the current material of the mesh should be use to render the effect.
         * @param mesh defines the current mesh to render
         */
        protected _useMeshMaterial(mesh: AbstractMesh): boolean;
        /**
         * Rebuild the required buffers.
         * @internal Internal use only.
         */
        _rebuild(): void;
        /**
         * Dispose only the render target textures and post process.
         */
        private _disposeTextureAndPostProcesses;
        /**
         * Dispose the highlight layer and free resources.
         */
        dispose(): void;
        /**
         * Gets the class name of the effect layer
         * @returns the string with the class name of the effect layer
         */
        getClassName(): string;
        /**
         * Creates an effect layer from parsed effect layer data
         * @param parsedEffectLayer defines effect layer data
         * @param scene defines the current scene
         * @param rootUrl defines the root URL containing the effect layer information
         * @returns a parsed effect Layer
         */
        static Parse(parsedEffectLayer: any, scene: Scene, rootUrl: string): EffectLayer;
    }


        interface AbstractScene {
            /**
             * The list of effect layers (highlights/glow) added to the scene
             * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/highlightLayer
             * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/glowLayer
             */
            effectLayers: Array<EffectLayer>;
            /**
             * Removes the given effect layer from this scene.
             * @param toRemove defines the effect layer to remove
             * @returns the index of the removed effect layer
             */
            removeEffectLayer(toRemove: EffectLayer): number;
            /**
             * Adds the given effect layer to this scene
             * @param newEffectLayer defines the effect layer to add
             */
            addEffectLayer(newEffectLayer: EffectLayer): void;
        }
    /**
     * Defines the layer scene component responsible to manage any effect layers
     * in a given scene.
     */
    export class EffectLayerSceneComponent implements ISceneSerializableComponent {
        /**
         * The component name helpful to identify the component in the list of scene components.
         */
        readonly name = "EffectLayer";
        /**
         * The scene the component belongs to.
         */
        scene: Scene;
        private _engine;
        private _renderEffects;
        private _needStencil;
        private _previousStencilState;
        /**
         * Creates a new instance of the component for the given scene
         * @param scene Defines the scene to register the component in
         */
        constructor(scene?: Scene);
        /**
         * Registers the component in a given scene
         */
        register(): void;
        /**
         * Rebuilds the elements related to this component in case of
         * context lost for instance.
         */
        rebuild(): void;
        /**
         * Serializes the component data to the specified json object
         * @param serializationObject The object to serialize to
         */
        serialize(serializationObject: any): void;
        /**
         * Adds all the elements from the container to the scene
         * @param container the container holding the elements
         */
        addFromContainer(container: AbstractScene): void;
        /**
         * Removes all the elements in the container from the scene
         * @param container contains the elements to remove
         * @param dispose if the removed element should be disposed (default: false)
         */
        removeFromContainer(container: AbstractScene, dispose?: boolean): void;
        /**
         * Disposes the component and the associated resources.
         */
        dispose(): void;
        private _isReadyForMesh;
        private _renderMainTexture;
        private _setStencil;
        private _setStencilBack;
        private _draw;
        private _drawCamera;
        private _drawRenderingGroup;
    }


        interface AbstractScene {
            /**
             * Return the first glow layer of the scene with a given name.
             * @param name The name of the glow layer to look for.
             * @returns The glow layer if found otherwise null.
             */
            getGlowLayerByName(name: string): Nullable<GlowLayer>;
        }
    /**
     * Glow layer options. This helps customizing the behaviour
     * of the glow layer.
     */
    export interface IGlowLayerOptions {
        /**
         * Multiplication factor apply to the canvas size to compute the render target size
         * used to generated the glowing objects (the smaller the faster). Default: 0.5
         */
        mainTextureRatio: number;
        /**
         * Enforces a fixed size texture to ensure resize independent blur. Default: undefined
         */
        mainTextureFixedSize?: number;
        /**
         * How big is the kernel of the blur texture. Default: 32
         */
        blurKernelSize: number;
        /**
         * The camera attached to the layer. Default: null
         */
        camera: Nullable<Camera>;
        /**
         * Enable MSAA by choosing the number of samples. Default: 1
         */
        mainTextureSamples?: number;
        /**
         * The rendering group to draw the layer in. Default: -1
         */
        renderingGroupId: number;
        /**
         * Forces the merge step to be done in ldr (clamp values > 1). Default: false
         */
        ldrMerge?: boolean;
        /**
         * Defines the blend mode used by the merge. Default: ALPHA_ADD
         */
        alphaBlendingMode?: number;
        /**
         * The type of the main texture. Default: TEXTURETYPE_UNSIGNED_INT
         */
        mainTextureType: number;
    }
    /**
     * The glow layer Helps adding a glow effect around the emissive parts of a mesh.
     *
     * Once instantiated in a scene, by default, all the emissive meshes will glow.
     *
     * Documentation: https://doc.babylonjs.com/features/featuresDeepDive/mesh/glowLayer
     */
    export class GlowLayer extends EffectLayer {
        /**
         * Effect Name of the layer.
         */
        static readonly EffectName = "GlowLayer";
        /**
         * The default blur kernel size used for the glow.
         */
        static DefaultBlurKernelSize: number;
        /**
         * The default texture size ratio used for the glow.
         */
        static DefaultTextureRatio: number;
        /**
         * Sets the kernel size of the blur.
         */
        set blurKernelSize(value: number);
        /**
         * Gets the kernel size of the blur.
         */
        get blurKernelSize(): number;
        /**
         * Sets the glow intensity.
         */
        set intensity(value: number);
        /**
         * Gets the glow intensity.
         */
        get intensity(): number;
        private _options;
        private _intensity;
        private _horizontalBlurPostprocess1;
        private _verticalBlurPostprocess1;
        private _horizontalBlurPostprocess2;
        private _verticalBlurPostprocess2;
        private _blurTexture1;
        private _blurTexture2;
        private _postProcesses1;
        private _postProcesses2;
        private _includedOnlyMeshes;
        private _excludedMeshes;
        private _meshesUsingTheirOwnMaterials;
        /**
         * Callback used to let the user override the color selection on a per mesh basis
         */
        customEmissiveColorSelector: (mesh: Mesh, subMesh: SubMesh, material: Material, result: Color4) => void;
        /**
         * Callback used to let the user override the texture selection on a per mesh basis
         */
        customEmissiveTextureSelector: (mesh: Mesh, subMesh: SubMesh, material: Material) => Texture;
        /**
         * Instantiates a new glow Layer and references it to the scene.
         * @param name The name of the layer
         * @param scene The scene to use the layer in
         * @param options Sets of none mandatory options to use with the layer (see IGlowLayerOptions for more information)
         */
        constructor(name: string, scene?: Scene, options?: Partial<IGlowLayerOptions>);
        /**
         * Get the effect name of the layer.
         * @returns The effect name
         */
        getEffectName(): string;
        /**
         * Create the merge effect. This is the shader use to blit the information back
         * to the main canvas at the end of the scene rendering.
         */
        protected _createMergeEffect(): Effect;
        /**
         * Creates the render target textures and post processes used in the glow layer.
         */
        protected _createTextureAndPostProcesses(): void;
        /**
         * @returns The blur kernel size used by the glow.
         * Note: The value passed in the options is divided by 2 for back compatibility.
         */
        private _getEffectiveBlurKernelSize;
        /**
         * Checks for the readiness of the element composing the layer.
         * @param subMesh the mesh to check for
         * @param useInstances specify whether or not to use instances to render the mesh
         * @returns true if ready otherwise, false
         */
        isReady(subMesh: SubMesh, useInstances: boolean): boolean;
        /**
         * Returns whether or not the layer needs stencil enabled during the mesh rendering.
         */
        needStencil(): boolean;
        /**
         * Returns true if the mesh can be rendered, otherwise false.
         * @param mesh The mesh to render
         * @param material The material used on the mesh
         * @returns true if it can be rendered otherwise false
         */
        protected _canRenderMesh(mesh: AbstractMesh, material: Material): boolean;
        /**
         * Implementation specific of rendering the generating effect on the main canvas.
         * @param effect The effect used to render through
         */
        protected _internalRender(effect: Effect): void;
        /**
         * Sets the required values for both the emissive texture and and the main color.
         * @param mesh
         * @param subMesh
         * @param material
         */
        protected _setEmissiveTextureAndColor(mesh: Mesh, subMesh: SubMesh, material: Material): void;
        /**
         * Returns true if the mesh should render, otherwise false.
         * @param mesh The mesh to render
         * @returns true if it should render otherwise false
         */
        protected _shouldRenderMesh(mesh: Mesh): boolean;
        /**
         * Adds specific effects defines.
         * @param defines The defines to add specifics to.
         */
        protected _addCustomEffectDefines(defines: string[]): void;
        /**
         * Add a mesh in the exclusion list to prevent it to impact or being impacted by the glow layer.
         * @param mesh The mesh to exclude from the glow layer
         */
        addExcludedMesh(mesh: Mesh): void;
        /**
         * Remove a mesh from the exclusion list to let it impact or being impacted by the glow layer.
         * @param mesh The mesh to remove
         */
        removeExcludedMesh(mesh: Mesh): void;
        /**
         * Add a mesh in the inclusion list to impact or being impacted by the glow layer.
         * @param mesh The mesh to include in the glow layer
         */
        addIncludedOnlyMesh(mesh: Mesh): void;
        /**
         * Remove a mesh from the Inclusion list to prevent it to impact or being impacted by the glow layer.
         * @param mesh The mesh to remove
         */
        removeIncludedOnlyMesh(mesh: Mesh): void;
        /**
         * Determine if a given mesh will be used in the glow layer
         * @param mesh The mesh to test
         * @returns true if the mesh will be highlighted by the current glow layer
         */
        hasMesh(mesh: AbstractMesh): boolean;
        /**
         * Defines whether the current material of the mesh should be use to render the effect.
         * @param mesh defines the current mesh to render
         */
        protected _useMeshMaterial(mesh: AbstractMesh): boolean;
        /**
         * Add a mesh to be rendered through its own material and not with emissive only.
         * @param mesh The mesh for which we need to use its material
         */
        referenceMeshToUseItsOwnMaterial(mesh: AbstractMesh): void;
        /**
         * Remove a mesh from being rendered through its own material and not with emissive only.
         * @param mesh The mesh for which we need to not use its material
         */
        unReferenceMeshFromUsingItsOwnMaterial(mesh: AbstractMesh): void;
        /**
         * Free any resources and references associated to a mesh.
         * Internal use
         * @param mesh The mesh to free.
         * @internal
         */
        _disposeMesh(mesh: Mesh): void;
        /**
         * Gets the class name of the effect layer
         * @returns the string with the class name of the effect layer
         */
        getClassName(): string;
        /**
         * Serializes this glow layer
         * @returns a serialized glow layer object
         */
        serialize(): any;
        /**
         * Creates a Glow Layer from parsed glow layer data
         * @param parsedGlowLayer defines glow layer data
         * @param scene defines the current scene
         * @param rootUrl defines the root URL containing the glow layer information
         * @returns a parsed Glow Layer
         */
        static Parse(parsedGlowLayer: any, scene: Scene, rootUrl: string): GlowLayer;
    }


        interface AbstractScene {
            /**
             * Return a the first highlight layer of the scene with a given name.
             * @param name The name of the highlight layer to look for.
             * @returns The highlight layer if found otherwise null.
             */
            getHighlightLayerByName(name: string): Nullable<HighlightLayer>;
        }
    /**
     * Highlight layer options. This helps customizing the behaviour
     * of the highlight layer.
     */
    export interface IHighlightLayerOptions {
        /**
         * Multiplication factor apply to the canvas size to compute the render target size
         * used to generated the glowing objects (the smaller the faster). Default: 0.5
         */
        mainTextureRatio: number;
        /**
         * Enforces a fixed size texture to ensure resize independent blur. Default: undefined
         */
        mainTextureFixedSize?: number;
        /**
         * Multiplication factor apply to the main texture size in the first step of the blur to reduce the size
         * of the picture to blur (the smaller the faster). Default: 0.5
         */
        blurTextureSizeRatio: number;
        /**
         * How big in texel of the blur texture is the vertical blur. Default: 1
         */
        blurVerticalSize: number;
        /**
         * How big in texel of the blur texture is the horizontal blur. Default: 1
         */
        blurHorizontalSize: number;
        /**
         * Alpha blending mode used to apply the blur.  Default: ALPHA_COMBINE
         */
        alphaBlendingMode: number;
        /**
         * The camera attached to the layer. Default: null
         */
        camera: Nullable<Camera>;
        /**
         * Should we display highlight as a solid stroke? Default: false
         */
        isStroke?: boolean;
        /**
         * The rendering group to draw the layer in. Default: -1
         */
        renderingGroupId: number;
        /**
         * The type of the main texture. Default: TEXTURETYPE_UNSIGNED_INT
         */
        mainTextureType: number;
    }
    /**
     * The highlight layer Helps adding a glow effect around a mesh.
     *
     * Once instantiated in a scene, simply use the addMesh or removeMesh method to add or remove
     * glowy meshes to your scene.
     *
     * !!! THIS REQUIRES AN ACTIVE STENCIL BUFFER ON THE CANVAS !!!
     */
    export class HighlightLayer extends EffectLayer {
        name: string;
        /**
         * Effect Name of the highlight layer.
         */
        static readonly EffectName = "HighlightLayer";
        /**
         * The neutral color used during the preparation of the glow effect.
         * This is black by default as the blend operation is a blend operation.
         */
        static NeutralColor: Color4;
        /**
         * Stencil value used for glowing meshes.
         */
        static GlowingMeshStencilReference: number;
        /**
         * Stencil value used for the other meshes in the scene.
         */
        static NormalMeshStencilReference: number;
        /**
         * Specifies whether or not the inner glow is ACTIVE in the layer.
         */
        innerGlow: boolean;
        /**
         * Specifies whether or not the outer glow is ACTIVE in the layer.
         */
        outerGlow: boolean;
        /**
         * Specifies the horizontal size of the blur.
         */
        set blurHorizontalSize(value: number);
        /**
         * Specifies the vertical size of the blur.
         */
        set blurVerticalSize(value: number);
        /**
         * Gets the horizontal size of the blur.
         */
        get blurHorizontalSize(): number;
        /**
         * Gets the vertical size of the blur.
         */
        get blurVerticalSize(): number;
        /**
         * An event triggered when the highlight layer is being blurred.
         */
        onBeforeBlurObservable: Observable<HighlightLayer>;
        /**
         * An event triggered when the highlight layer has been blurred.
         */
        onAfterBlurObservable: Observable<HighlightLayer>;
        private _instanceGlowingMeshStencilReference;
        private _options;
        private _downSamplePostprocess;
        private _horizontalBlurPostprocess;
        private _verticalBlurPostprocess;
        private _blurTexture;
        private _meshes;
        private _excludedMeshes;
        /**
         * Instantiates a new highlight Layer and references it to the scene..
         * @param name The name of the layer
         * @param scene The scene to use the layer in
         * @param options Sets of none mandatory options to use with the layer (see IHighlightLayerOptions for more information)
         */
        constructor(name: string, scene?: Scene, options?: Partial<IHighlightLayerOptions>);
        /**
         * Get the effect name of the layer.
         * @returns The effect name
         */
        getEffectName(): string;
        protected _numInternalDraws(): number;
        /**
         * Create the merge effect. This is the shader use to blit the information back
         * to the main canvas at the end of the scene rendering.
         */
        protected _createMergeEffect(): Effect;
        /**
         * Creates the render target textures and post processes used in the highlight layer.
         */
        protected _createTextureAndPostProcesses(): void;
        /**
         * Returns whether or not the layer needs stencil enabled during the mesh rendering.
         */
        needStencil(): boolean;
        /**
         * Checks for the readiness of the element composing the layer.
         * @param subMesh the mesh to check for
         * @param useInstances specify whether or not to use instances to render the mesh
         * @returns true if ready otherwise, false
         */
        isReady(subMesh: SubMesh, useInstances: boolean): boolean;
        /**
         * Implementation specific of rendering the generating effect on the main canvas.
         * @param effect The effect used to render through
         * @param renderIndex
         */
        protected _internalRender(effect: Effect, renderIndex: number): void;
        /**
         * Returns true if the layer contains information to display, otherwise false.
         */
        shouldRender(): boolean;
        /**
         * Returns true if the mesh should render, otherwise false.
         * @param mesh The mesh to render
         * @returns true if it should render otherwise false
         */
        protected _shouldRenderMesh(mesh: Mesh): boolean;
        /**
         * Returns true if the mesh can be rendered, otherwise false.
         * @param mesh The mesh to render
         * @param material The material used on the mesh
         * @returns true if it can be rendered otherwise false
         */
        protected _canRenderMesh(mesh: AbstractMesh, material: Material): boolean;
        /**
         * Adds specific effects defines.
         * @param defines The defines to add specifics to.
         */
        protected _addCustomEffectDefines(defines: string[]): void;
        /**
         * Sets the required values for both the emissive texture and and the main color.
         * @param mesh
         * @param subMesh
         * @param material
         */
        protected _setEmissiveTextureAndColor(mesh: Mesh, subMesh: SubMesh, material: Material): void;
        /**
         * Add a mesh in the exclusion list to prevent it to impact or being impacted by the highlight layer.
         * @param mesh The mesh to exclude from the highlight layer
         */
        addExcludedMesh(mesh: Mesh): void;
        /**
         * Remove a mesh from the exclusion list to let it impact or being impacted by the highlight layer.
         * @param mesh The mesh to highlight
         */
        removeExcludedMesh(mesh: Mesh): void;
        /**
         * Determine if a given mesh will be highlighted by the current HighlightLayer
         * @param mesh mesh to test
         * @returns true if the mesh will be highlighted by the current HighlightLayer
         */
        hasMesh(mesh: AbstractMesh): boolean;
        /**
         * Add a mesh in the highlight layer in order to make it glow with the chosen color.
         * @param mesh The mesh to highlight
         * @param color The color of the highlight
         * @param glowEmissiveOnly Extract the glow from the emissive texture
         */
        addMesh(mesh: Mesh, color: Color3, glowEmissiveOnly?: boolean): void;
        /**
         * Remove a mesh from the highlight layer in order to make it stop glowing.
         * @param mesh The mesh to highlight
         */
        removeMesh(mesh: Mesh): void;
        /**
         * Remove all the meshes currently referenced in the highlight layer
         */
        removeAllMeshes(): void;
        /**
         * Force the stencil to the normal expected value for none glowing parts
         * @param mesh
         */
        private _defaultStencilReference;
        /**
         * Free any resources and references associated to a mesh.
         * Internal use
         * @param mesh The mesh to free.
         * @internal
         */
        _disposeMesh(mesh: Mesh): void;
        /**
         * Dispose the highlight layer and free resources.
         */
        dispose(): void;
        /**
         * Gets the class name of the effect layer
         * @returns the string with the class name of the effect layer
         */
        getClassName(): string;
        /**
         * Serializes this Highlight layer
         * @returns a serialized Highlight layer object
         */
        serialize(): any;
        /**
         * Creates a Highlight layer from parsed Highlight layer data
         * @param parsedHightlightLayer defines the Highlight layer data
         * @param scene defines the current scene
         * @param rootUrl defines the root URL containing the Highlight layer information
         * @returns a parsed Highlight layer
         */
        static Parse(parsedHightlightLayer: any, scene: Scene, rootUrl: string): HighlightLayer;
    }




    /**
     * This represents a full screen 2d layer.
     * This can be useful to display a picture in the  background of your scene for instance.
     * @see https://www.babylonjs-playground.com/#08A2BS#1
     */
    export class Layer {
        /**
         * Define the name of the layer.
         */
        name: string;
        /**
         * Define the texture the layer should display.
         */
        texture: Nullable<BaseTexture>;
        /**
         * Is the layer in background or foreground.
         */
        isBackground: boolean;
        private _applyPostProcess;
        /**
         * Determines if the layer is drawn before (true) or after (false) post-processing.
         * If the layer is background, it is always before.
         */
        set applyPostProcess(value: boolean);
        get applyPostProcess(): boolean;
        /**
         * Define the color of the layer (instead of texture).
         */
        color: Color4;
        /**
         * Define the scale of the layer in order to zoom in out of the texture.
         */
        scale: Vector2;
        /**
         * Define an offset for the layer in order to shift the texture.
         */
        offset: Vector2;
        /**
         * Define the alpha blending mode used in the layer in case the texture or color has an alpha.
         */
        alphaBlendingMode: number;
        /**
         * Define if the layer should alpha test or alpha blend with the rest of the scene.
         * Alpha test will not mix with the background color in case of transparency.
         * It will either use the texture color or the background depending on the alpha value of the current pixel.
         */
        alphaTest: boolean;
        /**
         * Define a mask to restrict the layer to only some of the scene cameras.
         */
        layerMask: number;
        /**
         * Define the list of render target the layer is visible into.
         */
        renderTargetTextures: RenderTargetTexture[];
        /**
         * Define if the layer is only used in renderTarget or if it also
         * renders in the main frame buffer of the canvas.
         */
        renderOnlyInRenderTargetTextures: boolean;
        /**
         * Define if the layer is enabled (ie. should be displayed). Default: true
         */
        isEnabled: boolean;
        private _scene;
        private _vertexBuffers;
        private _indexBuffer;
        private _drawWrapper;
        private _previousDefines;
        /**
         * An event triggered when the layer is disposed.
         */
        onDisposeObservable: Observable<Layer>;
        private _onDisposeObserver;
        /**
         * Back compatibility with callback before the onDisposeObservable existed.
         * The set callback will be triggered when the layer has been disposed.
         */
        set onDispose(callback: () => void);
        /**
         * An event triggered before rendering the scene
         */
        onBeforeRenderObservable: Observable<Layer>;
        private _onBeforeRenderObserver;
        /**
         * Back compatibility with callback before the onBeforeRenderObservable existed.
         * The set callback will be triggered just before rendering the layer.
         */
        set onBeforeRender(callback: () => void);
        /**
         * An event triggered after rendering the scene
         */
        onAfterRenderObservable: Observable<Layer>;
        private _onAfterRenderObserver;
        /**
         * Back compatibility with callback before the onAfterRenderObservable existed.
         * The set callback will be triggered just after rendering the layer.
         */
        set onAfterRender(callback: () => void);
        /**
         * Instantiates a new layer.
         * This represents a full screen 2d layer.
         * This can be useful to display a picture in the  background of your scene for instance.
         * @see https://www.babylonjs-playground.com/#08A2BS#1
         * @param name Define the name of the layer in the scene
         * @param imgUrl Define the url of the texture to display in the layer
         * @param scene Define the scene the layer belongs to
         * @param isBackground Defines whether the layer is displayed in front or behind the scene
         * @param color Defines a color for the layer
         */
        constructor(
        /**
         * Define the name of the layer.
         */
        name: string, imgUrl: Nullable<string>, scene: Nullable<Scene>, isBackground?: boolean, color?: Color4);
        private _createIndexBuffer;
        /** @internal */
        _rebuild(): void;
        /**
         * Renders the layer in the scene.
         */
        render(): void;
        /**
         * Disposes and releases the associated resources.
         */
        dispose(): void;
    }


        interface AbstractScene {
            /**
             * The list of layers (background and foreground) of the scene
             */
            layers: Array<Layer>;
        }
    /**
     * Defines the layer scene component responsible to manage any layers
     * in a given scene.
     */
    export class LayerSceneComponent implements ISceneComponent {
        /**
         * The component name helpful to identify the component in the list of scene components.
         */
        readonly name = "Layer";
        /**
         * The scene the component belongs to.
         */
        scene: Scene;
        private _engine;
        /**
         * Creates a new instance of the component for the given scene
         * @param scene Defines the scene to register the component in
         */
        constructor(scene?: Scene);
        /**
         * Registers the component in a given scene
         */
        register(): void;
        /**
         * Rebuilds the elements related to this component in case of
         * context lost for instance.
         */
        rebuild(): void;
        /**
         * Disposes the component and the associated resources.
         */
        dispose(): void;
        private _draw;
        private _drawCameraPredicate;
        private _drawCameraBackground;
        private _drawCameraForegroundWithPostProcessing;
        private _drawCameraForegroundWithoutPostProcessing;
        private _drawRenderTargetPredicate;
        private _drawRenderTargetBackground;
        private _drawRenderTargetForegroundWithPostProcessing;
        private _drawRenderTargetForegroundWithoutPostProcessing;
        /**
         * Adds all the elements from the container to the scene
         * @param container the container holding the elements
         */
        addFromContainer(container: AbstractScene): void;
        /**
         * Removes all the elements in the container from the scene
         * @param container contains the elements to remove
         * @param dispose if the removed element should be disposed (default: false)
         */
        removeFromContainer(container: AbstractScene, dispose?: boolean): void;
    }




    /**
     * This represents one of the lens effect in a `lensFlareSystem`.
     * It controls one of the individual texture used in the effect.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/environment/lenseFlare
     */
    export class LensFlare {
        /**
         * Define the size of the lens flare in the system (a floating value between 0 and 1)
         */
        size: number;
        /**
         * Define the position of the lens flare in the system. (a floating value between -1 and 1). A value of 0 is located on the emitter. A value greater than 0 is beyond the emitter and a value lesser than 0 is behind.
         */
        position: number;
        /**
         * Define the lens color.
         */
        color: Color3;
        /**
         * Define the lens texture.
         */
        texture: Nullable<Texture>;
        /**
         * Define the alpha mode to render this particular lens.
         */
        alphaMode: number;
        /** @internal */
        _drawWrapper: DrawWrapper;
        private _system;
        /**
         * Creates a new Lens Flare.
         * This represents one of the lens effect in a `lensFlareSystem`.
         * It controls one of the individual texture used in the effect.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/environment/lenseFlare
         * @param size Define the size of the lens flare (a floating value between 0 and 1)
         * @param position Define the position of the lens flare in the system. (a floating value between -1 and 1). A value of 0 is located on the emitter. A value greater than 0 is beyond the emitter and a value lesser than 0 is behind.
         * @param color Define the lens color
         * @param imgUrl Define the lens texture url
         * @param system Define the `lensFlareSystem` this flare is part of
         * @returns The newly created Lens Flare
         */
        static AddFlare(size: number, position: number, color: Color3, imgUrl: string, system: LensFlareSystem): LensFlare;
        /**
         * Instantiates a new Lens Flare.
         * This represents one of the lens effect in a `lensFlareSystem`.
         * It controls one of the individual texture used in the effect.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/environment/lenseFlare
         * @param size Define the size of the lens flare in the system (a floating value between 0 and 1)
         * @param position Define the position of the lens flare in the system. (a floating value between -1 and 1). A value of 0 is located on the emitter. A value greater than 0 is beyond the emitter and a value lesser than 0 is behind.
         * @param color Define the lens color
         * @param imgUrl Define the lens texture url
         * @param system Define the `lensFlareSystem` this flare is part of
         */
        constructor(
        /**
         * Define the size of the lens flare in the system (a floating value between 0 and 1)
         */
        size: number, 
        /**
         * Define the position of the lens flare in the system. (a floating value between -1 and 1). A value of 0 is located on the emitter. A value greater than 0 is beyond the emitter and a value lesser than 0 is behind.
         */
        position: number, color: Color3, imgUrl: string, system: LensFlareSystem);
        /**
         * Dispose and release the lens flare with its associated resources.
         */
        dispose(): void;
    }


    /**
     * This represents a Lens Flare System or the shiny effect created by the light reflection on the  camera lenses.
     * It is usually composed of several `lensFlare`.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/environment/lenseFlare
     */
    export class LensFlareSystem {
        /**
         * Define the name of the lens flare system
         */
        name: string;
        /**
         * List of lens flares used in this system.
         */
        lensFlares: LensFlare[];
        /**
         * Define a limit from the border the lens flare can be visible.
         */
        borderLimit: number;
        /**
         * Define a viewport border we do not want to see the lens flare in.
         */
        viewportBorder: number;
        /**
         * Define a predicate which could limit the list of meshes able to occlude the effect.
         */
        meshesSelectionPredicate: (mesh: AbstractMesh) => boolean;
        /**
         * Restricts the rendering of the effect to only the camera rendering this layer mask.
         */
        layerMask: number;
        /** Gets the scene */
        get scene(): Scene;
        /**
         * Define the id of the lens flare system in the scene.
         * (equal to name by default)
         */
        id: string;
        private _scene;
        private _emitter;
        private _vertexBuffers;
        private _indexBuffer;
        private _positionX;
        private _positionY;
        private _isEnabled;
        /**
         * @internal
         */
        static _SceneComponentInitialization: (scene: Scene) => void;
        /**
         * Instantiates a lens flare system.
         * This represents a Lens Flare System or the shiny effect created by the light reflection on the  camera lenses.
         * It is usually composed of several `lensFlare`.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/environment/lenseFlare
         * @param name Define the name of the lens flare system in the scene
         * @param emitter Define the source (the emitter) of the lens flares (it can be a camera, a light or a mesh).
         * @param scene Define the scene the lens flare system belongs to
         */
        constructor(
        /**
         * Define the name of the lens flare system
         */
        name: string, emitter: any, scene: Scene);
        private _createIndexBuffer;
        /**
         * Define if the lens flare system is enabled.
         */
        get isEnabled(): boolean;
        set isEnabled(value: boolean);
        /**
         * Get the scene the effects belongs to.
         * @returns the scene holding the lens flare system
         */
        getScene(): Scene;
        /**
         * Get the emitter of the lens flare system.
         * It defines the source of the lens flares (it can be a camera, a light or a mesh).
         * @returns the emitter of the lens flare system
         */
        getEmitter(): any;
        /**
         * Set the emitter of the lens flare system.
         * It defines the source of the lens flares (it can be a camera, a light or a mesh).
         * @param newEmitter Define the new emitter of the system
         */
        setEmitter(newEmitter: any): void;
        /**
         * Get the lens flare system emitter position.
         * The emitter defines the source of the lens flares (it can be a camera, a light or a mesh).
         * @returns the position
         */
        getEmitterPosition(): Vector3;
        /**
         * @internal
         */
        computeEffectivePosition(globalViewport: Viewport): boolean;
        /** @internal */
        _isVisible(): boolean;
        /**
         * @internal
         */
        render(): boolean;
        /**
         * Rebuilds the lens flare system
         */
        rebuild(): void;
        /**
         * Dispose and release the lens flare with its associated resources.
         */
        dispose(): void;
        /**
         * Parse a lens flare system from a JSON representation
         * @param parsedLensFlareSystem Define the JSON to parse
         * @param scene Define the scene the parsed system should be instantiated in
         * @param rootUrl Define the rootUrl of the load sequence to easily find a load relative dependencies such as textures
         * @returns the parsed system
         */
        static Parse(parsedLensFlareSystem: any, scene: Scene, rootUrl: string): LensFlareSystem;
        /**
         * Serialize the current Lens Flare System into a JSON representation.
         * @returns the serialized JSON
         */
        serialize(): any;
    }


        interface AbstractScene {
            /**
             * The list of lens flare system added to the scene
             * @see https://doc.babylonjs.com/features/featuresDeepDive/environment/lenseFlare
             */
            lensFlareSystems: Array<LensFlareSystem>;
            /**
             * Removes the given lens flare system from this scene.
             * @param toRemove The lens flare system to remove
             * @returns The index of the removed lens flare system
             */
            removeLensFlareSystem(toRemove: LensFlareSystem): number;
            /**
             * Adds the given lens flare system to this scene
             * @param newLensFlareSystem The lens flare system to add
             */
            addLensFlareSystem(newLensFlareSystem: LensFlareSystem): void;
            /**
             * Gets a lens flare system using its name
             * @param name defines the name to look for
             * @returns the lens flare system or null if not found
             */
            getLensFlareSystemByName(name: string): Nullable<LensFlareSystem>;
            /**
             * Gets a lens flare system using its Id
             * @param id defines the Id to look for
             * @returns the lens flare system or null if not found
             * @deprecated Please use getLensFlareSystemById instead
             */
            getLensFlareSystemByID(id: string): Nullable<LensFlareSystem>;
            /**
             * Gets a lens flare system using its Id
             * @param id defines the Id to look for
             * @returns the lens flare system or null if not found
             */
            getLensFlareSystemById(id: string): Nullable<LensFlareSystem>;
        }
    /**
     * Defines the lens flare scene component responsible to manage any lens flares
     * in a given scene.
     */
    export class LensFlareSystemSceneComponent implements ISceneSerializableComponent {
        /**
         * The component name helpful to identify the component in the list of scene components.
         */
        readonly name = "LensFlareSystem";
        /**
         * The scene the component belongs to.
         */
        scene: Scene;
        /**
         * Creates a new instance of the component for the given scene
         * @param scene Defines the scene to register the component in
         */
        constructor(scene: Scene);
        /**
         * Registers the component in a given scene
         */
        register(): void;
        /**
         * Rebuilds the elements related to this component in case of
         * context lost for instance.
         */
        rebuild(): void;
        /**
         * Adds all the elements from the container to the scene
         * @param container the container holding the elements
         */
        addFromContainer(container: AbstractScene): void;
        /**
         * Removes all the elements in the container from the scene
         * @param container contains the elements to remove
         * @param dispose if the removed element should be disposed (default: false)
         */
        removeFromContainer(container: AbstractScene, dispose?: boolean): void;
        /**
         * Serializes the component data to the specified json object
         * @param serializationObject The object to serialize to
         */
        serialize(serializationObject: any): void;
        /**
         * Disposes the component and the associated resources.
         */
        dispose(): void;
        private _draw;
    }


    /**
     * A directional light is defined by a direction (what a surprise!).
     * The light is emitted from everywhere in the specified direction, and has an infinite range.
     * An example of a directional light is when a distance planet is lit by the apparently parallel lines of light from its sun. Light in a downward direction will light the top of an object.
     * Documentation: https://doc.babylonjs.com/features/featuresDeepDive/lights/lights_introduction
     */
    export class DirectionalLight extends ShadowLight {
        private _shadowFrustumSize;
        /**
         * Fix frustum size for the shadow generation. This is disabled if the value is 0.
         */
        get shadowFrustumSize(): number;
        /**
         * Specifies a fix frustum size for the shadow generation.
         */
        set shadowFrustumSize(value: number);
        private _shadowOrthoScale;
        /**
         * Gets the shadow projection scale against the optimal computed one.
         * 0.1 by default which means that the projection window is increase by 10% from the optimal size.
         * This does not impact in fixed frustum size (shadowFrustumSize being set)
         */
        get shadowOrthoScale(): number;
        /**
         * Sets the shadow projection scale against the optimal computed one.
         * 0.1 by default which means that the projection window is increase by 10% from the optimal size.
         * This does not impact in fixed frustum size (shadowFrustumSize being set)
         */
        set shadowOrthoScale(value: number);
        /**
         * Automatically compute the projection matrix to best fit (including all the casters)
         * on each frame.
         */
        autoUpdateExtends: boolean;
        /**
         * Automatically compute the shadowMinZ and shadowMaxZ for the projection matrix to best fit (including all the casters)
         * on each frame. autoUpdateExtends must be set to true for this to work
         */
        autoCalcShadowZBounds: boolean;
        private _orthoLeft;
        private _orthoRight;
        private _orthoTop;
        private _orthoBottom;
        /**
         * Gets or sets the orthoLeft property used to build the light frustum
         */
        get orthoLeft(): number;
        set orthoLeft(left: number);
        /**
         * Gets or sets the orthoRight property used to build the light frustum
         */
        get orthoRight(): number;
        set orthoRight(right: number);
        /**
         * Gets or sets the orthoTop property used to build the light frustum
         */
        get orthoTop(): number;
        set orthoTop(top: number);
        /**
         * Gets or sets the orthoBottom property used to build the light frustum
         */
        get orthoBottom(): number;
        set orthoBottom(bottom: number);
        /**
         * Creates a DirectionalLight object in the scene, oriented towards the passed direction (Vector3).
         * The directional light is emitted from everywhere in the given direction.
         * It can cast shadows.
         * Documentation : https://doc.babylonjs.com/features/featuresDeepDive/lights/lights_introduction
         * @param name The friendly name of the light
         * @param direction The direction of the light
         * @param scene The scene the light belongs to
         */
        constructor(name: string, direction: Vector3, scene: Scene);
        /**
         * Returns the string "DirectionalLight".
         * @returns The class name
         */
        getClassName(): string;
        /**
         * Returns the integer 1.
         * @returns The light Type id as a constant defines in Light.LIGHTTYPEID_x
         */
        getTypeID(): number;
        /**
         * Sets the passed matrix "matrix" as projection matrix for the shadows cast by the light according to the passed view matrix.
         * Returns the DirectionalLight Shadow projection matrix.
         * @param matrix
         * @param viewMatrix
         * @param renderList
         */
        protected _setDefaultShadowProjectionMatrix(matrix: Matrix, viewMatrix: Matrix, renderList: Array<AbstractMesh>): void;
        /**
         * Sets the passed matrix "matrix" as fixed frustum projection matrix for the shadows cast by the light according to the passed view matrix.
         * Returns the DirectionalLight Shadow projection matrix.
         * @param matrix
         */
        protected _setDefaultFixedFrustumShadowProjectionMatrix(matrix: Matrix): void;
        /**
         * Sets the passed matrix "matrix" as auto extend projection matrix for the shadows cast by the light according to the passed view matrix.
         * Returns the DirectionalLight Shadow projection matrix.
         * @param matrix
         * @param viewMatrix
         * @param renderList
         */
        protected _setDefaultAutoExtendShadowProjectionMatrix(matrix: Matrix, viewMatrix: Matrix, renderList: Array<AbstractMesh>): void;
        protected _buildUniformLayout(): void;
        /**
         * Sets the passed Effect object with the DirectionalLight transformed position (or position if not parented) and the passed name.
         * @param effect The effect to update
         * @param lightIndex The index of the light in the effect to update
         * @returns The directional light
         */
        transferToEffect(effect: Effect, lightIndex: string): DirectionalLight;
        transferToNodeMaterialEffect(effect: Effect, lightDataUniformName: string): Light;
        /**
         * Gets the minZ used for shadow according to both the scene and the light.
         *
         * Values are fixed on directional lights as it relies on an ortho projection hence the need to convert being
         * -1 and 1 to 0 and 1 doing (depth + min) / (min + max) -> (depth + 1) / (1 + 1) -> (depth * 0.5) + 0.5.
         * (when not using reverse depth buffer / NDC half Z range)
         * @param activeCamera The camera we are returning the min for
         * @returns the depth min z
         */
        getDepthMinZ(activeCamera: Camera): number;
        /**
         * Gets the maxZ used for shadow according to both the scene and the light.
         *
         * Values are fixed on directional lights as it relies on an ortho projection hence the need to convert being
         * -1 and 1 to 0 and 1 doing (depth + min) / (min + max) -> (depth + 1) / (1 + 1) -> (depth * 0.5) + 0.5.
         * (when not using reverse depth buffer / NDC half Z range)
         * @param activeCamera The camera we are returning the max for
         * @returns the depth max z
         */
        getDepthMaxZ(activeCamera: Camera): number;
        /**
         * Prepares the list of defines specific to the light type.
         * @param defines the list of defines
         * @param lightIndex defines the index of the light for the effect
         */
        prepareLightSpecificDefines(defines: any, lightIndex: number): void;
    }


    /**
     * The HemisphericLight simulates the ambient environment light,
     * so the passed direction is the light reflection direction, not the incoming direction.
     */
    export class HemisphericLight extends Light {
        /**
         * The groundColor is the light in the opposite direction to the one specified during creation.
         * You can think of the diffuse and specular light as coming from the centre of the object in the given direction and the groundColor light in the opposite direction.
         */
        groundColor: Color3;
        /**
         * The light reflection direction, not the incoming direction.
         */
        direction: Vector3;
        /**
         * Creates a HemisphericLight object in the scene according to the passed direction (Vector3).
         * The HemisphericLight simulates the ambient environment light, so the passed direction is the light reflection direction, not the incoming direction.
         * The HemisphericLight can't cast shadows.
         * Documentation : https://doc.babylonjs.com/features/featuresDeepDive/lights/lights_introduction
         * @param name The friendly name of the light
         * @param direction The direction of the light reflection
         * @param scene The scene the light belongs to
         */
        constructor(name: string, direction: Vector3, scene: Scene);
        protected _buildUniformLayout(): void;
        /**
         * Returns the string "HemisphericLight".
         * @returns The class name
         */
        getClassName(): string;
        /**
         * Sets the HemisphericLight direction towards the passed target (Vector3).
         * Returns the updated direction.
         * @param target The target the direction should point to
         * @returns The computed direction
         */
        setDirectionToTarget(target: Vector3): Vector3;
        /**
         * Returns the shadow generator associated to the light.
         * @returns Always null for hemispheric lights because it does not support shadows.
         */
        getShadowGenerator(): Nullable<IShadowGenerator>;
        /**
         * Sets the passed Effect object with the HemisphericLight normalized direction and color and the passed name (string).
         * @param _effect The effect to update
         * @param lightIndex The index of the light in the effect to update
         * @returns The hemispheric light
         */
        transferToEffect(_effect: Effect, lightIndex: string): HemisphericLight;
        transferToNodeMaterialEffect(effect: Effect, lightDataUniformName: string): this;
        /**
         * Computes the world matrix of the node
         * @returns the world matrix
         */
        computeWorldMatrix(): Matrix;
        /**
         * Returns the integer 3.
         * @returns The light Type id as a constant defines in Light.LIGHTTYPEID_x
         */
        getTypeID(): number;
        /**
         * Prepares the list of defines specific to the light type.
         * @param defines the list of defines
         * @param lightIndex defines the index of the light for the effect
         */
        prepareLightSpecificDefines(defines: any, lightIndex: number): void;
    }




    /**
     * Base class of all the lights in Babylon. It groups all the generic information about lights.
     * Lights are used, as you would expect, to affect how meshes are seen, in terms of both illumination and colour.
     * All meshes allow light to pass through them unless shadow generation is activated. The default number of lights allowed is four but this can be increased.
     */
    export abstract class Light extends Node implements ISortableLight {
        /**
         * Falloff Default: light is falling off following the material specification:
         * standard material is using standard falloff whereas pbr material can request special falloff per materials.
         */
        static readonly FALLOFF_DEFAULT = 0;
        /**
         * Falloff Physical: light is falling off following the inverse squared distance law.
         */
        static readonly FALLOFF_PHYSICAL = 1;
        /**
         * Falloff gltf: light is falling off as described in the gltf moving to PBR document
         * to enhance interoperability with other engines.
         */
        static readonly FALLOFF_GLTF = 2;
        /**
         * Falloff Standard: light is falling off like in the standard material
         * to enhance interoperability with other materials.
         */
        static readonly FALLOFF_STANDARD = 3;
        /**
         * If every light affecting the material is in this lightmapMode,
         * material.lightmapTexture adds or multiplies
         * (depends on material.useLightmapAsShadowmap)
         * after every other light calculations.
         */
        static readonly LIGHTMAP_DEFAULT = 0;
        /**
         * material.lightmapTexture as only diffuse lighting from this light
         * adds only specular lighting from this light
         * adds dynamic shadows
         */
        static readonly LIGHTMAP_SPECULAR = 1;
        /**
         * material.lightmapTexture as only lighting
         * no light calculation from this light
         * only adds dynamic shadows from this light
         */
        static readonly LIGHTMAP_SHADOWSONLY = 2;
        /**
         * Each light type uses the default quantity according to its type:
         *      point/spot lights use luminous intensity
         *      directional lights use illuminance
         */
        static readonly INTENSITYMODE_AUTOMATIC = 0;
        /**
         * lumen (lm)
         */
        static readonly INTENSITYMODE_LUMINOUSPOWER = 1;
        /**
         * candela (lm/sr)
         */
        static readonly INTENSITYMODE_LUMINOUSINTENSITY = 2;
        /**
         * lux (lm/m^2)
         */
        static readonly INTENSITYMODE_ILLUMINANCE = 3;
        /**
         * nit (cd/m^2)
         */
        static readonly INTENSITYMODE_LUMINANCE = 4;
        /**
         * Light type var id of the point light.
         */
        static readonly LIGHTTYPEID_POINTLIGHT = 0;
        /**
         * Light type var id of the directional light.
         */
        static readonly LIGHTTYPEID_DIRECTIONALLIGHT = 1;
        /**
         * Light type var id of the spot light.
         */
        static readonly LIGHTTYPEID_SPOTLIGHT = 2;
        /**
         * Light type var id of the hemispheric light.
         */
        static readonly LIGHTTYPEID_HEMISPHERICLIGHT = 3;
        /**
         * Diffuse gives the basic color to an object.
         */
        diffuse: Color3;
        /**
         * Specular produces a highlight color on an object.
         * Note: This is not affecting PBR materials.
         */
        specular: Color3;
        /**
         * Defines the falloff type for this light. This lets overriding how punctual light are
         * falling off base on range or angle.
         * This can be set to any values in Light.FALLOFF_x.
         *
         * Note: This is only useful for PBR Materials at the moment. This could be extended if required to
         * other types of materials.
         */
        falloffType: number;
        /**
         * Strength of the light.
         * Note: By default it is define in the framework own unit.
         * Note: In PBR materials the intensityMode can be use to chose what unit the intensity is defined in.
         */
        intensity: number;
        private _range;
        protected _inverseSquaredRange: number;
        /**
         * Defines how far from the source the light is impacting in scene units.
         * Note: Unused in PBR material as the distance light falloff is defined following the inverse squared falloff.
         */
        get range(): number;
        /**
         * Defines how far from the source the light is impacting in scene units.
         * Note: Unused in PBR material as the distance light falloff is defined following the inverse squared falloff.
         */
        set range(value: number);
        /**
         * Cached photometric scale default to 1.0 as the automatic intensity mode defaults to 1.0 for every type
         * of light.
         */
        private _photometricScale;
        private _intensityMode;
        /**
         * Gets the photometric scale used to interpret the intensity.
         * This is only relevant with PBR Materials where the light intensity can be defined in a physical way.
         */
        get intensityMode(): number;
        /**
         * Sets the photometric scale used to interpret the intensity.
         * This is only relevant with PBR Materials where the light intensity can be defined in a physical way.
         */
        set intensityMode(value: number);
        private _radius;
        /**
         * Gets the light radius used by PBR Materials to simulate soft area lights.
         */
        get radius(): number;
        /**
         * sets the light radius used by PBR Materials to simulate soft area lights.
         */
        set radius(value: number);
        private _renderPriority;
        /**
         * Defines the rendering priority of the lights. It can help in case of fallback or number of lights
         * exceeding the number allowed of the materials.
         */
        renderPriority: number;
        private _shadowEnabled;
        /**
         * Gets whether or not the shadows are enabled for this light. This can help turning off/on shadow without detaching
         * the current shadow generator.
         */
        get shadowEnabled(): boolean;
        /**
         * Sets whether or not the shadows are enabled for this light. This can help turning off/on shadow without detaching
         * the current shadow generator.
         */
        set shadowEnabled(value: boolean);
        private _includedOnlyMeshes;
        /**
         * Gets the only meshes impacted by this light.
         */
        get includedOnlyMeshes(): AbstractMesh[];
        /**
         * Sets the only meshes impacted by this light.
         */
        set includedOnlyMeshes(value: AbstractMesh[]);
        private _excludedMeshes;
        /**
         * Gets the meshes not impacted by this light.
         */
        get excludedMeshes(): AbstractMesh[];
        /**
         * Sets the meshes not impacted by this light.
         */
        set excludedMeshes(value: AbstractMesh[]);
        private _excludeWithLayerMask;
        /**
         * Gets the layer id use to find what meshes are not impacted by the light.
         * Inactive if 0
         */
        get excludeWithLayerMask(): number;
        /**
         * Sets the layer id use to find what meshes are not impacted by the light.
         * Inactive if 0
         */
        set excludeWithLayerMask(value: number);
        private _includeOnlyWithLayerMask;
        /**
         * Gets the layer id use to find what meshes are impacted by the light.
         * Inactive if 0
         */
        get includeOnlyWithLayerMask(): number;
        /**
         * Sets the layer id use to find what meshes are impacted by the light.
         * Inactive if 0
         */
        set includeOnlyWithLayerMask(value: number);
        private _lightmapMode;
        /**
         * Gets the lightmap mode of this light (should be one of the constants defined by Light.LIGHTMAP_x)
         */
        get lightmapMode(): number;
        /**
         * Sets the lightmap mode of this light (should be one of the constants defined by Light.LIGHTMAP_x)
         */
        set lightmapMode(value: number);
        /**
         * Shadow generators associated to the light.
         * @internal Internal use only.
         */
        _shadowGenerators: Nullable<Map<Nullable<Camera>, IShadowGenerator>>;
        /**
         * @internal Internal use only.
         */
        _excludedMeshesIds: string[];
        /**
         * @internal Internal use only.
         */
        _includedOnlyMeshesIds: string[];
        /**
         * The current light uniform buffer.
         * @internal Internal use only.
         */
        _uniformBuffer: UniformBuffer;
        /** @internal */
        _renderId: number;
        private _lastUseSpecular;
        /**
         * Creates a Light object in the scene.
         * Documentation : https://doc.babylonjs.com/features/featuresDeepDive/lights/lights_introduction
         * @param name The friendly name of the light
         * @param scene The scene the light belongs too
         */
        constructor(name: string, scene: Scene);
        protected abstract _buildUniformLayout(): void;
        /**
         * Sets the passed Effect "effect" with the Light information.
         * @param effect The effect to update
         * @param lightIndex The index of the light in the effect to update
         * @returns The light
         */
        abstract transferToEffect(effect: Effect, lightIndex: string): Light;
        /**
         * Sets the passed Effect "effect" with the Light textures.
         * @param effect The effect to update
         * @param lightIndex The index of the light in the effect to update
         * @returns The light
         */
        transferTexturesToEffect(effect: Effect, lightIndex: string): Light;
        /**
         * Binds the lights information from the scene to the effect for the given mesh.
         * @param lightIndex Light index
         * @param scene The scene where the light belongs to
         * @param effect The effect we are binding the data to
         * @param useSpecular Defines if specular is supported
         * @param receiveShadows Defines if the effect (mesh) we bind the light for receives shadows
         */
        _bindLight(lightIndex: number, scene: Scene, effect: Effect, useSpecular: boolean, receiveShadows?: boolean): void;
        /**
         * Sets the passed Effect "effect" with the Light information.
         * @param effect The effect to update
         * @param lightDataUniformName The uniform used to store light data (position or direction)
         * @returns The light
         */
        abstract transferToNodeMaterialEffect(effect: Effect, lightDataUniformName: string): Light;
        /**
         * Returns the string "Light".
         * @returns the class name
         */
        getClassName(): string;
        /** @internal */
        readonly _isLight = true;
        /**
         * Converts the light information to a readable string for debug purpose.
         * @param fullDetails Supports for multiple levels of logging within scene loading
         * @returns the human readable light info
         */
        toString(fullDetails?: boolean): string;
        /** @internal */
        protected _syncParentEnabledState(): void;
        /**
         * Set the enabled state of this node.
         * @param value - the new enabled state
         */
        setEnabled(value: boolean): void;
        /**
         * Returns the Light associated shadow generator if any.
         * @param camera Camera for which the shadow generator should be retrieved (default: null). If null, retrieves the default shadow generator
         * @returns the associated shadow generator.
         */
        getShadowGenerator(camera?: Nullable<Camera>): Nullable<IShadowGenerator>;
        /**
         * Returns all the shadow generators associated to this light
         * @returns
         */
        getShadowGenerators(): Nullable<Map<Nullable<Camera>, IShadowGenerator>>;
        /**
         * Returns a Vector3, the absolute light position in the World.
         * @returns the world space position of the light
         */
        getAbsolutePosition(): Vector3;
        /**
         * Specifies if the light will affect the passed mesh.
         * @param mesh The mesh to test against the light
         * @returns true the mesh is affected otherwise, false.
         */
        canAffectMesh(mesh: AbstractMesh): boolean;
        /**
         * Releases resources associated with this node.
         * @param doNotRecurse Set to true to not recurse into each children (recurse into each children by default)
         * @param disposeMaterialAndTextures Set to true to also dispose referenced materials and textures (false by default)
         */
        dispose(doNotRecurse?: boolean, disposeMaterialAndTextures?: boolean): void;
        /**
         * Returns the light type ID (integer).
         * @returns The light Type id as a constant defines in Light.LIGHTTYPEID_x
         */
        getTypeID(): number;
        /**
         * Returns the intensity scaled by the Photometric Scale according to the light type and intensity mode.
         * @returns the scaled intensity in intensity mode unit
         */
        getScaledIntensity(): number;
        /**
         * Returns a new Light object, named "name", from the current one.
         * @param name The name of the cloned light
         * @param newParent The parent of this light, if it has one
         * @returns the new created light
         */
        clone(name: string, newParent?: Nullable<Node>): Nullable<Light>;
        /**
         * Serializes the current light into a Serialization object.
         * @returns the serialized object.
         */
        serialize(): any;
        /**
         * Creates a new typed light from the passed type (integer) : point light = 0, directional light = 1, spot light = 2, hemispheric light = 3.
         * This new light is named "name" and added to the passed scene.
         * @param type Type according to the types available in Light.LIGHTTYPEID_x
         * @param name The friendly name of the light
         * @param scene The scene the new light will belong to
         * @returns the constructor function
         */
        static GetConstructorFromName(type: number, name: string, scene: Scene): Nullable<() => Light>;
        /**
         * Parses the passed "parsedLight" and returns a new instanced Light from this parsing.
         * @param parsedLight The JSON representation of the light
         * @param scene The scene to create the parsed light in
         * @returns the created light after parsing
         */
        static Parse(parsedLight: any, scene: Scene): Nullable<Light>;
        private _hookArrayForExcluded;
        private _hookArrayForIncludedOnly;
        private _resyncMeshes;
        /**
         * Forces the meshes to update their light related information in their rendering used effects
         * @internal Internal Use Only
         */
        _markMeshesAsLightDirty(): void;
        /**
         * Recomputes the cached photometric scale if needed.
         */
        private _computePhotometricScale;
        /**
         * Returns the Photometric Scale according to the light type and intensity mode.
         */
        private _getPhotometricScale;
        /**
         * Reorder the light in the scene according to their defined priority.
         * @internal Internal Use Only
         */
        _reorderLightsInScene(): void;
        /**
         * Prepares the list of defines specific to the light type.
         * @param defines the list of defines
         * @param lightIndex defines the index of the light for the effect
         */
        abstract prepareLightSpecificDefines(defines: any, lightIndex: number): void;
    }


    /** Defines the cross module constantsused by lights to avoid circular dependencies */
    export class LightConstants {
        /**
         * Falloff Default: light is falling off following the material specification:
         * standard material is using standard falloff whereas pbr material can request special falloff per materials.
         */
        static readonly FALLOFF_DEFAULT = 0;
        /**
         * Falloff Physical: light is falling off following the inverse squared distance law.
         */
        static readonly FALLOFF_PHYSICAL = 1;
        /**
         * Falloff gltf: light is falling off as described in the gltf moving to PBR document
         * to enhance interoperability with other engines.
         */
        static readonly FALLOFF_GLTF = 2;
        /**
         * Falloff Standard: light is falling off like in the standard material
         * to enhance interoperability with other materials.
         */
        static readonly FALLOFF_STANDARD = 3;
        /**
         * If every light affecting the material is in this lightmapMode,
         * material.lightmapTexture adds or multiplies
         * (depends on material.useLightmapAsShadowmap)
         * after every other light calculations.
         */
        static readonly LIGHTMAP_DEFAULT = 0;
        /**
         * material.lightmapTexture as only diffuse lighting from this light
         * adds only specular lighting from this light
         * adds dynamic shadows
         */
        static readonly LIGHTMAP_SPECULAR = 1;
        /**
         * material.lightmapTexture as only lighting
         * no light calculation from this light
         * only adds dynamic shadows from this light
         */
        static readonly LIGHTMAP_SHADOWSONLY = 2;
        /**
         * Each light type uses the default quantity according to its type:
         *      point/spot lights use luminous intensity
         *      directional lights use illuminance
         */
        static readonly INTENSITYMODE_AUTOMATIC = 0;
        /**
         * lumen (lm)
         */
        static readonly INTENSITYMODE_LUMINOUSPOWER = 1;
        /**
         * candela (lm/sr)
         */
        static readonly INTENSITYMODE_LUMINOUSINTENSITY = 2;
        /**
         * lux (lm/m^2)
         */
        static readonly INTENSITYMODE_ILLUMINANCE = 3;
        /**
         * nit (cd/m^2)
         */
        static readonly INTENSITYMODE_LUMINANCE = 4;
        /**
         * Light type var id of the point light.
         */
        static readonly LIGHTTYPEID_POINTLIGHT = 0;
        /**
         * Light type var id of the directional light.
         */
        static readonly LIGHTTYPEID_DIRECTIONALLIGHT = 1;
        /**
         * Light type var id of the spot light.
         */
        static readonly LIGHTTYPEID_SPOTLIGHT = 2;
        /**
         * Light type var id of the hemispheric light.
         */
        static readonly LIGHTTYPEID_HEMISPHERICLIGHT = 3;
        /**
         * Sort function to order lights for rendering.
         * @param a First Light object to compare to second.
         * @param b Second Light object to compare first.
         * @returns -1 to reduce's a's index relative to be, 0 for no change, 1 to increase a's index relative to b.
         */
        static CompareLightsPriority(a: ISortableLight, b: ISortableLight): number;
    }
    /**
     * Defines the common interface of sortable lights
     */
    export interface ISortableLight {
        /**
         * Gets or sets whether or not the shadows are enabled for this light. This can help turning off/on shadow without detaching
         * the current shadow generator.
         */
        shadowEnabled: boolean;
        /**
         * Defines the rendering priority of the lights. It can help in case of fallback or number of lights
         * exceeding the number allowed of the materials.
         */
        renderPriority: number;
    }


    /**
     * A point light is a light defined by an unique point in world space.
     * The light is emitted in every direction from this point.
     * A good example of a point light is a standard light bulb.
     * Documentation: https://doc.babylonjs.com/features/featuresDeepDive/lights/lights_introduction
     */
    export class PointLight extends ShadowLight {
        private _shadowAngle;
        /**
         * Getter: In case of direction provided, the shadow will not use a cube texture but simulate a spot shadow as a fallback
         * This specifies what angle the shadow will use to be created.
         *
         * It default to 90 degrees to work nicely with the cube texture generation for point lights shadow maps.
         */
        get shadowAngle(): number;
        /**
         * Setter: In case of direction provided, the shadow will not use a cube texture but simulate a spot shadow as a fallback
         * This specifies what angle the shadow will use to be created.
         *
         * It default to 90 degrees to work nicely with the cube texture generation for point lights shadow maps.
         */
        set shadowAngle(value: number);
        /**
         * Gets the direction if it has been set.
         * In case of direction provided, the shadow will not use a cube texture but simulate a spot shadow as a fallback
         */
        get direction(): Vector3;
        /**
         * In case of direction provided, the shadow will not use a cube texture but simulate a spot shadow as a fallback
         */
        set direction(value: Vector3);
        /**
         * Creates a PointLight object from the passed name and position (Vector3) and adds it in the scene.
         * A PointLight emits the light in every direction.
         * It can cast shadows.
         * If the scene camera is already defined and you want to set your PointLight at the camera position, just set it :
         * ```javascript
         * var pointLight = new PointLight("pl", camera.position, scene);
         * ```
         * Documentation : https://doc.babylonjs.com/features/featuresDeepDive/lights/lights_introduction
         * @param name The light friendly name
         * @param position The position of the point light in the scene
         * @param scene The scene the lights belongs to
         */
        constructor(name: string, position: Vector3, scene: Scene);
        /**
         * Returns the string "PointLight"
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Returns the integer 0.
         * @returns The light Type id as a constant defines in Light.LIGHTTYPEID_x
         */
        getTypeID(): number;
        /**
         * Specifies whether or not the shadowmap should be a cube texture.
         * @returns true if the shadowmap needs to be a cube texture.
         */
        needCube(): boolean;
        /**
         * Returns a new Vector3 aligned with the PointLight cube system according to the passed cube face index (integer).
         * @param faceIndex The index of the face we are computed the direction to generate shadow
         * @returns The set direction in 2d mode otherwise the direction to the cubemap face if needCube() is true
         */
        getShadowDirection(faceIndex?: number): Vector3;
        /**
         * Sets the passed matrix "matrix" as a left-handed perspective projection matrix with the following settings :
         * - fov = PI / 2
         * - aspect ratio : 1.0
         * - z-near and far equal to the active camera minZ and maxZ.
         * Returns the PointLight.
         * @param matrix
         * @param viewMatrix
         * @param renderList
         */
        protected _setDefaultShadowProjectionMatrix(matrix: Matrix, viewMatrix: Matrix, renderList: Array<AbstractMesh>): void;
        protected _buildUniformLayout(): void;
        /**
         * Sets the passed Effect "effect" with the PointLight transformed position (or position, if none) and passed name (string).
         * @param effect The effect to update
         * @param lightIndex The index of the light in the effect to update
         * @returns The point light
         */
        transferToEffect(effect: Effect, lightIndex: string): PointLight;
        transferToNodeMaterialEffect(effect: Effect, lightDataUniformName: string): this;
        /**
         * Prepares the list of defines specific to the light type.
         * @param defines the list of defines
         * @param lightIndex defines the index of the light for the effect
         */
        prepareLightSpecificDefines(defines: any, lightIndex: number): void;
    }


    /**
     * Interface describing all the common properties and methods a shadow light needs to implement.
     * This helps both the shadow generator and materials to generate the corresponding shadow maps
     * as well as binding the different shadow properties to the effects.
     */
    export interface IShadowLight extends Light {
        /**
         * The light id in the scene (used in scene.getLightById for instance)
         */
        id: string;
        /**
         * The position the shadow will be casted from.
         */
        position: Vector3;
        /**
         * In 2d mode (needCube being false), the direction used to cast the shadow.
         */
        direction: Vector3;
        /**
         * The transformed position. Position of the light in world space taking parenting in account.
         */
        transformedPosition: Vector3;
        /**
         * The transformed direction. Direction of the light in world space taking parenting in account.
         */
        transformedDirection: Vector3;
        /**
         * The friendly name of the light in the scene.
         */
        name: string;
        /**
         * Defines the shadow projection clipping minimum z value.
         */
        shadowMinZ: number;
        /**
         * Defines the shadow projection clipping maximum z value.
         */
        shadowMaxZ: number;
        /**
         * Computes the transformed information (transformedPosition and transformedDirection in World space) of the current light
         * @returns true if the information has been computed, false if it does not need to (no parenting)
         */
        computeTransformedInformation(): boolean;
        /**
         * Gets the scene the light belongs to.
         * @returns The scene
         */
        getScene(): Scene;
        /**
         * Callback defining a custom Projection Matrix Builder.
         * This can be used to override the default projection matrix computation.
         */
        customProjectionMatrixBuilder: (viewMatrix: Matrix, renderList: Array<AbstractMesh>, result: Matrix) => void;
        /**
         * Sets the shadow projection matrix in parameter to the generated projection matrix.
         * @param matrix The matrix to updated with the projection information
         * @param viewMatrix The transform matrix of the light
         * @param renderList The list of mesh to render in the map
         * @returns The current light
         */
        setShadowProjectionMatrix(matrix: Matrix, viewMatrix: Matrix, renderList: Array<AbstractMesh>): IShadowLight;
        /**
         * Gets the current depth scale used in ESM.
         * @returns The scale
         */
        getDepthScale(): number;
        /**
         * Returns whether or not the shadow generation require a cube texture or a 2d texture.
         * @returns true if a cube texture needs to be use
         */
        needCube(): boolean;
        /**
         * Detects if the projection matrix requires to be recomputed this frame.
         * @returns true if it requires to be recomputed otherwise, false.
         */
        needProjectionMatrixCompute(): boolean;
        /**
         * Forces the shadow generator to recompute the projection matrix even if position and direction did not changed.
         */
        forceProjectionMatrixCompute(): void;
        /**
         * Get the direction to use to render the shadow map. In case of cube texture, the face index can be passed.
         * @param faceIndex The index of the face we are computed the direction to generate shadow
         * @returns The set direction in 2d mode otherwise the direction to the cubemap face if needCube() is true
         */
        getShadowDirection(faceIndex?: number): Vector3;
        /**
         * Gets the minZ used for shadow according to both the scene and the light.
         * @param activeCamera The camera we are returning the min for
         * @returns the depth min z
         */
        getDepthMinZ(activeCamera: Camera): number;
        /**
         * Gets the maxZ used for shadow according to both the scene and the light.
         * @param activeCamera The camera we are returning the max for
         * @returns the depth max z
         */
        getDepthMaxZ(activeCamera: Camera): number;
    }
    /**
     * Base implementation IShadowLight
     * It groups all the common behaviour in order to reduce duplication and better follow the DRY pattern.
     */
    export abstract class ShadowLight extends Light implements IShadowLight {
        protected abstract _setDefaultShadowProjectionMatrix(matrix: Matrix, viewMatrix: Matrix, renderList: Array<AbstractMesh>): void;
        protected _position: Vector3;
        protected _setPosition(value: Vector3): void;
        /**
         * Sets the position the shadow will be casted from. Also use as the light position for both
         * point and spot lights.
         */
        get position(): Vector3;
        /**
         * Sets the position the shadow will be casted from. Also use as the light position for both
         * point and spot lights.
         */
        set position(value: Vector3);
        protected _direction: Vector3;
        protected _setDirection(value: Vector3): void;
        /**
         * In 2d mode (needCube being false), gets the direction used to cast the shadow.
         * Also use as the light direction on spot and directional lights.
         */
        get direction(): Vector3;
        /**
         * In 2d mode (needCube being false), sets the direction used to cast the shadow.
         * Also use as the light direction on spot and directional lights.
         */
        set direction(value: Vector3);
        protected _shadowMinZ: number;
        /**
         * Gets the shadow projection clipping minimum z value.
         */
        get shadowMinZ(): number;
        /**
         * Sets the shadow projection clipping minimum z value.
         */
        set shadowMinZ(value: number);
        protected _shadowMaxZ: number;
        /**
         * Sets the shadow projection clipping maximum z value.
         */
        get shadowMaxZ(): number;
        /**
         * Gets the shadow projection clipping maximum z value.
         */
        set shadowMaxZ(value: number);
        /**
         * Callback defining a custom Projection Matrix Builder.
         * This can be used to override the default projection matrix computation.
         */
        customProjectionMatrixBuilder: (viewMatrix: Matrix, renderList: Array<AbstractMesh>, result: Matrix) => void;
        /**
         * The transformed position. Position of the light in world space taking parenting in account.
         */
        transformedPosition: Vector3;
        /**
         * The transformed direction. Direction of the light in world space taking parenting in account.
         */
        transformedDirection: Vector3;
        private _needProjectionMatrixCompute;
        /**
         * Computes the transformed information (transformedPosition and transformedDirection in World space) of the current light
         * @returns true if the information has been computed, false if it does not need to (no parenting)
         */
        computeTransformedInformation(): boolean;
        /**
         * Return the depth scale used for the shadow map.
         * @returns the depth scale.
         */
        getDepthScale(): number;
        /**
         * Get the direction to use to render the shadow map. In case of cube texture, the face index can be passed.
         * @param faceIndex The index of the face we are computed the direction to generate shadow
         * @returns The set direction in 2d mode otherwise the direction to the cubemap face if needCube() is true
         */
        getShadowDirection(faceIndex?: number): Vector3;
        /**
         * Returns the ShadowLight absolute position in the World.
         * @returns the position vector in world space
         */
        getAbsolutePosition(): Vector3;
        /**
         * Sets the ShadowLight direction toward the passed target.
         * @param target The point to target in local space
         * @returns the updated ShadowLight direction
         */
        setDirectionToTarget(target: Vector3): Vector3;
        /**
         * Returns the light rotation in euler definition.
         * @returns the x y z rotation in local space.
         */
        getRotation(): Vector3;
        /**
         * Returns whether or not the shadow generation require a cube texture or a 2d texture.
         * @returns true if a cube texture needs to be use
         */
        needCube(): boolean;
        /**
         * Detects if the projection matrix requires to be recomputed this frame.
         * @returns true if it requires to be recomputed otherwise, false.
         */
        needProjectionMatrixCompute(): boolean;
        /**
         * Forces the shadow generator to recompute the projection matrix even if position and direction did not changed.
         */
        forceProjectionMatrixCompute(): void;
        /** @internal */
        _initCache(): void;
        /** @internal */
        _isSynchronized(): boolean;
        /**
         * Computes the world matrix of the node
         * @param force defines if the cache version should be invalidated forcing the world matrix to be created from scratch
         * @returns the world matrix
         */
        computeWorldMatrix(force?: boolean): Matrix;
        /**
         * Gets the minZ used for shadow according to both the scene and the light.
         * @param activeCamera The camera we are returning the min for
         * @returns the depth min z
         */
        getDepthMinZ(activeCamera: Camera): number;
        /**
         * Gets the maxZ used for shadow according to both the scene and the light.
         * @param activeCamera The camera we are returning the max for
         * @returns the depth max z
         */
        getDepthMaxZ(activeCamera: Camera): number;
        /**
         * Sets the shadow projection matrix in parameter to the generated projection matrix.
         * @param matrix The matrix to updated with the projection information
         * @param viewMatrix The transform matrix of the light
         * @param renderList The list of mesh to render in the map
         * @returns The current light
         */
        setShadowProjectionMatrix(matrix: Matrix, viewMatrix: Matrix, renderList: Array<AbstractMesh>): IShadowLight;
        /** @internal */
        protected _syncParentEnabledState(): void;
    }


    /**
     * A CSM implementation allowing casting shadows on large scenes.
     * Documentation : https://doc.babylonjs.com/babylon101/cascadedShadows
     * Based on: https://github.com/TheRealMJP/Shadows and https://johanmedestrom.wordpress.com/2016/03/18/opengl-cascaded-shadow-maps/
     */
    export class CascadedShadowGenerator extends ShadowGenerator {
        private static readonly _FrustumCornersNDCSpace;
        /**
         * Name of the CSM class
         */
        static CLASSNAME: string;
        /**
         * Defines the default number of cascades used by the CSM.
         */
        static readonly DEFAULT_CASCADES_COUNT = 4;
        /**
         * Defines the minimum number of cascades used by the CSM.
         */
        static readonly MIN_CASCADES_COUNT = 2;
        /**
         * Defines the maximum number of cascades used by the CSM.
         */
        static readonly MAX_CASCADES_COUNT = 4;
        protected _validateFilter(filter: number): number;
        /**
         * Gets or sets the actual darkness of the soft shadows while using PCSS filtering (value between 0. and 1.)
         */
        penumbraDarkness: number;
        private _numCascades;
        /**
         * Gets or set the number of cascades used by the CSM.
         */
        get numCascades(): number;
        set numCascades(value: number);
        /**
         * Sets this to true if you want that the edges of the shadows don't "swimm" / "shimmer" when rotating the camera.
         * The trade off is that you lose some precision in the shadow rendering when enabling this setting.
         */
        stabilizeCascades: boolean;
        private _freezeShadowCastersBoundingInfo;
        private _freezeShadowCastersBoundingInfoObservable;
        /**
         * Enables or disables the shadow casters bounding info computation.
         * If your shadow casters don't move, you can disable this feature.
         * If it is enabled, the bounding box computation is done every frame.
         */
        get freezeShadowCastersBoundingInfo(): boolean;
        set freezeShadowCastersBoundingInfo(freeze: boolean);
        private _scbiMin;
        private _scbiMax;
        protected _computeShadowCastersBoundingInfo(): void;
        protected _shadowCastersBoundingInfo: BoundingInfo;
        /**
         * Gets or sets the shadow casters bounding info.
         * If you provide your own shadow casters bounding info, first enable freezeShadowCastersBoundingInfo
         * so that the system won't overwrite the bounds you provide
         */
        get shadowCastersBoundingInfo(): BoundingInfo;
        set shadowCastersBoundingInfo(boundingInfo: BoundingInfo);
        protected _breaksAreDirty: boolean;
        protected _minDistance: number;
        protected _maxDistance: number;
        /**
         * Sets the minimal and maximal distances to use when computing the cascade breaks.
         *
         * The values of min / max are typically the depth zmin and zmax values of your scene, for a given frame.
         * If you don't know these values, simply leave them to their defaults and don't call this function.
         * @param min minimal distance for the breaks (default to 0.)
         * @param max maximal distance for the breaks (default to 1.)
         */
        setMinMaxDistance(min: number, max: number): void;
        /** Gets the minimal distance used in the cascade break computation */
        get minDistance(): number;
        /** Gets the maximal distance used in the cascade break computation */
        get maxDistance(): number;
        /**
         * Gets the class name of that object
         * @returns "CascadedShadowGenerator"
         */
        getClassName(): string;
        private _cascadeMinExtents;
        private _cascadeMaxExtents;
        /**
         * Gets a cascade minimum extents
         * @param cascadeIndex index of the cascade
         * @returns the minimum cascade extents
         */
        getCascadeMinExtents(cascadeIndex: number): Nullable<Vector3>;
        /**
         * Gets a cascade maximum extents
         * @param cascadeIndex index of the cascade
         * @returns the maximum cascade extents
         */
        getCascadeMaxExtents(cascadeIndex: number): Nullable<Vector3>;
        private _cascades;
        private _currentLayer;
        private _viewSpaceFrustumsZ;
        private _viewMatrices;
        private _projectionMatrices;
        private _transformMatrices;
        private _transformMatricesAsArray;
        private _frustumLengths;
        private _lightSizeUVCorrection;
        private _depthCorrection;
        private _frustumCornersWorldSpace;
        private _frustumCenter;
        private _shadowCameraPos;
        private _shadowMaxZ;
        /**
         * Gets the shadow max z distance. It's the limit beyond which shadows are not displayed.
         * It defaults to camera.maxZ
         */
        get shadowMaxZ(): number;
        /**
         * Sets the shadow max z distance.
         */
        set shadowMaxZ(value: number);
        protected _debug: boolean;
        /**
         * Gets or sets the debug flag.
         * When enabled, the cascades are materialized by different colors on the screen.
         */
        get debug(): boolean;
        set debug(dbg: boolean);
        private _depthClamp;
        /**
         * Gets or sets the depth clamping value.
         *
         * When enabled, it improves the shadow quality because the near z plane of the light frustum don't need to be adjusted
         * to account for the shadow casters far away.
         *
         * Note that this property is incompatible with PCSS filtering, so it won't be used in that case.
         */
        get depthClamp(): boolean;
        set depthClamp(value: boolean);
        private _cascadeBlendPercentage;
        /**
         * Gets or sets the percentage of blending between two cascades (value between 0. and 1.).
         * It defaults to 0.1 (10% blending).
         */
        get cascadeBlendPercentage(): number;
        set cascadeBlendPercentage(value: number);
        private _lambda;
        /**
         * Gets or set the lambda parameter.
         * This parameter is used to split the camera frustum and create the cascades.
         * It's a value between 0. and 1.: If 0, the split is a uniform split of the frustum, if 1 it is a logarithmic split.
         * For all values in-between, it's a linear combination of the uniform and logarithm split algorithm.
         */
        get lambda(): number;
        set lambda(value: number);
        /**
         * Gets the view matrix corresponding to a given cascade
         * @param cascadeNum cascade to retrieve the view matrix from
         * @returns the cascade view matrix
         */
        getCascadeViewMatrix(cascadeNum: number): Nullable<Matrix>;
        /**
         * Gets the projection matrix corresponding to a given cascade
         * @param cascadeNum cascade to retrieve the projection matrix from
         * @returns the cascade projection matrix
         */
        getCascadeProjectionMatrix(cascadeNum: number): Nullable<Matrix>;
        /**
         * Gets the transformation matrix corresponding to a given cascade
         * @param cascadeNum cascade to retrieve the transformation matrix from
         * @returns the cascade transformation matrix
         */
        getCascadeTransformMatrix(cascadeNum: number): Nullable<Matrix>;
        private _depthRenderer;
        /**
         * Sets the depth renderer to use when autoCalcDepthBounds is enabled.
         *
         * Note that if no depth renderer is set, a new one will be automatically created internally when necessary.
         *
         * You should call this function if you already have a depth renderer enabled in your scene, to avoid
         * doing multiple depth rendering each frame. If you provide your own depth renderer, make sure it stores linear depth!
         * @param depthRenderer The depth renderer to use when autoCalcDepthBounds is enabled. If you pass null or don't call this function at all, a depth renderer will be automatically created
         */
        setDepthRenderer(depthRenderer: Nullable<DepthRenderer>): void;
        private _depthReducer;
        private _autoCalcDepthBounds;
        /**
         * Gets or sets the autoCalcDepthBounds property.
         *
         * When enabled, a depth rendering pass is first performed (with an internally created depth renderer or with the one
         * you provide by calling setDepthRenderer). Then, a min/max reducing is applied on the depth map to compute the
         * minimal and maximal depth of the map and those values are used as inputs for the setMinMaxDistance() function.
         * It can greatly enhance the shadow quality, at the expense of more GPU works.
         * When using this option, you should increase the value of the lambda parameter, and even set it to 1 for best results.
         */
        get autoCalcDepthBounds(): boolean;
        set autoCalcDepthBounds(value: boolean);
        /**
         * Defines the refresh rate of the min/max computation used when autoCalcDepthBounds is set to true
         * Use 0 to compute just once, 1 to compute on every frame, 2 to compute every two frames and so on...
         * Note that if you provided your own depth renderer through a call to setDepthRenderer, you are responsible
         * for setting the refresh rate on the renderer yourself!
         */
        get autoCalcDepthBoundsRefreshRate(): number;
        set autoCalcDepthBoundsRefreshRate(value: number);
        /**
         * Create the cascade breaks according to the lambda, shadowMaxZ and min/max distance properties, as well as the camera near and far planes.
         * This function is automatically called when updating lambda, shadowMaxZ and min/max distances, however you should call it yourself if
         * you change the camera near/far planes!
         */
        splitFrustum(): void;
        private _splitFrustum;
        private _computeMatrices;
        private _computeFrustumInWorldSpace;
        private _computeCascadeFrustum;
        protected _recreateSceneUBOs(): void;
        /**
         *  Support test.
         */
        static get IsSupported(): boolean;
        /**
         * @internal
         */
        static _SceneComponentInitialization: (scene: Scene) => void;
        /**
         * Creates a Cascaded Shadow Generator object.
         * A ShadowGenerator is the required tool to use the shadows.
         * Each directional light casting shadows needs to use its own ShadowGenerator.
         * Documentation : https://doc.babylonjs.com/babylon101/cascadedShadows
         * @param mapSize The size of the texture what stores the shadows. Example : 1024.
         * @param light The directional light object generating the shadows.
         * @param usefulFloatFirst By default the generator will try to use half float textures but if you need precision (for self shadowing for instance), you can use this option to enforce full float texture.
         * @param camera Camera associated with this shadow generator (default: null). If null, takes the scene active camera at the time we need to access it
         */
        constructor(mapSize: number, light: DirectionalLight, usefulFloatFirst?: boolean, camera?: Nullable<Camera>);
        protected _initializeGenerator(): void;
        protected _createTargetRenderTexture(): void;
        protected _initializeShadowMap(): void;
        protected _bindCustomEffectForRenderSubMeshForShadowMap(subMesh: SubMesh, effect: Effect): void;
        protected _isReadyCustomDefines(defines: any): void;
        /**
         * Prepare all the defines in a material relying on a shadow map at the specified light index.
         * @param defines Defines of the material we want to update
         * @param lightIndex Index of the light in the enabled light list of the material
         */
        prepareDefines(defines: any, lightIndex: number): void;
        /**
         * Binds the shadow related information inside of an effect (information like near, far, darkness...
         * defined in the generator but impacting the effect).
         * @param lightIndex Index of the light in the enabled light list of the material owning the effect
         * @param effect The effect we are binfing the information for
         */
        bindShadowLight(lightIndex: string, effect: Effect): void;
        /**
         * Gets the transformation matrix of the first cascade used to project the meshes into the map from the light point of view.
         * (eq to view projection * shadow projection matrices)
         * @returns The transform matrix used to create the shadow map
         */
        getTransformMatrix(): Matrix;
        /**
         * Disposes the ShadowGenerator.
         * Returns nothing.
         */
        dispose(): void;
        /**
         * Serializes the shadow generator setup to a json object.
         * @returns The serialized JSON object
         */
        serialize(): any;
        /**
         * Parses a serialized ShadowGenerator and returns a new ShadowGenerator.
         * @param parsedShadowGenerator The JSON object to parse
         * @param scene The scene to create the shadow map for
         * @returns The parsed shadow generator
         */
        static Parse(parsedShadowGenerator: any, scene: Scene): ShadowGenerator;
    }




    /**
     * Defines the options associated with the creation of a custom shader for a shadow generator.
     */
    export interface ICustomShaderOptions {
        /**
         * Gets or sets the custom shader name to use
         */
        shaderName: string;
        /**
         * The list of attribute names used in the shader
         */
        attributes?: string[];
        /**
         * The list of uniform names used in the shader
         */
        uniforms?: string[];
        /**
         * The list of sampler names used in the shader
         */
        samplers?: string[];
        /**
         * The list of defines used in the shader
         */
        defines?: string[];
    }
    /**
     * Interface to implement to create a shadow generator compatible with BJS.
     */
    export interface IShadowGenerator {
        /** Gets or set the id of the shadow generator. It will be the one from the light if not defined */
        id: string;
        /**
         * Gets the main RTT containing the shadow map (usually storing depth from the light point of view).
         * @returns The render target texture if present otherwise, null
         */
        getShadowMap(): Nullable<RenderTargetTexture>;
        /**
         * Determine whether the shadow generator is ready or not (mainly all effects and related post processes needs to be ready).
         * @param subMesh The submesh we want to render in the shadow map
         * @param useInstances Defines whether will draw in the map using instances
         * @param isTransparent Indicates that isReady is called for a transparent subMesh
         * @returns true if ready otherwise, false
         */
        isReady(subMesh: SubMesh, useInstances: boolean, isTransparent: boolean): boolean;
        /**
         * Prepare all the defines in a material relying on a shadow map at the specified light index.
         * @param defines Defines of the material we want to update
         * @param lightIndex Index of the light in the enabled light list of the material
         */
        prepareDefines(defines: MaterialDefines, lightIndex: number): void;
        /**
         * Binds the shadow related information inside of an effect (information like near, far, darkness...
         * defined in the generator but impacting the effect).
         * It implies the uniforms available on the materials are the standard BJS ones.
         * @param lightIndex Index of the light in the enabled light list of the material owning the effect
         * @param effect The effect we are binding the information for
         */
        bindShadowLight(lightIndex: string, effect: Effect): void;
        /**
         * Gets the transformation matrix used to project the meshes into the map from the light point of view.
         * (eq to shadow projection matrix * light transform matrix)
         * @returns The transform matrix used to create the shadow map
         */
        getTransformMatrix(): Matrix;
        /**
         * Recreates the shadow map dependencies like RTT and post processes. This can be used during the switch between
         * Cube and 2D textures for instance.
         */
        recreateShadowMap(): void;
        /**
         * Forces all the attached effect to compile to enable rendering only once ready vs. lazily compiling effects.
         * @param onCompiled Callback triggered at the and of the effects compilation
         * @param options Sets of optional options forcing the compilation with different modes
         */
        forceCompilation(onCompiled?: (generator: IShadowGenerator) => void, options?: Partial<{
            useInstances: boolean;
        }>): void;
        /**
         * Forces all the attached effect to compile to enable rendering only once ready vs. lazily compiling effects.
         * @param options Sets of optional options forcing the compilation with different modes
         * @returns A promise that resolves when the compilation completes
         */
        forceCompilationAsync(options?: Partial<{
            useInstances: boolean;
        }>): Promise<void>;
        /**
         * Serializes the shadow generator setup to a json object.
         * @returns The serialized JSON object
         */
        serialize(): any;
        /**
         * Disposes the Shadow map and related Textures and effects.
         */
        dispose(): void;
    }
    /**
     * Default implementation IShadowGenerator.
     * This is the main object responsible of generating shadows in the framework.
     * Documentation: https://doc.babylonjs.com/features/featuresDeepDive/lights/shadows
     */
    export class ShadowGenerator implements IShadowGenerator {
        /**
         * Name of the shadow generator class
         */
        static CLASSNAME: string;
        /**
         * Shadow generator mode None: no filtering applied.
         */
        static readonly FILTER_NONE = 0;
        /**
         * Shadow generator mode ESM: Exponential Shadow Mapping.
         * (http://developer.download.nvidia.com/presentations/2008/GDC/GDC08_SoftShadowMapping.pdf)
         */
        static readonly FILTER_EXPONENTIALSHADOWMAP = 1;
        /**
         * Shadow generator mode Poisson Sampling: Percentage Closer Filtering.
         * (Multiple Tap around evenly distributed around the pixel are used to evaluate the shadow strength)
         */
        static readonly FILTER_POISSONSAMPLING = 2;
        /**
         * Shadow generator mode ESM: Blurred Exponential Shadow Mapping.
         * (http://developer.download.nvidia.com/presentations/2008/GDC/GDC08_SoftShadowMapping.pdf)
         */
        static readonly FILTER_BLUREXPONENTIALSHADOWMAP = 3;
        /**
         * Shadow generator mode ESM: Exponential Shadow Mapping using the inverse of the exponential preventing
         * edge artifacts on steep falloff.
         * (http://developer.download.nvidia.com/presentations/2008/GDC/GDC08_SoftShadowMapping.pdf)
         */
        static readonly FILTER_CLOSEEXPONENTIALSHADOWMAP = 4;
        /**
         * Shadow generator mode ESM: Blurred Exponential Shadow Mapping using the inverse of the exponential preventing
         * edge artifacts on steep falloff.
         * (http://developer.download.nvidia.com/presentations/2008/GDC/GDC08_SoftShadowMapping.pdf)
         */
        static readonly FILTER_BLURCLOSEEXPONENTIALSHADOWMAP = 5;
        /**
         * Shadow generator mode PCF: Percentage Closer Filtering
         * benefits from Webgl 2 shadow samplers. Fallback to Poisson Sampling in Webgl 1
         * (https://developer.nvidia.com/gpugems/GPUGems/gpugems_ch11.html)
         */
        static readonly FILTER_PCF = 6;
        /**
         * Shadow generator mode PCSS: Percentage Closering Soft Shadow.
         * benefits from Webgl 2 shadow samplers. Fallback to Poisson Sampling in Webgl 1
         * Contact Hardening
         */
        static readonly FILTER_PCSS = 7;
        /**
         * Reserved for PCF and PCSS
         * Highest Quality.
         *
         * Execute PCF on a 5*5 kernel improving a lot the shadow aliasing artifacts.
         *
         * Execute PCSS with 32 taps blocker search and 64 taps PCF.
         */
        static readonly QUALITY_HIGH = 0;
        /**
         * Reserved for PCF and PCSS
         * Good tradeoff for quality/perf cross devices
         *
         * Execute PCF on a 3*3 kernel.
         *
         * Execute PCSS with 16 taps blocker search and 32 taps PCF.
         */
        static readonly QUALITY_MEDIUM = 1;
        /**
         * Reserved for PCF and PCSS
         * The lowest quality but the fastest.
         *
         * Execute PCF on a 1*1 kernel.
         *
         * Execute PCSS with 16 taps blocker search and 16 taps PCF.
         */
        static readonly QUALITY_LOW = 2;
        /**
         * Defines the default alpha cutoff value used for transparent alpha tested materials.
         */
        static DEFAULT_ALPHA_CUTOFF: number;
        /** Gets or set the id of the shadow generator. It will be the one from the light if not defined */
        id: string;
        /** Gets or sets the custom shader name to use */
        customShaderOptions: ICustomShaderOptions;
        /** Gets or sets a custom function to allow/disallow rendering a sub mesh in the shadow map */
        customAllowRendering: (subMesh: SubMesh) => boolean;
        /**
         * Observable triggered before the shadow is rendered. Can be used to update internal effect state
         */
        onBeforeShadowMapRenderObservable: Observable<Effect>;
        /**
         * Observable triggered after the shadow is rendered. Can be used to restore internal effect state
         */
        onAfterShadowMapRenderObservable: Observable<Effect>;
        /**
         * Observable triggered before a mesh is rendered in the shadow map.
         * Can be used to update internal effect state (that you can get from the onBeforeShadowMapRenderObservable)
         */
        onBeforeShadowMapRenderMeshObservable: Observable<Mesh>;
        /**
         * Observable triggered after a mesh is rendered in the shadow map.
         * Can be used to update internal effect state (that you can get from the onAfterShadowMapRenderObservable)
         */
        onAfterShadowMapRenderMeshObservable: Observable<Mesh>;
        protected _bias: number;
        /**
         * Gets the bias: offset applied on the depth preventing acnea (in light direction).
         */
        get bias(): number;
        /**
         * Sets the bias: offset applied on the depth preventing acnea (in light direction).
         */
        set bias(bias: number);
        protected _normalBias: number;
        /**
         * Gets the normalBias: offset applied on the depth preventing acnea (along side the normal direction and proportional to the light/normal angle).
         */
        get normalBias(): number;
        /**
         * Sets the normalBias: offset applied on the depth preventing acnea (along side the normal direction and proportional to the light/normal angle).
         */
        set normalBias(normalBias: number);
        protected _blurBoxOffset: number;
        /**
         * Gets the blur box offset: offset applied during the blur pass.
         * Only useful if useKernelBlur = false
         */
        get blurBoxOffset(): number;
        /**
         * Sets the blur box offset: offset applied during the blur pass.
         * Only useful if useKernelBlur = false
         */
        set blurBoxOffset(value: number);
        protected _blurScale: number;
        /**
         * Gets the blur scale: scale of the blurred texture compared to the main shadow map.
         * 2 means half of the size.
         */
        get blurScale(): number;
        /**
         * Sets the blur scale: scale of the blurred texture compared to the main shadow map.
         * 2 means half of the size.
         */
        set blurScale(value: number);
        protected _blurKernel: number;
        /**
         * Gets the blur kernel: kernel size of the blur pass.
         * Only useful if useKernelBlur = true
         */
        get blurKernel(): number;
        /**
         * Sets the blur kernel: kernel size of the blur pass.
         * Only useful if useKernelBlur = true
         */
        set blurKernel(value: number);
        protected _useKernelBlur: boolean;
        /**
         * Gets whether the blur pass is a kernel blur (if true) or box blur.
         * Only useful in filtered mode (useBlurExponentialShadowMap...)
         */
        get useKernelBlur(): boolean;
        /**
         * Sets whether the blur pass is a kernel blur (if true) or box blur.
         * Only useful in filtered mode (useBlurExponentialShadowMap...)
         */
        set useKernelBlur(value: boolean);
        protected _depthScale: number;
        /**
         * Gets the depth scale used in ESM mode.
         */
        get depthScale(): number;
        /**
         * Sets the depth scale used in ESM mode.
         * This can override the scale stored on the light.
         */
        set depthScale(value: number);
        protected _validateFilter(filter: number): number;
        protected _filter: number;
        /**
         * Gets the current mode of the shadow generator (normal, PCF, ESM...).
         * The returned value is a number equal to one of the available mode defined in ShadowMap.FILTER_x like _FILTER_NONE
         */
        get filter(): number;
        /**
         * Sets the current mode of the shadow generator (normal, PCF, ESM...).
         * The returned value is a number equal to one of the available mode defined in ShadowMap.FILTER_x like _FILTER_NONE
         */
        set filter(value: number);
        /**
         * Gets if the current filter is set to Poisson Sampling.
         */
        get usePoissonSampling(): boolean;
        /**
         * Sets the current filter to Poisson Sampling.
         */
        set usePoissonSampling(value: boolean);
        /**
         * Gets if the current filter is set to ESM.
         */
        get useExponentialShadowMap(): boolean;
        /**
         * Sets the current filter is to ESM.
         */
        set useExponentialShadowMap(value: boolean);
        /**
         * Gets if the current filter is set to filtered ESM.
         */
        get useBlurExponentialShadowMap(): boolean;
        /**
         * Gets if the current filter is set to filtered  ESM.
         */
        set useBlurExponentialShadowMap(value: boolean);
        /**
         * Gets if the current filter is set to "close ESM" (using the inverse of the
         * exponential to prevent steep falloff artifacts).
         */
        get useCloseExponentialShadowMap(): boolean;
        /**
         * Sets the current filter to "close ESM" (using the inverse of the
         * exponential to prevent steep falloff artifacts).
         */
        set useCloseExponentialShadowMap(value: boolean);
        /**
         * Gets if the current filter is set to filtered "close ESM" (using the inverse of the
         * exponential to prevent steep falloff artifacts).
         */
        get useBlurCloseExponentialShadowMap(): boolean;
        /**
         * Sets the current filter to filtered "close ESM" (using the inverse of the
         * exponential to prevent steep falloff artifacts).
         */
        set useBlurCloseExponentialShadowMap(value: boolean);
        /**
         * Gets if the current filter is set to "PCF" (percentage closer filtering).
         */
        get usePercentageCloserFiltering(): boolean;
        /**
         * Sets the current filter to "PCF" (percentage closer filtering).
         */
        set usePercentageCloserFiltering(value: boolean);
        protected _filteringQuality: number;
        /**
         * Gets the PCF or PCSS Quality.
         * Only valid if usePercentageCloserFiltering or usePercentageCloserFiltering is true.
         */
        get filteringQuality(): number;
        /**
         * Sets the PCF or PCSS Quality.
         * Only valid if usePercentageCloserFiltering or usePercentageCloserFiltering is true.
         */
        set filteringQuality(filteringQuality: number);
        /**
         * Gets if the current filter is set to "PCSS" (contact hardening).
         */
        get useContactHardeningShadow(): boolean;
        /**
         * Sets the current filter to "PCSS" (contact hardening).
         */
        set useContactHardeningShadow(value: boolean);
        protected _contactHardeningLightSizeUVRatio: number;
        /**
         * Gets the Light Size (in shadow map uv unit) used in PCSS to determine the blocker search area and the penumbra size.
         * Using a ratio helps keeping shape stability independently of the map size.
         *
         * It does not account for the light projection as it was having too much
         * instability during the light setup or during light position changes.
         *
         * Only valid if useContactHardeningShadow is true.
         */
        get contactHardeningLightSizeUVRatio(): number;
        /**
         * Sets the Light Size (in shadow map uv unit) used in PCSS to determine the blocker search area and the penumbra size.
         * Using a ratio helps keeping shape stability independently of the map size.
         *
         * It does not account for the light projection as it was having too much
         * instability during the light setup or during light position changes.
         *
         * Only valid if useContactHardeningShadow is true.
         */
        set contactHardeningLightSizeUVRatio(contactHardeningLightSizeUVRatio: number);
        protected _darkness: number;
        /** Gets or sets the actual darkness of a shadow */
        get darkness(): number;
        set darkness(value: number);
        /**
         * Returns the darkness value (float). This can only decrease the actual darkness of a shadow.
         * 0 means strongest and 1 would means no shadow.
         * @returns the darkness.
         */
        getDarkness(): number;
        /**
         * Sets the darkness value (float). This can only decrease the actual darkness of a shadow.
         * @param darkness The darkness value 0 means strongest and 1 would means no shadow.
         * @returns the shadow generator allowing fluent coding.
         */
        setDarkness(darkness: number): ShadowGenerator;
        protected _transparencyShadow: boolean;
        /** Gets or sets the ability to have transparent shadow  */
        get transparencyShadow(): boolean;
        set transparencyShadow(value: boolean);
        /**
         * Sets the ability to have transparent shadow (boolean).
         * @param transparent True if transparent else False
         * @returns the shadow generator allowing fluent coding
         */
        setTransparencyShadow(transparent: boolean): ShadowGenerator;
        /**
         * Enables or disables shadows with varying strength based on the transparency
         * When it is enabled, the strength of the shadow is taken equal to mesh.visibility
         * If you enabled an alpha texture on your material, the alpha value red from the texture is also combined to compute the strength:
         *          mesh.visibility * alphaTexture.a
         * The texture used is the diffuse by default, but it can be set to the opacity by setting useOpacityTextureForTransparentShadow
         * Note that by definition transparencyShadow must be set to true for enableSoftTransparentShadow to work!
         */
        enableSoftTransparentShadow: boolean;
        /**
         * If this is true, use the opacity texture's alpha channel for transparent shadows instead of the diffuse one
         */
        useOpacityTextureForTransparentShadow: boolean;
        protected _shadowMap: Nullable<RenderTargetTexture>;
        protected _shadowMap2: Nullable<RenderTargetTexture>;
        /**
         * Gets the main RTT containing the shadow map (usually storing depth from the light point of view).
         * @returns The render target texture if present otherwise, null
         */
        getShadowMap(): Nullable<RenderTargetTexture>;
        /**
         * Gets the RTT used during rendering (can be a blurred version of the shadow map or the shadow map itself).
         * @returns The render target texture if the shadow map is present otherwise, null
         */
        getShadowMapForRendering(): Nullable<RenderTargetTexture>;
        /**
         * Gets the class name of that object
         * @returns "ShadowGenerator"
         */
        getClassName(): string;
        /**
         * Helper function to add a mesh and its descendants to the list of shadow casters.
         * @param mesh Mesh to add
         * @param includeDescendants boolean indicating if the descendants should be added. Default to true
         * @returns the Shadow Generator itself
         */
        addShadowCaster(mesh: AbstractMesh, includeDescendants?: boolean): ShadowGenerator;
        /**
         * Helper function to remove a mesh and its descendants from the list of shadow casters
         * @param mesh Mesh to remove
         * @param includeDescendants boolean indicating if the descendants should be removed. Default to true
         * @returns the Shadow Generator itself
         */
        removeShadowCaster(mesh: AbstractMesh, includeDescendants?: boolean): ShadowGenerator;
        /**
         * Controls the extent to which the shadows fade out at the edge of the frustum
         */
        frustumEdgeFalloff: number;
        protected _light: IShadowLight;
        /**
         * Returns the associated light object.
         * @returns the light generating the shadow
         */
        getLight(): IShadowLight;
        /**
         * If true the shadow map is generated by rendering the back face of the mesh instead of the front face.
         * This can help with self-shadowing as the geometry making up the back of objects is slightly offset.
         * It might on the other hand introduce peter panning.
         */
        forceBackFacesOnly: boolean;
        protected _camera: Nullable<Camera>;
        protected _getCamera(): Nullable<Camera>;
        protected _scene: Scene;
        protected _lightDirection: Vector3;
        protected _viewMatrix: Matrix;
        protected _projectionMatrix: Matrix;
        protected _transformMatrix: Matrix;
        protected _cachedPosition: Vector3;
        protected _cachedDirection: Vector3;
        protected _cachedDefines: string;
        protected _currentRenderId: number;
        protected _boxBlurPostprocess: Nullable<PostProcess>;
        protected _kernelBlurXPostprocess: Nullable<PostProcess>;
        protected _kernelBlurYPostprocess: Nullable<PostProcess>;
        protected _blurPostProcesses: PostProcess[];
        protected _mapSize: number;
        protected _currentFaceIndex: number;
        protected _currentFaceIndexCache: number;
        protected _textureType: number;
        protected _defaultTextureMatrix: Matrix;
        protected _storedUniqueId: Nullable<number>;
        protected _useUBO: boolean;
        protected _sceneUBOs: UniformBuffer[];
        protected _currentSceneUBO: UniformBuffer;
        /**
         * @internal
         */
        static _SceneComponentInitialization: (scene: Scene) => void;
        /**
         * Gets or sets the size of the texture what stores the shadows
         */
        get mapSize(): number;
        set mapSize(size: number);
        /**
         * Creates a ShadowGenerator object.
         * A ShadowGenerator is the required tool to use the shadows.
         * Each light casting shadows needs to use its own ShadowGenerator.
         * Documentation : https://doc.babylonjs.com/features/featuresDeepDive/lights/shadows
         * @param mapSize The size of the texture what stores the shadows. Example : 1024.
         * @param light The light object generating the shadows.
         * @param usefullFloatFirst By default the generator will try to use half float textures but if you need precision (for self shadowing for instance), you can use this option to enforce full float texture.
         * @param camera Camera associated with this shadow generator (default: null). If null, takes the scene active camera at the time we need to access it
         */
        constructor(mapSize: number, light: IShadowLight, usefullFloatFirst?: boolean, camera?: Nullable<Camera>);
        protected _initializeGenerator(): void;
        protected _createTargetRenderTexture(): void;
        protected _initializeShadowMap(): void;
        protected _initializeBlurRTTAndPostProcesses(): void;
        protected _renderForShadowMap(opaqueSubMeshes: SmartArray<SubMesh>, alphaTestSubMeshes: SmartArray<SubMesh>, transparentSubMeshes: SmartArray<SubMesh>, depthOnlySubMeshes: SmartArray<SubMesh>): void;
        protected _bindCustomEffectForRenderSubMeshForShadowMap(subMesh: SubMesh, effect: Effect, mesh: AbstractMesh): void;
        protected _renderSubMeshForShadowMap(subMesh: SubMesh, isTransparent?: boolean): void;
        protected _applyFilterValues(): void;
        /**
         * Forces all the attached effect to compile to enable rendering only once ready vs. lazily compiling effects.
         * @param onCompiled Callback triggered at the and of the effects compilation
         * @param options Sets of optional options forcing the compilation with different modes
         */
        forceCompilation(onCompiled?: (generator: IShadowGenerator) => void, options?: Partial<{
            useInstances: boolean;
        }>): void;
        /**
         * Forces all the attached effect to compile to enable rendering only once ready vs. lazily compiling effects.
         * @param options Sets of optional options forcing the compilation with different modes
         * @returns A promise that resolves when the compilation completes
         */
        forceCompilationAsync(options?: Partial<{
            useInstances: boolean;
        }>): Promise<void>;
        protected _isReadyCustomDefines(defines: any, subMesh: SubMesh, useInstances: boolean): void;
        private _prepareShadowDefines;
        /**
         * Determine whether the shadow generator is ready or not (mainly all effects and related post processes needs to be ready).
         * @param subMesh The submesh we want to render in the shadow map
         * @param useInstances Defines whether will draw in the map using instances
         * @param isTransparent Indicates that isReady is called for a transparent subMesh
         * @returns true if ready otherwise, false
         */
        isReady(subMesh: SubMesh, useInstances: boolean, isTransparent: boolean): boolean;
        /**
         * Prepare all the defines in a material relying on a shadow map at the specified light index.
         * @param defines Defines of the material we want to update
         * @param lightIndex Index of the light in the enabled light list of the material
         */
        prepareDefines(defines: any, lightIndex: number): void;
        /**
         * Binds the shadow related information inside of an effect (information like near, far, darkness...
         * defined in the generator but impacting the effect).
         * @param lightIndex Index of the light in the enabled light list of the material owning the effect
         * @param effect The effect we are binding the information for
         */
        bindShadowLight(lightIndex: string, effect: Effect): void;
        /**
         * Gets the transformation matrix used to project the meshes into the map from the light point of view.
         * (eq to shadow projection matrix * light transform matrix)
         * @returns The transform matrix used to create the shadow map
         */
        getTransformMatrix(): Matrix;
        /**
         * Recreates the shadow map dependencies like RTT and post processes. This can be used during the switch between
         * Cube and 2D textures for instance.
         */
        recreateShadowMap(): void;
        protected _disposeBlurPostProcesses(): void;
        protected _disposeRTTandPostProcesses(): void;
        protected _disposeSceneUBOs(): void;
        /**
         * Disposes the ShadowGenerator.
         * Returns nothing.
         */
        dispose(): void;
        /**
         * Serializes the shadow generator setup to a json object.
         * @returns The serialized JSON object
         */
        serialize(): any;
        /**
         * Parses a serialized ShadowGenerator and returns a new ShadowGenerator.
         * @param parsedShadowGenerator The JSON object to parse
         * @param scene The scene to create the shadow map for
         * @param constr A function that builds a shadow generator or undefined to create an instance of the default shadow generator
         * @returns The parsed shadow generator
         */
        static Parse(parsedShadowGenerator: any, scene: Scene, constr?: (mapSize: number, light: IShadowLight, camera: Nullable<Camera>) => ShadowGenerator): ShadowGenerator;
    }


    /**
     * Defines the shadow generator component responsible to manage any shadow generators
     * in a given scene.
     */
    export class ShadowGeneratorSceneComponent implements ISceneSerializableComponent {
        /**
         * The component name helpful to identify the component in the list of scene components.
         */
        readonly name = "ShadowGenerator";
        /**
         * The scene the component belongs to.
         */
        scene: Scene;
        /**
         * Creates a new instance of the component for the given scene
         * @param scene Defines the scene to register the component in
         */
        constructor(scene: Scene);
        /**
         * Registers the component in a given scene
         */
        register(): void;
        /**
         * Rebuilds the elements related to this component in case of
         * context lost for instance.
         */
        rebuild(): void;
        /**
         * Serializes the component data to the specified json object
         * @param serializationObject The object to serialize to
         */
        serialize(serializationObject: any): void;
        /**
         * Adds all the elements from the container to the scene
         * @param container the container holding the elements
         */
        addFromContainer(container: AbstractScene): void;
        /**
         * Removes all the elements in the container from the scene
         * @param container contains the elements to remove
         * @param dispose if the removed element should be disposed (default: false)
         */
        removeFromContainer(container: AbstractScene, dispose?: boolean): void;
        /**
         * Rebuilds the elements related to this component in case of
         * context lost for instance.
         */
        dispose(): void;
        private _gatherRenderTargets;
    }


    /**
     * A spot light is defined by a position, a direction, an angle, and an exponent.
     * These values define a cone of light starting from the position, emitting toward the direction.
     * The angle, in radians, defines the size (field of illumination) of the spotlight's conical beam,
     * and the exponent defines the speed of the decay of the light with distance (reach).
     * Documentation: https://doc.babylonjs.com/features/featuresDeepDive/lights/lights_introduction
     */
    export class SpotLight extends ShadowLight {
        private _angle;
        private _innerAngle;
        private _cosHalfAngle;
        private _lightAngleScale;
        private _lightAngleOffset;
        /**
         * Gets the cone angle of the spot light in Radians.
         */
        get angle(): number;
        /**
         * Sets the cone angle of the spot light in Radians.
         */
        set angle(value: number);
        /**
         * Only used in gltf falloff mode, this defines the angle where
         * the directional falloff will start before cutting at angle which could be seen
         * as outer angle.
         */
        get innerAngle(): number;
        /**
         * Only used in gltf falloff mode, this defines the angle where
         * the directional falloff will start before cutting at angle which could be seen
         * as outer angle.
         */
        set innerAngle(value: number);
        private _shadowAngleScale;
        /**
         * Allows scaling the angle of the light for shadow generation only.
         */
        get shadowAngleScale(): number;
        /**
         * Allows scaling the angle of the light for shadow generation only.
         */
        set shadowAngleScale(value: number);
        /**
         * The light decay speed with the distance from the emission spot.
         */
        exponent: number;
        private _projectionTextureMatrix;
        /**
         * Allows reading the projection texture
         */
        get projectionTextureMatrix(): Matrix;
        protected _projectionTextureLightNear: number;
        /**
         * Gets the near clip of the Spotlight for texture projection.
         */
        get projectionTextureLightNear(): number;
        /**
         * Sets the near clip of the Spotlight for texture projection.
         */
        set projectionTextureLightNear(value: number);
        protected _projectionTextureLightFar: number;
        /**
         * Gets the far clip of the Spotlight for texture projection.
         */
        get projectionTextureLightFar(): number;
        /**
         * Sets the far clip of the Spotlight for texture projection.
         */
        set projectionTextureLightFar(value: number);
        protected _projectionTextureUpDirection: Vector3;
        /**
         * Gets the Up vector of the Spotlight for texture projection.
         */
        get projectionTextureUpDirection(): Vector3;
        /**
         * Sets the Up vector of the Spotlight for texture projection.
         */
        set projectionTextureUpDirection(value: Vector3);
        private _projectionTexture;
        /**
         * Gets the projection texture of the light.
         */
        get projectionTexture(): Nullable<BaseTexture>;
        /**
         * Sets the projection texture of the light.
         */
        set projectionTexture(value: Nullable<BaseTexture>);
        private static _IsProceduralTexture;
        private static _IsTexture;
        private _projectionTextureViewLightDirty;
        private _projectionTextureProjectionLightDirty;
        private _projectionTextureDirty;
        private _projectionTextureViewTargetVector;
        private _projectionTextureViewLightMatrix;
        private _projectionTextureProjectionLightMatrix;
        /**
         * Gets or sets the light projection matrix as used by the projection texture
         */
        get projectionTextureProjectionLightMatrix(): Matrix;
        set projectionTextureProjectionLightMatrix(projection: Matrix);
        private _projectionTextureScalingMatrix;
        /**
         * Creates a SpotLight object in the scene. A spot light is a simply light oriented cone.
         * It can cast shadows.
         * Documentation : https://doc.babylonjs.com/features/featuresDeepDive/lights/lights_introduction
         * @param name The light friendly name
         * @param position The position of the spot light in the scene
         * @param direction The direction of the light in the scene
         * @param angle The cone angle of the light in Radians
         * @param exponent The light decay speed with the distance from the emission spot
         * @param scene The scene the lights belongs to
         */
        constructor(name: string, position: Vector3, direction: Vector3, angle: number, exponent: number, scene: Scene);
        /**
         * Returns the string "SpotLight".
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Returns the integer 2.
         * @returns The light Type id as a constant defines in Light.LIGHTTYPEID_x
         */
        getTypeID(): number;
        /**
         * Overrides the direction setter to recompute the projection texture view light Matrix.
         * @param value
         */
        protected _setDirection(value: Vector3): void;
        /**
         * Overrides the position setter to recompute the projection texture view light Matrix.
         * @param value
         */
        protected _setPosition(value: Vector3): void;
        /**
         * Sets the passed matrix "matrix" as perspective projection matrix for the shadows and the passed view matrix with the fov equal to the SpotLight angle and and aspect ratio of 1.0.
         * Returns the SpotLight.
         * @param matrix
         * @param viewMatrix
         * @param renderList
         */
        protected _setDefaultShadowProjectionMatrix(matrix: Matrix, viewMatrix: Matrix, renderList: Array<AbstractMesh>): void;
        protected _computeProjectionTextureViewLightMatrix(): void;
        protected _computeProjectionTextureProjectionLightMatrix(): void;
        /**
         * Main function for light texture projection matrix computing.
         */
        protected _computeProjectionTextureMatrix(): void;
        protected _buildUniformLayout(): void;
        private _computeAngleValues;
        /**
         * Sets the passed Effect "effect" with the Light textures.
         * @param effect The effect to update
         * @param lightIndex The index of the light in the effect to update
         * @returns The light
         */
        transferTexturesToEffect(effect: Effect, lightIndex: string): Light;
        /**
         * Sets the passed Effect object with the SpotLight transformed position (or position if not parented) and normalized direction.
         * @param effect The effect to update
         * @param lightIndex The index of the light in the effect to update
         * @returns The spot light
         */
        transferToEffect(effect: Effect, lightIndex: string): SpotLight;
        transferToNodeMaterialEffect(effect: Effect, lightDataUniformName: string): this;
        /**
         * Disposes the light and the associated resources.
         */
        dispose(): void;
        /**
         * Gets the minZ used for shadow according to both the scene and the light.
         * @param activeCamera The camera we are returning the min for
         * @returns the depth min z
         */
        getDepthMinZ(activeCamera: Camera): number;
        /**
         * Gets the maxZ used for shadow according to both the scene and the light.
         * @param activeCamera The camera we are returning the max for
         * @returns the depth max z
         */
        getDepthMaxZ(activeCamera: Camera): number;
        /**
         * Prepares the list of defines specific to the light type.
         * @param defines the list of defines
         * @param lightIndex defines the index of the light for the effect
         */
        prepareLightSpecificDefines(defines: any, lightIndex: number): void;
    }




    /**
     * Interface used to present a loading screen while loading a scene
     * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/customLoadingScreen
     */
    export interface ILoadingScreen {
        /**
         * Function called to display the loading screen
         */
        displayLoadingUI: () => void;
        /**
         * Function called to hide the loading screen
         */
        hideLoadingUI: () => void;
        /**
         * Gets or sets the color to use for the background
         */
        loadingUIBackgroundColor: string;
        /**
         * Gets or sets the text to display while loading
         */
        loadingUIText: string;
    }
    /**
     * Class used for the default loading screen
     * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/customLoadingScreen
     */
    export class DefaultLoadingScreen implements ILoadingScreen {
        private _renderingCanvas;
        private _loadingText;
        private _loadingDivBackgroundColor;
        private _loadingDiv;
        private _loadingTextDiv;
        private _style;
        /** Gets or sets the logo url to use for the default loading screen */
        static DefaultLogoUrl: string;
        /** Gets or sets the spinner url to use for the default loading screen */
        static DefaultSpinnerUrl: string;
        /**
         * Creates a new default loading screen
         * @param _renderingCanvas defines the canvas used to render the scene
         * @param _loadingText defines the default text to display
         * @param _loadingDivBackgroundColor defines the default background color
         */
        constructor(_renderingCanvas: HTMLCanvasElement, _loadingText?: string, _loadingDivBackgroundColor?: string);
        /**
         * Function called to display the loading screen
         */
        displayLoadingUI(): void;
        /**
         * Function called to hide the loading screen
         */
        hideLoadingUI(): void;
        /**
         * Gets or sets the text to display while loading
         */
        set loadingUIText(text: string);
        get loadingUIText(): string;
        /**
         * Gets or sets the color to use for the background
         */
        get loadingUIBackgroundColor(): string;
        set loadingUIBackgroundColor(color: string);
        private _resizeLoadingUI;
    }


    /** @internal */
    export var _BabylonLoaderRegistered: boolean;
    /**
     * Helps setting up some configuration for the babylon file loader.
     */
    export class BabylonFileLoaderConfiguration {
        /**
         * The loader does not allow injecting custom physics engine into the plugins.
         * Unfortunately in ES6, we need to manually inject them into the plugin.
         * So you could set this variable to your engine import to make it work.
         */
        static LoaderInjectedPhysicsEngine: any;
    }




    /**
     * Type used for the success callback of ImportMesh
     */
    export type SceneLoaderSuccessCallback = (meshes: AbstractMesh[], particleSystems: IParticleSystem[], skeletons: Skeleton[], animationGroups: AnimationGroup[], transformNodes: TransformNode[], geometries: Geometry[], lights: Light[]) => void;
    /**
     * Interface used for the result of ImportMeshAsync
     */
    export interface ISceneLoaderAsyncResult {
        /**
         * The array of loaded meshes
         */
        readonly meshes: AbstractMesh[];
        /**
         * The array of loaded particle systems
         */
        readonly particleSystems: IParticleSystem[];
        /**
         * The array of loaded skeletons
         */
        readonly skeletons: Skeleton[];
        /**
         * The array of loaded animation groups
         */
        readonly animationGroups: AnimationGroup[];
        /**
         * The array of loaded transform nodes
         */
        readonly transformNodes: TransformNode[];
        /**
         * The array of loaded geometries
         */
        readonly geometries: Geometry[];
        /**
         * The array of loaded lights
         */
        readonly lights: Light[];
    }
    /**
     * Interface used to represent data loading progression
     */
    export interface ISceneLoaderProgressEvent {
        /**
         * Defines if data length to load can be evaluated
         */
        readonly lengthComputable: boolean;
        /**
         * Defines the loaded data length
         */
        readonly loaded: number;
        /**
         * Defines the data length to load
         */
        readonly total: number;
    }
    /**
     * Interface used by SceneLoader plugins to define supported file extensions
     */
    export interface ISceneLoaderPluginExtensions {
        /**
         * Defines the list of supported extensions
         */
        [extension: string]: {
            isBinary: boolean;
        };
    }
    /**
     * Interface used by SceneLoader plugin factory
     */
    export interface ISceneLoaderPluginFactory {
        /**
         * Defines the name of the factory
         */
        name: string;
        /**
         * Function called to create a new plugin
         * @returns the new plugin
         */
        createPlugin(): ISceneLoaderPlugin | ISceneLoaderPluginAsync;
        /**
         * The callback that returns true if the data can be directly loaded.
         * @param data string containing the file data
         * @returns if the data can be loaded directly
         */
        canDirectLoad?(data: string): boolean;
    }
    /**
     * Interface used to define the base of ISceneLoaderPlugin and ISceneLoaderPluginAsync
     */
    export interface ISceneLoaderPluginBase {
        /**
         * The friendly name of this plugin.
         */
        name: string;
        /**
         * The file extensions supported by this plugin.
         */
        extensions: string | ISceneLoaderPluginExtensions;
        /**
         * The callback called when loading from a url.
         * @param scene scene loading this url
         * @param fileOrUrl file or url to load
         * @param onSuccess callback called when the file successfully loads
         * @param onProgress callback called while file is loading (if the server supports this mode)
         * @param useArrayBuffer defines a boolean indicating that date must be returned as ArrayBuffer
         * @param onError callback called when the file fails to load
         * @returns a file request object
         */
        loadFile?(scene: Scene, fileOrUrl: File | string, onSuccess: (data: any, responseURL?: string) => void, onProgress?: (ev: ISceneLoaderProgressEvent) => void, useArrayBuffer?: boolean, onError?: (request?: WebRequest, exception?: LoadFileError) => void): IFileRequest;
        /**
         * The callback that returns true if the data can be directly loaded.
         * @param data string containing the file data
         * @returns if the data can be loaded directly
         */
        canDirectLoad?(data: string): boolean;
        /**
         * The callback that returns the data to pass to the plugin if the data can be directly loaded.
         * @param scene scene loading this data
         * @param data string containing the data
         * @returns data to pass to the plugin
         */
        directLoad?(scene: Scene, data: string): any;
        /**
         * The callback that allows custom handling of the root url based on the response url.
         * @param rootUrl the original root url
         * @param responseURL the response url if available
         * @returns the new root url
         */
        rewriteRootURL?(rootUrl: string, responseURL?: string): string;
    }
    /**
     * Interface used to define a SceneLoader plugin
     */
    export interface ISceneLoaderPlugin extends ISceneLoaderPluginBase {
        /**
         * Import meshes into a scene.
         * @param meshesNames An array of mesh names, a single mesh name, or empty string for all meshes that filter what meshes are imported
         * @param scene The scene to import into
         * @param data The data to import
         * @param rootUrl The root url for scene and resources
         * @param meshes The meshes array to import into
         * @param particleSystems The particle systems array to import into
         * @param skeletons The skeletons array to import into
         * @param onError The callback when import fails
         * @returns True if successful or false otherwise
         */
        importMesh(meshesNames: any, scene: Scene, data: any, rootUrl: string, meshes: AbstractMesh[], particleSystems: IParticleSystem[], skeletons: Skeleton[], onError?: (message: string, exception?: any) => void): boolean;
        /**
         * Load into a scene.
         * @param scene The scene to load into
         * @param data The data to import
         * @param rootUrl The root url for scene and resources
         * @param onError The callback when import fails
         * @returns True if successful or false otherwise
         */
        load(scene: Scene, data: any, rootUrl: string, onError?: (message: string, exception?: any) => void): boolean;
        /**
         * Load into an asset container.
         * @param scene The scene to load into
         * @param data The data to import
         * @param rootUrl The root url for scene and resources
         * @param onError The callback when import fails
         * @returns The loaded asset container
         */
        loadAssetContainer(scene: Scene, data: any, rootUrl: string, onError?: (message: string, exception?: any) => void): AssetContainer;
    }
    /**
     * Interface used to define an async SceneLoader plugin
     */
    export interface ISceneLoaderPluginAsync extends ISceneLoaderPluginBase {
        /**
         * Import meshes into a scene.
         * @param meshesNames An array of mesh names, a single mesh name, or empty string for all meshes that filter what meshes are imported
         * @param scene The scene to import into
         * @param data The data to import
         * @param rootUrl The root url for scene and resources
         * @param onProgress The callback when the load progresses
         * @param fileName Defines the name of the file to load
         * @returns The loaded objects (e.g. meshes, particle systems, skeletons, animation groups, etc.)
         */
        importMeshAsync(meshesNames: any, scene: Scene, data: any, rootUrl: string, onProgress?: (event: ISceneLoaderProgressEvent) => void, fileName?: string): Promise<ISceneLoaderAsyncResult>;
        /**
         * Load into a scene.
         * @param scene The scene to load into
         * @param data The data to import
         * @param rootUrl The root url for scene and resources
         * @param onProgress The callback when the load progresses
         * @param fileName Defines the name of the file to load
         * @returns Nothing
         */
        loadAsync(scene: Scene, data: any, rootUrl: string, onProgress?: (event: ISceneLoaderProgressEvent) => void, fileName?: string): Promise<void>;
        /**
         * Load into an asset container.
         * @param scene The scene to load into
         * @param data The data to import
         * @param rootUrl The root url for scene and resources
         * @param onProgress The callback when the load progresses
         * @param fileName Defines the name of the file to load
         * @returns The loaded asset container
         */
        loadAssetContainerAsync(scene: Scene, data: any, rootUrl: string, onProgress?: (event: ISceneLoaderProgressEvent) => void, fileName?: string): Promise<AssetContainer>;
    }
    /**
     * Mode that determines how to handle old animation groups before loading new ones.
     */
    export enum SceneLoaderAnimationGroupLoadingMode {
        /**
         * Reset all old animations to initial state then dispose them.
         */
        Clean = 0,
        /**
         * Stop all old animations.
         */
        Stop = 1,
        /**
         * Restart old animations from first frame.
         */
        Sync = 2,
        /**
         * Old animations remains untouched.
         */
        NoSync = 3
    }
    /**
     * Defines a plugin registered by the SceneLoader
     */
    interface IRegisteredPlugin {
        /**
         * Defines the plugin to use
         */
        plugin: ISceneLoaderPlugin | ISceneLoaderPluginAsync | ISceneLoaderPluginFactory;
        /**
         * Defines if the plugin supports binary data
         */
        isBinary: boolean;
    }
    /**
     * Class used to load scene from various file formats using registered plugins
     * @see https://doc.babylonjs.com/features/featuresDeepDive/importers/loadingFileTypes
     */
    export class SceneLoader {
        /**
         * No logging while loading
         */
        static readonly NO_LOGGING = 0;
        /**
         * Minimal logging while loading
         */
        static readonly MINIMAL_LOGGING = 1;
        /**
         * Summary logging while loading
         */
        static readonly SUMMARY_LOGGING = 2;
        /**
         * Detailed logging while loading
         */
        static readonly DETAILED_LOGGING = 3;
        /**
         * Gets or sets a boolean indicating if entire scene must be loaded even if scene contains incremental data
         */
        static get ForceFullSceneLoadingForIncremental(): boolean;
        static set ForceFullSceneLoadingForIncremental(value: boolean);
        /**
         * Gets or sets a boolean indicating if loading screen must be displayed while loading a scene
         */
        static get ShowLoadingScreen(): boolean;
        static set ShowLoadingScreen(value: boolean);
        /**
         * Defines the current logging level (while loading the scene)
         * @ignorenaming
         */
        static get loggingLevel(): number;
        static set loggingLevel(value: number);
        /**
         * Gets or set a boolean indicating if matrix weights must be cleaned upon loading
         */
        static get CleanBoneMatrixWeights(): boolean;
        static set CleanBoneMatrixWeights(value: boolean);
        /**
         * Event raised when a plugin is used to load a scene
         */
        static OnPluginActivatedObservable: Observable<ISceneLoaderPlugin | ISceneLoaderPluginAsync>;
        private static _RegisteredPlugins;
        private static _ShowingLoadingScreen;
        /**
         * Gets the default plugin (used to load Babylon files)
         * @returns the .babylon plugin
         */
        static GetDefaultPlugin(): IRegisteredPlugin;
        private static _GetPluginForExtension;
        private static _GetPluginForDirectLoad;
        private static _GetPluginForFilename;
        private static _GetDirectLoad;
        private static _FormatErrorMessage;
        private static _LoadData;
        private static _GetFileInfo;
        /**
         * Gets a plugin that can load the given extension
         * @param extension defines the extension to load
         * @returns a plugin or null if none works
         */
        static GetPluginForExtension(extension: string): ISceneLoaderPlugin | ISceneLoaderPluginAsync | ISceneLoaderPluginFactory;
        /**
         * Gets a boolean indicating that the given extension can be loaded
         * @param extension defines the extension to load
         * @returns true if the extension is supported
         */
        static IsPluginForExtensionAvailable(extension: string): boolean;
        /**
         * Adds a new plugin to the list of registered plugins
         * @param plugin defines the plugin to add
         */
        static RegisterPlugin(plugin: ISceneLoaderPlugin | ISceneLoaderPluginAsync): void;
        /**
         * Import meshes into a scene
         * @param meshNames an array of mesh names, a single mesh name, or empty string for all meshes that filter what meshes are imported
         * @param rootUrl a string that defines the root url for the scene and resources or the concatenation of rootURL and filename (e.g. http://example.com/test.glb)
         * @param sceneFilename a string that defines the name of the scene file or starts with "data:" following by the stringified version of the scene or a File object (default: empty string)
         * @param scene the instance of BABYLON.Scene to append to
         * @param onSuccess a callback with a list of imported meshes, particleSystems, skeletons, and animationGroups when import succeeds
         * @param onProgress a callback with a progress event for each file being loaded
         * @param onError a callback with the scene, a message, and possibly an exception when import fails
         * @param pluginExtension the extension used to determine the plugin
         * @returns The loaded plugin
         */
        static ImportMesh(meshNames: any, rootUrl: string, sceneFilename?: string | File, scene?: Nullable<Scene>, onSuccess?: Nullable<SceneLoaderSuccessCallback>, onProgress?: Nullable<(event: ISceneLoaderProgressEvent) => void>, onError?: Nullable<(scene: Scene, message: string, exception?: any) => void>, pluginExtension?: Nullable<string>): Nullable<ISceneLoaderPlugin | ISceneLoaderPluginAsync>;
        /**
         * Import meshes into a scene
         * @param meshNames an array of mesh names, a single mesh name, or empty string for all meshes that filter what meshes are imported
         * @param rootUrl a string that defines the root url for the scene and resources or the concatenation of rootURL and filename (e.g. http://example.com/test.glb)
         * @param sceneFilename a string that defines the name of the scene file or starts with "data:" following by the stringified version of the scene or a File object (default: empty string)
         * @param scene the instance of BABYLON.Scene to append to
         * @param onProgress a callback with a progress event for each file being loaded
         * @param pluginExtension the extension used to determine the plugin
         * @returns The loaded list of imported meshes, particle systems, skeletons, and animation groups
         */
        static ImportMeshAsync(meshNames: any, rootUrl: string, sceneFilename?: string | File, scene?: Nullable<Scene>, onProgress?: Nullable<(event: ISceneLoaderProgressEvent) => void>, pluginExtension?: Nullable<string>): Promise<ISceneLoaderAsyncResult>;
        /**
         * Load a scene
         * @param rootUrl a string that defines the root url for the scene and resources or the concatenation of rootURL and filename (e.g. http://example.com/test.glb)
         * @param sceneFilename a string that defines the name of the scene file or starts with "data:" following by the stringified version of the scene or a File object (default: empty string)
         * @param engine is the instance of BABYLON.Engine to use to create the scene
         * @param onSuccess a callback with the scene when import succeeds
         * @param onProgress a callback with a progress event for each file being loaded
         * @param onError a callback with the scene, a message, and possibly an exception when import fails
         * @param pluginExtension the extension used to determine the plugin
         * @returns The loaded plugin
         */
        static Load(rootUrl: string, sceneFilename?: string | File, engine?: Nullable<Engine>, onSuccess?: Nullable<(scene: Scene) => void>, onProgress?: Nullable<(event: ISceneLoaderProgressEvent) => void>, onError?: Nullable<(scene: Scene, message: string, exception?: any) => void>, pluginExtension?: Nullable<string>): Nullable<ISceneLoaderPlugin | ISceneLoaderPluginAsync>;
        /**
         * Load a scene
         * @param rootUrl a string that defines the root url for the scene and resources or the concatenation of rootURL and filename (e.g. http://example.com/test.glb)
         * @param sceneFilename a string that defines the name of the scene file or starts with "data:" following by the stringified version of the scene or a File object (default: empty string)
         * @param engine is the instance of BABYLON.Engine to use to create the scene
         * @param onProgress a callback with a progress event for each file being loaded
         * @param pluginExtension the extension used to determine the plugin
         * @returns The loaded scene
         */
        static LoadAsync(rootUrl: string, sceneFilename?: string | File, engine?: Nullable<Engine>, onProgress?: Nullable<(event: ISceneLoaderProgressEvent) => void>, pluginExtension?: Nullable<string>): Promise<Scene>;
        /**
         * Append a scene
         * @param rootUrl a string that defines the root url for the scene and resources or the concatenation of rootURL and filename (e.g. http://example.com/test.glb)
         * @param sceneFilename a string that defines the name of the scene file or starts with "data:" following by the stringified version of the scene or a File object (default: empty string)
         * @param scene is the instance of BABYLON.Scene to append to
         * @param onSuccess a callback with the scene when import succeeds
         * @param onProgress a callback with a progress event for each file being loaded
         * @param onError a callback with the scene, a message, and possibly an exception when import fails
         * @param pluginExtension the extension used to determine the plugin
         * @returns The loaded plugin
         */
        static Append(rootUrl: string, sceneFilename?: string | File, scene?: Nullable<Scene>, onSuccess?: Nullable<(scene: Scene) => void>, onProgress?: Nullable<(event: ISceneLoaderProgressEvent) => void>, onError?: Nullable<(scene: Scene, message: string, exception?: any) => void>, pluginExtension?: Nullable<string>): Nullable<ISceneLoaderPlugin | ISceneLoaderPluginAsync>;
        /**
         * Append a scene
         * @param rootUrl a string that defines the root url for the scene and resources or the concatenation of rootURL and filename (e.g. http://example.com/test.glb)
         * @param sceneFilename a string that defines the name of the scene file or starts with "data:" following by the stringified version of the scene or a File object (default: empty string)
         * @param scene is the instance of BABYLON.Scene to append to
         * @param onProgress a callback with a progress event for each file being loaded
         * @param pluginExtension the extension used to determine the plugin
         * @returns The given scene
         */
        static AppendAsync(rootUrl: string, sceneFilename?: string | File, scene?: Nullable<Scene>, onProgress?: Nullable<(event: ISceneLoaderProgressEvent) => void>, pluginExtension?: Nullable<string>): Promise<Scene>;
        /**
         * Load a scene into an asset container
         * @param rootUrl a string that defines the root url for the scene and resources or the concatenation of rootURL and filename (e.g. http://example.com/test.glb)
         * @param sceneFilename a string that defines the name of the scene file or starts with "data:" following by the stringified version of the scene or a File object (default: empty string)
         * @param scene is the instance of BABYLON.Scene to append to (default: last created scene)
         * @param onSuccess a callback with the scene when import succeeds
         * @param onProgress a callback with a progress event for each file being loaded
         * @param onError a callback with the scene, a message, and possibly an exception when import fails
         * @param pluginExtension the extension used to determine the plugin
         * @returns The loaded plugin
         */
        static LoadAssetContainer(rootUrl: string, sceneFilename?: string | File, scene?: Nullable<Scene>, onSuccess?: Nullable<(assets: AssetContainer) => void>, onProgress?: Nullable<(event: ISceneLoaderProgressEvent) => void>, onError?: Nullable<(scene: Scene, message: string, exception?: any) => void>, pluginExtension?: Nullable<string>): Nullable<ISceneLoaderPlugin | ISceneLoaderPluginAsync>;
        /**
         * Load a scene into an asset container
         * @param rootUrl a string that defines the root url for the scene and resources or the concatenation of rootURL and filename (e.g. http://example.com/test.glb)
         * @param sceneFilename a string that defines the name of the scene file or starts with "data:" following by the stringified version of the scene (default: empty string)
         * @param scene is the instance of Scene to append to
         * @param onProgress a callback with a progress event for each file being loaded
         * @param pluginExtension the extension used to determine the plugin
         * @returns The loaded asset container
         */
        static LoadAssetContainerAsync(rootUrl: string, sceneFilename?: string | File, scene?: Nullable<Scene>, onProgress?: Nullable<(event: ISceneLoaderProgressEvent) => void>, pluginExtension?: Nullable<string>): Promise<AssetContainer>;
        /**
         * Import animations from a file into a scene
         * @param rootUrl a string that defines the root url for the scene and resources or the concatenation of rootURL and filename (e.g. http://example.com/test.glb)
         * @param sceneFilename a string that defines the name of the scene file or starts with "data:" following by the stringified version of the scene or a File object (default: empty string)
         * @param scene is the instance of BABYLON.Scene to append to (default: last created scene)
         * @param overwriteAnimations when true, animations are cleaned before importing new ones. Animations are appended otherwise
         * @param animationGroupLoadingMode defines how to handle old animations groups before importing new ones
         * @param targetConverter defines a function used to convert animation targets from loaded scene to current scene (default: search node by name)
         * @param onSuccess a callback with the scene when import succeeds
         * @param onProgress a callback with a progress event for each file being loaded
         * @param onError a callback with the scene, a message, and possibly an exception when import fails
         * @param pluginExtension the extension used to determine the plugin
         */
        static ImportAnimations(rootUrl: string, sceneFilename?: string | File, scene?: Nullable<Scene>, overwriteAnimations?: boolean, animationGroupLoadingMode?: SceneLoaderAnimationGroupLoadingMode, targetConverter?: Nullable<(target: any) => any>, onSuccess?: Nullable<(scene: Scene) => void>, onProgress?: Nullable<(event: ISceneLoaderProgressEvent) => void>, onError?: Nullable<(scene: Scene, message: string, exception?: any) => void>, pluginExtension?: Nullable<string>): void;
        /**
         * Import animations from a file into a scene
         * @param rootUrl a string that defines the root url for the scene and resources or the concatenation of rootURL and filename (e.g. http://example.com/test.glb)
         * @param sceneFilename a string that defines the name of the scene file or starts with "data:" following by the stringified version of the scene or a File object (default: empty string)
         * @param scene is the instance of BABYLON.Scene to append to (default: last created scene)
         * @param overwriteAnimations when true, animations are cleaned before importing new ones. Animations are appended otherwise
         * @param animationGroupLoadingMode defines how to handle old animations groups before importing new ones
         * @param targetConverter defines a function used to convert animation targets from loaded scene to current scene (default: search node by name)
         * @param onSuccess a callback with the scene when import succeeds
         * @param onProgress a callback with a progress event for each file being loaded
         * @param onError a callback with the scene, a message, and possibly an exception when import fails
         * @param pluginExtension the extension used to determine the plugin
         * @returns the updated scene with imported animations
         */
        static ImportAnimationsAsync(rootUrl: string, sceneFilename?: string | File, scene?: Nullable<Scene>, overwriteAnimations?: boolean, animationGroupLoadingMode?: SceneLoaderAnimationGroupLoadingMode, targetConverter?: Nullable<(target: any) => any>, onSuccess?: Nullable<(scene: Scene) => void>, onProgress?: Nullable<(event: ISceneLoaderProgressEvent) => void>, onError?: Nullable<(scene: Scene, message: string, exception?: any) => void>, pluginExtension?: Nullable<string>): Promise<Scene>;
    }


    /**
     * Class used to represent data loading progression
     */
    export class SceneLoaderFlags {
        private static _ForceFullSceneLoadingForIncremental;
        private static _ShowLoadingScreen;
        private static _CleanBoneMatrixWeights;
        private static _LoggingLevel;
        /**
         * Gets or sets a boolean indicating if entire scene must be loaded even if scene contains incremental data
         */
        static get ForceFullSceneLoadingForIncremental(): boolean;
        static set ForceFullSceneLoadingForIncremental(value: boolean);
        /**
         * Gets or sets a boolean indicating if loading screen must be displayed while loading a scene
         */
        static get ShowLoadingScreen(): boolean;
        static set ShowLoadingScreen(value: boolean);
        /**
         * Defines the current logging level (while loading the scene)
         * @ignorenaming
         */
        static get loggingLevel(): number;
        static set loggingLevel(value: number);
        /**
         * Gets or set a boolean indicating if matrix weights must be cleaned upon loading
         */
        static get CleanBoneMatrixWeights(): boolean;
        static set CleanBoneMatrixWeights(value: boolean);
    }


    /**
     * Background material used to create an efficient environment around your scene.
     */
    export class BackgroundMaterial extends PushMaterial {
        /**
         * Standard reflectance value at parallel view angle.
         */
        static StandardReflectance0: number;
        /**
         * Standard reflectance value at grazing angle.
         */
        static StandardReflectance90: number;
        protected _primaryColor: Color3;
        /**
         * Key light Color (multiply against the environment texture)
         */
        primaryColor: Color3;
        protected __perceptualColor: Nullable<Color3>;
        /**
         * Experimental Internal Use Only.
         *
         * Key light Color in "perceptual value" meaning the color you would like to see on screen.
         * This acts as a helper to set the primary color to a more "human friendly" value.
         * Conversion to linear space as well as exposure and tone mapping correction will be applied to keep the
         * output color as close as possible from the chosen value.
         * (This does not account for contrast color grading and color curves as they are considered post effect and not directly
         * part of lighting setup.)
         */
        get _perceptualColor(): Nullable<Color3>;
        set _perceptualColor(value: Nullable<Color3>);
        protected _primaryColorShadowLevel: float;
        /**
         * Defines the level of the shadows (dark area of the reflection map) in order to help scaling the colors.
         * The color opposite to the primary color is used at the level chosen to define what the black area would look.
         */
        get primaryColorShadowLevel(): float;
        set primaryColorShadowLevel(value: float);
        protected _primaryColorHighlightLevel: float;
        /**
         * Defines the level of the highlights (highlight area of the reflection map) in order to help scaling the colors.
         * The primary color is used at the level chosen to define what the white area would look.
         */
        get primaryColorHighlightLevel(): float;
        set primaryColorHighlightLevel(value: float);
        protected _reflectionTexture: Nullable<BaseTexture>;
        /**
         * Reflection Texture used in the material.
         * Should be author in a specific way for the best result (refer to the documentation).
         */
        reflectionTexture: Nullable<BaseTexture>;
        protected _reflectionBlur: float;
        /**
         * Reflection Texture level of blur.
         *
         * Can be use to reuse an existing HDR Texture and target a specific LOD to prevent authoring the
         * texture twice.
         */
        reflectionBlur: float;
        protected _diffuseTexture: Nullable<BaseTexture>;
        /**
         * Diffuse Texture used in the material.
         * Should be author in a specific way for the best result (refer to the documentation).
         */
        diffuseTexture: Nullable<BaseTexture>;
        protected _shadowLights: Nullable<IShadowLight[]>;
        /**
         * Specify the list of lights casting shadow on the material.
         * All scene shadow lights will be included if null.
         */
        shadowLights: Nullable<IShadowLight[]>;
        protected _shadowLevel: float;
        /**
         * Helps adjusting the shadow to a softer level if required.
         * 0 means black shadows and 1 means no shadows.
         */
        shadowLevel: float;
        protected _sceneCenter: Vector3;
        /**
         * In case of opacity Fresnel or reflection falloff, this is use as a scene center.
         * It is usually zero but might be interesting to modify according to your setup.
         */
        sceneCenter: Vector3;
        protected _opacityFresnel: boolean;
        /**
         * This helps specifying that the material is falling off to the sky box at grazing angle.
         * This helps ensuring a nice transition when the camera goes under the ground.
         */
        opacityFresnel: boolean;
        protected _reflectionFresnel: boolean;
        /**
         * This helps specifying that the material is falling off from diffuse to the reflection texture at grazing angle.
         * This helps adding a mirror texture on the ground.
         */
        reflectionFresnel: boolean;
        protected _reflectionFalloffDistance: number;
        /**
         * This helps specifying the falloff radius off the reflection texture from the sceneCenter.
         * This helps adding a nice falloff effect to the reflection if used as a mirror for instance.
         */
        reflectionFalloffDistance: number;
        protected _reflectionAmount: number;
        /**
         * This specifies the weight of the reflection against the background in case of reflection Fresnel.
         */
        reflectionAmount: number;
        protected _reflectionReflectance0: number;
        /**
         * This specifies the weight of the reflection at grazing angle.
         */
        reflectionReflectance0: number;
        protected _reflectionReflectance90: number;
        /**
         * This specifies the weight of the reflection at a perpendicular point of view.
         */
        reflectionReflectance90: number;
        /**
         * Sets the reflection reflectance fresnel values according to the default standard
         * empirically know to work well :-)
         */
        set reflectionStandardFresnelWeight(value: number);
        protected _useRGBColor: boolean;
        /**
         * Helps to directly use the maps channels instead of their level.
         */
        useRGBColor: boolean;
        protected _enableNoise: boolean;
        /**
         * This helps reducing the banding effect that could occur on the background.
         */
        enableNoise: boolean;
        /**
         * The current fov(field of view) multiplier, 0.0 - 2.0. Defaults to 1.0. Lower values "zoom in" and higher values "zoom out".
         * Best used when trying to implement visual zoom effects like fish-eye or binoculars while not adjusting camera fov.
         * Recommended to be keep at 1.0 except for special cases.
         */
        get fovMultiplier(): number;
        set fovMultiplier(value: number);
        private _fovMultiplier;
        /**
         * Enable the FOV adjustment feature controlled by fovMultiplier.
         */
        useEquirectangularFOV: boolean;
        private _maxSimultaneousLights;
        /**
         * Number of Simultaneous lights allowed on the material.
         */
        maxSimultaneousLights: int;
        private _shadowOnly;
        /**
         * Make the material only render shadows
         */
        shadowOnly: boolean;
        /**
         * Default configuration related to image processing available in the Background Material.
         */
        protected _imageProcessingConfiguration: ImageProcessingConfiguration;
        /**
         * Keep track of the image processing observer to allow dispose and replace.
         */
        private _imageProcessingObserver;
        /**
         * Attaches a new image processing configuration to the PBR Material.
         * @param configuration (if null the scene configuration will be use)
         */
        protected _attachImageProcessingConfiguration(configuration: Nullable<ImageProcessingConfiguration>): void;
        /**
         * Gets the image processing configuration used either in this material.
         */
        get imageProcessingConfiguration(): Nullable<ImageProcessingConfiguration>;
        /**
         * Sets the Default image processing configuration used either in the this material.
         *
         * If sets to null, the scene one is in use.
         */
        set imageProcessingConfiguration(value: Nullable<ImageProcessingConfiguration>);
        /**
         * Gets whether the color curves effect is enabled.
         */
        get cameraColorCurvesEnabled(): boolean;
        /**
         * Sets whether the color curves effect is enabled.
         */
        set cameraColorCurvesEnabled(value: boolean);
        /**
         * Gets whether the color grading effect is enabled.
         */
        get cameraColorGradingEnabled(): boolean;
        /**
         * Gets whether the color grading effect is enabled.
         */
        set cameraColorGradingEnabled(value: boolean);
        /**
         * Gets whether tonemapping is enabled or not.
         */
        get cameraToneMappingEnabled(): boolean;
        /**
         * Sets whether tonemapping is enabled or not
         */
        set cameraToneMappingEnabled(value: boolean);
        /**
         * The camera exposure used on this material.
         * This property is here and not in the camera to allow controlling exposure without full screen post process.
         * This corresponds to a photographic exposure.
         */
        get cameraExposure(): float;
        /**
         * The camera exposure used on this material.
         * This property is here and not in the camera to allow controlling exposure without full screen post process.
         * This corresponds to a photographic exposure.
         */
        set cameraExposure(value: float);
        /**
         * Gets The camera contrast used on this material.
         */
        get cameraContrast(): float;
        /**
         * Sets The camera contrast used on this material.
         */
        set cameraContrast(value: float);
        /**
         * Gets the Color Grading 2D Lookup Texture.
         */
        get cameraColorGradingTexture(): Nullable<BaseTexture>;
        /**
         * Sets the Color Grading 2D Lookup Texture.
         */
        set cameraColorGradingTexture(value: Nullable<BaseTexture>);
        /**
         * The color grading curves provide additional color adjustment that is applied after any color grading transform (3D LUT).
         * They allow basic adjustment of saturation and small exposure adjustments, along with color filter tinting to provide white balance adjustment or more stylistic effects.
         * These are similar to controls found in many professional imaging or colorist software. The global controls are applied to the entire image. For advanced tuning, extra controls are provided to adjust the shadow, midtone and highlight areas of the image;
         * corresponding to low luminance, medium luminance, and high luminance areas respectively.
         */
        get cameraColorCurves(): Nullable<ColorCurves>;
        /**
         * The color grading curves provide additional color adjustment that is applied after any color grading transform (3D LUT).
         * They allow basic adjustment of saturation and small exposure adjustments, along with color filter tinting to provide white balance adjustment or more stylistic effects.
         * These are similar to controls found in many professional imaging or colorist software. The global controls are applied to the entire image. For advanced tuning, extra controls are provided to adjust the shadow, midtone and highlight areas of the image;
         * corresponding to low luminance, medium luminance, and high luminance areas respectively.
         */
        set cameraColorCurves(value: Nullable<ColorCurves>);
        /**
         * Due to a bug in iOS10, video tags (which are using the background material) are in BGR and not RGB.
         * Setting this flag to true (not done automatically!) will convert it back to RGB.
         */
        switchToBGR: boolean;
        private _renderTargets;
        private _reflectionControls;
        private _white;
        private _primaryShadowColor;
        private _primaryHighlightColor;
        /**
         * Instantiates a Background Material in the given scene
         * @param name The friendly name of the material
         * @param scene The scene to add the material to
         */
        constructor(name: string, scene?: Scene);
        /**
         * Gets a boolean indicating that current material needs to register RTT
         */
        get hasRenderTargetTextures(): boolean;
        /**
         * The entire material has been created in order to prevent overdraw.
         * @returns false
         */
        needAlphaTesting(): boolean;
        /**
         * The entire material has been created in order to prevent overdraw.
         * @returns true if blending is enable
         */
        needAlphaBlending(): boolean;
        /**
         * Checks whether the material is ready to be rendered for a given mesh.
         * @param mesh The mesh to render
         * @param subMesh The submesh to check against
         * @param useInstances Specify wether or not the material is used with instances
         * @returns true if all the dependencies are ready (Textures, Effects...)
         */
        isReadyForSubMesh(mesh: AbstractMesh, subMesh: SubMesh, useInstances?: boolean): boolean;
        /**
         * Compute the primary color according to the chosen perceptual color.
         */
        private _computePrimaryColorFromPerceptualColor;
        /**
         * Compute the highlights and shadow colors according to their chosen levels.
         */
        private _computePrimaryColors;
        /**
         * Build the uniform buffer used in the material.
         */
        buildUniformLayout(): void;
        /**
         * Unbind the material.
         */
        unbind(): void;
        /**
         * Bind only the world matrix to the material.
         * @param world The world matrix to bind.
         */
        bindOnlyWorldMatrix(world: Matrix): void;
        /**
         * Bind the material for a dedicated submeh (every used meshes will be considered opaque).
         * @param world The world matrix to bind.
         * @param mesh
         * @param subMesh The submesh to bind for.
         */
        bindForSubMesh(world: Matrix, mesh: Mesh, subMesh: SubMesh): void;
        /**
         * Checks to see if a texture is used in the material.
         * @param texture - Base texture to use.
         * @returns - Boolean specifying if a texture is used in the material.
         */
        hasTexture(texture: BaseTexture): boolean;
        /**
         * Dispose the material.
         * @param forceDisposeEffect Force disposal of the associated effect.
         * @param forceDisposeTextures Force disposal of the associated textures.
         */
        dispose(forceDisposeEffect?: boolean, forceDisposeTextures?: boolean): void;
        /**
         * Clones the material.
         * @param name The cloned name.
         * @returns The cloned material.
         */
        clone(name: string): BackgroundMaterial;
        /**
         * Serializes the current material to its JSON representation.
         * @returns The JSON representation.
         */
        serialize(): any;
        /**
         * Gets the class name of the material
         * @returns "BackgroundMaterial"
         */
        getClassName(): string;
        /**
         * Parse a JSON input to create back a background material.
         * @param source The JSON data to parse
         * @param scene The scene to create the parsed material in
         * @param rootUrl The root url of the assets the material depends upon
         * @returns the instantiated BackgroundMaterial.
         */
        static Parse(source: any, scene: Scene, rootUrl: string): BackgroundMaterial;
    }




    /** @internal */
    export function addClipPlaneUniforms(uniforms: string[]): void;
    /** @internal */
    export function prepareDefinesForClipPlanes(primaryHolder: IClipPlanesHolder, secondaryHolder: IClipPlanesHolder, defines: string[] | Record<string, any>): boolean;
    /** @internal */
    export function bindClipPlane(effect: Effect, primaryHolder: IClipPlanesHolder, secondaryHolder: IClipPlanesHolder): void;


    /**
     * The color grading curves provide additional color adjustment that is applied after any color grading transform (3D LUT).
     * They allow basic adjustment of saturation and small exposure adjustments, along with color filter tinting to provide white balance adjustment or more stylistic effects.
     * These are similar to controls found in many professional imaging or colorist software. The global controls are applied to the entire image. For advanced tuning, extra controls are provided to adjust the shadow, midtone and highlight areas of the image;
     * corresponding to low luminance, medium luminance, and high luminance areas respectively.
     */
    export class ColorCurves {
        private _dirty;
        private _tempColor;
        private _globalCurve;
        private _highlightsCurve;
        private _midtonesCurve;
        private _shadowsCurve;
        private _positiveCurve;
        private _negativeCurve;
        private _globalHue;
        private _globalDensity;
        private _globalSaturation;
        private _globalExposure;
        /**
         * Gets the global Hue value.
         * The hue value is a standard HSB hue in the range [0,360] where 0=red, 120=green and 240=blue. The default value is 30 degrees (orange).
         */
        get globalHue(): number;
        /**
         * Sets the global Hue value.
         * The hue value is a standard HSB hue in the range [0,360] where 0=red, 120=green and 240=blue. The default value is 30 degrees (orange).
         */
        set globalHue(value: number);
        /**
         * Gets the global Density value.
         * The density value is in range [-100,+100] where 0 means the color filter has no effect and +100 means the color filter has maximum effect.
         * Values less than zero provide a filter of opposite hue.
         */
        get globalDensity(): number;
        /**
         * Sets the global Density value.
         * The density value is in range [-100,+100] where 0 means the color filter has no effect and +100 means the color filter has maximum effect.
         * Values less than zero provide a filter of opposite hue.
         */
        set globalDensity(value: number);
        /**
         * Gets the global Saturation value.
         * This is an adjustment value in the range [-100,+100], where the default value of 0.0 makes no adjustment, positive values increase saturation and negative values decrease saturation.
         */
        get globalSaturation(): number;
        /**
         * Sets the global Saturation value.
         * This is an adjustment value in the range [-100,+100], where the default value of 0.0 makes no adjustment, positive values increase saturation and negative values decrease saturation.
         */
        set globalSaturation(value: number);
        /**
         * Gets the global Exposure value.
         * This is an adjustment value in the range [-100,+100], where the default value of 0.0 makes no adjustment, positive values increase exposure and negative values decrease exposure.
         */
        get globalExposure(): number;
        /**
         * Sets the global Exposure value.
         * This is an adjustment value in the range [-100,+100], where the default value of 0.0 makes no adjustment, positive values increase exposure and negative values decrease exposure.
         */
        set globalExposure(value: number);
        private _highlightsHue;
        private _highlightsDensity;
        private _highlightsSaturation;
        private _highlightsExposure;
        /**
         * Gets the highlights Hue value.
         * The hue value is a standard HSB hue in the range [0,360] where 0=red, 120=green and 240=blue. The default value is 30 degrees (orange).
         */
        get highlightsHue(): number;
        /**
         * Sets the highlights Hue value.
         * The hue value is a standard HSB hue in the range [0,360] where 0=red, 120=green and 240=blue. The default value is 30 degrees (orange).
         */
        set highlightsHue(value: number);
        /**
         * Gets the highlights Density value.
         * The density value is in range [-100,+100] where 0 means the color filter has no effect and +100 means the color filter has maximum effect.
         * Values less than zero provide a filter of opposite hue.
         */
        get highlightsDensity(): number;
        /**
         * Sets the highlights Density value.
         * The density value is in range [-100,+100] where 0 means the color filter has no effect and +100 means the color filter has maximum effect.
         * Values less than zero provide a filter of opposite hue.
         */
        set highlightsDensity(value: number);
        /**
         * Gets the highlights Saturation value.
         * This is an adjustment value in the range [-100,+100], where the default value of 0.0 makes no adjustment, positive values increase saturation and negative values decrease saturation.
         */
        get highlightsSaturation(): number;
        /**
         * Sets the highlights Saturation value.
         * This is an adjustment value in the range [-100,+100], where the default value of 0.0 makes no adjustment, positive values increase saturation and negative values decrease saturation.
         */
        set highlightsSaturation(value: number);
        /**
         * Gets the highlights Exposure value.
         * This is an adjustment value in the range [-100,+100], where the default value of 0.0 makes no adjustment, positive values increase exposure and negative values decrease exposure.
         */
        get highlightsExposure(): number;
        /**
         * Sets the highlights Exposure value.
         * This is an adjustment value in the range [-100,+100], where the default value of 0.0 makes no adjustment, positive values increase exposure and negative values decrease exposure.
         */
        set highlightsExposure(value: number);
        private _midtonesHue;
        private _midtonesDensity;
        private _midtonesSaturation;
        private _midtonesExposure;
        /**
         * Gets the midtones Hue value.
         * The hue value is a standard HSB hue in the range [0,360] where 0=red, 120=green and 240=blue. The default value is 30 degrees (orange).
         */
        get midtonesHue(): number;
        /**
         * Sets the midtones Hue value.
         * The hue value is a standard HSB hue in the range [0,360] where 0=red, 120=green and 240=blue. The default value is 30 degrees (orange).
         */
        set midtonesHue(value: number);
        /**
         * Gets the midtones Density value.
         * The density value is in range [-100,+100] where 0 means the color filter has no effect and +100 means the color filter has maximum effect.
         * Values less than zero provide a filter of opposite hue.
         */
        get midtonesDensity(): number;
        /**
         * Sets the midtones Density value.
         * The density value is in range [-100,+100] where 0 means the color filter has no effect and +100 means the color filter has maximum effect.
         * Values less than zero provide a filter of opposite hue.
         */
        set midtonesDensity(value: number);
        /**
         * Gets the midtones Saturation value.
         * This is an adjustment value in the range [-100,+100], where the default value of 0.0 makes no adjustment, positive values increase saturation and negative values decrease saturation.
         */
        get midtonesSaturation(): number;
        /**
         * Sets the midtones Saturation value.
         * This is an adjustment value in the range [-100,+100], where the default value of 0.0 makes no adjustment, positive values increase saturation and negative values decrease saturation.
         */
        set midtonesSaturation(value: number);
        /**
         * Gets the midtones Exposure value.
         * This is an adjustment value in the range [-100,+100], where the default value of 0.0 makes no adjustment, positive values increase exposure and negative values decrease exposure.
         */
        get midtonesExposure(): number;
        /**
         * Sets the midtones Exposure value.
         * This is an adjustment value in the range [-100,+100], where the default value of 0.0 makes no adjustment, positive values increase exposure and negative values decrease exposure.
         */
        set midtonesExposure(value: number);
        private _shadowsHue;
        private _shadowsDensity;
        private _shadowsSaturation;
        private _shadowsExposure;
        /**
         * Gets the shadows Hue value.
         * The hue value is a standard HSB hue in the range [0,360] where 0=red, 120=green and 240=blue. The default value is 30 degrees (orange).
         */
        get shadowsHue(): number;
        /**
         * Sets the shadows Hue value.
         * The hue value is a standard HSB hue in the range [0,360] where 0=red, 120=green and 240=blue. The default value is 30 degrees (orange).
         */
        set shadowsHue(value: number);
        /**
         * Gets the shadows Density value.
         * The density value is in range [-100,+100] where 0 means the color filter has no effect and +100 means the color filter has maximum effect.
         * Values less than zero provide a filter of opposite hue.
         */
        get shadowsDensity(): number;
        /**
         * Sets the shadows Density value.
         * The density value is in range [-100,+100] where 0 means the color filter has no effect and +100 means the color filter has maximum effect.
         * Values less than zero provide a filter of opposite hue.
         */
        set shadowsDensity(value: number);
        /**
         * Gets the shadows Saturation value.
         * This is an adjustment value in the range [-100,+100], where the default value of 0.0 makes no adjustment, positive values increase saturation and negative values decrease saturation.
         */
        get shadowsSaturation(): number;
        /**
         * Sets the shadows Saturation value.
         * This is an adjustment value in the range [-100,+100], where the default value of 0.0 makes no adjustment, positive values increase saturation and negative values decrease saturation.
         */
        set shadowsSaturation(value: number);
        /**
         * Gets the shadows Exposure value.
         * This is an adjustment value in the range [-100,+100], where the default value of 0.0 makes no adjustment, positive values increase exposure and negative values decrease exposure.
         */
        get shadowsExposure(): number;
        /**
         * Sets the shadows Exposure value.
         * This is an adjustment value in the range [-100,+100], where the default value of 0.0 makes no adjustment, positive values increase exposure and negative values decrease exposure.
         */
        set shadowsExposure(value: number);
        /**
         * Returns the class name
         * @returns The class name
         */
        getClassName(): string;
        /**
         * Binds the color curves to the shader.
         * @param colorCurves The color curve to bind
         * @param effect The effect to bind to
         * @param positiveUniform The positive uniform shader parameter
         * @param neutralUniform The neutral uniform shader parameter
         * @param negativeUniform The negative uniform shader parameter
         */
        static Bind(colorCurves: ColorCurves, effect: Effect, positiveUniform?: string, neutralUniform?: string, negativeUniform?: string): void;
        /**
         * Prepare the list of uniforms associated with the ColorCurves effects.
         * @param uniformsList The list of uniforms used in the effect
         */
        static PrepareUniforms(uniformsList: string[]): void;
        /**
         * Returns color grading data based on a hue, density, saturation and exposure value.
         * @param hue
         * @param density
         * @param saturation The saturation.
         * @param exposure The exposure.
         * @param result The result data container.
         */
        private _getColorGradingDataToRef;
        /**
         * Takes an input slider value and returns an adjusted value that provides extra control near the centre.
         * @param value The input slider value in range [-100,100].
         * @returns Adjusted value.
         */
        private static _ApplyColorGradingSliderNonlinear;
        /**
         * Returns an RGBA Color4 based on Hue, Saturation and Brightness (also referred to as value, HSV).
         * @param hue The hue (H) input.
         * @param saturation The saturation (S) input.
         * @param brightness The brightness (B) input.
         * @param result
         * @result An RGBA color represented as Vector4.
         */
        private static _FromHSBToRef;
        /**
         * Returns a value clamped between min and max
         * @param value The value to clamp
         * @param min The minimum of value
         * @param max The maximum of value
         * @returns The clamped value.
         */
        private static _Clamp;
        /**
         * Clones the current color curve instance.
         * @returns The cloned curves
         */
        clone(): ColorCurves;
        /**
         * Serializes the current color curve instance to a json representation.
         * @returns a JSON representation
         */
        serialize(): any;
        /**
         * Parses the color curve from a json representation.
         * @param source the JSON source to parse
         * @returns The parsed curves
         */
        static Parse(source: any): ColorCurves;
    }


    /** @internal */
    export class DrawWrapper {
        effect: Nullable<Effect>;
        defines: Nullable<string | MaterialDefines>;
        materialContext?: IMaterialContext;
        drawContext?: IDrawContext;
        static IsWrapper(effect: Effect | DrawWrapper): effect is DrawWrapper;
        static GetEffect(effect: Effect | DrawWrapper): Nullable<Effect>;
        constructor(engine: ThinEngine, createMaterialContext?: boolean);
        setEffect(effect: Nullable<Effect>, defines?: Nullable<string | MaterialDefines>, resetContext?: boolean): void;
        dispose(): void;
    }


    /**
     * Options to be used when creating an effect.
     */
    export interface IEffectCreationOptions {
        /**
         * Attributes that will be used in the shader.
         */
        attributes: string[];
        /**
         * Uniform variable names that will be set in the shader.
         */
        uniformsNames: string[];
        /**
         * Uniform buffer variable names that will be set in the shader.
         */
        uniformBuffersNames: string[];
        /**
         * Sampler texture variable names that will be set in the shader.
         */
        samplers: string[];
        /**
         * Define statements that will be set in the shader.
         */
        defines: any;
        /**
         * Possible fallbacks for this effect to improve performance when needed.
         */
        fallbacks: Nullable<IEffectFallbacks>;
        /**
         * Callback that will be called when the shader is compiled.
         */
        onCompiled: Nullable<(effect: Effect) => void>;
        /**
         * Callback that will be called if an error occurs during shader compilation.
         */
        onError: Nullable<(effect: Effect, errors: string) => void>;
        /**
         * Parameters to be used with Babylons include syntax to iterate over an array (eg. {lights: 10})
         */
        indexParameters?: any;
        /**
         * Max number of lights that can be used in the shader.
         */
        maxSimultaneousLights?: number;
        /**
         * See https://developer.mozilla.org/en-US/docs/Web/API/WebGL2RenderingContext/transformFeedbackVaryings
         */
        transformFeedbackVaryings?: Nullable<string[]>;
        /**
         * If provided, will be called two times with the vertex and fragment code so that this code can be updated before it is compiled by the GPU
         */
        processFinalCode?: Nullable<ShaderCustomProcessingFunction>;
        /**
         * If provided, will be called two times with the vertex and fragment code so that this code can be updated after the #include have been processed
         */
        processCodeAfterIncludes?: Nullable<ShaderCustomProcessingFunction>;
        /**
         * Is this effect rendering to several color attachments ?
         */
        multiTarget?: boolean;
        /**
         * The language the shader is written in (default: GLSL)
         */
        shaderLanguage?: ShaderLanguage;
    }
    /**
     * Effect containing vertex and fragment shader that can be executed on an object.
     */
    export class Effect implements IDisposable {
        /**
         * Gets or sets the relative url used to load shaders if using the engine in non-minified mode
         */
        static get ShadersRepository(): string;
        static set ShadersRepository(repo: string);
        /**
         * Enable logging of the shader code when a compilation error occurs
         */
        static LogShaderCodeOnCompilationError: boolean;
        /**
         * Name of the effect.
         */
        name: any;
        /**
         * String container all the define statements that should be set on the shader.
         */
        defines: string;
        /**
         * Callback that will be called when the shader is compiled.
         */
        onCompiled: Nullable<(effect: Effect) => void>;
        /**
         * Callback that will be called if an error occurs during shader compilation.
         */
        onError: Nullable<(effect: Effect, errors: string) => void>;
        /**
         * Callback that will be called when effect is bound.
         */
        onBind: Nullable<(effect: Effect) => void>;
        /**
         * Unique ID of the effect.
         */
        uniqueId: number;
        /**
         * Observable that will be called when the shader is compiled.
         * It is recommended to use executeWhenCompile() or to make sure that scene.isReady() is called to get this observable raised.
         */
        onCompileObservable: Observable<Effect>;
        /**
         * Observable that will be called if an error occurs during shader compilation.
         */
        onErrorObservable: Observable<Effect>;
        /** @internal */
        _onBindObservable: Nullable<Observable<Effect>>;
        /**
         * @internal
         * Specifies if the effect was previously ready
         */
        _wasPreviouslyReady: boolean;
        /**
         * @internal
         * Forces the code from bindForSubMesh to be fully run the next time it is called
         * It is used in frozen mode to make sure the effect is properly rebound when a new effect is created
         */
        _forceRebindOnNextCall: boolean;
        /**
         * @internal
         * Specifies if the effect was previously using instances
         */
        _wasPreviouslyUsingInstances: Nullable<boolean>;
        private _isDisposed;
        /**
         * Observable that will be called when effect is bound.
         */
        get onBindObservable(): Observable<Effect>;
        /** @internal */
        _bonesComputationForcedToCPU: boolean;
        /** @internal */
        _uniformBuffersNames: {
            [key: string]: number;
        };
        /** @internal */
        _samplerList: string[];
        /** @internal */
        _multiTarget: boolean;
        private static _UniqueIdSeed;
        /** @internal */
        _engine: Engine;
        private _uniformBuffersNamesList;
        private _uniformsNames;
        private _samplers;
        private _isReady;
        private _compilationError;
        private _allFallbacksProcessed;
        private _attributesNames;
        private _attributes;
        private _attributeLocationByName;
        private _uniforms;
        /**
         * Key for the effect.
         * @internal
         */
        _key: string;
        private _indexParameters;
        private _fallbacks;
        private _vertexSourceCodeOverride;
        private _fragmentSourceCodeOverride;
        private _transformFeedbackVaryings;
        private _shaderLanguage;
        /**
         * Compiled shader to webGL program.
         * @internal
         */
        _pipelineContext: Nullable<IPipelineContext>;
        /** @internal */
        _vertexSourceCode: string;
        /** @internal */
        _fragmentSourceCode: string;
        /** @internal */
        private _vertexSourceCodeBeforeMigration;
        /** @internal */
        private _fragmentSourceCodeBeforeMigration;
        /** @internal */
        private _rawVertexSourceCode;
        /** @internal */
        private _rawFragmentSourceCode;
        private static _BaseCache;
        private _processingContext;
        /**
         * Instantiates an effect.
         * An effect can be used to create/manage/execute vertex and fragment shaders.
         * @param baseName Name of the effect.
         * @param attributesNamesOrOptions List of attribute names that will be passed to the shader or set of all options to create the effect.
         * @param uniformsNamesOrEngine List of uniform variable names that will be passed to the shader or the engine that will be used to render effect.
         * @param samplers List of sampler variables that will be passed to the shader.
         * @param engine Engine to be used to render the effect
         * @param defines Define statements to be added to the shader.
         * @param fallbacks Possible fallbacks for this effect to improve performance when needed.
         * @param onCompiled Callback that will be called when the shader is compiled.
         * @param onError Callback that will be called if an error occurs during shader compilation.
         * @param indexParameters Parameters to be used with Babylons include syntax to iterate over an array (eg. {lights: 10})
         * @param key Effect Key identifying uniquely compiled shader variants
         * @param shaderLanguage the language the shader is written in (default: GLSL)
         */
        constructor(baseName: any, attributesNamesOrOptions: string[] | IEffectCreationOptions, uniformsNamesOrEngine: string[] | ThinEngine, samplers?: Nullable<string[]>, engine?: ThinEngine, defines?: Nullable<string>, fallbacks?: Nullable<IEffectFallbacks>, onCompiled?: Nullable<(effect: Effect) => void>, onError?: Nullable<(effect: Effect, errors: string) => void>, indexParameters?: any, key?: string, shaderLanguage?: ShaderLanguage);
        private _useFinalCode;
        /**
         * Unique key for this effect
         */
        get key(): string;
        /**
         * If the effect has been compiled and prepared.
         * @returns if the effect is compiled and prepared.
         */
        isReady(): boolean;
        private _isReadyInternal;
        /**
         * The engine the effect was initialized with.
         * @returns the engine.
         */
        getEngine(): Engine;
        /**
         * The pipeline context for this effect
         * @returns the associated pipeline context
         */
        getPipelineContext(): Nullable<IPipelineContext>;
        /**
         * The set of names of attribute variables for the shader.
         * @returns An array of attribute names.
         */
        getAttributesNames(): string[];
        /**
         * Returns the attribute at the given index.
         * @param index The index of the attribute.
         * @returns The location of the attribute.
         */
        getAttributeLocation(index: number): number;
        /**
         * Returns the attribute based on the name of the variable.
         * @param name of the attribute to look up.
         * @returns the attribute location.
         */
        getAttributeLocationByName(name: string): number;
        /**
         * The number of attributes.
         * @returns the number of attributes.
         */
        getAttributesCount(): number;
        /**
         * Gets the index of a uniform variable.
         * @param uniformName of the uniform to look up.
         * @returns the index.
         */
        getUniformIndex(uniformName: string): number;
        /**
         * Returns the attribute based on the name of the variable.
         * @param uniformName of the uniform to look up.
         * @returns the location of the uniform.
         */
        getUniform(uniformName: string): Nullable<WebGLUniformLocation>;
        /**
         * Returns an array of sampler variable names
         * @returns The array of sampler variable names.
         */
        getSamplers(): string[];
        /**
         * Returns an array of uniform variable names
         * @returns The array of uniform variable names.
         */
        getUniformNames(): string[];
        /**
         * Returns an array of uniform buffer variable names
         * @returns The array of uniform buffer variable names.
         */
        getUniformBuffersNames(): string[];
        /**
         * Returns the index parameters used to create the effect
         * @returns The index parameters object
         */
        getIndexParameters(): any;
        /**
         * The error from the last compilation.
         * @returns the error string.
         */
        getCompilationError(): string;
        /**
         * Gets a boolean indicating that all fallbacks were used during compilation
         * @returns true if all fallbacks were used
         */
        allFallbacksProcessed(): boolean;
        /**
         * Adds a callback to the onCompiled observable and call the callback immediately if already ready.
         * @param func The callback to be used.
         */
        executeWhenCompiled(func: (effect: Effect) => void): void;
        private _checkIsReady;
        private _loadShader;
        /**
         * Gets the vertex shader source code of this effect
         * This is the final source code that will be compiled, after all the processing has been done (pre-processing applied, code injection/replacement, etc)
         */
        get vertexSourceCode(): string;
        /**
         * Gets the fragment shader source code of this effect
         * This is the final source code that will be compiled, after all the processing has been done (pre-processing applied, code injection/replacement, etc)
         */
        get fragmentSourceCode(): string;
        /**
         * Gets the vertex shader source code before migration.
         * This is the source code after the include directives have been replaced by their contents but before the code is migrated, i.e. before ShaderProcess._ProcessShaderConversion is executed.
         * This method is, among other things, responsible for parsing #if/#define directives as well as converting GLES2 syntax to GLES3 (in the case of WebGL).
         */
        get vertexSourceCodeBeforeMigration(): string;
        /**
         * Gets the fragment shader source code before migration.
         * This is the source code after the include directives have been replaced by their contents but before the code is migrated, i.e. before ShaderProcess._ProcessShaderConversion is executed.
         * This method is, among other things, responsible for parsing #if/#define directives as well as converting GLES2 syntax to GLES3 (in the case of WebGL).
         */
        get fragmentSourceCodeBeforeMigration(): string;
        /**
         * Gets the vertex shader source code before it has been modified by any processing
         */
        get rawVertexSourceCode(): string;
        /**
         * Gets the fragment shader source code before it has been modified by any processing
         */
        get rawFragmentSourceCode(): string;
        /**
         * Recompiles the webGL program
         * @param vertexSourceCode The source code for the vertex shader.
         * @param fragmentSourceCode The source code for the fragment shader.
         * @param onCompiled Callback called when completed.
         * @param onError Callback called on error.
         * @internal
         */
        _rebuildProgram(vertexSourceCode: string, fragmentSourceCode: string, onCompiled: (pipelineContext: IPipelineContext) => void, onError: (message: string) => void): void;
        /**
         * Prepares the effect
         * @internal
         */
        _prepareEffect(): void;
        private _getShaderCodeAndErrorLine;
        private _processCompilationErrors;
        /**
         * Checks if the effect is supported. (Must be called after compilation)
         */
        get isSupported(): boolean;
        /**
         * Binds a texture to the engine to be used as output of the shader.
         * @param channel Name of the output variable.
         * @param texture Texture to bind.
         * @internal
         */
        _bindTexture(channel: string, texture: Nullable<InternalTexture>): void;
        /**
         * Sets a texture on the engine to be used in the shader.
         * @param channel Name of the sampler variable.
         * @param texture Texture to set.
         */
        setTexture(channel: string, texture: Nullable<ThinTexture>): void;
        /**
         * Sets a depth stencil texture from a render target on the engine to be used in the shader.
         * @param channel Name of the sampler variable.
         * @param texture Texture to set.
         */
        setDepthStencilTexture(channel: string, texture: Nullable<RenderTargetTexture>): void;
        /**
         * Sets an array of textures on the engine to be used in the shader.
         * @param channel Name of the variable.
         * @param textures Textures to set.
         */
        setTextureArray(channel: string, textures: ThinTexture[]): void;
        /**
         * Sets a texture to be the input of the specified post process. (To use the output, pass in the next post process in the pipeline)
         * @param channel Name of the sampler variable.
         * @param postProcess Post process to get the input texture from.
         */
        setTextureFromPostProcess(channel: string, postProcess: Nullable<PostProcess>): void;
        /**
         * (Warning! setTextureFromPostProcessOutput may be desired instead)
         * Sets the input texture of the passed in post process to be input of this effect. (To use the output of the passed in post process use setTextureFromPostProcessOutput)
         * @param channel Name of the sampler variable.
         * @param postProcess Post process to get the output texture from.
         */
        setTextureFromPostProcessOutput(channel: string, postProcess: Nullable<PostProcess>): void;
        /**
         * Binds a buffer to a uniform.
         * @param buffer Buffer to bind.
         * @param name Name of the uniform variable to bind to.
         */
        bindUniformBuffer(buffer: DataBuffer, name: string): void;
        /**
         * Binds block to a uniform.
         * @param blockName Name of the block to bind.
         * @param index Index to bind.
         */
        bindUniformBlock(blockName: string, index: number): void;
        /**
         * Sets an integer value on a uniform variable.
         * @param uniformName Name of the variable.
         * @param value Value to be set.
         * @returns this effect.
         */
        setInt: (uniformName: string, value: number) => Effect;
        /**
         * Sets an int2 value on a uniform variable.
         * @param uniformName Name of the variable.
         * @param x First int in int2.
         * @param y Second int in int2.
         * @returns this effect.
         */
        setInt2: (uniformName: string, x: number, y: number) => Effect;
        /**
         * Sets an int3 value on a uniform variable.
         * @param uniformName Name of the variable.
         * @param x First int in int3.
         * @param y Second int in int3.
         * @param z Third int in int3.
         * @returns this effect.
         */
        setInt3: (uniformName: string, x: number, y: number, z: number) => Effect;
        /**
         * Sets an int4 value on a uniform variable.
         * @param uniformName Name of the variable.
         * @param x First int in int4.
         * @param y Second int in int4.
         * @param z Third int in int4.
         * @param w Fourth int in int4.
         * @returns this effect.
         */
        setInt4: (uniformName: string, x: number, y: number, z: number, w: number) => Effect;
        /**
         * Sets an int array on a uniform variable.
         * @param uniformName Name of the variable.
         * @param array array to be set.
         * @returns this effect.
         */
        setIntArray: (uniformName: string, array: Int32Array) => Effect;
        /**
         * Sets an int array 2 on a uniform variable. (Array is specified as single array eg. [1,2,3,4] will result in [[1,2],[3,4]] in the shader)
         * @param uniformName Name of the variable.
         * @param array array to be set.
         * @returns this effect.
         */
        setIntArray2: (uniformName: string, array: Int32Array) => Effect;
        /**
         * Sets an int array 3 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6] will result in [[1,2,3],[4,5,6]] in the shader)
         * @param uniformName Name of the variable.
         * @param array array to be set.
         * @returns this effect.
         */
        setIntArray3: (uniformName: string, array: Int32Array) => Effect;
        /**
         * Sets an int array 4 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6,7,8] will result in [[1,2,3,4],[5,6,7,8]] in the shader)
         * @param uniformName Name of the variable.
         * @param array array to be set.
         * @returns this effect.
         */
        setIntArray4: (uniformName: string, array: Int32Array) => Effect;
        /**
         * Sets an unsigned integer value on a uniform variable.
         * @param uniformName Name of the variable.
         * @param value Value to be set.
         * @returns this effect.
         */
        setUInt: (uniformName: string, value: number) => Effect;
        /**
         * Sets an unsigned int2 value on a uniform variable.
         * @param uniformName Name of the variable.
         * @param x First unsigned int in uint2.
         * @param y Second unsigned int in uint2.
         * @returns this effect.
         */
        setUInt2: (uniformName: string, x: number, y: number) => Effect;
        /**
         * Sets an unsigned int3 value on a uniform variable.
         * @param uniformName Name of the variable.
         * @param x First unsigned int in uint3.
         * @param y Second unsigned int in uint3.
         * @param z Third unsigned int in uint3.
         * @returns this effect.
         */
        setUInt3: (uniformName: string, x: number, y: number, z: number) => Effect;
        /**
         * Sets an unsigned int4 value on a uniform variable.
         * @param uniformName Name of the variable.
         * @param x First unsigned int in uint4.
         * @param y Second unsigned int in uint4.
         * @param z Third unsigned int in uint4.
         * @param w Fourth unsigned int in uint4.
         * @returns this effect.
         */
        setUInt4: (uniformName: string, x: number, y: number, z: number, w: number) => Effect;
        /**
         * Sets an unsigned int array on a uniform variable.
         * @param uniformName Name of the variable.
         * @param array array to be set.
         * @returns this effect.
         */
        setUIntArray: (uniformName: string, array: Uint32Array) => Effect;
        /**
         * Sets an unsigned int array 2 on a uniform variable. (Array is specified as single array eg. [1,2,3,4] will result in [[1,2],[3,4]] in the shader)
         * @param uniformName Name of the variable.
         * @param array array to be set.
         * @returns this effect.
         */
        setUIntArray2: (uniformName: string, array: Uint32Array) => Effect;
        /**
         * Sets an unsigned int array 3 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6] will result in [[1,2,3],[4,5,6]] in the shader)
         * @param uniformName Name of the variable.
         * @param array array to be set.
         * @returns this effect.
         */
        setUIntArray3: (uniformName: string, array: Uint32Array) => Effect;
        /**
         * Sets an unsigned int array 4 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6,7,8] will result in [[1,2,3,4],[5,6,7,8]] in the shader)
         * @param uniformName Name of the variable.
         * @param array array to be set.
         * @returns this effect.
         */
        setUIntArray4: (uniformName: string, array: Uint32Array) => Effect;
        /**
         * Sets an float array on a uniform variable.
         * @param uniformName Name of the variable.
         * @param array array to be set.
         * @returns this effect.
         */
        setFloatArray(uniformName: string, array: FloatArray): Effect;
        /**
         * Sets an float array 2 on a uniform variable. (Array is specified as single array eg. [1,2,3,4] will result in [[1,2],[3,4]] in the shader)
         * @param uniformName Name of the variable.
         * @param array array to be set.
         * @returns this effect.
         */
        setFloatArray2(uniformName: string, array: FloatArray): Effect;
        /**
         * Sets an float array 3 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6] will result in [[1,2,3],[4,5,6]] in the shader)
         * @param uniformName Name of the variable.
         * @param array array to be set.
         * @returns this effect.
         */
        setFloatArray3(uniformName: string, array: FloatArray): Effect;
        /**
         * Sets an float array 4 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6,7,8] will result in [[1,2,3,4],[5,6,7,8]] in the shader)
         * @param uniformName Name of the variable.
         * @param array array to be set.
         * @returns this effect.
         */
        setFloatArray4(uniformName: string, array: FloatArray): Effect;
        /**
         * Sets an array on a uniform variable.
         * @param uniformName Name of the variable.
         * @param array array to be set.
         * @returns this effect.
         */
        setArray: (uniformName: string, array: number[]) => Effect;
        /**
         * Sets an array 2 on a uniform variable. (Array is specified as single array eg. [1,2,3,4] will result in [[1,2],[3,4]] in the shader)
         * @param uniformName Name of the variable.
         * @param array array to be set.
         * @returns this effect.
         */
        setArray2: (uniformName: string, array: number[]) => Effect;
        /**
         * Sets an array 3 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6] will result in [[1,2,3],[4,5,6]] in the shader)
         * @param uniformName Name of the variable.
         * @param array array to be set.
         * @returns this effect.
         */
        setArray3: (uniformName: string, array: number[]) => Effect;
        /**
         * Sets an array 4 on a uniform variable. (Array is specified as single array eg. [1,2,3,4,5,6,7,8] will result in [[1,2,3,4],[5,6,7,8]] in the shader)
         * @param uniformName Name of the variable.
         * @param array array to be set.
         * @returns this effect.
         */
        setArray4: (uniformName: string, array: number[]) => Effect;
        /**
         * Sets matrices on a uniform variable.
         * @param uniformName Name of the variable.
         * @param matrices matrices to be set.
         * @returns this effect.
         */
        setMatrices: (uniformName: string, matrices: Float32Array | Array<number>) => Effect;
        /**
         * Sets matrix on a uniform variable.
         * @param uniformName Name of the variable.
         * @param matrix matrix to be set.
         * @returns this effect.
         */
        setMatrix: (uniformName: string, matrix: IMatrixLike) => Effect;
        /**
         * Sets a 3x3 matrix on a uniform variable. (Specified as [1,2,3,4,5,6,7,8,9] will result in [1,2,3][4,5,6][7,8,9] matrix)
         * @param uniformName Name of the variable.
         * @param matrix matrix to be set.
         * @returns this effect.
         */
        setMatrix3x3: (uniformName: string, matrix: Float32Array | Array<number>) => Effect;
        /**
         * Sets a 2x2 matrix on a uniform variable. (Specified as [1,2,3,4] will result in [1,2][3,4] matrix)
         * @param uniformName Name of the variable.
         * @param matrix matrix to be set.
         * @returns this effect.
         */
        setMatrix2x2: (uniformName: string, matrix: Float32Array | Array<number>) => Effect;
        /**
         * Sets a float on a uniform variable.
         * @param uniformName Name of the variable.
         * @param value value to be set.
         * @returns this effect.
         */
        setFloat: (uniformName: string, value: number) => Effect;
        /**
         * Sets a boolean on a uniform variable.
         * @param uniformName Name of the variable.
         * @param bool value to be set.
         * @returns this effect.
         */
        setBool(uniformName: string, bool: boolean): Effect;
        /**
         * Sets a Vector2 on a uniform variable.
         * @param uniformName Name of the variable.
         * @param vector2 vector2 to be set.
         * @returns this effect.
         */
        setVector2: (uniformName: string, vector2: IVector2Like) => Effect;
        /**
         * Sets a float2 on a uniform variable.
         * @param uniformName Name of the variable.
         * @param x First float in float2.
         * @param y Second float in float2.
         * @returns this effect.
         */
        setFloat2: (uniformName: string, x: number, y: number) => Effect;
        /**
         * Sets a Vector3 on a uniform variable.
         * @param uniformName Name of the variable.
         * @param vector3 Value to be set.
         * @returns this effect.
         */
        setVector3: (uniformName: string, vector3: IVector3Like) => Effect;
        /**
         * Sets a float3 on a uniform variable.
         * @param uniformName Name of the variable.
         * @param x First float in float3.
         * @param y Second float in float3.
         * @param z Third float in float3.
         * @returns this effect.
         */
        setFloat3: (uniformName: string, x: number, y: number, z: number) => Effect;
        /**
         * Sets a Vector4 on a uniform variable.
         * @param uniformName Name of the variable.
         * @param vector4 Value to be set.
         * @returns this effect.
         */
        setVector4: (uniformName: string, vector4: IVector4Like) => Effect;
        /**
         * Sets a Quaternion on a uniform variable.
         * @param uniformName Name of the variable.
         * @param quaternion Value to be set.
         * @returns this effect.
         */
        setQuaternion: (uniformName: string, quaternion: IQuaternionLike) => Effect;
        /**
         * Sets a float4 on a uniform variable.
         * @param uniformName Name of the variable.
         * @param x First float in float4.
         * @param y Second float in float4.
         * @param z Third float in float4.
         * @param w Fourth float in float4.
         * @returns this effect.
         */
        setFloat4: (uniformName: string, x: number, y: number, z: number, w: number) => Effect;
        /**
         * Sets a Color3 on a uniform variable.
         * @param uniformName Name of the variable.
         * @param color3 Value to be set.
         * @returns this effect.
         */
        setColor3: (uniformName: string, color3: IColor3Like) => Effect;
        /**
         * Sets a Color4 on a uniform variable.
         * @param uniformName Name of the variable.
         * @param color3 Value to be set.
         * @param alpha Alpha value to be set.
         * @returns this effect.
         */
        setColor4: (uniformName: string, color3: IColor3Like, alpha: number) => Effect;
        /**
         * Sets a Color4 on a uniform variable
         * @param uniformName defines the name of the variable
         * @param color4 defines the value to be set
         * @returns this effect.
         */
        setDirectColor4: (uniformName: string, color4: IColor4Like) => Effect;
        /**
         * Release all associated resources.
         **/
        dispose(): void;
        /**
         * This function will add a new shader to the shader store
         * @param name the name of the shader
         * @param pixelShader optional pixel shader content
         * @param vertexShader optional vertex shader content
         * @param shaderLanguage the language the shader is written in (default: GLSL)
         */
        static RegisterShader(name: string, pixelShader?: string, vertexShader?: string, shaderLanguage?: ShaderLanguage): void;
        /**
         * Store of each shader (The can be looked up using effect.key)
         */
        static ShadersStore: {
            [key: string]: string;
        };
        /**
         * Store of each included file for a shader (The can be looked up using effect.key)
         */
        static IncludesShadersStore: {
            [key: string]: string;
        };
        /**
         * Resets the cache of effects.
         */
        static ResetCache(): void;
    }


    /**
     * EffectFallbacks can be used to add fallbacks (properties to disable) to certain properties when desired to improve performance.
     * (Eg. Start at high quality with reflection and fog, if fps is low, remove reflection, if still low remove fog)
     */
    export class EffectFallbacks implements IEffectFallbacks {
        private _defines;
        private _currentRank;
        private _maxRank;
        private _mesh;
        /**
         * Removes the fallback from the bound mesh.
         */
        unBindMesh(): void;
        /**
         * Adds a fallback on the specified property.
         * @param rank The rank of the fallback (Lower ranks will be fallbacked to first)
         * @param define The name of the define in the shader
         */
        addFallback(rank: number, define: string): void;
        /**
         * Sets the mesh to use CPU skinning when needing to fallback.
         * @param rank The rank of the fallback (Lower ranks will be fallbacked to first)
         * @param mesh The mesh to use the fallbacks.
         */
        addCPUSkinningFallback(rank: number, mesh: AbstractMesh): void;
        /**
         * Checks to see if more fallbacks are still available.
         */
        get hasMoreFallbacks(): boolean;
        /**
         * Removes the defines that should be removed when falling back.
         * @param currentDefines defines the current define statements for the shader.
         * @param effect defines the current effect we try to compile
         * @returns The resulting defines with defines of the current rank removed.
         */
        reduce(currentDefines: string, effect: Effect): string;
    }


    /**
     * Effect Render Options
     */
    export interface IEffectRendererOptions {
        /**
         * Defines the vertices positions.
         */
        positions?: number[];
        /**
         * Defines the indices.
         */
        indices?: number[];
    }
    /**
     * Helper class to render one or more effects.
     * You can access the previous rendering in your shader by declaring a sampler named textureSampler
     */
    export class EffectRenderer {
        /**
         * The engine the effect renderer has been created for.
         */
        readonly engine: ThinEngine;
        private _vertexBuffers;
        private _indexBuffer;
        private _fullscreenViewport;
        private _onContextRestoredObserver;
        /**
         * Creates an effect renderer
         * @param engine the engine to use for rendering
         * @param options defines the options of the effect renderer
         */
        constructor(engine: ThinEngine, options?: IEffectRendererOptions);
        /**
         * Sets the current viewport in normalized coordinates 0-1
         * @param viewport Defines the viewport to set (defaults to 0 0 1 1)
         */
        setViewport(viewport?: Viewport): void;
        /**
         * Binds the embedded attributes buffer to the effect.
         * @param effect Defines the effect to bind the attributes for
         */
        bindBuffers(effect: Effect): void;
        /**
         * Sets the current effect wrapper to use during draw.
         * The effect needs to be ready before calling this api.
         * This also sets the default full screen position attribute.
         * @param effectWrapper Defines the effect to draw with
         */
        applyEffectWrapper(effectWrapper: EffectWrapper): void;
        /**
         * Restores engine states
         */
        restoreStates(): void;
        /**
         * Draws a full screen quad.
         */
        draw(): void;
        private _isRenderTargetTexture;
        /**
         * renders one or more effects to a specified texture
         * @param effectWrapper the effect to renderer
         * @param outputTexture texture to draw to, if null it will render to the screen.
         */
        render(effectWrapper: EffectWrapper, outputTexture?: Nullable<RenderTargetWrapper | IRenderTargetTexture>): void;
        /**
         * Disposes of the effect renderer
         */
        dispose(): void;
    }
    /**
     * Options to create an EffectWrapper
     */
    interface EffectWrapperCreationOptions {
        /**
         * Engine to use to create the effect
         */
        engine: ThinEngine;
        /**
         * Fragment shader for the effect
         */
        fragmentShader: string;
        /**
         * Use the shader store instead of direct source code
         */
        useShaderStore?: boolean;
        /**
         * Vertex shader for the effect
         */
        vertexShader?: string;
        /**
         * Attributes to use in the shader
         */
        attributeNames?: Array<string>;
        /**
         * Uniforms to use in the shader
         */
        uniformNames?: Array<string>;
        /**
         * Texture sampler names to use in the shader
         */
        samplerNames?: Array<string>;
        /**
         * Defines to use in the shader
         */
        defines?: Array<string>;
        /**
         * Callback when effect is compiled
         */
        onCompiled?: Nullable<(effect: Effect) => void>;
        /**
         * The friendly name of the effect displayed in Spector.
         */
        name?: string;
        /**
         * The language the shader is written in (default: GLSL)
         */
        shaderLanguage?: ShaderLanguage;
    }
    /**
     * Wraps an effect to be used for rendering
     */
    export class EffectWrapper {
        /**
         * Event that is fired right before the effect is drawn (should be used to update uniforms)
         */
        onApplyObservable: Observable<{}>;
        /**
         * The underlying effect
         */
        get effect(): Effect;
        set effect(effect: Effect);
        /** @internal */
        _drawWrapper: DrawWrapper;
        private _onContextRestoredObserver;
        /**
         * Creates an effect to be renderer
         * @param creationOptions options to create the effect
         */
        constructor(creationOptions: EffectWrapperCreationOptions);
        /**
         * Disposes of the effect wrapper
         */
        dispose(): void;
    }


    /**
     * Options to be used when creating a FresnelParameters.
     */
    export type IFresnelParametersCreationOptions = {
        /**
         * Define the color used on edges (grazing angle)
         */
        leftColor?: Color3;
        /**
         * Define the color used on center
         */
        rightColor?: Color3;
        /**
         * Define bias applied to computed fresnel term
         */
        bias?: number;
        /**
         * Defined the power exponent applied to fresnel term
         */
        power?: number;
        /**
         * Define if the fresnel effect is enable or not.
         */
        isEnabled?: boolean;
    };
    /**
     * Serialized format for FresnelParameters.
     */
    export type IFresnelParametersSerialized = {
        /**
         * Define the color used on edges (grazing angle) [as an array]
         */
        leftColor: number[];
        /**
         * Define the color used on center [as an array]
         */
        rightColor: number[];
        /**
         * Define bias applied to computed fresnel term
         */
        bias: number;
        /**
         * Defined the power exponent applied to fresnel term
         */
        power?: number;
        /**
         * Define if the fresnel effect is enable or not.
         */
        isEnabled: boolean;
    };
    /**
     * This represents all the required information to add a fresnel effect on a material:
     * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/fresnelParameters
     */
    export class FresnelParameters {
        private _isEnabled;
        /**
         * Define if the fresnel effect is enable or not.
         */
        get isEnabled(): boolean;
        set isEnabled(value: boolean);
        /**
         * Define the color used on edges (grazing angle)
         */
        leftColor: Color3;
        /**
         * Define the color used on center
         */
        rightColor: Color3;
        /**
         * Define bias applied to computed fresnel term
         */
        bias: number;
        /**
         * Defined the power exponent applied to fresnel term
         */
        power: number;
        /**
         * Creates a new FresnelParameters object.
         *
         * @param options provide your own settings to optionally to override defaults
         */
        constructor(options?: IFresnelParametersCreationOptions);
        /**
         * Clones the current fresnel and its values
         * @returns a clone fresnel configuration
         */
        clone(): FresnelParameters;
        /**
         * Determines equality between FresnelParameters objects
         * @param otherFresnelParameters defines the second operand
         * @returns true if the power, bias, leftColor, rightColor and isEnabled values are equal to the given ones
         */
        equals(otherFresnelParameters: DeepImmutable<FresnelParameters>): boolean;
        /**
         * Serializes the current fresnel parameters to a JSON representation.
         * @returns the JSON serialization
         */
        serialize(): IFresnelParametersSerialized;
        /**
         * Parse a JSON object and deserialize it to a new Fresnel parameter object.
         * @param parsedFresnelParameters Define the JSON representation
         * @returns the parsed parameters
         */
        static Parse(parsedFresnelParameters: IFresnelParametersSerialized): FresnelParameters;
    }


    /**
     * Interface used to define common properties for effect fallbacks
     */
    export interface IEffectFallbacks {
        /**
         * Removes the defines that should be removed when falling back.
         * @param currentDefines defines the current define statements for the shader.
         * @param effect defines the current effect we try to compile
         * @returns The resulting defines with defines of the current rank removed.
         */
        reduce(currentDefines: string, effect: Effect): string;
        /**
         * Removes the fallback from the bound mesh.
         */
        unBindMesh(): void;
        /**
         * Checks to see if more fallbacks are still available.
         */
        hasMoreFallbacks: boolean;
    }


    /**
     * Interface to follow in your material defines to integrate easily the
     * Image processing functions.
     * @internal
     */
    export interface IImageProcessingConfigurationDefines {
        IMAGEPROCESSING: boolean;
        VIGNETTE: boolean;
        VIGNETTEBLENDMODEMULTIPLY: boolean;
        VIGNETTEBLENDMODEOPAQUE: boolean;
        TONEMAPPING: boolean;
        TONEMAPPING_ACES: boolean;
        CONTRAST: boolean;
        EXPOSURE: boolean;
        COLORCURVES: boolean;
        COLORGRADING: boolean;
        COLORGRADING3D: boolean;
        SAMPLER3DGREENDEPTH: boolean;
        SAMPLER3DBGRMAP: boolean;
        DITHER: boolean;
        IMAGEPROCESSINGPOSTPROCESS: boolean;
        SKIPFINALCOLORCLAMP: boolean;
    }
    /**
     * @internal
     */
    export class ImageProcessingConfigurationDefines extends MaterialDefines implements IImageProcessingConfigurationDefines {
        IMAGEPROCESSING: boolean;
        VIGNETTE: boolean;
        VIGNETTEBLENDMODEMULTIPLY: boolean;
        VIGNETTEBLENDMODEOPAQUE: boolean;
        TONEMAPPING: boolean;
        TONEMAPPING_ACES: boolean;
        CONTRAST: boolean;
        COLORCURVES: boolean;
        COLORGRADING: boolean;
        COLORGRADING3D: boolean;
        SAMPLER3DGREENDEPTH: boolean;
        SAMPLER3DBGRMAP: boolean;
        DITHER: boolean;
        IMAGEPROCESSINGPOSTPROCESS: boolean;
        EXPOSURE: boolean;
        SKIPFINALCOLORCLAMP: boolean;
        constructor();
    }
    /**
     * This groups together the common properties used for image processing either in direct forward pass
     * or through post processing effect depending on the use of the image processing pipeline in your scene
     * or not.
     */
    export class ImageProcessingConfiguration {
        /**
         * Default tone mapping applied in BabylonJS.
         */
        static readonly TONEMAPPING_STANDARD = 0;
        /**
         * ACES Tone mapping (used by default in unreal and unity). This can help getting closer
         * to other engines rendering to increase portability.
         */
        static readonly TONEMAPPING_ACES = 1;
        /**
         * Color curves setup used in the effect if colorCurvesEnabled is set to true
         */
        colorCurves: Nullable<ColorCurves>;
        private _colorCurvesEnabled;
        /**
         * Gets whether the color curves effect is enabled.
         */
        get colorCurvesEnabled(): boolean;
        /**
         * Sets whether the color curves effect is enabled.
         */
        set colorCurvesEnabled(value: boolean);
        private _colorGradingTexture;
        /**
         * Color grading LUT texture used in the effect if colorGradingEnabled is set to true
         */
        get colorGradingTexture(): Nullable<BaseTexture>;
        /**
         * Color grading LUT texture used in the effect if colorGradingEnabled is set to true
         */
        set colorGradingTexture(value: Nullable<BaseTexture>);
        private _colorGradingEnabled;
        /**
         * Gets whether the color grading effect is enabled.
         */
        get colorGradingEnabled(): boolean;
        /**
         * Sets whether the color grading effect is enabled.
         */
        set colorGradingEnabled(value: boolean);
        private _colorGradingWithGreenDepth;
        /**
         * Gets whether the color grading effect is using a green depth for the 3d Texture.
         */
        get colorGradingWithGreenDepth(): boolean;
        /**
         * Sets whether the color grading effect is using a green depth for the 3d Texture.
         */
        set colorGradingWithGreenDepth(value: boolean);
        private _colorGradingBGR;
        /**
         * Gets whether the color grading texture contains BGR values.
         */
        get colorGradingBGR(): boolean;
        /**
         * Sets whether the color grading texture contains BGR values.
         */
        set colorGradingBGR(value: boolean);
        /** @internal */
        _exposure: number;
        /**
         * Gets the Exposure used in the effect.
         */
        get exposure(): number;
        /**
         * Sets the Exposure used in the effect.
         */
        set exposure(value: number);
        private _toneMappingEnabled;
        /**
         * Gets whether the tone mapping effect is enabled.
         */
        get toneMappingEnabled(): boolean;
        /**
         * Sets whether the tone mapping effect is enabled.
         */
        set toneMappingEnabled(value: boolean);
        private _toneMappingType;
        /**
         * Gets the type of tone mapping effect.
         */
        get toneMappingType(): number;
        /**
         * Sets the type of tone mapping effect used in BabylonJS.
         */
        set toneMappingType(value: number);
        protected _contrast: number;
        /**
         * Gets the contrast used in the effect.
         */
        get contrast(): number;
        /**
         * Sets the contrast used in the effect.
         */
        set contrast(value: number);
        /**
         * Vignette stretch size.
         */
        vignetteStretch: number;
        /**
         * Vignette center X Offset.
         */
        vignetteCenterX: number;
        /**
         * Vignette center Y Offset.
         */
        vignetteCenterY: number;
        /**
         * Back Compat: Vignette center Y Offset.
         * @deprecated use vignetteCenterY instead
         */
        get vignetteCentreY(): number;
        set vignetteCentreY(value: number);
        /**
         * Back Compat: Vignette center X Offset.
         * @deprecated use vignetteCenterX instead
         */
        get vignetteCentreX(): number;
        set vignetteCentreX(value: number);
        /**
         * Vignette weight or intensity of the vignette effect.
         */
        vignetteWeight: number;
        /**
         * Color of the vignette applied on the screen through the chosen blend mode (vignetteBlendMode)
         * if vignetteEnabled is set to true.
         */
        vignetteColor: Color4;
        /**
         * Camera field of view used by the Vignette effect.
         */
        vignetteCameraFov: number;
        private _vignetteBlendMode;
        /**
         * Gets the vignette blend mode allowing different kind of effect.
         */
        get vignetteBlendMode(): number;
        /**
         * Sets the vignette blend mode allowing different kind of effect.
         */
        set vignetteBlendMode(value: number);
        private _vignetteEnabled;
        /**
         * Gets whether the vignette effect is enabled.
         */
        get vignetteEnabled(): boolean;
        /**
         * Sets whether the vignette effect is enabled.
         */
        set vignetteEnabled(value: boolean);
        private _ditheringEnabled;
        /**
         * Gets whether the dithering effect is enabled.
         * The dithering effect can be used to reduce banding.
         */
        get ditheringEnabled(): boolean;
        /**
         * Sets whether the dithering effect is enabled.
         * The dithering effect can be used to reduce banding.
         */
        set ditheringEnabled(value: boolean);
        private _ditheringIntensity;
        /**
         * Gets the dithering intensity. 0 is no dithering. Default is 1.0 / 255.0.
         */
        get ditheringIntensity(): number;
        /**
         * Sets the dithering intensity. 0 is no dithering. Default is 1.0 / 255.0.
         */
        set ditheringIntensity(value: number);
        /** @internal */
        _skipFinalColorClamp: boolean;
        /**
         * If apply by post process is set to true, setting this to true will skip the the final color clamp step in the fragment shader
         * Applies to PBR materials.
         */
        get skipFinalColorClamp(): boolean;
        /**
         * If apply by post process is set to true, setting this to true will skip the the final color clamp step in the fragment shader
         * Applies to PBR materials.
         */
        set skipFinalColorClamp(value: boolean);
        /** @internal */
        _applyByPostProcess: boolean;
        /**
         * Gets whether the image processing is applied through a post process or not.
         */
        get applyByPostProcess(): boolean;
        /**
         * Sets whether the image processing is applied through a post process or not.
         */
        set applyByPostProcess(value: boolean);
        private _isEnabled;
        /**
         * Gets whether the image processing is enabled or not.
         */
        get isEnabled(): boolean;
        /**
         * Sets whether the image processing is enabled or not.
         */
        set isEnabled(value: boolean);
        /**
         * An event triggered when the configuration changes and requires Shader to Update some parameters.
         */
        onUpdateParameters: Observable<ImageProcessingConfiguration>;
        /**
         * Method called each time the image processing information changes requires to recompile the effect.
         */
        protected _updateParameters(): void;
        /**
         * Gets the current class name.
         * @returns "ImageProcessingConfiguration"
         */
        getClassName(): string;
        /**
         * Prepare the list of uniforms associated with the Image Processing effects.
         * @param uniforms The list of uniforms used in the effect
         * @param defines the list of defines currently in use
         */
        static PrepareUniforms(uniforms: string[], defines: IImageProcessingConfigurationDefines): void;
        /**
         * Prepare the list of samplers associated with the Image Processing effects.
         * @param samplersList The list of uniforms used in the effect
         * @param defines the list of defines currently in use
         */
        static PrepareSamplers(samplersList: string[], defines: IImageProcessingConfigurationDefines): void;
        /**
         * Prepare the list of defines associated to the shader.
         * @param defines the list of defines to complete
         * @param forPostProcess Define if we are currently in post process mode or not
         */
        prepareDefines(defines: IImageProcessingConfigurationDefines, forPostProcess?: boolean): void;
        /**
         * Returns true if all the image processing information are ready.
         * @returns True if ready, otherwise, false
         */
        isReady(): boolean;
        /**
         * Binds the image processing to the shader.
         * @param effect The effect to bind to
         * @param overrideAspectRatio Override the aspect ratio of the effect
         */
        bind(effect: Effect, overrideAspectRatio?: number): void;
        /**
         * Clones the current image processing instance.
         * @returns The cloned image processing
         */
        clone(): ImageProcessingConfiguration;
        /**
         * Serializes the current image processing instance to a json representation.
         * @returns a JSON representation
         */
        serialize(): any;
        /**
         * Parses the image processing from a json representation.
         * @param source the JSON source to parse
         * @returns The parsed image processing
         */
        static Parse(source: any): ImageProcessingConfiguration;
        private static _VIGNETTEMODE_MULTIPLY;
        private static _VIGNETTEMODE_OPAQUE;
        /**
         * Used to apply the vignette as a mix with the pixel color.
         */
        static get VIGNETTEMODE_MULTIPLY(): number;
        /**
         * Used to apply the vignette as a replacement of the pixel color.
         */
        static get VIGNETTEMODE_OPAQUE(): number;
    }




    /**
     * Options for compiling materials.
     */
    export interface IMaterialCompilationOptions {
        /**
         * Defines whether clip planes are enabled.
         */
        clipPlane: boolean;
        /**
         * Defines whether instances are enabled.
         */
        useInstances: boolean;
    }
    /**
     * Options passed when calling customShaderNameResolve
     */
    export interface ICustomShaderNameResolveOptions {
        /**
         * If provided, will be called two times with the vertex and fragment code so that this code can be updated before it is compiled by the GPU
         */
        processFinalCode?: Nullable<ShaderCustomProcessingFunction>;
    }
    /**
     * Base class for the main features of a material in Babylon.js
     */
    export class Material implements IAnimatable, IClipPlanesHolder {
        /**
         * Returns the triangle fill mode
         */
        static readonly TriangleFillMode = 0;
        /**
         * Returns the wireframe mode
         */
        static readonly WireFrameFillMode = 1;
        /**
         * Returns the point fill mode
         */
        static readonly PointFillMode = 2;
        /**
         * Returns the point list draw mode
         */
        static readonly PointListDrawMode = 3;
        /**
         * Returns the line list draw mode
         */
        static readonly LineListDrawMode = 4;
        /**
         * Returns the line loop draw mode
         */
        static readonly LineLoopDrawMode = 5;
        /**
         * Returns the line strip draw mode
         */
        static readonly LineStripDrawMode = 6;
        /**
         * Returns the triangle strip draw mode
         */
        static readonly TriangleStripDrawMode = 7;
        /**
         * Returns the triangle fan draw mode
         */
        static readonly TriangleFanDrawMode = 8;
        /**
         * Stores the clock-wise side orientation
         */
        static readonly ClockWiseSideOrientation = 0;
        /**
         * Stores the counter clock-wise side orientation
         */
        static readonly CounterClockWiseSideOrientation = 1;
        /**
         * The dirty texture flag value
         */
        static readonly TextureDirtyFlag = 1;
        /**
         * The dirty light flag value
         */
        static readonly LightDirtyFlag = 2;
        /**
         * The dirty fresnel flag value
         */
        static readonly FresnelDirtyFlag = 4;
        /**
         * The dirty attribute flag value
         */
        static readonly AttributesDirtyFlag = 8;
        /**
         * The dirty misc flag value
         */
        static readonly MiscDirtyFlag = 16;
        /**
         * The dirty prepass flag value
         */
        static readonly PrePassDirtyFlag = 32;
        /**
         * The all dirty flag value
         */
        static readonly AllDirtyFlag = 63;
        /**
         * MaterialTransparencyMode: No transparency mode, Alpha channel is not use.
         */
        static readonly MATERIAL_OPAQUE = 0;
        /**
         * MaterialTransparencyMode: Alpha Test mode, pixel are discarded below a certain threshold defined by the alpha cutoff value.
         */
        static readonly MATERIAL_ALPHATEST = 1;
        /**
         * MaterialTransparencyMode: Pixels are blended (according to the alpha mode) with the already drawn pixels in the current frame buffer.
         */
        static readonly MATERIAL_ALPHABLEND = 2;
        /**
         * MaterialTransparencyMode: Pixels are blended (according to the alpha mode) with the already drawn pixels in the current frame buffer.
         * They are also discarded below the alpha cutoff threshold to improve performances.
         */
        static readonly MATERIAL_ALPHATESTANDBLEND = 3;
        /**
         * The Whiteout method is used to blend normals.
         * Details of the algorithm can be found here: https://blog.selfshadow.com/publications/blending-in-detail/
         */
        static readonly MATERIAL_NORMALBLENDMETHOD_WHITEOUT = 0;
        /**
         * The Reoriented Normal Mapping method is used to blend normals.
         * Details of the algorithm can be found here: https://blog.selfshadow.com/publications/blending-in-detail/
         */
        static readonly MATERIAL_NORMALBLENDMETHOD_RNM = 1;
        /**
         * Event observable which raises global events common to all materials (like MaterialPluginEvent.Created)
         */
        static OnEventObservable: Observable<Material>;
        /**
         * Custom callback helping to override the default shader used in the material.
         */
        customShaderNameResolve: (shaderName: string, uniforms: string[], uniformBuffers: string[], samplers: string[], defines: MaterialDefines | string[], attributes?: string[], options?: ICustomShaderNameResolveOptions) => string;
        /**
         * Custom shadow depth material to use for shadow rendering instead of the in-built one
         */
        shadowDepthWrapper: Nullable<ShadowDepthWrapper>;
        /**
         * Gets or sets a boolean indicating that the material is allowed (if supported) to do shader hot swapping.
         * This means that the material can keep using a previous shader while a new one is being compiled.
         * This is mostly used when shader parallel compilation is supported (true by default)
         */
        allowShaderHotSwapping: boolean;
        /**
         * The ID of the material
         */
        id: string;
        /**
         * Gets or sets the unique id of the material
         */
        uniqueId: number;
        /** @internal */
        _loadedUniqueId: string;
        /**
         * The name of the material
         */
        name: string;
        /**
         * Gets or sets user defined metadata
         */
        metadata: any;
        /** @internal */
        _internalMetadata: any;
        /**
         * For internal use only. Please do not use.
         */
        reservedDataStore: any;
        /**
         * Specifies if the ready state should be checked on each call
         */
        checkReadyOnEveryCall: boolean;
        /**
         * Specifies if the ready state should be checked once
         */
        checkReadyOnlyOnce: boolean;
        /**
         * The state of the material
         */
        state: string;
        /**
         * If the material can be rendered to several textures with MRT extension
         */
        get canRenderToMRT(): boolean;
        /**
         * The alpha value of the material
         */
        protected _alpha: number;
        /**
         * List of inspectable custom properties (used by the Inspector)
         * @see https://doc.babylonjs.com/toolsAndResources/inspector#extensibility
         */
        inspectableCustomProperties: IInspectable[];
        /**
         * Sets the alpha value of the material
         */
        set alpha(value: number);
        /**
         * Gets the alpha value of the material
         */
        get alpha(): number;
        /**
         * Specifies if back face culling is enabled
         */
        protected _backFaceCulling: boolean;
        /**
         * Sets the culling state (true to enable culling, false to disable)
         */
        set backFaceCulling(value: boolean);
        /**
         * Gets the culling state
         */
        get backFaceCulling(): boolean;
        /**
         * Specifies if back or front faces should be culled (when culling is enabled)
         */
        protected _cullBackFaces: boolean;
        /**
         * Sets the type of faces that should be culled (true for back faces, false for front faces)
         */
        set cullBackFaces(value: boolean);
        /**
         * Gets the type of faces that should be culled
         */
        get cullBackFaces(): boolean;
        private _blockDirtyMechanism;
        /**
         * Block the dirty-mechanism for this specific material
         * When set to false after being true the material will be marked as dirty.
         */
        get blockDirtyMechanism(): boolean;
        set blockDirtyMechanism(value: boolean);
        /**
         * This allows you to modify the material without marking it as dirty after every change.
         * This function should be used if you need to make more than one dirty-enabling change to the material - adding a texture, setting a new fill mode and so on.
         * The callback will pass the material as an argument, so you can make your changes to it.
         * @param callback the callback to be executed that will update the material
         */
        atomicMaterialsUpdate(callback: (material: this) => void): void;
        /**
         * Stores the value for side orientation
         */
        sideOrientation: number;
        /**
         * Callback triggered when the material is compiled
         */
        onCompiled: Nullable<(effect: Effect) => void>;
        /**
         * Callback triggered when an error occurs
         */
        onError: Nullable<(effect: Effect, errors: string) => void>;
        /**
         * Callback triggered to get the render target textures
         */
        getRenderTargetTextures: Nullable<() => SmartArray<RenderTargetTexture>>;
        /**
         * Gets a boolean indicating that current material needs to register RTT
         */
        get hasRenderTargetTextures(): boolean;
        /**
         * Specifies if the material should be serialized
         */
        doNotSerialize: boolean;
        /**
         * @internal
         */
        _storeEffectOnSubMeshes: boolean;
        /**
         * Stores the animations for the material
         */
        animations: Nullable<Array<Animation>>;
        /**
         * An event triggered when the material is disposed
         */
        onDisposeObservable: Observable<Material>;
        /**
         * An observer which watches for dispose events
         */
        private _onDisposeObserver;
        private _onUnBindObservable;
        /**
         * Called during a dispose event
         */
        set onDispose(callback: () => void);
        private _onBindObservable;
        /**
         * An event triggered when the material is bound
         */
        get onBindObservable(): Observable<AbstractMesh>;
        /**
         * An observer which watches for bind events
         */
        private _onBindObserver;
        /**
         * Called during a bind event
         */
        set onBind(callback: (Mesh: AbstractMesh) => void);
        /**
         * An event triggered when the material is unbound
         */
        get onUnBindObservable(): Observable<Material>;
        protected _onEffectCreatedObservable: Nullable<Observable<{
            effect: Effect;
            subMesh: Nullable<SubMesh>;
        }>>;
        /**
         * An event triggered when the effect is (re)created
         */
        get onEffectCreatedObservable(): Observable<{
            effect: Effect;
            subMesh: Nullable<SubMesh>;
        }>;
        /**
         * Stores the value of the alpha mode
         */
        private _alphaMode;
        /**
         * Sets the value of the alpha mode.
         *
         * | Value | Type | Description |
         * | --- | --- | --- |
         * | 0 | ALPHA_DISABLE |   |
         * | 1 | ALPHA_ADD |   |
         * | 2 | ALPHA_COMBINE |   |
         * | 3 | ALPHA_SUBTRACT |   |
         * | 4 | ALPHA_MULTIPLY |   |
         * | 5 | ALPHA_MAXIMIZED |   |
         * | 6 | ALPHA_ONEONE |   |
         * | 7 | ALPHA_PREMULTIPLIED |   |
         * | 8 | ALPHA_PREMULTIPLIED_PORTERDUFF |   |
         * | 9 | ALPHA_INTERPOLATE |   |
         * | 10 | ALPHA_SCREENMODE |   |
         *
         */
        set alphaMode(value: number);
        /**
         * Gets the value of the alpha mode
         */
        get alphaMode(): number;
        /**
         * Stores the state of the need depth pre-pass value
         */
        private _needDepthPrePass;
        /**
         * Sets the need depth pre-pass value
         */
        set needDepthPrePass(value: boolean);
        /**
         * Gets the depth pre-pass value
         */
        get needDepthPrePass(): boolean;
        /**
         * Can this material render to prepass
         */
        get isPrePassCapable(): boolean;
        /**
         * Specifies if depth writing should be disabled
         */
        disableDepthWrite: boolean;
        /**
         * Specifies if color writing should be disabled
         */
        disableColorWrite: boolean;
        /**
         * Specifies if depth writing should be forced
         */
        forceDepthWrite: boolean;
        /**
         * Specifies the depth function that should be used. 0 means the default engine function
         */
        depthFunction: number;
        /**
         * Specifies if there should be a separate pass for culling
         */
        separateCullingPass: boolean;
        /**
         * Stores the state specifying if fog should be enabled
         */
        private _fogEnabled;
        /**
         * Sets the state for enabling fog
         */
        set fogEnabled(value: boolean);
        /**
         * Gets the value of the fog enabled state
         */
        get fogEnabled(): boolean;
        /**
         * Stores the size of points
         */
        pointSize: number;
        /**
         * Stores the z offset Factor value
         */
        zOffset: number;
        /**
         * Stores the z offset Units value
         */
        zOffsetUnits: number;
        get wireframe(): boolean;
        /**
         * Sets the state of wireframe mode
         */
        set wireframe(value: boolean);
        /**
         * Gets the value specifying if point clouds are enabled
         */
        get pointsCloud(): boolean;
        /**
         * Sets the state of point cloud mode
         */
        set pointsCloud(value: boolean);
        /**
         * Gets the material fill mode
         */
        get fillMode(): number;
        /**
         * Sets the material fill mode
         */
        set fillMode(value: number);
        /**
         * Gets or sets the active clipplane 1
         */
        clipPlane: Nullable<Plane>;
        /**
         * Gets or sets the active clipplane 2
         */
        clipPlane2: Nullable<Plane>;
        /**
         * Gets or sets the active clipplane 3
         */
        clipPlane3: Nullable<Plane>;
        /**
         * Gets or sets the active clipplane 4
         */
        clipPlane4: Nullable<Plane>;
        /**
         * Gets or sets the active clipplane 5
         */
        clipPlane5: Nullable<Plane>;
        /**
         * Gets or sets the active clipplane 6
         */
        clipPlane6: Nullable<Plane>;
        /**
         * Gives access to the stencil properties of the material
         */
        readonly stencil: MaterialStencilState;
        /**
         * @internal
         * Stores the effects for the material
         */
        protected _materialContext: IMaterialContext | undefined;
        protected _drawWrapper: DrawWrapper;
        /** @internal */
        _getDrawWrapper(): DrawWrapper;
        /**
         * @internal
         */
        _setDrawWrapper(drawWrapper: DrawWrapper): void;
        /**
         * Specifies if uniform buffers should be used
         */
        private _useUBO;
        /**
         * Stores a reference to the scene
         */
        private _scene;
        protected _needToBindSceneUbo: boolean;
        /**
         * Stores the fill mode state
         */
        private _fillMode;
        /**
         * Specifies if the depth write state should be cached
         */
        private _cachedDepthWriteState;
        /**
         * Specifies if the color write state should be cached
         */
        private _cachedColorWriteState;
        /**
         * Specifies if the depth function state should be cached
         */
        private _cachedDepthFunctionState;
        /**
         * Stores the uniform buffer
         * @internal
         */
        _uniformBuffer: UniformBuffer;
        /** @internal */
        _indexInSceneMaterialArray: number;
        /** @internal */
        meshMap: Nullable<{
            [id: string]: AbstractMesh | undefined;
        }>;
        /** @internal */
        _parentContainer: Nullable<AbstractScene>;
        /** @internal */
        _dirtyCallbacks: {
            [code: number]: () => void;
        };
        /** @internal */
        _uniformBufferLayoutBuilt: boolean;
        protected _eventInfo: MaterialPluginCreated & MaterialPluginDisposed & MaterialPluginHasTexture & MaterialPluginIsReadyForSubMesh & MaterialPluginGetDefineNames & MaterialPluginPrepareEffect & MaterialPluginPrepareDefines & MaterialPluginPrepareUniformBuffer & MaterialPluginBindForSubMesh & MaterialPluginGetAnimatables & MaterialPluginGetActiveTextures & MaterialPluginFillRenderTargetTextures & MaterialPluginHasRenderTargetTextures & MaterialPluginHardBindForSubMesh;
        /** @internal */
        _callbackPluginEventGeneric: (id: number, info: MaterialPluginGetActiveTextures | MaterialPluginGetAnimatables | MaterialPluginHasTexture | MaterialPluginDisposed | MaterialPluginGetDefineNames | MaterialPluginPrepareEffect | MaterialPluginPrepareUniformBuffer) => void;
        /** @internal */
        _callbackPluginEventIsReadyForSubMesh: (eventData: MaterialPluginIsReadyForSubMesh) => void;
        /** @internal */
        _callbackPluginEventPrepareDefines: (eventData: MaterialPluginPrepareDefines) => void;
        /** @internal */
        _callbackPluginEventPrepareDefinesBeforeAttributes: (eventData: MaterialPluginPrepareDefines) => void;
        /** @internal */
        _callbackPluginEventHardBindForSubMesh: (eventData: MaterialPluginHardBindForSubMesh) => void;
        /** @internal */
        _callbackPluginEventBindForSubMesh: (eventData: MaterialPluginBindForSubMesh) => void;
        /** @internal */
        _callbackPluginEventHasRenderTargetTextures: (eventData: MaterialPluginHasRenderTargetTextures) => void;
        /** @internal */
        _callbackPluginEventFillRenderTargetTextures: (eventData: MaterialPluginFillRenderTargetTextures) => void;
        /**
         * Creates a material instance
         * @param name defines the name of the material
         * @param scene defines the scene to reference
         * @param doNotAdd specifies if the material should be added to the scene
         */
        constructor(name: string, scene?: Nullable<Scene>, doNotAdd?: boolean);
        /**
         * Returns a string representation of the current material
         * @param fullDetails defines a boolean indicating which levels of logging is desired
         * @returns a string with material information
         */
        toString(fullDetails?: boolean): string;
        /**
         * Gets the class name of the material
         * @returns a string with the class name of the material
         */
        getClassName(): string;
        /** @internal */
        get _isMaterial(): boolean;
        /**
         * Specifies if updates for the material been locked
         */
        get isFrozen(): boolean;
        /**
         * Locks updates for the material
         */
        freeze(): void;
        /**
         * Unlocks updates for the material
         */
        unfreeze(): void;
        /**
         * Specifies if the material is ready to be used
         * @param mesh defines the mesh to check
         * @param useInstances specifies if instances should be used
         * @returns a boolean indicating if the material is ready to be used
         */
        isReady(mesh?: AbstractMesh, useInstances?: boolean): boolean;
        /**
         * Specifies that the submesh is ready to be used
         * @param mesh defines the mesh to check
         * @param subMesh defines which submesh to check
         * @param useInstances specifies that instances should be used
         * @returns a boolean indicating that the submesh is ready or not
         */
        isReadyForSubMesh(mesh: AbstractMesh, subMesh: SubMesh, useInstances?: boolean): boolean;
        /**
         * Returns the material effect
         * @returns the effect associated with the material
         */
        getEffect(): Nullable<Effect>;
        /**
         * Returns the current scene
         * @returns a Scene
         */
        getScene(): Scene;
        /**
         * Enforces alpha test in opaque or blend mode in order to improve the performances of some situations.
         */
        protected _forceAlphaTest: boolean;
        /**
         * The transparency mode of the material.
         */
        protected _transparencyMode: Nullable<number>;
        /**
         * Gets the current transparency mode.
         */
        get transparencyMode(): Nullable<number>;
        /**
         * Sets the transparency mode of the material.
         *
         * | Value | Type                                | Description |
         * | ----- | ----------------------------------- | ----------- |
         * | 0     | OPAQUE                              |             |
         * | 1     | ALPHATEST                           |             |
         * | 2     | ALPHABLEND                          |             |
         * | 3     | ALPHATESTANDBLEND                   |             |
         *
         */
        set transparencyMode(value: Nullable<number>);
        /**
         * Returns true if alpha blending should be disabled.
         */
        protected get _disableAlphaBlending(): boolean;
        /**
         * Specifies whether or not this material should be rendered in alpha blend mode.
         * @returns a boolean specifying if alpha blending is needed
         */
        needAlphaBlending(): boolean;
        /**
         * Specifies if the mesh will require alpha blending
         * @param mesh defines the mesh to check
         * @returns a boolean specifying if alpha blending is needed for the mesh
         */
        needAlphaBlendingForMesh(mesh: AbstractMesh): boolean;
        /**
         * Specifies whether or not this material should be rendered in alpha test mode.
         * @returns a boolean specifying if an alpha test is needed.
         */
        needAlphaTesting(): boolean;
        /**
         * Specifies if material alpha testing should be turned on for the mesh
         * @param mesh defines the mesh to check
         */
        protected _shouldTurnAlphaTestOn(mesh: AbstractMesh): boolean;
        /**
         * Gets the texture used for the alpha test
         * @returns the texture to use for alpha testing
         */
        getAlphaTestTexture(): Nullable<BaseTexture>;
        /**
         * Marks the material to indicate that it needs to be re-calculated
         * @param forceMaterialDirty - Forces the material to be marked as dirty for all components (same as this.markAsDirty(Material.AllDirtyFlag)). You should use this flag if the material is frozen and you want to force a recompilation.
         */
        markDirty(forceMaterialDirty?: boolean): void;
        /**
         * @internal
         */
        _preBind(effect?: Effect | DrawWrapper, overrideOrientation?: Nullable<number>): boolean;
        /**
         * Binds the material to the mesh
         * @param world defines the world transformation matrix
         * @param mesh defines the mesh to bind the material to
         */
        bind(world: Matrix, mesh?: Mesh): void;
        /**
         * Initializes the uniform buffer layout for the shader.
         */
        buildUniformLayout(): void;
        /**
         * Binds the submesh to the material
         * @param world defines the world transformation matrix
         * @param mesh defines the mesh containing the submesh
         * @param subMesh defines the submesh to bind the material to
         */
        bindForSubMesh(world: Matrix, mesh: Mesh, subMesh: SubMesh): void;
        /**
         * Binds the world matrix to the material
         * @param world defines the world transformation matrix
         */
        bindOnlyWorldMatrix(world: Matrix): void;
        /**
         * Binds the view matrix to the effect
         * @param effect defines the effect to bind the view matrix to
         */
        bindView(effect: Effect): void;
        /**
         * Binds the view projection and projection matrices to the effect
         * @param effect defines the effect to bind the view projection and projection matrices to
         */
        bindViewProjection(effect: Effect): void;
        /**
         * Binds the view matrix to the effect
         * @param effect defines the effect to bind the view matrix to
         * @param variableName name of the shader variable that will hold the eye position
         */
        bindEyePosition(effect: Effect, variableName?: string): void;
        /**
         * Processes to execute after binding the material to a mesh
         * @param mesh defines the rendered mesh
         * @param effect
         */
        protected _afterBind(mesh?: Mesh, effect?: Nullable<Effect>): void;
        /**
         * Unbinds the material from the mesh
         */
        unbind(): void;
        /**
         * Returns the animatable textures.
         * @returns - Array of animatable textures.
         */
        getAnimatables(): IAnimatable[];
        /**
         * Gets the active textures from the material
         * @returns an array of textures
         */
        getActiveTextures(): BaseTexture[];
        /**
         * Specifies if the material uses a texture
         * @param texture defines the texture to check against the material
         * @returns a boolean specifying if the material uses the texture
         */
        hasTexture(texture: BaseTexture): boolean;
        /**
         * Makes a duplicate of the material, and gives it a new name
         * @param name defines the new name for the duplicated material
         * @returns the cloned material
         */
        clone(name: string): Nullable<Material>;
        /**
         * Gets the meshes bound to the material
         * @returns an array of meshes bound to the material
         */
        getBindedMeshes(): AbstractMesh[];
        /**
         * Force shader compilation
         * @param mesh defines the mesh associated with this material
         * @param onCompiled defines a function to execute once the material is compiled
         * @param options defines the options to configure the compilation
         * @param onError defines a function to execute if the material fails compiling
         */
        forceCompilation(mesh: AbstractMesh, onCompiled?: (material: Material) => void, options?: Partial<IMaterialCompilationOptions>, onError?: (reason: string) => void): void;
        /**
         * Force shader compilation
         * @param mesh defines the mesh that will use this material
         * @param options defines additional options for compiling the shaders
         * @returns a promise that resolves when the compilation completes
         */
        forceCompilationAsync(mesh: AbstractMesh, options?: Partial<IMaterialCompilationOptions>): Promise<void>;
        private static readonly _AllDirtyCallBack;
        private static readonly _ImageProcessingDirtyCallBack;
        private static readonly _TextureDirtyCallBack;
        private static readonly _FresnelDirtyCallBack;
        private static readonly _MiscDirtyCallBack;
        private static readonly _PrePassDirtyCallBack;
        private static readonly _LightsDirtyCallBack;
        private static readonly _AttributeDirtyCallBack;
        private static _FresnelAndMiscDirtyCallBack;
        private static _TextureAndMiscDirtyCallBack;
        private static readonly _DirtyCallbackArray;
        private static readonly _RunDirtyCallBacks;
        /**
         * Marks a define in the material to indicate that it needs to be re-computed
         * @param flag defines a flag used to determine which parts of the material have to be marked as dirty
         */
        markAsDirty(flag: number): void;
        /**
         * Resets the draw wrappers cache for all submeshes that are using this material
         */
        resetDrawCache(): void;
        /**
         * Marks all submeshes of a material to indicate that their material defines need to be re-calculated
         * @param func defines a function which checks material defines against the submeshes
         */
        protected _markAllSubMeshesAsDirty(func: (defines: MaterialDefines) => void): void;
        /**
         * Indicates that the scene should check if the rendering now needs a prepass
         */
        protected _markScenePrePassDirty(): void;
        /**
         * Indicates that we need to re-calculated for all submeshes
         */
        protected _markAllSubMeshesAsAllDirty(): void;
        /**
         * Indicates that image processing needs to be re-calculated for all submeshes
         */
        protected _markAllSubMeshesAsImageProcessingDirty(): void;
        /**
         * Indicates that textures need to be re-calculated for all submeshes
         */
        protected _markAllSubMeshesAsTexturesDirty(): void;
        /**
         * Indicates that fresnel needs to be re-calculated for all submeshes
         */
        protected _markAllSubMeshesAsFresnelDirty(): void;
        /**
         * Indicates that fresnel and misc need to be re-calculated for all submeshes
         */
        protected _markAllSubMeshesAsFresnelAndMiscDirty(): void;
        /**
         * Indicates that lights need to be re-calculated for all submeshes
         */
        protected _markAllSubMeshesAsLightsDirty(): void;
        /**
         * Indicates that attributes need to be re-calculated for all submeshes
         */
        protected _markAllSubMeshesAsAttributesDirty(): void;
        /**
         * Indicates that misc needs to be re-calculated for all submeshes
         */
        protected _markAllSubMeshesAsMiscDirty(): void;
        /**
         * Indicates that prepass needs to be re-calculated for all submeshes
         */
        protected _markAllSubMeshesAsPrePassDirty(): void;
        /**
         * Indicates that textures and misc need to be re-calculated for all submeshes
         */
        protected _markAllSubMeshesAsTexturesAndMiscDirty(): void;
        /**
         * Sets the required values to the prepass renderer.
         * @param prePassRenderer defines the prepass renderer to setup.
         * @returns true if the pre pass is needed.
         */
        setPrePassRenderer(prePassRenderer: PrePassRenderer): boolean;
        /**
         * Disposes the material
         * @param forceDisposeEffect specifies if effects should be forcefully disposed
         * @param forceDisposeTextures specifies if textures should be forcefully disposed
         * @param notBoundToMesh specifies if the material that is being disposed is known to be not bound to any mesh
         */
        dispose(forceDisposeEffect?: boolean, forceDisposeTextures?: boolean, notBoundToMesh?: boolean): void;
        /**
         * @internal
         */
        private releaseVertexArrayObject;
        /**
         * Serializes this material
         * @returns the serialized material object
         */
        serialize(): any;
        /**
         * Creates a material from parsed material data
         * @param parsedMaterial defines parsed material data
         * @param scene defines the hosting scene
         * @param rootUrl defines the root URL to use to load textures
         * @returns a new material
         */
        static Parse(parsedMaterial: any, scene: Scene, rootUrl: string): Nullable<Material>;
    }


    /**
     * @internal
     */
    export class MaterialDetailMapDefines extends MaterialDefines {
        DETAIL: boolean;
        DETAILDIRECTUV: number;
        DETAIL_NORMALBLENDMETHOD: number;
    }
    /**
     * Plugin that implements the detail map component of a material
     *
     * Inspired from:
     *   Unity: https://docs.unity3d.com/Packages/com.unity.render-pipelines.high-definition@9.0/manual/Mask-Map-and-Detail-Map.html and https://docs.unity3d.com/Manual/StandardShaderMaterialParameterDetail.html
     *   Unreal: https://docs.unrealengine.com/en-US/Engine/Rendering/Materials/HowTo/DetailTexturing/index.html
     *   Cryengine: https://docs.cryengine.com/display/SDKDOC2/Detail+Maps
     */
    export class DetailMapConfiguration extends MaterialPluginBase {
        private _texture;
        /**
         * The detail texture of the material.
         */
        texture: Nullable<BaseTexture>;
        /**
         * Defines how strongly the detail diffuse/albedo channel is blended with the regular diffuse/albedo texture
         * Bigger values mean stronger blending
         */
        diffuseBlendLevel: number;
        /**
         * Defines how strongly the detail roughness channel is blended with the regular roughness value
         * Bigger values mean stronger blending. Only used with PBR materials
         */
        roughnessBlendLevel: number;
        /**
         * Defines how strong the bump effect from the detail map is
         * Bigger values mean stronger effect
         */
        bumpLevel: number;
        private _normalBlendMethod;
        /**
         * The method used to blend the bump and detail normals together
         */
        normalBlendMethod: number;
        private _isEnabled;
        /**
         * Enable or disable the detail map on this material
         */
        isEnabled: boolean;
        /** @internal */
        private _internalMarkAllSubMeshesAsTexturesDirty;
        /** @internal */
        _markAllSubMeshesAsTexturesDirty(): void;
        constructor(material: PBRBaseMaterial | StandardMaterial, addToPluginList?: boolean);
        isReadyForSubMesh(defines: MaterialDetailMapDefines, scene: Scene, engine: Engine): boolean;
        prepareDefines(defines: MaterialDetailMapDefines, scene: Scene): void;
        bindForSubMesh(uniformBuffer: UniformBuffer, scene: Scene): void;
        hasTexture(texture: BaseTexture): boolean;
        getActiveTextures(activeTextures: BaseTexture[]): void;
        getAnimatables(animatables: IAnimatable[]): void;
        dispose(forceDisposeTextures?: boolean): void;
        getClassName(): string;
        getSamplers(samplers: string[]): void;
        getUniforms(): {
            ubo?: Array<{
                name: string;
                size: number;
                type: string;
            }>;
            vertex?: string;
            fragment?: string;
        };
    }


    /**
     * Manages the defines for the Material
     */
    export class MaterialDefines {
        /** @internal */
        protected _keys: string[];
        private _isDirty;
        /** @internal */
        _renderId: number;
        /** @internal */
        _areLightsDirty: boolean;
        /** @internal */
        _areLightsDisposed: boolean;
        /** @internal */
        _areAttributesDirty: boolean;
        /** @internal */
        _areTexturesDirty: boolean;
        /** @internal */
        _areFresnelDirty: boolean;
        /** @internal */
        _areMiscDirty: boolean;
        /** @internal */
        _arePrePassDirty: boolean;
        /** @internal */
        _areImageProcessingDirty: boolean;
        /** @internal */
        _normals: boolean;
        /** @internal */
        _uvs: boolean;
        /** @internal */
        _needNormals: boolean;
        /** @internal */
        _needUVs: boolean;
        protected _externalProperties?: {
            [name: string]: {
                type: string;
                default: any;
            };
        };
        [id: string]: any;
        /**
         * Creates a new instance
         * @param externalProperties list of external properties to inject into the object
         */
        constructor(externalProperties?: {
            [name: string]: {
                type: string;
                default: any;
            };
        });
        /**
         * Specifies if the material needs to be re-calculated
         */
        get isDirty(): boolean;
        /**
         * Marks the material to indicate that it has been re-calculated
         */
        markAsProcessed(): void;
        /**
         * Marks the material to indicate that it needs to be re-calculated
         */
        markAsUnprocessed(): void;
        /**
         * Marks the material to indicate all of its defines need to be re-calculated
         */
        markAllAsDirty(): void;
        /**
         * Marks the material to indicate that image processing needs to be re-calculated
         */
        markAsImageProcessingDirty(): void;
        /**
         * Marks the material to indicate the lights need to be re-calculated
         * @param disposed Defines whether the light is dirty due to dispose or not
         */
        markAsLightDirty(disposed?: boolean): void;
        /**
         * Marks the attribute state as changed
         */
        markAsAttributesDirty(): void;
        /**
         * Marks the texture state as changed
         */
        markAsTexturesDirty(): void;
        /**
         * Marks the fresnel state as changed
         */
        markAsFresnelDirty(): void;
        /**
         * Marks the misc state as changed
         */
        markAsMiscDirty(): void;
        /**
         * Marks the prepass state as changed
         */
        markAsPrePassDirty(): void;
        /**
         * Rebuilds the material defines
         */
        rebuild(): void;
        /**
         * Specifies if two material defines are equal
         * @param other - A material define instance to compare to
         * @returns - Boolean indicating if the material defines are equal (true) or not (false)
         */
        isEqual(other: MaterialDefines): boolean;
        /**
         * Clones this instance's defines to another instance
         * @param other - material defines to clone values to
         */
        cloneTo(other: MaterialDefines): void;
        /**
         * Resets the material define values
         */
        reset(): void;
        private _setDefaultValue;
        /**
         * Converts the material define values to a string
         * @returns - String of material define information
         */
        toString(): string;
    }


    /**
     * This groups all the flags used to control the materials channel.
     */
    export class MaterialFlags {
        private static _DiffuseTextureEnabled;
        /**
         * Are diffuse textures enabled in the application.
         */
        static get DiffuseTextureEnabled(): boolean;
        static set DiffuseTextureEnabled(value: boolean);
        private static _DetailTextureEnabled;
        /**
         * Are detail textures enabled in the application.
         */
        static get DetailTextureEnabled(): boolean;
        static set DetailTextureEnabled(value: boolean);
        private static _AmbientTextureEnabled;
        /**
         * Are ambient textures enabled in the application.
         */
        static get AmbientTextureEnabled(): boolean;
        static set AmbientTextureEnabled(value: boolean);
        private static _OpacityTextureEnabled;
        /**
         * Are opacity textures enabled in the application.
         */
        static get OpacityTextureEnabled(): boolean;
        static set OpacityTextureEnabled(value: boolean);
        private static _ReflectionTextureEnabled;
        /**
         * Are reflection textures enabled in the application.
         */
        static get ReflectionTextureEnabled(): boolean;
        static set ReflectionTextureEnabled(value: boolean);
        private static _EmissiveTextureEnabled;
        /**
         * Are emissive textures enabled in the application.
         */
        static get EmissiveTextureEnabled(): boolean;
        static set EmissiveTextureEnabled(value: boolean);
        private static _SpecularTextureEnabled;
        /**
         * Are specular textures enabled in the application.
         */
        static get SpecularTextureEnabled(): boolean;
        static set SpecularTextureEnabled(value: boolean);
        private static _BumpTextureEnabled;
        /**
         * Are bump textures enabled in the application.
         */
        static get BumpTextureEnabled(): boolean;
        static set BumpTextureEnabled(value: boolean);
        private static _LightmapTextureEnabled;
        /**
         * Are lightmap textures enabled in the application.
         */
        static get LightmapTextureEnabled(): boolean;
        static set LightmapTextureEnabled(value: boolean);
        private static _RefractionTextureEnabled;
        /**
         * Are refraction textures enabled in the application.
         */
        static get RefractionTextureEnabled(): boolean;
        static set RefractionTextureEnabled(value: boolean);
        private static _ColorGradingTextureEnabled;
        /**
         * Are color grading textures enabled in the application.
         */
        static get ColorGradingTextureEnabled(): boolean;
        static set ColorGradingTextureEnabled(value: boolean);
        private static _FresnelEnabled;
        /**
         * Are fresnels enabled in the application.
         */
        static get FresnelEnabled(): boolean;
        static set FresnelEnabled(value: boolean);
        private static _ClearCoatTextureEnabled;
        /**
         * Are clear coat textures enabled in the application.
         */
        static get ClearCoatTextureEnabled(): boolean;
        static set ClearCoatTextureEnabled(value: boolean);
        private static _ClearCoatBumpTextureEnabled;
        /**
         * Are clear coat bump textures enabled in the application.
         */
        static get ClearCoatBumpTextureEnabled(): boolean;
        static set ClearCoatBumpTextureEnabled(value: boolean);
        private static _ClearCoatTintTextureEnabled;
        /**
         * Are clear coat tint textures enabled in the application.
         */
        static get ClearCoatTintTextureEnabled(): boolean;
        static set ClearCoatTintTextureEnabled(value: boolean);
        private static _SheenTextureEnabled;
        /**
         * Are sheen textures enabled in the application.
         */
        static get SheenTextureEnabled(): boolean;
        static set SheenTextureEnabled(value: boolean);
        private static _AnisotropicTextureEnabled;
        /**
         * Are anisotropic textures enabled in the application.
         */
        static get AnisotropicTextureEnabled(): boolean;
        static set AnisotropicTextureEnabled(value: boolean);
        private static _ThicknessTextureEnabled;
        /**
         * Are thickness textures enabled in the application.
         */
        static get ThicknessTextureEnabled(): boolean;
        static set ThicknessTextureEnabled(value: boolean);
        private static _RefractionIntensityTextureEnabled;
        /**
         * Are refraction intensity textures enabled in the application.
         */
        static get RefractionIntensityTextureEnabled(): boolean;
        static set RefractionIntensityTextureEnabled(value: boolean);
        private static _TranslucencyIntensityTextureEnabled;
        /**
         * Are translucency intensity textures enabled in the application.
         */
        static get TranslucencyIntensityTextureEnabled(): boolean;
        static set TranslucencyIntensityTextureEnabled(value: boolean);
        private static _IridescenceTextureEnabled;
        /**
         * Are translucency intensity textures enabled in the application.
         */
        static get IridescenceTextureEnabled(): boolean;
        static set IridescenceTextureEnabled(value: boolean);
    }


    /**
     * "Static Class" containing the most commonly used helper while dealing with material for rendering purpose.
     *
     * It contains the basic tools to help defining defines, binding uniform for the common part of the materials.
     *
     * This works by convention in BabylonJS but is meant to be use only with shader following the in place naming rules and conventions.
     */
    export class MaterialHelper {
        /**
         * Binds the scene's uniform buffer to the effect.
         * @param effect defines the effect to bind to the scene uniform buffer
         * @param sceneUbo defines the uniform buffer storing scene data
         */
        static BindSceneUniformBuffer(effect: Effect, sceneUbo: UniformBuffer): void;
        /**
         * Helps preparing the defines values about the UVs in used in the effect.
         * UVs are shared as much as we can across channels in the shaders.
         * @param texture The texture we are preparing the UVs for
         * @param defines The defines to update
         * @param key The channel key "diffuse", "specular"... used in the shader
         */
        static PrepareDefinesForMergedUV(texture: BaseTexture, defines: any, key: string): void;
        /**
         * Binds a texture matrix value to its corresponding uniform
         * @param texture The texture to bind the matrix for
         * @param uniformBuffer The uniform buffer receiving the data
         * @param key The channel key "diffuse", "specular"... used in the shader
         */
        static BindTextureMatrix(texture: BaseTexture, uniformBuffer: UniformBuffer, key: string): void;
        /**
         * Gets the current status of the fog (should it be enabled?)
         * @param mesh defines the mesh to evaluate for fog support
         * @param scene defines the hosting scene
         * @returns true if fog must be enabled
         */
        static GetFogState(mesh: AbstractMesh, scene: Scene): boolean;
        /**
         * Helper used to prepare the list of defines associated with misc. values for shader compilation
         * @param mesh defines the current mesh
         * @param scene defines the current scene
         * @param useLogarithmicDepth defines if logarithmic depth has to be turned on
         * @param pointsCloud defines if point cloud rendering has to be turned on
         * @param fogEnabled defines if fog has to be turned on
         * @param alphaTest defines if alpha testing has to be turned on
         * @param defines defines the current list of defines
         */
        static PrepareDefinesForMisc(mesh: AbstractMesh, scene: Scene, useLogarithmicDepth: boolean, pointsCloud: boolean, fogEnabled: boolean, alphaTest: boolean, defines: any): void;
        /**
         * Helper used to prepare the defines relative to the active camera
         * @param scene defines the current scene
         * @param defines specifies the list of active defines
         * @returns true if the defines have been updated, else false
         */
        static PrepareDefinesForCamera(scene: Scene, defines: any): boolean;
        /**
         * Helper used to prepare the list of defines associated with frame values for shader compilation
         * @param scene defines the current scene
         * @param engine defines the current engine
         * @param material defines the material we are compiling the shader for
         * @param defines specifies the list of active defines
         * @param useInstances defines if instances have to be turned on
         * @param useClipPlane defines if clip plane have to be turned on
         * @param useThinInstances defines if thin instances have to be turned on
         */
        static PrepareDefinesForFrameBoundValues(scene: Scene, engine: Engine, material: Material, defines: any, useInstances: boolean, useClipPlane?: Nullable<boolean>, useThinInstances?: boolean): void;
        /**
         * Prepares the defines for bones
         * @param mesh The mesh containing the geometry data we will draw
         * @param defines The defines to update
         */
        static PrepareDefinesForBones(mesh: AbstractMesh, defines: any): void;
        /**
         * Prepares the defines for morph targets
         * @param mesh The mesh containing the geometry data we will draw
         * @param defines The defines to update
         */
        static PrepareDefinesForMorphTargets(mesh: AbstractMesh, defines: any): void;
        /**
         * Prepares the defines for baked vertex animation
         * @param mesh The mesh containing the geometry data we will draw
         * @param defines The defines to update
         */
        static PrepareDefinesForBakedVertexAnimation(mesh: AbstractMesh, defines: any): void;
        /**
         * Prepares the defines used in the shader depending on the attributes data available in the mesh
         * @param mesh The mesh containing the geometry data we will draw
         * @param defines The defines to update
         * @param useVertexColor Precise whether vertex colors should be used or not (override mesh info)
         * @param useBones Precise whether bones should be used or not (override mesh info)
         * @param useMorphTargets Precise whether morph targets should be used or not (override mesh info)
         * @param useVertexAlpha Precise whether vertex alpha should be used or not (override mesh info)
         * @param useBakedVertexAnimation Precise whether baked vertex animation should be used or not (override mesh info)
         * @returns false if defines are considered not dirty and have not been checked
         */
        static PrepareDefinesForAttributes(mesh: AbstractMesh, defines: any, useVertexColor: boolean, useBones: boolean, useMorphTargets?: boolean, useVertexAlpha?: boolean, useBakedVertexAnimation?: boolean): boolean;
        /**
         * Prepares the defines related to multiview
         * @param scene The scene we are intending to draw
         * @param defines The defines to update
         */
        static PrepareDefinesForMultiview(scene: Scene, defines: any): void;
        /**
         * Prepares the defines related to order independant transparency
         * @param scene The scene we are intending to draw
         * @param defines The defines to update
         * @param needAlphaBlending Determines if the material needs alpha blending
         */
        static PrepareDefinesForOIT(scene: Scene, defines: any, needAlphaBlending: boolean): void;
        /**
         * Prepares the defines related to the prepass
         * @param scene The scene we are intending to draw
         * @param defines The defines to update
         * @param canRenderToMRT Indicates if this material renders to several textures in the prepass
         */
        static PrepareDefinesForPrePass(scene: Scene, defines: any, canRenderToMRT: boolean): void;
        /**
         * Prepares the defines related to the light information passed in parameter
         * @param scene The scene we are intending to draw
         * @param mesh The mesh the effect is compiling for
         * @param light The light the effect is compiling for
         * @param lightIndex The index of the light
         * @param defines The defines to update
         * @param specularSupported Specifies whether specular is supported or not (override lights data)
         * @param state Defines the current state regarding what is needed (normals, etc...)
         * @param state.needNormals
         * @param state.needRebuild
         * @param state.shadowEnabled
         * @param state.specularEnabled
         * @param state.lightmapMode
         */
        static PrepareDefinesForLight(scene: Scene, mesh: AbstractMesh, light: Light, lightIndex: number, defines: any, specularSupported: boolean, state: {
            needNormals: boolean;
            needRebuild: boolean;
            shadowEnabled: boolean;
            specularEnabled: boolean;
            lightmapMode: boolean;
        }): void;
        /**
         * Prepares the defines related to the light information passed in parameter
         * @param scene The scene we are intending to draw
         * @param mesh The mesh the effect is compiling for
         * @param defines The defines to update
         * @param specularSupported Specifies whether specular is supported or not (override lights data)
         * @param maxSimultaneousLights Specifies how manuy lights can be added to the effect at max
         * @param disableLighting Specifies whether the lighting is disabled (override scene and light)
         * @returns true if normals will be required for the rest of the effect
         */
        static PrepareDefinesForLights(scene: Scene, mesh: AbstractMesh, defines: any, specularSupported: boolean, maxSimultaneousLights?: number, disableLighting?: boolean): boolean;
        /**
         * Prepares the uniforms and samplers list to be used in the effect (for a specific light)
         * @param lightIndex defines the light index
         * @param uniformsList The uniform list
         * @param samplersList The sampler list
         * @param projectedLightTexture defines if projected texture must be used
         * @param uniformBuffersList defines an optional list of uniform buffers
         * @param updateOnlyBuffersList True to only update the uniformBuffersList array
         */
        static PrepareUniformsAndSamplersForLight(lightIndex: number, uniformsList: string[], samplersList: string[], projectedLightTexture?: any, uniformBuffersList?: Nullable<string[]>, updateOnlyBuffersList?: boolean): void;
        /**
         * Prepares the uniforms and samplers list to be used in the effect
         * @param uniformsListOrOptions The uniform names to prepare or an EffectCreationOptions containing the list and extra information
         * @param samplersList The sampler list
         * @param defines The defines helping in the list generation
         * @param maxSimultaneousLights The maximum number of simultaneous light allowed in the effect
         */
        static PrepareUniformsAndSamplersList(uniformsListOrOptions: string[] | IEffectCreationOptions, samplersList?: string[], defines?: any, maxSimultaneousLights?: number): void;
        /**
         * This helps decreasing rank by rank the shadow quality (0 being the highest rank and quality)
         * @param defines The defines to update while falling back
         * @param fallbacks The authorized effect fallbacks
         * @param maxSimultaneousLights The maximum number of lights allowed
         * @param rank the current rank of the Effect
         * @returns The newly affected rank
         */
        static HandleFallbacksForShadows(defines: any, fallbacks: EffectFallbacks, maxSimultaneousLights?: number, rank?: number): number;
        private static _TmpMorphInfluencers;
        /**
         * Prepares the list of attributes required for morph targets according to the effect defines.
         * @param attribs The current list of supported attribs
         * @param mesh The mesh to prepare the morph targets attributes for
         * @param influencers The number of influencers
         */
        static PrepareAttributesForMorphTargetsInfluencers(attribs: string[], mesh: AbstractMesh, influencers: number): void;
        /**
         * Prepares the list of attributes required for morph targets according to the effect defines.
         * @param attribs The current list of supported attribs
         * @param mesh The mesh to prepare the morph targets attributes for
         * @param defines The current Defines of the effect
         */
        static PrepareAttributesForMorphTargets(attribs: string[], mesh: AbstractMesh, defines: any): void;
        /**
         * Prepares the list of attributes required for baked vertex animations according to the effect defines.
         * @param attribs The current list of supported attribs
         * @param mesh The mesh to prepare the morph targets attributes for
         * @param defines The current Defines of the effect
         */
        static PrepareAttributesForBakedVertexAnimation(attribs: string[], mesh: AbstractMesh, defines: any): void;
        /**
         * Prepares the list of attributes required for bones according to the effect defines.
         * @param attribs The current list of supported attribs
         * @param mesh The mesh to prepare the bones attributes for
         * @param defines The current Defines of the effect
         * @param fallbacks The current effect fallback strategy
         */
        static PrepareAttributesForBones(attribs: string[], mesh: AbstractMesh, defines: any, fallbacks: EffectFallbacks): void;
        /**
         * Check and prepare the list of attributes required for instances according to the effect defines.
         * @param attribs The current list of supported attribs
         * @param defines The current MaterialDefines of the effect
         */
        static PrepareAttributesForInstances(attribs: string[], defines: MaterialDefines): void;
        /**
         * Add the list of attributes required for instances to the attribs array.
         * @param attribs The current list of supported attribs
         * @param needsPreviousMatrices If the shader needs previous matrices
         */
        static PushAttributesForInstances(attribs: string[], needsPreviousMatrices?: boolean): void;
        /**
         * Binds the light information to the effect.
         * @param light The light containing the generator
         * @param effect The effect we are binding the data to
         * @param lightIndex The light index in the effect used to render
         */
        static BindLightProperties(light: Light, effect: Effect, lightIndex: number): void;
        /**
         * Binds the lights information from the scene to the effect for the given mesh.
         * @param light Light to bind
         * @param lightIndex Light index
         * @param scene The scene where the light belongs to
         * @param effect The effect we are binding the data to
         * @param useSpecular Defines if specular is supported
         * @param receiveShadows Defines if the effect (mesh) we bind the light for receives shadows
         */
        static BindLight(light: Light, lightIndex: number, scene: Scene, effect: Effect, useSpecular: boolean, receiveShadows?: boolean): void;
        /**
         * Binds the lights information from the scene to the effect for the given mesh.
         * @param scene The scene the lights belongs to
         * @param mesh The mesh we are binding the information to render
         * @param effect The effect we are binding the data to
         * @param defines The generated defines for the effect
         * @param maxSimultaneousLights The maximum number of light that can be bound to the effect
         */
        static BindLights(scene: Scene, mesh: AbstractMesh, effect: Effect, defines: any, maxSimultaneousLights?: number): void;
        private static _TempFogColor;
        /**
         * Binds the fog information from the scene to the effect for the given mesh.
         * @param scene The scene the lights belongs to
         * @param mesh The mesh we are binding the information to render
         * @param effect The effect we are binding the data to
         * @param linearSpace Defines if the fog effect is applied in linear space
         */
        static BindFogParameters(scene: Scene, mesh: AbstractMesh, effect: Effect, linearSpace?: boolean): void;
        /**
         * Binds the bones information from the mesh to the effect.
         * @param mesh The mesh we are binding the information to render
         * @param effect The effect we are binding the data to
         * @param prePassConfiguration Configuration for the prepass, in case prepass is activated
         */
        static BindBonesParameters(mesh?: AbstractMesh, effect?: Effect, prePassConfiguration?: PrePassConfiguration): void;
        private static _CopyBonesTransformationMatrices;
        /**
         * Binds the morph targets information from the mesh to the effect.
         * @param abstractMesh The mesh we are binding the information to render
         * @param effect The effect we are binding the data to
         */
        static BindMorphTargetParameters(abstractMesh: AbstractMesh, effect: Effect): void;
        /**
         * Binds the logarithmic depth information from the scene to the effect for the given defines.
         * @param defines The generated defines used in the effect
         * @param effect The effect we are binding the data to
         * @param scene The scene we are willing to render with logarithmic scale for
         */
        static BindLogDepth(defines: any, effect: Effect, scene: Scene): void;
    }


    /**
     * Base class for material plugins.
     * @since 5.0
     */
    export class MaterialPluginBase {
        /**
         * Defines the name of the plugin
         */
        name: string;
        /**
         * Defines the priority of the plugin. Lower numbers run first.
         */
        priority: number;
        /**
         * Indicates that this plugin should be notified for the extra events (HasRenderTargetTextures / FillRenderTargetTextures / HardBindForSubMesh)
         */
        registerForExtraEvents: boolean;
        protected _material: Material;
        protected _pluginManager: MaterialPluginManager;
        protected _pluginDefineNames?: {
            [name: string]: any;
        };
        protected _enable(enable: boolean): void;
        /**
         * Helper function to mark defines as being dirty.
         */
        readonly markAllDefinesAsDirty: () => void;
        /**
         * Creates a new material plugin
         * @param material parent material of the plugin
         * @param name name of the plugin
         * @param priority priority of the plugin
         * @param defines list of defines used by the plugin. The value of the property is the default value for this property
         * @param addToPluginList true to add the plugin to the list of plugins managed by the material plugin manager of the material (default: true)
         * @param enable true to enable the plugin (it is handy if the plugin does not handle properties to switch its current activation)
         */
        constructor(material: Material, name: string, priority: number, defines?: {
            [key: string]: any;
        }, addToPluginList?: boolean, enable?: boolean);
        /**
         * Gets the current class name useful for serialization or dynamic coding.
         * @returns The class name.
         */
        getClassName(): string;
        /**
         * Specifies that the submesh is ready to be used.
         * @param defines the list of "defines" to update.
         * @param scene defines the scene the material belongs to.
         * @param engine the engine this scene belongs to.
         * @param subMesh the submesh to check for readiness
         * @returns - boolean indicating that the submesh is ready or not.
         */
        isReadyForSubMesh(defines: MaterialDefines, scene: Scene, engine: Engine, subMesh: SubMesh): boolean;
        /**
         * Binds the material data (this function is called even if mustRebind() returns false)
         * @param uniformBuffer defines the Uniform buffer to fill in.
         * @param scene defines the scene the material belongs to.
         * @param engine defines the engine the material belongs to.
         * @param subMesh the submesh to bind data for
         */
        hardBindForSubMesh(uniformBuffer: UniformBuffer, scene: Scene, engine: Engine, subMesh: SubMesh): void;
        /**
         * Binds the material data.
         * @param uniformBuffer defines the Uniform buffer to fill in.
         * @param scene defines the scene the material belongs to.
         * @param engine the engine this scene belongs to.
         * @param subMesh the submesh to bind data for
         */
        bindForSubMesh(uniformBuffer: UniformBuffer, scene: Scene, engine: Engine, subMesh: SubMesh): void;
        /**
         * Disposes the resources of the material.
         * @param forceDisposeTextures - Forces the disposal of all textures.
         */
        dispose(forceDisposeTextures?: boolean): void;
        /**
         * Returns a list of custom shader code fragments to customize the shader.
         * @param shaderType "vertex" or "fragment"
         * @returns null if no code to be added, or a list of pointName => code.
         * Note that `pointName` can also be a regular expression if it starts with a `!`.
         * In that case, the string found by the regular expression (if any) will be
         * replaced by the code provided.
         */
        getCustomCode(shaderType: string): Nullable<{
            [pointName: string]: string;
        }>;
        /**
         * Collects all defines.
         * @param defines The object to append to.
         */
        collectDefines(defines: {
            [name: string]: {
                type: string;
                default: any;
            };
        }): void;
        /**
         * Sets the defines for the next rendering. Called before MaterialHelper.PrepareDefinesForAttributes is called.
         * @param defines the list of "defines" to update.
         * @param scene defines the scene to the material belongs to.
         * @param mesh the mesh being rendered
         */
        prepareDefinesBeforeAttributes(defines: MaterialDefines, scene: Scene, mesh: AbstractMesh): void;
        /**
         * Sets the defines for the next rendering
         * @param defines the list of "defines" to update.
         * @param scene defines the scene to the material belongs to.
         * @param mesh the mesh being rendered
         */
        prepareDefines(defines: MaterialDefines, scene: Scene, mesh: AbstractMesh): void;
        /**
         * Checks to see if a texture is used in the material.
         * @param texture - Base texture to use.
         * @returns - Boolean specifying if a texture is used in the material.
         */
        hasTexture(texture: BaseTexture): boolean;
        /**
         * Gets a boolean indicating that current material needs to register RTT
         * @returns true if this uses a render target otherwise false.
         */
        hasRenderTargetTextures(): boolean;
        /**
         * Fills the list of render target textures.
         * @param renderTargets the list of render targets to update
         */
        fillRenderTargetTextures(renderTargets: SmartArray<RenderTargetTexture>): void;
        /**
         * Returns an array of the actively used textures.
         * @param activeTextures Array of BaseTextures
         */
        getActiveTextures(activeTextures: BaseTexture[]): void;
        /**
         * Returns the animatable textures.
         * @param animatables Array of animatable textures.
         */
        getAnimatables(animatables: IAnimatable[]): void;
        /**
         * Add fallbacks to the effect fallbacks list.
         * @param defines defines the Base texture to use.
         * @param fallbacks defines the current fallback list.
         * @param currentRank defines the current fallback rank.
         * @returns the new fallback rank.
         */
        addFallbacks(defines: MaterialDefines, fallbacks: EffectFallbacks, currentRank: number): number;
        /**
         * Gets the samplers used by the plugin.
         * @param samplers list that the sampler names should be added to.
         */
        getSamplers(samplers: string[]): void;
        /**
         * Gets the attributes used by the plugin.
         * @param attributes list that the attribute names should be added to.
         * @param scene the scene that the material belongs to.
         * @param mesh the mesh being rendered.
         */
        getAttributes(attributes: string[], scene: Scene, mesh: AbstractMesh): void;
        /**
         * Gets the uniform buffers names added by the plugin.
         * @param ubos list that the ubo names should be added to.
         */
        getUniformBuffersNames(ubos: string[]): void;
        /**
         * Gets the description of the uniforms to add to the ubo (if engine supports ubos) or to inject directly in the vertex/fragment shaders (if engine does not support ubos)
         * @returns the description of the uniforms
         */
        getUniforms(): {
            ubo?: Array<{
                name: string;
                size: number;
                type: string;
            }>;
            vertex?: string;
            fragment?: string;
        };
        /**
         * Makes a duplicate of the current configuration into another one.
         * @param plugin define the config where to copy the info
         */
        copyTo(plugin: MaterialPluginBase): void;
        /**
         * Serializes this clear coat configuration.
         * @returns - An object with the serialized config.
         */
        serialize(): any;
        /**
         * Parses a anisotropy Configuration from a serialized object.
         * @param source - Serialized object.
         * @param scene Defines the scene we are parsing for
         * @param rootUrl Defines the rootUrl to load from
         */
        parse(source: any, scene: Scene, rootUrl: string): void;
    }


    /** @internal */
    export type MaterialPluginCreated = {};
    /** @internal */
    export type MaterialPluginDisposed = {
        forceDisposeTextures?: boolean;
    };
    /** @internal */
    export type MaterialPluginHasTexture = {
        hasTexture: boolean;
        texture: BaseTexture;
    };
    /** @internal */
    export type MaterialPluginIsReadyForSubMesh = {
        isReadyForSubMesh: boolean;
        defines: MaterialDefines;
        subMesh: SubMesh;
    };
    /** @internal */
    export type MaterialPluginGetDefineNames = {
        defineNames?: {
            [name: string]: {
                type: string;
                default: any;
            };
        };
    };
    /** @internal */
    export type MaterialPluginPrepareEffect = {
        defines: MaterialDefines;
        fallbacks: EffectFallbacks;
        fallbackRank: number;
        customCode?: ShaderCustomProcessingFunction;
        attributes: string[];
        uniforms: string[];
        samplers: string[];
        uniformBuffersNames: string[];
        mesh: AbstractMesh;
    };
    /** @internal */
    export type MaterialPluginPrepareDefines = {
        defines: MaterialDefines;
        mesh: AbstractMesh;
    };
    /** @internal */
    export type MaterialPluginPrepareUniformBuffer = {
        ubo: UniformBuffer;
    };
    /** @internal */
    export type MaterialPluginBindForSubMesh = {
        subMesh: SubMesh;
    };
    /** @internal */
    export type MaterialPluginGetAnimatables = {
        animatables: IAnimatable[];
    };
    /** @internal */
    export type MaterialPluginGetActiveTextures = {
        activeTextures: BaseTexture[];
    };
    /** @internal */
    export type MaterialPluginFillRenderTargetTextures = {
        renderTargets: SmartArray<RenderTargetTexture>;
    };
    /** @internal */
    export type MaterialPluginHasRenderTargetTextures = {
        hasRenderTargetTextures: boolean;
    };
    /** @internal */
    export type MaterialPluginHardBindForSubMesh = {
        subMesh: SubMesh;
    };
    /**
     * @internal
     */
    export enum MaterialPluginEvent {
        Created = 1,
        Disposed = 2,
        GetDefineNames = 4,
        PrepareUniformBuffer = 8,
        IsReadyForSubMesh = 16,
        PrepareDefines = 32,
        BindForSubMesh = 64,
        PrepareEffect = 128,
        GetAnimatables = 256,
        GetActiveTextures = 512,
        HasTexture = 1024,
        FillRenderTargetTextures = 2048,
        HasRenderTargetTextures = 4096,
        HardBindForSubMesh = 8192
    }


    /**
     * Creates an instance of the anisotropic plugin
     * @param material parent material the plugin will be created for
     * @returns the plugin instance or null if the plugin is incompatible with material
     */
    export function createPBRAnisotropicPlugin(material: Material): Nullable<MaterialPluginBase>;
    /**
     * Creates an instance of the brdf plugin
     * @param material parent material the plugin will be created for
     * @returns the plugin instance or null if the plugin is incompatible with material
     */
    export function createPBRBRDFPlugin(material: Material): Nullable<MaterialPluginBase>;
    /**
     * Creates an instance of the clear coat plugin
     * @param material parent material the plugin will be created for
     * @returns the plugin instance or null if the plugin is incompatible with material
     */
    export function createPBRClearCoatPlugin(material: Material): Nullable<MaterialPluginBase>;
    /**
     * Creates an instance of the iridescence plugin
     * @param material parent material the plugin will be created for
     * @returns the plugin instance or null if the plugin is incompatible with material
     */
    export function createPBRIridescencePlugin(material: Material): Nullable<MaterialPluginBase>;
    /**
     * Creates an instance of the sheen plugin
     * @param material parent material the plugin will be created for
     * @returns the plugin instance or null if the plugin is incompatible with material
     */
    export function createPBRSheenPlugin(material: Material): Nullable<MaterialPluginBase>;
    /**
     * Creates an instance of the sub surface plugin
     * @param material parent material the plugin will be created for
     * @returns the plugin instance or null if the plugin is incompatible with material
     */
    export function createPBRSubSurfacePlugin(material: Material): Nullable<MaterialPluginBase>;
    /**
     * Creates an instance of the detail map plugin
     * @param material parent material the plugin will be created for
     * @returns the plugin instance or null if the plugin is incompatible with material
     */
    export function createDetailMapPlugin(material: Material): Nullable<MaterialPluginBase>;


        interface Material {
            /**
             * Plugin manager for this material
             */
            pluginManager?: MaterialPluginManager;
        }
    /**
     * Class that manages the plugins of a material
     * @since 5.0
     */
    export class MaterialPluginManager {
        /** Map a plugin class name to a #define name (used in the vertex/fragment shaders as a marker of the plugin usage) */
        private static _MaterialPluginClassToMainDefine;
        private static _MaterialPluginCounter;
        protected _material: Material;
        protected _scene: Scene;
        protected _engine: Engine;
        protected _plugins: MaterialPluginBase[];
        protected _activePlugins: MaterialPluginBase[];
        protected _activePluginsForExtraEvents: MaterialPluginBase[];
        protected _codeInjectionPoints: {
            [shaderType: string]: {
                [codeName: string]: boolean;
            };
        };
        protected _defineNamesFromPlugins?: {
            [name: string]: {
                type: string;
                default: any;
            };
        };
        protected _uboDeclaration: string;
        protected _vertexDeclaration: string;
        protected _fragmentDeclaration: string;
        protected _uniformList: string[];
        protected _samplerList: string[];
        protected _uboList: string[];
        /**
         * Creates a new instance of the plugin manager
         * @param material material that this manager will manage the plugins for
         */
        constructor(material: Material);
        /**
         * @internal
         */
        _addPlugin(plugin: MaterialPluginBase): void;
        /**
         * @internal
         */
        _activatePlugin(plugin: MaterialPluginBase): void;
        /**
         * Gets a plugin from the list of plugins managed by this manager
         * @param name name of the plugin
         * @returns the plugin if found, else null
         */
        getPlugin(name: string): Nullable<MaterialPluginBase>;
        protected _handlePluginEventIsReadyForSubMesh(eventData: MaterialPluginIsReadyForSubMesh): void;
        protected _handlePluginEventPrepareDefinesBeforeAttributes(eventData: MaterialPluginPrepareDefines): void;
        protected _handlePluginEventPrepareDefines(eventData: MaterialPluginPrepareDefines): void;
        protected _handlePluginEventHardBindForSubMesh(eventData: MaterialPluginHardBindForSubMesh): void;
        protected _handlePluginEventBindForSubMesh(eventData: MaterialPluginBindForSubMesh): void;
        protected _handlePluginEventHasRenderTargetTextures(eventData: MaterialPluginHasRenderTargetTextures): void;
        protected _handlePluginEventFillRenderTargetTextures(eventData: MaterialPluginFillRenderTargetTextures): void;
        protected _handlePluginEvent(id: number, info: MaterialPluginGetActiveTextures | MaterialPluginGetAnimatables | MaterialPluginHasTexture | MaterialPluginDisposed | MaterialPluginGetDefineNames | MaterialPluginPrepareEffect | MaterialPluginPrepareUniformBuffer): void;
        protected _collectPointNames(shaderType: string, customCode: Nullable<{
            [pointName: string]: string;
        }> | undefined): void;
        protected _injectCustomCode(existingCallback?: (shaderType: string, code: string) => string): ShaderCustomProcessingFunction;
    }
    /**
     * Type for plugin material factories.
     */
    export type PluginMaterialFactory = (material: Material) => Nullable<MaterialPluginBase>;
    /**
     * Registers a new material plugin through a factory, or updates it. This makes the plugin available to all materials instantiated after its registration.
     * @param pluginName The plugin name
     * @param factory The factory function which allows to create the plugin
     */
    export function RegisterMaterialPlugin(pluginName: string, factory: PluginMaterialFactory): void;
    /**
     * Removes a material plugin from the list of global plugins.
     * @param pluginName The plugin name
     * @returns true if the plugin has been removed, else false
     */
    export function UnregisterMaterialPlugin(pluginName: string): boolean;
    /**
     * Clear the list of global material plugins
     */
    export function UnregisterAllMaterialPlugins(): void;


    /**
     * Class that holds the different stencil states of a material
     * Usage example: https://playground.babylonjs.com/#CW5PRI#10
     */
    export class MaterialStencilState implements IStencilState {
        /**
         * Creates a material stencil state instance
         */
        constructor();
        /**
         * Resets all the stencil states to default values
         */
        reset(): void;
        private _func;
        /**
         * Gets or sets the stencil function
         */
        get func(): number;
        set func(value: number);
        private _funcRef;
        /**
         * Gets or sets the stencil function reference
         */
        get funcRef(): number;
        set funcRef(value: number);
        private _funcMask;
        /**
         * Gets or sets the stencil function mask
         */
        get funcMask(): number;
        set funcMask(value: number);
        private _opStencilFail;
        /**
         * Gets or sets the operation when the stencil test fails
         */
        get opStencilFail(): number;
        set opStencilFail(value: number);
        private _opDepthFail;
        /**
         * Gets or sets the operation when the depth test fails
         */
        get opDepthFail(): number;
        set opDepthFail(value: number);
        private _opStencilDepthPass;
        /**
         * Gets or sets the operation when the stencil+depth test succeeds
         */
        get opStencilDepthPass(): number;
        set opStencilDepthPass(value: number);
        private _mask;
        /**
         * Gets or sets the stencil mask
         */
        get mask(): number;
        set mask(value: number);
        private _enabled;
        /**
         * Enables or disables the stencil test
         */
        get enabled(): boolean;
        set enabled(value: boolean);
        /**
         * Get the current class name, useful for serialization or dynamic coding.
         * @returns "MaterialStencilState"
         */
        getClassName(): string;
        /**
         * Makes a duplicate of the current configuration into another one.
         * @param stencilState defines stencil state where to copy the info
         */
        copyTo(stencilState: MaterialStencilState): void;
        /**
         * Serializes this stencil configuration.
         * @returns - An object with the serialized config.
         */
        serialize(): any;
        /**
         * Parses a stencil state configuration from a serialized object.
         * @param source - Serialized object.
         * @param scene Defines the scene we are parsing for
         * @param rootUrl Defines the rootUrl to load from
         */
        parse(source: any, scene: Scene, rootUrl: string): void;
    }


    /**
     * A multi-material is used to apply different materials to different parts of the same object without the need of
     * separate meshes. This can be use to improve performances.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/multiMaterials
     */
    export class MultiMaterial extends Material {
        private _subMaterials;
        /** @internal */
        _waitingSubMaterialsUniqueIds: string[];
        /**
         * Gets or Sets the list of Materials used within the multi material.
         * They need to be ordered according to the submeshes order in the associated mesh
         */
        get subMaterials(): Nullable<Material>[];
        set subMaterials(value: Nullable<Material>[]);
        /**
         * Function used to align with Node.getChildren()
         * @returns the list of Materials used within the multi material
         */
        getChildren(): Nullable<Material>[];
        /**
         * Instantiates a new Multi Material
         * A multi-material is used to apply different materials to different parts of the same object without the need of
         * separate meshes. This can be use to improve performances.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/multiMaterials
         * @param name Define the name in the scene
         * @param scene Define the scene the material belongs to
         */
        constructor(name: string, scene?: Scene);
        private _hookArray;
        /**
         * Get one of the submaterial by its index in the submaterials array
         * @param index The index to look the sub material at
         * @returns The Material if the index has been defined
         */
        getSubMaterial(index: number): Nullable<Material>;
        /**
         * Get the list of active textures for the whole sub materials list.
         * @returns All the textures that will be used during the rendering
         */
        getActiveTextures(): BaseTexture[];
        /**
         * Specifies if any sub-materials of this multi-material use a given texture.
         * @param texture Defines the texture to check against this multi-material's sub-materials.
         * @returns A boolean specifying if any sub-material of this multi-material uses the texture.
         */
        hasTexture(texture: BaseTexture): boolean;
        /**
         * Gets the current class name of the material e.g. "MultiMaterial"
         * Mainly use in serialization.
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Checks if the material is ready to render the requested sub mesh
         * @param mesh Define the mesh the submesh belongs to
         * @param subMesh Define the sub mesh to look readiness for
         * @param useInstances Define whether or not the material is used with instances
         * @returns true if ready, otherwise false
         */
        isReadyForSubMesh(mesh: AbstractMesh, subMesh: SubMesh, useInstances?: boolean): boolean;
        /**
         * Clones the current material and its related sub materials
         * @param name Define the name of the newly cloned material
         * @param cloneChildren Define if submaterial will be cloned or shared with the parent instance
         * @returns the cloned material
         */
        clone(name: string, cloneChildren?: boolean): MultiMaterial;
        /**
         * Serializes the materials into a JSON representation.
         * @returns the JSON representation
         */
        serialize(): any;
        /**
         * Dispose the material and release its associated resources
         * @param forceDisposeEffect Define if we want to force disposing the associated effect (if false the shader is not released and could be reuse later on)
         * @param forceDisposeTextures Define if we want to force disposing the associated textures (if false, they will not be disposed and can still be use elsewhere in the app)
         * @param forceDisposeChildren Define if we want to force disposing the associated submaterials (if false, they will not be disposed and can still be use elsewhere in the app)
         */
        dispose(forceDisposeEffect?: boolean, forceDisposeTextures?: boolean, forceDisposeChildren?: boolean): void;
        /**
         * Creates a MultiMaterial from parsed MultiMaterial data.
         * @param parsedMultiMaterial defines parsed MultiMaterial data.
         * @param scene defines the hosting scene
         * @returns a new MultiMaterial
         */
        static ParseMultiMaterial(parsedMultiMaterial: any, scene: Scene): MultiMaterial;
    }


    /**
     * Block used to add 2 vectors
     */
    export class AddBlock extends NodeMaterialBlock {
        /**
         * Creates a new AddBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the left operand input component
         */
        get left(): NodeMaterialConnectionPoint;
        /**
         * Gets the right operand input component
         */
        get right(): NodeMaterialConnectionPoint;
        /**
         * Gets the output component
         */
        get output(): NodeMaterialConnectionPoint;
        protected _buildBlock(state: NodeMaterialBuildState): this;
    }


    /**
     * Block used to compute arc tangent of 2 values
     */
    export class ArcTan2Block extends NodeMaterialBlock {
        /**
         * Creates a new ArcTan2Block
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the x operand input component
         */
        get x(): NodeMaterialConnectionPoint;
        /**
         * Gets the y operand input component
         */
        get y(): NodeMaterialConnectionPoint;
        /**
         * Gets the output component
         */
        get output(): NodeMaterialConnectionPoint;
        protected _buildBlock(state: NodeMaterialBuildState): this;
    }


    /**
     * Block used to read a texture with triplanar mapping (see https://iquilezles.org/articles/biplanar/)
     */
    export class BiPlanarBlock extends TriPlanarBlock {
        /**
         * Create a new BiPlanarBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        protected _generateTextureLookup(state: NodeMaterialBuildState): void;
    }


    /**
     * Block used to clamp a float
     */
    export class ClampBlock extends NodeMaterialBlock {
        /** Gets or sets the minimum range */
        minimum: number;
        /** Gets or sets the maximum range */
        maximum: number;
        /**
         * Creates a new ClampBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the value input component
         */
        get value(): NodeMaterialConnectionPoint;
        /**
         * Gets the output component
         */
        get output(): NodeMaterialConnectionPoint;
        protected _buildBlock(state: NodeMaterialBuildState): this;
        protected _dumpPropertiesCode(): string;
        serialize(): any;
        _deserialize(serializationObject: any, scene: Scene, rootUrl: string): void;
    }


    /**
     * block used to Generate Fractal Brownian Motion Clouds
     */
    export class CloudBlock extends NodeMaterialBlock {
        /** Gets or sets the number of octaves */
        octaves: number;
        /**
         * Creates a new CloudBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the seed input component
         */
        get seed(): NodeMaterialConnectionPoint;
        /**
         * Gets the chaos input component
         */
        get chaos(): NodeMaterialConnectionPoint;
        /**
         * Gets the offset X input component
         */
        get offsetX(): NodeMaterialConnectionPoint;
        /**
         * Gets the offset Y input component
         */
        get offsetY(): NodeMaterialConnectionPoint;
        /**
         * Gets the offset Z input component
         */
        get offsetZ(): NodeMaterialConnectionPoint;
        /**
         * Gets the output component
         */
        get output(): NodeMaterialConnectionPoint;
        protected _buildBlock(state: NodeMaterialBuildState): this | undefined;
        protected _dumpPropertiesCode(): string;
        serialize(): any;
        _deserialize(serializationObject: any, scene: Scene, rootUrl: string): void;
    }


    /**
     * Block used to create a Color3/4 out of individual inputs (one for each component)
     */
    export class ColorMergerBlock extends NodeMaterialBlock {
        /**
         * Gets or sets the swizzle for r (meaning which component to affect to the output.r)
         */
        rSwizzle: "r" | "g" | "b" | "a";
        /**
         * Gets or sets the swizzle for g (meaning which component to affect to the output.g)
         */
        gSwizzle: "r" | "g" | "b" | "a";
        /**
         * Gets or sets the swizzle for b (meaning which component to affect to the output.b)
         */
        bSwizzle: "r" | "g" | "b" | "a";
        /**
         * Gets or sets the swizzle for a (meaning which component to affect to the output.a)
         */
        aSwizzle: "r" | "g" | "b" | "a";
        /**
         * Create a new ColorMergerBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the rgb component (input)
         */
        get rgbIn(): NodeMaterialConnectionPoint;
        /**
         * Gets the r component (input)
         */
        get r(): NodeMaterialConnectionPoint;
        /**
         * Gets the g component (input)
         */
        get g(): NodeMaterialConnectionPoint;
        /**
         * Gets the b component (input)
         */
        get b(): NodeMaterialConnectionPoint;
        /**
         * Gets the a component (input)
         */
        get a(): NodeMaterialConnectionPoint;
        /**
         * Gets the rgba component (output)
         */
        get rgba(): NodeMaterialConnectionPoint;
        /**
         * Gets the rgb component (output)
         */
        get rgbOut(): NodeMaterialConnectionPoint;
        /**
         * Gets the rgb component (output)
         * @deprecated Please use rgbOut instead.
         */
        get rgb(): NodeMaterialConnectionPoint;
        protected _inputRename(name: string): string;
        private _buildSwizzle;
        protected _buildBlock(state: NodeMaterialBuildState): this;
        serialize(): any;
        _deserialize(serializationObject: any, scene: Scene, rootUrl: string): void;
        protected _dumpPropertiesCode(): string;
    }


    /**
     * Block used to expand a Color3/4 into 4 outputs (one for each component)
     */
    export class ColorSplitterBlock extends NodeMaterialBlock {
        /**
         * Create a new ColorSplitterBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the rgba component (input)
         */
        get rgba(): NodeMaterialConnectionPoint;
        /**
         * Gets the rgb component (input)
         */
        get rgbIn(): NodeMaterialConnectionPoint;
        /**
         * Gets the rgb component (output)
         */
        get rgbOut(): NodeMaterialConnectionPoint;
        /**
         * Gets the r component (output)
         */
        get r(): NodeMaterialConnectionPoint;
        /**
         * Gets the g component (output)
         */
        get g(): NodeMaterialConnectionPoint;
        /**
         * Gets the b component (output)
         */
        get b(): NodeMaterialConnectionPoint;
        /**
         * Gets the a component (output)
         */
        get a(): NodeMaterialConnectionPoint;
        protected _inputRename(name: string): string;
        protected _outputRename(name: string): string;
        protected _buildBlock(state: NodeMaterialBuildState): this | undefined;
    }


    /**
     * Operations supported by the ConditionalBlock block
     */
    export enum ConditionalBlockConditions {
        /** Equal */
        Equal = 0,
        /** NotEqual */
        NotEqual = 1,
        /** LessThan */
        LessThan = 2,
        /** GreaterThan */
        GreaterThan = 3,
        /** LessOrEqual */
        LessOrEqual = 4,
        /** GreaterOrEqual */
        GreaterOrEqual = 5,
        /** Logical Exclusive OR */
        Xor = 6,
        /** Logical Or */
        Or = 7,
        /** Logical And */
        And = 8
    }
    /**
     * Block used to apply conditional operation between floats
     * @since 5.0.0
     */
    export class ConditionalBlock extends NodeMaterialBlock {
        /**
         * Gets or sets the condition applied by the block
         */
        condition: ConditionalBlockConditions;
        /**
         * Creates a new ConditionalBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the first operand component
         */
        get a(): NodeMaterialConnectionPoint;
        /**
         * Gets the second operand component
         */
        get b(): NodeMaterialConnectionPoint;
        /**
         * Gets the value to return if condition is true
         */
        get true(): NodeMaterialConnectionPoint;
        /**
         * Gets the value to return if condition is false
         */
        get false(): NodeMaterialConnectionPoint;
        /**
         * Gets the output component
         */
        get output(): NodeMaterialConnectionPoint;
        protected _buildBlock(state: NodeMaterialBuildState): this;
        serialize(): any;
        _deserialize(serializationObject: any, scene: Scene, rootUrl: string): void;
        protected _dumpPropertiesCode(): string;
    }


    /**
     * Block used to apply a cross product between 2 vectors
     */
    export class CrossBlock extends NodeMaterialBlock {
        /**
         * Creates a new CrossBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the left operand input component
         */
        get left(): NodeMaterialConnectionPoint;
        /**
         * Gets the right operand input component
         */
        get right(): NodeMaterialConnectionPoint;
        /**
         * Gets the output component
         */
        get output(): NodeMaterialConnectionPoint;
        protected _buildBlock(state: NodeMaterialBuildState): this;
    }


    /**
     * Custom block created from user-defined json
     */
    export class CustomBlock extends NodeMaterialBlock {
        private _options;
        private _code;
        /**
         * Gets or sets the options for this custom block
         */
        get options(): any;
        set options(options: any);
        /**
         * Creates a new CustomBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        protected _buildBlock(state: NodeMaterialBuildState): this;
        protected _dumpPropertiesCode(): string;
        serialize(): any;
        _deserialize(serializationObject: any, scene: Scene, rootUrl: string): void;
        private _deserializeOptions;
        private _findInputByName;
    }


    /**
     * Block used to desaturate a color
     */
    export class DesaturateBlock extends NodeMaterialBlock {
        /**
         * Creates a new DesaturateBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the color operand input component
         */
        get color(): NodeMaterialConnectionPoint;
        /**
         * Gets the level operand input component
         */
        get level(): NodeMaterialConnectionPoint;
        /**
         * Gets the output component
         */
        get output(): NodeMaterialConnectionPoint;
        protected _buildBlock(state: NodeMaterialBuildState): this;
    }


    /**
     * Block used to get the distance between 2 values
     */
    export class DistanceBlock extends NodeMaterialBlock {
        /**
         * Creates a new DistanceBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the left operand input component
         */
        get left(): NodeMaterialConnectionPoint;
        /**
         * Gets the right operand input component
         */
        get right(): NodeMaterialConnectionPoint;
        /**
         * Gets the output component
         */
        get output(): NodeMaterialConnectionPoint;
        protected _buildBlock(state: NodeMaterialBuildState): this;
    }


    /**
     * Block used to divide 2 vectors
     */
    export class DivideBlock extends NodeMaterialBlock {
        /**
         * Creates a new DivideBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the left operand input component
         */
        get left(): NodeMaterialConnectionPoint;
        /**
         * Gets the right operand input component
         */
        get right(): NodeMaterialConnectionPoint;
        /**
         * Gets the output component
         */
        get output(): NodeMaterialConnectionPoint;
        protected _buildBlock(state: NodeMaterialBuildState): this;
    }


    /**
     * Block used to apply a dot product between 2 vectors
     */
    export class DotBlock extends NodeMaterialBlock {
        /**
         * Creates a new DotBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the left operand input component
         */
        get left(): NodeMaterialConnectionPoint;
        /**
         * Gets the right operand input component
         */
        get right(): NodeMaterialConnectionPoint;
        /**
         * Gets the output component
         */
        get output(): NodeMaterialConnectionPoint;
        protected _buildBlock(state: NodeMaterialBuildState): this;
    }


    /**
     * Block used to implement clip planes
     */
    export class ClipPlanesBlock extends NodeMaterialBlock {
        /**
         * Create a new ClipPlanesBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Initialize the block and prepare the context for build
         * @param state defines the state that will be used for the build
         */
        initialize(state: NodeMaterialBuildState): void;
        /**
         * Gets the worldPosition input component
         */
        get worldPosition(): NodeMaterialConnectionPoint;
        get target(): NodeMaterialBlockTargets;
        set target(value: NodeMaterialBlockTargets);
        prepareDefines(mesh: AbstractMesh, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines): void;
        bind(effect: Effect, nodeMaterial: NodeMaterial, mesh?: Mesh): void;
        protected _buildBlock(state: NodeMaterialBuildState): this | undefined;
    }


    /**
     * Base block used as input for post process
     */
    export class CurrentScreenBlock extends NodeMaterialBlock {
        private _samplerName;
        private _linearDefineName;
        private _gammaDefineName;
        private _mainUVName;
        private _tempTextureRead;
        /**
         * Gets or sets the texture associated with the node
         */
        texture: Nullable<BaseTexture>;
        /**
         * Gets or sets a boolean indicating if content needs to be converted to gamma space
         */
        convertToGammaSpace: boolean;
        /**
         * Gets or sets a boolean indicating if content needs to be converted to linear space
         */
        convertToLinearSpace: boolean;
        /**
         * Create a new CurrentScreenBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the uv input component
         */
        get uv(): NodeMaterialConnectionPoint;
        /**
         * Gets the rgba output component
         */
        get rgba(): NodeMaterialConnectionPoint;
        /**
         * Gets the rgb output component
         */
        get rgb(): NodeMaterialConnectionPoint;
        /**
         * Gets the r output component
         */
        get r(): NodeMaterialConnectionPoint;
        /**
         * Gets the g output component
         */
        get g(): NodeMaterialConnectionPoint;
        /**
         * Gets the b output component
         */
        get b(): NodeMaterialConnectionPoint;
        /**
         * Gets the a output component
         */
        get a(): NodeMaterialConnectionPoint;
        /**
         * Initialize the block and prepare the context for build
         * @param state defines the state that will be used for the build
         */
        initialize(state: NodeMaterialBuildState): void;
        get target(): NodeMaterialBlockTargets;
        prepareDefines(mesh: AbstractMesh, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines): void;
        isReady(): boolean;
        private _injectVertexCode;
        private _writeTextureRead;
        private _writeOutput;
        protected _buildBlock(state: NodeMaterialBuildState): this | undefined;
        serialize(): any;
        _deserialize(serializationObject: any, scene: Scene, rootUrl: string): void;
    }


    /**
     * Block used to add support for scene fog
     */
    export class FogBlock extends NodeMaterialBlock {
        private _fogDistanceName;
        private _fogParameters;
        /**
         * Create a new FogBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the world position input component
         */
        get worldPosition(): NodeMaterialConnectionPoint;
        /**
         * Gets the view input component
         */
        get view(): NodeMaterialConnectionPoint;
        /**
         * Gets the color input component
         */
        get input(): NodeMaterialConnectionPoint;
        /**
         * Gets the fog color input component
         */
        get fogColor(): NodeMaterialConnectionPoint;
        /**
         * Gets the output component
         */
        get output(): NodeMaterialConnectionPoint;
        autoConfigure(material: NodeMaterial): void;
        prepareDefines(mesh: AbstractMesh, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines): void;
        bind(effect: Effect, nodeMaterial: NodeMaterial, mesh?: Mesh): void;
        protected _buildBlock(state: NodeMaterialBuildState): this;
    }


    /**
     * Block used to provide an image for a TextureBlock
     */
    export class ImageSourceBlock extends NodeMaterialBlock {
        private _samplerName;
        protected _texture: Nullable<Texture>;
        /**
         * Gets or sets the texture associated with the node
         */
        get texture(): Nullable<Texture>;
        set texture(texture: Nullable<Texture>);
        /**
         * Gets the sampler name associated with this image source
         */
        get samplerName(): string;
        /**
         * Creates a new ImageSourceBlock
         * @param name defines the block name
         */
        constructor(name: string);
        bind(effect: Effect): void;
        isReady(): boolean;
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the output component
         */
        get source(): NodeMaterialConnectionPoint;
        protected _buildBlock(state: NodeMaterialBuildState): this;
        protected _dumpPropertiesCode(): string;
        serialize(): any;
        _deserialize(serializationObject: any, scene: Scene, rootUrl: string): void;
    }




    /**
     * Block used to add light in the fragment shader
     */
    export class LightBlock extends NodeMaterialBlock {
        private _lightId;
        /**
         * Gets or sets the light associated with this block
         */
        light: Nullable<Light>;
        /** Indicates that no code should be generated in the vertex shader. Can be useful in some specific circumstances (like when doing ray marching for eg) */
        generateOnlyFragmentCode: boolean;
        private static _OnGenerateOnlyFragmentCodeChanged;
        private _setTarget;
        /**
         * Create a new LightBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the world position input component
         */
        get worldPosition(): NodeMaterialConnectionPoint;
        /**
         * Gets the world normal input component
         */
        get worldNormal(): NodeMaterialConnectionPoint;
        /**
         * Gets the camera (or eye) position component
         */
        get cameraPosition(): NodeMaterialConnectionPoint;
        /**
         * Gets the glossiness component
         */
        get glossiness(): NodeMaterialConnectionPoint;
        /**
         * Gets the glossiness power component
         */
        get glossPower(): NodeMaterialConnectionPoint;
        /**
         * Gets the diffuse color component
         */
        get diffuseColor(): NodeMaterialConnectionPoint;
        /**
         * Gets the specular color component
         */
        get specularColor(): NodeMaterialConnectionPoint;
        /**
         * Gets the view matrix component
         */
        get view(): NodeMaterialConnectionPoint;
        /**
         * Gets the diffuse output component
         */
        get diffuseOutput(): NodeMaterialConnectionPoint;
        /**
         * Gets the specular output component
         */
        get specularOutput(): NodeMaterialConnectionPoint;
        /**
         * Gets the shadow output component
         */
        get shadow(): NodeMaterialConnectionPoint;
        autoConfigure(material: NodeMaterial): void;
        prepareDefines(mesh: AbstractMesh, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines): void;
        updateUniformsAndSamples(state: NodeMaterialBuildState, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines, uniformBuffers: string[]): void;
        bind(effect: Effect, nodeMaterial: NodeMaterial, mesh?: Mesh): void;
        private _injectVertexCode;
        protected _buildBlock(state: NodeMaterialBuildState): this | undefined;
        serialize(): any;
        _deserialize(serializationObject: any, scene: Scene, rootUrl: string): void;
    }


    /**
     * Base block used to read a reflection texture from a sampler
     */
    export abstract class ReflectionTextureBaseBlock extends NodeMaterialBlock {
        /** @internal */
        _define3DName: string;
        /** @internal */
        _defineCubicName: string;
        /** @internal */
        _defineExplicitName: string;
        /** @internal */
        _defineProjectionName: string;
        /** @internal */
        _defineLocalCubicName: string;
        /** @internal */
        _defineSphericalName: string;
        /** @internal */
        _definePlanarName: string;
        /** @internal */
        _defineEquirectangularName: string;
        /** @internal */
        _defineMirroredEquirectangularFixedName: string;
        /** @internal */
        _defineEquirectangularFixedName: string;
        /** @internal */
        _defineSkyboxName: string;
        /** @internal */
        _defineOppositeZ: string;
        /** @internal */
        _cubeSamplerName: string;
        /** @internal */
        _2DSamplerName: string;
        /** @internal */
        _reflectionPositionName: string;
        /** @internal */
        _reflectionSizeName: string;
        protected _positionUVWName: string;
        protected _directionWName: string;
        protected _reflectionVectorName: string;
        /** @internal */
        _reflectionCoordsName: string;
        /** @internal */
        _reflectionMatrixName: string;
        protected _reflectionColorName: string;
        protected _worldPositionNameInFragmentOnlyMode: string;
        protected _texture: Nullable<BaseTexture>;
        /**
         * Gets or sets the texture associated with the node
         */
        get texture(): Nullable<BaseTexture>;
        set texture(texture: Nullable<BaseTexture>);
        /** Indicates that no code should be generated in the vertex shader. Can be useful in some specific circumstances (like when doing ray marching for eg) */
        generateOnlyFragmentCode: boolean;
        protected static _OnGenerateOnlyFragmentCodeChanged(block: NodeMaterialBlock, _propertyName: string): boolean;
        protected _onGenerateOnlyFragmentCodeChanged(): boolean;
        protected _setTarget(): void;
        /**
         * Create a new ReflectionTextureBaseBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the world position input component
         */
        abstract get position(): NodeMaterialConnectionPoint;
        /**
         * Gets the world position input component
         */
        abstract get worldPosition(): NodeMaterialConnectionPoint;
        /**
         * Gets the world normal input component
         */
        abstract get worldNormal(): NodeMaterialConnectionPoint;
        /**
         * Gets the world input component
         */
        abstract get world(): NodeMaterialConnectionPoint;
        /**
         * Gets the camera (or eye) position component
         */
        abstract get cameraPosition(): NodeMaterialConnectionPoint;
        /**
         * Gets the view input component
         */
        abstract get view(): NodeMaterialConnectionPoint;
        protected _getTexture(): Nullable<BaseTexture>;
        autoConfigure(material: NodeMaterial): void;
        prepareDefines(mesh: AbstractMesh, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines): void;
        isReady(): boolean;
        bind(effect: Effect, nodeMaterial: NodeMaterial, mesh?: Mesh): void;
        /**
         * Gets the code to inject in the vertex shader
         * @param state current state of the node material building
         * @returns the shader code
         */
        handleVertexSide(state: NodeMaterialBuildState): string;
        /**
         * Handles the inits for the fragment code path
         * @param state node material build state
         */
        handleFragmentSideInits(state: NodeMaterialBuildState): void;
        /**
         * Generates the reflection coords code for the fragment code path
         * @param worldNormalVarName name of the world normal variable
         * @param worldPos name of the world position variable. If not provided, will use the world position connected to this block
         * @param onlyReflectionVector if true, generates code only for the reflection vector computation, not for the reflection coordinates
         * @param doNotEmitInvertZ if true, does not emit the invertZ code
         * @returns the shader code
         */
        handleFragmentSideCodeReflectionCoords(worldNormalVarName: string, worldPos?: string, onlyReflectionVector?: boolean, doNotEmitInvertZ?: boolean): string;
        /**
         * Generates the reflection color code for the fragment code path
         * @param lodVarName name of the lod variable
         * @param swizzleLookupTexture swizzle to use for the final color variable
         * @returns the shader code
         */
        handleFragmentSideCodeReflectionColor(lodVarName?: string, swizzleLookupTexture?: string): string;
        /**
         * Generates the code corresponding to the connected output points
         * @param state node material build state
         * @param varName name of the variable to output
         * @returns the shader code
         */
        writeOutputs(state: NodeMaterialBuildState, varName: string): string;
        protected _buildBlock(state: NodeMaterialBuildState): this;
        protected _dumpPropertiesCode(): string;
        serialize(): any;
        _deserialize(serializationObject: any, scene: Scene, rootUrl: string): void;
    }


    /**
     * Block used to read a reflection texture from a sampler
     */
    export class ReflectionTextureBlock extends ReflectionTextureBaseBlock {
        protected _onGenerateOnlyFragmentCodeChanged(): boolean;
        protected _setTarget(): void;
        /**
         * Create a new ReflectionTextureBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the world position input component
         */
        get position(): NodeMaterialConnectionPoint;
        /**
         * Gets the world position input component
         */
        get worldPosition(): NodeMaterialConnectionPoint;
        /**
         * Gets the world normal input component
         */
        get worldNormal(): NodeMaterialConnectionPoint;
        /**
         * Gets the world input component
         */
        get world(): NodeMaterialConnectionPoint;
        /**
         * Gets the camera (or eye) position component
         */
        get cameraPosition(): NodeMaterialConnectionPoint;
        /**
         * Gets the view input component
         */
        get view(): NodeMaterialConnectionPoint;
        /**
         * Gets the rgb output component
         */
        get rgb(): NodeMaterialConnectionPoint;
        /**
         * Gets the rgba output component
         */
        get rgba(): NodeMaterialConnectionPoint;
        /**
         * Gets the r output component
         */
        get r(): NodeMaterialConnectionPoint;
        /**
         * Gets the g output component
         */
        get g(): NodeMaterialConnectionPoint;
        /**
         * Gets the b output component
         */
        get b(): NodeMaterialConnectionPoint;
        /**
         * Gets the a output component
         */
        get a(): NodeMaterialConnectionPoint;
        autoConfigure(material: NodeMaterial): void;
        protected _buildBlock(state: NodeMaterialBuildState): this;
    }


    /**
     * Block used to retrieve the depth (zbuffer) of the scene
     * @since 5.0.0
     */
    export class SceneDepthBlock extends NodeMaterialBlock {
        private _samplerName;
        private _mainUVName;
        private _tempTextureRead;
        /**
         * Defines if the depth renderer should be setup in non linear mode
         */
        useNonLinearDepth: boolean;
        /**
         * Defines if the depth renderer should be setup in full 32 bits float mode
         */
        force32itsFloat: boolean;
        /**
         * Create a new SceneDepthBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the uv input component
         */
        get uv(): NodeMaterialConnectionPoint;
        /**
         * Gets the depth output component
         */
        get depth(): NodeMaterialConnectionPoint;
        /**
         * Initialize the block and prepare the context for build
         * @param state defines the state that will be used for the build
         */
        initialize(state: NodeMaterialBuildState): void;
        get target(): NodeMaterialBlockTargets;
        private _getTexture;
        bind(effect: Effect, nodeMaterial: NodeMaterial): void;
        private _injectVertexCode;
        private _writeTextureRead;
        private _writeOutput;
        protected _buildBlock(state: NodeMaterialBuildState): this | undefined;
        serialize(): any;
        _deserialize(serializationObject: any, scene: Scene, rootUrl: string): void;
    }


    /**
     * Block used to read a texture from a sampler
     */
    export class TextureBlock extends NodeMaterialBlock {
        private _defineName;
        private _linearDefineName;
        private _gammaDefineName;
        private _tempTextureRead;
        private _samplerName;
        private _transformedUVName;
        private _textureTransformName;
        private _textureInfoName;
        private _mainUVName;
        private _mainUVDefineName;
        private _fragmentOnly;
        private _imageSource;
        protected _texture: Nullable<Texture>;
        /**
         * Gets or sets the texture associated with the node
         */
        get texture(): Nullable<Texture>;
        set texture(texture: Nullable<Texture>);
        /**
         * Gets the sampler name associated with this texture
         */
        get samplerName(): string;
        /**
         * Gets a boolean indicating that this block is linked to an ImageSourceBlock
         */
        get hasImageSource(): boolean;
        private _convertToGammaSpace;
        /**
         * Gets or sets a boolean indicating if content needs to be converted to gamma space
         */
        set convertToGammaSpace(value: boolean);
        get convertToGammaSpace(): boolean;
        private _convertToLinearSpace;
        /**
         * Gets or sets a boolean indicating if content needs to be converted to linear space
         */
        set convertToLinearSpace(value: boolean);
        get convertToLinearSpace(): boolean;
        /**
         * Gets or sets a boolean indicating if multiplication of texture with level should be disabled
         */
        disableLevelMultiplication: boolean;
        /**
         * Create a new TextureBlock
         * @param name defines the block name
         * @param fragmentOnly
         */
        constructor(name: string, fragmentOnly?: boolean);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the uv input component
         */
        get uv(): NodeMaterialConnectionPoint;
        /**
         * Gets the source input component
         */
        get source(): NodeMaterialConnectionPoint;
        /**
         * Gets the rgba output component
         */
        get rgba(): NodeMaterialConnectionPoint;
        /**
         * Gets the rgb output component
         */
        get rgb(): NodeMaterialConnectionPoint;
        /**
         * Gets the r output component
         */
        get r(): NodeMaterialConnectionPoint;
        /**
         * Gets the g output component
         */
        get g(): NodeMaterialConnectionPoint;
        /**
         * Gets the b output component
         */
        get b(): NodeMaterialConnectionPoint;
        /**
         * Gets the a output component
         */
        get a(): NodeMaterialConnectionPoint;
        /**
         * Gets the level output component
         */
        get level(): NodeMaterialConnectionPoint;
        get target(): NodeMaterialBlockTargets;
        set target(value: NodeMaterialBlockTargets);
        autoConfigure(material: NodeMaterial): void;
        initializeDefines(mesh: AbstractMesh, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines): void;
        prepareDefines(mesh: AbstractMesh, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines): void;
        isReady(): boolean;
        bind(effect: Effect): void;
        private get _isMixed();
        private _injectVertexCode;
        private _generateTextureLookup;
        private _writeTextureRead;
        private _generateConversionCode;
        private _writeOutput;
        protected _buildBlock(state: NodeMaterialBuildState): this | undefined;
        protected _dumpPropertiesCode(): string;
        serialize(): any;
        _deserialize(serializationObject: any, scene: Scene, rootUrl: string): void;
    }


    /**
     * Block used as a pass through
     */
    export class ElbowBlock extends NodeMaterialBlock {
        /**
         * Creates a new ElbowBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the input component
         */
        get input(): NodeMaterialConnectionPoint;
        /**
         * Gets the output component
         */
        get output(): NodeMaterialConnectionPoint;
        /**
         * Gets or sets the target of the block
         */
        get target(): NodeMaterialBlockTargets;
        set target(value: NodeMaterialBlockTargets);
        protected _buildBlock(state: NodeMaterialBuildState): this;
    }


    /**
     * Block used to get the derivative value on x and y of a given input
     */
    export class DerivativeBlock extends NodeMaterialBlock {
        /**
         * Create a new DerivativeBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the input component
         */
        get input(): NodeMaterialConnectionPoint;
        /**
         * Gets the derivative output on x
         */
        get dx(): NodeMaterialConnectionPoint;
        /**
         * Gets the derivative output on y
         */
        get dy(): NodeMaterialConnectionPoint;
        protected _buildBlock(state: NodeMaterialBuildState): this;
    }


    /**
     * Block used to discard a pixel if a value is smaller than a cutoff
     */
    export class DiscardBlock extends NodeMaterialBlock {
        /**
         * Create a new DiscardBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the color input component
         */
        get value(): NodeMaterialConnectionPoint;
        /**
         * Gets the cutoff input component
         */
        get cutoff(): NodeMaterialConnectionPoint;
        protected _buildBlock(state: NodeMaterialBuildState): this | undefined;
    }


    /**
     * Block used to make gl_FragCoord available
     */
    export class FragCoordBlock extends NodeMaterialBlock {
        /**
         * Creates a new FragCoordBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the xy component
         */
        get xy(): NodeMaterialConnectionPoint;
        /**
         * Gets the xyz component
         */
        get xyz(): NodeMaterialConnectionPoint;
        /**
         * Gets the xyzw component
         */
        get xyzw(): NodeMaterialConnectionPoint;
        /**
         * Gets the x component
         */
        get x(): NodeMaterialConnectionPoint;
        /**
         * Gets the y component
         */
        get y(): NodeMaterialConnectionPoint;
        /**
         * Gets the z component
         */
        get z(): NodeMaterialConnectionPoint;
        /**
         * Gets the w component
         */
        get output(): NodeMaterialConnectionPoint;
        protected writeOutputs(state: NodeMaterialBuildState): string;
        protected _buildBlock(state: NodeMaterialBuildState): this;
    }


    /**
     * Block used to write the fragment depth
     */
    export class FragDepthBlock extends NodeMaterialBlock {
        /**
         * Create a new FragDepthBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the depth input component
         */
        get depth(): NodeMaterialConnectionPoint;
        /**
         * Gets the worldPos input component
         */
        get worldPos(): NodeMaterialConnectionPoint;
        /**
         * Gets the viewProjection input component
         */
        get viewProjection(): NodeMaterialConnectionPoint;
        protected _buildBlock(state: NodeMaterialBuildState): this;
    }


    /**
     * Block used to output the final color
     */
    export class FragmentOutputBlock extends NodeMaterialBlock {
        private _linearDefineName;
        private _gammaDefineName;
        /**
         * Create a new FragmentOutputBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /** Gets or sets a boolean indicating if content needs to be converted to gamma space */
        convertToGammaSpace: boolean;
        /** Gets or sets a boolean indicating if content needs to be converted to linear space */
        convertToLinearSpace: boolean;
        /** Gets or sets a boolean indicating if logarithmic depth should be used */
        useLogarithmicDepth: boolean;
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Initialize the block and prepare the context for build
         * @param state defines the state that will be used for the build
         */
        initialize(state: NodeMaterialBuildState): void;
        /**
         * Gets the rgba input component
         */
        get rgba(): NodeMaterialConnectionPoint;
        /**
         * Gets the rgb input component
         */
        get rgb(): NodeMaterialConnectionPoint;
        /**
         * Gets the a input component
         */
        get a(): NodeMaterialConnectionPoint;
        prepareDefines(mesh: AbstractMesh, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines): void;
        bind(effect: Effect, nodeMaterial: NodeMaterial, mesh?: Mesh): void;
        protected _buildBlock(state: NodeMaterialBuildState): this;
        protected _dumpPropertiesCode(): string;
        serialize(): any;
        _deserialize(serializationObject: any, scene: Scene, rootUrl: string): void;
    }


    /**
     * Block used to test if the fragment shader is front facing
     */
    export class FrontFacingBlock extends NodeMaterialBlock {
        /**
         * Creates a new FrontFacingBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the output component
         */
        get output(): NodeMaterialConnectionPoint;
        protected _buildBlock(state: NodeMaterialBuildState): this;
    }


    /**
     * Block used to convert a height vector to a normal
     */
    export class HeightToNormalBlock extends NodeMaterialBlock {
        /**
         * Creates a new HeightToNormalBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Defines if the output should be generated in world or tangent space.
         * Note that in tangent space the result is also scaled by 0.5 and offsetted by 0.5 so that it can directly be used as a PerturbNormal.normalMapColor input
         */
        generateInWorldSpace: boolean;
        /**
         * Defines that the worldNormal input will be normalized by the HeightToNormal block before being used
         */
        automaticNormalizationNormal: boolean;
        /**
         * Defines that the worldTangent input will be normalized by the HeightToNormal block before being used
         */
        automaticNormalizationTangent: boolean;
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the input component
         */
        get input(): NodeMaterialConnectionPoint;
        /**
         * Gets the position component
         */
        get worldPosition(): NodeMaterialConnectionPoint;
        /**
         * Gets the normal component
         */
        get worldNormal(): NodeMaterialConnectionPoint;
        /**
         * Gets the tangent component
         */
        get worldTangent(): NodeMaterialConnectionPoint;
        /**
         * Gets the output component
         */
        get output(): NodeMaterialConnectionPoint;
        /**
         * Gets the xyz component
         */
        get xyz(): NodeMaterialConnectionPoint;
        protected _buildBlock(state: NodeMaterialBuildState): this;
        protected _dumpPropertiesCode(): string;
        serialize(): any;
        _deserialize(serializationObject: any, scene: Scene, rootUrl: string): void;
    }


    /**
     * Block used to add image processing support to fragment shader
     */
    export class ImageProcessingBlock extends NodeMaterialBlock {
        /**
         * Create a new ImageProcessingBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Defines if the input should be converted to linear space (default: true)
         */
        convertInputToLinearSpace: boolean;
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the color input component
         */
        get color(): NodeMaterialConnectionPoint;
        /**
         * Gets the output component
         */
        get output(): NodeMaterialConnectionPoint;
        /**
         * Gets the rgb component
         */
        get rgb(): NodeMaterialConnectionPoint;
        /**
         * Initialize the block and prepare the context for build
         * @param state defines the state that will be used for the build
         */
        initialize(state: NodeMaterialBuildState): void;
        isReady(mesh: AbstractMesh, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines): boolean;
        prepareDefines(mesh: AbstractMesh, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines): void;
        bind(effect: Effect, nodeMaterial: NodeMaterial, mesh?: Mesh): void;
        protected _buildBlock(state: NodeMaterialBuildState): this;
        protected _dumpPropertiesCode(): string;
        serialize(): any;
        _deserialize(serializationObject: any, scene: Scene, rootUrl: string): void;
    }




    /**
     * Block used to perturb normals based on a normal map
     */
    export class PerturbNormalBlock extends NodeMaterialBlock {
        private _tangentSpaceParameterName;
        private _tangentCorrectionFactorName;
        private _worldMatrixName;
        /** Gets or sets a boolean indicating that normal should be inverted on X axis */
        invertX: boolean;
        /** Gets or sets a boolean indicating that normal should be inverted on Y axis */
        invertY: boolean;
        /** Gets or sets a boolean indicating that parallax occlusion should be enabled */
        useParallaxOcclusion: boolean;
        /** Gets or sets a boolean indicating that sampling mode is in Object space */
        useObjectSpaceNormalMap: boolean;
        /**
         * Create a new PerturbNormalBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the world position input component
         */
        get worldPosition(): NodeMaterialConnectionPoint;
        /**
         * Gets the world normal input component
         */
        get worldNormal(): NodeMaterialConnectionPoint;
        /**
         * Gets the world tangent input component
         */
        get worldTangent(): NodeMaterialConnectionPoint;
        /**
         * Gets the uv input component
         */
        get uv(): NodeMaterialConnectionPoint;
        /**
         * Gets the normal map color input component
         */
        get normalMapColor(): NodeMaterialConnectionPoint;
        /**
         * Gets the strength input component
         */
        get strength(): NodeMaterialConnectionPoint;
        /**
         * Gets the view direction input component
         */
        get viewDirection(): NodeMaterialConnectionPoint;
        /**
         * Gets the parallax scale input component
         */
        get parallaxScale(): NodeMaterialConnectionPoint;
        /**
         * Gets the parallax height input component
         */
        get parallaxHeight(): NodeMaterialConnectionPoint;
        /**
         * Gets the TBN input component
         */
        get TBN(): NodeMaterialConnectionPoint;
        /**
         * Gets the World input component
         */
        get world(): NodeMaterialConnectionPoint;
        /**
         * Gets the output component
         */
        get output(): NodeMaterialConnectionPoint;
        /**
         * Gets the uv offset output component
         */
        get uvOffset(): NodeMaterialConnectionPoint;
        prepareDefines(mesh: AbstractMesh, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines): void;
        bind(effect: Effect, nodeMaterial: NodeMaterial, mesh?: Mesh): void;
        autoConfigure(material: NodeMaterial): void;
        protected _buildBlock(state: NodeMaterialBuildState): this;
        protected _dumpPropertiesCode(): string;
        serialize(): any;
        _deserialize(serializationObject: any, scene: Scene, rootUrl: string): void;
    }


    /**
     * Block used to get the screen sizes
     */
    export class ScreenSizeBlock extends NodeMaterialBlock {
        private _varName;
        private _scene;
        /**
         * Creates a new ScreenSizeBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the xy component
         */
        get xy(): NodeMaterialConnectionPoint;
        /**
         * Gets the x component
         */
        get x(): NodeMaterialConnectionPoint;
        /**
         * Gets the y component
         */
        get y(): NodeMaterialConnectionPoint;
        bind(effect: Effect): void;
        protected writeOutputs(state: NodeMaterialBuildState, varName: string): string;
        protected _buildBlock(state: NodeMaterialBuildState): this;
    }


    /**
     * Block used to transform a vector3 or a vector4 into screen space
     */
    export class ScreenSpaceBlock extends NodeMaterialBlock {
        /**
         * Creates a new ScreenSpaceBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the vector input
         */
        get vector(): NodeMaterialConnectionPoint;
        /**
         * Gets the worldViewProjection transform input
         */
        get worldViewProjection(): NodeMaterialConnectionPoint;
        /**
         * Gets the output component
         */
        get output(): NodeMaterialConnectionPoint;
        /**
         * Gets the x output component
         */
        get x(): NodeMaterialConnectionPoint;
        /**
         * Gets the y output component
         */
        get y(): NodeMaterialConnectionPoint;
        autoConfigure(material: NodeMaterial): void;
        protected _buildBlock(state: NodeMaterialBuildState): this | undefined;
    }


    /**
     * Block used to output the depth to a shadow map
     */
    export class ShadowMapBlock extends NodeMaterialBlock {
        /**
         * Create a new ShadowMapBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Initialize the block and prepare the context for build
         * @param state defines the state that will be used for the build
         */
        initialize(state: NodeMaterialBuildState): void;
        /**
         * Gets the world position input component
         */
        get worldPosition(): NodeMaterialConnectionPoint;
        /**
         * Gets the view x projection input component
         */
        get viewProjection(): NodeMaterialConnectionPoint;
        /**
         * Gets the world normal input component
         */
        get worldNormal(): NodeMaterialConnectionPoint;
        /**
         * Gets the depth output component
         */
        get depth(): NodeMaterialConnectionPoint;
        protected _buildBlock(state: NodeMaterialBuildState): this;
    }


    /**
     * Block used to implement TBN matrix
     */
    export class TBNBlock extends NodeMaterialBlock {
        /**
         * Create a new TBNBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Initialize the block and prepare the context for build
         * @param state defines the state that will be used for the build
         */
        initialize(state: NodeMaterialBuildState): void;
        /**
         * Gets the normal input component
         */
        get normal(): NodeMaterialConnectionPoint;
        /**
         * Gets the tangent input component
         */
        get tangent(): NodeMaterialConnectionPoint;
        /**
         * Gets the world matrix input component
         */
        get world(): NodeMaterialConnectionPoint;
        /**
         * Gets the TBN output component
         */
        get TBN(): NodeMaterialConnectionPoint;
        /**
         * Gets the row0 of the output matrix
         */
        get row0(): NodeMaterialConnectionPoint;
        /**
         * Gets the row1 of the output matrix
         */
        get row1(): NodeMaterialConnectionPoint;
        /**
         * Gets the row2 of the output matrix
         */
        get row2(): NodeMaterialConnectionPoint;
        get target(): NodeMaterialBlockTargets;
        set target(value: NodeMaterialBlockTargets);
        autoConfigure(material: NodeMaterial): void;
        prepareDefines(mesh: AbstractMesh, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines): void;
        protected _buildBlock(state: NodeMaterialBuildState): this;
    }


    /**
     * Block used to generate a twirl
     */
    export class TwirlBlock extends NodeMaterialBlock {
        /**
         * Creates a new TwirlBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the input component
         */
        get input(): NodeMaterialConnectionPoint;
        /**
         * Gets the strength component
         */
        get strength(): NodeMaterialConnectionPoint;
        /**
         * Gets the center component
         */
        get center(): NodeMaterialConnectionPoint;
        /**
         * Gets the offset component
         */
        get offset(): NodeMaterialConnectionPoint;
        /**
         * Gets the output component
         */
        get output(): NodeMaterialConnectionPoint;
        /**
         * Gets the x output component
         */
        get x(): NodeMaterialConnectionPoint;
        /**
         * Gets the y output component
         */
        get y(): NodeMaterialConnectionPoint;
        autoConfigure(): void;
        protected _buildBlock(state: NodeMaterialBuildState): this;
    }


    /**
     * Block used to compute fresnel value
     */
    export class FresnelBlock extends NodeMaterialBlock {
        /**
         * Create a new FresnelBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the world normal input component
         */
        get worldNormal(): NodeMaterialConnectionPoint;
        /**
         * Gets the view direction input component
         */
        get viewDirection(): NodeMaterialConnectionPoint;
        /**
         * Gets the bias input component
         */
        get bias(): NodeMaterialConnectionPoint;
        /**
         * Gets the camera (or eye) position component
         */
        get power(): NodeMaterialConnectionPoint;
        /**
         * Gets the fresnel output component
         */
        get fresnel(): NodeMaterialConnectionPoint;
        autoConfigure(material: NodeMaterial): void;
        protected _buildBlock(state: NodeMaterialBuildState): this;
    }


    /**
     * Class used to store a color step for the GradientBlock
     */
    export class GradientBlockColorStep {
        private _step;
        /**
         * Gets value indicating which step this color is associated with (between 0 and 1)
         */
        get step(): number;
        /**
         * Sets a value indicating which step this color is associated with (between 0 and 1)
         */
        set step(val: number);
        private _color;
        /**
         * Gets the color associated with this step
         */
        get color(): Color3;
        /**
         * Sets the color associated with this step
         */
        set color(val: Color3);
        /**
         * Creates a new GradientBlockColorStep
         * @param step defines a value indicating which step this color is associated with (between 0 and 1)
         * @param color defines the color associated with this step
         */
        constructor(step: number, color: Color3);
    }
    /**
     * Block used to return a color from a gradient based on an input value between 0 and 1
     */
    export class GradientBlock extends NodeMaterialBlock {
        /**
         * Gets or sets the list of color steps
         */
        colorSteps: GradientBlockColorStep[];
        /** Gets an observable raised when the value is changed */
        onValueChangedObservable: Observable<GradientBlock>;
        /** calls observable when the value is changed*/
        colorStepsUpdated(): void;
        /**
         * Creates a new GradientBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the gradient input component
         */
        get gradient(): NodeMaterialConnectionPoint;
        /**
         * Gets the output component
         */
        get output(): NodeMaterialConnectionPoint;
        private _writeColorConstant;
        protected _buildBlock(state: NodeMaterialBuildState): this | undefined;
        serialize(): any;
        _deserialize(serializationObject: any, scene: Scene, rootUrl: string): void;
        protected _dumpPropertiesCode(): string;
    }




    /**
     * Enum defining the type of animations supported by InputBlock
     */
    export enum AnimatedInputBlockTypes {
        /** No animation */
        None = 0,
        /** Time based animation (is incremented by 0.6 each second). Will only work for floats */
        Time = 1,
        /** Time elapsed (in seconds) since the engine was initialized. Will only work for floats */
        RealTime = 2
    }




    /**
     * Block used to expose an input value
     */
    export class InputBlock extends NodeMaterialBlock {
        private _mode;
        private _associatedVariableName;
        private _storedValue;
        private _valueCallback;
        private _type;
        private _animationType;
        /** Gets or set a value used to limit the range of float values */
        min: number;
        /** Gets or set a value used to limit the range of float values */
        max: number;
        /** Gets or set a value indicating that this input can only get 0 and 1 values */
        isBoolean: boolean;
        /** Gets or sets a value used by the Node Material editor to determine how to configure the current value if it is a matrix */
        matrixMode: number;
        /** @internal */
        _systemValue: Nullable<NodeMaterialSystemValues>;
        /** Gets or sets a boolean indicating that the value of this input will not change after a build */
        isConstant: boolean;
        /** Gets or sets the group to use to display this block in the Inspector */
        groupInInspector: string;
        /** Gets an observable raised when the value is changed */
        onValueChangedObservable: Observable<InputBlock>;
        /** Gets or sets a boolean indicating if content needs to be converted to gamma space (for color3/4 only) */
        convertToGammaSpace: boolean;
        /** Gets or sets a boolean indicating if content needs to be converted to linear space (for color3/4 only) */
        convertToLinearSpace: boolean;
        /**
         * Gets or sets the connection point type (default is float)
         */
        get type(): NodeMaterialBlockConnectionPointTypes;
        /**
         * Creates a new InputBlock
         * @param name defines the block name
         * @param target defines the target of that block (Vertex by default)
         * @param type defines the type of the input (can be set to NodeMaterialBlockConnectionPointTypes.AutoDetect)
         */
        constructor(name: string, target?: NodeMaterialBlockTargets, type?: NodeMaterialBlockConnectionPointTypes);
        /**
         * Validates if a name is a reserve word.
         * @param newName the new name to be given to the node.
         * @returns false if the name is a reserve word, else true.
         */
        validateBlockName(newName: string): boolean;
        /**
         * Gets the output component
         */
        get output(): NodeMaterialConnectionPoint;
        /**
         * Set the source of this connection point to a vertex attribute
         * @param attributeName defines the attribute name (position, uv, normal, etc...). If not specified it will take the connection point name
         * @returns the current connection point
         */
        setAsAttribute(attributeName?: string): InputBlock;
        /**
         * Set the source of this connection point to a system value
         * @param value define the system value to use (world, view, etc...) or null to switch to manual value
         * @returns the current connection point
         */
        setAsSystemValue(value: Nullable<NodeMaterialSystemValues>): InputBlock;
        /**
         * Gets or sets the value of that point.
         * Please note that this value will be ignored if valueCallback is defined
         */
        get value(): any;
        set value(value: any);
        /**
         * Gets or sets a callback used to get the value of that point.
         * Please note that setting this value will force the connection point to ignore the value property
         */
        get valueCallback(): () => any;
        set valueCallback(value: () => any);
        /**
         * Gets or sets the associated variable name in the shader
         */
        get associatedVariableName(): string;
        set associatedVariableName(value: string);
        /** Gets or sets the type of animation applied to the input */
        get animationType(): AnimatedInputBlockTypes;
        set animationType(value: AnimatedInputBlockTypes);
        /**
         * Gets a boolean indicating that this connection point not defined yet
         */
        get isUndefined(): boolean;
        /**
         * Gets or sets a boolean indicating that this connection point is coming from an uniform.
         * In this case the connection point name must be the name of the uniform to use.
         * Can only be set on inputs
         */
        get isUniform(): boolean;
        set isUniform(value: boolean);
        /**
         * Gets or sets a boolean indicating that this connection point is coming from an attribute.
         * In this case the connection point name must be the name of the attribute to use
         * Can only be set on inputs
         */
        get isAttribute(): boolean;
        set isAttribute(value: boolean);
        /**
         * Gets or sets a boolean indicating that this connection point is generating a varying variable.
         * Can only be set on exit points
         */
        get isVarying(): boolean;
        set isVarying(value: boolean);
        /**
         * Gets a boolean indicating that the current connection point is a system value
         */
        get isSystemValue(): boolean;
        /**
         * Gets or sets the current well known value or null if not defined as a system value
         */
        get systemValue(): Nullable<NodeMaterialSystemValues>;
        set systemValue(value: Nullable<NodeMaterialSystemValues>);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Animate the input if animationType !== None
         * @param scene defines the rendering scene
         */
        animate(scene: Scene): void;
        private _emitDefine;
        initialize(): void;
        /**
         * Set the input block to its default value (based on its type)
         */
        setDefaultValue(): void;
        private _emitConstant;
        /** @internal */
        get _noContextSwitch(): boolean;
        private _emit;
        /**
         * @internal
         */
        _transmitWorld(effect: Effect, world: Matrix, worldView: Matrix, worldViewProjection: Matrix): void;
        /**
         * @internal
         */
        _transmit(effect: Effect, scene: Scene, material: NodeMaterial): void;
        protected _buildBlock(state: NodeMaterialBuildState): void;
        protected _dumpPropertiesCode(): string;
        dispose(): void;
        serialize(): any;
        _deserialize(serializationObject: any, scene: Scene, rootUrl: string): void;
    }


    /**
     * Block used to get the length of a vector
     */
    export class LengthBlock extends NodeMaterialBlock {
        /**
         * Creates a new LengthBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the value input component
         */
        get value(): NodeMaterialConnectionPoint;
        /**
         * Gets the output component
         */
        get output(): NodeMaterialConnectionPoint;
        protected _buildBlock(state: NodeMaterialBuildState): this;
    }


    /**
     * Block used to lerp between 2 values
     */
    export class LerpBlock extends NodeMaterialBlock {
        /**
         * Creates a new LerpBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the left operand input component
         */
        get left(): NodeMaterialConnectionPoint;
        /**
         * Gets the right operand input component
         */
        get right(): NodeMaterialConnectionPoint;
        /**
         * Gets the gradient operand input component
         */
        get gradient(): NodeMaterialConnectionPoint;
        /**
         * Gets the output component
         */
        get output(): NodeMaterialConnectionPoint;
        protected _buildBlock(state: NodeMaterialBuildState): this;
    }


    /**
     * Block used to build a matrix from 4 Vector4
     */
    export class MatrixBuilderBlock extends NodeMaterialBlock {
        /**
         * Creates a new MatrixBuilder
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the row0 vector
         */
        get row0(): NodeMaterialConnectionPoint;
        /**
         * Gets the row1 vector
         */
        get row1(): NodeMaterialConnectionPoint;
        /**
         * Gets the row2 vector
         */
        get row2(): NodeMaterialConnectionPoint;
        /**
         * Gets the row3 vector
         */
        get row3(): NodeMaterialConnectionPoint;
        /**
         * Gets the output component
         */
        get output(): NodeMaterialConnectionPoint;
        autoConfigure(): void;
        protected _buildBlock(state: NodeMaterialBuildState): this;
    }


    /**
     * Block used to compute the determinant of a matrix
     */
    export class MatrixDeterminantBlock extends NodeMaterialBlock {
        /**
         * Creates a new MatrixDeterminantBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the input matrix
         */
        get input(): NodeMaterialConnectionPoint;
        /**
         * Gets the output component
         */
        get output(): NodeMaterialConnectionPoint;
        protected _buildBlock(state: NodeMaterialBuildState): this;
    }


    /**
     * Block used to transpose a matrix
     */
    export class MatrixTransposeBlock extends NodeMaterialBlock {
        /**
         * Creates a new MatrixTransposeBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the input matrix
         */
        get input(): NodeMaterialConnectionPoint;
        /**
         * Gets the output component
         */
        get output(): NodeMaterialConnectionPoint;
        protected _buildBlock(state: NodeMaterialBuildState): this;
    }


    /**
     * Block used to get the max of 2 values
     */
    export class MaxBlock extends NodeMaterialBlock {
        /**
         * Creates a new MaxBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the left operand input component
         */
        get left(): NodeMaterialConnectionPoint;
        /**
         * Gets the right operand input component
         */
        get right(): NodeMaterialConnectionPoint;
        /**
         * Gets the output component
         */
        get output(): NodeMaterialConnectionPoint;
        protected _buildBlock(state: NodeMaterialBuildState): this;
    }


    /**
     * Block used to get the min of 2 values
     */
    export class MinBlock extends NodeMaterialBlock {
        /**
         * Creates a new MinBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the left operand input component
         */
        get left(): NodeMaterialConnectionPoint;
        /**
         * Gets the right operand input component
         */
        get right(): NodeMaterialConnectionPoint;
        /**
         * Gets the output component
         */
        get output(): NodeMaterialConnectionPoint;
        protected _buildBlock(state: NodeMaterialBuildState): this;
    }


    /**
     * Block used to compute value of one parameter modulo another
     */
    export class ModBlock extends NodeMaterialBlock {
        /**
         * Creates a new ModBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the left operand input component
         */
        get left(): NodeMaterialConnectionPoint;
        /**
         * Gets the right operand input component
         */
        get right(): NodeMaterialConnectionPoint;
        /**
         * Gets the output component
         */
        get output(): NodeMaterialConnectionPoint;
        protected _buildBlock(state: NodeMaterialBuildState): this;
    }


    /**
     * Block used to multiply 2 values
     */
    export class MultiplyBlock extends NodeMaterialBlock {
        /**
         * Creates a new MultiplyBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the left operand input component
         */
        get left(): NodeMaterialConnectionPoint;
        /**
         * Gets the right operand input component
         */
        get right(): NodeMaterialConnectionPoint;
        /**
         * Gets the output component
         */
        get output(): NodeMaterialConnectionPoint;
        protected _buildBlock(state: NodeMaterialBuildState): this;
    }


    /**
     * Block used to get negative version of a value (i.e. x * -1)
     */
    export class NegateBlock extends NodeMaterialBlock {
        /**
         * Creates a new NegateBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the value input component
         */
        get value(): NodeMaterialConnectionPoint;
        /**
         * Gets the output component
         */
        get output(): NodeMaterialConnectionPoint;
        protected _buildBlock(state: NodeMaterialBuildState): this;
    }


    /**
     * Block used to normalize lerp between 2 values
     */
    export class NLerpBlock extends NodeMaterialBlock {
        /**
         * Creates a new NLerpBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the left operand input component
         */
        get left(): NodeMaterialConnectionPoint;
        /**
         * Gets the right operand input component
         */
        get right(): NodeMaterialConnectionPoint;
        /**
         * Gets the gradient operand input component
         */
        get gradient(): NodeMaterialConnectionPoint;
        /**
         * Gets the output component
         */
        get output(): NodeMaterialConnectionPoint;
        protected _buildBlock(state: NodeMaterialBuildState): this;
    }


    /**
     * Block used to blend normals
     */
    export class NormalBlendBlock extends NodeMaterialBlock {
        /**
         * Creates a new NormalBlendBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the first input component
         */
        get normalMap0(): NodeMaterialConnectionPoint;
        /**
         * Gets the second input component
         */
        get normalMap1(): NodeMaterialConnectionPoint;
        /**
         * Gets the output component
         */
        get output(): NodeMaterialConnectionPoint;
        protected _buildBlock(state: NodeMaterialBuildState): this;
    }


    /**
     * Block used to normalize a vector
     */
    export class NormalizeBlock extends NodeMaterialBlock {
        /**
         * Creates a new NormalizeBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the input component
         */
        get input(): NodeMaterialConnectionPoint;
        /**
         * Gets the output component
         */
        get output(): NodeMaterialConnectionPoint;
        protected _buildBlock(state: NodeMaterialBuildState): this;
    }


    /**
     * Block used to get the opposite (1 - x) of a value
     */
    export class OneMinusBlock extends NodeMaterialBlock {
        /**
         * Creates a new OneMinusBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the input component
         */
        get input(): NodeMaterialConnectionPoint;
        /**
         * Gets the output component
         */
        get output(): NodeMaterialConnectionPoint;
        protected _buildBlock(state: NodeMaterialBuildState): this;
    }




    /**
     * Block used for the particle blend multiply section
     */
    export class ParticleBlendMultiplyBlock extends NodeMaterialBlock {
        /**
         * Create a new ParticleBlendMultiplyBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the color input component
         */
        get color(): NodeMaterialConnectionPoint;
        /**
         * Gets the alphaTexture input component
         */
        get alphaTexture(): NodeMaterialConnectionPoint;
        /**
         * Gets the alphaColor input component
         */
        get alphaColor(): NodeMaterialConnectionPoint;
        /**
         * Gets the blendColor output component
         */
        get blendColor(): NodeMaterialConnectionPoint;
        /**
         * Initialize the block and prepare the context for build
         * @param state defines the state that will be used for the build
         */
        initialize(state: NodeMaterialBuildState): void;
        protected _buildBlock(state: NodeMaterialBuildState): this | undefined;
    }


    /**
     * Block used for the particle ramp gradient section
     */
    export class ParticleRampGradientBlock extends NodeMaterialBlock {
        /**
         * Create a new ParticleRampGradientBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the color input component
         */
        get color(): NodeMaterialConnectionPoint;
        /**
         * Gets the rampColor output component
         */
        get rampColor(): NodeMaterialConnectionPoint;
        /**
         * Initialize the block and prepare the context for build
         * @param state defines the state that will be used for the build
         */
        initialize(state: NodeMaterialBuildState): void;
        protected _buildBlock(state: NodeMaterialBuildState): this | undefined;
    }


    /**
     * Base block used for the particle texture
     */
    export class ParticleTextureBlock extends NodeMaterialBlock {
        private _samplerName;
        private _linearDefineName;
        private _gammaDefineName;
        private _tempTextureRead;
        /**
         * Gets or sets the texture associated with the node
         */
        texture: Nullable<BaseTexture>;
        /**
         * Gets or sets a boolean indicating if content needs to be converted to gamma space
         */
        convertToGammaSpace: boolean;
        /**
         * Gets or sets a boolean indicating if content needs to be converted to linear space
         */
        convertToLinearSpace: boolean;
        /**
         * Create a new ParticleTextureBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the uv input component
         */
        get uv(): NodeMaterialConnectionPoint;
        /**
         * Gets the rgba output component
         */
        get rgba(): NodeMaterialConnectionPoint;
        /**
         * Gets the rgb output component
         */
        get rgb(): NodeMaterialConnectionPoint;
        /**
         * Gets the r output component
         */
        get r(): NodeMaterialConnectionPoint;
        /**
         * Gets the g output component
         */
        get g(): NodeMaterialConnectionPoint;
        /**
         * Gets the b output component
         */
        get b(): NodeMaterialConnectionPoint;
        /**
         * Gets the a output component
         */
        get a(): NodeMaterialConnectionPoint;
        /**
         * Initialize the block and prepare the context for build
         * @param state defines the state that will be used for the build
         */
        initialize(state: NodeMaterialBuildState): void;
        autoConfigure(material: NodeMaterial): void;
        prepareDefines(mesh: AbstractMesh, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines): void;
        isReady(): boolean;
        private _writeOutput;
        protected _buildBlock(state: NodeMaterialBuildState): this | undefined;
        serialize(): any;
        _deserialize(serializationObject: any, scene: Scene, rootUrl: string): void;
    }


    /**
     * Block used to implement the anisotropy module of the PBR material
     */
    export class AnisotropyBlock extends NodeMaterialBlock {
        private _tangentCorrectionFactorName;
        /**
         * The two properties below are set by the main PBR block prior to calling methods of this class.
         * This is to avoid having to add them as inputs here whereas they are already inputs of the main block, so already known.
         * It's less burden on the user side in the editor part.
         */
        /** @internal */
        worldPositionConnectionPoint: NodeMaterialConnectionPoint;
        /** @internal */
        worldNormalConnectionPoint: NodeMaterialConnectionPoint;
        /**
         * Create a new AnisotropyBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Initialize the block and prepare the context for build
         * @param state defines the state that will be used for the build
         */
        initialize(state: NodeMaterialBuildState): void;
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the intensity input component
         */
        get intensity(): NodeMaterialConnectionPoint;
        /**
         * Gets the direction input component
         */
        get direction(): NodeMaterialConnectionPoint;
        /**
         * Gets the uv input component
         */
        get uv(): NodeMaterialConnectionPoint;
        /**
         * Gets the worldTangent input component
         */
        get worldTangent(): NodeMaterialConnectionPoint;
        /**
         * Gets the TBN input component
         */
        get TBN(): NodeMaterialConnectionPoint;
        /**
         * Gets the anisotropy object output component
         */
        get anisotropy(): NodeMaterialConnectionPoint;
        private _generateTBNSpace;
        /**
         * Gets the main code of the block (fragment side)
         * @param state current state of the node material building
         * @param generateTBNSpace if true, the code needed to create the TBN coordinate space is generated
         * @returns the shader code
         */
        getCode(state: NodeMaterialBuildState, generateTBNSpace?: boolean): string;
        prepareDefines(mesh: AbstractMesh, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines): void;
        bind(effect: Effect, nodeMaterial: NodeMaterial, mesh?: Mesh): void;
        protected _buildBlock(state: NodeMaterialBuildState): this;
    }


    /**
     * Block used to implement the clear coat module of the PBR material
     */
    export class ClearCoatBlock extends NodeMaterialBlock {
        private _scene;
        private _tangentCorrectionFactorName;
        /**
         * Create a new ClearCoatBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Defines if the F0 value should be remapped to account for the interface change in the material.
         */
        remapF0OnInterfaceChange: boolean;
        /**
         * Initialize the block and prepare the context for build
         * @param state defines the state that will be used for the build
         */
        initialize(state: NodeMaterialBuildState): void;
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the intensity input component
         */
        get intensity(): NodeMaterialConnectionPoint;
        /**
         * Gets the roughness input component
         */
        get roughness(): NodeMaterialConnectionPoint;
        /**
         * Gets the ior input component
         */
        get indexOfRefraction(): NodeMaterialConnectionPoint;
        /**
         * Gets the bump texture input component
         */
        get normalMapColor(): NodeMaterialConnectionPoint;
        /**
         * Gets the uv input component
         */
        get uv(): NodeMaterialConnectionPoint;
        /**
         * Gets the tint color input component
         */
        get tintColor(): NodeMaterialConnectionPoint;
        /**
         * Gets the tint "at distance" input component
         */
        get tintAtDistance(): NodeMaterialConnectionPoint;
        /**
         * Gets the tint thickness input component
         */
        get tintThickness(): NodeMaterialConnectionPoint;
        /**
         * Gets the world tangent input component
         */
        get worldTangent(): NodeMaterialConnectionPoint;
        /**
         * Gets the world normal input component
         */
        get worldNormal(): NodeMaterialConnectionPoint;
        /**
         * Gets the TBN input component
         */
        get TBN(): NodeMaterialConnectionPoint;
        /**
         * Gets the clear coat object output component
         */
        get clearcoat(): NodeMaterialConnectionPoint;
        autoConfigure(): void;
        prepareDefines(mesh: AbstractMesh, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines): void;
        bind(effect: Effect, nodeMaterial: NodeMaterial, mesh?: Mesh): void;
        private _generateTBNSpace;
        /**
         * Gets the main code of the block (fragment side)
         * @param state current state of the node material building
         * @param ccBlock instance of a ClearCoatBlock or null if the code must be generated without an active clear coat module
         * @param reflectionBlock instance of a ReflectionBlock null if the code must be generated without an active reflection module
         * @param worldPosVarName name of the variable holding the world position
         * @param generateTBNSpace if true, the code needed to create the TBN coordinate space is generated
         * @param vTBNAvailable indicate that the vTBN variable is already existing because it has already been generated by another block (PerturbNormal or Anisotropy)
         * @param worldNormalVarName name of the variable holding the world normal
         * @returns the shader code
         */
        static GetCode(state: NodeMaterialBuildState, ccBlock: Nullable<ClearCoatBlock>, reflectionBlock: Nullable<ReflectionBlock>, worldPosVarName: string, generateTBNSpace: boolean, vTBNAvailable: boolean, worldNormalVarName: string): string;
        protected _buildBlock(state: NodeMaterialBuildState): this;
        protected _dumpPropertiesCode(): string;
        serialize(): any;
        _deserialize(serializationObject: any, scene: Scene, rootUrl: string): void;
    }




    /**
     * Block used to implement the iridescence module of the PBR material
     */
    export class IridescenceBlock extends NodeMaterialBlock {
        /**
         * Create a new IridescenceBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Initialize the block and prepare the context for build
         * @param state defines the state that will be used for the build
         */
        initialize(state: NodeMaterialBuildState): void;
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the intensity input component
         */
        get intensity(): NodeMaterialConnectionPoint;
        /**
         * Gets the indexOfRefraction input component
         */
        get indexOfRefraction(): NodeMaterialConnectionPoint;
        /**
         * Gets the thickness input component
         */
        get thickness(): NodeMaterialConnectionPoint;
        /**
         * Gets the iridescence object output component
         */
        get iridescence(): NodeMaterialConnectionPoint;
        autoConfigure(): void;
        prepareDefines(mesh: AbstractMesh, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines): void;
        /**
         * Gets the main code of the block (fragment side)
         * @param iridescenceBlock instance of a IridescenceBlock or null if the code must be generated without an active iridescence module
         * @returns the shader code
         */
        static GetCode(iridescenceBlock: Nullable<IridescenceBlock>): string;
        protected _buildBlock(state: NodeMaterialBuildState): this;
        serialize(): any;
        _deserialize(serializationObject: any, scene: Scene, rootUrl: string): void;
    }


    /**
     * Block used to implement the PBR metallic/roughness model
     */
    export class PBRMetallicRoughnessBlock extends NodeMaterialBlock {
        /**
         * Gets or sets the light associated with this block
         */
        light: Nullable<Light>;
        private static _OnGenerateOnlyFragmentCodeChanged;
        private _setTarget;
        private _lightId;
        private _scene;
        private _environmentBRDFTexture;
        private _environmentBrdfSamplerName;
        private _vNormalWName;
        private _invertNormalName;
        private _metallicReflectanceColor;
        private _metallicF0Factor;
        private _vMetallicReflectanceFactorsName;
        /**
         * Create a new ReflectionBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Intensity of the direct lights e.g. the four lights available in your scene.
         * This impacts both the direct diffuse and specular highlights.
         */
        directIntensity: number;
        /**
         * Intensity of the environment e.g. how much the environment will light the object
         * either through harmonics for rough material or through the reflection for shiny ones.
         */
        environmentIntensity: number;
        /**
         * This is a special control allowing the reduction of the specular highlights coming from the
         * four lights of the scene. Those highlights may not be needed in full environment lighting.
         */
        specularIntensity: number;
        /**
         * Defines the  falloff type used in this material.
         * It by default is Physical.
         */
        lightFalloff: number;
        /**
         * Specifies that alpha test should be used
         */
        useAlphaTest: boolean;
        /**
         * Defines the alpha limits in alpha test mode.
         */
        alphaTestCutoff: number;
        /**
         * Specifies that alpha blending should be used
         */
        useAlphaBlending: boolean;
        /**
         * Specifies that the material will keeps the reflection highlights over a transparent surface (only the most luminous ones).
         * A car glass is a good example of that. When the street lights reflects on it you can not see what is behind.
         */
        useRadianceOverAlpha: boolean;
        /**
         * Specifies that the material will keeps the specular highlights over a transparent surface (only the most luminous ones).
         * A car glass is a good example of that. When sun reflects on it you can not see what is behind.
         */
        useSpecularOverAlpha: boolean;
        /**
         * Enables specular anti aliasing in the PBR shader.
         * It will both interacts on the Geometry for analytical and IBL lighting.
         * It also prefilter the roughness map based on the bump values.
         */
        enableSpecularAntiAliasing: boolean;
        /**
         * Enables realtime filtering on the texture.
         */
        realTimeFiltering: boolean;
        /**
         * Quality switch for realtime filtering
         */
        realTimeFilteringQuality: number;
        /**
         * Defines if the material uses energy conservation.
         */
        useEnergyConservation: boolean;
        /**
         * This parameters will enable/disable radiance occlusion by preventing the radiance to lit
         * too much the area relying on ambient texture to define their ambient occlusion.
         */
        useRadianceOcclusion: boolean;
        /**
         * This parameters will enable/disable Horizon occlusion to prevent normal maps to look shiny when the normal
         * makes the reflect vector face the model (under horizon).
         */
        useHorizonOcclusion: boolean;
        /**
         * If set to true, no lighting calculations will be applied.
         */
        unlit: boolean;
        /**
         * Force normal to face away from face.
         */
        forceNormalForward: boolean;
        /** Indicates that no code should be generated in the vertex shader. Can be useful in some specific circumstances (like when doing ray marching for eg) */
        generateOnlyFragmentCode: boolean;
        /**
         * Defines the material debug mode.
         * It helps seeing only some components of the material while troubleshooting.
         */
        debugMode: number;
        /**
         * Specify from where on screen the debug mode should start.
         * The value goes from -1 (full screen) to 1 (not visible)
         * It helps with side by side comparison against the final render
         * This defaults to 0
         */
        debugLimit: number;
        /**
         * As the default viewing range might not be enough (if the ambient is really small for instance)
         * You can use the factor to better multiply the final value.
         */
        debugFactor: number;
        /**
         * Initialize the block and prepare the context for build
         * @param state defines the state that will be used for the build
         */
        initialize(state: NodeMaterialBuildState): void;
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the world position input component
         */
        get worldPosition(): NodeMaterialConnectionPoint;
        /**
         * Gets the world normal input component
         */
        get worldNormal(): NodeMaterialConnectionPoint;
        /**
         * Gets the view matrix parameter
         */
        get view(): NodeMaterialConnectionPoint;
        /**
         * Gets the camera position input component
         */
        get cameraPosition(): NodeMaterialConnectionPoint;
        /**
         * Gets the perturbed normal input component
         */
        get perturbedNormal(): NodeMaterialConnectionPoint;
        /**
         * Gets the base color input component
         */
        get baseColor(): NodeMaterialConnectionPoint;
        /**
         * Gets the metallic input component
         */
        get metallic(): NodeMaterialConnectionPoint;
        /**
         * Gets the roughness input component
         */
        get roughness(): NodeMaterialConnectionPoint;
        /**
         * Gets the ambient occlusion input component
         */
        get ambientOcc(): NodeMaterialConnectionPoint;
        /**
         * Gets the opacity input component
         */
        get opacity(): NodeMaterialConnectionPoint;
        /**
         * Gets the index of refraction input component
         */
        get indexOfRefraction(): NodeMaterialConnectionPoint;
        /**
         * Gets the ambient color input component
         */
        get ambientColor(): NodeMaterialConnectionPoint;
        /**
         * Gets the reflection object parameters
         */
        get reflection(): NodeMaterialConnectionPoint;
        /**
         * Gets the clear coat object parameters
         */
        get clearcoat(): NodeMaterialConnectionPoint;
        /**
         * Gets the sheen object parameters
         */
        get sheen(): NodeMaterialConnectionPoint;
        /**
         * Gets the sub surface object parameters
         */
        get subsurface(): NodeMaterialConnectionPoint;
        /**
         * Gets the anisotropy object parameters
         */
        get anisotropy(): NodeMaterialConnectionPoint;
        /**
         * Gets the iridescence object parameters
         */
        get iridescence(): NodeMaterialConnectionPoint;
        /**
         * Gets the ambient output component
         */
        get ambientClr(): NodeMaterialConnectionPoint;
        /**
         * Gets the diffuse output component
         */
        get diffuseDir(): NodeMaterialConnectionPoint;
        /**
         * Gets the specular output component
         */
        get specularDir(): NodeMaterialConnectionPoint;
        /**
         * Gets the clear coat output component
         */
        get clearcoatDir(): NodeMaterialConnectionPoint;
        /**
         * Gets the sheen output component
         */
        get sheenDir(): NodeMaterialConnectionPoint;
        /**
         * Gets the indirect diffuse output component
         */
        get diffuseInd(): NodeMaterialConnectionPoint;
        /**
         * Gets the indirect specular output component
         */
        get specularInd(): NodeMaterialConnectionPoint;
        /**
         * Gets the indirect clear coat output component
         */
        get clearcoatInd(): NodeMaterialConnectionPoint;
        /**
         * Gets the indirect sheen output component
         */
        get sheenInd(): NodeMaterialConnectionPoint;
        /**
         * Gets the refraction output component
         */
        get refraction(): NodeMaterialConnectionPoint;
        /**
         * Gets the global lighting output component
         */
        get lighting(): NodeMaterialConnectionPoint;
        /**
         * Gets the shadow output component
         */
        get shadow(): NodeMaterialConnectionPoint;
        /**
         * Gets the alpha output component
         */
        get alpha(): NodeMaterialConnectionPoint;
        autoConfigure(material: NodeMaterial): void;
        prepareDefines(mesh: AbstractMesh, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines): void;
        updateUniformsAndSamples(state: NodeMaterialBuildState, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines, uniformBuffers: string[]): void;
        isReady(mesh: AbstractMesh, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines): boolean;
        bind(effect: Effect, nodeMaterial: NodeMaterial, mesh?: Mesh): void;
        private _injectVertexCode;
        private _getAlbedoOpacityCode;
        private _getAmbientOcclusionCode;
        private _getReflectivityCode;
        protected _buildBlock(state: NodeMaterialBuildState): this;
        protected _dumpPropertiesCode(): string;
        serialize(): any;
        _deserialize(serializationObject: any, scene: Scene, rootUrl: string): void;
    }


    /**
     * Block used to implement the reflection module of the PBR material
     */
    export class ReflectionBlock extends ReflectionTextureBaseBlock {
        /** @internal */
        _defineLODReflectionAlpha: string;
        /** @internal */
        _defineLinearSpecularReflection: string;
        private _vEnvironmentIrradianceName;
        /** @internal */
        _vReflectionMicrosurfaceInfosName: string;
        /** @internal */
        _vReflectionInfosName: string;
        /** @internal */
        _vReflectionFilteringInfoName: string;
        private _scene;
        /**
         * The properties below are set by the main PBR block prior to calling methods of this class.
         * This is to avoid having to add them as inputs here whereas they are already inputs of the main block, so already known.
         * It's less burden on the user side in the editor part.
         */
        /** @internal */
        worldPositionConnectionPoint: NodeMaterialConnectionPoint;
        /** @internal */
        worldNormalConnectionPoint: NodeMaterialConnectionPoint;
        /** @internal */
        cameraPositionConnectionPoint: NodeMaterialConnectionPoint;
        /** @internal */
        viewConnectionPoint: NodeMaterialConnectionPoint;
        /**
         * Defines if the material uses spherical harmonics vs spherical polynomials for the
         * diffuse part of the IBL.
         */
        useSphericalHarmonics: boolean;
        /**
         * Force the shader to compute irradiance in the fragment shader in order to take bump in account.
         */
        forceIrradianceInFragment: boolean;
        protected _onGenerateOnlyFragmentCodeChanged(): boolean;
        protected _setTarget(): void;
        /**
         * Create a new ReflectionBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the position input component
         */
        get position(): NodeMaterialConnectionPoint;
        /**
         * Gets the world position input component
         */
        get worldPosition(): NodeMaterialConnectionPoint;
        /**
         * Gets the world normal input component
         */
        get worldNormal(): NodeMaterialConnectionPoint;
        /**
         * Gets the world input component
         */
        get world(): NodeMaterialConnectionPoint;
        /**
         * Gets the camera (or eye) position component
         */
        get cameraPosition(): NodeMaterialConnectionPoint;
        /**
         * Gets the view input component
         */
        get view(): NodeMaterialConnectionPoint;
        /**
         * Gets the color input component
         */
        get color(): NodeMaterialConnectionPoint;
        /**
         * Gets the reflection object output component
         */
        get reflection(): NodeMaterialConnectionPoint;
        /**
         * Returns true if the block has a texture (either its own texture or the environment texture from the scene, if set)
         */
        get hasTexture(): boolean;
        /**
         * Gets the reflection color (either the name of the variable if the color input is connected, else a default value)
         */
        get reflectionColor(): string;
        protected _getTexture(): Nullable<BaseTexture>;
        prepareDefines(mesh: AbstractMesh, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines): void;
        bind(effect: Effect, nodeMaterial: NodeMaterial, mesh?: Mesh, subMesh?: SubMesh): void;
        /**
         * Gets the code to inject in the vertex shader
         * @param state current state of the node material building
         * @returns the shader code
         */
        handleVertexSide(state: NodeMaterialBuildState): string;
        /**
         * Gets the main code of the block (fragment side)
         * @param state current state of the node material building
         * @param normalVarName name of the existing variable corresponding to the normal
         * @returns the shader code
         */
        getCode(state: NodeMaterialBuildState, normalVarName: string): string;
        protected _buildBlock(state: NodeMaterialBuildState): this;
        protected _dumpPropertiesCode(): string;
        serialize(): any;
        _deserialize(serializationObject: any, scene: Scene, rootUrl: string): void;
    }


    /**
     * Block used to implement the refraction part of the sub surface module of the PBR material
     */
    export class RefractionBlock extends NodeMaterialBlock {
        /** @internal */
        _define3DName: string;
        /** @internal */
        _refractionMatrixName: string;
        /** @internal */
        _defineLODRefractionAlpha: string;
        /** @internal */
        _defineLinearSpecularRefraction: string;
        /** @internal */
        _defineOppositeZ: string;
        /** @internal */
        _cubeSamplerName: string;
        /** @internal */
        _2DSamplerName: string;
        /** @internal */
        _vRefractionMicrosurfaceInfosName: string;
        /** @internal */
        _vRefractionInfosName: string;
        /** @internal */
        _vRefractionFilteringInfoName: string;
        private _scene;
        /**
         * The properties below are set by the main PBR block prior to calling methods of this class.
         * This is to avoid having to add them as inputs here whereas they are already inputs of the main block, so already known.
         * It's less burden on the user side in the editor part.
         */
        /** @internal */
        viewConnectionPoint: NodeMaterialConnectionPoint;
        /** @internal */
        indexOfRefractionConnectionPoint: NodeMaterialConnectionPoint;
        /**
         * This parameters will make the material used its opacity to control how much it is refracting against not.
         * Materials half opaque for instance using refraction could benefit from this control.
         */
        linkRefractionWithTransparency: boolean;
        /**
         * Controls if refraction needs to be inverted on Y. This could be useful for procedural texture.
         */
        invertRefractionY: boolean;
        /**
         * Controls if refraction needs to be inverted on Y. This could be useful for procedural texture.
         */
        useThicknessAsDepth: boolean;
        /**
         * Gets or sets the texture associated with the node
         */
        texture: Nullable<BaseTexture>;
        /**
         * Create a new RefractionBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Initialize the block and prepare the context for build
         * @param state defines the state that will be used for the build
         */
        initialize(state: NodeMaterialBuildState): void;
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the intensity input component
         */
        get intensity(): NodeMaterialConnectionPoint;
        /**
         * Gets the tint at distance input component
         */
        get tintAtDistance(): NodeMaterialConnectionPoint;
        /**
         * Gets the volume index of refraction input component
         */
        get volumeIndexOfRefraction(): NodeMaterialConnectionPoint;
        /**
         * Gets the view input component
         */
        get view(): NodeMaterialConnectionPoint;
        /**
         * Gets the refraction object output component
         */
        get refraction(): NodeMaterialConnectionPoint;
        /**
         * Returns true if the block has a texture
         */
        get hasTexture(): boolean;
        protected _getTexture(): Nullable<BaseTexture>;
        autoConfigure(material: NodeMaterial): void;
        prepareDefines(mesh: AbstractMesh, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines): void;
        isReady(): boolean;
        bind(effect: Effect, nodeMaterial: NodeMaterial, mesh?: Mesh): void;
        /**
         * Gets the main code of the block (fragment side)
         * @param state current state of the node material building
         * @returns the shader code
         */
        getCode(state: NodeMaterialBuildState): string;
        protected _buildBlock(state: NodeMaterialBuildState): this;
        protected _dumpPropertiesCode(): string;
        serialize(): any;
        _deserialize(serializationObject: any, scene: Scene, rootUrl: string): void;
    }


    /**
     * Block used to implement the sheen module of the PBR material
     */
    export class SheenBlock extends NodeMaterialBlock {
        /**
         * Create a new SheenBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * If true, the sheen effect is layered above the base BRDF with the albedo-scaling technique.
         * It allows the strength of the sheen effect to not depend on the base color of the material,
         * making it easier to setup and tweak the effect
         */
        albedoScaling: boolean;
        /**
         * Defines if the sheen is linked to the sheen color.
         */
        linkSheenWithAlbedo: boolean;
        /**
         * Initialize the block and prepare the context for build
         * @param state defines the state that will be used for the build
         */
        initialize(state: NodeMaterialBuildState): void;
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the intensity input component
         */
        get intensity(): NodeMaterialConnectionPoint;
        /**
         * Gets the color input component
         */
        get color(): NodeMaterialConnectionPoint;
        /**
         * Gets the roughness input component
         */
        get roughness(): NodeMaterialConnectionPoint;
        /**
         * Gets the sheen object output component
         */
        get sheen(): NodeMaterialConnectionPoint;
        prepareDefines(mesh: AbstractMesh, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines): void;
        /**
         * Gets the main code of the block (fragment side)
         * @param reflectionBlock instance of a ReflectionBlock null if the code must be generated without an active reflection module
         * @returns the shader code
         */
        getCode(reflectionBlock: Nullable<ReflectionBlock>): string;
        protected _buildBlock(state: NodeMaterialBuildState): this;
        protected _dumpPropertiesCode(): string;
        serialize(): any;
        _deserialize(serializationObject: any, scene: Scene, rootUrl: string): void;
    }


    /**
     * Block used to implement the sub surface module of the PBR material
     */
    export class SubSurfaceBlock extends NodeMaterialBlock {
        /**
         * Create a new SubSurfaceBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Initialize the block and prepare the context for build
         * @param state defines the state that will be used for the build
         */
        initialize(state: NodeMaterialBuildState): void;
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the thickness component
         */
        get thickness(): NodeMaterialConnectionPoint;
        /**
         * Gets the tint color input component
         */
        get tintColor(): NodeMaterialConnectionPoint;
        /**
         * Gets the translucency intensity input component
         */
        get translucencyIntensity(): NodeMaterialConnectionPoint;
        /**
         * Gets the translucency diffusion distance input component
         */
        get translucencyDiffusionDist(): NodeMaterialConnectionPoint;
        /**
         * Gets the refraction object parameters
         */
        get refraction(): NodeMaterialConnectionPoint;
        /**
         * Gets the sub surface object output component
         */
        get subsurface(): NodeMaterialConnectionPoint;
        autoConfigure(): void;
        prepareDefines(mesh: AbstractMesh, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines): void;
        /**
         * Gets the main code of the block (fragment side)
         * @param state current state of the node material building
         * @param ssBlock instance of a SubSurfaceBlock or null if the code must be generated without an active sub surface module
         * @param reflectionBlock instance of a ReflectionBlock null if the code must be generated without an active reflection module
         * @param worldPosVarName name of the variable holding the world position
         * @returns the shader code
         */
        static GetCode(state: NodeMaterialBuildState, ssBlock: Nullable<SubSurfaceBlock>, reflectionBlock: Nullable<ReflectionBlock>, worldPosVarName: string): string;
        protected _buildBlock(state: NodeMaterialBuildState): this;
    }


    /**
     * Block used to posterize a value
     * @see https://en.wikipedia.org/wiki/Posterization
     */
    export class PosterizeBlock extends NodeMaterialBlock {
        /**
         * Creates a new PosterizeBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the value input component
         */
        get value(): NodeMaterialConnectionPoint;
        /**
         * Gets the steps input component
         */
        get steps(): NodeMaterialConnectionPoint;
        /**
         * Gets the output component
         */
        get output(): NodeMaterialConnectionPoint;
        protected _buildBlock(state: NodeMaterialBuildState): this;
    }


    /**
     * Block used to get the value of the first parameter raised to the power of the second
     */
    export class PowBlock extends NodeMaterialBlock {
        /**
         * Creates a new PowBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the value operand input component
         */
        get value(): NodeMaterialConnectionPoint;
        /**
         * Gets the power operand input component
         */
        get power(): NodeMaterialConnectionPoint;
        /**
         * Gets the output component
         */
        get output(): NodeMaterialConnectionPoint;
        protected _buildBlock(state: NodeMaterialBuildState): this;
    }


    /**
     * Block used to get a random number
     */
    export class RandomNumberBlock extends NodeMaterialBlock {
        /**
         * Creates a new RandomNumberBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the seed input component
         */
        get seed(): NodeMaterialConnectionPoint;
        /**
         * Gets the output component
         */
        get output(): NodeMaterialConnectionPoint;
        protected _buildBlock(state: NodeMaterialBuildState): this;
    }


    /**
     * Block used to get the reciprocal (1 / x) of a value
     */
    export class ReciprocalBlock extends NodeMaterialBlock {
        /**
         * Creates a new ReciprocalBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the input component
         */
        get input(): NodeMaterialConnectionPoint;
        /**
         * Gets the output component
         */
        get output(): NodeMaterialConnectionPoint;
        protected _buildBlock(state: NodeMaterialBuildState): this;
    }


    /**
     * Block used to get the reflected vector from a direction and a normal
     */
    export class ReflectBlock extends NodeMaterialBlock {
        /**
         * Creates a new ReflectBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the incident component
         */
        get incident(): NodeMaterialConnectionPoint;
        /**
         * Gets the normal component
         */
        get normal(): NodeMaterialConnectionPoint;
        /**
         * Gets the output component
         */
        get output(): NodeMaterialConnectionPoint;
        protected _buildBlock(state: NodeMaterialBuildState): this;
    }


    /**
     * Block used to get the refracted vector from a direction and a normal
     */
    export class RefractBlock extends NodeMaterialBlock {
        /**
         * Creates a new RefractBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the incident component
         */
        get incident(): NodeMaterialConnectionPoint;
        /**
         * Gets the normal component
         */
        get normal(): NodeMaterialConnectionPoint;
        /**
         * Gets the index of refraction component
         */
        get ior(): NodeMaterialConnectionPoint;
        /**
         * Gets the output component
         */
        get output(): NodeMaterialConnectionPoint;
        protected _buildBlock(state: NodeMaterialBuildState): this;
    }


    /**
     * Block used to remap a float from a range to a new one
     */
    export class RemapBlock extends NodeMaterialBlock {
        /**
         * Gets or sets the source range
         */
        sourceRange: Vector2;
        /**
         * Gets or sets the target range
         */
        targetRange: Vector2;
        /**
         * Creates a new RemapBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the input component
         */
        get input(): NodeMaterialConnectionPoint;
        /**
         * Gets the source min input component
         */
        get sourceMin(): NodeMaterialConnectionPoint;
        /**
         * Gets the source max input component
         */
        get sourceMax(): NodeMaterialConnectionPoint;
        /**
         * Gets the target min input component
         */
        get targetMin(): NodeMaterialConnectionPoint;
        /**
         * Gets the target max input component
         */
        get targetMax(): NodeMaterialConnectionPoint;
        /**
         * Gets the output component
         */
        get output(): NodeMaterialConnectionPoint;
        protected _buildBlock(state: NodeMaterialBuildState): this;
        protected _dumpPropertiesCode(): string;
        serialize(): any;
        _deserialize(serializationObject: any, scene: Scene, rootUrl: string): void;
    }


    /**
     * Block used to replace a color by another one
     */
    export class ReplaceColorBlock extends NodeMaterialBlock {
        /**
         * Creates a new ReplaceColorBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the value input component
         */
        get value(): NodeMaterialConnectionPoint;
        /**
         * Gets the reference input component
         */
        get reference(): NodeMaterialConnectionPoint;
        /**
         * Gets the distance input component
         */
        get distance(): NodeMaterialConnectionPoint;
        /**
         * Gets the replacement input component
         */
        get replacement(): NodeMaterialConnectionPoint;
        /**
         * Gets the output component
         */
        get output(): NodeMaterialConnectionPoint;
        protected _buildBlock(state: NodeMaterialBuildState): this;
    }


    /**
     * Block used to rotate a 2d vector by a given angle
     */
    export class Rotate2dBlock extends NodeMaterialBlock {
        /**
         * Creates a new Rotate2dBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the input vector
         */
        get input(): NodeMaterialConnectionPoint;
        /**
         * Gets the input angle
         */
        get angle(): NodeMaterialConnectionPoint;
        /**
         * Gets the output component
         */
        get output(): NodeMaterialConnectionPoint;
        autoConfigure(): void;
        protected _buildBlock(state: NodeMaterialBuildState): this;
    }


    /**
     * Block used to scale a vector by a float
     */
    export class ScaleBlock extends NodeMaterialBlock {
        /**
         * Creates a new ScaleBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the input component
         */
        get input(): NodeMaterialConnectionPoint;
        /**
         * Gets the factor input component
         */
        get factor(): NodeMaterialConnectionPoint;
        /**
         * Gets the output component
         */
        get output(): NodeMaterialConnectionPoint;
        protected _buildBlock(state: NodeMaterialBuildState): this;
    }


    /**
     * block used to Generate a Simplex Perlin 3d Noise Pattern
     */
    export class SimplexPerlin3DBlock extends NodeMaterialBlock {
        /**
         * Creates a new SimplexPerlin3DBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the seed operand input component
         */
        get seed(): NodeMaterialConnectionPoint;
        /**
         * Gets the output component
         */
        get output(): NodeMaterialConnectionPoint;
        protected _buildBlock(state: NodeMaterialBuildState): this | undefined;
    }


    /**
     * Block used to smooth step a value
     */
    export class SmoothStepBlock extends NodeMaterialBlock {
        /**
         * Creates a new SmoothStepBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the value operand input component
         */
        get value(): NodeMaterialConnectionPoint;
        /**
         * Gets the first edge operand input component
         */
        get edge0(): NodeMaterialConnectionPoint;
        /**
         * Gets the second edge operand input component
         */
        get edge1(): NodeMaterialConnectionPoint;
        /**
         * Gets the output component
         */
        get output(): NodeMaterialConnectionPoint;
        protected _buildBlock(state: NodeMaterialBuildState): this;
    }


    /**
     * Block used to step a value
     */
    export class StepBlock extends NodeMaterialBlock {
        /**
         * Creates a new StepBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the value operand input component
         */
        get value(): NodeMaterialConnectionPoint;
        /**
         * Gets the edge operand input component
         */
        get edge(): NodeMaterialConnectionPoint;
        /**
         * Gets the output component
         */
        get output(): NodeMaterialConnectionPoint;
        protected _buildBlock(state: NodeMaterialBuildState): this;
    }


    /**
     * Block used to subtract 2 vectors
     */
    export class SubtractBlock extends NodeMaterialBlock {
        /**
         * Creates a new SubtractBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the left operand input component
         */
        get left(): NodeMaterialConnectionPoint;
        /**
         * Gets the right operand input component
         */
        get right(): NodeMaterialConnectionPoint;
        /**
         * Gets the output component
         */
        get output(): NodeMaterialConnectionPoint;
        protected _buildBlock(state: NodeMaterialBuildState): this;
    }


    /**
     * Block used to transform a vector (2, 3 or 4) with a matrix. It will generate a Vector4
     */
    export class TransformBlock extends NodeMaterialBlock {
        /**
         * Defines the value to use to complement W value to transform it to a Vector4
         */
        complementW: number;
        /**
         * Defines the value to use to complement z value to transform it to a Vector4
         */
        complementZ: number;
        /**
         * Creates a new TransformBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the vector input
         */
        get vector(): NodeMaterialConnectionPoint;
        /**
         * Gets the output component
         */
        get output(): NodeMaterialConnectionPoint;
        /**
         * Gets the xyz output component
         */
        get xyz(): NodeMaterialConnectionPoint;
        /**
         * Gets the matrix transform input
         */
        get transform(): NodeMaterialConnectionPoint;
        protected _buildBlock(state: NodeMaterialBuildState): this;
        /**
         * Update defines for shader compilation
         * @param mesh defines the mesh to be rendered
         * @param nodeMaterial defines the node material requesting the update
         * @param defines defines the material defines to update
         */
        prepareDefines(mesh: AbstractMesh, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines): void;
        serialize(): any;
        _deserialize(serializationObject: any, scene: Scene, rootUrl: string): void;
        protected _dumpPropertiesCode(): string;
    }


    /**
     * Operations supported by the Trigonometry block
     */
    export enum TrigonometryBlockOperations {
        /** Cos */
        Cos = 0,
        /** Sin */
        Sin = 1,
        /** Abs */
        Abs = 2,
        /** Exp */
        Exp = 3,
        /** Exp2 */
        Exp2 = 4,
        /** Round */
        Round = 5,
        /** Floor */
        Floor = 6,
        /** Ceiling */
        Ceiling = 7,
        /** Square root */
        Sqrt = 8,
        /** Log */
        Log = 9,
        /** Tangent */
        Tan = 10,
        /** Arc tangent */
        ArcTan = 11,
        /** Arc cosinus */
        ArcCos = 12,
        /** Arc sinus */
        ArcSin = 13,
        /** Fraction */
        Fract = 14,
        /** Sign */
        Sign = 15,
        /** To radians (from degrees) */
        Radians = 16,
        /** To degrees (from radians) */
        Degrees = 17
    }
    /**
     * Block used to apply trigonometry operation to floats
     */
    export class TrigonometryBlock extends NodeMaterialBlock {
        /**
         * Gets or sets the operation applied by the block
         */
        operation: TrigonometryBlockOperations;
        /**
         * Creates a new TrigonometryBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the input component
         */
        get input(): NodeMaterialConnectionPoint;
        /**
         * Gets the output component
         */
        get output(): NodeMaterialConnectionPoint;
        protected _buildBlock(state: NodeMaterialBuildState): this;
        serialize(): any;
        _deserialize(serializationObject: any, scene: Scene, rootUrl: string): void;
        protected _dumpPropertiesCode(): string;
    }


    /**
     * Block used to read a texture with triplanar mapping (see "boxmap" in https://iquilezles.org/articles/biplanar/)
     */
    export class TriPlanarBlock extends NodeMaterialBlock {
        private _linearDefineName;
        private _gammaDefineName;
        protected _tempTextureRead: string;
        private _samplerName;
        private _textureInfoName;
        private _imageSource;
        protected _texture: Nullable<Texture>;
        /**
         * Gets or sets the texture associated with the node
         */
        get texture(): Nullable<Texture>;
        set texture(texture: Nullable<Texture>);
        /**
         * Gets the textureY associated with the node
         */
        get textureY(): Nullable<Texture>;
        /**
         * Gets the textureZ associated with the node
         */
        get textureZ(): Nullable<Texture>;
        protected _getImageSourceBlock(connectionPoint: Nullable<NodeMaterialConnectionPoint>): Nullable<ImageSourceBlock>;
        /**
         * Gets the sampler name associated with this texture
         */
        get samplerName(): string;
        /**
         * Gets the samplerY name associated with this texture
         */
        get samplerYName(): Nullable<string>;
        /**
         * Gets the samplerZ name associated with this texture
         */
        get samplerZName(): Nullable<string>;
        /**
         * Gets a boolean indicating that this block is linked to an ImageSourceBlock
         */
        get hasImageSource(): boolean;
        private _convertToGammaSpace;
        /**
         * Gets or sets a boolean indicating if content needs to be converted to gamma space
         */
        set convertToGammaSpace(value: boolean);
        get convertToGammaSpace(): boolean;
        private _convertToLinearSpace;
        /**
         * Gets or sets a boolean indicating if content needs to be converted to linear space
         */
        set convertToLinearSpace(value: boolean);
        get convertToLinearSpace(): boolean;
        /**
         * Gets or sets a boolean indicating if multiplication of texture with level should be disabled
         */
        disableLevelMultiplication: boolean;
        /**
         * Create a new TriPlanarBlock
         * @param name defines the block name
         */
        constructor(name: string, hideSourceZ?: boolean);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the position input component
         */
        get position(): NodeMaterialConnectionPoint;
        /**
         * Gets the normal input component
         */
        get normal(): NodeMaterialConnectionPoint;
        /**
         * Gets the sharpness input component
         */
        get sharpness(): NodeMaterialConnectionPoint;
        /**
         * Gets the source input component
         */
        get source(): NodeMaterialConnectionPoint;
        /**
         * Gets the sourceY input component
         */
        get sourceY(): NodeMaterialConnectionPoint;
        /**
         * Gets the sourceZ input component
         */
        get sourceZ(): Nullable<NodeMaterialConnectionPoint>;
        /**
         * Gets the rgba output component
         */
        get rgba(): NodeMaterialConnectionPoint;
        /**
         * Gets the rgb output component
         */
        get rgb(): NodeMaterialConnectionPoint;
        /**
         * Gets the r output component
         */
        get r(): NodeMaterialConnectionPoint;
        /**
         * Gets the g output component
         */
        get g(): NodeMaterialConnectionPoint;
        /**
         * Gets the b output component
         */
        get b(): NodeMaterialConnectionPoint;
        /**
         * Gets the a output component
         */
        get a(): NodeMaterialConnectionPoint;
        /**
         * Gets the level output component
         */
        get level(): NodeMaterialConnectionPoint;
        prepareDefines(mesh: AbstractMesh, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines): void;
        isReady(): boolean;
        bind(effect: Effect): void;
        protected _generateTextureLookup(state: NodeMaterialBuildState): void;
        private _generateConversionCode;
        private _writeOutput;
        protected _buildBlock(state: NodeMaterialBuildState): this;
        protected _dumpPropertiesCode(): string;
        serialize(): any;
        _deserialize(serializationObject: any, scene: Scene, rootUrl: string): void;
    }


    /**
     * Block used to create a Vector2/3/4 out of individual inputs (one for each component)
     */
    export class VectorMergerBlock extends NodeMaterialBlock {
        /**
         * Gets or sets the swizzle for x (meaning which component to affect to the output.x)
         */
        xSwizzle: "x" | "y" | "z" | "w";
        /**
         * Gets or sets the swizzle for y (meaning which component to affect to the output.y)
         */
        ySwizzle: "x" | "y" | "z" | "w";
        /**
         * Gets or sets the swizzle for z (meaning which component to affect to the output.z)
         */
        zSwizzle: "x" | "y" | "z" | "w";
        /**
         * Gets or sets the swizzle for w (meaning which component to affect to the output.w)
         */
        wSwizzle: "x" | "y" | "z" | "w";
        /**
         * Create a new VectorMergerBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the xyzw component (input)
         */
        get xyzwIn(): NodeMaterialConnectionPoint;
        /**
         * Gets the xyz component (input)
         */
        get xyzIn(): NodeMaterialConnectionPoint;
        /**
         * Gets the xy component (input)
         */
        get xyIn(): NodeMaterialConnectionPoint;
        /**
         * Gets the zw component (input)
         */
        get zwIn(): NodeMaterialConnectionPoint;
        /**
         * Gets the x component (input)
         */
        get x(): NodeMaterialConnectionPoint;
        /**
         * Gets the y component (input)
         */
        get y(): NodeMaterialConnectionPoint;
        /**
         * Gets the z component (input)
         */
        get z(): NodeMaterialConnectionPoint;
        /**
         * Gets the w component (input)
         */
        get w(): NodeMaterialConnectionPoint;
        /**
         * Gets the xyzw component (output)
         */
        get xyzw(): NodeMaterialConnectionPoint;
        /**
         * Gets the xyz component (output)
         */
        get xyzOut(): NodeMaterialConnectionPoint;
        /**
         * Gets the xy component (output)
         */
        get xyOut(): NodeMaterialConnectionPoint;
        /**
         * Gets the zw component (output)
         */
        get zwOut(): NodeMaterialConnectionPoint;
        /**
         * Gets the xy component (output)
         * @deprecated Please use xyOut instead.
         */
        get xy(): NodeMaterialConnectionPoint;
        /**
         * Gets the xyz component (output)
         * @deprecated Please use xyzOut instead.
         */
        get xyz(): NodeMaterialConnectionPoint;
        protected _inputRename(name: string): string;
        private _buildSwizzle;
        protected _buildBlock(state: NodeMaterialBuildState): this;
        serialize(): any;
        _deserialize(serializationObject: any, scene: Scene, rootUrl: string): void;
        protected _dumpPropertiesCode(): string;
    }


    /**
     * Block used to expand a Vector3/4 into 4 outputs (one for each component)
     */
    export class VectorSplitterBlock extends NodeMaterialBlock {
        /**
         * Create a new VectorSplitterBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the xyzw component (input)
         */
        get xyzw(): NodeMaterialConnectionPoint;
        /**
         * Gets the xyz component (input)
         */
        get xyzIn(): NodeMaterialConnectionPoint;
        /**
         * Gets the xy component (input)
         */
        get xyIn(): NodeMaterialConnectionPoint;
        /**
         * Gets the xyz component (output)
         */
        get xyzOut(): NodeMaterialConnectionPoint;
        /**
         * Gets the xy component (output)
         */
        get xyOut(): NodeMaterialConnectionPoint;
        /**
         * Gets the zw component (output)
         */
        get zw(): NodeMaterialConnectionPoint;
        /**
         * Gets the x component (output)
         */
        get x(): NodeMaterialConnectionPoint;
        /**
         * Gets the y component (output)
         */
        get y(): NodeMaterialConnectionPoint;
        /**
         * Gets the z component (output)
         */
        get z(): NodeMaterialConnectionPoint;
        /**
         * Gets the w component (output)
         */
        get w(): NodeMaterialConnectionPoint;
        protected _inputRename(name: string): string;
        protected _outputRename(name: string): string;
        protected _buildBlock(state: NodeMaterialBuildState): this;
    }


    /**
     * Block used to add support for vertex skinning (bones)
     */
    export class BonesBlock extends NodeMaterialBlock {
        /**
         * Creates a new BonesBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Initialize the block and prepare the context for build
         * @param state defines the state that will be used for the build
         */
        initialize(state: NodeMaterialBuildState): void;
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the matrix indices input component
         */
        get matricesIndices(): NodeMaterialConnectionPoint;
        /**
         * Gets the matrix weights input component
         */
        get matricesWeights(): NodeMaterialConnectionPoint;
        /**
         * Gets the extra matrix indices input component
         */
        get matricesIndicesExtra(): NodeMaterialConnectionPoint;
        /**
         * Gets the extra matrix weights input component
         */
        get matricesWeightsExtra(): NodeMaterialConnectionPoint;
        /**
         * Gets the world input component
         */
        get world(): NodeMaterialConnectionPoint;
        /**
         * Gets the output component
         */
        get output(): NodeMaterialConnectionPoint;
        autoConfigure(material: NodeMaterial): void;
        provideFallbacks(mesh: AbstractMesh, fallbacks: EffectFallbacks): void;
        bind(effect: Effect, nodeMaterial: NodeMaterial, mesh?: Mesh): void;
        prepareDefines(mesh: AbstractMesh, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines): void;
        protected _buildBlock(state: NodeMaterialBuildState): this;
    }




    /**
     * Block used to add support for instances
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/copies/instances
     */
    export class InstancesBlock extends NodeMaterialBlock {
        /**
         * Creates a new InstancesBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the first world row input component
         */
        get world0(): NodeMaterialConnectionPoint;
        /**
         * Gets the second world row input component
         */
        get world1(): NodeMaterialConnectionPoint;
        /**
         * Gets the third world row input component
         */
        get world2(): NodeMaterialConnectionPoint;
        /**
         * Gets the forth world row input component
         */
        get world3(): NodeMaterialConnectionPoint;
        /**
         * Gets the world input component
         */
        get world(): NodeMaterialConnectionPoint;
        /**
         * Gets the output component
         */
        get output(): NodeMaterialConnectionPoint;
        /**
         * Gets the instanceID component
         */
        get instanceID(): NodeMaterialConnectionPoint;
        autoConfigure(material: NodeMaterial): void;
        prepareDefines(mesh: AbstractMesh, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines, useInstances?: boolean, subMesh?: SubMesh): void;
        protected _buildBlock(state: NodeMaterialBuildState): this;
    }


    /**
     * Block used to get data information from a light
     */
    export class LightInformationBlock extends NodeMaterialBlock {
        private _lightDataUniformName;
        private _lightColorUniformName;
        private _lightShadowUniformName;
        private _lightShadowExtraUniformName;
        private _lightTypeDefineName;
        private _forcePrepareDefines;
        /**
         * Gets or sets the light associated with this block
         */
        light: Nullable<Light>;
        /**
         * Creates a new LightInformationBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the world position input component
         */
        get worldPosition(): NodeMaterialConnectionPoint;
        /**
         * Gets the direction output component
         */
        get direction(): NodeMaterialConnectionPoint;
        /**
         * Gets the direction output component
         */
        get color(): NodeMaterialConnectionPoint;
        /**
         * Gets the direction output component
         */
        get intensity(): NodeMaterialConnectionPoint;
        /**
         * Gets the shadow bias output component
         */
        get shadowBias(): NodeMaterialConnectionPoint;
        /**
         * Gets the shadow normal bias output component
         */
        get shadowNormalBias(): NodeMaterialConnectionPoint;
        /**
         * Gets the shadow depth scale component
         */
        get shadowDepthScale(): NodeMaterialConnectionPoint;
        /**
         * Gets the shadow depth range component
         */
        get shadowDepthRange(): NodeMaterialConnectionPoint;
        bind(effect: Effect, nodeMaterial: NodeMaterial, mesh?: Mesh): void;
        prepareDefines(mesh: AbstractMesh, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines): void;
        protected _buildBlock(state: NodeMaterialBuildState): this;
        serialize(): any;
        _deserialize(serializationObject: any, scene: Scene, rootUrl: string): void;
    }


    /**
     * Block used to add morph targets support to vertex shader
     */
    export class MorphTargetsBlock extends NodeMaterialBlock {
        private _repeatableContentAnchor;
        /**
         * Create a new MorphTargetsBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the position input component
         */
        get position(): NodeMaterialConnectionPoint;
        /**
         * Gets the normal input component
         */
        get normal(): NodeMaterialConnectionPoint;
        /**
         * Gets the tangent input component
         */
        get tangent(): NodeMaterialConnectionPoint;
        /**
         * Gets the tangent input component
         */
        get uv(): NodeMaterialConnectionPoint;
        /**
         * Gets the position output component
         */
        get positionOutput(): NodeMaterialConnectionPoint;
        /**
         * Gets the normal output component
         */
        get normalOutput(): NodeMaterialConnectionPoint;
        /**
         * Gets the tangent output component
         */
        get tangentOutput(): NodeMaterialConnectionPoint;
        /**
         * Gets the tangent output component
         */
        get uvOutput(): NodeMaterialConnectionPoint;
        initialize(state: NodeMaterialBuildState): void;
        autoConfigure(material: NodeMaterial): void;
        prepareDefines(mesh: AbstractMesh, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines): void;
        bind(effect: Effect, nodeMaterial: NodeMaterial, mesh?: Mesh): void;
        replaceRepeatableContent(vertexShaderState: NodeMaterialBuildState, fragmentShaderState: NodeMaterialBuildState, mesh: AbstractMesh, defines: NodeMaterialDefines): void;
        protected _buildBlock(state: NodeMaterialBuildState): this;
    }


    /**
     * Block used to output the vertex position
     */
    export class VertexOutputBlock extends NodeMaterialBlock {
        /**
         * Creates a new VertexOutputBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the vector input component
         */
        get vector(): NodeMaterialConnectionPoint;
        private _isLogarithmicDepthEnabled;
        protected _buildBlock(state: NodeMaterialBuildState): this;
    }


    /**
     * Block used to get the view direction
     */
    export class ViewDirectionBlock extends NodeMaterialBlock {
        /**
         * Creates a new ViewDirectionBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the world position component
         */
        get worldPosition(): NodeMaterialConnectionPoint;
        /**
         * Gets the camera position component
         */
        get cameraPosition(): NodeMaterialConnectionPoint;
        /**
         * Gets the output component
         */
        get output(): NodeMaterialConnectionPoint;
        autoConfigure(material: NodeMaterial): void;
        protected _buildBlock(state: NodeMaterialBuildState): this;
    }


    /**
     * block used to Generate a Voronoi Noise Pattern
     */
    export class VoronoiNoiseBlock extends NodeMaterialBlock {
        /**
         * Creates a new VoronoiNoiseBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the seed input component
         */
        get seed(): NodeMaterialConnectionPoint;
        /**
         * Gets the offset input component
         */
        get offset(): NodeMaterialConnectionPoint;
        /**
         * Gets the density input component
         */
        get density(): NodeMaterialConnectionPoint;
        /**
         * Gets the output component
         */
        get output(): NodeMaterialConnectionPoint;
        /**
         * Gets the output component
         */
        get cells(): NodeMaterialConnectionPoint;
        protected _buildBlock(state: NodeMaterialBuildState): this | undefined;
    }


    /**
     * Operations supported by the Wave block
     */
    export enum WaveBlockKind {
        /** SawTooth */
        SawTooth = 0,
        /** Square */
        Square = 1,
        /** Triangle */
        Triangle = 2
    }
    /**
     * Block used to apply wave operation to floats
     */
    export class WaveBlock extends NodeMaterialBlock {
        /**
         * Gets or sets the kibnd of wave to be applied by the block
         */
        kind: WaveBlockKind;
        /**
         * Creates a new WaveBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the input component
         */
        get input(): NodeMaterialConnectionPoint;
        /**
         * Gets the output component
         */
        get output(): NodeMaterialConnectionPoint;
        protected _buildBlock(state: NodeMaterialBuildState): this;
        serialize(): any;
        _deserialize(serializationObject: any, scene: Scene, rootUrl: string): void;
    }


    /**
     * block used to Generate a Worley Noise 3D Noise Pattern
     */
    export class WorleyNoise3DBlock extends NodeMaterialBlock {
        /** Gets or sets a boolean indicating that normal should be inverted on X axis */
        manhattanDistance: boolean;
        /**
         * Creates a new WorleyNoise3DBlock
         * @param name defines the block name
         */
        constructor(name: string);
        /**
         * Gets the current class name
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets the seed input component
         */
        get seed(): NodeMaterialConnectionPoint;
        /**
         * Gets the jitter input component
         */
        get jitter(): NodeMaterialConnectionPoint;
        /**
         * Gets the output component
         */
        get output(): NodeMaterialConnectionPoint;
        /**
         * Gets the x component
         */
        get x(): NodeMaterialConnectionPoint;
        /**
         * Gets the y component
         */
        get y(): NodeMaterialConnectionPoint;
        protected _buildBlock(state: NodeMaterialBuildState): this | undefined;
        /**
         * Exposes the properties to the UI?
         */
        protected _dumpPropertiesCode(): string;
        /**
         * Exposes the properties to the Serialize?
         */
        serialize(): any;
        /**
         * Exposes the properties to the deserialize?
         * @param serializationObject
         * @param scene
         * @param rootUrl
         */
        _deserialize(serializationObject: any, scene: Scene, rootUrl: string): void;
    }




    /**
     * Enum defining the mode of a NodeMaterialBlockConnectionPoint
     */
    export enum NodeMaterialBlockConnectionPointMode {
        /** Value is an uniform */
        Uniform = 0,
        /** Value is a mesh attribute */
        Attribute = 1,
        /** Value is a varying between vertex and fragment shaders */
        Varying = 2,
        /** Mode is undefined */
        Undefined = 3
    }


    /**
     * Defines the kind of connection point for node based material
     */
    export enum NodeMaterialBlockConnectionPointTypes {
        /** Float */
        Float = 1,
        /** Int */
        Int = 2,
        /** Vector2 */
        Vector2 = 4,
        /** Vector3 */
        Vector3 = 8,
        /** Vector4 */
        Vector4 = 16,
        /** Color3 */
        Color3 = 32,
        /** Color4 */
        Color4 = 64,
        /** Matrix */
        Matrix = 128,
        /** Custom object */
        Object = 256,
        /** Detect type based on connection */
        AutoDetect = 1024,
        /** Output type that will be defined by input type */
        BasedOnInput = 2048,
        /** Bitmask of all types */
        All = 4095
    }


    /**
     * Enum used to define the target of a block
     */
    export enum NodeMaterialBlockTargets {
        /** Vertex shader */
        Vertex = 1,
        /** Fragment shader */
        Fragment = 2,
        /** Neutral */
        Neutral = 4,
        /** Vertex and Fragment */
        VertexAndFragment = 3
    }


    /**
     * Enum used to define the material modes
     */
    export enum NodeMaterialModes {
        /** Regular material */
        Material = 0,
        /** For post process */
        PostProcess = 1,
        /** For particle system */
        Particle = 2,
        /** For procedural texture */
        ProceduralTexture = 3
    }


    /**
     * Enum used to define system values e.g. values automatically provided by the system
     */
    export enum NodeMaterialSystemValues {
        /** World */
        World = 1,
        /** View */
        View = 2,
        /** Projection */
        Projection = 3,
        /** ViewProjection */
        ViewProjection = 4,
        /** WorldView */
        WorldView = 5,
        /** WorldViewProjection */
        WorldViewProjection = 6,
        /** CameraPosition */
        CameraPosition = 7,
        /** Fog Color */
        FogColor = 8,
        /** Delta time */
        DeltaTime = 9,
        /** Camera parameters */
        CameraParameters = 10,
        /** Material alpha */
        MaterialAlpha = 11
    }




    /**
     * Interface used to configure the node material editor
     */
    export interface INodeMaterialEditorOptions {
        /** Define the URl to load node editor script */
        editorURL?: string;
    }
    /** @internal */
    export class NodeMaterialDefines extends MaterialDefines implements IImageProcessingConfigurationDefines {
        NORMAL: boolean;
        TANGENT: boolean;
        UV1: boolean;
        UV2: boolean;
        UV3: boolean;
        UV4: boolean;
        UV5: boolean;
        UV6: boolean;
        /** BONES */
        NUM_BONE_INFLUENCERS: number;
        BonesPerMesh: number;
        BONETEXTURE: boolean;
        /** MORPH TARGETS */
        MORPHTARGETS: boolean;
        MORPHTARGETS_NORMAL: boolean;
        MORPHTARGETS_TANGENT: boolean;
        MORPHTARGETS_UV: boolean;
        NUM_MORPH_INFLUENCERS: number;
        MORPHTARGETS_TEXTURE: boolean;
        /** IMAGE PROCESSING */
        IMAGEPROCESSING: boolean;
        VIGNETTE: boolean;
        VIGNETTEBLENDMODEMULTIPLY: boolean;
        VIGNETTEBLENDMODEOPAQUE: boolean;
        TONEMAPPING: boolean;
        TONEMAPPING_ACES: boolean;
        CONTRAST: boolean;
        EXPOSURE: boolean;
        COLORCURVES: boolean;
        COLORGRADING: boolean;
        COLORGRADING3D: boolean;
        SAMPLER3DGREENDEPTH: boolean;
        SAMPLER3DBGRMAP: boolean;
        DITHER: boolean;
        IMAGEPROCESSINGPOSTPROCESS: boolean;
        SKIPFINALCOLORCLAMP: boolean;
        /** MISC. */
        BUMPDIRECTUV: number;
        CAMERA_ORTHOGRAPHIC: boolean;
        CAMERA_PERSPECTIVE: boolean;
        constructor();
        setValue(name: string, value: any, markAsUnprocessedIfDirty?: boolean): void;
    }
    /**
     * Class used to configure NodeMaterial
     */
    export interface INodeMaterialOptions {
        /**
         * Defines if blocks should emit comments
         */
        emitComments: boolean;
    }
    /**
     * Blocks that manage a texture
     */
    export type NodeMaterialTextureBlocks = TextureBlock | ReflectionTextureBaseBlock | RefractionBlock | CurrentScreenBlock | ParticleTextureBlock | ImageSourceBlock | TriPlanarBlock | BiPlanarBlock;
    /**
     * Class used to create a node based material built by assembling shader blocks
     */
    export class NodeMaterial extends PushMaterial {
        private static _BuildIdGenerator;
        private _options;
        private _vertexCompilationState;
        private _fragmentCompilationState;
        private _sharedData;
        private _buildId;
        private _buildWasSuccessful;
        private _cachedWorldViewMatrix;
        private _cachedWorldViewProjectionMatrix;
        private _optimizers;
        private _animationFrame;
        /** Define the Url to load node editor script */
        static EditorURL: string;
        /** Define the Url to load snippets */
        static SnippetUrl: string;
        /** Gets or sets a boolean indicating that node materials should not deserialize textures from json / snippet content */
        static IgnoreTexturesAtLoadTime: boolean;
        /**
         * Checks if a block is a texture block
         * @param block The block to check
         * @returns True if the block is a texture block
         */
        static _BlockIsTextureBlock(block: NodeMaterialBlock): block is NodeMaterialTextureBlocks;
        private BJSNODEMATERIALEDITOR;
        /** Get the inspector from bundle or global */
        private _getGlobalNodeMaterialEditor;
        /**
         * Snippet ID if the material was created from the snippet server
         */
        snippetId: string;
        /**
         * Gets or sets data used by visual editor
         * @see https://nme.babylonjs.com
         */
        editorData: any;
        /**
         * Gets or sets a boolean indicating that alpha value must be ignored (This will turn alpha blending off even if an alpha value is produced by the material)
         */
        ignoreAlpha: boolean;
        /**
         * Defines the maximum number of lights that can be used in the material
         */
        maxSimultaneousLights: number;
        /**
         * Observable raised when the material is built
         */
        onBuildObservable: Observable<NodeMaterial>;
        /**
         * Gets or sets the root nodes of the material vertex shader
         */
        _vertexOutputNodes: NodeMaterialBlock[];
        /**
         * Gets or sets the root nodes of the material fragment (pixel) shader
         */
        _fragmentOutputNodes: NodeMaterialBlock[];
        /** Gets or sets options to control the node material overall behavior */
        get options(): INodeMaterialOptions;
        set options(options: INodeMaterialOptions);
        /**
         * Default configuration related to image processing available in the standard Material.
         */
        protected _imageProcessingConfiguration: ImageProcessingConfiguration;
        /**
         * Gets the image processing configuration used either in this material.
         */
        get imageProcessingConfiguration(): ImageProcessingConfiguration;
        /**
         * Sets the Default image processing configuration used either in the this material.
         *
         * If sets to null, the scene one is in use.
         */
        set imageProcessingConfiguration(value: ImageProcessingConfiguration);
        /**
         * Gets an array of blocks that needs to be serialized even if they are not yet connected
         */
        attachedBlocks: NodeMaterialBlock[];
        /**
         * Specifies the mode of the node material
         * @internal
         */
        _mode: NodeMaterialModes;
        /**
         * Gets or sets the mode property
         */
        get mode(): NodeMaterialModes;
        set mode(value: NodeMaterialModes);
        /** Gets or sets the unique identifier used to identified the effect associated with the material */
        get buildId(): number;
        set buildId(value: number);
        /**
         * A free comment about the material
         */
        comment: string;
        /**
         * Create a new node based material
         * @param name defines the material name
         * @param scene defines the hosting scene
         * @param options defines creation option
         */
        constructor(name: string, scene?: Scene, options?: Partial<INodeMaterialOptions>);
        /**
         * Gets the current class name of the material e.g. "NodeMaterial"
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Keep track of the image processing observer to allow dispose and replace.
         */
        private _imageProcessingObserver;
        /**
         * Attaches a new image processing configuration to the Standard Material.
         * @param configuration
         */
        protected _attachImageProcessingConfiguration(configuration: Nullable<ImageProcessingConfiguration>): void;
        /**
         * Get a block by its name
         * @param name defines the name of the block to retrieve
         * @returns the required block or null if not found
         */
        getBlockByName(name: string): NodeMaterialBlock | null;
        /**
         * Get a block by its name
         * @param predicate defines the predicate used to find the good candidate
         * @returns the required block or null if not found
         */
        getBlockByPredicate(predicate: (block: NodeMaterialBlock) => boolean): NodeMaterialBlock | null;
        /**
         * Get an input block by its name
         * @param predicate defines the predicate used to find the good candidate
         * @returns the required input block or null if not found
         */
        getInputBlockByPredicate(predicate: (block: InputBlock) => boolean): Nullable<InputBlock>;
        /**
         * Gets the list of input blocks attached to this material
         * @returns an array of InputBlocks
         */
        getInputBlocks(): InputBlock[];
        /**
         * Adds a new optimizer to the list of optimizers
         * @param optimizer defines the optimizers to add
         * @returns the current material
         */
        registerOptimizer(optimizer: NodeMaterialOptimizer): this | undefined;
        /**
         * Remove an optimizer from the list of optimizers
         * @param optimizer defines the optimizers to remove
         * @returns the current material
         */
        unregisterOptimizer(optimizer: NodeMaterialOptimizer): this | undefined;
        /**
         * Add a new block to the list of output nodes
         * @param node defines the node to add
         * @returns the current material
         */
        addOutputNode(node: NodeMaterialBlock): this;
        /**
         * Remove a block from the list of root nodes
         * @param node defines the node to remove
         * @returns the current material
         */
        removeOutputNode(node: NodeMaterialBlock): this;
        private _addVertexOutputNode;
        private _removeVertexOutputNode;
        private _addFragmentOutputNode;
        private _removeFragmentOutputNode;
        /**
         * Gets or sets a boolean indicating that alpha blending must be enabled no matter what alpha value or alpha channel of the FragmentBlock are
         */
        forceAlphaBlending: boolean;
        /**
         * Specifies if the material will require alpha blending
         * @returns a boolean specifying if alpha blending is needed
         */
        needAlphaBlending(): boolean;
        /**
         * Specifies if this material should be rendered in alpha test mode
         * @returns a boolean specifying if an alpha test is needed.
         */
        needAlphaTesting(): boolean;
        private _initializeBlock;
        private _resetDualBlocks;
        /**
         * Remove a block from the current node material
         * @param block defines the block to remove
         */
        removeBlock(block: NodeMaterialBlock): void;
        /**
         * Build the material and generates the inner effect
         * @param verbose defines if the build should log activity
         * @param updateBuildId defines if the internal build Id should be updated (default is true)
         * @param autoConfigure defines if the autoConfigure method should be called when initializing blocks (default is true)
         */
        build(verbose?: boolean, updateBuildId?: boolean, autoConfigure?: boolean): void;
        /**
         * Runs an otpimization phase to try to improve the shader code
         */
        optimize(): void;
        private _prepareDefinesForAttributes;
        /**
         * Create a post process from the material
         * @param camera The camera to apply the render pass to.
         * @param options The required width/height ratio to downsize to before computing the render pass. (Use 1.0 for full size)
         * @param samplingMode The sampling mode to be used when computing the pass. (default: 0)
         * @param engine The engine which the post process will be applied. (default: current engine)
         * @param reusable If the post process can be reused on the same frame. (default: false)
         * @param textureType Type of textures used when performing the post process. (default: 0)
         * @param textureFormat Format of textures used when performing the post process. (default: TEXTUREFORMAT_RGBA)
         * @returns the post process created
         */
        createPostProcess(camera: Nullable<Camera>, options?: number | PostProcessOptions, samplingMode?: number, engine?: Engine, reusable?: boolean, textureType?: number, textureFormat?: number): Nullable<PostProcess>;
        /**
         * Create the post process effect from the material
         * @param postProcess The post process to create the effect for
         */
        createEffectForPostProcess(postProcess: PostProcess): void;
        private _createEffectForPostProcess;
        /**
         * Create a new procedural texture based on this node material
         * @param size defines the size of the texture
         * @param scene defines the hosting scene
         * @returns the new procedural texture attached to this node material
         */
        createProceduralTexture(size: number | {
            width: number;
            height: number;
            layers?: number;
        }, scene: Scene): Nullable<ProceduralTexture>;
        private _createEffectForParticles;
        private _checkInternals;
        /**
         * Create the effect to be used as the custom effect for a particle system
         * @param particleSystem Particle system to create the effect for
         * @param onCompiled defines a function to call when the effect creation is successful
         * @param onError defines a function to call when the effect creation has failed
         */
        createEffectForParticles(particleSystem: IParticleSystem, onCompiled?: (effect: Effect) => void, onError?: (effect: Effect, errors: string) => void): void;
        /**
         * Use this material as the shadow depth wrapper of a target material
         * @param targetMaterial defines the target material
         */
        createAsShadowDepthWrapper(targetMaterial: Material): void;
        private _processDefines;
        /**
         * Get if the submesh is ready to be used and all its information available.
         * Child classes can use it to update shaders
         * @param mesh defines the mesh to check
         * @param subMesh defines which submesh to check
         * @param useInstances specifies that instances should be used
         * @returns a boolean indicating that the submesh is ready or not
         */
        isReadyForSubMesh(mesh: AbstractMesh, subMesh: SubMesh, useInstances?: boolean): boolean;
        /**
         * Get a string representing the shaders built by the current node graph
         */
        get compiledShaders(): string;
        /**
         * Binds the world matrix to the material
         * @param world defines the world transformation matrix
         */
        bindOnlyWorldMatrix(world: Matrix): void;
        /**
         * Binds the submesh to this material by preparing the effect and shader to draw
         * @param world defines the world transformation matrix
         * @param mesh defines the mesh containing the submesh
         * @param subMesh defines the submesh to bind the material to
         */
        bindForSubMesh(world: Matrix, mesh: Mesh, subMesh: SubMesh): void;
        /**
         * Gets the active textures from the material
         * @returns an array of textures
         */
        getActiveTextures(): BaseTexture[];
        /**
         * Gets the list of texture blocks
         * Note that this method will only return blocks that are reachable from the final block(s) and only after the material has been built!
         * @returns an array of texture blocks
         */
        getTextureBlocks(): NodeMaterialTextureBlocks[];
        /**
         * Gets the list of all texture blocks
         * Note that this method will scan all attachedBlocks and return blocks that are texture blocks
         * @returns
         */
        getAllTextureBlocks(): NodeMaterialTextureBlocks[];
        /**
         * Specifies if the material uses a texture
         * @param texture defines the texture to check against the material
         * @returns a boolean specifying if the material uses the texture
         */
        hasTexture(texture: BaseTexture): boolean;
        /**
         * Disposes the material
         * @param forceDisposeEffect specifies if effects should be forcefully disposed
         * @param forceDisposeTextures specifies if textures should be forcefully disposed
         * @param notBoundToMesh specifies if the material that is being disposed is known to be not bound to any mesh
         */
        dispose(forceDisposeEffect?: boolean, forceDisposeTextures?: boolean, notBoundToMesh?: boolean): void;
        /** Creates the node editor window. */
        private _createNodeEditor;
        /**
         * Launch the node material editor
         * @param config Define the configuration of the editor
         * @returns a promise fulfilled when the node editor is visible
         */
        edit(config?: INodeMaterialEditorOptions): Promise<void>;
        /**
         * Clear the current material
         */
        clear(): void;
        /**
         * Clear the current material and set it to a default state
         */
        setToDefault(): void;
        /**
         * Clear the current material and set it to a default state for post process
         */
        setToDefaultPostProcess(): void;
        /**
         * Clear the current material and set it to a default state for procedural texture
         */
        setToDefaultProceduralTexture(): void;
        /**
         * Clear the current material and set it to a default state for particle
         */
        setToDefaultParticle(): void;
        /**
         * Loads the current Node Material from a url pointing to a file save by the Node Material Editor
         * @deprecated Please use NodeMaterial.ParseFromFileAsync instead
         * @param url defines the url to load from
         * @param rootUrl defines the root URL for nested url in the node material
         * @returns a promise that will fulfil when the material is fully loaded
         */
        loadAsync(url: string, rootUrl?: string): Promise<NodeMaterial>;
        private _gatherBlocks;
        /**
         * Generate a string containing the code declaration required to create an equivalent of this material
         * @returns a string
         */
        generateCode(): string;
        /**
         * Serializes this material in a JSON representation
         * @param selectedBlocks
         * @returns the serialized material object
         */
        serialize(selectedBlocks?: NodeMaterialBlock[]): any;
        private _restoreConnections;
        /**
         * Clear the current graph and load a new one from a serialization object
         * @param source defines the JSON representation of the material
         * @param rootUrl defines the root URL to use to load textures and relative dependencies
         * @param merge defines whether or not the source must be merged or replace the current content
         */
        parseSerializedObject(source: any, rootUrl?: string, merge?: boolean): void;
        /**
         * Clear the current graph and load a new one from a serialization object
         * @param source defines the JSON representation of the material
         * @param rootUrl defines the root URL to use to load textures and relative dependencies
         * @param merge defines whether or not the source must be merged or replace the current content
         * @deprecated Please use the parseSerializedObject method instead
         */
        loadFromSerialization(source: any, rootUrl?: string, merge?: boolean): void;
        /**
         * Makes a duplicate of the current material.
         * @param name defines the name to use for the new material
         * @param shareEffect defines if the clone material should share the same effect (default is false)
         */
        clone(name: string, shareEffect?: boolean): NodeMaterial;
        /**
         * Creates a node material from parsed material data
         * @param source defines the JSON representation of the material
         * @param scene defines the hosting scene
         * @param rootUrl defines the root URL to use to load textures and relative dependencies
         * @returns a new node material
         */
        static Parse(source: any, scene: Scene, rootUrl?: string): NodeMaterial;
        /**
         * Creates a node material from a snippet saved in a remote file
         * @param name defines the name of the material to create
         * @param url defines the url to load from
         * @param scene defines the hosting scene
         * @param rootUrl defines the root URL for nested url in the node material
         * @param skipBuild defines whether to build the node material
         * @param targetMaterial defines a material to use instead of creating a new one
         * @returns a promise that will resolve to the new node material
         */
        static ParseFromFileAsync(name: string, url: string, scene: Scene, rootUrl?: string, skipBuild?: boolean, targetMaterial?: NodeMaterial): Promise<NodeMaterial>;
        /**
         * Creates a node material from a snippet saved by the node material editor
         * @param snippetId defines the snippet to load
         * @param scene defines the hosting scene
         * @param rootUrl defines the root URL to use to load textures and relative dependencies
         * @param nodeMaterial defines a node material to update (instead of creating a new one)
         * @param skipBuild defines whether to build the node material
         * @returns a promise that will resolve to the new node material
         */
        static ParseFromSnippetAsync(snippetId: string, scene?: Scene, rootUrl?: string, nodeMaterial?: NodeMaterial, skipBuild?: boolean): Promise<NodeMaterial>;
        /**
         * Creates a new node material set to default basic configuration
         * @param name defines the name of the material
         * @param scene defines the hosting scene
         * @returns a new NodeMaterial
         */
        static CreateDefault(name: string, scene?: Scene): NodeMaterial;
    }


    /**
     * Defines a block that can be used inside a node based material
     */
    export class NodeMaterialBlock {
        private _buildId;
        private _buildTarget;
        protected _target: NodeMaterialBlockTargets;
        private _isFinalMerger;
        private _isInput;
        private _name;
        protected _isUnique: boolean;
        /** Gets or sets a boolean indicating that only one input can be connected at a time */
        inputsAreExclusive: boolean;
        /** @internal */
        _codeVariableName: string;
        /** @internal */
        _inputs: NodeMaterialConnectionPoint[];
        /** @internal */
        _outputs: NodeMaterialConnectionPoint[];
        /** @internal */
        _preparationId: number;
        /** @internal */
        readonly _originalTargetIsNeutral: boolean;
        /**
         * Gets the name of the block
         */
        get name(): string;
        /**
         * Sets the name of the block. Will check if the name is valid.
         */
        set name(newName: string);
        /**
         * Gets or sets the unique id of the node
         */
        uniqueId: number;
        /**
         * Gets or sets the comments associated with this block
         */
        comments: string;
        /**
         * Gets a boolean indicating that this block can only be used once per NodeMaterial
         */
        get isUnique(): boolean;
        /**
         * Gets a boolean indicating that this block is an end block (e.g. it is generating a system value)
         */
        get isFinalMerger(): boolean;
        /**
         * Gets a boolean indicating that this block is an input (e.g. it sends data to the shader)
         */
        get isInput(): boolean;
        /**
         * Gets or sets the build Id
         */
        get buildId(): number;
        set buildId(value: number);
        /**
         * Gets or sets the target of the block
         */
        get target(): NodeMaterialBlockTargets;
        set target(value: NodeMaterialBlockTargets);
        /**
         * Gets the list of input points
         */
        get inputs(): NodeMaterialConnectionPoint[];
        /** Gets the list of output points */
        get outputs(): NodeMaterialConnectionPoint[];
        /**
         * Find an input by its name
         * @param name defines the name of the input to look for
         * @returns the input or null if not found
         */
        getInputByName(name: string): NodeMaterialConnectionPoint | null;
        /**
         * Find an output by its name
         * @param name defines the name of the output to look for
         * @returns the output or null if not found
         */
        getOutputByName(name: string): NodeMaterialConnectionPoint | null;
        /** Gets or sets a boolean indicating that this input can be edited in the Inspector (false by default) */
        visibleInInspector: boolean;
        /** Gets or sets a boolean indicating that this input can be edited from a collapsed frame */
        visibleOnFrame: boolean;
        /**
         * Creates a new NodeMaterialBlock
         * @param name defines the block name
         * @param target defines the target of that block (Vertex by default)
         * @param isFinalMerger defines a boolean indicating that this block is an end block (e.g. it is generating a system value). Default is false
         * @param isInput defines a boolean indicating that this block is an input (e.g. it sends data to the shader). Default is false
         */
        constructor(name: string, target?: NodeMaterialBlockTargets, isFinalMerger?: boolean, isInput?: boolean);
        /** @internal */
        _setInitialTarget(target: NodeMaterialBlockTargets): void;
        /**
         * Initialize the block and prepare the context for build
         * @param state defines the state that will be used for the build
         */
        initialize(state: NodeMaterialBuildState): void;
        /**
         * Bind data to effect. Will only be called for blocks with isBindable === true
         * @param effect defines the effect to bind data to
         * @param nodeMaterial defines the hosting NodeMaterial
         * @param mesh defines the mesh that will be rendered
         * @param subMesh defines the submesh that will be rendered
         */
        bind(effect: Effect, nodeMaterial: NodeMaterial, mesh?: Mesh, subMesh?: SubMesh): void;
        protected _declareOutput(output: NodeMaterialConnectionPoint, state: NodeMaterialBuildState): string;
        protected _writeVariable(currentPoint: NodeMaterialConnectionPoint): string;
        protected _writeFloat(value: number): string;
        /**
         * Gets the current class name e.g. "NodeMaterialBlock"
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Register a new input. Must be called inside a block constructor
         * @param name defines the connection point name
         * @param type defines the connection point type
         * @param isOptional defines a boolean indicating that this input can be omitted
         * @param target defines the target to use to limit the connection point (will be VertexAndFragment by default)
         * @param point an already created connection point. If not provided, create a new one
         * @returns the current block
         */
        registerInput(name: string, type: NodeMaterialBlockConnectionPointTypes, isOptional?: boolean, target?: NodeMaterialBlockTargets, point?: NodeMaterialConnectionPoint): this;
        /**
         * Register a new output. Must be called inside a block constructor
         * @param name defines the connection point name
         * @param type defines the connection point type
         * @param target defines the target to use to limit the connection point (will be VertexAndFragment by default)
         * @param point an already created connection point. If not provided, create a new one
         * @returns the current block
         */
        registerOutput(name: string, type: NodeMaterialBlockConnectionPointTypes, target?: NodeMaterialBlockTargets, point?: NodeMaterialConnectionPoint): this;
        /**
         * Will return the first available input e.g. the first one which is not an uniform or an attribute
         * @param forOutput defines an optional connection point to check compatibility with
         * @returns the first available input or null
         */
        getFirstAvailableInput(forOutput?: Nullable<NodeMaterialConnectionPoint>): NodeMaterialConnectionPoint | null;
        /**
         * Will return the first available output e.g. the first one which is not yet connected and not a varying
         * @param forBlock defines an optional block to check compatibility with
         * @returns the first available input or null
         */
        getFirstAvailableOutput(forBlock?: Nullable<NodeMaterialBlock>): NodeMaterialConnectionPoint | null;
        /**
         * Gets the sibling of the given output
         * @param current defines the current output
         * @returns the next output in the list or null
         */
        getSiblingOutput(current: NodeMaterialConnectionPoint): NodeMaterialConnectionPoint | null;
        /**
         * Checks if the current block is an ancestor of a given block
         * @param block defines the potential descendant block to check
         * @returns true if block is a descendant
         */
        isAnAncestorOf(block: NodeMaterialBlock): boolean;
        /**
         * Connect current block with another block
         * @param other defines the block to connect with
         * @param options define the various options to help pick the right connections
         * @param options.input
         * @param options.output
         * @param options.outputSwizzle
         * @returns the current block
         */
        connectTo(other: NodeMaterialBlock, options?: {
            input?: string;
            output?: string;
            outputSwizzle?: string;
        }): this | undefined;
        protected _buildBlock(state: NodeMaterialBuildState): void;
        /**
         * Add uniforms, samplers and uniform buffers at compilation time
         * @param state defines the state to update
         * @param nodeMaterial defines the node material requesting the update
         * @param defines defines the material defines to update
         * @param uniformBuffers defines the list of uniform buffer names
         */
        updateUniformsAndSamples(state: NodeMaterialBuildState, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines, uniformBuffers: string[]): void;
        /**
         * Add potential fallbacks if shader compilation fails
         * @param mesh defines the mesh to be rendered
         * @param fallbacks defines the current prioritized list of fallbacks
         */
        provideFallbacks(mesh: AbstractMesh, fallbacks: EffectFallbacks): void;
        /**
         * Initialize defines for shader compilation
         * @param mesh defines the mesh to be rendered
         * @param nodeMaterial defines the node material requesting the update
         * @param defines defines the material defines to update
         * @param useInstances specifies that instances should be used
         */
        initializeDefines(mesh: AbstractMesh, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines, useInstances?: boolean): void;
        /**
         * Update defines for shader compilation
         * @param mesh defines the mesh to be rendered
         * @param nodeMaterial defines the node material requesting the update
         * @param defines defines the material defines to update
         * @param useInstances specifies that instances should be used
         * @param subMesh defines which submesh to render
         */
        prepareDefines(mesh: AbstractMesh, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines, useInstances?: boolean, subMesh?: SubMesh): void;
        /**
         * Lets the block try to connect some inputs automatically
         * @param material defines the hosting NodeMaterial
         */
        autoConfigure(material: NodeMaterial): void;
        /**
         * Function called when a block is declared as repeatable content generator
         * @param vertexShaderState defines the current compilation state for the vertex shader
         * @param fragmentShaderState defines the current compilation state for the fragment shader
         * @param mesh defines the mesh to be rendered
         * @param defines defines the material defines to update
         */
        replaceRepeatableContent(vertexShaderState: NodeMaterialBuildState, fragmentShaderState: NodeMaterialBuildState, mesh: AbstractMesh, defines: NodeMaterialDefines): void;
        /** Gets a boolean indicating that the code of this block will be promoted to vertex shader even if connected to fragment output */
        get willBeGeneratedIntoVertexShaderFromFragmentShader(): boolean;
        /**
         * Checks if the block is ready
         * @param mesh defines the mesh to be rendered
         * @param nodeMaterial defines the node material requesting the update
         * @param defines defines the material defines to update
         * @param useInstances specifies that instances should be used
         * @returns true if the block is ready
         */
        isReady(mesh: AbstractMesh, nodeMaterial: NodeMaterial, defines: NodeMaterialDefines, useInstances?: boolean): boolean;
        protected _linkConnectionTypes(inputIndex0: number, inputIndex1: number, looseCoupling?: boolean): void;
        private _processBuild;
        /**
         * Validates the new name for the block node.
         * @param newName the new name to be given to the node.
         * @returns false if the name is a reserve word, else true.
         */
        validateBlockName(newName: string): boolean;
        /**
         * Compile the current node and generate the shader code
         * @param state defines the current compilation state (uniforms, samplers, current string)
         * @param activeBlocks defines the list of active blocks (i.e. blocks to compile)
         * @returns true if already built
         */
        build(state: NodeMaterialBuildState, activeBlocks: NodeMaterialBlock[]): boolean;
        protected _inputRename(name: string): string;
        protected _outputRename(name: string): string;
        protected _dumpPropertiesCode(): string;
        /**
         * @internal
         */
        _dumpCode(uniqueNames: string[], alreadyDumped: NodeMaterialBlock[]): string;
        /**
         * @internal
         */
        _dumpCodeForOutputConnections(alreadyDumped: NodeMaterialBlock[]): string;
        /**
         * Clone the current block to a new identical block
         * @param scene defines the hosting scene
         * @param rootUrl defines the root URL to use to load textures and relative dependencies
         * @returns a copy of the current block
         */
        clone(scene: Scene, rootUrl?: string): NodeMaterialBlock | null;
        /**
         * Serializes this block in a JSON representation
         * @returns the serialized block object
         */
        serialize(): any;
        /**
         * @internal
         */
        _deserialize(serializationObject: any, scene: Scene, rootUrl: string): void;
        private _deserializePortDisplayNamesAndExposedOnFrame;
        /**
         * Release resources
         */
        dispose(): void;
    }


    /**
     * Enum used to define the compatibility state between two connection points
     */
    export enum NodeMaterialConnectionPointCompatibilityStates {
        /** Points are compatibles */
        Compatible = 0,
        /** Points are incompatible because of their types */
        TypeIncompatible = 1,
        /** Points are incompatible because of their targets (vertex vs fragment) */
        TargetIncompatible = 2,
        /** Points are incompatible because they are in the same hierarchy **/
        HierarchyIssue = 3
    }
    /**
     * Defines the direction of a connection point
     */
    export enum NodeMaterialConnectionPointDirection {
        /** Input */
        Input = 0,
        /** Output */
        Output = 1
    }
    /**
     * Defines a connection point for a block
     */
    export class NodeMaterialConnectionPoint {
        /**
         * Checks if two types are equivalent
         * @param type1 type 1 to check
         * @param type2 type 2 to check
         * @returns true if both types are equivalent, else false
         */
        static AreEquivalentTypes(type1: number, type2: number): boolean;
        /** @internal */
        _ownerBlock: NodeMaterialBlock;
        /** @internal */
        _connectedPoint: Nullable<NodeMaterialConnectionPoint>;
        private _endpoints;
        private _associatedVariableName;
        private _direction;
        /** @internal */
        _typeConnectionSource: Nullable<NodeMaterialConnectionPoint>;
        /** @internal */
        _defaultConnectionPointType: Nullable<NodeMaterialBlockConnectionPointTypes>;
        /** @internal */
        _linkedConnectionSource: Nullable<NodeMaterialConnectionPoint>;
        /** @internal */
        _acceptedConnectionPointType: Nullable<NodeMaterialConnectionPoint>;
        private _type;
        /** @internal */
        _enforceAssociatedVariableName: boolean;
        /** Gets the direction of the point */
        get direction(): NodeMaterialConnectionPointDirection;
        /** Indicates that this connection point needs dual validation before being connected to another point */
        needDualDirectionValidation: boolean;
        /**
         * Gets or sets the additional types supported by this connection point
         */
        acceptedConnectionPointTypes: NodeMaterialBlockConnectionPointTypes[];
        /**
         * Gets or sets the additional types excluded by this connection point
         */
        excludedConnectionPointTypes: NodeMaterialBlockConnectionPointTypes[];
        /**
         * Observable triggered when this point is connected
         */
        onConnectionObservable: Observable<NodeMaterialConnectionPoint>;
        /**
         * Gets or sets the associated variable name in the shader
         */
        get associatedVariableName(): string;
        set associatedVariableName(value: string);
        /** Get the inner type (ie AutoDetect for instance instead of the inferred one) */
        get innerType(): NodeMaterialBlockConnectionPointTypes;
        /**
         * Gets or sets the connection point type (default is float)
         */
        get type(): NodeMaterialBlockConnectionPointTypes;
        set type(value: NodeMaterialBlockConnectionPointTypes);
        /**
         * Gets or sets the connection point name
         */
        name: string;
        /**
         * Gets or sets the connection point name
         */
        displayName: string;
        /**
         * Gets or sets a boolean indicating that this connection point can be omitted
         */
        isOptional: boolean;
        /**
         * Gets or sets a boolean indicating that this connection point is exposed on a frame
         */
        isExposedOnFrame: boolean;
        /**
         * Gets or sets number indicating the position that the port is exposed to on a frame
         */
        exposedPortPosition: number;
        /**
         * Gets or sets a string indicating that this uniform must be defined under a #ifdef
         */
        define: string;
        /** @internal */
        _prioritizeVertex: boolean;
        private _target;
        /** Gets or sets the target of that connection point */
        get target(): NodeMaterialBlockTargets;
        set target(value: NodeMaterialBlockTargets);
        /**
         * Gets a boolean indicating that the current point is connected to another NodeMaterialBlock
         */
        get isConnected(): boolean;
        /**
         * Gets a boolean indicating that the current point is connected to an input block
         */
        get isConnectedToInputBlock(): boolean;
        /**
         * Gets a the connected input block (if any)
         */
        get connectInputBlock(): Nullable<InputBlock>;
        /** Get the other side of the connection (if any) */
        get connectedPoint(): Nullable<NodeMaterialConnectionPoint>;
        /** Get the block that owns this connection point */
        get ownerBlock(): NodeMaterialBlock;
        /** Get the block connected on the other side of this connection (if any) */
        get sourceBlock(): Nullable<NodeMaterialBlock>;
        /** Get the block connected on the endpoints of this connection (if any) */
        get connectedBlocks(): Array<NodeMaterialBlock>;
        /** Gets the list of connected endpoints */
        get endpoints(): NodeMaterialConnectionPoint[];
        /** Gets a boolean indicating if that output point is connected to at least one input */
        get hasEndpoints(): boolean;
        /** Gets a boolean indicating that this connection has a path to the vertex output*/
        get isDirectlyConnectedToVertexOutput(): boolean;
        /** Gets a boolean indicating that this connection will be used in the vertex shader */
        get isConnectedInVertexShader(): boolean;
        /** Gets a boolean indicating that this connection will be used in the fragment shader */
        get isConnectedInFragmentShader(): boolean;
        /**
         * Creates a block suitable to be used as an input for this input point.
         * If null is returned, a block based on the point type will be created.
         * @returns The returned string parameter is the name of the output point of NodeMaterialBlock (first parameter of the returned array) that can be connected to the input
         */
        createCustomInputBlock(): Nullable<[NodeMaterialBlock, string]>;
        /**
         * Creates a new connection point
         * @param name defines the connection point name
         * @param ownerBlock defines the block hosting this connection point
         * @param direction defines the direction of the connection point
         */
        constructor(name: string, ownerBlock: NodeMaterialBlock, direction: NodeMaterialConnectionPointDirection);
        /**
         * Gets the current class name e.g. "NodeMaterialConnectionPoint"
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Gets a boolean indicating if the current point can be connected to another point
         * @param connectionPoint defines the other connection point
         * @returns a boolean
         */
        canConnectTo(connectionPoint: NodeMaterialConnectionPoint): boolean;
        /**
         * Gets a number indicating if the current point can be connected to another point
         * @param connectionPoint defines the other connection point
         * @returns a number defining the compatibility state
         */
        checkCompatibilityState(connectionPoint: NodeMaterialConnectionPoint): NodeMaterialConnectionPointCompatibilityStates;
        /**
         * Connect this point to another connection point
         * @param connectionPoint defines the other connection point
         * @param ignoreConstraints defines if the system will ignore connection type constraints (default is false)
         * @returns the current connection point
         */
        connectTo(connectionPoint: NodeMaterialConnectionPoint, ignoreConstraints?: boolean): NodeMaterialConnectionPoint;
        /**
         * Disconnect this point from one of his endpoint
         * @param endpoint defines the other connection point
         * @returns the current connection point
         */
        disconnectFrom(endpoint: NodeMaterialConnectionPoint): NodeMaterialConnectionPoint;
        /**
         * Fill the list of excluded connection point types with all types other than those passed in the parameter
         * @param mask Types (ORed values of NodeMaterialBlockConnectionPointTypes) that are allowed, and thus will not be pushed to the excluded list
         */
        addExcludedConnectionPointFromAllowedTypes(mask: number): void;
        /**
         * Serializes this point in a JSON representation
         * @param isInput defines if the connection point is an input (default is true)
         * @returns the serialized point object
         */
        serialize(isInput?: boolean): any;
        /**
         * Release resources
         */
        dispose(): void;
    }


    /**
     * Class used to store node based material build state
     */
    export class NodeMaterialBuildState {
        /** Gets or sets a boolean indicating if the current state can emit uniform buffers */
        supportUniformBuffers: boolean;
        /**
         * Gets the list of emitted attributes
         */
        attributes: string[];
        /**
         * Gets the list of emitted uniforms
         */
        uniforms: string[];
        /**
         * Gets the list of emitted constants
         */
        constants: string[];
        /**
         * Gets the list of emitted samplers
         */
        samplers: string[];
        /**
         * Gets the list of emitted functions
         */
        functions: {
            [key: string]: string;
        };
        /**
         * Gets the list of emitted extensions
         */
        extensions: {
            [key: string]: string;
        };
        /**
         * Gets the target of the compilation state
         */
        target: NodeMaterialBlockTargets;
        /**
         * Gets the list of emitted counters
         */
        counters: {
            [key: string]: number;
        };
        /**
         * Shared data between multiple NodeMaterialBuildState instances
         */
        sharedData: NodeMaterialBuildStateSharedData;
        /** @internal */
        _vertexState: NodeMaterialBuildState;
        /** @internal */
        _attributeDeclaration: string;
        /** @internal */
        _uniformDeclaration: string;
        /** @internal */
        _constantDeclaration: string;
        /** @internal */
        _samplerDeclaration: string;
        /** @internal */
        _varyingTransfer: string;
        /** @internal */
        _injectAtEnd: string;
        private _repeatableContentAnchorIndex;
        /** @internal */
        _builtCompilationString: string;
        /**
         * Gets the emitted compilation strings
         */
        compilationString: string;
        /**
         * Finalize the compilation strings
         * @param state defines the current compilation state
         */
        finalize(state: NodeMaterialBuildState): void;
        /** @internal */
        get _repeatableContentAnchor(): string;
        /**
         * @internal
         */
        _getFreeVariableName(prefix: string): string;
        /**
         * @internal
         */
        _getFreeDefineName(prefix: string): string;
        /**
         * @internal
         */
        _excludeVariableName(name: string): void;
        /**
         * @internal
         */
        _emit2DSampler(name: string): void;
        /**
         * @internal
         */
        _getGLType(type: NodeMaterialBlockConnectionPointTypes): string;
        /**
         * @internal
         */
        _emitExtension(name: string, extension: string, define?: string): void;
        /**
         * @internal
         */
        _emitFunction(name: string, code: string, comments: string): void;
        /**
         * @internal
         */
        _emitCodeFromInclude(includeName: string, comments: string, options?: {
            replaceStrings?: {
                search: RegExp;
                replace: string;
            }[];
            repeatKey?: string;
            substitutionVars?: string;
        }): string;
        /**
         * @internal
         */
        _emitFunctionFromInclude(includeName: string, comments: string, options?: {
            repeatKey?: string;
            substitutionVars?: string;
            removeAttributes?: boolean;
            removeUniforms?: boolean;
            removeVaryings?: boolean;
            removeIfDef?: boolean;
            replaceStrings?: {
                search: RegExp;
                replace: string;
            }[];
        }, storeKey?: string): void;
        /**
         * @internal
         */
        _registerTempVariable(name: string): boolean;
        /**
         * @internal
         */
        _emitVaryingFromString(name: string, type: string, define?: string, notDefine?: boolean): boolean;
        /**
         * @internal
         */
        _emitUniformFromString(name: string, type: string, define?: string, notDefine?: boolean): void;
        /**
         * @internal
         */
        _emitFloat(value: number): string;
    }


    /**
     * Class used to store shared data between 2 NodeMaterialBuildState
     */
    export class NodeMaterialBuildStateSharedData {
        /**
         * Gets the list of emitted varyings
         */
        temps: string[];
        /**
         * Gets the list of emitted varyings
         */
        varyings: string[];
        /**
         * Gets the varying declaration string
         */
        varyingDeclaration: string;
        /**
         * List of the fragment output nodes
         */
        fragmentOutputNodes: Immutable<Array<NodeMaterialBlock>>;
        /**
         * Input blocks
         */
        inputBlocks: InputBlock[];
        /**
         * Input blocks
         */
        textureBlocks: NodeMaterialTextureBlocks[];
        /**
         * Bindable blocks (Blocks that need to set data to the effect)
         */
        bindableBlocks: NodeMaterialBlock[];
        /**
         * Bindable blocks (Blocks that need to set data to the effect) that will always be called (by bindForSubMesh), contrary to bindableBlocks that won't be called if _mustRebind() returns false
         */
        forcedBindableBlocks: NodeMaterialBlock[];
        /**
         * List of blocks that can provide a compilation fallback
         */
        blocksWithFallbacks: NodeMaterialBlock[];
        /**
         * List of blocks that can provide a define update
         */
        blocksWithDefines: NodeMaterialBlock[];
        /**
         * List of blocks that can provide a repeatable content
         */
        repeatableContentBlocks: NodeMaterialBlock[];
        /**
         * List of blocks that can provide a dynamic list of uniforms
         */
        dynamicUniformBlocks: NodeMaterialBlock[];
        /**
         * List of blocks that can block the isReady function for the material
         */
        blockingBlocks: NodeMaterialBlock[];
        /**
         * Gets the list of animated inputs
         */
        animatedInputs: InputBlock[];
        /**
         * Build Id used to avoid multiple recompilations
         */
        buildId: number;
        /** List of emitted variables */
        variableNames: {
            [key: string]: number;
        };
        /** List of emitted defines */
        defineNames: {
            [key: string]: number;
        };
        /** Should emit comments? */
        emitComments: boolean;
        /** Emit build activity */
        verbose: boolean;
        /** Gets or sets the hosting scene */
        scene: Scene;
        /**
         * Gets the compilation hints emitted at compilation time
         */
        hints: {
            needWorldViewMatrix: boolean;
            needWorldViewProjectionMatrix: boolean;
            needAlphaBlending: boolean;
            needAlphaTesting: boolean;
        };
        /**
         * List of compilation checks
         */
        checks: {
            emitVertex: boolean;
            emitFragment: boolean;
            notConnectedNonOptionalInputs: NodeMaterialConnectionPoint[];
        };
        /**
         * Is vertex program allowed to be empty?
         */
        allowEmptyVertexProgram: boolean;
        /** Creates a new shared data */
        constructor();
        /**
         * Emits console errors and exceptions if there is a failing check
         */
        emitErrors(): void;
    }


    /**
     * Defines a connection point to be used for points with a custom object type
     */
    export class NodeMaterialConnectionPointCustomObject<T extends NodeMaterialBlock> extends NodeMaterialConnectionPoint {
        private _blockType;
        private _blockName;
        /**
         * Creates a new connection point
         * @param name defines the connection point name
         * @param ownerBlock defines the block hosting this connection point
         * @param direction defines the direction of the connection point
         * @param _blockType
         * @param _blockName
         */
        constructor(name: string, ownerBlock: NodeMaterialBlock, direction: NodeMaterialConnectionPointDirection, _blockType: new (...args: any[]) => T, _blockName: string);
        /**
         * Gets a number indicating if the current point can be connected to another point
         * @param connectionPoint defines the other connection point
         * @returns a number defining the compatibility state
         */
        checkCompatibilityState(connectionPoint: NodeMaterialConnectionPoint): NodeMaterialConnectionPointCompatibilityStates;
        /**
         * Creates a block suitable to be used as an input for this input point.
         * If null is returned, a block based on the point type will be created.
         * @returns The returned string parameter is the name of the output point of NodeMaterialBlock (first parameter of the returned array) that can be connected to the input
         */
        createCustomInputBlock(): Nullable<[NodeMaterialBlock, string]>;
    }


    /**
     * Enum defining the type of properties that can be edited in the property pages in the NME
     */
    export enum PropertyTypeForEdition {
        /** property is a boolean */
        Boolean = 0,
        /** property is a float */
        Float = 1,
        /** property is a int */
        Int = 2,
        /** property is a Vector2 */
        Vector2 = 3,
        /** property is a list of values */
        List = 4
    }
    /**
     * Interface that defines an option in a variable of type list
     */
    export interface IEditablePropertyListOption {
        /** label of the option */
        label: string;
        /** value of the option */
        value: number;
    }
    /**
     * Interface that defines the options available for an editable property
     */
    export interface IEditablePropertyOption {
        /** min value */
        min?: number;
        /** max value */
        max?: number;
        /** notifiers: indicates which actions to take when the property is changed */
        notifiers?: {
            /** the material should be rebuilt */
            rebuild?: boolean;
            /** the preview should be updated */
            update?: boolean;
            /** the onPreviewCommandActivated observer of the preview manager should be triggered */
            activatePreviewCommand?: boolean;
            /** a callback to trigger */
            callback?: (scene: Scene, block: NodeMaterialBlock) => void;
            /** a callback to validate the property. Returns true if the property is ok, else false. If false, the rebuild/update/callback events won't be called */
            onValidation?: (block: NodeMaterialBlock, propertyName: string) => boolean;
        };
        /** list of the options for a variable of type list */
        options?: IEditablePropertyListOption[];
    }
    /**
     * Interface that describes an editable property
     */
    export interface IPropertyDescriptionForEdition {
        /** name of the property */
        propertyName: string;
        /** display name of the property */
        displayName: string;
        /** type of the property */
        type: PropertyTypeForEdition;
        /** group of the property - all properties with the same group value will be displayed in a specific section */
        groupName: string;
        /** options for the property */
        options: IEditablePropertyOption;
    }
    /**
     * Decorator that flags a property in a node material block as being editable
     * @param displayName
     * @param propertyType
     * @param groupName
     * @param options
     */
    export function editableInPropertyPage(displayName: string, propertyType?: PropertyTypeForEdition, groupName?: string, options?: IEditablePropertyOption): (target: any, propertyKey: string) => void;




    /**
     * Root class for all node material optimizers
     */
    export class NodeMaterialOptimizer {
        /**
         * Function used to optimize a NodeMaterial graph
         * @param _vertexOutputNodes defines the list of output nodes for the vertex shader
         * @param _fragmentOutputNodes defines the list of output nodes for the fragment shader
         */
        optimize(_vertexOutputNodes: NodeMaterialBlock[], _fragmentOutputNodes: NodeMaterialBlock[]): void;
    }




    /**
     * A material to use for fast depth-only rendering.
     * @since 5.0.0
     */
    export class OcclusionMaterial extends ShaderMaterial {
        constructor(name: string, scene: Scene);
    }




    /**
     * @internal
     */
    export class MaterialAnisotropicDefines extends MaterialDefines {
        ANISOTROPIC: boolean;
        ANISOTROPIC_TEXTURE: boolean;
        ANISOTROPIC_TEXTUREDIRECTUV: number;
        MAINUV1: boolean;
    }
    /**
     * Plugin that implements the anisotropic component of the PBR material
     */
    export class PBRAnisotropicConfiguration extends MaterialPluginBase {
        private _isEnabled;
        /**
         * Defines if the anisotropy is enabled in the material.
         */
        isEnabled: boolean;
        /**
         * Defines the anisotropy strength (between 0 and 1) it defaults to 1.
         */
        intensity: number;
        /**
         * Defines if the effect is along the tangents, bitangents or in between.
         * By default, the effect is "stretching" the highlights along the tangents.
         */
        direction: Vector2;
        private _texture;
        /**
         * Stores the anisotropy values in a texture.
         * rg is direction (like normal from -1 to 1)
         * b is a intensity
         */
        texture: Nullable<BaseTexture>;
        /** @internal */
        private _internalMarkAllSubMeshesAsTexturesDirty;
        /** @internal */
        _markAllSubMeshesAsTexturesDirty(): void;
        constructor(material: PBRBaseMaterial, addToPluginList?: boolean);
        isReadyForSubMesh(defines: MaterialAnisotropicDefines, scene: Scene): boolean;
        prepareDefinesBeforeAttributes(defines: MaterialAnisotropicDefines, scene: Scene, mesh: AbstractMesh): void;
        bindForSubMesh(uniformBuffer: UniformBuffer, scene: Scene): void;
        hasTexture(texture: BaseTexture): boolean;
        getActiveTextures(activeTextures: BaseTexture[]): void;
        getAnimatables(animatables: IAnimatable[]): void;
        dispose(forceDisposeTextures?: boolean): void;
        getClassName(): string;
        addFallbacks(defines: MaterialAnisotropicDefines, fallbacks: EffectFallbacks, currentRank: number): number;
        getSamplers(samplers: string[]): void;
        getUniforms(): {
            ubo?: Array<{
                name: string;
                size: number;
                type: string;
            }>;
            vertex?: string;
            fragment?: string;
        };
    }


    /**
     * Manages the defines for the PBR Material.
     * @internal
     */
    export class PBRMaterialDefines extends MaterialDefines implements IImageProcessingConfigurationDefines {
        PBR: boolean;
        NUM_SAMPLES: string;
        REALTIME_FILTERING: boolean;
        MAINUV1: boolean;
        MAINUV2: boolean;
        MAINUV3: boolean;
        MAINUV4: boolean;
        MAINUV5: boolean;
        MAINUV6: boolean;
        UV1: boolean;
        UV2: boolean;
        UV3: boolean;
        UV4: boolean;
        UV5: boolean;
        UV6: boolean;
        ALBEDO: boolean;
        GAMMAALBEDO: boolean;
        ALBEDODIRECTUV: number;
        VERTEXCOLOR: boolean;
        BAKED_VERTEX_ANIMATION_TEXTURE: boolean;
        AMBIENT: boolean;
        AMBIENTDIRECTUV: number;
        AMBIENTINGRAYSCALE: boolean;
        OPACITY: boolean;
        VERTEXALPHA: boolean;
        OPACITYDIRECTUV: number;
        OPACITYRGB: boolean;
        ALPHATEST: boolean;
        DEPTHPREPASS: boolean;
        ALPHABLEND: boolean;
        ALPHAFROMALBEDO: boolean;
        ALPHATESTVALUE: string;
        SPECULAROVERALPHA: boolean;
        RADIANCEOVERALPHA: boolean;
        ALPHAFRESNEL: boolean;
        LINEARALPHAFRESNEL: boolean;
        PREMULTIPLYALPHA: boolean;
        EMISSIVE: boolean;
        EMISSIVEDIRECTUV: number;
        GAMMAEMISSIVE: boolean;
        REFLECTIVITY: boolean;
        REFLECTIVITY_GAMMA: boolean;
        REFLECTIVITYDIRECTUV: number;
        SPECULARTERM: boolean;
        MICROSURFACEFROMREFLECTIVITYMAP: boolean;
        MICROSURFACEAUTOMATIC: boolean;
        LODBASEDMICROSFURACE: boolean;
        MICROSURFACEMAP: boolean;
        MICROSURFACEMAPDIRECTUV: number;
        METALLICWORKFLOW: boolean;
        ROUGHNESSSTOREINMETALMAPALPHA: boolean;
        ROUGHNESSSTOREINMETALMAPGREEN: boolean;
        METALLNESSSTOREINMETALMAPBLUE: boolean;
        AOSTOREINMETALMAPRED: boolean;
        METALLIC_REFLECTANCE: boolean;
        METALLIC_REFLECTANCE_GAMMA: boolean;
        METALLIC_REFLECTANCEDIRECTUV: number;
        METALLIC_REFLECTANCE_USE_ALPHA_ONLY: boolean;
        REFLECTANCE: boolean;
        REFLECTANCE_GAMMA: boolean;
        REFLECTANCEDIRECTUV: number;
        ENVIRONMENTBRDF: boolean;
        ENVIRONMENTBRDF_RGBD: boolean;
        NORMAL: boolean;
        TANGENT: boolean;
        BUMP: boolean;
        BUMPDIRECTUV: number;
        OBJECTSPACE_NORMALMAP: boolean;
        PARALLAX: boolean;
        PARALLAXOCCLUSION: boolean;
        NORMALXYSCALE: boolean;
        LIGHTMAP: boolean;
        LIGHTMAPDIRECTUV: number;
        USELIGHTMAPASSHADOWMAP: boolean;
        GAMMALIGHTMAP: boolean;
        RGBDLIGHTMAP: boolean;
        REFLECTION: boolean;
        REFLECTIONMAP_3D: boolean;
        REFLECTIONMAP_SPHERICAL: boolean;
        REFLECTIONMAP_PLANAR: boolean;
        REFLECTIONMAP_CUBIC: boolean;
        USE_LOCAL_REFLECTIONMAP_CUBIC: boolean;
        REFLECTIONMAP_PROJECTION: boolean;
        REFLECTIONMAP_SKYBOX: boolean;
        REFLECTIONMAP_EXPLICIT: boolean;
        REFLECTIONMAP_EQUIRECTANGULAR: boolean;
        REFLECTIONMAP_EQUIRECTANGULAR_FIXED: boolean;
        REFLECTIONMAP_MIRROREDEQUIRECTANGULAR_FIXED: boolean;
        INVERTCUBICMAP: boolean;
        USESPHERICALFROMREFLECTIONMAP: boolean;
        USEIRRADIANCEMAP: boolean;
        USESPHERICALINVERTEX: boolean;
        REFLECTIONMAP_OPPOSITEZ: boolean;
        LODINREFLECTIONALPHA: boolean;
        GAMMAREFLECTION: boolean;
        RGBDREFLECTION: boolean;
        LINEARSPECULARREFLECTION: boolean;
        RADIANCEOCCLUSION: boolean;
        HORIZONOCCLUSION: boolean;
        INSTANCES: boolean;
        THIN_INSTANCES: boolean;
        INSTANCESCOLOR: boolean;
        PREPASS: boolean;
        PREPASS_IRRADIANCE: boolean;
        PREPASS_IRRADIANCE_INDEX: number;
        PREPASS_ALBEDO_SQRT: boolean;
        PREPASS_ALBEDO_SQRT_INDEX: number;
        PREPASS_DEPTH: boolean;
        PREPASS_DEPTH_INDEX: number;
        PREPASS_NORMAL: boolean;
        PREPASS_NORMAL_INDEX: number;
        PREPASS_POSITION: boolean;
        PREPASS_POSITION_INDEX: number;
        PREPASS_VELOCITY: boolean;
        PREPASS_VELOCITY_INDEX: number;
        PREPASS_REFLECTIVITY: boolean;
        PREPASS_REFLECTIVITY_INDEX: number;
        SCENE_MRT_COUNT: number;
        NUM_BONE_INFLUENCERS: number;
        BonesPerMesh: number;
        BONETEXTURE: boolean;
        BONES_VELOCITY_ENABLED: boolean;
        NONUNIFORMSCALING: boolean;
        MORPHTARGETS: boolean;
        MORPHTARGETS_NORMAL: boolean;
        MORPHTARGETS_TANGENT: boolean;
        MORPHTARGETS_UV: boolean;
        NUM_MORPH_INFLUENCERS: number;
        MORPHTARGETS_TEXTURE: boolean;
        IMAGEPROCESSING: boolean;
        VIGNETTE: boolean;
        VIGNETTEBLENDMODEMULTIPLY: boolean;
        VIGNETTEBLENDMODEOPAQUE: boolean;
        TONEMAPPING: boolean;
        TONEMAPPING_ACES: boolean;
        CONTRAST: boolean;
        COLORCURVES: boolean;
        COLORGRADING: boolean;
        COLORGRADING3D: boolean;
        SAMPLER3DGREENDEPTH: boolean;
        SAMPLER3DBGRMAP: boolean;
        DITHER: boolean;
        IMAGEPROCESSINGPOSTPROCESS: boolean;
        SKIPFINALCOLORCLAMP: boolean;
        EXPOSURE: boolean;
        MULTIVIEW: boolean;
        ORDER_INDEPENDENT_TRANSPARENCY: boolean;
        ORDER_INDEPENDENT_TRANSPARENCY_16BITS: boolean;
        USEPHYSICALLIGHTFALLOFF: boolean;
        USEGLTFLIGHTFALLOFF: boolean;
        TWOSIDEDLIGHTING: boolean;
        SHADOWFLOAT: boolean;
        CLIPPLANE: boolean;
        CLIPPLANE2: boolean;
        CLIPPLANE3: boolean;
        CLIPPLANE4: boolean;
        CLIPPLANE5: boolean;
        CLIPPLANE6: boolean;
        POINTSIZE: boolean;
        FOG: boolean;
        LOGARITHMICDEPTH: boolean;
        CAMERA_ORTHOGRAPHIC: boolean;
        CAMERA_PERSPECTIVE: boolean;
        FORCENORMALFORWARD: boolean;
        SPECULARAA: boolean;
        UNLIT: boolean;
        DEBUGMODE: number;
        /**
         * Initializes the PBR Material defines.
         * @param externalProperties The external properties
         */
        constructor(externalProperties?: {
            [name: string]: {
                type: string;
                default: any;
            };
        });
        /**
         * Resets the PBR Material defines.
         */
        reset(): void;
    }
    /**
     * The Physically based material base class of BJS.
     *
     * This offers the main features of a standard PBR material.
     * For more information, please refer to the documentation :
     * https://doc.babylonjs.com/features/featuresDeepDive/materials/using/introToPBR
     */
    export abstract class PBRBaseMaterial extends PushMaterial {
        /**
         * PBRMaterialTransparencyMode: No transparency mode, Alpha channel is not use.
         */
        static readonly PBRMATERIAL_OPAQUE = 0;
        /**
         * PBRMaterialTransparencyMode: Alpha Test mode, pixel are discarded below a certain threshold defined by the alpha cutoff value.
         */
        static readonly PBRMATERIAL_ALPHATEST = 1;
        /**
         * PBRMaterialTransparencyMode: Pixels are blended (according to the alpha mode) with the already drawn pixels in the current frame buffer.
         */
        static readonly PBRMATERIAL_ALPHABLEND = 2;
        /**
         * PBRMaterialTransparencyMode: Pixels are blended (according to the alpha mode) with the already drawn pixels in the current frame buffer.
         * They are also discarded below the alpha cutoff threshold to improve performances.
         */
        static readonly PBRMATERIAL_ALPHATESTANDBLEND = 3;
        /**
         * Defines the default value of how much AO map is occluding the analytical lights
         * (point spot...).
         */
        static DEFAULT_AO_ON_ANALYTICAL_LIGHTS: number;
        /**
         * PBRMaterialLightFalloff Physical: light is falling off following the inverse squared distance law.
         */
        static readonly LIGHTFALLOFF_PHYSICAL = 0;
        /**
         * PBRMaterialLightFalloff gltf: light is falling off as described in the gltf moving to PBR document
         * to enhance interoperability with other engines.
         */
        static readonly LIGHTFALLOFF_GLTF = 1;
        /**
         * PBRMaterialLightFalloff Standard: light is falling off like in the standard material
         * to enhance interoperability with other materials.
         */
        static readonly LIGHTFALLOFF_STANDARD = 2;
        /**
         * Intensity of the direct lights e.g. the four lights available in your scene.
         * This impacts both the direct diffuse and specular highlights.
         * @internal
         */
        _directIntensity: number;
        /**
         * Intensity of the emissive part of the material.
         * This helps controlling the emissive effect without modifying the emissive color.
         * @internal
         */
        _emissiveIntensity: number;
        /**
         * Intensity of the environment e.g. how much the environment will light the object
         * either through harmonics for rough material or through the reflection for shiny ones.
         * @internal
         */
        _environmentIntensity: number;
        /**
         * This is a special control allowing the reduction of the specular highlights coming from the
         * four lights of the scene. Those highlights may not be needed in full environment lighting.
         * @internal
         */
        _specularIntensity: number;
        /**
         * This stores the direct, emissive, environment, and specular light intensities into a Vector4.
         */
        private _lightingInfos;
        /**
         * Debug Control allowing disabling the bump map on this material.
         * @internal
         */
        _disableBumpMap: boolean;
        /**
         * AKA Diffuse Texture in standard nomenclature.
         * @internal
         */
        _albedoTexture: Nullable<BaseTexture>;
        /**
         * AKA Occlusion Texture in other nomenclature.
         * @internal
         */
        _ambientTexture: Nullable<BaseTexture>;
        /**
         * AKA Occlusion Texture Intensity in other nomenclature.
         * @internal
         */
        _ambientTextureStrength: number;
        /**
         * Defines how much the AO map is occluding the analytical lights (point spot...).
         * 1 means it completely occludes it
         * 0 mean it has no impact
         * @internal
         */
        _ambientTextureImpactOnAnalyticalLights: number;
        /**
         * Stores the alpha values in a texture.
         * @internal
         */
        _opacityTexture: Nullable<BaseTexture>;
        /**
         * Stores the reflection values in a texture.
         * @internal
         */
        _reflectionTexture: Nullable<BaseTexture>;
        /**
         * Stores the emissive values in a texture.
         * @internal
         */
        _emissiveTexture: Nullable<BaseTexture>;
        /**
         * AKA Specular texture in other nomenclature.
         * @internal
         */
        _reflectivityTexture: Nullable<BaseTexture>;
        /**
         * Used to switch from specular/glossiness to metallic/roughness workflow.
         * @internal
         */
        _metallicTexture: Nullable<BaseTexture>;
        /**
         * Specifies the metallic scalar of the metallic/roughness workflow.
         * Can also be used to scale the metalness values of the metallic texture.
         * @internal
         */
        _metallic: Nullable<number>;
        /**
         * Specifies the roughness scalar of the metallic/roughness workflow.
         * Can also be used to scale the roughness values of the metallic texture.
         * @internal
         */
        _roughness: Nullable<number>;
        /**
         * In metallic workflow, specifies an F0 factor to help configuring the material F0.
         * By default the indexOfrefraction is used to compute F0;
         *
         * This is used as a factor against the default reflectance at normal incidence to tweak it.
         *
         * F0 = defaultF0 * metallicF0Factor * metallicReflectanceColor;
         * F90 = metallicReflectanceColor;
         * @internal
         */
        _metallicF0Factor: number;
        /**
         * In metallic workflow, specifies an F90 color to help configuring the material F90.
         * By default the F90 is always 1;
         *
         * Please note that this factor is also used as a factor against the default reflectance at normal incidence.
         *
         * F0 = defaultF0 * metallicF0Factor * metallicReflectanceColor
         * F90 = metallicReflectanceColor;
         * @internal
         */
        _metallicReflectanceColor: Color3;
        /**
         * Specifies that only the A channel from _metallicReflectanceTexture should be used.
         * If false, both RGB and A channels will be used
         * @internal
         */
        _useOnlyMetallicFromMetallicReflectanceTexture: boolean;
        /**
         * Defines to store metallicReflectanceColor in RGB and metallicF0Factor in A
         * This is multiply against the scalar values defined in the material.
         * @internal
         */
        _metallicReflectanceTexture: Nullable<BaseTexture>;
        /**
         * Defines to store reflectanceColor in RGB
         * This is multiplied against the scalar values defined in the material.
         * If both _reflectanceTexture and _metallicReflectanceTexture textures are provided and _useOnlyMetallicFromMetallicReflectanceTexture
         * is false, _metallicReflectanceTexture takes precedence and _reflectanceTexture is not used
         * @internal
         */
        _reflectanceTexture: Nullable<BaseTexture>;
        /**
         * Used to enable roughness/glossiness fetch from a separate channel depending on the current mode.
         * Gray Scale represents roughness in metallic mode and glossiness in specular mode.
         * @internal
         */
        _microSurfaceTexture: Nullable<BaseTexture>;
        /**
         * Stores surface normal data used to displace a mesh in a texture.
         * @internal
         */
        _bumpTexture: Nullable<BaseTexture>;
        /**
         * Stores the pre-calculated light information of a mesh in a texture.
         * @internal
         */
        _lightmapTexture: Nullable<BaseTexture>;
        /**
         * The color of a material in ambient lighting.
         * @internal
         */
        _ambientColor: Color3;
        /**
         * AKA Diffuse Color in other nomenclature.
         * @internal
         */
        _albedoColor: Color3;
        /**
         * AKA Specular Color in other nomenclature.
         * @internal
         */
        _reflectivityColor: Color3;
        /**
         * The color applied when light is reflected from a material.
         * @internal
         */
        _reflectionColor: Color3;
        /**
         * The color applied when light is emitted from a material.
         * @internal
         */
        _emissiveColor: Color3;
        /**
         * AKA Glossiness in other nomenclature.
         * @internal
         */
        _microSurface: number;
        /**
         * Specifies that the material will use the light map as a show map.
         * @internal
         */
        _useLightmapAsShadowmap: boolean;
        /**
         * This parameters will enable/disable Horizon occlusion to prevent normal maps to look shiny when the normal
         * makes the reflect vector face the model (under horizon).
         * @internal
         */
        _useHorizonOcclusion: boolean;
        /**
         * This parameters will enable/disable radiance occlusion by preventing the radiance to lit
         * too much the area relying on ambient texture to define their ambient occlusion.
         * @internal
         */
        _useRadianceOcclusion: boolean;
        /**
         * Specifies that the alpha is coming form the albedo channel alpha channel for alpha blending.
         * @internal
         */
        _useAlphaFromAlbedoTexture: boolean;
        /**
         * Specifies that the material will keeps the specular highlights over a transparent surface (only the most luminous ones).
         * A car glass is a good example of that. When sun reflects on it you can not see what is behind.
         * @internal
         */
        _useSpecularOverAlpha: boolean;
        /**
         * Specifies if the reflectivity texture contains the glossiness information in its alpha channel.
         * @internal
         */
        _useMicroSurfaceFromReflectivityMapAlpha: boolean;
        /**
         * Specifies if the metallic texture contains the roughness information in its alpha channel.
         * @internal
         */
        _useRoughnessFromMetallicTextureAlpha: boolean;
        /**
         * Specifies if the metallic texture contains the roughness information in its green channel.
         * @internal
         */
        _useRoughnessFromMetallicTextureGreen: boolean;
        /**
         * Specifies if the metallic texture contains the metallness information in its blue channel.
         * @internal
         */
        _useMetallnessFromMetallicTextureBlue: boolean;
        /**
         * Specifies if the metallic texture contains the ambient occlusion information in its red channel.
         * @internal
         */
        _useAmbientOcclusionFromMetallicTextureRed: boolean;
        /**
         * Specifies if the ambient texture contains the ambient occlusion information in its red channel only.
         * @internal
         */
        _useAmbientInGrayScale: boolean;
        /**
         * In case the reflectivity map does not contain the microsurface information in its alpha channel,
         * The material will try to infer what glossiness each pixel should be.
         * @internal
         */
        _useAutoMicroSurfaceFromReflectivityMap: boolean;
        /**
         * Defines the  falloff type used in this material.
         * It by default is Physical.
         * @internal
         */
        _lightFalloff: number;
        /**
         * Specifies that the material will keeps the reflection highlights over a transparent surface (only the most luminous ones).
         * A car glass is a good example of that. When the street lights reflects on it you can not see what is behind.
         * @internal
         */
        _useRadianceOverAlpha: boolean;
        /**
         * Allows using an object space normal map (instead of tangent space).
         * @internal
         */
        _useObjectSpaceNormalMap: boolean;
        /**
         * Allows using the bump map in parallax mode.
         * @internal
         */
        _useParallax: boolean;
        /**
         * Allows using the bump map in parallax occlusion mode.
         * @internal
         */
        _useParallaxOcclusion: boolean;
        /**
         * Controls the scale bias of the parallax mode.
         * @internal
         */
        _parallaxScaleBias: number;
        /**
         * If sets to true, disables all the lights affecting the material.
         * @internal
         */
        _disableLighting: boolean;
        /**
         * Number of Simultaneous lights allowed on the material.
         * @internal
         */
        _maxSimultaneousLights: number;
        /**
         * If sets to true, x component of normal map value will be inverted (x = 1.0 - x).
         * @internal
         */
        _invertNormalMapX: boolean;
        /**
         * If sets to true, y component of normal map value will be inverted (y = 1.0 - y).
         * @internal
         */
        _invertNormalMapY: boolean;
        /**
         * If sets to true and backfaceCulling is false, normals will be flipped on the backside.
         * @internal
         */
        _twoSidedLighting: boolean;
        /**
         * Defines the alpha limits in alpha test mode.
         * @internal
         */
        _alphaCutOff: number;
        /**
         * Enforces alpha test in opaque or blend mode in order to improve the performances of some situations.
         * @internal
         */
        _forceAlphaTest: boolean;
        /**
         * A fresnel is applied to the alpha of the model to ensure grazing angles edges are not alpha tested.
         * And/Or occlude the blended part. (alpha is converted to gamma to compute the fresnel)
         * @internal
         */
        _useAlphaFresnel: boolean;
        /**
         * A fresnel is applied to the alpha of the model to ensure grazing angles edges are not alpha tested.
         * And/Or occlude the blended part. (alpha stays linear to compute the fresnel)
         * @internal
         */
        _useLinearAlphaFresnel: boolean;
        /**
         * Specifies the environment BRDF texture used to compute the scale and offset roughness values
         * from cos theta and roughness:
         * http://blog.selfshadow.com/publications/s2013-shading-course/karis/s2013_pbs_epic_notes_v2.pdf
         * @internal
         */
        _environmentBRDFTexture: Nullable<BaseTexture>;
        /**
         * Force the shader to compute irradiance in the fragment shader in order to take bump in account.
         * @internal
         */
        _forceIrradianceInFragment: boolean;
        private _realTimeFiltering;
        /**
         * Enables realtime filtering on the texture.
         */
        get realTimeFiltering(): boolean;
        set realTimeFiltering(b: boolean);
        private _realTimeFilteringQuality;
        /**
         * Quality switch for realtime filtering
         */
        get realTimeFilteringQuality(): number;
        set realTimeFilteringQuality(n: number);
        /**
         * Can this material render to several textures at once
         */
        get canRenderToMRT(): boolean;
        /**
         * Force normal to face away from face.
         * @internal
         */
        _forceNormalForward: boolean;
        /**
         * Enables specular anti aliasing in the PBR shader.
         * It will both interacts on the Geometry for analytical and IBL lighting.
         * It also prefilter the roughness map based on the bump values.
         * @internal
         */
        _enableSpecularAntiAliasing: boolean;
        /**
         * Default configuration related to image processing available in the PBR Material.
         */
        protected _imageProcessingConfiguration: ImageProcessingConfiguration;
        /**
         * Keep track of the image processing observer to allow dispose and replace.
         */
        private _imageProcessingObserver;
        /**
         * Attaches a new image processing configuration to the PBR Material.
         * @param configuration
         */
        protected _attachImageProcessingConfiguration(configuration: Nullable<ImageProcessingConfiguration>): void;
        /**
         * Stores the available render targets.
         */
        private _renderTargets;
        /**
         * Sets the global ambient color for the material used in lighting calculations.
         */
        private _globalAmbientColor;
        /**
         * Enables the use of logarithmic depth buffers, which is good for wide depth buffers.
         */
        private _useLogarithmicDepth;
        /**
         * If set to true, no lighting calculations will be applied.
         */
        private _unlit;
        private _debugMode;
        /**
         * @internal
         * This is reserved for the inspector.
         * Defines the material debug mode.
         * It helps seeing only some components of the material while troubleshooting.
         */
        debugMode: number;
        /**
         * @internal
         * This is reserved for the inspector.
         * Specify from where on screen the debug mode should start.
         * The value goes from -1 (full screen) to 1 (not visible)
         * It helps with side by side comparison against the final render
         * This defaults to -1
         */
        private _debugLimit;
        /**
         * @internal
         * This is reserved for the inspector.
         * As the default viewing range might not be enough (if the ambient is really small for instance)
         * You can use the factor to better multiply the final value.
         */
        private _debugFactor;
        /**
         * Defines the clear coat layer parameters for the material.
         */
        readonly clearCoat: PBRClearCoatConfiguration;
        /**
         * Defines the iridescence layer parameters for the material.
         */
        readonly iridescence: PBRIridescenceConfiguration;
        /**
         * Defines the anisotropic parameters for the material.
         */
        readonly anisotropy: PBRAnisotropicConfiguration;
        /**
         * Defines the BRDF parameters for the material.
         */
        readonly brdf: PBRBRDFConfiguration;
        /**
         * Defines the Sheen parameters for the material.
         */
        readonly sheen: PBRSheenConfiguration;
        /**
         * Defines the SubSurface parameters for the material.
         */
        readonly subSurface: PBRSubSurfaceConfiguration;
        /**
         * Defines additional PrePass parameters for the material.
         */
        readonly prePassConfiguration: PrePassConfiguration;
        /**
         * Defines the detail map parameters for the material.
         */
        readonly detailMap: DetailMapConfiguration;
        protected _cacheHasRenderTargetTextures: boolean;
        /**
         * Instantiates a new PBRMaterial instance.
         *
         * @param name The material name
         * @param scene The scene the material will be use in.
         */
        constructor(name: string, scene?: Scene);
        /**
         * Gets a boolean indicating that current material needs to register RTT
         */
        get hasRenderTargetTextures(): boolean;
        /**
         * Can this material render to prepass
         */
        get isPrePassCapable(): boolean;
        /**
         * Gets the name of the material class.
         */
        getClassName(): string;
        /**
         * Enabled the use of logarithmic depth buffers, which is good for wide depth buffers.
         */
        get useLogarithmicDepth(): boolean;
        /**
         * Enabled the use of logarithmic depth buffers, which is good for wide depth buffers.
         */
        set useLogarithmicDepth(value: boolean);
        /**
         * Returns true if alpha blending should be disabled.
         */
        protected get _disableAlphaBlending(): boolean;
        /**
         * Specifies whether or not this material should be rendered in alpha blend mode.
         */
        needAlphaBlending(): boolean;
        /**
         * Specifies whether or not this material should be rendered in alpha test mode.
         */
        needAlphaTesting(): boolean;
        /**
         * Specifies whether or not the alpha value of the albedo texture should be used for alpha blending.
         */
        protected _shouldUseAlphaFromAlbedoTexture(): boolean;
        /**
         * Specifies whether or not there is a usable alpha channel for transparency.
         */
        protected _hasAlphaChannel(): boolean;
        /**
         * Gets the texture used for the alpha test.
         */
        getAlphaTestTexture(): Nullable<BaseTexture>;
        /**
         * Specifies that the submesh is ready to be used.
         * @param mesh - BJS mesh.
         * @param subMesh - A submesh of the BJS mesh.  Used to check if it is ready.
         * @param useInstances - Specifies that instances should be used.
         * @returns - boolean indicating that the submesh is ready or not.
         */
        isReadyForSubMesh(mesh: AbstractMesh, subMesh: SubMesh, useInstances?: boolean): boolean;
        /**
         * Specifies if the material uses metallic roughness workflow.
         * @returns boolean specifying if the material uses metallic roughness workflow.
         */
        isMetallicWorkflow(): boolean;
        private _prepareEffect;
        private _prepareDefines;
        /**
         * Force shader compilation
         * @param mesh
         * @param onCompiled
         * @param options
         */
        forceCompilation(mesh: AbstractMesh, onCompiled?: (material: Material) => void, options?: Partial<IMaterialCompilationOptions>): void;
        /**
         * Initializes the uniform buffer layout for the shader.
         */
        buildUniformLayout(): void;
        /**
         * Binds the submesh data.
         * @param world - The world matrix.
         * @param mesh - The BJS mesh.
         * @param subMesh - A submesh of the BJS mesh.
         */
        bindForSubMesh(world: Matrix, mesh: Mesh, subMesh: SubMesh): void;
        /**
         * Returns the animatable textures.
         * If material have animatable metallic texture, then reflectivity texture will not be returned, even if it has animations.
         * @returns - Array of animatable textures.
         */
        getAnimatables(): IAnimatable[];
        /**
         * Returns the texture used for reflections.
         * @returns - Reflection texture if present.  Otherwise, returns the environment texture.
         */
        private _getReflectionTexture;
        /**
         * Returns an array of the actively used textures.
         * @returns - Array of BaseTextures
         */
        getActiveTextures(): BaseTexture[];
        /**
         * Checks to see if a texture is used in the material.
         * @param texture - Base texture to use.
         * @returns - Boolean specifying if a texture is used in the material.
         */
        hasTexture(texture: BaseTexture): boolean;
        /**
         * Sets the required values to the prepass renderer.
         * It can't be sets when subsurface scattering of this material is disabled.
         * When scene have ability to enable subsurface prepass effect, it will enable.
         */
        setPrePassRenderer(): boolean;
        /**
         * Disposes the resources of the material.
         * @param forceDisposeEffect - Forces the disposal of effects.
         * @param forceDisposeTextures - Forces the disposal of all textures.
         */
        dispose(forceDisposeEffect?: boolean, forceDisposeTextures?: boolean): void;
    }


    /**
     * The Physically based simple base material of BJS.
     *
     * This enables better naming and convention enforcements on top of the pbrMaterial.
     * It is used as the base class for both the specGloss and metalRough conventions.
     */
    export abstract class PBRBaseSimpleMaterial extends PBRBaseMaterial {
        /**
         * Number of Simultaneous lights allowed on the material.
         */
        maxSimultaneousLights: number;
        /**
         * If sets to true, disables all the lights affecting the material.
         */
        disableLighting: boolean;
        /**
         * Environment Texture used in the material (this is use for both reflection and environment lighting).
         */
        environmentTexture: Nullable<BaseTexture>;
        /**
         * If sets to true, x component of normal map value will invert (x = 1.0 - x).
         */
        invertNormalMapX: boolean;
        /**
         * If sets to true, y component of normal map value will invert (y = 1.0 - y).
         */
        invertNormalMapY: boolean;
        /**
         * Normal map used in the model.
         */
        normalTexture: Nullable<BaseTexture>;
        /**
         * Emissivie color used to self-illuminate the model.
         */
        emissiveColor: Color3;
        /**
         * Emissivie texture used to self-illuminate the model.
         */
        emissiveTexture: Nullable<BaseTexture>;
        /**
         * Occlusion Channel Strength.
         */
        occlusionStrength: number;
        /**
         * Occlusion Texture of the material (adding extra occlusion effects).
         */
        occlusionTexture: Nullable<BaseTexture>;
        /**
         * Defines the alpha limits in alpha test mode.
         */
        alphaCutOff: number;
        /**
         * Gets the current double sided mode.
         */
        get doubleSided(): boolean;
        /**
         * If sets to true and backfaceCulling is false, normals will be flipped on the backside.
         */
        set doubleSided(value: boolean);
        /**
         * Stores the pre-calculated light information of a mesh in a texture.
         */
        lightmapTexture: Nullable<BaseTexture>;
        /**
         * If true, the light map contains occlusion information instead of lighting info.
         */
        useLightmapAsShadowmap: boolean;
        /**
         * Instantiates a new PBRMaterial instance.
         *
         * @param name The material name
         * @param scene The scene the material will be use in.
         */
        constructor(name: string, scene?: Scene);
        getClassName(): string;
    }


    /**
     * @internal
     */
    export class MaterialBRDFDefines extends MaterialDefines {
        BRDF_V_HEIGHT_CORRELATED: boolean;
        MS_BRDF_ENERGY_CONSERVATION: boolean;
        SPHERICAL_HARMONICS: boolean;
        SPECULAR_GLOSSINESS_ENERGY_CONSERVATION: boolean;
    }
    /**
     * Plugin that implements the BRDF component of the PBR material
     */
    export class PBRBRDFConfiguration extends MaterialPluginBase {
        /**
         * Default value used for the energy conservation.
         * This should only be changed to adapt to the type of texture in scene.environmentBRDFTexture.
         */
        static DEFAULT_USE_ENERGY_CONSERVATION: boolean;
        /**
         * Default value used for the Smith Visibility Height Correlated mode.
         * This should only be changed to adapt to the type of texture in scene.environmentBRDFTexture.
         */
        static DEFAULT_USE_SMITH_VISIBILITY_HEIGHT_CORRELATED: boolean;
        /**
         * Default value used for the IBL diffuse part.
         * This can help switching back to the polynomials mode globally which is a tiny bit
         * less GPU intensive at the drawback of a lower quality.
         */
        static DEFAULT_USE_SPHERICAL_HARMONICS: boolean;
        /**
         * Default value used for activating energy conservation for the specular workflow.
         * If activated, the albedo color is multiplied with (1. - maxChannel(specular color)).
         * If deactivated, a material is only physically plausible, when (albedo color + specular color) < 1.
         */
        static DEFAULT_USE_SPECULAR_GLOSSINESS_INPUT_ENERGY_CONSERVATION: boolean;
        private _useEnergyConservation;
        /**
         * Defines if the material uses energy conservation.
         */
        useEnergyConservation: boolean;
        private _useSmithVisibilityHeightCorrelated;
        /**
         * LEGACY Mode set to false
         * Defines if the material uses height smith correlated visibility term.
         * If you intent to not use our default BRDF, you need to load a separate BRDF Texture for the PBR
         * You can either load https://assets.babylonjs.com/environments/uncorrelatedBRDF.png
         * or https://assets.babylonjs.com/environments/uncorrelatedBRDF.dds to have more precision
         * Not relying on height correlated will also disable energy conservation.
         */
        useSmithVisibilityHeightCorrelated: boolean;
        private _useSphericalHarmonics;
        /**
         * LEGACY Mode set to false
         * Defines if the material uses spherical harmonics vs spherical polynomials for the
         * diffuse part of the IBL.
         * The harmonics despite a tiny bigger cost has been proven to provide closer results
         * to the ground truth.
         */
        useSphericalHarmonics: boolean;
        private _useSpecularGlossinessInputEnergyConservation;
        /**
         * Defines if the material uses energy conservation, when the specular workflow is active.
         * If activated, the albedo color is multiplied with (1. - maxChannel(specular color)).
         * If deactivated, a material is only physically plausible, when (albedo color + specular color) < 1.
         * In the deactivated case, the material author has to ensure energy conservation, for a physically plausible rendering.
         */
        useSpecularGlossinessInputEnergyConservation: boolean;
        /** @internal */
        private _internalMarkAllSubMeshesAsMiscDirty;
        /** @internal */
        _markAllSubMeshesAsMiscDirty(): void;
        constructor(material: PBRBaseMaterial, addToPluginList?: boolean);
        prepareDefines(defines: MaterialBRDFDefines): void;
        getClassName(): string;
    }


    /**
     * @internal
     */
    export class MaterialClearCoatDefines extends MaterialDefines {
        CLEARCOAT: boolean;
        CLEARCOAT_DEFAULTIOR: boolean;
        CLEARCOAT_TEXTURE: boolean;
        CLEARCOAT_TEXTURE_ROUGHNESS: boolean;
        CLEARCOAT_TEXTUREDIRECTUV: number;
        CLEARCOAT_TEXTURE_ROUGHNESSDIRECTUV: number;
        CLEARCOAT_BUMP: boolean;
        CLEARCOAT_BUMPDIRECTUV: number;
        CLEARCOAT_USE_ROUGHNESS_FROM_MAINTEXTURE: boolean;
        CLEARCOAT_TEXTURE_ROUGHNESS_IDENTICAL: boolean;
        CLEARCOAT_REMAP_F0: boolean;
        CLEARCOAT_TINT: boolean;
        CLEARCOAT_TINT_TEXTURE: boolean;
        CLEARCOAT_TINT_TEXTUREDIRECTUV: number;
        CLEARCOAT_TINT_GAMMATEXTURE: boolean;
    }
    /**
     * Plugin that implements the clear coat component of the PBR material
     */
    export class PBRClearCoatConfiguration extends MaterialPluginBase {
        protected _material: PBRBaseMaterial;
        /**
         * This defaults to 1.5 corresponding to a 0.04 f0 or a 4% reflectance at normal incidence
         * The default fits with a polyurethane material.
         * @internal
         */
        static readonly _DefaultIndexOfRefraction = 1.5;
        private _isEnabled;
        /**
         * Defines if the clear coat is enabled in the material.
         */
        isEnabled: boolean;
        /**
         * Defines the clear coat layer strength (between 0 and 1) it defaults to 1.
         */
        intensity: number;
        /**
         * Defines the clear coat layer roughness.
         */
        roughness: number;
        private _indexOfRefraction;
        /**
         * Defines the index of refraction of the clear coat.
         * This defaults to 1.5 corresponding to a 0.04 f0 or a 4% reflectance at normal incidence
         * The default fits with a polyurethane material.
         * Changing the default value is more performance intensive.
         */
        indexOfRefraction: number;
        private _texture;
        /**
         * Stores the clear coat values in a texture (red channel is intensity and green channel is roughness)
         * If useRoughnessFromMainTexture is false, the green channel of texture is not used and the green channel of textureRoughness is used instead
         * if textureRoughness is not empty, else no texture roughness is used
         */
        texture: Nullable<BaseTexture>;
        private _useRoughnessFromMainTexture;
        /**
         * Indicates that the green channel of the texture property will be used for roughness (default: true)
         * If false, the green channel from textureRoughness is used for roughness
         */
        useRoughnessFromMainTexture: boolean;
        private _textureRoughness;
        /**
         * Stores the clear coat roughness in a texture (green channel)
         * Not used if useRoughnessFromMainTexture is true
         */
        textureRoughness: Nullable<BaseTexture>;
        private _remapF0OnInterfaceChange;
        /**
         * Defines if the F0 value should be remapped to account for the interface change in the material.
         */
        remapF0OnInterfaceChange: boolean;
        private _bumpTexture;
        /**
         * Define the clear coat specific bump texture.
         */
        bumpTexture: Nullable<BaseTexture>;
        private _isTintEnabled;
        /**
         * Defines if the clear coat tint is enabled in the material.
         */
        isTintEnabled: boolean;
        /**
         * Defines the clear coat tint of the material.
         * This is only use if tint is enabled
         */
        tintColor: Color3;
        /**
         * Defines the distance at which the tint color should be found in the
         * clear coat media.
         * This is only use if tint is enabled
         */
        tintColorAtDistance: number;
        /**
         * Defines the clear coat layer thickness.
         * This is only use if tint is enabled
         */
        tintThickness: number;
        private _tintTexture;
        /**
         * Stores the clear tint values in a texture.
         * rgb is tint
         * a is a thickness factor
         */
        tintTexture: Nullable<BaseTexture>;
        /** @internal */
        private _internalMarkAllSubMeshesAsTexturesDirty;
        /** @internal */
        _markAllSubMeshesAsTexturesDirty(): void;
        constructor(material: PBRBaseMaterial, addToPluginList?: boolean);
        isReadyForSubMesh(defines: MaterialClearCoatDefines, scene: Scene, engine: Engine): boolean;
        prepareDefinesBeforeAttributes(defines: MaterialClearCoatDefines, scene: Scene): void;
        bindForSubMesh(uniformBuffer: UniformBuffer, scene: Scene, engine: Engine, subMesh: SubMesh): void;
        hasTexture(texture: BaseTexture): boolean;
        getActiveTextures(activeTextures: BaseTexture[]): void;
        getAnimatables(animatables: IAnimatable[]): void;
        dispose(forceDisposeTextures?: boolean): void;
        getClassName(): string;
        addFallbacks(defines: MaterialClearCoatDefines, fallbacks: EffectFallbacks, currentRank: number): number;
        getSamplers(samplers: string[]): void;
        getUniforms(): {
            ubo?: Array<{
                name: string;
                size: number;
                type: string;
            }>;
            vertex?: string;
            fragment?: string;
        };
    }


    /**
     * @internal
     */
    export class MaterialIridescenceDefines extends MaterialDefines {
        IRIDESCENCE: boolean;
        IRIDESCENCE_TEXTURE: boolean;
        IRIDESCENCE_TEXTUREDIRECTUV: number;
        IRIDESCENCE_THICKNESS_TEXTURE: boolean;
        IRIDESCENCE_THICKNESS_TEXTUREDIRECTUV: number;
        IRIDESCENCE_USE_THICKNESS_FROM_MAINTEXTURE: boolean;
    }
    /**
     * Plugin that implements the iridescence (thin film) component of the PBR material
     */
    export class PBRIridescenceConfiguration extends MaterialPluginBase {
        protected _material: PBRBaseMaterial;
        /**
         * The default minimum thickness of the thin-film layer given in nanometers (nm).
         * Defaults to 100 nm.
         * @internal
         */
        static readonly _DefaultMinimumThickness = 100;
        /**
         * The default maximum thickness of the thin-film layer given in nanometers (nm).
         * Defaults to 400 nm.
         * @internal
         */
        static readonly _DefaultMaximumThickness = 400;
        /**
         * The default index of refraction of the thin-film layer.
         * Defaults to 1.3
         * @internal
         */
        static readonly _DefaultIndexOfRefraction = 1.3;
        private _isEnabled;
        /**
         * Defines if the iridescence is enabled in the material.
         */
        isEnabled: boolean;
        /**
         * Defines the iridescence layer strength (between 0 and 1) it defaults to 1.
         */
        intensity: number;
        /**
         * Defines the minimum thickness of the thin-film layer given in nanometers (nm).
         */
        minimumThickness: number;
        /**
         * Defines the maximum thickness of the thin-film layer given in nanometers (nm). This will be the thickness used if not thickness texture has been set.
         */
        maximumThickness: number;
        /**
         * Defines the maximum thickness of the thin-film layer given in nanometers (nm).
         */
        indexOfRefraction: number;
        private _texture;
        /**
         * Stores the iridescence intensity in a texture (red channel)
         */
        texture: Nullable<BaseTexture>;
        private _thicknessTexture;
        /**
         * Stores the iridescence thickness in a texture (green channel)
         */
        thicknessTexture: Nullable<BaseTexture>;
        /** @internal */
        private _internalMarkAllSubMeshesAsTexturesDirty;
        /** @internal */
        _markAllSubMeshesAsTexturesDirty(): void;
        constructor(material: PBRBaseMaterial, addToPluginList?: boolean);
        isReadyForSubMesh(defines: MaterialIridescenceDefines, scene: Scene): boolean;
        prepareDefinesBeforeAttributes(defines: MaterialIridescenceDefines, scene: Scene): void;
        bindForSubMesh(uniformBuffer: UniformBuffer, scene: Scene, engine: Engine, subMesh: SubMesh): void;
        hasTexture(texture: BaseTexture): boolean;
        getActiveTextures(activeTextures: BaseTexture[]): void;
        getAnimatables(animatables: IAnimatable[]): void;
        dispose(forceDisposeTextures?: boolean): void;
        getClassName(): string;
        addFallbacks(defines: MaterialIridescenceDefines, fallbacks: EffectFallbacks, currentRank: number): number;
        getSamplers(samplers: string[]): void;
        getUniforms(): {
            ubo?: Array<{
                name: string;
                size: number;
                type: string;
            }>;
            vertex?: string;
            fragment?: string;
        };
    }


    /**
     * The Physically based material of BJS.
     *
     * This offers the main features of a standard PBR material.
     * For more information, please refer to the documentation :
     * https://doc.babylonjs.com/features/featuresDeepDive/materials/using/introToPBR
     */
    export class PBRMaterial extends PBRBaseMaterial {
        /**
         * PBRMaterialTransparencyMode: No transparency mode, Alpha channel is not use.
         */
        static readonly PBRMATERIAL_OPAQUE = 0;
        /**
         * PBRMaterialTransparencyMode: Alpha Test mode, pixel are discarded below a certain threshold defined by the alpha cutoff value.
         */
        static readonly PBRMATERIAL_ALPHATEST = 1;
        /**
         * PBRMaterialTransparencyMode: Pixels are blended (according to the alpha mode) with the already drawn pixels in the current frame buffer.
         */
        static readonly PBRMATERIAL_ALPHABLEND = 2;
        /**
         * PBRMaterialTransparencyMode: Pixels are blended (according to the alpha mode) with the already drawn pixels in the current frame buffer.
         * They are also discarded below the alpha cutoff threshold to improve performances.
         */
        static readonly PBRMATERIAL_ALPHATESTANDBLEND = 3;
        /**
         * Defines the default value of how much AO map is occluding the analytical lights
         * (point spot...).
         */
        static DEFAULT_AO_ON_ANALYTICAL_LIGHTS: number;
        /**
         * Intensity of the direct lights e.g. the four lights available in your scene.
         * This impacts both the direct diffuse and specular highlights.
         */
        directIntensity: number;
        /**
         * Intensity of the emissive part of the material.
         * This helps controlling the emissive effect without modifying the emissive color.
         */
        emissiveIntensity: number;
        /**
         * Intensity of the environment e.g. how much the environment will light the object
         * either through harmonics for rough material or through the reflection for shiny ones.
         */
        environmentIntensity: number;
        /**
         * This is a special control allowing the reduction of the specular highlights coming from the
         * four lights of the scene. Those highlights may not be needed in full environment lighting.
         */
        specularIntensity: number;
        /**
         * Debug Control allowing disabling the bump map on this material.
         */
        disableBumpMap: boolean;
        /**
         * AKA Diffuse Texture in standard nomenclature.
         */
        albedoTexture: Nullable<BaseTexture>;
        /**
         * AKA Occlusion Texture in other nomenclature.
         */
        ambientTexture: Nullable<BaseTexture>;
        /**
         * AKA Occlusion Texture Intensity in other nomenclature.
         */
        ambientTextureStrength: number;
        /**
         * Defines how much the AO map is occluding the analytical lights (point spot...).
         * 1 means it completely occludes it
         * 0 mean it has no impact
         */
        ambientTextureImpactOnAnalyticalLights: number;
        /**
         * Stores the alpha values in a texture. Use luminance if texture.getAlphaFromRGB is true.
         */
        opacityTexture: Nullable<BaseTexture>;
        /**
         * Stores the reflection values in a texture.
         */
        reflectionTexture: Nullable<BaseTexture>;
        /**
         * Stores the emissive values in a texture.
         */
        emissiveTexture: Nullable<BaseTexture>;
        /**
         * AKA Specular texture in other nomenclature.
         */
        reflectivityTexture: Nullable<BaseTexture>;
        /**
         * Used to switch from specular/glossiness to metallic/roughness workflow.
         */
        metallicTexture: Nullable<BaseTexture>;
        /**
         * Specifies the metallic scalar of the metallic/roughness workflow.
         * Can also be used to scale the metalness values of the metallic texture.
         */
        metallic: Nullable<number>;
        /**
         * Specifies the roughness scalar of the metallic/roughness workflow.
         * Can also be used to scale the roughness values of the metallic texture.
         */
        roughness: Nullable<number>;
        /**
         * In metallic workflow, specifies an F0 factor to help configuring the material F0.
         * By default the indexOfrefraction is used to compute F0;
         *
         * This is used as a factor against the default reflectance at normal incidence to tweak it.
         *
         * F0 = defaultF0 * metallicF0Factor * metallicReflectanceColor;
         * F90 = metallicReflectanceColor;
         */
        metallicF0Factor: number;
        /**
         * In metallic workflow, specifies an F90 color to help configuring the material F90.
         * By default the F90 is always 1;
         *
         * Please note that this factor is also used as a factor against the default reflectance at normal incidence.
         *
         * F0 = defaultF0 * metallicF0Factor * metallicReflectanceColor
         * F90 = metallicReflectanceColor;
         */
        metallicReflectanceColor: Color3;
        /**
         * Specifies that only the A channel from metallicReflectanceTexture should be used.
         * If false, both RGB and A channels will be used
         */
        useOnlyMetallicFromMetallicReflectanceTexture: boolean;
        /**
         * Defines to store metallicReflectanceColor in RGB and metallicF0Factor in A
         * This is multiplied against the scalar values defined in the material.
         * If useOnlyMetallicFromMetallicReflectanceTexture is true, don't use the RGB channels, only A
         */
        metallicReflectanceTexture: Nullable<BaseTexture>;
        /**
         * Defines to store reflectanceColor in RGB
         * This is multiplied against the scalar values defined in the material.
         * If both reflectanceTexture and metallicReflectanceTexture textures are provided and useOnlyMetallicFromMetallicReflectanceTexture
         * is false, metallicReflectanceTexture takes priority and reflectanceTexture is not used
         */
        reflectanceTexture: Nullable<BaseTexture>;
        /**
         * Used to enable roughness/glossiness fetch from a separate channel depending on the current mode.
         * Gray Scale represents roughness in metallic mode and glossiness in specular mode.
         */
        microSurfaceTexture: Nullable<BaseTexture>;
        /**
         * Stores surface normal data used to displace a mesh in a texture.
         */
        bumpTexture: Nullable<BaseTexture>;
        /**
         * Stores the pre-calculated light information of a mesh in a texture.
         */
        lightmapTexture: Nullable<BaseTexture>;
        /**
         * Stores the refracted light information in a texture.
         */
        get refractionTexture(): Nullable<BaseTexture>;
        set refractionTexture(value: Nullable<BaseTexture>);
        /**
         * The color of a material in ambient lighting.
         */
        ambientColor: Color3;
        /**
         * AKA Diffuse Color in other nomenclature.
         */
        albedoColor: Color3;
        /**
         * AKA Specular Color in other nomenclature.
         */
        reflectivityColor: Color3;
        /**
         * The color reflected from the material.
         */
        reflectionColor: Color3;
        /**
         * The color emitted from the material.
         */
        emissiveColor: Color3;
        /**
         * AKA Glossiness in other nomenclature.
         */
        microSurface: number;
        /**
         * Index of refraction of the material base layer.
         * https://en.wikipedia.org/wiki/List_of_refractive_indices
         *
         * This does not only impact refraction but also the Base F0 of Dielectric Materials.
         *
         * From dielectric fresnel rules: F0 = square((iorT - iorI) / (iorT + iorI))
         */
        get indexOfRefraction(): number;
        set indexOfRefraction(value: number);
        /**
         * Controls if refraction needs to be inverted on Y. This could be useful for procedural texture.
         */
        get invertRefractionY(): boolean;
        set invertRefractionY(value: boolean);
        /**
         * This parameters will make the material used its opacity to control how much it is refracting against not.
         * Materials half opaque for instance using refraction could benefit from this control.
         */
        get linkRefractionWithTransparency(): boolean;
        set linkRefractionWithTransparency(value: boolean);
        /**
         * If true, the light map contains occlusion information instead of lighting info.
         */
        useLightmapAsShadowmap: boolean;
        /**
         * Specifies that the alpha is coming form the albedo channel alpha channel for alpha blending.
         */
        useAlphaFromAlbedoTexture: boolean;
        /**
         * Enforces alpha test in opaque or blend mode in order to improve the performances of some situations.
         */
        forceAlphaTest: boolean;
        /**
         * Defines the alpha limits in alpha test mode.
         */
        alphaCutOff: number;
        /**
         * Specifies that the material will keep the specular highlights over a transparent surface (only the most luminous ones).
         * A car glass is a good example of that. When sun reflects on it you can not see what is behind.
         */
        useSpecularOverAlpha: boolean;
        /**
         * Specifies if the reflectivity texture contains the glossiness information in its alpha channel.
         */
        useMicroSurfaceFromReflectivityMapAlpha: boolean;
        /**
         * Specifies if the metallic texture contains the roughness information in its alpha channel.
         */
        useRoughnessFromMetallicTextureAlpha: boolean;
        /**
         * Specifies if the metallic texture contains the roughness information in its green channel.
         */
        useRoughnessFromMetallicTextureGreen: boolean;
        /**
         * Specifies if the metallic texture contains the metallness information in its blue channel.
         */
        useMetallnessFromMetallicTextureBlue: boolean;
        /**
         * Specifies if the metallic texture contains the ambient occlusion information in its red channel.
         */
        useAmbientOcclusionFromMetallicTextureRed: boolean;
        /**
         * Specifies if the ambient texture contains the ambient occlusion information in its red channel only.
         */
        useAmbientInGrayScale: boolean;
        /**
         * In case the reflectivity map does not contain the microsurface information in its alpha channel,
         * The material will try to infer what glossiness each pixel should be.
         */
        useAutoMicroSurfaceFromReflectivityMap: boolean;
        /**
         * BJS is using an hardcoded light falloff based on a manually sets up range.
         * In PBR, one way to represents the falloff is to use the inverse squared root algorithm.
         * This parameter can help you switch back to the BJS mode in order to create scenes using both materials.
         */
        get usePhysicalLightFalloff(): boolean;
        /**
         * BJS is using an hardcoded light falloff based on a manually sets up range.
         * In PBR, one way to represents the falloff is to use the inverse squared root algorithm.
         * This parameter can help you switch back to the BJS mode in order to create scenes using both materials.
         */
        set usePhysicalLightFalloff(value: boolean);
        /**
         * In order to support the falloff compatibility with gltf, a special mode has been added
         * to reproduce the gltf light falloff.
         */
        get useGLTFLightFalloff(): boolean;
        /**
         * In order to support the falloff compatibility with gltf, a special mode has been added
         * to reproduce the gltf light falloff.
         */
        set useGLTFLightFalloff(value: boolean);
        /**
         * Specifies that the material will keeps the reflection highlights over a transparent surface (only the most luminous ones).
         * A car glass is a good example of that. When the street lights reflects on it you can not see what is behind.
         */
        useRadianceOverAlpha: boolean;
        /**
         * Allows using an object space normal map (instead of tangent space).
         */
        useObjectSpaceNormalMap: boolean;
        /**
         * Allows using the bump map in parallax mode.
         */
        useParallax: boolean;
        /**
         * Allows using the bump map in parallax occlusion mode.
         */
        useParallaxOcclusion: boolean;
        /**
         * Controls the scale bias of the parallax mode.
         */
        parallaxScaleBias: number;
        /**
         * If sets to true, disables all the lights affecting the material.
         */
        disableLighting: boolean;
        /**
         * Force the shader to compute irradiance in the fragment shader in order to take bump in account.
         */
        forceIrradianceInFragment: boolean;
        /**
         * Number of Simultaneous lights allowed on the material.
         */
        maxSimultaneousLights: number;
        /**
         * If sets to true, x component of normal map value will invert (x = 1.0 - x).
         */
        invertNormalMapX: boolean;
        /**
         * If sets to true, y component of normal map value will invert (y = 1.0 - y).
         */
        invertNormalMapY: boolean;
        /**
         * If sets to true and backfaceCulling is false, normals will be flipped on the backside.
         */
        twoSidedLighting: boolean;
        /**
         * A fresnel is applied to the alpha of the model to ensure grazing angles edges are not alpha tested.
         * And/Or occlude the blended part. (alpha is converted to gamma to compute the fresnel)
         */
        useAlphaFresnel: boolean;
        /**
         * A fresnel is applied to the alpha of the model to ensure grazing angles edges are not alpha tested.
         * And/Or occlude the blended part. (alpha stays linear to compute the fresnel)
         */
        useLinearAlphaFresnel: boolean;
        /**
         * Let user defines the brdf lookup texture used for IBL.
         * A default 8bit version is embedded but you could point at :
         * * Default texture: https://assets.babylonjs.com/environments/correlatedMSBRDF_RGBD.png
         * * Default 16bit pixel depth texture: https://assets.babylonjs.com/environments/correlatedMSBRDF.dds
         * * LEGACY Default None correlated https://assets.babylonjs.com/environments/uncorrelatedBRDF_RGBD.png
         * * LEGACY Default None correlated 16bit pixel depth https://assets.babylonjs.com/environments/uncorrelatedBRDF.dds
         */
        environmentBRDFTexture: Nullable<BaseTexture>;
        /**
         * Force normal to face away from face.
         */
        forceNormalForward: boolean;
        /**
         * Enables specular anti aliasing in the PBR shader.
         * It will both interacts on the Geometry for analytical and IBL lighting.
         * It also prefilter the roughness map based on the bump values.
         */
        enableSpecularAntiAliasing: boolean;
        /**
         * This parameters will enable/disable Horizon occlusion to prevent normal maps to look shiny when the normal
         * makes the reflect vector face the model (under horizon).
         */
        useHorizonOcclusion: boolean;
        /**
         * This parameters will enable/disable radiance occlusion by preventing the radiance to lit
         * too much the area relying on ambient texture to define their ambient occlusion.
         */
        useRadianceOcclusion: boolean;
        /**
         * If set to true, no lighting calculations will be applied.
         */
        unlit: boolean;
        /**
         * Gets the image processing configuration used either in this material.
         */
        get imageProcessingConfiguration(): ImageProcessingConfiguration;
        /**
         * Sets the Default image processing configuration used either in the this material.
         *
         * If sets to null, the scene one is in use.
         */
        set imageProcessingConfiguration(value: ImageProcessingConfiguration);
        /**
         * Gets whether the color curves effect is enabled.
         */
        get cameraColorCurvesEnabled(): boolean;
        /**
         * Sets whether the color curves effect is enabled.
         */
        set cameraColorCurvesEnabled(value: boolean);
        /**
         * Gets whether the color grading effect is enabled.
         */
        get cameraColorGradingEnabled(): boolean;
        /**
         * Gets whether the color grading effect is enabled.
         */
        set cameraColorGradingEnabled(value: boolean);
        /**
         * Gets whether tonemapping is enabled or not.
         */
        get cameraToneMappingEnabled(): boolean;
        /**
         * Sets whether tonemapping is enabled or not
         */
        set cameraToneMappingEnabled(value: boolean);
        /**
         * The camera exposure used on this material.
         * This property is here and not in the camera to allow controlling exposure without full screen post process.
         * This corresponds to a photographic exposure.
         */
        get cameraExposure(): number;
        /**
         * The camera exposure used on this material.
         * This property is here and not in the camera to allow controlling exposure without full screen post process.
         * This corresponds to a photographic exposure.
         */
        set cameraExposure(value: number);
        /**
         * Gets The camera contrast used on this material.
         */
        get cameraContrast(): number;
        /**
         * Sets The camera contrast used on this material.
         */
        set cameraContrast(value: number);
        /**
         * Gets the Color Grading 2D Lookup Texture.
         */
        get cameraColorGradingTexture(): Nullable<BaseTexture>;
        /**
         * Sets the Color Grading 2D Lookup Texture.
         */
        set cameraColorGradingTexture(value: Nullable<BaseTexture>);
        /**
         * The color grading curves provide additional color adjustment that is applied after any color grading transform (3D LUT).
         * They allow basic adjustment of saturation and small exposure adjustments, along with color filter tinting to provide white balance adjustment or more stylistic effects.
         * These are similar to controls found in many professional imaging or colorist software. The global controls are applied to the entire image. For advanced tuning, extra controls are provided to adjust the shadow, midtone and highlight areas of the image;
         * corresponding to low luminance, medium luminance, and high luminance areas respectively.
         */
        get cameraColorCurves(): Nullable<ColorCurves>;
        /**
         * The color grading curves provide additional color adjustment that is applied after any color grading transform (3D LUT).
         * They allow basic adjustment of saturation and small exposure adjustments, along with color filter tinting to provide white balance adjustment or more stylistic effects.
         * These are similar to controls found in many professional imaging or colorist software. The global controls are applied to the entire image. For advanced tuning, extra controls are provided to adjust the shadow, midtone and highlight areas of the image;
         * corresponding to low luminance, medium luminance, and high luminance areas respectively.
         */
        set cameraColorCurves(value: Nullable<ColorCurves>);
        /**
         * Instantiates a new PBRMaterial instance.
         *
         * @param name The material name
         * @param scene The scene the material will be use in.
         */
        constructor(name: string, scene?: Scene);
        /**
         * Returns the name of this material class.
         */
        getClassName(): string;
        /**
         * Makes a duplicate of the current material.
         * @param name - name to use for the new material.
         */
        clone(name: string): PBRMaterial;
        /**
         * Serializes this PBR Material.
         * @returns - An object with the serialized material.
         */
        serialize(): any;
        /**
         * Parses a PBR Material from a serialized object.
         * @param source - Serialized object.
         * @param scene - BJS scene instance.
         * @param rootUrl - url for the scene object
         * @returns - PBRMaterial
         */
        static Parse(source: any, scene: Scene, rootUrl: string): PBRMaterial;
    }


    /**
     * The PBR material of BJS following the metal roughness convention.
     *
     * This fits to the PBR convention in the GLTF definition:
     * https://github.com/KhronosGroup/glTF/blob/main/extensions/2.0/Archived/KHR_materials_pbrSpecularGlossiness/README.md
     */
    export class PBRMetallicRoughnessMaterial extends PBRBaseSimpleMaterial {
        /**
         * The base color has two different interpretations depending on the value of metalness.
         * When the material is a metal, the base color is the specific measured reflectance value
         * at normal incidence (F0). For a non-metal the base color represents the reflected diffuse color
         * of the material.
         */
        baseColor: Color3;
        /**
         * Base texture of the metallic workflow. It contains both the baseColor information in RGB as
         * well as opacity information in the alpha channel.
         */
        baseTexture: Nullable<BaseTexture>;
        /**
         * Specifies the metallic scalar value of the material.
         * Can also be used to scale the metalness values of the metallic texture.
         */
        metallic: number;
        /**
         * Specifies the roughness scalar value of the material.
         * Can also be used to scale the roughness values of the metallic texture.
         */
        roughness: number;
        /**
         * Texture containing both the metallic value in the B channel and the
         * roughness value in the G channel to keep better precision.
         */
        metallicRoughnessTexture: Nullable<BaseTexture>;
        /**
         * Instantiates a new PBRMetalRoughnessMaterial instance.
         *
         * @param name The material name
         * @param scene The scene the material will be use in.
         */
        constructor(name: string, scene?: Scene);
        /**
         * Return the current class name of the material.
         */
        getClassName(): string;
        /**
         * Makes a duplicate of the current material.
         * @param name - name to use for the new material.
         */
        clone(name: string): PBRMetallicRoughnessMaterial;
        /**
         * Serialize the material to a parsable JSON object.
         */
        serialize(): any;
        /**
         * Parses a JSON object corresponding to the serialize function.
         * @param source
         * @param scene
         * @param rootUrl
         */
        static Parse(source: any, scene: Scene, rootUrl: string): PBRMetallicRoughnessMaterial;
    }


    /**
     * @internal
     */
    export class MaterialSheenDefines extends MaterialDefines {
        SHEEN: boolean;
        SHEEN_TEXTURE: boolean;
        SHEEN_GAMMATEXTURE: boolean;
        SHEEN_TEXTURE_ROUGHNESS: boolean;
        SHEEN_TEXTUREDIRECTUV: number;
        SHEEN_TEXTURE_ROUGHNESSDIRECTUV: number;
        SHEEN_LINKWITHALBEDO: boolean;
        SHEEN_ROUGHNESS: boolean;
        SHEEN_ALBEDOSCALING: boolean;
        SHEEN_USE_ROUGHNESS_FROM_MAINTEXTURE: boolean;
        SHEEN_TEXTURE_ROUGHNESS_IDENTICAL: boolean;
    }
    /**
     * Plugin that implements the sheen component of the PBR material.
     */
    export class PBRSheenConfiguration extends MaterialPluginBase {
        private _isEnabled;
        /**
         * Defines if the material uses sheen.
         */
        isEnabled: boolean;
        private _linkSheenWithAlbedo;
        /**
         * Defines if the sheen is linked to the sheen color.
         */
        linkSheenWithAlbedo: boolean;
        /**
         * Defines the sheen intensity.
         */
        intensity: number;
        /**
         * Defines the sheen color.
         */
        color: Color3;
        private _texture;
        /**
         * Stores the sheen tint values in a texture.
         * rgb is tint
         * a is a intensity or roughness if the roughness property has been defined and useRoughnessFromTexture is true (in that case, textureRoughness won't be used)
         * If the roughness property has been defined and useRoughnessFromTexture is false then the alpha channel is not used to modulate roughness
         */
        texture: Nullable<BaseTexture>;
        private _useRoughnessFromMainTexture;
        /**
         * Indicates that the alpha channel of the texture property will be used for roughness.
         * Has no effect if the roughness (and texture!) property is not defined
         */
        useRoughnessFromMainTexture: boolean;
        private _roughness;
        /**
         * Defines the sheen roughness.
         * It is not taken into account if linkSheenWithAlbedo is true.
         * To stay backward compatible, material roughness is used instead if sheen roughness = null
         */
        roughness: Nullable<number>;
        private _textureRoughness;
        /**
         * Stores the sheen roughness in a texture.
         * alpha channel is the roughness. This texture won't be used if the texture property is not empty and useRoughnessFromTexture is true
         */
        textureRoughness: Nullable<BaseTexture>;
        private _albedoScaling;
        /**
         * If true, the sheen effect is layered above the base BRDF with the albedo-scaling technique.
         * It allows the strength of the sheen effect to not depend on the base color of the material,
         * making it easier to setup and tweak the effect
         */
        albedoScaling: boolean;
        /** @internal */
        private _internalMarkAllSubMeshesAsTexturesDirty;
        /** @internal */
        _markAllSubMeshesAsTexturesDirty(): void;
        constructor(material: PBRBaseMaterial, addToPluginList?: boolean);
        isReadyForSubMesh(defines: MaterialSheenDefines, scene: Scene): boolean;
        prepareDefinesBeforeAttributes(defines: MaterialSheenDefines, scene: Scene): void;
        bindForSubMesh(uniformBuffer: UniformBuffer, scene: Scene, engine: Engine, subMesh: SubMesh): void;
        hasTexture(texture: BaseTexture): boolean;
        getActiveTextures(activeTextures: BaseTexture[]): void;
        getAnimatables(animatables: IAnimatable[]): void;
        dispose(forceDisposeTextures?: boolean): void;
        getClassName(): string;
        addFallbacks(defines: MaterialSheenDefines, fallbacks: EffectFallbacks, currentRank: number): number;
        getSamplers(samplers: string[]): void;
        getUniforms(): {
            ubo?: Array<{
                name: string;
                size: number;
                type: string;
            }>;
            vertex?: string;
            fragment?: string;
        };
    }


    /**
     * The PBR material of BJS following the specular glossiness convention.
     *
     * This fits to the PBR convention in the GLTF definition:
     * https://github.com/KhronosGroup/glTF/tree/2.0/extensions/Khronos/KHR_materials_pbrSpecularGlossiness
     */
    export class PBRSpecularGlossinessMaterial extends PBRBaseSimpleMaterial {
        /**
         * Specifies the diffuse color of the material.
         */
        diffuseColor: Color3;
        /**
         * Specifies the diffuse texture of the material. This can also contains the opacity value in its alpha
         * channel.
         */
        diffuseTexture: Nullable<BaseTexture>;
        /**
         * Specifies the specular color of the material. This indicates how reflective is the material (none to mirror).
         */
        specularColor: Color3;
        /**
         * Specifies the glossiness of the material. This indicates "how sharp is the reflection".
         */
        glossiness: number;
        /**
         * Specifies both the specular color RGB and the glossiness A of the material per pixels.
         */
        specularGlossinessTexture: Nullable<BaseTexture>;
        /**
         * Specifies if the reflectivity texture contains the glossiness information in its alpha channel.
         */
        get useMicroSurfaceFromReflectivityMapAlpha(): boolean;
        /**
         * Instantiates a new PBRSpecularGlossinessMaterial instance.
         *
         * @param name The material name
         * @param scene The scene the material will be use in.
         */
        constructor(name: string, scene?: Scene);
        /**
         * Return the current class name of the material.
         */
        getClassName(): string;
        /**
         * Makes a duplicate of the current material.
         * @param name - name to use for the new material.
         */
        clone(name: string): PBRSpecularGlossinessMaterial;
        /**
         * Serialize the material to a parsable JSON object.
         */
        serialize(): any;
        /**
         * Parses a JSON object corresponding to the serialize function.
         * @param source
         * @param scene
         * @param rootUrl
         */
        static Parse(source: any, scene: Scene, rootUrl: string): PBRSpecularGlossinessMaterial;
    }


    /**
     * @internal
     */
    export class MaterialSubSurfaceDefines extends MaterialDefines {
        SUBSURFACE: boolean;
        SS_REFRACTION: boolean;
        SS_REFRACTION_USE_INTENSITY_FROM_TEXTURE: boolean;
        SS_TRANSLUCENCY: boolean;
        SS_TRANSLUCENCY_USE_INTENSITY_FROM_TEXTURE: boolean;
        SS_SCATTERING: boolean;
        SS_THICKNESSANDMASK_TEXTURE: boolean;
        SS_THICKNESSANDMASK_TEXTUREDIRECTUV: number;
        SS_HAS_THICKNESS: boolean;
        SS_REFRACTIONINTENSITY_TEXTURE: boolean;
        SS_REFRACTIONINTENSITY_TEXTUREDIRECTUV: number;
        SS_TRANSLUCENCYINTENSITY_TEXTURE: boolean;
        SS_TRANSLUCENCYINTENSITY_TEXTUREDIRECTUV: number;
        SS_REFRACTIONMAP_3D: boolean;
        SS_REFRACTIONMAP_OPPOSITEZ: boolean;
        SS_LODINREFRACTIONALPHA: boolean;
        SS_GAMMAREFRACTION: boolean;
        SS_RGBDREFRACTION: boolean;
        SS_LINEARSPECULARREFRACTION: boolean;
        SS_LINKREFRACTIONTOTRANSPARENCY: boolean;
        SS_ALBEDOFORREFRACTIONTINT: boolean;
        SS_ALBEDOFORTRANSLUCENCYTINT: boolean;
        SS_USE_LOCAL_REFRACTIONMAP_CUBIC: boolean;
        SS_USE_THICKNESS_AS_DEPTH: boolean;
        SS_MASK_FROM_THICKNESS_TEXTURE: boolean;
        SS_USE_GLTF_TEXTURES: boolean;
    }
    /**
     * Plugin that implements the sub surface component of the PBR material
     */
    export class PBRSubSurfaceConfiguration extends MaterialPluginBase {
        protected _material: PBRBaseMaterial;
        private _isRefractionEnabled;
        /**
         * Defines if the refraction is enabled in the material.
         */
        isRefractionEnabled: boolean;
        private _isTranslucencyEnabled;
        /**
         * Defines if the translucency is enabled in the material.
         */
        isTranslucencyEnabled: boolean;
        private _isScatteringEnabled;
        /**
         * Defines if the sub surface scattering is enabled in the material.
         */
        isScatteringEnabled: boolean;
        private _scatteringDiffusionProfileIndex;
        /**
         * Diffusion profile for subsurface scattering.
         * Useful for better scattering in the skins or foliages.
         */
        get scatteringDiffusionProfile(): Nullable<Color3>;
        set scatteringDiffusionProfile(c: Nullable<Color3>);
        /**
         * Defines the refraction intensity of the material.
         * The refraction when enabled replaces the Diffuse part of the material.
         * The intensity helps transitioning between diffuse and refraction.
         */
        refractionIntensity: number;
        /**
         * Defines the translucency intensity of the material.
         * When translucency has been enabled, this defines how much of the "translucency"
         * is added to the diffuse part of the material.
         */
        translucencyIntensity: number;
        /**
         * When enabled, transparent surfaces will be tinted with the albedo colour (independent of thickness)
         */
        useAlbedoToTintRefraction: boolean;
        /**
         * When enabled, translucent surfaces will be tinted with the albedo colour (independent of thickness)
         */
        useAlbedoToTintTranslucency: boolean;
        private _thicknessTexture;
        /**
         * Stores the average thickness of a mesh in a texture (The texture is holding the values linearly).
         * The red (or green if useGltfStyleTextures=true) channel of the texture should contain the thickness remapped between 0 and 1.
         * 0 would mean minimumThickness
         * 1 would mean maximumThickness
         * The other channels might be use as a mask to vary the different effects intensity.
         */
        thicknessTexture: Nullable<BaseTexture>;
        private _refractionTexture;
        /**
         * Defines the texture to use for refraction.
         */
        refractionTexture: Nullable<BaseTexture>;
        /** @internal */
        _indexOfRefraction: number;
        /**
         * Index of refraction of the material base layer.
         * https://en.wikipedia.org/wiki/List_of_refractive_indices
         *
         * This does not only impact refraction but also the Base F0 of Dielectric Materials.
         *
         * From dielectric fresnel rules: F0 = square((iorT - iorI) / (iorT + iorI))
         */
        indexOfRefraction: number;
        private _volumeIndexOfRefraction;
        /**
         * Index of refraction of the material's volume.
         * https://en.wikipedia.org/wiki/List_of_refractive_indices
         *
         * This ONLY impacts refraction. If not provided or given a non-valid value,
         * the volume will use the same IOR as the surface.
         */
        get volumeIndexOfRefraction(): number;
        set volumeIndexOfRefraction(value: number);
        private _invertRefractionY;
        /**
         * Controls if refraction needs to be inverted on Y. This could be useful for procedural texture.
         */
        invertRefractionY: boolean;
        /** @internal */
        _linkRefractionWithTransparency: boolean;
        /**
         * This parameters will make the material used its opacity to control how much it is refracting against not.
         * Materials half opaque for instance using refraction could benefit from this control.
         */
        linkRefractionWithTransparency: boolean;
        /**
         * Defines the minimum thickness stored in the thickness map.
         * If no thickness map is defined, this value will be used to simulate thickness.
         */
        minimumThickness: number;
        /**
         * Defines the maximum thickness stored in the thickness map.
         */
        maximumThickness: number;
        /**
         * Defines that the thickness should be used as a measure of the depth volume.
         */
        useThicknessAsDepth: boolean;
        /**
         * Defines the volume tint of the material.
         * This is used for both translucency and scattering.
         */
        tintColor: Color3;
        /**
         * Defines the distance at which the tint color should be found in the media.
         * This is used for refraction only.
         */
        tintColorAtDistance: number;
        /**
         * Defines how far each channel transmit through the media.
         * It is defined as a color to simplify it selection.
         */
        diffusionDistance: Color3;
        private _useMaskFromThicknessTexture;
        /**
         * Stores the intensity of the different subsurface effects in the thickness texture.
         * Note that if refractionIntensityTexture and/or translucencyIntensityTexture is provided it takes precedence over thicknessTexture + useMaskFromThicknessTexture
         * * the green (red if useGltfStyleTextures = true) channel is the refraction intensity.
         * * the blue channel is the translucency intensity.
         */
        useMaskFromThicknessTexture: boolean;
        private _refractionIntensityTexture;
        /**
         * Stores the intensity of the refraction. If provided, it takes precedence over thicknessTexture + useMaskFromThicknessTexture
         * * the green (red if useGltfStyleTextures = true) channel is the refraction intensity.
         */
        refractionIntensityTexture: Nullable<BaseTexture>;
        private _translucencyIntensityTexture;
        /**
         * Stores the intensity of the translucency. If provided, it takes precedence over thicknessTexture + useMaskFromThicknessTexture
         * * the blue channel is the translucency intensity.
         */
        translucencyIntensityTexture: Nullable<BaseTexture>;
        private _scene;
        private _useGltfStyleTextures;
        /**
         * Use channels layout used by glTF:
         * * thicknessTexture: the green (instead of red) channel is the thickness
         * * thicknessTexture/refractionIntensityTexture: the red (instead of green) channel is the refraction intensity
         * * thicknessTexture/translucencyIntensityTexture: no change, use the blue channel for the translucency intensity
         */
        useGltfStyleTextures: boolean;
        /** @internal */
        private _internalMarkAllSubMeshesAsTexturesDirty;
        private _internalMarkScenePrePassDirty;
        /** @internal */
        _markAllSubMeshesAsTexturesDirty(): void;
        /** @internal */
        _markScenePrePassDirty(): void;
        constructor(material: PBRBaseMaterial, addToPluginList?: boolean);
        isReadyForSubMesh(defines: MaterialSubSurfaceDefines, scene: Scene): boolean;
        prepareDefinesBeforeAttributes(defines: MaterialSubSurfaceDefines, scene: Scene): void;
        /**
         * Binds the material data (this function is called even if mustRebind() returns false)
         * @param uniformBuffer defines the Uniform buffer to fill in.
         * @param scene defines the scene the material belongs to.
         * @param engine defines the engine the material belongs to.
         * @param subMesh the submesh to bind data for
         */
        hardBindForSubMesh(uniformBuffer: UniformBuffer, scene: Scene, engine: Engine, subMesh: SubMesh): void;
        bindForSubMesh(uniformBuffer: UniformBuffer, scene: Scene, engine: Engine, subMesh: SubMesh): void;
        /**
         * Returns the texture used for refraction or null if none is used.
         * @param scene defines the scene the material belongs to.
         * @returns - Refraction texture if present.  If no refraction texture and refraction
         * is linked with transparency, returns environment texture.  Otherwise, returns null.
         */
        private _getRefractionTexture;
        /**
         * Returns true if alpha blending should be disabled.
         */
        get disableAlphaBlending(): boolean;
        /**
         * Fills the list of render target textures.
         * @param renderTargets the list of render targets to update
         */
        fillRenderTargetTextures(renderTargets: SmartArray<RenderTargetTexture>): void;
        hasTexture(texture: BaseTexture): boolean;
        hasRenderTargetTextures(): boolean;
        getActiveTextures(activeTextures: BaseTexture[]): void;
        getAnimatables(animatables: IAnimatable[]): void;
        dispose(forceDisposeTextures?: boolean): void;
        getClassName(): string;
        addFallbacks(defines: MaterialSubSurfaceDefines, fallbacks: EffectFallbacks, currentRank: number): number;
        getSamplers(samplers: string[]): void;
        getUniforms(): {
            ubo?: Array<{
                name: string;
                size: number;
                type: string;
            }>;
            vertex?: string;
            fragment?: string;
        };
    }


    /**
     * Configuration needed for prepass-capable materials
     */
    export class PrePassConfiguration {
        /**
         * Previous world matrices of meshes carrying this material
         * Used for computing velocity
         */
        previousWorldMatrices: {
            [index: number]: Matrix;
        };
        /**
         * Previous view project matrix
         * Used for computing velocity
         */
        previousViewProjection: Matrix;
        /**
         * Current view projection matrix
         * Used for computing velocity
         */
        currentViewProjection: Matrix;
        /**
         * Previous bones of meshes carrying this material
         * Used for computing velocity
         */
        previousBones: {
            [index: number]: Float32Array;
        };
        private _lastUpdateFrameId;
        /**
         * Add the required uniforms to the current list.
         * @param uniforms defines the current uniform list.
         */
        static AddUniforms(uniforms: string[]): void;
        /**
         * Add the required samplers to the current list.
         * @param samplers defines the current sampler list.
         */
        static AddSamplers(samplers: string[]): void;
        /**
         * Binds the material data.
         * @param effect defines the effect to update
         * @param scene defines the scene the material belongs to.
         * @param mesh The mesh
         * @param world World matrix of this mesh
         * @param isFrozen Is the material frozen
         */
        bindForSubMesh(effect: Effect, scene: Scene, mesh: Mesh, world: Matrix, isFrozen: boolean): void;
    }


    /**
     * Base class of materials working in push mode in babylon JS
     * @internal
     */
    export class PushMaterial extends Material {
        protected _activeEffect: Effect;
        protected _normalMatrix: Matrix;
        constructor(name: string, scene?: Scene, storeEffectOnSubMeshes?: boolean);
        getEffect(): Effect;
        isReady(mesh?: AbstractMesh, useInstances?: boolean): boolean;
        protected _isReadyForSubMesh(subMesh: SubMesh): boolean;
        /**
         * Binds the given world matrix to the active effect
         *
         * @param world the matrix to bind
         */
        bindOnlyWorldMatrix(world: Matrix): void;
        /**
         * Binds the given normal matrix to the active effect
         *
         * @param normalMatrix the matrix to bind
         */
        bindOnlyNormalMatrix(normalMatrix: Matrix): void;
        bind(world: Matrix, mesh?: Mesh): void;
        protected _afterBind(mesh?: Mesh, effect?: Nullable<Effect>): void;
        protected _mustRebind(scene: Scene, effect: Effect, visibility?: number): boolean;
    }


    /**
     * Language of the shader code
     */
    export enum ShaderLanguage {
        /** language is GLSL (used by WebGL) */
        GLSL = 0,
        /** language is WGSL (used by WebGPU) */
        WGSL = 1
    }


    /**
     * Defines the options associated with the creation of a shader material.
     */
    export interface IShaderMaterialOptions {
        /**
         * Does the material work in alpha blend mode
         */
        needAlphaBlending: boolean;
        /**
         * Does the material work in alpha test mode
         */
        needAlphaTesting: boolean;
        /**
         * The list of attribute names used in the shader
         */
        attributes: string[];
        /**
         * The list of uniform names used in the shader
         */
        uniforms: string[];
        /**
         * The list of UBO names used in the shader
         */
        uniformBuffers: string[];
        /**
         * The list of sampler (texture) names used in the shader
         */
        samplers: string[];
        /**
         * The list of external texture names used in the shader
         */
        externalTextures: string[];
        /**
         * The list of sampler object names used in the shader
         */
        samplerObjects: string[];
        /**
         * The list of storage buffer names used in the shader
         */
        storageBuffers: string[];
        /**
         * The list of defines used in the shader
         */
        defines: string[];
        /**
         * Defines if clip planes have to be turned on: true to turn them on, false to turn them off and null to turn them on/off depending on the scene configuration (scene.clipPlaneX)
         */
        useClipPlane: Nullable<boolean>;
        /**
         * The language the shader is written in (default: GLSL)
         */
        shaderLanguage?: ShaderLanguage;
    }
    /**
     * The ShaderMaterial object has the necessary methods to pass data from your scene to the Vertex and Fragment Shaders and returns a material that can be applied to any mesh.
     *
     * This returned material effects how the mesh will look based on the code in the shaders.
     *
     * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/shaders/shaderMaterial
     */
    export class ShaderMaterial extends PushMaterial {
        private _shaderPath;
        private _options;
        private _textures;
        private _textureArrays;
        private _externalTextures;
        private _floats;
        private _ints;
        private _uints;
        private _floatsArrays;
        private _colors3;
        private _colors3Arrays;
        private _colors4;
        private _colors4Arrays;
        private _vectors2;
        private _vectors3;
        private _vectors4;
        private _quaternions;
        private _quaternionsArrays;
        private _matrices;
        private _matrixArrays;
        private _matrices3x3;
        private _matrices2x2;
        private _vectors2Arrays;
        private _vectors3Arrays;
        private _vectors4Arrays;
        private _uniformBuffers;
        private _textureSamplers;
        private _storageBuffers;
        private _cachedWorldViewMatrix;
        private _cachedWorldViewProjectionMatrix;
        private _multiview;
        /** Define the Url to load snippets */
        static SnippetUrl: string;
        /** Snippet ID if the material was created from the snippet server */
        snippetId: string;
        /**
         * Instantiate a new shader material.
         * The ShaderMaterial object has the necessary methods to pass data from your scene to the Vertex and Fragment Shaders and returns a material that can be applied to any mesh.
         * This returned material effects how the mesh will look based on the code in the shaders.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/shaders/shaderMaterial
         * @param name Define the name of the material in the scene
         * @param scene Define the scene the material belongs to
         * @param shaderPath Defines  the route to the shader code in one of three ways:
         *  * object: { vertex: "custom", fragment: "custom" }, used with Effect.ShadersStore["customVertexShader"] and Effect.ShadersStore["customFragmentShader"]
         *  * object: { vertexElement: "vertexShaderCode", fragmentElement: "fragmentShaderCode" }, used with shader code in script tags
         *  * object: { vertexSource: "vertex shader code string", fragmentSource: "fragment shader code string" } using with strings containing the shaders code
         *  * string: "./COMMON_NAME", used with external files COMMON_NAME.vertex.fx and COMMON_NAME.fragment.fx in index.html folder.
         * @param options Define the options used to create the shader
         * @param storeEffectOnSubMeshes true to store effect on submeshes, false to store the effect directly in the material class.
         */
        constructor(name: string, scene: Scene, shaderPath: any, options?: Partial<IShaderMaterialOptions>, storeEffectOnSubMeshes?: boolean);
        /**
         * Gets the shader path used to define the shader code
         * It can be modified to trigger a new compilation
         */
        get shaderPath(): any;
        /**
         * Sets the shader path used to define the shader code
         * It can be modified to trigger a new compilation
         */
        set shaderPath(shaderPath: any);
        /**
         * Gets the options used to compile the shader.
         * They can be modified to trigger a new compilation
         */
        get options(): IShaderMaterialOptions;
        /**
         * Gets the current class name of the material e.g. "ShaderMaterial"
         * Mainly use in serialization.
         * @returns the class name
         */
        getClassName(): string;
        /**
         * Specifies if the material will require alpha blending
         * @returns a boolean specifying if alpha blending is needed
         */
        needAlphaBlending(): boolean;
        /**
         * Specifies if this material should be rendered in alpha test mode
         * @returns a boolean specifying if an alpha test is needed.
         */
        needAlphaTesting(): boolean;
        private _checkUniform;
        /**
         * Set a texture in the shader.
         * @param name Define the name of the uniform samplers as defined in the shader
         * @param texture Define the texture to bind to this sampler
         * @returns the material itself allowing "fluent" like uniform updates
         */
        setTexture(name: string, texture: BaseTexture): ShaderMaterial;
        /**
         * Set a texture array in the shader.
         * @param name Define the name of the uniform sampler array as defined in the shader
         * @param textures Define the list of textures to bind to this sampler
         * @returns the material itself allowing "fluent" like uniform updates
         */
        setTextureArray(name: string, textures: BaseTexture[]): ShaderMaterial;
        /**
         * Set an internal texture in the shader.
         * @param name Define the name of the uniform samplers as defined in the shader
         * @param texture Define the texture to bind to this sampler
         * @returns the material itself allowing "fluent" like uniform updates
         */
        setExternalTexture(name: string, texture: ExternalTexture): ShaderMaterial;
        /**
         * Set a float in the shader.
         * @param name Define the name of the uniform as defined in the shader
         * @param value Define the value to give to the uniform
         * @returns the material itself allowing "fluent" like uniform updates
         */
        setFloat(name: string, value: number): ShaderMaterial;
        /**
         * Set a int in the shader.
         * @param name Define the name of the uniform as defined in the shader
         * @param value Define the value to give to the uniform
         * @returns the material itself allowing "fluent" like uniform updates
         */
        setInt(name: string, value: number): ShaderMaterial;
        /**
         * Set a unsigned int in the shader.
         * @param name Define the name of the uniform as defined in the shader
         * @param value Define the value to give to the uniform
         * @return the material itself allowing "fluent" like uniform updates
         */
        setUInt(name: string, value: number): ShaderMaterial;
        /**
         * Set an array of floats in the shader.
         * @param name Define the name of the uniform as defined in the shader
         * @param value Define the value to give to the uniform
         * @returns the material itself allowing "fluent" like uniform updates
         */
        setFloats(name: string, value: number[]): ShaderMaterial;
        /**
         * Set a vec3 in the shader from a Color3.
         * @param name Define the name of the uniform as defined in the shader
         * @param value Define the value to give to the uniform
         * @returns the material itself allowing "fluent" like uniform updates
         */
        setColor3(name: string, value: Color3): ShaderMaterial;
        /**
         * Set a vec3 array in the shader from a Color3 array.
         * @param name Define the name of the uniform as defined in the shader
         * @param value Define the value to give to the uniform
         * @returns the material itself allowing "fluent" like uniform updates
         */
        setColor3Array(name: string, value: Color3[]): ShaderMaterial;
        /**
         * Set a vec4 in the shader from a Color4.
         * @param name Define the name of the uniform as defined in the shader
         * @param value Define the value to give to the uniform
         * @returns the material itself allowing "fluent" like uniform updates
         */
        setColor4(name: string, value: Color4): ShaderMaterial;
        /**
         * Set a vec4 array in the shader from a Color4 array.
         * @param name Define the name of the uniform as defined in the shader
         * @param value Define the value to give to the uniform
         * @returns the material itself allowing "fluent" like uniform updates
         */
        setColor4Array(name: string, value: Color4[]): ShaderMaterial;
        /**
         * Set a vec2 in the shader from a Vector2.
         * @param name Define the name of the uniform as defined in the shader
         * @param value Define the value to give to the uniform
         * @returns the material itself allowing "fluent" like uniform updates
         */
        setVector2(name: string, value: Vector2): ShaderMaterial;
        /**
         * Set a vec3 in the shader from a Vector3.
         * @param name Define the name of the uniform as defined in the shader
         * @param value Define the value to give to the uniform
         * @returns the material itself allowing "fluent" like uniform updates
         */
        setVector3(name: string, value: Vector3): ShaderMaterial;
        /**
         * Set a vec4 in the shader from a Vector4.
         * @param name Define the name of the uniform as defined in the shader
         * @param value Define the value to give to the uniform
         * @returns the material itself allowing "fluent" like uniform updates
         */
        setVector4(name: string, value: Vector4): ShaderMaterial;
        /**
         * Set a vec4 in the shader from a Quaternion.
         * @param name Define the name of the uniform as defined in the shader
         * @param value Define the value to give to the uniform
         * @returns the material itself allowing "fluent" like uniform updates
         */
        setQuaternion(name: string, value: Quaternion): ShaderMaterial;
        /**
         * Set a vec4 array in the shader from a Quaternion array.
         * @param name Define the name of the uniform as defined in the shader
         * @param value Define the value to give to the uniform
         * @returns the material itself allowing "fluent" like uniform updates
         */
        setQuaternionArray(name: string, value: Quaternion[]): ShaderMaterial;
        /**
         * Set a mat4 in the shader from a Matrix.
         * @param name Define the name of the uniform as defined in the shader
         * @param value Define the value to give to the uniform
         * @returns the material itself allowing "fluent" like uniform updates
         */
        setMatrix(name: string, value: Matrix): ShaderMaterial;
        /**
         * Set a float32Array in the shader from a matrix array.
         * @param name Define the name of the uniform as defined in the shader
         * @param value Define the value to give to the uniform
         * @returns the material itself allowing "fluent" like uniform updates
         */
        setMatrices(name: string, value: Matrix[]): ShaderMaterial;
        /**
         * Set a mat3 in the shader from a Float32Array.
         * @param name Define the name of the uniform as defined in the shader
         * @param value Define the value to give to the uniform
         * @returns the material itself allowing "fluent" like uniform updates
         */
        setMatrix3x3(name: string, value: Float32Array | Array<number>): ShaderMaterial;
        /**
         * Set a mat2 in the shader from a Float32Array.
         * @param name Define the name of the uniform as defined in the shader
         * @param value Define the value to give to the uniform
         * @returns the material itself allowing "fluent" like uniform updates
         */
        setMatrix2x2(name: string, value: Float32Array | Array<number>): ShaderMaterial;
        /**
         * Set a vec2 array in the shader from a number array.
         * @param name Define the name of the uniform as defined in the shader
         * @param value Define the value to give to the uniform
         * @returns the material itself allowing "fluent" like uniform updates
         */
        setArray2(name: string, value: number[]): ShaderMaterial;
        /**
         * Set a vec3 array in the shader from a number array.
         * @param name Define the name of the uniform as defined in the shader
         * @param value Define the value to give to the uniform
         * @returns the material itself allowing "fluent" like uniform updates
         */
        setArray3(name: string, value: number[]): ShaderMaterial;
        /**
         * Set a vec4 array in the shader from a number array.
         * @param name Define the name of the uniform as defined in the shader
         * @param value Define the value to give to the uniform
         * @returns the material itself allowing "fluent" like uniform updates
         */
        setArray4(name: string, value: number[]): ShaderMaterial;
        /**
         * Set a uniform buffer in the shader
         * @param name Define the name of the uniform as defined in the shader
         * @param buffer Define the value to give to the uniform
         * @returns the material itself allowing "fluent" like uniform updates
         */
        setUniformBuffer(name: string, buffer: UniformBuffer): ShaderMaterial;
        /**
         * Set a texture sampler in the shader
         * @param name Define the name of the uniform as defined in the shader
         * @param sampler Define the value to give to the uniform
         * @returns the material itself allowing "fluent" like uniform updates
         */
        setTextureSampler(name: string, sampler: TextureSampler): ShaderMaterial;
        /**
         * Set a storage buffer in the shader
         * @param name Define the name of the storage buffer as defined in the shader
         * @param buffer Define the value to give to the uniform
         * @returns the material itself allowing "fluent" like uniform updates
         */
        setStorageBuffer(name: string, buffer: StorageBuffer): ShaderMaterial;
        /**
         * Specifies that the submesh is ready to be used
         * @param mesh defines the mesh to check
         * @param subMesh defines which submesh to check
         * @param useInstances specifies that instances should be used
         * @returns a boolean indicating that the submesh is ready or not
         */
        isReadyForSubMesh(mesh: AbstractMesh, subMesh: SubMesh, useInstances?: boolean): boolean;
        /**
         * Checks if the material is ready to render the requested mesh
         * @param mesh Define the mesh to render
         * @param useInstances Define whether or not the material is used with instances
         * @param subMesh defines which submesh to render
         * @returns true if ready, otherwise false
         */
        isReady(mesh?: AbstractMesh, useInstances?: boolean, subMesh?: SubMesh): boolean;
        /**
         * Binds the world matrix to the material
         * @param world defines the world transformation matrix
         * @param effectOverride - If provided, use this effect instead of internal effect
         */
        bindOnlyWorldMatrix(world: Matrix, effectOverride?: Nullable<Effect>): void;
        /**
         * Binds the submesh to this material by preparing the effect and shader to draw
         * @param world defines the world transformation matrix
         * @param mesh defines the mesh containing the submesh
         * @param subMesh defines the submesh to bind the material to
         */
        bindForSubMesh(world: Matrix, mesh: Mesh, subMesh: SubMesh): void;
        /**
         * Binds the material to the mesh
         * @param world defines the world transformation matrix
         * @param mesh defines the mesh to bind the material to
         * @param effectOverride - If provided, use this effect instead of internal effect
         * @param subMesh defines the submesh to bind the material to
         */
        bind(world: Matrix, mesh?: Mesh, effectOverride?: Nullable<Effect>, subMesh?: SubMesh): void;
        /**
         * Gets the active textures from the material
         * @returns an array of textures
         */
        getActiveTextures(): BaseTexture[];
        /**
         * Specifies if the material uses a texture
         * @param texture defines the texture to check against the material
         * @returns a boolean specifying if the material uses the texture
         */
        hasTexture(texture: BaseTexture): boolean;
        /**
         * Makes a duplicate of the material, and gives it a new name
         * @param name defines the new name for the duplicated material
         * @returns the cloned material
         */
        clone(name: string): ShaderMaterial;
        /**
         * Disposes the material
         * @param forceDisposeEffect specifies if effects should be forcefully disposed
         * @param forceDisposeTextures specifies if textures should be forcefully disposed
         * @param notBoundToMesh specifies if the material that is being disposed is known to be not bound to any mesh
         */
        dispose(forceDisposeEffect?: boolean, forceDisposeTextures?: boolean, notBoundToMesh?: boolean): void;
        /**
         * Serializes this material in a JSON representation
         * @returns the serialized material object
         */
        serialize(): any;
        /**
         * Creates a shader material from parsed shader material data
         * @param source defines the JSON representation of the material
         * @param scene defines the hosting scene
         * @param rootUrl defines the root URL to use to load textures and relative dependencies
         * @returns a new material
         */
        static Parse(source: any, scene: Scene, rootUrl: string): ShaderMaterial;
        /**
         * Creates a new ShaderMaterial from a snippet saved in a remote file
         * @param name defines the name of the ShaderMaterial to create (can be null or empty to use the one from the json data)
         * @param url defines the url to load from
         * @param scene defines the hosting scene
         * @param rootUrl defines the root URL to use to load textures and relative dependencies
         * @returns a promise that will resolve to the new ShaderMaterial
         */
        static ParseFromFileAsync(name: Nullable<string>, url: string, scene: Scene, rootUrl?: string): Promise<ShaderMaterial>;
        /**
         * Creates a ShaderMaterial from a snippet saved by the Inspector
         * @param snippetId defines the snippet to load
         * @param scene defines the hosting scene
         * @param rootUrl defines the root URL to use to load textures and relative dependencies
         * @returns a promise that will resolve to the new ShaderMaterial
         */
        static ParseFromSnippetAsync(snippetId: string, scene: Scene, rootUrl?: string): Promise<ShaderMaterial>;
        /**
         * Creates a ShaderMaterial from a snippet saved by the Inspector
         * @deprecated Please use ParseFromSnippetAsync instead
         * @param snippetId defines the snippet to load
         * @param scene defines the hosting scene
         * @param rootUrl defines the root URL to use to load textures and relative dependencies
         * @returns a promise that will resolve to the new ShaderMaterial
         */
        static CreateFromSnippetAsync: typeof ShaderMaterial.ParseFromSnippetAsync;
    }


    /**
     * Options to be used when creating a shadow depth material
     */
    export interface IIOptionShadowDepthMaterial {
        /** Variables in the vertex shader code that need to have their names remapped.
         * The format is: ["var_name", "var_remapped_name", "var_name", "var_remapped_name", ...]
         * "var_name" should be either: worldPos or vNormalW
         * So, if the variable holding the world position in your vertex shader is not named worldPos, you must tell the system
         * the name to use instead by using: ["worldPos", "myWorldPosVar"] assuming the variable is named myWorldPosVar in your code.
         * If the normal must also be remapped: ["worldPos", "myWorldPosVar", "vNormalW", "myWorldNormal"]
         */
        remappedVariables?: string[];
        /** Set standalone to true if the base material wrapped by ShadowDepthMaterial is not used for a regular object but for depth shadow generation only */
        standalone?: boolean;
        /** Set doNotInjectCode if the specific shadow map generation code is already implemented by the material. That will prevent this code to be injected twice by ShadowDepthWrapper */
        doNotInjectCode?: boolean;
    }
    /**
     * Class that can be used to wrap a base material to generate accurate shadows when using custom vertex/fragment code in the base material
     */
    export class ShadowDepthWrapper {
        private _scene;
        private _options?;
        private _baseMaterial;
        private _onEffectCreatedObserver;
        private _subMeshToEffect;
        private _subMeshToDepthWrapper;
        private _meshes;
        /** Gets the standalone status of the wrapper */
        get standalone(): boolean;
        /** Gets the base material the wrapper is built upon */
        get baseMaterial(): Material;
        /** Gets the doNotInjectCode status of the wrapper */
        get doNotInjectCode(): boolean;
        /**
         * Instantiate a new shadow depth wrapper.
         * It works by injecting some specific code in the vertex/fragment shaders of the base material and is used by a shadow generator to
         * generate the shadow depth map. For more information, please refer to the documentation:
         * https://doc.babylonjs.com/features/featuresDeepDive/lights/shadows
         * @param baseMaterial Material to wrap
         * @param scene Define the scene the material belongs to
         * @param options Options used to create the wrapper
         */
        constructor(baseMaterial: Material, scene?: Scene, options?: IIOptionShadowDepthMaterial);
        /**
         * Gets the effect to use to generate the depth map
         * @param subMesh subMesh to get the effect for
         * @param shadowGenerator shadow generator to get the effect for
         * @param passIdForDrawWrapper Id of the pass for which the effect from the draw wrapper must be retrieved from
         * @returns the effect to use to generate the depth map for the subMesh + shadow generator specified
         */
        getEffect(subMesh: Nullable<SubMesh>, shadowGenerator: ShadowGenerator, passIdForDrawWrapper: number): Nullable<DrawWrapper>;
        /**
         * Specifies that the submesh is ready to be used for depth rendering
         * @param subMesh submesh to check
         * @param defines the list of defines to take into account when checking the effect
         * @param shadowGenerator combined with subMesh, it defines the effect to check
         * @param useInstances specifies that instances should be used
         * @param passIdForDrawWrapper Id of the pass for which the draw wrapper should be created
         * @returns a boolean indicating that the submesh is ready or not
         */
        isReadyForSubMesh(subMesh: SubMesh, defines: string[], shadowGenerator: ShadowGenerator, useInstances: boolean, passIdForDrawWrapper: number): boolean;
        /**
         * Disposes the resources
         */
        dispose(): void;
        private _makeEffect;
    }


    /** @internal */
    export class StandardMaterialDefines extends MaterialDefines implements IImageProcessingConfigurationDefines {
        MAINUV1: boolean;
        MAINUV2: boolean;
        MAINUV3: boolean;
        MAINUV4: boolean;
        MAINUV5: boolean;
        MAINUV6: boolean;
        DIFFUSE: boolean;
        DIFFUSEDIRECTUV: number;
        BAKED_VERTEX_ANIMATION_TEXTURE: boolean;
        AMBIENT: boolean;
        AMBIENTDIRECTUV: number;
        OPACITY: boolean;
        OPACITYDIRECTUV: number;
        OPACITYRGB: boolean;
        REFLECTION: boolean;
        EMISSIVE: boolean;
        EMISSIVEDIRECTUV: number;
        SPECULAR: boolean;
        SPECULARDIRECTUV: number;
        BUMP: boolean;
        BUMPDIRECTUV: number;
        PARALLAX: boolean;
        PARALLAXOCCLUSION: boolean;
        SPECULAROVERALPHA: boolean;
        CLIPPLANE: boolean;
        CLIPPLANE2: boolean;
        CLIPPLANE3: boolean;
        CLIPPLANE4: boolean;
        CLIPPLANE5: boolean;
        CLIPPLANE6: boolean;
        ALPHATEST: boolean;
        DEPTHPREPASS: boolean;
        ALPHAFROMDIFFUSE: boolean;
        POINTSIZE: boolean;
        FOG: boolean;
        SPECULARTERM: boolean;
        DIFFUSEFRESNEL: boolean;
        OPACITYFRESNEL: boolean;
        REFLECTIONFRESNEL: boolean;
        REFRACTIONFRESNEL: boolean;
        EMISSIVEFRESNEL: boolean;
        FRESNEL: boolean;
        NORMAL: boolean;
        TANGENT: boolean;
        UV1: boolean;
        UV2: boolean;
        UV3: boolean;
        UV4: boolean;
        UV5: boolean;
        UV6: boolean;
        VERTEXCOLOR: boolean;
        VERTEXALPHA: boolean;
        NUM_BONE_INFLUENCERS: number;
        BonesPerMesh: number;
        BONETEXTURE: boolean;
        BONES_VELOCITY_ENABLED: boolean;
        INSTANCES: boolean;
        THIN_INSTANCES: boolean;
        INSTANCESCOLOR: boolean;
        GLOSSINESS: boolean;
        ROUGHNESS: boolean;
        EMISSIVEASILLUMINATION: boolean;
        LINKEMISSIVEWITHDIFFUSE: boolean;
        REFLECTIONFRESNELFROMSPECULAR: boolean;
        LIGHTMAP: boolean;
        LIGHTMAPDIRECTUV: number;
        OBJECTSPACE_NORMALMAP: boolean;
        USELIGHTMAPASSHADOWMAP: boolean;
        REFLECTIONMAP_3D: boolean;
        REFLECTIONMAP_SPHERICAL: boolean;
        REFLECTIONMAP_PLANAR: boolean;
        REFLECTIONMAP_CUBIC: boolean;
        USE_LOCAL_REFLECTIONMAP_CUBIC: boolean;
        USE_LOCAL_REFRACTIONMAP_CUBIC: boolean;
        REFLECTIONMAP_PROJECTION: boolean;
        REFLECTIONMAP_SKYBOX: boolean;
        REFLECTIONMAP_EXPLICIT: boolean;
        REFLECTIONMAP_EQUIRECTANGULAR: boolean;
        REFLECTIONMAP_EQUIRECTANGULAR_FIXED: boolean;
        REFLECTIONMAP_MIRROREDEQUIRECTANGULAR_FIXED: boolean;
        REFLECTIONMAP_OPPOSITEZ: boolean;
        INVERTCUBICMAP: boolean;
        LOGARITHMICDEPTH: boolean;
        REFRACTION: boolean;
        REFRACTIONMAP_3D: boolean;
        REFLECTIONOVERALPHA: boolean;
        TWOSIDEDLIGHTING: boolean;
        SHADOWFLOAT: boolean;
        MORPHTARGETS: boolean;
        MORPHTARGETS_NORMAL: boolean;
        MORPHTARGETS_TANGENT: boolean;
        MORPHTARGETS_UV: boolean;
        NUM_MORPH_INFLUENCERS: number;
        MORPHTARGETS_TEXTURE: boolean;
        NONUNIFORMSCALING: boolean;
        PREMULTIPLYALPHA: boolean;
        ALPHATEST_AFTERALLALPHACOMPUTATIONS: boolean;
        ALPHABLEND: boolean;
        PREPASS: boolean;
        PREPASS_IRRADIANCE: boolean;
        PREPASS_IRRADIANCE_INDEX: number;
        PREPASS_ALBEDO_SQRT: boolean;
        PREPASS_ALBEDO_SQRT_INDEX: number;
        PREPASS_DEPTH: boolean;
        PREPASS_DEPTH_INDEX: number;
        PREPASS_NORMAL: boolean;
        PREPASS_NORMAL_INDEX: number;
        PREPASS_POSITION: boolean;
        PREPASS_POSITION_INDEX: number;
        PREPASS_VELOCITY: boolean;
        PREPASS_VELOCITY_INDEX: number;
        PREPASS_REFLECTIVITY: boolean;
        PREPASS_REFLECTIVITY_INDEX: number;
        SCENE_MRT_COUNT: number;
        RGBDLIGHTMAP: boolean;
        RGBDREFLECTION: boolean;
        RGBDREFRACTION: boolean;
        IMAGEPROCESSING: boolean;
        VIGNETTE: boolean;
        VIGNETTEBLENDMODEMULTIPLY: boolean;
        VIGNETTEBLENDMODEOPAQUE: boolean;
        TONEMAPPING: boolean;
        TONEMAPPING_ACES: boolean;
        CONTRAST: boolean;
        COLORCURVES: boolean;
        COLORGRADING: boolean;
        COLORGRADING3D: boolean;
        SAMPLER3DGREENDEPTH: boolean;
        SAMPLER3DBGRMAP: boolean;
        DITHER: boolean;
        IMAGEPROCESSINGPOSTPROCESS: boolean;
        SKIPFINALCOLORCLAMP: boolean;
        MULTIVIEW: boolean;
        ORDER_INDEPENDENT_TRANSPARENCY: boolean;
        ORDER_INDEPENDENT_TRANSPARENCY_16BITS: boolean;
        CAMERA_ORTHOGRAPHIC: boolean;
        CAMERA_PERSPECTIVE: boolean;
        /**
         * If the reflection texture on this material is in linear color space
         * @internal
         */
        IS_REFLECTION_LINEAR: boolean;
        /**
         * If the refraction texture on this material is in linear color space
         * @internal
         */
        IS_REFRACTION_LINEAR: boolean;
        EXPOSURE: boolean;
        /**
         * Initializes the Standard Material defines.
         * @param externalProperties The external properties
         */
        constructor(externalProperties?: {
            [name: string]: {
                type: string;
                default: any;
            };
        });
        setReflectionMode(modeToEnable: string): void;
    }
    /**
     * This is the default material used in Babylon. It is the best trade off between quality
     * and performances.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/materials_introduction
     */
    export class StandardMaterial extends PushMaterial {
        private _diffuseTexture;
        /**
         * The basic texture of the material as viewed under a light.
         */
        diffuseTexture: Nullable<BaseTexture>;
        private _ambientTexture;
        /**
         * AKA Occlusion Texture in other nomenclature, it helps adding baked shadows into your material.
         */
        ambientTexture: Nullable<BaseTexture>;
        private _opacityTexture;
        /**
         * Define the transparency of the material from a texture.
         * The final alpha value can be read either from the red channel (if texture.getAlphaFromRGB is false)
         * or from the luminance or the current texel (if texture.getAlphaFromRGB is true)
         */
        opacityTexture: Nullable<BaseTexture>;
        private _reflectionTexture;
        /**
         * Define the texture used to display the reflection.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/reflectionTexture#how-to-obtain-reflections-and-refractions
         */
        reflectionTexture: Nullable<BaseTexture>;
        private _emissiveTexture;
        /**
         * Define texture of the material as if self lit.
         * This will be mixed in the final result even in the absence of light.
         */
        emissiveTexture: Nullable<BaseTexture>;
        private _specularTexture;
        /**
         * Define how the color and intensity of the highlight given by the light in the material.
         */
        specularTexture: Nullable<BaseTexture>;
        private _bumpTexture;
        /**
         * Bump mapping is a technique to simulate bump and dents on a rendered surface.
         * These are made by creating a normal map from an image. The means to do this can be found on the web, a search for 'normal map generator' will bring up free and paid for methods of doing this.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/moreMaterials#bump-map
         */
        bumpTexture: Nullable<BaseTexture>;
        private _lightmapTexture;
        /**
         * Complex lighting can be computationally expensive to compute at runtime.
         * To save on computation, lightmaps may be used to store calculated lighting in a texture which will be applied to a given mesh.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/lights/lights_introduction#lightmaps
         */
        lightmapTexture: Nullable<BaseTexture>;
        private _refractionTexture;
        /**
         * Define the texture used to display the refraction.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/reflectionTexture#how-to-obtain-reflections-and-refractions
         */
        refractionTexture: Nullable<BaseTexture>;
        /**
         * The color of the material lit by the environmental background lighting.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/materials_introduction#ambient-color-example
         */
        ambientColor: Color3;
        /**
         * The basic color of the material as viewed under a light.
         */
        diffuseColor: Color3;
        /**
         * Define how the color and intensity of the highlight given by the light in the material.
         */
        specularColor: Color3;
        /**
         * Define the color of the material as if self lit.
         * This will be mixed in the final result even in the absence of light.
         */
        emissiveColor: Color3;
        /**
         * Defines how sharp are the highlights in the material.
         * The bigger the value the sharper giving a more glossy feeling to the result.
         * Reversely, the smaller the value the blurrier giving a more rough feeling to the result.
         */
        specularPower: number;
        private _useAlphaFromDiffuseTexture;
        /**
         * Does the transparency come from the diffuse texture alpha channel.
         */
        useAlphaFromDiffuseTexture: boolean;
        private _useEmissiveAsIllumination;
        /**
         * If true, the emissive value is added into the end result, otherwise it is multiplied in.
         */
        useEmissiveAsIllumination: boolean;
        private _linkEmissiveWithDiffuse;
        /**
         * If true, some kind of energy conservation will prevent the end result to be more than 1 by reducing
         * the emissive level when the final color is close to one.
         */
        linkEmissiveWithDiffuse: boolean;
        private _useSpecularOverAlpha;
        /**
         * Specifies that the material will keep the specular highlights over a transparent surface (only the most luminous ones).
         * A car glass is a good exemple of that. When sun reflects on it you can not see what is behind.
         */
        useSpecularOverAlpha: boolean;
        private _useReflectionOverAlpha;
        /**
         * Specifies that the material will keeps the reflection highlights over a transparent surface (only the most luminous ones).
         * A car glass is a good exemple of that. When the street lights reflects on it you can not see what is behind.
         */
        useReflectionOverAlpha: boolean;
        private _disableLighting;
        /**
         * Does lights from the scene impacts this material.
         * It can be a nice trick for performance to disable lighting on a fully emissive material.
         */
        disableLighting: boolean;
        private _useObjectSpaceNormalMap;
        /**
         * Allows using an object space normal map (instead of tangent space).
         */
        useObjectSpaceNormalMap: boolean;
        private _useParallax;
        /**
         * Is parallax enabled or not.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/parallaxMapping
         */
        useParallax: boolean;
        private _useParallaxOcclusion;
        /**
         * Is parallax occlusion enabled or not.
         * If true, the outcome is way more realistic than traditional Parallax but you can expect a performance hit that worthes consideration.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/parallaxMapping
         */
        useParallaxOcclusion: boolean;
        /**
         * Apply a scaling factor that determine which "depth" the height map should reprensent. A value between 0.05 and 0.1 is reasonnable in Parallax, you can reach 0.2 using Parallax Occlusion.
         */
        parallaxScaleBias: number;
        private _roughness;
        /**
         * Helps to define how blurry the reflections should appears in the material.
         */
        roughness: number;
        /**
         * In case of refraction, define the value of the index of refraction.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/reflectionTexture#how-to-obtain-reflections-and-refractions
         */
        indexOfRefraction: number;
        /**
         * Invert the refraction texture alongside the y axis.
         * It can be useful with procedural textures or probe for instance.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/reflectionTexture#how-to-obtain-reflections-and-refractions
         */
        invertRefractionY: boolean;
        /**
         * Defines the alpha limits in alpha test mode.
         */
        alphaCutOff: number;
        private _useLightmapAsShadowmap;
        /**
         * In case of light mapping, define whether the map contains light or shadow informations.
         */
        useLightmapAsShadowmap: boolean;
        private _diffuseFresnelParameters;
        /**
         * Define the diffuse fresnel parameters of the material.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/fresnelParameters
         */
        diffuseFresnelParameters: FresnelParameters;
        private _opacityFresnelParameters;
        /**
         * Define the opacity fresnel parameters of the material.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/fresnelParameters
         */
        opacityFresnelParameters: FresnelParameters;
        private _reflectionFresnelParameters;
        /**
         * Define the reflection fresnel parameters of the material.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/fresnelParameters
         */
        reflectionFresnelParameters: FresnelParameters;
        private _refractionFresnelParameters;
        /**
         * Define the refraction fresnel parameters of the material.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/fresnelParameters
         */
        refractionFresnelParameters: FresnelParameters;
        private _emissiveFresnelParameters;
        /**
         * Define the emissive fresnel parameters of the material.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/fresnelParameters
         */
        emissiveFresnelParameters: FresnelParameters;
        private _useReflectionFresnelFromSpecular;
        /**
         * If true automatically deducts the fresnels values from the material specularity.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/fresnelParameters
         */
        useReflectionFresnelFromSpecular: boolean;
        private _useGlossinessFromSpecularMapAlpha;
        /**
         * Defines if the glossiness/roughness of the material should be read from the specular map alpha channel
         */
        useGlossinessFromSpecularMapAlpha: boolean;
        private _maxSimultaneousLights;
        /**
         * Defines the maximum number of lights that can be used in the material
         */
        maxSimultaneousLights: number;
        private _invertNormalMapX;
        /**
         * If sets to true, x component of normal map value will invert (x = 1.0 - x).
         */
        invertNormalMapX: boolean;
        private _invertNormalMapY;
        /**
         * If sets to true, y component of normal map value will invert (y = 1.0 - y).
         */
        invertNormalMapY: boolean;
        private _twoSidedLighting;
        /**
         * If sets to true and backfaceCulling is false, normals will be flipped on the backside.
         */
        twoSidedLighting: boolean;
        /**
         * Default configuration related to image processing available in the standard Material.
         */
        protected _imageProcessingConfiguration: ImageProcessingConfiguration;
        /**
         * Gets the image processing configuration used either in this material.
         */
        get imageProcessingConfiguration(): ImageProcessingConfiguration;
        /**
         * Sets the Default image processing configuration used either in the this material.
         *
         * If sets to null, the scene one is in use.
         */
        set imageProcessingConfiguration(value: ImageProcessingConfiguration);
        /**
         * Keep track of the image processing observer to allow dispose and replace.
         */
        private _imageProcessingObserver;
        /**
         * Attaches a new image processing configuration to the Standard Material.
         * @param configuration
         */
        protected _attachImageProcessingConfiguration(configuration: Nullable<ImageProcessingConfiguration>): void;
        /**
         * Defines additional PrePass parameters for the material.
         */
        readonly prePassConfiguration: PrePassConfiguration;
        /**
         * Can this material render to prepass
         */
        get isPrePassCapable(): boolean;
        /**
         * Gets whether the color curves effect is enabled.
         */
        get cameraColorCurvesEnabled(): boolean;
        /**
         * Sets whether the color curves effect is enabled.
         */
        set cameraColorCurvesEnabled(value: boolean);
        /**
         * Gets whether the color grading effect is enabled.
         */
        get cameraColorGradingEnabled(): boolean;
        /**
         * Gets whether the color grading effect is enabled.
         */
        set cameraColorGradingEnabled(value: boolean);
        /**
         * Gets whether tonemapping is enabled or not.
         */
        get cameraToneMappingEnabled(): boolean;
        /**
         * Sets whether tonemapping is enabled or not
         */
        set cameraToneMappingEnabled(value: boolean);
        /**
         * The camera exposure used on this material.
         * This property is here and not in the camera to allow controlling exposure without full screen post process.
         * This corresponds to a photographic exposure.
         */
        get cameraExposure(): number;
        /**
         * The camera exposure used on this material.
         * This property is here and not in the camera to allow controlling exposure without full screen post process.
         * This corresponds to a photographic exposure.
         */
        set cameraExposure(value: number);
        /**
         * Gets The camera contrast used on this material.
         */
        get cameraContrast(): number;
        /**
         * Sets The camera contrast used on this material.
         */
        set cameraContrast(value: number);
        /**
         * Gets the Color Grading 2D Lookup Texture.
         */
        get cameraColorGradingTexture(): Nullable<BaseTexture>;
        /**
         * Sets the Color Grading 2D Lookup Texture.
         */
        set cameraColorGradingTexture(value: Nullable<BaseTexture>);
        /**
         * The color grading curves provide additional color adjustmnent that is applied after any color grading transform (3D LUT).
         * They allow basic adjustment of saturation and small exposure adjustments, along with color filter tinting to provide white balance adjustment or more stylistic effects.
         * These are similar to controls found in many professional imaging or colorist software. The global controls are applied to the entire image. For advanced tuning, extra controls are provided to adjust the shadow, midtone and highlight areas of the image;
         * corresponding to low luminance, medium luminance, and high luminance areas respectively.
         */
        get cameraColorCurves(): Nullable<ColorCurves>;
        /**
         * The color grading curves provide additional color adjustment that is applied after any color grading transform (3D LUT).
         * They allow basic adjustment of saturation and small exposure adjustments, along with color filter tinting to provide white balance adjustment or more stylistic effects.
         * These are similar to controls found in many professional imaging or colorist software. The global controls are applied to the entire image. For advanced tuning, extra controls are provided to adjust the shadow, midtone and highlight areas of the image;
         * corresponding to low luminance, medium luminance, and high luminance areas respectively.
         */
        set cameraColorCurves(value: Nullable<ColorCurves>);
        /**
         * Can this material render to several textures at once
         */
        get canRenderToMRT(): boolean;
        /**
         * Defines the detail map parameters for the material.
         */
        readonly detailMap: DetailMapConfiguration;
        protected _renderTargets: SmartArray<RenderTargetTexture>;
        protected _worldViewProjectionMatrix: Matrix;
        protected _globalAmbientColor: Color3;
        protected _useLogarithmicDepth: boolean;
        protected _cacheHasRenderTargetTextures: boolean;
        /**
         * Instantiates a new standard material.
         * This is the default material used in Babylon. It is the best trade off between quality
         * and performances.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/materials_introduction
         * @param name Define the name of the material in the scene
         * @param scene Define the scene the material belong to
         */
        constructor(name: string, scene?: Scene);
        /**
         * Gets a boolean indicating that current material needs to register RTT
         */
        get hasRenderTargetTextures(): boolean;
        /**
         * Gets the current class name of the material e.g. "StandardMaterial"
         * Mainly use in serialization.
         * @returns the class name
         */
        getClassName(): string;
        /**
         * In case the depth buffer does not allow enough depth precision for your scene (might be the case in large scenes)
         * You can try switching to logarithmic depth.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/advanced/logarithmicDepthBuffer
         */
        get useLogarithmicDepth(): boolean;
        set useLogarithmicDepth(value: boolean);
        /**
         * Specifies if the material will require alpha blending
         * @returns a boolean specifying if alpha blending is needed
         */
        needAlphaBlending(): boolean;
        /**
         * Specifies if this material should be rendered in alpha test mode
         * @returns a boolean specifying if an alpha test is needed.
         */
        needAlphaTesting(): boolean;
        /**
         * Specifies whether or not the alpha value of the diffuse texture should be used for alpha blending.
         */
        protected _shouldUseAlphaFromDiffuseTexture(): boolean;
        /**
         * Specifies whether or not there is a usable alpha channel for transparency.
         */
        protected _hasAlphaChannel(): boolean;
        /**
         * Get the texture used for alpha test purpose.
         * @returns the diffuse texture in case of the standard material.
         */
        getAlphaTestTexture(): Nullable<BaseTexture>;
        /**
         * Get if the submesh is ready to be used and all its information available.
         * Child classes can use it to update shaders
         * @param mesh defines the mesh to check
         * @param subMesh defines which submesh to check
         * @param useInstances specifies that instances should be used
         * @returns a boolean indicating that the submesh is ready or not
         */
        isReadyForSubMesh(mesh: AbstractMesh, subMesh: SubMesh, useInstances?: boolean): boolean;
        /**
         * Builds the material UBO layouts.
         * Used internally during the effect preparation.
         */
        buildUniformLayout(): void;
        /**
         * Binds the submesh to this material by preparing the effect and shader to draw
         * @param world defines the world transformation matrix
         * @param mesh defines the mesh containing the submesh
         * @param subMesh defines the submesh to bind the material to
         */
        bindForSubMesh(world: Matrix, mesh: Mesh, subMesh: SubMesh): void;
        /**
         * Get the list of animatables in the material.
         * @returns the list of animatables object used in the material
         */
        getAnimatables(): IAnimatable[];
        /**
         * Gets the active textures from the material
         * @returns an array of textures
         */
        getActiveTextures(): BaseTexture[];
        /**
         * Specifies if the material uses a texture
         * @param texture defines the texture to check against the material
         * @returns a boolean specifying if the material uses the texture
         */
        hasTexture(texture: BaseTexture): boolean;
        /**
         * Disposes the material
         * @param forceDisposeEffect specifies if effects should be forcefully disposed
         * @param forceDisposeTextures specifies if textures should be forcefully disposed
         */
        dispose(forceDisposeEffect?: boolean, forceDisposeTextures?: boolean): void;
        /**
         * Makes a duplicate of the material, and gives it a new name
         * @param name defines the new name for the duplicated material
         * @returns the cloned material
         */
        clone(name: string): StandardMaterial;
        /**
         * Creates a standard material from parsed material data
         * @param source defines the JSON representation of the material
         * @param scene defines the hosting scene
         * @param rootUrl defines the root URL to use to load textures and relative dependencies
         * @returns a new standard material
         */
        static Parse(source: any, scene: Scene, rootUrl: string): StandardMaterial;
        /**
         * Are diffuse textures enabled in the application.
         */
        static get DiffuseTextureEnabled(): boolean;
        static set DiffuseTextureEnabled(value: boolean);
        /**
         * Are detail textures enabled in the application.
         */
        static get DetailTextureEnabled(): boolean;
        static set DetailTextureEnabled(value: boolean);
        /**
         * Are ambient textures enabled in the application.
         */
        static get AmbientTextureEnabled(): boolean;
        static set AmbientTextureEnabled(value: boolean);
        /**
         * Are opacity textures enabled in the application.
         */
        static get OpacityTextureEnabled(): boolean;
        static set OpacityTextureEnabled(value: boolean);
        /**
         * Are reflection textures enabled in the application.
         */
        static get ReflectionTextureEnabled(): boolean;
        static set ReflectionTextureEnabled(value: boolean);
        /**
         * Are emissive textures enabled in the application.
         */
        static get EmissiveTextureEnabled(): boolean;
        static set EmissiveTextureEnabled(value: boolean);
        /**
         * Are specular textures enabled in the application.
         */
        static get SpecularTextureEnabled(): boolean;
        static set SpecularTextureEnabled(value: boolean);
        /**
         * Are bump textures enabled in the application.
         */
        static get BumpTextureEnabled(): boolean;
        static set BumpTextureEnabled(value: boolean);
        /**
         * Are lightmap textures enabled in the application.
         */
        static get LightmapTextureEnabled(): boolean;
        static set LightmapTextureEnabled(value: boolean);
        /**
         * Are refraction textures enabled in the application.
         */
        static get RefractionTextureEnabled(): boolean;
        static set RefractionTextureEnabled(value: boolean);
        /**
         * Are color grading textures enabled in the application.
         */
        static get ColorGradingTextureEnabled(): boolean;
        static set ColorGradingTextureEnabled(value: boolean);
        /**
         * Are fresnels enabled in the application.
         */
        static get FresnelEnabled(): boolean;
        static set FresnelEnabled(value: boolean);
    }


    /**
     * Base class of all the textures in babylon.
     * It groups all the common properties the materials, post process, lights... might need
     * in order to make a correct use of the texture.
     */
    export class BaseTexture extends ThinTexture implements IAnimatable {
        /**
         * Default anisotropic filtering level for the application.
         * It is set to 4 as a good tradeoff between perf and quality.
         */
        static DEFAULT_ANISOTROPIC_FILTERING_LEVEL: number;
        /**
         * Gets or sets the unique id of the texture
         */
        uniqueId: number;
        /**
         * Define the name of the texture.
         */
        name: string;
        /**
         * Gets or sets an object used to store user defined information.
         */
        metadata: any;
        /** @internal */
        _internalMetadata: any;
        /**
         * For internal use only. Please do not use.
         */
        reservedDataStore: any;
        private _hasAlpha;
        /**
         * Define if the texture is having a usable alpha value (can be use for transparency or glossiness for instance).
         */
        set hasAlpha(value: boolean);
        get hasAlpha(): boolean;
        private _getAlphaFromRGB;
        /**
         * Defines if the alpha value should be determined via the rgb values.
         * If true the luminance of the pixel might be used to find the corresponding alpha value.
         */
        set getAlphaFromRGB(value: boolean);
        get getAlphaFromRGB(): boolean;
        /**
         * Intensity or strength of the texture.
         * It is commonly used by materials to fine tune the intensity of the texture
         */
        level: number;
        protected _coordinatesIndex: number;
        /**
         * Gets or sets a boolean indicating that the texture should try to reduce shader code if there is no UV manipulation.
         * (ie. when texture.getTextureMatrix().isIdentityAs3x2() returns true)
         */
        optimizeUVAllocation: boolean;
        /**
         * Define the UV channel to use starting from 0 and defaulting to 0.
         * This is part of the texture as textures usually maps to one uv set.
         */
        set coordinatesIndex(value: number);
        get coordinatesIndex(): number;
        protected _coordinatesMode: number;
        /**
         * How a texture is mapped.
         *
         * | Value | Type                                | Description |
         * | ----- | ----------------------------------- | ----------- |
         * | 0     | EXPLICIT_MODE                       |             |
         * | 1     | SPHERICAL_MODE                      |             |
         * | 2     | PLANAR_MODE                         |             |
         * | 3     | CUBIC_MODE                          |             |
         * | 4     | PROJECTION_MODE                     |             |
         * | 5     | SKYBOX_MODE                         |             |
         * | 6     | INVCUBIC_MODE                       |             |
         * | 7     | EQUIRECTANGULAR_MODE                |             |
         * | 8     | FIXED_EQUIRECTANGULAR_MODE          |             |
         * | 9     | FIXED_EQUIRECTANGULAR_MIRRORED_MODE |             |
         */
        set coordinatesMode(value: number);
        get coordinatesMode(): number;
        /**
         * | Value | Type               | Description |
         * | ----- | ------------------ | ----------- |
         * | 0     | CLAMP_ADDRESSMODE  |             |
         * | 1     | WRAP_ADDRESSMODE   |             |
         * | 2     | MIRROR_ADDRESSMODE |             |
         */
        get wrapU(): number;
        set wrapU(value: number);
        /**
         * | Value | Type               | Description |
         * | ----- | ------------------ | ----------- |
         * | 0     | CLAMP_ADDRESSMODE  |             |
         * | 1     | WRAP_ADDRESSMODE   |             |
         * | 2     | MIRROR_ADDRESSMODE |             |
         */
        get wrapV(): number;
        set wrapV(value: number);
        /**
         * | Value | Type               | Description |
         * | ----- | ------------------ | ----------- |
         * | 0     | CLAMP_ADDRESSMODE  |             |
         * | 1     | WRAP_ADDRESSMODE   |             |
         * | 2     | MIRROR_ADDRESSMODE |             |
         */
        wrapR: number;
        /**
         * With compliant hardware and browser (supporting anisotropic filtering)
         * this defines the level of anisotropic filtering in the texture.
         * The higher the better but the slower. This defaults to 4 as it seems to be the best tradeoff.
         */
        anisotropicFilteringLevel: number;
        /** @internal */
        _isCube: boolean;
        /**
         * Define if the texture is a cube texture or if false a 2d texture.
         */
        get isCube(): boolean;
        protected set isCube(value: boolean);
        /**
         * Define if the texture is a 3d texture (webgl 2) or if false a 2d texture.
         */
        get is3D(): boolean;
        protected set is3D(value: boolean);
        /**
         * Define if the texture is a 2d array texture (webgl 2) or if false a 2d texture.
         */
        get is2DArray(): boolean;
        protected set is2DArray(value: boolean);
        private _gammaSpace;
        /**
         * Define if the texture contains data in gamma space (most of the png/jpg aside bump).
         * HDR texture are usually stored in linear space.
         * This only impacts the PBR and Background materials
         */
        get gammaSpace(): boolean;
        set gammaSpace(gamma: boolean);
        /**
         * Gets or sets whether or not the texture contains RGBD data.
         */
        get isRGBD(): boolean;
        set isRGBD(value: boolean);
        /**
         * Is Z inverted in the texture (useful in a cube texture).
         */
        invertZ: boolean;
        /**
         * Are mip maps generated for this texture or not.
         */
        get noMipmap(): boolean;
        /**
         * @internal
         */
        lodLevelInAlpha: boolean;
        /**
         * With prefiltered texture, defined the offset used during the prefiltering steps.
         */
        get lodGenerationOffset(): number;
        set lodGenerationOffset(value: number);
        /**
         * With prefiltered texture, defined the scale used during the prefiltering steps.
         */
        get lodGenerationScale(): number;
        set lodGenerationScale(value: number);
        /**
         * With prefiltered texture, defined if the specular generation is based on a linear ramp.
         * By default we are using a log2 of the linear roughness helping to keep a better resolution for
         * average roughness values.
         */
        get linearSpecularLOD(): boolean;
        set linearSpecularLOD(value: boolean);
        /**
         * In case a better definition than spherical harmonics is required for the diffuse part of the environment.
         * You can set the irradiance texture to rely on a texture instead of the spherical approach.
         * This texture need to have the same characteristics than its parent (Cube vs 2d, coordinates mode, Gamma/Linear, RGBD).
         */
        get irradianceTexture(): Nullable<BaseTexture>;
        set irradianceTexture(value: Nullable<BaseTexture>);
        /**
         * Define if the texture is a render target.
         */
        isRenderTarget: boolean;
        /**
         * Define the unique id of the texture in the scene.
         */
        get uid(): string;
        /** @internal */
        _prefiltered: boolean;
        /** @internal */
        _forceSerialize: boolean;
        /**
         * Return a string representation of the texture.
         * @returns the texture as a string
         */
        toString(): string;
        /**
         * Get the class name of the texture.
         * @returns "BaseTexture"
         */
        getClassName(): string;
        /**
         * Define the list of animation attached to the texture.
         */
        animations: Animation[];
        /**
         * An event triggered when the texture is disposed.
         */
        onDisposeObservable: Observable<BaseTexture>;
        private _onDisposeObserver;
        /**
         * Callback triggered when the texture has been disposed.
         * Kept for back compatibility, you can use the onDisposeObservable instead.
         */
        set onDispose(callback: () => void);
        protected _scene: Nullable<Scene>;
        /** @internal */
        private _uid;
        /**
         * Define if the texture is preventing a material to render or not.
         * If not and the texture is not ready, the engine will use a default black texture instead.
         */
        get isBlocking(): boolean;
        /** @internal */
        _parentContainer: Nullable<AbstractScene>;
        protected _loadingError: boolean;
        protected _errorObject?: {
            message?: string;
            exception?: any;
        };
        /**
         * Was there any loading error?
         */
        get loadingError(): boolean;
        /**
         * If a loading error occurred this object will be populated with information about the error.
         */
        get errorObject(): {
            message?: string;
            exception?: any;
        } | undefined;
        /**
         * Instantiates a new BaseTexture.
         * Base class of all the textures in babylon.
         * It groups all the common properties the materials, post process, lights... might need
         * in order to make a correct use of the texture.
         * @param sceneOrEngine Define the scene or engine the texture belongs to
         * @param internalTexture Define the internal texture associated with the texture
         */
        constructor(sceneOrEngine?: Nullable<Scene | ThinEngine>, internalTexture?: Nullable<InternalTexture>);
        /**
         * Get the scene the texture belongs to.
         * @returns the scene or null if undefined
         */
        getScene(): Nullable<Scene>;
        /** @internal */
        protected _getEngine(): Nullable<ThinEngine>;
        /**
         * Checks if the texture has the same transform matrix than another texture
         * @param texture texture to check against
         * @returns true if the transforms are the same, else false
         */
        checkTransformsAreIdentical(texture: Nullable<BaseTexture>): boolean;
        /**
         * Get the texture transform matrix used to offset tile the texture for instance.
         * @returns the transformation matrix
         */
        getTextureMatrix(): Matrix;
        /**
         * Get the texture reflection matrix used to rotate/transform the reflection.
         * @returns the reflection matrix
         */
        getReflectionTextureMatrix(): Matrix;
        /**
         * Get if the texture is ready to be consumed (either it is ready or it is not blocking)
         * @returns true if ready, not blocking or if there was an error loading the texture
         */
        isReadyOrNotBlocking(): boolean;
        /**
         * Scales the texture if is `canRescale()`
         * @param ratio the resize factor we want to use to rescale
         */
        scale(ratio: number): void;
        /**
         * Get if the texture can rescale.
         */
        get canRescale(): boolean;
        /**
         * @internal
         */
        _getFromCache(url: Nullable<string>, noMipmap: boolean, sampling?: number, invertY?: boolean, useSRGBBuffer?: boolean, isCube?: boolean): Nullable<InternalTexture>;
        /** @internal */
        _rebuild(): void;
        /**
         * Clones the texture.
         * @returns the cloned texture
         */
        clone(): Nullable<BaseTexture>;
        /**
         * Get the texture underlying type (INT, FLOAT...)
         */
        get textureType(): number;
        /**
         * Get the texture underlying format (RGB, RGBA...)
         */
        get textureFormat(): number;
        /**
         * Indicates that textures need to be re-calculated for all materials
         */
        protected _markAllSubMeshesAsTexturesDirty(): void;
        /**
         * Reads the pixels stored in the webgl texture and returns them as an ArrayBuffer.
         * This will returns an RGBA array buffer containing either in values (0-255) or
         * float values (0-1) depending of the underlying buffer type.
         * @param faceIndex defines the face of the texture to read (in case of cube texture)
         * @param level defines the LOD level of the texture to read (in case of Mip Maps)
         * @param buffer defines a user defined buffer to fill with data (can be null)
         * @param flushRenderer true to flush the renderer from the pending commands before reading the pixels
         * @param noDataConversion false to convert the data to Uint8Array (if texture type is UNSIGNED_BYTE) or to Float32Array (if texture type is anything but UNSIGNED_BYTE). If true, the type of the generated buffer (if buffer==null) will depend on the type of the texture
         * @param x defines the region x coordinates to start reading from (default to 0)
         * @param y defines the region y coordinates to start reading from (default to 0)
         * @param width defines the region width to read from (default to the texture size at level)
         * @param height defines the region width to read from (default to the texture size at level)
         * @returns The Array buffer promise containing the pixels data.
         */
        readPixels(faceIndex?: number, level?: number, buffer?: Nullable<ArrayBufferView>, flushRenderer?: boolean, noDataConversion?: boolean, x?: number, y?: number, width?: number, height?: number): Nullable<Promise<ArrayBufferView>>;
        /**
         * @internal
         */
        _readPixelsSync(faceIndex?: number, level?: number, buffer?: Nullable<ArrayBufferView>, flushRenderer?: boolean, noDataConversion?: boolean): Nullable<ArrayBufferView>;
        /** @internal */
        get _lodTextureHigh(): Nullable<BaseTexture>;
        /** @internal */
        get _lodTextureMid(): Nullable<BaseTexture>;
        /** @internal */
        get _lodTextureLow(): Nullable<BaseTexture>;
        /**
         * Dispose the texture and release its associated resources.
         */
        dispose(): void;
        /**
         * Serialize the texture into a JSON representation that can be parsed later on.
         * @param allowEmptyName True to force serialization even if name is empty. Default: false
         * @returns the JSON representation of the texture
         */
        serialize(allowEmptyName?: boolean): any;
        /**
         * Helper function to be called back once a list of texture contains only ready textures.
         * @param textures Define the list of textures to wait for
         * @param callback Define the callback triggered once the entire list will be ready
         */
        static WhenAllReady(textures: BaseTexture[], callback: () => void): void;
        private static _IsScene;
    }


        interface BaseTexture {
            /**
             * Get the polynomial representation of the texture data.
             * This is mainly use as a fast way to recover IBL Diffuse irradiance data.
             * @see https://learnopengl.com/PBR/IBL/Diffuse-irradiance
             */
            sphericalPolynomial: Nullable<SphericalPolynomial>;
            /**
             * Force recomputation of spherical polynomials.
             * Can be useful if you generate a cubemap multiple times (from a probe for eg) and you need the proper polynomials each time
             */
            forceSphericalPolynomialsRecompute(): void;
        }


    /**
     * This represents a color grading texture. This acts as a lookup table LUT, useful during post process
     * It can help converting any input color in a desired output one. This can then be used to create effects
     * from sepia, black and white to sixties or futuristic rendering...
     *
     * The only supported format is currently 3dl.
     * More information on LUT: https://en.wikipedia.org/wiki/3D_lookup_table
     */
    export class ColorGradingTexture extends BaseTexture {
        /**
         * The texture URL.
         */
        url: string;
        /**
         * Empty line regex stored for GC.
         */
        private static _NoneEmptyLineRegex;
        private _textureMatrix;
        private _onLoad;
        /**
         * Instantiates a ColorGradingTexture from the following parameters.
         *
         * @param url The location of the color grading data (currently only supporting 3dl)
         * @param sceneOrEngine The scene or engine the texture will be used in
         * @param onLoad defines a callback triggered when the texture has been loaded
         */
        constructor(url: string, sceneOrEngine: Scene | ThinEngine, onLoad?: Nullable<() => void>);
        /**
         * Fires the onload event from the constructor if requested.
         */
        private _triggerOnLoad;
        /**
         * Returns the texture matrix used in most of the material.
         * This is not used in color grading but keep for troubleshooting purpose (easily swap diffuse by colorgrading to look in).
         */
        getTextureMatrix(): Matrix;
        /**
         * Occurs when the file being loaded is a .3dl LUT file.
         */
        private _load3dlTexture;
        /**
         * Starts the loading process of the texture.
         */
        private _loadTexture;
        /**
         * Clones the color grading texture.
         */
        clone(): ColorGradingTexture;
        /**
         * Called during delayed load for textures.
         */
        delayLoad(): void;
        /**
         * Parses a color grading texture serialized by Babylon.
         * @param parsedTexture The texture information being parsedTexture
         * @param scene The scene to load the texture in
         * @returns A color grading texture
         */
        static Parse(parsedTexture: any, scene: Scene): Nullable<ColorGradingTexture>;
        /**
         * Serializes the LUT texture to json format.
         */
        serialize(): any;
    }


    /**
     * Class for creating a cube texture
     */
    export class CubeTexture extends BaseTexture {
        private _delayedOnLoad;
        private _delayedOnError;
        private _lodScale;
        private _lodOffset;
        /**
         * Observable triggered once the texture has been loaded.
         */
        onLoadObservable: Observable<CubeTexture>;
        /**
         * The url of the texture
         */
        url: string;
        /**
         * Gets or sets the center of the bounding box associated with the cube texture.
         * It must define where the camera used to render the texture was set
         * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/reflectionTexture#using-local-cubemap-mode
         */
        boundingBoxPosition: Vector3;
        private _boundingBoxSize;
        /**
         * Gets or sets the size of the bounding box associated with the cube texture
         * When defined, the cubemap will switch to local mode
         * @see https://community.arm.com/graphics/b/blog/posts/reflections-based-on-local-cubemaps-in-unity
         * @example https://www.babylonjs-playground.com/#RNASML
         */
        set boundingBoxSize(value: Vector3);
        /**
         * Returns the bounding box size
         * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/reflectionTexture#using-local-cubemap-mode
         */
        get boundingBoxSize(): Vector3;
        protected _rotationY: number;
        /**
         * Sets texture matrix rotation angle around Y axis in radians.
         */
        set rotationY(value: number);
        /**
         * Gets texture matrix rotation angle around Y axis radians.
         */
        get rotationY(): number;
        /**
         * Are mip maps generated for this texture or not.
         */
        get noMipmap(): boolean;
        private _noMipmap;
        /** @internal */
        _files: Nullable<string[]>;
        protected _forcedExtension: Nullable<string>;
        /**
         * Gets the forced extension (if any)
         */
        get forcedExtension(): Nullable<string>;
        private _extensions;
        private _textureMatrix;
        private _format;
        private _createPolynomials;
        private _loaderOptions;
        private _useSRGBBuffer?;
        /**
         * Creates a cube texture from an array of image urls
         * @param files defines an array of image urls
         * @param scene defines the hosting scene
         * @param noMipmap specifies if mip maps are not used
         * @returns a cube texture
         */
        static CreateFromImages(files: string[], scene: Scene, noMipmap?: boolean): CubeTexture;
        /**
         * Creates and return a texture created from prefilterd data by tools like IBL Baker or Lys.
         * @param url defines the url of the prefiltered texture
         * @param scene defines the scene the texture is attached to
         * @param forcedExtension defines the extension of the file if different from the url
         * @param createPolynomials defines whether or not to create polynomial harmonics from the texture data if necessary
         * @returns the prefiltered texture
         */
        static CreateFromPrefilteredData(url: string, scene: Scene, forcedExtension?: any, createPolynomials?: boolean): CubeTexture;
        /**
         * Creates a cube texture to use with reflection for instance. It can be based upon dds or six images as well
         * as prefiltered data.
         * @param rootUrl defines the url of the texture or the root name of the six images
         * @param sceneOrEngine defines the scene or engine the texture is attached to
         * @param extensions defines the suffixes add to the picture name in case six images are in use like _px.jpg...
         * @param noMipmap defines if mipmaps should be created or not
         * @param files defines the six files to load for the different faces in that order: px, py, pz, nx, ny, nz
         * @param onLoad defines a callback triggered at the end of the file load if no errors occurred
         * @param onError defines a callback triggered in case of error during load
         * @param format defines the internal format to use for the texture once loaded
         * @param prefiltered defines whether or not the texture is created from prefiltered data
         * @param forcedExtension defines the extensions to use (force a special type of file to load) in case it is different from the file name
         * @param createPolynomials defines whether or not to create polynomial harmonics from the texture data if necessary
         * @param lodScale defines the scale applied to environment texture. This manages the range of LOD level used for IBL according to the roughness
         * @param lodOffset defines the offset applied to environment texture. This manages first LOD level used for IBL according to the roughness
         * @param loaderOptions options to be passed to the loader
         * @param useSRGBBuffer Defines if the texture must be loaded in a sRGB GPU buffer (if supported by the GPU) (default: false)
         * @returns the cube texture
         */
        constructor(rootUrl: string, sceneOrEngine: Scene | ThinEngine, extensions?: Nullable<string[]>, noMipmap?: boolean, files?: Nullable<string[]>, onLoad?: Nullable<() => void>, onError?: Nullable<(message?: string, exception?: any) => void>, format?: number, prefiltered?: boolean, forcedExtension?: any, createPolynomials?: boolean, lodScale?: number, lodOffset?: number, loaderOptions?: any, useSRGBBuffer?: boolean);
        /**
         * Get the current class name of the texture useful for serialization or dynamic coding.
         * @returns "CubeTexture"
         */
        getClassName(): string;
        /**
         * Update the url (and optional buffer) of this texture if url was null during construction.
         * @param url the url of the texture
         * @param forcedExtension defines the extension to use
         * @param onLoad callback called when the texture is loaded  (defaults to null)
         * @param prefiltered Defines whether the updated texture is prefiltered or not
         * @param onError callback called if there was an error during the loading process (defaults to null)
         * @param extensions defines the suffixes add to the picture name in case six images are in use like _px.jpg...
         * @param delayLoad defines if the texture should be loaded now (false by default)
         * @param files defines the six files to load for the different faces in that order: px, py, pz, nx, ny, nz
         */
        updateURL(url: string, forcedExtension?: string, onLoad?: Nullable<() => void>, prefiltered?: boolean, onError?: Nullable<(message?: string, exception?: any) => void>, extensions?: Nullable<string[]>, delayLoad?: boolean, files?: Nullable<string[]>): void;
        /**
         * Delays loading of the cube texture
         * @param forcedExtension defines the extension to use
         */
        delayLoad(forcedExtension?: string): void;
        /**
         * Returns the reflection texture matrix
         * @returns the reflection texture matrix
         */
        getReflectionTextureMatrix(): Matrix;
        /**
         * Sets the reflection texture matrix
         * @param value Reflection texture matrix
         */
        setReflectionTextureMatrix(value: Matrix): void;
        private _loadTexture;
        /**
         * Parses text to create a cube texture
         * @param parsedTexture define the serialized text to read from
         * @param scene defines the hosting scene
         * @param rootUrl defines the root url of the cube texture
         * @returns a cube texture
         */
        static Parse(parsedTexture: any, scene: Scene, rootUrl: string): CubeTexture;
        /**
         * Makes a clone, or deep copy, of the cube texture
         * @returns a new cube texture
         */
        clone(): CubeTexture;
    }


    /**
     * A class extending Texture allowing drawing on a texture
     * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/dynamicTexture
     */
    export class DynamicTexture extends Texture {
        private _generateMipMaps;
        private _canvas;
        private _context;
        /**
         * Creates a DynamicTexture
         * @param name defines the name of the texture
         * @param options provides 3 alternatives for width and height of texture, a canvas, object with width and height properties, number for both width and height
         * @param scene defines the scene where you want the texture
         * @param generateMipMaps defines the use of MinMaps or not (default is false)
         * @param samplingMode defines the sampling mode to use (default is Texture.TRILINEAR_SAMPLINGMODE)
         * @param format defines the texture format to use (default is Engine.TEXTUREFORMAT_RGBA)
         * @param invertY defines if the texture needs to be inverted on the y axis during loading
         */
        constructor(name: string, options: any, scene?: Nullable<Scene>, generateMipMaps?: boolean, samplingMode?: number, format?: number, invertY?: boolean);
        /**
         * Get the current class name of the texture useful for serialization or dynamic coding.
         * @returns "DynamicTexture"
         */
        getClassName(): string;
        /**
         * Gets the current state of canRescale
         */
        get canRescale(): boolean;
        private _recreate;
        /**
         * Scales the texture
         * @param ratio the scale factor to apply to both width and height
         */
        scale(ratio: number): void;
        /**
         * Resizes the texture
         * @param width the new width
         * @param height the new height
         */
        scaleTo(width: number, height: number): void;
        /**
         * Gets the context of the canvas used by the texture
         * @returns the canvas context of the dynamic texture
         */
        getContext(): ICanvasRenderingContext;
        /**
         * Clears the texture
         */
        clear(): void;
        /**
         * Updates the texture
         * @param invertY defines the direction for the Y axis (default is true - y increases downwards)
         * @param premulAlpha defines if alpha is stored as premultiplied (default is false)
         * @param allowGPUOptimization true to allow some specific GPU optimizations (subject to engine feature "allowGPUOptimizationsForGUI" being true)
         */
        update(invertY?: boolean, premulAlpha?: boolean, allowGPUOptimization?: boolean): void;
        /**
         * Draws text onto the texture
         * @param text defines the text to be drawn
         * @param x defines the placement of the text from the left
         * @param y defines the placement of the text from the top when invertY is true and from the bottom when false
         * @param font defines the font to be used with font-style, font-size, font-name
         * @param color defines the color used for the text
         * @param clearColor defines the color for the canvas, use null to not overwrite canvas
         * @param invertY defines the direction for the Y axis (default is true - y increases downwards)
         * @param update defines whether texture is immediately update (default is true)
         */
        drawText(text: string, x: number | null | undefined, y: number | null | undefined, font: string, color: string | null, clearColor: string | null, invertY?: boolean, update?: boolean): void;
        /**
         * Clones the texture
         * @returns the clone of the texture.
         */
        clone(): DynamicTexture;
        /**
         * Serializes the dynamic texture.  The scene should be ready before the dynamic texture is serialized
         * @returns a serialized dynamic texture object
         */
        serialize(): any;
        private static _IsCanvasElement;
        /** @internal */
        _rebuild(): void;
    }


    /**
     * This represents a texture coming from an equirectangular image supported by the web browser canvas.
     */
    export class EquiRectangularCubeTexture extends BaseTexture {
        /** The six faces of the cube. */
        private static _FacesMapping;
        private _noMipmap;
        private _onLoad;
        private _onError;
        /** The size of the cubemap. */
        private _size;
        /** The buffer of the image. */
        private _buffer;
        /** The width of the input image. */
        private _width;
        /** The height of the input image. */
        private _height;
        /** The URL to the image. */
        url: string;
        /**
         * Instantiates an EquiRectangularCubeTexture from the following parameters.
         * @param url The location of the image
         * @param scene The scene the texture will be used in
         * @param size The cubemap desired size (the more it increases the longer the generation will be)
         * @param noMipmap Forces to not generate the mipmap if true
         * @param gammaSpace Specifies if the texture will be used in gamma or linear space
         * (the PBR material requires those textures in linear space, but the standard material would require them in Gamma space)
         * @param onLoad — defines a callback called when texture is loaded
         * @param onError — defines a callback called if there is an error
         */
        constructor(url: string, scene: Scene, size: number, noMipmap?: boolean, gammaSpace?: boolean, onLoad?: Nullable<() => void>, onError?: Nullable<(message?: string, exception?: any) => void>);
        /**
         * Load the image data, by putting the image on a canvas and extracting its buffer.
         * @param loadTextureCallback
         * @param onError
         */
        private _loadImage;
        /**
         * Convert the image buffer into a cubemap and create a CubeTexture.
         */
        private _loadTexture;
        /**
         * Convert the ArrayBuffer into a Float32Array and drop the transparency channel.
         * @param buffer The ArrayBuffer that should be converted.
         * @returns The buffer as Float32Array.
         */
        private _getFloat32ArrayFromArrayBuffer;
        /**
         * Get the current class name of the texture useful for serialization or dynamic coding.
         * @returns "EquiRectangularCubeTexture"
         */
        getClassName(): string;
        /**
         * Create a clone of the current EquiRectangularCubeTexture and return it.
         * @returns A clone of the current EquiRectangularCubeTexture.
         */
        clone(): EquiRectangularCubeTexture;
    }


    /**
     * Class used to store an external texture (like GPUExternalTexture in WebGPU)
     */
    export class ExternalTexture {
        /**
         * Checks if a texture is an external or internal texture
         * @param texture the external or internal texture
         * @returns true if the texture is an external texture, else false
         */
        static IsExternalTexture(texture: ExternalTexture | InternalTexture): texture is ExternalTexture;
        private _video;
        /**
         * Get the class name of the texture.
         * @returns "ExternalTexture"
         */
        getClassName(): string;
        /**
         * Gets the underlying texture object
         */
        get underlyingResource(): any;
        /**
         * Gets a boolean indicating if the texture uses mipmaps
         */
        useMipMaps: boolean;
        /**
         * The type of the underlying texture is implementation dependent, so return "UNDEFINED" for the type
         */
        readonly type = 16;
        /**
         * Gets the unique id of this texture
         */
        readonly uniqueId: number;
        /**
         * Constructs the texture
         * @param video The video the texture should be wrapped around
         */
        constructor(video: HTMLVideoElement);
        /**
         * Get if the texture is ready to be used (downloaded, converted, mip mapped...).
         * @returns true if fully ready
         */
        isReady(): boolean;
        /**
         * Dispose the texture and release its associated resources.
         */
        dispose(): void;
    }


    /**
     * Options for texture filtering
     */
    interface IHDRFilteringOptions {
        /**
         * Scales pixel intensity for the input HDR map.
         */
        hdrScale?: number;
        /**
         * Quality of the filter. Should be `Constants.TEXTURE_FILTERING_QUALITY_OFFLINE` for prefiltering
         */
        quality?: number;
    }
    /**
     * Filters HDR maps to get correct renderings of PBR reflections
     */
    export class HDRFiltering {
        private _engine;
        private _effectRenderer;
        private _effectWrapper;
        private _lodGenerationOffset;
        private _lodGenerationScale;
        /**
         * Quality switch for prefiltering. Should be set to `Constants.TEXTURE_FILTERING_QUALITY_OFFLINE` unless
         * you care about baking speed.
         */
        quality: number;
        /**
         * Scales pixel intensity for the input HDR map.
         */
        hdrScale: number;
        /**
         * Instantiates HDR filter for reflection maps
         *
         * @param engine Thin engine
         * @param options Options
         */
        constructor(engine: ThinEngine, options?: IHDRFilteringOptions);
        private _createRenderTarget;
        private _prefilterInternal;
        private _createEffect;
        /**
         * Get a value indicating if the filter is ready to be used
         * @param texture Texture to filter
         * @returns true if the filter is ready
         */
        isReady(texture: BaseTexture): boolean;
        /**
         * Prefilters a cube texture to have mipmap levels representing roughness values.
         * Prefiltering will be invoked at the end of next rendering pass.
         * This has to be done once the map is loaded, and has not been prefiltered by a third party software.
         * See http://blog.selfshadow.com/publications/s2013-shading-course/karis/s2013_pbs_epic_notes_v2.pdf for more information
         * @param texture Texture to filter
         * @param onFinished Callback when filtering is done
         * @returns Promise called when prefiltering is done
         */
        prefilter(texture: BaseTexture, onFinished?: Nullable<() => void>): Promise<void>;
    }


    /** @internal */
    export interface HardwareTextureWrapper {
        underlyingResource: any;
        set(hardwareTexture: any): void;
        setUsage(textureSource: number, generateMipMaps: boolean, isCube: boolean, width: number, height: number): void;
        reset(): void;
        release(): void;
    }


    /**
     * This represents a texture coming from an HDR input.
     *
     * The only supported format is currently panorama picture stored in RGBE format.
     * Example of such files can be found on Poly Haven: https://polyhaven.com/hdris
     */
    export class HDRCubeTexture extends BaseTexture {
        private static _FacesMapping;
        private _generateHarmonics;
        private _noMipmap;
        private _prefilterOnLoad;
        private _textureMatrix;
        private _size;
        private _onLoad;
        private _onError;
        /**
         * The texture URL.
         */
        url: string;
        protected _isBlocking: boolean;
        /**
         * Sets whether or not the texture is blocking during loading.
         */
        set isBlocking(value: boolean);
        /**
         * Gets whether or not the texture is blocking during loading.
         */
        get isBlocking(): boolean;
        protected _rotationY: number;
        /**
         * Sets texture matrix rotation angle around Y axis in radians.
         */
        set rotationY(value: number);
        /**
         * Gets texture matrix rotation angle around Y axis radians.
         */
        get rotationY(): number;
        /**
         * Gets or sets the center of the bounding box associated with the cube texture
         * It must define where the camera used to render the texture was set
         */
        boundingBoxPosition: Vector3;
        private _boundingBoxSize;
        /**
         * Gets or sets the size of the bounding box associated with the cube texture
         * When defined, the cubemap will switch to local mode
         * @see https://community.arm.com/graphics/b/blog/posts/reflections-based-on-local-cubemaps-in-unity
         * @example https://www.babylonjs-playground.com/#RNASML
         */
        set boundingBoxSize(value: Vector3);
        get boundingBoxSize(): Vector3;
        /**
         * Observable triggered once the texture has been loaded.
         */
        onLoadObservable: Observable<HDRCubeTexture>;
        /**
         * Instantiates an HDRTexture from the following parameters.
         *
         * @param url The location of the HDR raw data (Panorama stored in RGBE format)
         * @param sceneOrEngine The scene or engine the texture will be used in
         * @param size The cubemap desired size (the more it increases the longer the generation will be)
         * @param noMipmap Forces to not generate the mipmap if true
         * @param generateHarmonics Specifies whether you want to extract the polynomial harmonics during the generation process
         * @param gammaSpace Specifies if the texture will be use in gamma or linear space (the PBR material requires those texture in linear space, but the standard material would require them in Gamma space)
         * @param prefilterOnLoad Prefilters HDR texture to allow use of this texture as a PBR reflection texture.
         * @param onLoad
         * @param onError
         */
        constructor(url: string, sceneOrEngine: Scene | ThinEngine, size: number, noMipmap?: boolean, generateHarmonics?: boolean, gammaSpace?: boolean, prefilterOnLoad?: boolean, onLoad?: Nullable<() => void>, onError?: Nullable<(message?: string, exception?: any) => void>);
        /**
         * Get the current class name of the texture useful for serialization or dynamic coding.
         * @returns "HDRCubeTexture"
         */
        getClassName(): string;
        /**
         * Occurs when the file is raw .hdr file.
         */
        private _loadTexture;
        clone(): HDRCubeTexture;
        delayLoad(): void;
        /**
         * Get the texture reflection matrix used to rotate/transform the reflection.
         * @returns the reflection matrix
         */
        getReflectionTextureMatrix(): Matrix;
        /**
         * Set the texture reflection matrix used to rotate/transform the reflection.
         * @param value Define the reflection matrix to set
         */
        setReflectionTextureMatrix(value: Matrix): void;
        /**
         * Dispose the texture and release its associated resources.
         */
        dispose(): void;
        /**
         * Parses a JSON representation of an HDR Texture in order to create the texture
         * @param parsedTexture Define the JSON representation
         * @param scene Define the scene the texture should be created in
         * @param rootUrl Define the root url in case we need to load relative dependencies
         * @returns the newly created texture after parsing
         */
        static Parse(parsedTexture: any, scene: Scene, rootUrl: string): Nullable<HDRCubeTexture>;
        serialize(): any;
    }


    /**
     * Defines the options related to the creation of an HtmlElementTexture
     */
    export interface IHtmlElementTextureOptions {
        /**
         * Defines whether mip maps should be created or not.
         */
        generateMipMaps?: boolean;
        /**
         * Defines the sampling mode of the texture.
         */
        samplingMode?: number;
        /**
         * Defines the associated texture format.
         */
        format?: number;
        /**
         * Defines the engine instance to use the texture with. It is not mandatory if you define a scene.
         */
        engine: Nullable<ThinEngine>;
        /**
         * Defines the scene the texture belongs to. It is not mandatory if you define an engine.
         */
        scene: Nullable<Scene>;
    }
    /**
     * This represents the smallest workload to use an already existing element (Canvas or Video) as a texture.
     * To be as efficient as possible depending on your constraints nothing aside the first upload
     * is automatically managed.
     * It is a cheap VideoTexture or DynamicTexture if you prefer to keep full control of the elements
     * in your application.
     *
     * As the update is not automatic, you need to call them manually.
     */
    export class HtmlElementTexture extends BaseTexture {
        /**
         * The texture URL.
         */
        element: HTMLVideoElement | HTMLCanvasElement;
        /**
         * Observable triggered once the texture has been loaded.
         */
        onLoadObservable: Observable<HtmlElementTexture>;
        private static readonly _DefaultOptions;
        private readonly _format;
        private _textureMatrix;
        private _isVideo;
        private _generateMipMaps;
        private _samplingMode;
        private _externalTexture;
        /**
         * Instantiates a HtmlElementTexture from the following parameters.
         *
         * @param name Defines the name of the texture
         * @param element Defines the video or canvas the texture is filled with
         * @param options Defines the other none mandatory texture creation options
         */
        constructor(name: string, element: HTMLVideoElement | HTMLCanvasElement, options: IHtmlElementTextureOptions);
        private _createInternalTexture;
        /**
         * Returns the texture matrix used in most of the material.
         */
        getTextureMatrix(): Matrix;
        /**
         * Updates the content of the texture.
         * @param invertY Defines whether the texture should be inverted on Y (false by default on video and true on canvas)
         */
        update(invertY?: Nullable<boolean>): void;
        /**
         * Dispose the texture and release its associated resources.
         */
        dispose(): void;
    }




    /**
     * Defines the source of the internal texture
     */
    export enum InternalTextureSource {
        /**
         * The source of the texture data is unknown
         */
        Unknown = 0,
        /**
         * Texture data comes from an URL
         */
        Url = 1,
        /**
         * Texture data is only used for temporary storage
         */
        Temp = 2,
        /**
         * Texture data comes from raw data (ArrayBuffer)
         */
        Raw = 3,
        /**
         * Texture content is dynamic (video or dynamic texture)
         */
        Dynamic = 4,
        /**
         * Texture content is generated by rendering to it
         */
        RenderTarget = 5,
        /**
         * Texture content is part of a multi render target process
         */
        MultiRenderTarget = 6,
        /**
         * Texture data comes from a cube data file
         */
        Cube = 7,
        /**
         * Texture data comes from a raw cube data
         */
        CubeRaw = 8,
        /**
         * Texture data come from a prefiltered cube data file
         */
        CubePrefiltered = 9,
        /**
         * Texture content is raw 3D data
         */
        Raw3D = 10,
        /**
         * Texture content is raw 2D array data
         */
        Raw2DArray = 11,
        /**
         * Texture content is a depth/stencil texture
         */
        DepthStencil = 12,
        /**
         * Texture data comes from a raw cube data encoded with RGBD
         */
        CubeRawRGBD = 13,
        /**
         * Texture content is a depth texture
         */
        Depth = 14
    }
    /**
     * Class used to store data associated with WebGL texture data for the engine
     * This class should not be used directly
     */
    export class InternalTexture extends TextureSampler {
        /**
         * Defines if the texture is ready
         */
        isReady: boolean;
        /**
         * Defines if the texture is a cube texture
         */
        isCube: boolean;
        /**
         * Defines if the texture contains 3D data
         */
        is3D: boolean;
        /**
         * Defines if the texture contains 2D array data
         */
        is2DArray: boolean;
        /**
         * Defines if the texture contains multiview data
         */
        isMultiview: boolean;
        /**
         * Gets the URL used to load this texture
         */
        url: string;
        /** @internal */
        _originalUrl: string;
        /**
         * Gets a boolean indicating if the texture needs mipmaps generation
         */
        generateMipMaps: boolean;
        /**
         * Gets a boolean indicating if the texture uses mipmaps
         * TODO implements useMipMaps as a separate setting from generateMipMaps
         */
        get useMipMaps(): boolean;
        set useMipMaps(value: boolean);
        /**
         * Gets the number of samples used by the texture (WebGL2+ only)
         */
        samples: number;
        /**
         * Gets the type of the texture (int, float...)
         */
        type: number;
        /**
         * Gets the format of the texture (RGB, RGBA...)
         */
        format: number;
        /**
         * Observable called when the texture is loaded
         */
        onLoadedObservable: Observable<InternalTexture>;
        /**
         * Observable called when the texture load is raising an error
         */
        onErrorObservable: Observable<Partial<{
            message: string;
            exception: any;
        }>>;
        /**
         * If this callback is defined it will be called instead of the default _rebuild function
         */
        onRebuildCallback: Nullable<(internalTexture: InternalTexture) => {
            proxy: Nullable<InternalTexture | Promise<InternalTexture>>;
            isReady: boolean;
            isAsync: boolean;
        }>;
        /**
         * Gets the width of the texture
         */
        width: number;
        /**
         * Gets the height of the texture
         */
        height: number;
        /**
         * Gets the depth of the texture
         */
        depth: number;
        /**
         * Gets the initial width of the texture (It could be rescaled if the current system does not support non power of two textures)
         */
        baseWidth: number;
        /**
         * Gets the initial height of the texture (It could be rescaled if the current system does not support non power of two textures)
         */
        baseHeight: number;
        /**
         * Gets the initial depth of the texture (It could be rescaled if the current system does not support non power of two textures)
         */
        baseDepth: number;
        /**
         * Gets a boolean indicating if the texture is inverted on Y axis
         */
        invertY: boolean;
        /** @internal */
        _invertVScale: boolean;
        /** @internal */
        _associatedChannel: number;
        /** @internal */
        _source: InternalTextureSource;
        /** @internal */
        _buffer: Nullable<string | ArrayBuffer | ArrayBufferView | HTMLImageElement | Blob | ImageBitmap>;
        /** @internal */
        _bufferView: Nullable<ArrayBufferView>;
        /** @internal */
        _bufferViewArray: Nullable<ArrayBufferView[]>;
        /** @internal */
        _bufferViewArrayArray: Nullable<ArrayBufferView[][]>;
        /** @internal */
        _size: number;
        /** @internal */
        _extension: string;
        /** @internal */
        _files: Nullable<string[]>;
        /** @internal */
        _workingCanvas: Nullable<ICanvas>;
        /** @internal */
        _workingContext: Nullable<ICanvasRenderingContext>;
        /** @internal */
        _cachedCoordinatesMode: Nullable<number>;
        /** @internal */
        _isDisabled: boolean;
        /** @internal */
        _compression: Nullable<string>;
        /** @internal */
        _sphericalPolynomial: Nullable<SphericalPolynomial>;
        /** @internal */
        _sphericalPolynomialPromise: Nullable<Promise<SphericalPolynomial>>;
        /** @internal */
        _sphericalPolynomialComputed: boolean;
        /** @internal */
        _lodGenerationScale: number;
        /** @internal */
        _lodGenerationOffset: number;
        /** @internal */
        _useSRGBBuffer: boolean;
        /** @internal */
        _lodTextureHigh: Nullable<BaseTexture>;
        /** @internal */
        _lodTextureMid: Nullable<BaseTexture>;
        /** @internal */
        _lodTextureLow: Nullable<BaseTexture>;
        /** @internal */
        _isRGBD: boolean;
        /** @internal */
        _linearSpecularLOD: boolean;
        /** @internal */
        _irradianceTexture: Nullable<BaseTexture>;
        /** @internal */
        _hardwareTexture: Nullable<HardwareTextureWrapper>;
        /** @internal */
        _maxLodLevel: Nullable<number>;
        /** @internal */
        _references: number;
        /** @internal */
        _gammaSpace: Nullable<boolean>;
        private _engine;
        private _uniqueId;
        /** @internal */
        static _Counter: number;
        /** Gets the unique id of the internal texture */
        get uniqueId(): number;
        /** @internal */
        _setUniqueId(id: number): void;
        /**
         * Gets the Engine the texture belongs to.
         * @returns The babylon engine
         */
        getEngine(): ThinEngine;
        /**
         * Gets the data source type of the texture
         */
        get source(): InternalTextureSource;
        /**
         * Creates a new InternalTexture
         * @param engine defines the engine to use
         * @param source defines the type of data that will be used
         * @param delayAllocation if the texture allocation should be delayed (default: false)
         */
        constructor(engine: ThinEngine, source: InternalTextureSource, delayAllocation?: boolean);
        /**
         * Increments the number of references (ie. the number of Texture that point to it)
         */
        incrementReferences(): void;
        /**
         * Change the size of the texture (not the size of the content)
         * @param width defines the new width
         * @param height defines the new height
         * @param depth defines the new depth (1 by default)
         */
        updateSize(width: int, height: int, depth?: int): void;
        /** @internal */
        _rebuild(): void;
        /**
         * @internal
         */
        _swapAndDie(target: InternalTexture, swapAll?: boolean): void;
        /**
         * Dispose the current allocated resources
         */
        dispose(): void;
    }


    /**
     * This represents the required contract to create a new type of texture loader.
     */
    export interface IInternalTextureLoader {
        /**
         * Defines whether the loader supports cascade loading the different faces.
         */
        supportCascades: boolean;
        /**
         * This returns if the loader support the current file information.
         * @param extension defines the file extension of the file being loaded
         * @param mimeType defines the optional mime type of the file being loaded
         * @returns true if the loader can load the specified file
         */
        canLoad(extension: string, mimeType?: string): boolean;
        /**
         * Uploads the cube texture data to the WebGL texture. It has already been bound.
         * @param data contains the texture data
         * @param texture defines the BabylonJS internal texture
         * @param createPolynomials will be true if polynomials have been requested
         * @param onLoad defines the callback to trigger once the texture is ready
         * @param onError defines the callback to trigger in case of error
         * @param options options to be passed to the loader
         */
        loadCubeData(data: ArrayBufferView | ArrayBufferView[], texture: InternalTexture, createPolynomials: boolean, onLoad: Nullable<(data?: any) => void>, onError: Nullable<(message?: string, exception?: any) => void>, options?: any): void;
        /**
         * Uploads the 2D texture data to the WebGL texture. It has already been bound once in the callback.
         * @param data contains the texture data
         * @param texture defines the BabylonJS internal texture
         * @param callback defines the method to call once ready to upload
         * @param options options to be passed to the loader
         */
        loadData(data: ArrayBufferView, texture: InternalTexture, callback: (width: number, height: number, loadMipmap: boolean, isCompressed: boolean, done: () => void, loadFailed?: boolean) => void, options?: any): void;
    }


    export enum SourceTextureFormat {
        ETC1S = 0,
        UASTC4x4 = 1
    }
    export enum TranscodeTarget {
        ASTC_4X4_RGBA = 0,
        BC7_RGBA = 1,
        BC3_RGBA = 2,
        BC1_RGB = 3,
        PVRTC1_4_RGBA = 4,
        PVRTC1_4_RGB = 5,
        ETC2_RGBA = 6,
        ETC1_RGB = 7,
        RGBA32 = 8,
        R8 = 9,
        RG8 = 10
    }
    export enum EngineFormat {
        COMPRESSED_RGBA_BPTC_UNORM_EXT = 36492,
        COMPRESSED_RGBA_ASTC_4X4_KHR = 37808,
        COMPRESSED_RGB_S3TC_DXT1_EXT = 33776,
        COMPRESSED_RGBA_S3TC_DXT5_EXT = 33779,
        COMPRESSED_RGBA_PVRTC_4BPPV1_IMG = 35842,
        COMPRESSED_RGB_PVRTC_4BPPV1_IMG = 35840,
        COMPRESSED_RGBA8_ETC2_EAC = 37496,
        COMPRESSED_RGB8_ETC2 = 37492,
        COMPRESSED_RGB_ETC1_WEBGL = 36196,
        RGBA8Format = 32856,
        R8Format = 33321,
        RG8Format = 33323
    }
    /**
     * Leaf node of a decision tree
     * It defines the transcoding format to use to transcode the texture as well as the corresponding format to use at the engine level when creating the texture
     */
    export interface ILeaf {
        /**
         * The format to transcode to
         */
        transcodeFormat: TranscodeTarget;
        /**
         * The format to use when creating the texture at the engine level after it has been transcoded to transcodeFormat
         */
        engineFormat: EngineFormat;
        /**
         * Whether the texture must be rounded to a multiple of 4 (should normally be the case for all compressed formats). Default: true
         */
        roundToMultiple4?: boolean;
    }
    /**
     * Regular node of a decision tree
     *
     * Each property (except for "yes" and "no"), if not empty, will be checked in order to determine the next node to select.
     * If all checks are successful, the "yes" node will be selected, else the "no" node will be selected.
     */
    export interface INode {
        /**
         * The name of the capability to check. Can be one of the following:
         *      astc
         *      bptc
         *      s3tc
         *      pvrtc
         *      etc2
         *      etc1
         */
        cap?: string;
        /**
         * The name of the option to check from the options object passed to the KTX2 decode function. {@link IKTX2DecoderOptions}
         */
        option?: string;
        /**
         * Checks if alpha is present in the texture
         */
        alpha?: boolean;
        /**
         * Checks the currently selected transcoding format.
         */
        transcodeFormat?: TranscodeTarget | TranscodeTarget[];
        /**
         * Checks that the texture is a power of two
         */
        needsPowerOfTwo?: boolean;
        /**
         * The node to select if all checks are successful
         */
        yes?: INode | ILeaf;
        /**
         * The node to select if at least one check is not successful
         */
        no?: INode | ILeaf;
    }
    /**
     * Decision tree used to determine the transcoding format to use for a given source texture format
     */
    export interface IDecisionTree {
        /**
         * textureFormat can be either UASTC or ETC1S
         */
        [textureFormat: string]: INode;
    }
    /**
     * Result of the KTX2 decode function
     */
    export interface IDecodedData {
        /**
         * Width of the texture
         */
        width: number;
        /**
         * Height of the texture
         */
        height: number;
        /**
         * The format to use when creating the texture at the engine level
         * This corresponds to the engineFormat property of the leaf node of the decision tree
         */
        transcodedFormat: number;
        /**
         * List of mipmap levels.
         * The first element is the base level, the last element is the smallest mipmap level (if more than one mipmap level is present)
         */
        mipmaps: Array<IMipmap>;
        /**
         * Whether the texture data is in gamma space or not
         */
        isInGammaSpace: boolean;
        /**
         * Whether the texture has an alpha channel or not
         */
        hasAlpha: boolean;
        /**
         * The name of the transcoder used to transcode the texture
         */
        transcoderName: string;
        /**
         * The errors (if any) encountered during the decoding process
         */
        errors?: string;
    }
    /**
     * Defines a mipmap level
     */
    export interface IMipmap {
        /**
         * The data of the mipmap level
         */
        data: Uint8Array | null;
        /**
         * The width of the mipmap level
         */
        width: number;
        /**
         * The height of the mipmap level
         */
        height: number;
    }
    /**
     * The compressed texture formats supported by the browser
     */
    export interface ICompressedFormatCapabilities {
        /**
         * Whether the browser supports ASTC
         */
        astc?: boolean;
        /**
         * Whether the browser supports BPTC
         */
        bptc?: boolean;
        /**
         * Whether the browser supports S3TC
         */
        s3tc?: boolean;
        /**
         * Whether the browser supports PVRTC
         */
        pvrtc?: boolean;
        /**
         * Whether the browser supports ETC2
         */
        etc2?: boolean;
        /**
         * Whether the browser supports ETC1
         */
        etc1?: boolean;
    }
    /**
     * Options passed to the KTX2 decode function
     */
    export interface IKTX2DecoderOptions {
        /** use RGBA format if ASTC and BC7 are not available as transcoded format */
        useRGBAIfASTCBC7NotAvailableWhenUASTC?: boolean;
        /** force to always use (uncompressed) RGBA for transcoded format */
        forceRGBA?: boolean;
        /** force to always use (uncompressed) R8 for transcoded format */
        forceR8?: boolean;
        /** force to always use (uncompressed) RG8 for transcoded format */
        forceRG8?: boolean;
        /**
         * list of transcoders to bypass when looking for a suitable transcoder. The available transcoders are:
         *      UniversalTranscoder_UASTC_ASTC
         *      UniversalTranscoder_UASTC_BC7
         *      UniversalTranscoder_UASTC_RGBA_UNORM
         *      UniversalTranscoder_UASTC_RGBA_SRGB
         *      UniversalTranscoder_UASTC_R8_UNORM
         *      UniversalTranscoder_UASTC_RG8_UNORM
         *      MSCTranscoder
         */
        bypassTranscoders?: string[];
        /**
         * Custom decision tree to apply after the default decision tree has selected a transcoding format.
         * Allows the user to override the default decision tree selection.
         * The decision tree can use the INode.transcodeFormat property to base its decision on the transcoding format selected by the default decision tree.
         */
        transcodeFormatDecisionTree?: IDecisionTree;
    }


    /**
     * Loader for .basis file format
     */
    export class _BasisTextureLoader implements IInternalTextureLoader {
        /**
         * Defines whether the loader supports cascade loading the different faces.
         */
        readonly supportCascades = false;
        /**
         * This returns if the loader support the current file information.
         * @param extension defines the file extension of the file being loaded
         * @returns true if the loader can load the specified file
         */
        canLoad(extension: string): boolean;
        /**
         * Uploads the cube texture data to the WebGL texture. It has already been bound.
         * @param data contains the texture data
         * @param texture defines the BabylonJS internal texture
         * @param createPolynomials will be true if polynomials have been requested
         * @param onLoad defines the callback to trigger once the texture is ready
         * @param onError defines the callback to trigger in case of error
         */
        loadCubeData(data: ArrayBufferView | ArrayBufferView[], texture: InternalTexture, createPolynomials: boolean, onLoad: Nullable<(data?: any) => void>, onError: Nullable<(message?: string, exception?: any) => void>): void;
        /**
         * Uploads the 2D texture data to the WebGL texture. It has already been bound once in the callback.
         * @param data contains the texture data
         * @param texture defines the BabylonJS internal texture
         * @param callback defines the method to call once ready to upload
         */
        loadData(data: ArrayBufferView, texture: InternalTexture, callback: (width: number, height: number, loadMipmap: boolean, isCompressed: boolean, done: () => void, failedLoading?: boolean) => void): void;
    }


    /**
     * Implementation of the DDS Texture Loader.
     * @internal
     */
    export class _DDSTextureLoader implements IInternalTextureLoader {
        /**
         * Defines whether the loader supports cascade loading the different faces.
         */
        readonly supportCascades = true;
        /**
         * This returns if the loader support the current file information.
         * @param extension defines the file extension of the file being loaded
         * @returns true if the loader can load the specified file
         */
        canLoad(extension: string): boolean;
        /**
         * Uploads the cube texture data to the WebGL texture. It has already been bound.
         * @param imgs contains the cube maps
         * @param texture defines the BabylonJS internal texture
         * @param createPolynomials will be true if polynomials have been requested
         * @param onLoad defines the callback to trigger once the texture is ready
         */
        loadCubeData(imgs: ArrayBufferView | ArrayBufferView[], texture: InternalTexture, createPolynomials: boolean, onLoad: Nullable<(data?: any) => void>): void;
        /**
         * Uploads the 2D texture data to the WebGL texture. It has already been bound once in the callback.
         * @param data contains the texture data
         * @param texture defines the BabylonJS internal texture
         * @param callback defines the method to call once ready to upload
         */
        loadData(data: ArrayBufferView, texture: InternalTexture, callback: (width: number, height: number, loadMipmap: boolean, isCompressed: boolean, done: () => void) => void): void;
    }


    /**
     * Implementation of the ENV Texture Loader.
     * @internal
     */
    export class _ENVTextureLoader implements IInternalTextureLoader {
        /**
         * Defines whether the loader supports cascade loading the different faces.
         */
        readonly supportCascades = false;
        /**
         * This returns if the loader support the current file information.
         * @param extension defines the file extension of the file being loaded
         * @returns true if the loader can load the specified file
         */
        canLoad(extension: string): boolean;
        /**
         * Uploads the cube texture data to the WebGL texture. It has already been bound.
         * @param data contains the texture data
         * @param texture defines the BabylonJS internal texture
         * @param createPolynomials will be true if polynomials have been requested
         * @param onLoad defines the callback to trigger once the texture is ready
         * @param onError defines the callback to trigger in case of error
         */
        loadCubeData(data: ArrayBufferView | ArrayBufferView[], texture: InternalTexture, createPolynomials: boolean, onLoad: Nullable<(data?: any) => void>, onError: Nullable<(message?: string, exception?: any) => void>): void;
        /**
         * Uploads the 2D texture data to the WebGL texture. It has already been bound once in the callback.
         */
        loadData(): void;
    }


    /**
     * Implementation of the HDR Texture Loader.
     * @internal
     */
    export class _HDRTextureLoader implements IInternalTextureLoader {
        /**
         * Defines whether the loader supports cascade loading the different faces.
         */
        readonly supportCascades = false;
        /**
         * This returns if the loader support the current file information.
         * @param extension defines the file extension of the file being loaded
         * @returns true if the loader can load the specified file
         */
        canLoad(extension: string): boolean;
        /**
         * Uploads the cube texture data to the WebGL texture. It has already been bound.
         */
        loadCubeData(): void;
        /**
         * Uploads the 2D texture data to the WebGL texture. It has already been bound once in the callback.
         * @param data contains the texture data
         * @param texture defines the BabylonJS internal texture
         * @param callback defines the method to call once ready to upload
         */
        loadData(data: ArrayBufferView, texture: InternalTexture, callback: (width: number, height: number, loadMipmap: boolean, isCompressed: boolean, done: () => void) => void): void;
    }




    /**
     * Implementation of the KTX Texture Loader.
     * @internal
     */
    export class _KTXTextureLoader implements IInternalTextureLoader {
        /**
         * Defines whether the loader supports cascade loading the different faces.
         */
        readonly supportCascades = false;
        /**
         * This returns if the loader support the current file information.
         * @param extension defines the file extension of the file being loaded
         * @param mimeType defines the optional mime type of the file being loaded
         * @returns true if the loader can load the specified file
         */
        canLoad(extension: string, mimeType?: string): boolean;
        /**
         * Uploads the cube texture data to the WebGL texture. It has already been bound.
         * @param data contains the texture data
         * @param texture defines the BabylonJS internal texture
         * @param createPolynomials will be true if polynomials have been requested
         * @param onLoad defines the callback to trigger once the texture is ready
         */
        loadCubeData(data: ArrayBufferView | ArrayBufferView[], texture: InternalTexture, createPolynomials: boolean, onLoad: Nullable<(data?: any) => void>): void;
        /**
         * Uploads the 2D texture data to the WebGL texture. It has already been bound once in the callback.
         * @param data contains the texture data
         * @param texture defines the BabylonJS internal texture
         * @param callback defines the method to call once ready to upload
         * @param options
         */
        loadData(data: ArrayBufferView, texture: InternalTexture, callback: (width: number, height: number, loadMipmap: boolean, isCompressed: boolean, done: () => void, loadFailed: boolean) => void, options?: any): void;
    }


    /**
     * Implementation of the TGA Texture Loader.
     * @internal
     */
    export class _TGATextureLoader implements IInternalTextureLoader {
        /**
         * Defines whether the loader supports cascade loading the different faces.
         */
        readonly supportCascades = false;
        /**
         * This returns if the loader support the current file information.
         * @param extension defines the file extension of the file being loaded
         * @returns true if the loader can load the specified file
         */
        canLoad(extension: string): boolean;
        /**
         * Uploads the cube texture data to the WebGL texture. It has already been bound.
         */
        loadCubeData(): void;
        /**
         * Uploads the 2D texture data to the WebGL texture. It has already been bound once in the callback.
         * @param data contains the texture data
         * @param texture defines the BabylonJS internal texture
         * @param callback defines the method to call once ready to upload
         */
        loadData(data: ArrayBufferView, texture: InternalTexture, callback: (width: number, height: number, loadMipmap: boolean, isCompressed: boolean, done: () => void) => void): void;
    }


    /**
     * Mirror texture can be used to simulate the view from a mirror in a scene.
     * It will dynamically be rendered every frame to adapt to the camera point of view.
     * You can then easily use it as a reflectionTexture on a flat surface.
     * In case the surface is not a plane, please consider relying on reflection probes.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/reflectionTexture#mirrortexture
     */
    export class MirrorTexture extends RenderTargetTexture {
        /**
         * Define the reflection plane we want to use. The mirrorPlane is usually set to the constructed reflector.
         * It is possible to directly set the mirrorPlane by directly using a Plane(a, b, c, d) where a, b and c give the plane normal vector (a, b, c) and d is a scalar displacement from the mirrorPlane to the origin. However in all but the very simplest of situations it is more straight forward to set it to the reflector as stated in the doc.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/reflectionTexture#mirrors
         */
        mirrorPlane: Plane;
        /**
         * Define the blur ratio used to blur the reflection if needed.
         */
        set blurRatio(value: number);
        get blurRatio(): number;
        /**
         * Define the adaptive blur kernel used to blur the reflection if needed.
         * This will autocompute the closest best match for the `blurKernel`
         */
        set adaptiveBlurKernel(value: number);
        /**
         * Define the blur kernel used to blur the reflection if needed.
         * Please consider using `adaptiveBlurKernel` as it could find the closest best value for you.
         */
        set blurKernel(value: number);
        /**
         * Define the blur kernel on the X Axis used to blur the reflection if needed.
         * Please consider using `adaptiveBlurKernel` as it could find the closest best value for you.
         */
        set blurKernelX(value: number);
        get blurKernelX(): number;
        /**
         * Define the blur kernel on the Y Axis used to blur the reflection if needed.
         * Please consider using `adaptiveBlurKernel` as it could find the closest best value for you.
         */
        set blurKernelY(value: number);
        get blurKernelY(): number;
        private _autoComputeBlurKernel;
        protected _onRatioRescale(): void;
        private _updateGammaSpace;
        private _imageProcessingConfigChangeObserver;
        private _transformMatrix;
        private _mirrorMatrix;
        private _blurX;
        private _blurY;
        private _adaptiveBlurKernel;
        private _blurKernelX;
        private _blurKernelY;
        private _blurRatio;
        private _sceneUBO;
        private _currentSceneUBO;
        /**
         * Instantiates a Mirror Texture.
         * Mirror texture can be used to simulate the view from a mirror in a scene.
         * It will dynamically be rendered every frame to adapt to the camera point of view.
         * You can then easily use it as a reflectionTexture on a flat surface.
         * In case the surface is not a plane, please consider relying on reflection probes.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/reflectionTexture#mirrors
         * @param name
         * @param size
         * @param scene
         * @param generateMipMaps
         * @param type
         * @param samplingMode
         * @param generateDepthBuffer
         */
        constructor(name: string, size: number | {
            width: number;
            height: number;
        } | {
            ratio: number;
        }, scene?: Scene, generateMipMaps?: boolean, type?: number, samplingMode?: number, generateDepthBuffer?: boolean);
        private _preparePostProcesses;
        /**
         * Clone the mirror texture.
         * @returns the cloned texture
         */
        clone(): MirrorTexture;
        /**
         * Serialize the texture to a JSON representation you could use in Parse later on
         * @returns the serialized JSON representation
         */
        serialize(): any;
        /**
         * Dispose the texture and release its associated resources.
         */
        dispose(): void;
    }


    /**
     * Creation options of the multi render target texture.
     */
    export interface IMultiRenderTargetOptions {
        /**
         * Define if the texture needs to create mip maps after render.
         */
        generateMipMaps?: boolean;
        /**
         * Define the types of all the draw buffers we want to create
         */
        types?: number[];
        /**
         * Define the sampling modes of all the draw buffers we want to create
         */
        samplingModes?: number[];
        /**
         * Define if sRGB format should be used for each of the draw buffers we want to create
         */
        useSRGBBuffers?: boolean[];
        /**
         * Define if a depth buffer is required
         */
        generateDepthBuffer?: boolean;
        /**
         * Define if a stencil buffer is required
         */
        generateStencilBuffer?: boolean;
        /**
         * Define if a depth texture is required instead of a depth buffer
         */
        generateDepthTexture?: boolean;
        /**
         * Define depth texture format to use
         */
        depthTextureFormat?: number;
        /**
         * Define the number of desired draw buffers
         */
        textureCount?: number;
        /**
         * Define if aspect ratio should be adapted to the texture or stay the scene one
         */
        doNotChangeAspectRatio?: boolean;
        /**
         * Define the default type of the buffers we are creating
         */
        defaultType?: number;
        /**
         * Define the default type of the buffers we are creating
         */
        drawOnlyOnFirstAttachmentByDefault?: boolean;
    }
    /**
     * A multi render target, like a render target provides the ability to render to a texture.
     * Unlike the render target, it can render to several draw buffers in one draw.
     * This is specially interesting in deferred rendering or for any effects requiring more than
     * just one color from a single pass.
     */
    export class MultiRenderTarget extends RenderTargetTexture {
        private _textures;
        private _multiRenderTargetOptions;
        private _count;
        private _drawOnlyOnFirstAttachmentByDefault;
        /**
         * Get if draw buffers are currently supported by the used hardware and browser.
         */
        get isSupported(): boolean;
        /**
         * Get the list of textures generated by the multi render target.
         */
        get textures(): Texture[];
        /**
         * Gets the number of textures in this MRT. This number can be different from `_textures.length` in case a depth texture is generated.
         */
        get count(): number;
        /**
         * Get the depth texture generated by the multi render target if options.generateDepthTexture has been set
         */
        get depthTexture(): Texture;
        /**
         * Set the wrapping mode on U of all the textures we are rendering to.
         * Can be any of the Texture. (CLAMP_ADDRESSMODE, MIRROR_ADDRESSMODE or WRAP_ADDRESSMODE)
         */
        set wrapU(wrap: number);
        /**
         * Set the wrapping mode on V of all the textures we are rendering to.
         * Can be any of the Texture. (CLAMP_ADDRESSMODE, MIRROR_ADDRESSMODE or WRAP_ADDRESSMODE)
         */
        set wrapV(wrap: number);
        /**
         * Instantiate a new multi render target texture.
         * A multi render target, like a render target provides the ability to render to a texture.
         * Unlike the render target, it can render to several draw buffers in one draw.
         * This is specially interesting in deferred rendering or for any effects requiring more than
         * just one color from a single pass.
         * @param name Define the name of the texture
         * @param size Define the size of the buffers to render to
         * @param count Define the number of target we are rendering into
         * @param scene Define the scene the texture belongs to
         * @param options Define the options used to create the multi render target
         * @param textureNames Define the names to set to the textures (if count > 0 - optional)
         */
        constructor(name: string, size: any, count: number, scene?: Scene, options?: IMultiRenderTargetOptions, textureNames?: string[]);
        private _initTypes;
        /**
         * @internal
         */
        _rebuild(forceFullRebuild?: boolean, textureNames?: string[]): void;
        private _createInternalTextures;
        private _releaseTextures;
        private _createTextures;
        /**
         * Replaces an internal texture within the MRT. Useful to share textures between MultiRenderTarget.
         * @param texture The new texture to set in the MRT
         * @param index The index of the texture to replace
         * @param disposePrevious Set to true if the previous internal texture should be disposed
         */
        setInternalTexture(texture: InternalTexture, index: number, disposePrevious?: boolean): void;
        /**
         * Define the number of samples used if MSAA is enabled.
         */
        get samples(): number;
        set samples(value: number);
        /**
         * Resize all the textures in the multi render target.
         * Be careful as it will recreate all the data in the new texture.
         * @param size Define the new size
         */
        resize(size: any): void;
        /**
         * Changes the number of render targets in this MRT
         * Be careful as it will recreate all the data in the new texture.
         * @param count new texture count
         * @param options Specifies texture types and sampling modes for new textures
         * @param textureNames Specifies the names of the textures (optional)
         */
        updateCount(count: number, options?: IMultiRenderTargetOptions, textureNames?: string[]): void;
        protected _unbindFrameBuffer(engine: Engine, faceIndex: number): void;
        /**
         * Dispose the render targets and their associated resources
         * @param doNotDisposeInternalTextures
         */
        dispose(doNotDisposeInternalTextures?: boolean): void;
        /**
         * Release all the underlying texture used as draw buffers.
         */
        releaseInternalTextures(): void;
    }


    /**
     * Renders to multiple views with a single draw call
     * @see https://www.khronos.org/registry/webgl/extensions/OVR_multiview2/
     */
    export class MultiviewRenderTarget extends RenderTargetTexture {
        set samples(value: number);
        get samples(): number;
        /**
         * Creates a multiview render target
         * @param scene scene used with the render target
         * @param size the size of the render target (used for each view)
         */
        constructor(scene?: Scene, size?: number | {
            width: number;
            height: number;
        } | {
            ratio: number;
        });
        /**
         * @internal
         */
        _bindFrameBuffer(): void;
        /**
         * Gets the number of views the corresponding to the texture (eg. a MultiviewRenderTarget will have > 1)
         * @returns the view count
         */
        getViewCount(): number;
    }


    /**
     * Defines the basic options interface of a TexturePacker Frame
     */
    export interface ITexturePackerFrame {
        /**
         * The frame ID
         */
        id: number;
        /**
         * The frames Scale
         */
        scale: Vector2;
        /**
         * The Frames offset
         */
        offset: Vector2;
    }
    /**
     * This is a support class for frame Data on texture packer sets.
     */
    export class TexturePackerFrame implements ITexturePackerFrame {
        /**
         * The frame ID
         */
        id: number;
        /**
         * The frames Scale
         */
        scale: Vector2;
        /**
         * The Frames offset
         */
        offset: Vector2;
        /**
         * Initializes a texture package frame.
         * @param id The numerical frame identifier
         * @param scale Scalar Vector2 for UV frame
         * @param offset Vector2 for the frame position in UV units.
         * @returns TexturePackerFrame
         */
        constructor(id: number, scale: Vector2, offset: Vector2);
    }




    /**
     * Defines the basic options interface of a TexturePacker
     */
    export interface ITexturePackerOptions {
        /**
         * Custom targets for the channels of a texture packer.  Default is all the channels of the Standard Material
         */
        map?: string[];
        /**
         * the UV input targets, as a single value for all meshes. Defaults to VertexBuffer.UVKind
         */
        uvsIn?: string;
        /**
         * the UV output targets, as a single value for all meshes.  Defaults to VertexBuffer.UVKind
         */
        uvsOut?: string;
        /**
         * number representing the layout style. Defaults to LAYOUT_STRIP
         */
        layout?: number;
        /**
         * number of columns if using custom column count layout(2).  This defaults to 4.
         */
        colnum?: number;
        /**
         * flag to update the input meshes to the new packed texture after compilation. Defaults to true.
         */
        updateInputMeshes?: boolean;
        /**
         * boolean flag to dispose all the source textures.  Defaults to true.
         */
        disposeSources?: boolean;
        /**
         * Fills the blank cells in a set to the customFillColor.  Defaults to true.
         */
        fillBlanks?: boolean;
        /**
         * string value representing the context fill style color.  Defaults to 'black'.
         */
        customFillColor?: string;
        /**
         * Width and Height Value of each Frame in the TexturePacker Sets
         */
        frameSize?: number;
        /**
         * Ratio of the value to add padding wise to each cell.  Defaults to 0.0115
         */
        paddingRatio?: number;
        /**
         * Number that declares the fill method for the padding gutter.
         */
        paddingMode?: number;
        /**
         * If in SUBUV_COLOR padding mode what color to use.
         */
        paddingColor?: Color3 | Color4;
    }
    /**
     * Defines the basic interface of a TexturePacker JSON File
     */
    export interface ITexturePackerJSON {
        /**
         * The frame ID
         */
        name: string;
        /**
         * The base64 channel data
         */
        sets: any;
        /**
         * The options of the Packer
         */
        options: ITexturePackerOptions;
        /**
         * The frame data of the Packer
         */
        frames: Array<number>;
    }
    /**
     * This is a support class that generates a series of packed texture sets.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/materials_introduction
     */
    export class TexturePacker {
        /** Packer Layout Constant 0 */
        static readonly LAYOUT_STRIP = 0;
        /** Packer Layout Constant 1 */
        static readonly LAYOUT_POWER2 = 1;
        /** Packer Layout Constant 2 */
        static readonly LAYOUT_COLNUM = 2;
        /** Packer Layout Constant 0 */
        static readonly SUBUV_WRAP = 0;
        /** Packer Layout Constant 1 */
        static readonly SUBUV_EXTEND = 1;
        /** Packer Layout Constant 2 */
        static readonly SUBUV_COLOR = 2;
        /** The Name of the Texture Package */
        name: string;
        /** The scene scope of the TexturePacker */
        scene: Scene;
        /** The Meshes to target */
        meshes: AbstractMesh[];
        /** Arguments passed with the Constructor */
        options: ITexturePackerOptions;
        /** The promise that is started upon initialization */
        promise: Nullable<Promise<TexturePacker | string>>;
        /** The Container object for the channel sets that are generated */
        sets: object;
        /** The Container array for the frames that are generated */
        frames: TexturePackerFrame[];
        /** The expected number of textures the system is parsing. */
        private _expecting;
        /** The padding value from Math.ceil(frameSize * paddingRatio) */
        private _paddingValue;
        /**
         * Initializes a texture package series from an array of meshes or a single mesh.
         * @param name The name of the package
         * @param meshes The target meshes to compose the package from
         * @param options The arguments that texture packer should follow while building.
         * @param scene The scene which the textures are scoped to.
         * @returns TexturePacker
         */
        constructor(name: string, meshes: AbstractMesh[], options: ITexturePackerOptions, scene: Scene);
        /**
         * Starts the package process
         * @param resolve The promises resolution function
         * @returns TexturePacker
         */
        private _createFrames;
        /**
         * Calculates the Size of the Channel Sets
         * @returns Vector2
         */
        private _calculateSize;
        /**
         * Calculates the UV data for the frames.
         * @param baseSize the base frameSize
         * @param padding the base frame padding
         * @param dtSize size of the Dynamic Texture for that channel
         * @param dtUnits is 1/dtSize
         * @param update flag to update the input meshes
         */
        private _calculateMeshUVFrames;
        /**
         * Calculates the frames Offset.
         * @param index of the frame
         * @returns Vector2
         */
        private _getFrameOffset;
        /**
         * Updates a Mesh to the frame data
         * @param mesh that is the target
         * @param frameID or the frame index
         */
        private _updateMeshUV;
        /**
         * Updates a Meshes materials to use the texture packer channels
         * @param m is the mesh to target
         * @param force all channels on the packer to be set.
         */
        private _updateTextureReferences;
        /**
         * Public method to set a Mesh to a frame
         * @param m that is the target
         * @param frameID or the frame index
         * @param updateMaterial trigger for if the Meshes attached Material be updated?
         */
        setMeshToFrame(m: AbstractMesh, frameID: number, updateMaterial?: boolean): void;
        /**
         * Starts the async promise to compile the texture packer.
         * @returns Promise<void>
         */
        processAsync(): Promise<void>;
        /**
         * Disposes all textures associated with this packer
         */
        dispose(): void;
        /**
         * Starts the download process for all the channels converting them to base64 data and embedding it all in a JSON file.
         * @param imageType is the image type to use.
         * @param quality of the image if downloading as jpeg, Ranges from >0 to 1.
         */
        download(imageType?: string, quality?: number): void;
        /**
         * Public method to load a texturePacker JSON file.
         * @param data of the JSON file in string format.
         */
        updateFromJSON(data: string): void;
    }


    /**
     * A multi render target designed to render the prepass.
     * Prepass is a scene component used to render information in multiple textures
     * alongside with the scene materials rendering.
     * Note : This is an internal class, and you should NOT need to instanciate this.
     * Only the `PrePassRenderer` should instanciate this class.
     * It is more likely that you need a regular `MultiRenderTarget`
     * @internal
     */
    export class PrePassRenderTarget extends MultiRenderTarget {
        /**
         * @internal
         */
        _beforeCompositionPostProcesses: PostProcess[];
        /**
         * Image processing post process for composition
         */
        imageProcessingPostProcess: ImageProcessingPostProcess;
        /**
         * @internal
         */
        _engine: Engine;
        /**
         * @internal
         */
        _scene: Scene;
        /**
         * @internal
         */
        _outputPostProcess: Nullable<PostProcess>;
        /**
         * @internal
         */
        _internalTextureDirty: boolean;
        /**
         * Is this render target enabled for prepass rendering
         */
        enabled: boolean;
        /**
         * Render target associated with this prePassRenderTarget
         * If this is `null`, it means this prePassRenderTarget is associated with the scene
         */
        renderTargetTexture: Nullable<RenderTargetTexture>;
        constructor(name: string, renderTargetTexture: Nullable<RenderTargetTexture>, size: any, count: number, scene?: Scene, options?: IMultiRenderTargetOptions | undefined);
        /**
         * Creates a composition effect for this RT
         * @internal
         */
        _createCompositionEffect(): void;
        /**
         * Checks that the size of this RT is still adapted to the desired render size.
         * @internal
         */
        _checkSize(): void;
        /**
         * Changes the number of render targets in this MRT
         * Be careful as it will recreate all the data in the new texture.
         * @param count new texture count
         * @param options Specifies texture types and sampling modes for new textures
         * @param textureNames Specifies the names of the textures (optional)
         */
        updateCount(count: number, options?: IMultiRenderTargetOptions, textureNames?: string[]): void;
        /**
         * Resets the post processes chains applied to this RT.
         * @internal
         */
        _resetPostProcessChain(): void;
        /**
         * Diposes this render target
         */
        dispose(): void;
    }


    /**
     * Procedural texturing is a way to programmatically create a texture. There are 2 types of procedural textures: code-only, and code that references some classic 2D images, sometimes called 'refMaps' or 'sampler' images.
     * Custom Procedural textures are the easiest way to create your own procedural in your application.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/proceduralTextures#creating-custom-procedural-textures
     */
    export class CustomProceduralTexture extends ProceduralTexture {
        private _animate;
        private _time;
        private _config;
        private _texturePath;
        /**
         * Instantiates a new Custom Procedural Texture.
         * Procedural texturing is a way to programmatically create a texture. There are 2 types of procedural textures: code-only, and code that references some classic 2D images, sometimes called 'refMaps' or 'sampler' images.
         * Custom Procedural textures are the easiest way to create your own procedural in your application.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/proceduralTextures#creating-custom-procedural-textures
         * @param name Define the name of the texture
         * @param texturePath Define the folder path containing all the custom texture related files (config, shaders...)
         * @param size Define the size of the texture to create
         * @param scene Define the scene the texture belongs to
         * @param fallbackTexture Define a fallback texture in case there were issues to create the custom texture
         * @param generateMipMaps Define if the texture should creates mip maps or not
         * @param skipJson Define a boolena indicating that there is no json config file to load
         */
        constructor(name: string, texturePath: string, size: TextureSize, scene: Scene, fallbackTexture?: Texture, generateMipMaps?: boolean, skipJson?: boolean);
        private _loadJson;
        /**
         * Is the texture ready to be used ? (rendered at least once)
         * @returns true if ready, otherwise, false.
         */
        isReady(): boolean;
        /**
         * Render the texture to its associated render target.
         * @param useCameraPostProcess Define if camera post process should be applied to the texture
         */
        render(useCameraPostProcess?: boolean): void;
        /**
         * Update the list of dependant textures samplers in the shader.
         */
        updateTextures(): void;
        /**
         * Update the uniform values of the procedural texture in the shader.
         */
        updateShaderUniforms(): void;
        /**
         * Define if the texture animates or not.
         */
        get animate(): boolean;
        set animate(value: boolean);
    }




    /**
     * Class used to generate noise procedural textures
     */
    export class NoiseProceduralTexture extends ProceduralTexture {
        /** Gets or sets the start time (default is 0) */
        time: number;
        /** Gets or sets a value between 0 and 1 indicating the overall brightness of the texture (default is 0.2) */
        brightness: number;
        /** Defines the number of octaves to process */
        octaves: number;
        /** Defines the level of persistence (0.8 by default) */
        persistence: number;
        /** Gets or sets animation speed factor (default is 1) */
        animationSpeedFactor: number;
        /**
         * Creates a new NoiseProceduralTexture
         * @param name defines the name fo the texture
         * @param size defines the size of the texture (default is 256)
         * @param scene defines the hosting scene
         * @param fallbackTexture defines the texture to use if the NoiseProceduralTexture can't be created
         * @param generateMipMaps defines if mipmaps must be generated (true by default)
         */
        constructor(name: string, size?: number, scene?: Nullable<Scene>, fallbackTexture?: Texture, generateMipMaps?: boolean);
        private _updateShaderUniforms;
        protected _getDefines(): string;
        /**
         * Generate the current state of the procedural texture
         * @param useCameraPostProcess
         */
        render(useCameraPostProcess?: boolean): void;
        /**
         * Serializes this noise procedural texture
         * @returns a serialized noise procedural texture object
         */
        serialize(): any;
        /**
         * Clone the texture.
         * @returns the cloned texture
         */
        clone(): NoiseProceduralTexture;
        /**
         * Creates a NoiseProceduralTexture from parsed noise procedural texture data
         * @param parsedTexture defines parsed texture data
         * @param scene defines the current scene
         * @returns a parsed NoiseProceduralTexture
         */
        static Parse(parsedTexture: any, scene: Scene): NoiseProceduralTexture;
    }


    /**
     * Procedural texturing is a way to programmatically create a texture. There are 2 types of procedural textures: code-only, and code that references some classic 2D images, sometimes calmpler' images.
     * This is the base class of any Procedural texture and contains most of the shareable code.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/proceduralTextures
     */
    export class ProceduralTexture extends Texture {
        /**
         * Define if the texture is enabled or not (disabled texture will not render)
         */
        isEnabled: boolean;
        /**
         * Define if the texture must be cleared before rendering (default is true)
         */
        autoClear: boolean;
        /**
         * Callback called when the texture is generated
         */
        onGenerated: () => void;
        /**
         * Event raised when the texture is generated
         */
        onGeneratedObservable: Observable<ProceduralTexture>;
        /**
         * Event raised before the texture is generated
         */
        onBeforeGenerationObservable: Observable<ProceduralTexture>;
        /**
         * Gets or sets the node material used to create this texture (null if the texture was manually created)
         */
        nodeMaterialSource: Nullable<NodeMaterial>;
        /** @internal */
        _generateMipMaps: boolean;
        private _drawWrapper;
        /** @internal */
        _textures: {
            [key: string]: Texture;
        };
        /** @internal */
        protected _fallbackTexture: Nullable<Texture>;
        private _size;
        private _textureType;
        private _currentRefreshId;
        private _frameId;
        private _refreshRate;
        private _vertexBuffers;
        private _indexBuffer;
        private _uniforms;
        private _samplers;
        private _fragment;
        private _floats;
        private _ints;
        private _floatsArrays;
        private _colors3;
        private _colors4;
        private _vectors2;
        private _vectors3;
        private _matrices;
        private _fallbackTextureUsed;
        private _fullEngine;
        private _cachedDefines;
        private _contentUpdateId;
        private _contentData;
        private _rtWrapper;
        /**
         * Instantiates a new procedural texture.
         * Procedural texturing is a way to programmatically create a texture. There are 2 types of procedural textures: code-only, and code that references some classic 2D images, sometimes called 'refMaps' or 'sampler' images.
         * This is the base class of any Procedural texture and contains most of the shareable code.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/proceduralTextures
         * @param name  Define the name of the texture
         * @param size Define the size of the texture to create
         * @param fragment Define the fragment shader to use to generate the texture or null if it is defined later
         * @param scene Define the scene the texture belongs to
         * @param fallbackTexture Define a fallback texture in case there were issues to create the custom texture
         * @param generateMipMaps Define if the texture should creates mip maps or not
         * @param isCube Define if the texture is a cube texture or not (this will render each faces of the cube)
         * @param textureType The FBO internal texture type
         */
        constructor(name: string, size: TextureSize, fragment: any, scene: Nullable<Scene>, fallbackTexture?: Nullable<Texture>, generateMipMaps?: boolean, isCube?: boolean, textureType?: number);
        private _createRtWrapper;
        /**
         * The effect that is created when initializing the post process.
         * @returns The created effect corresponding the the postprocess.
         */
        getEffect(): Effect;
        /**
         * @internal*
         */
        _setEffect(effect: Effect): void;
        /**
         * Gets texture content (Use this function wisely as reading from a texture can be slow)
         * @returns an ArrayBufferView promise (Uint8Array or Float32Array)
         */
        getContent(): Nullable<Promise<ArrayBufferView>>;
        private _createIndexBuffer;
        /** @internal */
        _rebuild(): void;
        /**
         * Resets the texture in order to recreate its associated resources.
         * This can be called in case of context loss
         */
        reset(): void;
        protected _getDefines(): string;
        /**
         * Is the texture ready to be used ? (rendered at least once)
         * @returns true if ready, otherwise, false.
         */
        isReady(): boolean;
        /**
         * Resets the refresh counter of the texture and start bak from scratch.
         * Could be useful to regenerate the texture if it is setup to render only once.
         */
        resetRefreshCounter(): void;
        /**
         * Set the fragment shader to use in order to render the texture.
         * @param fragment This can be set to a path (into the shader store) or to a json object containing a fragmentElement property.
         */
        setFragment(fragment: any): void;
        /**
         * Define the refresh rate of the texture or the rendering frequency.
         * Use 0 to render just once, 1 to render on every frame, 2 to render every two frames and so on...
         */
        get refreshRate(): number;
        set refreshRate(value: number);
        /** @internal */
        _shouldRender(): boolean;
        /**
         * Get the size the texture is rendering at.
         * @returns the size (on cube texture it is always squared)
         */
        getRenderSize(): TextureSize;
        /**
         * Resize the texture to new value.
         * @param size Define the new size the texture should have
         * @param generateMipMaps Define whether the new texture should create mip maps
         */
        resize(size: TextureSize, generateMipMaps: boolean): void;
        private _checkUniform;
        /**
         * Set a texture in the shader program used to render.
         * @param name Define the name of the uniform samplers as defined in the shader
         * @param texture Define the texture to bind to this sampler
         * @returns the texture itself allowing "fluent" like uniform updates
         */
        setTexture(name: string, texture: Texture): ProceduralTexture;
        /**
         * Set a float in the shader.
         * @param name Define the name of the uniform as defined in the shader
         * @param value Define the value to give to the uniform
         * @returns the texture itself allowing "fluent" like uniform updates
         */
        setFloat(name: string, value: number): ProceduralTexture;
        /**
         * Set a int in the shader.
         * @param name Define the name of the uniform as defined in the shader
         * @param value Define the value to give to the uniform
         * @returns the texture itself allowing "fluent" like uniform updates
         */
        setInt(name: string, value: number): ProceduralTexture;
        /**
         * Set an array of floats in the shader.
         * @param name Define the name of the uniform as defined in the shader
         * @param value Define the value to give to the uniform
         * @returns the texture itself allowing "fluent" like uniform updates
         */
        setFloats(name: string, value: number[]): ProceduralTexture;
        /**
         * Set a vec3 in the shader from a Color3.
         * @param name Define the name of the uniform as defined in the shader
         * @param value Define the value to give to the uniform
         * @returns the texture itself allowing "fluent" like uniform updates
         */
        setColor3(name: string, value: Color3): ProceduralTexture;
        /**
         * Set a vec4 in the shader from a Color4.
         * @param name Define the name of the uniform as defined in the shader
         * @param value Define the value to give to the uniform
         * @returns the texture itself allowing "fluent" like uniform updates
         */
        setColor4(name: string, value: Color4): ProceduralTexture;
        /**
         * Set a vec2 in the shader from a Vector2.
         * @param name Define the name of the uniform as defined in the shader
         * @param value Define the value to give to the uniform
         * @returns the texture itself allowing "fluent" like uniform updates
         */
        setVector2(name: string, value: Vector2): ProceduralTexture;
        /**
         * Set a vec3 in the shader from a Vector3.
         * @param name Define the name of the uniform as defined in the shader
         * @param value Define the value to give to the uniform
         * @returns the texture itself allowing "fluent" like uniform updates
         */
        setVector3(name: string, value: Vector3): ProceduralTexture;
        /**
         * Set a mat4 in the shader from a MAtrix.
         * @param name Define the name of the uniform as defined in the shader
         * @param value Define the value to give to the uniform
         * @returns the texture itself allowing "fluent" like uniform updates
         */
        setMatrix(name: string, value: Matrix): ProceduralTexture;
        /**
         * Render the texture to its associated render target.
         * @param useCameraPostProcess Define if camera post process should be applied to the texture
         */
        render(useCameraPostProcess?: boolean): void;
        /**
         * Clone the texture.
         * @returns the cloned texture
         */
        clone(): ProceduralTexture;
        /**
         * Dispose the texture and release its associated resources.
         */
        dispose(): void;
    }


        interface AbstractScene {
            /**
             * The list of procedural textures added to the scene
             * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/proceduralTextures
             */
            proceduralTextures: Array<ProceduralTexture>;
        }
    /**
     * Defines the Procedural Texture scene component responsible to manage any Procedural Texture
     * in a given scene.
     */
    export class ProceduralTextureSceneComponent implements ISceneComponent {
        /**
         * The component name helpful to identify the component in the list of scene components.
         */
        readonly name = "ProceduralTexture";
        /**
         * The scene the component belongs to.
         */
        scene: Scene;
        /**
         * Creates a new instance of the component for the given scene
         * @param scene Defines the scene to register the component in
         */
        constructor(scene: Scene);
        /**
         * Registers the component in a given scene
         */
        register(): void;
        /**
         * Rebuilds the elements related to this component in case of
         * context lost for instance.
         */
        rebuild(): void;
        /**
         * Disposes the component and the associated resources.
         */
        dispose(): void;
        private _beforeClear;
    }


    /**
     * Raw cube texture where the raw buffers are passed in
     */
    export class RawCubeTexture extends CubeTexture {
        /**
         * Creates a cube texture where the raw buffers are passed in.
         * @param scene defines the scene the texture is attached to
         * @param data defines the array of data to use to create each face
         * @param size defines the size of the textures
         * @param format defines the format of the data
         * @param type defines the type of the data (like Engine.TEXTURETYPE_UNSIGNED_INT)
         * @param generateMipMaps  defines if the engine should generate the mip levels
         * @param invertY defines if data must be stored with Y axis inverted
         * @param samplingMode defines the required sampling mode (like Texture.NEAREST_SAMPLINGMODE)
         * @param compression defines the compression used (null by default)
         */
        constructor(scene: Scene, data: Nullable<ArrayBufferView[]>, size: number, format?: number, type?: number, generateMipMaps?: boolean, invertY?: boolean, samplingMode?: number, compression?: Nullable<string>);
        /**
         * Updates the raw cube texture.
         * @param data defines the data to store
         * @param format defines the data format
         * @param type defines the type fo the data (Engine.TEXTURETYPE_UNSIGNED_INT by default)
         * @param invertY defines if data must be stored with Y axis inverted
         * @param compression defines the compression used (null by default)
         */
        update(data: ArrayBufferView[], format: number, type: number, invertY: boolean, compression?: Nullable<string>): void;
        /**
         * Updates a raw cube texture with RGBD encoded data.
         * @param data defines the array of data [mipmap][face] to use to create each face
         * @param sphericalPolynomial defines the spherical polynomial for irradiance
         * @param lodScale defines the scale applied to environment texture. This manages the range of LOD level used for IBL according to the roughness
         * @param lodOffset defines the offset applied to environment texture. This manages first LOD level used for IBL according to the roughness
         * @returns a promise that resolves when the operation is complete
         */
        updateRGBDAsync(data: ArrayBufferView[][], sphericalPolynomial?: Nullable<SphericalPolynomial>, lodScale?: number, lodOffset?: number): Promise<void>;
        /**
         * Clones the raw cube texture.
         * @returns a new cube texture
         */
        clone(): CubeTexture;
    }


    /**
     * Raw texture can help creating a texture directly from an array of data.
     * This can be super useful if you either get the data from an uncompressed source or
     * if you wish to create your texture pixel by pixel.
     */
    export class RawTexture extends Texture {
        /**
         * Define the format of the data (RGB, RGBA... Engine.TEXTUREFORMAT_xxx)
         */
        format: number;
        /**
         * Instantiates a new RawTexture.
         * Raw texture can help creating a texture directly from an array of data.
         * This can be super useful if you either get the data from an uncompressed source or
         * if you wish to create your texture pixel by pixel.
         * @param data define the array of data to use to create the texture (null to create an empty texture)
         * @param width define the width of the texture
         * @param height define the height of the texture
         * @param format define the format of the data (RGB, RGBA... Engine.TEXTUREFORMAT_xxx)
         * @param sceneOrEngine defines the scene or engine the texture will belong to
         * @param generateMipMaps define whether mip maps should be generated or not
         * @param invertY define if the data should be flipped on Y when uploaded to the GPU
         * @param samplingMode define the texture sampling mode (Texture.xxx_SAMPLINGMODE)
         * @param type define the format of the data (int, float... Engine.TEXTURETYPE_xxx)
         * @param creationFlags specific flags to use when creating the texture (Constants.TEXTURE_CREATIONFLAG_STORAGE for storage textures, for eg)
         * @param useSRGBBuffer defines if the texture must be loaded in a sRGB GPU buffer (if supported by the GPU).
         */
        constructor(data: Nullable<ArrayBufferView>, width: number, height: number, 
        /**
         * Define the format of the data (RGB, RGBA... Engine.TEXTUREFORMAT_xxx)
         */
        format: number, sceneOrEngine: Nullable<Scene | ThinEngine>, generateMipMaps?: boolean, invertY?: boolean, samplingMode?: number, type?: number, creationFlags?: number, useSRGBBuffer?: boolean);
        /**
         * Updates the texture underlying data.
         * @param data Define the new data of the texture
         */
        update(data: ArrayBufferView): void;
        /**
         * Creates a luminance texture from some data.
         * @param data Define the texture data
         * @param width Define the width of the texture
         * @param height Define the height of the texture
         * @param sceneOrEngine defines the scene or engine the texture will belong to
         * @param generateMipMaps Define whether or not to create mip maps for the texture
         * @param invertY define if the data should be flipped on Y when uploaded to the GPU
         * @param samplingMode define the texture sampling mode (Texture.xxx_SAMPLINGMODE)
         * @returns the luminance texture
         */
        static CreateLuminanceTexture(data: Nullable<ArrayBufferView>, width: number, height: number, sceneOrEngine: Nullable<Scene | ThinEngine>, generateMipMaps?: boolean, invertY?: boolean, samplingMode?: number): RawTexture;
        /**
         * Creates a luminance alpha texture from some data.
         * @param data Define the texture data
         * @param width Define the width of the texture
         * @param height Define the height of the texture
         * @param sceneOrEngine defines the scene or engine the texture will belong to
         * @param generateMipMaps Define whether or not to create mip maps for the texture
         * @param invertY define if the data should be flipped on Y when uploaded to the GPU
         * @param samplingMode define the texture sampling mode (Texture.xxx_SAMPLINGMODE)
         * @returns the luminance alpha texture
         */
        static CreateLuminanceAlphaTexture(data: Nullable<ArrayBufferView>, width: number, height: number, sceneOrEngine: Nullable<Scene | ThinEngine>, generateMipMaps?: boolean, invertY?: boolean, samplingMode?: number): RawTexture;
        /**
         * Creates an alpha texture from some data.
         * @param data Define the texture data
         * @param width Define the width of the texture
         * @param height Define the height of the texture
         * @param sceneOrEngine defines the scene or engine the texture will belong to
         * @param generateMipMaps Define whether or not to create mip maps for the texture
         * @param invertY define if the data should be flipped on Y when uploaded to the GPU
         * @param samplingMode define the texture sampling mode (Texture.xxx_SAMPLINGMODE)
         * @returns the alpha texture
         */
        static CreateAlphaTexture(data: Nullable<ArrayBufferView>, width: number, height: number, sceneOrEngine: Nullable<Scene | ThinEngine>, generateMipMaps?: boolean, invertY?: boolean, samplingMode?: number): RawTexture;
        /**
         * Creates a RGB texture from some data.
         * @param data Define the texture data
         * @param width Define the width of the texture
         * @param height Define the height of the texture
         * @param sceneOrEngine defines the scene or engine the texture will belong to
         * @param generateMipMaps Define whether or not to create mip maps for the texture
         * @param invertY define if the data should be flipped on Y when uploaded to the GPU
         * @param samplingMode define the texture sampling mode (Texture.xxx_SAMPLINGMODE)
         * @param type define the format of the data (int, float... Engine.TEXTURETYPE_xxx)
         * @param creationFlags specific flags to use when creating the texture (Constants.TEXTURE_CREATIONFLAG_STORAGE for storage textures, for eg)
         * @param useSRGBBuffer defines if the texture must be loaded in a sRGB GPU buffer (if supported by the GPU).
         * @returns the RGB alpha texture
         */
        static CreateRGBTexture(data: Nullable<ArrayBufferView>, width: number, height: number, sceneOrEngine: Nullable<Scene | ThinEngine>, generateMipMaps?: boolean, invertY?: boolean, samplingMode?: number, type?: number, creationFlags?: number, useSRGBBuffer?: boolean): RawTexture;
        /**
         * Creates a RGBA texture from some data.
         * @param data Define the texture data
         * @param width Define the width of the texture
         * @param height Define the height of the texture
         * @param sceneOrEngine defines the scene or engine the texture will belong to
         * @param generateMipMaps Define whether or not to create mip maps for the texture
         * @param invertY define if the data should be flipped on Y when uploaded to the GPU
         * @param samplingMode define the texture sampling mode (Texture.xxx_SAMPLINGMODE)
         * @param type define the format of the data (int, float... Engine.TEXTURETYPE_xxx)
         * @param creationFlags specific flags to use when creating the texture (Constants.TEXTURE_CREATIONFLAG_STORAGE for storage textures, for eg)
         * @param useSRGBBuffer defines if the texture must be loaded in a sRGB GPU buffer (if supported by the GPU).
         * @returns the RGBA texture
         */
        static CreateRGBATexture(data: Nullable<ArrayBufferView>, width: number, height: number, sceneOrEngine: Nullable<Scene | ThinEngine>, generateMipMaps?: boolean, invertY?: boolean, samplingMode?: number, type?: number, creationFlags?: number, useSRGBBuffer?: boolean): RawTexture;
        /**
         * Creates a RGBA storage texture from some data.
         * @param data Define the texture data
         * @param width Define the width of the texture
         * @param height Define the height of the texture
         * @param sceneOrEngine defines the scene or engine the texture will belong to
         * @param generateMipMaps Define whether or not to create mip maps for the texture
         * @param invertY define if the data should be flipped on Y when uploaded to the GPU
         * @param samplingMode define the texture sampling mode (Texture.xxx_SAMPLINGMODE)
         * @param type define the format of the data (int, float... Engine.TEXTURETYPE_xxx)
         * @param useSRGBBuffer defines if the texture must be loaded in a sRGB GPU buffer (if supported by the GPU).
         * @returns the RGBA texture
         */
        static CreateRGBAStorageTexture(data: Nullable<ArrayBufferView>, width: number, height: number, sceneOrEngine: Nullable<Scene | ThinEngine>, generateMipMaps?: boolean, invertY?: boolean, samplingMode?: number, type?: number, useSRGBBuffer?: boolean): RawTexture;
        /**
         * Creates a R texture from some data.
         * @param data Define the texture data
         * @param width Define the width of the texture
         * @param height Define the height of the texture
         * @param sceneOrEngine defines the scene or engine the texture will belong to
         * @param generateMipMaps Define whether or not to create mip maps for the texture
         * @param invertY define if the data should be flipped on Y when uploaded to the GPU
         * @param samplingMode define the texture sampling mode (Texture.xxx_SAMPLINGMODE)
         * @param type define the format of the data (int, float... Engine.TEXTURETYPE_xxx)
         * @returns the R texture
         */
        static CreateRTexture(data: Nullable<ArrayBufferView>, width: number, height: number, sceneOrEngine: Nullable<Scene | ThinEngine>, generateMipMaps?: boolean, invertY?: boolean, samplingMode?: number, type?: number): RawTexture;
        /**
         * Creates a R storage texture from some data.
         * @param data Define the texture data
         * @param width Define the width of the texture
         * @param height Define the height of the texture
         * @param sceneOrEngine defines the scene or engine the texture will belong to
         * @param generateMipMaps Define whether or not to create mip maps for the texture
         * @param invertY define if the data should be flipped on Y when uploaded to the GPU
         * @param samplingMode define the texture sampling mode (Texture.xxx_SAMPLINGMODE)
         * @param type define the format of the data (int, float... Engine.TEXTURETYPE_xxx)
         * @returns the R texture
         */
        static CreateRStorageTexture(data: Nullable<ArrayBufferView>, width: number, height: number, sceneOrEngine: Nullable<Scene | ThinEngine>, generateMipMaps?: boolean, invertY?: boolean, samplingMode?: number, type?: number): RawTexture;
    }


    /**
     * Class used to store 2D array textures containing user data
     */
    export class RawTexture2DArray extends Texture {
        /** Gets or sets the texture format to use */
        format: number;
        private _depth;
        /**
         * Gets the number of layers of the texture
         */
        get depth(): number;
        /**
         * Create a new RawTexture2DArray
         * @param data defines the data of the texture
         * @param width defines the width of the texture
         * @param height defines the height of the texture
         * @param depth defines the number of layers of the texture
         * @param format defines the texture format to use
         * @param scene defines the hosting scene
         * @param generateMipMaps defines a boolean indicating if mip levels should be generated (true by default)
         * @param invertY defines if texture must be stored with Y axis inverted
         * @param samplingMode defines the sampling mode to use (Texture.TRILINEAR_SAMPLINGMODE by default)
         * @param textureType defines the texture Type (Engine.TEXTURETYPE_UNSIGNED_INT, Engine.TEXTURETYPE_FLOAT...)
         */
        constructor(data: ArrayBufferView, width: number, height: number, depth: number, 
        /** Gets or sets the texture format to use */
        format: number, scene: Scene, generateMipMaps?: boolean, invertY?: boolean, samplingMode?: number, textureType?: number);
        /**
         * Update the texture with new data
         * @param data defines the data to store in the texture
         */
        update(data: ArrayBufferView): void;
        /**
         * Creates a RGBA texture from some data.
         * @param data Define the texture data
         * @param width Define the width of the texture
         * @param height Define the height of the texture
         * @param depth defines the number of layers of the texture
         * @param scene defines the scene the texture will belong to
         * @param generateMipMaps Define whether or not to create mip maps for the texture
         * @param invertY define if the data should be flipped on Y when uploaded to the GPU
         * @param samplingMode define the texture sampling mode (Texture.xxx_SAMPLINGMODE)
         * @param type define the format of the data (int, float... Engine.TEXTURETYPE_xxx)
         * @returns the RGBA texture
         */
        static CreateRGBATexture(data: ArrayBufferView, width: number, height: number, depth: number, scene: Scene, generateMipMaps?: boolean, invertY?: boolean, samplingMode?: number, type?: number): RawTexture2DArray;
    }


    /**
     * Class used to store 3D textures containing user data
     */
    export class RawTexture3D extends Texture {
        /** Gets or sets the texture format to use */
        format: number;
        /**
         * Create a new RawTexture3D
         * @param data defines the data of the texture
         * @param width defines the width of the texture
         * @param height defines the height of the texture
         * @param depth defines the depth of the texture
         * @param format defines the texture format to use
         * @param scene defines the hosting scene
         * @param generateMipMaps defines a boolean indicating if mip levels should be generated (true by default)
         * @param invertY defines if texture must be stored with Y axis inverted
         * @param samplingMode defines the sampling mode to use (Texture.TRILINEAR_SAMPLINGMODE by default)
         * @param textureType defines the texture Type (Engine.TEXTURETYPE_UNSIGNED_INT, Engine.TEXTURETYPE_FLOAT...)
         */
        constructor(data: ArrayBufferView, width: number, height: number, depth: number, 
        /** Gets or sets the texture format to use */
        format: number, scene: Scene, generateMipMaps?: boolean, invertY?: boolean, samplingMode?: number, textureType?: number);
        /**
         * Update the texture with new data
         * @param data defines the data to store in the texture
         */
        update(data: ArrayBufferView): void;
    }


    /**
     * Creates a refraction texture used by refraction channel of the standard material.
     * It is like a mirror but to see through a material.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/reflectionTexture#refractiontexture
     */
    export class RefractionTexture extends RenderTargetTexture {
        /**
         * Define the reflection plane we want to use. The refractionPlane is usually set to the constructed refractor.
         * It is possible to directly set the refractionPlane by directly using a Plane(a, b, c, d) where a, b and c give the plane normal vector (a, b, c) and d is a scalar displacement from the refractionPlane to the origin. However in all but the very simplest of situations it is more straight forward to set it to the refractor as stated in the doc.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/reflectionTexture#refraction
         */
        refractionPlane: Plane;
        /**
         * Define how deep under the surface we should see.
         */
        depth: number;
        /**
         * Creates a refraction texture used by refraction channel of the standard material.
         * It is like a mirror but to see through a material.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/reflectionTexture#refraction
         * @param name Define the texture name
         * @param size Define the size of the underlying texture
         * @param scene Define the scene the refraction belongs to
         * @param generateMipMaps Define if we need to generate mips level for the refraction
         */
        constructor(name: string, size: number, scene?: Scene, generateMipMaps?: boolean);
        /**
         * Clone the refraction texture.
         * @returns the cloned texture
         */
        clone(): RefractionTexture;
        /**
         * Serialize the texture to a JSON representation you could use in Parse later on
         * @returns the serialized JSON representation
         */
        serialize(): any;
    }


    /**
     * Options for the RenderTargetTexture constructor
     */
    export interface RenderTargetTextureOptions {
        /** True (default: false) if mipmaps need to be generated after render */
        generateMipMaps?: boolean;
        /** True (default) to not change the aspect ratio of the scene in the RTT */
        doNotChangeAspectRatio?: boolean;
        /** The type of the buffer in the RTT (byte (default), half float, float...) */
        type?: number;
        /** True (default: false) if a cube texture needs to be created */
        isCube?: boolean;
        /** The sampling mode to be used with the render target (Trilinear (default), Linear, Nearest...) */
        samplingMode?: number;
        /** True (default) to generate a depth buffer */
        generateDepthBuffer?: boolean;
        /** True (default: false) to generate a stencil buffer */
        generateStencilBuffer?: boolean;
        /** True (default: false) if multiple textures need to be created (Draw Buffers) */
        isMulti?: boolean;
        /** The internal format of the buffer in the RTT (RED, RG, RGB, RGBA (default), ALPHA...) */
        format?: number;
        /** True (default: false) if the texture allocation should be delayed */
        delayAllocation?: boolean;
        /** Sample count to use when creating the RTT */
        samples?: number;
        /** specific flags to use when creating the texture (e.g., Constants.TEXTURE_CREATIONFLAG_STORAGE for storage textures) */
        creationFlags?: number;
        /** True (default: false) to indicate that no color target should be created. (e.g., if you only want to write to the depth buffer) */
        noColorAttachment?: boolean;
        /** Specifies the internal texture to use directly instead of creating one (ignores `noColorAttachment` flag when set) **/
        colorAttachment?: InternalTexture;
        /** True (default: false) to create a SRGB texture */
        useSRGBBuffer?: boolean;
    }
    /**
     * This Helps creating a texture that will be created from a camera in your scene.
     * It is basically a dynamic texture that could be used to create special effects for instance.
     * Actually, It is the base of lot of effects in the framework like post process, shadows, effect layers and rendering pipelines...
     */
    export class RenderTargetTexture extends Texture implements IRenderTargetTexture {
        /**
         * The texture will only be rendered once which can be useful to improve performance if everything in your render is static for instance.
         */
        static readonly REFRESHRATE_RENDER_ONCE: number;
        /**
         * The texture will only be rendered rendered every frame and is recommended for dynamic contents.
         */
        static readonly REFRESHRATE_RENDER_ONEVERYFRAME: number;
        /**
         * The texture will be rendered every 2 frames which could be enough if your dynamic objects are not
         * the central point of your effect and can save a lot of performances.
         */
        static readonly REFRESHRATE_RENDER_ONEVERYTWOFRAMES: number;
        /**
         * Use this predicate to dynamically define the list of mesh you want to render.
         * If set, the renderList property will be overwritten.
         */
        renderListPredicate: (AbstractMesh: AbstractMesh) => boolean;
        private _renderList;
        private _unObserveRenderList;
        /**
         * Use this list to define the list of mesh you want to render.
         */
        get renderList(): Nullable<Array<AbstractMesh>>;
        set renderList(value: Nullable<Array<AbstractMesh>>);
        private _renderListHasChanged;
        /**
         * Use this function to overload the renderList array at rendering time.
         * Return null to render with the current renderList, else return the list of meshes to use for rendering.
         * For 2DArray RTT, layerOrFace is the index of the layer that is going to be rendered, else it is the faceIndex of
         * the cube (if the RTT is a cube, else layerOrFace=0).
         * The renderList passed to the function is the current render list (the one that will be used if the function returns null).
         * The length of this list is passed through renderListLength: don't use renderList.length directly because the array can
         * hold dummy elements!
         */
        getCustomRenderList: (layerOrFace: number, renderList: Nullable<Immutable<Array<AbstractMesh>>>, renderListLength: number) => Nullable<Array<AbstractMesh>>;
        /**
         * Define if particles should be rendered in your texture.
         */
        renderParticles: boolean;
        /**
         * Define if sprites should be rendered in your texture.
         */
        renderSprites: boolean;
        /**
         * Force checking the layerMask property even if a custom list of meshes is provided (ie. if renderList is not undefined)
         */
        forceLayerMaskCheck: boolean;
        /**
         * Define the camera used to render the texture.
         */
        activeCamera: Nullable<Camera>;
        /**
         * Override the mesh isReady function with your own one.
         */
        customIsReadyFunction: (mesh: AbstractMesh, refreshRate: number, preWarm?: boolean) => boolean;
        /**
         * Override the render function of the texture with your own one.
         */
        customRenderFunction: (opaqueSubMeshes: SmartArray<SubMesh>, alphaTestSubMeshes: SmartArray<SubMesh>, transparentSubMeshes: SmartArray<SubMesh>, depthOnlySubMeshes: SmartArray<SubMesh>, beforeTransparents?: () => void) => void;
        /**
         * Define if camera post processes should be use while rendering the texture.
         */
        useCameraPostProcesses: boolean;
        /**
         * Define if the camera viewport should be respected while rendering the texture or if the render should be done to the entire texture.
         */
        ignoreCameraViewport: boolean;
        private _postProcessManager;
        /**
         * Post-processes for this render target
         */
        get postProcesses(): PostProcess[];
        private _postProcesses;
        private _resizeObserver;
        private get _prePassEnabled();
        /**
         * An event triggered when the texture is unbind.
         */
        onBeforeBindObservable: Observable<RenderTargetTexture>;
        /**
         * An event triggered when the texture is unbind.
         */
        onAfterUnbindObservable: Observable<RenderTargetTexture>;
        private _onAfterUnbindObserver;
        /**
         * Set a after unbind callback in the texture.
         * This has been kept for backward compatibility and use of onAfterUnbindObservable is recommended.
         */
        set onAfterUnbind(callback: () => void);
        /**
         * An event triggered before rendering the texture
         */
        onBeforeRenderObservable: Observable<number>;
        private _onBeforeRenderObserver;
        /**
         * Set a before render callback in the texture.
         * This has been kept for backward compatibility and use of onBeforeRenderObservable is recommended.
         */
        set onBeforeRender(callback: (faceIndex: number) => void);
        /**
         * An event triggered after rendering the texture
         */
        onAfterRenderObservable: Observable<number>;
        private _onAfterRenderObserver;
        /**
         * Set a after render callback in the texture.
         * This has been kept for backward compatibility and use of onAfterRenderObservable is recommended.
         */
        set onAfterRender(callback: (faceIndex: number) => void);
        /**
         * An event triggered after the texture clear
         */
        onClearObservable: Observable<Engine>;
        private _onClearObserver;
        /**
         * Set a clear callback in the texture.
         * This has been kept for backward compatibility and use of onClearObservable is recommended.
         */
        set onClear(callback: (Engine: Engine) => void);
        /**
         * An event triggered when the texture is resized.
         */
        onResizeObservable: Observable<RenderTargetTexture>;
        /**
         * Define the clear color of the Render Target if it should be different from the scene.
         */
        clearColor: Color4;
        protected _size: TextureSize;
        protected _initialSizeParameter: number | {
            width: number;
            height: number;
        } | {
            ratio: number;
        };
        protected _sizeRatio: Nullable<number>;
        /** @internal */
        _generateMipMaps: boolean;
        /** @internal */
        _cleared: boolean;
        /**
         * Skip the initial clear of the rtt at the beginning of the frame render loop
         */
        skipInitialClear: boolean;
        protected _renderingManager: RenderingManager;
        /** @internal */
        _waitingRenderList?: string[];
        protected _doNotChangeAspectRatio: boolean;
        protected _currentRefreshId: number;
        protected _refreshRate: number;
        protected _textureMatrix: Matrix;
        protected _samples: number;
        protected _renderTargetOptions: RenderTargetCreationOptions;
        private _canRescale;
        protected _renderTarget: Nullable<RenderTargetWrapper>;
        /**
         * Current render pass id of the render target texture. Note it can change over the rendering as there's a separate id for each face of a cube / each layer of an array layer!
         */
        renderPassId: number;
        private _renderPassIds;
        /**
         * Gets the render pass ids used by the render target texture. For a single render target the array length will be 1, for a cube texture it will be 6 and for
         * a 2D texture array it will return an array of ids the size of the 2D texture array
         */
        get renderPassIds(): readonly number[];
        /**
         * Gets the current value of the refreshId counter
         */
        get currentRefreshId(): number;
        /**
         * Sets a specific material to be used to render a mesh/a list of meshes in this render target texture
         * @param mesh mesh or array of meshes
         * @param material material or array of materials to use for this render pass. If undefined is passed, no specific material will be used but the regular material instead (mesh.material). It's possible to provide an array of materials to use a different material for each rendering in the case of a cube texture (6 rendering) and a 2D texture array (as many rendering as the length of the array)
         */
        setMaterialForRendering(mesh: AbstractMesh | AbstractMesh[], material?: Material | Material[]): void;
        private _isCubeData;
        /**
         * Gets render target creation options that were used.
         */
        get renderTargetOptions(): RenderTargetCreationOptions;
        /**
         * Gets the render target wrapper associated with this render target
         */
        get renderTarget(): Nullable<RenderTargetWrapper>;
        protected _onRatioRescale(): void;
        /**
         * Gets or sets the center of the bounding box associated with the texture (when in cube mode)
         * It must define where the camera used to render the texture is set
         */
        boundingBoxPosition: Vector3;
        private _boundingBoxSize;
        /**
         * Gets or sets the size of the bounding box associated with the texture (when in cube mode)
         * When defined, the cubemap will switch to local mode
         * @see https://community.arm.com/graphics/b/blog/posts/reflections-based-on-local-cubemaps-in-unity
         * @example https://www.babylonjs-playground.com/#RNASML
         */
        set boundingBoxSize(value: Vector3);
        get boundingBoxSize(): Vector3;
        /**
         * In case the RTT has been created with a depth texture, get the associated
         * depth texture.
         * Otherwise, return null.
         */
        get depthStencilTexture(): Nullable<InternalTexture>;
        /**
         * Instantiate a render target texture. This is mainly used to render of screen the scene to for instance apply post process
         * or used a shadow, depth texture...
         * @param name The friendly name of the texture
         * @param size The size of the RTT (number if square, or {width: number, height:number} or {ratio:} to define a ratio from the main scene)
         * @param scene The scene the RTT belongs to. Default is the last created scene.
         * @param options The options for creating the render target texture.
         */
        constructor(name: string, size: number | {
            width: number;
            height: number;
            layers?: number;
        } | {
            ratio: number;
        }, scene?: Nullable<Scene>, options?: RenderTargetTextureOptions);
        /**
         * Instantiate a render target texture. This is mainly used to render of screen the scene to for instance apply post process
         * or used a shadow, depth texture...
         * @param name The friendly name of the texture
         * @param size The size of the RTT (number if square, or {width: number, height:number} or {ratio:} to define a ratio from the main scene)
         * @param scene The scene the RTT belongs to. Default is the last created scene
         * @param generateMipMaps True (default: false) if mipmaps need to be generated after render
         * @param doNotChangeAspectRatio True (default) to not change the aspect ratio of the scene in the RTT
         * @param type The type of the buffer in the RTT (byte (default), half float, float...)
         * @param isCube True (default: false) if a cube texture needs to be created
         * @param samplingMode The sampling mode to be used with the render target (Trilinear (default), Linear, Nearest...)
         * @param generateDepthBuffer True (default) to generate a depth buffer
         * @param generateStencilBuffer True (default: false) to generate a stencil buffer
         * @param isMulti True (default: false) if multiple textures need to be created (Draw Buffers)
         * @param format The internal format of the buffer in the RTT (RED, RG, RGB, RGBA (default), ALPHA...)
         * @param delayAllocation True (default: false) if the texture allocation should be delayed
         * @param samples Sample count to use when creating the RTT
         * @param creationFlags specific flags to use when creating the texture (e.g., Constants.TEXTURE_CREATIONFLAG_STORAGE for storage textures)
         * @param noColorAttachment True (default: false) to indicate that no color target should be created. (e.g., if you only want to write to the depth buffer)
         * @param useSRGBBuffer True (default: false) to create a SRGB texture
         */
        constructor(name: string, size: number | {
            width: number;
            height: number;
            layers?: number;
        } | {
            ratio: number;
        }, scene?: Nullable<Scene>, generateMipMaps?: boolean, doNotChangeAspectRatio?: boolean, type?: number, isCube?: boolean, samplingMode?: number, generateDepthBuffer?: boolean, generateStencilBuffer?: boolean, isMulti?: boolean, format?: number, delayAllocation?: boolean, samples?: number, creationFlags?: number, noColorAttachment?: boolean, useSRGBBuffer?: boolean);
        /**
         * Creates a depth stencil texture.
         * This is only available in WebGL 2 or with the depth texture extension available.
         * @param comparisonFunction Specifies the comparison function to set on the texture. If 0 or undefined, the texture is not in comparison mode (default: 0)
         * @param bilinearFiltering Specifies whether or not bilinear filtering is enable on the texture (default: true)
         * @param generateStencil Specifies whether or not a stencil should be allocated in the texture (default: false)
         * @param samples sample count of the depth/stencil texture (default: 1)
         * @param format format of the depth texture (default: Constants.TEXTUREFORMAT_DEPTH32_FLOAT)
         */
        createDepthStencilTexture(comparisonFunction?: number, bilinearFiltering?: boolean, generateStencil?: boolean, samples?: number, format?: number): void;
        private _releaseRenderPassId;
        private _createRenderPassId;
        private _processSizeParameter;
        /**
         * Define the number of samples to use in case of MSAA.
         * It defaults to one meaning no MSAA has been enabled.
         */
        get samples(): number;
        set samples(value: number);
        /**
         * Resets the refresh counter of the texture and start bak from scratch.
         * Could be useful to regenerate the texture if it is setup to render only once.
         */
        resetRefreshCounter(): void;
        /**
         * Define the refresh rate of the texture or the rendering frequency.
         * Use 0 to render just once, 1 to render on every frame, 2 to render every two frames and so on...
         */
        get refreshRate(): number;
        set refreshRate(value: number);
        /**
         * Adds a post process to the render target rendering passes.
         * @param postProcess define the post process to add
         */
        addPostProcess(postProcess: PostProcess): void;
        /**
         * Clear all the post processes attached to the render target
         * @param dispose define if the cleared post processes should also be disposed (false by default)
         */
        clearPostProcesses(dispose?: boolean): void;
        /**
         * Remove one of the post process from the list of attached post processes to the texture
         * @param postProcess define the post process to remove from the list
         */
        removePostProcess(postProcess: PostProcess): void;
        /** @internal */
        _shouldRender(): boolean;
        /**
         * Gets the actual render size of the texture.
         * @returns the width of the render size
         */
        getRenderSize(): number;
        /**
         * Gets the actual render width of the texture.
         * @returns the width of the render size
         */
        getRenderWidth(): number;
        /**
         * Gets the actual render height of the texture.
         * @returns the height of the render size
         */
        getRenderHeight(): number;
        /**
         * Gets the actual number of layers of the texture.
         * @returns the number of layers
         */
        getRenderLayers(): number;
        /**
         * Don't allow this render target texture to rescale. Mainly used to prevent rescaling by the scene optimizer.
         */
        disableRescaling(): void;
        /**
         * Get if the texture can be rescaled or not.
         */
        get canRescale(): boolean;
        /**
         * Resize the texture using a ratio.
         * @param ratio the ratio to apply to the texture size in order to compute the new target size
         */
        scale(ratio: number): void;
        /**
         * Get the texture reflection matrix used to rotate/transform the reflection.
         * @returns the reflection matrix
         */
        getReflectionTextureMatrix(): Matrix;
        /**
         * Resize the texture to a new desired size.
         * Be careful as it will recreate all the data in the new texture.
         * @param size Define the new size. It can be:
         *   - a number for squared texture,
         *   - an object containing { width: number, height: number }
         *   - or an object containing a ratio { ratio: number }
         */
        resize(size: number | {
            width: number;
            height: number;
        } | {
            ratio: number;
        }): void;
        private _defaultRenderListPrepared;
        /**
         * Renders all the objects from the render list into the texture.
         * @param useCameraPostProcess Define if camera post processes should be used during the rendering
         * @param dumpForDebug Define if the rendering result should be dumped (copied) for debugging purpose
         */
        render(useCameraPostProcess?: boolean, dumpForDebug?: boolean): void;
        /**
         * This function will check if the render target texture can be rendered (textures are loaded, shaders are compiled)
         * @returns true if all required resources are ready
         */
        isReadyForRendering(): boolean;
        private _render;
        private _bestReflectionRenderTargetDimension;
        private _prepareRenderingManager;
        /**
         * @internal
         * @param faceIndex face index to bind to if this is a cubetexture
         * @param layer defines the index of the texture to bind in the array
         */
        _bindFrameBuffer(faceIndex?: number, layer?: number): void;
        protected _unbindFrameBuffer(engine: Engine, faceIndex: number): void;
        /**
         * @internal
         */
        _prepareFrame(scene: Scene, faceIndex?: number, layer?: number, useCameraPostProcess?: boolean): void;
        private _renderToTarget;
        /**
         * Overrides the default sort function applied in the rendering group to prepare the meshes.
         * This allowed control for front to back rendering or reversely depending of the special needs.
         *
         * @param renderingGroupId The rendering group id corresponding to its index
         * @param opaqueSortCompareFn The opaque queue comparison function use to sort.
         * @param alphaTestSortCompareFn The alpha test queue comparison function use to sort.
         * @param transparentSortCompareFn The transparent queue comparison function use to sort.
         */
        setRenderingOrder(renderingGroupId: number, opaqueSortCompareFn?: Nullable<(a: SubMesh, b: SubMesh) => number>, alphaTestSortCompareFn?: Nullable<(a: SubMesh, b: SubMesh) => number>, transparentSortCompareFn?: Nullable<(a: SubMesh, b: SubMesh) => number>): void;
        /**
         * Specifies whether or not the stencil and depth buffer are cleared between two rendering groups.
         *
         * @param renderingGroupId The rendering group id corresponding to its index
         * @param autoClearDepthStencil Automatically clears depth and stencil between groups if true.
         */
        setRenderingAutoClearDepthStencil(renderingGroupId: number, autoClearDepthStencil: boolean): void;
        /**
         * Clones the texture.
         * @returns the cloned texture
         */
        clone(): RenderTargetTexture;
        /**
         * Serialize the texture to a JSON representation we can easily use in the respective Parse function.
         * @returns The JSON representation of the texture
         */
        serialize(): any;
        /**
         *  This will remove the attached framebuffer objects. The texture will not be able to be used as render target anymore
         */
        disposeFramebufferObjects(): void;
        /**
         * Release and destroy the underlying lower level texture aka internalTexture.
         */
        releaseInternalTexture(): void;
        /**
         * Dispose the texture and release its associated resources.
         */
        dispose(): void;
        /** @internal */
        _rebuild(): void;
        /**
         * Clear the info related to rendering groups preventing retention point in material dispose.
         */
        freeRenderingGroups(): void;
        /**
         * Gets the number of views the corresponding to the texture (eg. a MultiviewRenderTarget will have > 1)
         * @returns the view count
         */
        getViewCount(): number;
    }


    /**
     * Defines the available options when creating a texture
     */
    export interface ITextureCreationOptions {
        /** Defines if the texture will require mip maps or not (default: false) */
        noMipmap?: boolean;
        /** Defines if the texture needs to be inverted on the y axis during loading (default: true) */
        invertY?: boolean;
        /** Defines the sampling mode we want for the texture while fetching from it (Texture.NEAREST_SAMPLINGMODE...) (default: Texture.TRILINEAR_SAMPLINGMODE) */
        samplingMode?: number;
        /** Defines a callback triggered when the texture has been loaded (default: null) */
        onLoad?: Nullable<() => void>;
        /** Defines a callback triggered when an error occurred during the loading session (default: null) */
        onError?: Nullable<(message?: string, exception?: any) => void>;
        /** Defines the buffer to load the texture from in case the texture is loaded from a buffer representation (default: null) */
        buffer?: Nullable<string | ArrayBuffer | ArrayBufferView | HTMLImageElement | Blob | ImageBitmap>;
        /** Defines if the buffer we are loading the texture from should be deleted after load (default: false) */
        deleteBuffer?: boolean;
        /** Defines the format of the texture we are trying to load (Engine.TEXTUREFORMAT_RGBA...) (default: ) */
        format?: number;
        /** Defines an optional mime type information (default: undefined) */
        mimeType?: string;
        /** Options to be passed to the loader (default: undefined) */
        loaderOptions?: any;
        /** Specific flags to use when creating the texture (Constants.TEXTURE_CREATIONFLAG_STORAGE for storage textures, for eg) (default: undefined) */
        creationFlags?: number;
        /** Defines if the texture must be loaded in a sRGB GPU buffer (if supported by the GPU) (default: false) */
        useSRGBBuffer?: boolean;
        /** Defines the underlying texture from an already existing one */
        internalTexture?: InternalTexture;
    }
    /**
     * This represents a texture in babylon. It can be easily loaded from a network, base64 or html input.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/materials_introduction#texture
     */
    export class Texture extends BaseTexture {
        /**
         * Gets or sets a general boolean used to indicate that textures containing direct data (buffers) must be saved as part of the serialization process
         */
        static SerializeBuffers: boolean;
        /**
         * Gets or sets a general boolean used to indicate that texture buffers must be saved as part of the serialization process.
         * If no buffer exists, one will be created as base64 string from the internal webgl data.
         */
        static ForceSerializeBuffers: boolean;
        /**
         * This observable will notify when any texture had a loading error
         */
        static OnTextureLoadErrorObservable: Observable<BaseTexture>;
        /** @internal */
        static _SerializeInternalTextureUniqueId: boolean;
        /**
         * @internal
         */
        static _CubeTextureParser: (jsonTexture: any, scene: Scene, rootUrl: string) => CubeTexture;
        /**
         * @internal
         */
        static _CreateMirror: (name: string, renderTargetSize: number, scene: Scene, generateMipMaps: boolean) => MirrorTexture;
        /**
         * @internal
         */
        static _CreateRenderTargetTexture: (name: string, renderTargetSize: number, scene: Scene, generateMipMaps: boolean, creationFlags?: number) => RenderTargetTexture;
        /** nearest is mag = nearest and min = nearest and no mip */
        static readonly NEAREST_SAMPLINGMODE = 1;
        /** nearest is mag = nearest and min = nearest and mip = linear */
        static readonly NEAREST_NEAREST_MIPLINEAR = 8;
        /** Bilinear is mag = linear and min = linear and no mip */
        static readonly BILINEAR_SAMPLINGMODE = 2;
        /** Bilinear is mag = linear and min = linear and mip = nearest */
        static readonly LINEAR_LINEAR_MIPNEAREST = 11;
        /** Trilinear is mag = linear and min = linear and mip = linear */
        static readonly TRILINEAR_SAMPLINGMODE = 3;
        /** Trilinear is mag = linear and min = linear and mip = linear */
        static readonly LINEAR_LINEAR_MIPLINEAR = 3;
        /** mag = nearest and min = nearest and mip = nearest */
        static readonly NEAREST_NEAREST_MIPNEAREST = 4;
        /** mag = nearest and min = linear and mip = nearest */
        static readonly NEAREST_LINEAR_MIPNEAREST = 5;
        /** mag = nearest and min = linear and mip = linear */
        static readonly NEAREST_LINEAR_MIPLINEAR = 6;
        /** mag = nearest and min = linear and mip = none */
        static readonly NEAREST_LINEAR = 7;
        /** mag = nearest and min = nearest and mip = none */
        static readonly NEAREST_NEAREST = 1;
        /** mag = linear and min = nearest and mip = nearest */
        static readonly LINEAR_NEAREST_MIPNEAREST = 9;
        /** mag = linear and min = nearest and mip = linear */
        static readonly LINEAR_NEAREST_MIPLINEAR = 10;
        /** mag = linear and min = linear and mip = none */
        static readonly LINEAR_LINEAR = 2;
        /** mag = linear and min = nearest and mip = none */
        static readonly LINEAR_NEAREST = 12;
        /** Explicit coordinates mode */
        static readonly EXPLICIT_MODE = 0;
        /** Spherical coordinates mode */
        static readonly SPHERICAL_MODE = 1;
        /** Planar coordinates mode */
        static readonly PLANAR_MODE = 2;
        /** Cubic coordinates mode */
        static readonly CUBIC_MODE = 3;
        /** Projection coordinates mode */
        static readonly PROJECTION_MODE = 4;
        /** Inverse Cubic coordinates mode */
        static readonly SKYBOX_MODE = 5;
        /** Inverse Cubic coordinates mode */
        static readonly INVCUBIC_MODE = 6;
        /** Equirectangular coordinates mode */
        static readonly EQUIRECTANGULAR_MODE = 7;
        /** Equirectangular Fixed coordinates mode */
        static readonly FIXED_EQUIRECTANGULAR_MODE = 8;
        /** Equirectangular Fixed Mirrored coordinates mode */
        static readonly FIXED_EQUIRECTANGULAR_MIRRORED_MODE = 9;
        /** Texture is not repeating outside of 0..1 UVs */
        static readonly CLAMP_ADDRESSMODE = 0;
        /** Texture is repeating outside of 0..1 UVs */
        static readonly WRAP_ADDRESSMODE = 1;
        /** Texture is repeating and mirrored */
        static readonly MIRROR_ADDRESSMODE = 2;
        /**
         * Gets or sets a boolean which defines if the texture url must be build from the serialized URL instead of just using the name and loading them side by side with the scene file
         */
        static UseSerializedUrlIfAny: boolean;
        /**
         * Define the url of the texture.
         */
        url: Nullable<string>;
        /**
         * Define an offset on the texture to offset the u coordinates of the UVs
         * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/moreMaterials#offsetting
         */
        uOffset: number;
        /**
         * Define an offset on the texture to offset the v coordinates of the UVs
         * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/moreMaterials#offsetting
         */
        vOffset: number;
        /**
         * Define an offset on the texture to scale the u coordinates of the UVs
         * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/moreMaterials#tiling
         */
        uScale: number;
        /**
         * Define an offset on the texture to scale the v coordinates of the UVs
         * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/moreMaterials#tiling
         */
        vScale: number;
        /**
         * Define an offset on the texture to rotate around the u coordinates of the UVs
         * The angle is defined in radians.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/moreMaterials
         */
        uAng: number;
        /**
         * Define an offset on the texture to rotate around the v coordinates of the UVs
         * The angle is defined in radians.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/moreMaterials
         */
        vAng: number;
        /**
         * Define an offset on the texture to rotate around the w coordinates of the UVs (in case of 3d texture)
         * The angle is defined in radians.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/moreMaterials
         */
        wAng: number;
        /**
         * Defines the center of rotation (U)
         */
        uRotationCenter: number;
        /**
         * Defines the center of rotation (V)
         */
        vRotationCenter: number;
        /**
         * Defines the center of rotation (W)
         */
        wRotationCenter: number;
        /**
         * Sets this property to true to avoid deformations when rotating the texture with non-uniform scaling
         */
        homogeneousRotationInUVTransform: boolean;
        /**
         * Are mip maps generated for this texture or not.
         */
        get noMipmap(): boolean;
        /**
         * List of inspectable custom properties (used by the Inspector)
         * @see https://doc.babylonjs.com/toolsAndResources/inspector#extensibility
         */
        inspectableCustomProperties: Nullable<IInspectable[]>;
        private _noMipmap;
        /** @internal */
        _invertY: boolean;
        private _rowGenerationMatrix;
        private _cachedTextureMatrix;
        private _projectionModeMatrix;
        private _t0;
        private _t1;
        private _t2;
        private _cachedUOffset;
        private _cachedVOffset;
        private _cachedUScale;
        private _cachedVScale;
        private _cachedUAng;
        private _cachedVAng;
        private _cachedWAng;
        private _cachedReflectionProjectionMatrixId;
        private _cachedURotationCenter;
        private _cachedVRotationCenter;
        private _cachedWRotationCenter;
        private _cachedHomogeneousRotationInUVTransform;
        private _cachedReflectionTextureMatrix;
        private _cachedReflectionUOffset;
        private _cachedReflectionVOffset;
        private _cachedReflectionUScale;
        private _cachedReflectionVScale;
        private _cachedReflectionCoordinatesMode;
        /** @internal */
        _buffer: Nullable<string | ArrayBuffer | ArrayBufferView | HTMLImageElement | Blob | ImageBitmap>;
        private _deleteBuffer;
        protected _format: Nullable<number>;
        private _delayedOnLoad;
        private _delayedOnError;
        private _mimeType?;
        private _loaderOptions?;
        private _creationFlags?;
        private _useSRGBBuffer?;
        private _forcedExtension?;
        /** Returns the texture mime type if it was defined by a loader (undefined else) */
        get mimeType(): string | undefined;
        /**
         * Observable triggered once the texture has been loaded.
         */
        onLoadObservable: Observable<Texture>;
        protected _isBlocking: boolean;
        /**
         * Is the texture preventing material to render while loading.
         * If false, a default texture will be used instead of the loading one during the preparation step.
         */
        set isBlocking(value: boolean);
        get isBlocking(): boolean;
        /**
         * Gets a boolean indicating if the texture needs to be inverted on the y axis during loading
         */
        get invertY(): boolean;
        /**
         * Instantiates a new texture.
         * This represents a texture in babylon. It can be easily loaded from a network, base64 or html input.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/materials_introduction#texture
         * @param url defines the url of the picture to load as a texture
         * @param sceneOrEngine defines the scene or engine the texture will belong to
         * @param noMipmapOrOptions defines if the texture will require mip maps or not or set of all options to create the texture
         * @param invertY defines if the texture needs to be inverted on the y axis during loading
         * @param samplingMode defines the sampling mode we want for the texture while fetching from it (Texture.NEAREST_SAMPLINGMODE...)
         * @param onLoad defines a callback triggered when the texture has been loaded
         * @param onError defines a callback triggered when an error occurred during the loading session
         * @param buffer defines the buffer to load the texture from in case the texture is loaded from a buffer representation
         * @param deleteBuffer defines if the buffer we are loading the texture from should be deleted after load
         * @param format defines the format of the texture we are trying to load (Engine.TEXTUREFORMAT_RGBA...)
         * @param mimeType defines an optional mime type information
         * @param loaderOptions options to be passed to the loader
         * @param creationFlags specific flags to use when creating the texture (Constants.TEXTURE_CREATIONFLAG_STORAGE for storage textures, for eg)
         * @param forcedExtension defines the extension to use to pick the right loader
         */
        constructor(url: Nullable<string>, sceneOrEngine?: Nullable<Scene | ThinEngine>, noMipmapOrOptions?: boolean | ITextureCreationOptions, invertY?: boolean, samplingMode?: number, onLoad?: Nullable<() => void>, onError?: Nullable<(message?: string, exception?: any) => void>, buffer?: Nullable<string | ArrayBuffer | ArrayBufferView | HTMLImageElement | Blob | ImageBitmap>, deleteBuffer?: boolean, format?: number, mimeType?: string, loaderOptions?: any, creationFlags?: number, forcedExtension?: string);
        /**
         * Update the url (and optional buffer) of this texture if url was null during construction.
         * @param url the url of the texture
         * @param buffer the buffer of the texture (defaults to null)
         * @param onLoad callback called when the texture is loaded  (defaults to null)
         * @param forcedExtension defines the extension to use to pick the right loader
         */
        updateURL(url: string, buffer?: Nullable<string | ArrayBuffer | ArrayBufferView | HTMLImageElement | Blob | ImageBitmap>, onLoad?: () => void, forcedExtension?: string): void;
        /**
         * Finish the loading sequence of a texture flagged as delayed load.
         * @internal
         */
        delayLoad(): void;
        private _prepareRowForTextureGeneration;
        /**
         * Checks if the texture has the same transform matrix than another texture
         * @param texture texture to check against
         * @returns true if the transforms are the same, else false
         */
        checkTransformsAreIdentical(texture: Nullable<Texture>): boolean;
        /**
         * Get the current texture matrix which includes the requested offsetting, tiling and rotation components.
         * @param uBase
         * @returns the transform matrix of the texture.
         */
        getTextureMatrix(uBase?: number): Matrix;
        /**
         * Get the current matrix used to apply reflection. This is useful to rotate an environment texture for instance.
         * @returns The reflection texture transform
         */
        getReflectionTextureMatrix(): Matrix;
        /**
         * Clones the texture.
         * @returns the cloned texture
         */
        clone(): Texture;
        /**
         * Serialize the texture to a JSON representation we can easily use in the respective Parse function.
         * @returns The JSON representation of the texture
         */
        serialize(): any;
        /**
         * Get the current class name of the texture useful for serialization or dynamic coding.
         * @returns "Texture"
         */
        getClassName(): string;
        /**
         * Dispose the texture and release its associated resources.
         */
        dispose(): void;
        /**
         * Parse the JSON representation of a texture in order to recreate the texture in the given scene.
         * @param parsedTexture Define the JSON representation of the texture
         * @param scene Define the scene the parsed texture should be instantiated in
         * @param rootUrl Define the root url of the parsing sequence in the case of relative dependencies
         * @returns The parsed texture if successful
         */
        static Parse(parsedTexture: any, scene: Scene, rootUrl: string): Nullable<BaseTexture>;
        /**
         * Creates a texture from its base 64 representation.
         * @param data Define the base64 payload without the data: prefix
         * @param name Define the name of the texture in the scene useful fo caching purpose for instance
         * @param scene Define the scene the texture should belong to
         * @param noMipmapOrOptions defines if the texture will require mip maps or not or set of all options to create the texture
         * @param invertY define if the texture needs to be inverted on the y axis during loading
         * @param samplingMode define the sampling mode we want for the texture while fetching from it (Texture.NEAREST_SAMPLINGMODE...)
         * @param onLoad define a callback triggered when the texture has been loaded
         * @param onError define a callback triggered when an error occurred during the loading session
         * @param format define the format of the texture we are trying to load (Engine.TEXTUREFORMAT_RGBA...)
         * @param creationFlags specific flags to use when creating the texture (Constants.TEXTURE_CREATIONFLAG_STORAGE for storage textures, for eg)
         * @returns the created texture
         */
        static CreateFromBase64String(data: string, name: string, scene: Scene, noMipmapOrOptions?: boolean | ITextureCreationOptions, invertY?: boolean, samplingMode?: number, onLoad?: Nullable<() => void>, onError?: Nullable<() => void>, format?: number, creationFlags?: number): Texture;
        /**
         * Creates a texture from its data: representation. (data: will be added in case only the payload has been passed in)
         * @param name Define the name of the texture in the scene useful fo caching purpose for instance
         * @param buffer define the buffer to load the texture from in case the texture is loaded from a buffer representation
         * @param scene Define the scene the texture should belong to
         * @param deleteBuffer define if the buffer we are loading the texture from should be deleted after load
         * @param noMipmapOrOptions defines if the texture will require mip maps or not or set of all options to create the texture
         * @param invertY define if the texture needs to be inverted on the y axis during loading
         * @param samplingMode define the sampling mode we want for the texture while fetching from it (Texture.NEAREST_SAMPLINGMODE...)
         * @param onLoad define a callback triggered when the texture has been loaded
         * @param onError define a callback triggered when an error occurred during the loading session
         * @param format define the format of the texture we are trying to load (Engine.TEXTUREFORMAT_RGBA...)
         * @param creationFlags specific flags to use when creating the texture (Constants.TEXTURE_CREATIONFLAG_STORAGE for storage textures, for eg)
         * @returns the created texture
         */
        static LoadFromDataString(name: string, buffer: any, scene: Scene, deleteBuffer?: boolean, noMipmapOrOptions?: boolean | ITextureCreationOptions, invertY?: boolean, samplingMode?: number, onLoad?: Nullable<() => void>, onError?: Nullable<(message?: string, exception?: any) => void>, format?: number, creationFlags?: number): Texture;
    }


    /**
     * Define options used to create an internal texture
     */
    export interface InternalTextureCreationOptions {
        /**
         * Specifies if mipmaps must be created. If undefined, the value from generateMipMaps is taken instead
         */
        createMipMaps?: boolean;
        /**
         * Specifies if mipmaps must be generated
         */
        generateMipMaps?: boolean;
        /** Defines texture type (int by default) */
        type?: number;
        /** Defines sampling mode (trilinear by default) */
        samplingMode?: number;
        /** Defines format (RGBA by default) */
        format?: number;
        /** Defines sample count (1 by default) */
        samples?: number;
        /** Texture creation flags */
        creationFlags?: number;
        /** Creates the RTT in sRGB space */
        useSRGBBuffer?: boolean;
    }
    /**
     * Define options used to create a render target texture
     */
    export interface RenderTargetCreationOptions extends InternalTextureCreationOptions {
        /** Specifies whether or not a depth should be allocated in the texture (true by default) */
        generateDepthBuffer?: boolean;
        /** Specifies whether or not a stencil should be allocated in the texture (false by default)*/
        generateStencilBuffer?: boolean;
        /** Specifies that no color target should be bound to the render target (useful if you only want to write to the depth buffer, for eg) */
        noColorAttachment?: boolean;
        /** Specifies the internal texture to use directly instead of creating one (ignores `noColorAttachment` flag when set) **/
        colorAttachment?: InternalTexture;
    }
    /**
     * Define options used to create a depth texture
     */
    export interface DepthTextureCreationOptions {
        /** Specifies whether or not a stencil should be allocated in the texture */
        generateStencil?: boolean;
        /** Specifies whether or not bilinear filtering is enable on the texture */
        bilinearFiltering?: boolean;
        /** Specifies the comparison function to set on the texture. If 0 or undefined, the texture is not in comparison mode */
        comparisonFunction?: number;
        /** Specifies if the created texture is a cube texture */
        isCube?: boolean;
        /** Specifies the sample count of the depth/stencil texture texture */
        samples?: number;
        /** Specifies the depth texture format to use */
        depthTextureFormat?: number;
    }
    /**
     * Type used to define a texture size (either with a number or with a rect width and height)
     */
    export type TextureSize = number | {
        width: number;
        height: number;
        layers?: number;
    };


    /**
     * Class used to store a texture sampler data
     */
    export class TextureSampler {
        /**
         * Gets the sampling mode of the texture
         */
        samplingMode: number;
        /**
         * | Value | Type               | Description |
         * | ----- | ------------------ | ----------- |
         * | 0     | CLAMP_ADDRESSMODE  |             |
         * | 1     | WRAP_ADDRESSMODE   |             |
         * | 2     | MIRROR_ADDRESSMODE |             |
         */
        get wrapU(): Nullable<number>;
        set wrapU(value: Nullable<number>);
        /**
         * | Value | Type               | Description |
         * | ----- | ------------------ | ----------- |
         * | 0     | CLAMP_ADDRESSMODE  |             |
         * | 1     | WRAP_ADDRESSMODE   |             |
         * | 2     | MIRROR_ADDRESSMODE |             |
         */
        get wrapV(): Nullable<number>;
        set wrapV(value: Nullable<number>);
        /**
         * | Value | Type               | Description |
         * | ----- | ------------------ | ----------- |
         * | 0     | CLAMP_ADDRESSMODE  |             |
         * | 1     | WRAP_ADDRESSMODE   |             |
         * | 2     | MIRROR_ADDRESSMODE |             |
         */
        get wrapR(): Nullable<number>;
        set wrapR(value: Nullable<number>);
        /**
         * With compliant hardware and browser (supporting anisotropic filtering)
         * this defines the level of anisotropic filtering in the texture.
         * The higher the better but the slower.
         */
        get anisotropicFilteringLevel(): Nullable<number>;
        set anisotropicFilteringLevel(value: Nullable<number>);
        /**
         * Gets or sets the comparison function (Constants.LESS, Constants.EQUAL, etc). Set 0 to not use a comparison function
         */
        get comparisonFunction(): number;
        set comparisonFunction(value: number);
        private _useMipMaps;
        /**
         * Indicates to use the mip maps (if available on the texture).
         * Thanks to this flag, you can instruct the sampler to not sample the mipmaps even if they exist (and if the sampling mode is set to a value that normally samples the mipmaps!)
         */
        get useMipMaps(): boolean;
        set useMipMaps(value: boolean);
        /** @internal */
        _cachedWrapU: Nullable<number>;
        /** @internal */
        _cachedWrapV: Nullable<number>;
        /** @internal */
        _cachedWrapR: Nullable<number>;
        /** @internal */
        _cachedAnisotropicFilteringLevel: Nullable<number>;
        /** @internal */
        _comparisonFunction: number;
        /**
         * Creates a Sampler instance
         */
        constructor();
        /**
         * Sets all the parameters of the sampler
         * @param wrapU u address mode (default: TEXTURE_WRAP_ADDRESSMODE)
         * @param wrapV v address mode (default: TEXTURE_WRAP_ADDRESSMODE)
         * @param wrapR r address mode (default: TEXTURE_WRAP_ADDRESSMODE)
         * @param anisotropicFilteringLevel anisotropic level (default: 1)
         * @param samplingMode sampling mode (default: Constants.TEXTURE_BILINEAR_SAMPLINGMODE)
         * @param comparisonFunction comparison function (default: 0 - no comparison function)
         * @returns the current sampler instance
         */
        setParameters(wrapU?: number, wrapV?: number, wrapR?: number, anisotropicFilteringLevel?: number, samplingMode?: number, comparisonFunction?: number): TextureSampler;
        /**
         * Compares this sampler with another one
         * @param other sampler to compare with
         * @returns true if the samplers have the same parametres, else false
         */
        compareSampler(other: TextureSampler): boolean;
    }


    /**
     * This is a tiny helper class to wrap a RenderTargetWrapper in a texture
     * usable as the input of an effect.
     */
    export class ThinRenderTargetTexture extends ThinTexture implements IRenderTargetTexture {
        private readonly _renderTargetOptions;
        private _renderTarget;
        private _size;
        /**
         * Gets the render target wrapper associated with this render target
         */
        get renderTarget(): Nullable<RenderTargetWrapper>;
        /**
         * Instantiates a new ThinRenderTargetTexture.
         * Tiny helper class to wrap a RenderTargetWrapper in a texture.
         * This can be used as an internal texture wrapper in ThinEngine to benefit from the cache and to hold on the associated RTT
         * @param engine Define the internalTexture to wrap
         * @param size Define the size of the RTT to create
         * @param options Define rendertarget options
         */
        constructor(engine: ThinEngine, size: TextureSize, options: RenderTargetCreationOptions);
        /**
         * Resize the texture to a new desired size.
         * Be careful as it will recreate all the data in the new texture.
         * @param size Define the new size. It can be:
         *   - a number for squared texture,
         *   - an object containing { width: number, height: number }
         */
        resize(size: TextureSize): void;
        /**
         * Get the underlying lower level texture from Babylon.
         * @returns the internal texture
         */
        getInternalTexture(): Nullable<InternalTexture>;
        /**
         * Get the class name of the texture.
         * @returns "ThinRenderTargetTexture"
         */
        getClassName(): string;
        /**
         * Dispose the texture and release its associated resources.
         * @param disposeOnlyFramebuffers
         */
        dispose(disposeOnlyFramebuffers?: boolean): void;
    }


    /**
     * Base class of all the textures in babylon.
     * It groups all the common properties required to work with Thin Engine.
     */
    export class ThinTexture {
        protected _wrapU: number;
        /**
         * | Value | Type               | Description |
         * | ----- | ------------------ | ----------- |
         * | 0     | CLAMP_ADDRESSMODE  |             |
         * | 1     | WRAP_ADDRESSMODE   |             |
         * | 2     | MIRROR_ADDRESSMODE |             |
         */
        get wrapU(): number;
        set wrapU(value: number);
        protected _wrapV: number;
        /**
         * | Value | Type               | Description |
         * | ----- | ------------------ | ----------- |
         * | 0     | CLAMP_ADDRESSMODE  |             |
         * | 1     | WRAP_ADDRESSMODE   |             |
         * | 2     | MIRROR_ADDRESSMODE |             |
         */
        get wrapV(): number;
        set wrapV(value: number);
        /**
         * | Value | Type               | Description |
         * | ----- | ------------------ | ----------- |
         * | 0     | CLAMP_ADDRESSMODE  |             |
         * | 1     | WRAP_ADDRESSMODE   |             |
         * | 2     | MIRROR_ADDRESSMODE |             |
         */
        wrapR: number;
        /**
         * With compliant hardware and browser (supporting anisotropic filtering)
         * this defines the level of anisotropic filtering in the texture.
         * The higher the better but the slower. This defaults to 4 as it seems to be the best tradeoff.
         */
        anisotropicFilteringLevel: number;
        /**
         * Define the current state of the loading sequence when in delayed load mode.
         */
        delayLoadState: number;
        /**
         * How a texture is mapped.
         * Unused in thin texture mode.
         */
        get coordinatesMode(): number;
        /**
         * Define if the texture is a cube texture or if false a 2d texture.
         */
        get isCube(): boolean;
        protected set isCube(value: boolean);
        /**
         * Define if the texture is a 3d texture (webgl 2) or if false a 2d texture.
         */
        get is3D(): boolean;
        protected set is3D(value: boolean);
        /**
         * Define if the texture is a 2d array texture (webgl 2) or if false a 2d texture.
         */
        get is2DArray(): boolean;
        protected set is2DArray(value: boolean);
        /**
         * Get the class name of the texture.
         * @returns "ThinTexture"
         */
        getClassName(): string;
        /** @internal */
        _texture: Nullable<InternalTexture>;
        protected _engine: Nullable<ThinEngine>;
        private _cachedSize;
        private _cachedBaseSize;
        /**
         * Instantiates a new ThinTexture.
         * Base class of all the textures in babylon.
         * This can be used as an internal texture wrapper in ThinEngine to benefit from the cache
         * @param internalTexture Define the internalTexture to wrap
         */
        constructor(internalTexture: Nullable<InternalTexture>);
        /**
         * Get if the texture is ready to be used (downloaded, converted, mip mapped...).
         * @returns true if fully ready
         */
        isReady(): boolean;
        /**
         * Triggers the load sequence in delayed load mode.
         */
        delayLoad(): void;
        /**
         * Get the underlying lower level texture from Babylon.
         * @returns the internal texture
         */
        getInternalTexture(): Nullable<InternalTexture>;
        /**
         * Get the size of the texture.
         * @returns the texture size.
         */
        getSize(): ISize;
        /**
         * Get the base size of the texture.
         * It can be different from the size if the texture has been resized for POT for instance
         * @returns the base size
         */
        getBaseSize(): ISize;
        /** @internal */
        protected _initialSamplingMode: number;
        /**
         * Get the current sampling mode associated with the texture.
         */
        get samplingMode(): number;
        /**
         * Update the sampling mode of the texture.
         * Default is Trilinear mode.
         *
         * | Value | Type               | Description |
         * | ----- | ------------------ | ----------- |
         * | 1     | NEAREST_SAMPLINGMODE or NEAREST_NEAREST_MIPLINEAR  | Nearest is: mag = nearest, min = nearest, mip = linear |
         * | 2     | BILINEAR_SAMPLINGMODE or LINEAR_LINEAR_MIPNEAREST | Bilinear is: mag = linear, min = linear, mip = nearest |
         * | 3     | TRILINEAR_SAMPLINGMODE or LINEAR_LINEAR_MIPLINEAR | Trilinear is: mag = linear, min = linear, mip = linear |
         * | 4     | NEAREST_NEAREST_MIPNEAREST |             |
         * | 5    | NEAREST_LINEAR_MIPNEAREST |             |
         * | 6    | NEAREST_LINEAR_MIPLINEAR |             |
         * | 7    | NEAREST_LINEAR |             |
         * | 8    | NEAREST_NEAREST |             |
         * | 9   | LINEAR_NEAREST_MIPNEAREST |             |
         * | 10   | LINEAR_NEAREST_MIPLINEAR |             |
         * | 11   | LINEAR_LINEAR |             |
         * | 12   | LINEAR_NEAREST |             |
         *
         *    > _mag_: magnification filter (close to the viewer)
         *    > _min_: minification filter (far from the viewer)
         *    > _mip_: filter used between mip map levels
         *@param samplingMode Define the new sampling mode of the texture
         */
        updateSamplingMode(samplingMode: number): void;
        /**
         * Release and destroy the underlying lower level texture aka internalTexture.
         */
        releaseInternalTexture(): void;
        /**
         * Dispose the texture and release its associated resources.
         */
        dispose(): void;
    }


    /**
     * Settings for finer control over video usage
     */
    export interface VideoTextureSettings {
        /**
         * Applies `autoplay` to video, if specified
         */
        autoPlay?: boolean;
        /**
         * Applies `muted` to video, if specified
         */
        muted?: boolean;
        /**
         * Applies `loop` to video, if specified
         */
        loop?: boolean;
        /**
         * Automatically updates internal texture from video at every frame in the render loop
         */
        autoUpdateTexture: boolean;
        /**
         * Image src displayed during the video loading or until the user interacts with the video.
         */
        poster?: string;
        /**
         * Defines the associated texture format.
         */
        format?: number;
        /**
         * Notify babylon to not modify any video settings and not control the video's playback.
         * Set this to true if you are controlling the way the video is being played, stopped and paused.
         */
        independentVideoSource?: boolean;
    }
    /**
     * If you want to display a video in your scene, this is the special texture for that.
     * This special texture works similar to other textures, with the exception of a few parameters.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/videoTexture
     */
    export class VideoTexture extends Texture {
        /**
         * Tells whether textures will be updated automatically or user is required to call `updateTexture` manually
         */
        readonly autoUpdateTexture: boolean;
        /**
         * The video instance used by the texture internally
         */
        readonly video: HTMLVideoElement;
        private _externalTexture;
        private _onUserActionRequestedObservable;
        /**
         * Event triggered when a dom action is required by the user to play the video.
         * This happens due to recent changes in browser policies preventing video to auto start.
         */
        get onUserActionRequestedObservable(): Observable<Texture>;
        private _generateMipMaps;
        private _stillImageCaptured;
        private _displayingPosterTexture;
        private _settings;
        private _createInternalTextureOnEvent;
        private _frameId;
        private _currentSrc;
        private _onError?;
        private _errorFound;
        private _processError;
        private _handlePlay;
        /**
         * Creates a video texture.
         * If you want to display a video in your scene, this is the special texture for that.
         * This special texture works similar to other textures, with the exception of a few parameters.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/videoTexture
         * @param name optional name, will detect from video source, if not defined
         * @param src can be used to provide an url, array of urls or an already setup HTML video element.
         * @param scene is obviously the current scene.
         * @param generateMipMaps can be used to turn on mipmaps (Can be expensive for videoTextures because they are often updated).
         * @param invertY is false by default but can be used to invert video on Y axis
         * @param samplingMode controls the sampling method and is set to TRILINEAR_SAMPLINGMODE by default
         * @param settings allows finer control over video usage
         * @param onError defines a callback triggered when an error occurred during the loading session
         * @param format defines the texture format to use (Engine.TEXTUREFORMAT_RGBA by default)
         */
        constructor(name: Nullable<string>, src: string | string[] | HTMLVideoElement, scene: Nullable<Scene>, generateMipMaps?: boolean, invertY?: boolean, samplingMode?: number, settings?: Partial<VideoTextureSettings>, onError?: Nullable<(message?: string, exception?: any) => void>, format?: number);
        /**
         * Get the current class name of the video texture useful for serialization or dynamic coding.
         * @returns "VideoTexture"
         */
        getClassName(): string;
        private _getName;
        private _getVideo;
        private _resizeInternalTexture;
        private _createInternalTexture;
        private _reset;
        /**
         * @internal Internal method to initiate `update`.
         */
        _rebuild(): void;
        /**
         * Update Texture in the `auto` mode. Does not do anything if `settings.autoUpdateTexture` is false.
         */
        update(): void;
        /**
         * Update Texture in `manual` mode. Does not do anything if not visible or paused.
         * @param isVisible Visibility state, detected by user using `scene.getActiveMeshes()` or otherwise.
         */
        updateTexture(isVisible: boolean): void;
        protected _updateInternalTexture: () => void;
        /**
         * Change video content. Changing video instance or setting multiple urls (as in constructor) is not supported.
         * @param url New url.
         */
        updateURL(url: string): void;
        /**
         * Clones the texture.
         * @returns the cloned texture
         */
        clone(): VideoTexture;
        /**
         * Dispose the texture and release its associated resources.
         */
        dispose(): void;
        /**
         * Creates a video texture straight from a stream.
         * @param scene Define the scene the texture should be created in
         * @param stream Define the stream the texture should be created from
         * @param constraints video constraints
         * @param invertY Defines if the video should be stored with invert Y set to true (true by default)
         * @returns The created video texture as a promise
         */
        static CreateFromStreamAsync(scene: Scene, stream: MediaStream, constraints: any, invertY?: boolean): Promise<VideoTexture>;
        /**
         * Creates a video texture straight from your WebCam video feed.
         * @param scene Define the scene the texture should be created in
         * @param constraints Define the constraints to use to create the web cam feed from WebRTC
         * @param audioConstaints Define the audio constraints to use to create the web cam feed from WebRTC
         * @param invertY Defines if the video should be stored with invert Y set to true (true by default)
         * @returns The created video texture as a promise
         */
        static CreateFromWebCamAsync(scene: Scene, constraints: {
            minWidth: number;
            maxWidth: number;
            minHeight: number;
            maxHeight: number;
            deviceId: string;
        } & MediaTrackConstraints, audioConstaints?: boolean | MediaTrackConstraints, invertY?: boolean): Promise<VideoTexture>;
        /**
         * Creates a video texture straight from your WebCam video feed.
         * @param scene Defines the scene the texture should be created in
         * @param onReady Defines a callback to triggered once the texture will be ready
         * @param constraints Defines the constraints to use to create the web cam feed from WebRTC
         * @param audioConstaints Defines the audio constraints to use to create the web cam feed from WebRTC
         * @param invertY Defines if the video should be stored with invert Y set to true (true by default)
         */
        static CreateFromWebCam(scene: Scene, onReady: (videoTexture: VideoTexture) => void, constraints: {
            minWidth: number;
            maxWidth: number;
            minHeight: number;
            maxHeight: number;
            deviceId: string;
        } & MediaTrackConstraints, audioConstaints?: boolean | MediaTrackConstraints, invertY?: boolean): void;
    }


    /**
     * Uniform buffer objects.
     *
     * Handles blocks of uniform on the GPU.
     *
     * If WebGL 2 is not available, this class falls back on traditional setUniformXXX calls.
     *
     * For more information, please refer to :
     * https://www.khronos.org/opengl/wiki/Uniform_Buffer_Object
     */
    export class UniformBuffer {
        /** @internal */
        static _UpdatedUbosInFrame: {
            [name: string]: number;
        };
        private _engine;
        private _buffer;
        private _buffers;
        private _bufferIndex;
        private _createBufferOnWrite;
        private _data;
        private _bufferData;
        private _dynamic?;
        private _uniformLocations;
        private _uniformSizes;
        private _uniformArraySizes;
        private _uniformLocationPointer;
        private _needSync;
        private _noUBO;
        private _currentEffect;
        private _currentEffectName;
        private _name;
        private _currentFrameId;
        private static _MAX_UNIFORM_SIZE;
        private static _TempBuffer;
        private static _TempBufferInt32View;
        private static _TempBufferUInt32View;
        /**
         * Lambda to Update a 3x3 Matrix in a uniform buffer.
         * This is dynamic to allow compat with webgl 1 and 2.
         * You will need to pass the name of the uniform as well as the value.
         */
        updateMatrix3x3: (name: string, matrix: Float32Array) => void;
        /**
         * Lambda to Update a 2x2 Matrix in a uniform buffer.
         * This is dynamic to allow compat with webgl 1 and 2.
         * You will need to pass the name of the uniform as well as the value.
         */
        updateMatrix2x2: (name: string, matrix: Float32Array) => void;
        /**
         * Lambda to Update a single float in a uniform buffer.
         * This is dynamic to allow compat with webgl 1 and 2.
         * You will need to pass the name of the uniform as well as the value.
         */
        updateFloat: (name: string, x: number) => void;
        /**
         * Lambda to Update a vec2 of float in a uniform buffer.
         * This is dynamic to allow compat with webgl 1 and 2.
         * You will need to pass the name of the uniform as well as the value.
         */
        updateFloat2: (name: string, x: number, y: number, suffix?: string) => void;
        /**
         * Lambda to Update a vec3 of float in a uniform buffer.
         * This is dynamic to allow compat with webgl 1 and 2.
         * You will need to pass the name of the uniform as well as the value.
         */
        updateFloat3: (name: string, x: number, y: number, z: number, suffix?: string) => void;
        /**
         * Lambda to Update a vec4 of float in a uniform buffer.
         * This is dynamic to allow compat with webgl 1 and 2.
         * You will need to pass the name of the uniform as well as the value.
         */
        updateFloat4: (name: string, x: number, y: number, z: number, w: number, suffix?: string) => void;
        /**
         * Lambda to Update an array of float in a uniform buffer.
         * This is dynamic to allow compat with webgl 1 and 2.
         * You will need to pass the name of the uniform as well as the value.
         */
        updateFloatArray: (name: string, array: Float32Array) => void;
        /**
         * Lambda to Update an array of number in a uniform buffer.
         * This is dynamic to allow compat with webgl 1 and 2.
         * You will need to pass the name of the uniform as well as the value.
         */
        updateArray: (name: string, array: number[]) => void;
        /**
         * Lambda to Update an array of number in a uniform buffer.
         * This is dynamic to allow compat with webgl 1 and 2.
         * You will need to pass the name of the uniform as well as the value.
         */
        updateIntArray: (name: string, array: Int32Array) => void;
        /**
         * Lambda to Update an array of number in a uniform buffer.
         * This is dynamic to allow compat with webgl 1 and 2.
         * You will need to pass the name of the uniform as well as the value.
         */
        updateUIntArray: (name: string, array: Uint32Array) => void;
        /**
         * Lambda to Update a 4x4 Matrix in a uniform buffer.
         * This is dynamic to allow compat with webgl 1 and 2.
         * You will need to pass the name of the uniform as well as the value.
         */
        updateMatrix: (name: string, mat: IMatrixLike) => void;
        /**
         * Lambda to Update an array of 4x4 Matrix in a uniform buffer.
         * This is dynamic to allow compat with webgl 1 and 2.
         * You will need to pass the name of the uniform as well as the value.
         */
        updateMatrices: (name: string, mat: Float32Array) => void;
        /**
         * Lambda to Update vec3 of float from a Vector in a uniform buffer.
         * This is dynamic to allow compat with webgl 1 and 2.
         * You will need to pass the name of the uniform as well as the value.
         */
        updateVector3: (name: string, vector: IVector3Like) => void;
        /**
         * Lambda to Update vec4 of float from a Vector in a uniform buffer.
         * This is dynamic to allow compat with webgl 1 and 2.
         * You will need to pass the name of the uniform as well as the value.
         */
        updateVector4: (name: string, vector: IVector4Like) => void;
        /**
         * Lambda to Update vec3 of float from a Color in a uniform buffer.
         * This is dynamic to allow compat with webgl 1 and 2.
         * You will need to pass the name of the uniform as well as the value.
         */
        updateColor3: (name: string, color: IColor3Like, suffix?: string) => void;
        /**
         * Lambda to Update vec4 of float from a Color in a uniform buffer.
         * This is dynamic to allow compat with webgl 1 and 2.
         * You will need to pass the name of the uniform as well as the value.
         */
        updateColor4: (name: string, color: IColor3Like, alpha: number, suffix?: string) => void;
        /**
         * Lambda to Update vec4 of float from a Color in a uniform buffer.
         * This is dynamic to allow compat with webgl 1 and 2.
         * You will need to pass the name of the uniform as well as the value.
         */
        updateDirectColor4: (name: string, color: IColor4Like, suffix?: string) => void;
        /**
         * Lambda to Update a int a uniform buffer.
         * This is dynamic to allow compat with webgl 1 and 2.
         * You will need to pass the name of the uniform as well as the value.
         */
        updateInt: (name: string, x: number, suffix?: string) => void;
        /**
         * Lambda to Update a vec2 of int in a uniform buffer.
         * This is dynamic to allow compat with webgl 1 and 2.
         * You will need to pass the name of the uniform as well as the value.
         */
        updateInt2: (name: string, x: number, y: number, suffix?: string) => void;
        /**
         * Lambda to Update a vec3 of int in a uniform buffer.
         * This is dynamic to allow compat with webgl 1 and 2.
         * You will need to pass the name of the uniform as well as the value.
         */
        updateInt3: (name: string, x: number, y: number, z: number, suffix?: string) => void;
        /**
         * Lambda to Update a vec4 of int in a uniform buffer.
         * This is dynamic to allow compat with webgl 1 and 2.
         * You will need to pass the name of the uniform as well as the value.
         */
        updateInt4: (name: string, x: number, y: number, z: number, w: number, suffix?: string) => void;
        /**
         * Lambda to Update a unsigned int a uniform buffer.
         * This is dynamic to allow compat with webgl 1 and 2.
         * You will need to pass the name of the uniform as well as the value.
         */
        updateUInt: (name: string, x: number, suffix?: string) => void;
        /**
         * Lambda to Update a vec2 of unsigned int in a uniform buffer.
         * This is dynamic to allow compat with webgl 1 and 2.
         * You will need to pass the name of the uniform as well as the value.
         */
        updateUInt2: (name: string, x: number, y: number, suffix?: string) => void;
        /**
         * Lambda to Update a vec3 of unsigned int in a uniform buffer.
         * This is dynamic to allow compat with webgl 1 and 2.
         * You will need to pass the name of the uniform as well as the value.
         */
        updateUInt3: (name: string, x: number, y: number, z: number, suffix?: string) => void;
        /**
         * Lambda to Update a vec4 of unsigned int in a uniform buffer.
         * This is dynamic to allow compat with webgl 1 and 2.
         * You will need to pass the name of the uniform as well as the value.
         */
        updateUInt4: (name: string, x: number, y: number, z: number, w: number, suffix?: string) => void;
        /**
         * Instantiates a new Uniform buffer objects.
         *
         * Handles blocks of uniform on the GPU.
         *
         * If WebGL 2 is not available, this class falls back on traditional setUniformXXX calls.
         *
         * For more information, please refer to :
         * @see https://www.khronos.org/opengl/wiki/Uniform_Buffer_Object
         * @param engine Define the engine the buffer is associated with
         * @param data Define the data contained in the buffer
         * @param dynamic Define if the buffer is updatable
         * @param name to assign to the buffer (debugging purpose)
         * @param forceNoUniformBuffer define that this object must not rely on UBO objects
         */
        constructor(engine: ThinEngine, data?: number[], dynamic?: boolean, name?: string, forceNoUniformBuffer?: boolean);
        /**
         * Indicates if the buffer is using the WebGL2 UBO implementation,
         * or just falling back on setUniformXXX calls.
         */
        get useUbo(): boolean;
        /**
         * Indicates if the WebGL underlying uniform buffer is in sync
         * with the javascript cache data.
         */
        get isSync(): boolean;
        /**
         * Indicates if the WebGL underlying uniform buffer is dynamic.
         * Also, a dynamic UniformBuffer will disable cache verification and always
         * update the underlying WebGL uniform buffer to the GPU.
         * @returns if Dynamic, otherwise false
         */
        isDynamic(): boolean;
        /**
         * The data cache on JS side.
         * @returns the underlying data as a float array
         */
        getData(): Float32Array;
        /**
         * The underlying WebGL Uniform buffer.
         * @returns the webgl buffer
         */
        getBuffer(): Nullable<DataBuffer>;
        /**
         * std140 layout specifies how to align data within an UBO structure.
         * See https://khronos.org/registry/OpenGL/specs/gl/glspec45.core.pdf#page=159
         * for specs.
         * @param size
         */
        private _fillAlignment;
        /**
         * Adds an uniform in the buffer.
         * Warning : the subsequents calls of this function must be in the same order as declared in the shader
         * for the layout to be correct ! The addUniform function only handles types like float, vec2, vec3, vec4, mat4,
         * meaning size=1,2,3,4 or 16. It does not handle struct types.
         * @param name Name of the uniform, as used in the uniform block in the shader.
         * @param size Data size, or data directly.
         * @param arraySize The number of elements in the array, 0 if not an array.
         */
        addUniform(name: string, size: number | number[], arraySize?: number): void;
        /**
         * Adds a Matrix 4x4 to the uniform buffer.
         * @param name Name of the uniform, as used in the uniform block in the shader.
         * @param mat A 4x4 matrix.
         */
        addMatrix(name: string, mat: IMatrixLike): void;
        /**
         * Adds a vec2 to the uniform buffer.
         * @param name Name of the uniform, as used in the uniform block in the shader.
         * @param x Define the x component value of the vec2
         * @param y Define the y component value of the vec2
         */
        addFloat2(name: string, x: number, y: number): void;
        /**
         * Adds a vec3 to the uniform buffer.
         * @param name Name of the uniform, as used in the uniform block in the shader.
         * @param x Define the x component value of the vec3
         * @param y Define the y component value of the vec3
         * @param z Define the z component value of the vec3
         */
        addFloat3(name: string, x: number, y: number, z: number): void;
        /**
         * Adds a vec3 to the uniform buffer.
         * @param name Name of the uniform, as used in the uniform block in the shader.
         * @param color Define the vec3 from a Color
         */
        addColor3(name: string, color: IColor3Like): void;
        /**
         * Adds a vec4 to the uniform buffer.
         * @param name Name of the uniform, as used in the uniform block in the shader.
         * @param color Define the rgb components from a Color
         * @param alpha Define the a component of the vec4
         */
        addColor4(name: string, color: IColor3Like, alpha: number): void;
        /**
         * Adds a vec3 to the uniform buffer.
         * @param name Name of the uniform, as used in the uniform block in the shader.
         * @param vector Define the vec3 components from a Vector
         */
        addVector3(name: string, vector: IVector3Like): void;
        /**
         * Adds a Matrix 3x3 to the uniform buffer.
         * @param name Name of the uniform, as used in the uniform block in the shader.
         */
        addMatrix3x3(name: string): void;
        /**
         * Adds a Matrix 2x2 to the uniform buffer.
         * @param name Name of the uniform, as used in the uniform block in the shader.
         */
        addMatrix2x2(name: string): void;
        /**
         * Effectively creates the WebGL Uniform Buffer, once layout is completed with `addUniform`.
         */
        create(): void;
        /** @internal */
        _rebuild(): void;
        /** @internal */
        get _numBuffers(): number;
        /** @internal */
        get _indexBuffer(): number;
        /** Gets the name of this buffer */
        get name(): string;
        private _buffersEqual;
        private _copyBuffer;
        /**
         * Updates the WebGL Uniform Buffer on the GPU.
         * If the `dynamic` flag is set to true, no cache comparison is done.
         * Otherwise, the buffer will be updated only if the cache differs.
         */
        update(): void;
        private _createNewBuffer;
        private _checkNewFrame;
        /**
         * Updates the value of an uniform. The `update` method must be called afterwards to make it effective in the GPU.
         * @param uniformName Define the name of the uniform, as used in the uniform block in the shader.
         * @param data Define the flattened data
         * @param size Define the size of the data.
         */
        updateUniform(uniformName: string, data: FloatArray, size: number): void;
        /**
         * Updates the value of an uniform. The `update` method must be called afterwards to make it effective in the GPU.
         * @param uniformName Define the name of the uniform, as used in the uniform block in the shader.
         * @param data Define the flattened data
         * @param size Define the size of the data.
         */
        updateUniformArray(uniformName: string, data: FloatArray, size: number): void;
        private _valueCache;
        private _cacheMatrix;
        private _updateMatrix3x3ForUniform;
        private _updateMatrix3x3ForEffect;
        private _updateMatrix2x2ForEffect;
        private _updateMatrix2x2ForUniform;
        private _updateFloatForEffect;
        private _updateFloatForUniform;
        private _updateFloat2ForEffect;
        private _updateFloat2ForUniform;
        private _updateFloat3ForEffect;
        private _updateFloat3ForUniform;
        private _updateFloat4ForEffect;
        private _updateFloat4ForUniform;
        private _updateFloatArrayForEffect;
        private _updateFloatArrayForUniform;
        private _updateArrayForEffect;
        private _updateArrayForUniform;
        private _updateIntArrayForEffect;
        private _updateIntArrayForUniform;
        private _updateUIntArrayForEffect;
        private _updateUIntArrayForUniform;
        private _updateMatrixForEffect;
        private _updateMatrixForUniform;
        private _updateMatricesForEffect;
        private _updateMatricesForUniform;
        private _updateVector3ForEffect;
        private _updateVector3ForUniform;
        private _updateVector4ForEffect;
        private _updateVector4ForUniform;
        private _updateColor3ForEffect;
        private _updateColor3ForUniform;
        private _updateColor4ForEffect;
        private _updateDirectColor4ForEffect;
        private _updateColor4ForUniform;
        private _updateDirectColor4ForUniform;
        private _updateIntForEffect;
        private _updateIntForUniform;
        private _updateInt2ForEffect;
        private _updateInt2ForUniform;
        private _updateInt3ForEffect;
        private _updateInt3ForUniform;
        private _updateInt4ForEffect;
        private _updateInt4ForUniform;
        private _updateUIntForEffect;
        private _updateUIntForUniform;
        private _updateUInt2ForEffect;
        private _updateUInt2ForUniform;
        private _updateUInt3ForEffect;
        private _updateUInt3ForUniform;
        private _updateUInt4ForEffect;
        private _updateUInt4ForUniform;
        /**
         * Sets a sampler uniform on the effect.
         * @param name Define the name of the sampler.
         * @param texture Define the texture to set in the sampler
         */
        setTexture(name: string, texture: Nullable<ThinTexture>): void;
        /**
         * Directly updates the value of the uniform in the cache AND on the GPU.
         * @param uniformName Define the name of the uniform, as used in the uniform block in the shader.
         * @param data Define the flattened data
         */
        updateUniformDirectly(uniformName: string, data: FloatArray): void;
        /**
         * Associates an effect to this uniform buffer
         * @param effect Define the effect to associate the buffer to
         * @param name Name of the uniform block in the shader.
         */
        bindToEffect(effect: Effect, name: string): void;
        /**
         * Binds the current (GPU) buffer to the effect
         */
        bindUniformBuffer(): void;
        /**
         * Dissociates the current effect from this uniform buffer
         */
        unbindEffect(): void;
        /**
         * Sets the current state of the class (_bufferIndex, _buffer) to point to the data buffer passed in parameter if this buffer is one of the buffers handled by the class (meaning if it can be found in the _buffers array)
         * This method is meant to be able to update a buffer at any time: just call setDataBuffer to set the class in the right state, call some updateXXX methods and then call udpate() => that will update the GPU buffer on the graphic card
         * @param dataBuffer buffer to look for
         * @returns true if the buffer has been found and the class internal state points to it, else false
         */
        setDataBuffer(dataBuffer: DataBuffer): boolean;
        /**
         * Disposes the uniform buffer.
         */
        dispose(): void;
    }


    /** @internal */
    export class UniformBufferEffectCommonAccessor {
        setMatrix3x3: (name: string, matrix: Float32Array) => void;
        setMatrix2x2: (name: string, matrix: Float32Array) => void;
        setFloat: (name: string, x: number) => void;
        setFloat2: (name: string, x: number, y: number, suffix?: string) => void;
        setFloat3: (name: string, x: number, y: number, z: number, suffix?: string) => void;
        setFloat4: (name: string, x: number, y: number, z: number, w: number, suffix?: string) => void;
        setFloatArray: (name: string, array: Float32Array) => void;
        setArray: (name: string, array: number[]) => void;
        setIntArray: (name: string, array: Int32Array) => void;
        setMatrix: (name: string, mat: IMatrixLike) => void;
        setMatrices: (name: string, mat: Float32Array) => void;
        setVector3: (name: string, vector: IVector3Like) => void;
        setVector4: (name: string, vector: IVector4Like) => void;
        setColor3: (name: string, color: IColor3Like, suffix?: string) => void;
        setColor4: (name: string, color: IColor3Like, alpha: number, suffix?: string) => void;
        setDirectColor4: (name: string, color: IColor4Like) => void;
        setInt: (name: string, x: number, suffix?: string) => void;
        setInt2: (name: string, x: number, y: number, suffix?: string) => void;
        setInt3: (name: string, x: number, y: number, z: number, suffix?: string) => void;
        setInt4: (name: string, x: number, y: number, z: number, w: number, suffix?: string) => void;
        private _isUbo;
        constructor(uboOrEffect: UniformBuffer | Effect);
    }




    /** Defines supported spaces */
    export enum Space {
        /** Local (object) space */
        LOCAL = 0,
        /** World space */
        WORLD = 1,
        /** Bone space */
        BONE = 2
    }
    /** Defines the 3 main axes */
    export class Axis {
        /** X axis */
        static X: Vector3;
        /** Y axis */
        static Y: Vector3;
        /** Z axis */
        static Z: Vector3;
    }
    /**
     * Defines cartesian components.
     */
    export enum Coordinate {
        /** X axis */
        X = 0,
        /** Y axis */
        Y = 1,
        /** Z axis */
        Z = 2
    }


    /**
     * Class used to hold a RGB color
     */
    export class Color3 {
        /**
         * Defines the red component (between 0 and 1, default is 0)
         */
        r: number;
        /**
         * Defines the green component (between 0 and 1, default is 0)
         */
        g: number;
        /**
         * Defines the blue component (between 0 and 1, default is 0)
         */
        b: number;
        /**
         * Creates a new Color3 object from red, green, blue values, all between 0 and 1
         * @param r defines the red component (between 0 and 1, default is 0)
         * @param g defines the green component (between 0 and 1, default is 0)
         * @param b defines the blue component (between 0 and 1, default is 0)
         */
        constructor(
        /**
         * Defines the red component (between 0 and 1, default is 0)
         */
        r?: number, 
        /**
         * Defines the green component (between 0 and 1, default is 0)
         */
        g?: number, 
        /**
         * Defines the blue component (between 0 and 1, default is 0)
         */
        b?: number);
        /**
         * Creates a string with the Color3 current values
         * @returns the string representation of the Color3 object
         */
        toString(): string;
        /**
         * Returns the string "Color3"
         * @returns "Color3"
         */
        getClassName(): string;
        /**
         * Compute the Color3 hash code
         * @returns an unique number that can be used to hash Color3 objects
         */
        getHashCode(): number;
        /**
         * Stores in the given array from the given starting index the red, green, blue values as successive elements
         * @param array defines the array where to store the r,g,b components
         * @param index defines an optional index in the target array to define where to start storing values
         * @returns the current Color3 object
         */
        toArray(array: FloatArray, index?: number): Color3;
        /**
         * Update the current color with values stored in an array from the starting index of the given array
         * @param array defines the source array
         * @param offset defines an offset in the source array
         * @returns the current Color3 object
         */
        fromArray(array: DeepImmutable<ArrayLike<number>>, offset?: number): Color3;
        /**
         * Returns a new Color4 object from the current Color3 and the given alpha
         * @param alpha defines the alpha component on the new Color4 object (default is 1)
         * @returns a new Color4 object
         */
        toColor4(alpha?: number): Color4;
        /**
         * Returns a new array populated with 3 numeric elements : red, green and blue values
         * @returns the new array
         */
        asArray(): number[];
        /**
         * Returns the luminance value
         * @returns a float value
         */
        toLuminance(): number;
        /**
         * Multiply each Color3 rgb values by the given Color3 rgb values in a new Color3 object
         * @param otherColor defines the second operand
         * @returns the new Color3 object
         */
        multiply(otherColor: DeepImmutable<Color3>): Color3;
        /**
         * Multiply the rgb values of the Color3 and the given Color3 and stores the result in the object "result"
         * @param otherColor defines the second operand
         * @param result defines the Color3 object where to store the result
         * @returns the current Color3
         */
        multiplyToRef(otherColor: DeepImmutable<Color3>, result: Color3): Color3;
        /**
         * Determines equality between Color3 objects
         * @param otherColor defines the second operand
         * @returns true if the rgb values are equal to the given ones
         */
        equals(otherColor: DeepImmutable<Color3>): boolean;
        /**
         * Determines equality between the current Color3 object and a set of r,b,g values
         * @param r defines the red component to check
         * @param g defines the green component to check
         * @param b defines the blue component to check
         * @returns true if the rgb values are equal to the given ones
         */
        equalsFloats(r: number, g: number, b: number): boolean;
        /**
         * Creates a new Color3 with the current Color3 values multiplied by scale
         * @param scale defines the scaling factor to apply
         * @returns a new Color3 object
         */
        scale(scale: number): Color3;
        /**
         * Multiplies the Color3 values by the float "scale"
         * @param scale defines the scaling factor to apply
         * @returns the current updated Color3
         */
        scaleInPlace(scale: number): Color3;
        /**
         * Multiplies the rgb values by scale and stores the result into "result"
         * @param scale defines the scaling factor
         * @param result defines the Color3 object where to store the result
         * @returns the unmodified current Color3
         */
        scaleToRef(scale: number, result: Color3): Color3;
        /**
         * Scale the current Color3 values by a factor and add the result to a given Color3
         * @param scale defines the scale factor
         * @param result defines color to store the result into
         * @returns the unmodified current Color3
         */
        scaleAndAddToRef(scale: number, result: Color3): Color3;
        /**
         * Clamps the rgb values by the min and max values and stores the result into "result"
         * @param min defines minimum clamping value (default is 0)
         * @param max defines maximum clamping value (default is 1)
         * @param result defines color to store the result into
         * @returns the original Color3
         */
        clampToRef(min: number | undefined, max: number | undefined, result: Color3): Color3;
        /**
         * Creates a new Color3 set with the added values of the current Color3 and of the given one
         * @param otherColor defines the second operand
         * @returns the new Color3
         */
        add(otherColor: DeepImmutable<Color3>): Color3;
        /**
         * Stores the result of the addition of the current Color3 and given one rgb values into "result"
         * @param otherColor defines the second operand
         * @param result defines Color3 object to store the result into
         * @returns the unmodified current Color3
         */
        addToRef(otherColor: DeepImmutable<Color3>, result: Color3): Color3;
        /**
         * Returns a new Color3 set with the subtracted values of the given one from the current Color3
         * @param otherColor defines the second operand
         * @returns the new Color3
         */
        subtract(otherColor: DeepImmutable<Color3>): Color3;
        /**
         * Stores the result of the subtraction of given one from the current Color3 rgb values into "result"
         * @param otherColor defines the second operand
         * @param result defines Color3 object to store the result into
         * @returns the unmodified current Color3
         */
        subtractToRef(otherColor: DeepImmutable<Color3>, result: Color3): Color3;
        /**
         * Copy the current object
         * @returns a new Color3 copied the current one
         */
        clone(): Color3;
        /**
         * Copies the rgb values from the source in the current Color3
         * @param source defines the source Color3 object
         * @returns the updated Color3 object
         */
        copyFrom(source: DeepImmutable<Color3>): Color3;
        /**
         * Updates the Color3 rgb values from the given floats
         * @param r defines the red component to read from
         * @param g defines the green component to read from
         * @param b defines the blue component to read from
         * @returns the current Color3 object
         */
        copyFromFloats(r: number, g: number, b: number): Color3;
        /**
         * Updates the Color3 rgb values from the given floats
         * @param r defines the red component to read from
         * @param g defines the green component to read from
         * @param b defines the blue component to read from
         * @returns the current Color3 object
         */
        set(r: number, g: number, b: number): Color3;
        /**
         * Compute the Color3 hexadecimal code as a string
         * @returns a string containing the hexadecimal representation of the Color3 object
         */
        toHexString(): string;
        /**
         * Computes a new Color3 converted from the current one to linear space
         * @returns a new Color3 object
         */
        toLinearSpace(): Color3;
        /**
         * Converts current color in rgb space to HSV values
         * @returns a new color3 representing the HSV values
         */
        toHSV(): Color3;
        /**
         * Converts current color in rgb space to HSV values
         * @param result defines the Color3 where to store the HSV values
         */
        toHSVToRef(result: Color3): void;
        /**
         * Converts the Color3 values to linear space and stores the result in "convertedColor"
         * @param convertedColor defines the Color3 object where to store the linear space version
         * @returns the unmodified Color3
         */
        toLinearSpaceToRef(convertedColor: Color3): Color3;
        /**
         * Computes a new Color3 converted from the current one to gamma space
         * @returns a new Color3 object
         */
        toGammaSpace(): Color3;
        /**
         * Converts the Color3 values to gamma space and stores the result in "convertedColor"
         * @param convertedColor defines the Color3 object where to store the gamma space version
         * @returns the unmodified Color3
         */
        toGammaSpaceToRef(convertedColor: Color3): Color3;
        private static _BlackReadOnly;
        /**
         * Converts Hue, saturation and value to a Color3 (RGB)
         * @param hue defines the hue
         * @param saturation defines the saturation
         * @param value defines the value
         * @param result defines the Color3 where to store the RGB values
         */
        static HSVtoRGBToRef(hue: number, saturation: number, value: number, result: Color3): void;
        /**
         * Converts Hue, saturation and value to a new Color3 (RGB)
         * @param hue defines the hue (value between 0 and 360)
         * @param saturation defines the saturation (value between 0 and 1)
         * @param value defines the value (value between 0 and 1)
         * @returns a new Color3 object
         */
        static FromHSV(hue: number, saturation: number, value: number): Color3;
        /**
         * Creates a new Color3 from the string containing valid hexadecimal values
         * @param hex defines a string containing valid hexadecimal values
         * @returns a new Color3 object
         */
        static FromHexString(hex: string): Color3;
        /**
         * Creates a new Color3 from the starting index of the given array
         * @param array defines the source array
         * @param offset defines an offset in the source array
         * @returns a new Color3 object
         */
        static FromArray(array: DeepImmutable<ArrayLike<number>>, offset?: number): Color3;
        /**
         * Creates a new Color3 from the starting index element of the given array
         * @param array defines the source array to read from
         * @param offset defines the offset in the source array
         * @param result defines the target Color3 object
         */
        static FromArrayToRef(array: DeepImmutable<ArrayLike<number>>, offset: number | undefined, result: Color3): void;
        /**
         * Creates a new Color3 from integer values (< 256)
         * @param r defines the red component to read from (value between 0 and 255)
         * @param g defines the green component to read from (value between 0 and 255)
         * @param b defines the blue component to read from (value between 0 and 255)
         * @returns a new Color3 object
         */
        static FromInts(r: number, g: number, b: number): Color3;
        /**
         * Creates a new Color3 with values linearly interpolated of "amount" between the start Color3 and the end Color3
         * @param start defines the start Color3 value
         * @param end defines the end Color3 value
         * @param amount defines the gradient value between start and end
         * @returns a new Color3 object
         */
        static Lerp(start: DeepImmutable<Color3>, end: DeepImmutable<Color3>, amount: number): Color3;
        /**
         * Creates a new Color3 with values linearly interpolated of "amount" between the start Color3 and the end Color3
         * @param left defines the start value
         * @param right defines the end value
         * @param amount defines the gradient factor
         * @param result defines the Color3 object where to store the result
         */
        static LerpToRef(left: DeepImmutable<Color3>, right: DeepImmutable<Color3>, amount: number, result: Color3): void;
        /**
         * Returns a new Color3 located for "amount" (float) on the Hermite interpolation spline defined by the vectors "value1", "tangent1", "value2", "tangent2"
         * @param value1 defines the first control point
         * @param tangent1 defines the first tangent Color3
         * @param value2 defines the second control point
         * @param tangent2 defines the second tangent Color3
         * @param amount defines the amount on the interpolation spline (between 0 and 1)
         * @returns the new Color3
         */
        static Hermite(value1: DeepImmutable<Color3>, tangent1: DeepImmutable<Color3>, value2: DeepImmutable<Color3>, tangent2: DeepImmutable<Color3>, amount: number): Color3;
        /**
         * Returns a new Color3 which is the 1st derivative of the Hermite spline defined by the colors "value1", "value2", "tangent1", "tangent2".
         * @param value1 defines the first control point
         * @param tangent1 defines the first tangent
         * @param value2 defines the second control point
         * @param tangent2 defines the second tangent
         * @param time define where the derivative must be done
         * @returns 1st derivative
         */
        static Hermite1stDerivative(value1: DeepImmutable<Color3>, tangent1: DeepImmutable<Color3>, value2: DeepImmutable<Color3>, tangent2: DeepImmutable<Color3>, time: number): Color3;
        /**
         * Returns a new Color3 which is the 1st derivative of the Hermite spline defined by the colors "value1", "value2", "tangent1", "tangent2".
         * @param value1 defines the first control point
         * @param tangent1 defines the first tangent
         * @param value2 defines the second control point
         * @param tangent2 defines the second tangent
         * @param time define where the derivative must be done
         * @param result define where to store the derivative
         */
        static Hermite1stDerivativeToRef(value1: DeepImmutable<Color3>, tangent1: DeepImmutable<Color3>, value2: DeepImmutable<Color3>, tangent2: DeepImmutable<Color3>, time: number, result: Color3): void;
        /**
         * Returns a Color3 value containing a red color
         * @returns a new Color3 object
         */
        static Red(): Color3;
        /**
         * Returns a Color3 value containing a green color
         * @returns a new Color3 object
         */
        static Green(): Color3;
        /**
         * Returns a Color3 value containing a blue color
         * @returns a new Color3 object
         */
        static Blue(): Color3;
        /**
         * Returns a Color3 value containing a black color
         * @returns a new Color3 object
         */
        static Black(): Color3;
        /**
         * Gets a Color3 value containing a black color that must not be updated
         */
        static get BlackReadOnly(): DeepImmutable<Color3>;
        /**
         * Returns a Color3 value containing a white color
         * @returns a new Color3 object
         */
        static White(): Color3;
        /**
         * Returns a Color3 value containing a purple color
         * @returns a new Color3 object
         */
        static Purple(): Color3;
        /**
         * Returns a Color3 value containing a magenta color
         * @returns a new Color3 object
         */
        static Magenta(): Color3;
        /**
         * Returns a Color3 value containing a yellow color
         * @returns a new Color3 object
         */
        static Yellow(): Color3;
        /**
         * Returns a Color3 value containing a gray color
         * @returns a new Color3 object
         */
        static Gray(): Color3;
        /**
         * Returns a Color3 value containing a teal color
         * @returns a new Color3 object
         */
        static Teal(): Color3;
        /**
         * Returns a Color3 value containing a random color
         * @returns a new Color3 object
         */
        static Random(): Color3;
    }
    /**
     * Class used to hold a RBGA color
     */
    export class Color4 {
        /**
         * Defines the red component (between 0 and 1, default is 0)
         */
        r: number;
        /**
         * Defines the green component (between 0 and 1, default is 0)
         */
        g: number;
        /**
         * Defines the blue component (between 0 and 1, default is 0)
         */
        b: number;
        /**
         * Defines the alpha component (between 0 and 1, default is 1)
         */
        a: number;
        /**
         * Creates a new Color4 object from red, green, blue values, all between 0 and 1
         * @param r defines the red component (between 0 and 1, default is 0)
         * @param g defines the green component (between 0 and 1, default is 0)
         * @param b defines the blue component (between 0 and 1, default is 0)
         * @param a defines the alpha component (between 0 and 1, default is 1)
         */
        constructor(
        /**
         * Defines the red component (between 0 and 1, default is 0)
         */
        r?: number, 
        /**
         * Defines the green component (between 0 and 1, default is 0)
         */
        g?: number, 
        /**
         * Defines the blue component (between 0 and 1, default is 0)
         */
        b?: number, 
        /**
         * Defines the alpha component (between 0 and 1, default is 1)
         */
        a?: number);
        /**
         * Adds in place the given Color4 values to the current Color4 object
         * @param right defines the second operand
         * @returns the current updated Color4 object
         */
        addInPlace(right: DeepImmutable<Color4>): Color4;
        /**
         * Creates a new array populated with 4 numeric elements : red, green, blue, alpha values
         * @returns the new array
         */
        asArray(): number[];
        /**
         * Stores from the starting index in the given array the Color4 successive values
         * @param array defines the array where to store the r,g,b components
         * @param index defines an optional index in the target array to define where to start storing values
         * @returns the current Color4 object
         */
        toArray(array: FloatArray, index?: number): Color4;
        /**
         * Update the current color with values stored in an array from the starting index of the given array
         * @param array defines the source array
         * @param offset defines an offset in the source array
         * @returns the current Color4 object
         */
        fromArray(array: DeepImmutable<ArrayLike<number>>, offset?: number): Color4;
        /**
         * Determines equality between Color4 objects
         * @param otherColor defines the second operand
         * @returns true if the rgba values are equal to the given ones
         */
        equals(otherColor: DeepImmutable<Color4>): boolean;
        /**
         * Creates a new Color4 set with the added values of the current Color4 and of the given one
         * @param right defines the second operand
         * @returns a new Color4 object
         */
        add(right: DeepImmutable<Color4>): Color4;
        /**
         * Creates a new Color4 set with the subtracted values of the given one from the current Color4
         * @param right defines the second operand
         * @returns a new Color4 object
         */
        subtract(right: DeepImmutable<Color4>): Color4;
        /**
         * Subtracts the given ones from the current Color4 values and stores the results in "result"
         * @param right defines the second operand
         * @param result defines the Color4 object where to store the result
         * @returns the current Color4 object
         */
        subtractToRef(right: DeepImmutable<Color4>, result: Color4): Color4;
        /**
         * Creates a new Color4 with the current Color4 values multiplied by scale
         * @param scale defines the scaling factor to apply
         * @returns a new Color4 object
         */
        scale(scale: number): Color4;
        /**
         * Multiplies the Color4 values by the float "scale"
         * @param scale defines the scaling factor to apply
         * @returns the current updated Color4
         */
        scaleInPlace(scale: number): Color4;
        /**
         * Multiplies the current Color4 values by scale and stores the result in "result"
         * @param scale defines the scaling factor to apply
         * @param result defines the Color4 object where to store the result
         * @returns the current unmodified Color4
         */
        scaleToRef(scale: number, result: Color4): Color4;
        /**
         * Scale the current Color4 values by a factor and add the result to a given Color4
         * @param scale defines the scale factor
         * @param result defines the Color4 object where to store the result
         * @returns the unmodified current Color4
         */
        scaleAndAddToRef(scale: number, result: Color4): Color4;
        /**
         * Clamps the rgb values by the min and max values and stores the result into "result"
         * @param min defines minimum clamping value (default is 0)
         * @param max defines maximum clamping value (default is 1)
         * @param result defines color to store the result into.
         * @returns the current Color4
         */
        clampToRef(min: number | undefined, max: number | undefined, result: Color4): Color4;
        /**
         * Multiply an Color4 value by another and return a new Color4 object
         * @param color defines the Color4 value to multiply by
         * @returns a new Color4 object
         */
        multiply(color: Color4): Color4;
        /**
         * Multiply a Color4 value by another and push the result in a reference value
         * @param color defines the Color4 value to multiply by
         * @param result defines the Color4 to fill the result in
         * @returns the result Color4
         */
        multiplyToRef(color: Color4, result: Color4): Color4;
        /**
         * Creates a string with the Color4 current values
         * @returns the string representation of the Color4 object
         */
        toString(): string;
        /**
         * Returns the string "Color4"
         * @returns "Color4"
         */
        getClassName(): string;
        /**
         * Compute the Color4 hash code
         * @returns an unique number that can be used to hash Color4 objects
         */
        getHashCode(): number;
        /**
         * Creates a new Color4 copied from the current one
         * @returns a new Color4 object
         */
        clone(): Color4;
        /**
         * Copies the given Color4 values into the current one
         * @param source defines the source Color4 object
         * @returns the current updated Color4 object
         */
        copyFrom(source: Color4): Color4;
        /**
         * Copies the given float values into the current one
         * @param r defines the red component to read from
         * @param g defines the green component to read from
         * @param b defines the blue component to read from
         * @param a defines the alpha component to read from
         * @returns the current updated Color4 object
         */
        copyFromFloats(r: number, g: number, b: number, a: number): Color4;
        /**
         * Copies the given float values into the current one
         * @param r defines the red component to read from
         * @param g defines the green component to read from
         * @param b defines the blue component to read from
         * @param a defines the alpha component to read from
         * @returns the current updated Color4 object
         */
        set(r: number, g: number, b: number, a: number): Color4;
        /**
         * Compute the Color4 hexadecimal code as a string
         * @param returnAsColor3 defines if the string should only contains RGB values (off by default)
         * @returns a string containing the hexadecimal representation of the Color4 object
         */
        toHexString(returnAsColor3?: boolean): string;
        /**
         * Computes a new Color4 converted from the current one to linear space
         * @returns a new Color4 object
         */
        toLinearSpace(): Color4;
        /**
         * Converts the Color4 values to linear space and stores the result in "convertedColor"
         * @param convertedColor defines the Color4 object where to store the linear space version
         * @returns the unmodified Color4
         */
        toLinearSpaceToRef(convertedColor: Color4): Color4;
        /**
         * Computes a new Color4 converted from the current one to gamma space
         * @returns a new Color4 object
         */
        toGammaSpace(): Color4;
        /**
         * Converts the Color4 values to gamma space and stores the result in "convertedColor"
         * @param convertedColor defines the Color4 object where to store the gamma space version
         * @returns the unmodified Color4
         */
        toGammaSpaceToRef(convertedColor: Color4): Color4;
        /**
         * Creates a new Color4 from the string containing valid hexadecimal values.
         *
         * A valid hex string is either in the format #RRGGBB or #RRGGBBAA.
         *
         * When a hex string without alpha is passed, the resulting Color4 has
         * its alpha value set to 1.0.
         *
         * An invalid string results in a Color with all its channels set to 0.0,
         * i.e. "transparent black".
         *
         * @param hex defines a string containing valid hexadecimal values
         * @returns a new Color4 object
         */
        static FromHexString(hex: string): Color4;
        /**
         * Creates a new Color4 object set with the linearly interpolated values of "amount" between the left Color4 object and the right Color4 object
         * @param left defines the start value
         * @param right defines the end value
         * @param amount defines the gradient factor
         * @returns a new Color4 object
         */
        static Lerp(left: DeepImmutable<Color4>, right: DeepImmutable<Color4>, amount: number): Color4;
        /**
         * Set the given "result" with the linearly interpolated values of "amount" between the left Color4 object and the right Color4 object
         * @param left defines the start value
         * @param right defines the end value
         * @param amount defines the gradient factor
         * @param result defines the Color4 object where to store data
         */
        static LerpToRef(left: DeepImmutable<Color4>, right: DeepImmutable<Color4>, amount: number, result: Color4): void;
        /**
         * Interpolate between two Color4 using Hermite interpolation
         * @param value1 defines first Color4
         * @param tangent1 defines the incoming tangent
         * @param value2 defines second Color4
         * @param tangent2 defines the outgoing tangent
         * @param amount defines the target Color4
         * @returns the new interpolated Color4
         */
        static Hermite(value1: DeepImmutable<Color4>, tangent1: DeepImmutable<Color4>, value2: DeepImmutable<Color4>, tangent2: DeepImmutable<Color4>, amount: number): Color4;
        /**
         * Returns a new Color4 which is the 1st derivative of the Hermite spline defined by the colors "value1", "value2", "tangent1", "tangent2".
         * @param value1 defines the first control point
         * @param tangent1 defines the first tangent
         * @param value2 defines the second control point
         * @param tangent2 defines the second tangent
         * @param time define where the derivative must be done
         * @returns 1st derivative
         */
        static Hermite1stDerivative(value1: DeepImmutable<Color4>, tangent1: DeepImmutable<Color4>, value2: DeepImmutable<Color4>, tangent2: DeepImmutable<Color4>, time: number): Color4;
        /**
         * Update a Color4 with the 1st derivative of the Hermite spline defined by the colors "value1", "value2", "tangent1", "tangent2".
         * @param value1 defines the first control point
         * @param tangent1 defines the first tangent
         * @param value2 defines the second control point
         * @param tangent2 defines the second tangent
         * @param time define where the derivative must be done
         * @param result define where to store the derivative
         */
        static Hermite1stDerivativeToRef(value1: DeepImmutable<Color4>, tangent1: DeepImmutable<Color4>, value2: DeepImmutable<Color4>, tangent2: DeepImmutable<Color4>, time: number, result: Color4): void;
        /**
         * Creates a new Color4 from a Color3 and an alpha value
         * @param color3 defines the source Color3 to read from
         * @param alpha defines the alpha component (1.0 by default)
         * @returns a new Color4 object
         */
        static FromColor3(color3: DeepImmutable<Color3>, alpha?: number): Color4;
        /**
         * Creates a new Color4 from the starting index element of the given array
         * @param array defines the source array to read from
         * @param offset defines the offset in the source array
         * @returns a new Color4 object
         */
        static FromArray(array: DeepImmutable<ArrayLike<number>>, offset?: number): Color4;
        /**
         * Creates a new Color4 from the starting index element of the given array
         * @param array defines the source array to read from
         * @param offset defines the offset in the source array
         * @param result defines the target Color4 object
         */
        static FromArrayToRef(array: DeepImmutable<ArrayLike<number>>, offset: number | undefined, result: Color4): void;
        /**
         * Creates a new Color3 from integer values (< 256)
         * @param r defines the red component to read from (value between 0 and 255)
         * @param g defines the green component to read from (value between 0 and 255)
         * @param b defines the blue component to read from (value between 0 and 255)
         * @param a defines the alpha component to read from (value between 0 and 255)
         * @returns a new Color3 object
         */
        static FromInts(r: number, g: number, b: number, a: number): Color4;
        /**
         * Check the content of a given array and convert it to an array containing RGBA data
         * If the original array was already containing count * 4 values then it is returned directly
         * @param colors defines the array to check
         * @param count defines the number of RGBA data to expect
         * @returns an array containing count * 4 values (RGBA)
         */
        static CheckColors4(colors: number[], count: number): number[];
    }
    /**
     * @internal
     */
    export class TmpColors {
        static Color3: Color3[];
        static Color4: Color4[];
    }


    /**
     * Constant used to convert a value to gamma space
     * @ignorenaming
     */
    export var ToGammaSpace: number;
    /**
     * Constant used to convert a value to linear space
     * @ignorenaming
     */
    export const ToLinearSpace = 2.2;
    /**
     * Constant Golden Ratio value in Babylon.js
     * @ignorenaming
     */
    export var PHI: number;
    /**
     * Constant used to define the minimal number value in Babylon.js
     * @ignorenaming
     */
    const Epsilon = 0.001;




    /**
     * Represents a camera frustum
     */
    export class Frustum {
        /**
         * Gets the planes representing the frustum
         * @param transform matrix to be applied to the returned planes
         * @returns a new array of 6 Frustum planes computed by the given transformation matrix.
         */
        static GetPlanes(transform: DeepImmutable<Matrix>): Plane[];
        /**
         * Gets the near frustum plane transformed by the transform matrix
         * @param transform transformation matrix to be applied to the resulting frustum plane
         * @param frustumPlane the resulting frustum plane
         */
        static GetNearPlaneToRef(transform: DeepImmutable<Matrix>, frustumPlane: Plane): void;
        /**
         * Gets the far frustum plane transformed by the transform matrix
         * @param transform transformation matrix to be applied to the resulting frustum plane
         * @param frustumPlane the resulting frustum plane
         */
        static GetFarPlaneToRef(transform: DeepImmutable<Matrix>, frustumPlane: Plane): void;
        /**
         * Gets the left frustum plane transformed by the transform matrix
         * @param transform transformation matrix to be applied to the resulting frustum plane
         * @param frustumPlane the resulting frustum plane
         */
        static GetLeftPlaneToRef(transform: DeepImmutable<Matrix>, frustumPlane: Plane): void;
        /**
         * Gets the right frustum plane transformed by the transform matrix
         * @param transform transformation matrix to be applied to the resulting frustum plane
         * @param frustumPlane the resulting frustum plane
         */
        static GetRightPlaneToRef(transform: DeepImmutable<Matrix>, frustumPlane: Plane): void;
        /**
         * Gets the top frustum plane transformed by the transform matrix
         * @param transform transformation matrix to be applied to the resulting frustum plane
         * @param frustumPlane the resulting frustum plane
         */
        static GetTopPlaneToRef(transform: DeepImmutable<Matrix>, frustumPlane: Plane): void;
        /**
         * Gets the bottom frustum plane transformed by the transform matrix
         * @param transform transformation matrix to be applied to the resulting frustum plane
         * @param frustumPlane the resulting frustum plane
         */
        static GetBottomPlaneToRef(transform: DeepImmutable<Matrix>, frustumPlane: Plane): void;
        /**
         * Sets the given array "frustumPlanes" with the 6 Frustum planes computed by the given transformation matrix.
         * @param transform transformation matrix to be applied to the resulting frustum planes
         * @param frustumPlanes the resulting frustum planes
         */
        static GetPlanesToRef(transform: DeepImmutable<Matrix>, frustumPlanes: Plane[]): void;
    }


    /**
     * Extracts minimum and maximum values from a list of indexed positions
     * @param positions defines the positions to use
     * @param indices defines the indices to the positions
     * @param indexStart defines the start index
     * @param indexCount defines the end index
     * @param bias defines bias value to add to the result
     * @returns minimum and maximum values
     */
    export function extractMinAndMaxIndexed(positions: FloatArray, indices: IndicesArray, indexStart: number, indexCount: number, bias?: Nullable<Vector2>): {
        minimum: Vector3;
        maximum: Vector3;
    };
    /**
     * Extracts minimum and maximum values from a list of positions
     * @param positions defines the positions to use
     * @param start defines the start index in the positions array
     * @param count defines the number of positions to handle
     * @param bias defines bias value to add to the result
     * @param stride defines the stride size to use (distance between two positions in the positions array)
     * @returns minimum and maximum values
     */
    export function extractMinAndMax(positions: FloatArray, start: number, count: number, bias?: Nullable<Vector2>, stride?: number): {
        minimum: Vector3;
        maximum: Vector3;
    };


    /**
     * Class representing an isovector a vector containing 2 INTEGER coordinates
     * x axis is horizontal
     * y axis is 60 deg counter clockwise from positive y axis
     * @internal
     */
    export class _IsoVector {
        /** defines the first coordinate */
        x: number;
        /** defines the second coordinate */
        y: number;
        /**
         * Creates a new isovector from the given x and y coordinates
         * @param x defines the first coordinate, must be an integer
         * @param y defines the second coordinate, must be an integer
         */
        constructor(
        /** defines the first coordinate */
        x?: number, 
        /** defines the second coordinate */
        y?: number);
        /**
         * Gets a new IsoVector copied from the IsoVector
         * @returns a new IsoVector
         */
        clone(): _IsoVector;
        /**
         * Rotates one IsoVector 60 degrees counter clockwise about another
         * Please note that this is an in place operation
         * @param other an IsoVector a center of rotation
         * @returns the rotated IsoVector
         */
        rotate60About(other: _IsoVector): this;
        /**
         * Rotates one IsoVector 60 degrees clockwise about another
         * Please note that this is an in place operation
         * @param other an IsoVector as center of rotation
         * @returns the rotated IsoVector
         */
        rotateNeg60About(other: _IsoVector): this;
        /**
         * For an equilateral triangle OAB with O at isovector (0, 0) and A at isovector (m, n)
         * Rotates one IsoVector 120 degrees counter clockwise about the center of the triangle
         * Please note that this is an in place operation
         * @param m integer a measure a Primary triangle of order (m, n) m > n
         * @param n >= 0 integer a measure for a Primary triangle of order (m, n)
         * @returns the rotated IsoVector
         */
        rotate120(m: number, n: number): this;
        /**
         * For an equilateral triangle OAB with O at isovector (0, 0) and A at isovector (m, n)
         * Rotates one IsoVector 120 degrees clockwise about the center of the triangle
         * Please note that this is an in place operation
         * @param m integer a measure a Primary triangle of order (m, n) m > n
         * @param n >= 0 integer a measure for a Primary triangle of order (m, n)
         * @returns the rotated IsoVector
         */
        rotateNeg120(m: number, n: number): this;
        /**
         * Transforms an IsoVector to one in Cartesian 3D space based on an isovector
         * @param origin an IsoVector
         * @param isoGridSize
         * @returns Point as a Vector3
         */
        toCartesianOrigin(origin: _IsoVector, isoGridSize: number): Vector3;
        /**
         * Gets a new IsoVector(0, 0)
         * @returns a new IsoVector
         */
        static Zero(): _IsoVector;
    }


    /**
     * @internal
     */
    export interface IColor4Like {
        r: float;
        g: float;
        b: float;
        a: float;
    }
    /**
     * @internal
     */
    export interface IColor3Like {
        r: float;
        g: float;
        b: float;
    }
    export interface IQuaternionLike {
        x: float;
        y: float;
        z: float;
        w: float;
    }
    /**
     * @internal
     */
    export interface IVector4Like {
        x: float;
        y: float;
        z: float;
        w: float;
    }
    /**
     * @internal
     */
    export interface IVector3Like {
        x: float;
        y: float;
        z: float;
    }
    /**
     * @internal
     */
    export interface IVector2Like {
        x: float;
        y: float;
    }
    /**
     * @internal
     */
    export interface IMatrixLike {
        toArray(): DeepImmutable<Float32Array | Array<number>>;
        updateFlag: int;
    }
    /**
     * @internal
     */
    export interface IViewportLike {
        x: float;
        y: float;
        width: float;
        height: float;
    }
    /**
     * @internal
     */
    export interface IPlaneLike {
        normal: IVector3Like;
        d: float;
        normalize(): void;
    }


    /**
     * Defines potential orientation for back face culling
     */
    export enum Orientation {
        /**
         * Clockwise
         */
        CW = 0,
        /** Counter clockwise */
        CCW = 1
    }
    /** Class used to represent a Bezier curve */
    export class BezierCurve {
        /**
         * Returns the cubic Bezier interpolated value (float) at "t" (float) from the given x1, y1, x2, y2 floats
         * @param t defines the time
         * @param x1 defines the left coordinate on X axis
         * @param y1 defines the left coordinate on Y axis
         * @param x2 defines the right coordinate on X axis
         * @param y2 defines the right coordinate on Y axis
         * @returns the interpolated value
         */
        static Interpolate(t: number, x1: number, y1: number, x2: number, y2: number): number;
    }
    /**
     * Defines angle representation
     */
    export class Angle {
        private _radians;
        /**
         * Creates an Angle object of "radians" radians (float).
         * @param radians the angle in radians
         */
        constructor(radians: number);
        /**
         * Get value in degrees
         * @returns the Angle value in degrees (float)
         */
        degrees(): number;
        /**
         * Get value in radians
         * @returns the Angle value in radians (float)
         */
        radians(): number;
        /**
         * Gets a new Angle object valued with the gradient angle, in radians, of the line joining two points
         * @param a defines first point as the origin
         * @param b defines point
         * @returns a new Angle
         */
        static BetweenTwoPoints(a: DeepImmutable<Vector2>, b: DeepImmutable<Vector2>): Angle;
        /**
         * Gets a new Angle object from the given float in radians
         * @param radians defines the angle value in radians
         * @returns a new Angle
         */
        static FromRadians(radians: number): Angle;
        /**
         * Gets a new Angle object from the given float in degrees
         * @param degrees defines the angle value in degrees
         * @returns a new Angle
         */
        static FromDegrees(degrees: number): Angle;
    }
    /**
     * This represents an arc in a 2d space.
     */
    export class Arc2 {
        /** Defines the start point of the arc */
        startPoint: Vector2;
        /** Defines the mid point of the arc */
        midPoint: Vector2;
        /** Defines the end point of the arc */
        endPoint: Vector2;
        /**
         * Defines the center point of the arc.
         */
        centerPoint: Vector2;
        /**
         * Defines the radius of the arc.
         */
        radius: number;
        /**
         * Defines the angle of the arc (from mid point to end point).
         */
        angle: Angle;
        /**
         * Defines the start angle of the arc (from start point to middle point).
         */
        startAngle: Angle;
        /**
         * Defines the orientation of the arc (clock wise/counter clock wise).
         */
        orientation: Orientation;
        /**
         * Creates an Arc object from the three given points : start, middle and end.
         * @param startPoint Defines the start point of the arc
         * @param midPoint Defines the middle point of the arc
         * @param endPoint Defines the end point of the arc
         */
        constructor(
        /** Defines the start point of the arc */
        startPoint: Vector2, 
        /** Defines the mid point of the arc */
        midPoint: Vector2, 
        /** Defines the end point of the arc */
        endPoint: Vector2);
    }
    /**
     * Represents a 2D path made up of multiple 2D points
     */
    export class Path2 {
        private _points;
        private _length;
        /**
         * If the path start and end point are the same
         */
        closed: boolean;
        /**
         * Creates a Path2 object from the starting 2D coordinates x and y.
         * @param x the starting points x value
         * @param y the starting points y value
         */
        constructor(x: number, y: number);
        /**
         * Adds a new segment until the given coordinates (x, y) to the current Path2.
         * @param x the added points x value
         * @param y the added points y value
         * @returns the updated Path2.
         */
        addLineTo(x: number, y: number): Path2;
        /**
         * Adds _numberOfSegments_ segments according to the arc definition (middle point coordinates, end point coordinates, the arc start point being the current Path2 last point) to the current Path2.
         * @param midX middle point x value
         * @param midY middle point y value
         * @param endX end point x value
         * @param endY end point y value
         * @param numberOfSegments (default: 36)
         * @returns the updated Path2.
         */
        addArcTo(midX: number, midY: number, endX: number, endY: number, numberOfSegments?: number): Path2;
        /**
         * Closes the Path2.
         * @returns the Path2.
         */
        close(): Path2;
        /**
         * Gets the sum of the distance between each sequential point in the path
         * @returns the Path2 total length (float).
         */
        length(): number;
        /**
         * Gets the points which construct the path
         * @returns the Path2 internal array of points.
         */
        getPoints(): Vector2[];
        /**
         * Retreives the point at the distance aways from the starting point
         * @param normalizedLengthPosition the length along the path to retrieve the point from
         * @returns a new Vector2 located at a percentage of the Path2 total length on this path.
         */
        getPointAtLengthPosition(normalizedLengthPosition: number): Vector2;
        /**
         * Creates a new path starting from an x and y position
         * @param x starting x value
         * @param y starting y value
         * @returns a new Path2 starting at the coordinates (x, y).
         */
        static StartingAt(x: number, y: number): Path2;
    }
    /**
     * Represents a 3D path made up of multiple 3D points
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/path3D
     */
    export class Path3D {
        /**
         * an array of Vector3, the curve axis of the Path3D
         */
        path: Vector3[];
        private _curve;
        private _distances;
        private _tangents;
        private _normals;
        private _binormals;
        private _raw;
        private _alignTangentsWithPath;
        private readonly _pointAtData;
        /**
         * new Path3D(path, normal, raw)
         * Creates a Path3D. A Path3D is a logical math object, so not a mesh.
         * please read the description in the tutorial : https://doc.babylonjs.com/features/featuresDeepDive/mesh/path3D
         * @param path an array of Vector3, the curve axis of the Path3D
         * @param firstNormal (options) Vector3, the first wanted normal to the curve. Ex (0, 1, 0) for a vertical normal.
         * @param raw (optional, default false) : boolean, if true the returned Path3D isn't normalized. Useful to depict path acceleration or speed.
         * @param alignTangentsWithPath (optional, default false) : boolean, if true the tangents will be aligned with the path.
         */
        constructor(
        /**
         * an array of Vector3, the curve axis of the Path3D
         */
        path: Vector3[], firstNormal?: Nullable<Vector3>, raw?: boolean, alignTangentsWithPath?: boolean);
        /**
         * Returns the Path3D array of successive Vector3 designing its curve.
         * @returns the Path3D array of successive Vector3 designing its curve.
         */
        getCurve(): Vector3[];
        /**
         * Returns the Path3D array of successive Vector3 designing its curve.
         * @returns the Path3D array of successive Vector3 designing its curve.
         */
        getPoints(): Vector3[];
        /**
         * @returns the computed length (float) of the path.
         */
        length(): number;
        /**
         * Returns an array populated with tangent vectors on each Path3D curve point.
         * @returns an array populated with tangent vectors on each Path3D curve point.
         */
        getTangents(): Vector3[];
        /**
         * Returns an array populated with normal vectors on each Path3D curve point.
         * @returns an array populated with normal vectors on each Path3D curve point.
         */
        getNormals(): Vector3[];
        /**
         * Returns an array populated with binormal vectors on each Path3D curve point.
         * @returns an array populated with binormal vectors on each Path3D curve point.
         */
        getBinormals(): Vector3[];
        /**
         * Returns an array populated with distances (float) of the i-th point from the first curve point.
         * @returns an array populated with distances (float) of the i-th point from the first curve point.
         */
        getDistances(): number[];
        /**
         * Returns an interpolated point along this path
         * @param position the position of the point along this path, from 0.0 to 1.0
         * @returns a new Vector3 as the point
         */
        getPointAt(position: number): Vector3;
        /**
         * Returns the tangent vector of an interpolated Path3D curve point at the specified position along this path.
         * @param position the position of the point along this path, from 0.0 to 1.0
         * @param interpolated (optional, default false) : boolean, if true returns an interpolated tangent instead of the tangent of the previous path point.
         * @returns a tangent vector corresponding to the interpolated Path3D curve point, if not interpolated, the tangent is taken from the precomputed tangents array.
         */
        getTangentAt(position: number, interpolated?: boolean): Vector3;
        /**
         * Returns the tangent vector of an interpolated Path3D curve point at the specified position along this path.
         * @param position the position of the point along this path, from 0.0 to 1.0
         * @param interpolated (optional, default false) : boolean, if true returns an interpolated normal instead of the normal of the previous path point.
         * @returns a normal vector corresponding to the interpolated Path3D curve point, if not interpolated, the normal is taken from the precomputed normals array.
         */
        getNormalAt(position: number, interpolated?: boolean): Vector3;
        /**
         * Returns the binormal vector of an interpolated Path3D curve point at the specified position along this path.
         * @param position the position of the point along this path, from 0.0 to 1.0
         * @param interpolated (optional, default false) : boolean, if true returns an interpolated binormal instead of the binormal of the previous path point.
         * @returns a binormal vector corresponding to the interpolated Path3D curve point, if not interpolated, the binormal is taken from the precomputed binormals array.
         */
        getBinormalAt(position: number, interpolated?: boolean): Vector3;
        /**
         * Returns the distance (float) of an interpolated Path3D curve point at the specified position along this path.
         * @param position the position of the point along this path, from 0.0 to 1.0
         * @returns the distance of the interpolated Path3D curve point at the specified position along this path.
         */
        getDistanceAt(position: number): number;
        /**
         * Returns the array index of the previous point of an interpolated point along this path
         * @param position the position of the point to interpolate along this path, from 0.0 to 1.0
         * @returns the array index
         */
        getPreviousPointIndexAt(position: number): number;
        /**
         * Returns the position of an interpolated point relative to the two path points it lies between, from 0.0 (point A) to 1.0 (point B)
         * @param position the position of the point to interpolate along this path, from 0.0 to 1.0
         * @returns the sub position
         */
        getSubPositionAt(position: number): number;
        /**
         * Returns the position of the closest virtual point on this path to an arbitrary Vector3, from 0.0 to 1.0
         * @param target the vector of which to get the closest position to
         * @returns the position of the closest virtual point on this path to the target vector
         */
        getClosestPositionTo(target: Vector3): number;
        /**
         * Returns a sub path (slice) of this path
         * @param start the position of the fist path point, from 0.0 to 1.0, or a negative value, which will get wrapped around from the end of the path to 0.0 to 1.0 values
         * @param end the position of the last path point, from 0.0 to 1.0, or a negative value, which will get wrapped around from the end of the path to 0.0 to 1.0 values
         * @returns a sub path (slice) of this path
         */
        slice(start?: number, end?: number): Path3D;
        /**
         * Forces the Path3D tangent, normal, binormal and distance recomputation.
         * @param path path which all values are copied into the curves points
         * @param firstNormal which should be projected onto the curve
         * @param alignTangentsWithPath (optional, default false) : boolean, if true the tangents will be aligned with the path
         * @returns the same object updated.
         */
        update(path: Vector3[], firstNormal?: Nullable<Vector3>, alignTangentsWithPath?: boolean): Path3D;
        private _compute;
        private _getFirstNonNullVector;
        private _getLastNonNullVector;
        private _normalVector;
        /**
         * Updates the point at data for an interpolated point along this curve
         * @param position the position of the point along this curve, from 0.0 to 1.0
         * @param interpolateTNB
         * @interpolateTNB whether to compute the interpolated tangent, normal and binormal
         * @returns the (updated) point at data
         */
        private _updatePointAtData;
        /**
         * Updates the point at data from the specified parameters
         * @param position where along the path the interpolated point is, from 0.0 to 1.0
         * @param subPosition
         * @param point the interpolated point
         * @param parentIndex the index of an existing curve point that is on, or else positionally the first behind, the interpolated point
         * @param interpolateTNB
         */
        private _setPointAtData;
        /**
         * Updates the point at interpolation matrix for the tangents, normals and binormals
         */
        private _updateInterpolationMatrix;
    }
    /**
     * A Curve3 object is a logical object, so not a mesh, to handle curves in the 3D geometric space.
     * A Curve3 is designed from a series of successive Vector3.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/drawCurves
     */
    export class Curve3 {
        private _points;
        private _length;
        /**
         * Returns a Curve3 object along a Quadratic Bezier curve : https://doc.babylonjs.com/features/featuresDeepDive/mesh/drawCurves#quadratic-bezier-curve
         * @param v0 (Vector3) the origin point of the Quadratic Bezier
         * @param v1 (Vector3) the control point
         * @param v2 (Vector3) the end point of the Quadratic Bezier
         * @param nbPoints (integer) the wanted number of points in the curve
         * @returns the created Curve3
         */
        static CreateQuadraticBezier(v0: DeepImmutable<Vector3>, v1: DeepImmutable<Vector3>, v2: DeepImmutable<Vector3>, nbPoints: number): Curve3;
        /**
         * Returns a Curve3 object along a Cubic Bezier curve : https://doc.babylonjs.com/features/featuresDeepDive/mesh/drawCurves#cubic-bezier-curve
         * @param v0 (Vector3) the origin point of the Cubic Bezier
         * @param v1 (Vector3) the first control point
         * @param v2 (Vector3) the second control point
         * @param v3 (Vector3) the end point of the Cubic Bezier
         * @param nbPoints (integer) the wanted number of points in the curve
         * @returns the created Curve3
         */
        static CreateCubicBezier(v0: DeepImmutable<Vector3>, v1: DeepImmutable<Vector3>, v2: DeepImmutable<Vector3>, v3: DeepImmutable<Vector3>, nbPoints: number): Curve3;
        /**
         * Returns a Curve3 object along a Hermite Spline curve : https://doc.babylonjs.com/features/featuresDeepDive/mesh/drawCurves#hermite-spline
         * @param p1 (Vector3) the origin point of the Hermite Spline
         * @param t1 (Vector3) the tangent vector at the origin point
         * @param p2 (Vector3) the end point of the Hermite Spline
         * @param t2 (Vector3) the tangent vector at the end point
         * @param nSeg (integer) the number of curve segments or nSeg + 1 points in the array
         * @returns the created Curve3
         */
        static CreateHermiteSpline(p1: DeepImmutable<Vector3>, t1: DeepImmutable<Vector3>, p2: DeepImmutable<Vector3>, t2: DeepImmutable<Vector3>, nSeg: number): Curve3;
        /**
         * Returns a Curve3 object along a CatmullRom Spline curve :
         * @param points (array of Vector3) the points the spline must pass through. At least, four points required
         * @param nbPoints (integer) the wanted number of points between each curve control points
         * @param closed (boolean) optional with default false, when true forms a closed loop from the points
         * @returns the created Curve3
         */
        static CreateCatmullRomSpline(points: DeepImmutable<Vector3[]>, nbPoints: number, closed?: boolean): Curve3;
        /**
         * Returns a Curve3 object along an arc through three vector3 points:
         * The three points should not be colinear. When they are the Curve3 is empty.
         * @param first (Vector3) the first point the arc must pass through.
         * @param second (Vector3) the second point the arc must pass through.
         * @param third (Vector3) the third point the arc must pass through.
         * @param steps (number) the larger the number of steps the more detailed the arc.
         * @param closed (boolean) optional with default false, when true forms the chord from the first and third point
         * @param fullCircle Circle (boolean) optional with default false, when true forms the complete circle through the three points
         * @returns the created Curve3
         */
        static ArcThru3Points(first: Vector3, second: Vector3, third: Vector3, steps?: number, closed?: boolean, fullCircle?: boolean): Curve3;
        /**
         * A Curve3 object is a logical object, so not a mesh, to handle curves in the 3D geometric space.
         * A Curve3 is designed from a series of successive Vector3.
         * Tuto : https://doc.babylonjs.com/features/featuresDeepDive/mesh/drawCurves#curve3-object
         * @param points points which make up the curve
         */
        constructor(points: Vector3[]);
        /**
         * @returns the Curve3 stored array of successive Vector3
         */
        getPoints(): Vector3[];
        /**
         * @returns the computed length (float) of the curve.
         */
        length(): number;
        /**
         * Returns a new instance of Curve3 object : var curve = curveA.continue(curveB);
         * This new Curve3 is built by translating and sticking the curveB at the end of the curveA.
         * curveA and curveB keep unchanged.
         * @param curve the curve to continue from this curve
         * @returns the newly constructed curve
         */
        continue(curve: DeepImmutable<Curve3>): Curve3;
        private _computeLength;
    }


    /**
     * Represents a plane by the equation ax + by + cz + d = 0
     */
    export class Plane {
        private static _TmpMatrix;
        /**
         * Normal of the plane (a,b,c)
         */
        normal: Vector3;
        /**
         * d component of the plane
         */
        d: number;
        /**
         * Creates a Plane object according to the given floats a, b, c, d and the plane equation : ax + by + cz + d = 0
         * @param a a component of the plane
         * @param b b component of the plane
         * @param c c component of the plane
         * @param d d component of the plane
         */
        constructor(a: number, b: number, c: number, d: number);
        /**
         * @returns the plane coordinates as a new array of 4 elements [a, b, c, d].
         */
        asArray(): number[];
        /**
         * @returns a new plane copied from the current Plane.
         */
        clone(): Plane;
        /**
         * @returns the string "Plane".
         */
        getClassName(): string;
        /**
         * @returns the Plane hash code.
         */
        getHashCode(): number;
        /**
         * Normalize the current Plane in place.
         * @returns the updated Plane.
         */
        normalize(): Plane;
        /**
         * Applies a transformation the plane and returns the result
         * @param transformation the transformation matrix to be applied to the plane
         * @returns a new Plane as the result of the transformation of the current Plane by the given matrix.
         */
        transform(transformation: DeepImmutable<Matrix>): Plane;
        /**
         * Compute the dot product between the point and the plane normal
         * @param point point to calculate the dot product with
         * @returns the dot product (float) of the point coordinates and the plane normal.
         */
        dotCoordinate(point: DeepImmutable<Vector3>): number;
        /**
         * Updates the current Plane from the plane defined by the three given points.
         * @param point1 one of the points used to construct the plane
         * @param point2 one of the points used to construct the plane
         * @param point3 one of the points used to construct the plane
         * @returns the updated Plane.
         */
        copyFromPoints(point1: DeepImmutable<Vector3>, point2: DeepImmutable<Vector3>, point3: DeepImmutable<Vector3>): Plane;
        /**
         * Checks if the plane is facing a given direction (meaning if the plane's normal is pointing in the opposite direction of the given vector).
         * Note that for this function to work as expected you should make sure that:
         *   - direction and the plane normal are normalized
         *   - epsilon is a number just bigger than -1, something like -0.99 for eg
         * @param direction the direction to check if the plane is facing
         * @param epsilon value the dot product is compared against (returns true if dot <= epsilon)
         * @returns True if the plane is facing the given direction
         */
        isFrontFacingTo(direction: DeepImmutable<Vector3>, epsilon: number): boolean;
        /**
         * Calculates the distance to a point
         * @param point point to calculate distance to
         * @returns the signed distance (float) from the given point to the Plane.
         */
        signedDistanceTo(point: DeepImmutable<Vector3>): number;
        /**
         * Creates a plane from an  array
         * @param array the array to create a plane from
         * @returns a new Plane from the given array.
         */
        static FromArray(array: DeepImmutable<ArrayLike<number>>): Plane;
        /**
         * Creates a plane from three points
         * @param point1 point used to create the plane
         * @param point2 point used to create the plane
         * @param point3 point used to create the plane
         * @returns a new Plane defined by the three given points.
         */
        static FromPoints(point1: DeepImmutable<Vector3>, point2: DeepImmutable<Vector3>, point3: DeepImmutable<Vector3>): Plane;
        /**
         * Creates a plane from an origin point and a normal
         * @param origin origin of the plane to be constructed
         * @param normal normal of the plane to be constructed
         * @returns a new Plane the normal vector to this plane at the given origin point.
         * Note : the vector "normal" is updated because normalized.
         */
        static FromPositionAndNormal(origin: DeepImmutable<Vector3>, normal: Vector3): Plane;
        /**
         * Calculates the distance from a plane and a point
         * @param origin origin of the plane to be constructed
         * @param normal normal of the plane to be constructed
         * @param point point to calculate distance to
         * @returns the signed distance between the plane defined by the normal vector at the "origin"" point and the given other point.
         */
        static SignedDistanceToPlaneFromPositionAndNormal(origin: DeepImmutable<Vector3>, normal: DeepImmutable<Vector3>, point: DeepImmutable<Vector3>): number;
    }


    /**
     * Class used to store (r, theta) vector representation
     */
    export class Polar {
        radius: number;
        theta: number;
        /**
         * Creates a new Polar object
         * @param radius the radius of the vector
         * @param theta the angle of the vector
         */
        constructor(radius: number, theta: number);
        /**
         * Gets the class name
         * @returns the string "Polar"
         */
        getClassName(): string;
        /**
         * Converts the current polar to a string
         * @returns the current polar as a string
         */
        toString(): string;
        /**
         * Converts the current polar to an array
         * @reutrns the current polar as an array
         */
        asArray(): number[];
        /**
         * Adds the current Polar and the given Polar and stores the result
         * @param polar the polar to add
         * @param ref the polar to store the result in
         * @returns the updated ref
         */
        addToRef(polar: Polar, ref: Polar): Polar;
        /**
         * Adds the current Polar and the given Polar
         * @param polar the polar to add
         * @returns the sum polar
         */
        add(polar: Polar): Polar;
        /**
         * Adds the given polar to the current polar
         * @param polar the polar to add
         * @returns the current polar
         */
        addInPlace(polar: Polar): this;
        /**
         * Adds the provided values to the current polar
         * @param radius the amount to add to the radius
         * @param theta the amount to add to the theta
         * @returns the current polar
         */
        addInPlaceFromFloats(radius: number, theta: number): this;
        /**
         * Subtracts the given Polar from the current Polar and stores the result
         * @param polar the polar to subtract
         * @param ref the polar to store the result in
         * @returns the updated ref
         */
        subtractToRef(polar: Polar, ref: Polar): Polar;
        /**
         * Subtracts the given Polar from the current Polar
         * @param polar the polar to subtract
         * @returns the difference polar
         */
        subtract(polar: Polar): Polar;
        /**
         * Subtracts the given Polar from the current Polar
         * @param polar the polar to subtract
         * @returns the current polar
         */
        subtractInPlace(polar: Polar): this;
        /**
         * Subtracts the given floats from the current polar
         * @param radius the amount to subtract from the radius
         * @param theta the amount to subtract from the theta
         * @param ref the polar to store the result in
         * @returns the updated ref
         */
        subtractFromFloatsToRef(radius: number, theta: number, ref: Polar): Polar;
        /**
         * Subtracts the given floats from the current polar
         * @param radius the amount to subtract from the radius
         * @param theta the amount to subtract from the theta
         * @returns the difference polar
         */
        subtractFromFloats(radius: number, theta: number): Polar;
        /**
         * Multiplies the given Polar with the current Polar and stores the result
         * @param polar the polar to multiply
         * @param ref the polar to store the result in
         * @returns the updated ref
         */
        multiplyToRef(polar: Polar, ref: Polar): Polar;
        /**
         * Multiplies the given Polar with the current Polar
         * @param polar the polar to multiply
         * @returns the product polar
         */
        multiply(polar: Polar): Polar;
        /**
         * Multiplies the given Polar with the current Polar
         * @param polar the polar to multiply
         * @returns the current polar
         */
        multiplyInPlace(polar: Polar): this;
        /**
         * Divides the current Polar by the given Polar and stores the result
         * @param polar the polar to divide
         * @param ref the polar to store the result in
         * @returns the updated ref
         */
        divideToRef(polar: Polar, ref: Polar): Polar;
        /**
         * Divides the current Polar by the given Polar
         * @param polar the polar to divide
         * @returns the quotient polar
         */
        divide(polar: Polar): Polar;
        /**
         * Divides the current Polar by the given Polar
         * @param polar the polar to divide
         * @returns the current polar
         */
        divideInPlace(polar: Polar): this;
        /**
         * Clones the current polar
         * @returns a clone of the current polar
         */
        clone(): Polar;
        /**
         * Copies the source polar into the current polar
         * @param source the polar to copy from
         * @returns the current polar
         */
        copyFrom(source: Polar): this;
        /**
         * Copies the given values into the current polar
         * @param radius the radius to use
         * @param theta the theta to use
         * @returns the current polar
         */
        copyFromFloats(radius: number, theta: number): this;
        /**
         * Scales the current polar and stores the result
         * @param scale defines the multiplication factor
         * @param ref where to store the result
         * @returns the updated ref
         */
        scaleToRef(scale: number, ref: Polar): Polar;
        /**
         * Scales the current polar and returns a new polar with the scaled coordinates
         * @param scale defines the multiplication factor
         * @returns the scaled polar
         */
        scale(scale: number): Polar;
        /**
         * Scales the current polar
         * @param scale defines the multiplication factor
         * @returns the current polar
         */
        scaleInPlace(scale: number): this;
        /**
         * Sets the values of the current polar
         * @param radius the new radius
         * @param theta the new theta
         * @returns the current polar
         */
        set(radius: number, theta: number): this;
        /**
         * Sets the values of the current polar
         * @param value the new values
         * @returns the current polar
         */
        setAll(value: number): this;
        /**
         * Gets the rectangular coordinates of the current Polar
         * @param ref the reference to assign the result
         * @returns the updated reference
         */
        toVector2ToRef(ref: Vector2): Vector2;
        /**
         * Gets the rectangular coordinates of the current Polar
         * @returns the rectangular coordinates
         */
        toVector2(): Vector2;
        /**
         * Converts a given Vector2 to its polar coordinates
         * @param v the Vector2 to convert
         * @param ref the reference to assign the result
         * @returns the updated reference
         */
        static FromVector2ToRef(v: Vector2, ref: Polar): Polar;
        /**
         * Converts a given Vector2 to its polar coordinates
         * @param v the Vector2 to convert
         * @returns a Polar
         */
        static FromVector2(v: Vector2): Polar;
        /**
         * Converts an array of floats to a polar
         * @param array the array to convert
         * @returns the converted polar
         */
        static FromArray(array: number[]): Polar;
    }
    /**
     * Class used for (radius, theta, phi) vector representation.
     */
    export class Spherical {
        radius: number;
        theta: number;
        phi: number;
        /**
         * @param radius spherical radius
         * @param theta angle from positive y axis to radial line from 0 to PI (vertical)
         * @param phi angle from positive x axis measured anticlockwise from -PI to PI (horizontal)
         */
        constructor(radius: number, theta: number, phi: number);
        /**
         * Gets the class name
         * @returns the string "Spherical"
         */
        getClassName(): string;
        /**
         * Converts the current spherical to a string
         * @returns the current spherical as a string
         */
        toString(): string;
        /**
         * Converts the current spherical to an array
         * @reutrns the current spherical as an array
         */
        asArray(): number[];
        /**
         * Adds the current Spherical and the given Spherical and stores the result
         * @param spherical the spherical to add
         * @param ref the spherical to store the result in
         * @returns the updated ref
         */
        addToRef(spherical: Spherical, ref: Spherical): Spherical;
        /**
         * Adds the current Spherical and the given Spherical
         * @param spherical the spherical to add
         * @returns the sum spherical
         */
        add(spherical: Spherical): Spherical;
        /**
         * Adds the given spherical to the current spherical
         * @param spherical the spherical to add
         * @returns the current spherical
         */
        addInPlace(spherical: Spherical): this;
        /**
         * Adds the provided values to the current spherical
         * @param radius the amount to add to the radius
         * @param theta the amount to add to the theta
         * @param phi the amount to add to the phi
         * @returns the current spherical
         */
        addInPlaceFromFloats(radius: number, theta: number, phi: number): this;
        /**
         * Subtracts the given Spherical from the current Spherical and stores the result
         * @param spherical the spherical to subtract
         * @param ref the spherical to store the result in
         * @returns the updated ref
         */
        subtractToRef(spherical: Spherical, ref: Spherical): Spherical;
        /**
         * Subtracts the given Spherical from the current Spherical
         * @param spherical the spherical to subtract
         * @returns the difference spherical
         */
        subtract(spherical: Spherical): Spherical;
        /**
         * Subtracts the given Spherical from the current Spherical
         * @param spherical the spherical to subtract
         * @returns the current spherical
         */
        subtractInPlace(spherical: Spherical): this;
        /**
         * Subtracts the given floats from the current spherical
         * @param radius the amount to subtract from the radius
         * @param theta the amount to subtract from the theta
         * @param phi the amount to subtract from the phi
         * @param ref the spherical to store the result in
         * @returns the updated ref
         */
        subtractFromFloatsToRef(radius: number, theta: number, phi: number, ref: Spherical): Spherical;
        /**
         * Subtracts the given floats from the current spherical
         * @param radius the amount to subtract from the radius
         * @param theta the amount to subtract from the theta
         * @param phi the amount to subtract from the phi
         * @returns the difference spherical
         */
        subtractFromFloats(radius: number, theta: number, phi: number): Spherical;
        /**
         * Multiplies the given Spherical with the current Spherical and stores the result
         * @param spherical the spherical to multiply
         * @param ref the spherical to store the result in
         * @returns the updated ref
         */
        multiplyToRef(spherical: Spherical, ref: Spherical): Spherical;
        /**
         * Multiplies the given Spherical with the current Spherical
         * @param spherical the spherical to multiply
         * @returns the product spherical
         */
        multiply(spherical: Spherical): Spherical;
        /**
         * Multiplies the given Spherical with the current Spherical
         * @param spherical the spherical to multiply
         * @returns the current spherical
         */
        multiplyInPlace(spherical: Spherical): this;
        /**
         * Divides the current Spherical by the given Spherical and stores the result
         * @param spherical the spherical to divide
         * @param ref the spherical to store the result in
         * @returns the updated ref
         */
        divideToRef(spherical: Spherical, ref: Spherical): Spherical;
        /**
         * Divides the current Spherical by the given Spherical
         * @param spherical the spherical to divide
         * @returns the quotient spherical
         */
        divide(spherical: Spherical): Spherical;
        /**
         * Divides the current Spherical by the given Spherical
         * @param spherical the spherical to divide
         * @returns the current spherical
         */
        divideInPlace(spherical: Spherical): this;
        /**
         * Clones the current spherical
         * @returns a clone of the current spherical
         */
        clone(): Spherical;
        /**
         * Copies the source spherical into the current spherical
         * @param source the spherical to copy from
         * @returns the current spherical
         */
        copyFrom(source: Spherical): this;
        /**
         * Copies the given values into the current spherical
         * @param radius the radius to use
         * @param theta the theta to use
         * @param phi the phi to use
         * @returns the current spherical
         */
        copyFromFloats(radius: number, theta: number, phi: number): this;
        /**
         * Scales the current spherical and stores the result
         * @param scale defines the multiplication factor
         * @param ref where to store the result
         * @returns the updated ref
         */
        scaleToRef(scale: number, ref: Spherical): Spherical;
        /**
         * Scales the current spherical and returns a new spherical with the scaled coordinates
         * @param scale defines the multiplication factor
         * @returns the scaled spherical
         */
        scale(scale: number): Spherical;
        /**
         * Scales the current spherical
         * @param scale defines the multiplication factor
         * @returns the current spherical
         */
        scaleInPlace(scale: number): this;
        /**
         * Sets the values of the current spherical
         * @param radius the new radius
         * @param theta the new theta
         * @param phi the new phi
         * @returns the current spherical
         */
        set(radius: number, theta: number, phi: number): this;
        /**
         * Sets the values of the current spherical
         * @param value the new values
         * @returns the current spherical
         */
        setAll(value: number): this;
        /**
         * Assigns the rectangular coordinates of the current Spherical to a Vector3
         * @param ref the Vector3 to update
         * @returns the updated Vector3
         */
        toVector3ToRef(ref: DeepImmutable<Vector3>): Vector3;
        /**
         * Gets a Vector3 from the current spherical coordinates
         * @returns the (x, y,z) form of the current Spherical
         */
        toVector3(): Vector3;
        /**
         * Assigns the spherical coordinates from a Vector3
         * @param vector the vector to convert
         * @param ref the Spherical to update
         * @returns the updated ref
         */
        static FromVector3ToRef(vector: DeepImmutable<Vector3>, ref: Spherical): Spherical;
        /**
         * Gets a Spherical from a Vector3
         * @param vector defines the vector in (x, y, z) coordinate space
         * @returns a new Spherical
         */
        static FromVector3(vector: DeepImmutable<Vector3>): Spherical;
        /**
         * Converts an array of floats to a spherical
         * @param array the array to convert
         * @returns the converted spherical
         */
        static FromArray(array: number[]): Spherical;
    }


    /**
     * Scalar computation library
     */
    export class Scalar {
        /**
         * Two pi constants convenient for computation.
         */
        static TwoPi: number;
        /**
         * Boolean : true if the absolute difference between a and b is lower than epsilon (default = 1.401298E-45)
         * @param a number
         * @param b number
         * @param epsilon (default = 1.401298E-45)
         * @returns true if the absolute difference between a and b is lower than epsilon (default = 1.401298E-45)
         */
        static WithinEpsilon(a: number, b: number, epsilon?: number): boolean;
        /**
         * Returns a string : the upper case translation of the number i to hexadecimal.
         * @param i number
         * @returns the upper case translation of the number i to hexadecimal.
         */
        static ToHex(i: number): string;
        /**
         * Returns -1 if value is negative and +1 is value is positive.
         * @param value the value
         * @returns the value itself if it's equal to zero.
         */
        static Sign(value: number): number;
        /**
         * Returns the value itself if it's between min and max.
         * Returns min if the value is lower than min.
         * Returns max if the value is greater than max.
         * @param value the value to clmap
         * @param min the min value to clamp to (default: 0)
         * @param max the max value to clamp to (default: 1)
         * @returns the clamped value
         */
        static Clamp(value: number, min?: number, max?: number): number;
        /**
         * the log2 of value.
         * @param value the value to compute log2 of
         * @returns the log2 of value.
         */
        static Log2(value: number): number;
        /**
         * the floor part of a log2 value.
         * @param value the value to compute log2 of
         * @returns the log2 of value.
         */
        static ILog2(value: number): number;
        /**
         * Loops the value, so that it is never larger than length and never smaller than 0.
         *
         * This is similar to the modulo operator but it works with floating point numbers.
         * For example, using 3.0 for t and 2.5 for length, the result would be 0.5.
         * With t = 5 and length = 2.5, the result would be 0.0.
         * Note, however, that the behaviour is not defined for negative numbers as it is for the modulo operator
         * @param value the value
         * @param length the length
         * @returns the looped value
         */
        static Repeat(value: number, length: number): number;
        /**
         * Normalize the value between 0.0 and 1.0 using min and max values
         * @param value value to normalize
         * @param min max to normalize between
         * @param max min to normalize between
         * @returns the normalized value
         */
        static Normalize(value: number, min: number, max: number): number;
        /**
         * Denormalize the value from 0.0 and 1.0 using min and max values
         * @param normalized value to denormalize
         * @param min max to denormalize between
         * @param max min to denormalize between
         * @returns the denormalized value
         */
        static Denormalize(normalized: number, min: number, max: number): number;
        /**
         * Calculates the shortest difference between two given angles given in degrees.
         * @param current current angle in degrees
         * @param target target angle in degrees
         * @returns the delta
         */
        static DeltaAngle(current: number, target: number): number;
        /**
         * PingPongs the value t, so that it is never larger than length and never smaller than 0.
         * @param tx value
         * @param length length
         * @returns The returned value will move back and forth between 0 and length
         */
        static PingPong(tx: number, length: number): number;
        /**
         * Interpolates between min and max with smoothing at the limits.
         *
         * This function interpolates between min and max in a similar way to Lerp. However, the interpolation will gradually speed up
         * from the start and slow down toward the end. This is useful for creating natural-looking animation, fading and other transitions.
         * @param from from
         * @param to to
         * @param tx value
         * @returns the smooth stepped value
         */
        static SmoothStep(from: number, to: number, tx: number): number;
        /**
         * Moves a value current towards target.
         *
         * This is essentially the same as Mathf.Lerp but instead the function will ensure that the speed never exceeds maxDelta.
         * Negative values of maxDelta pushes the value away from target.
         * @param current current value
         * @param target target value
         * @param maxDelta max distance to move
         * @returns resulting value
         */
        static MoveTowards(current: number, target: number, maxDelta: number): number;
        /**
         * Same as MoveTowards but makes sure the values interpolate correctly when they wrap around 360 degrees.
         *
         * Variables current and target are assumed to be in degrees. For optimization reasons, negative values of maxDelta
         *  are not supported and may cause oscillation. To push current away from a target angle, add 180 to that angle instead.
         * @param current current value
         * @param target target value
         * @param maxDelta max distance to move
         * @returns resulting angle
         */
        static MoveTowardsAngle(current: number, target: number, maxDelta: number): number;
        /**
         * Creates a new scalar with values linearly interpolated of "amount" between the start scalar and the end scalar.
         * @param start start value
         * @param end target value
         * @param amount amount to lerp between
         * @returns the lerped value
         */
        static Lerp(start: number, end: number, amount: number): number;
        /**
         * Same as Lerp but makes sure the values interpolate correctly when they wrap around 360 degrees.
         * The parameter t is clamped to the range [0, 1]. Variables a and b are assumed to be in degrees.
         * @param start start value
         * @param end target value
         * @param amount amount to lerp between
         * @returns the lerped value
         */
        static LerpAngle(start: number, end: number, amount: number): number;
        /**
         * Calculates the linear parameter t that produces the interpolant value within the range [a, b].
         * @param a start value
         * @param b target value
         * @param value value between a and b
         * @returns the inverseLerp value
         */
        static InverseLerp(a: number, b: number, value: number): number;
        /**
         * Returns a new scalar located for "amount" (float) on the Hermite spline defined by the scalars "value1", "value3", "tangent1", "tangent2".
         * @see http://mathworld.wolfram.com/HermitePolynomial.html
         * @param value1 defines the first control point
         * @param tangent1 defines the first tangent
         * @param value2 defines the second control point
         * @param tangent2 defines the second tangent
         * @param amount defines the amount on the interpolation spline (between 0 and 1)
         * @returns hermite result
         */
        static Hermite(value1: number, tangent1: number, value2: number, tangent2: number, amount: number): number;
        /**
         * Returns a new scalar which is the 1st derivative of the Hermite spline defined by the scalars "value1", "value2", "tangent1", "tangent2".
         * @param value1 defines the first control point
         * @param tangent1 defines the first tangent
         * @param value2 defines the second control point
         * @param tangent2 defines the second tangent
         * @param time define where the derivative must be done
         * @returns 1st derivative
         */
        static Hermite1stDerivative(value1: number, tangent1: number, value2: number, tangent2: number, time: number): number;
        /**
         * Returns a random float number between and min and max values
         * @param min min value of random
         * @param max max value of random
         * @returns random value
         */
        static RandomRange(min: number, max: number): number;
        /**
         * This function returns percentage of a number in a given range.
         *
         * RangeToPercent(40,20,60) will return 0.5 (50%)
         * RangeToPercent(34,0,100) will return 0.34 (34%)
         * @param number to convert to percentage
         * @param min min range
         * @param max max range
         * @returns the percentage
         */
        static RangeToPercent(number: number, min: number, max: number): number;
        /**
         * This function returns number that corresponds to the percentage in a given range.
         *
         * PercentToRange(0.34,0,100) will return 34.
         * @param percent to convert to number
         * @param min min range
         * @param max max range
         * @returns the number
         */
        static PercentToRange(percent: number, min: number, max: number): number;
        /**
         * Returns the angle converted to equivalent value between -Math.PI and Math.PI radians.
         * @param angle The angle to normalize in radian.
         * @returns The converted angle.
         */
        static NormalizeRadians(angle: number): number;
        /**
         * Returns the highest common factor of two integers.
         * @param a first parameter
         * @param b second parameter
         * @returns HCF of a and b
         */
        static HCF(a: number, b: number): number;
    }


    /**
     * Interface for the size containing width and height
     */
    export interface ISize {
        /**
         * Width
         */
        width: number;
        /**
         * Height
         */
        height: number;
    }
    /**
     * Size containing width and height
     */
    export class Size implements ISize {
        /**
         * Width
         */
        width: number;
        /**
         * Height
         */
        height: number;
        /**
         * Creates a Size object from the given width and height (floats).
         * @param width width of the new size
         * @param height height of the new size
         */
        constructor(width: number, height: number);
        /**
         * Returns a string with the Size width and height
         * @returns a string with the Size width and height
         */
        toString(): string;
        /**
         * "Size"
         * @returns the string "Size"
         */
        getClassName(): string;
        /**
         * Returns the Size hash code.
         * @returns a hash code for a unique width and height
         */
        getHashCode(): number;
        /**
         * Updates the current size from the given one.
         * @param src the given size
         */
        copyFrom(src: Size): void;
        /**
         * Updates in place the current Size from the given floats.
         * @param width width of the new size
         * @param height height of the new size
         * @returns the updated Size.
         */
        copyFromFloats(width: number, height: number): Size;
        /**
         * Updates in place the current Size from the given floats.
         * @param width width to set
         * @param height height to set
         * @returns the updated Size.
         */
        set(width: number, height: number): Size;
        /**
         * Multiplies the width and height by numbers
         * @param w factor to multiple the width by
         * @param h factor to multiple the height by
         * @returns a new Size set with the multiplication result of the current Size and the given floats.
         */
        multiplyByFloats(w: number, h: number): Size;
        /**
         * Clones the size
         * @returns a new Size copied from the given one.
         */
        clone(): Size;
        /**
         * True if the current Size and the given one width and height are strictly equal.
         * @param other the other size to compare against
         * @returns True if the current Size and the given one width and height are strictly equal.
         */
        equals(other: Size): boolean;
        /**
         * The surface of the Size : width * height (float).
         */
        get surface(): number;
        /**
         * Create a new size of zero
         * @returns a new Size set to (0.0, 0.0)
         */
        static Zero(): Size;
        /**
         * Sums the width and height of two sizes
         * @param otherSize size to add to this size
         * @returns a new Size set as the addition result of the current Size and the given one.
         */
        add(otherSize: Size): Size;
        /**
         * Subtracts the width and height of two
         * @param otherSize size to subtract to this size
         * @returns a new Size set as the subtraction result of  the given one from the current Size.
         */
        subtract(otherSize: Size): Size;
        /**
         * Creates a new Size set at the linear interpolation "amount" between "start" and "end"
         * @param start starting size to lerp between
         * @param end end size to lerp between
         * @param amount amount to lerp between the start and end values
         * @returns a new Size set at the linear interpolation "amount" between "start" and "end"
         */
        static Lerp(start: Size, end: Size, amount: number): Size;
    }


    export type Vector2Constructor<T extends Vector2> = new (...args: ConstructorParameters<typeof Vector2>) => T;
    export type Vector3Constructor<T extends Vector3> = new (...args: ConstructorParameters<typeof Vector3>) => T;
    export type Vector4Constructor<T extends Vector4> = new (...args: ConstructorParameters<typeof Vector4>) => T;
    export type QuaternionConstructor<T extends Quaternion> = new (...args: ConstructorParameters<typeof Quaternion>) => T;
    export type MatrixConstructor<T extends Matrix> = new () => T;
    /**
     * Class representing a vector containing 2 coordinates
     * Example Playground - Overview -  https://playground.babylonjs.com/#QYBWV4#9
     */
    export class Vector2 {
        /** defines the first coordinate */
        x: number;
        /** defines the second coordinate */
        y: number;
        private static _ZeroReadOnly;
        /**
         * Creates a new Vector2 from the given x and y coordinates
         * @param x defines the first coordinate
         * @param y defines the second coordinate
         */
        constructor(
        /** defines the first coordinate */
        x?: number, 
        /** defines the second coordinate */
        y?: number);
        /**
         * Gets a string with the Vector2 coordinates
         * @returns a string with the Vector2 coordinates
         */
        toString(): string;
        /**
         * Gets class name
         * @returns the string "Vector2"
         */
        getClassName(): string;
        /**
         * Gets current vector hash code
         * @returns the Vector2 hash code as a number
         */
        getHashCode(): number;
        /**
         * Sets the Vector2 coordinates in the given array or Float32Array from the given index.
         * Example Playground https://playground.babylonjs.com/#QYBWV4#15
         * @param array defines the source array
         * @param index defines the offset in source array
         * @returns the current Vector2
         */
        toArray(array: FloatArray, index?: number): this;
        /**
         * Update the current vector from an array
         * Example Playground https://playground.babylonjs.com/#QYBWV4#39
         * @param array defines the destination array
         * @param index defines the offset in the destination array
         * @returns the current Vector2
         */
        fromArray(array: FloatArray, index?: number): this;
        /**
         * Copy the current vector to an array
         * Example Playground https://playground.babylonjs.com/#QYBWV4#40
         * @returns a new array with 2 elements: the Vector2 coordinates.
         */
        asArray(): number[];
        /**
         * Sets the Vector2 coordinates with the given Vector2 coordinates
         * Example Playground https://playground.babylonjs.com/#QYBWV4#24
         * @param source defines the source Vector2
         * @returns the current updated Vector2
         */
        copyFrom(source: DeepImmutable<Vector2>): this;
        /**
         * Sets the Vector2 coordinates with the given floats
         * Example Playground https://playground.babylonjs.com/#QYBWV4#25
         * @param x defines the first coordinate
         * @param y defines the second coordinate
         * @returns the current updated Vector2
         */
        copyFromFloats(x: number, y: number): this;
        /**
         * Sets the Vector2 coordinates with the given floats
         * Example Playground https://playground.babylonjs.com/#QYBWV4#62
         * @param x defines the first coordinate
         * @param y defines the second coordinate
         * @returns the current updated Vector2
         */
        set(x: number, y: number): this;
        /**
         * Add another vector with the current one
         * Example Playground https://playground.babylonjs.com/#QYBWV4#11
         * @param otherVector defines the other vector
         * @returns a new Vector2 set with the addition of the current Vector2 and the given one coordinates
         */
        add(otherVector: DeepImmutable<Vector2>): this;
        /**
         * Sets the "result" coordinates with the addition of the current Vector2 and the given one coordinates
         * Example Playground https://playground.babylonjs.com/#QYBWV4#12
         * @param otherVector defines the other vector
         * @param result defines the target vector
         * @returns result input
         */
        addToRef<T extends Vector2>(otherVector: DeepImmutable<Vector2>, result: T): T;
        /**
         * Set the Vector2 coordinates by adding the given Vector2 coordinates
         * Example Playground https://playground.babylonjs.com/#QYBWV4#13
         * @param otherVector defines the other vector
         * @returns the current updated Vector2
         */
        addInPlace(otherVector: DeepImmutable<Vector2>): this;
        /**
         * Gets a new Vector2 by adding the current Vector2 coordinates to the given Vector3 x, y coordinates
         * Example Playground https://playground.babylonjs.com/#QYBWV4#14
         * @param otherVector defines the other vector
         * @returns a new Vector2
         */
        addVector3(otherVector: Vector3): this;
        /**
         * Gets a new Vector2 set with the subtracted coordinates of the given one from the current Vector2
         * Example Playground https://playground.babylonjs.com/#QYBWV4#61
         * @param otherVector defines the other vector
         * @returns a new Vector2
         */
        subtract(otherVector: Vector2): this;
        /**
         * Sets the "result" coordinates with the subtraction of the given one from the current Vector2 coordinates.
         * Example Playground https://playground.babylonjs.com/#QYBWV4#63
         * @param otherVector defines the other vector
         * @param result defines the target vector
         * @returns result input
         */
        subtractToRef<T extends Vector2>(otherVector: DeepImmutable<Vector2>, result: T): T;
        /**
         * Sets the current Vector2 coordinates by subtracting from it the given one coordinates
         * Example Playground https://playground.babylonjs.com/#QYBWV4#88
         * @param otherVector defines the other vector
         * @returns the current updated Vector2
         */
        subtractInPlace(otherVector: DeepImmutable<Vector2>): this;
        /**
         * Multiplies in place the current Vector2 coordinates by the given ones
         * Example Playground https://playground.babylonjs.com/#QYBWV4#43
         * @param otherVector defines the other vector
         * @returns the current updated Vector2
         */
        multiplyInPlace(otherVector: DeepImmutable<Vector2>): this;
        /**
         * Returns a new Vector2 set with the multiplication of the current Vector2 and the given one coordinates
         * Example Playground https://playground.babylonjs.com/#QYBWV4#42
         * @param otherVector defines the other vector
         * @returns a new Vector2
         */
        multiply(otherVector: DeepImmutable<Vector2>): this;
        /**
         * Sets "result" coordinates with the multiplication of the current Vector2 and the given one coordinates
         * Example Playground https://playground.babylonjs.com/#QYBWV4#44
         * @param otherVector defines the other vector
         * @param result defines the target vector
         * @returns result input
         */
        multiplyToRef<T extends Vector2>(otherVector: DeepImmutable<Vector2>, result: T): T;
        /**
         * Gets a new Vector2 set with the Vector2 coordinates multiplied by the given floats
         * Example Playground https://playground.babylonjs.com/#QYBWV4#89
         * @param x defines the first coordinate
         * @param y defines the second coordinate
         * @returns a new Vector2
         */
        multiplyByFloats(x: number, y: number): this;
        /**
         * Returns a new Vector2 set with the Vector2 coordinates divided by the given one coordinates
         * Example Playground https://playground.babylonjs.com/#QYBWV4#27
         * @param otherVector defines the other vector
         * @returns a new Vector2
         */
        divide(otherVector: Vector2): this;
        /**
         * Sets the "result" coordinates with the Vector2 divided by the given one coordinates
         * Example Playground https://playground.babylonjs.com/#QYBWV4#30
         * @param otherVector defines the other vector
         * @param result defines the target vector
         * @returns result input
         */
        divideToRef<T extends Vector2>(otherVector: DeepImmutable<Vector2>, result: T): T;
        /**
         * Divides the current Vector2 coordinates by the given ones
         * Example Playground https://playground.babylonjs.com/#QYBWV4#28
         * @param otherVector defines the other vector
         * @returns the current updated Vector2
         */
        divideInPlace(otherVector: DeepImmutable<Vector2>): this;
        /**
         * Gets a new Vector2 with current Vector2 negated coordinates
         * Example Playground https://playground.babylonjs.com/#QYBWV4#22
         * @returns a new Vector2
         */
        negate(): this;
        /**
         * Negate this vector in place
         * Example Playground https://playground.babylonjs.com/#QYBWV4#23
         * @returns this
         */
        negateInPlace(): this;
        /**
         * Negate the current Vector2 and stores the result in the given vector "result" coordinates
         * Example Playground https://playground.babylonjs.com/#QYBWV4#41
         * @param result defines the Vector3 object where to store the result
         * @returns the result
         */
        negateToRef<T extends Vector2>(result: T): T;
        /**
         * Multiply the Vector2 coordinates by
         * Example Playground https://playground.babylonjs.com/#QYBWV4#59
         * @param scale defines the scaling factor
         * @returns the current updated Vector2
         */
        scaleInPlace(scale: number): this;
        /**
         * Returns a new Vector2 scaled by "scale" from the current Vector2
         * Example Playground https://playground.babylonjs.com/#QYBWV4#52
         * @param scale defines the scaling factor
         * @returns a new Vector2
         */
        scale(scale: number): this;
        /**
         * Scale the current Vector2 values by a factor to a given Vector2
         * Example Playground https://playground.babylonjs.com/#QYBWV4#57
         * @param scale defines the scale factor
         * @param result defines the Vector2 object where to store the result
         * @returns result input
         */
        scaleToRef<T extends Vector2>(scale: number, result: T): T;
        /**
         * Scale the current Vector2 values by a factor and add the result to a given Vector2
         * Example Playground https://playground.babylonjs.com/#QYBWV4#58
         * @param scale defines the scale factor
         * @param result defines the Vector2 object where to store the result
         * @returns result input
         */
        scaleAndAddToRef<T extends Vector2>(scale: number, result: T): T;
        /**
         * Gets a boolean if two vectors are equals
         * Example Playground https://playground.babylonjs.com/#QYBWV4#31
         * @param otherVector defines the other vector
         * @returns true if the given vector coordinates strictly equal the current Vector2 ones
         */
        equals(otherVector: DeepImmutable<Vector2>): boolean;
        /**
         * Gets a boolean if two vectors are equals (using an epsilon value)
         * Example Playground https://playground.babylonjs.com/#QYBWV4#32
         * @param otherVector defines the other vector
         * @param epsilon defines the minimal distance to consider equality
         * @returns true if the given vector coordinates are close to the current ones by a distance of epsilon.
         */
        equalsWithEpsilon(otherVector: DeepImmutable<Vector2>, epsilon?: number): boolean;
        /**
         * Gets a new Vector2 from current Vector2 floored values
         * Example Playground https://playground.babylonjs.com/#QYBWV4#35
         * eg (1.2, 2.31) returns (1, 2)
         * @returns a new Vector2
         */
        floor(): this;
        /**
         * Gets a new Vector2 from current Vector2 fractional values
         * Example Playground https://playground.babylonjs.com/#QYBWV4#34
         * eg (1.2, 2.31) returns (0.2, 0.31)
         * @returns a new Vector2
         */
        fract(): this;
        /**
         * Rotate the current vector into a given result vector
         * Example Playground https://playground.babylonjs.com/#QYBWV4#49
         * @param angle defines the rotation angle
         * @param result defines the result vector where to store the rotated vector
         * @returns result input
         */
        rotateToRef<T extends Vector2>(angle: number, result: T): T;
        /**
         * Gets the length of the vector
         * @returns the vector length (float)
         */
        length(): number;
        /**
         * Gets the vector squared length
         * @returns the vector squared length (float)
         */
        lengthSquared(): number;
        /**
         * Normalize the vector
         * Example Playground https://playground.babylonjs.com/#QYBWV4#48
         * @returns the current updated Vector2
         */
        normalize(): this;
        /**
         * Gets a new Vector2 copied from the Vector2
         * Example Playground https://playground.babylonjs.com/#QYBWV4#20
         * @returns a new Vector2
         */
        clone(): this;
        /**
         * Gets a new Vector2(0, 0)
         * @returns a new Vector2
         */
        static Zero(): Vector2;
        /**
         * Gets a new Vector2(1, 1)
         * @returns a new Vector2
         */
        static One(): Vector2;
        /**
         * Gets a zero Vector2 that must not be updated
         */
        static get ZeroReadOnly(): DeepImmutable<Vector2>;
        /**
         * Gets a new Vector2 set from the given index element of the given array
         * Example Playground https://playground.babylonjs.com/#QYBWV4#79
         * @param array defines the data source
         * @param offset defines the offset in the data source
         * @returns a new Vector2
         */
        static FromArray(array: DeepImmutable<ArrayLike<number>>, offset?: number): Vector2;
        /**
         * Sets "result" from the given index element of the given array
         * Example Playground https://playground.babylonjs.com/#QYBWV4#80
         * @param array defines the data source
         * @param offset defines the offset in the data source
         * @param result defines the target vector
         * @returns result input
         */
        static FromArrayToRef<T extends Vector2>(array: DeepImmutable<ArrayLike<number>>, offset: number, result: T): T;
        /**
         * Gets a new Vector2 located for "amount" (float) on the CatmullRom spline defined by the given four Vector2
         * Example Playground https://playground.babylonjs.com/#QYBWV4#65
         * @param value1 defines 1st point of control
         * @param value2 defines 2nd point of control
         * @param value3 defines 3rd point of control
         * @param value4 defines 4th point of control
         * @param amount defines the interpolation factor
         * @returns a new Vector2
         */
        static CatmullRom<T extends Vector2>(value1: DeepImmutable<T>, value2: DeepImmutable<Vector2>, value3: DeepImmutable<Vector2>, value4: DeepImmutable<Vector2>, amount: number): T;
        /**
         * Returns a new Vector2 set with same the coordinates than "value" ones if the vector "value" is in the square defined by "min" and "max".
         * If a coordinate of "value" is lower than "min" coordinates, the returned Vector2 is given this "min" coordinate.
         * If a coordinate of "value" is greater than "max" coordinates, the returned Vector2 is given this "max" coordinate
         * Example Playground https://playground.babylonjs.com/#QYBWV4#76
         * @param value defines the value to clamp
         * @param min defines the lower limit
         * @param max defines the upper limit
         * @returns a new Vector2
         */
        static Clamp<T extends Vector2>(value: DeepImmutable<T>, min: DeepImmutable<Vector2>, max: DeepImmutable<Vector2>): T;
        /**
         * Returns a new Vector2 located for "amount" (float) on the Hermite spline defined by the vectors "value1", "value2", "tangent1", "tangent2"
         * Example Playground https://playground.babylonjs.com/#QYBWV4#81
         * @param value1 defines the 1st control point
         * @param tangent1 defines the outgoing tangent
         * @param value2 defines the 2nd control point
         * @param tangent2 defines the incoming tangent
         * @param amount defines the interpolation factor
         * @returns a new Vector2
         */
        static Hermite<T extends Vector2>(value1: DeepImmutable<T>, tangent1: DeepImmutable<Vector2>, value2: DeepImmutable<Vector2>, tangent2: DeepImmutable<Vector2>, amount: number): T;
        /**
         * Returns a new Vector2 which is the 1st derivative of the Hermite spline defined by the vectors "value1", "value2", "tangent1", "tangent2".
         * Example Playground https://playground.babylonjs.com/#QYBWV4#82
         * @param value1 defines the first control point
         * @param tangent1 defines the first tangent
         * @param value2 defines the second control point
         * @param tangent2 defines the second tangent
         * @param time define where the derivative must be done
         * @returns 1st derivative
         */
        static Hermite1stDerivative<T extends Vector2>(value1: DeepImmutable<T>, tangent1: DeepImmutable<Vector2>, value2: DeepImmutable<Vector2>, tangent2: DeepImmutable<Vector2>, time: number): T;
        /**
         * Returns a new Vector2 which is the 1st derivative of the Hermite spline defined by the vectors "value1", "value2", "tangent1", "tangent2".
         * Example Playground https://playground.babylonjs.com/#QYBWV4#83
         * @param value1 defines the first control point
         * @param tangent1 defines the first tangent
         * @param value2 defines the second control point
         * @param tangent2 defines the second tangent
         * @param time define where the derivative must be done
         * @param result define where the derivative will be stored
         * @returns result input
         */
        static Hermite1stDerivativeToRef<T extends Vector2>(value1: DeepImmutable<Vector2>, tangent1: DeepImmutable<Vector2>, value2: DeepImmutable<Vector2>, tangent2: DeepImmutable<Vector2>, time: number, result: T): T;
        /**
         * Returns a new Vector2 located for "amount" (float) on the linear interpolation between the vector "start" adn the vector "end".
         * Example Playground https://playground.babylonjs.com/#QYBWV4#84
         * @param start defines the start vector
         * @param end defines the end vector
         * @param amount defines the interpolation factor
         * @returns a new Vector2
         */
        static Lerp<T extends Vector2>(start: DeepImmutable<T>, end: DeepImmutable<Vector2>, amount: number): Vector2;
        /**
         * Gets the dot product of the vector "left" and the vector "right"
         * Example Playground https://playground.babylonjs.com/#QYBWV4#90
         * @param left defines first vector
         * @param right defines second vector
         * @returns the dot product (float)
         */
        static Dot(left: DeepImmutable<Vector2>, right: DeepImmutable<Vector2>): number;
        /**
         * Returns a new Vector2 equal to the normalized given vector
         * Example Playground https://playground.babylonjs.com/#QYBWV4#46
         * @param vector defines the vector to normalize
         * @returns a new Vector2
         */
        static Normalize<T extends Vector2>(vector: DeepImmutable<T>): T;
        /**
         * Normalize a given vector into a second one
         * Example Playground https://playground.babylonjs.com/#QYBWV4#50
         * @param vector defines the vector to normalize
         * @param result defines the vector where to store the result
         * @returns result input
         */
        static NormalizeToRef<T extends Vector2>(vector: DeepImmutable<Vector2>, result: T): T;
        /**
         * Gets a new Vector2 set with the minimal coordinate values from the "left" and "right" vectors
         * Example Playground https://playground.babylonjs.com/#QYBWV4#86
         * @param left defines 1st vector
         * @param right defines 2nd vector
         * @returns a new Vector2
         */
        static Minimize<T extends Vector2>(left: DeepImmutable<T>, right: DeepImmutable<Vector2>): T;
        /**
         * Gets a new Vector2 set with the maximal coordinate values from the "left" and "right" vectors
         * Example Playground https://playground.babylonjs.com/#QYBWV4#86
         * @param left defines 1st vector
         * @param right defines 2nd vector
         * @returns a new Vector2
         */
        static Maximize<T extends Vector2>(left: DeepImmutable<T>, right: DeepImmutable<Vector2>): T;
        /**
         * Gets a new Vector2 set with the transformed coordinates of the given vector by the given transformation matrix
         * Example Playground https://playground.babylonjs.com/#QYBWV4#17
         * @param vector defines the vector to transform
         * @param transformation defines the matrix to apply
         * @returns a new Vector2
         */
        static Transform<T extends Vector2>(vector: DeepImmutable<T>, transformation: DeepImmutable<Matrix>): T;
        /**
         * Transforms the given vector coordinates by the given transformation matrix and stores the result in the vector "result" coordinates
         * Example Playground https://playground.babylonjs.com/#QYBWV4#19
         * @param vector defines the vector to transform
         * @param transformation defines the matrix to apply
         * @param result defines the target vector
         * @returns result input
         */
        static TransformToRef<T extends Vector2>(vector: DeepImmutable<Vector2>, transformation: DeepImmutable<Matrix>, result: T): T;
        /**
         * Determines if a given vector is included in a triangle
         * Example Playground https://playground.babylonjs.com/#QYBWV4#87
         * @param p defines the vector to test
         * @param p0 defines 1st triangle point
         * @param p1 defines 2nd triangle point
         * @param p2 defines 3rd triangle point
         * @returns true if the point "p" is in the triangle defined by the vectors "p0", "p1", "p2"
         */
        static PointInTriangle(p: DeepImmutable<Vector2>, p0: DeepImmutable<Vector2>, p1: DeepImmutable<Vector2>, p2: DeepImmutable<Vector2>): boolean;
        /**
         * Gets the distance between the vectors "value1" and "value2"
         * Example Playground https://playground.babylonjs.com/#QYBWV4#71
         * @param value1 defines first vector
         * @param value2 defines second vector
         * @returns the distance between vectors
         */
        static Distance(value1: DeepImmutable<Vector2>, value2: DeepImmutable<Vector2>): number;
        /**
         * Returns the squared distance between the vectors "value1" and "value2"
         * Example Playground https://playground.babylonjs.com/#QYBWV4#72
         * @param value1 defines first vector
         * @param value2 defines second vector
         * @returns the squared distance between vectors
         */
        static DistanceSquared(value1: DeepImmutable<Vector2>, value2: DeepImmutable<Vector2>): number;
        /**
         * Gets a new Vector2 located at the center of the vectors "value1" and "value2"
         * Example Playground https://playground.babylonjs.com/#QYBWV4#86
         * Example Playground https://playground.babylonjs.com/#QYBWV4#66
         * @param value1 defines first vector
         * @param value2 defines second vector
         * @returns a new Vector2
         */
        static Center<T extends Vector2>(value1: DeepImmutable<T>, value2: DeepImmutable<Vector2>): T;
        /**
         * Gets the center of the vectors "value1" and "value2" and stores the result in the vector "ref"
         * Example Playground https://playground.babylonjs.com/#QYBWV4#66
         * @param value1 defines first vector
         * @param value2 defines second vector
         * @param ref defines third vector
         * @returns ref
         */
        static CenterToRef<T extends Vector2>(value1: DeepImmutable<Vector2>, value2: DeepImmutable<Vector2>, ref: T): T;
        /**
         * Gets the shortest distance (float) between the point "p" and the segment defined by the two points "segA" and "segB".
         * Example Playground https://playground.babylonjs.com/#QYBWV4#77
         * @param p defines the middle point
         * @param segA defines one point of the segment
         * @param segB defines the other point of the segment
         * @returns the shortest distance
         */
        static DistanceOfPointFromSegment(p: DeepImmutable<Vector2>, segA: DeepImmutable<Vector2>, segB: DeepImmutable<Vector2>): number;
    }
    /**
     * Class used to store (x,y,z) vector representation
     * A Vector3 is the main object used in 3D geometry
     * It can represent either the coordinates of a point the space, either a direction
     * Reminder: js uses a left handed forward facing system
     * Example Playground - Overview - https://playground.babylonjs.com/#R1F8YU
     */
    export class Vector3 {
        private static _UpReadOnly;
        private static _DownReadOnly;
        private static _LeftHandedForwardReadOnly;
        private static _RightHandedForwardReadOnly;
        private static _RightReadOnly;
        private static _LeftReadOnly;
        private static _ZeroReadOnly;
        /** @internal */
        _x: number;
        /** @internal */
        _y: number;
        /** @internal */
        _z: number;
        /** @internal */
        _isDirty: boolean;
        /** Gets or sets the x coordinate */
        get x(): number;
        set x(value: number);
        /** Gets or sets the y coordinate */
        get y(): number;
        set y(value: number);
        /** Gets or sets the z coordinate */
        get z(): number;
        set z(value: number);
        /**
         * Creates a new Vector3 object from the given x, y, z (floats) coordinates.
         * @param x defines the first coordinates (on X axis)
         * @param y defines the second coordinates (on Y axis)
         * @param z defines the third coordinates (on Z axis)
         */
        constructor(x?: number, y?: number, z?: number);
        /**
         * Creates a string representation of the Vector3
         * Example Playground https://playground.babylonjs.com/#R1F8YU#67
         * @returns a string with the Vector3 coordinates.
         */
        toString(): string;
        /**
         * Gets the class name
         * @returns the string "Vector3"
         */
        getClassName(): string;
        /**
         * Creates the Vector3 hash code
         * @returns a number which tends to be unique between Vector3 instances
         */
        getHashCode(): number;
        /**
         * Creates an array containing three elements : the coordinates of the Vector3
         * Example Playground https://playground.babylonjs.com/#R1F8YU#10
         * @returns a new array of numbers
         */
        asArray(): number[];
        /**
         * Populates the given array or Float32Array from the given index with the successive coordinates of the Vector3
         * Example Playground https://playground.babylonjs.com/#R1F8YU#65
         * @param array defines the destination array
         * @param index defines the offset in the destination array
         * @returns the current Vector3
         */
        toArray(array: FloatArray, index?: number): this;
        /**
         * Update the current vector from an array
         * Example Playground https://playground.babylonjs.com/#R1F8YU#24
         * @param array defines the destination array
         * @param index defines the offset in the destination array
         * @returns the current Vector3
         */
        fromArray(array: FloatArray, index?: number): this;
        /**
         * Converts the current Vector3 into a quaternion (considering that the Vector3 contains Euler angles representation of a rotation)
         * Example Playground https://playground.babylonjs.com/#R1F8YU#66
         * @returns a new Quaternion object, computed from the Vector3 coordinates
         */
        toQuaternion(): Quaternion;
        /**
         * Adds the given vector to the current Vector3
         * Example Playground https://playground.babylonjs.com/#R1F8YU#4
         * @param otherVector defines the second operand
         * @returns the current updated Vector3
         */
        addInPlace(otherVector: DeepImmutable<Vector3>): this;
        /**
         * Adds the given coordinates to the current Vector3
         * Example Playground https://playground.babylonjs.com/#R1F8YU#5
         * @param x defines the x coordinate of the operand
         * @param y defines the y coordinate of the operand
         * @param z defines the z coordinate of the operand
         * @returns the current updated Vector3
         */
        addInPlaceFromFloats(x: number, y: number, z: number): this;
        /**
         * Gets a new Vector3, result of the addition the current Vector3 and the given vector
         * Example Playground https://playground.babylonjs.com/#R1F8YU#3
         * @param otherVector defines the second operand
         * @returns the resulting Vector3
         */
        add(otherVector: DeepImmutable<Vector3>): this;
        /**
         * Adds the current Vector3 to the given one and stores the result in the vector "result"
         * Example Playground https://playground.babylonjs.com/#R1F8YU#6
         * @param otherVector defines the second operand
         * @param result defines the Vector3 object where to store the result
         * @returns the result
         */
        addToRef<T extends Vector3>(otherVector: DeepImmutable<Vector3>, result: T): T;
        /**
         * Subtract the given vector from the current Vector3
         * Example Playground https://playground.babylonjs.com/#R1F8YU#61
         * @param otherVector defines the second operand
         * @returns the current updated Vector3
         */
        subtractInPlace(otherVector: DeepImmutable<Vector3>): this;
        /**
         * Returns a new Vector3, result of the subtraction of the given vector from the current Vector3
         * Example Playground https://playground.babylonjs.com/#R1F8YU#60
         * @param otherVector defines the second operand
         * @returns the resulting Vector3
         */
        subtract(otherVector: DeepImmutable<Vector3>): this;
        /**
         * Subtracts the given vector from the current Vector3 and stores the result in the vector "result".
         * Example Playground https://playground.babylonjs.com/#R1F8YU#63
         * @param otherVector defines the second operand
         * @param result defines the Vector3 object where to store the result
         * @returns the result
         */
        subtractToRef<T extends Vector3>(otherVector: DeepImmutable<Vector3>, result: T): T;
        /**
         * Returns a new Vector3 set with the subtraction of the given floats from the current Vector3 coordinates
         * Example Playground https://playground.babylonjs.com/#R1F8YU#62
         * @param x defines the x coordinate of the operand
         * @param y defines the y coordinate of the operand
         * @param z defines the z coordinate of the operand
         * @returns the resulting Vector3
         */
        subtractFromFloats(x: number, y: number, z: number): this;
        /**
         * Subtracts the given floats from the current Vector3 coordinates and set the given vector "result" with this result
         * Example Playground https://playground.babylonjs.com/#R1F8YU#64
         * @param x defines the x coordinate of the operand
         * @param y defines the y coordinate of the operand
         * @param z defines the z coordinate of the operand
         * @param result defines the Vector3 object where to store the result
         * @returns the result
         */
        subtractFromFloatsToRef<T extends Vector3>(x: number, y: number, z: number, result: T): T;
        /**
         * Gets a new Vector3 set with the current Vector3 negated coordinates
         * Example Playground https://playground.babylonjs.com/#R1F8YU#35
         * @returns a new Vector3
         */
        negate(): this;
        /**
         * Negate this vector in place
         * Example Playground https://playground.babylonjs.com/#R1F8YU#36
         * @returns this
         */
        negateInPlace(): this;
        /**
         * Negate the current Vector3 and stores the result in the given vector "result" coordinates
         * Example Playground https://playground.babylonjs.com/#R1F8YU#37
         * @param result defines the Vector3 object where to store the result
         * @returns the result
         */
        negateToRef<T extends Vector3 = Vector3>(result: T): T;
        /**
         * Multiplies the Vector3 coordinates by the float "scale"
         * Example Playground https://playground.babylonjs.com/#R1F8YU#56
         * @param scale defines the multiplier factor
         * @returns the current updated Vector3
         */
        scaleInPlace(scale: number): this;
        /**
         * Returns a new Vector3 set with the current Vector3 coordinates multiplied by the float "scale"
         * Example Playground https://playground.babylonjs.com/#R1F8YU#53
         * @param scale defines the multiplier factor
         * @returns a new Vector3
         */
        scale(scale: number): this;
        /**
         * Multiplies the current Vector3 coordinates by the float "scale" and stores the result in the given vector "result" coordinates
         * Example Playground https://playground.babylonjs.com/#R1F8YU#57
         * @param scale defines the multiplier factor
         * @param result defines the Vector3 object where to store the result
         * @returns the result
         */
        scaleToRef<T extends Vector3>(scale: number, result: T): T;
        /**
         * Creates a vector normal (perpendicular) to the current Vector3 and stores the result in the given vector
         * Out of the infinite possibilities the normal chosen is the one formed by rotating the current vector
         * 90 degrees about an axis which lies perpendicular to the current vector
         * and its projection on the xz plane. In the case of a current vector in the xz plane
         * the normal is calculated to be along the y axis.
         * Example Playground https://playground.babylonjs.com/#R1F8YU#230
         * Example Playground https://playground.babylonjs.com/#R1F8YU#231
         * @param result defines the Vector3 object where to store the resultant normal
         * returns the result
         */
        getNormalToRef(result: DeepImmutable<Vector3>): Vector3;
        /**
         * Rotates the vector using the given unit quaternion and stores the new vector in result
         * Example Playground https://playground.babylonjs.com/#R1F8YU#9
         * @param q the unit quaternion representing the rotation
         * @param result the output vector
         * @returns the result
         */
        applyRotationQuaternionToRef<T extends Vector3>(q: Quaternion, result: T): T;
        /**
         * Rotates the vector in place using the given unit quaternion
         * Example Playground https://playground.babylonjs.com/#R1F8YU#8
         * @param q the unit quaternion representing the rotation
         * @returns the current updated Vector3
         */
        applyRotationQuaternionInPlace(q: Quaternion): this;
        /**
         * Rotates the vector using the given unit quaternion and returns the new vector
         * Example Playground https://playground.babylonjs.com/#R1F8YU#7
         * @param q the unit quaternion representing the rotation
         * @returns a new Vector3
         */
        applyRotationQuaternion(q: Quaternion): this;
        /**
         * Scale the current Vector3 values by a factor and add the result to a given Vector3
         * Example Playground https://playground.babylonjs.com/#R1F8YU#55
         * @param scale defines the scale factor
         * @param result defines the Vector3 object where to store the result
         * @returns result input
         */
        scaleAndAddToRef<T extends Vector3>(scale: number, result: T): T;
        /**
         * Projects the current point Vector3 to a plane along a ray starting from a specified origin and passing through the current point Vector3.
         * Example Playground https://playground.babylonjs.com/#R1F8YU#48
         * @param plane defines the plane to project to
         * @param origin defines the origin of the projection ray
         * @returns the projected vector3
         */
        projectOnPlane<T extends Vector3>(plane: Plane, origin: Vector3): T;
        /**
         * Projects the current point Vector3 to a plane along a ray starting from a specified origin and passing through the current point Vector3.
         * Example Playground https://playground.babylonjs.com/#R1F8YU#49
         * @param plane defines the plane to project to
         * @param origin defines the origin of the projection ray
         * @param result defines the Vector3 where to store the result
         * @returns result input
         */
        projectOnPlaneToRef<T extends Vector3>(plane: Plane, origin: Vector3, result: T): T;
        /**
         * Returns true if the current Vector3 and the given vector coordinates are strictly equal
         * Example Playground https://playground.babylonjs.com/#R1F8YU#19
         * @param otherVector defines the second operand
         * @returns true if both vectors are equals
         */
        equals(otherVector: DeepImmutable<Vector3>): boolean;
        /**
         * Returns true if the current Vector3 and the given vector coordinates are distant less than epsilon
         * Example Playground https://playground.babylonjs.com/#R1F8YU#21
         * @param otherVector defines the second operand
         * @param epsilon defines the minimal distance to define values as equals
         * @returns true if both vectors are distant less than epsilon
         */
        equalsWithEpsilon(otherVector: DeepImmutable<Vector3>, epsilon?: number): boolean;
        /**
         * Returns true if the current Vector3 coordinates equals the given floats
         * Example Playground https://playground.babylonjs.com/#R1F8YU#20
         * @param x defines the x coordinate of the operand
         * @param y defines the y coordinate of the operand
         * @param z defines the z coordinate of the operand
         * @returns true if both vectors are equal
         */
        equalsToFloats(x: number, y: number, z: number): boolean;
        /**
         * Multiplies the current Vector3 coordinates by the given ones
         * Example Playground https://playground.babylonjs.com/#R1F8YU#32
         * @param otherVector defines the second operand
         * @returns the current updated Vector3
         */
        multiplyInPlace(otherVector: DeepImmutable<Vector3>): this;
        /**
         * Returns a new Vector3, result of the multiplication of the current Vector3 by the given vector
         * Example Playground https://playground.babylonjs.com/#R1F8YU#31
         * @param otherVector defines the second operand
         * @returns the new Vector3
         */
        multiply(otherVector: DeepImmutable<Vector3>): this;
        /**
         * Multiplies the current Vector3 by the given one and stores the result in the given vector "result"
         * Example Playground https://playground.babylonjs.com/#R1F8YU#33
         * @param otherVector defines the second operand
         * @param result defines the Vector3 object where to store the result
         * @returns the result
         */
        multiplyToRef<T extends Vector3>(otherVector: DeepImmutable<Vector3>, result: T): T;
        /**
         * Returns a new Vector3 set with the result of the multiplication of the current Vector3 coordinates by the given floats
         * Example Playground https://playground.babylonjs.com/#R1F8YU#34
         * @param x defines the x coordinate of the operand
         * @param y defines the y coordinate of the operand
         * @param z defines the z coordinate of the operand
         * @returns the new Vector3
         */
        multiplyByFloats(x: number, y: number, z: number): this;
        /**
         * Returns a new Vector3 set with the result of the division of the current Vector3 coordinates by the given ones
         * Example Playground https://playground.babylonjs.com/#R1F8YU#16
         * @param otherVector defines the second operand
         * @returns the new Vector3
         */
        divide(otherVector: DeepImmutable<Vector3>): this;
        /**
         * Divides the current Vector3 coordinates by the given ones and stores the result in the given vector "result"
         * Example Playground https://playground.babylonjs.com/#R1F8YU#18
         * @param otherVector defines the second operand
         * @param result defines the Vector3 object where to store the result
         * @returns the result
         */
        divideToRef<T extends Vector3>(otherVector: DeepImmutable<Vector3>, result: T): T;
        /**
         * Divides the current Vector3 coordinates by the given ones.
         * Example Playground https://playground.babylonjs.com/#R1F8YU#17
         * @param otherVector defines the second operand
         * @returns the current updated Vector3
         */
        divideInPlace(otherVector: Vector3): this;
        /**
         * Updates the current Vector3 with the minimal coordinate values between its and the given vector ones
         * Example Playground https://playground.babylonjs.com/#R1F8YU#29
         * @param other defines the second operand
         * @returns the current updated Vector3
         */
        minimizeInPlace(other: DeepImmutable<Vector3>): this;
        /**
         * Updates the current Vector3 with the maximal coordinate values between its and the given vector ones.
         * Example Playground https://playground.babylonjs.com/#R1F8YU#27
         * @param other defines the second operand
         * @returns the current updated Vector3
         */
        maximizeInPlace(other: DeepImmutable<Vector3>): this;
        /**
         * Updates the current Vector3 with the minimal coordinate values between its and the given coordinates
         * Example Playground https://playground.babylonjs.com/#R1F8YU#30
         * @param x defines the x coordinate of the operand
         * @param y defines the y coordinate of the operand
         * @param z defines the z coordinate of the operand
         * @returns the current updated Vector3
         */
        minimizeInPlaceFromFloats(x: number, y: number, z: number): this;
        /**
         * Updates the current Vector3 with the maximal coordinate values between its and the given coordinates.
         * Example Playground https://playground.babylonjs.com/#R1F8YU#28
         * @param x defines the x coordinate of the operand
         * @param y defines the y coordinate of the operand
         * @param z defines the z coordinate of the operand
         * @returns the current updated Vector3
         */
        maximizeInPlaceFromFloats(x: number, y: number, z: number): this;
        /**
         * Due to float precision, scale of a mesh could be uniform but float values are off by a small fraction
         * Check if is non uniform within a certain amount of decimal places to account for this
         * @param epsilon the amount the values can differ
         * @returns if the the vector is non uniform to a certain number of decimal places
         */
        isNonUniformWithinEpsilon(epsilon: number): boolean;
        /**
         * Gets a boolean indicating that the vector is non uniform meaning x, y or z are not all the same
         */
        get isNonUniform(): boolean;
        /**
         * Gets a new Vector3 from current Vector3 floored values
         * Example Playground https://playground.babylonjs.com/#R1F8YU#22
         * @returns a new Vector3
         */
        floor(): this;
        /**
         * Gets a new Vector3 from current Vector3 fractional values
         * Example Playground https://playground.babylonjs.com/#R1F8YU#23
         * @returns a new Vector3
         */
        fract(): this;
        /**
         * Gets the length of the Vector3
         * Example Playground https://playground.babylonjs.com/#R1F8YU#25
         * @returns the length of the Vector3
         */
        length(): number;
        /**
         * Gets the squared length of the Vector3
         * Example Playground https://playground.babylonjs.com/#R1F8YU#26
         * @returns squared length of the Vector3
         */
        lengthSquared(): number;
        /**
         * Gets a boolean indicating if the vector contains a zero in one of its components
         * Example Playground https://playground.babylonjs.com/#R1F8YU#1
         */
        get hasAZeroComponent(): boolean;
        /**
         * Normalize the current Vector3.
         * Please note that this is an in place operation.
         * Example Playground https://playground.babylonjs.com/#R1F8YU#122
         * @returns the current updated Vector3
         */
        normalize(): this;
        /**
         * Reorders the x y z properties of the vector in place
         * Example Playground https://playground.babylonjs.com/#R1F8YU#44
         * @param order new ordering of the properties (eg. for vector 1,2,3 with "ZYX" will produce 3,2,1)
         * @returns the current updated vector
         */
        reorderInPlace(order: string): this;
        /**
         * Rotates the vector around 0,0,0 by a quaternion
         * Example Playground https://playground.babylonjs.com/#R1F8YU#47
         * @param quaternion the rotation quaternion
         * @param result vector to store the result
         * @returns the resulting vector
         */
        rotateByQuaternionToRef<T extends Vector3>(quaternion: Quaternion, result: T): T;
        /**
         * Rotates a vector around a given point
         * Example Playground https://playground.babylonjs.com/#R1F8YU#46
         * @param quaternion the rotation quaternion
         * @param point the point to rotate around
         * @param result vector to store the result
         * @returns the resulting vector
         */
        rotateByQuaternionAroundPointToRef<T extends Vector3>(quaternion: Quaternion, point: Vector3, result: T): T;
        /**
         * Returns a new Vector3 as the cross product of the current vector and the "other" one
         * The cross product is then orthogonal to both current and "other"
         * Example Playground https://playground.babylonjs.com/#R1F8YU#14
         * @param other defines the right operand
         * @returns the cross product
         */
        cross(other: Vector3): this;
        /**
         * Normalize the current Vector3 with the given input length.
         * Please note that this is an in place operation.
         * Example Playground https://playground.babylonjs.com/#R1F8YU#123
         * @param len the length of the vector
         * @returns the current updated Vector3
         */
        normalizeFromLength(len: number): this;
        /**
         * Normalize the current Vector3 to a new vector
         * Example Playground https://playground.babylonjs.com/#R1F8YU#124
         * @returns the new Vector3
         */
        normalizeToNew(): this;
        /**
         * Normalize the current Vector3 to the reference
         * Example Playground https://playground.babylonjs.com/#R1F8YU#125
         * @param reference define the Vector3 to update
         * @returns the updated Vector3
         */
        normalizeToRef<T extends Vector3>(reference: T): T;
        /**
         * Creates a new Vector3 copied from the current Vector3
         * Example Playground https://playground.babylonjs.com/#R1F8YU#11
         * @returns the new Vector3
         */
        clone(): this;
        /**
         * Copies the given vector coordinates to the current Vector3 ones
         * Example Playground https://playground.babylonjs.com/#R1F8YU#12
         * @param source defines the source Vector3
         * @returns the current updated Vector3
         */
        copyFrom(source: DeepImmutable<Vector3>): this;
        /**
         * Copies the given floats to the current Vector3 coordinates
         * Example Playground https://playground.babylonjs.com/#R1F8YU#13
         * @param x defines the x coordinate of the operand
         * @param y defines the y coordinate of the operand
         * @param z defines the z coordinate of the operand
         * @returns the current updated Vector3
         */
        copyFromFloats(x: number, y: number, z: number): this;
        /**
         * Copies the given floats to the current Vector3 coordinates
         * Example Playground https://playground.babylonjs.com/#R1F8YU#58
         * @param x defines the x coordinate of the operand
         * @param y defines the y coordinate of the operand
         * @param z defines the z coordinate of the operand
         * @returns the current updated Vector3
         */
        set(x: number, y: number, z: number): this;
        /**
         * Copies the given float to the current Vector3 coordinates
         * Example Playground https://playground.babylonjs.com/#R1F8YU#59
         * @param v defines the x, y and z coordinates of the operand
         * @returns the current updated Vector3
         */
        setAll(v: number): this;
        /**
         * Get the clip factor between two vectors
         * Example Playground https://playground.babylonjs.com/#R1F8YU#126
         * @param vector0 defines the first operand
         * @param vector1 defines the second operand
         * @param axis defines the axis to use
         * @param size defines the size along the axis
         * @returns the clip factor
         */
        static GetClipFactor(vector0: DeepImmutable<Vector3>, vector1: DeepImmutable<Vector3>, axis: DeepImmutable<Vector3>, size: number): number;
        /**
         * Get angle between two vectors
         * Example Playground https://playground.babylonjs.com/#R1F8YU#86
         * @param vector0 the starting point
         * @param vector1 the ending point
         * @param normal direction of the normal
         * @returns the angle between vector0 and vector1
         */
        static GetAngleBetweenVectors(vector0: DeepImmutable<Vector3>, vector1: DeepImmutable<Vector3>, normal: DeepImmutable<Vector3>): number;
        /**
         * Get angle between two vectors projected on a plane
         * Example Playground https://playground.babylonjs.com/#R1F8YU#87
         * Expectation compute time: 0.01 ms (median) and 0.02 ms (percentile 95%)
         * @param vector0 angle between vector0 and vector1
         * @param vector1 angle between vector0 and vector1
         * @param normal Normal of the projection plane
         * @returns the angle in radians (float) between vector0 and vector1 projected on the plane with the specified normal
         */
        static GetAngleBetweenVectorsOnPlane(vector0: Vector3, vector1: Vector3, normal: Vector3): number;
        /**
         * Gets the rotation that aligns the roll axis (Y) to the line joining the start point to the target point and stores it in the ref Vector3
         * Example PG https://playground.babylonjs.com/#R1F8YU#189
         * @param start the starting point
         * @param target the target point
         * @param ref the vector3 to store the result
         * @returns ref in the form (pitch, yaw, 0)
         */
        static PitchYawRollToMoveBetweenPointsToRef<T extends Vector3>(start: Vector3, target: Vector3, ref: T): T;
        /**
         * Gets the rotation that aligns the roll axis (Y) to the line joining the start point to the target point
         * Example PG https://playground.babylonjs.com/#R1F8YU#188
         * @param start the starting point
         * @param target the target point
         * @returns the rotation in the form (pitch, yaw, 0)
         */
        static PitchYawRollToMoveBetweenPoints(start: Vector3, target: Vector3): Vector3;
        /**
         * Slerp between two vectors. See also `SmoothToRef`
         * Slerp is a spherical linear interpolation
         * giving a slow in and out effect
         * Example Playground 1 https://playground.babylonjs.com/#R1F8YU#108
         * Example Playground 2 https://playground.babylonjs.com/#R1F8YU#109
         * @param vector0 Start vector
         * @param vector1 End vector
         * @param slerp amount (will be clamped between 0 and 1)
         * @param result The slerped vector
         */
        static SlerpToRef<T extends Vector3 = Vector3>(vector0: Vector3, vector1: Vector3, slerp: number, result: T): T;
        /**
         * Smooth interpolation between two vectors using Slerp
         * Example Playground https://playground.babylonjs.com/#R1F8YU#110
         * @param source source vector
         * @param goal goal vector
         * @param deltaTime current interpolation frame
         * @param lerpTime total interpolation time
         * @param result the smoothed vector
         */
        static SmoothToRef<T extends Vector3 = Vector3>(source: Vector3, goal: Vector3, deltaTime: number, lerpTime: number, result: T): T;
        /**
         * Returns a new Vector3 set from the index "offset" of the given array
         * Example Playground https://playground.babylonjs.com/#R1F8YU#83
         * @param array defines the source array
         * @param offset defines the offset in the source array
         * @returns the new Vector3
         */
        static FromArray(array: DeepImmutable<ArrayLike<number>>, offset?: number): Vector3;
        /**
         * Returns a new Vector3 set from the index "offset" of the given Float32Array
         * @param array defines the source array
         * @param offset defines the offset in the source array
         * @returns the new Vector3
         * @deprecated Please use FromArray instead.
         */
        static FromFloatArray(array: DeepImmutable<Float32Array>, offset?: number): Vector3;
        /**
         * Sets the given vector "result" with the element values from the index "offset" of the given array
         * Example Playground https://playground.babylonjs.com/#R1F8YU#84
         * @param array defines the source array
         * @param offset defines the offset in the source array
         * @param result defines the Vector3 where to store the result
         * @returns result input
         */
        static FromArrayToRef<T extends Vector3>(array: DeepImmutable<ArrayLike<number>>, offset: number, result: T): T;
        /**
         * Sets the given vector "result" with the element values from the index "offset" of the given Float32Array
         * @param array defines the source array
         * @param offset defines the offset in the source array
         * @param result defines the Vector3 where to store the result
         * @deprecated Please use FromArrayToRef instead.
         */
        static FromFloatArrayToRef<T extends Vector3>(array: DeepImmutable<Float32Array>, offset: number, result: T): T;
        /**
         * Sets the given vector "result" with the given floats.
         * Example Playground https://playground.babylonjs.com/#R1F8YU#85
         * @param x defines the x coordinate of the source
         * @param y defines the y coordinate of the source
         * @param z defines the z coordinate of the source
         * @param result defines the Vector3 where to store the result
         */
        static FromFloatsToRef<T extends Vector3 = Vector3>(x: number, y: number, z: number, result: T): T;
        /**
         * Returns a new Vector3 set to (0.0, 0.0, 0.0)
         * @returns a new empty Vector3
         */
        static Zero(): Vector3;
        /**
         * Returns a new Vector3 set to (1.0, 1.0, 1.0)
         * @returns a new Vector3
         */
        static One(): Vector3;
        /**
         * Returns a new Vector3 set to (0.0, 1.0, 0.0)
         * Example Playground https://playground.babylonjs.com/#R1F8YU#71
         * @returns a new up Vector3
         */
        static Up(): Vector3;
        /**
         * Gets an up Vector3 that must not be updated
         */
        static get UpReadOnly(): DeepImmutable<Vector3>;
        /**
         * Gets a down Vector3 that must not be updated
         */
        static get DownReadOnly(): DeepImmutable<Vector3>;
        /**
         * Gets a right Vector3 that must not be updated
         */
        static get RightReadOnly(): DeepImmutable<Vector3>;
        /**
         * Gets a left Vector3 that must not be updated
         */
        static get LeftReadOnly(): DeepImmutable<Vector3>;
        /**
         * Gets a forward Vector3 that must not be updated
         */
        static get LeftHandedForwardReadOnly(): DeepImmutable<Vector3>;
        /**
         * Gets a forward Vector3 that must not be updated
         */
        static get RightHandedForwardReadOnly(): DeepImmutable<Vector3>;
        /**
         * Gets a zero Vector3 that must not be updated
         */
        static get ZeroReadOnly(): DeepImmutable<Vector3>;
        /**
         * Returns a new Vector3 set to (0.0, -1.0, 0.0)
         * Example Playground https://playground.babylonjs.com/#R1F8YU#71
         * @returns a new down Vector3
         */
        static Down(): Vector3;
        /**
         * Returns a new Vector3 set to (0.0, 0.0, 1.0)
         * Example Playground https://playground.babylonjs.com/#R1F8YU#71
         * @param rightHandedSystem is the scene right-handed (negative z)
         * @returns a new forward Vector3
         */
        static Forward(rightHandedSystem?: boolean): Vector3;
        /**
         * Returns a new Vector3 set to (0.0, 0.0, -1.0)
         * Example Playground https://playground.babylonjs.com/#R1F8YU#71
         * @param rightHandedSystem is the scene right-handed (negative-z)
         * @returns a new Backward Vector3
         */
        static Backward(rightHandedSystem?: boolean): Vector3;
        /**
         * Returns a new Vector3 set to (1.0, 0.0, 0.0)
         * Example Playground https://playground.babylonjs.com/#R1F8YU#71
         * @returns a new right Vector3
         */
        static Right(): Vector3;
        /**
         * Returns a new Vector3 set to (-1.0, 0.0, 0.0)
         * Example Playground https://playground.babylonjs.com/#R1F8YU#71
         * @returns a new left Vector3
         */
        static Left(): Vector3;
        /**
         * Returns a new Vector3 set with the result of the transformation by the given matrix of the given vector.
         * This method computes transformed coordinates only, not transformed direction vectors (ie. it takes translation in account)
         * Example Playground https://playground.babylonjs.com/#R1F8YU#111
         * @param vector defines the Vector3 to transform
         * @param transformation defines the transformation matrix
         * @returns the transformed Vector3
         */
        static TransformCoordinates<T extends Vector3>(vector: DeepImmutable<Vector3>, transformation: DeepImmutable<Matrix>): Vector3;
        /**
         * Sets the given vector "result" coordinates with the result of the transformation by the given matrix of the given vector
         * This method computes transformed coordinates only, not transformed direction vectors (ie. it takes translation in account)
         * Example Playground https://playground.babylonjs.com/#R1F8YU#113
         * @param vector defines the Vector3 to transform
         * @param transformation defines the transformation matrix
         * @param result defines the Vector3 where to store the result
         * @returns result input
         */
        static TransformCoordinatesToRef<T extends Vector3>(vector: DeepImmutable<Vector3>, transformation: DeepImmutable<Matrix>, result: T): T;
        /**
         * Sets the given vector "result" coordinates with the result of the transformation by the given matrix of the given floats (x, y, z)
         * This method computes transformed coordinates only, not transformed direction vectors
         * Example Playground https://playground.babylonjs.com/#R1F8YU#115
         * @param x define the x coordinate of the source vector
         * @param y define the y coordinate of the source vector
         * @param z define the z coordinate of the source vector
         * @param transformation defines the transformation matrix
         * @param result defines the Vector3 where to store the result
         * @returns result input
         */
        static TransformCoordinatesFromFloatsToRef<T extends Vector3>(x: number, y: number, z: number, transformation: DeepImmutable<Matrix>, result: T): T;
        /**
         * Returns a new Vector3 set with the result of the normal transformation by the given matrix of the given vector
         * This methods computes transformed normalized direction vectors only (ie. it does not apply translation)
         * Example Playground https://playground.babylonjs.com/#R1F8YU#112
         * @param vector defines the Vector3 to transform
         * @param transformation defines the transformation matrix
         * @returns the new Vector3
         */
        static TransformNormal(vector: DeepImmutable<Vector3>, transformation: DeepImmutable<Matrix>): Vector3;
        /**
         * Sets the given vector "result" with the result of the normal transformation by the given matrix of the given vector
         * This methods computes transformed normalized direction vectors only (ie. it does not apply translation)
         * Example Playground https://playground.babylonjs.com/#R1F8YU#114
         * @param vector defines the Vector3 to transform
         * @param transformation defines the transformation matrix
         * @param result defines the Vector3 where to store the result
         * @returns result input
         */
        static TransformNormalToRef<T extends Vector3>(vector: DeepImmutable<Vector3>, transformation: DeepImmutable<Matrix>, result: T): T;
        /**
         * Sets the given vector "result" with the result of the normal transformation by the given matrix of the given floats (x, y, z)
         * This methods computes transformed normalized direction vectors only (ie. it does not apply translation)
         * Example Playground https://playground.babylonjs.com/#R1F8YU#116
         * @param x define the x coordinate of the source vector
         * @param y define the y coordinate of the source vector
         * @param z define the z coordinate of the source vector
         * @param transformation defines the transformation matrix
         * @param result defines the Vector3 where to store the result
         * @returns result input
         */
        static TransformNormalFromFloatsToRef<T extends Vector3>(x: number, y: number, z: number, transformation: DeepImmutable<Matrix>, result: T): T;
        /**
         * Returns a new Vector3 located for "amount" on the CatmullRom interpolation spline defined by the vectors "value1", "value2", "value3", "value4"
         * Example Playground https://playground.babylonjs.com/#R1F8YU#69
         * @param value1 defines the first control point
         * @param value2 defines the second control point
         * @param value3 defines the third control point
         * @param value4 defines the fourth control point
         * @param amount defines the amount on the spline to use
         * @returns the new Vector3
         */
        static CatmullRom<T extends Vector3>(value1: DeepImmutable<T>, value2: DeepImmutable<Vector3>, value3: DeepImmutable<Vector3>, value4: DeepImmutable<Vector3>, amount: number): T;
        /**
         * Returns a new Vector3 set with the coordinates of "value", if the vector "value" is in the cube defined by the vectors "min" and "max"
         * If a coordinate value of "value" is lower than one of the "min" coordinate, then this "value" coordinate is set with the "min" one
         * If a coordinate value of "value" is greater than one of the "max" coordinate, then this "value" coordinate is set with the "max" one
         * Example Playground https://playground.babylonjs.com/#R1F8YU#76
         * @param value defines the current value
         * @param min defines the lower range value
         * @param max defines the upper range value
         * @returns the new Vector3
         */
        static Clamp<T extends Vector3>(value: DeepImmutable<T>, min: DeepImmutable<Vector3>, max: DeepImmutable<Vector3>): T;
        /**
         * Sets the given vector "result" with the coordinates of "value", if the vector "value" is in the cube defined by the vectors "min" and "max"
         * If a coordinate value of "value" is lower than one of the "min" coordinate, then this "value" coordinate is set with the "min" one
         * If a coordinate value of "value" is greater than one of the "max" coordinate, then this "value" coordinate is set with the "max" one
         * Example Playground https://playground.babylonjs.com/#R1F8YU#77
         * @param value defines the current value
         * @param min defines the lower range value
         * @param max defines the upper range value
         * @param result defines the Vector3 where to store the result
         * @returns result input
         */
        static ClampToRef<T extends Vector3>(value: DeepImmutable<Vector3>, min: DeepImmutable<Vector3>, max: DeepImmutable<Vector3>, result: T): T;
        /**
         * Checks if a given vector is inside a specific range
         * Example Playground https://playground.babylonjs.com/#R1F8YU#75
         * @param v defines the vector to test
         * @param min defines the minimum range
         * @param max defines the maximum range
         */
        static CheckExtends(v: Vector3, min: Vector3, max: Vector3): void;
        /**
         * Returns a new Vector3 located for "amount" (float) on the Hermite interpolation spline defined by the vectors "value1", "tangent1", "value2", "tangent2"
         * Example Playground https://playground.babylonjs.com/#R1F8YU#89
         * @param value1 defines the first control point
         * @param tangent1 defines the first tangent vector
         * @param value2 defines the second control point
         * @param tangent2 defines the second tangent vector
         * @param amount defines the amount on the interpolation spline (between 0 and 1)
         * @returns the new Vector3
         */
        static Hermite<T extends Vector3>(value1: DeepImmutable<T>, tangent1: DeepImmutable<Vector3>, value2: DeepImmutable<Vector3>, tangent2: DeepImmutable<Vector3>, amount: number): T;
        /**
         * Returns a new Vector3 which is the 1st derivative of the Hermite spline defined by the vectors "value1", "value2", "tangent1", "tangent2".
         * Example Playground https://playground.babylonjs.com/#R1F8YU#90
         * @param value1 defines the first control point
         * @param tangent1 defines the first tangent
         * @param value2 defines the second control point
         * @param tangent2 defines the second tangent
         * @param time define where the derivative must be done
         * @returns 1st derivative
         */
        static Hermite1stDerivative<T extends Vector3>(value1: DeepImmutable<T>, tangent1: DeepImmutable<Vector3>, value2: DeepImmutable<Vector3>, tangent2: DeepImmutable<Vector3>, time: number): T;
        /**
         * Update a Vector3 with the 1st derivative of the Hermite spline defined by the vectors "value1", "value2", "tangent1", "tangent2".
         * Example Playground https://playground.babylonjs.com/#R1F8YU#91
         * @param value1 defines the first control point
         * @param tangent1 defines the first tangent
         * @param value2 defines the second control point
         * @param tangent2 defines the second tangent
         * @param time define where the derivative must be done
         * @param result define where to store the derivative
         * @returns result input
         */
        static Hermite1stDerivativeToRef<T extends Vector3>(value1: DeepImmutable<Vector3>, tangent1: DeepImmutable<Vector3>, value2: DeepImmutable<Vector3>, tangent2: DeepImmutable<Vector3>, time: number, result: T): T;
        /**
         * Returns a new Vector3 located for "amount" (float) on the linear interpolation between the vectors "start" and "end"
         * Example Playground https://playground.babylonjs.com/#R1F8YU#95
         * @param start defines the start value
         * @param end defines the end value
         * @param amount max defines amount between both (between 0 and 1)
         * @returns the new Vector3
         */
        static Lerp<T extends Vector3>(start: DeepImmutable<T>, end: DeepImmutable<Vector3>, amount: number): T;
        /**
         * Sets the given vector "result" with the result of the linear interpolation from the vector "start" for "amount" to the vector "end"
         * Example Playground https://playground.babylonjs.com/#R1F8YU#93
         * @param start defines the start value
         * @param end defines the end value
         * @param amount max defines amount between both (between 0 and 1)
         * @param result defines the Vector3 where to store the result
         * @returns result input
         */
        static LerpToRef<T extends Vector3>(start: DeepImmutable<Vector3>, end: DeepImmutable<Vector3>, amount: number, result: T): T;
        /**
         * Returns the dot product (float) between the vectors "left" and "right"
         * Example Playground https://playground.babylonjs.com/#R1F8YU#82
         * @param left defines the left operand
         * @param right defines the right operand
         * @returns the dot product
         */
        static Dot(left: DeepImmutable<Vector3>, right: DeepImmutable<Vector3>): number;
        /**
         * Returns a new Vector3 as the cross product of the vectors "left" and "right"
         * The cross product is then orthogonal to both "left" and "right"
         * Example Playground https://playground.babylonjs.com/#R1F8YU#15
         * @param left defines the left operand
         * @param right defines the right operand
         * @returns the cross product
         */
        static Cross<T extends Vector3>(left: DeepImmutable<T>, right: DeepImmutable<Vector3>): T;
        /**
         * Sets the given vector "result" with the cross product of "left" and "right"
         * The cross product is then orthogonal to both "left" and "right"
         * Example Playground https://playground.babylonjs.com/#R1F8YU#78
         * @param left defines the left operand
         * @param right defines the right operand
         * @param result defines the Vector3 where to store the result
         * @returns result input
         */
        static CrossToRef<T extends Vector3>(left: DeepImmutable<Vector3>, right: DeepImmutable<Vector3>, result: T): T;
        /**
         * Returns a new Vector3 as the normalization of the given vector
         * Example Playground https://playground.babylonjs.com/#R1F8YU#98
         * @param vector defines the Vector3 to normalize
         * @returns the new Vector3
         */
        static Normalize(vector: DeepImmutable<Vector3>): Vector3;
        /**
         * Sets the given vector "result" with the normalization of the given first vector
         * Example Playground https://playground.babylonjs.com/#R1F8YU#98
         * @param vector defines the Vector3 to normalize
         * @param result defines the Vector3 where to store the result
         * @returns result input
         */
        static NormalizeToRef<T extends Vector3>(vector: DeepImmutable<Vector3>, result: T): T;
        /**
         * Project a Vector3 onto screen space
         * Example Playground https://playground.babylonjs.com/#R1F8YU#101
         * @param vector defines the Vector3 to project
         * @param world defines the world matrix to use
         * @param transform defines the transform (view x projection) matrix to use
         * @param viewport defines the screen viewport to use
         * @returns the new Vector3
         */
        static Project<T extends Vector3>(vector: DeepImmutable<T>, world: DeepImmutable<Matrix>, transform: DeepImmutable<Matrix>, viewport: DeepImmutable<Viewport>): T;
        /**
         * Project a Vector3 onto screen space to reference
         * Example Playground https://playground.babylonjs.com/#R1F8YU#102
         * @param vector defines the Vector3 to project
         * @param world defines the world matrix to use
         * @param transform defines the transform (view x projection) matrix to use
         * @param viewport defines the screen viewport to use
         * @param result the vector in which the screen space will be stored
         * @returns result input
         */
        static ProjectToRef<T extends Vector3>(vector: DeepImmutable<Vector3>, world: DeepImmutable<Matrix>, transform: DeepImmutable<Matrix>, viewport: DeepImmutable<Viewport>, result: T): T;
        /**
         * Reflects a vector off the plane defined by a normalized normal
         * @param inDirection defines the vector direction
         * @param normal defines the normal - Must be normalized
         * @returns the resulting vector
         */
        static Reflect<T extends Vector3>(inDirection: DeepImmutable<Vector3>, normal: DeepImmutable<Vector3>): Vector3;
        /**
         * Reflects a vector off the plane defined by a normalized normal to reference
         * @param inDirection defines the vector direction
         * @param normal defines the normal - Must be normalized
         * @param result defines the Vector3 where to store the result
         * @returns the resulting vector
         */
        static ReflectToRef<T extends Vector3>(inDirection: DeepImmutable<Vector3>, normal: DeepImmutable<Vector3>, ref: T): T;
        /**
         * @internal
         */
        static _UnprojectFromInvertedMatrixToRef<T extends Vector3>(source: DeepImmutable<Vector3>, matrix: DeepImmutable<Matrix>, result: T): T;
        /**
         * Unproject from screen space to object space
         * Example Playground https://playground.babylonjs.com/#R1F8YU#121
         * @param source defines the screen space Vector3 to use
         * @param viewportWidth defines the current width of the viewport
         * @param viewportHeight defines the current height of the viewport
         * @param world defines the world matrix to use (can be set to Identity to go to world space)
         * @param transform defines the transform (view x projection) matrix to use
         * @returns the new Vector3
         */
        static UnprojectFromTransform<T extends Vector3>(source: DeepImmutable<T>, viewportWidth: number, viewportHeight: number, world: DeepImmutable<Matrix>, transform: DeepImmutable<Matrix>): T;
        /**
         * Unproject from screen space to object space
         * Example Playground https://playground.babylonjs.com/#R1F8YU#117
         * @param source defines the screen space Vector3 to use
         * @param viewportWidth defines the current width of the viewport
         * @param viewportHeight defines the current height of the viewport
         * @param world defines the world matrix to use (can be set to Identity to go to world space)
         * @param view defines the view matrix to use
         * @param projection defines the projection matrix to use
         * @returns the new Vector3
         */
        static Unproject<T extends Vector3>(source: DeepImmutable<T>, viewportWidth: number, viewportHeight: number, world: DeepImmutable<Matrix>, view: DeepImmutable<Matrix>, projection: DeepImmutable<Matrix>): T;
        /**
         * Unproject from screen space to object space
         * Example Playground https://playground.babylonjs.com/#R1F8YU#119
         * @param source defines the screen space Vector3 to use
         * @param viewportWidth defines the current width of the viewport
         * @param viewportHeight defines the current height of the viewport
         * @param world defines the world matrix to use (can be set to Identity to go to world space)
         * @param view defines the view matrix to use
         * @param projection defines the projection matrix to use
         * @param result defines the Vector3 where to store the result
         * @returns result input
         */
        static UnprojectToRef<T extends Vector3>(source: DeepImmutable<Vector3>, viewportWidth: number, viewportHeight: number, world: DeepImmutable<Matrix>, view: DeepImmutable<Matrix>, projection: DeepImmutable<Matrix>, result: T): T;
        /**
         * Unproject from screen space to object space
         * Example Playground https://playground.babylonjs.com/#R1F8YU#120
         * @param sourceX defines the screen space x coordinate to use
         * @param sourceY defines the screen space y coordinate to use
         * @param sourceZ defines the screen space z coordinate to use
         * @param viewportWidth defines the current width of the viewport
         * @param viewportHeight defines the current height of the viewport
         * @param world defines the world matrix to use (can be set to Identity to go to world space)
         * @param view defines the view matrix to use
         * @param projection defines the projection matrix to use
         * @param result defines the Vector3 where to store the result
         * @returns result input
         */
        static UnprojectFloatsToRef<T extends Vector3>(sourceX: float, sourceY: float, sourceZ: float, viewportWidth: number, viewportHeight: number, world: DeepImmutable<Matrix>, view: DeepImmutable<Matrix>, projection: DeepImmutable<Matrix>, result: T): T;
        /**
         * Gets the minimal coordinate values between two Vector3
         * Example Playground https://playground.babylonjs.com/#R1F8YU#97
         * @param left defines the first operand
         * @param right defines the second operand
         * @returns the new Vector3
         */
        static Minimize<T extends Vector3>(left: DeepImmutable<T>, right: DeepImmutable<Vector3>): T;
        /**
         * Gets the maximal coordinate values between two Vector3
         * Example Playground https://playground.babylonjs.com/#R1F8YU#96
         * @param left defines the first operand
         * @param right defines the second operand
         * @returns the new Vector3
         */
        static Maximize<T extends Vector3>(left: DeepImmutable<T>, right: DeepImmutable<Vector3>): T;
        /**
         * Returns the distance between the vectors "value1" and "value2"
         * Example Playground https://playground.babylonjs.com/#R1F8YU#81
         * @param value1 defines the first operand
         * @param value2 defines the second operand
         * @returns the distance
         */
        static Distance(value1: DeepImmutable<Vector3>, value2: DeepImmutable<Vector3>): number;
        /**
         * Returns the squared distance between the vectors "value1" and "value2"
         * Example Playground https://playground.babylonjs.com/#R1F8YU#80
         * @param value1 defines the first operand
         * @param value2 defines the second operand
         * @returns the squared distance
         */
        static DistanceSquared(value1: DeepImmutable<Vector3>, value2: DeepImmutable<Vector3>): number;
        /**
         * Projects "vector" on the triangle determined by its extremities "p0", "p1" and "p2", stores the result in "ref"
         * and returns the distance to the projected point.
         * Example Playground https://playground.babylonjs.com/#R1F8YU#104
         * From http://citeseerx.ist.psu.edu/viewdoc/download?doi=10.1.1.104.4264&rep=rep1&type=pdf
         *
         * @param vector the vector to get distance from
         * @param p0 extremity of the triangle
         * @param p1 extremity of the triangle
         * @param p2 extremity of the triangle
         * @param ref variable to store the result to
         * @returns The distance between "ref" and "vector"
         */
        static ProjectOnTriangleToRef(vector: DeepImmutable<Vector3>, p0: DeepImmutable<Vector3>, p1: DeepImmutable<Vector3>, p2: DeepImmutable<Vector3>, ref: Vector3): number;
        /**
         * Returns a new Vector3 located at the center between "value1" and "value2"
         * Example Playground https://playground.babylonjs.com/#R1F8YU#72
         * @param value1 defines the first operand
         * @param value2 defines the second operand
         * @returns the new Vector3
         */
        static Center(value1: DeepImmutable<Vector3>, value2: DeepImmutable<Vector3>): Vector3;
        /**
         * Gets the center of the vectors "value1" and "value2" and stores the result in the vector "ref"
         * Example Playground https://playground.babylonjs.com/#R1F8YU#73
         * @param value1 defines first vector
         * @param value2 defines second vector
         * @param ref defines third vector
         * @returns ref
         */
        static CenterToRef<T extends Vector3>(value1: DeepImmutable<Vector3>, value2: DeepImmutable<Vector3>, ref: T): T;
        /**
         * Given three orthogonal normalized left-handed oriented Vector3 axis in space (target system),
         * RotationFromAxis() returns the rotation Euler angles (ex : rotation.x, rotation.y, rotation.z) to apply
         * to something in order to rotate it from its local system to the given target system
         * Note: axis1, axis2 and axis3 are normalized during this operation
         * Example Playground https://playground.babylonjs.com/#R1F8YU#106
         * @param axis1 defines the first axis
         * @param axis2 defines the second axis
         * @param axis3 defines the third axis
         * @returns a new Vector3
         * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/transforms/center_origin/target_align
         */
        static RotationFromAxis<T extends Vector3>(axis1: DeepImmutable<T>, axis2: DeepImmutable<Vector3>, axis3: DeepImmutable<Vector3>): T;
        /**
         * The same than RotationFromAxis but updates the given ref Vector3 parameter instead of returning a new Vector3
         * Example Playground https://playground.babylonjs.com/#R1F8YU#107
         * @param axis1 defines the first axis
         * @param axis2 defines the second axis
         * @param axis3 defines the third axis
         * @param ref defines the Vector3 where to store the result
         * @returns result input
         */
        static RotationFromAxisToRef<T extends Vector3>(axis1: DeepImmutable<Vector3>, axis2: DeepImmutable<Vector3>, axis3: DeepImmutable<Vector3>, ref: T): T;
    }
    /**
     * Vector4 class created for EulerAngle class conversion to Quaternion
     */
    export class Vector4 {
        /** x value of the vector */
        x: number;
        /** y value of the vector */
        y: number;
        /** z value of the vector */
        z: number;
        /** w value of the vector */
        w: number;
        private static _ZeroReadOnly;
        /**
         * Creates a Vector4 object from the given floats.
         * @param x x value of the vector
         * @param y y value of the vector
         * @param z z value of the vector
         * @param w w value of the vector
         */
        constructor(
        /** x value of the vector */
        x?: number, 
        /** y value of the vector */
        y?: number, 
        /** z value of the vector */
        z?: number, 
        /** w value of the vector */
        w?: number);
        /**
         * Returns the string with the Vector4 coordinates.
         * @returns a string containing all the vector values
         */
        toString(): string;
        /**
         * Returns the string "Vector4".
         * @returns "Vector4"
         */
        getClassName(): string;
        /**
         * Returns the Vector4 hash code.
         * @returns a unique hash code
         */
        getHashCode(): number;
        /**
         * Returns a new array populated with 4 elements : the Vector4 coordinates.
         * @returns the resulting array
         */
        asArray(): number[];
        /**
         * Populates the given array from the given index with the Vector4 coordinates.
         * @param array array to populate
         * @param index index of the array to start at (default: 0)
         * @returns the Vector4.
         */
        toArray(array: FloatArray, index?: number): this;
        /**
         * Update the current vector from an array
         * @param array defines the destination array
         * @param index defines the offset in the destination array
         * @returns the current Vector3
         */
        fromArray(array: FloatArray, index?: number): this;
        /**
         * Adds the given vector to the current Vector4.
         * @param otherVector the vector to add
         * @returns the updated Vector4.
         */
        addInPlace(otherVector: DeepImmutable<Vector4>): this;
        /**
         * Returns a new Vector4 as the result of the addition of the current Vector4 and the given one.
         * @param otherVector the vector to add
         * @returns the resulting vector
         */
        add(otherVector: DeepImmutable<Vector4>): this;
        /**
         * Updates the given vector "result" with the result of the addition of the current Vector4 and the given one.
         * @param otherVector the vector to add
         * @param result the vector to store the result
         * @returns result input
         */
        addToRef<T extends Vector4>(otherVector: DeepImmutable<Vector4>, result: T): T;
        /**
         * Subtract in place the given vector from the current Vector4.
         * @param otherVector the vector to subtract
         * @returns the updated Vector4.
         */
        subtractInPlace(otherVector: DeepImmutable<Vector4>): this;
        /**
         * Returns a new Vector4 with the result of the subtraction of the given vector from the current Vector4.
         * @param otherVector the vector to add
         * @returns the new vector with the result
         */
        subtract(otherVector: DeepImmutable<Vector4>): this;
        /**
         * Sets the given vector "result" with the result of the subtraction of the given vector from the current Vector4.
         * @param otherVector the vector to subtract
         * @param result the vector to store the result
         * @returns result input
         */
        subtractToRef<T extends Vector4>(otherVector: DeepImmutable<Vector4>, result: T): T;
        /**
         * Returns a new Vector4 set with the result of the subtraction of the given floats from the current Vector4 coordinates.
         */
        /**
         * Returns a new Vector4 set with the result of the subtraction of the given floats from the current Vector4 coordinates.
         * @param x value to subtract
         * @param y value to subtract
         * @param z value to subtract
         * @param w value to subtract
         * @returns new vector containing the result
         */
        subtractFromFloats(x: number, y: number, z: number, w: number): this;
        /**
         * Sets the given vector "result" set with the result of the subtraction of the given floats from the current Vector4 coordinates.
         * @param x value to subtract
         * @param y value to subtract
         * @param z value to subtract
         * @param w value to subtract
         * @param result the vector to store the result in
         * @returns result input
         */
        subtractFromFloatsToRef<T extends Vector4>(x: number, y: number, z: number, w: number, result: T): T;
        /**
         * Returns a new Vector4 set with the current Vector4 negated coordinates.
         * @returns a new vector with the negated values
         */
        negate(): this;
        /**
         * Negate this vector in place
         * @returns this
         */
        negateInPlace(): this;
        /**
         * Negate the current Vector4 and stores the result in the given vector "result" coordinates
         * @param result defines the Vector3 object where to store the result
         * @returns the result
         */
        negateToRef<T extends Vector4>(result: T): T;
        /**
         * Multiplies the current Vector4 coordinates by scale (float).
         * @param scale the number to scale with
         * @returns the updated Vector4.
         */
        scaleInPlace(scale: number): this;
        /**
         * Returns a new Vector4 set with the current Vector4 coordinates multiplied by scale (float).
         * @param scale the number to scale with
         * @returns a new vector with the result
         */
        scale(scale: number): this;
        /**
         * Sets the given vector "result" with the current Vector4 coordinates multiplied by scale (float).
         * @param scale the number to scale with
         * @param result a vector to store the result in
         * @returns result input
         */
        scaleToRef<T extends Vector4>(scale: number, result: T): T;
        /**
         * Scale the current Vector4 values by a factor and add the result to a given Vector4
         * @param scale defines the scale factor
         * @param result defines the Vector4 object where to store the result
         * @returns result input
         */
        scaleAndAddToRef<T extends Vector4>(scale: number, result: T): T;
        /**
         * Boolean : True if the current Vector4 coordinates are stricly equal to the given ones.
         * @param otherVector the vector to compare against
         * @returns true if they are equal
         */
        equals(otherVector: DeepImmutable<Vector4>): boolean;
        /**
         * Boolean : True if the current Vector4 coordinates are each beneath the distance "epsilon" from the given vector ones.
         * @param otherVector vector to compare against
         * @param epsilon (Default: very small number)
         * @returns true if they are equal
         */
        equalsWithEpsilon(otherVector: DeepImmutable<Vector4>, epsilon?: number): boolean;
        /**
         * Boolean : True if the given floats are strictly equal to the current Vector4 coordinates.
         * @param x x value to compare against
         * @param y y value to compare against
         * @param z z value to compare against
         * @param w w value to compare against
         * @returns true if equal
         */
        equalsToFloats(x: number, y: number, z: number, w: number): boolean;
        /**
         * Multiplies in place the current Vector4 by the given one.
         * @param otherVector vector to multiple with
         * @returns the updated Vector4.
         */
        multiplyInPlace(otherVector: Vector4): this;
        /**
         * Returns a new Vector4 set with the multiplication result of the current Vector4 and the given one.
         * @param otherVector vector to multiple with
         * @returns resulting new vector
         */
        multiply(otherVector: DeepImmutable<Vector4>): this;
        /**
         * Updates the given vector "result" with the multiplication result of the current Vector4 and the given one.
         * @param otherVector vector to multiple with
         * @param result vector to store the result
         * @returns result input
         */
        multiplyToRef<T extends Vector4>(otherVector: DeepImmutable<Vector4>, result: T): T;
        /**
         * Returns a new Vector4 set with the multiplication result of the given floats and the current Vector4 coordinates.
         * @param x x value multiply with
         * @param y y value multiply with
         * @param z z value multiply with
         * @param w w value multiply with
         * @returns resulting new vector
         */
        multiplyByFloats(x: number, y: number, z: number, w: number): this;
        /**
         * Returns a new Vector4 set with the division result of the current Vector4 by the given one.
         * @param otherVector vector to devide with
         * @returns resulting new vector
         */
        divide(otherVector: DeepImmutable<Vector4>): this;
        /**
         * Updates the given vector "result" with the division result of the current Vector4 by the given one.
         * @param otherVector vector to devide with
         * @param result vector to store the result
         * @returns result input
         */
        divideToRef<T extends Vector4>(otherVector: DeepImmutable<Vector4>, result: T): T;
        /**
         * Divides the current Vector3 coordinates by the given ones.
         * @param otherVector vector to devide with
         * @returns the updated Vector3.
         */
        divideInPlace(otherVector: DeepImmutable<Vector4>): this;
        /**
         * Updates the Vector4 coordinates with the minimum values between its own and the given vector ones
         * @param other defines the second operand
         * @returns the current updated Vector4
         */
        minimizeInPlace(other: DeepImmutable<Vector4>): this;
        /**
         * Updates the Vector4 coordinates with the maximum values between its own and the given vector ones
         * @param other defines the second operand
         * @returns the current updated Vector4
         */
        maximizeInPlace(other: DeepImmutable<Vector4>): this;
        /**
         * Gets a new Vector4 from current Vector4 floored values
         * @returns a new Vector4
         */
        floor(): this;
        /**
         * Gets a new Vector4 from current Vector4 fractional values
         * @returns a new Vector4
         */
        fract(): this;
        /**
         * Returns the Vector4 length (float).
         * @returns the length
         */
        length(): number;
        /**
         * Returns the Vector4 squared length (float).
         * @returns the length squared
         */
        lengthSquared(): number;
        /**
         * Normalizes in place the Vector4.
         * @returns the updated Vector4.
         */
        normalize(): this;
        /**
         * Returns a new Vector3 from the Vector4 (x, y, z) coordinates.
         * @returns this converted to a new vector3
         */
        toVector3(): Vector3;
        /**
         * Returns a new Vector4 copied from the current one.
         * @returns the new cloned vector
         */
        clone(): this;
        /**
         * Updates the current Vector4 with the given one coordinates.
         * @param source the source vector to copy from
         * @returns the updated Vector4.
         */
        copyFrom(source: DeepImmutable<Vector4>): this;
        /**
         * Updates the current Vector4 coordinates with the given floats.
         * @param x float to copy from
         * @param y float to copy from
         * @param z float to copy from
         * @param w float to copy from
         * @returns the updated Vector4.
         */
        copyFromFloats(x: number, y: number, z: number, w: number): this;
        /**
         * Updates the current Vector4 coordinates with the given floats.
         * @param x float to set from
         * @param y float to set from
         * @param z float to set from
         * @param w float to set from
         * @returns the updated Vector4.
         */
        set(x: number, y: number, z: number, w: number): this;
        /**
         * Copies the given float to the current Vector3 coordinates
         * @param v defines the x, y, z and w coordinates of the operand
         * @returns the current updated Vector3
         */
        setAll(v: number): this;
        /**
         * Returns a new Vector4 set from the starting index of the given array.
         * @param array the array to pull values from
         * @param offset the offset into the array to start at
         * @returns the new vector
         */
        static FromArray(array: DeepImmutable<ArrayLike<number>>, offset?: number): Vector4;
        /**
         * Updates the given vector "result" from the starting index of the given array.
         * @param array the array to pull values from
         * @param offset the offset into the array to start at
         * @param result the vector to store the result in
         * @returns result input
         */
        static FromArrayToRef<T extends Vector4>(array: DeepImmutable<ArrayLike<number>>, offset: number, result: T): T;
        /**
         * Updates the given vector "result" from the starting index of the given Float32Array.
         * @param array the array to pull values from
         * @param offset the offset into the array to start at
         * @param result the vector to store the result in
         * @returns result input
         */
        static FromFloatArrayToRef<T extends Vector4>(array: DeepImmutable<Float32Array>, offset: number, result: T): T;
        /**
         * Updates the given vector "result" coordinates from the given floats.
         * @param x float to set from
         * @param y float to set from
         * @param z float to set from
         * @param w float to set from
         * @param result the vector to the floats in
         * @returns result input
         */
        static FromFloatsToRef<T extends Vector4>(x: number, y: number, z: number, w: number, result: T): T;
        /**
         * Returns a new Vector4 set to (0.0, 0.0, 0.0, 0.0)
         * @returns the new vector
         */
        static Zero(): Vector4;
        /**
         * Returns a new Vector4 set to (1.0, 1.0, 1.0, 1.0)
         * @returns the new vector
         */
        static One(): Vector4;
        /**
         * Gets a zero Vector4 that must not be updated
         */
        static get ZeroReadOnly(): DeepImmutable<Vector4>;
        /**
         * Returns a new normalized Vector4 from the given one.
         * @param vector the vector to normalize
         * @returns the vector
         */
        static Normalize(vector: DeepImmutable<Vector4>): Vector4;
        /**
         * Updates the given vector "result" from the normalization of the given one.
         * @param vector the vector to normalize
         * @param result the vector to store the result in
         * @returns result input
         */
        static NormalizeToRef<T extends Vector4>(vector: DeepImmutable<Vector4>, result: T): T;
        /**
         * Returns a vector with the minimum values from the left and right vectors
         * @param left left vector to minimize
         * @param right right vector to minimize
         * @returns a new vector with the minimum of the left and right vector values
         */
        static Minimize<T extends Vector4>(left: DeepImmutable<T>, right: DeepImmutable<Vector4>): T;
        /**
         * Returns a vector with the maximum values from the left and right vectors
         * @param left left vector to maximize
         * @param right right vector to maximize
         * @returns a new vector with the maximum of the left and right vector values
         */
        static Maximize<T extends Vector4>(left: DeepImmutable<T>, right: DeepImmutable<Vector4>): T;
        /**
         * Returns the distance (float) between the vectors "value1" and "value2".
         * @param value1 value to calulate the distance between
         * @param value2 value to calulate the distance between
         * @returns the distance between the two vectors
         */
        static Distance(value1: DeepImmutable<Vector4>, value2: DeepImmutable<Vector4>): number;
        /**
         * Returns the squared distance (float) between the vectors "value1" and "value2".
         * @param value1 value to calulate the distance between
         * @param value2 value to calulate the distance between
         * @returns the distance between the two vectors squared
         */
        static DistanceSquared(value1: DeepImmutable<Vector4>, value2: DeepImmutable<Vector4>): number;
        /**
         * Returns a new Vector4 located at the center between the vectors "value1" and "value2".
         * @param value1 value to calulate the center between
         * @param value2 value to calulate the center between
         * @returns the center between the two vectors
         */
        static Center(value1: DeepImmutable<Vector4>, value2: DeepImmutable<Vector4>): Vector4;
        /**
         * Gets the center of the vectors "value1" and "value2" and stores the result in the vector "ref"
         * @param value1 defines first vector
         * @param value2 defines second vector
         * @param ref defines third vector
         * @returns ref
         */
        static CenterToRef<T extends Vector4>(value1: DeepImmutable<Vector4>, value2: DeepImmutable<Vector4>, ref: T): T;
        /**
         * Returns a new Vector4 set with the result of the transformation by the given matrix of the given vector.
         * This method computes tranformed coordinates only, not transformed direction vectors (ie. it takes translation in account)
         * The difference with Vector3.TransformCoordinates is that the w component is not used to divide the other coordinates but is returned in the w coordinate instead
         * @param vector defines the Vector3 to transform
         * @param transformation defines the transformation matrix
         * @returns the transformed Vector4
         */
        static TransformCoordinates(vector: DeepImmutable<Vector3>, transformation: DeepImmutable<Matrix>): Vector4;
        /**
         * Sets the given vector "result" coordinates with the result of the transformation by the given matrix of the given vector
         * This method computes tranformed coordinates only, not transformed direction vectors (ie. it takes translation in account)
         * The difference with Vector3.TransformCoordinatesToRef is that the w component is not used to divide the other coordinates but is returned in the w coordinate instead
         * @param vector defines the Vector3 to transform
         * @param transformation defines the transformation matrix
         * @param result defines the Vector4 where to store the result
         * @returns result input
         */
        static TransformCoordinatesToRef<T extends Vector4>(vector: DeepImmutable<Vector3>, transformation: DeepImmutable<Matrix>, result: T): T;
        /**
         * Sets the given vector "result" coordinates with the result of the transformation by the given matrix of the given floats (x, y, z)
         * This method computes tranformed coordinates only, not transformed direction vectors
         * The difference with Vector3.TransformCoordinatesFromFloatsToRef is that the w component is not used to divide the other coordinates but is returned in the w coordinate instead
         * @param x define the x coordinate of the source vector
         * @param y define the y coordinate of the source vector
         * @param z define the z coordinate of the source vector
         * @param transformation defines the transformation matrix
         * @param result defines the Vector4 where to store the result
         * @returns result input
         */
        static TransformCoordinatesFromFloatsToRef<T extends Vector4>(x: number, y: number, z: number, transformation: DeepImmutable<Matrix>, result: T): T;
        /**
         * Returns a new Vector4 set with the result of the normal transformation by the given matrix of the given vector.
         * This methods computes transformed normalized direction vectors only.
         * @param vector the vector to transform
         * @param transformation the transformation matrix to apply
         * @returns the new vector
         */
        static TransformNormal<T extends Vector4>(vector: DeepImmutable<T>, transformation: DeepImmutable<Matrix>): T;
        /**
         * Sets the given vector "result" with the result of the normal transformation by the given matrix of the given vector.
         * This methods computes transformed normalized direction vectors only.
         * @param vector the vector to transform
         * @param transformation the transformation matrix to apply
         * @param result the vector to store the result in
         * @returns result input
         */
        static TransformNormalToRef<T extends Vector4>(vector: DeepImmutable<Vector4>, transformation: DeepImmutable<Matrix>, result: T): T;
        /**
         * Sets the given vector "result" with the result of the normal transformation by the given matrix of the given floats (x, y, z, w).
         * This methods computes transformed normalized direction vectors only.
         * @param x value to transform
         * @param y value to transform
         * @param z value to transform
         * @param w value to transform
         * @param transformation the transformation matrix to apply
         * @param result the vector to store the results in
         * @returns result input
         */
        static TransformNormalFromFloatsToRef<T extends Vector4>(x: number, y: number, z: number, w: number, transformation: DeepImmutable<Matrix>, result: T): T;
        /**
         * Creates a new Vector4 from a Vector3
         * @param source defines the source data
         * @param w defines the 4th component (default is 0)
         * @returns a new Vector4
         */
        static FromVector3(source: Vector3, w?: number): Vector4;
    }
    /**
     * Class used to store quaternion data
     * Example Playground - Overview - https://playground.babylonjs.com/#L49EJ7#100
     * @see https://en.wikipedia.org/wiki/Quaternion
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/transforms
     */
    export class Quaternion {
        /** @internal */
        _x: number;
        /** @internal */
        _y: number;
        /** @internal */
        _z: number;
        /** @internal */
        _w: number;
        /** @internal */
        _isDirty: boolean;
        /** Gets or sets the x coordinate */
        get x(): number;
        set x(value: number);
        /** Gets or sets the y coordinate */
        get y(): number;
        set y(value: number);
        /** Gets or sets the z coordinate */
        get z(): number;
        set z(value: number);
        /** Gets or sets the w coordinate */
        get w(): number;
        set w(value: number);
        /**
         * Creates a new Quaternion from the given floats
         * @param x defines the first component (0 by default)
         * @param y defines the second component (0 by default)
         * @param z defines the third component (0 by default)
         * @param w defines the fourth component (1.0 by default)
         */
        constructor(x?: number, y?: number, z?: number, w?: number);
        /**
         * Gets a string representation for the current quaternion
         * @returns a string with the Quaternion coordinates
         */
        toString(): string;
        /**
         * Gets the class name of the quaternion
         * @returns the string "Quaternion"
         */
        getClassName(): string;
        /**
         * Gets a hash code for this quaternion
         * @returns the quaternion hash code
         */
        getHashCode(): number;
        /**
         * Copy the quaternion to an array
         * Example Playground https://playground.babylonjs.com/#L49EJ7#13
         * @returns a new array populated with 4 elements from the quaternion coordinates
         */
        asArray(): number[];
        /**
         * Stores from the starting index in the given array the Quaternion successive values
         * Example Playground https://playground.babylonjs.com/#L49EJ7#59
         * @param array defines the array where to store the x,y,z,w components
         * @param index defines an optional index in the target array to define where to start storing values
         * @returns the current Quaternion object
         */
        toArray(array: FloatArray, index?: number): Quaternion;
        /**
         * Check if two quaternions are equals
         * Example Playground https://playground.babylonjs.com/#L49EJ7#38
         * @param otherQuaternion defines the second operand
         * @returns true if the current quaternion and the given one coordinates are strictly equals
         */
        equals(otherQuaternion: DeepImmutable<Quaternion>): boolean;
        /**
         * Gets a boolean if two quaternions are equals (using an epsilon value)
         * Example Playground https://playground.babylonjs.com/#L49EJ7#37
         * @param otherQuaternion defines the other quaternion
         * @param epsilon defines the minimal distance to consider equality
         * @returns true if the given quaternion coordinates are close to the current ones by a distance of epsilon.
         */
        equalsWithEpsilon(otherQuaternion: DeepImmutable<Quaternion>, epsilon?: number): boolean;
        /**
         * Clone the current quaternion
         * Example Playground https://playground.babylonjs.com/#L49EJ7#12
         * @returns a new quaternion copied from the current one
         */
        clone(): this;
        /**
         * Copy a quaternion to the current one
         * Example Playground https://playground.babylonjs.com/#L49EJ7#86
         * @param other defines the other quaternion
         * @returns the updated current quaternion
         */
        copyFrom(other: DeepImmutable<Quaternion>): this;
        /**
         * Updates the current quaternion with the given float coordinates
         * Example Playground https://playground.babylonjs.com/#L49EJ7#87
         * @param x defines the x coordinate
         * @param y defines the y coordinate
         * @param z defines the z coordinate
         * @param w defines the w coordinate
         * @returns the updated current quaternion
         */
        copyFromFloats(x: number, y: number, z: number, w: number): this;
        /**
         * Updates the current quaternion from the given float coordinates
         * Example Playground https://playground.babylonjs.com/#L49EJ7#56
         * @param x defines the x coordinate
         * @param y defines the y coordinate
         * @param z defines the z coordinate
         * @param w defines the w coordinate
         * @returns the updated current quaternion
         */
        set(x: number, y: number, z: number, w: number): this;
        /**
         * Adds two quaternions
         * Example Playground https://playground.babylonjs.com/#L49EJ7#10
         * @param other defines the second operand
         * @returns a new quaternion as the addition result of the given one and the current quaternion
         */
        add(other: DeepImmutable<Quaternion>): this;
        /**
         * Add a quaternion to the current one
         * Example Playground https://playground.babylonjs.com/#L49EJ7#11
         * @param other defines the quaternion to add
         * @returns the current quaternion
         */
        addInPlace(other: DeepImmutable<Quaternion>): this;
        /**
         * Subtract two quaternions
         * Example Playground https://playground.babylonjs.com/#L49EJ7#57
         * @param other defines the second operand
         * @returns a new quaternion as the subtraction result of the given one from the current one
         */
        subtract(other: Quaternion): this;
        /**
         * Subtract a quaternion to the current one
         * Example Playground https://playground.babylonjs.com/#L49EJ7#58
         * @param other defines the quaternion to subtract
         * @returns the current quaternion
         */
        subtractInPlace(other: DeepImmutable<Quaternion>): this;
        /**
         * Multiplies the current quaternion by a scale factor
         * Example Playground https://playground.babylonjs.com/#L49EJ7#88
         * @param value defines the scale factor
         * @returns a new quaternion set by multiplying the current quaternion coordinates by the float "scale"
         */
        scale(value: number): this;
        /**
         * Scale the current quaternion values by a factor and stores the result to a given quaternion
         * Example Playground https://playground.babylonjs.com/#L49EJ7#89
         * @param scale defines the scale factor
         * @param result defines the Quaternion object where to store the result
         * @returns result input
         */
        scaleToRef<T extends Quaternion>(scale: number, result: T): T;
        /**
         * Multiplies in place the current quaternion by a scale factor
         * Example Playground https://playground.babylonjs.com/#L49EJ7#90
         * @param value defines the scale factor
         * @returns the current modified quaternion
         */
        scaleInPlace(value: number): this;
        /**
         * Scale the current quaternion values by a factor and add the result to a given quaternion
         * Example Playground https://playground.babylonjs.com/#L49EJ7#91
         * @param scale defines the scale factor
         * @param result defines the Quaternion object where to store the result
         * @returns result input
         */
        scaleAndAddToRef<T extends Quaternion>(scale: number, result: T): T;
        /**
         * Multiplies two quaternions
         * Example Playground https://playground.babylonjs.com/#L49EJ7#43
         * @param q1 defines the second operand
         * @returns a new quaternion set as the multiplication result of the current one with the given one "q1"
         */
        multiply(q1: DeepImmutable<Quaternion>): this;
        /**
         * Sets the given "result" as the the multiplication result of the current one with the given one "q1"
         * Example Playground https://playground.babylonjs.com/#L49EJ7#45
         * @param q1 defines the second operand
         * @param result defines the target quaternion
         * @returns the current quaternion
         */
        multiplyToRef<T extends Quaternion>(q1: DeepImmutable<Quaternion>, result: T): T;
        /**
         * Updates the current quaternion with the multiplication of itself with the given one "q1"
         * Example Playground https://playground.babylonjs.com/#L49EJ7#46
         * @param q1 defines the second operand
         * @returns the currentupdated quaternion
         */
        multiplyInPlace(q1: DeepImmutable<Quaternion>): this;
        /**
         * Conjugates the current quaternion and stores the result in the given quaternion
         * Example Playground https://playground.babylonjs.com/#L49EJ7#81
         * @param ref defines the target quaternion
         * @returns result input
         */
        conjugateToRef<T extends Quaternion>(ref: T): T;
        /**
         * Conjugates in place the current quaternion
         * Example Playground https://playground.babylonjs.com/#L49EJ7#82
         * @returns the current updated quaternion
         */
        conjugateInPlace(): this;
        /**
         * Conjugates (1-q) the current quaternion
         * Example Playground https://playground.babylonjs.com/#L49EJ7#83
         * @returns a new quaternion
         */
        conjugate(): this;
        /**
         * Returns the inverse of the current quaternion
         * Example Playground https://playground.babylonjs.com/#L49EJ7#84
         * @returns a new quaternion
         */
        invert(): this;
        /**
         * Invert in place the current quaternion
         * Example Playground https://playground.babylonjs.com/#L49EJ7#85
         * @returns this quaternion
         */
        invertInPlace(): this;
        /**
         * Gets squared length of current quaternion
         * Example Playground https://playground.babylonjs.com/#L49EJ7#29
         * @returns the quaternion length (float)
         */
        lengthSquared(): number;
        /**
         * Gets length of current quaternion
         * Example Playground https://playground.babylonjs.com/#L49EJ7#28
         * @returns the quaternion length (float)
         */
        length(): number;
        /**
         * Normalize in place the current quaternion
         * Example Playground https://playground.babylonjs.com/#L49EJ7#54
         * @returns the current updated quaternion
         */
        normalize(): this;
        /**
         * Normalize a copy of the current quaternion
         * Example Playground https://playground.babylonjs.com/#L49EJ7#55
         * @returns the normalized quaternion
         */
        normalizeToNew(): this;
        /**
         * Returns a new Vector3 set with the Euler angles translated from the current quaternion
         * Example Playground https://playground.babylonjs.com/#L49EJ7#32
         * @returns a new Vector3 containing the Euler angles
         * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/transforms/center_origin/rotation_conventions
         */
        toEulerAngles(): Vector3;
        /**
         * Sets the given vector3 "result" with the Euler angles translated from the current quaternion
         * Example Playground https://playground.babylonjs.com/#L49EJ7#31
         * @param result defines the vector which will be filled with the Euler angles
         * @returns result input
         * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/transforms/center_origin/rotation_conventions
         */
        toEulerAnglesToRef<T extends Vector3>(result: T): T;
        /**
         * Updates the given rotation matrix with the current quaternion values
         * Example Playground https://playground.babylonjs.com/#L49EJ7#67
         * @param result defines the target matrix
         * @returns the current unchanged quaternion
         */
        toRotationMatrix<T extends Matrix>(result: T): T;
        /**
         * Updates the current quaternion from the given rotation matrix values
         * Example Playground https://playground.babylonjs.com/#L49EJ7#41
         * @param matrix defines the source matrix
         * @returns the current updated quaternion
         */
        fromRotationMatrix(matrix: DeepImmutable<Matrix>): this;
        /**
         * Creates a new quaternion from a rotation matrix
         * Example Playground https://playground.babylonjs.com/#L49EJ7#101
         * @param matrix defines the source matrix
         * @returns a new quaternion created from the given rotation matrix values
         */
        static FromRotationMatrix(matrix: DeepImmutable<Matrix>): Quaternion;
        /**
         * Updates the given quaternion with the given rotation matrix values
         * Example Playground https://playground.babylonjs.com/#L49EJ7#102
         * @param matrix defines the source matrix
         * @param result defines the target quaternion
         * @returns result input
         */
        static FromRotationMatrixToRef<T extends Quaternion>(matrix: DeepImmutable<Matrix>, result: T): T;
        /**
         * Returns the dot product (float) between the quaternions "left" and "right"
         * Example Playground https://playground.babylonjs.com/#L49EJ7#61
         * @param left defines the left operand
         * @param right defines the right operand
         * @returns the dot product
         */
        static Dot(left: DeepImmutable<Quaternion>, right: DeepImmutable<Quaternion>): number;
        /**
         * Checks if the orientations of two rotation quaternions are close to each other
         * Example Playground https://playground.babylonjs.com/#L49EJ7#60
         * @param quat0 defines the first quaternion to check
         * @param quat1 defines the second quaternion to check
         * @param epsilon defines closeness, 0 same orientation, 1 PI apart, default 0.1
         * @returns true if the two quaternions are close to each other within epsilon
         */
        static AreClose(quat0: DeepImmutable<Quaternion>, quat1: DeepImmutable<Quaternion>, epsilon?: number): boolean;
        /**
         * Smooth interpolation between two quaternions using Slerp
         * Example Playground https://playground.babylonjs.com/#L49EJ7#93
         * @param source source quaternion
         * @param goal goal quaternion
         * @param deltaTime current interpolation frame
         * @param lerpTime total interpolation time
         * @param result the smoothed quaternion
         */
        static SmoothToRef<T extends Quaternion>(source: Quaternion, goal: Quaternion, deltaTime: number, lerpTime: number, result: T): T;
        /**
         * Creates an empty quaternion
         * @returns a new quaternion set to (0.0, 0.0, 0.0)
         */
        static Zero(): Quaternion;
        /**
         * Inverse a given quaternion
         * Example Playground https://playground.babylonjs.com/#L49EJ7#103
         * @param q defines the source quaternion
         * @returns a new quaternion as the inverted current quaternion
         */
        static Inverse<T extends Quaternion>(q: DeepImmutable<T>): T;
        /**
         * Inverse a given quaternion
         * Example Playground https://playground.babylonjs.com/#L49EJ7#104
         * @param q defines the source quaternion
         * @param result the quaternion the result will be stored in
         * @returns the result quaternion
         */
        static InverseToRef<T extends Quaternion>(q: Quaternion, result: T): T;
        /**
         * Creates an identity quaternion
         * @returns the identity quaternion
         */
        static Identity(): Quaternion;
        /**
         * Gets a boolean indicating if the given quaternion is identity
         * @param quaternion defines the quaternion to check
         * @returns true if the quaternion is identity
         */
        static IsIdentity(quaternion: DeepImmutable<Quaternion>): boolean;
        /**
         * Creates a quaternion from a rotation around an axis
         * Example Playground https://playground.babylonjs.com/#L49EJ7#72
         * @param axis defines the axis to use
         * @param angle defines the angle to use
         * @returns a new quaternion created from the given axis (Vector3) and angle in radians (float)
         */
        static RotationAxis(axis: DeepImmutable<Vector3>, angle: number): Quaternion;
        /**
         * Creates a rotation around an axis and stores it into the given quaternion
         * Example Playground https://playground.babylonjs.com/#L49EJ7#73
         * @param axis defines the axis to use
         * @param angle defines the angle to use
         * @param result defines the target quaternion
         * @returns the target quaternion
         */
        static RotationAxisToRef<T extends Quaternion>(axis: DeepImmutable<Vector3>, angle: number, result: T): T;
        /**
         * Creates a new quaternion from data stored into an array
         * Example Playground https://playground.babylonjs.com/#L49EJ7#63
         * @param array defines the data source
         * @param offset defines the offset in the source array where the data starts
         * @returns a new quaternion
         */
        static FromArray(array: DeepImmutable<ArrayLike<number>>, offset?: number): Quaternion;
        /**
         * Updates the given quaternion "result" from the starting index of the given array.
         * Example Playground https://playground.babylonjs.com/#L49EJ7#64
         * @param array the array to pull values from
         * @param offset the offset into the array to start at
         * @param result the quaternion to store the result in
         * @returns result input
         */
        static FromArrayToRef<T extends Quaternion>(array: DeepImmutable<ArrayLike<number>>, offset: number, result: T): T;
        /**
         * Create a quaternion from Euler rotation angles
         * Example Playground https://playground.babylonjs.com/#L49EJ7#33
         * @param x Pitch
         * @param y Yaw
         * @param z Roll
         * @returns the new Quaternion
         */
        static FromEulerAngles(x: number, y: number, z: number): Quaternion;
        /**
         * Updates a quaternion from Euler rotation angles
         * Example Playground https://playground.babylonjs.com/#L49EJ7#34
         * @param x Pitch
         * @param y Yaw
         * @param z Roll
         * @param result the quaternion to store the result
         * @returns the updated quaternion
         */
        static FromEulerAnglesToRef<T extends Quaternion>(x: number, y: number, z: number, result: T): T;
        /**
         * Create a quaternion from Euler rotation vector
         * Example Playground https://playground.babylonjs.com/#L49EJ7#35
         * @param vec the Euler vector (x Pitch, y Yaw, z Roll)
         * @returns the new Quaternion
         */
        static FromEulerVector(vec: DeepImmutable<Vector3>): Quaternion;
        /**
         * Updates a quaternion from Euler rotation vector
         * Example Playground https://playground.babylonjs.com/#L49EJ7#36
         * @param vec the Euler vector (x Pitch, y Yaw, z Roll)
         * @param result the quaternion to store the result
         * @returns the updated quaternion
         */
        static FromEulerVectorToRef<T extends Quaternion>(vec: DeepImmutable<Vector3>, result: T): T;
        /**
         * Updates a quaternion so that it rotates vector vecFrom to vector vecTo
         * Example Playground - https://playground.babylonjs.com/#L49EJ7#70
         * @param vecFrom defines the direction vector from which to rotate
         * @param vecTo defines the direction vector to which to rotate
         * @param result the quaternion to store the result
         * @returns the updated quaternion
         */
        static FromUnitVectorsToRef<T extends Quaternion>(vecFrom: DeepImmutable<Vector3>, vecTo: DeepImmutable<Vector3>, result: T): T;
        /**
         * Creates a new quaternion from the given Euler float angles (y, x, z)
         * Example Playground https://playground.babylonjs.com/#L49EJ7#77
         * @param yaw defines the rotation around Y axis
         * @param pitch defines the rotation around X axis
         * @param roll defines the rotation around Z axis
         * @returns the new quaternion
         */
        static RotationYawPitchRoll(yaw: number, pitch: number, roll: number): Quaternion;
        /**
         * Creates a new rotation from the given Euler float angles (y, x, z) and stores it in the target quaternion
         * Example Playground https://playground.babylonjs.com/#L49EJ7#78
         * @param yaw defines the rotation around Y axis
         * @param pitch defines the rotation around X axis
         * @param roll defines the rotation around Z axis
         * @param result defines the target quaternion
         * @returns result input
         */
        static RotationYawPitchRollToRef<T extends Quaternion>(yaw: number, pitch: number, roll: number, result: T): T;
        /**
         * Creates a new quaternion from the given Euler float angles expressed in z-x-z orientation
         * Example Playground https://playground.babylonjs.com/#L49EJ7#68
         * @param alpha defines the rotation around first axis
         * @param beta defines the rotation around second axis
         * @param gamma defines the rotation around third axis
         * @returns the new quaternion
         */
        static RotationAlphaBetaGamma(alpha: number, beta: number, gamma: number): Quaternion;
        /**
         * Creates a new quaternion from the given Euler float angles expressed in z-x-z orientation and stores it in the target quaternion
         * Example Playground https://playground.babylonjs.com/#L49EJ7#69
         * @param alpha defines the rotation around first axis
         * @param beta defines the rotation around second axis
         * @param gamma defines the rotation around third axis
         * @param result defines the target quaternion
         * @returns result input
         */
        static RotationAlphaBetaGammaToRef<T extends Quaternion>(alpha: number, beta: number, gamma: number, result: T): T;
        /**
         * Creates a new quaternion containing the rotation value to reach the target (axis1, axis2, axis3) orientation as a rotated XYZ system (axis1, axis2 and axis3 are normalized during this operation)
         * Example Playground https://playground.babylonjs.com/#L49EJ7#75
         * @param axis1 defines the first axis
         * @param axis2 defines the second axis
         * @param axis3 defines the third axis
         * @returns the new quaternion
         */
        static RotationQuaternionFromAxis(axis1: DeepImmutable<Vector3>, axis2: DeepImmutable<Vector3>, axis3: DeepImmutable<Vector3>): Quaternion;
        /**
         * Creates a rotation value to reach the target (axis1, axis2, axis3) orientation as a rotated XYZ system (axis1, axis2 and axis3 are normalized during this operation) and stores it in the target quaternion
         * Example Playground https://playground.babylonjs.com/#L49EJ7#76
         * @param axis1 defines the first axis
         * @param axis2 defines the second axis
         * @param axis3 defines the third axis
         * @param ref defines the target quaternion
         * @returns result input
         */
        static RotationQuaternionFromAxisToRef<T extends Quaternion>(axis1: DeepImmutable<Vector3>, axis2: DeepImmutable<Vector3>, axis3: DeepImmutable<Vector3>, ref: T): T;
        /**
         * Creates a new rotation value to orient an object to look towards the given forward direction, the up direction being oriented like "up".
         * This function works in left handed mode
         * Example Playground https://playground.babylonjs.com/#L49EJ7#96
         * @param forward defines the forward direction - Must be normalized and orthogonal to up.
         * @param up defines the up vector for the entity - Must be normalized and orthogonal to forward.
         * @returns A new quaternion oriented toward the specified forward and up.
         */
        static FromLookDirectionLH(forward: DeepImmutable<Vector3>, up: DeepImmutable<Vector3>): Quaternion;
        /**
         * Creates a new rotation value to orient an object to look towards the given forward direction with the up direction being oriented like "up", and stores it in the target quaternion.
         * This function works in left handed mode
         * Example Playground https://playground.babylonjs.com/#L49EJ7#97
         * @param forward defines the forward direction - Must be normalized and orthogonal to up.
         * @param up defines the up vector for the entity - Must be normalized and orthogonal to forward.
         * @param ref defines the target quaternion.
         * @returns result input
         */
        static FromLookDirectionLHToRef<T extends Quaternion>(forward: DeepImmutable<Vector3>, up: DeepImmutable<Vector3>, ref: T): T;
        /**
         * Creates a new rotation value to orient an object to look towards the given forward direction, the up direction being oriented like "up".
         * This function works in right handed mode
         * Example Playground https://playground.babylonjs.com/#L49EJ7#98
         * @param forward defines the forward direction - Must be normalized and orthogonal to up.
         * @param up defines the up vector for the entity - Must be normalized and orthogonal to forward.
         * @returns A new quaternion oriented toward the specified forward and up.
         */
        static FromLookDirectionRH(forward: DeepImmutable<Vector3>, up: DeepImmutable<Vector3>): Quaternion;
        /**
         * Creates a new rotation value to orient an object to look towards the given forward direction with the up direction being oriented like "up", and stores it in the target quaternion.
         * This function works in right handed mode
         * Example Playground https://playground.babylonjs.com/#L49EJ7#105
         * @param forward defines the forward direction - Must be normalized and orthogonal to up.
         * @param up defines the up vector for the entity - Must be normalized and orthogonal to forward.
         * @param ref defines the target quaternion.
         * @returns result input
         */
        static FromLookDirectionRHToRef<T extends Quaternion>(forward: DeepImmutable<Vector3>, up: DeepImmutable<Vector3>, ref: T): T;
        /**
         * Interpolates between two quaternions
         * Example Playground https://playground.babylonjs.com/#L49EJ7#79
         * @param left defines first quaternion
         * @param right defines second quaternion
         * @param amount defines the gradient to use
         * @returns the new interpolated quaternion
         */
        static Slerp(left: DeepImmutable<Quaternion>, right: DeepImmutable<Quaternion>, amount: number): Quaternion;
        /**
         * Interpolates between two quaternions and stores it into a target quaternion
         * Example Playground https://playground.babylonjs.com/#L49EJ7#92
         * @param left defines first quaternion
         * @param right defines second quaternion
         * @param amount defines the gradient to use
         * @param result defines the target quaternion
         * @returns result input
         */
        static SlerpToRef<T extends Quaternion>(left: DeepImmutable<Quaternion>, right: DeepImmutable<Quaternion>, amount: number, result: T): T;
        /**
         * Interpolate between two quaternions using Hermite interpolation
         * Example Playground https://playground.babylonjs.com/#L49EJ7#47
         * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/drawCurves#hermite-quaternion-spline
         * @param value1 defines first quaternion
         * @param tangent1 defines the incoming tangent
         * @param value2 defines second quaternion
         * @param tangent2 defines the outgoing tangent
         * @param amount defines the target quaternion
         * @returns the new interpolated quaternion
         */
        static Hermite<T extends Quaternion>(value1: DeepImmutable<T>, tangent1: DeepImmutable<Quaternion>, value2: DeepImmutable<Quaternion>, tangent2: DeepImmutable<Quaternion>, amount: number): T;
        /**
         * Returns a new Quaternion which is the 1st derivative of the Hermite spline defined by the quaternions "value1", "value2", "tangent1", "tangent2".
         * Example Playground https://playground.babylonjs.com/#L49EJ7#48
         * @param value1 defines the first control point
         * @param tangent1 defines the first tangent
         * @param value2 defines the second control point
         * @param tangent2 defines the second tangent
         * @param time define where the derivative must be done
         * @returns 1st derivative
         */
        static Hermite1stDerivative<T extends Quaternion>(value1: DeepImmutable<T>, tangent1: DeepImmutable<Quaternion>, value2: DeepImmutable<Quaternion>, tangent2: DeepImmutable<Quaternion>, time: number): T;
        /**
         * Update a Quaternion with the 1st derivative of the Hermite spline defined by the quaternions "value1", "value2", "tangent1", "tangent2".
         * Example Playground https://playground.babylonjs.com/#L49EJ7#49
         * @param value1 defines the first control point
         * @param tangent1 defines the first tangent
         * @param value2 defines the second control point
         * @param tangent2 defines the second tangent
         * @param time define where the derivative must be done
         * @param result define where to store the derivative
         * @returns result input
         */
        static Hermite1stDerivativeToRef<T extends Quaternion>(value1: DeepImmutable<Quaternion>, tangent1: DeepImmutable<Quaternion>, value2: DeepImmutable<Quaternion>, tangent2: DeepImmutable<Quaternion>, time: number, result: T): T;
    }
    /**
     * Class used to store matrix data (4x4)
     * Note on matrix definitions in Babylon.js for setting values directly
     * rather than using one of the methods available.
     * Matrix size is given by rows x columns.
     * A Vector3 is a 1 X 3 matrix [x, y, z].
     *
     * In Babylon.js multiplying a 1 x 3 matrix by a 4 x 4 matrix
     * is done using BABYLON.Vector4.TransformCoordinates(Vector3, Matrix).
     * and extending the passed Vector3 to a Vector4, V = [x, y, z, 1].
     * Let M be a matrix with elements m(row, column), so that
     * m(2, 3) is the element in row 2 column 3 of M.
     *
     * Multiplication is of the form VM and has the resulting Vector4
     * VM = [xm(0, 0) + ym(1, 0) + zm(2, 0) + m(3, 0), xm(0, 1) + ym(1, 1) + zm(2, 1) + m(3, 1), xm(0, 2) + ym(1, 2) + zm(2, 2) + m(3, 2), xm(0, 3) + ym(1, 3) + zm(2, 3) + m(3, 3)].
     * On the web you will find many examples that use the opposite convention of MV,
     * in which case to make use of the examples you will need to transpose the matrix.
     *
     * Example Playground - Overview Linear Algebra - https://playground.babylonjs.com/#AV9X17
     * Example Playground - Overview Transformation - https://playground.babylonjs.com/#AV9X17#1
     * Example Playground - Overview Projection - https://playground.babylonjs.com/#AV9X17#2
     */
    export class Matrix {
        /**
         * Gets the precision of matrix computations
         */
        static get Use64Bits(): boolean;
        private static _UpdateFlagSeed;
        private static _IdentityReadOnly;
        private _isIdentity;
        private _isIdentityDirty;
        private _isIdentity3x2;
        private _isIdentity3x2Dirty;
        /**
         * Gets the update flag of the matrix which is an unique number for the matrix.
         * It will be incremented every time the matrix data change.
         * You can use it to speed the comparison between two versions of the same matrix.
         */
        updateFlag: number;
        private readonly _m;
        /**
         * Gets the internal data of the matrix
         */
        get m(): DeepImmutable<Float32Array | Array<number>>;
        /**
         * Update the updateFlag to indicate that the matrix has been updated
         */
        markAsUpdated(): void;
        private _updateIdentityStatus;
        /**
         * Creates an empty matrix (filled with zeros)
         */
        constructor();
        /**
         * Check if the current matrix is identity
         * @returns true is the matrix is the identity matrix
         */
        isIdentity(): boolean;
        /**
         * Check if the current matrix is identity as a texture matrix (3x2 store in 4x4)
         * @returns true is the matrix is the identity matrix
         */
        isIdentityAs3x2(): boolean;
        /**
         * Gets the determinant of the matrix
         * Example Playground - https://playground.babylonjs.com/#AV9X17#34
         * @returns the matrix determinant
         */
        determinant(): number;
        /**
         * Returns the matrix as a Float32Array or Array<number>
         * Example Playground - https://playground.babylonjs.com/#AV9X17#49
         * @returns the matrix underlying array
         */
        toArray(): DeepImmutable<Float32Array | Array<number>>;
        /**
         * Returns the matrix as a Float32Array or Array<number>
         * Example Playground - https://playground.babylonjs.com/#AV9X17#114
         * @returns the matrix underlying array.
         */
        asArray(): DeepImmutable<Float32Array | Array<number>>;
        /**
         * Inverts the current matrix in place
         * Example Playground - https://playground.babylonjs.com/#AV9X17#118
         * @returns the current inverted matrix
         */
        invert(): this;
        /**
         * Sets all the matrix elements to zero
         * @returns the current matrix
         */
        reset(): this;
        /**
         * Adds the current matrix with a second one
         * Example Playground - https://playground.babylonjs.com/#AV9X17#44
         * @param other defines the matrix to add
         * @returns a new matrix as the addition of the current matrix and the given one
         */
        add(other: DeepImmutable<Matrix>): this;
        /**
         * Sets the given matrix "result" to the addition of the current matrix and the given one
         * Example Playground - https://playground.babylonjs.com/#AV9X17#45
         * @param other defines the matrix to add
         * @param result defines the target matrix
         * @returns result input
         */
        addToRef<T extends Matrix>(other: DeepImmutable<Matrix>, result: T): T;
        /**
         * Adds in place the given matrix to the current matrix
         * Example Playground - https://playground.babylonjs.com/#AV9X17#46
         * @param other defines the second operand
         * @returns the current updated matrix
         */
        addToSelf(other: DeepImmutable<Matrix>): this;
        /**
         * Sets the given matrix to the current inverted Matrix
         * Example Playground - https://playground.babylonjs.com/#AV9X17#119
         * @param other defines the target matrix
         * @returns result input
         */
        invertToRef<T extends Matrix>(other: T): T;
        /**
         * add a value at the specified position in the current Matrix
         * Example Playground - https://playground.babylonjs.com/#AV9X17#47
         * @param index the index of the value within the matrix. between 0 and 15.
         * @param value the value to be added
         * @returns the current updated matrix
         */
        addAtIndex(index: number, value: number): this;
        /**
         * mutiply the specified position in the current Matrix by a value
         * @param index the index of the value within the matrix. between 0 and 15.
         * @param value the value to be added
         * @returns the current updated matrix
         */
        multiplyAtIndex(index: number, value: number): this;
        /**
         * Inserts the translation vector (using 3 floats) in the current matrix
         * Example Playground - https://playground.babylonjs.com/#AV9X17#120
         * @param x defines the 1st component of the translation
         * @param y defines the 2nd component of the translation
         * @param z defines the 3rd component of the translation
         * @returns the current updated matrix
         */
        setTranslationFromFloats(x: number, y: number, z: number): this;
        /**
         * Adds the translation vector (using 3 floats) in the current matrix
         * Example Playground - https://playground.babylonjs.com/#AV9X17#20
         * Example Playground - https://playground.babylonjs.com/#AV9X17#48
         * @param x defines the 1st component of the translation
         * @param y defines the 2nd component of the translation
         * @param z defines the 3rd component of the translation
         * @returns the current updated matrix
         */
        addTranslationFromFloats(x: number, y: number, z: number): this;
        /**
         * Inserts the translation vector in the current matrix
         * Example Playground - https://playground.babylonjs.com/#AV9X17#121
         * @param vector3 defines the translation to insert
         * @returns the current updated matrix
         */
        setTranslation(vector3: DeepImmutable<Vector3>): this;
        /**
         * Gets the translation value of the current matrix
         * Example Playground - https://playground.babylonjs.com/#AV9X17#122
         * @returns a new Vector3 as the extracted translation from the matrix
         */
        getTranslation(): Vector3;
        /**
         * Fill a Vector3 with the extracted translation from the matrix
         * Example Playground - https://playground.babylonjs.com/#AV9X17#123
         * @param result defines the Vector3 where to store the translation
         * @returns the current matrix
         */
        getTranslationToRef<T extends Vector3>(result: T): T;
        /**
         * Remove rotation and scaling part from the matrix
         * @returns the updated matrix
         */
        removeRotationAndScaling(): this;
        /**
         * Multiply two matrices
         * Example Playground - https://playground.babylonjs.com/#AV9X17#15
         * A.multiply(B) means apply B to A so result is B x A
         * @param other defines the second operand
         * @returns a new matrix set with the multiplication result of the current Matrix and the given one
         */
        multiply(other: DeepImmutable<Matrix>): this;
        /**
         * Copy the current matrix from the given one
         * Example Playground - https://playground.babylonjs.com/#AV9X17#21
         * @param other defines the source matrix
         * @returns the current updated matrix
         */
        copyFrom(other: DeepImmutable<Matrix>): this;
        /**
         * Populates the given array from the starting index with the current matrix values
         * @param array defines the target array
         * @param offset defines the offset in the target array where to start storing values
         * @returns the current matrix
         */
        copyToArray(array: Float32Array | Array<number>, offset?: number): this;
        /**
         * Sets the given matrix "result" with the multiplication result of the current Matrix and the given one
         * A.multiplyToRef(B, R) means apply B to A and store in R and R = B x A
         * Example Playground - https://playground.babylonjs.com/#AV9X17#16
         * @param other defines the second operand
         * @param result defines the matrix where to store the multiplication
         * @returns result input
         */
        multiplyToRef<T extends Matrix>(other: DeepImmutable<Matrix>, result: T): T;
        /**
         * Sets the Float32Array "result" from the given index "offset" with the multiplication of the current matrix and the given one
         * @param other defines the second operand
         * @param result defines the array where to store the multiplication
         * @param offset defines the offset in the target array where to start storing values
         * @returns the current matrix
         */
        multiplyToArray(other: DeepImmutable<Matrix>, result: Float32Array | Array<number>, offset: number): this;
        /**
         * Check equality between this matrix and a second one
         * @param value defines the second matrix to compare
         * @returns true is the current matrix and the given one values are strictly equal
         */
        equals(value: DeepImmutable<Matrix>): boolean;
        /**
         * Clone the current matrix
         * Example Playground - https://playground.babylonjs.com/#AV9X17#18
         * @returns a new matrix from the current matrix
         */
        clone(): this;
        /**
         * Returns the name of the current matrix class
         * @returns the string "Matrix"
         */
        getClassName(): string;
        /**
         * Gets the hash code of the current matrix
         * @returns the hash code
         */
        getHashCode(): number;
        /**
         * Decomposes the current Matrix into a translation, rotation and scaling components of the provided node
         * Example Playground - https://playground.babylonjs.com/#AV9X17#13
         * @param node the node to decompose the matrix to
         * @returns true if operation was successful
         */
        decomposeToTransformNode(node: TransformNode): boolean;
        /**
         * Decomposes the current Matrix into a translation, rotation and scaling components
         * Example Playground - https://playground.babylonjs.com/#AV9X17#12
         * @param scale defines the scale vector3 given as a reference to update
         * @param rotation defines the rotation quaternion given as a reference to update
         * @param translation defines the translation vector3 given as a reference to update
         * @param preserveScalingNode Use scaling sign coming from this node. Otherwise scaling sign might change.
         * @returns true if operation was successful
         */
        decompose(scale?: Vector3, rotation?: Quaternion, translation?: Vector3, preserveScalingNode?: TransformNode): boolean;
        /**
         * Gets specific row of the matrix
         * Example Playground - https://playground.babylonjs.com/#AV9X17#36
         * @param index defines the number of the row to get
         * @returns the index-th row of the current matrix as a new Vector4
         */
        getRow(index: number): Nullable<Vector4>;
        /**
         * Gets specific row of the matrix to ref
         * Example Playground - https://playground.babylonjs.com/#AV9X17#36
         * @param index defines the number of the row to get
         * @param rowVector vector to store the index-th row of the current matrix
         * @returns result input
         */
        getRowToRef<T extends Vector4>(index: number, rowVector: T): T;
        /**
         * Sets the index-th row of the current matrix to the vector4 values
         * Example Playground - https://playground.babylonjs.com/#AV9X17#36
         * @param index defines the number of the row to set
         * @param row defines the target vector4
         * @returns the updated current matrix
         */
        setRow(index: number, row: Vector4): this;
        /**
         * Compute the transpose of the matrix
         * Example Playground - https://playground.babylonjs.com/#AV9X17#40
         * @returns the new transposed matrix
         */
        transpose(): this;
        /**
         * Compute the transpose of the matrix and store it in a given matrix
         * Example Playground - https://playground.babylonjs.com/#AV9X17#41
         * @param result defines the target matrix
         * @returns result input
         */
        transposeToRef<T extends Matrix>(result: T): T;
        /**
         * Sets the index-th row of the current matrix with the given 4 x float values
         * Example Playground - https://playground.babylonjs.com/#AV9X17#36
         * @param index defines the row index
         * @param x defines the x component to set
         * @param y defines the y component to set
         * @param z defines the z component to set
         * @param w defines the w component to set
         * @returns the updated current matrix
         */
        setRowFromFloats(index: number, x: number, y: number, z: number, w: number): this;
        /**
         * Compute a new matrix set with the current matrix values multiplied by scale (float)
         * @param scale defines the scale factor
         * @returns a new matrix
         */
        scale(scale: number): this;
        /**
         * Scale the current matrix values by a factor to a given result matrix
         * @param scale defines the scale factor
         * @param result defines the matrix to store the result
         * @returns result input
         */
        scaleToRef<T extends Matrix>(scale: number, result: T): T;
        /**
         * Scale the current matrix values by a factor and add the result to a given matrix
         * @param scale defines the scale factor
         * @param result defines the Matrix to store the result
         * @returns result input
         */
        scaleAndAddToRef<T extends Matrix>(scale: number, result: T): T;
        /**
         * Writes to the given matrix a normal matrix, computed from this one (using values from identity matrix for fourth row and column).
         * Example Playground - https://playground.babylonjs.com/#AV9X17#17
         * @param ref matrix to store the result
         */
        toNormalMatrix<T extends Matrix>(ref: T): T;
        /**
         * Gets only rotation part of the current matrix
         * @returns a new matrix sets to the extracted rotation matrix from the current one
         */
        getRotationMatrix(): this;
        /**
         * Extracts the rotation matrix from the current one and sets it as the given "result"
         * @param result defines the target matrix to store data to
         * @returns result input
         */
        getRotationMatrixToRef<T extends Matrix>(result: T): T;
        /**
         * Toggles model matrix from being right handed to left handed in place and vice versa
         */
        toggleModelMatrixHandInPlace(): this;
        /**
         * Toggles projection matrix from being right handed to left handed in place and vice versa
         */
        toggleProjectionMatrixHandInPlace(): this;
        /**
         * Creates a matrix from an array
         * Example Playground - https://playground.babylonjs.com/#AV9X17#42
         * @param array defines the source array
         * @param offset defines an offset in the source array
         * @returns a new Matrix set from the starting index of the given array
         */
        static FromArray(array: DeepImmutable<ArrayLike<number>>, offset?: number): Matrix;
        /**
         * Copy the content of an array into a given matrix
         * Example Playground - https://playground.babylonjs.com/#AV9X17#43
         * @param array defines the source array
         * @param offset defines an offset in the source array
         * @param result defines the target matrix
         * @returns result input
         */
        static FromArrayToRef<T extends Matrix>(array: DeepImmutable<ArrayLike<number>>, offset: number, result: T): T;
        /**
         * Stores an array into a matrix after having multiplied each component by a given factor
         * Example Playground - https://playground.babylonjs.com/#AV9X17#50
         * @param array defines the source array
         * @param offset defines the offset in the source array
         * @param scale defines the scaling factor
         * @param result defines the target matrix
         * @returns result input
         */
        static FromFloat32ArrayToRefScaled<T extends Matrix>(array: DeepImmutable<Float32Array | Array<number>>, offset: number, scale: number, result: T): T;
        /**
         * Gets an identity matrix that must not be updated
         */
        static get IdentityReadOnly(): DeepImmutable<Matrix>;
        /**
         * Stores a list of values (16) inside a given matrix
         * @param initialM11 defines 1st value of 1st row
         * @param initialM12 defines 2nd value of 1st row
         * @param initialM13 defines 3rd value of 1st row
         * @param initialM14 defines 4th value of 1st row
         * @param initialM21 defines 1st value of 2nd row
         * @param initialM22 defines 2nd value of 2nd row
         * @param initialM23 defines 3rd value of 2nd row
         * @param initialM24 defines 4th value of 2nd row
         * @param initialM31 defines 1st value of 3rd row
         * @param initialM32 defines 2nd value of 3rd row
         * @param initialM33 defines 3rd value of 3rd row
         * @param initialM34 defines 4th value of 3rd row
         * @param initialM41 defines 1st value of 4th row
         * @param initialM42 defines 2nd value of 4th row
         * @param initialM43 defines 3rd value of 4th row
         * @param initialM44 defines 4th value of 4th row
         * @param result defines the target matrix
         * @returns result input
         */
        static FromValuesToRef(initialM11: number, initialM12: number, initialM13: number, initialM14: number, initialM21: number, initialM22: number, initialM23: number, initialM24: number, initialM31: number, initialM32: number, initialM33: number, initialM34: number, initialM41: number, initialM42: number, initialM43: number, initialM44: number, result: Matrix): void;
        /**
         * Creates new matrix from a list of values (16)
         * @param initialM11 defines 1st value of 1st row
         * @param initialM12 defines 2nd value of 1st row
         * @param initialM13 defines 3rd value of 1st row
         * @param initialM14 defines 4th value of 1st row
         * @param initialM21 defines 1st value of 2nd row
         * @param initialM22 defines 2nd value of 2nd row
         * @param initialM23 defines 3rd value of 2nd row
         * @param initialM24 defines 4th value of 2nd row
         * @param initialM31 defines 1st value of 3rd row
         * @param initialM32 defines 2nd value of 3rd row
         * @param initialM33 defines 3rd value of 3rd row
         * @param initialM34 defines 4th value of 3rd row
         * @param initialM41 defines 1st value of 4th row
         * @param initialM42 defines 2nd value of 4th row
         * @param initialM43 defines 3rd value of 4th row
         * @param initialM44 defines 4th value of 4th row
         * @returns the new matrix
         */
        static FromValues(initialM11: number, initialM12: number, initialM13: number, initialM14: number, initialM21: number, initialM22: number, initialM23: number, initialM24: number, initialM31: number, initialM32: number, initialM33: number, initialM34: number, initialM41: number, initialM42: number, initialM43: number, initialM44: number): Matrix;
        /**
         * Creates a new matrix composed by merging scale (vector3), rotation (quaternion) and translation (vector3)
         * Example Playground - https://playground.babylonjs.com/#AV9X17#24
         * @param scale defines the scale vector3
         * @param rotation defines the rotation quaternion
         * @param translation defines the translation vector3
         * @returns a new matrix
         */
        static Compose(scale: DeepImmutable<Vector3>, rotation: DeepImmutable<Quaternion>, translation: DeepImmutable<Vector3>): Matrix;
        /**
         * Sets a matrix to a value composed by merging scale (vector3), rotation (quaternion) and translation (vector3)
         * Example Playground - https://playground.babylonjs.com/#AV9X17#25
         * @param scale defines the scale vector3
         * @param rotation defines the rotation quaternion
         * @param translation defines the translation vector3
         * @param result defines the target matrix
         * @returns result input
         */
        static ComposeToRef<T extends Matrix>(scale: DeepImmutable<Vector3>, rotation: DeepImmutable<Quaternion>, translation: DeepImmutable<Vector3>, result: T): T;
        /**
         * Creates a new identity matrix
         * @returns a new identity matrix
         */
        static Identity(): Matrix;
        /**
         * Creates a new identity matrix and stores the result in a given matrix
         * @param result defines the target matrix
         * @returns result input
         */
        static IdentityToRef<T extends Matrix>(result: T): T;
        /**
         * Creates a new zero matrix
         * @returns a new zero matrix
         */
        static Zero(): Matrix;
        /**
         * Creates a new rotation matrix for "angle" radians around the X axis
         * Example Playground - https://playground.babylonjs.com/#AV9X17#97
         * @param angle defines the angle (in radians) to use
         * @returns the new matrix
         */
        static RotationX(angle: number): Matrix;
        /**
         * Creates a new matrix as the invert of a given matrix
         * Example Playground - https://playground.babylonjs.com/#AV9X17#124
         * @param source defines the source matrix
         * @returns the new matrix
         */
        static Invert<T extends Matrix>(source: DeepImmutable<T>): T;
        /**
         * Creates a new rotation matrix for "angle" radians around the X axis and stores it in a given matrix
         * Example Playground - https://playground.babylonjs.com/#AV9X17#98
         * @param angle defines the angle (in radians) to use
         * @param result defines the target matrix
         * @returns result input
         */
        static RotationXToRef<T extends Matrix>(angle: number, result: T): T;
        /**
         * Creates a new rotation matrix for "angle" radians around the Y axis
         * Example Playground - https://playground.babylonjs.com/#AV9X17#99
         * @param angle defines the angle (in radians) to use
         * @returns the new matrix
         */
        static RotationY(angle: number): Matrix;
        /**
         * Creates a new rotation matrix for "angle" radians around the Y axis and stores it in a given matrix
         * Example Playground - https://playground.babylonjs.com/#AV9X17#100
         * @param angle defines the angle (in radians) to use
         * @param result defines the target matrix
         * @returns result input
         */
        static RotationYToRef<T extends Matrix>(angle: number, result: T): T;
        /**
         * Creates a new rotation matrix for "angle" radians around the Z axis
         * Example Playground - https://playground.babylonjs.com/#AV9X17#101
         * @param angle defines the angle (in radians) to use
         * @returns the new matrix
         */
        static RotationZ(angle: number): Matrix;
        /**
         * Creates a new rotation matrix for "angle" radians around the Z axis and stores it in a given matrix
         * Example Playground - https://playground.babylonjs.com/#AV9X17#102
         * @param angle defines the angle (in radians) to use
         * @param result defines the target matrix
         * @returns result input
         */
        static RotationZToRef<T extends Matrix>(angle: number, result: T): T;
        /**
         * Creates a new rotation matrix for "angle" radians around the given axis
         * Example Playground - https://playground.babylonjs.com/#AV9X17#96
         * @param axis defines the axis to use
         * @param angle defines the angle (in radians) to use
         * @returns the new matrix
         */
        static RotationAxis(axis: DeepImmutable<Vector3>, angle: number): Matrix;
        /**
         * Creates a new rotation matrix for "angle" radians around the given axis and stores it in a given matrix
         * Example Playground - https://playground.babylonjs.com/#AV9X17#94
         * @param axis defines the axis to use
         * @param angle defines the angle (in radians) to use
         * @param result defines the target matrix
         * @returns result input
         */
        static RotationAxisToRef<T extends Matrix>(axis: DeepImmutable<Vector3>, angle: number, result: T): T;
        /**
         * Takes normalised vectors and returns a rotation matrix to align "from" with "to".
         * Taken from http://www.iquilezles.org/www/articles/noacos/noacos.htm
         * Example Playground - https://playground.babylonjs.com/#AV9X17#93
         * @param from defines the vector to align
         * @param to defines the vector to align to
         * @param result defines the target matrix
         * @returns result input
         */
        static RotationAlignToRef<T extends Matrix>(from: DeepImmutable<Vector3>, to: DeepImmutable<Vector3>, result: T): T;
        /**
         * Creates a rotation matrix
         * Example Playground - https://playground.babylonjs.com/#AV9X17#103
         * Example Playground - https://playground.babylonjs.com/#AV9X17#105
         * @param yaw defines the yaw angle in radians (Y axis)
         * @param pitch defines the pitch angle in radians (X axis)
         * @param roll defines the roll angle in radians (Z axis)
         * @returns the new rotation matrix
         */
        static RotationYawPitchRoll(yaw: number, pitch: number, roll: number): Matrix;
        /**
         * Creates a rotation matrix and stores it in a given matrix
         * Example Playground - https://playground.babylonjs.com/#AV9X17#104
         * @param yaw defines the yaw angle in radians (Y axis)
         * @param pitch defines the pitch angle in radians (X axis)
         * @param roll defines the roll angle in radians (Z axis)
         * @param result defines the target matrix
         * @returns result input
         */
        static RotationYawPitchRollToRef<T extends Matrix>(yaw: number, pitch: number, roll: number, result: T): T;
        /**
         * Creates a scaling matrix
         * Example Playground - https://playground.babylonjs.com/#AV9X17#107
         * @param x defines the scale factor on X axis
         * @param y defines the scale factor on Y axis
         * @param z defines the scale factor on Z axis
         * @returns the new matrix
         */
        static Scaling(x: number, y: number, z: number): Matrix;
        /**
         * Creates a scaling matrix and stores it in a given matrix
         * Example Playground - https://playground.babylonjs.com/#AV9X17#108
         * @param x defines the scale factor on X axis
         * @param y defines the scale factor on Y axis
         * @param z defines the scale factor on Z axis
         * @param result defines the target matrix
         * @returns result input
         */
        static ScalingToRef<T extends Matrix>(x: number, y: number, z: number, result: T): T;
        /**
         * Creates a translation matrix
         * Example Playground - https://playground.babylonjs.com/#AV9X17#109
         * @param x defines the translation on X axis
         * @param y defines the translation on Y axis
         * @param z defines the translationon Z axis
         * @returns the new matrix
         */
        static Translation(x: number, y: number, z: number): Matrix;
        /**
         * Creates a translation matrix and stores it in a given matrix
         * Example Playground - https://playground.babylonjs.com/#AV9X17#110
         * @param x defines the translation on X axis
         * @param y defines the translation on Y axis
         * @param z defines the translationon Z axis
         * @param result defines the target matrix
         * @returns result input
         */
        static TranslationToRef<T extends Matrix>(x: number, y: number, z: number, result: T): T;
        /**
         * Returns a new Matrix whose values are the interpolated values for "gradient" (float) between the ones of the matrices "startValue" and "endValue".
         * Example Playground - https://playground.babylonjs.com/#AV9X17#55
         * @param startValue defines the start value
         * @param endValue defines the end value
         * @param gradient defines the gradient factor
         * @returns the new matrix
         */
        static Lerp<T extends Matrix>(startValue: DeepImmutable<T>, endValue: DeepImmutable<Matrix>, gradient: number): T;
        /**
         * Set the given matrix "result" as the interpolated values for "gradient" (float) between the ones of the matrices "startValue" and "endValue".
         * Example Playground - https://playground.babylonjs.com/#AV9X17#54
         * @param startValue defines the start value
         * @param endValue defines the end value
         * @param gradient defines the gradient factor
         * @param result defines the Matrix object where to store data
         * @returns result input
         */
        static LerpToRef<T extends Matrix>(startValue: DeepImmutable<Matrix>, endValue: DeepImmutable<Matrix>, gradient: number, result: T): T;
        /**
         * Builds a new matrix whose values are computed by:
         * * decomposing the the "startValue" and "endValue" matrices into their respective scale, rotation and translation matrices
         * * interpolating for "gradient" (float) the values between each of these decomposed matrices between the start and the end
         * * recomposing a new matrix from these 3 interpolated scale, rotation and translation matrices
         * Example Playground - https://playground.babylonjs.com/#AV9X17#22
         * Example Playground - https://playground.babylonjs.com/#AV9X17#51
         * @param startValue defines the first matrix
         * @param endValue defines the second matrix
         * @param gradient defines the gradient between the two matrices
         * @returns the new matrix
         */
        static DecomposeLerp<T extends Matrix>(startValue: DeepImmutable<T>, endValue: DeepImmutable<Matrix>, gradient: number): T;
        /**
         * Update a matrix to values which are computed by:
         * * decomposing the the "startValue" and "endValue" matrices into their respective scale, rotation and translation matrices
         * * interpolating for "gradient" (float) the values between each of these decomposed matrices between the start and the end
         * * recomposing a new matrix from these 3 interpolated scale, rotation and translation matrices
         * Example Playground - https://playground.babylonjs.com/#AV9X17#23
         * Example Playground - https://playground.babylonjs.com/#AV9X17#53
         * @param startValue defines the first matrix
         * @param endValue defines the second matrix
         * @param gradient defines the gradient between the two matrices
         * @param result defines the target matrix
         * @returns result input
         */
        static DecomposeLerpToRef<T extends Matrix>(startValue: DeepImmutable<Matrix>, endValue: DeepImmutable<Matrix>, gradient: number, result: T): T;
        /**
         * Creates a new matrix that transforms vertices from world space to camera space. It takes three vectors as arguments that together describe the position and orientation of the camera.
         * This function generates a matrix suitable for a left handed coordinate system
         * Example Playground - https://playground.babylonjs.com/#AV9X17#58
         * Example Playground - https://playground.babylonjs.com/#AV9X17#59
         * @param eye defines the final position of the entity
         * @param target defines where the entity should look at
         * @param up defines the up vector for the entity
         * @returns the new matrix
         */
        static LookAtLH(eye: DeepImmutable<Vector3>, target: DeepImmutable<Vector3>, up: DeepImmutable<Vector3>): Matrix;
        /**
         * Sets the given "result" Matrix to a matrix that transforms vertices from world space to camera space. It takes three vectors as arguments that together describe the position and orientation of the camera.
         * This function generates a matrix suitable for a left handed coordinate system
         * Example Playground - https://playground.babylonjs.com/#AV9X17#60
         * Example Playground - https://playground.babylonjs.com/#AV9X17#61
         * @param eye defines the final position of the entity
         * @param target defines where the entity should look at
         * @param up defines the up vector for the entity
         * @param result defines the target matrix
         * @returns result input
         */
        static LookAtLHToRef(eye: DeepImmutable<Vector3>, target: DeepImmutable<Vector3>, up: DeepImmutable<Vector3>, result: Matrix): void;
        /**
         * Creates a new matrix that transforms vertices from world space to camera space. It takes three vectors as arguments that together describe the position and orientation of the camera.
         * This function generates a matrix suitable for a right handed coordinate system
         * Example Playground - https://playground.babylonjs.com/#AV9X17#62
         * Example Playground - https://playground.babylonjs.com/#AV9X17#63
         * @param eye defines the final position of the entity
         * @param target defines where the entity should look at
         * @param up defines the up vector for the entity
         * @returns the new matrix
         */
        static LookAtRH(eye: DeepImmutable<Vector3>, target: DeepImmutable<Vector3>, up: DeepImmutable<Vector3>): Matrix;
        /**
         * Sets the given "result" Matrix to a matrix that transforms vertices from world space to camera space. It takes three vectors as arguments that together describe the position and orientation of the camera.
         * This function generates a matrix suitable for a right handed coordinate system
         * Example Playground - https://playground.babylonjs.com/#AV9X17#64
         * Example Playground - https://playground.babylonjs.com/#AV9X17#65
         * @param eye defines the final position of the entity
         * @param target defines where the entity should look at
         * @param up defines the up vector for the entity
         * @param result defines the target matrix
         * @returns result input
         */
        static LookAtRHToRef<T extends Matrix>(eye: DeepImmutable<Vector3>, target: DeepImmutable<Vector3>, up: DeepImmutable<Vector3>, result: T): T;
        /**
         * Creates a new matrix that transforms vertices from world space to camera space. It takes two vectors as arguments that together describe the orientation of the camera. The position is assumed to be at the origin (0,0,0)
         * This function generates a matrix suitable for a left handed coordinate system
         * Example Playground - https://playground.babylonjs.com/#AV9X17#66
         * @param forward defines the forward direction - Must be normalized and orthogonal to up.
         * @param up defines the up vector for the entity - Must be normalized and orthogonal to forward.
         * @returns the new matrix
         */
        static LookDirectionLH(forward: DeepImmutable<Vector3>, up: DeepImmutable<Vector3>): Matrix;
        /**
         * Sets the given "result" Matrix to a matrix that transforms vertices from world space to camera space. It takes two vectors as arguments that together describe the orientation of the camera. The position is assumed to be at the origin (0,0,0)
         * This function generates a matrix suitable for a left handed coordinate system
         * Example Playground - https://playground.babylonjs.com/#AV9X17#67
         * @param forward defines the forward direction - Must be normalized and orthogonal to up.
         * @param up defines the up vector for the entity - Must be normalized and orthogonal to forward.
         * @param result defines the target matrix
         * @returns result input
         */
        static LookDirectionLHToRef<T extends Matrix>(forward: DeepImmutable<Vector3>, up: DeepImmutable<Vector3>, result: T): T;
        /**
         * Creates a new matrix that transforms vertices from world space to camera space. It takes two vectors as arguments that together describe the orientation of the camera. The position is assumed to be at the origin (0,0,0)
         * This function generates a matrix suitable for a right handed coordinate system
         * Example Playground - https://playground.babylonjs.com/#AV9X17#68
         * @param forward defines the forward direction - Must be normalized and orthogonal to up.
         * @param up defines the up vector for the entity - Must be normalized and orthogonal to forward.
         * @returns the new matrix
         */
        static LookDirectionRH(forward: DeepImmutable<Vector3>, up: DeepImmutable<Vector3>): Matrix;
        /**
         * Sets the given "result" Matrix to a matrix that transforms vertices from world space to camera space. It takes two vectors as arguments that together describe the orientation of the camera. The position is assumed to be at the origin (0,0,0)
         * This function generates a matrix suitable for a right handed coordinate system
         * Example Playground - https://playground.babylonjs.com/#AV9X17#69
         * @param forward defines the forward direction - Must be normalized and orthogonal to up.
         * @param up defines the up vector for the entity - Must be normalized and orthogonal to forward.
         * @param result defines the target matrix
         * @returns result input
         */
        static LookDirectionRHToRef<T extends Matrix>(forward: DeepImmutable<Vector3>, up: DeepImmutable<Vector3>, result: T): T;
        /**
         * Create a left-handed orthographic projection matrix
         * Example Playground - https://playground.babylonjs.com/#AV9X17#70
         * @param width defines the viewport width
         * @param height defines the viewport height
         * @param znear defines the near clip plane
         * @param zfar defines the far clip plane
         * @param halfZRange true to generate NDC coordinates between 0 and 1 instead of -1 and 1 (default: false)
         * @returns a new matrix as a left-handed orthographic projection matrix
         */
        static OrthoLH(width: number, height: number, znear: number, zfar: number, halfZRange?: boolean): Matrix;
        /**
         * Store a left-handed orthographic projection to a given matrix
         * Example Playground - https://playground.babylonjs.com/#AV9X17#71
         * @param width defines the viewport width
         * @param height defines the viewport height
         * @param znear defines the near clip plane
         * @param zfar defines the far clip plane
         * @param result defines the target matrix
         * @param halfZRange true to generate NDC coordinates between 0 and 1 instead of -1 and 1 (default: false)
         * @returns result input
         */
        static OrthoLHToRef<T extends Matrix>(width: number, height: number, znear: number, zfar: number, result: T, halfZRange?: boolean): T;
        /**
         * Create a left-handed orthographic projection matrix
         * Example Playground - https://playground.babylonjs.com/#AV9X17#72
         * @param left defines the viewport left coordinate
         * @param right defines the viewport right coordinate
         * @param bottom defines the viewport bottom coordinate
         * @param top defines the viewport top coordinate
         * @param znear defines the near clip plane
         * @param zfar defines the far clip plane
         * @param halfZRange true to generate NDC coordinates between 0 and 1 instead of -1 and 1 (default: false)
         * @returns a new matrix as a left-handed orthographic projection matrix
         */
        static OrthoOffCenterLH(left: number, right: number, bottom: number, top: number, znear: number, zfar: number, halfZRange?: boolean): Matrix;
        /**
         * Stores a left-handed orthographic projection into a given matrix
         * Example Playground - https://playground.babylonjs.com/#AV9X17#73
         * @param left defines the viewport left coordinate
         * @param right defines the viewport right coordinate
         * @param bottom defines the viewport bottom coordinate
         * @param top defines the viewport top coordinate
         * @param znear defines the near clip plane
         * @param zfar defines the far clip plane
         * @param result defines the target matrix
         * @param halfZRange true to generate NDC coordinates between 0 and 1 instead of -1 and 1 (default: false)
         * @returns result input
         */
        static OrthoOffCenterLHToRef<T extends Matrix>(left: number, right: number, bottom: number, top: number, znear: number, zfar: number, result: T, halfZRange?: boolean): T;
        /**
         * Creates a right-handed orthographic projection matrix
         * Example Playground - https://playground.babylonjs.com/#AV9X17#76
         * @param left defines the viewport left coordinate
         * @param right defines the viewport right coordinate
         * @param bottom defines the viewport bottom coordinate
         * @param top defines the viewport top coordinate
         * @param znear defines the near clip plane
         * @param zfar defines the far clip plane
         * @param halfZRange true to generate NDC coordinates between 0 and 1 instead of -1 and 1 (default: false)
         * @returns a new matrix as a right-handed orthographic projection matrix
         */
        static OrthoOffCenterRH(left: number, right: number, bottom: number, top: number, znear: number, zfar: number, halfZRange?: boolean): Matrix;
        /**
         * Stores a right-handed orthographic projection into a given matrix
         * Example Playground - https://playground.babylonjs.com/#AV9X17#77
         * @param left defines the viewport left coordinate
         * @param right defines the viewport right coordinate
         * @param bottom defines the viewport bottom coordinate
         * @param top defines the viewport top coordinate
         * @param znear defines the near clip plane
         * @param zfar defines the far clip plane
         * @param result defines the target matrix
         * @param halfZRange true to generate NDC coordinates between 0 and 1 instead of -1 and 1 (default: false)
         * @returns result input
         */
        static OrthoOffCenterRHToRef<T extends Matrix>(left: number, right: number, bottom: number, top: number, znear: number, zfar: number, result: T, halfZRange?: boolean): T;
        /**
         * Creates a left-handed perspective projection matrix
         * Example Playground - https://playground.babylonjs.com/#AV9X17#85
         * @param width defines the viewport width
         * @param height defines the viewport height
         * @param znear defines the near clip plane
         * @param zfar defines the far clip plane
         * @param halfZRange true to generate NDC coordinates between 0 and 1 instead of -1 and 1 (default: false)
         * @param projectionPlaneTilt optional tilt angle of the projection plane around the X axis (horizontal)
         * @returns a new matrix as a left-handed perspective projection matrix
         */
        static PerspectiveLH(width: number, height: number, znear: number, zfar: number, halfZRange?: boolean, projectionPlaneTilt?: number): Matrix;
        /**
         * Creates a left-handed perspective projection matrix
         * Example Playground - https://playground.babylonjs.com/#AV9X17#78
         * @param fov defines the horizontal field of view
         * @param aspect defines the aspect ratio
         * @param znear defines the near clip plane
         * @param zfar defines the far clip plane. If 0, assume we are in "infinite zfar" mode
         * @param halfZRange true to generate NDC coordinates between 0 and 1 instead of -1 and 1 (default: false)
         * @param projectionPlaneTilt optional tilt angle of the projection plane around the X axis (horizontal)
         * @param reverseDepthBufferMode true to indicate that we are in a reverse depth buffer mode (meaning znear and zfar have been inverted when calling the function)
         * @returns a new matrix as a left-handed perspective projection matrix
         */
        static PerspectiveFovLH(fov: number, aspect: number, znear: number, zfar: number, halfZRange?: boolean, projectionPlaneTilt?: number, reverseDepthBufferMode?: boolean): Matrix;
        /**
         * Stores a left-handed perspective projection into a given matrix
         * Example Playground - https://playground.babylonjs.com/#AV9X17#81
         * @param fov defines the horizontal field of view
         * @param aspect defines the aspect ratio
         * @param znear defines the near clip plane
         * @param zfar defines the far clip plane. If 0, assume we are in "infinite zfar" mode
         * @param result defines the target matrix
         * @param isVerticalFovFixed defines it the fov is vertically fixed (default) or horizontally
         * @param halfZRange true to generate NDC coordinates between 0 and 1 instead of -1 and 1 (default: false)
         * @param projectionPlaneTilt optional tilt angle of the projection plane around the X axis (horizontal)
         * @param reverseDepthBufferMode true to indicate that we are in a reverse depth buffer mode (meaning znear and zfar have been inverted when calling the function)
         * @returns result input
         */
        static PerspectiveFovLHToRef<T extends Matrix>(fov: number, aspect: number, znear: number, zfar: number, result: T, isVerticalFovFixed?: boolean, halfZRange?: boolean, projectionPlaneTilt?: number, reverseDepthBufferMode?: boolean): T;
        /**
         * Stores a left-handed perspective projection into a given matrix with depth reversed
         * Example Playground - https://playground.babylonjs.com/#AV9X17#89
         * @param fov defines the horizontal field of view
         * @param aspect defines the aspect ratio
         * @param znear defines the near clip plane
         * @param zfar not used as infinity is used as far clip
         * @param result defines the target matrix
         * @param isVerticalFovFixed defines it the fov is vertically fixed (default) or horizontally
         * @param halfZRange true to generate NDC coordinates between 0 and 1 instead of -1 and 1 (default: false)
         * @param projectionPlaneTilt optional tilt angle of the projection plane around the X axis (horizontal)
         * @returns result input
         */
        static PerspectiveFovReverseLHToRef<T extends Matrix>(fov: number, aspect: number, znear: number, zfar: number, result: T, isVerticalFovFixed?: boolean, halfZRange?: boolean, projectionPlaneTilt?: number): T;
        /**
         * Creates a right-handed perspective projection matrix
         * Example Playground - https://playground.babylonjs.com/#AV9X17#83
         * @param fov defines the horizontal field of view
         * @param aspect defines the aspect ratio
         * @param znear defines the near clip plane
         * @param zfar defines the far clip plane. If 0, assume we are in "infinite zfar" mode
         * @param halfZRange true to generate NDC coordinates between 0 and 1 instead of -1 and 1 (default: false)
         * @param projectionPlaneTilt optional tilt angle of the projection plane around the X axis (horizontal)
         * @param reverseDepthBufferMode true to indicate that we are in a reverse depth buffer mode (meaning znear and zfar have been inverted when calling the function)
         * @returns a new matrix as a right-handed perspective projection matrix
         */
        static PerspectiveFovRH(fov: number, aspect: number, znear: number, zfar: number, halfZRange?: boolean, projectionPlaneTilt?: number, reverseDepthBufferMode?: boolean): Matrix;
        /**
         * Stores a right-handed perspective projection into a given matrix
         * Example Playground - https://playground.babylonjs.com/#AV9X17#84
         * @param fov defines the horizontal field of view
         * @param aspect defines the aspect ratio
         * @param znear defines the near clip plane
         * @param zfar defines the far clip plane. If 0, assume we are in "infinite zfar" mode
         * @param result defines the target matrix
         * @param isVerticalFovFixed defines it the fov is vertically fixed (default) or horizontally
         * @param halfZRange true to generate NDC coordinates between 0 and 1 instead of -1 and 1 (default: false)
         * @param projectionPlaneTilt optional tilt angle of the projection plane around the X axis (horizontal)
         * @param reverseDepthBufferMode true to indicate that we are in a reverse depth buffer mode (meaning znear and zfar have been inverted when calling the function)
         * @returns result input
         */
        static PerspectiveFovRHToRef<T extends Matrix>(fov: number, aspect: number, znear: number, zfar: number, result: T, isVerticalFovFixed?: boolean, halfZRange?: boolean, projectionPlaneTilt?: number, reverseDepthBufferMode?: boolean): T;
        /**
         * Stores a right-handed perspective projection into a given matrix
         * Example Playground - https://playground.babylonjs.com/#AV9X17#90
         * @param fov defines the horizontal field of view
         * @param aspect defines the aspect ratio
         * @param znear defines the near clip plane
         * @param zfar not used as infinity is used as far clip
         * @param result defines the target matrix
         * @param isVerticalFovFixed defines it the fov is vertically fixed (default) or horizontally
         * @param halfZRange true to generate NDC coordinates between 0 and 1 instead of -1 and 1 (default: false)
         * @param projectionPlaneTilt optional tilt angle of the projection plane around the X axis (horizontal)
         * @returns result input
         */
        static PerspectiveFovReverseRHToRef<T extends Matrix>(fov: number, aspect: number, znear: number, zfar: number, result: T, isVerticalFovFixed?: boolean, halfZRange?: boolean, projectionPlaneTilt?: number): T;
        /**
         * Stores a perspective projection for WebVR info a given matrix
         * Example Playground - https://playground.babylonjs.com/#AV9X17#92
         * @param fov defines the field of view
         * @param fov.upDegrees
         * @param fov.downDegrees
         * @param fov.leftDegrees
         * @param fov.rightDegrees
         * @param znear defines the near clip plane
         * @param zfar defines the far clip plane
         * @param result defines the target matrix
         * @param rightHanded defines if the matrix must be in right-handed mode (false by default)
         * @param halfZRange true to generate NDC coordinates between 0 and 1 instead of -1 and 1 (default: false)
         * @param projectionPlaneTilt optional tilt angle of the projection plane around the X axis (horizontal)
         * @returns result input
         */
        static PerspectiveFovWebVRToRef<T extends Matrix>(fov: {
            upDegrees: number;
            downDegrees: number;
            leftDegrees: number;
            rightDegrees: number;
        }, znear: number, zfar: number, result: T, rightHanded?: boolean, halfZRange?: boolean, projectionPlaneTilt?: number): T;
        /**
         * Computes a complete transformation matrix
         * Example Playground - https://playground.babylonjs.com/#AV9X17#113
         * @param viewport defines the viewport to use
         * @param world defines the world matrix
         * @param view defines the view matrix
         * @param projection defines the projection matrix
         * @param zmin defines the near clip plane
         * @param zmax defines the far clip plane
         * @returns the transformation matrix
         */
        static GetFinalMatrix<T extends Matrix>(viewport: DeepImmutable<Viewport>, world: DeepImmutable<T>, view: DeepImmutable<Matrix>, projection: DeepImmutable<Matrix>, zmin: number, zmax: number): T;
        /**
         * Extracts a 2x2 matrix from a given matrix and store the result in a Float32Array
         * @param matrix defines the matrix to use
         * @returns a new Float32Array array with 4 elements : the 2x2 matrix extracted from the given matrix
         */
        static GetAsMatrix2x2(matrix: DeepImmutable<Matrix>): Float32Array | Array<number>;
        /**
         * Extracts a 3x3 matrix from a given matrix and store the result in a Float32Array
         * @param matrix defines the matrix to use
         * @returns a new Float32Array array with 9 elements : the 3x3 matrix extracted from the given matrix
         */
        static GetAsMatrix3x3(matrix: DeepImmutable<Matrix>): Float32Array | Array<number>;
        /**
         * Compute the transpose of a given matrix
         * Example Playground - https://playground.babylonjs.com/#AV9X17#111
         * @param matrix defines the matrix to transpose
         * @returns the new matrix
         */
        static Transpose<T extends Matrix>(matrix: DeepImmutable<T>): T;
        /**
         * Compute the transpose of a matrix and store it in a target matrix
         * Example Playground - https://playground.babylonjs.com/#AV9X17#112
         * @param matrix defines the matrix to transpose
         * @param result defines the target matrix
         * @returns result input
         */
        static TransposeToRef<T extends Matrix>(matrix: DeepImmutable<Matrix>, result: T): T;
        /**
         * Computes a reflection matrix from a plane
         * Example Playground - https://playground.babylonjs.com/#AV9X17#87
         * @param plane defines the reflection plane
         * @returns a new matrix
         */
        static Reflection(plane: DeepImmutable<IPlaneLike>): Matrix;
        /**
         * Computes a reflection matrix from a plane
         * Example Playground - https://playground.babylonjs.com/#AV9X17#88
         * @param plane defines the reflection plane
         * @param result defines the target matrix
         * @returns result input
         */
        static ReflectionToRef<T extends Matrix>(plane: DeepImmutable<IPlaneLike>, result: T): T;
        /**
         * Sets the given matrix as a rotation matrix composed from the 3 left handed axes
         * @param xaxis defines the value of the 1st axis
         * @param yaxis defines the value of the 2nd axis
         * @param zaxis defines the value of the 3rd axis
         * @param result defines the target matrix
         * @returns result input
         */
        static FromXYZAxesToRef<T extends Matrix>(xaxis: DeepImmutable<Vector3>, yaxis: DeepImmutable<Vector3>, zaxis: DeepImmutable<Vector3>, result: T): T;
        /**
         * Creates a rotation matrix from a quaternion and stores it in a target matrix
         * @param quat defines the quaternion to use
         * @param result defines the target matrix
         * @returns result input
         */
        static FromQuaternionToRef<T extends Matrix>(quat: DeepImmutable<Quaternion>, result: T): T;
    }
    /**
     * @internal
     */
    export class TmpVectors {
        static Vector2: [Vector2, Vector2, Vector2];
        static Vector3: [Vector3, Vector3, Vector3, Vector3, Vector3, Vector3, Vector3, Vector3, Vector3, Vector3, Vector3, Vector3, Vector3];
        static Vector4: [Vector4, Vector4, Vector4];
        static Quaternion: [Quaternion, Quaternion];
        static Matrix: [Matrix, Matrix, Matrix, Matrix, Matrix, Matrix, Matrix, Matrix];
    }


    /**
     * Contains position and normal vectors for a vertex
     */
    export class PositionNormalVertex {
        /** the position of the vertex (defaut: 0,0,0) */
        position: Vector3;
        /** the normal of the vertex (defaut: 0,1,0) */
        normal: Vector3;
        /**
         * Creates a PositionNormalVertex
         * @param position the position of the vertex (defaut: 0,0,0)
         * @param normal the normal of the vertex (defaut: 0,1,0)
         */
        constructor(
        /** the position of the vertex (defaut: 0,0,0) */
        position?: Vector3, 
        /** the normal of the vertex (defaut: 0,1,0) */
        normal?: Vector3);
        /**
         * Clones the PositionNormalVertex
         * @returns the cloned PositionNormalVertex
         */
        clone(): PositionNormalVertex;
    }
    /**
     * Contains position, normal and uv vectors for a vertex
     */
    export class PositionNormalTextureVertex {
        /** the position of the vertex (defaut: 0,0,0) */
        position: Vector3;
        /** the normal of the vertex (defaut: 0,1,0) */
        normal: Vector3;
        /** the uv of the vertex (default: 0,0) */
        uv: Vector2;
        /**
         * Creates a PositionNormalTextureVertex
         * @param position the position of the vertex (defaut: 0,0,0)
         * @param normal the normal of the vertex (defaut: 0,1,0)
         * @param uv the uv of the vertex (default: 0,0)
         */
        constructor(
        /** the position of the vertex (defaut: 0,0,0) */
        position?: Vector3, 
        /** the normal of the vertex (defaut: 0,1,0) */
        normal?: Vector3, 
        /** the uv of the vertex (default: 0,0) */
        uv?: Vector2);
        /**
         * Clones the PositionNormalTextureVertex
         * @returns the cloned PositionNormalTextureVertex
         */
        clone(): PositionNormalTextureVertex;
    }


    /**
     * Class used to represent a viewport on screen
     */
    export class Viewport {
        /** viewport left coordinate */
        x: number;
        /** viewport top coordinate */
        y: number;
        /**viewport width */
        width: number;
        /** viewport height */
        height: number;
        /**
         * Creates a Viewport object located at (x, y) and sized (width, height)
         * @param x defines viewport left coordinate
         * @param y defines viewport top coordinate
         * @param width defines the viewport width
         * @param height defines the viewport height
         */
        constructor(
        /** viewport left coordinate */
        x: number, 
        /** viewport top coordinate */
        y: number, 
        /**viewport width */
        width: number, 
        /** viewport height */
        height: number);
        /**
         * Creates a new viewport using absolute sizing (from 0-> width, 0-> height instead of 0->1)
         * @param renderWidth defines the rendering width
         * @param renderHeight defines the rendering height
         * @returns a new Viewport
         */
        toGlobal(renderWidth: number, renderHeight: number): Viewport;
        /**
         * Stores absolute viewport value into a target viewport (from 0-> width, 0-> height instead of 0->1)
         * @param renderWidth defines the rendering width
         * @param renderHeight defines the rendering height
         * @param ref defines the target viewport
         * @returns the current viewport
         */
        toGlobalToRef(renderWidth: number, renderHeight: number, ref: Viewport): Viewport;
        /**
         * Returns a new Viewport copied from the current one
         * @returns a new Viewport
         */
        clone(): Viewport;
    }


    /**
     * Class representing spherical harmonics coefficients to the 3rd degree
     */
    export class SphericalHarmonics {
        /**
         * Defines whether or not the harmonics have been prescaled for rendering.
         */
        preScaled: boolean;
        /**
         * The l0,0 coefficients of the spherical harmonics
         */
        l00: Vector3;
        /**
         * The l1,-1 coefficients of the spherical harmonics
         */
        l1_1: Vector3;
        /**
         * The l1,0 coefficients of the spherical harmonics
         */
        l10: Vector3;
        /**
         * The l1,1 coefficients of the spherical harmonics
         */
        l11: Vector3;
        /**
         * The l2,-2 coefficients of the spherical harmonics
         */
        l2_2: Vector3;
        /**
         * The l2,-1 coefficients of the spherical harmonics
         */
        l2_1: Vector3;
        /**
         * The l2,0 coefficients of the spherical harmonics
         */
        l20: Vector3;
        /**
         * The l2,1 coefficients of the spherical harmonics
         */
        l21: Vector3;
        /**
         * The l2,2 coefficients of the spherical harmonics
         */
        l22: Vector3;
        /**
         * Adds a light to the spherical harmonics
         * @param direction the direction of the light
         * @param color the color of the light
         * @param deltaSolidAngle the delta solid angle of the light
         */
        addLight(direction: Vector3, color: Color3, deltaSolidAngle: number): void;
        /**
         * Scales the spherical harmonics by the given amount
         * @param scale the amount to scale
         */
        scaleInPlace(scale: number): void;
        /**
         * Convert from incident radiance (Li) to irradiance (E) by applying convolution with the cosine-weighted hemisphere.
         *
         * ```
         * E_lm = A_l * L_lm
         * ```
         *
         * In spherical harmonics this convolution amounts to scaling factors for each frequency band.
         * This corresponds to equation 5 in "An Efficient Representation for Irradiance Environment Maps", where
         * the scaling factors are given in equation 9.
         */
        convertIncidentRadianceToIrradiance(): void;
        /**
         * Convert from irradiance to outgoing radiance for Lambertian BDRF, suitable for efficient shader evaluation.
         *
         * ```
         * L = (1/pi) * E * rho
         * ```
         *
         * This is done by an additional scale by 1/pi, so is a fairly trivial operation but important conceptually.
         */
        convertIrradianceToLambertianRadiance(): void;
        /**
         * Integrates the reconstruction coefficients directly in to the SH preventing further
         * required operations at run time.
         *
         * This is simply done by scaling back the SH with Ylm constants parameter.
         * The trigonometric part being applied by the shader at run time.
         */
        preScaleForRendering(): void;
        /**
         * update the spherical harmonics coefficients from the given array
         * @param data defines the 9x3 coefficients (l00, l1-1, l10, l11, l2-2, l2-1, l20, l21, l22)
         * @returns the spherical harmonics (this)
         */
        updateFromArray(data: ArrayLike<ArrayLike<number>>): SphericalHarmonics;
        /**
         * update the spherical harmonics coefficients from the given floats array
         * @param data defines the 9x3 coefficients (l00, l1-1, l10, l11, l2-2, l2-1, l20, l21, l22)
         * @returns the spherical harmonics (this)
         */
        updateFromFloatsArray(data: ArrayLike<number>): SphericalHarmonics;
        /**
         * Constructs a spherical harmonics from an array.
         * @param data defines the 9x3 coefficients (l00, l1-1, l10, l11, l2-2, l2-1, l20, l21, l22)
         * @returns the spherical harmonics
         */
        static FromArray(data: ArrayLike<ArrayLike<number>>): SphericalHarmonics;
        /**
         * Gets the spherical harmonics from polynomial
         * @param polynomial the spherical polynomial
         * @returns the spherical harmonics
         */
        static FromPolynomial(polynomial: SphericalPolynomial): SphericalHarmonics;
    }
    /**
     * Class representing spherical polynomial coefficients to the 3rd degree
     */
    export class SphericalPolynomial {
        private _harmonics;
        /**
         * The spherical harmonics used to create the polynomials.
         */
        get preScaledHarmonics(): SphericalHarmonics;
        /**
         * The x coefficients of the spherical polynomial
         */
        x: Vector3;
        /**
         * The y coefficients of the spherical polynomial
         */
        y: Vector3;
        /**
         * The z coefficients of the spherical polynomial
         */
        z: Vector3;
        /**
         * The xx coefficients of the spherical polynomial
         */
        xx: Vector3;
        /**
         * The yy coefficients of the spherical polynomial
         */
        yy: Vector3;
        /**
         * The zz coefficients of the spherical polynomial
         */
        zz: Vector3;
        /**
         * The xy coefficients of the spherical polynomial
         */
        xy: Vector3;
        /**
         * The yz coefficients of the spherical polynomial
         */
        yz: Vector3;
        /**
         * The zx coefficients of the spherical polynomial
         */
        zx: Vector3;
        /**
         * Adds an ambient color to the spherical polynomial
         * @param color the color to add
         */
        addAmbient(color: Color3): void;
        /**
         * Scales the spherical polynomial by the given amount
         * @param scale the amount to scale
         */
        scaleInPlace(scale: number): void;
        /**
         * Updates the spherical polynomial from harmonics
         * @param harmonics the spherical harmonics
         * @returns the spherical polynomial
         */
        updateFromHarmonics(harmonics: SphericalHarmonics): SphericalPolynomial;
        /**
         * Gets the spherical polynomial from harmonics
         * @param harmonics the spherical harmonics
         * @returns the spherical polynomial
         */
        static FromHarmonics(harmonics: SphericalHarmonics): SphericalPolynomial;
        /**
         * Constructs a spherical polynomial from an array.
         * @param data defines the 9x3 coefficients (x, y, z, xx, yy, zz, yz, zx, xy)
         * @returns the spherical polynomial
         */
        static FromArray(data: ArrayLike<ArrayLike<number>>): SphericalPolynomial;
    }


    /** @internal */
    class _FacetDataStorage {
        facetPositions: Vector3[];
        facetNormals: Vector3[];
        facetPartitioning: number[][];
        facetNb: number;
        partitioningSubdivisions: number;
        partitioningBBoxRatio: number;
        facetDataEnabled: boolean;
        facetParameters: any;
        bbSize: Vector3;
        subDiv: {
            max: number;
            X: number;
            Y: number;
            Z: number;
        };
        facetDepthSort: boolean;
        facetDepthSortEnabled: boolean;
        depthSortedIndices: IndicesArray;
        depthSortedFacets: {
            ind: number;
            sqDistance: number;
        }[];
        facetDepthSortFunction: (f1: {
            ind: number;
            sqDistance: number;
        }, f2: {
            ind: number;
            sqDistance: number;
        }) => number;
        facetDepthSortFrom: Vector3;
        facetDepthSortOrigin: Vector3;
        invertedMatrix: Matrix;
    }
    /**
     * @internal
     **/
    class _InternalAbstractMeshDataInfo {
        _hasVertexAlpha: boolean;
        _useVertexColors: boolean;
        _numBoneInfluencers: number;
        _applyFog: boolean;
        _receiveShadows: boolean;
        _facetData: _FacetDataStorage;
        _visibility: number;
        _skeleton: Nullable<Skeleton>;
        _layerMask: number;
        _computeBonesUsingShaders: boolean;
        _isActive: boolean;
        _onlyForInstances: boolean;
        _isActiveIntermediate: boolean;
        _onlyForInstancesIntermediate: boolean;
        _actAsRegularMesh: boolean;
        _currentLOD: Nullable<AbstractMesh>;
        _currentLODIsUpToDate: boolean;
        _collisionRetryCount: number;
        _morphTargetManager: Nullable<MorphTargetManager>;
        _renderingGroupId: number;
        _bakedVertexAnimationManager: Nullable<IBakedVertexAnimationManager>;
        _material: Nullable<Material>;
        _materialForRenderPass: Array<Material | undefined>;
        _positions: Nullable<Vector3[]>;
        _pointerOverDisableMeshTesting: boolean;
        _meshCollisionData: _MeshCollisionData;
        _enableDistantPicking: boolean;
    }
    /**
     * Class used to store all common mesh properties
     */
    export class AbstractMesh extends TransformNode implements IDisposable, ICullable, IGetSetVerticesData {
        /** No occlusion */
        static OCCLUSION_TYPE_NONE: number;
        /** Occlusion set to optimistic */
        static OCCLUSION_TYPE_OPTIMISTIC: number;
        /** Occlusion set to strict */
        static OCCLUSION_TYPE_STRICT: number;
        /** Use an accurate occlusion algorithm */
        static OCCLUSION_ALGORITHM_TYPE_ACCURATE: number;
        /** Use a conservative occlusion algorithm */
        static OCCLUSION_ALGORITHM_TYPE_CONSERVATIVE: number;
        /** Default culling strategy : this is an exclusion test and it's the more accurate.
         *  Test order :
         *  Is the bounding sphere outside the frustum ?
         *  If not, are the bounding box vertices outside the frustum ?
         *  It not, then the cullable object is in the frustum.
         */
        static readonly CULLINGSTRATEGY_STANDARD = 0;
        /** Culling strategy : Bounding Sphere Only.
         *  This is an exclusion test. It's faster than the standard strategy because the bounding box is not tested.
         *  It's also less accurate than the standard because some not visible objects can still be selected.
         *  Test : is the bounding sphere outside the frustum ?
         *  If not, then the cullable object is in the frustum.
         */
        static readonly CULLINGSTRATEGY_BOUNDINGSPHERE_ONLY = 1;
        /** Culling strategy : Optimistic Inclusion.
         *  This in an inclusion test first, then the standard exclusion test.
         *  This can be faster when a cullable object is expected to be almost always in the camera frustum.
         *  This could also be a little slower than the standard test when the tested object center is not the frustum but one of its bounding box vertex is still inside.
         *  Anyway, it's as accurate as the standard strategy.
         *  Test :
         *  Is the cullable object bounding sphere center in the frustum ?
         *  If not, apply the default culling strategy.
         */
        static readonly CULLINGSTRATEGY_OPTIMISTIC_INCLUSION = 2;
        /** Culling strategy : Optimistic Inclusion then Bounding Sphere Only.
         *  This in an inclusion test first, then the bounding sphere only exclusion test.
         *  This can be the fastest test when a cullable object is expected to be almost always in the camera frustum.
         *  This could also be a little slower than the BoundingSphereOnly strategy when the tested object center is not in the frustum but its bounding sphere still intersects it.
         *  It's less accurate than the standard strategy and as accurate as the BoundingSphereOnly strategy.
         *  Test :
         *  Is the cullable object bounding sphere center in the frustum ?
         *  If not, apply the Bounding Sphere Only strategy. No Bounding Box is tested here.
         */
        static readonly CULLINGSTRATEGY_OPTIMISTIC_INCLUSION_THEN_BSPHERE_ONLY = 3;
        /**
         * No billboard
         */
        static get BILLBOARDMODE_NONE(): number;
        /** Billboard on X axis */
        static get BILLBOARDMODE_X(): number;
        /** Billboard on Y axis */
        static get BILLBOARDMODE_Y(): number;
        /** Billboard on Z axis */
        static get BILLBOARDMODE_Z(): number;
        /** Billboard on all axes */
        static get BILLBOARDMODE_ALL(): number;
        /** Billboard on using position instead of orientation */
        static get BILLBOARDMODE_USE_POSITION(): number;
        /** @internal */
        _internalAbstractMeshDataInfo: _InternalAbstractMeshDataInfo;
        /** @internal */
        _waitingMaterialId: Nullable<string>;
        /**
         * The culling strategy to use to check whether the mesh must be rendered or not.
         * This value can be changed at any time and will be used on the next render mesh selection.
         * The possible values are :
         * - AbstractMesh.CULLINGSTRATEGY_STANDARD
         * - AbstractMesh.CULLINGSTRATEGY_BOUNDINGSPHERE_ONLY
         * - AbstractMesh.CULLINGSTRATEGY_OPTIMISTIC_INCLUSION
         * - AbstractMesh.CULLINGSTRATEGY_OPTIMISTIC_INCLUSION_THEN_BSPHERE_ONLY
         * Please read each static variable documentation to get details about the culling process.
         * */
        cullingStrategy: number;
        /**
         * Gets the number of facets in the mesh
         * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/facetData#what-is-a-mesh-facet
         */
        get facetNb(): number;
        /**
         * Gets or set the number (integer) of subdivisions per axis in the partitioning space
         * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/facetData#tweaking-the-partitioning
         */
        get partitioningSubdivisions(): number;
        set partitioningSubdivisions(nb: number);
        /**
         * The ratio (float) to apply to the bounding box size to set to the partitioning space.
         * Ex : 1.01 (default) the partitioning space is 1% bigger than the bounding box
         * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/facetData#tweaking-the-partitioning
         */
        get partitioningBBoxRatio(): number;
        set partitioningBBoxRatio(ratio: number);
        /**
         * Gets or sets a boolean indicating that the facets must be depth sorted on next call to `updateFacetData()`.
         * Works only for updatable meshes.
         * Doesn't work with multi-materials
         * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/facetData#facet-depth-sort
         */
        get mustDepthSortFacets(): boolean;
        set mustDepthSortFacets(sort: boolean);
        /**
         * The location (Vector3) where the facet depth sort must be computed from.
         * By default, the active camera position.
         * Used only when facet depth sort is enabled
         * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/facetData#facet-depth-sort
         */
        get facetDepthSortFrom(): Vector3;
        set facetDepthSortFrom(location: Vector3);
        /** number of collision detection tries. Change this value if not all collisions are detected and handled properly */
        get collisionRetryCount(): number;
        set collisionRetryCount(retryCount: number);
        /**
         * gets a boolean indicating if facetData is enabled
         * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/facetData#what-is-a-mesh-facet
         */
        get isFacetDataEnabled(): boolean;
        /**
         * Gets or sets the morph target manager
         * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/morphTargets
         */
        get morphTargetManager(): Nullable<MorphTargetManager>;
        set morphTargetManager(value: Nullable<MorphTargetManager>);
        /**
         * Gets or sets the baked vertex animation manager
         * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/baked_texture_animations
         */
        get bakedVertexAnimationManager(): Nullable<IBakedVertexAnimationManager>;
        set bakedVertexAnimationManager(value: Nullable<IBakedVertexAnimationManager>);
        /** @internal */
        _syncGeometryWithMorphTargetManager(): void;
        /**
         * @internal
         */
        _updateNonUniformScalingState(value: boolean): boolean;
        /**
         * An event triggered when this mesh collides with another one
         */
        onCollideObservable: Observable<AbstractMesh>;
        /** Set a function to call when this mesh collides with another one */
        set onCollide(callback: (collidedMesh?: AbstractMesh) => void);
        /**
         * An event triggered when the collision's position changes
         */
        onCollisionPositionChangeObservable: Observable<Vector3>;
        /** Set a function to call when the collision's position changes */
        set onCollisionPositionChange(callback: () => void);
        /**
         * An event triggered when material is changed
         */
        onMaterialChangedObservable: Observable<AbstractMesh>;
        /**
         * Gets or sets the orientation for POV movement & rotation
         */
        definedFacingForward: boolean;
        /** @internal */
        _occlusionQuery: Nullable<WebGLQuery | number>;
        /** @internal */
        _renderingGroup: Nullable<RenderingGroup>;
        /**
         * Gets or sets mesh visibility between 0 and 1 (default is 1)
         */
        get visibility(): number;
        /**
         * Gets or sets mesh visibility between 0 and 1 (default is 1)
         */
        set visibility(value: number);
        /** Gets or sets the alpha index used to sort transparent meshes
         * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/advanced/transparent_rendering#alpha-index
         */
        alphaIndex: number;
        /**
         * Gets or sets a boolean indicating if the mesh is visible (renderable). Default is true
         */
        isVisible: boolean;
        /**
         * Gets or sets a boolean indicating if the mesh can be picked (by scene.pick for instance or through actions). Default is true
         */
        isPickable: boolean;
        /**
         * Gets or sets a boolean indicating if the mesh can be near picked. Default is false
         */
        isNearPickable: boolean;
        /**
         * Gets or sets a boolean indicating if the mesh can be near grabbed. Default is false
         */
        isNearGrabbable: boolean;
        /** Gets or sets a boolean indicating that bounding boxes of subMeshes must be rendered as well (false by default) */
        showSubMeshesBoundingBox: boolean;
        /** Gets or sets a boolean indicating if the mesh must be considered as a ray blocker for lens flares (false by default)
         * @see https://doc.babylonjs.com/features/featuresDeepDive/environment/lenseFlare
         */
        isBlocker: boolean;
        /**
         * Gets or sets a boolean indicating that pointer move events must be supported on this mesh (false by default)
         */
        enablePointerMoveEvents: boolean;
        /**
         * Gets or sets the property which disables the test that is checking that the mesh under the pointer is the same than the previous time we tested for it (default: false).
         * Set this property to true if you want thin instances picking to be reported accurately when moving over the mesh.
         * Note that setting this property to true will incur some performance penalties when dealing with pointer events for this mesh so use it sparingly.
         */
        get pointerOverDisableMeshTesting(): boolean;
        set pointerOverDisableMeshTesting(disable: boolean);
        /**
         * Specifies the rendering group id for this mesh (0 by default)
         * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/advanced/transparent_rendering#rendering-groups
         */
        get renderingGroupId(): number;
        set renderingGroupId(value: number);
        /** Gets or sets current material */
        get material(): Nullable<Material>;
        set material(value: Nullable<Material>);
        /**
         * Gets the material used to render the mesh in a specific render pass
         * @param renderPassId render pass id
         * @returns material used for the render pass. If no specific material is used for this render pass, undefined is returned (meaning mesh.material is used for this pass)
         */
        getMaterialForRenderPass(renderPassId: number): Material | undefined;
        /**
         * Sets the material to be used to render the mesh in a specific render pass
         * @param renderPassId render pass id
         * @param material material to use for this render pass. If undefined is passed, no specific material will be used for this render pass but the regular material will be used instead (mesh.material)
         */
        setMaterialForRenderPass(renderPassId: number, material?: Material): void;
        /**
         * Gets or sets a boolean indicating that this mesh can receive realtime shadows
         * @see https://doc.babylonjs.com/features/featuresDeepDive/lights/shadows
         */
        get receiveShadows(): boolean;
        set receiveShadows(value: boolean);
        /** Defines color to use when rendering outline */
        outlineColor: Color3;
        /** Define width to use when rendering outline */
        outlineWidth: number;
        /** Defines color to use when rendering overlay */
        overlayColor: Color3;
        /** Defines alpha to use when rendering overlay */
        overlayAlpha: number;
        /** Gets or sets a boolean indicating that this mesh contains vertex color data with alpha values */
        get hasVertexAlpha(): boolean;
        set hasVertexAlpha(value: boolean);
        /** Gets or sets a boolean indicating that this mesh needs to use vertex color data to render (if this kind of vertex data is available in the geometry) */
        get useVertexColors(): boolean;
        set useVertexColors(value: boolean);
        /**
         * Gets or sets a boolean indicating that bone animations must be computed by the CPU (false by default)
         */
        get computeBonesUsingShaders(): boolean;
        set computeBonesUsingShaders(value: boolean);
        /** Gets or sets the number of allowed bone influences per vertex (4 by default) */
        get numBoneInfluencers(): number;
        set numBoneInfluencers(value: number);
        /** Gets or sets a boolean indicating that this mesh will allow fog to be rendered on it (true by default) */
        get applyFog(): boolean;
        set applyFog(value: boolean);
        /** When enabled, decompose picking matrices for better precision with large values for mesh position and scling */
        get enableDistantPicking(): boolean;
        set enableDistantPicking(value: boolean);
        /** Gets or sets a boolean indicating that internal octree (if available) can be used to boost submeshes selection (true by default) */
        useOctreeForRenderingSelection: boolean;
        /** Gets or sets a boolean indicating that internal octree (if available) can be used to boost submeshes picking (true by default) */
        useOctreeForPicking: boolean;
        /** Gets or sets a boolean indicating that internal octree (if available) can be used to boost submeshes collision (true by default) */
        useOctreeForCollisions: boolean;
        /**
         * Gets or sets the current layer mask (default is 0x0FFFFFFF)
         * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/layerMasksAndMultiCam
         */
        get layerMask(): number;
        set layerMask(value: number);
        /**
         * True if the mesh must be rendered in any case (this will shortcut the frustum clipping phase)
         */
        alwaysSelectAsActiveMesh: boolean;
        /**
         * Gets or sets a boolean indicating that the bounding info does not need to be kept in sync (for performance reason)
         */
        doNotSyncBoundingInfo: boolean;
        /**
         * Gets or sets the current action manager
         * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions
         */
        actionManager: Nullable<AbstractActionManager>;
        /**
         * Gets or sets the ellipsoid used to impersonate this mesh when using collision engine (default is (0.5, 1, 0.5))
         * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_collisions
         */
        ellipsoid: Vector3;
        /**
         * Gets or sets the ellipsoid offset used to impersonate this mesh when using collision engine (default is (0, 0, 0))
         * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_collisions
         */
        ellipsoidOffset: Vector3;
        /**
         * Gets or sets a collision mask used to mask collisions (default is -1).
         * A collision between A and B will happen if A.collisionGroup & b.collisionMask !== 0
         */
        get collisionMask(): number;
        set collisionMask(mask: number);
        /**
         * Gets or sets a collision response flag (default is true).
         * when collisionResponse is false, events are still triggered but colliding entity has no response
         * This helps creating trigger volume when user wants collision feedback events but not position/velocity
         * to respond to the collision.
         */
        get collisionResponse(): boolean;
        set collisionResponse(response: boolean);
        /**
         * Gets or sets the current collision group mask (-1 by default).
         * A collision between A and B will happen if A.collisionGroup & b.collisionMask !== 0
         */
        get collisionGroup(): number;
        set collisionGroup(mask: number);
        /**
         * Gets or sets current surrounding meshes (null by default).
         *
         * By default collision detection is tested against every mesh in the scene.
         * It is possible to set surroundingMeshes to a defined list of meshes and then only these specified
         * meshes will be tested for the collision.
         *
         * Note: if set to an empty array no collision will happen when this mesh is moved.
         */
        get surroundingMeshes(): Nullable<AbstractMesh[]>;
        set surroundingMeshes(meshes: Nullable<AbstractMesh[]>);
        /**
         * Defines edge width used when edgesRenderer is enabled
         * @see https://www.babylonjs-playground.com/#10OJSG#13
         */
        edgesWidth: number;
        /**
         * Defines edge color used when edgesRenderer is enabled
         * @see https://www.babylonjs-playground.com/#10OJSG#13
         */
        edgesColor: Color4;
        /** @internal */
        _edgesRenderer: Nullable<IEdgesRenderer>;
        /** @internal */
        _masterMesh: Nullable<AbstractMesh>;
        protected _boundingInfo: Nullable<BoundingInfo>;
        protected _boundingInfoIsDirty: boolean;
        /** @internal */
        _renderId: number;
        /**
         * Gets or sets the list of subMeshes
         * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/multiMaterials
         */
        subMeshes: SubMesh[];
        /** @internal */
        _intersectionsInProgress: AbstractMesh[];
        /** @internal */
        _unIndexed: boolean;
        /** @internal */
        _lightSources: Light[];
        /** Gets the list of lights affecting that mesh */
        get lightSources(): Light[];
        /** @internal */
        get _positions(): Nullable<Vector3[]>;
        /** @internal */
        _waitingData: {
            lods: Nullable<any>;
            actions: Nullable<any>;
            freezeWorldMatrix: Nullable<boolean>;
        };
        /** @internal */
        _bonesTransformMatrices: Nullable<Float32Array>;
        /** @internal */
        _transformMatrixTexture: Nullable<RawTexture>;
        /**
         * Gets or sets a skeleton to apply skinning transformations
         * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/bonesSkeletons
         */
        set skeleton(value: Nullable<Skeleton>);
        get skeleton(): Nullable<Skeleton>;
        /**
         * An event triggered when the mesh is rebuilt.
         */
        onRebuildObservable: Observable<AbstractMesh>;
        /**
         * The current mesh uniform buffer.
         * @internal Internal use only.
         */
        _uniformBuffer: UniformBuffer;
        /**
         * Creates a new AbstractMesh
         * @param name defines the name of the mesh
         * @param scene defines the hosting scene
         */
        constructor(name: string, scene?: Nullable<Scene>);
        protected _buildUniformLayout(): void;
        /**
         * Transfer the mesh values to its UBO.
         * @param world The world matrix associated with the mesh
         */
        transferToEffect(world: Matrix): void;
        /**
         * Gets the mesh uniform buffer.
         * @returns the uniform buffer of the mesh.
         */
        getMeshUniformBuffer(): UniformBuffer;
        /**
         * Returns the string "AbstractMesh"
         * @returns "AbstractMesh"
         */
        getClassName(): string;
        /**
         * Gets a string representation of the current mesh
         * @param fullDetails defines a boolean indicating if full details must be included
         * @returns a string representation of the current mesh
         */
        toString(fullDetails?: boolean): string;
        /**
         * @internal
         */
        protected _getEffectiveParent(): Nullable<Node>;
        /**
         * @internal
         */
        _getActionManagerForTrigger(trigger?: number, initialCall?: boolean): Nullable<AbstractActionManager>;
        /**
         * @internal
         */
        _rebuild(dispose?: boolean): void;
        /** @internal */
        _resyncLightSources(): void;
        /**
         * @internal
         */
        _resyncLightSource(light: Light): void;
        /** @internal */
        _unBindEffect(): void;
        /**
         * @internal
         */
        _removeLightSource(light: Light, dispose: boolean): void;
        private _markSubMeshesAsDirty;
        /**
         * @internal
         */
        _markSubMeshesAsLightDirty(dispose?: boolean): void;
        /** @internal */
        _markSubMeshesAsAttributesDirty(): void;
        /** @internal */
        _markSubMeshesAsMiscDirty(): void;
        /**
         * Flag the AbstractMesh as dirty (Forcing it to update everything)
         * @param property if set to "rotation" the objects rotationQuaternion will be set to null
         * @returns this AbstractMesh
         */
        markAsDirty(property?: string): AbstractMesh;
        /**
         * Resets the draw wrappers cache for all submeshes of this abstract mesh
         * @param passId If provided, releases only the draw wrapper corresponding to this render pass id
         */
        resetDrawCache(passId?: number): void;
        /**
         * Returns true if the mesh is blocked. Implemented by child classes
         */
        get isBlocked(): boolean;
        /**
         * Returns the mesh itself by default. Implemented by child classes
         * @param camera defines the camera to use to pick the right LOD level
         * @returns the currentAbstractMesh
         */
        getLOD(camera: Camera): Nullable<AbstractMesh>;
        /**
         * Returns 0 by default. Implemented by child classes
         * @returns an integer
         */
        getTotalVertices(): number;
        /**
         * Returns a positive integer : the total number of indices in this mesh geometry.
         * @returns the number of indices or zero if the mesh has no geometry.
         */
        getTotalIndices(): number;
        /**
         * Returns null by default. Implemented by child classes
         * @returns null
         */
        getIndices(): Nullable<IndicesArray>;
        /**
         * Returns the array of the requested vertex data kind. Implemented by child classes
         * @param kind defines the vertex data kind to use
         * @returns null
         */
        getVerticesData(kind: string): Nullable<FloatArray>;
        /**
         * Sets the vertex data of the mesh geometry for the requested `kind`.
         * If the mesh has no geometry, a new Geometry object is set to the mesh and then passed this vertex data.
         * Note that a new underlying VertexBuffer object is created each call.
         * If the `kind` is the `PositionKind`, the mesh BoundingInfo is renewed, so the bounding box and sphere, and the mesh World Matrix is recomputed.
         * @param kind defines vertex data kind:
         * * VertexBuffer.PositionKind
         * * VertexBuffer.UVKind
         * * VertexBuffer.UV2Kind
         * * VertexBuffer.UV3Kind
         * * VertexBuffer.UV4Kind
         * * VertexBuffer.UV5Kind
         * * VertexBuffer.UV6Kind
         * * VertexBuffer.ColorKind
         * * VertexBuffer.MatricesIndicesKind
         * * VertexBuffer.MatricesIndicesExtraKind
         * * VertexBuffer.MatricesWeightsKind
         * * VertexBuffer.MatricesWeightsExtraKind
         * @param data defines the data source
         * @param updatable defines if the data must be flagged as updatable (or static)
         * @param stride defines the vertex stride (size of an entire vertex). Can be null and in this case will be deduced from vertex data kind
         * @returns the current mesh
         */
        setVerticesData(kind: string, data: FloatArray, updatable?: boolean, stride?: number): AbstractMesh;
        /**
         * Updates the existing vertex data of the mesh geometry for the requested `kind`.
         * If the mesh has no geometry, it is simply returned as it is.
         * @param kind defines vertex data kind:
         * * VertexBuffer.PositionKind
         * * VertexBuffer.UVKind
         * * VertexBuffer.UV2Kind
         * * VertexBuffer.UV3Kind
         * * VertexBuffer.UV4Kind
         * * VertexBuffer.UV5Kind
         * * VertexBuffer.UV6Kind
         * * VertexBuffer.ColorKind
         * * VertexBuffer.MatricesIndicesKind
         * * VertexBuffer.MatricesIndicesExtraKind
         * * VertexBuffer.MatricesWeightsKind
         * * VertexBuffer.MatricesWeightsExtraKind
         * @param data defines the data source
         * @param updateExtends If `kind` is `PositionKind` and if `updateExtends` is true, the mesh BoundingInfo is renewed, so the bounding box and sphere, and the mesh World Matrix is recomputed
         * @param makeItUnique If true, a new global geometry is created from this data and is set to the mesh
         * @returns the current mesh
         */
        updateVerticesData(kind: string, data: FloatArray, updateExtends?: boolean, makeItUnique?: boolean): AbstractMesh;
        /**
         * Sets the mesh indices,
         * If the mesh has no geometry, a new Geometry object is created and set to the mesh.
         * @param indices Expects an array populated with integers or a typed array (Int32Array, Uint32Array, Uint16Array)
         * @param totalVertices Defines the total number of vertices
         * @returns the current mesh
         */
        setIndices(indices: IndicesArray, totalVertices: Nullable<number>): AbstractMesh;
        /**
         * Gets a boolean indicating if specific vertex data is present
         * @param kind defines the vertex data kind to use
         * @returns true is data kind is present
         */
        isVerticesDataPresent(kind: string): boolean;
        /**
         * Returns the mesh BoundingInfo object or creates a new one and returns if it was undefined.
         * Note that it returns a shallow bounding of the mesh (i.e. it does not include children).
         * To get the full bounding of all children, call `getHierarchyBoundingVectors` instead.
         * @returns a BoundingInfo
         */
        getBoundingInfo(): BoundingInfo;
        /**
         * Overwrite the current bounding info
         * @param boundingInfo defines the new bounding info
         * @returns the current mesh
         */
        setBoundingInfo(boundingInfo: BoundingInfo): AbstractMesh;
        /**
         * Returns true if there is already a bounding info
         */
        get hasBoundingInfo(): boolean;
        /**
         * Creates a new bounding info for the mesh
         * @param minimum min vector of the bounding box/sphere
         * @param maximum max vector of the bounding box/sphere
         * @param worldMatrix defines the new world matrix
         * @returns the new bounding info
         */
        buildBoundingInfo(minimum: DeepImmutable<Vector3>, maximum: DeepImmutable<Vector3>, worldMatrix?: DeepImmutable<Matrix>): BoundingInfo;
        /**
         * Uniformly scales the mesh to fit inside of a unit cube (1 X 1 X 1 units)
         * @param includeDescendants Use the hierarchy's bounding box instead of the mesh's bounding box. Default is false
         * @param ignoreRotation ignore rotation when computing the scale (ie. object will be axis aligned). Default is false
         * @param predicate predicate that is passed in to getHierarchyBoundingVectors when selecting which object should be included when scaling
         * @returns the current mesh
         */
        normalizeToUnitCube(includeDescendants?: boolean, ignoreRotation?: boolean, predicate?: Nullable<(node: AbstractMesh) => boolean>): AbstractMesh;
        /** Gets a boolean indicating if this mesh has skinning data and an attached skeleton */
        get useBones(): boolean;
        /** @internal */
        _preActivate(): void;
        /**
         * @internal
         */
        _preActivateForIntermediateRendering(renderId: number): void;
        /**
         * @internal
         */
        _activate(renderId: number, intermediateRendering: boolean): boolean;
        /** @internal */
        _postActivate(): void;
        /** @internal */
        _freeze(): void;
        /** @internal */
        _unFreeze(): void;
        /**
         * Gets the current world matrix
         * @returns a Matrix
         */
        getWorldMatrix(): Matrix;
        /** @internal */
        _getWorldMatrixDeterminant(): number;
        /**
         * Gets a boolean indicating if this mesh is an instance or a regular mesh
         */
        get isAnInstance(): boolean;
        /**
         * Gets a boolean indicating if this mesh has instances
         */
        get hasInstances(): boolean;
        /**
         * Gets a boolean indicating if this mesh has thin instances
         */
        get hasThinInstances(): boolean;
        /**
         * Perform relative position change from the point of view of behind the front of the mesh.
         * This is performed taking into account the meshes current rotation, so you do not have to care.
         * Supports definition of mesh facing forward or backward {@link definedFacingForwardSearch | See definedFacingForwardSearch }.
         * @param amountRight defines the distance on the right axis
         * @param amountUp defines the distance on the up axis
         * @param amountForward defines the distance on the forward axis
         * @returns the current mesh
         */
        movePOV(amountRight: number, amountUp: number, amountForward: number): AbstractMesh;
        /**
         * Calculate relative position change from the point of view of behind the front of the mesh.
         * This is performed taking into account the meshes current rotation, so you do not have to care.
         * Supports definition of mesh facing forward or backward {@link definedFacingForwardSearch | See definedFacingForwardSearch }.
         * @param amountRight defines the distance on the right axis
         * @param amountUp defines the distance on the up axis
         * @param amountForward defines the distance on the forward axis
         * @returns the new displacement vector
         */
        calcMovePOV(amountRight: number, amountUp: number, amountForward: number): Vector3;
        /**
         * Perform relative rotation change from the point of view of behind the front of the mesh.
         * Supports definition of mesh facing forward or backward {@link definedFacingForwardSearch | See definedFacingForwardSearch }.
         * @param flipBack defines the flip
         * @param twirlClockwise defines the twirl
         * @param tiltRight defines the tilt
         * @returns the current mesh
         */
        rotatePOV(flipBack: number, twirlClockwise: number, tiltRight: number): AbstractMesh;
        /**
         * Calculate relative rotation change from the point of view of behind the front of the mesh.
         * Supports definition of mesh facing forward or backward {@link definedFacingForwardSearch | See definedFacingForwardSearch }.
         * @param flipBack defines the flip
         * @param twirlClockwise defines the twirl
         * @param tiltRight defines the tilt
         * @returns the new rotation vector
         */
        calcRotatePOV(flipBack: number, twirlClockwise: number, tiltRight: number): Vector3;
        /**
         * This method recomputes and sets a new BoundingInfo to the mesh unless it is locked.
         * This means the mesh underlying bounding box and sphere are recomputed.
         * @param applySkeleton defines whether to apply the skeleton before computing the bounding info
         * @param applyMorph  defines whether to apply the morph target before computing the bounding info
         * @returns the current mesh
         */
        refreshBoundingInfo(applySkeleton?: boolean, applyMorph?: boolean): AbstractMesh;
        /**
         * @internal
         */
        _refreshBoundingInfo(data: Nullable<FloatArray>, bias: Nullable<Vector2>): void;
        /**
         * Internal function to get buffer data and possibly apply morphs and normals
         * @param applySkeleton
         * @param applyMorph
         * @param data
         * @param kind the kind of data you want. Can be Normal or Position
         */
        private _getData;
        /**
         * Get the normals vertex data and optionally apply skeleton and morphing.
         * @param applySkeleton defines whether to apply the skeleton
         * @param applyMorph  defines whether to apply the morph target
         * @returns the normals data
         */
        getNormalsData(applySkeleton?: boolean, applyMorph?: boolean): Nullable<FloatArray>;
        /**
         * Get the position vertex data and optionally apply skeleton and morphing.
         * @param applySkeleton defines whether to apply the skeleton
         * @param applyMorph  defines whether to apply the morph target
         * @param data defines the position data to apply the skeleton and morph to
         * @returns the position data
         */
        getPositionData(applySkeleton?: boolean, applyMorph?: boolean, data?: Nullable<FloatArray>): Nullable<FloatArray>;
        /**
         * @internal
         */
        _getPositionData(applySkeleton: boolean, applyMorph: boolean): Nullable<FloatArray>;
        /** @internal */
        _updateBoundingInfo(): AbstractMesh;
        /**
         * @internal
         */
        _updateSubMeshesBoundingInfo(matrix: DeepImmutable<Matrix>): AbstractMesh;
        /** @internal */
        protected _afterComputeWorldMatrix(): void;
        /**
         * Returns `true` if the mesh is within the frustum defined by the passed array of planes.
         * A mesh is in the frustum if its bounding box intersects the frustum
         * @param frustumPlanes defines the frustum to test
         * @returns true if the mesh is in the frustum planes
         */
        isInFrustum(frustumPlanes: Plane[]): boolean;
        /**
         * Returns `true` if the mesh is completely in the frustum defined be the passed array of planes.
         * A mesh is completely in the frustum if its bounding box it completely inside the frustum.
         * @param frustumPlanes defines the frustum to test
         * @returns true if the mesh is completely in the frustum planes
         */
        isCompletelyInFrustum(frustumPlanes: Plane[]): boolean;
        /**
         * True if the mesh intersects another mesh or a SolidParticle object
         * @param mesh defines a target mesh or SolidParticle to test
         * @param precise Unless the parameter `precise` is set to `true` the intersection is computed according to Axis Aligned Bounding Boxes (AABB), else according to OBB (Oriented BBoxes)
         * @param includeDescendants Can be set to true to test if the mesh defined in parameters intersects with the current mesh or any child meshes
         * @returns true if there is an intersection
         */
        intersectsMesh(mesh: AbstractMesh | SolidParticle, precise?: boolean, includeDescendants?: boolean): boolean;
        /**
         * Returns true if the passed point (Vector3) is inside the mesh bounding box
         * @param point defines the point to test
         * @returns true if there is an intersection
         */
        intersectsPoint(point: Vector3): boolean;
        /**
         * Gets or sets a boolean indicating that this mesh can be used in the collision engine
         * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_collisions
         */
        get checkCollisions(): boolean;
        set checkCollisions(collisionEnabled: boolean);
        /**
         * Gets Collider object used to compute collisions (not physics)
         * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_collisions
         */
        get collider(): Nullable<Collider>;
        /**
         * Move the mesh using collision engine
         * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_collisions
         * @param displacement defines the requested displacement vector
         * @returns the current mesh
         */
        moveWithCollisions(displacement: Vector3): AbstractMesh;
        private _onCollisionPositionChange;
        /**
         * @internal
         */
        _collideForSubMesh(subMesh: SubMesh, transformMatrix: Matrix, collider: Collider): AbstractMesh;
        /**
         * @internal
         */
        _processCollisionsForSubMeshes(collider: Collider, transformMatrix: Matrix): AbstractMesh;
        /** @internal */
        _shouldConvertRHS(): boolean;
        /**
         * @internal
         */
        _checkCollision(collider: Collider): AbstractMesh;
        /** @internal */
        _generatePointsArray(): boolean;
        /**
         * Checks if the passed Ray intersects with the mesh
         * @param ray defines the ray to use
         * @param fastCheck defines if fast mode (but less precise) must be used (false by default)
         * @param trianglePredicate defines an optional predicate used to select faces when a mesh intersection is detected
         * @param onlyBoundingInfo defines a boolean indicating if picking should only happen using bounding info (false by default)
         * @param worldToUse defines the world matrix to use to get the world coordinate of the intersection point
         * @param skipBoundingInfo a boolean indicating if we should skip the bounding info check
         * @returns the picking info
         * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/interactions/mesh_intersect
         */
        intersects(ray: Ray, fastCheck?: boolean, trianglePredicate?: TrianglePickingPredicate, onlyBoundingInfo?: boolean, worldToUse?: Matrix, skipBoundingInfo?: boolean): PickingInfo;
        /**
         * Clones the current mesh
         * @param name defines the mesh name
         * @param newParent defines the new mesh parent
         * @param doNotCloneChildren defines a boolean indicating that children must not be cloned (false by default)
         * @returns the new mesh
         */
        clone(name: string, newParent: Nullable<Node>, doNotCloneChildren?: boolean): Nullable<AbstractMesh>;
        /**
         * Disposes all the submeshes of the current meshnp
         * @returns the current mesh
         */
        releaseSubMeshes(): AbstractMesh;
        /**
         * Releases resources associated with this abstract mesh.
         * @param doNotRecurse Set to true to not recurse into each children (recurse into each children by default)
         * @param disposeMaterialAndTextures Set to true to also dispose referenced materials and textures (false by default)
         */
        dispose(doNotRecurse?: boolean, disposeMaterialAndTextures?: boolean): void;
        /**
         * Adds the passed mesh as a child to the current mesh
         * @param mesh defines the child mesh
         * @param preserveScalingSign if true, keep scaling sign of child. Otherwise, scaling sign might change.
         * @returns the current mesh
         */
        addChild(mesh: AbstractMesh, preserveScalingSign?: boolean): AbstractMesh;
        /**
         * Removes the passed mesh from the current mesh children list
         * @param mesh defines the child mesh
         * @param preserveScalingSign if true, keep scaling sign of child. Otherwise, scaling sign might change.
         * @returns the current mesh
         */
        removeChild(mesh: AbstractMesh, preserveScalingSign?: boolean): AbstractMesh;
        /** @internal */
        private _initFacetData;
        /**
         * Updates the mesh facetData arrays and the internal partitioning when the mesh is morphed or updated.
         * This method can be called within the render loop.
         * You don't need to call this method by yourself in the render loop when you update/morph a mesh with the methods CreateXXX() as they automatically manage this computation
         * @returns the current mesh
         * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/facetData
         */
        updateFacetData(): AbstractMesh;
        /**
         * Returns the facetLocalNormals array.
         * The normals are expressed in the mesh local spac
         * @returns an array of Vector3
         * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/facetData
         */
        getFacetLocalNormals(): Vector3[];
        /**
         * Returns the facetLocalPositions array.
         * The facet positions are expressed in the mesh local space
         * @returns an array of Vector3
         * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/facetData
         */
        getFacetLocalPositions(): Vector3[];
        /**
         * Returns the facetLocalPartitioning array
         * @returns an array of array of numbers
         * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/facetData
         */
        getFacetLocalPartitioning(): number[][];
        /**
         * Returns the i-th facet position in the world system.
         * This method allocates a new Vector3 per call
         * @param i defines the facet index
         * @returns a new Vector3
         * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/facetData
         */
        getFacetPosition(i: number): Vector3;
        /**
         * Sets the reference Vector3 with the i-th facet position in the world system
         * @param i defines the facet index
         * @param ref defines the target vector
         * @returns the current mesh
         * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/facetData
         */
        getFacetPositionToRef(i: number, ref: Vector3): AbstractMesh;
        /**
         * Returns the i-th facet normal in the world system.
         * This method allocates a new Vector3 per call
         * @param i defines the facet index
         * @returns a new Vector3
         * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/facetData
         */
        getFacetNormal(i: number): Vector3;
        /**
         * Sets the reference Vector3 with the i-th facet normal in the world system
         * @param i defines the facet index
         * @param ref defines the target vector
         * @returns the current mesh
         * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/facetData
         */
        getFacetNormalToRef(i: number, ref: Vector3): this;
        /**
         * Returns the facets (in an array) in the same partitioning block than the one the passed coordinates are located (expressed in the mesh local system)
         * @param x defines x coordinate
         * @param y defines y coordinate
         * @param z defines z coordinate
         * @returns the array of facet indexes
         * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/facetData
         */
        getFacetsAtLocalCoordinates(x: number, y: number, z: number): Nullable<number[]>;
        /**
         * Returns the closest mesh facet index at (x,y,z) World coordinates, null if not found
         * @param x defines x coordinate
         * @param y defines y coordinate
         * @param z defines z coordinate
         * @param projected sets as the (x,y,z) world projection on the facet
         * @param checkFace if true (default false), only the facet "facing" to (x,y,z) or only the ones "turning their backs", according to the parameter "facing" are returned
         * @param facing if facing and checkFace are true, only the facet "facing" to (x, y, z) are returned : positive dot (x, y, z) * facet position. If facing si false and checkFace is true, only the facet "turning their backs" to (x, y, z) are returned : negative dot (x, y, z) * facet position
         * @returns the face index if found (or null instead)
         * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/facetData
         */
        getClosestFacetAtCoordinates(x: number, y: number, z: number, projected?: Vector3, checkFace?: boolean, facing?: boolean): Nullable<number>;
        /**
         * Returns the closest mesh facet index at (x,y,z) local coordinates, null if not found
         * @param x defines x coordinate
         * @param y defines y coordinate
         * @param z defines z coordinate
         * @param projected sets as the (x,y,z) local projection on the facet
         * @param checkFace if true (default false), only the facet "facing" to (x,y,z) or only the ones "turning their backs", according to the parameter "facing" are returned
         * @param facing if facing and checkFace are true, only the facet "facing" to (x, y, z) are returned : positive dot (x, y, z) * facet position. If facing si false and checkFace is true, only the facet "turning their backs" to (x, y, z) are returned : negative dot (x, y, z) * facet position
         * @returns the face index if found (or null instead)
         * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/facetData
         */
        getClosestFacetAtLocalCoordinates(x: number, y: number, z: number, projected?: Vector3, checkFace?: boolean, facing?: boolean): Nullable<number>;
        /**
         * Returns the object "parameter" set with all the expected parameters for facetData computation by ComputeNormals()
         * @returns the parameters
         * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/facetData
         */
        getFacetDataParameters(): any;
        /**
         * Disables the feature FacetData and frees the related memory
         * @returns the current mesh
         * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/facetData
         */
        disableFacetData(): AbstractMesh;
        /**
         * Updates the AbstractMesh indices array
         * @param indices defines the data source
         * @param offset defines the offset in the index buffer where to store the new data (can be null)
         * @param gpuMemoryOnly defines a boolean indicating that only the GPU memory must be updated leaving the CPU version of the indices unchanged (false by default)
         * @returns the current mesh
         */
        updateIndices(indices: IndicesArray, offset?: number, gpuMemoryOnly?: boolean): AbstractMesh;
        /**
         * Creates new normals data for the mesh
         * @param updatable defines if the normal vertex buffer must be flagged as updatable
         * @returns the current mesh
         */
        createNormals(updatable: boolean): AbstractMesh;
        /**
         * Align the mesh with a normal
         * @param normal defines the normal to use
         * @param upDirection can be used to redefined the up vector to use (will use the (0, 1, 0) by default)
         * @returns the current mesh
         */
        alignWithNormal(normal: Vector3, upDirection?: Vector3): AbstractMesh;
        /** @internal */
        _checkOcclusionQuery(): boolean;
        /**
         * Disables the mesh edge rendering mode
         * @returns the currentAbstractMesh
         */
        disableEdgesRendering(): AbstractMesh;
        /**
         * Enables the edge rendering mode on the mesh.
         * This mode makes the mesh edges visible
         * @param epsilon defines the maximal distance between two angles to detect a face
         * @param checkVerticesInsteadOfIndices indicates that we should check vertex list directly instead of faces
         * @param options options to the edge renderer
         * @returns the currentAbstractMesh
         * @see https://www.babylonjs-playground.com/#19O9TU#0
         */
        enableEdgesRendering(epsilon?: number, checkVerticesInsteadOfIndices?: boolean, options?: IEdgesRendererOptions): AbstractMesh;
        /**
         * This function returns all of the particle systems in the scene that use the mesh as an emitter.
         * @returns an array of particle systems in the scene that use the mesh as an emitter
         */
        getConnectedParticleSystems(): IParticleSystem[];
    }


    /**
     * This is here for backwards compatibility with 4.2
     * @internal
     */


    /**
     * Creates the VertexData for a box
     * @param options an object used to set the following optional parameters for the box, required but can be empty
     * * size sets the width, height and depth of the box to the value of size, optional default 1
     * * width sets the width (x direction) of the box, overwrites the width set by size, optional, default size
     * * height sets the height (y direction) of the box, overwrites the height set by size, optional, default size
     * * depth sets the depth (z direction) of the box, overwrites the depth set by size, optional, default size
     * * faceUV an array of 6 Vector4 elements used to set different images to each box side
     * * faceColors an array of 6 Color3 elements used to set different colors to each box side
     * * sideOrientation optional and takes the values : Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
     * * frontUvs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the front side, optional, default vector4 (0, 0, 1, 1)
     * * backUVs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the back side, optional, default vector4 (0, 0, 1, 1)
     * @param options.size
     * @param options.width
     * @param options.height
     * @param options.depth
     * @param options.faceUV
     * @param options.faceColors
     * @param options.sideOrientation
     * @param options.frontUVs
     * @param options.backUVs
     * @param options.wrap
     * @param options.topBaseAt
     * @param options.bottomBaseAt
     * @returns the VertexData of the box
     */
    export function CreateBoxVertexData(options: {
        size?: number;
        width?: number;
        height?: number;
        depth?: number;
        faceUV?: Vector4[];
        faceColors?: Color4[];
        sideOrientation?: number;
        frontUVs?: Vector4;
        backUVs?: Vector4;
        wrap?: boolean;
        topBaseAt?: number;
        bottomBaseAt?: number;
    }): VertexData;
    /**
     * Creates a box mesh
     * * The parameter `size` sets the size (float) of each box side (default 1)
     * * You can set some different box dimensions by using the parameters `width`, `height` and `depth` (all by default have the same value of `size`)
     * * You can set different colors and different images to each box side by using the parameters `faceColors` (an array of 6 Color3 elements) and `faceUV` (an array of 6 Vector4 elements)
     * * Please read this tutorial : https://doc.babylonjs.com/features/featuresDeepDive/materials/using/texturePerBoxFace
     * * You can also set the mesh side orientation with the values : BABYLON.Mesh.FRONTSIDE (default), BABYLON.Mesh.BACKSIDE or BABYLON.Mesh.DOUBLESIDE
     * * If you create a double-sided mesh, you can choose what parts of the texture image to crop and stick respectively on the front and the back sides with the parameters `frontUVs` and `backUVs` (Vector4). Detail here : https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#side-orientation
     * * The mesh can be set to updatable with the boolean parameter `updatable` (default false) if its internal geometry is supposed to change once created
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#box
     * @param name defines the name of the mesh
     * @param options defines the options used to create the mesh
     * @param options.size
     * @param options.width
     * @param options.height
     * @param options.depth
     * @param options.faceUV
     * @param options.faceColors
     * @param options.sideOrientation
     * @param options.frontUVs
     * @param options.backUVs
     * @param options.wrap
     * @param options.topBaseAt
     * @param options.bottomBaseAt
     * @param options.updatable
     * @param scene defines the hosting scene
     * @returns the box mesh
     */
    export function CreateBox(name: string, options?: {
        size?: number;
        width?: number;
        height?: number;
        depth?: number;
        faceUV?: Vector4[];
        faceColors?: Color4[];
        sideOrientation?: number;
        frontUVs?: Vector4;
        backUVs?: Vector4;
        wrap?: boolean;
        topBaseAt?: number;
        bottomBaseAt?: number;
        updatable?: boolean;
    }, scene?: Nullable<Scene>): Mesh;
    /**
     * Class containing static functions to help procedurally build meshes
     * @deprecated please use CreateBox directly
     */
    export var BoxBuilder: {
        CreateBox: typeof CreateBox;
    };


    /**
     * Scripts based off of https://github.com/maximeq/three-js-capsule-geometry/blob/master/src/CapsuleBufferGeometry.js
     * @param options the constructors options used to shape the mesh.
     * @returns the capsule VertexData
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set/capsule
     */
    export function CreateCapsuleVertexData(options?: ICreateCapsuleOptions): VertexData;
    /**
     * The options Interface for creating a Capsule Mesh
     */
    export interface ICreateCapsuleOptions {
        /** The Orientation of the capsule.  Default : Vector3.Up() */
        orientation?: Vector3;
        /** Number of sub segments on the tube section of the capsule running parallel to orientation. */
        subdivisions?: number;
        /** Number of cylindrical segments on the capsule. */
        tessellation?: number;
        /** Height or Length of the capsule. */
        height?: number;
        /** Radius of the capsule. */
        radius?: number;
        /** Number of sub segments on the cap sections of the capsule running parallel to orientation. */
        capSubdivisions?: number;
        /** Overwrite for the top radius. */
        radiusTop?: number;
        /** Overwrite for the bottom radius. */
        radiusBottom?: number;
        /** Overwrite for the top capSubdivisions. */
        topCapSubdivisions?: number;
        /** Overwrite for the bottom capSubdivisions. */
        bottomCapSubdivisions?: number;
        /** Internal geometry is supposed to change once created. */
        updatable?: boolean;
    }
    /**
     * Creates a capsule or a pill mesh
     * @param name defines the name of the mesh
     * @param options The constructors options.
     * @param scene The scene the mesh is scoped to.
     * @returns Capsule Mesh
     */
    export function CreateCapsule(name: string, options?: ICreateCapsuleOptions, scene?: Nullable<Scene>): Mesh;
    /**
     * Class containing static functions to help procedurally build meshes
     * @deprecated please use CreateCapsule directly
     */
    export var CapsuleBuilder: {
        CreateCapsule: typeof CreateCapsule;
    };


    /**
     * Creates the VertexData for a cylinder, cone or prism
     * @param options an object used to set the following optional parameters for the box, required but can be empty
     * * height sets the height (y direction) of the cylinder, optional, default 2
     * * diameterTop sets the diameter of the top of the cone, overwrites diameter,  optional, default diameter
     * * diameterBottom sets the diameter of the bottom of the cone, overwrites diameter,  optional, default diameter
     * * diameter sets the diameter of the top and bottom of the cone, optional default 1
     * * tessellation the number of prism sides, 3 for a triangular prism, optional, default 24
     * * subdivisions` the number of rings along the cylinder height, optional, default 1
     * * arc a number from 0 to 1, to create an unclosed cylinder based on the fraction of the circumference given by the arc value, optional, default 1
     * * faceColors an array of Color3 elements used to set different colors to the top, rings and bottom respectively
     * * faceUV an array of Vector4 elements used to set different images to the top, rings and bottom respectively
     * * hasRings when true makes each subdivision independently treated as a face for faceUV and faceColors, optional, default false
     * * enclose when true closes an open cylinder by adding extra flat faces between the height axis and vertical edges, think cut cake
     * * sideOrientation optional and takes the values : Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
     * * frontUvs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the front side, optional, default vector4 (0, 0, 1, 1)
     * * backUVs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the back side, optional, default vector4 (0, 0, 1, 1)
     * @param options.height
     * @param options.diameterTop
     * @param options.diameterBottom
     * @param options.diameter
     * @param options.tessellation
     * @param options.subdivisions
     * @param options.arc
     * @param options.faceColors
     * @param options.faceUV
     * @param options.hasRings
     * @param options.enclose
     * @param options.cap
     * @param options.sideOrientation
     * @param options.frontUVs
     * @param options.backUVs
     * @returns the VertexData of the cylinder, cone or prism
     */
    export function CreateCylinderVertexData(options: {
        height?: number;
        diameterTop?: number;
        diameterBottom?: number;
        diameter?: number;
        tessellation?: number;
        subdivisions?: number;
        arc?: number;
        faceColors?: Color4[];
        faceUV?: Vector4[];
        hasRings?: boolean;
        enclose?: boolean;
        cap?: number;
        sideOrientation?: number;
        frontUVs?: Vector4;
        backUVs?: Vector4;
    }): VertexData;
    /**
     * Creates a cylinder or a cone mesh
     * * The parameter `height` sets the height size (float) of the cylinder/cone (float, default 2).
     * * The parameter `diameter` sets the diameter of the top and bottom cap at once (float, default 1).
     * * The parameters `diameterTop` and `diameterBottom` overwrite the parameter `diameter` and set respectively the top cap and bottom cap diameter (floats, default 1). The parameter "diameterBottom" can't be zero.
     * * The parameter `tessellation` sets the number of cylinder sides (positive integer, default 24). Set it to 3 to get a prism for instance.
     * * The parameter `subdivisions` sets the number of rings along the cylinder height (positive integer, default 1).
     * * The parameter `hasRings` (boolean, default false) makes the subdivisions independent from each other, so they become different faces.
     * * The parameter `enclose`  (boolean, default false) adds two extra faces per subdivision to a sliced cylinder to close it around its height axis.
     * * The parameter `cap` sets the way the cylinder is capped. Possible values : BABYLON.Mesh.NO_CAP, BABYLON.Mesh.CAP_START, BABYLON.Mesh.CAP_END, BABYLON.Mesh.CAP_ALL (default).
     * * The parameter `arc` (float, default 1) is the ratio (max 1) to apply to the circumference to slice the cylinder.
     * * You can set different colors and different images to each box side by using the parameters `faceColors` (an array of n Color3 elements) and `faceUV` (an array of n Vector4 elements).
     * * The value of n is the number of cylinder faces. If the cylinder has only 1 subdivisions, n equals : top face + cylinder surface + bottom face = 3
     * * Now, if the cylinder has 5 independent subdivisions (hasRings = true), n equals : top face + 5 stripe surfaces + bottom face = 2 + 5 = 7
     * * Finally, if the cylinder has 5 independent subdivisions and is enclose, n equals : top face + 5 x (stripe surface + 2 closing faces) + bottom face = 2 + 5 * 3 = 17
     * * Each array (color or UVs) is always ordered the same way : the first element is the bottom cap, the last element is the top cap. The other elements are each a ring surface.
     * * If `enclose` is false, a ring surface is one element.
     * * If `enclose` is true, a ring surface is 3 successive elements in the array : the tubular surface, then the two closing faces.
     * * Example how to set colors and textures on a sliced cylinder : https://www.html5gamedevs.com/topic/17945-creating-a-closed-slice-of-a-cylinder/#comment-106379
     * * You can also set the mesh side orientation with the values : BABYLON.Mesh.FRONTSIDE (default), BABYLON.Mesh.BACKSIDE or BABYLON.Mesh.DOUBLESIDE
     * * If you create a double-sided mesh, you can choose what parts of the texture image to crop and stick respectively on the front and the back sides with the parameters `frontUVs` and `backUVs` (Vector4). Detail here : https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#side-orientation
     * * The mesh can be set to updatable with the boolean parameter `updatable` (default false) if its internal geometry is supposed to change once created.
     * @param name defines the name of the mesh
     * @param options defines the options used to create the mesh
     * @param options.height
     * @param options.diameterTop
     * @param options.diameterBottom
     * @param options.diameter
     * @param options.tessellation
     * @param options.subdivisions
     * @param options.arc
     * @param options.faceColors
     * @param options.faceUV
     * @param options.updatable
     * @param options.hasRings
     * @param options.enclose
     * @param options.cap
     * @param options.sideOrientation
     * @param options.frontUVs
     * @param options.backUVs
     * @param scene defines the hosting scene
     * @returns the cylinder mesh
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#cylinder-or-cone
     */
    export function CreateCylinder(name: string, options?: {
        height?: number;
        diameterTop?: number;
        diameterBottom?: number;
        diameter?: number;
        tessellation?: number;
        subdivisions?: number;
        arc?: number;
        faceColors?: Color4[];
        faceUV?: Vector4[];
        updatable?: boolean;
        hasRings?: boolean;
        enclose?: boolean;
        cap?: number;
        sideOrientation?: number;
        frontUVs?: Vector4;
        backUVs?: Vector4;
    }, scene?: Nullable<Scene>): Mesh;
    /**
     * Class containing static functions to help procedurally build meshes
     * @deprecated Please use CreateCylinder directly
     */
    export var CylinderBuilder: {
        CreateCylinder: typeof CreateCylinder;
    };


    /**
     * Creates a decal mesh.
     * A decal is a mesh usually applied as a model onto the surface of another mesh. So don't forget the parameter `sourceMesh` depicting the decal
     * * The parameter `position` (Vector3, default `(0, 0, 0)`) sets the position of the decal in World coordinates
     * * The parameter `normal` (Vector3, default `Vector3.Up`) sets the normal of the mesh where the decal is applied onto in World coordinates
     * * The parameter `size` (Vector3, default `(1, 1, 1)`) sets the decal scaling
     * * The parameter `angle` (float in radian, default 0) sets the angle to rotate the decal
     * * The parameter `captureUVS` defines if we need to capture the uvs or compute them
     * * The parameter `cullBackFaces` defines if the back faces should be removed from the decal mesh
     * * The parameter `localMode` defines that the computations should be done with the local mesh coordinates instead of the world space coordinates.
     * *    Use this mode if you want the decal to be parented to the sourceMesh and move/rotate with it.
     * Note: Meshes with morph targets are not supported!
     * @param name defines the name of the mesh
     * @param sourceMesh defines the mesh where the decal must be applied
     * @param options defines the options used to create the mesh
     * @param options.position
     * @param options.normal
     * @param options.size
     * @param options.angle
     * @param options.captureUVS
     * @param options.cullBackFaces
     * @param options.localMode
     * @returns the decal mesh
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/decals
     */
    export function CreateDecal(name: string, sourceMesh: AbstractMesh, options: {
        position?: Vector3;
        normal?: Vector3;
        size?: Vector3;
        angle?: number;
        captureUVS?: boolean;
        cullBackFaces?: boolean;
        localMode?: boolean;
    }): Mesh;
    /**
     * Class containing static functions to help procedurally build meshes
     * @deprecated use the function directly from the module
     */
    export var DecalBuilder: {
        CreateDecal: typeof CreateDecal;
    };


    /**
     * Creates the VertexData of the Disc or regular Polygon
     * @param options an object used to set the following optional parameters for the disc, required but can be empty
     * * radius the radius of the disc, optional default 0.5
     * * tessellation the number of polygon sides, optional, default 64
     * * arc a number from 0 to 1, to create an unclosed polygon based on the fraction of the circumference given by the arc value, optional, default 1
     * * sideOrientation optional and takes the values : Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
     * * frontUvs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the front side, optional, default vector4 (0, 0, 1, 1)
     * * backUVs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the back side, optional, default vector4 (0, 0, 1, 1)
     * @param options.radius
     * @param options.tessellation
     * @param options.arc
     * @param options.sideOrientation
     * @param options.frontUVs
     * @param options.backUVs
     * @returns the VertexData of the box
     */
    export function CreateDiscVertexData(options: {
        radius?: number;
        tessellation?: number;
        arc?: number;
        sideOrientation?: number;
        frontUVs?: Vector4;
        backUVs?: Vector4;
    }): VertexData;
    /**
     * Creates a plane polygonal mesh.  By default, this is a disc
     * * The parameter `radius` sets the radius size (float) of the polygon (default 0.5)
     * * The parameter `tessellation` sets the number of polygon sides (positive integer, default 64). So a tessellation valued to 3 will build a triangle, to 4 a square, etc
     * * You can create an unclosed polygon with the parameter `arc` (positive float, default 1), valued between 0 and 1, what is the ratio of the circumference : 2 x PI x ratio
     * * You can also set the mesh side orientation with the values : BABYLON.Mesh.FRONTSIDE (default), BABYLON.Mesh.BACKSIDE or BABYLON.Mesh.DOUBLESIDE
     * * If you create a double-sided mesh, you can choose what parts of the texture image to crop and stick respectively on the front and the back sides with the parameters `frontUVs` and `backUVs` (Vector4). Detail here : https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#side-orientation
     * * The mesh can be set to updatable with the boolean parameter `updatable` (default false) if its internal geometry is supposed to change once created
     * @param name defines the name of the mesh
     * @param options defines the options used to create the mesh
     * @param options.radius
     * @param options.tessellation
     * @param options.arc
     * @param options.updatable
     * @param options.sideOrientation
     * @param options.frontUVs
     * @param options.backUVs
     * @param scene defines the hosting scene
     * @returns the plane polygonal mesh
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#disc-or-regular-polygon
     */
    export function CreateDisc(name: string, options?: {
        radius?: number;
        tessellation?: number;
        arc?: number;
        updatable?: boolean;
        sideOrientation?: number;
        frontUVs?: Vector4;
        backUVs?: Vector4;
    }, scene?: Nullable<Scene>): Mesh;
    /**
     * Class containing static functions to help procedurally build meshes
     * @deprecated please use CreateDisc directly
     */
    export var DiscBuilder: {
        CreateDisc: typeof CreateDisc;
    };


    /**
     * Creates the Mesh for a Geodesic Polyhedron
     * @see https://en.wikipedia.org/wiki/Geodesic_polyhedron
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/polyhedra/geodesic_poly
     * @param name defines the name of the mesh
     * @param options an object used to set the following optional parameters for the polyhedron, required but can be empty
     * * m number of horizontal steps along an isogrid
     * * n number of angled steps along an isogrid
     * * size the size of the Geodesic, optional default 1
     * * sizeX allows stretching in the x direction, optional, default size
     * * sizeY allows stretching in the y direction, optional, default size
     * * sizeZ allows stretching in the z direction, optional, default size
     * * faceUV an array of Vector4 elements used to set different images to the top, rings and bottom respectively
     * * faceColors an array of Color3 elements used to set different colors to the top, rings and bottom respectively
     * * flat when true creates a flat shaded mesh, optional, default true
     * * subdivisions increasing the subdivisions increases the number of faces, optional, default 4
     * * sideOrientation optional and takes the values : Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
     * * frontUvs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the front side, optional, default vector4 (0, 0, 1, 1)
     * * backUVs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the back side, optional, default vector4 (0, 0, 1, 1)
     * @param options.n
     * @param options.size
     * @param options.sizeX
     * @param options.sizeY
     * @param options.sizeZ
     * @param options.faceUV
     * @param options.faceColors
     * @param options.flat
     * @param options.updatable
     * @param options.sideOrientation
     * @param options.frontUVs
     * @param options.backUVs
     * @param options.m
     * @param scene defines the hosting scene
     * @returns Geodesic mesh
     */
    export function CreateGeodesic(name: string, options: {
        m?: number;
        n?: number;
        size?: number;
        sizeX?: number;
        sizeY?: number;
        sizeZ?: number;
        faceUV?: Vector4[];
        faceColors?: Color4[];
        flat?: boolean;
        updatable?: boolean;
        sideOrientation?: number;
        frontUVs?: Vector4;
        backUVs?: Vector4;
    }, scene?: Nullable<Scene>): Mesh;


    /**
     * Defines the set of data required to create goldberg vertex data.
     */
    export type GoldbergVertexDataOption = {
        /**
         * the size of the Goldberg, optional default 1
         */
        size?: number;
        /**
         * allows stretching in the x direction, optional, default size
         */
        sizeX?: number;
        /**
         * allows stretching in the y direction, optional, default size
         */
        sizeY?: number;
        /**
         * allows stretching in the z direction, optional, default size
         */
        sizeZ?: number;
        /**
         * optional and takes the values : Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
         */
        sideOrientation?: number;
    };
    /**
     * Defines the set of data required to create a goldberg mesh.
     */
    export type GoldbergCreationOption = {
        /**
         * number of horizontal steps along an isogrid
         */
        m?: number;
        /**
         * number of angled steps along an isogrid
         */
        n?: number;
        /**
         * defines if the mesh must be flagged as updatable
         */
        updatable?: boolean;
    } & GoldbergVertexDataOption;
    /**
     * Creates the Mesh for a Goldberg Polyhedron
     * @param options an object used to set the following optional parameters for the polyhedron, required but can be empty
     * @param goldbergData polyhedronData defining the Goldberg polyhedron
     * @returns GoldbergSphere mesh
     */
    export function CreateGoldbergVertexData(options: GoldbergVertexDataOption, goldbergData: PolyhedronData): VertexData;
    /**
     * Creates the Mesh for a Goldberg Polyhedron which is made from 12 pentagonal and the rest hexagonal faces
     * @see https://en.wikipedia.org/wiki/Goldberg_polyhedron
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/polyhedra/goldberg_poly
     * @param name defines the name of the mesh
     * @param options an object used to set the following optional parameters for the polyhedron, required but can be empty
     * @param scene defines the hosting scene
     * @returns Goldberg mesh
     */
    export function CreateGoldberg(name: string, options: GoldbergCreationOption, scene?: Nullable<Scene>): GoldbergMesh;


    /**
     * Creates the VertexData for a Ground
     * @param options an object used to set the following optional parameters for the Ground, required but can be empty
     *  - width the width (x direction) of the ground, optional, default 1
     *  - height the height (z direction) of the ground, optional, default 1
     *  - subdivisions the number of subdivisions per side, optional, default 1
     * @param options.width
     * @param options.height
     * @param options.subdivisions
     * @param options.subdivisionsX
     * @param options.subdivisionsY
     * @returns the VertexData of the Ground
     */
    export function CreateGroundVertexData(options: {
        width?: number;
        height?: number;
        subdivisions?: number;
        subdivisionsX?: number;
        subdivisionsY?: number;
    }): VertexData;
    /**
     * Creates the VertexData for a TiledGround by subdividing the ground into tiles
     * @param options an object used to set the following optional parameters for the Ground, required but can be empty
     * * xmin the ground minimum X coordinate, optional, default -1
     * * zmin the ground minimum Z coordinate, optional, default -1
     * * xmax the ground maximum X coordinate, optional, default 1
     * * zmax the ground maximum Z coordinate, optional, default 1
     * * subdivisions a javascript object {w: positive integer, h: positive integer}, `w` and `h` are the numbers of subdivisions on the ground width and height creating 'tiles', default {w: 6, h: 6}
     * * precision a javascript object {w: positive integer, h: positive integer}, `w` and `h` are the numbers of subdivisions on the tile width and height, default {w: 2, h: 2}
     * @param options.xmin
     * @param options.zmin
     * @param options.xmax
     * @param options.zmax
     * @param options.subdivisions
     * @param options.subdivisions.w
     * @param options.subdivisions.h
     * @param options.precision
     * @param options.precision.w
     * @param options.precision.h
     * @returns the VertexData of the TiledGround
     */
    export function CreateTiledGroundVertexData(options: {
        xmin: number;
        zmin: number;
        xmax: number;
        zmax: number;
        subdivisions?: {
            w: number;
            h: number;
        };
        precision?: {
            w: number;
            h: number;
        };
    }): VertexData;
    /**
     * Creates the VertexData of the Ground designed from a heightmap
     * @param options an object used to set the following parameters for the Ground, required and provided by CreateGroundFromHeightMap
     * * width the width (x direction) of the ground
     * * height the height (z direction) of the ground
     * * subdivisions the number of subdivisions per side
     * * minHeight the minimum altitude on the ground, optional, default 0
     * * maxHeight the maximum altitude on the ground, optional default 1
     * * colorFilter the filter to apply to the image pixel colors to compute the height, optional Color3, default (0.3, 0.59, 0.11)
     * * buffer the array holding the image color data
     * * bufferWidth the width of image
     * * bufferHeight the height of image
     * * alphaFilter Remove any data where the alpha channel is below this value, defaults 0 (all data visible)
     * @param options.width
     * @param options.height
     * @param options.subdivisions
     * @param options.minHeight
     * @param options.maxHeight
     * @param options.colorFilter
     * @param options.buffer
     * @param options.bufferWidth
     * @param options.bufferHeight
     * @param options.alphaFilter
     * @returns the VertexData of the Ground designed from a heightmap
     */
    export function CreateGroundFromHeightMapVertexData(options: {
        width: number;
        height: number;
        subdivisions: number;
        minHeight: number;
        maxHeight: number;
        colorFilter: Color3;
        buffer: Uint8Array;
        bufferWidth: number;
        bufferHeight: number;
        alphaFilter: number;
    }): VertexData;
    /**
     * Creates a ground mesh
     * * The parameters `width` and `height` (floats, default 1) set the width and height sizes of the ground
     * * The parameter `subdivisions` (positive integer) sets the number of subdivisions per side
     * * The mesh can be set to updatable with the boolean parameter `updatable` (default false) if its internal geometry is supposed to change once created
     * @param name defines the name of the mesh
     * @param options defines the options used to create the mesh
     * @param options.width
     * @param options.height
     * @param options.subdivisions
     * @param options.subdivisionsX
     * @param options.subdivisionsY
     * @param options.updatable
     * @param scene defines the hosting scene
     * @returns the ground mesh
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#ground
     */
    export function CreateGround(name: string, options?: {
        width?: number;
        height?: number;
        subdivisions?: number;
        subdivisionsX?: number;
        subdivisionsY?: number;
        updatable?: boolean;
    }, scene?: Scene): GroundMesh;
    /**
     * Creates a tiled ground mesh
     * * The parameters `xmin` and `xmax` (floats, default -1 and 1) set the ground minimum and maximum X coordinates
     * * The parameters `zmin` and `zmax` (floats, default -1 and 1) set the ground minimum and maximum Z coordinates
     * * The parameter `subdivisions` is a javascript object `{w: positive integer, h: positive integer}` (default `{w: 6, h: 6}`). `w` and `h` are the numbers of subdivisions on the ground width and height. Each subdivision is called a tile
     * * The parameter `precision` is a javascript object `{w: positive integer, h: positive integer}` (default `{w: 2, h: 2}`). `w` and `h` are the numbers of subdivisions on the ground width and height of each tile
     * * The mesh can be set to updatable with the boolean parameter `updatable` (default false) if its internal geometry is supposed to change once created.
     * @param name defines the name of the mesh
     * @param options defines the options used to create the mesh
     * @param options.xmin
     * @param options.zmin
     * @param options.xmax
     * @param options.zmax
     * @param options.subdivisions
     * @param options.subdivisions.w
     * @param options.subdivisions.h
     * @param options.precision
     * @param options.precision.w
     * @param options.precision.h
     * @param options.updatable
     * @param scene defines the hosting scene
     * @returns the tiled ground mesh
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#tiled-ground
     */
    export function CreateTiledGround(name: string, options: {
        xmin: number;
        zmin: number;
        xmax: number;
        zmax: number;
        subdivisions?: {
            w: number;
            h: number;
        };
        precision?: {
            w: number;
            h: number;
        };
        updatable?: boolean;
    }, scene?: Nullable<Scene>): Mesh;
    /**
     * Creates a ground mesh from a height map
     * * The parameter `url` sets the URL of the height map image resource.
     * * The parameters `width` and `height` (positive floats, default 10) set the ground width and height sizes.
     * * The parameter `subdivisions` (positive integer, default 1) sets the number of subdivision per side.
     * * The parameter `minHeight` (float, default 0) is the minimum altitude on the ground.
     * * The parameter `maxHeight` (float, default 1) is the maximum altitude on the ground.
     * * The parameter `colorFilter` (optional Color3, default (0.3, 0.59, 0.11) ) is the filter to apply to the image pixel colors to compute the height.
     * * The parameter `onReady` is a javascript callback function that will be called  once the mesh is just built (the height map download can last some time).
     * * The parameter `alphaFilter` will filter any data where the alpha channel is below this value, defaults 0 (all data visible)
     * * The mesh can be set to updatable with the boolean parameter `updatable` (default false) if its internal geometry is supposed to change once created.
     * @param name defines the name of the mesh
     * @param url defines the url to the height map
     * @param options defines the options used to create the mesh
     * @param options.width
     * @param options.height
     * @param options.subdivisions
     * @param options.minHeight
     * @param options.maxHeight
     * @param options.colorFilter
     * @param options.alphaFilter
     * @param options.updatable
     * @param options.onReady
     * @param scene defines the hosting scene
     * @returns the ground mesh
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set/height_map
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#ground-from-a-height-map
     */
    export function CreateGroundFromHeightMap(name: string, url: string, options?: {
        width?: number;
        height?: number;
        subdivisions?: number;
        minHeight?: number;
        maxHeight?: number;
        colorFilter?: Color3;
        alphaFilter?: number;
        updatable?: boolean;
        onReady?: (mesh: GroundMesh) => void;
    }, scene?: Nullable<Scene>): GroundMesh;
    /**
     * Class containing static functions to help procedurally build meshes
     * @deprecated use the functions directly from the module
     */
    export var GroundBuilder: {
        CreateGround: typeof CreateGround;
        CreateGroundFromHeightMap: typeof CreateGroundFromHeightMap;
        CreateTiledGround: typeof CreateTiledGround;
    };


    /**
     * Creates a hemisphere mesh
     * @param name defines the name of the mesh
     * @param options defines the options used to create the mesh
     * @param options.segments
     * @param options.diameter
     * @param options.sideOrientation
     * @param scene defines the hosting scene
     * @returns the hemisphere mesh
     */
    export function CreateHemisphere(name: string, options?: {
        segments?: number;
        diameter?: number;
        sideOrientation?: number;
    }, scene?: Scene): Mesh;
    /**
     * Class containing static functions to help procedurally build meshes
     * @deprecated use the function directly from the module
     */
    export var HemisphereBuilder: {
        CreateHemisphere: typeof CreateHemisphere;
    };


    /**
     * Creates the VertexData of the IcoSphere
     * @param options an object used to set the following optional parameters for the IcoSphere, required but can be empty
     * * radius the radius of the IcoSphere, optional default 1
     * * radiusX allows stretching in the x direction, optional, default radius
     * * radiusY allows stretching in the y direction, optional, default radius
     * * radiusZ allows stretching in the z direction, optional, default radius
     * * flat when true creates a flat shaded mesh, optional, default true
     * * subdivisions increasing the subdivisions increases the number of faces, optional, default 4
     * * sideOrientation optional and takes the values : Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
     * * frontUvs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the front side, optional, default vector4 (0, 0, 1, 1)
     * * backUVs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the back side, optional, default vector4 (0, 0, 1, 1)
     * @param options.radius
     * @param options.radiusX
     * @param options.radiusY
     * @param options.radiusZ
     * @param options.flat
     * @param options.subdivisions
     * @param options.sideOrientation
     * @param options.frontUVs
     * @param options.backUVs
     * @returns the VertexData of the IcoSphere
     */
    export function CreateIcoSphereVertexData(options: {
        radius?: number;
        radiusX?: number;
        radiusY?: number;
        radiusZ?: number;
        flat?: boolean;
        subdivisions?: number;
        sideOrientation?: number;
        frontUVs?: Vector4;
        backUVs?: Vector4;
    }): VertexData;
    /**
     * Creates a sphere based upon an icosahedron with 20 triangular faces which can be subdivided
     * * The parameter `radius` sets the radius size (float) of the icosphere (default 1)
     * * You can set some different icosphere dimensions, for instance to build an ellipsoid, by using the parameters `radiusX`, `radiusY` and `radiusZ` (all by default have the same value of `radius`)
     * * The parameter `subdivisions` sets the number of subdivisions (positive integer, default 4). The more subdivisions, the more faces on the icosphere whatever its size
     * * The parameter `flat` (boolean, default true) gives each side its own normals. Set it to false to get a smooth continuous light reflection on the surface
     * * You can also set the mesh side orientation with the values : BABYLON.Mesh.FRONTSIDE (default), BABYLON.Mesh.BACKSIDE or BABYLON.Mesh.DOUBLESIDE
     * * If you create a double-sided mesh, you can choose what parts of the texture image to crop and stick respectively on the front and the back sides with the parameters `frontUVs` and `backUVs` (Vector4). Detail here : https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#side-orientation
     * * The mesh can be set to updatable with the boolean parameter `updatable` (default false) if its internal geometry is supposed to change once created
     * @param name defines the name of the mesh
     * @param options defines the options used to create the mesh
     * @param options.radius
     * @param options.radiusX
     * @param options.radiusY
     * @param options.radiusZ
     * @param options.flat
     * @param options.subdivisions
     * @param options.sideOrientation
     * @param options.frontUVs
     * @param options.backUVs
     * @param options.updatable
     * @param scene defines the hosting scene
     * @returns the icosahedron mesh
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/polyhedra#icosphere
     */
    export function CreateIcoSphere(name: string, options?: {
        radius?: number;
        radiusX?: number;
        radiusY?: number;
        radiusZ?: number;
        flat?: boolean;
        subdivisions?: number;
        sideOrientation?: number;
        frontUVs?: Vector4;
        backUVs?: Vector4;
        updatable?: boolean;
    }, scene?: Nullable<Scene>): Mesh;
    /**
     * Class containing static functions to help procedurally build meshes
     * @deprecated use the function directly from the module
     */
    export var IcoSphereBuilder: {
        CreateIcoSphere: typeof CreateIcoSphere;
    };




    /**
     * Creates lathe mesh.
     * The lathe is a shape with a symmetry axis : a 2D model shape is rotated around this axis to design the lathe
     * * The parameter `shape` is a required array of successive Vector3. This array depicts the shape to be rotated in its local space : the shape must be designed in the xOy plane and will be rotated around the Y axis. It's usually a 2D shape, so the Vector3 z coordinates are often set to zero
     * * The parameter `radius` (positive float, default 1) is the radius value of the lathe
     * * The parameter `tessellation` (positive integer, default 64) is the side number of the lathe
     * * The parameter `clip` (positive integer, default 0) is the number of sides to not create without effecting the general shape of the sides
     * * The parameter `arc` (positive float, default 1) is the ratio of the lathe. 0.5 builds for instance half a lathe, so an opened shape
     * * The parameter `closed` (boolean, default true) opens/closes the lathe circumference. This should be set to false when used with the parameter "arc"
     * * The parameter `cap` sets the way the extruded shape is capped. Possible values : BABYLON.Mesh.NO_CAP (default), BABYLON.Mesh.CAP_START, BABYLON.Mesh.CAP_END, BABYLON.Mesh.CAP_ALL
     * * You can also set the mesh side orientation with the values : BABYLON.Mesh.FRONTSIDE (default), BABYLON.Mesh.BACKSIDE or BABYLON.Mesh.DOUBLESIDE
     * * If you create a double-sided mesh, you can choose what parts of the texture image to crop and stick respectively on the front and the back sides with the parameters `frontUVs` and `backUVs` (Vector4). Detail here : https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#side-orientation
     * * The optional parameter `invertUV` (boolean, default false) swaps in the geometry the U and V coordinates to apply a texture
     * * The mesh can be set to updatable with the boolean parameter `updatable` (default false) if its internal geometry is supposed to change once created
     * @param name defines the name of the mesh
     * @param options defines the options used to create the mesh
     * @param options.shape
     * @param options.radius
     * @param options.tessellation
     * @param options.clip
     * @param options.arc
     * @param options.closed
     * @param options.updatable
     * @param options.sideOrientation
     * @param options.frontUVs
     * @param options.backUVs
     * @param options.cap
     * @param options.invertUV
     * @param scene defines the hosting scene
     * @returns the lathe mesh
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/param#lathe
     */
    export function CreateLathe(name: string, options: {
        shape: Vector3[];
        radius?: number;
        tessellation?: number;
        clip?: number;
        arc?: number;
        closed?: boolean;
        updatable?: boolean;
        sideOrientation?: number;
        frontUVs?: Vector4;
        backUVs?: Vector4;
        cap?: number;
        invertUV?: boolean;
    }, scene?: Nullable<Scene>): Mesh;
    /**
     * Class containing static functions to help procedurally build meshes
     * @deprecated use the function direction from the module
     */
    export var LatheBuilder: {
        CreateLathe: typeof CreateLathe;
    };


    /**
     * Creates the VertexData of the LineSystem
     * @param options an object used to set the following optional parameters for the LineSystem, required but can be empty
     *  - lines an array of lines, each line being an array of successive Vector3
     *  - colors an array of line colors, each of the line colors being an array of successive Color4, one per line point
     * @param options.lines
     * @param options.colors
     * @returns the VertexData of the LineSystem
     */
    export function CreateLineSystemVertexData(options: {
        lines: Vector3[][];
        colors?: Nullable<Color4[][]>;
    }): VertexData;
    /**
     * Create the VertexData for a DashedLines
     * @param options an object used to set the following optional parameters for the DashedLines, required but can be empty
     *  - points an array successive Vector3
     *  - dashSize the size of the dashes relative to the dash number, optional, default 3
     *  - gapSize the size of the gap between two successive dashes relative to the dash number, optional, default 1
     *  - dashNb the intended total number of dashes, optional, default 200
     * @param options.points
     * @param options.dashSize
     * @param options.gapSize
     * @param options.dashNb
     * @returns the VertexData for the DashedLines
     */
    export function CreateDashedLinesVertexData(options: {
        points: Vector3[];
        dashSize?: number;
        gapSize?: number;
        dashNb?: number;
    }): VertexData;
    /**
     * Creates a line system mesh. A line system is a pool of many lines gathered in a single mesh
     * * A line system mesh is considered as a parametric shape since it has no predefined original shape. Its shape is determined by the passed array of lines as an input parameter
     * * Like every other parametric shape, it is dynamically updatable by passing an existing instance of LineSystem to this static function
     * * The parameter `lines` is an array of lines, each line being an array of successive Vector3
     * * The optional parameter `instance` is an instance of an existing LineSystem object to be updated with the passed `lines` parameter
     * * The optional parameter `colors` is an array of line colors, each line colors being an array of successive Color4, one per line point
     * * The optional parameter `useVertexAlpha` is to be set to `false` (default `true`) when you don't need the alpha blending (faster)
     * * The optional parameter `material` is the material to use to draw the lines if provided. If not, a default material will be created
     * * Updating a simple Line mesh, you just need to update every line in the `lines` array : https://doc.babylonjs.com/features/featuresDeepDive/mesh/dynamicMeshMorph#lines-and-dashedlines
     * * When updating an instance, remember that only line point positions can change, not the number of points, neither the number of lines
     * * The mesh can be set to updatable with the boolean parameter `updatable` (default false) if its internal geometry is supposed to change once created
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/param#line-system
     * @param name defines the name of the new line system
     * @param options defines the options used to create the line system
     * @param options.lines
     * @param options.updatable
     * @param options.instance
     * @param options.colors
     * @param options.useVertexAlpha
     * @param options.material
     * @param scene defines the hosting scene
     * @returns a new line system mesh
     */
    export function CreateLineSystem(name: string, options: {
        lines: Vector3[][];
        updatable?: boolean;
        instance?: Nullable<LinesMesh>;
        colors?: Nullable<Color4[][]>;
        useVertexAlpha?: boolean;
        material?: Material;
    }, scene: Nullable<Scene>): LinesMesh;
    /**
     * Creates a line mesh
     * A line mesh is considered as a parametric shape since it has no predefined original shape. Its shape is determined by the passed array of points as an input parameter
     * * Like every other parametric shape, it is dynamically updatable by passing an existing instance of LineMesh to this static function
     * * The parameter `points` is an array successive Vector3
     * * The optional parameter `instance` is an instance of an existing LineMesh object to be updated with the passed `points` parameter : https://doc.babylonjs.com/features/featuresDeepDive/mesh/dynamicMeshMorph#lines-and-dashedlines
     * * The optional parameter `colors` is an array of successive Color4, one per line point
     * * The optional parameter `useVertexAlpha` is to be set to `false` (default `true`) when you don't need alpha blending (faster)
     * * The optional parameter `material` is the material to use to draw the lines if provided. If not, a default material will be created
     * * When updating an instance, remember that only point positions can change, not the number of points
     * * The mesh can be set to updatable with the boolean parameter `updatable` (default false) if its internal geometry is supposed to change once created
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/param#lines
     * @param name defines the name of the new line system
     * @param options defines the options used to create the line system
     * @param options.points
     * @param options.updatable
     * @param options.instance
     * @param options.colors
     * @param options.useVertexAlpha
     * @param options.material
     * @param scene defines the hosting scene
     * @returns a new line mesh
     */
    export function CreateLines(name: string, options: {
        points: Vector3[];
        updatable?: boolean;
        instance?: Nullable<LinesMesh>;
        colors?: Color4[];
        useVertexAlpha?: boolean;
        material?: Material;
    }, scene?: Nullable<Scene>): LinesMesh;
    /**
     * Creates a dashed line mesh
     * * A dashed line mesh is considered as a parametric shape since it has no predefined original shape. Its shape is determined by the passed array of points as an input parameter
     * * Like every other parametric shape, it is dynamically updatable by passing an existing instance of LineMesh to this static function
     * * The parameter `points` is an array successive Vector3
     * * The parameter `dashNb` is the intended total number of dashes (positive integer, default 200)
     * * The parameter `dashSize` is the size of the dashes relatively the dash number (positive float, default 3)
     * * The parameter `gapSize` is the size of the gap between two successive dashes relatively the dash number (positive float, default 1)
     * * The optional parameter `instance` is an instance of an existing LineMesh object to be updated with the passed `points` parameter : https://doc.babylonjs.com/features/featuresDeepDive/mesh/dynamicMeshMorph#lines-and-dashedlines
     * * The optional parameter `useVertexAlpha` is to be set to `false` (default `true`) when you don't need the alpha blending (faster)
     * * The optional parameter `material` is the material to use to draw the lines if provided. If not, a default material will be created
     * * When updating an instance, remember that only point positions can change, not the number of points
     * * The mesh can be set to updatable with the boolean parameter `updatable` (default false) if its internal geometry is supposed to change once created
     * @param name defines the name of the mesh
     * @param options defines the options used to create the mesh
     * @param options.points
     * @param options.dashSize
     * @param options.gapSize
     * @param options.dashNb
     * @param options.updatable
     * @param options.instance
     * @param options.useVertexAlpha
     * @param options.material
     * @param scene defines the hosting scene
     * @returns the dashed line mesh
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/param#dashed-lines
     */
    export function CreateDashedLines(name: string, options: {
        points: Vector3[];
        dashSize?: number;
        gapSize?: number;
        dashNb?: number;
        updatable?: boolean;
        instance?: LinesMesh;
        useVertexAlpha?: boolean;
        material?: Material;
    }, scene?: Nullable<Scene>): LinesMesh;
    /**
     * Class containing static functions to help procedurally build meshes
     * @deprecated use the functions directly from the module
     */
    export var LinesBuilder: {
        CreateDashedLines: typeof CreateDashedLines;
        CreateLineSystem: typeof CreateLineSystem;
        CreateLines: typeof CreateLines;
    };


    /**
     * Creates the VertexData for a Plane
     * @param options an object used to set the following optional parameters for the plane, required but can be empty
     * * size sets the width and height of the plane to the value of size, optional default 1
     * * width sets the width (x direction) of the plane, overwrites the width set by size, optional, default size
     * * height sets the height (y direction) of the plane, overwrites the height set by size, optional, default size
     * * sideOrientation optional and takes the values : Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
     * * frontUvs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the front side, optional, default vector4 (0, 0, 1, 1)
     * * backUVs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the back side, optional, default vector4 (0, 0, 1, 1)
     * @param options.size
     * @param options.width
     * @param options.height
     * @param options.sideOrientation
     * @param options.frontUVs
     * @param options.backUVs
     * @returns the VertexData of the box
     */
    export function CreatePlaneVertexData(options: {
        size?: number;
        width?: number;
        height?: number;
        sideOrientation?: number;
        frontUVs?: Vector4;
        backUVs?: Vector4;
    }): VertexData;
    /**
     * Creates a plane mesh
     * * The parameter `size` sets the size (float) of both sides of the plane at once (default 1)
     * * You can set some different plane dimensions by using the parameters `width` and `height` (both by default have the same value of `size`)
     * * The parameter `sourcePlane` is a Plane instance. It builds a mesh plane from a Math plane
     * * You can also set the mesh side orientation with the values : BABYLON.Mesh.FRONTSIDE (default), BABYLON.Mesh.BACKSIDE or BABYLON.Mesh.DOUBLESIDE
     * * If you create a double-sided mesh, you can choose what parts of the texture image to crop and stick respectively on the front and the back sides with the parameters `frontUVs` and `backUVs` (Vector4). Detail here : https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#side-orientation
     * * The mesh can be set to updatable with the boolean parameter `updatable` (default false) if its internal geometry is supposed to change once created
     * @param name defines the name of the mesh
     * @param options defines the options used to create the mesh
     * @param options.size
     * @param options.width
     * @param options.height
     * @param options.sideOrientation
     * @param options.frontUVs
     * @param options.backUVs
     * @param options.updatable
     * @param options.sourcePlane
     * @param scene defines the hosting scene
     * @returns the plane mesh
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#plane
     */
    export function CreatePlane(name: string, options?: {
        size?: number;
        width?: number;
        height?: number;
        sideOrientation?: number;
        frontUVs?: Vector4;
        backUVs?: Vector4;
        updatable?: boolean;
        sourcePlane?: Plane;
    }, scene?: Nullable<Scene>): Mesh;
    /**
     * Class containing static functions to help procedurally build meshes
     * @deprecated use the function directly from the module
     */
    export var PlaneBuilder: {
        CreatePlane: typeof CreatePlane;
    };


    /**
     * Creates the VertexData for an irregular Polygon in the XoZ plane using a mesh built by polygonTriangulation.build()
     * All parameters are provided by CreatePolygon as needed
     * @param polygon a mesh built from polygonTriangulation.build()
     * @param sideOrientation takes the values Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
     * @param fUV an array of Vector4 elements used to set different images to the top, rings and bottom respectively
     * @param fColors an array of Color3 elements used to set different colors to the top, rings and bottom respectively
     * @param frontUVs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the front side, optional, default vector4 (0, 0, 1, 1)
     * @param backUVs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the back side, optional, default vector4 (0, 0, 1, 1)
     * @param wrp a boolean, default false, when true and fUVs used texture is wrapped around all sides, when false texture is applied side
     * @returns the VertexData of the Polygon
     */
    export function CreatePolygonVertexData(polygon: Mesh, sideOrientation: number, fUV?: Vector4[], fColors?: Color4[], frontUVs?: Vector4, backUVs?: Vector4, wrp?: boolean): VertexData;
    /**
     * Creates a polygon mesh
     * The polygon's shape will depend on the input parameters and is constructed parallel to a ground mesh
     * * The parameter `shape` is a required array of successive Vector3 representing the corners of the polygon in th XoZ plane, that is y = 0 for all vectors
     * * You can set the mesh side orientation with the values : Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
     * * The mesh can be set to updatable with the boolean parameter `updatable` (default false) if its internal geometry is supposed to change once created
     * * If you create a double-sided mesh, you can choose what parts of the texture image to crop and stick respectively on the front and the back sides with the parameters `frontUVs` and `backUVs` (Vector4)
     * * Remember you can only change the shape positions, not their number when updating a polygon
     * @param name defines the name of the mesh
     * @param options defines the options used to create the mesh
     * @param options.shape
     * @param options.holes
     * @param options.depth
     * @param options.smoothingThreshold
     * @param options.faceUV
     * @param options.faceColors
     * @param options.updatable
     * @param options.sideOrientation
     * @param options.frontUVs
     * @param options.backUVs
     * @param options.wrap
     * @param scene defines the hosting scene
     * @param earcutInjection can be used to inject your own earcut reference
     * @returns the polygon mesh
     */
    export function CreatePolygon(name: string, options: {
        shape: Vector3[];
        holes?: Vector3[][];
        depth?: number;
        smoothingThreshold?: number;
        faceUV?: Vector4[];
        faceColors?: Color4[];
        updatable?: boolean;
        sideOrientation?: number;
        frontUVs?: Vector4;
        backUVs?: Vector4;
        wrap?: boolean;
    }, scene?: Nullable<Scene>, earcutInjection?: any): Mesh;
    /**
     * Creates an extruded polygon mesh, with depth in the Y direction.
     * * You can set different colors and different images to the top, bottom and extruded side by using the parameters `faceColors` (an array of 3 Color3 elements) and `faceUV` (an array of 3 Vector4 elements)
     * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/using/texturePerBoxFace
     * @param name defines the name of the mesh
     * @param options defines the options used to create the mesh
     * @param options.shape
     * @param options.holes
     * @param options.depth
     * @param options.faceUV
     * @param options.faceColors
     * @param options.updatable
     * @param options.sideOrientation
     * @param options.frontUVs
     * @param options.backUVs
     * @param options.wrap
     * @param scene defines the hosting scene
     * @param earcutInjection can be used to inject your own earcut reference
     * @returns the polygon mesh
     */
    export function ExtrudePolygon(name: string, options: {
        shape: Vector3[];
        holes?: Vector3[][];
        depth?: number;
        faceUV?: Vector4[];
        faceColors?: Color4[];
        updatable?: boolean;
        sideOrientation?: number;
        frontUVs?: Vector4;
        backUVs?: Vector4;
        wrap?: boolean;
    }, scene?: Nullable<Scene>, earcutInjection?: any): Mesh;
    /**
     * Class containing static functions to help procedurally build meshes
     * @deprecated use the functions directly from the module
     */
    export var PolygonBuilder: {
        ExtrudePolygon: typeof ExtrudePolygon;
        CreatePolygon: typeof CreatePolygon;
    };


    /**
     * Creates the VertexData for a Polyhedron
     * @param options an object used to set the following optional parameters for the polyhedron, required but can be empty
     * * type provided types are:
     *  * 0 : Tetrahedron, 1 : Octahedron, 2 : Dodecahedron, 3 : Icosahedron, 4 : Rhombicuboctahedron, 5 : Triangular Prism, 6 : Pentagonal Prism, 7 : Hexagonal Prism, 8 : Square Pyramid (J1)
     *  * 9 : Pentagonal Pyramid (J2), 10 : Triangular Dipyramid (J12), 11 : Pentagonal Dipyramid (J13), 12 : Elongated Square Dipyramid (J15), 13 : Elongated Pentagonal Dipyramid (J16), 14 : Elongated Pentagonal Cupola (J20)
     * * size the size of the IcoSphere, optional default 1
     * * sizeX allows stretching in the x direction, optional, default size
     * * sizeY allows stretching in the y direction, optional, default size
     * * sizeZ allows stretching in the z direction, optional, default size
     * * custom a number that overwrites the type to create from an extended set of polyhedron from https://www.babylonjs-playground.com/#21QRSK#15 with minimised editor
     * * faceUV an array of Vector4 elements used to set different images to the top, rings and bottom respectively
     * * faceColors an array of Color3 elements used to set different colors to the top, rings and bottom respectively
     * * flat when true creates a flat shaded mesh, optional, default true
     * * subdivisions increasing the subdivisions increases the number of faces, optional, default 4
     * * sideOrientation optional and takes the values : Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
     * * frontUvs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the front side, optional, default vector4 (0, 0, 1, 1)
     * * backUVs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the back side, optional, default vector4 (0, 0, 1, 1)
     * @param options.type
     * @param options.size
     * @param options.sizeX
     * @param options.sizeY
     * @param options.sizeZ
     * @param options.custom
     * @param options.faceUV
     * @param options.faceColors
     * @param options.flat
     * @param options.sideOrientation
     * @param options.frontUVs
     * @param options.backUVs
     * @returns the VertexData of the Polyhedron
     */
    export function CreatePolyhedronVertexData(options: {
        type?: number;
        size?: number;
        sizeX?: number;
        sizeY?: number;
        sizeZ?: number;
        custom?: any;
        faceUV?: Vector4[];
        faceColors?: Color4[];
        flat?: boolean;
        sideOrientation?: number;
        frontUVs?: Vector4;
        backUVs?: Vector4;
    }): VertexData;
    /**
     * Creates a polyhedron mesh
     * * The parameter `type` (positive integer, max 14, default 0) sets the polyhedron type to build among the 15 embbeded types. Please refer to the type sheet in the tutorial to choose the wanted type
     * * The parameter `size` (positive float, default 1) sets the polygon size
     * * You can overwrite the `size` on each dimension bu using the parameters `sizeX`, `sizeY` or `sizeZ` (positive floats, default to `size` value)
     * * You can build other polyhedron types than the 15 embbeded ones by setting the parameter `custom` (`polyhedronObject`, default null). If you set the parameter `custom`, this overrides the parameter `type`
     * * A `polyhedronObject` is a formatted javascript object. You'll find a full file with pre-set polyhedra here : https://github.com/BabylonJS/Extensions/tree/master/Polyhedron
     * * You can set the color and the UV of each side of the polyhedron with the parameters `faceColors` (Color4, default `(1, 1, 1, 1)`) and faceUV (Vector4, default `(0, 0, 1, 1)`)
     * * To understand how to set `faceUV` or `faceColors`, please read this by considering the right number of faces of your polyhedron, instead of only 6 for the box : https://doc.babylonjs.com/features/featuresDeepDive/materials/using/texturePerBoxFace
     * * The parameter `flat` (boolean, default true). If set to false, it gives the polyhedron a single global face, so less vertices and shared normals. In this case, `faceColors` and `faceUV` are ignored
     * * You can also set the mesh side orientation with the values : BABYLON.Mesh.FRONTSIDE (default), BABYLON.Mesh.BACKSIDE or BABYLON.Mesh.DOUBLESIDE
     * * If you create a double-sided mesh, you can choose what parts of the texture image to crop and stick respectively on the front and the back sides with the parameters `frontUVs` and `backUVs` (Vector4). Detail here : https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#side-orientation
     * * The mesh can be set to updatable with the boolean parameter `updatable` (default false) if its internal geometry is supposed to change once created
     * @param name defines the name of the mesh
     * @param options defines the options used to create the mesh
     * @param options.type
     * @param options.size
     * @param options.sizeX
     * @param options.sizeY
     * @param options.sizeZ
     * @param options.custom
     * @param options.faceUV
     * @param options.faceColors
     * @param options.flat
     * @param options.updatable
     * @param options.sideOrientation
     * @param options.frontUVs
     * @param options.backUVs
     * @param scene defines the hosting scene
     * @returns the polyhedron mesh
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/polyhedra
     */
    export function CreatePolyhedron(name: string, options?: {
        type?: number;
        size?: number;
        sizeX?: number;
        sizeY?: number;
        sizeZ?: number;
        custom?: any;
        faceUV?: Vector4[];
        faceColors?: Color4[];
        flat?: boolean;
        updatable?: boolean;
        sideOrientation?: number;
        frontUVs?: Vector4;
        backUVs?: Vector4;
    }, scene?: Nullable<Scene>): Mesh;
    /**
     * Class containing static functions to help procedurally build meshes
     * @deprecated use the function directly from the module
     */
    export var PolyhedronBuilder: {
        CreatePolyhedron: typeof CreatePolyhedron;
    };


    /**
     * Creates the VertexData for a Ribbon
     * @param options an object used to set the following optional parameters for the ribbon, required but can be empty
     * * pathArray array of paths, each of which an array of successive Vector3
     * * closeArray creates a seam between the first and the last paths of the pathArray, optional, default false
     * * closePath creates a seam between the first and the last points of each path of the path array, optional, default false
     * * offset a positive integer, only used when pathArray contains a single path (offset = 10 means the point 1 is joined to the point 11), default rounded half size of the pathArray length
     * * sideOrientation optional and takes the values : Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
     * * frontUvs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the front side, optional, default vector4 (0, 0, 1, 1)
     * * backUVs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the back side, optional, default vector4 (0, 0, 1, 1)
     * * invertUV swaps in the U and V coordinates when applying a texture, optional, default false
     * * uvs a linear array, of length 2 * number of vertices, of custom UV values, optional
     * * colors a linear array, of length 4 * number of vertices, of custom color values, optional
     * @param options.pathArray
     * @param options.closeArray
     * @param options.closePath
     * @param options.offset
     * @param options.sideOrientation
     * @param options.frontUVs
     * @param options.backUVs
     * @param options.invertUV
     * @param options.uvs
     * @param options.colors
     * @returns the VertexData of the ribbon
     */
    export function CreateRibbonVertexData(options: {
        pathArray: Vector3[][];
        closeArray?: boolean;
        closePath?: boolean;
        offset?: number;
        sideOrientation?: number;
        frontUVs?: Vector4;
        backUVs?: Vector4;
        invertUV?: boolean;
        uvs?: Vector2[];
        colors?: Color4[];
    }): VertexData;
    /**
     * Creates a ribbon mesh. The ribbon is a parametric shape.  It has no predefined shape. Its final shape will depend on the input parameters
     * * The parameter `pathArray` is a required array of paths, what are each an array of successive Vector3. The pathArray parameter depicts the ribbon geometry
     * * The parameter `closeArray` (boolean, default false) creates a seam between the first and the last paths of the path array
     * * The parameter `closePath` (boolean, default false) creates a seam between the first and the last points of each path of the path array
     * * The parameter `offset` (positive integer, default : rounded half size of the pathArray length), is taken in account only if the `pathArray` is containing a single path
     * * It's the offset to join the points from the same path. Ex : offset = 10 means the point 1 is joined to the point 11
     * * The optional parameter `instance` is an instance of an existing Ribbon object to be updated with the passed `pathArray` parameter : https://doc.babylonjs.com/features/featuresDeepDive/mesh/dynamicMeshMorph#ribbon
     * * You can also set the mesh side orientation with the values : BABYLON.Mesh.FRONTSIDE (default), BABYLON.Mesh.BACKSIDE or BABYLON.Mesh.DOUBLESIDE
     * * If you create a double-sided mesh, you can choose what parts of the texture image to crop and stick respectively on the front and the back sides with the parameters `frontUVs` and `backUVs` (Vector4). Detail here : https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#side-orientation
     * * The optional parameter `invertUV` (boolean, default false) swaps in the geometry the U and V coordinates to apply a texture
     * * The parameter `uvs` is an optional flat array of `Vector2` to update/set each ribbon vertex with its own custom UV values instead of the computed ones
     * * The parameters `colors` is an optional flat array of `Color4` to set/update each ribbon vertex with its own custom color values
     * * Note that if you use the parameters `uvs` or `colors`, the passed arrays must be populated with the right number of elements, it is to say the number of ribbon vertices. Remember that if you set `closePath` to `true`, there's one extra vertex per path in the geometry
     * * Moreover, you can use the parameter `color` with `instance` (to update the ribbon), only if you previously used it at creation time
     * * The mesh can be set to updatable with the boolean parameter `updatable` (default false) if its internal geometry is supposed to change once created
     * @param name defines the name of the mesh
     * @param options defines the options used to create the mesh
     * @param options.pathArray
     * @param options.closeArray
     * @param options.closePath
     * @param options.offset
     * @param options.updatable
     * @param options.sideOrientation
     * @param options.frontUVs
     * @param options.backUVs
     * @param options.instance
     * @param options.invertUV
     * @param options.uvs
     * @param options.colors
     * @param scene defines the hosting scene
     * @returns the ribbon mesh
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/param/ribbon_extra
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/param
     */
    export function CreateRibbon(name: string, options: {
        pathArray: Vector3[][];
        closeArray?: boolean;
        closePath?: boolean;
        offset?: number;
        updatable?: boolean;
        sideOrientation?: number;
        frontUVs?: Vector4;
        backUVs?: Vector4;
        instance?: Mesh;
        invertUV?: boolean;
        uvs?: Vector2[];
        colors?: Color4[];
    }, scene?: Nullable<Scene>): Mesh;
    /**
     * Class containing static functions to help procedurally build meshes
     * @deprecated use CreateRibbon directly
     */
    export var RibbonBuilder: {
        CreateRibbon: typeof CreateRibbon;
    };


    /**
     * Creates an extruded shape mesh. The extrusion is a parametric shape. It has no predefined shape. Its final shape will depend on the input parameters.
     * * The parameter `shape` is a required array of successive Vector3. This array depicts the shape to be extruded in its local space : the shape must be designed in the xOy plane and will be extruded along the Z axis.
     * * The parameter `path` is a required array of successive Vector3. This is the axis curve the shape is extruded along.
     * * The parameter `rotation` (float, default 0 radians) is the angle value to rotate the shape each step (each path point), from the former step (so rotation added each step) along the curve.
     * * The parameter `scale` (float, default 1) is the value to scale the shape.
     * * The parameter `closeShape` (boolean, default false) closes the shape when true, since v5.0.0.
     * * The parameter `closePath` (boolean, default false) closes the path when true and no caps, since v5.0.0.
     * * The parameter `cap` sets the way the extruded shape is capped. Possible values : BABYLON.Mesh.NO_CAP (default), BABYLON.Mesh.CAP_START, BABYLON.Mesh.CAP_END, BABYLON.Mesh.CAP_ALL
     * * The optional parameter `instance` is an instance of an existing ExtrudedShape object to be updated with the passed `shape`, `path`, `scale` or `rotation` parameters : https://doc.babylonjs.com/features/featuresDeepDive/mesh/dynamicMeshMorph#extruded-shape
     * * Remember you can only change the shape or path point positions, not their number when updating an extruded shape.
     * * You can also set the mesh side orientation with the values : BABYLON.Mesh.FRONTSIDE (default), BABYLON.Mesh.BACKSIDE or BABYLON.Mesh.DOUBLESIDE
     * * If you create a double-sided mesh, you can choose what parts of the texture image to crop and stick respectively on the front and the back sides with the parameters `frontUVs` and `backUVs` (Vector4). Detail here : https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#side-orientation
     * * The optional parameter `invertUV` (boolean, default false) swaps in the geometry the U and V coordinates to apply a texture.
     * * The mesh can be set to updatable with the boolean parameter `updatable` (default false) if its internal geometry is supposed to change once created.
     * * The optional parameter `firstNormal` (Vector3) defines the direction of the first normal of the supplied path. Consider using this for any path that is straight, and particular for paths in the xy plane.
     * * The optional `adjustFrame` (boolean, default false) will cause the internally generated Path3D tangents, normals, and binormals to be adjusted so that a) they are always well-defined, and b) they do not reverse from one path point to the next. This prevents the extruded shape from being flipped and/or rotated with resulting mesh self-intersections. This is primarily useful for straight paths that can reverse direction.
     * @param name defines the name of the mesh
     * @param options defines the options used to create the mesh
     * @param options.shape
     * @param options.path
     * @param options.scale
     * @param options.rotation
     * @param options.closeShape
     * @param options.closePath
     * @param options.cap
     * @param options.updatable
     * @param options.sideOrientation
     * @param options.frontUVs
     * @param options.backUVs
     * @param options.instance
     * @param options.invertUV
     * @param options.firstNormal
     * @param options.adjustFrame
     * @param scene defines the hosting scene
     * @returns the extruded shape mesh
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/param
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/param#extruded-shapes
     */
    export function ExtrudeShape(name: string, options: {
        shape: Vector3[];
        path: Vector3[];
        scale?: number;
        rotation?: number;
        closeShape?: boolean;
        closePath?: boolean;
        cap?: number;
        updatable?: boolean;
        sideOrientation?: number;
        frontUVs?: Vector4;
        backUVs?: Vector4;
        instance?: Mesh;
        invertUV?: boolean;
        firstNormal?: Vector3;
        adjustFrame?: boolean;
    }, scene?: Nullable<Scene>): Mesh;
    /**
     * Creates an custom extruded shape mesh.
     * The custom extrusion is a parametric shape. It has no predefined shape. Its final shape will depend on the input parameters.
     * * The parameter `shape` is a required array of successive Vector3. This array depicts the shape to be extruded in its local space : the shape must be designed in the xOy plane and will be extruded along the Z axis.
     * * The parameter `path` is a required array of successive Vector3. This is the axis curve the shape is extruded along.
     * * The parameter `rotationFunction` (JS function) is a custom Javascript function called on each path point. This function is passed the position i of the point in the path and the distance of this point from the beginning of the path
     * * It must returns a float value that will be the rotation in radians applied to the shape on each path point.
     * * The parameter `scaleFunction` (JS function) is a custom Javascript function called on each path point. This function is passed the position i of the point in the path and the distance of this point from the beginning of the path
     * * It must returns a float value that will be the scale value applied to the shape on each path point
     * * The parameter `closeShape` (boolean, default false) closes the shape when true, since v5.0.0.
     * * The parameter `closePath` (boolean, default false) closes the path when true and no caps, since v5.0.0.
     * * The parameter `ribbonClosePath` (boolean, default false) forces the extrusion underlying ribbon to close all the paths in its `pathArray` - depreciated in favor of closeShape
     * * The parameter `ribbonCloseArray` (boolean, default false) forces the extrusion underlying ribbon to close its `pathArray` - depreciated in favor of closePath
     * * The parameter `cap` sets the way the extruded shape is capped. Possible values : BABYLON.Mesh.NO_CAP (default), BABYLON.Mesh.CAP_START, BABYLON.Mesh.CAP_END, BABYLON.Mesh.CAP_ALL
     * * The optional parameter `instance` is an instance of an existing ExtrudedShape object to be updated with the passed `shape`, `path`, `scale` or `rotation` parameters : https://doc.babylonjs.com/features/featuresDeepDive/mesh/dynamicMeshMorph#extruded-shape
     * * Remember you can only change the shape or path point positions, not their number when updating an extruded shape
     * * You can also set the mesh side orientation with the values : BABYLON.Mesh.FRONTSIDE (default), BABYLON.Mesh.BACKSIDE or BABYLON.Mesh.DOUBLESIDE
     * * If you create a double-sided mesh, you can choose what parts of the texture image to crop and stick respectively on the front and the back sides with the parameters `frontUVs` and `backUVs` (Vector4). Detail here : https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#side-orientation
     * * The optional parameter `invertUV` (boolean, default false) swaps in the geometry the U and V coordinates to apply a texture
     * * The mesh can be set to updatable with the boolean parameter `updatable` (default false) if its internal geometry is supposed to change once created
     * * The optional parameter `firstNormal` (Vector3) defines the direction of the first normal of the supplied path. It should be supplied when the path is in the xy plane, and particularly if these sections are straight, because the underlying Path3D object will pick a normal in the xy plane that causes the extrusion to be collapsed into the plane. This should be used for any path that is straight.
     * * The optional `adjustFrame` (boolean, default false) will cause the internally generated Path3D tangents, normals, and binormals to be adjusted so that a) they are always well-defined, and b) they do not reverse from one path point to the next. This prevents the extruded shape from being flipped and/or rotated with resulting mesh self-intersections. This is primarily useful for straight paths that can reverse direction.
     * @param name defines the name of the mesh
     * @param options defines the options used to create the mesh
     * @param options.shape
     * @param options.path
     * @param options.scaleFunction
     * @param options.rotationFunction
     * @param options.ribbonCloseArray
     * @param options.ribbonClosePath
     * @param options.closeShape
     * @param options.closePath
     * @param options.cap
     * @param options.updatable
     * @param options.sideOrientation
     * @param options.frontUVs
     * @param options.backUVs
     * @param options.instance
     * @param options.invertUV
     * @param options.firstNormal
     * @param options.adjustFrame
     * @param scene defines the hosting scene
     * @returns the custom extruded shape mesh
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/param#custom-extruded-shapes
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/param
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/param#extruded-shapes
     */
    export function ExtrudeShapeCustom(name: string, options: {
        shape: Vector3[];
        path: Vector3[];
        scaleFunction?: Nullable<{
            (i: number, distance: number): number;
        }>;
        rotationFunction?: Nullable<{
            (i: number, distance: number): number;
        }>;
        ribbonCloseArray?: boolean;
        ribbonClosePath?: boolean;
        closeShape?: boolean;
        closePath?: boolean;
        cap?: number;
        updatable?: boolean;
        sideOrientation?: number;
        frontUVs?: Vector4;
        backUVs?: Vector4;
        instance?: Mesh;
        invertUV?: boolean;
        firstNormal?: Vector3;
        adjustFrame?: boolean;
    }, scene?: Nullable<Scene>): Mesh;
    /**
     * Class containing static functions to help procedurally build meshes
     * @deprecated please use the functions directly from the module
     */
    export var ShapeBuilder: {
        ExtrudeShape: typeof ExtrudeShape;
        ExtrudeShapeCustom: typeof ExtrudeShapeCustom;
    };


    /**
     * Creates the VertexData for an ellipsoid, defaults to a sphere
     * @param options an object used to set the following optional parameters for the box, required but can be empty
     * * segments sets the number of horizontal strips optional, default 32
     * * diameter sets the axes dimensions, diameterX, diameterY and diameterZ to the value of diameter, optional default 1
     * * diameterX sets the diameterX (x direction) of the ellipsoid, overwrites the diameterX set by diameter, optional, default diameter
     * * diameterY sets the diameterY (y direction) of the ellipsoid, overwrites the diameterY set by diameter, optional, default diameter
     * * diameterZ sets the diameterZ (z direction) of the ellipsoid, overwrites the diameterZ set by diameter, optional, default diameter
     * * arc a number from 0 to 1, to create an unclosed ellipsoid based on the fraction of the circumference (latitude) given by the arc value, optional, default 1
     * * slice a number from 0 to 1, to create an unclosed ellipsoid based on the fraction of the height (latitude) given by the arc value, optional, default 1
     * * sideOrientation optional and takes the values : Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
     * * frontUvs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the front side, optional, default vector4 (0, 0, 1, 1)
     * * backUVs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the back side, optional, default vector4 (0, 0, 1, 1)
     * @param options.segments
     * @param options.diameter
     * @param options.diameterX
     * @param options.diameterY
     * @param options.diameterZ
     * @param options.arc
     * @param options.slice
     * @param options.sideOrientation
     * @param options.frontUVs
     * @param options.backUVs
     * @param options.dedupTopBottomIndices
     * @returns the VertexData of the ellipsoid
     */
    export function CreateSphereVertexData(options: {
        segments?: number;
        diameter?: number;
        diameterX?: number;
        diameterY?: number;
        diameterZ?: number;
        arc?: number;
        slice?: number;
        sideOrientation?: number;
        frontUVs?: Vector4;
        backUVs?: Vector4;
        dedupTopBottomIndices?: boolean;
    }): VertexData;
    /**
     * Creates a sphere mesh
     * * The parameter `diameter` sets the diameter size (float) of the sphere (default 1)
     * * You can set some different sphere dimensions, for instance to build an ellipsoid, by using the parameters `diameterX`, `diameterY` and `diameterZ` (all by default have the same value of `diameter`)
     * * The parameter `segments` sets the sphere number of horizontal stripes (positive integer, default 32)
     * * You can create an unclosed sphere with the parameter `arc` (positive float, default 1), valued between 0 and 1, what is the ratio of the circumference (latitude) : 2 x PI x ratio
     * * You can create an unclosed sphere on its height with the parameter `slice` (positive float, default1), valued between 0 and 1, what is the height ratio (longitude)
     * * You can also set the mesh side orientation with the values : BABYLON.Mesh.FRONTSIDE (default), BABYLON.Mesh.BACKSIDE or BABYLON.Mesh.DOUBLESIDE
     * * If you create a double-sided mesh, you can choose what parts of the texture image to crop and stick respectively on the front and the back sides with the parameters `frontUVs` and `backUVs` (Vector4). Detail here : https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#side-orientation
     * * The mesh can be set to updatable with the boolean parameter `updatable` (default false) if its internal geometry is supposed to change once created
     * @param name defines the name of the mesh
     * @param options defines the options used to create the mesh
     * @param options.segments
     * @param options.diameter
     * @param options.diameterX
     * @param options.diameterY
     * @param options.diameterZ
     * @param options.arc
     * @param options.slice
     * @param options.sideOrientation
     * @param options.frontUVs
     * @param options.backUVs
     * @param options.updatable
     * @param scene defines the hosting scene
     * @returns the sphere mesh
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#sphere
     */
    export function CreateSphere(name: string, options?: {
        segments?: number;
        diameter?: number;
        diameterX?: number;
        diameterY?: number;
        diameterZ?: number;
        arc?: number;
        slice?: number;
        sideOrientation?: number;
        frontUVs?: Vector4;
        backUVs?: Vector4;
        updatable?: boolean;
    }, scene?: Nullable<Scene>): Mesh;
    /**
     * Class containing static functions to help procedurally build meshes
     * @deprecated use CreateSphere directly
     */
    export var SphereBuilder: {
        CreateSphere: typeof CreateSphere;
    };


    /**
     * Creates the VertexData for a tiled box
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set/tiled_box
     * @param options an object used to set the following optional parameters for the tiled box, required but can be empty
     * * pattern sets the rotation or reflection pattern for the tiles,
     * * size of the box
     * * width of the box, overwrites size
     * * height of the box, overwrites size
     * * depth of the box, overwrites size
     * * tileSize sets the size of a tile
     * * tileWidth sets the tile width and overwrites tileSize
     * * tileHeight sets the tile width and overwrites tileSize
     * * faceUV an array of 6 Vector4 elements used to set different images to each box side
     * * faceColors an array of 6 Color3 elements used to set different colors to each box side
     * * alignHorizontal places whole tiles aligned to the center, left or right of a row
     * * alignVertical places whole tiles aligned to the center, left or right of a column
     * @param options.pattern
     * @param options.size
     * @param options.width
     * @param options.height
     * @param options.depth
     * @param options.tileSize
     * @param options.tileWidth
     * @param options.tileHeight
     * @param options.faceUV
     * @param options.faceColors
     * @param options.alignHorizontal
     * @param options.alignVertical
     * @param options.sideOrientation
     * * sideOrientation optional and takes the values : Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
     * @returns the VertexData of the TiledBox
     */
    export function CreateTiledBoxVertexData(options: {
        pattern?: number;
        size?: number;
        width?: number;
        height?: number;
        depth?: number;
        tileSize?: number;
        tileWidth?: number;
        tileHeight?: number;
        faceUV?: Vector4[];
        faceColors?: Color4[];
        alignHorizontal?: number;
        alignVertical?: number;
        sideOrientation?: number;
    }): VertexData;
    /**
     * Creates a tiled box mesh
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set/tiled_box
     * @param name defines the name of the mesh
     * @param options an object used to set the following optional parameters for the tiled box, required but can be empty
     * * pattern sets the rotation or reflection pattern for the tiles,
     * * size of the box
     * * width of the box, overwrites size
     * * height of the box, overwrites size
     * * depth of the box, overwrites size
     * * tileSize sets the size of a tile
     * * tileWidth sets the tile width and overwrites tileSize
     * * tileHeight sets the tile width and overwrites tileSize
     * * faceUV an array of 6 Vector4 elements used to set different images to each box side
     * * faceColors an array of 6 Color3 elements used to set different colors to each box side
     * * alignHorizontal places whole tiles aligned to the center, left or right of a row
     * * alignVertical places whole tiles aligned to the center, left or right of a column
     * * sideOrientation optional and takes the values : Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
     * @param options.pattern
     * @param options.width
     * @param options.height
     * @param options.depth
     * @param options.tileSize
     * @param options.tileWidth
     * @param options.tileHeight
     * @param options.alignHorizontal
     * @param options.alignVertical
     * @param options.faceUV
     * @param options.faceColors
     * @param options.sideOrientation
     * @param options.updatable
     * @param scene defines the hosting scene
     * @returns the box mesh
     */
    export function CreateTiledBox(name: string, options: {
        pattern?: number;
        width?: number;
        height?: number;
        depth?: number;
        tileSize?: number;
        tileWidth?: number;
        tileHeight?: number;
        alignHorizontal?: number;
        alignVertical?: number;
        faceUV?: Vector4[];
        faceColors?: Color4[];
        sideOrientation?: number;
        updatable?: boolean;
    }, scene?: Nullable<Scene>): Mesh;
    /**
     * Class containing static functions to help procedurally build meshes
     * @deprecated use CreateTiledBox instead
     */
    export var TiledBoxBuilder: {
        CreateTiledBox: typeof CreateTiledBox;
    };


    /**
     * Creates the VertexData for a tiled plane
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set/tiled_plane
     * @param options an object used to set the following optional parameters for the tiled plane, required but can be empty
     * * pattern a limited pattern arrangement depending on the number
     * * size of the box
     * * width of the box, overwrites size
     * * height of the box, overwrites size
     * * tileSize sets the width, height and depth of the tile to the value of size, optional default 1
     * * tileWidth sets the width (x direction) of the tile, overwrites the width set by size, optional, default size
     * * tileHeight sets the height (y direction) of the tile, overwrites the height set by size, optional, default size
     * * sideOrientation optional and takes the values : Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
     * * alignHorizontal places whole tiles aligned to the center, left or right of a row
     * * alignVertical places whole tiles aligned to the center, left or right of a column
     * * frontUvs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the front side, optional, default vector4 (0, 0, 1, 1)
     * @param options.pattern
     * @param options.tileSize
     * @param options.tileWidth
     * @param options.tileHeight
     * @param options.size
     * @param options.width
     * @param options.height
     * @param options.alignHorizontal
     * @param options.alignVertical
     * @param options.sideOrientation
     * @param options.frontUVs
     * @param options.backUVs
     * * backUVs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the back side, optional, default vector4 (0, 0, 1, 1)
     * @returns the VertexData of the tiled plane
     */
    export function CreateTiledPlaneVertexData(options: {
        pattern?: number;
        tileSize?: number;
        tileWidth?: number;
        tileHeight?: number;
        size?: number;
        width?: number;
        height?: number;
        alignHorizontal?: number;
        alignVertical?: number;
        sideOrientation?: number;
        frontUVs?: Vector4;
        backUVs?: Vector4;
    }): VertexData;
    /**
     * Creates a tiled plane mesh
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set/tiled_plane
     * @param name defines the name of the mesh
     * @param options an object used to set the following optional parameters for the tiled plane, required but can be empty
     * * pattern a limited pattern arrangement depending on the number
     * * size of the box
     * * width of the box, overwrites size
     * * height of the box, overwrites size
     * * tileSize sets the width, height and depth of the tile to the value of size, optional default 1
     * * tileWidth sets the width (x direction) of the tile, overwrites the width set by size, optional, default size
     * * tileHeight sets the height (y direction) of the tile, overwrites the height set by size, optional, default size
     * * sideOrientation optional and takes the values : Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
     * * alignHorizontal places whole tiles aligned to the center, left or right of a row
     * * alignVertical places whole tiles aligned to the center, left or right of a column
     * * frontUvs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the front side, optional, default vector4 (0, 0, 1, 1)
     * * backUVs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the back side, optional, default vector4 (0, 0, 1, 1)
     * @param options.pattern
     * @param options.tileSize
     * @param options.tileWidth
     * @param options.tileHeight
     * @param options.size
     * @param options.width
     * @param options.height
     * @param options.alignHorizontal
     * @param options.alignVertical
     * @param options.sideOrientation
     * @param options.frontUVs
     * @param options.backUVs
     * @param options.updatable
     * @param scene defines the hosting scene
     * @returns the box mesh
     */
    export function CreateTiledPlane(name: string, options: {
        pattern?: number;
        tileSize?: number;
        tileWidth?: number;
        tileHeight?: number;
        size?: number;
        width?: number;
        height?: number;
        alignHorizontal?: number;
        alignVertical?: number;
        sideOrientation?: number;
        frontUVs?: Vector4;
        backUVs?: Vector4;
        updatable?: boolean;
    }, scene?: Nullable<Scene>): Mesh;
    /**
     * Class containing static functions to help procedurally build meshes
     * @deprecated use CreateTiledPlane instead
     */
    export var TiledPlaneBuilder: {
        CreateTiledPlane: typeof CreateTiledPlane;
    };


    /**
     * Creates the VertexData for a torus
     * @param options an object used to set the following optional parameters for the box, required but can be empty
     * * diameter the diameter of the torus, optional default 1
     * * thickness the diameter of the tube forming the torus, optional default 0.5
     * * tessellation the number of prism sides, 3 for a triangular prism, optional, default 24
     * * sideOrientation optional and takes the values : Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
     * * frontUvs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the front side, optional, default vector4 (0, 0, 1, 1)
     * * backUVs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the back side, optional, default vector4 (0, 0, 1, 1)
     * @param options.diameter
     * @param options.thickness
     * @param options.tessellation
     * @param options.sideOrientation
     * @param options.frontUVs
     * @param options.backUVs
     * @returns the VertexData of the torus
     */
    export function CreateTorusVertexData(options: {
        diameter?: number;
        thickness?: number;
        tessellation?: number;
        sideOrientation?: number;
        frontUVs?: Vector4;
        backUVs?: Vector4;
    }): VertexData;
    /**
     * Creates a torus mesh
     * * The parameter `diameter` sets the diameter size (float) of the torus (default 1)
     * * The parameter `thickness` sets the diameter size of the tube of the torus (float, default 0.5)
     * * The parameter `tessellation` sets the number of torus sides (positive integer, default 16)
     * * You can also set the mesh side orientation with the values : BABYLON.Mesh.FRONTSIDE (default), BABYLON.Mesh.BACKSIDE or BABYLON.Mesh.DOUBLESIDE
     * * If you create a double-sided mesh, you can choose what parts of the texture image to crop and stick respectively on the front and the back sides with the parameters `frontUVs` and `backUVs` (Vector4). Detail here : https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#side-orientation
     * * The mesh can be set to updatable with the boolean parameter `updatable` (default false) if its internal geometry is supposed to change once created.
     * @param name defines the name of the mesh
     * @param options defines the options used to create the mesh
     * @param options.diameter
     * @param options.thickness
     * @param options.tessellation
     * @param options.updatable
     * @param options.sideOrientation
     * @param options.frontUVs
     * @param options.backUVs
     * @param scene defines the hosting scene
     * @returns the torus mesh
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#torus
     */
    export function CreateTorus(name: string, options?: {
        diameter?: number;
        thickness?: number;
        tessellation?: number;
        updatable?: boolean;
        sideOrientation?: number;
        frontUVs?: Vector4;
        backUVs?: Vector4;
    }, scene?: Scene): Mesh;
    /**
     * Class containing static functions to help procedurally build meshes
     * @deprecated use CreateTorus instead
     */
    export var TorusBuilder: {
        CreateTorus: typeof CreateTorus;
    };


    /**
     * Creates the VertexData for a TorusKnot
     * @param options an object used to set the following optional parameters for the TorusKnot, required but can be empty
     * * radius the radius of the torus knot, optional, default 2
     * * tube the thickness of the tube, optional, default 0.5
     * * radialSegments the number of sides on each tube segments, optional, default 32
     * * tubularSegments the number of tubes to decompose the knot into, optional, default 32
     * * p the number of windings around the z axis, optional,  default 2
     * * q the number of windings around the x axis, optional,  default 3
     * * sideOrientation optional and takes the values : Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
     * * frontUvs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the front side, optional, default vector4 (0, 0, 1, 1)
     * * backUVs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the back side, optional, default vector4 (0, 0, 1, 1)
     * @param options.radius
     * @param options.tube
     * @param options.radialSegments
     * @param options.tubularSegments
     * @param options.p
     * @param options.q
     * @param options.sideOrientation
     * @param options.frontUVs
     * @param options.backUVs
     * @returns the VertexData of the Torus Knot
     */
    export function CreateTorusKnotVertexData(options: {
        radius?: number;
        tube?: number;
        radialSegments?: number;
        tubularSegments?: number;
        p?: number;
        q?: number;
        sideOrientation?: number;
        frontUVs?: Vector4;
        backUVs?: Vector4;
    }): VertexData;
    /**
     * Creates a torus knot mesh
     * * The parameter `radius` sets the global radius size (float) of the torus knot (default 2)
     * * The parameter `radialSegments` sets the number of sides on each tube segments (positive integer, default 32)
     * * The parameter `tubularSegments` sets the number of tubes to decompose the knot into (positive integer, default 32)
     * * The parameters `p` and `q` are the number of windings on each axis (positive integers, default 2 and 3)
     * * You can also set the mesh side orientation with the values : BABYLON.Mesh.FRONTSIDE (default), BABYLON.Mesh.BACKSIDE or BABYLON.Mesh.DOUBLESIDE
     * * If you create a double-sided mesh, you can choose what parts of the texture image to crop and stick respectively on the front and the back sides with the parameters `frontUVs` and `backUVs` (Vector4). Detail here : https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#side-orientation
     * * The mesh can be set to updatable with the boolean parameter `updatable` (default false) if its internal geometry is supposed to change once created.
     * @param name defines the name of the mesh
     * @param options defines the options used to create the mesh
     * @param options.radius
     * @param options.tube
     * @param options.radialSegments
     * @param options.tubularSegments
     * @param options.p
     * @param options.q
     * @param options.updatable
     * @param options.sideOrientation
     * @param options.frontUVs
     * @param options.backUVs
     * @param scene defines the hosting scene
     * @returns the torus knot mesh
     * @see  https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#torus-knot
     */
    export function CreateTorusKnot(name: string, options?: {
        radius?: number;
        tube?: number;
        radialSegments?: number;
        tubularSegments?: number;
        p?: number;
        q?: number;
        updatable?: boolean;
        sideOrientation?: number;
        frontUVs?: Vector4;
        backUVs?: Vector4;
    }, scene?: Scene): Mesh;
    /**
     * Class containing static functions to help procedurally build meshes
     * @deprecated use CreateTorusKnot instead
     */
    export var TorusKnotBuilder: {
        CreateTorusKnot: typeof CreateTorusKnot;
    };


    /**
     * Creates a tube mesh.
     * The tube is a parametric shape. It has no predefined shape. Its final shape will depend on the input parameters
     * * The parameter `path` is a required array of successive Vector3. It is the curve used as the axis of the tube
     * * The parameter `radius` (positive float, default 1) sets the tube radius size
     * * The parameter `tessellation` (positive float, default 64) is the number of sides on the tubular surface
     * * The parameter `radiusFunction` (javascript function, default null) is a vanilla javascript function. If it is not null, it overrides the parameter `radius`
     * * This function is called on each point of the tube path and is passed the index `i` of the i-th point and the distance of this point from the first point of the path. It must return a radius value (positive float)
     * * The parameter `arc` (positive float, maximum 1, default 1) is the ratio to apply to the tube circumference : 2 x PI x arc
     * * The parameter `cap` sets the way the extruded shape is capped. Possible values : BABYLON.Mesh.NO_CAP (default), BABYLON.Mesh.CAP_START, BABYLON.Mesh.CAP_END, BABYLON.Mesh.CAP_ALL
     * * The optional parameter `instance` is an instance of an existing Tube object to be updated with the passed `pathArray` parameter. The `path`Array HAS to have the SAME number of points as the previous one: https://doc.babylonjs.com/features/featuresDeepDive/mesh/dynamicMeshMorph#tube
     * * You can also set the mesh side orientation with the values : BABYLON.Mesh.FRONTSIDE (default), BABYLON.Mesh.BACKSIDE or BABYLON.Mesh.DOUBLESIDE
     * * If you create a double-sided mesh, you can choose what parts of the texture image to crop and stick respectively on the front and the back sides with the parameters `frontUVs` and `backUVs` (Vector4). Detail here : https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#side-orientation
     * * The optional parameter `invertUV` (boolean, default false) swaps in the geometry the U and V coordinates to apply a texture
     * * The mesh can be set to updatable with the boolean parameter `updatable` (default false) if its internal geometry is supposed to change once created. The NUMBER of points CAN'T CHANGE, only their positions.
     * @param name defines the name of the mesh
     * @param options defines the options used to create the mesh
     * @param options.path
     * @param options.radius
     * @param options.tessellation
     * @param options.radiusFunction
     * @param options.cap
     * @param options.arc
     * @param options.updatable
     * @param options.sideOrientation
     * @param options.frontUVs
     * @param options.backUVs
     * @param options.instance
     * @param options.invertUV
     * @param scene defines the hosting scene
     * @returns the tube mesh
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/param
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#tube
     */
    export function CreateTube(name: string, options: {
        path: Vector3[];
        radius?: number;
        tessellation?: number;
        radiusFunction?: {
            (i: number, distance: number): number;
        };
        cap?: number;
        arc?: number;
        updatable?: boolean;
        sideOrientation?: number;
        frontUVs?: Vector4;
        backUVs?: Vector4;
        instance?: Mesh;
        invertUV?: boolean;
    }, scene?: Nullable<Scene>): Mesh;
    /**
     * Class containing static functions to help procedurally build meshes
     * @deprecated use CreateTube directly
     */
    export var TubeBuilder: {
        CreateTube: typeof CreateTube;
    };


    /**
     * Configuration for Draco compression
     */
    export interface IDracoCompressionConfiguration {
        /**
         * Configuration for the decoder.
         */
        decoder: {
            /**
             * The url to the WebAssembly module.
             */
            wasmUrl?: string;
            /**
             * The url to the WebAssembly binary.
             */
            wasmBinaryUrl?: string;
            /**
             * The url to the fallback JavaScript module.
             */
            fallbackUrl?: string;
        };
    }
    /**
     * Draco compression (https://google.github.io/draco/)
     *
     * This class wraps the Draco module.
     *
     * **Encoder**
     *
     * The encoder is not currently implemented.
     *
     * **Decoder**
     *
     * By default, the configuration points to a copy of the Draco decoder files for glTF from the babylon.js preview cdn https://preview.babylonjs.com/draco_wasm_wrapper_gltf.js.
     *
     * To update the configuration, use the following code:
     * ```javascript
     *     DracoCompression.Configuration = {
     *         decoder: {
     *             wasmUrl: "<url to the WebAssembly library>",
     *             wasmBinaryUrl: "<url to the WebAssembly binary>",
     *             fallbackUrl: "<url to the fallback JavaScript library>",
     *         }
     *     };
     * ```
     *
     * Draco has two versions, one for WebAssembly and one for JavaScript. The decoder configuration can be set to only support WebAssembly or only support the JavaScript version.
     * Decoding will automatically fallback to the JavaScript version if WebAssembly version is not configured or if WebAssembly is not supported by the browser.
     * Use `DracoCompression.DecoderAvailable` to determine if the decoder configuration is available for the current context.
     *
     * To decode Draco compressed data, get the default DracoCompression object and call decodeMeshAsync:
     * ```javascript
     *     var vertexData = await DracoCompression.Default.decodeMeshAsync(data);
     * ```
     *
     * @see https://playground.babylonjs.com/#DMZIBD#0
     */
    export class DracoCompression implements IDisposable {
        private _workerPoolPromise?;
        private _decoderModulePromise?;
        /**
         * The configuration. Defaults to the following urls:
         * - wasmUrl: "https://preview.babylonjs.com/draco_wasm_wrapper_gltf.js"
         * - wasmBinaryUrl: "https://preview.babylonjs.com/draco_decoder_gltf.wasm"
         * - fallbackUrl: "https://preview.babylonjs.com/draco_decoder_gltf.js"
         */
        static Configuration: IDracoCompressionConfiguration;
        /**
         * Returns true if the decoder configuration is available.
         */
        static get DecoderAvailable(): boolean;
        /**
         * Default number of workers to create when creating the draco compression object.
         */
        static DefaultNumWorkers: number;
        private static GetDefaultNumWorkers;
        private static _Default;
        /**
         * Default instance for the draco compression object.
         */
        static get Default(): DracoCompression;
        /**
         * Constructor
         * @param numWorkers The number of workers for async operations. Specify `0` to disable web workers and run synchronously in the current context.
         */
        constructor(numWorkers?: number);
        /**
         * Stop all async operations and release resources.
         */
        dispose(): void;
        /**
         * Returns a promise that resolves when ready. Call this manually to ensure draco compression is ready before use.
         * @returns a promise that resolves when ready
         */
        whenReadyAsync(): Promise<void>;
        /**
         * Decode Draco compressed mesh data to vertex data.
         * @param data The ArrayBuffer or ArrayBufferView for the Draco compression data
         * @param attributes A map of attributes from vertex buffer kinds to Draco unique ids
         * @param dividers a list of optional dividers for normalization
         * @returns A promise that resolves with the decoded vertex data
         */
        decodeMeshAsync(data: ArrayBuffer | ArrayBufferView, attributes?: {
            [kind: string]: number;
        }, dividers?: {
            [kind: string]: number;
        }): Promise<VertexData>;
    }




    /**
     * Configuration for meshoptimizer compression
     */
    export interface IMeshoptCompressionConfiguration {
        /**
         * Configuration for the decoder.
         */
        decoder: {
            /**
             * The url to the meshopt decoder library.
             */
            url: string;
        };
    }
    /**
     * Meshopt compression (https://github.com/zeux/meshoptimizer)
     *
     * This class wraps the meshopt library from https://github.com/zeux/meshoptimizer/tree/master/js.
     *
     * **Encoder**
     *
     * The encoder is not currently implemented.
     *
     * **Decoder**
     *
     * By default, the configuration points to a copy of the meshopt files on the Babylon.js preview CDN (e.g. https://preview.babylonjs.com/meshopt_decoder.js).
     *
     * To update the configuration, use the following code:
     * ```javascript
     *     MeshoptCompression.Configuration = {
     *         decoder: {
     *             url: "<url to the meshopt decoder library>"
     *         }
     *     };
     * ```
     */
    export class MeshoptCompression implements IDisposable {
        private _decoderModulePromise?;
        /**
         * The configuration. Defaults to the following:
         * ```javascript
         * decoder: {
         *   url: "https://preview.babylonjs.com/meshopt_decoder.js"
         * }
         * ```
         */
        static Configuration: IMeshoptCompressionConfiguration;
        private static _Default;
        /**
         * Default instance for the meshoptimizer object.
         */
        static get Default(): MeshoptCompression;
        /**
         * Constructor
         */
        constructor();
        /**
         * Stop all async operations and release resources.
         */
        dispose(): void;
        /**
         * Decode meshopt data.
         * @see https://github.com/zeux/meshoptimizer/tree/master/js#decoder
         * @param source The input data.
         * @param count The number of elements.
         * @param stride The stride in bytes.
         * @param mode The compression mode.
         * @param filter The compression filter.
         * @returns a Promise<Uint8Array> that resolves to the decoded data
         */
        decodeGltfBufferAsync(source: Uint8Array, count: number, stride: number, mode: "ATTRIBUTES" | "TRIANGLES" | "INDICES", filter?: string): Promise<Uint8Array>;
    }


    /**
     * Class for building Constructive Solid Geometry
     */
    export class CSG {
        private _polygons;
        /**
         * The world matrix
         */
        matrix: Matrix;
        /**
         * Stores the position
         */
        position: Vector3;
        /**
         * Stores the rotation
         */
        rotation: Vector3;
        /**
         * Stores the rotation quaternion
         */
        rotationQuaternion: Nullable<Quaternion>;
        /**
         * Stores the scaling vector
         */
        scaling: Vector3;
        /**
         * Convert the Mesh to CSG
         * @param mesh The Mesh to convert to CSG
         * @param absolute If true, the final (local) matrix transformation is set to the identity and not to that of `mesh`. It can help when dealing with right-handed meshes (default: false)
         * @returns A new CSG from the Mesh
         */
        static FromMesh(mesh: Mesh, absolute?: boolean): CSG;
        /**
         * Construct a CSG solid from a list of `CSG.Polygon` instances.
         * @param polygons Polygons used to construct a CSG solid
         */
        private static _FromPolygons;
        /**
         * Clones, or makes a deep copy, of the CSG
         * @returns A new CSG
         */
        clone(): CSG;
        /**
         * Unions this CSG with another CSG
         * @param csg The CSG to union against this CSG
         * @returns The unioned CSG
         */
        union(csg: CSG): CSG;
        /**
         * Unions this CSG with another CSG in place
         * @param csg The CSG to union against this CSG
         */
        unionInPlace(csg: CSG): void;
        /**
         * Subtracts this CSG with another CSG
         * @param csg The CSG to subtract against this CSG
         * @returns A new CSG
         */
        subtract(csg: CSG): CSG;
        /**
         * Subtracts this CSG with another CSG in place
         * @param csg The CSG to subtract against this CSG
         */
        subtractInPlace(csg: CSG): void;
        /**
         * Intersect this CSG with another CSG
         * @param csg The CSG to intersect against this CSG
         * @returns A new CSG
         */
        intersect(csg: CSG): CSG;
        /**
         * Intersects this CSG with another CSG in place
         * @param csg The CSG to intersect against this CSG
         */
        intersectInPlace(csg: CSG): void;
        /**
         * Return a new CSG solid with solid and empty space switched. This solid is
         * not modified.
         * @returns A new CSG solid with solid and empty space switched
         */
        inverse(): CSG;
        /**
         * Inverses the CSG in place
         */
        inverseInPlace(): void;
        /**
         * This is used to keep meshes transformations so they can be restored
         * when we build back a Babylon Mesh
         * NB : All CSG operations are performed in world coordinates
         * @param csg The CSG to copy the transform attributes from
         * @returns This CSG
         */
        copyTransformAttributes(csg: CSG): CSG;
        /**
         * Build Raw mesh from CSG
         * Coordinates here are in world space
         * @param name The name of the mesh geometry
         * @param scene The Scene
         * @param keepSubMeshes Specifies if the submeshes should be kept
         * @returns A new Mesh
         */
        buildMeshGeometry(name: string, scene?: Scene, keepSubMeshes?: boolean): Mesh;
        /**
         * Build Mesh from CSG taking material and transforms into account
         * @param name The name of the Mesh
         * @param material The material of the Mesh
         * @param scene The Scene
         * @param keepSubMeshes Specifies if submeshes should be kept
         * @returns The new Mesh
         */
        toMesh(name: string, material?: Nullable<Material>, scene?: Scene, keepSubMeshes?: boolean): Mesh;
    }


    /**
     * Class representing data for one face OAB of an equilateral icosahedron
     * When O is the isovector (0, 0), A is isovector (m, n)
     * @internal
     */
    export class _PrimaryIsoTriangle {
        m: number;
        n: number;
        cartesian: Vector3[];
        vertices: _IsoVector[];
        max: number[];
        min: number[];
        vecToidx: {
            [key: string]: number;
        };
        vertByDist: {
            [key: string]: number[];
        };
        closestTo: number[][];
        innerFacets: string[][];
        isoVecsABOB: _IsoVector[][];
        isoVecsOBOA: _IsoVector[][];
        isoVecsBAOA: _IsoVector[][];
        vertexTypes: number[][];
        coau: number;
        cobu: number;
        coav: number;
        cobv: number;
        IDATA: PolyhedronData;
        /**
         * Creates the PrimaryIsoTriangle Triangle OAB
         * @param m an integer
         * @param n an integer
         */
        setIndices(): void;
        calcCoeffs(): void;
        createInnerFacets(): void;
        edgeVecsABOB(): void;
        mapABOBtoOBOA(): void;
        mapABOBtoBAOA(): void;
        MapToFace(faceNb: number, geodesicData: PolyhedronData): void;
        /**Creates a primary triangle
         * @internal
         */
        build(m: number, n: number): this;
    }
    /** Builds Polyhedron Data
     * @internal
     */
    export class PolyhedronData {
        name: string;
        category: string;
        vertex: number[][];
        face: number[][];
        edgematch: (number | string)[][];
        constructor(name: string, category: string, vertex: number[][], face: number[][]);
    }
    /**
     * This class Extends the PolyhedronData Class to provide measures for a Geodesic Polyhedron
     */
    export class GeodesicData extends PolyhedronData {
        /**
         * @internal
         */
        edgematch: (number | string)[][];
        /**
         * @internal
         */
        adjacentFaces: number[][];
        /**
         * @internal
         */
        sharedNodes: number;
        /**
         * @internal
         */
        poleNodes: number;
        /**
         * @internal
         */
        innerToData(face: number, primTri: _PrimaryIsoTriangle): void;
        /**
         * @internal
         */
        mapABOBtoDATA(faceNb: number, primTri: _PrimaryIsoTriangle): void;
        /**
         * @internal
         */
        mapOBOAtoDATA(faceNb: number, primTri: _PrimaryIsoTriangle): void;
        /**
         * @internal
         */
        mapBAOAtoDATA(faceNb: number, primTri: _PrimaryIsoTriangle): void;
        /**
         * @internal
         */
        orderData(primTri: _PrimaryIsoTriangle): void;
        /**
         * @internal
         */
        setOrder(m: number, faces: number[]): number[];
        /**
         * @internal
         */
        toGoldbergPolyhedronData(): PolyhedronData;
        /**Builds the data for a Geodesic Polyhedron from a primary triangle
         * @param primTri the primary triangle
         * @internal
         */
        static BuildGeodesicData(primTri: _PrimaryIsoTriangle): GeodesicData;
    }


    /**
     * Class used to store geometry data (vertex buffers + index buffer)
     */
    export class Geometry implements IGetSetVerticesData {
        /**
         * Gets or sets the ID of the geometry
         */
        id: string;
        /**
         * Gets or sets the unique ID of the geometry
         */
        uniqueId: number;
        /**
         * Gets the delay loading state of the geometry (none by default which means not delayed)
         */
        delayLoadState: number;
        /**
         * Gets the file containing the data to load when running in delay load state
         */
        delayLoadingFile: Nullable<string>;
        /**
         * Callback called when the geometry is updated
         */
        onGeometryUpdated: (geometry: Geometry, kind?: string) => void;
        private _scene;
        private _engine;
        private _meshes;
        private _totalVertices;
        /** @internal */
        _loadedUniqueId: string;
        /** @internal */
        _indices: IndicesArray;
        /** @internal */
        _vertexBuffers: {
            [key: string]: VertexBuffer;
        };
        private _isDisposed;
        private _extend;
        private _boundingBias;
        /** @internal */
        _delayInfo: Array<string>;
        private _indexBuffer;
        private _indexBufferIsUpdatable;
        /** @internal */
        _boundingInfo: Nullable<BoundingInfo>;
        /** @internal */
        _delayLoadingFunction: Nullable<(any: any, geometry: Geometry) => void>;
        /** @internal */
        _softwareSkinningFrameId: number;
        private _vertexArrayObjects;
        private _updatable;
        /** @internal */
        _positions: Nullable<Vector3[]>;
        private _positionsCache;
        /** @internal */
        _parentContainer: Nullable<AbstractScene>;
        /**
         *  Gets or sets the Bias Vector to apply on the bounding elements (box/sphere), the max extend is computed as v += v * bias.x + bias.y, the min is computed as v -= v * bias.x + bias.y
         */
        get boundingBias(): Vector2;
        /**
         *  Gets or sets the Bias Vector to apply on the bounding elements (box/sphere), the max extend is computed as v += v * bias.x + bias.y, the min is computed as v -= v * bias.x + bias.y
         */
        set boundingBias(value: Vector2);
        /**
         * Static function used to attach a new empty geometry to a mesh
         * @param mesh defines the mesh to attach the geometry to
         * @returns the new Geometry
         */
        static CreateGeometryForMesh(mesh: Mesh): Geometry;
        /** Get the list of meshes using this geometry */
        get meshes(): Mesh[];
        /**
         * If set to true (false by default), the bounding info applied to the meshes sharing this geometry will be the bounding info defined at the class level
         * and won't be computed based on the vertex positions (which is what we get when useBoundingInfoFromGeometry = false)
         */
        useBoundingInfoFromGeometry: boolean;
        /**
         * Creates a new geometry
         * @param id defines the unique ID
         * @param scene defines the hosting scene
         * @param vertexData defines the VertexData used to get geometry data
         * @param updatable defines if geometry must be updatable (false by default)
         * @param mesh defines the mesh that will be associated with the geometry
         */
        constructor(id: string, scene?: Scene, vertexData?: VertexData, updatable?: boolean, mesh?: Nullable<Mesh>);
        /**
         * Gets the current extend of the geometry
         */
        get extend(): {
            minimum: Vector3;
            maximum: Vector3;
        };
        /**
         * Gets the hosting scene
         * @returns the hosting Scene
         */
        getScene(): Scene;
        /**
         * Gets the hosting engine
         * @returns the hosting Engine
         */
        getEngine(): Engine;
        /**
         * Defines if the geometry is ready to use
         * @returns true if the geometry is ready to be used
         */
        isReady(): boolean;
        /**
         * Gets a value indicating that the geometry should not be serialized
         */
        get doNotSerialize(): boolean;
        /** @internal */
        _rebuild(): void;
        /**
         * Affects all geometry data in one call
         * @param vertexData defines the geometry data
         * @param updatable defines if the geometry must be flagged as updatable (false as default)
         */
        setAllVerticesData(vertexData: VertexData, updatable?: boolean): void;
        /**
         * Set specific vertex data
         * @param kind defines the data kind (Position, normal, etc...)
         * @param data defines the vertex data to use
         * @param updatable defines if the vertex must be flagged as updatable (false as default)
         * @param stride defines the stride to use (0 by default). This value is deduced from the kind value if not specified
         */
        setVerticesData(kind: string, data: FloatArray, updatable?: boolean, stride?: number): void;
        /**
         * Removes a specific vertex data
         * @param kind defines the data kind (Position, normal, etc...)
         */
        removeVerticesData(kind: string): void;
        /**
         * Affect a vertex buffer to the geometry. the vertexBuffer.getKind() function is used to determine where to store the data
         * @param buffer defines the vertex buffer to use
         * @param totalVertices defines the total number of vertices for position kind (could be null)
         * @param disposeExistingBuffer disposes the existing buffer, if any (default: true)
         */
        setVerticesBuffer(buffer: VertexBuffer, totalVertices?: Nullable<number>, disposeExistingBuffer?: boolean): void;
        /**
         * Update a specific vertex buffer
         * This function will directly update the underlying DataBuffer according to the passed numeric array or Float32Array
         * It will do nothing if the buffer is not updatable
         * @param kind defines the data kind (Position, normal, etc...)
         * @param data defines the data to use
         * @param offset defines the offset in the target buffer where to store the data
         * @param useBytes set to true if the offset is in bytes
         */
        updateVerticesDataDirectly(kind: string, data: DataArray, offset: number, useBytes?: boolean): void;
        /**
         * Update a specific vertex buffer
         * This function will create a new buffer if the current one is not updatable
         * @param kind defines the data kind (Position, normal, etc...)
         * @param data defines the data to use
         * @param updateExtends defines if the geometry extends must be recomputed (false by default)
         */
        updateVerticesData(kind: string, data: FloatArray, updateExtends?: boolean): void;
        private _updateBoundingInfo;
        /**
         * @internal
         */
        _bind(effect: Nullable<Effect>, indexToBind?: Nullable<DataBuffer>, overrideVertexBuffers?: {
            [kind: string]: Nullable<VertexBuffer>;
        }, overrideVertexArrayObjects?: {
            [key: string]: WebGLVertexArrayObject;
        }): void;
        /**
         * Gets total number of vertices
         * @returns the total number of vertices
         */
        getTotalVertices(): number;
        /**
         * Gets a specific vertex data attached to this geometry. Float data is constructed if the vertex buffer data cannot be returned directly.
         * @param kind defines the data kind (Position, normal, etc...)
         * @param copyWhenShared defines if the returned array must be cloned upon returning it if the current geometry is shared between multiple meshes
         * @param forceCopy defines a boolean indicating that the returned array must be cloned upon returning it
         * @returns a float array containing vertex data
         */
        getVerticesData(kind: string, copyWhenShared?: boolean, forceCopy?: boolean): Nullable<FloatArray>;
        /**
         * Returns a boolean defining if the vertex data for the requested `kind` is updatable
         * @param kind defines the data kind (Position, normal, etc...)
         * @returns true if the vertex buffer with the specified kind is updatable
         */
        isVertexBufferUpdatable(kind: string): boolean;
        /**
         * Gets a specific vertex buffer
         * @param kind defines the data kind (Position, normal, etc...)
         * @returns a VertexBuffer
         */
        getVertexBuffer(kind: string): Nullable<VertexBuffer>;
        /**
         * Returns all vertex buffers
         * @returns an object holding all vertex buffers indexed by kind
         */
        getVertexBuffers(): Nullable<{
            [key: string]: VertexBuffer;
        }>;
        /**
         * Gets a boolean indicating if specific vertex buffer is present
         * @param kind defines the data kind (Position, normal, etc...)
         * @returns true if data is present
         */
        isVerticesDataPresent(kind: string): boolean;
        /**
         * Gets a list of all attached data kinds (Position, normal, etc...)
         * @returns a list of string containing all kinds
         */
        getVerticesDataKinds(): string[];
        /**
         * Update index buffer
         * @param indices defines the indices to store in the index buffer
         * @param offset defines the offset in the target buffer where to store the data
         * @param gpuMemoryOnly defines a boolean indicating that only the GPU memory must be updated leaving the CPU version of the indices unchanged (false by default)
         */
        updateIndices(indices: IndicesArray, offset?: number, gpuMemoryOnly?: boolean): void;
        /**
         * Creates a new index buffer
         * @param indices defines the indices to store in the index buffer
         * @param totalVertices defines the total number of vertices (could be null)
         * @param updatable defines if the index buffer must be flagged as updatable (false by default)
         */
        setIndices(indices: IndicesArray, totalVertices?: Nullable<number>, updatable?: boolean): void;
        /**
         * Return the total number of indices
         * @returns the total number of indices
         */
        getTotalIndices(): number;
        /**
         * Gets the index buffer array
         * @param copyWhenShared defines if the returned array must be cloned upon returning it if the current geometry is shared between multiple meshes
         * @param forceCopy defines a boolean indicating that the returned array must be cloned upon returning it
         * @returns the index buffer array
         */
        getIndices(copyWhenShared?: boolean, forceCopy?: boolean): Nullable<IndicesArray>;
        /**
         * Gets the index buffer
         * @returns the index buffer
         */
        getIndexBuffer(): Nullable<DataBuffer>;
        /**
         * @internal
         */
        _releaseVertexArrayObject(effect?: Nullable<Effect>): void;
        /**
         * Release the associated resources for a specific mesh
         * @param mesh defines the source mesh
         * @param shouldDispose defines if the geometry must be disposed if there is no more mesh pointing to it
         */
        releaseForMesh(mesh: Mesh, shouldDispose?: boolean): void;
        /**
         * Apply current geometry to a given mesh
         * @param mesh defines the mesh to apply geometry to
         */
        applyToMesh(mesh: Mesh): void;
        private _updateExtend;
        private _applyToMesh;
        private _notifyUpdate;
        /**
         * Load the geometry if it was flagged as delay loaded
         * @param scene defines the hosting scene
         * @param onLoaded defines a callback called when the geometry is loaded
         */
        load(scene: Scene, onLoaded?: () => void): void;
        private _queueLoad;
        /**
         * Invert the geometry to move from a right handed system to a left handed one.
         */
        toLeftHanded(): void;
        /** @internal */
        _resetPointsArrayCache(): void;
        /** @internal */
        _generatePointsArray(): boolean;
        /**
         * Gets a value indicating if the geometry is disposed
         * @returns true if the geometry was disposed
         */
        isDisposed(): boolean;
        private _disposeVertexArrayObjects;
        /**
         * Free all associated resources
         */
        dispose(): void;
        /**
         * Clone the current geometry into a new geometry
         * @param id defines the unique ID of the new geometry
         * @returns a new geometry object
         */
        copy(id: string): Geometry;
        /**
         * Serialize the current geometry info (and not the vertices data) into a JSON object
         * @returns a JSON representation of the current geometry data (without the vertices data)
         */
        serialize(): any;
        private _toNumberArray;
        /**
         * Release any memory retained by the cached data on the Geometry.
         *
         * Call this function to reduce memory footprint of the mesh.
         * Vertex buffers will not store CPU data anymore (this will prevent picking, collisions or physics to work correctly)
         */
        clearCachedData(): void;
        /**
         * Serialize all vertices data into a JSON object
         * @returns a JSON representation of the current geometry data
         */
        serializeVerticeData(): any;
        /**
         * Extracts a clone of a mesh geometry
         * @param mesh defines the source mesh
         * @param id defines the unique ID of the new geometry object
         * @returns the new geometry object
         */
        static ExtractFromMesh(mesh: Mesh, id: string): Nullable<Geometry>;
        /**
         * You should now use Tools.RandomId(), this method is still here for legacy reasons.
         * Implementation from http://stackoverflow.com/questions/105034/how-to-create-a-guid-uuid-in-javascript/2117523#answer-2117523
         * Be aware Math.random() could cause collisions, but:
         * "All but 6 of the 128 bits of the ID are randomly generated, which means that for any two ids, there's a 1 in 2^^122 (or 5.3x10^^36) chance they'll collide"
         * @returns a string containing a new GUID
         */
        static RandomId(): string;
        private static _GetGeometryByLoadedUniqueId;
        /**
         * @internal
         */
        static _ImportGeometry(parsedGeometry: any, mesh: Mesh): void;
        private static _CleanMatricesWeights;
        /**
         * Create a new geometry from persisted data (Using .babylon file format)
         * @param parsedVertexData defines the persisted data
         * @param scene defines the hosting scene
         * @param rootUrl defines the root url to use to load assets (like delayed data)
         * @returns the new geometry object
         */
        static Parse(parsedVertexData: any, scene: Scene, rootUrl: string): Nullable<Geometry>;
    }


    /**
     * Defines the set of goldberg data used to create the polygon
     */
    export type GoldbergData = {
        /**
         * The list of Goldberg faces colors
         */
        faceColors: Color4[];
        /**
         * The list of Goldberg faces centers
         */
        faceCenters: Vector3[];
        /**
         * The list of Goldberg faces Z axis
         */
        faceZaxis: Vector3[];
        /**
         * The list of Goldberg faces Y axis
         */
        faceXaxis: Vector3[];
        /**
         * The list of Goldberg faces X axis
         */
        faceYaxis: Vector3[];
        /**
         * Defines the number of shared faces
         */
        nbSharedFaces: number;
        /**
         * Defines the number of unshared faces
         */
        nbUnsharedFaces: number;
        /**
         * Defines the total number of goldberg faces
         */
        nbFaces: number;
        /**
         * Defines the number of goldberg faces at the pole
         */
        nbFacesAtPole: number;
        /**
         * Defines the number of adjacent faces per goldberg faces
         */
        adjacentFaces: number[][];
    };
    /**
     * Mesh for a Goldberg Polyhedron which is made from 12 pentagonal and the rest hexagonal faces
     * @see https://en.wikipedia.org/wiki/Goldberg_polyhedron
     */
    export class GoldbergMesh extends Mesh {
        /**
         * Defines the specific Goldberg data used in this mesh construction.
         */
        goldbergData: GoldbergData;
        /**
         * Gets the related Goldberg face from pole infos
         * @param poleOrShared Defines the pole index or the shared face index if the fromPole parameter is passed in
         * @param fromPole Defines an optional pole index to find the related info from
         * @returns the goldberg face number
         */
        relatedGoldbergFace(poleOrShared: number, fromPole?: number): number;
        private _changeGoldbergFaceColors;
        /**
         * Set new goldberg face colors
         * @param colorRange the new color to apply to the mesh
         */
        setGoldbergFaceColors(colorRange: (number | Color4)[][]): void;
        /**
         * Updates new goldberg face colors
         * @param colorRange the new color to apply to the mesh
         */
        updateGoldbergFaceColors(colorRange: (number | Color4)[][]): void;
        private _changeGoldbergFaceUVs;
        /**
         * set new goldberg face UVs
         * @param uvRange the new UVs to apply to the mesh
         */
        setGoldbergFaceUVs(uvRange: (number | Vector2)[][]): void;
        /**
         * Updates new goldberg face UVs
         * @param uvRange the new UVs to apply to the mesh
         */
        updateGoldbergFaceUVs(uvRange: (number | Vector2)[][]): void;
        /**
         * Places a mesh on a particular face of the goldberg polygon
         * @param mesh Defines the mesh to position
         * @param face Defines the face to position onto
         * @param position Defines the position relative to the face we are positioning the mesh onto
         */
        placeOnGoldbergFaceAt(mesh: Mesh, face: number, position: Vector3): void;
        /**
         * Serialize current mesh
         * @param serializationObject defines the object which will receive the serialization data
         */
        serialize(serializationObject: any): void;
        /**
         * Parses a serialized goldberg mesh
         * @param parsedMesh the serialized mesh
         * @param scene the scene to create the goldberg mesh in
         * @returns the created goldberg mesh
         */
        static Parse(parsedMesh: any, scene: Scene): GoldbergMesh;
    }


    /**
     * Mesh representing the ground
     */
    export class GroundMesh extends Mesh {
        /** If octree should be generated */
        generateOctree: boolean;
        private _heightQuads;
        /** @internal */
        _subdivisionsX: number;
        /** @internal */
        _subdivisionsY: number;
        /** @internal */
        _width: number;
        /** @internal */
        _height: number;
        /** @internal */
        _minX: number;
        /** @internal */
        _maxX: number;
        /** @internal */
        _minZ: number;
        /** @internal */
        _maxZ: number;
        constructor(name: string, scene?: Scene);
        /**
         * "GroundMesh"
         * @returns "GroundMesh"
         */
        getClassName(): string;
        /**
         * The minimum of x and y subdivisions
         */
        get subdivisions(): number;
        /**
         * X subdivisions
         */
        get subdivisionsX(): number;
        /**
         * Y subdivisions
         */
        get subdivisionsY(): number;
        /**
         * This function will divide the mesh into submeshes and update an octree to help to select the right submeshes
         * for rendering, picking and collision computations. Please note that you must have a decent number of submeshes
         * to get performance improvements when using an octree.
         * @param chunksCount the number of submeshes the mesh will be divided into
         * @param octreeBlocksSize the maximum size of the octree blocks (Default: 32)
         */
        optimize(chunksCount: number, octreeBlocksSize?: number): void;
        /**
         * Returns a height (y) value in the World system :
         * the ground altitude at the coordinates (x, z) expressed in the World system.
         * @param x x coordinate
         * @param z z coordinate
         * @returns the ground y position if (x, z) are outside the ground surface.
         */
        getHeightAtCoordinates(x: number, z: number): number;
        /**
         * Returns a normalized vector (Vector3) orthogonal to the ground
         * at the ground coordinates (x, z) expressed in the World system.
         * @param x x coordinate
         * @param z z coordinate
         * @returns Vector3(0.0, 1.0, 0.0) if (x, z) are outside the ground surface.
         */
        getNormalAtCoordinates(x: number, z: number): Vector3;
        /**
         * Updates the Vector3 passed a reference with a normalized vector orthogonal to the ground
         * at the ground coordinates (x, z) expressed in the World system.
         * Doesn't update the reference Vector3 if (x, z) are outside the ground surface.
         * @param x x coordinate
         * @param z z coordinate
         * @param ref vector to store the result
         * @returns the GroundMesh.
         */
        getNormalAtCoordinatesToRef(x: number, z: number, ref: Vector3): GroundMesh;
        /**
         * Force the heights to be recomputed for getHeightAtCoordinates() or getNormalAtCoordinates()
         * if the ground has been updated.
         * This can be used in the render loop.
         * @returns the GroundMesh.
         */
        updateCoordinateHeights(): GroundMesh;
        private _getFacetAt;
        private _initHeightQuads;
        private _computeHeightQuads;
        /**
         * Serializes this ground mesh
         * @param serializationObject object to write serialization to
         */
        serialize(serializationObject: any): void;
        /**
         * Parses a serialized ground mesh
         * @param parsedMesh the serialized mesh
         * @param scene the scene to create the ground mesh in
         * @returns the created ground mesh
         */
        static Parse(parsedMesh: any, scene: Scene): GroundMesh;
    }




    /**
     * Creates an instance based on a source mesh.
     */
    export class InstancedMesh extends AbstractMesh {
        private _sourceMesh;
        private _currentLOD;
        private _billboardWorldMatrix;
        /** @internal */
        _indexInSourceMeshInstanceArray: number;
        /** @internal */
        _distanceToCamera: number;
        /** @internal */
        _previousWorldMatrix: Nullable<Matrix>;
        constructor(name: string, source: Mesh);
        /**
         * Returns the string "InstancedMesh".
         */
        getClassName(): string;
        /** Gets the list of lights affecting that mesh */
        get lightSources(): Light[];
        _resyncLightSources(): void;
        _resyncLightSource(): void;
        _removeLightSource(): void;
        /**
         * If the source mesh receives shadows
         */
        get receiveShadows(): boolean;
        /**
         * The material of the source mesh
         */
        get material(): Nullable<Material>;
        /**
         * Visibility of the source mesh
         */
        get visibility(): number;
        /**
         * Skeleton of the source mesh
         */
        get skeleton(): Nullable<Skeleton>;
        /**
         * Rendering ground id of the source mesh
         */
        get renderingGroupId(): number;
        set renderingGroupId(value: number);
        /**
         * Returns the total number of vertices (integer).
         */
        getTotalVertices(): number;
        /**
         * Returns a positive integer : the total number of indices in this mesh geometry.
         * @returns the number of indices or zero if the mesh has no geometry.
         */
        getTotalIndices(): number;
        /**
         * The source mesh of the instance
         */
        get sourceMesh(): Mesh;
        /**
         * Creates a new InstancedMesh object from the mesh model.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/copies/instances
         * @param name defines the name of the new instance
         * @returns a new InstancedMesh
         */
        createInstance(name: string): InstancedMesh;
        /**
         * Is this node ready to be used/rendered
         * @param completeCheck defines if a complete check (including materials and lights) has to be done (false by default)
         * @returns {boolean} is it ready
         */
        isReady(completeCheck?: boolean): boolean;
        /**
         * Returns an array of integers or a typed array (Int32Array, Uint32Array, Uint16Array) populated with the mesh indices.
         * @param kind kind of verticies to retrieve (eg. positions, normals, uvs, etc.)
         * @param copyWhenShared If true (default false) and and if the mesh geometry is shared among some other meshes, the returned array is a copy of the internal one.
         * @returns a float array or a Float32Array of the requested kind of data : positions, normals, uvs, etc.
         */
        getVerticesData(kind: string, copyWhenShared?: boolean): Nullable<FloatArray>;
        /**
         * Sets the vertex data of the mesh geometry for the requested `kind`.
         * If the mesh has no geometry, a new Geometry object is set to the mesh and then passed this vertex data.
         * The `data` are either a numeric array either a Float32Array.
         * The parameter `updatable` is passed as is to the underlying Geometry object constructor (if initially none) or updater.
         * The parameter `stride` is an optional positive integer, it is usually automatically deducted from the `kind` (3 for positions or normals, 2 for UV, etc).
         * Note that a new underlying VertexBuffer object is created each call.
         * If the `kind` is the `PositionKind`, the mesh BoundingInfo is renewed, so the bounding box and sphere, and the mesh World Matrix is recomputed.
         *
         * Possible `kind` values :
         * - VertexBuffer.PositionKind
         * - VertexBuffer.UVKind
         * - VertexBuffer.UV2Kind
         * - VertexBuffer.UV3Kind
         * - VertexBuffer.UV4Kind
         * - VertexBuffer.UV5Kind
         * - VertexBuffer.UV6Kind
         * - VertexBuffer.ColorKind
         * - VertexBuffer.MatricesIndicesKind
         * - VertexBuffer.MatricesIndicesExtraKind
         * - VertexBuffer.MatricesWeightsKind
         * - VertexBuffer.MatricesWeightsExtraKind
         *
         * Returns the Mesh.
         * @param kind
         * @param data
         * @param updatable
         * @param stride
         */
        setVerticesData(kind: string, data: FloatArray, updatable?: boolean, stride?: number): AbstractMesh;
        /**
         * Updates the existing vertex data of the mesh geometry for the requested `kind`.
         * If the mesh has no geometry, it is simply returned as it is.
         * The `data` are either a numeric array either a Float32Array.
         * No new underlying VertexBuffer object is created.
         * If the `kind` is the `PositionKind` and if `updateExtends` is true, the mesh BoundingInfo is renewed, so the bounding box and sphere, and the mesh World Matrix is recomputed.
         * If the parameter `makeItUnique` is true, a new global geometry is created from this positions and is set to the mesh.
         *
         * Possible `kind` values :
         * - VertexBuffer.PositionKind
         * - VertexBuffer.UVKind
         * - VertexBuffer.UV2Kind
         * - VertexBuffer.UV3Kind
         * - VertexBuffer.UV4Kind
         * - VertexBuffer.UV5Kind
         * - VertexBuffer.UV6Kind
         * - VertexBuffer.ColorKind
         * - VertexBuffer.MatricesIndicesKind
         * - VertexBuffer.MatricesIndicesExtraKind
         * - VertexBuffer.MatricesWeightsKind
         * - VertexBuffer.MatricesWeightsExtraKind
         *
         * Returns the Mesh.
         * @param kind
         * @param data
         * @param updateExtends
         * @param makeItUnique
         */
        updateVerticesData(kind: string, data: FloatArray, updateExtends?: boolean, makeItUnique?: boolean): Mesh;
        /**
         * Sets the mesh indices.
         * Expects an array populated with integers or a typed array (Int32Array, Uint32Array, Uint16Array).
         * If the mesh has no geometry, a new Geometry object is created and set to the mesh.
         * This method creates a new index buffer each call.
         * Returns the Mesh.
         * @param indices
         * @param totalVertices
         */
        setIndices(indices: IndicesArray, totalVertices?: Nullable<number>): Mesh;
        /**
         * Boolean : True if the mesh owns the requested kind of data.
         * @param kind
         */
        isVerticesDataPresent(kind: string): boolean;
        /**
         * Returns an array of indices (IndicesArray).
         */
        getIndices(): Nullable<IndicesArray>;
        get _positions(): Nullable<Vector3[]>;
        /**
         * This method recomputes and sets a new BoundingInfo to the mesh unless it is locked.
         * This means the mesh underlying bounding box and sphere are recomputed.
         * @param applySkeleton defines whether to apply the skeleton before computing the bounding info
         * @param applyMorph  defines whether to apply the morph target before computing the bounding info
         * @returns the current mesh
         */
        refreshBoundingInfo(applySkeleton?: boolean, applyMorph?: boolean): InstancedMesh;
        /** @internal */
        _preActivate(): InstancedMesh;
        /**
         * @internal
         */
        _activate(renderId: number, intermediateRendering: boolean): boolean;
        /** @internal */
        _postActivate(): void;
        getWorldMatrix(): Matrix;
        get isAnInstance(): boolean;
        /**
         * Returns the current associated LOD AbstractMesh.
         * @param camera
         */
        getLOD(camera: Camera): AbstractMesh;
        /**
         * @internal
         */
        _preActivateForIntermediateRendering(renderId: number): Mesh;
        /** @internal */
        _syncSubMeshes(): InstancedMesh;
        /** @internal */
        _generatePointsArray(): boolean;
        /** @internal */
        _updateBoundingInfo(): AbstractMesh;
        /**
         * Creates a new InstancedMesh from the current mesh.
         *
         * Returns the clone.
         * @param name the cloned mesh name
         * @param newParent the optional Node to parent the clone to.
         * @param doNotCloneChildren if `true` the model children aren't cloned.
         * @param newSourceMesh if set this mesh will be used as the source mesh instead of ths instance's one
         * @returns the clone
         */
        clone(name: string, newParent?: Nullable<Node>, doNotCloneChildren?: boolean, newSourceMesh?: Mesh): InstancedMesh;
        /**
         * Disposes the InstancedMesh.
         * Returns nothing.
         * @param doNotRecurse
         * @param disposeMaterialAndTextures
         */
        dispose(doNotRecurse?: boolean, disposeMaterialAndTextures?: boolean): void;
        /**
         * @internal
         */
        _serializeAsParent(serializationObject: any): void;
        /**
         * Instantiate (when possible) or clone that node with its hierarchy
         * @param newParent defines the new parent to use for the instance (or clone)
         * @param options defines options to configure how copy is done
         * @param options.doNotInstantiate defines if the model must be instantiated or just cloned
         * @param options.newSourcedMesh newSourcedMesh the new source mesh for the instance (or clone)
         * @param onNewNodeCreated defines an option callback to call when a clone or an instance is created
         * @returns an instance (or a clone) of the current node with its hierarchy
         */
        instantiateHierarchy(newParent?: Nullable<TransformNode>, options?: {
            doNotInstantiate: boolean | ((node: TransformNode) => boolean);
            newSourcedMesh?: Mesh;
        }, onNewNodeCreated?: (source: TransformNode, clone: TransformNode) => void): Nullable<TransformNode>;
    }
        interface Mesh {
            /**
             * Register a custom buffer that will be instanced
             * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/copies/instances#custom-buffers
             * @param kind defines the buffer kind
             * @param stride defines the stride in floats
             */
            registerInstancedBuffer(kind: string, stride: number): void;
            /**
             * Invalidate VertexArrayObjects belonging to the mesh (but not to the Geometry of the mesh).
             */
            _invalidateInstanceVertexArrayObject(): void;
            /**
             * true to use the edge renderer for all instances of this mesh
             */
            edgesShareWithInstances: boolean;
            /** @internal */
            _userInstancedBuffersStorage: {
                data: {
                    [key: string]: Float32Array;
                };
                sizes: {
                    [key: string]: number;
                };
                vertexBuffers: {
                    [key: string]: Nullable<VertexBuffer>;
                };
                strides: {
                    [key: string]: number;
                };
                vertexArrayObjects?: {
                    [key: string]: WebGLVertexArrayObject;
                };
            };
        }
        interface AbstractMesh {
            /**
             * Object used to store instanced buffers defined by user
             * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/copies/instances#custom-buffers
             */
            instancedBuffers: {
                [key: string]: any;
            };
        }


    /**
     * Line mesh
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/param
     */
    export class LinesMesh extends Mesh {
        /**
         * If vertex color should be applied to the mesh
         */
        readonly useVertexColor?: boolean | undefined;
        /**
         * If vertex alpha should be applied to the mesh
         */
        readonly useVertexAlpha?: boolean | undefined;
        /**
         * Color of the line (Default: White)
         */
        color: Color3;
        /**
         * Alpha of the line (Default: 1)
         */
        alpha: number;
        /**
         * The intersection Threshold is the margin applied when intersection a segment of the LinesMesh with a Ray.
         * This margin is expressed in world space coordinates, so its value may vary.
         * Default value is 0.1
         */
        intersectionThreshold: number;
        private _lineMaterial;
        private _isShaderMaterial;
        private _color4;
        /**
         * Creates a new LinesMesh
         * @param name defines the name
         * @param scene defines the hosting scene
         * @param parent defines the parent mesh if any
         * @param source defines the optional source LinesMesh used to clone data from
         * @param doNotCloneChildren When cloning, skip cloning child meshes of source, default False.
         * When false, achieved by calling a clone(), also passing False.
         * This will make creation of children, recursive.
         * @param useVertexColor defines if this LinesMesh supports vertex color
         * @param useVertexAlpha defines if this LinesMesh supports vertex alpha
         * @param material material to use to draw the line. If not provided, will create a new one
         */
        constructor(name: string, scene?: Nullable<Scene>, parent?: Nullable<Node>, source?: Nullable<LinesMesh>, doNotCloneChildren?: boolean, 
        /**
         * If vertex color should be applied to the mesh
         */
        useVertexColor?: boolean | undefined, 
        /**
         * If vertex alpha should be applied to the mesh
         */
        useVertexAlpha?: boolean | undefined, material?: Material);
        isReady(): boolean;
        /**
         * Returns the string "LineMesh"
         */
        getClassName(): string;
        /**
         * @internal
         */
        get material(): Material;
        /**
         * @internal
         */
        set material(value: Material);
        /**
         * @internal
         */
        get checkCollisions(): boolean;
        set checkCollisions(value: boolean);
        /**
         * @internal
         */
        _bind(_subMesh: SubMesh, colorEffect: Effect): Mesh;
        /**
         * @internal
         */
        _draw(subMesh: SubMesh, fillMode: number, instancesCount?: number): Mesh;
        /**
         * Disposes of the line mesh
         * @param doNotRecurse If children should be disposed
         * @param disposeMaterialAndTextures This parameter is not used by the LineMesh class
         * @param doNotDisposeMaterial If the material should not be disposed (default: false, meaning the material is disposed)
         */
        dispose(doNotRecurse?: boolean, disposeMaterialAndTextures?: boolean, doNotDisposeMaterial?: boolean): void;
        /**
         * Returns a new LineMesh object cloned from the current one.
         * @param name
         * @param newParent
         * @param doNotCloneChildren
         */
        clone(name: string, newParent?: Nullable<Node>, doNotCloneChildren?: boolean): LinesMesh;
        /**
         * Creates a new InstancedLinesMesh object from the mesh model.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/copies/instances
         * @param name defines the name of the new instance
         * @returns a new InstancedLinesMesh
         */
        createInstance(name: string): InstancedLinesMesh;
        /**
         * Serializes this ground mesh
         * @param serializationObject object to write serialization to
         */
        serialize(serializationObject: any): void;
        /**
         * Parses a serialized ground mesh
         * @param parsedMesh the serialized mesh
         * @param scene the scene to create the ground mesh in
         * @returns the created ground mesh
         */
        static Parse(parsedMesh: any, scene: Scene): LinesMesh;
    }
    /**
     * Creates an instance based on a source LinesMesh
     */
    export class InstancedLinesMesh extends InstancedMesh {
        /**
         * The intersection Threshold is the margin applied when intersection a segment of the LinesMesh with a Ray.
         * This margin is expressed in world space coordinates, so its value may vary.
         * Initialized with the intersectionThreshold value of the source LinesMesh
         */
        intersectionThreshold: number;
        constructor(name: string, source: LinesMesh);
        /**
         * Returns the string "InstancedLinesMesh".
         */
        getClassName(): string;
    }


    /**
     * @internal
     **/
    export class _CreationDataStorage {
        closePath?: boolean;
        closeArray?: boolean;
        idx: number[];
        dashSize: number;
        gapSize: number;
        path3D: Path3D;
        pathArray: Vector3[][];
        arc: number;
        radius: number;
        cap: number;
        tessellation: number;
    }
    /**
     * @internal
     **/
    class _InstanceDataStorage {
        visibleInstances: any;
        batchCache: _InstancesBatch;
        batchCacheReplacementModeInFrozenMode: _InstancesBatch;
        instancesBufferSize: number;
        instancesBuffer: Nullable<Buffer>;
        instancesPreviousBuffer: Nullable<Buffer>;
        instancesData: Float32Array;
        instancesPreviousData: Float32Array;
        overridenInstanceCount: number;
        isFrozen: boolean;
        forceMatrixUpdates: boolean;
        previousBatch: Nullable<_InstancesBatch>;
        hardwareInstancedRendering: boolean;
        sideOrientation: number;
        manualUpdate: boolean;
        previousManualUpdate: boolean;
        previousRenderId: number;
        masterMeshPreviousWorldMatrix: Nullable<Matrix>;
    }
    /**
     * @internal
     **/
    export class _InstancesBatch {
        mustReturn: boolean;
        visibleInstances: Nullable<InstancedMesh[]>[];
        renderSelf: boolean[];
        hardwareInstancedRendering: boolean[];
    }
    /**
     * @internal
     **/
    class _ThinInstanceDataStorage {
        instancesCount: number;
        matrixBuffer: Nullable<Buffer>;
        previousMatrixBuffer: Nullable<Buffer>;
        matrixBufferSize: number;
        matrixData: Nullable<Float32Array>;
        previousMatrixData: Nullable<Float32Array>;
        boundingVectors: Array<Vector3>;
        worldMatrices: Nullable<Matrix[]>;
        masterMeshPreviousWorldMatrix: Nullable<Matrix>;
    }
    /**
     * Class used to represent renderable models
     */
    export class Mesh extends AbstractMesh implements IGetSetVerticesData {
        /**
         * Mesh side orientation : usually the external or front surface
         */
        static readonly FRONTSIDE = 0;
        /**
         * Mesh side orientation : usually the internal or back surface
         */
        static readonly BACKSIDE = 1;
        /**
         * Mesh side orientation : both internal and external or front and back surfaces
         */
        static readonly DOUBLESIDE = 2;
        /**
         * Mesh side orientation : by default, `FRONTSIDE`
         */
        static readonly DEFAULTSIDE = 0;
        /**
         * Mesh cap setting : no cap
         */
        static readonly NO_CAP = 0;
        /**
         * Mesh cap setting : one cap at the beginning of the mesh
         */
        static readonly CAP_START = 1;
        /**
         * Mesh cap setting : one cap at the end of the mesh
         */
        static readonly CAP_END = 2;
        /**
         * Mesh cap setting : two caps, one at the beginning  and one at the end of the mesh
         */
        static readonly CAP_ALL = 3;
        /**
         * Mesh pattern setting : no flip or rotate
         */
        static readonly NO_FLIP = 0;
        /**
         * Mesh pattern setting : flip (reflect in y axis) alternate tiles on each row or column
         */
        static readonly FLIP_TILE = 1;
        /**
         * Mesh pattern setting : rotate (180degs) alternate tiles on each row or column
         */
        static readonly ROTATE_TILE = 2;
        /**
         * Mesh pattern setting : flip (reflect in y axis) all tiles on alternate rows
         */
        static readonly FLIP_ROW = 3;
        /**
         * Mesh pattern setting : rotate (180degs) all tiles on alternate rows
         */
        static readonly ROTATE_ROW = 4;
        /**
         * Mesh pattern setting : flip and rotate alternate tiles on each row or column
         */
        static readonly FLIP_N_ROTATE_TILE = 5;
        /**
         * Mesh pattern setting : rotate pattern and rotate
         */
        static readonly FLIP_N_ROTATE_ROW = 6;
        /**
         * Mesh tile positioning : part tiles same on left/right or top/bottom
         */
        static readonly CENTER = 0;
        /**
         * Mesh tile positioning : part tiles on left
         */
        static readonly LEFT = 1;
        /**
         * Mesh tile positioning : part tiles on right
         */
        static readonly RIGHT = 2;
        /**
         * Mesh tile positioning : part tiles on top
         */
        static readonly TOP = 3;
        /**
         * Mesh tile positioning : part tiles on bottom
         */
        static readonly BOTTOM = 4;
        /**
         * Indicates that the instanced meshes should be sorted from back to front before rendering if their material is transparent
         */
        static INSTANCEDMESH_SORT_TRANSPARENT: boolean;
        /**
         * Gets the default side orientation.
         * @param orientation the orientation to value to attempt to get
         * @returns the default orientation
         * @internal
         */
        static _GetDefaultSideOrientation(orientation?: number): number;
        private _internalMeshDataInfo;
        /**
         * Determines if the LOD levels are intended to be calculated using screen coverage (surface area ratio) instead of distance.
         */
        get useLODScreenCoverage(): boolean;
        set useLODScreenCoverage(value: boolean);
        /**
         * Will notify when the mesh is completely ready, including materials.
         * Observers added to this observable will be removed once triggered
         */
        onMeshReadyObservable: Observable<Mesh>;
        get computeBonesUsingShaders(): boolean;
        set computeBonesUsingShaders(value: boolean);
        /**
         * An event triggered before rendering the mesh
         */
        get onBeforeRenderObservable(): Observable<Mesh>;
        /**
         * An event triggered before binding the mesh
         */
        get onBeforeBindObservable(): Observable<Mesh>;
        /**
         * An event triggered after rendering the mesh
         */
        get onAfterRenderObservable(): Observable<Mesh>;
        /**
         * An event triggeredbetween rendering pass when using separateCullingPass = true
         */
        get onBetweenPassObservable(): Observable<SubMesh>;
        /**
         * An event triggered before drawing the mesh
         */
        get onBeforeDrawObservable(): Observable<Mesh>;
        private _onBeforeDrawObserver;
        /**
         * Sets a callback to call before drawing the mesh. It is recommended to use onBeforeDrawObservable instead
         */
        set onBeforeDraw(callback: () => void);
        get hasInstances(): boolean;
        get hasThinInstances(): boolean;
        /**
         * Gets the delay loading state of the mesh (when delay loading is turned on)
         * @see https://doc.babylonjs.com/features/featuresDeepDive/importers/incrementalLoading
         */
        delayLoadState: number;
        /**
         * Gets the list of instances created from this mesh
         * it is not supposed to be modified manually.
         * Note also that the order of the InstancedMesh wihin the array is not significant and might change.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/copies/instances
         */
        instances: InstancedMesh[];
        /**
         * Gets the file containing delay loading data for this mesh
         */
        delayLoadingFile: string;
        /** @internal */
        _binaryInfo: any;
        /**
         * User defined function used to change how LOD level selection is done
         * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/LOD
         */
        onLODLevelSelection: (distance: number, mesh: Mesh, selectedLevel: Nullable<Mesh>) => void;
        /** @internal */
        _creationDataStorage: Nullable<_CreationDataStorage>;
        /** @internal */
        _geometry: Nullable<Geometry>;
        /** @internal */
        _delayInfo: Array<string>;
        /** @internal */
        _delayLoadingFunction: (any: any, mesh: Mesh) => void;
        /**
         * Gets or sets the forced number of instances to display.
         * If 0 (default value), the number of instances is not forced and depends on the draw type
         * (regular / instance / thin instances mesh)
         */
        get forcedInstanceCount(): number;
        set forcedInstanceCount(count: number);
        /** @internal */
        _instanceDataStorage: _InstanceDataStorage;
        /** @internal */
        _thinInstanceDataStorage: _ThinInstanceDataStorage;
        /** @internal */
        _shouldGenerateFlatShading: boolean;
        /** @internal */
        _originalBuilderSideOrientation: number;
        /**
         * Use this property to change the original side orientation defined at construction time
         */
        overrideMaterialSideOrientation: Nullable<number>;
        /**
         * Gets or sets a boolean indicating whether to render ignoring the active camera's max z setting. (false by default)
         * Note this will reduce performance when set to true.
         */
        ignoreCameraMaxZ: boolean;
        /**
         * Gets the source mesh (the one used to clone this one from)
         */
        get source(): Nullable<Mesh>;
        /**
         * Gets the list of clones of this mesh
         * The scene must have been constructed with useClonedMeshMap=true for this to work!
         * Note that useClonedMeshMap=true is the default setting
         */
        get cloneMeshMap(): Nullable<{
            [id: string]: Mesh | undefined;
        }>;
        /**
         * Gets or sets a boolean indicating that this mesh does not use index buffer
         */
        get isUnIndexed(): boolean;
        set isUnIndexed(value: boolean);
        /** Gets the array buffer used to store the instanced buffer used for instances' world matrices */
        get worldMatrixInstancedBuffer(): Float32Array;
        /** Gets the array buffer used to store the instanced buffer used for instances' previous world matrices */
        get previousWorldMatrixInstancedBuffer(): Float32Array;
        /** Gets or sets a boolean indicating that the update of the instance buffer of the world matrices is manual */
        get manualUpdateOfWorldMatrixInstancedBuffer(): boolean;
        set manualUpdateOfWorldMatrixInstancedBuffer(value: boolean);
        /** Gets or sets a boolean indicating that the update of the instance buffer of the world matrices is manual */
        get manualUpdateOfPreviousWorldMatrixInstancedBuffer(): boolean;
        set manualUpdateOfPreviousWorldMatrixInstancedBuffer(value: boolean);
        /** Gets or sets a boolean indicating that the update of the instance buffer of the world matrices must be performed in all cases (and notably even in frozen mode) */
        get forceWorldMatrixInstancedBufferUpdate(): boolean;
        set forceWorldMatrixInstancedBufferUpdate(value: boolean);
        /**
         * @constructor
         * @param name The value used by scene.getMeshByName() to do a lookup.
         * @param scene The scene to add this mesh to.
         * @param parent The parent of this mesh, if it has one
         * @param source An optional Mesh from which geometry is shared, cloned.
         * @param doNotCloneChildren When cloning, skip cloning child meshes of source, default False.
         *                  When false, achieved by calling a clone(), also passing False.
         *                  This will make creation of children, recursive.
         * @param clonePhysicsImpostor When cloning, include cloning mesh physics impostor, default True.
         */
        constructor(name: string, scene?: Nullable<Scene>, parent?: Nullable<Node>, source?: Nullable<Mesh>, doNotCloneChildren?: boolean, clonePhysicsImpostor?: boolean);
        instantiateHierarchy(newParent?: Nullable<TransformNode>, options?: {
            doNotInstantiate: boolean | ((node: TransformNode) => boolean);
        }, onNewNodeCreated?: (source: TransformNode, clone: TransformNode) => void): Nullable<TransformNode>;
        /**
         * Gets the class name
         * @returns the string "Mesh".
         */
        getClassName(): string;
        /** @internal */
        get _isMesh(): boolean;
        /**
         * Returns a description of this mesh
         * @param fullDetails define if full details about this mesh must be used
         * @returns a descriptive string representing this mesh
         */
        toString(fullDetails?: boolean): string;
        /** @internal */
        _unBindEffect(): void;
        /**
         * Gets a boolean indicating if this mesh has LOD
         */
        get hasLODLevels(): boolean;
        /**
         * Gets the list of MeshLODLevel associated with the current mesh
         * @returns an array of MeshLODLevel
         */
        getLODLevels(): MeshLODLevel[];
        private _sortLODLevels;
        /**
         * Add a mesh as LOD level triggered at the given distance.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/LOD
         * @param distanceOrScreenCoverage Either distance from the center of the object to show this level or the screen coverage if `useScreenCoverage` is set to `true`.
         * If screen coverage, value is a fraction of the screen's total surface, between 0 and 1.
         * Example Playground for distance https://playground.babylonjs.com/#QE7KM#197
         * Example Playground for screen coverage https://playground.babylonjs.com/#QE7KM#196
         * @param mesh The mesh to be added as LOD level (can be null)
         * @returns This mesh (for chaining)
         */
        addLODLevel(distanceOrScreenCoverage: number, mesh: Nullable<Mesh>): Mesh;
        /**
         * Returns the LOD level mesh at the passed distance or null if not found.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/LOD
         * @param distance The distance from the center of the object to show this level
         * @returns a Mesh or `null`
         */
        getLODLevelAtDistance(distance: number): Nullable<Mesh>;
        /**
         * Remove a mesh from the LOD array
         * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/LOD
         * @param mesh defines the mesh to be removed
         * @returns This mesh (for chaining)
         */
        removeLODLevel(mesh: Nullable<Mesh>): Mesh;
        /**
         * Returns the registered LOD mesh distant from the parameter `camera` position if any, else returns the current mesh.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/LOD
         * @param camera defines the camera to use to compute distance
         * @param boundingSphere defines a custom bounding sphere to use instead of the one from this mesh
         * @returns This mesh (for chaining)
         */
        getLOD(camera: Camera, boundingSphere?: BoundingSphere): Nullable<AbstractMesh>;
        /**
         * Gets the mesh internal Geometry object
         */
        get geometry(): Nullable<Geometry>;
        /**
         * Returns the total number of vertices within the mesh geometry or zero if the mesh has no geometry.
         * @returns the total number of vertices
         */
        getTotalVertices(): number;
        /**
         * Returns the content of an associated vertex buffer
         * @param kind defines which buffer to read from (positions, indices, normals, etc). Possible `kind` values :
         * - VertexBuffer.PositionKind
         * - VertexBuffer.UVKind
         * - VertexBuffer.UV2Kind
         * - VertexBuffer.UV3Kind
         * - VertexBuffer.UV4Kind
         * - VertexBuffer.UV5Kind
         * - VertexBuffer.UV6Kind
         * - VertexBuffer.ColorKind
         * - VertexBuffer.MatricesIndicesKind
         * - VertexBuffer.MatricesIndicesExtraKind
         * - VertexBuffer.MatricesWeightsKind
         * - VertexBuffer.MatricesWeightsExtraKind
         * @param copyWhenShared defines a boolean indicating that if the mesh geometry is shared among some other meshes, the returned array is a copy of the internal one
         * @param forceCopy defines a boolean forcing the copy of the buffer no matter what the value of copyWhenShared is
         * @returns a FloatArray or null if the mesh has no geometry or no vertex buffer for this kind.
         */
        getVerticesData(kind: string, copyWhenShared?: boolean, forceCopy?: boolean): Nullable<FloatArray>;
        /**
         * Returns the mesh VertexBuffer object from the requested `kind`
         * @param kind defines which buffer to read from (positions, indices, normals, etc). Possible `kind` values :
         * - VertexBuffer.PositionKind
         * - VertexBuffer.NormalKind
         * - VertexBuffer.UVKind
         * - VertexBuffer.UV2Kind
         * - VertexBuffer.UV3Kind
         * - VertexBuffer.UV4Kind
         * - VertexBuffer.UV5Kind
         * - VertexBuffer.UV6Kind
         * - VertexBuffer.ColorKind
         * - VertexBuffer.MatricesIndicesKind
         * - VertexBuffer.MatricesIndicesExtraKind
         * - VertexBuffer.MatricesWeightsKind
         * - VertexBuffer.MatricesWeightsExtraKind
         * @returns a FloatArray or null if the mesh has no vertex buffer for this kind.
         */
        getVertexBuffer(kind: string): Nullable<VertexBuffer>;
        /**
         * Tests if a specific vertex buffer is associated with this mesh
         * @param kind defines which buffer to check (positions, indices, normals, etc). Possible `kind` values :
         * - VertexBuffer.PositionKind
         * - VertexBuffer.NormalKind
         * - VertexBuffer.UVKind
         * - VertexBuffer.UV2Kind
         * - VertexBuffer.UV3Kind
         * - VertexBuffer.UV4Kind
         * - VertexBuffer.UV5Kind
         * - VertexBuffer.UV6Kind
         * - VertexBuffer.ColorKind
         * - VertexBuffer.MatricesIndicesKind
         * - VertexBuffer.MatricesIndicesExtraKind
         * - VertexBuffer.MatricesWeightsKind
         * - VertexBuffer.MatricesWeightsExtraKind
         * @returns a boolean
         */
        isVerticesDataPresent(kind: string): boolean;
        /**
         * Returns a boolean defining if the vertex data for the requested `kind` is updatable.
         * @param kind defines which buffer to check (positions, indices, normals, etc). Possible `kind` values :
         * - VertexBuffer.PositionKind
         * - VertexBuffer.UVKind
         * - VertexBuffer.UV2Kind
         * - VertexBuffer.UV3Kind
         * - VertexBuffer.UV4Kind
         * - VertexBuffer.UV5Kind
         * - VertexBuffer.UV6Kind
         * - VertexBuffer.ColorKind
         * - VertexBuffer.MatricesIndicesKind
         * - VertexBuffer.MatricesIndicesExtraKind
         * - VertexBuffer.MatricesWeightsKind
         * - VertexBuffer.MatricesWeightsExtraKind
         * @returns a boolean
         */
        isVertexBufferUpdatable(kind: string): boolean;
        /**
         * Returns a string which contains the list of existing `kinds` of Vertex Data associated with this mesh.
         * @returns an array of strings
         */
        getVerticesDataKinds(): string[];
        /**
         * Returns a positive integer : the total number of indices in this mesh geometry.
         * @returns the numner of indices or zero if the mesh has no geometry.
         */
        getTotalIndices(): number;
        /**
         * Returns an array of integers or a typed array (Int32Array, Uint32Array, Uint16Array) populated with the mesh indices.
         * @param copyWhenShared If true (default false) and and if the mesh geometry is shared among some other meshes, the returned array is a copy of the internal one.
         * @param forceCopy defines a boolean indicating that the returned array must be cloned upon returning it
         * @returns the indices array or an empty array if the mesh has no geometry
         */
        getIndices(copyWhenShared?: boolean, forceCopy?: boolean): Nullable<IndicesArray>;
        get isBlocked(): boolean;
        /**
         * Determine if the current mesh is ready to be rendered
         * @param completeCheck defines if a complete check (including materials and lights) has to be done (false by default)
         * @param forceInstanceSupport will check if the mesh will be ready when used with instances (false by default)
         * @returns true if all associated assets are ready (material, textures, shaders)
         */
        isReady(completeCheck?: boolean, forceInstanceSupport?: boolean): boolean;
        /**
         * Gets a boolean indicating if the normals aren't to be recomputed on next mesh `positions` array update. This property is pertinent only for updatable parametric shapes.
         */
        get areNormalsFrozen(): boolean;
        /**
         * This function affects parametric shapes on vertex position update only : ribbons, tubes, etc. It has no effect at all on other shapes. It prevents the mesh normals from being recomputed on next `positions` array update.
         * @returns the current mesh
         */
        freezeNormals(): Mesh;
        /**
         * This function affects parametric shapes on vertex position update only : ribbons, tubes, etc. It has no effect at all on other shapes. It reactivates the mesh normals computation if it was previously frozen
         * @returns the current mesh
         */
        unfreezeNormals(): Mesh;
        /**
         * Sets a value overriding the instance count. Only applicable when custom instanced InterleavedVertexBuffer are used rather than InstancedMeshs
         */
        set overridenInstanceCount(count: number);
        /** @internal */
        _preActivate(): Mesh;
        /**
         * @internal
         */
        _preActivateForIntermediateRendering(renderId: number): Mesh;
        /**
         * @internal
         */
        _registerInstanceForRenderId(instance: InstancedMesh, renderId: number): Mesh;
        protected _afterComputeWorldMatrix(): void;
        /** @internal */
        _postActivate(): void;
        /**
         * This method recomputes and sets a new BoundingInfo to the mesh unless it is locked.
         * This means the mesh underlying bounding box and sphere are recomputed.
         * @param applySkeleton defines whether to apply the skeleton before computing the bounding info
         * @param applyMorph  defines whether to apply the morph target before computing the bounding info
         * @returns the current mesh
         */
        refreshBoundingInfo(applySkeleton?: boolean, applyMorph?: boolean): Mesh;
        /**
         * @internal
         */
        _createGlobalSubMesh(force: boolean): Nullable<SubMesh>;
        /**
         * This function will subdivide the mesh into multiple submeshes
         * @param count defines the expected number of submeshes
         */
        subdivide(count: number): void;
        /**
         * Copy a FloatArray into a specific associated vertex buffer
         * @param kind defines which buffer to write to (positions, indices, normals, etc). Possible `kind` values :
         * - VertexBuffer.PositionKind
         * - VertexBuffer.UVKind
         * - VertexBuffer.UV2Kind
         * - VertexBuffer.UV3Kind
         * - VertexBuffer.UV4Kind
         * - VertexBuffer.UV5Kind
         * - VertexBuffer.UV6Kind
         * - VertexBuffer.ColorKind
         * - VertexBuffer.MatricesIndicesKind
         * - VertexBuffer.MatricesIndicesExtraKind
         * - VertexBuffer.MatricesWeightsKind
         * - VertexBuffer.MatricesWeightsExtraKind
         * @param data defines the data source
         * @param updatable defines if the updated vertex buffer must be flagged as updatable
         * @param stride defines the data stride size (can be null)
         * @returns the current mesh
         */
        setVerticesData(kind: string, data: FloatArray, updatable?: boolean, stride?: number): AbstractMesh;
        /**
         * Delete a vertex buffer associated with this mesh
         * @param kind defines which buffer to delete (positions, indices, normals, etc). Possible `kind` values :
         * - VertexBuffer.PositionKind
         * - VertexBuffer.UVKind
         * - VertexBuffer.UV2Kind
         * - VertexBuffer.UV3Kind
         * - VertexBuffer.UV4Kind
         * - VertexBuffer.UV5Kind
         * - VertexBuffer.UV6Kind
         * - VertexBuffer.ColorKind
         * - VertexBuffer.MatricesIndicesKind
         * - VertexBuffer.MatricesIndicesExtraKind
         * - VertexBuffer.MatricesWeightsKind
         * - VertexBuffer.MatricesWeightsExtraKind
         */
        removeVerticesData(kind: string): void;
        /**
         * Flags an associated vertex buffer as updatable
         * @param kind defines which buffer to use (positions, indices, normals, etc). Possible `kind` values :
         * - VertexBuffer.PositionKind
         * - VertexBuffer.UVKind
         * - VertexBuffer.UV2Kind
         * - VertexBuffer.UV3Kind
         * - VertexBuffer.UV4Kind
         * - VertexBuffer.UV5Kind
         * - VertexBuffer.UV6Kind
         * - VertexBuffer.ColorKind
         * - VertexBuffer.MatricesIndicesKind
         * - VertexBuffer.MatricesIndicesExtraKind
         * - VertexBuffer.MatricesWeightsKind
         * - VertexBuffer.MatricesWeightsExtraKind
         * @param updatable defines if the updated vertex buffer must be flagged as updatable
         */
        markVerticesDataAsUpdatable(kind: string, updatable?: boolean): void;
        /**
         * Sets the mesh global Vertex Buffer
         * @param buffer defines the buffer to use
         * @param disposeExistingBuffer disposes the existing buffer, if any (default: true)
         * @returns the current mesh
         */
        setVerticesBuffer(buffer: VertexBuffer, disposeExistingBuffer?: boolean): Mesh;
        /**
         * Update a specific associated vertex buffer
         * @param kind defines which buffer to write to (positions, indices, normals, etc). Possible `kind` values :
         * - VertexBuffer.PositionKind
         * - VertexBuffer.UVKind
         * - VertexBuffer.UV2Kind
         * - VertexBuffer.UV3Kind
         * - VertexBuffer.UV4Kind
         * - VertexBuffer.UV5Kind
         * - VertexBuffer.UV6Kind
         * - VertexBuffer.ColorKind
         * - VertexBuffer.MatricesIndicesKind
         * - VertexBuffer.MatricesIndicesExtraKind
         * - VertexBuffer.MatricesWeightsKind
         * - VertexBuffer.MatricesWeightsExtraKind
         * @param data defines the data source
         * @param updateExtends defines if extends info of the mesh must be updated (can be null). This is mostly useful for "position" kind
         * @param makeItUnique defines if the geometry associated with the mesh must be cloned to make the change only for this mesh (and not all meshes associated with the same geometry)
         * @returns the current mesh
         */
        updateVerticesData(kind: string, data: FloatArray, updateExtends?: boolean, makeItUnique?: boolean): AbstractMesh;
        /**
         * This method updates the vertex positions of an updatable mesh according to the `positionFunction` returned values.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/dynamicMeshMorph#other-shapes-updatemeshpositions
         * @param positionFunction is a simple JS function what is passed the mesh `positions` array. It doesn't need to return anything
         * @param computeNormals is a boolean (default true) to enable/disable the mesh normal recomputation after the vertex position update
         * @returns the current mesh
         */
        updateMeshPositions(positionFunction: (data: FloatArray) => void, computeNormals?: boolean): Mesh;
        /**
         * Creates a un-shared specific occurence of the geometry for the mesh.
         * @returns the current mesh
         */
        makeGeometryUnique(): Mesh;
        /**
         * Set the index buffer of this mesh
         * @param indices defines the source data
         * @param totalVertices defines the total number of vertices referenced by this index data (can be null)
         * @param updatable defines if the updated index buffer must be flagged as updatable (default is false)
         * @returns the current mesh
         */
        setIndices(indices: IndicesArray, totalVertices?: Nullable<number>, updatable?: boolean): AbstractMesh;
        /**
         * Update the current index buffer
         * @param indices defines the source data
         * @param offset defines the offset in the index buffer where to store the new data (can be null)
         * @param gpuMemoryOnly defines a boolean indicating that only the GPU memory must be updated leaving the CPU version of the indices unchanged (false by default)
         * @returns the current mesh
         */
        updateIndices(indices: IndicesArray, offset?: number, gpuMemoryOnly?: boolean): AbstractMesh;
        /**
         * Invert the geometry to move from a right handed system to a left handed one.
         * @returns the current mesh
         */
        toLeftHanded(): Mesh;
        /**
         * @internal
         */
        _bind(subMesh: SubMesh, effect: Effect, fillMode: number): Mesh;
        /**
         * @internal
         */
        _draw(subMesh: SubMesh, fillMode: number, instancesCount?: number): Mesh;
        /**
         * Registers for this mesh a javascript function called just before the rendering process
         * @param func defines the function to call before rendering this mesh
         * @returns the current mesh
         */
        registerBeforeRender(func: (mesh: AbstractMesh) => void): Mesh;
        /**
         * Disposes a previously registered javascript function called before the rendering
         * @param func defines the function to remove
         * @returns the current mesh
         */
        unregisterBeforeRender(func: (mesh: AbstractMesh) => void): Mesh;
        /**
         * Registers for this mesh a javascript function called just after the rendering is complete
         * @param func defines the function to call after rendering this mesh
         * @returns the current mesh
         */
        registerAfterRender(func: (mesh: AbstractMesh) => void): Mesh;
        /**
         * Disposes a previously registered javascript function called after the rendering.
         * @param func defines the function to remove
         * @returns the current mesh
         */
        unregisterAfterRender(func: (mesh: AbstractMesh) => void): Mesh;
        /**
         * @internal
         */
        _getInstancesRenderList(subMeshId: number, isReplacementMode?: boolean): _InstancesBatch;
        /**
         * @internal
         */
        _renderWithInstances(subMesh: SubMesh, fillMode: number, batch: _InstancesBatch, effect: Effect, engine: Engine): Mesh;
        /**
         * @internal
         */
        _renderWithThinInstances(subMesh: SubMesh, fillMode: number, effect: Effect, engine: Engine): void;
        /**
         * @internal
         */
        _processInstancedBuffers(visibleInstances: Nullable<InstancedMesh[]>, renderSelf: boolean): void;
        /**
         * @internal
         */
        _processRendering(renderingMesh: AbstractMesh, subMesh: SubMesh, effect: Effect, fillMode: number, batch: _InstancesBatch, hardwareInstancedRendering: boolean, onBeforeDraw: (isInstance: boolean, world: Matrix, effectiveMaterial?: Material) => void, effectiveMaterial?: Material): Mesh;
        /**
         * @internal
         */
        _rebuild(dispose?: boolean): void;
        /** @internal */
        _freeze(): void;
        /** @internal */
        _unFreeze(): void;
        /**
         * Triggers the draw call for the mesh. Usually, you don't need to call this method by your own because the mesh rendering is handled by the scene rendering manager
         * @param subMesh defines the subMesh to render
         * @param enableAlphaMode defines if alpha mode can be changed
         * @param effectiveMeshReplacement defines an optional mesh used to provide info for the rendering
         * @returns the current mesh
         */
        render(subMesh: SubMesh, enableAlphaMode: boolean, effectiveMeshReplacement?: AbstractMesh): Mesh;
        private _onBeforeDraw;
        /**
         *   Renormalize the mesh and patch it up if there are no weights
         *   Similar to normalization by adding the weights compute the reciprocal and multiply all elements, this wil ensure that everything adds to 1.
         *   However in the case of zero weights then we set just a single influence to 1.
         *   We check in the function for extra's present and if so we use the normalizeSkinWeightsWithExtras rather than the FourWeights version.
         */
        cleanMatrixWeights(): void;
        private _normalizeSkinFourWeights;
        private _normalizeSkinWeightsAndExtra;
        /**
         * ValidateSkinning is used to determine that a mesh has valid skinning data along with skin metrics, if missing weights,
         * or not normalized it is returned as invalid mesh the string can be used for console logs, or on screen messages to let
         * the user know there was an issue with importing the mesh
         * @returns a validation object with skinned, valid and report string
         */
        validateSkinning(): {
            skinned: boolean;
            valid: boolean;
            report: string;
        };
        /** @internal */
        _checkDelayState(): Mesh;
        private _queueLoad;
        /**
         * Returns `true` if the mesh is within the frustum defined by the passed array of planes.
         * A mesh is in the frustum if its bounding box intersects the frustum
         * @param frustumPlanes defines the frustum to test
         * @returns true if the mesh is in the frustum planes
         */
        isInFrustum(frustumPlanes: Plane[]): boolean;
        /**
         * Sets the mesh material by the material or multiMaterial `id` property
         * @param id is a string identifying the material or the multiMaterial
         * @returns the current mesh
         */
        setMaterialById(id: string): Mesh;
        /**
         * Returns as a new array populated with the mesh material and/or skeleton, if any.
         * @returns an array of IAnimatable
         */
        getAnimatables(): IAnimatable[];
        /**
         * Modifies the mesh geometry according to the passed transformation matrix.
         * This method returns nothing, but it really modifies the mesh even if it's originally not set as updatable.
         * The mesh normals are modified using the same transformation.
         * Note that, under the hood, this method sets a new VertexBuffer each call.
         * @param transform defines the transform matrix to use
         * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/transforms/center_origin/bakingTransforms
         * @returns the current mesh
         */
        bakeTransformIntoVertices(transform: Matrix): Mesh;
        /**
         * Modifies the mesh geometry according to its own current World Matrix.
         * The mesh World Matrix is then reset.
         * This method returns nothing but really modifies the mesh even if it's originally not set as updatable.
         * Note that, under the hood, this method sets a new VertexBuffer each call.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/transforms/center_origin/bakingTransforms
         * @param bakeIndependentlyOfChildren indicates whether to preserve all child nodes' World Matrix during baking
         * @returns the current mesh
         */
        bakeCurrentTransformIntoVertices(bakeIndependentlyOfChildren?: boolean): Mesh;
        /** @internal */
        get _positions(): Nullable<Vector3[]>;
        /** @internal */
        _resetPointsArrayCache(): Mesh;
        /** @internal */
        _generatePointsArray(): boolean;
        /**
         * Returns a new Mesh object generated from the current mesh properties.
         * This method must not get confused with createInstance()
         * @param name is a string, the name given to the new mesh
         * @param newParent can be any Node object (default `null`)
         * @param doNotCloneChildren allows/denies the recursive cloning of the original mesh children if any (default `false`)
         * @param clonePhysicsImpostor allows/denies the cloning in the same time of the original mesh `body` used by the physics engine, if any (default `true`)
         * @returns a new mesh
         */
        clone(name?: string, newParent?: Nullable<Node>, doNotCloneChildren?: boolean, clonePhysicsImpostor?: boolean): Mesh;
        /**
         * Releases resources associated with this mesh.
         * @param doNotRecurse Set to true to not recurse into each children (recurse into each children by default)
         * @param disposeMaterialAndTextures Set to true to also dispose referenced materials and textures (false by default)
         */
        dispose(doNotRecurse?: boolean, disposeMaterialAndTextures?: boolean): void;
        /** @internal */
        _disposeInstanceSpecificData(): void;
        /** @internal */
        _disposeThinInstanceSpecificData(): void;
        /**
         * Modifies the mesh geometry according to a displacement map.
         * A displacement map is a colored image. Each pixel color value (actually a gradient computed from red, green, blue values) will give the displacement to apply to each mesh vertex.
         * The mesh must be set as updatable. Its internal geometry is directly modified, no new buffer are allocated.
         * @param url is a string, the URL from the image file is to be downloaded.
         * @param minHeight is the lower limit of the displacement.
         * @param maxHeight is the upper limit of the displacement.
         * @param onSuccess is an optional Javascript function to be called just after the mesh is modified. It is passed the modified mesh and must return nothing.
         * @param uvOffset is an optional vector2 used to offset UV.
         * @param uvScale is an optional vector2 used to scale UV.
         * @param forceUpdate defines whether or not to force an update of the generated buffers. This is useful to apply on a deserialized model for instance.
         * @returns the Mesh.
         */
        applyDisplacementMap(url: string, minHeight: number, maxHeight: number, onSuccess?: (mesh: Mesh) => void, uvOffset?: Vector2, uvScale?: Vector2, forceUpdate?: boolean): Mesh;
        /**
         * Modifies the mesh geometry according to a displacementMap buffer.
         * A displacement map is a colored image. Each pixel color value (actually a gradient computed from red, green, blue values) will give the displacement to apply to each mesh vertex.
         * The mesh must be set as updatable. Its internal geometry is directly modified, no new buffer are allocated.
         * @param buffer is a `Uint8Array` buffer containing series of `Uint8` lower than 255, the red, green, blue and alpha values of each successive pixel.
         * @param heightMapWidth is the width of the buffer image.
         * @param heightMapHeight is the height of the buffer image.
         * @param minHeight is the lower limit of the displacement.
         * @param maxHeight is the upper limit of the displacement.
         * @param uvOffset is an optional vector2 used to offset UV.
         * @param uvScale is an optional vector2 used to scale UV.
         * @param forceUpdate defines whether or not to force an update of the generated buffers. This is useful to apply on a deserialized model for instance.
         * @returns the Mesh.
         */
        applyDisplacementMapFromBuffer(buffer: Uint8Array, heightMapWidth: number, heightMapHeight: number, minHeight: number, maxHeight: number, uvOffset?: Vector2, uvScale?: Vector2, forceUpdate?: boolean): Mesh;
        /**
         * Modify the mesh to get a flat shading rendering.
         * This means each mesh facet will then have its own normals. Usually new vertices are added in the mesh geometry to get this result.
         * Warning : the mesh is really modified even if not set originally as updatable and, under the hood, a new VertexBuffer is allocated.
         * @returns current mesh
         */
        convertToFlatShadedMesh(): Mesh;
        /**
         * This method removes all the mesh indices and add new vertices (duplication) in order to unfold facets into buffers.
         * In other words, more vertices, no more indices and a single bigger VBO.
         * The mesh is really modified even if not set originally as updatable. Under the hood, a new VertexBuffer is allocated.
         * @returns current mesh
         */
        convertToUnIndexedMesh(): Mesh;
        /**
         * Inverses facet orientations.
         * Warning : the mesh is really modified even if not set originally as updatable. A new VertexBuffer is created under the hood each call.
         * @param flipNormals will also inverts the normals
         * @returns current mesh
         */
        flipFaces(flipNormals?: boolean): Mesh;
        /**
         * Increase the number of facets and hence vertices in a mesh
         * Vertex normals are interpolated from existing vertex normals
         * Warning : the mesh is really modified even if not set originally as updatable. A new VertexBuffer is created under the hood each call.
         * @param numberPerEdge the number of new vertices to add to each edge of a facet, optional default 1
         */
        increaseVertices(numberPerEdge?: number): void;
        /**
         * Force adjacent facets to share vertices and remove any facets that have all vertices in a line
         * This will undo any application of covertToFlatShadedMesh
         * Warning : the mesh is really modified even if not set originally as updatable. A new VertexBuffer is created under the hood each call.
         */
        forceSharedVertices(): void;
        /**
         * @internal
         */
        static _instancedMeshFactory(name: string, mesh: Mesh): InstancedMesh;
        /**
         * @internal
         */
        static _PhysicsImpostorParser(scene: Scene, physicObject: IPhysicsEnabledObject, jsonObject: any): PhysicsImpostor;
        /**
         * Creates a new InstancedMesh object from the mesh model.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/copies/instances
         * @param name defines the name of the new instance
         * @returns a new InstancedMesh
         */
        createInstance(name: string): InstancedMesh;
        /**
         * Synchronises all the mesh instance submeshes to the current mesh submeshes, if any.
         * After this call, all the mesh instances have the same submeshes than the current mesh.
         * @returns the current mesh
         */
        synchronizeInstances(): Mesh;
        /**
         * Optimization of the mesh's indices, in case a mesh has duplicated vertices.
         * The function will only reorder the indices and will not remove unused vertices to avoid problems with submeshes.
         * This should be used together with the simplification to avoid disappearing triangles.
         * @param successCallback an optional success callback to be called after the optimization finished.
         * @returns the current mesh
         */
        optimizeIndices(successCallback?: (mesh?: Mesh) => void): Mesh;
        /**
         * Serialize current mesh
         * @param serializationObject defines the object which will receive the serialization data
         */
        serialize(serializationObject?: any): any;
        /** @internal */
        _syncGeometryWithMorphTargetManager(): void;
        /**
         * @internal
         */
        static _GroundMeshParser: (parsedMesh: any, scene: Scene) => Mesh;
        /**
         * @internal
         */
        static _GoldbergMeshParser: (parsedMesh: any, scene: Scene) => GoldbergMesh;
        /**
         * @internal
         */
        static _LinesMeshParser: (parsedMesh: any, scene: Scene) => Mesh;
        /**
         * Returns a new Mesh object parsed from the source provided.
         * @param parsedMesh is the source
         * @param scene defines the hosting scene
         * @param rootUrl is the root URL to prefix the `delayLoadingFile` property with
         * @returns a new Mesh
         */
        static Parse(parsedMesh: any, scene: Scene, rootUrl: string): Mesh;
        /**
         * Prepare internal position array for software CPU skinning
         * @returns original positions used for CPU skinning. Useful for integrating Morphing with skeletons in same mesh
         */
        setPositionsForCPUSkinning(): Nullable<Float32Array>;
        /**
         * Prepare internal normal array for software CPU skinning
         * @returns original normals used for CPU skinning. Useful for integrating Morphing with skeletons in same mesh.
         */
        setNormalsForCPUSkinning(): Nullable<Float32Array>;
        /**
         * Updates the vertex buffer by applying transformation from the bones
         * @param skeleton defines the skeleton to apply to current mesh
         * @returns the current mesh
         */
        applySkeleton(skeleton: Skeleton): Mesh;
        /**
         * Returns an object containing a min and max Vector3 which are the minimum and maximum vectors of each mesh bounding box from the passed array, in the world coordinates
         * @param meshes defines the list of meshes to scan
         * @returns an object `{min:` Vector3`, max:` Vector3`}`
         */
        static MinMax(meshes: AbstractMesh[]): {
            min: Vector3;
            max: Vector3;
        };
        /**
         * Returns the center of the `{min:` Vector3`, max:` Vector3`}` or the center of MinMax vector3 computed from a mesh array
         * @param meshesOrMinMaxVector could be an array of meshes or a `{min:` Vector3`, max:` Vector3`}` object
         * @returns a vector3
         */
        static Center(meshesOrMinMaxVector: {
            min: Vector3;
            max: Vector3;
        } | AbstractMesh[]): Vector3;
        /**
         * Merge the array of meshes into a single mesh for performance reasons.
         * @param meshes array of meshes with the vertices to merge. Entries cannot be empty meshes.
         * @param disposeSource when true (default), dispose of the vertices from the source meshes.
         * @param allow32BitsIndices when the sum of the vertices > 64k, this must be set to true.
         * @param meshSubclass (optional) can be set to a Mesh where the merged vertices will be inserted.
         * @param subdivideWithSubMeshes when true (false default), subdivide mesh into subMeshes.
         * @param multiMultiMaterials when true (false default), subdivide mesh into subMeshes with multiple materials, ignores subdivideWithSubMeshes.
         * @returns a new mesh
         */
        static MergeMeshes(meshes: Array<Mesh>, disposeSource?: boolean, allow32BitsIndices?: boolean, meshSubclass?: Mesh, subdivideWithSubMeshes?: boolean, multiMultiMaterials?: boolean): Nullable<Mesh>;
        /**
         * Merge the array of meshes into a single mesh for performance reasons.
         * @param meshes array of meshes with the vertices to merge. Entries cannot be empty meshes.
         * @param disposeSource when true (default), dispose of the vertices from the source meshes.
         * @param allow32BitsIndices when the sum of the vertices > 64k, this must be set to true.
         * @param meshSubclass (optional) can be set to a Mesh where the merged vertices will be inserted.
         * @param subdivideWithSubMeshes when true (false default), subdivide mesh into subMeshes.
         * @param multiMultiMaterials when true (false default), subdivide mesh into subMeshes with multiple materials, ignores subdivideWithSubMeshes.
         * @returns a new mesh
         */
        static MergeMeshesAsync(meshes: Array<Mesh>, disposeSource?: boolean, allow32BitsIndices?: boolean, meshSubclass?: Mesh, subdivideWithSubMeshes?: boolean, multiMultiMaterials?: boolean): Promise<any>;
        private static _MergeMeshesCoroutine;
        /**
         * @internal
         */
        addInstance(instance: InstancedMesh): void;
        /**
         * @internal
         */
        removeInstance(instance: InstancedMesh): void;
        /** @internal */
        _shouldConvertRHS(): boolean;
    }
        interface Mesh {
            /**
             * Sets the mesh material by the material or multiMaterial `id` property
             * @param id is a string identifying the material or the multiMaterial
             * @returns the current mesh
             * @deprecated Please use MeshBuilder instead Please use setMaterialById instead
             */
            setMaterialByID(id: string): Mesh;
        }
        export namespace Mesh {
            /**
             * Creates a ribbon mesh.
             * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/param
             * @param name defines the name of the mesh to create
             * @param pathArray is a required array of paths, what are each an array of successive Vector3. The pathArray parameter depicts the ribbon geometry.
             * @param closeArray creates a seam between the first and the last paths of the path array (default is false)
             * @param closePath creates a seam between the first and the last points of each path of the path array
             * @param offset is taken in account only if the `pathArray` is containing a single path
             * @param scene defines the hosting scene
             * @param updatable defines if the mesh must be flagged as updatable
             * @param sideOrientation defines the mesh side orientation (https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#side-orientation)
             * @param instance defines an instance of an existing Ribbon object to be updated with the passed `pathArray` parameter (https://doc.babylonjs.com/how_to/How_to_dynamically_morph_a_mesh#ribbon)
             * @returns a new Mesh
             * @deprecated Please use MeshBuilder instead
             */
            function CreateRibbon(name: string, pathArray: Vector3[][], closeArray: boolean, closePath: boolean, offset: number, scene?: Scene, updatable?: boolean, sideOrientation?: number, instance?: Mesh): Mesh;
            /**
             * Creates a plane polygonal mesh.  By default, this is a disc.
             * @param name defines the name of the mesh to create
             * @param radius sets the radius size (float) of the polygon (default 0.5)
             * @param tessellation sets the number of polygon sides (positive integer, default 64). So a tessellation valued to 3 will build a triangle, to 4 a square, etc
             * @param scene defines the hosting scene
             * @param updatable defines if the mesh must be flagged as updatable
             * @param sideOrientation defines the mesh side orientation (https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#side-orientation)
             * @returns a new Mesh
             * @deprecated Please use MeshBuilder instead
             */
            function CreateDisc(name: string, radius: number, tessellation: number, scene: Nullable<Scene>, updatable?: boolean, sideOrientation?: number): Mesh;
            /**
             * Creates a box mesh.
             * @param name defines the name of the mesh to create
             * @param size sets the size (float) of each box side (default 1)
             * @param scene defines the hosting scene
             * @param updatable defines if the mesh must be flagged as updatable
             * @param sideOrientation defines the mesh side orientation (https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#side-orientation)
             * @returns a new Mesh
             * @deprecated Please use MeshBuilder instead
             */
            function CreateBox(name: string, size: number, scene: Nullable<Scene>, updatable?: boolean, sideOrientation?: number): Mesh;
            /**
             * Creates a sphere mesh.
             * @param name defines the name of the mesh to create
             * @param segments sets the sphere number of horizontal stripes (positive integer, default 32)
             * @param diameter sets the diameter size (float) of the sphere (default 1)
             * @param scene defines the hosting scene
             * @param updatable defines if the mesh must be flagged as updatable
             * @param sideOrientation defines the mesh side orientation (https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#side-orientation)
             * @returns a new Mesh
             * @deprecated Please use MeshBuilder instead
             */
            function CreateSphere(name: string, segments: number, diameter: number, scene?: Scene, updatable?: boolean, sideOrientation?: number): Mesh;
            /**
             * Creates a hemisphere mesh.
             * @param name defines the name of the mesh to create
             * @param segments sets the sphere number of horizontal stripes (positive integer, default 32)
             * @param diameter sets the diameter size (float) of the sphere (default 1)
             * @param scene defines the hosting scene
             * @returns a new Mesh
             * @deprecated Please use MeshBuilder instead
             */
            function CreateHemisphere(name: string, segments: number, diameter: number, scene?: Scene): Mesh;
            /**
             * Creates a cylinder or a cone mesh.
             * @param name defines the name of the mesh to create
             * @param height sets the height size (float) of the cylinder/cone (float, default 2)
             * @param diameterTop set the top cap diameter (floats, default 1)
             * @param diameterBottom set the bottom cap diameter (floats, default 1). This value can't be zero
             * @param tessellation sets the number of cylinder sides (positive integer, default 24). Set it to 3 to get a prism for instance
             * @param subdivisions sets the number of rings along the cylinder height (positive integer, default 1)
             * @param scene defines the hosting scene
             * @param updatable defines if the mesh must be flagged as updatable
             * @param sideOrientation defines the mesh side orientation (https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#side-orientation)
             * @returns a new Mesh
             * @deprecated Please use MeshBuilder instead
             */
            function CreateCylinder(name: string, height: number, diameterTop: number, diameterBottom: number, tessellation: number, subdivisions: any, scene?: Scene, updatable?: any, sideOrientation?: number): Mesh;
            /**
             * Creates a torus mesh.
             * @param name defines the name of the mesh to create
             * @param diameter sets the diameter size (float) of the torus (default 1)
             * @param thickness sets the diameter size of the tube of the torus (float, default 0.5)
             * @param tessellation sets the number of torus sides (positive integer, default 16)
             * @param scene defines the hosting scene
             * @param updatable defines if the mesh must be flagged as updatable
             * @param sideOrientation defines the mesh side orientation (https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#side-orientation)
             * @returns a new Mesh
             * @deprecated Please use MeshBuilder instead
             */
            function CreateTorus(name: string, diameter: number, thickness: number, tessellation: number, scene?: Scene, updatable?: boolean, sideOrientation?: number): Mesh;
            /**
             * Creates a torus knot mesh.
             * @param name defines the name of the mesh to create
             * @param radius sets the global radius size (float) of the torus knot (default 2)
             * @param tube sets the diameter size of the tube of the torus (float, default 0.5)
             * @param radialSegments sets the number of sides on each tube segments (positive integer, default 32)
             * @param tubularSegments sets the number of tubes to decompose the knot into (positive integer, default 32)
             * @param p the number of windings on X axis (positive integers, default 2)
             * @param q the number of windings on Y axis (positive integers, default 3)
             * @param scene defines the hosting scene
             * @param updatable defines if the mesh must be flagged as updatable
             * @param sideOrientation defines the mesh side orientation (https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#side-orientation)
             * @returns a new Mesh
             * @deprecated Please use MeshBuilder instead
             */
            function CreateTorusKnot(name: string, radius: number, tube: number, radialSegments: number, tubularSegments: number, p: number, q: number, scene?: Scene, updatable?: boolean, sideOrientation?: number): Mesh;
            /**
             * Creates a line mesh..
             * @param name defines the name of the mesh to create
             * @param points is an array successive Vector3
             * @param scene defines the hosting scene
             * @param updatable defines if the mesh must be flagged as updatable
             * @param instance is an instance of an existing LineMesh object to be updated with the passed `points` parameter (https://doc.babylonjs.com/how_to/How_to_dynamically_morph_a_mesh#lines-and-dashedlines).
             * @returns a new Mesh
             * @deprecated Please use MeshBuilder instead
             */
            function CreateLines(name: string, points: Vector3[], scene: Nullable<Scene>, updatable: boolean, instance?: Nullable<LinesMesh>): LinesMesh;
            /**
             * Creates a dashed line mesh.
             * @param name defines the name of the mesh to create
             * @param points is an array successive Vector3
             * @param dashSize is the size of the dashes relatively the dash number (positive float, default 3)
             * @param gapSize is the size of the gap between two successive dashes relatively the dash number (positive float, default 1)
             * @param dashNb is the intended total number of dashes (positive integer, default 200)
             * @param scene defines the hosting scene
             * @param updatable defines if the mesh must be flagged as updatable
             * @param instance is an instance of an existing LineMesh object to be updated with the passed `points` parameter (https://doc.babylonjs.com/how_to/How_to_dynamically_morph_a_mesh#lines-and-dashedlines)
             * @returns a new Mesh
             * @deprecated Please use MeshBuilder instead
             */
            function CreateDashedLines(name: string, points: Vector3[], dashSize: number, gapSize: number, dashNb: number, scene: Nullable<Scene>, updatable?: boolean, instance?: LinesMesh): LinesMesh;
            /**
             * Creates a polygon mesh.Please consider using the same method from the MeshBuilder class instead
             * The polygon's shape will depend on the input parameters and is constructed parallel to a ground mesh.
             * The parameter `shape` is a required array of successive Vector3 representing the corners of the polygon in th XoZ plane, that is y = 0 for all vectors.
             * You can set the mesh side orientation with the values : Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
             * The mesh can be set to updatable with the boolean parameter `updatable` (default false) if its internal geometry is supposed to change once created.
             * Remember you can only change the shape positions, not their number when updating a polygon.
             * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/param#non-regular-polygon
             * @param name defines the name of the mesh to create
             * @param shape is a required array of successive Vector3 representing the corners of the polygon in th XoZ plane, that is y = 0 for all vectors
             * @param scene defines the hosting scene
             * @param holes is a required array of arrays of successive Vector3 used to defines holes in the polygon
             * @param updatable defines if the mesh must be flagged as updatable
             * @param sideOrientation defines the mesh side orientation (https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#side-orientation)
             * @param earcutInjection can be used to inject your own earcut reference
             * @returns a new Mesh
             * @deprecated Please use MeshBuilder instead
             */
            function CreatePolygon(name: string, shape: Vector3[], scene: Scene, holes?: Vector3[][], updatable?: boolean, sideOrientation?: number, earcutInjection?: any): Mesh;
            /**
             * Creates an extruded polygon mesh, with depth in the Y direction..
             * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/param#extruded-non-regular-polygon
             * @param name defines the name of the mesh to create
             * @param shape is a required array of successive Vector3 representing the corners of the polygon in th XoZ plane, that is y = 0 for all vectors
             * @param depth defines the height of extrusion
             * @param scene defines the hosting scene
             * @param holes is a required array of arrays of successive Vector3 used to defines holes in the polygon
             * @param updatable defines if the mesh must be flagged as updatable
             * @param sideOrientation defines the mesh side orientation (https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#side-orientation)
             * @param earcutInjection can be used to inject your own earcut reference
             * @returns a new Mesh
             * @deprecated Please use MeshBuilder instead
             */
            function ExtrudePolygon(name: string, shape: Vector3[], depth: number, scene: Scene, holes?: Vector3[][], updatable?: boolean, sideOrientation?: number, earcutInjection?: any): Mesh;
            /**
             * Creates an extruded shape mesh.
             * The extrusion is a parametric shape. It has no predefined shape. Its final shape will depend on the input parameters.
             * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/param
             * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/param#extruded-shapes
             * @param name defines the name of the mesh to create
             * @param shape is a required array of successive Vector3. This array depicts the shape to be extruded in its local space : the shape must be designed in the xOy plane and will be extruded along the Z axis
             * @param path is a required array of successive Vector3. This is the axis curve the shape is extruded along
             * @param scale is the value to scale the shape
             * @param rotation is the angle value to rotate the shape each step (each path point), from the former step (so rotation added each step) along the curve
             * @param cap sets the way the extruded shape is capped. Possible values : Mesh.NO_CAP (default), Mesh.CAP_START, Mesh.CAP_END, Mesh.CAP_ALL
             * @param scene defines the hosting scene
             * @param updatable defines if the mesh must be flagged as updatable
             * @param sideOrientation defines the mesh side orientation (https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#side-orientation)
             * @param instance is an instance of an existing ExtrudedShape object to be updated with the passed `shape`, `path`, `scale` or `rotation` parameters (https://doc.babylonjs.com/how_to/How_to_dynamically_morph_a_mesh#extruded-shape)
             * @returns a new Mesh
             * @deprecated Please use MeshBuilder instead
             */
            function ExtrudeShape(name: string, shape: Vector3[], path: Vector3[], scale: number, rotation: number, cap: number, scene: Nullable<Scene>, updatable?: boolean, sideOrientation?: number, instance?: Mesh): Mesh;
            /**
             * Creates an custom extruded shape mesh.
             * The custom extrusion is a parametric shape.
             * It has no predefined shape. Its final shape will depend on the input parameters.
             *
             * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/param#extruded-shapes
             * @param name defines the name of the mesh to create
             * @param shape is a required array of successive Vector3. This array depicts the shape to be extruded in its local space : the shape must be designed in the xOy plane and will be extruded along the Z axis
             * @param path is a required array of successive Vector3. This is the axis curve the shape is extruded along
             * @param scaleFunction is a custom Javascript function called on each path point
             * @param rotationFunction is a custom Javascript function called on each path point
             * @param ribbonCloseArray forces the extrusion underlying ribbon to close all the paths in its `pathArray`
             * @param ribbonClosePath forces the extrusion underlying ribbon to close its `pathArray`
             * @param cap sets the way the extruded shape is capped. Possible values : Mesh.NO_CAP (default), Mesh.CAP_START, Mesh.CAP_END, Mesh.CAP_ALL
             * @param scene defines the hosting scene
             * @param updatable defines if the mesh must be flagged as updatable
             * @param sideOrientation defines the mesh side orientation (https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#side-orientation)
             * @param instance is an instance of an existing ExtrudedShape object to be updated with the passed `shape`, `path`, `scale` or `rotation` parameters (https://doc.babylonjs.com/features/featuresDeepDive/mesh/dynamicMeshMorph#extruded-shape)
             * @returns a new Mesh
             * @deprecated Please use MeshBuilder instead
             */
            function ExtrudeShapeCustom(name: string, shape: Vector3[], path: Vector3[], scaleFunction: Nullable<{
                (i: number, distance: number): number;
            }>, rotationFunction: Nullable<{
                (i: number, distance: number): number;
            }>, ribbonCloseArray: boolean, ribbonClosePath: boolean, cap: number, scene: Scene, updatable?: boolean, sideOrientation?: number, instance?: Mesh): Mesh;
            /**
             * Creates lathe mesh.
             * The lathe is a shape with a symmetry axis : a 2D model shape is rotated around this axis to design the lathe.
             * @param name defines the name of the mesh to create
             * @param shape is a required array of successive Vector3. This array depicts the shape to be rotated in its local space : the shape must be designed in the xOy plane and will be rotated around the Y axis. It's usually a 2D shape, so the Vector3 z coordinates are often set to zero
             * @param radius is the radius value of the lathe
             * @param tessellation is the side number of the lathe.
             * @param scene defines the hosting scene
             * @param updatable defines if the mesh must be flagged as updatable
             * @param sideOrientation defines the mesh side orientation (https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#side-orientation)
             * @returns a new Mesh
             * @deprecated Please use MeshBuilder instead
             */
            function CreateLathe(name: string, shape: Vector3[], radius: number, tessellation: number, scene: Scene, updatable?: boolean, sideOrientation?: number): Mesh;
            /**
             * Creates a plane mesh.
             * @param name defines the name of the mesh to create
             * @param size sets the size (float) of both sides of the plane at once (default 1)
             * @param scene defines the hosting scene
             * @param updatable defines if the mesh must be flagged as updatable
             * @param sideOrientation defines the mesh side orientation (https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#side-orientation)
             * @returns a new Mesh
             * @deprecated Please use MeshBuilder instead
             */
            function CreatePlane(name: string, size: number, scene: Scene, updatable?: boolean, sideOrientation?: number): Mesh;
            /**
             * Creates a ground mesh.
             * @param name defines the name of the mesh to create
             * @param width set the width of the ground
             * @param height set the height of the ground
             * @param subdivisions sets the number of subdivisions per side
             * @param scene defines the hosting scene
             * @param updatable defines if the mesh must be flagged as updatable
             * @returns a new Mesh
             * @deprecated Please use MeshBuilder instead
             */
            function CreateGround(name: string, width: number, height: number, subdivisions: number, scene?: Scene, updatable?: boolean): Mesh;
            /**
             * Creates a tiled ground mesh.
             * @param name defines the name of the mesh to create
             * @param xmin set the ground minimum X coordinate
             * @param zmin set the ground minimum Y coordinate
             * @param xmax set the ground maximum X coordinate
             * @param zmax set the ground maximum Z coordinate
             * @param subdivisions is an object `{w: positive integer, h: positive integer}` (default `{w: 6, h: 6}`). `w` and `h` are the numbers of subdivisions on the ground width and height. Each subdivision is called a tile
             * @param precision is an object `{w: positive integer, h: positive integer}` (default `{w: 2, h: 2}`). `w` and `h` are the numbers of subdivisions on the ground width and height of each tile
             * @param scene defines the hosting scene
             * @param updatable defines if the mesh must be flagged as updatable
             * @returns a new Mesh
             * @deprecated Please use MeshBuilder instead
             */
            function CreateTiledGround(name: string, xmin: number, zmin: number, xmax: number, zmax: number, subdivisions: {
                w: number;
                h: number;
            }, precision: {
                w: number;
                h: number;
            }, scene: Scene, updatable?: boolean): Mesh;
            /**
             * Creates a ground mesh from a height map.
             * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set/height_map
             * @param name defines the name of the mesh to create
             * @param url sets the URL of the height map image resource
             * @param width set the ground width size
             * @param height set the ground height size
             * @param subdivisions sets the number of subdivision per side
             * @param minHeight is the minimum altitude on the ground
             * @param maxHeight is the maximum altitude on the ground
             * @param scene defines the hosting scene
             * @param updatable defines if the mesh must be flagged as updatable
             * @param onReady  is a callback function that will be called  once the mesh is built (the height map download can last some time)
             * @param alphaFilter will filter any data where the alpha channel is below this value, defaults 0 (all data visible)
             * @returns a new Mesh
             * @deprecated Please use MeshBuilder instead
             */
            function CreateGroundFromHeightMap(name: string, url: string, width: number, height: number, subdivisions: number, minHeight: number, maxHeight: number, scene: Scene, updatable?: boolean, onReady?: (mesh: GroundMesh) => void, alphaFilter?: number): GroundMesh;
            /**
             * Creates a tube mesh.
             * The tube is a parametric shape.
             * It has no predefined shape. Its final shape will depend on the input parameters.
             *
             * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/param
             * @param name defines the name of the mesh to create
             * @param path is a required array of successive Vector3. It is the curve used as the axis of the tube
             * @param radius sets the tube radius size
             * @param tessellation is the number of sides on the tubular surface
             * @param radiusFunction is a custom function. If it is not null, it overrides the parameter `radius`. This function is called on each point of the tube path and is passed the index `i` of the i-th point and the distance of this point from the first point of the path
             * @param cap sets the way the extruded shape is capped. Possible values : Mesh.NO_CAP (default), Mesh.CAP_START, Mesh.CAP_END, Mesh.CAP_ALL
             * @param scene defines the hosting scene
             * @param updatable defines if the mesh must be flagged as updatable
             * @param sideOrientation defines the mesh side orientation (https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#side-orientation)
             * @param instance is an instance of an existing Tube object to be updated with the passed `pathArray` parameter (https://doc.babylonjs.com/how_to/How_to_dynamically_morph_a_mesh#tube)
             * @returns a new Mesh
             * @deprecated Please use MeshBuilder instead
             */
            function CreateTube(name: string, path: Vector3[], radius: number, tessellation: number, radiusFunction: {
                (i: number, distance: number): number;
            }, cap: number, scene: Scene, updatable?: boolean, sideOrientation?: number, instance?: Mesh): Mesh;
            /**
             * Creates a polyhedron mesh.
             *.
             * * The parameter `type` (positive integer, max 14, default 0) sets the polyhedron type to build among the 15 embedded types. Please refer to the type sheet in the tutorial to choose the wanted type
             * * The parameter `size` (positive float, default 1) sets the polygon size
             * * You can overwrite the `size` on each dimension bu using the parameters `sizeX`, `sizeY` or `sizeZ` (positive floats, default to `size` value)
             * * You can build other polyhedron types than the 15 embbeded ones by setting the parameter `custom` (`polyhedronObject`, default null). If you set the parameter `custom`, this overwrittes the parameter `type`
             * * A `polyhedronObject` is a formatted javascript object. You'll find a full file with pre-set polyhedra here : https://github.com/BabylonJS/Extensions/tree/master/Polyhedron
             * * You can set the color and the UV of each side of the polyhedron with the parameters `faceColors` (Color4, default `(1, 1, 1, 1)`) and faceUV (Vector4, default `(0, 0, 1, 1)`)
             * * To understand how to set `faceUV` or `faceColors`, please read this by considering the right number of faces of your polyhedron, instead of only 6 for the box : https://doc.babylonjs.com/features/featuresDeepDive/materials/using/texturePerBoxFace
             * * The parameter `flat` (boolean, default true). If set to false, it gives the polyhedron a single global face, so less vertices and shared normals. In this case, `faceColors` and `faceUV` are ignored
             * * You can also set the mesh side orientation with the values : Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
             * * If you create a double-sided mesh, you can choose what parts of the texture image to crop and stick respectively on the front and the back sides with the parameters `frontUVs` and `backUVs` (Vector4). Detail here : https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#side-orientation
             * * The mesh can be set to updatable with the boolean parameter `updatable` (default false) if its internal geometry is supposed to change once created
             * @param name defines the name of the mesh to create
             * @param options defines the options used to create the mesh
             * @param scene defines the hosting scene
             * @returns a new Mesh
             * @deprecated Please use MeshBuilder instead
             */
            function CreatePolyhedron(name: string, options: {
                type?: number;
                size?: number;
                sizeX?: number;
                sizeY?: number;
                sizeZ?: number;
                custom?: any;
                faceUV?: Vector4[];
                faceColors?: Color4[];
                updatable?: boolean;
                sideOrientation?: number;
            }, scene: Scene): Mesh;
            /**
             * Creates a sphere based upon an icosahedron with 20 triangular faces which can be subdivided
             * * The parameter `radius` sets the radius size (float) of the icosphere (default 1)
             * * You can set some different icosphere dimensions, for instance to build an ellipsoid, by using the parameters `radiusX`, `radiusY` and `radiusZ` (all by default have the same value than `radius`)
             * * The parameter `subdivisions` sets the number of subdivisions (positive integer, default 4). The more subdivisions, the more faces on the icosphere whatever its size
             * * The parameter `flat` (boolean, default true) gives each side its own normals. Set it to false to get a smooth continuous light reflection on the surface
             * * You can also set the mesh side orientation with the values : Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
             * * If you create a double-sided mesh, you can choose what parts of the texture image to crop and stick respectively on the front and the back sides with the parameters `frontUVs` and `backUVs` (Vector4). Detail here : https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/set#side-orientation
             * * The mesh can be set to updatable with the boolean parameter `updatable` (default false) if its internal geometry is supposed to change once created
             * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/polyhedra#icosphere
             * @param name defines the name of the mesh
             * @param options defines the options used to create the mesh
             * @param scene defines the hosting scene
             * @returns a new Mesh
             * @deprecated Please use MeshBuilder instead
             */
            function CreateIcoSphere(name: string, options: {
                radius?: number;
                flat?: boolean;
                subdivisions?: number;
                sideOrientation?: number;
                updatable?: boolean;
            }, scene: Scene): Mesh;
            /**
             * Creates a decal mesh.
             *.
             * A decal is a mesh usually applied as a model onto the surface of another mesh
             * @param name  defines the name of the mesh
             * @param sourceMesh defines the mesh receiving the decal
             * @param position sets the position of the decal in world coordinates
             * @param normal sets the normal of the mesh where the decal is applied onto in world coordinates
             * @param size sets the decal scaling
             * @param angle sets the angle to rotate the decal
             * @returns a new Mesh
             * @deprecated Please use MeshBuilder instead
             */
            function CreateDecal(name: string, sourceMesh: AbstractMesh, position: Vector3, normal: Vector3, size: Vector3, angle: number): Mesh;
            /** Creates a Capsule Mesh
             * @param name defines the name of the mesh.
             * @param options the constructors options used to shape the mesh.
             * @param scene defines the scene the mesh is scoped to.
             * @returns the capsule mesh
             * @see https://doc.babylonjs.com/how_to/capsule_shape
             * @deprecated Please use MeshBuilder instead
             */
            function CreateCapsule(name: string, options: ICreateCapsuleOptions, scene: Scene): Mesh;
            /**
             * Extends a mesh to a Goldberg mesh
             * Warning  the mesh to convert MUST be an import of a perviously exported Goldberg mesh
             * @param mesh the mesh to convert
             * @returns the extended mesh
             * @deprecated Please use ExtendMeshToGoldberg instead
             */
            function ExtendToGoldberg(mesh: Mesh): Mesh;
        }


    /**
     * Define an interface for all classes that will get and set the data on vertices
     */
    export interface IGetSetVerticesData {
        /**
         * Gets a boolean indicating if specific vertex data is present
         * @param kind defines the vertex data kind to use
         * @returns true is data kind is present
         */
        isVerticesDataPresent(kind: string): boolean;
        /**
         * Gets a specific vertex data attached to this geometry. Float data is constructed if the vertex buffer data cannot be returned directly.
         * @param kind defines the data kind (Position, normal, etc...)
         * @param copyWhenShared defines if the returned array must be cloned upon returning it if the current geometry is shared between multiple meshes
         * @param forceCopy defines a boolean indicating that the returned array must be cloned upon returning it
         * @returns a float array containing vertex data
         */
        getVerticesData(kind: string, copyWhenShared?: boolean, forceCopy?: boolean): Nullable<FloatArray>;
        /**
         * Returns an array of integers or a typed array (Int32Array, Uint32Array, Uint16Array) populated with the mesh indices.
         * @param copyWhenShared If true (default false) and and if the mesh geometry is shared among some other meshes, the returned array is a copy of the internal one.
         * @param forceCopy defines a boolean indicating that the returned array must be cloned upon returning it
         * @returns the indices array or an empty array if the mesh has no geometry
         */
        getIndices(copyWhenShared?: boolean, forceCopy?: boolean): Nullable<IndicesArray>;
        /**
         * Set specific vertex data
         * @param kind defines the data kind (Position, normal, etc...)
         * @param data defines the vertex data to use
         * @param updatable defines if the vertex must be flagged as updatable (false as default)
         * @param stride defines the stride to use (0 by default). This value is deduced from the kind value if not specified
         */
        setVerticesData(kind: string, data: FloatArray, updatable: boolean): void;
        /**
         * Update a specific associated vertex buffer
         * @param kind defines which buffer to write to (positions, indices, normals, etc). Possible `kind` values :
         * - VertexBuffer.PositionKind
         * - VertexBuffer.UVKind
         * - VertexBuffer.UV2Kind
         * - VertexBuffer.UV3Kind
         * - VertexBuffer.UV4Kind
         * - VertexBuffer.UV5Kind
         * - VertexBuffer.UV6Kind
         * - VertexBuffer.ColorKind
         * - VertexBuffer.MatricesIndicesKind
         * - VertexBuffer.MatricesIndicesExtraKind
         * - VertexBuffer.MatricesWeightsKind
         * - VertexBuffer.MatricesWeightsExtraKind
         * @param data defines the data source
         * @param updateExtends defines if extends info of the mesh must be updated (can be null). This is mostly useful for "position" kind
         * @param makeItUnique defines if the geometry associated with the mesh must be cloned to make the change only for this mesh (and not all meshes associated with the same geometry)
         */
        updateVerticesData(kind: string, data: FloatArray, updateExtends?: boolean, makeItUnique?: boolean): void;
        /**
         * Creates a new index buffer
         * @param indices defines the indices to store in the index buffer
         * @param totalVertices defines the total number of vertices (could be null)
         * @param updatable defines if the index buffer must be flagged as updatable (false by default)
         */
        setIndices(indices: IndicesArray, totalVertices: Nullable<number>, updatable?: boolean): void;
    }
    /**
     * This class contains the various kinds of data on every vertex of a mesh used in determining its shape and appearance
     */
    export class VertexData {
        /**
         * Mesh side orientation : usually the external or front surface
         */
        static readonly FRONTSIDE = 0;
        /**
         * Mesh side orientation : usually the internal or back surface
         */
        static readonly BACKSIDE = 1;
        /**
         * Mesh side orientation : both internal and external or front and back surfaces
         */
        static readonly DOUBLESIDE = 2;
        /**
         * Mesh side orientation : by default, `FRONTSIDE`
         */
        static readonly DEFAULTSIDE = 0;
        /**
         * An array of the x, y, z position of each vertex  [...., x, y, z, .....]
         */
        positions: Nullable<FloatArray>;
        /**
         * An array of the x, y, z normal vector of each vertex  [...., x, y, z, .....]
         */
        normals: Nullable<FloatArray>;
        /**
         * An array of the x, y, z tangent vector of each vertex  [...., x, y, z, .....]
         */
        tangents: Nullable<FloatArray>;
        /**
         * An array of u,v which maps a texture image onto each vertex  [...., u, v, .....]
         */
        uvs: Nullable<FloatArray>;
        /**
         * A second array of u,v which maps a texture image onto each vertex  [...., u, v, .....]
         */
        uvs2: Nullable<FloatArray>;
        /**
         * A third array of u,v which maps a texture image onto each vertex  [...., u, v, .....]
         */
        uvs3: Nullable<FloatArray>;
        /**
         * A fourth array of u,v which maps a texture image onto each vertex  [...., u, v, .....]
         */
        uvs4: Nullable<FloatArray>;
        /**
         * A fifth array of u,v which maps a texture image onto each vertex  [...., u, v, .....]
         */
        uvs5: Nullable<FloatArray>;
        /**
         * A sixth array of u,v which maps a texture image onto each vertex  [...., u, v, .....]
         */
        uvs6: Nullable<FloatArray>;
        /**
         * An array of the r, g, b, a, color of each vertex  [...., r, g, b, a, .....]
         */
        colors: Nullable<FloatArray>;
        /**
         * An array containing the list of indices to the array of matrices produced by bones, each vertex have up to 4 indices (8 if the matricesIndicesExtra is set).
         */
        matricesIndices: Nullable<FloatArray>;
        /**
         * An array containing the list of weights defining the weight of each indexed matrix in the final computation
         */
        matricesWeights: Nullable<FloatArray>;
        /**
         * An array extending the number of possible indices
         */
        matricesIndicesExtra: Nullable<FloatArray>;
        /**
         * An array extending the number of possible weights when the number of indices is extended
         */
        matricesWeightsExtra: Nullable<FloatArray>;
        /**
         * An array of i, j, k the three vertex indices required for each triangular facet  [...., i, j, k .....]
         */
        indices: Nullable<IndicesArray>;
        /**
         * Uses the passed data array to set the set the values for the specified kind of data
         * @param data a linear array of floating numbers
         * @param kind the type of data that is being set, eg positions, colors etc
         */
        set(data: FloatArray, kind: string): void;
        /**
         * Associates the vertexData to the passed Mesh.
         * Sets it as updatable or not (default `false`)
         * @param mesh the mesh the vertexData is applied to
         * @param updatable when used and having the value true allows new data to update the vertexData
         * @returns the VertexData
         */
        applyToMesh(mesh: Mesh, updatable?: boolean): VertexData;
        /**
         * Associates the vertexData to the passed Geometry.
         * Sets it as updatable or not (default `false`)
         * @param geometry the geometry the vertexData is applied to
         * @param updatable when used and having the value true allows new data to update the vertexData
         * @returns VertexData
         */
        applyToGeometry(geometry: Geometry, updatable?: boolean): VertexData;
        /**
         * Updates the associated mesh
         * @param mesh the mesh to be updated
         * @returns VertexData
         */
        updateMesh(mesh: Mesh): VertexData;
        /**
         * Updates the associated geometry
         * @param geometry the geometry to be updated
         * @returns VertexData.
         */
        updateGeometry(geometry: Geometry): VertexData;
        private readonly _applyTo;
        /**
         * @internal
         */
        _applyToCoroutine(meshOrGeometry: IGetSetVerticesData, updatable: boolean | undefined, isAsync: boolean): Coroutine<VertexData>;
        private _update;
        private static _TransformVector3Coordinates;
        private static _TransformVector3Normals;
        private static _TransformVector4Normals;
        private static _FlipFaces;
        /**
         * Transforms each position and each normal of the vertexData according to the passed Matrix
         * @param matrix the transforming matrix
         * @returns the VertexData
         */
        transform(matrix: Matrix): VertexData;
        /**
         * Merges the passed VertexData into the current one
         * @param others the VertexData to be merged into the current one
         * @param use32BitsIndices defines a boolean indicating if indices must be store in a 32 bits array
         * @param forceCloneIndices defines a boolean indicating if indices are forced to be cloned
         * @returns the modified VertexData
         */
        merge(others: VertexData | VertexData[], use32BitsIndices?: boolean, forceCloneIndices?: boolean): VertexData;
        /**
         * @internal
         */
        _mergeCoroutine(transform: Matrix | undefined, vertexDatas: (readonly [vertexData: VertexData, transform?: Matrix])[], use32BitsIndices: boolean | undefined, isAsync: boolean, forceCloneIndices: boolean): Coroutine<VertexData>;
        private static _MergeElement;
        private _validate;
        /**
         * Serializes the VertexData
         * @returns a serialized object
         */
        serialize(): any;
        /**
         * Extracts the vertexData from a mesh
         * @param mesh the mesh from which to extract the VertexData
         * @param copyWhenShared defines if the VertexData must be cloned when shared between multiple meshes, optional, default false
         * @param forceCopy indicating that the VertexData must be cloned, optional, default false
         * @returns the object VertexData associated to the passed mesh
         */
        static ExtractFromMesh(mesh: Mesh, copyWhenShared?: boolean, forceCopy?: boolean): VertexData;
        /**
         * Extracts the vertexData from the geometry
         * @param geometry the geometry from which to extract the VertexData
         * @param copyWhenShared defines if the VertexData must be cloned when the geometry is shared between multiple meshes, optional, default false
         * @param forceCopy indicating that the VertexData must be cloned, optional, default false
         * @returns the object VertexData associated to the passed mesh
         */
        static ExtractFromGeometry(geometry: Geometry, copyWhenShared?: boolean, forceCopy?: boolean): VertexData;
        private static _ExtractFrom;
        /**
         * Creates the VertexData for a Ribbon
         * @param options an object used to set the following optional parameters for the ribbon, required but can be empty
         * * pathArray array of paths, each of which an array of successive Vector3
         * * closeArray creates a seam between the first and the last paths of the pathArray, optional, default false
         * * closePath creates a seam between the first and the last points of each path of the path array, optional, default false
         * * offset a positive integer, only used when pathArray contains a single path (offset = 10 means the point 1 is joined to the point 11), default rounded half size of the pathArray length
         * * sideOrientation optional and takes the values : Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
         * * frontUvs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the front side, optional, default vector4 (0, 0, 1, 1)
         * * backUVs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the back side, optional, default vector4 (0, 0, 1, 1)
         * * invertUV swaps in the U and V coordinates when applying a texture, optional, default false
         * * uvs a linear array, of length 2 * number of vertices, of custom UV values, optional
         * * colors a linear array, of length 4 * number of vertices, of custom color values, optional
         * @param options.pathArray
         * @param options.closeArray
         * @param options.closePath
         * @param options.offset
         * @param options.sideOrientation
         * @param options.frontUVs
         * @param options.backUVs
         * @param options.invertUV
         * @param options.uvs
         * @param options.colors
         * @returns the VertexData of the ribbon
         * @deprecated use CreateRibbonVertexData instead
         */
        static CreateRibbon(options: {
            pathArray: Vector3[][];
            closeArray?: boolean;
            closePath?: boolean;
            offset?: number;
            sideOrientation?: number;
            frontUVs?: Vector4;
            backUVs?: Vector4;
            invertUV?: boolean;
            uvs?: Vector2[];
            colors?: Color4[];
        }): VertexData;
        /**
         * Creates the VertexData for a box
         * @param options an object used to set the following optional parameters for the box, required but can be empty
         * * size sets the width, height and depth of the box to the value of size, optional default 1
         * * width sets the width (x direction) of the box, overwrites the width set by size, optional, default size
         * * height sets the height (y direction) of the box, overwrites the height set by size, optional, default size
         * * depth sets the depth (z direction) of the box, overwrites the depth set by size, optional, default size
         * * faceUV an array of 6 Vector4 elements used to set different images to each box side
         * * faceColors an array of 6 Color3 elements used to set different colors to each box side
         * * sideOrientation optional and takes the values : Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
         * * frontUvs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the front side, optional, default vector4 (0, 0, 1, 1)
         * * backUVs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the back side, optional, default vector4 (0, 0, 1, 1)
         * @param options.size
         * @param options.width
         * @param options.height
         * @param options.depth
         * @param options.faceUV
         * @param options.faceColors
         * @param options.sideOrientation
         * @param options.frontUVs
         * @param options.backUVs
         * @returns the VertexData of the box
         * @deprecated Please use CreateBoxVertexData from the BoxBuilder file instead
         */
        static CreateBox(options: {
            size?: number;
            width?: number;
            height?: number;
            depth?: number;
            faceUV?: Vector4[];
            faceColors?: Color4[];
            sideOrientation?: number;
            frontUVs?: Vector4;
            backUVs?: Vector4;
        }): VertexData;
        /**
         * Creates the VertexData for a tiled box
         * @param options an object used to set the following optional parameters for the box, required but can be empty
         * * faceTiles sets the pattern, tile size and number of tiles for a face
         * * faceUV an array of 6 Vector4 elements used to set different images to each box side
         * * faceColors an array of 6 Color3 elements used to set different colors to each box side
         * * sideOrientation optional and takes the values : Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
         * @param options.pattern
         * @param options.width
         * @param options.height
         * @param options.depth
         * @param options.tileSize
         * @param options.tileWidth
         * @param options.tileHeight
         * @param options.alignHorizontal
         * @param options.alignVertical
         * @param options.faceUV
         * @param options.faceColors
         * @param options.sideOrientation
         * @returns the VertexData of the box
         * @deprecated Please use CreateTiledBoxVertexData instead
         */
        static CreateTiledBox(options: {
            pattern?: number;
            width?: number;
            height?: number;
            depth?: number;
            tileSize?: number;
            tileWidth?: number;
            tileHeight?: number;
            alignHorizontal?: number;
            alignVertical?: number;
            faceUV?: Vector4[];
            faceColors?: Color4[];
            sideOrientation?: number;
        }): VertexData;
        /**
         * Creates the VertexData for a tiled plane
         * @param options an object used to set the following optional parameters for the box, required but can be empty
         * * pattern a limited pattern arrangement depending on the number
         * * tileSize sets the width, height and depth of the tile to the value of size, optional default 1
         * * tileWidth sets the width (x direction) of the tile, overwrites the width set by size, optional, default size
         * * tileHeight sets the height (y direction) of the tile, overwrites the height set by size, optional, default size
         * * sideOrientation optional and takes the values : Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
         * * frontUvs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the front side, optional, default vector4 (0, 0, 1, 1)
         * * backUVs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the back side, optional, default vector4 (0, 0, 1, 1)
         * @param options.pattern
         * @param options.tileSize
         * @param options.tileWidth
         * @param options.tileHeight
         * @param options.size
         * @param options.width
         * @param options.height
         * @param options.alignHorizontal
         * @param options.alignVertical
         * @param options.sideOrientation
         * @param options.frontUVs
         * @param options.backUVs
         * @returns the VertexData of the tiled plane
         * @deprecated use CreateTiledPlaneVertexData instead
         */
        static CreateTiledPlane(options: {
            pattern?: number;
            tileSize?: number;
            tileWidth?: number;
            tileHeight?: number;
            size?: number;
            width?: number;
            height?: number;
            alignHorizontal?: number;
            alignVertical?: number;
            sideOrientation?: number;
            frontUVs?: Vector4;
            backUVs?: Vector4;
        }): VertexData;
        /**
         * Creates the VertexData for an ellipsoid, defaults to a sphere
         * @param options an object used to set the following optional parameters for the box, required but can be empty
         * * segments sets the number of horizontal strips optional, default 32
         * * diameter sets the axes dimensions, diameterX, diameterY and diameterZ to the value of diameter, optional default 1
         * * diameterX sets the diameterX (x direction) of the ellipsoid, overwrites the diameterX set by diameter, optional, default diameter
         * * diameterY sets the diameterY (y direction) of the ellipsoid, overwrites the diameterY set by diameter, optional, default diameter
         * * diameterZ sets the diameterZ (z direction) of the ellipsoid, overwrites the diameterZ set by diameter, optional, default diameter
         * * arc a number from 0 to 1, to create an unclosed ellipsoid based on the fraction of the circumference (latitude) given by the arc value, optional, default 1
         * * slice a number from 0 to 1, to create an unclosed ellipsoid based on the fraction of the height (latitude) given by the arc value, optional, default 1
         * * sideOrientation optional and takes the values : Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
         * * frontUvs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the front side, optional, default vector4 (0, 0, 1, 1)
         * * backUVs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the back side, optional, default vector4 (0, 0, 1, 1)
         * @param options.segments
         * @param options.diameter
         * @param options.diameterX
         * @param options.diameterY
         * @param options.diameterZ
         * @param options.arc
         * @param options.slice
         * @param options.sideOrientation
         * @param options.frontUVs
         * @param options.backUVs
         * @returns the VertexData of the ellipsoid
         * @deprecated use CreateSphereVertexData instead
         */
        static CreateSphere(options: {
            segments?: number;
            diameter?: number;
            diameterX?: number;
            diameterY?: number;
            diameterZ?: number;
            arc?: number;
            slice?: number;
            sideOrientation?: number;
            frontUVs?: Vector4;
            backUVs?: Vector4;
        }): VertexData;
        /**
         * Creates the VertexData for a cylinder, cone or prism
         * @param options an object used to set the following optional parameters for the box, required but can be empty
         * * height sets the height (y direction) of the cylinder, optional, default 2
         * * diameterTop sets the diameter of the top of the cone, overwrites diameter,  optional, default diameter
         * * diameterBottom sets the diameter of the bottom of the cone, overwrites diameter,  optional, default diameter
         * * diameter sets the diameter of the top and bottom of the cone, optional default 1
         * * tessellation the number of prism sides, 3 for a triangular prism, optional, default 24
         * * subdivisions` the number of rings along the cylinder height, optional, default 1
         * * arc a number from 0 to 1, to create an unclosed cylinder based on the fraction of the circumference given by the arc value, optional, default 1
         * * faceColors an array of Color3 elements used to set different colors to the top, rings and bottom respectively
         * * faceUV an array of Vector4 elements used to set different images to the top, rings and bottom respectively
         * * hasRings when true makes each subdivision independently treated as a face for faceUV and faceColors, optional, default false
         * * enclose when true closes an open cylinder by adding extra flat faces between the height axis and vertical edges, think cut cake
         * * sideOrientation optional and takes the values : Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
         * * frontUvs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the front side, optional, default vector4 (0, 0, 1, 1)
         * * backUVs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the back side, optional, default vector4 (0, 0, 1, 1)
         * @param options.height
         * @param options.diameterTop
         * @param options.diameterBottom
         * @param options.diameter
         * @param options.tessellation
         * @param options.subdivisions
         * @param options.arc
         * @param options.faceColors
         * @param options.faceUV
         * @param options.hasRings
         * @param options.enclose
         * @param options.sideOrientation
         * @param options.frontUVs
         * @param options.backUVs
         * @returns the VertexData of the cylinder, cone or prism
         * @deprecated please use CreateCylinderVertexData instead
         */
        static CreateCylinder(options: {
            height?: number;
            diameterTop?: number;
            diameterBottom?: number;
            diameter?: number;
            tessellation?: number;
            subdivisions?: number;
            arc?: number;
            faceColors?: Color4[];
            faceUV?: Vector4[];
            hasRings?: boolean;
            enclose?: boolean;
            sideOrientation?: number;
            frontUVs?: Vector4;
            backUVs?: Vector4;
        }): VertexData;
        /**
         * Creates the VertexData for a torus
         * @param options an object used to set the following optional parameters for the box, required but can be empty
         * * diameter the diameter of the torus, optional default 1
         * * thickness the diameter of the tube forming the torus, optional default 0.5
         * * tessellation the number of prism sides, 3 for a triangular prism, optional, default 24
         * * sideOrientation optional and takes the values : Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
         * * frontUvs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the front side, optional, default vector4 (0, 0, 1, 1)
         * * backUVs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the back side, optional, default vector4 (0, 0, 1, 1)
         * @param options.diameter
         * @param options.thickness
         * @param options.tessellation
         * @param options.sideOrientation
         * @param options.frontUVs
         * @param options.backUVs
         * @returns the VertexData of the torus
         * @deprecated use CreateTorusVertexData instead
         */
        static CreateTorus(options: {
            diameter?: number;
            thickness?: number;
            tessellation?: number;
            sideOrientation?: number;
            frontUVs?: Vector4;
            backUVs?: Vector4;
        }): VertexData;
        /**
         * Creates the VertexData of the LineSystem
         * @param options an object used to set the following optional parameters for the LineSystem, required but can be empty
         *  - lines an array of lines, each line being an array of successive Vector3
         *  - colors an array of line colors, each of the line colors being an array of successive Color4, one per line point
         * @param options.lines
         * @param options.colors
         * @returns the VertexData of the LineSystem
         * @deprecated use CreateLineSystemVertexData instead
         */
        static CreateLineSystem(options: {
            lines: Vector3[][];
            colors?: Nullable<Color4[][]>;
        }): VertexData;
        /**
         * Create the VertexData for a DashedLines
         * @param options an object used to set the following optional parameters for the DashedLines, required but can be empty
         *  - points an array successive Vector3
         *  - dashSize the size of the dashes relative to the dash number, optional, default 3
         *  - gapSize the size of the gap between two successive dashes relative to the dash number, optional, default 1
         *  - dashNb the intended total number of dashes, optional, default 200
         * @param options.points
         * @param options.dashSize
         * @param options.gapSize
         * @param options.dashNb
         * @returns the VertexData for the DashedLines
         * @deprecated use CreateDashedLinesVertexData instead
         */
        static CreateDashedLines(options: {
            points: Vector3[];
            dashSize?: number;
            gapSize?: number;
            dashNb?: number;
        }): VertexData;
        /**
         * Creates the VertexData for a Ground
         * @param options an object used to set the following optional parameters for the Ground, required but can be empty
         *  - width the width (x direction) of the ground, optional, default 1
         *  - height the height (z direction) of the ground, optional, default 1
         *  - subdivisions the number of subdivisions per side, optional, default 1
         * @param options.width
         * @param options.height
         * @param options.subdivisions
         * @param options.subdivisionsX
         * @param options.subdivisionsY
         * @returns the VertexData of the Ground
         * @deprecated Please use CreateGroundVertexData instead
         */
        static CreateGround(options: {
            width?: number;
            height?: number;
            subdivisions?: number;
            subdivisionsX?: number;
            subdivisionsY?: number;
        }): VertexData;
        /**
         * Creates the VertexData for a TiledGround by subdividing the ground into tiles
         * @param options an object used to set the following optional parameters for the Ground, required but can be empty
         * * xmin the ground minimum X coordinate, optional, default -1
         * * zmin the ground minimum Z coordinate, optional, default -1
         * * xmax the ground maximum X coordinate, optional, default 1
         * * zmax the ground maximum Z coordinate, optional, default 1
         * * subdivisions a javascript object {w: positive integer, h: positive integer}, `w` and `h` are the numbers of subdivisions on the ground width and height creating 'tiles', default {w: 6, h: 6}
         * * precision a javascript object {w: positive integer, h: positive integer}, `w` and `h` are the numbers of subdivisions on the tile width and height, default {w: 2, h: 2}
         * @param options.xmin
         * @param options.zmin
         * @param options.xmax
         * @param options.zmax
         * @param options.subdivisions
         * @param options.subdivisions.w
         * @param options.subdivisions.h
         * @param options.precision
         * @param options.precision.w
         * @param options.precision.h
         * @returns the VertexData of the TiledGround
         * @deprecated use CreateTiledGroundVertexData instead
         */
        static CreateTiledGround(options: {
            xmin: number;
            zmin: number;
            xmax: number;
            zmax: number;
            subdivisions?: {
                w: number;
                h: number;
            };
            precision?: {
                w: number;
                h: number;
            };
        }): VertexData;
        /**
         * Creates the VertexData of the Ground designed from a heightmap
         * @param options an object used to set the following parameters for the Ground, required and provided by CreateGroundFromHeightMap
         * * width the width (x direction) of the ground
         * * height the height (z direction) of the ground
         * * subdivisions the number of subdivisions per side
         * * minHeight the minimum altitude on the ground, optional, default 0
         * * maxHeight the maximum altitude on the ground, optional default 1
         * * colorFilter the filter to apply to the image pixel colors to compute the height, optional Color3, default (0.3, 0.59, 0.11)
         * * buffer the array holding the image color data
         * * bufferWidth the width of image
         * * bufferHeight the height of image
         * * alphaFilter Remove any data where the alpha channel is below this value, defaults 0 (all data visible)
         * @param options.width
         * @param options.height
         * @param options.subdivisions
         * @param options.minHeight
         * @param options.maxHeight
         * @param options.colorFilter
         * @param options.buffer
         * @param options.bufferWidth
         * @param options.bufferHeight
         * @param options.alphaFilter
         * @returns the VertexData of the Ground designed from a heightmap
         * @deprecated use CreateGroundFromHeightMapVertexData instead
         */
        static CreateGroundFromHeightMap(options: {
            width: number;
            height: number;
            subdivisions: number;
            minHeight: number;
            maxHeight: number;
            colorFilter: Color3;
            buffer: Uint8Array;
            bufferWidth: number;
            bufferHeight: number;
            alphaFilter: number;
        }): VertexData;
        /**
         * Creates the VertexData for a Plane
         * @param options an object used to set the following optional parameters for the plane, required but can be empty
         * * size sets the width and height of the plane to the value of size, optional default 1
         * * width sets the width (x direction) of the plane, overwrites the width set by size, optional, default size
         * * height sets the height (y direction) of the plane, overwrites the height set by size, optional, default size
         * * sideOrientation optional and takes the values : Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
         * * frontUvs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the front side, optional, default vector4 (0, 0, 1, 1)
         * * backUVs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the back side, optional, default vector4 (0, 0, 1, 1)
         * @param options.size
         * @param options.width
         * @param options.height
         * @param options.sideOrientation
         * @param options.frontUVs
         * @param options.backUVs
         * @returns the VertexData of the box
         * @deprecated use CreatePlaneVertexData instead
         */
        static CreatePlane(options: {
            size?: number;
            width?: number;
            height?: number;
            sideOrientation?: number;
            frontUVs?: Vector4;
            backUVs?: Vector4;
        }): VertexData;
        /**
         * Creates the VertexData of the Disc or regular Polygon
         * @param options an object used to set the following optional parameters for the disc, required but can be empty
         * * radius the radius of the disc, optional default 0.5
         * * tessellation the number of polygon sides, optional, default 64
         * * arc a number from 0 to 1, to create an unclosed polygon based on the fraction of the circumference given by the arc value, optional, default 1
         * * sideOrientation optional and takes the values : Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
         * * frontUvs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the front side, optional, default vector4 (0, 0, 1, 1)
         * * backUVs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the back side, optional, default vector4 (0, 0, 1, 1)
         * @param options.radius
         * @param options.tessellation
         * @param options.arc
         * @param options.sideOrientation
         * @param options.frontUVs
         * @param options.backUVs
         * @returns the VertexData of the box
         * @deprecated use CreateDiscVertexData instead
         */
        static CreateDisc(options: {
            radius?: number;
            tessellation?: number;
            arc?: number;
            sideOrientation?: number;
            frontUVs?: Vector4;
            backUVs?: Vector4;
        }): VertexData;
        /**
         * Creates the VertexData for an irregular Polygon in the XoZ plane using a mesh built by polygonTriangulation.build()
         * All parameters are provided by CreatePolygon as needed
         * @param polygon a mesh built from polygonTriangulation.build()
         * @param sideOrientation takes the values Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
         * @param fUV an array of Vector4 elements used to set different images to the top, rings and bottom respectively
         * @param fColors an array of Color3 elements used to set different colors to the top, rings and bottom respectively
         * @param frontUVs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the front side, optional, default vector4 (0, 0, 1, 1)
         * @param backUVs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the back side, optional, default vector4 (0, 0, 1, 1)
         * @param wrap a boolean, default false, when true and fUVs used texture is wrapped around all sides, when false texture is applied side
         * @returns the VertexData of the Polygon
         * @deprecated use CreatePolygonVertexData instead
         */
        static CreatePolygon(polygon: Mesh, sideOrientation: number, fUV?: Vector4[], fColors?: Color4[], frontUVs?: Vector4, backUVs?: Vector4, wrap?: boolean): VertexData;
        /**
         * Creates the VertexData of the IcoSphere
         * @param options an object used to set the following optional parameters for the IcoSphere, required but can be empty
         * * radius the radius of the IcoSphere, optional default 1
         * * radiusX allows stretching in the x direction, optional, default radius
         * * radiusY allows stretching in the y direction, optional, default radius
         * * radiusZ allows stretching in the z direction, optional, default radius
         * * flat when true creates a flat shaded mesh, optional, default true
         * * subdivisions increasing the subdivisions increases the number of faces, optional, default 4
         * * sideOrientation optional and takes the values : Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
         * * frontUvs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the front side, optional, default vector4 (0, 0, 1, 1)
         * * backUVs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the back side, optional, default vector4 (0, 0, 1, 1)
         * @param options.radius
         * @param options.radiusX
         * @param options.radiusY
         * @param options.radiusZ
         * @param options.flat
         * @param options.subdivisions
         * @param options.sideOrientation
         * @param options.frontUVs
         * @param options.backUVs
         * @returns the VertexData of the IcoSphere
         * @deprecated use CreateIcoSphereVertexData instead
         */
        static CreateIcoSphere(options: {
            radius?: number;
            radiusX?: number;
            radiusY?: number;
            radiusZ?: number;
            flat?: boolean;
            subdivisions?: number;
            sideOrientation?: number;
            frontUVs?: Vector4;
            backUVs?: Vector4;
        }): VertexData;
        /**
         * Creates the VertexData for a Polyhedron
         * @param options an object used to set the following optional parameters for the polyhedron, required but can be empty
         * * type provided types are:
         *  * 0 : Tetrahedron, 1 : Octahedron, 2 : Dodecahedron, 3 : Icosahedron, 4 : Rhombicuboctahedron, 5 : Triangular Prism, 6 : Pentagonal Prism, 7 : Hexagonal Prism, 8 : Square Pyramid (J1)
         *  * 9 : Pentagonal Pyramid (J2), 10 : Triangular Dipyramid (J12), 11 : Pentagonal Dipyramid (J13), 12 : Elongated Square Dipyramid (J15), 13 : Elongated Pentagonal Dipyramid (J16), 14 : Elongated Pentagonal Cupola (J20)
         * * size the size of the IcoSphere, optional default 1
         * * sizeX allows stretching in the x direction, optional, default size
         * * sizeY allows stretching in the y direction, optional, default size
         * * sizeZ allows stretching in the z direction, optional, default size
         * * custom a number that overwrites the type to create from an extended set of polyhedron from https://www.babylonjs-playground.com/#21QRSK#15 with minimised editor
         * * faceUV an array of Vector4 elements used to set different images to the top, rings and bottom respectively
         * * faceColors an array of Color3 elements used to set different colors to the top, rings and bottom respectively
         * * flat when true creates a flat shaded mesh, optional, default true
         * * subdivisions increasing the subdivisions increases the number of faces, optional, default 4
         * * sideOrientation optional and takes the values : Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
         * * frontUvs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the front side, optional, default vector4 (0, 0, 1, 1)
         * * backUVs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the back side, optional, default vector4 (0, 0, 1, 1)
         * @param options.type
         * @param options.size
         * @param options.sizeX
         * @param options.sizeY
         * @param options.sizeZ
         * @param options.custom
         * @param options.faceUV
         * @param options.faceColors
         * @param options.flat
         * @param options.sideOrientation
         * @param options.frontUVs
         * @param options.backUVs
         * @returns the VertexData of the Polyhedron
         * @deprecated use CreatePolyhedronVertexData instead
         */
        static CreatePolyhedron(options: {
            type?: number;
            size?: number;
            sizeX?: number;
            sizeY?: number;
            sizeZ?: number;
            custom?: any;
            faceUV?: Vector4[];
            faceColors?: Color4[];
            flat?: boolean;
            sideOrientation?: number;
            frontUVs?: Vector4;
            backUVs?: Vector4;
        }): VertexData;
        /**
         * Creates the VertexData for a Capsule, inspired from https://github.com/maximeq/three-js-capsule-geometry/blob/master/src/CapsuleBufferGeometry.js
         * @param options an object used to set the following optional parameters for the capsule, required but can be empty
         * @returns the VertexData of the Capsule
         * @deprecated Please use CreateCapsuleVertexData from the capsuleBuilder file instead
         */
        static CreateCapsule(options?: ICreateCapsuleOptions): VertexData;
        /**
         * Creates the VertexData for a TorusKnot
         * @param options an object used to set the following optional parameters for the TorusKnot, required but can be empty
         * * radius the radius of the torus knot, optional, default 2
         * * tube the thickness of the tube, optional, default 0.5
         * * radialSegments the number of sides on each tube segments, optional, default 32
         * * tubularSegments the number of tubes to decompose the knot into, optional, default 32
         * * p the number of windings around the z axis, optional,  default 2
         * * q the number of windings around the x axis, optional,  default 3
         * * sideOrientation optional and takes the values : Mesh.FRONTSIDE (default), Mesh.BACKSIDE or Mesh.DOUBLESIDE
         * * frontUvs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the front side, optional, default vector4 (0, 0, 1, 1)
         * * backUVs only usable when you create a double-sided mesh, used to choose what parts of the texture image to crop and apply on the back side, optional, default vector4 (0, 0, 1, 1)
         * @param options.radius
         * @param options.tube
         * @param options.radialSegments
         * @param options.tubularSegments
         * @param options.p
         * @param options.q
         * @param options.sideOrientation
         * @param options.frontUVs
         * @param options.backUVs
         * @returns the VertexData of the Torus Knot
         * @deprecated use CreateTorusKnotVertexData instead
         */
        static CreateTorusKnot(options: {
            radius?: number;
            tube?: number;
            radialSegments?: number;
            tubularSegments?: number;
            p?: number;
            q?: number;
            sideOrientation?: number;
            frontUVs?: Vector4;
            backUVs?: Vector4;
        }): VertexData;
        /**
         * Compute normals for given positions and indices
         * @param positions an array of vertex positions, [...., x, y, z, ......]
         * @param indices an array of indices in groups of three for each triangular facet, [...., i, j, k, ......]
         * @param normals an array of vertex normals, [...., x, y, z, ......]
         * @param options an object used to set the following optional parameters for the TorusKnot, optional
         * * facetNormals : optional array of facet normals (vector3)
         * * facetPositions : optional array of facet positions (vector3)
         * * facetPartitioning : optional partitioning array. facetPositions is required for facetPartitioning computation
         * * ratio : optional partitioning ratio / bounding box, required for facetPartitioning computation
         * * bInfo : optional bounding info, required for facetPartitioning computation
         * * bbSize : optional bounding box size data, required for facetPartitioning computation
         * * subDiv : optional partitioning data about subdivisions on  each axis (int), required for facetPartitioning computation
         * * useRightHandedSystem: optional boolean to for right handed system computation
         * * depthSort : optional boolean to enable the facet depth sort computation
         * * distanceTo : optional Vector3 to compute the facet depth from this location
         * * depthSortedFacets : optional array of depthSortedFacets to store the facet distances from the reference location
         * @param options.facetNormals
         * @param options.facetPositions
         * @param options.facetPartitioning
         * @param options.ratio
         * @param options.bInfo
         * @param options.bbSize
         * @param options.subDiv
         * @param options.useRightHandedSystem
         * @param options.depthSort
         * @param options.distanceTo
         * @param options.depthSortedFacets
         */
        static ComputeNormals(positions: any, indices: any, normals: any, options?: {
            facetNormals?: any;
            facetPositions?: any;
            facetPartitioning?: any;
            ratio?: number;
            bInfo?: any;
            bbSize?: Vector3;
            subDiv?: any;
            useRightHandedSystem?: boolean;
            depthSort?: boolean;
            distanceTo?: Vector3;
            depthSortedFacets?: any;
        }): void;
        /**
         * @internal
         */
        static _ComputeSides(sideOrientation: number, positions: FloatArray, indices: FloatArray | IndicesArray, normals: FloatArray, uvs: FloatArray, frontUVs?: Vector4, backUVs?: Vector4): void;
        /**
         * Applies VertexData created from the imported parameters to the geometry
         * @param parsedVertexData the parsed data from an imported file
         * @param geometry the geometry to apply the VertexData to
         */
        static ImportVertexData(parsedVertexData: any, geometry: Geometry): void;
    }


    /**
     * Class containing static functions to help procedurally build meshes
     */
    export var MeshBuilder: {
        CreateBox: typeof CreateBox;
        CreateTiledBox: typeof CreateTiledBox;
        CreateSphere: typeof CreateSphere;
        CreateDisc: typeof CreateDisc;
        CreateIcoSphere: typeof CreateIcoSphere;
        CreateRibbon: typeof CreateRibbon;
        CreateCylinder: typeof CreateCylinder;
        CreateTorus: typeof CreateTorus;
        CreateTorusKnot: typeof CreateTorusKnot;
        CreateLineSystem: typeof CreateLineSystem;
        CreateLines: typeof CreateLines;
        CreateDashedLines: typeof CreateDashedLines;
        ExtrudeShape: typeof ExtrudeShape;
        ExtrudeShapeCustom: typeof ExtrudeShapeCustom;
        CreateLathe: typeof CreateLathe;
        CreateTiledPlane: typeof CreateTiledPlane;
        CreatePlane: typeof CreatePlane;
        CreateGround: typeof CreateGround;
        CreateTiledGround: typeof CreateTiledGround;
        CreateGroundFromHeightMap: typeof CreateGroundFromHeightMap;
        CreatePolygon: typeof CreatePolygon;
        ExtrudePolygon: typeof ExtrudePolygon;
        CreateTube: typeof CreateTube;
        CreatePolyhedron: typeof CreatePolyhedron;
        CreateGeodesic: typeof CreateGeodesic;
        CreateGoldberg: typeof CreateGoldberg;
        CreateDecal: typeof CreateDecal;
        CreateCapsule: typeof CreateCapsule;
    };


    /**
     * Class used to represent a specific level of detail of a mesh
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/LOD
     */
    export class MeshLODLevel {
        /** Either distance from the center of the object to show this level or the screen coverage if `useLODScreenCoverage` is set to `true` on the mesh*/
        distanceOrScreenCoverage: number;
        /** Defines the mesh to use to render this level */
        mesh: Nullable<Mesh>;
        /**
         * Creates a new LOD level
         * @param distanceOrScreenCoverage defines either the distance or the screen coverage where this level should start being displayed
         * @param mesh defines the mesh to use to render this level
         */
        constructor(
        /** Either distance from the center of the object to show this level or the screen coverage if `useLODScreenCoverage` is set to `true` on the mesh*/
        distanceOrScreenCoverage: number, 
        /** Defines the mesh to use to render this level */
        mesh: Nullable<Mesh>);
    }


    /**
     * A simplifier interface for future simplification implementations
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/simplifyingMeshes
     */
    export interface ISimplifier {
        /**
         * Simplification of a given mesh according to the given settings.
         * Since this requires computation, it is assumed that the function runs async.
         * @param settings The settings of the simplification, including quality and distance
         * @param successCallback A callback that will be called after the mesh was simplified.
         * @param errorCallback in case of an error, this callback will be called. optional.
         */
        simplify(settings: ISimplificationSettings, successCallback: (simplifiedMeshes: Mesh) => void, errorCallback?: () => void): void;
    }
    /**
     * Expected simplification settings.
     * Quality should be between 0 and 1 (1 being 100%, 0 being 0%)
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/simplifyingMeshes
     */
    export interface ISimplificationSettings {
        /**
         * Gets or sets the expected quality
         */
        quality: number;
        /**
         * Gets or sets the distance when this optimized version should be used
         */
        distance: number;
        /**
         * Gets an already optimized mesh
         */
        optimizeMesh?: boolean;
    }
    /**
     * Class used to specify simplification options
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/simplifyingMeshes
     */
    export class SimplificationSettings implements ISimplificationSettings {
        /** expected quality */
        quality: number;
        /** distance when this optimized version should be used */
        distance: number;
        /** already optimized mesh  */
        optimizeMesh?: boolean | undefined;
        /**
         * Creates a SimplificationSettings
         * @param quality expected quality
         * @param distance distance when this optimized version should be used
         * @param optimizeMesh already optimized mesh
         */
        constructor(
        /** expected quality */
        quality: number, 
        /** distance when this optimized version should be used */
        distance: number, 
        /** already optimized mesh  */
        optimizeMesh?: boolean | undefined);
    }
    /**
     * Interface used to define a simplification task
     */
    export interface ISimplificationTask {
        /**
         * Array of settings
         */
        settings: Array<ISimplificationSettings>;
        /**
         * Simplification type
         */
        simplificationType: SimplificationType;
        /**
         * Mesh to simplify
         */
        mesh: Mesh;
        /**
         * Callback called on success
         */
        successCallback?: () => void;
        /**
         * Defines if parallel processing can be used
         */
        parallelProcessing: boolean;
    }
    /**
     * Queue used to order the simplification tasks
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/simplifyingMeshes
     */
    export class SimplificationQueue {
        private _simplificationArray;
        /**
         * Gets a boolean indicating that the process is still running
         */
        running: boolean;
        /**
         * Creates a new queue
         */
        constructor();
        /**
         * Adds a new simplification task
         * @param task defines a task to add
         */
        addTask(task: ISimplificationTask): void;
        /**
         * Execute next task
         */
        executeNext(): void;
        /**
         * Execute a simplification task
         * @param task defines the task to run
         */
        runSimplification(task: ISimplificationTask): void;
        private _getSimplifier;
    }
    /**
     * The implemented types of simplification
     * At the moment only Quadratic Error Decimation is implemented
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/simplifyingMeshes
     */
    export enum SimplificationType {
        /** Quadratic error decimation */
        QUADRATIC = 0
    }
    /**
     * An implementation of the Quadratic Error simplification algorithm.
     * Original paper : http://www1.cs.columbia.edu/~cs4162/html05s/garland97.pdf
     * Ported mostly from QSlim and http://voxels.blogspot.de/2014/05/quadric-mesh-simplification-with-source.html to babylon JS
     * @author RaananW
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/simplifyingMeshes
     */
    export class QuadraticErrorSimplification implements ISimplifier {
        private _mesh;
        private _triangles;
        private _vertices;
        private _references;
        private _reconstructedMesh;
        /** Gets or sets the number pf sync iterations */
        syncIterations: number;
        /** Gets or sets the aggressiveness of the simplifier */
        aggressiveness: number;
        /** Gets or sets the number of allowed iterations for decimation */
        decimationIterations: number;
        /** Gets or sets the espilon to use for bounding box computation */
        boundingBoxEpsilon: number;
        /**
         * Creates a new QuadraticErrorSimplification
         * @param _mesh defines the target mesh
         */
        constructor(_mesh: Mesh);
        /**
         * Simplification of a given mesh according to the given settings.
         * Since this requires computation, it is assumed that the function runs async.
         * @param settings The settings of the simplification, including quality and distance
         * @param successCallback A callback that will be called after the mesh was simplified.
         */
        simplify(settings: ISimplificationSettings, successCallback: (simplifiedMesh: Mesh) => void): void;
        private _runDecimation;
        private _initWithMesh;
        private _init;
        private _reconstructMesh;
        private _initDecimatedMesh;
        private _isFlipped;
        private _updateTriangles;
        private _identifyBorder;
        private _updateMesh;
        private _vertexError;
        private _calculateError;
    }


        interface Scene {
            /** @internal (Backing field) */
            _simplificationQueue: SimplificationQueue;
            /**
             * Gets or sets the simplification queue attached to the scene
             * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/simplifyingMeshes
             */
            simplificationQueue: SimplificationQueue;
        }
        interface Mesh {
            /**
             * Simplify the mesh according to the given array of settings.
             * Function will return immediately and will simplify async
             * @param settings a collection of simplification settings
             * @param parallelProcessing should all levels calculate parallel or one after the other
             * @param simplificationType the type of simplification to run
             * @param successCallback optional success callback to be called after the simplification finished processing all settings
             * @returns the current mesh
             */
            simplify(settings: Array<ISimplificationSettings>, parallelProcessing?: boolean, simplificationType?: SimplificationType, successCallback?: (mesh?: Mesh, submeshIndex?: number) => void): Mesh;
        }
    /**
     * Defines the simplification queue scene component responsible to help scheduling the various simplification task
     * created in a scene
     */
    export class SimplicationQueueSceneComponent implements ISceneComponent {
        /**
         * The component name helpfull to identify the component in the list of scene components.
         */
        readonly name = "SimplificationQueue";
        /**
         * The scene the component belongs to.
         */
        scene: Scene;
        /**
         * Creates a new instance of the component for the given scene
         * @param scene Defines the scene to register the component in
         */
        constructor(scene: Scene);
        /**
         * Registers the component in a given scene
         */
        register(): void;
        /**
         * Rebuilds the elements related to this component in case of
         * context lost for instance.
         */
        rebuild(): void;
        /**
         * Disposes the component and the associated resources
         */
        dispose(): void;
        private _beforeCameraUpdate;
    }


    /**
     * Polygon
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/param#non-regular-polygon
     */
    export class Polygon {
        /**
         * Creates a rectangle
         * @param xmin bottom X coord
         * @param ymin bottom Y coord
         * @param xmax top X coord
         * @param ymax top Y coord
         * @returns points that make the resulting rectangle
         */
        static Rectangle(xmin: number, ymin: number, xmax: number, ymax: number): Vector2[];
        /**
         * Creates a circle
         * @param radius radius of circle
         * @param cx scale in x
         * @param cy scale in y
         * @param numberOfSides number of sides that make up the circle
         * @returns points that make the resulting circle
         */
        static Circle(radius: number, cx?: number, cy?: number, numberOfSides?: number): Vector2[];
        /**
         * Creates a polygon from input string
         * @param input Input polygon data
         * @returns the parsed points
         */
        static Parse(input: string): Vector2[];
        /**
         * Starts building a polygon from x and y coordinates
         * @param x x coordinate
         * @param y y coordinate
         * @returns the started path2
         */
        static StartingAt(x: number, y: number): Path2;
    }
    /**
     * Builds a polygon
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/creation/param/polyMeshBuilder
     */
    export class PolygonMeshBuilder {
        private _points;
        private _outlinepoints;
        private _holes;
        private _name;
        private _scene;
        private _epoints;
        private _eholes;
        private _addToepoint;
        /**
         * Babylon reference to the earcut plugin.
         */
        bjsEarcut: any;
        /**
         * Creates a PolygonMeshBuilder
         * @param name name of the builder
         * @param contours Path of the polygon
         * @param scene scene to add to when creating the mesh
         * @param earcutInjection can be used to inject your own earcut reference
         */
        constructor(name: string, contours: Path2 | Vector2[] | any, scene?: Scene, earcutInjection?: any);
        /**
         * Adds a hole within the polygon
         * @param hole Array of points defining the hole
         * @returns this
         */
        addHole(hole: Vector2[]): PolygonMeshBuilder;
        /**
         * Creates the polygon
         * @param updatable If the mesh should be updatable
         * @param depth The depth of the mesh created
         * @param smoothingThreshold Dot product threshold for smoothed normals
         * @returns the created mesh
         */
        build(updatable?: boolean, depth?: number, smoothingThreshold?: number): Mesh;
        /**
         * Creates the polygon
         * @param depth The depth of the mesh created
         * @param smoothingThreshold Dot product threshold for smoothed normals
         * @returns the created VertexData
         */
        buildVertexData(depth?: number, smoothingThreshold?: number): VertexData;
        /**
         * Adds a side to the polygon
         * @param positions points that make the polygon
         * @param normals normals of the polygon
         * @param uvs uvs of the polygon
         * @param indices indices of the polygon
         * @param bounds bounds of the polygon
         * @param points points of the polygon
         * @param depth depth of the polygon
         * @param flip flip of the polygon
         * @param smoothingThreshold
         */
        private _addSide;
    }


    /**
     * Defines a subdivision inside a mesh
     */
    export class SubMesh implements ICullable {
        /** the material index to use */
        materialIndex: number;
        /** vertex index start */
        verticesStart: number;
        /** vertices count */
        verticesCount: number;
        /** index start */
        indexStart: number;
        /** indices count */
        indexCount: number;
        private _engine;
        /** @internal */
        _drawWrappers: Array<DrawWrapper>;
        private _mainDrawWrapperOverride;
        /**
         * Gets material defines used by the effect associated to the sub mesh
         */
        get materialDefines(): Nullable<MaterialDefines>;
        /**
         * Sets material defines used by the effect associated to the sub mesh
         */
        set materialDefines(defines: Nullable<MaterialDefines>);
        /**
         * @internal
         */
        _getDrawWrapper(passId?: number, createIfNotExisting?: boolean): DrawWrapper | undefined;
        /**
         * @internal
         */
        _removeDrawWrapper(passId: number, disposeWrapper?: boolean): void;
        /**
         * Gets associated (main) effect (possibly the effect override if defined)
         */
        get effect(): Nullable<Effect>;
        /** @internal */
        get _drawWrapper(): DrawWrapper;
        /** @internal */
        get _drawWrapperOverride(): Nullable<DrawWrapper>;
        /**
         * @internal
         */
        _setMainDrawWrapperOverride(wrapper: Nullable<DrawWrapper>): void;
        /**
         * Sets associated effect (effect used to render this submesh)
         * @param effect defines the effect to associate with
         * @param defines defines the set of defines used to compile this effect
         * @param materialContext material context associated to the effect
         * @param resetContext true to reset the draw context
         */
        setEffect(effect: Nullable<Effect>, defines?: Nullable<string | MaterialDefines>, materialContext?: IMaterialContext, resetContext?: boolean): void;
        /**
         * Resets the draw wrappers cache
         * @param passId If provided, releases only the draw wrapper corresponding to this render pass id
         */
        resetDrawCache(passId?: number): void;
        /** @internal */
        _linesIndexCount: number;
        private _mesh;
        private _renderingMesh;
        private _boundingInfo;
        private _linesIndexBuffer;
        /** @internal */
        _lastColliderWorldVertices: Nullable<Vector3[]>;
        /** @internal */
        _trianglePlanes: Plane[];
        /** @internal */
        _lastColliderTransformMatrix: Nullable<Matrix>;
        /** @internal */
        _wasDispatched: boolean;
        /** @internal */
        _renderId: number;
        /** @internal */
        _alphaIndex: number;
        /** @internal */
        _distanceToCamera: number;
        /** @internal */
        _id: number;
        private _currentMaterial;
        /**
         * Add a new submesh to a mesh
         * @param materialIndex defines the material index to use
         * @param verticesStart defines vertex index start
         * @param verticesCount defines vertices count
         * @param indexStart defines index start
         * @param indexCount defines indices count
         * @param mesh defines the parent mesh
         * @param renderingMesh defines an optional rendering mesh
         * @param createBoundingBox defines if bounding box should be created for this submesh
         * @returns the new submesh
         */
        static AddToMesh(materialIndex: number, verticesStart: number, verticesCount: number, indexStart: number, indexCount: number, mesh: AbstractMesh, renderingMesh?: Mesh, createBoundingBox?: boolean): SubMesh;
        /**
         * Creates a new submesh
         * @param materialIndex defines the material index to use
         * @param verticesStart defines vertex index start
         * @param verticesCount defines vertices count
         * @param indexStart defines index start
         * @param indexCount defines indices count
         * @param mesh defines the parent mesh
         * @param renderingMesh defines an optional rendering mesh
         * @param createBoundingBox defines if bounding box should be created for this submesh
         * @param addToMesh defines a boolean indicating that the submesh must be added to the mesh.subMeshes array (true by default)
         */
        constructor(
        /** the material index to use */
        materialIndex: number, 
        /** vertex index start */
        verticesStart: number, 
        /** vertices count */
        verticesCount: number, 
        /** index start */
        indexStart: number, 
        /** indices count */
        indexCount: number, mesh: AbstractMesh, renderingMesh?: Mesh, createBoundingBox?: boolean, addToMesh?: boolean);
        /**
         * Returns true if this submesh covers the entire parent mesh
         * @ignorenaming
         */
        get IsGlobal(): boolean;
        /**
         * Returns the submesh BoundingInfo object
         * @returns current bounding info (or mesh's one if the submesh is global)
         */
        getBoundingInfo(): BoundingInfo;
        /**
         * Sets the submesh BoundingInfo
         * @param boundingInfo defines the new bounding info to use
         * @returns the SubMesh
         */
        setBoundingInfo(boundingInfo: BoundingInfo): SubMesh;
        /**
         * Returns the mesh of the current submesh
         * @returns the parent mesh
         */
        getMesh(): AbstractMesh;
        /**
         * Returns the rendering mesh of the submesh
         * @returns the rendering mesh (could be different from parent mesh)
         */
        getRenderingMesh(): Mesh;
        /**
         * Returns the replacement mesh of the submesh
         * @returns the replacement mesh (could be different from parent mesh)
         */
        getReplacementMesh(): Nullable<AbstractMesh>;
        /**
         * Returns the effective mesh of the submesh
         * @returns the effective mesh (could be different from parent mesh)
         */
        getEffectiveMesh(): AbstractMesh;
        /**
         * Returns the submesh material
         * @param getDefaultMaterial Defines whether or not to get the default material if nothing has been defined.
         * @returns null or the current material
         */
        getMaterial(getDefaultMaterial?: boolean): Nullable<Material>;
        private _isMultiMaterial;
        /**
         * Sets a new updated BoundingInfo object to the submesh
         * @param data defines an optional position array to use to determine the bounding info
         * @returns the SubMesh
         */
        refreshBoundingInfo(data?: Nullable<FloatArray>): SubMesh;
        /**
         * @internal
         */
        _checkCollision(collider: Collider): boolean;
        /**
         * Updates the submesh BoundingInfo
         * @param world defines the world matrix to use to update the bounding info
         * @returns the submesh
         */
        updateBoundingInfo(world: DeepImmutable<Matrix>): SubMesh;
        /**
         * True is the submesh bounding box intersects the frustum defined by the passed array of planes.
         * @param frustumPlanes defines the frustum planes
         * @returns true if the submesh is intersecting with the frustum
         */
        isInFrustum(frustumPlanes: Plane[]): boolean;
        /**
         * True is the submesh bounding box is completely inside the frustum defined by the passed array of planes
         * @param frustumPlanes defines the frustum planes
         * @returns true if the submesh is inside the frustum
         */
        isCompletelyInFrustum(frustumPlanes: Plane[]): boolean;
        /**
         * Renders the submesh
         * @param enableAlphaMode defines if alpha needs to be used
         * @returns the submesh
         */
        render(enableAlphaMode: boolean): SubMesh;
        /**
         * @internal
         */
        _getLinesIndexBuffer(indices: IndicesArray, engine: Engine): DataBuffer;
        /**
         * Checks if the submesh intersects with a ray
         * @param ray defines the ray to test
         * @returns true is the passed ray intersects the submesh bounding box
         */
        canIntersects(ray: Ray): boolean;
        /**
         * Intersects current submesh with a ray
         * @param ray defines the ray to test
         * @param positions defines mesh's positions array
         * @param indices defines mesh's indices array
         * @param fastCheck defines if the first intersection will be used (and not the closest)
         * @param trianglePredicate defines an optional predicate used to select faces when a mesh intersection is detected
         * @returns intersection info or null if no intersection
         */
        intersects(ray: Ray, positions: Vector3[], indices: IndicesArray, fastCheck?: boolean, trianglePredicate?: TrianglePickingPredicate): Nullable<IntersectionInfo>;
        /**
         * @internal
         */
        private _intersectLines;
        /**
         * @internal
         */
        private _intersectUnIndexedLines;
        /**
         * @internal
         */
        private _intersectTriangles;
        /**
         * @internal
         */
        private _intersectUnIndexedTriangles;
        /** @internal */
        _rebuild(): void;
        /**
         * Creates a new submesh from the passed mesh
         * @param newMesh defines the new hosting mesh
         * @param newRenderingMesh defines an optional rendering mesh
         * @returns the new submesh
         */
        clone(newMesh: AbstractMesh, newRenderingMesh?: Mesh): SubMesh;
        /**
         * Release associated resources
         */
        dispose(): void;
        /**
         * Gets the class name
         * @returns the string "SubMesh".
         */
        getClassName(): string;
        /**
         * Creates a new submesh from indices data
         * @param materialIndex the index of the main mesh material
         * @param startIndex the index where to start the copy in the mesh indices array
         * @param indexCount the number of indices to copy then from the startIndex
         * @param mesh the main mesh to create the submesh from
         * @param renderingMesh the optional rendering mesh
         * @param createBoundingBox defines if bounding box should be created for this submesh
         * @returns a new submesh
         */
        static CreateFromIndices(materialIndex: number, startIndex: number, indexCount: number, mesh: AbstractMesh, renderingMesh?: Mesh, createBoundingBox?: boolean): SubMesh;
    }


        interface SubMesh {
            /** @internal */
            _projectOnTrianglesToRef(vector: Vector3, positions: Vector3[], indices: IndicesArray, step: number, checkStopper: boolean, ref: Vector3): number;
            /** @internal */
            _projectOnUnIndexedTrianglesToRef(vector: Vector3, positions: Vector3[], indices: IndicesArray, ref: Vector3): number;
            /**
             * Projects a point on this submesh and stores the result in "ref"
             *
             * @param vector point to project
             * @param positions defines mesh's positions array
             * @param indices defines mesh's indices array
             * @param ref vector that will store the result
             * @returns distance from the point and the submesh, or -1 if the mesh rendering mode doesn't support projections
             */
            projectToRef(vector: Vector3, positions: Vector3[], indices: IndicesArray, ref: Vector3): number;
        }


        interface Mesh {
            /**
             * Gets or sets a boolean defining if we want picking to pick thin instances as well
             */
            thinInstanceEnablePicking: boolean;
            /**
             * Creates a new thin instance
             * @param matrix the matrix or array of matrices (position, rotation, scale) of the thin instance(s) to create
             * @param refresh true to refresh the underlying gpu buffer (default: true). If you do multiple calls to this method in a row, set refresh to true only for the last call to save performance
             * @returns the thin instance index number. If you pass an array of matrices, other instance indexes are index+1, index+2, etc
             */
            thinInstanceAdd(matrix: DeepImmutableObject<Matrix> | Array<DeepImmutableObject<Matrix>>, refresh?: boolean): number;
            /**
             * Adds the transformation (matrix) of the current mesh as a thin instance
             * @param refresh true to refresh the underlying gpu buffer (default: true). If you do multiple calls to this method in a row, set refresh to true only for the last call to save performance
             * @returns the thin instance index number
             */
            thinInstanceAddSelf(refresh?: boolean): number;
            /**
             * Registers a custom attribute to be used with thin instances
             * @param kind name of the attribute
             * @param stride size in floats of the attribute
             */
            thinInstanceRegisterAttribute(kind: string, stride: number): void;
            /**
             * Sets the matrix of a thin instance
             * @param index index of the thin instance
             * @param matrix matrix to set
             * @param refresh true to refresh the underlying gpu buffer (default: true). If you do multiple calls to this method in a row, set refresh to true only for the last call to save performance
             */
            thinInstanceSetMatrixAt(index: number, matrix: DeepImmutableObject<Matrix>, refresh?: boolean): void;
            /**
             * Sets the value of a custom attribute for a thin instance
             * @param kind name of the attribute
             * @param index index of the thin instance
             * @param value value to set
             * @param refresh true to refresh the underlying gpu buffer (default: true). If you do multiple calls to this method in a row, set refresh to true only for the last call to save performance
             */
            thinInstanceSetAttributeAt(kind: string, index: number, value: Array<number>, refresh?: boolean): void;
            /**
             * Gets / sets the number of thin instances to display. Note that you can't set a number higher than what the underlying buffer can handle.
             */
            thinInstanceCount: number;
            /**
             * Sets a buffer to be used with thin instances. This method is a faster way to setup multiple instances than calling thinInstanceAdd repeatedly
             * @param kind name of the attribute. Use "matrix" to setup the buffer of matrices
             * @param buffer buffer to set
             * @param stride size in floats of each value of the buffer
             * @param staticBuffer indicates that the buffer is static, so that you won't change it after it is set (better performances - false by default)
             */
            thinInstanceSetBuffer(kind: string, buffer: Nullable<Float32Array>, stride?: number, staticBuffer?: boolean): void;
            /**
             * Gets the list of world matrices
             * @returns an array containing all the world matrices from the thin instances
             */
            thinInstanceGetWorldMatrices(): Matrix[];
            /**
             * Synchronize the gpu buffers with a thin instance buffer. Call this method if you update later on the buffers passed to thinInstanceSetBuffer
             * @param kind name of the attribute to update. Use "matrix" to update the buffer of matrices
             */
            thinInstanceBufferUpdated(kind: string): void;
            /**
             * Applies a partial update to a buffer directly on the GPU
             * Note that the buffer located on the CPU is NOT updated! It's up to you to update it (or not) with the same data you pass to this method
             * @param kind name of the attribute to update. Use "matrix" to update the buffer of matrices
             * @param data the data to set in the GPU buffer
             * @param offset the offset in the GPU buffer where to update the data
             */
            thinInstancePartialBufferUpdate(kind: string, data: Float32Array, offset: number): void;
            /**
             * Refreshes the bounding info, taking into account all the thin instances defined
             * @param forceRefreshParentInfo true to force recomputing the mesh bounding info and use it to compute the aggregated bounding info
             * @param applySkeleton defines whether to apply the skeleton before computing the bounding info
             * @param applyMorph  defines whether to apply the morph target before computing the bounding info
             */
            thinInstanceRefreshBoundingInfo(forceRefreshParentInfo?: boolean, applySkeleton?: boolean, applyMorph?: boolean): void;
            /** @internal */
            _thinInstanceInitializeUserStorage(): void;
            /** @internal */
            _thinInstanceUpdateBufferSize(kind: string, numInstances?: number): void;
            /** @internal */
            _thinInstanceCreateMatrixBuffer(kind: string, buffer: Nullable<Float32Array>, staticBuffer: boolean): Buffer;
            /** @internal */
            _userThinInstanceBuffersStorage: {
                data: {
                    [key: string]: Float32Array;
                };
                sizes: {
                    [key: string]: number;
                };
                vertexBuffers: {
                    [key: string]: Nullable<VertexBuffer>;
                };
                strides: {
                    [key: string]: number;
                };
            };
        }


    /**
     * Class used to create a trail following a mesh
     */
    export class TrailMesh extends Mesh {
        private _generator;
        private _autoStart;
        private _running;
        private _diameter;
        private _length;
        private _sectionPolygonPointsCount;
        private _sectionVectors;
        private _sectionNormalVectors;
        private _beforeRenderObserver;
        /**
         * @constructor
         * @param name The value used by scene.getMeshByName() to do a lookup.
         * @param generator The mesh or transform node to generate a trail.
         * @param scene The scene to add this mesh to.
         * @param diameter Diameter of trailing mesh. Default is 1.
         * @param length Length of trailing mesh. Default is 60.
         * @param autoStart Automatically start trailing mesh. Default true.
         */
        constructor(name: string, generator: TransformNode, scene?: Scene, diameter?: number, length?: number, autoStart?: boolean);
        /**
         * "TrailMesh"
         * @returns "TrailMesh"
         */
        getClassName(): string;
        private _createMesh;
        /**
         * Start trailing mesh.
         */
        start(): void;
        /**
         * Stop trailing mesh.
         */
        stop(): void;
        /**
         * Update trailing mesh geometry.
         */
        update(): void;
        /**
         * Returns a new TrailMesh object.
         * @param name is a string, the name given to the new mesh
         * @param newGenerator use new generator object for cloned trail mesh
         * @returns a new mesh
         */
        clone(name: string | undefined, newGenerator: TransformNode): TrailMesh;
        /**
         * Serializes this trail mesh
         * @param serializationObject object to write serialization to
         */
        serialize(serializationObject: any): void;
        /**
         * Parses a serialized trail mesh
         * @param parsedMesh the serialized mesh
         * @param scene the scene to create the trail mesh in
         * @returns the created trail mesh
         */
        static Parse(parsedMesh: any, scene: Scene): TrailMesh;
    }


    /**
     * A TransformNode is an object that is not rendered but can be used as a center of transformation. This can decrease memory usage and increase rendering speed compared to using an empty mesh as a parent and is less complicated than using a pivot matrix.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/transforms/parent_pivot/transform_node
     */
    export class TransformNode extends Node {
        /**
         * Object will not rotate to face the camera
         */
        static BILLBOARDMODE_NONE: number;
        /**
         * Object will rotate to face the camera but only on the x axis
         */
        static BILLBOARDMODE_X: number;
        /**
         * Object will rotate to face the camera but only on the y axis
         */
        static BILLBOARDMODE_Y: number;
        /**
         * Object will rotate to face the camera but only on the z axis
         */
        static BILLBOARDMODE_Z: number;
        /**
         * Object will rotate to face the camera
         */
        static BILLBOARDMODE_ALL: number;
        /**
         * Object will rotate to face the camera's position instead of orientation
         */
        static BILLBOARDMODE_USE_POSITION: number;
        /**
         * Child transform with Billboard flags should or should not apply parent rotation (default if off)
         */
        static BillboardUseParentOrientation: boolean;
        private static _TmpRotation;
        private static _TmpScaling;
        private static _TmpTranslation;
        private _forward;
        private _up;
        private _right;
        private _position;
        private _rotation;
        private _rotationQuaternion;
        protected _scaling: Vector3;
        private _transformToBoneReferal;
        private _currentParentWhenAttachingToBone;
        private _isAbsoluteSynced;
        private _billboardMode;
        /**
         * Gets or sets the billboard mode. Default is 0.
         *
         * | Value | Type | Description |
         * | --- | --- | --- |
         * | 0 | BILLBOARDMODE_NONE |  |
         * | 1 | BILLBOARDMODE_X |  |
         * | 2 | BILLBOARDMODE_Y |  |
         * | 4 | BILLBOARDMODE_Z |  |
         * | 7 | BILLBOARDMODE_ALL |  |
         *
         */
        get billboardMode(): number;
        set billboardMode(value: number);
        private _preserveParentRotationForBillboard;
        /**
         * Gets or sets a boolean indicating that parent rotation should be preserved when using billboards.
         * This could be useful for glTF objects where parent rotation helps converting from right handed to left handed
         */
        get preserveParentRotationForBillboard(): boolean;
        set preserveParentRotationForBillboard(value: boolean);
        private _computeUseBillboardPath;
        /**
         * Multiplication factor on scale x/y/z when computing the world matrix. Eg. for a 1x1x1 cube setting this to 2 will make it a 2x2x2 cube
         */
        scalingDeterminant: number;
        private _infiniteDistance;
        /**
         * Gets or sets the distance of the object to max, often used by skybox
         */
        get infiniteDistance(): boolean;
        set infiniteDistance(value: boolean);
        /**
         * Gets or sets a boolean indicating that non uniform scaling (when at least one component is different from others) should be ignored.
         * By default the system will update normals to compensate
         */
        ignoreNonUniformScaling: boolean;
        /**
         * Gets or sets a boolean indicating that even if rotationQuaternion is defined, you can keep updating rotation property and Babylon.js will just mix both
         */
        reIntegrateRotationIntoRotationQuaternion: boolean;
        /** @internal */
        _poseMatrix: Nullable<Matrix>;
        /** @internal */
        _localMatrix: Matrix;
        private _usePivotMatrix;
        private _absolutePosition;
        private _absoluteScaling;
        private _absoluteRotationQuaternion;
        private _pivotMatrix;
        private _pivotMatrixInverse;
        /** @internal */
        _postMultiplyPivotMatrix: boolean;
        protected _isWorldMatrixFrozen: boolean;
        /** @internal */
        _indexInSceneTransformNodesArray: number;
        /**
         * An event triggered after the world matrix is updated
         */
        onAfterWorldMatrixUpdateObservable: Observable<TransformNode>;
        constructor(name: string, scene?: Nullable<Scene>, isPure?: boolean);
        /**
         * Gets a string identifying the name of the class
         * @returns "TransformNode" string
         */
        getClassName(): string;
        /**
         * Gets or set the node position (default is (0.0, 0.0, 0.0))
         */
        get position(): Vector3;
        set position(newPosition: Vector3);
        /**
         * return true if a pivot has been set
         * @returns true if a pivot matrix is used
         */
        isUsingPivotMatrix(): boolean;
        /**
         * Gets or sets the rotation property : a Vector3 defining the rotation value in radians around each local axis X, Y, Z  (default is (0.0, 0.0, 0.0)).
         * If rotation quaternion is set, this Vector3 will be ignored and copy from the quaternion
         */
        get rotation(): Vector3;
        set rotation(newRotation: Vector3);
        /**
         * Gets or sets the scaling property : a Vector3 defining the node scaling along each local axis X, Y, Z (default is (1.0, 1.0, 1.0)).
         */
        get scaling(): Vector3;
        set scaling(newScaling: Vector3);
        /**
         * Gets or sets the rotation Quaternion property : this a Quaternion object defining the node rotation by using a unit quaternion (undefined by default, but can be null).
         * If set, only the rotationQuaternion is then used to compute the node rotation (ie. node.rotation will be ignored)
         */
        get rotationQuaternion(): Nullable<Quaternion>;
        set rotationQuaternion(quaternion: Nullable<Quaternion>);
        /**
         * The forward direction of that transform in world space.
         */
        get forward(): Vector3;
        /**
         * The up direction of that transform in world space.
         */
        get up(): Vector3;
        /**
         * The right direction of that transform in world space.
         */
        get right(): Vector3;
        /**
         * Copies the parameter passed Matrix into the mesh Pose matrix.
         * @param matrix the matrix to copy the pose from
         * @returns this TransformNode.
         */
        updatePoseMatrix(matrix: Matrix): TransformNode;
        /**
         * Returns the mesh Pose matrix.
         * @returns the pose matrix
         */
        getPoseMatrix(): Matrix;
        /** @internal */
        _isSynchronized(): boolean;
        /** @internal */
        _initCache(): void;
        /**
         * Returns the current mesh absolute position.
         * Returns a Vector3.
         */
        get absolutePosition(): Vector3;
        /**
         * Returns the current mesh absolute scaling.
         * Returns a Vector3.
         */
        get absoluteScaling(): Vector3;
        /**
         * Returns the current mesh absolute rotation.
         * Returns a Quaternion.
         */
        get absoluteRotationQuaternion(): Quaternion;
        /**
         * Sets a new matrix to apply before all other transformation
         * @param matrix defines the transform matrix
         * @returns the current TransformNode
         */
        setPreTransformMatrix(matrix: Matrix): TransformNode;
        /**
         * Sets a new pivot matrix to the current node
         * @param matrix defines the new pivot matrix to use
         * @param postMultiplyPivotMatrix defines if the pivot matrix must be cancelled in the world matrix. When this parameter is set to true (default), the inverse of the pivot matrix is also applied at the end to cancel the transformation effect
         * @returns the current TransformNode
         */
        setPivotMatrix(matrix: DeepImmutable<Matrix>, postMultiplyPivotMatrix?: boolean): TransformNode;
        /**
         * Returns the mesh pivot matrix.
         * Default : Identity.
         * @returns the matrix
         */
        getPivotMatrix(): Matrix;
        /**
         * Instantiate (when possible) or clone that node with its hierarchy
         * @param newParent defines the new parent to use for the instance (or clone)
         * @param options defines options to configure how copy is done
         * @param options.doNotInstantiate defines if the model must be instantiated or just cloned
         * @param onNewNodeCreated defines an option callback to call when a clone or an instance is created
         * @returns an instance (or a clone) of the current node with its hierarchy
         */
        instantiateHierarchy(newParent?: Nullable<TransformNode>, options?: {
            doNotInstantiate: boolean | ((node: TransformNode) => boolean);
        }, onNewNodeCreated?: (source: TransformNode, clone: TransformNode) => void): Nullable<TransformNode>;
        /**
         * Prevents the World matrix to be computed any longer
         * @param newWorldMatrix defines an optional matrix to use as world matrix
         * @param decompose defines whether to decompose the given newWorldMatrix or directly assign
         * @returns the TransformNode.
         */
        freezeWorldMatrix(newWorldMatrix?: Nullable<Matrix>, decompose?: boolean): TransformNode;
        /**
         * Allows back the World matrix computation.
         * @returns the TransformNode.
         */
        unfreezeWorldMatrix(): this;
        /**
         * True if the World matrix has been frozen.
         */
        get isWorldMatrixFrozen(): boolean;
        /**
         * Returns the mesh absolute position in the World.
         * @returns a Vector3.
         */
        getAbsolutePosition(): Vector3;
        /**
         * Sets the mesh absolute position in the World from a Vector3 or an Array(3).
         * @param absolutePosition the absolute position to set
         * @returns the TransformNode.
         */
        setAbsolutePosition(absolutePosition: Vector3): TransformNode;
        /**
         * Sets the mesh position in its local space.
         * @param vector3 the position to set in localspace
         * @returns the TransformNode.
         */
        setPositionWithLocalVector(vector3: Vector3): TransformNode;
        /**
         * Returns the mesh position in the local space from the current World matrix values.
         * @returns a new Vector3.
         */
        getPositionExpressedInLocalSpace(): Vector3;
        /**
         * Translates the mesh along the passed Vector3 in its local space.
         * @param vector3 the distance to translate in localspace
         * @returns the TransformNode.
         */
        locallyTranslate(vector3: Vector3): TransformNode;
        private static _LookAtVectorCache;
        /**
         * Orients a mesh towards a target point. Mesh must be drawn facing user.
         * @param targetPoint the position (must be in same space as current mesh) to look at
         * @param yawCor optional yaw (y-axis) correction in radians
         * @param pitchCor optional pitch (x-axis) correction in radians
         * @param rollCor optional roll (z-axis) correction in radians
         * @param space the chosen space of the target
         * @returns the TransformNode.
         */
        lookAt(targetPoint: Vector3, yawCor?: number, pitchCor?: number, rollCor?: number, space?: Space): TransformNode;
        /**
         * Returns a new Vector3 that is the localAxis, expressed in the mesh local space, rotated like the mesh.
         * This Vector3 is expressed in the World space.
         * @param localAxis axis to rotate
         * @returns a new Vector3 that is the localAxis, expressed in the mesh local space, rotated like the mesh.
         */
        getDirection(localAxis: Vector3): Vector3;
        /**
         * Sets the Vector3 "result" as the rotated Vector3 "localAxis" in the same rotation than the mesh.
         * localAxis is expressed in the mesh local space.
         * result is computed in the World space from the mesh World matrix.
         * @param localAxis axis to rotate
         * @param result the resulting transformnode
         * @returns this TransformNode.
         */
        getDirectionToRef(localAxis: Vector3, result: Vector3): TransformNode;
        /**
         * Sets this transform node rotation to the given local axis.
         * @param localAxis the axis in local space
         * @param yawCor optional yaw (y-axis) correction in radians
         * @param pitchCor optional pitch (x-axis) correction in radians
         * @param rollCor optional roll (z-axis) correction in radians
         * @returns this TransformNode
         */
        setDirection(localAxis: Vector3, yawCor?: number, pitchCor?: number, rollCor?: number): TransformNode;
        /**
         * Sets a new pivot point to the current node
         * @param point defines the new pivot point to use
         * @param space defines if the point is in world or local space (local by default)
         * @returns the current TransformNode
         */
        setPivotPoint(point: Vector3, space?: Space): TransformNode;
        /**
         * Returns a new Vector3 set with the mesh pivot point coordinates in the local space.
         * @returns the pivot point
         */
        getPivotPoint(): Vector3;
        /**
         * Sets the passed Vector3 "result" with the coordinates of the mesh pivot point in the local space.
         * @param result the vector3 to store the result
         * @returns this TransformNode.
         */
        getPivotPointToRef(result: Vector3): TransformNode;
        /**
         * Returns a new Vector3 set with the mesh pivot point World coordinates.
         * @returns a new Vector3 set with the mesh pivot point World coordinates.
         */
        getAbsolutePivotPoint(): Vector3;
        /**
         * Sets the Vector3 "result" coordinates with the mesh pivot point World coordinates.
         * @param result vector3 to store the result
         * @returns this TransformNode.
         */
        getAbsolutePivotPointToRef(result: Vector3): TransformNode;
        /**
         * Flag the transform node as dirty (Forcing it to update everything)
         * @param property if set to "rotation" the objects rotationQuaternion will be set to null
         * @returns this  node
         */
        markAsDirty(property?: string): Node;
        /**
         * Defines the passed node as the parent of the current node.
         * The node will remain exactly where it is and its position / rotation will be updated accordingly.
         * Note that if the mesh has a pivot matrix / point defined it will be applied after the parent was updated.
         * In that case the node will not remain in the same space as it is, as the pivot will be applied.
         * To avoid this, you can set updatePivot to true and the pivot will be updated to identity
         * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/transforms/parent_pivot/parent
         * @param node the node ot set as the parent
         * @param preserveScalingSign if true, keep scaling sign of child. Otherwise, scaling sign might change.
         * @param updatePivot if true, update the pivot matrix to keep the node in the same space as before
         * @returns this TransformNode.
         */
        setParent(node: Nullable<Node>, preserveScalingSign?: boolean, updatePivot?: boolean): TransformNode;
        private _nonUniformScaling;
        /**
         * True if the scaling property of this object is non uniform eg. (1,2,1)
         */
        get nonUniformScaling(): boolean;
        /**
         * @internal
         */
        _updateNonUniformScalingState(value: boolean): boolean;
        /**
         * Attach the current TransformNode to another TransformNode associated with a bone
         * @param bone Bone affecting the TransformNode
         * @param affectedTransformNode TransformNode associated with the bone
         * @returns this object
         */
        attachToBone(bone: Bone, affectedTransformNode: TransformNode): TransformNode;
        /**
         * Detach the transform node if its associated with a bone
         * @param resetToPreviousParent Indicates if the parent that was in effect when attachToBone was called should be set back or if we should set parent to null instead (defaults to the latter)
         * @returns this object
         */
        detachFromBone(resetToPreviousParent?: boolean): TransformNode;
        private static _RotationAxisCache;
        /**
         * Rotates the mesh around the axis vector for the passed angle (amount) expressed in radians, in the given space.
         * space (default LOCAL) can be either Space.LOCAL, either Space.WORLD.
         * Note that the property `rotationQuaternion` is then automatically updated and the property `rotation` is set to (0,0,0) and no longer used.
         * The passed axis is also normalized.
         * @param axis the axis to rotate around
         * @param amount the amount to rotate in radians
         * @param space Space to rotate in (Default: local)
         * @returns the TransformNode.
         */
        rotate(axis: Vector3, amount: number, space?: Space): TransformNode;
        /**
         * Rotates the mesh around the axis vector for the passed angle (amount) expressed in radians, in world space.
         * Note that the property `rotationQuaternion` is then automatically updated and the property `rotation` is set to (0,0,0) and no longer used.
         * The passed axis is also normalized. .
         * Method is based on http://www.euclideanspace.com/maths/geometry/affine/aroundPoint/index.htm
         * @param point the point to rotate around
         * @param axis the axis to rotate around
         * @param amount the amount to rotate in radians
         * @returns the TransformNode
         */
        rotateAround(point: Vector3, axis: Vector3, amount: number): TransformNode;
        /**
         * Translates the mesh along the axis vector for the passed distance in the given space.
         * space (default LOCAL) can be either Space.LOCAL, either Space.WORLD.
         * @param axis the axis to translate in
         * @param distance the distance to translate
         * @param space Space to rotate in (Default: local)
         * @returns the TransformNode.
         */
        translate(axis: Vector3, distance: number, space?: Space): TransformNode;
        /**
         * Adds a rotation step to the mesh current rotation.
         * x, y, z are Euler angles expressed in radians.
         * This methods updates the current mesh rotation, either mesh.rotation, either mesh.rotationQuaternion if it's set.
         * This means this rotation is made in the mesh local space only.
         * It's useful to set a custom rotation order different from the BJS standard one YXZ.
         * Example : this rotates the mesh first around its local X axis, then around its local Z axis, finally around its local Y axis.
         * ```javascript
         * mesh.addRotation(x1, 0, 0).addRotation(0, 0, z2).addRotation(0, 0, y3);
         * ```
         * Note that `addRotation()` accumulates the passed rotation values to the current ones and computes the .rotation or .rotationQuaternion updated values.
         * Under the hood, only quaternions are used. So it's a little faster is you use .rotationQuaternion because it doesn't need to translate them back to Euler angles.
         * @param x Rotation to add
         * @param y Rotation to add
         * @param z Rotation to add
         * @returns the TransformNode.
         */
        addRotation(x: number, y: number, z: number): TransformNode;
        /**
         * @internal
         */
        protected _getEffectiveParent(): Nullable<Node>;
        /**
         * Returns whether the transform node world matrix computation needs the camera information to be computed.
         * This is the case when the node is a billboard or has an infinite distance for instance.
         * @returns true if the world matrix computation needs the camera information to be computed
         */
        isWorldMatrixCameraDependent(): boolean;
        /**
         * Computes the world matrix of the node
         * @param force defines if the cache version should be invalidated forcing the world matrix to be created from scratch
         * @param camera defines the camera used if different from the scene active camera (This is used with modes like Billboard or infinite distance)
         * @returns the world matrix
         */
        computeWorldMatrix(force?: boolean, camera?: Nullable<Camera>): Matrix;
        /**
         * Resets this nodeTransform's local matrix to Matrix.Identity().
         * @param independentOfChildren indicates if all child nodeTransform's world-space transform should be preserved.
         */
        resetLocalMatrix(independentOfChildren?: boolean): void;
        protected _afterComputeWorldMatrix(): void;
        /**
         * If you'd like to be called back after the mesh position, rotation or scaling has been updated.
         * @param func callback function to add
         *
         * @returns the TransformNode.
         */
        registerAfterWorldMatrixUpdate(func: (mesh: TransformNode) => void): TransformNode;
        /**
         * Removes a registered callback function.
         * @param func callback function to remove
         * @returns the TransformNode.
         */
        unregisterAfterWorldMatrixUpdate(func: (mesh: TransformNode) => void): TransformNode;
        /**
         * Gets the position of the current mesh in camera space
         * @param camera defines the camera to use
         * @returns a position
         */
        getPositionInCameraSpace(camera?: Nullable<Camera>): Vector3;
        /**
         * Returns the distance from the mesh to the active camera
         * @param camera defines the camera to use
         * @returns the distance
         */
        getDistanceToCamera(camera?: Nullable<Camera>): number;
        /**
         * Clone the current transform node
         * @param name Name of the new clone
         * @param newParent New parent for the clone
         * @param doNotCloneChildren Do not clone children hierarchy
         * @returns the new transform node
         */
        clone(name: string, newParent: Nullable<Node>, doNotCloneChildren?: boolean): Nullable<TransformNode>;
        /**
         * Serializes the objects information.
         * @param currentSerializationObject defines the object to serialize in
         * @returns the serialized object
         */
        serialize(currentSerializationObject?: any): any;
        /**
         * Returns a new TransformNode object parsed from the source provided.
         * @param parsedTransformNode is the source.
         * @param scene the scene the object belongs to
         * @param rootUrl is a string, it's the root URL to prefix the `delayLoadingFile` property with
         * @returns a new TransformNode object parsed from the source provided.
         */
        static Parse(parsedTransformNode: any, scene: Scene, rootUrl: string): TransformNode;
        /**
         * Get all child-transformNodes of this node
         * @param directDescendantsOnly defines if true only direct descendants of 'this' will be considered, if false direct and also indirect (children of children, an so on in a recursive manner) descendants of 'this' will be considered
         * @param predicate defines an optional predicate that will be called on every evaluated child, the predicate must return true for a given child to be part of the result, otherwise it will be ignored
         * @returns an array of TransformNode
         */
        getChildTransformNodes(directDescendantsOnly?: boolean, predicate?: (node: Node) => boolean): TransformNode[];
        /**
         * Releases resources associated with this transform node.
         * @param doNotRecurse Set to true to not recurse into each children (recurse into each children by default)
         * @param disposeMaterialAndTextures Set to true to also dispose referenced materials and textures (false by default)
         */
        dispose(doNotRecurse?: boolean, disposeMaterialAndTextures?: boolean): void;
        /**
         * Uniformly scales the mesh to fit inside of a unit cube (1 X 1 X 1 units)
         * @param includeDescendants Use the hierarchy's bounding box instead of the mesh's bounding box. Default is false
         * @param ignoreRotation ignore rotation when computing the scale (ie. object will be axis aligned). Default is false
         * @param predicate predicate that is passed in to getHierarchyBoundingVectors when selecting which object should be included when scaling
         * @returns the current mesh
         */
        normalizeToUnitCube(includeDescendants?: boolean, ignoreRotation?: boolean, predicate?: Nullable<(node: AbstractMesh) => boolean>): TransformNode;
        private _syncAbsoluteScalingAndRotation;
    }


    /** @internal */
    export class WebGLDataBuffer extends DataBuffer {
        private _buffer;
        constructor(resource: WebGLBuffer);
        get underlyingResource(): any;
    }


    /** @internal */
    export class WebGPUDataBuffer extends DataBuffer {
        private _buffer;
        constructor(resource: GPUBuffer);
        get underlyingResource(): any;
    }


    /**
     * Class used to evaluate queries containing `and` and `or` operators
     */
    export class AndOrNotEvaluator {
        /**
         * Evaluate a query
         * @param query defines the query to evaluate
         * @param evaluateCallback defines the callback used to filter result
         * @returns true if the query matches
         */
        static Eval(query: string, evaluateCallback: (val: any) => boolean): boolean;
        private static _HandleParenthesisContent;
        private static _SimplifyNegation;
    }


    /** @internal */
    interface TupleTypes<T> {
        2: [T, T];
        3: [T, T, T];
        4: [T, T, T, T];
        5: [T, T, T, T, T];
        6: [T, T, T, T, T, T];
        7: [T, T, T, T, T, T, T];
        8: [T, T, T, T, T, T, T, T];
        9: [T, T, T, T, T, T, T, T, T];
        10: [T, T, T, T, T, T, T, T, T, T];
        11: [T, T, T, T, T, T, T, T, T, T, T];
        12: [T, T, T, T, T, T, T, T, T, T, T, T];
        13: [T, T, T, T, T, T, T, T, T, T, T, T, T];
        14: [T, T, T, T, T, T, T, T, T, T, T, T, T, T];
        15: [T, T, T, T, T, T, T, T, T, T, T, T, T, T, T];
    }
    /**
     * Class containing a set of static utilities functions for arrays.
     */
    export class ArrayTools {
        /**
         * Returns an array of the given size filled with elements built from the given constructor and the parameters.
         * @param size the number of element to construct and put in the array.
         * @param itemBuilder a callback responsible for creating new instance of item. Called once per array entry.
         * @returns a new array filled with new objects.
         */
        static BuildArray<T>(size: number, itemBuilder: () => T): Array<T>;
        /**
         * Returns a tuple of the given size filled with elements built from the given constructor and the parameters.
         * @param size he number of element to construct and put in the tuple.
         * @param itemBuilder a callback responsible for creating new instance of item. Called once per tuple entry.
         * @returns a new tuple filled with new objects.
         */
        static BuildTuple<T, N extends keyof TupleTypes<unknown>>(size: N, itemBuilder: () => T): TupleTypes<T>[N];
    }
    /**
     * Defines the callback type used when an observed array function is triggered.
     * @internal
     */
    export type _ObserveCallback = (functionName: string, previousLength: number) => void;
    /**
     * Observes an array and notifies the given observer when the array is modified.
     * @param array Defines the array to observe
     * @param callback Defines the function to call when the array is modified (in the limit of the observed array functions)
     * @returns A function to call to stop observing the array
     * @internal
     */
    export function _ObserveArray<T>(array: T[], callback: _ObserveCallback): () => void;


    /**
     * Defines the list of states available for a task inside a AssetsManager
     */
    export enum AssetTaskState {
        /**
         * Initialization
         */
        INIT = 0,
        /**
         * Running
         */
        RUNNING = 1,
        /**
         * Done
         */
        DONE = 2,
        /**
         * Error
         */
        ERROR = 3
    }
    /**
     * Define an abstract asset task used with a AssetsManager class to load assets into a scene
     */
    export abstract class AbstractAssetTask {
        /**
         * Task name
         */ name: string;
        /**
         * Callback called when the task is successful
         */
        onSuccess: (task: any) => void;
        /**
         * Callback called when the task is not successful
         */
        onError: (task: any, message?: string, exception?: any) => void;
        /**
         * Creates a new AssetsManager
         * @param name defines the name of the task
         */
        constructor(
        /**
         * Task name
         */ name: string);
        private _isCompleted;
        private _taskState;
        private _errorObject;
        /**
         * Get if the task is completed
         */
        get isCompleted(): boolean;
        /**
         * Gets the current state of the task
         */
        get taskState(): AssetTaskState;
        /**
         * Gets the current error object (if task is in error)
         */
        get errorObject(): {
            message?: string;
            exception?: any;
        };
        /**
         * Internal only
         * @internal
         */
        _setErrorObject(message?: string, exception?: any): void;
        /**
         * Execute the current task
         * @param scene defines the scene where you want your assets to be loaded
         * @param onSuccess is a callback called when the task is successfully executed
         * @param onError is a callback called if an error occurs
         */
        run(scene: Scene, onSuccess: () => void, onError: (message?: string, exception?: any) => void): void;
        /**
         * Execute the current task
         * @param scene defines the scene where you want your assets to be loaded
         * @param onSuccess is a callback called when the task is successfully executed
         * @param onError is a callback called if an error occurs
         */
        runTask(scene: Scene, onSuccess: () => void, onError: (message?: string, exception?: any) => void): void;
        /**
         * Reset will set the task state back to INIT, so the next load call of the assets manager will execute this task again.
         * This can be used with failed tasks that have the reason for failure fixed.
         */
        reset(): void;
        private _onErrorCallback;
        private _onDoneCallback;
    }
    /**
     * Define the interface used by progress events raised during assets loading
     */
    export interface IAssetsProgressEvent {
        /**
         * Defines the number of remaining tasks to process
         */
        remainingCount: number;
        /**
         * Defines the total number of tasks
         */
        totalCount: number;
        /**
         * Defines the task that was just processed
         */
        task: AbstractAssetTask;
    }
    /**
     * Class used to share progress information about assets loading
     */
    export class AssetsProgressEvent implements IAssetsProgressEvent {
        /**
         * Defines the number of remaining tasks to process
         */
        remainingCount: number;
        /**
         * Defines the total number of tasks
         */
        totalCount: number;
        /**
         * Defines the task that was just processed
         */
        task: AbstractAssetTask;
        /**
         * Creates a AssetsProgressEvent
         * @param remainingCount defines the number of remaining tasks to process
         * @param totalCount defines the total number of tasks
         * @param task defines the task that was just processed
         */
        constructor(remainingCount: number, totalCount: number, task: AbstractAssetTask);
    }
    /**
     * Define a task used by AssetsManager to load assets into a container
     */
    export class ContainerAssetTask extends AbstractAssetTask {
        /**
         * Defines the name of the task
         */
        name: string;
        /**
         * Defines the list of mesh's names you want to load
         */
        meshesNames: any;
        /**
         * Defines the root url to use as a base to load your meshes and associated resources
         */
        rootUrl: string;
        /**
         * Defines the filename or File of the scene to load from
         */
        sceneFilename: string | File;
        /**
         * Defines the extension to use to load the scene (if not defined, ".babylon" will be used)
         */
        extension?: string | undefined;
        /**
         * Get the loaded asset container
         */
        loadedContainer: AssetContainer;
        /**
         * Gets the list of loaded transforms
         */
        loadedTransformNodes: Array<TransformNode>;
        /**
         * Gets the list of loaded meshes
         */
        loadedMeshes: Array<AbstractMesh>;
        /**
         * Gets the list of loaded particle systems
         */
        loadedParticleSystems: Array<IParticleSystem>;
        /**
         * Gets the list of loaded skeletons
         */
        loadedSkeletons: Array<Skeleton>;
        /**
         * Gets the list of loaded animation groups
         */
        loadedAnimationGroups: Array<AnimationGroup>;
        /**
         * Callback called when the task is successful
         */
        onSuccess: (task: ContainerAssetTask) => void;
        /**
         * Callback called when the task is successful
         */
        onError: (task: ContainerAssetTask, message?: string, exception?: any) => void;
        /**
         * Creates a new ContainerAssetTask
         * @param name defines the name of the task
         * @param meshesNames defines the list of mesh's names you want to load
         * @param rootUrl defines the root url to use as a base to load your meshes and associated resources
         * @param sceneFilename defines the filename or File of the scene to load from
         */
        constructor(
        /**
         * Defines the name of the task
         */
        name: string, 
        /**
         * Defines the list of mesh's names you want to load
         */
        meshesNames: any, 
        /**
         * Defines the root url to use as a base to load your meshes and associated resources
         */
        rootUrl: string, 
        /**
         * Defines the filename or File of the scene to load from
         */
        sceneFilename: string | File, 
        /**
         * Defines the extension to use to load the scene (if not defined, ".babylon" will be used)
         */
        extension?: string | undefined);
        /**
         * Execute the current task
         * @param scene defines the scene where you want your assets to be loaded
         * @param onSuccess is a callback called when the task is successfully executed
         * @param onError is a callback called if an error occurs
         */
        runTask(scene: Scene, onSuccess: () => void, onError: (message?: string, exception?: any) => void): void;
    }
    /**
     * Define a task used by AssetsManager to load meshes
     */
    export class MeshAssetTask extends AbstractAssetTask {
        /**
         * Defines the name of the task
         */
        name: string;
        /**
         * Defines the list of mesh's names you want to load
         */
        meshesNames: any;
        /**
         * Defines the root url to use as a base to load your meshes and associated resources
         */
        rootUrl: string;
        /**
         * Defines the filename or File of the scene to load from
         */
        sceneFilename: string | File;
        /**
         * Defines the extension to use to load the scene (if not defined, ".babylon" will be used)
         */
        extension?: string | undefined;
        /**
         * Gets the list of loaded transforms
         */
        loadedTransformNodes: Array<TransformNode>;
        /**
         * Gets the list of loaded meshes
         */
        loadedMeshes: Array<AbstractMesh>;
        /**
         * Gets the list of loaded particle systems
         */
        loadedParticleSystems: Array<IParticleSystem>;
        /**
         * Gets the list of loaded skeletons
         */
        loadedSkeletons: Array<Skeleton>;
        /**
         * Gets the list of loaded animation groups
         */
        loadedAnimationGroups: Array<AnimationGroup>;
        /**
         * Callback called when the task is successful
         */
        onSuccess: (task: MeshAssetTask) => void;
        /**
         * Callback called when the task is successful
         */
        onError: (task: MeshAssetTask, message?: string, exception?: any) => void;
        /**
         * Creates a new MeshAssetTask
         * @param name defines the name of the task
         * @param meshesNames defines the list of mesh's names you want to load
         * @param rootUrl defines the root url to use as a base to load your meshes and associated resources
         * @param sceneFilename defines the filename or File of the scene to load from
         */
        constructor(
        /**
         * Defines the name of the task
         */
        name: string, 
        /**
         * Defines the list of mesh's names you want to load
         */
        meshesNames: any, 
        /**
         * Defines the root url to use as a base to load your meshes and associated resources
         */
        rootUrl: string, 
        /**
         * Defines the filename or File of the scene to load from
         */
        sceneFilename: string | File, 
        /**
         * Defines the extension to use to load the scene (if not defined, ".babylon" will be used)
         */
        extension?: string | undefined);
        /**
         * Execute the current task
         * @param scene defines the scene where you want your assets to be loaded
         * @param onSuccess is a callback called when the task is successfully executed
         * @param onError is a callback called if an error occurs
         */
        runTask(scene: Scene, onSuccess: () => void, onError: (message?: string, exception?: any) => void): void;
    }
    /**
     * Define a task used by AssetsManager to load text content
     */
    export class TextFileAssetTask extends AbstractAssetTask {
        /**
         * Defines the name of the task
         */
        name: string;
        /**
         * Defines the location of the file to load
         */
        url: string;
        /**
         * Gets the loaded text string
         */
        text: string;
        /**
         * Callback called when the task is successful
         */
        onSuccess: (task: TextFileAssetTask) => void;
        /**
         * Callback called when the task is successful
         */
        onError: (task: TextFileAssetTask, message?: string, exception?: any) => void;
        /**
         * Creates a new TextFileAssetTask object
         * @param name defines the name of the task
         * @param url defines the location of the file to load
         */
        constructor(
        /**
         * Defines the name of the task
         */
        name: string, 
        /**
         * Defines the location of the file to load
         */
        url: string);
        /**
         * Execute the current task
         * @param scene defines the scene where you want your assets to be loaded
         * @param onSuccess is a callback called when the task is successfully executed
         * @param onError is a callback called if an error occurs
         */
        runTask(scene: Scene, onSuccess: () => void, onError: (message?: string, exception?: any) => void): void;
    }
    /**
     * Define a task used by AssetsManager to load binary data
     */
    export class BinaryFileAssetTask extends AbstractAssetTask {
        /**
         * Defines the name of the task
         */
        name: string;
        /**
         * Defines the location of the file to load
         */
        url: string;
        /**
         * Gets the loaded data (as an array buffer)
         */
        data: ArrayBuffer;
        /**
         * Callback called when the task is successful
         */
        onSuccess: (task: BinaryFileAssetTask) => void;
        /**
         * Callback called when the task is successful
         */
        onError: (task: BinaryFileAssetTask, message?: string, exception?: any) => void;
        /**
         * Creates a new BinaryFileAssetTask object
         * @param name defines the name of the new task
         * @param url defines the location of the file to load
         */
        constructor(
        /**
         * Defines the name of the task
         */
        name: string, 
        /**
         * Defines the location of the file to load
         */
        url: string);
        /**
         * Execute the current task
         * @param scene defines the scene where you want your assets to be loaded
         * @param onSuccess is a callback called when the task is successfully executed
         * @param onError is a callback called if an error occurs
         */
        runTask(scene: Scene, onSuccess: () => void, onError: (message?: string, exception?: any) => void): void;
    }
    /**
     * Define a task used by AssetsManager to load images
     */
    export class ImageAssetTask extends AbstractAssetTask {
        /**
         * Defines the name of the task
         */
        name: string;
        /**
         * Defines the location of the image to load
         */
        url: string;
        /**
         * Gets the loaded images
         */
        image: HTMLImageElement;
        /**
         * Callback called when the task is successful
         */
        onSuccess: (task: ImageAssetTask) => void;
        /**
         * Callback called when the task is successful
         */
        onError: (task: ImageAssetTask, message?: string, exception?: any) => void;
        /**
         * Creates a new ImageAssetTask
         * @param name defines the name of the task
         * @param url defines the location of the image to load
         */
        constructor(
        /**
         * Defines the name of the task
         */
        name: string, 
        /**
         * Defines the location of the image to load
         */
        url: string);
        /**
         * Execute the current task
         * @param scene defines the scene where you want your assets to be loaded
         * @param onSuccess is a callback called when the task is successfully executed
         * @param onError is a callback called if an error occurs
         */
        runTask(scene: Scene, onSuccess: () => void, onError: (message?: string, exception?: any) => void): void;
    }
    /**
     * Defines the interface used by texture loading tasks
     */
    export interface ITextureAssetTask<TEX extends BaseTexture> {
        /**
         * Gets the loaded texture
         */
        texture: TEX;
    }
    /**
     * Define a task used by AssetsManager to load 2D textures
     */
    export class TextureAssetTask extends AbstractAssetTask implements ITextureAssetTask<Texture> {
        /**
         * Defines the name of the task
         */
        name: string;
        /**
         * Defines the location of the file to load
         */
        url: string;
        /**
         * Defines if mipmap should not be generated (default is false)
         */
        noMipmap?: boolean | undefined;
        /**
         * Defines if texture must be inverted on Y axis (default is true)
         */
        invertY: boolean;
        /**
         * Defines the sampling mode to use (default is Texture.TRILINEAR_SAMPLINGMODE)
         */
        samplingMode: number;
        /**
         * Gets the loaded texture
         */
        texture: Texture;
        /**
         * Callback called when the task is successful
         */
        onSuccess: (task: TextureAssetTask) => void;
        /**
         * Callback called when the task is successful
         */
        onError: (task: TextureAssetTask, message?: string, exception?: any) => void;
        /**
         * Creates a new TextureAssetTask object
         * @param name defines the name of the task
         * @param url defines the location of the file to load
         * @param noMipmap defines if mipmap should not be generated (default is false)
         * @param invertY defines if texture must be inverted on Y axis (default is true)
         * @param samplingMode defines the sampling mode to use (default is Texture.TRILINEAR_SAMPLINGMODE)
         */
        constructor(
        /**
         * Defines the name of the task
         */
        name: string, 
        /**
         * Defines the location of the file to load
         */
        url: string, 
        /**
         * Defines if mipmap should not be generated (default is false)
         */
        noMipmap?: boolean | undefined, 
        /**
         * Defines if texture must be inverted on Y axis (default is true)
         */
        invertY?: boolean, 
        /**
         * Defines the sampling mode to use (default is Texture.TRILINEAR_SAMPLINGMODE)
         */
        samplingMode?: number);
        /**
         * Execute the current task
         * @param scene defines the scene where you want your assets to be loaded
         * @param onSuccess is a callback called when the task is successfully executed
         * @param onError is a callback called if an error occurs
         */
        runTask(scene: Scene, onSuccess: () => void, onError: (message?: string, exception?: any) => void): void;
    }
    /**
     * Define a task used by AssetsManager to load cube textures
     */
    export class CubeTextureAssetTask extends AbstractAssetTask implements ITextureAssetTask<CubeTexture> {
        /**
         * Defines the name of the task
         */
        name: string;
        /**
         * Defines the location of the files to load (You have to specify the folder where the files are + filename with no extension)
         */
        url: string;
        /**
         * Defines the extensions to use to load files (["_px", "_py", "_pz", "_nx", "_ny", "_nz"] by default)
         */
        extensions?: string[] | undefined;
        /**
         * Defines if mipmaps should not be generated (default is false)
         */
        noMipmap?: boolean | undefined;
        /**
         * Defines the explicit list of files (undefined by default)
         */
        files?: string[] | undefined;
        /**
         * Defines the prefiltered texture option (default is false)
         */
        prefiltered?: boolean | undefined;
        /**
         * Gets the loaded texture
         */
        texture: CubeTexture;
        /**
         * Callback called when the task is successful
         */
        onSuccess: (task: CubeTextureAssetTask) => void;
        /**
         * Callback called when the task is successful
         */
        onError: (task: CubeTextureAssetTask, message?: string, exception?: any) => void;
        /**
         * Creates a new CubeTextureAssetTask
         * @param name defines the name of the task
         * @param url defines the location of the files to load (You have to specify the folder where the files are + filename with no extension)
         * @param extensions defines the extensions to use to load files (["_px", "_py", "_pz", "_nx", "_ny", "_nz"] by default)
         * @param noMipmap defines if mipmaps should not be generated (default is false)
         * @param files defines the explicit list of files (undefined by default)
         * @param prefiltered
         */
        constructor(
        /**
         * Defines the name of the task
         */
        name: string, 
        /**
         * Defines the location of the files to load (You have to specify the folder where the files are + filename with no extension)
         */
        url: string, 
        /**
         * Defines the extensions to use to load files (["_px", "_py", "_pz", "_nx", "_ny", "_nz"] by default)
         */
        extensions?: string[] | undefined, 
        /**
         * Defines if mipmaps should not be generated (default is false)
         */
        noMipmap?: boolean | undefined, 
        /**
         * Defines the explicit list of files (undefined by default)
         */
        files?: string[] | undefined, 
        /**
         * Defines the prefiltered texture option (default is false)
         */
        prefiltered?: boolean | undefined);
        /**
         * Execute the current task
         * @param scene defines the scene where you want your assets to be loaded
         * @param onSuccess is a callback called when the task is successfully executed
         * @param onError is a callback called if an error occurs
         */
        runTask(scene: Scene, onSuccess: () => void, onError: (message?: string, exception?: any) => void): void;
    }
    /**
     * Define a task used by AssetsManager to load HDR cube textures
     */
    export class HDRCubeTextureAssetTask extends AbstractAssetTask implements ITextureAssetTask<HDRCubeTexture> {
        /**
         * Defines the name of the task
         */
        name: string;
        /**
         * Defines the location of the file to load
         */
        url: string;
        /**
         * Defines the desired size (the more it increases the longer the generation will be)
         */
        size: number;
        /**
         * Defines if mipmaps should not be generated (default is false)
         */
        noMipmap: boolean;
        /**
         * Specifies whether you want to extract the polynomial harmonics during the generation process (default is true)
         */
        generateHarmonics: boolean;
        /**
         * Specifies if the texture will be use in gamma or linear space (the PBR material requires those texture in linear space, but the standard material would require them in Gamma space) (default is false)
         */
        gammaSpace: boolean;
        /**
         * Internal Use Only
         */
        reserved: boolean;
        /**
         * Gets the loaded texture
         */
        texture: HDRCubeTexture;
        /**
         * Callback called when the task is successful
         */
        onSuccess: (task: HDRCubeTextureAssetTask) => void;
        /**
         * Callback called when the task is successful
         */
        onError: (task: HDRCubeTextureAssetTask, message?: string, exception?: any) => void;
        /**
         * Creates a new HDRCubeTextureAssetTask object
         * @param name defines the name of the task
         * @param url defines the location of the file to load
         * @param size defines the desired size (the more it increases the longer the generation will be) If the size is omitted this implies you are using a preprocessed cubemap.
         * @param noMipmap defines if mipmaps should not be generated (default is false)
         * @param generateHarmonics specifies whether you want to extract the polynomial harmonics during the generation process (default is true)
         * @param gammaSpace specifies if the texture will be use in gamma or linear space (the PBR material requires those texture in linear space, but the standard material would require them in Gamma space) (default is false)
         * @param reserved Internal use only
         */
        constructor(
        /**
         * Defines the name of the task
         */
        name: string, 
        /**
         * Defines the location of the file to load
         */
        url: string, 
        /**
         * Defines the desired size (the more it increases the longer the generation will be)
         */
        size: number, 
        /**
         * Defines if mipmaps should not be generated (default is false)
         */
        noMipmap?: boolean, 
        /**
         * Specifies whether you want to extract the polynomial harmonics during the generation process (default is true)
         */
        generateHarmonics?: boolean, 
        /**
         * Specifies if the texture will be use in gamma or linear space (the PBR material requires those texture in linear space, but the standard material would require them in Gamma space) (default is false)
         */
        gammaSpace?: boolean, 
        /**
         * Internal Use Only
         */
        reserved?: boolean);
        /**
         * Execute the current task
         * @param scene defines the scene where you want your assets to be loaded
         * @param onSuccess is a callback called when the task is successfully executed
         * @param onError is a callback called if an error occurs
         */
        runTask(scene: Scene, onSuccess: () => void, onError: (message?: string, exception?: any) => void): void;
    }
    /**
     * Define a task used by AssetsManager to load Equirectangular cube textures
     */
    export class EquiRectangularCubeTextureAssetTask extends AbstractAssetTask implements ITextureAssetTask<EquiRectangularCubeTexture> {
        /**
         * Defines the name of the task
         */
        name: string;
        /**
         * Defines the location of the file to load
         */
        url: string;
        /**
         * Defines the desired size (the more it increases the longer the generation will be)
         */
        size: number;
        /**
         * Defines if mipmaps should not be generated (default is false)
         */
        noMipmap: boolean;
        /**
         * Specifies if the texture will be use in gamma or linear space (the PBR material requires those texture in linear space,
         * but the standard material would require them in Gamma space) (default is true)
         */
        gammaSpace: boolean;
        /**
         * Gets the loaded texture
         */
        texture: EquiRectangularCubeTexture;
        /**
         * Callback called when the task is successful
         */
        onSuccess: (task: EquiRectangularCubeTextureAssetTask) => void;
        /**
         * Callback called when the task is successful
         */
        onError: (task: EquiRectangularCubeTextureAssetTask, message?: string, exception?: any) => void;
        /**
         * Creates a new EquiRectangularCubeTextureAssetTask object
         * @param name defines the name of the task
         * @param url defines the location of the file to load
         * @param size defines the desired size (the more it increases the longer the generation will be)
         * If the size is omitted this implies you are using a preprocessed cubemap.
         * @param noMipmap defines if mipmaps should not be generated (default is false)
         * @param gammaSpace specifies if the texture will be used in gamma or linear space
         * (the PBR material requires those texture in linear space, but the standard material would require them in Gamma space)
         * (default is true)
         */
        constructor(
        /**
         * Defines the name of the task
         */
        name: string, 
        /**
         * Defines the location of the file to load
         */
        url: string, 
        /**
         * Defines the desired size (the more it increases the longer the generation will be)
         */
        size: number, 
        /**
         * Defines if mipmaps should not be generated (default is false)
         */
        noMipmap?: boolean, 
        /**
         * Specifies if the texture will be use in gamma or linear space (the PBR material requires those texture in linear space,
         * but the standard material would require them in Gamma space) (default is true)
         */
        gammaSpace?: boolean);
        /**
         * Execute the current task
         * @param scene defines the scene where you want your assets to be loaded
         * @param onSuccess is a callback called when the task is successfully executed
         * @param onError is a callback called if an error occurs
         */
        runTask(scene: Scene, onSuccess: () => void, onError: (message?: string, exception?: any) => void): void;
    }
    /**
     * This class can be used to easily import assets into a scene
     * @see https://doc.babylonjs.com/features/featuresDeepDive/importers/assetManager
     */
    export class AssetsManager {
        private _scene;
        private _isLoading;
        protected _tasks: AbstractAssetTask[];
        protected _waitingTasksCount: number;
        protected _totalTasksCount: number;
        /**
         * Callback called when all tasks are processed
         */
        onFinish: (tasks: AbstractAssetTask[]) => void;
        /**
         * Callback called when a task is successful
         */
        onTaskSuccess: (task: AbstractAssetTask) => void;
        /**
         * Callback called when a task had an error
         */
        onTaskError: (task: AbstractAssetTask) => void;
        /**
         * Callback called when a task is done (whatever the result is)
         */
        onProgress: (remainingCount: number, totalCount: number, task: AbstractAssetTask) => void;
        /**
         * Observable called when all tasks are processed
         */
        onTaskSuccessObservable: Observable<AbstractAssetTask>;
        /**
         * Observable called when a task had an error
         */
        onTaskErrorObservable: Observable<AbstractAssetTask>;
        /**
         * Observable called when all tasks were executed
         */
        onTasksDoneObservable: Observable<AbstractAssetTask[]>;
        /**
         * Observable called when a task is done (whatever the result is)
         */
        onProgressObservable: Observable<IAssetsProgressEvent>;
        /**
         * Gets or sets a boolean defining if the AssetsManager should use the default loading screen
         * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/customLoadingScreen
         */
        useDefaultLoadingScreen: boolean;
        /**
         * Gets or sets a boolean defining if the AssetsManager should automatically hide the loading screen
         * when all assets have been downloaded.
         * If set to false, you need to manually call in hideLoadingUI() once your scene is ready.
         */
        autoHideLoadingUI: boolean;
        /**
         * Creates a new AssetsManager
         * @param scene defines the scene to work on
         */
        constructor(scene?: Scene);
        /**
         * Add a ContainerAssetTask to the list of active tasks
         * @param taskName defines the name of the new task
         * @param meshesNames defines the name of meshes to load
         * @param rootUrl defines the root url to use to locate files
         * @param sceneFilename defines the filename of the scene file or the File itself
         * @param extension defines the extension to use to load the file
         * @returns a new ContainerAssetTask object
         */
        addContainerTask(taskName: string, meshesNames: any, rootUrl: string, sceneFilename: string | File, extension?: string): ContainerAssetTask;
        /**
         * Add a MeshAssetTask to the list of active tasks
         * @param taskName defines the name of the new task
         * @param meshesNames defines the name of meshes to load
         * @param rootUrl defines the root url to use to locate files
         * @param sceneFilename defines the filename of the scene file or the File itself
         * @param extension defines the extension to use to load the file
         * @returns a new MeshAssetTask object
         */
        addMeshTask(taskName: string, meshesNames: any, rootUrl: string, sceneFilename: string | File, extension?: string): MeshAssetTask;
        /**
         * Add a TextFileAssetTask to the list of active tasks
         * @param taskName defines the name of the new task
         * @param url defines the url of the file to load
         * @returns a new TextFileAssetTask object
         */
        addTextFileTask(taskName: string, url: string): TextFileAssetTask;
        /**
         * Add a BinaryFileAssetTask to the list of active tasks
         * @param taskName defines the name of the new task
         * @param url defines the url of the file to load
         * @returns a new BinaryFileAssetTask object
         */
        addBinaryFileTask(taskName: string, url: string): BinaryFileAssetTask;
        /**
         * Add a ImageAssetTask to the list of active tasks
         * @param taskName defines the name of the new task
         * @param url defines the url of the file to load
         * @returns a new ImageAssetTask object
         */
        addImageTask(taskName: string, url: string): ImageAssetTask;
        /**
         * Add a TextureAssetTask to the list of active tasks
         * @param taskName defines the name of the new task
         * @param url defines the url of the file to load
         * @param noMipmap defines if the texture must not receive mipmaps (false by default)
         * @param invertY defines if you want to invert Y axis of the loaded texture (true by default)
         * @param samplingMode defines the sampling mode to use (Texture.TRILINEAR_SAMPLINGMODE by default)
         * @returns a new TextureAssetTask object
         */
        addTextureTask(taskName: string, url: string, noMipmap?: boolean, invertY?: boolean, samplingMode?: number): TextureAssetTask;
        /**
         * Add a CubeTextureAssetTask to the list of active tasks
         * @param taskName defines the name of the new task
         * @param url defines the url of the file to load
         * @param extensions defines the extension to use to load the cube map (can be null)
         * @param noMipmap defines if the texture must not receive mipmaps (false by default)
         * @param files defines the list of files to load (can be null)
         * @param prefiltered defines the prefiltered texture option (default is false)
         * @returns a new CubeTextureAssetTask object
         */
        addCubeTextureTask(taskName: string, url: string, extensions?: string[], noMipmap?: boolean, files?: string[], prefiltered?: boolean): CubeTextureAssetTask;
        /**
         *
         * Add a HDRCubeTextureAssetTask to the list of active tasks
         * @param taskName defines the name of the new task
         * @param url defines the url of the file to load
         * @param size defines the size you want for the cubemap (can be null)
         * @param noMipmap defines if the texture must not receive mipmaps (false by default)
         * @param generateHarmonics defines if you want to automatically generate (true by default)
         * @param gammaSpace specifies if the texture will be use in gamma or linear space (the PBR material requires those texture in linear space, but the standard material would require them in Gamma space) (default is false)
         * @param reserved Internal use only
         * @returns a new HDRCubeTextureAssetTask object
         */
        addHDRCubeTextureTask(taskName: string, url: string, size: number, noMipmap?: boolean, generateHarmonics?: boolean, gammaSpace?: boolean, reserved?: boolean): HDRCubeTextureAssetTask;
        /**
         *
         * Add a EquiRectangularCubeTextureAssetTask to the list of active tasks
         * @param taskName defines the name of the new task
         * @param url defines the url of the file to load
         * @param size defines the size you want for the cubemap (can be null)
         * @param noMipmap defines if the texture must not receive mipmaps (false by default)
         * @param gammaSpace Specifies if the texture will be used in gamma or linear space
         * (the PBR material requires those textures in linear space, but the standard material would require them in Gamma space)
         * @returns a new EquiRectangularCubeTextureAssetTask object
         */
        addEquiRectangularCubeTextureAssetTask(taskName: string, url: string, size: number, noMipmap?: boolean, gammaSpace?: boolean): EquiRectangularCubeTextureAssetTask;
        /**
         * Remove a task from the assets manager.
         * @param task the task to remove
         */
        removeTask(task: AbstractAssetTask): void;
        private _decreaseWaitingTasksCount;
        private _runTask;
        private _formatTaskErrorMessage;
        /**
         * Reset the AssetsManager and remove all tasks
         * @returns the current instance of the AssetsManager
         */
        reset(): AssetsManager;
        /**
         * Start the loading process
         * @returns the current instance of the AssetsManager
         */
        load(): AssetsManager;
        /**
         * Start the loading process as an async operation
         * @returns a promise returning the list of failed tasks
         */
        loadAsync(): Promise<void>;
    }


    /**
     * Info about the .basis files
     */
    class BasisFileInfo {
        /**
         * If the file has alpha
         */
        hasAlpha: boolean;
        /**
         * Info about each image of the basis file
         */
        images: Array<{
            levels: Array<{
                width: number;
                height: number;
                transcodedPixels: ArrayBufferView;
            }>;
        }>;
    }
    /**
     * Result of transcoding a basis file
     */
    class TranscodeResult {
        /**
         * Info about the .basis file
         */
        fileInfo: BasisFileInfo;
        /**
         * Format to use when loading the file
         */
        format: number;
    }
    /**
     * Configuration options for the Basis transcoder
     */
    export class BasisTranscodeConfiguration {
        /**
         * Supported compression formats used to determine the supported output format of the transcoder
         */
        supportedCompressionFormats?: {
            /**
             * etc1 compression format
             */
            etc1?: boolean;
            /**
             * s3tc compression format
             */
            s3tc?: boolean;
            /**
             * pvrtc compression format
             */
            pvrtc?: boolean;
            /**
             * etc2 compression format
             */
            etc2?: boolean;
            /**
             * astc compression format
             */
            astc?: boolean;
            /**
             * bc7 compression format
             */
            bc7?: boolean;
        };
        /**
         * If mipmap levels should be loaded for transcoded images (Default: true)
         */
        loadMipmapLevels?: boolean;
        /**
         * Index of a single image to load (Default: all images)
         */
        loadSingleImage?: number;
    }
    /**
     * Used to load .Basis files
     * See https://github.com/BinomialLLC/basis_universal/tree/master/webgl
     */
    export var BasisToolsOptions: {
        /**
         * URL to use when loading the basis transcoder
         */
        JSModuleURL: string;
        /**
         * URL to use when loading the wasm module for the transcoder
         */
        WasmModuleURL: string;
    };
    /**
     * Get the internal format to be passed to texImage2D corresponding to the .basis format value
     * @param basisFormat format chosen from GetSupportedTranscodeFormat
     * @param engine
     * @returns internal format corresponding to the Basis format
     */
    export const GetInternalFormatFromBasisFormat: (basisFormat: number, engine: Engine) => number;
    /**
     * Transcodes a loaded image file to compressed pixel data
     * @param data image data to transcode
     * @param config configuration options for the transcoding
     * @returns a promise resulting in the transcoded image
     */
    export const TranscodeAsync: (data: ArrayBuffer | ArrayBufferView, config: BasisTranscodeConfiguration) => Promise<TranscodeResult>;
    /**
     * Loads a texture from the transcode result
     * @param texture texture load to
     * @param transcodeResult the result of transcoding the basis file to load from
     */
    export const LoadTextureFromTranscodeResult: (texture: InternalTexture, transcodeResult: TranscodeResult) => void;
    /**
     * Used to load .Basis files
     * See https://github.com/BinomialLLC/basis_universal/tree/master/webgl
     */
    export var BasisTools: {
        /**
         * URL to use when loading the basis transcoder
         */
        JSModuleURL: string;
        /**
         * URL to use when loading the wasm module for the transcoder
         */
        WasmModuleURL: string;
        /**
         * Get the internal format to be passed to texImage2D corresponding to the .basis format value
         * @param basisFormat format chosen from GetSupportedTranscodeFormat
         * @returns internal format corresponding to the Basis format
         */
        GetInternalFormatFromBasisFormat: (basisFormat: number, engine: Engine) => number;
        /**
         * Transcodes a loaded image file to compressed pixel data
         * @param data image data to transcode
         * @param config configuration options for the transcoding
         * @returns a promise resulting in the transcoded image
         */
        TranscodeAsync: (data: ArrayBuffer | ArrayBufferView, config: BasisTranscodeConfiguration) => Promise<TranscodeResult>;
        /**
         * Loads a texture from the transcode result
         * @param texture texture load to
         * @param transcodeResult the result of transcoding the basis file to load from
         */
        LoadTextureFromTranscodeResult: (texture: InternalTexture, transcodeResult: TranscodeResult) => void;
    };


    /**
     * Gets a default environment BRDF for MS-BRDF Height Correlated BRDF
     * @param scene defines the hosting scene
     * @returns the environment BRDF texture
     */
    export const GetEnvironmentBRDFTexture: (scene: Scene) => BaseTexture;
    /**
     * Class used to host texture specific utilities
     */
    export var BRDFTextureTools: {
        /**
         * Gets a default environment BRDF for MS-BRDF Height Correlated BRDF
         * @param scene defines the hosting scene
         * @returns the environment BRDF texture
         */
        GetEnvironmentBRDFTexture: (scene: Scene) => BaseTexture;
    };


    /**
     * Extracts the characters between two markers (for eg, between "(" and ")"). The function handles nested markers as well as markers inside strings (delimited by ", ' or `) and comments
     * @param markerOpen opening marker
     * @param markerClose closing marker
     * @param block code block to parse
     * @param startIndex starting index in block where the extraction must start. The character at block[startIndex] should be the markerOpen character!
     * @returns index of the last character for the extraction (or -1 if the string is invalid - no matching closing marker found). The string to extract (without the markers) is the string between startIndex + 1 and the returned value (exclusive)
     */
    export function ExtractBetweenMarkers(markerOpen: string, markerClose: string, block: string, startIndex: number): number;
    /**
     * Parses a string and skip whitespaces
     * @param s string to parse
     * @param index index where to start parsing
     * @returns the index after all whitespaces have been skipped
     */
    export function SkipWhitespaces(s: string, index: number): number;
    /**
     * Checks if a character is an identifier character (meaning, if it is 0-9, A-Z, a-z or _)
     * @param c character to check
     * @returns true if the character is an identifier character
     */
    export function IsIdentifierChar(c: string): boolean;
    /**
     * Removes the comments of a code block
     * @param block code block to parse
     * @returns block with the comments removed
     */
    export function RemoveComments(block: string): string;
    /**
     * Finds the first occurrence of a character in a string going backward
     * @param s the string to parse
     * @param index starting index in the string
     * @param c the character to find
     * @returns the index of the character if found, else -1
     */
    export function FindBackward(s: string, index: number, c: string): number;
    /**
     * Escapes a string so that it is usable as a regular expression
     * @param s string to escape
     * @returns escaped string
     */
    export function EscapeRegExp(s: string): string;


    /**
     * Conversion modes available when copying a texture into another one
     */
    export enum ConversionMode {
        None = 0,
        ToLinearSpace = 1,
        ToGammaSpace = 2
    }
    /**
     * Class used for fast copy from one texture to another
     */
    export class CopyTextureToTexture {
        private _engine;
        private _isDepthTexture;
        private _renderer;
        private _effectWrapper;
        private _source;
        private _conversion;
        private _textureIsInternal;
        /**
         * Constructs a new instance of the class
         * @param engine The engine to use for the copy
         * @param isDepthTexture True means that we should write (using gl_FragDepth) into the depth texture attached to the destination (default: false)
         */
        constructor(engine: ThinEngine, isDepthTexture?: boolean);
        /**
         * Indicates if the effect is ready to be used for the copy
         * @returns true if "copy" can be called without delay, else false
         */
        isReady(): boolean;
        /**
         * Copy one texture into another
         * @param source The source texture
         * @param destination The destination texture
         * @param conversion The conversion mode that should be applied when copying
         * @returns
         */
        copy(source: InternalTexture | ThinTexture, destination: RenderTargetWrapper | IRenderTargetTexture, conversion?: ConversionMode): boolean;
        /**
         * Releases all the resources used by the class
         */
        dispose(): void;
    }


    /**
     * Transform some pixel data to a base64 string
     * @param pixels defines the pixel data to transform to base64
     * @param size defines the width and height of the (texture) data
     * @param invertY true if the data must be inverted for the Y coordinate during the conversion
     * @returns The base64 encoded string or null
     */
    export function GenerateBase64StringFromPixelData(pixels: ArrayBufferView, size: ISize, invertY?: boolean): Nullable<string>;
    /**
     * Reads the pixels stored in the webgl texture and returns them as a base64 string
     * @param texture defines the texture to read pixels from
     * @param faceIndex defines the face of the texture to read (in case of cube texture)
     * @param level defines the LOD level of the texture to read (in case of Mip Maps)
     * @returns The base64 encoded string or null
     */
    export function GenerateBase64StringFromTexture(texture: BaseTexture, faceIndex?: number, level?: number): Nullable<string>;
    /**
     * Reads the pixels stored in the webgl texture and returns them as a base64 string
     * @param texture defines the texture to read pixels from
     * @param faceIndex defines the face of the texture to read (in case of cube texture)
     * @param level defines the LOD level of the texture to read (in case of Mip Maps)
     * @returns The base64 encoded string or null wrapped in a promise
     */
    export function GenerateBase64StringFromTextureAsync(texture: BaseTexture, faceIndex?: number, level?: number): Promise<Nullable<string>>;
    /**
     * Class used to host copy specific utilities
     * (Back-compat)
     */
    export var CopyTools: {
        /**
         * Transform some pixel data to a base64 string
         * @param pixels defines the pixel data to transform to base64
         * @param size defines the width and height of the (texture) data
         * @param invertY true if the data must be inverted for the Y coordinate during the conversion
         * @returns The base64 encoded string or null
         */
        GenerateBase64StringFromPixelData: typeof GenerateBase64StringFromPixelData;
        /**
         * Reads the pixels stored in the webgl texture and returns them as a base64 string
         * @param texture defines the texture to read pixels from
         * @param faceIndex defines the face of the texture to read (in case of cube texture)
         * @param level defines the LOD level of the texture to read (in case of Mip Maps)
         * @returns The base64 encoded string or null
         */
        GenerateBase64StringFromTexture: typeof GenerateBase64StringFromTexture;
        /**
         * Reads the pixels stored in the webgl texture and returns them as a base64 string
         * @param texture defines the texture to read pixels from
         * @param faceIndex defines the face of the texture to read (in case of cube texture)
         * @param level defines the LOD level of the texture to read (in case of Mip Maps)
         * @returns The base64 encoded string or null wrapped in a promise
         */
        GenerateBase64StringFromTextureAsync: typeof GenerateBase64StringFromTextureAsync;
    };


    /**
     * A Coroutine<T> is the intersection of:
     * 1. An Iterator that yields void, returns a T, and is not passed values with calls to next.
     * 2. An IterableIterator of void (since it only yields void).
     */
    type CoroutineBase<TStep, TReturn> = Iterator<TStep, TReturn, void> & IterableIterator<TStep>;
    /** @internal */
    export type Coroutine<T> = CoroutineBase<void, T>;
    /** @internal */
    export type AsyncCoroutine<T> = CoroutineBase<void | Promise<void>, T>;
    /** @internal */
    export type CoroutineStep<T> = IteratorResult<void, T>;
    /** @internal */
    export type CoroutineScheduler<T> = (coroutine: AsyncCoroutine<T>, onStep: (stepResult: CoroutineStep<T>) => void, onError: (stepError: any) => void) => void;
    /**
     * @internal
     */
    export function inlineScheduler<T>(coroutine: AsyncCoroutine<T>, onStep: (stepResult: CoroutineStep<T>) => void, onError: (stepError: any) => void): void;
    /**
     * @internal
     */
    export function createYieldingScheduler<T>(yieldAfterMS?: number): (coroutine: AsyncCoroutine<T>, onStep: (stepResult: CoroutineStep<T>) => void, onError: (stepError: any) => void) => void;
    /**
     * @internal
     */
    export function runCoroutine<T>(coroutine: AsyncCoroutine<T>, scheduler: CoroutineScheduler<T>, onSuccess: (result: T) => void, onError: (error: any) => void, abortSignal?: AbortSignal): void;
    /**
     * @internal
     */
    export function runCoroutineSync<T>(coroutine: Coroutine<T>, abortSignal?: AbortSignal): T;
    /**
     * @internal
     */
    export function runCoroutineAsync<T>(coroutine: AsyncCoroutine<T>, scheduler: CoroutineScheduler<T>, abortSignal?: AbortSignal): Promise<T>;
    /**
     * Given a function that returns a Coroutine<T>, produce a function with the same parameters that returns a T.
     * The returned function runs the coroutine synchronously.
     * @param coroutineFactory A function that returns a Coroutine<T>.
     * @param abortSignal
     * @returns A function that runs the coroutine synchronously.
     * @internal
     */
    export function makeSyncFunction<TParams extends unknown[], TReturn>(coroutineFactory: (...params: TParams) => Coroutine<TReturn>, abortSignal?: AbortSignal): (...params: TParams) => TReturn;
    /**
     * Given a function that returns a Coroutine<T>, product a function with the same parameters that returns a Promise<T>.
     * The returned function runs the coroutine asynchronously, yield control of the execution context occasionally to enable a more responsive experience.
     * @param coroutineFactory A function that returns a Coroutine<T>.
     * @param scheduler
     * @param abortSignal
     * @returns A function that runs the coroutine asynchronously.
     * @internal
     */
    export function makeAsyncFunction<TParams extends unknown[], TReturn>(coroutineFactory: (...params: TParams) => AsyncCoroutine<TReturn>, scheduler: CoroutineScheduler<TReturn>, abortSignal?: AbortSignal): (...params: TParams) => Promise<TReturn>;


    /**
     * Interface for any object that can request an animation frame
     */
    export interface ICustomAnimationFrameRequester {
        /**
         * This function will be called when the render loop is ready. If this is not populated, the engine's renderloop function will be called
         */
        renderFunction?: Function;
        /**
         * Called to request the next frame to render to
         * @see https://developer.mozilla.org/en-US/docs/Web/API/window/requestAnimationFrame
         */
        requestAnimationFrame: Function;
        /**
         * You can pass this value to cancelAnimationFrame() to cancel the refresh callback request
         * @see https://developer.mozilla.org/en-US/docs/Web/API/window/requestAnimationFrame#Return_value
         */
        requestID?: number;
    }


    /**
     * Interface for a data buffer
     */
    export interface IDataBuffer {
        /**
         * Reads bytes from the data buffer.
         * @param byteOffset The byte offset to read
         * @param byteLength The byte length to read
         * @returns A promise that resolves when the bytes are read
         */
        readAsync(byteOffset: number, byteLength: number): Promise<ArrayBufferView>;
        /**
         * The byte length of the buffer.
         */
        readonly byteLength: number;
    }
    /**
     * Utility class for reading from a data buffer
     */
    export class DataReader {
        /**
         * The data buffer associated with this data reader.
         */
        readonly buffer: IDataBuffer;
        /**
         * The current byte offset from the beginning of the data buffer.
         */
        byteOffset: number;
        private _dataView;
        private _dataByteOffset;
        /**
         * Constructor
         * @param buffer The buffer to read
         */
        constructor(buffer: IDataBuffer);
        /**
         * Loads the given byte length.
         * @param byteLength The byte length to load
         * @returns A promise that resolves when the load is complete
         */
        loadAsync(byteLength: number): Promise<void>;
        /**
         * Read a unsigned 32-bit integer from the currently loaded data range.
         * @returns The 32-bit integer read
         */
        readUint32(): number;
        /**
         * Read a byte array from the currently loaded data range.
         * @param byteLength The byte length to read
         * @returns The byte array read
         */
        readUint8Array(byteLength: number): Uint8Array;
        /**
         * Read a string from the currently loaded data range.
         * @param byteLength The byte length to read
         * @returns The string read
         */
        readString(byteLength: number): string;
        /**
         * Skips the given byte length the currently loaded data range.
         * @param byteLength The byte length to skip
         */
        skipBytes(byteLength: number): void;
    }


    /**
     * Class for storing data to local storage if available or in-memory storage otherwise
     */
    export class DataStorage {
        private static _Storage;
        private static _GetStorage;
        /**
         * Reads a string from the data storage
         * @param key The key to read
         * @param defaultValue The value if the key doesn't exist
         * @returns The string value
         */
        static ReadString(key: string, defaultValue: string): string;
        /**
         * Writes a string to the data storage
         * @param key The key to write
         * @param value The value to write
         */
        static WriteString(key: string, value: string): void;
        /**
         * Reads a boolean from the data storage
         * @param key The key to read
         * @param defaultValue The value if the key doesn't exist
         * @returns The boolean value
         */
        static ReadBoolean(key: string, defaultValue: boolean): boolean;
        /**
         * Writes a boolean to the data storage
         * @param key The key to write
         * @param value The value to write
         */
        static WriteBoolean(key: string, value: boolean): void;
        /**
         * Reads a number from the data storage
         * @param key The key to read
         * @param defaultValue The value if the key doesn't exist
         * @returns The number value
         */
        static ReadNumber(key: string, defaultValue: number): number;
        /**
         * Writes a number to the data storage
         * @param key The key to write
         * @param value The value to write
         */
        static WriteNumber(key: string, value: number): void;
    }


    /**
     * Direct draw surface info
     * @see https://docs.microsoft.com/en-us/windows/desktop/direct3ddds/dx-graphics-dds-pguide
     */
    export interface DDSInfo {
        /**
         * Width of the texture
         */
        width: number;
        /**
         * Width of the texture
         */
        height: number;
        /**
         * Number of Mipmaps for the texture
         * @see https://en.wikipedia.org/wiki/Mipmap
         */
        mipmapCount: number;
        /**
         * If the textures format is a known fourCC format
         * @see https://www.fourcc.org/
         */
        isFourCC: boolean;
        /**
         * If the texture is an RGB format eg. DXGI_FORMAT_B8G8R8X8_UNORM format
         */
        isRGB: boolean;
        /**
         * If the texture is a lumincance format
         */
        isLuminance: boolean;
        /**
         * If this is a cube texture
         * @see https://docs.microsoft.com/en-us/windows/desktop/direct3ddds/dds-file-layout-for-cubic-environment-maps
         */
        isCube: boolean;
        /**
         * If the texture is a compressed format eg. FOURCC_DXT1
         */
        isCompressed: boolean;
        /**
         * The dxgiFormat of the texture
         * @see https://docs.microsoft.com/en-us/windows/desktop/api/dxgiformat/ne-dxgiformat-dxgi_format
         */
        dxgiFormat: number;
        /**
         * Texture type eg. Engine.TEXTURETYPE_UNSIGNED_INT, Engine.TEXTURETYPE_FLOAT
         */
        textureType: number;
        /**
         * Sphericle polynomial created for the dds texture
         */
        sphericalPolynomial?: SphericalPolynomial;
    }
    /**
     * Class used to provide DDS decompression tools
     */
    export class DDSTools {
        /**
         * Gets or sets a boolean indicating that LOD info is stored in alpha channel (false by default)
         */
        static StoreLODInAlphaChannel: boolean;
        /**
         * Gets DDS information from an array buffer
         * @param data defines the array buffer view to read data from
         * @returns the DDS information
         */
        static GetDDSInfo(data: ArrayBufferView): DDSInfo;
        private static _GetHalfFloatAsFloatRGBAArrayBuffer;
        private static _GetHalfFloatRGBAArrayBuffer;
        private static _GetFloatRGBAArrayBuffer;
        private static _GetFloatAsHalfFloatRGBAArrayBuffer;
        private static _GetFloatAsUIntRGBAArrayBuffer;
        private static _GetHalfFloatAsUIntRGBAArrayBuffer;
        private static _GetRGBAArrayBuffer;
        private static _ExtractLongWordOrder;
        private static _GetRGBArrayBuffer;
        private static _GetLuminanceArrayBuffer;
        /**
         * Uploads DDS Levels to a Babylon Texture
         * @internal
         */
        static UploadDDSLevels(engine: ThinEngine, texture: InternalTexture, data: ArrayBufferView, info: DDSInfo, loadMipmaps: boolean, faces: number, lodIndex?: number, currentFace?: number, destTypeMustBeFilterable?: boolean): void;
    }
        interface ThinEngine {
            /**
             * Create a cube texture from prefiltered data (ie. the mipmaps contain ready to use data for PBR reflection)
             * @param rootUrl defines the url where the file to load is located
             * @param scene defines the current scene
             * @param lodScale defines scale to apply to the mip map selection
             * @param lodOffset defines offset to apply to the mip map selection
             * @param onLoad defines an optional callback raised when the texture is loaded
             * @param onError defines an optional callback raised if there is an issue to load the texture
             * @param format defines the format of the data
             * @param forcedExtension defines the extension to use to pick the right loader
             * @param createPolynomials defines wheter or not to create polynomails harmonics for the texture
             * @returns the cube texture as an InternalTexture
             */
            createPrefilteredCubeTexture(rootUrl: string, scene: Nullable<Scene>, lodScale: number, lodOffset: number, onLoad?: Nullable<(internalTexture: Nullable<InternalTexture>) => void>, onError?: Nullable<(message?: string, exception?: any) => void>, format?: number, forcedExtension?: any, createPolynomials?: boolean): InternalTexture;
        }


    export function expandToProperty(callback: string, targetKey?: Nullable<string>): (target: any, propertyKey: string) => void;
    export function serialize(sourceName?: string): (target: any, propertyKey: string | symbol) => void;
    export function serializeAsTexture(sourceName?: string): (target: any, propertyKey: string | symbol) => void;
    export function serializeAsColor3(sourceName?: string): (target: any, propertyKey: string | symbol) => void;
    export function serializeAsFresnelParameters(sourceName?: string): (target: any, propertyKey: string | symbol) => void;
    export function serializeAsVector2(sourceName?: string): (target: any, propertyKey: string | symbol) => void;
    export function serializeAsVector3(sourceName?: string): (target: any, propertyKey: string | symbol) => void;
    export function serializeAsMeshReference(sourceName?: string): (target: any, propertyKey: string | symbol) => void;
    export function serializeAsColorCurves(sourceName?: string): (target: any, propertyKey: string | symbol) => void;
    export function serializeAsColor4(sourceName?: string): (target: any, propertyKey: string | symbol) => void;
    export function serializeAsImageProcessingConfiguration(sourceName?: string): (target: any, propertyKey: string | symbol) => void;
    export function serializeAsQuaternion(sourceName?: string): (target: any, propertyKey: string | symbol) => void;
    export function serializeAsMatrix(sourceName?: string): (target: any, propertyKey: string | symbol) => void;
    /**
     * Decorator used to define property that can be serialized as reference to a camera
     * @param sourceName defines the name of the property to decorate
     */
    export function serializeAsCameraReference(sourceName?: string): (target: any, propertyKey: string | symbol) => void;
    /**
     * Class used to help serialization objects
     */
    export class SerializationHelper {
        /**
         * Gets or sets a boolean to indicate if the UniqueId property should be serialized
         */
        static AllowLoadingUniqueId: boolean;
        /**
         * @internal
         */
        static _ImageProcessingConfigurationParser: (sourceProperty: any) => ImageProcessingConfiguration;
        /**
         * @internal
         */
        static _FresnelParametersParser: (sourceProperty: any) => FresnelParameters;
        /**
         * @internal
         */
        static _ColorCurvesParser: (sourceProperty: any) => ColorCurves;
        /**
         * @internal
         */
        static _TextureParser: (sourceProperty: any, scene: Scene, rootUrl: string) => Nullable<BaseTexture>;
        /**
         * Appends the serialized animations from the source animations
         * @param source Source containing the animations
         * @param destination Target to store the animations
         */
        static AppendSerializedAnimations(source: IAnimatable, destination: any): void;
        /**
         * Static function used to serialized a specific entity
         * @param entity defines the entity to serialize
         * @param serializationObject defines the optional target object where serialization data will be stored
         * @returns a JSON compatible object representing the serialization of the entity
         */
        static Serialize<T>(entity: T, serializationObject?: any): any;
        /**
         * Creates a new entity from a serialization data object
         * @param creationFunction defines a function used to instanciated the new entity
         * @param source defines the source serialization data
         * @param scene defines the hosting scene
         * @param rootUrl defines the root url for resources
         * @returns a new entity
         */
        static Parse<T>(creationFunction: () => T, source: any, scene: Nullable<Scene>, rootUrl?: Nullable<string>): T;
        /**
         * Clones an object
         * @param creationFunction defines the function used to instanciate the new object
         * @param source defines the source object
         * @returns the cloned object
         */
        static Clone<T>(creationFunction: () => T, source: T): T;
        /**
         * Instanciates a new object based on a source one (some data will be shared between both object)
         * @param creationFunction defines the function used to instanciate the new object
         * @param source defines the source object
         * @returns the new object
         */
        static Instanciate<T>(creationFunction: () => T, source: T): T;
    }
    /**
     * Decorator used to redirect a function to a native implementation if available.
     * @internal
     */
    export function nativeOverride<T extends (...params: any[]) => boolean>(target: any, propertyKey: string, descriptor: TypedPropertyDescriptor<(...params: Parameters<T>) => unknown>, predicate?: T): void;
    export namespace nativeOverride {
        var filter: <T extends (...params: any) => boolean>(predicate: T) => (target: any, propertyKey: string, descriptor: TypedPropertyDescriptor<(...params: Parameters<T>) => unknown>) => void;
    }


    /**
     * Class containing a set of static utilities functions for deep copy.
     */
    export class DeepCopier {
        /**
         * Tries to copy an object by duplicating every property
         * @param source defines the source object
         * @param destination defines the target object
         * @param doNotCopyList defines a list of properties to avoid
         * @param mustCopyList defines a list of properties to copy (even if they start with _)
         */
        static DeepCopy(source: any, destination: any, doNotCopyList?: string[], mustCopyList?: string[]): void;
    }


    /**
     * Wrapper class for promise with external resolve and reject.
     */
    export class Deferred<T> {
        /**
         * The promise associated with this deferred object.
         */
        readonly promise: Promise<T>;
        private _resolve;
        private _reject;
        /**
         * The resolve method of the promise associated with this deferred object.
         */
        get resolve(): (value: T | PromiseLike<T>) => void;
        /**
         * The reject method of the promise associated with this deferred object.
         */
        get reject(): (reason?: any) => void;
        /**
         * Constructor for this deferred object.
         */
        constructor();
    }


    /**
     * This class is a small wrapper around the MinMaxReducer class to compute the min/max values of a depth texture
     */
    export class DepthReducer extends MinMaxReducer {
        private _depthRenderer;
        private _depthRendererId;
        /**
         * Gets the depth renderer used for the computation.
         * Note that the result is null if you provide your own renderer when calling setDepthRenderer.
         */
        get depthRenderer(): Nullable<DepthRenderer>;
        /**
         * Creates a depth reducer
         * @param camera The camera used to render the depth texture
         */
        constructor(camera: Camera);
        /**
         * Sets the depth renderer to use to generate the depth map
         * @param depthRenderer The depth renderer to use. If not provided, a new one will be created automatically
         * @param type The texture type of the depth map (default: TEXTURETYPE_HALF_FLOAT)
         * @param forceFullscreenViewport Forces the post processes used for the reduction to be applied without taking into account viewport (defaults to true)
         */
        setDepthRenderer(depthRenderer?: Nullable<DepthRenderer>, type?: number, forceFullscreenViewport?: boolean): void;
        /**
         * @internal
         */
        setSourceTexture(sourceTexture: RenderTargetTexture, depthRedux: boolean, type?: number, forceFullscreenViewport?: boolean): void;
        /**
         * Activates the reduction computation.
         * When activated, the observers registered in onAfterReductionPerformed are
         * called after the computation is performed
         */
        activate(): void;
        /**
         * Deactivates the reduction computation.
         */
        deactivate(): void;
        /**
         * Disposes the depth reducer
         * @param disposeAll true to dispose all the resources. You should always call this function with true as the parameter (or without any parameter as it is the default one). This flag is meant to be used internally.
         */
        dispose(disposeAll?: boolean): void;
    }


    /**
     * @internal
     */
    export function _WarnImport(name: string): string;


    /**
     * Checks if the window object exists
     * @returns true if the window object exists
     */
    export function IsWindowObjectExist(): boolean;
    /**
     * Checks if the navigator object exists
     * @returns true if the navigator object exists
     */
    export function IsNavigatorAvailable(): boolean;
    /**
     * Check if the document object exists
     * @returns true if the document object exists
     */
    export function IsDocumentAvailable(): boolean;
    /**
     * Extracts text content from a DOM element hierarchy
     * @param element defines the root element
     * @returns a string
     */
    export function GetDOMTextContent(element: HTMLElement): string;
    /**
     * Sets of helpers dealing with the DOM and some of the recurrent functions needed in
     * Babylon.js
     */
    export var DomManagement: {
        /**
         * Checks if the window object exists
         * @returns true if the window object exists
         */
        IsWindowObjectExist: typeof IsWindowObjectExist;
        /**
         * Checks if the navigator object exists
         * @returns true if the navigator object exists
         */
        IsNavigatorAvailable: typeof IsNavigatorAvailable;
        /**
         * Check if the document object exists
         * @returns true if the document object exists
         */
        IsDocumentAvailable: typeof IsDocumentAvailable;
        /**
         * Extracts text content from a DOM element hierarchy
         * @param element defines the root element
         * @returns a string
         */
        GetDOMTextContent: typeof GetDOMTextContent;
    };


    /**
     * Class containing a set of static utilities functions to dump data from a canvas
     */
    export class DumpTools {
        private static _DumpToolsEngine;
        private static _CreateDumpRenderer;
        /**
         * Dumps the current bound framebuffer
         * @param width defines the rendering width
         * @param height defines the rendering height
         * @param engine defines the hosting engine
         * @param successCallback defines the callback triggered once the data are available
         * @param mimeType defines the mime type of the result
         * @param fileName defines the filename to download. If present, the result will automatically be downloaded
         * @returns a void promise
         */
        static DumpFramebuffer(width: number, height: number, engine: Engine, successCallback?: (data: string) => void, mimeType?: string, fileName?: string): Promise<void>;
        /**
         * Dumps an array buffer
         * @param width defines the rendering width
         * @param height defines the rendering height
         * @param data the data array
         * @param mimeType defines the mime type of the result
         * @param fileName defines the filename to download. If present, the result will automatically be downloaded
         * @param invertY true to invert the picture in the Y dimension
         * @param toArrayBuffer true to convert the data to an ArrayBuffer (encoded as `mimeType`) instead of a base64 string
         * @param quality defines the quality of the result
         * @returns a promise that resolve to the final data
         */
        static DumpDataAsync(width: number, height: number, data: ArrayBufferView, mimeType?: string, fileName?: string, invertY?: boolean, toArrayBuffer?: boolean, quality?: number): Promise<string | ArrayBuffer>;
        /**
         * Dumps an array buffer
         * @param width defines the rendering width
         * @param height defines the rendering height
         * @param data the data array
         * @param successCallback defines the callback triggered once the data are available
         * @param mimeType defines the mime type of the result
         * @param fileName defines the filename to download. If present, the result will automatically be downloaded
         * @param invertY true to invert the picture in the Y dimension
         * @param toArrayBuffer true to convert the data to an ArrayBuffer (encoded as `mimeType`) instead of a base64 string
         * @param quality defines the quality of the result
         */
        static DumpData(width: number, height: number, data: ArrayBufferView, successCallback?: (data: string | ArrayBuffer) => void, mimeType?: string, fileName?: string, invertY?: boolean, toArrayBuffer?: boolean, quality?: number): void;
        /**
         * Dispose the dump tools associated resources
         */
        static Dispose(): void;
    }


    /**
     * Raw texture data and descriptor sufficient for WebGL texture upload
     */
    export type EnvironmentTextureInfo = EnvironmentTextureInfoV1 | EnvironmentTextureInfoV2;
    /**
     * v1 of EnvironmentTextureInfo
     */
    interface EnvironmentTextureInfoV1 {
        /**
         * Version of the environment map
         */
        version: 1;
        /**
         * Width of image
         */
        width: number;
        /**
         * Irradiance information stored in the file.
         */
        irradiance: any;
        /**
         * Specular information stored in the file.
         */
        specular: any;
    }
    /**
     * v2 of EnvironmentTextureInfo
     */
    interface EnvironmentTextureInfoV2 {
        /**
         * Version of the environment map
         */
        version: 2;
        /**
         * Width of image
         */
        width: number;
        /**
         * Irradiance information stored in the file.
         */
        irradiance: any;
        /**
         * Specular information stored in the file.
         */
        specular: any;
        /**
         * The mime type used to encode the image data.
         */
        imageType: string;
    }
    /**
     * Defines One Image in the file. It requires only the position in the file
     * as well as the length.
     */
    interface BufferImageData {
        /**
         * Length of the image data.
         */
        length: number;
        /**
         * Position of the data from the null terminator delimiting the end of the JSON.
         */
        position: number;
    }
    /**
     * Defines the specular data enclosed in the file.
     * This corresponds to the version 1 of the data.
     */
    export interface EnvironmentTextureSpecularInfoV1 {
        /**
         * Defines where the specular Payload is located. It is a runtime value only not stored in the file.
         */
        specularDataPosition?: number;
        /**
         * This contains all the images data needed to reconstruct the cubemap.
         */
        mipmaps: Array<BufferImageData>;
        /**
         * Defines the scale applied to environment texture. This manages the range of LOD level used for IBL according to the roughness.
         */
        lodGenerationScale: number;
    }
    /**
     * Options for creating environment textures
     */
    export interface CreateEnvTextureOptions {
        /**
         * The mime type of encoded images.
         */
        imageType?: string;
        /**
         * the image quality of encoded WebP images.
         */
        imageQuality?: number;
    }
    /**
     * Gets the environment info from an env file.
     * @param data The array buffer containing the .env bytes.
     * @returns the environment file info (the json header) if successfully parsed, normalized to the latest supported version.
     */
    export function GetEnvInfo(data: ArrayBufferView): Nullable<EnvironmentTextureInfoV2>;
    /**
     * Normalizes any supported version of the environment file info to the latest version
     * @param info environment file info on any supported version
     * @returns environment file info in the latest supported version
     * @private
     */
    export function normalizeEnvInfo(info: EnvironmentTextureInfo): EnvironmentTextureInfoV2;
    /**
     * Creates an environment texture from a loaded cube texture.
     * @param texture defines the cube texture to convert in env file
     * @param options options for the conversion process
     * @param options.imageType the mime type for the encoded images, with support for "image/png" (default) and "image/webp"
     * @param options.imageQuality the image quality of encoded WebP images.
     * @returns a promise containing the environment data if successful.
     */
    export function CreateEnvTextureAsync(texture: BaseTexture, options?: CreateEnvTextureOptions): Promise<ArrayBuffer>;
    /**
     * Creates the ArrayBufferViews used for initializing environment texture image data.
     * @param data the image data
     * @param info parameters that determine what views will be created for accessing the underlying buffer
     * @returns the views described by info providing access to the underlying buffer
     */
    export function CreateImageDataArrayBufferViews(data: ArrayBufferView, info: EnvironmentTextureInfo): Array<Array<ArrayBufferView>>;
    /**
     * Uploads the texture info contained in the env file to the GPU.
     * @param texture defines the internal texture to upload to
     * @param data defines the data to load
     * @param info defines the texture info retrieved through the GetEnvInfo method
     * @returns a promise
     */
    export function UploadEnvLevelsAsync(texture: InternalTexture, data: ArrayBufferView, info: EnvironmentTextureInfo): Promise<void>;
    /**
     * Uploads the levels of image data to the GPU.
     * @param texture defines the internal texture to upload to
     * @param imageData defines the array buffer views of image data [mipmap][face]
     * @param imageType the mime type of the image data
     * @returns a promise
     */
    export function UploadLevelsAsync(texture: InternalTexture, imageData: ArrayBufferView[][], imageType?: string): Promise<void>;
    /**
     * Uploads spherical polynomials information to the texture.
     * @param texture defines the texture we are trying to upload the information to
     * @param info defines the environment texture info retrieved through the GetEnvInfo method
     */
    export function UploadEnvSpherical(texture: InternalTexture, info: EnvironmentTextureInfo): void;
    /**
     * @internal
     */
    export function _UpdateRGBDAsync(internalTexture: InternalTexture, data: ArrayBufferView[][], sphericalPolynomial: Nullable<SphericalPolynomial>, lodScale: number, lodOffset: number): Promise<InternalTexture>;
    /**
     * Sets of helpers addressing the serialization and deserialization of environment texture
     * stored in a BabylonJS env file.
     * Those files are usually stored as .env files.
     */
    export var EnvironmentTextureTools: {
        /**
         * Gets the environment info from an env file.
         * @param data The array buffer containing the .env bytes.
         * @returns the environment file info (the json header) if successfully parsed, normalized to the latest supported version.
         */
        GetEnvInfo: typeof GetEnvInfo;
        /**
         * Creates an environment texture from a loaded cube texture.
         * @param texture defines the cube texture to convert in env file
         * @param options options for the conversion process
         * @param options.imageType the mime type for the encoded images, with support for "image/png" (default) and "image/webp"
         * @param options.imageQuality the image quality of encoded WebP images.
         * @returns a promise containing the environment data if successful.
         */
        CreateEnvTextureAsync: typeof CreateEnvTextureAsync;
        /**
         * Creates the ArrayBufferViews used for initializing environment texture image data.
         * @param data the image data
         * @param info parameters that determine what views will be created for accessing the underlying buffer
         * @returns the views described by info providing access to the underlying buffer
         */
        CreateImageDataArrayBufferViews: typeof CreateImageDataArrayBufferViews;
        /**
         * Uploads the texture info contained in the env file to the GPU.
         * @param texture defines the internal texture to upload to
         * @param data defines the data to load
         * @param info defines the texture info retrieved through the GetEnvInfo method
         * @returns a promise
         */
        UploadEnvLevelsAsync: typeof UploadEnvLevelsAsync;
        /**
         * Uploads the levels of image data to the GPU.
         * @param texture defines the internal texture to upload to
         * @param imageData defines the array buffer views of image data [mipmap][face]
         * @param imageType the mime type of the image data
         * @returns a promise
         */
        UploadLevelsAsync: typeof UploadLevelsAsync;
        /**
         * Uploads spherical polynomials information to the texture.
         * @param texture defines the texture we are trying to upload the information to
         * @param info defines the environment texture info retrieved through the GetEnvInfo method
         */
        UploadEnvSpherical: typeof UploadEnvSpherical;
    };


    /**
     * Base error. Due to limitations of typedoc-check and missing documentation
     * in lib.es5.d.ts, cannot extend Error directly for RuntimeError.
     * @ignore
     */
    export abstract class BaseError extends Error {
        protected static _setPrototypeOf: (o: any, proto: object | null) => any;
    }
    /**
     * Error codes for BaseError
     */
    export var ErrorCodes: {
        /** Invalid or empty mesh vertex positions. */
        readonly MeshInvalidPositionsError: 0;
        /** Unsupported texture found. */
        readonly UnsupportedTextureError: 1000;
        /** Unexpected magic number found in GLTF file header. */
        readonly GLTFLoaderUnexpectedMagicError: 2000;
        /** SceneLoader generic error code. Ideally wraps the inner exception. */
        readonly SceneLoaderError: 3000;
        /** Load file error */
        readonly LoadFileError: 4000;
        /** Request file error */
        readonly RequestFileError: 4001;
        /** Read file error */
        readonly ReadFileError: 4002;
    };
    /**
     * Error code type
     */
    export type ErrorCodesType = typeof ErrorCodes[keyof typeof ErrorCodes];
    /**
     * Application runtime error
     */
    export class RuntimeError extends BaseError {
        /**
         * The error code
         */
        errorCode: ErrorCodesType;
        /**
         * The error that caused this outer error
         */
        innerError?: Error;
        /**
         * Creates a new RuntimeError
         * @param message defines the message of the error
         * @param errorCode the error code
         * @param innerError the error that caused the outer error
         */
        constructor(message: string, errorCode: ErrorCodesType, innerError?: Error);
    }


    /**
     * File request interface
     */
    export interface IFileRequest {
        /**
         * Raised when the request is complete (success or error).
         */
        onCompleteObservable: Observable<IFileRequest>;
        /**
         * Aborts the request for a file.
         */
        abort: () => void;
    }


    /**
     * Class used to help managing file picking and drag-n-drop
     */
    export class FilesInput {
        /**
         * List of files ready to be loaded
         */
        static get FilesToLoad(): {
            [key: string]: File;
        };
        /**
         * Callback called when a file is processed
         */
        onProcessFileCallback: (file: File, name: string, extension: string, setSceneFileToLoad: (sceneFile: File) => void) => boolean;
        /**
         * Function used when loading the scene file
         * @param sceneFile
         * @param onProgress
         */
        loadAsync: (sceneFile: File, onProgress: Nullable<(event: ISceneLoaderProgressEvent) => void>) => Promise<Scene>;
        private _engine;
        private _currentScene;
        private _sceneLoadedCallback;
        private _progressCallback;
        private _additionalRenderLoopLogicCallback;
        private _textureLoadingCallback;
        private _startingProcessingFilesCallback;
        private _onReloadCallback;
        private _errorCallback;
        private _elementToMonitor;
        private _sceneFileToLoad;
        private _filesToLoad;
        /**
         * Creates a new FilesInput
         * @param engine defines the rendering engine
         * @param scene defines the hosting scene
         * @param sceneLoadedCallback callback called when scene is loaded
         * @param progressCallback callback called to track progress
         * @param additionalRenderLoopLogicCallback callback called to add user logic to the rendering loop
         * @param textureLoadingCallback callback called when a texture is loading
         * @param startingProcessingFilesCallback callback called when the system is about to process all files
         * @param onReloadCallback callback called when a reload is requested
         * @param errorCallback callback call if an error occurs
         */
        constructor(engine: Engine, scene: Nullable<Scene>, sceneLoadedCallback: Nullable<(sceneFile: File, scene: Scene) => void>, progressCallback: Nullable<(progress: ISceneLoaderProgressEvent) => void>, additionalRenderLoopLogicCallback: Nullable<() => void>, textureLoadingCallback: Nullable<(remaining: number) => void>, startingProcessingFilesCallback: Nullable<(files?: File[]) => void>, onReloadCallback: Nullable<(sceneFile: File) => void>, errorCallback: Nullable<(sceneFile: File, scene: Nullable<Scene>, message: string) => void>);
        private _dragEnterHandler;
        private _dragOverHandler;
        private _dropHandler;
        /**
         * Calls this function to listen to drag'n'drop events on a specific DOM element
         * @param elementToMonitor defines the DOM element to track
         */
        monitorElementForDragNDrop(elementToMonitor: HTMLElement): void;
        /** Gets the current list of files to load */
        get filesToLoad(): File[];
        /**
         * Release all associated resources
         */
        dispose(): void;
        private _renderFunction;
        private _drag;
        private _drop;
        private _traverseFolder;
        private _processFiles;
        /**
         * Load files from a drop event
         * @param event defines the drop event to use as source
         */
        loadFiles(event: any): void;
        private _processReload;
        /**
         * Reload the current scene from the loaded files
         */
        reload(): void;
    }


    /**
     * Class used to help managing file picking and drag'n'drop
     * File Storage
     */
    export class FilesInputStore {
        /**
         * List of files ready to be loaded
         */
        static FilesToLoad: {
            [key: string]: File;
        };
    }


    /** @ignore */
    export class LoadFileError extends RuntimeError {
        request?: WebRequest;
        file?: File;
        /**
         * Creates a new LoadFileError
         * @param message defines the message of the error
         * @param object defines the optional web request
         */
        constructor(message: string, object?: WebRequest | File);
    }
    /** @ignore */
    export class RequestFileError extends RuntimeError {
        request: WebRequest;
        /**
         * Creates a new LoadFileError
         * @param message defines the message of the error
         * @param request defines the optional web request
         */
        constructor(message: string, request: WebRequest);
    }
    /** @ignore */
    export class ReadFileError extends RuntimeError {
        file: File;
        /**
         * Creates a new ReadFileError
         * @param message defines the message of the error
         * @param file defines the optional file
         */
        constructor(message: string, file: File);
    }
    /**
     * @internal
     */
    export var FileToolsOptions: {
        DefaultRetryStrategy: (url: string, request: WebRequest, retryIndex: number) => number;
        BaseUrl: string;
        CorsBehavior: string | ((url: string | string[]) => string);
        PreprocessUrl: (url: string) => string;
    };
    /**
     * Sets the cors behavior on a dom element. This will add the required Tools.CorsBehavior to the element.
     * @param url define the url we are trying
     * @param element define the dom element where to configure the cors policy
     * @internal
     */
    export var SetCorsBehavior: (url: string | string[], element: {
        crossOrigin: string | null;
    }) => void;
    /**
     * Loads an image as an HTMLImageElement.
     * @param input url string, ArrayBuffer, or Blob to load
     * @param onLoad callback called when the image successfully loads
     * @param onError callback called when the image fails to load
     * @param offlineProvider offline provider for caching
     * @param mimeType optional mime type
     * @param imageBitmapOptions
     * @returns the HTMLImageElement of the loaded image
     * @internal
     */
    export const LoadImage: (input: string | ArrayBuffer | ArrayBufferView | Blob, onLoad: (img: HTMLImageElement | ImageBitmap) => void, onError: (message?: string, exception?: any) => void, offlineProvider: Nullable<IOfflineProvider>, mimeType?: string, imageBitmapOptions?: ImageBitmapOptions) => Nullable<HTMLImageElement>;
    /**
     * Reads a file from a File object
     * @param file defines the file to load
     * @param onSuccess defines the callback to call when data is loaded
     * @param onProgress defines the callback to call during loading process
     * @param useArrayBuffer defines a boolean indicating that data must be returned as an ArrayBuffer
     * @param onError defines the callback to call when an error occurs
     * @returns a file request object
     * @internal
     */
    export const ReadFile: (file: File, onSuccess: (data: any) => void, onProgress?: ((ev: ProgressEvent) => any) | undefined, useArrayBuffer?: boolean, onError?: ((error: ReadFileError) => void) | undefined) => IFileRequest;
    /**
     * Loads a file from a url, a data url, or a file url
     * @param fileOrUrl file, url, data url, or file url to load
     * @param onSuccess callback called when the file successfully loads
     * @param onProgress callback called while file is loading (if the server supports this mode)
     * @param offlineProvider defines the offline provider for caching
     * @param useArrayBuffer defines a boolean indicating that date must be returned as ArrayBuffer
     * @param onError callback called when the file fails to load
     * @param onOpened
     * @returns a file request object
     * @internal
     */
    export const LoadFile: (fileOrUrl: File | string, onSuccess: (data: string | ArrayBuffer, responseURL?: string, contentType?: Nullable<string>) => void, onProgress?: ((ev: ProgressEvent) => void) | undefined, offlineProvider?: IOfflineProvider, useArrayBuffer?: boolean, onError?: ((request?: WebRequest, exception?: LoadFileError) => void) | undefined, onOpened?: ((request: WebRequest) => void) | undefined) => IFileRequest;
    /**
     * Loads a file from a url
     * @param url url to load
     * @param onSuccess callback called when the file successfully loads
     * @param onProgress callback called while file is loading (if the server supports this mode)
     * @param offlineProvider defines the offline provider for caching
     * @param useArrayBuffer defines a boolean indicating that date must be returned as ArrayBuffer
     * @param onError callback called when the file fails to load
     * @param onOpened callback called when the web request is opened
     * @returns a file request object
     * @internal
     */
    export const RequestFile: (url: string, onSuccess?: ((data: string | ArrayBuffer, request?: WebRequest) => void) | undefined, onProgress?: ((event: ProgressEvent) => void) | undefined, offlineProvider?: IOfflineProvider, useArrayBuffer?: boolean, onError?: ((error: RequestFileError) => void) | undefined, onOpened?: ((request: WebRequest) => void) | undefined) => IFileRequest;
    /**
     * Checks if the loaded document was accessed via `file:`-Protocol.
     * @returns boolean
     * @internal
     */
    export const IsFileURL: () => boolean;
    /**
     * Test if the given uri is a valid base64 data url
     * @param uri The uri to test
     * @returns True if the uri is a base64 data url or false otherwise
     * @internal
     */
    export const IsBase64DataUrl: (uri: string) => boolean;
    export const TestBase64DataUrl: (uri: string) => {
        match: boolean;
        type: string;
    };
    /**
     * Decode the given base64 uri.
     * @param uri The uri to decode
     * @returns The decoded base64 data.
     * @internal
     */
    export function DecodeBase64UrlToBinary(uri: string): ArrayBuffer;
    /**
     * Decode the given base64 uri into a UTF-8 encoded string.
     * @param uri The uri to decode
     * @returns The decoded base64 data.
     * @internal
     */
    export const DecodeBase64UrlToString: (uri: string) => string;
    /**
    * FileTools defined as any.
    * This should not be imported or used in future releases or in any module in the framework
    * @internal
    * @deprecated import the needed function from fileTools.ts
    */
    export let FileTools: {
        DecodeBase64UrlToBinary: (uri: string) => ArrayBuffer;
        DecodeBase64UrlToString: (uri: string) => string;
        DefaultRetryStrategy: any;
        BaseUrl: any;
        CorsBehavior: any;
        PreprocessUrl: any;
        IsBase64DataUrl: (uri: string) => boolean;
        IsFileURL: () => boolean;
        LoadFile: (fileOrUrl: string | File, onSuccess: (data: string | ArrayBuffer, responseURL?: string | undefined) => void, onProgress?: ((ev: ProgressEvent<EventTarget>) => void) | undefined, offlineProvider?: IOfflineProvider | undefined, useArrayBuffer?: boolean | undefined, onError?: ((request?: WebRequest | undefined, exception?: LoadFileError | undefined) => void) | undefined, onOpened?: ((request: WebRequest) => void) | undefined) => IFileRequest;
        LoadImage: (input: string | ArrayBuffer | Blob | ArrayBufferView, onLoad: (img: HTMLImageElement | ImageBitmap) => void, onError: (message?: string | undefined, exception?: any) => void, offlineProvider: Nullable<IOfflineProvider>, mimeType?: string | undefined, imageBitmapOptions?: ImageBitmapOptions | undefined) => Nullable<HTMLImageElement>;
        ReadFile: (file: File, onSuccess: (data: any) => void, onProgress?: ((ev: ProgressEvent<EventTarget>) => any) | undefined, useArrayBuffer?: boolean | undefined, onError?: ((error: ReadFileError) => void) | undefined) => IFileRequest;
        RequestFile: (url: string, onSuccess: (data: string | ArrayBuffer, request?: WebRequest | undefined) => void, onProgress?: ((event: ProgressEvent<EventTarget>) => void) | undefined, offlineProvider?: IOfflineProvider | undefined, useArrayBuffer?: boolean | undefined, onError?: ((error: RequestFileError) => void) | undefined, onOpened?: ((request: WebRequest) => void) | undefined) => IFileRequest;
        SetCorsBehavior: (url: string | string[], element: {
            crossOrigin: string | null;
        }) => void;
    };
    /**
     * @param DecodeBase64UrlToBinary
     * @param DecodeBase64UrlToString
     * @param FileToolsOptions
     * @internal
     */
    export const _injectLTSFileTools: (DecodeBase64UrlToBinary: (uri: string) => ArrayBuffer, DecodeBase64UrlToString: (uri: string) => string, FileToolsOptions: {
        DefaultRetryStrategy: any;
        BaseUrl: any;
        CorsBehavior: any;
        PreprocessUrl: any;
    }, IsBase64DataUrl: (uri: string) => boolean, IsFileURL: () => boolean, LoadFile: (fileOrUrl: string | File, onSuccess: (data: string | ArrayBuffer, responseURL?: string | undefined) => void, onProgress?: ((ev: ProgressEvent<EventTarget>) => void) | undefined, offlineProvider?: IOfflineProvider | undefined, useArrayBuffer?: boolean | undefined, onError?: ((request?: WebRequest | undefined, exception?: LoadFileError | undefined) => void) | undefined, onOpened?: ((request: WebRequest) => void) | undefined) => IFileRequest, LoadImage: (input: string | ArrayBuffer | ArrayBufferView | Blob, onLoad: (img: HTMLImageElement | ImageBitmap) => void, onError: (message?: string | undefined, exception?: any) => void, offlineProvider: Nullable<IOfflineProvider>, mimeType?: string, imageBitmapOptions?: ImageBitmapOptions | undefined) => Nullable<HTMLImageElement>, ReadFile: (file: File, onSuccess: (data: any) => void, onProgress?: ((ev: ProgressEvent<EventTarget>) => any) | undefined, useArrayBuffer?: boolean | undefined, onError?: ((error: ReadFileError) => void) | undefined) => IFileRequest, RequestFile: (url: string, onSuccess: (data: string | ArrayBuffer, request?: WebRequest | undefined) => void, onProgress?: ((event: ProgressEvent<EventTarget>) => void) | undefined, offlineProvider?: IOfflineProvider | undefined, useArrayBuffer?: boolean | undefined, onError?: ((error: RequestFileError) => void) | undefined, onOpened?: ((request: WebRequest) => void) | undefined) => IFileRequest, SetCorsBehavior: (url: string | string[], element: {
        crossOrigin: string | null;
    }) => void) => void;


    /** Interface used by value gradients (color, factor, ...) */
    export interface IValueGradient {
        /**
         * Gets or sets the gradient value (between 0 and 1)
         */
        gradient: number;
    }
    /** Class used to store color4 gradient */
    export class ColorGradient implements IValueGradient {
        /**
         * Gets or sets the gradient value (between 0 and 1)
         */
        gradient: number;
        /**
         * Gets or sets first associated color
         */
        color1: Color4;
        /**
         * Gets or sets second associated color
         */
        color2?: Color4 | undefined;
        /**
         * Creates a new color4 gradient
         * @param gradient gets or sets the gradient value (between 0 and 1)
         * @param color1 gets or sets first associated color
         * @param color2 gets or sets first second color
         */
        constructor(
        /**
         * Gets or sets the gradient value (between 0 and 1)
         */
        gradient: number, 
        /**
         * Gets or sets first associated color
         */
        color1: Color4, 
        /**
         * Gets or sets second associated color
         */
        color2?: Color4 | undefined);
        /**
         * Will get a color picked randomly between color1 and color2.
         * If color2 is undefined then color1 will be used
         * @param result defines the target Color4 to store the result in
         */
        getColorToRef(result: Color4): void;
    }
    /** Class used to store color 3 gradient */
    export class Color3Gradient implements IValueGradient {
        /**
         * Gets or sets the gradient value (between 0 and 1)
         */
        gradient: number;
        /**
         * Gets or sets the associated color
         */
        color: Color3;
        /**
         * Creates a new color3 gradient
         * @param gradient gets or sets the gradient value (between 0 and 1)
         * @param color gets or sets associated color
         */
        constructor(
        /**
         * Gets or sets the gradient value (between 0 and 1)
         */
        gradient: number, 
        /**
         * Gets or sets the associated color
         */
        color: Color3);
    }
    /** Class used to store factor gradient */
    export class FactorGradient implements IValueGradient {
        /**
         * Gets or sets the gradient value (between 0 and 1)
         */
        gradient: number;
        /**
         * Gets or sets first associated factor
         */
        factor1: number;
        /**
         * Gets or sets second associated factor
         */
        factor2?: number | undefined;
        /**
         * Creates a new factor gradient
         * @param gradient gets or sets the gradient value (between 0 and 1)
         * @param factor1 gets or sets first associated factor
         * @param factor2 gets or sets second associated factor
         */
        constructor(
        /**
         * Gets or sets the gradient value (between 0 and 1)
         */
        gradient: number, 
        /**
         * Gets or sets first associated factor
         */
        factor1: number, 
        /**
         * Gets or sets second associated factor
         */
        factor2?: number | undefined);
        /**
         * Will get a number picked randomly between factor1 and factor2.
         * If factor2 is undefined then factor1 will be used
         * @returns the picked number
         */
        getFactor(): number;
    }
    /**
     * Helper used to simplify some generic gradient tasks
     */
    export class GradientHelper {
        /**
         * Gets the current gradient from an array of IValueGradient
         * @param ratio defines the current ratio to get
         * @param gradients defines the array of IValueGradient
         * @param updateFunc defines the callback function used to get the final value from the selected gradients
         */
        static GetCurrentGradient(ratio: number, gradients: IValueGradient[], updateFunc: (current: IValueGradient, next: IValueGradient, scale: number) => void): void;
    }


    /**
     * Implementation from http://stackoverflow.com/questions/105034/how-to-create-a-guid-uuid-in-javascript/2117523#answer-2117523
     * Be aware Math.random() could cause collisions, but:
     * "All but 6 of the 128 bits of the ID are randomly generated, which means that for any two ids, there's a 1 in 2^^122 (or 5.3x10^^36) chance they'll collide"
     * @returns a pseudo random id
     */
    export function RandomGUID(): string;
    /**
     * Class used to manipulate GUIDs
     */
    export var GUID: {
        /**
         * Implementation from http://stackoverflow.com/questions/105034/how-to-create-a-guid-uuid-in-javascript/2117523#answer-2117523
         * Be aware Math.random() could cause collisions, but:
         * "All but 6 of the 128 bits of the ID are randomly generated, which means that for any two ids, there's a 1 in 2^^122 (or 5.3x10^^36) chance they'll collide"
         * @returns a pseudo random id
         */
        RandomId: typeof RandomGUID;
    };


    /**
     * Helper class dealing with the extraction of spherical polynomial dataArray
     * from a cube map.
     */
    export class CubeMapToSphericalPolynomialTools {
        private static _FileFaces;
        /**
         * Converts a texture to the according Spherical Polynomial data.
         * This extracts the first 3 orders only as they are the only one used in the lighting.
         *
         * @param texture The texture to extract the information from.
         * @returns The Spherical Polynomial data.
         */
        static ConvertCubeMapTextureToSphericalPolynomial(texture: BaseTexture): Nullable<Promise<SphericalPolynomial>>;
        /**
         * Compute the area on the unit sphere of the rectangle defined by (x,y) and the origin
         * See https://www.rorydriscoll.com/2012/01/15/cubemap-texel-solid-angle/
         * @param x
         * @param y
         */
        private static _AreaElement;
        /**
         * Converts a cubemap to the according Spherical Polynomial data.
         * This extracts the first 3 orders only as they are the only one used in the lighting.
         *
         * @param cubeInfo The Cube map to extract the information from.
         * @returns The Spherical Polynomial data.
         */
        static ConvertCubeMapToSphericalPolynomial(cubeInfo: CubeMapInfo): SphericalPolynomial;
    }


    /**
     * Header information of HDR texture files.
     */
    export interface HDRInfo {
        /**
         * The height of the texture in pixels.
         */
        height: number;
        /**
         * The width of the texture in pixels.
         */
        width: number;
        /**
         * The index of the beginning of the data in the binary file.
         */
        dataPosition: number;
    }
    /**
     * This groups tools to convert HDR texture to native colors array.
     */
    export class HDRTools {
        private static _Ldexp;
        private static _Rgbe2float;
        private static _ReadStringLine;
        /**
         * Reads header information from an RGBE texture stored in a native array.
         * More information on this format are available here:
         * https://en.wikipedia.org/wiki/RGBE_image_format
         *
         * @param uint8array The binary file stored in  native array.
         * @returns The header information.
         */
        static RGBE_ReadHeader(uint8array: Uint8Array): HDRInfo;
        /**
         * Returns the cubemap information (each faces texture data) extracted from an RGBE texture.
         * This RGBE texture needs to store the information as a panorama.
         *
         * More information on this format are available here:
         * https://en.wikipedia.org/wiki/RGBE_image_format
         *
         * @param buffer The binary file stored in an array buffer.
         * @param size The expected size of the extracted cubemap.
         * @returns The Cube Map information.
         */
        static GetCubeMapTextureData(buffer: ArrayBuffer, size: number): CubeMapInfo;
        /**
         * Returns the pixels data extracted from an RGBE texture.
         * This pixels will be stored left to right up to down in the R G B order in one array.
         *
         * More information on this format are available here:
         * https://en.wikipedia.org/wiki/RGBE_image_format
         *
         * @param uint8array The binary file stored in an array buffer.
         * @param hdrInfo The header information of the file.
         * @returns The pixels data in RGB right to left up to down order.
         */
        static RGBE_ReadPixels(uint8array: Uint8Array, hdrInfo: HDRInfo): Float32Array;
        private static _RGBEReadPixelsRLE;
        private static _RGBEReadPixelsNOTRLE;
    }




    /**
     * CubeMap information grouping all the data for each faces as well as the cubemap size.
     */
    export interface CubeMapInfo {
        /**
         * The pixel array for the front face.
         * This is stored in format, left to right, up to down format.
         */
        front: Nullable<ArrayBufferView>;
        /**
         * The pixel array for the back face.
         * This is stored in format, left to right, up to down format.
         */
        back: Nullable<ArrayBufferView>;
        /**
         * The pixel array for the left face.
         * This is stored in format, left to right, up to down format.
         */
        left: Nullable<ArrayBufferView>;
        /**
         * The pixel array for the right face.
         * This is stored in format, left to right, up to down format.
         */
        right: Nullable<ArrayBufferView>;
        /**
         * The pixel array for the up face.
         * This is stored in format, left to right, up to down format.
         */
        up: Nullable<ArrayBufferView>;
        /**
         * The pixel array for the down face.
         * This is stored in format, left to right, up to down format.
         */
        down: Nullable<ArrayBufferView>;
        /**
         * The size of the cubemap stored.
         *
         * Each faces will be size * size pixels.
         */
        size: number;
        /**
         * The format of the texture.
         *
         * RGBA, RGB.
         */
        format: number;
        /**
         * The type of the texture data.
         *
         * UNSIGNED_INT, FLOAT.
         */
        type: number;
        /**
         * Specifies whether the texture is in gamma space.
         */
        gammaSpace: boolean;
    }
    /**
     * Helper class useful to convert panorama picture to their cubemap representation in 6 faces.
     */
    export class PanoramaToCubeMapTools {
        private static FACE_LEFT;
        private static FACE_RIGHT;
        private static FACE_FRONT;
        private static FACE_BACK;
        private static FACE_DOWN;
        private static FACE_UP;
        /**
         * Converts a panorama stored in RGB right to left up to down format into a cubemap (6 faces).
         *
         * @param float32Array The source data.
         * @param inputWidth The width of the input panorama.
         * @param inputHeight The height of the input panorama.
         * @param size The willing size of the generated cubemap (each faces will be size * size pixels)
         * @returns The cubemap data
         */
        static ConvertPanoramaToCubemap(float32Array: Float32Array, inputWidth: number, inputHeight: number, size: number): CubeMapInfo;
        private static CreateCubemapTexture;
        private static CalcProjectionSpherical;
    }


    /**
     * Enum that determines the text-wrapping mode to use.
     */
    export enum InspectableType {
        /**
         * Checkbox for booleans
         */
        Checkbox = 0,
        /**
         * Sliders for numbers
         */
        Slider = 1,
        /**
         * Vector3
         */
        Vector3 = 2,
        /**
         * Quaternions
         */
        Quaternion = 3,
        /**
         * Color3
         */
        Color3 = 4,
        /**
         * String
         */
        String = 5,
        /**
         * Button
         */
        Button = 6,
        /**
         * Options
         */
        Options = 7,
        /**
         * Tab
         */
        Tab = 8,
        /**
         * File button
         */
        FileButton = 9,
        /**
         * Vector2
         */
        Vector2 = 10
    }
    /**
     * Interface used to define custom inspectable options in "Options" mode.
     * This interface is used by the inspector to display the list of options
     */
    export interface IInspectableOptions {
        /**
         * Defines the visible part of the option
         */
        label: string;
        /**
         * Defines the value part of the option (returned through the callback)
         */
        value: number | string;
        /**
         * Defines if the option should be selected or not
         */
        selected?: boolean;
    }
    /**
     * Interface used to define custom inspectable properties.
     * This interface is used by the inspector to display custom property grids
     * @see https://doc.babylonjs.com/toolsAndResources/inspector#extensibility
     */
    export interface IInspectable {
        /**
         * Gets the label to display
         */
        label: string;
        /**
         * Gets the name of the property to edit
         */
        propertyName: string;
        /**
         * Gets the type of the editor to use
         */
        type: InspectableType;
        /**
         * Gets the minimum value of the property when using in "slider" mode
         */
        min?: number;
        /**
         * Gets the maximum value of the property when using in "slider" mode
         */
        max?: number;
        /**
         * Gets the setp to use when using in "slider" mode
         */
        step?: number;
        /**
         * Gets the callback function when using "Button" mode
         */
        callback?: () => void;
        /**
         * Gets the callback function when using "FileButton" mode
         */
        fileCallback?: (file: File) => void;
        /**
         * Gets the list of options when using "Option" mode
         */
        options?: IInspectableOptions[];
        /**
         * Gets the extensions to accept when using "FileButton" mode.
         * The value should be a comma separated string with the list of extensions to accept e.g., ".jpg, .png, .tga, .dds, .env".
         */
        accept?: string;
    }




    /**
     * Class used to enable instantiation of objects by class name
     */
    export class InstantiationTools {
        /**
         * Use this object to register external classes like custom textures or material
         * to allow the loaders to instantiate them
         */
        static RegisteredExternalClasses: {
            [key: string]: Object;
        };
        /**
         * Tries to instantiate a new object from a given class name
         * @param className defines the class name to instantiate
         * @returns the new object or null if the system was not able to do the instantiation
         */
        static Instantiate(className: string): any;
    }


    /**
     * Interface used to define entities containing multiple clip planes
     */
    export interface IClipPlanesHolder {
        /**
         * Gets or sets the active clipplane 1
         */
        clipPlane: Nullable<Plane>;
        /**
         * Gets or sets the active clipplane 2
         */
        clipPlane2: Nullable<Plane>;
        /**
         * Gets or sets the active clipplane 3
         */
        clipPlane3: Nullable<Plane>;
        /**
         * Gets or sets the active clipplane 4
         */
        clipPlane4: Nullable<Plane>;
        /**
         * Gets or sets the active clipplane 5
         */
        clipPlane5: Nullable<Plane>;
        /**
         * Gets or sets the active clipplane 6
         */
        clipPlane6: Nullable<Plane>;
    }


    /**
     * Defines the shape of a collection of datasets that our graphing service uses for drawing purposes.
     */
    export interface IPerfDatasets {
        /**
         * The ids of our dataset.
         */
        ids: string[];
        /**
         * The data to be processed by the performance graph. Each slice will be of the form of [timestamp, numberOfPoints, value1, value2...]
         */
        data: DynamicFloat32Array;
        /**
         * A list of starting indices for each slice of data collected. Used for fast access of an arbitrary slice inside the data array.
         */
        startingIndices: DynamicFloat32Array;
    }
    /**
     * Defines the shape of a the metadata the graphing service uses for drawing purposes.
     */
    export interface IPerfMetadata {
        /**
         * The color of the line to be drawn.
         */
        color?: string;
        /**
         * Specifies if data should be hidden, falsey by default.
         */
        hidden?: boolean;
        /**
         * Specifies the category of the data
         */
        category?: string;
    }
    /**
     * Defines the shape of a custom user registered event.
     */
    export interface IPerfCustomEvent {
        /**
         * The name of the event.
         */
        name: string;
        /**
         * The value for the event, if set we will use it as the value, otherwise we will count the number of occurrences.
         */
        value?: number;
    }


    /**
     * Interface used to define the mechanism to get data from the network
     */
    export interface IWebRequest {
        /**
         * Returns client's response url
         */
        responseURL: string;
        /**
         * Returns client's status
         */
        status: number;
        /**
         * Returns client's status as a text
         */
        statusText: string;
    }


    /**
     * Interface for screenshot methods with describe argument called `size` as object with options
     * @link https://doc.babylonjs.com/api/classes/babylon.screenshottools
     */
    export interface IScreenshotSize {
        /**
         * number in pixels for canvas height
         */
        height?: number;
        /**
         * multiplier allowing render at a higher or lower resolution
         * If value is defined then height and width will be ignored and taken from camera
         */
        precision?: number;
        /**
         * number in pixels for canvas width
         */
        width?: number;
    }


    /**
     * for description see https://www.khronos.org/opengles/sdk/tools/KTX/
     * for file layout see https://www.khronos.org/opengles/sdk/tools/KTX/file_format_spec/
     */
    export class KhronosTextureContainer {
        /** contents of the KTX container file */
        data: ArrayBufferView;
        private static HEADER_LEN;
        private static COMPRESSED_2D;
        private static COMPRESSED_3D;
        private static TEX_2D;
        private static TEX_3D;
        /**
         * Gets the openGL type
         */
        glType: number;
        /**
         * Gets the openGL type size
         */
        glTypeSize: number;
        /**
         * Gets the openGL format
         */
        glFormat: number;
        /**
         * Gets the openGL internal format
         */
        glInternalFormat: number;
        /**
         * Gets the base internal format
         */
        glBaseInternalFormat: number;
        /**
         * Gets image width in pixel
         */
        pixelWidth: number;
        /**
         * Gets image height in pixel
         */
        pixelHeight: number;
        /**
         * Gets image depth in pixels
         */
        pixelDepth: number;
        /**
         * Gets the number of array elements
         */
        numberOfArrayElements: number;
        /**
         * Gets the number of faces
         */
        numberOfFaces: number;
        /**
         * Gets the number of mipmap levels
         */
        numberOfMipmapLevels: number;
        /**
         * Gets the bytes of key value data
         */
        bytesOfKeyValueData: number;
        /**
         * Gets the load type
         */
        loadType: number;
        /**
         * If the container has been made invalid (eg. constructor failed to correctly load array buffer)
         */
        isInvalid: boolean;
        /**
         * Creates a new KhronosTextureContainer
         * @param data contents of the KTX container file
         * @param facesExpected should be either 1 or 6, based whether a cube texture or or
         */
        constructor(
        /** contents of the KTX container file */
        data: ArrayBufferView, facesExpected: number);
        /**
         * Uploads KTX content to a Babylon Texture.
         * It is assumed that the texture has already been created & is currently bound
         * @internal
         */
        uploadLevels(texture: InternalTexture, loadMipmaps: boolean): void;
        private _upload2DCompressedLevels;
        /**
         * Checks if the given data starts with a KTX file identifier.
         * @param data the data to check
         * @returns true if the data is a KTX file or false otherwise
         */
        static IsValid(data: ArrayBufferView): boolean;
    }


    /**
     * Class that defines the default KTX2 decoder options.
     *
     * This class is useful for providing options to the KTX2 decoder to control how the source data is transcoded.
     */
    export class DefaultKTX2DecoderOptions {
        private _isDirty;
        /**
         * Gets the dirty flag
         */
        get isDirty(): boolean;
        private _useRGBAIfASTCBC7NotAvailableWhenUASTC?;
        /**
         * force a (uncompressed) RGBA transcoded format if transcoding a UASTC source format and ASTC + BC7 are not available as a compressed transcoded format
         */
        get useRGBAIfASTCBC7NotAvailableWhenUASTC(): boolean | undefined;
        set useRGBAIfASTCBC7NotAvailableWhenUASTC(value: boolean | undefined);
        private _useRGBAIfOnlyBC1BC3AvailableWhenUASTC;
        /**
         * force a (uncompressed) RGBA transcoded format if transcoding a UASTC source format and only BC1 or BC3 are available as a compressed transcoded format.
         * This property is true by default to favor speed over memory, because currently transcoding from UASTC to BC1/3 is slow because the transcoder transcodes
         * to uncompressed and then recompresses the texture
         */
        get useRGBAIfOnlyBC1BC3AvailableWhenUASTC(): boolean;
        set useRGBAIfOnlyBC1BC3AvailableWhenUASTC(value: boolean);
        private _forceRGBA?;
        /**
         * force to always use (uncompressed) RGBA for transcoded format
         */
        get forceRGBA(): boolean | undefined;
        set forceRGBA(value: boolean | undefined);
        private _forceR8?;
        /**
         * force to always use (uncompressed) R8 for transcoded format
         */
        get forceR8(): boolean | undefined;
        set forceR8(value: boolean | undefined);
        private _forceRG8?;
        /**
         * force to always use (uncompressed) RG8 for transcoded format
         */
        get forceRG8(): boolean | undefined;
        set forceRG8(value: boolean | undefined);
        private _bypassTranscoders?;
        /**
         * list of transcoders to bypass when looking for a suitable transcoder. The available transcoders are:
         *      UniversalTranscoder_UASTC_ASTC
         *      UniversalTranscoder_UASTC_BC7
         *      UniversalTranscoder_UASTC_RGBA_UNORM
         *      UniversalTranscoder_UASTC_RGBA_SRGB
         *      UniversalTranscoder_UASTC_R8_UNORM
         *      UniversalTranscoder_UASTC_RG8_UNORM
         *      MSCTranscoder
         */
        get bypassTranscoders(): string[] | undefined;
        set bypassTranscoders(value: string[] | undefined);
        private _ktx2DecoderOptions;
        /** @internal */
        _getKTX2DecoderOptions(): IKTX2DecoderOptions;
    }
    /**
     * Class for loading KTX2 files
     */
    export class KhronosTextureContainer2 {
        private static _WorkerPoolPromise?;
        private static _DecoderModulePromise?;
        /**
         * URLs to use when loading the KTX2 decoder module as well as its dependencies
         * If a url is null, the default url is used (pointing to https://preview.babylonjs.com)
         * Note that jsDecoderModule can't be null and that the other dependencies will only be loaded if necessary
         * Urls you can change:
         *     URLConfig.jsDecoderModule
         *     URLConfig.wasmUASTCToASTC
         *     URLConfig.wasmUASTCToBC7
         *     URLConfig.wasmUASTCToRGBA_UNORM
         *     URLConfig.wasmUASTCToRGBA_SRGB
         *     URLConfig.wasmUASTCToR8_UNORM
         *     URLConfig.wasmUASTCToRG8_UNORM
         *     URLConfig.jsMSCTranscoder
         *     URLConfig.wasmMSCTranscoder
         *     URLConfig.wasmZSTDDecoder
         * You can see their default values in this PG: https://playground.babylonjs.com/#EIJH8L#29
         */
        static URLConfig: {
            jsDecoderModule: string;
            wasmUASTCToASTC: Nullable<string>;
            wasmUASTCToBC7: Nullable<string>;
            wasmUASTCToRGBA_UNORM: Nullable<string>;
            wasmUASTCToRGBA_SRGB: Nullable<string>;
            wasmUASTCToR8_UNORM: Nullable<string>;
            wasmUASTCToRG8_UNORM: Nullable<string>;
            jsMSCTranscoder: Nullable<string>;
            wasmMSCTranscoder: Nullable<string>;
            wasmZSTDDecoder: Nullable<string>;
        };
        /**
         * Default number of workers used to handle data decoding
         */
        static DefaultNumWorkers: number;
        /**
         * Default configuration for the KTX2 decoder.
         * The options defined in this way have priority over those passed when creating a KTX2 texture with new Texture(...).
         */
        static DefaultDecoderOptions: DefaultKTX2DecoderOptions;
        private static GetDefaultNumWorkers;
        private _engine;
        private static _Initialize;
        /**
         * Constructor
         * @param engine The engine to use
         * @param numWorkers The number of workers for async operations. Specify `0` to disable web workers and run synchronously in the current context.
         */
        constructor(engine: ThinEngine, numWorkers?: number);
        /**
         * @internal
         */
        uploadAsync(data: ArrayBufferView, internalTexture: InternalTexture, options?: IKTX2DecoderOptions & IDecodedData): Promise<void>;
        protected _createTexture(data: IDecodedData, internalTexture: InternalTexture, options?: IKTX2DecoderOptions & IDecodedData): void;
        /**
         * Checks if the given data starts with a KTX2 file identifier.
         * @param data the data to check
         * @returns true if the data is a KTX2 file or false otherwise
         */
        static IsValid(data: ArrayBufferView): boolean;
    }


    /**
     * Logger used throughout the application to allow configuration of
     * the log level required for the messages.
     */
    export class Logger {
        /**
         * No log
         */
        static readonly NoneLogLevel = 0;
        /**
         * Only message logs
         */
        static readonly MessageLogLevel = 1;
        /**
         * Only warning logs
         */
        static readonly WarningLogLevel = 2;
        /**
         * Only error logs
         */
        static readonly ErrorLogLevel = 4;
        /**
         * All logs
         */
        static readonly AllLogLevel = 7;
        /**
         * Message to display when a message has been logged too many times
         */
        static MessageLimitReached: string;
        private static _LogCache;
        private static _LogLimitOutputs;
        private static _Levels;
        /**
         * Gets a value indicating the number of loading errors
         * @ignorenaming
         */
        static errorsCount: number;
        /**
         * Callback called when a new log is added
         */
        static OnNewCacheEntry: (entry: string) => void;
        private static _CheckLimit;
        private static _GenerateLimitMessage;
        private static _AddLogEntry;
        private static _FormatMessage;
        private static _LogDisabled;
        private static _LogEnabled;
        /**
         * Log a message to the console
         */
        static Log: (message: string, limit?: number) => void;
        /**
         * Write a warning message to the console
         */
        static Warn: (message: string, limit?: number) => void;
        /**
         * Write an error message to the console
         */
        static Error: (message: string, limit?: number) => void;
        /**
         * Gets current log cache (list of logs)
         */
        static get LogCache(): string;
        /**
         * Clears the log cache
         */
        static ClearLogCache(): void;
        /**
         * Sets the current log level (MessageLogLevel / WarningLogLevel / ErrorLogLevel)
         */
        static set LogLevels(level: number);
    }


    /**
     * Class used to explode meshes (ie. to have a center and move them away from that center to better see the overall organization)
     */
    export class MeshExploder {
        private _centerMesh;
        private _meshes;
        private _meshesOrigins;
        private _toCenterVectors;
        private _scaledDirection;
        private _newPosition;
        private _centerPosition;
        /**
         * Explodes meshes from a center mesh.
         * @param meshes The meshes to explode.
         * @param centerMesh The mesh to be center of explosion.
         */
        constructor(meshes: Array<Mesh>, centerMesh?: Mesh);
        private _setCenterMesh;
        /**
         * Get class name
         * @returns "MeshExploder"
         */
        getClassName(): string;
        /**
         * "Exploded meshes"
         * @returns Array of meshes with the centerMesh at index 0.
         */
        getMeshes(): Array<Mesh>;
        /**
         * Explodes meshes giving a specific direction
         * @param direction Number to multiply distance of each mesh's origin from center. Use a negative number to implode, or zero to reset.
         */
        explode(direction?: number): void;
    }


    /**
     * This class computes a min/max reduction from a texture: it means it computes the minimum
     * and maximum values from all values of the texture.
     * It is performed on the GPU for better performances, thanks to a succession of post processes.
     * The source values are read from the red channel of the texture.
     */
    export class MinMaxReducer {
        /**
         * Observable triggered when the computation has been performed
         */
        onAfterReductionPerformed: Observable<{
            min: number;
            max: number;
        }>;
        protected _camera: Camera;
        protected _sourceTexture: Nullable<RenderTargetTexture>;
        protected _reductionSteps: Nullable<Array<PostProcess>>;
        protected _postProcessManager: PostProcessManager;
        protected _onAfterUnbindObserver: Nullable<Observer<RenderTargetTexture>>;
        protected _forceFullscreenViewport: boolean;
        protected _onContextRestoredObserver: Nullable<Observer<ThinEngine>>;
        /**
         * Creates a min/max reducer
         * @param camera The camera to use for the post processes
         */
        constructor(camera: Camera);
        /**
         * Gets the texture used to read the values from.
         */
        get sourceTexture(): Nullable<RenderTargetTexture>;
        /**
         * Sets the source texture to read the values from.
         * One must indicate if the texture is a depth texture or not through the depthRedux parameter
         * because in such textures '1' value must not be taken into account to compute the maximum
         * as this value is used to clear the texture.
         * Note that the computation is not activated by calling this function, you must call activate() for that!
         * @param sourceTexture The texture to read the values from. The values should be in the red channel.
         * @param depthRedux Indicates if the texture is a depth texture or not
         * @param type The type of the textures created for the reduction (defaults to TEXTURETYPE_HALF_FLOAT)
         * @param forceFullscreenViewport Forces the post processes used for the reduction to be applied without taking into account viewport (defaults to true)
         */
        setSourceTexture(sourceTexture: RenderTargetTexture, depthRedux: boolean, type?: number, forceFullscreenViewport?: boolean): void;
        /**
         * Defines the refresh rate of the computation.
         * Use 0 to compute just once, 1 to compute on every frame, 2 to compute every two frames and so on...
         */
        get refreshRate(): number;
        set refreshRate(value: number);
        protected _activated: boolean;
        /**
         * Gets the activation status of the reducer
         */
        get activated(): boolean;
        /**
         * Activates the reduction computation.
         * When activated, the observers registered in onAfterReductionPerformed are
         * called after the computation is performed
         */
        activate(): void;
        /**
         * Deactivates the reduction computation.
         */
        deactivate(): void;
        /**
         * Disposes the min/max reducer
         * @param disposeAll true to dispose all the resources. You should always call this function with true as the parameter (or without any parameter as it is the default one). This flag is meant to be used internally.
         */
        dispose(disposeAll?: boolean): void;
    }


    /**
     * A class serves as a medium between the observable and its observers
     */
    export class EventState {
        /**
         * Create a new EventState
         * @param mask defines the mask associated with this state
         * @param skipNextObservers defines a flag which will instruct the observable to skip following observers when set to true
         * @param target defines the original target of the state
         * @param currentTarget defines the current target of the state
         */
        constructor(mask: number, skipNextObservers?: boolean, target?: any, currentTarget?: any);
        /**
         * Initialize the current event state
         * @param mask defines the mask associated with this state
         * @param skipNextObservers defines a flag which will instruct the observable to skip following observers when set to true
         * @param target defines the original target of the state
         * @param currentTarget defines the current target of the state
         * @returns the current event state
         */
        initialize(mask: number, skipNextObservers?: boolean, target?: any, currentTarget?: any): EventState;
        /**
         * An Observer can set this property to true to prevent subsequent observers of being notified
         */
        skipNextObservers: boolean;
        /**
         * Get the mask value that were used to trigger the event corresponding to this EventState object
         */
        mask: number;
        /**
         * The object that originally notified the event
         */
        target?: any;
        /**
         * The current object in the bubbling phase
         */
        currentTarget?: any;
        /**
         * This will be populated with the return value of the last function that was executed.
         * If it is the first function in the callback chain it will be the event data.
         */
        lastReturnValue?: any;
        /**
         * User defined information that will be sent to observers
         */
        userInfo?: any;
    }
    /**
     * Represent an Observer registered to a given Observable object.
     */
    export class Observer<T> {
        /**
         * Defines the callback to call when the observer is notified
         */
        callback: (eventData: T, eventState: EventState) => void;
        /**
         * Defines the mask of the observer (used to filter notifications)
         */
        mask: number;
        /**
         * Defines the current scope used to restore the JS context
         */
        scope: any;
        /** @internal */
        _willBeUnregistered: boolean;
        /**
         * Gets or sets a property defining that the observer as to be unregistered after the next notification
         */
        unregisterOnNextCall: boolean;
        /**
         * Creates a new observer
         * @param callback defines the callback to call when the observer is notified
         * @param mask defines the mask of the observer (used to filter notifications)
         * @param scope defines the current scope used to restore the JS context
         */
        constructor(
        /**
         * Defines the callback to call when the observer is notified
         */
        callback: (eventData: T, eventState: EventState) => void, 
        /**
         * Defines the mask of the observer (used to filter notifications)
         */
        mask: number, 
        /**
         * Defines the current scope used to restore the JS context
         */
        scope?: any);
    }
    /**
     * The Observable class is a simple implementation of the Observable pattern.
     *
     * There's one slight particularity though: a given Observable can notify its observer using a particular mask value, only the Observers registered with this mask value will be notified.
     * This enable a more fine grained execution without having to rely on multiple different Observable objects.
     * For instance you may have a given Observable that have four different types of notifications: Move (mask = 0x01), Stop (mask = 0x02), Turn Right (mask = 0X04), Turn Left (mask = 0X08).
     * A given observer can register itself with only Move and Stop (mask = 0x03), then it will only be notified when one of these two occurs and will never be for Turn Left/Right.
     */
    export class Observable<T> {
        private _observers;
        private _numObserversMarkedAsDeleted;
        /**
         * @internal
         */
        _eventState: EventState;
        private _onObserverAdded;
        /**
         * Create an observable from a Promise.
         * @param promise a promise to observe for fulfillment.
         * @param onErrorObservable an observable to notify if a promise was rejected.
         * @returns the new Observable
         */
        static FromPromise<T, E = Error>(promise: Promise<T>, onErrorObservable?: Observable<E>): Observable<T>;
        /**
         * Gets the list of observers
         * Note that observers that were recently deleted may still be present in the list because they are only really deleted on the next javascript tick!
         */
        get observers(): Array<Observer<T>>;
        /**
         * Creates a new observable
         * @param onObserverAdded defines a callback to call when a new observer is added
         */
        constructor(onObserverAdded?: (observer: Observer<T>) => void);
        /**
         * Create a new Observer with the specified callback
         * @param callback the callback that will be executed for that Observer
         * @param mask the mask used to filter observers
         * @param insertFirst if true the callback will be inserted at the first position, hence executed before the others ones. If false (default behavior) the callback will be inserted at the last position, executed after all the others already present.
         * @param scope optional scope for the callback to be called from
         * @param unregisterOnFirstCall defines if the observer as to be unregistered after the next notification
         * @returns the new observer created for the callback
         */
        add(callback: (eventData: T, eventState: EventState) => void, mask?: number, insertFirst?: boolean, scope?: any, unregisterOnFirstCall?: boolean): Nullable<Observer<T>>;
        /**
         * Create a new Observer with the specified callback and unregisters after the next notification
         * @param callback the callback that will be executed for that Observer
         * @returns the new observer created for the callback
         */
        addOnce(callback: (eventData: T, eventState: EventState) => void): Nullable<Observer<T>>;
        /**
         * Remove an Observer from the Observable object
         * @param observer the instance of the Observer to remove
         * @returns false if it doesn't belong to this Observable
         */
        remove(observer: Nullable<Observer<T>>): boolean;
        /**
         * Remove a callback from the Observable object
         * @param callback the callback to remove
         * @param scope optional scope. If used only the callbacks with this scope will be removed
         * @returns false if it doesn't belong to this Observable
         */
        removeCallback(callback: (eventData: T, eventState: EventState) => void, scope?: any): boolean;
        /**
         * @internal
         */
        _deferUnregister(observer: Observer<T>): void;
        private _remove;
        /**
         * Moves the observable to the top of the observer list making it get called first when notified
         * @param observer the observer to move
         */
        makeObserverTopPriority(observer: Observer<T>): void;
        /**
         * Moves the observable to the bottom of the observer list making it get called last when notified
         * @param observer the observer to move
         */
        makeObserverBottomPriority(observer: Observer<T>): void;
        /**
         * Notify all Observers by calling their respective callback with the given data
         * Will return true if all observers were executed, false if an observer set skipNextObservers to true, then prevent the subsequent ones to execute
         * @param eventData defines the data to send to all observers
         * @param mask defines the mask of the current notification (observers with incompatible mask (ie mask & observer.mask === 0) will not be notified)
         * @param target defines the original target of the state
         * @param currentTarget defines the current target of the state
         * @param userInfo defines any user info to send to observers
         * @returns false if the complete observer chain was not processed (because one observer set the skipNextObservers to true)
         */
        notifyObservers(eventData: T, mask?: number, target?: any, currentTarget?: any, userInfo?: any): boolean;
        /**
         * Notify a specific observer
         * @param observer defines the observer to notify
         * @param eventData defines the data to be sent to each callback
         * @param mask is used to filter observers defaults to -1
         */
        notifyObserver(observer: Observer<T>, eventData: T, mask?: number): void;
        /**
         * Gets a boolean indicating if the observable has at least one observer
         * @returns true is the Observable has at least one Observer registered
         */
        hasObservers(): boolean;
        /**
         * Clear the list of observers
         */
        clear(): void;
        /**
         * Clone the current observable
         * @returns a new observable
         */
        clone(): Observable<T>;
        /**
         * Does this observable handles observer registered with a given mask
         * @param mask defines the mask to be tested
         * @returns whether or not one observer registered with the given mask is handled
         **/
        hasSpecificMask(mask?: number): boolean;
    }


    /**
     * Represent a list of observers registered to multiple Observables object.
     */
    export class MultiObserver<T> {
        private _observers;
        private _observables;
        /**
         * Release associated resources
         */
        dispose(): void;
        /**
         * Raise a callback when one of the observable will notify
         * @param observables defines a list of observables to watch
         * @param callback defines the callback to call on notification
         * @param mask defines the mask used to filter notifications
         * @param scope defines the current scope used to restore the JS context
         * @returns the new MultiObserver
         */
        static Watch<T>(observables: Observable<T>[], callback: (eventData: T, eventState: EventState) => void, mask?: number, scope?: any): MultiObserver<T>;
    }
        interface Observable<T> {
            /**
             * Calling this will execute each callback, expecting it to be a promise or return a value.
             * If at any point in the chain one function fails, the promise will fail and the execution will not continue.
             * This is useful when a chain of events (sometimes async events) is needed to initialize a certain object
             * and it is crucial that all callbacks will be executed.
             * The order of the callbacks is kept, callbacks are not executed parallel.
             *
             * @param eventData The data to be sent to each callback
             * @param mask is used to filter observers defaults to -1
             * @param target defines the callback target (see EventState)
             * @param currentTarget defines he current object in the bubbling phase
             * @param userInfo defines any user info to send to observers
             * @returns {Promise<T>} will return a Promise than resolves when all callbacks executed successfully.
             */
            notifyObserversWithPromise(eventData: T, mask?: number, target?: any, currentTarget?: any, userInfo?: any): Promise<T>;
        }


        interface Observable<T> {
            /**
             * Internal observable-based coroutine scheduler instance.
             */
            _coroutineScheduler?: CoroutineScheduler<void>;
            /**
             * Internal disposal method for observable-based coroutine scheduler instance.
             */
            _coroutineSchedulerDispose?: () => void;
            /**
             * Runs a coroutine asynchronously on this observable
             * @param coroutine the iterator resulting from having started the coroutine
             * @returns a promise which will be resolved when the coroutine finishes or rejected if the coroutine is cancelled
             */
            runCoroutineAsync(coroutine: AsyncCoroutine<void>): Promise<void>;
            /**
             * Cancels all coroutines currently running on this observable
             */
            cancelAllCoroutines(): void;
        }


    /**
     * This class is used to track a performance counter which is number based.
     * The user has access to many properties which give statistics of different nature.
     *
     * The implementer can track two kinds of Performance Counter: time and count.
     * For time you can optionally call fetchNewFrame() to notify the start of a new frame to monitor, then call beginMonitoring() to start and endMonitoring() to record the lapsed time. endMonitoring takes a newFrame parameter for you to specify if the monitored time should be set for a new frame or accumulated to the current frame being monitored.
     * For count you first have to call fetchNewFrame() to notify the start of a new frame to monitor, then call addCount() how many time required to increment the count value you monitor.
     */
    export class PerfCounter {
        /**
         * Gets or sets a global boolean to turn on and off all the counters
         */
        static Enabled: boolean;
        /**
         * Returns the smallest value ever
         */
        get min(): number;
        /**
         * Returns the biggest value ever
         */
        get max(): number;
        /**
         * Returns the average value since the performance counter is running
         */
        get average(): number;
        /**
         * Returns the average value of the last second the counter was monitored
         */
        get lastSecAverage(): number;
        /**
         * Returns the current value
         */
        get current(): number;
        /**
         * Gets the accumulated total
         */
        get total(): number;
        /**
         * Gets the total value count
         */
        get count(): number;
        /**
         * Creates a new counter
         */
        constructor();
        /**
         * Call this method to start monitoring a new frame.
         * This scenario is typically used when you accumulate monitoring time many times for a single frame, you call this method at the start of the frame, then beginMonitoring to start recording and endMonitoring(false) to accumulated the recorded time to the PerfCounter or addCount() to accumulate a monitored count.
         */
        fetchNewFrame(): void;
        /**
         * Call this method to monitor a count of something (e.g. mesh drawn in viewport count)
         * @param newCount the count value to add to the monitored count
         * @param fetchResult true when it's the last time in the frame you add to the counter and you wish to update the statistics properties (min/max/average), false if you only want to update statistics.
         */
        addCount(newCount: number, fetchResult: boolean): void;
        /**
         * Start monitoring this performance counter
         */
        beginMonitoring(): void;
        /**
         * Compute the time lapsed since the previous beginMonitoring() call.
         * @param newFrame true by default to fetch the result and monitor a new frame, if false the time monitored will be added to the current frame counter
         */
        endMonitoring(newFrame?: boolean): void;
        private _fetchResult;
        private _startMonitoringTime;
        private _min;
        private _max;
        private _average;
        private _current;
        private _totalValueCount;
        private _totalAccumulated;
        private _lastSecAverage;
        private _lastSecAccumulated;
        private _lastSecTime;
        private _lastSecValueCount;
    }


    /**
     * Performance monitor tracks rolling average frame-time and frame-time variance over a user defined sliding-window
     */
    export class PerformanceMonitor {
        private _enabled;
        private _rollingFrameTime;
        private _lastFrameTimeMs;
        /**
         * constructor
         * @param frameSampleSize The number of samples required to saturate the sliding window
         */
        constructor(frameSampleSize?: number);
        /**
         * Samples current frame
         * @param timeMs A timestamp in milliseconds of the current frame to compare with other frames
         */
        sampleFrame(timeMs?: number): void;
        /**
         * Returns the average frame time in milliseconds over the sliding window (or the subset of frames sampled so far)
         */
        get averageFrameTime(): number;
        /**
         * Returns the variance frame time in milliseconds over the sliding window (or the subset of frames sampled so far)
         */
        get averageFrameTimeVariance(): number;
        /**
         * Returns the frame time of the most recent frame
         */
        get instantaneousFrameTime(): number;
        /**
         * Returns the average framerate in frames per second over the sliding window (or the subset of frames sampled so far)
         */
        get averageFPS(): number;
        /**
         * Returns the average framerate in frames per second using the most recent frame time
         */
        get instantaneousFPS(): number;
        /**
         * Returns true if enough samples have been taken to completely fill the sliding window
         */
        get isSaturated(): boolean;
        /**
         * Enables contributions to the sliding window sample set
         */
        enable(): void;
        /**
         * Disables contributions to the sliding window sample set
         * Samples will not be interpolated over the disabled period
         */
        disable(): void;
        /**
         * Returns true if sampling is enabled
         */
        get isEnabled(): boolean;
        /**
         * Resets performance monitor
         */
        reset(): void;
    }
    /**
     * RollingAverage
     *
     * Utility to efficiently compute the rolling average and variance over a sliding window of samples
     */
    export class RollingAverage {
        /**
         * Current average
         */
        average: number;
        /**
         * Current variance
         */
        variance: number;
        protected _samples: Array<number>;
        protected _sampleCount: number;
        protected _pos: number;
        protected _m2: number;
        /**
         * constructor
         * @param length The number of samples required to saturate the sliding window
         */
        constructor(length: number);
        /**
         * Adds a sample to the sample set
         * @param v The sample value
         */
        add(v: number): void;
        /**
         * Returns previously added values or null if outside of history or outside the sliding window domain
         * @param i Index in history. For example, pass 0 for the most recent value and 1 for the value before that
         * @returns Value previously recorded with add() or null if outside of range
         */
        history(i: number): number;
        /**
         * Returns true if enough samples have been taken to completely fill the sliding window
         * @returns true if sample-set saturated
         */
        isSaturated(): boolean;
        /**
         * Resets the rolling average (equivalent to 0 samples taken so far)
         */
        reset(): void;
        /**
         * Wraps a value around the sample range boundaries
         * @param i Position in sample range, for example if the sample length is 5, and i is -3, then 2 will be returned.
         * @returns Wrapped position in sample range
         */
        protected _wrapPosition(i: number): number;
    }


    /**
     * A class acting as a dynamic float32array used in the performance viewer
     */
    export class DynamicFloat32Array {
        private _view;
        private _itemLength;
        /**
         * Creates a new DynamicFloat32Array with the desired item capacity.
         * @param itemCapacity The initial item capacity you would like to set for the array.
         */
        constructor(itemCapacity: number);
        /**
         * The number of items currently in the array.
         */
        get itemLength(): number;
        /**
         * Gets value at index, NaN if no such index exists.
         * @param index the index to get the value at.
         * @returns the value at the index provided.
         */
        at(index: number): number;
        /**
         * Gets a view of the original array from start to end (exclusive of end).
         * @param start starting index.
         * @param end ending index.
         * @returns a subarray of the original array.
         */
        subarray(start: number, end: number): Float32Array;
        /**
         * Pushes items to the end of the array.
         * @param item The item to push into the array.
         */
        push(item: number): void;
        /**
         * Grows the array by the growth factor when necessary.
         */
        private _growArray;
    }




    /**
     * Defines the general structure of what is necessary for a collection strategy.
     */
    export interface IPerfViewerCollectionStrategy {
        /**
         * The id of the strategy.
         */
        id: string;
        /**
         * Function which gets the data for the strategy.
         */
        getData: () => number;
        /**
         * Function which does any necessary cleanup. Called when performanceViewerCollector.dispose() is called.
         */
        dispose: () => void;
    }
    /**
     * Initializer callback for a strategy
     */
    export type PerfStrategyInitialization = (scene: Scene) => IPerfViewerCollectionStrategy;
    /**
     * Defines the predefined strategies used in the performance viewer.
     */
    export class PerfCollectionStrategy {
        /**
         * Gets the initializer for the strategy used for collection of fps metrics
         * @returns the initializer for the fps strategy
         */
        static FpsStrategy(): PerfStrategyInitialization;
        /**
         * Gets the initializer for the strategy used for collection of thermal utilization metrics.
         * Needs the experimental pressure API.
         * @returns the initializer for the thermal utilization strategy
         */
        static ThermalStrategy(): PerfStrategyInitialization;
        /**
         * Gets the initializer for the strategy used for collection of power supply utilization metrics.
         * Needs the experimental pressure API.
         * @returns the initializer for the power supply utilization strategy
         */
        static PowerSupplyStrategy(): PerfStrategyInitialization;
        /**
         * Gets the initializer for the strategy used for collection of pressure metrics.
         * Needs the experimental pressure API.
         * @returns the initializer for the pressure strategy
         */
        static PressureStrategy(): PerfStrategyInitialization;
        private static _PressureStrategy;
        /**
         * Gets the initializer for the strategy used for collection of total meshes metrics.
         * @returns the initializer for the total meshes strategy
         */
        static TotalMeshesStrategy(): PerfStrategyInitialization;
        /**
         * Gets the initializer for the strategy used for collection of active meshes metrics.
         * @returns the initializer for the active meshes strategy
         */
        static ActiveMeshesStrategy(): PerfStrategyInitialization;
        /**
         * Gets the initializer for the strategy used for collection of active indices metrics.
         * @returns the initializer for the active indices strategy
         */
        static ActiveIndicesStrategy(): PerfStrategyInitialization;
        /**
         * Gets the initializer for the strategy used for collection of active faces metrics.
         * @returns the initializer for the active faces strategy
         */
        static ActiveFacesStrategy(): PerfStrategyInitialization;
        /**
         * Gets the initializer for the strategy used for collection of active bones metrics.
         * @returns the initializer for the active bones strategy
         */
        static ActiveBonesStrategy(): PerfStrategyInitialization;
        /**
         * Gets the initializer for the strategy used for collection of active particles metrics.
         * @returns the initializer for the active particles strategy
         */
        static ActiveParticlesStrategy(): PerfStrategyInitialization;
        /**
         * Gets the initializer for the strategy used for collection of draw calls metrics.
         * @returns the initializer for the draw calls strategy
         */
        static DrawCallsStrategy(): PerfStrategyInitialization;
        /**
         * Gets the initializer for the strategy used for collection of total lights metrics.
         * @returns the initializer for the total lights strategy
         */
        static TotalLightsStrategy(): PerfStrategyInitialization;
        /**
         * Gets the initializer for the strategy used for collection of total vertices metrics.
         * @returns the initializer for the total vertices strategy
         */
        static TotalVerticesStrategy(): PerfStrategyInitialization;
        /**
         * Gets the initializer for the strategy used for collection of total materials metrics.
         * @returns the initializer for the total materials strategy
         */
        static TotalMaterialsStrategy(): PerfStrategyInitialization;
        /**
         * Gets the initializer for the strategy used for collection of total textures metrics.
         * @returns the initializer for the total textures strategy
         */
        static TotalTexturesStrategy(): PerfStrategyInitialization;
        /**
         * Gets the initializer for the strategy used for collection of absolute fps metrics.
         * @returns the initializer for the absolute fps strategy
         */
        static AbsoluteFpsStrategy(): PerfStrategyInitialization;
        /**
         * Gets the initializer for the strategy used for collection of meshes selection time metrics.
         * @returns the initializer for the meshes selection time strategy
         */
        static MeshesSelectionStrategy(): PerfStrategyInitialization;
        /**
         * Gets the initializer for the strategy used for collection of render targets time metrics.
         * @returns the initializer for the render targets time strategy
         */
        static RenderTargetsStrategy(): PerfStrategyInitialization;
        /**
         * Gets the initializer for the strategy used for collection of particles time metrics.
         * @returns the initializer for the particles time strategy
         */
        static ParticlesStrategy(): PerfStrategyInitialization;
        /**
         * Gets the initializer for the strategy used for collection of sprites time metrics.
         * @returns the initializer for the sprites time strategy
         */
        static SpritesStrategy(): PerfStrategyInitialization;
        /**
         * Gets the initializer for the strategy used for collection of animations time metrics.
         * @returns the initializer for the animations time strategy
         */
        static AnimationsStrategy(): PerfStrategyInitialization;
        /**
         * Gets the initializer for the strategy used for collection of physics time metrics.
         * @returns the initializer for the physics time strategy
         */
        static PhysicsStrategy(): PerfStrategyInitialization;
        /**
         * Gets the initializer for the strategy used for collection of render time metrics.
         * @returns the initializer for the render time strategy
         */
        static RenderStrategy(): PerfStrategyInitialization;
        /**
         * Gets the initializer for the strategy used for collection of total frame time metrics.
         * @returns the initializer for the total frame time strategy
         */
        static FrameTotalStrategy(): PerfStrategyInitialization;
        /**
         * Gets the initializer for the strategy used for collection of inter-frame time metrics.
         * @returns the initializer for the inter-frame time strategy
         */
        static InterFrameStrategy(): PerfStrategyInitialization;
        /**
         * Gets the initializer for the strategy used for collection of gpu frame time metrics.
         * @returns the initializer for the gpu frame time strategy
         */
        static GpuFrameTimeStrategy(): PerfStrategyInitialization;
    }


    /**
     * Callback strategy and optional category for data collection
     */
    interface IPerformanceViewerStrategyParameter {
        /**
         * The strategy for collecting data. Available strategies are located on the PerfCollectionStrategy class
         */
        strategyCallback: PerfStrategyInitialization;
        /**
         * Category for displaying this strategy on the viewer. Can be undefined or an empty string, in which case the strategy will be displayed on top
         */
        category?: string;
        /**
         * Starts hidden
         */
        hidden?: boolean;
    }
    /**
     * The collector class handles the collection and storage of data into the appropriate array.
     * The collector also handles notifying any observers of any updates.
     */
    export class PerformanceViewerCollector {
        private _scene;
        private _datasetMeta;
        private _strategies;
        private _startingTimestamp;
        private _hasLoadedData;
        private _isStarted;
        private readonly _customEventObservable;
        private readonly _eventRestoreSet;
        /**
         * Datastructure containing the collected datasets. Warning: you should not modify the values in here, data will be of the form [timestamp, numberOfPoints, value1, value2..., timestamp, etc...]
         */
        readonly datasets: IPerfDatasets;
        /**
         * An observable you can attach to get deltas in the dataset. Subscribing to this will increase memory consumption slightly, and may hurt performance due to increased garbage collection needed.
         * Updates of slices will be of the form [timestamp, numberOfPoints, value1, value2...].
         */
        readonly datasetObservable: Observable<number[]>;
        /**
         * An observable you can attach to get the most updated map of metadatas.
         */
        readonly metadataObservable: Observable<Map<string, IPerfMetadata>>;
        /**
         * The offset for when actual data values start appearing inside a slice.
         */
        static get SliceDataOffset(): number;
        /**
         * The offset for the value of the number of points inside a slice.
         */
        static get NumberOfPointsOffset(): number;
        /**
         * Handles the creation of a performance viewer collector.
         * @param _scene the scene to collect on.
         * @param _enabledStrategyCallbacks the list of data to collect with callbacks for initialization purposes.
         */
        constructor(_scene: Scene, _enabledStrategyCallbacks?: IPerformanceViewerStrategyParameter[]);
        /**
         * Registers a custom string event which will be callable via sendEvent. This method returns an event object which will contain the id of the event.
         * The user can set a value optionally, which will be used in the sendEvent method. If the value is set, we will record this value at the end of each frame,
         * if not we will increment our counter and record the value of the counter at the end of each frame. The value recorded is 0 if no sendEvent method is called, within a frame.
         * @param name The name of the event to register
         * @param forceUpdate if the code should force add an event, and replace the last one.
         * @param category the category for that event
         * @returns The event registered, used in sendEvent
         */
        registerEvent(name: string, forceUpdate?: boolean, category?: string): IPerfCustomEvent | undefined;
        /**
         * Lets the perf collector handle an event, occurences or event value depending on if the event.value params is set.
         * @param event the event to handle an occurence for
         */
        sendEvent(event: IPerfCustomEvent): void;
        /**
         * This event restores all custom string events if necessary.
         */
        private _restoreStringEvents;
        /**
         * This method adds additional collection strategies for data collection purposes.
         * @param strategyCallbacks the list of data to collect with callbacks.
         */
        addCollectionStrategies(...strategyCallbacks: IPerformanceViewerStrategyParameter[]): void;
        /**
         * Gets a 6 character hexcode representing the colour from a passed in string.
         * @param id the string to get a hex code for.
         * @returns a hexcode hashed from the id.
         */
        private _getHexColorFromId;
        /**
         * Collects data for every dataset by using the appropriate strategy. This is called every frame.
         * This method will then notify all observers with the latest slice.
         */
        private _collectDataAtFrame;
        /**
         * Collects and then sends the latest slice to any observers by using the appropriate strategy when the user wants.
         * The slice will be of the form [timestamp, numberOfPoints, value1, value2...]
         * This method does not add onto the collected data accessible via the datasets variable.
         */
        getCurrentSlice(): void;
        /**
         * Updates a property for a dataset's metadata with the value provided.
         * @param id the id of the dataset which needs its metadata updated.
         * @param prop the property to update.
         * @param value the value to update the property with.
         */
        updateMetadata<T extends keyof IPerfMetadata>(id: string, prop: T, value: IPerfMetadata[T]): void;
        /**
         * Completely clear, data, ids, and strategies saved to this performance collector.
         * @param preserveStringEventsRestore if it should preserve the string events, by default will clear string events registered when called.
         */
        clear(preserveStringEventsRestore?: boolean): void;
        /**
         * Accessor which lets the caller know if the performance collector has data loaded from a file or not!
         * Call clear() to reset this value.
         * @returns true if the data is loaded from a file, false otherwise.
         */
        get hasLoadedData(): boolean;
        /**
         * Given a string containing file data, this function parses the file data into the datasets object.
         * It returns a boolean to indicate if this object was successfully loaded with the data.
         * @param data string content representing the file data.
         * @param keepDatasetMeta if it should use reuse the existing dataset metadata
         * @returns true if the data was successfully loaded, false otherwise.
         */
        loadFromFileData(data: string, keepDatasetMeta?: boolean): boolean;
        /**
         * Exports the datasets inside of the collector to a csv.
         */
        exportDataToCsv(): void;
        /**
         * Starts the realtime collection of data.
         * @param shouldPreserve optional boolean param, if set will preserve the dataset between calls of start.
         */
        start(shouldPreserve?: boolean): void;
        /**
         * Stops the collection of data.
         */
        stop(): void;
        /**
         * Returns if the perf collector has been started or not.
         */
        get isStarted(): boolean;
        /**
         * Disposes of the object
         */
        dispose(): void;
    }




    /**
     * Class containing a set of static utilities functions for managing Pivots
     * @internal
     */
    export class PivotTools {
        private static _PivotCached;
        private static _OldPivotPoint;
        private static _PivotTranslation;
        private static _PivotTmpVector;
        private static _PivotPostMultiplyPivotMatrix;
        /**
         * @internal
         */
        static _RemoveAndStorePivotPoint(mesh: TransformNode): void;
        /**
         * @internal
         */
        static _RestorePivotPoint(mesh: TransformNode): void;
    }


    /**
     * Class containing a set of static utilities functions for precision date
     */
    export class PrecisionDate {
        /**
         * Gets either window.performance.now() if supported or Date.now() else
         */
        static get Now(): number;
    }


    /**
     * A wrapper for the experimental pressure api which allows a callback to be called whenever certain thresholds are met.
     */
    export class PressureObserverWrapper {
        private _observer;
        private _currentState;
        /**
         * An event triggered when the cpu usage/speed meets certain thresholds.
         * Note: pressure is an experimental API.
         */
        onPressureChanged: Observable<PressureRecord[]>;
        /**
         * A pressure observer will call this callback, whenever these thresholds are met.
         * @param options An object containing the thresholds used to decide what value to to return for each update property (average of start and end of a threshold boundary).
         */
        constructor(options?: PressureObserverOptions);
        /**
         * Returns true if PressureObserver is available for use, false otherwise.
         */
        static get IsAvailable(): boolean;
        /**
         * Method that must be called to begin observing changes, and triggering callbacks.
         * @param source defines the source to observe
         */
        observe(source: PressureSource): void;
        /**
         * Method that must be called to stop observing changes and triggering callbacks (cleanup function).
         * @param source defines the source to unobserve
         */
        unobserve(source: PressureSource): void;
        /**
         * Release the associated resources.
         */
        dispose(): void;
    }


    /**
     * Class used to connect with the reflector zone of the sandbox via the reflector bridge
     * @since 5.0.0
     */
    export class Reflector {
        private static readonly _SERVER_PREFIX;
        private _scene;
        private _webSocket;
        /**
         * Constructs a reflector object.
         * @param scene The scene to use
         * @param hostname The hostname of the reflector bridge
         * @param port The port of the reflector bridge
         */
        constructor(scene: Scene, hostname: string, port: number);
        /**
         * Closes the reflector connection
         */
        close(): void;
        private _handleServerMessage;
        private _handleClientMessage;
    }


    /**
     * Class used to define a retry strategy when error happens while loading assets
     */
    export class RetryStrategy {
        /**
         * Function used to defines an exponential back off strategy
         * @param maxRetries defines the maximum number of retries (3 by default)
         * @param baseInterval defines the interval between retries
         * @returns the strategy function to use
         */
        static ExponentialBackoff(maxRetries?: number, baseInterval?: number): (url: string, request: WebRequest, retryIndex: number) => number;
    }


    /**
     * Class used to host RGBD texture specific utilities
     */
    export class RGBDTextureTools {
        /**
         * Expand the RGBD Texture from RGBD to Half Float if possible.
         * @param texture the texture to expand.
         */
        static ExpandRGBDTexture(texture: Texture): void;
        /**
         * Encode the texture to RGBD if possible.
         * @param internalTexture the texture to encode
         * @param scene the scene hosting the texture
         * @param outputTextureType type of the texture in which the encoding is performed
         * @returns a promise with the internalTexture having its texture replaced by the result of the processing
         */
        static EncodeTextureToRGBD(internalTexture: InternalTexture, scene: Scene, outputTextureType?: number): Promise<InternalTexture>;
    }


    /**
     * Defines the root class used to create scene optimization to use with SceneOptimizer
     * @description More details at https://doc.babylonjs.com/features/featuresDeepDive/scene/sceneOptimizer
     */
    export class SceneOptimization {
        /**
         * Defines the priority of this optimization (0 by default which means first in the list)
         */
        priority: number;
        /**
         * Gets a string describing the action executed by the current optimization
         * @returns description string
         */
        getDescription(): string;
        /**
         * This function will be called by the SceneOptimizer when its priority is reached in order to apply the change required by the current optimization
         * @param scene defines the current scene where to apply this optimization
         * @param optimizer defines the current optimizer
         * @returns true if everything that can be done was applied
         */
        apply(scene: Scene, optimizer: SceneOptimizer): boolean;
        /**
         * Creates the SceneOptimization object
         * @param priority defines the priority of this optimization (0 by default which means first in the list)
         */
        constructor(
        /**
         * Defines the priority of this optimization (0 by default which means first in the list)
         */
        priority?: number);
    }
    /**
     * Defines an optimization used to reduce the size of render target textures
     * @description More details at https://doc.babylonjs.com/features/featuresDeepDive/scene/sceneOptimizer
     */
    export class TextureOptimization extends SceneOptimization {
        /**
         * Defines the priority of this optimization (0 by default which means first in the list)
         */
        priority: number;
        /**
         * Defines the maximum sized allowed for textures (1024 is the default value). If a texture is bigger, it will be scaled down using a factor defined by the step parameter
         */
        maximumSize: number;
        /**
         * Defines the factor (0.5 by default) used to scale down textures bigger than maximum sized allowed.
         */
        step: number;
        /**
         * Gets a string describing the action executed by the current optimization
         * @returns description string
         */
        getDescription(): string;
        /**
         * Creates the TextureOptimization object
         * @param priority defines the priority of this optimization (0 by default which means first in the list)
         * @param maximumSize defines the maximum sized allowed for textures (1024 is the default value). If a texture is bigger, it will be scaled down using a factor defined by the step parameter
         * @param step defines the factor (0.5 by default) used to scale down textures bigger than maximum sized allowed.
         */
        constructor(
        /**
         * Defines the priority of this optimization (0 by default which means first in the list)
         */
        priority?: number, 
        /**
         * Defines the maximum sized allowed for textures (1024 is the default value). If a texture is bigger, it will be scaled down using a factor defined by the step parameter
         */
        maximumSize?: number, 
        /**
         * Defines the factor (0.5 by default) used to scale down textures bigger than maximum sized allowed.
         */
        step?: number);
        /**
         * This function will be called by the SceneOptimizer when its priority is reached in order to apply the change required by the current optimization
         * @param scene defines the current scene where to apply this optimization
         * @param optimizer defines the current optimizer
         * @returns true if everything that can be done was applied
         */
        apply(scene: Scene, optimizer: SceneOptimizer): boolean;
    }
    /**
     * Defines an optimization used to increase or decrease the rendering resolution
     * @description More details at https://doc.babylonjs.com/features/featuresDeepDive/scene/sceneOptimizer
     */
    export class HardwareScalingOptimization extends SceneOptimization {
        /**
         * Defines the priority of this optimization (0 by default which means first in the list)
         */
        priority: number;
        /**
         * Defines the maximum scale to use (2 by default)
         */
        maximumScale: number;
        /**
         * Defines the step to use between two passes (0.5 by default)
         */
        step: number;
        private _currentScale;
        private _directionOffset;
        /**
         * Gets a string describing the action executed by the current optimization
         * @returns description string
         */
        getDescription(): string;
        /**
         * Creates the HardwareScalingOptimization object
         * @param priority defines the priority of this optimization (0 by default which means first in the list)
         * @param maximumScale defines the maximum scale to use (2 by default)
         * @param step defines the step to use between two passes (0.5 by default)
         */
        constructor(
        /**
         * Defines the priority of this optimization (0 by default which means first in the list)
         */
        priority?: number, 
        /**
         * Defines the maximum scale to use (2 by default)
         */
        maximumScale?: number, 
        /**
         * Defines the step to use between two passes (0.5 by default)
         */
        step?: number);
        /**
         * This function will be called by the SceneOptimizer when its priority is reached in order to apply the change required by the current optimization
         * @param scene defines the current scene where to apply this optimization
         * @param optimizer defines the current optimizer
         * @returns true if everything that can be done was applied
         */
        apply(scene: Scene, optimizer: SceneOptimizer): boolean;
    }
    /**
     * Defines an optimization used to remove shadows
     * @description More details at https://doc.babylonjs.com/features/featuresDeepDive/scene/sceneOptimizer
     */
    export class ShadowsOptimization extends SceneOptimization {
        /**
         * Gets a string describing the action executed by the current optimization
         * @returns description string
         */
        getDescription(): string;
        /**
         * This function will be called by the SceneOptimizer when its priority is reached in order to apply the change required by the current optimization
         * @param scene defines the current scene where to apply this optimization
         * @param optimizer defines the current optimizer
         * @returns true if everything that can be done was applied
         */
        apply(scene: Scene, optimizer: SceneOptimizer): boolean;
    }
    /**
     * Defines an optimization used to turn post-processes off
     * @description More details at https://doc.babylonjs.com/features/featuresDeepDive/scene/sceneOptimizer
     */
    export class PostProcessesOptimization extends SceneOptimization {
        /**
         * Gets a string describing the action executed by the current optimization
         * @returns description string
         */
        getDescription(): string;
        /**
         * This function will be called by the SceneOptimizer when its priority is reached in order to apply the change required by the current optimization
         * @param scene defines the current scene where to apply this optimization
         * @param optimizer defines the current optimizer
         * @returns true if everything that can be done was applied
         */
        apply(scene: Scene, optimizer: SceneOptimizer): boolean;
    }
    /**
     * Defines an optimization used to turn lens flares off
     * @description More details at https://doc.babylonjs.com/features/featuresDeepDive/scene/sceneOptimizer
     */
    export class LensFlaresOptimization extends SceneOptimization {
        /**
         * Gets a string describing the action executed by the current optimization
         * @returns description string
         */
        getDescription(): string;
        /**
         * This function will be called by the SceneOptimizer when its priority is reached in order to apply the change required by the current optimization
         * @param scene defines the current scene where to apply this optimization
         * @param optimizer defines the current optimizer
         * @returns true if everything that can be done was applied
         */
        apply(scene: Scene, optimizer: SceneOptimizer): boolean;
    }
    /**
     * Defines an optimization based on user defined callback.
     * @description More details at https://doc.babylonjs.com/features/featuresDeepDive/scene/sceneOptimizer
     */
    export class CustomOptimization extends SceneOptimization {
        /**
         * Callback called to apply the custom optimization.
         */
        onApply: (scene: Scene, optimizer: SceneOptimizer) => boolean;
        /**
         * Callback called to get custom description
         */
        onGetDescription: () => string;
        /**
         * Gets a string describing the action executed by the current optimization
         * @returns description string
         */
        getDescription(): string;
        /**
         * This function will be called by the SceneOptimizer when its priority is reached in order to apply the change required by the current optimization
         * @param scene defines the current scene where to apply this optimization
         * @param optimizer defines the current optimizer
         * @returns true if everything that can be done was applied
         */
        apply(scene: Scene, optimizer: SceneOptimizer): boolean;
    }
    /**
     * Defines an optimization used to turn particles off
     * @description More details at https://doc.babylonjs.com/features/featuresDeepDive/scene/sceneOptimizer
     */
    export class ParticlesOptimization extends SceneOptimization {
        /**
         * Gets a string describing the action executed by the current optimization
         * @returns description string
         */
        getDescription(): string;
        /**
         * This function will be called by the SceneOptimizer when its priority is reached in order to apply the change required by the current optimization
         * @param scene defines the current scene where to apply this optimization
         * @param optimizer defines the current optimizer
         * @returns true if everything that can be done was applied
         */
        apply(scene: Scene, optimizer: SceneOptimizer): boolean;
    }
    /**
     * Defines an optimization used to turn render targets off
     * @description More details at https://doc.babylonjs.com/features/featuresDeepDive/scene/sceneOptimizer
     */
    export class RenderTargetsOptimization extends SceneOptimization {
        /**
         * Gets a string describing the action executed by the current optimization
         * @returns description string
         */
        getDescription(): string;
        /**
         * This function will be called by the SceneOptimizer when its priority is reached in order to apply the change required by the current optimization
         * @param scene defines the current scene where to apply this optimization
         * @param optimizer defines the current optimizer
         * @returns true if everything that can be done was applied
         */
        apply(scene: Scene, optimizer: SceneOptimizer): boolean;
    }
    /**
     * Defines an optimization used to merge meshes with compatible materials
     * @description More details at https://doc.babylonjs.com/features/featuresDeepDive/scene/sceneOptimizer
     */
    export class MergeMeshesOptimization extends SceneOptimization {
        private static _UpdateSelectionTree;
        /**
         * Gets or sets a boolean which defines if optimization octree has to be updated
         */
        static get UpdateSelectionTree(): boolean;
        /**
         * Gets or sets a boolean which defines if optimization octree has to be updated
         */
        static set UpdateSelectionTree(value: boolean);
        /**
         * Gets a string describing the action executed by the current optimization
         * @returns description string
         */
        getDescription(): string;
        private _canBeMerged;
        /**
         * This function will be called by the SceneOptimizer when its priority is reached in order to apply the change required by the current optimization
         * @param scene defines the current scene where to apply this optimization
         * @param optimizer defines the current optimizer
         * @param updateSelectionTree defines that the selection octree has to be updated (false by default)
         * @returns true if everything that can be done was applied
         */
        apply(scene: Scene, optimizer: SceneOptimizer, updateSelectionTree?: boolean): boolean;
    }
    /**
     * Defines a list of options used by SceneOptimizer
     * @description More details at https://doc.babylonjs.com/features/featuresDeepDive/scene/sceneOptimizer
     */
    export class SceneOptimizerOptions {
        /**
         * Defines the target frame rate to reach (60 by default)
         */
        targetFrameRate: number;
        /**
         * Defines the interval between two checks (2000ms by default)
         */
        trackerDuration: number;
        /**
         * Gets the list of optimizations to apply
         */
        optimizations: SceneOptimization[];
        /**
         * Creates a new list of options used by SceneOptimizer
         * @param targetFrameRate defines the target frame rate to reach (60 by default)
         * @param trackerDuration defines the interval between two checks (2000ms by default)
         */
        constructor(
        /**
         * Defines the target frame rate to reach (60 by default)
         */
        targetFrameRate?: number, 
        /**
         * Defines the interval between two checks (2000ms by default)
         */
        trackerDuration?: number);
        /**
         * Add a new optimization
         * @param optimization defines the SceneOptimization to add to the list of active optimizations
         * @returns the current SceneOptimizerOptions
         */
        addOptimization(optimization: SceneOptimization): SceneOptimizerOptions;
        /**
         * Add a new custom optimization
         * @param onApply defines the callback called to apply the custom optimization (true if everything that can be done was applied)
         * @param onGetDescription defines the callback called to get the description attached with the optimization.
         * @param priority defines the priority of this optimization (0 by default which means first in the list)
         * @returns the current SceneOptimizerOptions
         */
        addCustomOptimization(onApply: (scene: Scene, optimizer: SceneOptimizer) => boolean, onGetDescription: () => string, priority?: number): SceneOptimizerOptions;
        /**
         * Creates a list of pre-defined optimizations aimed to reduce the visual impact on the scene
         * @param targetFrameRate defines the target frame rate (60 by default)
         * @returns a SceneOptimizerOptions object
         */
        static LowDegradationAllowed(targetFrameRate?: number): SceneOptimizerOptions;
        /**
         * Creates a list of pre-defined optimizations aimed to have a moderate impact on the scene visual
         * @param targetFrameRate defines the target frame rate (60 by default)
         * @returns a SceneOptimizerOptions object
         */
        static ModerateDegradationAllowed(targetFrameRate?: number): SceneOptimizerOptions;
        /**
         * Creates a list of pre-defined optimizations aimed to have a big impact on the scene visual
         * @param targetFrameRate defines the target frame rate (60 by default)
         * @returns a SceneOptimizerOptions object
         */
        static HighDegradationAllowed(targetFrameRate?: number): SceneOptimizerOptions;
    }
    /**
     * Class used to run optimizations in order to reach a target frame rate
     * @description More details at https://doc.babylonjs.com/features/featuresDeepDive/scene/sceneOptimizer
     */
    export class SceneOptimizer implements IDisposable {
        private _isRunning;
        private _options;
        private _scene;
        private _currentPriorityLevel;
        private _targetFrameRate;
        private _trackerDuration;
        private _currentFrameRate;
        private _sceneDisposeObserver;
        private _improvementMode;
        /**
         * Defines an observable called when the optimizer reaches the target frame rate
         */
        onSuccessObservable: Observable<SceneOptimizer>;
        /**
         * Defines an observable called when the optimizer enables an optimization
         */
        onNewOptimizationAppliedObservable: Observable<SceneOptimization>;
        /**
         * Defines an observable called when the optimizer is not able to reach the target frame rate
         */
        onFailureObservable: Observable<SceneOptimizer>;
        /**
         * Gets or sets a boolean indicating if the optimizer is in improvement mode
         */
        get isInImprovementMode(): boolean;
        set isInImprovementMode(value: boolean);
        /**
         * Gets the current priority level (0 at start)
         */
        get currentPriorityLevel(): number;
        /**
         * Gets the current frame rate checked by the SceneOptimizer
         */
        get currentFrameRate(): number;
        /**
         * Gets or sets the current target frame rate (60 by default)
         */
        get targetFrameRate(): number;
        /**
         * Gets or sets the current target frame rate (60 by default)
         */
        set targetFrameRate(value: number);
        /**
         * Gets or sets the current interval between two checks (every 2000ms by default)
         */
        get trackerDuration(): number;
        /**
         * Gets or sets the current interval between two checks (every 2000ms by default)
         */
        set trackerDuration(value: number);
        /**
         * Gets the list of active optimizations
         */
        get optimizations(): SceneOptimization[];
        /**
         * Creates a new SceneOptimizer
         * @param scene defines the scene to work on
         * @param options defines the options to use with the SceneOptimizer
         * @param autoGeneratePriorities defines if priorities must be generated and not read from SceneOptimization property (true by default)
         * @param improvementMode defines if the scene optimizer must run the maximum optimization while staying over a target frame instead of trying to reach the target framerate (false by default)
         */
        constructor(scene: Scene, options?: SceneOptimizerOptions, autoGeneratePriorities?: boolean, improvementMode?: boolean);
        /**
         * Stops the current optimizer
         */
        stop(): void;
        /**
         * Reset the optimizer to initial step (current priority level = 0)
         */
        reset(): void;
        /**
         * Start the optimizer. By default it will try to reach a specific framerate
         * but if the optimizer is set with improvementMode === true then it will run all optimization while frame rate is above the target frame rate
         */
        start(): void;
        private _checkCurrentState;
        /**
         * Release all resources
         */
        dispose(): void;
        /**
         * Helper function to create a SceneOptimizer with one single line of code
         * @param scene defines the scene to work on
         * @param options defines the options to use with the SceneOptimizer
         * @param onSuccess defines a callback to call on success
         * @param onFailure defines a callback to call on failure
         * @returns the new SceneOptimizer object
         */
        static OptimizeAsync(scene: Scene, options?: SceneOptimizerOptions, onSuccess?: () => void, onFailure?: () => void): SceneOptimizer;
    }


    /**
     * Class used to record delta files between 2 scene states
     */
    export class SceneRecorder {
        private _trackedScene;
        private _savedJSON;
        /**
         * Track a given scene. This means the current scene state will be considered the original state
         * @param scene defines the scene to track
         */
        track(scene: Scene): void;
        /**
         * Get the delta between current state and original state
         * @returns a any containing the delta
         */
        getDelta(): any;
        private _compareArray;
        private _compareObjects;
        private _compareCollections;
        private static GetShadowGeneratorById;
        /**
         * Apply a given delta to a given scene
         * @param deltaJSON defines the JSON containing the delta
         * @param scene defines the scene to apply the delta to
         */
        static ApplyDelta(deltaJSON: any | string, scene: Scene): void;
        private static _ApplyPropertiesToEntity;
        private static _ApplyDeltaForEntity;
    }


    /**
     * Class used to serialize a scene into a string
     */
    export class SceneSerializer {
        /**
         * Clear cache used by a previous serialization
         */
        static ClearCache(): void;
        /**
         * Serialize a scene into a JSON compatible object
         * Note that if the current engine does not support synchronous texture reading (like WebGPU), you should use SerializeAsync instead
         * as else you may not retrieve the proper base64 encoded texture data (when using the Texture.ForceSerializeBuffers flag)
         * @param scene defines the scene to serialize
         * @returns a JSON compatible object
         */
        static Serialize(scene: Scene): any;
        private static _Serialize;
        /**
         * Serialize a scene into a JSON compatible object
         * @param scene defines the scene to serialize
         * @returns a JSON promise compatible object
         */
        static SerializeAsync(scene: Scene): Promise<any>;
        private static _CollectPromises;
        /**
         * Serialize a mesh into a JSON compatible object
         * @param toSerialize defines the mesh to serialize
         * @param withParents defines if parents must be serialized as well
         * @param withChildren defines if children must be serialized as well
         * @returns a JSON compatible object
         */
        static SerializeMesh(toSerialize: any, withParents?: boolean, withChildren?: boolean): any;
    }


    /**
     * Captures a screenshot of the current rendering
     * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/renderToPNG
     * @param engine defines the rendering engine
     * @param camera defines the source camera
     * @param size This parameter can be set to a single number or to an object with the
     * following (optional) properties: precision, width, height. If a single number is passed,
     * it will be used for both width and height. If an object is passed, the screenshot size
     * will be derived from the parameters. The precision property is a multiplier allowing
     * rendering at a higher or lower resolution
     * @param successCallback defines the callback receives a single parameter which contains the
     * screenshot as a string of base64-encoded characters. This string can be assigned to the
     * src parameter of an <img> to display it
     * @param mimeType defines the MIME type of the screenshot image (default: image/png).
     * Check your browser for supported MIME types
     * @param forceDownload force the system to download the image even if a successCallback is provided
     */
    export function CreateScreenshot(engine: Engine, camera: Camera, size: IScreenshotSize | number, successCallback?: (data: string) => void, mimeType?: string, forceDownload?: boolean): void;
    /**
     * Captures a screenshot of the current rendering
     * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/renderToPNG
     * @param engine defines the rendering engine
     * @param camera defines the source camera
     * @param size This parameter can be set to a single number or to an object with the
     * following (optional) properties: precision, width, height. If a single number is passed,
     * it will be used for both width and height. If an object is passed, the screenshot size
     * will be derived from the parameters. The precision property is a multiplier allowing
     * rendering at a higher or lower resolution
     * @param mimeType defines the MIME type of the screenshot image (default: image/png).
     * Check your browser for supported MIME types
     * @returns screenshot as a string of base64-encoded characters. This string can be assigned
     * to the src parameter of an <img> to display it
     */
    export function CreateScreenshotAsync(engine: Engine, camera: Camera, size: IScreenshotSize | number, mimeType?: string): Promise<string>;
    /**
     * Captures a screenshot of the current rendering for a specific size. This will render the entire canvas but will generate a blink (due to canvas resize)
     * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/renderToPNG
     * @param engine defines the rendering engine
     * @param camera defines the source camera
     * @param width defines the expected width
     * @param height defines the expected height
     * @param mimeType defines the MIME type of the screenshot image (default: image/png).
     * Check your browser for supported MIME types
     * @returns screenshot as a string of base64-encoded characters. This string can be assigned
     * to the src parameter of an <img> to display it
     */
    export function CreateScreenshotWithResizeAsync(engine: Engine, camera: Camera, width: number, height: number, mimeType?: string): Promise<void>;
    /**
     * Generates an image screenshot from the specified camera.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/renderToPNG
     * @param engine The engine to use for rendering
     * @param camera The camera to use for rendering
     * @param size This parameter can be set to a single number or to an object with the
     * following (optional) properties: precision, width, height. If a single number is passed,
     * it will be used for both width and height. If an object is passed, the screenshot size
     * will be derived from the parameters. The precision property is a multiplier allowing
     * rendering at a higher or lower resolution
     * @param successCallback The callback receives a single parameter which contains the
     * screenshot as a string of base64-encoded characters. This string can be assigned to the
     * src parameter of an <img> to display it
     * @param mimeType The MIME type of the screenshot image (default: image/png).
     * Check your browser for supported MIME types
     * @param samples Texture samples (default: 1)
     * @param antialiasing Whether antialiasing should be turned on or not (default: false)
     * @param fileName A name for for the downloaded file.
     * @param renderSprites Whether the sprites should be rendered or not (default: false)
     * @param enableStencilBuffer Whether the stencil buffer should be enabled or not (default: false)
     * @param useLayerMask if the camera's layer mask should be used to filter what should be rendered (default: true)
     */
    export function CreateScreenshotUsingRenderTarget(engine: Engine, camera: Camera, size: IScreenshotSize | number, successCallback?: (data: string) => void, mimeType?: string, samples?: number, antialiasing?: boolean, fileName?: string, renderSprites?: boolean, enableStencilBuffer?: boolean, useLayerMask?: boolean): void;
    /**
     * Generates an image screenshot from the specified camera.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/renderToPNG
     * @param engine The engine to use for rendering
     * @param camera The camera to use for rendering
     * @param size This parameter can be set to a single number or to an object with the
     * following (optional) properties: precision, width, height. If a single number is passed,
     * it will be used for both width and height. If an object is passed, the screenshot size
     * will be derived from the parameters. The precision property is a multiplier allowing
     * rendering at a higher or lower resolution
     * @param mimeType The MIME type of the screenshot image (default: image/png).
     * Check your browser for supported MIME types
     * @param samples Texture samples (default: 1)
     * @param antialiasing Whether antialiasing should be turned on or not (default: false)
     * @param fileName A name for for the downloaded file.
     * @param renderSprites Whether the sprites should be rendered or not (default: false)
     * @param enableStencilBuffer Whether the stencil buffer should be enabled or not (default: false)
     * @param useLayerMask if the camera's layer mask should be used to filter what should be rendered (default: true)
     * @returns screenshot as a string of base64-encoded characters. This string can be assigned
     * to the src parameter of an <img> to display it
     */
    export function CreateScreenshotUsingRenderTargetAsync(engine: Engine, camera: Camera, size: IScreenshotSize | number, mimeType?: string, samples?: number, antialiasing?: boolean, fileName?: string, renderSprites?: boolean, enableStencilBuffer?: boolean, useLayerMask?: boolean): Promise<string>;
    /**
     * Class containing a set of static utilities functions for screenshots
     */
    export var ScreenshotTools: {
        /**
         * Captures a screenshot of the current rendering
         * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/renderToPNG
         * @param engine defines the rendering engine
         * @param camera defines the source camera
         * @param size This parameter can be set to a single number or to an object with the
         * following (optional) properties: precision, width, height. If a single number is passed,
         * it will be used for both width and height. If an object is passed, the screenshot size
         * will be derived from the parameters. The precision property is a multiplier allowing
         * rendering at a higher or lower resolution
         * @param successCallback defines the callback receives a single parameter which contains the
         * screenshot as a string of base64-encoded characters. This string can be assigned to the
         * src parameter of an <img> to display it
         * @param mimeType defines the MIME type of the screenshot image (default: image/png).
         * Check your browser for supported MIME types
         * @param forceDownload force the system to download the image even if a successCallback is provided
         */
        CreateScreenshot: typeof CreateScreenshot;
        /**
         * Captures a screenshot of the current rendering
         * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/renderToPNG
         * @param engine defines the rendering engine
         * @param camera defines the source camera
         * @param size This parameter can be set to a single number or to an object with the
         * following (optional) properties: precision, width, height. If a single number is passed,
         * it will be used for both width and height. If an object is passed, the screenshot size
         * will be derived from the parameters. The precision property is a multiplier allowing
         * rendering at a higher or lower resolution
         * @param mimeType defines the MIME type of the screenshot image (default: image/png).
         * Check your browser for supported MIME types
         * @returns screenshot as a string of base64-encoded characters. This string can be assigned
         * to the src parameter of an <img> to display it
         */
        CreateScreenshotAsync: typeof CreateScreenshotAsync;
        /**
         * Captures a screenshot of the current rendering for a specific size. This will render the entire canvas but will generate a blink (due to canvas resize)
         * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/renderToPNG
         * @param engine defines the rendering engine
         * @param camera defines the source camera
         * @param width defines the expected width
         * @param height defines the expected height
         * @param mimeType defines the MIME type of the screenshot image (default: image/png).
         * Check your browser for supported MIME types
         * @returns screenshot as a string of base64-encoded characters. This string can be assigned
         * to the src parameter of an <img> to display it
         */
        CreateScreenshotWithResizeAsync: typeof CreateScreenshotWithResizeAsync;
        /**
         * Generates an image screenshot from the specified camera.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/renderToPNG
         * @param engine The engine to use for rendering
         * @param camera The camera to use for rendering
         * @param size This parameter can be set to a single number or to an object with the
         * following (optional) properties: precision, width, height. If a single number is passed,
         * it will be used for both width and height. If an object is passed, the screenshot size
         * will be derived from the parameters. The precision property is a multiplier allowing
         * rendering at a higher or lower resolution
         * @param successCallback The callback receives a single parameter which contains the
         * screenshot as a string of base64-encoded characters. This string can be assigned to the
         * src parameter of an <img> to display it
         * @param mimeType The MIME type of the screenshot image (default: image/png).
         * Check your browser for supported MIME types
         * @param samples Texture samples (default: 1)
         * @param antialiasing Whether antialiasing should be turned on or not (default: false)
         * @param fileName A name for for the downloaded file.
         * @param renderSprites Whether the sprites should be rendered or not (default: false)
         * @param enableStencilBuffer Whether the stencil buffer should be enabled or not (default: false)
         */
        CreateScreenshotUsingRenderTarget: typeof CreateScreenshotUsingRenderTarget;
        /**
         * Generates an image screenshot from the specified camera.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/renderToPNG
         * @param engine The engine to use for rendering
         * @param camera The camera to use for rendering
         * @param size This parameter can be set to a single number or to an object with the
         * following (optional) properties: precision, width, height. If a single number is passed,
         * it will be used for both width and height. If an object is passed, the screenshot size
         * will be derived from the parameters. The precision property is a multiplier allowing
         * rendering at a higher or lower resolution
         * @param mimeType The MIME type of the screenshot image (default: image/png).
         * Check your browser for supported MIME types
         * @param samples Texture samples (default: 1)
         * @param antialiasing Whether antialiasing should be turned on or not (default: false)
         * @param fileName A name for for the downloaded file.
         * @param renderSprites Whether the sprites should be rendered or not (default: false)
         * @returns screenshot as a string of base64-encoded characters. This string can be assigned
         * to the src parameter of an <img> to display it
         */
        CreateScreenshotUsingRenderTargetAsync: typeof CreateScreenshotUsingRenderTargetAsync;
    };


    /**
     * Defines an array and its length.
     * It can be helpful to group result from both Arrays and smart arrays in one structure.
     */
    export interface ISmartArrayLike<T> {
        /**
         * The data of the array.
         */
        data: Array<T>;
        /**
         * The active length of the array.
         */
        length: number;
    }
    /**
     * Defines an GC Friendly array where the backfield array do not shrink to prevent over allocations.
     */
    export class SmartArray<T> implements ISmartArrayLike<T> {
        /**
         * The full set of data from the array.
         */
        data: Array<T>;
        /**
         * The active length of the array.
         */
        length: number;
        protected _id: number;
        /**
         * Instantiates a Smart Array.
         * @param capacity defines the default capacity of the array.
         */
        constructor(capacity: number);
        /**
         * Pushes a value at the end of the active data.
         * @param value defines the object to push in the array.
         */
        push(value: T): void;
        /**
         * Iterates over the active data and apply the lambda to them.
         * @param func defines the action to apply on each value.
         */
        forEach(func: (content: T) => void): void;
        /**
         * Sorts the full sets of data.
         * @param compareFn defines the comparison function to apply.
         */
        sort(compareFn: (a: T, b: T) => number): void;
        /**
         * Resets the active data to an empty array.
         */
        reset(): void;
        /**
         * Releases all the data from the array as well as the array.
         */
        dispose(): void;
        /**
         * Concats the active data with a given array.
         * @param array defines the data to concatenate with.
         */
        concat(array: any): void;
        /**
         * Returns the position of a value in the active data.
         * @param value defines the value to find the index for
         * @returns the index if found in the active data otherwise -1
         */
        indexOf(value: T): number;
        /**
         * Returns whether an element is part of the active data.
         * @param value defines the value to look for
         * @returns true if found in the active data otherwise false
         */
        contains(value: T): boolean;
        private static _GlobalId;
    }
    /**
     * Defines an GC Friendly array where the backfield array do not shrink to prevent over allocations.
     * The data in this array can only be present once
     */
    export class SmartArrayNoDuplicate<T> extends SmartArray<T> {
        private _duplicateId;
        /**
         * Pushes a value at the end of the active data.
         * THIS DOES NOT PREVENT DUPPLICATE DATA
         * @param value defines the object to push in the array.
         */
        push(value: T): void;
        /**
         * Pushes a value at the end of the active data.
         * If the data is already present, it won t be added again
         * @param value defines the object to push in the array.
         * @returns true if added false if it was already present
         */
        pushNoDuplicate(value: T): boolean;
        /**
         * Resets the active data to an empty array.
         */
        reset(): void;
        /**
         * Concats the active data with a given array.
         * This ensures no duplicate will be present in the result.
         * @param array defines the data to concatenate with.
         */
        concatWithNoDuplicate(array: any): void;
    }


    /**
     * This class implement a typical dictionary using a string as key and the generic type T as value.
     * The underlying implementation relies on an associative array to ensure the best performances.
     * The value can be anything including 'null' but except 'undefined'
     */
    export class StringDictionary<T> {
        /**
         * This will clear this dictionary and copy the content from the 'source' one.
         * If the T value is a custom object, it won't be copied/cloned, the same object will be used
         * @param source the dictionary to take the content from and copy to this dictionary
         */
        copyFrom(source: StringDictionary<T>): void;
        /**
         * Get a value based from its key
         * @param key the given key to get the matching value from
         * @returns the value if found, otherwise undefined is returned
         */
        get(key: string): T | undefined;
        /**
         * Get a value from its key or add it if it doesn't exist.
         * This method will ensure you that a given key/data will be present in the dictionary.
         * @param key the given key to get the matching value from
         * @param factory the factory that will create the value if the key is not present in the dictionary.
         * The factory will only be invoked if there's no data for the given key.
         * @returns the value corresponding to the key.
         */
        getOrAddWithFactory(key: string, factory: (key: string) => T): T;
        /**
         * Get a value from its key if present in the dictionary otherwise add it
         * @param key the key to get the value from
         * @param val if there's no such key/value pair in the dictionary add it with this value
         * @returns the value corresponding to the key
         */
        getOrAdd(key: string, val: T): T;
        /**
         * Check if there's a given key in the dictionary
         * @param key the key to check for
         * @returns true if the key is present, false otherwise
         */
        contains(key: string): boolean;
        /**
         * Add a new key and its corresponding value
         * @param key the key to add
         * @param value the value corresponding to the key
         * @returns true if the operation completed successfully, false if we couldn't insert the key/value because there was already this key in the dictionary
         */
        add(key: string, value: T): boolean;
        /**
         * Update a specific value associated to a key
         * @param key defines the key to use
         * @param value defines the value to store
         * @returns true if the value was updated (or false if the key was not found)
         */
        set(key: string, value: T): boolean;
        /**
         * Get the element of the given key and remove it from the dictionary
         * @param key defines the key to search
         * @returns the value associated with the key or null if not found
         */
        getAndRemove(key: string): Nullable<T>;
        /**
         * Remove a key/value from the dictionary.
         * @param key the key to remove
         * @returns true if the item was successfully deleted, false if no item with such key exist in the dictionary
         */
        remove(key: string): boolean;
        /**
         * Clear the whole content of the dictionary
         */
        clear(): void;
        /**
         * Gets the current count
         */
        get count(): number;
        /**
         * Execute a callback on each key/val of the dictionary.
         * Note that you can remove any element in this dictionary in the callback implementation
         * @param callback the callback to execute on a given key/value pair
         */
        forEach(callback: (key: string, val: T) => void): void;
        /**
         * Execute a callback on every occurrence of the dictionary until it returns a valid TRes object.
         * If the callback returns null or undefined the method will iterate to the next key/value pair
         * Note that you can remove any element in this dictionary in the callback implementation
         * @param callback the callback to execute, if it return a valid T instanced object the enumeration will stop and the object will be returned
         * @returns the first item
         */
        first<TRes>(callback: (key: string, val: T) => TRes): NonNullable<TRes> | null;
        private _count;
        private _data;
    }


    /**
     * Checks for a matching suffix at the end of a string (for ES5 and lower)
     * @param str Source string
     * @param suffix Suffix to search for in the source string
     * @returns Boolean indicating whether the suffix was found (true) or not (false)
     * @deprecated Please use native string function instead
     */
    export const EndsWith: (str: string, suffix: string) => boolean;
    /**
     * Checks for a matching suffix at the beginning of a string (for ES5 and lower)
     * @param str Source string
     * @param suffix Suffix to search for in the source string
     * @returns Boolean indicating whether the suffix was found (true) or not (false)
     * @deprecated Please use native string function instead
     */
    export const StartsWith: (str: string, suffix: string) => boolean;
    /**
     * Decodes a buffer into a string
     * @param buffer The buffer to decode
     * @returns The decoded string
     */
    export const Decode: (buffer: Uint8Array | Uint16Array) => string;
    /**
     * Encode a buffer to a base64 string
     * @param buffer defines the buffer to encode
     * @returns the encoded string
     */
    export const EncodeArrayBufferToBase64: (buffer: ArrayBuffer | ArrayBufferView) => string;
    /**
     * Converts a given base64 string as an ASCII encoded stream of data
     * @param base64Data The base64 encoded string to decode
     * @returns Decoded ASCII string
     */
    export const DecodeBase64ToString: (base64Data: string) => string;
    /**
     * Converts a given base64 string into an ArrayBuffer of raw byte data
     * @param base64Data The base64 encoded string to decode
     * @returns ArrayBuffer of byte data
     */
    export const DecodeBase64ToBinary: (base64Data: string) => ArrayBuffer;
    /**
     * Converts a number to string and pads with preceding zeroes until it is of specified length.
     * @param num the number to convert and pad
     * @param length the expected length of the string
     * @returns the padded string
     */
    export const PadNumber: (num: number, length: number) => string;
    /**
     * Helper to manipulate strings
     */
    export var StringTools: {
        EndsWith: (str: string, suffix: string) => boolean;
        StartsWith: (str: string, suffix: string) => boolean;
        Decode: (buffer: Uint8Array | Uint16Array) => string;
        EncodeArrayBufferToBase64: (buffer: ArrayBuffer | ArrayBufferView) => string;
        DecodeBase64ToString: (base64Data: string) => string;
        DecodeBase64ToBinary: (base64Data: string) => ArrayBuffer;
        PadNumber: (num: number, length: number) => string;
    };


    /**
     * Class used to store custom tags
     */
    export class Tags {
        /**
         * Adds support for tags on the given object
         * @param obj defines the object to use
         */
        static EnableFor(obj: any): void;
        /**
         * Removes tags support
         * @param obj defines the object to use
         */
        static DisableFor(obj: any): void;
        /**
         * Gets a boolean indicating if the given object has tags
         * @param obj defines the object to use
         * @returns a boolean
         */
        static HasTags(obj: any): boolean;
        /**
         * Gets the tags available on a given object
         * @param obj defines the object to use
         * @param asString defines if the tags must be returned as a string instead of an array of strings
         * @returns the tags
         */
        static GetTags(obj: any, asString?: boolean): any;
        /**
         * Adds tags to an object
         * @param obj defines the object to use
         * @param tagsString defines the tag string. The tags 'true' and 'false' are reserved and cannot be used as tags.
         * A tag cannot start with '||', '&&', and '!'. It cannot contain whitespaces
         */
        static AddTagsTo(obj: any, tagsString: string): void;
        /**
         * @internal
         */
        static _AddTagTo(obj: any, tag: string): void;
        /**
         * Removes specific tags from a specific object
         * @param obj defines the object to use
         * @param tagsString defines the tags to remove
         */
        static RemoveTagsFrom(obj: any, tagsString: string): void;
        /**
         * @internal
         */
        static _RemoveTagFrom(obj: any, tag: string): void;
        /**
         * Defines if tags hosted on an object match a given query
         * @param obj defines the object to use
         * @param tagsQuery defines the tag query
         * @returns a boolean
         */
        static MatchesQuery(obj: any, tagsQuery: string): boolean;
    }


    /**
     * Uses the GPU to create a copy texture rescaled at a given size
     * @param texture Texture to copy from
     * @param width defines the desired width
     * @param height defines the desired height
     * @param useBilinearMode defines if bilinear mode has to be used
     * @returns the generated texture
     */
    export function CreateResizedCopy(texture: Texture, width: number, height: number, useBilinearMode?: boolean): Texture;
    /**
     * Apply a post process to a texture
     * @param postProcessName name of the fragment post process
     * @param internalTexture the texture to encode
     * @param scene the scene hosting the texture
     * @param type type of the output texture. If not provided, use the one from internalTexture
     * @param samplingMode sampling mode to use to sample the source texture. If not provided, use the one from internalTexture
     * @param format format of the output texture. If not provided, use the one from internalTexture
     * @returns a promise with the internalTexture having its texture replaced by the result of the processing
     */
    export function ApplyPostProcess(postProcessName: string, internalTexture: InternalTexture, scene: Scene, type?: number, samplingMode?: number, format?: number): Promise<InternalTexture>;
    /**
     * Converts a number to half float
     * @param value number to convert
     * @returns converted number
     */
    export function ToHalfFloat(value: number): number;
    /**
     * Converts a half float to a number
     * @param value half float to convert
     * @returns converted half float
     */
    export function FromHalfFloat(value: number): number;
    /**
     * Class used to host texture specific utilities
     */
    export var TextureTools: {
        /**
         * Uses the GPU to create a copy texture rescaled at a given size
         * @param texture Texture to copy from
         * @param width defines the desired width
         * @param height defines the desired height
         * @param useBilinearMode defines if bilinear mode has to be used
         * @returns the generated texture
         */
        CreateResizedCopy: typeof CreateResizedCopy;
        /**
         * Apply a post process to a texture
         * @param postProcessName name of the fragment post process
         * @param internalTexture the texture to encode
         * @param scene the scene hosting the texture
         * @param type type of the output texture. If not provided, use the one from internalTexture
         * @param samplingMode sampling mode to use to sample the source texture. If not provided, use the one from internalTexture
         * @param format format of the output texture. If not provided, use the one from internalTexture
         * @returns a promise with the internalTexture having its texture replaced by the result of the processing
         */
        ApplyPostProcess: typeof ApplyPostProcess;
        /**
         * Converts a number to half float
         * @param value number to convert
         * @returns converted number
         */
        ToHalfFloat: typeof ToHalfFloat;
        /**
         * Converts a half float to a number
         * @param value half float to convert
         * @returns converted half float
         */
        FromHalfFloat: typeof FromHalfFloat;
    };


    /**
     * Gets the header of a TGA file
     * @param data defines the TGA data
     * @returns the header
     */
    export function GetTGAHeader(data: Uint8Array): any;
    /**
     * Uploads TGA content to a Babylon Texture
     * @internal
     */
    export function UploadContent(texture: InternalTexture, data: Uint8Array): void;
    /**
     * @internal
     */
    function _getImageData8bits(header: any, palettes: Uint8Array, pixel_data: Uint8Array, y_start: number, y_step: number, y_end: number, x_start: number, x_step: number, x_end: number): Uint8Array;
    /**
     * @internal
     */
    function _getImageData16bits(header: any, palettes: Uint8Array, pixel_data: Uint8Array, y_start: number, y_step: number, y_end: number, x_start: number, x_step: number, x_end: number): Uint8Array;
    /**
     * @internal
     */
    function _getImageData24bits(header: any, palettes: Uint8Array, pixel_data: Uint8Array, y_start: number, y_step: number, y_end: number, x_start: number, x_step: number, x_end: number): Uint8Array;
    /**
     * @internal
     */
    function _getImageData32bits(header: any, palettes: Uint8Array, pixel_data: Uint8Array, y_start: number, y_step: number, y_end: number, x_start: number, x_step: number, x_end: number): Uint8Array;
    /**
     * @internal
     */
    function _getImageDataGrey8bits(header: any, palettes: Uint8Array, pixel_data: Uint8Array, y_start: number, y_step: number, y_end: number, x_start: number, x_step: number, x_end: number): Uint8Array;
    /**
     * @internal
     */
    function _getImageDataGrey16bits(header: any, palettes: Uint8Array, pixel_data: Uint8Array, y_start: number, y_step: number, y_end: number, x_start: number, x_step: number, x_end: number): Uint8Array;
    /**
     * Based on jsTGALoader - Javascript loader for TGA file
     * By Vincent Thibault
     * @see http://blog.robrowser.com/javascript-tga-loader.html
     */
    export var TGATools: {
        /**
         * Gets the header of a TGA file
         * @param data defines the TGA data
         * @returns the header
         */
        GetTGAHeader: typeof GetTGAHeader;
        /**
         * Uploads TGA content to a Babylon Texture
         * @internal
         */
        UploadContent: typeof UploadContent;
        /** @internal */
        _getImageData8bits: typeof _getImageData8bits;
        /** @internal */
        _getImageData16bits: typeof _getImageData16bits;
        /** @internal */
        _getImageData24bits: typeof _getImageData24bits;
        /** @internal */
        _getImageData32bits: typeof _getImageData32bits;
        /** @internal */
        _getImageDataGrey8bits: typeof _getImageDataGrey8bits;
        /** @internal */
        _getImageDataGrey16bits: typeof _getImageDataGrey16bits;
    };


    /**
     * Construction options for a timer
     */
    export interface ITimerOptions<T> {
        /**
         * Time-to-end
         */
        timeout: number;
        /**
         * The context observable is used to calculate time deltas and provides the context of the timer's callbacks. Will usually be OnBeforeRenderObservable.
         * Countdown calculation is done ONLY when the observable is notifying its observers, meaning that if
         * you choose an observable that doesn't trigger too often, the wait time might extend further than the requested max time
         */
        contextObservable: Observable<T>;
        /**
         * Optional parameters when adding an observer to the observable
         */
        observableParameters?: {
            mask?: number;
            insertFirst?: boolean;
            scope?: any;
        };
        /**
         * An optional break condition that will stop the times prematurely. In this case onEnded will not be triggered!
         */
        breakCondition?: (data?: ITimerData<T>) => boolean;
        /**
         * Will be triggered when the time condition has met
         */
        onEnded?: (data: ITimerData<any>) => void;
        /**
         * Will be triggered when the break condition has met (prematurely ended)
         */
        onAborted?: (data: ITimerData<any>) => void;
        /**
         * Optional function to execute on each tick (or count)
         */
        onTick?: (data: ITimerData<any>) => void;
    }
    /**
     * An interface defining the data sent by the timer
     */
    export interface ITimerData<T> {
        /**
         * When did it start
         */
        startTime: number;
        /**
         * Time now
         */
        currentTime: number;
        /**
         * Time passed since started
         */
        deltaTime: number;
        /**
         * How much is completed, in [0.0...1.0].
         * Note that this CAN be higher than 1 due to the fact that we don't actually measure time but delta between observable calls
         */
        completeRate: number;
        /**
         * What the registered observable sent in the last count
         */
        payload: T;
    }
    /**
     * The current state of the timer
     */
    export enum TimerState {
        /**
         * Timer initialized, not yet started
         */
        INIT = 0,
        /**
         * Timer started and counting
         */
        STARTED = 1,
        /**
         * Timer ended (whether aborted or time reached)
         */
        ENDED = 2
    }
    /**
     * A simple version of the timer. Will take options and start the timer immediately after calling it
     *
     * @param options options with which to initialize this timer
     */
    export function setAndStartTimer(options: ITimerOptions<any>): Nullable<Observer<any>>;
    /**
     * An advanced implementation of a timer class
     */
    export class AdvancedTimer<T = any> implements IDisposable {
        /**
         * Will notify each time the timer calculates the remaining time
         */
        onEachCountObservable: Observable<ITimerData<T>>;
        /**
         * Will trigger when the timer was aborted due to the break condition
         */
        onTimerAbortedObservable: Observable<ITimerData<T>>;
        /**
         * Will trigger when the timer ended successfully
         */
        onTimerEndedObservable: Observable<ITimerData<T>>;
        /**
         * Will trigger when the timer state has changed
         */
        onStateChangedObservable: Observable<TimerState>;
        private _observer;
        private _contextObservable;
        private _observableParameters;
        private _startTime;
        private _timer;
        private _state;
        private _breakCondition;
        private _timeToEnd;
        private _breakOnNextTick;
        /**
         * Will construct a new advanced timer based on the options provided. Timer will not start until start() is called.
         * @param options construction options for this advanced timer
         */
        constructor(options: ITimerOptions<T>);
        /**
         * set a breaking condition for this timer. Default is to never break during count
         * @param predicate the new break condition. Returns true to break, false otherwise
         */
        set breakCondition(predicate: (data: ITimerData<T>) => boolean);
        /**
         * Reset ALL associated observables in this advanced timer
         */
        clearObservables(): void;
        /**
         * Will start a new iteration of this timer. Only one instance of this timer can run at a time.
         *
         * @param timeToEnd how much time to measure until timer ended
         */
        start(timeToEnd?: number): void;
        /**
         * Will force a stop on the next tick.
         */
        stop(): void;
        /**
         * Dispose this timer, clearing all resources
         */
        dispose(): void;
        private _setState;
        private _tick;
        private _stop;
    }


    /**
     * Class used to provide helper for timing
     */
    export class TimingTools {
        /**
         * Polyfill for setImmediate
         * @param action defines the action to execute after the current execution block
         */
        static SetImmediate(action: () => void): void;
    }


    /**
     * Class containing a set of static utilities functions
     */
    export class Tools {
        /**
         * Gets or sets the base URL to use to load assets
         */
        static get BaseUrl(): string;
        static set BaseUrl(value: string);
        /**
         * Enable/Disable Custom HTTP Request Headers globally.
         * default = false
         * @see CustomRequestHeaders
         */
        static UseCustomRequestHeaders: boolean;
        /**
         * Custom HTTP Request Headers to be sent with XMLHttpRequests
         * i.e. when loading files, where the server/service expects an Authorization header
         */
        static CustomRequestHeaders: {
            [key: string]: string;
        };
        /**
         * Gets or sets the retry strategy to apply when an error happens while loading an asset
         */
        static get DefaultRetryStrategy(): (url: string, request: WebRequest, retryIndex: number) => number;
        static set DefaultRetryStrategy(strategy: (url: string, request: WebRequest, retryIndex: number) => number);
        /**
         * Default behaviour for cors in the application.
         * It can be a string if the expected behavior is identical in the entire app.
         * Or a callback to be able to set it per url or on a group of them (in case of Video source for instance)
         */
        static get CorsBehavior(): string | ((url: string | string[]) => string);
        static set CorsBehavior(value: string | ((url: string | string[]) => string));
        /**
         * Gets or sets a global variable indicating if fallback texture must be used when a texture cannot be loaded
         * @ignorenaming
         */
        static get UseFallbackTexture(): boolean;
        static set UseFallbackTexture(value: boolean);
        /**
         * Use this object to register external classes like custom textures or material
         * to allow the loaders to instantiate them
         */
        static get RegisteredExternalClasses(): {
            [key: string]: Object;
        };
        static set RegisteredExternalClasses(classes: {
            [key: string]: Object;
        });
        /**
         * Texture content used if a texture cannot loaded
         * @ignorenaming
         */
        static get fallbackTexture(): string;
        static set fallbackTexture(value: string);
        /**
         * Read the content of a byte array at a specified coordinates (taking in account wrapping)
         * @param u defines the coordinate on X axis
         * @param v defines the coordinate on Y axis
         * @param width defines the width of the source data
         * @param height defines the height of the source data
         * @param pixels defines the source byte array
         * @param color defines the output color
         */
        static FetchToRef(u: number, v: number, width: number, height: number, pixels: Uint8Array, color: IColor4Like): void;
        /**
         * Interpolates between a and b via alpha
         * @param a The lower value (returned when alpha = 0)
         * @param b The upper value (returned when alpha = 1)
         * @param alpha The interpolation-factor
         * @returns The mixed value
         */
        static Mix(a: number, b: number, alpha: number): number;
        /**
         * Tries to instantiate a new object from a given class name
         * @param className defines the class name to instantiate
         * @returns the new object or null if the system was not able to do the instantiation
         */
        static Instantiate(className: string): any;
        /**
         * Polyfill for setImmediate
         * @param action defines the action to execute after the current execution block
         */
        static SetImmediate(action: () => void): void;
        /**
         * Function indicating if a number is an exponent of 2
         * @param value defines the value to test
         * @returns true if the value is an exponent of 2
         */
        static IsExponentOfTwo(value: number): boolean;
        private static _TmpFloatArray;
        /**
         * Returns the nearest 32-bit single precision float representation of a Number
         * @param value A Number.  If the parameter is of a different type, it will get converted
         * to a number or to NaN if it cannot be converted
         * @returns number
         */
        static FloatRound(value: number): number;
        /**
         * Extracts the filename from a path
         * @param path defines the path to use
         * @returns the filename
         */
        static GetFilename(path: string): string;
        /**
         * Extracts the "folder" part of a path (everything before the filename).
         * @param uri The URI to extract the info from
         * @param returnUnchangedIfNoSlash Do not touch the URI if no slashes are present
         * @returns The "folder" part of the path
         */
        static GetFolderPath(uri: string, returnUnchangedIfNoSlash?: boolean): string;
        /**
         * Extracts text content from a DOM element hierarchy
         * Back Compat only, please use GetDOMTextContent instead.
         */
        static GetDOMTextContent: typeof GetDOMTextContent;
        /**
         * Convert an angle in radians to degrees
         * @param angle defines the angle to convert
         * @returns the angle in degrees
         */
        static ToDegrees(angle: number): number;
        /**
         * Convert an angle in degrees to radians
         * @param angle defines the angle to convert
         * @returns the angle in radians
         */
        static ToRadians(angle: number): number;
        /**
         * Smooth angle changes (kind of low-pass filter), in particular for device orientation "shaking"
         * Use trigonometric functions to avoid discontinuity (0/360, -180/180)
         * @param previousAngle defines last angle value, in degrees
         * @param newAngle defines new angle value, in degrees
         * @param smoothFactor defines smoothing sensitivity; min 0: no smoothing, max 1: new data ignored
         * @returns the angle in degrees
         */
        static SmoothAngleChange(previousAngle: number, newAngle: number, smoothFactor?: number): number;
        /**
         * Returns an array if obj is not an array
         * @param obj defines the object to evaluate as an array
         * @param allowsNullUndefined defines a boolean indicating if obj is allowed to be null or undefined
         * @returns either obj directly if obj is an array or a new array containing obj
         */
        static MakeArray(obj: any, allowsNullUndefined?: boolean): Nullable<Array<any>>;
        /**
         * Gets the pointer prefix to use
         * @param engine defines the engine we are finding the prefix for
         * @returns "pointer" if touch is enabled. Else returns "mouse"
         */
        static GetPointerPrefix(engine: Engine): string;
        /**
         * Sets the cors behavior on a dom element. This will add the required Tools.CorsBehavior to the element.
         * @param url define the url we are trying
         * @param element define the dom element where to configure the cors policy
         * @param element.crossOrigin
         */
        static SetCorsBehavior(url: string | string[], element: {
            crossOrigin: string | null;
        }): void;
        /**
         * Sets the referrerPolicy behavior on a dom element.
         * @param referrerPolicy define the referrer policy to use
         * @param element define the dom element where to configure the referrer policy
         * @param element.referrerPolicy
         */
        static SetReferrerPolicyBehavior(referrerPolicy: Nullable<ReferrerPolicy>, element: {
            referrerPolicy: string | null;
        }): void;
        /**
         * Removes unwanted characters from an url
         * @param url defines the url to clean
         * @returns the cleaned url
         */
        static CleanUrl(url: string): string;
        /**
         * Gets or sets a function used to pre-process url before using them to load assets
         */
        static get PreprocessUrl(): (url: string) => string;
        static set PreprocessUrl(processor: (url: string) => string);
        /**
         * Loads an image as an HTMLImageElement.
         * @param input url string, ArrayBuffer, or Blob to load
         * @param onLoad callback called when the image successfully loads
         * @param onError callback called when the image fails to load
         * @param offlineProvider offline provider for caching
         * @param mimeType optional mime type
         * @param imageBitmapOptions optional the options to use when creating an ImageBitmap
         * @returns the HTMLImageElement of the loaded image
         */
        static LoadImage(input: string | ArrayBuffer | Blob, onLoad: (img: HTMLImageElement | ImageBitmap) => void, onError: (message?: string, exception?: any) => void, offlineProvider: Nullable<IOfflineProvider>, mimeType?: string, imageBitmapOptions?: ImageBitmapOptions): Nullable<HTMLImageElement>;
        /**
         * Loads a file from a url
         * @param url url string, ArrayBuffer, or Blob to load
         * @param onSuccess callback called when the file successfully loads
         * @param onProgress callback called while file is loading (if the server supports this mode)
         * @param offlineProvider defines the offline provider for caching
         * @param useArrayBuffer defines a boolean indicating that date must be returned as ArrayBuffer
         * @param onError callback called when the file fails to load
         * @returns a file request object
         */
        static LoadFile(url: string, onSuccess: (data: string | ArrayBuffer, responseURL?: string) => void, onProgress?: (data: any) => void, offlineProvider?: IOfflineProvider, useArrayBuffer?: boolean, onError?: (request?: WebRequest, exception?: any) => void): IFileRequest;
        /**
         * Loads a file from a url
         * @param url the file url to load
         * @param useArrayBuffer defines a boolean indicating that date must be returned as ArrayBuffer
         * @returns a promise containing an ArrayBuffer corresponding to the loaded file
         */
        static LoadFileAsync(url: string, useArrayBuffer?: boolean): Promise<ArrayBuffer | string>;
        /**
         * Load a script (identified by an url). When the url returns, the
         * content of this file is added into a new script element, attached to the DOM (body element)
         * @param scriptUrl defines the url of the script to laod
         * @param onSuccess defines the callback called when the script is loaded
         * @param onError defines the callback to call if an error occurs
         * @param scriptId defines the id of the script element
         */
        static LoadScript(scriptUrl: string, onSuccess: () => void, onError?: (message?: string, exception?: any) => void, scriptId?: string): void;
        /**
         * Load an asynchronous script (identified by an url). When the url returns, the
         * content of this file is added into a new script element, attached to the DOM (body element)
         * @param scriptUrl defines the url of the script to laod
         * @returns a promise request object
         */
        static LoadScriptAsync(scriptUrl: string): Promise<void>;
        /**
         * Loads a file from a blob
         * @param fileToLoad defines the blob to use
         * @param callback defines the callback to call when data is loaded
         * @param progressCallback defines the callback to call during loading process
         * @returns a file request object
         */
        static ReadFileAsDataURL(fileToLoad: Blob, callback: (data: any) => void, progressCallback: (ev: ProgressEvent) => any): IFileRequest;
        /**
         * Reads a file from a File object
         * @param file defines the file to load
         * @param onSuccess defines the callback to call when data is loaded
         * @param onProgress defines the callback to call during loading process
         * @param useArrayBuffer defines a boolean indicating that data must be returned as an ArrayBuffer
         * @param onError defines the callback to call when an error occurs
         * @returns a file request object
         */
        static ReadFile(file: File, onSuccess: (data: any) => void, onProgress?: (ev: ProgressEvent) => any, useArrayBuffer?: boolean, onError?: (error: ReadFileError) => void): IFileRequest;
        /**
         * Creates a data url from a given string content
         * @param content defines the content to convert
         * @returns the new data url link
         */
        static FileAsURL(content: string): string;
        /**
         * Format the given number to a specific decimal format
         * @param value defines the number to format
         * @param decimals defines the number of decimals to use
         * @returns the formatted string
         */
        static Format(value: number, decimals?: number): string;
        /**
         * Tries to copy an object by duplicating every property
         * @param source defines the source object
         * @param destination defines the target object
         * @param doNotCopyList defines a list of properties to avoid
         * @param mustCopyList defines a list of properties to copy (even if they start with _)
         */
        static DeepCopy(source: any, destination: any, doNotCopyList?: string[], mustCopyList?: string[]): void;
        /**
         * Gets a boolean indicating if the given object has no own property
         * @param obj defines the object to test
         * @returns true if object has no own property
         */
        static IsEmpty(obj: any): boolean;
        /**
         * Function used to register events at window level
         * @param windowElement defines the Window object to use
         * @param events defines the events to register
         */
        static RegisterTopRootEvents(windowElement: Window, events: {
            name: string;
            handler: Nullable<(e: FocusEvent) => any>;
        }[]): void;
        /**
         * Function used to unregister events from window level
         * @param windowElement defines the Window object to use
         * @param events defines the events to unregister
         */
        static UnregisterTopRootEvents(windowElement: Window, events: {
            name: string;
            handler: Nullable<(e: FocusEvent) => any>;
        }[]): void;
        /**
         * Dumps the current bound framebuffer
         * @param width defines the rendering width
         * @param height defines the rendering height
         * @param engine defines the hosting engine
         * @param successCallback defines the callback triggered once the data are available
         * @param mimeType defines the mime type of the result
         * @param fileName defines the filename to download. If present, the result will automatically be downloaded
         * @returns a void promise
         */
        static DumpFramebuffer(width: number, height: number, engine: Engine, successCallback?: (data: string) => void, mimeType?: string, fileName?: string): Promise<void>;
        /**
         * Dumps an array buffer
         * @param width defines the rendering width
         * @param height defines the rendering height
         * @param data the data array
         * @param successCallback defines the callback triggered once the data are available
         * @param mimeType defines the mime type of the result
         * @param fileName defines the filename to download. If present, the result will automatically be downloaded
         * @param invertY true to invert the picture in the Y dimension
         * @param toArrayBuffer true to convert the data to an ArrayBuffer (encoded as `mimeType`) instead of a base64 string
         * @param quality defines the quality of the result
         */
        static DumpData(width: number, height: number, data: ArrayBufferView, successCallback?: (data: string | ArrayBuffer) => void, mimeType?: string, fileName?: string, invertY?: boolean, toArrayBuffer?: boolean, quality?: number): void;
        /**
         * Dumps an array buffer
         * @param width defines the rendering width
         * @param height defines the rendering height
         * @param data the data array
         * @param mimeType defines the mime type of the result
         * @param fileName defines the filename to download. If present, the result will automatically be downloaded
         * @param invertY true to invert the picture in the Y dimension
         * @param toArrayBuffer true to convert the data to an ArrayBuffer (encoded as `mimeType`) instead of a base64 string
         * @param quality defines the quality of the result
         * @returns a promise that resolve to the final data
         */
        static DumpDataAsync(width: number, height: number, data: ArrayBufferView, mimeType?: string, fileName?: string, invertY?: boolean, toArrayBuffer?: boolean, quality?: number): Promise<string | ArrayBuffer>;
        /**
         * Converts the canvas data to blob.
         * This acts as a polyfill for browsers not supporting the to blob function.
         * @param canvas Defines the canvas to extract the data from
         * @param successCallback Defines the callback triggered once the data are available
         * @param mimeType Defines the mime type of the result
         * @param quality defines the quality of the result
         */
        static ToBlob(canvas: HTMLCanvasElement, successCallback: (blob: Nullable<Blob>) => void, mimeType?: string, quality?: number): void;
        /**
         * Download a Blob object
         * @param blob the Blob object
         * @param fileName the file name to download
         * @returns
         */
        static DownloadBlob(blob: Blob, fileName?: string): void;
        /**
         * Encodes the canvas data to base 64 or automatically download the result if filename is defined
         * @param canvas canvas to get the data from.
         * @param successCallback defines the callback triggered once the data are available
         * @param mimeType defines the mime type of the result
         * @param fileName defines he filename to download. If present, the result will automatically be downloaded
         * @param quality defines the quality of the result
         */
        static EncodeScreenshotCanvasData(canvas: HTMLCanvasElement, successCallback?: (data: string) => void, mimeType?: string, fileName?: string, quality?: number): void;
        /**
         * Downloads a blob in the browser
         * @param blob defines the blob to download
         * @param fileName defines the name of the downloaded file
         */
        static Download(blob: Blob, fileName: string): void;
        /**
         * Will return the right value of the noPreventDefault variable
         * Needed to keep backwards compatibility to the old API.
         *
         * @param args arguments passed to the attachControl function
         * @returns the correct value for noPreventDefault
         */
        static BackCompatCameraNoPreventDefault(args: IArguments): boolean;
        /**
         * Captures a screenshot of the current rendering
         * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/renderToPNG
         * @param engine defines the rendering engine
         * @param camera defines the source camera
         * @param size This parameter can be set to a single number or to an object with the
         * following (optional) properties: precision, width, height. If a single number is passed,
         * it will be used for both width and height. If an object is passed, the screenshot size
         * will be derived from the parameters. The precision property is a multiplier allowing
         * rendering at a higher or lower resolution
         * @param successCallback defines the callback receives a single parameter which contains the
         * screenshot as a string of base64-encoded characters. This string can be assigned to the
         * src parameter of an <img> to display it
         * @param mimeType defines the MIME type of the screenshot image (default: image/png).
         * Check your browser for supported MIME types
         */
        static CreateScreenshot(engine: Engine, camera: Camera, size: IScreenshotSize | number, successCallback?: (data: string) => void, mimeType?: string): void;
        /**
         * Captures a screenshot of the current rendering
         * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/renderToPNG
         * @param engine defines the rendering engine
         * @param camera defines the source camera
         * @param size This parameter can be set to a single number or to an object with the
         * following (optional) properties: precision, width, height. If a single number is passed,
         * it will be used for both width and height. If an object is passed, the screenshot size
         * will be derived from the parameters. The precision property is a multiplier allowing
         * rendering at a higher or lower resolution
         * @param mimeType defines the MIME type of the screenshot image (default: image/png).
         * Check your browser for supported MIME types
         * @returns screenshot as a string of base64-encoded characters. This string can be assigned
         * to the src parameter of an <img> to display it
         */
        static CreateScreenshotAsync(engine: Engine, camera: Camera, size: IScreenshotSize | number, mimeType?: string): Promise<string>;
        /**
         * Generates an image screenshot from the specified camera.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/renderToPNG
         * @param engine The engine to use for rendering
         * @param camera The camera to use for rendering
         * @param size This parameter can be set to a single number or to an object with the
         * following (optional) properties: precision, width, height. If a single number is passed,
         * it will be used for both width and height. If an object is passed, the screenshot size
         * will be derived from the parameters. The precision property is a multiplier allowing
         * rendering at a higher or lower resolution
         * @param successCallback The callback receives a single parameter which contains the
         * screenshot as a string of base64-encoded characters. This string can be assigned to the
         * src parameter of an <img> to display it
         * @param mimeType The MIME type of the screenshot image (default: image/png).
         * Check your browser for supported MIME types
         * @param samples Texture samples (default: 1)
         * @param antialiasing Whether antialiasing should be turned on or not (default: false)
         * @param fileName A name for for the downloaded file.
         */
        static CreateScreenshotUsingRenderTarget(engine: Engine, camera: Camera, size: IScreenshotSize | number, successCallback?: (data: string) => void, mimeType?: string, samples?: number, antialiasing?: boolean, fileName?: string): void;
        /**
         * Generates an image screenshot from the specified camera.
         * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/renderToPNG
         * @param engine The engine to use for rendering
         * @param camera The camera to use for rendering
         * @param size This parameter can be set to a single number or to an object with the
         * following (optional) properties: precision, width, height. If a single number is passed,
         * it will be used for both width and height. If an object is passed, the screenshot size
         * will be derived from the parameters. The precision property is a multiplier allowing
         * rendering at a higher or lower resolution
         * @param mimeType The MIME type of the screenshot image (default: image/png).
         * Check your browser for supported MIME types
         * @param samples Texture samples (default: 1)
         * @param antialiasing Whether antialiasing should be turned on or not (default: false)
         * @param fileName A name for for the downloaded file.
         * @returns screenshot as a string of base64-encoded characters. This string can be assigned
         * to the src parameter of an <img> to display it
         */
        static CreateScreenshotUsingRenderTargetAsync(engine: Engine, camera: Camera, size: IScreenshotSize | number, mimeType?: string, samples?: number, antialiasing?: boolean, fileName?: string): Promise<string>;
        /**
         * Implementation from http://stackoverflow.com/questions/105034/how-to-create-a-guid-uuid-in-javascript/2117523#answer-2117523
         * Be aware Math.random() could cause collisions, but:
         * "All but 6 of the 128 bits of the ID are randomly generated, which means that for any two ids, there's a 1 in 2^^122 (or 5.3x10^^36) chance they'll collide"
         * @returns a pseudo random id
         */
        static RandomId(): string;
        /**
         * Test if the given uri is a base64 string
         * @deprecated Please use FileTools.IsBase64DataUrl instead.
         * @param uri The uri to test
         * @returns True if the uri is a base64 string or false otherwise
         */
        static IsBase64(uri: string): boolean;
        /**
         * Decode the given base64 uri.
         * @deprecated Please use FileTools.DecodeBase64UrlToBinary instead.
         * @param uri The uri to decode
         * @returns The decoded base64 data.
         */
        static DecodeBase64(uri: string): ArrayBuffer;
        static GetAbsoluteUrl: (url: string) => string;
        /**
         * No log
         */
        static readonly NoneLogLevel = 0;
        /**
         * Only message logs
         */
        static readonly MessageLogLevel = 1;
        /**
         * Only warning logs
         */
        static readonly WarningLogLevel = 2;
        /**
         * Only error logs
         */
        static readonly ErrorLogLevel = 4;
        /**
         * All logs
         */
        static readonly AllLogLevel = 7;
        /**
         * Gets a value indicating the number of loading errors
         * @ignorenaming
         */
        static get errorsCount(): number;
        /**
         * Callback called when a new log is added
         */
        static OnNewCacheEntry: (entry: string) => void;
        /**
         * Log a message to the console
         * @param message defines the message to log
         */
        static Log(message: string): void;
        /**
         * Write a warning message to the console
         * @param message defines the message to log
         */
        static Warn(message: string): void;
        /**
         * Write an error message to the console
         * @param message defines the message to log
         */
        static Error(message: string): void;
        /**
         * Gets current log cache (list of logs)
         */
        static get LogCache(): string;
        /**
         * Clears the log cache
         */
        static ClearLogCache(): void;
        /**
         * Sets the current log level (MessageLogLevel / WarningLogLevel / ErrorLogLevel)
         */
        static set LogLevels(level: number);
        /**
         * Checks if the window object exists
         * Back Compat only, please use IsWindowObjectExist instead.
         */
        static IsWindowObjectExist: typeof IsWindowObjectExist;
        /**
         * No performance log
         */
        static readonly PerformanceNoneLogLevel = 0;
        /**
         * Use user marks to log performance
         */
        static readonly PerformanceUserMarkLogLevel = 1;
        /**
         * Log performance to the console
         */
        static readonly PerformanceConsoleLogLevel = 2;
        private static _Performance;
        /**
         * Sets the current performance log level
         */
        static set PerformanceLogLevel(level: number);
        private static _StartPerformanceCounterDisabled;
        private static _EndPerformanceCounterDisabled;
        private static _StartUserMark;
        private static _EndUserMark;
        private static _StartPerformanceConsole;
        private static _EndPerformanceConsole;
        /**
         * Starts a performance counter
         */
        static StartPerformanceCounter: (counterName: string, condition?: boolean) => void;
        /**
         * Ends a specific performance counter
         */
        static EndPerformanceCounter: (counterName: string, condition?: boolean) => void;
        /**
         * Gets either window.performance.now() if supported or Date.now() else
         */
        static get Now(): number;
        /**
         * This method will return the name of the class used to create the instance of the given object.
         * It will works only on Javascript basic data types (number, string, ...) and instance of class declared with the @className decorator.
         * @param object the object to get the class name from
         * @param isType defines if the object is actually a type
         * @returns the name of the class, will be "object" for a custom data type not using the @className decorator
         */
        static GetClassName(object: any, isType?: boolean): string;
        /**
         * Gets the first element of an array satisfying a given predicate
         * @param array defines the array to browse
         * @param predicate defines the predicate to use
         * @returns null if not found or the element
         */
        static First<T>(array: Array<T>, predicate: (item: T) => boolean): Nullable<T>;
        /**
         * This method will return the name of the full name of the class, including its owning module (if any).
         * It will works only on Javascript basic data types (number, string, ...) and instance of class declared with the @className decorator or implementing a method getClassName():string (in which case the module won't be specified).
         * @param object the object to get the class name from
         * @param isType defines if the object is actually a type
         * @returns a string that can have two forms: "moduleName.className" if module was specified when the class' Name was registered or "className" if there was not module specified.
         * @ignorenaming
         */
        static getFullClassName(object: any, isType?: boolean): Nullable<string>;
        /**
         * Returns a promise that resolves after the given amount of time.
         * @param delay Number of milliseconds to delay
         * @returns Promise that resolves after the given amount of time
         */
        static DelayAsync(delay: number): Promise<void>;
        /**
         * Utility function to detect if the current user agent is Safari
         * @returns whether or not the current user agent is safari
         */
        static IsSafari(): boolean;
    }
    /**
     * Use this className as a decorator on a given class definition to add it a name and optionally its module.
     * You can then use the Tools.getClassName(obj) on an instance to retrieve its class name.
     * This method is the only way to get it done in all cases, even if the .js file declaring the class is minified
     * @param name The name of the class, case should be preserved
     * @param module The name of the Module hosting the class, optional, but strongly recommended to specify if possible. Case should be preserved.
     */
    export function className(name: string, module?: string): (target: Object) => void;
    /**
     * An implementation of a loop for asynchronous functions.
     */
    export class AsyncLoop {
        /**
         * Defines the number of iterations for the loop
         */
        iterations: number;
        /**
         * Defines the current index of the loop.
         */
        index: number;
        private _done;
        private _fn;
        private _successCallback;
        /**
         * Constructor.
         * @param iterations the number of iterations.
         * @param func the function to run each iteration
         * @param successCallback the callback that will be called upon successful execution
         * @param offset starting offset.
         */
        constructor(
        /**
         * Defines the number of iterations for the loop
         */
        iterations: number, func: (asyncLoop: AsyncLoop) => void, successCallback: () => void, offset?: number);
        /**
         * Execute the next iteration. Must be called after the last iteration was finished.
         */
        executeNext(): void;
        /**
         * Break the loop and run the success callback.
         */
        breakLoop(): void;
        /**
         * Create and run an async loop.
         * @param iterations the number of iterations.
         * @param fn the function to run each iteration
         * @param successCallback the callback that will be called upon successful execution
         * @param offset starting offset.
         * @returns the created async loop object
         */
        static Run(iterations: number, fn: (asyncLoop: AsyncLoop) => void, successCallback: () => void, offset?: number): AsyncLoop;
        /**
         * A for-loop that will run a given number of iterations synchronous and the rest async.
         * @param iterations total number of iterations
         * @param syncedIterations number of synchronous iterations in each async iteration.
         * @param fn the function to call each iteration.
         * @param callback a success call back that will be called when iterating stops.
         * @param breakFunction a break condition (optional)
         * @param timeout timeout settings for the setTimeout function. default - 0.
         * @returns the created async loop object
         */
        static SyncAsyncForLoop(iterations: number, syncedIterations: number, fn: (iteration: number) => void, callback: () => void, breakFunction?: () => boolean, timeout?: number): AsyncLoop;
    }


    /**
     * A 3D trajectory consisting of an order list of vectors describing a
     * path of motion through 3D space.
     */
    export class Trajectory {
        private _points;
        private readonly _segmentLength;
        /**
         * Serialize to JSON.
         * @returns serialized JSON string
         */
        serialize(): string;
        /**
         * Deserialize from JSON.
         * @param json serialized JSON string
         * @returns deserialized Trajectory
         */
        static Deserialize(json: string): Trajectory;
        /**
         * Create a new empty Trajectory.
         * @param segmentLength radius of discretization for Trajectory points
         */
        constructor(segmentLength?: number);
        /**
         * Get the length of the Trajectory.
         * @returns length of the Trajectory
         */
        getLength(): number;
        /**
         * Append a new point to the Trajectory.
         * NOTE: This implementation has many allocations.
         * @param point point to append to the Trajectory
         */
        add(point: DeepImmutable<Vector3>): void;
        /**
         * Create a new Trajectory with a segment length chosen to make it
         * probable that the new Trajectory will have a specified number of
         * segments. This operation is imprecise.
         * @param targetResolution number of segments desired
         * @returns new Trajectory with approximately the requested number of segments
         */
        resampleAtTargetResolution(targetResolution: number): Trajectory;
        /**
         * Convert Trajectory segments into tokenized representation. This
         * representation is an array of numbers where each nth number is the
         * index of the token which is most similar to the nth segment of the
         * Trajectory.
         * @param tokens list of vectors which serve as discrete tokens
         * @returns list of indices of most similar token per segment
         */
        tokenize(tokens: DeepImmutable<Vector3[]>): number[];
        private static _ForwardDir;
        private static _InverseFromVec;
        private static _UpDir;
        private static _FromToVec;
        private static _LookMatrix;
        /**
         * Transform the rotation (i.e., direction) of a segment to isolate
         * the relative transformation represented by the segment. This operation
         * may or may not succeed due to singularities in the equations that define
         * motion relativity in this context.
         * @param priorVec the origin of the prior segment
         * @param fromVec the origin of the current segment
         * @param toVec the destination of the current segment
         * @param result reference to output variable
         * @returns whether or not transformation was successful
         */
        private static _TransformSegmentDirToRef;
        private static _BestMatch;
        private static _Score;
        private static _BestScore;
        /**
         * Determine which token vector is most similar to the
         * segment vector.
         * @param segment segment vector
         * @param tokens token vector list
         * @returns index of the most similar token to the segment
         */
        private static _TokenizeSegment;
    }
    /**
     * Class representing a set of known, named trajectories to which Trajectories can be
     * added and using which Trajectories can be recognized.
     */
    export class TrajectoryClassifier {
        private _maximumAllowableMatchCost;
        private _vector3Alphabet;
        private _levenshteinAlphabet;
        private _nameToDescribedTrajectory;
        /**
         * Serialize to JSON.
         * @returns JSON serialization
         */
        serialize(): string;
        /**
         * Deserialize from JSON.
         * @param json JSON serialization
         * @returns deserialized TrajectorySet
         */
        static Deserialize(json: string): TrajectoryClassifier;
        /**
         * Initialize a new empty TrajectorySet with auto-generated Alphabets.
         * VERY naive, need to be generating these things from known
         * sets. Better version later, probably eliminating this one.
         * @returns auto-generated TrajectorySet
         */
        static Generate(): TrajectoryClassifier;
        private constructor();
        /**
         * Add a new Trajectory to the set with a given name.
         * @param trajectory new Trajectory to be added
         * @param classification name to which to add the Trajectory
         */
        addTrajectoryToClassification(trajectory: Trajectory, classification: string): void;
        /**
         * Remove a known named trajectory and all Trajectories associated with it.
         * @param classification name to remove
         * @returns whether anything was removed
         */
        deleteClassification(classification: string): boolean;
        /**
         * Attempt to recognize a Trajectory from among all the classifications
         * already known to the classifier.
         * @param trajectory Trajectory to be recognized
         * @returns classification of Trajectory if recognized, null otherwise
         */
        classifyTrajectory(trajectory: Trajectory): Nullable<string>;
    }


    /**
     * @internal
     */
    export function RegisterClass(className: string, type: Object): void;
    /**
     * @internal
     */
    export function GetClass(fqdn: string): any;


    /**
     * Helper class used to generate session unique ID
     */
    export class UniqueIdGenerator {
        private static _UniqueIdCounter;
        /**
         * Gets an unique (relatively to the current scene) Id
         */
        static get UniqueId(): number;
    }


    /**
     * This represents the different options available for the video capture.
     */
    export interface VideoRecorderOptions {
        /** Defines the mime type of the video. */
        mimeType: string;
        /** Defines the FPS the video should be recorded at. */
        fps: number;
        /** Defines the chunk size for the recording data. */
        recordChunckSize: number;
        /** The audio tracks to attach to the recording. */
        audioTracks?: MediaStreamTrack[];
    }
    /**
     * This can help with recording videos from BabylonJS.
     * This is based on the available WebRTC functionalities of the browser.
     *
     * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/renderToVideo
     */
    export class VideoRecorder {
        private static readonly _DefaultOptions;
        /**
         * Returns whether or not the VideoRecorder is available in your browser.
         * @param engine Defines the Babylon Engine.
         * @returns true if supported otherwise false.
         */
        static IsSupported(engine: Engine): boolean;
        private readonly _options;
        private _canvas;
        private _mediaRecorder;
        private _recordedChunks;
        private _fileName;
        private _resolve;
        private _reject;
        /**
         * True when a recording is already in progress.
         */
        get isRecording(): boolean;
        /**
         * Create a new VideoCapture object which can help converting what you see in Babylon to a video file.
         * @param engine Defines the BabylonJS Engine you wish to record.
         * @param options Defines options that can be used to customize the capture.
         */
        constructor(engine: Engine, options?: Partial<VideoRecorderOptions>);
        /**
         * Stops the current recording before the default capture timeout passed in the startRecording function.
         */
        stopRecording(): void;
        /**
         * Starts recording the canvas for a max duration specified in parameters.
         * @param fileName Defines the name of the file to be downloaded when the recording stop.
         * If null no automatic download will start and you can rely on the promise to get the data back.
         * @param maxDuration Defines the maximum recording time in seconds.
         * It defaults to 7 seconds. A value of zero will not stop automatically, you would need to call stopRecording manually.
         * @returns A promise callback at the end of the recording with the video data in Blob.
         */
        startRecording(fileName?: Nullable<string>, maxDuration?: number): Promise<Blob>;
        /**
         * Releases internal resources used during the recording.
         */
        dispose(): void;
        private _handleDataAvailable;
        private _handleError;
        private _handleStop;
    }


    /**
     * Defines the potential axis of a Joystick
     */
    export enum JoystickAxis {
        /** X axis */
        X = 0,
        /** Y axis */
        Y = 1,
        /** Z axis */
        Z = 2
    }
    /**
     * Represents the different customization options available
     * for VirtualJoystick
     */
    interface VirtualJoystickCustomizations {
        /**
         * Size of the joystick's puck
         */
        puckSize: number;
        /**
         * Size of the joystick's container
         */
        containerSize: number;
        /**
         * Color of the joystick && puck
         */
        color: string;
        /**
         * Image URL for the joystick's puck
         */
        puckImage?: string;
        /**
         * Image URL for the joystick's container
         */
        containerImage?: string;
        /**
         * Defines the unmoving position of the joystick container
         */
        position?: {
            x: number;
            y: number;
        };
        /**
         * Defines whether or not the joystick container is always visible
         */
        alwaysVisible: boolean;
        /**
         * Defines whether or not to limit the movement of the puck to the joystick's container
         */
        limitToContainer: boolean;
    }
    /**
     * Class used to define virtual joystick (used in touch mode)
     */
    export class VirtualJoystick {
        /**
         * Gets or sets a boolean indicating that left and right values must be inverted
         */
        reverseLeftRight: boolean;
        /**
         * Gets or sets a boolean indicating that up and down values must be inverted
         */
        reverseUpDown: boolean;
        /**
         * Gets the offset value for the position (ie. the change of the position value)
         */
        deltaPosition: Vector3;
        /**
         * Gets a boolean indicating if the virtual joystick was pressed
         */
        pressed: boolean;
        /**
         * Canvas the virtual joystick will render onto, default z-index of this is 5
         */
        static Canvas: Nullable<HTMLCanvasElement>;
        /**
         * boolean indicating whether or not the joystick's puck's movement should be limited to the joystick's container area
         */
        limitToContainer: boolean;
        private static _GlobalJoystickIndex;
        private static _AlwaysVisibleSticks;
        private static _VJCanvasContext;
        private static _VJCanvasWidth;
        private static _VJCanvasHeight;
        private static _HalfWidth;
        private static _GetDefaultOptions;
        private _action;
        private _axisTargetedByLeftAndRight;
        private _axisTargetedByUpAndDown;
        private _joystickSensibility;
        private _inversedSensibility;
        private _joystickPointerId;
        private _joystickColor;
        private _joystickPointerPos;
        private _joystickPreviousPointerPos;
        private _joystickPointerStartPos;
        private _deltaJoystickVector;
        private _leftJoystick;
        private _touches;
        private _joystickPosition;
        private _alwaysVisible;
        private _puckImage;
        private _containerImage;
        private _released;
        private _joystickPuckSize;
        private _joystickContainerSize;
        private _clearPuckSize;
        private _clearContainerSize;
        private _clearPuckSizeOffset;
        private _clearContainerSizeOffset;
        private _onPointerDownHandlerRef;
        private _onPointerMoveHandlerRef;
        private _onPointerUpHandlerRef;
        private _onResize;
        /**
         * Creates a new virtual joystick
         * @param leftJoystick defines that the joystick is for left hand (false by default)
         * @param customizations Defines the options we want to customize the VirtualJoystick
         */
        constructor(leftJoystick?: boolean, customizations?: Partial<VirtualJoystickCustomizations>);
        /**
         * Defines joystick sensibility (ie. the ratio between a physical move and virtual joystick position change)
         * @param newJoystickSensibility defines the new sensibility
         */
        setJoystickSensibility(newJoystickSensibility: number): void;
        private _onPointerDown;
        private _onPointerMove;
        private _onPointerUp;
        /**
         * Change the color of the virtual joystick
         * @param newColor a string that must be a CSS color value (like "red") or the hexa value (like "#FF0000")
         */
        setJoystickColor(newColor: string): void;
        /**
         * Size of the joystick's container
         */
        set containerSize(newSize: number);
        get containerSize(): number;
        /**
         * Size of the joystick's puck
         */
        set puckSize(newSize: number);
        get puckSize(): number;
        /**
         * Clears the set position of the joystick
         */
        clearPosition(): void;
        /**
         * Defines whether or not the joystick container is always visible
         */
        set alwaysVisible(value: boolean);
        get alwaysVisible(): boolean;
        /**
         * Sets the constant position of the Joystick container
         * @param x X axis coordinate
         * @param y Y axis coordinate
         */
        setPosition(x: number, y: number): void;
        /**
         * Defines a callback to call when the joystick is touched
         * @param action defines the callback
         */
        setActionOnTouch(action: () => any): void;
        /**
         * Defines which axis you'd like to control for left & right
         * @param axis defines the axis to use
         */
        setAxisForLeftRight(axis: JoystickAxis): void;
        /**
         * Defines which axis you'd like to control for up & down
         * @param axis defines the axis to use
         */
        setAxisForUpDown(axis: JoystickAxis): void;
        /**
         * Clears the canvas from the previous puck / container draw
         */
        private _clearPreviousDraw;
        /**
         * Loads `urlPath` to be used for the container's image
         * @param urlPath defines the urlPath of an image to use
         */
        setContainerImage(urlPath: string): void;
        /**
         * Loads `urlPath` to be used for the puck's image
         * @param urlPath defines the urlPath of an image to use
         */
        setPuckImage(urlPath: string): void;
        /**
         * Draws the Virtual Joystick's container
         */
        private _drawContainer;
        /**
         * Draws the Virtual Joystick's puck
         */
        private _drawPuck;
        private _drawVirtualJoystick;
        /**
         * Release internal HTML canvas
         */
        releaseCanvas(): void;
    }


    /**
     * Extended version of XMLHttpRequest with support for customizations (headers, ...)
     */
    export class WebRequest implements IWebRequest {
        private readonly _xhr;
        /**
         * Custom HTTP Request Headers to be sent with XMLHttpRequests
         * i.e. when loading files, where the server/service expects an Authorization header
         */
        static CustomRequestHeaders: {
            [key: string]: string;
        };
        /**
         * Add callback functions in this array to update all the requests before they get sent to the network
         */
        static CustomRequestModifiers: ((request: XMLHttpRequest, url: string) => void)[];
        static SkipRequestModificationForBabylonCDN: boolean;
        private _requestURL;
        private _injectCustomRequestHeaders;
        private _shouldSkipRequestModifications;
        /**
         * Gets or sets a function to be called when loading progress changes
         */
        get onprogress(): ((this: XMLHttpRequest, ev: ProgressEvent) => any) | null;
        set onprogress(value: ((this: XMLHttpRequest, ev: ProgressEvent) => any) | null);
        /**
         * Returns client's state
         */
        get readyState(): number;
        /**
         * Returns client's status
         */
        get status(): number;
        /**
         * Returns client's status as a text
         */
        get statusText(): string;
        /**
         * Returns client's response
         */
        get response(): any;
        /**
         * Returns client's response url
         */
        get responseURL(): string;
        /**
         * Returns client's response as text
         */
        get responseText(): string;
        /**
         * Gets or sets the expected response type
         */
        get responseType(): XMLHttpRequestResponseType;
        set responseType(value: XMLHttpRequestResponseType);
        /**
         * Gets or sets the timeout value in milliseconds
         */
        get timeout(): number;
        set timeout(value: number);
        /** @internal */
        addEventListener<K extends keyof XMLHttpRequestEventMap>(type: K, listener: (this: XMLHttpRequest, ev: XMLHttpRequestEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;
        /** @internal */
        removeEventListener<K extends keyof XMLHttpRequestEventMap>(type: K, listener: (this: XMLHttpRequest, ev: XMLHttpRequestEventMap[K]) => any, options?: boolean | EventListenerOptions): void;
        /**
         * Cancels any network activity
         */
        abort(): void;
        /**
         * Initiates the request. The optional argument provides the request body. The argument is ignored if request method is GET or HEAD
         * @param body defines an optional request body
         */
        send(body?: Document | XMLHttpRequestBodyInit | null): void;
        /**
         * Sets the request method, request URL
         * @param method defines the method to use (GET, POST, etc..)
         * @param url defines the url to connect with
         */
        open(method: string, url: string): void;
        /**
         * Sets the value of a request header.
         * @param name The name of the header whose value is to be set
         * @param value The value to set as the body of the header
         */
        setRequestHeader(name: string, value: string): void;
        /**
         * Get the string containing the text of a particular header's value.
         * @param name The name of the header
         * @returns The string containing the text of the given header name
         */
        getResponseHeader(name: string): Nullable<string>;
    }


    /** @ignore */
    interface WorkerInfo {
        workerPromise: Promise<Worker>;
        idle: boolean;
        timeoutId?: ReturnType<typeof setTimeout>;
    }
    /**
     * Helper class to push actions to a pool of workers.
     */
    export class WorkerPool implements IDisposable {
        protected _workerInfos: Array<WorkerInfo>;
        protected _pendingActions: ((worker: Worker, onComplete: () => void) => void)[];
        /**
         * Constructor
         * @param workers Array of workers to use for actions
         */
        constructor(workers: Array<Worker>);
        /**
         * Terminates all workers and clears any pending actions.
         */
        dispose(): void;
        /**
         * Pushes an action to the worker pool. If all the workers are active, the action will be
         * pended until a worker has completed its action.
         * @param action The action to perform. Call onComplete when the action is complete.
         */
        push(action: (worker: Worker, onComplete: () => void) => void): void;
        protected _executeOnIdleWorker(action: (worker: Worker, onComplete: () => void) => void): boolean;
        protected _execute(workerInfo: WorkerInfo, action: (worker: Worker, onComplete: () => void) => void): void;
    }
    /**
     * Options for AutoReleaseWorkerPool
     */
    export interface AutoReleaseWorkerPoolOptions {
        /**
         * Idle time elapsed before workers are terminated.
         */
        idleTimeElapsedBeforeRelease: number;
    }
    /**
     * Similar to the WorkerPool class except it creates and destroys workers automatically with a maximum of `maxWorkers` workers.
     * Workers are terminated when it is idle for at least `idleTimeElapsedBeforeRelease` milliseconds.
     */
    export class AutoReleaseWorkerPool extends WorkerPool {
        /**
         * Default options for the constructor.
         * Override to change the defaults.
         */
        static DefaultOptions: AutoReleaseWorkerPoolOptions;
        private readonly _maxWorkers;
        private readonly _createWorkerAsync;
        private readonly _options;
        constructor(maxWorkers: number, createWorkerAsync: () => Promise<Worker>, options?: AutoReleaseWorkerPoolOptions);
        push(action: (worker: Worker, onComplete: () => void) => void): void;
        protected _execute(workerInfo: WorkerInfo, action: (worker: Worker, onComplete: () => void) => void): void;
    }




    /**
     * Defines a target to use with MorphTargetManager
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/morphTargets
     */
    export class MorphTarget implements IAnimatable {
        /** defines the name of the target */
        name: string;
        /**
         * Gets or sets the list of animations
         */
        animations: Animation[];
        private _scene;
        private _positions;
        private _normals;
        private _tangents;
        private _uvs;
        private _influence;
        private _uniqueId;
        /**
         * Observable raised when the influence changes
         */
        onInfluenceChanged: Observable<boolean>;
        /** @internal */
        _onDataLayoutChanged: Observable<void>;
        /**
         * Gets or sets the influence of this target (ie. its weight in the overall morphing)
         */
        get influence(): number;
        set influence(influence: number);
        /**
         * Gets or sets the id of the morph Target
         */
        id: string;
        private _animationPropertiesOverride;
        /**
         * Gets or sets the animation properties override
         */
        get animationPropertiesOverride(): Nullable<AnimationPropertiesOverride>;
        set animationPropertiesOverride(value: Nullable<AnimationPropertiesOverride>);
        /**
         * Creates a new MorphTarget
         * @param name defines the name of the target
         * @param influence defines the influence to use
         * @param scene defines the scene the morphtarget belongs to
         */
        constructor(
        /** defines the name of the target */
        name: string, influence?: number, scene?: Nullable<Scene>);
        /**
         * Gets the unique ID of this manager
         */
        get uniqueId(): number;
        /**
         * Gets a boolean defining if the target contains position data
         */
        get hasPositions(): boolean;
        /**
         * Gets a boolean defining if the target contains normal data
         */
        get hasNormals(): boolean;
        /**
         * Gets a boolean defining if the target contains tangent data
         */
        get hasTangents(): boolean;
        /**
         * Gets a boolean defining if the target contains texture coordinates data
         */
        get hasUVs(): boolean;
        /**
         * Affects position data to this target
         * @param data defines the position data to use
         */
        setPositions(data: Nullable<FloatArray>): void;
        /**
         * Gets the position data stored in this target
         * @returns a FloatArray containing the position data (or null if not present)
         */
        getPositions(): Nullable<FloatArray>;
        /**
         * Affects normal data to this target
         * @param data defines the normal data to use
         */
        setNormals(data: Nullable<FloatArray>): void;
        /**
         * Gets the normal data stored in this target
         * @returns a FloatArray containing the normal data (or null if not present)
         */
        getNormals(): Nullable<FloatArray>;
        /**
         * Affects tangent data to this target
         * @param data defines the tangent data to use
         */
        setTangents(data: Nullable<FloatArray>): void;
        /**
         * Gets the tangent data stored in this target
         * @returns a FloatArray containing the tangent data (or null if not present)
         */
        getTangents(): Nullable<FloatArray>;
        /**
         * Affects texture coordinates data to this target
         * @param data defines the texture coordinates data to use
         */
        setUVs(data: Nullable<FloatArray>): void;
        /**
         * Gets the texture coordinates data stored in this target
         * @returns a FloatArray containing the texture coordinates data (or null if not present)
         */
        getUVs(): Nullable<FloatArray>;
        /**
         * Clone the current target
         * @returns a new MorphTarget
         */
        clone(): MorphTarget;
        /**
         * Serializes the current target into a Serialization object
         * @returns the serialized object
         */
        serialize(): any;
        /**
         * Returns the string "MorphTarget"
         * @returns "MorphTarget"
         */
        getClassName(): string;
        /**
         * Creates a new target from serialized data
         * @param serializationObject defines the serialized data to use
         * @param scene defines the hosting scene
         * @returns a new MorphTarget
         */
        static Parse(serializationObject: any, scene?: Scene): MorphTarget;
        /**
         * Creates a MorphTarget from mesh data
         * @param mesh defines the source mesh
         * @param name defines the name to use for the new target
         * @param influence defines the influence to attach to the target
         * @returns a new MorphTarget
         */
        static FromMesh(mesh: AbstractMesh, name?: string, influence?: number): MorphTarget;
    }


    /**
     * This class is used to deform meshes using morphing between different targets
     * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/morphTargets
     */
    export class MorphTargetManager implements IDisposable {
        /** Enable storing morph target data into textures when set to true (true by default) */
        static EnableTextureStorage: boolean;
        /** Maximum number of active morph targets supported in the "vertex attribute" mode (i.e., not the "texture" mode) */
        static MaxActiveMorphTargetsInVertexAttributeMode: number;
        private _targets;
        private _targetInfluenceChangedObservers;
        private _targetDataLayoutChangedObservers;
        private _activeTargets;
        private _scene;
        private _influences;
        private _morphTargetTextureIndices;
        private _supportsNormals;
        private _supportsTangents;
        private _supportsUVs;
        private _vertexCount;
        private _textureVertexStride;
        private _textureWidth;
        private _textureHeight;
        private _uniqueId;
        private _tempInfluences;
        private _canUseTextureForTargets;
        private _blockCounter;
        /** @internal */
        _parentContainer: Nullable<AbstractScene>;
        /** @internal */
        _targetStoreTexture: Nullable<RawTexture2DArray>;
        /**
         * Gets or sets a boolean indicating if influencers must be optimized (eg. recompiling the shader if less influencers are used)
         */
        optimizeInfluencers: boolean;
        /**
         * Gets or sets a boolean indicating if normals must be morphed
         */
        enableNormalMorphing: boolean;
        /**
         * Gets or sets a boolean indicating if tangents must be morphed
         */
        enableTangentMorphing: boolean;
        /**
         * Gets or sets a boolean indicating if UV must be morphed
         */
        enableUVMorphing: boolean;
        /**
         * Sets a boolean indicating that adding new target or updating an existing target will not update the underlying data buffers
         */
        set areUpdatesFrozen(block: boolean);
        get areUpdatesFrozen(): boolean;
        /**
         * Creates a new MorphTargetManager
         * @param scene defines the current scene
         */
        constructor(scene?: Nullable<Scene>);
        /**
         * Gets the unique ID of this manager
         */
        get uniqueId(): number;
        /**
         * Gets the number of vertices handled by this manager
         */
        get vertexCount(): number;
        /**
         * Gets a boolean indicating if this manager supports morphing of normals
         */
        get supportsNormals(): boolean;
        /**
         * Gets a boolean indicating if this manager supports morphing of tangents
         */
        get supportsTangents(): boolean;
        /**
         * Gets a boolean indicating if this manager supports morphing of texture coordinates
         */
        get supportsUVs(): boolean;
        /**
         * Gets the number of targets stored in this manager
         */
        get numTargets(): number;
        /**
         * Gets the number of influencers (ie. the number of targets with influences > 0)
         */
        get numInfluencers(): number;
        /**
         * Gets the list of influences (one per target)
         */
        get influences(): Float32Array;
        private _useTextureToStoreTargets;
        /**
         * Gets or sets a boolean indicating that targets should be stored as a texture instead of using vertex attributes (default is true).
         * Please note that this option is not available if the hardware does not support it
         */
        get useTextureToStoreTargets(): boolean;
        set useTextureToStoreTargets(value: boolean);
        /**
         * Gets a boolean indicating that the targets are stored into a texture (instead of as attributes)
         */
        get isUsingTextureForTargets(): boolean;
        /**
         * Gets the active target at specified index. An active target is a target with an influence > 0
         * @param index defines the index to check
         * @returns the requested target
         */
        getActiveTarget(index: number): MorphTarget;
        /**
         * Gets the target at specified index
         * @param index defines the index to check
         * @returns the requested target
         */
        getTarget(index: number): MorphTarget;
        /**
         * Add a new target to this manager
         * @param target defines the target to add
         */
        addTarget(target: MorphTarget): void;
        /**
         * Removes a target from the manager
         * @param target defines the target to remove
         */
        removeTarget(target: MorphTarget): void;
        /**
         * @internal
         */
        _bind(effect: Effect): void;
        /**
         * Clone the current manager
         * @returns a new MorphTargetManager
         */
        clone(): MorphTargetManager;
        /**
         * Serializes the current manager into a Serialization object
         * @returns the serialized object
         */
        serialize(): any;
        private _syncActiveTargets;
        /**
         * Synchronize the targets with all the meshes using this morph target manager
         */
        synchronize(): void;
        /**
         * Release all resources
         */
        dispose(): void;
        /**
         * Creates a new MorphTargetManager from serialized data
         * @param serializationObject defines the serialized data
         * @param scene defines the hosting scene
         * @returns the new MorphTargetManager
         */
        static Parse(serializationObject: any, scene: Scene): MorphTargetManager;
    }


    /**
     * Navigation plugin interface to add navigation constrained by a navigation mesh
     */
    export interface INavigationEnginePlugin {
        /**
         * plugin name
         */
        name: string;
        /**
         * Creates a navigation mesh
         * @param meshes array of all the geometry used to compute the navigation mesh
         * @param parameters bunch of parameters used to filter geometry
         */
        createNavMesh(meshes: Array<Mesh>, parameters: INavMeshParameters): void;
        /**
         * Create a navigation mesh debug mesh
         * @param scene is where the mesh will be added
         * @returns debug display mesh
         */
        createDebugNavMesh(scene: Scene): Mesh;
        /**
         * Get a navigation mesh constrained position, closest to the parameter position
         * @param position world position
         * @returns the closest point to position constrained by the navigation mesh
         */
        getClosestPoint(position: Vector3): Vector3;
        /**
         * Get a navigation mesh constrained position, closest to the parameter position
         * @param position world position
         * @param result output the closest point to position constrained by the navigation mesh
         */
        getClosestPointToRef(position: Vector3, result: Vector3): void;
        /**
         * Get a navigation mesh constrained position, within a particular radius
         * @param position world position
         * @param maxRadius the maximum distance to the constrained world position
         * @returns the closest point to position constrained by the navigation mesh
         */
        getRandomPointAround(position: Vector3, maxRadius: number): Vector3;
        /**
         * Get a navigation mesh constrained position, within a particular radius
         * @param position world position
         * @param maxRadius the maximum distance to the constrained world position
         * @param result output the closest point to position constrained by the navigation mesh
         */
        getRandomPointAroundToRef(position: Vector3, maxRadius: number, result: Vector3): void;
        /**
         * Compute the final position from a segment made of destination-position
         * @param position world position
         * @param destination world position
         * @returns the resulting point along the navmesh
         */
        moveAlong(position: Vector3, destination: Vector3): Vector3;
        /**
         * Compute the final position from a segment made of destination-position
         * @param position world position
         * @param destination world position
         * @param result output the resulting point along the navmesh
         */
        moveAlongToRef(position: Vector3, destination: Vector3, result: Vector3): void;
        /**
         * Compute a navigation path from start to end. Returns an empty array if no path can be computed
         * @param start world position
         * @param end world position
         * @returns array containing world position composing the path
         */
        computePath(start: Vector3, end: Vector3): Vector3[];
        /**
         * If this plugin is supported
         * @returns true if plugin is supported
         */
        isSupported(): boolean;
        /**
         * Create a new Crowd so you can add agents
         * @param maxAgents the maximum agent count in the crowd
         * @param maxAgentRadius the maximum radius an agent can have
         * @param scene to attach the crowd to
         * @returns the crowd you can add agents to
         */
        createCrowd(maxAgents: number, maxAgentRadius: number, scene: Scene): ICrowd;
        /**
         * Set the Bounding box extent for doing spatial queries (getClosestPoint, getRandomPointAround, ...)
         * The queries will try to find a solution within those bounds
         * default is (1,1,1)
         * @param extent x,y,z value that define the extent around the queries point of reference
         */
        setDefaultQueryExtent(extent: Vector3): void;
        /**
         * Get the Bounding box extent specified by setDefaultQueryExtent
         * @returns the box extent values
         */
        getDefaultQueryExtent(): Vector3;
        /**
         * build the navmesh from a previously saved state using getNavmeshData
         * @param data the Uint8Array returned by getNavmeshData
         */
        buildFromNavmeshData(data: Uint8Array): void;
        /**
         * returns the navmesh data that can be used later. The navmesh must be built before retrieving the data
         * @returns data the Uint8Array that can be saved and reused
         */
        getNavmeshData(): Uint8Array;
        /**
         * Get the Bounding box extent result specified by setDefaultQueryExtent
         * @param result output the box extent values
         */
        getDefaultQueryExtentToRef(result: Vector3): void;
        /**
         * Set the time step of the navigation tick update.
         * Default is 1/60.
         * A value of 0 will disable fixed time update
         * @param newTimeStep the new timestep to apply to this world.
         */
        setTimeStep(newTimeStep: number): void;
        /**
         * Get the time step of the navigation tick update.
         * @returns the current time step
         */
        getTimeStep(): number;
        /**
         * If delta time in navigation tick update is greater than the time step
         * a number of sub iterations are done. If more iterations are need to reach deltatime
         * they will be discarded.
         * A value of 0 will set to no maximum and update will use as many substeps as needed
         * @param newStepCount the maximum number of iterations
         */
        setMaximumSubStepCount(newStepCount: number): void;
        /**
         * Get the maximum number of iterations per navigation tick update
         * @returns the maximum number of iterations
         */
        getMaximumSubStepCount(): number;
        /**
         * Creates a cylinder obstacle and add it to the navigation
         * @param position world position
         * @param radius cylinder radius
         * @param height cylinder height
         * @returns the obstacle freshly created
         */
        addCylinderObstacle(position: Vector3, radius: number, height: number): IObstacle;
        /**
         * Creates an oriented box obstacle and add it to the navigation
         * @param position world position
         * @param extent box size
         * @param angle angle in radians of the box orientation on Y axis
         * @returns the obstacle freshly created
         */
        addBoxObstacle(position: Vector3, extent: Vector3, angle: number): IObstacle;
        /**
         * Removes an obstacle created by addCylinderObstacle or addBoxObstacle
         * @param obstacle obstacle to remove from the navigation
         */
        removeObstacle(obstacle: IObstacle): void;
        /**
         * Release all resources
         */
        dispose(): void;
    }
    /**
     * Obstacle interface
     */
    export interface IObstacle {
    }
    /**
     * Crowd Interface. A Crowd is a collection of moving agents constrained by a navigation mesh
     */
    export interface ICrowd {
        /**
         * Add a new agent to the crowd with the specified parameter a corresponding transformNode.
         * You can attach anything to that node. The node position is updated in the scene update tick.
         * @param pos world position that will be constrained by the navigation mesh
         * @param parameters agent parameters
         * @param transform hooked to the agent that will be update by the scene
         * @returns agent index
         */
        addAgent(pos: Vector3, parameters: IAgentParameters, transform: TransformNode): number;
        /**
         * Returns the agent position in world space
         * @param index agent index returned by addAgent
         * @returns world space position
         */
        getAgentPosition(index: number): Vector3;
        /**
         * Gets the agent position result in world space
         * @param index agent index returned by addAgent
         * @param result output world space position
         */
        getAgentPositionToRef(index: number, result: Vector3): void;
        /**
         * Gets the agent velocity in world space
         * @param index agent index returned by addAgent
         * @returns world space velocity
         */
        getAgentVelocity(index: number): Vector3;
        /**
         * Gets the agent velocity result in world space
         * @param index agent index returned by addAgent
         * @param result output world space velocity
         */
        getAgentVelocityToRef(index: number, result: Vector3): void;
        /**
         * Gets the agent next target point on the path
         * @param index agent index returned by addAgent
         * @returns world space position
         */
        getAgentNextTargetPath(index: number): Vector3;
        /**
         * Gets the agent state
         * @param index agent index returned by addAgent
         * @returns agent state
         */
        getAgentState(index: number): number;
        /**
         * returns true if the agent in over an off mesh link connection
         * @param index agent index returned by addAgent
         * @returns true if over an off mesh link connection
         */
        overOffmeshConnection(index: number): boolean;
        /**
         * Gets the agent next target point on the path
         * @param index agent index returned by addAgent
         * @param result output world space position
         */
        getAgentNextTargetPathToRef(index: number, result: Vector3): void;
        /**
         * remove a particular agent previously created
         * @param index agent index returned by addAgent
         */
        removeAgent(index: number): void;
        /**
         * get the list of all agents attached to this crowd
         * @returns list of agent indices
         */
        getAgents(): number[];
        /**
         * Tick update done by the Scene. Agent position/velocity/acceleration is updated by this function
         * @param deltaTime in seconds
         */
        update(deltaTime: number): void;
        /**
         * Asks a particular agent to go to a destination. That destination is constrained by the navigation mesh
         * @param index agent index returned by addAgent
         * @param destination targeted world position
         */
        agentGoto(index: number, destination: Vector3): void;
        /**
         * Teleport the agent to a new position
         * @param index agent index returned by addAgent
         * @param destination targeted world position
         */
        agentTeleport(index: number, destination: Vector3): void;
        /**
         * Update agent parameters
         * @param index agent index returned by addAgent
         * @param parameters agent parameters
         */
        updateAgentParameters(index: number, parameters: IAgentParameters): void;
        /**
         * Set the Bounding box extent for doing spatial queries (getClosestPoint, getRandomPointAround, ...)
         * The queries will try to find a solution within those bounds
         * default is (1,1,1)
         * @param extent x,y,z value that define the extent around the queries point of reference
         */
        setDefaultQueryExtent(extent: Vector3): void;
        /**
         * Get the Bounding box extent specified by setDefaultQueryExtent
         * @returns the box extent values
         */
        getDefaultQueryExtent(): Vector3;
        /**
         * Get the Bounding box extent result specified by setDefaultQueryExtent
         * @param result output the box extent values
         */
        getDefaultQueryExtentToRef(result: Vector3): void;
        /**
         * Get the next corner points composing the path (max 4 points)
         * @param index agent index returned by addAgent
         * @returns array containing world position composing the path
         */
        getCorners(index: number): Vector3[];
        /**
         * Release all resources
         */
        dispose(): void;
    }
    /**
     * Configures an agent
     */
    export interface IAgentParameters {
        /**
         *  Agent radius. [Limit: >= 0]
         */
        radius: number;
        /**
         * Agent height. [Limit: > 0]
         */
        height: number;
        /**
         *  Maximum allowed acceleration. [Limit: >= 0]
         */
        maxAcceleration: number;
        /**
         * Maximum allowed speed. [Limit: >= 0]
         */
        maxSpeed: number;
        /**
         * Defines how close a collision element must be before it is considered for steering behaviors. [Limits: > 0]
         */
        collisionQueryRange: number;
        /**
         * The path visibility optimization range. [Limit: > 0]
         */
        pathOptimizationRange: number;
        /**
         * How aggressive the agent manager should be at avoiding collisions with this agent. [Limit: >= 0]
         */
        separationWeight: number;
        /**
         * Observers will be notified when agent gets inside the virtual circle with this Radius around destination point.
         * Default is agent radius
         */
        reachRadius?: number;
    }
    /**
     * Configures the navigation mesh creation
     */
    export interface INavMeshParameters {
        /**
         * The xz-plane cell size to use for fields. [Limit: > 0] [Units: wu]
         */
        cs: number;
        /**
         * The y-axis cell size to use for fields. [Limit: > 0] [Units: wu]
         */
        ch: number;
        /**
         * The maximum slope that is considered walkable. [Limits: 0 <= value < 90] [Units: Degrees]
         */
        walkableSlopeAngle: number;
        /**
         * Minimum floor to 'ceiling' height that will still allow the floor area to
         * be considered walkable. [Limit: >= 3] [Units: vx]
         */
        walkableHeight: number;
        /**
         * Maximum ledge height that is considered to still be traversable. [Limit: >=0] [Units: vx]
         */
        walkableClimb: number;
        /**
         * The distance to erode/shrink the walkable area of the heightfield away from
         * obstructions.  [Limit: >=0] [Units: vx]
         */
        walkableRadius: number;
        /**
         * The maximum allowed length for contour edges along the border of the mesh. [Limit: >=0] [Units: vx]
         */
        maxEdgeLen: number;
        /**
         * The maximum distance a simplified contour's border edges should deviate
         * the original raw contour. [Limit: >=0] [Units: vx]
         */
        maxSimplificationError: number;
        /**
         * The minimum number of cells allowed to form isolated island areas. [Limit: >=0] [Units: vx]
         */
        minRegionArea: number;
        /**
         * Any regions with a span count smaller than this value will, if possible,
         * be merged with larger regions. [Limit: >=0] [Units: vx]
         */
        mergeRegionArea: number;
        /**
         * The maximum number of vertices allowed for polygons generated during the
         * contour to polygon conversion process. [Limit: >= 3]
         */
        maxVertsPerPoly: number;
        /**
         * Sets the sampling distance to use when generating the detail mesh.
         * (For height detail only.) [Limits: 0 or >= 0.9] [Units: wu]
         */
        detailSampleDist: number;
        /**
         * The maximum distance the detail mesh surface should deviate from heightfield
         * data. (For height detail only.) [Limit: >=0] [Units: wu]
         */
        detailSampleMaxError: number;
        /**
         * If using obstacles, the navmesh must be subdivided internaly by tiles.
         * This member defines the tile cube side length in world units.
         * If no obstacles are needed, leave it undefined or 0.
         */
        tileSize?: number;
        /**
         * The size of the non-navigable border around the heightfield.
         */
        borderSize?: number;
    }






    /**
     * RecastJS navigation plugin
     */
    export class RecastJSPlugin implements INavigationEnginePlugin {
        /**
         * Reference to the Recast library
         */
        bjsRECAST: any;
        /**
         * plugin name
         */
        name: string;
        /**
         * the first navmesh created. We might extend this to support multiple navmeshes
         */
        navMesh: any;
        private _maximumSubStepCount;
        private _timeStep;
        private _timeFactor;
        private _tempVec1;
        private _tempVec2;
        private _worker;
        /**
         * Initializes the recastJS plugin
         * @param recastInjection can be used to inject your own recast reference
         */
        constructor(recastInjection?: any);
        /**
         * Set worker URL to be used when generating a new navmesh
         * @param workerURL url string
         * @returns boolean indicating if worker is created
         */
        setWorkerURL(workerURL: string): boolean;
        /**
         * Set the time step of the navigation tick update.
         * Default is 1/60.
         * A value of 0 will disable fixed time update
         * @param newTimeStep the new timestep to apply to this world.
         */
        setTimeStep(newTimeStep?: number): void;
        /**
         * Get the time step of the navigation tick update.
         * @returns the current time step
         */
        getTimeStep(): number;
        /**
         * If delta time in navigation tick update is greater than the time step
         * a number of sub iterations are done. If more iterations are need to reach deltatime
         * they will be discarded.
         * A value of 0 will set to no maximum and update will use as many substeps as needed
         * @param newStepCount the maximum number of iterations
         */
        setMaximumSubStepCount(newStepCount?: number): void;
        /**
         * Get the maximum number of iterations per navigation tick update
         * @returns the maximum number of iterations
         */
        getMaximumSubStepCount(): number;
        /**
         * Time factor applied when updating crowd agents (default 1). A value of 0 will pause crowd updates.
         * @param value the time factor applied at update
         */
        set timeFactor(value: number);
        /**
         * Get the time factor used for crowd agent update
         * @returns the time factor
         */
        get timeFactor(): number;
        /**
         * Creates a navigation mesh
         * @param meshes array of all the geometry used to compute the navigation mesh
         * @param parameters bunch of parameters used to filter geometry
         * @param completion callback when data is available from the worker. Not used without a worker
         */
        createNavMesh(meshes: Array<Mesh>, parameters: INavMeshParameters, completion?: (navmeshData: Uint8Array) => void): void;
        /**
         * Create a navigation mesh debug mesh
         * @param scene is where the mesh will be added
         * @returns debug display mesh
         */
        createDebugNavMesh(scene: Scene): Mesh;
        /**
         * Get a navigation mesh constrained position, closest to the parameter position
         * @param position world position
         * @returns the closest point to position constrained by the navigation mesh
         */
        getClosestPoint(position: Vector3): Vector3;
        /**
         * Get a navigation mesh constrained position, closest to the parameter position
         * @param position world position
         * @param result output the closest point to position constrained by the navigation mesh
         */
        getClosestPointToRef(position: Vector3, result: Vector3): void;
        /**
         * Get a navigation mesh constrained position, within a particular radius
         * @param position world position
         * @param maxRadius the maximum distance to the constrained world position
         * @returns the closest point to position constrained by the navigation mesh
         */
        getRandomPointAround(position: Vector3, maxRadius: number): Vector3;
        /**
         * Get a navigation mesh constrained position, within a particular radius
         * @param position world position
         * @param maxRadius the maximum distance to the constrained world position
         * @param result output the closest point to position constrained by the navigation mesh
         */
        getRandomPointAroundToRef(position: Vector3, maxRadius: number, result: Vector3): void;
        /**
         * Compute the final position from a segment made of destination-position
         * @param position world position
         * @param destination world position
         * @returns the resulting point along the navmesh
         */
        moveAlong(position: Vector3, destination: Vector3): Vector3;
        /**
         * Compute the final position from a segment made of destination-position
         * @param position world position
         * @param destination world position
         * @param result output the resulting point along the navmesh
         */
        moveAlongToRef(position: Vector3, destination: Vector3, result: Vector3): void;
        /**
         * Compute a navigation path from start to end. Returns an empty array if no path can be computed
         * @param start world position
         * @param end world position
         * @returns array containing world position composing the path
         */
        computePath(start: Vector3, end: Vector3): Vector3[];
        /**
         * Create a new Crowd so you can add agents
         * @param maxAgents the maximum agent count in the crowd
         * @param maxAgentRadius the maximum radius an agent can have
         * @param scene to attach the crowd to
         * @returns the crowd you can add agents to
         */
        createCrowd(maxAgents: number, maxAgentRadius: number, scene: Scene): ICrowd;
        /**
         * Set the Bounding box extent for doing spatial queries (getClosestPoint, getRandomPointAround, ...)
         * The queries will try to find a solution within those bounds
         * default is (1,1,1)
         * @param extent x,y,z value that define the extent around the queries point of reference
         */
        setDefaultQueryExtent(extent: Vector3): void;
        /**
         * Get the Bounding box extent specified by setDefaultQueryExtent
         * @returns the box extent values
         */
        getDefaultQueryExtent(): Vector3;
        /**
         * build the navmesh from a previously saved state using getNavmeshData
         * @param data the Uint8Array returned by getNavmeshData
         */
        buildFromNavmeshData(data: Uint8Array): void;
        /**
         * returns the navmesh data that can be used later. The navmesh must be built before retrieving the data
         * @returns data the Uint8Array that can be saved and reused
         */
        getNavmeshData(): Uint8Array;
        /**
         * Get the Bounding box extent result specified by setDefaultQueryExtent
         * @param result output the box extent values
         */
        getDefaultQueryExtentToRef(result: Vector3): void;
        /**
         * Disposes
         */
        dispose(): void;
        /**
         * Creates a cylinder obstacle and add it to the navigation
         * @param position world position
         * @param radius cylinder radius
         * @param height cylinder height
         * @returns the obstacle freshly created
         */
        addCylinderObstacle(position: Vector3, radius: number, height: number): IObstacle;
        /**
         * Creates an oriented box obstacle and add it to the navigation
         * @param position world position
         * @param extent box size
         * @param angle angle in radians of the box orientation on Y axis
         * @returns the obstacle freshly created
         */
        addBoxObstacle(position: Vector3, extent: Vector3, angle: number): IObstacle;
        /**
         * Removes an obstacle created by addCylinderObstacle or addBoxObstacle
         * @param obstacle obstacle to remove from the navigation
         */
        removeObstacle(obstacle: IObstacle): void;
        /**
         * If this plugin is supported
         * @returns true if plugin is supported
         */
        isSupported(): boolean;
    }
    /**
     * Recast detour crowd implementation
     */
    export class RecastJSCrowd implements ICrowd {
        /**
         * Recast/detour plugin
         */
        bjsRECASTPlugin: RecastJSPlugin;
        /**
         * Link to the detour crowd
         */
        recastCrowd: any;
        /**
         * One transform per agent
         */
        transforms: TransformNode[];
        /**
         * All agents created
         */
        agents: number[];
        /**
         * agents reach radius
         */
        reachRadii: number[];
        /**
         * true when a destination is active for an agent and notifier hasn't been notified of reach
         */
        private _agentDestinationArmed;
        /**
         * agent current target
         */
        private _agentDestination;
        /**
         * Link to the scene is kept to unregister the crowd from the scene
         */
        private _scene;
        /**
         * Observer for crowd updates
         */
        private _onBeforeAnimationsObserver;
        /**
         * Fires each time an agent is in reach radius of its destination
         */
        onReachTargetObservable: Observable<{
            agentIndex: number;
            destination: Vector3;
        }>;
        /**
         * Constructor
         * @param plugin recastJS plugin
         * @param maxAgents the maximum agent count in the crowd
         * @param maxAgentRadius the maximum radius an agent can have
         * @param scene to attach the crowd to
         * @returns the crowd you can add agents to
         */
        constructor(plugin: RecastJSPlugin, maxAgents: number, maxAgentRadius: number, scene: Scene);
        /**
         * Add a new agent to the crowd with the specified parameter a corresponding transformNode.
         * You can attach anything to that node. The node position is updated in the scene update tick.
         * @param pos world position that will be constrained by the navigation mesh
         * @param parameters agent parameters
         * @param transform hooked to the agent that will be update by the scene
         * @returns agent index
         */
        addAgent(pos: Vector3, parameters: IAgentParameters, transform: TransformNode): number;
        /**
         * Returns the agent position in world space
         * @param index agent index returned by addAgent
         * @returns world space position
         */
        getAgentPosition(index: number): Vector3;
        /**
         * Returns the agent position result in world space
         * @param index agent index returned by addAgent
         * @param result output world space position
         */
        getAgentPositionToRef(index: number, result: Vector3): void;
        /**
         * Returns the agent velocity in world space
         * @param index agent index returned by addAgent
         * @returns world space velocity
         */
        getAgentVelocity(index: number): Vector3;
        /**
         * Returns the agent velocity result in world space
         * @param index agent index returned by addAgent
         * @param result output world space velocity
         */
        getAgentVelocityToRef(index: number, result: Vector3): void;
        /**
         * Returns the agent next target point on the path
         * @param index agent index returned by addAgent
         * @returns world space position
         */
        getAgentNextTargetPath(index: number): Vector3;
        /**
         * Returns the agent next target point on the path
         * @param index agent index returned by addAgent
         * @param result output world space position
         */
        getAgentNextTargetPathToRef(index: number, result: Vector3): void;
        /**
         * Gets the agent state
         * @param index agent index returned by addAgent
         * @returns agent state
         */
        getAgentState(index: number): number;
        /**
         * returns true if the agent in over an off mesh link connection
         * @param index agent index returned by addAgent
         * @returns true if over an off mesh link connection
         */
        overOffmeshConnection(index: number): boolean;
        /**
         * Asks a particular agent to go to a destination. That destination is constrained by the navigation mesh
         * @param index agent index returned by addAgent
         * @param destination targeted world position
         */
        agentGoto(index: number, destination: Vector3): void;
        /**
         * Teleport the agent to a new position
         * @param index agent index returned by addAgent
         * @param destination targeted world position
         */
        agentTeleport(index: number, destination: Vector3): void;
        /**
         * Update agent parameters
         * @param index agent index returned by addAgent
         * @param parameters agent parameters
         */
        updateAgentParameters(index: number, parameters: IAgentParameters): void;
        /**
         * remove a particular agent previously created
         * @param index agent index returned by addAgent
         */
        removeAgent(index: number): void;
        /**
         * get the list of all agents attached to this crowd
         * @returns list of agent indices
         */
        getAgents(): number[];
        /**
         * Tick update done by the Scene. Agent position/velocity/acceleration is updated by this function
         * @param deltaTime in seconds
         */
        update(deltaTime: number): void;
        /**
         * Set the Bounding box extent for doing spatial queries (getClosestPoint, getRandomPointAround, ...)
         * The queries will try to find a solution within those bounds
         * default is (1,1,1)
         * @param extent x,y,z value that define the extent around the queries point of reference
         */
        setDefaultQueryExtent(extent: Vector3): void;
        /**
         * Get the Bounding box extent specified by setDefaultQueryExtent
         * @returns the box extent values
         */
        getDefaultQueryExtent(): Vector3;
        /**
         * Get the Bounding box extent result specified by setDefaultQueryExtent
         * @param result output the box extent values
         */
        getDefaultQueryExtentToRef(result: Vector3): void;
        /**
         * Get the next corner points composing the path (max 4 points)
         * @param index agent index returned by addAgent
         * @returns array containing world position composing the path
         */
        getCorners(index: number): Vector3[];
        /**
         * Release all resources
         */
        dispose(): void;
    }


    /**
     * Defines how a node can be built from a string name.
     */
    export type NodeConstructor = (name: string, scene: Scene, options?: any) => () => Node;
    /**
     * Node is the basic class for all scene objects (Mesh, Light, Camera.)
     */
    export class Node implements IBehaviorAware<Node> {
        protected _isDirty: boolean;
        /**
         * @internal
         */
        static _AnimationRangeFactory: (_name: string, _from: number, _to: number) => AnimationRange;
        private static _NodeConstructors;
        /**
         * Add a new node constructor
         * @param type defines the type name of the node to construct
         * @param constructorFunc defines the constructor function
         */
        static AddNodeConstructor(type: string, constructorFunc: NodeConstructor): void;
        /**
         * Returns a node constructor based on type name
         * @param type defines the type name
         * @param name defines the new node name
         * @param scene defines the hosting scene
         * @param options defines optional options to transmit to constructors
         * @returns the new constructor or null
         */
        static Construct(type: string, name: string, scene: Scene, options?: any): Nullable<() => Node>;
        private _nodeDataStorage;
        /**
         * Gets or sets the name of the node
         */
        name: string;
        /**
         * Gets or sets the id of the node
         */
        id: string;
        /**
         * Gets or sets the unique id of the node
         */
        uniqueId: number;
        /**
         * Gets or sets a string used to store user defined state for the node
         */
        state: string;
        /**
         * Gets or sets an object used to store user defined information for the node
         */
        metadata: any;
        /** @internal */
        _internalMetadata: any;
        /**
         * For internal use only. Please do not use.
         */
        reservedDataStore: any;
        /**
         * List of inspectable custom properties (used by the Inspector)
         * @see https://doc.babylonjs.com/toolsAndResources/inspector#extensibility
         */
        inspectableCustomProperties: IInspectable[];
        /**
         * Gets or sets the accessibility tag to describe the node for accessibility purpose.
         */
        set accessibilityTag(value: Nullable<IAccessibilityTag>);
        get accessibilityTag(): Nullable<IAccessibilityTag>;
        protected _accessibilityTag: Nullable<IAccessibilityTag>;
        onAccessibilityTagChangedObservable: Observable<Nullable<IAccessibilityTag>>;
        /**
         * Gets or sets a boolean used to define if the node must be serialized
         */
        get doNotSerialize(): boolean;
        set doNotSerialize(value: boolean);
        /** @internal */
        _parentContainer: Nullable<AbstractScene>;
        /**
         * Gets a list of Animations associated with the node
         */
        animations: Animation[];
        protected _ranges: {
            [name: string]: Nullable<AnimationRange>;
        };
        /**
         * Callback raised when the node is ready to be used
         */
        onReady: Nullable<(node: Node) => void>;
        /** @internal */
        _currentRenderId: number;
        private _parentUpdateId;
        /** @internal */
        _childUpdateId: number;
        /** @internal */
        _waitingParentId: Nullable<string>;
        /** @internal */
        _waitingParentInstanceIndex: Nullable<string>;
        /** @internal */
        _waitingParsedUniqueId: Nullable<number>;
        /** @internal */
        _scene: Scene;
        /** @internal */
        _cache: any;
        protected _parentNode: Nullable<Node>;
        /** @internal */
        protected _children: Nullable<Node[]>;
        /** @internal */
        _worldMatrix: Matrix;
        /** @internal */
        _worldMatrixDeterminant: number;
        /** @internal */
        _worldMatrixDeterminantIsDirty: boolean;
        /**
         * Gets a boolean indicating if the node has been disposed
         * @returns true if the node was disposed
         */
        isDisposed(): boolean;
        /**
         * Gets or sets the parent of the node (without keeping the current position in the scene)
         * @see https://doc.babylonjs.com/features/featuresDeepDive/mesh/transforms/parent_pivot/parent
         */
        set parent(parent: Nullable<Node>);
        get parent(): Nullable<Node>;
        /**
         * @internal
         */
        _serializeAsParent(serializationObject: any): void;
        /** @internal */
        _addToSceneRootNodes(): void;
        /** @internal */
        _removeFromSceneRootNodes(): void;
        private _animationPropertiesOverride;
        /**
         * Gets or sets the animation properties override
         */
        get animationPropertiesOverride(): Nullable<AnimationPropertiesOverride>;
        set animationPropertiesOverride(value: Nullable<AnimationPropertiesOverride>);
        /**
         * Gets a string identifying the name of the class
         * @returns "Node" string
         */
        getClassName(): string;
        /** @internal */
        readonly _isNode = true;
        /**
         * An event triggered when the mesh is disposed
         */
        onDisposeObservable: Observable<Node>;
        private _onDisposeObserver;
        /**
         * Sets a callback that will be raised when the node will be disposed
         */
        set onDispose(callback: () => void);
        /**
         * An event triggered when the enabled state of the node changes
         */
        get onEnabledStateChangedObservable(): Observable<boolean>;
        /**
         * An event triggered when the node is cloned
         */
        get onClonedObservable(): Observable<Node>;
        /**
         * Creates a new Node
         * @param name the name and id to be given to this node
         * @param scene the scene this node will be added to
         */
        constructor(name: string, scene?: Nullable<Scene>);
        /**
         * Gets the scene of the node
         * @returns a scene
         */
        getScene(): Scene;
        /**
         * Gets the engine of the node
         * @returns a Engine
         */
        getEngine(): Engine;
        private _behaviors;
        /**
         * Attach a behavior to the node
         * @see https://doc.babylonjs.com/features/featuresDeepDive/behaviors
         * @param behavior defines the behavior to attach
         * @param attachImmediately defines that the behavior must be attached even if the scene is still loading
         * @returns the current Node
         */
        addBehavior(behavior: Behavior<Node>, attachImmediately?: boolean): Node;
        /**
         * Remove an attached behavior
         * @see https://doc.babylonjs.com/features/featuresDeepDive/behaviors
         * @param behavior defines the behavior to attach
         * @returns the current Node
         */
        removeBehavior(behavior: Behavior<Node>): Node;
        /**
         * Gets the list of attached behaviors
         * @see https://doc.babylonjs.com/features/featuresDeepDive/behaviors
         */
        get behaviors(): Behavior<Node>[];
        /**
         * Gets an attached behavior by name
         * @param name defines the name of the behavior to look for
         * @see https://doc.babylonjs.com/features/featuresDeepDive/behaviors
         * @returns null if behavior was not found else the requested behavior
         */
        getBehaviorByName(name: string): Nullable<Behavior<Node>>;
        /**
         * Returns the latest update of the World matrix
         * @returns a Matrix
         */
        getWorldMatrix(): Matrix;
        /** @internal */
        _getWorldMatrixDeterminant(): number;
        /**
         * Returns directly the latest state of the mesh World matrix.
         * A Matrix is returned.
         */
        get worldMatrixFromCache(): Matrix;
        /** @internal */
        _initCache(): void;
        /**
         * @internal
         */
        updateCache(force?: boolean): void;
        /**
         * @internal
         */
        _getActionManagerForTrigger(trigger?: number, _initialCall?: boolean): Nullable<AbstractActionManager>;
        /**
         * @internal
         */
        _updateCache(_ignoreParentClass?: boolean): void;
        /** @internal */
        _isSynchronized(): boolean;
        /** @internal */
        _markSyncedWithParent(): void;
        /** @internal */
        isSynchronizedWithParent(): boolean;
        /** @internal */
        isSynchronized(): boolean;
        /**
         * Is this node ready to be used/rendered
         * @param _completeCheck defines if a complete check (including materials and lights) has to be done (false by default)
         * @returns true if the node is ready
         */
        isReady(_completeCheck?: boolean): boolean;
        /**
         * Flag the  node as dirty (Forcing it to update everything)
         * @param _property helps children apply precise "dirtyfication"
         * @returns this node
         */
        markAsDirty(_property?: string): Node;
        /**
         * Is this node enabled?
         * If the node has a parent, all ancestors will be checked and false will be returned if any are false (not enabled), otherwise will return true
         * @param checkAncestors indicates if this method should check the ancestors. The default is to check the ancestors. If set to false, the method will return the value of this node without checking ancestors
         * @returns whether this node (and its parent) is enabled
         */
        isEnabled(checkAncestors?: boolean): boolean;
        /** @internal */
        protected _syncParentEnabledState(): void;
        /**
         * Set the enabled state of this node
         * @param value defines the new enabled state
         */
        setEnabled(value: boolean): void;
        /**
         * Is this node a descendant of the given node?
         * The function will iterate up the hierarchy until the ancestor was found or no more parents defined
         * @param ancestor defines the parent node to inspect
         * @returns a boolean indicating if this node is a descendant of the given node
         */
        isDescendantOf(ancestor: Node): boolean;
        /**
         * @internal
         */
        _getDescendants(results: Node[], directDescendantsOnly?: boolean, predicate?: (node: Node) => boolean): void;
        /**
         * Will return all nodes that have this node as ascendant
         * @param directDescendantsOnly defines if true only direct descendants of 'this' will be considered, if false direct and also indirect (children of children, an so on in a recursive manner) descendants of 'this' will be considered
         * @param predicate defines an optional predicate that will be called on every evaluated child, the predicate must return true for a given child to be part of the result, otherwise it will be ignored
         * @returns all children nodes of all types
         */
        getDescendants<T extends Node>(directDescendantsOnly?: boolean, predicate?: (node: Node) => node is T): T[];
        /**
         * Will return all nodes that have this node as ascendant
         * @param directDescendantsOnly defines if true only direct descendants of 'this' will be considered, if false direct and also indirect (children of children, an so on in a recursive manner) descendants of 'this' will be considered
         * @param predicate defines an optional predicate that will be called on every evaluated child, the predicate must return true for a given child to be part of the result, otherwise it will be ignored
         * @returns all children nodes of all types
         */
        getDescendants(directDescendantsOnly?: boolean, predicate?: (node: Node) => boolean): Node[];
        /**
         * Get all child-meshes of this node
         * @param directDescendantsOnly defines if true only direct descendants of 'this' will be considered, if false direct and also indirect (children of children, an so on in a recursive manner) descendants of 'this' will be considered (Default: false)
         * @param predicate defines an optional predicate that will be called on every evaluated child, the predicate must return true for a given child to be part of the result, otherwise it will be ignored
         * @returns an array of AbstractMesh
         */
        getChildMeshes<T extends AbstractMesh>(directDescendantsOnly?: boolean, predicate?: (node: Node) => node is T): T[];
        /**
         * Get all child-meshes of this node
         * @param directDescendantsOnly defines if true only direct descendants of 'this' will be considered, if false direct and also indirect (children of children, an so on in a recursive manner) descendants of 'this' will be considered (Default: false)
         * @param predicate defines an optional predicate that will be called on every evaluated child, the predicate must return true for a given child to be part of the result, otherwise it will be ignored
         * @returns an array of AbstractMesh
         */
        getChildMeshes(directDescendantsOnly?: boolean, predicate?: (node: Node) => boolean): AbstractMesh[];
        /**
         * Get all direct children of this node
         * @param predicate defines an optional predicate that will be called on every evaluated child, the predicate must return true for a given child to be part of the result, otherwise it will be ignored
         * @param directDescendantsOnly defines if true only direct descendants of 'this' will be considered, if false direct and also indirect (children of children, an so on in a recursive manner) descendants of 'this' will be considered (Default: true)
         * @returns an array of Node
         */
        getChildren<T extends Node>(predicate?: (node: Node) => node is T, directDescendantsOnly?: boolean): T[];
        /**
         * Get all direct children of this node
         * @param predicate defines an optional predicate that will be called on every evaluated child, the predicate must return true for a given child to be part of the result, otherwise it will be ignored
         * @param directDescendantsOnly defines if true only direct descendants of 'this' will be considered, if false direct and also indirect (children of children, an so on in a recursive manner) descendants of 'this' will be considered (Default: true)
         * @returns an array of Node
         */
        getChildren(predicate?: (node: Node) => boolean, directDescendantsOnly?: boolean): Node[];
        /**
         * @internal
         */
        _setReady(state: boolean): void;
        /**
         * Get an animation by name
         * @param name defines the name of the animation to look for
         * @returns null if not found else the requested animation
         */
        getAnimationByName(name: string): Nullable<Animation>;
        /**
         * Creates an animation range for this node
         * @param name defines the name of the range
         * @param from defines the starting key
         * @param to defines the end key
         */
        createAnimationRange(name: string, from: number, to: number): void;
        /**
         * Delete a specific animation range
         * @param name defines the name of the range to delete
         * @param deleteFrames defines if animation frames from the range must be deleted as well
         */
        deleteAnimationRange(name: string, deleteFrames?: boolean): void;
        /**
         * Get an animation range by name
         * @param name defines the name of the animation range to look for
         * @returns null if not found else the requested animation range
         */
        getAnimationRange(name: string): Nullable<AnimationRange>;
        /**
         * Gets the list of all animation ranges defined on this node
         * @returns an array
         */
        getAnimationRanges(): Nullable<AnimationRange>[];
        /**
         * Will start the animation sequence
         * @param name defines the range frames for animation sequence
         * @param loop defines if the animation should loop (false by default)
         * @param speedRatio defines the speed factor in which to run the animation (1 by default)
         * @param onAnimationEnd defines a function to be executed when the animation ended (undefined by default)
         * @returns the object created for this animation. If range does not exist, it will return null
         */
        beginAnimation(name: string, loop?: boolean, speedRatio?: number, onAnimationEnd?: () => void): Nullable<Animatable>;
        /**
         * Serialize animation ranges into a JSON compatible object
         * @returns serialization object
         */
        serializeAnimationRanges(): any;
        /**
         * Computes the world matrix of the node
         * @param _force defines if the cache version should be invalidated forcing the world matrix to be created from scratch
         * @returns the world matrix
         */
        computeWorldMatrix(_force?: boolean): Matrix;
        /**
         * Releases resources associated with this node.
         * @param doNotRecurse Set to true to not recurse into each children (recurse into each children by default)
         * @param disposeMaterialAndTextures Set to true to also dispose referenced materials and textures (false by default)
         */
        dispose(doNotRecurse?: boolean, disposeMaterialAndTextures?: boolean): void;
        /**
         * Parse animation range data from a serialization object and store them into a given node
         * @param node defines where to store the animation ranges
         * @param parsedNode defines the serialization object to read data from
         * @param _scene defines the hosting scene
         */
        static ParseAnimationRanges(node: Node, parsedNode: any, _scene: Scene): void;
        /**
         * Return the minimum and maximum world vectors of the entire hierarchy under current node
         * @param includeDescendants Include bounding info from descendants as well (true by default)
         * @param predicate defines a callback function that can be customize to filter what meshes should be included in the list used to compute the bounding vectors
         * @returns the new bounding vectors
         */
        getHierarchyBoundingVectors(includeDescendants?: boolean, predicate?: Nullable<(abstractMesh: AbstractMesh) => boolean>): {
            min: Vector3;
            max: Vector3;
        };
    }


    /**
     * Class used to enable access to IndexedDB
     * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/optimizeCached
     */
    export class Database implements IOfflineProvider {
        private _currentSceneUrl;
        private _db;
        private _enableSceneOffline;
        private _enableTexturesOffline;
        private _manifestVersionFound;
        private _mustUpdateRessources;
        private _hasReachedQuota;
        private _isSupported;
        private _idbFactory;
        /** Gets a boolean indicating if the user agent supports blob storage (this value will be updated after creating the first Database object) */
        private static _IsUASupportingBlobStorage;
        /**
         * Gets a boolean indicating if Database storage is enabled (off by default)
         */
        static IDBStorageEnabled: boolean;
        /**
         * Gets a boolean indicating if scene must be saved in the database
         */
        get enableSceneOffline(): boolean;
        /**
         * Gets a boolean indicating if textures must be saved in the database
         */
        get enableTexturesOffline(): boolean;
        /**
         * Creates a new Database
         * @param urlToScene defines the url to load the scene
         * @param callbackManifestChecked defines the callback to use when manifest is checked
         * @param disableManifestCheck defines a boolean indicating that we want to skip the manifest validation (it will be considered validated and up to date)
         */
        constructor(urlToScene: string, callbackManifestChecked: (checked: boolean) => any, disableManifestCheck?: boolean);
        private static _ParseURL;
        private static _ReturnFullUrlLocation;
        private _checkManifestFile;
        /**
         * Open the database and make it available
         * @param successCallback defines the callback to call on success
         * @param errorCallback defines the callback to call on error
         */
        open(successCallback: () => void, errorCallback: () => void): void;
        /**
         * Loads an image from the database
         * @param url defines the url to load from
         * @param image defines the target DOM image
         */
        loadImage(url: string, image: HTMLImageElement): void;
        private _loadImageFromDBAsync;
        private _saveImageIntoDBAsync;
        private _checkVersionFromDB;
        private _loadVersionFromDBAsync;
        private _saveVersionIntoDBAsync;
        /**
         * Loads a file from database
         * @param url defines the URL to load from
         * @param sceneLoaded defines a callback to call on success
         * @param progressCallBack defines a callback to call when progress changed
         * @param errorCallback defines a callback to call on error
         * @param useArrayBuffer defines a boolean to use array buffer instead of text string
         */
        loadFile(url: string, sceneLoaded: (data: any) => void, progressCallBack?: (data: any) => void, errorCallback?: () => void, useArrayBuffer?: boolean): void;
        private _loadFileAsync;
        private _saveFileAsync;
        /**
         * Validates if xhr data is correct
         * @param xhr defines the request to validate
         * @param dataType defines the expected data type
         * @returns true if data is correct
         */
        private static _ValidateXHRData;
    }




    /**
     * Class used to enable access to offline support
     * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/optimizeCached
     */
    export interface IOfflineProvider {
        /**
         * Gets a boolean indicating if scene must be saved in the database
         */
        enableSceneOffline: boolean;
        /**
         * Gets a boolean indicating if textures must be saved in the database
         */
        enableTexturesOffline: boolean;
        /**
         * Open the offline support and make it available
         * @param successCallback defines the callback to call on success
         * @param errorCallback defines the callback to call on error
         */
        open(successCallback: () => void, errorCallback: () => void): void;
        /**
         * Loads an image from the offline support
         * @param url defines the url to load from
         * @param image defines the target DOM image
         */
        loadImage(url: string, image: HTMLImageElement): void;
        /**
         * Loads a file from offline support
         * @param url defines the URL to load from
         * @param sceneLoaded defines a callback to call on success
         * @param progressCallBack defines a callback to call when progress changed
         * @param errorCallback defines a callback to call on error
         * @param useArrayBuffer defines a boolean to use array buffer instead of text string
         */
        loadFile(url: string, sceneLoaded: (data: any) => void, progressCallBack?: (data: any) => void, errorCallback?: () => void, useArrayBuffer?: boolean): void;
    }


    /**
     * This represents the base class for particle system in Babylon.
     * Particles are often small sprites used to simulate hard-to-reproduce phenomena like fire, smoke, water, or abstract visual effects like magic glitter and faery dust.
     * Particles can take different shapes while emitted like box, sphere, cone or you can write your custom function.
     * @example https://doc.babylonjs.com/features/featuresDeepDive/particles/particle_system/particle_system_intro
     */
    export class BaseParticleSystem implements IClipPlanesHolder {
        /**
         * Source color is added to the destination color without alpha affecting the result
         */
        static BLENDMODE_ONEONE: number;
        /**
         * Blend current color and particle color using particle’s alpha
         */
        static BLENDMODE_STANDARD: number;
        /**
         * Add current color and particle color multiplied by particle’s alpha
         */
        static BLENDMODE_ADD: number;
        /**
         * Multiply current color with particle color
         */
        static BLENDMODE_MULTIPLY: number;
        /**
         * Multiply current color with particle color then add current color and particle color multiplied by particle’s alpha
         */
        static BLENDMODE_MULTIPLYADD: number;
        /**
         * List of animations used by the particle system.
         */
        animations: Animation[];
        /**
         * Gets or sets the unique id of the particle system
         */
        uniqueId: number;
        /**
         * The id of the Particle system.
         */
        id: string;
        /**
         * The friendly name of the Particle system.
         */
        name: string;
        /**
         * Snippet ID if the particle system was created from the snippet server
         */
        snippetId: string;
        /**
         * The rendering group used by the Particle system to chose when to render.
         */
        renderingGroupId: number;
        /**
         * The emitter represents the Mesh or position we are attaching the particle system to.
         */
        emitter: Nullable<AbstractMesh | Vector3>;
        /**
         * The maximum number of particles to emit per frame
         */
        emitRate: number;
        /**
         * If you want to launch only a few particles at once, that can be done, as well.
         */
        manualEmitCount: number;
        /**
         * The overall motion speed (0.01 is default update speed, faster updates = faster animation)
         */
        updateSpeed: number;
        /**
         * The amount of time the particle system is running (depends of the overall update speed).
         */
        targetStopDuration: number;
        /**
         * Specifies whether the particle system will be disposed once it reaches the end of the animation.
         */
        disposeOnStop: boolean;
        /**
         * Minimum power of emitting particles.
         */
        minEmitPower: number;
        /**
         * Maximum power of emitting particles.
         */
        maxEmitPower: number;
        /**
         * Minimum life time of emitting particles.
         */
        minLifeTime: number;
        /**
         * Maximum life time of emitting particles.
         */
        maxLifeTime: number;
        /**
         * Minimum Size of emitting particles.
         */
        minSize: number;
        /**
         * Maximum Size of emitting particles.
         */
        maxSize: number;
        /**
         * Minimum scale of emitting particles on X axis.
         */
        minScaleX: number;
        /**
         * Maximum scale of emitting particles on X axis.
         */
        maxScaleX: number;
        /**
         * Minimum scale of emitting particles on Y axis.
         */
        minScaleY: number;
        /**
         * Maximum scale of emitting particles on Y axis.
         */
        maxScaleY: number;
        /**
         * Gets or sets the minimal initial rotation in radians.
         */
        minInitialRotation: number;
        /**
         * Gets or sets the maximal initial rotation in radians.
         */
        maxInitialRotation: number;
        /**
         * Minimum angular speed of emitting particles (Z-axis rotation for each particle).
         */
        minAngularSpeed: number;
        /**
         * Maximum angular speed of emitting particles (Z-axis rotation for each particle).
         */
        maxAngularSpeed: number;
        /**
         * The texture used to render each particle. (this can be a spritesheet)
         */
        particleTexture: Nullable<BaseTexture>;
        /**
         * The layer mask we are rendering the particles through.
         */
        layerMask: number;
        /**
         * This can help using your own shader to render the particle system.
         * The according effect will be created
         */
        customShader: any;
        /**
         * By default particle system starts as soon as they are created. This prevents the
         * automatic start to happen and let you decide when to start emitting particles.
         */
        preventAutoStart: boolean;
        /** @internal */
        _wasDispatched: boolean;
        protected _rootUrl: string;
        private _noiseTexture;
        /**
         * Gets or sets a texture used to add random noise to particle positions
         */
        get noiseTexture(): Nullable<ProceduralTexture>;
        set noiseTexture(value: Nullable<ProceduralTexture>);
        /** Gets or sets the strength to apply to the noise value (default is (10, 10, 10)) */
        noiseStrength: Vector3;
        /**
         * Callback triggered when the particle animation is ending.
         */
        onAnimationEnd: Nullable<() => void>;
        /**
         * Blend mode use to render the particle, it can be either ParticleSystem.BLENDMODE_ONEONE or ParticleSystem.BLENDMODE_STANDARD.
         */
        blendMode: number;
        /**
         * Forces the particle to write their depth information to the depth buffer. This can help preventing other draw calls
         * to override the particles.
         */
        forceDepthWrite: boolean;
        /** Gets or sets a value indicating how many cycles (or frames) must be executed before first rendering (this value has to be set before starting the system). Default is 0 */
        preWarmCycles: number;
        /** Gets or sets a value indicating the time step multiplier to use in pre-warm mode (default is 1) */
        preWarmStepOffset: number;
        /**
         * If using a spritesheet (isAnimationSheetEnabled) defines the speed of the sprite loop (default is 1 meaning the animation will play once during the entire particle lifetime)
         */
        spriteCellChangeSpeed: number;
        /**
         * If using a spritesheet (isAnimationSheetEnabled) defines the first sprite cell to display
         */
        startSpriteCellID: number;
        /**
         * If using a spritesheet (isAnimationSheetEnabled) defines the last sprite cell to display
         */
        endSpriteCellID: number;
        /**
         * If using a spritesheet (isAnimationSheetEnabled), defines the sprite cell width to use
         */
        spriteCellWidth: number;
        /**
         * If using a spritesheet (isAnimationSheetEnabled), defines the sprite cell height to use
         */
        spriteCellHeight: number;
        /**
         * If using a spritesheet (isAnimationSheetEnabled), defines wether the sprite animation is looping
         */
        spriteCellLoop: boolean;
        /**
         * This allows the system to random pick the start cell ID between startSpriteCellID and endSpriteCellID
         */
        spriteRandomStartCell: boolean;
        /** Gets or sets a Vector2 used to move the pivot (by default (0,0)) */
        translationPivot: Vector2;
        /** @internal */
        _isAnimationSheetEnabled: boolean;
        /**
         * Gets or sets a boolean indicating that hosted animations (in the system.animations array) must be started when system.start() is called
         */
        beginAnimationOnStart: boolean;
        /**
         * Gets or sets the frame to start the animation from when beginAnimationOnStart is true
         */
        beginAnimationFrom: number;
        /**
         * Gets or sets the frame to end the animation on when beginAnimationOnStart is true
         */
        beginAnimationTo: number;
        /**
         * Gets or sets a boolean indicating if animations must loop when beginAnimationOnStart is true
         */
        beginAnimationLoop: boolean;
        /**
         * Gets or sets a world offset applied to all particles
         */
        worldOffset: Vector3;
        /**
         * Gets or sets the active clipplane 1
         */
        clipPlane: Nullable<Plane>;
        /**
         * Gets or sets the active clipplane 2
         */
        clipPlane2: Nullable<Plane>;
        /**
         * Gets or sets the active clipplane 3
         */
        clipPlane3: Nullable<Plane>;
        /**
         * Gets or sets the active clipplane 4
         */
        clipPlane4: Nullable<Plane>;
        /**
         * Gets or sets the active clipplane 5
         */
        clipPlane5: Nullable<Plane>;
        /**
         * Gets or sets the active clipplane 6
         */
        clipPlane6: Nullable<Plane>;
        /**
         * Gets or sets whether an animation sprite sheet is enabled or not on the particle system
         */
        get isAnimationSheetEnabled(): boolean;
        set isAnimationSheetEnabled(value: boolean);
        private _useLogarithmicDepth;
        /**
         * Gets or sets a boolean enabling the use of logarithmic depth buffers, which is good for wide depth buffers.
         */
        get useLogarithmicDepth(): boolean;
        set useLogarithmicDepth(value: boolean);
        /**
         * Get hosting scene
         * @returns the scene
         */
        getScene(): Nullable<Scene>;
        /**
         * You can use gravity if you want to give an orientation to your particles.
         */
        gravity: Vector3;
        protected _colorGradients: Nullable<Array<ColorGradient>>;
        protected _sizeGradients: Nullable<Array<FactorGradient>>;
        protected _lifeTimeGradients: Nullable<Array<FactorGradient>>;
        protected _angularSpeedGradients: Nullable<Array<FactorGradient>>;
        protected _velocityGradients: Nullable<Array<FactorGradient>>;
        protected _limitVelocityGradients: Nullable<Array<FactorGradient>>;
        protected _dragGradients: Nullable<Array<FactorGradient>>;
        protected _emitRateGradients: Nullable<Array<FactorGradient>>;
        protected _startSizeGradients: Nullable<Array<FactorGradient>>;
        protected _rampGradients: Nullable<Array<Color3Gradient>>;
        protected _colorRemapGradients: Nullable<Array<FactorGradient>>;
        protected _alphaRemapGradients: Nullable<Array<FactorGradient>>;
        protected _hasTargetStopDurationDependantGradient(): boolean | null;
        /**
         * Defines the delay in milliseconds before starting the system (0 by default)
         */
        startDelay: number;
        /**
         * Gets the current list of drag gradients.
         * You must use addDragGradient and removeDragGradient to update this list
         * @returns the list of drag gradients
         */
        getDragGradients(): Nullable<Array<FactorGradient>>;
        /** Gets or sets a value indicating the damping to apply if the limit velocity factor is reached */
        limitVelocityDamping: number;
        /**
         * Gets the current list of limit velocity gradients.
         * You must use addLimitVelocityGradient and removeLimitVelocityGradient to update this list
         * @returns the list of limit velocity gradients
         */
        getLimitVelocityGradients(): Nullable<Array<FactorGradient>>;
        /**
         * Gets the current list of color gradients.
         * You must use addColorGradient and removeColorGradient to update this list
         * @returns the list of color gradients
         */
        getColorGradients(): Nullable<Array<ColorGradient>>;
        /**
         * Gets the current list of size gradients.
         * You must use addSizeGradient and removeSizeGradient to update this list
         * @returns the list of size gradients
         */
        getSizeGradients(): Nullable<Array<FactorGradient>>;
        /**
         * Gets the current list of color remap gradients.
         * You must use addColorRemapGradient and removeColorRemapGradient to update this list
         * @returns the list of color remap gradients
         */
        getColorRemapGradients(): Nullable<Array<FactorGradient>>;
        /**
         * Gets the current list of alpha remap gradients.
         * You must use addAlphaRemapGradient and removeAlphaRemapGradient to update this list
         * @returns the list of alpha remap gradients
         */
        getAlphaRemapGradients(): Nullable<Array<FactorGradient>>;
        /**
         * Gets the current list of life time gradients.
         * You must use addLifeTimeGradient and removeLifeTimeGradient to update this list
         * @returns the list of life time gradients
         */
        getLifeTimeGradients(): Nullable<Array<FactorGradient>>;
        /**
         * Gets the current list of angular speed gradients.
         * You must use addAngularSpeedGradient and removeAngularSpeedGradient to update this list
         * @returns the list of angular speed gradients
         */
        getAngularSpeedGradients(): Nullable<Array<FactorGradient>>;
        /**
         * Gets the current list of velocity gradients.
         * You must use addVelocityGradient and removeVelocityGradient to update this list
         * @returns the list of velocity gradients
         */
        getVelocityGradients(): Nullable<Array<FactorGradient>>;
        /**
         * Gets the current list of start size gradients.
         * You must use addStartSizeGradient and removeStartSizeGradient to update this list
         * @returns the list of start size gradients
         */
        getStartSizeGradients(): Nullable<Array<FactorGradient>>;
        /**
         * Gets the current list of emit rate gradients.
         * You must use addEmitRateGradient and removeEmitRateGradient to update this list
         * @returns the list of emit rate gradients
         */
        getEmitRateGradients(): Nullable<Array<FactorGradient>>;
        /**
         * Random direction of each particle after it has been emitted, between direction1 and direction2 vectors.
         * This only works when particleEmitterTyps is a BoxParticleEmitter
         */
        get direction1(): Vector3;
        set direction1(value: Vector3);
        /**
         * Random direction of each particle after it has been emitted, between direction1 and direction2 vectors.
         * This only works when particleEmitterTyps is a BoxParticleEmitter
         */
        get direction2(): Vector3;
        set direction2(value: Vector3);
        /**
         * Minimum box point around our emitter. Our emitter is the center of particles source, but if you want your particles to emit from more than one point, then you can tell it to do so.
         * This only works when particleEmitterTyps is a BoxParticleEmitter
         */
        get minEmitBox(): Vector3;
        set minEmitBox(value: Vector3);
        /**
         * Maximum box point around our emitter. Our emitter is the center of particles source, but if you want your particles to emit from more than one point, then you can tell it to do so.
         * This only works when particleEmitterTyps is a BoxParticleEmitter
         */
        get maxEmitBox(): Vector3;
        set maxEmitBox(value: Vector3);
        /**
         * Random color of each particle after it has been emitted, between color1 and color2 vectors
         */
        color1: Color4;
        /**
         * Random color of each particle after it has been emitted, between color1 and color2 vectors
         */
        color2: Color4;
        /**
         * Color the particle will have at the end of its lifetime
         */
        colorDead: Color4;
        /**
         * An optional mask to filter some colors out of the texture, or filter a part of the alpha channel
         */
        textureMask: Color4;
        /**
         * The particle emitter type defines the emitter used by the particle system.
         * It can be for example box, sphere, or cone...
         */
        particleEmitterType: IParticleEmitterType;
        /** @internal */
        _isSubEmitter: boolean;
        /** @internal */
        _billboardMode: number;
        /**
         * Gets or sets the billboard mode to use when isBillboardBased = true.
         * Value can be: ParticleSystem.BILLBOARDMODE_ALL, ParticleSystem.BILLBOARDMODE_Y, ParticleSystem.BILLBOARDMODE_STRETCHED
         */
        get billboardMode(): number;
        set billboardMode(value: number);
        /** @internal */
        _isBillboardBased: boolean;
        /**
         * Gets or sets a boolean indicating if the particles must be rendered as billboard or aligned with the direction
         */
        get isBillboardBased(): boolean;
        set isBillboardBased(value: boolean);
        /**
         * The scene the particle system belongs to.
         */
        protected _scene: Nullable<Scene>;
        /**
         * The engine the particle system belongs to.
         */
        protected _engine: ThinEngine;
        /**
         * Local cache of defines for image processing.
         */
        protected _imageProcessingConfigurationDefines: ImageProcessingConfigurationDefines;
        /**
         * Default configuration related to image processing available in the standard Material.
         */
        protected _imageProcessingConfiguration: Nullable<ImageProcessingConfiguration>;
        /**
         * Gets the image processing configuration used either in this material.
         */
        get imageProcessingConfiguration(): Nullable<ImageProcessingConfiguration>;
        /**
         * Sets the Default image processing configuration used either in the this material.
         *
         * If sets to null, the scene one is in use.
         */
        set imageProcessingConfiguration(value: Nullable<ImageProcessingConfiguration>);
        /**
         * Attaches a new image processing configuration to the Standard Material.
         * @param configuration
         */
        protected _attachImageProcessingConfiguration(configuration: Nullable<ImageProcessingConfiguration>): void;
        /** @internal */
        protected _reset(): void;
        /**
         * @internal
         */
        protected _removeGradientAndTexture(gradient: number, gradients: Nullable<IValueGradient[]>, texture: Nullable<RawTexture>): BaseParticleSystem;
        /**
         * Instantiates a particle system.
         * Particles are often small sprites used to simulate hard-to-reproduce phenomena like fire, smoke, water, or abstract visual effects like magic glitter and faery dust.
         * @param name The name of the particle system
         */
        constructor(name: string);
        /**
         * Creates a Point Emitter for the particle system (emits directly from the emitter position)
         * @param direction1 Particles are emitted between the direction1 and direction2 from within the box
         * @param direction2 Particles are emitted between the direction1 and direction2 from within the box
         * @returns the emitter
         */
        createPointEmitter(direction1: Vector3, direction2: Vector3): PointParticleEmitter;
        /**
         * Creates a Hemisphere Emitter for the particle system (emits along the hemisphere radius)
         * @param radius The radius of the hemisphere to emit from
         * @param radiusRange The range of the hemisphere to emit from [0-1] 0 Surface Only, 1 Entire Radius
         * @returns the emitter
         */
        createHemisphericEmitter(radius?: number, radiusRange?: number): HemisphericParticleEmitter;
        /**
         * Creates a Sphere Emitter for the particle system (emits along the sphere radius)
         * @param radius The radius of the sphere to emit from
         * @param radiusRange The range of the sphere to emit from [0-1] 0 Surface Only, 1 Entire Radius
         * @returns the emitter
         */
        createSphereEmitter(radius?: number, radiusRange?: number): SphereParticleEmitter;
        /**
         * Creates a Directed Sphere Emitter for the particle system (emits between direction1 and direction2)
         * @param radius The radius of the sphere to emit from
         * @param direction1 Particles are emitted between the direction1 and direction2 from within the sphere
         * @param direction2 Particles are emitted between the direction1 and direction2 from within the sphere
         * @returns the emitter
         */
        createDirectedSphereEmitter(radius?: number, direction1?: Vector3, direction2?: Vector3): SphereDirectedParticleEmitter;
        /**
         * Creates a Cylinder Emitter for the particle system (emits from the cylinder to the particle position)
         * @param radius The radius of the emission cylinder
         * @param height The height of the emission cylinder
         * @param radiusRange The range of emission [0-1] 0 Surface only, 1 Entire Radius
         * @param directionRandomizer How much to randomize the particle direction [0-1]
         * @returns the emitter
         */
        createCylinderEmitter(radius?: number, height?: number, radiusRange?: number, directionRandomizer?: number): CylinderParticleEmitter;
        /**
         * Creates a Directed Cylinder Emitter for the particle system (emits between direction1 and direction2)
         * @param radius The radius of the cylinder to emit from
         * @param height The height of the emission cylinder
         * @param radiusRange the range of the emission cylinder [0-1] 0 Surface only, 1 Entire Radius (1 by default)
         * @param direction1 Particles are emitted between the direction1 and direction2 from within the cylinder
         * @param direction2 Particles are emitted between the direction1 and direction2 from within the cylinder
         * @returns the emitter
         */
        createDirectedCylinderEmitter(radius?: number, height?: number, radiusRange?: number, direction1?: Vector3, direction2?: Vector3): CylinderDirectedParticleEmitter;
        /**
         * Creates a Cone Emitter for the particle system (emits from the cone to the particle position)
         * @param radius The radius of the cone to emit from
         * @param angle The base angle of the cone
         * @returns the emitter
         */
        createConeEmitter(radius?: number, angle?: number): ConeParticleEmitter;
        /**
         * Creates a Box Emitter for the particle system. (emits between direction1 and direction2 from withing the box defined by minEmitBox and maxEmitBox)
         * @param direction1 Particles are emitted between the direction1 and direction2 from within the box
         * @param direction2 Particles are emitted between the direction1 and direction2 from within the box
         * @param minEmitBox Particles are emitted from the box between minEmitBox and maxEmitBox
         * @param maxEmitBox  Particles are emitted from the box between minEmitBox and maxEmitBox
         * @returns the emitter
         */
        createBoxEmitter(direction1: Vector3, direction2: Vector3, minEmitBox: Vector3, maxEmitBox: Vector3): BoxParticleEmitter;
    }


    /**
     * Represents one particle of a points cloud system.
     */
    export class CloudPoint {
        /**
         * particle global index
         */
        idx: number;
        /**
         * The color of the particle
         */
        color: Nullable<Color4>;
        /**
         * The world space position of the particle.
         */
        position: Vector3;
        /**
         * The world space rotation of the particle. (Not use if rotationQuaternion is set)
         */
        rotation: Vector3;
        /**
         * The world space rotation quaternion of the particle.
         */
        rotationQuaternion: Nullable<Quaternion>;
        /**
         * The uv of the particle.
         */
        uv: Nullable<Vector2>;
        /**
         * The current speed of the particle.
         */
        velocity: Vector3;
        /**
         * The pivot point in the particle local space.
         */
        pivot: Vector3;
        /**
         * Must the particle be translated from its pivot point in its local space ?
         * In this case, the pivot point is set at the origin of the particle local space and the particle is translated.
         * Default : false
         */
        translateFromPivot: boolean;
        /**
         * Index of this particle in the global "positions" array (Internal use)
         * @internal
         */
        _pos: number;
        /**
         * @internal Index of this particle in the global "indices" array (Internal use)
         */
        _ind: number;
        /**
         * Group this particle belongs to
         */
        _group: PointsGroup;
        /**
         * Group id of this particle
         */
        groupId: number;
        /**
         * Index of the particle in its group id (Internal use)
         */
        idxInGroup: number;
        /**
         * @internal Particle BoundingInfo object (Internal use)
         */
        _boundingInfo: BoundingInfo;
        /**
         * @internal Reference to the PCS that the particle belongs to (Internal use)
         */
        _pcs: PointsCloudSystem;
        /**
         * @internal Still set as invisible in order to skip useless computations (Internal use)
         */
        _stillInvisible: boolean;
        /**
         * @internal Last computed particle rotation matrix
         */
        _rotationMatrix: number[];
        /**
         * Parent particle Id, if any.
         * Default null.
         */
        parentId: Nullable<number>;
        /**
         * @internal Internal global position in the PCS.
         */
        _globalPosition: Vector3;
        /**
         * Creates a Point Cloud object.
         * Don't create particles manually, use instead the PCS internal tools like _addParticle()
         * @param particleIndex (integer) is the particle index in the PCS pool. It's also the particle identifier.
         * @param group (PointsGroup) is the group the particle belongs to
         * @param groupId (integer) is the group identifier in the PCS.
         * @param idxInGroup (integer) is the index of the particle in the current point group (ex: the 10th point of addPoints(30))
         * @param pcs defines the PCS it is associated to
         */
        constructor(particleIndex: number, group: PointsGroup, groupId: number, idxInGroup: number, pcs: PointsCloudSystem);
        /**
         * get point size
         */
        get size(): Vector3;
        /**
         * Set point size
         */
        set size(scale: Vector3);
        /**
         * Legacy support, changed quaternion to rotationQuaternion
         */
        get quaternion(): Nullable<Quaternion>;
        /**
         * Legacy support, changed quaternion to rotationQuaternion
         */
        set quaternion(q: Nullable<Quaternion>);
        /**
         * Returns a boolean. True if the particle intersects a mesh, else false
         * The intersection is computed on the particle position and Axis Aligned Bounding Box (AABB) or Sphere
         * @param target is the object (point or mesh) what the intersection is computed against
         * @param isSphere is boolean flag when false (default) bounding box of mesh is used, when true the bounding sphere is used
         * @returns true if it intersects
         */
        intersectsMesh(target: Mesh, isSphere: boolean): boolean;
        /**
         * get the rotation matrix of the particle
         * @internal
         */
        getRotationMatrix(m: Matrix): void;
    }
    /**
     * Represents a group of points in a points cloud system
     *  * PCS internal tool, don't use it manually.
     */
    export class PointsGroup {
        /**
         * Get or set the groupId
         * @deprecated Please use groupId instead
         */
        get groupID(): number;
        set groupID(groupID: number);
        /**
         * The group id
         * @internal
         */
        groupId: number;
        /**
         * image data for group (internal use)
         * @internal
         */
        _groupImageData: Nullable<ArrayBufferView>;
        /**
         * Image Width (internal use)
         * @internal
         */
        _groupImgWidth: number;
        /**
         * Image Height (internal use)
         * @internal
         */
        _groupImgHeight: number;
        /**
         * Custom position function (internal use)
         * @internal
         */
        _positionFunction: Nullable<(particle: CloudPoint, i?: number, s?: number) => void>;
        /**
         * density per facet for surface points
         * @internal
         */
        _groupDensity: number[];
        /**
         * Only when points are colored by texture carries pointer to texture list array
         * @internal
         */
        _textureNb: number;
        /**
         * Creates a points group object. This is an internal reference to produce particles for the PCS.
         * PCS internal tool, don't use it manually.
         * @internal
         */
        constructor(id: number, posFunction: Nullable<(particle: CloudPoint, i?: number, s?: number) => void>);
    }


    /** @internal */
    export class ComputeShaderParticleSystem implements IGPUParticleSystemPlatform {
        private _parent;
        private _engine;
        private _updateComputeShader;
        private _simParamsComputeShader;
        private _bufferComputeShader;
        private _renderVertexBuffers;
        readonly alignDataInBuffer = true;
        constructor(parent: GPUParticleSystem, engine: ThinEngine);
        isUpdateBufferCreated(): boolean;
        isUpdateBufferReady(): boolean;
        createUpdateBuffer(defines: string): UniformBufferEffectCommonAccessor;
        createVertexBuffers(updateBuffer: Buffer, renderVertexBuffers: {
            [key: string]: VertexBuffer;
        }): void;
        createParticleBuffer(data: number[]): DataArray | DataBuffer;
        bindDrawBuffers(index: number, effect: Effect): void;
        preUpdateParticleBuffer(): void;
        updateParticleBuffer(index: number, targetBuffer: Buffer, currentActiveCount: number): void;
        releaseBuffers(): void;
        releaseVertexBuffers(): void;
    }


    /**
     * Particle emitter emitting particles from the inside of a box.
     * It emits the particles randomly between 2 given directions.
     */
    export class BoxParticleEmitter implements IParticleEmitterType {
        /**
         * Random direction of each particle after it has been emitted, between direction1 and direction2 vectors.
         */
        direction1: Vector3;
        /**
         * Random direction of each particle after it has been emitted, between direction1 and direction2 vectors.
         */
        direction2: Vector3;
        /**
         * Minimum box point around our emitter. Our emitter is the center of particles source, but if you want your particles to emit from more than one point, then you can tell it to do so.
         */
        minEmitBox: Vector3;
        /**
         * Maximum box point around our emitter. Our emitter is the center of particles source, but if you want your particles to emit from more than one point, then you can tell it to do so.
         */
        maxEmitBox: Vector3;
        /**
         * Creates a new instance BoxParticleEmitter
         */
        constructor();
        /**
         * Called by the particle System when the direction is computed for the created particle.
         * @param worldMatrix is the world matrix of the particle system
         * @param directionToUpdate is the direction vector to update with the result
         * @param particle is the particle we are computed the direction for
         * @param isLocal defines if the direction should be set in local space
         */
        startDirectionFunction(worldMatrix: Matrix, directionToUpdate: Vector3, particle: Particle, isLocal: boolean): void;
        /**
         * Called by the particle System when the position is computed for the created particle.
         * @param worldMatrix is the world matrix of the particle system
         * @param positionToUpdate is the position vector to update with the result
         * @param particle is the particle we are computed the position for
         * @param isLocal defines if the position should be set in local space
         */
        startPositionFunction(worldMatrix: Matrix, positionToUpdate: Vector3, particle: Particle, isLocal: boolean): void;
        /**
         * Clones the current emitter and returns a copy of it
         * @returns the new emitter
         */
        clone(): BoxParticleEmitter;
        /**
         * Called by the GPUParticleSystem to setup the update shader
         * @param uboOrEffect defines the update shader
         */
        applyToShader(uboOrEffect: UniformBufferEffectCommonAccessor): void;
        /**
         * Creates the structure of the ubo for this particle emitter
         * @param ubo ubo to create the structure for
         */
        buildUniformLayout(ubo: UniformBuffer): void;
        /**
         * Returns a string to use to update the GPU particles update shader
         * @returns a string containing the defines string
         */
        getEffectDefines(): string;
        /**
         * Returns the string "BoxParticleEmitter"
         * @returns a string containing the class name
         */
        getClassName(): string;
        /**
         * Serializes the particle system to a JSON object.
         * @returns the JSON object
         */
        serialize(): any;
        /**
         * Parse properties from a JSON object
         * @param serializationObject defines the JSON object
         */
        parse(serializationObject: any): void;
    }


    /**
     * Particle emitter emitting particles from the inside of a cone.
     * It emits the particles alongside the cone volume from the base to the particle.
     * The emission direction might be randomized.
     */
    export class ConeParticleEmitter implements IParticleEmitterType {
        /** defines how much to randomize the particle direction [0-1] (default is 0) */
        directionRandomizer: number;
        private _radius;
        private _angle;
        private _height;
        /**
         * Gets or sets a value indicating where on the radius the start position should be picked (1 = everywhere, 0 = only surface)
         */
        radiusRange: number;
        /**
         * Gets or sets a value indicating where on the height the start position should be picked (1 = everywhere, 0 = only surface)
         */
        heightRange: number;
        /**
         * Gets or sets a value indicating if all the particles should be emitted from the spawn point only (the base of the cone)
         */
        emitFromSpawnPointOnly: boolean;
        /**
         * Gets or sets the radius of the emission cone
         */
        get radius(): number;
        set radius(value: number);
        /**
         * Gets or sets the angle of the emission cone
         */
        get angle(): number;
        set angle(value: number);
        private _buildHeight;
        /**
         * Creates a new instance ConeParticleEmitter
         * @param radius the radius of the emission cone (1 by default)
         * @param angle the cone base angle (PI by default)
         * @param directionRandomizer defines how much to randomize the particle direction [0-1] (default is 0)
         */
        constructor(radius?: number, angle?: number, 
        /** defines how much to randomize the particle direction [0-1] (default is 0) */
        directionRandomizer?: number);
        /**
         * Called by the particle System when the direction is computed for the created particle.
         * @param worldMatrix is the world matrix of the particle system
         * @param directionToUpdate is the direction vector to update with the result
         * @param particle is the particle we are computed the direction for
         * @param isLocal defines if the direction should be set in local space
         */
        startDirectionFunction(worldMatrix: Matrix, directionToUpdate: Vector3, particle: Particle, isLocal: boolean): void;
        /**
         * Called by the particle System when the position is computed for the created particle.
         * @param worldMatrix is the world matrix of the particle system
         * @param positionToUpdate is the position vector to update with the result
         * @param particle is the particle we are computed the position for
         * @param isLocal defines if the position should be set in local space
         */
        startPositionFunction(worldMatrix: Matrix, positionToUpdate: Vector3, particle: Particle, isLocal: boolean): void;
        /**
         * Clones the current emitter and returns a copy of it
         * @returns the new emitter
         */
        clone(): ConeParticleEmitter;
        /**
         * Called by the GPUParticleSystem to setup the update shader
         * @param uboOrEffect defines the update shader
         */
        applyToShader(uboOrEffect: UniformBufferEffectCommonAccessor): void;
        /**
         * Creates the structure of the ubo for this particle emitter
         * @param ubo ubo to create the structure for
         */
        buildUniformLayout(ubo: UniformBuffer): void;
        /**
         * Returns a string to use to update the GPU particles update shader
         * @returns a string containing the defines string
         */
        getEffectDefines(): string;
        /**
         * Returns the string "ConeParticleEmitter"
         * @returns a string containing the class name
         */
        getClassName(): string;
        /**
         * Serializes the particle system to a JSON object.
         * @returns the JSON object
         */
        serialize(): any;
        /**
         * Parse properties from a JSON object
         * @param serializationObject defines the JSON object
         */
        parse(serializationObject: any): void;
    }


    /**
     * Particle emitter emitting particles from a custom list of positions.
     */
    export class CustomParticleEmitter implements IParticleEmitterType {
        /**
         * Gets or sets the position generator that will create the initial position of each particle.
         * Index will be provided when used with GPU particle. Particle will be provided when used with CPU particles
         */
        particlePositionGenerator: (index: number, particle: Nullable<Particle>, outPosition: Vector3) => void;
        /**
         * Gets or sets the destination generator that will create the final destination of each particle.
         *  * Index will be provided when used with GPU particle. Particle will be provided when used with CPU particles
         */
        particleDestinationGenerator: (index: number, particle: Nullable<Particle>, outDestination: Vector3) => void;
        /**
         * Creates a new instance CustomParticleEmitter
         */
        constructor();
        /**
         * Called by the particle System when the direction is computed for the created particle.
         * @param worldMatrix is the world matrix of the particle system
         * @param directionToUpdate is the direction vector to update with the result
         * @param particle is the particle we are computed the direction for
         * @param isLocal defines if the direction should be set in local space
         */
        startDirectionFunction(worldMatrix: Matrix, directionToUpdate: Vector3, particle: Particle, isLocal: boolean): void;
        /**
         * Called by the particle System when the position is computed for the created particle.
         * @param worldMatrix is the world matrix of the particle system
         * @param positionToUpdate is the position vector to update with the result
         * @param particle is the particle we are computed the position for
         * @param isLocal defines if the position should be set in local space
         */
        startPositionFunction(worldMatrix: Matrix, positionToUpdate: Vector3, particle: Particle, isLocal: boolean): void;
        /**
         * Clones the current emitter and returns a copy of it
         * @returns the new emitter
         */
        clone(): CustomParticleEmitter;
        /**
         * Called by the GPUParticleSystem to setup the update shader
         * @param uboOrEffect defines the update shader
         */
        applyToShader(uboOrEffect: UniformBufferEffectCommonAccessor): void;
        /**
         * Creates the structure of the ubo for this particle emitter
         * @param ubo ubo to create the structure for
         */
        buildUniformLayout(ubo: UniformBuffer): void;
        /**
         * Returns a string to use to update the GPU particles update shader
         * @returns a string containing the defines string
         */
        getEffectDefines(): string;
        /**
         * Returns the string "PointParticleEmitter"
         * @returns a string containing the class name
         */
        getClassName(): string;
        /**
         * Serializes the particle system to a JSON object.
         * @returns the JSON object
         */
        serialize(): any;
        /**
         * Parse properties from a JSON object
         * @param serializationObject defines the JSON object
         */
        parse(serializationObject: any): void;
    }


    /**
     * Particle emitter emitting particles from the inside of a cylinder.
     * It emits the particles alongside the cylinder radius. The emission direction might be randomized.
     */
    export class CylinderParticleEmitter implements IParticleEmitterType {
        /**
         * The radius of the emission cylinder.
         */
        radius: number;
        /**
         * The height of the emission cylinder.
         */
        height: number;
        /**
         * The range of emission [0-1] 0 Surface only, 1 Entire Radius.
         */
        radiusRange: number;
        /**
         * How much to randomize the particle direction [0-1].
         */
        directionRandomizer: number;
        private _tempVector;
        /**
         * Creates a new instance CylinderParticleEmitter
         * @param radius the radius of the emission cylinder (1 by default)
         * @param height the height of the emission cylinder (1 by default)
         * @param radiusRange the range of the emission cylinder [0-1] 0 Surface only, 1 Entire Radius (1 by default)
         * @param directionRandomizer defines how much to randomize the particle direction [0-1]
         */
        constructor(
        /**
         * The radius of the emission cylinder.
         */
        radius?: number, 
        /**
         * The height of the emission cylinder.
         */
        height?: number, 
        /**
         * The range of emission [0-1] 0 Surface only, 1 Entire Radius.
         */
        radiusRange?: number, 
        /**
         * How much to randomize the particle direction [0-1].
         */
        directionRandomizer?: number);
        /**
         * Called by the particle System when the direction is computed for the created particle.
         * @param worldMatrix is the world matrix of the particle system
         * @param directionToUpdate is the direction vector to update with the result
         * @param particle is the particle we are computed the direction for
         * @param isLocal defines if the direction should be set in local space
         * @param inverseWorldMatrix defines the inverted world matrix to use if isLocal is false
         */
        startDirectionFunction(worldMatrix: Matrix, directionToUpdate: Vector3, particle: Particle, isLocal: boolean, inverseWorldMatrix: Matrix): void;
        /**
         * Called by the particle System when the position is computed for the created particle.
         * @param worldMatrix is the world matrix of the particle system
         * @param positionToUpdate is the position vector to update with the result
         * @param particle is the particle we are computed the position for
         * @param isLocal defines if the position should be set in local space
         */
        startPositionFunction(worldMatrix: Matrix, positionToUpdate: Vector3, particle: Particle, isLocal: boolean): void;
        /**
         * Clones the current emitter and returns a copy of it
         * @returns the new emitter
         */
        clone(): CylinderParticleEmitter;
        /**
         * Called by the GPUParticleSystem to setup the update shader
         * @param uboOrEffect defines the update shader
         */
        applyToShader(uboOrEffect: UniformBufferEffectCommonAccessor): void;
        /**
         * Creates the structure of the ubo for this particle emitter
         * @param ubo ubo to create the structure for
         */
        buildUniformLayout(ubo: UniformBuffer): void;
        /**
         * Returns a string to use to update the GPU particles update shader
         * @returns a string containing the defines string
         */
        getEffectDefines(): string;
        /**
         * Returns the string "CylinderParticleEmitter"
         * @returns a string containing the class name
         */
        getClassName(): string;
        /**
         * Serializes the particle system to a JSON object.
         * @returns the JSON object
         */
        serialize(): any;
        /**
         * Parse properties from a JSON object
         * @param serializationObject defines the JSON object
         */
        parse(serializationObject: any): void;
    }
    /**
     * Particle emitter emitting particles from the inside of a cylinder.
     * It emits the particles randomly between two vectors.
     */
    export class CylinderDirectedParticleEmitter extends CylinderParticleEmitter {
        /**
         * The min limit of the emission direction.
         */
        direction1: Vector3;
        /**
         * The max limit of the emission direction.
         */
        direction2: Vector3;
        /**
         * Creates a new instance CylinderDirectedParticleEmitter
         * @param radius the radius of the emission cylinder (1 by default)
         * @param height the height of the emission cylinder (1 by default)
         * @param radiusRange the range of the emission cylinder [0-1] 0 Surface only, 1 Entire Radius (1 by default)
         * @param direction1 the min limit of the emission direction (up vector by default)
         * @param direction2 the max limit of the emission direction (up vector by default)
         */
        constructor(radius?: number, height?: number, radiusRange?: number, 
        /**
         * The min limit of the emission direction.
         */
        direction1?: Vector3, 
        /**
         * The max limit of the emission direction.
         */
        direction2?: Vector3);
        /**
         * Called by the particle System when the direction is computed for the created particle.
         * @param worldMatrix is the world matrix of the particle system
         * @param directionToUpdate is the direction vector to update with the result
         */
        startDirectionFunction(worldMatrix: Matrix, directionToUpdate: Vector3): void;
        /**
         * Clones the current emitter and returns a copy of it
         * @returns the new emitter
         */
        clone(): CylinderDirectedParticleEmitter;
        /**
         * Called by the GPUParticleSystem to setup the update shader
         * @param uboOrEffect defines the update shader
         */
        applyToShader(uboOrEffect: UniformBufferEffectCommonAccessor): void;
        /**
         * Creates the structure of the ubo for this particle emitter
         * @param ubo ubo to create the structure for
         */
        buildUniformLayout(ubo: UniformBuffer): void;
        /**
         * Returns a string to use to update the GPU particles update shader
         * @returns a string containing the defines string
         */
        getEffectDefines(): string;
        /**
         * Returns the string "CylinderDirectedParticleEmitter"
         * @returns a string containing the class name
         */
        getClassName(): string;
        /**
         * Serializes the particle system to a JSON object.
         * @returns the JSON object
         */
        serialize(): any;
        /**
         * Parse properties from a JSON object
         * @param serializationObject defines the JSON object
         */
        parse(serializationObject: any): void;
    }


    /**
     * Particle emitter emitting particles from the inside of a hemisphere.
     * It emits the particles alongside the hemisphere radius. The emission direction might be randomized.
     */
    export class HemisphericParticleEmitter implements IParticleEmitterType {
        /**
         * The radius of the emission hemisphere.
         */
        radius: number;
        /**
         * The range of emission [0-1] 0 Surface only, 1 Entire Radius.
         */
        radiusRange: number;
        /**
         * How much to randomize the particle direction [0-1].
         */
        directionRandomizer: number;
        /**
         * Creates a new instance HemisphericParticleEmitter
         * @param radius the radius of the emission hemisphere (1 by default)
         * @param radiusRange the range of the emission hemisphere [0-1] 0 Surface only, 1 Entire Radius (1 by default)
         * @param directionRandomizer defines how much to randomize the particle direction [0-1]
         */
        constructor(
        /**
         * The radius of the emission hemisphere.
         */
        radius?: number, 
        /**
         * The range of emission [0-1] 0 Surface only, 1 Entire Radius.
         */
        radiusRange?: number, 
        /**
         * How much to randomize the particle direction [0-1].
         */
        directionRandomizer?: number);
        /**
         * Called by the particle System when the direction is computed for the created particle.
         * @param worldMatrix is the world matrix of the particle system
         * @param directionToUpdate is the direction vector to update with the result
         * @param particle is the particle we are computed the direction for
         * @param isLocal defines if the direction should be set in local space
         */
        startDirectionFunction(worldMatrix: Matrix, directionToUpdate: Vector3, particle: Particle, isLocal: boolean): void;
        /**
         * Called by the particle System when the position is computed for the created particle.
         * @param worldMatrix is the world matrix of the particle system
         * @param positionToUpdate is the position vector to update with the result
         * @param particle is the particle we are computed the position for
         * @param isLocal defines if the position should be set in local space
         */
        startPositionFunction(worldMatrix: Matrix, positionToUpdate: Vector3, particle: Particle, isLocal: boolean): void;
        /**
         * Clones the current emitter and returns a copy of it
         * @returns the new emitter
         */
        clone(): HemisphericParticleEmitter;
        /**
         * Called by the GPUParticleSystem to setup the update shader
         * @param uboOrEffect defines the update shader
         */
        applyToShader(uboOrEffect: UniformBufferEffectCommonAccessor): void;
        /**
         * Creates the structure of the ubo for this particle emitter
         * @param ubo ubo to create the structure for
         */
        buildUniformLayout(ubo: UniformBuffer): void;
        /**
         * Returns a string to use to update the GPU particles update shader
         * @returns a string containing the defines string
         */
        getEffectDefines(): string;
        /**
         * Returns the string "HemisphericParticleEmitter"
         * @returns a string containing the class name
         */
        getClassName(): string;
        /**
         * Serializes the particle system to a JSON object.
         * @returns the JSON object
         */
        serialize(): any;
        /**
         * Parse properties from a JSON object
         * @param serializationObject defines the JSON object
         */
        parse(serializationObject: any): void;
    }




    /**
     * Particle emitter represents a volume emitting particles.
     * This is the responsibility of the implementation to define the volume shape like cone/sphere/box.
     */
    export interface IParticleEmitterType {
        /**
         * Called by the particle System when the direction is computed for the created particle.
         * @param worldMatrix is the world matrix of the particle system
         * @param directionToUpdate is the direction vector to update with the result
         * @param particle is the particle we are computed the direction for
         * @param isLocal defines if the direction should be set in local space
         * @param inverseWorldMatrix defines the inverted world matrix to use if isLocal is false
         */
        startDirectionFunction(worldMatrix: Matrix, directionToUpdate: Vector3, particle: Particle, isLocal: boolean, inverseWorldMatrix: Matrix): void;
        /**
         * Called by the particle System when the position is computed for the created particle.
         * @param worldMatrix is the world matrix of the particle system
         * @param positionToUpdate is the position vector to update with the result
         * @param particle is the particle we are computed the position for
         * @param isLocal defines if the position should be set in local space
         */
        startPositionFunction(worldMatrix: Matrix, positionToUpdate: Vector3, particle: Particle, isLocal: boolean): void;
        /**
         * Clones the current emitter and returns a copy of it
         * @returns the new emitter
         */
        clone(): IParticleEmitterType;
        /**
         * Called by the GPUParticleSystem to setup the update shader
         * @param uboOrEffect defines the update shader
         */
        applyToShader(uboOrEffect: UniformBufferEffectCommonAccessor): void;
        /**
         * Creates the structure of the ubo for this particle emitter
         * @param ubo ubo to create the structure for
         */
        buildUniformLayout(ubo: UniformBuffer): void;
        /**
         * Returns a string to use to update the GPU particles update shader
         * @returns the effect defines string
         */
        getEffectDefines(): string;
        /**
         * Returns a string representing the class name
         * @returns a string containing the class name
         */
        getClassName(): string;
        /**
         * Serializes the particle system to a JSON object.
         * @returns the JSON object
         */
        serialize(): any;
        /**
         * Parse properties from a JSON object
         * @param serializationObject defines the JSON object
         * @param scene defines the hosting scene
         */
        parse(serializationObject: any, scene: Nullable<Scene>): void;
    }


    /**
     * Particle emitter emitting particles from the inside of a box.
     * It emits the particles randomly between 2 given directions.
     */
    export class MeshParticleEmitter implements IParticleEmitterType {
        private _indices;
        private _positions;
        private _normals;
        private _storedNormal;
        private _mesh;
        /**
         * Random direction of each particle after it has been emitted, between direction1 and direction2 vectors.
         */
        direction1: Vector3;
        /**
         * Random direction of each particle after it has been emitted, between direction1 and direction2 vectors.
         */
        direction2: Vector3;
        /**
         * Gets or sets a boolean indicating that particle directions must be built from mesh face normals
         */
        useMeshNormalsForDirection: boolean;
        /** Defines the mesh to use as source */
        get mesh(): Nullable<AbstractMesh>;
        set mesh(value: Nullable<AbstractMesh>);
        /**
         * Creates a new instance MeshParticleEmitter
         * @param mesh defines the mesh to use as source
         */
        constructor(mesh?: Nullable<AbstractMesh>);
        /**
         * Called by the particle System when the direction is computed for the created particle.
         * @param worldMatrix is the world matrix of the particle system
         * @param directionToUpdate is the direction vector to update with the result
         * @param particle is the particle we are computed the direction for
         * @param isLocal defines if the direction should be set in local space
         */
        startDirectionFunction(worldMatrix: Matrix, directionToUpdate: Vector3, particle: Particle, isLocal: boolean): void;
        /**
         * Called by the particle System when the position is computed for the created particle.
         * @param worldMatrix is the world matrix of the particle system
         * @param positionToUpdate is the position vector to update with the result
         * @param particle is the particle we are computed the position for
         * @param isLocal defines if the position should be set in local space
         */
        startPositionFunction(worldMatrix: Matrix, positionToUpdate: Vector3, particle: Particle, isLocal: boolean): void;
        /**
         * Clones the current emitter and returns a copy of it
         * @returns the new emitter
         */
        clone(): MeshParticleEmitter;
        /**
         * Called by the GPUParticleSystem to setup the update shader
         * @param uboOrEffect defines the update shader
         */
        applyToShader(uboOrEffect: UniformBufferEffectCommonAccessor): void;
        /**
         * Creates the structure of the ubo for this particle emitter
         * @param ubo ubo to create the structure for
         */
        buildUniformLayout(ubo: UniformBuffer): void;
        /**
         * Returns a string to use to update the GPU particles update shader
         * @returns a string containing the defines string
         */
        getEffectDefines(): string;
        /**
         * Returns the string "BoxParticleEmitter"
         * @returns a string containing the class name
         */
        getClassName(): string;
        /**
         * Serializes the particle system to a JSON object.
         * @returns the JSON object
         */
        serialize(): any;
        /**
         * Parse properties from a JSON object
         * @param serializationObject defines the JSON object
         * @param scene defines the hosting scene
         */
        parse(serializationObject: any, scene: Nullable<Scene>): void;
    }


    /**
     * Particle emitter emitting particles from a point.
     * It emits the particles randomly between 2 given directions.
     */
    export class PointParticleEmitter implements IParticleEmitterType {
        /**
         * Random direction of each particle after it has been emitted, between direction1 and direction2 vectors.
         */
        direction1: Vector3;
        /**
         * Random direction of each particle after it has been emitted, between direction1 and direction2 vectors.
         */
        direction2: Vector3;
        /**
         * Creates a new instance PointParticleEmitter
         */
        constructor();
        /**
         * Called by the particle System when the direction is computed for the created particle.
         * @param worldMatrix is the world matrix of the particle system
         * @param directionToUpdate is the direction vector to update with the result
         * @param particle is the particle we are computed the direction for
         * @param isLocal defines if the direction should be set in local space
         */
        startDirectionFunction(worldMatrix: Matrix, directionToUpdate: Vector3, particle: Particle, isLocal: boolean): void;
        /**
         * Called by the particle System when the position is computed for the created particle.
         * @param worldMatrix is the world matrix of the particle system
         * @param positionToUpdate is the position vector to update with the result
         * @param particle is the particle we are computed the position for
         * @param isLocal defines if the position should be set in local space
         */
        startPositionFunction(worldMatrix: Matrix, positionToUpdate: Vector3, particle: Particle, isLocal: boolean): void;
        /**
         * Clones the current emitter and returns a copy of it
         * @returns the new emitter
         */
        clone(): PointParticleEmitter;
        /**
         * Called by the GPUParticleSystem to setup the update shader
         * @param uboOrEffect defines the update shader
         */
        applyToShader(uboOrEffect: UniformBufferEffectCommonAccessor): void;
        /**
         * Creates the structure of the ubo for this particle emitter
         * @param ubo ubo to create the structure for
         */
        buildUniformLayout(ubo: UniformBuffer): void;
        /**
         * Returns a string to use to update the GPU particles update shader
         * @returns a string containing the defines string
         */
        getEffectDefines(): string;
        /**
         * Returns the string "PointParticleEmitter"
         * @returns a string containing the class name
         */
        getClassName(): string;
        /**
         * Serializes the particle system to a JSON object.
         * @returns the JSON object
         */
        serialize(): any;
        /**
         * Parse properties from a JSON object
         * @param serializationObject defines the JSON object
         */
        parse(serializationObject: any): void;
    }


    /**
     * Particle emitter emitting particles from the inside of a sphere.
     * It emits the particles alongside the sphere radius. The emission direction might be randomized.
     */
    export class SphereParticleEmitter implements IParticleEmitterType {
        /**
         * The radius of the emission sphere.
         */
        radius: number;
        /**
         * The range of emission [0-1] 0 Surface only, 1 Entire Radius.
         */
        radiusRange: number;
        /**
         * How much to randomize the particle direction [0-1].
         */
        directionRandomizer: number;
        /**
         * Creates a new instance SphereParticleEmitter
         * @param radius the radius of the emission sphere (1 by default)
         * @param radiusRange the range of the emission sphere [0-1] 0 Surface only, 1 Entire Radius (1 by default)
         * @param directionRandomizer defines how much to randomize the particle direction [0-1]
         */
        constructor(
        /**
         * The radius of the emission sphere.
         */
        radius?: number, 
        /**
         * The range of emission [0-1] 0 Surface only, 1 Entire Radius.
         */
        radiusRange?: number, 
        /**
         * How much to randomize the particle direction [0-1].
         */
        directionRandomizer?: number);
        /**
         * Called by the particle System when the direction is computed for the created particle.
         * @param worldMatrix is the world matrix of the particle system
         * @param directionToUpdate is the direction vector to update with the result
         * @param particle is the particle we are computed the direction for
         * @param isLocal defines if the direction should be set in local space
         */
        startDirectionFunction(worldMatrix: Matrix, directionToUpdate: Vector3, particle: Particle, isLocal: boolean): void;
        /**
         * Called by the particle System when the position is computed for the created particle.
         * @param worldMatrix is the world matrix of the particle system
         * @param positionToUpdate is the position vector to update with the result
         * @param particle is the particle we are computed the position for
         * @param isLocal defines if the position should be set in local space
         */
        startPositionFunction(worldMatrix: Matrix, positionToUpdate: Vector3, particle: Particle, isLocal: boolean): void;
        /**
         * Clones the current emitter and returns a copy of it
         * @returns the new emitter
         */
        clone(): SphereParticleEmitter;
        /**
         * Called by the GPUParticleSystem to setup the update shader
         * @param uboOrEffect defines the update shader
         */
        applyToShader(uboOrEffect: UniformBufferEffectCommonAccessor): void;
        /**
         * Creates the structure of the ubo for this particle emitter
         * @param ubo ubo to create the structure for
         */
        buildUniformLayout(ubo: UniformBuffer): void;
        /**
         * Returns a string to use to update the GPU particles update shader
         * @returns a string containing the defines string
         */
        getEffectDefines(): string;
        /**
         * Returns the string "SphereParticleEmitter"
         * @returns a string containing the class name
         */
        getClassName(): string;
        /**
         * Serializes the particle system to a JSON object.
         * @returns the JSON object
         */
        serialize(): any;
        /**
         * Parse properties from a JSON object
         * @param serializationObject defines the JSON object
         */
        parse(serializationObject: any): void;
    }
    /**
     * Particle emitter emitting particles from the inside of a sphere.
     * It emits the particles randomly between two vectors.
     */
    export class SphereDirectedParticleEmitter extends SphereParticleEmitter {
        /**
         * The min limit of the emission direction.
         */
        direction1: Vector3;
        /**
         * The max limit of the emission direction.
         */
        direction2: Vector3;
        /**
         * Creates a new instance SphereDirectedParticleEmitter
         * @param radius the radius of the emission sphere (1 by default)
         * @param direction1 the min limit of the emission direction (up vector by default)
         * @param direction2 the max limit of the emission direction (up vector by default)
         */
        constructor(radius?: number, 
        /**
         * The min limit of the emission direction.
         */
        direction1?: Vector3, 
        /**
         * The max limit of the emission direction.
         */
        direction2?: Vector3);
        /**
         * Called by the particle System when the direction is computed for the created particle.
         * @param worldMatrix is the world matrix of the particle system
         * @param directionToUpdate is the direction vector to update with the result
         */
        startDirectionFunction(worldMatrix: Matrix, directionToUpdate: Vector3): void;
        /**
         * Clones the current emitter and returns a copy of it
         * @returns the new emitter
         */
        clone(): SphereDirectedParticleEmitter;
        /**
         * Called by the GPUParticleSystem to setup the update shader
         * @param uboOrEffect defines the update shader
         */
        applyToShader(uboOrEffect: UniformBufferEffectCommonAccessor): void;
        /**
         * Creates the structure of the ubo for this particle emitter
         * @param ubo ubo to create the structure for
         */
        buildUniformLayout(ubo: UniformBuffer): void;
        /**
         * Returns a string to use to update the GPU particles update shader
         * @returns a string containing the defines string
         */
        getEffectDefines(): string;
        /**
         * Returns the string "SphereDirectedParticleEmitter"
         * @returns a string containing the class name
         */
        getClassName(): string;
        /**
         * Serializes the particle system to a JSON object.
         * @returns the JSON object
         */
        serialize(): any;
        /**
         * Parse properties from a JSON object
         * @param serializationObject defines the JSON object
         */
        parse(serializationObject: any): void;
    }


    /**
     * This represents a GPU particle system in Babylon
     * This is the fastest particle system in Babylon as it uses the GPU to update the individual particle data
     * @see https://www.babylonjs-playground.com/#PU4WYI#4
     */
    export class GPUParticleSystem extends BaseParticleSystem implements IDisposable, IParticleSystem, IAnimatable {
        /**
         * The layer mask we are rendering the particles through.
         */
        layerMask: number;
        private _capacity;
        private _activeCount;
        private _currentActiveCount;
        private _accumulatedCount;
        private _updateBuffer;
        private _buffer0;
        private _buffer1;
        private _spriteBuffer;
        private _renderVertexBuffers;
        private _targetIndex;
        private _sourceBuffer;
        private _targetBuffer;
        private _currentRenderId;
        private _currentRenderingCameraUniqueId;
        private _started;
        private _stopped;
        private _timeDelta;
        /** @internal */
        _randomTexture: RawTexture;
        /** @internal */
        _randomTexture2: RawTexture;
        /** Indicates that the update of particles is done in the animate function (and not in render). Default: false */
        updateInAnimate: boolean;
        private _attributesStrideSize;
        private _cachedUpdateDefines;
        private _randomTextureSize;
        private _actualFrame;
        private _drawWrappers;
        private _customWrappers;
        private readonly _rawTextureWidth;
        private _platform;
        /**
         * Gets a boolean indicating if the GPU particles can be rendered on current browser
         */
        static get IsSupported(): boolean;
        /**
         * An event triggered when the system is disposed.
         */
        onDisposeObservable: Observable<IParticleSystem>;
        /**
         * An event triggered when the system is stopped
         */
        onStoppedObservable: Observable<IParticleSystem>;
        /**
         * Gets the maximum number of particles active at the same time.
         * @returns The max number of active particles.
         */
        getCapacity(): number;
        /**
         * Forces the particle to write their depth information to the depth buffer. This can help preventing other draw calls
         * to override the particles.
         */
        forceDepthWrite: boolean;
        /**
         * Gets or set the number of active particles
         */
        get activeParticleCount(): number;
        set activeParticleCount(value: number);
        private _preWarmDone;
        /**
         * Specifies if the particles are updated in emitter local space or world space.
         */
        isLocal: boolean;
        /** Indicates that the particle system is GPU based */
        readonly isGPU = true;
        /** Gets or sets a matrix to use to compute projection */
        defaultProjectionMatrix: Matrix;
        /**
         * Is this system ready to be used/rendered
         * @returns true if the system is ready
         */
        isReady(): boolean;
        /**
         * Gets if the system has been started. (Note: this will still be true after stop is called)
         * @returns True if it has been started, otherwise false.
         */
        isStarted(): boolean;
        /**
         * Gets if the system has been stopped. (Note: rendering is still happening but the system is frozen)
         * @returns True if it has been stopped, otherwise false.
         */
        isStopped(): boolean;
        /**
         * Gets a boolean indicating that the system is stopping
         * @returns true if the system is currently stopping
         */
        isStopping(): boolean;
        /**
         * Gets the number of particles active at the same time.
         * @returns The number of active particles.
         */
        getActiveCount(): number;
        /**
         * Starts the particle system and begins to emit
         * @param delay defines the delay in milliseconds before starting the system (this.startDelay by default)
         */
        start(delay?: number): void;
        /**
         * Stops the particle system.
         */
        stop(): void;
        /**
         * Remove all active particles
         */
        reset(): void;
        /**
         * Returns the string "GPUParticleSystem"
         * @returns a string containing the class name
         */
        getClassName(): string;
        /**
         * Gets the custom effect used to render the particles
         * @param blendMode Blend mode for which the effect should be retrieved
         * @returns The effect
         */
        getCustomEffect(blendMode?: number): Nullable<Effect>;
        private _getCustomDrawWrapper;
        /**
         * Sets the custom effect used to render the particles
         * @param effect The effect to set
         * @param blendMode Blend mode for which the effect should be set
         */
        setCustomEffect(effect: Nullable<Effect>, blendMode?: number): void;
        /** @internal */
        protected _onBeforeDrawParticlesObservable: Nullable<Observable<Nullable<Effect>>>;
        /**
         * Observable that will be called just before the particles are drawn
         */
        get onBeforeDrawParticlesObservable(): Observable<Nullable<Effect>>;
        /**
         * Gets the name of the particle vertex shader
         */
        get vertexShaderName(): string;
        /**
         * Gets the vertex buffers used by the particle system
         * Should be called after render() has been called for the current frame so that the buffers returned are the ones that have been updated
         * in the current frame (there's a ping-pong between two sets of buffers - for a given frame, one set is used as the source and the other as the destination)
         */
        get vertexBuffers(): Immutable<{
            [key: string]: VertexBuffer;
        }>;
        /**
         * Gets the index buffer used by the particle system (null for GPU particle systems)
         */
        get indexBuffer(): Nullable<DataBuffer>;
        /** @internal */
        _colorGradientsTexture: RawTexture;
        protected _removeGradientAndTexture(gradient: number, gradients: Nullable<IValueGradient[]>, texture: RawTexture): BaseParticleSystem;
        /**
         * Adds a new color gradient
         * @param gradient defines the gradient to use (between 0 and 1)
         * @param color1 defines the color to affect to the specified gradient
         * @returns the current particle system
         */
        addColorGradient(gradient: number, color1: Color4): GPUParticleSystem;
        private _refreshColorGradient;
        /** Force the system to rebuild all gradients that need to be resync */
        forceRefreshGradients(): void;
        /**
         * Remove a specific color gradient
         * @param gradient defines the gradient to remove
         * @returns the current particle system
         */
        removeColorGradient(gradient: number): GPUParticleSystem;
        /**
         * Resets the draw wrappers cache
         */
        resetDrawCache(): void;
        /** @internal */
        _angularSpeedGradientsTexture: RawTexture;
        /** @internal */
        _sizeGradientsTexture: RawTexture;
        /** @internal */
        _velocityGradientsTexture: RawTexture;
        /** @internal */
        _limitVelocityGradientsTexture: RawTexture;
        /** @internal */
        _dragGradientsTexture: RawTexture;
        private _addFactorGradient;
        /**
         * Adds a new size gradient
         * @param gradient defines the gradient to use (between 0 and 1)
         * @param factor defines the size factor to affect to the specified gradient
         * @returns the current particle system
         */
        addSizeGradient(gradient: number, factor: number): GPUParticleSystem;
        /**
         * Remove a specific size gradient
         * @param gradient defines the gradient to remove
         * @returns the current particle system
         */
        removeSizeGradient(gradient: number): GPUParticleSystem;
        private _refreshFactorGradient;
        /**
         * Adds a new angular speed gradient
         * @param gradient defines the gradient to use (between 0 and 1)
         * @param factor defines the angular speed to affect to the specified gradient
         * @returns the current particle system
         */
        addAngularSpeedGradient(gradient: number, factor: number): GPUParticleSystem;
        /**
         * Remove a specific angular speed gradient
         * @param gradient defines the gradient to remove
         * @returns the current particle system
         */
        removeAngularSpeedGradient(gradient: number): GPUParticleSystem;
        /**
         * Adds a new velocity gradient
         * @param gradient defines the gradient to use (between 0 and 1)
         * @param factor defines the velocity to affect to the specified gradient
         * @returns the current particle system
         */
        addVelocityGradient(gradient: number, factor: number): GPUParticleSystem;
        /**
         * Remove a specific velocity gradient
         * @param gradient defines the gradient to remove
         * @returns the current particle system
         */
        removeVelocityGradient(gradient: number): GPUParticleSystem;
        /**
         * Adds a new limit velocity gradient
         * @param gradient defines the gradient to use (between 0 and 1)
         * @param factor defines the limit velocity value to affect to the specified gradient
         * @returns the current particle system
         */
        addLimitVelocityGradient(gradient: number, factor: number): GPUParticleSystem;
        /**
         * Remove a specific limit velocity gradient
         * @param gradient defines the gradient to remove
         * @returns the current particle system
         */
        removeLimitVelocityGradient(gradient: number): GPUParticleSystem;
        /**
         * Adds a new drag gradient
         * @param gradient defines the gradient to use (between 0 and 1)
         * @param factor defines the drag value to affect to the specified gradient
         * @returns the current particle system
         */
        addDragGradient(gradient: number, factor: number): GPUParticleSystem;
        /**
         * Remove a specific drag gradient
         * @param gradient defines the gradient to remove
         * @returns the current particle system
         */
        removeDragGradient(gradient: number): GPUParticleSystem;
        /**
         * Not supported by GPUParticleSystem
         * @returns the current particle system
         */
        addEmitRateGradient(): IParticleSystem;
        /**
         * Not supported by GPUParticleSystem
         * @returns the current particle system
         */
        removeEmitRateGradient(): IParticleSystem;
        /**
         * Not supported by GPUParticleSystem
         * @returns the current particle system
         */
        addStartSizeGradient(): IParticleSystem;
        /**
         * Not supported by GPUParticleSystem
         * @returns the current particle system
         */
        removeStartSizeGradient(): IParticleSystem;
        /**
         * Not supported by GPUParticleSystem
         * @returns the current particle system
         */
        addColorRemapGradient(): IParticleSystem;
        /**
         * Not supported by GPUParticleSystem
         * @returns the current particle system
         */
        removeColorRemapGradient(): IParticleSystem;
        /**
         * Not supported by GPUParticleSystem
         * @returns the current particle system
         */
        addAlphaRemapGradient(): IParticleSystem;
        /**
         * Not supported by GPUParticleSystem
         * @returns the current particle system
         */
        removeAlphaRemapGradient(): IParticleSystem;
        /**
         * Not supported by GPUParticleSystem
         * @returns the current particle system
         */
        addRampGradient(): IParticleSystem;
        /**
         * Not supported by GPUParticleSystem
         * @returns the current particle system
         */
        removeRampGradient(): IParticleSystem;
        /**
         * Not supported by GPUParticleSystem
         * @returns the list of ramp gradients
         */
        getRampGradients(): Nullable<Array<Color3Gradient>>;
        /**
         * Not supported by GPUParticleSystem
         * Gets or sets a boolean indicating that ramp gradients must be used
         * @see https://doc.babylonjs.com/features/featuresDeepDive/particles/particle_system/particle_system_intro#ramp-gradients
         */
        get useRampGradients(): boolean;
        set useRampGradients(value: boolean);
        /**
         * Not supported by GPUParticleSystem
         * @returns the current particle system
         */
        addLifeTimeGradient(): IParticleSystem;
        /**
         * Not supported by GPUParticleSystem
         * @returns the current particle system
         */
        removeLifeTimeGradient(): IParticleSystem;
        /**
         * Instantiates a GPU particle system.
         * Particles are often small sprites used to simulate hard-to-reproduce phenomena like fire, smoke, water, or abstract visual effects like magic glitter and faery dust.
         * @param name The name of the particle system
         * @param options The options used to create the system
         * @param sceneOrEngine The scene the particle system belongs to or the engine to use if no scene
         * @param customEffect a custom effect used to change the way particles are rendered by default
         * @param isAnimationSheetEnabled Must be true if using a spritesheet to animate the particles texture
         */
        constructor(name: string, options: Partial<{
            capacity: number;
            randomTextureSize: number;
        }>, sceneOrEngine: Scene | ThinEngine, customEffect?: Nullable<Effect>, isAnimationSheetEnabled?: boolean);
        protected _reset(): void;
        private _createVertexBuffers;
        private _initialize;
        /** @internal */
        _recreateUpdateEffect(): boolean;
        /**
         * @internal
         */
        _getWrapper(blendMode: number): DrawWrapper;
        /**
         * @internal
         */
        static _GetAttributeNamesOrOptions(hasColorGradients?: boolean, isAnimationSheetEnabled?: boolean, isBillboardBased?: boolean, isBillboardStretched?: boolean): string[];
        /**
         * @internal
         */
        static _GetEffectCreationOptions(isAnimationSheetEnabled?: boolean, useLogarithmicDepth?: boolean): string[];
        /**
         * Fill the defines array according to the current settings of the particle system
         * @param defines Array to be updated
         * @param blendMode blend mode to take into account when updating the array
         */
        fillDefines(defines: Array<string>, blendMode?: number): void;
        /**
         * Fill the uniforms, attributes and samplers arrays according to the current settings of the particle system
         * @param uniforms Uniforms array to fill
         * @param attributes Attributes array to fill
         * @param samplers Samplers array to fill
         */
        fillUniformsAttributesAndSamplerNames(uniforms: Array<string>, attributes: Array<string>, samplers: Array<string>): void;
        /**
         * Animates the particle system for the current frame by emitting new particles and or animating the living ones.
         * @param preWarm defines if we are in the pre-warmimg phase
         */
        animate(preWarm?: boolean): void;
        private _createFactorGradientTexture;
        private _createSizeGradientTexture;
        private _createAngularSpeedGradientTexture;
        private _createVelocityGradientTexture;
        private _createLimitVelocityGradientTexture;
        private _createDragGradientTexture;
        private _createColorGradientTexture;
        private _render;
        /** @internal */
        _update(emitterWM?: Matrix): void;
        /**
         * Renders the particle system in its current state
         * @param preWarm defines if the system should only update the particles but not render them
         * @param forceUpdateOnly if true, force to only update the particles and never display them (meaning, even if preWarm=false, when forceUpdateOnly=true the particles won't be displayed)
         * @returns the current number of particles
         */
        render(preWarm?: boolean, forceUpdateOnly?: boolean): number;
        /**
         * Rebuilds the particle system
         */
        rebuild(): void;
        private _releaseBuffers;
        /**
         * Disposes the particle system and free the associated resources
         * @param disposeTexture defines if the particule texture must be disposed as well (true by default)
         */
        dispose(disposeTexture?: boolean): void;
        /**
         * Clones the particle system.
         * @param name The name of the cloned object
         * @param newEmitter The new emitter to use
         * @param cloneTexture Also clone the textures if true
         * @returns the cloned particle system
         */
        clone(name: string, newEmitter: any, cloneTexture?: boolean): GPUParticleSystem;
        /**
         * Serializes the particle system to a JSON object
         * @param serializeTexture defines if the texture must be serialized as well
         * @returns the JSON object
         */
        serialize(serializeTexture?: boolean): any;
        /**
         * Parses a JSON object to create a GPU particle system.
         * @param parsedParticleSystem The JSON object to parse
         * @param sceneOrEngine The scene or the engine to create the particle system in
         * @param rootUrl The root url to use to load external dependencies like texture
         * @param doNotStart Ignore the preventAutoStart attribute and does not start
         * @param capacity defines the system capacity (if null or undefined the sotred capacity will be used)
         * @returns the parsed GPU particle system
         */
        static Parse(parsedParticleSystem: any, sceneOrEngine: Scene | ThinEngine, rootUrl: string, doNotStart?: boolean, capacity?: number): GPUParticleSystem;
    }


    /** @internal */
    export interface IGPUParticleSystemPlatform {
        alignDataInBuffer: boolean;
        isUpdateBufferCreated: () => boolean;
        isUpdateBufferReady: () => boolean;
        createUpdateBuffer: (defines: string) => UniformBufferEffectCommonAccessor;
        createVertexBuffers: (updateBuffer: Buffer, renderVertexBuffers: {
            [key: string]: VertexBuffer;
        }) => void;
        createParticleBuffer: (data: number[]) => DataArray | DataBuffer;
        bindDrawBuffers: (index: number, effect: Effect) => void;
        preUpdateParticleBuffer: () => void;
        updateParticleBuffer: (index: number, targetBuffer: Buffer, currentActiveCount: number) => void;
        releaseBuffers: () => void;
        releaseVertexBuffers: () => void;
    }




    /**
     * Interface representing a particle system in Babylon.js.
     * This groups the common functionalities that needs to be implemented in order to create a particle system.
     * A particle system represents a way to manage particles from their emission to their animation and rendering.
     */
    export interface IParticleSystem {
        /**
         * List of animations used by the particle system.
         */
        animations: Animation[];
        /**
         * The id of the Particle system.
         */
        id: string;
        /**
         * The name of the Particle system.
         */
        name: string;
        /**
         * The emitter represents the Mesh or position we are attaching the particle system to.
         */
        emitter: Nullable<AbstractMesh | Vector3>;
        /**
         * Gets or sets a boolean indicating if the particles must be rendered as billboard or aligned with the direction
         */
        isBillboardBased: boolean;
        /**
         * The rendering group used by the Particle system to chose when to render.
         */
        renderingGroupId: number;
        /**
         * The layer mask we are rendering the particles through.
         */
        layerMask: number;
        /**
         * The overall motion speed (0.01 is default update speed, faster updates = faster animation)
         */
        updateSpeed: number;
        /**
         * The amount of time the particle system is running (depends of the overall update speed).
         */
        targetStopDuration: number;
        /**
         * The texture used to render each particle. (this can be a spritesheet)
         */
        particleTexture: Nullable<BaseTexture>;
        /**
         * Blend mode use to render the particle, it can be either ParticleSystem.BLENDMODE_ONEONE, ParticleSystem.BLENDMODE_STANDARD or ParticleSystem.BLENDMODE_ADD.
         */
        blendMode: number;
        /**
         * Minimum life time of emitting particles.
         */
        minLifeTime: number;
        /**
         * Maximum life time of emitting particles.
         */
        maxLifeTime: number;
        /**
         * Minimum Size of emitting particles.
         */
        minSize: number;
        /**
         * Maximum Size of emitting particles.
         */
        maxSize: number;
        /**
         * Minimum scale of emitting particles on X axis.
         */
        minScaleX: number;
        /**
         * Maximum scale of emitting particles on X axis.
         */
        maxScaleX: number;
        /**
         * Minimum scale of emitting particles on Y axis.
         */
        minScaleY: number;
        /**
         * Maximum scale of emitting particles on Y axis.
         */
        maxScaleY: number;
        /**
         * Random color of each particle after it has been emitted, between color1 and color2 vectors.
         */
        color1: Color4;
        /**
         * Random color of each particle after it has been emitted, between color1 and color2 vectors.
         */
        color2: Color4;
        /**
         * Color the particle will have at the end of its lifetime.
         */
        colorDead: Color4;
        /**
         * The maximum number of particles to emit per frame until we reach the activeParticleCount value
         */
        emitRate: number;
        /**
         * You can use gravity if you want to give an orientation to your particles.
         */
        gravity: Vector3;
        /**
         * Minimum power of emitting particles.
         */
        minEmitPower: number;
        /**
         * Maximum power of emitting particles.
         */
        maxEmitPower: number;
        /**
         * Minimum angular speed of emitting particles (Z-axis rotation for each particle).
         */
        minAngularSpeed: number;
        /**
         * Maximum angular speed of emitting particles (Z-axis rotation for each particle).
         */
        maxAngularSpeed: number;
        /**
         * Gets or sets the minimal initial rotation in radians.
         */
        minInitialRotation: number;
        /**
         * Gets or sets the maximal initial rotation in radians.
         */
        maxInitialRotation: number;
        /**
         * The particle emitter type defines the emitter used by the particle system.
         * It can be for example box, sphere, or cone...
         */
        particleEmitterType: Nullable<IParticleEmitterType>;
        /**
         * Defines the delay in milliseconds before starting the system (0 by default)
         */
        startDelay: number;
        /**
         * Gets or sets a value indicating how many cycles (or frames) must be executed before first rendering (this value has to be set before starting the system). Default is 0
         */
        preWarmCycles: number;
        /**
         * Gets or sets a value indicating the time step multiplier to use in pre-warm mode (default is 1)
         */
        preWarmStepOffset: number;
        /**
         * If using a spritesheet (isAnimationSheetEnabled) defines the speed of the sprite loop (default is 1 meaning the animation will play once during the entire particle lifetime)
         */
        spriteCellChangeSpeed: number;
        /**
         * If using a spritesheet (isAnimationSheetEnabled) defines the first sprite cell to display
         */
        startSpriteCellID: number;
        /**
         * If using a spritesheet (isAnimationSheetEnabled) defines the last sprite cell to display
         */
        endSpriteCellID: number;
        /**
         * If using a spritesheet (isAnimationSheetEnabled), defines whether the sprite animation is looping
         */
        spriteCellLoop: boolean;
        /**
         * If using a spritesheet (isAnimationSheetEnabled), defines the sprite cell width to use
         */
        spriteCellWidth: number;
        /**
         * If using a spritesheet (isAnimationSheetEnabled), defines the sprite cell height to use
         */
        spriteCellHeight: number;
        /**
         * This allows the system to random pick the start cell ID between startSpriteCellID and endSpriteCellID
         */
        spriteRandomStartCell: boolean;
        /**
         * Gets or sets a boolean indicating if a spritesheet is used to animate the particles texture
         */
        isAnimationSheetEnabled: boolean;
        /** Gets or sets a Vector2 used to move the pivot (by default (0,0)) */
        translationPivot: Vector2;
        /**
         * Gets or sets a texture used to add random noise to particle positions
         */
        noiseTexture: Nullable<BaseTexture>;
        /** Gets or sets the strength to apply to the noise value (default is (10, 10, 10)) */
        noiseStrength: Vector3;
        /**
         * Gets or sets the billboard mode to use when isBillboardBased = true.
         * Value can be: ParticleSystem.BILLBOARDMODE_ALL, ParticleSystem.BILLBOARDMODE_Y, ParticleSystem.BILLBOARDMODE_STRETCHED
         */
        billboardMode: number;
        /**
         * Gets or sets a boolean enabling the use of logarithmic depth buffers, which is good for wide depth buffers.
         */
        useLogarithmicDepth: boolean;
        /** Gets or sets a value indicating the damping to apply if the limit velocity factor is reached */
        limitVelocityDamping: number;
        /**
         * Gets or sets a boolean indicating that hosted animations (in the system.animations array) must be started when system.start() is called
         */
        beginAnimationOnStart: boolean;
        /**
         * Gets or sets the frame to start the animation from when beginAnimationOnStart is true
         */
        beginAnimationFrom: number;
        /**
         * Gets or sets the frame to end the animation on when beginAnimationOnStart is true
         */
        beginAnimationTo: number;
        /**
         * Gets or sets a boolean indicating if animations must loop when beginAnimationOnStart is true
         */
        beginAnimationLoop: boolean;
        /**
         * Specifies whether the particle system will be disposed once it reaches the end of the animation.
         */
        disposeOnStop: boolean;
        /**
         * If you want to launch only a few particles at once, that can be done, as well.
         */
        manualEmitCount: number;
        /**
         * Specifies if the particles are updated in emitter local space or world space
         */
        isLocal: boolean;
        /** Snippet ID if the particle system was created from the snippet server */
        snippetId: string;
        /** Gets or sets a matrix to use to compute projection */
        defaultProjectionMatrix: Matrix;
        /** Indicates that the update of particles is done in the animate function (and not in render) */
        updateInAnimate: boolean;
        /** @internal */
        _wasDispatched: boolean;
        /**
         * Gets the maximum number of particles active at the same time.
         * @returns The max number of active particles.
         */
        getCapacity(): number;
        /**
         * Gets the number of particles active at the same time.
         * @returns The number of active particles.
         */
        getActiveCount(): number;
        /**
         * Gets if the system has been started. (Note: this will still be true after stop is called)
         * @returns True if it has been started, otherwise false.
         */
        isStarted(): boolean;
        /**
         * Animates the particle system for this frame.
         */
        animate(): void;
        /**
         * Renders the particle system in its current state.
         * @returns the current number of particles
         */
        render(): number;
        /**
         * Dispose the particle system and frees its associated resources.
         * @param disposeTexture defines if the particle texture must be disposed as well (true by default)
         */
        dispose(disposeTexture?: boolean): void;
        /**
         * An event triggered when the system is disposed
         */
        onDisposeObservable: Observable<IParticleSystem>;
        /**
         * An event triggered when the system is stopped
         */
        onStoppedObservable: Observable<IParticleSystem>;
        /**
         * Clones the particle system.
         * @param name The name of the cloned object
         * @param newEmitter The new emitter to use
         * @returns the cloned particle system
         */
        clone(name: string, newEmitter: any): Nullable<IParticleSystem>;
        /**
         * Serializes the particle system to a JSON object
         * @param serializeTexture defines if the texture must be serialized as well
         * @returns the JSON object
         */
        serialize(serializeTexture: boolean): any;
        /**
         * Rebuild the particle system
         */
        rebuild(): void;
        /** Force the system to rebuild all gradients that need to be resync */
        forceRefreshGradients(): void;
        /**
         * Starts the particle system and begins to emit
         * @param delay defines the delay in milliseconds before starting the system (0 by default)
         */
        start(delay?: number): void;
        /**
         * Stops the particle system.
         */
        stop(): void;
        /**
         * Remove all active particles
         */
        reset(): void;
        /**
         * Gets a boolean indicating that the system is stopping
         * @returns true if the system is currently stopping
         */
        isStopping(): boolean;
        /**
         * Is this system ready to be used/rendered
         * @returns true if the system is ready
         */
        isReady(): boolean;
        /**
         * Returns the string "ParticleSystem"
         * @returns a string containing the class name
         */
        getClassName(): string;
        /**
         * Gets the custom effect used to render the particles
         * @param blendMode Blend mode for which the effect should be retrieved
         * @returns The effect
         */
        getCustomEffect(blendMode: number): Nullable<Effect>;
        /**
         * Sets the custom effect used to render the particles
         * @param effect The effect to set
         * @param blendMode Blend mode for which the effect should be set
         */
        setCustomEffect(effect: Nullable<Effect>, blendMode: number): void;
        /**
         * Fill the defines array according to the current settings of the particle system
         * @param defines Array to be updated
         * @param blendMode blend mode to take into account when updating the array
         */
        fillDefines(defines: Array<string>, blendMode: number): void;
        /**
         * Fill the uniforms, attributes and samplers arrays according to the current settings of the particle system
         * @param uniforms Uniforms array to fill
         * @param attributes Attributes array to fill
         * @param samplers Samplers array to fill
         */
        fillUniformsAttributesAndSamplerNames(uniforms: Array<string>, attributes: Array<string>, samplers: Array<string>): void;
        /**
         * Observable that will be called just before the particles are drawn
         */
        onBeforeDrawParticlesObservable: Observable<Nullable<Effect>>;
        /**
         * Gets the name of the particle vertex shader
         */
        vertexShaderName: string;
        /**
         * Gets the vertex buffers used by the particle system
         */
        vertexBuffers: Immutable<{
            [key: string]: VertexBuffer;
        }>;
        /**
         * Gets the index buffer used by the particle system (or null if no index buffer is used)
         */
        indexBuffer: Nullable<DataBuffer>;
        /**
         * Adds a new color gradient
         * @param gradient defines the gradient to use (between 0 and 1)
         * @param color1 defines the color to affect to the specified gradient
         * @param color2 defines an additional color used to define a range ([color, color2]) with main color to pick the final color from
         * @returns the current particle system
         */
        addColorGradient(gradient: number, color1: Color4, color2?: Color4): IParticleSystem;
        /**
         * Remove a specific color gradient
         * @param gradient defines the gradient to remove
         * @returns the current particle system
         */
        removeColorGradient(gradient: number): IParticleSystem;
        /**
         * Adds a new size gradient
         * @param gradient defines the gradient to use (between 0 and 1)
         * @param factor defines the size factor to affect to the specified gradient
         * @param factor2 defines an additional factor used to define a range ([factor, factor2]) with main value to pick the final value from
         * @returns the current particle system
         */
        addSizeGradient(gradient: number, factor: number, factor2?: number): IParticleSystem;
        /**
         * Remove a specific size gradient
         * @param gradient defines the gradient to remove
         * @returns the current particle system
         */
        removeSizeGradient(gradient: number): IParticleSystem;
        /**
         * Gets the current list of color gradients.
         * You must use addColorGradient and removeColorGradient to update this list
         * @returns the list of color gradients
         */
        getColorGradients(): Nullable<Array<ColorGradient>>;
        /**
         * Gets the current list of size gradients.
         * You must use addSizeGradient and removeSizeGradient to update this list
         * @returns the list of size gradients
         */
        getSizeGradients(): Nullable<Array<FactorGradient>>;
        /**
         * Gets the current list of angular speed gradients.
         * You must use addAngularSpeedGradient and removeAngularSpeedGradient to update this list
         * @returns the list of angular speed gradients
         */
        getAngularSpeedGradients(): Nullable<Array<FactorGradient>>;
        /**
         * Adds a new angular speed gradient
         * @param gradient defines the gradient to use (between 0 and 1)
         * @param factor defines the angular speed to affect to the specified gradient
         * @param factor2 defines an additional factor used to define a range ([factor, factor2]) with main value to pick the final value from
         * @returns the current particle system
         */
        addAngularSpeedGradient(gradient: number, factor: number, factor2?: number): IParticleSystem;
        /**
         * Remove a specific angular speed gradient
         * @param gradient defines the gradient to remove
         * @returns the current particle system
         */
        removeAngularSpeedGradient(gradient: number): IParticleSystem;
        /**
         * Gets the current list of velocity gradients.
         * You must use addVelocityGradient and removeVelocityGradient to update this list
         * @returns the list of velocity gradients
         */
        getVelocityGradients(): Nullable<Array<FactorGradient>>;
        /**
         * Adds a new velocity gradient
         * @param gradient defines the gradient to use (between 0 and 1)
         * @param factor defines the velocity to affect to the specified gradient
         * @param factor2 defines an additional factor used to define a range ([factor, factor2]) with main value to pick the final value from
         * @returns the current particle system
         */
        addVelocityGradient(gradient: number, factor: number, factor2?: number): IParticleSystem;
        /**
         * Remove a specific velocity gradient
         * @param gradient defines the gradient to remove
         * @returns the current particle system
         */
        removeVelocityGradient(gradient: number): IParticleSystem;
        /**
         * Gets the current list of limit velocity gradients.
         * You must use addLimitVelocityGradient and removeLimitVelocityGradient to update this list
         * @returns the list of limit velocity gradients
         */
        getLimitVelocityGradients(): Nullable<Array<FactorGradient>>;
        /**
         * Adds a new limit velocity gradient
         * @param gradient defines the gradient to use (between 0 and 1)
         * @param factor defines the limit velocity to affect to the specified gradient
         * @param factor2 defines an additional factor used to define a range ([factor, factor2]) with main value to pick the final value from
         * @returns the current particle system
         */
        addLimitVelocityGradient(gradient: number, factor: number, factor2?: number): IParticleSystem;
        /**
         * Remove a specific limit velocity gradient
         * @param gradient defines the gradient to remove
         * @returns the current particle system
         */
        removeLimitVelocityGradient(gradient: number): IParticleSystem;
        /**
         * Adds a new drag gradient
         * @param gradient defines the gradient to use (between 0 and 1)
         * @param factor defines the drag to affect to the specified gradient
         * @param factor2 defines an additional factor used to define a range ([factor, factor2]) with main value to pick the final value from
         * @returns the current particle system
         */
        addDragGradient(gradient: number, factor: number, factor2?: number): IParticleSystem;
        /**
         * Remove a specific drag gradient
         * @param gradient defines the gradient to remove
         * @returns the current particle system
         */
        removeDragGradient(gradient: number): IParticleSystem;
        /**
         * Gets the current list of drag gradients.
         * You must use addDragGradient and removeDragGradient to update this list
         * @returns the list of drag gradients
         */
        getDragGradients(): Nullable<Array<FactorGradient>>;
        /**
         * Adds a new emit rate gradient (please note that this will only work if you set the targetStopDuration property)
         * @param gradient defines the gradient to use (between 0 and 1)
         * @param factor defines the emit rate to affect to the specified gradient
         * @param factor2 defines an additional factor used to define a range ([factor, factor2]) with main value to pick the final value from
         * @returns the current particle system
         */
        addEmitRateGradient(gradient: number, factor: number, factor2?: number): IParticleSystem;
        /**
         * Remove a specific emit rate gradient
         * @param gradient defines the gradient to remove
         * @returns the current particle system
         */
        removeEmitRateGradient(gradient: number): IParticleSystem;
        /**
         * Gets the current list of emit rate gradients.
         * You must use addEmitRateGradient and removeEmitRateGradient to update this list
         * @returns the list of emit rate gradients
         */
        getEmitRateGradients(): Nullable<Array<FactorGradient>>;
        /**
         * Adds a new start size gradient (please note that this will only work if you set the targetStopDuration property)
         * @param gradient defines the gradient to use (between 0 and 1)
         * @param factor defines the start size to affect to the specified gradient
         * @param factor2 defines an additional factor used to define a range ([factor, factor2]) with main value to pick the final value from
         * @returns the current particle system
         */
        addStartSizeGradient(gradient: number, factor: number, factor2?: number): IParticleSystem;
        /**
         * Remove a specific start size gradient
         * @param gradient defines the gradient to remove
         * @returns the current particle system
         */
        removeStartSizeGradient(gradient: number): IParticleSystem;
        /**
         * Gets the current list of start size gradients.
         * You must use addStartSizeGradient and removeStartSizeGradient to update this list
         * @returns the list of start size gradients
         */
        getStartSizeGradients(): Nullable<Array<FactorGradient>>;
        /**
         * Adds a new life time gradient
         * @param gradient defines the gradient to use (between 0 and 1)
         * @param factor defines the life time factor to affect to the specified gradient
         * @param factor2 defines an additional factor used to define a range ([factor, factor2]) with main value to pick the final value from
         * @returns the current particle system
         */
        addLifeTimeGradient(gradient: number, factor: number, factor2?: number): IParticleSystem;
        /**
         * Remove a specific life time gradient
         * @param gradient defines the gradient to remove
         * @returns the current particle system
         */
        removeLifeTimeGradient(gradient: number): IParticleSystem;
        /**
         * Gets the current list of life time gradients.
         * You must use addLifeTimeGradient and removeLifeTimeGradient to update this list
         * @returns the list of life time gradients
         */
        getLifeTimeGradients(): Nullable<Array<FactorGradient>>;
        /**
         * Gets the current list of color gradients.
         * You must use addColorGradient and removeColorGradient to update this list
         * @returns the list of color gradients
         */
        getColorGradients(): Nullable<Array<ColorGradient>>;
        /**
         * Adds a new ramp gradient used to remap particle colors
         * @param gradient defines the gradient to use (between 0 and 1)
         * @param color defines the color to affect to the specified gradient
         * @returns the current particle system
         */
        addRampGradient(gradient: number, color: Color3): IParticleSystem;
        /**
         * Gets the current list of ramp gradients.
         * You must use addRampGradient and removeRampGradient to update this list
         * @returns the list of ramp gradients
         */
        getRampGradients(): Nullable<Array<Color3Gradient>>;
        /** Gets or sets a boolean indicating that ramp gradients must be used
         * @see https://doc.babylonjs.com/features/featuresDeepDive/particles/particle_system/ramps_and_blends
         */
        useRampGradients: boolean;
        /**
         * Adds a new color remap gradient
         * @param gradient defines the gradient to use (between 0 and 1)
         * @param min defines the color remap minimal range
         * @param max defines the color remap maximal range
         * @returns the current particle system
         */
        addColorRemapGradient(gradient: number, min: number, max: number): IParticleSystem;
        /**
         * Gets the current list of color remap gradients.
         * You must use addColorRemapGradient and removeColorRemapGradient to update this list
         * @returns the list of color remap gradients
         */
        getColorRemapGradients(): Nullable<Array<FactorGradient>>;
        /**
         * Adds a new alpha remap gradient
         * @param gradient defines the gradient to use (between 0 and 1)
         * @param min defines the alpha remap minimal range
         * @param max defines the alpha remap maximal range
         * @returns the current particle system
         */
        addAlphaRemapGradient(gradient: number, min: number, max: number): IParticleSystem;
        /**
         * Gets the current list of alpha remap gradients.
         * You must use addAlphaRemapGradient and removeAlphaRemapGradient to update this list
         * @returns the list of alpha remap gradients
         */
        getAlphaRemapGradients(): Nullable<Array<FactorGradient>>;
        /**
         * Creates a Point Emitter for the particle system (emits directly from the emitter position)
         * @param direction1 Particles are emitted between the direction1 and direction2 from within the box
         * @param direction2 Particles are emitted between the direction1 and direction2 from within the box
         * @returns the emitter
         */
        createPointEmitter(direction1: Vector3, direction2: Vector3): PointParticleEmitter;
        /**
         * Creates a Hemisphere Emitter for the particle system (emits along the hemisphere radius)
         * @param radius The radius of the hemisphere to emit from
         * @param radiusRange The range of the hemisphere to emit from [0-1] 0 Surface Only, 1 Entire Radius
         * @returns the emitter
         */
        createHemisphericEmitter(radius: number, radiusRange: number): HemisphericParticleEmitter;
        /**
         * Creates a Sphere Emitter for the particle system (emits along the sphere radius)
         * @param radius The radius of the sphere to emit from
         * @param radiusRange The range of the sphere to emit from [0-1] 0 Surface Only, 1 Entire Radius
         * @returns the emitter
         */
        createSphereEmitter(radius: number, radiusRange: number): SphereParticleEmitter;
        /**
         * Creates a Directed Sphere Emitter for the particle system (emits between direction1 and direction2)
         * @param radius The radius of the sphere to emit from
         * @param direction1 Particles are emitted between the direction1 and direction2 from within the sphere
         * @param direction2 Particles are emitted between the direction1 and direction2 from within the sphere
         * @returns the emitter
         */
        createDirectedSphereEmitter(radius: number, direction1: Vector3, direction2: Vector3): SphereDirectedParticleEmitter;
        /**
         * Creates a Cylinder Emitter for the particle system (emits from the cylinder to the particle position)
         * @param radius The radius of the emission cylinder
         * @param height The height of the emission cylinder
         * @param radiusRange The range of emission [0-1] 0 Surface only, 1 Entire Radius
         * @param directionRandomizer How much to randomize the particle direction [0-1]
         * @returns the emitter
         */
        createCylinderEmitter(radius: number, height: number, radiusRange: number, directionRandomizer: number): CylinderParticleEmitter;
        /**
         * Creates a Directed Cylinder Emitter for the particle system (emits between direction1 and direction2)
         * @param radius The radius of the cylinder to emit from
         * @param height The height of the emission cylinder
         * @param radiusRange the range of the emission cylinder [0-1] 0 Surface only, 1 Entire Radius (1 by default)
         * @param direction1 Particles are emitted between the direction1 and direction2 from within the cylinder
         * @param direction2 Particles are emitted between the direction1 and direction2 from within the cylinder
         * @returns the emitter
         */
        createDirectedCylinderEmitter(radius: number, height: number, radiusRange: number, direction1: Vector3, direction2: Vector3): SphereDirectedParticleEmitter;
        /**
         * Creates a Cone Emitter for the particle system (emits from the cone to the particle position)
         * @param radius The radius of the cone to emit from
         * @param angle The base angle of the cone
         * @returns the emitter
         */
        createConeEmitter(radius: number, angle: number): ConeParticleEmitter;
        /**
         * Creates a Box Emitter for the particle system. (emits between direction1 and direction2 from withing the box defined by minEmitBox and maxEmitBox)
         * @param direction1 Particles are emitted between the direction1 and direction2 from within the box
         * @param direction2 Particles are emitted between the direction1 and direction2 from within the box
         * @param minEmitBox Particles are emitted from the box between minEmitBox and maxEmitBox
         * @param maxEmitBox  Particles are emitted from the box between minEmitBox and maxEmitBox
         * @returns the emitter
         */
        createBoxEmitter(direction1: Vector3, direction2: Vector3, minEmitBox: Vector3, maxEmitBox: Vector3): BoxParticleEmitter;
        /**
         * Get hosting scene
         * @returns the scene
         */
        getScene(): Nullable<Scene>;
    }


    /**
     * A particle represents one of the element emitted by a particle system.
     * This is mainly define by its coordinates, direction, velocity and age.
     */
    export class Particle {
        /**
         * The particle system the particle belongs to.
         */
        particleSystem: ParticleSystem;
        private static _Count;
        /**
         * Unique ID of the particle
         */
        id: number;
        /**
         * The world position of the particle in the scene.
         */
        position: Vector3;
        /**
         * The world direction of the particle in the scene.
         */
        direction: Vector3;
        /**
         * The color of the particle.
         */
        color: Color4;
        /**
         * The color change of the particle per step.
         */
        colorStep: Color4;
        /**
         * Defines how long will the life of the particle be.
         */
        lifeTime: number;
        /**
         * The current age of the particle.
         */
        age: number;
        /**
         * The current size of the particle.
         */
        size: number;
        /**
         * The current scale of the particle.
         */
        scale: Vector2;
        /**
         * The current angle of the particle.
         */
        angle: number;
        /**
         * Defines how fast is the angle changing.
         */
        angularSpeed: number;
        /**
         * Defines the cell index used by the particle to be rendered from a sprite.
         */
        cellIndex: number;
        /**
         * The information required to support color remapping
         */
        remapData: Vector4;
        /** @internal */
        _randomCellOffset?: number;
        /** @internal */
        _initialDirection: Nullable<Vector3>;
        /** @internal */
        _attachedSubEmitters: Nullable<Array<SubEmitter>>;
        /** @internal */
        _initialStartSpriteCellID: number;
        /** @internal */
        _initialEndSpriteCellID: number;
        /** @internal */
        _initialSpriteCellLoop: boolean;
        /** @internal */
        _currentColorGradient: Nullable<ColorGradient>;
        /** @internal */
        _currentColor1: Color4;
        /** @internal */
        _currentColor2: Color4;
        /** @internal */
        _currentSizeGradient: Nullable<FactorGradient>;
        /** @internal */
        _currentSize1: number;
        /** @internal */
        _currentSize2: number;
        /** @internal */
        _currentAngularSpeedGradient: Nullable<FactorGradient>;
        /** @internal */
        _currentAngularSpeed1: number;
        /** @internal */
        _currentAngularSpeed2: number;
        /** @internal */
        _currentVelocityGradient: Nullable<FactorGradient>;
        /** @internal */
        _currentVelocity1: number;
        /** @internal */
        _currentVelocity2: number;
        /** @internal */
        _currentLimitVelocityGradient: Nullable<FactorGradient>;
        /** @internal */
        _currentLimitVelocity1: number;
        /** @internal */
        _currentLimitVelocity2: number;
        /** @internal */
        _currentDragGradient: Nullable<FactorGradient>;
        /** @internal */
        _currentDrag1: number;
        /** @internal */
        _currentDrag2: number;
        /** @internal */
        _randomNoiseCoordinates1: Vector3;
        /** @internal */
        _randomNoiseCoordinates2: Vector3;
        /** @internal */
        _localPosition?: Vector3;
        /**
         * Creates a new instance Particle
         * @param particleSystem the particle system the particle belongs to
         */
        constructor(
        /**
         * The particle system the particle belongs to.
         */
        particleSystem: ParticleSystem);
        private _updateCellInfoFromSystem;
        /**
         * Defines how the sprite cell index is updated for the particle
         */
        updateCellIndex(): void;
        /**
         * @internal
         */
        _inheritParticleInfoToSubEmitter(subEmitter: SubEmitter): void;
        /** @internal */
        _inheritParticleInfoToSubEmitters(): void;
        /** @internal */
        _reset(): void;
        /**
         * Copy the properties of particle to another one.
         * @param other the particle to copy the information to.
         */
        copyTo(other: Particle): void;
    }


    /**
     * This class is made for on one-liner static method to help creating particle system set.
     */
    export class ParticleHelper {
        /**
         * Gets or sets base Assets URL
         */
        static BaseAssetsUrl: string;
        /** Define the Url to load snippets */
        static SnippetUrl: string;
        /**
         * Create a default particle system that you can tweak
         * @param emitter defines the emitter to use
         * @param capacity defines the system capacity (default is 500 particles)
         * @param scene defines the hosting scene
         * @param useGPU defines if a GPUParticleSystem must be created (default is false)
         * @returns the new Particle system
         */
        static CreateDefault(emitter: Nullable<AbstractMesh | Vector3>, capacity?: number, scene?: Scene, useGPU?: boolean): IParticleSystem;
        /**
         * This is the main static method (one-liner) of this helper to create different particle systems
         * @param type This string represents the type to the particle system to create
         * @param scene The scene where the particle system should live
         * @param gpu If the system will use gpu
         * @param capacity defines the system capacity (if null or undefined the sotred capacity will be used)
         * @returns the ParticleSystemSet created
         */
        static CreateAsync(type: string, scene: Nullable<Scene>, gpu?: boolean, capacity?: number): Promise<ParticleSystemSet>;
        /**
         * Static function used to export a particle system to a ParticleSystemSet variable.
         * Please note that the emitter shape is not exported
         * @param systems defines the particle systems to export
         * @returns the created particle system set
         */
        static ExportSet(systems: IParticleSystem[]): ParticleSystemSet;
        /**
         * Creates a particle system from a snippet saved in a remote file
         * @param name defines the name of the particle system to create (can be null or empty to use the one from the json data)
         * @param url defines the url to load from
         * @param scene defines the hosting scene
         * @param gpu If the system will use gpu
         * @param rootUrl defines the root URL to use to load textures and relative dependencies
         * @param capacity defines the system capacity (if null or undefined the sotred capacity will be used)
         * @returns a promise that will resolve to the new particle system
         */
        static ParseFromFileAsync(name: Nullable<string>, url: string, scene: Scene, gpu?: boolean, rootUrl?: string, capacity?: number): Promise<IParticleSystem>;
        /**
         * Creates a particle system from a snippet saved by the particle system editor
         * @param snippetId defines the snippet to load (can be set to _BLANK to create a default one)
         * @param scene defines the hosting scene
         * @param gpu If the system will use gpu
         * @param rootUrl defines the root URL to use to load textures and relative dependencies
         * @param capacity defines the system capacity (if null or undefined the sotred capacity will be used)
         * @returns a promise that will resolve to the new particle system
         */
        static ParseFromSnippetAsync(snippetId: string, scene: Scene, gpu?: boolean, rootUrl?: string, capacity?: number): Promise<IParticleSystem>;
        /**
         * Creates a particle system from a snippet saved by the particle system editor
         * @deprecated Please use ParseFromSnippetAsync instead
         * @param snippetId defines the snippet to load (can be set to _BLANK to create a default one)
         * @param scene defines the hosting scene
         * @param gpu If the system will use gpu
         * @param rootUrl defines the root URL to use to load textures and relative dependencies
         * @param capacity defines the system capacity (if null or undefined the sotred capacity will be used)
         * @returns a promise that will resolve to the new particle system
         */
        static CreateFromSnippetAsync: typeof ParticleHelper.ParseFromSnippetAsync;
    }


    /**
     * This represents a particle system in Babylon.
     * Particles are often small sprites used to simulate hard-to-reproduce phenomena like fire, smoke, water, or abstract visual effects like magic glitter and faery dust.
     * Particles can take different shapes while emitted like box, sphere, cone or you can write your custom function.
     * @example https://doc.babylonjs.com/features/featuresDeepDive/particles/particle_system/particle_system_intro
     */
    export class ParticleSystem extends BaseParticleSystem implements IDisposable, IAnimatable, IParticleSystem {
        /**
         * Billboard mode will only apply to Y axis
         */
        static readonly BILLBOARDMODE_Y = 2;
        /**
         * Billboard mode will apply to all axes
         */
        static readonly BILLBOARDMODE_ALL = 7;
        /**
         * Special billboard mode where the particle will be biilboard to the camera but rotated to align with direction
         */
        static readonly BILLBOARDMODE_STRETCHED = 8;
        /**
         * Special billboard mode where the particle will be billboard to the camera but only around the axis of the direction of particle emission
         */
        static readonly BILLBOARDMODE_STRETCHED_LOCAL = 9;
        /**
         * This function can be defined to provide custom update for active particles.
         * This function will be called instead of regular update (age, position, color, etc.).
         * Do not forget that this function will be called on every frame so try to keep it simple and fast :)
         */
        updateFunction: (particles: Particle[]) => void;
        private _emitterWorldMatrix;
        private _emitterInverseWorldMatrix;
        /**
         * This function can be defined to specify initial direction for every new particle.
         * It by default use the emitterType defined function
         */
        startDirectionFunction: (worldMatrix: Matrix, directionToUpdate: Vector3, particle: Particle, isLocal: boolean) => void;
        /**
         * This function can be defined to specify initial position for every new particle.
         * It by default use the emitterType defined function
         */
        startPositionFunction: (worldMatrix: Matrix, positionToUpdate: Vector3, particle: Particle, isLocal: boolean) => void;
        /**
         * @internal
         */
        _inheritedVelocityOffset: Vector3;
        /**
         * An event triggered when the system is disposed
         */
        onDisposeObservable: Observable<IParticleSystem>;
        /**
         * An event triggered when the system is stopped
         */
        onStoppedObservable: Observable<IParticleSystem>;
        private _onDisposeObserver;
        /**
         * Sets a callback that will be triggered when the system is disposed
         */
        set onDispose(callback: () => void);
        private _particles;
        private _epsilon;
        private _capacity;
        private _stockParticles;
        private _newPartsExcess;
        private _vertexData;
        private _vertexBuffer;
        private _vertexBuffers;
        private _spriteBuffer;
        private _indexBuffer;
        private _drawWrappers;
        private _customWrappers;
        private _scaledColorStep;
        private _colorDiff;
        private _scaledDirection;
        private _scaledGravity;
        private _currentRenderId;
        private _alive;
        private _useInstancing;
        private _vertexArrayObject;
        private _started;
        private _stopped;
        private _actualFrame;
        private _scaledUpdateSpeed;
        private _vertexBufferSize;
        /** @internal */
        _currentEmitRateGradient: Nullable<FactorGradient>;
        /** @internal */
        _currentEmitRate1: number;
        /** @internal */
        _currentEmitRate2: number;
        /** @internal */
        _currentStartSizeGradient: Nullable<FactorGradient>;
        /** @internal */
        _currentStartSize1: number;
        /** @internal */
        _currentStartSize2: number;
        /** Indicates that the update of particles is done in the animate function */
        readonly updateInAnimate = true;
        private readonly _rawTextureWidth;
        private _rampGradientsTexture;
        private _useRampGradients;
        /** Gets or sets a matrix to use to compute projection */
        defaultProjectionMatrix: Matrix;
        /** Gets or sets a matrix to use to compute view */
        defaultViewMatrix: Matrix;
        /** Gets or sets a boolean indicating that ramp gradients must be used
         * @see https://doc.babylonjs.com/features/featuresDeepDive/particles/particle_system/particle_system_intro#ramp-gradients
         */
        get useRampGradients(): boolean;
        set useRampGradients(value: boolean);
        /**
         * The Sub-emitters templates that will be used to generate the sub particle system to be associated with the system, this property is used by the root particle system only.
         * When a particle is spawned, an array will be chosen at random and all the emitters in that array will be attached to the particle.  (Default: [])
         */
        subEmitters: Array<ParticleSystem | SubEmitter | Array<SubEmitter>>;
        private _subEmitters;
        /**
         * @internal
         * If the particle systems emitter should be disposed when the particle system is disposed
         */
        _disposeEmitterOnDispose: boolean;
        /**
         * The current active Sub-systems, this property is used by the root particle system only.
         */
        activeSubSystems: Array<ParticleSystem>;
        /**
         * Specifies if the particles are updated in emitter local space or world space
         */
        isLocal: boolean;
        /** Indicates that the particle system is CPU based */
        readonly isGPU = false;
        private _rootParticleSystem;
        /**
         * Gets the current list of active particles
         */
        get particles(): Particle[];
        /**
         * Gets the number of particles active at the same time.
         * @returns The number of active particles.
         */
        getActiveCount(): number;
        /**
         * Returns the string "ParticleSystem"
         * @returns a string containing the class name
         */
        getClassName(): string;
        /**
         * Gets a boolean indicating that the system is stopping
         * @returns true if the system is currently stopping
         */
        isStopping(): boolean;
        /**
         * Gets the custom effect used to render the particles
         * @param blendMode Blend mode for which the effect should be retrieved
         * @returns The effect
         */
        getCustomEffect(blendMode?: number): Nullable<Effect>;
        private _getCustomDrawWrapper;
        /**
         * Sets the custom effect used to render the particles
         * @param effect The effect to set
         * @param blendMode Blend mode for which the effect should be set
         */
        setCustomEffect(effect: Nullable<Effect>, blendMode?: number): void;
        /** @internal */
        private _onBeforeDrawParticlesObservable;
        /**
         * Observable that will be called just before the particles are drawn
         */
        get onBeforeDrawParticlesObservable(): Observable<Nullable<Effect>>;
        /**
         * Gets the name of the particle vertex shader
         */
        get vertexShaderName(): string;
        /**
         * Gets the vertex buffers used by the particle system
         */
        get vertexBuffers(): Immutable<{
            [key: string]: VertexBuffer;
        }>;
        /**
         * Gets the index buffer used by the particle system (or null if no index buffer is used (if _useInstancing=true))
         */
        get indexBuffer(): Nullable<DataBuffer>;
        /**
         * Instantiates a particle system.
         * Particles are often small sprites used to simulate hard-to-reproduce phenomena like fire, smoke, water, or abstract visual effects like magic glitter and faery dust.
         * @param name The name of the particle system
         * @param capacity The max number of particles alive at the same time
         * @param sceneOrEngine The scene the particle system belongs to or the engine to use if no scene
         * @param customEffect a custom effect used to change the way particles are rendered by default
         * @param isAnimationSheetEnabled Must be true if using a spritesheet to animate the particles texture
         * @param epsilon Offset used to render the particles
         */
        constructor(name: string, capacity: number, sceneOrEngine: Scene | ThinEngine, customEffect?: Nullable<Effect>, isAnimationSheetEnabled?: boolean, epsilon?: number);
        private _addFactorGradient;
        private _removeFactorGradient;
        /**
         * Adds a new life time gradient
         * @param gradient defines the gradient to use (between 0 and 1)
         * @param factor defines the life time factor to affect to the specified gradient
         * @param factor2 defines an additional factor used to define a range ([factor, factor2]) with main value to pick the final value from
         * @returns the current particle system
         */
        addLifeTimeGradient(gradient: number, factor: number, factor2?: number): IParticleSystem;
        /**
         * Remove a specific life time gradient
         * @param gradient defines the gradient to remove
         * @returns the current particle system
         */
        removeLifeTimeGradient(gradient: number): IParticleSystem;
        /**
         * Adds a new size gradient
         * @param gradient defines the gradient to use (between 0 and 1)
         * @param factor defines the size factor to affect to the specified gradient
         * @param factor2 defines an additional factor used to define a range ([factor, factor2]) with main value to pick the final value from
         * @returns the current particle system
         */
        addSizeGradient(gradient: number, factor: number, factor2?: number): IParticleSystem;
        /**
         * Remove a specific size gradient
         * @param gradient defines the gradient to remove
         * @returns the current particle system
         */
        removeSizeGradient(gradient: number): IParticleSystem;
        /**
         * Adds a new color remap gradient
         * @param gradient defines the gradient to use (between 0 and 1)
         * @param min defines the color remap minimal range
         * @param max defines the color remap maximal range
         * @returns the current particle system
         */
        addColorRemapGradient(gradient: number, min: number, max: number): IParticleSystem;
        /**
         * Remove a specific color remap gradient
         * @param gradient defines the gradient to remove
         * @returns the current particle system
         */
        removeColorRemapGradient(gradient: number): IParticleSystem;
        /**
         * Adds a new alpha remap gradient
         * @param gradient defines the gradient to use (between 0 and 1)
         * @param min defines the alpha remap minimal range
         * @param max defines the alpha remap maximal range
         * @returns the current particle system
         */
        addAlphaRemapGradient(gradient: number, min: number, max: number): IParticleSystem;
        /**
         * Remove a specific alpha remap gradient
         * @param gradient defines the gradient to remove
         * @returns the current particle system
         */
        removeAlphaRemapGradient(gradient: number): IParticleSystem;
        /**
         * Adds a new angular speed gradient
         * @param gradient defines the gradient to use (between 0 and 1)
         * @param factor defines the angular speed  to affect to the specified gradient
         * @param factor2 defines an additional factor used to define a range ([factor, factor2]) with main value to pick the final value from
         * @returns the current particle system
         */
        addAngularSpeedGradient(gradient: number, factor: number, factor2?: number): IParticleSystem;
        /**
         * Remove a specific angular speed gradient
         * @param gradient defines the gradient to remove
         * @returns the current particle system
         */
        removeAngularSpeedGradient(gradient: number): IParticleSystem;
        /**
         * Adds a new velocity gradient
         * @param gradient defines the gradient to use (between 0 and 1)
         * @param factor defines the velocity to affect to the specified gradient
         * @param factor2 defines an additional factor used to define a range ([factor, factor2]) with main value to pick the final value from
         * @returns the current particle system
         */
        addVelocityGradient(gradient: number, factor: number, factor2?: number): IParticleSystem;
        /**
         * Remove a specific velocity gradient
         * @param gradient defines the gradient to remove
         * @returns the current particle system
         */
        removeVelocityGradient(gradient: number): IParticleSystem;
        /**
         * Adds a new limit velocity gradient
         * @param gradient defines the gradient to use (between 0 and 1)
         * @param factor defines the limit velocity value to affect to the specified gradient
         * @param factor2 defines an additional factor used to define a range ([factor, factor2]) with main value to pick the final value from
         * @returns the current particle system
         */
        addLimitVelocityGradient(gradient: number, factor: number, factor2?: number): IParticleSystem;
        /**
         * Remove a specific limit velocity gradient
         * @param gradient defines the gradient to remove
         * @returns the current particle system
         */
        removeLimitVelocityGradient(gradient: number): IParticleSystem;
        /**
         * Adds a new drag gradient
         * @param gradient defines the gradient to use (between 0 and 1)
         * @param factor defines the drag value to affect to the specified gradient
         * @param factor2 defines an additional factor used to define a range ([factor, factor2]) with main value to pick the final value from
         * @returns the current particle system
         */
        addDragGradient(gradient: number, factor: number, factor2?: number): IParticleSystem;
        /**
         * Remove a specific drag gradient
         * @param gradient defines the gradient to remove
         * @returns the current particle system
         */
        removeDragGradient(gradient: number): IParticleSystem;
        /**
         * Adds a new emit rate gradient (please note that this will only work if you set the targetStopDuration property)
         * @param gradient defines the gradient to use (between 0 and 1)
         * @param factor defines the emit rate value to affect to the specified gradient
         * @param factor2 defines an additional factor used to define a range ([factor, factor2]) with main value to pick the final value from
         * @returns the current particle system
         */
        addEmitRateGradient(gradient: number, factor: number, factor2?: number): IParticleSystem;
        /**
         * Remove a specific emit rate gradient
         * @param gradient defines the gradient to remove
         * @returns the current particle system
         */
        removeEmitRateGradient(gradient: number): IParticleSystem;
        /**
         * Adds a new start size gradient (please note that this will only work if you set the targetStopDuration property)
         * @param gradient defines the gradient to use (between 0 and 1)
         * @param factor defines the start size value to affect to the specified gradient
         * @param factor2 defines an additional factor used to define a range ([factor, factor2]) with main value to pick the final value from
         * @returns the current particle system
         */
        addStartSizeGradient(gradient: number, factor: number, factor2?: number): IParticleSystem;
        /**
         * Remove a specific start size gradient
         * @param gradient defines the gradient to remove
         * @returns the current particle system
         */
        removeStartSizeGradient(gradient: number): IParticleSystem;
        private _createRampGradientTexture;
        /**
         * Gets the current list of ramp gradients.
         * You must use addRampGradient and removeRampGradient to update this list
         * @returns the list of ramp gradients
         */
        getRampGradients(): Nullable<Array<Color3Gradient>>;
        /** Force the system to rebuild all gradients that need to be resync */
        forceRefreshGradients(): void;
        private _syncRampGradientTexture;
        /**
         * Adds a new ramp gradient used to remap particle colors
         * @param gradient defines the gradient to use (between 0 and 1)
         * @param color defines the color to affect to the specified gradient
         * @returns the current particle system
         */
        addRampGradient(gradient: number, color: Color3): ParticleSystem;
        /**
         * Remove a specific ramp gradient
         * @param gradient defines the gradient to remove
         * @returns the current particle system
         */
        removeRampGradient(gradient: number): ParticleSystem;
        /**
         * Adds a new color gradient
         * @param gradient defines the gradient to use (between 0 and 1)
         * @param color1 defines the color to affect to the specified gradient
         * @param color2 defines an additional color used to define a range ([color, color2]) with main color to pick the final color from
         * @returns this particle system
         */
        addColorGradient(gradient: number, color1: Color4, color2?: Color4): IParticleSystem;
        /**
         * Remove a specific color gradient
         * @param gradient defines the gradient to remove
         * @returns this particle system
         */
        removeColorGradient(gradient: number): IParticleSystem;
        /**
         * Resets the draw wrappers cache
         */
        resetDrawCache(): void;
        private _fetchR;
        protected _reset(): void;
        private _resetEffect;
        private _createVertexBuffers;
        private _createIndexBuffer;
        /**
         * Gets the maximum number of particles active at the same time.
         * @returns The max number of active particles.
         */
        getCapacity(): number;
        /**
         * Gets whether there are still active particles in the system.
         * @returns True if it is alive, otherwise false.
         */
        isAlive(): boolean;
        /**
         * Gets if the system has been started. (Note: this will still be true after stop is called)
         * @returns True if it has been started, otherwise false.
         */
        isStarted(): boolean;
        private _prepareSubEmitterInternalArray;
        /**
         * Starts the particle system and begins to emit
         * @param delay defines the delay in milliseconds before starting the system (this.startDelay by default)
         */
        start(delay?: number): void;
        /**
         * Stops the particle system.
         * @param stopSubEmitters if true it will stop the current system and all created sub-Systems if false it will stop the current root system only, this param is used by the root particle system only. the default value is true.
         */
        stop(stopSubEmitters?: boolean): void;
        /**
         * Remove all active particles
         */
        reset(): void;
        /**
         * @internal (for internal use only)
         */
        _appendParticleVertex(index: number, particle: Particle, offsetX: number, offsetY: number): void;
        /**
         * "Recycles" one of the particle by copying it back to the "stock" of particles and removing it from the active list.
         * Its lifetime will start back at 0.
         * @param particle
         */
        recycleParticle: (particle: Particle) => void;
        private _stopSubEmitters;
        private _createParticle;
        private _removeFromRoot;
        private _emitFromParticle;
        private _update;
        /**
         * @internal
         */
        static _GetAttributeNamesOrOptions(isAnimationSheetEnabled?: boolean, isBillboardBased?: boolean, useRampGradients?: boolean): string[];
        /**
         * @internal
         */
        static _GetEffectCreationOptions(isAnimationSheetEnabled?: boolean, useLogarithmicDepth?: boolean): string[];
        /**
         * Fill the defines array according to the current settings of the particle system
         * @param defines Array to be updated
         * @param blendMode blend mode to take into account when updating the array
         */
        fillDefines(defines: Array<string>, blendMode: number): void;
        /**
         * Fill the uniforms, attributes and samplers arrays according to the current settings of the particle system
         * @param uniforms Uniforms array to fill
         * @param attributes Attributes array to fill
         * @param samplers Samplers array to fill
         */
        fillUniformsAttributesAndSamplerNames(uniforms: Array<string>, attributes: Array<string>, samplers: Array<string>): void;
        /**
         * @internal
         */
        private _getWrapper;
        /**
         * Animates the particle system for the current frame by emitting new particles and or animating the living ones.
         * @param preWarmOnly will prevent the system from updating the vertex buffer (default is false)
         */
        animate(preWarmOnly?: boolean): void;
        private _appendParticleVertices;
        /**
         * Rebuilds the particle system.
         */
        rebuild(): void;
        /**
         * Is this system ready to be used/rendered
         * @returns true if the system is ready
         */
        isReady(): boolean;
        private _render;
        /**
         * Renders the particle system in its current state.
         * @returns the current number of particles
         */
        render(): number;
        /**
         * Disposes the particle system and free the associated resources
         * @param disposeTexture defines if the particle texture must be disposed as well (true by default)
         */
        dispose(disposeTexture?: boolean): void;
        /**
         * Clones the particle system.
         * @param name The name of the cloned object
         * @param newEmitter The new emitter to use
         * @param cloneTexture Also clone the textures if true
         * @returns the cloned particle system
         */
        clone(name: string, newEmitter: any, cloneTexture?: boolean): ParticleSystem;
        /**
         * Serializes the particle system to a JSON object
         * @param serializeTexture defines if the texture must be serialized as well
         * @returns the JSON object
         */
        serialize(serializeTexture?: boolean): any;
        /**
         * @internal
         */
        static _Serialize(serializationObject: any, particleSystem: IParticleSystem, serializeTexture: boolean): void;
        /**
         * @internal
         */
        static _Parse(parsedParticleSystem: any, particleSystem: IParticleSystem, sceneOrEngine: Scene | ThinEngine, rootUrl: string): void;
        /**
         * Parses a JSON object to create a particle system.
         * @param parsedParticleSystem The JSON object to parse
         * @param sceneOrEngine The scene or the engine to create the particle system in
         * @param rootUrl The root url to use to load external dependencies like texture
         * @param doNotStart Ignore the preventAutoStart attribute and does not start
         * @param capacity defines the system capacity (if null or undefined the sotred capacity will be used)
         * @returns the Parsed particle system
         */
        static Parse(parsedParticleSystem: any, sceneOrEngine: Scene | ThinEngine, rootUrl: string, doNotStart?: boolean, capacity?: number): ParticleSystem;
    }


        interface Engine {
            /**
             * Create an effect to use with particle systems.
             * Please note that some parameters like animation sheets or not being billboard are not supported in this configuration, except if you pass
             * the particle system for which you want to create a custom effect in the last parameter
             * @param fragmentName defines the base name of the effect (The name of file without .fragment.fx)
             * @param uniformsNames defines a list of attribute names
             * @param samplers defines an array of string used to represent textures
             * @param defines defines the string containing the defines to use to compile the shaders
             * @param fallbacks defines the list of potential fallbacks to use if shader compilation fails
             * @param onCompiled defines a function to call when the effect creation is successful
             * @param onError defines a function to call when the effect creation has failed
             * @param particleSystem the particle system you want to create the effect for
             * @returns the new Effect
             */
            createEffectForParticles(fragmentName: string, uniformsNames: string[], samplers: string[], defines: string, fallbacks?: EffectFallbacks, onCompiled?: (effect: Effect) => void, onError?: (effect: Effect, errors: string) => void, particleSystem?: IParticleSystem): Effect;
        }
        interface Mesh {
            /**
             * Returns an array populated with IParticleSystem objects whose the mesh is the emitter
             * @returns an array of IParticleSystem
             */
            getEmittedParticleSystems(): IParticleSystem[];
            /**
             * Returns an array populated with IParticleSystem objects whose the mesh or its children are the emitter
             * @returns an array of IParticleSystem
             */
            getHierarchyEmittedParticleSystems(): IParticleSystem[];
        }


    /**
     * Represents a set of particle systems working together to create a specific effect
     */
    export class ParticleSystemSet implements IDisposable {
        /**
         * Gets or sets base Assets URL
         */
        static BaseAssetsUrl: string;
        private _emitterCreationOptions;
        private _emitterNode;
        private _emitterNodeIsOwned;
        /**
         * Gets the particle system list
         */
        systems: IParticleSystem[];
        /**
         * Gets or sets the emitter node used with this set
         */
        get emitterNode(): Nullable<AbstractMesh | Vector3>;
        set emitterNode(value: Nullable<AbstractMesh | Vector3>);
        /**
         * Creates a new emitter mesh as a sphere
         * @param options defines the options used to create the sphere
         * @param options.diameter
         * @param options.segments
         * @param options.color
         * @param renderingGroupId defines the renderingGroupId to use for the sphere
         * @param scene defines the hosting scene
         */
        setEmitterAsSphere(options: {
            diameter: number;
            segments: number;
            color: Color3;
        }, renderingGroupId: number, scene: Scene): void;
        /**
         * Starts all particle systems of the set
         * @param emitter defines an optional mesh to use as emitter for the particle systems
         */
        start(emitter?: AbstractMesh): void;
        /**
         * Release all associated resources
         */
        dispose(): void;
        /**
         * Serialize the set into a JSON compatible object
         * @param serializeTexture defines if the texture must be serialized as well
         * @returns a JSON compatible representation of the set
         */
        serialize(serializeTexture?: boolean): any;
        /**
         * Parse a new ParticleSystemSet from a serialized source
         * @param data defines a JSON compatible representation of the set
         * @param scene defines the hosting scene
         * @param gpu defines if we want GPU particles or CPU particles
         * @param capacity defines the system capacity (if null or undefined the sotred capacity will be used)
         * @returns a new ParticleSystemSet
         */
        static Parse(data: any, scene: Scene, gpu?: boolean, capacity?: number): ParticleSystemSet;
    }


    /** Defines the 4 color options */
    export enum PointColor {
        /** color value */
        Color = 2,
        /** uv value */
        UV = 1,
        /** random value */
        Random = 0,
        /** stated value */
        Stated = 3
    }
    /**
     * The PointCloudSystem (PCS) is a single updatable mesh. The points corresponding to the vertices of this big mesh.
     * As it is just a mesh, the PointCloudSystem has all the same properties as any other BJS mesh : not more, not less. It can be scaled, rotated, translated, enlighted, textured, moved, etc.
     * The PointCloudSystem is also a particle system, with each point being a particle. It provides some methods to manage the particles.
     * However it is behavior agnostic. This means it has no emitter, no particle physics, no particle recycler. You have to implement your own behavior.
     *
     * Full documentation here : TO BE ENTERED
     */
    export class PointsCloudSystem implements IDisposable {
        /**
         *  The PCS array of cloud point objects. Just access each particle as with any classic array.
         *  Example : var p = SPS.particles[i];
         */
        particles: CloudPoint[];
        /**
         * The PCS total number of particles. Read only. Use PCS.counter instead if you need to set your own value.
         */
        nbParticles: number;
        /**
         * This a counter for your own usage. It's not set by any SPS functions.
         */
        counter: number;
        /**
         * The PCS name. This name is also given to the underlying mesh.
         */
        name: string;
        /**
         * The PCS mesh. It's a standard BJS Mesh, so all the methods from the Mesh class are available.
         */
        mesh?: Mesh;
        /**
         * This empty object is intended to store some PCS specific or temporary values in order to lower the Garbage Collector activity.
         * Please read :
         */
        vars: any;
        /**
         * @internal
         */
        _size: number;
        private _scene;
        private _promises;
        private _positions;
        private _indices;
        private _normals;
        private _colors;
        private _uvs;
        private _indices32;
        private _positions32;
        private _colors32;
        private _uvs32;
        private _updatable;
        private _isVisibilityBoxLocked;
        private _alwaysVisible;
        private _groups;
        private _groupCounter;
        private _computeParticleColor;
        private _computeParticleTexture;
        private _computeParticleRotation;
        private _computeBoundingBox;
        private _isReady;
        /**
         * Gets the particle positions computed by the Point Cloud System
         */
        get positions(): Float32Array;
        /**
         * Gets the particle colors computed by the Point Cloud System
         */
        get colors(): Float32Array;
        /**
         * Gets the particle uvs computed by the Point Cloud System
         */
        get uvs(): Float32Array;
        /**
         * Creates a PCS (Points Cloud System) object
         * @param name (String) is the PCS name, this will be the underlying mesh name
         * @param pointSize (number) is the size for each point. Has no effect on a WebGPU engine.
         * @param scene (Scene) is the scene in which the PCS is added
         * @param options defines the options of the PCS e.g.
         * * updatable (optional boolean, default true) : if the PCS must be updatable or immutable
         * @param options.updatable
         */
        constructor(name: string, pointSize: number, scene: Scene, options?: {
            updatable?: boolean;
        });
        /**
         * Builds the PCS underlying mesh. Returns a standard Mesh.
         * If no points were added to the PCS, the returned mesh is just a single point.
         * @param material The material to use to render the mesh. If not provided, will create a default one
         * @returns a promise for the created mesh
         */
        buildMeshAsync(material?: Material): Promise<Mesh>;
        /**
         * @internal
         */
        private _buildMesh;
        private _addParticle;
        private _randomUnitVector;
        private _getColorIndicesForCoord;
        private _setPointsColorOrUV;
        private _colorFromTexture;
        private _calculateDensity;
        /**
         * Adds points to the PCS in random positions within a unit sphere
         * @param nb (positive integer) the number of particles to be created from this model
         * @param pointFunction is an optional javascript function to be called for each particle on PCS creation
         * @returns the number of groups in the system
         */
        addPoints(nb: number, pointFunction?: any): number;
        /**
         * Adds points to the PCS from the surface of the model shape
         * @param mesh is any Mesh object that will be used as a surface model for the points
         * @param nb (positive integer) the number of particles to be created from this model
         * @param colorWith determines whether a point is colored using color (default), uv, random, stated or none (invisible)
         * @param color (color4) to be used when colorWith is stated or color (number) when used to specify texture position
         * @param range (number from 0 to 1) to determine the variation in shape and tone for a stated color
         * @returns the number of groups in the system
         */
        addSurfacePoints(mesh: Mesh, nb: number, colorWith?: number, color?: Color4 | number, range?: number): number;
        /**
         * Adds points to the PCS inside the model shape
         * @param mesh is any Mesh object that will be used as a surface model for the points
         * @param nb (positive integer) the number of particles to be created from this model
         * @param colorWith determines whether a point is colored using color (default), uv, random, stated or none (invisible)
         * @param color (color4) to be used when colorWith is stated or color (number) when used to specify texture position
         * @param range (number from 0 to 1) to determine the variation in shape and tone for a stated color
         * @returns the number of groups in the system
         */
        addVolumePoints(mesh: Mesh, nb: number, colorWith?: number, color?: Color4 | number, range?: number): number;
        /**
         *  Sets all the particles : this method actually really updates the mesh according to the particle positions, rotations, colors, textures, etc.
         *  This method calls `updateParticle()` for each particle of the SPS.
         *  For an animated SPS, it is usually called within the render loop.
         * @param start The particle index in the particle array where to start to compute the particle property values _(default 0)_
         * @param end The particle index in the particle array where to stop to compute the particle property values _(default nbParticle - 1)_
         * @param update If the mesh must be finally updated on this call after all the particle computations _(default true)_
         * @returns the PCS.
         */
        setParticles(start?: number, end?: number, update?: boolean): PointsCloudSystem;
        /**
         * Disposes the PCS.
         */
        dispose(): void;
        /**
         * Visibility helper : Recomputes the visible size according to the mesh bounding box
         * doc :
         * @returns the PCS.
         */
        refreshVisibleSize(): PointsCloudSystem;
        /**
         * Visibility helper : Sets the size of a visibility box, this sets the underlying mesh bounding box.
         * @param size the size (float) of the visibility box
         * note : this doesn't lock the PCS mesh bounding box.
         * doc :
         */
        setVisibilityBox(size: number): void;
        /**
         * Gets whether the PCS is always visible or not
         * doc :
         */
        get isAlwaysVisible(): boolean;
        /**
         * Sets the PCS as always visible or not
         * doc :
         */
        set isAlwaysVisible(val: boolean);
        /**
         * Tells to `setParticles()` to compute the particle rotations or not
         * Default value : false. The PCS is faster when it's set to false
         * Note : particle rotations are only applied to parent particles
         * Note : the particle rotations aren't stored values, so setting `computeParticleRotation` to false will prevents the particle to rotate
         */
        set computeParticleRotation(val: boolean);
        /**
         * Tells to `setParticles()` to compute the particle colors or not.
         * Default value : true. The PCS is faster when it's set to false.
         * Note : the particle colors are stored values, so setting `computeParticleColor` to false will keep yet the last colors set.
         */
        set computeParticleColor(val: boolean);
        set computeParticleTexture(val: boolean);
        /**
         * Gets if `setParticles()` computes the particle colors or not.
         * Default value : false. The PCS is faster when it's set to false.
         * Note : the particle colors are stored values, so setting `computeParticleColor` to false will keep yet the last colors set.
         */
        get computeParticleColor(): boolean;
        /**
         * Gets if `setParticles()` computes the particle textures or not.
         * Default value : false. The PCS is faster when it's set to false.
         * Note : the particle textures are stored values, so setting `computeParticleTexture` to false will keep yet the last colors set.
         */
        get computeParticleTexture(): boolean;
        /**
         * Tells to `setParticles()` to compute or not the mesh bounding box when computing the particle positions.
         */
        set computeBoundingBox(val: boolean);
        /**
         * Gets if `setParticles()` computes or not the mesh bounding box when computing the particle positions.
         */
        get computeBoundingBox(): boolean;
        /**
         * This function does nothing. It may be overwritten to set all the particle first values.
         * The PCS doesn't call this function, you may have to call it by your own.
         * doc :
         */
        initParticles(): void;
        /**
         * This function does nothing. It may be overwritten to recycle a particle
         * The PCS doesn't call this function, you can to call it
         * doc :
         * @param particle The particle to recycle
         * @returns the recycled particle
         */
        recycleParticle(particle: CloudPoint): CloudPoint;
        /**
         * Updates a particle : this function should  be overwritten by the user.
         * It is called on each particle by `setParticles()`. This is the place to code each particle behavior.
         * doc :
         * @example : just set a particle position or velocity and recycle conditions
         * @param particle The particle to update
         * @returns the updated particle
         */
        updateParticle(particle: CloudPoint): CloudPoint;
        /**
         * This will be called before any other treatment by `setParticles()` and will be passed three parameters.
         * This does nothing and may be overwritten by the user.
         * @param start the particle index in the particle array where to start to iterate, same than the value passed to setParticle()
         * @param stop the particle index in the particle array where to stop to iterate, same than the value passed to setParticle()
         * @param update the boolean update value actually passed to setParticles()
         */
        beforeUpdateParticles(start?: number, stop?: number, update?: boolean): void;
        /**
         * This will be called  by `setParticles()` after all the other treatments and just before the actual mesh update.
         * This will be passed three parameters.
         * This does nothing and may be overwritten by the user.
         * @param start the particle index in the particle array where to start to iterate, same than the value passed to setParticle()
         * @param stop the particle index in the particle array where to stop to iterate, same than the value passed to setParticle()
         * @param update the boolean update value actually passed to setParticles()
         */
        afterUpdateParticles(start?: number, stop?: number, update?: boolean): void;
    }


    /**
     * Represents one particle of a solid particle system.
     */
    export class SolidParticle {
        /**
         * particle global index
         */
        idx: number;
        /**
         * particle identifier
         */
        id: number;
        /**
         * The color of the particle
         */
        color: Nullable<Color4>;
        /**
         * The world space position of the particle.
         */
        position: Vector3;
        /**
         * The world space rotation of the particle. (Not use if rotationQuaternion is set)
         */
        rotation: Vector3;
        /**
         * The world space rotation quaternion of the particle.
         */
        rotationQuaternion: Nullable<Quaternion>;
        /**
         * The scaling of the particle.
         */
        scaling: Vector3;
        /**
         * The uvs of the particle.
         */
        uvs: Vector4;
        /**
         * The current speed of the particle.
         */
        velocity: Vector3;
        /**
         * The pivot point in the particle local space.
         */
        pivot: Vector3;
        /**
         * Must the particle be translated from its pivot point in its local space ?
         * In this case, the pivot point is set at the origin of the particle local space and the particle is translated.
         * Default : false
         */
        translateFromPivot: boolean;
        /**
         * Is the particle active or not ?
         */
        alive: boolean;
        /**
         * Is the particle visible or not ?
         */
        isVisible: boolean;
        /**
         * Index of this particle in the global "positions" array (Internal use)
         * @internal
         */
        _pos: number;
        /**
         * @internal Index of this particle in the global "indices" array (Internal use)
         */
        _ind: number;
        /**
         * @internal ModelShape of this particle (Internal use)
         */
        _model: ModelShape;
        /**
         * ModelShape id of this particle
         */
        shapeId: number;
        /**
         * Index of the particle in its shape id
         */
        idxInShape: number;
        /**
         * @internal Reference to the shape model BoundingInfo object (Internal use)
         */
        _modelBoundingInfo: BoundingInfo;
        private _boundingInfo;
        /**
         * @internal Reference to the SPS what the particle belongs to (Internal use)
         */
        _sps: SolidParticleSystem;
        /**
         * @internal Still set as invisible in order to skip useless computations (Internal use)
         */
        _stillInvisible: boolean;
        /**
         * @internal Last computed particle rotation matrix
         */
        _rotationMatrix: number[];
        /**
         * Parent particle Id, if any.
         * Default null.
         */
        parentId: Nullable<number>;
        /**
         * The particle material identifier (integer) when MultiMaterials are enabled in the SPS.
         */
        materialIndex: Nullable<number>;
        /**
         * Custom object or properties.
         */
        props: Nullable<any>;
        /**
         * The culling strategy to use to check whether the solid particle must be culled or not when using isInFrustum().
         * The possible values are :
         * - AbstractMesh.CULLINGSTRATEGY_STANDARD
         * - AbstractMesh.CULLINGSTRATEGY_BOUNDINGSPHERE_ONLY
         * - AbstractMesh.CULLINGSTRATEGY_OPTIMISTIC_INCLUSION
         * - AbstractMesh.CULLINGSTRATEGY_OPTIMISTIC_INCLUSION_THEN_BSPHERE_ONLY
         * The default value for solid particles is AbstractMesh.CULLINGSTRATEGY_BOUNDINGSPHERE_ONLY
         * Please read each static variable documentation in the class AbstractMesh to get details about the culling process.
         * */
        cullingStrategy: number;
        /**
         * @internal Internal global position in the SPS.
         */
        _globalPosition: Vector3;
        /**
         * Particle BoundingInfo object
         * @returns a BoundingInfo
         */
        getBoundingInfo(): BoundingInfo;
        /**
         * Returns true if there is already a bounding info
         */
        get hasBoundingInfo(): boolean;
        /**
         * Creates a Solid Particle object.
         * Don't create particles manually, use instead the Solid Particle System internal tools like _addParticle()
         * @param particleIndex (integer) is the particle index in the Solid Particle System pool.
         * @param particleId (integer) is the particle identifier. Unless some particles are removed from the SPS, it's the same value than the particle idx.
         * @param positionIndex (integer) is the starting index of the particle vertices in the SPS "positions" array.
         * @param indiceIndex (integer) is the starting index of the particle indices in the SPS "indices" array.
         * @param model (ModelShape) is a reference to the model shape on what the particle is designed.
         * @param shapeId (integer) is the model shape identifier in the SPS.
         * @param idxInShape (integer) is the index of the particle in the current model (ex: the 10th box of addShape(box, 30))
         * @param sps defines the sps it is associated to
         * @param modelBoundingInfo is the reference to the model BoundingInfo used for intersection computations.
         * @param materialIndex is the particle material identifier (integer) when the MultiMaterials are enabled in the SPS.
         */
        constructor(particleIndex: number, particleId: number, positionIndex: number, indiceIndex: number, model: Nullable<ModelShape>, shapeId: number, idxInShape: number, sps: SolidParticleSystem, modelBoundingInfo?: Nullable<BoundingInfo>, materialIndex?: Nullable<number>);
        /**
         * Copies the particle property values into the existing target : position, rotation, scaling, uvs, colors, pivot, parent, visibility, alive
         * @param target the particle target
         * @returns the current particle
         */
        copyToRef(target: SolidParticle): SolidParticle;
        /**
         * Legacy support, changed scale to scaling
         */
        get scale(): Vector3;
        /**
         * Legacy support, changed scale to scaling
         */
        set scale(scale: Vector3);
        /**
         * Legacy support, changed quaternion to rotationQuaternion
         */
        get quaternion(): Nullable<Quaternion>;
        /**
         * Legacy support, changed quaternion to rotationQuaternion
         */
        set quaternion(q: Nullable<Quaternion>);
        /**
         * Returns a boolean. True if the particle intersects another particle or another mesh, else false.
         * The intersection is computed on the particle bounding sphere and Axis Aligned Bounding Box (AABB)
         * @param target is the object (solid particle or mesh) what the intersection is computed against.
         * @returns true if it intersects
         */
        intersectsMesh(target: Mesh | SolidParticle): boolean;
        /**
         * Returns `true` if the solid particle is within the frustum defined by the passed array of planes.
         * A particle is in the frustum if its bounding box intersects the frustum
         * @param frustumPlanes defines the frustum to test
         * @returns true if the particle is in the frustum planes
         */
        isInFrustum(frustumPlanes: Plane[]): boolean;
        /**
         * get the rotation matrix of the particle
         * @internal
         */
        getRotationMatrix(m: Matrix): void;
    }
    /**
     * Represents the shape of the model used by one particle of a solid particle system.
     * SPS internal tool, don't use it manually.
     */
    export class ModelShape {
        /**
         * Get or set the shapeId
         * @deprecated Please use shapeId instead
         */
        get shapeID(): number;
        set shapeID(shapeID: number);
        /**
         * The shape id
         * @internal
         */
        shapeId: number;
        /**
         * flat array of model positions (internal use)
         * @internal
         */
        _shape: Vector3[];
        /**
         * flat array of model UVs (internal use)
         * @internal
         */
        _shapeUV: number[];
        /**
         * color array of the model
         * @internal
         */
        _shapeColors: number[];
        /**
         * indices array of the model
         * @internal
         */
        _indices: number[];
        /**
         * normals array of the model
         * @internal
         */
        _normals: number[];
        /**
         * length of the shape in the model indices array (internal use)
         * @internal
         */
        _indicesLength: number;
        /**
         * Custom position function (internal use)
         * @internal
         */
        _positionFunction: Nullable<(particle: SolidParticle, i: number, s: number) => void>;
        /**
         * Custom vertex function (internal use)
         * @internal
         */
        _vertexFunction: Nullable<(particle: SolidParticle, vertex: Vector3, i: number) => void>;
        /**
         * Model material (internal use)
         * @internal
         */
        _material: Nullable<Material>;
        /**
         * Creates a ModelShape object. This is an internal simplified reference to a mesh used as for a model to replicate particles from by the SPS.
         * SPS internal tool, don't use it manually.
         * @internal
         */
        constructor(id: number, shape: Vector3[], indices: number[], normals: number[], colors: number[], shapeUV: number[], posFunction: Nullable<(particle: SolidParticle, i: number, s: number) => void>, vtxFunction: Nullable<(particle: SolidParticle, vertex: Vector3, i: number) => void>, material: Nullable<Material>);
    }
    /**
     * Represents a Depth Sorted Particle in the solid particle system.
     * @internal
     */
    export class DepthSortedParticle {
        /**
         * Particle index
         */
        idx: number;
        /**
         * Index of the particle in the "indices" array
         */
        ind: number;
        /**
         * Length of the particle shape in the "indices" array
         */
        indicesLength: number;
        /**
         * Squared distance from the particle to the camera
         */
        sqDistance: number;
        /**
         * Material index when used with MultiMaterials
         */
        materialIndex: number;
        /**
         * Creates a new sorted particle
         * @param idx
         * @param ind
         * @param indLength
         * @param materialIndex
         */
        constructor(idx: number, ind: number, indLength: number, materialIndex: number);
    }
    /**
     * Represents a solid particle vertex
     */
    export class SolidParticleVertex {
        /**
         * Vertex position
         */
        position: Vector3;
        /**
         * Vertex color
         */
        color: Color4;
        /**
         * Vertex UV
         */
        uv: Vector2;
        /**
         * Creates a new solid particle vertex
         */
        constructor();
        /** Vertex x coordinate */
        get x(): number;
        set x(val: number);
        /** Vertex y coordinate */
        get y(): number;
        set y(val: number);
        /** Vertex z coordinate */
        get z(): number;
        set z(val: number);
    }


    /**
     * The SPS is a single updatable mesh. The solid particles are simply separate parts or faces fo this big mesh.
     *As it is just a mesh, the SPS has all the same properties than any other BJS mesh : not more, not less. It can be scaled, rotated, translated, enlighted, textured, moved, etc.
     * The SPS is also a particle system. It provides some methods to manage the particles.
     * However it is behavior agnostic. This means it has no emitter, no particle physics, no particle recycler. You have to implement your own behavior.
     *
     * Full documentation here : https://doc.babylonjs.com/features/featuresDeepDive/particles/solid_particle_system/sps_intro
     */
    export class SolidParticleSystem implements IDisposable {
        /**
         *  The SPS array of Solid Particle objects. Just access each particle as with any classic array.
         *  Example : var p = SPS.particles[i];
         */
        particles: SolidParticle[];
        /**
         * The SPS total number of particles. Read only. Use SPS.counter instead if you need to set your own value.
         */
        nbParticles: number;
        /**
         * If the particles must ever face the camera (default false). Useful for planar particles.
         */
        billboard: boolean;
        /**
         * Recompute normals when adding a shape
         */
        recomputeNormals: boolean;
        /**
         * This a counter ofr your own usage. It's not set by any SPS functions.
         */
        counter: number;
        /**
         * The SPS name. This name is also given to the underlying mesh.
         */
        name: string;
        /**
         * The SPS mesh. It's a standard BJS Mesh, so all the methods from the Mesh class are available.
         */
        mesh: Mesh;
        /**
         * This empty object is intended to store some SPS specific or temporary values in order to lower the Garbage Collector activity.
         * Please read : https://doc.babylonjs.com/features/featuresDeepDive/particles/solid_particle_system/optimize_sps#limit-garbage-collection
         */
        vars: any;
        /**
         * This array is populated when the SPS is set as 'pickable'.
         * Each key of this array is a `faceId` value that you can get from a pickResult object.
         * Each element of this array is an object `{idx: int, faceId: int}`.
         * `idx` is the picked particle index in the `SPS.particles` array
         * `faceId` is the picked face index counted within this particle.
         * This array is the first element of the pickedBySubMesh array : sps.pickBySubMesh[0].
         * It's not pertinent to use it when using a SPS with the support for MultiMaterial enabled.
         * Use the method SPS.pickedParticle(pickingInfo) instead.
         * Please read : https://doc.babylonjs.com/features/featuresDeepDive/particles/solid_particle_system/picking_sps
         */
        pickedParticles: {
            idx: number;
            faceId: number;
        }[];
        /**
         * This array is populated when the SPS is set as 'pickable'
         * Each key of this array is a submesh index.
         * Each element of this array is a second array defined like this :
         * Each key of this second array is a `faceId` value that you can get from a pickResult object.
         * Each element of this second array is an object `{idx: int, faceId: int}`.
         * `idx` is the picked particle index in the `SPS.particles` array
         * `faceId` is the picked face index counted within this particle.
         * It's better to use the method SPS.pickedParticle(pickingInfo) rather than using directly this array.
         * Please read : https://doc.babylonjs.com/features/featuresDeepDive/particles/solid_particle_system/picking_sps
         */
        pickedBySubMesh: {
            idx: number;
            faceId: number;
        }[][];
        /**
         * This array is populated when `enableDepthSort` is set to true.
         * Each element of this array is an instance of the class DepthSortedParticle.
         */
        depthSortedParticles: DepthSortedParticle[];
        /**
         * If the particle intersection must be computed only with the bounding sphere (no bounding box computation, so faster). (Internal use only)
         * @internal
         */
        _bSphereOnly: boolean;
        /**
         * A number to multiply the bounding sphere radius by in order to reduce it for instance. (Internal use only)
         * @internal
         */
        _bSphereRadiusFactor: number;
        protected _scene: Scene;
        protected _positions: number[];
        protected _indices: number[];
        protected _normals: number[];
        protected _colors: number[];
        protected _uvs: number[];
        protected _indices32: IndicesArray;
        protected _positions32: Float32Array;
        protected _normals32: Float32Array;
        protected _fixedNormal32: Float32Array;
        protected _colors32: Float32Array;
        protected _uvs32: Float32Array;
        protected _index: number;
        protected _updatable: boolean;
        protected _pickable: boolean;
        protected _isVisibilityBoxLocked: boolean;
        protected _alwaysVisible: boolean;
        protected _depthSort: boolean;
        protected _expandable: boolean;
        protected _shapeCounter: number;
        protected _copy: SolidParticle;
        protected _color: Color4;
        protected _computeParticleColor: boolean;
        protected _computeParticleTexture: boolean;
        protected _computeParticleRotation: boolean;
        protected _computeParticleVertex: boolean;
        protected _computeBoundingBox: boolean;
        protected _autoFixFaceOrientation: boolean;
        protected _depthSortParticles: boolean;
        protected _camera: TargetCamera;
        protected _mustUnrotateFixedNormals: boolean;
        protected _particlesIntersect: boolean;
        protected _needs32Bits: boolean;
        protected _isNotBuilt: boolean;
        protected _lastParticleId: number;
        protected _idxOfId: number[];
        protected _multimaterialEnabled: boolean;
        protected _useModelMaterial: boolean;
        protected _indicesByMaterial: number[];
        protected _materialIndexes: number[];
        protected _depthSortFunction: (p1: DepthSortedParticle, p2: DepthSortedParticle) => number;
        protected _materialSortFunction: (p1: DepthSortedParticle, p2: DepthSortedParticle) => number;
        protected _materials: Material[];
        protected _multimaterial: MultiMaterial;
        protected _materialIndexesById: any;
        protected _defaultMaterial: Material;
        protected _autoUpdateSubMeshes: boolean;
        protected _tmpVertex: SolidParticleVertex;
        protected _recomputeInvisibles: boolean;
        /**
         * Creates a SPS (Solid Particle System) object.
         * @param name (String) is the SPS name, this will be the underlying mesh name.
         * @param scene (Scene) is the scene in which the SPS is added.
         * @param options defines the options of the sps e.g.
         * * updatable (optional boolean, default true) : if the SPS must be updatable or immutable.
         * * isPickable (optional boolean, default false) : if the solid particles must be pickable.
         * * enableDepthSort (optional boolean, default false) : if the solid particles must be sorted in the geometry according to their distance to the camera.
         * * useModelMaterial (optional boolean, default false) : if the model materials must be used to create the SPS multimaterial. This enables the multimaterial supports of the SPS.
         * * enableMultiMaterial (optional boolean, default false) : if the solid particles can be given different materials.
         * * expandable (optional boolean, default false) : if particles can still be added after the initial SPS mesh creation.
         * * particleIntersection (optional boolean, default false) : if the solid particle intersections must be computed.
         * * boundingSphereOnly (optional boolean, default false) : if the particle intersection must be computed only with the bounding sphere (no bounding box computation, so faster).
         * * bSphereRadiusFactor (optional float, default 1.0) : a number to multiply the bounding sphere radius by in order to reduce it for instance.
         * * computeBoundingBox (optional boolean, default false): if the bounding box of the entire SPS will be computed (for occlusion detection, for example). If it is false, the bounding box will be the bounding box of the first particle.
         * * autoFixFaceOrientation (optional boolean, default false): if the particle face orientations will be flipped for transformations that change orientation (scale (-1, 1, 1), for example)
         * @param options.updatable
         * @param options.isPickable
         * @param options.enableDepthSort
         * @param options.particleIntersection
         * @param options.boundingSphereOnly
         * @param options.bSphereRadiusFactor
         * @param options.expandable
         * @param options.useModelMaterial
         * @param options.enableMultiMaterial
         * @param options.computeBoundingBox
         * @param options.autoFixFaceOrientation
         * @example bSphereRadiusFactor = 1.0 / Math.sqrt(3.0) => the bounding sphere exactly matches a spherical mesh.
         */
        constructor(name: string, scene: Scene, options?: {
            updatable?: boolean;
            isPickable?: boolean;
            enableDepthSort?: boolean;
            particleIntersection?: boolean;
            boundingSphereOnly?: boolean;
            bSphereRadiusFactor?: number;
            expandable?: boolean;
            useModelMaterial?: boolean;
            enableMultiMaterial?: boolean;
            computeBoundingBox?: boolean;
            autoFixFaceOrientation?: boolean;
        });
        /**
         * Builds the SPS underlying mesh. Returns a standard Mesh.
         * If no model shape was added to the SPS, the returned mesh is just a single triangular plane.
         * @returns the created mesh
         */
        buildMesh(): Mesh;
        /**
         * Digests the mesh and generates as many solid particles in the system as wanted. Returns the SPS.
         * These particles will have the same geometry than the mesh parts and will be positioned at the same localisation than the mesh original places.
         * Thus the particles generated from `digest()` have their property `position` set yet.
         * @param mesh ( Mesh ) is the mesh to be digested
         * @param options {facetNb} (optional integer, default 1) is the number of mesh facets per particle, this parameter is overridden by the parameter `number` if any
         * {delta} (optional integer, default 0) is the random extra number of facets per particle , each particle will have between `facetNb` and `facetNb + delta` facets
         * {number} (optional positive integer) is the wanted number of particles : each particle is built with `mesh_total_facets / number` facets
         * {storage} (optional existing array) is an array where the particles will be stored for a further use instead of being inserted in the SPS.
         * @param options.facetNb
         * @param options.number
         * @param options.delta
         * @param options.storage
         * @returns the current SPS
         */
        digest(mesh: Mesh, options?: {
            facetNb?: number;
            number?: number;
            delta?: number;
            storage?: [];
        }): SolidParticleSystem;
        /**
         * Unrotate the fixed normals in case the mesh was built with pre-rotated particles, ex : use of positionFunction in addShape()
         * @internal
         */
        protected _unrotateFixedNormals(): void;
        /**
         * Resets the temporary working copy particle
         * @internal
         */
        protected _resetCopy(): void;
        /**
         * Inserts the shape model geometry in the global SPS mesh by updating the positions, indices, normals, colors, uvs arrays
         * @param p the current index in the positions array to be updated
         * @param ind the current index in the indices array
         * @param shape a Vector3 array, the shape geometry
         * @param positions the positions array to be updated
         * @param meshInd the shape indices array
         * @param indices the indices array to be updated
         * @param meshUV the shape uv array
         * @param uvs the uv array to be updated
         * @param meshCol the shape color array
         * @param colors the color array to be updated
         * @param meshNor the shape normals array
         * @param normals the normals array to be updated
         * @param idx the particle index
         * @param idxInShape the particle index in its shape
         * @param options the addShape() method  passed options
         * @param model
         * @model the particle model
         * @internal
         */
        protected _meshBuilder(p: number, ind: number, shape: Vector3[], positions: number[], meshInd: IndicesArray, indices: number[], meshUV: number[] | Float32Array, uvs: number[], meshCol: number[] | Float32Array, colors: number[], meshNor: number[] | Float32Array, normals: number[], idx: number, idxInShape: number, options: any, model: ModelShape): SolidParticle;
        /**
         * Returns a shape Vector3 array from positions float array
         * @param positions float array
         * @returns a vector3 array
         * @internal
         */
        protected _posToShape(positions: number[] | Float32Array): Vector3[];
        /**
         * Returns a shapeUV array from a float uvs (array deep copy)
         * @param uvs as a float array
         * @returns a shapeUV array
         * @internal
         */
        protected _uvsToShapeUV(uvs: number[] | Float32Array): number[];
        /**
         * Adds a new particle object in the particles array
         * @param idx particle index in particles array
         * @param id particle id
         * @param idxpos positionIndex : the starting index of the particle vertices in the SPS "positions" array
         * @param idxind indiceIndex : he starting index of the particle indices in the SPS "indices" array
         * @param model particle ModelShape object
         * @param shapeId model shape identifier
         * @param idxInShape index of the particle in the current model
         * @param bInfo model bounding info object
         * @param storage target storage array, if any
         * @internal
         */
        protected _addParticle(idx: number, id: number, idxpos: number, idxind: number, model: ModelShape, shapeId: number, idxInShape: number, bInfo?: Nullable<BoundingInfo>, storage?: Nullable<[]>): SolidParticle;
        /**
         * Adds some particles to the SPS from the model shape. Returns the shape id.
         * Please read the doc : https://doc.babylonjs.com/features/featuresDeepDive/particles/solid_particle_system/immutable_sps
         * @param mesh is any Mesh object that will be used as a model for the solid particles.
         * @param nb (positive integer) the number of particles to be created from this model
         * @param options {positionFunction} is an optional javascript function to called for each particle on SPS creation.
         * {vertexFunction} is an optional javascript function to called for each vertex of each particle on SPS creation
         * {storage} (optional existing array) is an array where the particles will be stored for a further use instead of being inserted in the SPS.
         * @param options.positionFunction
         * @param options.vertexFunction
         * @param options.storage
         * @returns the number of shapes in the system
         */
        addShape(mesh: Mesh, nb: number, options?: {
            positionFunction?: any;
            vertexFunction?: any;
            storage?: [];
        }): number;
        /**
         * Rebuilds a particle back to its just built status : if needed, recomputes the custom positions and vertices
         * @internal
         */
        protected _rebuildParticle(particle: SolidParticle, reset?: boolean): void;
        /**
         * Rebuilds the whole mesh and updates the VBO : custom positions and vertices are recomputed if needed.
         * @param reset boolean, default false : if the particles must be reset at position and rotation zero, scaling 1, color white, initial UVs and not parented.
         * @returns the SPS.
         */
        rebuildMesh(reset?: boolean): SolidParticleSystem;
        /** Removes the particles from the start-th to the end-th included from an expandable SPS (required).
         *  Returns an array with the removed particles.
         *  If the number of particles to remove is lower than zero or greater than the global remaining particle number, then an empty array is returned.
         *  The SPS can't be empty so at least one particle needs to remain in place.
         *  Under the hood, the VertexData array, so the VBO buffer, is recreated each call.
         * @param start index of the first particle to remove
         * @param end index of the last particle to remove (included)
         * @returns an array populated with the removed particles
         */
        removeParticles(start: number, end: number): SolidParticle[];
        /**
         * Inserts some pre-created particles in the solid particle system so that they can be managed by setParticles().
         * @param solidParticleArray an array populated with Solid Particles objects
         * @returns the SPS
         */
        insertParticlesFromArray(solidParticleArray: SolidParticle[]): SolidParticleSystem;
        /**
         * Creates a new particle and modifies the SPS mesh geometry :
         * - calls _meshBuilder() to increase the SPS mesh geometry step by step
         * - calls _addParticle() to populate the particle array
         * factorized code from addShape() and insertParticlesFromArray()
         * @param idx particle index in the particles array
         * @param i particle index in its shape
         * @param modelShape particle ModelShape object
         * @param shape shape vertex array
         * @param meshInd shape indices array
         * @param meshUV shape uv array
         * @param meshCol shape color array
         * @param meshNor shape normals array
         * @param bbInfo shape bounding info
         * @param storage target particle storage
         * @param options
         * @options addShape() passed options
         * @internal
         */
        protected _insertNewParticle(idx: number, i: number, modelShape: ModelShape, shape: Vector3[], meshInd: IndicesArray, meshUV: number[] | Float32Array, meshCol: number[] | Float32Array, meshNor: number[] | Float32Array, bbInfo: Nullable<BoundingInfo>, storage: Nullable<[]>, options: any): Nullable<SolidParticle>;
        /**
         *  Sets all the particles : this method actually really updates the mesh according to the particle positions, rotations, colors, textures, etc.
         *  This method calls `updateParticle()` for each particle of the SPS.
         *  For an animated SPS, it is usually called within the render loop.
         * This methods does nothing if called on a non updatable or not yet built SPS. Example : buildMesh() not called after having added or removed particles from an expandable SPS.
         * @param start The particle index in the particle array where to start to compute the particle property values _(default 0)_
         * @param end The particle index in the particle array where to stop to compute the particle property values _(default nbParticle - 1)_
         * @param update If the mesh must be finally updated on this call after all the particle computations _(default true)_
         * @returns the SPS.
         */
        setParticles(start?: number, end?: number, update?: boolean): SolidParticleSystem;
        /**
         * Disposes the SPS.
         */
        dispose(): void;
        /** Returns an object {idx: number faceId: number} for the picked particle from the passed pickingInfo object.
         * idx is the particle index in the SPS
         * faceId is the picked face index counted within this particle.
         * Returns null if the pickInfo can't identify a picked particle.
         * @param pickingInfo (PickingInfo object)
         * @returns {idx: number, faceId: number} or null
         */
        pickedParticle(pickingInfo: PickingInfo): Nullable<{
            idx: number;
            faceId: number;
        }>;
        /**
         * Returns a SolidParticle object from its identifier : particle.id
         * @param id (integer) the particle Id
         * @returns the searched particle or null if not found in the SPS.
         */
        getParticleById(id: number): Nullable<SolidParticle>;
        /**
         * Returns a new array populated with the particles having the passed shapeId.
         * @param shapeId (integer) the shape identifier
         * @returns a new solid particle array
         */
        getParticlesByShapeId(shapeId: number): SolidParticle[];
        /**
         * Populates the passed array "ref" with the particles having the passed shapeId.
         * @param shapeId the shape identifier
         * @returns the SPS
         * @param ref
         */
        getParticlesByShapeIdToRef(shapeId: number, ref: SolidParticle[]): SolidParticleSystem;
        /**
         * Computes the required SubMeshes according the materials assigned to the particles.
         * @returns the solid particle system.
         * Does nothing if called before the SPS mesh is built.
         */
        computeSubMeshes(): SolidParticleSystem;
        /**
         * Sorts the solid particles by material when MultiMaterial is enabled.
         * Updates the indices32 array.
         * Updates the indicesByMaterial array.
         * Updates the mesh indices array.
         * @returns the SPS
         * @internal
         */
        protected _sortParticlesByMaterial(): SolidParticleSystem;
        /**
         * Sets the material indexes by id materialIndexesById[id] = materialIndex
         * @internal
         */
        protected _setMaterialIndexesById(): void;
        /**
         * Returns an array with unique values of Materials from the passed array
         * @param array the material array to be checked and filtered
         * @internal
         */
        protected _filterUniqueMaterialId(array: Material[]): Material[];
        /**
         * Sets a new Standard Material as _defaultMaterial if not already set.
         * @internal
         */
        protected _setDefaultMaterial(): Material;
        /**
         * Visibility helper : Recomputes the visible size according to the mesh bounding box
         * doc : https://doc.babylonjs.com/features/featuresDeepDive/particles/solid_particle_system/sps_visibility
         * @returns the SPS.
         */
        refreshVisibleSize(): SolidParticleSystem;
        /**
         * Visibility helper : Sets the size of a visibility box, this sets the underlying mesh bounding box.
         * @param size the size (float) of the visibility box
         * note : this doesn't lock the SPS mesh bounding box.
         * doc : https://doc.babylonjs.com/features/featuresDeepDive/particles/solid_particle_system/sps_visibility
         */
        setVisibilityBox(size: number): void;
        /**
         * Gets whether the SPS as always visible or not
         * doc : https://doc.babylonjs.com/features/featuresDeepDive/particles/solid_particle_system/sps_visibility
         */
        get isAlwaysVisible(): boolean;
        /**
         * Sets the SPS as always visible or not
         * doc : https://doc.babylonjs.com/features/featuresDeepDive/particles/solid_particle_system/sps_visibility
         */
        set isAlwaysVisible(val: boolean);
        /**
         * Sets the SPS visibility box as locked or not. This enables/disables the underlying mesh bounding box updates.
         * doc : https://doc.babylonjs.com/features/featuresDeepDive/particles/solid_particle_system/sps_visibility
         */
        set isVisibilityBoxLocked(val: boolean);
        /**
         * Gets if the SPS visibility box as locked or not. This enables/disables the underlying mesh bounding box updates.
         * doc : https://doc.babylonjs.com/features/featuresDeepDive/particles/solid_particle_system/sps_visibility
         */
        get isVisibilityBoxLocked(): boolean;
        /**
         * Tells to `setParticles()` to compute the particle rotations or not.
         * Default value : true. The SPS is faster when it's set to false.
         * Note : the particle rotations aren't stored values, so setting `computeParticleRotation` to false will prevents the particle to rotate.
         */
        set computeParticleRotation(val: boolean);
        /**
         * Tells to `setParticles()` to compute the particle colors or not.
         * Default value : true. The SPS is faster when it's set to false.
         * Note : the particle colors are stored values, so setting `computeParticleColor` to false will keep yet the last colors set.
         */
        set computeParticleColor(val: boolean);
        set computeParticleTexture(val: boolean);
        /**
         * Tells to `setParticles()` to call the vertex function for each vertex of each particle, or not.
         * Default value : false. The SPS is faster when it's set to false.
         * Note : the particle custom vertex positions aren't stored values.
         */
        set computeParticleVertex(val: boolean);
        /**
         * Tells to `setParticles()` to compute or not the mesh bounding box when computing the particle positions.
         */
        set computeBoundingBox(val: boolean);
        /**
         * Tells to `setParticles()` to sort or not the distance between each particle and the camera.
         * Skipped when `enableDepthSort` is set to `false` (default) at construction time.
         * Default : `true`
         */
        set depthSortParticles(val: boolean);
        /**
         * Gets if `setParticles()` computes the particle rotations or not.
         * Default value : true. The SPS is faster when it's set to false.
         * Note : the particle rotations aren't stored values, so setting `computeParticleRotation` to false will prevents the particle to rotate.
         */
        get computeParticleRotation(): boolean;
        /**
         * Gets if `setParticles()` computes the particle colors or not.
         * Default value : true. The SPS is faster when it's set to false.
         * Note : the particle colors are stored values, so setting `computeParticleColor` to false will keep yet the last colors set.
         */
        get computeParticleColor(): boolean;
        /**
         * Gets if `setParticles()` computes the particle textures or not.
         * Default value : true. The SPS is faster when it's set to false.
         * Note : the particle textures are stored values, so setting `computeParticleTexture` to false will keep yet the last colors set.
         */
        get computeParticleTexture(): boolean;
        /**
         * Gets if `setParticles()` calls the vertex function for each vertex of each particle, or not.
         * Default value : false. The SPS is faster when it's set to false.
         * Note : the particle custom vertex positions aren't stored values.
         */
        get computeParticleVertex(): boolean;
        /**
         * Gets if `setParticles()` computes or not the mesh bounding box when computing the particle positions.
         */
        get computeBoundingBox(): boolean;
        /**
         * Gets if `setParticles()` sorts or not the distance between each particle and the camera.
         * Skipped when `enableDepthSort` is set to `false` (default) at construction time.
         * Default : `true`
         */
        get depthSortParticles(): boolean;
        /**
         * Gets if the SPS is created as expandable at construction time.
         * Default : `false`
         */
        get expandable(): boolean;
        /**
         * Gets if the SPS supports the Multi Materials
         */
        get multimaterialEnabled(): boolean;
        /**
         * Gets if the SPS uses the model materials for its own multimaterial.
         */
        get useModelMaterial(): boolean;
        /**
         * The SPS used material array.
         */
        get materials(): Material[];
        /**
         * Sets the SPS MultiMaterial from the passed materials.
         * Note : the passed array is internally copied and not used then by reference.
         * @param materials an array of material objects. This array indexes are the materialIndex values of the particles.
         */
        setMultiMaterial(materials: Material[]): void;
        /**
         * The SPS computed multimaterial object
         */
        get multimaterial(): MultiMaterial;
        set multimaterial(mm: MultiMaterial);
        /**
         * If the subMeshes must be updated on the next call to setParticles()
         */
        get autoUpdateSubMeshes(): boolean;
        set autoUpdateSubMeshes(val: boolean);
        /**
         * This function does nothing. It may be overwritten to set all the particle first values.
         * The SPS doesn't call this function, you may have to call it by your own.
         * doc : https://doc.babylonjs.com/features/featuresDeepDive/particles/solid_particle_system/manage_sps_particles
         */
        initParticles(): void;
        /**
         * This function does nothing. It may be overwritten to recycle a particle.
         * The SPS doesn't call this function, you may have to call it by your own.
         * doc : https://doc.babylonjs.com/features/featuresDeepDive/particles/solid_particle_system/manage_sps_particles
         * @param particle The particle to recycle
         * @returns the recycled particle
         */
        recycleParticle(particle: SolidParticle): SolidParticle;
        /**
         * Updates a particle : this function should  be overwritten by the user.
         * It is called on each particle by `setParticles()`. This is the place to code each particle behavior.
         * doc : https://doc.babylonjs.com/features/featuresDeepDive/particles/solid_particle_system/manage_sps_particles
         * @example : just set a particle position or velocity and recycle conditions
         * @param particle The particle to update
         * @returns the updated particle
         */
        updateParticle(particle: SolidParticle): SolidParticle;
        /**
         * Updates a vertex of a particle : it can be overwritten by the user.
         * This will be called on each vertex particle by `setParticles()` if `computeParticleVertex` is set to true only.
         * @param particle the current particle
         * @param vertex the current vertex of the current particle : a SolidParticleVertex object
         * @param pt the index of the current vertex in the particle shape
         * doc : https://doc.babylonjs.com/features/featuresDeepDive/particles/solid_particle_system/sps_vertices
         * @example : just set a vertex particle position or color
         * @returns the sps
         */
        updateParticleVertex(particle: SolidParticle, vertex: SolidParticleVertex, pt: number): SolidParticleSystem;
        /**
         * This will be called before any other treatment by `setParticles()` and will be passed three parameters.
         * This does nothing and may be overwritten by the user.
         * @param start the particle index in the particle array where to stop to iterate, same than the value passed to setParticle()
         * @param stop the particle index in the particle array where to stop to iterate, same than the value passed to setParticle()
         * @param update the boolean update value actually passed to setParticles()
         */
        beforeUpdateParticles(start?: number, stop?: number, update?: boolean): void;
        /**
         * This will be called  by `setParticles()` after all the other treatments and just before the actual mesh update.
         * This will be passed three parameters.
         * This does nothing and may be overwritten by the user.
         * @param start the particle index in the particle array where to stop to iterate, same than the value passed to setParticle()
         * @param stop the particle index in the particle array where to stop to iterate, same than the value passed to setParticle()
         * @param update the boolean update value actually passed to setParticles()
         */
        afterUpdateParticles(start?: number, stop?: number, update?: boolean): void;
    }


    /**
     * Type of sub emitter
     */
    export enum SubEmitterType {
        /**
         * Attached to the particle over it's lifetime
         */
        ATTACHED = 0,
        /**
         * Created when the particle dies
         */
        END = 1
    }
    /**
     * Sub emitter class used to emit particles from an existing particle
     */
    export class SubEmitter {
        /**
         * the particle system to be used by the sub emitter
         */
        particleSystem: ParticleSystem;
        /**
         * Type of the submitter (Default: END)
         */
        type: SubEmitterType;
        /**
         * If the particle should inherit the direction from the particle it's attached to. (+Y will face the direction the particle is moving) (Default: false)
         * Note: This only is supported when using an emitter of type Mesh
         */
        inheritDirection: boolean;
        /**
         * How much of the attached particles speed should be added to the sub emitted particle (default: 0)
         */
        inheritedVelocityAmount: number;
        /**
         * Creates a sub emitter
         * @param particleSystem the particle system to be used by the sub emitter
         */
        constructor(
        /**
         * the particle system to be used by the sub emitter
         */
        particleSystem: ParticleSystem);
        /**
         * Clones the sub emitter
         * @returns the cloned sub emitter
         */
        clone(): SubEmitter;
        /**
         * Serialize current object to a JSON object
         * @param serializeTexture defines if the texture must be serialized as well
         * @returns the serialized object
         */
        serialize(serializeTexture?: boolean): any;
        /**
         * @internal
         */
        static _ParseParticleSystem(system: any, sceneOrEngine: Scene | ThinEngine, rootUrl: string, doNotStart?: boolean): ParticleSystem;
        /**
         * Creates a new SubEmitter from a serialized JSON version
         * @param serializationObject defines the JSON object to read from
         * @param sceneOrEngine defines the hosting scene or the hosting engine
         * @param rootUrl defines the rootUrl for data loading
         * @returns a new SubEmitter
         */
        static Parse(serializationObject: any, sceneOrEngine: Scene | ThinEngine, rootUrl: string): SubEmitter;
        /** Release associated resources */
        dispose(): void;
    }


    /** @internal */
    export class WebGL2ParticleSystem implements IGPUParticleSystemPlatform {
        private _parent;
        private _engine;
        private _updateEffect;
        private _updateEffectOptions;
        private _renderVAO;
        private _updateVAO;
        readonly alignDataInBuffer = false;
        constructor(parent: GPUParticleSystem, engine: ThinEngine);
        isUpdateBufferCreated(): boolean;
        isUpdateBufferReady(): boolean;
        createUpdateBuffer(defines: string): UniformBufferEffectCommonAccessor;
        createVertexBuffers(updateBuffer: Buffer, renderVertexBuffers: {
            [key: string]: VertexBuffer;
        }): void;
        createParticleBuffer(data: number[]): DataArray | DataBuffer;
        bindDrawBuffers(index: number): void;
        preUpdateParticleBuffer(): void;
        updateParticleBuffer(index: number, targetBuffer: Buffer, currentActiveCount: number): void;
        releaseBuffers(): void;
        releaseVertexBuffers(): void;
        private _createUpdateVAO;
    }




    /**
     * Interface used to define a physics engine
     * @see https://doc.babylonjs.com/features/featuresDeepDive/physics/usingPhysicsEngine
     */
    export interface IPhysicsEngine {
        /**
         * Gets the gravity vector used by the simulation
         */
        gravity: Vector3;
        /**
         *
         */
        getPluginVersion(): number;
        /**
         * Sets the gravity vector used by the simulation
         * @param gravity defines the gravity vector to use
         */
        setGravity(gravity: Vector3): void;
        /**
         * Set the time step of the physics engine.
         * Default is 1/60.
         * To slow it down, enter 1/600 for example.
         * To speed it up, 1/30
         * @param newTimeStep the new timestep to apply to this world.
         */
        setTimeStep(newTimeStep: number): void;
        /**
         * Get the time step of the physics engine.
         * @returns the current time step
         */
        getTimeStep(): number;
        /**
         * Set the sub time step of the physics engine.
         * Default is 0 meaning there is no sub steps
         * To increase physics resolution precision, set a small value (like 1 ms)
         * @param subTimeStep defines the new sub timestep used for physics resolution.
         */
        setSubTimeStep(subTimeStep: number): void;
        /**
         * Get the sub time step of the physics engine.
         * @returns the current sub time step
         */
        getSubTimeStep(): number;
        /**
         * Release all resources
         */
        dispose(): void;
        /**
         * Gets the name of the current physics plugin
         * @returns the name of the plugin
         */
        getPhysicsPluginName(): string;
        /**
         * Gets the current plugin used to run the simulation
         * @returns current plugin
         */
        getPhysicsPlugin(): BABYLON.IPhysicsEnginePlugin | IPhysicsEnginePluginV2 | null;
        /**
         * Does a raycast in the physics world
         * @param from when should the ray start?
         * @param to when should the ray end?
         * @returns PhysicsRaycastResult
         */
        raycast(from: Vector3, to: Vector3): PhysicsRaycastResult;
        /**
         * Called by the scene. No need to call it.
         * @param delta defines the timespan between frames
         */
        _step(delta: number): void;
    }


        /**
         *
         */
        interface Scene {
            /** @internal (Backing field) */
            _physicsEngine: Nullable<IPhysicsEngine>;
            /** @internal */
            _physicsTimeAccumulator: number;
            /**
             * Gets the current physics engine
             * @returns a IPhysicsEngine or null if none attached
             */
            getPhysicsEngine(): Nullable<IPhysicsEngine>;
            /**
             * Enables physics to the current scene
             * @param gravity defines the scene's gravity for the physics engine. defaults to real earth gravity : (0, -9.81, 0)
             * @param plugin defines the physics engine to be used. defaults to CannonJS.
             * @returns a boolean indicating if the physics engine was initialized
             */
            enablePhysics(gravity?: Nullable<Vector3>, plugin?: BABYLON.IPhysicsEnginePlugin | IPhysicsEnginePluginV2): boolean;
            /**
             * Disables and disposes the physics engine associated with the scene
             */
            disablePhysicsEngine(): void;
            /**
             * Gets a boolean indicating if there is an active physics engine
             * @returns a boolean indicating if there is an active physics engine
             */
            isPhysicsEnabled(): boolean;
            /**
             * Deletes a physics compound impostor
             * @param compound defines the compound to delete
             */
            deleteCompoundImpostor(compound: any): void;
            /**
             * An event triggered when physic simulation is about to be run
             */
            onBeforePhysicsObservable: Observable<Scene>;
            /**
             * An event triggered when physic simulation has been done
             */
            onAfterPhysicsObservable: Observable<Scene>;
        }
    /**
     * Defines the physics engine scene component responsible to manage a physics engine
     */
    export class PhysicsEngineSceneComponent implements ISceneComponent {
        /**
         * The component name helpful to identify the component in the list of scene components.
         */
        readonly name = "PhysicsEngine";
        /**
         * The scene the component belongs to.
         */
        scene: Scene;
        /**
         * Creates a new instance of the component for the given scene
         * @param scene Defines the scene to register the component in
         */
        constructor(scene: Scene);
        /**
         * Registers the component in a given scene
         */
        register(): void;
        /**
         * Rebuilds the elements related to this component in case of
         * context lost for instance.
         */
        rebuild(): void;
        /**
         * Disposes the component and the associated resources
         */
        dispose(): void;
    }






    /**
     * A helper for physics simulations
     * @see https://doc.babylonjs.com/features/featuresDeepDive/physics/usingPhysicsEngine#further-functionality-of-the-impostor-class
     */
    export class PhysicsHelper {
        private _scene;
        private _physicsEngine;
        private _hitData;
        /**
         * Initializes the Physics helper
         * @param scene Babylon.js scene
         */
        constructor(scene: Scene);
        /**
         * Applies a radial explosion impulse
         * @param origin the origin of the explosion
         * @param radiusOrEventOptions the radius or the options of radial explosion
         * @param strength the explosion strength
         * @param falloff possible options: Constant & Linear. Defaults to Constant
         * @returns A physics radial explosion event, or null
         */
        applyRadialExplosionImpulse(origin: Vector3, radiusOrEventOptions: number | PhysicsRadialExplosionEventOptions, strength?: number, falloff?: PhysicsRadialImpulseFalloff): Nullable<PhysicsRadialExplosionEvent>;
        /**
         * Applies a radial explosion force
         * @param origin the origin of the explosion
         * @param radiusOrEventOptions the radius or the options of radial explosion
         * @param strength the explosion strength
         * @param falloff possible options: Constant & Linear. Defaults to Constant
         * @returns A physics radial explosion event, or null
         */
        applyRadialExplosionForce(origin: Vector3, radiusOrEventOptions: number | PhysicsRadialExplosionEventOptions, strength?: number, falloff?: PhysicsRadialImpulseFalloff): Nullable<PhysicsRadialExplosionEvent>;
        /**
         * Creates a gravitational field
         * @param origin the origin of the gravitational field
         * @param radiusOrEventOptions the radius or the options of radial gravitational field
         * @param strength the gravitational field strength
         * @param falloff possible options: Constant & Linear. Defaults to Constant
         * @returns A physics gravitational field event, or null
         */
        gravitationalField(origin: Vector3, radiusOrEventOptions: number | PhysicsRadialExplosionEventOptions, strength?: number, falloff?: PhysicsRadialImpulseFalloff): Nullable<PhysicsGravitationalFieldEvent>;
        /**
         * Creates a physics updraft event
         * @param origin the origin of the updraft
         * @param radiusOrEventOptions the radius or the options of the updraft
         * @param strength the strength of the updraft
         * @param height the height of the updraft
         * @param updraftMode possible options: Center & Perpendicular. Defaults to Center
         * @returns A physics updraft event, or null
         */
        updraft(origin: Vector3, radiusOrEventOptions: number | PhysicsUpdraftEventOptions, strength?: number, height?: number, updraftMode?: PhysicsUpdraftMode): Nullable<PhysicsUpdraftEvent>;
        /**
         * Creates a physics vortex event
         * @param origin the of the vortex
         * @param radiusOrEventOptions the radius or the options of the vortex
         * @param strength the strength of the vortex
         * @param height   the height of the vortex
         * @returns a Physics vortex event, or null
         * A physics vortex event or null
         */
        vortex(origin: Vector3, radiusOrEventOptions: number | PhysicsVortexEventOptions, strength?: number, height?: number): Nullable<PhysicsVortexEvent>;
        private _copyPhysicsHitData;
    }
    /**
     * Represents a physics radial explosion event
     */
    class PhysicsRadialExplosionEvent {
        private _scene;
        private _options;
        private _sphere;
        private _dataFetched;
        /**
         * Initializes a radial explosion event
         * @param _scene BabylonJS scene
         * @param _options The options for the vortex event
         */
        constructor(_scene: Scene, _options: PhysicsRadialExplosionEventOptions);
        /**
         * Returns the data related to the radial explosion event (sphere).
         * @returns The radial explosion event data
         */
        getData(): PhysicsRadialExplosionEventData;
        private _getHitData;
        /**
         * Returns the force and contact point of the body or false, if the body is not affected by the force/impulse.
         * @param body A physics body
         * @param origin the origin of the explosion
         * @returns A physics force and contact point, or null
         */
        getBodyHitData(body: PhysicsBody, origin: Vector3, data: PhysicsHitData): boolean;
        /**
         * Returns the force and contact point of the impostor or false, if the impostor is not affected by the force/impulse.
         * @param impostor A physics imposter
         * @param origin the origin of the explosion
         * @returns A physics force and contact point, or null
         */
        getImpostorHitData(impostor: PhysicsImpostor, origin: Vector3, data: PhysicsHitData): boolean;
        /**
         * Triggers affected impostors callbacks
         * @param affectedImpostorsWithData defines the list of affected impostors (including associated data)
         */
        triggerAffectedImpostorsCallback(affectedImpostorsWithData: Array<PhysicsAffectedImpostorWithData>): void;
        /**
         * Triggers affected bodies callbacks
         * @param affectedBodiesWithData defines the list of affected bodies (including associated data)
         */
        triggerAffectedBodiesCallback(affectedBodiesWithData: Array<PhysicsAffectedBodyWithData>): void;
        /**
         * Disposes the sphere.
         * @param force Specifies if the sphere should be disposed by force
         */
        dispose(force?: boolean): void;
        /*** Helpers ***/
        private _prepareSphere;
        private _intersectsWithSphere;
    }
    /**
     * Represents a gravitational field event
     */
    class PhysicsGravitationalFieldEvent {
        private _physicsHelper;
        private _scene;
        private _origin;
        private _options;
        private _tickCallback;
        private _sphere;
        private _dataFetched;
        /**
         * Initializes the physics gravitational field event
         * @param _physicsHelper A physics helper
         * @param _scene BabylonJS scene
         * @param _origin The origin position of the gravitational field event
         * @param _options The options for the vortex event
         */
        constructor(_physicsHelper: PhysicsHelper, _scene: Scene, _origin: Vector3, _options: PhysicsRadialExplosionEventOptions);
        /**
         * Returns the data related to the gravitational field event (sphere).
         * @returns A gravitational field event
         */
        getData(): PhysicsGravitationalFieldEventData;
        /**
         * Enables the gravitational field.
         */
        enable(): void;
        /**
         * Disables the gravitational field.
         */
        disable(): void;
        /**
         * Disposes the sphere.
         * @param force The force to dispose from the gravitational field event
         */
        dispose(force?: boolean): void;
        private _tick;
    }
    /**
     * Represents a physics updraft event
     */
    class PhysicsUpdraftEvent {
        private _scene;
        private _origin;
        private _options;
        private _physicsEngine;
        private _originTop;
        private _originDirection;
        private _tickCallback;
        private _cylinder;
        private _cylinderPosition;
        private _dataFetched;
        private static hitData;
        /**
         * Initializes the physics updraft event
         * @param _scene BabylonJS scene
         * @param _origin The origin position of the updraft
         * @param _options The options for the updraft event
         */
        constructor(_scene: Scene, _origin: Vector3, _options: PhysicsUpdraftEventOptions);
        /**
         * Returns the data related to the updraft event (cylinder).
         * @returns A physics updraft event
         */
        getData(): PhysicsUpdraftEventData;
        /**
         * Enables the updraft.
         */
        enable(): void;
        /**
         * Disables the updraft.
         */
        disable(): void;
        /**
         * Disposes the cylinder.
         * @param force Specifies if the updraft should be disposed by force
         */
        dispose(force?: boolean): void;
        private _getHitData;
        private _getBodyHitData;
        private _getImpostorHitData;
        private _tick;
        /*** Helpers ***/
        private _prepareCylinder;
        private _intersectsWithCylinder;
    }
    /**
     * Represents a physics vortex event
     */
    class PhysicsVortexEvent {
        private _scene;
        private _origin;
        private _options;
        private _physicsEngine;
        private _originTop;
        private _tickCallback;
        private _cylinder;
        private _cylinderPosition;
        private _dataFetched;
        private static originOnPlane;
        private static hitData;
        /**
         * Initializes the physics vortex event
         * @param _scene The BabylonJS scene
         * @param _origin The origin position of the vortex
         * @param _options The options for the vortex event
         */
        constructor(_scene: Scene, _origin: Vector3, _options: PhysicsVortexEventOptions);
        /**
         * Returns the data related to the vortex event (cylinder).
         * @returns The physics vortex event data
         */
        getData(): PhysicsVortexEventData;
        /**
         * Enables the vortex.
         */
        enable(): void;
        /**
         * Disables the cortex.
         */
        disable(): void;
        /**
         * Disposes the sphere.
         * @param force
         */
        dispose(force?: boolean): void;
        private _getHitData;
        private _getBodyHitData;
        private _getImpostorHitData;
        private _tick;
        /*** Helpers ***/
        private _prepareCylinder;
        private _intersectsWithCylinder;
    }
    /**
     * Options fot the radial explosion event
     * @see https://doc.babylonjs.com/features/featuresDeepDive/physics/usingPhysicsEngine#further-functionality-of-the-impostor-class
     */
    export class PhysicsRadialExplosionEventOptions {
        /**
         * The radius of the sphere for the radial explosion.
         */
        radius: number;
        /**
         * The strength of the explosion.
         */
        strength: number;
        /**
         * The strength of the force in correspondence to the distance of the affected object
         */
        falloff: PhysicsRadialImpulseFalloff;
        /**
         * Sphere options for the radial explosion.
         */
        sphere: {
            segments: number;
            diameter: number;
        };
        /**
         * Sphere options for the radial explosion.
         */
        affectedImpostorsCallback: (affectedImpostorsWithData: Array<PhysicsAffectedImpostorWithData>) => void;
        /**
         * Sphere options for the radial explosion.
         */
        affectedBodiesCallback: (affectedBodiesWithData: Array<PhysicsAffectedBodyWithData>) => void;
    }
    /**
     * Options fot the updraft event
     * @see https://doc.babylonjs.com/features/featuresDeepDive/physics/usingPhysicsEngine#further-functionality-of-the-impostor-class
     */
    export class PhysicsUpdraftEventOptions {
        /**
         * The radius of the cylinder for the vortex
         */
        radius: number;
        /**
         * The strength of the updraft.
         */
        strength: number;
        /**
         * The height of the cylinder for the updraft.
         */
        height: number;
        /**
         * The mode for the the updraft.
         */
        updraftMode: PhysicsUpdraftMode;
    }
    /**
     * Options fot the vortex event
     * @see https://doc.babylonjs.com/features/featuresDeepDive/physics/usingPhysicsEngine#further-functionality-of-the-impostor-class
     */
    export class PhysicsVortexEventOptions {
        /**
         * The radius of the cylinder for the vortex
         */
        radius: number;
        /**
         * The strength of the vortex.
         */
        strength: number;
        /**
         * The height of the cylinder for the vortex.
         */
        height: number;
        /**
         * At which distance, relative to the radius the centripetal forces should kick in? Range: 0-1
         */
        centripetalForceThreshold: number;
        /**
         * This multiplier determines with how much force the objects will be pushed sideways/around the vortex, when below the threshold.
         */
        centripetalForceMultiplier: number;
        /**
         * This multiplier determines with how much force the objects will be pushed sideways/around the vortex, when above the threshold.
         */
        centrifugalForceMultiplier: number;
        /**
         * This multiplier determines with how much force the objects will be pushed upwards, when in the vortex.
         */
        updraftForceMultiplier: number;
    }
    /**
     * The strength of the force in correspondence to the distance of the affected object
     * @see https://doc.babylonjs.com/features/featuresDeepDive/physics/usingPhysicsEngine#further-functionality-of-the-impostor-class
     */
    export enum PhysicsRadialImpulseFalloff {
        /** Defines that impulse is constant in strength across it's whole radius */
        Constant = 0,
        /** Defines that impulse gets weaker if it's further from the origin */
        Linear = 1
    }
    /**
     * The strength of the force in correspondence to the distance of the affected object
     * @see https://doc.babylonjs.com/features/featuresDeepDive/physics/usingPhysicsEngine#further-functionality-of-the-impostor-class
     */
    export enum PhysicsUpdraftMode {
        /** Defines that the upstream forces will pull towards the top center of the cylinder */
        Center = 0,
        /** Defines that once a impostor is inside the cylinder, it will shoot out perpendicular from the ground of the cylinder */
        Perpendicular = 1
    }
    /**
     * Interface for a physics hit data
     * @see https://doc.babylonjs.com/features/featuresDeepDive/physics/usingPhysicsEngine#further-functionality-of-the-impostor-class
     */
    export interface PhysicsHitData {
        /**
         * The force applied at the contact point
         */
        force: Vector3;
        /**
         * The contact point
         */
        contactPoint: Vector3;
        /**
         * The distance from the origin to the contact point
         */
        distanceFromOrigin: number;
    }
    /**
     * Interface for radial explosion event data
     * @see https://doc.babylonjs.com/features/featuresDeepDive/physics/usingPhysicsEngine#further-functionality-of-the-impostor-class
     */
    export interface PhysicsRadialExplosionEventData {
        /**
         * A sphere used for the radial explosion event
         */
        sphere: Mesh;
    }
    /**
     * Interface for gravitational field event data
     * @see https://doc.babylonjs.com/features/featuresDeepDive/physics/usingPhysicsEngine#further-functionality-of-the-impostor-class
     */
    export interface PhysicsGravitationalFieldEventData {
        /**
         * A sphere mesh used for the gravitational field event
         */
        sphere: Mesh;
    }
    /**
     * Interface for updraft event data
     * @see https://doc.babylonjs.com/features/featuresDeepDive/physics/usingPhysicsEngine#further-functionality-of-the-impostor-class
     */
    export interface PhysicsUpdraftEventData {
        /**
         * A cylinder used for the updraft event
         */
        cylinder: Mesh;
    }
    /**
     * Interface for vortex event data
     * @see https://doc.babylonjs.com/features/featuresDeepDive/physics/usingPhysicsEngine#further-functionality-of-the-impostor-class
     */
    export interface PhysicsVortexEventData {
        /**
         * A cylinder used for the vortex event
         */
        cylinder: Mesh;
    }
    /**
     * Interface for an affected physics impostor
     * @see https://doc.babylonjs.com/features/featuresDeepDive/physics/usingPhysicsEngine#further-functionality-of-the-impostor-class
     */
    export interface PhysicsAffectedImpostorWithData {
        /**
         * The impostor affected by the effect
         */
        impostor: PhysicsImpostor;
        /**
         * The data about the hit/force from the explosion
         */
        hitData: PhysicsHitData;
    }
    /**
     * Interface for an affected physics body
     * @see
     */
    export interface PhysicsAffectedBodyWithData {
        /**
         * The impostor affected by the effect
         */
        body: PhysicsBody;
        /**
         * The data about the hit/force from the explosion
         */
        hitData: PhysicsHitData;
    }






    /**
     * Holds the data for the raycast result
     * @see https://doc.babylonjs.com/features/featuresDeepDive/physics/usingPhysicsEngine
     */
    export class PhysicsRaycastResult {
        private _hasHit;
        private _hitDistance;
        private _hitNormalWorld;
        private _hitPointWorld;
        private _rayFromWorld;
        private _rayToWorld;
        /**
         * Gets if there was a hit
         */
        get hasHit(): boolean;
        /**
         * Gets the distance from the hit
         */
        get hitDistance(): number;
        /**
         * Gets the hit normal/direction in the world
         */
        get hitNormalWorld(): Vector3;
        /**
         * Gets the hit point in the world
         */
        get hitPointWorld(): Vector3;
        /**
         * Gets the ray "start point" of the ray in the world
         */
        get rayFromWorld(): Vector3;
        /**
         * Gets the ray "end point" of the ray in the world
         */
        get rayToWorld(): Vector3;
        /**
         * Sets the hit data (normal & point in world space)
         * @param hitNormalWorld defines the normal in world space
         * @param hitPointWorld defines the point in world space
         */
        setHitData(hitNormalWorld: IXYZ, hitPointWorld: IXYZ): void;
        /**
         * Sets the distance from the start point to the hit point
         * @param distance
         */
        setHitDistance(distance: number): void;
        /**
         * Calculates the distance manually
         */
        calculateHitDistance(): void;
        /**
         * Resets all the values to default
         * @param from The from point on world space
         * @param to The to point on world space
         */
        reset(from?: Vector3, to?: Vector3): void;
    }
    /**
     * Interface for the size containing width and height
     */
    interface IXYZ {
        /**
         * X
         */
        x: number;
        /**
         * Y
         */
        y: number;
        /**
         * Z
         */
        z: number;
    }










    /**
     * Interface used to describe a physics joint
     */
    export interface PhysicsImpostorJoint {
        /** Defines the main impostor to which the joint is linked */
        mainImpostor: PhysicsImpostor;
        /** Defines the impostor that is connected to the main impostor using this joint */
        connectedImpostor: PhysicsImpostor;
        /** Defines the joint itself */
        joint: PhysicsJoint;
    }
    /** @internal */
    export interface IPhysicsEnginePlugin {
        /**
         *
         */
        world: any;
        /**
         *
         */
        name: string;
        setGravity(gravity: Vector3): void;
        setTimeStep(timeStep: number): void;
        getTimeStep(): number;
        executeStep(delta: number, impostors: Array<PhysicsImpostor>): void;
        getPluginVersion(): number;
        applyImpulse(impostor: PhysicsImpostor, force: Vector3, contactPoint: Vector3): void;
        applyForce(impostor: PhysicsImpostor, force: Vector3, contactPoint: Vector3): void;
        generatePhysicsBody(impostor: PhysicsImpostor): void;
        removePhysicsBody(impostor: PhysicsImpostor): void;
        generateJoint(joint: PhysicsImpostorJoint): void;
        removeJoint(joint: PhysicsImpostorJoint): void;
        isSupported(): boolean;
        setTransformationFromPhysicsBody(impostor: PhysicsImpostor): void;
        setPhysicsBodyTransformation(impostor: PhysicsImpostor, newPosition: Vector3, newRotation: Quaternion): void;
        setLinearVelocity(impostor: PhysicsImpostor, velocity: Nullable<Vector3>): void;
        setAngularVelocity(impostor: PhysicsImpostor, velocity: Nullable<Vector3>): void;
        getLinearVelocity(impostor: PhysicsImpostor): Nullable<Vector3>;
        getAngularVelocity(impostor: PhysicsImpostor): Nullable<Vector3>;
        setBodyMass(impostor: PhysicsImpostor, mass: number): void;
        getBodyMass(impostor: PhysicsImpostor): number;
        getBodyFriction(impostor: PhysicsImpostor): number;
        setBodyFriction(impostor: PhysicsImpostor, friction: number): void;
        getBodyRestitution(impostor: PhysicsImpostor): number;
        setBodyRestitution(impostor: PhysicsImpostor, restitution: number): void;
        getBodyPressure?(impostor: PhysicsImpostor): number;
        setBodyPressure?(impostor: PhysicsImpostor, pressure: number): void;
        getBodyStiffness?(impostor: PhysicsImpostor): number;
        setBodyStiffness?(impostor: PhysicsImpostor, stiffness: number): void;
        getBodyVelocityIterations?(impostor: PhysicsImpostor): number;
        setBodyVelocityIterations?(impostor: PhysicsImpostor, velocityIterations: number): void;
        getBodyPositionIterations?(impostor: PhysicsImpostor): number;
        setBodyPositionIterations?(impostor: PhysicsImpostor, positionIterations: number): void;
        appendAnchor?(impostor: PhysicsImpostor, otherImpostor: PhysicsImpostor, width: number, height: number, influence: number, noCollisionBetweenLinkedBodies: boolean): void;
        appendHook?(impostor: PhysicsImpostor, otherImpostor: PhysicsImpostor, length: number, influence: number, noCollisionBetweenLinkedBodies: boolean): void;
        sleepBody(impostor: PhysicsImpostor): void;
        wakeUpBody(impostor: PhysicsImpostor): void;
        raycast(from: Vector3, to: Vector3): PhysicsRaycastResult;
        raycastToRef(from: Vector3, to: Vector3, result: PhysicsRaycastResult): void;
        updateDistanceJoint(joint: PhysicsJoint, maxDistance: number, minDistance?: number): void;
        setMotor(joint: IMotorEnabledJoint, speed: number, maxForce?: number, motorIndex?: number): void;
        setLimit(joint: IMotorEnabledJoint, upperLimit: number, lowerLimit?: number, motorIndex?: number): void;
        getRadius(impostor: PhysicsImpostor): number;
        getBoxSizeToRef(impostor: PhysicsImpostor, result: Vector3): void;
        syncMeshWithImpostor(mesh: AbstractMesh, impostor: PhysicsImpostor): void;
        dispose(): void;
    }


    /**
     * Class used to control physics engine
     * @see https://doc.babylonjs.com/features/featuresDeepDive/physics/usingPhysicsEngine
     */
    export class PhysicsEngine implements IPhysicsEngine {
        private _physicsPlugin;
        /**
         * Global value used to control the smallest number supported by the simulation
         */
        private _impostors;
        private _joints;
        private _subTimeStep;
        private _uniqueIdCounter;
        /**
         * Gets the gravity vector used by the simulation
         */
        gravity: Vector3;
        /**
         *
         * @returns version
         */
        getPluginVersion(): number;
        /**
         * Factory used to create the default physics plugin.
         * @returns The default physics plugin
         */
        static DefaultPluginFactory(): IPhysicsEnginePlugin;
        /**
         * Creates a new Physics Engine
         * @param gravity defines the gravity vector used by the simulation
         * @param _physicsPlugin defines the plugin to use (CannonJS by default)
         */
        constructor(gravity: Nullable<Vector3>, _physicsPlugin?: IPhysicsEnginePlugin);
        /**
         * Sets the gravity vector used by the simulation
         * @param gravity defines the gravity vector to use
         */
        setGravity(gravity: Vector3): void;
        /**
         * Set the time step of the physics engine.
         * Default is 1/60.
         * To slow it down, enter 1/600 for example.
         * To speed it up, 1/30
         * @param newTimeStep defines the new timestep to apply to this world.
         */
        setTimeStep(newTimeStep?: number): void;
        /**
         * Get the time step of the physics engine.
         * @returns the current time step
         */
        getTimeStep(): number;
        /**
         * Set the sub time step of the physics engine.
         * Default is 0 meaning there is no sub steps
         * To increase physics resolution precision, set a small value (like 1 ms)
         * @param subTimeStep defines the new sub timestep used for physics resolution.
         */
        setSubTimeStep(subTimeStep?: number): void;
        /**
         * Get the sub time step of the physics engine.
         * @returns the current sub time step
         */
        getSubTimeStep(): number;
        /**
         * Release all resources
         */
        dispose(): void;
        /**
         * Gets the name of the current physics plugin
         * @returns the name of the plugin
         */
        getPhysicsPluginName(): string;
        /**
         * Adding a new impostor for the impostor tracking.
         * This will be done by the impostor itself.
         * @param impostor the impostor to add
         */
        addImpostor(impostor: PhysicsImpostor): void;
        /**
         * Remove an impostor from the engine.
         * This impostor and its mesh will not longer be updated by the physics engine.
         * @param impostor the impostor to remove
         */
        removeImpostor(impostor: PhysicsImpostor): void;
        /**
         * Add a joint to the physics engine
         * @param mainImpostor defines the main impostor to which the joint is added.
         * @param connectedImpostor defines the impostor that is connected to the main impostor using this joint
         * @param joint defines the joint that will connect both impostors.
         */
        addJoint(mainImpostor: PhysicsImpostor, connectedImpostor: PhysicsImpostor, joint: PhysicsJoint): void;
        /**
         * Removes a joint from the simulation
         * @param mainImpostor defines the impostor used with the joint
         * @param connectedImpostor defines the other impostor connected to the main one by the joint
         * @param joint defines the joint to remove
         */
        removeJoint(mainImpostor: PhysicsImpostor, connectedImpostor: PhysicsImpostor, joint: PhysicsJoint): void;
        /**
         * Called by the scene. No need to call it.
         * @param delta defines the timespan between frames
         */
        _step(delta: number): void;
        /**
         * Gets the current plugin used to run the simulation
         * @returns current plugin
         */
        getPhysicsPlugin(): IPhysicsEnginePlugin;
        /**
         * Gets the list of physic impostors
         * @returns an array of PhysicsImpostor
         */
        getImpostors(): Array<PhysicsImpostor>;
        /**
         * Gets the impostor for a physics enabled object
         * @param object defines the object impersonated by the impostor
         * @returns the PhysicsImpostor or null if not found
         */
        getImpostorForPhysicsObject(object: IPhysicsEnabledObject): Nullable<PhysicsImpostor>;
        /**
         * Gets the impostor for a physics body object
         * @param body defines physics body used by the impostor
         * @returns the PhysicsImpostor or null if not found
         */
        getImpostorWithPhysicsBody(body: any): Nullable<PhysicsImpostor>;
        /**
         * Does a raycast in the physics world
         * @param from when should the ray start?
         * @param to when should the ray end?
         * @returns PhysicsRaycastResult
         */
        raycast(from: Vector3, to: Vector3): PhysicsRaycastResult;
        /**
         * Does a raycast in the physics world
         * @param from when should the ray start?
         * @param to when should the ray end?
         * @param result resulting PhysicsRaycastResult
         */
        raycastToRef(from: Vector3, to: Vector3, result: PhysicsRaycastResult): void;
    }


        /**
         *
         */
        interface AbstractMesh {
            /** @internal */
            _physicsImpostor: Nullable<PhysicsImpostor>;
            /**
             * Gets or sets impostor used for physic simulation
             * @see https://doc.babylonjs.com/features/featuresDeepDive/physics
             */
            physicsImpostor: Nullable<PhysicsImpostor>;
            /**
             * Gets the current physics impostor
             * @see https://doc.babylonjs.com/features/featuresDeepDive/physics
             * @returns a physics impostor or null
             */
            getPhysicsImpostor(): Nullable<PhysicsImpostor>;
            /** Apply a physic impulse to the mesh
             * @param force defines the force to apply
             * @param contactPoint defines where to apply the force
             * @returns the current mesh
             * @see https://doc.babylonjs.com/features/featuresDeepDive/physics/usingPhysicsEngine
             */
            applyImpulse(force: Vector3, contactPoint: Vector3): AbstractMesh;
            /**
             * Creates a physic joint between two meshes
             * @param otherMesh defines the other mesh to use
             * @param pivot1 defines the pivot to use on this mesh
             * @param pivot2 defines the pivot to use on the other mesh
             * @param options defines additional options (can be plugin dependent)
             * @returns the current mesh
             * @see https://www.babylonjs-playground.com/#0BS5U0#0
             */
            setPhysicsLinkWith(otherMesh: Mesh, pivot1: Vector3, pivot2: Vector3, options?: any): AbstractMesh;
            /** @internal */
            _disposePhysicsObserver: Nullable<Observer<Node>>;
        }


    /**
     * The interface for the physics imposter parameters
     * @see https://doc.babylonjs.com/features/featuresDeepDive/physics/usingPhysicsEngine
     */
    export interface PhysicsImpostorParameters {
        /**
         * The mass of the physics imposter
         */
        mass: number;
        /**
         * The friction of the physics imposter
         */
        friction?: number;
        /**
         * The coefficient of restitution of the physics imposter
         */
        restitution?: number;
        /**
         * The native options of the physics imposter
         */
        nativeOptions?: any;
        /**
         * Specifies if the parent should be ignored
         */
        ignoreParent?: boolean;
        /**
         * Specifies if bi-directional transformations should be disabled
         */
        disableBidirectionalTransformation?: boolean;
        /**
         * The pressure inside the physics imposter, soft object only
         */
        pressure?: number;
        /**
         * The stiffness the physics imposter, soft object only
         */
        stiffness?: number;
        /**
         * The number of iterations used in maintaining consistent vertex velocities, soft object only
         */
        velocityIterations?: number;
        /**
         * The number of iterations used in maintaining consistent vertex positions, soft object only
         */
        positionIterations?: number;
        /**
         * The number used to fix points on a cloth (0, 1, 2, 4, 8) or rope (0, 1, 2) only
         * 0 None, 1, back left or top, 2, back right or bottom, 4, front left, 8, front right
         * Add to fix multiple points
         */
        fixedPoints?: number;
        /**
         * The collision margin around a soft object
         */
        margin?: number;
        /**
         * The collision margin around a soft object
         */
        damping?: number;
        /**
         * The path for a rope based on an extrusion
         */
        path?: any;
        /**
         * The shape of an extrusion used for a rope based on an extrusion
         */
        shape?: any;
    }
    /**
     * Interface for a physics-enabled object
     * @see https://doc.babylonjs.com/features/featuresDeepDive/physics/usingPhysicsEngine
     */
    export interface IPhysicsEnabledObject {
        /**
         * The position of the physics-enabled object
         */
        position: Vector3;
        /**
         * The rotation of the physics-enabled object
         */
        rotationQuaternion: Nullable<Quaternion>;
        /**
         * The scale of the physics-enabled object
         */
        scaling: Vector3;
        /**
         * The rotation of the physics-enabled object
         */
        rotation?: Vector3;
        /**
         * The parent of the physics-enabled object
         */
        parent?: any;
        /**
         * The bounding info of the physics-enabled object
         * @returns The bounding info of the physics-enabled object
         */
        getBoundingInfo(): BoundingInfo;
        /**
         * Computes the world matrix
         * @param force Specifies if the world matrix should be computed by force
         * @returns A world matrix
         */
        computeWorldMatrix(force: boolean): Matrix;
        /**
         * Gets the world matrix
         * @returns A world matrix
         */
        getWorldMatrix?(): Matrix;
        /**
         * Gets the child meshes
         * @param directDescendantsOnly Specifies if only direct-descendants should be obtained
         * @returns An array of abstract meshes
         */
        getChildMeshes?(directDescendantsOnly?: boolean): Array<AbstractMesh>;
        /**
         * Gets the vertex data
         * @param kind The type of vertex data
         * @returns A nullable array of numbers, or a float32 array
         */
        getVerticesData(kind: string): Nullable<Array<number> | Float32Array>;
        /**
         * Gets the indices from the mesh
         * @returns A nullable array of index arrays
         */
        getIndices?(): Nullable<IndicesArray>;
        /**
         * Gets the scene from the mesh
         * @returns the indices array or null
         */
        getScene?(): Scene;
        /**
         * Gets the absolute position from the mesh
         * @returns the absolute position
         */
        getAbsolutePosition(): Vector3;
        /**
         * Gets the absolute pivot point from the mesh
         * @returns the absolute pivot point
         */
        getAbsolutePivotPoint(): Vector3;
        /**
         * Rotates the mesh
         * @param axis The axis of rotation
         * @param amount The amount of rotation
         * @param space The space of the rotation
         * @returns The rotation transform node
         */
        rotate(axis: Vector3, amount: number, space?: Space): TransformNode;
        /**
         * Translates the mesh
         * @param axis The axis of translation
         * @param distance The distance of translation
         * @param space The space of the translation
         * @returns The transform node
         */
        translate(axis: Vector3, distance: number, space?: Space): TransformNode;
        /**
         * Sets the absolute position of the mesh
         * @param absolutePosition The absolute position of the mesh
         * @returns The transform node
         */
        setAbsolutePosition(absolutePosition: Vector3): TransformNode;
        /**
         * Gets the class name of the mesh
         * @returns The class name
         */
        getClassName(): string;
    }
    /**
     * Represents a physics imposter
     * @see https://doc.babylonjs.com/features/featuresDeepDive/physics/usingPhysicsEngine
     */
    export class PhysicsImpostor {
        /**
         * The physics-enabled object used as the physics imposter
         */
        object: IPhysicsEnabledObject;
        /**
         * The type of the physics imposter
         */
        type: number;
        private _options;
        private _scene?;
        /**
         * The default object size of the imposter
         */
        static DEFAULT_OBJECT_SIZE: Vector3;
        /**
         * The identity quaternion of the imposter
         */
        static IDENTITY_QUATERNION: Quaternion;
        /** @internal */
        _pluginData: any;
        private _physicsEngine;
        private _physicsBody;
        private _bodyUpdateRequired;
        private _onBeforePhysicsStepCallbacks;
        private _onAfterPhysicsStepCallbacks;
        /** @internal */
        _onPhysicsCollideCallbacks: Array<{
            callback: (collider: PhysicsImpostor, collidedAgainst: PhysicsImpostor, point: Nullable<Vector3>, distance: number, impulse: number, normal: Nullable<Vector3>) => void;
            otherImpostors: Array<PhysicsImpostor>;
        }>;
        private _deltaPosition;
        private _deltaRotation;
        private _deltaRotationConjugated;
        /** @internal */
        _isFromLine: boolean;
        private _parent;
        private _isDisposed;
        private static _TmpVecs;
        private static _TmpQuat;
        /**
         * Specifies if the physics imposter is disposed
         */
        get isDisposed(): boolean;
        /**
         * Gets the mass of the physics imposter
         */
        get mass(): number;
        set mass(value: number);
        /**
         * Gets the coefficient of friction
         */
        get friction(): number;
        /**
         * Sets the coefficient of friction
         */
        set friction(value: number);
        /**
         * Gets the coefficient of restitution
         */
        get restitution(): number;
        /**
         * Sets the coefficient of restitution
         */
        set restitution(value: number);
        /**
         * Gets the pressure of a soft body; only supported by the AmmoJSPlugin
         */
        get pressure(): number;
        /**
         * Sets the pressure of a soft body; only supported by the AmmoJSPlugin
         */
        set pressure(value: number);
        /**
         * Gets the stiffness of a soft body; only supported by the AmmoJSPlugin
         */
        get stiffness(): number;
        /**
         * Sets the stiffness of a soft body; only supported by the AmmoJSPlugin
         */
        set stiffness(value: number);
        /**
         * Gets the velocityIterations of a soft body; only supported by the AmmoJSPlugin
         */
        get velocityIterations(): number;
        /**
         * Sets the velocityIterations of a soft body; only supported by the AmmoJSPlugin
         */
        set velocityIterations(value: number);
        /**
         * Gets the positionIterations of a soft body; only supported by the AmmoJSPlugin
         */
        get positionIterations(): number;
        /**
         * Sets the positionIterations of a soft body; only supported by the AmmoJSPlugin
         */
        set positionIterations(value: number);
        /**
         * The unique id of the physics imposter
         * set by the physics engine when adding this impostor to the array
         */
        uniqueId: number;
        /**
         * @internal
         */
        soft: boolean;
        /**
         * @internal
         */
        segments: number;
        private _joints;
        /**
         * Initializes the physics imposter
         * @param object The physics-enabled object used as the physics imposter
         * @param type The type of the physics imposter. Types are available as static members of this class.
         * @param _options The options for the physics imposter
         * @param _scene The Babylon scene
         */
        constructor(
        /**
         * The physics-enabled object used as the physics imposter
         */
        object: IPhysicsEnabledObject, 
        /**
         * The type of the physics imposter
         */
        type: number, _options?: PhysicsImpostorParameters, _scene?: Scene | undefined);
        /**
         * This function will completely initialize this impostor.
         * It will create a new body - but only if this mesh has no parent.
         * If it has, this impostor will not be used other than to define the impostor
         * of the child mesh.
         * @internal
         */
        _init(): void;
        private _getPhysicsParent;
        /**
         * Should a new body be generated.
         * @returns boolean specifying if body initialization is required
         */
        isBodyInitRequired(): boolean;
        /**
         * Sets the updated scaling
         */
        setScalingUpdated(): void;
        /**
         * Force a regeneration of this or the parent's impostor's body.
         * Use with caution - This will remove all previously-instantiated joints.
         */
        forceUpdate(): void;
        /**
         * Gets the body that holds this impostor. Either its own, or its parent.
         */
        get physicsBody(): any;
        /**
         * Get the parent of the physics imposter
         * @returns Physics imposter or null
         */
        get parent(): Nullable<PhysicsImpostor>;
        /**
         * Sets the parent of the physics imposter
         */
        set parent(value: Nullable<PhysicsImpostor>);
        /**
         * Set the physics body. Used mainly by the physics engine/plugin
         */
        set physicsBody(physicsBody: any);
        /**
         * Resets the update flags
         */
        resetUpdateFlags(): void;
        /**
         * Gets the object extents
         * @returns the object extents
         */
        getObjectExtents(): Vector3;
        /**
         * Gets the object center
         * @returns The object center
         */
        getObjectCenter(): Vector3;
        /**
         * Get a specific parameter from the options parameters
         * @param paramName The object parameter name
         * @returns The object parameter
         */
        getParam(paramName: string): any;
        /**
         * Sets a specific parameter in the options given to the physics plugin
         * @param paramName The parameter name
         * @param value The value of the parameter
         */
        setParam(paramName: string, value: number): void;
        /**
         * Specifically change the body's mass. Won't recreate the physics body object
         * @param mass The mass of the physics imposter
         */
        setMass(mass: number): void;
        /**
         * Gets the linear velocity
         * @returns  linear velocity or null
         */
        getLinearVelocity(): Nullable<Vector3>;
        /**
         * Sets the linear velocity
         * @param velocity  linear velocity or null
         */
        setLinearVelocity(velocity: Nullable<Vector3>): void;
        /**
         * Gets the angular velocity
         * @returns angular velocity or null
         */
        getAngularVelocity(): Nullable<Vector3>;
        /**
         * Sets the angular velocity
         * @param velocity The velocity or null
         */
        setAngularVelocity(velocity: Nullable<Vector3>): void;
        /**
         * Execute a function with the physics plugin native code
         * Provide a function the will have two variables - the world object and the physics body object
         * @param func The function to execute with the physics plugin native code
         */
        executeNativeFunction(func: (world: any, physicsBody: any) => void): void;
        /**
         * Register a function that will be executed before the physics world is stepping forward
         * @param func The function to execute before the physics world is stepped forward
         */
        registerBeforePhysicsStep(func: (impostor: PhysicsImpostor) => void): void;
        /**
         * Unregister a function that will be executed before the physics world is stepping forward
         * @param func The function to execute before the physics world is stepped forward
         */
        unregisterBeforePhysicsStep(func: (impostor: PhysicsImpostor) => void): void;
        /**
         * Register a function that will be executed after the physics step
         * @param func The function to execute after physics step
         */
        registerAfterPhysicsStep(func: (impostor: PhysicsImpostor) => void): void;
        /**
         * Unregisters a function that will be executed after the physics step
         * @param func The function to execute after physics step
         */
        unregisterAfterPhysicsStep(func: (impostor: PhysicsImpostor) => void): void;
        /**
         * register a function that will be executed when this impostor collides against a different body
         * @param collideAgainst Physics imposter, or array of physics imposters to collide against
         * @param func Callback that is executed on collision
         */
        registerOnPhysicsCollide(collideAgainst: PhysicsImpostor | Array<PhysicsImpostor>, func: (collider: PhysicsImpostor, collidedAgainst: PhysicsImpostor, point: Nullable<Vector3>) => void): void;
        /**
         * Unregisters the physics imposter's collision callback
         * @param collideAgainst The physics object to collide against
         * @param func Callback to execute on collision
         */
        unregisterOnPhysicsCollide(collideAgainst: PhysicsImpostor | Array<PhysicsImpostor>, func: (collider: PhysicsImpostor, collidedAgainst: PhysicsImpostor | Array<PhysicsImpostor>, point: Nullable<Vector3>) => void): void;
        private _tmpQuat;
        private _tmpQuat2;
        /**
         * Get the parent rotation
         * @returns The parent rotation
         */
        getParentsRotation(): Quaternion;
        /**
         * this function is executed by the physics engine.
         */
        beforeStep: () => void;
        /**
         * this function is executed by the physics engine
         */
        afterStep: () => void;
        /**
         * Legacy collision detection event support
         */
        onCollideEvent: Nullable<(collider: PhysicsImpostor, collidedWith: PhysicsImpostor) => void>;
        /**
         *
         * @param e
         * @returns
         */
        onCollide: (e: {
            body: any;
            point: Nullable<Vector3>;
            distance: number;
            impulse: number;
            normal: Nullable<Vector3>;
        }) => void;
        /**
         * Apply a force
         * @param force The force to apply
         * @param contactPoint The contact point for the force
         * @returns The physics imposter
         */
        applyForce(force: Vector3, contactPoint: Vector3): PhysicsImpostor;
        /**
         * Apply an impulse
         * @param force The impulse force
         * @param contactPoint The contact point for the impulse force
         * @returns The physics imposter
         */
        applyImpulse(force: Vector3, contactPoint: Vector3): PhysicsImpostor;
        /**
         * A help function to create a joint
         * @param otherImpostor A physics imposter used to create a joint
         * @param jointType The type of joint
         * @param jointData The data for the joint
         * @returns The physics imposter
         */
        createJoint(otherImpostor: PhysicsImpostor, jointType: number, jointData: PhysicsJointData): PhysicsImpostor;
        /**
         * Add a joint to this impostor with a different impostor
         * @param otherImpostor A physics imposter used to add a joint
         * @param joint The joint to add
         * @returns The physics imposter
         */
        addJoint(otherImpostor: PhysicsImpostor, joint: PhysicsJoint): PhysicsImpostor;
        /**
         * Add an anchor to a cloth impostor
         * @param otherImpostor rigid impostor to anchor to
         * @param width ratio across width from 0 to 1
         * @param height ratio up height from 0 to 1
         * @param influence the elasticity between cloth impostor and anchor from 0, very stretchy to 1, little stretch
         * @param noCollisionBetweenLinkedBodies when true collisions between cloth impostor and anchor are ignored; default false
         * @returns impostor the soft imposter
         */
        addAnchor(otherImpostor: PhysicsImpostor, width: number, height: number, influence: number, noCollisionBetweenLinkedBodies: boolean): PhysicsImpostor;
        /**
         * Add a hook to a rope impostor
         * @param otherImpostor rigid impostor to anchor to
         * @param length ratio across rope from 0 to 1
         * @param influence the elasticity between rope impostor and anchor from 0, very stretchy to 1, little stretch
         * @param noCollisionBetweenLinkedBodies when true collisions between soft impostor and anchor are ignored; default false
         * @returns impostor the rope imposter
         */
        addHook(otherImpostor: PhysicsImpostor, length: number, influence: number, noCollisionBetweenLinkedBodies: boolean): PhysicsImpostor;
        /**
         * Will keep this body still, in a sleep mode.
         * @returns the physics imposter
         */
        sleep(): PhysicsImpostor;
        /**
         * Wake the body up.
         * @returns The physics imposter
         */
        wakeUp(): PhysicsImpostor;
        /**
         * Clones the physics imposter
         * @param newObject The physics imposter clones to this physics-enabled object
         * @returns A nullable physics imposter
         */
        clone(newObject: IPhysicsEnabledObject): Nullable<PhysicsImpostor>;
        /**
         * Disposes the physics imposter
         */
        dispose(): void;
        /**
         * Sets the delta position
         * @param position The delta position amount
         */
        setDeltaPosition(position: Vector3): void;
        /**
         * Sets the delta rotation
         * @param rotation The delta rotation amount
         */
        setDeltaRotation(rotation: Quaternion): void;
        /**
         * Gets the box size of the physics imposter and stores the result in the input parameter
         * @param result Stores the box size
         * @returns The physics imposter
         */
        getBoxSizeToRef(result: Vector3): PhysicsImpostor;
        /**
         * Gets the radius of the physics imposter
         * @returns Radius of the physics imposter
         */
        getRadius(): number;
        /**
         * Sync a bone with this impostor
         * @param bone The bone to sync to the impostor.
         * @param boneMesh The mesh that the bone is influencing.
         * @param jointPivot The pivot of the joint / bone in local space.
         * @param distToJoint Optional distance from the impostor to the joint.
         * @param adjustRotation Optional quaternion for adjusting the local rotation of the bone.
         */
        syncBoneWithImpostor(bone: Bone, boneMesh: AbstractMesh, jointPivot: Vector3, distToJoint?: number, adjustRotation?: Quaternion): void;
        /**
         * Sync impostor to a bone
         * @param bone The bone that the impostor will be synced to.
         * @param boneMesh The mesh that the bone is influencing.
         * @param jointPivot The pivot of the joint / bone in local space.
         * @param distToJoint Optional distance from the impostor to the joint.
         * @param adjustRotation Optional quaternion for adjusting the local rotation of the bone.
         * @param boneAxis Optional vector3 axis the bone is aligned with
         */
        syncImpostorWithBone(bone: Bone, boneMesh: AbstractMesh, jointPivot: Vector3, distToJoint?: number, adjustRotation?: Quaternion, boneAxis?: Vector3): void;
        /**
         * No-Imposter type
         */
        static NoImpostor: number;
        /**
         * Sphere-Imposter type
         */
        static SphereImpostor: number;
        /**
         * Box-Imposter type
         */
        static BoxImpostor: number;
        /**
         * Plane-Imposter type
         */
        static PlaneImpostor: number;
        /**
         * Mesh-imposter type (Only available to objects with vertices data)
         */
        static MeshImpostor: number;
        /**
         * Capsule-Impostor type (Ammo.js plugin only)
         */
        static CapsuleImpostor: number;
        /**
         * Cylinder-Imposter type
         */
        static CylinderImpostor: number;
        /**
         * Particle-Imposter type
         */
        static ParticleImpostor: number;
        /**
         * Heightmap-Imposter type
         */
        static HeightmapImpostor: number;
        /**
         * ConvexHull-Impostor type (Ammo.js plugin only)
         */
        static ConvexHullImpostor: number;
        /**
         * Custom-Imposter type (Ammo.js plugin only)
         */
        static CustomImpostor: number;
        /**
         * Rope-Imposter type
         */
        static RopeImpostor: number;
        /**
         * Cloth-Imposter type
         */
        static ClothImpostor: number;
        /**
         * Softbody-Imposter type
         */
        static SoftbodyImpostor: number;
    }


    /**
     * Interface for Physics-Joint data
     * @see https://doc.babylonjs.com/features/featuresDeepDive/physics/usingPhysicsEngine
     */
    export interface PhysicsJointData {
        /**
         * The main pivot of the joint
         */
        mainPivot?: Vector3;
        /**
         * The connected pivot of the joint
         */
        connectedPivot?: Vector3;
        /**
         * The main axis of the joint
         */
        mainAxis?: Vector3;
        /**
         * The connected axis of the joint
         */
        connectedAxis?: Vector3;
        /**
         * The collision of the joint
         */
        collision?: boolean;
        /**
         * Native Oimo/Cannon/Energy data
         */
        nativeParams?: any;
    }
    /**
     * This is a holder class for the physics joint created by the physics plugin
     * It holds a set of functions to control the underlying joint
     * @see https://doc.babylonjs.com/features/featuresDeepDive/physics/usingPhysicsEngine
     */
    export class PhysicsJoint {
        /**
         * The type of the physics joint
         */
        type: number;
        /**
         * The data for the physics joint
         */
        jointData: PhysicsJointData;
        private _physicsJoint;
        protected _physicsPlugin: IPhysicsEnginePlugin;
        /**
         * Initializes the physics joint
         * @param type The type of the physics joint
         * @param jointData The data for the physics joint
         */
        constructor(
        /**
         * The type of the physics joint
         */
        type: number, 
        /**
         * The data for the physics joint
         */
        jointData: PhysicsJointData);
        /**
         * Gets the physics joint
         */
        get physicsJoint(): any;
        /**
         * Sets the physics joint
         */
        set physicsJoint(newJoint: any);
        /**
         * Sets the physics plugin
         */
        set physicsPlugin(physicsPlugin: IPhysicsEnginePlugin);
        /**
         * Execute a function that is physics-plugin specific.
         * @param {Function} func the function that will be executed.
         *                        It accepts two parameters: the physics world and the physics joint
         */
        executeNativeFunction(func: (world: any, physicsJoint: any) => void): void;
        /**
         * Distance-Joint type
         */
        static DistanceJoint: number;
        /**
         * Hinge-Joint type
         */
        static HingeJoint: number;
        /**
         * Ball-and-Socket joint type
         */
        static BallAndSocketJoint: number;
        /**
         * Wheel-Joint type
         */
        static WheelJoint: number;
        /**
         * Slider-Joint type
         */
        static SliderJoint: number;
        /**
         * Prismatic-Joint type
         */
        static PrismaticJoint: number;
        /**
         * Universal-Joint type
         * ENERGY FTW! (compare with this - @see http://ode-wiki.org/wiki/index.php?title=Manual:_Joint_Types_and_Functions)
         */
        static UniversalJoint: number;
        /**
         * Hinge-Joint 2 type
         */
        static Hinge2Joint: number;
        /**
         * Point to Point Joint type.  Similar to a Ball-Joint.  Different in parameters
         */
        static PointToPointJoint: number;
        /**
         * Spring-Joint type
         */
        static SpringJoint: number;
        /**
         * Lock-Joint type
         */
        static LockJoint: number;
    }
    /**
     * A class representing a physics distance joint
     * @see https://doc.babylonjs.com/features/featuresDeepDive/physics/usingPhysicsEngine
     */
    export class DistanceJoint extends PhysicsJoint {
        /**
         *
         * @param jointData The data for the Distance-Joint
         */
        constructor(jointData: DistanceJointData);
        /**
         * Update the predefined distance.
         * @param maxDistance The maximum preferred distance
         * @param minDistance The minimum preferred distance
         */
        updateDistance(maxDistance: number, minDistance?: number): void;
    }
    /**
     * Represents a Motor-Enabled Joint
     * @see https://doc.babylonjs.com/features/featuresDeepDive/physics/usingPhysicsEngine
     */
    export class MotorEnabledJoint extends PhysicsJoint implements IMotorEnabledJoint {
        /**
         * Initializes the Motor-Enabled Joint
         * @param type The type of the joint
         * @param jointData The physical joint data for the joint
         */
        constructor(type: number, jointData: PhysicsJointData);
        /**
         * Set the motor values.
         * Attention, this function is plugin specific. Engines won't react 100% the same.
         * @param force the force to apply
         * @param maxForce max force for this motor.
         */
        setMotor(force?: number, maxForce?: number): void;
        /**
         * Set the motor's limits.
         * Attention, this function is plugin specific. Engines won't react 100% the same.
         * @param upperLimit The upper limit of the motor
         * @param lowerLimit The lower limit of the motor
         */
        setLimit(upperLimit: number, lowerLimit?: number): void;
    }
    /**
     * This class represents a single physics Hinge-Joint
     * @see https://doc.babylonjs.com/features/featuresDeepDive/physics/usingPhysicsEngine
     */
    export class HingeJoint extends MotorEnabledJoint {
        /**
         * Initializes the Hinge-Joint
         * @param jointData The joint data for the Hinge-Joint
         */
        constructor(jointData: PhysicsJointData);
        /**
         * Set the motor values.
         * Attention, this function is plugin specific. Engines won't react 100% the same.
         * @param {number} force the force to apply
         * @param {number} maxForce max force for this motor.
         */
        setMotor(force?: number, maxForce?: number): void;
        /**
         * Set the motor's limits.
         * Attention, this function is plugin specific. Engines won't react 100% the same.
         * @param upperLimit The upper limit of the motor
         * @param lowerLimit The lower limit of the motor
         */
        setLimit(upperLimit: number, lowerLimit?: number): void;
    }
    /**
     * This class represents a dual hinge physics joint (same as wheel joint)
     * @see https://doc.babylonjs.com/features/featuresDeepDive/physics/usingPhysicsEngine
     */
    export class Hinge2Joint extends MotorEnabledJoint {
        /**
         * Initializes the Hinge2-Joint
         * @param jointData The joint data for the Hinge2-Joint
         */
        constructor(jointData: PhysicsJointData);
        /**
         * Set the motor values.
         * Attention, this function is plugin specific. Engines won't react 100% the same.
         * @param targetSpeed the speed the motor is to reach
         * @param maxForce max force for this motor.
         * @param motorIndex motor's index, 0 or 1.
         */
        setMotor(targetSpeed?: number, maxForce?: number, motorIndex?: number): void;
        /**
         * Set the motor limits.
         * Attention, this function is plugin specific. Engines won't react 100% the same.
         * @param upperLimit the upper limit
         * @param lowerLimit lower limit
         * @param motorIndex the motor's index, 0 or 1.
         */
        setLimit(upperLimit: number, lowerLimit?: number, motorIndex?: number): void;
    }
    /**
     * Interface for a motor enabled joint
     * @see https://doc.babylonjs.com/features/featuresDeepDive/physics/usingPhysicsEngine
     */
    export interface IMotorEnabledJoint {
        /**
         * Physics joint
         */
        physicsJoint: any;
        /**
         * Sets the motor of the motor-enabled joint
         * @param force The force of the motor
         * @param maxForce The maximum force of the motor
         * @param motorIndex The index of the motor
         */
        setMotor(force?: number, maxForce?: number, motorIndex?: number): void;
        /**
         * Sets the limit of the motor
         * @param upperLimit The upper limit of the motor
         * @param lowerLimit The lower limit of the motor
         * @param motorIndex The index of the motor
         */
        setLimit(upperLimit: number, lowerLimit?: number, motorIndex?: number): void;
    }
    /**
     * Joint data for a Distance-Joint
     * @see https://doc.babylonjs.com/features/featuresDeepDive/physics/usingPhysicsEngine
     */
    export interface DistanceJointData extends PhysicsJointData {
        /**
         * Max distance the 2 joint objects can be apart
         */
        maxDistance: number;
    }
    /**
     * Joint data from a spring joint
     * @see https://doc.babylonjs.com/features/featuresDeepDive/physics/usingPhysicsEngine
     */
    export interface SpringJointData extends PhysicsJointData {
        /**
         * Length of the spring
         */
        length: number;
        /**
         * Stiffness of the spring
         */
        stiffness: number;
        /**
         * Damping of the spring
         */
        damping: number;
        /** this callback will be called when applying the force to the impostors. */
        forceApplicationCallback: () => void;
    }


    /**
     * AmmoJS Physics plugin
     * @see https://doc.babylonjs.com/features/featuresDeepDive/physics/usingPhysicsEngine
     * @see https://github.com/kripken/ammo.js/
     */
    export class AmmoJSPlugin implements IPhysicsEnginePlugin {
        private _useDeltaForWorldStep;
        /**
         * Reference to the Ammo library
         */
        bjsAMMO: any;
        /**
         * Created ammoJS world which physics bodies are added to
         */
        world: any;
        /**
         * Name of the plugin
         */
        name: string;
        private _timeStep;
        private _fixedTimeStep;
        private _maxSteps;
        private _tmpQuaternion;
        private _tmpAmmoTransform;
        private _tmpAmmoQuaternion;
        private _tmpAmmoConcreteContactResultCallback;
        private _collisionConfiguration;
        private _dispatcher;
        private _overlappingPairCache;
        private _solver;
        private _softBodySolver;
        private _tmpAmmoVectorA;
        private _tmpAmmoVectorB;
        private _tmpAmmoVectorC;
        private _tmpAmmoVectorD;
        private _tmpContactCallbackResult;
        private _tmpAmmoVectorRCA;
        private _tmpAmmoVectorRCB;
        private _raycastResult;
        private _tmpContactPoint;
        private _tmpContactNormal;
        private _tmpContactDistance;
        private _tmpContactImpulse;
        private _tmpVec3;
        private static readonly _DISABLE_COLLISION_FLAG;
        private static readonly _KINEMATIC_FLAG;
        private static readonly _DISABLE_DEACTIVATION_FLAG;
        /**
         * Initializes the ammoJS plugin
         * @param _useDeltaForWorldStep if the time between frames should be used when calculating physics steps (Default: true)
         * @param ammoInjection can be used to inject your own ammo reference
         * @param overlappingPairCache can be used to specify your own overlapping pair cache
         */
        constructor(_useDeltaForWorldStep?: boolean, ammoInjection?: any, overlappingPairCache?: any);
        /**
         *
         * @returns plugin version
         */
        getPluginVersion(): number;
        /**
         * Sets the gravity of the physics world (m/(s^2))
         * @param gravity Gravity to set
         */
        setGravity(gravity: Vector3): void;
        /**
         * Amount of time to step forward on each frame (only used if useDeltaForWorldStep is false in the constructor)
         * @param timeStep timestep to use in seconds
         */
        setTimeStep(timeStep: number): void;
        /**
         * Increment to step forward in the physics engine (If timeStep is set to 1/60 and fixedTimeStep is set to 1/120 the physics engine should run 2 steps per frame) (Default: 1/60)
         * @param fixedTimeStep fixedTimeStep to use in seconds
         */
        setFixedTimeStep(fixedTimeStep: number): void;
        /**
         * Sets the maximum number of steps by the physics engine per frame (Default: 5)
         * @param maxSteps the maximum number of steps by the physics engine per frame
         */
        setMaxSteps(maxSteps: number): void;
        /**
         * Gets the current timestep (only used if useDeltaForWorldStep is false in the constructor)
         * @returns the current timestep in seconds
         */
        getTimeStep(): number;
        /**
         * The create custom shape handler function to be called when using BABYLON.PhysicsImposter.CustomImpostor
         */
        onCreateCustomShape: (impostor: PhysicsImpostor) => any;
        /**
         * The create custom mesh impostor handler function to support building custom mesh impostor vertex data
         */
        onCreateCustomMeshImpostor: (impostor: PhysicsImpostor) => any;
        /**
         * The create custom convex hull impostor handler function to support building custom convex hull impostor vertex data
         */
        onCreateCustomConvexHullImpostor: (impostor: PhysicsImpostor) => any;
        private _isImpostorInContact;
        private _isImpostorPairInContact;
        private _stepSimulation;
        /**
         * Moves the physics simulation forward delta seconds and updates the given physics imposters
         * Prior to the step the imposters physics location is set to the position of the babylon meshes
         * After the step the babylon meshes are set to the position of the physics imposters
         * @param delta amount of time to step forward
         * @param impostors array of imposters to update before/after the step
         */
        executeStep(delta: number, impostors: Array<PhysicsImpostor>): void;
        /**
         * Update babylon mesh to match physics world object
         * @param impostor imposter to match
         */
        private _afterSoftStep;
        /**
         * Update babylon mesh vertices vertices to match physics world softbody or cloth
         * @param impostor imposter to match
         */
        private _ropeStep;
        /**
         * Update babylon mesh vertices vertices to match physics world softbody or cloth
         * @param impostor imposter to match
         */
        private _softbodyOrClothStep;
        private _tmpMatrix;
        /**
         * Applies an impulse on the imposter
         * @param impostor imposter to apply impulse to
         * @param force amount of force to be applied to the imposter
         * @param contactPoint the location to apply the impulse on the imposter
         */
        applyImpulse(impostor: PhysicsImpostor, force: Vector3, contactPoint: Vector3): void;
        /**
         * Applies a force on the imposter
         * @param impostor imposter to apply force
         * @param force amount of force to be applied to the imposter
         * @param contactPoint the location to apply the force on the imposter
         */
        applyForce(impostor: PhysicsImpostor, force: Vector3, contactPoint: Vector3): void;
        /**
         * Creates a physics body using the plugin
         * @param impostor the imposter to create the physics body on
         */
        generatePhysicsBody(impostor: PhysicsImpostor): void;
        /**
         * Removes the physics body from the imposter and disposes of the body's memory
         * @param impostor imposter to remove the physics body from
         */
        removePhysicsBody(impostor: PhysicsImpostor): void;
        /**
         * Generates a joint
         * @param impostorJoint the imposter joint to create the joint with
         */
        generateJoint(impostorJoint: PhysicsImpostorJoint): void;
        /**
         * Removes a joint
         * @param impostorJoint the imposter joint to remove the joint from
         */
        removeJoint(impostorJoint: PhysicsImpostorJoint): void;
        private _addMeshVerts;
        /**
         * Initialise the soft body vertices to match its object's (mesh) vertices
         * Softbody vertices (nodes) are in world space and to match this
         * The object's position and rotation is set to zero and so its vertices are also then set in world space
         * @param impostor to create the softbody for
         */
        private _softVertexData;
        /**
         * Create an impostor's soft body
         * @param impostor to create the softbody for
         */
        private _createSoftbody;
        /**
         * Create cloth for an impostor
         * @param impostor to create the softbody for
         */
        private _createCloth;
        /**
         * Create rope for an impostor
         * @param impostor to create the softbody for
         */
        private _createRope;
        /**
         * Create a custom physics impostor shape using the plugin's onCreateCustomShape handler
         * @param impostor to create the custom physics shape for
         */
        private _createCustom;
        private _addHullVerts;
        private _createShape;
        /**
         * Sets the mesh body position/rotation from the babylon impostor
         * @param impostor imposter containing the physics body and babylon object
         */
        setTransformationFromPhysicsBody(impostor: PhysicsImpostor): void;
        /**
         * Sets the babylon object's position/rotation from the physics body's position/rotation
         * @param impostor imposter containing the physics body and babylon object
         * @param newPosition new position
         * @param newRotation new rotation
         */
        setPhysicsBodyTransformation(impostor: PhysicsImpostor, newPosition: Vector3, newRotation: Quaternion): void;
        /**
         * If this plugin is supported
         * @returns true if its supported
         */
        isSupported(): boolean;
        /**
         * Sets the linear velocity of the physics body
         * @param impostor imposter to set the velocity on
         * @param velocity velocity to set
         */
        setLinearVelocity(impostor: PhysicsImpostor, velocity: Vector3): void;
        /**
         * Sets the angular velocity of the physics body
         * @param impostor imposter to set the velocity on
         * @param velocity velocity to set
         */
        setAngularVelocity(impostor: PhysicsImpostor, velocity: Vector3): void;
        /**
         * gets the linear velocity
         * @param impostor imposter to get linear velocity from
         * @returns linear velocity
         */
        getLinearVelocity(impostor: PhysicsImpostor): Nullable<Vector3>;
        /**
         * gets the angular velocity
         * @param impostor imposter to get angular velocity from
         * @returns angular velocity
         */
        getAngularVelocity(impostor: PhysicsImpostor): Nullable<Vector3>;
        /**
         * Sets the mass of physics body
         * @param impostor imposter to set the mass on
         * @param mass mass to set
         */
        setBodyMass(impostor: PhysicsImpostor, mass: number): void;
        /**
         * Gets the mass of the physics body
         * @param impostor imposter to get the mass from
         * @returns mass
         */
        getBodyMass(impostor: PhysicsImpostor): number;
        /**
         * Gets friction of the impostor
         * @param impostor impostor to get friction from
         * @returns friction value
         */
        getBodyFriction(impostor: PhysicsImpostor): number;
        /**
         * Sets friction of the impostor
         * @param impostor impostor to set friction on
         * @param friction friction value
         */
        setBodyFriction(impostor: PhysicsImpostor, friction: number): void;
        /**
         * Gets restitution of the impostor
         * @param impostor impostor to get restitution from
         * @returns restitution value
         */
        getBodyRestitution(impostor: PhysicsImpostor): number;
        /**
         * Sets restitution of the impostor
         * @param impostor impostor to set resitution on
         * @param restitution resitution value
         */
        setBodyRestitution(impostor: PhysicsImpostor, restitution: number): void;
        /**
         * Gets pressure inside the impostor
         * @param impostor impostor to get pressure from
         * @returns pressure value
         */
        getBodyPressure(impostor: PhysicsImpostor): number;
        /**
         * Sets pressure inside a soft body impostor
         * Cloth and rope must remain 0 pressure
         * @param impostor impostor to set pressure on
         * @param pressure pressure value
         */
        setBodyPressure(impostor: PhysicsImpostor, pressure: number): void;
        /**
         * Gets stiffness of the impostor
         * @param impostor impostor to get stiffness from
         * @returns pressure value
         */
        getBodyStiffness(impostor: PhysicsImpostor): number;
        /**
         * Sets stiffness of the impostor
         * @param impostor impostor to set stiffness on
         * @param stiffness stiffness value from 0 to 1
         */
        setBodyStiffness(impostor: PhysicsImpostor, stiffness: number): void;
        /**
         * Gets velocityIterations of the impostor
         * @param impostor impostor to get velocity iterations from
         * @returns velocityIterations value
         */
        getBodyVelocityIterations(impostor: PhysicsImpostor): number;
        /**
         * Sets velocityIterations of the impostor
         * @param impostor impostor to set velocity iterations on
         * @param velocityIterations velocityIterations value
         */
        setBodyVelocityIterations(impostor: PhysicsImpostor, velocityIterations: number): void;
        /**
         * Gets positionIterations of the impostor
         * @param impostor impostor to get position iterations from
         * @returns positionIterations value
         */
        getBodyPositionIterations(impostor: PhysicsImpostor): number;
        /**
         * Sets positionIterations of the impostor
         * @param impostor impostor to set position on
         * @param positionIterations positionIterations value
         */
        setBodyPositionIterations(impostor: PhysicsImpostor, positionIterations: number): void;
        /**
         * Append an anchor to a cloth object
         * @param impostor is the cloth impostor to add anchor to
         * @param otherImpostor is the rigid impostor to anchor to
         * @param width ratio across width from 0 to 1
         * @param height ratio up height from 0 to 1
         * @param influence the elasticity between cloth impostor and anchor from 0, very stretchy to 1, little stretch
         * @param noCollisionBetweenLinkedBodies when true collisions between soft impostor and anchor are ignored; default false
         */
        appendAnchor(impostor: PhysicsImpostor, otherImpostor: PhysicsImpostor, width: number, height: number, influence?: number, noCollisionBetweenLinkedBodies?: boolean): void;
        /**
         * Append an hook to a rope object
         * @param impostor is the rope impostor to add hook to
         * @param otherImpostor is the rigid impostor to hook to
         * @param length ratio along the rope from 0 to 1
         * @param influence the elasticity between soft impostor and anchor from 0, very stretchy to 1, little stretch
         * @param noCollisionBetweenLinkedBodies when true collisions between soft impostor and anchor are ignored; default false
         */
        appendHook(impostor: PhysicsImpostor, otherImpostor: PhysicsImpostor, length: number, influence?: number, noCollisionBetweenLinkedBodies?: boolean): void;
        /**
         * Sleeps the physics body and stops it from being active
         * @param impostor impostor to sleep
         */
        sleepBody(impostor: PhysicsImpostor): void;
        /**
         * Activates the physics body
         * @param impostor impostor to activate
         */
        wakeUpBody(impostor: PhysicsImpostor): void;
        /**
         * Updates the distance parameters of the joint
         */
        updateDistanceJoint(): void;
        /**
         * Sets a motor on the joint
         * @param joint joint to set motor on
         * @param speed speed of the motor
         * @param maxForce maximum force of the motor
         */
        setMotor(joint: IMotorEnabledJoint, speed?: number, maxForce?: number): void;
        /**
         * Sets the motors limit
         */
        setLimit(): void;
        /**
         * Syncs the position and rotation of a mesh with the impostor
         * @param mesh mesh to sync
         * @param impostor impostor to update the mesh with
         */
        syncMeshWithImpostor(mesh: AbstractMesh, impostor: PhysicsImpostor): void;
        /**
         * Gets the radius of the impostor
         * @param impostor impostor to get radius from
         * @returns the radius
         */
        getRadius(impostor: PhysicsImpostor): number;
        /**
         * Gets the box size of the impostor
         * @param impostor impostor to get box size from
         * @param result the resulting box size
         */
        getBoxSizeToRef(impostor: PhysicsImpostor, result: Vector3): void;
        /**
         * Disposes of the impostor
         */
        dispose(): void;
        /**
         * Does a raycast in the physics world
         * @param from where should the ray start?
         * @param to where should the ray end?
         * @returns PhysicsRaycastResult
         */
        raycast(from: Vector3, to: Vector3): PhysicsRaycastResult;
        /**
         * Does a raycast in the physics world
         * @param from when should the ray start?
         * @param to when should the ray end?
         * @param result resulting PhysicsRaycastResult
         */
        raycastToRef(from: Vector3, to: Vector3, result: PhysicsRaycastResult): void;
    }


    /** @internal */
    export class CannonJSPlugin implements IPhysicsEnginePlugin {
        private _useDeltaForWorldStep;
        world: any;
        name: string;
        private _physicsMaterials;
        private _fixedTimeStep;
        private _cannonRaycastResult;
        private _raycastResult;
        private _physicsBodiesToRemoveAfterStep;
        private _firstFrame;
        private _tmpQuaternion;
        BJSCANNON: any;
        constructor(_useDeltaForWorldStep?: boolean, iterations?: number, cannonInjection?: any);
        /**
         *
         * @returns plugin version
         */
        getPluginVersion(): number;
        setGravity(gravity: Vector3): void;
        setTimeStep(timeStep: number): void;
        getTimeStep(): number;
        executeStep(delta: number, impostors: Array<PhysicsImpostor>): void;
        private _removeMarkedPhysicsBodiesFromWorld;
        applyImpulse(impostor: PhysicsImpostor, force: Vector3, contactPoint: Vector3): void;
        applyForce(impostor: PhysicsImpostor, force: Vector3, contactPoint: Vector3): void;
        generatePhysicsBody(impostor: PhysicsImpostor): void;
        private _processChildMeshes;
        removePhysicsBody(impostor: PhysicsImpostor): void;
        generateJoint(impostorJoint: PhysicsImpostorJoint): void;
        removeJoint(impostorJoint: PhysicsImpostorJoint): void;
        private _addMaterial;
        private _checkWithEpsilon;
        private _createShape;
        private _createHeightmap;
        private _minus90X;
        private _plus90X;
        private _tmpPosition;
        private _tmpDeltaPosition;
        private _tmpUnityRotation;
        private _updatePhysicsBodyTransformation;
        setTransformationFromPhysicsBody(impostor: PhysicsImpostor): void;
        setPhysicsBodyTransformation(impostor: PhysicsImpostor, newPosition: Vector3, newRotation: Quaternion): void;
        isSupported(): boolean;
        setLinearVelocity(impostor: PhysicsImpostor, velocity: Vector3): void;
        setAngularVelocity(impostor: PhysicsImpostor, velocity: Vector3): void;
        getLinearVelocity(impostor: PhysicsImpostor): Nullable<Vector3>;
        getAngularVelocity(impostor: PhysicsImpostor): Nullable<Vector3>;
        setBodyMass(impostor: PhysicsImpostor, mass: number): void;
        getBodyMass(impostor: PhysicsImpostor): number;
        getBodyFriction(impostor: PhysicsImpostor): number;
        setBodyFriction(impostor: PhysicsImpostor, friction: number): void;
        getBodyRestitution(impostor: PhysicsImpostor): number;
        setBodyRestitution(impostor: PhysicsImpostor, restitution: number): void;
        sleepBody(impostor: PhysicsImpostor): void;
        wakeUpBody(impostor: PhysicsImpostor): void;
        updateDistanceJoint(joint: PhysicsJoint, maxDistance: number): void;
        setMotor(joint: IMotorEnabledJoint, speed?: number, maxForce?: number, motorIndex?: number): void;
        setLimit(joint: IMotorEnabledJoint, minForce: number, maxForce?: number): void;
        syncMeshWithImpostor(mesh: AbstractMesh, impostor: PhysicsImpostor): void;
        getRadius(impostor: PhysicsImpostor): number;
        getBoxSizeToRef(impostor: PhysicsImpostor, result: Vector3): void;
        dispose(): void;
        private _extendNamespace;
        /**
         * Does a raycast in the physics world
         * @param from when should the ray start?
         * @param to when should the ray end?
         * @returns PhysicsRaycastResult
         */
        raycast(from: Vector3, to: Vector3): PhysicsRaycastResult;
        /**
         * Does a raycast in the physics world
         * @param from when should the ray start?
         * @param to when should the ray end?
         * @param result resulting PhysicsRaycastResult
         */
        raycastToRef(from: Vector3, to: Vector3, result: PhysicsRaycastResult): void;
    }




    /** @internal */
    export class OimoJSPlugin implements IPhysicsEnginePlugin {
        private _useDeltaForWorldStep;
        world: any;
        name: string;
        BJSOIMO: any;
        private _raycastResult;
        private _fixedTimeStep;
        constructor(_useDeltaForWorldStep?: boolean, iterations?: number, oimoInjection?: any);
        /**
         *
         * @returns plugin version
         */
        getPluginVersion(): number;
        setGravity(gravity: Vector3): void;
        setTimeStep(timeStep: number): void;
        getTimeStep(): number;
        private _tmpImpostorsArray;
        executeStep(delta: number, impostors: Array<PhysicsImpostor>): void;
        applyImpulse(impostor: PhysicsImpostor, force: Vector3, contactPoint: Vector3): void;
        applyForce(impostor: PhysicsImpostor, force: Vector3, contactPoint: Vector3): void;
        generatePhysicsBody(impostor: PhysicsImpostor): void;
        private _tmpPositionVector;
        removePhysicsBody(impostor: PhysicsImpostor): void;
        generateJoint(impostorJoint: PhysicsImpostorJoint): void;
        removeJoint(impostorJoint: PhysicsImpostorJoint): void;
        isSupported(): boolean;
        setTransformationFromPhysicsBody(impostor: PhysicsImpostor): void;
        setPhysicsBodyTransformation(impostor: PhysicsImpostor, newPosition: Vector3, newRotation: Quaternion): void;
        setLinearVelocity(impostor: PhysicsImpostor, velocity: Vector3): void;
        setAngularVelocity(impostor: PhysicsImpostor, velocity: Vector3): void;
        getLinearVelocity(impostor: PhysicsImpostor): Nullable<Vector3>;
        getAngularVelocity(impostor: PhysicsImpostor): Nullable<Vector3>;
        setBodyMass(impostor: PhysicsImpostor, mass: number): void;
        getBodyMass(impostor: PhysicsImpostor): number;
        getBodyFriction(impostor: PhysicsImpostor): number;
        setBodyFriction(impostor: PhysicsImpostor, friction: number): void;
        getBodyRestitution(impostor: PhysicsImpostor): number;
        setBodyRestitution(impostor: PhysicsImpostor, restitution: number): void;
        sleepBody(impostor: PhysicsImpostor): void;
        wakeUpBody(impostor: PhysicsImpostor): void;
        updateDistanceJoint(joint: PhysicsJoint, maxDistance: number, minDistance?: number): void;
        setMotor(joint: IMotorEnabledJoint, speed: number, force?: number, motorIndex?: number): void;
        setLimit(joint: IMotorEnabledJoint, upperLimit: number, lowerLimit?: number, motorIndex?: number): void;
        syncMeshWithImpostor(mesh: AbstractMesh, impostor: PhysicsImpostor): void;
        getRadius(impostor: PhysicsImpostor): number;
        getBoxSizeToRef(impostor: PhysicsImpostor, result: Vector3): void;
        dispose(): void;
        /**
         * Does a raycast in the physics world
         * @param from when should the ray start?
         * @param to when should the ray end?
         * @returns PhysicsRaycastResult
         */
        raycast(from: Vector3, to: Vector3): PhysicsRaycastResult;
        /**
         * Does a raycast in the physics world
         * @param from when should the ray start?
         * @param to when should the ray end?
         * @param result resulting PhysicsRaycastResult
         */
        raycastToRef(from: Vector3, to: Vector3, result: PhysicsRaycastResult): void;
    }




    /** @internal */
    export enum ConstraintAxisLimitMode {
        FREE = 0,
        LIMITED = 1,
        LOCKED = 2,
        NONE = 3
    }
    /** @internal */
    export enum ConstraintAxis {
        LINEAR_X = 0,
        LINEAR_Y = 1,
        LINEAR_Z = 2,
        ANGULAR_X = 3,
        ANGULAR_Y = 4,
        ANGULAR_Z = 5,
        LINEAR_DISTANCE = 6
    }
    /** @internal */
    export enum ConstraintType {
        BALL_AND_SOCKET = 0,
        DISTANCE = 1,
        HINGE = 2,
        SLIDER = 3,
        LOCK = 4
    }
    /** @internal */
    export enum ShapeType {
        SPHERE = 0,
        CAPSULE = 1,
        CYLINDER = 2,
        BOX = 3,
        CONVEX_HULL = 4,
        CONTAINER = 5,
        MESH = 6,
        HEIGHTFIELD = 7
    }
    /** @internal */
    export enum ConstraintMotorType {
        NONE = 0,
        VELOCITY = 1,
        POSITION = 2
    }
    /** @internal */
    export interface PhysicsShapeParameters {
        center?: Vector3;
        radius?: number;
        pointA?: Vector3;
        pointB?: Vector3;
        rotation?: Quaternion;
        extents?: Vector3;
        mesh?: Mesh;
        includeChildMeshes?: boolean;
    }
    /** @internal */
    export interface PhysicsConstraintParameters {
        pivotA?: Vector3;
        pivotB?: Vector3;
        axisA?: Vector3;
        axisB?: Vector3;
    }
    /**
     *
     */
    /** @internal */
    export interface MassProperties {
        /**
         *
         */
        centerOfMass: Vector3;
        /**
         *
         */
        mass: number;
        /**
         *
         */
        inertia: Vector3;
        /**
         *
         */
        inertiaOrientation: Quaternion;
    }
    /** @internal */
    export interface IPhysicsEnginePluginV2 {
        /**
         *
         */
        world: any;
        /**
         *
         */
        name: string;
        setGravity(gravity: Vector3): void;
        setTimeStep(timeStep: number): void;
        getTimeStep(): number;
        executeStep(delta: number, bodies: Array<PhysicsBody>): void;
        getPluginVersion(): number;
        registerOnCollide(func: (collider: PhysicsBody, collidedAgainst: PhysicsBody, point: Nullable<Vector3>) => void): void;
        unregisterOnCollide(func: (collider: PhysicsBody, collidedAgainst: PhysicsBody, point: Nullable<Vector3>) => void): void;
        initBody(body: PhysicsBody, position: Vector3, orientation: Quaternion): void;
        initBodyInstances(body: PhysicsBody, mesh: Mesh): void;
        removeBody(body: PhysicsBody): void;
        sync(body: PhysicsBody): void;
        syncTransform(body: PhysicsBody, transformNode: TransformNode): void;
        setShape(body: PhysicsBody, shape: PhysicsShape): void;
        getShape(body: PhysicsBody): PhysicsShape;
        setFilterGroup(body: PhysicsBody, group: number): void;
        getFilterGroup(body: PhysicsBody): number;
        setEventMask(body: PhysicsBody, eventMask: number): void;
        getEventMask(body: PhysicsBody): number;
        setMassProperties(body: PhysicsBody, massProps: MassProperties): void;
        getMassProperties(body: PhysicsBody): MassProperties;
        setLinearDamping(body: PhysicsBody, damping: number): void;
        getLinearDamping(body: PhysicsBody): number;
        setAngularDamping(body: PhysicsBody, damping: number): void;
        getAngularDamping(body: PhysicsBody): number;
        setLinearVelocity(body: PhysicsBody, linVel: Vector3): void;
        getLinearVelocityToRef(body: PhysicsBody, linVel: Vector3): void;
        applyImpulse(body: PhysicsBody, location: Vector3, impulse: Vector3): void;
        applyForce(body: PhysicsBody, location: Vector3, force: Vector3): void;
        setAngularVelocity(body: PhysicsBody, angVel: Vector3): void;
        getAngularVelocityToRef(body: PhysicsBody, angVel: Vector3): void;
        getBodyGeometry(body: PhysicsBody): {};
        disposeBody(body: PhysicsBody): void;
        registerOnBodyCollide(body: PhysicsBody, func: (collider: PhysicsBody, collidedAgainst: PhysicsBody, point: Nullable<Vector3>) => void): void;
        unregisterOnBodyCollide(body: PhysicsBody, func: (collider: PhysicsBody, collidedAgainst: PhysicsBody, point: Nullable<Vector3>) => void): void;
        initShape(shape: PhysicsShape, type: ShapeType, options: PhysicsShapeParameters): void;
        setFilterLayer(shape: PhysicsShape, layer: number): void;
        getFilterLayer(shape: PhysicsShape): number;
        setMaterial(shape: PhysicsShape, material: PhysicsMaterial): void;
        getMaterial(shape: PhysicsShape): PhysicsMaterial;
        setDensity(shape: PhysicsShape, density: number): void;
        getDensity(shape: PhysicsShape): number;
        addChild(shape: PhysicsShape, newChild: PhysicsShape, childTransform: TransformNode): void;
        removeChild(shape: PhysicsShape, childIndex: number): void;
        getNumChildren(shape: PhysicsShape): number;
        getBoundingBox(shape: PhysicsShape): BoundingBox;
        disposeShape(shape: PhysicsShape): void;
        initMaterial(material: PhysicsMaterial): void;
        setFriction(material: PhysicsMaterial, friction: number): void;
        getFriction(material: PhysicsMaterial): number;
        setRestitution(material: PhysicsMaterial, restitution: number): void;
        getRestitution(material: PhysicsMaterial): number;
        disposeMaterial(material: PhysicsMaterial): void;
        initConstraint(constraint: PhysicsConstraint, type: ConstraintType, options: PhysicsConstraintParameters): void;
        setParentBody(constraint: PhysicsConstraint, body: PhysicsBody): void;
        getParentBody(constraint: PhysicsConstraint): PhysicsBody;
        setChildBody(constraint: PhysicsConstraint, body: PhysicsBody): void;
        getChildBody(constraint: PhysicsConstraint): PhysicsBody;
        setAnchorInParent(constraint: PhysicsConstraint, pivot: Vector3, axisX: Vector3, axisY: Vector3): void;
        setAnchorInChild(constraint: PhysicsConstraint, pivot: Vector3, axisX: Vector3, axisY: Vector3): void;
        setEnabled(constraint: PhysicsConstraint, isEnabled: boolean): void;
        getEnabled(constraint: PhysicsConstraint): boolean;
        setCollisionsEnabled(constraint: PhysicsConstraint, isEnabled: boolean): void;
        getCollisionsEnabled(constraint: PhysicsConstraint): boolean;
        setAxisFriction(constraint: PhysicsConstraint, axis: ConstraintAxis, friction: number): void;
        getAxisFriction(constraint: PhysicsConstraint, axis: ConstraintAxis): number;
        setAxisMode(constraint: PhysicsConstraint, axis: ConstraintAxis, limitMode: ConstraintAxisLimitMode): void;
        getAxisMode(constraint: PhysicsConstraint, axis: ConstraintAxis): ConstraintAxisLimitMode;
        setAxisMinLimit(constraint: PhysicsConstraint, axis: ConstraintAxis, minLimit: number): void;
        getAxisMinLimit(constraint: PhysicsConstraint, axis: ConstraintAxis): number;
        setAxisMaxLimit(constraint: PhysicsConstraint, axis: ConstraintAxis, limit: number): void;
        getAxisMaxLimit(constraint: PhysicsConstraint, axis: ConstraintAxis): number;
        setAxisMotorType(constraint: PhysicsConstraint, axis: ConstraintAxis, motorType: ConstraintMotorType): void;
        getAxisMotorType(constraint: PhysicsConstraint, axis: ConstraintAxis): ConstraintMotorType;
        setAxisMotorTarget(constraint: PhysicsConstraint, axis: ConstraintAxis, target: number): void;
        getAxisMotorTarget(constraint: PhysicsConstraint, axis: ConstraintAxis): number;
        setAxisMotorMaxForce(constraint: PhysicsConstraint, axis: ConstraintAxis, maxForce: number): void;
        getAxisMotorMaxForce(constraint: PhysicsConstraint, axis: ConstraintAxis): number;
        disposeConstraint(constraint: PhysicsConstraint): void;
        raycast(from: Vector3, to: Vector3, result: PhysicsRaycastResult): void;
        dispose(): void;
    }


    /**
     * The interface for the physics aggregate parameters
     */
    export interface PhysicsAggregateParameters {
        /**
         * The mass of the physics aggregate
         */
        mass: number;
        /**
         * The friction of the physics aggregate
         */
        friction?: number;
        /**
         * The coefficient of restitution of the physics aggregate
         */
        restitution?: number;
        /**
         * The native options of the physics aggregate
         */
        nativeOptions?: any;
        /**
         * Specifies if the parent should be ignored
         */
        ignoreParent?: boolean;
        /**
         * Specifies if bi-directional transformations should be disabled
         */
        disableBidirectionalTransformation?: boolean;
        /**
         * The pressure inside the physics aggregate, soft object only
         */
        pressure?: number;
        /**
         * The stiffness the physics aggregate, soft object only
         */
        stiffness?: number;
        /**
         * The number of iterations used in maintaining consistent vertex velocities, soft object only
         */
        velocityIterations?: number;
        /**
         * The number of iterations used in maintaining consistent vertex positions, soft object only
         */
        positionIterations?: number;
        /**
         * The number used to fix points on a cloth (0, 1, 2, 4, 8) or rope (0, 1, 2) only
         * 0 None, 1, back left or top, 2, back right or bottom, 4, front left, 8, front right
         * Add to fix multiple points
         */
        fixedPoints?: number;
        /**
         * The collision margin around a soft object
         */
        margin?: number;
        /**
         * The collision margin around a soft object
         */
        damping?: number;
        /**
         * The path for a rope based on an extrusion
         */
        path?: any;
        /**
         * The shape of an extrusion used for a rope based on an extrusion
         */
        shape?: any;
        /**
         * Radius for sphere, cylinder and capsule
         */
        radius?: number;
        /**
         * Starting point for cylinder/capsule
         */
        pointA?: Vector3;
        /**
         * Ending point for cylinder/capsule
         */
        pointB?: Vector3;
        /**
         * Extents for box
         */
        extents?: Vector3;
    }
    /**
     * Helper class to create and interact with a PhysicsAggregate.
     * This is a transition object that works like Physics Plugin V1 Impostors.
     * This helper instanciate all mandatory physics objects to get a body/shape and material.
     * It's less efficient that handling body and shapes independently but for prototyping or
     * a small numbers of physics objects, it's good enough.
     */
    export class PhysicsAggregate {
        /**
         * The physics-enabled object used as the physics aggregate
         */
        transformNode: TransformNode;
        /**
         * The type of the physics aggregate
         */
        type: number;
        private _options;
        private _scene?;
        /**
         * The body that is associated with this aggregate
         */
        body: PhysicsBody;
        /**
         * The shape that is associated with this aggregate
         */
        shape: PhysicsShape;
        /**
         * The material that is associated with this aggregate
         */
        material: PhysicsMaterial;
        constructor(
        /**
         * The physics-enabled object used as the physics aggregate
         */
        transformNode: TransformNode, 
        /**
         * The type of the physics aggregate
         */
        type: number, _options?: PhysicsAggregateParameters, _scene?: Scene | undefined);
        private _addSizeOptions;
        /**
         * Releases the body, shape and material
         */
        dispose(): void;
    }


    /**
     * PhysicsBody is useful for creating a physics body that can be used in a physics engine. It allows
     * the user to set the mass and velocity of the body, which can then be used to calculate the
     * motion of the body in the physics engine.
     */
    export class PhysicsBody {
        /**
         * V2 Physics plugin private data for single Transform
         */
        _pluginData: any;
        /**
         * V2 Physics plugin private data for instances
         */
        _pluginDataInstances: Array<any>;
        /**
         * The V2 plugin used to create and manage this Physics Body
         */
        private _physicsPlugin;
        /**
         * The engine used to create and manage this Physics Body
         */
        private _physicsEngine;
        /**
         * The transform node associated with this Physics Body
         */
        transformNode: TransformNode;
        /**
         * Disable pre-step that consists in updating Physics Body from Transform Node Translation/Orientation.
         * True by default for maximum performance.
         */
        disablePreStep: boolean;
        private static _DEFAULT_OBJECT_SIZE;
        private static _IDENTITY_QUATERNION;
        /**
         * Constructs a new physics body for the given node.
         * @param transformNode - The Transform Node to construct the physics body for.
         * @param scene - The scene containing the physics engine.
         *
         * This code is useful for creating a physics body for a given Transform Node in a scene.
         * It checks the version of the physics engine and the physics plugin, and initializes the body accordingly.
         * It also sets the node's rotation quaternion if it is not already set. Finally, it adds the body to the physics engine.
         */
        constructor(transformNode: TransformNode, scene: Scene);
        /**
         * Sets the shape of the physics body.
         * @param shape - The shape of the physics body.
         *
         * This method is useful for setting the shape of the physics body, which is necessary for the physics engine to accurately simulate the body's behavior.
         * The shape is used to calculate the body's mass, inertia, and other properties.
         */
        setShape(shape: PhysicsShape): void;
        /**
         * Retrieves the physics shape associated with this object.
         *
         * @returns The physics shape associated with this object, or `undefined` if no
         * shape is associated.
         *
         * This method is useful for retrieving the physics shape associated with this object,
         * which can be used to apply physical forces to the object or to detect collisions.
         */
        getShape(): PhysicsShape | undefined;
        /**
         * Sets the filter group of the physics body.
         * @param group - The filter group of the physics body.
         *
         * This method is useful for setting the filter group of the physics body.
         * The filter group is used to determine which bodies should collide with each other.
         * This allows for more control over the physics engine and can be used to create more realistic simulations.
         */
        setFilterGroup(group: number): void;
        /**
         * Gets the filter group of the physics engine.
         *
         * @returns The filter group of the physics engine.
         *
         * This method is useful for getting the filter group of the physics engine,
         * which is used to determine which objects will interact with each other.
         * This is important for creating realistic physics simulations.
         */
        getFilterGroup(): number;
        /**
         * Sets the event mask for the physics engine.
         *
         * @param eventMask - A bitmask that determines which events will be sent to the physics engine.
         *
         * This method is useful for setting the event mask for the physics engine, which determines which events
         * will be sent to the physics engine. This allows the user to control which events the physics engine will respond to.
         */
        setEventMask(eventMask: number): void;
        /**
         * Gets the event mask of the physics engine.
         *
         * @returns The event mask of the physics engine.
         *
         * This method is useful for getting the event mask of the physics engine,
         * which is used to determine which events the engine will respond to.
         * This is important for ensuring that the engine is responding to the correct events and not
         * wasting resources on unnecessary events.
         */
        getEventMask(): number;
        /**
         * Sets the mass properties of the physics object.
         *
         * @param massProps - The mass properties to set.
         *
         * This method is useful for setting the mass properties of a physics object, such as its mass,
         * inertia, and center of mass. This is important for accurately simulating the physics of the object in the physics engine.
         */
        setMassProperties(massProps: MassProperties): void;
        /**
         * Retrieves the mass properties of the object.
         *
         * @returns The mass properties of the object, or `undefined` if the physics
         * plugin does not support mass properties.
         *
         * This method is useful for physics simulations, as it allows the user to
         * retrieve the mass properties of the object, such as its mass, center of mass,
         * and moment of inertia. This information is necessary for accurate physics
         * simulations.
         */
        getMassProperties(): MassProperties | undefined;
        /**
         * Sets the linear damping of the physics body.
         *
         * @param damping - The linear damping value.
         *
         * This method is useful for controlling the linear damping of the physics body,
         * which is the rate at which the body's velocity decreases over time. This is useful for simulating
         * the effects of air resistance or other forms of friction.
         */
        setLinearDamping(damping: number): void;
        /**
         * Gets the linear damping of the physics body.
         * @returns The linear damping of the physics body.
         *
         * This method is useful for retrieving the linear damping of the physics body, which is the amount of
         * resistance the body has to linear motion. This is useful for simulating realistic physics behavior
         * in a game.
         */
        getLinearDamping(): number;
        /**
         * Sets the angular damping of the physics body.
         * @param damping The angular damping of the body.
         *
         * This method is useful for controlling the angular velocity of a physics body.
         * By setting the damping, the body's angular velocity will be reduced over time, simulating the effect of friction.
         * This can be used to create realistic physical behavior in a physics engine.
         */
        setAngularDamping(damping: number): void;
        /**
         * Gets the angular damping of the physics body.
         *
         * @returns The angular damping of the physics body.
         *
         * This method is useful for getting the angular damping of the physics body,
         * which is the rate of reduction of the angular velocity over time.
         * This is important for simulating realistic physics behavior in a game.
         */
        getAngularDamping(): number;
        /**
         * Sets the linear velocity of the physics object.
         * @param linVel - The linear velocity to set.
         *
         * This method is useful for setting the linear velocity of a physics object,
         * which is necessary for simulating realistic physics in a game engine.
         * By setting the linear velocity, the physics object will move in the direction and speed specified by the vector.
         * This allows for realistic physics simulations, such as simulating the motion of a ball rolling down a hill.
         */
        setLinearVelocity(linVel: Vector3): void;
        /**
         * Gets the linear velocity of the physics body and stores it in the given vector3.
         * @param linVel - The vector3 to store the linear velocity in.
         *
         * This method is useful for getting the linear velocity of a physics body in a physics engine.
         * This can be used to determine the speed and direction of the body, which can be used to calculate the motion of the body.*/
        getLinearVelocityToRef(linVel: Vector3): void;
        /**
         * Sets the angular velocity of the physics object.
         * @param angVel - The angular velocity to set.
         *
         * This method is useful for setting the angular velocity of a physics object, which is necessary for
         * simulating realistic physics behavior. The angular velocity is used to determine the rate of rotation of the object,
         * which is important for simulating realistic motion.
         */
        setAngularVelocity(angVel: Vector3): void;
        /**
         * Gets the angular velocity of the physics body and stores it in the given vector3.
         * @param angVel - The vector3 to store the angular velocity in.
         *
         * This method is useful for getting the angular velocity of a physics body, which can be used to determine the body's
         * rotational speed. This information can be used to create realistic physics simulations.
         */
        getAngularVelocityToRef(angVel: Vector3): void;
        /**
         * Applies an impulse to the physics object.
         *
         * @param location The location of the impulse.
         * @param impulse The impulse vector.
         *
         * This method is useful for applying an impulse to a physics object, which can be used to simulate physical forces such as gravity,
         * collisions, and explosions. This can be used to create realistic physics simulations in a game or other application.
         */
        applyImpulse(location: Vector3, impulse: Vector3): void;
        /**
         * Applies a force to the physics object.
         *
         * @param location The location of the force.
         * @param force The force vector.
         *
         * This method is useful for applying a force to a physics object, which can be used to simulate physical forces such as gravity,
         * collisions, and explosions. This can be used to create realistic physics simulations in a game or other application.
         */
        applyForce(location: Vector3, force: Vector3): void;
        /**
         * Retrieves the geometry of the body from the physics plugin.
         *
         * @returns The geometry of the body.
         *
         * This method is useful for retrieving the geometry of the body from the physics plugin, which can be used for various physics calculations.
         */
        getGeometry(): {};
        /**
         * Register a collision callback that is called when the body collides
         * Filtering by body is inefficient. It's more preferable to register a collision callback for the entire world
         * and do the filtering on the user side.
         */
        registerOnCollide(func: (collider: PhysicsBody, collidedAgainst: PhysicsBody, point: Nullable<Vector3>) => void): void;
        /**
         * Unregister a collision callback that is called when the body collides
         */
        unregisterOnCollide(func: (collider: PhysicsBody, collidedAgainst: PhysicsBody, point: Nullable<Vector3>) => void): void;
        /**
         * Gets the object extents
         * @returns the object extents
         */
        getObjectExtents(): Vector3;
        /**
         * return geometric center of the associated mesh
         */
        getObjectCenter(): Vector3;
        /**
         * Disposes the body from the physics engine.
         *
         * This method is useful for cleaning up the physics engine when a body is no longer needed. Disposing the body will free up resources and prevent memory leaks.
         */
        dispose(): void;
    }


    /**
     * This is a holder class for the physics constraint created by the physics plugin
     * It holds a set of functions to control the underlying constraint
     * @see https://doc.babylonjs.com/features/featuresDeepDive/physics/usingPhysicsEngine
     */
    export class PhysicsConstraint {
        /**
         * V2 Physics plugin private data for a physics material
         */
        _pluginData: any;
        /**
         * The V2 plugin used to create and manage this Physics Body
         */
        protected _physicsPlugin: IPhysicsEnginePluginV2;
        /**
         * Constructs a new constraint for the physics constraint.
         * @param type The type of constraint to create.
         * @param options The options for the constraint.
         * @param scene The scene the constraint belongs to.
         *
         * This code is useful for creating a new constraint for the physics engine. It checks if the scene has a physics engine, and if the plugin version is correct.
         * If all checks pass, it initializes the constraint with the given type and options.
         */
        constructor(type: ConstraintType, options: PhysicsConstraintParameters, scene: Scene);
        /**
         * Sets the parent body of this body.
         * @param body - The parent body to set.
         *
         * This method is useful for setting the parent body of a physics body in a physics engine.
         * This allows the engine to accurately simulate the motion of the body in relation to its parent body.
         * For example, if the parent body is a planet, the engine can accurately simulate the motion of the body in relation to the planet's gravity.
         */
        setParentBody(body: PhysicsBody): void;
        /**
         * Retrieves the parent body of the current physics constraint.
         *
         * @returns The parent body of the current physics constraint, or `undefined` if the
         * current constraint does not have a parent body.
         */
        getParentBody(): PhysicsBody | undefined;
        /**
         * Sets the child body of the current body.
         *
         * @param body - The child body to set.
         *
         * This method is useful for setting the child body of the current body in a physics engine.
         * This allows for the creation of complex structures of bodies that interact with each other in a realistic way.
         */
        setChildBody(body: PhysicsBody): void;
        /**
         * Retrieves the child body of the current physics constraint.
         *
         * @returns The child body of the current physics constraint, or `undefined` if no
         * child body is present.
         */
        getChildBody(): PhysicsBody | undefined;
        /**
         *
         * @param pivot +
         * @param axisX
         * @param axisY
         */
        setAnchorInParent(pivot: Vector3, axisX: Vector3, axisY: Vector3): void;
        /**
         *
         * @param pivot
         * @param axisX
         * @param axisY
         */
        setAnchorInChild(pivot: Vector3, axisX: Vector3, axisY: Vector3): void;
        /**
         *
         * @param isEnabled
         */
        setEnabled(isEnabled: boolean): void;
        /**
         *
         * @returns
         */
        getEnabled(): boolean;
        /**
         *
         * @param isEnabled
         */
        setCollisionsEnabled(isEnabled: boolean): void;
        /**
         *
         * @returns
         */
        getCollisionsEnabled(): boolean;
        /**
         *
         * @param axis
         * @param friction
         */
        setAxisFriction(axis: ConstraintAxis, friction: number): void;
        /**
         *
         * @param axis
         * @returns
         */
        getAxisFriction(axis: ConstraintAxis): number;
        /**
         *
         * @param axis
         * @param limitMode
         */
        setAxisMode(axis: ConstraintAxis, limitMode: ConstraintAxisLimitMode): void;
        /**
         *
         * @param axis
         */
        getAxisMode(axis: ConstraintAxis): ConstraintAxisLimitMode;
        /**
         *
         */
        setAxisMinLimit(axis: ConstraintAxis, minLimit: number): void;
        /**
         *
         */
        getAxisMinLimit(axis: ConstraintAxis): number;
        /**
         *
         */
        setAxisMaxLimit(axis: ConstraintAxis, limit: number): void;
        /**
         *
         */
        getAxisMaxLimit(axis: ConstraintAxis): number;
        /**
         *
         */
        setAxisMotorType(axis: ConstraintAxis, motorType: ConstraintMotorType): void;
        /**
         *
         */
        getAxisMotorType(axis: ConstraintAxis): ConstraintMotorType;
        /**
         *
         */
        setAxisMotorTarget(axis: ConstraintAxis, target: number): void;
        /**
         *
         */
        getAxisMotorTarget(axis: ConstraintAxis): number;
        /**
         *
         */
        setAxisMotorMaxForce(axis: ConstraintAxis, maxForce: number): void;
        /**
         *
         */
        getAxisMotorMaxForce(axis: ConstraintAxis): number;
        /**
         *
         */
        dispose(): void;
    }
    /**
     *
     */
    /** @internal */
    export class PhysicsConstraintBallAndSocket extends PhysicsConstraint {
        /** @internal */
        constructor(pivotA: Vector3, pivotB: Vector3, axisA: Vector3, axisB: Vector3, scene: Scene);
    }
    /**
     *
     */
    /** @internal */
    export class PhysicsConstraintDistance extends PhysicsConstraint {
        /** @internal */
        constructor(pivotA: Vector3, pivotB: Vector3, axisA: Vector3, axisB: Vector3, scene: Scene);
    }
    /**
     *
     */
    /** @internal */
    export class PhysicsConstraintHinge extends PhysicsConstraint {
        /** @internal */
        constructor(pivotA: Vector3, pivotB: Vector3, axisA: Vector3, axisB: Vector3, scene: Scene);
    }
    /**
     *
     */
    /** @internal */
    export class PhysicsConstraintSlider extends PhysicsConstraint {
        /** @internal */
        constructor(pivotA: Vector3, pivotB: Vector3, axisA: Vector3, axisB: Vector3, scene: Scene);
    }
    /**
     *
     */
    /** @internal */
    export class PhysicsConstraintLock extends PhysicsConstraint {
        /** @internal */
        constructor(pivotA: Vector3, pivotB: Vector3, axisA: Vector3, axisB: Vector3, scene: Scene);
    }


    /**
     * Class used to control physics engine
     * @see https://doc.babylonjs.com/features/featuresDeepDive/physics/usingPhysicsEngine
     */
    /** @internal */
    export class PhysicsEngineV2 implements IPhysicsEngine {
        private _physicsPlugin;
        /** @internal */
        private _physicsBodies;
        private _subTimeStep;
        /**
         * Gets the gravity vector used by the simulation
         */
        gravity: Vector3;
        /**
         *
         * @returns physics plugin version
         */
        getPluginVersion(): number;
        /**
         * Factory used to create the default physics plugin.
         * @returns The default physics plugin
         */
        static DefaultPluginFactory(): IPhysicsEnginePluginV2;
        /**
         * Creates a new Physics Engine
         * @param gravity defines the gravity vector used by the simulation
         * @param _physicsPlugin defines the plugin to use (CannonJS by default)
         */
        constructor(gravity: Nullable<Vector3>, _physicsPlugin?: IPhysicsEnginePluginV2);
        /**
         * Sets the gravity vector used by the simulation
         * @param gravity defines the gravity vector to use
         */
        setGravity(gravity: Vector3): void;
        /**
         * Set the time step of the physics engine.
         * Default is 1/60.
         * To slow it down, enter 1/600 for example.
         * To speed it up, 1/30
         * @param newTimeStep defines the new timestep to apply to this world.
         */
        setTimeStep(newTimeStep?: number): void;
        /**
         * Get the time step of the physics engine.
         * @returns the current time step
         */
        getTimeStep(): number;
        /**
         * Set the sub time step of the physics engine.
         * Default is 0 meaning there is no sub steps
         * To increase physics resolution precision, set a small value (like 1 ms)
         * @param subTimeStep defines the new sub timestep used for physics resolution.
         */
        setSubTimeStep(subTimeStep?: number): void;
        /**
         * Get the sub time step of the physics engine.
         * @returns the current sub time step
         */
        getSubTimeStep(): number;
        /**
         * Release all resources
         */
        dispose(): void;
        /**
         * Gets the name of the current physics plugin
         * @returns the name of the plugin
         */
        getPhysicsPluginName(): string;
        /**
         * Adding a new impostor for the impostor tracking.
         * This will be done by the impostor itself.
         * @param impostor the impostor to add
         */
        /**
         * Called by the scene. No need to call it.
         * @param delta defines the timespan between frames
         */
        _step(delta: number): void;
        /**
         * Add a body as an active component of this engine
         * @param body
         */
        addBody(physicsBody: PhysicsBody): void;
        /**
         * Removes a particular body from this engine
         */
        removeBody(physicsBody: PhysicsBody): void;
        /**
         * Returns an array of bodies added to this engine
         */
        getBodies(): Array<PhysicsBody>;
        /**
         * Gets the current plugin used to run the simulation
         * @returns current plugin
         */
        getPhysicsPlugin(): IPhysicsEnginePluginV2;
        /**
         * Does a raycast in the physics world
         * @param from when should the ray start?
         * @param to when should the ray end?
         * @param result resulting PhysicsRaycastResult
         */
        raycastToRef(from: Vector3, to: Vector3, result: PhysicsRaycastResult): void;
        /**
         * Does a raycast in the physics world
         * @param from when should the ray start?
         * @param to when should the ray end?
         * @returns PhysicsRaycastResult
         */
        raycast(from: Vector3, to: Vector3): PhysicsRaycastResult;
    }


        /**
         *
         */
        /** @internal */
        interface TransformNode {
            /** @internal */
            _physicsBody: Nullable<PhysicsBody>;
            /**
             * @see
             */
            physicsBody: Nullable<PhysicsBody>;
            /**
             *
             */
            getPhysicsBody(): Nullable<PhysicsBody>;
            /** Apply a physic impulse to the mesh
             * @param force defines the force to apply
             * @param contactPoint defines where to apply the force
             * @returns the current mesh
             */
            applyImpulse(force: Vector3, contactPoint: Vector3): TransformNode;
            /** @internal */
            _disposePhysicsObserver: Nullable<Observer<Node>>;
        }


    /**
     * Physics material class
     * Helps setting friction and restitution that are used to compute responding forces in collision response
     */
    export class PhysicsMaterial {
        /**
         * V2 Physics plugin private data for a physics material
         */
        _pluginData: any;
        /**
         * The V2 plugin used to create and manage this Physics Body
         */
        protected _physicsPlugin: IPhysicsEnginePluginV2;
        /**
         * Constructor of the physics material.
         * @param friction - The friction of the material.
         * @param restitution - The restitution of the material.
         * @param scene - The scene to which the physics engine belongs.
         *
         * This code is useful for initializing the physics material with the
         * given friction, restitution and scene. It also checks if the
         * physics engine and plugin are available and if the plugin version
         * is correct. If all conditions are met, it initializes the material
         * with the physics plugin.
         */
        constructor(friction: number, restitution: number, scene: Scene);
        /**
         * Sets the friction of the physics engine.
         * @param friction - The friction to set.
         *
         * This method is useful for setting the friction of the physics engine, which is important for simulating realistic physics.
         * The friction determines how much an object will slow down when it is in contact with another object.
         * This is important for simulating realistic physics, such as when an object slides across a surface.
         */
        setFriction(friction: number): void;
        /**
         * Gets the friction of the physics engine.
         *
         * @returns The friction of the physics engine.
         *
         * This method is useful for getting the friction of the physics engine, which is used to calculate the force of friction between two objects.
         * Knowing the friction of the engine can help to accurately simulate the physical behavior of objects in the engine.
         */
        getFriction(): number;
        /**
         * Sets the restitution of the physics body.
         * @param restitution A number between 0 and 1 that represents the restitution of the body.
         *
         * This method is useful for setting the restitution of a physics body, which is the amount of energy that is retained after a collision.
         * A restitution of 0 means that no energy is retained, while a restitution of 1 means that all energy is retained.
         * Setting the restitution of a body can help to create realistic physics simulations.
         */
        setRestitution(restitution: number): void;
        /**
         * Gets the restitution of the physics engine.
         * @returns The restitution of the physics engine.
         *
         * This method is useful for retrieving the restitution of the physics engine, which is the amount of energy that is preserved after two objects collide.
         * Knowing the restitution of the physics engine can help to accurately simulate the behavior of objects in the physics engine.
         */
        getRestitution(): number;
        /**
         * Disposes the material.
         *
         * This method is useful for cleaning up the material when it is no longer needed.
         * It calls the disposeMaterial method of the physics plugin, which is responsible for disposing the material and freeing up any resources associated with it.
         * This ensures that the material is properly disposed of and does not cause any memory leaks.
         */
        dispose(): void;
    }


    /**
     * PhysicsShape class.
     * This class is useful for creating a physics shape that can be used in a physics engine.
     * A Physic Shape determine how collision are computed. It must be attached to a body.
     */
    export class PhysicsShape {
        /**
         * V2 Physics plugin private data for single shape
         */
        _pluginData: any;
        /**
         * The V2 plugin used to create and manage this Physics Body
         */
        private _physicsPlugin;
        private _type;
        /**
         * Constructs a new physics shape.
         * @param type The type of the shape.
         * @param options The options of the shape.
         * @param scene The scene the shape belongs to.
         *
         * This code is useful for creating a new physics shape with the given type, options, and scene.
         * It also checks that the physics engine and plugin version are correct.
         * If not, it throws an error. This ensures that the shape is created with the correct parameters and is compatible with the physics engine.
         */
        constructor(type: number, options: PhysicsShapeParameters | undefined, scene: Scene);
        /**
         *
         */
        get type(): ShapeType;
        /**
         *
         * @param layer
         */
        setFilterLayer(layer: number): void;
        /**
         *
         * @returns
         */
        getFilterLayer(): number;
        /**
         *
         * @param materialId
         */
        setMaterial(material: PhysicsMaterial): void;
        /**
         *
         * @returns
         */
        getMaterial(): PhysicsMaterial | undefined;
        /**
         *
         * @param density
         */
        setDensity(density: number): void;
        /**
         *
         */
        getDensity(): number;
        /**
         *
         * @param newChild
         * @param childTransform
         */
        addChild(newChild: PhysicsShape, childTransform: TransformNode): void;
        /**
         *
         * @param childIndex
         */
        removeChild(childIndex: number): void;
        /**
         *
         * @returns
         */
        getNumChildren(): number;
        /**
         *
         */
        getBoundingBox(): BoundingBox;
        /**
         *
         */
        dispose(): void;
    }
    /**
     * Helper object to create a sphere shape
     */
    export class PhysicsShapeSphere extends PhysicsShape {
        /** @internal */
        /**
         * Constructor for the Sphere Shape
         * @param center local center of the sphere
         * @param radius radius
         * @param scene scene to attach to
         */
        constructor(center: Vector3, radius: number, scene: Scene);
    }
    /***
     * Helper object to create a capsule shape
     */
    export class PhysicsShapeCapsule extends PhysicsShape {
        /**
         *
         * @param pointA Starting point that defines the capsule segment
         * @param pointB ending point of that same segment
         * @param radius radius
         * @param scene scene to attach to
         */
        constructor(pointA: Vector3, pointB: Vector3, radius: number, scene: Scene);
    }
    /**
     * Helper object to create a cylinder shape
     */
    export class PhysicsShapeCylinder extends PhysicsShape {
        /**
         *
         * @param pointA Starting point that defines the cylinder segment
         * @param pointB ending point of that same segment
         * @param radius radius
         * @param scene scene to attach to
         */
        constructor(pointA: Vector3, pointB: Vector3, radius: number, scene: Scene);
    }
    /**
     * Helper object to create a box shape
     */
    export class PhysicsShapeBox extends PhysicsShape {
        /**
         *
         * @param center local center of the sphere
         * @param rotation local orientation
         * @param extents size of the box in each direction
         * @param scene scene to attach to
         */
        constructor(center: Vector3, rotation: Quaternion, extents: Vector3, scene: Scene);
    }
    /**
     * Helper object to create a convex hull shape
     */
    export class PhysicsShapeConvexHull extends PhysicsShape {
        /**
         *
         * @param mesh the mesh to be used as topology infos for the convex hull
         * @param scene scene to attach to
         */
        constructor(mesh: Mesh, scene: Scene);
    }
    /**
     * Helper object to create a mesh shape
     */
    export class PhysicsShapeMesh extends PhysicsShape {
        /**
         *
         * @param mesh the mesh topology that will be used to create the shape
         * @param scene scene to attach to
         */
        constructor(mesh: Mesh, scene: Scene);
    }
    /**
     * A shape container holds a variable number of shapes. Use AddChild to append to newly created parent container.
     */
    export class PhysicsShapeContainer extends PhysicsShape {
        /**
         * Constructor of the Shape container
         * @param scene scene to attach to
         */
        constructor(scene: Scene);
    }




    /**
     * Postprocess used to generate anaglyphic rendering
     */
    export class AnaglyphPostProcess extends PostProcess {
        private _passedProcess;
        /**
         * Gets a string identifying the name of the class
         * @returns "AnaglyphPostProcess" string
         */
        getClassName(): string;
        /**
         * Creates a new AnaglyphPostProcess
         * @param name defines postprocess name
         * @param options defines creation options or target ratio scale
         * @param rigCameras defines cameras using this postprocess
         * @param samplingMode defines required sampling mode (BABYLON.Texture.NEAREST_SAMPLINGMODE by default)
         * @param engine defines hosting engine
         * @param reusable defines if the postprocess will be reused multiple times per frame
         */
        constructor(name: string, options: number | PostProcessOptions, rigCameras: Camera[], samplingMode?: number, engine?: Engine, reusable?: boolean);
    }


    /**
     * Post process used to render in black and white
     */
    export class BlackAndWhitePostProcess extends PostProcess {
        /**
         * Linear about to convert he result to black and white (default: 1)
         */
        degree: number;
        /**
         * Gets a string identifying the name of the class
         * @returns "BlackAndWhitePostProcess" string
         */
        getClassName(): string;
        /**
         * Creates a black and white post process
         * @see https://doc.babylonjs.com/features/featuresDeepDive/postProcesses/usePostProcesses#black-and-white
         * @param name The name of the effect.
         * @param options The required width/height ratio to downsize to before computing the render pass.
         * @param camera The camera to apply the render pass to.
         * @param samplingMode The sampling mode to be used when computing the pass. (default: 0)
         * @param engine The engine which the post process will be applied. (default: current engine)
         * @param reusable If the post process can be reused on the same frame. (default: false)
         */
        constructor(name: string, options: number | PostProcessOptions, camera: Nullable<Camera>, samplingMode?: number, engine?: Engine, reusable?: boolean);
        /**
         * @internal
         */
        static _Parse(parsedPostProcess: any, targetCamera: Camera, scene: Scene, rootUrl: string): Nullable<BlackAndWhitePostProcess>;
    }


    /**
     * The bloom effect spreads bright areas of an image to simulate artifacts seen in cameras
     */
    export class BloomEffect extends PostProcessRenderEffect {
        private _bloomScale;
        /**
         * @internal Internal
         */
        _effects: Array<PostProcess>;
        /**
         * @internal Internal
         */
        _downscale: ExtractHighlightsPostProcess;
        private _blurX;
        private _blurY;
        private _merge;
        /**
         * The luminance threshold to find bright areas of the image to bloom.
         */
        get threshold(): number;
        set threshold(value: number);
        /**
         * The strength of the bloom.
         */
        get weight(): number;
        set weight(value: number);
        /**
         * Specifies the size of the bloom blur kernel, relative to the final output size
         */
        get kernel(): number;
        set kernel(value: number);
        /**
         * Creates a new instance of @see BloomEffect
         * @param scene The scene the effect belongs to.
         * @param _bloomScale The ratio of the blur texture to the input texture that should be used to compute the bloom.
         * @param bloomWeight The the strength of bloom.
         * @param bloomKernel The size of the kernel to be used when applying the blur.
         * @param pipelineTextureType The type of texture to be used when performing the post processing.
         * @param blockCompilation If compilation of the shader should not be done in the constructor. The updateEffect method can be used to compile the shader at a later time. (default: false)
         */
        constructor(scene: Scene, _bloomScale: number, bloomWeight: number, bloomKernel: number, pipelineTextureType?: number, blockCompilation?: boolean);
        /**
         * Disposes each of the internal effects for a given camera.
         * @param camera The camera to dispose the effect on.
         */
        disposeEffects(camera: Camera): void;
        /**
         * @internal Internal
         */
        _updateEffects(): void;
        /**
         * Internal
         * @returns if all the contained post processes are ready.
         * @internal
         */
        _isReady(): boolean;
    }


    /**
     * The BloomMergePostProcess merges blurred images with the original based on the values of the circle of confusion.
     */
    export class BloomMergePostProcess extends PostProcess {
        /** Weight of the bloom to be added to the original input. */
        weight: number;
        /**
         * Gets a string identifying the name of the class
         * @returns "BloomMergePostProcess" string
         */
        getClassName(): string;
        /**
         * Creates a new instance of @see BloomMergePostProcess
         * @param name The name of the effect.
         * @param originalFromInput Post process which's input will be used for the merge.
         * @param blurred Blurred highlights post process which's output will be used.
         * @param weight Weight of the bloom to be added to the original input.
         * @param options The required width/height ratio to downsize to before computing the render pass.
         * @param camera The camera to apply the render pass to.
         * @param samplingMode The sampling mode to be used when computing the pass. (default: 0)
         * @param engine The engine which the post process will be applied. (default: current engine)
         * @param reusable If the post process can be reused on the same frame. (default: false)
         * @param textureType Type of textures used when performing the post process. (default: 0)
         * @param blockCompilation If compilation of the shader should not be done in the constructor. The updateEffect method can be used to compile the shader at a later time. (default: false)
         */
        constructor(name: string, originalFromInput: PostProcess, blurred: PostProcess, 
        /** Weight of the bloom to be added to the original input. */
        weight: number, options: number | PostProcessOptions, camera: Nullable<Camera>, samplingMode?: number, engine?: Engine, reusable?: boolean, textureType?: number, blockCompilation?: boolean);
    }


    /**
     * The Blur Post Process which blurs an image based on a kernel and direction.
     * Can be used twice in x and y directions to perform a gaussian blur in two passes.
     */
    export class BlurPostProcess extends PostProcess {
        private _blockCompilation;
        protected _kernel: number;
        protected _idealKernel: number;
        protected _packedFloat: boolean;
        private _staticDefines;
        /** The direction in which to blur the image. */
        direction: Vector2;
        /**
         * Sets the length in pixels of the blur sample region
         */
        set kernel(v: number);
        /**
         * Gets the length in pixels of the blur sample region
         */
        get kernel(): number;
        /**
         * Sets whether or not the blur needs to unpack/repack floats
         */
        set packedFloat(v: boolean);
        /**
         * Gets whether or not the blur is unpacking/repacking floats
         */
        get packedFloat(): boolean;
        /**
         * Gets a string identifying the name of the class
         * @returns "BlurPostProcess" string
         */
        getClassName(): string;
        /**
         * Creates a new instance BlurPostProcess
         * @param name The name of the effect.
         * @param direction The direction in which to blur the image.
         * @param kernel The size of the kernel to be used when computing the blur. eg. Size of 3 will blur the center pixel by 2 pixels surrounding it.
         * @param options The required width/height ratio to downsize to before computing the render pass. (Use 1.0 for full size)
         * @param camera The camera to apply the render pass to.
         * @param samplingMode The sampling mode to be used when computing the pass. (default: 0)
         * @param engine The engine which the post process will be applied. (default: current engine)
         * @param reusable If the post process can be reused on the same frame. (default: false)
         * @param textureType Type of textures used when performing the post process. (default: 0)
         * @param defines
         * @param _blockCompilation If compilation of the shader should not be done in the constructor. The updateEffect method can be used to compile the shader at a later time. (default: false)
         * @param textureFormat Format of textures used when performing the post process. (default: TEXTUREFORMAT_RGBA)
         */
        constructor(name: string, direction: Vector2, kernel: number, options: number | PostProcessOptions, camera: Nullable<Camera>, samplingMode?: number, engine?: Engine, reusable?: boolean, textureType?: number, defines?: string, _blockCompilation?: boolean, textureFormat?: number);
        /**
         * Updates the effect with the current post process compile time values and recompiles the shader.
         * @param defines Define statements that should be added at the beginning of the shader. (default: null)
         * @param uniforms Set of uniform variables that will be passed to the shader. (default: null)
         * @param samplers Set of Texture2D variables that will be passed to the shader. (default: null)
         * @param indexParameters The index parameters to be used for babylons include syntax "#include<kernelBlurVaryingDeclaration>[0..varyingCount]". (default: undefined) See usage in babylon.blurPostProcess.ts and kernelBlur.vertex.fx
         * @param onCompiled Called when the shader has been compiled.
         * @param onError Called if there is an error when compiling a shader.
         */
        updateEffect(defines?: Nullable<string>, uniforms?: Nullable<string[]>, samplers?: Nullable<string[]>, indexParameters?: any, onCompiled?: (effect: Effect) => void, onError?: (effect: Effect, errors: string) => void): void;
        protected _updateParameters(onCompiled?: (effect: Effect) => void, onError?: (effect: Effect, errors: string) => void): void;
        /**
         * Best kernels are odd numbers that when divided by 2, their integer part is even, so 5, 9 or 13.
         * Other odd kernels optimize correctly but require proportionally more samples, even kernels are
         * possible but will produce minor visual artifacts. Since each new kernel requires a new shader we
         * want to minimize kernel changes, having gaps between physical kernels is helpful in that regard.
         * The gaps between physical kernels are compensated for in the weighting of the samples
         * @param idealKernel Ideal blur kernel.
         * @returns Nearest best kernel.
         */
        protected _nearestBestKernel(idealKernel: number): number;
        /**
         * Calculates the value of a Gaussian distribution with sigma 3 at a given point.
         * @param x The point on the Gaussian distribution to sample.
         * @returns the value of the Gaussian function at x.
         */
        protected _gaussianWeight(x: number): number;
        /**
         * Generates a string that can be used as a floating point number in GLSL.
         * @param x Value to print.
         * @param decimalFigures Number of decimal places to print the number to (excluding trailing 0s).
         * @returns GLSL float string.
         */
        protected _glslFloat(x: number, decimalFigures?: number): string;
        /**
         * @internal
         */
        static _Parse(parsedPostProcess: any, targetCamera: Camera, scene: Scene, rootUrl: string): Nullable<BlurPostProcess>;
    }


    /**
     * The ChromaticAberrationPostProcess separates the rgb channels in an image to produce chromatic distortion around the edges of the screen
     */
    export class ChromaticAberrationPostProcess extends PostProcess {
        /**
         * The amount of separation of rgb channels (default: 30)
         */
        aberrationAmount: number;
        /**
         * The amount the effect will increase for pixels closer to the edge of the screen. (default: 0)
         */
        radialIntensity: number;
        /**
         * The normalized direction in which the rgb channels should be separated. If set to 0,0 radial direction will be used. (default: Vector2(0.707,0.707))
         */
        direction: Vector2;
        /**
         * The center position where the radialIntensity should be around. [0.5,0.5 is center of screen, 1,1 is top right corner] (default: Vector2(0.5 ,0.5))
         */
        centerPosition: Vector2;
        /** The width of the screen to apply the effect on */
        screenWidth: number;
        /** The height of the screen to apply the effect on */
        screenHeight: number;
        /**
         * Gets a string identifying the name of the class
         * @returns "ChromaticAberrationPostProcess" string
         */
        getClassName(): string;
        /**
         * Creates a new instance ChromaticAberrationPostProcess
         * @param name The name of the effect.
         * @param screenWidth The width of the screen to apply the effect on.
         * @param screenHeight The height of the screen to apply the effect on.
         * @param options The required width/height ratio to downsize to before computing the render pass.
         * @param camera The camera to apply the render pass to.
         * @param samplingMode The sampling mode to be used when computing the pass. (default: 0)
         * @param engine The engine which the post process will be applied. (default: current engine)
         * @param reusable If the post process can be reused on the same frame. (default: false)
         * @param textureType Type of textures used when performing the post process. (default: 0)
         * @param blockCompilation If compilation of the shader should not be done in the constructor. The updateEffect method can be used to compile the shader at a later time. (default: false)
         */
        constructor(name: string, screenWidth: number, screenHeight: number, options: number | PostProcessOptions, camera: Nullable<Camera>, samplingMode?: number, engine?: Engine, reusable?: boolean, textureType?: number, blockCompilation?: boolean);
        /**
         * @internal
         */
        static _Parse(parsedPostProcess: any, targetCamera: Camera, scene: Scene, rootUrl: string): Nullable<ChromaticAberrationPostProcess>;
    }


    /**
     * The CircleOfConfusionPostProcess computes the circle of confusion value for each pixel given required lens parameters. See https://en.wikipedia.org/wiki/Circle_of_confusion
     */
    export class CircleOfConfusionPostProcess extends PostProcess {
        /**
         * Max lens size in scene units/1000 (eg. millimeter). Standard cameras are 50mm. (default: 50) The diameter of the resulting aperture can be computed by lensSize/fStop.
         */
        lensSize: number;
        /**
         * F-Stop of the effect's camera. The diameter of the resulting aperture can be computed by lensSize/fStop. (default: 1.4)
         */
        fStop: number;
        /**
         * Distance away from the camera to focus on in scene units/1000 (eg. millimeter). (default: 2000)
         */
        focusDistance: number;
        /**
         * Focal length of the effect's camera in scene units/1000 (eg. millimeter). (default: 50)
         */
        focalLength: number;
        /**
         * Gets a string identifying the name of the class
         * @returns "CircleOfConfusionPostProcess" string
         */
        getClassName(): string;
        private _depthTexture;
        /**
         * Creates a new instance CircleOfConfusionPostProcess
         * @param name The name of the effect.
         * @param depthTexture The depth texture of the scene to compute the circle of confusion. This must be set in order for this to function but may be set after initialization if needed.
         * @param options The required width/height ratio to downsize to before computing the render pass.
         * @param camera The camera to apply the render pass to.
         * @param samplingMode The sampling mode to be used when computing the pass. (default: 0)
         * @param engine The engine which the post process will be applied. (default: current engine)
         * @param reusable If the post process can be reused on the same frame. (default: false)
         * @param textureType Type of textures used when performing the post process. (default: 0)
         * @param blockCompilation If compilation of the shader should not be done in the constructor. The updateEffect method can be used to compile the shader at a later time. (default: false)
         */
        constructor(name: string, depthTexture: Nullable<RenderTargetTexture>, options: number | PostProcessOptions, camera: Nullable<Camera>, samplingMode?: number, engine?: Engine, reusable?: boolean, textureType?: number, blockCompilation?: boolean);
        /**
         * Depth texture to be used to compute the circle of confusion. This must be set here or in the constructor in order for the post process to function.
         */
        set depthTexture(value: RenderTargetTexture);
    }


    /**
     *
     * This post-process allows the modification of rendered colors by using
     * a 'look-up table' (LUT). This effect is also called Color Grading.
     *
     * The object needs to be provided an url to a texture containing the color
     * look-up table: the texture must be 256 pixels wide and 16 pixels high.
     * Use an image editing software to tweak the LUT to match your needs.
     *
     * For an example of a color LUT, see here:
     * @see http://udn.epicgames.com/Three/rsrc/Three/ColorGrading/RGBTable16x1.png
     * For explanations on color grading, see here:
     * @see http://udn.epicgames.com/Three/ColorGrading.html
     *
     */
    export class ColorCorrectionPostProcess extends PostProcess {
        private _colorTableTexture;
        /**
         * Gets the color table url used to create the LUT texture
         */
        colorTableUrl: string;
        /**
         * Gets a string identifying the name of the class
         * @returns "ColorCorrectionPostProcess" string
         */
        getClassName(): string;
        constructor(name: string, colorTableUrl: string, options: number | PostProcessOptions, camera: Nullable<Camera>, samplingMode?: number, engine?: Engine, reusable?: boolean);
        /**
         * @internal
         */
        static _Parse(parsedPostProcess: any, targetCamera: Camera, scene: Scene, rootUrl: string): Nullable<ColorCorrectionPostProcess>;
    }


    /**
     * The ConvolutionPostProcess applies a 3x3 kernel to every pixel of the
     * input texture to perform effects such as edge detection or sharpening
     * See http://en.wikipedia.org/wiki/Kernel_(image_processing)
     */
    export class ConvolutionPostProcess extends PostProcess {
        /** Array of 9 values corresponding to the 3x3 kernel to be applied */
        kernel: number[];
        /**
         * Gets a string identifying the name of the class
         * @returns "ConvolutionPostProcess" string
         */
        getClassName(): string;
        /**
         * Creates a new instance ConvolutionPostProcess
         * @param name The name of the effect.
         * @param kernel Array of 9 values corresponding to the 3x3 kernel to be applied
         * @param options The required width/height ratio to downsize to before computing the render pass.
         * @param camera The camera to apply the render pass to.
         * @param samplingMode The sampling mode to be used when computing the pass. (default: 0)
         * @param engine The engine which the post process will be applied. (default: current engine)
         * @param reusable If the post process can be reused on the same frame. (default: false)
         * @param textureType Type of textures used when performing the post process. (default: 0)
         */
        constructor(name: string, kernel: number[], options: number | PostProcessOptions, camera: Nullable<Camera>, samplingMode?: number, engine?: Engine, reusable?: boolean, textureType?: number);
        /**
         * @internal
         */
        static _Parse(parsedPostProcess: any, targetCamera: Camera, scene: Scene, rootUrl: string): Nullable<ConvolutionPostProcess>;
        /**
         * Edge detection 0 see https://en.wikipedia.org/wiki/Kernel_(image_processing)
         */
        static EdgeDetect0Kernel: number[];
        /**
         * Edge detection 1 see https://en.wikipedia.org/wiki/Kernel_(image_processing)
         */
        static EdgeDetect1Kernel: number[];
        /**
         * Edge detection 2 see https://en.wikipedia.org/wiki/Kernel_(image_processing)
         */
        static EdgeDetect2Kernel: number[];
        /**
         * Kernel to sharpen an image see https://en.wikipedia.org/wiki/Kernel_(image_processing)
         */
        static SharpenKernel: number[];
        /**
         * Kernel to emboss an image see https://en.wikipedia.org/wiki/Kernel_(image_processing)
         */
        static EmbossKernel: number[];
        /**
         * Kernel to blur an image see https://en.wikipedia.org/wiki/Kernel_(image_processing)
         */
        static GaussianKernel: number[];
    }


    /**
     * The DepthOfFieldBlurPostProcess applied a blur in a give direction.
     * This blur differs from the standard BlurPostProcess as it attempts to avoid blurring pixels
     * based on samples that have a large difference in distance than the center pixel.
     * See section 2.6.2 http://fileadmin.cs.lth.se/cs/education/edan35/lectures/12dof.pdf
     */
    export class DepthOfFieldBlurPostProcess extends BlurPostProcess {
        /**
         * The direction the blur should be applied
         */
        direction: Vector2;
        /**
         * Gets a string identifying the name of the class
         * @returns "DepthOfFieldBlurPostProcess" string
         */
        getClassName(): string;
        /**
         * Creates a new instance DepthOfFieldBlurPostProcess
         * @param name The name of the effect.
         * @param scene The scene the effect belongs to.
         * @param direction The direction the blur should be applied.
         * @param kernel The size of the kernel used to blur.
         * @param options The required width/height ratio to downsize to before computing the render pass.
         * @param camera The camera to apply the render pass to.
         * @param circleOfConfusion The circle of confusion + depth map to be used to avoid blurring across edges
         * @param imageToBlur The image to apply the blur to (default: Current rendered frame)
         * @param samplingMode The sampling mode to be used when computing the pass. (default: 0)
         * @param engine The engine which the post process will be applied. (default: current engine)
         * @param reusable If the post process can be reused on the same frame. (default: false)
         * @param textureType Type of textures used when performing the post process. (default: 0)
         * @param blockCompilation If compilation of the shader should not be done in the constructor. The updateEffect method can be used to compile the shader at a later time. (default: false)
         * @param textureFormat Format of textures used when performing the post process. (default: TEXTUREFORMAT_RGBA)
         */
        constructor(name: string, scene: Scene, direction: Vector2, kernel: number, options: number | PostProcessOptions, camera: Nullable<Camera>, circleOfConfusion: PostProcess, imageToBlur?: Nullable<PostProcess>, samplingMode?: number, engine?: Engine, reusable?: boolean, textureType?: number, blockCompilation?: boolean, textureFormat?: number);
    }


    /**
     * Specifies the level of max blur that should be applied when using the depth of field effect
     */
    export enum DepthOfFieldEffectBlurLevel {
        /**
         * Subtle blur
         */
        Low = 0,
        /**
         * Medium blur
         */
        Medium = 1,
        /**
         * Large blur
         */
        High = 2
    }
    /**
     * The depth of field effect applies a blur to objects that are closer or further from where the camera is focusing.
     */
    export class DepthOfFieldEffect extends PostProcessRenderEffect {
        private _circleOfConfusion;
        /**
         * @internal Internal, blurs from high to low
         */
        _depthOfFieldBlurX: Array<DepthOfFieldBlurPostProcess>;
        private _depthOfFieldBlurY;
        private _dofMerge;
        /**
         * @internal Internal post processes in depth of field effect
         */
        _effects: Array<PostProcess>;
        /**
         * The focal the length of the camera used in the effect in scene units/1000 (eg. millimeter)
         */
        set focalLength(value: number);
        get focalLength(): number;
        /**
         * F-Stop of the effect's camera. The diameter of the resulting aperture can be computed by lensSize/fStop. (default: 1.4)
         */
        set fStop(value: number);
        get fStop(): number;
        /**
         * Distance away from the camera to focus on in scene units/1000 (eg. millimeter). (default: 2000)
         */
        set focusDistance(value: number);
        get focusDistance(): number;
        /**
         * Max lens size in scene units/1000 (eg. millimeter). Standard cameras are 50mm. (default: 50) The diameter of the resulting aperture can be computed by lensSize/fStop.
         */
        set lensSize(value: number);
        get lensSize(): number;
        /**
         * Creates a new instance DepthOfFieldEffect
         * @param scene The scene the effect belongs to.
         * @param depthTexture The depth texture of the scene to compute the circle of confusion.This must be set in order for this to function but may be set after initialization if needed.
         * @param blurLevel
         * @param pipelineTextureType The type of texture to be used when performing the post processing.
         * @param blockCompilation If compilation of the shader should not be done in the constructor. The updateEffect method can be used to compile the shader at a later time. (default: false)
         */
        constructor(scene: Scene, depthTexture: Nullable<RenderTargetTexture>, blurLevel?: DepthOfFieldEffectBlurLevel, pipelineTextureType?: number, blockCompilation?: boolean);
        /**
         * Get the current class name of the current effect
         * @returns "DepthOfFieldEffect"
         */
        getClassName(): string;
        /**
         * Depth texture to be used to compute the circle of confusion. This must be set here or in the constructor in order for the post process to function.
         */
        set depthTexture(value: RenderTargetTexture);
        /**
         * Disposes each of the internal effects for a given camera.
         * @param camera The camera to dispose the effect on.
         */
        disposeEffects(camera: Camera): void;
        /**
         * @internal Internal
         */
        _updateEffects(): void;
        /**
         * Internal
         * @returns if all the contained post processes are ready.
         * @internal
         */
        _isReady(): boolean;
    }


    /**
     * Options to be set when merging outputs from the default pipeline.
     */
    export class DepthOfFieldMergePostProcessOptions {
        /**
         * The original image to merge on top of
         */
        originalFromInput: PostProcess;
        /**
         * Parameters to perform the merge of the depth of field effect
         */
        depthOfField?: {
            circleOfConfusion: PostProcess;
            blurSteps: Array<PostProcess>;
        };
        /**
         * Parameters to perform the merge of bloom effect
         */
        bloom?: {
            blurred: PostProcess;
            weight: number;
        };
    }
    /**
     * The DepthOfFieldMergePostProcess merges blurred images with the original based on the values of the circle of confusion.
     */
    export class DepthOfFieldMergePostProcess extends PostProcess {
        private _blurSteps;
        /**
         * Gets a string identifying the name of the class
         * @returns "DepthOfFieldMergePostProcess" string
         */
        getClassName(): string;
        /**
         * Creates a new instance of DepthOfFieldMergePostProcess
         * @param name The name of the effect.
         * @param originalFromInput Post process which's input will be used for the merge.
         * @param circleOfConfusion Circle of confusion post process which's output will be used to blur each pixel.
         * @param _blurSteps Blur post processes from low to high which will be mixed with the original image.
         * @param options The required width/height ratio to downsize to before computing the render pass.
         * @param camera The camera to apply the render pass to.
         * @param samplingMode The sampling mode to be used when computing the pass. (default: 0)
         * @param engine The engine which the post process will be applied. (default: current engine)
         * @param reusable If the post process can be reused on the same frame. (default: false)
         * @param textureType Type of textures used when performing the post process. (default: 0)
         * @param blockCompilation If compilation of the shader should not be done in the constructor. The updateEffect method can be used to compile the shader at a later time. (default: false)
         */
        constructor(name: string, originalFromInput: PostProcess, circleOfConfusion: PostProcess, _blurSteps: Array<PostProcess>, options: number | PostProcessOptions, camera: Nullable<Camera>, samplingMode?: number, engine?: Engine, reusable?: boolean, textureType?: number, blockCompilation?: boolean);
        /**
         * Updates the effect with the current post process compile time values and recompiles the shader.
         * @param defines Define statements that should be added at the beginning of the shader. (default: null)
         * @param uniforms Set of uniform variables that will be passed to the shader. (default: null)
         * @param samplers Set of Texture2D variables that will be passed to the shader. (default: null)
         * @param indexParameters The index parameters to be used for babylons include syntax "#include<kernelBlurVaryingDeclaration>[0..varyingCount]". (default: undefined) See usage in babylon.blurPostProcess.ts and kernelBlur.vertex.fx
         * @param onCompiled Called when the shader has been compiled.
         * @param onError Called if there is an error when compiling a shader.
         */
        updateEffect(defines?: Nullable<string>, uniforms?: Nullable<string[]>, samplers?: Nullable<string[]>, indexParameters?: any, onCompiled?: (effect: Effect) => void, onError?: (effect: Effect, errors: string) => void): void;
    }


    /**
     * DisplayPassPostProcess which produces an output the same as it's input
     */
    export class DisplayPassPostProcess extends PostProcess {
        /**
         * Gets a string identifying the name of the class
         * @returns "DisplayPassPostProcess" string
         */
        getClassName(): string;
        /**
         * Creates the DisplayPassPostProcess
         * @param name The name of the effect.
         * @param options The required width/height ratio to downsize to before computing the render pass.
         * @param camera The camera to apply the render pass to.
         * @param samplingMode The sampling mode to be used when computing the pass. (default: 0)
         * @param engine The engine which the post process will be applied. (default: current engine)
         * @param reusable If the post process can be reused on the same frame. (default: false)
         */
        constructor(name: string, options: number | PostProcessOptions, camera: Nullable<Camera>, samplingMode?: number, engine?: Engine, reusable?: boolean);
        /**
         * @internal
         */
        static _Parse(parsedPostProcess: any, targetCamera: Camera, scene: Scene, rootUrl: string): Nullable<DisplayPassPostProcess>;
    }


    /**
     * The extract highlights post process sets all pixels to black except pixels above the specified luminance threshold. Used as the first step for a bloom effect.
     */
    export class ExtractHighlightsPostProcess extends PostProcess {
        /**
         * The luminance threshold, pixels below this value will be set to black.
         */
        threshold: number;
        /** @internal */
        _exposure: number;
        /**
         * Post process which has the input texture to be used when performing highlight extraction
         * @internal
         */
        _inputPostProcess: Nullable<PostProcess>;
        /**
         * Gets a string identifying the name of the class
         * @returns "ExtractHighlightsPostProcess" string
         */
        getClassName(): string;
        constructor(name: string, options: number | PostProcessOptions, camera: Nullable<Camera>, samplingMode?: number, engine?: Engine, reusable?: boolean, textureType?: number, blockCompilation?: boolean);
    }


    /**
     * Applies a kernel filter to the image
     */
    export class FilterPostProcess extends PostProcess {
        /** The matrix to be applied to the image */
        kernelMatrix: Matrix;
        /**
         * Gets a string identifying the name of the class
         * @returns "FilterPostProcess" string
         */
        getClassName(): string;
        /**
         *
         * @param name The name of the effect.
         * @param kernelMatrix The matrix to be applied to the image
         * @param options The required width/height ratio to downsize to before computing the render pass.
         * @param camera The camera to apply the render pass to.
         * @param samplingMode The sampling mode to be used when computing the pass. (default: 0)
         * @param engine The engine which the post process will be applied. (default: current engine)
         * @param reusable If the post process can be reused on the same frame. (default: false)
         */
        constructor(name: string, kernelMatrix: Matrix, options: number | PostProcessOptions, camera: Nullable<Camera>, samplingMode?: number, engine?: Engine, reusable?: boolean);
        /**
         * @internal
         */
        static _Parse(parsedPostProcess: any, targetCamera: Camera, scene: Scene, rootUrl: string): Nullable<FilterPostProcess>;
    }


    /**
     * Fxaa post process
     * @see https://doc.babylonjs.com/features/featuresDeepDive/postProcesses/usePostProcesses#fxaa
     */
    export class FxaaPostProcess extends PostProcess {
        /**
         * Gets a string identifying the name of the class
         * @returns "FxaaPostProcess" string
         */
        getClassName(): string;
        constructor(name: string, options: number | PostProcessOptions, camera?: Nullable<Camera>, samplingMode?: number, engine?: Engine, reusable?: boolean, textureType?: number);
        private _getDefines;
        /**
         * @internal
         */
        static _Parse(parsedPostProcess: any, targetCamera: Camera, scene: Scene, rootUrl: string): FxaaPostProcess;
    }


    /**
     * The GrainPostProcess adds noise to the image at mid luminance levels
     */
    export class GrainPostProcess extends PostProcess {
        /**
         * The intensity of the grain added (default: 30)
         */
        intensity: number;
        /**
         * If the grain should be randomized on every frame
         */
        animated: boolean;
        /**
         * Gets a string identifying the name of the class
         * @returns "GrainPostProcess" string
         */
        getClassName(): string;
        /**
         * Creates a new instance of @see GrainPostProcess
         * @param name The name of the effect.
         * @param options The required width/height ratio to downsize to before computing the render pass.
         * @param camera The camera to apply the render pass to.
         * @param samplingMode The sampling mode to be used when computing the pass. (default: 0)
         * @param engine The engine which the post process will be applied. (default: current engine)
         * @param reusable If the post process can be reused on the same frame. (default: false)
         * @param textureType Type of textures used when performing the post process. (default: 0)
         * @param blockCompilation If compilation of the shader should not be done in the constructor. The updateEffect method can be used to compile the shader at a later time. (default: false)
         */
        constructor(name: string, options: number | PostProcessOptions, camera: Nullable<Camera>, samplingMode?: number, engine?: Engine, reusable?: boolean, textureType?: number, blockCompilation?: boolean);
        /**
         * @internal
         */
        static _Parse(parsedPostProcess: any, targetCamera: Camera, scene: Scene, rootUrl: string): GrainPostProcess;
    }


    /**
     * Extracts highlights from the image
     * @see https://doc.babylonjs.com/features/featuresDeepDive/postProcesses/usePostProcesses
     */
    export class HighlightsPostProcess extends PostProcess {
        /**
         * Gets a string identifying the name of the class
         * @returns "HighlightsPostProcess" string
         */
        getClassName(): string;
        /**
         * Extracts highlights from the image
         * @see https://doc.babylonjs.com/features/featuresDeepDive/postProcesses/usePostProcesses
         * @param name The name of the effect.
         * @param options The required width/height ratio to downsize to before computing the render pass.
         * @param camera The camera to apply the render pass to.
         * @param samplingMode The sampling mode to be used when computing the pass. (default: 0)
         * @param engine The engine which the post process will be applied. (default: current engine)
         * @param reusable If the post process can be reused on the same frame. (default: false)
         * @param textureType Type of texture for the post process (default: Engine.TEXTURETYPE_UNSIGNED_INT)
         */
        constructor(name: string, options: number | PostProcessOptions, camera: Nullable<Camera>, samplingMode?: number, engine?: Engine, reusable?: boolean, textureType?: number);
    }


    /**
     * ImageProcessingPostProcess
     * @see https://doc.babylonjs.com/features/featuresDeepDive/postProcesses/usePostProcesses#imageprocessing
     */
    export class ImageProcessingPostProcess extends PostProcess {
        /**
         * Default configuration related to image processing available in the PBR Material.
         */
        protected _imageProcessingConfiguration: ImageProcessingConfiguration;
        /**
         * Gets the image processing configuration used either in this material.
         */
        get imageProcessingConfiguration(): ImageProcessingConfiguration;
        /**
         * Sets the Default image processing configuration used either in the this material.
         *
         * If sets to null, the scene one is in use.
         */
        set imageProcessingConfiguration(value: ImageProcessingConfiguration);
        /**
         * Keep track of the image processing observer to allow dispose and replace.
         */
        private _imageProcessingObserver;
        /**
         * Attaches a new image processing configuration to the PBR Material.
         * @param configuration
         * @param doNotBuild
         */
        protected _attachImageProcessingConfiguration(configuration: Nullable<ImageProcessingConfiguration>, doNotBuild?: boolean): void;
        /**
         * If the post process is supported.
         */
        get isSupported(): boolean;
        /**
         * Gets Color curves setup used in the effect if colorCurvesEnabled is set to true .
         */
        get colorCurves(): Nullable<ColorCurves>;
        /**
         * Sets Color curves setup used in the effect if colorCurvesEnabled is set to true .
         */
        set colorCurves(value: Nullable<ColorCurves>);
        /**
         * Gets whether the color curves effect is enabled.
         */
        get colorCurvesEnabled(): boolean;
        /**
         * Sets whether the color curves effect is enabled.
         */
        set colorCurvesEnabled(value: boolean);
        /**
         * Gets Color grading LUT texture used in the effect if colorGradingEnabled is set to true.
         */
        get colorGradingTexture(): Nullable<BaseTexture>;
        /**
         * Sets Color grading LUT texture used in the effect if colorGradingEnabled is set to true.
         */
        set colorGradingTexture(value: Nullable<BaseTexture>);
        /**
         * Gets whether the color grading effect is enabled.
         */
        get colorGradingEnabled(): boolean;
        /**
         * Gets whether the color grading effect is enabled.
         */
        set colorGradingEnabled(value: boolean);
        /**
         * Gets exposure used in the effect.
         */
        get exposure(): number;
        /**
         * Sets exposure used in the effect.
         */
        set exposure(value: number);
        /**
         * Gets whether tonemapping is enabled or not.
         */
        get toneMappingEnabled(): boolean;
        /**
         * Sets whether tonemapping is enabled or not
         */
        set toneMappingEnabled(value: boolean);
        /**
         * Gets the type of tone mapping effect.
         */
        get toneMappingType(): number;
        /**
         * Sets the type of tone mapping effect.
         */
        set toneMappingType(value: number);
        /**
         * Gets contrast used in the effect.
         */
        get contrast(): number;
        /**
         * Sets contrast used in the effect.
         */
        set contrast(value: number);
        /**
         * Gets Vignette stretch size.
         */
        get vignetteStretch(): number;
        /**
         * Sets Vignette stretch size.
         */
        set vignetteStretch(value: number);
        /**
         * Gets Vignette center X Offset.
         * @deprecated use vignetteCenterX instead
         */
        get vignetteCentreX(): number;
        /**
         * Sets Vignette center X Offset.
         * @deprecated use vignetteCenterX instead
         */
        set vignetteCentreX(value: number);
        /**
         * Gets Vignette center Y Offset.
         * @deprecated use vignetteCenterY instead
         */
        get vignetteCentreY(): number;
        /**
         * Sets Vignette center Y Offset.
         * @deprecated use vignetteCenterY instead
         */
        set vignetteCentreY(value: number);
        /**
         * Vignette center Y Offset.
         */
        get vignetteCenterY(): number;
        set vignetteCenterY(value: number);
        /**
         * Vignette center X Offset.
         */
        get vignetteCenterX(): number;
        set vignetteCenterX(value: number);
        /**
         * Gets Vignette weight or intensity of the vignette effect.
         */
        get vignetteWeight(): number;
        /**
         * Sets Vignette weight or intensity of the vignette effect.
         */
        set vignetteWeight(value: number);
        /**
         * Gets Color of the vignette applied on the screen through the chosen blend mode (vignetteBlendMode)
         * if vignetteEnabled is set to true.
         */
        get vignetteColor(): Color4;
        /**
         * Sets Color of the vignette applied on the screen through the chosen blend mode (vignetteBlendMode)
         * if vignetteEnabled is set to true.
         */
        set vignetteColor(value: Color4);
        /**
         * Gets Camera field of view used by the Vignette effect.
         */
        get vignetteCameraFov(): number;
        /**
         * Sets Camera field of view used by the Vignette effect.
         */
        set vignetteCameraFov(value: number);
        /**
         * Gets the vignette blend mode allowing different kind of effect.
         */
        get vignetteBlendMode(): number;
        /**
         * Sets the vignette blend mode allowing different kind of effect.
         */
        set vignetteBlendMode(value: number);
        /**
         * Gets whether the vignette effect is enabled.
         */
        get vignetteEnabled(): boolean;
        /**
         * Sets whether the vignette effect is enabled.
         */
        set vignetteEnabled(value: boolean);
        /**
         * Gets intensity of the dithering effect.
         */
        get ditheringIntensity(): number;
        /**
         * Sets intensity of the dithering effect.
         */
        set ditheringIntensity(value: number);
        /**
         * Gets whether the dithering effect is enabled.
         */
        get ditheringEnabled(): boolean;
        /**
         * Sets whether the dithering effect is enabled.
         */
        set ditheringEnabled(value: boolean);
        private _fromLinearSpace;
        /**
         * Gets whether the input of the processing is in Gamma or Linear Space.
         */
        get fromLinearSpace(): boolean;
        /**
         * Sets whether the input of the processing is in Gamma or Linear Space.
         */
        set fromLinearSpace(value: boolean);
        /**
         * Defines cache preventing GC.
         */
        private _defines;
        constructor(name: string, options: number | PostProcessOptions, camera?: Nullable<Camera>, samplingMode?: number, engine?: Engine, reusable?: boolean, textureType?: number, imageProcessingConfiguration?: ImageProcessingConfiguration);
        /**
         *  "ImageProcessingPostProcess"
         * @returns "ImageProcessingPostProcess"
         */
        getClassName(): string;
        /**
         * @internal
         */
        _updateParameters(): void;
        dispose(camera?: Camera): void;
    }




    /**
     * The Motion Blur Post Process which blurs an image based on the objects velocity in scene.
     * Velocity can be affected by each object's rotation, position and scale depending on the transformation speed.
     * As an example, all you have to do is to create the post-process:
     *  var mb = new BABYLON.MotionBlurPostProcess(
     *      'mb', // The name of the effect.
     *      scene, // The scene containing the objects to blur according to their velocity.
     *      1.0, // The required width/height ratio to downsize to before computing the render pass.
     *      camera // The camera to apply the render pass to.
     * );
     * Then, all objects moving, rotating and/or scaling will be blurred depending on the transformation speed.
     */
    export class MotionBlurPostProcess extends PostProcess {
        /**
         * Defines how much the image is blurred by the movement. Default value is equal to 1
         */
        motionStrength: number;
        /**
         * Gets the number of iterations are used for motion blur quality. Default value is equal to 32
         */
        get motionBlurSamples(): number;
        /**
         * Sets the number of iterations to be used for motion blur quality
         */
        set motionBlurSamples(samples: number);
        private _motionBlurSamples;
        /**
         * Gets whether or not the motion blur post-process is in object based mode.
         */
        get isObjectBased(): boolean;
        /**
         * Sets whether or not the motion blur post-process is in object based mode.
         */
        set isObjectBased(value: boolean);
        private _isObjectBased;
        private _forceGeometryBuffer;
        private get _geometryBufferRenderer();
        private get _prePassRenderer();
        private _invViewProjection;
        private _previousViewProjection;
        /**
         * Gets a string identifying the name of the class
         * @returns "MotionBlurPostProcess" string
         */
        getClassName(): string;
        /**
         * Creates a new instance MotionBlurPostProcess
         * @param name The name of the effect.
         * @param scene The scene containing the objects to blur according to their velocity.
         * @param options The required width/height ratio to downsize to before computing the render pass.
         * @param camera The camera to apply the render pass to.
         * @param samplingMode The sampling mode to be used when computing the pass. (default: 0)
         * @param engine The engine which the post process will be applied. (default: current engine)
         * @param reusable If the post process can be reused on the same frame. (default: false)
         * @param textureType Type of textures used when performing the post process. (default: 0)
         * @param blockCompilation If compilation of the shader should not be done in the constructor. The updateEffect method can be used to compile the shader at a later time. (default: true)
         * @param forceGeometryBuffer If this post process should use geometry buffer instead of prepass (default: false)
         */
        constructor(name: string, scene: Scene, options: number | PostProcessOptions, camera: Nullable<Camera>, samplingMode?: number, engine?: Engine, reusable?: boolean, textureType?: number, blockCompilation?: boolean, forceGeometryBuffer?: boolean);
        /**
         * Excludes the given skinned mesh from computing bones velocities.
         * Computing bones velocities can have a cost and that cost. The cost can be saved by calling this function and by passing the skinned mesh reference to ignore.
         * @param skinnedMesh The mesh containing the skeleton to ignore when computing the velocity map.
         */
        excludeSkinnedMesh(skinnedMesh: AbstractMesh): void;
        /**
         * Removes the given skinned mesh from the excluded meshes to integrate bones velocities while rendering the velocity map.
         * @param skinnedMesh The mesh containing the skeleton that has been ignored previously.
         * @see excludeSkinnedMesh to exclude a skinned mesh from bones velocity computation.
         */
        removeExcludedSkinnedMesh(skinnedMesh: AbstractMesh): void;
        /**
         * Disposes the post process.
         * @param camera The camera to dispose the post process on.
         */
        dispose(camera?: Camera): void;
        /**
         * Called on the mode changed (object based or screen based).
         */
        private _applyMode;
        /**
         * Called on the effect is applied when the motion blur post-process is in object based mode.
         * @param effect
         */
        private _onApplyObjectBased;
        /**
         * Called on the effect is applied when the motion blur post-process is in screen based mode.
         * @param effect
         */
        private _onApplyScreenBased;
        /**
         * Called on the effect must be updated (changed mode, samples count, etc.).
         */
        private _updateEffect;
        /**
         * @internal
         */
        static _Parse(parsedPostProcess: any, targetCamera: Camera, scene: Scene, rootUrl: string): Nullable<MotionBlurPostProcess>;
    }


    /**
     * PassPostProcess which produces an output the same as it's input
     */
    export class PassPostProcess extends PostProcess {
        /**
         * Gets a string identifying the name of the class
         * @returns "PassPostProcess" string
         */
        getClassName(): string;
        /**
         * Creates the PassPostProcess
         * @param name The name of the effect.
         * @param options The required width/height ratio to downsize to before computing the render pass.
         * @param camera The camera to apply the render pass to.
         * @param samplingMode The sampling mode to be used when computing the pass. (default: 0)
         * @param engine The engine which the post process will be applied. (default: current engine)
         * @param reusable If the post process can be reused on the same frame. (default: false)
         * @param textureType The type of texture to be used when performing the post processing.
         * @param blockCompilation If compilation of the shader should not be done in the constructor. The updateEffect method can be used to compile the shader at a later time. (default: false)
         */
        constructor(name: string, options: number | PostProcessOptions, camera?: Nullable<Camera>, samplingMode?: number, engine?: Engine, reusable?: boolean, textureType?: number, blockCompilation?: boolean);
        /**
         * @internal
         */
        static _Parse(parsedPostProcess: any, targetCamera: Camera, scene: Scene, rootUrl: string): PassPostProcess;
    }
    /**
     * PassCubePostProcess which produces an output the same as it's input (which must be a cube texture)
     */
    export class PassCubePostProcess extends PostProcess {
        private _face;
        /**
         * Gets or sets the cube face to display.
         *  * 0 is +X
         *  * 1 is -X
         *  * 2 is +Y
         *  * 3 is -Y
         *  * 4 is +Z
         *  * 5 is -Z
         */
        get face(): number;
        set face(value: number);
        /**
         * Gets a string identifying the name of the class
         * @returns "PassCubePostProcess" string
         */
        getClassName(): string;
        /**
         * Creates the PassCubePostProcess
         * @param name The name of the effect.
         * @param options The required width/height ratio to downsize to before computing the render pass.
         * @param camera The camera to apply the render pass to.
         * @param samplingMode The sampling mode to be used when computing the pass. (default: 0)
         * @param engine The engine which the post process will be applied. (default: current engine)
         * @param reusable If the post process can be reused on the same frame. (default: false)
         * @param textureType The type of texture to be used when performing the post processing.
         * @param blockCompilation If compilation of the shader should not be done in the constructor. The updateEffect method can be used to compile the shader at a later time. (default: false)
         */
        constructor(name: string, options: number | PostProcessOptions, camera?: Nullable<Camera>, samplingMode?: number, engine?: Engine, reusable?: boolean, textureType?: number, blockCompilation?: boolean);
        /**
         * @internal
         */
        static _Parse(parsedPostProcess: any, targetCamera: Camera, scene: Scene, rootUrl: string): PassCubePostProcess;
    }


    /**
     * Allows for custom processing of the shader code used by a post process
     */
    export type PostProcessCustomShaderCodeProcessing = {
        /**
         * If provided, will be called two times with the vertex and fragment code so that this code can be updated after the #include have been processed
         */
        processCodeAfterIncludes?: (postProcessName: string, shaderType: string, code: string) => string;
        /**
         * If provided, will be called two times with the vertex and fragment code so that this code can be updated before it is compiled by the GPU
         */
        processFinalCode?: (postProcessName: string, shaderType: string, code: string) => string;
        /**
         * If provided, will be called before creating the effect to collect additional custom bindings (defines, uniforms, samplers)
         */
        defineCustomBindings?: (postProcessName: string, defines: Nullable<string>, uniforms: string[], samplers: string[]) => Nullable<string>;
        /**
         * If provided, will be called when binding inputs to the shader code to allow the user to add custom bindings
         */
        bindCustomBindings?: (postProcessName: string, effect: Effect) => void;
    };
    /**
     * Size options for a post process
     */
    export type PostProcessOptions = {
        width: number;
        height: number;
    };
    /**
     * PostProcess can be used to apply a shader to a texture after it has been rendered
     * See https://doc.babylonjs.com/features/featuresDeepDive/postProcesses/usePostProcesses
     */
    export class PostProcess {
        /** @internal */
        _parentContainer: Nullable<AbstractScene>;
        private static _CustomShaderCodeProcessing;
        /**
         * Registers a shader code processing with a post process name.
         * @param postProcessName name of the post process. Use null for the fallback shader code processing. This is the shader code processing that will be used in case no specific shader code processing has been associated to a post process name
         * @param customShaderCodeProcessing shader code processing to associate to the post process name
         * @returns
         */
        static RegisterShaderCodeProcessing(postProcessName: Nullable<string>, customShaderCodeProcessing?: PostProcessCustomShaderCodeProcessing): void;
        private static _GetShaderCodeProcessing;
        /**
         * Gets or sets the unique id of the post process
         */
        uniqueId: number;
        /** Name of the PostProcess. */
        name: string;
        /**
         * Width of the texture to apply the post process on
         */
        width: number;
        /**
         * Height of the texture to apply the post process on
         */
        height: number;
        /**
         * Gets the node material used to create this postprocess (null if the postprocess was manually created)
         */
        nodeMaterialSource: Nullable<NodeMaterial>;
        /**
         * Internal, reference to the location where this postprocess was output to. (Typically the texture on the next postprocess in the chain)
         * @internal
         */
        _outputTexture: Nullable<RenderTargetWrapper>;
        /**
         * Sampling mode used by the shader
         * See https://doc.babylonjs.com/classes/3.1/texture
         */
        renderTargetSamplingMode: number;
        /**
         * Clear color to use when screen clearing
         */
        clearColor: Color4;
        /**
         * If the buffer needs to be cleared before applying the post process. (default: true)
         * Should be set to false if shader will overwrite all previous pixels.
         */
        autoClear: boolean;
        /**
         * Type of alpha mode to use when performing the post process (default: Engine.ALPHA_DISABLE)
         */
        alphaMode: number;
        /**
         * Sets the setAlphaBlendConstants of the babylon engine
         */
        alphaConstants: Color4;
        /**
         * Animations to be used for the post processing
         */
        animations: Animation[];
        /**
         * Enable Pixel Perfect mode where texture is not scaled to be power of 2.
         * Can only be used on a single postprocess or on the last one of a chain. (default: false)
         */
        enablePixelPerfectMode: boolean;
        /**
         * Force the postprocess to be applied without taking in account viewport
         */
        forceFullscreenViewport: boolean;
        /**
         * List of inspectable custom properties (used by the Inspector)
         * @see https://doc.babylonjs.com/toolsAndResources/inspector#extensibility
         */
        inspectableCustomProperties: IInspectable[];
        /**
         * Scale mode for the post process (default: Engine.SCALEMODE_FLOOR)
         *
         * | Value | Type                                | Description |
         * | ----- | ----------------------------------- | ----------- |
         * | 1     | SCALEMODE_FLOOR                     | [engine.scalemode_floor](https://doc.babylonjs.com/api/classes/babylon.engine#scalemode_floor) |
         * | 2     | SCALEMODE_NEAREST                   | [engine.scalemode_nearest](https://doc.babylonjs.com/api/classes/babylon.engine#scalemode_nearest) |
         * | 3     | SCALEMODE_CEILING                   | [engine.scalemode_ceiling](https://doc.babylonjs.com/api/classes/babylon.engine#scalemode_ceiling) |
         *
         */
        scaleMode: number;
        /**
         * Force textures to be a power of two (default: false)
         */
        alwaysForcePOT: boolean;
        private _samples;
        /**
         * Number of sample textures (default: 1)
         */
        get samples(): number;
        set samples(n: number);
        /**
         * Modify the scale of the post process to be the same as the viewport (default: false)
         */
        adaptScaleToCurrentViewport: boolean;
        private _camera;
        protected _scene: Scene;
        private _engine;
        private _options;
        private _reusable;
        private _renderId;
        private _textureType;
        private _textureFormat;
        private _shaderLanguage;
        /**
         * if externalTextureSamplerBinding is true, the "apply" method won't bind the textureSampler texture, it is expected to be done by the "outside" (by the onApplyObservable observer most probably).
         * counter-productive in some cases because if the texture bound by "apply" is different from the currently texture bound, (the one set by the onApplyObservable observer, for eg) some
         * internal structures (materialContext) will be dirtified, which may impact performances
         */
        externalTextureSamplerBinding: boolean;
        /**
         * Smart array of input and output textures for the post process.
         * @internal
         */
        _textures: SmartArray<RenderTargetWrapper>;
        /**
         * Smart array of input and output textures for the post process.
         * @internal
         */
        private _textureCache;
        /**
         * The index in _textures that corresponds to the output texture.
         * @internal
         */
        _currentRenderTextureInd: number;
        private _drawWrapper;
        private _samplers;
        private _fragmentUrl;
        private _vertexUrl;
        private _parameters;
        protected _postProcessDefines: Nullable<string>;
        private _scaleRatio;
        protected _indexParameters: any;
        private _shareOutputWithPostProcess;
        private _texelSize;
        /** @internal */
        _forcedOutputTexture: Nullable<RenderTargetWrapper>;
        /**
         * Prepass configuration in case this post process needs a texture from prepass
         * @internal
         */
        _prePassEffectConfiguration: PrePassEffectConfiguration;
        /**
         * Returns the fragment url or shader name used in the post process.
         * @returns the fragment url or name in the shader store.
         */
        getEffectName(): string;
        /**
         * An event triggered when the postprocess is activated.
         */
        onActivateObservable: Observable<Camera>;
        private _onActivateObserver;
        /**
         * A function that is added to the onActivateObservable
         */
        set onActivate(callback: Nullable<(camera: Camera) => void>);
        /**
         * An event triggered when the postprocess changes its size.
         */
        onSizeChangedObservable: Observable<PostProcess>;
        private _onSizeChangedObserver;
        /**
         * A function that is added to the onSizeChangedObservable
         */
        set onSizeChanged(callback: (postProcess: PostProcess) => void);
        /**
         * An event triggered when the postprocess applies its effect.
         */
        onApplyObservable: Observable<Effect>;
        private _onApplyObserver;
        /**
         * A function that is added to the onApplyObservable
         */
        set onApply(callback: (effect: Effect) => void);
        /**
         * An event triggered before rendering the postprocess
         */
        onBeforeRenderObservable: Observable<Effect>;
        private _onBeforeRenderObserver;
        /**
         * A function that is added to the onBeforeRenderObservable
         */
        set onBeforeRender(callback: (effect: Effect) => void);
        /**
         * An event triggered after rendering the postprocess
         */
        onAfterRenderObservable: Observable<Effect>;
        private _onAfterRenderObserver;
        /**
         * A function that is added to the onAfterRenderObservable
         */
        set onAfterRender(callback: (efect: Effect) => void);
        /**
         * The input texture for this post process and the output texture of the previous post process. When added to a pipeline the previous post process will
         * render it's output into this texture and this texture will be used as textureSampler in the fragment shader of this post process.
         */
        get inputTexture(): RenderTargetWrapper;
        set inputTexture(value: RenderTargetWrapper);
        /**
         * Since inputTexture should always be defined, if we previously manually set `inputTexture`,
         * the only way to unset it is to use this function to restore its internal state
         */
        restoreDefaultInputTexture(): void;
        /**
         * Gets the camera which post process is applied to.
         * @returns The camera the post process is applied to.
         */
        getCamera(): Camera;
        /**
         * Gets the texel size of the postprocess.
         * See https://en.wikipedia.org/wiki/Texel_(graphics)
         */
        get texelSize(): Vector2;
        /**
         * Creates a new instance PostProcess
         * @param name The name of the PostProcess.
         * @param fragmentUrl The url of the fragment shader to be used.
         * @param parameters Array of the names of uniform non-sampler2D variables that will be passed to the shader.
         * @param samplers Array of the names of uniform sampler2D variables that will be passed to the shader.
         * @param options The required width/height ratio to downsize to before computing the render pass. (Use 1.0 for full size)
         * @param camera The camera to apply the render pass to.
         * @param samplingMode The sampling mode to be used when computing the pass. (default: 0)
         * @param engine The engine which the post process will be applied. (default: current engine)
         * @param reusable If the post process can be reused on the same frame. (default: false)
         * @param defines String of defines that will be set when running the fragment shader. (default: null)
         * @param textureType Type of textures used when performing the post process. (default: 0)
         * @param vertexUrl The url of the vertex shader to be used. (default: "postprocess")
         * @param indexParameters The index parameters to be used for babylons include syntax "#include<kernelBlurVaryingDeclaration>[0..varyingCount]". (default: undefined) See usage in babylon.blurPostProcess.ts and kernelBlur.vertex.fx
         * @param blockCompilation If the shader should not be compiled immediatly. (default: false)
         * @param textureFormat Format of textures used when performing the post process. (default: TEXTUREFORMAT_RGBA)
         */
        constructor(name: string, fragmentUrl: string, parameters: Nullable<string[]>, samplers: Nullable<string[]>, options: number | PostProcessOptions, camera: Nullable<Camera>, samplingMode?: number, engine?: Engine, reusable?: boolean, defines?: Nullable<string>, textureType?: number, vertexUrl?: string, indexParameters?: any, blockCompilation?: boolean, textureFormat?: number, shaderLanguage?: ShaderLanguage);
        /**
         * Gets a string identifying the name of the class
         * @returns "PostProcess" string
         */
        getClassName(): string;
        /**
         * Gets the engine which this post process belongs to.
         * @returns The engine the post process was enabled with.
         */
        getEngine(): Engine;
        /**
         * The effect that is created when initializing the post process.
         * @returns The created effect corresponding the the postprocess.
         */
        getEffect(): Effect;
        /**
         * To avoid multiple redundant textures for multiple post process, the output the output texture for this post process can be shared with another.
         * @param postProcess The post process to share the output with.
         * @returns This post process.
         */
        shareOutputWith(postProcess: PostProcess): PostProcess;
        /**
         * Reverses the effect of calling shareOutputWith and returns the post process back to its original state.
         * This should be called if the post process that shares output with this post process is disabled/disposed.
         */
        useOwnOutput(): void;
        /**
         * Updates the effect with the current post process compile time values and recompiles the shader.
         * @param defines Define statements that should be added at the beginning of the shader. (default: null)
         * @param uniforms Set of uniform variables that will be passed to the shader. (default: null)
         * @param samplers Set of Texture2D variables that will be passed to the shader. (default: null)
         * @param indexParameters The index parameters to be used for babylons include syntax "#include<kernelBlurVaryingDeclaration>[0..varyingCount]". (default: undefined) See usage in babylon.blurPostProcess.ts and kernelBlur.vertex.fx
         * @param onCompiled Called when the shader has been compiled.
         * @param onError Called if there is an error when compiling a shader.
         * @param vertexUrl The url of the vertex shader to be used (default: the one given at construction time)
         * @param fragmentUrl The url of the fragment shader to be used (default: the one given at construction time)
         */
        updateEffect(defines?: Nullable<string>, uniforms?: Nullable<string[]>, samplers?: Nullable<string[]>, indexParameters?: any, onCompiled?: (effect: Effect) => void, onError?: (effect: Effect, errors: string) => void, vertexUrl?: string, fragmentUrl?: string): void;
        /**
         * The post process is reusable if it can be used multiple times within one frame.
         * @returns If the post process is reusable
         */
        isReusable(): boolean;
        /** invalidate frameBuffer to hint the postprocess to create a depth buffer */
        markTextureDirty(): void;
        private _createRenderTargetTexture;
        private _flushTextureCache;
        private _resize;
        /**
         * Activates the post process by intializing the textures to be used when executed. Notifies onActivateObservable.
         * When this post process is used in a pipeline, this is call will bind the input texture of this post process to the output of the previous.
         * @param camera The camera that will be used in the post process. This camera will be used when calling onActivateObservable.
         * @param sourceTexture The source texture to be inspected to get the width and height if not specified in the post process constructor. (default: null)
         * @param forceDepthStencil If true, a depth and stencil buffer will be generated. (default: false)
         * @returns The render target wrapper that was bound to be written to.
         */
        activate(camera: Nullable<Camera>, sourceTexture?: Nullable<InternalTexture>, forceDepthStencil?: boolean): RenderTargetWrapper;
        /**
         * If the post process is supported.
         */
        get isSupported(): boolean;
        /**
         * The aspect ratio of the output texture.
         */
        get aspectRatio(): number;
        /**
         * Get a value indicating if the post-process is ready to be used
         * @returns true if the post-process is ready (shader is compiled)
         */
        isReady(): boolean;
        /**
         * Binds all textures and uniforms to the shader, this will be run on every pass.
         * @returns the effect corresponding to this post process. Null if not compiled or not ready.
         */
        apply(): Nullable<Effect>;
        private _disposeTextures;
        private _disposeTextureCache;
        /**
         * Sets the required values to the prepass renderer.
         * @param prePassRenderer defines the prepass renderer to setup.
         * @returns true if the pre pass is needed.
         */
        setPrePassRenderer(prePassRenderer: PrePassRenderer): boolean;
        /**
         * Disposes the post process.
         * @param camera The camera to dispose the post process on.
         */
        dispose(camera?: Camera): void;
        /**
         * Serializes the post process to a JSON object
         * @returns the JSON object
         */
        serialize(): any;
        /**
         * Clones this post process
         * @returns a new post process similar to this one
         */
        clone(): Nullable<PostProcess>;
        /**
         * Creates a material from parsed material data
         * @param parsedPostProcess defines parsed post process data
         * @param scene defines the hosting scene
         * @param rootUrl defines the root URL to use to load textures
         * @returns a new post process
         */
        static Parse(parsedPostProcess: any, scene: Scene, rootUrl: string): Nullable<PostProcess>;
        /**
         * @internal
         */
        static _Parse(parsedPostProcess: any, targetCamera: Camera, scene: Scene, rootUrl: string): Nullable<PostProcess>;
    }


    /**
     * PostProcessManager is used to manage one or more post processes or post process pipelines
     * See https://doc.babylonjs.com/features/featuresDeepDive/postProcesses/usePostProcesses
     */
    export class PostProcessManager {
        private _scene;
        private _indexBuffer;
        private _vertexBuffers;
        /**
         * Creates a new instance PostProcess
         * @param scene The scene that the post process is associated with.
         */
        constructor(scene: Scene);
        private _prepareBuffers;
        private _buildIndexBuffer;
        /**
         * Rebuilds the vertex buffers of the manager.
         * @internal
         */
        _rebuild(): void;
        /**
         * Prepares a frame to be run through a post process.
         * @param sourceTexture The input texture to the post processes. (default: null)
         * @param postProcesses An array of post processes to be run. (default: null)
         * @returns True if the post processes were able to be run.
         * @internal
         */
        _prepareFrame(sourceTexture?: Nullable<InternalTexture>, postProcesses?: Nullable<PostProcess[]>): boolean;
        /**
         * Manually render a set of post processes to a texture.
         * Please note, the frame buffer won't be unbound after the call in case you have more render to do.
         * @param postProcesses An array of post processes to be run.
         * @param targetTexture The render target wrapper to render to.
         * @param forceFullscreenViewport force gl.viewport to be full screen eg. 0,0,textureWidth,textureHeight
         * @param faceIndex defines the face to render to if a cubemap is defined as the target
         * @param lodLevel defines which lod of the texture to render to
         * @param doNotBindFrambuffer If set to true, assumes that the framebuffer has been bound previously
         */
        directRender(postProcesses: PostProcess[], targetTexture?: Nullable<RenderTargetWrapper>, forceFullscreenViewport?: boolean, faceIndex?: number, lodLevel?: number, doNotBindFrambuffer?: boolean): void;
        /**
         * Finalize the result of the output of the postprocesses.
         * @param doNotPresent If true the result will not be displayed to the screen.
         * @param targetTexture The render target wrapper to render to.
         * @param faceIndex The index of the face to bind the target texture to.
         * @param postProcesses The array of post processes to render.
         * @param forceFullscreenViewport force gl.viewport to be full screen eg. 0,0,textureWidth,textureHeight (default: false)
         * @internal
         */
        _finalizeFrame(doNotPresent?: boolean, targetTexture?: RenderTargetWrapper, faceIndex?: number, postProcesses?: Array<PostProcess>, forceFullscreenViewport?: boolean): void;
        /**
         * Disposes of the post process manager.
         */
        dispose(): void;
    }


    /**
     * Post process which applies a refraction texture
     * @see https://doc.babylonjs.com/features/featuresDeepDive/postProcesses/usePostProcesses#refraction
     */
    export class RefractionPostProcess extends PostProcess {
        private _refTexture;
        private _ownRefractionTexture;
        /** the base color of the refraction (used to taint the rendering) */
        color: Color3;
        /** simulated refraction depth */
        depth: number;
        /** the coefficient of the base color (0 to remove base color tainting) */
        colorLevel: number;
        /** Gets the url used to load the refraction texture */
        refractionTextureUrl: string;
        /**
         * Gets or sets the refraction texture
         * Please note that you are responsible for disposing the texture if you set it manually
         */
        get refractionTexture(): Texture;
        set refractionTexture(value: Texture);
        /**
         * Gets a string identifying the name of the class
         * @returns "RefractionPostProcess" string
         */
        getClassName(): string;
        /**
         * Initializes the RefractionPostProcess
         * @see https://doc.babylonjs.com/features/featuresDeepDive/postProcesses/usePostProcesses#refraction
         * @param name The name of the effect.
         * @param refractionTextureUrl Url of the refraction texture to use
         * @param color the base color of the refraction (used to taint the rendering)
         * @param depth simulated refraction depth
         * @param colorLevel the coefficient of the base color (0 to remove base color tainting)
         * @param options The required width/height ratio to downsize to before computing the render pass.
         * @param camera The camera to apply the render pass to.
         * @param samplingMode The sampling mode to be used when computing the pass. (default: 0)
         * @param engine The engine which the post process will be applied. (default: current engine)
         * @param reusable If the post process can be reused on the same frame. (default: false)
         */
        constructor(name: string, refractionTextureUrl: string, color: Color3, depth: number, colorLevel: number, options: number | PostProcessOptions, camera: Nullable<Camera>, samplingMode?: number, engine?: Engine, reusable?: boolean);
        /**
         * Disposes of the post process
         * @param camera Camera to dispose post process on
         */
        dispose(camera: Camera): void;
        /**
         * @internal
         */
        static _Parse(parsedPostProcess: any, targetCamera: Camera, scene: Scene, rootUrl: string): RefractionPostProcess;
    }




    /**
     * The default rendering pipeline can be added to a scene to apply common post processing effects such as anti-aliasing or depth of field.
     * See https://doc.babylonjs.com/features/featuresDeepDive/postProcesses/defaultRenderingPipeline
     */
    export class DefaultRenderingPipeline extends PostProcessRenderPipeline implements IDisposable, IAnimatable {
        private _scene;
        private _camerasToBeAttached;
        /**
         * ID of the sharpen post process,
         */
        private readonly SharpenPostProcessId;
        /**
         * @ignore
         * ID of the image processing post process;
         */
        readonly ImageProcessingPostProcessId = "ImageProcessingPostProcessEffect";
        /**
         * @ignore
         * ID of the Fast Approximate Anti-Aliasing post process;
         */
        readonly FxaaPostProcessId = "FxaaPostProcessEffect";
        /**
         * ID of the chromatic aberration post process,
         */
        private readonly ChromaticAberrationPostProcessId;
        /**
         * ID of the grain post process
         */
        private readonly GrainPostProcessId;
        /**
         * Sharpen post process which will apply a sharpen convolution to enhance edges
         */
        sharpen: SharpenPostProcess;
        private _sharpenEffect;
        private bloom;
        /**
         * Depth of field effect, applies a blur based on how far away objects are from the focus distance.
         */
        depthOfField: DepthOfFieldEffect;
        /**
         * The Fast Approximate Anti-Aliasing post process which attempts to remove aliasing from an image.
         */
        fxaa: FxaaPostProcess;
        /**
         * Image post processing pass used to perform operations such as tone mapping or color grading.
         */
        imageProcessing: ImageProcessingPostProcess;
        /**
         * Chromatic aberration post process which will shift rgb colors in the image
         */
        chromaticAberration: ChromaticAberrationPostProcess;
        private _chromaticAberrationEffect;
        /**
         * Grain post process which add noise to the image
         */
        grain: GrainPostProcess;
        private _grainEffect;
        /**
         * Glow post process which adds a glow to emissive areas of the image
         */
        private _glowLayer;
        /**
         * Animations which can be used to tweak settings over a period of time
         */
        animations: Animation[];
        private _imageProcessingConfigurationObserver;
        private _sharpenEnabled;
        private _bloomEnabled;
        private _depthOfFieldEnabled;
        private _depthOfFieldBlurLevel;
        private _fxaaEnabled;
        private _imageProcessingEnabled;
        private _defaultPipelineTextureType;
        private _bloomScale;
        private _chromaticAberrationEnabled;
        private _grainEnabled;
        private _buildAllowed;
        /**
         * Enable or disable automatic building of the pipeline when effects are enabled and disabled.
         * If false, you will have to manually call prepare() to update the pipeline.
         */
        get automaticBuild(): boolean;
        set automaticBuild(value: boolean);
        /**
         * This is triggered each time the pipeline has been built.
         */
        onBuildObservable: Observable<DefaultRenderingPipeline>;
        /**
         * Gets active scene
         */
        get scene(): Scene;
        /**
         * Enable or disable the sharpen process from the pipeline
         */
        set sharpenEnabled(enabled: boolean);
        get sharpenEnabled(): boolean;
        private _resizeObserver;
        private _hardwareScaleLevel;
        private _bloomKernel;
        /**
         * Specifies the size of the bloom blur kernel, relative to the final output size
         */
        get bloomKernel(): number;
        set bloomKernel(value: number);
        /**
         * Specifies the weight of the bloom in the final rendering
         */
        private _bloomWeight;
        /**
         * Specifies the luma threshold for the area that will be blurred by the bloom
         */
        private _bloomThreshold;
        private _hdr;
        /**
         * The strength of the bloom.
         */
        set bloomWeight(value: number);
        get bloomWeight(): number;
        /**
         * The luminance threshold to find bright areas of the image to bloom.
         */
        set bloomThreshold(value: number);
        get bloomThreshold(): number;
        /**
         * The scale of the bloom, lower value will provide better performance.
         */
        set bloomScale(value: number);
        get bloomScale(): number;
        /**
         * Enable or disable the bloom from the pipeline
         */
        set bloomEnabled(enabled: boolean);
        get bloomEnabled(): boolean;
        private _rebuildBloom;
        /**
         * If the depth of field is enabled.
         */
        get depthOfFieldEnabled(): boolean;
        set depthOfFieldEnabled(enabled: boolean);
        /**
         * Blur level of the depth of field effect. (Higher blur will effect performance)
         */
        get depthOfFieldBlurLevel(): DepthOfFieldEffectBlurLevel;
        set depthOfFieldBlurLevel(value: DepthOfFieldEffectBlurLevel);
        /**
         * If the anti aliasing is enabled.
         */
        set fxaaEnabled(enabled: boolean);
        get fxaaEnabled(): boolean;
        private _samples;
        /**
         * MSAA sample count, setting this to 4 will provide 4x anti aliasing. (default: 1)
         */
        set samples(sampleCount: number);
        get samples(): number;
        /**
         * If image processing is enabled.
         */
        set imageProcessingEnabled(enabled: boolean);
        get imageProcessingEnabled(): boolean;
        /**
         * If glow layer is enabled. (Adds a glow effect to emmissive materials)
         */
        set glowLayerEnabled(enabled: boolean);
        get glowLayerEnabled(): boolean;
        /**
         * Gets the glow layer (or null if not defined)
         */
        get glowLayer(): Nullable<GlowLayer>;
        /**
         * Enable or disable the chromaticAberration process from the pipeline
         */
        set chromaticAberrationEnabled(enabled: boolean);
        get chromaticAberrationEnabled(): boolean;
        /**
         * Enable or disable the grain process from the pipeline
         */
        set grainEnabled(enabled: boolean);
        get grainEnabled(): boolean;
        /**
         * Instantiates a DefaultRenderingPipeline.
         * @param name The rendering pipeline name (default: "")
         * @param hdr If high dynamic range textures should be used (default: true)
         * @param scene The scene linked to this pipeline (default: the last created scene)
         * @param cameras The array of cameras that the rendering pipeline will be attached to (default: scene.cameras)
         * @param automaticBuild If false, you will have to manually call prepare() to update the pipeline (default: true)
         */
        constructor(name?: string, hdr?: boolean, scene?: Scene, cameras?: Camera[], automaticBuild?: boolean);
        /**
         * Get the class name
         * @returns "DefaultRenderingPipeline"
         */
        getClassName(): string;
        /**
         * Force the compilation of the entire pipeline.
         */
        prepare(): void;
        private _hasCleared;
        private _prevPostProcess;
        private _prevPrevPostProcess;
        private _setAutoClearAndTextureSharing;
        private _depthOfFieldSceneObserver;
        private _activeCameraChangedObserver;
        private _activeCamerasChangedObserver;
        private _buildPipeline;
        private _disposePostProcesses;
        /**
         * Adds a camera to the pipeline
         * @param camera the camera to be added
         */
        addCamera(camera: Camera): void;
        /**
         * Removes a camera from the pipeline
         * @param camera the camera to remove
         */
        removeCamera(camera: Camera): void;
        /**
         * Dispose of the pipeline and stop all post processes
         */
        dispose(): void;
        /**
         * Serialize the rendering pipeline (Used when exporting)
         * @returns the serialized object
         */
        serialize(): any;
        /**
         * Parse the serialized pipeline
         * @param source Source pipeline.
         * @param scene The scene to load the pipeline to.
         * @param rootUrl The URL of the serialized pipeline.
         * @returns An instantiated pipeline from the serialized object.
         */
        static Parse(source: any, scene: Scene, rootUrl: string): DefaultRenderingPipeline;
    }




    /**
     * BABYLON.JS Chromatic Aberration GLSL Shader
     * Author: Olivier Guyot
     * Separates very slightly R, G and B colors on the edges of the screen
     * Inspired by Francois Tarlier & Martins Upitis
     */
    export class LensRenderingPipeline extends PostProcessRenderPipeline {
        /**
         * @ignore
         * The chromatic aberration PostProcess id in the pipeline
         */
        LensChromaticAberrationEffect: string;
        /**
         * @ignore
         * The highlights enhancing PostProcess id in the pipeline
         */
        HighlightsEnhancingEffect: string;
        /**
         * @ignore
         * The depth-of-field PostProcess id in the pipeline
         */
        LensDepthOfFieldEffect: string;
        private _scene;
        private _depthTexture;
        private _grainTexture;
        private _chromaticAberrationPostProcess;
        private _highlightsPostProcess;
        private _depthOfFieldPostProcess;
        private _edgeBlur;
        private _grainAmount;
        private _chromaticAberration;
        private _distortion;
        private _highlightsGain;
        private _highlightsThreshold;
        private _dofDistance;
        private _dofAperture;
        private _dofDarken;
        private _dofPentagon;
        private _blurNoise;
        /**
         * @constructor
         *
         * Effect parameters are as follow:
         * {
         *      chromatic_aberration: number;       // from 0 to x (1 for realism)
         *      edge_blur: number;                  // from 0 to x (1 for realism)
         *      distortion: number;                 // from 0 to x (1 for realism), note that this will effect the pointer position precision
         *      grain_amount: number;               // from 0 to 1
         *      grain_texture: BABYLON.Texture;     // texture to use for grain effect; if unset, use random B&W noise
         *      dof_focus_distance: number;         // depth-of-field: focus distance; unset to disable (disabled by default)
         *      dof_aperture: number;               // depth-of-field: focus blur bias (default: 1)
         *      dof_darken: number;                 // depth-of-field: darken that which is out of focus (from 0 to 1, disabled by default)
         *      dof_pentagon: boolean;              // depth-of-field: makes a pentagon-like "bokeh" effect
         *      dof_gain: number;                   // depth-of-field: highlights gain; unset to disable (disabled by default)
         *      dof_threshold: number;              // depth-of-field: highlights threshold (default: 1)
         *      blur_noise: boolean;                // add a little bit of noise to the blur (default: true)
         * }
         * Note: if an effect parameter is unset, effect is disabled
         *
         * @param name The rendering pipeline name
         * @param parameters - An object containing all parameters (see above)
         * @param scene The scene linked to this pipeline
         * @param ratio The size of the postprocesses (0.5 means that your postprocess will have a width = canvas.width 0.5 and a height = canvas.height 0.5)
         * @param cameras The array of cameras that the rendering pipeline will be attached to
         */
        constructor(name: string, parameters: any, scene: Scene, ratio?: number, cameras?: Camera[]);
        /**
         * Get the class name
         * @returns "LensRenderingPipeline"
         */
        getClassName(): string;
        /**
         * Gets associated scene
         */
        get scene(): Scene;
        /**
         * Gets or sets the edge blur
         */
        get edgeBlur(): number;
        set edgeBlur(value: number);
        /**
         * Gets or sets the grain amount
         */
        get grainAmount(): number;
        set grainAmount(value: number);
        /**
         * Gets or sets the chromatic aberration amount
         */
        get chromaticAberration(): number;
        set chromaticAberration(value: number);
        /**
         * Gets or sets the depth of field aperture
         */
        get dofAperture(): number;
        set dofAperture(value: number);
        /**
         * Gets or sets the edge distortion
         */
        get edgeDistortion(): number;
        set edgeDistortion(value: number);
        /**
         * Gets or sets the depth of field distortion
         */
        get dofDistortion(): number;
        set dofDistortion(value: number);
        /**
         * Gets or sets the darken out of focus amount
         */
        get darkenOutOfFocus(): number;
        set darkenOutOfFocus(value: number);
        /**
         * Gets or sets a boolean indicating if blur noise is enabled
         */
        get blurNoise(): boolean;
        set blurNoise(value: boolean);
        /**
         * Gets or sets a boolean indicating if pentagon bokeh is enabled
         */
        get pentagonBokeh(): boolean;
        set pentagonBokeh(value: boolean);
        /**
         * Gets or sets the highlight grain amount
         */
        get highlightsGain(): number;
        set highlightsGain(value: number);
        /**
         * Gets or sets the highlight threshold
         */
        get highlightsThreshold(): number;
        set highlightsThreshold(value: number);
        /**
         * Sets the amount of blur at the edges
         * @param amount blur amount
         */
        setEdgeBlur(amount: number): void;
        /**
         * Sets edge blur to 0
         */
        disableEdgeBlur(): void;
        /**
         * Sets the amount of grain
         * @param amount Amount of grain
         */
        setGrainAmount(amount: number): void;
        /**
         * Set grain amount to 0
         */
        disableGrain(): void;
        /**
         * Sets the chromatic aberration amount
         * @param amount amount of chromatic aberration
         */
        setChromaticAberration(amount: number): void;
        /**
         * Sets chromatic aberration amount to 0
         */
        disableChromaticAberration(): void;
        /**
         * Sets the EdgeDistortion amount
         * @param amount amount of EdgeDistortion
         */
        setEdgeDistortion(amount: number): void;
        /**
         * Sets edge distortion to 0
         */
        disableEdgeDistortion(): void;
        /**
         * Sets the FocusDistance amount
         * @param amount amount of FocusDistance
         */
        setFocusDistance(amount: number): void;
        /**
         * Disables depth of field
         */
        disableDepthOfField(): void;
        /**
         * Sets the Aperture amount
         * @param amount amount of Aperture
         */
        setAperture(amount: number): void;
        /**
         * Sets the DarkenOutOfFocus amount
         * @param amount amount of DarkenOutOfFocus
         */
        setDarkenOutOfFocus(amount: number): void;
        private _pentagonBokehIsEnabled;
        /**
         * Creates a pentagon bokeh effect
         */
        enablePentagonBokeh(): void;
        /**
         * Disables the pentagon bokeh effect
         */
        disablePentagonBokeh(): void;
        /**
         * Enables noise blur
         */
        enableNoiseBlur(): void;
        /**
         * Disables noise blur
         */
        disableNoiseBlur(): void;
        /**
         * Sets the HighlightsGain amount
         * @param amount amount of HighlightsGain
         */
        setHighlightsGain(amount: number): void;
        /**
         * Sets the HighlightsThreshold amount
         * @param amount amount of HighlightsThreshold
         */
        setHighlightsThreshold(amount: number): void;
        /**
         * Disables highlights
         */
        disableHighlights(): void;
        /**
         * Removes the internal pipeline assets and detaches the pipeline from the scene cameras
         * @param disableDepthRender If the scene's depth rendering should be disabled (default: false)
         */
        dispose(disableDepthRender?: boolean): void;
        private _createChromaticAberrationPostProcess;
        private _createHighlightsPostProcess;
        private _createDepthOfFieldPostProcess;
        private _createGrainTexture;
    }


    /**
     * Render pipeline to produce ssao effect
     */
    export class SSAO2RenderingPipeline extends PostProcessRenderPipeline {
        /**
         * @ignore
         * The PassPostProcess id in the pipeline that contains the original scene color
         */
        SSAOOriginalSceneColorEffect: string;
        /**
         * @ignore
         * The SSAO PostProcess id in the pipeline
         */
        SSAORenderEffect: string;
        /**
         * @ignore
         * The horizontal blur PostProcess id in the pipeline
         */
        SSAOBlurHRenderEffect: string;
        /**
         * @ignore
         * The vertical blur PostProcess id in the pipeline
         */
        SSAOBlurVRenderEffect: string;
        /**
         * @ignore
         * The PostProcess id in the pipeline that combines the SSAO-Blur output with the original scene color (SSAOOriginalSceneColorEffect)
         */
        SSAOCombineRenderEffect: string;
        /**
         * The output strength of the SSAO post-process. Default value is 1.0.
         */
        totalStrength: number;
        /**
         * Maximum depth value to still render AO. A smooth falloff makes the dimming more natural, so there will be no abrupt shading change.
         */
        maxZ: number;
        /**
         * In order to save performances, SSAO radius is clamped on close geometry. This ratio changes by how much
         */
        minZAspect: number;
        private _samples;
        /**
         * Number of samples used for the SSAO calculations. Default value is 8
         */
        set samples(n: number);
        get samples(): number;
        private _textureSamples;
        /**
         * Number of samples to use for antialiasing
         */
        set textureSamples(n: number);
        get textureSamples(): number;
        /**
         * Force rendering the geometry through geometry buffer
         */
        private _forceGeometryBuffer;
        private get _geometryBufferRenderer();
        private get _prePassRenderer();
        /**
         * Ratio object used for SSAO ratio and blur ratio
         */
        private _ratio;
        /**
         * Dynamically generated sphere sampler.
         */
        private _sampleSphere;
        /**
         * Blur filter offsets
         */
        private _samplerOffsets;
        private _expensiveBlur;
        /**
         * If bilateral blur should be used
         */
        set expensiveBlur(b: boolean);
        get expensiveBlur(): boolean;
        /**
         * The radius around the analyzed pixel used by the SSAO post-process. Default value is 2.0
         */
        radius: number;
        /**
         * The base color of the SSAO post-process
         * The final result is "base + ssao" between [0, 1]
         */
        base: number;
        /**
         *  Support test.
         */
        static get IsSupported(): boolean;
        private _scene;
        private _randomTexture;
        private _originalColorPostProcess;
        private _ssaoPostProcess;
        private _blurHPostProcess;
        private _blurVPostProcess;
        private _ssaoCombinePostProcess;
        /**
         * Gets active scene
         */
        get scene(): Scene;
        /**
         * @constructor
         * @param name The rendering pipeline name
         * @param scene The scene linked to this pipeline
         * @param ratio The size of the postprocesses. Can be a number shared between passes or an object for more precision: { ssaoRatio: 0.5, blurRatio: 1.0 }
         * @param cameras The array of cameras that the rendering pipeline will be attached to
         * @param forceGeometryBuffer Set to true if you want to use the legacy geometry buffer renderer
         * @param textureType The texture type used by the different post processes created by SSAO (default: Constants.TEXTURETYPE_UNSIGNED_INT)
         */
        constructor(name: string, scene: Scene, ratio: any, cameras?: Camera[], forceGeometryBuffer?: boolean, textureType?: number);
        /**
         * Get the class name
         * @returns "SSAO2RenderingPipeline"
         */
        getClassName(): string;
        /**
         * Removes the internal pipeline assets and detaches the pipeline from the scene cameras
         * @param disableGeometryBufferRenderer
         */
        dispose(disableGeometryBufferRenderer?: boolean): void;
        private _createBlurPostProcess;
        /** @internal */
        _rebuild(): void;
        private _bits;
        private _radicalInverse_VdC;
        private _hammersley;
        private _hemisphereSample_uniform;
        private _generateHemisphere;
        private _getDefinesForSSAO;
        private static readonly ORTHO_DEPTH_PROJECTION;
        private static readonly PERSPECTIVE_DEPTH_PROJECTION;
        private _createSSAOPostProcess;
        private _createSSAOCombinePostProcess;
        private _createRandomTexture;
        /**
         * Serialize the rendering pipeline (Used when exporting)
         * @returns the serialized object
         */
        serialize(): any;
        /**
         * Parse the serialized pipeline
         * @param source Source pipeline.
         * @param scene The scene to load the pipeline to.
         * @param rootUrl The URL of the serialized pipeline.
         * @returns An instantiated pipeline from the serialized object.
         */
        static Parse(source: any, scene: Scene, rootUrl: string): SSAO2RenderingPipeline;
    }


    /**
     * Render pipeline to produce ssao effect
     */
    export class SSAORenderingPipeline extends PostProcessRenderPipeline {
        /**
         * @ignore
         * The PassPostProcess id in the pipeline that contains the original scene color
         */
        SSAOOriginalSceneColorEffect: string;
        /**
         * @ignore
         * The SSAO PostProcess id in the pipeline
         */
        SSAORenderEffect: string;
        /**
         * @ignore
         * The horizontal blur PostProcess id in the pipeline
         */
        SSAOBlurHRenderEffect: string;
        /**
         * @ignore
         * The vertical blur PostProcess id in the pipeline
         */
        SSAOBlurVRenderEffect: string;
        /**
         * @ignore
         * The PostProcess id in the pipeline that combines the SSAO-Blur output with the original scene color (SSAOOriginalSceneColorEffect)
         */
        SSAOCombineRenderEffect: string;
        /**
         * The output strength of the SSAO post-process. Default value is 1.0.
         */
        totalStrength: number;
        /**
         * The radius around the analyzed pixel used by the SSAO post-process. Default value is 0.0006
         */
        radius: number;
        /**
         * Related to fallOff, used to interpolate SSAO samples (first interpolate function input) based on the occlusion difference of each pixel
         * Must not be equal to fallOff and superior to fallOff.
         * Default value is 0.0075
         */
        area: number;
        /**
         * Related to area, used to interpolate SSAO samples (second interpolate function input) based on the occlusion difference of each pixel
         * Must not be equal to area and inferior to area.
         * Default value is 0.000001
         */
        fallOff: number;
        /**
         * The base color of the SSAO post-process
         * The final result is "base + ssao" between [0, 1]
         */
        base: number;
        private _scene;
        private _randomTexture;
        private _originalColorPostProcess;
        private _ssaoPostProcess;
        private _blurHPostProcess;
        private _blurVPostProcess;
        private _ssaoCombinePostProcess;
        private _firstUpdate;
        /**
         * Gets active scene
         */
        get scene(): Scene;
        /**
         * @constructor
         * @param name - The rendering pipeline name
         * @param scene - The scene linked to this pipeline
         * @param ratio - The size of the postprocesses. Can be a number shared between passes or an object for more precision: { ssaoRatio: 0.5, combineRatio: 1.0 }
         * @param cameras - The array of cameras that the rendering pipeline will be attached to
         */
        constructor(name: string, scene: Scene, ratio: any, cameras?: Camera[]);
        /**
         * @internal
         */
        _attachCameras(cameras: any, unique: boolean): void;
        /**
         * Get the class name
         * @returns "SSAORenderingPipeline"
         */
        getClassName(): string;
        /**
         * Removes the internal pipeline assets and detaches the pipeline from the scene cameras
         * @param disableDepthRender
         */
        dispose(disableDepthRender?: boolean): void;
        private _createBlurPostProcess;
        /** @internal */
        _rebuild(): void;
        private _createSSAOPostProcess;
        private _createSSAOCombinePostProcess;
        private _createRandomTexture;
    }


    /**
     * Standard rendering pipeline
     * Default pipeline should be used going forward but the standard pipeline will be kept for backwards compatibility.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/postProcesses/standardRenderingPipeline
     */
    export class StandardRenderingPipeline extends PostProcessRenderPipeline implements IDisposable, IAnimatable {
        /**
         * Public members
         */
        /**
         * Post-process which contains the original scene color before the pipeline applies all the effects
         */
        originalPostProcess: Nullable<PostProcess>;
        /**
         * Post-process used to down scale an image x4
         */
        downSampleX4PostProcess: Nullable<PostProcess>;
        /**
         * Post-process used to calculate the illuminated surfaces controlled by a threshold
         */
        brightPassPostProcess: Nullable<PostProcess>;
        /**
         * Post-process array storing all the horizontal blur post-processes used by the pipeline
         */
        blurHPostProcesses: PostProcess[];
        /**
         * Post-process array storing all the vertical blur post-processes used by the pipeline
         */
        blurVPostProcesses: PostProcess[];
        /**
         * Post-process used to add colors of 2 textures (typically brightness + real scene color)
         */
        textureAdderPostProcess: Nullable<PostProcess>;
        /**
         * Post-process used to create volumetric lighting effect
         */
        volumetricLightPostProcess: Nullable<PostProcess>;
        /**
         * Post-process used to smooth the previous volumetric light post-process on the X axis
         */
        volumetricLightSmoothXPostProcess: Nullable<BlurPostProcess>;
        /**
         * Post-process used to smooth the previous volumetric light post-process on the Y axis
         */
        volumetricLightSmoothYPostProcess: Nullable<BlurPostProcess>;
        /**
         * Post-process used to merge the volumetric light effect and the real scene color
         */
        volumetricLightMergePostProces: Nullable<PostProcess>;
        /**
         * Post-process used to store the final volumetric light post-process (attach/detach for debug purpose)
         */
        volumetricLightFinalPostProcess: Nullable<PostProcess>;
        /**
         * Base post-process used to calculate the average luminance of the final image for HDR
         */
        luminancePostProcess: Nullable<PostProcess>;
        /**
         * Post-processes used to create down sample post-processes in order to get
         * the average luminance of the final image for HDR
         * Array of length "StandardRenderingPipeline.LuminanceSteps"
         */
        luminanceDownSamplePostProcesses: PostProcess[];
        /**
         * Post-process used to create a HDR effect (light adaptation)
         */
        hdrPostProcess: Nullable<PostProcess>;
        /**
         * Post-process used to store the final texture adder post-process (attach/detach for debug purpose)
         */
        textureAdderFinalPostProcess: Nullable<PostProcess>;
        /**
         * Post-process used to store the final lens flare post-process (attach/detach for debug purpose)
         */
        lensFlareFinalPostProcess: Nullable<PostProcess>;
        /**
         * Post-process used to merge the final HDR post-process and the real scene color
         */
        hdrFinalPostProcess: Nullable<PostProcess>;
        /**
         * Post-process used to create a lens flare effect
         */
        lensFlarePostProcess: Nullable<PostProcess>;
        /**
         * Post-process that merges the result of the lens flare post-process and the real scene color
         */
        lensFlareComposePostProcess: Nullable<PostProcess>;
        /**
         * Post-process used to create a motion blur effect
         */
        motionBlurPostProcess: Nullable<PostProcess>;
        /**
         * Post-process used to create a depth of field effect
         */
        depthOfFieldPostProcess: Nullable<PostProcess>;
        /**
         * The Fast Approximate Anti-Aliasing post process which attempts to remove aliasing from an image.
         */
        fxaaPostProcess: Nullable<FxaaPostProcess>;
        /**
         * Post-process used to simulate realtime reflections using the screen space and geometry renderer.
         */
        screenSpaceReflectionPostProcess: Nullable<ScreenSpaceReflectionPostProcess>;
        /**
         * Represents the brightness threshold in order to configure the illuminated surfaces
         */
        brightThreshold: number;
        /**
         * Configures the blur intensity used for surexposed surfaces are highlighted surfaces (light halo)
         */
        blurWidth: number;
        /**
         * Sets if the blur for highlighted surfaces must be only horizontal
         */
        horizontalBlur: boolean;
        /**
         * Gets the overall exposure used by the pipeline
         */
        get exposure(): number;
        /**
         * Sets the overall exposure used by the pipeline
         */
        set exposure(value: number);
        /**
         * Texture used typically to simulate "dirty" on camera lens
         */
        lensTexture: Nullable<Texture>;
        /**
         * Represents the offset coefficient based on Rayleigh principle. Typically in interval [-0.2, 0.2]
         */
        volumetricLightCoefficient: number;
        /**
         * The overall power of volumetric lights, typically in interval [0, 10] maximum
         */
        volumetricLightPower: number;
        /**
         * Used the set the blur intensity to smooth the volumetric lights
         */
        volumetricLightBlurScale: number;
        /**
         * Light (spot or directional) used to generate the volumetric lights rays
         * The source light must have a shadow generate so the pipeline can get its
         * depth map
         */
        sourceLight: Nullable<SpotLight | DirectionalLight>;
        /**
         * For eye adaptation, represents the minimum luminance the eye can see
         */
        hdrMinimumLuminance: number;
        /**
         * For eye adaptation, represents the decrease luminance speed
         */
        hdrDecreaseRate: number;
        /**
         * For eye adaptation, represents the increase luminance speed
         */
        hdrIncreaseRate: number;
        /**
         * Gets whether or not the exposure of the overall pipeline should be automatically adjusted by the HDR post-process
         */
        get hdrAutoExposure(): boolean;
        /**
         * Sets whether or not the exposure of the overall pipeline should be automatically adjusted by the HDR post-process
         */
        set hdrAutoExposure(value: boolean);
        /**
         * Lens color texture used by the lens flare effect. Mandatory if lens flare effect enabled
         */
        lensColorTexture: Nullable<Texture>;
        /**
         * The overall strength for the lens flare effect
         */
        lensFlareStrength: number;
        /**
         * Dispersion coefficient for lens flare ghosts
         */
        lensFlareGhostDispersal: number;
        /**
         * Main lens flare halo width
         */
        lensFlareHaloWidth: number;
        /**
         * Based on the lens distortion effect, defines how much the lens flare result
         * is distorted
         */
        lensFlareDistortionStrength: number;
        /**
         * Configures the blur intensity used for for lens flare (halo)
         */
        lensFlareBlurWidth: number;
        /**
         * Lens star texture must be used to simulate rays on the flares and is available
         * in the documentation
         */
        lensStarTexture: Nullable<Texture>;
        /**
         * As the "lensTexture" (can be the same texture or different), it is used to apply the lens
         * flare effect by taking account of the dirt texture
         */
        lensFlareDirtTexture: Nullable<Texture>;
        /**
         * Represents the focal length for the depth of field effect
         */
        depthOfFieldDistance: number;
        /**
         * Represents the blur intensity for the blurred part of the depth of field effect
         */
        depthOfFieldBlurWidth: number;
        /**
         * Gets how much the image is blurred by the movement while using the motion blur post-process
         */
        get motionStrength(): number;
        /**
         * Sets how much the image is blurred by the movement while using the motion blur post-process
         */
        set motionStrength(strength: number);
        /**
         * Gets whether or not the motion blur post-process is object based or screen based.
         */
        get objectBasedMotionBlur(): boolean;
        /**
         * Sets whether or not the motion blur post-process should be object based or screen based
         */
        set objectBasedMotionBlur(value: boolean);
        /**
         * List of animations for the pipeline (IAnimatable implementation)
         */
        animations: Animation[];
        /**
         * Private members
         */
        private _scene;
        private _currentDepthOfFieldSource;
        private _basePostProcess;
        private _fixedExposure;
        private _currentExposure;
        private _hdrAutoExposure;
        private _hdrCurrentLuminance;
        private _motionStrength;
        private _isObjectBasedMotionBlur;
        private _floatTextureType;
        private _camerasToBeAttached;
        private _ratio;
        private _bloomEnabled;
        private _depthOfFieldEnabled;
        private _vlsEnabled;
        private _lensFlareEnabled;
        private _hdrEnabled;
        private _motionBlurEnabled;
        private _fxaaEnabled;
        private _screenSpaceReflectionsEnabled;
        private _motionBlurSamples;
        private _volumetricLightStepsCount;
        private _samples;
        /**
         * @ignore
         * Specifies if the bloom pipeline is enabled
         */
        get BloomEnabled(): boolean;
        set BloomEnabled(enabled: boolean);
        /**
         * @ignore
         * Specifies if the depth of field pipeline is enabled
         */
        get DepthOfFieldEnabled(): boolean;
        set DepthOfFieldEnabled(enabled: boolean);
        /**
         * @ignore
         * Specifies if the lens flare pipeline is enabled
         */
        get LensFlareEnabled(): boolean;
        set LensFlareEnabled(enabled: boolean);
        /**
         * @ignore
         * Specifies if the HDR pipeline is enabled
         */
        get HDREnabled(): boolean;
        set HDREnabled(enabled: boolean);
        /**
         * @ignore
         * Specifies if the volumetric lights scattering effect is enabled
         */
        get VLSEnabled(): boolean;
        set VLSEnabled(enabled: boolean);
        /**
         * @ignore
         * Specifies if the motion blur effect is enabled
         */
        get MotionBlurEnabled(): boolean;
        set MotionBlurEnabled(enabled: boolean);
        /**
         * Specifies if anti-aliasing is enabled
         */
        get fxaaEnabled(): boolean;
        set fxaaEnabled(enabled: boolean);
        /**
         * Specifies if screen space reflections are enabled.
         */
        get screenSpaceReflectionsEnabled(): boolean;
        set screenSpaceReflectionsEnabled(enabled: boolean);
        /**
         * Specifies the number of steps used to calculate the volumetric lights
         * Typically in interval [50, 200]
         */
        get volumetricLightStepsCount(): number;
        set volumetricLightStepsCount(count: number);
        /**
         * Specifies the number of samples used for the motion blur effect
         * Typically in interval [16, 64]
         */
        get motionBlurSamples(): number;
        set motionBlurSamples(samples: number);
        /**
         * Specifies MSAA sample count, setting this to 4 will provide 4x anti aliasing. (default: 1)
         */
        get samples(): number;
        set samples(sampleCount: number);
        /**
         * Default pipeline should be used going forward but the standard pipeline will be kept for backwards compatibility.
         * @constructor
         * @param name The rendering pipeline name
         * @param scene The scene linked to this pipeline
         * @param ratio The size of the postprocesses (0.5 means that your postprocess will have a width = canvas.width 0.5 and a height = canvas.height 0.5)
         * @param originalPostProcess the custom original color post-process. Must be "reusable". Can be null.
         * @param cameras The array of cameras that the rendering pipeline will be attached to
         */
        constructor(name: string, scene: Scene, ratio: number, originalPostProcess?: Nullable<PostProcess>, cameras?: Camera[]);
        private _buildPipeline;
        private _createDownSampleX4PostProcess;
        private _createBrightPassPostProcess;
        private _createBlurPostProcesses;
        private _createTextureAdderPostProcess;
        private _createVolumetricLightPostProcess;
        private _createLuminancePostProcesses;
        private _createHdrPostProcess;
        private _createLensFlarePostProcess;
        private _createDepthOfFieldPostProcess;
        private _createMotionBlurPostProcess;
        private _getDepthTexture;
        private _disposePostProcesses;
        /**
         * Dispose of the pipeline and stop all post processes
         */
        dispose(): void;
        /**
         * Serialize the rendering pipeline (Used when exporting)
         * @returns the serialized object
         */
        serialize(): any;
        /**
         * Parse the serialized pipeline
         * @param source Source pipeline.
         * @param scene The scene to load the pipeline to.
         * @param rootUrl The URL of the serialized pipeline.
         * @returns An instantiated pipeline from the serialized object.
         */
        static Parse(source: any, scene: Scene, rootUrl: string): StandardRenderingPipeline;
        /**
         * Luminance steps
         */
        static LuminanceSteps: number;
    }


    /**
     * This represents a set of one or more post processes in Babylon.
     * A post process can be used to apply a shader to a texture after it is rendered.
     * @example https://doc.babylonjs.com/features/featuresDeepDive/postProcesses/postProcessRenderPipeline
     */
    export class PostProcessRenderEffect {
        private _postProcesses;
        private _getPostProcesses;
        private _singleInstance;
        private _cameras;
        private _indicesForCamera;
        /**
         * Name of the effect
         * @internal
         */
        _name: string;
        /**
         * Instantiates a post process render effect.
         * A post process can be used to apply a shader to a texture after it is rendered.
         * @param engine The engine the effect is tied to
         * @param name The name of the effect
         * @param getPostProcesses A function that returns a set of post processes which the effect will run in order to be run.
         * @param singleInstance False if this post process can be run on multiple cameras. (default: true)
         */
        constructor(engine: Engine, name: string, getPostProcesses: () => Nullable<PostProcess | Array<PostProcess>>, singleInstance?: boolean);
        /**
         * Checks if all the post processes in the effect are supported.
         */
        get isSupported(): boolean;
        /**
         * Updates the current state of the effect
         * @internal
         */
        _update(): void;
        /**
         * Attaches the effect on cameras
         * @param cameras The camera to attach to.
         * @internal
         */
        _attachCameras(cameras: Camera): void;
        /**
         * Attaches the effect on cameras
         * @param cameras The camera to attach to.
         * @internal
         */
        _attachCameras(cameras: Camera[]): void;
        /**
         * Detaches the effect on cameras
         * @param cameras The camera to detach from.
         * @internal
         */
        _detachCameras(cameras: Camera): void;
        /**
         * Detaches the effect on cameras
         * @param cameras The camera to detach from.
         * @internal
         */
        _detachCameras(cameras: Camera[]): void;
        /**
         * Enables the effect on given cameras
         * @param cameras The camera to enable.
         * @internal
         */
        _enable(cameras: Camera): void;
        /**
         * Enables the effect on given cameras
         * @param cameras The camera to enable.
         * @internal
         */
        _enable(cameras: Nullable<Camera[]>): void;
        /**
         * Disables the effect on the given cameras
         * @param cameras The camera to disable.
         * @internal
         */
        _disable(cameras: Camera): void;
        /**
         * Disables the effect on the given cameras
         * @param cameras The camera to disable.
         * @internal
         */
        _disable(cameras: Nullable<Camera[]>): void;
        /**
         * Gets a list of the post processes contained in the effect.
         * @param camera The camera to get the post processes on.
         * @returns The list of the post processes in the effect.
         */
        getPostProcesses(camera?: Camera): Nullable<Array<PostProcess>>;
    }


    /**
     * PostProcessRenderPipeline
     * @see https://doc.babylonjs.com/features/featuresDeepDive/postProcesses/postProcessRenderPipeline
     */
    export class PostProcessRenderPipeline {
        private _engine;
        protected _renderEffects: {
            [key: string]: PostProcessRenderEffect;
        };
        protected _renderEffectsForIsolatedPass: PostProcessRenderEffect[];
        /**
         * List of inspectable custom properties (used by the Inspector)
         * @see https://doc.babylonjs.com/toolsAndResources/inspector#extensibility
         */
        inspectableCustomProperties: IInspectable[];
        /**
         * @internal
         */
        protected _cameras: Camera[];
        /** @internal */
        _name: string;
        /**
         * Gets pipeline name
         */
        get name(): string;
        /** Gets the list of attached cameras */
        get cameras(): Camera[];
        /**
         * Initializes a PostProcessRenderPipeline
         * @param _engine engine to add the pipeline to
         * @param name name of the pipeline
         */
        constructor(_engine: Engine, name: string);
        /**
         * Gets the class name
         * @returns "PostProcessRenderPipeline"
         */
        getClassName(): string;
        /**
         * If all the render effects in the pipeline are supported
         */
        get isSupported(): boolean;
        /**
         * Adds an effect to the pipeline
         * @param renderEffect the effect to add
         */
        addEffect(renderEffect: PostProcessRenderEffect): void;
        /** @internal */
        _rebuild(): void;
        /** @internal */
        _enableEffect(renderEffectName: string, cameras: Camera): void;
        /** @internal */
        _enableEffect(renderEffectName: string, cameras: Camera[]): void;
        /** @internal */
        _disableEffect(renderEffectName: string, cameras: Nullable<Camera[]>): void;
        /** @internal */
        _disableEffect(renderEffectName: string, cameras: Nullable<Camera[]>): void;
        /** @internal */
        _attachCameras(cameras: Camera, unique: boolean): void;
        /** @internal */
        _attachCameras(cameras: Camera[], unique: boolean): void;
        /** @internal */
        _detachCameras(cameras: Camera): void;
        /** @internal */
        _detachCameras(cameras: Nullable<Camera[]>): void;
        /** @internal */
        _update(): void;
        /** @internal */
        _reset(): void;
        protected _enableMSAAOnFirstPostProcess(sampleCount: number): boolean;
        /**
         * Sets the required values to the prepass renderer.
         * @param prePassRenderer defines the prepass renderer to setup.
         * @returns true if the pre pass is needed.
         */
        setPrePassRenderer(prePassRenderer: PrePassRenderer): boolean;
        /**
         * Disposes of the pipeline
         */
        dispose(): void;
    }


    /**
     * PostProcessRenderPipelineManager class
     * @see https://doc.babylonjs.com/features/featuresDeepDive/postProcesses/postProcessRenderPipeline
     */
    export class PostProcessRenderPipelineManager {
        private _renderPipelines;
        /**
         * Initializes a PostProcessRenderPipelineManager
         * @see https://doc.babylonjs.com/features/featuresDeepDive/postProcesses/postProcessRenderPipeline
         */
        constructor();
        /**
         * Gets the list of supported render pipelines
         */
        get supportedPipelines(): PostProcessRenderPipeline[];
        /**
         * Adds a pipeline to the manager
         * @param renderPipeline The pipeline to add
         */
        addPipeline(renderPipeline: PostProcessRenderPipeline): void;
        /**
         * Attaches a camera to the pipeline
         * @param renderPipelineName The name of the pipeline to attach to
         * @param cameras the camera to attach
         * @param unique if the camera can be attached multiple times to the pipeline
         */
        attachCamerasToRenderPipeline(renderPipelineName: string, cameras: any | Camera[] | Camera, unique?: boolean): void;
        /**
         * Detaches a camera from the pipeline
         * @param renderPipelineName The name of the pipeline to detach from
         * @param cameras the camera to detach
         */
        detachCamerasFromRenderPipeline(renderPipelineName: string, cameras: any | Camera[] | Camera): void;
        /**
         * Enables an effect by name on a pipeline
         * @param renderPipelineName the name of the pipeline to enable the effect in
         * @param renderEffectName the name of the effect to enable
         * @param cameras the cameras that the effect should be enabled on
         */
        enableEffectInPipeline(renderPipelineName: string, renderEffectName: string, cameras: any | Camera[] | Camera): void;
        /**
         * Disables an effect by name on a pipeline
         * @param renderPipelineName the name of the pipeline to disable the effect in
         * @param renderEffectName the name of the effect to disable
         * @param cameras the cameras that the effect should be disabled on
         */
        disableEffectInPipeline(renderPipelineName: string, renderEffectName: string, cameras: any | Camera[] | Camera): void;
        /**
         * Updates the state of all contained render pipelines and disposes of any non supported pipelines
         */
        update(): void;
        /** @internal */
        _rebuild(): void;
        /**
         * Disposes of the manager and pipelines
         */
        dispose(): void;
    }


        interface Scene {
            /** @internal (Backing field) */
            _postProcessRenderPipelineManager: PostProcessRenderPipelineManager;
            /**
             * Gets the postprocess render pipeline manager
             * @see https://doc.babylonjs.com/features/featuresDeepDive/postProcesses/postProcessRenderPipeline
             * @see https://doc.babylonjs.com/features/featuresDeepDive/postProcesses/defaultRenderingPipeline
             */
            readonly postProcessRenderPipelineManager: PostProcessRenderPipelineManager;
        }
    /**
     * Defines the Render Pipeline scene component responsible to rendering pipelines
     */
    export class PostProcessRenderPipelineManagerSceneComponent implements ISceneComponent {
        /**
         * The component name helpful to identify the component in the list of scene components.
         */
        readonly name = "PostProcessRenderPipelineManager";
        /**
         * The scene the component belongs to.
         */
        scene: Scene;
        /**
         * Creates a new instance of the component for the given scene
         * @param scene Defines the scene to register the component in
         */
        constructor(scene: Scene);
        /**
         * Registers the component in a given scene
         */
        register(): void;
        /**
         * Rebuilds the elements related to this component in case of
         * context lost for instance.
         */
        rebuild(): void;
        /**
         * Disposes the component and the associated resources
         */
        dispose(): void;
        private _gatherRenderTargets;
    }


    /**
     * The Screen Space curvature effect can help highlighting ridge and valley of a model.
     */
    export class ScreenSpaceCurvaturePostProcess extends PostProcess {
        /**
         * Defines how much ridge the curvature effect displays.
         */
        ridge: number;
        /**
         * Defines how much valley the curvature effect displays.
         */
        valley: number;
        private _geometryBufferRenderer;
        /**
         * Gets a string identifying the name of the class
         * @returns "ScreenSpaceCurvaturePostProcess" string
         */
        getClassName(): string;
        /**
         * Creates a new instance ScreenSpaceCurvaturePostProcess
         * @param name The name of the effect.
         * @param scene The scene containing the objects to blur according to their velocity.
         * @param options The required width/height ratio to downsize to before computing the render pass.
         * @param camera The camera to apply the render pass to.
         * @param samplingMode The sampling mode to be used when computing the pass. (default: 0)
         * @param engine The engine which the post process will be applied. (default: current engine)
         * @param reusable If the post process can be reused on the same frame. (default: false)
         * @param textureType Type of textures used when performing the post process. (default: 0)
         * @param blockCompilation If compilation of the shader should not be done in the constructor. The updateEffect method can be used to compile the shader at a later time. (default: false)
         */
        constructor(name: string, scene: Scene, options: number | PostProcessOptions, camera: Nullable<Camera>, samplingMode?: number, engine?: Engine, reusable?: boolean, textureType?: number, blockCompilation?: boolean);
        /**
         * Support test.
         */
        static get IsSupported(): boolean;
        /**
         * @internal
         */
        static _Parse(parsedPostProcess: any, targetCamera: Camera, scene: Scene, rootUrl: string): ScreenSpaceCurvaturePostProcess;
    }


    /**
     * The ScreenSpaceReflectionPostProcess performs realtime reflections using only and only the available informations on the screen (positions and normals).
     * Basically, the screen space reflection post-process will compute reflections according the material's reflectivity.
     */
    export class ScreenSpaceReflectionPostProcess extends PostProcess {
        /**
         * Gets or sets a reflection threshold mainly used to adjust the reflection's height.
         */
        threshold: number;
        /**
         * Gets or sets the current reflection strength. 1.0 is an ideal value but can be increased/decreased for particular results.
         */
        strength: number;
        /**
         * Gets or sets the falloff exponent used while computing fresnel. More the exponent is high, more the reflections will be discrete.
         */
        reflectionSpecularFalloffExponent: number;
        /**
         * Gets or sets the step size used to iterate until the effect finds the color of the reflection's pixel. Typically in interval [0.1, 1.0]
         */
        step: number;
        /**
         * Gets or sets the factor applied when computing roughness. Default value is 0.2.
         */
        roughnessFactor: number;
        private _forceGeometryBuffer;
        private get _geometryBufferRenderer();
        private get _prePassRenderer();
        private _enableSmoothReflections;
        private _reflectionSamples;
        private _smoothSteps;
        private _isSceneRightHanded;
        /**
         * Gets a string identifying the name of the class
         * @returns "ScreenSpaceReflectionPostProcess" string
         */
        getClassName(): string;
        /**
         * Creates a new instance of ScreenSpaceReflectionPostProcess.
         * @param name The name of the effect.
         * @param scene The scene containing the objects to calculate reflections.
         * @param options The required width/height ratio to downsize to before computing the render pass.
         * @param camera The camera to apply the render pass to.
         * @param samplingMode The sampling mode to be used when computing the pass. (default: 0)
         * @param engine The engine which the post process will be applied. (default: current engine)
         * @param reusable If the post process can be reused on the same frame. (default: false)
         * @param textureType Type of textures used when performing the post process. (default: 0)
         * @param blockCompilation If compilation of the shader should not be done in the constructor. The updateEffect method can be used to compile the shader at a later time. (default: true)
         * @param forceGeometryBuffer If this post process should use geometry buffer instead of prepass (default: false)
         */
        constructor(name: string, scene: Scene, options: number | PostProcessOptions, camera: Nullable<Camera>, samplingMode?: number, engine?: Engine, reusable?: boolean, textureType?: number, blockCompilation?: boolean, forceGeometryBuffer?: boolean);
        /**
         * Gets whether or not smoothing reflections is enabled.
         * Enabling smoothing will require more GPU power and can generate a drop in FPS.
         */
        get enableSmoothReflections(): boolean;
        /**
         * Sets whether or not smoothing reflections is enabled.
         * Enabling smoothing will require more GPU power and can generate a drop in FPS.
         */
        set enableSmoothReflections(enabled: boolean);
        /**
         * Gets the number of samples taken while computing reflections. More samples count is high,
         * more the post-process wil require GPU power and can generate a drop in FPS. Basically in interval [25, 100].
         */
        get reflectionSamples(): number;
        /**
         * Sets the number of samples taken while computing reflections. More samples count is high,
         * more the post-process wil require GPU power and can generate a drop in FPS. Basically in interval [25, 100].
         */
        set reflectionSamples(samples: number);
        /**
         * Gets the number of samples taken while smoothing reflections. More samples count is high,
         * more the post-process will require GPU power and can generate a drop in FPS.
         * Default value (5.0) work pretty well in all cases but can be adjusted.
         */
        get smoothSteps(): number;
        set smoothSteps(steps: number);
        private _updateEffectDefines;
        /**
         * @internal
         */
        static _Parse(parsedPostProcess: any, targetCamera: Camera, scene: Scene, rootUrl: string): ScreenSpaceReflectionPostProcess;
    }


    /**
     * The SharpenPostProcess applies a sharpen kernel to every pixel
     * See http://en.wikipedia.org/wiki/Kernel_(image_processing)
     */
    export class SharpenPostProcess extends PostProcess {
        /**
         * How much of the original color should be applied. Setting this to 0 will display edge detection. (default: 1)
         */
        colorAmount: number;
        /**
         * How much sharpness should be applied (default: 0.3)
         */
        edgeAmount: number;
        /**
         * Gets a string identifying the name of the class
         * @returns "SharpenPostProcess" string
         */
        getClassName(): string;
        /**
         * Creates a new instance ConvolutionPostProcess
         * @param name The name of the effect.
         * @param options The required width/height ratio to downsize to before computing the render pass.
         * @param camera The camera to apply the render pass to.
         * @param samplingMode The sampling mode to be used when computing the pass. (default: 0)
         * @param engine The engine which the post process will be applied. (default: current engine)
         * @param reusable If the post process can be reused on the same frame. (default: false)
         * @param textureType Type of textures used when performing the post process. (default: 0)
         * @param blockCompilation If compilation of the shader should not be done in the constructor. The updateEffect method can be used to compile the shader at a later time. (default: false)
         */
        constructor(name: string, options: number | PostProcessOptions, camera: Nullable<Camera>, samplingMode?: number, engine?: Engine, reusable?: boolean, textureType?: number, blockCompilation?: boolean);
        /**
         * @internal
         */
        static _Parse(parsedPostProcess: any, targetCamera: Camera, scene: Scene, rootUrl: string): SharpenPostProcess;
    }


    /**
     * StereoscopicInterlacePostProcessI used to render stereo views from a rigged camera with support for alternate line interlacing
     */
    export class StereoscopicInterlacePostProcessI extends PostProcess {
        private _stepSize;
        private _passedProcess;
        /**
         * Gets a string identifying the name of the class
         * @returns "StereoscopicInterlacePostProcessI" string
         */
        getClassName(): string;
        /**
         * Initializes a StereoscopicInterlacePostProcessI
         * @param name The name of the effect.
         * @param rigCameras The rig cameras to be applied to the post process
         * @param isStereoscopicHoriz If the rendered results are horizontal or vertical
         * @param isStereoscopicInterlaced If the rendered results are alternate line interlaced
         * @param samplingMode The sampling mode to be used when computing the pass. (default: 0)
         * @param engine The engine which the post process will be applied. (default: current engine)
         * @param reusable If the post process can be reused on the same frame. (default: false)
         */
        constructor(name: string, rigCameras: Camera[], isStereoscopicHoriz: boolean, isStereoscopicInterlaced: boolean, samplingMode?: number, engine?: Engine, reusable?: boolean);
    }
    /**
     * StereoscopicInterlacePostProcess used to render stereo views from a rigged camera
     */
    export class StereoscopicInterlacePostProcess extends PostProcess {
        private _stepSize;
        private _passedProcess;
        /**
         * Gets a string identifying the name of the class
         * @returns "StereoscopicInterlacePostProcess" string
         */
        getClassName(): string;
        /**
         * Initializes a StereoscopicInterlacePostProcess
         * @param name The name of the effect.
         * @param rigCameras The rig cameras to be applied to the post process
         * @param isStereoscopicHoriz If the rendered results are horizontal or vertical
         * @param samplingMode The sampling mode to be used when computing the pass. (default: 0)
         * @param engine The engine which the post process will be applied. (default: current engine)
         * @param reusable If the post process can be reused on the same frame. (default: false)
         */
        constructor(name: string, rigCameras: Camera[], isStereoscopicHoriz: boolean, samplingMode?: number, engine?: Engine, reusable?: boolean);
    }


    /**
     * Sub surface scattering post process
     */
    export class SubSurfaceScatteringPostProcess extends PostProcess {
        /**
         * Gets a string identifying the name of the class
         * @returns "SubSurfaceScatteringPostProcess" string
         */
        getClassName(): string;
        constructor(name: string, scene: Scene, options: number | PostProcessOptions, camera?: Nullable<Camera>, samplingMode?: number, engine?: Engine, reusable?: boolean, textureType?: number);
    }


    /** Defines operator used for tonemapping */
    export enum TonemappingOperator {
        /** Hable */
        Hable = 0,
        /** Reinhard */
        Reinhard = 1,
        /** HejiDawson */
        HejiDawson = 2,
        /** Photographic */
        Photographic = 3
    }
    /**
     * Defines a post process to apply tone mapping
     */
    export class TonemapPostProcess extends PostProcess {
        private _operator;
        /** Defines the required exposure adjustment */
        exposureAdjustment: number;
        /**
         * Gets a string identifying the name of the class
         * @returns "TonemapPostProcess" string
         */
        getClassName(): string;
        /**
         * Creates a new TonemapPostProcess
         * @param name defines the name of the postprocess
         * @param _operator defines the operator to use
         * @param exposureAdjustment defines the required exposure adjustment
         * @param camera defines the camera to use (can be null)
         * @param samplingMode defines the required sampling mode (BABYLON.Texture.BILINEAR_SAMPLINGMODE by default)
         * @param engine defines the hosting engine (can be ignore if camera is set)
         * @param textureFormat defines the texture format to use (BABYLON.Engine.TEXTURETYPE_UNSIGNED_INT by default)
         * @param reusable If the post process can be reused on the same frame. (default: false)
         */
        constructor(name: string, _operator: TonemappingOperator, 
        /** Defines the required exposure adjustment */
        exposureAdjustment: number, camera: Nullable<Camera>, samplingMode?: number, engine?: Engine, textureFormat?: number, reusable?: boolean);
    }


    /**
     *  Inspired by https://developer.nvidia.com/gpugems/gpugems3/part-ii-light-and-shadows/chapter-13-volumetric-light-scattering-post-process
     */
    export class VolumetricLightScatteringPostProcess extends PostProcess {
        private _volumetricLightScatteringRTT;
        private _viewPort;
        private _screenCoordinates;
        /**
         * If not undefined, the mesh position is computed from the attached node position
         */
        attachedNode: {
            position: Vector3;
        };
        /**
         * Custom position of the mesh. Used if "useCustomMeshPosition" is set to "true"
         */
        customMeshPosition: Vector3;
        /**
         * Set if the post-process should use a custom position for the light source (true) or the internal mesh position (false)
         */
        useCustomMeshPosition: boolean;
        /**
         * If the post-process should inverse the light scattering direction
         */
        invert: boolean;
        /**
         * The internal mesh used by the post-process
         */
        mesh: Mesh;
        /**
         * @internal
         * VolumetricLightScatteringPostProcess.useDiffuseColor is no longer used, use the mesh material directly instead
         */
        get useDiffuseColor(): boolean;
        set useDiffuseColor(useDiffuseColor: boolean);
        /**
         * Array containing the excluded meshes not rendered in the internal pass
         */
        excludedMeshes: AbstractMesh[];
        /**
         * Array containing the only meshes rendered in the internal pass.
         * If this array is not empty, only the meshes from this array are rendered in the internal pass
         */
        includedMeshes: AbstractMesh[];
        /**
         * Controls the overall intensity of the post-process
         */
        exposure: number;
        /**
         * Dissipates each sample's contribution in range [0, 1]
         */
        decay: number;
        /**
         * Controls the overall intensity of each sample
         */
        weight: number;
        /**
         * Controls the density of each sample
         */
        density: number;
        /**
         * @constructor
         * @param name The post-process name
         * @param ratio The size of the post-process and/or internal pass (0.5 means that your postprocess will have a width = canvas.width 0.5 and a height = canvas.height 0.5)
         * @param camera The camera that the post-process will be attached to
         * @param mesh The mesh used to create the light scattering
         * @param samples The post-process quality, default 100
         * @param samplingMode The post-process filtering mode
         * @param engine The babylon engine
         * @param reusable If the post-process is reusable
         * @param scene The constructor needs a scene reference to initialize internal components. If "camera" is null a "scene" must be provided
         */
        constructor(name: string, ratio: any, camera: Nullable<Camera>, mesh?: Mesh, samples?: number, samplingMode?: number, engine?: Engine, reusable?: boolean, scene?: Scene);
        /**
         * Returns the string "VolumetricLightScatteringPostProcess"
         * @returns "VolumetricLightScatteringPostProcess"
         */
        getClassName(): string;
        private _isReady;
        /**
         * Sets the new light position for light scattering effect
         * @param position The new custom light position
         */
        setCustomMeshPosition(position: Vector3): void;
        /**
         * Returns the light position for light scattering effect
         * @returns Vector3 The custom light position
         */
        getCustomMeshPosition(): Vector3;
        /**
         * Disposes the internal assets and detaches the post-process from the camera
         * @param camera
         */
        dispose(camera: Camera): void;
        /**
         * Returns the render target texture used by the post-process
         * @returns the render target texture used by the post-process
         */
        getPass(): RenderTargetTexture;
        private _meshExcluded;
        private _createPass;
        private _updateMeshScreenCoordinates;
        /**
         * Creates a default mesh for the Volumeric Light Scattering post-process
         * @param name The mesh name
         * @param scene The scene where to create the mesh
         * @returns the default mesh
         */
        static CreateDefaultMesh(name: string, scene: Scene): Mesh;
    }


    /**
     * VRDistortionCorrectionPostProcess used for mobile VR
     */
    export class VRDistortionCorrectionPostProcess extends PostProcess {
        private _isRightEye;
        private _distortionFactors;
        private _postProcessScaleFactor;
        private _lensCenterOffset;
        private _scaleIn;
        private _scaleFactor;
        private _lensCenter;
        /**
         * Gets a string identifying the name of the class
         * @returns "VRDistortionCorrectionPostProcess" string
         */
        getClassName(): string;
        /**
         * Initializes the VRDistortionCorrectionPostProcess
         * @param name The name of the effect.
         * @param camera The camera to apply the render pass to.
         * @param isRightEye If this is for the right eye distortion
         * @param vrMetrics All the required metrics for the VR camera
         */
        constructor(name: string, camera: Nullable<Camera>, isRightEye: boolean, vrMetrics: VRCameraMetrics);
    }


    /**
     * VRMultiviewToSingleview used to convert multiview texture arrays to standard textures for scenarios such as webVR
     * This will not be used for webXR as it supports displaying texture arrays directly
     */
    export class VRMultiviewToSingleviewPostProcess extends PostProcess {
        /**
         * Gets a string identifying the name of the class
         * @returns "VRMultiviewToSingleviewPostProcess" string
         */
        getClassName(): string;
        /**
         * Initializes a VRMultiviewToSingleview
         * @param name name of the post process
         * @param camera camera to be applied to
         * @param scaleFactor scaling factor to the size of the output texture
         */
        constructor(name: string, camera: Nullable<Camera>, scaleFactor: number);
    }




        interface AbstractScene {
            /**
             * The list of reflection probes added to the scene
             * @see https://doc.babylonjs.com/features/featuresDeepDive/environment/reflectionProbes
             */
            reflectionProbes: Array<ReflectionProbe>;
            /**
             * Removes the given reflection probe from this scene.
             * @param toRemove The reflection probe to remove
             * @returns The index of the removed reflection probe
             */
            removeReflectionProbe(toRemove: ReflectionProbe): number;
            /**
             * Adds the given reflection probe to this scene.
             * @param newReflectionProbe The reflection probe to add
             */
            addReflectionProbe(newReflectionProbe: ReflectionProbe): void;
        }
    /**
     * Class used to generate realtime reflection / refraction cube textures
     * @see https://doc.babylonjs.com/features/featuresDeepDive/environment/reflectionProbes
     */
    export class ReflectionProbe {
        /** defines the name of the probe */
        name: string;
        private _scene;
        private _renderTargetTexture;
        private _projectionMatrix;
        private _viewMatrix;
        private _target;
        private _add;
        private _attachedMesh;
        private _invertYAxis;
        private _sceneUBOs;
        private _currentSceneUBO;
        /** Gets or sets probe position (center of the cube map) */
        position: Vector3;
        /**
         * Gets or sets an object used to store user defined information for the reflection probe.
         */
        metadata: any;
        /** @internal */
        _parentContainer: Nullable<AbstractScene>;
        /**
         * Creates a new reflection probe
         * @param name defines the name of the probe
         * @param size defines the texture resolution (for each face)
         * @param scene defines the hosting scene
         * @param generateMipMaps defines if mip maps should be generated automatically (true by default)
         * @param useFloat defines if HDR data (float data) should be used to store colors (false by default)
         * @param linearSpace defines if the probe should be generated in linear space or not (false by default)
         */
        constructor(
        /** defines the name of the probe */
        name: string, size: number, scene: Scene, generateMipMaps?: boolean, useFloat?: boolean, linearSpace?: boolean);
        /** Gets or sets the number of samples to use for multi-sampling (0 by default). Required WebGL2 */
        get samples(): number;
        set samples(value: number);
        /** Gets or sets the refresh rate to use (on every frame by default) */
        get refreshRate(): number;
        set refreshRate(value: number);
        /**
         * Gets the hosting scene
         * @returns a Scene
         */
        getScene(): Scene;
        /** Gets the internal CubeTexture used to render to */
        get cubeTexture(): RenderTargetTexture;
        /** Gets the list of meshes to render */
        get renderList(): Nullable<AbstractMesh[]>;
        /**
         * Attach the probe to a specific mesh (Rendering will be done from attached mesh's position)
         * @param mesh defines the mesh to attach to
         */
        attachToMesh(mesh: Nullable<AbstractMesh>): void;
        /**
         * Specifies whether or not the stencil and depth buffer are cleared between two rendering groups
         * @param renderingGroupId The rendering group id corresponding to its index
         * @param autoClearDepthStencil Automatically clears depth and stencil between groups if true.
         */
        setRenderingAutoClearDepthStencil(renderingGroupId: number, autoClearDepthStencil: boolean): void;
        /**
         * Clean all associated resources
         */
        dispose(): void;
        /**
         * Converts the reflection probe information to a readable string for debug purpose.
         * @param fullDetails Supports for multiple levels of logging within scene loading
         * @returns the human readable reflection probe info
         */
        toString(fullDetails?: boolean): string;
        /**
         * Get the class name of the refection probe.
         * @returns "ReflectionProbe"
         */
        getClassName(): string;
        /**
         * Serialize the reflection probe to a JSON representation we can easily use in the respective Parse function.
         * @returns The JSON representation of the texture
         */
        serialize(): any;
        /**
         * Parse the JSON representation of a reflection probe in order to recreate the reflection probe in the given scene.
         * @param parsedReflectionProbe Define the JSON representation of the reflection probe
         * @param scene Define the scene the parsed reflection probe should be instantiated in
         * @param rootUrl Define the root url of the parsing sequence in the case of relative dependencies
         * @returns The parsed reflection probe if successful
         */
        static Parse(parsedReflectionProbe: any, scene: Scene, rootUrl: string): Nullable<ReflectionProbe>;
    }


        interface Scene {
            /** @internal (Backing field) */
            _boundingBoxRenderer: BoundingBoxRenderer;
            /** @internal (Backing field) */
            _forceShowBoundingBoxes: boolean;
            /**
             * Gets or sets a boolean indicating if all bounding boxes must be rendered
             */
            forceShowBoundingBoxes: boolean;
            /**
             * Gets the bounding box renderer associated with the scene
             * @returns a BoundingBoxRenderer
             */
            getBoundingBoxRenderer(): BoundingBoxRenderer;
        }
        interface AbstractMesh {
            /** @internal (Backing field) */
            _showBoundingBox: boolean;
            /**
             * Gets or sets a boolean indicating if the bounding box must be rendered as well (false by default)
             */
            showBoundingBox: boolean;
        }
    /**
     * Component responsible of rendering the bounding box of the meshes in a scene.
     * This is usually used through the mesh.showBoundingBox or the scene.forceShowBoundingBoxes properties
     */
    export class BoundingBoxRenderer implements ISceneComponent {
        /**
         * The component name helpful to identify the component in the list of scene components.
         */
        readonly name = "BoundingBoxRenderer";
        /**
         * The scene the component belongs to.
         */
        scene: Scene;
        /**
         * Color of the bounding box lines placed in front of an object
         */
        frontColor: Color3;
        /**
         * Color of the bounding box lines placed behind an object
         */
        backColor: Color3;
        /**
         * Defines if the renderer should show the back lines or not
         */
        showBackLines: boolean;
        /**
         * Observable raised before rendering a bounding box
         */
        onBeforeBoxRenderingObservable: Observable<BoundingBox>;
        /**
         * Observable raised after rendering a bounding box
         */
        onAfterBoxRenderingObservable: Observable<BoundingBox>;
        /**
         * Observable raised after resources are created
         */
        onResourcesReadyObservable: Observable<BoundingBoxRenderer>;
        /**
         * When false, no bounding boxes will be rendered
         */
        enabled: boolean;
        /**
         * @internal
         */
        renderList: SmartArray<BoundingBox>;
        private _colorShader;
        private _colorShaderForOcclusionQuery;
        private _vertexBuffers;
        private _indexBuffer;
        private _fillIndexBuffer;
        private _fillIndexData;
        private _uniformBufferFront;
        private _uniformBufferBack;
        private _renderPassIdForOcclusionQuery;
        /**
         * Instantiates a new bounding box renderer in a scene.
         * @param scene the scene the  renderer renders in
         */
        constructor(scene: Scene);
        private _buildUniformLayout;
        /**
         * Registers the component in a given scene
         */
        register(): void;
        private _evaluateSubMesh;
        private _preActiveMesh;
        private _prepareResources;
        private _createIndexBuffer;
        /**
         * Rebuilds the elements related to this component in case of
         * context lost for instance.
         */
        rebuild(): void;
        /**
         * @internal
         */
        reset(): void;
        /**
         * Render the bounding boxes of a specific rendering group
         * @param renderingGroupId defines the rendering group to render
         */
        render(renderingGroupId: number): void;
        private _createWrappersForBoundingBox;
        /**
         * In case of occlusion queries, we can render the occlusion bounding box through this method
         * @param mesh Define the mesh to render the occlusion bounding box for
         */
        renderOcclusionBoundingBox(mesh: AbstractMesh): void;
        /**
         * Dispose and release the resources attached to this renderer.
         */
        dispose(): void;
    }


    /**
     * The depth peeling renderer that performs
     * Order independant transparency (OIT).
     * This should not be instanciated directly, as it is part of a scene component
     */
    export class DepthPeelingRenderer {
        private _scene;
        private _engine;
        private _depthMrts;
        private _thinTextures;
        private _colorMrts;
        private _blendBackMrt;
        private _outputRT;
        private _blendBackEffectWrapper;
        private _blendBackEffectWrapperPingPong;
        private _finalEffectWrapper;
        private _effectRenderer;
        private _currentPingPongState;
        private _prePassEffectConfiguration;
        private _blendBackTexture;
        private _layoutCacheFormat;
        private _layoutCache;
        private _renderPassIds;
        private _candidateSubMeshes;
        private _excludedSubMeshes;
        private _excludedMeshes;
        private static _DEPTH_CLEAR_VALUE;
        private static _MIN_DEPTH;
        private static _MAX_DEPTH;
        private _colorCache;
        private _passCount;
        /**
         * Number of depth peeling passes. As we are using dual depth peeling, each pass two levels of transparency are processed.
         */
        get passCount(): number;
        set passCount(count: number);
        private _useRenderPasses;
        /**
         * Instructs the renderer to use render passes. It is an optimization that makes the rendering faster for some engines (like WebGPU) but that consumes more memory, so it is disabled by default.
         */
        get useRenderPasses(): boolean;
        set useRenderPasses(usePasses: boolean);
        /**
         * Add a mesh in the exclusion list to prevent it to be handled by the depth peeling renderer
         * @param mesh The mesh to exclude from the depth peeling renderer
         */
        addExcludedMesh(mesh: AbstractMesh): void;
        /**
         * Remove a mesh from the exclusion list of the depth peeling renderer
         * @param mesh The mesh to remove
         */
        removeExcludedMesh(mesh: AbstractMesh): void;
        /**
         * Instanciates the depth peeling renderer
         * @param scene Scene to attach to
         * @param passCount Number of depth layers to peel
         * @returns The depth peeling renderer
         */
        constructor(scene: Scene, passCount?: number);
        private _createRenderPassIds;
        private _releaseRenderPassIds;
        private _createTextures;
        private _disposeTextures;
        private _updateTextures;
        private _updateTextureReferences;
        private _createEffects;
        /**
         * Links to the prepass renderer
         * @param prePassRenderer The scene PrePassRenderer
         */
        setPrePassRenderer(prePassRenderer: PrePassRenderer): void;
        /**
         * Binds depth peeling textures on an effect
         * @param effect The effect to bind textures on
         */
        bind(effect: Effect): void;
        private _renderSubMeshes;
        private _finalCompose;
        /**
         * Renders transparent submeshes with depth peeling
         * @param transparentSubMeshes List of transparent meshes to render
         * @returns The array of submeshes that could not be handled by this renderer
         */
        render(transparentSubMeshes: SmartArray<SubMesh>): SmartArray<SubMesh>;
        /**
         * Disposes the depth peeling renderer and associated ressources
         */
        dispose(): void;
    }


        interface Scene {
            /**
             * The depth peeling renderer
             */
            depthPeelingRenderer: Nullable<DepthPeelingRenderer>;
            /** @internal (Backing field) */
            _depthPeelingRenderer: Nullable<DepthPeelingRenderer>;
            /**
             * Flag to indicate if we want to use order independent transparency, despite the performance hit
             */
            useOrderIndependentTransparency: boolean;
            /** @internal */
            _useOrderIndependentTransparency: boolean;
        }
    /**
     * Scene component to render order independent transparency with depth peeling
     */
    export class DepthPeelingSceneComponent implements ISceneComponent {
        /**
         * The component name helpful to identify the component in the list of scene components.
         */
        readonly name = "DepthPeelingRenderer";
        /**
         * The scene the component belongs to.
         */
        scene: Scene;
        /**
         * Creates a new instance of the component for the given scene
         * @param scene Defines the scene to register the component in
         */
        constructor(scene: Scene);
        /**
         * Registers the component in a given scene
         */
        register(): void;
        /**
         * Rebuilds the elements related to this component in case of
         * context lost for instance.
         */
        rebuild(): void;
        /**
         * Disposes the component and the associated resources.
         */
        dispose(): void;
    }


    /**
     * This represents a depth renderer in Babylon.
     * A depth renderer will render to it's depth map every frame which can be displayed or used in post processing
     */
    export class DepthRenderer {
        private _scene;
        private _depthMap;
        private readonly _storeNonLinearDepth;
        private readonly _clearColor;
        /** Get if the depth renderer is using packed depth or not */
        readonly isPacked: boolean;
        private _camera;
        /** Enable or disable the depth renderer. When disabled, the depth texture is not updated */
        enabled: boolean;
        /** Force writing the transparent objects into the depth map */
        forceDepthWriteTransparentMeshes: boolean;
        /**
         * Specifies that the depth renderer will only be used within
         * the camera it is created for.
         * This can help forcing its rendering during the camera processing.
         */
        useOnlyInActiveCamera: boolean;
        /**
         * @internal
         */
        static _SceneComponentInitialization: (scene: Scene) => void;
        /**
         * Sets a specific material to be used to render a mesh/a list of meshes by the depth renderer
         * @param mesh mesh or array of meshes
         * @param material material to use by the depth render when rendering the mesh(es). If undefined is passed, the specific material created by the depth renderer will be used.
         */
        setMaterialForRendering(mesh: AbstractMesh | AbstractMesh[], material?: Material): void;
        /**
         * Instantiates a depth renderer
         * @param scene The scene the renderer belongs to
         * @param type The texture type of the depth map (default: Engine.TEXTURETYPE_FLOAT)
         * @param camera The camera to be used to render the depth map (default: scene's active camera)
         * @param storeNonLinearDepth Defines whether the depth is stored linearly like in Babylon Shadows or directly like glFragCoord.z
         * @param samplingMode The sampling mode to be used with the render target (Linear, Nearest...)
         */
        constructor(scene: Scene, type?: number, camera?: Nullable<Camera>, storeNonLinearDepth?: boolean, samplingMode?: number);
        /**
         * Creates the depth rendering effect and checks if the effect is ready.
         * @param subMesh The submesh to be used to render the depth map of
         * @param useInstances If multiple world instances should be used
         * @returns if the depth renderer is ready to render the depth map
         */
        isReady(subMesh: SubMesh, useInstances: boolean): boolean;
        /**
         * Gets the texture which the depth map will be written to.
         * @returns The depth map texture
         */
        getDepthMap(): RenderTargetTexture;
        /**
         * Disposes of the depth renderer.
         */
        dispose(): void;
    }


        interface Scene {
            /** @internal (Backing field) */
            _depthRenderer: {
                [id: string]: DepthRenderer;
            };
            /**
             * Creates a depth renderer a given camera which contains a depth map which can be used for post processing.
             * @param camera The camera to create the depth renderer on (default: scene's active camera)
             * @param storeNonLinearDepth Defines whether the depth is stored linearly like in Babylon Shadows or directly like glFragCoord.z
             * @param force32bitsFloat Forces 32 bits float when supported (else 16 bits float is prioritized over 32 bits float if supported)
             * @param samplingMode The sampling mode to be used with the render target (Linear, Nearest...)
             * @returns the created depth renderer
             */
            enableDepthRenderer(camera?: Nullable<Camera>, storeNonLinearDepth?: boolean, force32bitsFloat?: boolean, samplingMode?: number): DepthRenderer;
            /**
             * Disables a depth renderer for a given camera
             * @param camera The camera to disable the depth renderer on (default: scene's active camera)
             */
            disableDepthRenderer(camera?: Nullable<Camera>): void;
        }
    /**
     * Defines the Depth Renderer scene component responsible to manage a depth buffer useful
     * in several rendering techniques.
     */
    export class DepthRendererSceneComponent implements ISceneComponent {
        /**
         * The component name helpful to identify the component in the list of scene components.
         */
        readonly name = "DepthRenderer";
        /**
         * The scene the component belongs to.
         */
        scene: Scene;
        /**
         * Creates a new instance of the component for the given scene
         * @param scene Defines the scene to register the component in
         */
        constructor(scene: Scene);
        /**
         * Registers the component in a given scene
         */
        register(): void;
        /**
         * Rebuilds the elements related to this component in case of
         * context lost for instance.
         */
        rebuild(): void;
        /**
         * Disposes the component and the associated resources
         */
        dispose(): void;
        private _gatherRenderTargets;
        private _gatherActiveCameraRenderTargets;
    }


        interface Scene {
            /** @internal */
            _edgeRenderLineShader: Nullable<ShaderMaterial>;
        }
        interface AbstractMesh {
            /**
             * Gets the edgesRenderer associated with the mesh
             */
            edgesRenderer: Nullable<EdgesRenderer>;
        }
        interface LinesMesh {
            /**
             * Enables the edge rendering mode on the mesh.
             * This mode makes the mesh edges visible
             * @param epsilon defines the maximal distance between two angles to detect a face
             * @param checkVerticesInsteadOfIndices indicates that we should check vertex list directly instead of faces
             * @returns the currentAbstractMesh
             * @see https://www.babylonjs-playground.com/#19O9TU#0
             */
            enableEdgesRendering(epsilon?: number, checkVerticesInsteadOfIndices?: boolean): AbstractMesh;
        }
        interface InstancedLinesMesh {
            /**
             * Enables the edge rendering mode on the mesh.
             * This mode makes the mesh edges visible
             * @param epsilon defines the maximal distance between two angles to detect a face
             * @param checkVerticesInsteadOfIndices indicates that we should check vertex list directly instead of faces
             * @returns the current InstancedLinesMesh
             * @see https://www.babylonjs-playground.com/#19O9TU#0
             */
            enableEdgesRendering(epsilon?: number, checkVerticesInsteadOfIndices?: boolean): InstancedLinesMesh;
        }
    /**
     * Defines the minimum contract an Edges renderer should follow.
     */
    export interface IEdgesRenderer extends IDisposable {
        /**
         * Gets or sets a boolean indicating if the edgesRenderer is active
         */
        isEnabled: boolean;
        /**
         * Renders the edges of the attached mesh,
         */
        render(): void;
        /**
         * Checks whether or not the edges renderer is ready to render.
         * @returns true if ready, otherwise false.
         */
        isReady(): boolean;
        /**
         * List of instances to render in case the source mesh has instances
         */
        customInstances: SmartArray<Matrix>;
    }
    /**
     * Defines the additional options of the edges renderer
     */
    export interface IEdgesRendererOptions {
        /**
         * Gets or sets a boolean indicating that the alternate edge finder algorithm must be used
         * If not defined, the default value is true
         */
        useAlternateEdgeFinder?: boolean;
        /**
         * Gets or sets a boolean indicating that the vertex merger fast processing must be used.
         * If not defined, the default value is true.
         * You should normally leave it undefined (or set it to true), except if you see some artifacts in the edges rendering (can happen with complex geometries)
         * This option is used only if useAlternateEdgeFinder = true
         */
        useFastVertexMerger?: boolean;
        /**
         * During edges processing, the vertices are merged if they are close enough: epsilonVertexMerge is the limit within which vertices are considered to be equal.
         * The default value is 1e-6
         * This option is used only if useAlternateEdgeFinder = true
         */
        epsilonVertexMerge?: number;
        /**
         * Gets or sets a boolean indicating that tessellation should be applied before finding the edges. You may need to activate this option if your geometry is a bit
         * unusual, like having a vertex of a triangle in-between two vertices of an edge of another triangle. It happens often when using CSG to construct meshes.
         * This option is used only if useAlternateEdgeFinder = true
         */
        applyTessellation?: boolean;
        /**
         * The limit under which 3 vertices are considered to be aligned. 3 vertices PQR are considered aligned if distance(PQ) + distance(QR) - distance(PR) < epsilonVertexAligned
         * The default value is 1e-6
         * This option is used only if useAlternateEdgeFinder = true
         */
        epsilonVertexAligned?: number;
        /**
         * Gets or sets a boolean indicating that degenerated triangles should not be processed.
         * Degenerated triangles are triangles that have 2 or 3 vertices with the same coordinates
         */
        removeDegeneratedTriangles?: boolean;
    }
    /**
     * This class is used to generate edges of the mesh that could then easily be rendered in a scene.
     */
    export class EdgesRenderer implements IEdgesRenderer {
        /**
         * Define the size of the edges with an orthographic camera
         */
        edgesWidthScalerForOrthographic: number;
        /**
         * Define the size of the edges with a perspective camera
         */
        edgesWidthScalerForPerspective: number;
        protected _source: AbstractMesh;
        protected _linesPositions: number[];
        protected _linesNormals: number[];
        protected _linesIndices: number[];
        protected _epsilon: number;
        protected _indicesCount: number;
        protected _drawWrapper?: DrawWrapper;
        protected _lineShader: ShaderMaterial;
        protected _ib: DataBuffer;
        protected _buffers: {
            [key: string]: Nullable<VertexBuffer>;
        };
        protected _buffersForInstances: {
            [key: string]: Nullable<VertexBuffer>;
        };
        protected _checkVerticesInsteadOfIndices: boolean;
        protected _options: Nullable<IEdgesRendererOptions>;
        private _meshRebuildObserver;
        private _meshDisposeObserver;
        /** Gets or sets a boolean indicating if the edgesRenderer is active */
        isEnabled: boolean;
        /** Gets the vertices generated by the edge renderer */
        get linesPositions(): Immutable<Array<number>>;
        /** Gets the normals generated by the edge renderer */
        get linesNormals(): Immutable<Array<number>>;
        /** Gets the indices generated by the edge renderer */
        get linesIndices(): Immutable<Array<number>>;
        /**
         * Gets or sets the shader used to draw the lines
         */
        get lineShader(): ShaderMaterial;
        set lineShader(shader: ShaderMaterial);
        /**
         * List of instances to render in case the source mesh has instances
         */
        customInstances: SmartArray<Matrix>;
        private static _GetShader;
        /**
         * Creates an instance of the EdgesRenderer. It is primarily use to display edges of a mesh.
         * Beware when you use this class with complex objects as the adjacencies computation can be really long
         * @param  source Mesh used to create edges
         * @param  epsilon sum of angles in adjacency to check for edge
         * @param  checkVerticesInsteadOfIndices bases the edges detection on vertices vs indices. Note that this parameter is not used if options.useAlternateEdgeFinder = true
         * @param  generateEdgesLines - should generate Lines or only prepare resources.
         * @param  options The options to apply when generating the edges
         */
        constructor(source: AbstractMesh, epsilon?: number, checkVerticesInsteadOfIndices?: boolean, generateEdgesLines?: boolean, options?: IEdgesRendererOptions);
        protected _prepareRessources(): void;
        /** @internal */
        _rebuild(): void;
        /**
         * Releases the required resources for the edges renderer
         */
        dispose(): void;
        protected _processEdgeForAdjacencies(pa: number, pb: number, p0: number, p1: number, p2: number): number;
        protected _processEdgeForAdjacenciesWithVertices(pa: Vector3, pb: Vector3, p0: Vector3, p1: Vector3, p2: Vector3): number;
        /**
         * Checks if the pair of p0 and p1 is en edge
         * @param faceIndex
         * @param edge
         * @param faceNormals
         * @param  p0
         * @param  p1
         * @private
         */
        protected _checkEdge(faceIndex: number, edge: number, faceNormals: Array<Vector3>, p0: Vector3, p1: Vector3): void;
        /**
         * push line into the position, normal and index buffer
         * @param p0
         * @param p1
         * @param offset
         * @protected
         */
        protected createLine(p0: Vector3, p1: Vector3, offset: number): void;
        /**
         * See https://playground.babylonjs.com/#R3JR6V#1 for a visual display of the algorithm
         * @param edgePoints
         * @param indexTriangle
         * @param indices
         * @param remapVertexIndices
         */
        private _tessellateTriangle;
        private _generateEdgesLinesAlternate;
        /**
         * Generates lines edges from adjacencjes
         * @private
         */
        _generateEdgesLines(): void;
        /**
         * Checks whether or not the edges renderer is ready to render.
         * @returns true if ready, otherwise false.
         */
        isReady(): boolean;
        /**
         * Renders the edges of the attached mesh,
         */
        render(): void;
    }
    /**
     * LineEdgesRenderer for LineMeshes to remove unnecessary triangulation
     */
    export class LineEdgesRenderer extends EdgesRenderer {
        /**
         * This constructor turns off auto generating edges line in Edges Renderer to make it here.
         * @param  source LineMesh used to generate edges
         * @param  epsilon not important (specified angle for edge detection)
         * @param  checkVerticesInsteadOfIndices not important for LineMesh
         */
        constructor(source: AbstractMesh, epsilon?: number, checkVerticesInsteadOfIndices?: boolean);
        /**
         * Generate edges for each line in LinesMesh. Every Line should be rendered as edge.
         */
        _generateEdgesLines(): void;
    }


        interface AbstractScene {
            /** @internal (Backing field) */
            _fluidRenderer: Nullable<FluidRenderer>;
            /**
             * Gets or Sets the fluid renderer associated to the scene.
             */
            fluidRenderer: Nullable<FluidRenderer>;
            /**
             * Enables the fluid renderer and associates it with the scene
             * @returns the FluidRenderer
             */
            enableFluidRenderer(): Nullable<FluidRenderer>;
            /**
             * Disables the fluid renderer associated with the scene
             */
            disableFluidRenderer(): void;
        }
    /**
     * Defines the fluid renderer scene component responsible to render objects as fluids
     */
    export class FluidRendererSceneComponent implements ISceneComponent {
        /**
         * The component name helpful to identify the component in the list of scene components.
         */
        readonly name = "FluidRenderer";
        /**
         * The scene the component belongs to.
         */
        scene: Scene;
        /**
         * Creates a new instance of the component for the given scene
         * @param scene Defines the scene to register the component in
         */
        constructor(scene: Scene);
        /**
         * Registers the component in a given scene
         */
        register(): void;
        private _gatherActiveCameraRenderTargets;
        private _afterCameraDraw;
        /**
         * Rebuilds the elements related to this component in case of
         * context lost for instance.
         */
        rebuild(): void;
        /**
         * Disposes the component and the associated resources
         */
        dispose(): void;
    }
    /**
     * An object rendered as a fluid.
     * It consists of the object itself as well as the render target renderer (which is used to generate the textures (render target) needed for fluid rendering)
     */
    export interface IFluidRenderingRenderObject {
        /** object rendered as a fluid */
        object: FluidRenderingObject;
        /** target renderer used to render the fluid object */
        targetRenderer: FluidRenderingTargetRenderer;
    }
    /**
     * Class responsible for fluid rendering.
     * It is implementing the method described in https://developer.download.nvidia.com/presentations/2010/gdc/Direct3D_Effects.pdf
     */
    export class FluidRenderer {
        /** @internal */
        static _SceneComponentInitialization(scene: Scene): void;
        private _scene;
        private _engine;
        private _onEngineResizeObserver;
        private _cameras;
        /** Retrieves all the render objects managed by the class */
        readonly renderObjects: Array<IFluidRenderingRenderObject>;
        /** Retrieves all the render target renderers managed by the class */
        readonly targetRenderers: FluidRenderingTargetRenderer[];
        /**
         * Initializes the class
         * @param scene Scene in which the objects are part of
         */
        constructor(scene: Scene);
        /**
         * Reinitializes the class
         * Can be used if you change the object priority (FluidRenderingObject.priority), to make sure the objects are rendered in the right order
         */
        recreate(): void;
        /**
         * Gets the render object corresponding to a particle system (null if the particle system is not rendered as a fluid)
         * @param ps The particle system
         * @returns the render object corresponding to this particle system if any, otherwise null
         */
        getRenderObjectFromParticleSystem(ps: IParticleSystem): Nullable<IFluidRenderingRenderObject>;
        /**
         * Adds a particle system to the fluid renderer.
         * Note that you should not normally call this method directly, as you can simply use the renderAsFluid property of the ParticleSystem/GPUParticleSystem class
         * @param ps particle system
         * @param generateDiffuseTexture True if you want to generate a diffuse texture from the particle system and use it as part of the fluid rendering (default: false)
         * @param targetRenderer The target renderer used to display the particle system as a fluid. If not provided, the method will create a new one
         * @param camera The camera used by the target renderer (if the target renderer is created by the method)
         * @returns the render object corresponding to the particle system
         */
        addParticleSystem(ps: IParticleSystem, generateDiffuseTexture?: boolean, targetRenderer?: FluidRenderingTargetRenderer, camera?: Camera): IFluidRenderingRenderObject;
        /**
         * Adds a custom particle set to the fluid renderer.
         * @param buffers The list of buffers (should contain at least a "position" buffer!)
         * @param numParticles Number of particles in each buffer
         * @param generateDiffuseTexture True if you want to generate a diffuse texture from buffers and use it as part of the fluid rendering (default: false). For the texture to be generated correctly, you need a "color" buffer in the set!
         * @param targetRenderer The target renderer used to display the particle system as a fluid. If not provided, the method will create a new one
         * @param camera The camera used by the target renderer (if the target renderer is created by the method)
         * @returns the render object corresponding to the custom particle set
         */
        addCustomParticles(buffers: {
            [key: string]: FloatArray;
        }, numParticles: number, generateDiffuseTexture?: boolean, targetRenderer?: FluidRenderingTargetRenderer, camera?: Camera): IFluidRenderingRenderObject;
        /**
         * Removes a render object from the fluid renderer
         * @param renderObject the render object to remove
         * @param removeUnusedTargetRenderer True to remove/dispose of the target renderer if it's not used anymore (default: true)
         * @returns True if the render object has been found and released, else false
         */
        removeRenderObject(renderObject: IFluidRenderingRenderObject, removeUnusedTargetRenderer?: boolean): boolean;
        private _sortRenderingObjects;
        private _removeUnusedTargetRenderers;
        private _getParticleSystemIndex;
        private _initialize;
        private _setParticleSizeForRenderTargets;
        private _setUseVelocityForRenderObject;
        /** @internal */
        _prepareRendering(): void;
        /** @internal */
        _render(forCamera?: Camera): void;
        /**
         * Disposes of all the ressources used by the class
         */
        dispose(): void;
    }


    /** @internal */
    export class FluidRenderingDepthTextureCopy {
        private _engine;
        private _depthRTWrapper;
        private _copyTextureToTexture;
        get depthRTWrapper(): RenderTargetWrapper;
        constructor(engine: Engine, width: number, height: number, samples?: number);
        copy(source: InternalTexture): boolean;
        dispose(): void;
    }


    /**
     * Defines the base object used for fluid rendering.
     * It is based on a list of vertices (particles)
     */
    export abstract class FluidRenderingObject {
        protected _scene: Scene;
        protected _engine: Engine;
        protected _effectsAreDirty: boolean;
        protected _depthEffectWrapper: Nullable<EffectWrapper>;
        protected _thicknessEffectWrapper: Nullable<EffectWrapper>;
        /** Defines the priority of the object. Objects will be rendered in ascending order of priority */
        priority: number;
        protected _particleSize: number;
        /** Observable triggered when the size of the particle is changed */
        onParticleSizeChanged: Observable<FluidRenderingObject>;
        /** Gets or sets the size of the particle */
        get particleSize(): number;
        set particleSize(size: number);
        /** Defines the alpha value of a particle */
        particleThicknessAlpha: number;
        /** Indicates if the object uses instancing or not */
        get useInstancing(): boolean;
        private _useVelocity;
        /** Indicates if velocity of particles should be used when rendering the object. The vertex buffer set must contain a "velocity" buffer for this to work! */
        get useVelocity(): boolean;
        set useVelocity(use: boolean);
        private _hasVelocity;
        /**
         * Gets the vertex buffers
         */
        abstract get vertexBuffers(): {
            [key: string]: VertexBuffer;
        };
        /**
         * Gets the index buffer (or null if the object is using instancing)
         */
        get indexBuffer(): Nullable<DataBuffer>;
        /**
         * Gets the name of the class
         */
        getClassName(): string;
        /**
         * Instantiates a fluid rendering object
         * @param scene The scene the object is part of
         */
        constructor(scene: Scene);
        protected _createEffects(): void;
        /**
         * Indicates if the object is ready to be rendered
         * @returns True if everything is ready for the object to be rendered, otherwise false
         */
        isReady(): boolean;
        /**
         * Gets the number of particles (vertices) of this object
         * @returns The number of particles
         */
        abstract get numParticles(): number;
        /**
         * Render the depth texture for this object
         */
        renderDepthTexture(): void;
        /**
         * Render the thickness texture for this object
         */
        renderThicknessTexture(): void;
        /**
         * Render the diffuse texture for this object
         */
        renderDiffuseTexture(): void;
        /**
         * Releases the ressources used by the class
         */
        dispose(): void;
    }


    /**
     * Defines a rendering object based on a list of custom buffers
     * The list must contain at least a "position" buffer!
     */
    export class FluidRenderingObjectCustomParticles extends FluidRenderingObject {
        private _numParticles;
        private _diffuseEffectWrapper;
        private _vertexBuffers;
        /**
         * Gets the name of the class
         */
        getClassName(): string;
        /**
         * Gets the vertex buffers
         */
        get vertexBuffers(): {
            [key: string]: VertexBuffer;
        };
        /**
         * Creates a new instance of the class
         * @param scene The scene the particles should be rendered into
         * @param buffers The list of buffers (must contain at least one "position" buffer!). Note that you don't have to pass all (or any!) buffers at once in the constructor, you can use the addBuffers method to add more later.
         * @param numParticles Number of vertices to take into account from the buffers
         */
        constructor(scene: Scene, buffers: {
            [key: string]: FloatArray;
        }, numParticles: number);
        /**
         * Add some new buffers
         * @param buffers List of buffers
         */
        addBuffers(buffers: {
            [key: string]: FloatArray;
        }): void;
        protected _createEffects(): void;
        /**
         * Indicates if the object is ready to be rendered
         * @returns True if everything is ready for the object to be rendered, otherwise false
         */
        isReady(): boolean;
        /**
         * Gets the number of particles in this object
         * @returns The number of particles
         */
        get numParticles(): number;
        /**
         * Sets the number of particles in this object
         * @param num The number of particles to take into account
         */
        setNumParticles(num: number): void;
        /**
         * Render the diffuse texture for this object
         */
        renderDiffuseTexture(): void;
        /**
         * Releases the ressources used by the class
         */
        dispose(): void;
    }


    /**
     * Defines a rendering object based on a particle system
     */
    export class FluidRenderingObjectParticleSystem extends FluidRenderingObject {
        private _particleSystem;
        private _originalRender;
        private _blendMode;
        private _onBeforeDrawParticleObserver;
        private _updateInAnimate;
        /** Gets the particle system */
        get particleSystem(): IParticleSystem;
        /**
         * Gets the name of the class
         */
        getClassName(): string;
        private _useTrueRenderingForDiffuseTexture;
        /**
         * Gets or sets a boolean indicating that the diffuse texture should be generated based on the regular rendering of the particle system (default: true).
         * Sometimes, generating the diffuse texture this way may be sub-optimal. In that case, you can disable this property, in which case the particle system will be
         * rendered using a ALPHA_COMBINE mode instead of the one used by the particle system.
         */
        get useTrueRenderingForDiffuseTexture(): boolean;
        set useTrueRenderingForDiffuseTexture(use: boolean);
        /**
         * Gets the vertex buffers
         */
        get vertexBuffers(): {
            [key: string]: VertexBuffer;
        };
        /**
         * Gets the index buffer (or null if the object is using instancing)
         */
        get indexBuffer(): Nullable<DataBuffer>;
        /**
         * Creates a new instance of the class
         * @param scene The scene the particle system is part of
         * @param ps The particle system
         */
        constructor(scene: Scene, ps: IParticleSystem);
        /**
         * Indicates if the object is ready to be rendered
         * @returns True if everything is ready for the object to be rendered, otherwise false
         */
        isReady(): boolean;
        /**
         * Gets the number of particles in this particle system
         * @returns The number of particles
         */
        get numParticles(): number;
        /**
         * Render the diffuse texture for this object
         */
        renderDiffuseTexture(): void;
        /**
         * Releases the ressources used by the class
         */
        dispose(): void;
    }


    /**
     * Textures that can be displayed as a debugging tool
     */
    export enum FluidRenderingDebug {
        DepthTexture = 0,
        DepthBlurredTexture = 1,
        ThicknessTexture = 2,
        ThicknessBlurredTexture = 3,
        DiffuseTexture = 4,
        Normals = 5,
        DiffuseRendering = 6
    }
    /**
     * Class used to render an object as a fluid thanks to different render target textures (depth, thickness, diffuse)
     */
    export class FluidRenderingTargetRenderer {
        protected _scene: Scene;
        protected _camera: Nullable<Camera>;
        protected _engine: Engine;
        protected _invProjectionMatrix: Matrix;
        protected _depthClearColor: Color4;
        protected _thicknessClearColor: Color4;
        protected _needInitialization: boolean;
        /**
         * Returns true if the class needs to be reinitialized (because of changes in parameterization)
         */
        get needInitialization(): boolean;
        private _generateDiffuseTexture;
        /**
         * Gets or sets a boolean indicating that the diffuse texture should be generated and used for the rendering
         */
        get generateDiffuseTexture(): boolean;
        set generateDiffuseTexture(generate: boolean);
        /**
         * Fluid color. Not used if generateDiffuseTexture is true
         */
        fluidColor: Color3;
        /**
         * Density of the fluid (positive number). The higher the value, the more opaque the fluid.
         */
        density: number;
        /**
         * Strength of the refraction (positive number, but generally between 0 and 0.3).
         */
        refractionStrength: number;
        /**
         * Strength of the fresnel effect (value between 0 and 1). Lower the value if you want to soften the specular effect
         */
        fresnelClamp: number;
        /**
         * Strength of the specular power (positive number). Increase the value to make the specular effect more concentrated
         */
        specularPower: number;
        /**
         * Minimum thickness of the particles (positive number). If useFixedThickness is true, minimumThickness is the thickness used
         */
        minimumThickness: number;
        /**
         * Direction of the light. The fluid is assumed to be lit by a directional light
         */
        dirLight: Vector3;
        private _debugFeature;
        /**
         * Gets or sets the feature (texture) to be debugged. Not used if debug is false
         */
        get debugFeature(): FluidRenderingDebug;
        set debugFeature(feature: FluidRenderingDebug);
        private _debug;
        /**
         * Gets or sets a boolean indicating if we should display a specific texture (given by debugFeature) for debugging purpose
         */
        get debug(): boolean;
        set debug(debug: boolean);
        private _environmentMap?;
        /**
         * Gets or sets the environment map used for the reflection part of the shading
         * If null, no map will be used. If undefined, the scene.environmentMap will be used (if defined)
         */
        get environmentMap(): Nullable<BaseTexture> | undefined;
        set environmentMap(map: Nullable<BaseTexture> | undefined);
        private _enableBlurDepth;
        /**
         * Gets or sets a boolean indicating that the depth texture should be blurred
         */
        get enableBlurDepth(): boolean;
        set enableBlurDepth(enable: boolean);
        private _blurDepthSizeDivisor;
        /**
         * Gets or sets the depth size divisor (positive number, generally between 1 and 4), which is used as a divisor when creating the texture used for blurring the depth
         * For eg. if blurDepthSizeDivisor=2, the texture used to blur the depth will be half the size of the depth texture
         */
        get blurDepthSizeDivisor(): number;
        set blurDepthSizeDivisor(scale: number);
        private _blurDepthFilterSize;
        /**
         * Size of the kernel used to filter the depth blur texture (positive number, generally between 1 and 20 - higher values will require more processing power from the GPU)
         */
        get blurDepthFilterSize(): number;
        set blurDepthFilterSize(filterSize: number);
        private _blurDepthNumIterations;
        /**
         * Number of blurring iterations used to generate the depth blur texture (positive number, generally between 1 and 10 - higher values will require more processing power from the GPU)
         */
        get blurDepthNumIterations(): number;
        set blurDepthNumIterations(numIterations: number);
        private _blurDepthMaxFilterSize;
        /**
         * Maximum size of the kernel used to blur the depth texture (positive number, generally between 1 and 200 - higher values will require more processing power from the GPU when the particles are larger on screen)
         */
        get blurDepthMaxFilterSize(): number;
        set blurDepthMaxFilterSize(maxFilterSize: number);
        private _blurDepthDepthScale;
        /**
         * Depth weight in the calculation when applying the bilateral blur to generate the depth blur texture (positive number, generally between 0 and 100)
         */
        get blurDepthDepthScale(): number;
        set blurDepthDepthScale(scale: number);
        private _enableBlurThickness;
        /**
         * Gets or sets a boolean indicating that the thickness texture should be blurred
         */
        get enableBlurThickness(): boolean;
        set enableBlurThickness(enable: boolean);
        private _blurThicknessSizeDivisor;
        /**
         * Gets or sets the thickness size divisor (positive number, generally between 1 and 4), which is used as a divisor when creating the texture used for blurring the thickness
         * For eg. if blurThicknessSizeDivisor=2, the texture used to blur the thickness will be half the size of the thickness texture
         */
        get blurThicknessSizeDivisor(): number;
        set blurThicknessSizeDivisor(scale: number);
        private _blurThicknessFilterSize;
        /**
         * Size of the kernel used to filter the thickness blur texture (positive number, generally between 1 and 20 - higher values will require more processing power from the GPU)
         */
        get blurThicknessFilterSize(): number;
        set blurThicknessFilterSize(filterSize: number);
        private _blurThicknessNumIterations;
        /**
         * Number of blurring iterations used to generate the thickness blur texture (positive number, generally between 1 and 10 - higher values will require more processing power from the GPU)
         */
        get blurThicknessNumIterations(): number;
        set blurThicknessNumIterations(numIterations: number);
        private _useFixedThickness;
        /**
         * Gets or sets a boolean indicating that a fixed thickness should be used instead of generating a thickness texture
         */
        get useFixedThickness(): boolean;
        set useFixedThickness(use: boolean);
        /** @internal */
        _bgDepthTexture: Nullable<InternalTexture>;
        /** @internal */
        _onUseVelocityChanged: Observable<FluidRenderingTargetRenderer>;
        private _useVelocity;
        /**
         * Gets or sets a boolean indicating that the velocity should be used when rendering the particles as a fluid.
         * Note: the vertex buffers must contain a "velocity" buffer for this to work!
         */
        get useVelocity(): boolean;
        set useVelocity(use: boolean);
        private _depthMapSize;
        /**
         * Defines the size of the depth texture.
         * If null, the texture will have the size of the screen
         */
        get depthMapSize(): Nullable<number>;
        set depthMapSize(size: Nullable<number>);
        private _thicknessMapSize;
        /**
         * Defines the size of the thickness texture.
         * If null, the texture will have the size of the screen
         */
        get thicknessMapSize(): Nullable<number>;
        set thicknessMapSize(size: Nullable<number>);
        private _diffuseMapSize;
        /**
         * Defines the size of the diffuse texture.
         * If null, the texture will have the size of the screen
         */
        get diffuseMapSize(): Nullable<number>;
        set diffuseMapSize(size: Nullable<number>);
        private _samples;
        /**
         * Gets or sets the number of samples used by MSAA
         * Note: changing this value in WebGL does not work because depth/stencil textures can't be created with MSAA (see https://github.com/BabylonJS/Babylon.js/issues/12444)
         */
        get samples(): number;
        set samples(samples: number);
        /**
         * Gets the camera used for the rendering
         */
        get camera(): Nullable<Camera>;
        /** @internal */
        _renderPostProcess: Nullable<PostProcess>;
        /** @internal */
        _depthRenderTarget: Nullable<FluidRenderingTextures>;
        /** @internal */
        _diffuseRenderTarget: Nullable<FluidRenderingTextures>;
        /** @internal */
        _thicknessRenderTarget: Nullable<FluidRenderingTextures>;
        /**
         * Creates an instance of the class
         * @param scene Scene used to render the fluid object into
         * @param camera Camera used to render the fluid object. If not provided, use the active camera of the scene instead
         */
        constructor(scene: Scene, camera?: Camera);
        /** @internal */
        _initialize(): void;
        protected _setBlurParameters(renderTarget?: Nullable<FluidRenderingTextures>): void;
        protected _setBlurDepthParameters(): void;
        protected _setBlurThicknessParameters(): void;
        protected _initializeRenderTarget(renderTarget: FluidRenderingTextures): void;
        protected _createLiquidRenderingPostProcess(): void;
        /** @internal */
        _clearTargets(): void;
        /** @internal */
        _render(fluidObject: FluidRenderingObject): void;
        /**
         * Releases all the ressources used by the class
         * @param onlyPostProcesses If true, releases only the ressources used by the render post processes
         */
        dispose(onlyPostProcesses?: boolean): void;
    }


    /** @internal */
    export class FluidRenderingTextures {
        protected _name: string;
        protected _scene: Scene;
        protected _camera: Nullable<Camera>;
        protected _engine: Engine;
        protected _width: number;
        protected _height: number;
        protected _blurTextureSizeX: number;
        protected _blurTextureSizeY: number;
        protected _textureType: number;
        protected _textureFormat: number;
        protected _blurTextureType: number;
        protected _blurTextureFormat: number;
        protected _useStandardBlur: boolean;
        protected _generateDepthBuffer: boolean;
        protected _samples: number;
        protected _postProcessRunningIndex: number;
        protected _rt: Nullable<RenderTargetWrapper>;
        protected _texture: Nullable<Texture>;
        protected _rtBlur: Nullable<RenderTargetWrapper>;
        protected _textureBlurred: Nullable<Texture>;
        protected _blurPostProcesses: Nullable<PostProcess[]>;
        enableBlur: boolean;
        blurSizeDivisor: number;
        blurFilterSize: number;
        private _blurNumIterations;
        get blurNumIterations(): number;
        set blurNumIterations(numIterations: number);
        blurMaxFilterSize: number;
        blurDepthScale: number;
        particleSize: number;
        onDisposeObservable: Observable<FluidRenderingTextures>;
        get renderTarget(): Nullable<RenderTargetWrapper>;
        get renderTargetBlur(): Nullable<RenderTargetWrapper>;
        get texture(): Nullable<Texture>;
        get textureBlur(): Nullable<Texture>;
        constructor(name: string, scene: Scene, width: number, height: number, blurTextureSizeX: number, blurTextureSizeY: number, textureType?: number, textureFormat?: number, blurTextureType?: number, blurTextureFormat?: number, useStandardBlur?: boolean, camera?: Nullable<Camera>, generateDepthBuffer?: boolean, samples?: number);
        initialize(): void;
        applyBlurPostProcesses(): void;
        protected _createRenderTarget(): void;
        protected _createBlurPostProcesses(textureBlurSource: ThinTexture, textureType: number, textureFormat: number, blurSizeDivisor: number, debugName: string, useStandardBlur?: boolean): [RenderTargetWrapper, Texture, PostProcess[]];
        private _fixReusablePostProcess;
        private _getProjectedParticleConstant;
        private _getDepthThreshold;
        dispose(): void;
    }




    /** @internal */
    interface ISavedTransformationMatrix {
        world: Matrix;
        viewProjection: Matrix;
    }
    /**
     * This renderer is helpful to fill one of the render target with a geometry buffer.
     */
    export class GeometryBufferRenderer {
        /**
         * Constant used to retrieve the depth texture index in the G-Buffer textures array
         * using getIndex(GeometryBufferRenderer.DEPTH_TEXTURE_INDEX)
         */
        static readonly DEPTH_TEXTURE_TYPE = 0;
        /**
         * Constant used to retrieve the normal texture index in the G-Buffer textures array
         * using getIndex(GeometryBufferRenderer.NORMAL_TEXTURE_INDEX)
         */
        static readonly NORMAL_TEXTURE_TYPE = 1;
        /**
         * Constant used to retrieve the position texture index in the G-Buffer textures array
         * using getIndex(GeometryBufferRenderer.POSITION_TEXTURE_INDEX)
         */
        static readonly POSITION_TEXTURE_TYPE = 2;
        /**
         * Constant used to retrieve the velocity texture index in the G-Buffer textures array
         * using getIndex(GeometryBufferRenderer.VELOCITY_TEXTURE_INDEX)
         */
        static readonly VELOCITY_TEXTURE_TYPE = 3;
        /**
         * Constant used to retrieve the reflectivity texture index in the G-Buffer textures array
         * using the getIndex(GeometryBufferRenderer.REFLECTIVITY_TEXTURE_TYPE)
         */
        static readonly REFLECTIVITY_TEXTURE_TYPE = 4;
        /**
         * Dictionary used to store the previous transformation matrices of each rendered mesh
         * in order to compute objects velocities when enableVelocity is set to "true"
         * @internal
         */
        _previousTransformationMatrices: {
            [index: number]: ISavedTransformationMatrix;
        };
        /**
         * Dictionary used to store the previous bones transformation matrices of each rendered mesh
         * in order to compute objects velocities when enableVelocity is set to "true"
         * @internal
         */
        _previousBonesTransformationMatrices: {
            [index: number]: Float32Array;
        };
        /**
         * Array used to store the ignored skinned meshes while computing velocity map (typically used by the motion blur post-process).
         * Avoids computing bones velocities and computes only mesh's velocity itself (position, rotation, scaling).
         */
        excludedSkinnedMeshesFromVelocity: AbstractMesh[];
        /** Gets or sets a boolean indicating if transparent meshes should be rendered */
        renderTransparentMeshes: boolean;
        private _scene;
        private _resizeObserver;
        private _multiRenderTarget;
        private _ratio;
        private _enablePosition;
        private _enableVelocity;
        private _enableReflectivity;
        private _depthFormat;
        private _positionIndex;
        private _velocityIndex;
        private _reflectivityIndex;
        private _depthIndex;
        private _normalIndex;
        private _linkedWithPrePass;
        private _prePassRenderer;
        private _attachments;
        private _useUbo;
        protected _cachedDefines: string;
        /**
         * @internal
         * Sets up internal structures to share outputs with PrePassRenderer
         * This method should only be called by the PrePassRenderer itself
         */
        _linkPrePassRenderer(prePassRenderer: PrePassRenderer): void;
        /**
         * @internal
         * Separates internal structures from PrePassRenderer so the geometry buffer can now operate by itself.
         * This method should only be called by the PrePassRenderer itself
         */
        _unlinkPrePassRenderer(): void;
        /**
         * @internal
         * Resets the geometry buffer layout
         */
        _resetLayout(): void;
        /**
         * @internal
         * Replaces a texture in the geometry buffer renderer
         * Useful when linking textures of the prepass renderer
         */
        _forceTextureType(geometryBufferType: number, index: number): void;
        /**
         * @internal
         * Sets texture attachments
         * Useful when linking textures of the prepass renderer
         */
        _setAttachments(attachments: number[]): void;
        /**
         * @internal
         * Replaces the first texture which is hard coded as a depth texture in the geometry buffer
         * Useful when linking textures of the prepass renderer
         */
        _linkInternalTexture(internalTexture: InternalTexture): void;
        /**
         * Gets the render list (meshes to be rendered) used in the G buffer.
         */
        get renderList(): Nullable<AbstractMesh[]>;
        /**
         * Set the render list (meshes to be rendered) used in the G buffer.
         */
        set renderList(meshes: Nullable<AbstractMesh[]>);
        /**
         * Gets whether or not G buffer are supported by the running hardware.
         * This requires draw buffer supports
         */
        get isSupported(): boolean;
        /**
         * Returns the index of the given texture type in the G-Buffer textures array
         * @param textureType The texture type constant. For example GeometryBufferRenderer.POSITION_TEXTURE_INDEX
         * @returns the index of the given texture type in the G-Buffer textures array
         */
        getTextureIndex(textureType: number): number;
        /**
         * Gets a boolean indicating if objects positions are enabled for the G buffer.
         */
        get enablePosition(): boolean;
        /**
         * Sets whether or not objects positions are enabled for the G buffer.
         */
        set enablePosition(enable: boolean);
        /**
         * Gets a boolean indicating if objects velocities are enabled for the G buffer.
         */
        get enableVelocity(): boolean;
        /**
         * Sets whether or not objects velocities are enabled for the G buffer.
         */
        set enableVelocity(enable: boolean);
        /**
         * Gets a boolean indicating if objects reflectivity are enabled in the G buffer.
         */
        get enableReflectivity(): boolean;
        /**
         * Sets whether or not objects reflectivity are enabled for the G buffer.
         * For Metallic-Roughness workflow with ORM texture, we assume that ORM texture is defined according to the default layout:
         * pbr.useRoughnessFromMetallicTextureAlpha = false;
         * pbr.useRoughnessFromMetallicTextureGreen = true;
         * pbr.useMetallnessFromMetallicTextureBlue = true;
         */
        set enableReflectivity(enable: boolean);
        /**
         * Gets the scene associated with the buffer.
         */
        get scene(): Scene;
        /**
         * Gets the ratio used by the buffer during its creation.
         * How big is the buffer related to the main canvas.
         */
        get ratio(): number;
        /**
         * @internal
         */
        static _SceneComponentInitialization: (scene: Scene) => void;
        /**
         * Creates a new G Buffer for the scene
         * @param scene The scene the buffer belongs to
         * @param ratio How big is the buffer related to the main canvas (default: 1)
         * @param depthFormat Format of the depth texture (default: Constants.TEXTUREFORMAT_DEPTH16)
         */
        constructor(scene: Scene, ratio?: number, depthFormat?: number);
        /**
         * Checks whether everything is ready to render a submesh to the G buffer.
         * @param subMesh the submesh to check readiness for
         * @param useInstances is the mesh drawn using instance or not
         * @returns true if ready otherwise false
         */
        isReady(subMesh: SubMesh, useInstances: boolean): boolean;
        /**
         * Gets the current underlying G Buffer.
         * @returns the buffer
         */
        getGBuffer(): MultiRenderTarget;
        /**
         * Gets the number of samples used to render the buffer (anti aliasing).
         */
        get samples(): number;
        /**
         * Sets the number of samples used to render the buffer (anti aliasing).
         */
        set samples(value: number);
        /**
         * Disposes the renderer and frees up associated resources.
         */
        dispose(): void;
        private _assignRenderTargetIndices;
        protected _createRenderTargets(): void;
        private _copyBonesTransformationMatrices;
    }


        interface Scene {
            /** @internal (Backing field) */
            _geometryBufferRenderer: Nullable<GeometryBufferRenderer>;
            /**
             * Gets or Sets the current geometry buffer associated to the scene.
             */
            geometryBufferRenderer: Nullable<GeometryBufferRenderer>;
            /**
             * Enables a GeometryBufferRender and associates it with the scene
             * @param ratio defines the scaling ratio to apply to the renderer (1 by default which means same resolution)
             * @param depthFormat Format of the depth texture (default: Constants.TEXTUREFORMAT_DEPTH16)
             * @returns the GeometryBufferRenderer
             */
            enableGeometryBufferRenderer(ratio?: number, depthFormat?: number): Nullable<GeometryBufferRenderer>;
            /**
             * Disables the GeometryBufferRender associated with the scene
             */
            disableGeometryBufferRenderer(): void;
        }
    /**
     * Defines the Geometry Buffer scene component responsible to manage a G-Buffer useful
     * in several rendering techniques.
     */
    export class GeometryBufferRendererSceneComponent implements ISceneComponent {
        /**
         * The component name helpful to identify the component in the list of scene components.
         */
        readonly name = "GeometryBufferRenderer";
        /**
         * The scene the component belongs to.
         */
        scene: Scene;
        /**
         * Creates a new instance of the component for the given scene
         * @param scene Defines the scene to register the component in
         */
        constructor(scene: Scene);
        /**
         * Registers the component in a given scene
         */
        register(): void;
        /**
         * Rebuilds the elements related to this component in case of
         * context lost for instance.
         */
        rebuild(): void;
        /**
         * Disposes the component and the associated resources
         */
        dispose(): void;
        private _gatherRenderTargets;
    }




    /**
     * Contains all parameters needed for the prepass to perform
     * motion blur
     */
    export class MotionBlurConfiguration implements PrePassEffectConfiguration {
        /**
         * Is motion blur enabled
         */
        enabled: boolean;
        /**
         * Name of the configuration
         */
        name: string;
        /**
         * Textures that should be present in the MRT for this effect to work
         */
        readonly texturesRequired: number[];
    }


        interface Scene {
            /** @internal */
            _outlineRenderer: OutlineRenderer;
            /**
             * Gets the outline renderer associated with the scene
             * @returns a OutlineRenderer
             */
            getOutlineRenderer(): OutlineRenderer;
        }
        interface AbstractMesh {
            /** @internal (Backing field) */
            _renderOutline: boolean;
            /**
             * Gets or sets a boolean indicating if the outline must be rendered as well
             * @see https://www.babylonjs-playground.com/#10WJ5S#3
             */
            renderOutline: boolean;
            /** @internal (Backing field) */
            _renderOverlay: boolean;
            /**
             * Gets or sets a boolean indicating if the overlay must be rendered as well
             * @see https://www.babylonjs-playground.com/#10WJ5S#2
             */
            renderOverlay: boolean;
        }
    /**
     * This class is responsible to draw the outline/overlay of meshes.
     * It should not be used directly but through the available method on mesh.
     */
    export class OutlineRenderer implements ISceneComponent {
        /**
         * Stencil value used to avoid outline being seen within the mesh when the mesh is transparent
         */
        private static _StencilReference;
        /**
         * The name of the component. Each component must have a unique name.
         */
        name: string;
        /**
         * The scene the component belongs to.
         */
        scene: Scene;
        /**
         * Defines a zOffset default Factor to prevent zFighting between the overlay and the mesh.
         */
        zOffset: number;
        /**
         * Defines a zOffset default Unit to prevent zFighting between the overlay and the mesh.
         */
        zOffsetUnits: number;
        private _engine;
        private _savedDepthWrite;
        private _passIdForDrawWrapper;
        /**
         * Instantiates a new outline renderer. (There could be only one per scene).
         * @param scene Defines the scene it belongs to
         */
        constructor(scene: Scene);
        /**
         * Register the component to one instance of a scene.
         */
        register(): void;
        /**
         * Rebuilds the elements related to this component in case of
         * context lost for instance.
         */
        rebuild(): void;
        /**
         * Disposes the component and the associated resources.
         */
        dispose(): void;
        /**
         * Renders the outline in the canvas.
         * @param subMesh Defines the sumesh to render
         * @param batch Defines the batch of meshes in case of instances
         * @param useOverlay Defines if the rendering is for the overlay or the outline
         * @param renderPassId Render pass id to use to render the mesh
         */
        render(subMesh: SubMesh, batch: _InstancesBatch, useOverlay?: boolean, renderPassId?: number): void;
        /**
         * Returns whether or not the outline renderer is ready for a given submesh.
         * All the dependencies e.g. submeshes, texture, effect... mus be ready
         * @param subMesh Defines the submesh to check readiness for
         * @param useInstances Defines whether wee are trying to render instances or not
         * @param renderPassId Render pass id to use to render the mesh
         * @returns true if ready otherwise false
         */
        isReady(subMesh: SubMesh, useInstances: boolean, renderPassId?: number): boolean;
        private _beforeRenderingMesh;
        private _afterRenderingMesh;
    }


    /**
     * Interface for defining prepass effects in the prepass post-process pipeline
     */
    export interface PrePassEffectConfiguration {
        /**
         * Name of the effect
         */
        name: string;
        /**
         * Post process to attach for this effect
         */
        postProcess?: PostProcess;
        /**
         * Textures required in the MRT
         */
        texturesRequired: number[];
        /**
         * Is the effect enabled
         */
        enabled: boolean;
        /**
         * Does the output of this prepass need to go through imageprocessing
         */
        needsImageProcessing?: boolean;
        /**
         * Disposes the effect configuration
         */
        dispose?: () => void;
        /**
         * Creates the associated post process
         */
        createPostProcess?: () => PostProcess;
    }


    /**
     * Renders a pre pass of the scene
     * This means every mesh in the scene will be rendered to a render target texture
     * And then this texture will be composited to the rendering canvas with post processes
     * It is necessary for effects like subsurface scattering or deferred shading
     */
    export class PrePassRenderer {
        /**
         * @internal
         */
        static _SceneComponentInitialization: (scene: Scene) => void;
        /**
         * To save performance, we can excluded skinned meshes from the prepass
         */
        excludedSkinnedMesh: AbstractMesh[];
        /**
         * Force material to be excluded from the prepass
         * Can be useful when `useGeometryBufferFallback` is set to `true`
         * and you don't want a material to show in the effect.
         */
        excludedMaterials: Material[];
        private _scene;
        private _engine;
        /**
         * Number of textures in the multi render target texture where the scene is directly rendered
         */
        mrtCount: number;
        private _mrtFormats;
        private _mrtLayout;
        private _mrtNames;
        private _textureIndices;
        private _multiRenderAttachments;
        private _defaultAttachments;
        private _clearAttachments;
        /**
         * Returns the index of a texture in the multi render target texture array.
         * @param type Texture type
         * @returns The index
         */
        getIndex(type: number): number;
        /**
         * How many samples are used for MSAA of the scene render target
         */
        get samples(): number;
        set samples(n: number);
        private static _TextureFormats;
        private _isDirty;
        /**
         * The render target where the scene is directly rendered
         */
        defaultRT: PrePassRenderTarget;
        /**
         * Configuration for prepass effects
         */
        private _effectConfigurations;
        /**
         * @returns the prepass render target for the rendering pass.
         * If we are currently rendering a render target, it returns the PrePassRenderTarget
         * associated with that render target. Otherwise, it returns the scene default PrePassRenderTarget
         */
        getRenderTarget(): PrePassRenderTarget;
        /**
         * @internal
         * Managed by the scene component
         * @param prePassRenderTarget
         */
        _setRenderTarget(prePassRenderTarget: Nullable<PrePassRenderTarget>): void;
        /**
         * Returns true if the currently rendered prePassRenderTarget is the one
         * associated with the scene.
         */
        get currentRTisSceneRT(): boolean;
        private _geometryBuffer;
        /**
         * Prevents the PrePassRenderer from using the GeometryBufferRenderer as a fallback
         */
        doNotUseGeometryRendererFallback: boolean;
        private _refreshGeometryBufferRendererLink;
        private _currentTarget;
        /**
         * All the render targets generated by prepass
         */
        renderTargets: PrePassRenderTarget[];
        private readonly _clearColor;
        private _enabled;
        private _needsCompositionForThisPass;
        private _postProcessesSourceForThisPass;
        /**
         * Indicates if the prepass is enabled
         */
        get enabled(): boolean;
        /**
         * Set to true to disable gamma transform in PrePass.
         * Can be useful in case you already proceed to gamma transform on a material level
         * and your post processes don't need to be in linear color space.
         */
        disableGammaTransform: boolean;
        /**
         * Instantiates a prepass renderer
         * @param scene The scene
         */
        constructor(scene: Scene);
        /**
         * Creates a new PrePassRenderTarget
         * This should be the only way to instantiate a `PrePassRenderTarget`
         * @param name Name of the `PrePassRenderTarget`
         * @param renderTargetTexture RenderTarget the `PrePassRenderTarget` will be attached to.
         * Can be `null` if the created `PrePassRenderTarget` is attached to the scene (default framebuffer).
         * @internal
         */
        _createRenderTarget(name: string, renderTargetTexture: Nullable<RenderTargetTexture>): PrePassRenderTarget;
        /**
         * Indicates if rendering a prepass is supported
         */
        get isSupported(): boolean;
        /**
         * Sets the proper output textures to draw in the engine.
         * @param effect The effect that is drawn. It can be or not be compatible with drawing to several output textures.
         * @param subMesh Submesh on which the effect is applied
         */
        bindAttachmentsForEffect(effect: Effect, subMesh: SubMesh): void;
        private _reinitializeAttachments;
        private _resetLayout;
        private _updateGeometryBufferLayout;
        /**
         * Restores attachments for single texture draw.
         */
        restoreAttachments(): void;
        /**
         * @internal
         */
        _beforeDraw(camera?: Camera, faceIndex?: number, layer?: number): void;
        private _prepareFrame;
        /**
         * Sets an intermediary texture between prepass and postprocesses. This texture
         * will be used as input for post processes
         * @param rt
         * @returns true if there are postprocesses that will use this texture,
         * false if there is no postprocesses - and the function has no effect
         */
        setCustomOutput(rt: RenderTargetTexture): boolean;
        private _renderPostProcesses;
        /**
         * @internal
         */
        _afterDraw(faceIndex?: number, layer?: number): void;
        /**
         * Clears the current prepass render target (in the sense of settings pixels to the scene clear color value)
         * @internal
         */
        _clear(): void;
        private _bindFrameBuffer;
        private _setEnabled;
        private _setRenderTargetEnabled;
        /**
         * Adds an effect configuration to the prepass render target.
         * If an effect has already been added, it won't add it twice and will return the configuration
         * already present.
         * @param cfg the effect configuration
         * @returns the effect configuration now used by the prepass
         */
        addEffectConfiguration(cfg: PrePassEffectConfiguration): PrePassEffectConfiguration;
        private _enable;
        private _disable;
        private _getPostProcessesSource;
        private _setupOutputForThisPass;
        private _linkInternalTexture;
        /**
         * @internal
         */
        _unlinkInternalTexture(prePassRenderTarget: PrePassRenderTarget): void;
        private _needsImageProcessing;
        private _hasImageProcessing;
        /**
         * Internal, gets the first post proces.
         * @param postProcesses
         * @returns the first post process to be run on this camera.
         */
        private _getFirstPostProcess;
        /**
         * Marks the prepass renderer as dirty, triggering a check if the prepass is necessary for the next rendering.
         */
        markAsDirty(): void;
        /**
         * Enables a texture on the MultiRenderTarget for prepass
         * @param types
         */
        private _enableTextures;
        private _update;
        private _markAllMaterialsAsPrePassDirty;
        /**
         * Disposes the prepass renderer.
         */
        dispose(): void;
    }


        interface AbstractScene {
            /** @internal (Backing field) */
            _prePassRenderer: Nullable<PrePassRenderer>;
            /**
             * Gets or Sets the current prepass renderer associated to the scene.
             */
            prePassRenderer: Nullable<PrePassRenderer>;
            /**
             * Enables the prepass and associates it with the scene
             * @returns the PrePassRenderer
             */
            enablePrePassRenderer(): Nullable<PrePassRenderer>;
            /**
             * Disables the prepass associated with the scene
             */
            disablePrePassRenderer(): void;
        }
        interface RenderTargetTexture {
            /** @internal */
            _prePassRenderTarget: Nullable<PrePassRenderTarget>;
        }
    /**
     * Defines the Geometry Buffer scene component responsible to manage a G-Buffer useful
     * in several rendering techniques.
     */
    export class PrePassRendererSceneComponent implements ISceneComponent {
        /**
         * The component name helpful to identify the component in the list of scene components.
         */
        readonly name = "PrePassRenderer";
        /**
         * The scene the component belongs to.
         */
        scene: Scene;
        /**
         * Creates a new instance of the component for the given scene
         * @param scene Defines the scene to register the component in
         */
        constructor(scene: Scene);
        /**
         * Registers the component in a given scene
         */
        register(): void;
        private _beforeRenderTargetDraw;
        private _afterRenderTargetDraw;
        private _beforeRenderTargetClearStage;
        private _beforeCameraDraw;
        private _afterCameraDraw;
        private _beforeClearStage;
        private _beforeRenderingMeshStage;
        private _afterRenderingMeshStage;
        /**
         * Rebuilds the elements related to this component in case of
         * context lost for instance.
         */
        rebuild(): void;
        /**
         * Disposes the component and the associated resources
         */
        dispose(): void;
    }


    /**
     * This represents the object necessary to create a rendering group.
     * This is exclusively used and created by the rendering manager.
     * To modify the behavior, you use the available helpers in your scene or meshes.
     * @internal
     */
    export class RenderingGroup {
        index: number;
        private static _ZeroVector;
        private _scene;
        private _opaqueSubMeshes;
        private _transparentSubMeshes;
        private _alphaTestSubMeshes;
        private _depthOnlySubMeshes;
        private _particleSystems;
        private _spriteManagers;
        private _opaqueSortCompareFn;
        private _alphaTestSortCompareFn;
        private _transparentSortCompareFn;
        private _renderOpaque;
        private _renderAlphaTest;
        private _renderTransparent;
        /** @internal */
        _empty: boolean;
        /** @internal */
        _edgesRenderers: SmartArrayNoDuplicate<IEdgesRenderer>;
        onBeforeTransparentRendering: () => void;
        /**
         * Set the opaque sort comparison function.
         * If null the sub meshes will be render in the order they were created
         */
        set opaqueSortCompareFn(value: Nullable<(a: SubMesh, b: SubMesh) => number>);
        /**
         * Set the alpha test sort comparison function.
         * If null the sub meshes will be render in the order they were created
         */
        set alphaTestSortCompareFn(value: Nullable<(a: SubMesh, b: SubMesh) => number>);
        /**
         * Set the transparent sort comparison function.
         * If null the sub meshes will be render in the order they were created
         */
        set transparentSortCompareFn(value: Nullable<(a: SubMesh, b: SubMesh) => number>);
        /**
         * Creates a new rendering group.
         * @param index The rendering group index
         * @param scene
         * @param opaqueSortCompareFn The opaque sort comparison function. If null no order is applied
         * @param alphaTestSortCompareFn The alpha test sort comparison function. If null no order is applied
         * @param transparentSortCompareFn The transparent sort comparison function. If null back to front + alpha index sort is applied
         */
        constructor(index: number, scene: Scene, opaqueSortCompareFn?: Nullable<(a: SubMesh, b: SubMesh) => number>, alphaTestSortCompareFn?: Nullable<(a: SubMesh, b: SubMesh) => number>, transparentSortCompareFn?: Nullable<(a: SubMesh, b: SubMesh) => number>);
        /**
         * Render all the sub meshes contained in the group.
         * @param customRenderFunction Used to override the default render behaviour of the group.
         * @param renderSprites
         * @param renderParticles
         * @param activeMeshes
         * @returns true if rendered some submeshes.
         */
        render(customRenderFunction: Nullable<(opaqueSubMeshes: SmartArray<SubMesh>, transparentSubMeshes: SmartArray<SubMesh>, alphaTestSubMeshes: SmartArray<SubMesh>, depthOnlySubMeshes: SmartArray<SubMesh>) => void>, renderSprites: boolean, renderParticles: boolean, activeMeshes: Nullable<AbstractMesh[]>): void;
        /**
         * Renders the opaque submeshes in the order from the opaqueSortCompareFn.
         * @param subMeshes The submeshes to render
         */
        private _renderOpaqueSorted;
        /**
         * Renders the opaque submeshes in the order from the alphatestSortCompareFn.
         * @param subMeshes The submeshes to render
         */
        private _renderAlphaTestSorted;
        /**
         * Renders the opaque submeshes in the order from the transparentSortCompareFn.
         * @param subMeshes The submeshes to render
         */
        private _renderTransparentSorted;
        /**
         * Renders the submeshes in a specified order.
         * @param subMeshes The submeshes to sort before render
         * @param sortCompareFn The comparison function use to sort
         * @param camera The camera position use to preprocess the submeshes to help sorting
         * @param transparent Specifies to activate blending if true
         */
        private static _RenderSorted;
        /**
         * Build in function which can be applied to ensure meshes of a special queue (opaque, alpha test, transparent)
         * are rendered back to front if in the same alpha index.
         *
         * @param a The first submesh
         * @param b The second submesh
         * @returns The result of the comparison
         */
        static defaultTransparentSortCompare(a: SubMesh, b: SubMesh): number;
        /**
         * Build in function which can be applied to ensure meshes of a special queue (opaque, alpha test, transparent)
         * are rendered back to front.
         *
         * @param a The first submesh
         * @param b The second submesh
         * @returns The result of the comparison
         */
        static backToFrontSortCompare(a: SubMesh, b: SubMesh): number;
        /**
         * Build in function which can be applied to ensure meshes of a special queue (opaque, alpha test, transparent)
         * are rendered front to back (prevent overdraw).
         *
         * @param a The first submesh
         * @param b The second submesh
         * @returns The result of the comparison
         */
        static frontToBackSortCompare(a: SubMesh, b: SubMesh): number;
        /**
         * Build in function which can be applied to ensure meshes of a special queue (opaque, alpha test, transparent)
         * are grouped by material then geometry.
         *
         * @param a The first submesh
         * @param b The second submesh
         * @returns The result of the comparison
         */
        static PainterSortCompare(a: SubMesh, b: SubMesh): number;
        /**
         * Resets the different lists of submeshes to prepare a new frame.
         */
        prepare(): void;
        /**
         * Resets the different lists of sprites to prepare a new frame.
         */
        prepareSprites(): void;
        dispose(): void;
        /**
         * Inserts the submesh in its correct queue depending on its material.
         * @param subMesh The submesh to dispatch
         * @param [mesh] Optional reference to the submeshes's mesh. Provide if you have an exiting reference to improve performance.
         * @param [material] Optional reference to the submeshes's material. Provide if you have an exiting reference to improve performance.
         */
        dispatch(subMesh: SubMesh, mesh?: AbstractMesh, material?: Nullable<Material>): void;
        dispatchSprites(spriteManager: ISpriteManager): void;
        dispatchParticles(particleSystem: IParticleSystem): void;
        private _renderParticles;
        private _renderSprites;
    }


    /**
     * Interface describing the different options available in the rendering manager
     * regarding Auto Clear between groups.
     */
    export interface IRenderingManagerAutoClearSetup {
        /**
         * Defines whether or not autoclear is enable.
         */
        autoClear: boolean;
        /**
         * Defines whether or not to autoclear the depth buffer.
         */
        depth: boolean;
        /**
         * Defines whether or not to autoclear the stencil buffer.
         */
        stencil: boolean;
    }
    /**
     * This class is used by the onRenderingGroupObservable
     */
    export class RenderingGroupInfo {
        /**
         * The Scene that being rendered
         */
        scene: Scene;
        /**
         * The camera currently used for the rendering pass
         */
        camera: Nullable<Camera>;
        /**
         * The ID of the renderingGroup being processed
         */
        renderingGroupId: number;
    }
    /**
     * This is the manager responsible of all the rendering for meshes sprites and particles.
     * It is enable to manage the different groups as well as the different necessary sort functions.
     * This should not be used directly aside of the few static configurations
     */
    export class RenderingManager {
        /**
         * The max id used for rendering groups (not included)
         */
        static MAX_RENDERINGGROUPS: number;
        /**
         * The min id used for rendering groups (included)
         */
        static MIN_RENDERINGGROUPS: number;
        /**
         * Used to globally prevent autoclearing scenes.
         */
        static AUTOCLEAR: boolean;
        /**
         * @internal
         */
        _useSceneAutoClearSetup: boolean;
        private _scene;
        private _renderingGroups;
        private _depthStencilBufferAlreadyCleaned;
        private _autoClearDepthStencil;
        private _customOpaqueSortCompareFn;
        private _customAlphaTestSortCompareFn;
        private _customTransparentSortCompareFn;
        private _renderingGroupInfo;
        private _maintainStateBetweenFrames;
        /**
         * Gets or sets a boolean indicating that the manager will not reset between frames.
         * This means that if a mesh becomes invisible or transparent it will not be visible until this boolean is set to false again.
         * By default, the rendering manager will dispatch all active meshes per frame (moving them to the transparent, opaque or alpha testing lists).
         * By turning this property on, you will accelerate the rendering by keeping all these lists unchanged between frames.
         */
        get maintainStateBetweenFrames(): boolean;
        set maintainStateBetweenFrames(value: boolean);
        /**
         * Instantiates a new rendering group for a particular scene
         * @param scene Defines the scene the groups belongs to
         */
        constructor(scene: Scene);
        /**
         * Gets the rendering group with the specified id.
         */
        getRenderingGroup(id: number): RenderingGroup;
        private _clearDepthStencilBuffer;
        /**
         * Renders the entire managed groups. This is used by the scene or the different render targets.
         * @internal
         */
        render(customRenderFunction: Nullable<(opaqueSubMeshes: SmartArray<SubMesh>, transparentSubMeshes: SmartArray<SubMesh>, alphaTestSubMeshes: SmartArray<SubMesh>, depthOnlySubMeshes: SmartArray<SubMesh>) => void>, activeMeshes: Nullable<AbstractMesh[]>, renderParticles: boolean, renderSprites: boolean): void;
        /**
         * Resets the different information of the group to prepare a new frame
         * @internal
         */
        reset(): void;
        /**
         * Resets the sprites information of the group to prepare a new frame
         * @internal
         */
        resetSprites(): void;
        /**
         * Dispose and release the group and its associated resources.
         * @internal
         */
        dispose(): void;
        /**
         * Clear the info related to rendering groups preventing retention points during dispose.
         */
        freeRenderingGroups(): void;
        private _prepareRenderingGroup;
        /**
         * Add a sprite manager to the rendering manager in order to render it this frame.
         * @param spriteManager Define the sprite manager to render
         */
        dispatchSprites(spriteManager: ISpriteManager): void;
        /**
         * Add a particle system to the rendering manager in order to render it this frame.
         * @param particleSystem Define the particle system to render
         */
        dispatchParticles(particleSystem: IParticleSystem): void;
        /**
         * Add a submesh to the manager in order to render it this frame
         * @param subMesh The submesh to dispatch
         * @param mesh Optional reference to the submeshes's mesh. Provide if you have an exiting reference to improve performance.
         * @param material Optional reference to the submeshes's material. Provide if you have an exiting reference to improve performance.
         */
        dispatch(subMesh: SubMesh, mesh?: AbstractMesh, material?: Nullable<Material>): void;
        /**
         * Overrides the default sort function applied in the rendering group to prepare the meshes.
         * This allowed control for front to back rendering or reversely depending of the special needs.
         *
         * @param renderingGroupId The rendering group id corresponding to its index
         * @param opaqueSortCompareFn The opaque queue comparison function use to sort.
         * @param alphaTestSortCompareFn The alpha test queue comparison function use to sort.
         * @param transparentSortCompareFn The transparent queue comparison function use to sort.
         */
        setRenderingOrder(renderingGroupId: number, opaqueSortCompareFn?: Nullable<(a: SubMesh, b: SubMesh) => number>, alphaTestSortCompareFn?: Nullable<(a: SubMesh, b: SubMesh) => number>, transparentSortCompareFn?: Nullable<(a: SubMesh, b: SubMesh) => number>): void;
        /**
         * Specifies whether or not the stencil and depth buffer are cleared between two rendering groups.
         *
         * @param renderingGroupId The rendering group id corresponding to its index
         * @param autoClearDepthStencil Automatically clears depth and stencil between groups if true.
         * @param depth Automatically clears depth between groups if true and autoClear is true.
         * @param stencil Automatically clears stencil between groups if true and autoClear is true.
         */
        setRenderingAutoClearDepthStencil(renderingGroupId: number, autoClearDepthStencil: boolean, depth?: boolean, stencil?: boolean): void;
        /**
         * Gets the current auto clear configuration for one rendering group of the rendering
         * manager.
         * @param index the rendering group index to get the information for
         * @returns The auto clear setup for the requested rendering group
         */
        getAutoClearDepthStencilSetup(index: number): IRenderingManagerAutoClearSetup;
    }


    /**
     * Contains all parameters needed for the prepass to perform
     * screen space reflections
     */
    export class ScreenSpaceReflectionsConfiguration implements PrePassEffectConfiguration {
        /**
         * Is ssr enabled
         */
        enabled: boolean;
        /**
         * Name of the configuration
         */
        name: string;
        /**
         * Textures that should be present in the MRT for this effect to work
         */
        readonly texturesRequired: number[];
    }


    /**
     * Contains all parameters needed for the prepass to perform
     * screen space subsurface scattering
     */
    export class SSAO2Configuration implements PrePassEffectConfiguration {
        /**
         * Is subsurface enabled
         */
        enabled: boolean;
        /**
         * Name of the configuration
         */
        name: string;
        /**
         * Textures that should be present in the MRT for this effect to work
         */
        readonly texturesRequired: number[];
    }


    /**
     * Contains all parameters needed for the prepass to perform
     * screen space subsurface scattering
     */
    export class SubSurfaceConfiguration implements PrePassEffectConfiguration {
        /**
         * @internal
         */
        static _SceneComponentInitialization: (scene: Scene) => void;
        private _ssDiffusionS;
        private _ssFilterRadii;
        private _ssDiffusionD;
        /**
         * Post process to attach for screen space subsurface scattering
         */
        postProcess: SubSurfaceScatteringPostProcess;
        /**
         * Diffusion profile color for subsurface scattering
         */
        get ssDiffusionS(): number[];
        /**
         * Diffusion profile max color channel value for subsurface scattering
         */
        get ssDiffusionD(): number[];
        /**
         * Diffusion profile filter radius for subsurface scattering
         */
        get ssFilterRadii(): number[];
        /**
         * Is subsurface enabled
         */
        enabled: boolean;
        /**
         * Does the output of this prepass need to go through imageprocessing
         */
        needsImageProcessing: boolean;
        /**
         * Name of the configuration
         */
        name: string;
        /**
         * Diffusion profile colors for subsurface scattering
         * You can add one diffusion color using `addDiffusionProfile` on `scene.prePassRenderer`
         * See ...
         * Note that you can only store up to 5 of them
         */
        ssDiffusionProfileColors: Color3[];
        /**
         * Defines the ratio real world => scene units.
         * Used for subsurface scattering
         */
        metersPerUnit: number;
        /**
         * Textures that should be present in the MRT for this effect to work
         */
        readonly texturesRequired: number[];
        private _scene;
        /**
         * Builds a subsurface configuration object
         * @param scene The scene
         */
        constructor(scene: Scene);
        /**
         * Adds a new diffusion profile.
         * Useful for more realistic subsurface scattering on diverse materials.
         * @param color The color of the diffusion profile. Should be the average color of the material.
         * @returns The index of the diffusion profile for the material subsurface configuration
         */
        addDiffusionProfile(color: Color3): number;
        /**
         * Creates the sss post process
         * @returns The created post process
         */
        createPostProcess(): SubSurfaceScatteringPostProcess;
        /**
         * Deletes all diffusion profiles.
         * Note that in order to render subsurface scattering, you should have at least 1 diffusion profile.
         */
        clearAllDiffusionProfiles(): void;
        /**
         * Disposes this object
         */
        dispose(): void;
        /**
         * @internal
         * https://zero-radiance.github.io/post/sampling-diffusion/
         *
         * Importance sample the normalized diffuse reflectance profile for the computed value of 's'.
         * ------------------------------------------------------------------------------------
         * R[r, phi, s]   = s * (Exp[-r * s] + Exp[-r * s / 3]) / (8 * Pi * r)
         * PDF[r, phi, s] = r * R[r, phi, s]
         * CDF[r, s]      = 1 - 1/4 * Exp[-r * s] - 3/4 * Exp[-r * s / 3]
         * ------------------------------------------------------------------------------------
         * We importance sample the color channel with the widest scattering distance.
         */
        getDiffusionProfileParameters(color: Color3): number;
        /**
         * Performs sampling of a Normalized Burley diffusion profile in polar coordinates.
         * 'u' is the random number (the value of the CDF): [0, 1).
         * rcp(s) = 1 / ShapeParam = ScatteringDistance.
         * Returns the sampled radial distance, s.t. (u = 0 -> r = 0) and (u = 1 -> r = Inf).
         * @param u
         * @param rcpS
         */
        private _sampleBurleyDiffusionProfile;
    }


        interface AbstractScene {
            /** @internal (Backing field) */
            _subSurfaceConfiguration: Nullable<SubSurfaceConfiguration>;
            /**
             * Gets or Sets the current prepass renderer associated to the scene.
             */
            subSurfaceConfiguration: Nullable<SubSurfaceConfiguration>;
            /**
             * Enables the subsurface effect for prepass
             * @returns the SubSurfaceConfiguration
             */
            enableSubSurfaceForPrePass(): Nullable<SubSurfaceConfiguration>;
            /**
             * Disables the subsurface effect for prepass
             */
            disableSubSurfaceForPrePass(): void;
        }
    /**
     * Defines the Geometry Buffer scene component responsible to manage a G-Buffer useful
     * in several rendering techniques.
     */
    export class SubSurfaceSceneComponent implements ISceneSerializableComponent {
        /**
         * The component name helpful to identify the component in the list of scene components.
         */
        readonly name = "PrePassRenderer";
        /**
         * The scene the component belongs to.
         */
        scene: Scene;
        /**
         * Creates a new instance of the component for the given scene
         * @param scene Defines the scene to register the component in
         */
        constructor(scene: Scene);
        /**
         * Registers the component in a given scene
         */
        register(): void;
        /**
         * Serializes the component data to the specified json object
         * @param serializationObject The object to serialize to
         */
        serialize(serializationObject: any): void;
        /**
         * Adds all the elements from the container to the scene
         */
        addFromContainer(): void;
        /**
         * Removes all the elements in the container from the scene
         */
        removeFromContainer(): void;
        /**
         * Rebuilds the elements related to this component in case of
         * context lost for instance.
         */
        rebuild(): void;
        /**
         * Disposes the component and the associated resources
         */
        dispose(): void;
    }


    /**
     * Renders a layer on top of an existing scene
     */
    export class UtilityLayerRenderer implements IDisposable {
        /** the original scene that will be rendered on top of */
        originalScene: Scene;
        private _pointerCaptures;
        private _lastPointerEvents;
        /** @internal */
        static _DefaultUtilityLayer: Nullable<UtilityLayerRenderer>;
        /** @internal */
        static _DefaultKeepDepthUtilityLayer: Nullable<UtilityLayerRenderer>;
        private _sharedGizmoLight;
        private _renderCamera;
        /**
         * Gets the camera that is used to render the utility layer (when not set, this will be the last active camera)
         * @param getRigParentIfPossible if the current active camera is a rig camera, should its parent camera be returned
         * @returns the camera that is used when rendering the utility layer
         */
        getRenderCamera(getRigParentIfPossible?: boolean): Camera;
        /**
         * Sets the camera that should be used when rendering the utility layer (If set to null the last active camera will be used)
         * @param cam the camera that should be used when rendering the utility layer
         */
        setRenderCamera(cam: Nullable<Camera>): void;
        /**
         * @internal
         * Light which used by gizmos to get light shading
         */
        _getSharedGizmoLight(): HemisphericLight;
        /**
         * If the picking should be done on the utility layer prior to the actual scene (Default: true)
         */
        pickUtilitySceneFirst: boolean;
        /**
         * A shared utility layer that can be used to overlay objects into a scene (Depth map of the previous scene is cleared before drawing on top of it)
         */
        static get DefaultUtilityLayer(): UtilityLayerRenderer;
        /**
         * Creates an utility layer, and set it as a default utility layer
         * @param scene associated scene
         * @internal
         */
        static _CreateDefaultUtilityLayerFromScene(scene: Scene): UtilityLayerRenderer;
        /**
         * A shared utility layer that can be used to embed objects into a scene (Depth map of the previous scene is not cleared before drawing on top of it)
         */
        static get DefaultKeepDepthUtilityLayer(): UtilityLayerRenderer;
        /**
         * The scene that is rendered on top of the original scene
         */
        utilityLayerScene: Scene;
        /**
         *  If the utility layer should automatically be rendered on top of existing scene
         */
        shouldRender: boolean;
        /**
         * If set to true, only pointer down onPointerObservable events will be blocked when picking is occluded by original scene
         */
        onlyCheckPointerDownEvents: boolean;
        /**
         * If set to false, only pointerUp, pointerDown and pointerMove will be sent to the utilityLayerScene (false by default)
         */
        processAllEvents: boolean;
        /**
         * Set to false to disable picking
         */
        pickingEnabled: boolean;
        /**
         * Observable raised when the pointer moves from the utility layer scene to the main scene
         */
        onPointerOutObservable: Observable<number>;
        /** Gets or sets a predicate that will be used to indicate utility meshes present in the main scene */
        mainSceneTrackerPredicate: (mesh: Nullable<AbstractMesh>) => boolean;
        private _afterRenderObserver;
        private _sceneDisposeObserver;
        private _originalPointerObserver;
        /**
         * Instantiates a UtilityLayerRenderer
         * @param originalScene the original scene that will be rendered on top of
         * @param handleEvents boolean indicating if the utility layer should handle events
         */
        constructor(
        /** the original scene that will be rendered on top of */
        originalScene: Scene, handleEvents?: boolean);
        private _notifyObservers;
        /**
         * Renders the utility layers scene on top of the original scene
         */
        render(): void;
        /**
         * Disposes of the renderer
         */
        dispose(): void;
        private _updateCamera;
    }


    /**
     * Define an interface for all classes that will hold resources
     */
    export interface IDisposable {
        /**
         * Releases all held resources
         */
        dispose(): void;
    }
    /** Interface defining initialization parameters for Scene class */
    export interface SceneOptions {
        /**
         * Defines that scene should keep up-to-date a map of geometry to enable fast look-up by uniqueId
         * It will improve performance when the number of geometries becomes important.
         */
        useGeometryUniqueIdsMap?: boolean;
        /**
         * Defines that each material of the scene should keep up-to-date a map of referencing meshes for fast disposing
         * It will improve performance when the number of mesh becomes important, but might consume a bit more memory
         */
        useMaterialMeshMap?: boolean;
        /**
         * Defines that each mesh of the scene should keep up-to-date a map of referencing cloned meshes for fast disposing
         * It will improve performance when the number of mesh becomes important, but might consume a bit more memory
         */
        useClonedMeshMap?: boolean;
        /** Defines if the creation of the scene should impact the engine (Eg. UtilityLayer's scene) */
        virtual?: boolean;
    }
    /**
     * Define how the scene should favor performance over ease of use
     */
    export enum ScenePerformancePriority {
        /** Default mode. No change. Performance will be treated as less important than backward compatibility */
        BackwardCompatible = 0,
        /** Some performance options will be turned on trying to strike a balance between perf and ease of use */
        Intermediate = 1,
        /** Performance will be top priority */
        Aggressive = 2
    }
    /**
     * Represents a scene to be rendered by the engine.
     * @see https://doc.babylonjs.com/features/featuresDeepDive/scene
     */
    export class Scene extends AbstractScene implements IAnimatable, IClipPlanesHolder {
        /** The fog is deactivated */
        static readonly FOGMODE_NONE = 0;
        /** The fog density is following an exponential function */
        static readonly FOGMODE_EXP = 1;
        /** The fog density is following an exponential function faster than FOGMODE_EXP */
        static readonly FOGMODE_EXP2 = 2;
        /** The fog density is following a linear function. */
        static readonly FOGMODE_LINEAR = 3;
        /**
         * Gets or sets the minimum deltatime when deterministic lock step is enabled
         * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/advanced_animations#deterministic-lockstep
         */
        static MinDeltaTime: number;
        /**
         * Gets or sets the maximum deltatime when deterministic lock step is enabled
         * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/advanced_animations#deterministic-lockstep
         */
        static MaxDeltaTime: number;
        /**
         * Factory used to create the default material.
         * @param scene The scene to create the material for
         * @returns The default material
         */
        static DefaultMaterialFactory(scene: Scene): Material;
        /**
         * Factory used to create the a collision coordinator.
         * @returns The collision coordinator
         */
        static CollisionCoordinatorFactory(): ICollisionCoordinator;
        /** @internal */
        _inputManager: InputManager;
        /** Define this parameter if you are using multiple cameras and you want to specify which one should be used for pointer position */
        cameraToUseForPointers: Nullable<Camera>;
        /** @internal */
        readonly _isScene = true;
        /** @internal */
        _blockEntityCollection: boolean;
        /**
         * Gets or sets a boolean that indicates if the scene must clear the render buffer before rendering a frame
         */
        autoClear: boolean;
        /**
         * Gets or sets a boolean that indicates if the scene must clear the depth and stencil buffers before rendering a frame
         */
        autoClearDepthAndStencil: boolean;
        /**
         * Defines the color used to clear the render buffer (Default is (0.2, 0.2, 0.3, 1.0))
         */
        clearColor: Color4;
        /**
         * Defines the color used to simulate the ambient color (Default is (0, 0, 0))
         */
        ambientColor: Color3;
        /**
         * This is use to store the default BRDF lookup for PBR materials in your scene.
         * It should only be one of the following (if not the default embedded one):
         * * For uncorrelated BRDF (pbr.brdf.useEnergyConservation = false and pbr.brdf.useSmithVisibilityHeightCorrelated = false) : https://assets.babylonjs.com/environments/uncorrelatedBRDF.dds
         * * For correlated BRDF (pbr.brdf.useEnergyConservation = false and pbr.brdf.useSmithVisibilityHeightCorrelated = true) : https://assets.babylonjs.com/environments/correlatedBRDF.dds
         * * For correlated multi scattering BRDF (pbr.brdf.useEnergyConservation = true and pbr.brdf.useSmithVisibilityHeightCorrelated = true) : https://assets.babylonjs.com/environments/correlatedMSBRDF.dds
         * The material properties need to be setup according to the type of texture in use.
         */
        environmentBRDFTexture: BaseTexture;
        /**
         * Texture used in all pbr material as the reflection texture.
         * As in the majority of the scene they are the same (exception for multi room and so on),
         * this is easier to reference from here than from all the materials.
         */
        get environmentTexture(): Nullable<BaseTexture>;
        /**
         * Texture used in all pbr material as the reflection texture.
         * As in the majority of the scene they are the same (exception for multi room and so on),
         * this is easier to set here than in all the materials.
         */
        set environmentTexture(value: Nullable<BaseTexture>);
        /**
         * Intensity of the environment in all pbr material.
         * This dims or reinforces the IBL lighting overall (reflection and diffuse).
         * As in the majority of the scene they are the same (exception for multi room and so on),
         * this is easier to reference from here than from all the materials.
         */
        environmentIntensity: number;
        /** @internal */
        protected _imageProcessingConfiguration: ImageProcessingConfiguration;
        /**
         * Default image processing configuration used either in the rendering
         * Forward main pass or through the imageProcessingPostProcess if present.
         * As in the majority of the scene they are the same (exception for multi camera),
         * this is easier to reference from here than from all the materials and post process.
         *
         * No setter as we it is a shared configuration, you can set the values instead.
         */
        get imageProcessingConfiguration(): ImageProcessingConfiguration;
        private _performancePriority;
        /**
         * Gets or sets a value indicating how to treat performance relatively to ease of use and backward compatibility
         */
        get performancePriority(): ScenePerformancePriority;
        set performancePriority(value: ScenePerformancePriority);
        private _forceWireframe;
        /**
         * Gets or sets a boolean indicating if all rendering must be done in wireframe
         */
        set forceWireframe(value: boolean);
        get forceWireframe(): boolean;
        private _skipFrustumClipping;
        /**
         * Gets or sets a boolean indicating if we should skip the frustum clipping part of the active meshes selection
         */
        set skipFrustumClipping(value: boolean);
        get skipFrustumClipping(): boolean;
        private _forcePointsCloud;
        /**
         * Gets or sets a boolean indicating if all rendering must be done in point cloud
         */
        set forcePointsCloud(value: boolean);
        get forcePointsCloud(): boolean;
        /**
         * Gets or sets the active clipplane 1
         */
        clipPlane: Nullable<Plane>;
        /**
         * Gets or sets the active clipplane 2
         */
        clipPlane2: Nullable<Plane>;
        /**
         * Gets or sets the active clipplane 3
         */
        clipPlane3: Nullable<Plane>;
        /**
         * Gets or sets the active clipplane 4
         */
        clipPlane4: Nullable<Plane>;
        /**
         * Gets or sets the active clipplane 5
         */
        clipPlane5: Nullable<Plane>;
        /**
         * Gets or sets the active clipplane 6
         */
        clipPlane6: Nullable<Plane>;
        /**
         * Gets or sets a boolean indicating if animations are enabled
         */
        animationsEnabled: boolean;
        private _animationPropertiesOverride;
        /**
         * Gets or sets the animation properties override
         */
        get animationPropertiesOverride(): Nullable<AnimationPropertiesOverride>;
        set animationPropertiesOverride(value: Nullable<AnimationPropertiesOverride>);
        /**
         * Gets or sets a boolean indicating if a constant deltatime has to be used
         * This is mostly useful for testing purposes when you do not want the animations to scale with the framerate
         */
        useConstantAnimationDeltaTime: boolean;
        /**
         * Gets or sets a boolean indicating if the scene must keep the meshUnderPointer property updated
         * Please note that it requires to run a ray cast through the scene on every frame
         */
        constantlyUpdateMeshUnderPointer: boolean;
        /**
         * Defines the HTML cursor to use when hovering over interactive elements
         */
        hoverCursor: string;
        /**
         * Defines the HTML default cursor to use (empty by default)
         */
        defaultCursor: string;
        /**
         * Defines whether cursors are handled by the scene.
         */
        doNotHandleCursors: boolean;
        /**
         * This is used to call preventDefault() on pointer down
         * in order to block unwanted artifacts like system double clicks
         */
        preventDefaultOnPointerDown: boolean;
        /**
         * This is used to call preventDefault() on pointer up
         * in order to block unwanted artifacts like system double clicks
         */
        preventDefaultOnPointerUp: boolean;
        /**
         * Gets or sets user defined metadata
         */
        metadata: any;
        /**
         * For internal use only. Please do not use.
         */
        reservedDataStore: any;
        /**
         * Gets the name of the plugin used to load this scene (null by default)
         */
        loadingPluginName: string;
        /**
         * Use this array to add regular expressions used to disable offline support for specific urls
         */
        disableOfflineSupportExceptionRules: RegExp[];
        /**
         * An event triggered when the scene is disposed.
         */
        onDisposeObservable: Observable<Scene>;
        private _onDisposeObserver;
        /** Sets a function to be executed when this scene is disposed. */
        set onDispose(callback: () => void);
        /**
         * An event triggered before rendering the scene (right after animations and physics)
         */
        onBeforeRenderObservable: Observable<Scene>;
        private _onBeforeRenderObserver;
        /** Sets a function to be executed before rendering this scene */
        set beforeRender(callback: Nullable<() => void>);
        /**
         * An event triggered after rendering the scene
         */
        onAfterRenderObservable: Observable<Scene>;
        /**
         * An event triggered after rendering the scene for an active camera (When scene.render is called this will be called after each camera)
         * This is triggered for each "sub" camera in a Camera Rig unlike onAfterCameraRenderObservable
         */
        onAfterRenderCameraObservable: Observable<Camera>;
        private _onAfterRenderObserver;
        /** Sets a function to be executed after rendering this scene */
        set afterRender(callback: Nullable<() => void>);
        /**
         * An event triggered before animating the scene
         */
        onBeforeAnimationsObservable: Observable<Scene>;
        /**
         * An event triggered after animations processing
         */
        onAfterAnimationsObservable: Observable<Scene>;
        /**
         * An event triggered before draw calls are ready to be sent
         */
        onBeforeDrawPhaseObservable: Observable<Scene>;
        /**
         * An event triggered after draw calls have been sent
         */
        onAfterDrawPhaseObservable: Observable<Scene>;
        /**
         * An event triggered when the scene is ready
         */
        onReadyObservable: Observable<Scene>;
        /**
         * An event triggered before rendering a camera
         */
        onBeforeCameraRenderObservable: Observable<Camera>;
        private _onBeforeCameraRenderObserver;
        /** Sets a function to be executed before rendering a camera*/
        set beforeCameraRender(callback: () => void);
        /**
         * An event triggered after rendering a camera
         * This is triggered for the full rig Camera only unlike onAfterRenderCameraObservable
         */
        onAfterCameraRenderObservable: Observable<Camera>;
        private _onAfterCameraRenderObserver;
        /** Sets a function to be executed after rendering a camera*/
        set afterCameraRender(callback: () => void);
        /**
         * An event triggered when active meshes evaluation is about to start
         */
        onBeforeActiveMeshesEvaluationObservable: Observable<Scene>;
        /**
         * An event triggered when active meshes evaluation is done
         */
        onAfterActiveMeshesEvaluationObservable: Observable<Scene>;
        /**
         * An event triggered when particles rendering is about to start
         * Note: This event can be trigger more than once per frame (because particles can be rendered by render target textures as well)
         */
        onBeforeParticlesRenderingObservable: Observable<Scene>;
        /**
         * An event triggered when particles rendering is done
         * Note: This event can be trigger more than once per frame (because particles can be rendered by render target textures as well)
         */
        onAfterParticlesRenderingObservable: Observable<Scene>;
        /**
         * An event triggered when SceneLoader.Append or SceneLoader.Load or SceneLoader.ImportMesh were successfully executed
         */
        onDataLoadedObservable: Observable<Scene>;
        /**
         * An event triggered when a camera is created
         */
        onNewCameraAddedObservable: Observable<Camera>;
        /**
         * An event triggered when a camera is removed
         */
        onCameraRemovedObservable: Observable<Camera>;
        /**
         * An event triggered when a light is created
         */
        onNewLightAddedObservable: Observable<Light>;
        /**
         * An event triggered when a light is removed
         */
        onLightRemovedObservable: Observable<Light>;
        /**
         * An event triggered when a geometry is created
         */
        onNewGeometryAddedObservable: Observable<Geometry>;
        /**
         * An event triggered when a geometry is removed
         */
        onGeometryRemovedObservable: Observable<Geometry>;
        /**
         * An event triggered when a transform node is created
         */
        onNewTransformNodeAddedObservable: Observable<TransformNode>;
        /**
         * An event triggered when a transform node is removed
         */
        onTransformNodeRemovedObservable: Observable<TransformNode>;
        /**
         * An event triggered when a mesh is created
         */
        onNewMeshAddedObservable: Observable<AbstractMesh>;
        /**
         * An event triggered when a mesh is removed
         */
        onMeshRemovedObservable: Observable<AbstractMesh>;
        /**
         * An event triggered when a skeleton is created
         */
        onNewSkeletonAddedObservable: Observable<Skeleton>;
        /**
         * An event triggered when a skeleton is removed
         */
        onSkeletonRemovedObservable: Observable<Skeleton>;
        /**
         * An event triggered when a material is created
         */
        onNewMaterialAddedObservable: Observable<Material>;
        /**
         * An event triggered when a multi material is created
         */
        onNewMultiMaterialAddedObservable: Observable<MultiMaterial>;
        /**
         * An event triggered when a material is removed
         */
        onMaterialRemovedObservable: Observable<Material>;
        /**
         * An event triggered when a multi material is removed
         */
        onMultiMaterialRemovedObservable: Observable<MultiMaterial>;
        /**
         * An event triggered when a texture is created
         */
        onNewTextureAddedObservable: Observable<BaseTexture>;
        /**
         * An event triggered when a texture is removed
         */
        onTextureRemovedObservable: Observable<BaseTexture>;
        /**
         * An event triggered when render targets are about to be rendered
         * Can happen multiple times per frame.
         */
        onBeforeRenderTargetsRenderObservable: Observable<Scene>;
        /**
         * An event triggered when render targets were rendered.
         * Can happen multiple times per frame.
         */
        onAfterRenderTargetsRenderObservable: Observable<Scene>;
        /**
         * An event triggered before calculating deterministic simulation step
         */
        onBeforeStepObservable: Observable<Scene>;
        /**
         * An event triggered after calculating deterministic simulation step
         */
        onAfterStepObservable: Observable<Scene>;
        /**
         * An event triggered when the activeCamera property is updated
         */
        onActiveCameraChanged: Observable<Scene>;
        /**
         * An event triggered when the activeCameras property is updated
         */
        onActiveCamerasChanged: Observable<Scene>;
        /**
         * This Observable will be triggered before rendering each renderingGroup of each rendered camera.
         * The RenderingGroupInfo class contains all the information about the context in which the observable is called
         * If you wish to register an Observer only for a given set of renderingGroup, use the mask with a combination of the renderingGroup index elevated to the power of two (1 for renderingGroup 0, 2 for renderingrOup1, 4 for 2 and 8 for 3)
         */
        onBeforeRenderingGroupObservable: Observable<RenderingGroupInfo>;
        /**
         * This Observable will be triggered after rendering each renderingGroup of each rendered camera.
         * The RenderingGroupInfo class contains all the information about the context in which the observable is called
         * If you wish to register an Observer only for a given set of renderingGroup, use the mask with a combination of the renderingGroup index elevated to the power of two (1 for renderingGroup 0, 2 for renderingrOup1, 4 for 2 and 8 for 3)
         */
        onAfterRenderingGroupObservable: Observable<RenderingGroupInfo>;
        /**
         * This Observable will when a mesh has been imported into the scene.
         */
        onMeshImportedObservable: Observable<AbstractMesh>;
        /**
         * This Observable will when an animation file has been imported into the scene.
         */
        onAnimationFileImportedObservable: Observable<Scene>;
        /**
         * Gets or sets a user defined funtion to select LOD from a mesh and a camera.
         * By default this function is undefined and Babylon.js will select LOD based on distance to camera
         */
        customLODSelector: (mesh: AbstractMesh, camera: Camera) => Nullable<AbstractMesh>;
        /** @internal */
        _registeredForLateAnimationBindings: SmartArrayNoDuplicate<any>;
        /**
         * Gets or sets a predicate used to select candidate meshes for a pointer down event
         */
        pointerDownPredicate: (Mesh: AbstractMesh) => boolean;
        /**
         * Gets or sets a predicate used to select candidate meshes for a pointer up event
         */
        pointerUpPredicate: (Mesh: AbstractMesh) => boolean;
        /**
         * Gets or sets a predicate used to select candidate meshes for a pointer move event
         */
        pointerMovePredicate: (Mesh: AbstractMesh) => boolean;
        /**
         * Gets or sets a boolean indicating if the user want to entirely skip the picking phase when a pointer move event occurs.
         */
        skipPointerMovePicking: boolean;
        /**
         * Gets or sets a boolean indicating if the user want to entirely skip the picking phase when a pointer down event occurs.
         */
        skipPointerDownPicking: boolean;
        /**
         * Gets or sets a boolean indicating if the user want to entirely skip the picking phase when a pointer up event occurs.  Off by default.
         */
        skipPointerUpPicking: boolean;
        /** Callback called when a pointer move is detected */
        onPointerMove?: (evt: IPointerEvent, pickInfo: PickingInfo, type: PointerEventTypes) => void;
        /** Callback called when a pointer down is detected  */
        onPointerDown?: (evt: IPointerEvent, pickInfo: PickingInfo, type: PointerEventTypes) => void;
        /** Callback called when a pointer up is detected  */
        onPointerUp?: (evt: IPointerEvent, pickInfo: Nullable<PickingInfo>, type: PointerEventTypes) => void;
        /** Callback called when a pointer pick is detected */
        onPointerPick?: (evt: IPointerEvent, pickInfo: PickingInfo) => void;
        /**
         * Gets or sets a predicate used to select candidate faces for a pointer move event
         */
        pointerMoveTrianglePredicate: ((p0: Vector3, p1: Vector3, p2: Vector3, ray: Ray) => boolean) | undefined;
        /**
         * This observable event is triggered when any ponter event is triggered. It is registered during Scene.attachControl() and it is called BEFORE the 3D engine process anything (mesh/sprite picking for instance).
         * You have the possibility to skip the process and the call to onPointerObservable by setting PointerInfoPre.skipOnPointerObservable to true
         */
        onPrePointerObservable: Observable<PointerInfoPre>;
        /**
         * Observable event triggered each time an input event is received from the rendering canvas
         */
        onPointerObservable: Observable<PointerInfo>;
        /**
         * Gets the pointer coordinates without any translation (ie. straight out of the pointer event)
         */
        get unTranslatedPointer(): Vector2;
        /**
         * Gets or sets the distance in pixel that you have to move to prevent some events. Default is 10 pixels
         */
        static get DragMovementThreshold(): number;
        static set DragMovementThreshold(value: number);
        /**
         * Time in milliseconds to wait to raise long press events if button is still pressed. Default is 500 ms
         */
        static get LongPressDelay(): number;
        static set LongPressDelay(value: number);
        /**
         * Time in milliseconds to wait to raise long press events if button is still pressed. Default is 300 ms
         */
        static get DoubleClickDelay(): number;
        static set DoubleClickDelay(value: number);
        /** If you need to check double click without raising a single click at first click, enable this flag */
        static get ExclusiveDoubleClickMode(): boolean;
        static set ExclusiveDoubleClickMode(value: boolean);
        /**
         * Bind the current view position to an effect.
         * @param effect The effect to be bound
         * @param variableName name of the shader variable that will hold the eye position
         * @param isVector3 true to indicates that variableName is a Vector3 and not a Vector4
         * @returns the computed eye position
         */
        bindEyePosition(effect: Nullable<Effect>, variableName?: string, isVector3?: boolean): Vector4;
        /**
         * Update the scene ubo before it can be used in rendering processing
         * @returns the scene UniformBuffer
         */
        finalizeSceneUbo(): UniformBuffer;
        /** @internal */
        _mirroredCameraPosition: Nullable<Vector3>;
        /**
         * This observable event is triggered when any keyboard event si raised and registered during Scene.attachControl()
         * You have the possibility to skip the process and the call to onKeyboardObservable by setting KeyboardInfoPre.skipOnPointerObservable to true
         */
        onPreKeyboardObservable: Observable<KeyboardInfoPre>;
        /**
         * Observable event triggered each time an keyboard event is received from the hosting window
         */
        onKeyboardObservable: Observable<KeyboardInfo>;
        private _useRightHandedSystem;
        /**
         * Gets or sets a boolean indicating if the scene must use right-handed coordinates system
         */
        set useRightHandedSystem(value: boolean);
        get useRightHandedSystem(): boolean;
        private _timeAccumulator;
        private _currentStepId;
        private _currentInternalStep;
        /**
         * Sets the step Id used by deterministic lock step
         * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/advanced_animations#deterministic-lockstep
         * @param newStepId defines the step Id
         */
        setStepId(newStepId: number): void;
        /**
         * Gets the step Id used by deterministic lock step
         * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/advanced_animations#deterministic-lockstep
         * @returns the step Id
         */
        getStepId(): number;
        /**
         * Gets the internal step used by deterministic lock step
         * @see https://doc.babylonjs.com/features/featuresDeepDive/animation/advanced_animations#deterministic-lockstep
         * @returns the internal step
         */
        getInternalStep(): number;
        private _fogEnabled;
        /**
         * Gets or sets a boolean indicating if fog is enabled on this scene
         * @see https://doc.babylonjs.com/features/featuresDeepDive/environment/environment_introduction#fog
         * (Default is true)
         */
        set fogEnabled(value: boolean);
        get fogEnabled(): boolean;
        private _fogMode;
        /**
         * Gets or sets the fog mode to use
         * @see https://doc.babylonjs.com/features/featuresDeepDive/environment/environment_introduction#fog
         * | mode | value |
         * | --- | --- |
         * | FOGMODE_NONE | 0 |
         * | FOGMODE_EXP | 1 |
         * | FOGMODE_EXP2 | 2 |
         * | FOGMODE_LINEAR | 3 |
         */
        set fogMode(value: number);
        get fogMode(): number;
        /**
         * Gets or sets the fog color to use
         * @see https://doc.babylonjs.com/features/featuresDeepDive/environment/environment_introduction#fog
         * (Default is Color3(0.2, 0.2, 0.3))
         */
        fogColor: Color3;
        /**
         * Gets or sets the fog density to use
         * @see https://doc.babylonjs.com/features/featuresDeepDive/environment/environment_introduction#fog
         * (Default is 0.1)
         */
        fogDensity: number;
        /**
         * Gets or sets the fog start distance to use
         * @see https://doc.babylonjs.com/features/featuresDeepDive/environment/environment_introduction#fog
         * (Default is 0)
         */
        fogStart: number;
        /**
         * Gets or sets the fog end distance to use
         * @see https://doc.babylonjs.com/features/featuresDeepDive/environment/environment_introduction#fog
         * (Default is 1000)
         */
        fogEnd: number;
        /**
         * Flag indicating that the frame buffer binding is handled by another component
         */
        get prePass(): boolean;
        /**
         * Flag indicating if we need to store previous matrices when rendering
         */
        needsPreviousWorldMatrices: boolean;
        private _shadowsEnabled;
        /**
         * Gets or sets a boolean indicating if shadows are enabled on this scene
         */
        set shadowsEnabled(value: boolean);
        get shadowsEnabled(): boolean;
        private _lightsEnabled;
        /**
         * Gets or sets a boolean indicating if lights are enabled on this scene
         */
        set lightsEnabled(value: boolean);
        get lightsEnabled(): boolean;
        private _activeCameras;
        private _unObserveActiveCameras;
        /** All of the active cameras added to this scene. */
        get activeCameras(): Nullable<Camera[]>;
        set activeCameras(cameras: Nullable<Camera[]>);
        /** @internal */
        _activeCamera: Nullable<Camera>;
        /** Gets or sets the current active camera */
        get activeCamera(): Nullable<Camera>;
        set activeCamera(value: Nullable<Camera>);
        private _defaultMaterial;
        /** The default material used on meshes when no material is affected */
        get defaultMaterial(): Material;
        /** The default material used on meshes when no material is affected */
        set defaultMaterial(value: Material);
        private _texturesEnabled;
        /**
         * Gets or sets a boolean indicating if textures are enabled on this scene
         */
        set texturesEnabled(value: boolean);
        get texturesEnabled(): boolean;
        /**
         * Gets or sets a boolean indicating if physic engines are enabled on this scene
         */
        physicsEnabled: boolean;
        /**
         * Gets or sets a boolean indicating if particles are enabled on this scene
         */
        particlesEnabled: boolean;
        /**
         * Gets or sets a boolean indicating if sprites are enabled on this scene
         */
        spritesEnabled: boolean;
        private _skeletonsEnabled;
        /**
         * Gets or sets a boolean indicating if skeletons are enabled on this scene
         */
        set skeletonsEnabled(value: boolean);
        get skeletonsEnabled(): boolean;
        /**
         * Gets or sets a boolean indicating if lens flares are enabled on this scene
         */
        lensFlaresEnabled: boolean;
        /**
         * Gets or sets a boolean indicating if collisions are enabled on this scene
         * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_collisions
         */
        collisionsEnabled: boolean;
        private _collisionCoordinator;
        /** @internal */
        get collisionCoordinator(): ICollisionCoordinator;
        /**
         * Defines the gravity applied to this scene (used only for collisions)
         * @see https://doc.babylonjs.com/features/featuresDeepDive/cameras/camera_collisions
         */
        gravity: Vector3;
        /**
         * Gets or sets a boolean indicating if postprocesses are enabled on this scene
         */
        postProcessesEnabled: boolean;
        /**
         * Gets the current postprocess manager
         */
        postProcessManager: PostProcessManager;
        /**
         * Gets or sets a boolean indicating if render targets are enabled on this scene
         */
        renderTargetsEnabled: boolean;
        /**
         * Gets or sets a boolean indicating if next render targets must be dumped as image for debugging purposes
         * We recommend not using it and instead rely on Spector.js: http://spector.babylonjs.com
         */
        dumpNextRenderTargets: boolean;
        /**
         * The list of user defined render targets added to the scene
         */
        customRenderTargets: RenderTargetTexture[];
        /**
         * Defines if texture loading must be delayed
         * If true, textures will only be loaded when they need to be rendered
         */
        useDelayedTextureLoading: boolean;
        /**
         * Gets the list of meshes imported to the scene through SceneLoader
         */
        importedMeshesFiles: string[];
        /**
         * Gets or sets a boolean indicating if probes are enabled on this scene
         */
        probesEnabled: boolean;
        /**
         * Gets or sets the current offline provider to use to store scene data
         * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/optimizeCached
         */
        offlineProvider: IOfflineProvider;
        /**
         * Gets or sets the action manager associated with the scene
         * @see https://doc.babylonjs.com/features/featuresDeepDive/events/actions
         */
        actionManager: AbstractActionManager;
        private _meshesForIntersections;
        /**
         * Gets or sets a boolean indicating if procedural textures are enabled on this scene
         */
        proceduralTexturesEnabled: boolean;
        private _engine;
        private _totalVertices;
        /** @internal */
        _activeIndices: PerfCounter;
        /** @internal */
        _activeParticles: PerfCounter;
        /** @internal */
        _activeBones: PerfCounter;
        private _animationRatio;
        /** @internal */
        _animationTimeLast: number;
        /** @internal */
        _animationTime: number;
        /**
         * Gets or sets a general scale for animation speed
         * @see https://www.babylonjs-playground.com/#IBU2W7#3
         */
        animationTimeScale: number;
        /** @internal */
        _cachedMaterial: Nullable<Material>;
        /** @internal */
        _cachedEffect: Nullable<Effect>;
        /** @internal */
        _cachedVisibility: Nullable<number>;
        private _renderId;
        private _frameId;
        private _executeWhenReadyTimeoutId;
        private _intermediateRendering;
        private _defaultFrameBufferCleared;
        private _viewUpdateFlag;
        private _projectionUpdateFlag;
        /** @internal */
        _toBeDisposed: Nullable<IDisposable>[];
        private _activeRequests;
        /** @internal */
        _pendingData: any[];
        private _isDisposed;
        /**
         * Gets or sets a boolean indicating that all submeshes of active meshes must be rendered
         * Use this boolean to avoid computing frustum clipping on submeshes (This could help when you are CPU bound)
         */
        dispatchAllSubMeshesOfActiveMeshes: boolean;
        private _activeMeshes;
        private _processedMaterials;
        private _renderTargets;
        private _materialsRenderTargets;
        /** @internal */
        _activeParticleSystems: SmartArray<IParticleSystem>;
        private _activeSkeletons;
        private _softwareSkinnedMeshes;
        private _renderingManager;
        /**
         * Gets the scene's rendering manager
         */
        get renderingManager(): RenderingManager;
        /** @internal */
        _activeAnimatables: Animatable[];
        private _transformMatrix;
        private _sceneUbo;
        /** @internal */
        _viewMatrix: Matrix;
        /** @internal */
        _projectionMatrix: Matrix;
        /** @internal */
        _forcedViewPosition: Nullable<Vector3>;
        /** @internal */
        _frustumPlanes: Plane[];
        /**
         * Gets the list of frustum planes (built from the active camera)
         */
        get frustumPlanes(): Plane[];
        /**
         * Gets or sets a boolean indicating if lights must be sorted by priority (off by default)
         * This is useful if there are more lights that the maximum simulteanous authorized
         */
        requireLightSorting: boolean;
        /** @internal */
        readonly useMaterialMeshMap: boolean;
        /** @internal */
        readonly useClonedMeshMap: boolean;
        private _externalData;
        private _uid;
        /**
         * @internal
         * Backing store of defined scene components.
         */
        _components: ISceneComponent[];
        /**
         * @internal
         * Backing store of defined scene components.
         */
        _serializableComponents: ISceneSerializableComponent[];
        /**
         * List of components to register on the next registration step.
         */
        private _transientComponents;
        /**
         * Registers the transient components if needed.
         */
        private _registerTransientComponents;
        /**
         * @internal
         * Add a component to the scene.
         * Note that the ccomponent could be registered on th next frame if this is called after
         * the register component stage.
         * @param component Defines the component to add to the scene
         */
        _addComponent(component: ISceneComponent): void;
        /**
         * @internal
         * Gets a component from the scene.
         * @param name defines the name of the component to retrieve
         * @returns the component or null if not present
         */
        _getComponent(name: string): Nullable<ISceneComponent>;
        /**
         * @internal
         * Defines the actions happening before camera updates.
         */
        _beforeCameraUpdateStage: Stage<SimpleStageAction>;
        /**
         * @internal
         * Defines the actions happening before clear the canvas.
         */
        _beforeClearStage: Stage<SimpleStageAction>;
        /**
         * @internal
         * Defines the actions happening before clear the canvas.
         */
        _beforeRenderTargetClearStage: Stage<RenderTargetStageAction>;
        /**
         * @internal
         * Defines the actions when collecting render targets for the frame.
         */
        _gatherRenderTargetsStage: Stage<RenderTargetsStageAction>;
        /**
         * @internal
         * Defines the actions happening for one camera in the frame.
         */
        _gatherActiveCameraRenderTargetsStage: Stage<RenderTargetsStageAction>;
        /**
         * @internal
         * Defines the actions happening during the per mesh ready checks.
         */
        _isReadyForMeshStage: Stage<MeshStageAction>;
        /**
         * @internal
         * Defines the actions happening before evaluate active mesh checks.
         */
        _beforeEvaluateActiveMeshStage: Stage<SimpleStageAction>;
        /**
         * @internal
         * Defines the actions happening during the evaluate sub mesh checks.
         */
        _evaluateSubMeshStage: Stage<EvaluateSubMeshStageAction>;
        /**
         * @internal
         * Defines the actions happening during the active mesh stage.
         */
        _preActiveMeshStage: Stage<PreActiveMeshStageAction>;
        /**
         * @internal
         * Defines the actions happening during the per camera render target step.
         */
        _cameraDrawRenderTargetStage: Stage<CameraStageFrameBufferAction>;
        /**
         * @internal
         * Defines the actions happening just before the active camera is drawing.
         */
        _beforeCameraDrawStage: Stage<CameraStageAction>;
        /**
         * @internal
         * Defines the actions happening just before a render target is drawing.
         */
        _beforeRenderTargetDrawStage: Stage<RenderTargetStageAction>;
        /**
         * @internal
         * Defines the actions happening just before a rendering group is drawing.
         */
        _beforeRenderingGroupDrawStage: Stage<RenderingGroupStageAction>;
        /**
         * @internal
         * Defines the actions happening just before a mesh is drawing.
         */
        _beforeRenderingMeshStage: Stage<RenderingMeshStageAction>;
        /**
         * @internal
         * Defines the actions happening just after a mesh has been drawn.
         */
        _afterRenderingMeshStage: Stage<RenderingMeshStageAction>;
        /**
         * @internal
         * Defines the actions happening just after a rendering group has been drawn.
         */
        _afterRenderingGroupDrawStage: Stage<RenderingGroupStageAction>;
        /**
         * @internal
         * Defines the actions happening just after the active camera has been drawn.
         */
        _afterCameraDrawStage: Stage<CameraStageAction>;
        /**
         * @internal
         * Defines the actions happening just after the post processing
         */
        _afterCameraPostProcessStage: Stage<CameraStageAction>;
        /**
         * @internal
         * Defines the actions happening just after a render target has been drawn.
         */
        _afterRenderTargetDrawStage: Stage<RenderTargetStageAction>;
        /**
         * Defines the actions happening just after the post processing on a render target
         */
        _afterRenderTargetPostProcessStage: Stage<RenderTargetStageAction>;
        /**
         * @internal
         * Defines the actions happening just after rendering all cameras and computing intersections.
         */
        _afterRenderStage: Stage<SimpleStageAction>;
        /**
         * @internal
         * Defines the actions happening when a pointer move event happens.
         */
        _pointerMoveStage: Stage<PointerMoveStageAction>;
        /**
         * @internal
         * Defines the actions happening when a pointer down event happens.
         */
        _pointerDownStage: Stage<PointerUpDownStageAction>;
        /**
         * @internal
         * Defines the actions happening when a pointer up event happens.
         */
        _pointerUpStage: Stage<PointerUpDownStageAction>;
        /**
         * an optional map from Geometry Id to Geometry index in the 'geometries' array
         */
        private _geometriesByUniqueId;
        /**
         * Creates a new Scene
         * @param engine defines the engine to use to render this scene
         * @param options defines the scene options
         */
        constructor(engine: Engine, options?: SceneOptions);
        /**
         * Gets a string identifying the name of the class
         * @returns "Scene" string
         */
        getClassName(): string;
        private _defaultMeshCandidates;
        /**
         * @internal
         */
        _getDefaultMeshCandidates(): ISmartArrayLike<AbstractMesh>;
        private _defaultSubMeshCandidates;
        /**
         * @internal
         */
        _getDefaultSubMeshCandidates(mesh: AbstractMesh): ISmartArrayLike<SubMesh>;
        /**
         * Sets the default candidate providers for the scene.
         * This sets the getActiveMeshCandidates, getActiveSubMeshCandidates, getIntersectingSubMeshCandidates
         * and getCollidingSubMeshCandidates to their default function
         */
        setDefaultCandidateProviders(): void;
        /**
         * Gets the mesh that is currently under the pointer
         */
        get meshUnderPointer(): Nullable<AbstractMesh>;
        /**
         * Gets or sets the current on-screen X position of the pointer
         */
        get pointerX(): number;
        set pointerX(value: number);
        /**
         * Gets or sets the current on-screen Y position of the pointer
         */
        get pointerY(): number;
        set pointerY(value: number);
        /**
         * Gets the cached material (ie. the latest rendered one)
         * @returns the cached material
         */
        getCachedMaterial(): Nullable<Material>;
        /**
         * Gets the cached effect (ie. the latest rendered one)
         * @returns the cached effect
         */
        getCachedEffect(): Nullable<Effect>;
        /**
         * Gets the cached visibility state (ie. the latest rendered one)
         * @returns the cached visibility state
         */
        getCachedVisibility(): Nullable<number>;
        /**
         * Gets a boolean indicating if the current material / effect / visibility must be bind again
         * @param material defines the current material
         * @param effect defines the current effect
         * @param visibility defines the current visibility state
         * @returns true if one parameter is not cached
         */
        isCachedMaterialInvalid(material: Material, effect: Effect, visibility?: number): boolean;
        /**
         * Gets the engine associated with the scene
         * @returns an Engine
         */
        getEngine(): Engine;
        /**
         * Gets the total number of vertices rendered per frame
         * @returns the total number of vertices rendered per frame
         */
        getTotalVertices(): number;
        /**
         * Gets the performance counter for total vertices
         * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/optimize_your_scene#instrumentation
         */
        get totalVerticesPerfCounter(): PerfCounter;
        /**
         * Gets the total number of active indices rendered per frame (You can deduce the number of rendered triangles by dividing this number by 3)
         * @returns the total number of active indices rendered per frame
         */
        getActiveIndices(): number;
        /**
         * Gets the performance counter for active indices
         * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/optimize_your_scene#instrumentation
         */
        get totalActiveIndicesPerfCounter(): PerfCounter;
        /**
         * Gets the total number of active particles rendered per frame
         * @returns the total number of active particles rendered per frame
         */
        getActiveParticles(): number;
        /**
         * Gets the performance counter for active particles
         * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/optimize_your_scene#instrumentation
         */
        get activeParticlesPerfCounter(): PerfCounter;
        /**
         * Gets the total number of active bones rendered per frame
         * @returns the total number of active bones rendered per frame
         */
        getActiveBones(): number;
        /**
         * Gets the performance counter for active bones
         * @see https://doc.babylonjs.com/features/featuresDeepDive/scene/optimize_your_scene#instrumentation
         */
        get activeBonesPerfCounter(): PerfCounter;
        /**
         * Gets the array of active meshes
         * @returns an array of AbstractMesh
         */
        getActiveMeshes(): SmartArray<AbstractMesh>;
        /**
         * Gets the animation ratio (which is 1.0 is the scene renders at 60fps and 2 if the scene renders at 30fps, etc.)
         * @returns a number
         */
        getAnimationRatio(): number;
        /**
         * Gets an unique Id for the current render phase
         * @returns a number
         */
        getRenderId(): number;
        /**
         * Gets an unique Id for the current frame
         * @returns a number
         */
        getFrameId(): number;
        /** Call this function if you want to manually increment the render Id*/
        incrementRenderId(): void;
        private _createUbo;
        /**
         * Use this method to simulate a pointer move on a mesh
         * The pickResult parameter can be obtained from a scene.pick or scene.pickWithRay
         * @param pickResult pickingInfo of the object wished to simulate pointer event on
         * @param pointerEventInit pointer event state to be used when simulating the pointer event (eg. pointer id for multitouch)
         * @returns the current scene
         */
        simulatePointerMove(pickResult: PickingInfo, pointerEventInit?: PointerEventInit): Scene;
        /**
         * Use this method to simulate a pointer down on a mesh
         * The pickResult parameter can be obtained from a scene.pick or scene.pickWithRay
         * @param pickResult pickingInfo of the object wished to simulate pointer event on
         * @param pointerEventInit pointer event state to be used when simulating the pointer event (eg. pointer id for multitouch)
         * @returns the current scene
         */
        simulatePointerDown(pickResult: PickingInfo, pointerEventInit?: PointerEventInit): Scene;
        /**
         * Use this method to simulate a pointer up on a mesh
         * The pickResult parameter can be obtained from a scene.pick or scene.pickWithRay
         * @param pickResult pickingInfo of the object wished to simulate pointer event on
         * @param pointerEventInit pointer event state to be used when simulating the pointer event (eg. pointer id for multitouch)
         * @param doubleTap indicates that the pointer up event should be considered as part of a double click (false by default)
         * @returns the current scene
         */
        simulatePointerUp(pickResult: PickingInfo, pointerEventInit?: PointerEventInit, doubleTap?: boolean): Scene;
        /**
         * Gets a boolean indicating if the current pointer event is captured (meaning that the scene has already handled the pointer down)
         * @param pointerId defines the pointer id to use in a multi-touch scenario (0 by default)
         * @returns true if the pointer was captured
         */
        isPointerCaptured(pointerId?: number): boolean;
        /**
         * Attach events to the canvas (To handle actionManagers triggers and raise onPointerMove, onPointerDown and onPointerUp
         * @param attachUp defines if you want to attach events to pointerup
         * @param attachDown defines if you want to attach events to pointerdown
         * @param attachMove defines if you want to attach events to pointermove
         */
        attachControl(attachUp?: boolean, attachDown?: boolean, attachMove?: boolean): void;
        /** Detaches all event handlers*/
        detachControl(): void;
        /**
         * This function will check if the scene can be rendered (textures are loaded, shaders are compiled)
         * Delay loaded resources are not taking in account
         * @param checkRenderTargets true to also check that the meshes rendered as part of a render target are ready (default: true)
         * @returns true if all required resources are ready
         */
        isReady(checkRenderTargets?: boolean): boolean;
        /** Resets all cached information relative to material (including effect and visibility) */
        resetCachedMaterial(): void;
        /**
         * Registers a function to be called before every frame render
         * @param func defines the function to register
         */
        registerBeforeRender(func: () => void): void;
        /**
         * Unregisters a function called before every frame render
         * @param func defines the function to unregister
         */
        unregisterBeforeRender(func: () => void): void;
        /**
         * Registers a function to be called after every frame render
         * @param func defines the function to register
         */
        registerAfterRender(func: () => void): void;
        /**
         * Unregisters a function called after every frame render
         * @param func defines the function to unregister
         */
        unregisterAfterRender(func: () => void): void;
        private _executeOnceBeforeRender;
        /**
         * The provided function will run before render once and will be disposed afterwards.
         * A timeout delay can be provided so that the function will be executed in N ms.
         * The timeout is using the browser's native setTimeout so time percision cannot be guaranteed.
         * @param func The function to be executed.
         * @param timeout optional delay in ms
         */
        executeOnceBeforeRender(func: () => void, timeout?: number): void;
        /**
         * This function can help adding any object to the list of data awaited to be ready in order to check for a complete scene loading.
         * @param data defines the object to wait for
         */
        addPendingData(data: any): void;
        /**
         * Remove a pending data from the loading list which has previously been added with addPendingData.
         * @param data defines the object to remove from the pending list
         */
        removePendingData(data: any): void;
        /**
         * Returns the number of items waiting to be loaded
         * @returns the number of items waiting to be loaded
         */
        getWaitingItemsCount(): number;
        /**
         * Returns a boolean indicating if the scene is still loading data
         */
        get isLoading(): boolean;
        /**
         * Registers a function to be executed when the scene is ready
         * @param {Function} func - the function to be executed
         * @param checkRenderTargets true to also check that the meshes rendered as part of a render target are ready (default: false)
         */
        executeWhenReady(func: () => void, checkRenderTargets?: boolean): void;
        /**
         * Returns a promise that resolves when the scene is ready
         * @param checkRenderTargets true to also check that the meshes rendered as part of a render target are ready (default: false)
         * @returns A promise that resolves when the scene is ready
         */
        whenReadyAsync(checkRenderTargets?: boolean): Promise<void>;
        /**
         * @internal
         */
        _checkIsReady(checkRenderTargets?: boolean): void;
        /**
         * Gets all animatable attached to the scene
         */
        get animatables(): Animatable[];
        /**
         * Resets the last animation time frame.
         * Useful to override when animations start running when loading a scene for the first time.
         */
        resetLastAnimationTimeFrame(): void;
        /**
         * Gets the current view matrix
         * @returns a Matrix
         */
        getViewMatrix(): Matrix;
        /**
         * Gets the current projection matrix
         * @returns a Matrix
         */
        getProjectionMatrix(): Matrix;
        /**
         * Gets the current transform matrix
         * @returns a Matrix made of View * Projection
         */
        getTransformMatrix(): Matrix;
        /**
         * Sets the current transform matrix
         * @param viewL defines the View matrix to use
         * @param projectionL defines the Projection matrix to use
         * @param viewR defines the right View matrix to use (if provided)
         * @param projectionR defines the right Projection matrix to use (if provided)
         */
        setTransformMatrix(viewL: Matrix, projectionL: Matrix, viewR?: Matrix, projectionR?: Matrix): void;
        /**
         * Gets the uniform buffer used to store scene data
         * @returns a UniformBuffer
         */
        getSceneUniformBuffer(): UniformBuffer;
        /**
         * Creates a scene UBO
         * @param name name of the uniform buffer (optional, for debugging purpose only)
         * @returns a new ubo
         */
        createSceneUniformBuffer(name?: string): UniformBuffer;
        /**
         * Sets the scene ubo
         * @param ubo the ubo to set for the scene
         */
        setSceneUniformBuffer(ubo: UniformBuffer): void;
        /**
         * Gets an unique (relatively to the current scene) Id
         * @returns an unique number for the scene
         */
        getUniqueId(): number;
        /**
         * Add a mesh to the list of scene's meshes
         * @param newMesh defines the mesh to add
         * @param recursive if all child meshes should also be added to the scene
         */
        addMesh(newMesh: AbstractMesh, recursive?: boolean): void;
        /**
         * Remove a mesh for the list of scene's meshes
         * @param toRemove defines the mesh to remove
         * @param recursive if all child meshes should also be removed from the scene
         * @returns the index where the mesh was in the mesh list
         */
        removeMesh(toRemove: AbstractMesh, recursive?: boolean): number;
        /**
         * Add a transform node to the list of scene's transform nodes
         * @param newTransformNode defines the transform node to add
         */
        addTransformNode(newTransformNode: TransformNode): void;
        /**
         * Remove a transform node for the list of scene's transform nodes
         * @param toRemove defines the transform node to remove
         * @returns the index where the transform node was in the transform node list
         */
        removeTransformNode(toRemove: TransformNode): number;
        /**
         * Remove a skeleton for the list of scene's skeletons
         * @param toRemove defines the skeleton to remove
         * @returns the index where the skeleton was in the skeleton list
         */
        removeSkeleton(toRemove: Skeleton): number;
        /**
         * Remove a morph target for the list of scene's morph targets
         * @param toRemove defines the morph target to remove
         * @returns the index where the morph target was in the morph target list
         */
        removeMorphTargetManager(toRemove: MorphTargetManager): number;
        /**
         * Remove a light for the list of scene's lights
         * @param toRemove defines the light to remove
         * @returns the index where the light was in the light list
         */
        removeLight(toRemove: Light): number;
        /**
         * Remove a camera for the list of scene's cameras
         * @param toRemove defines the camera to remove
         * @returns the index where the camera was in the camera list
         */
        removeCamera(toRemove: Camera): number;
        /**
         * Remove a particle system for the list of scene's particle systems
         * @param toRemove defines the particle system to remove
         * @returns the index where the particle system was in the particle system list
         */
        removeParticleSystem(toRemove: IParticleSystem): number;
        /**
         * Remove a animation for the list of scene's animations
         * @param toRemove defines the animation to remove
         * @returns the index where the animation was in the animation list
         */
        removeAnimation(toRemove: Animation): number;
        /**
         * Will stop the animation of the given target
         * @param target - the target
         * @param animationName - the name of the animation to stop (all animations will be stopped if both this and targetMask are empty)
         * @param targetMask - a function that determines if the animation should be stopped based on its target (all animations will be stopped if both this and animationName are empty)
         */
        stopAnimation(target: any, animationName?: string, targetMask?: (target: any) => boolean): void;
        /**
         * Removes the given animation group from this scene.
         * @param toRemove The animation group to remove
         * @returns The index of the removed animation group
         */
        removeAnimationGroup(toRemove: AnimationGroup): number;
        /**
         * Removes the given multi-material from this scene.
         * @param toRemove The multi-material to remove
         * @returns The index of the removed multi-material
         */
        removeMultiMaterial(toRemove: MultiMaterial): number;
        /**
         * Removes the given material from this scene.
         * @param toRemove The material to remove
         * @returns The index of the removed material
         */
        removeMaterial(toRemove: Material): number;
        /**
         * Removes the given action manager from this scene.
         * @deprecated
         * @param toRemove The action manager to remove
         * @returns The index of the removed action manager
         */
        removeActionManager(toRemove: AbstractActionManager): number;
        /**
         * Removes the given texture from this scene.
         * @param toRemove The texture to remove
         * @returns The index of the removed texture
         */
        removeTexture(toRemove: BaseTexture): number;
        /**
         * Adds the given light to this scene
         * @param newLight The light to add
         */
        addLight(newLight: Light): void;
        /**
         * Sorts the list list based on light priorities
         */
        sortLightsByPriority(): void;
        /**
         * Adds the given camera to this scene
         * @param newCamera The camera to add
         */
        addCamera(newCamera: Camera): void;
        /**
         * Adds the given skeleton to this scene
         * @param newSkeleton The skeleton to add
         */
        addSkeleton(newSkeleton: Skeleton): void;
        /**
         * Adds the given particle system to this scene
         * @param newParticleSystem The particle system to add
         */
        addParticleSystem(newParticleSystem: IParticleSystem): void;
        /**
         * Adds the given animation to this scene
         * @param newAnimation The animation to add
         */
        addAnimation(newAnimation: Animation): void;
        /**
         * Adds the given animation group to this scene.
         * @param newAnimationGroup The animation group to add
         */
        addAnimationGroup(newAnimationGroup: AnimationGroup): void;
        /**
         * Adds the given multi-material to this scene
         * @param newMultiMaterial The multi-material to add
         */
        addMultiMaterial(newMultiMaterial: MultiMaterial): void;
        /**
         * Adds the given material to this scene
         * @param newMaterial The material to add
         */
        addMaterial(newMaterial: Material): void;
        /**
         * Adds the given morph target to this scene
         * @param newMorphTargetManager The morph target to add
         */
        addMorphTargetManager(newMorphTargetManager: MorphTargetManager): void;
        /**
         * Adds the given geometry to this scene
         * @param newGeometry The geometry to add
         */
        addGeometry(newGeometry: Geometry): void;
        /**
         * Adds the given action manager to this scene
         * @deprecated
         * @param newActionManager The action manager to add
         */
        addActionManager(newActionManager: AbstractActionManager): void;
        /**
         * Adds the given texture to this scene.
         * @param newTexture The texture to add
         */
        addTexture(newTexture: BaseTexture): void;
        /**
         * Switch active camera
         * @param newCamera defines the new active camera
         * @param attachControl defines if attachControl must be called for the new active camera (default: true)
         */
        switchActiveCamera(newCamera: Camera, attachControl?: boolean): void;
        /**
         * sets the active camera of the scene using its Id
         * @param id defines the camera's Id
         * @returns the new active camera or null if none found.
         */
        setActiveCameraById(id: string): Nullable<Camera>;
        /**
         * sets the active camera of the scene using its name
         * @param name defines the camera's name
         * @returns the new active camera or null if none found.
         */
        setActiveCameraByName(name: string): Nullable<Camera>;
        /**
         * get an animation group using its name
         * @param name defines the material's name
         * @returns the animation group or null if none found.
         */
        getAnimationGroupByName(name: string): Nullable<AnimationGroup>;
        private _getMaterial;
        /**
         * Get a material using its unique id
         * @param uniqueId defines the material's unique id
         * @param allowMultiMaterials determines whether multimaterials should be considered
         * @returns the material or null if none found.
         */
        getMaterialByUniqueID(uniqueId: number, allowMultiMaterials?: boolean): Nullable<Material>;
        /**
         * get a material using its id
         * @param id defines the material's Id
         * @param allowMultiMaterials determines whether multimaterials should be considered
         * @returns the material or null if none found.
         */
        getMaterialById(id: string, allowMultiMaterials?: boolean): Nullable<Material>;
        /**
         * Gets a material using its name
         * @param name defines the material's name
         * @param allowMultiMaterials determines whether multimaterials should be considered
         * @returns the material or null if none found.
         */
        getMaterialByName(name: string, allowMultiMaterials?: boolean): Nullable<Material>;
        /**
         * Gets a last added material using a given id
         * @param id defines the material's id
         * @param allowMultiMaterials determines whether multimaterials should be considered
         * @returns the last material with the given id or null if none found.
         */
        getLastMaterialById(id: string, allowMultiMaterials?: boolean): Nullable<Material>;
        /**
         * Get a texture using its unique id
         * @param uniqueId defines the texture's unique id
         * @returns the texture or null if none found.
         */
        getTextureByUniqueId(uniqueId: number): Nullable<BaseTexture>;
        /**
         * Gets a texture using its name
         * @param name defines the texture's name
         * @returns the texture or null if none found.
         */
        getTextureByName(name: string): Nullable<BaseTexture>;
        /**
         * Gets a camera using its Id
         * @param id defines the Id to look for
         * @returns the camera or null if not found
         */
        getCameraById(id: string): Nullable<Camera>;
        /**
         * Gets a camera using its unique Id
         * @param uniqueId defines the unique Id to look for
         * @returns the camera or null if not found
         */
        getCameraByUniqueId(uniqueId: number): Nullable<Camera>;
        /**
         * Gets a camera using its name
         * @param name defines the camera's name
         * @returns the camera or null if none found.
         */
        getCameraByName(name: string): Nullable<Camera>;
        /**
         * Gets a bone using its Id
         * @param id defines the bone's Id
         * @returns the bone or null if not found
         */
        getBoneById(id: string): Nullable<Bone>;
        /**
         * Gets a bone using its id
         * @param name defines the bone's name
         * @returns the bone or null if not found
         */
        getBoneByName(name: string): Nullable<Bone>;
        /**
         * Gets a light node using its name
         * @param name defines the the light's name
         * @returns the light or null if none found.
         */
        getLightByName(name: string): Nullable<Light>;
        /**
         * Gets a light node using its Id
         * @param id defines the light's Id
         * @returns the light or null if none found.
         */
        getLightById(id: string): Nullable<Light>;
        /**
         * Gets a light node using its scene-generated unique Id
         * @param uniqueId defines the light's unique Id
         * @returns the light or null if none found.
         */
        getLightByUniqueId(uniqueId: number): Nullable<Light>;
        /**
         * Gets a particle system by Id
         * @param id defines the particle system Id
         * @returns the corresponding system or null if none found
         */
        getParticleSystemById(id: string): Nullable<IParticleSystem>;
        /**
         * Gets a geometry using its Id
         * @param id defines the geometry's Id
         * @returns the geometry or null if none found.
         */
        getGeometryById(id: string): Nullable<Geometry>;
        private _getGeometryByUniqueId;
        /**
         * Add a new geometry to this scene
         * @param geometry defines the geometry to be added to the scene.
         * @param force defines if the geometry must be pushed even if a geometry with this id already exists
         * @returns a boolean defining if the geometry was added or not
         */
        pushGeometry(geometry: Geometry, force?: boolean): boolean;
        /**
         * Removes an existing geometry
         * @param geometry defines the geometry to be removed from the scene
         * @returns a boolean defining if the geometry was removed or not
         */
        removeGeometry(geometry: Geometry): boolean;
        /**
         * Gets the list of geometries attached to the scene
         * @returns an array of Geometry
         */
        getGeometries(): Geometry[];
        /**
         * Gets the first added mesh found of a given Id
         * @param id defines the Id to search for
         * @returns the mesh found or null if not found at all
         */
        getMeshById(id: string): Nullable<AbstractMesh>;
        /**
         * Gets a list of meshes using their Id
         * @param id defines the Id to search for
         * @returns a list of meshes
         */
        getMeshesById(id: string): Array<AbstractMesh>;
        /**
         * Gets the first added transform node found of a given Id
         * @param id defines the Id to search for
         * @returns the found transform node or null if not found at all.
         */
        getTransformNodeById(id: string): Nullable<TransformNode>;
        /**
         * Gets a transform node with its auto-generated unique Id
         * @param uniqueId defines the unique Id to search for
         * @returns the found transform node or null if not found at all.
         */
        getTransformNodeByUniqueId(uniqueId: number): Nullable<TransformNode>;
        /**
         * Gets a list of transform nodes using their Id
         * @param id defines the Id to search for
         * @returns a list of transform nodes
         */
        getTransformNodesById(id: string): Array<TransformNode>;
        /**
         * Gets a mesh with its auto-generated unique Id
         * @param uniqueId defines the unique Id to search for
         * @returns the found mesh or null if not found at all.
         */
        getMeshByUniqueId(uniqueId: number): Nullable<AbstractMesh>;
        /**
         * Gets a the last added mesh using a given Id
         * @param id defines the Id to search for
         * @returns the found mesh or null if not found at all.
         */
        getLastMeshById(id: string): Nullable<AbstractMesh>;
        /**
         * Gets a the last added node (Mesh, Camera, Light) using a given Id
         * @param id defines the Id to search for
         * @returns the found node or null if not found at all
         */
        getLastEntryById(id: string): Nullable<Node>;
        /**
         * Gets a node (Mesh, Camera, Light) using a given Id
         * @param id defines the Id to search for
         * @returns the found node or null if not found at all
         */
        getNodeById(id: string): Nullable<Node>;
        /**
         * Gets a node (Mesh, Camera, Light) using a given name
         * @param name defines the name to search for
         * @returns the found node or null if not found at all.
         */
        getNodeByName(name: string): Nullable<Node>;
        /**
         * Gets a mesh using a given name
         * @param name defines the name to search for
         * @returns the found mesh or null if not found at all.
         */
        getMeshByName(name: string): Nullable<AbstractMesh>;
        /**
         * Gets a transform node using a given name
         * @param name defines the name to search for
         * @returns the found transform node or null if not found at all.
         */
        getTransformNodeByName(name: string): Nullable<TransformNode>;
        /**
         * Gets a skeleton using a given Id (if many are found, this function will pick the last one)
         * @param id defines the Id to search for
         * @returns the found skeleton or null if not found at all.
         */
        getLastSkeletonById(id: string): Nullable<Skeleton>;
        /**
         * Gets a skeleton using a given auto generated unique id
         * @param  uniqueId defines the unique id to search for
         * @returns the found skeleton or null if not found at all.
         */
        getSkeletonByUniqueId(uniqueId: number): Nullable<Skeleton>;
        /**
         * Gets a skeleton using a given id (if many are found, this function will pick the first one)
         * @param id defines the id to search for
         * @returns the found skeleton or null if not found at all.
         */
        getSkeletonById(id: string): Nullable<Skeleton>;
        /**
         * Gets a skeleton using a given name
         * @param name defines the name to search for
         * @returns the found skeleton or null if not found at all.
         */
        getSkeletonByName(name: string): Nullable<Skeleton>;
        /**
         * Gets a morph target manager  using a given id (if many are found, this function will pick the last one)
         * @param id defines the id to search for
         * @returns the found morph target manager or null if not found at all.
         */
        getMorphTargetManagerById(id: number): Nullable<MorphTargetManager>;
        /**
         * Gets a morph target using a given id (if many are found, this function will pick the first one)
         * @param id defines the id to search for
         * @returns the found morph target or null if not found at all.
         */
        getMorphTargetById(id: string): Nullable<MorphTarget>;
        /**
         * Gets a morph target using a given name (if many are found, this function will pick the first one)
         * @param name defines the name to search for
         * @returns the found morph target or null if not found at all.
         */
        getMorphTargetByName(name: string): Nullable<MorphTarget>;
        /**
         * Gets a post process using a given name (if many are found, this function will pick the first one)
         * @param name defines the name to search for
         * @returns the found post process or null if not found at all.
         */
        getPostProcessByName(name: string): Nullable<PostProcess>;
        /**
         * Gets a boolean indicating if the given mesh is active
         * @param mesh defines the mesh to look for
         * @returns true if the mesh is in the active list
         */
        isActiveMesh(mesh: AbstractMesh): boolean;
        /**
         * Return a unique id as a string which can serve as an identifier for the scene
         */
        get uid(): string;
        /**
         * Add an externally attached data from its key.
         * This method call will fail and return false, if such key already exists.
         * If you don't care and just want to get the data no matter what, use the more convenient getOrAddExternalDataWithFactory() method.
         * @param key the unique key that identifies the data
         * @param data the data object to associate to the key for this Engine instance
         * @returns true if no such key were already present and the data was added successfully, false otherwise
         */
        addExternalData<T extends Object>(key: string, data: T): boolean;
        /**
         * Get an externally attached data from its key
         * @param key the unique key that identifies the data
         * @returns the associated data, if present (can be null), or undefined if not present
         */
        getExternalData<T>(key: string): Nullable<T>;
        /**
         * Get an externally attached data from its key, create it using a factory if it's not already present
         * @param key the unique key that identifies the data
         * @param factory the factory that will be called to create the instance if and only if it doesn't exists
         * @returns the associated data, can be null if the factory returned null.
         */
        getOrAddExternalDataWithFactory<T extends Object>(key: string, factory: (k: string) => T): T;
        /**
         * Remove an externally attached data from the Engine instance
         * @param key the unique key that identifies the data
         * @returns true if the data was successfully removed, false if it doesn't exist
         */
        removeExternalData(key: string): boolean;
        private _evaluateSubMesh;
        /**
         * Clear the processed materials smart array preventing retention point in material dispose.
         */
        freeProcessedMaterials(): void;
        private _preventFreeActiveMeshesAndRenderingGroups;
        /** Gets or sets a boolean blocking all the calls to freeActiveMeshes and freeRenderingGroups
         * It can be used in order to prevent going through methods freeRenderingGroups and freeActiveMeshes several times to improve performance
         * when disposing several meshes in a row or a hierarchy of meshes.
         * When used, it is the responsibility of the user to blockfreeActiveMeshesAndRenderingGroups back to false.
         */
        get blockfreeActiveMeshesAndRenderingGroups(): boolean;
        set blockfreeActiveMeshesAndRenderingGroups(value: boolean);
        /**
         * Clear the active meshes smart array preventing retention point in mesh dispose.
         */
        freeActiveMeshes(): void;
        /**
         * Clear the info related to rendering groups preventing retention points during dispose.
         */
        freeRenderingGroups(): void;
        /** @internal */
        _isInIntermediateRendering(): boolean;
        /**
         * Lambda returning the list of potentially active meshes.
         */
        getActiveMeshCandidates: () => ISmartArrayLike<AbstractMesh>;
        /**
         * Lambda returning the list of potentially active sub meshes.
         */
        getActiveSubMeshCandidates: (mesh: AbstractMesh) => ISmartArrayLike<SubMesh>;
        /**
         * Lambda returning the list of potentially intersecting sub meshes.
         */
        getIntersectingSubMeshCandidates: (mesh: AbstractMesh, localRay: Ray) => ISmartArrayLike<SubMesh>;
        /**
         * Lambda returning the list of potentially colliding sub meshes.
         */
        getCollidingSubMeshCandidates: (mesh: AbstractMesh, collider: Collider) => ISmartArrayLike<SubMesh>;
        /** @internal */
        _activeMeshesFrozen: boolean;
        /** @internal */
        _activeMeshesFrozenButKeepClipping: boolean;
        private _skipEvaluateActiveMeshesCompletely;
        /**
         * Use this function to stop evaluating active meshes. The current list will be keep alive between frames
         * @param skipEvaluateActiveMeshes defines an optional boolean indicating that the evaluate active meshes step must be completely skipped
         * @param onSuccess optional success callback
         * @param onError optional error callback
         * @param freezeMeshes defines if meshes should be frozen (true by default)
         * @param keepFrustumCulling defines if you want to keep running the frustum clipping (false by default)
         * @returns the current scene
         */
        freezeActiveMeshes(skipEvaluateActiveMeshes?: boolean, onSuccess?: () => void, onError?: (message: string) => void, freezeMeshes?: boolean, keepFrustumCulling?: boolean): Scene;
        /**
         * Use this function to restart evaluating active meshes on every frame
         * @returns the current scene
         */
        unfreezeActiveMeshes(): Scene;
        private _executeActiveContainerCleanup;
        private _evaluateActiveMeshes;
        private _activeMesh;
        /**
         * Update the transform matrix to update from the current active camera
         * @param force defines a boolean used to force the update even if cache is up to date
         */
        updateTransformMatrix(force?: boolean): void;
        private _bindFrameBuffer;
        private _clearFrameBuffer;
        /** @internal */
        _allowPostProcessClearColor: boolean;
        /**
         * @internal
         */
        _renderForCamera(camera: Camera, rigParent?: Camera, bindFrameBuffer?: boolean): void;
        private _processSubCameras;
        private _checkIntersections;
        /**
         * @internal
         */
        _advancePhysicsEngineStep(step: number): void;
        /**
         * User updatable function that will return a deterministic frame time when engine is in deterministic lock step mode
         */
        getDeterministicFrameTime: () => number;
        /** @internal */
        _animate(): void;
        /** Execute all animations (for a frame) */
        animate(): void;
        private _clear;
        private _checkCameraRenderTarget;
        /**
         * Resets the draw wrappers cache of all meshes
         * @param passId If provided, releases only the draw wrapper corresponding to this render pass id
         */
        resetDrawCache(passId?: number): void;
        /**
         * Render the scene
         * @param updateCameras defines a boolean indicating if cameras must update according to their inputs (true by default)
         * @param ignoreAnimations defines a boolean indicating if animations should not be executed (false by default)
         */
        render(updateCameras?: boolean, ignoreAnimations?: boolean): void;
        /**
         * Freeze all materials
         * A frozen material will not be updatable but should be faster to render
         * Note: multimaterials will not be frozen, but their submaterials will
         */
        freezeMaterials(): void;
        /**
         * Unfreeze all materials
         * A frozen material will not be updatable but should be faster to render
         */
        unfreezeMaterials(): void;
        /**
         * Releases all held resources
         */
        dispose(): void;
        private _disposeList;
        /**
         * Gets if the scene is already disposed
         */
        get isDisposed(): boolean;
        /**
         * Call this function to reduce memory footprint of the scene.
         * Vertex buffers will not store CPU data anymore (this will prevent picking, collisions or physics to work correctly)
         */
        clearCachedVertexData(): void;
        /**
         * This function will remove the local cached buffer data from texture.
         * It will save memory but will prevent the texture from being rebuilt
         */
        cleanCachedTextureBuffer(): void;
        /**
         * Get the world extend vectors with an optional filter
         *
         * @param filterPredicate the predicate - which meshes should be included when calculating the world size
         * @returns {{ min: Vector3; max: Vector3 }} min and max vectors
         */
        getWorldExtends(filterPredicate?: (mesh: AbstractMesh) => boolean): {
            min: Vector3;
            max: Vector3;
        };
        /**
         * Creates a ray that can be used to pick in the scene
         * @param x defines the x coordinate of the origin (on-screen)
         * @param y defines the y coordinate of the origin (on-screen)
         * @param world defines the world matrix to use if you want to pick in object space (instead of world space)
         * @param camera defines the camera to use for the picking
         * @param cameraViewSpace defines if picking will be done in view space (false by default)
         * @returns a Ray
         */
        createPickingRay(x: number, y: number, world: Nullable<Matrix>, camera: Nullable<Camera>, cameraViewSpace?: boolean): Ray;
        /**
         * Creates a ray that can be used to pick in the scene
         * @param x defines the x coordinate of the origin (on-screen)
         * @param y defines the y coordinate of the origin (on-screen)
         * @param world defines the world matrix to use if you want to pick in object space (instead of world space)
         * @param result defines the ray where to store the picking ray
         * @param camera defines the camera to use for the picking
         * @param cameraViewSpace defines if picking will be done in view space (false by default)
         * @param enableDistantPicking defines if picking should handle large values for mesh position/scaling (false by default)
         * @returns the current scene
         */
        createPickingRayToRef(x: number, y: number, world: Nullable<Matrix>, result: Ray, camera: Nullable<Camera>, cameraViewSpace?: boolean, enableDistantPicking?: boolean): Scene;
        /**
         * Creates a ray that can be used to pick in the scene
         * @param x defines the x coordinate of the origin (on-screen)
         * @param y defines the y coordinate of the origin (on-screen)
         * @param camera defines the camera to use for the picking
         * @returns a Ray
         */
        createPickingRayInCameraSpace(x: number, y: number, camera?: Camera): Ray;
        /**
         * Creates a ray that can be used to pick in the scene
         * @param x defines the x coordinate of the origin (on-screen)
         * @param y defines the y coordinate of the origin (on-screen)
         * @param result defines the ray where to store the picking ray
         * @param camera defines the camera to use for the picking
         * @returns the current scene
         */
        createPickingRayInCameraSpaceToRef(x: number, y: number, result: Ray, camera?: Camera): Scene;
        /** @internal */
        get _pickingAvailable(): boolean;
        /** @internal */
        _registeredActions: number;
        /** Launch a ray to try to pick a mesh in the scene
         * @param x position on screen
         * @param y position on screen
         * @param predicate Predicate function used to determine eligible meshes. Can be set to null. In this case, a mesh must be enabled, visible and with isPickable set to true
         * @param fastCheck defines if the first intersection will be used (and not the closest)
         * @param camera to use for computing the picking ray. Can be set to null. In this case, the scene.activeCamera will be used
         * @param trianglePredicate defines an optional predicate used to select faces when a mesh intersection is detected
         * @returns a PickingInfo
         */
        pick(x: number, y: number, predicate?: (mesh: AbstractMesh) => boolean, fastCheck?: boolean, camera?: Nullable<Camera>, trianglePredicate?: TrianglePickingPredicate): PickingInfo;
        /** Launch a ray to try to pick a mesh in the scene using only bounding information of the main mesh (not using submeshes)
         * @param x position on screen
         * @param y position on screen
         * @param predicate Predicate function used to determine eligible meshes. Can be set to null. In this case, a mesh must be enabled, visible and with isPickable set to true
         * @param fastCheck defines if the first intersection will be used (and not the closest)
         * @param camera to use for computing the picking ray. Can be set to null. In this case, the scene.activeCamera will be used
         * @returns a PickingInfo (Please note that some info will not be set like distance, bv, bu and everything that cannot be capture by only using bounding infos)
         */
        pickWithBoundingInfo(x: number, y: number, predicate?: (mesh: AbstractMesh) => boolean, fastCheck?: boolean, camera?: Nullable<Camera>): Nullable<PickingInfo>;
        /** Use the given ray to pick a mesh in the scene
         * @param ray The ray to use to pick meshes
         * @param predicate Predicate function used to determine eligible meshes. Can be set to null. In this case, a mesh must have isPickable set to true
         * @param fastCheck defines if the first intersection will be used (and not the closest)
         * @param trianglePredicate defines an optional predicate used to select faces when a mesh intersection is detected
         * @returns a PickingInfo
         */
        pickWithRay(ray: Ray, predicate?: (mesh: AbstractMesh) => boolean, fastCheck?: boolean, trianglePredicate?: TrianglePickingPredicate): Nullable<PickingInfo>;
        /**
         * Launch a ray to try to pick a mesh in the scene
         * @param x X position on screen
         * @param y Y position on screen
         * @param predicate Predicate function used to determine eligible meshes. Can be set to null. In this case, a mesh must be enabled, visible and with isPickable set to true
         * @param camera camera to use for computing the picking ray. Can be set to null. In this case, the scene.activeCamera will be used
         * @param trianglePredicate defines an optional predicate used to select faces when a mesh intersection is detected
         * @returns an array of PickingInfo
         */
        multiPick(x: number, y: number, predicate?: (mesh: AbstractMesh) => boolean, camera?: Camera, trianglePredicate?: TrianglePickingPredicate): Nullable<PickingInfo[]>;
        /**
         * Launch a ray to try to pick a mesh in the scene
         * @param ray Ray to use
         * @param predicate Predicate function used to determine eligible meshes. Can be set to null. In this case, a mesh must be enabled, visible and with isPickable set to true
         * @param trianglePredicate defines an optional predicate used to select faces when a mesh intersection is detected
         * @returns an array of PickingInfo
         */
        multiPickWithRay(ray: Ray, predicate?: (mesh: AbstractMesh) => boolean, trianglePredicate?: TrianglePickingPredicate): Nullable<PickingInfo[]>;
        /**
         * Force the value of meshUnderPointer
         * @param mesh defines the mesh to use
         * @param pointerId optional pointer id when using more than one pointer
         * @param pickResult optional pickingInfo data used to find mesh
         */
        setPointerOverMesh(mesh: Nullable<AbstractMesh>, pointerId?: number, pickResult?: Nullable<PickingInfo>): void;
        /**
         * Gets the mesh under the pointer
         * @returns a Mesh or null if no mesh is under the pointer
         */
        getPointerOverMesh(): Nullable<AbstractMesh>;
        /** @internal */
        _rebuildGeometries(): void;
        /** @internal */
        _rebuildTextures(): void;
        private _getByTags;
        /**
         * Get a list of meshes by tags
         * @param tagsQuery defines the tags query to use
         * @param forEach defines a predicate used to filter results
         * @returns an array of Mesh
         */
        getMeshesByTags(tagsQuery: string, forEach?: (mesh: AbstractMesh) => void): Mesh[];
        /**
         * Get a list of cameras by tags
         * @param tagsQuery defines the tags query to use
         * @param forEach defines a predicate used to filter results
         * @returns an array of Camera
         */
        getCamerasByTags(tagsQuery: string, forEach?: (camera: Camera) => void): Camera[];
        /**
         * Get a list of lights by tags
         * @param tagsQuery defines the tags query to use
         * @param forEach defines a predicate used to filter results
         * @returns an array of Light
         */
        getLightsByTags(tagsQuery: string, forEach?: (light: Light) => void): Light[];
        /**
         * Get a list of materials by tags
         * @param tagsQuery defines the tags query to use
         * @param forEach defines a predicate used to filter results
         * @returns an array of Material
         */
        getMaterialByTags(tagsQuery: string, forEach?: (material: Material) => void): Material[];
        /**
         * Get a list of transform nodes by tags
         * @param tagsQuery defines the tags query to use
         * @param forEach defines a predicate used to filter results
         * @returns an array of TransformNode
         */
        getTransformNodesByTags(tagsQuery: string, forEach?: (transform: TransformNode) => void): TransformNode[];
        /**
         * Overrides the default sort function applied in the rendering group to prepare the meshes.
         * This allowed control for front to back rendering or reversly depending of the special needs.
         *
         * @param renderingGroupId The rendering group id corresponding to its index
         * @param opaqueSortCompareFn The opaque queue comparison function use to sort.
         * @param alphaTestSortCompareFn The alpha test queue comparison function use to sort.
         * @param transparentSortCompareFn The transparent queue comparison function use to sort.
         */
        setRenderingOrder(renderingGroupId: number, opaqueSortCompareFn?: Nullable<(a: SubMesh, b: SubMesh) => number>, alphaTestSortCompareFn?: Nullable<(a: SubMesh, b: SubMesh) => number>, transparentSortCompareFn?: Nullable<(a: SubMesh, b: SubMesh) => number>): void;
        /**
         * Specifies whether or not the stencil and depth buffer are cleared between two rendering groups.
         *
         * @param renderingGroupId The rendering group id corresponding to its index
         * @param autoClearDepthStencil Automatically clears depth and stencil between groups if true.
         * @param depth Automatically clears depth between groups if true and autoClear is true.
         * @param stencil Automatically clears stencil between groups if true and autoClear is true.
         */
        setRenderingAutoClearDepthStencil(renderingGroupId: number, autoClearDepthStencil: boolean, depth?: boolean, stencil?: boolean): void;
        /**
         * Gets the current auto clear configuration for one rendering group of the rendering
         * manager.
         * @param index the rendering group index to get the information for
         * @returns The auto clear setup for the requested rendering group
         */
        getAutoClearDepthStencilSetup(index: number): IRenderingManagerAutoClearSetup;
        private _blockMaterialDirtyMechanism;
        /** Gets or sets a boolean blocking all the calls to markAllMaterialsAsDirty (ie. the materials won't be updated if they are out of sync) */
        get blockMaterialDirtyMechanism(): boolean;
        set blockMaterialDirtyMechanism(value: boolean);
        /**
         * Will flag all materials as dirty to trigger new shader compilation
         * @param flag defines the flag used to specify which material part must be marked as dirty
         * @param predicate If not null, it will be used to specify if a material has to be marked as dirty
         */
        markAllMaterialsAsDirty(flag: number, predicate?: (mat: Material) => boolean): void;
        /**
         * @internal
         */
        _loadFile(fileOrUrl: File | string, onSuccess: (data: string | ArrayBuffer, responseURL?: string) => void, onProgress?: (ev: ProgressEvent) => void, useOfflineSupport?: boolean, useArrayBuffer?: boolean, onError?: (request?: WebRequest, exception?: LoadFileError) => void, onOpened?: (request: WebRequest) => void): IFileRequest;
        /**
         * @internal
         */
        _loadFileAsync(fileOrUrl: File | string, onProgress?: (data: any) => void, useOfflineSupport?: boolean, useArrayBuffer?: boolean, onOpened?: (request: WebRequest) => void): Promise<string | ArrayBuffer>;
        /**
         * @internal
         */
        _requestFile(url: string, onSuccess: (data: string | ArrayBuffer, request?: WebRequest) => void, onProgress?: (ev: ProgressEvent) => void, useOfflineSupport?: boolean, useArrayBuffer?: boolean, onError?: (error: RequestFileError) => void, onOpened?: (request: WebRequest) => void): IFileRequest;
        /**
         * @internal
         */
        _requestFileAsync(url: string, onProgress?: (ev: ProgressEvent) => void, useOfflineSupport?: boolean, useArrayBuffer?: boolean, onOpened?: (request: WebRequest) => void): Promise<string | ArrayBuffer>;
        /**
         * @internal
         */
        _readFile(file: File, onSuccess: (data: string | ArrayBuffer) => void, onProgress?: (ev: ProgressEvent) => any, useArrayBuffer?: boolean, onError?: (error: ReadFileError) => void): IFileRequest;
        /**
         * @internal
         */
        _readFileAsync(file: File, onProgress?: (ev: ProgressEvent) => any, useArrayBuffer?: boolean): Promise<string | ArrayBuffer>;
        /**
         * Internal perfCollector instance used for sharing between inspector and playground.
         * Marked as protected to allow sharing between prototype extensions, but disallow access at toplevel.
         */
        protected _perfCollector: Nullable<PerformanceViewerCollector>;
        /**
         * This method gets the performance collector belonging to the scene, which is generally shared with the inspector.
         * @returns the perf collector belonging to the scene.
         */
        getPerfCollector(): PerformanceViewerCollector;
    }
        interface Scene {
            /**
             * Sets the active camera of the scene using its Id
             * @param id defines the camera's Id
             * @returns the new active camera or null if none found.
             * @deprecated Please use setActiveCameraById instead
             */
            setActiveCameraByID(id: string): Nullable<Camera>;
            /**
             * Get a material using its id
             * @param id defines the material's Id
             * @returns the material or null if none found.
             * @deprecated Please use getMaterialById instead
             */
            getMaterialByID(id: string): Nullable<Material>;
            /**
             * Gets a the last added material using a given id
             * @param id defines the material's Id
             * @returns the last material with the given id or null if none found.
             * @deprecated Please use getLastMaterialById instead
             */
            getLastMaterialByID(id: string): Nullable<Material>;
            /**
             * Get a texture using its unique id
             * @param uniqueId defines the texture's unique id
             * @returns the texture or null if none found.
             * @deprecated Please use getTextureByUniqueId instead
             */
            getTextureByUniqueID(uniqueId: number): Nullable<BaseTexture>;
            /**
             * Gets a camera using its Id
             * @param id defines the Id to look for
             * @returns the camera or null if not found
             * @deprecated Please use getCameraById instead
             */
            getCameraByID(id: string): Nullable<Camera>;
            /**
             * Gets a camera using its unique Id
             * @param uniqueId defines the unique Id to look for
             * @returns the camera or null if not found
             * @deprecated Please use getCameraByUniqueId instead
             */
            getCameraByUniqueID(uniqueId: number): Nullable<Camera>;
            /**
             * Gets a bone using its Id
             * @param id defines the bone's Id
             * @returns the bone or null if not found
             * @deprecated Please use getBoneById instead
             */
            getBoneByID(id: string): Nullable<Bone>;
            /**
             * Gets a light node using its Id
             * @param id defines the light's Id
             * @returns the light or null if none found.
             * @deprecated Please use getLightById instead
             */
            getLightByID(id: string): Nullable<Light>;
            /**
             * Gets a light node using its scene-generated unique Id
             * @param uniqueId defines the light's unique Id
             * @returns the light or null if none found.
             * @deprecated Please use getLightByUniqueId instead
             */
            getLightByUniqueID(uniqueId: number): Nullable<Light>;
            /**
             * Gets a particle system by Id
             * @param id defines the particle system Id
             * @returns the corresponding system or null if none found
             * @deprecated Please use getParticleSystemById instead
             */
            getParticleSystemByID(id: string): Nullable<IParticleSystem>;
            /**
             * Gets a geometry using its Id
             * @param id defines the geometry's Id
             * @returns the geometry or null if none found.
             * @deprecated Please use getGeometryById instead
             */
            getGeometryByID(id: string): Nullable<Geometry>;
            /**
             * Gets the first added mesh found of a given Id
             * @param id defines the Id to search for
             * @returns the mesh found or null if not found at all
             * @deprecated Please use getMeshById instead
             */
            getMeshByID(id: string): Nullable<AbstractMesh>;
            /**
             * Gets a mesh with its auto-generated unique Id
             * @param uniqueId defines the unique Id to search for
             * @returns the found mesh or null if not found at all.
             * @deprecated Please use getMeshByUniqueId instead
             */
            getMeshByUniqueID(uniqueId: number): Nullable<AbstractMesh>;
            /**
             * Gets a the last added mesh using a given Id
             * @param id defines the Id to search for
             * @returns the found mesh or null if not found at all.
             * @deprecated Please use getLastMeshById instead
             */
            getLastMeshByID(id: string): Nullable<AbstractMesh>;
            /**
             * Gets a list of meshes using their Id
             * @param id defines the Id to search for
             * @returns a list of meshes
             * @deprecated Please use getMeshesById instead
             */
            getMeshesByID(id: string): Array<AbstractMesh>;
            /**
             * Gets the first added transform node found of a given Id
             * @param id defines the Id to search for
             * @returns the found transform node or null if not found at all.
             * @deprecated Please use getTransformNodeById instead
             */
            getTransformNodeByID(id: string): Nullable<TransformNode>;
            /**
             * Gets a transform node with its auto-generated unique Id
             * @param uniqueId defines the unique Id to search for
             * @returns the found transform node or null if not found at all.
             * @deprecated Please use getTransformNodeByUniqueId instead
             */
            getTransformNodeByUniqueID(uniqueId: number): Nullable<TransformNode>;
            /**
             * Gets a list of transform nodes using their Id
             * @param id defines the Id to search for
             * @returns a list of transform nodes
             * @deprecated Please use getTransformNodesById instead
             */
            getTransformNodesByID(id: string): Array<TransformNode>;
            /**
             * Gets a node (Mesh, Camera, Light) using a given Id
             * @param id defines the Id to search for
             * @returns the found node or null if not found at all
             * @deprecated Please use getNodeById instead
             */
            getNodeByID(id: string): Nullable<Node>;
            /**
             * Gets a the last added node (Mesh, Camera, Light) using a given Id
             * @param id defines the Id to search for
             * @returns the found node or null if not found at all
             * @deprecated Please use getLastEntryById instead
             */
            getLastEntryByID(id: string): Nullable<Node>;
            /**
             * Gets a skeleton using a given Id (if many are found, this function will pick the last one)
             * @param id defines the Id to search for
             * @returns the found skeleton or null if not found at all.
             * @deprecated Please use getLastSkeletonById instead
             */
            getLastSkeletonByID(id: string): Nullable<Skeleton>;
        }


    /**
     * Groups all the scene component constants in one place to ease maintenance.
     * @internal
     */
    export class SceneComponentConstants {
        static readonly NAME_EFFECTLAYER = "EffectLayer";
        static readonly NAME_LAYER = "Layer";
        static readonly NAME_LENSFLARESYSTEM = "LensFlareSystem";
        static readonly NAME_BOUNDINGBOXRENDERER = "BoundingBoxRenderer";
        static readonly NAME_PARTICLESYSTEM = "ParticleSystem";
        static readonly NAME_GAMEPAD = "Gamepad";
        static readonly NAME_SIMPLIFICATIONQUEUE = "SimplificationQueue";
        static readonly NAME_GEOMETRYBUFFERRENDERER = "GeometryBufferRenderer";
        static readonly NAME_PREPASSRENDERER = "PrePassRenderer";
        static readonly NAME_DEPTHRENDERER = "DepthRenderer";
        static readonly NAME_DEPTHPEELINGRENDERER = "DepthPeelingRenderer";
        static readonly NAME_POSTPROCESSRENDERPIPELINEMANAGER = "PostProcessRenderPipelineManager";
        static readonly NAME_SPRITE = "Sprite";
        static readonly NAME_SUBSURFACE = "SubSurface";
        static readonly NAME_OUTLINERENDERER = "Outline";
        static readonly NAME_PROCEDURALTEXTURE = "ProceduralTexture";
        static readonly NAME_SHADOWGENERATOR = "ShadowGenerator";
        static readonly NAME_OCTREE = "Octree";
        static readonly NAME_PHYSICSENGINE = "PhysicsEngine";
        static readonly NAME_AUDIO = "Audio";
        static readonly NAME_FLUIDRENDERER = "FluidRenderer";
        static readonly STEP_ISREADYFORMESH_EFFECTLAYER = 0;
        static readonly STEP_BEFOREEVALUATEACTIVEMESH_BOUNDINGBOXRENDERER = 0;
        static readonly STEP_EVALUATESUBMESH_BOUNDINGBOXRENDERER = 0;
        static readonly STEP_PREACTIVEMESH_BOUNDINGBOXRENDERER = 0;
        static readonly STEP_CAMERADRAWRENDERTARGET_EFFECTLAYER = 1;
        static readonly STEP_BEFORECAMERADRAW_PREPASS = 0;
        static readonly STEP_BEFORECAMERADRAW_EFFECTLAYER = 1;
        static readonly STEP_BEFORECAMERADRAW_LAYER = 2;
        static readonly STEP_BEFORERENDERTARGETDRAW_PREPASS = 0;
        static readonly STEP_BEFORERENDERTARGETDRAW_LAYER = 1;
        static readonly STEP_BEFORERENDERINGMESH_PREPASS = 0;
        static readonly STEP_BEFORERENDERINGMESH_OUTLINE = 1;
        static readonly STEP_AFTERRENDERINGMESH_PREPASS = 0;
        static readonly STEP_AFTERRENDERINGMESH_OUTLINE = 1;
        static readonly STEP_AFTERRENDERINGGROUPDRAW_EFFECTLAYER_DRAW = 0;
        static readonly STEP_AFTERRENDERINGGROUPDRAW_BOUNDINGBOXRENDERER = 1;
        static readonly STEP_BEFORECAMERAUPDATE_SIMPLIFICATIONQUEUE = 0;
        static readonly STEP_BEFORECAMERAUPDATE_GAMEPAD = 1;
        static readonly STEP_BEFORECLEAR_PROCEDURALTEXTURE = 0;
        static readonly STEP_BEFORECLEAR_PREPASS = 1;
        static readonly STEP_BEFORERENDERTARGETCLEAR_PREPASS = 0;
        static readonly STEP_AFTERRENDERTARGETDRAW_PREPASS = 0;
        static readonly STEP_AFTERRENDERTARGETDRAW_LAYER = 1;
        static readonly STEP_AFTERCAMERADRAW_PREPASS = 0;
        static readonly STEP_AFTERCAMERADRAW_EFFECTLAYER = 1;
        static readonly STEP_AFTERCAMERADRAW_LENSFLARESYSTEM = 2;
        static readonly STEP_AFTERCAMERADRAW_EFFECTLAYER_DRAW = 3;
        static readonly STEP_AFTERCAMERADRAW_LAYER = 4;
        static readonly STEP_AFTERCAMERADRAW_FLUIDRENDERER = 5;
        static readonly STEP_AFTERCAMERAPOSTPROCESS_LAYER = 0;
        static readonly STEP_AFTERRENDERTARGETPOSTPROCESS_LAYER = 0;
        static readonly STEP_AFTERRENDER_AUDIO = 0;
        static readonly STEP_GATHERRENDERTARGETS_DEPTHRENDERER = 0;
        static readonly STEP_GATHERRENDERTARGETS_GEOMETRYBUFFERRENDERER = 1;
        static readonly STEP_GATHERRENDERTARGETS_SHADOWGENERATOR = 2;
        static readonly STEP_GATHERRENDERTARGETS_POSTPROCESSRENDERPIPELINEMANAGER = 3;
        static readonly STEP_GATHERACTIVECAMERARENDERTARGETS_DEPTHRENDERER = 0;
        static readonly STEP_GATHERACTIVECAMERARENDERTARGETS_FLUIDRENDERER = 1;
        static readonly STEP_POINTERMOVE_SPRITE = 0;
        static readonly STEP_POINTERDOWN_SPRITE = 0;
        static readonly STEP_POINTERUP_SPRITE = 0;
    }
    /**
     * This represents a scene component.
     *
     * This is used to decouple the dependency the scene is having on the different workloads like
     * layers, post processes...
     */
    export interface ISceneComponent {
        /**
         * The name of the component. Each component must have a unique name.
         */
        name: string;
        /**
         * The scene the component belongs to.
         */
        scene: Scene;
        /**
         * Register the component to one instance of a scene.
         */
        register(): void;
        /**
         * Rebuilds the elements related to this component in case of
         * context lost for instance.
         */
        rebuild(): void;
        /**
         * Disposes the component and the associated ressources.
         */
        dispose(): void;
    }
    /**
     * This represents a SERIALIZABLE scene component.
     *
     * This extends Scene Component to add Serialization methods on top.
     */
    export interface ISceneSerializableComponent extends ISceneComponent {
        /**
         * Adds all the elements from the container to the scene
         * @param container the container holding the elements
         */
        addFromContainer(container: AbstractScene): void;
        /**
         * Removes all the elements in the container from the scene
         * @param container contains the elements to remove
         * @param dispose if the removed element should be disposed (default: false)
         */
        removeFromContainer(container: AbstractScene, dispose?: boolean): void;
        /**
         * Serializes the component data to the specified json object
         * @param serializationObject The object to serialize to
         */
        serialize(serializationObject: any): void;
    }
    /**
     * Strong typing of a Mesh related stage step action
     */
    export type MeshStageAction = (mesh: AbstractMesh, hardwareInstancedRendering: boolean) => boolean;
    /**
     * Strong typing of a Evaluate Sub Mesh related stage step action
     */
    export type EvaluateSubMeshStageAction = (mesh: AbstractMesh, subMesh: SubMesh) => void;
    /**
     * Strong typing of a pre active Mesh related stage step action
     */
    export type PreActiveMeshStageAction = (mesh: AbstractMesh) => void;
    /**
     * Strong typing of a Camera related stage step action
     */
    export type CameraStageAction = (camera: Camera) => void;
    /**
     * Strong typing of a Camera Frame buffer related stage step action
     */
    export type CameraStageFrameBufferAction = (camera: Camera) => boolean;
    /**
     * Strong typing of a Render Target related stage step action
     */
    export type RenderTargetStageAction = (renderTarget: RenderTargetTexture, faceIndex?: number, layer?: number) => void;
    /**
     * Strong typing of a RenderingGroup related stage step action
     */
    export type RenderingGroupStageAction = (renderingGroupId: number) => void;
    /**
     * Strong typing of a Mesh Render related stage step action
     */
    export type RenderingMeshStageAction = (mesh: Mesh, subMesh: SubMesh, batch: any, effect: Nullable<Effect>) => void;
    /**
     * Strong typing of a simple stage step action
     */
    export type SimpleStageAction = () => void;
    /**
     * Strong typing of a render target action.
     */
    export type RenderTargetsStageAction = (renderTargets: SmartArrayNoDuplicate<RenderTargetTexture>) => void;
    /**
     * Strong typing of a pointer move action.
     */
    export type PointerMoveStageAction = (unTranslatedPointerX: number, unTranslatedPointerY: number, pickResult: Nullable<PickingInfo>, isMeshPicked: boolean, element: Nullable<HTMLElement>) => Nullable<PickingInfo>;
    /**
     * Strong typing of a pointer up/down action.
     */
    export type PointerUpDownStageAction = (unTranslatedPointerX: number, unTranslatedPointerY: number, pickResult: Nullable<PickingInfo>, evt: IPointerEvent, doubleClick: boolean) => Nullable<PickingInfo>;
    /**
     * Representation of a stage in the scene (Basically a list of ordered steps)
     * @internal
     */
    export class Stage<T extends Function> extends Array<{
        index: number;
        component: ISceneComponent;
        action: T;
    }> {
        /**
         * Hide ctor from the rest of the world.
         * @param items The items to add.
         */
        private constructor();
        /**
         * Creates a new Stage.
         * @returns A new instance of a Stage
         */
        static Create<T extends Function>(): Stage<T>;
        /**
         * Registers a step in an ordered way in the targeted stage.
         * @param index Defines the position to register the step in
         * @param component Defines the component attached to the step
         * @param action Defines the action to launch during the step
         */
        registerStep(index: number, component: ISceneComponent, action: T): void;
        /**
         * Clears all the steps from the stage.
         */
        clear(): void;
    }


    /** @internal */
    export var anaglyphPixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var backgroundPixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var backgroundVertexShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var blackAndWhitePixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var bloomMergePixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var blurPixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var boundingBoxRendererPixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var boundingBoxRendererVertexShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var chromaticAberrationPixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var circleOfConfusionPixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var clearQuadPixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var clearQuadVertexShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var colorPixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var colorVertexShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var colorCorrectionPixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var convolutionPixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var copyTextureToTexturePixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var defaultPixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var defaultVertexShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var depthPixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var depthVertexShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var depthBoxBlurPixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var depthOfFieldPixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var depthOfFieldMergePixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var displayPassPixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var extractHighlightsPixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var filterPixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var fluidRenderingBilateralBlurPixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var fluidRenderingParticleDepthPixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var fluidRenderingParticleDepthVertexShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var fluidRenderingParticleDiffusePixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var fluidRenderingParticleDiffuseVertexShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var fluidRenderingParticleThicknessPixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var fluidRenderingParticleThicknessVertexShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var fluidRenderingRenderPixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var fluidRenderingStandardBlurPixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var fxaaPixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var fxaaVertexShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var geometryPixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var geometryVertexShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var glowBlurPostProcessPixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var glowMapGenerationPixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var glowMapGenerationVertexShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var glowMapMergePixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var glowMapMergeVertexShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var gpuRenderParticlesPixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var gpuRenderParticlesVertexShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var gpuUpdateParticlesPixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var gpuUpdateParticlesVertexShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var grainPixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var hdrFilteringPixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var hdrFilteringVertexShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var highlightsPixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var imageProcessingPixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var kernelBlurPixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var kernelBlurVertexShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var layerPixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var layerVertexShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var lensFlarePixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var lensFlareVertexShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var lensHighlightsPixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var linePixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var lineVertexShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var minmaxReduxPixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var motionBlurPixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var noisePixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var oitBackBlendPixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var oitFinalPixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var outlinePixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var outlineVertexShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var particlesPixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var particlesVertexShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var passPixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var passCubePixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var pbrPixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var pbrVertexShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var postprocessVertexShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var proceduralVertexShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var refractionPixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var rgbdDecodePixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var rgbdEncodePixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var screenSpaceCurvaturePixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var screenSpaceReflectionPixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var backgroundFragmentDeclaration: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var backgroundUboDeclaration: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var backgroundVertexDeclaration: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var bakedVertexAnimation: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var bakedVertexAnimationDeclaration: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var bayerDitherFunctions: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var bonesDeclaration: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var bonesVertex: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var boundingBoxRendererFragmentDeclaration: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var boundingBoxRendererUboDeclaration: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var boundingBoxRendererVertexDeclaration: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var bumpFragment: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var bumpFragmentFunctions: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var bumpFragmentMainFunctions: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var bumpVertex: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var bumpVertexDeclaration: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var clipPlaneFragment: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var clipPlaneFragmentDeclaration: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var clipPlaneFragmentDeclaration2: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var clipPlaneVertex: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var clipPlaneVertexDeclaration: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var clipPlaneVertexDeclaration2: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var defaultFragmentDeclaration: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var defaultUboDeclaration: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var defaultVertexDeclaration: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var depthPrePass: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var diffusionProfile: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var fibonacci: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var fogFragment: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var fogFragmentDeclaration: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var fogVertex: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var fogVertexDeclaration: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var fresnelFunction: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var geometryUboDeclaration: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var geometryVertexDeclaration: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var harmonicsFunctions: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var hdrFilteringFunctions: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var helperFunctions: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var imageProcessingCompatibility: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var imageProcessingDeclaration: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var imageProcessingFunctions: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var importanceSampling: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var instancesDeclaration: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var instancesVertex: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var kernelBlurFragment: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var kernelBlurFragment2: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var kernelBlurVaryingDeclaration: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var kernelBlurVertex: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var lightFragment: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var lightFragmentDeclaration: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var lightsFragmentFunctions: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var lightUboDeclaration: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var lightVxFragmentDeclaration: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var lightVxUboDeclaration: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var logDepthDeclaration: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var logDepthFragment: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var logDepthVertex: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var mainUVVaryingDeclaration: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var meshFragmentDeclaration: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var meshUboDeclaration: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var meshVertexDeclaration: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var morphTargetsVertex: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var morphTargetsVertexDeclaration: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var morphTargetsVertexGlobal: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var morphTargetsVertexGlobalDeclaration: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var mrtFragmentDeclaration: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var oitDeclaration: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var oitFragment: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var packingFunctions: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var pbrBlockAlbedoOpacity: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var pbrBlockAlphaFresnel: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var pbrBlockAmbientOcclusion: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var pbrBlockAnisotropic: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var pbrBlockClearcoat: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var pbrBlockDirectLighting: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var pbrBlockFinalColorComposition: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var pbrBlockFinalLitComponents: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var pbrBlockFinalUnlitComponents: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var pbrBlockGeometryInfo: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var pbrBlockImageProcessing: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var pbrBlockIridescence: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var pbrBlockLightmapInit: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var pbrBlockNormalFinal: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var pbrBlockNormalGeometric: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var pbrBlockReflectance: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var pbrBlockReflectance0: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var pbrBlockReflection: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var pbrBlockReflectivity: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var pbrBlockSheen: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var pbrBlockSubSurface: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var pbrBRDFFunctions: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var pbrDebug: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var pbrDirectLightingFalloffFunctions: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var pbrDirectLightingFunctions: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var pbrDirectLightingSetupFunctions: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var pbrFragmentDeclaration: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var pbrFragmentExtraDeclaration: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var pbrFragmentSamplersDeclaration: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var pbrHelperFunctions: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var pbrIBLFunctions: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var pbrUboDeclaration: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var pbrVertexDeclaration: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var pointCloudVertex: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var pointCloudVertexDeclaration: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var prePassDeclaration: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var prePassVertex: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var prePassVertexDeclaration: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var reflectionFunction: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var samplerFragmentAlternateDeclaration: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var samplerFragmentDeclaration: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var samplerVertexDeclaration: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var samplerVertexImplementation: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var sceneFragmentDeclaration: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var sceneUboDeclaration: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var sceneVertexDeclaration: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var shadowMapFragment: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var shadowMapFragmentExtraDeclaration: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var shadowMapFragmentSoftTransparentShadow: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var shadowMapUboDeclaration: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var shadowMapVertexDeclaration: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var shadowMapVertexExtraDeclaration: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var shadowMapVertexMetric: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var shadowMapVertexNormalBias: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var shadowsFragmentFunctions: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var shadowsVertex: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var subSurfaceScatteringFunctions: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var uvAttributeDeclaration: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var uvVariableDeclaration: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var vertexColorMixing: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var shadowMapPixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var shadowMapVertexShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var sharpenPixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var spriteMapPixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var spriteMapVertexShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var spritesPixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var spritesVertexShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var ssaoPixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var ssao2PixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var ssaoCombinePixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var standardPixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var stereoscopicInterlacePixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var subSurfaceScatteringPixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var tonemapPixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var volumetricLightScatteringPixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var volumetricLightScatteringPassPixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var volumetricLightScatteringPassVertexShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var vrDistortionCorrectionPixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var vrMultiviewToSingleviewPixelShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var gpuUpdateParticlesComputeShader: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var bakedVertexAnimation: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var bakedVertexAnimationDeclaration: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var bonesDeclaration: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var bonesVertex: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var clipPlaneFragment: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var clipPlaneFragmentDeclaration: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var clipPlaneVertex: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var clipPlaneVertexDeclaration: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var instancesDeclaration: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var instancesVertex: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var meshUboDeclaration: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var morphTargetsVertex: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var morphTargetsVertexDeclaration: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var morphTargetsVertexGlobal: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var morphTargetsVertexGlobalDeclaration: {
        name: string;
        shader: string;
    };


    /** @internal */
    export var sceneUboDeclaration: {
        name: string;
        shader: string;
    };




    /**
     * Defines the basic options interface of a Sprite Frame Source Size.
     */
    export interface ISpriteJSONSpriteSourceSize {
        /**
         * number of the original width of the Frame
         */
        w: number;
        /**
         * number of the original height of the Frame
         */
        h: number;
    }
    /**
     * Defines the basic options interface of a Sprite Frame Data.
     */
    export interface ISpriteJSONSpriteFrameData {
        /**
         * number of the x offset of the Frame
         */
        x: number;
        /**
         * number of the y offset of the Frame
         */
        y: number;
        /**
         * number of the width of the Frame
         */
        w: number;
        /**
         * number of the height of the Frame
         */
        h: number;
    }
    /**
     * Defines the basic options interface of a JSON Sprite.
     */
    export interface ISpriteJSONSprite {
        /**
         * string name of the Frame
         */
        filename: string;
        /**
         * ISpriteJSONSpriteFrame basic object of the frame data
         */
        frame: ISpriteJSONSpriteFrameData;
        /**
         * boolean to flag is the frame was rotated.
         */
        rotated: boolean;
        /**
         * boolean to flag is the frame was trimmed.
         */
        trimmed: boolean;
        /**
         * ISpriteJSONSpriteFrame basic object of the source data
         */
        spriteSourceSize: ISpriteJSONSpriteFrameData;
        /**
         * ISpriteJSONSpriteFrame basic object of the source data
         */
        sourceSize: ISpriteJSONSpriteSourceSize;
    }
    /**
     * Defines the basic options interface of a JSON atlas.
     */
    export interface ISpriteJSONAtlas {
        /**
         * Array of objects that contain the frame data.
         */
        frames: Array<ISpriteJSONSprite>;
        /**
         * object basic object containing the sprite meta data.
         */
        meta?: object;
    }


    /**
     * Class used to represent a sprite
     * @see https://doc.babylonjs.com/features/featuresDeepDive/sprites
     */
    export class Sprite extends ThinSprite implements IAnimatable {
        /** defines the name */
        name: string;
        /** Gets or sets the current world position */
        position: Vector3;
        /** Gets or sets the main color */
        color: Color4;
        /** Gets or sets a boolean indicating that this sprite should be disposed after animation ends */
        disposeWhenFinishedAnimating: boolean;
        /** Gets the list of attached animations */
        animations: Nullable<Array<Animation>>;
        /** Gets or sets a boolean indicating if the sprite can be picked */
        isPickable: boolean;
        /** Gets or sets a boolean indicating that sprite texture alpha will be used for precise picking (false by default) */
        useAlphaForPicking: boolean;
        /**
         * Gets or sets the associated action manager
         */
        actionManager: Nullable<ActionManager>;
        /**
         * An event triggered when the control has been disposed
         */
        onDisposeObservable: Observable<Sprite>;
        private _manager;
        private _onAnimationEnd;
        /**
         * Gets or sets the sprite size
         */
        get size(): number;
        set size(value: number);
        /**
         * Gets or sets the unique id of the sprite
         */
        uniqueId: number;
        /**
         * Gets the manager of this sprite
         */
        get manager(): ISpriteManager;
        /**
         * Creates a new Sprite
         * @param name defines the name
         * @param manager defines the manager
         */
        constructor(
        /** defines the name */
        name: string, manager: ISpriteManager);
        /**
         * Returns the string "Sprite"
         * @returns "Sprite"
         */
        getClassName(): string;
        /** Gets or sets the initial key for the animation (setting it will restart the animation)  */
        get fromIndex(): number;
        set fromIndex(value: number);
        /** Gets or sets the end key for the animation (setting it will restart the animation)  */
        get toIndex(): number;
        set toIndex(value: number);
        /** Gets or sets a boolean indicating if the animation is looping (setting it will restart the animation)  */
        get loopAnimation(): boolean;
        set loopAnimation(value: boolean);
        /** Gets or sets the delay between cell changes (setting it will restart the animation)  */
        get delay(): number;
        set delay(value: number);
        /**
         * Starts an animation
         * @param from defines the initial key
         * @param to defines the end key
         * @param loop defines if the animation must loop
         * @param delay defines the start delay (in ms)
         * @param onAnimationEnd defines a callback to call when animation ends
         */
        playAnimation(from: number, to: number, loop: boolean, delay: number, onAnimationEnd?: Nullable<() => void>): void;
        private _endAnimation;
        /** Release associated resources */
        dispose(): void;
        /**
         * Serializes the sprite to a JSON object
         * @returns the JSON object
         */
        serialize(): any;
        /**
         * Parses a JSON object to create a new sprite
         * @param parsedSprite The JSON object to parse
         * @param manager defines the hosting manager
         * @returns the new sprite
         */
        static Parse(parsedSprite: any, manager: SpriteManager): Sprite;
    }


    /**
     * Defines the minimum interface to fulfill in order to be a sprite manager.
     */
    export interface ISpriteManager extends IDisposable {
        /**
         * Gets manager's name
         */
        name: string;
        /**
         * Restricts the camera to viewing objects with the same layerMask.
         * A camera with a layerMask of 1 will render spriteManager.layerMask & camera.layerMask!== 0
         */
        layerMask: number;
        /**
         * Gets or sets a boolean indicating if the mesh can be picked (by scene.pick for instance or through actions). Default is true
         */
        isPickable: boolean;
        /**
         * Gets the hosting scene
         */
        scene: Scene;
        /**
         * Specifies the rendering group id for this mesh (0 by default)
         * @see https://doc.babylonjs.com/features/featuresDeepDive/materials/advanced/transparent_rendering#rendering-groups
         */
        renderingGroupId: number;
        /**
         * Defines the list of sprites managed by the manager.
         */
        sprites: Array<Sprite>;
        /**
         * Gets or sets the spritesheet texture
         */
        texture: Texture;
        /** Defines the default width of a cell in the spritesheet */
        cellWidth: number;
        /** Defines the default height of a cell in the spritesheet */
        cellHeight: number;
        /** @internal */
        _wasDispatched: boolean;
        /**
         * Tests the intersection of a sprite with a specific ray.
         * @param ray The ray we are sending to test the collision
         * @param camera The camera space we are sending rays in
         * @param predicate A predicate allowing excluding sprites from the list of object to test
         * @param fastCheck defines if the first intersection will be used (and not the closest)
         * @returns picking info or null.
         */
        intersects(ray: Ray, camera: Camera, predicate?: (sprite: Sprite) => boolean, fastCheck?: boolean): Nullable<PickingInfo>;
        /**
         * Intersects the sprites with a ray
         * @param ray defines the ray to intersect with
         * @param camera defines the current active camera
         * @param predicate defines a predicate used to select candidate sprites
         * @returns null if no hit or a PickingInfo array
         */
        multiIntersects(ray: Ray, camera: Camera, predicate?: (sprite: Sprite) => boolean): Nullable<PickingInfo[]>;
        /**
         * Renders the list of sprites on screen.
         */
        render(): void;
        /**
         * Rebuilds the manager (after a context lost, for eg)
         */
        rebuild(): void;
    }
    /**
     * Class used to manage multiple sprites on the same spritesheet
     * @see https://doc.babylonjs.com/features/featuresDeepDive/sprites
     */
    export class SpriteManager implements ISpriteManager {
        /** defines the manager's name */
        name: string;
        /** Define the Url to load snippets */
        static SnippetUrl: string;
        /** Snippet ID if the manager was created from the snippet server */
        snippetId: string;
        /** Gets the list of sprites */
        sprites: Sprite[];
        /** Gets or sets the rendering group id (0 by default) */
        renderingGroupId: number;
        /** Gets or sets camera layer mask */
        layerMask: number;
        /** Gets or sets a boolean indicating if the sprites are pickable */
        isPickable: boolean;
        /**
         * Gets or sets an object used to store user defined information for the sprite manager
         */
        metadata: any;
        /** @internal */
        _wasDispatched: boolean;
        /**
         * An event triggered when the manager is disposed.
         */
        onDisposeObservable: Observable<SpriteManager>;
        /**
         * Callback called when the manager is disposed
         */
        set onDispose(callback: () => void);
        /**
         * Gets or sets the unique id of the sprite
         */
        uniqueId: number;
        /**
         * Gets the array of sprites
         */
        get children(): Sprite[];
        /**
         * Gets the hosting scene
         */
        get scene(): Scene;
        /**
         * Gets the capacity of the manager
         */
        get capacity(): number;
        /**
         * Gets or sets the spritesheet texture
         */
        get texture(): Texture;
        set texture(value: Texture);
        /** Defines the default width of a cell in the spritesheet */
        get cellWidth(): number;
        set cellWidth(value: number);
        /** Defines the default height of a cell in the spritesheet */
        get cellHeight(): number;
        set cellHeight(value: number);
        /** Gets or sets a boolean indicating if the manager must consider scene fog when rendering */
        get fogEnabled(): boolean;
        set fogEnabled(value: boolean);
        /**
         * Blend mode use to render the particle, it can be any of
         * the static Constants.ALPHA_x properties provided in this class.
         * Default value is Constants.ALPHA_COMBINE
         */
        get blendMode(): number;
        set blendMode(blendMode: number);
        private _disableDepthWrite;
        /** Disables writing to the depth buffer when rendering the sprites.
         *  It can be handy to disable depth writing when using textures without alpha channel
         *  and setting some specific blend modes.
         */
        get disableDepthWrite(): boolean;
        set disableDepthWrite(value: boolean);
        private _spriteRenderer;
        /** Associative array from JSON sprite data file */
        private _cellData;
        /** Array of sprite names from JSON sprite data file */
        private _spriteMap;
        /** True when packed cell data from JSON file is ready*/
        private _packedAndReady;
        private _textureContent;
        private _onDisposeObserver;
        private _fromPacked;
        private _scene;
        /**
         * Creates a new sprite manager
         * @param name defines the manager's name
         * @param imgUrl defines the sprite sheet url
         * @param capacity defines the maximum allowed number of sprites
         * @param cellSize defines the size of a sprite cell
         * @param scene defines the hosting scene
         * @param epsilon defines the epsilon value to align texture (0.01 by default)
         * @param samplingMode defines the sampling mode to use with spritesheet
         * @param fromPacked set to false; do not alter
         * @param spriteJSON null otherwise a JSON object defining sprite sheet data; do not alter
         */
        constructor(
        /** defines the manager's name */
        name: string, imgUrl: string, capacity: number, cellSize: any, scene: Scene, epsilon?: number, samplingMode?: number, fromPacked?: boolean, spriteJSON?: any | null);
        /**
         * Returns the string "SpriteManager"
         * @returns "SpriteManager"
         */
        getClassName(): string;
        private _makePacked;
        private _checkTextureAlpha;
        /**
         * Intersects the sprites with a ray
         * @param ray defines the ray to intersect with
         * @param camera defines the current active camera
         * @param predicate defines a predicate used to select candidate sprites
         * @param fastCheck defines if a fast check only must be done (the first potential sprite is will be used and not the closer)
         * @returns null if no hit or a PickingInfo
         */
        intersects(ray: Ray, camera: Camera, predicate?: (sprite: Sprite) => boolean, fastCheck?: boolean): Nullable<PickingInfo>;
        /**
         * Intersects the sprites with a ray
         * @param ray defines the ray to intersect with
         * @param camera defines the current active camera
         * @param predicate defines a predicate used to select candidate sprites
         * @returns null if no hit or a PickingInfo array
         */
        multiIntersects(ray: Ray, camera: Camera, predicate?: (sprite: Sprite) => boolean): Nullable<PickingInfo[]>;
        /**
         * Render all child sprites
         */
        render(): void;
        private _customUpdate;
        /**
         * Rebuilds the manager (after a context lost, for eg)
         */
        rebuild(): void;
        /**
         * Release associated resources
         */
        dispose(): void;
        /**
         * Serializes the sprite manager to a JSON object
         * @param serializeTexture defines if the texture must be serialized as well
         * @returns the JSON object
         */
        serialize(serializeTexture?: boolean): any;
        /**
         * Parses a JSON object to create a new sprite manager.
         * @param parsedManager The JSON object to parse
         * @param scene The scene to create the sprite manager
         * @param rootUrl The root url to use to load external dependencies like texture
         * @returns the new sprite manager
         */
        static Parse(parsedManager: any, scene: Scene, rootUrl: string): SpriteManager;
        /**
         * Creates a sprite manager from a snippet saved in a remote file
         * @param name defines the name of the sprite manager to create (can be null or empty to use the one from the json data)
         * @param url defines the url to load from
         * @param scene defines the hosting scene
         * @param rootUrl defines the root URL to use to load textures and relative dependencies
         * @returns a promise that will resolve to the new sprite manager
         */
        static ParseFromFileAsync(name: Nullable<string>, url: string, scene: Scene, rootUrl?: string): Promise<SpriteManager>;
        /**
         * Creates a sprite manager from a snippet saved by the sprite editor
         * @param snippetId defines the snippet to load (can be set to _BLANK to create a default one)
         * @param scene defines the hosting scene
         * @param rootUrl defines the root URL to use to load textures and relative dependencies
         * @returns a promise that will resolve to the new sprite manager
         */
        static ParseFromSnippetAsync(snippetId: string, scene: Scene, rootUrl?: string): Promise<SpriteManager>;
        /**
         * Creates a sprite manager from a snippet saved by the sprite editor
         * @deprecated Please use ParseFromSnippetAsync instead
         * @param snippetId defines the snippet to load (can be set to _BLANK to create a default one)
         * @param scene defines the hosting scene
         * @param rootUrl defines the root URL to use to load textures and relative dependencies
         * @returns a promise that will resolve to the new sprite manager
         */
        static CreateFromSnippetAsync: typeof SpriteManager.ParseFromSnippetAsync;
    }


    /**
     * Defines the basic options interface of a SpriteMap
     */
    export interface ISpriteMapOptions {
        /**
         * Vector2 of the number of cells in the grid.
         */
        stageSize?: Vector2;
        /**
         * Vector2 of the size of the output plane in World Units.
         */
        outputSize?: Vector2;
        /**
         * Vector3 of the position of the output plane in World Units.
         */
        outputPosition?: Vector3;
        /**
         * Vector3 of the rotation of the output plane.
         */
        outputRotation?: Vector3;
        /**
         * number of layers that the system will reserve in resources.
         */
        layerCount?: number;
        /**
         * number of max animation frames a single cell will reserve in resources.
         */
        maxAnimationFrames?: number;
        /**
         * number cell index of the base tile when the system compiles.
         */
        baseTile?: number;
        /**
         * boolean flip the sprite after its been repositioned by the framing data.
         */
        flipU?: boolean;
        /**
         * Vector3 scalar of the global RGB values of the SpriteMap.
         */
        colorMultiply?: Vector3;
    }
    /**
     * Defines the IDisposable interface in order to be cleanable from resources.
     */
    export interface ISpriteMap extends IDisposable {
        /**
         * String name of the SpriteMap.
         */
        name: string;
        /**
         * The JSON Array file from a https://www.codeandweb.com/texturepacker export.  Or similar structure.
         */
        atlasJSON: ISpriteJSONAtlas;
        /**
         * Texture of the SpriteMap.
         */
        spriteSheet: Texture;
        /**
         * The parameters to initialize the SpriteMap with.
         */
        options: ISpriteMapOptions;
    }
    /**
     * Class used to manage a grid restricted sprite deployment on an Output plane.
     */
    export class SpriteMap implements ISpriteMap {
        /** The Name of the spriteMap */
        name: string;
        /** The JSON file with the frame and meta data */
        atlasJSON: ISpriteJSONAtlas;
        /** The systems Sprite Sheet Texture */
        spriteSheet: Texture;
        /** Arguments passed with the Constructor */
        options: ISpriteMapOptions;
        /** Public Sprite Storage array, parsed from atlasJSON */
        sprites: Array<ISpriteJSONSprite>;
        /** Returns the Number of Sprites in the System */
        get spriteCount(): number;
        /** Returns the Position of Output Plane*/
        get position(): Vector3;
        /** Returns the Position of Output Plane*/
        set position(v: Vector3);
        /** Returns the Rotation of Output Plane*/
        get rotation(): Vector3;
        /** Returns the Rotation of Output Plane*/
        set rotation(v: Vector3);
        /** Sets the AnimationMap*/
        get animationMap(): RawTexture;
        /** Sets the AnimationMap*/
        set animationMap(v: RawTexture);
        /** Scene that the SpriteMap was created in */
        private _scene;
        /** Texture Buffer of Float32 that holds tile frame data*/
        private _frameMap;
        /** Texture Buffers of Float32 that holds tileMap data*/
        private _tileMaps;
        /** Texture Buffer of Float32 that holds Animation Data*/
        private _animationMap;
        /** Custom ShaderMaterial Central to the System*/
        private _material;
        /** Custom ShaderMaterial Central to the System*/
        private _output;
        /** Systems Time Ticker*/
        private _time;
        /**
         * Creates a new SpriteMap
         * @param name defines the SpriteMaps Name
         * @param atlasJSON is the JSON file that controls the Sprites Frames and Meta
         * @param spriteSheet is the Texture that the Sprites are on.
         * @param options a basic deployment configuration
         * @param scene The Scene that the map is deployed on
         */
        constructor(name: string, atlasJSON: ISpriteJSONAtlas, spriteSheet: Texture, options: ISpriteMapOptions, scene: Scene);
        /**
         * Returns tileID location
         * @returns Vector2 the cell position ID
         */
        getTileID(): Vector2;
        /**
         * Gets the UV location of the mouse over the SpriteMap.
         * @returns Vector2 the UV position of the mouse interaction
         */
        getMousePosition(): Vector2;
        /**
         * Creates the "frame" texture Buffer
         * -------------------------------------
         * Structure of frames
         *  "filename": "Falling-Water-2.png",
         * "frame": {"x":69,"y":103,"w":24,"h":32},
         * "rotated": true,
         * "trimmed": true,
         * "spriteSourceSize": {"x":4,"y":0,"w":24,"h":32},
         * "sourceSize": {"w":32,"h":32}
         * @returns RawTexture of the frameMap
         */
        private _createFrameBuffer;
        /**
         * Creates the tileMap texture Buffer
         * @param buffer normally and array of numbers, or a false to generate from scratch
         * @param _layer indicates what layer for a logic trigger dealing with the baseTile.  The system uses this
         * @returns RawTexture of the tileMap
         */
        private _createTileBuffer;
        /**
         * Modifies the data of the tileMaps
         * @param _layer is the ID of the layer you want to edit on the SpriteMap
         * @param pos is the iVector2 Coordinates of the Tile
         * @param tile The SpriteIndex of the new Tile
         */
        changeTiles(_layer: number | undefined, pos: Vector2 | Vector2[], tile?: number): void;
        /**
         * Creates the animationMap texture Buffer
         * @param buffer normally and array of numbers, or a false to generate from scratch
         * @returns RawTexture of the animationMap
         */
        private _createTileAnimationBuffer;
        /**
         * Modifies the data of the animationMap
         * @param cellID is the Index of the Sprite
         * @param _frame is the target Animation frame
         * @param toCell is the Target Index of the next frame of the animation
         * @param time is a value between 0-1 that is the trigger for when the frame should change tiles
         * @param speed is a global scalar of the time variable on the map.
         */
        addAnimationToTile(cellID?: number, _frame?: number, toCell?: number, time?: number, speed?: number): void;
        /**
         * Exports the .tilemaps file
         */
        saveTileMaps(): void;
        /**
         * Imports the .tilemaps file
         * @param url of the .tilemaps file
         */
        loadTileMaps(url: string): void;
        /**
         * Release associated resources
         */
        dispose(): void;
    }


    /**
     * Class used to manage multiple sprites of different sizes on the same spritesheet
     * @see https://doc.babylonjs.com/features/featuresDeepDive/sprites
     */
    export class SpritePackedManager extends SpriteManager {
        /** defines the packed manager's name */
        name: string;
        /**
         * Creates a new sprite manager from a packed sprite sheet
         * @param name defines the manager's name
         * @param imgUrl defines the sprite sheet url
         * @param capacity defines the maximum allowed number of sprites
         * @param scene defines the hosting scene
         * @param spriteJSON null otherwise a JSON object defining sprite sheet data
         * @param epsilon defines the epsilon value to align texture (0.01 by default)
         * @param samplingMode defines the sampling mode to use with spritesheet
         * @param fromPacked set to true; do not alter
         */
        constructor(
        /** defines the packed manager's name */
        name: string, imgUrl: string, capacity: number, scene: Scene, spriteJSON?: string | null, epsilon?: number, samplingMode?: number);
    }


    /**
     * Class used to render sprites.
     *
     * It can be used either to render Sprites or ThinSprites with ThinEngine only.
     */
    export class SpriteRenderer {
        /**
         * Defines the texture of the spritesheet
         */
        texture: Nullable<ThinTexture>;
        /**
         * Defines the default width of a cell in the spritesheet
         */
        cellWidth: number;
        /**
         * Defines the default height of a cell in the spritesheet
         */
        cellHeight: number;
        /**
         * Blend mode use to render the particle, it can be any of
         * the static Constants.ALPHA_x properties provided in this class.
         * Default value is Constants.ALPHA_COMBINE
         */
        blendMode: number;
        /**
         * Gets or sets a boolean indicating if alpha mode is automatically
         * reset.
         */
        autoResetAlpha: boolean;
        /**
         * Disables writing to the depth buffer when rendering the sprites.
         * It can be handy to disable depth writing when using textures without alpha channel
         * and setting some specific blend modes.
         */
        disableDepthWrite: boolean;
        /**
         * Gets or sets a boolean indicating if the manager must consider scene fog when rendering
         */
        fogEnabled: boolean;
        /**
         * Gets the capacity of the manager
         */
        get capacity(): number;
        private readonly _engine;
        private readonly _useVAO;
        private readonly _useInstancing;
        private readonly _scene;
        private readonly _capacity;
        private readonly _epsilon;
        private _vertexBufferSize;
        private _vertexData;
        private _buffer;
        private _vertexBuffers;
        private _spriteBuffer;
        private _indexBuffer;
        private _drawWrapperBase;
        private _drawWrapperFog;
        private _drawWrapperDepth;
        private _drawWrapperFogDepth;
        private _vertexArrayObject;
        /**
         * Creates a new sprite Renderer
         * @param engine defines the engine the renderer works with
         * @param capacity defines the maximum allowed number of sprites
         * @param epsilon defines the epsilon value to align texture (0.01 by default)
         * @param scene defines the hosting scene
         */
        constructor(engine: ThinEngine, capacity: number, epsilon?: number, scene?: Nullable<Scene>);
        /**
         * Render all child sprites
         * @param sprites defines the list of sprites to render
         * @param deltaTime defines the time since last frame
         * @param viewMatrix defines the viewMatrix to use to render the sprites
         * @param projectionMatrix defines the projectionMatrix to use to render the sprites
         * @param customSpriteUpdate defines a custom function to update the sprites data before they render
         */
        render(sprites: ThinSprite[], deltaTime: number, viewMatrix: IMatrixLike, projectionMatrix: IMatrixLike, customSpriteUpdate?: Nullable<(sprite: ThinSprite, baseSize: ISize) => void>): void;
        private _appendSpriteVertex;
        private _buildIndexBuffer;
        /**
         * Rebuilds the renderer (after a context lost, for eg)
         */
        rebuild(): void;
        /**
         * Release associated resources
         */
        dispose(): void;
    }


        interface Scene {
            /** @internal */
            _pointerOverSprite: Nullable<Sprite>;
            /** @internal */
            _pickedDownSprite: Nullable<Sprite>;
            /** @internal */
            _tempSpritePickingRay: Nullable<Ray>;
            /**
             * All of the sprite managers added to this scene
             * @see https://doc.babylonjs.com/features/featuresDeepDive/sprites
             */
            spriteManagers: Array<ISpriteManager>;
            /**
             * An event triggered when sprites rendering is about to start
             * Note: This event can be trigger more than once per frame (because sprites can be rendered by render target textures as well)
             */
            onBeforeSpritesRenderingObservable: Observable<Scene>;
            /**
             * An event triggered when sprites rendering is done
             * Note: This event can be trigger more than once per frame (because sprites can be rendered by render target textures as well)
             */
            onAfterSpritesRenderingObservable: Observable<Scene>;
            /** @internal */
            _internalPickSprites(ray: Ray, predicate?: (sprite: Sprite) => boolean, fastCheck?: boolean, camera?: Camera): Nullable<PickingInfo>;
            /** Launch a ray to try to pick a sprite in the scene
             * @param x position on screen
             * @param y position on screen
             * @param predicate Predicate function used to determine eligible sprites. Can be set to null. In this case, a sprite must have isPickable set to true
             * @param fastCheck defines if the first intersection will be used (and not the closest)
             * @param camera camera to use for computing the picking ray. Can be set to null. In this case, the scene.activeCamera will be used
             * @returns a PickingInfo
             */
            pickSprite(x: number, y: number, predicate?: (sprite: Sprite) => boolean, fastCheck?: boolean, camera?: Camera): Nullable<PickingInfo>;
            /** Use the given ray to pick a sprite in the scene
             * @param ray The ray (in world space) to use to pick meshes
             * @param predicate Predicate function used to determine eligible sprites. Can be set to null. In this case, a sprite must have isPickable set to true
             * @param fastCheck defines if the first intersection will be used (and not the closest)
             * @param camera camera to use. Can be set to null. In this case, the scene.activeCamera will be used
             * @returns a PickingInfo
             */
            pickSpriteWithRay(ray: Ray, predicate?: (sprite: Sprite) => boolean, fastCheck?: boolean, camera?: Camera): Nullable<PickingInfo>;
            /** @internal */
            _internalMultiPickSprites(ray: Ray, predicate?: (sprite: Sprite) => boolean, camera?: Camera): Nullable<PickingInfo[]>;
            /** Launch a ray to try to pick sprites in the scene
             * @param x position on screen
             * @param y position on screen
             * @param predicate Predicate function used to determine eligible sprites. Can be set to null. In this case, a sprite must have isPickable set to true
             * @param camera camera to use for computing the picking ray. Can be set to null. In this case, the scene.activeCamera will be used
             * @returns a PickingInfo array
             */
            multiPickSprite(x: number, y: number, predicate?: (sprite: Sprite) => boolean, camera?: Camera): Nullable<PickingInfo[]>;
            /** Use the given ray to pick sprites in the scene
             * @param ray The ray (in world space) to use to pick meshes
             * @param predicate Predicate function used to determine eligible sprites. Can be set to null. In this case, a sprite must have isPickable set to true
             * @param camera camera to use. Can be set to null. In this case, the scene.activeCamera will be used
             * @returns a PickingInfo array
             */
            multiPickSpriteWithRay(ray: Ray, predicate?: (sprite: Sprite) => boolean, camera?: Camera): Nullable<PickingInfo[]>;
            /**
             * Force the sprite under the pointer
             * @param sprite defines the sprite to use
             */
            setPointerOverSprite(sprite: Nullable<Sprite>): void;
            /**
             * Gets the sprite under the pointer
             * @returns a Sprite or null if no sprite is under the pointer
             */
            getPointerOverSprite(): Nullable<Sprite>;
        }
    /**
     * Defines the sprite scene component responsible to manage sprites
     * in a given scene.
     */
    export class SpriteSceneComponent implements ISceneComponent {
        /**
         * The component name helpfull to identify the component in the list of scene components.
         */
        readonly name = "Sprite";
        /**
         * The scene the component belongs to.
         */
        scene: Scene;
        /** @internal */
        private _spritePredicate;
        /**
         * Creates a new instance of the component for the given scene
         * @param scene Defines the scene to register the component in
         */
        constructor(scene: Scene);
        /**
         * Registers the component in a given scene
         */
        register(): void;
        /**
         * Rebuilds the elements related to this component in case of
         * context lost for instance.
         */
        rebuild(): void;
        /**
         * Disposes the component and the associated resources.
         */
        dispose(): void;
        private _pickSpriteButKeepRay;
        private _pointerMove;
        private _pointerDown;
        private _pointerUp;
    }


    /**
     * ThinSprite Class used to represent a thin sprite
     * This is the base class for sprites but can also directly be used with ThinEngine
     * @see https://doc.babylonjs.com/features/featuresDeepDive/sprites
     */
    export class ThinSprite {
        /** Gets or sets the cell index in the sprite sheet */
        cellIndex: number;
        /** Gets or sets the cell reference in the sprite sheet, uses sprite's filename when added to sprite sheet */
        cellRef: string;
        /** Gets or sets the current world position */
        position: IVector3Like;
        /** Gets or sets the main color */
        color: IColor4Like;
        /** Gets or sets the width */
        width: number;
        /** Gets or sets the height */
        height: number;
        /** Gets or sets rotation angle */
        angle: number;
        /** Gets or sets a boolean indicating if UV coordinates should be inverted in U axis */
        invertU: boolean;
        /** Gets or sets a boolean indicating if UV coordinates should be inverted in B axis */
        invertV: boolean;
        /** Gets or sets a boolean indicating if the sprite is visible (renderable). Default is true */
        isVisible: boolean;
        /**
         * Returns a boolean indicating if the animation is started
         */
        get animationStarted(): boolean;
        /** Gets the initial key for the animation (setting it will restart the animation)  */
        get fromIndex(): number;
        /** Gets or sets the end key for the animation (setting it will restart the animation)  */
        get toIndex(): number;
        /** Gets or sets a boolean indicating if the animation is looping (setting it will restart the animation)  */
        get loopAnimation(): boolean;
        /** Gets or sets the delay between cell changes (setting it will restart the animation)  */
        get delay(): number;
        /** @internal */
        _xOffset: number;
        /** @internal */
        _yOffset: number;
        /** @internal */
        _xSize: number;
        /** @internal */
        _ySize: number;
        private _animationStarted;
        protected _loopAnimation: boolean;
        protected _fromIndex: number;
        protected _toIndex: number;
        protected _delay: number;
        private _direction;
        private _time;
        private _onBaseAnimationEnd;
        /**
         * Creates a new Thin Sprite
         */
        constructor();
        /**
         * Starts an animation
         * @param from defines the initial key
         * @param to defines the end key
         * @param loop defines if the animation must loop
         * @param delay defines the start delay (in ms)
         * @param onAnimationEnd defines a callback for when the animation ends
         */
        playAnimation(from: number, to: number, loop: boolean, delay: number, onAnimationEnd: Nullable<() => void>): void;
        /** Stops current animation (if any) */
        stopAnimation(): void;
        /**
         * @internal
         */
        _animate(deltaTime: number): void;
    }


    /**
     * @internal
     **/
    export class AlphaState {
        _blendFunctionParameters: Nullable<number>[];
        _blendEquationParameters: Nullable<number>[];
        _blendConstants: Nullable<number>[];
        _isBlendConstantsDirty: boolean;
        private _alphaBlend;
        private _isAlphaBlendDirty;
        private _isBlendFunctionParametersDirty;
        private _isBlendEquationParametersDirty;
        /**
         * Initializes the state.
         */
        constructor();
        get isDirty(): boolean;
        get alphaBlend(): boolean;
        set alphaBlend(value: boolean);
        setAlphaBlendConstants(r: number, g: number, b: number, a: number): void;
        setAlphaBlendFunctionParameters(value0: number, value1: number, value2: number, value3: number): void;
        setAlphaEquationParameters(rgb: number, alpha: number): void;
        reset(): void;
        apply(gl: WebGLRenderingContext): void;
    }


    /**
     * @internal
     **/
    export class DepthCullingState {
        protected _isDepthTestDirty: boolean;
        protected _isDepthMaskDirty: boolean;
        protected _isDepthFuncDirty: boolean;
        protected _isCullFaceDirty: boolean;
        protected _isCullDirty: boolean;
        protected _isZOffsetDirty: boolean;
        protected _isFrontFaceDirty: boolean;
        protected _depthTest: boolean;
        protected _depthMask: boolean;
        protected _depthFunc: Nullable<number>;
        protected _cull: Nullable<boolean>;
        protected _cullFace: Nullable<number>;
        protected _zOffset: number;
        protected _zOffsetUnits: number;
        protected _frontFace: Nullable<number>;
        /**
         * Initializes the state.
         * @param reset
         */
        constructor(reset?: boolean);
        get isDirty(): boolean;
        get zOffset(): number;
        set zOffset(value: number);
        get zOffsetUnits(): number;
        set zOffsetUnits(value: number);
        get cullFace(): Nullable<number>;
        set cullFace(value: Nullable<number>);
        get cull(): Nullable<boolean>;
        set cull(value: Nullable<boolean>);
        get depthFunc(): Nullable<number>;
        set depthFunc(value: Nullable<number>);
        get depthMask(): boolean;
        set depthMask(value: boolean);
        get depthTest(): boolean;
        set depthTest(value: boolean);
        get frontFace(): Nullable<number>;
        set frontFace(value: Nullable<number>);
        reset(): void;
        apply(gl: WebGLRenderingContext): void;
    }




    /** @internal */
    export interface IStencilState {
        enabled: boolean;
        mask: number;
        func: number;
        funcRef: number;
        funcMask: number;
        opStencilDepthPass: number;
        opStencilFail: number;
        opDepthFail: number;
        reset(): void;
    }


    /**
     * @internal
     **/
    export class StencilState implements IStencilState {
        /** Passed to depthFunction or stencilFunction to specify depth or stencil tests will always pass. i.e. Pixels will be drawn in the order they are drawn */
        static readonly ALWAYS = 519;
        /** Passed to stencilOperation to specify that stencil value must be kept */
        static readonly KEEP = 7680;
        /** Passed to stencilOperation to specify that stencil value must be replaced */
        static readonly REPLACE = 7681;
        constructor();
        reset(): void;
        func: number;
        get stencilFunc(): number;
        set stencilFunc(value: number);
        funcRef: number;
        get stencilFuncRef(): number;
        set stencilFuncRef(value: number);
        funcMask: number;
        get stencilFuncMask(): number;
        set stencilFuncMask(value: number);
        opStencilFail: number;
        get stencilOpStencilFail(): number;
        set stencilOpStencilFail(value: number);
        opDepthFail: number;
        get stencilOpDepthFail(): number;
        set stencilOpDepthFail(value: number);
        opStencilDepthPass: number;
        get stencilOpStencilDepthPass(): number;
        set stencilOpStencilDepthPass(value: number);
        mask: number;
        get stencilMask(): number;
        set stencilMask(value: number);
        enabled: boolean;
        get stencilTest(): boolean;
        set stencilTest(value: boolean);
    }


    /**
     * @internal
     **/
    export class StencilStateComposer {
        protected _isStencilTestDirty: boolean;
        protected _isStencilMaskDirty: boolean;
        protected _isStencilFuncDirty: boolean;
        protected _isStencilOpDirty: boolean;
        protected _enabled: boolean;
        protected _mask: number;
        protected _func: number;
        protected _funcRef: number;
        protected _funcMask: number;
        protected _opStencilFail: number;
        protected _opDepthFail: number;
        protected _opStencilDepthPass: number;
        stencilGlobal: IStencilState;
        stencilMaterial: IStencilState | undefined;
        useStencilGlobalOnly: boolean;
        get isDirty(): boolean;
        get func(): number;
        set func(value: number);
        get funcRef(): number;
        set funcRef(value: number);
        get funcMask(): number;
        set funcMask(value: number);
        get opStencilFail(): number;
        set opStencilFail(value: number);
        get opDepthFail(): number;
        set opDepthFail(value: number);
        get opStencilDepthPass(): number;
        set opStencilDepthPass(value: number);
        get mask(): number;
        set mask(value: number);
        get enabled(): boolean;
        set enabled(value: boolean);
        constructor(reset?: boolean);
        reset(): void;
        apply(gl?: WebGLRenderingContext): void;
    }


    /** Alias type for value that can be null */
    export type Nullable<T> = T | null;
    /**
     * Alias type for number that are floats
     * @ignorenaming
     */
    export type float = number;
    /**
     * Alias type for number that are doubles.
     * @ignorenaming
     */
    export type double = number;
    /**
     * Alias type for number that are integer
     * @ignorenaming
     */
    export type int = number;
    /** Alias type for number array or Float32Array */
    export type FloatArray = number[] | Float32Array;
    /** Alias type for number array or Float32Array or Int32Array or Uint32Array or Uint16Array */
    export type IndicesArray = number[] | Int32Array | Uint32Array | Uint16Array;
    /**
     * Alias for types that can be used by a Buffer or VertexBuffer.
     */
    export type DataArray = number[] | ArrayBuffer | ArrayBufferView;
    /**
     * Alias type for primitive types
     * @ignorenaming
     */
    type Primitive = undefined | null | boolean | string | number | Function | Element;
    /**
     * Type modifier to make all the properties of an object Readonly
     */
    export type Immutable<T> = T extends Primitive ? T : T extends Array<infer U> ? ReadonlyArray<U> : DeepImmutable<T>;
    /**
     * Type modifier to make all the properties of an object Readonly recursively
     */
    export type DeepImmutable<T> = T extends Primitive ? T : T extends Array<infer U> ? DeepImmutableArray<U> : DeepImmutableObject<T>;
    /**
     * Type modifier to make object properties readonly.
     */
    export type DeepImmutableObject<T> = {
        readonly [K in keyof T]: DeepImmutable<T[K]>;
    };
    /** @internal */
    interface DeepImmutableArray<T> extends ReadonlyArray<DeepImmutable<T>> {
    }
    /** @internal */




    /**
     * This is the base class for all WebXR features.
     * Since most features require almost the same resources and callbacks, this class can be used to simplify the development
     * Note that since the features manager is using the `IWebXRFeature` you are in no way obligated to use this class
     */
    export abstract class WebXRAbstractFeature implements IWebXRFeature {
        protected _xrSessionManager: WebXRSessionManager;
        private _attached;
        private _removeOnDetach;
        /**
         * Is this feature disposed?
         */
        isDisposed: boolean;
        /**
         * Should auto-attach be disabled?
         */
        disableAutoAttach: boolean;
        /**
         * The name of the native xr feature name (like anchor, hit-test, or hand-tracking)
         */
        xrNativeFeatureName: string;
        /**
         * Construct a new (abstract) WebXR feature
         * @param _xrSessionManager the xr session manager for this feature
         */
        constructor(_xrSessionManager: WebXRSessionManager);
        /**
         * Is this feature attached
         */
        get attached(): boolean;
        /**
         * attach this feature
         *
         * @param force should attachment be forced (even when already attached)
         * @returns true if successful, false is failed or already attached
         */
        attach(force?: boolean): boolean;
        /**
         * detach this feature.
         *
         * @returns true if successful, false if failed or already detached
         */
        detach(): boolean;
        /**
         * Dispose this feature and all of the resources attached
         */
        dispose(): void;
        /**
         * This function will be executed during before enabling the feature and can be used to not-allow enabling it.
         * Note that at this point the session has NOT started, so this is purely checking if the browser supports it
         *
         * @returns whether or not the feature is compatible in this environment
         */
        isCompatible(): boolean;
        /**
         * This is used to register callbacks that will automatically be removed when detach is called.
         * @param observable the observable to which the observer will be attached
         * @param callback the callback to register
         */
        protected _addNewAttachObserver<T>(observable: Observable<T>, callback: (eventData: T, eventState: EventState) => void): void;
        /**
         * Code in this function will be executed on each xrFrame received from the browser.
         * This function will not execute after the feature is detached.
         * @param _xrFrame the current frame
         */
        protected abstract _onXRFrame(_xrFrame: XRFrame): void;
    }


    /**
     * Configuration options of the anchor system
     */
    export interface IWebXRAnchorSystemOptions {
        /**
         * a node that will be used to convert local to world coordinates
         */
        worldParentNode?: TransformNode;
        /**
         * If set to true a reference of the created anchors will be kept until the next session starts
         * If not defined, anchors will be removed from the array when the feature is detached or the session ended.
         */
        doNotRemoveAnchorsOnSessionEnded?: boolean;
    }
    /**
     * A babylon container for an XR Anchor
     */
    export interface IWebXRAnchor {
        /**
         * A babylon-assigned ID for this anchor
         */
        id: number;
        /**
         * Transformation matrix to apply to an object attached to this anchor
         */
        transformationMatrix: Matrix;
        /**
         * The native anchor object
         */
        xrAnchor: XRAnchor;
        /**
         * if defined, this object will be constantly updated by the anchor's position and rotation
         */
        attachedNode?: TransformNode;
        /**
         * Remove this anchor from the scene
         */
        remove(): void;
    }
    /**
     * An implementation of the anchor system for WebXR.
     * For further information see https://github.com/immersive-web/anchors/
     */
    export class WebXRAnchorSystem extends WebXRAbstractFeature {
        private _options;
        private _lastFrameDetected;
        private _trackedAnchors;
        private _referenceSpaceForFrameAnchors;
        private _futureAnchors;
        /**
         * The module's name
         */
        static readonly Name = "xr-anchor-system";
        /**
         * The (Babylon) version of this module.
         * This is an integer representing the implementation version.
         * This number does not correspond to the WebXR specs version
         */
        static readonly Version = 1;
        /**
         * Observers registered here will be executed when a new anchor was added to the session
         */
        onAnchorAddedObservable: Observable<IWebXRAnchor>;
        /**
         * Observers registered here will be executed when an anchor was removed from the session
         */
        onAnchorRemovedObservable: Observable<IWebXRAnchor>;
        /**
         * Observers registered here will be executed when an existing anchor updates
         * This can execute N times every frame
         */
        onAnchorUpdatedObservable: Observable<IWebXRAnchor>;
        /**
         * Set the reference space to use for anchor creation, when not using a hit test.
         * Will default to the session's reference space if not defined
         */
        set referenceSpaceForFrameAnchors(referenceSpace: XRReferenceSpace);
        /**
         * constructs a new anchor system
         * @param _xrSessionManager an instance of WebXRSessionManager
         * @param _options configuration object for this feature
         */
        constructor(_xrSessionManager: WebXRSessionManager, _options?: IWebXRAnchorSystemOptions);
        private _tmpVector;
        private _tmpQuaternion;
        private _populateTmpTransformation;
        /**
         * Create a new anchor point using a hit test result at a specific point in the scene
         * An anchor is tracked only after it is added to the trackerAnchors in xrFrame. The promise returned here does not yet guaranty that.
         * Use onAnchorAddedObservable to get newly added anchors if you require tracking guaranty.
         *
         * @param hitTestResult The hit test result to use for this anchor creation
         * @param position an optional position offset for this anchor
         * @param rotationQuaternion an optional rotation offset for this anchor
         * @returns A promise that fulfills when babylon has created the corresponding WebXRAnchor object and tracking has begun
         */
        addAnchorPointUsingHitTestResultAsync(hitTestResult: IWebXRHitResult, position?: Vector3, rotationQuaternion?: Quaternion): Promise<IWebXRAnchor>;
        /**
         * Add a new anchor at a specific position and rotation
         * This function will add a new anchor per default in the next available frame. Unless forced, the createAnchor function
         * will be called in the next xrFrame loop to make sure that the anchor can be created correctly.
         * An anchor is tracked only after it is added to the trackerAnchors in xrFrame. The promise returned here does not yet guaranty that.
         * Use onAnchorAddedObservable to get newly added anchors if you require tracking guaranty.
         *
         * @param position the position in which to add an anchor
         * @param rotationQuaternion an optional rotation for the anchor transformation
         * @param forceCreateInCurrentFrame force the creation of this anchor in the current frame. Must be called inside xrFrame loop!
         * @returns A promise that fulfills when babylon has created the corresponding WebXRAnchor object and tracking has begun
         */
        addAnchorAtPositionAndRotationAsync(position: Vector3, rotationQuaternion?: Quaternion, forceCreateInCurrentFrame?: boolean): Promise<IWebXRAnchor>;
        /**
         * Get the list of anchors currently being tracked by the system
         */
        get anchors(): IWebXRAnchor[];
        /**
         * detach this feature.
         * Will usually be called by the features manager
         *
         * @returns true if successful.
         */
        detach(): boolean;
        /**
         * Dispose this feature and all of the resources attached
         */
        dispose(): void;
        protected _onXRFrame(frame: XRFrame): void;
        /**
         * avoiding using Array.find for global support.
         * @param xrAnchor the plane to find in the array
         */
        private _findIndexInAnchorArray;
        private _updateAnchorWithXRFrame;
        private _createAnchorAtTransformation;
    }


    /**
     * Options interface for the background remover plugin
     */
    export interface IWebXRBackgroundRemoverOptions {
        /**
         * Further background meshes to disable when entering AR
         */
        backgroundMeshes?: AbstractMesh[];
        /**
         * flags to configure the removal of the environment helper.
         * If not set, the entire background will be removed. If set, flags should be set as well.
         */
        environmentHelperRemovalFlags?: {
            /**
             * Should the skybox be removed (default false)
             */
            skyBox?: boolean;
            /**
             * Should the ground be removed (default false)
             */
            ground?: boolean;
        };
        /**
         * don't disable the environment helper
         */
        ignoreEnvironmentHelper?: boolean;
    }
    /**
     * A module that will automatically disable background meshes when entering AR and will enable them when leaving AR.
     */
    export class WebXRBackgroundRemover extends WebXRAbstractFeature {
        /**
         * read-only options to be used in this module
         */
        readonly options: IWebXRBackgroundRemoverOptions;
        /**
         * The module's name
         */
        static readonly Name = "xr-background-remover";
        /**
         * The (Babylon) version of this module.
         * This is an integer representing the implementation version.
         * This number does not correspond to the WebXR specs version
         */
        static readonly Version = 1;
        /**
         * registered observers will be triggered when the background state changes
         */
        onBackgroundStateChangedObservable: Observable<boolean>;
        /**
         * constructs a new background remover module
         * @param _xrSessionManager the session manager for this module
         * @param options read-only options to be used in this module
         */
        constructor(_xrSessionManager: WebXRSessionManager, 
        /**
         * read-only options to be used in this module
         */
        options?: IWebXRBackgroundRemoverOptions);
        /**
         * attach this feature
         * Will usually be called by the features manager
         *
         * @returns true if successful.
         */
        attach(): boolean;
        /**
         * detach this feature.
         * Will usually be called by the features manager
         *
         * @returns true if successful.
         */
        detach(): boolean;
        /**
         * Dispose this feature and all of the resources attached
         */
        dispose(): void;
        protected _onXRFrame(_xrFrame: XRFrame): void;
        private _setBackgroundState;
    }


    /**
     * The options container for the controller movement module
     */
    export interface IWebXRControllerMovementOptions {
        /**
         * Override default behaviour and provide your own movement controls
         */
        customRegistrationConfigurations?: WebXRControllerMovementRegistrationConfiguration[];
        /**
         * Is movement enabled
         */
        movementEnabled?: boolean;
        /**
         * Camera direction follows view pose and movement by default will move independently of the viewer's pose.
         */
        movementOrientationFollowsViewerPose: boolean;
        /**
         * Movement speed factor (default is 1.0)
         */
        movementSpeed?: number;
        /**
         * Minimum threshold the controller's thumbstick/touchpad must pass before being recognized for movement (avoids jitter/unintentional movement)
         */
        movementThreshold?: number;
        /**
         * Is rotation enabled
         */
        rotationEnabled?: boolean;
        /**
         * Minimum threshold the controller's thumstick/touchpad must pass before being recognized for rotation (avoids jitter/unintentional rotation)
         */
        rotationThreshold?: number;
        /**
         * Movement speed factor (default is 1.0)
         */
        rotationSpeed?: number;
        /**
         * Babylon XR Input class for controller
         */
        xrInput: WebXRInput;
    }
    /**
     * Feature context is used in handlers and on each XR frame to control the camera movement/direction.
     */
    export type WebXRControllerMovementFeatureContext = {
        movementEnabled: boolean;
        movementOrientationFollowsViewerPose: boolean;
        movementSpeed: number;
        movementThreshold: number;
        rotationEnabled: boolean;
        rotationSpeed: number;
        rotationThreshold: number;
    };
    /**
     * Current state of Movements shared across components and handlers.
     */
    export type WebXRControllerMovementState = {
        moveX: number;
        moveY: number;
        rotateX: number;
        rotateY: number;
    };
    /**
     * Button of Axis Handler must be specified.
     */
    export type WebXRControllerMovementRegistrationConfiguration = {
        /**
         * handlers are filtered to these types only
         */
        allowedComponentTypes?: MotionControllerComponentType[];
        /**
         * For registering movement to specific hand only.  Useful if your app has a "main hand" and "off hand" for determining the functionality of a controller.
         */
        forceHandedness?: XRHandedness;
        /**
         * For main component only (useful for buttons and may not trigger axis changes).
         */
        mainComponentOnly?: boolean;
        /**
         * Additional predicate to apply to controllers to restrict a handler being added.
         */
        componentSelectionPredicate?: (xrController: WebXRInputSource) => Nullable<WebXRControllerComponent>;
    } & ({
        /**
         * Called when axis changes occur.
         */
        axisChangedHandler: (axes: IWebXRMotionControllerAxesValue, movementState: WebXRControllerMovementState, featureContext: WebXRControllerMovementFeatureContext, xrInput: WebXRInput) => void;
    } | {
        /**
         * Called when the button state changes.
         */
        buttonChangedhandler: (pressed: IWebXRMotionControllerComponentChangesValues<boolean>, movementState: WebXRControllerMovementState, featureContext: WebXRControllerMovementFeatureContext, xrInput: WebXRInput) => void;
    });
    /**
     * This is a movement feature to be used with WebXR-enabled motion controllers.
     * When enabled and attached, the feature will allow a user to move around and rotate in the scene using
     * the input of the attached controllers.
     */
    export class WebXRControllerMovement extends WebXRAbstractFeature {
        private _controllers;
        private _currentRegistrationConfigurations;
        private _featureContext;
        private _movementDirection;
        private _movementState;
        private _xrInput;
        private _tmpRotationMatrix;
        private _tmpTranslationDirection;
        private _tmpMovementTranslation;
        /**
         * The module's name
         */
        static readonly Name = "xr-controller-movement";
        /**
         * Standard controller configurations.
         */
        static readonly REGISTRATIONS: {
            [key: string]: WebXRControllerMovementRegistrationConfiguration[];
        };
        /**
         * The (Babylon) version of this module.
         * This is an integer representing the implementation version.
         * This number does not correspond to the webxr specs version
         */
        static readonly Version = 1;
        /**
         * Current movement direction.  Will be null before XR Frames have been processed.
         */
        get movementDirection(): Nullable<Quaternion>;
        /**
         * Is movement enabled
         */
        get movementEnabled(): boolean;
        /**
         * Sets whether movement is enabled or not
         * @param enabled is movement enabled
         */
        set movementEnabled(enabled: boolean);
        /**
         * If movement follows viewer pose
         */
        get movementOrientationFollowsViewerPose(): boolean;
        /**
         * Sets whether movement follows viewer pose
         * @param followsPose is movement should follow viewer pose
         */
        set movementOrientationFollowsViewerPose(followsPose: boolean);
        /**
         * Gets movement speed
         */
        get movementSpeed(): number;
        /**
         * Sets movement speed
         * @param movementSpeed movement speed
         */
        set movementSpeed(movementSpeed: number);
        /**
         * Gets minimum threshold the controller's thumbstick/touchpad must pass before being recognized for movement (avoids jitter/unintentional movement)
         */
        get movementThreshold(): number;
        /**
         * Sets minimum threshold the controller's thumbstick/touchpad must pass before being recognized for movement (avoids jitter/unintentional movement)
         * @param movementThreshold new threshold
         */
        set movementThreshold(movementThreshold: number);
        /**
         * Is rotation enabled
         */
        get rotationEnabled(): boolean;
        /**
         * Sets whether rotation is enabled or not
         * @param enabled is rotation enabled
         */
        set rotationEnabled(enabled: boolean);
        /**
         * Gets rotation speed factor
         */
        get rotationSpeed(): number;
        /**
         * Sets rotation speed factor (1.0 is default)
         * @param rotationSpeed new rotation speed factor
         */
        set rotationSpeed(rotationSpeed: number);
        /**
         * Gets minimum threshold the controller's thumbstick/touchpad must pass before being recognized for rotation (avoids jitter/unintentional rotation)
         */
        get rotationThreshold(): number;
        /**
         * Sets minimum threshold the controller's thumbstick/touchpad must pass before being recognized for rotation (avoids jitter/unintentional rotation)
         * @param threshold new threshold
         */
        set rotationThreshold(threshold: number);
        /**
         * constructs a new movement controller system
         * @param _xrSessionManager an instance of WebXRSessionManager
         * @param options configuration object for this feature
         */
        constructor(_xrSessionManager: WebXRSessionManager, options: IWebXRControllerMovementOptions);
        attach(): boolean;
        detach(): boolean;
        /**
         * Occurs on every XR frame.
         * @param _xrFrame
         */
        protected _onXRFrame(_xrFrame: XRFrame): void;
        private _attachController;
        private _detachController;
    }


    /**
     * Options for the controller physics feature
     */
    export class IWebXRControllerPhysicsOptions {
        /**
         * Should the headset get its own impostor
         */
        enableHeadsetImpostor?: boolean;
        /**
         * Optional parameters for the headset impostor
         */
        headsetImpostorParams?: {
            /**
             * The type of impostor to create. Default is sphere
             */
            impostorType: number;
            /**
             * the size of the impostor. Defaults to 10cm
             */
            impostorSize?: number | {
                width: number;
                height: number;
                depth: number;
            };
            /**
             * Friction definitions
             */
            friction?: number;
            /**
             * Restitution
             */
            restitution?: number;
        };
        /**
         * The physics properties of the future impostors
         */
        physicsProperties?: {
            /**
             * If set to true, a mesh impostor will be created when the controller mesh was loaded
             * Note that this requires a physics engine that supports mesh impostors!
             */
            useControllerMesh?: boolean;
            /**
             * The type of impostor to create. Default is sphere
             */
            impostorType?: number;
            /**
             * the size of the impostor. Defaults to 10cm
             */
            impostorSize?: number | {
                width: number;
                height: number;
                depth: number;
            };
            /**
             * Friction definitions
             */
            friction?: number;
            /**
             * Restitution
             */
            restitution?: number;
        };
        /**
         * the xr input to use with this pointer selection
         */
        xrInput: WebXRInput;
    }
    /**
     * Add physics impostor to your webxr controllers,
     * including naive calculation of their linear and angular velocity
     */
    export class WebXRControllerPhysics extends WebXRAbstractFeature {
        private readonly _options;
        private _attachController;
        private _createPhysicsImpostor;
        private _controllers;
        private _debugMode;
        private _delta;
        private _headsetImpostor?;
        private _headsetMesh?;
        private _lastTimestamp;
        private _tmpQuaternion;
        private _tmpVector;
        /**
         * The module's name
         */
        static readonly Name = "xr-physics-controller";
        /**
         * The (Babylon) version of this module.
         * This is an integer representing the implementation version.
         * This number does not correspond to the webxr specs version
         */
        static readonly Version = 1;
        /**
         * Construct a new Controller Physics Feature
         * @param _xrSessionManager the corresponding xr session manager
         * @param _options options to create this feature with
         */
        constructor(_xrSessionManager: WebXRSessionManager, _options: IWebXRControllerPhysicsOptions);
        /**
         * @internal
         * enable debugging - will show console outputs and the impostor mesh
         */
        _enablePhysicsDebug(): void;
        /**
         * Manually add a controller (if no xrInput was provided or physics engine was not enabled)
         * @param xrController the controller to add
         */
        addController(xrController: WebXRInputSource): void;
        /**
         * attach this feature
         * Will usually be called by the features manager
         *
         * @returns true if successful.
         */
        attach(): boolean;
        /**
         * detach this feature.
         * Will usually be called by the features manager
         *
         * @returns true if successful.
         */
        detach(): boolean;
        /**
         * Get the headset impostor, if enabled
         * @returns the impostor
         */
        getHeadsetImpostor(): PhysicsImpostor | undefined;
        /**
         * Get the physics impostor of a specific controller.
         * The impostor is not attached to a mesh because a mesh for each controller is not obligatory
         * @param controller the controller or the controller id of which to get the impostor
         * @returns the impostor or null
         */
        getImpostorForController(controller: WebXRInputSource | string): Nullable<PhysicsImpostor>;
        /**
         * Update the physics properties provided in the constructor
         * @param newProperties the new properties object
         * @param newProperties.impostorType
         * @param newProperties.impostorSize
         * @param newProperties.friction
         * @param newProperties.restitution
         */
        setPhysicsProperties(newProperties: {
            impostorType?: number;
            impostorSize?: number | {
                width: number;
                height: number;
                depth: number;
            };
            friction?: number;
            restitution?: number;
        }): void;
        protected _onXRFrame(_xrFrame: any): void;
        private _detachController;
    }


    /**
     * Options interface for the pointer selection module
     */
    export interface IWebXRControllerPointerSelectionOptions {
        /**
         * if provided, this scene will be used to render meshes.
         */
        customUtilityLayerScene?: Scene;
        /**
         * Disable the pointer up event when the xr controller in screen and gaze mode is disposed (meaning - when the user removed the finger from the screen)
         * If not disabled, the last picked point will be used to execute a pointer up event
         * If disabled, pointer up event will be triggered right after the pointer down event.
         * Used in screen and gaze target ray mode only
         */
        disablePointerUpOnTouchOut: boolean;
        /**
         * For gaze mode for tracked-pointer / controllers (time to select instead of button press)
         */
        forceGazeMode: boolean;
        /**
         * Factor to be applied to the pointer-moved function in the gaze mode. How sensitive should the gaze mode be when checking if the pointer moved
         * to start a new countdown to the pointer down event.
         * Defaults to 1.
         */
        gazeModePointerMovedFactor?: number;
        /**
         * Different button type to use instead of the main component
         */
        overrideButtonId?: string;
        /**
         *  use this rendering group id for the meshes (optional)
         */
        renderingGroupId?: number;
        /**
         * The amount of time in milliseconds it takes between pick found something to a pointer down event.
         * Used in gaze modes. Tracked pointer uses the trigger, screen uses touch events
         * 3000 means 3 seconds between pointing at something and selecting it
         */
        timeToSelect?: number;
        /**
         * Should meshes created here be added to a utility layer or the main scene
         */
        useUtilityLayer?: boolean;
        /**
         * Optional WebXR camera to be used for gaze selection
         */
        gazeCamera?: WebXRCamera;
        /**
         * the xr input to use with this pointer selection
         */
        xrInput: WebXRInput;
        /**
         * Should the scene pointerX and pointerY update be disabled
         * This is required for fullscreen AR GUI, but might slow down other experiences.
         * Disable in VR, if not needed.
         * The first rig camera (left eye) will be used to calculate the projection
         */
        disableScenePointerVectorUpdate: boolean;
        /**
         * Enable pointer selection on all controllers instead of switching between them
         */
        enablePointerSelectionOnAllControllers?: boolean;
        /**
         * The preferred hand to give the pointer selection to. This will be prioritized when the controller initialize.
         * If switch is enabled, it will still allow the user to switch between the different controllers
         */
        preferredHandedness?: XRHandedness;
        /**
         * Disable switching the pointer selection from one controller to the other.
         * If the preferred hand is set it will be fixed on this hand, and if not it will be fixed on the first controller added to the scene
         */
        disableSwitchOnClick?: boolean;
        /**
         * The maximum distance of the pointer selection feature. Defaults to 100.
         */
        maxPointerDistance?: number;
        /**
         * A function that will be called when a new selection mesh is generated.
         * This function should return a mesh that will be used as the selection mesh.
         * The default is a torus with a 0.01 diameter and 0.0075 thickness .
         */
        customSelectionMeshGenerator?: () => Mesh;
        /**
         * A function that will be called when a new laser pointer mesh is generated.
         * This function should return a mesh that will be used as the laser pointer mesh.
         * The height (y) of the mesh must be 1.
         */
        customLasterPointerMeshGenerator?: () => AbstractMesh;
    }
    /**
     * A module that will enable pointer selection for motion controllers of XR Input Sources
     */
    export class WebXRControllerPointerSelection extends WebXRAbstractFeature {
        private readonly _options;
        private static _IdCounter;
        private _attachController;
        private _controllers;
        private _scene;
        private _tmpVectorForPickCompare;
        private _attachedController;
        /**
         * The module's name
         */
        static readonly Name = "xr-controller-pointer-selection";
        /**
         * The (Babylon) version of this module.
         * This is an integer representing the implementation version.
         * This number does not correspond to the WebXR specs version
         */
        static readonly Version = 1;
        /**
         * Disable lighting on the laser pointer (so it will always be visible)
         */
        disablePointerLighting: boolean;
        /**
         * Disable lighting on the selection mesh (so it will always be visible)
         */
        disableSelectionMeshLighting: boolean;
        /**
         * Should the laser pointer be displayed
         */
        displayLaserPointer: boolean;
        /**
         * Should the selection mesh be displayed (The ring at the end of the laser pointer)
         */
        displaySelectionMesh: boolean;
        /**
         * This color will be set to the laser pointer when selection is triggered
         */
        laserPointerPickedColor: Color3;
        /**
         * Default color of the laser pointer
         */
        laserPointerDefaultColor: Color3;
        /**
         * default color of the selection ring
         */
        selectionMeshDefaultColor: Color3;
        /**
         * This color will be applied to the selection ring when selection is triggered
         */
        selectionMeshPickedColor: Color3;
        /**
         * Optional filter to be used for ray selection.  This predicate shares behavior with
         * scene.pointerMovePredicate which takes priority if it is also assigned.
         */
        raySelectionPredicate: (mesh: AbstractMesh) => boolean;
        /**
         * constructs a new background remover module
         * @param _xrSessionManager the session manager for this module
         * @param _options read-only options to be used in this module
         */
        constructor(_xrSessionManager: WebXRSessionManager, _options: IWebXRControllerPointerSelectionOptions);
        /**
         * attach this feature
         * Will usually be called by the features manager
         *
         * @returns true if successful.
         */
        attach(): boolean;
        /**
         * detach this feature.
         * Will usually be called by the features manager
         *
         * @returns true if successful.
         */
        detach(): boolean;
        /**
         * Will get the mesh under a specific pointer.
         * `scene.meshUnderPointer` will only return one mesh - either left or right.
         * @param controllerId the controllerId to check
         * @returns The mesh under pointer or null if no mesh is under the pointer
         */
        getMeshUnderPointer(controllerId: string): Nullable<AbstractMesh>;
        /**
         * Get the xr controller that correlates to the pointer id in the pointer event
         *
         * @param id the pointer id to search for
         * @returns the controller that correlates to this id or null if not found
         */
        getXRControllerByPointerId(id: number): Nullable<WebXRInputSource>;
        /**
         * @internal
         */
        _getPointerSelectionDisabledByPointerId(id: number): boolean;
        /**
         * @internal
         */
        _setPointerSelectionDisabledByPointerId(id: number, state: boolean): void;
        private _identityMatrix;
        private _screenCoordinatesRef;
        private _viewportRef;
        protected _onXRFrame(_xrFrame: XRFrame): void;
        private get _utilityLayerScene();
        private _attachGazeMode;
        private _attachScreenRayMode;
        private _attachTrackedPointerRayMode;
        private _convertNormalToDirectionOfRay;
        private _detachController;
        private _generateNewMeshPair;
        private _pickingMoved;
        private _updatePointerDistance;
        private _augmentPointerInit;
        /** @internal */
        get lasterPointerDefaultColor(): Color3;
    }


    /**
     * The options container for the teleportation module
     */
    export interface IWebXRTeleportationOptions {
        /**
         * if provided, this scene will be used to render meshes.
         */
        customUtilityLayerScene?: Scene;
        /**
         * Values to configure the default target mesh
         */
        defaultTargetMeshOptions?: {
            /**
             * Fill color of the teleportation area
             */
            teleportationFillColor?: string;
            /**
             * Border color for the teleportation area
             */
            teleportationBorderColor?: string;
            /**
             * Disable the mesh's animation sequence
             */
            disableAnimation?: boolean;
            /**
             * Disable lighting on the material or the ring and arrow
             */
            disableLighting?: boolean;
            /**
             * Override the default material of the torus and arrow
             */
            torusArrowMaterial?: Material;
            /**
             * Override the default material of the Landing Zone
             */
            teleportationCircleMaterial?: Material;
        };
        /**
         * A list of meshes to use as floor meshes.
         * Meshes can be added and removed after initializing the feature using the
         * addFloorMesh and removeFloorMesh functions
         * If empty, rotation will still work
         */
        floorMeshes?: AbstractMesh[];
        /**
         *  use this rendering group id for the meshes (optional)
         */
        renderingGroupId?: number;
        /**
         * Should teleportation move only to snap points
         */
        snapPointsOnly?: boolean;
        /**
         * An array of points to which the teleportation will snap to.
         * If the teleportation ray is in the proximity of one of those points, it will be corrected to this point.
         */
        snapPositions?: Vector3[];
        /**
         * How close should the teleportation ray be in order to snap to position.
         * Default to 0.8 units (meters)
         */
        snapToPositionRadius?: number;
        /**
         * Provide your own teleportation mesh instead of babylon's wonderful doughnut.
         * If you want to support rotation, make sure your mesh has a direction indicator.
         *
         * When left untouched, the default mesh will be initialized.
         */
        teleportationTargetMesh?: AbstractMesh;
        /**
         * If main component is used (no thumbstick), how long should the "long press" take before teleport
         */
        timeToTeleport?: number;
        /**
         * Disable using the thumbstick and use the main component (usually trigger) on long press.
         * This will be automatically true if the controller doesn't have a thumbstick or touchpad.
         */
        useMainComponentOnly?: boolean;
        /**
         * Should meshes created here be added to a utility layer or the main scene
         */
        useUtilityLayer?: boolean;
        /**
         * Babylon XR Input class for controller
         */
        xrInput: WebXRInput;
        /**
         * Meshes that the teleportation ray cannot go through
         */
        pickBlockerMeshes?: AbstractMesh[];
        /**
         * Should teleport work only on a specific hand?
         */
        forceHandedness?: XRHandedness;
        /**
         * If provided, this function will be used to generate the ray mesh instead of the lines mesh being used per default
         */
        generateRayPathMesh?: (points: Vector3[], pickingInfo: PickingInfo) => AbstractMesh;
    }
    /**
     * This is a teleportation feature to be used with WebXR-enabled motion controllers.
     * When enabled and attached, the feature will allow a user to move around and rotate in the scene using
     * the input of the attached controllers.
     */
    export class WebXRMotionControllerTeleportation extends WebXRAbstractFeature {
        private _options;
        private _controllers;
        private _currentTeleportationControllerId;
        private _floorMeshes;
        private _quadraticBezierCurve;
        private _selectionFeature;
        private _snapToPositions;
        private _snappedToPoint;
        private _teleportationRingMaterial?;
        private _tmpRay;
        private _tmpVector;
        private _tmpQuaternion;
        /**
         * Skip the next teleportation. This can be controlled by the user to prevent the user from teleportation
         * to sections that are not yet "unlocked", but should still show the teleportation mesh.
         */
        skipNextTeleportation: boolean;
        /**
         * The module's name
         */
        static readonly Name = "xr-controller-teleportation";
        /**
         * The (Babylon) version of this module.
         * This is an integer representing the implementation version.
         * This number does not correspond to the webxr specs version
         */
        static readonly Version = 1;
        /**
         * Is movement backwards enabled
         */
        backwardsMovementEnabled: boolean;
        /**
         * Distance to travel when moving backwards
         */
        backwardsTeleportationDistance: number;
        /**
         * The distance from the user to the inspection point in the direction of the controller
         * A higher number will allow the user to move further
         * defaults to 5 (meters, in xr units)
         */
        parabolicCheckRadius: number;
        /**
         * Should the module support parabolic ray on top of direct ray
         * If enabled, the user will be able to point "at the sky" and move according to predefined radius distance
         * Very helpful when moving between floors / different heights
         */
        parabolicRayEnabled: boolean;
        /**
         * The second type of ray - straight line.
         * Should it be enabled or should the parabolic line be the only one.
         */
        straightRayEnabled: boolean;
        /**
         * How much rotation should be applied when rotating right and left
         */
        rotationAngle: number;
        /**
         * This observable will notify when the target mesh position was updated.
         * The picking info it provides contains the point to which the target mesh will move ()
         */
        onTargetMeshPositionUpdatedObservable: Observable<PickingInfo>;
        /**
         * Is teleportation enabled. Can be used to allow rotation only.
         */
        teleportationEnabled: boolean;
        private _rotationEnabled;
        /**
         * Is rotation enabled when moving forward?
         * Disabling this feature will prevent the user from deciding the direction when teleporting
         */
        get rotationEnabled(): boolean;
        /**
         * Sets whether rotation is enabled or not
         * @param enabled is rotation enabled when teleportation is shown
         */
        set rotationEnabled(enabled: boolean);
        /**
         * Exposes the currently set teleportation target mesh.
         */
        get teleportationTargetMesh(): Nullable<AbstractMesh>;
        /**
         * constructs a new teleportation system
         * @param _xrSessionManager an instance of WebXRSessionManager
         * @param _options configuration object for this feature
         */
        constructor(_xrSessionManager: WebXRSessionManager, _options: IWebXRTeleportationOptions);
        /**
         * Get the snapPointsOnly flag
         */
        get snapPointsOnly(): boolean;
        /**
         * Sets the snapPointsOnly flag
         * @param snapToPoints should teleportation be exclusively to snap points
         */
        set snapPointsOnly(snapToPoints: boolean);
        /**
         * Add a new mesh to the floor meshes array
         * @param mesh the mesh to use as floor mesh
         */
        addFloorMesh(mesh: AbstractMesh): void;
        /**
         * Add a mesh to the list of meshes blocking the teleportation ray
         * @param mesh The mesh to add to the teleportation-blocking meshes
         */
        addBlockerMesh(mesh: AbstractMesh): void;
        /**
         * Add a new snap-to point to fix teleportation to this position
         * @param newSnapPoint The new Snap-To point
         */
        addSnapPoint(newSnapPoint: Vector3): void;
        attach(): boolean;
        detach(): boolean;
        dispose(): void;
        /**
         * Remove a mesh from the floor meshes array
         * @param mesh the mesh to remove
         */
        removeFloorMesh(mesh: AbstractMesh): void;
        /**
         * Remove a mesh from the blocker meshes array
         * @param mesh the mesh to remove
         */
        removeBlockerMesh(mesh: AbstractMesh): void;
        /**
         * Remove a mesh from the floor meshes array using its name
         * @param name the mesh name to remove
         */
        removeFloorMeshByName(name: string): void;
        /**
         * This function will iterate through the array, searching for this point or equal to it. It will then remove it from the snap-to array
         * @param snapPointToRemove the point (or a clone of it) to be removed from the array
         * @returns was the point found and removed or not
         */
        removeSnapPoint(snapPointToRemove: Vector3): boolean;
        /**
         * This function sets a selection feature that will be disabled when
         * the forward ray is shown and will be reattached when hidden.
         * This is used to remove the selection rays when moving.
         * @param selectionFeature the feature to disable when forward movement is enabled
         */
        setSelectionFeature(selectionFeature: Nullable<IWebXRFeature>): void;
        protected _onXRFrame(_xrFrame: XRFrame): void;
        private _attachController;
        private _createDefaultTargetMesh;
        private _detachController;
        private _findClosestSnapPointWithRadius;
        private _setTargetMeshPosition;
        private _setTargetMeshVisibility;
        private _showParabolicPath;
        private _teleportForward;
    }


    /**
     * Options for DOM Overlay feature
     */
    export interface IWebXRDomOverlayOptions {
        /**
         * DOM Element or document query selector string for overlay.
         *
         * NOTE: UA may make this element background transparent in XR.
         */
        element: Element | string;
        /**
         * Supress XR Select events on container element (DOM blocks interaction to scene).
         */
        supressXRSelectEvents?: boolean;
    }
    /**
     * Type of DOM overlay provided by UA.
     */
    type WebXRDomOverlayType = 
    /**
     * Covers the entire physical screen for a screen-based device, for example handheld AR
     */
    "screen"
    /**
     * Appears as a floating rectangle in space
     */
     | "floating"
    /**
     * Follows the user’s head movement consistently, appearing similar to a HUD
     */
     | "head-locked";
    /**
     * DOM Overlay Feature
     *
     * @since 5.0.0
     */
    export class WebXRDomOverlay extends WebXRAbstractFeature {
        /**
         * options to use when constructing this feature
         */
        readonly options: IWebXRDomOverlayOptions;
        /**
         * Type of overlay - non-null when available
         */
        private _domOverlayType;
        /**
         * Event Listener to supress "beforexrselect" events.
         */
        private _beforeXRSelectListener;
        /**
         * Element used for overlay
         */
        private _element;
        /**
         * The module's name
         */
        static readonly Name = "xr-dom-overlay";
        /**
         * The (Babylon) version of this module.
         * This is an integer representing the implementation version.
         * This number does not correspond to the WebXR specs version
         */
        static readonly Version = 1;
        /**
         * Creates a new instance of the dom-overlay feature
         * @param _xrSessionManager an instance of WebXRSessionManager
         * @param options options to use when constructing this feature
         */
        constructor(_xrSessionManager: WebXRSessionManager, 
        /**
         * options to use when constructing this feature
         */
        options: IWebXRDomOverlayOptions);
        /**
         * attach this feature
         * Will usually be called by the features manager
         *
         * @returns true if successful.
         */
        attach(): boolean;
        /**
         * The type of DOM overlay (null when not supported).  Provided by UA and remains unchanged for duration of session.
         */
        get domOverlayType(): Nullable<WebXRDomOverlayType>;
        /**
         * Dispose this feature and all of the resources attached
         */
        dispose(): void;
        protected _onXRFrame(_xrFrame: XRFrame): void;
        /**
         * Extends the session init object if needed
         * @returns augmentation object for the xr session init object.
         */
        getXRSessionInitExtension(): Promise<Partial<XRSessionInit>>;
    }


    /**
     * The WebXR Eye Tracking feature grabs eye data from the device and provides it in an easy-access format.
     * Currently only enabled for BabylonNative applications.
     */
    export class WebXREyeTracking extends WebXRAbstractFeature {
        private _latestEyeSpace;
        private _gazeRay;
        /**
         * The module's name
         */
        static readonly Name = "xr-eye-tracking";
        /**
         * The (Babylon) version of this module.
         * This is an integer representing the implementation version.
         * This number does not correspond to the WebXR specs version
         */
        static readonly Version = 1;
        /**
         * This observable will notify registered observers when eye tracking starts
         */
        readonly onEyeTrackingStartedObservable: Observable<Ray>;
        /**
         * This observable will notify registered observers when eye tracking ends
         */
        readonly onEyeTrackingEndedObservable: Observable<void>;
        /**
         * This observable will notify registered observers on each frame that has valid tracking
         */
        readonly onEyeTrackingFrameUpdateObservable: Observable<Ray>;
        /**
         * Creates a new instance of the XR eye tracking feature.
         * @param _xrSessionManager An instance of WebXRSessionManager.
         */
        constructor(_xrSessionManager: WebXRSessionManager);
        /**
         * Dispose this feature and all of the resources attached.
         */
        dispose(): void;
        /**
         * Returns whether the gaze data is valid or not
         * @returns true if the data is valid
         */
        get isEyeGazeValid(): boolean;
        /**
         * Get a reference to the gaze ray. This data is valid while eye tracking persists, and will be set to null when gaze data is no longer available
         * @returns a reference to the gaze ray if it exists and is valid, returns null otherwise.
         */
        getEyeGaze(): Nullable<Ray>;
        protected _onXRFrame(frame: XRFrame): void;
        private _eyeTrackingStartListener;
        private _eyeTrackingEndListener;
        private _init;
    }


    /**
     * A babylon interface for a "WebXR" feature point.
     * Represents the position and confidence value of a given feature point.
     */
    export interface IWebXRFeaturePoint {
        /**
         * Represents the position of the feature point in world space.
         */
        position: Vector3;
        /**
         * Represents the confidence value of the feature point in world space. 0 being least confident, and 1 being most confident.
         */
        confidenceValue: number;
    }
    /**
     * The feature point system is used to detect feature points from real world geometry.
     * This feature is currently experimental and only supported on BabylonNative, and should not be used in the browser.
     * The newly introduced API can be seen in webxr.nativeextensions.d.ts and described in FeaturePoints.md.
     */
    export class WebXRFeaturePointSystem extends WebXRAbstractFeature {
        private _enabled;
        private _featurePointCloud;
        /**
         * The module's name
         */
        static readonly Name = "xr-feature-points";
        /**
         * The (Babylon) version of this module.
         * This is an integer representing the implementation version.
         * This number does not correspond to the WebXR specs version
         */
        static readonly Version = 1;
        /**
         * Observers registered here will be executed whenever new feature points are added (on XRFrame while the session is tracking).
         * Will notify the observers about which feature points have been added.
         */
        readonly onFeaturePointsAddedObservable: Observable<number[]>;
        /**
         * Observers registered here will be executed whenever a feature point has been updated (on XRFrame while the session is tracking).
         * Will notify the observers about which feature points have been updated.
         */
        readonly onFeaturePointsUpdatedObservable: Observable<number[]>;
        /**
         * The current feature point cloud maintained across frames.
         */
        get featurePointCloud(): Array<IWebXRFeaturePoint>;
        /**
         * construct the feature point system
         * @param _xrSessionManager an instance of xr Session manager
         */
        constructor(_xrSessionManager: WebXRSessionManager);
        /**
         * Detach this feature.
         * Will usually be called by the features manager
         *
         * @returns true if successful.
         */
        detach(): boolean;
        /**
         * Dispose this feature and all of the resources attached
         */
        dispose(): void;
        /**
         * On receiving a new XR frame if this feature is attached notify observers new feature point data is available.
         * @param frame
         */
        protected _onXRFrame(frame: XRFrame): void;
        /**
         * Initializes the feature. If the feature point feature is not available for this environment do not mark the feature as enabled.
         */
        private _init;
    }


    /**
     * Configuration interface for the hand tracking feature
     */
    export interface IWebXRHandTrackingOptions {
        /**
         * The xrInput that will be used as source for new hands
         */
        xrInput: WebXRInput;
        /**
         * Configuration object for the joint meshes.
         */
        jointMeshes?: {
            /**
             * Should the meshes created be invisible (defaults to false).
             */
            invisible?: boolean;
            /**
             * A source mesh to be used to create instances. Defaults to an icosphere with two subdivisions and smooth lighting.
             * This mesh will be the source for all other (25) meshes.
             * It should have the general size of a single unit, as the instances will be scaled according to the provided radius.
             */
            sourceMesh?: Mesh;
            /**
             * This function will be called after a mesh was created for a specific joint.
             * Using this function you can either manipulate the instance or return a new mesh.
             * When returning a new mesh the instance created before will be disposed.
             * @param meshInstance An instance of the original joint mesh being used for the joint.
             * @param jointId The joint's index, see https://immersive-web.github.io/webxr-hand-input/#skeleton-joints-section for more info.
             * @param hand Which hand ("left", "right") the joint will be on.
             */
            onHandJointMeshGenerated?: (meshInstance: InstancedMesh, jointId: number, hand: XRHandedness) => AbstractMesh | undefined;
            /**
             * Should the source mesh stay visible (defaults to false).
             */
            keepOriginalVisible?: boolean;
            /**
             * Should each instance have its own physics impostor
             */
            enablePhysics?: boolean;
            /**
             * If enabled, override default physics properties
             */
            physicsProps?: {
                friction?: number;
                restitution?: number;
                impostorType?: number;
            };
            /**
             * Scale factor for all joint meshes (defaults to 1)
             */
            scaleFactor?: number;
        };
        /**
         * Configuration object for the hand meshes.
         */
        handMeshes?: {
            /**
             * Should the default hand mesh be disabled. In this case, the spheres will be visible (unless set invisible).
             */
            disableDefaultMeshes?: boolean;
            /**
             * Rigged hand meshes that will be tracked to the user's hands. This will override the default hand mesh.
             */
            customMeshes?: {
                right: AbstractMesh;
                left: AbstractMesh;
            };
            /**
             * Are the meshes prepared for a left-handed system. Default hand meshes are right-handed.
             */
            meshesUseLeftHandedCoordinates?: boolean;
            /**
             * If a hand mesh was provided, this array will define what axis will update which node. This will override the default hand mesh
             */
            customRigMappings?: {
                right: XRHandMeshRigMapping;
                left: XRHandMeshRigMapping;
            };
            /**
             * Override the colors of the hand meshes.
             */
            customColors?: {
                base?: Color3;
                fresnel?: Color3;
                fingerColor?: Color3;
                tipFresnel?: Color3;
            };
        };
    }
    /**
     * Parts of the hands divided to writs and finger names
     */
    export enum HandPart {
        /**
         * HandPart - Wrist
         */
        WRIST = "wrist",
        /**
         * HandPart - The thumb
         */
        THUMB = "thumb",
        /**
         * HandPart - Index finger
         */
        INDEX = "index",
        /**
         * HandPart - Middle finger
         */
        MIDDLE = "middle",
        /**
         * HandPart - Ring finger
         */
        RING = "ring",
        /**
         * HandPart - Little finger
         */
        LITTLE = "little"
    }
    /**
     * Joints of the hand as defined by the WebXR specification.
     * https://immersive-web.github.io/webxr-hand-input/#skeleton-joints-section
     */
    export enum WebXRHandJoint {
        /** Wrist */
        WRIST = "wrist",
        /** Thumb near wrist */
        THUMB_METACARPAL = "thumb-metacarpal",
        /** Thumb first knuckle */
        THUMB_PHALANX_PROXIMAL = "thumb-phalanx-proximal",
        /** Thumb second knuckle */
        THUMB_PHALANX_DISTAL = "thumb-phalanx-distal",
        /** Thumb tip */
        THUMB_TIP = "thumb-tip",
        /** Index finger near wrist */
        INDEX_FINGER_METACARPAL = "index-finger-metacarpal",
        /** Index finger first knuckle */
        INDEX_FINGER_PHALANX_PROXIMAL = "index-finger-phalanx-proximal",
        /** Index finger second knuckle */
        INDEX_FINGER_PHALANX_INTERMEDIATE = "index-finger-phalanx-intermediate",
        /** Index finger third knuckle */
        INDEX_FINGER_PHALANX_DISTAL = "index-finger-phalanx-distal",
        /** Index finger tip */
        INDEX_FINGER_TIP = "index-finger-tip",
        /** Middle finger near wrist */
        MIDDLE_FINGER_METACARPAL = "middle-finger-metacarpal",
        /** Middle finger first knuckle */
        MIDDLE_FINGER_PHALANX_PROXIMAL = "middle-finger-phalanx-proximal",
        /** Middle finger second knuckle */
        MIDDLE_FINGER_PHALANX_INTERMEDIATE = "middle-finger-phalanx-intermediate",
        /** Middle finger third knuckle */
        MIDDLE_FINGER_PHALANX_DISTAL = "middle-finger-phalanx-distal",
        /** Middle finger tip */
        MIDDLE_FINGER_TIP = "middle-finger-tip",
        /** Ring finger near wrist */
        RING_FINGER_METACARPAL = "ring-finger-metacarpal",
        /** Ring finger first knuckle */
        RING_FINGER_PHALANX_PROXIMAL = "ring-finger-phalanx-proximal",
        /** Ring finger second knuckle */
        RING_FINGER_PHALANX_INTERMEDIATE = "ring-finger-phalanx-intermediate",
        /** Ring finger third knuckle */
        RING_FINGER_PHALANX_DISTAL = "ring-finger-phalanx-distal",
        /** Ring finger tip */
        RING_FINGER_TIP = "ring-finger-tip",
        /** Pinky finger near wrist */
        PINKY_FINGER_METACARPAL = "pinky-finger-metacarpal",
        /** Pinky finger first knuckle */
        PINKY_FINGER_PHALANX_PROXIMAL = "pinky-finger-phalanx-proximal",
        /** Pinky finger second knuckle */
        PINKY_FINGER_PHALANX_INTERMEDIATE = "pinky-finger-phalanx-intermediate",
        /** Pinky finger third knuckle */
        PINKY_FINGER_PHALANX_DISTAL = "pinky-finger-phalanx-distal",
        /** Pinky finger tip */
        PINKY_FINGER_TIP = "pinky-finger-tip"
    }
    /** A type encapsulating a dictionary mapping WebXR joints to bone names in a rigged hand mesh.  */
    export type XRHandMeshRigMapping = {
        [webXRJointName in WebXRHandJoint]: string;
    };
    /**
     * Representing a single hand (with its corresponding native XRHand object)
     */
    export class WebXRHand implements IDisposable {
        /** The controller to which the hand correlates. */
        readonly xrController: WebXRInputSource;
        private readonly _jointMeshes;
        private _handMesh;
        /** An optional rig mapping for the hand mesh. If not provided (but a hand mesh is provided),
         * it will be assumed that the hand mesh's bones are named directly after the WebXR bone names. */
        readonly rigMapping: Nullable<XRHandMeshRigMapping>;
        private readonly _leftHandedMeshes;
        private readonly _jointsInvisible;
        private readonly _jointScaleFactor;
        private _scene;
        /**
         * Transform nodes that will directly receive the transforms from the WebXR matrix data.
         */
        private _jointTransforms;
        /**
         * The float array that will directly receive the transform matrix data from WebXR.
         */
        private _jointTransformMatrices;
        private _tempJointMatrix;
        /**
         * The float array that will directly receive the joint radii from WebXR.
         */
        private _jointRadii;
        /**
         * Get the hand mesh.
         */
        get handMesh(): Nullable<AbstractMesh>;
        /**
         * Get meshes of part of the hand.
         * @param part The part of hand to get.
         * @returns An array of meshes that correlate to the hand part requested.
         */
        getHandPartMeshes(part: HandPart): AbstractMesh[];
        /**
         * Retrieves a mesh linked to a named joint in the hand.
         * @param jointName The name of the joint.
         * @returns An AbstractMesh whose position corresponds with the joint position.
         */
        getJointMesh(jointName: WebXRHandJoint): AbstractMesh;
        /**
         * Construct a new hand object
         * @param xrController The controller to which the hand correlates.
         * @param _jointMeshes The meshes to be used to track the hand joints.
         * @param _handMesh An optional hand mesh.
         * @param rigMapping An optional rig mapping for the hand mesh.
         *                   If not provided (but a hand mesh is provided),
         *                   it will be assumed that the hand mesh's bones are named
         *                   directly after the WebXR bone names.
         * @param _leftHandedMeshes Are the hand meshes left-handed-system meshes
         * @param _jointsInvisible Are the tracked joint meshes visible
         * @param _jointScaleFactor Scale factor for all joint meshes
         */
        constructor(
        /** The controller to which the hand correlates. */
        xrController: WebXRInputSource, _jointMeshes: AbstractMesh[], _handMesh: Nullable<AbstractMesh>, 
        /** An optional rig mapping for the hand mesh. If not provided (but a hand mesh is provided),
         * it will be assumed that the hand mesh's bones are named directly after the WebXR bone names. */
        rigMapping: Nullable<XRHandMeshRigMapping>, _leftHandedMeshes?: boolean, _jointsInvisible?: boolean, _jointScaleFactor?: number);
        /**
         * Sets the current hand mesh to render for the WebXRHand.
         * @param handMesh The rigged hand mesh that will be tracked to the user's hand.
         * @param rigMapping The mapping from XRHandJoint to bone names to use with the mesh.
         */
        setHandMesh(handMesh: AbstractMesh, rigMapping: Nullable<XRHandMeshRigMapping>): void;
        /**
         * Update this hand from the latest xr frame.
         * @param xrFrame The latest frame received from WebXR.
         * @param referenceSpace The current viewer reference space.
         */
        updateFromXRFrame(xrFrame: XRFrame, referenceSpace: XRReferenceSpace): void;
        /**
         * Dispose this Hand object
         */
        dispose(): void;
    }
    /**
     * WebXR Hand Joint tracking feature, available for selected browsers and devices
     */
    export class WebXRHandTracking extends WebXRAbstractFeature {
        /** Options to use when constructing this feature. */
        readonly options: IWebXRHandTrackingOptions;
        /**
         * The module's name
         */
        static readonly Name = "xr-hand-tracking";
        /**
         * The (Babylon) version of this module.
         * This is an integer representing the implementation version.
         * This number does not correspond to the WebXR specs version
         */
        static readonly Version = 1;
        /** The base URL for the default hand model. */
        static DEFAULT_HAND_MODEL_BASE_URL: string;
        /** The filename to use for the default right hand model. */
        static DEFAULT_HAND_MODEL_RIGHT_FILENAME: string;
        /** The filename to use for the default left hand model. */
        static DEFAULT_HAND_MODEL_LEFT_FILENAME: string;
        /** The URL pointing to the default hand model NodeMaterial shader. */
        static DEFAULT_HAND_MODEL_SHADER_URL: string;
        private static readonly _ICOSPHERE_PARAMS;
        private static _RightHandGLB;
        private static _LeftHandGLB;
        private static _GenerateTrackedJointMeshes;
        private static _GenerateDefaultHandMeshesAsync;
        /**
         * Generates a mapping from XRHandJoint to bone name for the default hand mesh.
         * @param handedness The handedness being mapped for.
         */
        private static _GenerateDefaultHandMeshRigMapping;
        private _attachedHands;
        private _trackingHands;
        private _handResources;
        /**
         * This observable will notify registered observers when a new hand object was added and initialized
         */
        onHandAddedObservable: Observable<WebXRHand>;
        /**
         * This observable will notify its observers right before the hand object is disposed
         */
        onHandRemovedObservable: Observable<WebXRHand>;
        /**
         * Check if the needed objects are defined.
         * This does not mean that the feature is enabled, but that the objects needed are well defined.
         */
        isCompatible(): boolean;
        /**
         * Get the hand object according to the controller id
         * @param controllerId the controller id to which we want to get the hand
         * @returns null if not found or the WebXRHand object if found
         */
        getHandByControllerId(controllerId: string): Nullable<WebXRHand>;
        /**
         * Get a hand object according to the requested handedness
         * @param handedness the handedness to request
         * @returns null if not found or the WebXRHand object if found
         */
        getHandByHandedness(handedness: XRHandedness): Nullable<WebXRHand>;
        /**
         * Creates a new instance of the XR hand tracking feature.
         * @param _xrSessionManager An instance of WebXRSessionManager.
         * @param options Options to use when constructing this feature.
         */
        constructor(_xrSessionManager: WebXRSessionManager, 
        /** Options to use when constructing this feature. */
        options: IWebXRHandTrackingOptions);
        /**
         * Attach this feature.
         * Will usually be called by the features manager.
         *
         * @returns true if successful.
         */
        attach(): boolean;
        protected _onXRFrame(_xrFrame: XRFrame): void;
        private _attachHand;
        private _detachHandById;
        private _detachHand;
        /**
         * Detach this feature.
         * Will usually be called by the features manager.
         *
         * @returns true if successful.
         */
        detach(): boolean;
        /**
         * Dispose this feature and all of the resources attached.
         */
        dispose(): void;
    }


    /**
     * Options used for hit testing (version 2)
     */
    export interface IWebXRHitTestOptions extends IWebXRLegacyHitTestOptions {
        /**
         * Do not create a permanent hit test. Will usually be used when only
         * transient inputs are needed.
         */
        disablePermanentHitTest?: boolean;
        /**
         * Enable transient (for example touch-based) hit test inspections
         */
        enableTransientHitTest?: boolean;
        /**
         * Override the default transient hit test profile (generic-touchscreen).
         */
        transientHitTestProfile?: string;
        /**
         * Offset ray for the permanent hit test
         */
        offsetRay?: Vector3;
        /**
         * Offset ray for the transient hit test
         */
        transientOffsetRay?: Vector3;
        /**
         * Instead of using viewer space for hit tests, use the reference space defined in the session manager
         */
        useReferenceSpace?: boolean;
        /**
         * Override the default entity type(s) of the hit-test result
         */
        entityTypes?: XRHitTestTrackableType[];
    }
    /**
     * Interface defining the babylon result of hit-test
     */
    export interface IWebXRHitResult extends IWebXRLegacyHitResult {
        /**
         * The input source that generated this hit test (if transient)
         */
        inputSource?: XRInputSource;
        /**
         * Is this a transient hit test
         */
        isTransient?: boolean;
        /**
         * Position of the hit test result
         */
        position: Vector3;
        /**
         * Rotation of the hit test result
         */
        rotationQuaternion: Quaternion;
        /**
         * The native hit test result
         */
        xrHitResult: XRHitTestResult;
    }
    /**
     * The currently-working hit-test module.
     * Hit test (or Ray-casting) is used to interact with the real world.
     * For further information read here - https://github.com/immersive-web/hit-test
     *
     * Tested on chrome (mobile) 80.
     */
    export class WebXRHitTest extends WebXRAbstractFeature implements IWebXRHitTestFeature<IWebXRHitResult> {
        /**
         * options to use when constructing this feature
         */
        readonly options: IWebXRHitTestOptions;
        private _tmpMat;
        private _tmpPos;
        private _tmpQuat;
        private _transientXrHitTestSource;
        private _xrHitTestSource;
        private _initHitTestSource;
        /**
         * The module's name
         */
        static readonly Name = "xr-hit-test";
        /**
         * The (Babylon) version of this module.
         * This is an integer representing the implementation version.
         * This number does not correspond to the WebXR specs version
         */
        static readonly Version = 2;
        /**
         * When set to true, each hit test will have its own position/rotation objects
         * When set to false, position and rotation objects will be reused for each hit test. It is expected that
         * the developers will clone them or copy them as they see fit.
         */
        autoCloneTransformation: boolean;
        /**
         * Triggered when new babylon (transformed) hit test results are available
         * Note - this will be called when results come back from the device. It can be an empty array!!
         */
        onHitTestResultObservable: Observable<IWebXRHitResult[]>;
        /**
         * Use this to temporarily pause hit test checks.
         */
        paused: boolean;
        /**
         * Creates a new instance of the hit test feature
         * @param _xrSessionManager an instance of WebXRSessionManager
         * @param options options to use when constructing this feature
         */
        constructor(_xrSessionManager: WebXRSessionManager, 
        /**
         * options to use when constructing this feature
         */
        options?: IWebXRHitTestOptions);
        /**
         * attach this feature
         * Will usually be called by the features manager
         *
         * @returns true if successful.
         */
        attach(): boolean;
        /**
         * detach this feature.
         * Will usually be called by the features manager
         *
         * @returns true if successful.
         */
        detach(): boolean;
        /**
         * Dispose this feature and all of the resources attached
         */
        dispose(): void;
        protected _onXRFrame(frame: XRFrame): void;
        private _processWebXRHitTestResult;
    }


    /**
     * An interface for all Hit test features
     */
    export interface IWebXRHitTestFeature<T extends IWebXRLegacyHitResult> extends IWebXRFeature {
        /**
         * Triggered when new babylon (transformed) hit test results are available
         */
        onHitTestResultObservable: Observable<T[]>;
    }
    /**
     * Options used for hit testing
     */
    export interface IWebXRLegacyHitTestOptions {
        /**
         * Only test when user interacted with the scene. Default - hit test every frame
         */
        testOnPointerDownOnly?: boolean;
        /**
         * The node to use to transform the local results to world coordinates
         */
        worldParentNode?: TransformNode;
    }
    /**
     * Interface defining the babylon result of raycasting/hit-test
     */
    export interface IWebXRLegacyHitResult {
        /**
         * Transformation matrix that can be applied to a node that will put it in the hit point location
         */
        transformationMatrix: Matrix;
        /**
         * The native hit test result
         */
        xrHitResult: XRHitResult | XRHitTestResult;
    }
    /**
     * The currently-working hit-test module.
     * Hit test (or Ray-casting) is used to interact with the real world.
     * For further information read here - https://github.com/immersive-web/hit-test
     */
    export class WebXRHitTestLegacy extends WebXRAbstractFeature implements IWebXRHitTestFeature<IWebXRLegacyHitResult> {
        /**
         * options to use when constructing this feature
         */
        readonly options: IWebXRLegacyHitTestOptions;
        private _direction;
        private _mat;
        private _onSelectEnabled;
        private _origin;
        /**
         * The module's name
         */
        static readonly Name = "xr-hit-test";
        /**
         * The (Babylon) version of this module.
         * This is an integer representing the implementation version.
         * This number does not correspond to the WebXR specs version
         */
        static readonly Version = 1;
        /**
         * Populated with the last native XR Hit Results
         */
        lastNativeXRHitResults: XRHitResult[];
        /**
         * Triggered when new babylon (transformed) hit test results are available
         */
        onHitTestResultObservable: Observable<IWebXRLegacyHitResult[]>;
        /**
         * Creates a new instance of the (legacy version) hit test feature
         * @param _xrSessionManager an instance of WebXRSessionManager
         * @param options options to use when constructing this feature
         */
        constructor(_xrSessionManager: WebXRSessionManager, 
        /**
         * options to use when constructing this feature
         */
        options?: IWebXRLegacyHitTestOptions);
        /**
         * execute a hit test with an XR Ray
         *
         * @param xrSession a native xrSession that will execute this hit test
         * @param xrRay the ray (position and direction) to use for ray-casting
         * @param referenceSpace native XR reference space to use for the hit-test
         * @param filter filter function that will filter the results
         * @returns a promise that resolves with an array of native XR hit result in xr coordinates system
         */
        static XRHitTestWithRay(xrSession: XRSession, xrRay: XRRay, referenceSpace: XRReferenceSpace, filter?: (result: XRHitResult) => boolean): Promise<XRHitResult[]>;
        /**
         * Execute a hit test on the current running session using a select event returned from a transient input (such as touch)
         * @param event the (select) event to use to select with
         * @param referenceSpace the reference space to use for this hit test
         * @returns a promise that resolves with an array of native XR hit result in xr coordinates system
         */
        static XRHitTestWithSelectEvent(event: XRInputSourceEvent, referenceSpace: XRReferenceSpace): Promise<XRHitResult[]>;
        /**
         * attach this feature
         * Will usually be called by the features manager
         *
         * @returns true if successful.
         */
        attach(): boolean;
        /**
         * detach this feature.
         * Will usually be called by the features manager
         *
         * @returns true if successful.
         */
        detach(): boolean;
        /**
         * Dispose this feature and all of the resources attached
         */
        dispose(): void;
        protected _onXRFrame(frame: XRFrame): void;
        private _onHitTestResults;
        private _onSelect;
    }


    /**
     * Options interface for the background remover plugin
     */
    export interface IWebXRImageTrackingOptions {
        /**
         * A required array with images to track
         */
        images: {
            /**
             * The source of the image. can be a URL or an image bitmap
             */
            src: string | ImageBitmap;
            /**
             * The estimated width in the real world (in meters)
             */
            estimatedRealWorldWidth: number;
        }[];
    }
    /**
     * An object representing an image tracked by the system
     */
    export interface IWebXRTrackedImage {
        /**
         * The ID of this image (which is the same as the position in the array that was used to initialize the feature)
         */
        id: number;
        /**
         * Is the transformation provided emulated. If it is, the system "guesses" its real position. Otherwise it can be considered as exact position.
         */
        emulated?: boolean;
        /**
         * Just in case it is needed - the image bitmap that is being tracked
         */
        originalBitmap: ImageBitmap;
        /**
         * The native XR result image tracking result, untouched
         */
        xrTrackingResult?: XRImageTrackingResult;
        /**
         * Width in real world (meters)
         */
        realWorldWidth?: number;
        /**
         * A transformation matrix of this current image in the current reference space.
         */
        transformationMatrix: Matrix;
        /**
         * The width/height ratio of this image. can be used to calculate the size of the detected object/image
         */
        ratio?: number;
    }
    /**
     * Image tracking for immersive AR sessions.
     * Providing a list of images and their estimated widths will enable tracking those images in the real world.
     */
    export class WebXRImageTracking extends WebXRAbstractFeature {
        /**
         * read-only options to be used in this module
         */
        readonly options: IWebXRImageTrackingOptions;
        /**
         * The module's name
         */
        static readonly Name = "xr-image-tracking";
        /**
         * The (Babylon) version of this module.
         * This is an integer representing the implementation version.
         * This number does not correspond to the WebXR specs version
         */
        static readonly Version = 1;
        /**
         * This will be triggered if the underlying system deems an image untrackable.
         * The index is the index of the image from the array used to initialize the feature.
         */
        onUntrackableImageFoundObservable: Observable<number>;
        /**
         * An image was deemed trackable, and the system will start tracking it.
         */
        onTrackableImageFoundObservable: Observable<IWebXRTrackedImage>;
        /**
         * The image was found and its state was updated.
         */
        onTrackedImageUpdatedObservable: Observable<IWebXRTrackedImage>;
        private _trackableScoreStatus;
        private _trackedImages;
        private _originalTrackingRequest;
        /**
         * constructs the image tracking feature
         * @param _xrSessionManager the session manager for this module
         * @param options read-only options to be used in this module
         */
        constructor(_xrSessionManager: WebXRSessionManager, 
        /**
         * read-only options to be used in this module
         */
        options: IWebXRImageTrackingOptions);
        /**
         * attach this feature
         * Will usually be called by the features manager
         *
         * @returns true if successful.
         */
        attach(): boolean;
        /**
         * detach this feature.
         * Will usually be called by the features manager
         *
         * @returns true if successful.
         */
        detach(): boolean;
        /**
         * Get a tracked image by its ID.
         *
         * @param id the id of the image to load (position in the init array)
         * @returns a trackable image, if exists in this location
         */
        getTrackedImageById(id: number): Nullable<IWebXRTrackedImage>;
        /**
         * Dispose this feature and all of the resources attached
         */
        dispose(): void;
        /**
         * Extends the session init object if needed
         * @returns augmentation object fo the xr session init object.
         */
        getXRSessionInitExtension(): Promise<Partial<XRSessionInit>>;
        protected _onXRFrame(_xrFrame: XRFrame): void;
        private _checkScoresAsync;
    }


    /**
     * Wraps xr composition layers.
     * @internal
     */
    export class WebXRCompositionLayerWrapper extends WebXRLayerWrapper {
        getWidth: () => number;
        getHeight: () => number;
        readonly layer: XRCompositionLayer;
        readonly layerType: WebXRLayerType;
        readonly isMultiview: boolean;
        createRTTProvider: (xrSessionManager: WebXRSessionManager) => WebXRLayerRenderTargetTextureProvider;
        constructor(getWidth: () => number, getHeight: () => number, layer: XRCompositionLayer, layerType: WebXRLayerType, isMultiview: boolean, createRTTProvider: (xrSessionManager: WebXRSessionManager) => WebXRLayerRenderTargetTextureProvider);
    }
    /**
     * Wraps xr projection layers.
     * @internal
     */
    export class WebXRProjectionLayerWrapper extends WebXRCompositionLayerWrapper {
        readonly layer: XRProjectionLayer;
        constructor(layer: XRProjectionLayer, isMultiview: boolean, xrGLBinding: XRWebGLBinding);
    }
    /**
     * Configuration options of the layers feature
     */
    export interface IWebXRLayersOptions {
        /**
         * Whether to try initializing the base projection layer as a multiview render target, if multiview is supported.
         * Defaults to false.
         */
        preferMultiviewOnInit?: boolean;
    }
    /**
     * Exposes the WebXR Layers API.
     */
    export class WebXRLayers extends WebXRAbstractFeature {
        private readonly _options;
        /**
         * The module's name
         */
        static readonly Name = "xr-layers";
        /**
         * The (Babylon) version of this module.
         * This is an integer representing the implementation version.
         * This number does not correspond to the WebXR specs version
         */
        static readonly Version = 1;
        /**
         * Already-created layers
         */
        private _existingLayers;
        private _glContext;
        private _xrWebGLBinding;
        constructor(_xrSessionManager: WebXRSessionManager, _options?: IWebXRLayersOptions);
        /**
         * Attach this feature.
         * Will usually be called by the features manager.
         *
         * @returns true if successful.
         */
        attach(): boolean;
        detach(): boolean;
        /**
         * Creates a new XRWebGLLayer.
         * @param params an object providing configuration options for the new XRWebGLLayer
         * @returns the XRWebGLLayer
         */
        createXRWebGLLayer(params?: XRWebGLLayerInit): WebXRWebGLLayerWrapper;
        /**
         * Creates a new XRProjectionLayer.
         * @param params an object providing configuration options for the new XRProjectionLayer.
         * @param multiview whether the projection layer should render with multiview.
         * @returns the projection layer
         */
        createProjectionLayer(params?: XRProjectionLayerInit, multiview?: boolean): WebXRProjectionLayerWrapper;
        /**
         * Add a new layer to the already-existing list of layers
         * @param wrappedLayer the new layer to add to the existing ones
         */
        addXRSessionLayer(wrappedLayer: WebXRLayerWrapper): void;
        /**
         * Sets the layers to be used by the XR session.
         * Note that you must call this function with any layers you wish to render to
         * since it adds them to the XR session's render state
         * (replacing any layers that were added in a previous call to setXRSessionLayers or updateRenderState).
         * This method also sets up the session manager's render target texture provider
         * as the first layer in the array, which feeds the WebXR camera(s) attached to the session.
         * @param wrappedLayers An array of WebXRLayerWrapper, usually returned from the WebXRLayers createLayer functions.
         */
        setXRSessionLayers(wrappedLayers: Array<WebXRLayerWrapper>): void;
        isCompatible(): boolean;
        /**
         * Dispose this feature and all of the resources attached.
         */
        dispose(): void;
        protected _onXRFrame(_xrFrame: XRFrame): void;
    }


    /**
     * Options for Light Estimation feature
     */
    export interface IWebXRLightEstimationOptions {
        /**
         * Disable the cube map reflection feature. In this case only light direction and color will be updated
         */
        disableCubeMapReflection?: boolean;
        /**
         * Should the scene's env texture be set to the cube map reflection texture
         * Note that this doesn't work is disableCubeMapReflection if set to false
         */
        setSceneEnvironmentTexture?: boolean;
        /**
         * How often should the cubemap update in ms.
         * If not set the cubemap will be updated every time the underlying system updates the environment texture.
         */
        cubeMapPollInterval?: number;
        /**
         * How often should the light estimation properties update in ms.
         * If not set the light estimation properties will be updated on every frame (depending on the underlying system)
         */
        lightEstimationPollInterval?: number;
        /**
         * Should a directional light source be created.
         * If created, this light source will be updated whenever the light estimation values change
         */
        createDirectionalLightSource?: boolean;
        /**
         * Define the format to be used for the light estimation texture.
         */
        reflectionFormat?: XRReflectionFormat;
        /**
         * Should the light estimation's needed vectors be constructed on each frame.
         * Use this when you use those vectors and don't want their values to change outside of the light estimation feature
         */
        disableVectorReuse?: boolean;
        /**
         * disable applying the spherical polynomial to the cube map texture
         */
        disableSphericalPolynomial?: boolean;
    }
    /**
     * An interface describing the result of a light estimation
     */
    export interface IWebXRLightEstimation {
        /**
         * The intensity of the light source
         */
        lightIntensity: number;
        /**
         * Color of light source
         */
        lightColor: Color3;
        /**
         * The direction from the light source
         */
        lightDirection: Vector3;
        /**
         * Spherical harmonics coefficients of the light source
         */
        sphericalHarmonics: SphericalHarmonics;
    }
    /**
     * Light Estimation Feature
     *
     * @since 5.0.0
     */
    export class WebXRLightEstimation extends WebXRAbstractFeature {
        /**
         * options to use when constructing this feature
         */
        readonly options: IWebXRLightEstimationOptions;
        private _canvasContext;
        private _reflectionCubeMap;
        private _xrLightEstimate;
        private _xrLightProbe;
        private _xrWebGLBinding;
        private _lightDirection;
        private _lightColor;
        private _intensity;
        private _sphericalHarmonics;
        private _cubeMapPollTime;
        private _lightEstimationPollTime;
        /**
         * The module's name
         */
        static readonly Name = "xr-light-estimation";
        /**
         * The (Babylon) version of this module.
         * This is an integer representing the implementation version.
         * This number does not correspond to the WebXR specs version
         */
        static readonly Version = 1;
        /**
         * ARCore's reflection cube map size is 16x16.
         * Once other systems support this feature we will need to change this to be dynamic.
         * see https://github.com/immersive-web/lighting-estimation/blob/main/lighting-estimation-explainer.md#cube-map-open-questions
         */
        private _reflectionCubeMapTextureSize;
        /**
         * If createDirectionalLightSource is set to true this light source will be created automatically.
         * Otherwise this can be set with an external directional light source.
         * This light will be updated whenever the light estimation values change.
         */
        directionalLight: Nullable<DirectionalLight>;
        /**
         * This observable will notify when the reflection cube map is updated.
         */
        onReflectionCubeMapUpdatedObservable: Observable<BaseTexture>;
        /**
         * Creates a new instance of the light estimation feature
         * @param _xrSessionManager an instance of WebXRSessionManager
         * @param options options to use when constructing this feature
         */
        constructor(_xrSessionManager: WebXRSessionManager, 
        /**
         * options to use when constructing this feature
         */
        options: IWebXRLightEstimationOptions);
        /**
         * While the estimated cube map is expected to update over time to better reflect the user's environment as they move around those changes are unlikely to happen with every XRFrame.
         * Since creating and processing the cube map is potentially expensive, especially if mip maps are needed, you can listen to the onReflectionCubeMapUpdatedObservable to determine
         * when it has been updated.
         */
        get reflectionCubeMapTexture(): Nullable<BaseTexture>;
        /**
         * The most recent light estimate.  Available starting on the first frame where the device provides a light probe.
         */
        get xrLightingEstimate(): Nullable<IWebXRLightEstimation>;
        private _getCanvasContext;
        private _getXRGLBinding;
        /**
         * Event Listener for "reflectionchange" events.
         */
        private _updateReflectionCubeMap;
        /**
         * attach this feature
         * Will usually be called by the features manager
         *
         * @returns true if successful.
         */
        attach(): boolean;
        /**
         * detach this feature.
         * Will usually be called by the features manager
         *
         * @returns true if successful.
         */
        detach(): boolean;
        /**
         * Dispose this feature and all of the resources attached
         */
        dispose(): void;
        protected _onXRFrame(_xrFrame: XRFrame): void;
    }


    /**
     * Options used in the mesh detector module
     */
    export interface IWebXRMeshDetectorOptions {
        /**
         * The node to use to transform the local results to world coordinates
         */
        worldParentNode?: TransformNode;
        /**
         * If set to true a reference of the created meshes will be kept until the next session starts
         * If not defined, meshes will be removed from the array when the feature is detached or the session ended.
         */
        doNotRemoveMeshesOnSessionEnded?: boolean;
        /**
         * Preferred detector configuration, not all preferred options will be supported by all platforms.
         */
        preferredDetectorOptions?: XRGeometryDetectorOptions;
        /**
         * If set to true, WebXRMeshDetector will convert coordinate systems for meshes.
         * If not defined, mesh conversions from right handed to left handed coordinate systems won't be conducted.
         * Right handed mesh data will be available through IWebXRVertexData.xrMesh.
         */
        convertCoordinateSystems?: boolean;
    }
    /**
     * A babylon interface for a XR mesh's vertex data.
     *
     * Currently not supported by WebXR, available only with BabylonNative
     */
    export interface IWebXRVertexData {
        /**
         * A babylon-assigned ID for this mesh
         */
        id: number;
        /**
         * Data required for constructing a mesh in Babylon.js.
         */
        xrMesh: XRMesh;
        /**
         * The node to use to transform the local results to world coordinates.
         * WorldParentNode will only exist if it was declared in the IWebXRMeshDetectorOptions.
         */
        worldParentNode?: TransformNode;
        /**
         * An array of vertex positions in babylon space. right/left hand system is taken into account.
         * Positions will only be calculated if convertCoordinateSystems is set to true in the IWebXRMeshDetectorOptions.
         */
        positions?: Float32Array;
        /**
         * An array of indices in babylon space. Indices have a counterclockwise winding order.
         * Indices will only be populated if convertCoordinateSystems is set to true in the IWebXRMeshDetectorOptions.
         */
        indices?: Uint32Array;
        /**
         * An array of vertex normals in babylon space. right/left hand system is taken into account.
         * Normals will not be calculated if convertCoordinateSystems is undefined in the IWebXRMeshDetectorOptions.
         * Different platforms may or may not support mesh normals when convertCoordinateSystems is set to true.
         */
        normals?: Float32Array;
        /**
         * A transformation matrix to apply on the mesh that will be built using the meshDefinition.
         * Local vs. World are decided if worldParentNode was provided or not in the options when constructing the module.
         * TransformationMatrix will only be calculated if convertCoordinateSystems is set to true in the IWebXRMeshDetectorOptions.
         */
        transformationMatrix?: Matrix;
    }
    /**
     * The mesh detector is used to detect meshes in the real world when in AR
     */
    export class WebXRMeshDetector extends WebXRAbstractFeature {
        private _options;
        private _detectedMeshes;
        /**
         * The module's name
         */
        static readonly Name = "xr-mesh-detection";
        /**
         * The (Babylon) version of this module.
         * This is an integer representing the implementation version.
         * This number does not correspond to the WebXR specs version
         */
        static readonly Version = 1;
        /**
         * Observers registered here will be executed when a new mesh was added to the session
         */
        onMeshAddedObservable: Observable<IWebXRVertexData>;
        /**
         * Observers registered here will be executed when a mesh is no longer detected in the session
         */
        onMeshRemovedObservable: Observable<IWebXRVertexData>;
        /**
         * Observers registered here will be executed when an existing mesh updates
         */
        onMeshUpdatedObservable: Observable<IWebXRVertexData>;
        constructor(_xrSessionManager: WebXRSessionManager, _options?: IWebXRMeshDetectorOptions);
        detach(): boolean;
        dispose(): void;
        protected _onXRFrame(frame: XRFrame): void;
        private _init;
        private _updateVertexDataWithXRMesh;
    }


    /**
     * Where should the near interaction mesh be attached to when using a motion controller for near interaction
     */
    export enum WebXRNearControllerMode {
        /**
         * Motion controllers will not support near interaction
         */
        DISABLED = 0,
        /**
         * The interaction point for motion controllers will be inside of them
         */
        CENTERED_ON_CONTROLLER = 1,
        /**
         * The interaction point for motion controllers will be in front of the controller
         */
        CENTERED_IN_FRONT = 2
    }
    /**
     * Options interface for the near interaction module
     */
    export interface IWebXRNearInteractionOptions {
        /**
         * If provided, this scene will be used to render meshes.
         */
        customUtilityLayerScene?: Scene;
        /**
         * Should meshes created here be added to a utility layer or the main scene
         */
        useUtilityLayer?: boolean;
        /**
         * The xr input to use with this near interaction
         */
        xrInput: WebXRInput;
        /**
         * Enable near interaction on all controllers instead of switching between them
         */
        enableNearInteractionOnAllControllers?: boolean;
        /**
         * The preferred hand to give the near interaction to. This will be prioritized when the controller initialize.
         * If switch is enabled, it will still allow the user to switch between the different controllers
         */
        preferredHandedness?: XRHandedness;
        /**
         * Disable switching the near interaction from one controller to the other.
         * If the preferred hand is set it will be fixed on this hand, and if not it will be fixed on the first controller added to the scene
         */
        disableSwitchOnClick?: boolean;
        /**
         * Far interaction feature to toggle when near interaction takes precedence
         */
        farInteractionFeature?: WebXRControllerPointerSelection;
        /**
         * Near interaction mode for motion controllers
         */
        nearInteractionControllerMode?: WebXRNearControllerMode;
        /**
         * Optional material for the motion controller orb, if enabled
         */
        motionControllerOrbMaterial?: Material;
    }
    /**
     * A module that will enable near interaction near interaction for hands and motion controllers of XR Input Sources
     */
    export class WebXRNearInteraction extends WebXRAbstractFeature {
        private readonly _options;
        private static _IdCounter;
        private _tmpRay;
        private _attachController;
        private _controllers;
        private _scene;
        private _attachedController;
        private _farInteractionFeature;
        /**
         * The module's name
         */
        static readonly Name = "xr-near-interaction";
        /**
         * The (Babylon) version of this module.
         * This is an integer representing the implementation version.
         * This number does not correspond to the WebXR specs version
         */
        static readonly Version = 1;
        /**
         * default color of the selection ring
         */
        selectionMeshDefaultColor: Color3;
        /**
         * This color will be applied to the selection ring when selection is triggered
         */
        selectionMeshPickedColor: Color3;
        /**
         * constructs a new background remover module
         * @param _xrSessionManager the session manager for this module
         * @param _options read-only options to be used in this module
         */
        constructor(_xrSessionManager: WebXRSessionManager, _options: IWebXRNearInteractionOptions);
        /**
         * Attach this feature
         * Will usually be called by the features manager
         *
         * @returns true if successful.
         */
        attach(): boolean;
        /**
         * Detach this feature.
         * Will usually be called by the features manager
         *
         * @returns true if successful.
         */
        detach(): boolean;
        /**
         * Will get the mesh under a specific pointer.
         * `scene.meshUnderPointer` will only return one mesh - either left or right.
         * @param controllerId the controllerId to check
         * @returns The mesh under pointer or null if no mesh is under the pointer
         */
        getMeshUnderPointer(controllerId: string): Nullable<AbstractMesh>;
        /**
         * Get the xr controller that correlates to the pointer id in the pointer event
         *
         * @param id the pointer id to search for
         * @returns the controller that correlates to this id or null if not found
         */
        getXRControllerByPointerId(id: number): Nullable<WebXRInputSource>;
        /**
         * This function sets webXRControllerPointerSelection feature that will be disabled when
         * the hover range is reached for a mesh and will be reattached when not in hover range.
         * This is used to remove the selection rays when moving.
         * @param farInteractionFeature the feature to disable when finger is in hover range for a mesh
         */
        setFarInteractionFeature(farInteractionFeature: Nullable<WebXRControllerPointerSelection>): void;
        /**
         * Filter used for near interaction pick and hover
         * @param mesh
         */
        private _nearPickPredicate;
        /**
         * Filter used for near interaction grab
         * @param mesh
         */
        private _nearGrabPredicate;
        /**
         * Filter used for any near interaction
         * @param mesh
         */
        private _nearInteractionPredicate;
        private _controllerAvailablePredicate;
        private _handleTransitionAnimation;
        private readonly _hoverRadius;
        private readonly _pickRadius;
        private readonly _controllerPickRadius;
        private readonly _nearGrabLengthScale;
        private _processTouchPoint;
        protected _onXRFrame(_xrFrame: XRFrame): void;
        private get _utilityLayerScene();
        private _generateVisualCue;
        private _isControllerReadyForNearInteraction;
        private _attachNearInteractionMode;
        private _detachController;
        private _generateNewTouchPointMesh;
        private _pickWithSphere;
        /**
         * Picks a mesh with a sphere
         * @param mesh the mesh to pick
         * @param sphere picking sphere in world coordinates
         * @param skipBoundingInfo a boolean indicating if we should skip the bounding info check
         * @returns the picking info
         */
        static PickMeshWithSphere(mesh: AbstractMesh, sphere: BoundingSphere, skipBoundingInfo?: boolean): PickingInfo;
    }


    /**
     * Options used in the plane detector module
     */
    export interface IWebXRPlaneDetectorOptions {
        /**
         * The node to use to transform the local results to world coordinates
         */
        worldParentNode?: TransformNode;
        /**
         * If set to true a reference of the created planes will be kept until the next session starts
         * If not defined, planes will be removed from the array when the feature is detached or the session ended.
         */
        doNotRemovePlanesOnSessionEnded?: boolean;
        /**
         * Preferred detector configuration, not all preferred options will be supported by all platforms.
         */
        preferredDetectorOptions?: XRGeometryDetectorOptions;
    }
    /**
     * A babylon interface for a WebXR plane.
     * A Plane is actually a polygon, built from N points in space
     *
     * Supported in chrome 79, not supported in canary 81 ATM
     */
    export interface IWebXRPlane {
        /**
         * a babylon-assigned ID for this polygon
         */
        id: number;
        /**
         * an array of vector3 points in babylon space. right/left hand system is taken into account.
         */
        polygonDefinition: Array<Vector3>;
        /**
         * A transformation matrix to apply on the mesh that will be built using the polygonDefinition
         * Local vs. World are decided if worldParentNode was provided or not in the options when constructing the module
         */
        transformationMatrix: Matrix;
        /**
         * the native xr-plane object
         */
        xrPlane: XRPlane;
    }
    /**
     * The plane detector is used to detect planes in the real world when in AR
     * For more information see https://github.com/immersive-web/real-world-geometry/
     */
    export class WebXRPlaneDetector extends WebXRAbstractFeature {
        private _options;
        private _detectedPlanes;
        private _enabled;
        private _lastFrameDetected;
        /**
         * The module's name
         */
        static readonly Name = "xr-plane-detection";
        /**
         * The (Babylon) version of this module.
         * This is an integer representing the implementation version.
         * This number does not correspond to the WebXR specs version
         */
        static readonly Version = 1;
        /**
         * Observers registered here will be executed when a new plane was added to the session
         */
        onPlaneAddedObservable: Observable<IWebXRPlane>;
        /**
         * Observers registered here will be executed when a plane is no longer detected in the session
         */
        onPlaneRemovedObservable: Observable<IWebXRPlane>;
        /**
         * Observers registered here will be executed when an existing plane updates (for example - expanded)
         * This can execute N times every frame
         */
        onPlaneUpdatedObservable: Observable<IWebXRPlane>;
        /**
         * construct a new Plane Detector
         * @param _xrSessionManager an instance of xr Session manager
         * @param _options configuration to use when constructing this feature
         */
        constructor(_xrSessionManager: WebXRSessionManager, _options?: IWebXRPlaneDetectorOptions);
        /**
         * detach this feature.
         * Will usually be called by the features manager
         *
         * @returns true if successful.
         */
        detach(): boolean;
        /**
         * Dispose this feature and all of the resources attached
         */
        dispose(): void;
        /**
         * Check if the needed objects are defined.
         * This does not mean that the feature is enabled, but that the objects needed are well defined.
         */
        isCompatible(): boolean;
        protected _onXRFrame(frame: XRFrame): void;
        private _init;
        private _updatePlaneWithXRPlane;
        /**
         * avoiding using Array.find for global support.
         * @param xrPlane the plane to find in the array
         */
        private _findIndexInPlaneArray;
    }


    /**
     * Options for the walking locomotion feature.
     */
    export interface IWebXRWalkingLocomotionOptions {
        /**
         * The target to be moved by walking locomotion. This should be the transform node
         * which is the root of the XR space (i.e., the WebXRCamera's parent node). However,
         * for simple cases and legacy purposes, articulating the WebXRCamera itself is also
         * supported as a deprecated feature.
         */
        locomotionTarget: WebXRCamera | TransformNode;
    }
    /**
     * A module that will enable VR locomotion by detecting when the user walks in place.
     */
    export class WebXRWalkingLocomotion extends WebXRAbstractFeature {
        /**
         * The module's name.
         */
        static get Name(): string;
        /**
         * The (Babylon) version of this module.
         * This is an integer representing the implementation version.
         * This number has no external basis.
         */
        static get Version(): number;
        private _sessionManager;
        private _up;
        private _forward;
        private _position;
        private _movement;
        private _walker;
        private _locomotionTarget;
        private _isLocomotionTargetWebXRCamera;
        /**
         * The target to be articulated by walking locomotion.
         * When the walking locomotion feature detects walking in place, this element's
         * X and Z coordinates will be modified to reflect locomotion. This target should
         * be either the XR space's origin (i.e., the parent node of the WebXRCamera) or
         * the WebXRCamera itself. Note that the WebXRCamera path will modify the position
         * of the WebXRCamera directly and is thus discouraged.
         */
        get locomotionTarget(): WebXRCamera | TransformNode;
        /**
         * The target to be articulated by walking locomotion.
         * When the walking locomotion feature detects walking in place, this element's
         * X and Z coordinates will be modified to reflect locomotion. This target should
         * be either the XR space's origin (i.e., the parent node of the WebXRCamera) or
         * the WebXRCamera itself. Note that the WebXRCamera path will modify the position
         * of the WebXRCamera directly and is thus discouraged.
         */
        set locomotionTarget(locomotionTarget: WebXRCamera | TransformNode);
        /**
         * Construct a new Walking Locomotion feature.
         * @param sessionManager manager for the current XR session
         * @param options creation options, prominently including the vector target for locomotion
         */
        constructor(sessionManager: WebXRSessionManager, options: IWebXRWalkingLocomotionOptions);
        /**
         * Checks whether this feature is compatible with the current WebXR session.
         * Walking locomotion is only compatible with "immersive-vr" sessions.
         * @returns true if compatible, false otherwise
         */
        isCompatible(): boolean;
        /**
         * Attaches the feature.
         * Typically called automatically by the features manager.
         * @returns true if attach succeeded, false otherwise
         */
        attach(): boolean;
        /**
         * Detaches the feature.
         * Typically called automatically by the features manager.
         * @returns true if detach succeeded, false otherwise
         */
        detach(): boolean;
        protected _onXRFrame(frame: XRFrame): void;
    }






    /**
     * Handedness type in xrInput profiles. These can be used to define layouts in the Layout Map.
     */
    export type MotionControllerHandedness = "none" | "left" | "right";
    /**
     * The type of components available in motion controllers.
     * This is not the name of the component.
     */
    export type MotionControllerComponentType = "trigger" | "squeeze" | "touchpad" | "thumbstick" | "button";
    /**
     * The state of a controller component
     */
    export type MotionControllerComponentStateType = "default" | "touched" | "pressed";
    /**
     * The schema of motion controller layout.
     * No object will be initialized using this interface
     * This is used just to define the profile.
     */
    export interface IMotionControllerLayout {
        /**
         * Path to load the assets. Usually relative to the base path
         */
        assetPath: string;
        /**
         * Available components (unsorted)
         */
        components: {
            /**
             * A map of component Ids
             */
            [componentId: string]: {
                /**
                 * The type of input the component outputs
                 */
                type: MotionControllerComponentType;
                /**
                 * The indices of this component in the gamepad object
                 */
                gamepadIndices: {
                    /**
                     * Index of button
                     */
                    button?: number;
                    /**
                     * If available, index of x-axis
                     */
                    xAxis?: number;
                    /**
                     * If available, index of y-axis
                     */
                    yAxis?: number;
                };
                /**
                 * The mesh's root node name
                 */
                rootNodeName: string;
                /**
                 * Animation definitions for this model
                 */
                visualResponses: {
                    [stateKey: string]: {
                        /**
                         * What property will be animated
                         */
                        componentProperty: "xAxis" | "yAxis" | "button" | "state";
                        /**
                         * What states influence this visual response
                         */
                        states: MotionControllerComponentStateType[];
                        /**
                         * Type of animation - movement or visibility
                         */
                        valueNodeProperty: "transform" | "visibility";
                        /**
                         * Base node name to move. Its position will be calculated according to the min and max nodes
                         */
                        valueNodeName?: string;
                        /**
                         * Minimum movement node
                         */
                        minNodeName?: string;
                        /**
                         * Max movement node
                         */
                        maxNodeName?: string;
                    };
                };
                /**
                 * If touch enabled, what is the name of node to display user feedback
                 */
                touchPointNodeName?: string;
            };
        };
        /**
         * Is it xr standard mapping or not
         */
        gamepadMapping: "" | "xr-standard";
        /**
         * Base root node of this entire model
         */
        rootNodeName: string;
        /**
         * Defines the main button component id
         */
        selectComponentId: string;
    }
    /**
     * A definition for the layout map in the input profile
     */
    export interface IMotionControllerLayoutMap {
        /**
         * Layouts with handedness type as a key
         */
        [handedness: string]: IMotionControllerLayout;
    }
    /**
     * The XR Input profile schema
     * Profiles can be found here:
     * https://github.com/immersive-web/webxr-input-profiles/tree/master/packages/registry/profiles
     */
    export interface IMotionControllerProfile {
        /**
         * fallback profiles for this profileId
         */
        fallbackProfileIds: string[];
        /**
         * The layout map, with handedness as key
         */
        layouts: IMotionControllerLayoutMap;
        /**
         * The id of this profile
         * correlates to the profile(s) in the xrInput.profiles array
         */
        profileId: string;
    }
    /**
     * A helper-interface for the 3 meshes needed for controller button animation
     * The meshes are provided to the _lerpButtonTransform function to calculate the current position of the value mesh
     */
    export interface IMotionControllerButtonMeshMap {
        /**
         * the mesh that defines the pressed value mesh position.
         * This is used to find the max-position of this button
         */
        pressedMesh: AbstractMesh;
        /**
         * the mesh that defines the unpressed value mesh position.
         * This is used to find the min (or initial) position of this button
         */
        unpressedMesh: AbstractMesh;
        /**
         * The mesh that will be changed when value changes
         */
        valueMesh: AbstractMesh;
    }
    /**
     * A helper-interface for the 3 meshes needed for controller axis animation.
     * This will be expanded when touchpad animations are fully supported
     * The meshes are provided to the _lerpAxisTransform function to calculate the current position of the value mesh
     */
    export interface IMotionControllerMeshMap {
        /**
         * the mesh that defines the maximum value mesh position.
         */
        maxMesh?: AbstractMesh;
        /**
         * the mesh that defines the minimum value mesh position.
         */
        minMesh?: AbstractMesh;
        /**
         * The mesh that will be changed when axis value changes
         */
        valueMesh?: AbstractMesh;
    }
    /**
     * The elements needed for change-detection of the gamepad objects in motion controllers
     */
    export interface IMinimalMotionControllerObject {
        /**
         * Available axes of this controller
         */
        axes: number[];
        /**
         * An array of available buttons
         */
        buttons: Array<{
            /**
             * Value of the button/trigger
             */
            value: number;
            /**
             * If the button/trigger is currently touched
             */
            touched: boolean;
            /**
             * If the button/trigger is currently pressed
             */
            pressed: boolean;
        }>;
        /**
         * EXPERIMENTAL haptic support.
         */
        hapticActuators?: Array<{
            pulse: (value: number, duration: number) => Promise<boolean>;
        }>;
    }
    /**
     * An Abstract Motion controller
     * This class receives an xrInput and a profile layout and uses those to initialize the components
     * Each component has an observable to check for changes in value and state
     */
    export abstract class WebXRAbstractMotionController implements IDisposable {
        protected scene: Scene;
        protected layout: IMotionControllerLayout;
        /**
         * The gamepad object correlating to this controller
         */
        gamepadObject: IMinimalMotionControllerObject;
        /**
         * handedness (left/right/none) of this controller
         */
        handedness: MotionControllerHandedness;
        /**
         * @internal
         */
        _doNotLoadControllerMesh: boolean;
        private _controllerCache?;
        private _initComponent;
        private _modelReady;
        /**
         * A map of components (WebXRControllerComponent) in this motion controller
         * Components have a ComponentType and can also have both button and axis definitions
         */
        readonly components: {
            [id: string]: WebXRControllerComponent;
        };
        /**
         * Disable the model's animation. Can be set at any time.
         */
        disableAnimation: boolean;
        /**
         * Observers registered here will be triggered when the model of this controller is done loading
         */
        onModelLoadedObservable: Observable<WebXRAbstractMotionController>;
        /**
         * The profile id of this motion controller
         */
        abstract profileId: string;
        /**
         * The root mesh of the model. It is null if the model was not yet initialized
         */
        rootMesh: Nullable<AbstractMesh>;
        /**
         * constructs a new abstract motion controller
         * @param scene the scene to which the model of the controller will be added
         * @param layout The profile layout to load
         * @param gamepadObject The gamepad object correlating to this controller
         * @param handedness handedness (left/right/none) of this controller
         * @param _doNotLoadControllerMesh set this flag to ignore the mesh loading
         * @param _controllerCache a cache holding controller models already loaded in this session
         */
        constructor(scene: Scene, layout: IMotionControllerLayout, 
        /**
         * The gamepad object correlating to this controller
         */
        gamepadObject: IMinimalMotionControllerObject, 
        /**
         * handedness (left/right/none) of this controller
         */
        handedness: MotionControllerHandedness, 
        /**
         * @internal
         */
        _doNotLoadControllerMesh?: boolean, _controllerCache?: {
            filename: string;
            path: string;
            meshes: AbstractMesh[];
        }[] | undefined);
        /**
         * Dispose this controller, the model mesh and all its components
         */
        dispose(): void;
        /**
         * Returns all components of specific type
         * @param type the type to search for
         * @returns an array of components with this type
         */
        getAllComponentsOfType(type: MotionControllerComponentType): WebXRControllerComponent[];
        /**
         * get a component based an its component id as defined in layout.components
         * @param id the id of the component
         * @returns the component correlates to the id or undefined if not found
         */
        getComponent(id: string): WebXRControllerComponent;
        /**
         * Get the list of components available in this motion controller
         * @returns an array of strings correlating to available components
         */
        getComponentIds(): string[];
        /**
         * Get the first component of specific type
         * @param type type of component to find
         * @returns a controller component or null if not found
         */
        getComponentOfType(type: MotionControllerComponentType): Nullable<WebXRControllerComponent>;
        /**
         * Get the main (Select) component of this controller as defined in the layout
         * @returns the main component of this controller
         */
        getMainComponent(): WebXRControllerComponent;
        /**
         * Loads the model correlating to this controller
         * When the mesh is loaded, the onModelLoadedObservable will be triggered
         * @returns A promise fulfilled with the result of the model loading
         */
        loadModel(): Promise<boolean>;
        /**
         * Update this model using the current XRFrame
         * @param xrFrame the current xr frame to use and update the model
         */
        updateFromXRFrame(xrFrame: XRFrame): void;
        /**
         * Backwards compatibility due to a deeply-integrated typo
         */
        get handness(): MotionControllerHandedness;
        /**
         * Pulse (vibrate) this controller
         * If the controller does not support pulses, this function will fail silently and return Promise<false> directly after called
         * Consecutive calls to this function will cancel the last pulse call
         *
         * @param value the strength of the pulse in 0.0...1.0 range
         * @param duration Duration of the pulse in milliseconds
         * @param hapticActuatorIndex optional index of actuator (will usually be 0)
         * @returns a promise that will send true when the pulse has ended and false if the device doesn't support pulse or an error accrued
         */
        pulse(value: number, duration: number, hapticActuatorIndex?: number): Promise<boolean>;
        protected _getChildByName(node: AbstractMesh, name: string): AbstractMesh | undefined;
        protected _getImmediateChildByName(node: AbstractMesh, name: string): AbstractMesh | undefined;
        /**
         * Moves the axis on the controller mesh based on its current state
         * @param axisMap
         * @param axisValue the value of the axis which determines the meshes new position
         * @internal
         */
        protected _lerpTransform(axisMap: IMotionControllerMeshMap, axisValue: number, fixValueCoordinates?: boolean): void;
        /**
         * Update the model itself with the current frame data
         * @param xrFrame the frame to use for updating the model mesh
         */
        protected updateModel(xrFrame: XRFrame): void;
        /**
         * Get the filename and path for this controller's model
         * @returns a map of filename and path
         */
        protected abstract _getFilenameAndPath(): {
            filename: string;
            path: string;
        };
        /**
         * This function is called before the mesh is loaded. It checks for loading constraints.
         * For example, this function can check if the GLB loader is available
         * If this function returns false, the generic controller will be loaded instead
         * @returns Is the client ready to load the mesh
         */
        protected abstract _getModelLoadingConstraints(): boolean;
        /**
         * This function will be called after the model was successfully loaded and can be used
         * for mesh transformations before it is available for the user
         * @param meshes the loaded meshes
         */
        protected abstract _processLoadedModel(meshes: AbstractMesh[]): void;
        /**
         * Set the root mesh for this controller. Important for the WebXR controller class
         * @param meshes the loaded meshes
         */
        protected abstract _setRootMesh(meshes: AbstractMesh[]): void;
        /**
         * A function executed each frame that updates the mesh (if needed)
         * @param xrFrame the current xrFrame
         */
        protected abstract _updateModel(xrFrame: XRFrame): void;
        private _getGenericFilenameAndPath;
        private _getGenericParentMesh;
    }


    /**
     * X-Y values for axes in WebXR
     */
    export interface IWebXRMotionControllerAxesValue {
        /**
         * The value of the x axis
         */
        x: number;
        /**
         * The value of the y-axis
         */
        y: number;
    }
    /**
     * changed / previous values for the values of this component
     */
    export interface IWebXRMotionControllerComponentChangesValues<T> {
        /**
         * current (this frame) value
         */
        current: T;
        /**
         * previous (last change) value
         */
        previous: T;
    }
    /**
     * Represents changes in the component between current frame and last values recorded
     */
    export interface IWebXRMotionControllerComponentChanges {
        /**
         * will be populated with previous and current values if axes changed
         */
        axes?: IWebXRMotionControllerComponentChangesValues<IWebXRMotionControllerAxesValue>;
        /**
         * will be populated with previous and current values if pressed changed
         */
        pressed?: IWebXRMotionControllerComponentChangesValues<boolean>;
        /**
         * will be populated with previous and current values if touched changed
         */
        touched?: IWebXRMotionControllerComponentChangesValues<boolean>;
        /**
         * will be populated with previous and current values if value changed
         */
        value?: IWebXRMotionControllerComponentChangesValues<number>;
    }
    /**
     * This class represents a single component (for example button or thumbstick) of a motion controller
     */
    export class WebXRControllerComponent implements IDisposable {
        /**
         * the id of this component
         */
        id: string;
        /**
         * the type of the component
         */
        type: MotionControllerComponentType;
        private _buttonIndex;
        private _axesIndices;
        private _axes;
        private _changes;
        private _currentValue;
        private _hasChanges;
        private _pressed;
        private _touched;
        /**
         * button component type
         */
        static BUTTON_TYPE: MotionControllerComponentType;
        /**
         * squeeze component type
         */
        static SQUEEZE_TYPE: MotionControllerComponentType;
        /**
         * Thumbstick component type
         */
        static THUMBSTICK_TYPE: MotionControllerComponentType;
        /**
         * Touchpad component type
         */
        static TOUCHPAD_TYPE: MotionControllerComponentType;
        /**
         * trigger component type
         */
        static TRIGGER_TYPE: MotionControllerComponentType;
        /**
         * If axes are available for this component (like a touchpad or thumbstick) the observers will be notified when
         * the axes data changes
         */
        onAxisValueChangedObservable: Observable<{
            x: number;
            y: number;
        }>;
        /**
         * Observers registered here will be triggered when the state of a button changes
         * State change is either pressed / touched / value
         */
        onButtonStateChangedObservable: Observable<WebXRControllerComponent>;
        /**
         * Creates a new component for a motion controller.
         * It is created by the motion controller itself
         *
         * @param id the id of this component
         * @param type the type of the component
         * @param _buttonIndex index in the buttons array of the gamepad
         * @param _axesIndices indices of the values in the axes array of the gamepad
         */
        constructor(
        /**
         * the id of this component
         */
        id: string, 
        /**
         * the type of the component
         */
        type: MotionControllerComponentType, _buttonIndex?: number, _axesIndices?: number[]);
        /**
         * The current axes data. If this component has no axes it will still return an object { x: 0, y: 0 }
         */
        get axes(): IWebXRMotionControllerAxesValue;
        /**
         * Get the changes. Elements will be populated only if they changed with their previous and current value
         */
        get changes(): IWebXRMotionControllerComponentChanges;
        /**
         * Return whether or not the component changed the last frame
         */
        get hasChanges(): boolean;
        /**
         * is the button currently pressed
         */
        get pressed(): boolean;
        /**
         * is the button currently touched
         */
        get touched(): boolean;
        /**
         * Get the current value of this component
         */
        get value(): number;
        /**
         * Dispose this component
         */
        dispose(): void;
        /**
         * Are there axes correlating to this component
         * @returns true is axes data is available
         */
        isAxes(): boolean;
        /**
         * Is this component a button (hence - pressable)
         * @returns true if can be pressed
         */
        isButton(): boolean;
        /**
         * update this component using the gamepad object it is in. Called on every frame
         * @param nativeController the native gamepad controller object
         */
        update(nativeController: IMinimalMotionControllerObject): void;
    }


    /**
     * A generic hand controller class that supports select and a secondary grasp
     */
    export class WebXRGenericHandController extends WebXRAbstractMotionController {
        profileId: string;
        /**
         * Create a new hand controller object, without loading a controller model
         * @param scene the scene to use to create this controller
         * @param gamepadObject the corresponding gamepad object
         * @param handedness the handedness of the controller
         */
        constructor(scene: Scene, gamepadObject: IMinimalMotionControllerObject, handedness: MotionControllerHandedness);
        protected _getFilenameAndPath(): {
            filename: string;
            path: string;
        };
        protected _getModelLoadingConstraints(): boolean;
        protected _processLoadedModel(_meshes: AbstractMesh[]): void;
        protected _setRootMesh(meshes: AbstractMesh[]): void;
        protected _updateModel(): void;
    }


    /**
     * A generic trigger-only motion controller for WebXR
     */
    export class WebXRGenericTriggerMotionController extends WebXRAbstractMotionController {
        /**
         * Static version of the profile id of this controller
         */
        static ProfileId: string;
        profileId: string;
        constructor(scene: Scene, gamepadObject: IMinimalMotionControllerObject, handedness: MotionControllerHandedness);
        protected _getFilenameAndPath(): {
            filename: string;
            path: string;
        };
        protected _getModelLoadingConstraints(): boolean;
        protected _processLoadedModel(meshes: AbstractMesh[]): void;
        protected _setRootMesh(meshes: AbstractMesh[]): void;
        protected _updateModel(): void;
    }


    /**
     * The motion controller class for the standard HTC-Vive controllers
     */
    export class WebXRHTCViveMotionController extends WebXRAbstractMotionController {
        private _modelRootNode;
        /**
         * The base url used to load the left and right controller models
         */
        static MODEL_BASE_URL: string;
        /**
         * File name for the controller model.
         */
        static MODEL_FILENAME: string;
        profileId: string;
        /**
         * Create a new Vive motion controller object
         * @param scene the scene to use to create this controller
         * @param gamepadObject the corresponding gamepad object
         * @param handedness the handedness of the controller
         */
        constructor(scene: Scene, gamepadObject: IMinimalMotionControllerObject, handedness: MotionControllerHandedness);
        protected _getFilenameAndPath(): {
            filename: string;
            path: string;
        };
        protected _getModelLoadingConstraints(): boolean;
        protected _processLoadedModel(_meshes: AbstractMesh[]): void;
        protected _setRootMesh(meshes: AbstractMesh[]): void;
        protected _updateModel(): void;
    }


    /**
     * The motion controller class for all microsoft mixed reality controllers
     */
    export class WebXRMicrosoftMixedRealityController extends WebXRAbstractMotionController {
        protected readonly _mapping: {
            defaultButton: {
                valueNodeName: string;
                unpressedNodeName: string;
                pressedNodeName: string;
            };
            defaultAxis: {
                valueNodeName: string;
                minNodeName: string;
                maxNodeName: string;
            };
            buttons: {
                "xr-standard-trigger": {
                    rootNodeName: string;
                    componentProperty: string;
                    states: string[];
                };
                "xr-standard-squeeze": {
                    rootNodeName: string;
                    componentProperty: string;
                    states: string[];
                };
                "xr-standard-touchpad": {
                    rootNodeName: string;
                    labelAnchorNodeName: string;
                    touchPointNodeName: string;
                };
                "xr-standard-thumbstick": {
                    rootNodeName: string;
                    componentProperty: string;
                    states: string[];
                };
            };
            axes: {
                "xr-standard-touchpad": {
                    "x-axis": {
                        rootNodeName: string;
                    };
                    "y-axis": {
                        rootNodeName: string;
                    };
                };
                "xr-standard-thumbstick": {
                    "x-axis": {
                        rootNodeName: string;
                    };
                    "y-axis": {
                        rootNodeName: string;
                    };
                };
            };
        };
        /**
         * The base url used to load the left and right controller models
         */
        static MODEL_BASE_URL: string;
        /**
         * The name of the left controller model file
         */
        static MODEL_LEFT_FILENAME: string;
        /**
         * The name of the right controller model file
         */
        static MODEL_RIGHT_FILENAME: string;
        profileId: string;
        constructor(scene: Scene, gamepadObject: IMinimalMotionControllerObject, handedness: MotionControllerHandedness);
        protected _getFilenameAndPath(): {
            filename: string;
            path: string;
        };
        protected _getModelLoadingConstraints(): boolean;
        protected _processLoadedModel(_meshes: AbstractMesh[]): void;
        protected _setRootMesh(meshes: AbstractMesh[]): void;
        protected _updateModel(): void;
    }


    /**
     * A construction function type to create a new controller based on an xrInput object
     */
    export type MotionControllerConstructor = (xrInput: XRInputSource, scene: Scene) => WebXRAbstractMotionController;
    /**
     * Motion controller manager is managing the different webxr profiles and makes sure the right
     * controller is being loaded.
     */
    export class WebXRMotionControllerManager {
        private static _AvailableControllers;
        private static _Fallbacks;
        private static _ProfileLoadingPromises;
        private static _ProfilesList;
        /**
         * The base URL of the online controller repository. Can be changed at any time.
         */
        static BaseRepositoryUrl: string;
        /**
         * Which repository gets priority - local or online
         */
        static PrioritizeOnlineRepository: boolean;
        /**
         * Use the online repository, or use only locally-defined controllers
         */
        static UseOnlineRepository: boolean;
        /**
         * Disable the controller cache and load the models each time a new WebXRProfileMotionController is loaded.
         * Defaults to true.
         */
        static DisableControllerCache: boolean;
        /**
         * Clear the cache used for profile loading and reload when requested again
         */
        static ClearProfilesCache(): void;
        /**
         * Register the default fallbacks.
         * This function is called automatically when this file is imported.
         */
        static DefaultFallbacks(): void;
        /**
         * Find a fallback profile if the profile was not found. There are a few predefined generic profiles.
         * @param profileId the profile to which a fallback needs to be found
         * @returns an array with corresponding fallback profiles
         */
        static FindFallbackWithProfileId(profileId: string): string[];
        /**
         * When acquiring a new xrInput object (usually by the WebXRInput class), match it with the correct profile.
         * The order of search:
         *
         * 1) Iterate the profiles array of the xr input and try finding a corresponding motion controller
         * 2) (If not found) search in the gamepad id and try using it (legacy versions only)
         * 3) search for registered fallbacks (should be redundant, nonetheless it makes sense to check)
         * 4) return the generic trigger controller if none were found
         *
         * @param xrInput the xrInput to which a new controller is initialized
         * @param scene the scene to which the model will be added
         * @param forceProfile force a certain profile for this controller
         * @returns A promise that fulfils with the motion controller class for this profile id or the generic standard class if none was found
         */
        static GetMotionControllerWithXRInput(xrInput: XRInputSource, scene: Scene, forceProfile?: string): Promise<WebXRAbstractMotionController>;
        /**
         * Register a new controller based on its profile. This function will be called by the controller classes themselves.
         *
         * If you are missing a profile, make sure it is imported in your source, otherwise it will not register.
         *
         * @param type the profile type to register
         * @param constructFunction the function to be called when loading this profile
         */
        static RegisterController(type: string, constructFunction: MotionControllerConstructor): void;
        /**
         * Register a fallback to a specific profile.
         * @param profileId the profileId that will receive the fallbacks
         * @param fallbacks A list of fallback profiles
         */
        static RegisterFallbacksForProfileId(profileId: string, fallbacks: string[]): void;
        /**
         * Will update the list of profiles available in the repository
         * @returns a promise that resolves to a map of profiles available online
         */
        static UpdateProfilesList(): Promise<{
            [profile: string]: string;
        }>;
        /**
         * Clear the controller's cache (usually happens at the end of a session)
         */
        static ClearControllerCache(): void;
        private static _LoadProfileFromRepository;
        private static _LoadProfilesFromAvailableControllers;
    }


    /**
     * The motion controller class for oculus touch (quest, rift).
     * This class supports legacy mapping as well the standard xr mapping
     */
    export class WebXROculusTouchMotionController extends WebXRAbstractMotionController {
        private _forceLegacyControllers;
        private _modelRootNode;
        /**
         * The base url used to load the left and right controller models
         */
        static MODEL_BASE_URL: string;
        /**
         * The name of the left controller model file
         */
        static MODEL_LEFT_FILENAME: string;
        /**
         * The name of the right controller model file
         */
        static MODEL_RIGHT_FILENAME: string;
        /**
         * Base Url for the Quest controller model.
         */
        static QUEST_MODEL_BASE_URL: string;
        profileId: string;
        constructor(scene: Scene, gamepadObject: IMinimalMotionControllerObject, handedness: MotionControllerHandedness, _legacyMapping?: boolean, _forceLegacyControllers?: boolean);
        protected _getFilenameAndPath(): {
            filename: string;
            path: string;
        };
        protected _getModelLoadingConstraints(): boolean;
        protected _processLoadedModel(_meshes: AbstractMesh[]): void;
        protected _setRootMesh(meshes: AbstractMesh[]): void;
        protected _updateModel(): void;
        /**
         * Is this the new type of oculus touch. At the moment both have the same profile and it is impossible to differentiate
         * between the touch and touch 2.
         */
        private _isQuest;
    }


    /**
     * A profiled motion controller has its profile loaded from an online repository.
     * The class is responsible of loading the model, mapping the keys and enabling model-animations
     */
    export class WebXRProfiledMotionController extends WebXRAbstractMotionController {
        private _repositoryUrl;
        private controllerCache?;
        private _buttonMeshMapping;
        private _touchDots;
        /**
         * The profile ID of this controller. Will be populated when the controller initializes.
         */
        profileId: string;
        constructor(scene: Scene, xrInput: XRInputSource, _profile: IMotionControllerProfile, _repositoryUrl: string, controllerCache?: {
            filename: string;
            path: string;
            meshes: AbstractMesh[];
        }[] | undefined);
        dispose(): void;
        protected _getFilenameAndPath(): {
            filename: string;
            path: string;
        };
        protected _getModelLoadingConstraints(): boolean;
        protected _processLoadedModel(_meshes: AbstractMesh[]): void;
        protected _setRootMesh(meshes: AbstractMesh[]): void;
        protected _updateModel(_xrFrame: XRFrame): void;
    }




    /** @internal */
    interface INativeXRFrame extends XRFrame {
        getPoseData: (space: XRSpace, baseSpace: XRReferenceSpace, vectorBuffer: ArrayBuffer, matrixBuffer: ArrayBuffer) => XRPose;
        _imageTrackingResults?: XRImageTrackingResult[];
    }
    /** @internal */
    export class NativeXRFrame implements XRFrame {
        private _nativeImpl;
        private readonly _xrTransform;
        private readonly _xrPose;
        private readonly _xrPoseVectorData;
        get session(): XRSession;
        constructor(_nativeImpl: INativeXRFrame);
        getPose(space: XRSpace, baseSpace: XRReferenceSpace): XRPose | undefined;
        readonly fillPoses: any;
        readonly getViewerPose: any;
        readonly getHitTestResults: any;
        readonly getHitTestResultsForTransientInput: () => never;
        get trackedAnchors(): XRAnchorSet | undefined;
        readonly createAnchor: any;
        get worldInformation(): XRWorldInformation | undefined;
        get detectedPlanes(): XRPlaneSet | undefined;
        readonly getJointPose: any;
        readonly fillJointRadii: any;
        readonly getLightEstimate: () => never;
        get featurePointCloud(): number[] | undefined;
        readonly getImageTrackingResults: () => XRImageTrackingResult[];
    }


    /**
     * Wraps XRWebGLLayer's created by Babylon Native.
     * @internal
     */
    export class NativeXRLayerWrapper extends WebXRLayerWrapper {
        readonly layer: XRWebGLLayer;
        constructor(layer: XRWebGLLayer);
    }
    /**
     * Provides render target textures for layers created by Babylon Native.
     * @internal
     */
    export class NativeXRLayerRenderTargetTextureProvider extends WebXRLayerRenderTargetTextureProvider {
        readonly layerWrapper: NativeXRLayerWrapper;
        private _nativeRTTProvider;
        private _nativeLayer;
        constructor(sessionManager: WebXRSessionManager, layerWrapper: NativeXRLayerWrapper);
        trySetViewportForView(viewport: Viewport): boolean;
        getRenderTargetTextureForEye(eye: XREye): Nullable<RenderTargetTexture>;
        getRenderTargetTextureForView(view: XRView): Nullable<RenderTargetTexture>;
        getFramebufferDimensions(): Nullable<{
            framebufferWidth: number;
            framebufferHeight: number;
        }>;
    }
    /**
     * Creates the xr layer that will be used as the xr session's base layer.
     * @internal
     */
    export class NativeXRRenderTarget implements WebXRRenderTarget {
        canvasContext: WebGLRenderingContext;
        xrLayer: Nullable<XRWebGLLayer>;
        private _nativeRenderTarget;
        constructor(_xrSessionManager: WebXRSessionManager);
        initializeXRLayerAsync(xrSession: XRSession): Promise<XRWebGLLayer>;
        dispose(): void;
    }


    /**
     * WebXR Camera which holds the views for the xrSession
     * @see https://doc.babylonjs.com/features/featuresDeepDive/webXR/webXRCamera
     */
    export class WebXRCamera extends FreeCamera {
        private _xrSessionManager;
        private static _ScaleReadOnly;
        private _firstFrame;
        private _referenceQuaternion;
        private _referencedPosition;
        private _trackingState;
        /**
         * Observable raised before camera teleportation
         */
        onBeforeCameraTeleport: Observable<Vector3>;
        /**
         *  Observable raised after camera teleportation
         */
        onAfterCameraTeleport: Observable<Vector3>;
        /**
         * Notifies when the camera's tracking state has changed.
         * Notice - will also be triggered when tracking has started (at the beginning of the session)
         */
        onTrackingStateChanged: Observable<WebXRTrackingState>;
        /**
         * Should position compensation execute on first frame.
         * This is used when copying the position from a native (non XR) camera
         */
        compensateOnFirstFrame: boolean;
        /**
         * The last XRViewerPose from the current XRFrame
         * @internal
         */
        _lastXRViewerPose?: XRViewerPose;
        /**
         * Creates a new webXRCamera, this should only be set at the camera after it has been updated by the xrSessionManager
         * @param name the name of the camera
         * @param scene the scene to add the camera to
         * @param _xrSessionManager a constructed xr session manager
         */
        constructor(name: string, scene: Scene, _xrSessionManager: WebXRSessionManager);
        /**
         * Get the current XR tracking state of the camera
         */
        get trackingState(): WebXRTrackingState;
        private _setTrackingState;
        /**
         * Return the user's height, unrelated to the current ground.
         * This will be the y position of this camera, when ground level is 0.
         */
        get realWorldHeight(): number;
        /** @internal */
        _updateForDualEyeDebugging(): void;
        /**
         * Sets this camera's transformation based on a non-vr camera
         * @param otherCamera the non-vr camera to copy the transformation from
         * @param resetToBaseReferenceSpace should XR reset to the base reference space
         */
        setTransformationFromNonVRCamera(otherCamera?: Camera, resetToBaseReferenceSpace?: boolean): void;
        /**
         * Gets the current instance class name ("WebXRCamera").
         * @returns the class name
         */
        getClassName(): string;
        dispose(): void;
        private _updateFromXRSession;
        private _updateNumberOfRigCameras;
        private _updateReferenceSpace;
    }


    /**
     * Options for the default xr helper
     */
    export class WebXRDefaultExperienceOptions {
        /**
         * Enable or disable default UI to enter XR
         */
        disableDefaultUI?: boolean;
        /**
         * Should pointer selection not initialize.
         * Note that disabling pointer selection also disables teleportation.
         * Defaults to false.
         */
        disablePointerSelection?: boolean;
        /**
         * Should teleportation not initialize. Defaults to false.
         */
        disableTeleportation?: boolean;
        /**
         * Should nearInteraction not initialize. Defaults to false.
         */
        disableNearInteraction?: boolean;
        /**
         * Floor meshes that will be used for teleport
         */
        floorMeshes?: Array<AbstractMesh>;
        /**
         * If set to true, the first frame will not be used to reset position
         * The first frame is mainly used when copying transformation from the old camera
         * Mainly used in AR
         */
        ignoreNativeCameraTransformation?: boolean;
        /**
         * Optional configuration for the XR input object
         */
        inputOptions?: Partial<IWebXRInputOptions>;
        /**
         * optional configuration for pointer selection
         */
        pointerSelectionOptions?: Partial<IWebXRControllerPointerSelectionOptions>;
        /**
         * optional configuration for near interaction
         */
        nearInteractionOptions?: Partial<IWebXRNearInteractionOptions>;
        /**
         * optional configuration for teleportation
         */
        teleportationOptions?: Partial<IWebXRTeleportationOptions>;
        /**
         * optional configuration for the output canvas
         */
        outputCanvasOptions?: WebXRManagedOutputCanvasOptions;
        /**
         * optional UI options. This can be used among other to change session mode and reference space type
         */
        uiOptions?: Partial<WebXREnterExitUIOptions>;
        /**
         * When loading teleportation and pointer select, use stable versions instead of latest.
         */
        useStablePlugins?: boolean;
        /**
         * An optional rendering group id that will be set globally for teleportation, pointer selection and default controller meshes
         */
        renderingGroupId?: number;
        /**
         * A list of optional features to init the session with
         * If set to true, all features we support will be added
         */
        optionalFeatures?: boolean | string[];
    }
    /**
     * Default experience which provides a similar setup to the previous webVRExperience
     */
    export class WebXRDefaultExperience {
        /**
         * Base experience
         */
        baseExperience: WebXRExperienceHelper;
        /**
         * Enables ui for entering/exiting xr
         */
        enterExitUI: WebXREnterExitUI;
        /**
         * Input experience extension
         */
        input: WebXRInput;
        /**
         * Enables laser pointer and selection
         */
        pointerSelection: WebXRControllerPointerSelection;
        /**
         * Default target xr should render to
         */
        renderTarget: WebXRRenderTarget;
        /**
         * Enables teleportation
         */
        teleportation: WebXRMotionControllerTeleportation;
        /**
         * Enables near interaction for hands/controllers
         */
        nearInteraction: WebXRNearInteraction;
        private constructor();
        /**
         * Creates the default xr experience
         * @param scene scene
         * @param options options for basic configuration
         * @returns resulting WebXRDefaultExperience
         */
        static CreateAsync(scene: Scene, options?: WebXRDefaultExperienceOptions): Promise<WebXRDefaultExperience>;
        /**
         * Disposes of the experience helper
         */
        dispose(): void;
    }


    /**
     * Button which can be used to enter a different mode of XR
     */
    export class WebXREnterExitUIButton {
        /** button element */
        element: HTMLElement;
        /** XR initialization options for the button */
        sessionMode: XRSessionMode;
        /** Reference space type */
        referenceSpaceType: XRReferenceSpaceType;
        /**
         * Creates a WebXREnterExitUIButton
         * @param element button element
         * @param sessionMode XR initialization session mode
         * @param referenceSpaceType the type of reference space to be used
         */
        constructor(
        /** button element */
        element: HTMLElement, 
        /** XR initialization options for the button */
        sessionMode: XRSessionMode, 
        /** Reference space type */
        referenceSpaceType: XRReferenceSpaceType);
        /**
         * Extendable function which can be used to update the button's visuals when the state changes
         * @param activeButton the current active button in the UI
         */
        update(activeButton: Nullable<WebXREnterExitUIButton>): void;
    }
    /**
     * Options to create the webXR UI
     */
    export class WebXREnterExitUIOptions {
        /**
         * User provided buttons to enable/disable WebXR. The system will provide default if not set
         */
        customButtons?: Array<WebXREnterExitUIButton>;
        /**
         * A reference space type to use when creating the default button.
         * Default is local-floor
         */
        referenceSpaceType?: XRReferenceSpaceType;
        /**
         * Context to enter xr with
         */
        renderTarget?: Nullable<WebXRRenderTarget>;
        /**
         * A session mode to use when creating the default button.
         * Default is immersive-vr
         */
        sessionMode?: XRSessionMode;
        /**
         * A list of optional features to init the session with
         */
        optionalFeatures?: string[];
        /**
         * A list of optional features to init the session with
         */
        requiredFeatures?: string[];
        /**
         * If set, the `sessiongranted` event will not be registered. `sessiongranted` is used to move seamlessly between WebXR experiences.
         * If set to true the user will be forced to press the "enter XR" button even if sessiongranted event was triggered.
         * If not set and a sessiongranted event was triggered, the XR session will start automatically.
         */
        ignoreSessionGrantedEvent?: boolean;
        /**
         * If defined, this function will be executed if the UI encounters an error when entering XR
         */
        onError?: (error: any) => void;
    }
    /**
     * UI to allow the user to enter/exit XR mode
     */
    export class WebXREnterExitUI implements IDisposable {
        private _scene;
        /** version of the options passed to this UI */
        options: WebXREnterExitUIOptions;
        private _activeButton;
        private _buttons;
        private _helper;
        private _renderTarget?;
        /**
         * The HTML Div Element to which buttons are added.
         */
        readonly overlay: HTMLDivElement;
        /**
         * Fired every time the active button is changed.
         *
         * When xr is entered via a button that launches xr that button will be the callback parameter
         *
         * When exiting xr the callback parameter will be null)
         */
        activeButtonChangedObservable: Observable<Nullable<WebXREnterExitUIButton>>;
        /**
         * Construct a new EnterExit UI class
         *
         * @param _scene babylon scene object to use
         * @param options (read-only) version of the options passed to this UI
         */
        constructor(_scene: Scene, 
        /** version of the options passed to this UI */
        options: WebXREnterExitUIOptions);
        /**
         * Set the helper to be used with this UI component.
         * The UI is bound to an experience helper. If not provided the UI can still be used but the events should be registered by the developer.
         *
         * @param helper the experience helper to attach
         * @param renderTarget an optional render target (in case it is created outside of the helper scope)
         * @returns a promise that resolves when the ui is ready
         */
        setHelperAsync(helper: WebXRExperienceHelper, renderTarget?: WebXRRenderTarget): Promise<void>;
        /**
         * Creates UI to allow the user to enter/exit XR mode
         * @param scene the scene to add the ui to
         * @param helper the xr experience helper to enter/exit xr with
         * @param options options to configure the UI
         * @returns the created ui
         */
        static CreateAsync(scene: Scene, helper: WebXRExperienceHelper, options: WebXREnterExitUIOptions): Promise<WebXREnterExitUI>;
        private _enterXRWithButtonIndex;
        /**
         * Disposes of the XR UI component
         */
        dispose(): void;
        private _onSessionGranted;
        private _updateButtons;
    }


    /**
     * Options for setting up XR spectator camera.
     */
    export interface WebXRSpectatorModeOption {
        /**
         * Expected refresh rate (frames per sec) for a spectator camera.
         */
        fps?: number;
        /**
         * The index of rigCameras array in a WebXR camera.
         */
        preferredCameraIndex?: number;
    }
    /**
     * Base set of functionality needed to create an XR experience (WebXRSessionManager, Camera, StateManagement, etc.)
     * @see https://doc.babylonjs.com/features/featuresDeepDive/webXR/webXRExperienceHelpers
     */
    export class WebXRExperienceHelper implements IDisposable {
        private _scene;
        private _nonVRCamera;
        private _attachedToElement;
        private _spectatorCamera;
        private _originalSceneAutoClear;
        private _supported;
        private _spectatorMode;
        private _lastTimestamp;
        /**
         * Camera used to render xr content
         */
        camera: WebXRCamera;
        /** A features manager for this xr session */
        featuresManager: WebXRFeaturesManager;
        /**
         * Observers registered here will be triggered after the camera's initial transformation is set
         * This can be used to set a different ground level or an extra rotation.
         *
         * Note that ground level is considered to be at 0. The height defined by the XR camera will be added
         * to the position set after this observable is done executing.
         */
        onInitialXRPoseSetObservable: Observable<WebXRCamera>;
        /**
         * Fires when the state of the experience helper has changed
         */
        onStateChangedObservable: Observable<WebXRState>;
        /** Session manager used to keep track of xr session */
        sessionManager: WebXRSessionManager;
        /**
         * The current state of the XR experience (eg. transitioning, in XR or not in XR)
         */
        state: WebXRState;
        /**
         * Creates a WebXRExperienceHelper
         * @param _scene The scene the helper should be created in
         */
        private constructor();
        /**
         * Creates the experience helper
         * @param scene the scene to attach the experience helper to
         * @returns a promise for the experience helper
         */
        static CreateAsync(scene: Scene): Promise<WebXRExperienceHelper>;
        /**
         * Disposes of the experience helper
         */
        dispose(): void;
        /**
         * Enters XR mode (This must be done within a user interaction in most browsers eg. button click)
         * @param sessionMode options for the XR session
         * @param referenceSpaceType frame of reference of the XR session
         * @param renderTarget the output canvas that will be used to enter XR mode
         * @param sessionCreationOptions optional XRSessionInit object to init the session with
         * @returns promise that resolves after xr mode has entered
         */
        enterXRAsync(sessionMode: XRSessionMode, referenceSpaceType: XRReferenceSpaceType, renderTarget?: WebXRRenderTarget, sessionCreationOptions?: XRSessionInit): Promise<WebXRSessionManager>;
        /**
         * Exits XR mode and returns the scene to its original state
         * @returns promise that resolves after xr mode has exited
         */
        exitXRAsync(): Promise<void>;
        /**
         * Enable spectator mode for desktop VR experiences.
         * When spectator mode is enabled a camera will be attached to the desktop canvas and will
         * display the first rig camera's view on the desktop canvas.
         * Please note that this will degrade performance, as it requires another camera render.
         * It is also not recommended to enable this in devices like the quest, as it brings no benefit there.
         * @param options giving WebXRSpectatorModeOption for specutator camera to setup when the spectator mode is enabled.
         */
        enableSpectatorMode(options?: WebXRSpectatorModeOption): void;
        /**
         * Disable spectator mode for desktop VR experiences.
         */
        disableSpecatatorMode(): void;
        private _switchSpectatorMode;
        private _nonXRToXRCamera;
        private _setState;
    }


    /**
     * Defining the interface required for a (webxr) feature
     */
    export interface IWebXRFeature extends IDisposable {
        /**
         * Is this feature attached
         */
        attached: boolean;
        /**
         * Should auto-attach be disabled?
         */
        disableAutoAttach: boolean;
        /**
         * Attach the feature to the session
         * Will usually be called by the features manager
         *
         * @param force should attachment be forced (even when already attached)
         * @returns true if successful.
         */
        attach(force?: boolean): boolean;
        /**
         * Detach the feature from the session
         * Will usually be called by the features manager
         *
         * @returns true if successful.
         */
        detach(): boolean;
        /**
         * This function will be executed during before enabling the feature and can be used to not-allow enabling it.
         * Note that at this point the session has NOT started, so this is purely checking if the browser supports it
         *
         * @returns whether or not the feature is compatible in this environment
         */
        isCompatible(): boolean;
        /**
         * Was this feature disposed;
         */
        isDisposed: boolean;
        /**
         * The name of the native xr feature name, if applicable (like anchor, hit-test, or hand-tracking)
         */
        xrNativeFeatureName?: string;
        /**
         * A list of (Babylon WebXR) features this feature depends on
         */
        dependsOn?: string[];
        /**
         * If this feature requires to extend the XRSessionInit object, this function will return the partial XR session init object
         */
        getXRSessionInitExtension?: () => Promise<Partial<XRSessionInit>>;
    }
    /**
     * A list of the currently available features without referencing them
     */
    export class WebXRFeatureName {
        /**
         * The name of the anchor system feature
         */
        static readonly ANCHOR_SYSTEM = "xr-anchor-system";
        /**
         * The name of the background remover feature
         */
        static readonly BACKGROUND_REMOVER = "xr-background-remover";
        /**
         * The name of the hit test feature
         */
        static readonly HIT_TEST = "xr-hit-test";
        /**
         * The name of the mesh detection feature
         */
        static readonly MESH_DETECTION = "xr-mesh-detection";
        /**
         * physics impostors for xr controllers feature
         */
        static readonly PHYSICS_CONTROLLERS = "xr-physics-controller";
        /**
         * The name of the plane detection feature
         */
        static readonly PLANE_DETECTION = "xr-plane-detection";
        /**
         * The name of the pointer selection feature
         */
        static readonly POINTER_SELECTION = "xr-controller-pointer-selection";
        /**
         * The name of the teleportation feature
         */
        static readonly TELEPORTATION = "xr-controller-teleportation";
        /**
         * The name of the feature points feature.
         */
        static readonly FEATURE_POINTS = "xr-feature-points";
        /**
         * The name of the hand tracking feature.
         */
        static readonly HAND_TRACKING = "xr-hand-tracking";
        /**
         * The name of the image tracking feature
         */
        static readonly IMAGE_TRACKING = "xr-image-tracking";
        /**
         * The name of the near interaction feature
         */
        static readonly NEAR_INTERACTION = "xr-near-interaction";
        /**
         * The name of the DOM overlay feature
         */
        static readonly DOM_OVERLAY = "xr-dom-overlay";
        /**
         * The name of the movement feature
         */
        static readonly MOVEMENT = "xr-controller-movement";
        /**
         * The name of the light estimation feature
         */
        static readonly LIGHT_ESTIMATION = "xr-light-estimation";
        /**
         * The name of the eye tracking feature
         */
        static readonly EYE_TRACKING = "xr-eye-tracking";
        /**
         * The name of the walking locomotion feature
         */
        static readonly WALKING_LOCOMOTION = "xr-walking-locomotion";
        /**
         * The name of the composition layers feature
         */
        static readonly LAYERS = "xr-layers";
    }
    /**
     * Defining the constructor of a feature. Used to register the modules.
     */
    export type WebXRFeatureConstructor = (xrSessionManager: WebXRSessionManager, options?: any) => () => IWebXRFeature;
    /**
     * The WebXR features manager is responsible of enabling or disabling features required for the current XR session.
     * It is mainly used in AR sessions.
     *
     * A feature can have a version that is defined by Babylon (and does not correspond with the webxr version).
     */
    export class WebXRFeaturesManager implements IDisposable {
        private _xrSessionManager;
        private static readonly _AvailableFeatures;
        private _features;
        /**
         * The key is the feature to check and the value is the feature that conflicts.
         */
        private static readonly _ConflictingFeatures;
        /**
         * constructs a new features manages.
         *
         * @param _xrSessionManager an instance of WebXRSessionManager
         */
        constructor(_xrSessionManager: WebXRSessionManager);
        /**
         * Used to register a module. After calling this function a developer can use this feature in the scene.
         * Mainly used internally.
         *
         * @param featureName the name of the feature to register
         * @param constructorFunction the function used to construct the module
         * @param version the (babylon) version of the module
         * @param stable is that a stable version of this module
         */
        static AddWebXRFeature(featureName: string, constructorFunction: WebXRFeatureConstructor, version?: number, stable?: boolean): void;
        /**
         * Returns a constructor of a specific feature.
         *
         * @param featureName the name of the feature to construct
         * @param version the version of the feature to load
         * @param xrSessionManager the xrSessionManager. Used to construct the module
         * @param options optional options provided to the module.
         * @returns a function that, when called, will return a new instance of this feature
         */
        static ConstructFeature(featureName: string, version: number | undefined, xrSessionManager: WebXRSessionManager, options?: any): () => IWebXRFeature;
        /**
         * Can be used to return the list of features currently registered
         *
         * @returns an Array of available features
         */
        static GetAvailableFeatures(): string[];
        /**
         * Gets the versions available for a specific feature
         * @param featureName the name of the feature
         * @returns an array with the available versions
         */
        static GetAvailableVersions(featureName: string): string[];
        /**
         * Return the latest unstable version of this feature
         * @param featureName the name of the feature to search
         * @returns the version number. if not found will return -1
         */
        static GetLatestVersionOfFeature(featureName: string): number;
        /**
         * Return the latest stable version of this feature
         * @param featureName the name of the feature to search
         * @returns the version number. if not found will return -1
         */
        static GetStableVersionOfFeature(featureName: string): number;
        /**
         * Attach a feature to the current session. Mainly used when session started to start the feature effect.
         * Can be used during a session to start a feature
         * @param featureName the name of feature to attach
         */
        attachFeature(featureName: string): void;
        /**
         * Can be used inside a session or when the session ends to detach a specific feature
         * @param featureName the name of the feature to detach
         */
        detachFeature(featureName: string): void;
        /**
         * Used to disable an already-enabled feature
         * The feature will be disposed and will be recreated once enabled.
         * @param featureName the feature to disable
         * @returns true if disable was successful
         */
        disableFeature(featureName: string | {
            Name: string;
        }): boolean;
        /**
         * dispose this features manager
         */
        dispose(): void;
        /**
         * Enable a feature using its name and a version. This will enable it in the scene, and will be responsible to attach it when the session starts.
         * If used twice, the old version will be disposed and a new one will be constructed. This way you can re-enable with different configuration.
         *
         * @param featureName the name of the feature to load or the class of the feature
         * @param version optional version to load. if not provided the latest version will be enabled
         * @param moduleOptions options provided to the module. Ses the module documentation / constructor
         * @param attachIfPossible if set to true (default) the feature will be automatically attached, if it is currently possible
         * @param required is this feature required to the app. If set to true the session init will fail if the feature is not available.
         * @returns a new constructed feature or throws an error if feature not found or conflicts with another enabled feature.
         */
        enableFeature(featureName: string | {
            Name: string;
        }, version?: number | string, moduleOptions?: any, attachIfPossible?: boolean, required?: boolean): IWebXRFeature;
        /**
         * get the implementation of an enabled feature.
         * @param featureName the name of the feature to load
         * @returns the feature class, if found
         */
        getEnabledFeature(featureName: string): IWebXRFeature;
        /**
         * Get the list of enabled features
         * @returns an array of enabled features
         */
        getEnabledFeatures(): string[];
        /**
         * This function will extend the session creation configuration object with enabled features.
         * If, for example, the anchors feature is enabled, it will be automatically added to the optional or required features list,
         * according to the defined "required" variable, provided during enableFeature call
         * @param xrSessionInit the xr Session init object to extend
         *
         * @returns an extended XRSessionInit object
         */
        _extendXRSessionInitObject(xrSessionInit: XRSessionInit): Promise<XRSessionInit>;
    }


    /**
     * The schema for initialization options of the XR Input class
     */
    export interface IWebXRInputOptions {
        /**
         * If set to true no model will be automatically loaded
         */
        doNotLoadControllerMeshes?: boolean;
        /**
         * If set, this profile will be used for all controllers loaded (for example "microsoft-mixed-reality")
         * If not found, the xr input profile data will be used.
         * Profiles are defined here - https://github.com/immersive-web/webxr-input-profiles/
         */
        forceInputProfile?: string;
        /**
         * Do not send a request to the controller repository to load the profile.
         *
         * Instead, use the controllers available in babylon itself.
         */
        disableOnlineControllerRepository?: boolean;
        /**
         * A custom URL for the controllers repository
         */
        customControllersRepositoryURL?: string;
        /**
         * Should the controller model's components not move according to the user input
         */
        disableControllerAnimation?: boolean;
        /**
         * Optional options to pass to the controller. Will be overridden by the Input options where applicable
         */
        controllerOptions?: IWebXRControllerOptions;
    }
    /**
     * XR input used to track XR inputs such as controllers/rays
     */
    export class WebXRInput implements IDisposable {
        /**
         * the xr session manager for this session
         */
        xrSessionManager: WebXRSessionManager;
        /**
         * the WebXR camera for this session. Mainly used for teleportation
         */
        xrCamera: WebXRCamera;
        private readonly _options;
        /**
         * XR controllers being tracked
         */
        controllers: Array<WebXRInputSource>;
        private _frameObserver;
        private _sessionEndedObserver;
        private _sessionInitObserver;
        /**
         * Event when a controller has been connected/added
         */
        onControllerAddedObservable: Observable<WebXRInputSource>;
        /**
         * Event when a controller has been removed/disconnected
         */
        onControllerRemovedObservable: Observable<WebXRInputSource>;
        /**
         * Initializes the WebXRInput
         * @param xrSessionManager the xr session manager for this session
         * @param xrCamera the WebXR camera for this session. Mainly used for teleportation
         * @param _options = initialization options for this xr input
         */
        constructor(
        /**
         * the xr session manager for this session
         */
        xrSessionManager: WebXRSessionManager, 
        /**
         * the WebXR camera for this session. Mainly used for teleportation
         */
        xrCamera: WebXRCamera, _options?: IWebXRInputOptions);
        private _onInputSourcesChange;
        private _addAndRemoveControllers;
        /**
         * Disposes of the object
         */
        dispose(): void;
    }


    /**
     * Configuration options for the WebXR controller creation
     */
    export interface IWebXRControllerOptions {
        /**
         * Should the controller mesh be animated when a user interacts with it
         * The pressed buttons / thumbstick and touchpad animations will be disabled
         */
        disableMotionControllerAnimation?: boolean;
        /**
         * Do not load the controller mesh, in case a different mesh needs to be loaded.
         */
        doNotLoadControllerMesh?: boolean;
        /**
         * Force a specific controller type for this controller.
         * This can be used when creating your own profile or when testing different controllers
         */
        forceControllerProfile?: string;
        /**
         * Defines a rendering group ID for meshes that will be loaded.
         * This is for the default controllers only.
         */
        renderingGroupId?: number;
    }
    /**
     * Represents an XR controller
     */
    export class WebXRInputSource {
        private _scene;
        /** The underlying input source for the controller  */
        inputSource: XRInputSource;
        private _options;
        private _tmpVector;
        private _uniqueId;
        private _disposed;
        /**
         * Represents the part of the controller that is held. This may not exist if the controller is the head mounted display itself, if that's the case only the pointer from the head will be available
         */
        grip?: AbstractMesh;
        /**
         * If available, this is the gamepad object related to this controller.
         * Using this object it is possible to get click events and trackpad changes of the
         * webxr controller that is currently being used.
         */
        motionController?: WebXRAbstractMotionController;
        /**
         * Event that fires when the controller is removed/disposed.
         * The object provided as event data is this controller, after associated assets were disposed.
         * uniqueId is still available.
         */
        onDisposeObservable: Observable<WebXRInputSource>;
        /**
         * Will be triggered when the mesh associated with the motion controller is done loading.
         * It is also possible that this will never trigger (!) if no mesh was loaded, or if the developer decides to load a different mesh
         * A shortened version of controller -> motion controller -> on mesh loaded.
         */
        onMeshLoadedObservable: Observable<AbstractMesh>;
        /**
         * Observers registered here will trigger when a motion controller profile was assigned to this xr controller
         */
        onMotionControllerInitObservable: Observable<WebXRAbstractMotionController>;
        /**
         * Pointer which can be used to select objects or attach a visible laser to
         */
        pointer: AbstractMesh;
        /**
         * The last XRPose the was calculated on the current XRFrame
         * @internal
         */
        _lastXRPose?: XRPose;
        /**
         * Creates the input source object
         * @see https://doc.babylonjs.com/features/featuresDeepDive/webXR/webXRInputControllerSupport
         * @param _scene the scene which the controller should be associated to
         * @param inputSource the underlying input source for the controller
         * @param _options options for this controller creation
         */
        constructor(_scene: Scene, 
        /** The underlying input source for the controller  */
        inputSource: XRInputSource, _options?: IWebXRControllerOptions);
        /**
         * Get this controllers unique id
         */
        get uniqueId(): string;
        /**
         * Disposes of the object
         */
        dispose(): void;
        /**
         * Gets a world space ray coming from the pointer or grip
         * @param result the resulting ray
         * @param gripIfAvailable use the grip mesh instead of the pointer, if available
         */
        getWorldPointerRayToRef(result: Ray, gripIfAvailable?: boolean): void;
        /**
         * Updates the controller pose based on the given XRFrame
         * @param xrFrame xr frame to update the pose with
         * @param referenceSpace reference space to use
         * @param xrCamera the xr camera, used for parenting
         */
        updateFromXRFrame(xrFrame: XRFrame, referenceSpace: XRReferenceSpace, xrCamera: WebXRCamera): void;
    }


    /** Covers all supported subclasses of WebXR's XRCompositionLayer */
    export type WebXRCompositionLayerType = "XRProjectionLayer";
    /** Covers all supported subclasses of WebXR's XRLayer */
    export type WebXRLayerType = "XRWebGLLayer" | WebXRCompositionLayerType;
    /**
     * Wrapper over subclasses of XRLayer.
     * @internal
     */
    export class WebXRLayerWrapper {
        /** The width of the layer's framebuffer. */
        getWidth: () => number;
        /** The height of the layer's framebuffer. */
        getHeight: () => number;
        /** The XR layer that this WebXRLayerWrapper wraps. */
        readonly layer: XRLayer;
        /** The type of XR layer that is being wrapped. */
        readonly layerType: WebXRLayerType;
        /** Create a render target provider for the wrapped layer. */
        createRenderTargetTextureProvider: (xrSessionManager: WebXRSessionManager) => WebXRLayerRenderTargetTextureProvider;
        /**
         * Check if fixed foveation is supported on this device
         */
        get isFixedFoveationSupported(): boolean;
        /**
         * Get the fixed foveation currently set, as specified by the webxr specs
         * If this returns null, then fixed foveation is not supported
         */
        get fixedFoveation(): Nullable<number>;
        /**
         * Set the fixed foveation to the specified value, as specified by the webxr specs
         * This value will be normalized to be between 0 and 1, 1 being max foveation, 0 being no foveation
         */
        set fixedFoveation(value: Nullable<number>);
        protected constructor(
        /** The width of the layer's framebuffer. */
        getWidth: () => number, 
        /** The height of the layer's framebuffer. */
        getHeight: () => number, 
        /** The XR layer that this WebXRLayerWrapper wraps. */
        layer: XRLayer, 
        /** The type of XR layer that is being wrapped. */
        layerType: WebXRLayerType, 
        /** Create a render target provider for the wrapped layer. */
        createRenderTargetTextureProvider: (xrSessionManager: WebXRSessionManager) => WebXRLayerRenderTargetTextureProvider);
    }


    /**
     * Configuration object for WebXR output canvas
     */
    export class WebXRManagedOutputCanvasOptions {
        /**
         * An optional canvas in case you wish to create it yourself and provide it here.
         * If not provided, a new canvas will be created
         */
        canvasElement?: HTMLCanvasElement;
        /**
         * Options for this XR Layer output
         */
        canvasOptions?: XRWebGLLayerInit;
        /**
         * CSS styling for a newly created canvas (if not provided)
         */
        newCanvasCssStyle?: string;
        /**
         * Get the default values of the configuration object
         * @param engine defines the engine to use (can be null)
         * @returns default values of this configuration object
         */
        static GetDefaults(engine?: ThinEngine): WebXRManagedOutputCanvasOptions;
    }
    /**
     * Creates a canvas that is added/removed from the webpage when entering/exiting XR
     */
    export class WebXRManagedOutputCanvas implements WebXRRenderTarget {
        private _options;
        private _canvas;
        private _engine;
        private _originalCanvasSize;
        /**
         * Rendering context of the canvas which can be used to display/mirror xr content
         */
        canvasContext: WebGLRenderingContext;
        /**
         * xr layer for the canvas
         */
        xrLayer: Nullable<XRWebGLLayer>;
        private _xrLayerWrapper;
        /**
         * Observers registered here will be triggered when the xr layer was initialized
         */
        onXRLayerInitObservable: Observable<XRWebGLLayer>;
        /**
         * Initializes the canvas to be added/removed upon entering/exiting xr
         * @param _xrSessionManager The XR Session manager
         * @param _options optional configuration for this canvas output. defaults will be used if not provided
         */
        constructor(_xrSessionManager: WebXRSessionManager, _options?: WebXRManagedOutputCanvasOptions);
        /**
         * Disposes of the object
         */
        dispose(): void;
        /**
         * Initializes a XRWebGLLayer to be used as the session's baseLayer.
         * @param xrSession xr session
         * @returns a promise that will resolve once the XR Layer has been created
         */
        initializeXRLayerAsync(xrSession: XRSession): Promise<XRWebGLLayer>;
        private _addCanvas;
        private _removeCanvas;
        private _setCanvasSize;
        private _setManagedOutputCanvas;
    }


    /**
     * An interface for objects that provide render target textures for XR rendering.
     */
    export interface IWebXRRenderTargetTextureProvider extends IDisposable {
        /**
         * Attempts to set the framebuffer-size-normalized viewport to be rendered this frame for this view.
         * In the event of a failure, the supplied viewport is not updated.
         * @param viewport the viewport to which the view will be rendered
         * @param view the view for which to set the viewport
         * @returns whether the operation was successful
         */
        trySetViewportForView(viewport: Viewport, view: XRView): boolean;
        /**
         * Gets the correct render target texture to be rendered this frame for this eye
         * @param eye the eye for which to get the render target
         * @returns the render target for the specified eye or null if not available
         */
        getRenderTargetTextureForEye(eye: XREye): Nullable<RenderTargetTexture>;
        /**
         * Gets the correct render target texture to be rendered this frame for this view
         * @param view the view for which to get the render target
         * @returns the render target for the specified view or null if not available
         */
        getRenderTargetTextureForView(view: XRView): Nullable<RenderTargetTexture>;
    }
    /**
     * Provides render target textures and other important rendering information for a given XRLayer.
     * @internal
     */
    export abstract class WebXRLayerRenderTargetTextureProvider implements IWebXRRenderTargetTextureProvider {
        private readonly _scene;
        readonly layerWrapper: WebXRLayerWrapper;
        abstract trySetViewportForView(viewport: Viewport, view: XRView): boolean;
        abstract getRenderTargetTextureForEye(eye: XREye): Nullable<RenderTargetTexture>;
        abstract getRenderTargetTextureForView(view: XRView): Nullable<RenderTargetTexture>;
        protected _renderTargetTextures: RenderTargetTexture[];
        protected _framebufferDimensions: Nullable<{
            framebufferWidth: number;
            framebufferHeight: number;
        }>;
        private _engine;
        constructor(_scene: Scene, layerWrapper: WebXRLayerWrapper);
        private _createInternalTexture;
        protected _createRenderTargetTexture(width: number, height: number, framebuffer: Nullable<WebGLFramebuffer>, colorTexture?: WebGLTexture, depthStencilTexture?: WebGLTexture, multiview?: boolean): RenderTargetTexture;
        protected _destroyRenderTargetTexture(renderTargetTexture: RenderTargetTexture): void;
        getFramebufferDimensions(): Nullable<{
            framebufferWidth: number;
            framebufferHeight: number;
        }>;
        dispose(): void;
    }


    /**
     * Manages an XRSession to work with Babylon's engine
     * @see https://doc.babylonjs.com/features/featuresDeepDive/webXR/webXRSessionManagers
     */
    export class WebXRSessionManager implements IDisposable, IWebXRRenderTargetTextureProvider {
        /** The scene which the session should be created for */
        scene: Scene;
        private _engine;
        private _referenceSpace;
        private _baseLayerWrapper;
        private _baseLayerRTTProvider;
        private _xrNavigator;
        private _sessionMode;
        private _onEngineDisposedObserver;
        /**
         * The base reference space from which the session started. good if you want to reset your
         * reference space
         */
        baseReferenceSpace: XRReferenceSpace;
        /**
         * Current XR frame
         */
        currentFrame: Nullable<XRFrame>;
        /** WebXR timestamp updated every frame */
        currentTimestamp: number;
        /**
         * Used just in case of a failure to initialize an immersive session.
         * The viewer reference space is compensated using this height, creating a kind of "viewer-floor" reference space
         */
        defaultHeightCompensation: number;
        /**
         * Fires every time a new xrFrame arrives which can be used to update the camera
         */
        onXRFrameObservable: Observable<XRFrame>;
        /**
         * Fires when the reference space changed
         */
        onXRReferenceSpaceChanged: Observable<XRReferenceSpace>;
        /**
         * Fires when the xr session is ended either by the device or manually done
         */
        onXRSessionEnded: Observable<any>;
        /**
         * Fires when the xr session is initialized: right after requestSession was called and returned with a successful result
         */
        onXRSessionInit: Observable<XRSession>;
        /**
         * Underlying xr session
         */
        session: XRSession;
        /**
         * The viewer (head position) reference space. This can be used to get the XR world coordinates
         * or get the offset the player is currently at.
         */
        viewerReferenceSpace: XRReferenceSpace;
        /**
         * Are we currently in the XR loop?
         */
        inXRFrameLoop: boolean;
        /**
         * Are we in an XR session?
         */
        inXRSession: boolean;
        /**
         * Constructs a WebXRSessionManager, this must be initialized within a user action before usage
         * @param scene The scene which the session should be created for
         */
        constructor(
        /** The scene which the session should be created for */
        scene: Scene);
        /**
         * The current reference space used in this session. This reference space can constantly change!
         * It is mainly used to offset the camera's position.
         */
        get referenceSpace(): XRReferenceSpace;
        /**
         * Set a new reference space and triggers the observable
         */
        set referenceSpace(newReferenceSpace: XRReferenceSpace);
        /**
         * The mode for the managed XR session
         */
        get sessionMode(): XRSessionMode;
        /**
         * Disposes of the session manager
         * This should be called explicitly by the dev, if required.
         */
        dispose(): void;
        /**
         * Stops the xrSession and restores the render loop
         * @returns Promise which resolves after it exits XR
         */
        exitXRAsync(): Promise<void>;
        /**
         * Attempts to set the framebuffer-size-normalized viewport to be rendered this frame for this view.
         * In the event of a failure, the supplied viewport is not updated.
         * @param viewport the viewport to which the view will be rendered
         * @param view the view for which to set the viewport
         * @returns whether the operation was successful
         */
        trySetViewportForView(viewport: Viewport, view: XRView): boolean;
        /**
         * Gets the correct render target texture to be rendered this frame for this eye
         * @param eye the eye for which to get the render target
         * @returns the render target for the specified eye or null if not available
         */
        getRenderTargetTextureForEye(eye: XREye): Nullable<RenderTargetTexture>;
        /**
         * Gets the correct render target texture to be rendered this frame for this view
         * @param view the view for which to get the render target
         * @returns the render target for the specified view or null if not available
         */
        getRenderTargetTextureForView(view: XRView): Nullable<RenderTargetTexture>;
        /**
         * Creates a WebXRRenderTarget object for the XR session
         * @param options optional options to provide when creating a new render target
         * @returns a WebXR render target to which the session can render
         */
        getWebXRRenderTarget(options?: WebXRManagedOutputCanvasOptions): WebXRRenderTarget;
        /**
         * Initializes the manager
         * After initialization enterXR can be called to start an XR session
         * @returns Promise which resolves after it is initialized
         */
        initializeAsync(): Promise<void>;
        /**
         * Initializes an xr session
         * @param xrSessionMode mode to initialize
         * @param xrSessionInit defines optional and required values to pass to the session builder
         * @returns a promise which will resolve once the session has been initialized
         */
        initializeSessionAsync(xrSessionMode?: XRSessionMode, xrSessionInit?: XRSessionInit): Promise<XRSession>;
        /**
         * Checks if a session would be supported for the creation options specified
         * @param sessionMode session mode to check if supported eg. immersive-vr
         * @returns A Promise that resolves to true if supported and false if not
         */
        isSessionSupportedAsync(sessionMode: XRSessionMode): Promise<boolean>;
        /**
         * Resets the reference space to the one started the session
         */
        resetReferenceSpace(): void;
        /**
         * Starts rendering to the xr layer
         */
        runXRRenderLoop(): void;
        /**
         * Sets the reference space on the xr session
         * @param referenceSpaceType space to set
         * @returns a promise that will resolve once the reference space has been set
         */
        setReferenceSpaceTypeAsync(referenceSpaceType?: XRReferenceSpaceType): Promise<XRReferenceSpace>;
        /**
         * Updates the render state of the session.
         * Note that this is deprecated in favor of WebXRSessionManager.updateRenderState().
         * @param state state to set
         * @returns a promise that resolves once the render state has been updated
         * @deprecated
         */
        updateRenderStateAsync(state: XRRenderState): Promise<void>;
        /**
         * @internal
         */
        _setBaseLayerWrapper(baseLayerWrapper: Nullable<WebXRLayerWrapper>): void;
        /**
         * Updates the render state of the session
         * @param state state to set
         */
        updateRenderState(state: XRRenderStateInit): void;
        /**
         * Returns a promise that resolves with a boolean indicating if the provided session mode is supported by this browser
         * @param sessionMode defines the session to test
         * @returns a promise with boolean as final value
         */
        static IsSessionSupportedAsync(sessionMode: XRSessionMode): Promise<boolean>;
        /**
         * Returns true if Babylon.js is using the BabylonNative backend, otherwise false
         */
        get isNative(): boolean;
        /**
         * The current frame rate as reported by the device
         */
        get currentFrameRate(): number | undefined;
        /**
         * A list of supported frame rates (only available in-session!
         */
        get supportedFrameRates(): Float32Array | undefined;
        /**
         * Set the framerate of the session.
         * @param rate the new framerate. This value needs to be in the supportedFrameRates array
         * @returns a promise that resolves once the framerate has been set
         */
        updateTargetFrameRate(rate: number): Promise<void>;
        /**
         * Run a callback in the xr render loop
         * @param callback the callback to call when in XR Frame
         * @param ignoreIfNotInSession if no session is currently running, run it first thing on the next session
         */
        runInXRFrame(callback: () => void, ignoreIfNotInSession?: boolean): void;
        /**
         * Check if fixed foveation is supported on this device
         */
        get isFixedFoveationSupported(): boolean;
        /**
         * Get the fixed foveation currently set, as specified by the webxr specs
         * If this returns null, then fixed foveation is not supported
         */
        get fixedFoveation(): Nullable<number>;
        /**
         * Set the fixed foveation to the specified value, as specified by the webxr specs
         * This value will be normalized to be between 0 and 1, 1 being max foveation, 0 being no foveation
         */
        set fixedFoveation(value: Nullable<number>);
    }


    /**
     * States of the webXR experience
     */
    export enum WebXRState {
        /**
         * Transitioning to being in XR mode
         */
        ENTERING_XR = 0,
        /**
         * Transitioning to non XR mode
         */
        EXITING_XR = 1,
        /**
         * In XR mode and presenting
         */
        IN_XR = 2,
        /**
         * Not entered XR mode
         */
        NOT_IN_XR = 3
    }
    /**
     * The state of the XR camera's tracking
     */
    export enum WebXRTrackingState {
        /**
         * No transformation received, device is not being tracked
         */
        NOT_TRACKING = 0,
        /**
         * Tracking lost - using emulated position
         */
        TRACKING_LOST = 1,
        /**
         * Transformation tracking works normally
         */
        TRACKING = 2
    }
    /**
     * Abstraction of the XR render target
     */
    export interface WebXRRenderTarget extends IDisposable {
        /**
         * xrpresent context of the canvas which can be used to display/mirror xr content
         */
        canvasContext: WebGLRenderingContext;
        /**
         * xr layer for the canvas
         */
        xrLayer: Nullable<XRWebGLLayer>;
        /**
         * Initializes a XRWebGLLayer to be used as the session's baseLayer.
         * @param xrSession xr session
         * @returns a promise that will resolve once the XR Layer has been created
         */
        initializeXRLayerAsync(xrSession: XRSession): Promise<XRWebGLLayer>;
    }


    /**
     * Wraps xr webgl layers.
     * @internal
     */
    export class WebXRWebGLLayerWrapper extends WebXRLayerWrapper {
        readonly layer: XRWebGLLayer;
        /**
         * @param layer is the layer to be wrapped.
         * @returns a new WebXRLayerWrapper wrapping the provided XRWebGLLayer.
         */
        constructor(layer: XRWebGLLayer);
    }
    /**
     * Provides render target textures and other important rendering information for a given XRWebGLLayer.
     * @internal
     */
    export class WebXRWebGLLayerRenderTargetTextureProvider extends WebXRLayerRenderTargetTextureProvider {
        readonly layerWrapper: WebXRWebGLLayerWrapper;
        protected _framebufferDimensions: {
            framebufferWidth: number;
            framebufferHeight: number;
        };
        private _rtt;
        private _framebuffer;
        private _layer;
        constructor(scene: Scene, layerWrapper: WebXRWebGLLayerWrapper);
        trySetViewportForView(viewport: Viewport, view: XRView): boolean;
        getRenderTargetTextureForEye(eye: XREye): Nullable<RenderTargetTexture>;
        getRenderTargetTextureForView(view: XRView): Nullable<RenderTargetTexture>;
    }



}


/* eslint-disable no-var */
/* eslint-disable @typescript-eslint/naming-convention */
// Mixins
interface Window {
    mozIndexedDB: IDBFactory;
    webkitIndexedDB: IDBFactory;
    msIndexedDB: IDBFactory;
    webkitURL: typeof URL;
    mozRequestAnimationFrame(callback: FrameRequestCallback): number;
    oRequestAnimationFrame(callback: FrameRequestCallback): number;
    WebGLRenderingContext: WebGLRenderingContext;
    CANNON: any;
    AudioContext: AudioContext;
    webkitAudioContext: AudioContext;
    PointerEvent: any;
    Math: Math;
    Uint8Array: Uint8ArrayConstructor;
    Float32Array: Float32ArrayConstructor;
    mozURL: typeof URL;
    msURL: typeof URL;
    DracoDecoderModule: any;
    setImmediate(handler: (...args: any[]) => void): number;
}

interface WorkerGlobalScope {
    importScripts: (...args: string[]) => void;
}

type WorkerSelf = WindowOrWorkerGlobalScope & WorkerGlobalScope;

interface HTMLCanvasElement {
    requestPointerLock(): void;
    msRequestPointerLock?(): void;
    mozRequestPointerLock?(): void;
    webkitRequestPointerLock?(): void;

    /** Track whether a record is in progress */
    isRecording: boolean;
    /** Capture Stream method defined by some browsers */
    captureStream(fps?: number): MediaStream;
}

interface CanvasRenderingContext2D {
    msImageSmoothingEnabled: boolean;
}

// Babylon Extension to enable UIEvents to work with our IUIEvents
interface UIEvent {
    inputIndex: number;
}

interface MouseEvent {
    mozMovementX: number;
    mozMovementY: number;
    webkitMovementX: number;
    webkitMovementY: number;
    msMovementX: number;
    msMovementY: number;
}

interface Navigator {
    mozGetVRDevices: (any: any) => any;
    webkitGetUserMedia(constraints: MediaStreamConstraints, successCallback: any, errorCallback: any): void;
    mozGetUserMedia(constraints: MediaStreamConstraints, successCallback: any, errorCallback: any): void;
    msGetUserMedia(constraints: MediaStreamConstraints, successCallback: any, errorCallback: any): void;

    webkitGetGamepads(): Gamepad[];
    msGetGamepads(): Gamepad[];
    webkitGamepads(): Gamepad[];
}

interface HTMLVideoElement {
    mozSrcObject: any;
}

interface Math {
    fround(x: number): number;
    imul(a: number, b: number): number;
    log2(x: number): number;
}

interface OffscreenCanvas extends EventTarget {
    width: number;
    height: number;
}

declare var OffscreenCanvas: {
    prototype: OffscreenCanvas;
    new (width: number, height: number): OffscreenCanvas;
};

// Experimental Pressure API https://wicg.github.io/compute-pressure/
type PressureSource = "cpu";

type PressureState = "nominal" | "fair" | "serious" | "critical";

type PressureFactor = "thermal" | "power-supply";

interface PressureRecord {
    source: PressureSource;
    state: PressureState;
    factors: ReadonlyArray<PressureFactor>;
    time: number;
}

interface PressureObserver {
    observe(source: PressureSource): void;
    unobserve(source: PressureSource): void;
    disconnect(): void;
    takeRecords(): Array<PressureRecord>;
}

interface PressureObserverOptions {
    sampleRate?: number;
}

type PressureUpdateCallback = (changes: Array<PressureRecord>, observer: PressureObserver) => void;

declare const PressureObserver: {
    prototype: PressureObserver;
    new (callback: PressureUpdateCallback, options?: PressureObserverOptions): PressureObserver;

    supportedSources: ReadonlyArray<PressureSource>;
};


/* eslint-disable no-var */
/* eslint-disable @typescript-eslint/naming-convention */
// Type definitions for WebGL 2 extended with Babylon specific types

interface WebGL2RenderingContext extends WebGL2RenderingContextBase {
    HALF_FLOAT_OES: number;
    RGBA16F: number;
    RGBA32F: number;
    DEPTH24_STENCIL8: number;
    COMPRESSED_SRGB8_ALPHA8_ASTC_4x4_KHR: number;
    COMPRESSED_SRGB_S3TC_DXT1_EXT: number;
    COMPRESSED_SRGB_ALPHA_S3TC_DXT1_EXT: number;
    COMPRESSED_SRGB_ALPHA_S3TC_DXT5_EXT: number;
    COMPRESSED_SRGB_ALPHA_BPTC_UNORM_EXT: number;
    COMPRESSED_SRGB8_ETC2: number;
    COMPRESSED_SRGB8_ALPHA8_ETC2_EAC: number;
    DRAW_FRAMEBUFFER: number;
    UNSIGNED_INT_24_8: number;
    MAX: number;
    MIN: number;
    SRGB: number;
    SRGB8: number;
    SRGB8_ALPHA8: number;
}

interface EXT_disjoint_timer_query {
    QUERY_COUNTER_BITS_EXT: number;
    TIME_ELAPSED_EXT: number;
    TIMESTAMP_EXT: number;
    GPU_DISJOINT_EXT: number;
    QUERY_RESULT_EXT: number;
    QUERY_RESULT_AVAILABLE_EXT: number;
    queryCounterEXT(query: WebGLQuery, target: number): void;
    createQueryEXT(): WebGLQuery;
    beginQueryEXT(target: number, query: WebGLQuery): void;
    endQueryEXT(target: number): void;
    getQueryObjectEXT(query: WebGLQuery, target: number): any;
    deleteQueryEXT(query: WebGLQuery): void;
}

interface WebGLProgram {
    __SPECTOR_rebuildProgram?:
        | ((vertexSourceCode: string, fragmentSourceCode: string, onCompiled: (program: WebGLProgram) => void, onError: (message: string) => void) => void)
        | null;
}

interface WebGLUniformLocation {
    _currentState: any;
}


/* eslint-disable @typescript-eslint/naming-convention */
interface GPUObjectBase {
    label: string | undefined;
}

interface GPUObjectDescriptorBase {
    label?: string;
}

interface GPUSupportedLimits {
    readonly maxTextureDimension1D: GPUSize32;
    readonly maxTextureDimension2D: GPUSize32;
    readonly maxTextureDimension3D: GPUSize32;
    readonly maxTextureArrayLayers: GPUSize32;
    readonly maxBindGroups: GPUSize32;
    readonly maxDynamicUniformBuffersPerPipelineLayout: GPUSize32;
    readonly maxDynamicStorageBuffersPerPipelineLayout: GPUSize32;
    readonly maxSampledTexturesPerShaderStage: GPUSize32;
    readonly maxSamplersPerShaderStage: GPUSize32;
    readonly maxStorageBuffersPerShaderStage: GPUSize32;
    readonly maxStorageTexturesPerShaderStage: GPUSize32;
    readonly maxUniformBuffersPerShaderStage: GPUSize32;
    readonly maxUniformBufferBindingSize: GPUSize64;
    readonly maxStorageBufferBindingSize: GPUSize64;
    readonly minUniformBufferOffsetAlignment: GPUSize32;
    readonly minStorageBufferOffsetAlignment: GPUSize32;
    readonly maxVertexBuffers: GPUSize32;
    readonly maxVertexAttributes: GPUSize32;
    readonly maxVertexBufferArrayStride: GPUSize32;
    readonly maxInterStageShaderComponents: GPUSize32;
    readonly maxComputeWorkgroupStorageSize: GPUSize32;
    readonly maxComputeInvocationsPerWorkgroup: GPUSize32;
    readonly maxComputeWorkgroupSizeX: GPUSize32;
    readonly maxComputeWorkgroupSizeY: GPUSize32;
    readonly maxComputeWorkgroupSizeZ: GPUSize32;
    readonly maxComputeWorkgroupsPerDimension: GPUSize32;
}

type GPUSupportedFeatures = ReadonlySet<string>;

type GPUPredefinedColorSpace = "srgb";

interface Navigator {
    readonly gpu: GPU | undefined;
}

interface WorkerNavigator {
    readonly gpu: GPU | undefined;
}

declare class GPU {
    requestAdapter(options?: GPURequestAdapterOptions): Promise<GPUAdapter | null>;
    getPreferredCanvasFormat(): GPUTextureFormat;
}

interface GPURequestAdapterOptions {
    powerPreference?: GPUPowerPreference;
    forceFallbackAdapter?: boolean /* default=false */;
}

type GPUPowerPreference = "low-power" | "high-performance";

declare class GPUAdapter {
    // https://michalzalecki.com/nominal-typing-in-typescript/#approach-1-class-with-a-private-property
    readonly name: string;
    readonly features: GPUSupportedFeatures;
    readonly limits: GPUSupportedLimits;
    readonly isFallbackAdapter: boolean;

    requestDevice(descriptor?: GPUDeviceDescriptor): Promise<GPUDevice>;
}

interface GPUDeviceDescriptor extends GPUObjectDescriptorBase {
    requiredFeatures?: GPUFeatureName[] /* default=[] */;
    requiredLimits?: { [name: string]: GPUSize64 } /* default={} */;
    defaultQueue?: GPUQueueDescriptor /* default={} */;
}

type GPUFeatureName =
    | "depth-clip-control"
    | "depth24unorm-stencil8"
    | "depth32float-stencil8"
    | "texture-compression-bc"
    | "texture-compression-etc2"
    | "texture-compression-astc"
    | "timestamp-query"
    | "indirect-first-instance"
    | "shader-f16"
    | "bgra8unorm-storage";

declare class GPUDevice extends EventTarget implements GPUObjectBase {
    label: string | undefined;

    readonly features: GPUSupportedFeatures;
    readonly limits: GPUSupportedLimits;

    readonly queue: GPUQueue;

    destroy(): void;

    createBuffer(descriptor: GPUBufferDescriptor): GPUBuffer;
    createTexture(descriptor: GPUTextureDescriptor): GPUTexture;
    createSampler(descriptor?: GPUSamplerDescriptor): GPUSampler;
    importExternalTexture(descriptor: GPUExternalTextureDescriptor): GPUExternalTexture;

    createBindGroupLayout(descriptor: GPUBindGroupLayoutDescriptor): GPUBindGroupLayout;
    createPipelineLayout(descriptor: GPUPipelineLayoutDescriptor): GPUPipelineLayout;
    createBindGroup(descriptor: GPUBindGroupDescriptor): GPUBindGroup;

    createShaderModule(descriptor: GPUShaderModuleDescriptor): GPUShaderModule;

    createComputePipeline(descriptor: GPUComputePipelineDescriptor): GPUComputePipeline;
    createRenderPipeline(descriptor: GPURenderPipelineDescriptor): GPURenderPipeline;
    createComputePipelineAsync(descriptor: GPUComputePipelineDescriptor): Promise<GPUComputePipeline>;
    createRenderPipelineAsync(descriptor: GPURenderPipelineDescriptor): Promise<GPURenderPipeline>;

    createCommandEncoder(descriptor?: GPUCommandEncoderDescriptor): GPUCommandEncoder;
    createRenderBundleEncoder(descriptor: GPURenderBundleEncoderDescriptor): GPURenderBundleEncoder;

    createQuerySet(descriptor: GPUQuerySetDescriptor): GPUQuerySet;

    readonly lost: Promise<GPUDeviceLostInfo>;
    pushErrorScope(filter: GPUErrorFilter): void;
    popErrorScope(): Promise<GPUError | undefined>;
    onuncapturederror: Event | undefined;
}

declare class GPUBuffer implements GPUObjectBase {
    label: string | undefined;

    mapAsync(mode: GPUMapModeFlags, offset?: GPUSize64 /*default=0*/, size?: GPUSize64): Promise<void>;
    getMappedRange(offset?: GPUSize64 /*default=0*/, size?: GPUSize64): ArrayBuffer;
    unmap(): void;

    destroy(): void;
}

interface GPUBufferDescriptor extends GPUObjectDescriptorBase {
    size: GPUSize64;
    usage: GPUBufferUsageFlags;
    mappedAtCreation: boolean /* default=false */;
}

type GPUBufferUsageFlags = number;

type GPUMapModeFlags = number;

declare class GPUTexture implements GPUObjectBase {
    label: string | undefined;

    createView(descriptor?: GPUTextureViewDescriptor): GPUTextureView;
    destroy(): void;
}

interface GPUTextureDescriptor extends GPUObjectDescriptorBase {
    size: GPUExtent3D;
    mipLevelCount?: GPUIntegerCoordinate /* default=1 */;
    sampleCount?: GPUSize32 /* default=1 */;
    dimension?: GPUTextureDimension /* default="2d" */;
    format: GPUTextureFormat;
    usage: GPUTextureUsageFlags;
    viewFormats?: GPUTextureFormat[] /* default=[] */;
}

type GPUTextureDimension = "1d" | "2d" | "3d";

type GPUTextureUsageFlags = number;

declare class GPUTextureView implements GPUObjectBase {
    label: string | undefined;
}

interface GPUTextureViewDescriptor extends GPUObjectDescriptorBase {
    format: GPUTextureFormat;
    dimension: GPUTextureViewDimension;
    aspect?: GPUTextureAspect /* default="all" */;
    baseMipLevel?: GPUIntegerCoordinate /* default=0 */;
    mipLevelCount: GPUIntegerCoordinate;
    baseArrayLayer?: GPUIntegerCoordinate /* default=0*/;
    arrayLayerCount: GPUIntegerCoordinate;
}

type GPUTextureViewDimension = "1d" | "2d" | "2d-array" | "cube" | "cube-array" | "3d";

type GPUTextureAspect = "all" | "stencil-only" | "depth-only";

type GPUTextureFormat =
    // 8-bit formats
    | "r8unorm"
    | "r8snorm"
    | "r8uint"
    | "r8sint"

    // 16-bit formats
    | "r16uint"
    | "r16sint"
    | "r16float"
    | "rg8unorm"
    | "rg8snorm"
    | "rg8uint"
    | "rg8sint"

    // 32-bit formats
    | "r32uint"
    | "r32sint"
    | "r32float"
    | "rg16uint"
    | "rg16sint"
    | "rg16float"
    | "rgba8unorm"
    | "rgba8unorm-srgb"
    | "rgba8snorm"
    | "rgba8uint"
    | "rgba8sint"
    | "bgra8unorm"
    | "bgra8unorm-srgb"
    // Packed 32-bit formats
    | "rgb9e5ufloat"
    | "rgb10a2unorm"
    | "rg11b10ufloat"

    // 64-bit formats
    | "rg32uint"
    | "rg32sint"
    | "rg32float"
    | "rgba16uint"
    | "rgba16sint"
    | "rgba16float"

    // 128-bit formats
    | "rgba32uint"
    | "rgba32sint"
    | "rgba32float"

    // Depth and stencil formats
    | "stencil8"
    | "depth16unorm"
    | "depth24plus"
    | "depth24plus-stencil8"
    | "depth32float"

    // "depth24unorm-stencil8" feature
    | "depth24unorm-stencil8"

    // "depth32float-stencil8" feature
    | "depth32float-stencil8"

    // BC compressed formats usable if "texture-compression-bc" is both
    // supported by the device/user agent and enabled in requestDevice.
    | "bc1-rgba-unorm"
    | "bc1-rgba-unorm-srgb"
    | "bc2-rgba-unorm"
    | "bc2-rgba-unorm-srgb"
    | "bc3-rgba-unorm"
    | "bc3-rgba-unorm-srgb"
    | "bc4-r-unorm"
    | "bc4-r-snorm"
    | "bc5-rg-unorm"
    | "bc5-rg-snorm"
    | "bc6h-rgb-ufloat"
    | "bc6h-rgb-float"
    | "bc7-rgba-unorm"
    | "bc7-rgba-unorm-srgb"

    // ETC2 compressed formats usable if "texture-compression-etc2" is both
    // supported by the device/user agent and enabled in requestDevice.
    | "etc2-rgb8unorm"
    | "etc2-rgb8unorm-srgb"
    | "etc2-rgb8a1unorm"
    | "etc2-rgb8a1unorm-srgb"
    | "etc2-rgba8unorm"
    | "etc2-rgba8unorm-srgb"
    | "eac-r11unorm"
    | "eac-r11snorm"
    | "eac-rg11unorm"
    | "eac-rg11snorm"

    // ASTC compressed formats usable if "texture-compression-astc" is both
    // supported by the device/user agent and enabled in requestDevice.
    | "astc-4x4-unorm"
    | "astc-4x4-unorm-srgb"
    | "astc-5x4-unorm"
    | "astc-5x4-unorm-srgb"
    | "astc-5x5-unorm"
    | "astc-5x5-unorm-srgb"
    | "astc-6x5-unorm"
    | "astc-6x5-unorm-srgb"
    | "astc-6x6-unorm"
    | "astc-6x6-unorm-srgb"
    | "astc-8x5-unorm"
    | "astc-8x5-unorm-srgb"
    | "astc-8x6-unorm"
    | "astc-8x6-unorm-srgb"
    | "astc-8x8-unorm"
    | "astc-8x8-unorm-srgb"
    | "astc-10x5-unorm"
    | "astc-10x5-unorm-srgb"
    | "astc-10x6-unorm"
    | "astc-10x6-unorm-srgb"
    | "astc-10x8-unorm"
    | "astc-10x8-unorm-srgb"
    | "astc-10x10-unorm"
    | "astc-10x10-unorm-srgb"
    | "astc-12x10-unorm"
    | "astc-12x10-unorm-srgb"
    | "astc-12x12-unorm"
    | "astc-12x12-unorm-srgb";

declare class GPUExternalTexture implements GPUObjectBase {
    label: string | undefined;
}

interface GPUExternalTextureDescriptor extends GPUObjectDescriptorBase {
    source: HTMLVideoElement;
    colorSpace?: GPUPredefinedColorSpace /* default="srgb" */;
}

declare class GPUSampler implements GPUObjectBase {
    label: string | undefined;
}

interface GPUSamplerDescriptor extends GPUObjectDescriptorBase {
    addressModeU?: GPUAddressMode /* default="clamp-to-edge" */;
    addressModeV?: GPUAddressMode /* default="clamp-to-edge" */;
    addressModeW?: GPUAddressMode /* default="clamp-to-edge" */;
    magFilter?: GPUFilterMode /* default="nearest" */;
    minFilter?: GPUFilterMode /* default="nearest" */;
    mipmapFilter?: GPUFilterMode /* default="nearest" */;
    lodMinClamp?: number /* default=0 */;
    lodMaxClamp?: number /* default=32 */;
    compare?: GPUCompareFunction;
    maxAnisotropy?: number /* default=1 */;
}

type GPUAddressMode = "clamp-to-edge" | "repeat" | "mirror-repeat";

type GPUFilterMode = "nearest" | "linear";

type GPUCompareFunction = "never" | "less" | "equal" | "less-equal" | "greater" | "not-equal" | "greater-equal" | "always";

declare class GPUBindGroupLayout implements GPUObjectBase {
    label: string | undefined;
}

interface GPUBindGroupLayoutDescriptor extends GPUObjectDescriptorBase {
    entries: GPUBindGroupLayoutEntry[];
}

interface GPUBindGroupLayoutEntry {
    binding: GPUIndex32;
    visibility: GPUShaderStageFlags;

    buffer?: GPUBufferBindingLayout;
    sampler?: GPUSamplerBindingLayout;
    texture?: GPUTextureBindingLayout;
    storageTexture?: GPUStorageTextureBindingLayout;
    externalTexture?: GPUExternalTextureBindingLayout;
}

type GPUShaderStageFlags = number;

type GPUBufferBindingType = "uniform" | "storage" | "read-only-storage";

interface GPUBufferBindingLayout {
    type?: GPUBufferBindingType /* default="uniform" */;
    hasDynamicOffset?: boolean /* default=false */;
    minBindingSize?: GPUSize64 /* default=0 */;
}

type GPUSamplerBindingType = "filtering" | "non-filtering" | "comparison";

interface GPUSamplerBindingLayout {
    type?: GPUSamplerBindingType /* default="filtering" */;
}

type GPUTextureSampleType = "float" | "unfilterable-float" | "depth" | "sint" | "uint";

interface GPUTextureBindingLayout {
    sampleType?: GPUTextureSampleType /* default="float" */;
    viewDimension?: GPUTextureViewDimension /* default="2d" */;
    multisampled?: boolean /* default=false */;
}

type GPUStorageTextureAccess = "write-only";

interface GPUStorageTextureBindingLayout {
    access?: GPUStorageTextureAccess /* default=write-only */;
    format: GPUTextureFormat;
    viewDimension?: GPUTextureViewDimension /* default="2d" */;
}

interface GPUExternalTextureBindingLayout {}

declare class GPUBindGroup implements GPUObjectBase {
    label: string | undefined;
}

interface GPUBindGroupDescriptor extends GPUObjectDescriptorBase {
    layout: GPUBindGroupLayout;
    entries: GPUBindGroupEntry[];
}

type GPUBindingResource = GPUSampler | GPUTextureView | GPUBufferBinding | GPUExternalTexture;

interface GPUBindGroupEntry {
    binding: GPUIndex32;
    resource: GPUBindingResource;
}

interface GPUBufferBinding {
    buffer: GPUBuffer;
    offset?: GPUSize64 /* default=0 */;
    size?: GPUSize64 /* default=size_of_buffer - offset */;
}

declare class GPUPipelineLayout implements GPUObjectBase {
    label: string | undefined;
}

interface GPUPipelineLayoutDescriptor extends GPUObjectDescriptorBase {
    bindGroupLayouts: GPUBindGroupLayout[];
}

declare class GPUShaderModule implements GPUObjectBase {
    label: string | undefined;

    compilationInfo(): Promise<GPUCompilationInfo>;
}

interface GPUShaderModuleDescriptor extends GPUObjectDescriptorBase {
    code: string | Uint32Array;
    sourceMap?: object;
    hints?: { [name: string]: GPUShaderModuleCompilationHint };
}

interface GPUShaderModuleCompilationHint {
    layout: GPUPipelineLayout | GPUAutoLayoutMode;
}

type GPUCompilationMessageType = "error" | "warning" | "info";

interface GPUCompilationMessage {
    readonly message: string;
    readonly type: GPUCompilationMessageType;
    readonly lineNum: number;
    readonly linePos: number;
    readonly offset: number;
    readonly length: number;
}

interface GPUCompilationInfo {
    readonly messages: readonly GPUCompilationMessage[];
}

type GPUAutoLayoutMode = "auto";

interface GPUPipelineDescriptorBase extends GPUObjectDescriptorBase {
    layout?: GPUPipelineLayout | GPUAutoLayoutMode;
}

interface GPUPipelineBase {
    getBindGroupLayout(index: number): GPUBindGroupLayout;
}

interface GPUProgrammableStage {
    module: GPUShaderModule;
    entryPoint: string | Uint32Array;
    constants?: { [name: string]: GPUPipelineConstantValue };
}

type GPUPipelineConstantValue = number; // May represent WGSL’s bool, f32, i32, u32, and f16 if enabled.

declare class GPUComputePipeline implements GPUObjectBase, GPUPipelineBase {
    label: string | undefined;

    getBindGroupLayout(index: number): GPUBindGroupLayout;
}

interface GPUComputePipelineDescriptor extends GPUPipelineDescriptorBase {
    compute: GPUProgrammableStage;
}

declare class GPURenderPipeline implements GPUObjectBase, GPUPipelineBase {
    label: string | undefined;

    getBindGroupLayout(index: number): GPUBindGroupLayout;
}

interface GPURenderPipelineDescriptor extends GPUPipelineDescriptorBase {
    vertex: GPUVertexState;
    primitive?: GPUPrimitiveState /* default={} */;
    depthStencil?: GPUDepthStencilState;
    multisample?: GPUMultisampleState /* default={} */;
    fragment?: GPUFragmentState;
}

interface GPUPrimitiveState {
    topology?: GPUPrimitiveTopology /* default="triangle-list" */;
    stripIndexFormat?: GPUIndexFormat;
    frontFace?: GPUFrontFace /* default="ccw" */;
    cullMode?: GPUCullMode /* default="none" */;

    // Requires "depth-clip-control" feature.
    unclippedDepth?: boolean /* default=false */;
}

type GPUPrimitiveTopology = "point-list" | "line-list" | "line-strip" | "triangle-list" | "triangle-strip";

type GPUFrontFace = "ccw" | "cw";

type GPUCullMode = "none" | "front" | "back";

interface GPUMultisampleState {
    count?: GPUSize32 /* default=1 */;
    mask?: GPUSampleMask /* default=0xFFFFFFFF */;
    alphaToCoverageEnabled?: boolean /* default=false */;
}

interface GPUFragmentState extends GPUProgrammableStage {
    targets: (GPUColorTargetState | null)[];
}

interface GPUColorTargetState {
    format: GPUTextureFormat;

    blend?: GPUBlendState;
    writeMask?: GPUColorWriteFlags /* default=0xF - GPUColorWrite.ALL */;
}

interface GPUBlendState {
    color: GPUBlendComponent;
    alpha: GPUBlendComponent;
}

type GPUColorWriteFlags = number;

interface GPUBlendComponent {
    operation?: GPUBlendOperation /* default="add" */;
    srcFactor?: GPUBlendFactor /* default="one" */;
    dstFactor?: GPUBlendFactor /* default="zero" */;
}

type GPUBlendFactor =
    | "zero"
    | "one"
    | "src"
    | "one-minus-src"
    | "src-alpha"
    | "one-minus-src-alpha"
    | "dst"
    | "one-minus-dst"
    | "dst-alpha"
    | "one-minus-dst-alpha"
    | "src-alpha-saturated"
    | "constant"
    | "one-minus-constant";

type GPUBlendOperation = "add" | "subtract" | "reverse-subtract" | "min" | "max";

interface GPUDepthStencilState {
    format: GPUTextureFormat;

    depthWriteEnabled?: boolean /* default=false */;
    depthCompare?: GPUCompareFunction /* default="always" */;

    stencilFront?: GPUStencilStateFace /* default={} */;
    stencilBack?: GPUStencilStateFace /* default={} */;

    stencilReadMask?: GPUStencilValue /* default=0xFFFFFFFF */;
    stencilWriteMask?: GPUStencilValue /* default=0xFFFFFFFF */;

    depthBias?: GPUDepthBias /* default=0 */;
    depthBiasSlopeScale?: number /* default= 0 */;
    depthBiasClamp?: number /* default=0 */;
}

interface GPUStencilStateFace {
    compare?: GPUCompareFunction /* default="always" */;
    failOp?: GPUStencilOperation /* default="keep" */;
    depthFailOp?: GPUStencilOperation /* default="keep" */;
    passOp?: GPUStencilOperation /* default="keep" */;
}

type GPUStencilOperation = "keep" | "zero" | "replace" | "invert" | "increment-clamp" | "decrement-clamp" | "increment-wrap" | "decrement-wrap";

type GPUIndexFormat = "uint16" | "uint32";

type GPUVertexFormat =
    | "uint8x2"
    | "uint8x4"
    | "sint8x2"
    | "sint8x4"
    | "unorm8x2"
    | "unorm8x4"
    | "snorm8x2"
    | "snorm8x4"
    | "uint16x2"
    | "uint16x4"
    | "sint16x2"
    | "sint16x4"
    | "unorm16x2"
    | "unorm16x4"
    | "snorm16x2"
    | "snorm16x4"
    | "float16x2"
    | "float16x4"
    | "float32"
    | "float32x2"
    | "float32x3"
    | "float32x4"
    | "uint32"
    | "uint32x2"
    | "uint32x3"
    | "uint32x4"
    | "sint32"
    | "sint32x2"
    | "sint32x3"
    | "sint32x4";

type GPUVertexStepMode = "vertex" | "instance";

interface GPUVertexState extends GPUProgrammableStage {
    buffers?: GPUVertexBufferLayout[] /* default=[] */;
}

interface GPUVertexBufferLayout {
    arrayStride: GPUSize64;
    stepMode?: GPUVertexStepMode /* default="vertex" */;
    attributes: GPUVertexAttribute[];
}

interface GPUVertexAttribute {
    format: GPUVertexFormat;
    offset: GPUSize64;
    shaderLocation: GPUIndex32;
}

declare class GPUCommandBuffer implements GPUObjectBase {
    label: string | undefined;
}

interface GPUCommandBufferDescriptor extends GPUObjectDescriptorBase {}

interface GPUCommandsMixin {}

declare class GPUCommandEncoder implements GPUObjectBase, GPUCommandsMixin, GPUDebugCommandsMixin {
    label: string | undefined;

    beginRenderPass(descriptor: GPURenderPassDescriptor): GPURenderPassEncoder;
    beginComputePass(descriptor?: GPUComputePassDescriptor): GPUComputePassEncoder;

    copyBufferToBuffer(source: GPUBuffer, sourceOffset: GPUSize64, destination: GPUBuffer, destinationOffset: GPUSize64, size: GPUSize64): void;
    copyBufferToTexture(source: GPUImageCopyBuffer, destination: GPUImageCopyTexture, copySize: GPUExtent3D): void;
    copyTextureToBuffer(source: GPUImageCopyTexture, destination: GPUImageCopyBuffer, copySize: GPUExtent3D): void;
    copyTextureToTexture(source: GPUImageCopyTexture, destination: GPUImageCopyTexture, copySize: GPUExtent3D): void;
    clearBuffer(buffer: GPUBuffer, offset?: GPUSize64 /* default=0 */, size?: GPUSize64): void;

    pushDebugGroup(groupLabel: string): void;
    popDebugGroup(): void;
    insertDebugMarker(markerLabel: string): void;

    writeTimestamp(querySet: GPUQuerySet, queryIndex: GPUSize32): void;

    resolveQuerySet(querySet: GPUQuerySet, firstQuery: GPUSize32, queryCount: GPUSize32, destination: GPUBuffer, destinationOffset: GPUSize64): void;

    finish(descriptor?: GPUCommandBufferDescriptor): GPUCommandBuffer;
}

interface GPUCommandEncoderDescriptor extends GPUObjectDescriptorBase {}

interface GPUImageDataLayout {
    offset?: GPUSize64 /* default=0 */;
    bytesPerRow: GPUSize32;
    rowsPerImage?: GPUSize32;
}

interface GPUImageCopyBuffer extends GPUImageDataLayout {
    buffer: GPUBuffer;
}

interface GPUImageCopyTexture {
    texture: GPUTexture;
    mipLevel?: GPUIntegerCoordinate /* default=0 */;
    origin?: GPUOrigin3D /* default={} */;
    aspect?: GPUTextureAspect /* default="all" */;
}

interface GPUImageCopyTextureTagged extends GPUImageCopyTexture {
    colorSpace?: GPUPredefinedColorSpace /* default="srgb" */;
    premultipliedAlpha?: boolean /* default=false */;
}

interface GPUImageCopyExternalImage {
    source: ImageBitmap | HTMLCanvasElement | OffscreenCanvas;
    origin?: GPUOrigin2D /* default={} */;
    flipY?: boolean /* default=false */;
}

interface GPUBindingCommandsMixin {
    setBindGroup(index: GPUIndex32, bindGroup: GPUBindGroup, dynamicOffsets?: GPUBufferDynamicOffset[]): void;
    setBindGroup(index: GPUIndex32, bindGroup: GPUBindGroup, dynamicOffsetData: Uint32Array, dynamicOffsetsDataStart: GPUSize64, dynamicOffsetsDataLength: GPUSize32): void;
}

interface GPUDebugCommandsMixin {
    pushDebugGroup(groupLabel: string): void;
    popDebugGroup(): void;
    insertDebugMarker(markerLabel: string): void;
}

declare class GPUComputePassEncoder implements GPUObjectBase, GPUCommandsMixin, GPUDebugCommandsMixin, GPUBindingCommandsMixin {
    label: string | undefined;

    setBindGroup(index: number, bindGroup: GPUBindGroup, dynamicOffsets?: GPUBufferDynamicOffset[]): void;
    setBindGroup(index: GPUIndex32, bindGroup: GPUBindGroup, dynamicOffsetData: Uint32Array, dynamicOffsetsDataStart: GPUSize64, dynamicOffsetsDataLength: GPUSize32): void;

    pushDebugGroup(groupLabel: string): void;
    popDebugGroup(): void;
    insertDebugMarker(markerLabel: string): void;

    setPipeline(pipeline: GPUComputePipeline): void;
    dispatchWorkgroups(workgroupCountX: GPUSize32, workgroupCountY?: GPUSize32 /* default=1 */, workgroupCountZ?: GPUSize32 /* default=1 */): void;
    dispatchWorkgroupsIndirect(indirectBuffer: GPUBuffer, indirectOffset: GPUSize64): void;

    end(): void;
}

type GPUComputePassTimestampLocation = "beginning" | "end";

interface GPUComputePassTimestampWrite {
    querySet: GPUQuerySet;
    queryIndex: GPUSize32;
    location: GPUComputePassTimestampLocation;
}

type GPUComputePassTimestampWrites = Array<GPUComputePassTimestampWrite>;

interface GPUComputePassDescriptor extends GPUObjectDescriptorBase {
    timestampWrites?: GPUComputePassTimestampWrites /* default=[] */;
}

declare class GPURenderPassEncoder implements GPUObjectBase, GPUCommandsMixin, GPUDebugCommandsMixin, GPUBindingCommandsMixin, GPURenderCommandsMixin {
    label: string | undefined;

    setBindGroup(index: GPUIndex32, bindGroup: GPUBindGroup, dynamicOffsets?: GPUBufferDynamicOffset[]): void;
    setBindGroup(index: GPUIndex32, bindGroup: GPUBindGroup, dynamicOffsetData: Uint32Array, dynamicOffsetsDataStart: GPUSize64, dynamicOffsetsDataLength: GPUSize32): void;

    pushDebugGroup(groupLabel: string): void;
    popDebugGroup(): void;
    insertDebugMarker(markerLabel: string): void;

    setPipeline(pipeline: GPURenderPipeline): void;

    setIndexBuffer(buffer: GPUBuffer, indexFormat: GPUIndexFormat, offset?: GPUSize64 /* default=0 */, size?: GPUSize64 /* default=0 */): void;
    setVertexBuffer(slot: GPUIndex32, buffer: GPUBuffer, offset?: GPUSize64 /* default=0 */, size?: GPUSize64 /* default=0 */): void;

    draw(vertexCount: GPUSize32, instanceCount?: GPUSize32 /* default=1 */, firstVertex?: GPUSize32 /* default=0 */, firstInstance?: GPUSize32 /* default=0 */): void;
    drawIndexed(
        indexCount: GPUSize32,
        instanceCount?: GPUSize32 /* default=1 */,
        firstIndex?: GPUSize32 /* default=0 */,
        baseVertex?: GPUSignedOffset32 /* default=0 */,
        firstInstance?: GPUSize32 /* default=0 */
    ): void;

    drawIndirect(indirectBuffer: GPUBuffer, indirectOffset: GPUSize64): void;
    drawIndexedIndirect(indirectBuffer: GPUBuffer, indirectOffset: GPUSize64): void;

    setViewport(x: number, y: number, width: number, height: number, minDepth: number, maxDepth: number): void;

    setScissorRect(x: GPUIntegerCoordinate, y: GPUIntegerCoordinate, width: GPUIntegerCoordinate, height: GPUIntegerCoordinate): void;

    setBlendConstant(color: GPUColor): void;
    setStencilReference(reference: GPUStencilValue): void;

    beginOcclusionQuery(queryIndex: GPUSize32): void;
    endOcclusionQuery(): void;

    executeBundles(bundles: GPURenderBundle[]): void;
    end(): void;
}

type GPURenderPassTimestampLocation = "beginning" | "end";

interface GPURenderPassTimestampWrite {
    querySet: GPUQuerySet;
    queryIndex: GPUSize32;
    location: GPURenderPassTimestampLocation;
}

type GPURenderPassTimestampWrites = Array<GPURenderPassTimestampWrite>;

interface GPURenderPassDescriptor extends GPUObjectDescriptorBase {
    colorAttachments: (GPURenderPassColorAttachment | null)[];
    depthStencilAttachment?: GPURenderPassDepthStencilAttachment;
    occlusionQuerySet?: GPUQuerySet;
    timestampWrites?: GPURenderPassTimestampWrites /* default=[] */;
}

interface GPURenderPassColorAttachment {
    view: GPUTextureView;
    resolveTarget?: GPUTextureView;

    clearValue?: GPUColor;
    loadOp: GPULoadOp;
    storeOp: GPUStoreOp;
}

interface GPURenderPassDepthStencilAttachment {
    view: GPUTextureView;

    depthClearValue?: number /* default=0 */;
    depthLoadOp: GPULoadOp;
    depthStoreOp: GPUStoreOp;
    depthReadOnly?: boolean /* default=false */;

    stencilClearValue?: GPUStencilValue /* default=0 */;
    stencilLoadOp?: GPULoadOp;
    stencilStoreOp?: GPUStoreOp;
    stencilReadOnly?: boolean /* default=false */;
}

type GPULoadOp = "load" | "clear";

type GPUStoreOp = "store" | "discard";

interface GPURenderPassLayout extends GPUObjectDescriptorBase {
    colorFormats: (GPUTextureFormat | null)[];
    depthStencilFormat?: GPUTextureFormat;
    sampleCount?: GPUSize32 /* default=1 */;
}

interface GPURenderCommandsMixin {
    setPipeline(pipeline: GPURenderPipeline): void;

    setIndexBuffer(buffer: GPUBuffer, indexFormat: GPUIndexFormat, offset?: GPUSize64 /* default=0 */, size?: GPUSize64 /* default=0 */): void;
    setVertexBuffer(slot: GPUIndex32, buffer: GPUBuffer, offset?: GPUSize64 /* default=0 */, size?: GPUSize64 /* default=0 */): void;

    draw(vertexCount: GPUSize32, instanceCount?: GPUSize32 /* default=1 */, firstVertex?: GPUSize32 /* default=0 */, firstInstance?: GPUSize32 /* default=0 */): void;
    drawIndexed(
        indexCount: GPUSize32,
        instanceCount?: GPUSize32 /* default=1 */,
        firstIndex?: GPUSize32 /* default=0 */,
        baseVertex?: GPUSignedOffset32 /* default=0 */,
        firstInstance?: GPUSize32 /* default=0 */
    ): void;

    drawIndirect(indirectBuffer: GPUBuffer, indirectOffset: GPUSize64): void;
    drawIndexedIndirect(indirectBuffer: GPUBuffer, indirectOffset: GPUSize64): void;
}

declare class GPURenderBundle implements GPUObjectBase {
    label: string | undefined;
}

interface GPURenderBundleDescriptor extends GPUObjectDescriptorBase {}

declare class GPURenderBundleEncoder implements GPUObjectBase, GPUCommandsMixin, GPUDebugCommandsMixin, GPUBindingCommandsMixin, GPURenderCommandsMixin {
    label: string | undefined;

    setBindGroup(index: GPUIndex32, bindGroup: GPUBindGroup, dynamicOffsets?: GPUBufferDynamicOffset[]): void;
    setBindGroup(index: GPUIndex32, bindGroup: GPUBindGroup, dynamicOffsetData: Uint32Array, dynamicOffsetsDataStart: GPUSize64, dynamicOffsetsDataLength: GPUSize32): void;

    pushDebugGroup(groupLabel: string): void;
    popDebugGroup(): void;
    insertDebugMarker(markerLabel: string): void;

    setPipeline(pipeline: GPURenderPipeline): void;

    setIndexBuffer(buffer: GPUBuffer, indexFormat: GPUIndexFormat, offset?: GPUSize64 /* default=0 */, size?: GPUSize64 /* default=0 */): void;
    setVertexBuffer(slot: GPUIndex32, buffer: GPUBuffer, offset?: GPUSize64 /* default=0 */, size?: GPUSize64 /* default=0 */): void;

    draw(vertexCount: GPUSize32, instanceCount?: GPUSize32 /* default=1 */, firstVertex?: GPUSize32 /* default=0 */, firstInstance?: GPUSize32 /* default=0 */): void;
    drawIndexed(
        indexCount: GPUSize32,
        instanceCount?: GPUSize32 /* default=1 */,
        firstIndex?: GPUSize32 /* default=0 */,
        baseVertex?: GPUSignedOffset32 /* default=0 */,
        firstInstance?: GPUSize32 /* default=0 */
    ): void;

    drawIndirect(indirectBuffer: GPUBuffer, indirectOffset: GPUSize64): void;
    drawIndexedIndirect(indirectBuffer: GPUBuffer, indirectOffset: GPUSize64): void;

    finish(descriptor?: GPURenderBundleDescriptor): GPURenderBundle;
}

interface GPURenderBundleEncoderDescriptor extends GPURenderPassLayout {
    depthReadOnly?: boolean /* default=false */;
    stencilReadOnly?: boolean /* default=false */;
}

interface GPUQueueDescriptor extends GPUObjectDescriptorBase {}

declare class GPUQueue implements GPUObjectBase {
    label: string | undefined;

    submit(commandBuffers: GPUCommandBuffer[]): void;

    onSubmittedWorkDone(): Promise<void>;

    writeBuffer(buffer: GPUBuffer, bufferOffset: GPUSize64, data: BufferSource, dataOffset?: GPUSize64 /* default=0 */, size?: GPUSize64): void;

    writeTexture(destination: GPUImageCopyTexture, data: BufferSource, dataLayout: GPUImageDataLayout, size: GPUExtent3D): void;

    copyExternalImageToTexture(source: GPUImageCopyExternalImage, destination: GPUImageCopyTextureTagged, copySize: GPUExtent3D): void;
}

declare class GPUQuerySet implements GPUObjectBase {
    label: string | undefined;

    destroy(): void;
}

interface GPUQuerySetDescriptor extends GPUObjectDescriptorBase {
    type: GPUQueryType;
    count: GPUSize32;
}

type GPUQueryType = "occlusion" | "timestamp";

declare class GPUCanvasContext {
    readonly canvas: HTMLCanvasElement | OffscreenCanvas;

    configure(configuration?: GPUCanvasConfiguration): void;
    unconfigure(): void;

    getCurrentTexture(): GPUTexture;
}

type GPUCanvasAlphaMode = "opaque" | "premultiplied";

interface GPUCanvasConfiguration extends GPUObjectDescriptorBase {
    device: GPUDevice;
    format: GPUTextureFormat;
    usage?: GPUTextureUsageFlags /* default=0x10 - GPUTextureUsage.RENDER_ATTACHMENT */;
    viewFormats?: GPUTextureFormat[] /* default=[] */;
    colorSpace?: GPUPredefinedColorSpace /* default="srgb" */;
    alphaMode?: GPUCanvasAlphaMode /* default="opaque" */;
}

type GPUDeviceLostReason = "destroyed";

declare class GPUDeviceLostInfo {
    readonly reason?: GPUDeviceLostReason;
    readonly message: string;
}

type GPUErrorFilter = "out-of-memory" | "validation";

declare class GPUError {
    readonly message: string;
}

declare class GPUOutOfMemoryError extends GPUError {
    constructor();
    readonly message: string;
}

declare class GPUValidationError extends GPUError {
    constructor(message: string);
    readonly message: string;
}

declare class GPUUncapturedErrorEvent extends Event {
    constructor(type: string, gpuUncapturedErrorEventInitDict: GPUUncapturedErrorEventInit);
    readonly error: GPUError;
}

interface GPUUncapturedErrorEventInit extends EventInit {
    error: GPUError;
}

type GPUBufferDynamicOffset = number; /* unsigned long */
type GPUStencilValue = number; /* unsigned long */
type GPUSampleMask = number; /* unsigned long */
type GPUDepthBias = number; /* long */
type GPUSize64 = number; /* unsigned long long */
type GPUIntegerCoordinate = number; /* unsigned long */
type GPUIndex32 = number; /* unsigned long */
type GPUSize32 = number; /* unsigned long */
type GPUSignedOffset32 = number; /* long */

interface GPUColorDict {
    r: number;
    g: number;
    b: number;
    a: number;
}
type GPUColor = [number, number, number, number] | GPUColorDict;

interface GPUOrigin2DDict {
    x?: GPUIntegerCoordinate /* default=0 */;
    y?: GPUIntegerCoordinate /* default=0 */;
}
type GPUOrigin2D = [GPUIntegerCoordinate, GPUIntegerCoordinate] | GPUOrigin2DDict;

interface GPUOrigin3DDict {
    x?: GPUIntegerCoordinate /* default=0 */;
    y?: GPUIntegerCoordinate /* default=0 */;
    z?: GPUIntegerCoordinate /* default=0 */;
}
type GPUOrigin3D = [GPUIntegerCoordinate, GPUIntegerCoordinate, GPUIntegerCoordinate] | GPUOrigin3DDict;

interface GPUExtent3DDict {
    width: GPUIntegerCoordinate;
    height?: GPUIntegerCoordinate /* default=1 */;
    depthOrArrayLayers?: GPUIntegerCoordinate /* default=1 */;
}
type GPUExtent3D = [GPUIntegerCoordinate, GPUIntegerCoordinate, GPUIntegerCoordinate] | GPUExtent3DDict;


/* eslint-disable @typescript-eslint/naming-convention */
/* eslint-disable no-var */
// Type definitions for WebVR API
// Project: https://w3c.github.io/webvr/
// Definitions by: six a <https://github.com/lostfictions>
// Definitions: https://github.com/DefinitelyTyped/DefinitelyTyped

interface VRDisplay extends EventTarget {
    /**
     * Dictionary of capabilities describing the VRDisplay.
     */
    readonly capabilities: VRDisplayCapabilities;

    /**
     * z-depth defining the far plane of the eye view frustum
     * enables mapping of values in the render target depth
     * attachment to scene coordinates. Initially set to 10000.0.
     */
    depthFar: number;

    /**
     * z-depth defining the near plane of the eye view frustum
     * enables mapping of values in the render target depth
     * attachment to scene coordinates. Initially set to 0.01.
     */
    depthNear: number;

    /**
     * An identifier for this distinct VRDisplay. Used as an
     * association point in the Gamepad API.
     */
    readonly displayId: number;

    /**
     * A display name, a user-readable name identifying it.
     */
    readonly displayName: string;
    readonly isConnected: boolean;
    readonly isPresenting: boolean;

    /**
     * If this VRDisplay supports room-scale experiences, the optional
     * stage attribute contains details on the room-scale parameters.
     */
    readonly stageParameters: VRStageParameters | null;

    /**
     * Passing the value returned by `requestAnimationFrame` to
     * `cancelAnimationFrame` will unregister the callback.
     * @param handle Define the handle of the request to cancel
     */
    cancelAnimationFrame(handle: number): void;

    /**
     * Stops presenting to the VRDisplay.
     * @returns a promise to know when it stopped
     */
    exitPresent(): Promise<void>;

    /**
     * Return the current VREyeParameters for the given eye.
     * @param whichEye Define the eye we want the parameter for
     * @returns the eye parameters
     */
    getEyeParameters(whichEye: string): VREyeParameters;

    /**
     * Populates the passed VRFrameData with the information required to render
     * the current frame.
     * @param frameData Define the data structure to populate
     * @returns true if ok otherwise false
     */
    getFrameData(frameData: VRFrameData): boolean;

    /**
     * Get the layers currently being presented.
     * @returns the list of VR layers
     */
    getLayers(): VRLayer[];

    /**
     * Return a VRPose containing the future predicted pose of the VRDisplay
     * when the current frame will be presented. The value returned will not
     * change until JavaScript has returned control to the browser.
     *
     * The VRPose will contain the position, orientation, velocity,
     * and acceleration of each of these properties.
     * @returns the pose object
     */
    getPose(): VRPose;

    /**
     * Return the current instantaneous pose of the VRDisplay, with no
     * prediction applied.
     * @returns the current instantaneous pose
     */
    getImmediatePose(): VRPose;

    /**
     * The callback passed to `requestAnimationFrame` will be called
     * any time a new frame should be rendered. When the VRDisplay is
     * presenting the callback will be called at the native refresh
     * rate of the HMD. When not presenting this function acts
     * identically to how window.requestAnimationFrame acts. Content should
     * make no assumptions of frame rate or vsync behavior as the HMD runs
     * asynchronously from other displays and at differing refresh rates.
     * @param callback Define the action to run next frame
     * @returns the request handle it
     */
    requestAnimationFrame(callback: FrameRequestCallback): number;

    /**
     * Begin presenting to the VRDisplay. Must be called in response to a user gesture.
     * Repeat calls while already presenting will update the VRLayers being displayed.
     * @param layers Define the list of layer to present
     * @returns a promise to know when the request has been fulfilled
     */
    requestPresent(layers: VRLayer[]): Promise<void>;

    /**
     * Reset the pose for this display, treating its current position and
     * orientation as the "origin/zero" values. VRPose.position,
     * VRPose.orientation, and VRStageParameters.sittingToStandingTransform may be
     * updated when calling resetPose(). This should be called in only
     * sitting-space experiences.
     */
    resetPose(): void;

    /**
     * The VRLayer provided to the VRDisplay will be captured and presented
     * in the HMD. Calling this function has the same effect on the source
     * canvas as any other operation that uses its source image, and canvases
     * created without preserveDrawingBuffer set to true will be cleared.
     * @param pose Define the pose to submit
     */
    submitFrame(pose?: VRPose): void;
}

declare var VRDisplay: {
    prototype: VRDisplay;
    new (): VRDisplay;
};

interface VRLayer {
    leftBounds?: number[] | Float32Array | null;
    rightBounds?: number[] | Float32Array | null;
    source?: HTMLCanvasElement | null;
}

interface VRDisplayCapabilities {
    readonly canPresent: boolean;
    readonly hasExternalDisplay: boolean;
    readonly hasOrientation: boolean;
    readonly hasPosition: boolean;
    readonly maxLayers: number;
}

interface VREyeParameters {
    /** @deprecated */
    readonly fieldOfView: VRFieldOfView;
    readonly offset: Float32Array;
    readonly renderHeight: number;
    readonly renderWidth: number;
}

interface VRFieldOfView {
    readonly downDegrees: number;
    readonly leftDegrees: number;
    readonly rightDegrees: number;
    readonly upDegrees: number;
}

interface VRFrameData {
    readonly leftProjectionMatrix: Float32Array;
    readonly leftViewMatrix: Float32Array;
    readonly pose: VRPose;
    readonly rightProjectionMatrix: Float32Array;
    readonly rightViewMatrix: Float32Array;
    readonly timestamp: number;
}

interface VRPose {
    readonly angularAcceleration: Float32Array | null;
    readonly angularVelocity: Float32Array | null;
    readonly linearAcceleration: Float32Array | null;
    readonly linearVelocity: Float32Array | null;
    readonly orientation: Float32Array | null;
    readonly position: Float32Array | null;
    readonly timestamp: number;
}

interface VRStageParameters {
    sittingToStandingTransform?: Float32Array;
    sizeX?: number;
    sizeY?: number;
}

interface Navigator {
    getVRDisplays(): Promise<VRDisplay[]>;
    readonly activeVRDisplays: ReadonlyArray<VRDisplay>;
}

interface Window {
    onvrdisplayconnected: ((this: Window, ev: Event) => any) | null;
    onvrdisplaydisconnected: ((this: Window, ev: Event) => any) | null;
    onvrdisplaypresentchange: ((this: Window, ev: Event) => any) | null;
    addEventListener(type: "vrdisplayconnected", listener: (ev: Event) => any, useCapture?: boolean): void;
    addEventListener(type: "vrdisplaydisconnected", listener: (ev: Event) => any, useCapture?: boolean): void;
    addEventListener(type: "vrdisplaypresentchange", listener: (ev: Event) => any, useCapture?: boolean): void;
}

interface Gamepad {
    readonly displayId: number;
}

declare var VRFrameData: any;


// Type definitions for non-npm package webxr 0.5
// Project: https://www.w3.org/TR/webxr/
// Definitions by: Rob Rohan <https://github.com/robrohan>
//                 Raanan Weber <https://github.com/RaananW>
//                 Sean T. McBeth <https://github.com/capnmidnight>
// Definitions: https://github.com/DefinitelyTyped/DefinitelyTyped
// Minimum TypeScript Version: 3.7

// Most of this was hand written and... more or less copied from the following
// sites:
//  https://www.w3.org/TR/webxr/
//  https://developer.mozilla.org/en-US/docs/Web/API/WebXR_Device_API
//  https://www.w3.org/immersive-web/
//  https://github.com/immersive-web
//

/**
 * ref: https://immersive-web.github.io/webxr/#navigator-xr-attribute
 */
interface Navigator {
    /**
     * An XRSystem object is the entry point to the API, used to query for XR features
     * available to the user agent and initiate communication with XR hardware via the
     * creation of XRSessions.
     */
    xr?: XRSystem | undefined;
}

/**
 * WebGL Context Compatability
 *
 * ref: https://immersive-web.github.io/webxr/#contextcompatibility
 */
interface WebGLContextAttributes {
    xrCompatible?: boolean | undefined;
}

interface WebGLRenderingContextBase {
    makeXRCompatible(): Promise<void>;
}

/**
 * Available session modes
 *
 * ref: https://immersive-web.github.io/webxr/#xrsessionmode-enum
 */
type XRSessionMode = "inline" | "immersive-vr" | "immersive-ar";

/**
 * Reference space types
 */
type XRReferenceSpaceType = "viewer" | "local" | "local-floor" | "bounded-floor" | "unbounded";

type XREnvironmentBlendMode = "opaque" | "additive" | "alpha-blend";

/**
 * ref: https://immersive-web.github.io/webxr/#xrsession-interface
 */
type XRVisibilityState = "visible" | "visible-blurred" | "hidden";

/**
 * Handedness types
 */
type XRHandedness = "none" | "left" | "right";

/**
 * InputSource target ray modes
 */
type XRTargetRayMode = "gaze" | "tracked-pointer" | "screen";

/**
 * Eye types
 */
type XREye = "none" | "left" | "right";

type XRFrameRequestCallback = (time: DOMHighResTimeStamp, frame: XRFrame) => void;

interface XRSystemDeviceChangeEvent extends Event {
    type: "devicechange";
}

interface XRSystemDeviceChangeEventHandler {
    (event: XRSystemDeviceChangeEvent): any;
}

interface XRSystemEventMap {
    devicechange: XRSystemDeviceChangeEvent;
}

/**
 * An XRSystem object is the entry point to the API, used to query for XR features available
 * to the user agent and initiate communication with XR hardware via the creation of
 * XRSessions.
 *
 * ref: https://immersive-web.github.io/webxr/#xrsystem-interface
 */
interface XRSystem extends EventTarget {
    /**
     * Attempts to initialize an XRSession for the given mode if possible, entering immersive
     * mode if necessary.
     * @param mode
     * @param options
     */
    requestSession(mode: XRSessionMode, options?: XRSessionInit): Promise<XRSession>;

    /**
     * Queries if a given mode may be supported by the user agent and device capabilities.
     * @param mode
     */
    isSessionSupported(mode: XRSessionMode): Promise<boolean>;

    ondevicechange: XRSystemDeviceChangeEventHandler | null;

    addEventListener<K extends keyof XRSystemEventMap>(type: K, listener: (this: XRSystem, ev: XRSystemEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;
    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;
    removeEventListener<K extends keyof XRSystemEventMap>(type: K, listener: (this: XRSystem, ev: XRSystemEventMap[K]) => any, options?: boolean | EventListenerOptions): void;
    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;
}

declare abstract class XRSystem implements XRSystem {}

/**
 * Describes a viewport, or rectangular region, of a graphics surface.
 *
 * ref: https://immersive-web.github.io/webxr/#xrviewport-interface
 */
interface XRViewport {
    readonly x: number;
    readonly y: number;
    readonly width: number;
    readonly height: number;
}

declare abstract class XRViewport implements XRViewport {}

/**
 * Represents a virtual coordinate system with an origin that corresponds to a physical location.
 * Spatial data that is requested from the API or given to the API is always expressed in relation
 * to a specific XRSpace at the time of a specific XRFrame. Numeric values such as pose positions
 * are coordinates in that space relative to its origin. The interface is intentionally opaque.
 *
 * ref: https://immersive-web.github.io/webxr/#xrspace-interface
 */
// tslint:disable-next-line no-empty-interface
interface XRSpace extends EventTarget {}

declare abstract class XRSpace implements XRSpace {}

interface XRRenderStateInit {
    baseLayer?: XRWebGLLayer | undefined;
    depthFar?: number | undefined;
    depthNear?: number | undefined;
    inlineVerticalFieldOfView?: number | undefined;
}

interface XRRenderState {
    readonly baseLayer?: XRWebGLLayer | undefined;
    readonly depthFar: number;
    readonly depthNear: number;
    readonly inlineVerticalFieldOfView?: number | undefined;
}

declare abstract class XRRenderState implements XRRenderState {}

interface XRReferenceSpaceEventInit extends EventInit {
    referenceSpace?: XRReferenceSpace | undefined;
    transform?: XRRigidTransform | undefined;
}

/**
 * XRReferenceSpaceEvents are fired to indicate changes to the state of an XRReferenceSpace.
 *
 * ref: https://immersive-web.github.io/webxr/#xrreferencespaceevent-interface
 */
interface XRReferenceSpaceEvent extends Event {
    readonly type: "reset";
    readonly referenceSpace: XRReferenceSpace;
    readonly transform?: XRRigidTransform | undefined;
}

// tslint:disable-next-line no-unnecessary-class
declare class XRReferenceSpaceEvent implements XRReferenceSpaceEvent {
    constructor(type: "reset", eventInitDict?: XRReferenceSpaceEventInit);
}

interface XRReferenceSpaceEventHandler {
    (event: XRReferenceSpaceEvent): any;
}

interface XRReferenceSpaceEventMap {
    reset: XRReferenceSpaceEvent;
}

/**
 * One of several common XRSpaces that applications can use to establish a spatial relationship
 * with the user's physical environment.
 *
 * ref: https://immersive-web.github.io/webxr/#xrreferencespace-interface
 */
interface XRReferenceSpace extends XRSpace {
    getOffsetReferenceSpace(originOffset: XRRigidTransform): XRReferenceSpace;
    onreset: XRReferenceSpaceEventHandler;

    addEventListener<K extends keyof XRReferenceSpaceEventMap>(
        type: K,
        listener: (this: XRReferenceSpace, ev: XRReferenceSpaceEventMap[K]) => any,
        options?: boolean | AddEventListenerOptions
    ): void;
    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;
    removeEventListener<K extends keyof XRReferenceSpaceEventMap>(
        type: K,
        listener: (this: XRReferenceSpace, ev: XRReferenceSpaceEventMap[K]) => any,
        options?: boolean | EventListenerOptions
    ): void;
    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;
}

declare abstract class XRReferenceSpace implements XRReferenceSpace {}

/**
 * Extends XRReferenceSpace to include boundsGeometry, indicating the pre-configured boundaries
 * of the user's space.
 *
 * ref: https://immersive-web.github.io/webxr/#xrboundedreferencespace-interface
 */
interface XRBoundedReferenceSpace extends XRReferenceSpace {
    readonly boundsGeometry: DOMPointReadOnly[];
}

declare abstract class XRBoundedReferenceSpace implements XRBoundedReferenceSpace {}

/**
 * Represents an XR input source, which is any input mechanism which allows the user to perform
 * targeted actions in the same virtual space as the viewer. Example XR input sources include,
 * but are not limited to, handheld controllers, optically tracked hands, and gaze-based input
 * methods that operate on the viewer's pose. Input mechanisms which are not explicitly associated
 * with the XR device, such as traditional gamepads, mice, or keyboards SHOULD NOT be considered
 * XR input sources.
 * ref: https://immersive-web.github.io/webxr/#xrinputsource-interface
 */
interface XRInputSource {
    readonly handedness: XRHandedness;
    readonly targetRayMode: XRTargetRayMode;
    readonly targetRaySpace: XRSpace;
    readonly gripSpace?: XRSpace | undefined;
    readonly gamepad?: Gamepad | undefined;
    readonly profiles: string[];
    readonly hand?: XRHand;
}

declare abstract class XRInputSource implements XRInputSource {}

/**
 * Represents a list of XRInputSources. It is used in favor of a frozen array type when the contents
 * of the list are expected to change over time, such as with the XRSession inputSources attribute.
 * ref: https://immersive-web.github.io/webxr/#xrinputsourcearray-interface
 */
interface XRInputSourceArray {
    [Symbol.iterator](): IterableIterator<XRInputSource>;
    [n: number]: XRInputSource;

    length: number;

    entries(): IterableIterator<[number, XRInputSource]>;
    keys(): IterableIterator<number>;
    values(): IterableIterator<XRInputSource>;

    forEach(callbackfn: (value: XRInputSource, index: number, array: XRInputSource[]) => void, thisArg?: any): void;
}

declare abstract class XRInputSourceArray implements XRInputSourceArray {}

/**
 * Describes a position and orientation in space relative to an XRSpace.
 *
 * ref: https://immersive-web.github.io/webxr/#xrpose-interface
 */
interface XRPose {
    readonly transform: XRRigidTransform;
    readonly emulatedPosition: boolean;
}

declare abstract class XRPose implements XRPose {}

/**
 * Represents a snapshot of the state of all of the tracked objects for an XRSession. Applications
 * can acquire an XRFrame by calling requestAnimationFrame() on an XRSession with an
 * XRFrameRequestCallback. When the callback is called it will be passed an XRFrame.
 * Events which need to communicate tracking state, such as the select event, will also provide an
 * XRFrame.
 *
 * ref: https://immersive-web.github.io/webxr/#xrframe-interface
 */
interface XRFrame {
    readonly session: XRSession;
    // BABYLON CHANGE - switched to optional
    readonly predictedDisplayTime?: DOMHighResTimeStamp;

    /**
     * Provides the pose of space relative to baseSpace as an XRPose, at the time represented by
     * the XRFrame.
     *
     * @param space
     * @param baseSpace
     */
    getPose(space: XRSpace, baseSpace: XRSpace): XRPose | undefined;

    /**
     * Provides the pose of the viewer relative to referenceSpace as an XRViewerPose, at the
     * XRFrame's time.
     *
     * @param referenceSpace
     */
    getViewerPose(referenceSpace: XRReferenceSpace): XRViewerPose | undefined;
}

declare abstract class XRFrame implements XRFrame {}

/**
 * Type of XR events available
 */
type XRInputSourceEventType = "select" | "selectend" | "selectstart" | "squeeze" | "squeezeend" | "squeezestart";

interface XRInputSourceEventInit extends EventInit {
    frame?: XRFrame | undefined;
    inputSource?: XRInputSource | undefined;
}

/**
 * XRInputSourceEvents are fired to indicate changes to the state of an XRInputSource.
 * ref: https://immersive-web.github.io/webxr/#xrinputsourceevent-interface
 */
declare class XRInputSourceEvent extends Event {
    readonly type: XRInputSourceEventType;
    readonly frame: XRFrame;
    readonly inputSource: XRInputSource;

    constructor(type: XRInputSourceEventType, eventInitDict?: XRInputSourceEventInit);
}

interface XRInputSourceEventHandler {
    (evt: XRInputSourceEvent): any;
}

type XRSessionEventType = "end" | "visibilitychange" | "frameratechange";

interface XRSessionEventInit extends EventInit {
    session: XRSession;
}

/**
 * XRSessionEvents are fired to indicate changes to the state of an XRSession.
 * ref: https://immersive-web.github.io/webxr/#xrsessionevent-interface
 */
declare class XRSessionEvent extends Event {
    readonly session: XRSession;
    constructor(type: XRSessionEventType, eventInitDict?: XRSessionEventInit);
}

interface XRSessionEventHandler {
    (evt: XRSessionEvent): any;
}

/**
 * ref: https://immersive-web.github.io/webxr/#feature-dependencies
 */
interface XRSessionInit {
    optionalFeatures?: string[] | undefined;
    requiredFeatures?: string[] | undefined;
}

interface XRSessionEventMap {
    inputsourceschange: XRInputSourceChangeEvent;
    end: XRSessionEvent;
    visibilitychange: XRSessionEvent;
    frameratechange: XRSessionEvent;
    select: XRInputSourceEvent;
    selectstart: XRInputSourceEvent;
    selectend: XRInputSourceEvent;
    squeeze: XRInputSourceEvent;
    squeezestart: XRInputSourceEvent;
    squeezeend: XRInputSourceEvent;
    eyetrackingstart: XREyeTrackingSourceEvent;
    eyetrackingend: XREyeTrackingSourceEvent;
}

/**
 * Any interaction with XR hardware is done via an XRSession object, which can only be
 * retrieved by calling requestSession() on the XRSystem object. Once a session has been
 * successfully acquired, it can be used to poll the viewer pose, query information about
 * the user's environment, and present imagery to the user.
 *
 * ref: https://immersive-web.github.io/webxr/#xrsession-interface
 */
interface XRSession extends EventTarget {
    /**
     * Returns a list of this session's XRInputSources, each representing an input device
     * used to control the camera and/or scene.
     */
    readonly inputSources: XRInputSourceArray;
    /**
     * object which contains options affecting how the imagery is rendered.
     * This includes things such as the near and far clipping planes
     */
    readonly renderState: XRRenderState;
    readonly environmentBlendMode: XREnvironmentBlendMode;
    readonly visibilityState: XRVisibilityState;
    readonly frameRate?: number | undefined;
    readonly supportedFrameRates?: Float32Array | undefined;

    /**
     * Removes a callback from the animation frame painting callback from
     * XRSession's set of animation frame rendering callbacks, given the
     * identifying handle returned by a previous call to requestAnimationFrame().
     */
    cancelAnimationFrame(id: number): void;

    /**
     * Ends the WebXR session. Returns a promise which resolves when the
     * session has been shut down.
     */
    end(): Promise<void>;

    /**
     * Schedules the specified method to be called the next time the user agent
     * is working on rendering an animation frame for the WebXR device. Returns an
     * integer value which can be used to identify the request for the purposes of
     * canceling the callback using cancelAnimationFrame(). This method is comparable
     * to the Window.requestAnimationFrame() method.
     */
    requestAnimationFrame(callback: XRFrameRequestCallback): number;

    /**
     * Requests that a new XRReferenceSpace of the specified type be created.
     * Returns a promise which resolves with the XRReferenceSpace or
     * XRBoundedReferenceSpace which was requested, or throws a NotSupportedError if
     * the requested space type isn't supported by the device.
     */
    requestReferenceSpace(type: XRReferenceSpaceType): Promise<XRReferenceSpace | XRBoundedReferenceSpace>;

    updateRenderState(renderStateInit?: XRRenderStateInit): Promise<void>;

    updateTargetFrameRate(rate: number): Promise<void>;

    onend: XRSessionEventHandler;
    oninputsourceschange: XRInputSourceChangeEventHandler;
    onselect: XRInputSourceEventHandler;
    onselectstart: XRInputSourceEventHandler;
    onselectend: XRInputSourceEventHandler;
    onsqueeze: XRInputSourceEventHandler;
    onsqueezestart: XRInputSourceEventHandler;
    onsqueezeend: XRInputSourceEventHandler;
    onvisibilitychange: XRSessionEventHandler;
    onframeratechange: XRSessionEventHandler;

    addEventListener<K extends keyof XRSessionEventMap>(type: K, listener: (this: XRSession, ev: XRSessionEventMap[K]) => any, options?: boolean | AddEventListenerOptions): void;
    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;
    removeEventListener<K extends keyof XRSessionEventMap>(type: K, listener: (this: XRSession, ev: XRSessionEventMap[K]) => any, options?: boolean | EventListenerOptions): void;
    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;
}

declare abstract class XRSession implements XRSession {}

/**
 * An XRPose describing the state of a viewer of the XR scene as tracked by the XR
 * device. A viewer may represent a tracked piece of hardware, the observed position
 * of a user's head relative to the hardware, or some other means of computing a series
 * of viewpoints into the XR scene. XRViewerPoses can only be queried relative to an
 * XRReferenceSpace. It provides, in addition to the XRPose values, an array of views
 * which include rigid transforms to indicate the viewpoint and projection matrices.
 * These values should be used by the application when rendering a frame of an XR scene.
 *
 * ref: https://immersive-web.github.io/webxr/#xrviewerpose-interface
 */
interface XRViewerPose extends XRPose {
    readonly views: ReadonlyArray<XRView>;
}

declare abstract class XRViewerPose implements XRViewerPose {}

/**
 * A transform described by a position and orientation. When interpreting an
 * XRRigidTransform the orientation is always applied prior to the position.
 *
 * ref: https://immersive-web.github.io/webxr/#xrrigidtransform-interface
 */
declare class XRRigidTransform {
    readonly position: DOMPointReadOnly;
    readonly orientation: DOMPointReadOnly;
    readonly matrix: Float32Array;
    readonly inverse: XRRigidTransform;

    constructor(position?: DOMPointInit, direction?: DOMPointInit);
}

/**
 * Describes a single view into an XR scene for a given frame.
 *
 * ref: https://immersive-web.github.io/webxr/#xrview-interface
 */
interface XRView {
    readonly eye: XREye;
    readonly projectionMatrix: Float32Array;
    readonly transform: XRRigidTransform;
    readonly recommendedViewportScale?: number | undefined;
    requestViewportScale(scale: number): void;
}

declare abstract class XRView implements XRView {}

/**
 * XRInputSourcesChangeEvents are fired to indicate changes to the XRInputSources that are
 * available to an XRSession.
 * ref: https://immersive-web.github.io/webxr/#xrinputsourceschangeevent-interface
 */
interface XRInputSourceChangeEvent extends XRSessionEvent {
    readonly removed: ReadonlyArray<XRInputSource>;
    readonly added: ReadonlyArray<XRInputSource>;
}

interface XRInputSourceChangeEventHandler {
    (evt: XRInputSourceChangeEvent): any;
}

// Experimental/Draft features

// Anchors
type XRAnchorSet = Set<XRAnchor>;

interface XRAnchor {
    anchorSpace: XRSpace;
    delete(): void;
}

declare abstract class XRAnchor implements XRAnchor {}

interface XRFrame {
    trackedAnchors?: XRAnchorSet | undefined;
    createAnchor?: (pose: XRRigidTransform, space: XRSpace) => Promise<XRAnchor>;
}

// AR Hit testing
declare class XRRay {
    readonly origin: DOMPointReadOnly;
    readonly direction: DOMPointReadOnly;
    readonly matrix: Float32Array;

    constructor(transformOrOrigin?: XRRigidTransform | DOMPointInit, direction?: DOMPointInit);
}

type XRHitTestTrackableType = "point" | "plane" | "mesh";

interface XRTransientInputHitTestResult {
    readonly inputSource: XRInputSource;
    readonly results: ReadonlyArray<XRHitTestResult>;
}

declare class XRTransientInputHitTestResult {
    prototype: XRTransientInputHitTestResult;
}

interface XRHitTestResult {
    getPose(baseSpace: XRSpace): XRPose | undefined;
    // When anchor system is enabled
    createAnchor?: (pose: XRRigidTransform) => Promise<XRAnchor> | undefined;
}

declare abstract class XRHitTestResult implements XRHitTestResult {}

interface XRHitTestSource {
    cancel(): void;
}

declare abstract class XRHitTestSource implements XRHitTestSource {}

interface XRTransientInputHitTestSource {
    cancel(): void;
}

declare abstract class XRTransientInputHitTestSource implements XRTransientInputHitTestSource {}

interface XRHitTestOptionsInit {
    space: XRSpace;
    entityTypes?: XRHitTestTrackableType[] | undefined;
    offsetRay?: XRRay | undefined;
}

interface XRTransientInputHitTestOptionsInit {
    profile: string;
    entityTypes?: XRHitTestTrackableType[] | undefined;
    offsetRay?: XRRay | undefined;
}

interface XRSession {
    requestHitTestSource?: (options: XRHitTestOptionsInit) => Promise<XRHitTestSource>;
    requestHitTestSourceForTransientInput?: (options: XRTransientInputHitTestOptionsInit) => Promise<XRTransientInputHitTestSource>;

    // Legacy
    requestHitTest?: (ray: XRRay, referenceSpace: XRReferenceSpace) => Promise<XRHitResult[]>;
}

interface XRFrame {
    getHitTestResults(hitTestSource: XRHitTestSource): XRHitTestResult[];
    getHitTestResultsForTransientInput(hitTestSource: XRTransientInputHitTestSource): XRTransientInputHitTestResult[];
}

// Legacy
interface XRHitResult {
    hitMatrix: Float32Array;
}

// Plane detection
type XRPlaneSet = Set<XRPlane>;

type XRPlaneOrientation = "horizontal" | "vertical";

interface XRPlane {
    orientation: XRPlaneOrientation;
    planeSpace: XRSpace;
    polygon: DOMPointReadOnly[];
    lastChangedTime: number;
}

declare abstract class XRPlane implements XRPlane {}

interface XRSession {
    // Legacy
    updateWorldTrackingState?: (options: { planeDetectionState?: { enabled: boolean } | undefined }) => void | undefined;
}

// interface XRFrame {
//     worldInformation?:
//         | {
//               detectedPlanes?: XRPlaneSet | undefined;
//           }
//         | undefined;
// }

// Hand Tracking
type XRHandJoint =
    | "wrist"
    | "thumb-metacarpal"
    | "thumb-phalanx-proximal"
    | "thumb-phalanx-distal"
    | "thumb-tip"
    | "index-finger-metacarpal"
    | "index-finger-phalanx-proximal"
    | "index-finger-phalanx-intermediate"
    | "index-finger-phalanx-distal"
    | "index-finger-tip"
    | "middle-finger-metacarpal"
    | "middle-finger-phalanx-proximal"
    | "middle-finger-phalanx-intermediate"
    | "middle-finger-phalanx-distal"
    | "middle-finger-tip"
    | "ring-finger-metacarpal"
    | "ring-finger-phalanx-proximal"
    | "ring-finger-phalanx-intermediate"
    | "ring-finger-phalanx-distal"
    | "ring-finger-tip"
    | "pinky-finger-metacarpal"
    | "pinky-finger-phalanx-proximal"
    | "pinky-finger-phalanx-intermediate"
    | "pinky-finger-phalanx-distal"
    | "pinky-finger-tip";

interface XRJointSpace extends XRSpace {
    readonly jointName: XRHandJoint;
}

declare abstract class XRJointSpace implements XRJointSpace {}

interface XRJointPose extends XRPose {
    readonly radius: number | undefined;
}

declare abstract class XRJointPose implements XRJointPose {}

interface XRHand extends Map<string, XRJointSpace> {
    readonly WRIST: number;

    readonly THUMB_METACARPAL: number;
    readonly THUMB_PHALANX_PROXIMAL: number;
    readonly THUMB_PHALANX_DISTAL: number;
    readonly THUMB_PHALANX_TIP: number;

    readonly INDEX_METACARPAL: number;
    readonly INDEX_PHALANX_PROXIMAL: number;
    readonly INDEX_PHALANX_INTERMEDIATE: number;
    readonly INDEX_PHALANX_DISTAL: number;
    readonly INDEX_PHALANX_TIP: number;

    readonly MIDDLE_METACARPAL: number;
    readonly MIDDLE_PHALANX_PROXIMAL: number;
    readonly MIDDLE_PHALANX_INTERMEDIATE: number;
    readonly MIDDLE_PHALANX_DISTAL: number;
    readonly MIDDLE_PHALANX_TIP: number;

    readonly RING_METACARPAL: number;
    readonly RING_PHALANX_PROXIMAL: number;
    readonly RING_PHALANX_INTERMEDIATE: number;
    readonly RING_PHALANX_DISTAL: number;
    readonly RING_PHALANX_TIP: number;

    readonly LITTLE_METACARPAL: number;
    readonly LITTLE_PHALANX_PROXIMAL: number;
    readonly LITTLE_PHALANX_INTERMEDIATE: number;
    readonly LITTLE_PHALANX_DISTAL: number;
    readonly LITTLE_PHALANX_TIP: number;
}

declare abstract class XRHand extends Map<string, XRJointSpace> implements XRHand {}

// WebXR Layers

/**
 * The base class for XRWebGLLayer and other layer types introduced by future extensions.
 * ref: https://immersive-web.github.io/webxr/#xrlayer-interface
 */
// tslint:disable-next-line no-empty-interface
interface XRLayer extends EventTarget {}

declare abstract class XRLayer implements XRLayer {}

interface XRWebGLLayerInit {
    antialias?: boolean | undefined;
    depth?: boolean | undefined;
    stencil?: boolean | undefined;
    alpha?: boolean | undefined;
    ignoreDepthValues?: boolean | undefined;
    framebufferScaleFactor?: number | undefined;
}

/**
 * A layer which provides a WebGL framebuffer to render into, enabling hardware accelerated
 * rendering of 3D graphics to be presented on the XR device. *
 * ref: https://immersive-web.github.io/webxr/#xrwebgllayer-interface
 */
declare class XRWebGLLayer extends XRLayer {
    static getNativeFramebufferScaleFactor(session: XRSession): number;

    constructor(session: XRSession, context: WebGLRenderingContext | WebGL2RenderingContext, layerInit?: XRWebGLLayerInit);

    readonly antialias: boolean;
    readonly ignoreDepthValues: boolean;
    fixedFoveation?: number | undefined;

    readonly framebuffer: WebGLFramebuffer;
    readonly framebufferWidth: number;
    readonly framebufferHeight: number;

    getViewport(view: XRView): XRViewport | undefined;
}

interface XRRenderStateInit {
    layers?: XRLayer[] | undefined;
}

interface XRRenderState {
    readonly layers?: XRLayer[] | undefined;
}

type XRLayerEventType = "redraw";

interface XRLayerEvent extends Event {
    readonly type: XRLayerEventType;
    readonly layer: XRLayer;
}

interface XRCompositionLayerEventMap {
    redraw: XRLayerEvent;
}

interface XRCompositionLayer extends XRLayer {
    readonly layout: XRLayerLayout;
    blendTextureSourceAlpha: boolean;
    chromaticAberrationCorrection?: boolean | undefined;
    readonly mipLevels: number;
    readonly needsRedraw: boolean;
    destroy(): void;

    space: XRSpace;

    // Events
    onredraw: (evt: XRCompositionLayerEventMap["redraw"]) => any;

    addEventListener<K extends keyof XRCompositionLayerEventMap>(
        this: XRCompositionLayer,
        type: K,
        callback: (evt: XRCompositionLayerEventMap[K]) => any,
        options?: boolean | AddEventListenerOptions
    ): void;
    addEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | AddEventListenerOptions): void;

    removeEventListener<K extends keyof XRCompositionLayerEventMap>(this: XRCompositionLayer, type: K, callback: (evt: XRCompositionLayerEventMap[K]) => any): void;
    removeEventListener(type: string, listener: EventListenerOrEventListenerObject, options?: boolean | EventListenerOptions): void;
}

declare abstract class XRCompositionLayer implements XRCompositionLayer {}

type XRTextureType = "texture" | "texture-array";

type XRLayerLayout = "default" | "mono" | "stereo" | "stereo-left-right" | "stereo-top-bottom";

interface XRProjectionLayerInit {
    scaleFactor?: number | undefined;
    textureType?: XRTextureType | undefined;
    colorFormat?: GLenum | undefined;
    depthFormat?: GLenum | undefined;
}

interface XRProjectionLayer extends XRCompositionLayer {
    readonly textureWidth: number;
    readonly textureHeight: number;
    readonly textureArrayLength: number;
    readonly ignoreDepthValues: number;
    fixedFoveation: number;
}

declare abstract class XRProjectionLayer implements XRProjectionLayer {}

interface XRLayerInit {
    mipLevels?: number | undefined;
    viewPixelWidth: number;
    viewPixelHeight: number;
    isStatic?: boolean | undefined;
    colorFormat?: GLenum | undefined;
    depthFormat?: GLenum | undefined;
    space: XRSpace;
    layout?: XRLayerLayout | undefined;
}

interface XRMediaLayerInit {
    invertStereo?: boolean | undefined;
    space: XRSpace;
    layout?: XRLayerLayout | undefined;
}

interface XRCylinderLayerInit extends XRLayerInit {
    textureType?: XRTextureType | undefined;
    transform: XRRigidTransform;
    radius?: number | undefined;
    centralAngle?: number | undefined;
    aspectRatio?: number | undefined;
}

interface XRMediaCylinderLayerInit extends XRMediaLayerInit {
    transform?: XRRigidTransform | undefined;
    radius?: number | undefined;
    centralAngle?: number | undefined;
    aspectRatio?: number | undefined;
}

interface XRCylinderLayer extends XRCompositionLayer {
    transform: XRRigidTransform;
    radius: number;
    centralAngle: number;
    aspectRatio: number;
}

declare abstract class XRCylinderLayer implements XRCylinderLayer {}

interface XRQuadLayerInit extends XRLayerInit {
    textureType?: XRTextureType | undefined;
    transform?: XRRigidTransform | undefined;
    width?: number | undefined;
    height?: number | undefined;
}

interface XRMediaQuadLayerInit extends XRMediaLayerInit {
    transform?: XRRigidTransform | undefined;
    width?: number | undefined;
    height?: number | undefined;
}

interface XRQuadLayer extends XRCompositionLayer {
    transform: XRRigidTransform;
    width: number;
    height: number;
}

declare abstract class XRQuadLayer implements XRQuadLayer {}

interface XREquirectLayerInit extends XRLayerInit {
    textureType?: XRTextureType | undefined;
    transform?: XRRigidTransform | undefined;
    radius?: number | undefined;
    centralHorizontalAngle?: number | undefined;
    upperVerticalAngle?: number | undefined;
    lowerVerticalAngle?: number | undefined;
}

interface XRMediaEquirectLayerInit extends XRMediaLayerInit {
    transform?: XRRigidTransform | undefined;
    radius?: number | undefined;
    centralHorizontalAngle?: number | undefined;
    upperVerticalAngle?: number | undefined;
    lowerVerticalAngle?: number | undefined;
}

interface XREquirectLayer extends XRCompositionLayer {
    transform: XRRigidTransform;
    radius: number;
    centralHorizontalAngle: number;
    upperVerticalAngle: number;
    lowerVerticalAngle: number;
}

declare abstract class XREquirectLayer implements XREquirectLayer {}

interface XRCubeLayerInit extends XRLayerInit {
    orientation?: DOMPointReadOnly | undefined;
}

interface XRCubeLayer extends XRCompositionLayer {
    orientation: DOMPointReadOnly;
}

declare abstract class XRCubeLayer implements XRCubeLayer {}

interface XRSubImage {
    readonly viewport: XRViewport;
}

declare abstract class XRSubImage implements XRSubImage {}

interface XRWebGLSubImage extends XRSubImage {
    readonly colorTexture: WebGLTexture;
    readonly depthStencilTexture: WebGLTexture;
    readonly imageIndex: number;
    readonly textureWidth: number;
    readonly textureHeight: number;
}

declare abstract class XRWebGLSubImage implements XRWebGLSubImage {}

declare class XRWebGLBinding {
    readonly nativeProjectionScaleFactor: number;

    constructor(session: XRSession, context: WebGLRenderingContext);

    createProjectionLayer(init?: XRProjectionLayerInit): XRProjectionLayer;
    createQuadLayer(init?: XRQuadLayerInit): XRQuadLayer;
    createCylinderLayer(init?: XRCylinderLayerInit): XRCylinderLayer;
    createEquirectLayer(init?: XREquirectLayerInit): XREquirectLayer;
    createCubeLayer(init?: XRCubeLayerInit): XRCubeLayer;

    getSubImage(layer: XRCompositionLayer, frame: XRFrame, eye?: XREye): XRWebGLSubImage;
    getViewSubImage(layer: XRProjectionLayer, view: XRView): XRWebGLSubImage;

    // BABYLON addition
    getReflectionCubeMap: (lightProbe: XRLightProbe) => WebGLTexture;
}

declare class XRMediaBinding {
    constructor(sesion: XRSession);

    createQuadLayer(video: HTMLVideoElement, init?: XRMediaQuadLayerInit): XRQuadLayer;
    createCylinderLayer(video: HTMLVideoElement, init?: XRMediaCylinderLayerInit): XRCylinderLayer;
    createEquirectLayer(video: HTMLVideoElement, init?: XRMediaEquirectLayerInit): XREquirectLayer;
}

// WebGL extensions
interface WebGLRenderingContextBase {
    getExtension(extensionName: "OCULUS_multiview"): OCULUS_multiview | null;
}

declare enum XOVR_multiview2 {
    FRAMEBUFFER_ATTACHMENT_TEXTURE_NUM_VIEWS_OVR = 0x9630,
    FRAMEBUFFER_ATTACHMENT_TEXTURE_BASE_VIEW_INDEX_OVR = 0x9632,
    MAX_VIEWS_OVR = 0x9631,
    FRAMEBUFFER_INCOMPLETE_VIEW_TARGETS_OVR = 0x9633,
}

interface OVR_multiview2 {
    readonly FRAMEBUFFER_ATTACHMENT_TEXTURE_NUM_VIEWS_OVR: number;
    readonly FRAMEBUFFER_ATTACHMENT_TEXTURE_BASE_VIEW_INDEX_OVR: number;
    readonly MAX_VIEWS_OVR: number;
    readonly FRAMEBUFFER_INCOMPLETE_VIEW_TARGETS_OVR: number;

    framebufferTextureMultiviewOVR(target: GLenum, attachment: GLenum, texture: WebGLTexture, level: number, baseViewIndex: number, numViews: number): WebGLRenderbuffer;
}

declare abstract class OVR_multiview2 implements OVR_multiview2 {}

// Oculus extensions
interface XRSessionGrant {
    mode: XRSessionMode;
}

interface XRSystemSessionGrantedEvent extends Event {
    type: "sessiongranted";
    session: XRSessionGrant;
}

interface XRSystemSessionGrantedEventHandler {
    (event: XRSystemSessionGrantedEvent): any;
}

interface XRSystemEventMap {
    // Session Grant events are an Meta Oculus Browser extension
    sessiongranted: XRSystemSessionGrantedEvent;
}

interface XRSystem {
    onsessiongranted: XRSystemSessionGrantedEventHandler | null;
}

interface OCULUS_multiview extends OVR_multiview2 {
    framebufferTextureMultisampleMultiviewOVR(
        target: GLenum,
        attachment: GLenum,
        texture: WebGLTexture | null,
        level: GLint,
        samples: GLsizei,
        baseViewIndex: GLint,
        numViews: GLsizei
    ): void;
}

declare abstract class OCULUS_multiview implements OCULUS_multiview {}

/**
 * BEGIN: WebXR DOM Overlays Module
 * https://immersive-web.github.io/dom-overlays/
 */

interface GlobalEventHandlersEventMap {
    beforexrselect: XRSessionEvent;
}

interface GlobalEventHandlers {
    /**
     * An XRSessionEvent of type beforexrselect is dispatched on the DOM overlay
     * element before generating a WebXR selectstart input event if the -Z axis
     * of the input source's targetRaySpace intersects the DOM overlay element
     * at the time the input device's primary action is triggered.
     */
    onbeforexrselect: ((this: GlobalEventHandlers, ev: XRSessionEvent) => any) | null;
}

interface XRDOMOverlayInit {
    root: Element;
}

interface XRSessionInit {
    domOverlay?: XRDOMOverlayInit | undefined;
}

type XRDOMOverlayType = "screen" | "floating" | "head-locked";

interface XRDOMOverlayState {
    type: XRDOMOverlayType;
}

interface XRSession {
    readonly domOverlayState?: XRDOMOverlayState | undefined;
}

/// BABYLON EDITS
interface XREyeTrackingSourceEvent extends XRSessionEvent {
    readonly gazeSpace: XRSpace;
}

interface XRFrame {
    fillPoses?(spaces: XRSpace[], baseSpace: XRSpace, transforms: Float32Array): boolean;
    // Anchors
    trackedAnchors?: XRAnchorSet;
    // World geometries. DEPRECATED
    worldInformation?: XRWorldInformation;
    detectedPlanes?: XRPlaneSet;
    // Hand tracking
    getJointPose?(joint: XRJointSpace, baseSpace: XRSpace): XRJointPose;
    fillJointRadii?(jointSpaces: XRJointSpace[], radii: Float32Array): boolean;
    // Image tracking
    getImageTrackingResults?(): Array<XRImageTrackingResult>;
    getLightEstimate(xrLightProbe: XRLightProbe): XRLightEstimate;
}

type XREventType = keyof XRSessionEventMap;

type XRImageTrackingState = "tracked" | "emulated";
type XRImageTrackingScore = "untrackable" | "trackable";

interface XRTrackedImageInit {
    image: ImageBitmap;
    widthInMeters: number;
}

interface XRImageTrackingResult {
    readonly imageSpace: XRSpace;
    readonly index: number;
    readonly trackingState: XRImageTrackingState;
    readonly measuredWidthInMeters: number;
}

interface XRPose {
    readonly linearVelocity?: DOMPointReadOnly;
    readonly angularVelocity?: DOMPointReadOnly;
}

type XRLightProbeInit = {
    reflectionFormat: XRReflectionFormat;
};

type XRReflectionFormat = "srgba8" | "rgba16f";

interface XRSession {
    readonly preferredReflectionFormat?: XRReflectionFormat;
    /**
     * The XRSession interface is extended with the ability to create new XRLightProbe instances.
     * XRLightProbe instances have a session object, which is the XRSession that created this XRLightProbe.
     *
     * Can reject with with a "NotSupportedError" DOMException
     */
    requestLightProbe(options?: XRLightProbeInit): Promise<XRLightProbe>;

    getTrackedImageScores?(): Promise<XRImageTrackingScore[]>;
}

interface XRWorldInformation {
    detectedPlanes?: XRPlaneSet;
}

interface XRSessionInit {
    trackedImages?: XRTrackedImageInit[];
}

interface XRLightEstimate {
    readonly sphericalHarmonicsCoefficients: Float32Array;
    readonly primaryLightDirection: DOMPointReadOnly;
    readonly primaryLightIntensity: DOMPointReadOnly;
}

interface XREventHandler {
    (evt: Event): any;
}

interface XRLightProbe extends EventTarget {
    readonly probeSpace: XRSpace;
    onreflectionchange: XREventHandler;
}

/**
 * END: WebXR DOM Overlays Module
 * https://immersive-web.github.io/dom-overlays/
 */


// This file contains native only extensions for WebXR. These APIs are not supported in the browser yet.
// They are intended for use with either Babylon Native https://github.com/BabylonJS/BabylonNative or
// Babylon React Native: https://github.com/BabylonJS/BabylonReactNative

type XRSceneObjectType = "unknown" | "background" | "wall" | "floor" | "ceiling" | "platform" | "inferred" | "world";

interface XRSceneObject {
    type: XRSceneObjectType;
}

interface XRFieldOfView {
    angleLeft: number;
    angleRight: number;
    angleUp: number;
    angleDown: number;
}

interface XRFrustum {
    position: DOMPointReadOnly;
    orientation: DOMPointReadOnly;
    fieldOfView: XRFieldOfView;
    farDistance: number;
}

interface XRPlane {
    parentSceneObject?: XRSceneObject;
}

interface XRMesh {
    meshSpace: XRSpace;
    positions: Float32Array;
    indices: Uint32Array;
    normals?: Float32Array;
    lastChangedTime: number;
    parentSceneObject?: XRSceneObject;
}

interface XRFrustumDetectionBoundary {
    type: "frustum";
    frustum: XRFrustum;
}

interface XRSphereDetectionBoundary {
    type: "sphere";
    radius: number;
}

interface XRBoxDetectionBoundary {
    type: "box";
    extent: DOMPointReadOnly;
}

type XRDetectionBoundary = XRFrustumDetectionBoundary | XRSphereDetectionBoundary | XRBoxDetectionBoundary;

interface XRGeometryDetectorOptions {
    detectionBoundary?: XRDetectionBoundary;
    updateInterval?: number;
}

interface XRSession {
    trySetFeaturePointCloudEnabled(enabled: boolean): boolean;
    trySetPreferredPlaneDetectorOptions(preferredOptions: XRGeometryDetectorOptions): boolean;
    trySetMeshDetectorEnabled(enabled: boolean): boolean;
    trySetPreferredMeshDetectorOptions(preferredOptions: XRGeometryDetectorOptions): boolean;
}

interface XRFrame {
    featurePointCloud?: Array<number>;
}

type XRMeshSet = Set<XRMesh>;

interface XRWorldInformation {
    detectedMeshes?: XRMeshSet;
}

